/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

// This file is a Mako template: http://www.makotemplates.org/

// Please note that valid Rust syntax may be mangled by the Mako parser.
// For example, Vec&Foo> will be mangled as Vec&Foo>. To work around these issues, the code
// can be escaped. In the above example, Vec<&Foo> or Vec< &Foo> achieves the desired result of Vec&Foo>.



use app_units::Au;
use servo_arc::{Arc, UniqueArc};
use smallbitvec::SmallBitVec;
use std::borrow::Cow;
use std::{fmt, mem, ops};
use std::cell::RefCell;
#[cfg(feature = "gecko")] use std::ptr;

#[cfg(feature = "servo")] use cssparser::RGBA;
use cssparser::{CowRcStr, Parser, TokenSerializationType, serialize_identifier};
use cssparser::ParserInput;
#[cfg(feature = "servo")] use euclid::SideOffsets2D;
use computed_values;
use context::QuirksMode;
use font_metrics::FontMetricsProvider;
#[cfg(feature = "gecko")] use gecko_bindings::bindings;
#[cfg(feature = "gecko")] use gecko_bindings::structs::{self, nsCSSPropertyID};
#[cfg(feature = "servo")] use logical_geometry::{LogicalMargin, PhysicalSide};
use logical_geometry::WritingMode;
use media_queries::Device;
use parser::ParserContext;
use properties::animated_properties::AnimatableLonghand;
#[cfg(feature = "gecko")] use properties::longhands::system_font::SystemFont;
use rule_cache::{RuleCache, RuleCacheConditions};
use selector_map::PrecomputedHashSet;
use selector_parser::PseudoElement;
use selectors::parser::SelectorParseError;
#[cfg(feature = "servo")] use servo_config::prefs::PREFS;
use shared_lock::StylesheetGuards;
use style_traits::{PARSING_MODE_DEFAULT, ToCss, ParseError};
use style_traits::{PropertyDeclarationParseError, StyleParseError, ValueParseError};
use stylesheets::{CssRuleType, Origin, UrlExtraData};
#[cfg(feature = "servo")] use values::Either;
use values::generics::text::LineHeight;
use values::computed;
use values::computed::NonNegativeLength;
use rule_tree::{CascadeLevel, StrongRuleNode};
use self::computed_value_flags::*;
use style_adjuster::StyleAdjuster;
#[cfg(feature = "servo")] use values::specified::BorderStyle;

pub use self::declaration_block::*;

#[cfg(feature = "gecko")]
#[macro_export]
macro_rules! property_name {
    ($s: tt) => { atom!($s) }
}

#[cfg(feature = "gecko")]
macro_rules! impl_bitflags_conversions {
    ($name: ident) => {
        impl From<u8> for $name {
            fn from(bits: u8) -> $name {
                $name::from_bits(bits).expect("bits contain valid flag")
            }
        }

        impl From<$name> for u8 {
            fn from(v: $name) -> u8 {
                v.bits()
            }
        }
    };
}



#[path="/builddir/build/BUILD/firefox-57.0b9/servo/components/style/properties/computed_value_flags.rs"]
pub mod computed_value_flags;
#[path="/builddir/build/BUILD/firefox-57.0b9/servo/components/style/properties/declaration_block.rs"]
pub mod declaration_block;

/// Conversion with fewer impls than From/Into
pub trait MaybeBoxed<Out> {
    /// Convert
    fn maybe_boxed(self) -> Out;
}


/// This is where we store extra font data while
/// while computing font sizes.
///
/// font-size keyword values (and font-size-relative values applied
/// to keyword values) need to preserve their identity as originating
/// from keywords and relative font sizes. We store this information
/// out of band in the ComputedValues. When None, the font size on the
/// current struct was computed from a value that was not a keyword
/// or a chain of font-size-relative values applying to successive parents
/// terminated by a keyword. When Some, this means the font-size was derived
/// from a keyword value or a keyword value on some ancestor with only
/// font-size-relative keywords and regular inheritance in between. The
/// integer stores the final ratio of the chain of font size relative values.
/// and is 1 when there was just a keyword and no relative values.
///
/// When this is Some, we compute font sizes by computing the keyword against
/// the generic font, and then multiplying it by the ratio (as well as adding any
/// absolute offset from calcs)
pub type FontComputationData = Option<(longhands::font_size::KeywordSize, f32, NonNegativeLength)>;

/// Default value for FontComputationData
pub fn default_font_size_keyword() -> FontComputationData {
    Some((Default::default(), 1., Au(0).into()))
}

impl<T> MaybeBoxed<T> for T {
    #[inline]
    fn maybe_boxed(self) -> T { self }
}

impl<T> MaybeBoxed<Box<T>> for T {
    #[inline]
    fn maybe_boxed(self) -> Box<T> { Box::new(self) }
}

macro_rules! expanded {
    ( $( $name: ident: $value: expr ),+ ) => {
        expanded!( $( $name: $value, )+ )
    };
    ( $( $name: ident: $value: expr, )+ ) => {
        Longhands {
            $(
                $name: MaybeBoxed::maybe_boxed($value),
            )+
        }
    }
}

/// A module with all the code for longhand properties.
#[allow(missing_docs)]
pub mod longhands {
    /* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */






    
        
    
    /// https://drafts.csswg.org/css-backgrounds/#background-color
    pub mod background_color {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::Color as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::Color as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed_value::T::transparent() }
        #[inline] pub fn get_initial_specified_value() -> SpecifiedValue { SpecifiedValue::transparent() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::Color::parse_quirky(context, input, AllowQuirks::Yes)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::BackgroundColor(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::BackgroundColor);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::BackgroundColor);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_background_color(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_background_color();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_background_color();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::BackgroundColor)
            }
    }




    
        
    
    
    /// https://drafts.csswg.org/css-backgrounds/#the-background-image
    pub mod background_image {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        use std::fmt;
        use style_traits::{Separator, ToCss};

        pub mod single_value {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::ShorthandId;
            #[allow(unused_imports)]
            use selectors::parser::SelectorParseError;
            #[allow(unused_imports)]
            use style_traits::{ParseError, StyleParseError};
            #[allow(unused_imports)]
            use values::computed::{Context, ToComputedValue};
            #[allow(unused_imports)]
            use values::{computed, specified};
            #[allow(unused_imports)]
            use values::{Auto, Either, None_, Normal};
            
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::ImageLayer as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::ImageLayer as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { Either::First(None_) }
        #[inline] pub fn get_initial_specified_value() -> SpecifiedValue { Either::First(None_) }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::ImageLayer::parse(context, input)
        }
    
        
        }

        /// The definition of the computed value for background-image.
        pub mod computed_value {
            pub use super::single_value::computed_value as single_value;
            pub use self::single_value::T as SingleComputedValue;
            use smallvec::{IntoIter, SmallVec};
            use values::computed::ComputedVecIter;

            /// The computed value, effectively a list of single values.
            #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
            #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
            #[derive(Clone, Debug, PartialEq)]
            pub struct T(
                pub SmallVec<[single_value::T; 1]>,
            );


            pub type Iter<'a, 'cx, 'cx_a> = ComputedVecIter<'a, 'cx, 'cx_a, super::single_value::SpecifiedValue>;

            impl IntoIterator for T {
                type Item = single_value::T;
                type IntoIter = IntoIter<[single_value::T; 1]>;
                fn into_iter(self) -> Self::IntoIter {
                    self.0.into_iter()
                }
            }
        }

        impl ToCss for computed_value::T {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        warn!("Found empty value for property background-image");
                }
                for i in iter {
                    dest.write_str(::style_traits::Comma::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        /// The specified value of background-image.
        #[derive(Clone, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        pub struct SpecifiedValue(pub Vec<single_value::SpecifiedValue>);

        impl ToCss for SpecifiedValue {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        warn!("Found empty value for property background-image");
                }
                for i in iter {
                    dest.write_str(::style_traits::Comma::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        pub fn get_initial_value() -> computed_value::T {
                let mut v = SmallVec::new();
                v.push(single_value::get_initial_value());
                computed_value::T(v)
        }

        pub fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                -> Result<SpecifiedValue, ParseError<'i>> {
            use style_traits::Separator;


            ::style_traits::Comma::parse(input, |parser| {
                single_value::parse(context, parser)
            }).map(SpecifiedValue)
        }

        pub use self::single_value::SpecifiedValue as SingleSpecifiedValue;

        impl SpecifiedValue {
            pub fn compute_iter<'a, 'cx, 'cx_a>(&'a self, context: &'cx Context<'cx_a>)
                -> computed_value::Iter<'a, 'cx, 'cx_a> {
                computed_value::Iter::new(context, &self.0)
            }
        }

        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, context: &Context) -> computed_value::T {
                computed_value::T(self.compute_iter(context).collect())
            }

            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                SpecifiedValue(computed.0.iter()
                                    .map(ToComputedValue::from_computed_value)
                                    .collect())
            }
        }
    
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::BackgroundImage(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::BackgroundImage);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::BackgroundImage);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            // In the case of a vector property we want to pass
                            // down an iterator so that this can be computed
                            // without allocation
                            //
                            // However, computing requires a context, but the
                            // style struct being mutated is on the context. We
                            // temporarily remove it, mutate it, and then put it
                            // back. Vector longhands cannot touch their own
                            // style struct whilst computing, else this will
                            // panic.
                            let mut s =
                                context.builder.take_background();
                            {
                                let iter = specified_value.compute_iter(context);
                                s.set_background_image(iter);
                            }
                            context.builder.put_background(s);
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_background_image();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_background_image();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::BackgroundImage)
            }
    }




    
    
        
    
    
    /// https://drafts.csswg.org/css-backgrounds-4/#propdef-background-position-x
    pub mod background_position_x {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        use std::fmt;
        use style_traits::{Separator, ToCss};

        pub mod single_value {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::ShorthandId;
            #[allow(unused_imports)]
            use selectors::parser::SelectorParseError;
            #[allow(unused_imports)]
            use style_traits::{ParseError, StyleParseError};
            #[allow(unused_imports)]
            use values::computed::{Context, ToComputedValue};
            #[allow(unused_imports)]
            use values::{computed, specified};
            #[allow(unused_imports)]
            use values::{Auto, Either, None_, Normal};
            
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::position::HorizontalPosition as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::position::HorizontalPosition as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::LengthOrPercentage::zero() }
        #[inline] pub fn get_initial_specified_value() -> SpecifiedValue { SpecifiedValue::initial_specified_value() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::position::HorizontalPosition::parse(context, input)
        }
    
        
        }

        /// The definition of the computed value for background-position-x.
        pub mod computed_value {
            pub use super::single_value::computed_value as single_value;
            pub use self::single_value::T as SingleComputedValue;
            use smallvec::{IntoIter, SmallVec};
            use values::computed::ComputedVecIter;

            /// The computed value, effectively a list of single values.
            #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
            #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
            #[derive(Clone, Debug, PartialEq)]
            #[derive(Animate, ComputeSquaredDistance)]
            pub struct T(
                pub SmallVec<[single_value::T; 1]>,
            );

                use values::animated::{ToAnimatedZero};

                impl ToAnimatedZero for T {
                    #[inline]
                    fn to_animated_zero(&self) -> Result<Self, ()> { Err(()) }
                }

            pub type Iter<'a, 'cx, 'cx_a> = ComputedVecIter<'a, 'cx, 'cx_a, super::single_value::SpecifiedValue>;

            impl IntoIterator for T {
                type Item = single_value::T;
                type IntoIter = IntoIter<[single_value::T; 1]>;
                fn into_iter(self) -> Self::IntoIter {
                    self.0.into_iter()
                }
            }
        }

        impl ToCss for computed_value::T {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        warn!("Found empty value for property background-position-x");
                }
                for i in iter {
                    dest.write_str(::style_traits::Comma::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        /// The specified value of background-position-x.
        #[derive(Clone, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        pub struct SpecifiedValue(pub Vec<single_value::SpecifiedValue>);

        impl ToCss for SpecifiedValue {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        warn!("Found empty value for property background-position-x");
                }
                for i in iter {
                    dest.write_str(::style_traits::Comma::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        pub fn get_initial_value() -> computed_value::T {
                let mut v = SmallVec::new();
                v.push(single_value::get_initial_value());
                computed_value::T(v)
        }

        pub fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                -> Result<SpecifiedValue, ParseError<'i>> {
            use style_traits::Separator;


            ::style_traits::Comma::parse(input, |parser| {
                single_value::parse(context, parser)
            }).map(SpecifiedValue)
        }

        pub use self::single_value::SpecifiedValue as SingleSpecifiedValue;

        impl SpecifiedValue {
            pub fn compute_iter<'a, 'cx, 'cx_a>(&'a self, context: &'cx Context<'cx_a>)
                -> computed_value::Iter<'a, 'cx, 'cx_a> {
                computed_value::Iter::new(context, &self.0)
            }
        }

        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, context: &Context) -> computed_value::T {
                computed_value::T(self.compute_iter(context).collect())
            }

            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                SpecifiedValue(computed.0.iter()
                                    .map(ToComputedValue::from_computed_value)
                                    .collect())
            }
        }
    
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::BackgroundPositionX(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::BackgroundPositionX);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::BackgroundPositionX);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            // In the case of a vector property we want to pass
                            // down an iterator so that this can be computed
                            // without allocation
                            //
                            // However, computing requires a context, but the
                            // style struct being mutated is on the context. We
                            // temporarily remove it, mutate it, and then put it
                            // back. Vector longhands cannot touch their own
                            // style struct whilst computing, else this will
                            // panic.
                            let mut s =
                                context.builder.take_background();
                            {
                                let iter = specified_value.compute_iter(context);
                                s.set_background_position_x(iter);
                            }
                            context.builder.put_background(s);
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_background_position_x();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_background_position_x();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::BackgroundPositionX)
            }
    }



    
    
        
    
    
    /// https://drafts.csswg.org/css-backgrounds-4/#propdef-background-position-y
    pub mod background_position_y {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        use std::fmt;
        use style_traits::{Separator, ToCss};

        pub mod single_value {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::ShorthandId;
            #[allow(unused_imports)]
            use selectors::parser::SelectorParseError;
            #[allow(unused_imports)]
            use style_traits::{ParseError, StyleParseError};
            #[allow(unused_imports)]
            use values::computed::{Context, ToComputedValue};
            #[allow(unused_imports)]
            use values::{computed, specified};
            #[allow(unused_imports)]
            use values::{Auto, Either, None_, Normal};
            
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::position::VerticalPosition as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::position::VerticalPosition as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::LengthOrPercentage::zero() }
        #[inline] pub fn get_initial_specified_value() -> SpecifiedValue { SpecifiedValue::initial_specified_value() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::position::VerticalPosition::parse(context, input)
        }
    
        
        }

        /// The definition of the computed value for background-position-y.
        pub mod computed_value {
            pub use super::single_value::computed_value as single_value;
            pub use self::single_value::T as SingleComputedValue;
            use smallvec::{IntoIter, SmallVec};
            use values::computed::ComputedVecIter;

            /// The computed value, effectively a list of single values.
            #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
            #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
            #[derive(Clone, Debug, PartialEq)]
            #[derive(Animate, ComputeSquaredDistance)]
            pub struct T(
                pub SmallVec<[single_value::T; 1]>,
            );

                use values::animated::{ToAnimatedZero};

                impl ToAnimatedZero for T {
                    #[inline]
                    fn to_animated_zero(&self) -> Result<Self, ()> { Err(()) }
                }

            pub type Iter<'a, 'cx, 'cx_a> = ComputedVecIter<'a, 'cx, 'cx_a, super::single_value::SpecifiedValue>;

            impl IntoIterator for T {
                type Item = single_value::T;
                type IntoIter = IntoIter<[single_value::T; 1]>;
                fn into_iter(self) -> Self::IntoIter {
                    self.0.into_iter()
                }
            }
        }

        impl ToCss for computed_value::T {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        warn!("Found empty value for property background-position-y");
                }
                for i in iter {
                    dest.write_str(::style_traits::Comma::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        /// The specified value of background-position-y.
        #[derive(Clone, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        pub struct SpecifiedValue(pub Vec<single_value::SpecifiedValue>);

        impl ToCss for SpecifiedValue {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        warn!("Found empty value for property background-position-y");
                }
                for i in iter {
                    dest.write_str(::style_traits::Comma::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        pub fn get_initial_value() -> computed_value::T {
                let mut v = SmallVec::new();
                v.push(single_value::get_initial_value());
                computed_value::T(v)
        }

        pub fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                -> Result<SpecifiedValue, ParseError<'i>> {
            use style_traits::Separator;


            ::style_traits::Comma::parse(input, |parser| {
                single_value::parse(context, parser)
            }).map(SpecifiedValue)
        }

        pub use self::single_value::SpecifiedValue as SingleSpecifiedValue;

        impl SpecifiedValue {
            pub fn compute_iter<'a, 'cx, 'cx_a>(&'a self, context: &'cx Context<'cx_a>)
                -> computed_value::Iter<'a, 'cx, 'cx_a> {
                computed_value::Iter::new(context, &self.0)
            }
        }

        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, context: &Context) -> computed_value::T {
                computed_value::T(self.compute_iter(context).collect())
            }

            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                SpecifiedValue(computed.0.iter()
                                    .map(ToComputedValue::from_computed_value)
                                    .collect())
            }
        }
    
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::BackgroundPositionY(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::BackgroundPositionY);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::BackgroundPositionY);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            // In the case of a vector property we want to pass
                            // down an iterator so that this can be computed
                            // without allocation
                            //
                            // However, computing requires a context, but the
                            // style struct being mutated is on the context. We
                            // temporarily remove it, mutate it, and then put it
                            // back. Vector longhands cannot touch their own
                            // style struct whilst computing, else this will
                            // panic.
                            let mut s =
                                context.builder.take_background();
                            {
                                let iter = specified_value.compute_iter(context);
                                s.set_background_position_y(iter);
                            }
                            context.builder.put_background(s);
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_background_position_y();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_background_position_y();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::BackgroundPositionY)
            }
    }





    
    
    /// https://drafts.csswg.org/css-backgrounds/#the-background-repeat
    pub mod background_repeat {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        use std::fmt;
        use style_traits::{Separator, ToCss};

        pub mod single_value {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::ShorthandId;
            #[allow(unused_imports)]
            use selectors::parser::SelectorParseError;
            #[allow(unused_imports)]
            use style_traits::{ParseError, StyleParseError};
            #[allow(unused_imports)]
            use values::computed::{Context, ToComputedValue};
            #[allow(unused_imports)]
            use values::{computed, specified};
            #[allow(unused_imports)]
            use values::{Auto, Either, None_, Normal};
            
    use std::fmt;
    use style_traits::ToCss;

    define_css_keyword_enum!(RepeatKeyword:
                             "repeat" => Repeat,
                             "space" => Space,
                             "round" => Round,
                             "no-repeat" => NoRepeat);

    #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
    #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
    #[derive(Clone, Debug, PartialEq, ToCss)]
    pub enum SpecifiedValue {
        RepeatX,
        RepeatY,
        Other(RepeatKeyword, Option<RepeatKeyword>),
    }

    pub mod computed_value {
        pub use super::RepeatKeyword;

        #[derive(Clone, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        pub struct T(pub RepeatKeyword, pub RepeatKeyword);
    }


    impl ToCss for computed_value::T {
        fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
            match (self.0, self.1) {
                (RepeatKeyword::Repeat, RepeatKeyword::NoRepeat) => dest.write_str("repeat-x"),
                (RepeatKeyword::NoRepeat, RepeatKeyword::Repeat) => dest.write_str("repeat-y"),
                (horizontal, vertical) => {
                    horizontal.to_css(dest)?;
                    if horizontal != vertical {
                        dest.write_str(" ")?;
                        vertical.to_css(dest)?;
                    }
                    Ok(())
                },
            }
        }
    }

    #[inline]
    pub fn get_initial_value() -> computed_value::T {
        computed_value::T(RepeatKeyword::Repeat, RepeatKeyword::Repeat)
    }

    #[inline]
    pub fn get_initial_specified_value() -> SpecifiedValue {
        SpecifiedValue::Other(RepeatKeyword::Repeat, None)
    }

    impl ToComputedValue for SpecifiedValue {
        type ComputedValue = computed_value::T;

        #[inline]
        fn to_computed_value(&self, _context: &Context) -> computed_value::T {
            match *self {
                SpecifiedValue::RepeatX =>
                    computed_value::T(RepeatKeyword::Repeat, RepeatKeyword::NoRepeat),
                SpecifiedValue::RepeatY =>
                    computed_value::T(RepeatKeyword::NoRepeat, RepeatKeyword::Repeat),
                SpecifiedValue::Other(horizontal, vertical) =>
                    computed_value::T(horizontal, vertical.unwrap_or(horizontal))
            }
        }

        #[inline]
        fn from_computed_value(computed: &computed_value::T) -> Self {
            match (computed.0, computed.1) {
                (RepeatKeyword::Repeat, RepeatKeyword::NoRepeat) => SpecifiedValue::RepeatX,
                (RepeatKeyword::NoRepeat, RepeatKeyword::Repeat) => SpecifiedValue::RepeatY,
                (horizontal, vertical) => SpecifiedValue::Other(horizontal, Some(vertical)),
            }
        }
    }

    pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                         -> Result<SpecifiedValue, ParseError<'i>> {
        let ident = input.expect_ident_cloned()?;
        (match_ignore_ascii_case! { &ident,
            "repeat-x" => Ok(SpecifiedValue::RepeatX),
            "repeat-y" => Ok(SpecifiedValue::RepeatY),
            _ => Err(()),
        }).or_else(|()| {
            let horizontal: Result<_, ParseError> = RepeatKeyword::from_ident(&ident)
                .map_err(|()| SelectorParseError::UnexpectedIdent(ident.clone()).into());
            let horizontal = horizontal?;
            let vertical = input.try(RepeatKeyword::parse).ok();
            Ok(SpecifiedValue::Other(horizontal, vertical))
        })
    }

        }

        /// The definition of the computed value for background-repeat.
        pub mod computed_value {
            pub use super::single_value::computed_value as single_value;
            pub use self::single_value::T as SingleComputedValue;
            use smallvec::{IntoIter, SmallVec};
            use values::computed::ComputedVecIter;

            /// The computed value, effectively a list of single values.
            #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
            #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
            #[derive(Clone, Debug, PartialEq)]
            pub struct T(
                pub SmallVec<[single_value::T; 1]>,
            );


            pub type Iter<'a, 'cx, 'cx_a> = ComputedVecIter<'a, 'cx, 'cx_a, super::single_value::SpecifiedValue>;

            impl IntoIterator for T {
                type Item = single_value::T;
                type IntoIter = IntoIter<[single_value::T; 1]>;
                fn into_iter(self) -> Self::IntoIter {
                    self.0.into_iter()
                }
            }
        }

        impl ToCss for computed_value::T {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        warn!("Found empty value for property background-repeat");
                }
                for i in iter {
                    dest.write_str(::style_traits::Comma::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        /// The specified value of background-repeat.
        #[derive(Clone, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        pub struct SpecifiedValue(pub Vec<single_value::SpecifiedValue>);

        impl ToCss for SpecifiedValue {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        warn!("Found empty value for property background-repeat");
                }
                for i in iter {
                    dest.write_str(::style_traits::Comma::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        pub fn get_initial_value() -> computed_value::T {
                let mut v = SmallVec::new();
                v.push(single_value::get_initial_value());
                computed_value::T(v)
        }

        pub fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                -> Result<SpecifiedValue, ParseError<'i>> {
            use style_traits::Separator;


            ::style_traits::Comma::parse(input, |parser| {
                single_value::parse(context, parser)
            }).map(SpecifiedValue)
        }

        pub use self::single_value::SpecifiedValue as SingleSpecifiedValue;

        impl SpecifiedValue {
            pub fn compute_iter<'a, 'cx, 'cx_a>(&'a self, context: &'cx Context<'cx_a>)
                -> computed_value::Iter<'a, 'cx, 'cx_a> {
                computed_value::Iter::new(context, &self.0)
            }
        }

        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, context: &Context) -> computed_value::T {
                computed_value::T(self.compute_iter(context).collect())
            }

            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                SpecifiedValue(computed.0.iter()
                                    .map(ToComputedValue::from_computed_value)
                                    .collect())
            }
        }
    
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::BackgroundRepeat(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::BackgroundRepeat);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::BackgroundRepeat);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            // In the case of a vector property we want to pass
                            // down an iterator so that this can be computed
                            // without allocation
                            //
                            // However, computing requires a context, but the
                            // style struct being mutated is on the context. We
                            // temporarily remove it, mutate it, and then put it
                            // back. Vector longhands cannot touch their own
                            // style struct whilst computing, else this will
                            // panic.
                            let mut s =
                                context.builder.take_background();
                            {
                                let iter = specified_value.compute_iter(context);
                                s.set_background_repeat(iter);
                            }
                            context.builder.put_background(s);
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_background_repeat();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_background_repeat();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::BackgroundRepeat)
            }
    }




    
    

    
        
    
    
    /// https://drafts.csswg.org/css-backgrounds/#the-background-attachment
    pub mod background_attachment {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        use std::fmt;
        use style_traits::{Separator, ToCss};

        pub mod single_value {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::ShorthandId;
            #[allow(unused_imports)]
            use selectors::parser::SelectorParseError;
            #[allow(unused_imports)]
            use style_traits::{ParseError, StyleParseError};
            #[allow(unused_imports)]
            use values::computed::{Context, ToComputedValue};
            #[allow(unused_imports)]
            use values::{computed, specified};
            #[allow(unused_imports)]
            use values::{Auto, Either, None_, Normal};
            
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "scroll" => scroll,
                    "fixed" => fixed,
                    "local" => local,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::scroll
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::scroll
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::scroll => computed_value::T::scroll,
                        SpecifiedValue::fixed => computed_value::T::fixed,
                        SpecifiedValue::local => computed_value::T::local,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::scroll => SpecifiedValue::scroll,
                        computed_value::T::fixed => SpecifiedValue::fixed,
                        computed_value::T::local => SpecifiedValue::local,
                }
            }
        }
    
        
        }

        /// The definition of the computed value for background-attachment.
        pub mod computed_value {
            pub use super::single_value::computed_value as single_value;
            pub use self::single_value::T as SingleComputedValue;
            use smallvec::{IntoIter, SmallVec};
            use values::computed::ComputedVecIter;

            /// The computed value, effectively a list of single values.
            #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
            #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
            #[derive(Clone, Debug, PartialEq)]
            pub struct T(
                pub SmallVec<[single_value::T; 1]>,
            );


            pub type Iter<'a, 'cx, 'cx_a> = ComputedVecIter<'a, 'cx, 'cx_a, super::single_value::SpecifiedValue>;

            impl IntoIterator for T {
                type Item = single_value::T;
                type IntoIter = IntoIter<[single_value::T; 1]>;
                fn into_iter(self) -> Self::IntoIter {
                    self.0.into_iter()
                }
            }
        }

        impl ToCss for computed_value::T {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        warn!("Found empty value for property background-attachment");
                }
                for i in iter {
                    dest.write_str(::style_traits::Comma::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        /// The specified value of background-attachment.
        #[derive(Clone, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        pub struct SpecifiedValue(pub Vec<single_value::SpecifiedValue>);

        impl ToCss for SpecifiedValue {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        warn!("Found empty value for property background-attachment");
                }
                for i in iter {
                    dest.write_str(::style_traits::Comma::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        pub fn get_initial_value() -> computed_value::T {
                let mut v = SmallVec::new();
                v.push(single_value::get_initial_value());
                computed_value::T(v)
        }

        pub fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                -> Result<SpecifiedValue, ParseError<'i>> {
            use style_traits::Separator;


            ::style_traits::Comma::parse(input, |parser| {
                single_value::parse(context, parser)
            }).map(SpecifiedValue)
        }

        pub use self::single_value::SpecifiedValue as SingleSpecifiedValue;

        impl SpecifiedValue {
            pub fn compute_iter<'a, 'cx, 'cx_a>(&'a self, context: &'cx Context<'cx_a>)
                -> computed_value::Iter<'a, 'cx, 'cx_a> {
                computed_value::Iter::new(context, &self.0)
            }
        }

        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, context: &Context) -> computed_value::T {
                computed_value::T(self.compute_iter(context).collect())
            }

            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                SpecifiedValue(computed.0.iter()
                                    .map(ToComputedValue::from_computed_value)
                                    .collect())
            }
        }
    
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::BackgroundAttachment(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::BackgroundAttachment);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::BackgroundAttachment);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            // In the case of a vector property we want to pass
                            // down an iterator so that this can be computed
                            // without allocation
                            //
                            // However, computing requires a context, but the
                            // style struct being mutated is on the context. We
                            // temporarily remove it, mutate it, and then put it
                            // back. Vector longhands cannot touch their own
                            // style struct whilst computing, else this will
                            // panic.
                            let mut s =
                                context.builder.take_background();
                            {
                                let iter = specified_value.compute_iter(context);
                                s.set_background_attachment(iter);
                            }
                            context.builder.put_background(s);
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_background_attachment();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_background_attachment();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::BackgroundAttachment)
            }
    }






    
    

    
        
    
    
    /// https://drafts.csswg.org/css-backgrounds/#the-background-clip
    pub mod background_clip {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        use std::fmt;
        use style_traits::{Separator, ToCss};

        pub mod single_value {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::ShorthandId;
            #[allow(unused_imports)]
            use selectors::parser::SelectorParseError;
            #[allow(unused_imports)]
            use style_traits::{ParseError, StyleParseError};
            #[allow(unused_imports)]
            use values::computed::{Context, ToComputedValue};
            #[allow(unused_imports)]
            use values::{computed, specified};
            #[allow(unused_imports)]
            use values::{Auto, Either, None_, Normal};
            
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "border-box" => border_box,
                    "padding-box" => padding_box,
                    "content-box" => content_box,
                    "text" => text,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::border_box
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::border_box
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::border_box => computed_value::T::border_box,
                        SpecifiedValue::padding_box => computed_value::T::padding_box,
                        SpecifiedValue::content_box => computed_value::T::content_box,
                        SpecifiedValue::text => computed_value::T::text,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::border_box => SpecifiedValue::border_box,
                        computed_value::T::padding_box => SpecifiedValue::padding_box,
                        computed_value::T::content_box => SpecifiedValue::content_box,
                        computed_value::T::text => SpecifiedValue::text,
                }
            }
        }
    
        
        }

        /// The definition of the computed value for background-clip.
        pub mod computed_value {
            pub use super::single_value::computed_value as single_value;
            pub use self::single_value::T as SingleComputedValue;
            use smallvec::{IntoIter, SmallVec};
            use values::computed::ComputedVecIter;

            /// The computed value, effectively a list of single values.
            #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
            #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
            #[derive(Clone, Debug, PartialEq)]
            pub struct T(
                pub SmallVec<[single_value::T; 1]>,
            );


            pub type Iter<'a, 'cx, 'cx_a> = ComputedVecIter<'a, 'cx, 'cx_a, super::single_value::SpecifiedValue>;

            impl IntoIterator for T {
                type Item = single_value::T;
                type IntoIter = IntoIter<[single_value::T; 1]>;
                fn into_iter(self) -> Self::IntoIter {
                    self.0.into_iter()
                }
            }
        }

        impl ToCss for computed_value::T {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        warn!("Found empty value for property background-clip");
                }
                for i in iter {
                    dest.write_str(::style_traits::Comma::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        /// The specified value of background-clip.
        #[derive(Clone, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        pub struct SpecifiedValue(pub Vec<single_value::SpecifiedValue>);

        impl ToCss for SpecifiedValue {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        warn!("Found empty value for property background-clip");
                }
                for i in iter {
                    dest.write_str(::style_traits::Comma::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        pub fn get_initial_value() -> computed_value::T {
                let mut v = SmallVec::new();
                v.push(single_value::get_initial_value());
                computed_value::T(v)
        }

        pub fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                -> Result<SpecifiedValue, ParseError<'i>> {
            use style_traits::Separator;


            ::style_traits::Comma::parse(input, |parser| {
                single_value::parse(context, parser)
            }).map(SpecifiedValue)
        }

        pub use self::single_value::SpecifiedValue as SingleSpecifiedValue;

        impl SpecifiedValue {
            pub fn compute_iter<'a, 'cx, 'cx_a>(&'a self, context: &'cx Context<'cx_a>)
                -> computed_value::Iter<'a, 'cx, 'cx_a> {
                computed_value::Iter::new(context, &self.0)
            }
        }

        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, context: &Context) -> computed_value::T {
                computed_value::T(self.compute_iter(context).collect())
            }

            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                SpecifiedValue(computed.0.iter()
                                    .map(ToComputedValue::from_computed_value)
                                    .collect())
            }
        }
    
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::BackgroundClip(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::BackgroundClip);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::BackgroundClip);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            // In the case of a vector property we want to pass
                            // down an iterator so that this can be computed
                            // without allocation
                            //
                            // However, computing requires a context, but the
                            // style struct being mutated is on the context. We
                            // temporarily remove it, mutate it, and then put it
                            // back. Vector longhands cannot touch their own
                            // style struct whilst computing, else this will
                            // panic.
                            let mut s =
                                context.builder.take_background();
                            {
                                let iter = specified_value.compute_iter(context);
                                s.set_background_clip(iter);
                            }
                            context.builder.put_background(s);
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_background_clip();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_background_clip();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::BackgroundClip)
            }
    }






    
    

    
        
    
    
    /// https://drafts.csswg.org/css-backgrounds/#the-background-origin
    pub mod background_origin {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        use std::fmt;
        use style_traits::{Separator, ToCss};

        pub mod single_value {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::ShorthandId;
            #[allow(unused_imports)]
            use selectors::parser::SelectorParseError;
            #[allow(unused_imports)]
            use style_traits::{ParseError, StyleParseError};
            #[allow(unused_imports)]
            use values::computed::{Context, ToComputedValue};
            #[allow(unused_imports)]
            use values::{computed, specified};
            #[allow(unused_imports)]
            use values::{Auto, Either, None_, Normal};
            
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "padding-box" => padding_box,
                    "border-box" => border_box,
                    "content-box" => content_box,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::padding_box
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::padding_box
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::padding_box => computed_value::T::padding_box,
                        SpecifiedValue::border_box => computed_value::T::border_box,
                        SpecifiedValue::content_box => computed_value::T::content_box,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::padding_box => SpecifiedValue::padding_box,
                        computed_value::T::border_box => SpecifiedValue::border_box,
                        computed_value::T::content_box => SpecifiedValue::content_box,
                }
            }
        }
    
        
        }

        /// The definition of the computed value for background-origin.
        pub mod computed_value {
            pub use super::single_value::computed_value as single_value;
            pub use self::single_value::T as SingleComputedValue;
            use smallvec::{IntoIter, SmallVec};
            use values::computed::ComputedVecIter;

            /// The computed value, effectively a list of single values.
            #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
            #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
            #[derive(Clone, Debug, PartialEq)]
            pub struct T(
                pub SmallVec<[single_value::T; 1]>,
            );


            pub type Iter<'a, 'cx, 'cx_a> = ComputedVecIter<'a, 'cx, 'cx_a, super::single_value::SpecifiedValue>;

            impl IntoIterator for T {
                type Item = single_value::T;
                type IntoIter = IntoIter<[single_value::T; 1]>;
                fn into_iter(self) -> Self::IntoIter {
                    self.0.into_iter()
                }
            }
        }

        impl ToCss for computed_value::T {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        warn!("Found empty value for property background-origin");
                }
                for i in iter {
                    dest.write_str(::style_traits::Comma::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        /// The specified value of background-origin.
        #[derive(Clone, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        pub struct SpecifiedValue(pub Vec<single_value::SpecifiedValue>);

        impl ToCss for SpecifiedValue {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        warn!("Found empty value for property background-origin");
                }
                for i in iter {
                    dest.write_str(::style_traits::Comma::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        pub fn get_initial_value() -> computed_value::T {
                let mut v = SmallVec::new();
                v.push(single_value::get_initial_value());
                computed_value::T(v)
        }

        pub fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                -> Result<SpecifiedValue, ParseError<'i>> {
            use style_traits::Separator;


            ::style_traits::Comma::parse(input, |parser| {
                single_value::parse(context, parser)
            }).map(SpecifiedValue)
        }

        pub use self::single_value::SpecifiedValue as SingleSpecifiedValue;

        impl SpecifiedValue {
            pub fn compute_iter<'a, 'cx, 'cx_a>(&'a self, context: &'cx Context<'cx_a>)
                -> computed_value::Iter<'a, 'cx, 'cx_a> {
                computed_value::Iter::new(context, &self.0)
            }
        }

        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, context: &Context) -> computed_value::T {
                computed_value::T(self.compute_iter(context).collect())
            }

            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                SpecifiedValue(computed.0.iter()
                                    .map(ToComputedValue::from_computed_value)
                                    .collect())
            }
        }
    
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::BackgroundOrigin(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::BackgroundOrigin);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::BackgroundOrigin);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            // In the case of a vector property we want to pass
                            // down an iterator so that this can be computed
                            // without allocation
                            //
                            // However, computing requires a context, but the
                            // style struct being mutated is on the context. We
                            // temporarily remove it, mutate it, and then put it
                            // back. Vector longhands cannot touch their own
                            // style struct whilst computing, else this will
                            // panic.
                            let mut s =
                                context.builder.take_background();
                            {
                                let iter = specified_value.compute_iter(context);
                                s.set_background_origin(iter);
                            }
                            context.builder.put_background(s);
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_background_origin();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_background_origin();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::BackgroundOrigin)
            }
    }






    
        
    
    
    /// https://drafts.csswg.org/css-backgrounds/#the-background-size
    pub mod background_size {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        use std::fmt;
        use style_traits::{Separator, ToCss};

        pub mod single_value {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::ShorthandId;
            #[allow(unused_imports)]
            use selectors::parser::SelectorParseError;
            #[allow(unused_imports)]
            use style_traits::{ParseError, StyleParseError};
            #[allow(unused_imports)]
            use values::computed::{Context, ToComputedValue};
            #[allow(unused_imports)]
            use values::{computed, specified};
            #[allow(unused_imports)]
            use values::{Auto, Either, None_, Normal};
            
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::BackgroundSize as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::BackgroundSize as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::BackgroundSize::auto() }
        #[inline] pub fn get_initial_specified_value() -> SpecifiedValue { specified::BackgroundSize::auto() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::BackgroundSize::parse(context, input)
        }
    
        
        }

        /// The definition of the computed value for background-size.
        pub mod computed_value {
            pub use super::single_value::computed_value as single_value;
            pub use self::single_value::T as SingleComputedValue;
            use smallvec::{IntoIter, SmallVec};
            use values::computed::ComputedVecIter;

            /// The computed value, effectively a list of single values.
            #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
            #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
            #[derive(Clone, Debug, PartialEq)]
            #[derive(Animate, ComputeSquaredDistance)]
            pub struct T(
                pub SmallVec<[single_value::T; 1]>,
            );

                use values::animated::{ToAnimatedZero};

                impl ToAnimatedZero for T {
                    #[inline]
                    fn to_animated_zero(&self) -> Result<Self, ()> { Err(()) }
                }

            pub type Iter<'a, 'cx, 'cx_a> = ComputedVecIter<'a, 'cx, 'cx_a, super::single_value::SpecifiedValue>;

            impl IntoIterator for T {
                type Item = single_value::T;
                type IntoIter = IntoIter<[single_value::T; 1]>;
                fn into_iter(self) -> Self::IntoIter {
                    self.0.into_iter()
                }
            }
        }

        impl ToCss for computed_value::T {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        warn!("Found empty value for property background-size");
                }
                for i in iter {
                    dest.write_str(::style_traits::Comma::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        /// The specified value of background-size.
        #[derive(Clone, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        pub struct SpecifiedValue(pub Vec<single_value::SpecifiedValue>);

        impl ToCss for SpecifiedValue {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        warn!("Found empty value for property background-size");
                }
                for i in iter {
                    dest.write_str(::style_traits::Comma::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        pub fn get_initial_value() -> computed_value::T {
                let mut v = SmallVec::new();
                v.push(single_value::get_initial_value());
                computed_value::T(v)
        }

        pub fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                -> Result<SpecifiedValue, ParseError<'i>> {
            use style_traits::Separator;


            ::style_traits::Comma::parse(input, |parser| {
                single_value::parse(context, parser)
            }).map(SpecifiedValue)
        }

        pub use self::single_value::SpecifiedValue as SingleSpecifiedValue;

        impl SpecifiedValue {
            pub fn compute_iter<'a, 'cx, 'cx_a>(&'a self, context: &'cx Context<'cx_a>)
                -> computed_value::Iter<'a, 'cx, 'cx_a> {
                computed_value::Iter::new(context, &self.0)
            }
        }

        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, context: &Context) -> computed_value::T {
                computed_value::T(self.compute_iter(context).collect())
            }

            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                SpecifiedValue(computed.0.iter()
                                    .map(ToComputedValue::from_computed_value)
                                    .collect())
            }
        }
    
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::BackgroundSize(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::BackgroundSize);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::BackgroundSize);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            // In the case of a vector property we want to pass
                            // down an iterator so that this can be computed
                            // without allocation
                            //
                            // However, computing requires a context, but the
                            // style struct being mutated is on the context. We
                            // temporarily remove it, mutate it, and then put it
                            // back. Vector longhands cannot touch their own
                            // style struct whilst computing, else this will
                            // panic.
                            let mut s =
                                context.builder.take_background();
                            {
                                let iter = specified_value.compute_iter(context);
                                s.set_background_size(iter);
                            }
                            context.builder.put_background(s);
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_background_size();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_background_size();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::BackgroundSize)
            }
    }




// https://drafts.fxtf.org/compositing/#background-blend-mode

    
    

    
        
    
    
    /// https://drafts.fxtf.org/compositing/#background-blend-mode
    pub mod background_blend_mode {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        use std::fmt;
        use style_traits::{Separator, ToCss};

        pub mod single_value {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::ShorthandId;
            #[allow(unused_imports)]
            use selectors::parser::SelectorParseError;
            #[allow(unused_imports)]
            use style_traits::{ParseError, StyleParseError};
            #[allow(unused_imports)]
            use values::computed::{Context, ToComputedValue};
            #[allow(unused_imports)]
            use values::{computed, specified};
            #[allow(unused_imports)]
            use values::{Auto, Either, None_, Normal};
            
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "normal" => normal,
                    "multiply" => multiply,
                    "screen" => screen,
                    "overlay" => overlay,
                    "darken" => darken,
                    "lighten" => lighten,
                    "color-dodge" => color_dodge,
                    "color-burn" => color_burn,
                    "hard-light" => hard_light,
                    "soft-light" => soft_light,
                    "difference" => difference,
                    "exclusion" => exclusion,
                    "hue" => hue,
                    "saturation" => saturation,
                    "color" => color,
                    "luminosity" => luminosity,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::normal
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::normal
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::normal => computed_value::T::normal,
                        SpecifiedValue::multiply => computed_value::T::multiply,
                        SpecifiedValue::screen => computed_value::T::screen,
                        SpecifiedValue::overlay => computed_value::T::overlay,
                        SpecifiedValue::darken => computed_value::T::darken,
                        SpecifiedValue::lighten => computed_value::T::lighten,
                        SpecifiedValue::color_dodge => computed_value::T::color_dodge,
                        SpecifiedValue::color_burn => computed_value::T::color_burn,
                        SpecifiedValue::hard_light => computed_value::T::hard_light,
                        SpecifiedValue::soft_light => computed_value::T::soft_light,
                        SpecifiedValue::difference => computed_value::T::difference,
                        SpecifiedValue::exclusion => computed_value::T::exclusion,
                        SpecifiedValue::hue => computed_value::T::hue,
                        SpecifiedValue::saturation => computed_value::T::saturation,
                        SpecifiedValue::color => computed_value::T::color,
                        SpecifiedValue::luminosity => computed_value::T::luminosity,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::normal => SpecifiedValue::normal,
                        computed_value::T::multiply => SpecifiedValue::multiply,
                        computed_value::T::screen => SpecifiedValue::screen,
                        computed_value::T::overlay => SpecifiedValue::overlay,
                        computed_value::T::darken => SpecifiedValue::darken,
                        computed_value::T::lighten => SpecifiedValue::lighten,
                        computed_value::T::color_dodge => SpecifiedValue::color_dodge,
                        computed_value::T::color_burn => SpecifiedValue::color_burn,
                        computed_value::T::hard_light => SpecifiedValue::hard_light,
                        computed_value::T::soft_light => SpecifiedValue::soft_light,
                        computed_value::T::difference => SpecifiedValue::difference,
                        computed_value::T::exclusion => SpecifiedValue::exclusion,
                        computed_value::T::hue => SpecifiedValue::hue,
                        computed_value::T::saturation => SpecifiedValue::saturation,
                        computed_value::T::color => SpecifiedValue::color,
                        computed_value::T::luminosity => SpecifiedValue::luminosity,
                }
            }
        }
    
        
        }

        /// The definition of the computed value for background-blend-mode.
        pub mod computed_value {
            pub use super::single_value::computed_value as single_value;
            pub use self::single_value::T as SingleComputedValue;
            use smallvec::{IntoIter, SmallVec};
            use values::computed::ComputedVecIter;

            /// The computed value, effectively a list of single values.
            #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
            #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
            #[derive(Clone, Debug, PartialEq)]
            pub struct T(
                pub SmallVec<[single_value::T; 1]>,
            );


            pub type Iter<'a, 'cx, 'cx_a> = ComputedVecIter<'a, 'cx, 'cx_a, super::single_value::SpecifiedValue>;

            impl IntoIterator for T {
                type Item = single_value::T;
                type IntoIter = IntoIter<[single_value::T; 1]>;
                fn into_iter(self) -> Self::IntoIter {
                    self.0.into_iter()
                }
            }
        }

        impl ToCss for computed_value::T {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        warn!("Found empty value for property background-blend-mode");
                }
                for i in iter {
                    dest.write_str(::style_traits::Comma::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        /// The specified value of background-blend-mode.
        #[derive(Clone, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        pub struct SpecifiedValue(pub Vec<single_value::SpecifiedValue>);

        impl ToCss for SpecifiedValue {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        warn!("Found empty value for property background-blend-mode");
                }
                for i in iter {
                    dest.write_str(::style_traits::Comma::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        pub fn get_initial_value() -> computed_value::T {
                let mut v = SmallVec::new();
                v.push(single_value::get_initial_value());
                computed_value::T(v)
        }

        pub fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                -> Result<SpecifiedValue, ParseError<'i>> {
            use style_traits::Separator;


            ::style_traits::Comma::parse(input, |parser| {
                single_value::parse(context, parser)
            }).map(SpecifiedValue)
        }

        pub use self::single_value::SpecifiedValue as SingleSpecifiedValue;

        impl SpecifiedValue {
            pub fn compute_iter<'a, 'cx, 'cx_a>(&'a self, context: &'cx Context<'cx_a>)
                -> computed_value::Iter<'a, 'cx, 'cx_a> {
                computed_value::Iter::new(context, &self.0)
            }
        }

        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, context: &Context) -> computed_value::T {
                computed_value::T(self.compute_iter(context).collect())
            }

            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                SpecifiedValue(computed.0.iter()
                                    .map(ToComputedValue::from_computed_value)
                                    .collect())
            }
        }
    
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::BackgroundBlendMode(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::BackgroundBlendMode);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::BackgroundBlendMode);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            // In the case of a vector property we want to pass
                            // down an iterator so that this can be computed
                            // without allocation
                            //
                            // However, computing requires a context, but the
                            // style struct being mutated is on the context. We
                            // temporarily remove it, mutate it, and then put it
                            // back. Vector longhands cannot touch their own
                            // style struct whilst computing, else this will
                            // panic.
                            let mut s =
                                context.builder.take_background();
                            {
                                let iter = specified_value.compute_iter(context);
                                s.set_background_blend_mode(iter);
                            }
                            context.builder.put_background(s);
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_background_blend_mode();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_background_blend_mode();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::BackgroundBlendMode)
            }
    }





    /* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */






    
    
    
        
    
    /// https://drafts.csswg.org/css-backgrounds/#border-top-color
    pub mod border_top_color {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::Color as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::Color as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed_value::T::currentcolor() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::Color::parse_quirky(context, input, AllowQuirks::Yes)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::BorderTopColor(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::BorderTopColor);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::BorderTopColor);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_border_top_color(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_border_top_color();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_border_top_color();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::BorderTopColor)
            }
    }



    
    
        
    
    /// https://drafts.csswg.org/css-backgrounds/#border-top-style
    pub mod border_top_style {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::BorderStyle as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::BorderStyle as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { specified::BorderStyle::none }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::BorderStyle::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::BorderTopStyle(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::BorderTopStyle);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::BorderTopStyle);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_border_top_style(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_border_top_style();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_border_top_style();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::BorderTopStyle)
            }
    }



    
    
        
    
    /// https://drafts.csswg.org/css-backgrounds/#border-top-width
    pub mod border_top_width {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::BorderSideWidth as SpecifiedValue;
        pub mod computed_value {
            pub use ::values::computed::NonNegativeLength as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { ::values::computed::NonNegativeLength::new(3.) }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::BorderSideWidth::parse_quirky(context, input, AllowQuirks::Yes)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::BorderTopWidth(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::BorderTopWidth);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::BorderTopWidth);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_border_top_width(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_border_top_width();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_border_top_width();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::BorderTopWidth)
            }
    }


    
    
    
        
    
    /// https://drafts.csswg.org/css-backgrounds/#border-left-color
    pub mod border_left_color {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::Color as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::Color as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed_value::T::currentcolor() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::Color::parse_quirky(context, input, AllowQuirks::Yes)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::BorderLeftColor(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::BorderLeftColor);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::BorderLeftColor);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_border_left_color(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_border_left_color();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_border_left_color();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::BorderLeftColor)
            }
    }



    
    
        
    
    /// https://drafts.csswg.org/css-backgrounds/#border-left-style
    pub mod border_left_style {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::BorderStyle as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::BorderStyle as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { specified::BorderStyle::none }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::BorderStyle::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::BorderLeftStyle(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::BorderLeftStyle);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::BorderLeftStyle);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_border_left_style(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_border_left_style();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_border_left_style();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::BorderLeftStyle)
            }
    }



    
    
        
    
    /// https://drafts.csswg.org/css-backgrounds/#border-left-width
    pub mod border_left_width {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::BorderSideWidth as SpecifiedValue;
        pub mod computed_value {
            pub use ::values::computed::NonNegativeLength as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { ::values::computed::NonNegativeLength::new(3.) }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::BorderSideWidth::parse_quirky(context, input, AllowQuirks::Yes)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::BorderLeftWidth(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::BorderLeftWidth);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::BorderLeftWidth);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_border_left_width(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_border_left_width();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_border_left_width();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::BorderLeftWidth)
            }
    }


    
    
    
        
    
    /// https://drafts.csswg.org/css-backgrounds/#border-bottom-color
    pub mod border_bottom_color {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::Color as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::Color as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed_value::T::currentcolor() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::Color::parse_quirky(context, input, AllowQuirks::Yes)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::BorderBottomColor(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::BorderBottomColor);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::BorderBottomColor);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_border_bottom_color(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_border_bottom_color();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_border_bottom_color();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::BorderBottomColor)
            }
    }



    
    
        
    
    /// https://drafts.csswg.org/css-backgrounds/#border-bottom-style
    pub mod border_bottom_style {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::BorderStyle as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::BorderStyle as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { specified::BorderStyle::none }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::BorderStyle::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::BorderBottomStyle(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::BorderBottomStyle);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::BorderBottomStyle);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_border_bottom_style(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_border_bottom_style();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_border_bottom_style();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::BorderBottomStyle)
            }
    }



    
    
        
    
    /// https://drafts.csswg.org/css-backgrounds/#border-bottom-width
    pub mod border_bottom_width {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::BorderSideWidth as SpecifiedValue;
        pub mod computed_value {
            pub use ::values::computed::NonNegativeLength as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { ::values::computed::NonNegativeLength::new(3.) }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::BorderSideWidth::parse_quirky(context, input, AllowQuirks::Yes)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::BorderBottomWidth(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::BorderBottomWidth);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::BorderBottomWidth);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_border_bottom_width(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_border_bottom_width();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_border_bottom_width();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::BorderBottomWidth)
            }
    }


    
    
    
        
    
    /// https://drafts.csswg.org/css-backgrounds/#border-right-color
    pub mod border_right_color {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::Color as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::Color as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed_value::T::currentcolor() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::Color::parse_quirky(context, input, AllowQuirks::Yes)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::BorderRightColor(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::BorderRightColor);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::BorderRightColor);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_border_right_color(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_border_right_color();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_border_right_color();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::BorderRightColor)
            }
    }



    
    
        
    
    /// https://drafts.csswg.org/css-backgrounds/#border-right-style
    pub mod border_right_style {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::BorderStyle as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::BorderStyle as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { specified::BorderStyle::none }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::BorderStyle::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::BorderRightStyle(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::BorderRightStyle);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::BorderRightStyle);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_border_right_style(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_border_right_style();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_border_right_style();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::BorderRightStyle)
            }
    }



    
    
        
    
    /// https://drafts.csswg.org/css-backgrounds/#border-right-width
    pub mod border_right_width {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::BorderSideWidth as SpecifiedValue;
        pub mod computed_value {
            pub use ::values::computed::NonNegativeLength as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { ::values::computed::NonNegativeLength::new(3.) }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::BorderSideWidth::parse_quirky(context, input, AllowQuirks::Yes)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::BorderRightWidth(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::BorderRightWidth);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::BorderRightWidth);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_border_right_width(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_border_right_width();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_border_right_width();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::BorderRightWidth)
            }
    }


    
    
    
        
    
    /// https://drafts.csswg.org/css-logical-props/#propdef-border-block-start-color
    pub mod border_block_start_color {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::Color as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::Color as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed_value::T::currentcolor() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::Color::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::BorderBlockStartColor(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::BorderBlockStartColor);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::BorderBlockStartColor);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            context.rule_cache_conditions.borrow_mut()
                                .set_writing_mode_dependency(context.builder.writing_mode);
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_border_block_start_color(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_border_block_start_color();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_border_block_start_color();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::BorderBlockStartColor)
            }
    }



    
    
        
    
    /// https://drafts.csswg.org/css-logical-props/#propdef-border-block-start-style
    pub mod border_block_start_style {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::BorderStyle as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::BorderStyle as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { specified::BorderStyle::none }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::BorderStyle::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::BorderBlockStartStyle(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::BorderBlockStartStyle);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::BorderBlockStartStyle);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            context.rule_cache_conditions.borrow_mut()
                                .set_writing_mode_dependency(context.builder.writing_mode);
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_border_block_start_style(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_border_block_start_style();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_border_block_start_style();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::BorderBlockStartStyle)
            }
    }



    
    
        
    
    /// https://drafts.csswg.org/css-logical-props/#propdef-border-block-start-width
    pub mod border_block_start_width {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::BorderSideWidth as SpecifiedValue;
        pub mod computed_value {
            pub use ::values::computed::NonNegativeLength as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { ::values::computed::NonNegativeLength::new(3.) }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::BorderSideWidth::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::BorderBlockStartWidth(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::BorderBlockStartWidth);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::BorderBlockStartWidth);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            context.rule_cache_conditions.borrow_mut()
                                .set_writing_mode_dependency(context.builder.writing_mode);
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_border_block_start_width(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_border_block_start_width();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_border_block_start_width();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::BorderBlockStartWidth)
            }
    }


    
    
    
        
    
    /// https://drafts.csswg.org/css-logical-props/#propdef-border-block-end-color
    pub mod border_block_end_color {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::Color as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::Color as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed_value::T::currentcolor() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::Color::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::BorderBlockEndColor(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::BorderBlockEndColor);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::BorderBlockEndColor);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            context.rule_cache_conditions.borrow_mut()
                                .set_writing_mode_dependency(context.builder.writing_mode);
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_border_block_end_color(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_border_block_end_color();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_border_block_end_color();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::BorderBlockEndColor)
            }
    }



    
    
        
    
    /// https://drafts.csswg.org/css-logical-props/#propdef-border-block-end-style
    pub mod border_block_end_style {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::BorderStyle as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::BorderStyle as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { specified::BorderStyle::none }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::BorderStyle::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::BorderBlockEndStyle(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::BorderBlockEndStyle);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::BorderBlockEndStyle);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            context.rule_cache_conditions.borrow_mut()
                                .set_writing_mode_dependency(context.builder.writing_mode);
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_border_block_end_style(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_border_block_end_style();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_border_block_end_style();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::BorderBlockEndStyle)
            }
    }



    
    
        
    
    /// https://drafts.csswg.org/css-logical-props/#propdef-border-block-end-width
    pub mod border_block_end_width {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::BorderSideWidth as SpecifiedValue;
        pub mod computed_value {
            pub use ::values::computed::NonNegativeLength as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { ::values::computed::NonNegativeLength::new(3.) }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::BorderSideWidth::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::BorderBlockEndWidth(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::BorderBlockEndWidth);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::BorderBlockEndWidth);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            context.rule_cache_conditions.borrow_mut()
                                .set_writing_mode_dependency(context.builder.writing_mode);
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_border_block_end_width(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_border_block_end_width();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_border_block_end_width();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::BorderBlockEndWidth)
            }
    }


    
    
    
        
    
    /// https://drafts.csswg.org/css-logical-props/#propdef-border-inline-start-color
    pub mod border_inline_start_color {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::Color as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::Color as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed_value::T::currentcolor() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::Color::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::BorderInlineStartColor(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::BorderInlineStartColor);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::BorderInlineStartColor);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            context.rule_cache_conditions.borrow_mut()
                                .set_writing_mode_dependency(context.builder.writing_mode);
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_border_inline_start_color(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_border_inline_start_color();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_border_inline_start_color();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::BorderInlineStartColor)
            }
    }



    
    
        
    
    /// https://drafts.csswg.org/css-logical-props/#propdef-border-inline-start-style
    pub mod border_inline_start_style {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::BorderStyle as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::BorderStyle as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { specified::BorderStyle::none }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::BorderStyle::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::BorderInlineStartStyle(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::BorderInlineStartStyle);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::BorderInlineStartStyle);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            context.rule_cache_conditions.borrow_mut()
                                .set_writing_mode_dependency(context.builder.writing_mode);
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_border_inline_start_style(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_border_inline_start_style();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_border_inline_start_style();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::BorderInlineStartStyle)
            }
    }



    
    
        
    
    /// https://drafts.csswg.org/css-logical-props/#propdef-border-inline-start-width
    pub mod border_inline_start_width {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::BorderSideWidth as SpecifiedValue;
        pub mod computed_value {
            pub use ::values::computed::NonNegativeLength as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { ::values::computed::NonNegativeLength::new(3.) }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::BorderSideWidth::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::BorderInlineStartWidth(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::BorderInlineStartWidth);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::BorderInlineStartWidth);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            context.rule_cache_conditions.borrow_mut()
                                .set_writing_mode_dependency(context.builder.writing_mode);
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_border_inline_start_width(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_border_inline_start_width();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_border_inline_start_width();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::BorderInlineStartWidth)
            }
    }


    
    
    
        
    
    /// https://drafts.csswg.org/css-logical-props/#propdef-border-inline-end-color
    pub mod border_inline_end_color {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::Color as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::Color as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed_value::T::currentcolor() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::Color::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::BorderInlineEndColor(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::BorderInlineEndColor);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::BorderInlineEndColor);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            context.rule_cache_conditions.borrow_mut()
                                .set_writing_mode_dependency(context.builder.writing_mode);
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_border_inline_end_color(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_border_inline_end_color();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_border_inline_end_color();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::BorderInlineEndColor)
            }
    }



    
    
        
    
    /// https://drafts.csswg.org/css-logical-props/#propdef-border-inline-end-style
    pub mod border_inline_end_style {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::BorderStyle as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::BorderStyle as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { specified::BorderStyle::none }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::BorderStyle::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::BorderInlineEndStyle(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::BorderInlineEndStyle);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::BorderInlineEndStyle);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            context.rule_cache_conditions.borrow_mut()
                                .set_writing_mode_dependency(context.builder.writing_mode);
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_border_inline_end_style(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_border_inline_end_style();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_border_inline_end_style();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::BorderInlineEndStyle)
            }
    }



    
    
        
    
    /// https://drafts.csswg.org/css-logical-props/#propdef-border-inline-end-width
    pub mod border_inline_end_width {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::BorderSideWidth as SpecifiedValue;
        pub mod computed_value {
            pub use ::values::computed::NonNegativeLength as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { ::values::computed::NonNegativeLength::new(3.) }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::BorderSideWidth::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::BorderInlineEndWidth(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::BorderInlineEndWidth);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::BorderInlineEndWidth);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            context.rule_cache_conditions.borrow_mut()
                                .set_writing_mode_dependency(context.builder.writing_mode);
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_border_inline_end_width(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_border_inline_end_width();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_border_inline_end_width();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::BorderInlineEndWidth)
            }
    }




    
    #[cfg(feature = "gecko")]
    impl ::values::specified::BorderStyle {
        /// Obtain a specified value from a Gecko keyword value
        ///
        /// Intended for use with presentation attributes, not style structs
        pub fn from_gecko_keyword(kw: u32) -> Self {
            use gecko_bindings::structs;
                // We can't match on enum values if we're matching on a u32
                const NONE: u32
                    = structs::NS_STYLE_BORDER_STYLE_NONE as u32;
                // We can't match on enum values if we're matching on a u32
                const SOLID: u32
                    = structs::NS_STYLE_BORDER_STYLE_SOLID as u32;
                // We can't match on enum values if we're matching on a u32
                const DOUBLE: u32
                    = structs::NS_STYLE_BORDER_STYLE_DOUBLE as u32;
                // We can't match on enum values if we're matching on a u32
                const DOTTED: u32
                    = structs::NS_STYLE_BORDER_STYLE_DOTTED as u32;
                // We can't match on enum values if we're matching on a u32
                const DASHED: u32
                    = structs::NS_STYLE_BORDER_STYLE_DASHED as u32;
                // We can't match on enum values if we're matching on a u32
                const HIDDEN: u32
                    = structs::NS_STYLE_BORDER_STYLE_HIDDEN as u32;
                // We can't match on enum values if we're matching on a u32
                const GROOVE: u32
                    = structs::NS_STYLE_BORDER_STYLE_GROOVE as u32;
                // We can't match on enum values if we're matching on a u32
                const RIDGE: u32
                    = structs::NS_STYLE_BORDER_STYLE_RIDGE as u32;
                // We can't match on enum values if we're matching on a u32
                const INSET: u32
                    = structs::NS_STYLE_BORDER_STYLE_INSET as u32;
                // We can't match on enum values if we're matching on a u32
                const OUTSET: u32
                    = structs::NS_STYLE_BORDER_STYLE_OUTSET as u32;
            match kw  {
                    NONE => ::values::specified::BorderStyle::none,
                    SOLID => ::values::specified::BorderStyle::solid,
                    DOUBLE => ::values::specified::BorderStyle::double,
                    DOTTED => ::values::specified::BorderStyle::dotted,
                    DASHED => ::values::specified::BorderStyle::dashed,
                    HIDDEN => ::values::specified::BorderStyle::hidden,
                    GROOVE => ::values::specified::BorderStyle::groove,
                    RIDGE => ::values::specified::BorderStyle::ridge,
                    INSET => ::values::specified::BorderStyle::inset,
                    OUTSET => ::values::specified::BorderStyle::outset,
                x => panic!("Found unexpected value in style struct for border-style property: {:?}", x),
            }
        }
    }


// FIXME(#4126): when gfx supports painting it, make this Size2D<LengthOrPercentage>
    
    
        
    
    /// https://drafts.csswg.org/css-backgrounds/#border-top-left-radius
    pub mod border_top_left_radius {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::BorderCornerRadius as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::BorderCornerRadius as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::BorderCornerRadius::zero() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::BorderCornerRadius::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::BorderTopLeftRadius(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::BorderTopLeftRadius);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::BorderTopLeftRadius);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = (**specified_value).to_computed_value(context);
                                context.builder.set_border_top_left_radius(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_border_top_left_radius();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_border_top_left_radius();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<Box<SpecifiedValue>, ParseError<'i>> {
                    parse(context, input).map(|result| Box::new(result))
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::BorderTopLeftRadius)
            }
    }


    
    
        
    
    /// https://drafts.csswg.org/css-backgrounds/#border-top-right-radius
    pub mod border_top_right_radius {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::BorderCornerRadius as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::BorderCornerRadius as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::BorderCornerRadius::zero() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::BorderCornerRadius::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::BorderTopRightRadius(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::BorderTopRightRadius);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::BorderTopRightRadius);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = (**specified_value).to_computed_value(context);
                                context.builder.set_border_top_right_radius(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_border_top_right_radius();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_border_top_right_radius();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<Box<SpecifiedValue>, ParseError<'i>> {
                    parse(context, input).map(|result| Box::new(result))
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::BorderTopRightRadius)
            }
    }


    
    
        
    
    /// https://drafts.csswg.org/css-backgrounds/#border-bottom-right-radius
    pub mod border_bottom_right_radius {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::BorderCornerRadius as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::BorderCornerRadius as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::BorderCornerRadius::zero() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::BorderCornerRadius::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::BorderBottomRightRadius(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::BorderBottomRightRadius);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::BorderBottomRightRadius);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = (**specified_value).to_computed_value(context);
                                context.builder.set_border_bottom_right_radius(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_border_bottom_right_radius();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_border_bottom_right_radius();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<Box<SpecifiedValue>, ParseError<'i>> {
                    parse(context, input).map(|result| Box::new(result))
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::BorderBottomRightRadius)
            }
    }


    
    
        
    
    /// https://drafts.csswg.org/css-backgrounds/#border-bottom-left-radius
    pub mod border_bottom_left_radius {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::BorderCornerRadius as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::BorderCornerRadius as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::BorderCornerRadius::zero() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::BorderCornerRadius::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::BorderBottomLeftRadius(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::BorderBottomLeftRadius);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::BorderBottomLeftRadius);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = (**specified_value).to_computed_value(context);
                                context.builder.set_border_bottom_left_radius(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_border_bottom_left_radius();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_border_bottom_left_radius();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<Box<SpecifiedValue>, ParseError<'i>> {
                    parse(context, input).map(|result| Box::new(result))
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::BorderBottomLeftRadius)
            }
    }



/// -moz-border-*-colors: color, string, enum, none, inherit/initial
/// These non-spec properties are just for Gecko (Stylo) internal use.
    
    
    /// Nonstandard (https://developer.mozilla.org/en-US/docs/Web/CSS/-moz-border-*-colors)
    pub mod _moz_border_top_colors {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
        use std::fmt;
        use style_traits::ToCss;
        use values::specified::RGBAColor;

        pub mod computed_value {
            use cssparser::RGBA;
            #[derive(Clone, Debug, PartialEq)]
            #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
            #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
            pub struct T(pub Option<Vec<RGBA>>);
        }

        #[derive(Clone, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        pub enum SpecifiedValue {
            None,
            Colors(Vec<RGBAColor>),
        }

        impl ToCss for computed_value::T {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
                match self.0 {
                    None => return dest.write_str("none"),
                    Some(ref vec) => {
                        let mut first = true;
                        for ref color in vec {
                            if !first {
                                dest.write_str(" ")?;
                            }
                            first = false;
                            color.to_css(dest)?
                        }
                        Ok(())
                    }
                }
            }
        }

        impl ToCss for SpecifiedValue {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
                match *self {
                    SpecifiedValue::None => return dest.write_str("none"),
                    SpecifiedValue::Colors(ref vec) => {
                        let mut first = true;
                        for ref color in vec {
                            if !first {
                                dest.write_str(" ")?;
                            }
                            first = false;
                            color.to_css(dest)?
                        }
                        Ok(())
                    }
                }
            }
        }

        #[inline] pub fn get_initial_value() -> computed_value::T {
            computed_value::T(None)
        }

        #[inline] pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::None
        }

        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, context: &Context) -> computed_value::T {
                match *self {
                    SpecifiedValue::Colors(ref vec) => {
                        computed_value::T(Some(vec.iter()
                                                  .map(|c| c.to_computed_value(context))
                                                  .collect()))
                    },
                    SpecifiedValue::None => {
                        computed_value::T(None)
                    }
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                    computed_value::T(Some(ref vec)) => {
                        SpecifiedValue::Colors(vec.iter()
                                                  .map(ToComputedValue::from_computed_value)
                                                  .collect())
                    },
                    computed_value::T(None) => {
                        SpecifiedValue::None
                    }
                }
            }
        }

        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            if input.try(|input| input.expect_ident_matching("none")).is_ok() {
                return Ok(SpecifiedValue::None)
            }

            let mut result = Vec::new();
            while let Ok(value) = input.try(|i| RGBAColor::parse(context, i)) {
                result.push(value);
            }

            if !result.is_empty() {
                Ok(SpecifiedValue::Colors(result))
            } else {
                Err(StyleParseError::UnspecifiedError.into())
            }
        }
    
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MozBorderTopColors(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MozBorderTopColors);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::MozBorderTopColors);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set__moz_border_top_colors(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset__moz_border_top_colors();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit__moz_border_top_colors();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MozBorderTopColors)
            }
    }

    
    
    /// Nonstandard (https://developer.mozilla.org/en-US/docs/Web/CSS/-moz-border-*-colors)
    pub mod _moz_border_left_colors {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
        use std::fmt;
        use style_traits::ToCss;
        use values::specified::RGBAColor;

        pub mod computed_value {
            use cssparser::RGBA;
            #[derive(Clone, Debug, PartialEq)]
            #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
            #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
            pub struct T(pub Option<Vec<RGBA>>);
        }

        #[derive(Clone, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        pub enum SpecifiedValue {
            None,
            Colors(Vec<RGBAColor>),
        }

        impl ToCss for computed_value::T {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
                match self.0 {
                    None => return dest.write_str("none"),
                    Some(ref vec) => {
                        let mut first = true;
                        for ref color in vec {
                            if !first {
                                dest.write_str(" ")?;
                            }
                            first = false;
                            color.to_css(dest)?
                        }
                        Ok(())
                    }
                }
            }
        }

        impl ToCss for SpecifiedValue {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
                match *self {
                    SpecifiedValue::None => return dest.write_str("none"),
                    SpecifiedValue::Colors(ref vec) => {
                        let mut first = true;
                        for ref color in vec {
                            if !first {
                                dest.write_str(" ")?;
                            }
                            first = false;
                            color.to_css(dest)?
                        }
                        Ok(())
                    }
                }
            }
        }

        #[inline] pub fn get_initial_value() -> computed_value::T {
            computed_value::T(None)
        }

        #[inline] pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::None
        }

        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, context: &Context) -> computed_value::T {
                match *self {
                    SpecifiedValue::Colors(ref vec) => {
                        computed_value::T(Some(vec.iter()
                                                  .map(|c| c.to_computed_value(context))
                                                  .collect()))
                    },
                    SpecifiedValue::None => {
                        computed_value::T(None)
                    }
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                    computed_value::T(Some(ref vec)) => {
                        SpecifiedValue::Colors(vec.iter()
                                                  .map(ToComputedValue::from_computed_value)
                                                  .collect())
                    },
                    computed_value::T(None) => {
                        SpecifiedValue::None
                    }
                }
            }
        }

        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            if input.try(|input| input.expect_ident_matching("none")).is_ok() {
                return Ok(SpecifiedValue::None)
            }

            let mut result = Vec::new();
            while let Ok(value) = input.try(|i| RGBAColor::parse(context, i)) {
                result.push(value);
            }

            if !result.is_empty() {
                Ok(SpecifiedValue::Colors(result))
            } else {
                Err(StyleParseError::UnspecifiedError.into())
            }
        }
    
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MozBorderLeftColors(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MozBorderLeftColors);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::MozBorderLeftColors);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set__moz_border_left_colors(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset__moz_border_left_colors();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit__moz_border_left_colors();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MozBorderLeftColors)
            }
    }

    
    
    /// Nonstandard (https://developer.mozilla.org/en-US/docs/Web/CSS/-moz-border-*-colors)
    pub mod _moz_border_bottom_colors {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
        use std::fmt;
        use style_traits::ToCss;
        use values::specified::RGBAColor;

        pub mod computed_value {
            use cssparser::RGBA;
            #[derive(Clone, Debug, PartialEq)]
            #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
            #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
            pub struct T(pub Option<Vec<RGBA>>);
        }

        #[derive(Clone, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        pub enum SpecifiedValue {
            None,
            Colors(Vec<RGBAColor>),
        }

        impl ToCss for computed_value::T {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
                match self.0 {
                    None => return dest.write_str("none"),
                    Some(ref vec) => {
                        let mut first = true;
                        for ref color in vec {
                            if !first {
                                dest.write_str(" ")?;
                            }
                            first = false;
                            color.to_css(dest)?
                        }
                        Ok(())
                    }
                }
            }
        }

        impl ToCss for SpecifiedValue {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
                match *self {
                    SpecifiedValue::None => return dest.write_str("none"),
                    SpecifiedValue::Colors(ref vec) => {
                        let mut first = true;
                        for ref color in vec {
                            if !first {
                                dest.write_str(" ")?;
                            }
                            first = false;
                            color.to_css(dest)?
                        }
                        Ok(())
                    }
                }
            }
        }

        #[inline] pub fn get_initial_value() -> computed_value::T {
            computed_value::T(None)
        }

        #[inline] pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::None
        }

        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, context: &Context) -> computed_value::T {
                match *self {
                    SpecifiedValue::Colors(ref vec) => {
                        computed_value::T(Some(vec.iter()
                                                  .map(|c| c.to_computed_value(context))
                                                  .collect()))
                    },
                    SpecifiedValue::None => {
                        computed_value::T(None)
                    }
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                    computed_value::T(Some(ref vec)) => {
                        SpecifiedValue::Colors(vec.iter()
                                                  .map(ToComputedValue::from_computed_value)
                                                  .collect())
                    },
                    computed_value::T(None) => {
                        SpecifiedValue::None
                    }
                }
            }
        }

        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            if input.try(|input| input.expect_ident_matching("none")).is_ok() {
                return Ok(SpecifiedValue::None)
            }

            let mut result = Vec::new();
            while let Ok(value) = input.try(|i| RGBAColor::parse(context, i)) {
                result.push(value);
            }

            if !result.is_empty() {
                Ok(SpecifiedValue::Colors(result))
            } else {
                Err(StyleParseError::UnspecifiedError.into())
            }
        }
    
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MozBorderBottomColors(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MozBorderBottomColors);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::MozBorderBottomColors);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set__moz_border_bottom_colors(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset__moz_border_bottom_colors();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit__moz_border_bottom_colors();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MozBorderBottomColors)
            }
    }

    
    
    /// Nonstandard (https://developer.mozilla.org/en-US/docs/Web/CSS/-moz-border-*-colors)
    pub mod _moz_border_right_colors {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
        use std::fmt;
        use style_traits::ToCss;
        use values::specified::RGBAColor;

        pub mod computed_value {
            use cssparser::RGBA;
            #[derive(Clone, Debug, PartialEq)]
            #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
            #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
            pub struct T(pub Option<Vec<RGBA>>);
        }

        #[derive(Clone, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        pub enum SpecifiedValue {
            None,
            Colors(Vec<RGBAColor>),
        }

        impl ToCss for computed_value::T {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
                match self.0 {
                    None => return dest.write_str("none"),
                    Some(ref vec) => {
                        let mut first = true;
                        for ref color in vec {
                            if !first {
                                dest.write_str(" ")?;
                            }
                            first = false;
                            color.to_css(dest)?
                        }
                        Ok(())
                    }
                }
            }
        }

        impl ToCss for SpecifiedValue {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
                match *self {
                    SpecifiedValue::None => return dest.write_str("none"),
                    SpecifiedValue::Colors(ref vec) => {
                        let mut first = true;
                        for ref color in vec {
                            if !first {
                                dest.write_str(" ")?;
                            }
                            first = false;
                            color.to_css(dest)?
                        }
                        Ok(())
                    }
                }
            }
        }

        #[inline] pub fn get_initial_value() -> computed_value::T {
            computed_value::T(None)
        }

        #[inline] pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::None
        }

        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, context: &Context) -> computed_value::T {
                match *self {
                    SpecifiedValue::Colors(ref vec) => {
                        computed_value::T(Some(vec.iter()
                                                  .map(|c| c.to_computed_value(context))
                                                  .collect()))
                    },
                    SpecifiedValue::None => {
                        computed_value::T(None)
                    }
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                    computed_value::T(Some(ref vec)) => {
                        SpecifiedValue::Colors(vec.iter()
                                                  .map(ToComputedValue::from_computed_value)
                                                  .collect())
                    },
                    computed_value::T(None) => {
                        SpecifiedValue::None
                    }
                }
            }
        }

        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            if input.try(|input| input.expect_ident_matching("none")).is_ok() {
                return Ok(SpecifiedValue::None)
            }

            let mut result = Vec::new();
            while let Ok(value) = input.try(|i| RGBAColor::parse(context, i)) {
                result.push(value);
            }

            if !result.is_empty() {
                Ok(SpecifiedValue::Colors(result))
            } else {
                Err(StyleParseError::UnspecifiedError.into())
            }
        }
    
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MozBorderRightColors(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MozBorderRightColors);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::MozBorderRightColors);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set__moz_border_right_colors(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset__moz_border_right_colors();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit__moz_border_right_colors();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MozBorderRightColors)
            }
    }



    
    

    
        
    
    /// https://drafts.csswg.org/css-break/#propdef-box-decoration-break
    pub mod box_decoration_break {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "slice" => slice,
                    "clone" => clone,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::slice
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::slice
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::slice => computed_value::T::slice,
                        SpecifiedValue::clone => computed_value::T::clone,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::slice => SpecifiedValue::slice,
                        computed_value::T::clone => SpecifiedValue::clone,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::BoxDecorationBreak(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::BoxDecorationBreak);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::BoxDecorationBreak);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_box_decoration_break(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_box_decoration_break();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_box_decoration_break();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::BoxDecorationBreak)
            }
    }





    
    

    
        
    
    /// Nonstandard (https://developer.mozilla.org/en-US/docs/Web/CSS/-moz-float-edge)
    pub mod _moz_float_edge {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "content-box" => content_box,
                    "margin-box" => margin_box,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::content_box
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::content_box
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::content_box => computed_value::T::content_box,
                        SpecifiedValue::margin_box => computed_value::T::margin_box,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::content_box => SpecifiedValue::content_box,
                        computed_value::T::margin_box => SpecifiedValue::margin_box,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MozFloatEdge(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MozFloatEdge);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::MozFloatEdge);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set__moz_float_edge(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset__moz_float_edge();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit__moz_float_edge();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MozFloatEdge)
            }
    }





    
        
    
    /// https://drafts.csswg.org/css-backgrounds/#the-background-image
    pub mod border_image_source {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::ImageLayer as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::ImageLayer as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { Either::First(None_) }
        #[inline] pub fn get_initial_specified_value() -> SpecifiedValue { Either::First(None_) }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::ImageLayer::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::BorderImageSource(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::BorderImageSource);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::BorderImageSource);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = (**specified_value).to_computed_value(context);
                                context.builder.set_border_image_source(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_border_image_source();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_border_image_source();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<Box<SpecifiedValue>, ParseError<'i>> {
                    parse(context, input).map(|result| Box::new(result))
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::BorderImageSource)
            }
    }




    
        
    
    /// https://drafts.csswg.org/css-backgrounds/#border-image-outset
    pub mod border_image_outset {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::LengthOrNumberRect as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::LengthOrNumberRect as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::LengthOrNumberRect::all(computed::LengthOrNumber::zero()) }
        #[inline] pub fn get_initial_specified_value() -> SpecifiedValue { specified::LengthOrNumberRect::all(specified::LengthOrNumber::zero()) }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::LengthOrNumberRect::parse_non_negative(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::BorderImageOutset(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::BorderImageOutset);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::BorderImageOutset);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = (**specified_value).to_computed_value(context);
                                context.builder.set_border_image_outset(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_border_image_outset();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_border_image_outset();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<Box<SpecifiedValue>, ParseError<'i>> {
                    parse(context, input).map(|result| Box::new(result))
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::BorderImageOutset)
            }
    }




    
    /// https://drafts.csswg.org/css-backgrounds/#border-image-repeat
    pub mod border_image_repeat {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
    use style_traits::ToCss;


    pub mod computed_value {
        pub use super::RepeatKeyword;

        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        #[derive(Clone, Debug, PartialEq, ToCss)]
        pub struct T(pub RepeatKeyword, pub RepeatKeyword);
    }

    #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
    #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
    #[derive(Clone, Debug, PartialEq, ToCss)]
    pub struct SpecifiedValue(pub RepeatKeyword,
                              pub Option<RepeatKeyword>);

    define_css_keyword_enum!(RepeatKeyword:
                             "stretch" => Stretch,
                             "repeat" => Repeat,
                             "round" => Round,
                             "space" => Space);

    #[inline]
    pub fn get_initial_value() -> computed_value::T {
        computed_value::T(RepeatKeyword::Stretch, RepeatKeyword::Stretch)
    }

    #[inline]
    pub fn get_initial_specified_value() -> SpecifiedValue {
        SpecifiedValue(RepeatKeyword::Stretch, None)
    }

    impl ToComputedValue for SpecifiedValue {
        type ComputedValue = computed_value::T;

        #[inline]
        fn to_computed_value(&self, _context: &Context) -> computed_value::T {
            computed_value::T(self.0, self.1.unwrap_or(self.0))
        }
        #[inline]
        fn from_computed_value(computed: &computed_value::T) -> Self {
            SpecifiedValue(computed.0, Some(computed.1))
        }
    }

    pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                         -> Result<SpecifiedValue, ParseError<'i>> {
        let first = RepeatKeyword::parse(input)?;
        let second = input.try(RepeatKeyword::parse).ok();

        Ok(SpecifiedValue(first, second))
    }

        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::BorderImageRepeat(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::BorderImageRepeat);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::BorderImageRepeat);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_border_image_repeat(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_border_image_repeat();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_border_image_repeat();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::BorderImageRepeat)
            }
    }



    
        
    
    /// https://drafts.csswg.org/css-backgrounds/#border-image-width
    pub mod border_image_width {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::BorderImageWidth as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::BorderImageWidth as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::BorderImageWidth::all(computed::BorderImageSideWidth::one()) }
        #[inline] pub fn get_initial_specified_value() -> SpecifiedValue { specified::BorderImageWidth::all(specified::BorderImageSideWidth::one()) }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::BorderImageWidth::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::BorderImageWidth(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::BorderImageWidth);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::BorderImageWidth);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = (**specified_value).to_computed_value(context);
                                context.builder.set_border_image_width(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_border_image_width();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_border_image_width();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<Box<SpecifiedValue>, ParseError<'i>> {
                    parse(context, input).map(|result| Box::new(result))
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::BorderImageWidth)
            }
    }




    
        
    
    /// https://drafts.csswg.org/css-backgrounds/#border-image-slice
    pub mod border_image_slice {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::BorderImageSlice as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::BorderImageSlice as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::NumberOrPercentage::Percentage(computed::Percentage(1.)).into() }
        #[inline] pub fn get_initial_specified_value() -> SpecifiedValue { specified::NumberOrPercentage::Percentage(specified::Percentage::new(1.)).into() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::BorderImageSlice::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::BorderImageSlice(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::BorderImageSlice);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::BorderImageSlice);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = (**specified_value).to_computed_value(context);
                                context.builder.set_border_image_slice(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_border_image_slice();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_border_image_slice();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<Box<SpecifiedValue>, ParseError<'i>> {
                    parse(context, input).map(|result| Box::new(result))
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::BorderImageSlice)
            }
    }



#[cfg(feature = "gecko")]
impl ::values::computed::BorderImageWidth {
    pub fn to_gecko_rect(&self, sides: &mut ::gecko_bindings::structs::nsStyleSides) {
        use gecko_bindings::sugar::ns_style_coord::{CoordDataMut, CoordDataValue};
        use gecko::values::GeckoStyleCoordConvertible;
        use values::generics::border::BorderImageSideWidth;

        match self.0 {
            BorderImageSideWidth::Auto => {
                sides.data_at_mut(0).set_value(CoordDataValue::Auto)
            },
            BorderImageSideWidth::Length(l) => {
                l.to_gecko_style_coord(&mut sides.data_at_mut(0))
            },
            BorderImageSideWidth::Number(n) => {
                sides.data_at_mut(0).set_value(CoordDataValue::Factor(n))
            },
        }
        match self.1 {
            BorderImageSideWidth::Auto => {
                sides.data_at_mut(1).set_value(CoordDataValue::Auto)
            },
            BorderImageSideWidth::Length(l) => {
                l.to_gecko_style_coord(&mut sides.data_at_mut(1))
            },
            BorderImageSideWidth::Number(n) => {
                sides.data_at_mut(1).set_value(CoordDataValue::Factor(n))
            },
        }
        match self.2 {
            BorderImageSideWidth::Auto => {
                sides.data_at_mut(2).set_value(CoordDataValue::Auto)
            },
            BorderImageSideWidth::Length(l) => {
                l.to_gecko_style_coord(&mut sides.data_at_mut(2))
            },
            BorderImageSideWidth::Number(n) => {
                sides.data_at_mut(2).set_value(CoordDataValue::Factor(n))
            },
        }
        match self.3 {
            BorderImageSideWidth::Auto => {
                sides.data_at_mut(3).set_value(CoordDataValue::Auto)
            },
            BorderImageSideWidth::Length(l) => {
                l.to_gecko_style_coord(&mut sides.data_at_mut(3))
            },
            BorderImageSideWidth::Number(n) => {
                sides.data_at_mut(3).set_value(CoordDataValue::Factor(n))
            },
        }
    }

    pub fn from_gecko_rect(sides: &::gecko_bindings::structs::nsStyleSides)
                           -> Option<::values::computed::BorderImageWidth> {
        use gecko_bindings::structs::nsStyleUnit::{eStyleUnit_Factor, eStyleUnit_Auto};
        use gecko_bindings::sugar::ns_style_coord::CoordData;
        use gecko::values::GeckoStyleCoordConvertible;
        use values::computed::{LengthOrPercentage, Number};
        use values::generics::border::BorderImageSideWidth;

        Some(
            ::values::computed::BorderImageWidth::new(
                match sides.data_at(0).unit() {
                    eStyleUnit_Auto => {
                        BorderImageSideWidth::Auto
                    },
                    eStyleUnit_Factor => {
                        BorderImageSideWidth::Number(
                            Number::from_gecko_style_coord(&sides.data_at(0))
                                .expect("sides[0] could not convert to Number"))
                    },
                    _ => {
                        BorderImageSideWidth::Length(
                            LengthOrPercentage::from_gecko_style_coord(&sides.data_at(0))
                                .expect("sides[0] could not convert to LengthOrPercentager"))
                    },
                },
                match sides.data_at(1).unit() {
                    eStyleUnit_Auto => {
                        BorderImageSideWidth::Auto
                    },
                    eStyleUnit_Factor => {
                        BorderImageSideWidth::Number(
                            Number::from_gecko_style_coord(&sides.data_at(1))
                                .expect("sides[1] could not convert to Number"))
                    },
                    _ => {
                        BorderImageSideWidth::Length(
                            LengthOrPercentage::from_gecko_style_coord(&sides.data_at(1))
                                .expect("sides[1] could not convert to LengthOrPercentager"))
                    },
                },
                match sides.data_at(2).unit() {
                    eStyleUnit_Auto => {
                        BorderImageSideWidth::Auto
                    },
                    eStyleUnit_Factor => {
                        BorderImageSideWidth::Number(
                            Number::from_gecko_style_coord(&sides.data_at(2))
                                .expect("sides[2] could not convert to Number"))
                    },
                    _ => {
                        BorderImageSideWidth::Length(
                            LengthOrPercentage::from_gecko_style_coord(&sides.data_at(2))
                                .expect("sides[2] could not convert to LengthOrPercentager"))
                    },
                },
                match sides.data_at(3).unit() {
                    eStyleUnit_Auto => {
                        BorderImageSideWidth::Auto
                    },
                    eStyleUnit_Factor => {
                        BorderImageSideWidth::Number(
                            Number::from_gecko_style_coord(&sides.data_at(3))
                                .expect("sides[3] could not convert to Number"))
                    },
                    _ => {
                        BorderImageSideWidth::Length(
                            LengthOrPercentage::from_gecko_style_coord(&sides.data_at(3))
                                .expect("sides[3] could not convert to LengthOrPercentager"))
                    },
                },
            )
        )
    }
}

    /* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */






// TODO(SimonSapin): don't parse `inline-table`, since we don't support it
//
// We allow "display" to apply to placeholders because we need to make the
// placeholder pseudo-element an inline-block in the UA stylesheet in Gecko.

    
    /// https://drafts.csswg.org/css-display/#propdef-display
    pub mod display {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
    
    use style_traits::ToCss;

    pub mod computed_value {
        pub use super::SpecifiedValue as T;

        impl T {
            /// Returns whether this "display" value is the display of a flex or
            /// grid container.
            ///
            /// This is used to implement various style fixups.
            pub fn is_item_container(&self) -> bool {
                matches!(*self,
                         T::flex
                         | T::inline_flex
                         | T::grid
                         | T::inline_grid
                )
            }

            /// Returns whether an element with this display type is a line
            /// participant, which means it may lay its children on the same
            /// line as itself.
            pub fn is_line_participant(&self) -> bool {
                matches!(*self,
                         T::inline
                         | T::contents
                         | T::ruby
                         | T::ruby_base_container
                )
            }

            /// Returns whether this "display" value is one of the types for
            /// ruby.
            #[cfg(feature = "gecko")]
            pub fn is_ruby_type(&self) -> bool {
                matches!(*self, T::ruby | T::ruby_base | T::ruby_text |
                         T::ruby_base_container | T::ruby_text_container)
            }

            /// Returns whether this "display" value is a ruby level container.
            #[cfg(feature = "gecko")]
            pub fn is_ruby_level_container(&self) -> bool {
                matches!(*self, T::ruby_base_container | T::ruby_text_container)
            }

            /// Convert this display into an equivalent block display.
            ///
            /// Also used for style adjustments.
            pub fn equivalent_block_display(&self, _is_root_element: bool) -> Self {
                match *self {
                    // Values that have a corresponding block-outside version.
                    T::inline_table => T::table,
                    T::inline_flex => T::flex,

                    T::inline_grid => T::grid,
                    T::_webkit_inline_box => T::_webkit_box,

                    // Special handling for contents and list-item on the root
                    // element for Gecko.
                    T::contents | T::list_item if _is_root_element => T::block,

                    // These are not changed by blockification.
                    T::none | T::block | T::flex | T::list_item | T::table => *self,
                    T::contents | T::flow_root | T::grid | T::_webkit_box => *self,

                    // Everything else becomes block.
                    _ => T::block,
                }

            }

            /// Convert this display into an inline-outside display.
            ///
            /// Ideally it should implement spec: https://drafts.csswg.org/css-display/#inlinify
            /// but the spec isn't stable enough, so we copy what Gecko does for now.
            #[cfg(feature = "gecko")]
            pub fn inlinify(&self) -> Self {
                match *self {
                    T::block | T::flow_root => T::inline_block,
                    T::table => T::inline_table,
                    T::flex => T::inline_flex,
                    T::grid => T::inline_grid,
                    T::_moz_box => T::_moz_inline_box,
                    T::_moz_stack => T::_moz_inline_stack,
                    T::_webkit_box => T::_webkit_inline_box,
                    other => other,
                }
            }
        }
    }

    #[allow(non_camel_case_types)]
    #[derive(Clone, Copy, Debug, Eq, Hash, PartialEq, ToComputedValue)]
    #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
    #[cfg_attr(feature = "servo", derive(HeapSizeOf, Deserialize, Serialize))]
    pub enum SpecifiedValue {
            inline,
            block,
            inline_block,
            table,
            inline_table,
            table_row_group,
            table_header_group,
            table_footer_group,
            table_row,
            table_column_group,
            table_column,
            table_cell,
            table_caption,
            list_item,
            none,
            flex,
            inline_flex,
            grid,
            inline_grid,
            ruby,
            ruby_base,
            ruby_base_container,
            ruby_text,
            ruby_text_container,
            contents,
            flow_root,
            _webkit_box,
            _webkit_inline_box,
            _moz_box,
            _moz_inline_box,
            _moz_grid,
            _moz_inline_grid,
            _moz_grid_group,
            _moz_grid_line,
            _moz_stack,
            _moz_inline_stack,
            _moz_deck,
            _moz_popup,
            _moz_groupbox,
    }

    impl ToCss for SpecifiedValue {
        fn to_css<W>(&self, dest: &mut W) -> ::std::fmt::Result
            where W: ::std::fmt::Write,
        {
            match *self {
                    SpecifiedValue::inline => dest.write_str("inline"),
                    SpecifiedValue::block => dest.write_str("block"),
                    SpecifiedValue::inline_block => dest.write_str("inline-block"),
                    SpecifiedValue::table => dest.write_str("table"),
                    SpecifiedValue::inline_table => dest.write_str("inline-table"),
                    SpecifiedValue::table_row_group => dest.write_str("table-row-group"),
                    SpecifiedValue::table_header_group => dest.write_str("table-header-group"),
                    SpecifiedValue::table_footer_group => dest.write_str("table-footer-group"),
                    SpecifiedValue::table_row => dest.write_str("table-row"),
                    SpecifiedValue::table_column_group => dest.write_str("table-column-group"),
                    SpecifiedValue::table_column => dest.write_str("table-column"),
                    SpecifiedValue::table_cell => dest.write_str("table-cell"),
                    SpecifiedValue::table_caption => dest.write_str("table-caption"),
                    SpecifiedValue::list_item => dest.write_str("list-item"),
                    SpecifiedValue::none => dest.write_str("none"),
                    SpecifiedValue::flex => dest.write_str("flex"),
                    SpecifiedValue::inline_flex => dest.write_str("inline-flex"),
                    SpecifiedValue::grid => dest.write_str("grid"),
                    SpecifiedValue::inline_grid => dest.write_str("inline-grid"),
                    SpecifiedValue::ruby => dest.write_str("ruby"),
                    SpecifiedValue::ruby_base => dest.write_str("ruby-base"),
                    SpecifiedValue::ruby_base_container => dest.write_str("ruby-base-container"),
                    SpecifiedValue::ruby_text => dest.write_str("ruby-text"),
                    SpecifiedValue::ruby_text_container => dest.write_str("ruby-text-container"),
                    SpecifiedValue::contents => dest.write_str("contents"),
                    SpecifiedValue::flow_root => dest.write_str("flow-root"),
                    SpecifiedValue::_webkit_box => dest.write_str("-webkit-box"),
                    SpecifiedValue::_webkit_inline_box => dest.write_str("-webkit-inline-box"),
                    SpecifiedValue::_moz_box => dest.write_str("-moz-box"),
                    SpecifiedValue::_moz_inline_box => dest.write_str("-moz-inline-box"),
                    SpecifiedValue::_moz_grid => dest.write_str("-moz-grid"),
                    SpecifiedValue::_moz_inline_grid => dest.write_str("-moz-inline-grid"),
                    SpecifiedValue::_moz_grid_group => dest.write_str("-moz-grid-group"),
                    SpecifiedValue::_moz_grid_line => dest.write_str("-moz-grid-line"),
                    SpecifiedValue::_moz_stack => dest.write_str("-moz-stack"),
                    SpecifiedValue::_moz_inline_stack => dest.write_str("-moz-inline-stack"),
                    SpecifiedValue::_moz_deck => dest.write_str("-moz-deck"),
                    SpecifiedValue::_moz_popup => dest.write_str("-moz-popup"),
                    SpecifiedValue::_moz_groupbox => dest.write_str("-moz-groupbox"),
            }
        }
    }

    /// The initial display value.
    #[inline]
    pub fn get_initial_value() -> computed_value::T {
        computed_value::T::inline
    }

    /// Parse a display value.
    pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                         -> Result<SpecifiedValue, ParseError<'i>> {
        try_match_ident_ignore_ascii_case! { input.expect_ident()?,
                "inline" => {
                    Ok(computed_value::T::inline)
                },
                "block" => {
                    Ok(computed_value::T::block)
                },
                "inline-block" => {
                    Ok(computed_value::T::inline_block)
                },
                "table" => {
                    Ok(computed_value::T::table)
                },
                "inline-table" => {
                    Ok(computed_value::T::inline_table)
                },
                "table-row-group" => {
                    Ok(computed_value::T::table_row_group)
                },
                "table-header-group" => {
                    Ok(computed_value::T::table_header_group)
                },
                "table-footer-group" => {
                    Ok(computed_value::T::table_footer_group)
                },
                "table-row" => {
                    Ok(computed_value::T::table_row)
                },
                "table-column-group" => {
                    Ok(computed_value::T::table_column_group)
                },
                "table-column" => {
                    Ok(computed_value::T::table_column)
                },
                "table-cell" => {
                    Ok(computed_value::T::table_cell)
                },
                "table-caption" => {
                    Ok(computed_value::T::table_caption)
                },
                "list-item" => {
                    Ok(computed_value::T::list_item)
                },
                "none" => {
                    Ok(computed_value::T::none)
                },
                "flex" => {
                    Ok(computed_value::T::flex)
                },
                "inline-flex" => {
                    Ok(computed_value::T::inline_flex)
                },
                "grid" => {
                    Ok(computed_value::T::grid)
                },
                "inline-grid" => {
                    Ok(computed_value::T::inline_grid)
                },
                "ruby" => {
                    Ok(computed_value::T::ruby)
                },
                "ruby-base" => {
                    Ok(computed_value::T::ruby_base)
                },
                "ruby-base-container" => {
                    Ok(computed_value::T::ruby_base_container)
                },
                "ruby-text" => {
                    Ok(computed_value::T::ruby_text)
                },
                "ruby-text-container" => {
                    Ok(computed_value::T::ruby_text_container)
                },
                "contents" => {
                    Ok(computed_value::T::contents)
                },
                "flow-root" => {
                    Ok(computed_value::T::flow_root)
                },
                "-webkit-box" => {
                    Ok(computed_value::T::_webkit_box)
                },
                "-webkit-inline-box" => {
                    Ok(computed_value::T::_webkit_inline_box)
                },
                "-moz-box" => {
                    Ok(computed_value::T::_moz_box)
                },
                "-moz-inline-box" => {
                    Ok(computed_value::T::_moz_inline_box)
                },
                "-moz-grid" => {
                    Ok(computed_value::T::_moz_grid)
                },
                "-moz-inline-grid" => {
                    Ok(computed_value::T::_moz_inline_grid)
                },
                "-moz-grid-group" => {
                    Ok(computed_value::T::_moz_grid_group)
                },
                "-moz-grid-line" => {
                    Ok(computed_value::T::_moz_grid_line)
                },
                "-moz-stack" => {
                    Ok(computed_value::T::_moz_stack)
                },
                "-moz-inline-stack" => {
                    Ok(computed_value::T::_moz_inline_stack)
                },
                "-moz-deck" => {
                    Ok(computed_value::T::_moz_deck)
                },
                "-moz-popup" => {
                    Ok(computed_value::T::_moz_popup)
                },
                "-moz-groupbox" => {
                    Ok(computed_value::T::_moz_groupbox)
                },
                "-webkit-flex" => {
                    Ok(computed_value::T::flex)
                },
                "-webkit-inline-flex" => {
                    Ok(computed_value::T::inline_flex)
                },
        }
    }


    
    
    #[cfg(feature = "gecko")]
    impl SpecifiedValue {
        /// Obtain a specified value from a Gecko keyword value
        ///
        /// Intended for use with presentation attributes, not style structs
        pub fn from_gecko_keyword(kw: u32) -> Self {
            use gecko_bindings::structs;
                // We can't match on enum values if we're matching on a u32
                const INLINE: u32
                    = structs::StyleDisplay::Inline as u32;
                // We can't match on enum values if we're matching on a u32
                const BLOCK: u32
                    = structs::StyleDisplay::Block as u32;
                // We can't match on enum values if we're matching on a u32
                const INLINE_BLOCK: u32
                    = structs::StyleDisplay::InlineBlock as u32;
                // We can't match on enum values if we're matching on a u32
                const TABLE: u32
                    = structs::StyleDisplay::Table as u32;
                // We can't match on enum values if we're matching on a u32
                const INLINE_TABLE: u32
                    = structs::StyleDisplay::InlineTable as u32;
                // We can't match on enum values if we're matching on a u32
                const TABLE_ROW_GROUP: u32
                    = structs::StyleDisplay::TableRowGroup as u32;
                // We can't match on enum values if we're matching on a u32
                const TABLE_HEADER_GROUP: u32
                    = structs::StyleDisplay::TableHeaderGroup as u32;
                // We can't match on enum values if we're matching on a u32
                const TABLE_FOOTER_GROUP: u32
                    = structs::StyleDisplay::TableFooterGroup as u32;
                // We can't match on enum values if we're matching on a u32
                const TABLE_ROW: u32
                    = structs::StyleDisplay::TableRow as u32;
                // We can't match on enum values if we're matching on a u32
                const TABLE_COLUMN_GROUP: u32
                    = structs::StyleDisplay::TableColumnGroup as u32;
                // We can't match on enum values if we're matching on a u32
                const TABLE_COLUMN: u32
                    = structs::StyleDisplay::TableColumn as u32;
                // We can't match on enum values if we're matching on a u32
                const TABLE_CELL: u32
                    = structs::StyleDisplay::TableCell as u32;
                // We can't match on enum values if we're matching on a u32
                const TABLE_CAPTION: u32
                    = structs::StyleDisplay::TableCaption as u32;
                // We can't match on enum values if we're matching on a u32
                const LIST_ITEM: u32
                    = structs::StyleDisplay::ListItem as u32;
                // We can't match on enum values if we're matching on a u32
                const NONE: u32
                    = structs::StyleDisplay::None as u32;
                // We can't match on enum values if we're matching on a u32
                const FLEX: u32
                    = structs::StyleDisplay::Flex as u32;
                // We can't match on enum values if we're matching on a u32
                const INLINE_FLEX: u32
                    = structs::StyleDisplay::InlineFlex as u32;
                // We can't match on enum values if we're matching on a u32
                const GRID: u32
                    = structs::StyleDisplay::Grid as u32;
                // We can't match on enum values if we're matching on a u32
                const INLINE_GRID: u32
                    = structs::StyleDisplay::InlineGrid as u32;
                // We can't match on enum values if we're matching on a u32
                const RUBY: u32
                    = structs::StyleDisplay::Ruby as u32;
                // We can't match on enum values if we're matching on a u32
                const RUBY_BASE: u32
                    = structs::StyleDisplay::RubyBase as u32;
                // We can't match on enum values if we're matching on a u32
                const RUBY_BASE_CONTAINER: u32
                    = structs::StyleDisplay::RubyBaseContainer as u32;
                // We can't match on enum values if we're matching on a u32
                const RUBY_TEXT: u32
                    = structs::StyleDisplay::RubyText as u32;
                // We can't match on enum values if we're matching on a u32
                const RUBY_TEXT_CONTAINER: u32
                    = structs::StyleDisplay::RubyTextContainer as u32;
                // We can't match on enum values if we're matching on a u32
                const CONTENTS: u32
                    = structs::StyleDisplay::Contents as u32;
                // We can't match on enum values if we're matching on a u32
                const FLOW_ROOT: u32
                    = structs::StyleDisplay::FlowRoot as u32;
                // We can't match on enum values if we're matching on a u32
                const _WEBKIT_BOX: u32
                    = structs::StyleDisplay::WebkitBox as u32;
                // We can't match on enum values if we're matching on a u32
                const _WEBKIT_INLINE_BOX: u32
                    = structs::StyleDisplay::WebkitInlineBox as u32;
                // We can't match on enum values if we're matching on a u32
                const _MOZ_BOX: u32
                    = structs::StyleDisplay::MozBox as u32;
                // We can't match on enum values if we're matching on a u32
                const _MOZ_INLINE_BOX: u32
                    = structs::StyleDisplay::MozInlineBox as u32;
                // We can't match on enum values if we're matching on a u32
                const _MOZ_GRID: u32
                    = structs::StyleDisplay::MozGrid as u32;
                // We can't match on enum values if we're matching on a u32
                const _MOZ_INLINE_GRID: u32
                    = structs::StyleDisplay::MozInlineGrid as u32;
                // We can't match on enum values if we're matching on a u32
                const _MOZ_GRID_GROUP: u32
                    = structs::StyleDisplay::MozGridGroup as u32;
                // We can't match on enum values if we're matching on a u32
                const _MOZ_GRID_LINE: u32
                    = structs::StyleDisplay::MozGridLine as u32;
                // We can't match on enum values if we're matching on a u32
                const _MOZ_STACK: u32
                    = structs::StyleDisplay::MozStack as u32;
                // We can't match on enum values if we're matching on a u32
                const _MOZ_INLINE_STACK: u32
                    = structs::StyleDisplay::MozInlineStack as u32;
                // We can't match on enum values if we're matching on a u32
                const _MOZ_DECK: u32
                    = structs::StyleDisplay::MozDeck as u32;
                // We can't match on enum values if we're matching on a u32
                const _MOZ_POPUP: u32
                    = structs::StyleDisplay::MozPopup as u32;
                // We can't match on enum values if we're matching on a u32
                const _MOZ_GROUPBOX: u32
                    = structs::StyleDisplay::MozGroupbox as u32;
            match kw  {
                    INLINE => SpecifiedValue::inline,
                    BLOCK => SpecifiedValue::block,
                    INLINE_BLOCK => SpecifiedValue::inline_block,
                    TABLE => SpecifiedValue::table,
                    INLINE_TABLE => SpecifiedValue::inline_table,
                    TABLE_ROW_GROUP => SpecifiedValue::table_row_group,
                    TABLE_HEADER_GROUP => SpecifiedValue::table_header_group,
                    TABLE_FOOTER_GROUP => SpecifiedValue::table_footer_group,
                    TABLE_ROW => SpecifiedValue::table_row,
                    TABLE_COLUMN_GROUP => SpecifiedValue::table_column_group,
                    TABLE_COLUMN => SpecifiedValue::table_column,
                    TABLE_CELL => SpecifiedValue::table_cell,
                    TABLE_CAPTION => SpecifiedValue::table_caption,
                    LIST_ITEM => SpecifiedValue::list_item,
                    NONE => SpecifiedValue::none,
                    FLEX => SpecifiedValue::flex,
                    INLINE_FLEX => SpecifiedValue::inline_flex,
                    GRID => SpecifiedValue::grid,
                    INLINE_GRID => SpecifiedValue::inline_grid,
                    RUBY => SpecifiedValue::ruby,
                    RUBY_BASE => SpecifiedValue::ruby_base,
                    RUBY_BASE_CONTAINER => SpecifiedValue::ruby_base_container,
                    RUBY_TEXT => SpecifiedValue::ruby_text,
                    RUBY_TEXT_CONTAINER => SpecifiedValue::ruby_text_container,
                    CONTENTS => SpecifiedValue::contents,
                    FLOW_ROOT => SpecifiedValue::flow_root,
                    _WEBKIT_BOX => SpecifiedValue::_webkit_box,
                    _WEBKIT_INLINE_BOX => SpecifiedValue::_webkit_inline_box,
                    _MOZ_BOX => SpecifiedValue::_moz_box,
                    _MOZ_INLINE_BOX => SpecifiedValue::_moz_inline_box,
                    _MOZ_GRID => SpecifiedValue::_moz_grid,
                    _MOZ_INLINE_GRID => SpecifiedValue::_moz_inline_grid,
                    _MOZ_GRID_GROUP => SpecifiedValue::_moz_grid_group,
                    _MOZ_GRID_LINE => SpecifiedValue::_moz_grid_line,
                    _MOZ_STACK => SpecifiedValue::_moz_stack,
                    _MOZ_INLINE_STACK => SpecifiedValue::_moz_inline_stack,
                    _MOZ_DECK => SpecifiedValue::_moz_deck,
                    _MOZ_POPUP => SpecifiedValue::_moz_popup,
                    _MOZ_GROUPBOX => SpecifiedValue::_moz_groupbox,
                x => panic!("Found unexpected value in style struct for display property: {:?}", x),
            }
        }
    }



        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::Display(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::Display);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::Display);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_display(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_display();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_display();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::Display)
            }
    }



    
    

    
        
    
    /// Internal (not web-exposed)
    pub mod _moz_top_layer {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "none" => none,
                    "top" => top,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::none
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::none
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::none => computed_value::T::none,
                        SpecifiedValue::top => computed_value::T::top,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::none => SpecifiedValue::none,
                        computed_value::T::top => SpecifiedValue::top,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MozTopLayer(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MozTopLayer);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::MozTopLayer);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set__moz_top_layer(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset__moz_top_layer();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit__moz_top_layer();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MozTopLayer)
            }
    }





    
    

    
        
    
    /// https://drafts.csswg.org/css-position/#position-property
    pub mod position {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "static" => static_,
                    "absolute" => absolute,
                    "relative" => relative,
                    "fixed" => fixed,
                    "sticky" => sticky,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::static_
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::static_
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::static_ => computed_value::T::static_,
                        SpecifiedValue::absolute => computed_value::T::absolute,
                        SpecifiedValue::relative => computed_value::T::relative,
                        SpecifiedValue::fixed => computed_value::T::fixed,
                        SpecifiedValue::sticky => computed_value::T::sticky,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::static_ => SpecifiedValue::static_,
                        computed_value::T::absolute => SpecifiedValue::absolute,
                        computed_value::T::relative => SpecifiedValue::relative,
                        computed_value::T::fixed => SpecifiedValue::fixed,
                        computed_value::T::sticky => SpecifiedValue::sticky,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::Position(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::Position);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::Position);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_position(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_position();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_position();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::Position)
            }
    }





    

    
        
    
    /// https://drafts.csswg.org/css-box/#propdef-float
    pub mod float {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            use style_traits::ToCss;
            define_css_keyword_enum! { SpecifiedValue:
                values {
                        "none" => none,
                        "left" => left,
                        "right" => right,
                        "inline-start" => inline_start,
                        "inline-end" => inline_end,
                }
                aliases {
                }
            }
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "none" => none,
                    "left" => left,
                    "right" => right,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::none
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::none
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

            
            
    
    #[cfg(feature = "gecko")]
    impl SpecifiedValue {
        /// Obtain a specified value from a Gecko keyword value
        ///
        /// Intended for use with presentation attributes, not style structs
        pub fn from_gecko_keyword(kw: u32) -> Self {
            use gecko_bindings::structs;
                // We can't match on enum values if we're matching on a u32
                const NONE: u32
                    = structs::StyleFloat::None as u32;
                // We can't match on enum values if we're matching on a u32
                const LEFT: u32
                    = structs::StyleFloat::Left as u32;
                // We can't match on enum values if we're matching on a u32
                const RIGHT: u32
                    = structs::StyleFloat::Right as u32;
                // We can't match on enum values if we're matching on a u32
                const INLINE_START: u32
                    = structs::StyleFloat::InlineStart as u32;
                // We can't match on enum values if we're matching on a u32
                const INLINE_END: u32
                    = structs::StyleFloat::InlineEnd as u32;
            match kw  {
                    NONE => SpecifiedValue::none,
                    LEFT => SpecifiedValue::left,
                    RIGHT => SpecifiedValue::right,
                    INLINE_START => SpecifiedValue::inline_start,
                    INLINE_END => SpecifiedValue::inline_end,
                x => panic!("Found unexpected value in style struct for float property: {:?}", x),
            }
        }
    }

    
            
    impl ToComputedValue for SpecifiedValue {
        type ComputedValue = computed_value::T;

        #[inline]
        fn to_computed_value(&self, context: &Context) -> computed_value::T {
            let ltr = context.style().writing_mode.is_bidi_ltr();
            // https://drafts.csswg.org/css-logical-props/#float-clear
            match *self {
                SpecifiedValue::inline_start => {
                    context.rule_cache_conditions.borrow_mut()
                        .set_writing_mode_dependency(context.builder.writing_mode);
                    if ltr {
                        computed_value::T::left
                    } else {
                        computed_value::T::right
                    }
                }
                SpecifiedValue::inline_end => {
                    context.rule_cache_conditions.borrow_mut()
                        .set_writing_mode_dependency(context.builder.writing_mode);
                    if ltr {
                        computed_value::T::right
                    } else {
                        computed_value::T::left
                    }
                }
                    SpecifiedValue::none => computed_value::T::none,
                    SpecifiedValue::left => computed_value::T::left,
                    SpecifiedValue::right => computed_value::T::right,
            }
        }
        #[inline]
        fn from_computed_value(computed: &computed_value::T) -> SpecifiedValue {
            match *computed {
                    computed_value::T::none => SpecifiedValue::none,
                    computed_value::T::left => SpecifiedValue::left,
                    computed_value::T::right => SpecifiedValue::right,
            }
        }
    }

        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::Float(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::Float);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::Float);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_float(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_float();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_float();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::Float)
            }
    }




    

    
        
    
    /// https://www.w3.org/TR/CSS2/visuren.html#flow-control
    pub mod clear {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            use style_traits::ToCss;
            define_css_keyword_enum! { SpecifiedValue:
                values {
                        "none" => none,
                        "left" => left,
                        "right" => right,
                        "both" => both,
                        "inline-start" => inline_start,
                        "inline-end" => inline_end,
                }
                aliases {
                }
            }
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "none" => none,
                    "left" => left,
                    "right" => right,
                    "both" => both,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::none
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::none
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

            
            
    
    #[cfg(feature = "gecko")]
    impl SpecifiedValue {
        /// Obtain a specified value from a Gecko keyword value
        ///
        /// Intended for use with presentation attributes, not style structs
        pub fn from_gecko_keyword(kw: u32) -> Self {
            use gecko_bindings::structs;
                // We can't match on enum values if we're matching on a u32
                const NONE: u32
                    = structs::StyleClear::None as u32;
                // We can't match on enum values if we're matching on a u32
                const LEFT: u32
                    = structs::StyleClear::Left as u32;
                // We can't match on enum values if we're matching on a u32
                const RIGHT: u32
                    = structs::StyleClear::Right as u32;
                // We can't match on enum values if we're matching on a u32
                const BOTH: u32
                    = structs::StyleClear::Both as u32;
                // We can't match on enum values if we're matching on a u32
                const INLINE_START: u32
                    = structs::StyleClear::InlineStart as u32;
                // We can't match on enum values if we're matching on a u32
                const INLINE_END: u32
                    = structs::StyleClear::InlineEnd as u32;
            match kw  {
                    NONE => SpecifiedValue::none,
                    LEFT => SpecifiedValue::left,
                    RIGHT => SpecifiedValue::right,
                    BOTH => SpecifiedValue::both,
                    INLINE_START => SpecifiedValue::inline_start,
                    INLINE_END => SpecifiedValue::inline_end,
                x => panic!("Found unexpected value in style struct for clear property: {:?}", x),
            }
        }
    }

    
            
    impl ToComputedValue for SpecifiedValue {
        type ComputedValue = computed_value::T;

        #[inline]
        fn to_computed_value(&self, context: &Context) -> computed_value::T {
            let ltr = context.style().writing_mode.is_bidi_ltr();
            // https://drafts.csswg.org/css-logical-props/#float-clear
            match *self {
                SpecifiedValue::inline_start => {
                    context.rule_cache_conditions.borrow_mut()
                        .set_writing_mode_dependency(context.builder.writing_mode);
                    if ltr {
                        computed_value::T::left
                    } else {
                        computed_value::T::right
                    }
                }
                SpecifiedValue::inline_end => {
                    context.rule_cache_conditions.borrow_mut()
                        .set_writing_mode_dependency(context.builder.writing_mode);
                    if ltr {
                        computed_value::T::right
                    } else {
                        computed_value::T::left
                    }
                }
                    SpecifiedValue::none => computed_value::T::none,
                    SpecifiedValue::left => computed_value::T::left,
                    SpecifiedValue::right => computed_value::T::right,
                    SpecifiedValue::both => computed_value::T::both,
            }
        }
        #[inline]
        fn from_computed_value(computed: &computed_value::T) -> SpecifiedValue {
            match *computed {
                    computed_value::T::none => SpecifiedValue::none,
                    computed_value::T::left => SpecifiedValue::left,
                    computed_value::T::right => SpecifiedValue::right,
                    computed_value::T::both => SpecifiedValue::both,
            }
        }
    }

        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::Clear(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::Clear);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::Clear);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_clear(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_clear();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_clear();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::Clear)
            }
    }




    



    
        
    
    /// https://www.w3.org/TR/CSS2/visudet.html#propdef-vertical-align
    pub mod vertical_align {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::VerticalAlign as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::VerticalAlign as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::VerticalAlign::baseline() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::VerticalAlign::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::VerticalAlign(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::VerticalAlign);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::VerticalAlign);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_vertical_align(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_vertical_align();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_vertical_align();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::VerticalAlign)
            }
    }



// CSS 2.1, Section 11 - Visual effects


    
    

    
        
    




    
    

    
        
    
    /// Internal, not web-exposed,           may be standardized in the future (https://developer.mozilla.org/en-US/docs/Web/CSS/overflow-clip-box)
    pub mod overflow_clip_box {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "padding-box" => padding_box,
                    "content-box" => content_box,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::padding_box
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::padding_box
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::padding_box => computed_value::T::padding_box,
                        SpecifiedValue::content_box => computed_value::T::content_box,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::padding_box => SpecifiedValue::padding_box,
                        computed_value::T::content_box => SpecifiedValue::content_box,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::OverflowClipBox(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::OverflowClipBox);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::OverflowClipBox);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_overflow_clip_box(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_overflow_clip_box();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_overflow_clip_box();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::OverflowClipBox)
            }
    }






// FIXME(pcwalton, #2742): Implement scrolling for `scroll` and `auto`.

    
    

    
        
    
    /// https://drafts.csswg.org/css-overflow/#propdef-overflow-x
    pub mod overflow_x {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "visible" => visible,
                    "hidden" => hidden,
                    "scroll" => scroll,
                    "auto" => auto,
                    "-moz-hidden-unscrollable" => _moz_hidden_unscrollable,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::visible
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::visible
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::visible => computed_value::T::visible,
                        SpecifiedValue::hidden => computed_value::T::hidden,
                        SpecifiedValue::scroll => computed_value::T::scroll,
                        SpecifiedValue::auto => computed_value::T::auto,
                        SpecifiedValue::_moz_hidden_unscrollable => computed_value::T::_moz_hidden_unscrollable,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::visible => SpecifiedValue::visible,
                        computed_value::T::hidden => SpecifiedValue::hidden,
                        computed_value::T::scroll => SpecifiedValue::scroll,
                        computed_value::T::auto => SpecifiedValue::auto,
                        computed_value::T::_moz_hidden_unscrollable => SpecifiedValue::_moz_hidden_unscrollable,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::OverflowX(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::OverflowX);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::OverflowX);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_overflow_x(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_overflow_x();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_overflow_x();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::OverflowX)
            }
    }




// FIXME(pcwalton, #2742): Implement scrolling for `scroll` and `auto`.

    
    /// https://drafts.csswg.org/css-overflow/#propdef-overflow-y
    pub mod overflow_y {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
    pub use super::overflow_x::{SpecifiedValue, parse, get_initial_value, computed_value};

        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::OverflowY(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::OverflowY);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::OverflowY);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_overflow_y(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_overflow_y();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_overflow_y();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::OverflowY)
            }
    }



    
        
    
    
    /// https://drafts.csswg.org/css-transitions/#propdef-transition-duration
    pub mod transition_duration {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        use std::fmt;
        use style_traits::{Separator, ToCss};

        pub mod single_value {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::ShorthandId;
            #[allow(unused_imports)]
            use selectors::parser::SelectorParseError;
            #[allow(unused_imports)]
            use style_traits::{ParseError, StyleParseError};
            #[allow(unused_imports)]
            use values::computed::{Context, ToComputedValue};
            #[allow(unused_imports)]
            use values::{computed, specified};
            #[allow(unused_imports)]
            use values::{Auto, Either, None_, Normal};
            
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::Time as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::Time as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::Time::zero() }
        #[inline] pub fn get_initial_specified_value() -> SpecifiedValue { specified::Time::zero() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::Time::parse_non_negative(context, input)
        }
    
        
        }

        /// The definition of the computed value for transition-duration.
        pub mod computed_value {
            pub use super::single_value::computed_value as single_value;
            pub use self::single_value::T as SingleComputedValue;
            use smallvec::{IntoIter, SmallVec};
            use values::computed::ComputedVecIter;

            /// The computed value, effectively a list of single values.
            #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
            #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
            #[derive(Clone, Debug, PartialEq)]
            pub struct T(
                pub SmallVec<[single_value::T; 1]>,
            );


            pub type Iter<'a, 'cx, 'cx_a> = ComputedVecIter<'a, 'cx, 'cx_a, super::single_value::SpecifiedValue>;

            impl IntoIterator for T {
                type Item = single_value::T;
                type IntoIter = IntoIter<[single_value::T; 1]>;
                fn into_iter(self) -> Self::IntoIter {
                    self.0.into_iter()
                }
            }
        }

        impl ToCss for computed_value::T {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        warn!("Found empty value for property transition-duration");
                }
                for i in iter {
                    dest.write_str(::style_traits::Comma::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        /// The specified value of transition-duration.
        #[derive(Clone, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        pub struct SpecifiedValue(pub Vec<single_value::SpecifiedValue>);

        impl ToCss for SpecifiedValue {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        warn!("Found empty value for property transition-duration");
                }
                for i in iter {
                    dest.write_str(::style_traits::Comma::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        pub fn get_initial_value() -> computed_value::T {
                let mut v = SmallVec::new();
                v.push(single_value::get_initial_value());
                computed_value::T(v)
        }

        pub fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                -> Result<SpecifiedValue, ParseError<'i>> {
            use style_traits::Separator;


            ::style_traits::Comma::parse(input, |parser| {
                single_value::parse(context, parser)
            }).map(SpecifiedValue)
        }

        pub use self::single_value::SpecifiedValue as SingleSpecifiedValue;

        impl SpecifiedValue {
            pub fn compute_iter<'a, 'cx, 'cx_a>(&'a self, context: &'cx Context<'cx_a>)
                -> computed_value::Iter<'a, 'cx, 'cx_a> {
                computed_value::Iter::new(context, &self.0)
            }
        }

        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, context: &Context) -> computed_value::T {
                computed_value::T(self.compute_iter(context).collect())
            }

            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                SpecifiedValue(computed.0.iter()
                                    .map(ToComputedValue::from_computed_value)
                                    .collect())
            }
        }
    
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::TransitionDuration(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::TransitionDuration);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::TransitionDuration);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            // In the case of a vector property we want to pass
                            // down an iterator so that this can be computed
                            // without allocation
                            //
                            // However, computing requires a context, but the
                            // style struct being mutated is on the context. We
                            // temporarily remove it, mutate it, and then put it
                            // back. Vector longhands cannot touch their own
                            // style struct whilst computing, else this will
                            // panic.
                            let mut s =
                                context.builder.take_box();
                            {
                                let iter = specified_value.compute_iter(context);
                                s.set_transition_duration(iter);
                            }
                            context.builder.put_box(s);
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_transition_duration();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_transition_duration();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::TransitionDuration)
            }
    }





    
        
    
    
    /// https://drafts.csswg.org/css-transitions/#propdef-transition-timing-function
    pub mod transition_timing_function {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        use std::fmt;
        use style_traits::{Separator, ToCss};

        pub mod single_value {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::ShorthandId;
            #[allow(unused_imports)]
            use selectors::parser::SelectorParseError;
            #[allow(unused_imports)]
            use style_traits::{ParseError, StyleParseError};
            #[allow(unused_imports)]
            use values::computed::{Context, ToComputedValue};
            #[allow(unused_imports)]
            use values::{computed, specified};
            #[allow(unused_imports)]
            use values::{Auto, Either, None_, Normal};
            
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::TimingFunction as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::TimingFunction as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::TimingFunction::ease() }
        #[inline] pub fn get_initial_specified_value() -> SpecifiedValue { specified::TimingFunction::ease() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::TimingFunction::parse(context, input)
        }
    
        
        }

        /// The definition of the computed value for transition-timing-function.
        pub mod computed_value {
            pub use super::single_value::computed_value as single_value;
            pub use self::single_value::T as SingleComputedValue;
            use smallvec::{IntoIter, SmallVec};
            use values::computed::ComputedVecIter;

            /// The computed value, effectively a list of single values.
            #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
            #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
            #[derive(Clone, Debug, PartialEq)]
            pub struct T(
                pub SmallVec<[single_value::T; 1]>,
            );


            pub type Iter<'a, 'cx, 'cx_a> = ComputedVecIter<'a, 'cx, 'cx_a, super::single_value::SpecifiedValue>;

            impl IntoIterator for T {
                type Item = single_value::T;
                type IntoIter = IntoIter<[single_value::T; 1]>;
                fn into_iter(self) -> Self::IntoIter {
                    self.0.into_iter()
                }
            }
        }

        impl ToCss for computed_value::T {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        warn!("Found empty value for property transition-timing-function");
                }
                for i in iter {
                    dest.write_str(::style_traits::Comma::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        /// The specified value of transition-timing-function.
        #[derive(Clone, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        pub struct SpecifiedValue(pub Vec<single_value::SpecifiedValue>);

        impl ToCss for SpecifiedValue {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        warn!("Found empty value for property transition-timing-function");
                }
                for i in iter {
                    dest.write_str(::style_traits::Comma::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        pub fn get_initial_value() -> computed_value::T {
                let mut v = SmallVec::new();
                v.push(single_value::get_initial_value());
                computed_value::T(v)
        }

        pub fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                -> Result<SpecifiedValue, ParseError<'i>> {
            use style_traits::Separator;


            ::style_traits::Comma::parse(input, |parser| {
                single_value::parse(context, parser)
            }).map(SpecifiedValue)
        }

        pub use self::single_value::SpecifiedValue as SingleSpecifiedValue;

        impl SpecifiedValue {
            pub fn compute_iter<'a, 'cx, 'cx_a>(&'a self, context: &'cx Context<'cx_a>)
                -> computed_value::Iter<'a, 'cx, 'cx_a> {
                computed_value::Iter::new(context, &self.0)
            }
        }

        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, context: &Context) -> computed_value::T {
                computed_value::T(self.compute_iter(context).collect())
            }

            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                SpecifiedValue(computed.0.iter()
                                    .map(ToComputedValue::from_computed_value)
                                    .collect())
            }
        }
    
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::TransitionTimingFunction(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::TransitionTimingFunction);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::TransitionTimingFunction);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            // In the case of a vector property we want to pass
                            // down an iterator so that this can be computed
                            // without allocation
                            //
                            // However, computing requires a context, but the
                            // style struct being mutated is on the context. We
                            // temporarily remove it, mutate it, and then put it
                            // back. Vector longhands cannot touch their own
                            // style struct whilst computing, else this will
                            // panic.
                            let mut s =
                                context.builder.take_box();
                            {
                                let iter = specified_value.compute_iter(context);
                                s.set_transition_timing_function(iter);
                            }
                            context.builder.put_box(s);
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_transition_timing_function();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_transition_timing_function();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::TransitionTimingFunction)
            }
    }





    
        
    
    
    /// https://drafts.csswg.org/css-transitions/#propdef-transition-property
    pub mod transition_property {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        use std::fmt;
        use style_traits::{Separator, ToCss};

        pub mod single_value {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::ShorthandId;
            #[allow(unused_imports)]
            use selectors::parser::SelectorParseError;
            #[allow(unused_imports)]
            use style_traits::{ParseError, StyleParseError};
            #[allow(unused_imports)]
            use values::computed::{Context, ToComputedValue};
            #[allow(unused_imports)]
            use values::{computed, specified};
            #[allow(unused_imports)]
            use values::{Auto, Either, None_, Normal};
            
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::TransitionProperty as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::TransitionProperty as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::TransitionProperty::All }
        #[inline] pub fn get_initial_specified_value() -> SpecifiedValue { specified::TransitionProperty::All }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::TransitionProperty::parse(input)
        }
    
        
        }

        /// The definition of the computed value for transition-property.
        pub mod computed_value {
            pub use super::single_value::computed_value as single_value;
            pub use self::single_value::T as SingleComputedValue;
            use smallvec::{IntoIter, SmallVec};
            use values::computed::ComputedVecIter;

            /// The computed value, effectively a list of single values.
            #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
            #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
            #[derive(Clone, Debug, PartialEq)]
            pub struct T(
                pub SmallVec<[single_value::T; 1]>,
            );


            pub type Iter<'a, 'cx, 'cx_a> = ComputedVecIter<'a, 'cx, 'cx_a, super::single_value::SpecifiedValue>;

            impl IntoIterator for T {
                type Item = single_value::T;
                type IntoIter = IntoIter<[single_value::T; 1]>;
                fn into_iter(self) -> Self::IntoIter {
                    self.0.into_iter()
                }
            }
        }

        impl ToCss for computed_value::T {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        dest.write_str("none")?;
                }
                for i in iter {
                    dest.write_str(::style_traits::Comma::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        /// The specified value of transition-property.
        #[derive(Clone, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        pub struct SpecifiedValue(pub Vec<single_value::SpecifiedValue>);

        impl ToCss for SpecifiedValue {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        dest.write_str("none")?;
                }
                for i in iter {
                    dest.write_str(::style_traits::Comma::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        pub fn get_initial_value() -> computed_value::T {
                let mut v = SmallVec::new();
                v.push(single_value::get_initial_value());
                computed_value::T(v)
        }

        pub fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                -> Result<SpecifiedValue, ParseError<'i>> {
            use style_traits::Separator;

                if input.try(|input| input.expect_ident_matching("none")).is_ok() {
                    return Ok(SpecifiedValue(Vec::new()))
                }

            ::style_traits::Comma::parse(input, |parser| {
                single_value::parse(context, parser)
            }).map(SpecifiedValue)
        }

        pub use self::single_value::SpecifiedValue as SingleSpecifiedValue;

        impl SpecifiedValue {
            pub fn compute_iter<'a, 'cx, 'cx_a>(&'a self, context: &'cx Context<'cx_a>)
                -> computed_value::Iter<'a, 'cx, 'cx_a> {
                computed_value::Iter::new(context, &self.0)
            }
        }

        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, context: &Context) -> computed_value::T {
                computed_value::T(self.compute_iter(context).collect())
            }

            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                SpecifiedValue(computed.0.iter()
                                    .map(ToComputedValue::from_computed_value)
                                    .collect())
            }
        }
    
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::TransitionProperty(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::TransitionProperty);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::TransitionProperty);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            // In the case of a vector property we want to pass
                            // down an iterator so that this can be computed
                            // without allocation
                            //
                            // However, computing requires a context, but the
                            // style struct being mutated is on the context. We
                            // temporarily remove it, mutate it, and then put it
                            // back. Vector longhands cannot touch their own
                            // style struct whilst computing, else this will
                            // panic.
                            let mut s =
                                context.builder.take_box();
                            {
                                let iter = specified_value.compute_iter(context);
                                s.set_transition_property(iter);
                            }
                            context.builder.put_box(s);
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_transition_property();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_transition_property();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::TransitionProperty)
            }
    }





    
        
    
    
    /// https://drafts.csswg.org/css-transitions/#propdef-transition-delay
    pub mod transition_delay {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        use std::fmt;
        use style_traits::{Separator, ToCss};

        pub mod single_value {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::ShorthandId;
            #[allow(unused_imports)]
            use selectors::parser::SelectorParseError;
            #[allow(unused_imports)]
            use style_traits::{ParseError, StyleParseError};
            #[allow(unused_imports)]
            use values::computed::{Context, ToComputedValue};
            #[allow(unused_imports)]
            use values::{computed, specified};
            #[allow(unused_imports)]
            use values::{Auto, Either, None_, Normal};
            
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::Time as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::Time as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::Time::zero() }
        #[inline] pub fn get_initial_specified_value() -> SpecifiedValue { specified::Time::zero() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::Time::parse(context, input)
        }
    
        
        }

        /// The definition of the computed value for transition-delay.
        pub mod computed_value {
            pub use super::single_value::computed_value as single_value;
            pub use self::single_value::T as SingleComputedValue;
            use smallvec::{IntoIter, SmallVec};
            use values::computed::ComputedVecIter;

            /// The computed value, effectively a list of single values.
            #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
            #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
            #[derive(Clone, Debug, PartialEq)]
            pub struct T(
                pub SmallVec<[single_value::T; 1]>,
            );


            pub type Iter<'a, 'cx, 'cx_a> = ComputedVecIter<'a, 'cx, 'cx_a, super::single_value::SpecifiedValue>;

            impl IntoIterator for T {
                type Item = single_value::T;
                type IntoIter = IntoIter<[single_value::T; 1]>;
                fn into_iter(self) -> Self::IntoIter {
                    self.0.into_iter()
                }
            }
        }

        impl ToCss for computed_value::T {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        warn!("Found empty value for property transition-delay");
                }
                for i in iter {
                    dest.write_str(::style_traits::Comma::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        /// The specified value of transition-delay.
        #[derive(Clone, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        pub struct SpecifiedValue(pub Vec<single_value::SpecifiedValue>);

        impl ToCss for SpecifiedValue {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        warn!("Found empty value for property transition-delay");
                }
                for i in iter {
                    dest.write_str(::style_traits::Comma::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        pub fn get_initial_value() -> computed_value::T {
                let mut v = SmallVec::new();
                v.push(single_value::get_initial_value());
                computed_value::T(v)
        }

        pub fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                -> Result<SpecifiedValue, ParseError<'i>> {
            use style_traits::Separator;


            ::style_traits::Comma::parse(input, |parser| {
                single_value::parse(context, parser)
            }).map(SpecifiedValue)
        }

        pub use self::single_value::SpecifiedValue as SingleSpecifiedValue;

        impl SpecifiedValue {
            pub fn compute_iter<'a, 'cx, 'cx_a>(&'a self, context: &'cx Context<'cx_a>)
                -> computed_value::Iter<'a, 'cx, 'cx_a> {
                computed_value::Iter::new(context, &self.0)
            }
        }

        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, context: &Context) -> computed_value::T {
                computed_value::T(self.compute_iter(context).collect())
            }

            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                SpecifiedValue(computed.0.iter()
                                    .map(ToComputedValue::from_computed_value)
                                    .collect())
            }
        }
    
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::TransitionDelay(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::TransitionDelay);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::TransitionDelay);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            // In the case of a vector property we want to pass
                            // down an iterator so that this can be computed
                            // without allocation
                            //
                            // However, computing requires a context, but the
                            // style struct being mutated is on the context. We
                            // temporarily remove it, mutate it, and then put it
                            // back. Vector longhands cannot touch their own
                            // style struct whilst computing, else this will
                            // panic.
                            let mut s =
                                context.builder.take_box();
                            {
                                let iter = specified_value.compute_iter(context);
                                s.set_transition_delay(iter);
                            }
                            context.builder.put_box(s);
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_transition_delay();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_transition_delay();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::TransitionDelay)
            }
    }





    
    
    /// https://drafts.csswg.org/css-animations/#propdef-animation-name
    pub mod animation_name {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        use std::fmt;
        use style_traits::{Separator, ToCss};

        pub mod single_value {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::ShorthandId;
            #[allow(unused_imports)]
            use selectors::parser::SelectorParseError;
            #[allow(unused_imports)]
            use style_traits::{ParseError, StyleParseError};
            #[allow(unused_imports)]
            use values::computed::{Context, ToComputedValue};
            #[allow(unused_imports)]
            use values::{computed, specified};
            #[allow(unused_imports)]
            use values::{Auto, Either, None_, Normal};
            
    use Atom;
    use std::fmt;
    use style_traits::ToCss;
    use values::KeyframesName;

    pub mod computed_value {
        pub use super::SpecifiedValue as T;
    }

    #[derive(Clone, Debug, Eq, Hash, PartialEq, ToComputedValue)]
    #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
    #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
    pub struct SpecifiedValue(pub Option<KeyframesName>);

    impl SpecifiedValue {
        /// As an Atom
        pub fn as_atom(&self) -> Option< &Atom> {
            self.0.as_ref().map(|n| n.as_atom())
        }
    }

    #[inline]
    pub fn get_initial_value() -> computed_value::T {
        get_initial_specified_value()
    }

    #[inline]
    pub fn get_initial_specified_value() -> SpecifiedValue {
        SpecifiedValue(None)
    }

    impl fmt::Display for SpecifiedValue {
        fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
            self.to_css(f)
        }
    }

    impl ToCss for SpecifiedValue {
        fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
            if let Some(ref name) = self.0 {
                name.to_css(dest)
            } else {
                dest.write_str("none")
            }
        }
    }

    impl Parse for SpecifiedValue {
        fn parse<'i, 't>(
            context: &ParserContext,
            input: &mut ::cssparser::Parser<'i, 't>
        ) -> Result<Self, ParseError<'i>> {
            if let Ok(name) = input.try(|input| KeyframesName::parse(context, input)) {
                Ok(SpecifiedValue(Some(name)))
            } else {
                input.expect_ident_matching("none").map(|_| SpecifiedValue(None)).map_err(|e| e.into())
            }
        }
    }

    pub fn parse<'i, 't>(
        context: &ParserContext,
        input: &mut Parser<'i, 't>,
    ) -> Result<SpecifiedValue,ParseError<'i>> {
        SpecifiedValue::parse(context, input)
    }


        }

        /// The definition of the computed value for animation-name.
        pub mod computed_value {
            pub use super::single_value::computed_value as single_value;
            pub use self::single_value::T as SingleComputedValue;
            use smallvec::{IntoIter, SmallVec};
            use values::computed::ComputedVecIter;

            /// The computed value, effectively a list of single values.
            #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
            #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
            #[derive(Clone, Debug, PartialEq)]
            pub struct T(
                pub SmallVec<[single_value::T; 1]>,
            );


            pub type Iter<'a, 'cx, 'cx_a> = ComputedVecIter<'a, 'cx, 'cx_a, super::single_value::SpecifiedValue>;

            impl IntoIterator for T {
                type Item = single_value::T;
                type IntoIter = IntoIter<[single_value::T; 1]>;
                fn into_iter(self) -> Self::IntoIter {
                    self.0.into_iter()
                }
            }
        }

        impl ToCss for computed_value::T {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        warn!("Found empty value for property animation-name");
                }
                for i in iter {
                    dest.write_str(::style_traits::Comma::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        /// The specified value of animation-name.
        #[derive(Clone, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        pub struct SpecifiedValue(pub Vec<single_value::SpecifiedValue>);

        impl ToCss for SpecifiedValue {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        warn!("Found empty value for property animation-name");
                }
                for i in iter {
                    dest.write_str(::style_traits::Comma::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        pub fn get_initial_value() -> computed_value::T {
                let mut v = SmallVec::new();
                v.push(single_value::get_initial_value());
                computed_value::T(v)
        }

        pub fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                -> Result<SpecifiedValue, ParseError<'i>> {
            use style_traits::Separator;


            ::style_traits::Comma::parse(input, |parser| {
                single_value::parse(context, parser)
            }).map(SpecifiedValue)
        }

        pub use self::single_value::SpecifiedValue as SingleSpecifiedValue;

        impl SpecifiedValue {
            pub fn compute_iter<'a, 'cx, 'cx_a>(&'a self, context: &'cx Context<'cx_a>)
                -> computed_value::Iter<'a, 'cx, 'cx_a> {
                computed_value::Iter::new(context, &self.0)
            }
        }

        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, context: &Context) -> computed_value::T {
                computed_value::T(self.compute_iter(context).collect())
            }

            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                SpecifiedValue(computed.0.iter()
                                    .map(ToComputedValue::from_computed_value)
                                    .collect())
            }
        }
    
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::AnimationName(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::AnimationName);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::AnimationName);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            // In the case of a vector property we want to pass
                            // down an iterator so that this can be computed
                            // without allocation
                            //
                            // However, computing requires a context, but the
                            // style struct being mutated is on the context. We
                            // temporarily remove it, mutate it, and then put it
                            // back. Vector longhands cannot touch their own
                            // style struct whilst computing, else this will
                            // panic.
                            let mut s =
                                context.builder.take_box();
                            {
                                let iter = specified_value.compute_iter(context);
                                s.set_animation_name(iter);
                            }
                            context.builder.put_box(s);
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_animation_name();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_animation_name();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::AnimationName)
            }
    }




    
        
    
    
    /// https://drafts.csswg.org/css-transitions/#propdef-transition-duration
    pub mod animation_duration {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        use std::fmt;
        use style_traits::{Separator, ToCss};

        pub mod single_value {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::ShorthandId;
            #[allow(unused_imports)]
            use selectors::parser::SelectorParseError;
            #[allow(unused_imports)]
            use style_traits::{ParseError, StyleParseError};
            #[allow(unused_imports)]
            use values::computed::{Context, ToComputedValue};
            #[allow(unused_imports)]
            use values::{computed, specified};
            #[allow(unused_imports)]
            use values::{Auto, Either, None_, Normal};
            
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::Time as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::Time as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::Time::zero() }
        #[inline] pub fn get_initial_specified_value() -> SpecifiedValue { specified::Time::zero() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::Time::parse_non_negative(context, input)
        }
    
        
        }

        /// The definition of the computed value for animation-duration.
        pub mod computed_value {
            pub use super::single_value::computed_value as single_value;
            pub use self::single_value::T as SingleComputedValue;
            use smallvec::{IntoIter, SmallVec};
            use values::computed::ComputedVecIter;

            /// The computed value, effectively a list of single values.
            #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
            #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
            #[derive(Clone, Debug, PartialEq)]
            pub struct T(
                pub SmallVec<[single_value::T; 1]>,
            );


            pub type Iter<'a, 'cx, 'cx_a> = ComputedVecIter<'a, 'cx, 'cx_a, super::single_value::SpecifiedValue>;

            impl IntoIterator for T {
                type Item = single_value::T;
                type IntoIter = IntoIter<[single_value::T; 1]>;
                fn into_iter(self) -> Self::IntoIter {
                    self.0.into_iter()
                }
            }
        }

        impl ToCss for computed_value::T {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        warn!("Found empty value for property animation-duration");
                }
                for i in iter {
                    dest.write_str(::style_traits::Comma::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        /// The specified value of animation-duration.
        #[derive(Clone, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        pub struct SpecifiedValue(pub Vec<single_value::SpecifiedValue>);

        impl ToCss for SpecifiedValue {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        warn!("Found empty value for property animation-duration");
                }
                for i in iter {
                    dest.write_str(::style_traits::Comma::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        pub fn get_initial_value() -> computed_value::T {
                let mut v = SmallVec::new();
                v.push(single_value::get_initial_value());
                computed_value::T(v)
        }

        pub fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                -> Result<SpecifiedValue, ParseError<'i>> {
            use style_traits::Separator;


            ::style_traits::Comma::parse(input, |parser| {
                single_value::parse(context, parser)
            }).map(SpecifiedValue)
        }

        pub use self::single_value::SpecifiedValue as SingleSpecifiedValue;

        impl SpecifiedValue {
            pub fn compute_iter<'a, 'cx, 'cx_a>(&'a self, context: &'cx Context<'cx_a>)
                -> computed_value::Iter<'a, 'cx, 'cx_a> {
                computed_value::Iter::new(context, &self.0)
            }
        }

        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, context: &Context) -> computed_value::T {
                computed_value::T(self.compute_iter(context).collect())
            }

            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                SpecifiedValue(computed.0.iter()
                                    .map(ToComputedValue::from_computed_value)
                                    .collect())
            }
        }
    
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::AnimationDuration(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::AnimationDuration);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::AnimationDuration);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            // In the case of a vector property we want to pass
                            // down an iterator so that this can be computed
                            // without allocation
                            //
                            // However, computing requires a context, but the
                            // style struct being mutated is on the context. We
                            // temporarily remove it, mutate it, and then put it
                            // back. Vector longhands cannot touch their own
                            // style struct whilst computing, else this will
                            // panic.
                            let mut s =
                                context.builder.take_box();
                            {
                                let iter = specified_value.compute_iter(context);
                                s.set_animation_duration(iter);
                            }
                            context.builder.put_box(s);
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_animation_duration();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_animation_duration();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::AnimationDuration)
            }
    }




// animation-timing-function is the exception to the rule for allowed_in_keyframe_block:
// https://drafts.csswg.org/css-animations/#keyframes

    
        
    
    
    /// https://drafts.csswg.org/css-transitions/#propdef-animation-timing-function
    pub mod animation_timing_function {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        use std::fmt;
        use style_traits::{Separator, ToCss};

        pub mod single_value {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::ShorthandId;
            #[allow(unused_imports)]
            use selectors::parser::SelectorParseError;
            #[allow(unused_imports)]
            use style_traits::{ParseError, StyleParseError};
            #[allow(unused_imports)]
            use values::computed::{Context, ToComputedValue};
            #[allow(unused_imports)]
            use values::{computed, specified};
            #[allow(unused_imports)]
            use values::{Auto, Either, None_, Normal};
            
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::TimingFunction as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::TimingFunction as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::TimingFunction::ease() }
        #[inline] pub fn get_initial_specified_value() -> SpecifiedValue { specified::TimingFunction::ease() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::TimingFunction::parse(context, input)
        }
    
        
        }

        /// The definition of the computed value for animation-timing-function.
        pub mod computed_value {
            pub use super::single_value::computed_value as single_value;
            pub use self::single_value::T as SingleComputedValue;
            use smallvec::{IntoIter, SmallVec};
            use values::computed::ComputedVecIter;

            /// The computed value, effectively a list of single values.
            #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
            #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
            #[derive(Clone, Debug, PartialEq)]
            pub struct T(
                pub SmallVec<[single_value::T; 1]>,
            );


            pub type Iter<'a, 'cx, 'cx_a> = ComputedVecIter<'a, 'cx, 'cx_a, super::single_value::SpecifiedValue>;

            impl IntoIterator for T {
                type Item = single_value::T;
                type IntoIter = IntoIter<[single_value::T; 1]>;
                fn into_iter(self) -> Self::IntoIter {
                    self.0.into_iter()
                }
            }
        }

        impl ToCss for computed_value::T {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        warn!("Found empty value for property animation-timing-function");
                }
                for i in iter {
                    dest.write_str(::style_traits::Comma::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        /// The specified value of animation-timing-function.
        #[derive(Clone, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        pub struct SpecifiedValue(pub Vec<single_value::SpecifiedValue>);

        impl ToCss for SpecifiedValue {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        warn!("Found empty value for property animation-timing-function");
                }
                for i in iter {
                    dest.write_str(::style_traits::Comma::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        pub fn get_initial_value() -> computed_value::T {
                let mut v = SmallVec::new();
                v.push(single_value::get_initial_value());
                computed_value::T(v)
        }

        pub fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                -> Result<SpecifiedValue, ParseError<'i>> {
            use style_traits::Separator;


            ::style_traits::Comma::parse(input, |parser| {
                single_value::parse(context, parser)
            }).map(SpecifiedValue)
        }

        pub use self::single_value::SpecifiedValue as SingleSpecifiedValue;

        impl SpecifiedValue {
            pub fn compute_iter<'a, 'cx, 'cx_a>(&'a self, context: &'cx Context<'cx_a>)
                -> computed_value::Iter<'a, 'cx, 'cx_a> {
                computed_value::Iter::new(context, &self.0)
            }
        }

        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, context: &Context) -> computed_value::T {
                computed_value::T(self.compute_iter(context).collect())
            }

            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                SpecifiedValue(computed.0.iter()
                                    .map(ToComputedValue::from_computed_value)
                                    .collect())
            }
        }
    
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::AnimationTimingFunction(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::AnimationTimingFunction);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::AnimationTimingFunction);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            // In the case of a vector property we want to pass
                            // down an iterator so that this can be computed
                            // without allocation
                            //
                            // However, computing requires a context, but the
                            // style struct being mutated is on the context. We
                            // temporarily remove it, mutate it, and then put it
                            // back. Vector longhands cannot touch their own
                            // style struct whilst computing, else this will
                            // panic.
                            let mut s =
                                context.builder.take_box();
                            {
                                let iter = specified_value.compute_iter(context);
                                s.set_animation_timing_function(iter);
                            }
                            context.builder.put_box(s);
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_animation_timing_function();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_animation_timing_function();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::AnimationTimingFunction)
            }
    }





    
    
    /// https://drafts.csswg.org/css-animations/#propdef-animation-iteration-count
    pub mod animation_iteration_count {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        use std::fmt;
        use style_traits::{Separator, ToCss};

        pub mod single_value {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::ShorthandId;
            #[allow(unused_imports)]
            use selectors::parser::SelectorParseError;
            #[allow(unused_imports)]
            use style_traits::{ParseError, StyleParseError};
            #[allow(unused_imports)]
            use values::computed::{Context, ToComputedValue};
            #[allow(unused_imports)]
            use values::{computed, specified};
            #[allow(unused_imports)]
            use values::{Auto, Either, None_, Normal};
            
    pub mod computed_value {
        pub use super::SpecifiedValue as T;
    }

    // https://drafts.csswg.org/css-animations/#animation-iteration-count
    #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
    #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
    #[derive(Clone, Debug, PartialEq, ToCss, ToComputedValue)]
    pub enum SpecifiedValue {
        Number(f32),
        Infinite,
    }

    impl Parse for SpecifiedValue {
        fn parse<'i, 't>(_context: &ParserContext, input: &mut ::cssparser::Parser<'i, 't>)
                         -> Result<Self, ParseError<'i>> {
            if input.try(|input| input.expect_ident_matching("infinite")).is_ok() {
                return Ok(SpecifiedValue::Infinite)
            }

            let number = input.expect_number()?;
            if number < 0.0 {
                return Err(StyleParseError::UnspecifiedError.into());
            }

            Ok(SpecifiedValue::Number(number))
        }
    }


    #[inline]
    pub fn get_initial_value() -> computed_value::T {
        get_initial_specified_value()
    }

    #[inline]
    pub fn get_initial_specified_value() -> SpecifiedValue {
        SpecifiedValue::Number(1.0)
    }

    #[inline]
    pub fn parse<'i, 't>(
        context: &ParserContext,
        input: &mut Parser<'i, 't>,
    ) -> Result<SpecifiedValue, ParseError<'i>> {
        SpecifiedValue::parse(context, input)
    }

        }

        /// The definition of the computed value for animation-iteration-count.
        pub mod computed_value {
            pub use super::single_value::computed_value as single_value;
            pub use self::single_value::T as SingleComputedValue;
            use smallvec::{IntoIter, SmallVec};
            use values::computed::ComputedVecIter;

            /// The computed value, effectively a list of single values.
            #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
            #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
            #[derive(Clone, Debug, PartialEq)]
            pub struct T(
                pub SmallVec<[single_value::T; 1]>,
            );


            pub type Iter<'a, 'cx, 'cx_a> = ComputedVecIter<'a, 'cx, 'cx_a, super::single_value::SpecifiedValue>;

            impl IntoIterator for T {
                type Item = single_value::T;
                type IntoIter = IntoIter<[single_value::T; 1]>;
                fn into_iter(self) -> Self::IntoIter {
                    self.0.into_iter()
                }
            }
        }

        impl ToCss for computed_value::T {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        warn!("Found empty value for property animation-iteration-count");
                }
                for i in iter {
                    dest.write_str(::style_traits::Comma::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        /// The specified value of animation-iteration-count.
        #[derive(Clone, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        pub struct SpecifiedValue(pub Vec<single_value::SpecifiedValue>);

        impl ToCss for SpecifiedValue {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        warn!("Found empty value for property animation-iteration-count");
                }
                for i in iter {
                    dest.write_str(::style_traits::Comma::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        pub fn get_initial_value() -> computed_value::T {
                let mut v = SmallVec::new();
                v.push(single_value::get_initial_value());
                computed_value::T(v)
        }

        pub fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                -> Result<SpecifiedValue, ParseError<'i>> {
            use style_traits::Separator;


            ::style_traits::Comma::parse(input, |parser| {
                single_value::parse(context, parser)
            }).map(SpecifiedValue)
        }

        pub use self::single_value::SpecifiedValue as SingleSpecifiedValue;

        impl SpecifiedValue {
            pub fn compute_iter<'a, 'cx, 'cx_a>(&'a self, context: &'cx Context<'cx_a>)
                -> computed_value::Iter<'a, 'cx, 'cx_a> {
                computed_value::Iter::new(context, &self.0)
            }
        }

        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, context: &Context) -> computed_value::T {
                computed_value::T(self.compute_iter(context).collect())
            }

            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                SpecifiedValue(computed.0.iter()
                                    .map(ToComputedValue::from_computed_value)
                                    .collect())
            }
        }
    
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::AnimationIterationCount(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::AnimationIterationCount);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::AnimationIterationCount);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            // In the case of a vector property we want to pass
                            // down an iterator so that this can be computed
                            // without allocation
                            //
                            // However, computing requires a context, but the
                            // style struct being mutated is on the context. We
                            // temporarily remove it, mutate it, and then put it
                            // back. Vector longhands cannot touch their own
                            // style struct whilst computing, else this will
                            // panic.
                            let mut s =
                                context.builder.take_box();
                            {
                                let iter = specified_value.compute_iter(context);
                                s.set_animation_iteration_count(iter);
                            }
                            context.builder.put_box(s);
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_animation_iteration_count();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_animation_iteration_count();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::AnimationIterationCount)
            }
    }





    
    

    
        
    
    
    /// https://drafts.csswg.org/css-animations/#propdef-animation-direction
    pub mod animation_direction {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        use std::fmt;
        use style_traits::{Separator, ToCss};

        pub mod single_value {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::ShorthandId;
            #[allow(unused_imports)]
            use selectors::parser::SelectorParseError;
            #[allow(unused_imports)]
            use style_traits::{ParseError, StyleParseError};
            #[allow(unused_imports)]
            use values::computed::{Context, ToComputedValue};
            #[allow(unused_imports)]
            use values::{computed, specified};
            #[allow(unused_imports)]
            use values::{Auto, Either, None_, Normal};
            
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "normal" => normal,
                    "reverse" => reverse,
                    "alternate" => alternate,
                    "alternate-reverse" => alternate_reverse,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::normal
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::normal
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::normal => computed_value::T::normal,
                        SpecifiedValue::reverse => computed_value::T::reverse,
                        SpecifiedValue::alternate => computed_value::T::alternate,
                        SpecifiedValue::alternate_reverse => computed_value::T::alternate_reverse,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::normal => SpecifiedValue::normal,
                        computed_value::T::reverse => SpecifiedValue::reverse,
                        computed_value::T::alternate => SpecifiedValue::alternate,
                        computed_value::T::alternate_reverse => SpecifiedValue::alternate_reverse,
                }
            }
        }
    
        
        }

        /// The definition of the computed value for animation-direction.
        pub mod computed_value {
            pub use super::single_value::computed_value as single_value;
            pub use self::single_value::T as SingleComputedValue;
            use smallvec::{IntoIter, SmallVec};
            use values::computed::ComputedVecIter;

            /// The computed value, effectively a list of single values.
            #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
            #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
            #[derive(Clone, Debug, PartialEq)]
            pub struct T(
                pub SmallVec<[single_value::T; 1]>,
            );


            pub type Iter<'a, 'cx, 'cx_a> = ComputedVecIter<'a, 'cx, 'cx_a, super::single_value::SpecifiedValue>;

            impl IntoIterator for T {
                type Item = single_value::T;
                type IntoIter = IntoIter<[single_value::T; 1]>;
                fn into_iter(self) -> Self::IntoIter {
                    self.0.into_iter()
                }
            }
        }

        impl ToCss for computed_value::T {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        warn!("Found empty value for property animation-direction");
                }
                for i in iter {
                    dest.write_str(::style_traits::Comma::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        /// The specified value of animation-direction.
        #[derive(Clone, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        pub struct SpecifiedValue(pub Vec<single_value::SpecifiedValue>);

        impl ToCss for SpecifiedValue {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        warn!("Found empty value for property animation-direction");
                }
                for i in iter {
                    dest.write_str(::style_traits::Comma::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        pub fn get_initial_value() -> computed_value::T {
                let mut v = SmallVec::new();
                v.push(single_value::get_initial_value());
                computed_value::T(v)
        }

        pub fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                -> Result<SpecifiedValue, ParseError<'i>> {
            use style_traits::Separator;


            ::style_traits::Comma::parse(input, |parser| {
                single_value::parse(context, parser)
            }).map(SpecifiedValue)
        }

        pub use self::single_value::SpecifiedValue as SingleSpecifiedValue;

        impl SpecifiedValue {
            pub fn compute_iter<'a, 'cx, 'cx_a>(&'a self, context: &'cx Context<'cx_a>)
                -> computed_value::Iter<'a, 'cx, 'cx_a> {
                computed_value::Iter::new(context, &self.0)
            }
        }

        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, context: &Context) -> computed_value::T {
                computed_value::T(self.compute_iter(context).collect())
            }

            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                SpecifiedValue(computed.0.iter()
                                    .map(ToComputedValue::from_computed_value)
                                    .collect())
            }
        }
    
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::AnimationDirection(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::AnimationDirection);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::AnimationDirection);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            // In the case of a vector property we want to pass
                            // down an iterator so that this can be computed
                            // without allocation
                            //
                            // However, computing requires a context, but the
                            // style struct being mutated is on the context. We
                            // temporarily remove it, mutate it, and then put it
                            // back. Vector longhands cannot touch their own
                            // style struct whilst computing, else this will
                            // panic.
                            let mut s =
                                context.builder.take_box();
                            {
                                let iter = specified_value.compute_iter(context);
                                s.set_animation_direction(iter);
                            }
                            context.builder.put_box(s);
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_animation_direction();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_animation_direction();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::AnimationDirection)
            }
    }






    
    

    
        
    
    
    /// https://drafts.csswg.org/css-animations/#propdef-animation-play-state
    pub mod animation_play_state {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        use std::fmt;
        use style_traits::{Separator, ToCss};

        pub mod single_value {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::ShorthandId;
            #[allow(unused_imports)]
            use selectors::parser::SelectorParseError;
            #[allow(unused_imports)]
            use style_traits::{ParseError, StyleParseError};
            #[allow(unused_imports)]
            use values::computed::{Context, ToComputedValue};
            #[allow(unused_imports)]
            use values::{computed, specified};
            #[allow(unused_imports)]
            use values::{Auto, Either, None_, Normal};
            
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "running" => running,
                    "paused" => paused,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::running
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::running
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::running => computed_value::T::running,
                        SpecifiedValue::paused => computed_value::T::paused,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::running => SpecifiedValue::running,
                        computed_value::T::paused => SpecifiedValue::paused,
                }
            }
        }
    
        
        }

        /// The definition of the computed value for animation-play-state.
        pub mod computed_value {
            pub use super::single_value::computed_value as single_value;
            pub use self::single_value::T as SingleComputedValue;
            use smallvec::{IntoIter, SmallVec};
            use values::computed::ComputedVecIter;

            /// The computed value, effectively a list of single values.
            #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
            #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
            #[derive(Clone, Debug, PartialEq)]
            pub struct T(
                pub SmallVec<[single_value::T; 1]>,
            );


            pub type Iter<'a, 'cx, 'cx_a> = ComputedVecIter<'a, 'cx, 'cx_a, super::single_value::SpecifiedValue>;

            impl IntoIterator for T {
                type Item = single_value::T;
                type IntoIter = IntoIter<[single_value::T; 1]>;
                fn into_iter(self) -> Self::IntoIter {
                    self.0.into_iter()
                }
            }
        }

        impl ToCss for computed_value::T {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        warn!("Found empty value for property animation-play-state");
                }
                for i in iter {
                    dest.write_str(::style_traits::Comma::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        /// The specified value of animation-play-state.
        #[derive(Clone, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        pub struct SpecifiedValue(pub Vec<single_value::SpecifiedValue>);

        impl ToCss for SpecifiedValue {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        warn!("Found empty value for property animation-play-state");
                }
                for i in iter {
                    dest.write_str(::style_traits::Comma::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        pub fn get_initial_value() -> computed_value::T {
                let mut v = SmallVec::new();
                v.push(single_value::get_initial_value());
                computed_value::T(v)
        }

        pub fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                -> Result<SpecifiedValue, ParseError<'i>> {
            use style_traits::Separator;


            ::style_traits::Comma::parse(input, |parser| {
                single_value::parse(context, parser)
            }).map(SpecifiedValue)
        }

        pub use self::single_value::SpecifiedValue as SingleSpecifiedValue;

        impl SpecifiedValue {
            pub fn compute_iter<'a, 'cx, 'cx_a>(&'a self, context: &'cx Context<'cx_a>)
                -> computed_value::Iter<'a, 'cx, 'cx_a> {
                computed_value::Iter::new(context, &self.0)
            }
        }

        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, context: &Context) -> computed_value::T {
                computed_value::T(self.compute_iter(context).collect())
            }

            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                SpecifiedValue(computed.0.iter()
                                    .map(ToComputedValue::from_computed_value)
                                    .collect())
            }
        }
    
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::AnimationPlayState(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::AnimationPlayState);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::AnimationPlayState);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            // In the case of a vector property we want to pass
                            // down an iterator so that this can be computed
                            // without allocation
                            //
                            // However, computing requires a context, but the
                            // style struct being mutated is on the context. We
                            // temporarily remove it, mutate it, and then put it
                            // back. Vector longhands cannot touch their own
                            // style struct whilst computing, else this will
                            // panic.
                            let mut s =
                                context.builder.take_box();
                            {
                                let iter = specified_value.compute_iter(context);
                                s.set_animation_play_state(iter);
                            }
                            context.builder.put_box(s);
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_animation_play_state();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_animation_play_state();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::AnimationPlayState)
            }
    }






    
    

    
        
    
    
    /// https://drafts.csswg.org/css-animations/#propdef-animation-fill-mode
    pub mod animation_fill_mode {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        use std::fmt;
        use style_traits::{Separator, ToCss};

        pub mod single_value {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::ShorthandId;
            #[allow(unused_imports)]
            use selectors::parser::SelectorParseError;
            #[allow(unused_imports)]
            use style_traits::{ParseError, StyleParseError};
            #[allow(unused_imports)]
            use values::computed::{Context, ToComputedValue};
            #[allow(unused_imports)]
            use values::{computed, specified};
            #[allow(unused_imports)]
            use values::{Auto, Either, None_, Normal};
            
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "none" => none,
                    "forwards" => forwards,
                    "backwards" => backwards,
                    "both" => both,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::none
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::none
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::none => computed_value::T::none,
                        SpecifiedValue::forwards => computed_value::T::forwards,
                        SpecifiedValue::backwards => computed_value::T::backwards,
                        SpecifiedValue::both => computed_value::T::both,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::none => SpecifiedValue::none,
                        computed_value::T::forwards => SpecifiedValue::forwards,
                        computed_value::T::backwards => SpecifiedValue::backwards,
                        computed_value::T::both => SpecifiedValue::both,
                }
            }
        }
    
        
        }

        /// The definition of the computed value for animation-fill-mode.
        pub mod computed_value {
            pub use super::single_value::computed_value as single_value;
            pub use self::single_value::T as SingleComputedValue;
            use smallvec::{IntoIter, SmallVec};
            use values::computed::ComputedVecIter;

            /// The computed value, effectively a list of single values.
            #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
            #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
            #[derive(Clone, Debug, PartialEq)]
            pub struct T(
                pub SmallVec<[single_value::T; 1]>,
            );


            pub type Iter<'a, 'cx, 'cx_a> = ComputedVecIter<'a, 'cx, 'cx_a, super::single_value::SpecifiedValue>;

            impl IntoIterator for T {
                type Item = single_value::T;
                type IntoIter = IntoIter<[single_value::T; 1]>;
                fn into_iter(self) -> Self::IntoIter {
                    self.0.into_iter()
                }
            }
        }

        impl ToCss for computed_value::T {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        warn!("Found empty value for property animation-fill-mode");
                }
                for i in iter {
                    dest.write_str(::style_traits::Comma::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        /// The specified value of animation-fill-mode.
        #[derive(Clone, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        pub struct SpecifiedValue(pub Vec<single_value::SpecifiedValue>);

        impl ToCss for SpecifiedValue {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        warn!("Found empty value for property animation-fill-mode");
                }
                for i in iter {
                    dest.write_str(::style_traits::Comma::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        pub fn get_initial_value() -> computed_value::T {
                let mut v = SmallVec::new();
                v.push(single_value::get_initial_value());
                computed_value::T(v)
        }

        pub fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                -> Result<SpecifiedValue, ParseError<'i>> {
            use style_traits::Separator;


            ::style_traits::Comma::parse(input, |parser| {
                single_value::parse(context, parser)
            }).map(SpecifiedValue)
        }

        pub use self::single_value::SpecifiedValue as SingleSpecifiedValue;

        impl SpecifiedValue {
            pub fn compute_iter<'a, 'cx, 'cx_a>(&'a self, context: &'cx Context<'cx_a>)
                -> computed_value::Iter<'a, 'cx, 'cx_a> {
                computed_value::Iter::new(context, &self.0)
            }
        }

        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, context: &Context) -> computed_value::T {
                computed_value::T(self.compute_iter(context).collect())
            }

            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                SpecifiedValue(computed.0.iter()
                                    .map(ToComputedValue::from_computed_value)
                                    .collect())
            }
        }
    
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::AnimationFillMode(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::AnimationFillMode);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::AnimationFillMode);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            // In the case of a vector property we want to pass
                            // down an iterator so that this can be computed
                            // without allocation
                            //
                            // However, computing requires a context, but the
                            // style struct being mutated is on the context. We
                            // temporarily remove it, mutate it, and then put it
                            // back. Vector longhands cannot touch their own
                            // style struct whilst computing, else this will
                            // panic.
                            let mut s =
                                context.builder.take_box();
                            {
                                let iter = specified_value.compute_iter(context);
                                s.set_animation_fill_mode(iter);
                            }
                            context.builder.put_box(s);
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_animation_fill_mode();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_animation_fill_mode();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::AnimationFillMode)
            }
    }






    
        
    
    
    /// https://drafts.csswg.org/css-animations/#propdef-animation-delay
    pub mod animation_delay {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        use std::fmt;
        use style_traits::{Separator, ToCss};

        pub mod single_value {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::ShorthandId;
            #[allow(unused_imports)]
            use selectors::parser::SelectorParseError;
            #[allow(unused_imports)]
            use style_traits::{ParseError, StyleParseError};
            #[allow(unused_imports)]
            use values::computed::{Context, ToComputedValue};
            #[allow(unused_imports)]
            use values::{computed, specified};
            #[allow(unused_imports)]
            use values::{Auto, Either, None_, Normal};
            
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::Time as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::Time as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::Time::zero() }
        #[inline] pub fn get_initial_specified_value() -> SpecifiedValue { specified::Time::zero() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::Time::parse(context, input)
        }
    
        
        }

        /// The definition of the computed value for animation-delay.
        pub mod computed_value {
            pub use super::single_value::computed_value as single_value;
            pub use self::single_value::T as SingleComputedValue;
            use smallvec::{IntoIter, SmallVec};
            use values::computed::ComputedVecIter;

            /// The computed value, effectively a list of single values.
            #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
            #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
            #[derive(Clone, Debug, PartialEq)]
            pub struct T(
                pub SmallVec<[single_value::T; 1]>,
            );


            pub type Iter<'a, 'cx, 'cx_a> = ComputedVecIter<'a, 'cx, 'cx_a, super::single_value::SpecifiedValue>;

            impl IntoIterator for T {
                type Item = single_value::T;
                type IntoIter = IntoIter<[single_value::T; 1]>;
                fn into_iter(self) -> Self::IntoIter {
                    self.0.into_iter()
                }
            }
        }

        impl ToCss for computed_value::T {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        warn!("Found empty value for property animation-delay");
                }
                for i in iter {
                    dest.write_str(::style_traits::Comma::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        /// The specified value of animation-delay.
        #[derive(Clone, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        pub struct SpecifiedValue(pub Vec<single_value::SpecifiedValue>);

        impl ToCss for SpecifiedValue {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        warn!("Found empty value for property animation-delay");
                }
                for i in iter {
                    dest.write_str(::style_traits::Comma::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        pub fn get_initial_value() -> computed_value::T {
                let mut v = SmallVec::new();
                v.push(single_value::get_initial_value());
                computed_value::T(v)
        }

        pub fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                -> Result<SpecifiedValue, ParseError<'i>> {
            use style_traits::Separator;


            ::style_traits::Comma::parse(input, |parser| {
                single_value::parse(context, parser)
            }).map(SpecifiedValue)
        }

        pub use self::single_value::SpecifiedValue as SingleSpecifiedValue;

        impl SpecifiedValue {
            pub fn compute_iter<'a, 'cx, 'cx_a>(&'a self, context: &'cx Context<'cx_a>)
                -> computed_value::Iter<'a, 'cx, 'cx_a> {
                computed_value::Iter::new(context, &self.0)
            }
        }

        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, context: &Context) -> computed_value::T {
                computed_value::T(self.compute_iter(context).collect())
            }

            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                SpecifiedValue(computed.0.iter()
                                    .map(ToComputedValue::from_computed_value)
                                    .collect())
            }
        }
    
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::AnimationDelay(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::AnimationDelay);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::AnimationDelay);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            // In the case of a vector property we want to pass
                            // down an iterator so that this can be computed
                            // without allocation
                            //
                            // However, computing requires a context, but the
                            // style struct being mutated is on the context. We
                            // temporarily remove it, mutate it, and then put it
                            // back. Vector longhands cannot touch their own
                            // style struct whilst computing, else this will
                            // panic.
                            let mut s =
                                context.builder.take_box();
                            {
                                let iter = specified_value.compute_iter(context);
                                s.set_animation_delay(iter);
                            }
                            context.builder.put_box(s);
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_animation_delay();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_animation_delay();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::AnimationDelay)
            }
    }




    
    
        
    
    /// Nonstandard (https://www.w3.org/TR/2015/WD-css-snappoints-1-20150326/#scroll-snap-points)
    pub mod scroll_snap_points_x {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::ScrollSnapPoint as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::ScrollSnapPoint as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::ScrollSnapPoint::none() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::ScrollSnapPoint::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::ScrollSnapPointsX(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::ScrollSnapPointsX);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::ScrollSnapPointsX);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_scroll_snap_points_x(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_scroll_snap_points_x();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_scroll_snap_points_x();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::ScrollSnapPointsX)
            }
    }


    
    
        
    
    /// Nonstandard (https://www.w3.org/TR/2015/WD-css-snappoints-1-20150326/#scroll-snap-points)
    pub mod scroll_snap_points_y {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::ScrollSnapPoint as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::ScrollSnapPoint as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::ScrollSnapPoint::none() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::ScrollSnapPoint::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::ScrollSnapPointsY(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::ScrollSnapPointsY);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::ScrollSnapPointsY);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_scroll_snap_points_y(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_scroll_snap_points_y();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_scroll_snap_points_y();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::ScrollSnapPointsY)
            }
    }




    
        
    
    /// Nonstandard (https://developer.mozilla.org/en-US/docs/Web/CSS/scroll-snap-destination)
    pub mod scroll_snap_destination {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::Position as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::Position as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::Position::zero() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::Position::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::ScrollSnapDestination(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::ScrollSnapDestination);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::ScrollSnapDestination);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = (**specified_value).to_computed_value(context);
                                context.builder.set_scroll_snap_destination(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_scroll_snap_destination();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_scroll_snap_destination();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<Box<SpecifiedValue>, ParseError<'i>> {
                    parse(context, input).map(|result| Box::new(result))
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::ScrollSnapDestination)
            }
    }




    
        
    
    
    /// Nonstandard (https://developer.mozilla.org/en-US/docs/Web/CSS/scroll-snap-destination)
    pub mod scroll_snap_coordinate {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        use std::fmt;
        use style_traits::{Separator, ToCss};

        pub mod single_value {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::ShorthandId;
            #[allow(unused_imports)]
            use selectors::parser::SelectorParseError;
            #[allow(unused_imports)]
            use style_traits::{ParseError, StyleParseError};
            #[allow(unused_imports)]
            use values::computed::{Context, ToComputedValue};
            #[allow(unused_imports)]
            use values::{computed, specified};
            #[allow(unused_imports)]
            use values::{Auto, Either, None_, Normal};
            
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::Position as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::Position as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::Position::zero() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::Position::parse(context, input)
        }
    
        
        }

        /// The definition of the computed value for scroll-snap-coordinate.
        pub mod computed_value {
            pub use super::single_value::computed_value as single_value;
            pub use self::single_value::T as SingleComputedValue;
            use smallvec::{IntoIter, SmallVec};
            use values::computed::ComputedVecIter;

            /// The computed value, effectively a list of single values.
            #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
            #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
            #[derive(Clone, Debug, PartialEq)]
            pub struct T(
                pub SmallVec<[single_value::T; 1]>,
            );


            pub type Iter<'a, 'cx, 'cx_a> = ComputedVecIter<'a, 'cx, 'cx_a, super::single_value::SpecifiedValue>;

            impl IntoIterator for T {
                type Item = single_value::T;
                type IntoIter = IntoIter<[single_value::T; 1]>;
                fn into_iter(self) -> Self::IntoIter {
                    self.0.into_iter()
                }
            }
        }

        impl ToCss for computed_value::T {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        dest.write_str("none")?;
                }
                for i in iter {
                    dest.write_str(::style_traits::Comma::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        /// The specified value of scroll-snap-coordinate.
        #[derive(Clone, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        pub struct SpecifiedValue(pub Vec<single_value::SpecifiedValue>);

        impl ToCss for SpecifiedValue {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        dest.write_str("none")?;
                }
                for i in iter {
                    dest.write_str(::style_traits::Comma::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        pub fn get_initial_value() -> computed_value::T {
                let mut v = SmallVec::new();
                v.push(single_value::get_initial_value());
                computed_value::T(v)
        }

        pub fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                -> Result<SpecifiedValue, ParseError<'i>> {
            use style_traits::Separator;

                if input.try(|input| input.expect_ident_matching("none")).is_ok() {
                    return Ok(SpecifiedValue(Vec::new()))
                }

            ::style_traits::Comma::parse(input, |parser| {
                single_value::parse(context, parser)
            }).map(SpecifiedValue)
        }

        pub use self::single_value::SpecifiedValue as SingleSpecifiedValue;

        impl SpecifiedValue {
            pub fn compute_iter<'a, 'cx, 'cx_a>(&'a self, context: &'cx Context<'cx_a>)
                -> computed_value::Iter<'a, 'cx, 'cx_a> {
                computed_value::Iter::new(context, &self.0)
            }
        }

        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, context: &Context) -> computed_value::T {
                computed_value::T(self.compute_iter(context).collect())
            }

            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                SpecifiedValue(computed.0.iter()
                                    .map(ToComputedValue::from_computed_value)
                                    .collect())
            }
        }
    
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::ScrollSnapCoordinate(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::ScrollSnapCoordinate);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::ScrollSnapCoordinate);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            // In the case of a vector property we want to pass
                            // down an iterator so that this can be computed
                            // without allocation
                            //
                            // However, computing requires a context, but the
                            // style struct being mutated is on the context. We
                            // temporarily remove it, mutate it, and then put it
                            // back. Vector longhands cannot touch their own
                            // style struct whilst computing, else this will
                            // panic.
                            let mut s =
                                context.builder.take_box();
                            {
                                let iter = specified_value.compute_iter(context);
                                s.set_scroll_snap_coordinate(iter);
                            }
                            context.builder.put_box(s);
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_scroll_snap_coordinate();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_scroll_snap_coordinate();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::ScrollSnapCoordinate)
            }
    }





    
    /// https://drafts.csswg.org/css-transforms/#propdef-transform
    pub mod transform {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
    use values::computed::{LengthOrPercentageOrNumber as ComputedLoPoNumber, LengthOrNumber as ComputedLoN};
    use values::computed::{LengthOrPercentage as ComputedLoP, Length as ComputedLength};
    use values::generics::transform::Matrix;
    use values::specified::{Angle, Integer, Length, LengthOrPercentage};
    use values::specified::{LengthOrNumber, LengthOrPercentageOrNumber as LoPoNumber, Number};
    use style_traits::ToCss;

    use std::fmt;

    pub mod computed_value {
        use values::CSSFloat;
        use values::computed;
        use values::computed::{Length, LengthOrPercentage};

        #[derive(Clone, Copy, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        pub struct ComputedMatrix {
            pub m11: CSSFloat, pub m12: CSSFloat, pub m13: CSSFloat, pub m14: CSSFloat,
            pub m21: CSSFloat, pub m22: CSSFloat, pub m23: CSSFloat, pub m24: CSSFloat,
            pub m31: CSSFloat, pub m32: CSSFloat, pub m33: CSSFloat, pub m34: CSSFloat,
            pub m41: CSSFloat, pub m42: CSSFloat, pub m43: CSSFloat, pub m44: CSSFloat,
        }

        #[derive(Clone, Copy, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        pub struct ComputedMatrixWithPercents {
            pub m11: CSSFloat, pub m12: CSSFloat, pub m13: CSSFloat, pub m14: CSSFloat,
            pub m21: CSSFloat, pub m22: CSSFloat, pub m23: CSSFloat, pub m24: CSSFloat,
            pub m31: CSSFloat, pub m32: CSSFloat, pub m33: CSSFloat, pub m34: CSSFloat,
            pub m41: LengthOrPercentage, pub m42: LengthOrPercentage,
            pub m43: Length, pub m44: CSSFloat,
        }

        impl ComputedMatrix {
            pub fn identity() -> ComputedMatrix {
                ComputedMatrix {
                    m11: 1.0, m12: 0.0, m13: 0.0, m14: 0.0,
                    m21: 0.0, m22: 1.0, m23: 0.0, m24: 0.0,
                    m31: 0.0, m32: 0.0, m33: 1.0, m34: 0.0,
                    m41: 0.0, m42: 0.0, m43: 0.0, m44: 1.0
                }
            }
        }

        impl ComputedMatrixWithPercents {
            pub fn identity() -> ComputedMatrixWithPercents {
                ComputedMatrixWithPercents {
                    m11: 1.0, m12: 0.0, m13: 0.0, m14: 0.0,
                    m21: 0.0, m22: 1.0, m23: 0.0, m24: 0.0,
                    m31: 0.0, m32: 0.0, m33: 1.0, m34: 0.0,
                    m41: LengthOrPercentage::zero(), m42: LengthOrPercentage::zero(),
                    m43: Length::new(0.), m44: 1.0
                }
            }
        }

        #[derive(Clone, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        pub enum ComputedOperation {
            Matrix(ComputedMatrix),
            // For `-moz-transform` matrix and matrix3d.
            MatrixWithPercents(ComputedMatrixWithPercents),
            Skew(computed::Angle, computed::Angle),
            Translate(computed::LengthOrPercentage,
                      computed::LengthOrPercentage,
                      computed::Length),
            Scale(CSSFloat, CSSFloat, CSSFloat),
            Rotate(CSSFloat, CSSFloat, CSSFloat, computed::Angle),
            Perspective(computed::Length),
            // For mismatched transform lists.
            // A vector of |ComputedOperation| could contain an |InterpolateMatrix| and other
            // |ComputedOperation|s, and multiple nested |InterpolateMatrix|s is acceptable.
            // e.g.
            // [ InterpolateMatrix { from_list: [ InterpolateMatrix { ... },
            //                                    Scale(...) ],
            //                       to_list: [ AccumulateMatrix { from_list: ...,
            //                                                     to_list: [ InterpolateMatrix,
            //                                                                 ... ],
            //                                                     count: ... } ],
            //                       progress: ... } ]
            InterpolateMatrix { from_list: T,
                                to_list: T,
                                progress: computed::Percentage },
            // For accumulate operation of mismatched transform lists.
            AccumulateMatrix { from_list: T,
                               to_list: T,
                               count: computed::Integer },
        }

        #[derive(Clone, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        pub struct T(pub Option<Vec<ComputedOperation>>);
    }

    /// Describes a single parsed
    /// [Transform Function](https://drafts.csswg.org/css-transforms/#typedef-transform-function).
    ///
    /// Multiple transform functions compose a transformation.
    ///
    /// Some transformations can be expressed by other more general functions.
    #[derive(Clone, Debug, PartialEq)]
    #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
    #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
    pub enum SpecifiedOperation {
        /// Represents a 2D 2x3 matrix.
        Matrix(Matrix<Number>),
        /// Represents a 3D 4x4 matrix with percentage and length values.
        /// For `moz-transform`.
        PrefixedMatrix(Matrix<Number, LoPoNumber>),
        /// Represents a 3D 4x4 matrix.
        Matrix3D {
            m11: Number, m12: Number, m13: Number, m14: Number,
            m21: Number, m22: Number, m23: Number, m24: Number,
            m31: Number, m32: Number, m33: Number, m34: Number,
            m41: Number, m42: Number, m43: Number, m44: Number,
        },
        /// Represents a 3D 4x4 matrix with percentage and length values.
        /// For `moz-transform`.
        PrefixedMatrix3D {
            m11: Number,     m12: Number,     m13: Number,         m14: Number,
            m21: Number,     m22: Number,     m23: Number,         m24: Number,
            m31: Number,     m32: Number,     m33: Number,         m34: Number,
            m41: LoPoNumber, m42: LoPoNumber, m43: LengthOrNumber, m44: Number,
        },
        /// A 2D skew.
        ///
        /// If the second angle is not provided it is assumed zero.
        Skew(Angle, Option<Angle>),
        SkewX(Angle),
        SkewY(Angle),
        Translate(LengthOrPercentage, Option<LengthOrPercentage>),
        TranslateX(LengthOrPercentage),
        TranslateY(LengthOrPercentage),
        TranslateZ(Length),
        Translate3D(LengthOrPercentage, LengthOrPercentage, Length),
        /// A 2D scaling factor.
        ///
        /// `scale(2)` is parsed as `Scale(Number::new(2.0), None)` and is equivalent to
        /// writing `scale(2, 2)` (`Scale(Number::new(2.0), Some(Number::new(2.0)))`).
        ///
        /// Negative values are allowed and flip the element.
        Scale(Number, Option<Number>),
        ScaleX(Number),
        ScaleY(Number),
        ScaleZ(Number),
        Scale3D(Number, Number, Number),
        /// Describes a 2D Rotation.
        ///
        /// In a 3D scene `rotate(angle)` is equivalent to `rotateZ(angle)`.
        Rotate(Angle),
        /// Rotation in 3D space around the x-axis.
        RotateX(Angle),
        /// Rotation in 3D space around the y-axis.
        RotateY(Angle),
        /// Rotation in 3D space around the z-axis.
        RotateZ(Angle),
        /// Rotation in 3D space.
        ///
        /// Generalization of rotateX, rotateY and rotateZ.
        Rotate3D(Number, Number, Number, Angle),
        /// Specifies a perspective projection matrix.
        ///
        /// Part of CSS Transform Module Level 2 and defined at
        /// [§ 13.1. 3D Transform Function](https://drafts.csswg.org/css-transforms-2/#funcdef-perspective).
        ///
        /// The value must be greater than or equal to zero.
        Perspective(specified::Length),
        /// A intermediate type for interpolation of mismatched transform lists.
        InterpolateMatrix { from_list: SpecifiedValue,
                            to_list: SpecifiedValue,
                            progress: computed::Percentage },
        /// A intermediate type for accumulation of mismatched transform lists.
        AccumulateMatrix { from_list: SpecifiedValue,
                           to_list: SpecifiedValue,
                           count: Integer },
    }

    impl ToCss for computed_value::T {
        fn to_css<W>(&self, _: &mut W) -> fmt::Result where W: fmt::Write {
            // TODO(pcwalton)
            Ok(())
        }
    }

    impl ToCss for SpecifiedOperation {
        fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
            match *self {
                SpecifiedOperation::Matrix(ref m) => m.to_css(dest),
                SpecifiedOperation::PrefixedMatrix(ref m) => m.to_css(dest),
                SpecifiedOperation::Matrix3D {
                    m11, m12, m13, m14,
                    m21, m22, m23, m24,
                    m31, m32, m33, m34,
                    m41, m42, m43, m44,
                } => {
                    serialize_function!(dest, matrix3d(
                        m11, m12, m13, m14,
                        m21, m22, m23, m24,
                        m31, m32, m33, m34,
                        m41, m42, m43, m44,
                    ))
                }
                SpecifiedOperation::PrefixedMatrix3D {
                    m11, m12, m13, m14,
                    m21, m22, m23, m24,
                    m31, m32, m33, m34,
                    ref m41, ref m42, ref m43, m44,
                } => {
                    serialize_function!(dest, matrix3d(
                        m11, m12, m13, m14,
                        m21, m22, m23, m24,
                        m31, m32, m33, m34,
                        m41, m42, m43, m44,
                    ))
                }
                SpecifiedOperation::Skew(ax, None) => {
                    serialize_function!(dest, skew(ax))
                }
                SpecifiedOperation::Skew(ax, Some(ay)) => {
                    serialize_function!(dest, skew(ax, ay))
                }
                SpecifiedOperation::SkewX(angle) => {
                    serialize_function!(dest, skewX(angle))
                }
                SpecifiedOperation::SkewY(angle) => {
                    serialize_function!(dest, skewY(angle))
                }
                SpecifiedOperation::Translate(ref tx, None) => {
                    serialize_function!(dest, translate(tx))
                }
                SpecifiedOperation::Translate(ref tx, Some(ref ty)) => {
                    serialize_function!(dest, translate(tx, ty))
                }
                SpecifiedOperation::TranslateX(ref tx) => {
                    serialize_function!(dest, translateX(tx))
                }
                SpecifiedOperation::TranslateY(ref ty) => {
                    serialize_function!(dest, translateY(ty))
                }
                SpecifiedOperation::TranslateZ(ref tz) => {
                    serialize_function!(dest, translateZ(tz))
                }
                SpecifiedOperation::Translate3D(ref tx, ref ty, ref tz) => {
                    serialize_function!(dest, translate3d(tx, ty, tz))
                }
                SpecifiedOperation::Scale(factor, None) => {
                    serialize_function!(dest, scale(factor))
                }
                SpecifiedOperation::Scale(sx, Some(sy)) => {
                    serialize_function!(dest, scale(sx, sy))
                }
                SpecifiedOperation::ScaleX(sx) => {
                    serialize_function!(dest, scaleX(sx))
                }
                SpecifiedOperation::ScaleY(sy) => {
                    serialize_function!(dest, scaleY(sy))
                }
                SpecifiedOperation::ScaleZ(sz) => {
                    serialize_function!(dest, scaleZ(sz))
                }
                SpecifiedOperation::Scale3D(sx, sy, sz) => {
                    serialize_function!(dest, scale3d(sx, sy, sz))
                }
                SpecifiedOperation::Rotate(theta) => {
                    serialize_function!(dest, rotate(theta))
                }
                SpecifiedOperation::RotateX(theta) => {
                    serialize_function!(dest, rotateX(theta))
                }
                SpecifiedOperation::RotateY(theta) => {
                    serialize_function!(dest, rotateY(theta))
                }
                SpecifiedOperation::RotateZ(theta) => {
                    serialize_function!(dest, rotateZ(theta))
                }
                SpecifiedOperation::Rotate3D(x, y, z, theta) => {
                    serialize_function!(dest, rotate3d(x, y, z, theta))
                }
                SpecifiedOperation::Perspective(ref length) => {
                    serialize_function!(dest, perspective(length))
                }
                SpecifiedOperation::InterpolateMatrix { ref from_list, ref to_list, progress } => {
                    serialize_function!(dest, interpolatematrix(from_list, to_list, progress))
                }
                SpecifiedOperation::AccumulateMatrix { ref from_list, ref to_list, count } => {
                    serialize_function!(dest, accumulatematrix(from_list, to_list, count))
                }
            }
        }
    }

    #[derive(Clone, Debug, PartialEq)]
    #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
    #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
    pub struct SpecifiedValue(Vec<SpecifiedOperation>);

    impl ToCss for SpecifiedValue {
        fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {

            if self.0.is_empty() {
                return dest.write_str("none")
            }

            let mut first = true;
            for operation in &self.0 {
                if !first {
                    dest.write_str(" ")?;
                }
                first = false;
                operation.to_css(dest)?
            }
            Ok(())
        }
    }

    #[inline]
    pub fn get_initial_value() -> computed_value::T {
        computed_value::T(None)
    }

    // Allow unitless zero angle for rotate() and skew() to align with gecko
    fn parse_internal<'i, 't>(
        context: &ParserContext,
        input: &mut Parser<'i, 't>,
        prefixed: bool,
    ) -> Result<SpecifiedValue,ParseError<'i>> {
        use style_traits::{Separator, Space};

        if input.try(|input| input.expect_ident_matching("none")).is_ok() {
            return Ok(SpecifiedValue(Vec::new()))
        }

        Ok(SpecifiedValue(Space::parse(input, |input| {
            let function = input.expect_function()?.clone();
            input.parse_nested_block(|input| {
                let result = match_ignore_ascii_case! { &function,
                    "matrix" => {
                        let a = specified::parse_number(context, input)?;
                        input.expect_comma()?;
                        let b = specified::parse_number(context, input)?;
                        input.expect_comma()?;
                        let c = specified::parse_number(context, input)?;
                        input.expect_comma()?;
                        let d = specified::parse_number(context, input)?;
                        input.expect_comma()?;
                        if !prefixed {
                            // Standard matrix parsing.
                            let e = specified::parse_number(context, input)?;
                            input.expect_comma()?;
                            let f = specified::parse_number(context, input)?;
                            Ok(SpecifiedOperation::Matrix(Matrix { a, b, c, d, e, f }))
                        } else {
                            // Non-standard prefixed matrix parsing for -moz-transform.
                            let e = LoPoNumber::parse(context, input)?;
                            input.expect_comma()?;
                            let f = LoPoNumber::parse(context, input)?;
                            Ok(SpecifiedOperation::PrefixedMatrix(Matrix { a, b, c, d, e, f }))
                        }
                    },
                    "matrix3d" => {
                        let m11 = specified::parse_number(context, input)?;
                        input.expect_comma()?;
                        let m12 = specified::parse_number(context, input)?;
                        input.expect_comma()?;
                        let m13 = specified::parse_number(context, input)?;
                        input.expect_comma()?;
                        let m14 = specified::parse_number(context, input)?;
                        input.expect_comma()?;
                        let m21 = specified::parse_number(context, input)?;
                        input.expect_comma()?;
                        let m22 = specified::parse_number(context, input)?;
                        input.expect_comma()?;
                        let m23 = specified::parse_number(context, input)?;
                        input.expect_comma()?;
                        let m24 = specified::parse_number(context, input)?;
                        input.expect_comma()?;
                        let m31 = specified::parse_number(context, input)?;
                        input.expect_comma()?;
                        let m32 = specified::parse_number(context, input)?;
                        input.expect_comma()?;
                        let m33 = specified::parse_number(context, input)?;
                        input.expect_comma()?;
                        let m34 = specified::parse_number(context, input)?;
                        input.expect_comma()?;
                        if !prefixed {
                            // Standard matrix3d parsing.
                            let m41 = specified::parse_number(context, input)?;
                            input.expect_comma()?;
                            let m42 = specified::parse_number(context, input)?;
                            input.expect_comma()?;
                            let m43 = specified::parse_number(context, input)?;
                            input.expect_comma()?;
                            let m44 = specified::parse_number(context, input)?;
                            Ok(SpecifiedOperation::Matrix3D {
                                m11, m12, m13, m14,
                                m21, m22, m23, m24,
                                m31, m32, m33, m34,
                                m41, m42, m43, m44,
                            })
                        } else {
                            // Non-standard prefixed matrix parsing for -moz-transform.
                            let m41 = LoPoNumber::parse(context, input)?;
                            input.expect_comma()?;
                            let m42 = LoPoNumber::parse(context, input)?;
                            input.expect_comma()?;
                            let m43 = LengthOrNumber::parse(context, input)?;
                            input.expect_comma()?;
                            let m44 = specified::parse_number(context, input)?;
                            Ok(SpecifiedOperation::PrefixedMatrix3D {
                                m11, m12, m13, m14,
                                m21, m22, m23, m24,
                                m31, m32, m33, m34,
                                m41, m42, m43, m44,
                            })
                        }
                    },
                    "translate" => {
                        let sx = specified::LengthOrPercentage::parse(context, input)?;
                        if input.try(|input| input.expect_comma()).is_ok() {
                            let sy = specified::LengthOrPercentage::parse(context, input)?;
                            Ok(SpecifiedOperation::Translate(sx, Some(sy)))
                        } else {
                            Ok(SpecifiedOperation::Translate(sx, None))
                        }
                    },
                    "translatex" => {
                        let tx = specified::LengthOrPercentage::parse(context, input)?;
                        Ok(SpecifiedOperation::TranslateX(tx))
                    },
                    "translatey" => {
                        let ty = specified::LengthOrPercentage::parse(context, input)?;
                        Ok(SpecifiedOperation::TranslateY(ty))
                    },
                    "translatez" => {
                        let tz = specified::Length::parse(context, input)?;
                        Ok(SpecifiedOperation::TranslateZ(tz))
                    },
                    "translate3d" => {
                        let tx = specified::LengthOrPercentage::parse(context, input)?;
                        input.expect_comma()?;
                        let ty = specified::LengthOrPercentage::parse(context, input)?;
                        input.expect_comma()?;
                        let tz = specified::Length::parse(context, input)?;
                        Ok(SpecifiedOperation::Translate3D(tx, ty, tz))
                    },
                    "scale" => {
                        let sx = specified::parse_number(context, input)?;
                        if input.try(|input| input.expect_comma()).is_ok() {
                            let sy = specified::parse_number(context, input)?;
                            Ok(SpecifiedOperation::Scale(sx, Some(sy)))
                        } else {
                            Ok(SpecifiedOperation::Scale(sx, None))
                        }
                    },
                    "scalex" => {
                        let sx = specified::parse_number(context, input)?;
                        Ok(SpecifiedOperation::ScaleX(sx))
                    },
                    "scaley" => {
                        let sy = specified::parse_number(context, input)?;
                        Ok(SpecifiedOperation::ScaleY(sy))
                    },
                    "scalez" => {
                        let sz = specified::parse_number(context, input)?;
                        Ok(SpecifiedOperation::ScaleZ(sz))
                    },
                    "scale3d" => {
                        let sx = specified::parse_number(context, input)?;
                        input.expect_comma()?;
                        let sy = specified::parse_number(context, input)?;
                        input.expect_comma()?;
                        let sz = specified::parse_number(context, input)?;
                        Ok(SpecifiedOperation::Scale3D(sx, sy, sz))
                    },
                    "rotate" => {
                        let theta = specified::Angle::parse_with_unitless(context, input)?;
                        Ok(SpecifiedOperation::Rotate(theta))
                    },
                    "rotatex" => {
                        let theta = specified::Angle::parse_with_unitless(context, input)?;
                        Ok(SpecifiedOperation::RotateX(theta))
                    },
                    "rotatey" => {
                        let theta = specified::Angle::parse_with_unitless(context, input)?;
                        Ok(SpecifiedOperation::RotateY(theta))
                    },
                    "rotatez" => {
                        let theta = specified::Angle::parse_with_unitless(context, input)?;
                        Ok(SpecifiedOperation::RotateZ(theta))
                    },
                    "rotate3d" => {
                        let ax = specified::parse_number(context, input)?;
                        input.expect_comma()?;
                        let ay = specified::parse_number(context, input)?;
                        input.expect_comma()?;
                        let az = specified::parse_number(context, input)?;
                        input.expect_comma()?;
                        let theta = specified::Angle::parse_with_unitless(context, input)?;
                        // TODO(gw): Check that the axis can be normalized.
                        Ok(SpecifiedOperation::Rotate3D(ax, ay, az, theta))
                    },
                    "skew" => {
                        let ax = specified::Angle::parse_with_unitless(context, input)?;
                        if input.try(|input| input.expect_comma()).is_ok() {
                            let ay = specified::Angle::parse_with_unitless(context, input)?;
                            Ok(SpecifiedOperation::Skew(ax, Some(ay)))
                        } else {
                            Ok(SpecifiedOperation::Skew(ax, None))
                        }
                    },
                    "skewx" => {
                        let theta = specified::Angle::parse_with_unitless(context, input)?;
                        Ok(SpecifiedOperation::SkewX(theta))
                    },
                    "skewy" => {
                        let theta = specified::Angle::parse_with_unitless(context, input)?;
                        Ok(SpecifiedOperation::SkewY(theta))
                    },
                    "perspective" => {
                        let d = specified::Length::parse_non_negative(context, input)?;
                        Ok(SpecifiedOperation::Perspective(d))
                    },
                    _ => Err(()),
                };
                result
                    .map_err(|()| StyleParseError::UnexpectedFunction(function.clone()).into())
            })
        })?))
    }

    /// Parses `transform` property.
    #[inline]
    pub fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                         -> Result<SpecifiedValue,ParseError<'i>> {
        parse_internal(context, input, false)
    }

    /// Parses `-moz-transform` property. This prefixed property also accepts LengthOrPercentage
    /// in the nondiagonal homogeneous components of matrix and matrix3d.
    #[inline]
    pub fn parse_prefixed<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                  -> Result<SpecifiedValue,ParseError<'i>> {
        parse_internal(context, input, true)
    }

    impl ToComputedValue for SpecifiedValue {
        type ComputedValue = computed_value::T;

        #[inline]
        fn to_computed_value(&self, context: &Context) -> computed_value::T {
            if self.0.is_empty() {
                return computed_value::T(None)
            }

            let mut result = vec!();
            for operation in &self.0 {
                match *operation {
                    SpecifiedOperation::Matrix(Matrix { a, b, c, d, e, f }) => {
                        let mut comp = computed_value::ComputedMatrix::identity();
                        comp.m11 = a.to_computed_value(context);
                        comp.m12 = b.to_computed_value(context);
                        comp.m21 = c.to_computed_value(context);
                        comp.m22 = d.to_computed_value(context);
                        comp.m41 = e.to_computed_value(context);
                        comp.m42 = f.to_computed_value(context);
                        result.push(computed_value::ComputedOperation::Matrix(comp));
                    }
                    SpecifiedOperation::PrefixedMatrix(Matrix { a, b, c, d, ref e, ref f }) => {
                        let mut comp = computed_value::ComputedMatrixWithPercents::identity();
                        comp.m11 = a.to_computed_value(context);
                        comp.m12 = b.to_computed_value(context);
                        comp.m21 = c.to_computed_value(context);
                        comp.m22 = d.to_computed_value(context);
                        comp.m41 = lopon_to_lop(&e.to_computed_value(context));
                        comp.m42 = lopon_to_lop(&f.to_computed_value(context));
                        result.push(computed_value::ComputedOperation::MatrixWithPercents(comp));
                    }
                    SpecifiedOperation::Matrix3D {
                        m11, m12, m13, m14,
                        m21, m22, m23, m24,
                        m31, m32, m33, m34,
                        ref m41, ref m42, ref m43, m44 } => {
                            let comp = computed_value::ComputedMatrix {
                                m11: m11.to_computed_value(context),
                                m12: m12.to_computed_value(context),
                                m13: m13.to_computed_value(context),
                                m14: m14.to_computed_value(context),
                                m21: m21.to_computed_value(context),
                                m22: m22.to_computed_value(context),
                                m23: m23.to_computed_value(context),
                                m24: m24.to_computed_value(context),
                                m31: m31.to_computed_value(context),
                                m32: m32.to_computed_value(context),
                                m33: m33.to_computed_value(context),
                                m34: m34.to_computed_value(context),
                                m41: m41.to_computed_value(context),
                                m42: m42.to_computed_value(context),
                                m43: m43.to_computed_value(context),
                                m44: m44.to_computed_value(context),
                            };
                        result.push(computed_value::ComputedOperation::Matrix(comp));
                    }
                    SpecifiedOperation::PrefixedMatrix3D {
                        m11, m12, m13, m14,
                        m21, m22, m23, m24,
                        m31, m32, m33, m34,
                        ref m41, ref m42, ref m43, m44 } => {
                            let comp = computed_value::ComputedMatrixWithPercents {
                                m11: m11.to_computed_value(context),
                                m12: m12.to_computed_value(context),
                                m13: m13.to_computed_value(context),
                                m14: m14.to_computed_value(context),
                                m21: m21.to_computed_value(context),
                                m22: m22.to_computed_value(context),
                                m23: m23.to_computed_value(context),
                                m24: m24.to_computed_value(context),
                                m31: m31.to_computed_value(context),
                                m32: m32.to_computed_value(context),
                                m33: m33.to_computed_value(context),
                                m34: m34.to_computed_value(context),
                                m41: lopon_to_lop(&m41.to_computed_value(context)),
                                m42: lopon_to_lop(&m42.to_computed_value(context)),
                                m43: lon_to_length(&m43.to_computed_value(context)),
                                m44: m44.to_computed_value(context),
                            };
                        result.push(computed_value::ComputedOperation::MatrixWithPercents(comp));
                    }
                    SpecifiedOperation::Translate(ref tx, None) => {
                        let tx = tx.to_computed_value(context);
                        result.push(computed_value::ComputedOperation::Translate(
                            tx,
                            computed::length::LengthOrPercentage::zero(),
                            computed::length::Length::new(0.)));
                    }
                    SpecifiedOperation::Translate(ref tx, Some(ref ty)) => {
                        let tx = tx.to_computed_value(context);
                        let ty = ty.to_computed_value(context);
                        result.push(computed_value::ComputedOperation::Translate(
                            tx,
                            ty,
                            computed::length::Length::new(0.)));
                    }
                    SpecifiedOperation::TranslateX(ref tx) => {
                        let tx = tx.to_computed_value(context);
                        result.push(computed_value::ComputedOperation::Translate(
                            tx,
                            computed::length::LengthOrPercentage::zero(),
                            computed::length::Length::new(0.)));
                    }
                    SpecifiedOperation::TranslateY(ref ty) => {
                        let ty = ty.to_computed_value(context);
                        result.push(computed_value::ComputedOperation::Translate(
                            computed::length::LengthOrPercentage::zero(),
                            ty,
                            computed::length::Length::new(0.)));
                    }
                    SpecifiedOperation::TranslateZ(ref tz) => {
                        let tz = tz.to_computed_value(context);
                        result.push(computed_value::ComputedOperation::Translate(
                            computed::length::LengthOrPercentage::zero(),
                            computed::length::LengthOrPercentage::zero(),
                            tz));
                    }
                    SpecifiedOperation::Translate3D(ref tx, ref ty, ref tz) => {
                        let tx = tx.to_computed_value(context);
                        let ty = ty.to_computed_value(context);
                        let tz = tz.to_computed_value(context);
                        result.push(computed_value::ComputedOperation::Translate(tx, ty, tz));
                    }
                    SpecifiedOperation::Scale(factor, None) => {
                        let factor = factor.to_computed_value(context);
                        result.push(computed_value::ComputedOperation::Scale(factor, factor, 1.0));
                    }
                    SpecifiedOperation::Scale(sx, Some(sy)) => {
                        let sx = sx.to_computed_value(context);
                        let sy = sy.to_computed_value(context);
                        result.push(computed_value::ComputedOperation::Scale(sx, sy, 1.0));
                    }
                    SpecifiedOperation::ScaleX(sx) => {
                        let sx = sx.to_computed_value(context);
                        result.push(computed_value::ComputedOperation::Scale(sx, 1.0, 1.0));
                    }
                    SpecifiedOperation::ScaleY(sy) => {
                        let sy = sy.to_computed_value(context);
                        result.push(computed_value::ComputedOperation::Scale(1.0, sy, 1.0));
                    }
                    SpecifiedOperation::ScaleZ(sz) => {
                        let sz = sz.to_computed_value(context);
                        result.push(computed_value::ComputedOperation::Scale(1.0, 1.0, sz));
                    }
                    SpecifiedOperation::Scale3D(sx, sy, sz) => {
                        let sx = sx.to_computed_value(context);
                        let sy = sy.to_computed_value(context);
                        let sz = sz.to_computed_value(context);
                        result.push(computed_value::ComputedOperation::Scale(sx, sy, sz));
                    }
                    SpecifiedOperation::Rotate(theta) => {
                        let theta = theta.to_computed_value(context);
                        result.push(computed_value::ComputedOperation::Rotate(0.0, 0.0, 1.0, theta));
                    }
                    SpecifiedOperation::RotateX(theta) => {
                        let theta = theta.to_computed_value(context);
                        result.push(computed_value::ComputedOperation::Rotate(1.0, 0.0, 0.0, theta));
                    }
                    SpecifiedOperation::RotateY(theta) => {
                        let theta = theta.to_computed_value(context);
                        result.push(computed_value::ComputedOperation::Rotate(0.0, 1.0, 0.0, theta));
                    }
                    SpecifiedOperation::RotateZ(theta) => {
                        let theta = theta.to_computed_value(context);
                        result.push(computed_value::ComputedOperation::Rotate(0.0, 0.0, 1.0, theta));
                    }
                    SpecifiedOperation::Rotate3D(ax, ay, az, theta) => {
                        let ax = ax.to_computed_value(context);
                        let ay = ay.to_computed_value(context);
                        let az = az.to_computed_value(context);
                        let theta = theta.to_computed_value(context);
                        result.push(computed_value::ComputedOperation::Rotate(ax, ay, az, theta));
                    }
                    SpecifiedOperation::Skew(theta_x, None) => {
                        let theta_x = theta_x.to_computed_value(context);
                        result.push(computed_value::ComputedOperation::Skew(theta_x, computed::Angle::zero()));
                    }
                    SpecifiedOperation::Skew(theta_x, Some(theta_y)) => {
                        let theta_x = theta_x.to_computed_value(context);
                        let theta_y = theta_y.to_computed_value(context);
                        result.push(computed_value::ComputedOperation::Skew(theta_x, theta_y));
                    }
                    SpecifiedOperation::SkewX(theta_x) => {
                        let theta_x = theta_x.to_computed_value(context);
                        result.push(computed_value::ComputedOperation::Skew(theta_x, computed::Angle::zero()));
                    }
                    SpecifiedOperation::SkewY(theta_y) => {
                        let theta_y = theta_y.to_computed_value(context);
                        result.push(computed_value::ComputedOperation::Skew(computed::Angle::zero(), theta_y));
                    }
                    SpecifiedOperation::Perspective(ref d) => {
                        result.push(computed_value::ComputedOperation::Perspective(d.to_computed_value(context)));
                    }
                    SpecifiedOperation::InterpolateMatrix { ref from_list, ref to_list, progress } => {
                        result.push(computed_value::ComputedOperation::InterpolateMatrix {
                            from_list: from_list.to_computed_value(context),
                            to_list: to_list.to_computed_value(context),
                            progress: progress
                        });
                    }
                    SpecifiedOperation::AccumulateMatrix { ref from_list, ref to_list, count } => {
                        result.push(computed_value::ComputedOperation::AccumulateMatrix {
                            from_list: from_list.to_computed_value(context),
                            to_list: to_list.to_computed_value(context),
                            count: count.value()
                        });
                    }
                };
            }

            computed_value::T(Some(result))
        }

        #[inline]
        fn from_computed_value(computed: &computed_value::T) -> Self {
            SpecifiedValue(computed.0.as_ref().map(|computed| {
                let mut result = vec![];
                for operation in computed {
                    match *operation {
                        computed_value::ComputedOperation::Matrix(ref computed) => {
                            result.push(SpecifiedOperation::Matrix3D {
                                m11: Number::from_computed_value(&computed.m11),
                                m12: Number::from_computed_value(&computed.m12),
                                m13: Number::from_computed_value(&computed.m13),
                                m14: Number::from_computed_value(&computed.m14),
                                m21: Number::from_computed_value(&computed.m21),
                                m22: Number::from_computed_value(&computed.m22),
                                m23: Number::from_computed_value(&computed.m23),
                                m24: Number::from_computed_value(&computed.m24),
                                m31: Number::from_computed_value(&computed.m31),
                                m32: Number::from_computed_value(&computed.m32),
                                m33: Number::from_computed_value(&computed.m33),
                                m34: Number::from_computed_value(&computed.m34),
                                m41: Number::from_computed_value(&computed.m41),
                                m42: Number::from_computed_value(&computed.m42),
                                m43: Number::from_computed_value(&computed.m43),
                                m44: Number::from_computed_value(&computed.m44),
                            });
                        }
                        computed_value::ComputedOperation::MatrixWithPercents(ref computed) => {
                            result.push(SpecifiedOperation::PrefixedMatrix3D {
                                m11: Number::from_computed_value(&computed.m11),
                                m12: Number::from_computed_value(&computed.m12),
                                m13: Number::from_computed_value(&computed.m13),
                                m14: Number::from_computed_value(&computed.m14),
                                m21: Number::from_computed_value(&computed.m21),
                                m22: Number::from_computed_value(&computed.m22),
                                m23: Number::from_computed_value(&computed.m23),
                                m24: Number::from_computed_value(&computed.m24),
                                m31: Number::from_computed_value(&computed.m31),
                                m32: Number::from_computed_value(&computed.m32),
                                m33: Number::from_computed_value(&computed.m33),
                                m34: Number::from_computed_value(&computed.m34),
                                m41: Either::Second(LengthOrPercentage::from_computed_value(&computed.m41)),
                                m42: Either::Second(LengthOrPercentage::from_computed_value(&computed.m42)),
                                m43: LengthOrNumber::from_computed_value(&Either::First(computed.m43)),
                                m44: Number::from_computed_value(&computed.m44),
                            });
                        }
                        computed_value::ComputedOperation::Translate(ref tx, ref ty, ref tz) => {
                            // XXXManishearth we lose information here; perhaps we should try to
                            // recover the original function? Not sure if this can be observed.
                            result.push(SpecifiedOperation::Translate3D(
                                              ToComputedValue::from_computed_value(tx),
                                              ToComputedValue::from_computed_value(ty),
                                              ToComputedValue::from_computed_value(tz)));
                        }
                        computed_value::ComputedOperation::Scale(ref sx, ref sy, ref sz) => {
                            result.push(SpecifiedOperation::Scale3D(
                                    Number::from_computed_value(sx),
                                    Number::from_computed_value(sy),
                                    Number::from_computed_value(sz)));
                        }
                        computed_value::ComputedOperation::Rotate(ref ax, ref ay, ref az, ref theta) => {
                            result.push(SpecifiedOperation::Rotate3D(
                                    Number::from_computed_value(ax),
                                    Number::from_computed_value(ay),
                                    Number::from_computed_value(az),
                                    specified::Angle::from_computed_value(theta)));
                        }
                        computed_value::ComputedOperation::Skew(ref theta_x, ref theta_y) => {
                            result.push(SpecifiedOperation::Skew(
                                    specified::Angle::from_computed_value(theta_x),
                                    Some(specified::Angle::from_computed_value(theta_y))))
                        }
                        computed_value::ComputedOperation::Perspective(ref d) => {
                            result.push(SpecifiedOperation::Perspective(
                                ToComputedValue::from_computed_value(d)
                            ));
                        }
                        computed_value::ComputedOperation::InterpolateMatrix { ref from_list,
                                                                               ref to_list,
                                                                               progress } => {
                            result.push(SpecifiedOperation::InterpolateMatrix {
                                from_list: SpecifiedValue::from_computed_value(from_list),
                                to_list: SpecifiedValue::from_computed_value(to_list),
                                progress: progress
                            });
                        }
                        computed_value::ComputedOperation::AccumulateMatrix { ref from_list,
                                                                              ref to_list,
                                                                              count } => {
                            result.push(SpecifiedOperation::AccumulateMatrix {
                                from_list: SpecifiedValue::from_computed_value(from_list),
                                to_list: SpecifiedValue::from_computed_value(to_list),
                                count: Integer::new(count)
                            });
                        }
                    };
                }
                result
            }).unwrap_or(Vec::new()))
        }
    }

    // Converts computed LengthOrPercentageOrNumber into computed
    // LengthOrPercentage. Number maps into Length (pixel unit)
    fn lopon_to_lop(value: &ComputedLoPoNumber) -> ComputedLoP {
        match *value {
            Either::First(number) => ComputedLoP::Length(ComputedLength::new(number)),
            Either::Second(length_or_percentage) => length_or_percentage,
        }
    }

    // Converts computed LengthOrNumber into computed Length.
    // Number maps into Length.
    fn lon_to_length(value: &ComputedLoN) -> ComputedLength {
        match *value {
            Either::First(length) => length,
            Either::Second(number) => ComputedLength::new(number),
        }
    }

        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::Transform(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::Transform);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::Transform);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_transform(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_transform();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_transform();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::Transform)
            }
    }


// CSSOM View Module
// https://www.w3.org/TR/cssom-view-1/

    
    

    
        
    
    /// https://drafts.csswg.org/cssom-view/#propdef-scroll-behavior
    pub mod scroll_behavior {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "auto" => auto,
                    "smooth" => smooth,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::auto
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::auto
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::auto => computed_value::T::auto,
                        SpecifiedValue::smooth => computed_value::T::smooth,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::auto => SpecifiedValue::auto,
                        computed_value::T::smooth => SpecifiedValue::smooth,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::ScrollBehavior(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::ScrollBehavior);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::ScrollBehavior);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_scroll_behavior(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_scroll_behavior();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_scroll_behavior();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::ScrollBehavior)
            }
    }





    
    

    
        
    
    /// Nonstandard (https://developer.mozilla.org/en-US/docs/Web/CSS/scroll-snap-type-x)
    pub mod scroll_snap_type_x {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "none" => none,
                    "mandatory" => mandatory,
                    "proximity" => proximity,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::none
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::none
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::none => computed_value::T::none,
                        SpecifiedValue::mandatory => computed_value::T::mandatory,
                        SpecifiedValue::proximity => computed_value::T::proximity,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::none => SpecifiedValue::none,
                        computed_value::T::mandatory => SpecifiedValue::mandatory,
                        computed_value::T::proximity => SpecifiedValue::proximity,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::ScrollSnapTypeX(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::ScrollSnapTypeX);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::ScrollSnapTypeX);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_scroll_snap_type_x(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_scroll_snap_type_x();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_scroll_snap_type_x();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::ScrollSnapTypeX)
            }
    }





    
    /// Nonstandard (https://developer.mozilla.org/en-US/docs/Web/CSS/scroll-snap-type-x)
    pub mod scroll_snap_type_y {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
    pub use super::scroll_snap_type_x::SpecifiedValue;
    pub use super::scroll_snap_type_x::computed_value;
    pub use super::scroll_snap_type_x::get_initial_value;
    pub use super::scroll_snap_type_x::parse;

        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::ScrollSnapTypeY(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::ScrollSnapTypeY);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::ScrollSnapTypeY);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_scroll_snap_type_y(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_scroll_snap_type_y();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_scroll_snap_type_y();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::ScrollSnapTypeY)
            }
    }


// Compositing and Blending Level 1
// http://www.w3.org/TR/compositing-1/

    
    

    
        
    
    /// https://drafts.fxtf.org/compositing/#isolation
    pub mod isolation {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "auto" => auto,
                    "isolate" => isolate,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::auto
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::auto
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::auto => computed_value::T::auto,
                        SpecifiedValue::isolate => computed_value::T::isolate,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::auto => SpecifiedValue::auto,
                        computed_value::T::isolate => SpecifiedValue::isolate,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::Isolation(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::Isolation);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::Isolation);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_isolation(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_isolation();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_isolation();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::Isolation)
            }
    }




// TODO add support for logical values recto and verso

    
    

    
        
    
    /// https://drafts.csswg.org/css2/page.html#propdef-page-break-after
    pub mod page_break_after {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "auto" => auto,
                    "always" => always,
                    "avoid" => avoid,
                    "left" => left,
                    "right" => right,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::auto
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::auto
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::auto => computed_value::T::auto,
                        SpecifiedValue::always => computed_value::T::always,
                        SpecifiedValue::avoid => computed_value::T::avoid,
                        SpecifiedValue::left => computed_value::T::left,
                        SpecifiedValue::right => computed_value::T::right,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::auto => SpecifiedValue::auto,
                        computed_value::T::always => SpecifiedValue::always,
                        computed_value::T::avoid => SpecifiedValue::avoid,
                        computed_value::T::left => SpecifiedValue::left,
                        computed_value::T::right => SpecifiedValue::right,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::PageBreakAfter(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::PageBreakAfter);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::PageBreakAfter);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_page_break_after(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_page_break_after();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_page_break_after();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::PageBreakAfter)
            }
    }




    
    

    
        
    
    /// https://drafts.csswg.org/css2/page.html#propdef-page-break-before
    pub mod page_break_before {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "auto" => auto,
                    "always" => always,
                    "avoid" => avoid,
                    "left" => left,
                    "right" => right,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::auto
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::auto
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::auto => computed_value::T::auto,
                        SpecifiedValue::always => computed_value::T::always,
                        SpecifiedValue::avoid => computed_value::T::avoid,
                        SpecifiedValue::left => computed_value::T::left,
                        SpecifiedValue::right => computed_value::T::right,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::auto => SpecifiedValue::auto,
                        computed_value::T::always => SpecifiedValue::always,
                        computed_value::T::avoid => SpecifiedValue::avoid,
                        computed_value::T::left => SpecifiedValue::left,
                        computed_value::T::right => SpecifiedValue::right,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::PageBreakBefore(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::PageBreakBefore);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::PageBreakBefore);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_page_break_before(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_page_break_before();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_page_break_before();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::PageBreakBefore)
            }
    }




    
    

    
        
    
    /// https://drafts.csswg.org/css2/page.html#propdef-page-break-inside
    pub mod page_break_inside {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "auto" => auto,
                    "avoid" => avoid,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::auto
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::auto
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::auto => computed_value::T::auto,
                        SpecifiedValue::avoid => computed_value::T::avoid,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::auto => SpecifiedValue::auto,
                        computed_value::T::avoid => SpecifiedValue::avoid,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::PageBreakInside(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::PageBreakInside);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::PageBreakInside);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_page_break_inside(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_page_break_inside();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_page_break_inside();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::PageBreakInside)
            }
    }




// CSS Basic User Interface Module Level 3
// http://dev.w3.org/csswg/css-ui
// FIXME support logical values `block` and `inline` (https://drafts.csswg.org/css-logical-props/#resize)
//
// This is APPLIES_TO_PLACEHOLDER so we can override, in the UA sheet, the
// 'resize' property we'd inherit from textarea otherwise.  Basically, just
// makes the UA rules easier to write.

    
    

    
        
    
    /// https://drafts.csswg.org/css-ui/#propdef-resize
    pub mod resize {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "none" => none,
                    "both" => both,
                    "horizontal" => horizontal,
                    "vertical" => vertical,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::none
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::none
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::none => computed_value::T::none,
                        SpecifiedValue::both => computed_value::T::both,
                        SpecifiedValue::horizontal => computed_value::T::horizontal,
                        SpecifiedValue::vertical => computed_value::T::vertical,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::none => SpecifiedValue::none,
                        computed_value::T::both => SpecifiedValue::both,
                        computed_value::T::horizontal => SpecifiedValue::horizontal,
                        computed_value::T::vertical => SpecifiedValue::vertical,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::Resize(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::Resize);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::Resize);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_resize(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_resize();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_resize();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::Resize)
            }
    }






    
        
    
    /// https://drafts.csswg.org/css-transforms/#perspective
    pub mod perspective {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::LengthOrNone as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::LengthOrNone as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { Either::Second(None_) }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::LengthOrNone::parse_non_negative_length(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::Perspective(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::Perspective);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::Perspective);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_perspective(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_perspective();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_perspective();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::Perspective)
            }
    }




    
        
    
    /// https://drafts.csswg.org/css-transforms-2/#perspective-origin-property
    pub mod perspective_origin {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::position::Position as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::position::Position as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::position::Position::center() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::position::Position::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::PerspectiveOrigin(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::PerspectiveOrigin);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::PerspectiveOrigin);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = (**specified_value).to_computed_value(context);
                                context.builder.set_perspective_origin(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_perspective_origin();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_perspective_origin();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<Box<SpecifiedValue>, ParseError<'i>> {
                    parse(context, input).map(|result| Box::new(result))
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::PerspectiveOrigin)
            }
    }




    
    

    
        
    
    /// https://drafts.csswg.org/css-transforms/#backface-visibility-property
    pub mod backface_visibility {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "visible" => visible,
                    "hidden" => hidden,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::visible
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::visible
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::visible => computed_value::T::visible,
                        SpecifiedValue::hidden => computed_value::T::hidden,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::visible => SpecifiedValue::visible,
                        computed_value::T::hidden => SpecifiedValue::hidden,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::BackfaceVisibility(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::BackfaceVisibility);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::BackfaceVisibility);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_backface_visibility(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_backface_visibility();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_backface_visibility();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::BackfaceVisibility)
            }
    }





    
    

    
        
    
    /// https://drafts.csswg.org/css-transforms/#transform-box
    pub mod transform_box {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "border-box" => border_box,
                    "fill-box" => fill_box,
                    "view-box" => view_box,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::border_box
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::border_box
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::border_box => computed_value::T::border_box,
                        SpecifiedValue::fill_box => computed_value::T::fill_box,
                        SpecifiedValue::view_box => computed_value::T::view_box,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::border_box => SpecifiedValue::border_box,
                        computed_value::T::fill_box => SpecifiedValue::fill_box,
                        computed_value::T::view_box => SpecifiedValue::view_box,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::TransformBox(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::TransformBox);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::TransformBox);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_transform_box(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_transform_box();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_transform_box();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::TransformBox)
            }
    }




// `auto` keyword is not supported in gecko yet.

    
    

    
        
    
    /// https://drafts.csswg.org/css-transforms/#transform-style-property
    pub mod transform_style {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "flat" => flat,
                    "preserve-3d" => preserve_3d,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::flat
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::flat
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::flat => computed_value::T::flat,
                        SpecifiedValue::preserve_3d => computed_value::T::preserve_3d,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::flat => SpecifiedValue::flat,
                        computed_value::T::preserve_3d => SpecifiedValue::preserve_3d,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::TransformStyle(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::TransformStyle);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::TransformStyle);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_transform_style(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_transform_style();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_transform_style();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::TransformStyle)
            }
    }





    
        
    
    /// https://drafts.csswg.org/css-transforms/#transform-origin-property
    pub mod transform_origin {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::TransformOrigin as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::TransformOrigin as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::TransformOrigin::initial_value() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::TransformOrigin::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::TransformOrigin(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::TransformOrigin);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::TransformOrigin);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = (**specified_value).to_computed_value(context);
                                context.builder.set_transform_origin(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_transform_origin();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_transform_origin();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<Box<SpecifiedValue>, ParseError<'i>> {
                    parse(context, input).map(|result| Box::new(result))
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::TransformOrigin)
            }
    }



// FIXME: `size` and `content` values are not implemented and `strict` is implemented
// like `content`(layout style paint) in gecko. We should implement `size` and `content`,
// also update the glue once they are implemented in gecko.

    
    /// https://drafts.csswg.org/css-contain/#contain-property
    pub mod contain {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
    use std::fmt;
    use style_traits::ToCss;

    pub mod computed_value {
        pub type T = super::SpecifiedValue;
    }

    bitflags! {
        #[derive(ToComputedValue)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        pub flags SpecifiedValue: u8 {
            const LAYOUT = 0x01,
            const STYLE = 0x02,
            const PAINT = 0x04,
            const STRICT = 0x8,
            const STRICT_BITS = LAYOUT.bits | STYLE.bits | PAINT.bits,
        }
    }

    impl ToCss for SpecifiedValue {
        fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
            if self.is_empty() {
                return dest.write_str("none")
            }
            if self.contains(STRICT) {
                return dest.write_str("strict")
            }

            let mut has_any = false;
            macro_rules! maybe_write_value {
                ($ident:ident => $str:expr) => {
                    if self.contains($ident) {
                        if has_any {
                            dest.write_str(" ")?;
                        }
                        has_any = true;
                        dest.write_str($str)?;
                    }
                }
            }
            maybe_write_value!(LAYOUT => "layout");
            maybe_write_value!(STYLE => "style");
            maybe_write_value!(PAINT => "paint");

            debug_assert!(has_any);
            Ok(())
        }
    }

    #[inline]
    pub fn get_initial_value() -> computed_value::T {
        computed_value::T::empty()
    }

    /// none | strict | content | [ size || layout || style || paint ]
    pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                         -> Result<SpecifiedValue, ParseError<'i>> {
        let mut result = SpecifiedValue::empty();

        if input.try(|input| input.expect_ident_matching("none")).is_ok() {
            return Ok(result)
        }
        if input.try(|input| input.expect_ident_matching("strict")).is_ok() {
            result.insert(STRICT | STRICT_BITS);
            return Ok(result)
        }

        while let Ok(name) = input.try(|i| i.expect_ident_cloned()) {
            let flag = match_ignore_ascii_case! { &name,
                "layout" => Some(LAYOUT),
                "style" => Some(STYLE),
                "paint" => Some(PAINT),
                _ => None
            };
            let flag = match flag {
                Some(flag) if !result.contains(flag) => flag,
                _ => return Err(SelectorParseError::UnexpectedIdent(name.clone()).into())
            };
            result.insert(flag);
        }

        if !result.is_empty() {
            Ok(result)
        } else {
            Err(StyleParseError::UnspecifiedError.into())
        }
    }

        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::Contain(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::Contain);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::Contain);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_contain(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_contain();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_contain();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::Contain)
            }
    }


// Non-standard

    
    

    
        
    
    /// Nonstandard (https://developer.mozilla.org/en-US/docs/Web/CSS/-moz-appearance)
    pub mod _moz_appearance {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "none" => none,
                    "button" => button,
                    "button-arrow-down" => button_arrow_down,
                    "button-arrow-next" => button_arrow_next,
                    "button-arrow-previous" => button_arrow_previous,
                    "button-arrow-up" => button_arrow_up,
                    "button-bevel" => button_bevel,
                    "button-focus" => button_focus,
                    "caret" => caret,
                    "checkbox" => checkbox,
                    "checkbox-container" => checkbox_container,
                    "checkbox-label" => checkbox_label,
                    "checkmenuitem" => checkmenuitem,
                    "dialog" => dialog,
                    "dualbutton" => dualbutton,
                    "groupbox" => groupbox,
                    "listbox" => listbox,
                    "listitem" => listitem,
                    "menuarrow" => menuarrow,
                    "menubar" => menubar,
                    "menucheckbox" => menucheckbox,
                    "menuimage" => menuimage,
                    "menuitem" => menuitem,
                    "menuitemtext" => menuitemtext,
                    "menulist" => menulist,
                    "menulist-button" => menulist_button,
                    "menulist-text" => menulist_text,
                    "menulist-textfield" => menulist_textfield,
                    "menupopup" => menupopup,
                    "menuradio" => menuradio,
                    "menuseparator" => menuseparator,
                    "meterbar" => meterbar,
                    "meterchunk" => meterchunk,
                    "number-input" => number_input,
                    "progressbar" => progressbar,
                    "progressbar-vertical" => progressbar_vertical,
                    "progresschunk" => progresschunk,
                    "progresschunk-vertical" => progresschunk_vertical,
                    "radio" => radio,
                    "radio-container" => radio_container,
                    "radio-label" => radio_label,
                    "radiomenuitem" => radiomenuitem,
                    "range" => range,
                    "range-thumb" => range_thumb,
                    "resizer" => resizer,
                    "resizerpanel" => resizerpanel,
                    "scale-horizontal" => scale_horizontal,
                    "scalethumbend" => scalethumbend,
                    "scalethumb-horizontal" => scalethumb_horizontal,
                    "scalethumbstart" => scalethumbstart,
                    "scalethumbtick" => scalethumbtick,
                    "scalethumb-vertical" => scalethumb_vertical,
                    "scale-vertical" => scale_vertical,
                    "scrollbar" => scrollbar,
                    "scrollbar-horizontal" => scrollbar_horizontal,
                    "scrollbar-small" => scrollbar_small,
                    "scrollbar-vertical" => scrollbar_vertical,
                    "scrollbarbutton-down" => scrollbarbutton_down,
                    "scrollbarbutton-left" => scrollbarbutton_left,
                    "scrollbarbutton-right" => scrollbarbutton_right,
                    "scrollbarbutton-up" => scrollbarbutton_up,
                    "scrollbarthumb-horizontal" => scrollbarthumb_horizontal,
                    "scrollbarthumb-vertical" => scrollbarthumb_vertical,
                    "scrollbartrack-horizontal" => scrollbartrack_horizontal,
                    "scrollbartrack-vertical" => scrollbartrack_vertical,
                    "searchfield" => searchfield,
                    "separator" => separator,
                    "spinner" => spinner,
                    "spinner-downbutton" => spinner_downbutton,
                    "spinner-textfield" => spinner_textfield,
                    "spinner-upbutton" => spinner_upbutton,
                    "splitter" => splitter,
                    "statusbar" => statusbar,
                    "statusbarpanel" => statusbarpanel,
                    "tab" => tab,
                    "tabpanel" => tabpanel,
                    "tabpanels" => tabpanels,
                    "tab-scroll-arrow-back" => tab_scroll_arrow_back,
                    "tab-scroll-arrow-forward" => tab_scroll_arrow_forward,
                    "textfield" => textfield,
                    "textfield-multiline" => textfield_multiline,
                    "toolbar" => toolbar,
                    "toolbarbutton" => toolbarbutton,
                    "toolbarbutton-dropdown" => toolbarbutton_dropdown,
                    "toolbargripper" => toolbargripper,
                    "toolbox" => toolbox,
                    "tooltip" => tooltip,
                    "treeheader" => treeheader,
                    "treeheadercell" => treeheadercell,
                    "treeheadersortarrow" => treeheadersortarrow,
                    "treeitem" => treeitem,
                    "treeline" => treeline,
                    "treetwisty" => treetwisty,
                    "treetwistyopen" => treetwistyopen,
                    "treeview" => treeview,
                    "window" => window,
                    "-moz-gtk-info-bar" => _moz_gtk_info_bar,
                    "-moz-mac-active-source-list-selection" => _moz_mac_active_source_list_selection,
                    "-moz-mac-disclosure-button-closed" => _moz_mac_disclosure_button_closed,
                    "-moz-mac-disclosure-button-open" => _moz_mac_disclosure_button_open,
                    "-moz-mac-fullscreen-button" => _moz_mac_fullscreen_button,
                    "-moz-mac-help-button" => _moz_mac_help_button,
                    "-moz-mac-source-list" => _moz_mac_source_list,
                    "-moz-mac-source-list-selection" => _moz_mac_source_list_selection,
                    "-moz-mac-vibrancy-dark" => _moz_mac_vibrancy_dark,
                    "-moz-mac-vibrancy-light" => _moz_mac_vibrancy_light,
                    "-moz-win-borderless-glass" => _moz_win_borderless_glass,
                    "-moz-win-browsertabbar-toolbox" => _moz_win_browsertabbar_toolbox,
                    "-moz-win-communications-toolbox" => _moz_win_communications_toolbox,
                    "-moz-win-exclude-glass" => _moz_win_exclude_glass,
                    "-moz-win-glass" => _moz_win_glass,
                    "-moz-win-media-toolbox" => _moz_win_media_toolbox,
                    "-moz-window-button-box" => _moz_window_button_box,
                    "-moz-window-button-box-maximized" => _moz_window_button_box_maximized,
                    "-moz-window-button-close" => _moz_window_button_close,
                    "-moz-window-button-maximize" => _moz_window_button_maximize,
                    "-moz-window-button-minimize" => _moz_window_button_minimize,
                    "-moz-window-button-restore" => _moz_window_button_restore,
                    "-moz-window-frame-bottom" => _moz_window_frame_bottom,
                    "-moz-window-frame-left" => _moz_window_frame_left,
                    "-moz-window-frame-right" => _moz_window_frame_right,
                    "-moz-window-titlebar" => _moz_window_titlebar,
                    "-moz-window-titlebar-maximized" => _moz_window_titlebar_maximized,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::none
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::none
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::none => computed_value::T::none,
                        SpecifiedValue::button => computed_value::T::button,
                        SpecifiedValue::button_arrow_down => computed_value::T::button_arrow_down,
                        SpecifiedValue::button_arrow_next => computed_value::T::button_arrow_next,
                        SpecifiedValue::button_arrow_previous => computed_value::T::button_arrow_previous,
                        SpecifiedValue::button_arrow_up => computed_value::T::button_arrow_up,
                        SpecifiedValue::button_bevel => computed_value::T::button_bevel,
                        SpecifiedValue::button_focus => computed_value::T::button_focus,
                        SpecifiedValue::caret => computed_value::T::caret,
                        SpecifiedValue::checkbox => computed_value::T::checkbox,
                        SpecifiedValue::checkbox_container => computed_value::T::checkbox_container,
                        SpecifiedValue::checkbox_label => computed_value::T::checkbox_label,
                        SpecifiedValue::checkmenuitem => computed_value::T::checkmenuitem,
                        SpecifiedValue::dialog => computed_value::T::dialog,
                        SpecifiedValue::dualbutton => computed_value::T::dualbutton,
                        SpecifiedValue::groupbox => computed_value::T::groupbox,
                        SpecifiedValue::listbox => computed_value::T::listbox,
                        SpecifiedValue::listitem => computed_value::T::listitem,
                        SpecifiedValue::menuarrow => computed_value::T::menuarrow,
                        SpecifiedValue::menubar => computed_value::T::menubar,
                        SpecifiedValue::menucheckbox => computed_value::T::menucheckbox,
                        SpecifiedValue::menuimage => computed_value::T::menuimage,
                        SpecifiedValue::menuitem => computed_value::T::menuitem,
                        SpecifiedValue::menuitemtext => computed_value::T::menuitemtext,
                        SpecifiedValue::menulist => computed_value::T::menulist,
                        SpecifiedValue::menulist_button => computed_value::T::menulist_button,
                        SpecifiedValue::menulist_text => computed_value::T::menulist_text,
                        SpecifiedValue::menulist_textfield => computed_value::T::menulist_textfield,
                        SpecifiedValue::menupopup => computed_value::T::menupopup,
                        SpecifiedValue::menuradio => computed_value::T::menuradio,
                        SpecifiedValue::menuseparator => computed_value::T::menuseparator,
                        SpecifiedValue::meterbar => computed_value::T::meterbar,
                        SpecifiedValue::meterchunk => computed_value::T::meterchunk,
                        SpecifiedValue::number_input => computed_value::T::number_input,
                        SpecifiedValue::progressbar => computed_value::T::progressbar,
                        SpecifiedValue::progressbar_vertical => computed_value::T::progressbar_vertical,
                        SpecifiedValue::progresschunk => computed_value::T::progresschunk,
                        SpecifiedValue::progresschunk_vertical => computed_value::T::progresschunk_vertical,
                        SpecifiedValue::radio => computed_value::T::radio,
                        SpecifiedValue::radio_container => computed_value::T::radio_container,
                        SpecifiedValue::radio_label => computed_value::T::radio_label,
                        SpecifiedValue::radiomenuitem => computed_value::T::radiomenuitem,
                        SpecifiedValue::range => computed_value::T::range,
                        SpecifiedValue::range_thumb => computed_value::T::range_thumb,
                        SpecifiedValue::resizer => computed_value::T::resizer,
                        SpecifiedValue::resizerpanel => computed_value::T::resizerpanel,
                        SpecifiedValue::scale_horizontal => computed_value::T::scale_horizontal,
                        SpecifiedValue::scalethumbend => computed_value::T::scalethumbend,
                        SpecifiedValue::scalethumb_horizontal => computed_value::T::scalethumb_horizontal,
                        SpecifiedValue::scalethumbstart => computed_value::T::scalethumbstart,
                        SpecifiedValue::scalethumbtick => computed_value::T::scalethumbtick,
                        SpecifiedValue::scalethumb_vertical => computed_value::T::scalethumb_vertical,
                        SpecifiedValue::scale_vertical => computed_value::T::scale_vertical,
                        SpecifiedValue::scrollbar => computed_value::T::scrollbar,
                        SpecifiedValue::scrollbar_horizontal => computed_value::T::scrollbar_horizontal,
                        SpecifiedValue::scrollbar_small => computed_value::T::scrollbar_small,
                        SpecifiedValue::scrollbar_vertical => computed_value::T::scrollbar_vertical,
                        SpecifiedValue::scrollbarbutton_down => computed_value::T::scrollbarbutton_down,
                        SpecifiedValue::scrollbarbutton_left => computed_value::T::scrollbarbutton_left,
                        SpecifiedValue::scrollbarbutton_right => computed_value::T::scrollbarbutton_right,
                        SpecifiedValue::scrollbarbutton_up => computed_value::T::scrollbarbutton_up,
                        SpecifiedValue::scrollbarthumb_horizontal => computed_value::T::scrollbarthumb_horizontal,
                        SpecifiedValue::scrollbarthumb_vertical => computed_value::T::scrollbarthumb_vertical,
                        SpecifiedValue::scrollbartrack_horizontal => computed_value::T::scrollbartrack_horizontal,
                        SpecifiedValue::scrollbartrack_vertical => computed_value::T::scrollbartrack_vertical,
                        SpecifiedValue::searchfield => computed_value::T::searchfield,
                        SpecifiedValue::separator => computed_value::T::separator,
                        SpecifiedValue::spinner => computed_value::T::spinner,
                        SpecifiedValue::spinner_downbutton => computed_value::T::spinner_downbutton,
                        SpecifiedValue::spinner_textfield => computed_value::T::spinner_textfield,
                        SpecifiedValue::spinner_upbutton => computed_value::T::spinner_upbutton,
                        SpecifiedValue::splitter => computed_value::T::splitter,
                        SpecifiedValue::statusbar => computed_value::T::statusbar,
                        SpecifiedValue::statusbarpanel => computed_value::T::statusbarpanel,
                        SpecifiedValue::tab => computed_value::T::tab,
                        SpecifiedValue::tabpanel => computed_value::T::tabpanel,
                        SpecifiedValue::tabpanels => computed_value::T::tabpanels,
                        SpecifiedValue::tab_scroll_arrow_back => computed_value::T::tab_scroll_arrow_back,
                        SpecifiedValue::tab_scroll_arrow_forward => computed_value::T::tab_scroll_arrow_forward,
                        SpecifiedValue::textfield => computed_value::T::textfield,
                        SpecifiedValue::textfield_multiline => computed_value::T::textfield_multiline,
                        SpecifiedValue::toolbar => computed_value::T::toolbar,
                        SpecifiedValue::toolbarbutton => computed_value::T::toolbarbutton,
                        SpecifiedValue::toolbarbutton_dropdown => computed_value::T::toolbarbutton_dropdown,
                        SpecifiedValue::toolbargripper => computed_value::T::toolbargripper,
                        SpecifiedValue::toolbox => computed_value::T::toolbox,
                        SpecifiedValue::tooltip => computed_value::T::tooltip,
                        SpecifiedValue::treeheader => computed_value::T::treeheader,
                        SpecifiedValue::treeheadercell => computed_value::T::treeheadercell,
                        SpecifiedValue::treeheadersortarrow => computed_value::T::treeheadersortarrow,
                        SpecifiedValue::treeitem => computed_value::T::treeitem,
                        SpecifiedValue::treeline => computed_value::T::treeline,
                        SpecifiedValue::treetwisty => computed_value::T::treetwisty,
                        SpecifiedValue::treetwistyopen => computed_value::T::treetwistyopen,
                        SpecifiedValue::treeview => computed_value::T::treeview,
                        SpecifiedValue::window => computed_value::T::window,
                        SpecifiedValue::_moz_gtk_info_bar => computed_value::T::_moz_gtk_info_bar,
                        SpecifiedValue::_moz_mac_active_source_list_selection => computed_value::T::_moz_mac_active_source_list_selection,
                        SpecifiedValue::_moz_mac_disclosure_button_closed => computed_value::T::_moz_mac_disclosure_button_closed,
                        SpecifiedValue::_moz_mac_disclosure_button_open => computed_value::T::_moz_mac_disclosure_button_open,
                        SpecifiedValue::_moz_mac_fullscreen_button => computed_value::T::_moz_mac_fullscreen_button,
                        SpecifiedValue::_moz_mac_help_button => computed_value::T::_moz_mac_help_button,
                        SpecifiedValue::_moz_mac_source_list => computed_value::T::_moz_mac_source_list,
                        SpecifiedValue::_moz_mac_source_list_selection => computed_value::T::_moz_mac_source_list_selection,
                        SpecifiedValue::_moz_mac_vibrancy_dark => computed_value::T::_moz_mac_vibrancy_dark,
                        SpecifiedValue::_moz_mac_vibrancy_light => computed_value::T::_moz_mac_vibrancy_light,
                        SpecifiedValue::_moz_win_borderless_glass => computed_value::T::_moz_win_borderless_glass,
                        SpecifiedValue::_moz_win_browsertabbar_toolbox => computed_value::T::_moz_win_browsertabbar_toolbox,
                        SpecifiedValue::_moz_win_communications_toolbox => computed_value::T::_moz_win_communications_toolbox,
                        SpecifiedValue::_moz_win_exclude_glass => computed_value::T::_moz_win_exclude_glass,
                        SpecifiedValue::_moz_win_glass => computed_value::T::_moz_win_glass,
                        SpecifiedValue::_moz_win_media_toolbox => computed_value::T::_moz_win_media_toolbox,
                        SpecifiedValue::_moz_window_button_box => computed_value::T::_moz_window_button_box,
                        SpecifiedValue::_moz_window_button_box_maximized => computed_value::T::_moz_window_button_box_maximized,
                        SpecifiedValue::_moz_window_button_close => computed_value::T::_moz_window_button_close,
                        SpecifiedValue::_moz_window_button_maximize => computed_value::T::_moz_window_button_maximize,
                        SpecifiedValue::_moz_window_button_minimize => computed_value::T::_moz_window_button_minimize,
                        SpecifiedValue::_moz_window_button_restore => computed_value::T::_moz_window_button_restore,
                        SpecifiedValue::_moz_window_frame_bottom => computed_value::T::_moz_window_frame_bottom,
                        SpecifiedValue::_moz_window_frame_left => computed_value::T::_moz_window_frame_left,
                        SpecifiedValue::_moz_window_frame_right => computed_value::T::_moz_window_frame_right,
                        SpecifiedValue::_moz_window_titlebar => computed_value::T::_moz_window_titlebar,
                        SpecifiedValue::_moz_window_titlebar_maximized => computed_value::T::_moz_window_titlebar_maximized,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::none => SpecifiedValue::none,
                        computed_value::T::button => SpecifiedValue::button,
                        computed_value::T::button_arrow_down => SpecifiedValue::button_arrow_down,
                        computed_value::T::button_arrow_next => SpecifiedValue::button_arrow_next,
                        computed_value::T::button_arrow_previous => SpecifiedValue::button_arrow_previous,
                        computed_value::T::button_arrow_up => SpecifiedValue::button_arrow_up,
                        computed_value::T::button_bevel => SpecifiedValue::button_bevel,
                        computed_value::T::button_focus => SpecifiedValue::button_focus,
                        computed_value::T::caret => SpecifiedValue::caret,
                        computed_value::T::checkbox => SpecifiedValue::checkbox,
                        computed_value::T::checkbox_container => SpecifiedValue::checkbox_container,
                        computed_value::T::checkbox_label => SpecifiedValue::checkbox_label,
                        computed_value::T::checkmenuitem => SpecifiedValue::checkmenuitem,
                        computed_value::T::dialog => SpecifiedValue::dialog,
                        computed_value::T::dualbutton => SpecifiedValue::dualbutton,
                        computed_value::T::groupbox => SpecifiedValue::groupbox,
                        computed_value::T::listbox => SpecifiedValue::listbox,
                        computed_value::T::listitem => SpecifiedValue::listitem,
                        computed_value::T::menuarrow => SpecifiedValue::menuarrow,
                        computed_value::T::menubar => SpecifiedValue::menubar,
                        computed_value::T::menucheckbox => SpecifiedValue::menucheckbox,
                        computed_value::T::menuimage => SpecifiedValue::menuimage,
                        computed_value::T::menuitem => SpecifiedValue::menuitem,
                        computed_value::T::menuitemtext => SpecifiedValue::menuitemtext,
                        computed_value::T::menulist => SpecifiedValue::menulist,
                        computed_value::T::menulist_button => SpecifiedValue::menulist_button,
                        computed_value::T::menulist_text => SpecifiedValue::menulist_text,
                        computed_value::T::menulist_textfield => SpecifiedValue::menulist_textfield,
                        computed_value::T::menupopup => SpecifiedValue::menupopup,
                        computed_value::T::menuradio => SpecifiedValue::menuradio,
                        computed_value::T::menuseparator => SpecifiedValue::menuseparator,
                        computed_value::T::meterbar => SpecifiedValue::meterbar,
                        computed_value::T::meterchunk => SpecifiedValue::meterchunk,
                        computed_value::T::number_input => SpecifiedValue::number_input,
                        computed_value::T::progressbar => SpecifiedValue::progressbar,
                        computed_value::T::progressbar_vertical => SpecifiedValue::progressbar_vertical,
                        computed_value::T::progresschunk => SpecifiedValue::progresschunk,
                        computed_value::T::progresschunk_vertical => SpecifiedValue::progresschunk_vertical,
                        computed_value::T::radio => SpecifiedValue::radio,
                        computed_value::T::radio_container => SpecifiedValue::radio_container,
                        computed_value::T::radio_label => SpecifiedValue::radio_label,
                        computed_value::T::radiomenuitem => SpecifiedValue::radiomenuitem,
                        computed_value::T::range => SpecifiedValue::range,
                        computed_value::T::range_thumb => SpecifiedValue::range_thumb,
                        computed_value::T::resizer => SpecifiedValue::resizer,
                        computed_value::T::resizerpanel => SpecifiedValue::resizerpanel,
                        computed_value::T::scale_horizontal => SpecifiedValue::scale_horizontal,
                        computed_value::T::scalethumbend => SpecifiedValue::scalethumbend,
                        computed_value::T::scalethumb_horizontal => SpecifiedValue::scalethumb_horizontal,
                        computed_value::T::scalethumbstart => SpecifiedValue::scalethumbstart,
                        computed_value::T::scalethumbtick => SpecifiedValue::scalethumbtick,
                        computed_value::T::scalethumb_vertical => SpecifiedValue::scalethumb_vertical,
                        computed_value::T::scale_vertical => SpecifiedValue::scale_vertical,
                        computed_value::T::scrollbar => SpecifiedValue::scrollbar,
                        computed_value::T::scrollbar_horizontal => SpecifiedValue::scrollbar_horizontal,
                        computed_value::T::scrollbar_small => SpecifiedValue::scrollbar_small,
                        computed_value::T::scrollbar_vertical => SpecifiedValue::scrollbar_vertical,
                        computed_value::T::scrollbarbutton_down => SpecifiedValue::scrollbarbutton_down,
                        computed_value::T::scrollbarbutton_left => SpecifiedValue::scrollbarbutton_left,
                        computed_value::T::scrollbarbutton_right => SpecifiedValue::scrollbarbutton_right,
                        computed_value::T::scrollbarbutton_up => SpecifiedValue::scrollbarbutton_up,
                        computed_value::T::scrollbarthumb_horizontal => SpecifiedValue::scrollbarthumb_horizontal,
                        computed_value::T::scrollbarthumb_vertical => SpecifiedValue::scrollbarthumb_vertical,
                        computed_value::T::scrollbartrack_horizontal => SpecifiedValue::scrollbartrack_horizontal,
                        computed_value::T::scrollbartrack_vertical => SpecifiedValue::scrollbartrack_vertical,
                        computed_value::T::searchfield => SpecifiedValue::searchfield,
                        computed_value::T::separator => SpecifiedValue::separator,
                        computed_value::T::spinner => SpecifiedValue::spinner,
                        computed_value::T::spinner_downbutton => SpecifiedValue::spinner_downbutton,
                        computed_value::T::spinner_textfield => SpecifiedValue::spinner_textfield,
                        computed_value::T::spinner_upbutton => SpecifiedValue::spinner_upbutton,
                        computed_value::T::splitter => SpecifiedValue::splitter,
                        computed_value::T::statusbar => SpecifiedValue::statusbar,
                        computed_value::T::statusbarpanel => SpecifiedValue::statusbarpanel,
                        computed_value::T::tab => SpecifiedValue::tab,
                        computed_value::T::tabpanel => SpecifiedValue::tabpanel,
                        computed_value::T::tabpanels => SpecifiedValue::tabpanels,
                        computed_value::T::tab_scroll_arrow_back => SpecifiedValue::tab_scroll_arrow_back,
                        computed_value::T::tab_scroll_arrow_forward => SpecifiedValue::tab_scroll_arrow_forward,
                        computed_value::T::textfield => SpecifiedValue::textfield,
                        computed_value::T::textfield_multiline => SpecifiedValue::textfield_multiline,
                        computed_value::T::toolbar => SpecifiedValue::toolbar,
                        computed_value::T::toolbarbutton => SpecifiedValue::toolbarbutton,
                        computed_value::T::toolbarbutton_dropdown => SpecifiedValue::toolbarbutton_dropdown,
                        computed_value::T::toolbargripper => SpecifiedValue::toolbargripper,
                        computed_value::T::toolbox => SpecifiedValue::toolbox,
                        computed_value::T::tooltip => SpecifiedValue::tooltip,
                        computed_value::T::treeheader => SpecifiedValue::treeheader,
                        computed_value::T::treeheadercell => SpecifiedValue::treeheadercell,
                        computed_value::T::treeheadersortarrow => SpecifiedValue::treeheadersortarrow,
                        computed_value::T::treeitem => SpecifiedValue::treeitem,
                        computed_value::T::treeline => SpecifiedValue::treeline,
                        computed_value::T::treetwisty => SpecifiedValue::treetwisty,
                        computed_value::T::treetwistyopen => SpecifiedValue::treetwistyopen,
                        computed_value::T::treeview => SpecifiedValue::treeview,
                        computed_value::T::window => SpecifiedValue::window,
                        computed_value::T::_moz_gtk_info_bar => SpecifiedValue::_moz_gtk_info_bar,
                        computed_value::T::_moz_mac_active_source_list_selection => SpecifiedValue::_moz_mac_active_source_list_selection,
                        computed_value::T::_moz_mac_disclosure_button_closed => SpecifiedValue::_moz_mac_disclosure_button_closed,
                        computed_value::T::_moz_mac_disclosure_button_open => SpecifiedValue::_moz_mac_disclosure_button_open,
                        computed_value::T::_moz_mac_fullscreen_button => SpecifiedValue::_moz_mac_fullscreen_button,
                        computed_value::T::_moz_mac_help_button => SpecifiedValue::_moz_mac_help_button,
                        computed_value::T::_moz_mac_source_list => SpecifiedValue::_moz_mac_source_list,
                        computed_value::T::_moz_mac_source_list_selection => SpecifiedValue::_moz_mac_source_list_selection,
                        computed_value::T::_moz_mac_vibrancy_dark => SpecifiedValue::_moz_mac_vibrancy_dark,
                        computed_value::T::_moz_mac_vibrancy_light => SpecifiedValue::_moz_mac_vibrancy_light,
                        computed_value::T::_moz_win_borderless_glass => SpecifiedValue::_moz_win_borderless_glass,
                        computed_value::T::_moz_win_browsertabbar_toolbox => SpecifiedValue::_moz_win_browsertabbar_toolbox,
                        computed_value::T::_moz_win_communications_toolbox => SpecifiedValue::_moz_win_communications_toolbox,
                        computed_value::T::_moz_win_exclude_glass => SpecifiedValue::_moz_win_exclude_glass,
                        computed_value::T::_moz_win_glass => SpecifiedValue::_moz_win_glass,
                        computed_value::T::_moz_win_media_toolbox => SpecifiedValue::_moz_win_media_toolbox,
                        computed_value::T::_moz_window_button_box => SpecifiedValue::_moz_window_button_box,
                        computed_value::T::_moz_window_button_box_maximized => SpecifiedValue::_moz_window_button_box_maximized,
                        computed_value::T::_moz_window_button_close => SpecifiedValue::_moz_window_button_close,
                        computed_value::T::_moz_window_button_maximize => SpecifiedValue::_moz_window_button_maximize,
                        computed_value::T::_moz_window_button_minimize => SpecifiedValue::_moz_window_button_minimize,
                        computed_value::T::_moz_window_button_restore => SpecifiedValue::_moz_window_button_restore,
                        computed_value::T::_moz_window_frame_bottom => SpecifiedValue::_moz_window_frame_bottom,
                        computed_value::T::_moz_window_frame_left => SpecifiedValue::_moz_window_frame_left,
                        computed_value::T::_moz_window_frame_right => SpecifiedValue::_moz_window_frame_right,
                        computed_value::T::_moz_window_titlebar => SpecifiedValue::_moz_window_titlebar,
                        computed_value::T::_moz_window_titlebar_maximized => SpecifiedValue::_moz_window_titlebar_maximized,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MozAppearance(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MozAppearance);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::MozAppearance);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set__moz_appearance(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset__moz_appearance();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit__moz_appearance();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MozAppearance)
            }
    }





    
        
    
    /// Nonstandard (https://developer.mozilla.org/en-US/docs/Web/CSS/-moz-binding)
    pub mod _moz_binding {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::UrlOrNone as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::UrlOrNone as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { Either::Second(None_) }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::UrlOrNone::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MozBinding(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MozBinding);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::MozBinding);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = (**specified_value).to_computed_value(context);
                                context.builder.set__moz_binding(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset__moz_binding();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit__moz_binding();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<Box<SpecifiedValue>, ParseError<'i>> {
                    parse(context, input).map(|result| Box::new(result))
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MozBinding)
            }
    }




    
    

    
        
    
    /// Nonstandard (https://developer.mozilla.org/en-US/docs/Web/CSS/-moz-orient)
    pub mod _moz_orient {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "inline" => inline,
                    "block" => block,
                    "horizontal" => horizontal,
                    "vertical" => vertical,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::inline
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::inline
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::inline => computed_value::T::inline,
                        SpecifiedValue::block => computed_value::T::block,
                        SpecifiedValue::horizontal => computed_value::T::horizontal,
                        SpecifiedValue::vertical => computed_value::T::vertical,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::inline => SpecifiedValue::inline,
                        computed_value::T::block => SpecifiedValue::block,
                        computed_value::T::horizontal => SpecifiedValue::horizontal,
                        computed_value::T::vertical => SpecifiedValue::vertical,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MozOrient(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MozOrient);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::MozOrient);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set__moz_orient(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset__moz_orient();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit__moz_orient();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MozOrient)
            }
    }





    
    /// https://drafts.csswg.org/css-will-change/#will-change
    pub mod will_change {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
    use std::fmt;
    use style_traits::ToCss;
    use values::CustomIdent;

    pub mod computed_value {
        pub use super::SpecifiedValue as T;
    }

    #[derive(Clone, Debug, PartialEq, ToComputedValue)]
    #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
    #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
    pub enum SpecifiedValue {
        Auto,
        AnimateableFeatures(Vec<CustomIdent>),
    }

    impl ToCss for SpecifiedValue {
        fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
            match *self {
                SpecifiedValue::Auto => dest.write_str("auto"),
                SpecifiedValue::AnimateableFeatures(ref features) => {
                    let (first, rest) = features.split_first().unwrap();
                    first.to_css(dest)?;
                    for feature in rest {
                        dest.write_str(", ")?;
                        feature.to_css(dest)?;
                    }
                    Ok(())
                }
            }
        }
    }

    #[inline]
    pub fn get_initial_value() -> computed_value::T {
        computed_value::T::Auto
    }

    /// auto | <animateable-feature>#
    pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                         -> Result<SpecifiedValue, ParseError<'i>> {
        if input.try(|input| input.expect_ident_matching("auto")).is_ok() {
            Ok(computed_value::T::Auto)
        } else {
            input.parse_comma_separated(|i| {
                CustomIdent::from_ident(i.expect_ident()?, &[
                    "will-change",
                    "none",
                    "all",
                    "auto",
                ])
            }).map(SpecifiedValue::AnimateableFeatures)
        }
    }

        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::WillChange(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::WillChange);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::WillChange);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_will_change(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_will_change();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_will_change();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::WillChange)
            }
    }



    
        
    
    /// https://drafts.csswg.org/css-shapes/#shape-outside-property
    pub mod shape_outside {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::basic_shape::FloatAreaShape as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::basic_shape::FloatAreaShape as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { generics::basic_shape::ShapeSource::None }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::basic_shape::FloatAreaShape::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::ShapeOutside(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::ShapeOutside);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::ShapeOutside);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = (**specified_value).to_computed_value(context);
                                context.builder.set_shape_outside(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_shape_outside();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_shape_outside();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<Box<SpecifiedValue>, ParseError<'i>> {
                    parse(context, input).map(|result| Box::new(result))
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::ShapeOutside)
            }
    }




    
    /// https://compat.spec.whatwg.org/#touch-action
    pub mod touch_action {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
    use gecko_bindings::structs;
    use std::fmt;
    use style_traits::ToCss;

    pub mod computed_value {
        pub use super::SpecifiedValue as T;
    }

    bitflags! {
        /// These constants match Gecko's `NS_STYLE_TOUCH_ACTION_*` constants.
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[derive(ToComputedValue)]
        pub flags SpecifiedValue: u8 {
            const TOUCH_ACTION_NONE = structs::NS_STYLE_TOUCH_ACTION_NONE as u8,
            const TOUCH_ACTION_AUTO = structs::NS_STYLE_TOUCH_ACTION_AUTO as u8,
            const TOUCH_ACTION_PAN_X = structs::NS_STYLE_TOUCH_ACTION_PAN_X as u8,
            const TOUCH_ACTION_PAN_Y = structs::NS_STYLE_TOUCH_ACTION_PAN_Y as u8,
            const TOUCH_ACTION_MANIPULATION = structs::NS_STYLE_TOUCH_ACTION_MANIPULATION as u8,
        }
    }

    impl ToCss for SpecifiedValue {
        fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
            match *self {
                TOUCH_ACTION_NONE => dest.write_str("none"),
                TOUCH_ACTION_AUTO => dest.write_str("auto"),
                TOUCH_ACTION_MANIPULATION => dest.write_str("manipulation"),
                _ if self.contains(TOUCH_ACTION_PAN_X | TOUCH_ACTION_PAN_Y) => {
                    dest.write_str("pan-x pan-y")
                },
                _ if self.contains(TOUCH_ACTION_PAN_X) => {
                    dest.write_str("pan-x")
                },
                _ if self.contains(TOUCH_ACTION_PAN_Y) => {
                    dest.write_str("pan-y")
                },
                _ => panic!("invalid touch-action value"),
            }
        }
    }

    #[inline]
    pub fn get_initial_value() -> computed_value::T {
        TOUCH_ACTION_AUTO
    }

    pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                         -> Result<SpecifiedValue, ParseError<'i>> {
        // FIXME: remove clone() when lifetimes are non-lexical
        try_match_ident_ignore_ascii_case! { input.expect_ident()?.clone(),
            "auto" => Ok(TOUCH_ACTION_AUTO),
            "none" => Ok(TOUCH_ACTION_NONE),
            "manipulation" => Ok(TOUCH_ACTION_MANIPULATION),
            "pan-x" => {
                if input.try(|i| i.expect_ident_matching("pan-y")).is_ok() {
                    Ok(TOUCH_ACTION_PAN_X | TOUCH_ACTION_PAN_Y)
                } else {
                    Ok(TOUCH_ACTION_PAN_X)
                }
            },
            "pan-y" => {
                if input.try(|i| i.expect_ident_matching("pan-x")).is_ok() {
                    Ok(TOUCH_ACTION_PAN_X | TOUCH_ACTION_PAN_Y)
                } else {
                    Ok(TOUCH_ACTION_PAN_Y)
                }
            },
        }
    }

    #[cfg(feature = "gecko")]
    impl_bitflags_conversions!(SpecifiedValue);

        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::TouchAction(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::TouchAction);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::TouchAction);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_touch_action(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_touch_action();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_touch_action();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::TouchAction)
            }
    }


    /* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */








    
        
    
    /// https://drafts.csswg.org/css-color/#color
    pub mod color {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::ColorPropertyValue as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::ColorPropertyValue as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { ::cssparser::RGBA::new(0, 0, 0, 255) }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::ColorPropertyValue::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::Color(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::Color);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_color(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_color();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_color();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::Color)
            }
    }



// FIXME(#15973): Add servo support for system colors
//
// FIXME(emilio): Move outside of mako.
pub mod system_colors {
    
    use cssparser::Parser;
    use gecko_bindings::bindings::Gecko_GetLookAndFeelSystemColor;
    use gecko_bindings::structs::root::mozilla::LookAndFeel_ColorID;
    use std::fmt;
    use style_traits::ToCss;
    use values::computed::{Context, ToComputedValue};

    pub type SystemColor = LookAndFeel_ColorID;

    // It's hard to implement MallocSizeOf for LookAndFeel_ColorID because it
    // is a bindgen type. So we implement it on the typedef instead.
    size_of_is_0!(SystemColor);

    impl ToCss for SystemColor {
        fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
            let s = match *self {
                    LookAndFeel_ColorID::eColorID_activeborder => "activeborder",
                    LookAndFeel_ColorID::eColorID_activecaption => "activecaption",
                    LookAndFeel_ColorID::eColorID_appworkspace => "appworkspace",
                    LookAndFeel_ColorID::eColorID_background => "background",
                    LookAndFeel_ColorID::eColorID_buttonface => "buttonface",
                    LookAndFeel_ColorID::eColorID_buttonhighlight => "buttonhighlight",
                    LookAndFeel_ColorID::eColorID_buttonshadow => "buttonshadow",
                    LookAndFeel_ColorID::eColorID_buttontext => "buttontext",
                    LookAndFeel_ColorID::eColorID_captiontext => "captiontext",
                    LookAndFeel_ColorID::eColorID_graytext => "graytext",
                    LookAndFeel_ColorID::eColorID_highlight => "highlight",
                    LookAndFeel_ColorID::eColorID_highlighttext => "highlighttext",
                    LookAndFeel_ColorID::eColorID_inactiveborder => "inactiveborder",
                    LookAndFeel_ColorID::eColorID_inactivecaption => "inactivecaption",
                    LookAndFeel_ColorID::eColorID_inactivecaptiontext => "inactivecaptiontext",
                    LookAndFeel_ColorID::eColorID_infobackground => "infobackground",
                    LookAndFeel_ColorID::eColorID_infotext => "infotext",
                    LookAndFeel_ColorID::eColorID_menu => "menu",
                    LookAndFeel_ColorID::eColorID_menutext => "menutext",
                    LookAndFeel_ColorID::eColorID_scrollbar => "scrollbar",
                    LookAndFeel_ColorID::eColorID_threeddarkshadow => "threeddarkshadow",
                    LookAndFeel_ColorID::eColorID_threedface => "threedface",
                    LookAndFeel_ColorID::eColorID_threedhighlight => "threedhighlight",
                    LookAndFeel_ColorID::eColorID_threedlightshadow => "threedlightshadow",
                    LookAndFeel_ColorID::eColorID_threedshadow => "threedshadow",
                    LookAndFeel_ColorID::eColorID_window => "window",
                    LookAndFeel_ColorID::eColorID_windowframe => "windowframe",
                    LookAndFeel_ColorID::eColorID_windowtext => "windowtext",
                    LookAndFeel_ColorID::eColorID__moz_buttondefault => "-moz-buttondefault",
                    LookAndFeel_ColorID::eColorID__moz_buttonhoverface => "-moz-buttonhoverface",
                    LookAndFeel_ColorID::eColorID__moz_buttonhovertext => "-moz-buttonhovertext",
                    LookAndFeel_ColorID::eColorID__moz_cellhighlight => "-moz-cellhighlight",
                    LookAndFeel_ColorID::eColorID__moz_cellhighlighttext => "-moz-cellhighlighttext",
                    LookAndFeel_ColorID::eColorID__moz_eventreerow => "-moz-eventreerow",
                    LookAndFeel_ColorID::eColorID__moz_field => "-moz-field",
                    LookAndFeel_ColorID::eColorID__moz_fieldtext => "-moz-fieldtext",
                    LookAndFeel_ColorID::eColorID__moz_dialog => "-moz-dialog",
                    LookAndFeel_ColorID::eColorID__moz_dialogtext => "-moz-dialogtext",
                    LookAndFeel_ColorID::eColorID__moz_dragtargetzone => "-moz-dragtargetzone",
                    LookAndFeel_ColorID::eColorID__moz_gtk_info_bar_text => "-moz-gtk-info-bar-text",
                    LookAndFeel_ColorID::eColorID__moz_html_cellhighlight => "-moz-html-cellhighlight",
                    LookAndFeel_ColorID::eColorID__moz_html_cellhighlighttext => "-moz-html-cellhighlighttext",
                    LookAndFeel_ColorID::eColorID__moz_mac_buttonactivetext => "-moz-mac-buttonactivetext",
                    LookAndFeel_ColorID::eColorID__moz_mac_chrome_active => "-moz-mac-chrome-active",
                    LookAndFeel_ColorID::eColorID__moz_mac_chrome_inactive => "-moz-mac-chrome-inactive",
                    LookAndFeel_ColorID::eColorID__moz_mac_defaultbuttontext => "-moz-mac-defaultbuttontext",
                    LookAndFeel_ColorID::eColorID__moz_mac_focusring => "-moz-mac-focusring",
                    LookAndFeel_ColorID::eColorID__moz_mac_menuselect => "-moz-mac-menuselect",
                    LookAndFeel_ColorID::eColorID__moz_mac_menushadow => "-moz-mac-menushadow",
                    LookAndFeel_ColorID::eColorID__moz_mac_menutextdisable => "-moz-mac-menutextdisable",
                    LookAndFeel_ColorID::eColorID__moz_mac_menutextselect => "-moz-mac-menutextselect",
                    LookAndFeel_ColorID::eColorID__moz_mac_disabledtoolbartext => "-moz-mac-disabledtoolbartext",
                    LookAndFeel_ColorID::eColorID__moz_mac_secondaryhighlight => "-moz-mac-secondaryhighlight",
                    LookAndFeel_ColorID::eColorID__moz_mac_vibrancy_light => "-moz-mac-vibrancy-light",
                    LookAndFeel_ColorID::eColorID__moz_mac_vibrancy_dark => "-moz-mac-vibrancy-dark",
                    LookAndFeel_ColorID::eColorID__moz_mac_menupopup => "-moz-mac-menupopup",
                    LookAndFeel_ColorID::eColorID__moz_mac_menuitem => "-moz-mac-menuitem",
                    LookAndFeel_ColorID::eColorID__moz_mac_active_menuitem => "-moz-mac-active-menuitem",
                    LookAndFeel_ColorID::eColorID__moz_mac_source_list => "-moz-mac-source-list",
                    LookAndFeel_ColorID::eColorID__moz_mac_source_list_selection => "-moz-mac-source-list-selection",
                    LookAndFeel_ColorID::eColorID__moz_mac_active_source_list_selection => "-moz-mac-active-source-list-selection",
                    LookAndFeel_ColorID::eColorID__moz_mac_tooltip => "-moz-mac-tooltip",
                    LookAndFeel_ColorID::eColorID__moz_menuhover => "-moz-menuhover",
                    LookAndFeel_ColorID::eColorID__moz_menuhovertext => "-moz-menuhovertext",
                    LookAndFeel_ColorID::eColorID__moz_menubartext => "-moz-menubartext",
                    LookAndFeel_ColorID::eColorID__moz_menubarhovertext => "-moz-menubarhovertext",
                    LookAndFeel_ColorID::eColorID__moz_oddtreerow => "-moz-oddtreerow",
                    LookAndFeel_ColorID::eColorID__moz_win_mediatext => "-moz-win-mediatext",
                    LookAndFeel_ColorID::eColorID__moz_win_communicationstext => "-moz-win-communicationstext",
                    LookAndFeel_ColorID::eColorID__moz_win_accentcolor => "-moz-win-accentcolor",
                    LookAndFeel_ColorID::eColorID__moz_win_accentcolortext => "-moz-win-accentcolortext",
                    LookAndFeel_ColorID::eColorID__moz_nativehyperlinktext => "-moz-nativehyperlinktext",
                    LookAndFeel_ColorID::eColorID__moz_comboboxtext => "-moz-comboboxtext",
                    LookAndFeel_ColorID::eColorID__moz_combobox => "-moz-combobox",
                    LookAndFeel_ColorID::eColorID_WindowBackground => "WindowBackground",
                    LookAndFeel_ColorID::eColorID_WindowForeground => "WindowForeground",
                    LookAndFeel_ColorID::eColorID_WidgetBackground => "WidgetBackground",
                    LookAndFeel_ColorID::eColorID_WidgetForeground => "WidgetForeground",
                    LookAndFeel_ColorID::eColorID_WidgetSelectBackground => "WidgetSelectBackground",
                    LookAndFeel_ColorID::eColorID_WidgetSelectForeground => "WidgetSelectForeground",
                    LookAndFeel_ColorID::eColorID_Widget3DHighlight => "Widget3DHighlight",
                    LookAndFeel_ColorID::eColorID_Widget3DShadow => "Widget3DShadow",
                    LookAndFeel_ColorID::eColorID_TextBackground => "TextBackground",
                    LookAndFeel_ColorID::eColorID_TextForeground => "TextForeground",
                    LookAndFeel_ColorID::eColorID_TextSelectBackground => "TextSelectBackground",
                    LookAndFeel_ColorID::eColorID_TextSelectForeground => "TextSelectForeground",
                    LookAndFeel_ColorID::eColorID_TextSelectForegroundCustom => "TextSelectForegroundCustom",
                    LookAndFeel_ColorID::eColorID_TextSelectBackgroundDisabled => "TextSelectBackgroundDisabled",
                    LookAndFeel_ColorID::eColorID_TextSelectBackgroundAttention => "TextSelectBackgroundAttention",
                    LookAndFeel_ColorID::eColorID_TextHighlightBackground => "TextHighlightBackground",
                    LookAndFeel_ColorID::eColorID_TextHighlightForeground => "TextHighlightForeground",
                    LookAndFeel_ColorID::eColorID_IMERawInputBackground => "IMERawInputBackground",
                    LookAndFeel_ColorID::eColorID_IMERawInputForeground => "IMERawInputForeground",
                    LookAndFeel_ColorID::eColorID_IMERawInputUnderline => "IMERawInputUnderline",
                    LookAndFeel_ColorID::eColorID_IMESelectedRawTextBackground => "IMESelectedRawTextBackground",
                    LookAndFeel_ColorID::eColorID_IMESelectedRawTextForeground => "IMESelectedRawTextForeground",
                    LookAndFeel_ColorID::eColorID_IMESelectedRawTextUnderline => "IMESelectedRawTextUnderline",
                    LookAndFeel_ColorID::eColorID_IMEConvertedTextBackground => "IMEConvertedTextBackground",
                    LookAndFeel_ColorID::eColorID_IMEConvertedTextForeground => "IMEConvertedTextForeground",
                    LookAndFeel_ColorID::eColorID_IMEConvertedTextUnderline => "IMEConvertedTextUnderline",
                    LookAndFeel_ColorID::eColorID_IMESelectedConvertedTextBackground => "IMESelectedConvertedTextBackground",
                    LookAndFeel_ColorID::eColorID_IMESelectedConvertedTextForeground => "IMESelectedConvertedTextForeground",
                    LookAndFeel_ColorID::eColorID_IMESelectedConvertedTextUnderline => "IMESelectedConvertedTextUnderline",
                    LookAndFeel_ColorID::eColorID_SpellCheckerUnderline => "SpellCheckerUnderline",
                LookAndFeel_ColorID::eColorID_LAST_COLOR => unreachable!(),
            };
            dest.write_str(s)
        }
    }

    impl ToComputedValue for SystemColor {
        type ComputedValue = u32; // nscolor

        #[inline]
        fn to_computed_value(&self, cx: &Context) -> Self::ComputedValue {
            unsafe {
                Gecko_GetLookAndFeelSystemColor(*self as i32,
                                                cx.device().pres_context())
            }
        }

        #[inline]
        fn from_computed_value(_: &Self::ComputedValue) -> Self {
            unreachable!()
        }
    }

    impl SystemColor {
        pub fn parse<'i, 't>(input: &mut Parser<'i, 't>,) -> Result<Self, ()> {
            ascii_case_insensitive_phf_map! {
                color_name -> SystemColor = {
                        "activeborder" => LookAndFeel_ColorID::eColorID_activeborder,
                        "activecaption" => LookAndFeel_ColorID::eColorID_activecaption,
                        "appworkspace" => LookAndFeel_ColorID::eColorID_appworkspace,
                        "background" => LookAndFeel_ColorID::eColorID_background,
                        "buttonface" => LookAndFeel_ColorID::eColorID_buttonface,
                        "buttonhighlight" => LookAndFeel_ColorID::eColorID_buttonhighlight,
                        "buttonshadow" => LookAndFeel_ColorID::eColorID_buttonshadow,
                        "buttontext" => LookAndFeel_ColorID::eColorID_buttontext,
                        "captiontext" => LookAndFeel_ColorID::eColorID_captiontext,
                        "graytext" => LookAndFeel_ColorID::eColorID_graytext,
                        "highlight" => LookAndFeel_ColorID::eColorID_highlight,
                        "highlighttext" => LookAndFeel_ColorID::eColorID_highlighttext,
                        "inactiveborder" => LookAndFeel_ColorID::eColorID_inactiveborder,
                        "inactivecaption" => LookAndFeel_ColorID::eColorID_inactivecaption,
                        "inactivecaptiontext" => LookAndFeel_ColorID::eColorID_inactivecaptiontext,
                        "infobackground" => LookAndFeel_ColorID::eColorID_infobackground,
                        "infotext" => LookAndFeel_ColorID::eColorID_infotext,
                        "menu" => LookAndFeel_ColorID::eColorID_menu,
                        "menutext" => LookAndFeel_ColorID::eColorID_menutext,
                        "scrollbar" => LookAndFeel_ColorID::eColorID_scrollbar,
                        "threeddarkshadow" => LookAndFeel_ColorID::eColorID_threeddarkshadow,
                        "threedface" => LookAndFeel_ColorID::eColorID_threedface,
                        "threedhighlight" => LookAndFeel_ColorID::eColorID_threedhighlight,
                        "threedlightshadow" => LookAndFeel_ColorID::eColorID_threedlightshadow,
                        "threedshadow" => LookAndFeel_ColorID::eColorID_threedshadow,
                        "window" => LookAndFeel_ColorID::eColorID_window,
                        "windowframe" => LookAndFeel_ColorID::eColorID_windowframe,
                        "windowtext" => LookAndFeel_ColorID::eColorID_windowtext,
                        "-moz-buttondefault" => LookAndFeel_ColorID::eColorID__moz_buttondefault,
                        "-moz-buttonhoverface" => LookAndFeel_ColorID::eColorID__moz_buttonhoverface,
                        "-moz-buttonhovertext" => LookAndFeel_ColorID::eColorID__moz_buttonhovertext,
                        "-moz-cellhighlight" => LookAndFeel_ColorID::eColorID__moz_cellhighlight,
                        "-moz-cellhighlighttext" => LookAndFeel_ColorID::eColorID__moz_cellhighlighttext,
                        "-moz-eventreerow" => LookAndFeel_ColorID::eColorID__moz_eventreerow,
                        "-moz-field" => LookAndFeel_ColorID::eColorID__moz_field,
                        "-moz-fieldtext" => LookAndFeel_ColorID::eColorID__moz_fieldtext,
                        "-moz-dialog" => LookAndFeel_ColorID::eColorID__moz_dialog,
                        "-moz-dialogtext" => LookAndFeel_ColorID::eColorID__moz_dialogtext,
                        "-moz-dragtargetzone" => LookAndFeel_ColorID::eColorID__moz_dragtargetzone,
                        "-moz-gtk-info-bar-text" => LookAndFeel_ColorID::eColorID__moz_gtk_info_bar_text,
                        "-moz-html-cellhighlight" => LookAndFeel_ColorID::eColorID__moz_html_cellhighlight,
                        "-moz-html-cellhighlighttext" => LookAndFeel_ColorID::eColorID__moz_html_cellhighlighttext,
                        "-moz-mac-buttonactivetext" => LookAndFeel_ColorID::eColorID__moz_mac_buttonactivetext,
                        "-moz-mac-chrome-active" => LookAndFeel_ColorID::eColorID__moz_mac_chrome_active,
                        "-moz-mac-chrome-inactive" => LookAndFeel_ColorID::eColorID__moz_mac_chrome_inactive,
                        "-moz-mac-defaultbuttontext" => LookAndFeel_ColorID::eColorID__moz_mac_defaultbuttontext,
                        "-moz-mac-focusring" => LookAndFeel_ColorID::eColorID__moz_mac_focusring,
                        "-moz-mac-menuselect" => LookAndFeel_ColorID::eColorID__moz_mac_menuselect,
                        "-moz-mac-menushadow" => LookAndFeel_ColorID::eColorID__moz_mac_menushadow,
                        "-moz-mac-menutextdisable" => LookAndFeel_ColorID::eColorID__moz_mac_menutextdisable,
                        "-moz-mac-menutextselect" => LookAndFeel_ColorID::eColorID__moz_mac_menutextselect,
                        "-moz-mac-disabledtoolbartext" => LookAndFeel_ColorID::eColorID__moz_mac_disabledtoolbartext,
                        "-moz-mac-secondaryhighlight" => LookAndFeel_ColorID::eColorID__moz_mac_secondaryhighlight,
                        "-moz-mac-vibrancy-light" => LookAndFeel_ColorID::eColorID__moz_mac_vibrancy_light,
                        "-moz-mac-vibrancy-dark" => LookAndFeel_ColorID::eColorID__moz_mac_vibrancy_dark,
                        "-moz-mac-menupopup" => LookAndFeel_ColorID::eColorID__moz_mac_menupopup,
                        "-moz-mac-menuitem" => LookAndFeel_ColorID::eColorID__moz_mac_menuitem,
                        "-moz-mac-active-menuitem" => LookAndFeel_ColorID::eColorID__moz_mac_active_menuitem,
                        "-moz-mac-source-list" => LookAndFeel_ColorID::eColorID__moz_mac_source_list,
                        "-moz-mac-source-list-selection" => LookAndFeel_ColorID::eColorID__moz_mac_source_list_selection,
                        "-moz-mac-active-source-list-selection" => LookAndFeel_ColorID::eColorID__moz_mac_active_source_list_selection,
                        "-moz-mac-tooltip" => LookAndFeel_ColorID::eColorID__moz_mac_tooltip,
                        "-moz-menuhover" => LookAndFeel_ColorID::eColorID__moz_menuhover,
                        "-moz-menuhovertext" => LookAndFeel_ColorID::eColorID__moz_menuhovertext,
                        "-moz-menubartext" => LookAndFeel_ColorID::eColorID__moz_menubartext,
                        "-moz-menubarhovertext" => LookAndFeel_ColorID::eColorID__moz_menubarhovertext,
                        "-moz-oddtreerow" => LookAndFeel_ColorID::eColorID__moz_oddtreerow,
                        "-moz-win-mediatext" => LookAndFeel_ColorID::eColorID__moz_win_mediatext,
                        "-moz-win-communicationstext" => LookAndFeel_ColorID::eColorID__moz_win_communicationstext,
                        "-moz-win-accentcolor" => LookAndFeel_ColorID::eColorID__moz_win_accentcolor,
                        "-moz-win-accentcolortext" => LookAndFeel_ColorID::eColorID__moz_win_accentcolortext,
                        "-moz-nativehyperlinktext" => LookAndFeel_ColorID::eColorID__moz_nativehyperlinktext,
                        "-moz-comboboxtext" => LookAndFeel_ColorID::eColorID__moz_comboboxtext,
                        "-moz-combobox" => LookAndFeel_ColorID::eColorID__moz_combobox,
                }
            }

            let ident = input.expect_ident().map_err(|_| ())?;
            color_name(ident).cloned().ok_or(())
        }
    }
}

    /* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */






    
        
    
    /// https://drafts.csswg.org/css-multicol/#propdef-column-width
    pub mod column_width {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::length::NonNegativeLengthOrAuto as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::length::NonNegativeLengthOrAuto as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { Either::Second(Auto) }
        #[inline] pub fn get_initial_specified_value() -> SpecifiedValue { Either::Second(Auto) }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::length::NonNegativeLengthOrAuto::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::ColumnWidth(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::ColumnWidth);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::ColumnWidth);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_column_width(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_column_width();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_column_width();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::ColumnWidth)
            }
    }





    
        
    
    /// https://drafts.csswg.org/css-multicol/#propdef-column-count
    pub mod column_count {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::PositiveIntegerOrAuto as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::PositiveIntegerOrAuto as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { Either::Second(Auto) }
        #[inline] pub fn get_initial_specified_value() -> SpecifiedValue { Either::Second(Auto) }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::PositiveIntegerOrAuto::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::ColumnCount(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::ColumnCount);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::ColumnCount);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_column_count(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_column_count();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_column_count();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::ColumnCount)
            }
    }




    
        
    
    /// https://drafts.csswg.org/css-multicol/#propdef-column-gap
    pub mod column_gap {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::length::NonNegativeLengthOrNormal as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::length::NonNegativeLengthOrNormal as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { Either::Second(Normal) }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::length::NonNegativeLengthOrNormal::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::ColumnGap(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::ColumnGap);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::ColumnGap);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_column_gap(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_column_gap();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_column_gap();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::ColumnGap)
            }
    }




    
    

    
        
    
    /// https://drafts.csswg.org/css-multicol/#propdef-column-fill
    pub mod column_fill {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "balance" => balance,
                    "auto" => auto,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::balance
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::balance
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::balance => computed_value::T::balance,
                        SpecifiedValue::auto => computed_value::T::auto,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::balance => SpecifiedValue::balance,
                        computed_value::T::auto => SpecifiedValue::auto,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::ColumnFill(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::ColumnFill);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::ColumnFill);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_column_fill(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_column_fill();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_column_fill();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::ColumnFill)
            }
    }





    
        
    
    /// https://drafts.csswg.org/css-multicol/#propdef-column-rule-width
    pub mod column_rule_width {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::BorderSideWidth as SpecifiedValue;
        pub mod computed_value {
            pub use ::values::computed::NonNegativeLength as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { ::values::computed::NonNegativeLength::new(3.) }
        #[inline] pub fn get_initial_specified_value() -> SpecifiedValue { specified::BorderSideWidth::Medium }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::BorderSideWidth::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::ColumnRuleWidth(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::ColumnRuleWidth);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::ColumnRuleWidth);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_column_rule_width(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_column_rule_width();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_column_rule_width();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::ColumnRuleWidth)
            }
    }



// https://drafts.csswg.org/css-multicol-1/#crc

    
        
    
    /// https://drafts.csswg.org/css-multicol/#propdef-column-rule-color
    pub mod column_rule_color {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::Color as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::Color as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed_value::T::currentcolor() }
        #[inline] pub fn get_initial_specified_value() -> SpecifiedValue { specified::Color::currentcolor() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::Color::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::ColumnRuleColor(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::ColumnRuleColor);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::ColumnRuleColor);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_column_rule_color(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_column_rule_color();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_column_rule_color();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::ColumnRuleColor)
            }
    }




    
    

    
        
    
    /// https://drafts.csswg.org/css-multicol/#propdef-column-span
    pub mod column_span {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "none" => none,
                    "all" => all,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::none
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::none
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::none => computed_value::T::none,
                        SpecifiedValue::all => computed_value::T::all,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::none => SpecifiedValue::none,
                        computed_value::T::all => SpecifiedValue::all,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::ColumnSpan(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::ColumnSpan);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::ColumnSpan);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_column_span(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_column_span();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_column_span();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::ColumnSpan)
            }
    }





    
    

    
        
    
    /// https://drafts.csswg.org/css-multicol/#propdef-column-rule-style
    pub mod column_rule_style {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "none" => none,
                    "hidden" => hidden,
                    "dotted" => dotted,
                    "dashed" => dashed,
                    "solid" => solid,
                    "double" => double,
                    "groove" => groove,
                    "ridge" => ridge,
                    "inset" => inset,
                    "outset" => outset,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::none
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::none
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::none => computed_value::T::none,
                        SpecifiedValue::hidden => computed_value::T::hidden,
                        SpecifiedValue::dotted => computed_value::T::dotted,
                        SpecifiedValue::dashed => computed_value::T::dashed,
                        SpecifiedValue::solid => computed_value::T::solid,
                        SpecifiedValue::double => computed_value::T::double,
                        SpecifiedValue::groove => computed_value::T::groove,
                        SpecifiedValue::ridge => computed_value::T::ridge,
                        SpecifiedValue::inset => computed_value::T::inset,
                        SpecifiedValue::outset => computed_value::T::outset,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::none => SpecifiedValue::none,
                        computed_value::T::hidden => SpecifiedValue::hidden,
                        computed_value::T::dotted => SpecifiedValue::dotted,
                        computed_value::T::dashed => SpecifiedValue::dashed,
                        computed_value::T::solid => SpecifiedValue::solid,
                        computed_value::T::double => SpecifiedValue::double,
                        computed_value::T::groove => SpecifiedValue::groove,
                        computed_value::T::ridge => SpecifiedValue::ridge,
                        computed_value::T::inset => SpecifiedValue::inset,
                        computed_value::T::outset => SpecifiedValue::outset,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::ColumnRuleStyle(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::ColumnRuleStyle);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::ColumnRuleStyle);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_column_rule_style(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_column_rule_style();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_column_rule_style();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::ColumnRuleStyle)
            }
    }




    /* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */






    
    /// https://drafts.csswg.org/css-content/#propdef-content
    pub mod content {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
    #[cfg(feature = "gecko")]
    use values::generics::CounterStyleOrNone;
    #[cfg(feature = "gecko")]
    use values::specified::url::SpecifiedUrl;
    #[cfg(feature = "gecko")]
    use values::specified::Attr;

    #[cfg(feature = "servo")]
    use super::list_style_type;

    pub use self::computed_value::T as SpecifiedValue;
    pub use self::computed_value::ContentItem;

    pub mod computed_value {
        use cssparser;
        use std::fmt;
        use style_traits::ToCss;
        #[cfg(feature = "gecko")]
        use values::specified::url::SpecifiedUrl;

        #[cfg(feature = "servo")]
        type CounterStyleType = super::super::list_style_type::computed_value::T;
        #[cfg(feature = "gecko")]
        type CounterStyleType = ::values::generics::CounterStyleOrNone;

        #[cfg(feature = "gecko")]
        use values::specified::Attr;

        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[derive(Clone, Debug, Eq, PartialEq, ToComputedValue)]
        pub enum ContentItem {
            /// Literal string content.
            String(String),
            /// `counter(name, style)`.
            Counter(String, CounterStyleType),
            /// `counters(name, separator, style)`.
            Counters(String, String, CounterStyleType),
            /// `open-quote`.
            OpenQuote,
            /// `close-quote`.
            CloseQuote,
            /// `no-open-quote`.
            NoOpenQuote,
            /// `no-close-quote`.
            NoCloseQuote,

                /// `attr([namespace? `|`]? ident)`
                Attr(Attr),
                /// `url(url)`
                Url(SpecifiedUrl),
        }

        impl ToCss for ContentItem {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
                match *self {
                    ContentItem::String(ref s) => s.to_css(dest),
                    ContentItem::Counter(ref s, ref counter_style) => {
                        dest.write_str("counter(")?;
                        cssparser::serialize_identifier(&**s, dest)?;
                        dest.write_str(", ")?;
                        counter_style.to_css(dest)?;
                        dest.write_str(")")
                    }
                    ContentItem::Counters(ref s, ref separator, ref counter_style) => {
                        dest.write_str("counters(")?;
                        cssparser::serialize_identifier(&**s, dest)?;
                        dest.write_str(", ")?;
                        separator.to_css(dest)?;
                        dest.write_str(", ")?;
                        counter_style.to_css(dest)?;
                        dest.write_str(")")
                    }
                    ContentItem::OpenQuote => dest.write_str("open-quote"),
                    ContentItem::CloseQuote => dest.write_str("close-quote"),
                    ContentItem::NoOpenQuote => dest.write_str("no-open-quote"),
                    ContentItem::NoCloseQuote => dest.write_str("no-close-quote"),

                        ContentItem::Attr(ref attr) => {
                            attr.to_css(dest)
                        }
                        ContentItem::Url(ref url) => url.to_css(dest),
                }
            }
        }

        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        #[derive(Clone, Debug, Eq, PartialEq, ToComputedValue)]
        pub enum T {
            Normal,
            None,
            #[cfg(feature = "gecko")]
            MozAltContent,
            Items(Vec<ContentItem>),
        }

        impl ToCss for T {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
                match *self {
                    T::Normal => dest.write_str("normal"),
                    T::None => dest.write_str("none"),
                        T::MozAltContent => dest.write_str("-moz-alt-content"),
                    T::Items(ref content) => {
                        let mut iter = content.iter();
                        iter.next().unwrap().to_css(dest)?;
                        for c in iter {
                            dest.write_str(" ")?;
                            c.to_css(dest)?;
                        }
                        Ok(())
                    }
                }
            }
        }
    }
    #[inline]
    pub fn get_initial_value() -> computed_value::T {
        computed_value::T::Normal
    }

    #[cfg(feature = "servo")]
    fn parse_counter_style(context: &ParserContext, input: &mut Parser) -> list_style_type::computed_value::T {
        input.try(|input| {
            input.expect_comma()?;
            list_style_type::parse(context, input)
        }).unwrap_or(list_style_type::computed_value::T::decimal)
    }

    #[cfg(feature = "gecko")]
    fn parse_counter_style(context: &ParserContext, input: &mut Parser) -> CounterStyleOrNone {
        input.try(|input| {
            input.expect_comma()?;
            CounterStyleOrNone::parse(context, input)
        }).unwrap_or(CounterStyleOrNone::decimal())
    }

    // normal | none | [ <string> | <counter> | open-quote | close-quote | no-open-quote |
    // no-close-quote ]+
    // TODO: <uri>, attr(<identifier>)
    pub fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                         -> Result<SpecifiedValue, ParseError<'i>> {
        if input.try(|input| input.expect_ident_matching("normal")).is_ok() {
            return Ok(SpecifiedValue::Normal)
        }
        if input.try(|input| input.expect_ident_matching("none")).is_ok() {
            return Ok(SpecifiedValue::None)
        }
            if input.try(|input| input.expect_ident_matching("-moz-alt-content")).is_ok() {
                return Ok(SpecifiedValue::MozAltContent)
            }

        let mut content = vec![];
        loop {
                if let Ok(mut url) = input.try(|i| SpecifiedUrl::parse(context, i)) {
                    url.build_image_value();
                    content.push(ContentItem::Url(url));
                    continue;
                }
            // FIXME: remove clone() when lifetimes are non-lexical
            match input.next().map(|t| t.clone()) {
                Ok(Token::QuotedString(ref value)) => {
                    content.push(ContentItem::String(value.as_ref().to_owned()))
                }
                Ok(Token::Function(ref name)) => {
                    let result = match_ignore_ascii_case! { &name,
                        "counter" => Some(input.parse_nested_block(|input| {
                            let name = input.expect_ident()?.as_ref().to_owned();
                            let style = parse_counter_style(context, input);
                            Ok(ContentItem::Counter(name, style))
                        })),
                        "counters" => Some(input.parse_nested_block(|input| {
                            let name = input.expect_ident()?.as_ref().to_owned();
                            input.expect_comma()?;
                            let separator = input.expect_string()?.as_ref().to_owned();
                            let style = parse_counter_style(context, input);
                            Ok(ContentItem::Counters(name, separator, style))
                        })),
                            "attr" => Some(input.parse_nested_block(|input| {
                                Ok(ContentItem::Attr(Attr::parse_function(context, input)?))
                            })),
                        _ => None
                    };
                    match result {
                        Some(result) => content.push(result?),
                        None => return Err(StyleParseError::UnexpectedFunction(name.clone()).into())
                    }
                }
                Ok(Token::Ident(ref ident)) => {
                    let valid = match_ignore_ascii_case! { &ident,
                        "open-quote" => { content.push(ContentItem::OpenQuote); true },
                        "close-quote" => { content.push(ContentItem::CloseQuote); true },
                        "no-open-quote" => { content.push(ContentItem::NoOpenQuote); true },
                        "no-close-quote" => { content.push(ContentItem::NoCloseQuote); true },

                        _ => false,
                    };
                    if !valid {
                        return Err(SelectorParseError::UnexpectedIdent(ident.clone()).into())
                    }
                }
                Err(_) => break,
                Ok(t) => return Err(BasicParseError::UnexpectedToken(t).into())
            }
        }
        if content.is_empty() {
            return Err(StyleParseError::UnspecifiedError.into());
        }
        Ok(SpecifiedValue::Items(content))
    }

        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::Content(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::Content);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::Content);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = (**specified_value).to_computed_value(context);
                                context.builder.set_content(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_content();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_content();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<Box<SpecifiedValue>, ParseError<'i>> {
                    parse(context, input).map(|result| Box::new(result))
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::Content)
            }
    }



    
    /// https://drafts.csswg.org/css-lists/#propdef-counter-increment
    pub mod counter_increment {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
    use std::fmt;
    use style_traits::ToCss;
    use values::CustomIdent;

    #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
    #[derive(Clone, Debug, PartialEq)]
    pub struct SpecifiedValue(pub Vec<(CustomIdent, specified::Integer)>);

    pub mod computed_value {
        use std::fmt;
        use style_traits::ToCss;
        use values::CustomIdent;

        #[derive(Clone, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        pub struct T(pub Vec<(CustomIdent, i32)>);

        impl ToCss for T {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                if self.0.is_empty() {
                    return dest.write_str("none")
                }

                let mut first = true;
                for &(ref name, value) in &self.0 {
                    if !first {
                        dest.write_str(" ")?;
                    }
                    first = false;
                    name.to_css(dest)?;
                    dest.write_str(" ")?;
                    value.to_css(dest)?;
                }
                Ok(())
            }
        }
    }

    impl ToComputedValue for SpecifiedValue {
        type ComputedValue = computed_value::T;

        fn to_computed_value(&self, context: &Context) -> Self::ComputedValue {
            computed_value::T(self.0.iter().map(|&(ref name, ref value)| {
                (name.clone(), value.to_computed_value(context))
            }).collect::<Vec<_>>())
        }

        fn from_computed_value(computed: &Self::ComputedValue) -> Self {
            SpecifiedValue(computed.0.iter().map(|&(ref name, ref value)| {
                (name.clone(), specified::Integer::from_computed_value(&value))
            }).collect::<Vec<_>>())
        }
    }

    #[inline]
    pub fn get_initial_value() -> computed_value::T {
        computed_value::T(Vec::new())
    }


    impl ToCss for SpecifiedValue {
        fn to_css<W>(&self, dest: &mut W) -> fmt::Result
            where W: fmt::Write,
        {
            if self.0.is_empty() {
                return dest.write_str("none");
            }
            let mut first = true;
            for &(ref name, ref value) in &self.0 {
                if !first {
                    dest.write_str(" ")?;
                }
                first = false;
                name.to_css(dest)?;
                dest.write_str(" ")?;
                value.to_css(dest)?;
            }

            Ok(())
        }
    }

    pub fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                         -> Result<SpecifiedValue, ParseError<'i>> {
        parse_common(context, 1, input)
    }

    pub fn parse_common<'i, 't>(context: &ParserContext, default_value: i32, input: &mut Parser<'i, 't>)
                                -> Result<SpecifiedValue, ParseError<'i>> {
        if input.try(|input| input.expect_ident_matching("none")).is_ok() {
            return Ok(SpecifiedValue(Vec::new()))
        }

        let mut counters = Vec::new();
        loop {
            let counter_name = match input.next() {
                Ok(&Token::Ident(ref ident)) => CustomIdent::from_ident(ident, &["none"])?,
                Ok(t) => return Err(BasicParseError::UnexpectedToken(t.clone()).into()),
                Err(_) => break,
            };
            let counter_delta = input.try(|input| specified::parse_integer(context, input))
                                     .unwrap_or(specified::Integer::new(default_value));
            counters.push((counter_name, counter_delta))
        }

        if !counters.is_empty() {
            Ok(SpecifiedValue(counters))
        } else {
            Err(StyleParseError::UnspecifiedError.into())
        }
    }

        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::CounterIncrement(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::CounterIncrement);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::CounterIncrement);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_counter_increment(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_counter_increment();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_counter_increment();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::CounterIncrement)
            }
    }



    
    /// https://drafts.csswg.org/css-lists-3/#propdef-counter-reset
    pub mod counter_reset {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
    pub use super::counter_increment::{SpecifiedValue, computed_value, get_initial_value};
    use super::counter_increment::parse_common;

    pub fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                         -> Result<SpecifiedValue,ParseError<'i>> {
        parse_common(context, 0, input)
    }

        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::CounterReset(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::CounterReset);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::CounterReset);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_counter_reset(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_counter_reset();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_counter_reset();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::CounterReset)
            }
    }


    /* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */



// Box-shadow, etc.



    
        
    
    /// https://drafts.csswg.org/css-color/#opacity
    pub mod opacity {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::Opacity as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::Opacity as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { 1.0 }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::Opacity::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::Opacity(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::Opacity);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::Opacity);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_opacity(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_opacity();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_opacity();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::Opacity)
            }
    }




    
        
    
    
    /// https://drafts.csswg.org/css-backgrounds/#box-shadow
    pub mod box_shadow {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        use std::fmt;
        use style_traits::{Separator, ToCss};

        pub mod single_value {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::ShorthandId;
            #[allow(unused_imports)]
            use selectors::parser::SelectorParseError;
            #[allow(unused_imports)]
            use style_traits::{ParseError, StyleParseError};
            #[allow(unused_imports)]
            use values::computed::{Context, ToComputedValue};
            #[allow(unused_imports)]
            use values::{computed, specified};
            #[allow(unused_imports)]
            use values::{Auto, Either, None_, Normal};
            
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::BoxShadow as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::BoxShadow as T;
        }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::BoxShadow::parse(context, input)
        }
    
        
        }

        /// The definition of the computed value for box-shadow.
        pub mod computed_value {
            pub use super::single_value::computed_value as single_value;
            pub use self::single_value::T as SingleComputedValue;
            use std::vec::IntoIter;
            use values::computed::ComputedVecIter;

            /// The computed value, effectively a list of single values.
            #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
            #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
            #[derive(Clone, Debug, PartialEq)]
            pub struct T(
                pub Vec<single_value::T>,
            );


            pub type Iter<'a, 'cx, 'cx_a> = ComputedVecIter<'a, 'cx, 'cx_a, super::single_value::SpecifiedValue>;

            impl IntoIterator for T {
                type Item = single_value::T;
                type IntoIter = IntoIter<single_value::T>;
                fn into_iter(self) -> Self::IntoIter {
                    self.0.into_iter()
                }
            }
        }

        impl ToCss for computed_value::T {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        dest.write_str("none")?;
                }
                for i in iter {
                    dest.write_str(::style_traits::Comma::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        /// The specified value of box-shadow.
        #[derive(Clone, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        pub struct SpecifiedValue(pub Vec<single_value::SpecifiedValue>);

        impl ToCss for SpecifiedValue {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        dest.write_str("none")?;
                }
                for i in iter {
                    dest.write_str(::style_traits::Comma::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        pub fn get_initial_value() -> computed_value::T {
                computed_value::T(vec![])
        }

        pub fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                -> Result<SpecifiedValue, ParseError<'i>> {
            use style_traits::Separator;

                if input.try(|input| input.expect_ident_matching("none")).is_ok() {
                    return Ok(SpecifiedValue(Vec::new()))
                }

            ::style_traits::Comma::parse(input, |parser| {
                single_value::parse(context, parser)
            }).map(SpecifiedValue)
        }

        pub use self::single_value::SpecifiedValue as SingleSpecifiedValue;

        impl SpecifiedValue {
            pub fn compute_iter<'a, 'cx, 'cx_a>(&'a self, context: &'cx Context<'cx_a>)
                -> computed_value::Iter<'a, 'cx, 'cx_a> {
                computed_value::Iter::new(context, &self.0)
            }
        }

        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, context: &Context) -> computed_value::T {
                computed_value::T(self.compute_iter(context).collect())
            }

            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                SpecifiedValue(computed.0.iter()
                                    .map(ToComputedValue::from_computed_value)
                                    .collect())
            }
        }
    
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::BoxShadow(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::BoxShadow);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::BoxShadow);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            // In the case of a vector property we want to pass
                            // down an iterator so that this can be computed
                            // without allocation
                            //
                            // However, computing requires a context, but the
                            // style struct being mutated is on the context. We
                            // temporarily remove it, mutate it, and then put it
                            // back. Vector longhands cannot touch their own
                            // style struct whilst computing, else this will
                            // panic.
                            let mut s =
                                context.builder.take_effects();
                            {
                                let iter = specified_value.compute_iter(context);
                                s.set_box_shadow(iter);
                            }
                            context.builder.put_effects(s);
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_box_shadow();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_box_shadow();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::BoxShadow)
            }
    }





    
        
    
    /// https://drafts.fxtf.org/css-masking/#clip-property
    pub mod clip {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::ClipRectOrAuto as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::ClipRectOrAuto as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::ClipRectOrAuto::auto() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::ClipRectOrAuto::parse_quirky(context, input, AllowQuirks::Yes)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::Clip(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::Clip);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::Clip);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = (**specified_value).to_computed_value(context);
                                context.builder.set_clip(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_clip();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_clip();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<Box<SpecifiedValue>, ParseError<'i>> {
                    parse(context, input).map(|result| Box::new(result))
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::Clip)
            }
    }




    
        
    
    
    /// https://drafts.fxtf.org/filters/#propdef-filter
    pub mod filter {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        use std::fmt;
        use style_traits::{Separator, ToCss};

        pub mod single_value {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::ShorthandId;
            #[allow(unused_imports)]
            use selectors::parser::SelectorParseError;
            #[allow(unused_imports)]
            use style_traits::{ParseError, StyleParseError};
            #[allow(unused_imports)]
            use values::computed::{Context, ToComputedValue};
            #[allow(unused_imports)]
            use values::{computed, specified};
            #[allow(unused_imports)]
            use values::{Auto, Either, None_, Normal};
            
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::Filter as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::Filter as T;
        }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::Filter::parse(context, input)
        }
    
        
        }

        /// The definition of the computed value for filter.
        pub mod computed_value {
            pub use super::single_value::computed_value as single_value;
            pub use self::single_value::T as SingleComputedValue;
            use std::vec::IntoIter;
            use values::computed::ComputedVecIter;

            /// The computed value, effectively a list of single values.
            #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
            #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
            #[derive(Clone, Debug, PartialEq)]
            pub struct T(
                pub Vec<single_value::T>,
            );


            pub type Iter<'a, 'cx, 'cx_a> = ComputedVecIter<'a, 'cx, 'cx_a, super::single_value::SpecifiedValue>;

            impl IntoIterator for T {
                type Item = single_value::T;
                type IntoIter = IntoIter<single_value::T>;
                fn into_iter(self) -> Self::IntoIter {
                    self.0.into_iter()
                }
            }
        }

        impl ToCss for computed_value::T {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        dest.write_str("none")?;
                }
                for i in iter {
                    dest.write_str(::style_traits::Space::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        /// The specified value of filter.
        #[derive(Clone, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        pub struct SpecifiedValue(pub Vec<single_value::SpecifiedValue>);

        impl ToCss for SpecifiedValue {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        dest.write_str("none")?;
                }
                for i in iter {
                    dest.write_str(::style_traits::Space::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        pub fn get_initial_value() -> computed_value::T {
                computed_value::T(vec![])
        }

        pub fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                -> Result<SpecifiedValue, ParseError<'i>> {
            use style_traits::Separator;

                if input.try(|input| input.expect_ident_matching("none")).is_ok() {
                    return Ok(SpecifiedValue(Vec::new()))
                }

            ::style_traits::Space::parse(input, |parser| {
                single_value::parse(context, parser)
            }).map(SpecifiedValue)
        }

        pub use self::single_value::SpecifiedValue as SingleSpecifiedValue;

        impl SpecifiedValue {
            pub fn compute_iter<'a, 'cx, 'cx_a>(&'a self, context: &'cx Context<'cx_a>)
                -> computed_value::Iter<'a, 'cx, 'cx_a> {
                computed_value::Iter::new(context, &self.0)
            }
        }

        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, context: &Context) -> computed_value::T {
                computed_value::T(self.compute_iter(context).collect())
            }

            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                SpecifiedValue(computed.0.iter()
                                    .map(ToComputedValue::from_computed_value)
                                    .collect())
            }
        }
    
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::Filter(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::Filter);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::Filter);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            // In the case of a vector property we want to pass
                            // down an iterator so that this can be computed
                            // without allocation
                            //
                            // However, computing requires a context, but the
                            // style struct being mutated is on the context. We
                            // temporarily remove it, mutate it, and then put it
                            // back. Vector longhands cannot touch their own
                            // style struct whilst computing, else this will
                            // panic.
                            let mut s =
                                context.builder.take_effects();
                            {
                                let iter = specified_value.compute_iter(context);
                                s.set_filter(iter);
                            }
                            context.builder.put_effects(s);
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_filter();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_filter();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::Filter)
            }
    }





    
    

    
        
    
    /// https://drafts.fxtf.org/compositing/#propdef-mix-blend-mode
    pub mod mix_blend_mode {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "normal" => normal,
                    "multiply" => multiply,
                    "screen" => screen,
                    "overlay" => overlay,
                    "darken" => darken,
                    "lighten" => lighten,
                    "color-dodge" => color_dodge,
                    "color-burn" => color_burn,
                    "hard-light" => hard_light,
                    "soft-light" => soft_light,
                    "difference" => difference,
                    "exclusion" => exclusion,
                    "hue" => hue,
                    "saturation" => saturation,
                    "color" => color,
                    "luminosity" => luminosity,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::normal
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::normal
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::normal => computed_value::T::normal,
                        SpecifiedValue::multiply => computed_value::T::multiply,
                        SpecifiedValue::screen => computed_value::T::screen,
                        SpecifiedValue::overlay => computed_value::T::overlay,
                        SpecifiedValue::darken => computed_value::T::darken,
                        SpecifiedValue::lighten => computed_value::T::lighten,
                        SpecifiedValue::color_dodge => computed_value::T::color_dodge,
                        SpecifiedValue::color_burn => computed_value::T::color_burn,
                        SpecifiedValue::hard_light => computed_value::T::hard_light,
                        SpecifiedValue::soft_light => computed_value::T::soft_light,
                        SpecifiedValue::difference => computed_value::T::difference,
                        SpecifiedValue::exclusion => computed_value::T::exclusion,
                        SpecifiedValue::hue => computed_value::T::hue,
                        SpecifiedValue::saturation => computed_value::T::saturation,
                        SpecifiedValue::color => computed_value::T::color,
                        SpecifiedValue::luminosity => computed_value::T::luminosity,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::normal => SpecifiedValue::normal,
                        computed_value::T::multiply => SpecifiedValue::multiply,
                        computed_value::T::screen => SpecifiedValue::screen,
                        computed_value::T::overlay => SpecifiedValue::overlay,
                        computed_value::T::darken => SpecifiedValue::darken,
                        computed_value::T::lighten => SpecifiedValue::lighten,
                        computed_value::T::color_dodge => SpecifiedValue::color_dodge,
                        computed_value::T::color_burn => SpecifiedValue::color_burn,
                        computed_value::T::hard_light => SpecifiedValue::hard_light,
                        computed_value::T::soft_light => SpecifiedValue::soft_light,
                        computed_value::T::difference => SpecifiedValue::difference,
                        computed_value::T::exclusion => SpecifiedValue::exclusion,
                        computed_value::T::hue => SpecifiedValue::hue,
                        computed_value::T::saturation => SpecifiedValue::saturation,
                        computed_value::T::color => SpecifiedValue::color,
                        computed_value::T::luminosity => SpecifiedValue::luminosity,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MixBlendMode(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MixBlendMode);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::MixBlendMode);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_mix_blend_mode(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_mix_blend_mode();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_mix_blend_mode();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MixBlendMode)
            }
    }




    /* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */








#[cfg(feature = "gecko")]
macro_rules! impl_gecko_keyword_conversions {
    ($name: ident, $utype: ty) => {
        impl From<$utype> for $name {
            fn from(bits: $utype) -> $name {
                $name::from_gecko_keyword(bits)
            }
        }

        impl From<$name> for $utype {
            fn from(v: $name) -> $utype {
                v.to_gecko_keyword()
            }
        }
    };
}

// Define ToComputedValue, ToCss, and other boilerplate for a specified value
// which is of the form `enum SpecifiedValue {Value(..), System(SystemFont)}`



    
    /// https://drafts.csswg.org/css-fonts/#propdef-font-family
    pub mod font_family {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
    use properties::longhands::system_font::SystemFont;
    use self::computed_value::{FontFamily, FamilyName};
    use std::fmt;
    use style_traits::ToCss;


    pub mod computed_value {
        use cssparser::{CssStringWriter, Parser, serialize_identifier};
        use std::fmt::{self, Write};
        use Atom;
        use style_traits::{ToCss, ParseError};
        pub use self::FontFamily as SingleComputedValue;

        #[derive(Clone, Debug, Eq, Hash, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf, Deserialize, Serialize))]
        pub enum FontFamily {
            FamilyName(FamilyName),
            Generic(Atom),
        }

        #[derive(Clone, Debug, Eq, Hash, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf, Deserialize, Serialize))]
        pub struct FamilyName {
            pub name: Atom,
            pub syntax: FamilyNameSyntax,
        }

        #[derive(Clone, Debug, Eq, Hash, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf, Deserialize, Serialize))]
        pub enum FamilyNameSyntax {
            /// The family name was specified in a quoted form, e.g. "Font Name"
            /// or 'Font Name'.
            Quoted,

            /// The family name was specified in an unquoted form as a sequence of
            /// identifiers.  The `String` is the serialization of the sequence of
            /// identifiers.
            Identifiers(String),
        }

        impl FontFamily {
            #[inline]
            pub fn atom(&self) -> &Atom {
                match *self {
                    FontFamily::FamilyName(ref family_name) => &family_name.name,
                    FontFamily::Generic(ref name) => name,
                }
            }

            #[inline]
            #[cfg(not(feature = "gecko"))] // Gecko can't borrow atoms as UTF-8.
            pub fn name(&self) -> &str {
                self.atom()
            }

            #[cfg(not(feature = "gecko"))] // Gecko can't borrow atoms as UTF-8.
            pub fn from_atom(input: Atom) -> FontFamily {
                match input {
                    atom!("serif") |
                    atom!("sans-serif") |
                    atom!("cursive") |
                    atom!("fantasy") |
                    atom!("monospace") => {
                        return FontFamily::Generic(input)
                    }
                    _ => {}
                }
                match_ignore_ascii_case! { &input,
                    "serif" => return FontFamily::Generic(atom!("serif")),
                    "sans-serif" => return FontFamily::Generic(atom!("sans-serif")),
                    "cursive" => return FontFamily::Generic(atom!("cursive")),
                    "fantasy" => return FontFamily::Generic(atom!("fantasy")),
                    "monospace" => return FontFamily::Generic(atom!("monospace")),
                    _ => {}
                }

                // We don't know if it's quoted or not. So we set it to
                // quoted by default.
                FontFamily::FamilyName(FamilyName {
                    name: input,
                    syntax: FamilyNameSyntax::Quoted,
                })
            }

            /// Parse a font-family value
            pub fn parse<'i, 't>(input: &mut Parser<'i, 't>) -> Result<Self, ParseError<'i>> {
                if let Ok(value) = input.try(|i| i.expect_string_cloned()) {
                    return Ok(FontFamily::FamilyName(FamilyName {
                        name: Atom::from(&*value),
                        syntax: FamilyNameSyntax::Quoted,
                    }))
                }
                let first_ident = input.expect_ident()?.clone();

                // FIXME(bholley): The fast thing to do here would be to look up the
                // string (as lowercase) in the static atoms table. We don't have an
                // API to do that yet though, so we do the simple thing for now.
                let mut css_wide_keyword = false;
                match_ignore_ascii_case! { &first_ident,
                    "serif" => return Ok(FontFamily::Generic(atom!("serif"))),
                    "sans-serif" => return Ok(FontFamily::Generic(atom!("sans-serif"))),
                    "cursive" => return Ok(FontFamily::Generic(atom!("cursive"))),
                    "fantasy" => return Ok(FontFamily::Generic(atom!("fantasy"))),
                    "monospace" => return Ok(FontFamily::Generic(atom!("monospace"))),
                        "-moz-fixed" => return Ok(FontFamily::Generic(atom!("-moz-fixed"))),

                    // https://drafts.csswg.org/css-fonts/#propdef-font-family
                    // "Font family names that happen to be the same as a keyword value
                    //  (`inherit`, `serif`, `sans-serif`, `monospace`, `fantasy`, and `cursive`)
                    //  must be quoted to prevent confusion with the keywords with the same names.
                    //  The keywords ‘initial’ and ‘default’ are reserved for future use
                    //  and must also be quoted when used as font names.
                    //  UAs must not consider these keywords as matching the <family-name> type."
                    "inherit" => css_wide_keyword = true,
                    "initial" => css_wide_keyword = true,
                    "unset" => css_wide_keyword = true,
                    "default" => css_wide_keyword = true,
                    _ => {}
                }

                let mut value = first_ident.as_ref().to_owned();
                let mut serialization = String::new();
                serialize_identifier(&first_ident, &mut serialization).unwrap();

                // These keywords are not allowed by themselves.
                // The only way this value can be valid with with another keyword.
                if css_wide_keyword {
                    let ident = input.expect_ident()?;
                    value.push(' ');
                    value.push_str(&ident);
                    serialization.push(' ');
                    serialize_identifier(&ident, &mut serialization).unwrap();
                }
                while let Ok(ident) = input.try(|i| i.expect_ident_cloned()) {
                    value.push(' ');
                    value.push_str(&ident);
                    serialization.push(' ');
                    serialize_identifier(&ident, &mut serialization).unwrap();
                }
                Ok(FontFamily::FamilyName(FamilyName {
                    name: Atom::from(value),
                    syntax: FamilyNameSyntax::Identifiers(serialization),
                }))
            }

            #[cfg(feature = "gecko")]
            /// Return the generic ID for a given generic font name
            pub fn generic(name: &Atom) -> (::gecko_bindings::structs::FontFamilyType, u8) {
                use gecko_bindings::structs::{self, FontFamilyType};
                if *name == atom!("serif") {
                    (FontFamilyType::eFamily_serif,
                     structs::kGenericFont_serif)
                } else if *name == atom!("sans-serif") {
                    (FontFamilyType::eFamily_sans_serif,
                     structs::kGenericFont_sans_serif)
                } else if *name == atom!("cursive") {
                    (FontFamilyType::eFamily_cursive,
                     structs::kGenericFont_cursive)
                } else if *name == atom!("fantasy") {
                    (FontFamilyType::eFamily_fantasy,
                     structs::kGenericFont_fantasy)
                } else if *name == atom!("monospace") {
                    (FontFamilyType::eFamily_monospace,
                     structs::kGenericFont_monospace)
                } else if *name == atom!("-moz-fixed") {
                    (FontFamilyType::eFamily_moz_fixed,
                     structs::kGenericFont_moz_fixed)
                } else {
                    panic!("Unknown generic {}", name);
                }
            }
        }

        impl ToCss for FamilyName {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
                match self.syntax {
                    FamilyNameSyntax::Quoted => {
                        dest.write_char('"')?;
                        write!(CssStringWriter::new(dest), "{}", self.name)?;
                        dest.write_char('"')
                    }
                    FamilyNameSyntax::Identifiers(ref serialization) => {
                        // Note that `serialization` is already escaped/
                        // serialized appropriately.
                        dest.write_str(&*serialization)
                    }
                }
            }
        }

        impl ToCss for FontFamily {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
                match *self {
                    FontFamily::FamilyName(ref name) => name.to_css(dest),

                    // All generic values accepted by the parser are known to not require escaping.
                    FontFamily::Generic(ref name) => {
                            // We should treat -moz-fixed as monospace
                            if name == &atom!("-moz-fixed") {
                                return dest.write_str("monospace");
                            }

                        write!(dest, "{}", name)
                    },
                }
            }
        }

        impl ToCss for T {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
                let mut iter = self.0.iter();
                iter.next().unwrap().to_css(dest)?;
                for family in iter {
                    dest.write_str(", ")?;
                    family.to_css(dest)?;
                }
                Ok(())
            }
        }

        #[derive(Clone, Debug, Eq, Hash, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        pub struct T(pub Vec<FontFamily>);
    }

    #[inline]
    pub fn get_initial_value() -> computed_value::T {
        computed_value::T(vec![FontFamily::Generic(atom!("serif"))])
    }

    /// <family-name>#
    /// <family-name> = <string> | [ <ident>+ ]
    /// TODO: <generic-family>
    pub fn parse<'i, 't>(_: &ParserContext, input: &mut Parser<'i, 't>)
                         -> Result<SpecifiedValue, ParseError<'i>> {
        SpecifiedValue::parse(input)
    }

    #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
    #[derive(Clone, Debug, Eq, Hash, PartialEq)]
    pub enum SpecifiedValue {
        Values(Vec<FontFamily>),
        System(SystemFont),
    }

    #[cfg(feature = "gecko")]
    impl SpecifiedValue {
        /// Return the generic ID if it is a single generic font
        pub fn single_generic(&self) -> Option<u8> {
            if let SpecifiedValue::Values(ref values) = *self {
                if values.len() == 1 {
                    if let FontFamily::Generic(ref name) = values[0] {
                        return Some(FontFamily::generic(name).1);
                    }
                }
            }
            None
        }
    }

    impl ToComputedValue for SpecifiedValue {
        type ComputedValue = computed_value::T;
        fn to_computed_value(&self, _cx: &Context) -> Self::ComputedValue {
            match *self {
                SpecifiedValue::Values(ref v) => computed_value::T(v.clone()),
                SpecifiedValue::System(_) => {
                    
        
                        _cx.cached_system_font.as_ref().unwrap().font_family.clone()
                    

                }
            }
        }
        fn from_computed_value(other: &computed_value::T) -> Self {
            SpecifiedValue::Values(other.0.clone())
        }
    }

    impl SpecifiedValue {
        pub fn system_font(f: SystemFont) -> Self {
            SpecifiedValue::System(f)
        }
        pub fn get_system(&self) -> Option<SystemFont> {
            if let SpecifiedValue::System(s) = *self {
                Some(s)
            } else {
                None
            }
        }

        pub fn parse<'i, 't>(input: &mut Parser<'i, 't>) -> Result<Self, ParseError<'i>> {
            input.parse_comma_separated(|input| FontFamily::parse(input)).map(SpecifiedValue::Values)
        }
    }


    impl ToCss for SpecifiedValue {
        fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
            match *self {
                SpecifiedValue::Values(ref v) => {
                    let mut iter = v.iter();
                    iter.next().unwrap().to_css(dest)?;
                    for family in iter {
                        dest.write_str(", ")?;
                        family.to_css(dest)?;
                    }
                    Ok(())
                }
                SpecifiedValue::System(sys) => sys.to_css(dest),
            }
        }
    }

    /// `FamilyName::parse` is based on `FontFamily::parse` and not the other way around
    /// because we want the former to exclude generic family keywords.
    impl Parse for FamilyName {
        fn parse<'i, 't>(_: &ParserContext, input: &mut Parser<'i, 't>) -> Result<Self, ParseError<'i>> {
            match FontFamily::parse(input) {
                Ok(FontFamily::FamilyName(name)) => Ok(name),
                Ok(FontFamily::Generic(_)) => Err(StyleParseError::UnspecifiedError.into()),
                Err(e) => Err(e)
            }
        }
    }

        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::FontFamily(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::FontFamily);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            if let Some(sf) = specified_value.get_system() {
                                longhands::system_font::resolve_system_font(sf, context);
                            }
                            let computed = (**specified_value).to_computed_value(context);
                                context.builder.set_font_family(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_font_family();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_font_family();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<Box<SpecifiedValue>, ParseError<'i>> {
                    parse(context, input).map(|result| Box::new(result))
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::FontFamily)
            }
    }



    
    
    
    /// https://drafts.csswg.org/css-fonts/#propdef-font-style
    pub mod font_style {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
        use properties::longhands::system_font::SystemFont;

        pub mod computed_value {
            use cssparser::Parser;
            use parser::{Parse, ParserContext};

            use style_traits::{ToCss, ParseError};
            define_css_keyword_enum! { T:
                    "normal" => normal,
                    "italic" => italic,
                    "oblique" => oblique,
            }

            impl Parse for T {
                fn parse<'i, 't>(_: &ParserContext, input: &mut Parser<'i, 't>) -> Result<Self, ParseError<'i>> {
                    T::parse(input)
                }
            }

            
    
    #[cfg(feature = "gecko")]
    impl T {
        /// Obtain a specified value from a Gecko keyword value
        ///
        /// Intended for use with presentation attributes, not style structs
        pub fn from_gecko_keyword(kw: u32) -> Self {
            use gecko_bindings::structs;
                // We can't match on enum values if we're matching on a u32
                const NORMAL: i32
                    = structs::NS_FONT_STYLE_NORMAL as i32;
                // We can't match on enum values if we're matching on a u32
                const ITALIC: i32
                    = structs::NS_FONT_STYLE_ITALIC as i32;
                // We can't match on enum values if we're matching on a u32
                const OBLIQUE: i32
                    = structs::NS_FONT_STYLE_OBLIQUE as i32;
            match kw as i32 {
                    NORMAL => T::normal,
                    ITALIC => T::italic,
                    OBLIQUE => T::oblique,
                x => panic!("Found unexpected value in style struct for font-style property: {:?}", x),
            }
        }
    }

        }

        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[derive(Clone, Copy, Debug, Eq, PartialEq, ToCss)]
        pub enum SpecifiedValue {
            Keyword(computed_value::T),
            System(SystemFont),
        }

        pub fn parse<'i, 't>(_: &ParserContext, input: &mut Parser<'i, 't>) -> Result<SpecifiedValue, ParseError<'i>> {
            Ok(SpecifiedValue::Keyword(computed_value::T::parse(input)?))
        }

        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;
            fn to_computed_value(&self, _cx: &Context) -> Self::ComputedValue {
                match *self {
                    SpecifiedValue::Keyword(v) => v,
                    SpecifiedValue::System(_) => {
                            _cx.cached_system_font.as_ref().unwrap().font_style
                    }
                }
            }
            fn from_computed_value(other: &computed_value::T) -> Self {
                SpecifiedValue::Keyword(*other)
            }
        }

        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::normal
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::Keyword(computed_value::T::normal)
        }

        impl SpecifiedValue {
            pub fn system_font(f: SystemFont) -> Self {
                SpecifiedValue::System(f)
            }
            pub fn get_system(&self) -> Option<SystemFont> {
                if let SpecifiedValue::System(s) = *self {
                    Some(s)
                } else {
                    None
                }
            }
        }
    
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::FontStyle(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::FontStyle);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            if let Some(sf) = specified_value.get_system() {
                                longhands::system_font::resolve_system_font(sf, context);
                            }
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_font_style(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_font_style();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_font_style();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::FontStyle)
            }
    }







    
    
    
    /// https://drafts.csswg.org/css-fonts/#propdef-font-variant-caps
    pub mod font_variant_caps {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
        use properties::longhands::system_font::SystemFont;

        pub mod computed_value {
            use cssparser::Parser;
            use parser::{Parse, ParserContext};

            use style_traits::{ToCss, ParseError};
            define_css_keyword_enum! { T:
                    "normal" => normal,
                    "small-caps" => small_caps,
                    "all-small-caps" => all_small_caps,
                    "petite-caps" => petite_caps,
                    "all-petite-caps" => all_petite_caps,
                    "unicase" => unicase,
                    "titling-caps" => titling_caps,
            }

            impl Parse for T {
                fn parse<'i, 't>(_: &ParserContext, input: &mut Parser<'i, 't>) -> Result<Self, ParseError<'i>> {
                    T::parse(input)
                }
            }

            
    
    #[cfg(feature = "gecko")]
    impl T {
        /// Obtain a specified value from a Gecko keyword value
        ///
        /// Intended for use with presentation attributes, not style structs
        pub fn from_gecko_keyword(kw: u32) -> Self {
            use gecko_bindings::structs;
                // We can't match on enum values if we're matching on a u32
                const NORMAL: i32
                    = structs::NS_FONT_VARIANT_CAPS_NORMAL as i32;
                // We can't match on enum values if we're matching on a u32
                const SMALL_CAPS: i32
                    = structs::NS_FONT_VARIANT_CAPS_SMALLCAPS as i32;
                // We can't match on enum values if we're matching on a u32
                const ALL_SMALL_CAPS: i32
                    = structs::NS_FONT_VARIANT_CAPS_ALLSMALL as i32;
                // We can't match on enum values if we're matching on a u32
                const PETITE_CAPS: i32
                    = structs::NS_FONT_VARIANT_CAPS_PETITECAPS as i32;
                // We can't match on enum values if we're matching on a u32
                const ALL_PETITE_CAPS: i32
                    = structs::NS_FONT_VARIANT_CAPS_ALLPETITE as i32;
                // We can't match on enum values if we're matching on a u32
                const UNICASE: i32
                    = structs::NS_FONT_VARIANT_CAPS_UNICASE as i32;
                // We can't match on enum values if we're matching on a u32
                const TITLING_CAPS: i32
                    = structs::NS_FONT_VARIANT_CAPS_TITLING as i32;
            match kw as i32 {
                    NORMAL => T::normal,
                    SMALL_CAPS => T::small_caps,
                    ALL_SMALL_CAPS => T::all_small_caps,
                    PETITE_CAPS => T::petite_caps,
                    ALL_PETITE_CAPS => T::all_petite_caps,
                    UNICASE => T::unicase,
                    TITLING_CAPS => T::titling_caps,
                x => panic!("Found unexpected value in style struct for font-variant-caps property: {:?}", x),
            }
        }
    }

        }

        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[derive(Clone, Copy, Debug, Eq, PartialEq, ToCss)]
        pub enum SpecifiedValue {
            Keyword(computed_value::T),
            System(SystemFont),
        }

        pub fn parse<'i, 't>(_: &ParserContext, input: &mut Parser<'i, 't>) -> Result<SpecifiedValue, ParseError<'i>> {
            Ok(SpecifiedValue::Keyword(computed_value::T::parse(input)?))
        }

        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;
            fn to_computed_value(&self, _cx: &Context) -> Self::ComputedValue {
                match *self {
                    SpecifiedValue::Keyword(v) => v,
                    SpecifiedValue::System(_) => {
                            _cx.cached_system_font.as_ref().unwrap().font_variant_caps
                    }
                }
            }
            fn from_computed_value(other: &computed_value::T) -> Self {
                SpecifiedValue::Keyword(*other)
            }
        }

        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::normal
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::Keyword(computed_value::T::normal)
        }

        impl SpecifiedValue {
            pub fn system_font(f: SystemFont) -> Self {
                SpecifiedValue::System(f)
            }
            pub fn get_system(&self) -> Option<SystemFont> {
                if let SpecifiedValue::System(s) = *self {
                    Some(s)
                } else {
                    None
                }
            }
        }
    
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::FontVariantCaps(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::FontVariantCaps);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            if let Some(sf) = specified_value.get_system() {
                                longhands::system_font::resolve_system_font(sf, context);
                            }
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_font_variant_caps(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_font_variant_caps();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_font_variant_caps();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::FontVariantCaps)
            }
    }




    
    /// https://drafts.csswg.org/css-fonts/#propdef-font-weight
    pub mod font_weight {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
    use properties::longhands::system_font::SystemFont;


    #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
    #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
    #[derive(Clone, Copy, Debug, Eq, PartialEq, ToCss)]
    pub enum SpecifiedValue {
        Normal,
        Bold,
        Bolder,
        Lighter,
        Weight(computed_value::T),
        System(SystemFont),
    }

    /// normal | bold | bolder | lighter | 100 | 200 | 300 | 400 | 500 | 600 | 700 | 800 | 900
    pub fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                         -> Result<SpecifiedValue, ParseError<'i>> {
        let result = input.try(|input| {
            let ident = input.expect_ident().map_err(|_| ())?;
            match_ignore_ascii_case! { &ident,
                "normal" => Ok(SpecifiedValue::Normal),
                "bold" => Ok(SpecifiedValue::Bold),
                "bolder" => Ok(SpecifiedValue::Bolder),
                "lighter" => Ok(SpecifiedValue::Lighter),
                _ => Err(())
            }
        });
        result.or_else(|_| computed_value::T::parse(context, input).map(SpecifiedValue::Weight))
    }

    impl SpecifiedValue {
        pub fn from_gecko_keyword(kw: u32) -> Self {
            computed_value::T::from_int(kw as i32).map(SpecifiedValue::Weight)
                .expect("Found unexpected value in style struct for font-weight property")
        }
    }

    impl SpecifiedValue {
        pub fn system_font(f: SystemFont) -> Self {
            SpecifiedValue::System(f)
        }
        pub fn get_system(&self) -> Option<SystemFont> {
            if let SpecifiedValue::System(s) = *self {
                Some(s)
            } else {
                None
            }
        }
    }

    pub mod computed_value {
        /// As of CSS Fonts Module Level 3, only the following values are
        /// valid: 100 | 200 | 300 | 400 | 500 | 600 | 700 | 800 | 900
        ///
        /// However, system fonts may provide other values. Pango
        /// may provide 350, 380, and 1000 (on top of the existing values), for example.
        #[derive(Clone, ComputeSquaredDistance, Copy, Debug, Eq, Hash, PartialEq, ToCss)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf, Deserialize, Serialize))]
        pub struct T(pub u16);

        impl T {
            /// Value for normal
            pub fn normal() -> Self {
                T(400)
            }

            /// Value for bold
            pub fn bold() -> Self {
                T(700)
            }

            /// Convert from an integer to Weight
            pub fn from_int(n: i32) -> Result<Self, ()> {
                if n >= 100 && n <= 900 && n % 100 == 0 {
                    Ok(T(n as u16))
                } else {
                    Err(())
                }
            }

            /// Convert from an Gecko weight
            pub fn from_gecko_weight(weight: u16) -> Self {
                // we allow a wider range of weights than is parseable
                // because system fonts may provide custom values
                T(weight)
            }

            /// Weither this weight is bold
            pub fn is_bold(&self) -> bool {
                self.0 > 500
            }

            /// Return the bolder weight
            pub fn bolder(self) -> Self {
                if self.0 < 400 {
                    T(400)
                } else if self.0 < 600 {
                    T(700)
                } else {
                    T(900)
                }
            }

            /// Returns the lighter weight
            pub fn lighter(self) -> Self {
                if self.0 < 600 {
                    T(100)
                } else if self.0 < 800 {
                    T(400)
                } else {
                    T(700)
                }
            }
        }
    }

    impl Parse for computed_value::T {
        fn parse<'i, 't>(_: &ParserContext, input: &mut Parser<'i, 't>)
            -> Result<Self, ParseError<'i>> {
                Self::from_int(input.expect_integer()?)
                    .map_err(|_| StyleParseError::UnspecifiedError.into())
            }
    }

    #[inline]
    pub fn get_initial_value() -> computed_value::T {
        computed_value::T::normal()
    }

    #[inline]
    pub fn get_initial_specified_value() -> SpecifiedValue {
        SpecifiedValue::Normal
    }

    impl ToComputedValue for SpecifiedValue {
        type ComputedValue = computed_value::T;

        #[inline]
        fn to_computed_value(&self, context: &Context) -> computed_value::T {
            match *self {
                SpecifiedValue::Weight(weight) => weight,
                SpecifiedValue::Normal => computed_value::T::normal(),
                SpecifiedValue::Bold => computed_value::T::bold(),
                SpecifiedValue::Bolder =>
                    context.builder.get_parent_font().clone_font_weight().bolder(),
                SpecifiedValue::Lighter =>
                    context.builder.get_parent_font().clone_font_weight().lighter(),
                SpecifiedValue::System(_) => {
                    
        
                        context.cached_system_font.as_ref().unwrap().font_weight.clone()
                    

                }
            }
        }

        #[inline]
        fn from_computed_value(computed: &computed_value::T) -> Self {
            SpecifiedValue::Weight(*computed)
        }
    }

        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::FontWeight(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::FontWeight);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            if let Some(sf) = specified_value.get_system() {
                                longhands::system_font::resolve_system_font(sf, context);
                            }
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_font_weight(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_font_weight();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_font_weight();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::FontWeight)
            }
    }



    
    /// https://drafts.csswg.org/css-fonts/#propdef-font-size
    pub mod font_size {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
    use app_units::Au;
    use properties::longhands::system_font::SystemFont;
    use std::fmt;
    use style_traits::ToCss;
    use values::FONT_MEDIUM_PX;
    use values::computed::NonNegativeLength;
    use values::specified::{AllowQuirks, FontRelativeLength, LengthOrPercentage, NoCalcLength};
    use values::specified::length::FontBaseSize;

    impl ToCss for SpecifiedValue {
        fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
            match *self {
                SpecifiedValue::Length(ref lop) => lop.to_css(dest),
                SpecifiedValue::Keyword(kw, _, _) => kw.to_css(dest),
                SpecifiedValue::Smaller => dest.write_str("smaller"),
                SpecifiedValue::Larger => dest.write_str("larger"),
                SpecifiedValue::System(sys) => sys.to_css(dest),
            }
        }
    }

    #[derive(Clone, Debug, PartialEq)]
    #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
    #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
    pub enum SpecifiedValue {
        Length(specified::LengthOrPercentage),
        /// A keyword value, along with a ratio and absolute offset.
        /// The ratio in any specified keyword value
        /// will be 1 (with offset 0), but we cascade keywordness even
        /// after font-relative (percent and em) values
        /// have been applied, which is where the ratio
        /// comes in. The offset comes in if we cascaded a calc value,
        /// where the font-relative portion (em and percentage) will
        /// go into the ratio, and the remaining units all computed together
        /// will go into the offset.
        /// See bug 1355707.
        Keyword(KeywordSize, f32, NonNegativeLength),
        Smaller,
        Larger,
        System(SystemFont)
    }

    impl From<specified::LengthOrPercentage> for SpecifiedValue {
        fn from(other: specified::LengthOrPercentage) -> Self {
            SpecifiedValue::Length(other)
        }
    }

    pub mod computed_value {
        use values::computed::NonNegativeLength;
        pub type T = NonNegativeLength;
    }

    /// CSS font keywords
    #[derive(Clone, Copy, Debug, PartialEq)]
    #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
    #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
    pub enum KeywordSize {
        XXSmall = 1, // This is to enable the NonZero optimization
                     // which simplifies the representation of Option<KeywordSize>
                     // in bindgen
        XSmall,
        Small,
        Medium,
        Large,
        XLarge,
        XXLarge,
        // This is not a real font keyword and will not parse
        // HTML font-size 7 corresponds to this value
        XXXLarge,
    }

    pub use self::KeywordSize::*;

    impl KeywordSize {
        pub fn parse<'i, 't>(input: &mut Parser<'i, 't>) -> Result<Self, ParseError<'i>> {
            try_match_ident_ignore_ascii_case! { input.expect_ident()?,
                "xx-small" => Ok(XXSmall),
                "x-small" => Ok(XSmall),
                "small" => Ok(Small),
                "medium" => Ok(Medium),
                "large" => Ok(Large),
                "x-large" => Ok(XLarge),
                "xx-large" => Ok(XXLarge),
            }
        }

        pub fn html_size(&self) -> u8 {
            match *self {
                KeywordSize::XXSmall => 0,
                KeywordSize::XSmall => 1,
                KeywordSize::Small => 2,
                KeywordSize::Medium => 3,
                KeywordSize::Large => 4,
                KeywordSize::XLarge => 5,
                KeywordSize::XXLarge => 6,
                KeywordSize::XXXLarge => 7,
            }
        }
    }

    impl Default for KeywordSize {
        fn default() -> Self {
            Medium
        }
    }

    impl ToCss for KeywordSize {
        fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
            dest.write_str(match *self {
                XXSmall => "xx-small",
                XSmall => "x-small",
                Small => "small",
                Medium => "medium",
                Large => "large",
                XLarge => "x-large",
                XXLarge => "xx-large",
                XXXLarge => unreachable!("We should never serialize                                           specified values set via
                                          HTML presentation attributes"),
            })
        }
    }

        impl ToComputedValue for KeywordSize {
            type ComputedValue = NonNegativeLength;
            #[inline]
            fn to_computed_value(&self, cx: &Context) -> computed_value::T {
                use gecko_bindings::structs::nsIAtom;
                use values::specified::length::au_to_int_px;
                // Data from nsRuleNode.cpp in Gecko
                // Mapping from base size and HTML size to pixels
                // The first index is (base_size - 9), the second is the
                // HTML size. "0" is CSS keyword xx-small, not HTML size 0,
                // since HTML size 0 is the same as 1.
                //
                //  xxs   xs      s      m     l      xl     xxl   -
                //  -     0/1     2      3     4      5      6     7
                static FONT_SIZE_MAPPING: [[i32; 8]; 8] = [
                    [9,    9,     9,     9,    11,    14,    18,    27],
                    [9,    9,     9,    10,    12,    15,    20,    30],
                    [9,    9,    10,    11,    13,    17,    22,    33],
                    [9,    9,    10,    12,    14,    18,    24,    36],
                    [9,   10,    12,    13,    16,    20,    26,    39],
                    [9,   10,    12,    14,    17,    21,    28,    42],
                    [9,   10,    13,    15,    18,    23,    30,    45],
                    [9,   10,    13,    16,    18,    24,    32,    48]
                ];

                static FONT_SIZE_FACTORS: [i32; 8] = [60, 75, 89, 100, 120, 150, 200, 300];

                // XXXManishearth handle quirks mode

                let ref gecko_font = cx.style().get_font().gecko();
                let base_size = unsafe { Atom::with(gecko_font.mLanguage.raw::<nsIAtom>(), |atom| {
                    cx.font_metrics_provider.get_size(atom, gecko_font.mGenericID).0
                }) };

                let base_size_px = au_to_int_px(base_size as f32);
                let html_size = self.html_size() as usize;
                if base_size_px >= 9 && base_size_px <= 16 {
                    NonNegativeLength::new(FONT_SIZE_MAPPING[(base_size_px - 9) as usize][html_size] as f32)
                } else {
                    Au(FONT_SIZE_FACTORS[html_size] * base_size / 100).into()
                }
            }

            #[inline]
            fn from_computed_value(_: &computed_value::T) -> Self {
                unreachable!()
            }
        }

    /// This is the ratio applied for font-size: larger
    /// and smaller by both Firefox and Chrome
    const LARGER_FONT_SIZE_RATIO: f32 = 1.2;

    impl SpecifiedValue {
        /// https://html.spec.whatwg.org/multipage/#rules-for-parsing-a-legacy-font-size
        pub fn from_html_size(size: u8) -> Self {
            SpecifiedValue::Keyword(match size {
                // If value is less than 1, let it be 1.
                0 | 1 => XSmall,
                2 => Small,
                3 => Medium,
                4 => Large,
                5 => XLarge,
                6 => XXLarge,
                // If value is greater than 7, let it be 7.
                _ => XXXLarge,
            }, 1., Au(0).into())
        }

        /// If this value is specified as a ratio of the parent font (em units
        /// or percent) return the ratio
        pub fn as_font_ratio(&self, context: &Context) -> Option<(f32, NonNegativeLength)> {
            match *self {
                SpecifiedValue::Length(ref lop) => {
                    match *lop {
                        LengthOrPercentage::Percentage(pc) => {
                            Some((pc.0, NonNegativeLength::zero()))
                        }
                        LengthOrPercentage::Length(ref nocalc) => {
                            match *nocalc {
                                NoCalcLength::FontRelative(FontRelativeLength::Em(em)) => {
                                    Some((em, NonNegativeLength::zero()))
                                }
                                _ => None,
                            }
                        }
                        LengthOrPercentage::Calc(ref calc) => {
                            if calc.em.is_none() && calc.percentage.is_none() {
                                return None;
                            }
                            let ratio = calc.em.unwrap_or(0.) + calc.percentage.map_or(0., |pc| pc.0);
                            // Compute it, but shave off the font-relative part (em, %)
                            // This will mean that other font-relative units like ex and ch will be computed against
                            // the old font even when the font changes. There's no particular "right answer" for what
                            // to do here -- Gecko recascades as if the font had changed, we instead track the changes
                            // and reapply, which means that we carry over old computed ex/ch values whilst Gecko
                            // recomputes new ones. This is enough of an edge case to not really matter.
                            let abs = calc.to_computed_value_zoomed(context, FontBaseSize::Custom(Au(0)))
                                          .length_component().into();
                            Some((ratio, abs))
                        }
                    }
                }
                SpecifiedValue::Larger => Some((LARGER_FONT_SIZE_RATIO, Au(0).into())),
                SpecifiedValue::Smaller => Some((1. / LARGER_FONT_SIZE_RATIO, Au(0).into())),
                _ => None,
            }
        }

        /// Compute it against a given base font size
        pub fn to_computed_value_against(
            &self,
            context: &Context,
            base_size: FontBaseSize,
        ) -> NonNegativeLength {
            use values::specified::length::FontRelativeLength;
            match *self {
                SpecifiedValue::Length(LengthOrPercentage::Length(
                        NoCalcLength::FontRelative(value))) => {
                    value.to_computed_value(context, base_size).into()
                }
                SpecifiedValue::Length(LengthOrPercentage::Length(
                        NoCalcLength::ServoCharacterWidth(value))) => {
                    value.to_computed_value(base_size.resolve(context)).into()
                }
                SpecifiedValue::Length(LengthOrPercentage::Length(
                        NoCalcLength::Absolute(ref l))) => {
                    context.maybe_zoom_text(l.to_computed_value(context)).into()
                }
                SpecifiedValue::Length(LengthOrPercentage::Length(ref l)) => {
                    l.to_computed_value(context).into()
                }
                SpecifiedValue::Length(LengthOrPercentage::Percentage(pc)) => {
                    base_size.resolve(context).scale_by(pc.0).into()
                }
                SpecifiedValue::Length(LengthOrPercentage::Calc(ref calc)) => {
                    let calc = calc.to_computed_value_zoomed(context, base_size);
                    calc.to_used_value(Some(base_size.resolve(context))).unwrap().into()
                }
                SpecifiedValue::Keyword(ref key, fraction, offset) => {
                    let key_len = key.to_computed_value(context).scale_by(fraction) + offset;
                    context.maybe_zoom_text(key_len.0).into()
                }
                SpecifiedValue::Smaller => {
                    FontRelativeLength::Em(1. / LARGER_FONT_SIZE_RATIO)
                        .to_computed_value(context, base_size).into()
                }
                SpecifiedValue::Larger => {
                    FontRelativeLength::Em(LARGER_FONT_SIZE_RATIO)
                        .to_computed_value(context, base_size).into()
                }

                SpecifiedValue::System(_) => {
                    
        
                        context.cached_system_font.as_ref().unwrap().font_size
                    

                }
            }
        }
    }

    #[inline]
    #[allow(missing_docs)]
    pub fn get_initial_value() -> computed_value::T {
        NonNegativeLength::new(FONT_MEDIUM_PX as f32)
    }

    #[inline]
    pub fn get_initial_specified_value() -> SpecifiedValue {
        SpecifiedValue::Keyword(Medium, 1., Au(0).into())
    }


    impl ToComputedValue for SpecifiedValue {
        type ComputedValue = computed_value::T;

        #[inline]
        fn to_computed_value(&self, context: &Context) -> computed_value::T {
            self.to_computed_value_against(context, FontBaseSize::InheritedStyle)
        }

        #[inline]
        fn from_computed_value(computed: &computed_value::T) -> Self {
            SpecifiedValue::Length(LengthOrPercentage::Length(
                ToComputedValue::from_computed_value(&computed.0)
            ))
        }
    }

    /// <length> | <percentage> | <absolute-size> | <relative-size>
    pub fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                         -> Result<SpecifiedValue, ParseError<'i>> {
        parse_quirky(context, input, AllowQuirks::No)
    }

    /// Parses a font-size, with quirks.
    pub fn parse_quirky<'i, 't>(context: &ParserContext,
                                input: &mut Parser<'i, 't>,
                                allow_quirks: AllowQuirks)
                                -> Result<SpecifiedValue, ParseError<'i>> {
        use self::specified::LengthOrPercentage;
        if let Ok(lop) = input.try(|i| LengthOrPercentage::parse_non_negative_quirky(context, i, allow_quirks)) {
            return Ok(SpecifiedValue::Length(lop))
        }

        if let Ok(kw) = input.try(KeywordSize::parse) {
            return Ok(SpecifiedValue::Keyword(kw, 1., Au(0).into()))
        }

        try_match_ident_ignore_ascii_case! { input.expect_ident()?,
            "smaller" => Ok(SpecifiedValue::Smaller),
            "larger" => Ok(SpecifiedValue::Larger),
        }
    }

    impl SpecifiedValue {
        pub fn system_font(f: SystemFont) -> Self {
            SpecifiedValue::System(f)
        }
        pub fn get_system(&self) -> Option<SystemFont> {
            if let SpecifiedValue::System(s) = *self {
                Some(s)
            } else {
                None
            }
        }
    }

    #[allow(unused_mut)]
    pub fn cascade_specified_font_size(context: &mut Context,
                                       specified_value: &SpecifiedValue,
                                       mut computed: NonNegativeLength) {
        if let SpecifiedValue::Keyword(kw, fraction, offset) = *specified_value {
            context.builder.font_size_keyword = Some((kw, fraction, offset));
        } else if let Some((ratio, abs)) = specified_value.as_font_ratio(context) {
            // In case a font-size-relative value was applied to a keyword
            // value, we must preserve this fact in case the generic font family
            // changes. relative values (em and %) applied to keywords must be
            // recomputed from the base size for the keyword and the relative size.
            //
            // See bug 1355707
            if let Some((kw, fraction, old_abs)) = *context.builder.inherited_font_computation_data() {
                context.builder.font_size_keyword =
                    Some((kw, fraction * ratio, abs + old_abs.scale_by(ratio)));
            } else {
                context.builder.font_size_keyword = None;
            }
        } else {
            context.builder.font_size_keyword = None;
        }

        // we could use clone_language and clone_font_family() here but that's
        // expensive. Do it only in gecko mode for now.
            use gecko_bindings::structs::nsIAtom;
            // if the language or generic changed, we need to recalculate
            // the font size from the stored font-size origin information.
            if context.builder.get_font().gecko().mLanguage.raw::<nsIAtom>() !=
               context.builder.get_parent_font().gecko().mLanguage.raw::<nsIAtom>() ||
               context.builder.get_font().gecko().mGenericID !=
               context.builder.get_parent_font().gecko().mGenericID {
                if let Some((kw, ratio, offset)) = context.builder.font_size_keyword {
                    let len = kw.to_computed_value(context).scale_by(ratio) + offset;
                    computed = context.maybe_zoom_text(len.0).into();
                }
            }

        let device = context.builder.device;
        let mut font = context.builder.take_font();
        let parent_unconstrained = {
            let parent_font = context.builder.get_parent_font();
            font.apply_font_size(computed, parent_font, device)
        };
        context.builder.put_font(font);

        if let Some(parent) = parent_unconstrained {
            let new_unconstrained =
                specified_value
                    .to_computed_value_against(context, FontBaseSize::Custom(Au::from(parent)));
            context.builder
                   .mutate_font()
                   .apply_unconstrained_font_size(new_unconstrained);
        }
    }

    /// FIXME(emilio): This is very complex. Also, it should move to
    /// StyleBuilder.
    pub fn cascade_inherit_font_size(context: &mut Context) {
        // If inheriting, we must recompute font-size in case of language
        // changes using the font_size_keyword. We also need to do this to
        // handle mathml scriptlevel changes
        let kw_inherited_size = context.builder.font_size_keyword.map(|(kw, ratio, offset)| {
            let len = SpecifiedValue::Keyword(kw, ratio, offset).to_computed_value(context);
            context.maybe_zoom_text(len.0).into()
        });
        let parent_kw;
        let device = context.builder.device;
        let mut font = context.builder.take_font();
        let used_kw = {
            let parent_font = context.builder.get_parent_font();
            parent_kw = *context.builder.inherited_font_computation_data();

            font.inherit_font_size_from(parent_font, kw_inherited_size, device)
        };
        context.builder.put_font(font);
        context.builder.font_size_keyword =
            if used_kw { parent_kw } else { None };
    }

    /// Cascade the initial value for the `font-size` property.
    ///
    /// FIXME(emilio): This is the only function that is outside of the
    /// `StyleBuilder`, and should really move inside!
    ///
    /// Can we move the font stuff there?
    pub fn cascade_initial_font_size(context: &mut Context) {
        // font-size's default ("medium") does not always
        // compute to the same value and depends on the font
        let computed = context.maybe_zoom_text(
                            longhands::font_size::get_initial_specified_value()
                                .to_computed_value(context).0
                        ).into();
        context.builder.mutate_font().set_font_size(computed);
            let device = context.builder.device;
            context.builder.mutate_font().fixup_font_min_size(device);
        context.builder.font_size_keyword = Some((Default::default(), 1., Au(0).into()));
    }

        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::FontSize(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::FontSize);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            if let Some(sf) = specified_value.get_system() {
                                longhands::system_font::resolve_system_font(sf, context);
                            }
                            let computed = specified_value.to_computed_value(context);
                                 longhands::font_size::cascade_specified_font_size(
                                     context,
                                     &specified_value,
                                     computed,
                                 );
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                longhands::font_size::cascade_initial_font_size(context);
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                longhands::font_size::cascade_inherit_font_size(context);
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse_quirky(context, input, specified::AllowQuirks::Yes)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::FontSize)
            }
    }



    
    /// https://drafts.csswg.org/css-fonts/#propdef-font-size-adjust
    pub mod font_size_adjust {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
    use properties::longhands::system_font::SystemFont;


    #[derive(Clone, Copy, Debug, PartialEq, ToCss)]
    #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
    #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
    pub enum SpecifiedValue {
        None,
        Number(specified::Number),
        System(SystemFont),
    }

    impl ToComputedValue for SpecifiedValue {
        type ComputedValue = computed_value::T;

        fn to_computed_value(&self, context: &Context) -> Self::ComputedValue {
            match *self {
                SpecifiedValue::None => computed_value::T::None,
                SpecifiedValue::Number(ref n) => computed_value::T::Number(n.to_computed_value(context)),
                SpecifiedValue::System(_) => {
                    
        
                        context.cached_system_font.as_ref().unwrap().font_size_adjust
                    

                }
            }
        }

        fn from_computed_value(computed: &computed_value::T) -> Self {
            match *computed {
                computed_value::T::None => SpecifiedValue::None,
                computed_value::T::Number(ref v) => SpecifiedValue::Number(specified::Number::from_computed_value(v)),
            }
        }
    }

    impl SpecifiedValue {
        pub fn system_font(f: SystemFont) -> Self {
            SpecifiedValue::System(f)
        }
        pub fn get_system(&self) -> Option<SystemFont> {
            if let SpecifiedValue::System(s) = *self {
                Some(s)
            } else {
                None
            }
        }
    }

    pub mod computed_value {
        use values::CSSFloat;
        use values::animated::{ToAnimatedValue, ToAnimatedZero};

        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        #[derive(Animate, Clone, ComputeSquaredDistance, Copy, Debug, PartialEq, ToCss)]
        pub enum T {
            #[animation(error)]
            None,
            Number(CSSFloat),
        }

        impl T {
            pub fn from_gecko_adjust(gecko: f32) -> Self {
                if gecko == -1.0 {
                    T::None
                } else {
                    T::Number(gecko)
                }
            }
        }

        impl ToAnimatedZero for T {
            #[inline]
            fn to_animated_zero(&self) -> Result<Self, ()> { Err(()) }
        }

        impl ToAnimatedValue for T {
            type AnimatedValue = Self;

            #[inline]
            fn to_animated_value(self) -> Self {
                self
            }

            #[inline]
            fn from_animated_value(animated: Self::AnimatedValue) -> Self {
                match animated {
                    T::Number(number) => T::Number(number.max(0.)),
                    _ => animated
                }
            }
        }
    }

    #[inline]
    pub fn get_initial_value() -> computed_value::T {
        computed_value::T::None
    }

    #[inline]
    pub fn get_initial_specified_value() -> SpecifiedValue {
        SpecifiedValue::None
    }

    /// none | <number>
    pub fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                         -> Result<SpecifiedValue, ParseError<'i>> {
        use values::specified::Number;

        if input.try(|input| input.expect_ident_matching("none")).is_ok() {
            return Ok(SpecifiedValue::None);
        }

        Ok(SpecifiedValue::Number(Number::parse_non_negative(context, input)?))
    }

        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::FontSizeAdjust(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::FontSizeAdjust);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            if let Some(sf) = specified_value.get_system() {
                                longhands::system_font::resolve_system_font(sf, context);
                            }
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_font_size_adjust(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_font_size_adjust();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_font_size_adjust();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::FontSizeAdjust)
            }
    }



    
    /// https://drafts.csswg.org/css-fonts/#propdef-font-synthesis
    pub mod font_synthesis {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
    use std::fmt;
    use style_traits::ToCss;

    pub mod computed_value {
        pub use super::SpecifiedValue as T;
    }

    #[derive(Clone, Debug, PartialEq, ToComputedValue)]
    #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
    #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
    pub struct SpecifiedValue {
        pub weight: bool,
        pub style: bool,
    }

    impl ToCss for computed_value::T {
        fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
            if self.weight && self.style {
                dest.write_str("weight style")
            } else if self.style {
                dest.write_str("style")
            } else if self.weight {
                dest.write_str("weight")
            } else {
                dest.write_str("none")
            }
        }
    }

    #[inline]
    pub fn get_initial_value() -> computed_value::T {
        SpecifiedValue { weight: true, style: true }
    }

    pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                         -> Result<SpecifiedValue, ParseError<'i>> {
        let mut result = SpecifiedValue { weight: false, style: false };
        // FIXME: remove clone() when lifetimes are non-lexical
        try_match_ident_ignore_ascii_case! { input.expect_ident()?.clone(),
            "none" => Ok(result),
            "weight" => {
                result.weight = true;
                if input.try(|input| input.expect_ident_matching("style")).is_ok() {
                    result.style = true;
                }
                Ok(result)
            },
            "style" => {
                result.style = true;
                if input.try(|input| input.expect_ident_matching("weight")).is_ok() {
                    result.weight = true;
                }
                Ok(result)
            },
        }
    }

    #[cfg(feature = "gecko")]
    impl From<u8> for SpecifiedValue {
        fn from(bits: u8) -> SpecifiedValue {
            use gecko_bindings::structs;

            SpecifiedValue {
                weight: bits & structs::NS_FONT_SYNTHESIS_WEIGHT as u8 != 0,
                style: bits & structs::NS_FONT_SYNTHESIS_STYLE as u8 != 0
            }
        }
    }

    #[cfg(feature = "gecko")]
    impl From<SpecifiedValue> for u8 {
        fn from(v: SpecifiedValue) -> u8 {
            use gecko_bindings::structs;

            let mut bits: u8 = 0;
            if v.weight {
                bits |= structs::NS_FONT_SYNTHESIS_WEIGHT as u8;
            }
            if v.style {
                bits |= structs::NS_FONT_SYNTHESIS_STYLE as u8;
            }
            bits
        }
    }

        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::FontSynthesis(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::FontSynthesis);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_font_synthesis(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_font_synthesis();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_font_synthesis();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::FontSynthesis)
            }
    }



    
    
    
    /// https://drafts.csswg.org/css-fonts/#propdef-font-stretch
    pub mod font_stretch {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
        use properties::longhands::system_font::SystemFont;

        pub mod computed_value {
            use cssparser::Parser;
            use parser::{Parse, ParserContext};

            use style_traits::{ToCss, ParseError};
            define_css_keyword_enum! { T:
                    "normal" => normal,
                    "ultra-condensed" => ultra_condensed,
                    "extra-condensed" => extra_condensed,
                    "condensed" => condensed,
                    "semi-condensed" => semi_condensed,
                    "semi-expanded" => semi_expanded,
                    "expanded" => expanded,
                    "extra-expanded" => extra_expanded,
                    "ultra-expanded" => ultra_expanded,
            }

            impl Parse for T {
                fn parse<'i, 't>(_: &ParserContext, input: &mut Parser<'i, 't>) -> Result<Self, ParseError<'i>> {
                    T::parse(input)
                }
            }

            
    
    #[cfg(feature = "gecko")]
    impl T {
        /// Obtain a specified value from a Gecko keyword value
        ///
        /// Intended for use with presentation attributes, not style structs
        pub fn from_gecko_keyword(kw: u32) -> Self {
            use gecko_bindings::structs;
                // We can't match on enum values if we're matching on a u32
                const NORMAL: i32
                    = structs::NS_FONT_STRETCH_NORMAL as i32;
                // We can't match on enum values if we're matching on a u32
                const ULTRA_CONDENSED: i32
                    = structs::NS_FONT_STRETCH_ULTRA_CONDENSED as i32;
                // We can't match on enum values if we're matching on a u32
                const EXTRA_CONDENSED: i32
                    = structs::NS_FONT_STRETCH_EXTRA_CONDENSED as i32;
                // We can't match on enum values if we're matching on a u32
                const CONDENSED: i32
                    = structs::NS_FONT_STRETCH_CONDENSED as i32;
                // We can't match on enum values if we're matching on a u32
                const SEMI_CONDENSED: i32
                    = structs::NS_FONT_STRETCH_SEMI_CONDENSED as i32;
                // We can't match on enum values if we're matching on a u32
                const SEMI_EXPANDED: i32
                    = structs::NS_FONT_STRETCH_SEMI_EXPANDED as i32;
                // We can't match on enum values if we're matching on a u32
                const EXPANDED: i32
                    = structs::NS_FONT_STRETCH_EXPANDED as i32;
                // We can't match on enum values if we're matching on a u32
                const EXTRA_EXPANDED: i32
                    = structs::NS_FONT_STRETCH_EXTRA_EXPANDED as i32;
                // We can't match on enum values if we're matching on a u32
                const ULTRA_EXPANDED: i32
                    = structs::NS_FONT_STRETCH_ULTRA_EXPANDED as i32;
            match kw as i32 {
                    NORMAL => T::normal,
                    ULTRA_CONDENSED => T::ultra_condensed,
                    EXTRA_CONDENSED => T::extra_condensed,
                    CONDENSED => T::condensed,
                    SEMI_CONDENSED => T::semi_condensed,
                    SEMI_EXPANDED => T::semi_expanded,
                    EXPANDED => T::expanded,
                    EXTRA_EXPANDED => T::extra_expanded,
                    ULTRA_EXPANDED => T::ultra_expanded,
                x => panic!("Found unexpected value in style struct for font-stretch property: {:?}", x),
            }
        }
    }

        }

        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[derive(Clone, Copy, Debug, Eq, PartialEq, ToCss)]
        pub enum SpecifiedValue {
            Keyword(computed_value::T),
            System(SystemFont),
        }

        pub fn parse<'i, 't>(_: &ParserContext, input: &mut Parser<'i, 't>) -> Result<SpecifiedValue, ParseError<'i>> {
            Ok(SpecifiedValue::Keyword(computed_value::T::parse(input)?))
        }

        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;
            fn to_computed_value(&self, _cx: &Context) -> Self::ComputedValue {
                match *self {
                    SpecifiedValue::Keyword(v) => v,
                    SpecifiedValue::System(_) => {
                            _cx.cached_system_font.as_ref().unwrap().font_stretch
                    }
                }
            }
            fn from_computed_value(other: &computed_value::T) -> Self {
                SpecifiedValue::Keyword(*other)
            }
        }

        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::normal
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::Keyword(computed_value::T::normal)
        }

        impl SpecifiedValue {
            pub fn system_font(f: SystemFont) -> Self {
                SpecifiedValue::System(f)
            }
            pub fn get_system(&self) -> Option<SystemFont> {
                if let SpecifiedValue::System(s) = *self {
                    Some(s)
                } else {
                    None
                }
            }
        }
    
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::FontStretch(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::FontStretch);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            if let Some(sf) = specified_value.get_system() {
                                longhands::system_font::resolve_system_font(sf, context);
                            }
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_font_stretch(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_font_stretch();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_font_stretch();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::FontStretch)
            }
    }




    
    
    
    /// https://drafts.csswg.org/css-fonts/#propdef-font-kerning
    pub mod font_kerning {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
        use properties::longhands::system_font::SystemFont;

        pub mod computed_value {
            use cssparser::Parser;
            use parser::{Parse, ParserContext};

            use style_traits::{ToCss, ParseError};
            define_css_keyword_enum! { T:
                    "auto" => auto,
                    "none" => none,
                    "normal" => normal,
            }

            impl Parse for T {
                fn parse<'i, 't>(_: &ParserContext, input: &mut Parser<'i, 't>) -> Result<Self, ParseError<'i>> {
                    T::parse(input)
                }
            }

            
    
    #[cfg(feature = "gecko")]
    impl T {
        /// Obtain a specified value from a Gecko keyword value
        ///
        /// Intended for use with presentation attributes, not style structs
        pub fn from_gecko_keyword(kw: u32) -> Self {
            use gecko_bindings::structs;
                // We can't match on enum values if we're matching on a u32
                const AUTO: i32
                    = structs::NS_FONT_KERNING_AUTO as i32;
                // We can't match on enum values if we're matching on a u32
                const NONE: i32
                    = structs::NS_FONT_KERNING_NONE as i32;
                // We can't match on enum values if we're matching on a u32
                const NORMAL: i32
                    = structs::NS_FONT_KERNING_NORMAL as i32;
            match kw as i32 {
                    AUTO => T::auto,
                    NONE => T::none,
                    NORMAL => T::normal,
                x => panic!("Found unexpected value in style struct for font-kerning property: {:?}", x),
            }
        }
    }

        }

        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[derive(Clone, Copy, Debug, Eq, PartialEq, ToCss)]
        pub enum SpecifiedValue {
            Keyword(computed_value::T),
            System(SystemFont),
        }

        pub fn parse<'i, 't>(_: &ParserContext, input: &mut Parser<'i, 't>) -> Result<SpecifiedValue, ParseError<'i>> {
            Ok(SpecifiedValue::Keyword(computed_value::T::parse(input)?))
        }

        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;
            fn to_computed_value(&self, _cx: &Context) -> Self::ComputedValue {
                match *self {
                    SpecifiedValue::Keyword(v) => v,
                    SpecifiedValue::System(_) => {
                            _cx.cached_system_font.as_ref().unwrap().font_kerning
                    }
                }
            }
            fn from_computed_value(other: &computed_value::T) -> Self {
                SpecifiedValue::Keyword(*other)
            }
        }

        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::auto
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::Keyword(computed_value::T::auto)
        }

        impl SpecifiedValue {
            pub fn system_font(f: SystemFont) -> Self {
                SpecifiedValue::System(f)
            }
            pub fn get_system(&self) -> Option<SystemFont> {
                if let SpecifiedValue::System(s) = *self {
                    Some(s)
                } else {
                    None
                }
            }
        }
    
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::FontKerning(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::FontKerning);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            if let Some(sf) = specified_value.get_system() {
                                longhands::system_font::resolve_system_font(sf, context);
                            }
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_font_kerning(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_font_kerning();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_font_kerning();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::FontKerning)
            }
    }




    
    /// https://drafts.csswg.org/css-fonts/#propdef-font-variant-alternates
    pub mod font_variant_alternates {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
    use properties::longhands::system_font::SystemFont;
    use std::fmt;
    use style_traits::ToCss;
    use values::CustomIdent;


    #[derive(Clone, Debug, PartialEq)]
    #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
    #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
    pub enum VariantAlternates {
        Stylistic(CustomIdent),
        Styleset(Box<[CustomIdent]>),
        CharacterVariant(Box<[CustomIdent]>),
        Swash(CustomIdent),
        Ornaments(CustomIdent),
        Annotation(CustomIdent),
        HistoricalForms,
    }

    #[derive(Clone, Debug, PartialEq)]
    #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
    #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
    pub struct VariantAlternatesList(pub Box<[VariantAlternates]>);

    #[derive(Clone, Debug, PartialEq, ToCss)]
    #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
    #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
    pub enum SpecifiedValue {
        Value(VariantAlternatesList),
        System(SystemFont)
    }

    
    impl SpecifiedValue {
        pub fn system_font(f: SystemFont) -> Self {
            SpecifiedValue::System(f)
        }
        pub fn get_system(&self) -> Option<SystemFont> {
            if let SpecifiedValue::System(s) = *self {
                Some(s)
            } else {
                None
            }
        }
    }

    impl ToComputedValue for SpecifiedValue {
        type ComputedValue = computed_value::T;

        fn to_computed_value(&self, _context: &Context) -> computed_value::T {
            match *self {
                SpecifiedValue::Value(ref v) => v.clone(),
                SpecifiedValue::System(_) => {
                    
        
                        _context.cached_system_font.as_ref().unwrap().font_variant_alternates.clone()
                    

                }
            }
        }

        fn from_computed_value(other: &computed_value::T) -> Self {
            SpecifiedValue::Value(other.clone())
        }
    }


    impl ToCss for VariantAlternates {
        fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
            match *self {
                VariantAlternates::Swash(ref atom) => {
                    dest.write_str("swash")?;
                    dest.write_str("(")?;
                    atom.to_css(dest)?;
                    dest.write_str(")")
                },
                VariantAlternates::Stylistic(ref atom) => {
                    dest.write_str("stylistic")?;
                    dest.write_str("(")?;
                    atom.to_css(dest)?;
                    dest.write_str(")")
                },
                VariantAlternates::Ornaments(ref atom) => {
                    dest.write_str("ornaments")?;
                    dest.write_str("(")?;
                    atom.to_css(dest)?;
                    dest.write_str(")")
                },
                VariantAlternates::Annotation(ref atom) => {
                    dest.write_str("annotation")?;
                    dest.write_str("(")?;
                    atom.to_css(dest)?;
                    dest.write_str(")")
                },
                VariantAlternates::Styleset(ref vec) => {
                    dest.write_str("styleset")?;
                    dest.write_str("(")?;
                    let mut iter = vec.iter();
                    iter.next().unwrap().to_css(dest)?;
                    for c in iter {
                        dest.write_str(", ")?;
                        c.to_css(dest)?;
                    }
                    dest.write_str(")")
                },
                VariantAlternates::CharacterVariant(ref vec) => {
                    dest.write_str("character-variant")?;
                    dest.write_str("(")?;
                    let mut iter = vec.iter();
                    iter.next().unwrap().to_css(dest)?;
                    for c in iter {
                        dest.write_str(", ")?;
                        c.to_css(dest)?;
                    }
                    dest.write_str(")")
                },
                VariantAlternates::HistoricalForms => {
                    dest.write_str("historical-forms")
                },
            }
        }
    }

    impl ToCss for VariantAlternatesList {
        fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
            if self.0.is_empty() {
                return dest.write_str("normal");
            }

            let mut iter = self.0.iter();
            iter.next().unwrap().to_css(dest)?;
            for alternate in iter {
                dest.write_str(" ")?;
                alternate.to_css(dest)?;
            }
            Ok(())
        }
    }

    impl VariantAlternatesList {
        /// Returns the length of all variant alternates.
        pub fn len(&self) -> usize {
            self.0.iter().fold(0, |acc, alternate| {
                match *alternate {
                        VariantAlternates::Swash(_) => {
                            acc + 1
                        },
                        VariantAlternates::Stylistic(_) => {
                            acc + 1
                        },
                        VariantAlternates::Ornaments(_) => {
                            acc + 1
                        },
                        VariantAlternates::Annotation(_) => {
                            acc + 1
                        },
                        VariantAlternates::Styleset(ref slice) => {
                            acc + slice.len()
                        }
                        VariantAlternates::CharacterVariant(ref slice) => {
                            acc + slice.len()
                        }
                    _ => acc,
                }
            })
        }
    }

    pub mod computed_value {
        pub type T = super::VariantAlternatesList;
    }
    #[inline]
    pub fn get_initial_value() -> computed_value::T {
        VariantAlternatesList(vec![].into_boxed_slice())
    }
    #[inline]
    pub fn get_initial_specified_value() -> SpecifiedValue {
        SpecifiedValue::Value(VariantAlternatesList(vec![].into_boxed_slice()))
    }

    bitflags! {
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        pub flags ParsingFlags: u8 {
            const NORMAL = 0,
            const HISTORICAL_FORMS = 0x01,
            const STYLISTIC = 0x02,
            const STYLESET = 0x04,
            const CHARACTER_VARIANT = 0x08,
            const SWASH = 0x10,
            const ORNAMENTS = 0x20,
            const ANNOTATION = 0x40,
        }
    }

    /// normal |
    ///  [ stylistic(<feature-value-name>)           ||
    ///    historical-forms                          ||
    ///    styleset(<feature-value-name> #)          ||
    ///    character-variant(<feature-value-name> #) ||
    ///    swash(<feature-value-name>)               ||
    ///    ornaments(<feature-value-name>)           ||
    ///    annotation(<feature-value-name>) ]
    pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                         -> Result<SpecifiedValue, ParseError<'i>> {
        let mut alternates = Vec::new();
        if input.try(|input| input.expect_ident_matching("normal")).is_ok() {
            return Ok(SpecifiedValue::Value(VariantAlternatesList(alternates.into_boxed_slice())));
        }

        let mut parsed_alternates = ParsingFlags::empty();
        macro_rules! check_if_parsed(
            ($flag:ident) => (
                if parsed_alternates.contains($flag) {
                    return Err(StyleParseError::UnspecifiedError.into())
                }
                parsed_alternates |= $flag;
            )
        );
        while let Ok(_) = input.try(|input| {
            // FIXME: remove clone() when lifetimes are non-lexical
            match input.next()?.clone() {
                Token::Ident(ref ident) => {
                    if *ident == "historical-forms" {
                        check_if_parsed!(HISTORICAL_FORMS);
                        alternates.push(VariantAlternates::HistoricalForms);
                        Ok(())
                    } else {
                        return Err(StyleParseError::UnspecifiedError.into());
                    }
                },
                Token::Function(ref name) => {
                    input.parse_nested_block(|i| {
                        match_ignore_ascii_case! { &name,
                            "swash" => {
                                check_if_parsed!(SWASH);
                                let ident = CustomIdent::from_ident(i.expect_ident()?, &[])?;
                                alternates.push(VariantAlternates::Swash(ident));
                                Ok(())
                            },
                            "stylistic" => {
                                check_if_parsed!(STYLISTIC);
                                let ident = CustomIdent::from_ident(i.expect_ident()?, &[])?;
                                alternates.push(VariantAlternates::Stylistic(ident));
                                Ok(())
                            },
                            "ornaments" => {
                                check_if_parsed!(ORNAMENTS);
                                let ident = CustomIdent::from_ident(i.expect_ident()?, &[])?;
                                alternates.push(VariantAlternates::Ornaments(ident));
                                Ok(())
                            },
                            "annotation" => {
                                check_if_parsed!(ANNOTATION);
                                let ident = CustomIdent::from_ident(i.expect_ident()?, &[])?;
                                alternates.push(VariantAlternates::Annotation(ident));
                                Ok(())
                            },
                            "styleset" => {
                                check_if_parsed!(STYLESET);
                                let idents = i.parse_comma_separated(|i|
                                    CustomIdent::from_ident(i.expect_ident()?, &[]))?;
                                alternates.push(VariantAlternates::Styleset(idents.into_boxed_slice()));
                                Ok(())
                            },
                            "character-variant" => {
                                check_if_parsed!(CHARACTER_VARIANT);
                                let idents = i.parse_comma_separated(|i|
                                    CustomIdent::from_ident(i.expect_ident()?, &[]))?;
                                alternates.push(VariantAlternates::CharacterVariant(idents.into_boxed_slice()));
                                Ok(())
                            },
                            _ => return Err(StyleParseError::UnspecifiedError.into()),
                        }
                    })
                },
                _ => Err(StyleParseError::UnspecifiedError.into()),
            }
        }) { }

        if parsed_alternates.is_empty() {
            return Err(StyleParseError::UnspecifiedError.into());
        }
        Ok(SpecifiedValue::Value(VariantAlternatesList(alternates.into_boxed_slice())))
    }

        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::FontVariantAlternates(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::FontVariantAlternates);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            if let Some(sf) = specified_value.get_system() {
                                longhands::system_font::resolve_system_font(sf, context);
                            }
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_font_variant_alternates(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_font_variant_alternates();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_font_variant_alternates();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::FontVariantAlternates)
            }
    }


#[cfg(feature = "gecko")]
macro_rules! exclusive_value {
    (($value:ident, $set:expr) => $ident:ident) => {
        if $value.intersects($set) {
            return Err(())
        } else {
            $ident
        }
    }
}


    
    /// https://drafts.csswg.org/css-fonts/#propdef-font-variant-east-asian
    pub mod font_variant_east_asian {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
    use properties::longhands::system_font::SystemFont;
    use std::fmt;
    use style_traits::ToCss;


    bitflags! {
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        pub flags VariantEastAsian: u16 {
            const NORMAL = 0,
            const JIS78 = 0x01,
            const JIS83 = 0x02,
            const JIS90 = 0x04,
            const JIS04 = 0x08,
            const SIMPLIFIED = 0x10,
            const TRADITIONAL = 0x20,
            const FULL_WIDTH = 0x40,
            const PROPORTIONAL_WIDTH = 0x80,
            const RUBY = 0x100,
        }
    }

    #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
    #[derive(Clone, Debug, PartialEq, ToCss)]
    pub enum SpecifiedValue {
        Value(VariantEastAsian),
        System(SystemFont)
    }

    
    impl SpecifiedValue {
        pub fn system_font(f: SystemFont) -> Self {
            SpecifiedValue::System(f)
        }
        pub fn get_system(&self) -> Option<SystemFont> {
            if let SpecifiedValue::System(s) = *self {
                Some(s)
            } else {
                None
            }
        }
    }

    impl ToComputedValue for SpecifiedValue {
        type ComputedValue = computed_value::T;

        fn to_computed_value(&self, _context: &Context) -> computed_value::T {
            match *self {
                SpecifiedValue::Value(ref v) => v.clone(),
                SpecifiedValue::System(_) => {
                    
        
                        _context.cached_system_font.as_ref().unwrap().font_variant_east_asian.clone()
                    

                }
            }
        }

        fn from_computed_value(other: &computed_value::T) -> Self {
            SpecifiedValue::Value(other.clone())
        }
    }


    //                                 servo_bit: gecko_bit
    

    
    #[cfg(feature = "gecko")]
    impl VariantEastAsian {
        /// Obtain a specified value from a Gecko keyword value
        ///
        /// Intended for use with presentation attributes, not style structs
        pub fn from_gecko_keyword(kw: u16) -> Self {
            use gecko_bindings::structs::NS_FONT_VARIANT_EAST_ASIAN_TRADITIONAL;
            use gecko_bindings::structs::NS_FONT_VARIANT_EAST_ASIAN_JIS78;
            use gecko_bindings::structs::NS_FONT_VARIANT_EAST_ASIAN_JIS90;
            use gecko_bindings::structs::NS_FONT_VARIANT_EAST_ASIAN_PROP_WIDTH;
            use gecko_bindings::structs::NS_FONT_VARIANT_EAST_ASIAN_JIS04;
            use gecko_bindings::structs::NS_FONT_VARIANT_EAST_ASIAN_JIS83;
            use gecko_bindings::structs::NS_FONT_VARIANT_EAST_ASIAN_SIMPLIFIED;
            use gecko_bindings::structs::NS_FONT_VARIANT_EAST_ASIAN_FULL_WIDTH;
            use gecko_bindings::structs::NS_FONT_VARIANT_EAST_ASIAN_RUBY;

            let mut bits = VariantEastAsian::empty();
                if kw & (NS_FONT_VARIANT_EAST_ASIAN_TRADITIONAL as u16) != 0 {
                    bits |= TRADITIONAL;
                }
                if kw & (NS_FONT_VARIANT_EAST_ASIAN_JIS78 as u16) != 0 {
                    bits |= JIS78;
                }
                if kw & (NS_FONT_VARIANT_EAST_ASIAN_JIS90 as u16) != 0 {
                    bits |= JIS90;
                }
                if kw & (NS_FONT_VARIANT_EAST_ASIAN_PROP_WIDTH as u16) != 0 {
                    bits |= PROPORTIONAL_WIDTH;
                }
                if kw & (NS_FONT_VARIANT_EAST_ASIAN_JIS04 as u16) != 0 {
                    bits |= JIS04;
                }
                if kw & (NS_FONT_VARIANT_EAST_ASIAN_JIS83 as u16) != 0 {
                    bits |= JIS83;
                }
                if kw & (NS_FONT_VARIANT_EAST_ASIAN_SIMPLIFIED as u16) != 0 {
                    bits |= SIMPLIFIED;
                }
                if kw & (NS_FONT_VARIANT_EAST_ASIAN_FULL_WIDTH as u16) != 0 {
                    bits |= FULL_WIDTH;
                }
                if kw & (NS_FONT_VARIANT_EAST_ASIAN_RUBY as u16) != 0 {
                    bits |= RUBY;
                }
            bits
        }

        pub fn to_gecko_keyword(self) -> u16 {
            use gecko_bindings::structs::NS_FONT_VARIANT_EAST_ASIAN_TRADITIONAL;
            use gecko_bindings::structs::NS_FONT_VARIANT_EAST_ASIAN_JIS78;
            use gecko_bindings::structs::NS_FONT_VARIANT_EAST_ASIAN_JIS90;
            use gecko_bindings::structs::NS_FONT_VARIANT_EAST_ASIAN_PROP_WIDTH;
            use gecko_bindings::structs::NS_FONT_VARIANT_EAST_ASIAN_JIS04;
            use gecko_bindings::structs::NS_FONT_VARIANT_EAST_ASIAN_JIS83;
            use gecko_bindings::structs::NS_FONT_VARIANT_EAST_ASIAN_SIMPLIFIED;
            use gecko_bindings::structs::NS_FONT_VARIANT_EAST_ASIAN_FULL_WIDTH;
            use gecko_bindings::structs::NS_FONT_VARIANT_EAST_ASIAN_RUBY;

            let mut bits: u16 = 0;
            // FIXME: if we ensure that the Servo bitflags storage is the same
            // as Gecko's one, we can just copy it.
                if self.contains(TRADITIONAL) {
                    bits |= NS_FONT_VARIANT_EAST_ASIAN_TRADITIONAL as u16;
                }
                if self.contains(JIS78) {
                    bits |= NS_FONT_VARIANT_EAST_ASIAN_JIS78 as u16;
                }
                if self.contains(JIS90) {
                    bits |= NS_FONT_VARIANT_EAST_ASIAN_JIS90 as u16;
                }
                if self.contains(PROPORTIONAL_WIDTH) {
                    bits |= NS_FONT_VARIANT_EAST_ASIAN_PROP_WIDTH as u16;
                }
                if self.contains(JIS04) {
                    bits |= NS_FONT_VARIANT_EAST_ASIAN_JIS04 as u16;
                }
                if self.contains(JIS83) {
                    bits |= NS_FONT_VARIANT_EAST_ASIAN_JIS83 as u16;
                }
                if self.contains(SIMPLIFIED) {
                    bits |= NS_FONT_VARIANT_EAST_ASIAN_SIMPLIFIED as u16;
                }
                if self.contains(FULL_WIDTH) {
                    bits |= NS_FONT_VARIANT_EAST_ASIAN_FULL_WIDTH as u16;
                }
                if self.contains(RUBY) {
                    bits |= NS_FONT_VARIANT_EAST_ASIAN_RUBY as u16;
                }
            bits
        }
    }



    impl ToCss for VariantEastAsian {
        fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
            if self.is_empty() {
                return dest.write_str("normal")
            }

            let mut has_any = false;

            macro_rules! write_value {
                ($ident:ident => $str:expr) => {
                    if self.intersects($ident) {
                        if has_any {
                            dest.write_str(" ")?;
                        }
                        has_any = true;
                        dest.write_str($str)?;
                    }
                }
            }

            write_value!(JIS78 => "jis78");
            write_value!(JIS83 => "jis83");
            write_value!(JIS90 => "jis90");
            write_value!(JIS04 => "jis04");
            write_value!(SIMPLIFIED => "simplified");
            write_value!(TRADITIONAL => "traditional");
            write_value!(FULL_WIDTH => "full-width");
            write_value!(PROPORTIONAL_WIDTH => "proportional-width");
            write_value!(RUBY => "ruby");

            debug_assert!(has_any);
            Ok(())
        }
    }

    pub mod computed_value {
        pub type T = super::VariantEastAsian;
    }
    #[inline]
    pub fn get_initial_value() -> computed_value::T {
        computed_value::T::empty()
    }
    #[inline]
    pub fn get_initial_specified_value() -> SpecifiedValue {
        SpecifiedValue::Value(VariantEastAsian::empty())
    }

    /// normal | [ <east-asian-variant-values> || <east-asian-width-values> || ruby ]
    /// <east-asian-variant-values> = [ jis78 | jis83 | jis90 | jis04 | simplified | traditional ]
    /// <east-asian-width-values>   = [ full-width | proportional-width ]
    
    
    pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                         -> Result<SpecifiedValue, ParseError<'i>> {
        let mut result = VariantEastAsian::empty();

        if input.try(|input| input.expect_ident_matching("normal")).is_ok() {
            return Ok(SpecifiedValue::Value(result))
        }

        while let Ok(flag) = input.try(|input| {
            Ok(match_ignore_ascii_case! { &input.expect_ident().map_err(|_| ())?,
                "jis78" =>
                    exclusive_value!((result, JIS78 | JIS83 | JIS90 | JIS04 | SIMPLIFIED | TRADITIONAL) => JIS78),
                "jis83" =>
                    exclusive_value!((result, JIS78 | JIS83 | JIS90 | JIS04 | SIMPLIFIED | TRADITIONAL) => JIS83),
                "jis90" =>
                    exclusive_value!((result, JIS78 | JIS83 | JIS90 | JIS04 | SIMPLIFIED | TRADITIONAL) => JIS90),
                "jis04" =>
                    exclusive_value!((result, JIS78 | JIS83 | JIS90 | JIS04 | SIMPLIFIED | TRADITIONAL) => JIS04),
                "simplified" =>
                    exclusive_value!((result, JIS78 | JIS83 | JIS90 | JIS04 | SIMPLIFIED | TRADITIONAL) => SIMPLIFIED),
                "traditional" =>
                    exclusive_value!((result, JIS78 | JIS83 | JIS90 | JIS04 | SIMPLIFIED | TRADITIONAL) => TRADITIONAL),
                "full-width" =>
                    exclusive_value!((result, FULL_WIDTH | PROPORTIONAL_WIDTH) => FULL_WIDTH),
                "proportional-width" =>
                    exclusive_value!((result, FULL_WIDTH | PROPORTIONAL_WIDTH) => PROPORTIONAL_WIDTH),
                "ruby" =>
                    exclusive_value!((result, RUBY) => RUBY),
                _ => return Err(()),
            })
        }) {
            result.insert(flag);
        }

        if !result.is_empty() {
            Ok(SpecifiedValue::Value(result))
        } else {
            Err(StyleParseError::UnspecifiedError.into())
        }
    }

    #[cfg(feature = "gecko")]
    impl_gecko_keyword_conversions!(VariantEastAsian, u16);

        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::FontVariantEastAsian(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::FontVariantEastAsian);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            if let Some(sf) = specified_value.get_system() {
                                longhands::system_font::resolve_system_font(sf, context);
                            }
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_font_variant_east_asian(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_font_variant_east_asian();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_font_variant_east_asian();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::FontVariantEastAsian)
            }
    }



    
    /// https://drafts.csswg.org/css-fonts/#propdef-font-variant-ligatures
    pub mod font_variant_ligatures {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
    use properties::longhands::system_font::SystemFont;
    use std::fmt;
    use style_traits::ToCss;


    bitflags! {
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        pub flags VariantLigatures: u16 {
            const NORMAL = 0,
            const NONE = 0x01,
            const COMMON_LIGATURES = 0x02,
            const NO_COMMON_LIGATURES = 0x04,
            const DISCRETIONARY_LIGATURES = 0x08,
            const NO_DISCRETIONARY_LIGATURES = 0x10,
            const HISTORICAL_LIGATURES = 0x20,
            const NO_HISTORICAL_LIGATURES = 0x40,
            const CONTEXTUAL = 0x80,
            const NO_CONTEXTUAL = 0x100,
        }
    }

    #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
    #[derive(Clone, Debug, PartialEq, ToCss)]
    pub enum SpecifiedValue {
        Value(VariantLigatures),
        System(SystemFont)
    }

    
    impl SpecifiedValue {
        pub fn system_font(f: SystemFont) -> Self {
            SpecifiedValue::System(f)
        }
        pub fn get_system(&self) -> Option<SystemFont> {
            if let SpecifiedValue::System(s) = *self {
                Some(s)
            } else {
                None
            }
        }
    }

    impl ToComputedValue for SpecifiedValue {
        type ComputedValue = computed_value::T;

        fn to_computed_value(&self, _context: &Context) -> computed_value::T {
            match *self {
                SpecifiedValue::Value(ref v) => v.clone(),
                SpecifiedValue::System(_) => {
                    
        
                        _context.cached_system_font.as_ref().unwrap().font_variant_ligatures.clone()
                    

                }
            }
        }

        fn from_computed_value(other: &computed_value::T) -> Self {
            SpecifiedValue::Value(other.clone())
        }
    }


    //                                 servo_bit: gecko_bit
    

    
    #[cfg(feature = "gecko")]
    impl VariantLigatures {
        /// Obtain a specified value from a Gecko keyword value
        ///
        /// Intended for use with presentation attributes, not style structs
        pub fn from_gecko_keyword(kw: u16) -> Self {
            use gecko_bindings::structs::NS_FONT_VARIANT_LIGATURES_COMMON;
            use gecko_bindings::structs::NS_FONT_VARIANT_LIGATURES_DISCRETIONARY;
            use gecko_bindings::structs::NS_FONT_VARIANT_LIGATURES_NONE;
            use gecko_bindings::structs::NS_FONT_VARIANT_LIGATURES_NO_COMMON;
            use gecko_bindings::structs::NS_FONT_VARIANT_LIGATURES_NO_HISTORICAL;
            use gecko_bindings::structs::NS_FONT_VARIANT_LIGATURES_HISTORICAL;
            use gecko_bindings::structs::NS_FONT_VARIANT_LIGATURES_NO_CONTEXTUAL;
            use gecko_bindings::structs::NS_FONT_VARIANT_LIGATURES_NO_DISCRETIONARY;
            use gecko_bindings::structs::NS_FONT_VARIANT_LIGATURES_CONTEXTUAL;

            let mut bits = VariantLigatures::empty();
                if kw & (NS_FONT_VARIANT_LIGATURES_COMMON as u16) != 0 {
                    bits |= COMMON_LIGATURES;
                }
                if kw & (NS_FONT_VARIANT_LIGATURES_DISCRETIONARY as u16) != 0 {
                    bits |= DISCRETIONARY_LIGATURES;
                }
                if kw & (NS_FONT_VARIANT_LIGATURES_NONE as u16) != 0 {
                    bits |= NONE;
                }
                if kw & (NS_FONT_VARIANT_LIGATURES_NO_COMMON as u16) != 0 {
                    bits |= NO_COMMON_LIGATURES;
                }
                if kw & (NS_FONT_VARIANT_LIGATURES_NO_HISTORICAL as u16) != 0 {
                    bits |= NO_HISTORICAL_LIGATURES;
                }
                if kw & (NS_FONT_VARIANT_LIGATURES_HISTORICAL as u16) != 0 {
                    bits |= HISTORICAL_LIGATURES;
                }
                if kw & (NS_FONT_VARIANT_LIGATURES_NO_CONTEXTUAL as u16) != 0 {
                    bits |= NO_CONTEXTUAL;
                }
                if kw & (NS_FONT_VARIANT_LIGATURES_NO_DISCRETIONARY as u16) != 0 {
                    bits |= NO_DISCRETIONARY_LIGATURES;
                }
                if kw & (NS_FONT_VARIANT_LIGATURES_CONTEXTUAL as u16) != 0 {
                    bits |= CONTEXTUAL;
                }
            bits
        }

        pub fn to_gecko_keyword(self) -> u16 {
            use gecko_bindings::structs::NS_FONT_VARIANT_LIGATURES_COMMON;
            use gecko_bindings::structs::NS_FONT_VARIANT_LIGATURES_DISCRETIONARY;
            use gecko_bindings::structs::NS_FONT_VARIANT_LIGATURES_NONE;
            use gecko_bindings::structs::NS_FONT_VARIANT_LIGATURES_NO_COMMON;
            use gecko_bindings::structs::NS_FONT_VARIANT_LIGATURES_NO_HISTORICAL;
            use gecko_bindings::structs::NS_FONT_VARIANT_LIGATURES_HISTORICAL;
            use gecko_bindings::structs::NS_FONT_VARIANT_LIGATURES_NO_CONTEXTUAL;
            use gecko_bindings::structs::NS_FONT_VARIANT_LIGATURES_NO_DISCRETIONARY;
            use gecko_bindings::structs::NS_FONT_VARIANT_LIGATURES_CONTEXTUAL;

            let mut bits: u16 = 0;
            // FIXME: if we ensure that the Servo bitflags storage is the same
            // as Gecko's one, we can just copy it.
                if self.contains(COMMON_LIGATURES) {
                    bits |= NS_FONT_VARIANT_LIGATURES_COMMON as u16;
                }
                if self.contains(DISCRETIONARY_LIGATURES) {
                    bits |= NS_FONT_VARIANT_LIGATURES_DISCRETIONARY as u16;
                }
                if self.contains(NONE) {
                    bits |= NS_FONT_VARIANT_LIGATURES_NONE as u16;
                }
                if self.contains(NO_COMMON_LIGATURES) {
                    bits |= NS_FONT_VARIANT_LIGATURES_NO_COMMON as u16;
                }
                if self.contains(NO_HISTORICAL_LIGATURES) {
                    bits |= NS_FONT_VARIANT_LIGATURES_NO_HISTORICAL as u16;
                }
                if self.contains(HISTORICAL_LIGATURES) {
                    bits |= NS_FONT_VARIANT_LIGATURES_HISTORICAL as u16;
                }
                if self.contains(NO_CONTEXTUAL) {
                    bits |= NS_FONT_VARIANT_LIGATURES_NO_CONTEXTUAL as u16;
                }
                if self.contains(NO_DISCRETIONARY_LIGATURES) {
                    bits |= NS_FONT_VARIANT_LIGATURES_NO_DISCRETIONARY as u16;
                }
                if self.contains(CONTEXTUAL) {
                    bits |= NS_FONT_VARIANT_LIGATURES_CONTEXTUAL as u16;
                }
            bits
        }
    }


    impl ToCss for VariantLigatures {
        fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
            if self.is_empty() {
                return dest.write_str("normal")
            }
            if self.contains(NONE) {
                return dest.write_str("none")
            }

            let mut has_any = false;

            macro_rules! write_value {
                ($ident:ident => $str:expr) => {
                    if self.intersects($ident) {
                        if has_any {
                            dest.write_str(" ")?;
                        }
                        has_any = true;
                        dest.write_str($str)?;
                    }
                }
            }

            write_value!(COMMON_LIGATURES => "common-ligatures");
            write_value!(NO_COMMON_LIGATURES => "no-common-ligatures");
            write_value!(DISCRETIONARY_LIGATURES => "discretionary-ligatures");
            write_value!(NO_DISCRETIONARY_LIGATURES => "no-discretionary-ligatures");
            write_value!(HISTORICAL_LIGATURES => "historical-ligatures");
            write_value!(NO_HISTORICAL_LIGATURES => "no-historical-ligatures");
            write_value!(CONTEXTUAL => "contextual");
            write_value!(NO_CONTEXTUAL => "no-contextual");

            debug_assert!(has_any);
            Ok(())
        }
    }

    pub mod computed_value {
        pub type T = super::VariantLigatures;
    }
    #[inline]
    pub fn get_initial_value() -> computed_value::T {
        computed_value::T::empty()
    }
    #[inline]
    pub fn get_initial_specified_value() -> SpecifiedValue {
        SpecifiedValue::Value(VariantLigatures::empty())
    }
    #[inline]
    pub fn get_none_specified_value() -> SpecifiedValue {
        SpecifiedValue::Value(NONE)
    }

    /// normal | none |
    /// [ <common-lig-values> ||
    ///   <discretionary-lig-values> ||
    ///   <historical-lig-values> ||
    ///   <contextual-alt-values> ]
    /// <common-lig-values>        = [ common-ligatures | no-common-ligatures ]
    /// <discretionary-lig-values> = [ discretionary-ligatures | no-discretionary-ligatures ]
    /// <historical-lig-values>    = [ historical-ligatures | no-historical-ligatures ]
    /// <contextual-alt-values>    = [ contextual | no-contextual ]
    
    
    
    
    pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                         -> Result<SpecifiedValue, ParseError<'i>> {
        let mut result = VariantLigatures::empty();

        if input.try(|input| input.expect_ident_matching("normal")).is_ok() {
            return Ok(SpecifiedValue::Value(result))
        }
        if input.try(|input| input.expect_ident_matching("none")).is_ok() {
            return Ok(SpecifiedValue::Value(NONE))
        }

        while let Ok(flag) = input.try(|input| {
            Ok(match_ignore_ascii_case! { &input.expect_ident().map_err(|_| ())?,
                "common-ligatures" =>
                    exclusive_value!((result, COMMON_LIGATURES | NO_COMMON_LIGATURES) => COMMON_LIGATURES),
                "no-common-ligatures" =>
                    exclusive_value!((result, COMMON_LIGATURES | NO_COMMON_LIGATURES) => NO_COMMON_LIGATURES),
                "discretionary-ligatures" =>
                    exclusive_value!((result, DISCRETIONARY_LIGATURES | NO_DISCRETIONARY_LIGATURES) => DISCRETIONARY_LIGATURES),
                "no-discretionary-ligatures" =>
                    exclusive_value!((result, DISCRETIONARY_LIGATURES | NO_DISCRETIONARY_LIGATURES) => NO_DISCRETIONARY_LIGATURES),
                "historical-ligatures" =>
                    exclusive_value!((result, HISTORICAL_LIGATURES | NO_HISTORICAL_LIGATURES) => HISTORICAL_LIGATURES),
                "no-historical-ligatures" =>
                    exclusive_value!((result, HISTORICAL_LIGATURES | NO_HISTORICAL_LIGATURES) => NO_HISTORICAL_LIGATURES),
                "contextual" =>
                    exclusive_value!((result, CONTEXTUAL | NO_CONTEXTUAL) => CONTEXTUAL),
                "no-contextual" =>
                    exclusive_value!((result, CONTEXTUAL | NO_CONTEXTUAL) => NO_CONTEXTUAL),
                _ => return Err(()),
            })
        }) {
            result.insert(flag);
        }

        if !result.is_empty() {
            Ok(SpecifiedValue::Value(result))
        } else {
            Err(StyleParseError::UnspecifiedError.into())
        }
    }

    #[cfg(feature = "gecko")]
    impl_gecko_keyword_conversions!(VariantLigatures, u16);

        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::FontVariantLigatures(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::FontVariantLigatures);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            if let Some(sf) = specified_value.get_system() {
                                longhands::system_font::resolve_system_font(sf, context);
                            }
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_font_variant_ligatures(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_font_variant_ligatures();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_font_variant_ligatures();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::FontVariantLigatures)
            }
    }



    
    /// https://drafts.csswg.org/css-fonts/#propdef-font-variant-numeric
    pub mod font_variant_numeric {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
    use properties::longhands::system_font::SystemFont;
    use std::fmt;
    use style_traits::ToCss;


    bitflags! {
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        pub flags VariantNumeric: u8 {
            const NORMAL = 0,
            const LINING_NUMS = 0x01,
            const OLDSTYLE_NUMS = 0x02,
            const PROPORTIONAL_NUMS = 0x04,
            const TABULAR_NUMS = 0x08,
            const DIAGONAL_FRACTIONS = 0x10,
            const STACKED_FRACTIONS = 0x20,
            const SLASHED_ZERO = 0x40,
            const ORDINAL = 0x80,
        }
    }

    #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
    #[derive(Clone, Debug, PartialEq, ToCss)]
    pub enum SpecifiedValue {
        Value(VariantNumeric),
        System(SystemFont)
    }

    
    impl SpecifiedValue {
        pub fn system_font(f: SystemFont) -> Self {
            SpecifiedValue::System(f)
        }
        pub fn get_system(&self) -> Option<SystemFont> {
            if let SpecifiedValue::System(s) = *self {
                Some(s)
            } else {
                None
            }
        }
    }

    impl ToComputedValue for SpecifiedValue {
        type ComputedValue = computed_value::T;

        fn to_computed_value(&self, _context: &Context) -> computed_value::T {
            match *self {
                SpecifiedValue::Value(ref v) => v.clone(),
                SpecifiedValue::System(_) => {
                    
        
                        _context.cached_system_font.as_ref().unwrap().font_variant_numeric.clone()
                    

                }
            }
        }

        fn from_computed_value(other: &computed_value::T) -> Self {
            SpecifiedValue::Value(other.clone())
        }
    }



    //                              servo_bit: gecko_bit
    

    
    #[cfg(feature = "gecko")]
    impl VariantNumeric {
        /// Obtain a specified value from a Gecko keyword value
        ///
        /// Intended for use with presentation attributes, not style structs
        pub fn from_gecko_keyword(kw: u8) -> Self {
            use gecko_bindings::structs::NS_FONT_VARIANT_NUMERIC_ORDINAL;
            use gecko_bindings::structs::NS_FONT_VARIANT_NUMERIC_PROPORTIONAL;
            use gecko_bindings::structs::NS_FONT_VARIANT_NUMERIC_SLASHZERO;
            use gecko_bindings::structs::NS_FONT_VARIANT_NUMERIC_LINING;
            use gecko_bindings::structs::NS_FONT_VARIANT_NUMERIC_OLDSTYLE;
            use gecko_bindings::structs::NS_FONT_VARIANT_NUMERIC_TABULAR;
            use gecko_bindings::structs::NS_FONT_VARIANT_NUMERIC_STACKED_FRACTIONS;
            use gecko_bindings::structs::NS_FONT_VARIANT_NUMERIC_DIAGONAL_FRACTIONS;

            let mut bits = VariantNumeric::empty();
                if kw & (NS_FONT_VARIANT_NUMERIC_ORDINAL as u8) != 0 {
                    bits |= ORDINAL;
                }
                if kw & (NS_FONT_VARIANT_NUMERIC_PROPORTIONAL as u8) != 0 {
                    bits |= PROPORTIONAL_NUMS;
                }
                if kw & (NS_FONT_VARIANT_NUMERIC_SLASHZERO as u8) != 0 {
                    bits |= SLASHED_ZERO;
                }
                if kw & (NS_FONT_VARIANT_NUMERIC_LINING as u8) != 0 {
                    bits |= LINING_NUMS;
                }
                if kw & (NS_FONT_VARIANT_NUMERIC_OLDSTYLE as u8) != 0 {
                    bits |= OLDSTYLE_NUMS;
                }
                if kw & (NS_FONT_VARIANT_NUMERIC_TABULAR as u8) != 0 {
                    bits |= TABULAR_NUMS;
                }
                if kw & (NS_FONT_VARIANT_NUMERIC_STACKED_FRACTIONS as u8) != 0 {
                    bits |= STACKED_FRACTIONS;
                }
                if kw & (NS_FONT_VARIANT_NUMERIC_DIAGONAL_FRACTIONS as u8) != 0 {
                    bits |= DIAGONAL_FRACTIONS;
                }
            bits
        }

        pub fn to_gecko_keyword(self) -> u8 {
            use gecko_bindings::structs::NS_FONT_VARIANT_NUMERIC_ORDINAL;
            use gecko_bindings::structs::NS_FONT_VARIANT_NUMERIC_PROPORTIONAL;
            use gecko_bindings::structs::NS_FONT_VARIANT_NUMERIC_SLASHZERO;
            use gecko_bindings::structs::NS_FONT_VARIANT_NUMERIC_LINING;
            use gecko_bindings::structs::NS_FONT_VARIANT_NUMERIC_OLDSTYLE;
            use gecko_bindings::structs::NS_FONT_VARIANT_NUMERIC_TABULAR;
            use gecko_bindings::structs::NS_FONT_VARIANT_NUMERIC_STACKED_FRACTIONS;
            use gecko_bindings::structs::NS_FONT_VARIANT_NUMERIC_DIAGONAL_FRACTIONS;

            let mut bits: u8 = 0;
            // FIXME: if we ensure that the Servo bitflags storage is the same
            // as Gecko's one, we can just copy it.
                if self.contains(ORDINAL) {
                    bits |= NS_FONT_VARIANT_NUMERIC_ORDINAL as u8;
                }
                if self.contains(PROPORTIONAL_NUMS) {
                    bits |= NS_FONT_VARIANT_NUMERIC_PROPORTIONAL as u8;
                }
                if self.contains(SLASHED_ZERO) {
                    bits |= NS_FONT_VARIANT_NUMERIC_SLASHZERO as u8;
                }
                if self.contains(LINING_NUMS) {
                    bits |= NS_FONT_VARIANT_NUMERIC_LINING as u8;
                }
                if self.contains(OLDSTYLE_NUMS) {
                    bits |= NS_FONT_VARIANT_NUMERIC_OLDSTYLE as u8;
                }
                if self.contains(TABULAR_NUMS) {
                    bits |= NS_FONT_VARIANT_NUMERIC_TABULAR as u8;
                }
                if self.contains(STACKED_FRACTIONS) {
                    bits |= NS_FONT_VARIANT_NUMERIC_STACKED_FRACTIONS as u8;
                }
                if self.contains(DIAGONAL_FRACTIONS) {
                    bits |= NS_FONT_VARIANT_NUMERIC_DIAGONAL_FRACTIONS as u8;
                }
            bits
        }
    }


    impl ToCss for VariantNumeric {
        fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
            if self.is_empty() {
                return dest.write_str("normal")
            }

            let mut has_any = false;

            macro_rules! write_value {
                ($ident:ident => $str:expr) => {
                    if self.intersects($ident) {
                        if has_any {
                            dest.write_str(" ")?;
                        }
                        has_any = true;
                        dest.write_str($str)?;
                    }
                }
            }

            write_value!(LINING_NUMS => "lining-nums");
            write_value!(OLDSTYLE_NUMS => "oldstyle-nums");
            write_value!(PROPORTIONAL_NUMS => "proportional-nums");
            write_value!(TABULAR_NUMS => "tabular-nums");
            write_value!(DIAGONAL_FRACTIONS => "diagonal-fractions");
            write_value!(STACKED_FRACTIONS => "stacked-fractions");
            write_value!(SLASHED_ZERO => "slashed-zero");
            write_value!(ORDINAL => "ordinal");

            debug_assert!(has_any);
            Ok(())
        }
    }

    pub mod computed_value {
        pub type T = super::VariantNumeric;
    }
    #[inline]
    pub fn get_initial_value() -> computed_value::T {
        computed_value::T::empty()
    }
    #[inline]
    pub fn get_initial_specified_value() -> SpecifiedValue {
        SpecifiedValue::Value(VariantNumeric::empty())
    }

    /// normal |
    ///  [ <numeric-figure-values>   ||
    ///    <numeric-spacing-values>  ||
    ///    <numeric-fraction-values> ||
    ///    ordinal                   ||
    ///    slashed-zero ]
    /// <numeric-figure-values>   = [ lining-nums | oldstyle-nums ]
    /// <numeric-spacing-values>  = [ proportional-nums | tabular-nums ]
    /// <numeric-fraction-values> = [ diagonal-fractions | stacked-fractions ]
    
    
    
    pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                         -> Result<SpecifiedValue, ParseError<'i>> {
        let mut result = VariantNumeric::empty();

        if input.try(|input| input.expect_ident_matching("normal")).is_ok() {
            return Ok(SpecifiedValue::Value(result))
        }

        while let Ok(flag) = input.try(|input| {
            Ok(match_ignore_ascii_case! { &input.expect_ident().map_err(|_| ())?,
                "ordinal" =>
                    exclusive_value!((result, ORDINAL) => ORDINAL),
                "slashed-zero" =>
                    exclusive_value!((result, SLASHED_ZERO) => SLASHED_ZERO),
                "lining-nums" =>
                    exclusive_value!((result, LINING_NUMS | OLDSTYLE_NUMS) => LINING_NUMS),
                "oldstyle-nums" =>
                    exclusive_value!((result, LINING_NUMS | OLDSTYLE_NUMS) => OLDSTYLE_NUMS),
                "proportional-nums" =>
                    exclusive_value!((result, PROPORTIONAL_NUMS | TABULAR_NUMS) => PROPORTIONAL_NUMS),
                "tabular-nums" =>
                    exclusive_value!((result, PROPORTIONAL_NUMS | TABULAR_NUMS) => TABULAR_NUMS),
                "diagonal-fractions" =>
                    exclusive_value!((result, DIAGONAL_FRACTIONS | STACKED_FRACTIONS) => DIAGONAL_FRACTIONS),
                "stacked-fractions" =>
                    exclusive_value!((result, DIAGONAL_FRACTIONS | STACKED_FRACTIONS) => STACKED_FRACTIONS),
                _ => return Err(()),
            })
        }) {
            result.insert(flag);
        }

        if !result.is_empty() {
            Ok(SpecifiedValue::Value(result))
        } else {
            Err(StyleParseError::UnspecifiedError.into())
        }
    }

    #[cfg(feature = "gecko")]
    impl_gecko_keyword_conversions!(VariantNumeric, u8);

        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::FontVariantNumeric(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::FontVariantNumeric);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            if let Some(sf) = specified_value.get_system() {
                                longhands::system_font::resolve_system_font(sf, context);
                            }
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_font_variant_numeric(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_font_variant_numeric();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_font_variant_numeric();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::FontVariantNumeric)
            }
    }



    
    
    
    /// https://drafts.csswg.org/css-fonts/#propdef-font-variant-position
    pub mod font_variant_position {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
        use properties::longhands::system_font::SystemFont;

        pub mod computed_value {
            use cssparser::Parser;
            use parser::{Parse, ParserContext};

            use style_traits::{ToCss, ParseError};
            define_css_keyword_enum! { T:
                    "normal" => normal,
                    "sub" => sub,
                    "super" => super_,
            }

            impl Parse for T {
                fn parse<'i, 't>(_: &ParserContext, input: &mut Parser<'i, 't>) -> Result<Self, ParseError<'i>> {
                    T::parse(input)
                }
            }

            
    
    #[cfg(feature = "gecko")]
    impl T {
        /// Obtain a specified value from a Gecko keyword value
        ///
        /// Intended for use with presentation attributes, not style structs
        pub fn from_gecko_keyword(kw: u32) -> Self {
            use gecko_bindings::structs;
                // We can't match on enum values if we're matching on a u32
                const NORMAL: i32
                    = structs::NS_FONT_VARIANT_POSITION_NORMAL as i32;
                // We can't match on enum values if we're matching on a u32
                const SUB: i32
                    = structs::NS_FONT_VARIANT_POSITION_SUB as i32;
                // We can't match on enum values if we're matching on a u32
                const SUPER_: i32
                    = structs::NS_FONT_VARIANT_POSITION_SUPER as i32;
            match kw as i32 {
                    NORMAL => T::normal,
                    SUB => T::sub,
                    SUPER_ => T::super_,
                x => panic!("Found unexpected value in style struct for font-variant-position property: {:?}", x),
            }
        }
    }

        }

        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[derive(Clone, Copy, Debug, Eq, PartialEq, ToCss)]
        pub enum SpecifiedValue {
            Keyword(computed_value::T),
            System(SystemFont),
        }

        pub fn parse<'i, 't>(_: &ParserContext, input: &mut Parser<'i, 't>) -> Result<SpecifiedValue, ParseError<'i>> {
            Ok(SpecifiedValue::Keyword(computed_value::T::parse(input)?))
        }

        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;
            fn to_computed_value(&self, _cx: &Context) -> Self::ComputedValue {
                match *self {
                    SpecifiedValue::Keyword(v) => v,
                    SpecifiedValue::System(_) => {
                            _cx.cached_system_font.as_ref().unwrap().font_variant_position
                    }
                }
            }
            fn from_computed_value(other: &computed_value::T) -> Self {
                SpecifiedValue::Keyword(*other)
            }
        }

        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::normal
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::Keyword(computed_value::T::normal)
        }

        impl SpecifiedValue {
            pub fn system_font(f: SystemFont) -> Self {
                SpecifiedValue::System(f)
            }
            pub fn get_system(&self) -> Option<SystemFont> {
                if let SpecifiedValue::System(s) = *self {
                    Some(s)
                } else {
                    None
                }
            }
        }
    
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::FontVariantPosition(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::FontVariantPosition);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            if let Some(sf) = specified_value.get_system() {
                                longhands::system_font::resolve_system_font(sf, context);
                            }
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_font_variant_position(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_font_variant_position();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_font_variant_position();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::FontVariantPosition)
            }
    }




    
    /// https://drafts.csswg.org/css-fonts/#propdef-font-feature-settings
    pub mod font_feature_settings {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
    use properties::longhands::system_font::SystemFont;
    use values::generics::FontSettings;

    #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
    #[derive(Clone, Debug, PartialEq, ToCss)]
    pub enum SpecifiedValue {
        Value(computed_value::T),
        System(SystemFont)
    }

    
    impl SpecifiedValue {
        pub fn system_font(f: SystemFont) -> Self {
            SpecifiedValue::System(f)
        }
        pub fn get_system(&self) -> Option<SystemFont> {
            if let SpecifiedValue::System(s) = *self {
                Some(s)
            } else {
                None
            }
        }
    }

    impl ToComputedValue for SpecifiedValue {
        type ComputedValue = computed_value::T;

        fn to_computed_value(&self, _context: &Context) -> computed_value::T {
            match *self {
                SpecifiedValue::Value(ref v) => v.clone(),
                SpecifiedValue::System(_) => {
                    
        
                        _context.cached_system_font.as_ref().unwrap().font_feature_settings.clone()
                    

                }
            }
        }

        fn from_computed_value(other: &computed_value::T) -> Self {
            SpecifiedValue::Value(other.clone())
        }
    }


    pub mod computed_value {
        use values::generics::{FontSettings, FontSettingTagInt};
        pub type T = FontSettings<FontSettingTagInt>;
    }

    #[inline]
    pub fn get_initial_value() -> computed_value::T {
        FontSettings::Normal
    }

    #[inline]
    pub fn get_initial_specified_value() -> SpecifiedValue {
        SpecifiedValue::Value(FontSettings::Normal)
    }

    /// normal | <feature-tag-value>#
    pub fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                         -> Result<SpecifiedValue, ParseError<'i>> {
        computed_value::T::parse(context, input).map(SpecifiedValue::Value)
    }

        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::FontFeatureSettings(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::FontFeatureSettings);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            if let Some(sf) = specified_value.get_system() {
                                longhands::system_font::resolve_system_font(sf, context);
                            }
                            let computed = (**specified_value).to_computed_value(context);
                                context.builder.set_font_feature_settings(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_font_feature_settings();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_font_feature_settings();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<Box<SpecifiedValue>, ParseError<'i>> {
                    parse(context, input).map(|result| Box::new(result))
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::FontFeatureSettings)
            }
    }




    
    /// https://drafts.csswg.org/css-fonts-4/#low-level-font-variation-settings-control-the-font-variation-settings-property
    pub mod font_variation_settings {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
    use values::generics::FontSettings;

    pub type SpecifiedValue = computed_value::T;

    pub mod computed_value {
        use values::generics::{FontSettings, FontSettingTagFloat};
        pub type T = FontSettings<FontSettingTagFloat>;
    }

    #[inline]
    pub fn get_initial_value() -> computed_value::T {
        FontSettings::Normal
    }

    /// normal | <feature-tag-value>#
    pub fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                         -> Result<SpecifiedValue, ParseError<'i>> {
        computed_value::T::parse(context, input)
    }

        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::FontVariationSettings(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::FontVariationSettings);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_font_variation_settings(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_font_variation_settings();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_font_variation_settings();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::FontVariationSettings)
            }
    }



    
    /// https://drafts.csswg.org/css-fonts-3/#propdef-font-language-override
    pub mod font_language_override {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
    use properties::longhands::system_font::SystemFont;
    use std::fmt;
    use style_traits::ToCss;
    use byteorder::{BigEndian, ByteOrder};

    #[derive(Clone, Debug, Eq, PartialEq)]
    #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
    #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
    pub enum SpecifiedValue {
        Normal,
        Override(String),
        System(SystemFont)
    }

    impl ToCss for SpecifiedValue {
        fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
            match *self {
                SpecifiedValue::Normal => dest.write_str("normal"),
                SpecifiedValue::Override(ref lang) => lang.to_css(dest),
                SpecifiedValue::System(sys) => sys.to_css(dest),
            }
        }
    }

    impl SpecifiedValue {
        pub fn system_font(f: SystemFont) -> Self {
            SpecifiedValue::System(f)
        }
        pub fn get_system(&self) -> Option<SystemFont> {
            if let SpecifiedValue::System(s) = *self {
                Some(s)
            } else {
                None
            }
        }
    }

    pub mod computed_value {
        use std::{fmt, str};
        use style_traits::ToCss;
        use byteorder::{BigEndian, ByteOrder};

        impl ToCss for T {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
                if self.0 == 0 {
                    return dest.write_str("normal")
                }
                let mut buf = [0; 4];
                BigEndian::write_u32(&mut buf, self.0);
                // Safe because we ensure it's ASCII during computing
                let slice = if cfg!(debug_assertions) {
                    str::from_utf8(&buf).unwrap()
                } else {
                    unsafe { str::from_utf8_unchecked(&buf) }
                };
                slice.trim_right().to_css(dest)
            }
        }

        // font-language-override can only have a single three-letter
        // OpenType "language system" tag, so we should be able to compute
        // it and store it as a 32-bit integer
        // (see http://www.microsoft.com/typography/otspec/languagetags.htm).
        #[derive(Clone, Copy, Debug, Eq, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        pub struct T(pub u32);
    }

    #[inline]
    pub fn get_initial_value() -> computed_value::T {
        computed_value::T(0)
    }

    #[inline]
    pub fn get_initial_specified_value() -> SpecifiedValue {
        SpecifiedValue::Normal
    }

    impl ToComputedValue for SpecifiedValue {
        type ComputedValue = computed_value::T;

        #[inline]
        fn to_computed_value(&self, _context: &Context) -> computed_value::T {
            use std::ascii::AsciiExt;
            match *self {
                SpecifiedValue::Normal => computed_value::T(0),
                SpecifiedValue::Override(ref lang) => {
                    if lang.is_empty() || lang.len() > 4 || !lang.is_ascii() {
                        return computed_value::T(0)
                    }
                    let mut computed_lang = lang.clone();
                    while computed_lang.len() < 4 {
                        computed_lang.push(' ');
                    }
                    let bytes = computed_lang.into_bytes();
                    computed_value::T(BigEndian::read_u32(&bytes))
                }
                SpecifiedValue::System(_) => {
                    
        
                        _context.cached_system_font.as_ref().unwrap().font_language_override
                    

                }
            }
        }
        #[inline]
        fn from_computed_value(computed: &computed_value::T) -> Self {
            if computed.0 == 0 {
                return SpecifiedValue::Normal
            }
            let mut buf = [0; 4];
            BigEndian::write_u32(&mut buf, computed.0);
            SpecifiedValue::Override(
                if cfg!(debug_assertions) {
                    String::from_utf8(buf.to_vec()).unwrap()
                } else {
                    unsafe { String::from_utf8_unchecked(buf.to_vec()) }
                }
            )
        }
    }

    /// normal | <string>
    pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                         -> Result<SpecifiedValue, ParseError<'i>> {
        if input.try(|input| input.expect_ident_matching("normal")).is_ok() {
            Ok(SpecifiedValue::Normal)
        } else {
            input.expect_string().map(|s| {
                SpecifiedValue::Override(s.as_ref().to_owned())
            }).map_err(|e| e.into())
        }
    }

    /// Used in @font-face.
    impl Parse for SpecifiedValue {
        fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                         -> Result<Self, ParseError<'i>> {
            parse(context, input)
        }
    }

    #[cfg(feature = "gecko")]
    impl From<u32> for computed_value::T {
        fn from(bits: u32) -> computed_value::T {
            computed_value::T(bits)
        }
    }

    #[cfg(feature = "gecko")]
    impl From<computed_value::T> for u32 {
        fn from(v: computed_value::T) -> u32 {
            v.0
        }
    }

        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::FontLanguageOverride(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::FontLanguageOverride);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            if let Some(sf) = specified_value.get_system() {
                                longhands::system_font::resolve_system_font(sf, context);
                            }
                            let computed = (**specified_value).to_computed_value(context);
                                context.builder.set_font_language_override(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_font_language_override();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_font_language_override();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<Box<SpecifiedValue>, ParseError<'i>> {
                    parse(context, input).map(|result| Box::new(result))
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::FontLanguageOverride)
            }
    }



    
    /// Internal (not web-exposed)
    pub mod _x_lang {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
    pub use self::computed_value::T as SpecifiedValue;

    pub mod computed_value {
        use Atom;
        use std::fmt;
        use style_traits::ToCss;

        impl ToCss for T {
            fn to_css<W>(&self, _: &mut W) -> fmt::Result where W: fmt::Write {
                Ok(())
            }
        }

        #[derive(Clone, Debug, PartialEq, ToComputedValue)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        pub struct T(pub Atom);
    }

    #[inline]
    pub fn get_initial_value() -> computed_value::T {
        computed_value::T(atom!(""))
    }

    pub fn parse<'i, 't>(_context: &ParserContext, _input: &mut Parser<'i, 't>)
                         -> Result<SpecifiedValue, ParseError<'i>> {
        debug_assert!(false, "Should be set directly by presentation attributes only.");
        Err(StyleParseError::UnspecifiedError.into())
    }

        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::XLang(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::XLang);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set__x_lang(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset__x_lang();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit__x_lang();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::XLang)
            }
    }


// MathML properties

    
    /// Internal (not web-exposed)
    pub mod _moz_script_size_multiplier {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
    pub use self::computed_value::T as SpecifiedValue;

    pub mod computed_value {
        pub type T = f32;
    }

    #[inline]
    pub fn get_initial_value() -> computed_value::T {
        ::gecko_bindings::structs::NS_MATHML_DEFAULT_SCRIPT_SIZE_MULTIPLIER as f32
    }

    pub fn parse<'i, 't>(_context: &ParserContext, _input: &mut Parser<'i, 't>)
                         -> Result<SpecifiedValue, ParseError<'i>> {
        debug_assert!(false, "Should be set directly by presentation attributes only.");
        Err(StyleParseError::UnspecifiedError.into())
    }

        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MozScriptSizeMultiplier(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MozScriptSizeMultiplier);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set__moz_script_size_multiplier(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset__moz_script_size_multiplier();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit__moz_script_size_multiplier();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MozScriptSizeMultiplier)
            }
    }



    
    /// Internal (not web-exposed)
    pub mod _moz_script_level {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
    use std::fmt;
    use style_traits::ToCss;


    pub mod computed_value {
        pub type T = i8;
    }

    #[inline]
    pub fn get_initial_value() -> computed_value::T {
        0
    }

    #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
    #[derive(Clone, Copy, Debug, PartialEq)]
    pub enum SpecifiedValue {
        Relative(i32),
        Absolute(i32),
        Auto
    }

    impl ToCss for SpecifiedValue {
        fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
            match *self {
                SpecifiedValue::Auto => dest.write_str("auto"),
                SpecifiedValue::Relative(rel) => rel.to_css(dest),
                // can only be specified by pres attrs; should not
                // serialize to anything else
                SpecifiedValue::Absolute(_) => Ok(()),
            }
        }
    }

    impl ToComputedValue for SpecifiedValue {
        type ComputedValue = computed_value::T;

        fn to_computed_value(&self, cx: &Context) -> i8 {
            use properties::longhands::_moz_math_display::SpecifiedValue as DisplayValue;
            use std::{cmp, i8};

            let int = match *self {
                SpecifiedValue::Auto => {
                    let parent = cx.builder.get_parent_font().clone__moz_script_level() as i32;
                    let display = cx.builder.get_parent_font().clone__moz_math_display();
                    if display == DisplayValue::inline {
                        parent + 1
                    } else {
                        parent
                    }
                }
                SpecifiedValue::Relative(rel) => {
                    let parent = cx.builder.get_parent_font().clone__moz_script_level();
                    parent as i32 + rel
                }
                SpecifiedValue::Absolute(abs) => abs,
            };
            cmp::min(int, i8::MAX as i32) as i8
        }
        fn from_computed_value(other: &computed_value::T) -> Self {
            SpecifiedValue::Absolute(*other as i32)
        }
    }

    pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                         -> Result<SpecifiedValue, ParseError<'i>> {
        if let Ok(i) = input.try(|i| i.expect_integer()) {
            return Ok(SpecifiedValue::Relative(i))
        }
        input.expect_ident_matching("auto")?;
        Ok(SpecifiedValue::Auto)
    }

        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MozScriptLevel(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MozScriptLevel);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set__moz_script_level(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset__moz_script_level();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit__moz_script_level();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MozScriptLevel)
            }
    }



    
    

    
        
    
    /// Internal (not web-exposed)
    pub mod _moz_math_display {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "inline" => inline,
                    "block" => block,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::inline
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::inline
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::inline => computed_value::T::inline,
                        SpecifiedValue::block => computed_value::T::block,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::inline => SpecifiedValue::inline,
                        computed_value::T::block => SpecifiedValue::block,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MozMathDisplay(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MozMathDisplay);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set__moz_math_display(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset__moz_math_display();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit__moz_math_display();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MozMathDisplay)
            }
    }





    
    

    
        
    
    /// Internal (not web-exposed)
    pub mod _moz_math_variant {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "none" => none,
                    "normal" => normal,
                    "bold" => bold,
                    "italic" => italic,
                    "bold-italic" => bold_italic,
                    "script" => script,
                    "bold-script" => bold_script,
                    "fraktur" => fraktur,
                    "double-struck" => double_struck,
                    "bold-fraktur" => bold_fraktur,
                    "sans-serif" => sans_serif,
                    "bold-sans-serif" => bold_sans_serif,
                    "sans-serif-italic" => sans_serif_italic,
                    "sans-serif-bold-italic" => sans_serif_bold_italic,
                    "monospace" => monospace,
                    "initial" => initial,
                    "tailed" => tailed,
                    "looped" => looped,
                    "stretched" => stretched,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::none
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::none
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

            
            
    
    #[cfg(feature = "gecko")]
    impl SpecifiedValue {
        /// Obtain a specified value from a Gecko keyword value
        ///
        /// Intended for use with presentation attributes, not style structs
        pub fn from_gecko_keyword(kw: u32) -> Self {
            use gecko_bindings::structs;
                // We can't match on enum values if we're matching on a u32
                const NONE: u32
                    = structs::NS_MATHML_MATHVARIANT_NONE as u32;
                // We can't match on enum values if we're matching on a u32
                const NORMAL: u32
                    = structs::NS_MATHML_MATHVARIANT_NORMAL as u32;
                // We can't match on enum values if we're matching on a u32
                const BOLD: u32
                    = structs::NS_MATHML_MATHVARIANT_BOLD as u32;
                // We can't match on enum values if we're matching on a u32
                const ITALIC: u32
                    = structs::NS_MATHML_MATHVARIANT_ITALIC as u32;
                // We can't match on enum values if we're matching on a u32
                const BOLD_ITALIC: u32
                    = structs::NS_MATHML_MATHVARIANT_BOLD_ITALIC as u32;
                // We can't match on enum values if we're matching on a u32
                const SCRIPT: u32
                    = structs::NS_MATHML_MATHVARIANT_SCRIPT as u32;
                // We can't match on enum values if we're matching on a u32
                const BOLD_SCRIPT: u32
                    = structs::NS_MATHML_MATHVARIANT_BOLD_SCRIPT as u32;
                // We can't match on enum values if we're matching on a u32
                const FRAKTUR: u32
                    = structs::NS_MATHML_MATHVARIANT_FRAKTUR as u32;
                // We can't match on enum values if we're matching on a u32
                const DOUBLE_STRUCK: u32
                    = structs::NS_MATHML_MATHVARIANT_DOUBLE_STRUCK as u32;
                // We can't match on enum values if we're matching on a u32
                const BOLD_FRAKTUR: u32
                    = structs::NS_MATHML_MATHVARIANT_BOLD_FRAKTUR as u32;
                // We can't match on enum values if we're matching on a u32
                const SANS_SERIF: u32
                    = structs::NS_MATHML_MATHVARIANT_SANS_SERIF as u32;
                // We can't match on enum values if we're matching on a u32
                const BOLD_SANS_SERIF: u32
                    = structs::NS_MATHML_MATHVARIANT_BOLD_SANS_SERIF as u32;
                // We can't match on enum values if we're matching on a u32
                const SANS_SERIF_ITALIC: u32
                    = structs::NS_MATHML_MATHVARIANT_SANS_SERIF_ITALIC as u32;
                // We can't match on enum values if we're matching on a u32
                const SANS_SERIF_BOLD_ITALIC: u32
                    = structs::NS_MATHML_MATHVARIANT_SANS_SERIF_BOLD_ITALIC as u32;
                // We can't match on enum values if we're matching on a u32
                const MONOSPACE: u32
                    = structs::NS_MATHML_MATHVARIANT_MONOSPACE as u32;
                // We can't match on enum values if we're matching on a u32
                const INITIAL: u32
                    = structs::NS_MATHML_MATHVARIANT_INITIAL as u32;
                // We can't match on enum values if we're matching on a u32
                const TAILED: u32
                    = structs::NS_MATHML_MATHVARIANT_TAILED as u32;
                // We can't match on enum values if we're matching on a u32
                const LOOPED: u32
                    = structs::NS_MATHML_MATHVARIANT_LOOPED as u32;
                // We can't match on enum values if we're matching on a u32
                const STRETCHED: u32
                    = structs::NS_MATHML_MATHVARIANT_STRETCHED as u32;
            match kw  {
                    NONE => SpecifiedValue::none,
                    NORMAL => SpecifiedValue::normal,
                    BOLD => SpecifiedValue::bold,
                    ITALIC => SpecifiedValue::italic,
                    BOLD_ITALIC => SpecifiedValue::bold_italic,
                    SCRIPT => SpecifiedValue::script,
                    BOLD_SCRIPT => SpecifiedValue::bold_script,
                    FRAKTUR => SpecifiedValue::fraktur,
                    DOUBLE_STRUCK => SpecifiedValue::double_struck,
                    BOLD_FRAKTUR => SpecifiedValue::bold_fraktur,
                    SANS_SERIF => SpecifiedValue::sans_serif,
                    BOLD_SANS_SERIF => SpecifiedValue::bold_sans_serif,
                    SANS_SERIF_ITALIC => SpecifiedValue::sans_serif_italic,
                    SANS_SERIF_BOLD_ITALIC => SpecifiedValue::sans_serif_bold_italic,
                    MONOSPACE => SpecifiedValue::monospace,
                    INITIAL => SpecifiedValue::initial,
                    TAILED => SpecifiedValue::tailed,
                    LOOPED => SpecifiedValue::looped,
                    STRETCHED => SpecifiedValue::stretched,
                x => panic!("Found unexpected value in style struct for -moz-math-variant property: {:?}", x),
            }
        }
    }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::none => computed_value::T::none,
                        SpecifiedValue::normal => computed_value::T::normal,
                        SpecifiedValue::bold => computed_value::T::bold,
                        SpecifiedValue::italic => computed_value::T::italic,
                        SpecifiedValue::bold_italic => computed_value::T::bold_italic,
                        SpecifiedValue::script => computed_value::T::script,
                        SpecifiedValue::bold_script => computed_value::T::bold_script,
                        SpecifiedValue::fraktur => computed_value::T::fraktur,
                        SpecifiedValue::double_struck => computed_value::T::double_struck,
                        SpecifiedValue::bold_fraktur => computed_value::T::bold_fraktur,
                        SpecifiedValue::sans_serif => computed_value::T::sans_serif,
                        SpecifiedValue::bold_sans_serif => computed_value::T::bold_sans_serif,
                        SpecifiedValue::sans_serif_italic => computed_value::T::sans_serif_italic,
                        SpecifiedValue::sans_serif_bold_italic => computed_value::T::sans_serif_bold_italic,
                        SpecifiedValue::monospace => computed_value::T::monospace,
                        SpecifiedValue::initial => computed_value::T::initial,
                        SpecifiedValue::tailed => computed_value::T::tailed,
                        SpecifiedValue::looped => computed_value::T::looped,
                        SpecifiedValue::stretched => computed_value::T::stretched,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::none => SpecifiedValue::none,
                        computed_value::T::normal => SpecifiedValue::normal,
                        computed_value::T::bold => SpecifiedValue::bold,
                        computed_value::T::italic => SpecifiedValue::italic,
                        computed_value::T::bold_italic => SpecifiedValue::bold_italic,
                        computed_value::T::script => SpecifiedValue::script,
                        computed_value::T::bold_script => SpecifiedValue::bold_script,
                        computed_value::T::fraktur => SpecifiedValue::fraktur,
                        computed_value::T::double_struck => SpecifiedValue::double_struck,
                        computed_value::T::bold_fraktur => SpecifiedValue::bold_fraktur,
                        computed_value::T::sans_serif => SpecifiedValue::sans_serif,
                        computed_value::T::bold_sans_serif => SpecifiedValue::bold_sans_serif,
                        computed_value::T::sans_serif_italic => SpecifiedValue::sans_serif_italic,
                        computed_value::T::sans_serif_bold_italic => SpecifiedValue::sans_serif_bold_italic,
                        computed_value::T::monospace => SpecifiedValue::monospace,
                        computed_value::T::initial => SpecifiedValue::initial,
                        computed_value::T::tailed => SpecifiedValue::tailed,
                        computed_value::T::looped => SpecifiedValue::looped,
                        computed_value::T::stretched => SpecifiedValue::stretched,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MozMathVariant(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MozMathVariant);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set__moz_math_variant(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset__moz_math_variant();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit__moz_math_variant();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MozMathVariant)
            }
    }





    
    /// Internal (not web-exposed)
    pub mod _moz_script_min_size {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
    use gecko_bindings::structs::NS_MATHML_DEFAULT_SCRIPT_MIN_SIZE_PT;
    use values::computed::Length;
    use values::specified::length::{AU_PER_PT, AU_PER_PX, FontBaseSize, NoCalcLength};

    #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
    #[derive(Clone, Debug, PartialEq, ToCss)]
    pub struct SpecifiedValue(pub NoCalcLength);

    pub mod computed_value {
        pub type T = ::values::computed::Length;
    }

    impl ToComputedValue for SpecifiedValue {
        type ComputedValue = computed_value::T;

        fn to_computed_value(&self, cx: &Context) -> Length {
            // this value is used in the computation of font-size, so
            // we use the parent size
            let base_size = FontBaseSize::InheritedStyle;
            match self.0 {
                NoCalcLength::FontRelative(value) => {
                    value.to_computed_value(cx, base_size)
                }
                NoCalcLength::ServoCharacterWidth(value) => {
                    value.to_computed_value(base_size.resolve(cx))
                }
                ref l => {
                    l.to_computed_value(cx)
                }
            }
        }
        fn from_computed_value(other: &computed_value::T) -> Self {
            SpecifiedValue(ToComputedValue::from_computed_value(other))
        }
    }

    #[inline]
    pub fn get_initial_value() -> computed_value::T {
        Length::new(NS_MATHML_DEFAULT_SCRIPT_MIN_SIZE_PT as f32 * (AU_PER_PT / AU_PER_PX))
    }

    pub fn parse<'i, 't>(_context: &ParserContext, _input: &mut Parser<'i, 't>)
                         -> Result<SpecifiedValue, ParseError<'i>> {
        debug_assert!(false, "Should be set directly by presentation attributes only.");
        Err(StyleParseError::UnspecifiedError.into())
    }

        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MozScriptMinSize(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MozScriptMinSize);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set__moz_script_min_size(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset__moz_script_min_size();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit__moz_script_min_size();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MozScriptMinSize)
            }
    }



    
    /// Internal (not web-exposed)
    pub mod _x_text_zoom {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
    pub use self::computed_value::T as SpecifiedValue;

    pub mod computed_value {
        use std::fmt;
        use style_traits::ToCss;

        impl ToCss for T {
            fn to_css<W>(&self, _: &mut W) -> fmt::Result where W: fmt::Write {
                Ok(())
            }
        }

        #[derive(Clone, Debug, PartialEq, ToComputedValue)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        /// text-zoom. Enable if true, disable if false
        pub struct T(pub bool);
    }

    #[inline]
    pub fn get_initial_value() -> computed_value::T {
        computed_value::T(true)
    }

    pub fn parse<'i, 't>(_context: &ParserContext, _input: &mut Parser<'i, 't>)
                         -> Result<SpecifiedValue, ParseError<'i>> {
        debug_assert!(false, "Should be set directly by presentation attributes only.");
        Err(StyleParseError::UnspecifiedError.into())
    }

        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::XTextZoom(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::XTextZoom);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set__x_text_zoom(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset__x_text_zoom();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit__x_text_zoom();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::XTextZoom)
            }
    }


    pub mod system_font {
        //! We deal with system fonts here
        //!
        //! System fonts can only be set as a group via the font shorthand.
        //! They resolve at compute time (not parse time -- this lets the
        //! browser respond to changes to the OS font settings).
        //!
        //! While Gecko handles these as a separate property and keyword
        //! values on each property indicating that the font should be picked
        //! from the -x-system-font property, we avoid this. Instead,
        //! each font longhand has a special SystemFont variant which contains
        //! the specified system font. When the cascade function (in helpers)
        //! detects that a value has a system font, it will resolve it, and
        //! cache it on the ComputedValues. After this, it can be just fetched
        //! whenever a font longhand on the same element needs the system font.

        use app_units::Au;
        use cssparser::{Parser, ToCss};
        use gecko_bindings::structs::FontFamilyType;
        use properties::longhands;
        use std::fmt;
        use std::hash::{Hash, Hasher};
        use style_traits::ParseError;
        use values::computed::{ToComputedValue, Context};

        
        #[derive(Clone, Copy, Debug, Eq, Hash, MallocSizeOf, PartialEq, ToCss)]
        pub enum SystemFont {
                Caption,
                Icon,
                Menu,
                MessageBox,
                SmallCaption,
                StatusBar,
                MozWindow,
                MozDocument,
                MozWorkspace,
                MozDesktop,
                MozInfo,
                MozDialog,
                MozButton,
                MozPullDownMenu,
                MozList,
                MozField,
        }

        // ComputedValues are compared at times
        // so we need these impls. We don't want to
        // add Eq to Number (which contains a float)
        // so instead we have an eq impl which skips the
        // cached values
        impl PartialEq for ComputedSystemFont {
            fn eq(&self, other: &Self) -> bool {
                self.system_font == other.system_font
            }
        }
        impl Eq for ComputedSystemFont {}

        impl Hash for ComputedSystemFont {
            fn hash<H: Hasher>(&self, hasher: &mut H) {
                self.system_font.hash(hasher)
            }
        }

        impl ToComputedValue for SystemFont {
            type ComputedValue = ComputedSystemFont;

            fn to_computed_value(&self, cx: &Context) -> Self::ComputedValue {
                use gecko_bindings::bindings;
                use gecko_bindings::structs::{LookAndFeel_FontID, nsFont};
                use std::mem;

                let id = match *self {
                        SystemFont::Caption => {
                            LookAndFeel_FontID::eFont_Caption
                        }
                        SystemFont::Icon => {
                            LookAndFeel_FontID::eFont_Icon
                        }
                        SystemFont::Menu => {
                            LookAndFeel_FontID::eFont_Menu
                        }
                        SystemFont::MessageBox => {
                            LookAndFeel_FontID::eFont_MessageBox
                        }
                        SystemFont::SmallCaption => {
                            LookAndFeel_FontID::eFont_SmallCaption
                        }
                        SystemFont::StatusBar => {
                            LookAndFeel_FontID::eFont_StatusBar
                        }
                        SystemFont::MozWindow => {
                            LookAndFeel_FontID::eFont_Window
                        }
                        SystemFont::MozDocument => {
                            LookAndFeel_FontID::eFont_Document
                        }
                        SystemFont::MozWorkspace => {
                            LookAndFeel_FontID::eFont_Workspace
                        }
                        SystemFont::MozDesktop => {
                            LookAndFeel_FontID::eFont_Desktop
                        }
                        SystemFont::MozInfo => {
                            LookAndFeel_FontID::eFont_Info
                        }
                        SystemFont::MozDialog => {
                            LookAndFeel_FontID::eFont_Dialog
                        }
                        SystemFont::MozButton => {
                            LookAndFeel_FontID::eFont_Button
                        }
                        SystemFont::MozPullDownMenu => {
                            LookAndFeel_FontID::eFont_PullDownMenu
                        }
                        SystemFont::MozList => {
                            LookAndFeel_FontID::eFont_List
                        }
                        SystemFont::MozField => {
                            LookAndFeel_FontID::eFont_Field
                        }
                };

                let mut system: nsFont = unsafe { mem::uninitialized() };
                unsafe {
                    bindings::Gecko_nsFont_InitSystem(
                        &mut system,
                        id as i32,
                        cx.style().get_font().gecko(),
                        cx.device().pres_context()
                    )
                }
                let family = system.fontlist.mFontlist.iter().map(|font| {
                    use properties::longhands::font_family::computed_value::*;
                    FontFamily::FamilyName(FamilyName {
                        name: (&*font.mName).into(),
                        syntax: FamilyNameSyntax::Quoted,
                    })
                }).collect::<Vec<_>>();
                let weight = longhands::font_weight::computed_value::T::from_gecko_weight(system.weight);
                let ret = ComputedSystemFont {
                    font_family: longhands::font_family::computed_value::T(family),
                    font_size: Au(system.size).into(),
                    font_weight: weight,
                    font_size_adjust: longhands::font_size_adjust::computed_value
                                               ::T::from_gecko_adjust(system.sizeAdjust),
                        font_style: longhands::font_style::computed_value::T::from_gecko_keyword(
                            system.style
                                as u32
                        ),
                        font_variant_caps: longhands::font_variant_caps::computed_value::T::from_gecko_keyword(
                            system.variantCaps
                                as u32
                        ),
                        font_stretch: longhands::font_stretch::computed_value::T::from_gecko_keyword(
                            system.stretch
                                as u32
                        ),
                        font_kerning: longhands::font_kerning::computed_value::T::from_gecko_keyword(
                            system.kerning
                                as u32
                        ),
                        font_variant_position: longhands::font_variant_position::computed_value::T::from_gecko_keyword(
                            system.variantPosition
                                as u32
                        ),
                        font_variant_ligatures: longhands::font_variant_ligatures::computed_value::T::from_gecko_keyword(
                            system.variantLigatures
                        ),
                        font_variant_east_asian: longhands::font_variant_east_asian::computed_value::T::from_gecko_keyword(
                            system.variantEastAsian
                        ),
                        font_variant_numeric: longhands::font_variant_numeric::computed_value::T::from_gecko_keyword(
                            system.variantNumeric
                        ),
                    font_language_override: longhands::font_language_override::computed_value
                                                     ::T(system.languageOverride),
                    font_feature_settings: longhands::font_feature_settings::get_initial_value(),
                    font_variant_alternates: longhands::font_variant_alternates::get_initial_value(),
                    system_font: *self,
                    default_font_type: system.fontlist.mDefaultFontType,
                };
                unsafe { bindings::Gecko_nsFont_Destroy(&mut system); }
                ret
            }

            fn from_computed_value(_: &ComputedSystemFont) -> Self {
                unreachable!()
            }
        }

        #[inline]
        /// Compute and cache a system font
        ///
        /// Must be called before attempting to compute a system font
        /// specified value
        pub fn resolve_system_font(system: SystemFont, context: &mut Context) {
            // Checking if context.cached_system_font.is_none() isn't enough,
            // if animating from one system font to another the cached system font
            // may change
            if Some(system) != context.cached_system_font.as_ref().map(|x| x.system_font) {
                let computed = system.to_computed_value(context);
                context.cached_system_font = Some(computed);
            }
        }

        #[derive(Clone, Debug)]
        pub struct ComputedSystemFont {
                pub font_family: longhands::font_family::computed_value::T,
                pub font_size: longhands::font_size::computed_value::T,
                pub font_style: longhands::font_style::computed_value::T,
                pub font_variant_caps: longhands::font_variant_caps::computed_value::T,
                pub font_stretch: longhands::font_stretch::computed_value::T,
                pub font_kerning: longhands::font_kerning::computed_value::T,
                pub font_variant_position: longhands::font_variant_position::computed_value::T,
                pub font_weight: longhands::font_weight::computed_value::T,
                pub font_size_adjust: longhands::font_size_adjust::computed_value::T,
                pub font_variant_alternates: longhands::font_variant_alternates::computed_value::T,
                pub font_variant_ligatures: longhands::font_variant_ligatures::computed_value::T,
                pub font_variant_east_asian: longhands::font_variant_east_asian::computed_value::T,
                pub font_variant_numeric: longhands::font_variant_numeric::computed_value::T,
                pub font_language_override: longhands::font_language_override::computed_value::T,
                pub font_feature_settings: longhands::font_feature_settings::computed_value::T,
            pub system_font: SystemFont,
            pub default_font_type: FontFamilyType,
        }

        impl SystemFont {
            pub fn parse<'i, 't>(input: &mut Parser<'i, 't>) -> Result<Self, ParseError<'i>> {
                try_match_ident_ignore_ascii_case! { input.expect_ident()?,
                        "caption" => Ok(SystemFont::Caption),
                        "icon" => Ok(SystemFont::Icon),
                        "menu" => Ok(SystemFont::Menu),
                        "message-box" => Ok(SystemFont::MessageBox),
                        "small-caption" => Ok(SystemFont::SmallCaption),
                        "status-bar" => Ok(SystemFont::StatusBar),
                        "-moz-window" => Ok(SystemFont::MozWindow),
                        "-moz-document" => Ok(SystemFont::MozDocument),
                        "-moz-workspace" => Ok(SystemFont::MozWorkspace),
                        "-moz-desktop" => Ok(SystemFont::MozDesktop),
                        "-moz-info" => Ok(SystemFont::MozInfo),
                        "-moz-dialog" => Ok(SystemFont::MozDialog),
                        "-moz-button" => Ok(SystemFont::MozButton),
                        "-moz-pull-down-menu" => Ok(SystemFont::MozPullDownMenu),
                        "-moz-list" => Ok(SystemFont::MozList),
                        "-moz-field" => Ok(SystemFont::MozField),
                }
            }
        }

        impl ToCss for SystemFont {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
                // We may want to do something better in the future, see
                // w3c/csswg-drafts#1586.
                dest.write_str("-moz-use-system-font")
            }
        }
    }


    
    

    
        
    
    /// Nonstandard (https://developer.mozilla.org/en-US/docs/Web/CSS/font-smooth)
    pub mod _moz_osx_font_smoothing {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "auto" => auto,
                    "grayscale" => grayscale,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::auto
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::auto
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::auto => computed_value::T::auto,
                        SpecifiedValue::grayscale => computed_value::T::grayscale,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::auto => SpecifiedValue::auto,
                        computed_value::T::grayscale => SpecifiedValue::grayscale,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MozOsxFontSmoothing(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MozOsxFontSmoothing);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set__moz_osx_font_smoothing(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset__moz_osx_font_smoothing();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit__moz_osx_font_smoothing();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MozOsxFontSmoothing)
            }
    }





    
        
    
    /// Nonstandard (Internal-only)
    pub mod _moz_min_font_size_ratio {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::Percentage as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::Percentage as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::Percentage::hundred() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::Percentage::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MozMinFontSizeRatio(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MozMinFontSizeRatio);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set__moz_min_font_size_ratio(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset__moz_min_font_size_ratio();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit__moz_min_font_size_ratio();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MozMinFontSizeRatio)
            }
    }



    /* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */





// TODO: collapse. Well, do tables first.

    
    

    
        
    
    /// https://drafts.csswg.org/css-box/#propdef-visibility
    pub mod visibility {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "visible" => visible,
                    "hidden" => hidden,
                    "collapse" => collapse,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::visible
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::visible
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::visible => computed_value::T::visible,
                        SpecifiedValue::hidden => computed_value::T::hidden,
                        SpecifiedValue::collapse => computed_value::T::collapse,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::visible => SpecifiedValue::visible,
                        computed_value::T::hidden => SpecifiedValue::hidden,
                        computed_value::T::collapse => SpecifiedValue::collapse,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::Visibility(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::Visibility);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_visibility(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_visibility();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_visibility();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::Visibility)
            }
    }




// CSS Writing Modes Level 3
// https://drafts.csswg.org/css-writing-modes-3

    
    

    
        
    
    /// https://drafts.csswg.org/css-writing-modes/#propdef-writing-mode
    pub mod writing_mode {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            use style_traits::ToCss;
            define_css_keyword_enum! { SpecifiedValue:
                values {
                        "horizontal-tb" => horizontal_tb,
                        "vertical-rl" => vertical_rl,
                        "vertical-lr" => vertical_lr,
                        "sideways-rl" => sideways_rl,
                        "sideways-lr" => sideways_lr,
                }
                aliases {
                        "rl-tb" => horizontal_tb,
                        "lr" => horizontal_tb,
                        "rl" => horizontal_tb,
                        "lr-tb" => horizontal_tb,
                        "tb-rl" => vertical_rl,
                        "tb" => vertical_rl,
                }
            }
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "horizontal-tb" => horizontal_tb,
                    "vertical-rl" => vertical_rl,
                    "vertical-lr" => vertical_lr,
                    "sideways-rl" => sideways_rl,
                    "sideways-lr" => sideways_lr,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::horizontal_tb
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::horizontal_tb
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::horizontal_tb => computed_value::T::horizontal_tb,
                        SpecifiedValue::vertical_rl => computed_value::T::vertical_rl,
                        SpecifiedValue::vertical_lr => computed_value::T::vertical_lr,
                        SpecifiedValue::sideways_rl => computed_value::T::sideways_rl,
                        SpecifiedValue::sideways_lr => computed_value::T::sideways_lr,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::horizontal_tb => SpecifiedValue::horizontal_tb,
                        computed_value::T::vertical_rl => SpecifiedValue::vertical_rl,
                        computed_value::T::vertical_lr => SpecifiedValue::vertical_lr,
                        computed_value::T::sideways_rl => SpecifiedValue::sideways_rl,
                        computed_value::T::sideways_lr => SpecifiedValue::sideways_lr,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::WritingMode(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::WritingMode);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_writing_mode(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_writing_mode();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_writing_mode();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::WritingMode)
            }
    }





    
    

    
        
    
    /// https://drafts.csswg.org/css-writing-modes/#propdef-direction
    pub mod direction {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "ltr" => ltr,
                    "rtl" => rtl,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::ltr
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::ltr
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

            
            
    
    #[cfg(feature = "gecko")]
    impl SpecifiedValue {
        /// Obtain a specified value from a Gecko keyword value
        ///
        /// Intended for use with presentation attributes, not style structs
        pub fn from_gecko_keyword(kw: u32) -> Self {
            use gecko_bindings::structs;
                // We can't match on enum values if we're matching on a u32
                const LTR: u32
                    = structs::NS_STYLE_DIRECTION_LTR as u32;
                // We can't match on enum values if we're matching on a u32
                const RTL: u32
                    = structs::NS_STYLE_DIRECTION_RTL as u32;
            match kw  {
                    LTR => SpecifiedValue::ltr,
                    RTL => SpecifiedValue::rtl,
                x => panic!("Found unexpected value in style struct for direction property: {:?}", x),
            }
        }
    }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::ltr => computed_value::T::ltr,
                        SpecifiedValue::rtl => computed_value::T::rtl,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::ltr => SpecifiedValue::ltr,
                        computed_value::T::rtl => SpecifiedValue::rtl,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::Direction(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::Direction);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_direction(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_direction();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_direction();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::Direction)
            }
    }





    
    

    
        
    
    /// https://drafts.csswg.org/css-writing-modes/#propdef-text-orientation
    pub mod text_orientation {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            use style_traits::ToCss;
            define_css_keyword_enum! { SpecifiedValue:
                values {
                        "mixed" => mixed,
                        "upright" => upright,
                        "sideways" => sideways,
                }
                aliases {
                        "sideways-right" => sideways,
                }
            }
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "mixed" => mixed,
                    "upright" => upright,
                    "sideways" => sideways,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::mixed
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::mixed
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::mixed => computed_value::T::mixed,
                        SpecifiedValue::upright => computed_value::T::upright,
                        SpecifiedValue::sideways => computed_value::T::sideways,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::mixed => SpecifiedValue::mixed,
                        computed_value::T::upright => SpecifiedValue::upright,
                        computed_value::T::sideways => SpecifiedValue::sideways,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::TextOrientation(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::TextOrientation);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_text_orientation(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_text_orientation();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_text_orientation();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::TextOrientation)
            }
    }




// CSS Color Module Level 4
// https://drafts.csswg.org/css-color/

    
    

    
        
    
    /// https://drafts.csswg.org/css-color/#propdef-color-adjust
    pub mod color_adjust {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "economy" => economy,
                    "exact" => exact,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::economy
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::economy
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::economy => computed_value::T::economy,
                        SpecifiedValue::exact => computed_value::T::exact,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::economy => SpecifiedValue::economy,
                        computed_value::T::exact => SpecifiedValue::exact,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::ColorAdjust(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::ColorAdjust);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_color_adjust(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_color_adjust();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_color_adjust();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::ColorAdjust)
            }
    }





// According to to CSS-IMAGES-3, `optimizespeed` and `optimizequality` are synonyms for `auto`
// And, firefox doesn't support `pixelated` yet (https://bugzilla.mozilla.org/show_bug.cgi?id=856337)

    
    

    
        
    
    /// https://drafts.csswg.org/css-images/#propdef-image-rendering
    pub mod image_rendering {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "auto" => auto,
                    "optimizespeed" => optimizespeed,
                    "optimizequality" => optimizequality,
                    "-moz-crisp-edges" => _moz_crisp_edges,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::auto
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::auto
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::auto => computed_value::T::auto,
                        SpecifiedValue::optimizespeed => computed_value::T::optimizespeed,
                        SpecifiedValue::optimizequality => computed_value::T::optimizequality,
                        SpecifiedValue::_moz_crisp_edges => computed_value::T::_moz_crisp_edges,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::auto => SpecifiedValue::auto,
                        computed_value::T::optimizespeed => SpecifiedValue::optimizespeed,
                        computed_value::T::optimizequality => SpecifiedValue::optimizequality,
                        computed_value::T::_moz_crisp_edges => SpecifiedValue::_moz_crisp_edges,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::ImageRendering(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::ImageRendering);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_image_rendering(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_image_rendering();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_image_rendering();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::ImageRendering)
            }
    }




// Image Orientation

    
    /// https://drafts.csswg.org/css-images/#propdef-image-orientation, \
      /// additional values in https://developer.mozilla.org/en-US/docs/Web/CSS/image-orientation
    pub mod image_orientation {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
    use std::fmt;
    use style_traits::ToCss;
    use values::specified::Angle;


    use std::f64::consts::PI;
    const TWO_PI: f64 = 2.0 * PI;

    #[derive(Clone, Copy, Debug, PartialEq)]
    #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
    #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
    pub struct SpecifiedValue {
        pub angle: Option<Angle>,
        pub flipped: bool
    }

    impl ToCss for SpecifiedValue {
        fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
            if let Some(angle) = self.angle {
                angle.to_css(dest)?;
                if self.flipped {
                    dest.write_str(" flip")
                } else {
                    Ok(())
                }
            } else {
                if self.flipped {
                    dest.write_str("flip")
                } else {
                    dest.write_str("from-image")
                }
            }
        }
    }

    pub mod computed_value {
        use std::fmt;
        use style_traits::ToCss;
        use values::specified::Angle;

        #[derive(Clone, Copy, Debug, Eq, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        pub enum Orientation {
            Angle0 = 0,
            Angle90,
            Angle180,
            Angle270,
        }

        impl Orientation {
            pub fn angle(&self) -> Angle {
                match *self {
                    Orientation::Angle0 => Angle::from_degrees(0.0, false),
                    Orientation::Angle90 => Angle::from_degrees(90.0, false),
                    Orientation::Angle180 => Angle::from_degrees(180.0, false),
                    Orientation::Angle270 => Angle::from_degrees(270.0, false),
                }
            }
        }

        impl ToCss for Orientation {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
                // Should agree with Angle::to_css.
                match *self {
                    Orientation::Angle0 => dest.write_str("0deg"),
                    Orientation::Angle90 => dest.write_str("90deg"),
                    Orientation::Angle180 => dest.write_str("180deg"),
                    Orientation::Angle270 => dest.write_str("270deg"),
                }
            }
        }

        #[derive(Clone, Copy, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        pub enum T {
            FromImage,
            AngleWithFlipped(Orientation, bool),
        }
    }

    use self::computed_value::Orientation;

    #[inline]
    pub fn get_initial_value() -> computed_value::T {
        computed_value::T::AngleWithFlipped(Orientation::Angle0, false)
    }

    // According to CSS Content Module Level 3:
    // The computed value of the property is calculated by rounding the specified angle
    // to the nearest quarter-turn, rounding away from 0, then moduloing the value by 1 turn.
    // This mirrors the Gecko implementation in
    // nsStyleImageOrientation::CreateAsAngleAndFlip.
    #[inline]
    fn orientation_of_angle(angle: &computed::Angle) -> Orientation {
        // Note that `angle` can be negative.
        let mut rounded_angle = angle.radians64() % TWO_PI;
        if rounded_angle < 0.0 {
            // This computation introduces rounding error. Gecko previously
            // didn't handle the negative case correctly; by branching we can
            // match Gecko's behavior when it was correct.
            rounded_angle = rounded_angle + TWO_PI;
        }
        if rounded_angle < 0.25 * PI {
            return Orientation::Angle0
        }
        if rounded_angle < 0.75 * PI {
            return Orientation::Angle90
        }
        if rounded_angle < 1.25 * PI {
            return Orientation::Angle180
        }
        if rounded_angle < 1.75 * PI {
            return Orientation::Angle270
        }
        Orientation::Angle0
    }

    impl ToComputedValue for SpecifiedValue {
        type ComputedValue = computed_value::T;

        #[inline]
        fn to_computed_value(&self, context: &Context) -> computed_value::T {
            if let Some(ref angle) = self.angle {
                let angle = angle.to_computed_value(context);
                let orientation = orientation_of_angle(&angle);
                computed_value::T::AngleWithFlipped(orientation, self.flipped)
            } else {
                if self.flipped {
                    computed_value::T::AngleWithFlipped(Orientation::Angle0, true)
                } else {
                    computed_value::T::FromImage
                }
            }
        }

        #[inline]
        fn from_computed_value(computed: &computed_value::T) -> Self {
            match *computed {
                computed_value::T::FromImage => SpecifiedValue { angle: None, flipped: false },
                computed_value::T::AngleWithFlipped(ref orientation, flipped) => {
                    SpecifiedValue {
                        angle: Some(orientation.angle()),
                        flipped: flipped,
                    }
                }
            }
        }
    }

    impl ToCss for computed_value::T {
        fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
            match *self {
                computed_value::T::FromImage => dest.write_str("from-image"),
                computed_value::T::AngleWithFlipped(angle, flipped) => {
                    angle.to_css(dest)?;
                    if flipped {
                        dest.write_str(" flip")?;
                    }
                    Ok(())
                },
            }
        }
    }

    // from-image | <angle> | [<angle>? flip]
    pub fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                         -> Result<SpecifiedValue, ParseError<'i>> {
        if input.try(|input| input.expect_ident_matching("from-image")).is_ok() {
            // Handle from-image
            Ok(SpecifiedValue { angle: None, flipped: false })
        } else if input.try(|input| input.expect_ident_matching("flip")).is_ok() {
            // Handle flip
            Ok(SpecifiedValue { angle: Some(Angle::zero()), flipped: true })
        } else {
            // Handle <angle> | <angle> flip
            let angle = input.try(|input| Angle::parse(context, input)).ok();
            if angle.is_none() {
                return Err(StyleParseError::UnspecifiedError.into());
            }

            let flipped = input.try(|input| input.expect_ident_matching("flip")).is_ok();
            Ok(SpecifiedValue { angle: angle, flipped: flipped })
        }
    }

        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::ImageOrientation(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::ImageOrientation);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_image_orientation(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_image_orientation();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_image_orientation();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::ImageOrientation)
            }
    }


    /* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */






    
    

    
        
    
    /// https://drafts.csswg.org/css-tables/#propdef-border-collapse
    pub mod border_collapse {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "separate" => separate,
                    "collapse" => collapse,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::separate
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::separate
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::separate => computed_value::T::separate,
                        SpecifiedValue::collapse => computed_value::T::collapse,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::separate => SpecifiedValue::separate,
                        computed_value::T::collapse => SpecifiedValue::collapse,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::BorderCollapse(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::BorderCollapse);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_border_collapse(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_border_collapse();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_border_collapse();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::BorderCollapse)
            }
    }




    
    

    
        
    
    /// https://drafts.csswg.org/css-tables/#propdef-empty-cells
    pub mod empty_cells {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "show" => show,
                    "hide" => hide,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::show
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::show
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::show => computed_value::T::show,
                        SpecifiedValue::hide => computed_value::T::hide,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::show => SpecifiedValue::show,
                        computed_value::T::hide => SpecifiedValue::hide,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::EmptyCells(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::EmptyCells);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_empty_cells(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_empty_cells();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_empty_cells();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::EmptyCells)
            }
    }




    
    

    
        
    
    /// https://drafts.csswg.org/css-tables/#propdef-caption-side
    pub mod caption_side {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "top" => top,
                    "bottom" => bottom,
                    "right" => right,
                    "left" => left,
                    "top-outside" => top_outside,
                    "bottom-outside" => bottom_outside,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::top
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::top
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

            
            
    
    #[cfg(feature = "gecko")]
    impl SpecifiedValue {
        /// Obtain a specified value from a Gecko keyword value
        ///
        /// Intended for use with presentation attributes, not style structs
        pub fn from_gecko_keyword(kw: u32) -> Self {
            use gecko_bindings::structs;
                // We can't match on enum values if we're matching on a u32
                const TOP: u32
                    = structs::NS_STYLE_CAPTION_SIDE_TOP as u32;
                // We can't match on enum values if we're matching on a u32
                const BOTTOM: u32
                    = structs::NS_STYLE_CAPTION_SIDE_BOTTOM as u32;
                // We can't match on enum values if we're matching on a u32
                const RIGHT: u32
                    = structs::NS_STYLE_CAPTION_SIDE_RIGHT as u32;
                // We can't match on enum values if we're matching on a u32
                const LEFT: u32
                    = structs::NS_STYLE_CAPTION_SIDE_LEFT as u32;
                // We can't match on enum values if we're matching on a u32
                const TOP_OUTSIDE: u32
                    = structs::NS_STYLE_CAPTION_SIDE_TOP_OUTSIDE as u32;
                // We can't match on enum values if we're matching on a u32
                const BOTTOM_OUTSIDE: u32
                    = structs::NS_STYLE_CAPTION_SIDE_BOTTOM_OUTSIDE as u32;
            match kw  {
                    TOP => SpecifiedValue::top,
                    BOTTOM => SpecifiedValue::bottom,
                    RIGHT => SpecifiedValue::right,
                    LEFT => SpecifiedValue::left,
                    TOP_OUTSIDE => SpecifiedValue::top_outside,
                    BOTTOM_OUTSIDE => SpecifiedValue::bottom_outside,
                x => panic!("Found unexpected value in style struct for caption-side property: {:?}", x),
            }
        }
    }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::top => computed_value::T::top,
                        SpecifiedValue::bottom => computed_value::T::bottom,
                        SpecifiedValue::right => computed_value::T::right,
                        SpecifiedValue::left => computed_value::T::left,
                        SpecifiedValue::top_outside => computed_value::T::top_outside,
                        SpecifiedValue::bottom_outside => computed_value::T::bottom_outside,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::top => SpecifiedValue::top,
                        computed_value::T::bottom => SpecifiedValue::bottom,
                        computed_value::T::right => SpecifiedValue::right,
                        computed_value::T::left => SpecifiedValue::left,
                        computed_value::T::top_outside => SpecifiedValue::top_outside,
                        computed_value::T::bottom_outside => SpecifiedValue::bottom_outside,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::CaptionSide(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::CaptionSide);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_caption_side(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_caption_side();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_caption_side();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::CaptionSide)
            }
    }





    
        
    
    /// https://drafts.csswg.org/css-tables/#propdef-border-spacing
    pub mod border_spacing {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::BorderSpacing as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::BorderSpacing as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::BorderSpacing::zero() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::BorderSpacing::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::BorderSpacing(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::BorderSpacing);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = (**specified_value).to_computed_value(context);
                                context.builder.set_border_spacing(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_border_spacing();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_border_spacing();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<Box<SpecifiedValue>, ParseError<'i>> {
                    parse(context, input).map(|result| Box::new(result))
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::BorderSpacing)
            }
    }



    /* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */






    
        
    
    /// https://drafts.csswg.org/css2/visudet.html#propdef-line-height
    pub mod line_height {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::LineHeight as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::LineHeight as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::LineHeight::normal() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::LineHeight::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::LineHeight(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::LineHeight);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_line_height(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_line_height();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_line_height();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::LineHeight)
            }
    }



// CSS Text Module Level 3

// TODO(pcwalton): `full-width`

    
    

    
        
    
    /// https://drafts.csswg.org/css-text/#propdef-text-transform
    pub mod text_transform {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "none" => none,
                    "capitalize" => capitalize,
                    "uppercase" => uppercase,
                    "lowercase" => lowercase,
                    "full-width" => full_width,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::none
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::none
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::none => computed_value::T::none,
                        SpecifiedValue::capitalize => computed_value::T::capitalize,
                        SpecifiedValue::uppercase => computed_value::T::uppercase,
                        SpecifiedValue::lowercase => computed_value::T::lowercase,
                        SpecifiedValue::full_width => computed_value::T::full_width,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::none => SpecifiedValue::none,
                        computed_value::T::capitalize => SpecifiedValue::capitalize,
                        computed_value::T::uppercase => SpecifiedValue::uppercase,
                        computed_value::T::lowercase => SpecifiedValue::lowercase,
                        computed_value::T::full_width => SpecifiedValue::full_width,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::TextTransform(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::TextTransform);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_text_transform(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_text_transform();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_text_transform();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::TextTransform)
            }
    }





    
    

    
        
    
    /// https://drafts.csswg.org/css-text/#propdef-hyphens
    pub mod hyphens {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "manual" => manual,
                    "none" => none,
                    "auto" => auto,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::manual
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::manual
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::manual => computed_value::T::manual,
                        SpecifiedValue::none => computed_value::T::none,
                        SpecifiedValue::auto => computed_value::T::auto,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::manual => SpecifiedValue::manual,
                        computed_value::T::none => SpecifiedValue::none,
                        computed_value::T::auto => SpecifiedValue::auto,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::Hyphens(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::Hyphens);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_hyphens(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_hyphens();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_hyphens();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::Hyphens)
            }
    }




// TODO: Support <percentage>

    
    

    
        
    
    /// https://drafts.csswg.org/css-size-adjust/#adjustment-control
    pub mod _moz_text_size_adjust {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "auto" => auto,
                    "none" => none,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::auto
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::auto
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::auto => computed_value::T::auto,
                        SpecifiedValue::none => computed_value::T::none,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::auto => SpecifiedValue::auto,
                        computed_value::T::none => SpecifiedValue::none,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MozTextSizeAdjust(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MozTextSizeAdjust);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set__moz_text_size_adjust(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset__moz_text_size_adjust();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit__moz_text_size_adjust();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MozTextSizeAdjust)
            }
    }





    
        
    
    /// https://drafts.csswg.org/css-text/#propdef-text-indent
    pub mod text_indent {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::LengthOrPercentage as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::LengthOrPercentage as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::LengthOrPercentage::Length(computed::Length::new(0.)) }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::LengthOrPercentage::parse_quirky(context, input, AllowQuirks::Yes)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::TextIndent(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::TextIndent);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_text_indent(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_text_indent();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_text_indent();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::TextIndent)
            }
    }



// Also known as "word-wrap" (which is more popular because of IE), but this is the preferred
// name per CSS-TEXT 6.2.

    
    

    
        
    
    /// https://drafts.csswg.org/css-text/#propdef-overflow-wrap
    pub mod overflow_wrap {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "normal" => normal,
                    "break-word" => break_word,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::normal
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::normal
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::normal => computed_value::T::normal,
                        SpecifiedValue::break_word => computed_value::T::break_word,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::normal => SpecifiedValue::normal,
                        computed_value::T::break_word => SpecifiedValue::break_word,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::OverflowWrap(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::OverflowWrap);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_overflow_wrap(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_overflow_wrap();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_overflow_wrap();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::OverflowWrap)
            }
    }




// TODO(pcwalton): Support `word-break: keep-all` once we have better CJK support.

    
    

    
        
    
    /// https://drafts.csswg.org/css-text/#propdef-word-break
    pub mod word_break {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "normal" => normal,
                    "break-all" => break_all,
                    "keep-all" => keep_all,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::normal
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::normal
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::normal => computed_value::T::normal,
                        SpecifiedValue::break_all => computed_value::T::break_all,
                        SpecifiedValue::keep_all => computed_value::T::keep_all,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::normal => SpecifiedValue::normal,
                        computed_value::T::break_all => SpecifiedValue::break_all,
                        computed_value::T::keep_all => SpecifiedValue::keep_all,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::WordBreak(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::WordBreak);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_word_break(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_word_break();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_word_break();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::WordBreak)
            }
    }




// TODO(pcwalton): Support `text-justify: distribute`.

    

    
        
    
    /// https://drafts.csswg.org/css-text/#propdef-text-justify
    pub mod text_justify {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            use style_traits::ToCss;
            define_css_keyword_enum! { SpecifiedValue:
                values {
                        "auto" => auto,
                        "none" => none,
                        "inter-word" => inter_word,
                        "inter-character" => inter_character,
                        "distribute" => distribute,
                }
                aliases {
                }
            }
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "auto" => auto,
                    "none" => none,
                    "inter-word" => inter_word,
                    "inter-character" => inter_character,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::auto
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::auto
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            

    impl ToComputedValue for SpecifiedValue {
        type ComputedValue = computed_value::T;

        #[inline]
        fn to_computed_value(&self, _: &Context) -> computed_value::T {
            match *self {
                    SpecifiedValue::auto => computed_value::T::auto,
                    SpecifiedValue::none => computed_value::T::none,
                    SpecifiedValue::inter_word => computed_value::T::inter_word,
                    SpecifiedValue::inter_character => computed_value::T::inter_character,
                    // https://drafts.csswg.org/css-text-3/#valdef-text-justify-distribute
                    SpecifiedValue::distribute => computed_value::T::inter_character,
            }
        }

        #[inline]
        fn from_computed_value(computed: &computed_value::T) -> SpecifiedValue {
            match *computed {
                    computed_value::T::auto => SpecifiedValue::auto,
                    computed_value::T::none => SpecifiedValue::none,
                    computed_value::T::inter_word => SpecifiedValue::inter_word,
                    computed_value::T::inter_character => SpecifiedValue::inter_character,
            }
        }
    }

        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::TextJustify(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::TextJustify);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_text_justify(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_text_justify();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_text_justify();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::TextJustify)
            }
    }




    
    

    
        
    
    /// https://drafts.csswg.org/css-text/#propdef-text-align-last
    pub mod text_align_last {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "auto" => auto,
                    "start" => start,
                    "end" => end,
                    "left" => left,
                    "right" => right,
                    "center" => center,
                    "justify" => justify,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::auto
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::auto
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::auto => computed_value::T::auto,
                        SpecifiedValue::start => computed_value::T::start,
                        SpecifiedValue::end => computed_value::T::end,
                        SpecifiedValue::left => computed_value::T::left,
                        SpecifiedValue::right => computed_value::T::right,
                        SpecifiedValue::center => computed_value::T::center,
                        SpecifiedValue::justify => computed_value::T::justify,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::auto => SpecifiedValue::auto,
                        computed_value::T::start => SpecifiedValue::start,
                        computed_value::T::end => SpecifiedValue::end,
                        computed_value::T::left => SpecifiedValue::left,
                        computed_value::T::right => SpecifiedValue::right,
                        computed_value::T::center => SpecifiedValue::center,
                        computed_value::T::justify => SpecifiedValue::justify,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::TextAlignLast(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::TextAlignLast);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_text_align_last(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_text_align_last();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_text_align_last();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::TextAlignLast)
            }
    }




// TODO make this a shorthand and implement text-align-last/text-align-all

    
    /// https://drafts.csswg.org/css-text/#propdef-text-align
    pub mod text_align {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
    pub mod computed_value {
        use style_traits::ToCss;
        macro_rules! define_text_align {
            ( $( $name: ident ( $string: expr ) => $discriminant: expr, )+ ) => {
                define_css_keyword_enum! { T:
                    $(
                        $string => $name,
                    )+
                }
                impl T {
                    pub fn to_u32(self) -> u32 {
                        match self {
                            $(
                                T::$name => $discriminant,
                            )+
                        }
                    }
                    pub fn from_u32(discriminant: u32) -> Option<T> {
                        match discriminant {
                            $(
                                $discriminant => Some(T::$name),
                            )+
                            _ => None
                        }
                    }
                }
            }
        }
        define_text_align! {
            start("start") => 0,
            end("end") => 1,
            left("left") => 2,
            right("right") => 3,
            center("center") => 4,
            justify("justify") => 5,
            _moz_center("-moz-center") => 6,
            _moz_left("-moz-left") => 7,
            _moz_right("-moz-right") => 8,
            char("char") => 10,
        }

        
    
    #[cfg(feature = "gecko")]
    impl T {
        /// Obtain a specified value from a Gecko keyword value
        ///
        /// Intended for use with presentation attributes, not style structs
        pub fn from_gecko_keyword(kw: u32) -> Self {
            use gecko_bindings::structs;
                // We can't match on enum values if we're matching on a u32
                const LEFT: u32
                    = structs::NS_STYLE_TEXT_ALIGN_LEFT as u32;
                // We can't match on enum values if we're matching on a u32
                const RIGHT: u32
                    = structs::NS_STYLE_TEXT_ALIGN_RIGHT as u32;
                // We can't match on enum values if we're matching on a u32
                const CENTER: u32
                    = structs::NS_STYLE_TEXT_ALIGN_CENTER as u32;
                // We can't match on enum values if we're matching on a u32
                const JUSTIFY: u32
                    = structs::NS_STYLE_TEXT_ALIGN_JUSTIFY as u32;
                // We can't match on enum values if we're matching on a u32
                const _MOZ_LEFT: u32
                    = structs::NS_STYLE_TEXT_ALIGN_MOZ_LEFT as u32;
                // We can't match on enum values if we're matching on a u32
                const _MOZ_RIGHT: u32
                    = structs::NS_STYLE_TEXT_ALIGN_MOZ_RIGHT as u32;
                // We can't match on enum values if we're matching on a u32
                const _MOZ_CENTER: u32
                    = structs::NS_STYLE_TEXT_ALIGN_MOZ_CENTER as u32;
                // We can't match on enum values if we're matching on a u32
                const CHAR: u32
                    = structs::NS_STYLE_TEXT_ALIGN_CHAR as u32;
                // We can't match on enum values if we're matching on a u32
                const END: u32
                    = structs::NS_STYLE_TEXT_ALIGN_END as u32;
            match kw  {
                    LEFT => T::left,
                    RIGHT => T::right,
                    CENTER => T::center,
                    JUSTIFY => T::justify,
                    _MOZ_LEFT => T::_moz_left,
                    _MOZ_RIGHT => T::_moz_right,
                    _MOZ_CENTER => T::_moz_center,
                    CHAR => T::char,
                    END => T::end,
                x => panic!("Found unexpected value in style struct for text-align property: {:?}", x),
            }
        }
    }

    }

    #[inline] pub fn get_initial_value() -> computed_value::T {
        computed_value::T::start
    }


        use std::fmt;
        use style_traits::ToCss;

        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[derive(Clone, Copy, Debug, Eq, Hash, PartialEq)]
        pub enum SpecifiedValue {
            Keyword(computed_value::T),
            MatchParent,
            MozCenterOrInherit,
        }
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            // MozCenterOrInherit cannot be parsed, only set directly on th elements
            if let Ok(key) = input.try(computed_value::T::parse) {
                Ok(SpecifiedValue::Keyword(key))
            } else {
                input.expect_ident_matching("match-parent")?;
                Ok(SpecifiedValue::MatchParent)
            }
        }
        impl ToCss for SpecifiedValue {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
                match *self {
                    SpecifiedValue::Keyword(key) => key.to_css(dest),
                    SpecifiedValue::MatchParent => dest.write_str("match-parent"),
                    SpecifiedValue::MozCenterOrInherit => Ok(()),
                }
            }
        }

        impl SpecifiedValue {
            pub fn from_gecko_keyword(kw: u32) -> Self {
                use gecko_bindings::structs::NS_STYLE_TEXT_ALIGN_MATCH_PARENT;
                if kw == NS_STYLE_TEXT_ALIGN_MATCH_PARENT {
                    SpecifiedValue::MatchParent
                } else {
                    SpecifiedValue::Keyword(computed_value::T::from_gecko_keyword(kw))
                }
            }
        }
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, context: &Context) -> computed_value::T {
                match *self {
                    SpecifiedValue::Keyword(key) => key,
                    SpecifiedValue::MatchParent => {
                        // on the root <html> element we should still respect the dir
                        // but the parent dir of that element is LTR even if it's <html dir=rtl>
                        // and will only be RTL if certain prefs have been set.
                        // In that case, the default behavior here will set it to left,
                        // but we want to set it to right -- instead set it to the default (`start`),
                        // which will do the right thing in this case (but not the general case)
                        if context.is_root_element {
                            return get_initial_value();
                        }
                        let parent = context.builder.get_parent_inheritedtext().clone_text_align();
                        let ltr = context.builder.inherited_writing_mode().is_bidi_ltr();
                        match (parent, ltr) {
                            (computed_value::T::start, true) => computed_value::T::left,
                            (computed_value::T::start, false) => computed_value::T::right,
                            (computed_value::T::end, true) => computed_value::T::right,
                            (computed_value::T::end, false) => computed_value::T::left,
                            _ => parent
                        }
                    }
                    SpecifiedValue::MozCenterOrInherit => {
                        let parent = context.builder.get_parent_inheritedtext().clone_text_align();
                        if parent == computed_value::T::start {
                            computed_value::T::center
                        } else {
                            parent
                        }
                    }
                }
            }

            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                SpecifiedValue::Keyword(*computed)
            }
        }

        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::TextAlign(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::TextAlign);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_text_align(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_text_align();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_text_align();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::TextAlign)
            }
    }



    
        
    
    /// https://drafts.csswg.org/css-text/#propdef-letter-spacing
    pub mod letter_spacing {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::LetterSpacing as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::LetterSpacing as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::LetterSpacing::normal() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::LetterSpacing::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::LetterSpacing(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::LetterSpacing);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_letter_spacing(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_letter_spacing();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_letter_spacing();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::LetterSpacing)
            }
    }




    
        
    
    /// https://drafts.csswg.org/css-text/#propdef-word-spacing
    pub mod word_spacing {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::WordSpacing as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::WordSpacing as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::WordSpacing::normal() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::WordSpacing::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::WordSpacing(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::WordSpacing);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_word_spacing(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_word_spacing();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_word_spacing();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::WordSpacing)
            }
    }




    


    

    
        
    
    /// https://drafts.csswg.org/css-text/#propdef-white-space
    pub mod white_space {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "normal" => normal,
                    "pre" => pre,
                    "nowrap" => nowrap,
                    "pre-wrap" => pre_wrap,
                    "pre-line" => pre_line,
                    "-moz-pre-space" => _moz_pre_space,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::normal
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::normal
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

            
            
    
    #[cfg(feature = "gecko")]
    impl SpecifiedValue {
        /// Obtain a specified value from a Gecko keyword value
        ///
        /// Intended for use with presentation attributes, not style structs
        pub fn from_gecko_keyword(kw: u32) -> Self {
            use gecko_bindings::structs;
                // We can't match on enum values if we're matching on a u32
                const NORMAL: u32
                    = structs::StyleWhiteSpace::Normal as u32;
                // We can't match on enum values if we're matching on a u32
                const PRE: u32
                    = structs::StyleWhiteSpace::Pre as u32;
                // We can't match on enum values if we're matching on a u32
                const NOWRAP: u32
                    = structs::StyleWhiteSpace::Nowrap as u32;
                // We can't match on enum values if we're matching on a u32
                const PRE_WRAP: u32
                    = structs::StyleWhiteSpace::PreWrap as u32;
                // We can't match on enum values if we're matching on a u32
                const PRE_LINE: u32
                    = structs::StyleWhiteSpace::PreLine as u32;
                // We can't match on enum values if we're matching on a u32
                const _MOZ_PRE_SPACE: u32
                    = structs::StyleWhiteSpace::PreSpace as u32;
            match kw  {
                    NORMAL => SpecifiedValue::normal,
                    PRE => SpecifiedValue::pre,
                    NOWRAP => SpecifiedValue::nowrap,
                    PRE_WRAP => SpecifiedValue::pre_wrap,
                    PRE_LINE => SpecifiedValue::pre_line,
                    _MOZ_PRE_SPACE => SpecifiedValue::_moz_pre_space,
                x => panic!("Found unexpected value in style struct for white-space property: {:?}", x),
            }
        }
    }

    
            
    trivial_to_computed_value!(SpecifiedValue);

        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::WhiteSpace(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::WhiteSpace);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_white_space(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_white_space();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_white_space();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::WhiteSpace)
            }
    }




    
        
    
    
    /// https://drafts.csswg.org/css-text-decor-3/#text-shadow-property
    pub mod text_shadow {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        use std::fmt;
        use style_traits::{Separator, ToCss};

        pub mod single_value {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::ShorthandId;
            #[allow(unused_imports)]
            use selectors::parser::SelectorParseError;
            #[allow(unused_imports)]
            use style_traits::{ParseError, StyleParseError};
            #[allow(unused_imports)]
            use values::computed::{Context, ToComputedValue};
            #[allow(unused_imports)]
            use values::{computed, specified};
            #[allow(unused_imports)]
            use values::{Auto, Either, None_, Normal};
            
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::SimpleShadow as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::SimpleShadow as T;
        }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::SimpleShadow::parse(context, input)
        }
    
        
        }

        /// The definition of the computed value for text-shadow.
        pub mod computed_value {
            pub use super::single_value::computed_value as single_value;
            pub use self::single_value::T as SingleComputedValue;
            use std::vec::IntoIter;
            use values::computed::ComputedVecIter;

            /// The computed value, effectively a list of single values.
            #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
            #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
            #[derive(Clone, Debug, PartialEq)]
            pub struct T(
                pub Vec<single_value::T>,
            );


            pub type Iter<'a, 'cx, 'cx_a> = ComputedVecIter<'a, 'cx, 'cx_a, super::single_value::SpecifiedValue>;

            impl IntoIterator for T {
                type Item = single_value::T;
                type IntoIter = IntoIter<single_value::T>;
                fn into_iter(self) -> Self::IntoIter {
                    self.0.into_iter()
                }
            }
        }

        impl ToCss for computed_value::T {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        dest.write_str("none")?;
                }
                for i in iter {
                    dest.write_str(::style_traits::Comma::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        /// The specified value of text-shadow.
        #[derive(Clone, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        pub struct SpecifiedValue(pub Vec<single_value::SpecifiedValue>);

        impl ToCss for SpecifiedValue {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        dest.write_str("none")?;
                }
                for i in iter {
                    dest.write_str(::style_traits::Comma::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        pub fn get_initial_value() -> computed_value::T {
                computed_value::T(vec![])
        }

        pub fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                -> Result<SpecifiedValue, ParseError<'i>> {
            use style_traits::Separator;

                if input.try(|input| input.expect_ident_matching("none")).is_ok() {
                    return Ok(SpecifiedValue(Vec::new()))
                }

            ::style_traits::Comma::parse(input, |parser| {
                single_value::parse(context, parser)
            }).map(SpecifiedValue)
        }

        pub use self::single_value::SpecifiedValue as SingleSpecifiedValue;

        impl SpecifiedValue {
            pub fn compute_iter<'a, 'cx, 'cx_a>(&'a self, context: &'cx Context<'cx_a>)
                -> computed_value::Iter<'a, 'cx, 'cx_a> {
                computed_value::Iter::new(context, &self.0)
            }
        }

        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, context: &Context) -> computed_value::T {
                computed_value::T(self.compute_iter(context).collect())
            }

            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                SpecifiedValue(computed.0.iter()
                                    .map(ToComputedValue::from_computed_value)
                                    .collect())
            }
        }
    
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::TextShadow(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::TextShadow);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            // In the case of a vector property we want to pass
                            // down an iterator so that this can be computed
                            // without allocation
                            //
                            // However, computing requires a context, but the
                            // style struct being mutated is on the context. We
                            // temporarily remove it, mutate it, and then put it
                            // back. Vector longhands cannot touch their own
                            // style struct whilst computing, else this will
                            // panic.
                            let mut s =
                                context.builder.take_inheritedtext();
                            {
                                let iter = specified_value.compute_iter(context);
                                s.set_text_shadow(iter);
                            }
                            context.builder.put_inheritedtext(s);
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_text_shadow();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_text_shadow();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::TextShadow)
            }
    }





    
    /// https://drafts.csswg.org/css-text-decor/#propdef-text-emphasis-style
    pub mod text_emphasis_style {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
    use computed_values::writing_mode::T as writing_mode;
    use std::fmt;
    use style_traits::ToCss;
    use unicode_segmentation::UnicodeSegmentation;


    pub mod computed_value {
        #[derive(Clone, Debug, PartialEq, ToCss)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf, ToComputedValue))]
        pub enum T {
            Keyword(KeywordValue),
            None,
            String(String),
        }

        #[derive(Clone, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        pub struct KeywordValue {
            pub fill: bool,
            pub shape: super::ShapeKeyword,
        }
    }

    #[derive(Clone, Debug, PartialEq, ToCss)]
    #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
    #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
    pub enum SpecifiedValue {
        Keyword(KeywordValue),
        None,
        String(String),
    }

    #[derive(Clone, Debug, PartialEq)]
    #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
    #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
    pub enum KeywordValue {
        Fill(bool),
        Shape(ShapeKeyword),
        FillAndShape(bool, ShapeKeyword),
    }

    impl ToCss for KeywordValue {
        fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
            if let Some(fill) = self.fill() {
                if fill {
                    dest.write_str("filled")?;
                } else {
                    dest.write_str("open")?;
                }
            }
            if let Some(shape) = self.shape() {
                if self.fill().is_some() {
                    dest.write_str(" ")?;
                }
                shape.to_css(dest)?;
            }
            Ok(())
        }
    }
    impl ToCss for computed_value::KeywordValue {
        fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
            if self.fill {
                dest.write_str("filled")?;
            } else {
                dest.write_str("open")?;
            }
            dest.write_str(" ")?;
            self.shape.to_css(dest)
        }
    }

    impl KeywordValue {
        fn fill(&self) -> Option<bool> {
            match *self {
                KeywordValue::Fill(fill) |
                KeywordValue::FillAndShape(fill,_) => Some(fill),
                _ => None,
            }
        }
        fn shape(&self) -> Option<ShapeKeyword> {
            match *self {
                KeywordValue::Shape(shape) |
                KeywordValue::FillAndShape(_, shape) => Some(shape),
                _ => None,
            }
        }
    }

    define_css_keyword_enum!(ShapeKeyword:
                             "dot" => Dot,
                             "circle" => Circle,
                             "double-circle" => DoubleCircle,
                             "triangle" => Triangle,
                             "sesame" => Sesame);

    impl ShapeKeyword {
        pub fn char(&self, fill: bool) -> &str {
            match *self {
                ShapeKeyword::Dot => if fill { "\u{2022}" } else { "\u{25e6}" },
                ShapeKeyword::Circle =>  if fill { "\u{25cf}" } else { "\u{25cb}" },
                ShapeKeyword::DoubleCircle =>  if fill { "\u{25c9}" } else { "\u{25ce}" },
                ShapeKeyword::Triangle =>  if fill { "\u{25b2}" } else { "\u{25b3}" },
                ShapeKeyword::Sesame =>  if fill { "\u{fe45}" } else { "\u{fe46}" },
            }
        }
    }

    #[inline]
    pub fn get_initial_value() -> computed_value::T {
        computed_value::T::None
    }

    #[inline]
    pub fn get_initial_specified_value() -> SpecifiedValue {
        SpecifiedValue::None
    }

    impl ToComputedValue for SpecifiedValue {
        type ComputedValue = computed_value::T;

        #[inline]
        fn to_computed_value(&self, context: &Context) -> computed_value::T {
            match *self {
                SpecifiedValue::Keyword(ref keyword) => {
                    let default_shape = if context.style().get_inheritedbox()
                                                  .clone_writing_mode() == writing_mode::horizontal_tb {
                        ShapeKeyword::Circle
                    } else {
                        ShapeKeyword::Sesame
                    };
                    computed_value::T::Keyword(computed_value::KeywordValue {
                        fill: keyword.fill().unwrap_or(true),
                        shape: keyword.shape().unwrap_or(default_shape),
                    })
                },
                SpecifiedValue::None => computed_value::T::None,
                SpecifiedValue::String(ref s) => {
                    // Passing `true` to iterate over extended grapheme clusters, following
                    // recommendation at http://www.unicode.org/reports/tr29/#Grapheme_Cluster_Boundaries
                    let string = s.graphemes(true).next().unwrap_or("").to_string();
                    computed_value::T::String(string)
                }
            }
        }
        #[inline]
        fn from_computed_value(computed: &computed_value::T) -> Self {
            match *computed {
                computed_value::T::Keyword(ref keyword) =>
                    SpecifiedValue::Keyword(KeywordValue::FillAndShape(keyword.fill,keyword.shape)),
                computed_value::T::None => SpecifiedValue::None,
                computed_value::T::String(ref string) => SpecifiedValue::String(string.clone())
            }
        }
    }

    pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                         -> Result<SpecifiedValue, ParseError<'i>> {
        if input.try(|input| input.expect_ident_matching("none")).is_ok() {
            return Ok(SpecifiedValue::None);
        }

        if let Ok(s) = input.try(|i| i.expect_string().map(|s| s.as_ref().to_owned())) {
            // Handle <string>
            return Ok(SpecifiedValue::String(s));
        }

        // Handle a pair of keywords
        let mut shape = input.try(ShapeKeyword::parse);
        let fill = if input.try(|input| input.expect_ident_matching("filled")).is_ok() {
            Some(true)
        } else if input.try(|input| input.expect_ident_matching("open")).is_ok() {
            Some(false)
        } else { None };
        if shape.is_err() {
            shape = input.try(ShapeKeyword::parse);
        }

        // At least one of shape or fill must be handled
        let keyword_value = match (fill, shape) {
            (Some(fill), Ok(shape)) => KeywordValue::FillAndShape(fill,shape),
            (Some(fill), Err(_)) => KeywordValue::Fill(fill),
            (None, Ok(shape)) => KeywordValue::Shape(shape),
            _ => return Err(StyleParseError::UnspecifiedError.into()),
        };
        Ok(SpecifiedValue::Keyword(keyword_value))
    }

        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::TextEmphasisStyle(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::TextEmphasisStyle);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = (**specified_value).to_computed_value(context);
                                context.builder.set_text_emphasis_style(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_text_emphasis_style();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_text_emphasis_style();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<Box<SpecifiedValue>, ParseError<'i>> {
                    parse(context, input).map(|result| Box::new(result))
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::TextEmphasisStyle)
            }
    }



    
    /// https://drafts.csswg.org/css-text-decor/#propdef-text-emphasis-position
    pub mod text_emphasis_position {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
    use style_traits::ToCss;

    define_css_keyword_enum!(HorizontalWritingModeValue:
                             "over" => Over,
                             "under" => Under);
    add_impls_for_keyword_enum!(VerticalWritingModeValue);
    define_css_keyword_enum!(VerticalWritingModeValue:
                             "right" => Right,
                             "left" => Left);
    add_impls_for_keyword_enum!(HorizontalWritingModeValue);

    #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
    #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
    #[derive(Clone, Debug, PartialEq, ToComputedValue, ToCss)]
    pub struct SpecifiedValue(pub HorizontalWritingModeValue, pub VerticalWritingModeValue);

    pub mod computed_value {
        pub type T = super::SpecifiedValue;
    }

    pub fn get_initial_value() -> computed_value::T {
        SpecifiedValue(HorizontalWritingModeValue::Over, VerticalWritingModeValue::Right)
    }

    pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                         -> Result<SpecifiedValue, ParseError<'i>> {
       if let Ok(horizontal) = input.try(|input| HorizontalWritingModeValue::parse(input)) {
            let vertical = VerticalWritingModeValue::parse(input)?;
            Ok(SpecifiedValue(horizontal, vertical))
        } else {
            let vertical = VerticalWritingModeValue::parse(input)?;
            let horizontal = HorizontalWritingModeValue::parse(input)?;
            Ok(SpecifiedValue(horizontal, vertical))
        }
    }

        impl SpecifiedValue {
            pub fn from_gecko_keyword(kw: u32) -> Self {
                use gecko_bindings::structs;

                let vert = if kw & structs::NS_STYLE_TEXT_EMPHASIS_POSITION_RIGHT != 0 {
                    VerticalWritingModeValue::Right
                } else {
                    debug_assert!(kw & structs::NS_STYLE_TEXT_EMPHASIS_POSITION_LEFT != 0);
                    VerticalWritingModeValue::Left
                };
                let horiz = if kw & structs::NS_STYLE_TEXT_EMPHASIS_POSITION_OVER != 0 {
                    HorizontalWritingModeValue::Over
                } else {
                    debug_assert!(kw & structs::NS_STYLE_TEXT_EMPHASIS_POSITION_UNDER != 0);
                    HorizontalWritingModeValue::Under
                };
                SpecifiedValue(horiz, vert)
            }
        }

        impl From<u8> for SpecifiedValue {
            fn from(bits: u8) -> SpecifiedValue {
                SpecifiedValue::from_gecko_keyword(bits as u32)
            }
        }

        impl From<SpecifiedValue> for u8 {
            fn from(v: SpecifiedValue) -> u8 {
                use gecko_bindings::structs;

                let mut result = match v.0 {
                    HorizontalWritingModeValue::Over => structs::NS_STYLE_TEXT_EMPHASIS_POSITION_OVER,
                    HorizontalWritingModeValue::Under => structs::NS_STYLE_TEXT_EMPHASIS_POSITION_UNDER,
                };
                match v.1 {
                    VerticalWritingModeValue::Right => {
                        result |= structs::NS_STYLE_TEXT_EMPHASIS_POSITION_RIGHT;
                    }
                    VerticalWritingModeValue::Left => {
                        result |= structs::NS_STYLE_TEXT_EMPHASIS_POSITION_LEFT;
                    }
                };
                result as u8
            }
        }

        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::TextEmphasisPosition(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::TextEmphasisPosition);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_text_emphasis_position(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_text_emphasis_position();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_text_emphasis_position();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::TextEmphasisPosition)
            }
    }



    
        
    
    /// https://drafts.csswg.org/css-text-decor/#propdef-text-emphasis-color
    pub mod text_emphasis_color {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::Color as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::Color as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed_value::T::currentcolor() }
        #[inline] pub fn get_initial_specified_value() -> SpecifiedValue { specified::Color::currentcolor() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::Color::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::TextEmphasisColor(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::TextEmphasisColor);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_text_emphasis_color(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_text_emphasis_color();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_text_emphasis_color();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::TextEmphasisColor)
            }
    }




    
        
    
    /// https://drafts.csswg.org/css-text-3/#tab-size-property
    pub mod _moz_tab_size {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::length::NonNegativeLengthOrNumber as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::length::NonNegativeLengthOrNumber as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { ::values::Either::Second(From::from(8.0)) }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::length::NonNegativeLengthOrNumber::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MozTabSize(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MozTabSize);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set__moz_tab_size(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset__moz_tab_size();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit__moz_tab_size();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MozTabSize)
            }
    }




// CSS Compatibility
// https://compat.spec.whatwg.org

    
        
    
    /// https://compat.spec.whatwg.org/#the-webkit-text-fill-color
    pub mod _webkit_text_fill_color {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::Color as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::Color as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed_value::T::currentcolor() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::Color::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::WebkitTextFillColor(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::WebkitTextFillColor);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set__webkit_text_fill_color(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset__webkit_text_fill_color();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit__webkit_text_fill_color();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::WebkitTextFillColor)
            }
    }




    
        
    
    /// https://compat.spec.whatwg.org/#the-webkit-text-stroke-color
    pub mod _webkit_text_stroke_color {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::Color as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::Color as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed_value::T::currentcolor() }
        #[inline] pub fn get_initial_specified_value() -> SpecifiedValue { specified::Color::currentcolor() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::Color::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::WebkitTextStrokeColor(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::WebkitTextStrokeColor);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set__webkit_text_stroke_color(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset__webkit_text_stroke_color();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit__webkit_text_stroke_color();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::WebkitTextStrokeColor)
            }
    }




    
        
    
    /// https://compat.spec.whatwg.org/#the-webkit-text-stroke-width
    pub mod _webkit_text_stroke_width {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::BorderSideWidth as SpecifiedValue;
        pub mod computed_value {
            pub use ::values::computed::NonNegativeLength as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { ::values::computed::NonNegativeLength::new(0.) }
        #[inline] pub fn get_initial_specified_value() -> SpecifiedValue { specified::BorderSideWidth::Length(specified::Length::zero()) }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::BorderSideWidth::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::WebkitTextStrokeWidth(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::WebkitTextStrokeWidth);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set__webkit_text_stroke_width(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset__webkit_text_stroke_width();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit__webkit_text_stroke_width();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::WebkitTextStrokeWidth)
            }
    }



// CSS Ruby Layout Module Level 1
// https://drafts.csswg.org/css-ruby/

    
    

    
        
    
    /// https://drafts.csswg.org/css-ruby/#ruby-align-property
    pub mod ruby_align {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "space-around" => space_around,
                    "start" => start,
                    "center" => center,
                    "space-between" => space_between,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::space_around
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::space_around
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::space_around => computed_value::T::space_around,
                        SpecifiedValue::start => computed_value::T::start,
                        SpecifiedValue::center => computed_value::T::center,
                        SpecifiedValue::space_between => computed_value::T::space_between,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::space_around => SpecifiedValue::space_around,
                        computed_value::T::start => SpecifiedValue::start,
                        computed_value::T::center => SpecifiedValue::center,
                        computed_value::T::space_between => SpecifiedValue::space_between,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::RubyAlign(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::RubyAlign);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_ruby_align(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_ruby_align();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_ruby_align();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::RubyAlign)
            }
    }





    
    

    
        
    
    /// https://drafts.csswg.org/css-ruby/#ruby-position-property
    pub mod ruby_position {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "over" => over,
                    "under" => under,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::over
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::over
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::over => computed_value::T::over,
                        SpecifiedValue::under => computed_value::T::under,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::over => SpecifiedValue::over,
                        computed_value::T::under => SpecifiedValue::under,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::RubyPosition(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::RubyPosition);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_ruby_position(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_ruby_position();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_ruby_position();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::RubyPosition)
            }
    }




// CSS Writing Modes Module Level 3
// https://drafts.csswg.org/css-writing-modes-3/


    
    

    
        
    
    /// https://drafts.csswg.org/css-writing-modes-3/#text-combine-upright
    pub mod text_combine_upright {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "none" => none,
                    "all" => all,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::none
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::none
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::none => computed_value::T::none,
                        SpecifiedValue::all => computed_value::T::all,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::none => SpecifiedValue::none,
                        computed_value::T::all => SpecifiedValue::all,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::TextCombineUpright(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::TextCombineUpright);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_text_combine_upright(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_text_combine_upright();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_text_combine_upright();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::TextCombineUpright)
            }
    }




// SVG 1.1: Section 11 - Painting: Filling, Stroking and Marker Symbols

    
    

    
        
    
    /// https://www.w3.org/TR/SVG11/painting.html#TextRenderingProperty
    pub mod text_rendering {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "auto" => auto,
                    "optimizespeed" => optimizespeed,
                    "optimizelegibility" => optimizelegibility,
                    "geometricprecision" => geometricprecision,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::auto
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::auto
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::auto => computed_value::T::auto,
                        SpecifiedValue::optimizespeed => computed_value::T::optimizespeed,
                        SpecifiedValue::optimizelegibility => computed_value::T::optimizelegibility,
                        SpecifiedValue::geometricprecision => computed_value::T::geometricprecision,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::auto => SpecifiedValue::auto,
                        computed_value::T::optimizespeed => SpecifiedValue::optimizespeed,
                        computed_value::T::optimizelegibility => SpecifiedValue::optimizelegibility,
                        computed_value::T::geometricprecision => SpecifiedValue::geometricprecision,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::TextRendering(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::TextRendering);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_text_rendering(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_text_rendering();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_text_rendering();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::TextRendering)
            }
    }





    
    

    
        
    
    /// Nonstandard
    pub mod _moz_control_character_visibility {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "hidden" => hidden,
                    "visible" => visible,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::hidden
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::hidden
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::hidden => computed_value::T::hidden,
                        SpecifiedValue::visible => computed_value::T::visible,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::hidden => SpecifiedValue::hidden,
                        computed_value::T::visible => SpecifiedValue::visible,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MozControlCharacterVisibility(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MozControlCharacterVisibility);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set__moz_control_character_visibility(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset__moz_control_character_visibility();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit__moz_control_character_visibility();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MozControlCharacterVisibility)
            }
    }




    /* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */






    
    

    
        
    
    /// https://drafts.csswg.org/css-lists/#propdef-list-style-position
    pub mod list_style_position {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "outside" => outside,
                    "inside" => inside,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::outside
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::outside
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::outside => computed_value::T::outside,
                        SpecifiedValue::inside => computed_value::T::inside,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::outside => SpecifiedValue::outside,
                        computed_value::T::inside => SpecifiedValue::inside,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::ListStylePosition(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::ListStylePosition);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_list_style_position(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_list_style_position();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_list_style_position();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::ListStylePosition)
            }
    }




// TODO(pcwalton): Implement the full set of counter styles per CSS-COUNTER-STYLES [1] 6.1:
//
//     decimal-leading-zero, armenian, upper-armenian, lower-armenian, georgian, lower-roman,
//     upper-roman
//
// TODO(bholley): Missing quite a few gecko properties here as well.
//
// In gecko, {upper,lower}-{roman,alpha} are implemented as @counter-styles in the
// UA, however they can also be set from pres attrs. When @counter-style is supported
// we may need to look into this and handle these differently.
//
// [1]: http://dev.w3.org/csswg/css-counter-styles/
    
    
    /// https://drafts.csswg.org/css-lists/#propdef-list-style-type
    pub mod list_style_type {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
        use values::CustomIdent;
        use values::generics::CounterStyleOrNone;

        pub use self::computed_value::T as SpecifiedValue;

        pub mod computed_value {
            use values::generics::CounterStyleOrNone;

            /// <counter-style> | <string> | none
            #[derive(Clone, Debug, Eq, MallocSizeOf, PartialEq, ToComputedValue, ToCss)]
            pub enum T {
                CounterStyle(CounterStyleOrNone),
                String(String),
            }
        }

        #[cfg(feature = "gecko")]
        impl SpecifiedValue {
            /// Convert from gecko keyword to list-style-type.
            ///
            /// This should only be used for mapping type attribute to
            /// list-style-type, and thus only values possible in that
            /// attribute is considered here.
            pub fn from_gecko_keyword(value: u32) -> Self {
                use gecko_bindings::structs;
                SpecifiedValue::CounterStyle(if value == structs::NS_STYLE_LIST_STYLE_NONE {
                    CounterStyleOrNone::None
                } else {
                    
                    CounterStyleOrNone::Name(CustomIdent(match value {
                        structs::NS_STYLE_LIST_STYLE_DISC => atom!("disc"),
                        structs::NS_STYLE_LIST_STYLE_CIRCLE => atom!("circle"),
                        structs::NS_STYLE_LIST_STYLE_SQUARE => atom!("square"),
                        structs::NS_STYLE_LIST_STYLE_DECIMAL => atom!("decimal"),
                        structs::NS_STYLE_LIST_STYLE_LOWER_ROMAN => atom!("lower-roman"),
                        structs::NS_STYLE_LIST_STYLE_UPPER_ROMAN => atom!("upper-roman"),
                        structs::NS_STYLE_LIST_STYLE_LOWER_ALPHA => atom!("lower-alpha"),
                        structs::NS_STYLE_LIST_STYLE_UPPER_ALPHA => atom!("upper-alpha"),
                        _ => unreachable!("Unknown counter style keyword value"),
                    }))
                })
            }
        }

        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::CounterStyle(CounterStyleOrNone::disc())
        }

        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::CounterStyle(CounterStyleOrNone::disc())
        }

        pub fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            Ok(if let Ok(style) = input.try(|i| CounterStyleOrNone::parse(context, i)) {
                SpecifiedValue::CounterStyle(style)
            } else {
                SpecifiedValue::String(input.expect_string()?.as_ref().to_owned())
            })
        }
    
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::ListStyleType(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::ListStyleType);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = (**specified_value).to_computed_value(context);
                                context.builder.set_list_style_type(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_list_style_type();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_list_style_type();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<Box<SpecifiedValue>, ParseError<'i>> {
                    parse(context, input).map(|result| Box::new(result))
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::ListStyleType)
            }
    }



    
    /// https://drafts.csswg.org/css-lists/#propdef-list-style-image
    pub mod list_style_image {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
    use values::specified::UrlOrNone;
    pub use self::computed_value::T as SpecifiedValue;

    pub mod computed_value {
        use values::specified::UrlOrNone;

        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        #[derive(Clone, Debug, PartialEq, ToCss)]
        pub struct T(pub UrlOrNone);

        // FIXME(nox): This is wrong, there are different types for specified
        // and computed URLs in Servo.
        trivial_to_computed_value!(T);
    }

    #[inline]
    pub fn get_initial_value() -> computed_value::T {
        computed_value::T(Either::Second(None_))
    }
    #[inline]
    pub fn get_initial_specified_value() -> SpecifiedValue {
        SpecifiedValue(Either::Second(None_))
    }
    pub fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                         -> Result<SpecifiedValue,ParseError<'i>> {
        let mut value = input.try(|input| UrlOrNone::parse(context, input))?;
        if let Either::First(ref mut url) = value {
            url.build_image_value();
        }

        return Ok(SpecifiedValue(value));
    }

        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::ListStyleImage(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::ListStyleImage);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = (**specified_value).to_computed_value(context);
                                context.builder.set_list_style_image(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_list_style_image();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_list_style_image();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<Box<SpecifiedValue>, ParseError<'i>> {
                    parse(context, input).map(|result| Box::new(result))
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::ListStyleImage)
            }
    }



    
    /// https://drafts.csswg.org/css-content/#propdef-quotes
    pub mod quotes {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
    use cssparser::serialize_string;
    use std::fmt;
    use style_traits::ToCss;

    pub use self::computed_value::T as SpecifiedValue;

    pub mod computed_value {
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        #[derive(Clone, Debug, PartialEq, ToComputedValue)]
        pub struct T(pub Vec<(String, String)>);
    }

    impl ToCss for SpecifiedValue {
        fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
            if self.0.is_empty() {
                return dest.write_str("none")
            }

            let mut first = true;
            for pair in &self.0 {
                if !first {
                    dest.write_str(" ")?;
                }
                first = false;
                serialize_string(&*pair.0, dest)?;
                dest.write_str(" ")?;
                serialize_string(&*pair.1, dest)?;
            }
            Ok(())
        }
    }

    #[inline]
    pub fn get_initial_value() -> computed_value::T {
        computed_value::T(vec![
            ("\u{201c}".to_owned(), "\u{201d}".to_owned()),
            ("\u{2018}".to_owned(), "\u{2019}".to_owned()),
        ])
    }

    pub fn parse<'i, 't>(_: &ParserContext, input: &mut Parser<'i, 't>)
                         -> Result<SpecifiedValue,ParseError<'i>> {
        if input.try(|input| input.expect_ident_matching("none")).is_ok() {
            return Ok(SpecifiedValue(Vec::new()))
        }

        let mut quotes = Vec::new();
        loop {
            let first = match input.next() {
                Ok(&Token::QuotedString(ref value)) => value.as_ref().to_owned(),
                Ok(t) => return Err(BasicParseError::UnexpectedToken(t.clone()).into()),
                Err(_) => break,
            };
            let second = match input.next() {
                Ok(&Token::QuotedString(ref value)) => value.as_ref().to_owned(),
                Ok(t) => return Err(BasicParseError::UnexpectedToken(t.clone()).into()),
                Err(e) => return Err(e.into()),
            };
            quotes.push((first, second))
        }
        if !quotes.is_empty() {
            Ok(SpecifiedValue(quotes))
        } else {
            Err(StyleParseError::UnspecifiedError.into())
        }
    }

        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::Quotes(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::Quotes);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_quotes(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_quotes();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_quotes();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::Quotes)
            }
    }



    
        
    
    /// Nonstandard (https://developer.mozilla.org/en-US/docs/Web/CSS/-moz-image-region)
    pub mod _moz_image_region {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::ClipRectOrAuto as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::ClipRectOrAuto as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::ClipRectOrAuto::auto() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::ClipRectOrAuto::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MozImageRegion(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MozImageRegion);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = (**specified_value).to_computed_value(context);
                                context.builder.set__moz_image_region(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset__moz_image_region();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit__moz_image_region();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<Box<SpecifiedValue>, ParseError<'i>> {
                    parse(context, input).map(|result| Box::new(result))
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MozImageRegion)
            }
    }



    /* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */





    
    
    
        
    
    /// https://drafts.csswg.org/css-box/#propdef-margin-top
    pub mod margin_top {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::LengthOrPercentageOrAuto as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::LengthOrPercentageOrAuto as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::LengthOrPercentageOrAuto::Length(computed::Length::new(0.)) }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::LengthOrPercentageOrAuto::parse_quirky(context, input, AllowQuirks::Yes)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MarginTop(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MarginTop);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::MarginTop);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_margin_top(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_margin_top();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_margin_top();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MarginTop)
            }
    }


    
    
    
        
    
    /// https://drafts.csswg.org/css-box/#propdef-margin-left
    pub mod margin_left {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::LengthOrPercentageOrAuto as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::LengthOrPercentageOrAuto as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::LengthOrPercentageOrAuto::Length(computed::Length::new(0.)) }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::LengthOrPercentageOrAuto::parse_quirky(context, input, AllowQuirks::Yes)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MarginLeft(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MarginLeft);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::MarginLeft);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_margin_left(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_margin_left();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_margin_left();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MarginLeft)
            }
    }


    
    
    
        
    
    /// https://drafts.csswg.org/css-box/#propdef-margin-bottom
    pub mod margin_bottom {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::LengthOrPercentageOrAuto as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::LengthOrPercentageOrAuto as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::LengthOrPercentageOrAuto::Length(computed::Length::new(0.)) }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::LengthOrPercentageOrAuto::parse_quirky(context, input, AllowQuirks::Yes)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MarginBottom(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MarginBottom);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::MarginBottom);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_margin_bottom(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_margin_bottom();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_margin_bottom();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MarginBottom)
            }
    }


    
    
    
        
    
    /// https://drafts.csswg.org/css-box/#propdef-margin-right
    pub mod margin_right {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::LengthOrPercentageOrAuto as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::LengthOrPercentageOrAuto as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::LengthOrPercentageOrAuto::Length(computed::Length::new(0.)) }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::LengthOrPercentageOrAuto::parse_quirky(context, input, AllowQuirks::Yes)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MarginRight(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MarginRight);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::MarginRight);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_margin_right(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_margin_right();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_margin_right();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MarginRight)
            }
    }


    
    
    
        
    
    /// https://drafts.csswg.org/css-logical-props/#propdef-margin-True
    pub mod margin_block_start {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::LengthOrPercentageOrAuto as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::LengthOrPercentageOrAuto as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::LengthOrPercentageOrAuto::Length(computed::Length::new(0.)) }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::LengthOrPercentageOrAuto::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MarginBlockStart(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MarginBlockStart);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::MarginBlockStart);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            context.rule_cache_conditions.borrow_mut()
                                .set_writing_mode_dependency(context.builder.writing_mode);
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_margin_block_start(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_margin_block_start();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_margin_block_start();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MarginBlockStart)
            }
    }


    
    
    
        
    
    /// https://drafts.csswg.org/css-logical-props/#propdef-margin-True
    pub mod margin_block_end {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::LengthOrPercentageOrAuto as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::LengthOrPercentageOrAuto as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::LengthOrPercentageOrAuto::Length(computed::Length::new(0.)) }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::LengthOrPercentageOrAuto::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MarginBlockEnd(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MarginBlockEnd);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::MarginBlockEnd);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            context.rule_cache_conditions.borrow_mut()
                                .set_writing_mode_dependency(context.builder.writing_mode);
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_margin_block_end(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_margin_block_end();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_margin_block_end();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MarginBlockEnd)
            }
    }


    
    
    
        
    
    /// https://drafts.csswg.org/css-logical-props/#propdef-margin-True
    pub mod margin_inline_start {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::LengthOrPercentageOrAuto as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::LengthOrPercentageOrAuto as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::LengthOrPercentageOrAuto::Length(computed::Length::new(0.)) }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::LengthOrPercentageOrAuto::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MarginInlineStart(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MarginInlineStart);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::MarginInlineStart);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            context.rule_cache_conditions.borrow_mut()
                                .set_writing_mode_dependency(context.builder.writing_mode);
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_margin_inline_start(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_margin_inline_start();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_margin_inline_start();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MarginInlineStart)
            }
    }


    
    
    
        
    
    /// https://drafts.csswg.org/css-logical-props/#propdef-margin-True
    pub mod margin_inline_end {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::LengthOrPercentageOrAuto as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::LengthOrPercentageOrAuto as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::LengthOrPercentageOrAuto::Length(computed::Length::new(0.)) }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::LengthOrPercentageOrAuto::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MarginInlineEnd(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MarginInlineEnd);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::MarginInlineEnd);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            context.rule_cache_conditions.borrow_mut()
                                .set_writing_mode_dependency(context.builder.writing_mode);
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_margin_inline_end(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_margin_inline_end();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_margin_inline_end();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MarginInlineEnd)
            }
    }



    /* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */






// TODO(pcwalton): `invert`

    
        
    
    /// https://drafts.csswg.org/css-ui/#propdef-outline-color
    pub mod outline_color {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::Color as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::Color as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed_value::T::currentcolor() }
        #[inline] pub fn get_initial_specified_value() -> SpecifiedValue { specified::Color::currentcolor() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::Color::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::OutlineColor(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::OutlineColor);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::OutlineColor);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_outline_color(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_outline_color();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_outline_color();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::OutlineColor)
            }
    }




    
    /// https://drafts.csswg.org/css-ui/#propdef-outline-style
    pub mod outline_style {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
    use values::specified::BorderStyle;

    pub type SpecifiedValue = Either<Auto, BorderStyle>;

    impl SpecifiedValue {
        #[inline]
        pub fn none_or_hidden(&self) -> bool {
            match *self {
                Either::First(ref _auto) => false,
                Either::Second(ref border_style) => border_style.none_or_hidden()
            }
        }
    }

    #[inline]
    pub fn get_initial_value() -> computed_value::T {
        Either::Second(BorderStyle::none)
    }

    #[inline]
    pub fn get_initial_specified_value() -> SpecifiedValue {
        Either::Second(BorderStyle::none)
    }

    pub mod computed_value {
        pub type T = super::SpecifiedValue;
    }

    pub fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                         -> Result<SpecifiedValue, ParseError<'i>> {
        SpecifiedValue::parse(context, input)
            .and_then(|result| {
                if let Either::Second(BorderStyle::hidden) = result {
                    // The outline-style property accepts the same values as
                    // border-style, except that 'hidden' is not a legal outline
                    // style.
                    Err(SelectorParseError::UnexpectedIdent("hidden".into()).into())
                } else {
                    Ok(result)
                }
            })
    }

        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::OutlineStyle(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::OutlineStyle);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::OutlineStyle);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_outline_style(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_outline_style();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_outline_style();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::OutlineStyle)
            }
    }



    
        
    
    /// https://drafts.csswg.org/css-ui/#propdef-outline-width
    pub mod outline_width {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::BorderSideWidth as SpecifiedValue;
        pub mod computed_value {
            pub use ::values::computed::NonNegativeLength as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { ::values::computed::NonNegativeLength::new(3.) }
        #[inline] pub fn get_initial_specified_value() -> SpecifiedValue { specified::BorderSideWidth::Medium }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::BorderSideWidth::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::OutlineWidth(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::OutlineWidth);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::OutlineWidth);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_outline_width(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_outline_width();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_outline_width();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::OutlineWidth)
            }
    }



// The -moz-outline-radius-* properties are non-standard and not on a standards track.
    
    
        
    
    /// Nonstandard (https://developer.mozilla.org/en-US/docs/Web/CSS/-moz-outline-radius)
    pub mod _moz_outline_radius_topleft {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::BorderCornerRadius as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::BorderCornerRadius as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::BorderCornerRadius::zero() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::BorderCornerRadius::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MozOutlineRadiusTopleft(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MozOutlineRadiusTopleft);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::MozOutlineRadiusTopleft);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = (**specified_value).to_computed_value(context);
                                context.builder.set__moz_outline_radius_topleft(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset__moz_outline_radius_topleft();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit__moz_outline_radius_topleft();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<Box<SpecifiedValue>, ParseError<'i>> {
                    parse(context, input).map(|result| Box::new(result))
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MozOutlineRadiusTopleft)
            }
    }


    
    
        
    
    /// Nonstandard (https://developer.mozilla.org/en-US/docs/Web/CSS/-moz-outline-radius)
    pub mod _moz_outline_radius_topright {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::BorderCornerRadius as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::BorderCornerRadius as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::BorderCornerRadius::zero() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::BorderCornerRadius::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MozOutlineRadiusTopright(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MozOutlineRadiusTopright);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::MozOutlineRadiusTopright);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = (**specified_value).to_computed_value(context);
                                context.builder.set__moz_outline_radius_topright(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset__moz_outline_radius_topright();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit__moz_outline_radius_topright();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<Box<SpecifiedValue>, ParseError<'i>> {
                    parse(context, input).map(|result| Box::new(result))
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MozOutlineRadiusTopright)
            }
    }


    
    
        
    
    /// Nonstandard (https://developer.mozilla.org/en-US/docs/Web/CSS/-moz-outline-radius)
    pub mod _moz_outline_radius_bottomright {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::BorderCornerRadius as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::BorderCornerRadius as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::BorderCornerRadius::zero() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::BorderCornerRadius::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MozOutlineRadiusBottomright(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MozOutlineRadiusBottomright);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::MozOutlineRadiusBottomright);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = (**specified_value).to_computed_value(context);
                                context.builder.set__moz_outline_radius_bottomright(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset__moz_outline_radius_bottomright();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit__moz_outline_radius_bottomright();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<Box<SpecifiedValue>, ParseError<'i>> {
                    parse(context, input).map(|result| Box::new(result))
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MozOutlineRadiusBottomright)
            }
    }


    
    
        
    
    /// Nonstandard (https://developer.mozilla.org/en-US/docs/Web/CSS/-moz-outline-radius)
    pub mod _moz_outline_radius_bottomleft {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::BorderCornerRadius as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::BorderCornerRadius as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::BorderCornerRadius::zero() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::BorderCornerRadius::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MozOutlineRadiusBottomleft(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MozOutlineRadiusBottomleft);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::MozOutlineRadiusBottomleft);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = (**specified_value).to_computed_value(context);
                                context.builder.set__moz_outline_radius_bottomleft(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset__moz_outline_radius_bottomleft();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit__moz_outline_radius_bottomleft();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<Box<SpecifiedValue>, ParseError<'i>> {
                    parse(context, input).map(|result| Box::new(result))
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MozOutlineRadiusBottomleft)
            }
    }




    
        
    
    /// https://drafts.csswg.org/css-ui/#propdef-outline-offset
    pub mod outline_offset {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::Length as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::Length as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { ::values::computed::Length::new(0.) }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::Length::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::OutlineOffset(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::OutlineOffset);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::OutlineOffset);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_outline_offset(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_outline_offset();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_outline_offset();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::OutlineOffset)
            }
    }



    /* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */





// APPLIES_TO_PLACEHOLDER so we can set it in UA  stylesheets.  But we use a
// !important value there, so pages can't set it.
    
    
    
        
    
    /// https://drafts.csswg.org/css-box/#propdef-padding-top
    pub mod padding_top {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::NonNegativeLengthOrPercentage as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::NonNegativeLengthOrPercentage as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::NonNegativeLengthOrPercentage::zero() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::NonNegativeLengthOrPercentage::parse_quirky(context, input, AllowQuirks::Yes)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::PaddingTop(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::PaddingTop);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::PaddingTop);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_padding_top(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_padding_top();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_padding_top();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::PaddingTop)
            }
    }


    
    
    
        
    
    /// https://drafts.csswg.org/css-box/#propdef-padding-left
    pub mod padding_left {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::NonNegativeLengthOrPercentage as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::NonNegativeLengthOrPercentage as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::NonNegativeLengthOrPercentage::zero() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::NonNegativeLengthOrPercentage::parse_quirky(context, input, AllowQuirks::Yes)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::PaddingLeft(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::PaddingLeft);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::PaddingLeft);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_padding_left(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_padding_left();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_padding_left();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::PaddingLeft)
            }
    }


    
    
    
        
    
    /// https://drafts.csswg.org/css-box/#propdef-padding-bottom
    pub mod padding_bottom {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::NonNegativeLengthOrPercentage as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::NonNegativeLengthOrPercentage as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::NonNegativeLengthOrPercentage::zero() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::NonNegativeLengthOrPercentage::parse_quirky(context, input, AllowQuirks::Yes)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::PaddingBottom(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::PaddingBottom);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::PaddingBottom);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_padding_bottom(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_padding_bottom();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_padding_bottom();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::PaddingBottom)
            }
    }


    
    
    
        
    
    /// https://drafts.csswg.org/css-box/#propdef-padding-right
    pub mod padding_right {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::NonNegativeLengthOrPercentage as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::NonNegativeLengthOrPercentage as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::NonNegativeLengthOrPercentage::zero() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::NonNegativeLengthOrPercentage::parse_quirky(context, input, AllowQuirks::Yes)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::PaddingRight(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::PaddingRight);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::PaddingRight);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_padding_right(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_padding_right();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_padding_right();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::PaddingRight)
            }
    }


    
    
    
        
    
    /// https://drafts.csswg.org/css-logical-props/#propdef-padding-True
    pub mod padding_block_start {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::NonNegativeLengthOrPercentage as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::NonNegativeLengthOrPercentage as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::NonNegativeLengthOrPercentage::zero() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::NonNegativeLengthOrPercentage::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::PaddingBlockStart(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::PaddingBlockStart);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::PaddingBlockStart);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            context.rule_cache_conditions.borrow_mut()
                                .set_writing_mode_dependency(context.builder.writing_mode);
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_padding_block_start(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_padding_block_start();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_padding_block_start();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::PaddingBlockStart)
            }
    }


    
    
    
        
    
    /// https://drafts.csswg.org/css-logical-props/#propdef-padding-True
    pub mod padding_block_end {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::NonNegativeLengthOrPercentage as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::NonNegativeLengthOrPercentage as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::NonNegativeLengthOrPercentage::zero() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::NonNegativeLengthOrPercentage::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::PaddingBlockEnd(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::PaddingBlockEnd);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::PaddingBlockEnd);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            context.rule_cache_conditions.borrow_mut()
                                .set_writing_mode_dependency(context.builder.writing_mode);
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_padding_block_end(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_padding_block_end();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_padding_block_end();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::PaddingBlockEnd)
            }
    }


    
    
    
        
    
    /// https://drafts.csswg.org/css-logical-props/#propdef-padding-True
    pub mod padding_inline_start {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::NonNegativeLengthOrPercentage as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::NonNegativeLengthOrPercentage as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::NonNegativeLengthOrPercentage::zero() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::NonNegativeLengthOrPercentage::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::PaddingInlineStart(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::PaddingInlineStart);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::PaddingInlineStart);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            context.rule_cache_conditions.borrow_mut()
                                .set_writing_mode_dependency(context.builder.writing_mode);
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_padding_inline_start(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_padding_inline_start();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_padding_inline_start();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::PaddingInlineStart)
            }
    }


    
    
    
        
    
    /// https://drafts.csswg.org/css-logical-props/#propdef-padding-True
    pub mod padding_inline_end {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::NonNegativeLengthOrPercentage as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::NonNegativeLengthOrPercentage as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::NonNegativeLengthOrPercentage::zero() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::NonNegativeLengthOrPercentage::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::PaddingInlineEnd(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::PaddingInlineEnd);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::PaddingInlineEnd);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            context.rule_cache_conditions.borrow_mut()
                                .set_writing_mode_dependency(context.builder.writing_mode);
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_padding_inline_end(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_padding_inline_end();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_padding_inline_end();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::PaddingInlineEnd)
            }
    }



    /* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */






    
    /// https://drafts.csswg.org/css-ui/#cursor
    pub mod cursor {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
    pub use self::computed_value::T as SpecifiedValue;
    #[cfg(feature = "gecko")]
    use values::specified::url::SpecifiedUrl;

    pub mod computed_value {
        #[cfg(feature = "gecko")]
        use std::fmt;
        #[cfg(feature = "gecko")]
        use style_traits::ToCss;
        use style_traits::cursor::Cursor;
        #[cfg(feature = "gecko")]
        use values::specified::url::SpecifiedUrl;

        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        #[derive(Clone, Copy, Debug, PartialEq, ToComputedValue, ToCss)]
        pub enum Keyword {
            Auto,
            Cursor(Cursor),
        }

        #[cfg(not(feature = "gecko"))]
        pub type T = Keyword;

        #[cfg(feature = "gecko")]
        #[derive(Clone, Debug, MallocSizeOf, PartialEq, ToComputedValue)]
        pub struct Image {
            pub url: SpecifiedUrl,
            pub hotspot: Option<(f32, f32)>,
        }

        #[cfg(feature = "gecko")]
        #[derive(Clone, Debug, MallocSizeOf, PartialEq, ToComputedValue)]
        pub struct T {
            pub images: Vec<Image>,
            pub keyword: Keyword,
        }

        #[cfg(feature = "gecko")]
        impl ToCss for Image {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
                self.url.to_css(dest)?;
                if let Some((x, y)) = self.hotspot {
                    dest.write_str(" ")?;
                    x.to_css(dest)?;
                    dest.write_str(" ")?;
                    y.to_css(dest)?;
                }
                Ok(())
            }
        }

        #[cfg(feature = "gecko")]
        impl ToCss for T {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
                for url in &self.images {
                    url.to_css(dest)?;
                    dest.write_str(", ")?;
                }
                self.keyword.to_css(dest)
            }
        }
    }

    #[cfg(not(feature = "gecko"))]
    #[inline]
    pub fn get_initial_value() -> computed_value::T {
        computed_value::Keyword::Auto
    }

    #[cfg(feature = "gecko")]
    #[inline]
    pub fn get_initial_value() -> computed_value::T {
        computed_value::T {
            images: vec![],
            keyword: computed_value::Keyword::Auto
        }
    }

    impl Parse for computed_value::Keyword {
        fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                         -> Result<computed_value::Keyword, ParseError<'i>> {
            use std::ascii::AsciiExt;
            use style_traits::cursor::Cursor;
            let ident = input.expect_ident()?;
            if ident.eq_ignore_ascii_case("auto") {
                Ok(computed_value::Keyword::Auto)
            } else {
                Cursor::from_css_keyword(&ident)
                    .map(computed_value::Keyword::Cursor)
                    .map_err(|()| SelectorParseError::UnexpectedIdent(ident.clone()).into())
            }
        }
    }

    #[cfg(feature = "gecko")]
    fn parse_image<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                           -> Result<computed_value::Image, ParseError<'i>> {
        Ok(computed_value::Image {
            url: SpecifiedUrl::parse(context, input)?,
            hotspot: match input.try(|input| input.expect_number()) {
                Ok(number) => Some((number, input.expect_number()?)),
                Err(_) => None,
            },
        })
    }

    #[cfg(not(feature = "gecko"))]
    pub fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                         -> Result<SpecifiedValue, ParseError<'i>> {
        computed_value::Keyword::parse(context, input)
    }

    /// cursor: [<url> [<number> <number>]?]# [auto | default | ...]
    #[cfg(feature = "gecko")]
    pub fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                         -> Result<SpecifiedValue, ParseError<'i>> {
        let mut images = vec![];
        loop {
            match input.try(|input| parse_image(context, input)) {
                Ok(mut image) => {
                    image.url.build_image_value();
                    images.push(image)
                }
                Err(_) => break,
            }
            input.expect_comma()?;
        }

        Ok(computed_value::T {
            images: images,
            keyword: computed_value::Keyword::parse(context, input)?,
        })
    }

        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::Cursor(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::Cursor);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = (**specified_value).to_computed_value(context);
                                context.builder.set_cursor(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_cursor();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_cursor();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<Box<SpecifiedValue>, ParseError<'i>> {
                    parse(context, input).map(|result| Box::new(result))
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::Cursor)
            }
    }


// NB: `pointer-events: auto` (and use of `pointer-events` in anything that isn't SVG, in fact)
// is nonstandard, slated for CSS4-UI.
// TODO(pcwalton): SVG-only values.

    
    

    
        
    
    /// https://www.w3.org/TR/SVG11/interact.html#PointerEventsProperty
    pub mod pointer_events {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "auto" => auto,
                    "none" => none,
                    "visiblepainted" => visiblepainted,
                    "visiblefill" => visiblefill,
                    "visiblestroke" => visiblestroke,
                    "visible" => visible,
                    "painted" => painted,
                    "fill" => fill,
                    "stroke" => stroke,
                    "all" => all,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::auto
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::auto
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::auto => computed_value::T::auto,
                        SpecifiedValue::none => computed_value::T::none,
                        SpecifiedValue::visiblepainted => computed_value::T::visiblepainted,
                        SpecifiedValue::visiblefill => computed_value::T::visiblefill,
                        SpecifiedValue::visiblestroke => computed_value::T::visiblestroke,
                        SpecifiedValue::visible => computed_value::T::visible,
                        SpecifiedValue::painted => computed_value::T::painted,
                        SpecifiedValue::fill => computed_value::T::fill,
                        SpecifiedValue::stroke => computed_value::T::stroke,
                        SpecifiedValue::all => computed_value::T::all,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::auto => SpecifiedValue::auto,
                        computed_value::T::none => SpecifiedValue::none,
                        computed_value::T::visiblepainted => SpecifiedValue::visiblepainted,
                        computed_value::T::visiblefill => SpecifiedValue::visiblefill,
                        computed_value::T::visiblestroke => SpecifiedValue::visiblestroke,
                        computed_value::T::visible => SpecifiedValue::visible,
                        computed_value::T::painted => SpecifiedValue::painted,
                        computed_value::T::fill => SpecifiedValue::fill,
                        computed_value::T::stroke => SpecifiedValue::stroke,
                        computed_value::T::all => SpecifiedValue::all,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::PointerEvents(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::PointerEvents);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_pointer_events(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_pointer_events();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_pointer_events();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::PointerEvents)
            }
    }





    
    

    
        
    
    /// Nonstandard (https://developer.mozilla.org/en-US/docs/Web/CSS/-moz-user-input)
    pub mod _moz_user_input {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "auto" => auto,
                    "none" => none,
                    "enabled" => enabled,
                    "disabled" => disabled,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::auto
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::auto
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::auto => computed_value::T::auto,
                        SpecifiedValue::none => computed_value::T::none,
                        SpecifiedValue::enabled => computed_value::T::enabled,
                        SpecifiedValue::disabled => computed_value::T::disabled,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::auto => SpecifiedValue::auto,
                        computed_value::T::none => SpecifiedValue::none,
                        computed_value::T::enabled => SpecifiedValue::enabled,
                        computed_value::T::disabled => SpecifiedValue::disabled,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MozUserInput(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MozUserInput);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set__moz_user_input(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset__moz_user_input();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit__moz_user_input();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MozUserInput)
            }
    }





    
    

    
        
    
    /// Nonstandard (https://developer.mozilla.org/en-US/docs/Web/CSS/-moz-user-modify)
    pub mod _moz_user_modify {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "read-only" => read_only,
                    "read-write" => read_write,
                    "write-only" => write_only,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::read_only
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::read_only
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

            
            
    
    #[cfg(feature = "gecko")]
    impl SpecifiedValue {
        /// Obtain a specified value from a Gecko keyword value
        ///
        /// Intended for use with presentation attributes, not style structs
        pub fn from_gecko_keyword(kw: u32) -> Self {
            use gecko_bindings::structs;
                // We can't match on enum values if we're matching on a u32
                const READ_ONLY: u32
                    = structs::StyleUserModify::ReadOnly as u32;
                // We can't match on enum values if we're matching on a u32
                const READ_WRITE: u32
                    = structs::StyleUserModify::ReadWrite as u32;
                // We can't match on enum values if we're matching on a u32
                const WRITE_ONLY: u32
                    = structs::StyleUserModify::WriteOnly as u32;
            match kw  {
                    READ_ONLY => SpecifiedValue::read_only,
                    READ_WRITE => SpecifiedValue::read_write,
                    WRITE_ONLY => SpecifiedValue::write_only,
                x => panic!("Found unexpected value in style struct for -moz-user-modify property: {:?}", x),
            }
        }
    }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::read_only => computed_value::T::read_only,
                        SpecifiedValue::read_write => computed_value::T::read_write,
                        SpecifiedValue::write_only => computed_value::T::write_only,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::read_only => SpecifiedValue::read_only,
                        computed_value::T::read_write => SpecifiedValue::read_write,
                        computed_value::T::write_only => SpecifiedValue::write_only,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MozUserModify(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MozUserModify);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set__moz_user_modify(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset__moz_user_modify();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit__moz_user_modify();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MozUserModify)
            }
    }





    
    

    
        
    
    /// Nonstandard (https://developer.mozilla.org/en-US/docs/Web/CSS/-moz-user-focus)
    pub mod _moz_user_focus {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "none" => none,
                    "ignore" => ignore,
                    "normal" => normal,
                    "select-after" => select_after,
                    "select-before" => select_before,
                    "select-menu" => select_menu,
                    "select-same" => select_same,
                    "select-all" => select_all,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::none
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::none
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::none => computed_value::T::none,
                        SpecifiedValue::ignore => computed_value::T::ignore,
                        SpecifiedValue::normal => computed_value::T::normal,
                        SpecifiedValue::select_after => computed_value::T::select_after,
                        SpecifiedValue::select_before => computed_value::T::select_before,
                        SpecifiedValue::select_menu => computed_value::T::select_menu,
                        SpecifiedValue::select_same => computed_value::T::select_same,
                        SpecifiedValue::select_all => computed_value::T::select_all,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::none => SpecifiedValue::none,
                        computed_value::T::ignore => SpecifiedValue::ignore,
                        computed_value::T::normal => SpecifiedValue::normal,
                        computed_value::T::select_after => SpecifiedValue::select_after,
                        computed_value::T::select_before => SpecifiedValue::select_before,
                        computed_value::T::select_menu => SpecifiedValue::select_menu,
                        computed_value::T::select_same => SpecifiedValue::select_same,
                        computed_value::T::select_all => SpecifiedValue::select_all,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MozUserFocus(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MozUserFocus);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set__moz_user_focus(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset__moz_user_focus();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit__moz_user_focus();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MozUserFocus)
            }
    }





    
        
    
    /// https://drafts.csswg.org/css-ui/#caret-color
    pub mod caret_color {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::ColorOrAuto as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::ColorOrAuto as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { Either::Second(Auto) }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::ColorOrAuto::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::CaretColor(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::CaretColor);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = (**specified_value).to_computed_value(context);
                                context.builder.set_caret_color(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_caret_color();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_caret_color();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<Box<SpecifiedValue>, ParseError<'i>> {
                    parse(context, input).map(|result| Box::new(result))
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::CaretColor)
            }
    }



    /* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */







// "top" / "left" / "bottom" / "right"
    
    
        
    
    /// https://www.w3.org/TR/CSS2/visuren.html#propdef-top
    pub mod top {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::LengthOrPercentageOrAuto as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::LengthOrPercentageOrAuto as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::LengthOrPercentageOrAuto::Auto }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::LengthOrPercentageOrAuto::parse_quirky(context, input, AllowQuirks::Yes)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::Top(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::Top);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::Top);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_top(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_top();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_top();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::Top)
            }
    }


    
    
        
    
    /// https://www.w3.org/TR/CSS2/visuren.html#propdef-left
    pub mod left {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::LengthOrPercentageOrAuto as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::LengthOrPercentageOrAuto as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::LengthOrPercentageOrAuto::Auto }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::LengthOrPercentageOrAuto::parse_quirky(context, input, AllowQuirks::Yes)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::Left(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::Left);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::Left);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_left(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_left();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_left();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::Left)
            }
    }


    
    
        
    
    /// https://www.w3.org/TR/CSS2/visuren.html#propdef-bottom
    pub mod bottom {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::LengthOrPercentageOrAuto as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::LengthOrPercentageOrAuto as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::LengthOrPercentageOrAuto::Auto }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::LengthOrPercentageOrAuto::parse_quirky(context, input, AllowQuirks::Yes)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::Bottom(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::Bottom);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::Bottom);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_bottom(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_bottom();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_bottom();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::Bottom)
            }
    }


    
    
        
    
    /// https://www.w3.org/TR/CSS2/visuren.html#propdef-right
    pub mod right {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::LengthOrPercentageOrAuto as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::LengthOrPercentageOrAuto as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::LengthOrPercentageOrAuto::Auto }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::LengthOrPercentageOrAuto::parse_quirky(context, input, AllowQuirks::Yes)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::Right(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::Right);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::Right);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_right(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_right();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_right();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::Right)
            }
    }


// offset-* logical properties, map to "top" / "left" / "bottom" / "right"
    
    
        
    
    /// https://drafts.csswg.org/css-logical-props/#propdef-offset-block-start
    pub mod offset_block_start {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::LengthOrPercentageOrAuto as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::LengthOrPercentageOrAuto as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::LengthOrPercentageOrAuto::Auto }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::LengthOrPercentageOrAuto::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::OffsetBlockStart(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::OffsetBlockStart);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::OffsetBlockStart);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            context.rule_cache_conditions.borrow_mut()
                                .set_writing_mode_dependency(context.builder.writing_mode);
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_offset_block_start(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_offset_block_start();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_offset_block_start();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::OffsetBlockStart)
            }
    }


    
    
        
    
    /// https://drafts.csswg.org/css-logical-props/#propdef-offset-block-end
    pub mod offset_block_end {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::LengthOrPercentageOrAuto as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::LengthOrPercentageOrAuto as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::LengthOrPercentageOrAuto::Auto }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::LengthOrPercentageOrAuto::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::OffsetBlockEnd(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::OffsetBlockEnd);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::OffsetBlockEnd);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            context.rule_cache_conditions.borrow_mut()
                                .set_writing_mode_dependency(context.builder.writing_mode);
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_offset_block_end(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_offset_block_end();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_offset_block_end();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::OffsetBlockEnd)
            }
    }


    
    
        
    
    /// https://drafts.csswg.org/css-logical-props/#propdef-offset-inline-start
    pub mod offset_inline_start {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::LengthOrPercentageOrAuto as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::LengthOrPercentageOrAuto as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::LengthOrPercentageOrAuto::Auto }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::LengthOrPercentageOrAuto::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::OffsetInlineStart(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::OffsetInlineStart);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::OffsetInlineStart);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            context.rule_cache_conditions.borrow_mut()
                                .set_writing_mode_dependency(context.builder.writing_mode);
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_offset_inline_start(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_offset_inline_start();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_offset_inline_start();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::OffsetInlineStart)
            }
    }


    
    
        
    
    /// https://drafts.csswg.org/css-logical-props/#propdef-offset-inline-end
    pub mod offset_inline_end {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::LengthOrPercentageOrAuto as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::LengthOrPercentageOrAuto as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::LengthOrPercentageOrAuto::Auto }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::LengthOrPercentageOrAuto::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::OffsetInlineEnd(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::OffsetInlineEnd);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::OffsetInlineEnd);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            context.rule_cache_conditions.borrow_mut()
                                .set_writing_mode_dependency(context.builder.writing_mode);
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_offset_inline_end(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_offset_inline_end();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_offset_inline_end();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::OffsetInlineEnd)
            }
    }



#[cfg(feature = "gecko")]
macro_rules! impl_align_conversions {
    ($name: path) => {
        impl From<u8> for $name {
            fn from(bits: u8) -> $name {
                $name(::values::specified::align::AlignFlags::from_bits(bits)
                      .expect("bits contain valid flag"))
            }
        }

        impl From<$name> for u8 {
            fn from(v: $name) -> u8 {
                v.0.bits()
            }
        }
    };
}


    
        
    
    /// https://www.w3.org/TR/CSS2/visuren.html#z-index
    pub mod z_index {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::IntegerOrAuto as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::IntegerOrAuto as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { Either::Second(Auto) }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::IntegerOrAuto::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::ZIndex(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::ZIndex);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::ZIndex);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_z_index(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_z_index();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_z_index();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::ZIndex)
            }
    }




// CSS Flexible Box Layout Module Level 1
// http://www.w3.org/TR/css3-flexbox/

// Flex container properties

    
    

    
        
    
    /// https://drafts.csswg.org/css-flexbox/#flex-direction-property
    pub mod flex_direction {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "row" => row,
                    "row-reverse" => row_reverse,
                    "column" => column,
                    "column-reverse" => column_reverse,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::row
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::row
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::row => computed_value::T::row,
                        SpecifiedValue::row_reverse => computed_value::T::row_reverse,
                        SpecifiedValue::column => computed_value::T::column,
                        SpecifiedValue::column_reverse => computed_value::T::column_reverse,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::row => SpecifiedValue::row,
                        computed_value::T::row_reverse => SpecifiedValue::row_reverse,
                        computed_value::T::column => SpecifiedValue::column,
                        computed_value::T::column_reverse => SpecifiedValue::column_reverse,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::FlexDirection(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::FlexDirection);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::FlexDirection);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_flex_direction(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_flex_direction();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_flex_direction();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::FlexDirection)
            }
    }





    
    

    
        
    
    /// https://drafts.csswg.org/css-flexbox/#flex-wrap-property
    pub mod flex_wrap {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "nowrap" => nowrap,
                    "wrap" => wrap,
                    "wrap-reverse" => wrap_reverse,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::nowrap
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::nowrap
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::nowrap => computed_value::T::nowrap,
                        SpecifiedValue::wrap => computed_value::T::wrap,
                        SpecifiedValue::wrap_reverse => computed_value::T::wrap_reverse,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::nowrap => SpecifiedValue::nowrap,
                        computed_value::T::wrap => SpecifiedValue::wrap,
                        computed_value::T::wrap_reverse => SpecifiedValue::wrap_reverse,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::FlexWrap(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::FlexWrap);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::FlexWrap);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_flex_wrap(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_flex_wrap();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_flex_wrap();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::FlexWrap)
            }
    }




    
    
        
    
    /// https://drafts.csswg.org/css-align/#propdef-justify-content
    pub mod justify_content {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::AlignJustifyContent as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::AlignJustifyContent as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { specified::AlignJustifyContent::normal() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::AlignJustifyContent::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::JustifyContent(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::JustifyContent);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::JustifyContent);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_justify_content(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_justify_content();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_justify_content();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::JustifyContent)
            }
    }



    
    
        
    
    /// https://drafts.csswg.org/css-align/#propdef-align-content
    pub mod align_content {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::AlignJustifyContent as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::AlignJustifyContent as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { specified::AlignJustifyContent::normal() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::AlignJustifyContent::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::AlignContent(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::AlignContent);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::AlignContent);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_align_content(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_align_content();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_align_content();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::AlignContent)
            }
    }



    
    
        
    
    /// https://drafts.csswg.org/css-align/#propdef-align-items
    pub mod align_items {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::AlignItems as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::AlignItems as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { specified::AlignItems::normal() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::AlignItems::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::AlignItems(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::AlignItems);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::AlignItems);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_align_items(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_align_items();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_align_items();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::AlignItems)
            }
    }



    #[cfg(feature = "gecko")]
    impl_align_conversions!(::values::specified::align::AlignItems);

    
    
        
    
    /// https://drafts.csswg.org/css-align/#propdef-justify-items
    pub mod justify_items {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::JustifyItems as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::JustifyItems as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::JustifyItems::auto() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::JustifyItems::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::JustifyItems(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::JustifyItems);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::JustifyItems);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_justify_items(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_justify_items();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_justify_items();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::JustifyItems)
            }
    }



    #[cfg(feature = "gecko")]
    impl_align_conversions!(::values::specified::align::JustifyItems);

// Flex item properties

    
        
    
    /// https://drafts.csswg.org/css-flexbox/#flex-grow-property
    pub mod flex_grow {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::NonNegativeNumber as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::NonNegativeNumber as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { From::from(0.0) }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::NonNegativeNumber::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::FlexGrow(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::FlexGrow);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::FlexGrow);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_flex_grow(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_flex_grow();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_flex_grow();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::FlexGrow)
            }
    }




    
        
    
    /// https://drafts.csswg.org/css-flexbox/#flex-shrink-property
    pub mod flex_shrink {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::NonNegativeNumber as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::NonNegativeNumber as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { From::from(1.0) }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::NonNegativeNumber::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::FlexShrink(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::FlexShrink);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::FlexShrink);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_flex_shrink(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_flex_shrink();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_flex_shrink();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::FlexShrink)
            }
    }



// https://drafts.csswg.org/css-align/#align-self-property
    
    
        
    
    /// https://drafts.csswg.org/css-align/#align-self-property
    pub mod align_self {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::AlignJustifySelf as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::AlignJustifySelf as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { specified::AlignJustifySelf::auto() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::AlignJustifySelf::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::AlignSelf(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::AlignSelf);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::AlignSelf);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_align_self(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_align_self();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_align_self();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::AlignSelf)
            }
    }



    
    
        
    
    /// https://drafts.csswg.org/css-align/#justify-self-property
    pub mod justify_self {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::AlignJustifySelf as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::AlignJustifySelf as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { specified::AlignJustifySelf::auto() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::AlignJustifySelf::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::JustifySelf(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::JustifySelf);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::JustifySelf);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_justify_self(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_justify_self();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_justify_self();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::JustifySelf)
            }
    }



    #[cfg(feature = "gecko")]
    impl_align_conversions!(::values::specified::align::AlignJustifySelf);

// https://drafts.csswg.org/css-flexbox/#propdef-order

    
        
    
    /// https://drafts.csswg.org/css-flexbox/#order-property
    pub mod order {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::Integer as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::Integer as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { 0 }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::Integer::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::Order(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::Order);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::Order);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_order(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_order();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_order();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::Order)
            }
    }



    // FIXME: Gecko doesn't support content value yet.
    
    
    
    /// https://drafts.csswg.org/css-flexbox/#flex-basis-property
    pub mod flex_basis {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            use values::specified::AllowQuirks;
        use values::specified::MozLength;

        pub mod computed_value {
            pub type T = ::values::computed::MozLength;
        }

        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        #[derive(Clone, Debug, PartialEq, ToCss)]
        pub struct SpecifiedValue(pub MozLength);

        impl SpecifiedValue {
            /// Returns the `auto` value.
            pub fn auto() -> Self {
                use values::specified::length::LengthOrPercentageOrAuto;
                SpecifiedValue(MozLength::LengthOrPercentageOrAuto(LengthOrPercentageOrAuto::Auto))
            }

            /// Returns a value representing a `0` length.
            pub fn zero() -> Self {
                use values::specified::length::LengthOrPercentageOrAuto;
                SpecifiedValue(MozLength::LengthOrPercentageOrAuto(LengthOrPercentageOrAuto::zero()))
            }

            /// Returns a value representing a `0%` length.
            pub fn zero_percent() -> Self {
                use values::specified::length::LengthOrPercentageOrAuto;
                SpecifiedValue(MozLength::LengthOrPercentageOrAuto(LengthOrPercentageOrAuto::zero_percent()))
            }
        }

        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            use values::computed::MozLength;
            MozLength::auto()
        }
        fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                         -> Result<SpecifiedValue, ParseError<'i>> {
            let ret = MozLength::parse_quirky(context, input, AllowQuirks::Yes);
            // Keyword values don't make sense in the block direction; don't parse them
            ret.map(SpecifiedValue)
        }

        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;
            #[inline]
            fn to_computed_value(&self, context: &Context) -> computed_value::T {
                    use values::computed::MozLength;
                let computed = self.0.to_computed_value(context);

                // filter out keyword values in the block direction
                    if let MozLength::ExtremumLength(..) = computed {
                        
                        if false != context.builder.writing_mode.is_vertical() {
                            return get_initial_value()
                        }
                    }
                computed
            }

            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                SpecifiedValue(ToComputedValue::from_computed_value(computed))
            }
        }
    
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::FlexBasis(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::FlexBasis);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::FlexBasis);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_flex_basis(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_flex_basis();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_flex_basis();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::FlexBasis)
            }
    }


    
        // width, height, block-size, inline-size
        
    
    
    /// https://drafts.csswg.org/css-box/#propdef-width
    pub mod width {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            use values::specified::AllowQuirks;
        use values::specified::MozLength;

        pub mod computed_value {
            pub type T = ::values::computed::MozLength;
        }

        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        #[derive(Clone, Debug, PartialEq, ToCss)]
        pub struct SpecifiedValue(pub MozLength);

        impl SpecifiedValue {
            /// Returns the `auto` value.
            pub fn auto() -> Self {
                use values::specified::length::LengthOrPercentageOrAuto;
                SpecifiedValue(MozLength::LengthOrPercentageOrAuto(LengthOrPercentageOrAuto::Auto))
            }

            /// Returns a value representing a `0` length.
            pub fn zero() -> Self {
                use values::specified::length::LengthOrPercentageOrAuto;
                SpecifiedValue(MozLength::LengthOrPercentageOrAuto(LengthOrPercentageOrAuto::zero()))
            }

            /// Returns a value representing a `0%` length.
            pub fn zero_percent() -> Self {
                use values::specified::length::LengthOrPercentageOrAuto;
                SpecifiedValue(MozLength::LengthOrPercentageOrAuto(LengthOrPercentageOrAuto::zero_percent()))
            }
        }

        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            use values::computed::MozLength;
            MozLength::auto()
        }
        fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                         -> Result<SpecifiedValue, ParseError<'i>> {
            let ret = MozLength::parse_quirky(context, input, AllowQuirks::Yes);
            // Keyword values don't make sense in the block direction; don't parse them
            ret.map(SpecifiedValue)
        }

        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;
            #[inline]
            fn to_computed_value(&self, context: &Context) -> computed_value::T {
                    use values::computed::MozLength;
                let computed = self.0.to_computed_value(context);

                // filter out keyword values in the block direction
                    if let MozLength::ExtremumLength(..) = computed {
                        
                        if false != context.builder.writing_mode.is_vertical() {
                            return get_initial_value()
                        }
                    }
                computed
            }

            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                SpecifiedValue(ToComputedValue::from_computed_value(computed))
            }
        }
    
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::Width(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::Width);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::Width);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_width(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_width();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_width();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::Width)
            }
    }


        // min-width, min-height, min-block-size, min-inline-size,
        // max-width, max-height, max-block-size, max-inline-size
        
    
    
    /// https://drafts.csswg.org/css-box/#propdef-width
    pub mod min_width {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            use values::specified::AllowQuirks;
        use values::specified::MozLength;

        pub mod computed_value {
            pub type T = ::values::computed::MozLength;
        }

        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        #[derive(Clone, Debug, PartialEq, ToCss)]
        pub struct SpecifiedValue(pub MozLength);

        impl SpecifiedValue {
            /// Returns the `auto` value.
            pub fn auto() -> Self {
                use values::specified::length::LengthOrPercentageOrAuto;
                SpecifiedValue(MozLength::LengthOrPercentageOrAuto(LengthOrPercentageOrAuto::Auto))
            }

            /// Returns a value representing a `0` length.
            pub fn zero() -> Self {
                use values::specified::length::LengthOrPercentageOrAuto;
                SpecifiedValue(MozLength::LengthOrPercentageOrAuto(LengthOrPercentageOrAuto::zero()))
            }

            /// Returns a value representing a `0%` length.
            pub fn zero_percent() -> Self {
                use values::specified::length::LengthOrPercentageOrAuto;
                SpecifiedValue(MozLength::LengthOrPercentageOrAuto(LengthOrPercentageOrAuto::zero_percent()))
            }
        }

        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            use values::computed::MozLength;
            MozLength::auto()
        }
        fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                         -> Result<SpecifiedValue, ParseError<'i>> {
            let ret = MozLength::parse_quirky(context, input, AllowQuirks::Yes);
            // Keyword values don't make sense in the block direction; don't parse them
            ret.map(SpecifiedValue)
        }

        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;
            #[inline]
            fn to_computed_value(&self, context: &Context) -> computed_value::T {
                    use values::computed::MozLength;
                let computed = self.0.to_computed_value(context);

                // filter out keyword values in the block direction
                    if let MozLength::ExtremumLength(..) = computed {
                        
                        if false != context.builder.writing_mode.is_vertical() {
                            return get_initial_value()
                        }
                    }
                computed
            }

            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                SpecifiedValue(ToComputedValue::from_computed_value(computed))
            }
        }
    
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MinWidth(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MinWidth);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::MinWidth);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_min_width(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_min_width();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_min_width();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MinWidth)
            }
    }


        
    
    
    /// https://drafts.csswg.org/css-box/#propdef-width
    pub mod max_width {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            use values::specified::AllowQuirks;
        use values::specified::MaxLength;

        pub mod computed_value {
            pub type T = ::values::computed::MaxLength;
        }

        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        #[derive(Clone, Debug, PartialEq, ToCss)]
        pub struct SpecifiedValue(pub MaxLength);


        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            use values::computed::MaxLength;
            MaxLength::none()
        }
        fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                         -> Result<SpecifiedValue, ParseError<'i>> {
            let ret = MaxLength::parse_quirky(context, input, AllowQuirks::Yes);
            // Keyword values don't make sense in the block direction; don't parse them
            ret.map(SpecifiedValue)
        }

        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;
            #[inline]
            fn to_computed_value(&self, context: &Context) -> computed_value::T {
                    use values::computed::MaxLength;
                let computed = self.0.to_computed_value(context);

                // filter out keyword values in the block direction
                    if let MaxLength::ExtremumLength(..) = computed {
                        
                        if false != context.builder.writing_mode.is_vertical() {
                            return get_initial_value()
                        }
                    }
                computed
            }

            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                SpecifiedValue(ToComputedValue::from_computed_value(computed))
            }
        }
    
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MaxWidth(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MaxWidth);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::MaxWidth);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_max_width(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_max_width();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_max_width();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MaxWidth)
            }
    }


    
        // width, height, block-size, inline-size
        
    
    
    /// https://drafts.csswg.org/css-box/#propdef-height
    pub mod height {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            use values::specified::AllowQuirks;
        use values::specified::MozLength;

        pub mod computed_value {
            pub type T = ::values::computed::MozLength;
        }

        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        #[derive(Clone, Debug, PartialEq, ToCss)]
        pub struct SpecifiedValue(pub MozLength);

        impl SpecifiedValue {
            /// Returns the `auto` value.
            pub fn auto() -> Self {
                use values::specified::length::LengthOrPercentageOrAuto;
                SpecifiedValue(MozLength::LengthOrPercentageOrAuto(LengthOrPercentageOrAuto::Auto))
            }

            /// Returns a value representing a `0` length.
            pub fn zero() -> Self {
                use values::specified::length::LengthOrPercentageOrAuto;
                SpecifiedValue(MozLength::LengthOrPercentageOrAuto(LengthOrPercentageOrAuto::zero()))
            }

            /// Returns a value representing a `0%` length.
            pub fn zero_percent() -> Self {
                use values::specified::length::LengthOrPercentageOrAuto;
                SpecifiedValue(MozLength::LengthOrPercentageOrAuto(LengthOrPercentageOrAuto::zero_percent()))
            }
        }

        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            use values::computed::MozLength;
            MozLength::auto()
        }
        fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                         -> Result<SpecifiedValue, ParseError<'i>> {
            let ret = MozLength::parse_quirky(context, input, AllowQuirks::Yes);
            // Keyword values don't make sense in the block direction; don't parse them
            ret.map(SpecifiedValue)
        }

        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;
            #[inline]
            fn to_computed_value(&self, context: &Context) -> computed_value::T {
                    use values::computed::MozLength;
                let computed = self.0.to_computed_value(context);

                // filter out keyword values in the block direction
                    if let MozLength::ExtremumLength(..) = computed {
                        
                        if true != context.builder.writing_mode.is_vertical() {
                            return get_initial_value()
                        }
                    }
                computed
            }

            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                SpecifiedValue(ToComputedValue::from_computed_value(computed))
            }
        }
    
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::Height(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::Height);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::Height);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_height(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_height();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_height();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::Height)
            }
    }


        // min-width, min-height, min-block-size, min-inline-size,
        // max-width, max-height, max-block-size, max-inline-size
        
    
    
    /// https://drafts.csswg.org/css-box/#propdef-height
    pub mod min_height {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            use values::specified::AllowQuirks;
        use values::specified::MozLength;

        pub mod computed_value {
            pub type T = ::values::computed::MozLength;
        }

        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        #[derive(Clone, Debug, PartialEq, ToCss)]
        pub struct SpecifiedValue(pub MozLength);

        impl SpecifiedValue {
            /// Returns the `auto` value.
            pub fn auto() -> Self {
                use values::specified::length::LengthOrPercentageOrAuto;
                SpecifiedValue(MozLength::LengthOrPercentageOrAuto(LengthOrPercentageOrAuto::Auto))
            }

            /// Returns a value representing a `0` length.
            pub fn zero() -> Self {
                use values::specified::length::LengthOrPercentageOrAuto;
                SpecifiedValue(MozLength::LengthOrPercentageOrAuto(LengthOrPercentageOrAuto::zero()))
            }

            /// Returns a value representing a `0%` length.
            pub fn zero_percent() -> Self {
                use values::specified::length::LengthOrPercentageOrAuto;
                SpecifiedValue(MozLength::LengthOrPercentageOrAuto(LengthOrPercentageOrAuto::zero_percent()))
            }
        }

        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            use values::computed::MozLength;
            MozLength::auto()
        }
        fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                         -> Result<SpecifiedValue, ParseError<'i>> {
            let ret = MozLength::parse_quirky(context, input, AllowQuirks::Yes);
            // Keyword values don't make sense in the block direction; don't parse them
            ret.map(SpecifiedValue)
        }

        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;
            #[inline]
            fn to_computed_value(&self, context: &Context) -> computed_value::T {
                    use values::computed::MozLength;
                let computed = self.0.to_computed_value(context);

                // filter out keyword values in the block direction
                    if let MozLength::ExtremumLength(..) = computed {
                        
                        if true != context.builder.writing_mode.is_vertical() {
                            return get_initial_value()
                        }
                    }
                computed
            }

            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                SpecifiedValue(ToComputedValue::from_computed_value(computed))
            }
        }
    
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MinHeight(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MinHeight);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::MinHeight);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_min_height(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_min_height();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_min_height();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MinHeight)
            }
    }


        
    
    
    /// https://drafts.csswg.org/css-box/#propdef-height
    pub mod max_height {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            use values::specified::AllowQuirks;
        use values::specified::MaxLength;

        pub mod computed_value {
            pub type T = ::values::computed::MaxLength;
        }

        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        #[derive(Clone, Debug, PartialEq, ToCss)]
        pub struct SpecifiedValue(pub MaxLength);


        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            use values::computed::MaxLength;
            MaxLength::none()
        }
        fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                         -> Result<SpecifiedValue, ParseError<'i>> {
            let ret = MaxLength::parse_quirky(context, input, AllowQuirks::Yes);
            // Keyword values don't make sense in the block direction; don't parse them
            ret.map(SpecifiedValue)
        }

        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;
            #[inline]
            fn to_computed_value(&self, context: &Context) -> computed_value::T {
                    use values::computed::MaxLength;
                let computed = self.0.to_computed_value(context);

                // filter out keyword values in the block direction
                    if let MaxLength::ExtremumLength(..) = computed {
                        
                        if true != context.builder.writing_mode.is_vertical() {
                            return get_initial_value()
                        }
                    }
                computed
            }

            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                SpecifiedValue(ToComputedValue::from_computed_value(computed))
            }
        }
    
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MaxHeight(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MaxHeight);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::MaxHeight);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_max_height(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_max_height();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_max_height();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MaxHeight)
            }
    }


    
        // width, height, block-size, inline-size
        
    
    
    /// https://drafts.csswg.org/css-logical-props/#propdef-block-size
    pub mod block_size {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
        use values::specified::MozLength;

        pub mod computed_value {
            pub type T = ::values::computed::MozLength;
        }

        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        #[derive(Clone, Debug, PartialEq, ToCss)]
        pub struct SpecifiedValue(pub MozLength);

        impl SpecifiedValue {
            /// Returns the `auto` value.
            pub fn auto() -> Self {
                use values::specified::length::LengthOrPercentageOrAuto;
                SpecifiedValue(MozLength::LengthOrPercentageOrAuto(LengthOrPercentageOrAuto::Auto))
            }

            /// Returns a value representing a `0` length.
            pub fn zero() -> Self {
                use values::specified::length::LengthOrPercentageOrAuto;
                SpecifiedValue(MozLength::LengthOrPercentageOrAuto(LengthOrPercentageOrAuto::zero()))
            }

            /// Returns a value representing a `0%` length.
            pub fn zero_percent() -> Self {
                use values::specified::length::LengthOrPercentageOrAuto;
                SpecifiedValue(MozLength::LengthOrPercentageOrAuto(LengthOrPercentageOrAuto::zero_percent()))
            }
        }

        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            use values::computed::MozLength;
            MozLength::auto()
        }
        fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                         -> Result<SpecifiedValue, ParseError<'i>> {
            let ret = MozLength::parse(context, input);
            // Keyword values don't make sense in the block direction; don't parse them
                if let Ok(MozLength::ExtremumLength(..)) = ret {
                    return Err(StyleParseError::UnspecifiedError.into())
                }
            ret.map(SpecifiedValue)
        }

        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;
            #[inline]
            fn to_computed_value(&self, context: &Context) -> computed_value::T {
                    use values::computed::MozLength;
                let computed = self.0.to_computed_value(context);

                // filter out keyword values in the block direction
                        if let MozLength::ExtremumLength(..) = computed {
                            return get_initial_value()
                        }
                computed
            }

            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                SpecifiedValue(ToComputedValue::from_computed_value(computed))
            }
        }
    
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::BlockSize(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::BlockSize);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::BlockSize);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            context.rule_cache_conditions.borrow_mut()
                                .set_writing_mode_dependency(context.builder.writing_mode);
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_block_size(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_block_size();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_block_size();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::BlockSize)
            }
    }


        // min-width, min-height, min-block-size, min-inline-size,
        // max-width, max-height, max-block-size, max-inline-size
        
    
    
    /// https://drafts.csswg.org/css-logical-props/#propdef-block-size
    pub mod min_block_size {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
        use values::specified::MozLength;

        pub mod computed_value {
            pub type T = ::values::computed::MozLength;
        }

        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        #[derive(Clone, Debug, PartialEq, ToCss)]
        pub struct SpecifiedValue(pub MozLength);

        impl SpecifiedValue {
            /// Returns the `auto` value.
            pub fn auto() -> Self {
                use values::specified::length::LengthOrPercentageOrAuto;
                SpecifiedValue(MozLength::LengthOrPercentageOrAuto(LengthOrPercentageOrAuto::Auto))
            }

            /// Returns a value representing a `0` length.
            pub fn zero() -> Self {
                use values::specified::length::LengthOrPercentageOrAuto;
                SpecifiedValue(MozLength::LengthOrPercentageOrAuto(LengthOrPercentageOrAuto::zero()))
            }

            /// Returns a value representing a `0%` length.
            pub fn zero_percent() -> Self {
                use values::specified::length::LengthOrPercentageOrAuto;
                SpecifiedValue(MozLength::LengthOrPercentageOrAuto(LengthOrPercentageOrAuto::zero_percent()))
            }
        }

        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            use values::computed::MozLength;
            MozLength::auto()
        }
        fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                         -> Result<SpecifiedValue, ParseError<'i>> {
            let ret = MozLength::parse(context, input);
            // Keyword values don't make sense in the block direction; don't parse them
                if let Ok(MozLength::ExtremumLength(..)) = ret {
                    return Err(StyleParseError::UnspecifiedError.into())
                }
            ret.map(SpecifiedValue)
        }

        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;
            #[inline]
            fn to_computed_value(&self, context: &Context) -> computed_value::T {
                    use values::computed::MozLength;
                let computed = self.0.to_computed_value(context);

                // filter out keyword values in the block direction
                        if let MozLength::ExtremumLength(..) = computed {
                            return get_initial_value()
                        }
                computed
            }

            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                SpecifiedValue(ToComputedValue::from_computed_value(computed))
            }
        }
    
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MinBlockSize(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MinBlockSize);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::MinBlockSize);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            context.rule_cache_conditions.borrow_mut()
                                .set_writing_mode_dependency(context.builder.writing_mode);
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_min_block_size(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_min_block_size();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_min_block_size();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MinBlockSize)
            }
    }


        
    
    
    /// https://drafts.csswg.org/css-logical-props/#propdef-block-size
    pub mod max_block_size {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
        use values::specified::MaxLength;

        pub mod computed_value {
            pub type T = ::values::computed::MaxLength;
        }

        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        #[derive(Clone, Debug, PartialEq, ToCss)]
        pub struct SpecifiedValue(pub MaxLength);


        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            use values::computed::MaxLength;
            MaxLength::none()
        }
        fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                         -> Result<SpecifiedValue, ParseError<'i>> {
            let ret = MaxLength::parse(context, input);
            // Keyword values don't make sense in the block direction; don't parse them
                if let Ok(MaxLength::ExtremumLength(..)) = ret {
                    return Err(StyleParseError::UnspecifiedError.into())
                }
            ret.map(SpecifiedValue)
        }

        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;
            #[inline]
            fn to_computed_value(&self, context: &Context) -> computed_value::T {
                    use values::computed::MaxLength;
                let computed = self.0.to_computed_value(context);

                // filter out keyword values in the block direction
                        if let MaxLength::ExtremumLength(..) = computed {
                            return get_initial_value()
                        }
                computed
            }

            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                SpecifiedValue(ToComputedValue::from_computed_value(computed))
            }
        }
    
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MaxBlockSize(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MaxBlockSize);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::MaxBlockSize);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            context.rule_cache_conditions.borrow_mut()
                                .set_writing_mode_dependency(context.builder.writing_mode);
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_max_block_size(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_max_block_size();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_max_block_size();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MaxBlockSize)
            }
    }


    
        // width, height, block-size, inline-size
        
    
    
    /// https://drafts.csswg.org/css-logical-props/#propdef-inline-size
    pub mod inline_size {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
        use values::specified::MozLength;

        pub mod computed_value {
            pub type T = ::values::computed::MozLength;
        }

        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        #[derive(Clone, Debug, PartialEq, ToCss)]
        pub struct SpecifiedValue(pub MozLength);

        impl SpecifiedValue {
            /// Returns the `auto` value.
            pub fn auto() -> Self {
                use values::specified::length::LengthOrPercentageOrAuto;
                SpecifiedValue(MozLength::LengthOrPercentageOrAuto(LengthOrPercentageOrAuto::Auto))
            }

            /// Returns a value representing a `0` length.
            pub fn zero() -> Self {
                use values::specified::length::LengthOrPercentageOrAuto;
                SpecifiedValue(MozLength::LengthOrPercentageOrAuto(LengthOrPercentageOrAuto::zero()))
            }

            /// Returns a value representing a `0%` length.
            pub fn zero_percent() -> Self {
                use values::specified::length::LengthOrPercentageOrAuto;
                SpecifiedValue(MozLength::LengthOrPercentageOrAuto(LengthOrPercentageOrAuto::zero_percent()))
            }
        }

        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            use values::computed::MozLength;
            MozLength::auto()
        }
        fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                         -> Result<SpecifiedValue, ParseError<'i>> {
            let ret = MozLength::parse(context, input);
            // Keyword values don't make sense in the block direction; don't parse them
            ret.map(SpecifiedValue)
        }

        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;
            #[inline]
            fn to_computed_value(&self, context: &Context) -> computed_value::T {
                let computed = self.0.to_computed_value(context);

                // filter out keyword values in the block direction
                computed
            }

            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                SpecifiedValue(ToComputedValue::from_computed_value(computed))
            }
        }
    
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::InlineSize(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::InlineSize);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::InlineSize);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            context.rule_cache_conditions.borrow_mut()
                                .set_writing_mode_dependency(context.builder.writing_mode);
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_inline_size(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_inline_size();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_inline_size();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::InlineSize)
            }
    }


        // min-width, min-height, min-block-size, min-inline-size,
        // max-width, max-height, max-block-size, max-inline-size
        
    
    
    /// https://drafts.csswg.org/css-logical-props/#propdef-inline-size
    pub mod min_inline_size {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
        use values::specified::MozLength;

        pub mod computed_value {
            pub type T = ::values::computed::MozLength;
        }

        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        #[derive(Clone, Debug, PartialEq, ToCss)]
        pub struct SpecifiedValue(pub MozLength);

        impl SpecifiedValue {
            /// Returns the `auto` value.
            pub fn auto() -> Self {
                use values::specified::length::LengthOrPercentageOrAuto;
                SpecifiedValue(MozLength::LengthOrPercentageOrAuto(LengthOrPercentageOrAuto::Auto))
            }

            /// Returns a value representing a `0` length.
            pub fn zero() -> Self {
                use values::specified::length::LengthOrPercentageOrAuto;
                SpecifiedValue(MozLength::LengthOrPercentageOrAuto(LengthOrPercentageOrAuto::zero()))
            }

            /// Returns a value representing a `0%` length.
            pub fn zero_percent() -> Self {
                use values::specified::length::LengthOrPercentageOrAuto;
                SpecifiedValue(MozLength::LengthOrPercentageOrAuto(LengthOrPercentageOrAuto::zero_percent()))
            }
        }

        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            use values::computed::MozLength;
            MozLength::auto()
        }
        fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                         -> Result<SpecifiedValue, ParseError<'i>> {
            let ret = MozLength::parse(context, input);
            // Keyword values don't make sense in the block direction; don't parse them
            ret.map(SpecifiedValue)
        }

        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;
            #[inline]
            fn to_computed_value(&self, context: &Context) -> computed_value::T {
                let computed = self.0.to_computed_value(context);

                // filter out keyword values in the block direction
                computed
            }

            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                SpecifiedValue(ToComputedValue::from_computed_value(computed))
            }
        }
    
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MinInlineSize(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MinInlineSize);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::MinInlineSize);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            context.rule_cache_conditions.borrow_mut()
                                .set_writing_mode_dependency(context.builder.writing_mode);
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_min_inline_size(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_min_inline_size();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_min_inline_size();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MinInlineSize)
            }
    }


        
    
    
    /// https://drafts.csswg.org/css-logical-props/#propdef-inline-size
    pub mod max_inline_size {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
        use values::specified::MaxLength;

        pub mod computed_value {
            pub type T = ::values::computed::MaxLength;
        }

        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        #[derive(Clone, Debug, PartialEq, ToCss)]
        pub struct SpecifiedValue(pub MaxLength);


        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            use values::computed::MaxLength;
            MaxLength::none()
        }
        fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                         -> Result<SpecifiedValue, ParseError<'i>> {
            let ret = MaxLength::parse(context, input);
            // Keyword values don't make sense in the block direction; don't parse them
            ret.map(SpecifiedValue)
        }

        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;
            #[inline]
            fn to_computed_value(&self, context: &Context) -> computed_value::T {
                let computed = self.0.to_computed_value(context);

                // filter out keyword values in the block direction
                computed
            }

            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                SpecifiedValue(ToComputedValue::from_computed_value(computed))
            }
        }
    
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MaxInlineSize(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MaxInlineSize);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::MaxInlineSize);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            context.rule_cache_conditions.borrow_mut()
                                .set_writing_mode_dependency(context.builder.writing_mode);
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_max_inline_size(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_max_inline_size();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_max_inline_size();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MaxInlineSize)
            }
    }




    
    

    
        
    
    /// https://drafts.csswg.org/css-ui/#propdef-box-sizing
    pub mod box_sizing {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "content-box" => content_box,
                    "border-box" => border_box,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::content_box
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::content_box
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::content_box => computed_value::T::content_box,
                        SpecifiedValue::border_box => computed_value::T::border_box,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::content_box => SpecifiedValue::content_box,
                        computed_value::T::border_box => SpecifiedValue::border_box,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::BoxSizing(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::BoxSizing);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::BoxSizing);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_box_sizing(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_box_sizing();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_box_sizing();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::BoxSizing)
            }
    }





    
    

    
        
    
    /// https://drafts.csswg.org/css-images/#propdef-object-fit
    pub mod object_fit {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "fill" => fill,
                    "contain" => contain,
                    "cover" => cover,
                    "none" => none,
                    "scale-down" => scale_down,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::fill
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::fill
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::fill => computed_value::T::fill,
                        SpecifiedValue::contain => computed_value::T::contain,
                        SpecifiedValue::cover => computed_value::T::cover,
                        SpecifiedValue::none => computed_value::T::none,
                        SpecifiedValue::scale_down => computed_value::T::scale_down,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::fill => SpecifiedValue::fill,
                        computed_value::T::contain => SpecifiedValue::contain,
                        computed_value::T::cover => SpecifiedValue::cover,
                        computed_value::T::none => SpecifiedValue::none,
                        computed_value::T::scale_down => SpecifiedValue::scale_down,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::ObjectFit(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::ObjectFit);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::ObjectFit);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_object_fit(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_object_fit();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_object_fit();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::ObjectFit)
            }
    }





    
        
    
    /// https://drafts.csswg.org/css-images-3/#the-object-position
    pub mod object_position {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::Position as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::Position as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::Position::zero() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::Position::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::ObjectPosition(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::ObjectPosition);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::ObjectPosition);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = (**specified_value).to_computed_value(context);
                                context.builder.set_object_position(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_object_position();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_object_position();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<Box<SpecifiedValue>, ParseError<'i>> {
                    parse(context, input).map(|result| Box::new(result))
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::ObjectPosition)
            }
    }



    
    
        
    
    /// https://drafts.csswg.org/css-grid/#propdef-grid-row-gap
    pub mod grid_row_gap {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::NonNegativeLengthOrPercentage as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::NonNegativeLengthOrPercentage as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::NonNegativeLengthOrPercentage::zero() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::NonNegativeLengthOrPercentage::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::GridRowGap(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::GridRowGap);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::GridRowGap);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_grid_row_gap(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_grid_row_gap();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_grid_row_gap();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::GridRowGap)
            }
    }



        
    
        
    
    /// https://drafts.csswg.org/css-grid/#propdef-grid-row-start
    pub mod grid_row_start {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::GridLine as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::GridLine as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { Default::default() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::GridLine::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::GridRowStart(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::GridRowStart);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::GridRowStart);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = (**specified_value).to_computed_value(context);
                                context.builder.set_grid_row_start(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_grid_row_start();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_grid_row_start();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<Box<SpecifiedValue>, ParseError<'i>> {
                    parse(context, input).map(|result| Box::new(result))
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::GridRowStart)
            }
    }


        
    
        
    
    /// https://drafts.csswg.org/css-grid/#propdef-grid-row-end
    pub mod grid_row_end {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::GridLine as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::GridLine as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { Default::default() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::GridLine::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::GridRowEnd(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::GridRowEnd);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::GridRowEnd);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = (**specified_value).to_computed_value(context);
                                context.builder.set_grid_row_end(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_grid_row_end();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_grid_row_end();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<Box<SpecifiedValue>, ParseError<'i>> {
                    parse(context, input).map(|result| Box::new(result))
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::GridRowEnd)
            }
    }



    // NOTE: According to the spec, this should handle multiple values of `<track-size>`,
    // but gecko supports only a single value
    
    
        
    
    /// https://drafts.csswg.org/css-grid/#propdef-grid-auto-rows
    pub mod grid_auto_rows {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::TrackSize as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::TrackSize as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { Default::default() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::TrackSize::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::GridAutoRows(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::GridAutoRows);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::GridAutoRows);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = (**specified_value).to_computed_value(context);
                                context.builder.set_grid_auto_rows(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_grid_auto_rows();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_grid_auto_rows();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<Box<SpecifiedValue>, ParseError<'i>> {
                    parse(context, input).map(|result| Box::new(result))
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::GridAutoRows)
            }
    }



    
    
        
    
    /// https://drafts.csswg.org/css-grid/#propdef-grid-template-rows
    pub mod grid_template_rows {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::GridTemplateComponent as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::GridTemplateComponent as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { specified::GenericGridTemplateComponent::None }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::GridTemplateComponent::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::GridTemplateRows(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::GridTemplateRows);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::GridTemplateRows);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = (**specified_value).to_computed_value(context);
                                context.builder.set_grid_template_rows(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_grid_template_rows();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_grid_template_rows();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<Box<SpecifiedValue>, ParseError<'i>> {
                    parse(context, input).map(|result| Box::new(result))
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::GridTemplateRows)
            }
    }



    
    
        
    
    /// https://drafts.csswg.org/css-grid/#propdef-grid-column-gap
    pub mod grid_column_gap {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::NonNegativeLengthOrPercentage as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::NonNegativeLengthOrPercentage as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::NonNegativeLengthOrPercentage::zero() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::NonNegativeLengthOrPercentage::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::GridColumnGap(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::GridColumnGap);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::GridColumnGap);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_grid_column_gap(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_grid_column_gap();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_grid_column_gap();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::GridColumnGap)
            }
    }



        
    
        
    
    /// https://drafts.csswg.org/css-grid/#propdef-grid-column-start
    pub mod grid_column_start {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::GridLine as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::GridLine as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { Default::default() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::GridLine::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::GridColumnStart(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::GridColumnStart);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::GridColumnStart);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = (**specified_value).to_computed_value(context);
                                context.builder.set_grid_column_start(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_grid_column_start();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_grid_column_start();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<Box<SpecifiedValue>, ParseError<'i>> {
                    parse(context, input).map(|result| Box::new(result))
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::GridColumnStart)
            }
    }


        
    
        
    
    /// https://drafts.csswg.org/css-grid/#propdef-grid-column-end
    pub mod grid_column_end {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::GridLine as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::GridLine as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { Default::default() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::GridLine::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::GridColumnEnd(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::GridColumnEnd);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::GridColumnEnd);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = (**specified_value).to_computed_value(context);
                                context.builder.set_grid_column_end(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_grid_column_end();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_grid_column_end();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<Box<SpecifiedValue>, ParseError<'i>> {
                    parse(context, input).map(|result| Box::new(result))
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::GridColumnEnd)
            }
    }



    // NOTE: According to the spec, this should handle multiple values of `<track-size>`,
    // but gecko supports only a single value
    
    
        
    
    /// https://drafts.csswg.org/css-grid/#propdef-grid-auto-columns
    pub mod grid_auto_columns {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::TrackSize as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::TrackSize as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { Default::default() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::TrackSize::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::GridAutoColumns(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::GridAutoColumns);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::GridAutoColumns);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = (**specified_value).to_computed_value(context);
                                context.builder.set_grid_auto_columns(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_grid_auto_columns();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_grid_auto_columns();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<Box<SpecifiedValue>, ParseError<'i>> {
                    parse(context, input).map(|result| Box::new(result))
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::GridAutoColumns)
            }
    }



    
    
        
    
    /// https://drafts.csswg.org/css-grid/#propdef-grid-template-columns
    pub mod grid_template_columns {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::GridTemplateComponent as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::GridTemplateComponent as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { specified::GenericGridTemplateComponent::None }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::GridTemplateComponent::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::GridTemplateColumns(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::GridTemplateColumns);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::GridTemplateColumns);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = (**specified_value).to_computed_value(context);
                                context.builder.set_grid_template_columns(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_grid_template_columns();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_grid_template_columns();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<Box<SpecifiedValue>, ParseError<'i>> {
                    parse(context, input).map(|result| Box::new(result))
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::GridTemplateColumns)
            }
    }





    
    /// https://drafts.csswg.org/css-grid/#propdef-grid-auto-flow
    pub mod grid_auto_flow {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
    use std::fmt;
    use style_traits::ToCss;

    pub type SpecifiedValue = computed_value::T;

    pub mod computed_value {
        #[derive(Clone, Copy, Debug, Eq, PartialEq, ToComputedValue)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        pub enum AutoFlow {
            Row,
            Column,
        }

        #[derive(Clone, Copy, Debug, Eq, PartialEq, ToComputedValue)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        pub struct T {
            pub autoflow: AutoFlow,
            pub dense: bool,
        }
    }

    impl ToCss for computed_value::T {
        fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
            dest.write_str(match self.autoflow {
                computed_value::AutoFlow::Column => "column",
                computed_value::AutoFlow::Row => "row"
            })?;

            if self.dense { dest.write_str(" dense")?; }
            Ok(())
        }
    }

    #[inline]
    pub fn get_initial_value() -> computed_value::T {
        computed_value::T {
            autoflow: computed_value::AutoFlow::Row,
            dense: false,
        }
    }

    /// [ row | column ] || dense
    pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                         -> Result<SpecifiedValue, ParseError<'i>> {
        use self::computed_value::AutoFlow;

        let mut value = None;
        let mut dense = false;

        while !input.is_exhausted() {
            let ident = input.expect_ident()?;
            let success = match_ignore_ascii_case! { &ident,
                "row" if value.is_none() => {
                    value = Some(AutoFlow::Row);
                    true
                },
                "column" if value.is_none() => {
                    value = Some(AutoFlow::Column);
                    true
                },
                "dense" if !dense => {
                    dense = true;
                    true
                },
                _ => false
            };
            if !success {
                return Err(SelectorParseError::UnexpectedIdent(ident.clone()).into());
            }
        }

        if value.is_some() || dense {
            Ok(computed_value::T {
                autoflow: value.unwrap_or(AutoFlow::Row),
                dense: dense,
            })
        } else {
            Err(StyleParseError::UnspecifiedError.into())
        }
    }

    #[cfg(feature = "gecko")]
    impl From<u8> for SpecifiedValue {
        fn from(bits: u8) -> SpecifiedValue {
            use gecko_bindings::structs;
            use self::computed_value::AutoFlow;

            SpecifiedValue {
                autoflow:
                    if bits & structs::NS_STYLE_GRID_AUTO_FLOW_ROW as u8 != 0 {
                        AutoFlow::Row
                    } else {
                        AutoFlow::Column
                    },
                dense:
                    bits & structs::NS_STYLE_GRID_AUTO_FLOW_DENSE as u8 != 0,
            }
        }
    }

    #[cfg(feature = "gecko")]
    impl From<SpecifiedValue> for u8 {
        fn from(v: SpecifiedValue) -> u8 {
            use gecko_bindings::structs;
            use self::computed_value::AutoFlow;

            let mut result: u8 = match v.autoflow {
                AutoFlow::Row => structs::NS_STYLE_GRID_AUTO_FLOW_ROW as u8,
                AutoFlow::Column => structs::NS_STYLE_GRID_AUTO_FLOW_COLUMN as u8,
            };

            if v.dense {
                result |= structs::NS_STYLE_GRID_AUTO_FLOW_DENSE as u8;
            }
            result
        }
    }

        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::GridAutoFlow(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::GridAutoFlow);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::GridAutoFlow);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_grid_auto_flow(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_grid_auto_flow();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_grid_auto_flow();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::GridAutoFlow)
            }
    }



    
    /// https://drafts.csswg.org/css-grid/#propdef-grid-template-areas
    pub mod grid_template_areas {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
    use hash::FnvHashMap;
    use std::fmt;
    use std::ops::Range;
    use str::HTML_SPACE_CHARACTERS;
    use style_traits::ToCss;

    pub mod computed_value {
        pub use super::SpecifiedValue as T;
    }

    pub type SpecifiedValue = Either<TemplateAreas, None_>;

    #[inline]
    pub fn get_initial_value() -> computed_value::T {
        Either::Second(None_)
    }

    pub fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                         -> Result<SpecifiedValue, ParseError<'i>> {
        SpecifiedValue::parse(context, input)
    }

    #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
    #[derive(Clone, Debug, PartialEq)]
    pub struct TemplateAreas {
        pub areas: Box<[NamedArea]>,
        pub strings: Box<[Box<str>]>,
        pub width: u32,
    }

    #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
    #[derive(Clone, Debug, PartialEq)]
    pub struct NamedArea {
        pub name: Box<str>,
        pub rows: Range<u32>,
        pub columns: Range<u32>,
    }

    trivial_to_computed_value!(TemplateAreas);

    impl Parse for TemplateAreas {
        fn parse<'i, 't>(
            _context: &ParserContext,
            input: &mut Parser<'i, 't>,
        ) -> Result<Self, ParseError<'i>> {
            let mut strings = vec![];
            while let Ok(string) = input.try(|i| i.expect_string().map(|s| s.as_ref().into())) {
                strings.push(string);
            }

            TemplateAreas::from_vec(strings)
                .map_err(|()| StyleParseError::UnspecifiedError.into())
        }
    }

    impl TemplateAreas {
        pub fn from_vec(strings: Vec<Box<str>>) -> Result<TemplateAreas, ()> {
            if strings.is_empty() {
                return Err(());
            }
            let mut areas: Vec<NamedArea> = vec![];
            let mut width = 0;
            {
                let mut row = 0u32;
                let mut area_indices = FnvHashMap::<(&str), usize>::default();
                for string in &strings {
                    let mut current_area_index: Option<usize> = None;
                    row += 1;
                    let mut column = 0u32;
                    for token in Tokenizer(string) {
                        column += 1;
                        let token = if let Some(token) = token? {
                            token
                        } else {
                            if let Some(index) = current_area_index.take() {
                                if areas[index].columns.end != column {
                                    return Err(());
                                }
                            }
                            continue;
                        };
                        if let Some(index) = current_area_index {
                            if &*areas[index].name == token {
                                if areas[index].rows.start == row {
                                    areas[index].columns.end += 1;
                                }
                                continue;
                            }
                            if areas[index].columns.end != column {
                                return Err(());
                            }
                        }
                        if let Some(index) = area_indices.get(token).cloned() {
                            if areas[index].columns.start != column || areas[index].rows.end != row {
                                return Err(());
                            }
                            areas[index].rows.end += 1;
                            current_area_index = Some(index);
                            continue;
                        }
                        let index = areas.len();
                        areas.push(NamedArea {
                            name: token.to_owned().into_boxed_str(),
                            columns: column..(column + 1),
                            rows: row..(row + 1),
                        });
                        assert!(area_indices.insert(token, index).is_none());
                        current_area_index = Some(index);
                    }
                    if let Some(index) = current_area_index {
                        if areas[index].columns.end != column + 1 {
                            assert!(areas[index].rows.start != row);
                            return Err(());
                        }
                    }
                    if row == 1 {
                        width = column;
                    } else if width != column {
                        return Err(());
                    }
                }
            }
            Ok(TemplateAreas {
                areas: areas.into_boxed_slice(),
                strings: strings.into_boxed_slice(),
                width: width,
            })
        }
    }

    impl ToCss for TemplateAreas {
        fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
            for (i, string) in self.strings.iter().enumerate() {
                if i != 0 {
                    dest.write_str(" ")?;
                }
                string.to_css(dest)?;
            }
            Ok(())
        }
    }

    struct Tokenizer<'a>(&'a str);

    impl<'a> Iterator for Tokenizer<'a> {
        type Item = Result<Option<(&'a str)>, ()>;

        fn next(&mut self) -> Option<Self::Item> {
            let rest = self.0.trim_left_matches(HTML_SPACE_CHARACTERS);
            if rest.is_empty() {
                return None;
            }
            if rest.starts_with('.') {
                self.0 = &rest[rest.find(|c| c != '.').unwrap_or(rest.len())..];
                return Some(Ok(None));
            }
            if !rest.starts_with(is_name_code_point) {
                return Some(Err(()));
            }
            let token_len = rest.find(|c| !is_name_code_point(c)).unwrap_or(rest.len());
            let token = &rest[..token_len];
            self.0 = &rest[token_len..];
            Some(Ok(Some(token)))
        }
    }

    fn is_name_code_point(c: char) -> bool {
        c >= 'A' && c <= 'Z' || c >= 'a' && c <= 'z' ||
        c >= '\u{80}' || c == '_' ||
        c >= '0' && c <= '9' || c == '-'
    }

        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::GridTemplateAreas(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::GridTemplateAreas);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::GridTemplateAreas);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = (**specified_value).to_computed_value(context);
                                context.builder.set_grid_template_areas(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_grid_template_areas();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_grid_template_areas();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<Box<SpecifiedValue>, ParseError<'i>> {
                    parse(context, input).map(|result| Box::new(result))
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::GridTemplateAreas)
            }
    }


    /* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */






    
    

    
        
    
    /// https://drafts.csswg.org/css-tables/#propdef-table-layout
    pub mod table_layout {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "auto" => auto,
                    "fixed" => fixed,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::auto
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::auto
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::auto => computed_value::T::auto,
                        SpecifiedValue::fixed => computed_value::T::fixed,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::auto => SpecifiedValue::auto,
                        computed_value::T::fixed => SpecifiedValue::fixed,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::TableLayout(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::TableLayout);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::TableLayout);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_table_layout(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_table_layout();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_table_layout();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::TableLayout)
            }
    }





    
    /// Internal-only (for `<col span>` pres attr)
    pub mod _x_span {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
    pub type SpecifiedValue = computed_value::T;
    pub mod computed_value {
        use std::fmt;
        use style_traits::ToCss;

        #[derive(Clone, Copy, Debug, PartialEq, ToComputedValue)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        pub struct T(pub i32);

        impl ToCss for T {
            fn to_css<W>(&self, _: &mut W) -> fmt::Result where W: fmt::Write {
                Ok(())
            }
        }
    }

    #[inline]
    pub fn get_initial_value() -> computed_value::T {
        computed_value::T(1)
    }

    // never parse it, only set via presentation attribute
    fn parse<'i, 't>(
        _: &ParserContext,
        _: &mut Parser<'i, 't>,
    ) -> Result<SpecifiedValue, ParseError<'i>> {
        Err(StyleParseError::UnspecifiedError.into())
    }

        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::XSpan(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::XSpan);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::XSpan);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set__x_span(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset__x_span();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit__x_span();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::XSpan)
            }
    }


    /* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */







    
    /// https://drafts.csswg.org/css-ui/#propdef-text-overflow
    pub mod text_overflow {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
    use std::fmt;
    use style_traits::ToCss;


    #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
    #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
    #[derive(Clone, Debug, Eq, PartialEq, ToCss)]
    pub enum Side {
        Clip,
        Ellipsis,
        String(Box<str>),
    }

    #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
    #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
    #[derive(Clone, Debug, Eq, PartialEq, ToCss)]
    pub struct SpecifiedValue {
        pub first: Side,
        pub second: Option<Side>
    }

    pub mod computed_value {
        pub use super::Side;

        #[derive(Clone, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        pub struct T {
            // When the specified value only has one side, that's the "second"
            // side, and the sides are logical, so "second" means "end".  The
            // start side is Clip in that case.
            //
            // When the specified value has two sides, those are our "first"
            // and "second" sides, and they are physical sides ("left" and
            // "right").
            pub first: Side,
            pub second: Side,
            pub sides_are_logical: bool
        }
    }

    impl ToCss for computed_value::T {
        fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
            if self.sides_are_logical {
                assert!(self.first == Side::Clip);
                self.second.to_css(dest)?;
            } else {
                self.first.to_css(dest)?;
                dest.write_str(" ")?;
                self.second.to_css(dest)?;
            }
            Ok(())
        }
    }

    impl ToComputedValue for SpecifiedValue {
        type ComputedValue = computed_value::T;

        #[inline]
        fn to_computed_value(&self, _context: &Context) -> Self::ComputedValue {
            if let Some(ref second) = self.second {
                Self::ComputedValue { first: self.first.clone(),
                                      second: second.clone(),
                                      sides_are_logical: false }
            } else {
                Self::ComputedValue { first: Side::Clip,
                                      second: self.first.clone(),
                                      sides_are_logical: true }
            }
        }

        #[inline]
        fn from_computed_value(computed: &Self::ComputedValue) -> Self {
            if computed.sides_are_logical {
                assert!(computed.first == Side::Clip);
                SpecifiedValue { first: computed.second.clone(),
                                 second: None }
            } else {
                SpecifiedValue { first: computed.first.clone(),
                                 second: Some(computed.second.clone()) }
            }
        }
    }

    #[inline]
    pub fn get_initial_value() -> computed_value::T {
        computed_value::T {
            first: Side::Clip,
            second: Side::Clip,
            sides_are_logical: true,
        }
    }
    pub fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                         -> Result<SpecifiedValue, ParseError<'i>> {
        let first = Side::parse(context, input)?;
        let second = input.try(|input| Side::parse(context, input)).ok();
        Ok(SpecifiedValue {
            first: first,
            second: second,
        })
    }
    impl Parse for Side {
        fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                         -> Result<Side, ParseError<'i>> {
            match *input.next()? {
                Token::Ident(ref ident) => {
                    try_match_ident_ignore_ascii_case! { ident,
                        "clip" => Ok(Side::Clip),
                        "ellipsis" => Ok(Side::Ellipsis),
                    }
                }
                Token::QuotedString(ref v) => {
                    Ok(Side::String(v.as_ref().to_owned().into_boxed_str()))
                }
                ref t => Err(BasicParseError::UnexpectedToken(t.clone()).into()),
            }
        }
    }

        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::TextOverflow(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::TextOverflow);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::TextOverflow);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = (**specified_value).to_computed_value(context);
                                context.builder.set_text_overflow(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_text_overflow();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_text_overflow();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<Box<SpecifiedValue>, ParseError<'i>> {
                    parse(context, input).map(|result| Box::new(result))
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::TextOverflow)
            }
    }



    
    

    
        
    
    /// https://drafts.csswg.org/css-writing-modes/#propdef-unicode-bidi
    pub mod unicode_bidi {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "normal" => normal,
                    "embed" => embed,
                    "isolate" => isolate,
                    "bidi-override" => bidi_override,
                    "isolate-override" => isolate_override,
                    "plaintext" => plaintext,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::normal
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::normal
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::normal => computed_value::T::normal,
                        SpecifiedValue::embed => computed_value::T::embed,
                        SpecifiedValue::isolate => computed_value::T::isolate,
                        SpecifiedValue::bidi_override => computed_value::T::bidi_override,
                        SpecifiedValue::isolate_override => computed_value::T::isolate_override,
                        SpecifiedValue::plaintext => computed_value::T::plaintext,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::normal => SpecifiedValue::normal,
                        computed_value::T::embed => SpecifiedValue::embed,
                        computed_value::T::isolate => SpecifiedValue::isolate,
                        computed_value::T::bidi_override => SpecifiedValue::bidi_override,
                        computed_value::T::isolate_override => SpecifiedValue::isolate_override,
                        computed_value::T::plaintext => SpecifiedValue::plaintext,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::UnicodeBidi(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::UnicodeBidi);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::UnicodeBidi);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_unicode_bidi(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_unicode_bidi();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_unicode_bidi();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::UnicodeBidi)
            }
    }





    
    /// https://drafts.csswg.org/css-text-decor/#propdef-text-decoration-line
    pub mod text_decoration_line {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
    use std::fmt;
    use style_traits::ToCss;

    bitflags! {
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        #[derive(ToComputedValue)]
        pub flags SpecifiedValue: u8 {
            const NONE = 0,
            const UNDERLINE = 0x01,
            const OVERLINE = 0x02,
            const LINE_THROUGH = 0x04,
            const BLINK = 0x08,
            /// Only set by presentation attributes
            ///
            /// Setting this will mean that text-decorations use the color
            /// specified by `color` in quirks mode.
            ///
            /// For example, this gives <a href=foo><font color="red">text</font></a>
            /// a red text decoration
            const COLOR_OVERRIDE = 0x10,
        }
    }

    impl ToCss for SpecifiedValue {
        fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
            let mut has_any = false;

            macro_rules! write_value {
                ($line:ident => $css:expr) => {
                    if self.contains($line) {
                        if has_any {
                            dest.write_str(" ")?;
                        }
                        dest.write_str($css)?;
                        has_any = true;
                    }
                }
            }
            write_value!(UNDERLINE => "underline");
            write_value!(OVERLINE => "overline");
            write_value!(LINE_THROUGH => "line-through");
            write_value!(BLINK => "blink");
            if !has_any {
                dest.write_str("none")?;
            }

            Ok(())
        }
    }
    pub mod computed_value {
        pub type T = super::SpecifiedValue;
        #[allow(non_upper_case_globals)]
        pub const none: T = super::SpecifiedValue {
            bits: 0
        };
    }
    #[inline] pub fn get_initial_value() -> computed_value::T {
        computed_value::none
    }
    #[inline]
    pub fn get_initial_specified_value() -> SpecifiedValue {
        SpecifiedValue::empty()
    }
    /// none | [ underline || overline || line-through || blink ]
    pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                         -> Result<SpecifiedValue, ParseError<'i>> {
        let mut result = SpecifiedValue::empty();
        if input.try(|input| input.expect_ident_matching("none")).is_ok() {
            return Ok(result)
        }
        let mut empty = true;

        loop {
            let result: Result<_, ParseError> = input.try(|input| {
                match input.expect_ident() {
                    Ok(ident) => {
                        (match_ignore_ascii_case! { &ident,
                            "underline" => if result.contains(UNDERLINE) { Err(()) }
                                           else { empty = false; result.insert(UNDERLINE); Ok(()) },
                            "overline" => if result.contains(OVERLINE) { Err(()) }
                                          else { empty = false; result.insert(OVERLINE); Ok(()) },
                            "line-through" => if result.contains(LINE_THROUGH) { Err(()) }
                                              else { empty = false; result.insert(LINE_THROUGH); Ok(()) },
                            "blink" => if result.contains(BLINK) { Err(()) }
                                       else { empty = false; result.insert(BLINK); Ok(()) },
                            _ => Err(())
                        }).map_err(|()| SelectorParseError::UnexpectedIdent(ident.clone()).into())
                    }
                    Err(e) => return Err(e.into())
                }
            });
            if result.is_err() {
                break;
            }
        }

        if !empty { Ok(result) } else { Err(StyleParseError::UnspecifiedError.into()) }
    }


    #[cfg(feature = "gecko")]
    impl_bitflags_conversions!(SpecifiedValue);

        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::TextDecorationLine(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::TextDecorationLine);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::TextDecorationLine);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_text_decoration_line(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_text_decoration_line();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_text_decoration_line();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::TextDecorationLine)
            }
    }



    
    

    
        
    
    /// https://drafts.csswg.org/css-text-decor/#propdef-text-decoration-style
    pub mod text_decoration_style {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "solid" => solid,
                    "double" => double,
                    "dotted" => dotted,
                    "dashed" => dashed,
                    "wavy" => wavy,
                    "-moz-none" => _moz_none,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::solid
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::solid
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::solid => computed_value::T::solid,
                        SpecifiedValue::double => computed_value::T::double,
                        SpecifiedValue::dotted => computed_value::T::dotted,
                        SpecifiedValue::dashed => computed_value::T::dashed,
                        SpecifiedValue::wavy => computed_value::T::wavy,
                        SpecifiedValue::_moz_none => computed_value::T::_moz_none,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::solid => SpecifiedValue::solid,
                        computed_value::T::double => SpecifiedValue::double,
                        computed_value::T::dotted => SpecifiedValue::dotted,
                        computed_value::T::dashed => SpecifiedValue::dashed,
                        computed_value::T::wavy => SpecifiedValue::wavy,
                        computed_value::T::_moz_none => SpecifiedValue::_moz_none,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::TextDecorationStyle(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::TextDecorationStyle);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::TextDecorationStyle);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_text_decoration_style(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_text_decoration_style();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_text_decoration_style();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::TextDecorationStyle)
            }
    }





    
        
    
    /// https://drafts.csswg.org/css-text-decor/#propdef-text-decoration-color
    pub mod text_decoration_color {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::Color as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::Color as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed_value::T::currentcolor() }
        #[inline] pub fn get_initial_specified_value() -> SpecifiedValue { specified::Color::currentcolor() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::Color::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::TextDecorationColor(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::TextDecorationColor);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::TextDecorationColor);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_text_decoration_color(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_text_decoration_color();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_text_decoration_color();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::TextDecorationColor)
            }
    }




    
        
    
    /// https://drafts.csswg.org/css-inline/#sizing-drop-initials
    pub mod initial_letter {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::InitialLetter as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::InitialLetter as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::InitialLetter::normal() }
        #[inline] pub fn get_initial_specified_value() -> SpecifiedValue { specified::InitialLetter::normal() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::InitialLetter::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::InitialLetter(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::InitialLetter);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::InitialLetter);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_initial_letter(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_initial_letter();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_initial_letter();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::InitialLetter)
            }
    }



    /* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */




// CSS Basic User Interface Module Level 1
// https://drafts.csswg.org/css-ui-3/


// TODO spec says that UAs should not support this
// we should probably remove from gecko (https://bugzilla.mozilla.org/show_bug.cgi?id=1328331)

    
    

    
        
    
    /// https://drafts.csswg.org/css-ui/#input-method-editor
    pub mod ime_mode {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "auto" => auto,
                    "normal" => normal,
                    "active" => active,
                    "disabled" => disabled,
                    "inactive" => inactive,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::auto
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::auto
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::auto => computed_value::T::auto,
                        SpecifiedValue::normal => computed_value::T::normal,
                        SpecifiedValue::active => computed_value::T::active,
                        SpecifiedValue::disabled => computed_value::T::disabled,
                        SpecifiedValue::inactive => computed_value::T::inactive,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::auto => SpecifiedValue::auto,
                        computed_value::T::normal => SpecifiedValue::normal,
                        computed_value::T::active => SpecifiedValue::active,
                        computed_value::T::disabled => SpecifiedValue::disabled,
                        computed_value::T::inactive => SpecifiedValue::inactive,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::ImeMode(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::ImeMode);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::ImeMode);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_ime_mode(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_ime_mode();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_ime_mode();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::ImeMode)
            }
    }





    
    

    
        
    
    /// https://drafts.csswg.org/css-ui-4/#propdef-user-select
    pub mod _moz_user_select {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            use style_traits::ToCss;
            define_css_keyword_enum! { SpecifiedValue:
                values {
                        "auto" => auto,
                        "text" => text,
                        "none" => none,
                        "all" => all,
                        "element" => element,
                        "elements" => elements,
                        "toggle" => toggle,
                        "tri-state" => tri_state,
                        "-moz-all" => _moz_all,
                        "-moz-text" => _moz_text,
                }
                aliases {
                        "-moz-none" => none,
                }
            }
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "auto" => auto,
                    "text" => text,
                    "none" => none,
                    "all" => all,
                    "element" => element,
                    "elements" => elements,
                    "toggle" => toggle,
                    "tri-state" => tri_state,
                    "-moz-all" => _moz_all,
                    "-moz-text" => _moz_text,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::auto
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::auto
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::auto => computed_value::T::auto,
                        SpecifiedValue::text => computed_value::T::text,
                        SpecifiedValue::none => computed_value::T::none,
                        SpecifiedValue::all => computed_value::T::all,
                        SpecifiedValue::element => computed_value::T::element,
                        SpecifiedValue::elements => computed_value::T::elements,
                        SpecifiedValue::toggle => computed_value::T::toggle,
                        SpecifiedValue::tri_state => computed_value::T::tri_state,
                        SpecifiedValue::_moz_all => computed_value::T::_moz_all,
                        SpecifiedValue::_moz_text => computed_value::T::_moz_text,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::auto => SpecifiedValue::auto,
                        computed_value::T::text => SpecifiedValue::text,
                        computed_value::T::none => SpecifiedValue::none,
                        computed_value::T::all => SpecifiedValue::all,
                        computed_value::T::element => SpecifiedValue::element,
                        computed_value::T::elements => SpecifiedValue::elements,
                        computed_value::T::toggle => SpecifiedValue::toggle,
                        computed_value::T::tri_state => SpecifiedValue::tri_state,
                        computed_value::T::_moz_all => SpecifiedValue::_moz_all,
                        computed_value::T::_moz_text => SpecifiedValue::_moz_text,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MozUserSelect(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MozUserSelect);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::MozUserSelect);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set__moz_user_select(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset__moz_user_select();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit__moz_user_select();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MozUserSelect)
            }
    }





    
    

    
        
    
    /// None (Nonstandard Firefox-only property)
    pub mod _moz_window_dragging {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "default" => default,
                    "drag" => drag,
                    "no-drag" => no_drag,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::default
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::default
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::default => computed_value::T::default,
                        SpecifiedValue::drag => computed_value::T::drag,
                        SpecifiedValue::no_drag => computed_value::T::no_drag,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::default => SpecifiedValue::default,
                        computed_value::T::drag => SpecifiedValue::drag,
                        computed_value::T::no_drag => SpecifiedValue::no_drag,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MozWindowDragging(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MozWindowDragging);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::MozWindowDragging);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set__moz_window_dragging(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset__moz_window_dragging();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit__moz_window_dragging();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MozWindowDragging)
            }
    }





    
    

    
        
    
    /// None (Nonstandard internal property)
    pub mod _moz_window_shadow {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "none" => none,
                    "default" => default,
                    "menu" => menu,
                    "tooltip" => tooltip,
                    "sheet" => sheet,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::none
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::none
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::none => computed_value::T::none,
                        SpecifiedValue::default => computed_value::T::default,
                        SpecifiedValue::menu => computed_value::T::menu,
                        SpecifiedValue::tooltip => computed_value::T::tooltip,
                        SpecifiedValue::sheet => computed_value::T::sheet,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::none => SpecifiedValue::none,
                        computed_value::T::default => SpecifiedValue::default,
                        computed_value::T::menu => SpecifiedValue::menu,
                        computed_value::T::tooltip => SpecifiedValue::tooltip,
                        computed_value::T::sheet => SpecifiedValue::sheet,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MozWindowShadow(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MozWindowShadow);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::MozWindowShadow);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set__moz_window_shadow(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset__moz_window_shadow();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit__moz_window_shadow();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MozWindowShadow)
            }
    }





    
    /// None (Nonstandard Firefox-only property)
    pub mod _moz_force_broken_image_icon {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
    use std::fmt;
    use style_traits::ToCss;

    pub mod computed_value {
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        #[derive(Clone, Copy, Debug, PartialEq, ToComputedValue)]
        pub struct T(pub bool);
    }

    pub use self::computed_value::T as SpecifiedValue;

    impl ToCss for computed_value::T {
        fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
            dest.write_str(if self.0 { "1" } else { "0" })
        }
    }

    #[inline]
    pub fn get_initial_value() -> computed_value::T {
        computed_value::T(false)
    }

    #[inline]
    pub fn get_initial_specified_value() -> SpecifiedValue {
        computed_value::T(false)
    }

    pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                         -> Result<SpecifiedValue, ParseError<'i>> {
        match input.expect_integer()? {
            0 => Ok(computed_value::T(false)),
            1 => Ok(computed_value::T(true)),
            _ => Err(StyleParseError::UnspecifiedError.into()),
        }
    }

    impl From<u8> for SpecifiedValue {
        fn from(bits: u8) -> SpecifiedValue {
            SpecifiedValue(bits == 1)
        }
    }

    impl From<SpecifiedValue> for u8 {
        fn from(v: SpecifiedValue) -> u8 {
            match v.0 {
                true => 1u8,
                false => 0u8,
            }
        }
    }

        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MozForceBrokenImageIcon(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MozForceBrokenImageIcon);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::MozForceBrokenImageIcon);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set__moz_force_broken_image_icon(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset__moz_force_broken_image_icon();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit__moz_force_broken_image_icon();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MozForceBrokenImageIcon)
            }
    }


    /* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */



// SVG 1.1 (Second Edition)
// https://www.w3.org/TR/SVG/


// Section 10 - Text


    
    

    
        
    
    /// https://www.w3.org/TR/SVG/text.html#TextAnchorProperty
    pub mod text_anchor {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "start" => start,
                    "middle" => middle,
                    "end" => end,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::start
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::start
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::start => computed_value::T::start,
                        SpecifiedValue::middle => computed_value::T::middle,
                        SpecifiedValue::end => computed_value::T::end,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::start => SpecifiedValue::start,
                        computed_value::T::middle => SpecifiedValue::middle,
                        computed_value::T::end => SpecifiedValue::end,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::TextAnchor(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::TextAnchor);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_text_anchor(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_text_anchor();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_text_anchor();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::TextAnchor)
            }
    }




// Section 11 - Painting: Filling, Stroking and Marker Symbols

    
    

    
        
    
    /// https://www.w3.org/TR/SVG11/painting.html#ColorInterpolationProperty
    pub mod color_interpolation {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "srgb" => srgb,
                    "auto" => auto,
                    "linearrgb" => linearrgb,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::srgb
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::srgb
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::srgb => computed_value::T::srgb,
                        SpecifiedValue::auto => computed_value::T::auto,
                        SpecifiedValue::linearrgb => computed_value::T::linearrgb,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::srgb => SpecifiedValue::srgb,
                        computed_value::T::auto => SpecifiedValue::auto,
                        computed_value::T::linearrgb => SpecifiedValue::linearrgb,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::ColorInterpolation(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::ColorInterpolation);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_color_interpolation(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_color_interpolation();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_color_interpolation();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::ColorInterpolation)
            }
    }





    
    

    
        
    
    /// https://www.w3.org/TR/SVG11/painting.html#ColorInterpolationFiltersProperty
    pub mod color_interpolation_filters {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "linearrgb" => linearrgb,
                    "auto" => auto,
                    "srgb" => srgb,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::linearrgb
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::linearrgb
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::linearrgb => computed_value::T::linearrgb,
                        SpecifiedValue::auto => computed_value::T::auto,
                        SpecifiedValue::srgb => computed_value::T::srgb,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::linearrgb => SpecifiedValue::linearrgb,
                        computed_value::T::auto => SpecifiedValue::auto,
                        computed_value::T::srgb => SpecifiedValue::srgb,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::ColorInterpolationFilters(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::ColorInterpolationFilters);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_color_interpolation_filters(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_color_interpolation_filters();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_color_interpolation_filters();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::ColorInterpolationFilters)
            }
    }





    
        
    
    /// https://www.w3.org/TR/SVG2/painting.html#SpecifyingFillPaint
    pub mod fill {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::SVGPaint as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::SVGPaint as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { ::values::computed::SVGPaint::black() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::SVGPaint::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::Fill(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::Fill);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = (**specified_value).to_computed_value(context);
                                context.builder.set_fill(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_fill();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_fill();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<Box<SpecifiedValue>, ParseError<'i>> {
                    parse(context, input).map(|result| Box::new(result))
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::Fill)
            }
    }




    
        
    
    /// https://www.w3.org/TR/SVG11/painting.html#FillOpacityProperty
    pub mod fill_opacity {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::SVGOpacity as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::SVGOpacity as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { Default::default() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::SVGOpacity::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::FillOpacity(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::FillOpacity);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_fill_opacity(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_fill_opacity();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_fill_opacity();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::FillOpacity)
            }
    }




    
    

    
        
    
    /// https://www.w3.org/TR/SVG11/painting.html#FillRuleProperty
    pub mod fill_rule {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "nonzero" => nonzero,
                    "evenodd" => evenodd,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::nonzero
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::nonzero
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::nonzero => computed_value::T::nonzero,
                        SpecifiedValue::evenodd => computed_value::T::evenodd,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::nonzero => SpecifiedValue::nonzero,
                        computed_value::T::evenodd => SpecifiedValue::evenodd,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::FillRule(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::FillRule);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_fill_rule(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_fill_rule();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_fill_rule();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::FillRule)
            }
    }





    
    

    
        
    
    /// https://www.w3.org/TR/SVG11/painting.html#ShapeRenderingProperty
    pub mod shape_rendering {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "auto" => auto,
                    "optimizespeed" => optimizespeed,
                    "crispedges" => crispedges,
                    "geometricprecision" => geometricprecision,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::auto
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::auto
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::auto => computed_value::T::auto,
                        SpecifiedValue::optimizespeed => computed_value::T::optimizespeed,
                        SpecifiedValue::crispedges => computed_value::T::crispedges,
                        SpecifiedValue::geometricprecision => computed_value::T::geometricprecision,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::auto => SpecifiedValue::auto,
                        computed_value::T::optimizespeed => SpecifiedValue::optimizespeed,
                        computed_value::T::crispedges => SpecifiedValue::crispedges,
                        computed_value::T::geometricprecision => SpecifiedValue::geometricprecision,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::ShapeRendering(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::ShapeRendering);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_shape_rendering(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_shape_rendering();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_shape_rendering();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::ShapeRendering)
            }
    }





    
        
    
    /// https://www.w3.org/TR/SVG2/painting.html#SpecifyingStrokePaint
    pub mod stroke {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::SVGPaint as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::SVGPaint as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { Default::default() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::SVGPaint::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::Stroke(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::Stroke);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = (**specified_value).to_computed_value(context);
                                context.builder.set_stroke(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_stroke();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_stroke();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<Box<SpecifiedValue>, ParseError<'i>> {
                    parse(context, input).map(|result| Box::new(result))
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::Stroke)
            }
    }




    
        
    
    /// https://www.w3.org/TR/SVG2/painting.html#StrokeWidth
    pub mod stroke_width {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::SVGWidth as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::SVGWidth as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { ::values::computed::NonNegativeLength::new(1.).into() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::SVGWidth::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::StrokeWidth(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::StrokeWidth);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = (**specified_value).to_computed_value(context);
                                context.builder.set_stroke_width(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_stroke_width();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_stroke_width();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<Box<SpecifiedValue>, ParseError<'i>> {
                    parse(context, input).map(|result| Box::new(result))
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::StrokeWidth)
            }
    }




    
    

    
        
    
    /// https://www.w3.org/TR/SVG11/painting.html#StrokeLinecapProperty
    pub mod stroke_linecap {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "butt" => butt,
                    "round" => round,
                    "square" => square,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::butt
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::butt
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::butt => computed_value::T::butt,
                        SpecifiedValue::round => computed_value::T::round,
                        SpecifiedValue::square => computed_value::T::square,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::butt => SpecifiedValue::butt,
                        computed_value::T::round => SpecifiedValue::round,
                        computed_value::T::square => SpecifiedValue::square,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::StrokeLinecap(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::StrokeLinecap);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_stroke_linecap(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_stroke_linecap();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_stroke_linecap();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::StrokeLinecap)
            }
    }





    
    

    
        
    
    /// https://www.w3.org/TR/SVG11/painting.html#StrokeLinejoinProperty
    pub mod stroke_linejoin {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "miter" => miter,
                    "round" => round,
                    "bevel" => bevel,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::miter
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::miter
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::miter => computed_value::T::miter,
                        SpecifiedValue::round => computed_value::T::round,
                        SpecifiedValue::bevel => computed_value::T::bevel,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::miter => SpecifiedValue::miter,
                        computed_value::T::round => SpecifiedValue::round,
                        computed_value::T::bevel => SpecifiedValue::bevel,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::StrokeLinejoin(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::StrokeLinejoin);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_stroke_linejoin(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_stroke_linejoin();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_stroke_linejoin();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::StrokeLinejoin)
            }
    }





    
        
    
    /// https://www.w3.org/TR/SVG11/painting.html#StrokeMiterlimitProperty
    pub mod stroke_miterlimit {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::GreaterThanOrEqualToOneNumber as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::GreaterThanOrEqualToOneNumber as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { From::from(4.0) }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::GreaterThanOrEqualToOneNumber::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::StrokeMiterlimit(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::StrokeMiterlimit);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_stroke_miterlimit(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_stroke_miterlimit();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_stroke_miterlimit();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::StrokeMiterlimit)
            }
    }




    
        
    
    /// https://www.w3.org/TR/SVG11/painting.html#StrokeOpacityProperty
    pub mod stroke_opacity {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::SVGOpacity as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::SVGOpacity as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { Default::default() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::SVGOpacity::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::StrokeOpacity(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::StrokeOpacity);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_stroke_opacity(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_stroke_opacity();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_stroke_opacity();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::StrokeOpacity)
            }
    }




    
        
    
    /// https://www.w3.org/TR/SVG2/painting.html#StrokeDashing
    pub mod stroke_dasharray {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::SVGStrokeDashArray as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::SVGStrokeDashArray as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { Default::default() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::SVGStrokeDashArray::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::StrokeDasharray(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::StrokeDasharray);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_stroke_dasharray(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_stroke_dasharray();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_stroke_dasharray();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::StrokeDasharray)
            }
    }




    
        
    
    /// https://www.w3.org/TR/SVG2/painting.html#StrokeDashing
    pub mod stroke_dashoffset {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::SVGLength as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::SVGLength as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { Au(0).into() }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::SVGLength::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::StrokeDashoffset(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::StrokeDashoffset);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = (**specified_value).to_computed_value(context);
                                context.builder.set_stroke_dashoffset(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_stroke_dashoffset();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_stroke_dashoffset();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<Box<SpecifiedValue>, ParseError<'i>> {
                    parse(context, input).map(|result| Box::new(result))
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::StrokeDashoffset)
            }
    }



// Section 14 - Clipping, Masking and Compositing

    
    

    
        
    
    /// https://www.w3.org/TR/SVG11/masking.html#ClipRuleProperty
    pub mod clip_rule {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "nonzero" => nonzero,
                    "evenodd" => evenodd,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::nonzero
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::nonzero
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::nonzero => computed_value::T::nonzero,
                        SpecifiedValue::evenodd => computed_value::T::evenodd,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::nonzero => SpecifiedValue::nonzero,
                        computed_value::T::evenodd => SpecifiedValue::evenodd,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::ClipRule(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::ClipRule);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_clip_rule(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_clip_rule();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_clip_rule();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::ClipRule)
            }
    }





    
        
    
    /// https://www.w3.org/TR/SVG2/painting.html#VertexMarkerProperties
    pub mod marker_start {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::UrlOrNone as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::UrlOrNone as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { Either::Second(None_) }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::UrlOrNone::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MarkerStart(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MarkerStart);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = (**specified_value).to_computed_value(context);
                                context.builder.set_marker_start(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_marker_start();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_marker_start();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<Box<SpecifiedValue>, ParseError<'i>> {
                    parse(context, input).map(|result| Box::new(result))
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MarkerStart)
            }
    }




    
        
    
    /// https://www.w3.org/TR/SVG2/painting.html#VertexMarkerProperties
    pub mod marker_mid {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::UrlOrNone as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::UrlOrNone as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { Either::Second(None_) }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::UrlOrNone::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MarkerMid(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MarkerMid);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = (**specified_value).to_computed_value(context);
                                context.builder.set_marker_mid(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_marker_mid();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_marker_mid();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<Box<SpecifiedValue>, ParseError<'i>> {
                    parse(context, input).map(|result| Box::new(result))
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MarkerMid)
            }
    }




    
        
    
    /// https://www.w3.org/TR/SVG2/painting.html#VertexMarkerProperties
    pub mod marker_end {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::UrlOrNone as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::UrlOrNone as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { Either::Second(None_) }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::UrlOrNone::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MarkerEnd(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MarkerEnd);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = (**specified_value).to_computed_value(context);
                                context.builder.set_marker_end(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_marker_end();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_marker_end();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<Box<SpecifiedValue>, ParseError<'i>> {
                    parse(context, input).map(|result| Box::new(result))
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MarkerEnd)
            }
    }




    
    /// https://www.w3.org/TR/SVG2/painting.html#PaintOrder
    pub mod paint_order {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
    use std::fmt;
    use style_traits::ToCss;

    /// The specified value for a single CSS paint-order property.
    #[repr(u8)]
    #[derive(Clone, Copy, Debug, Eq, Ord, PartialEq, PartialOrd, ToCss)]
    pub enum PaintOrder {
        Normal = 0,
        Fill = 1,
        Stroke = 2,
        Markers = 3,
    }

    /// Number of non-normal components.
    const COUNT: u8 = 3;

    /// Number of bits for each component
    const SHIFT: u8 = 2;

    /// Mask with above bits set
    const MASK: u8 = 0b11;

    /// The specified value is tree `PaintOrder` values packed into the
    /// bitfields below, as a six-bit field, of 3 two-bit pairs
    ///
    /// Each pair can be set to FILL, STROKE, or MARKERS
    /// Lowest significant bit pairs are highest priority.
    ///  `normal` is the empty bitfield. The three pairs are
    /// never zero in any case other than `normal`.
    ///
    /// Higher priority values, i.e. the values specified first,
    /// will be painted first (and may be covered by paintings of lower priority)
    #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
    #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
    #[derive(Clone, Copy, Debug, PartialEq, ToComputedValue)]
    pub struct SpecifiedValue(pub u8);

    impl SpecifiedValue {
        fn normal() -> Self {
            SpecifiedValue(0)
        }
    }

    pub mod computed_value {
        pub use super::SpecifiedValue as T;
    }

    pub fn get_initial_value() -> SpecifiedValue {
        SpecifiedValue::normal()
    }

    impl SpecifiedValue {
        fn order_at(&self, pos: u8) -> PaintOrder {
            // Safe because PaintOrder covers all possible patterns.
            unsafe { ::std::mem::transmute((self.0 >> pos * SHIFT) & MASK) }
        }
    }

    pub fn parse<'i, 't>(
        _context: &ParserContext,
        input: &mut Parser<'i, 't>
    ) -> Result<SpecifiedValue,ParseError<'i>> {
        if let Ok(()) = input.try(|i| i.expect_ident_matching("normal")) {
            return Ok(SpecifiedValue::normal())
        }

        let mut value = 0;
        // bitfield representing what we've seen so far
        // bit 1 is fill, bit 2 is stroke, bit 3 is markers
        let mut seen = 0;
        let mut pos = 0;

        loop {
            let result: Result<_, ParseError> = input.try(|i| {
                try_match_ident_ignore_ascii_case! { i.expect_ident()?,
                    "fill" => Ok(PaintOrder::Fill),
                    "stroke" => Ok(PaintOrder::Stroke),
                    "markers" => Ok(PaintOrder::Markers),
                }
            });

            match result {
                Ok(val) => {
                    if (seen & (1 << val as u8)) != 0 {
                        // don't parse the same ident twice
                        return Err(StyleParseError::UnspecifiedError.into())
                    }

                    value |= (val as u8) << (pos * SHIFT);
                    seen |= 1 << (val as u8);
                    pos += 1;
                }
                Err(_) => break,
            }
        }

        if value == 0 {
            // Couldn't find any keyword
            return Err(StyleParseError::UnspecifiedError.into())
        }

        // fill in rest
        for i in pos..COUNT {
            for paint in 0..COUNT {
                // if not seen, set bit at position, mark as seen
                if (seen & (1 << paint)) == 0 {
                    seen |= 1 << paint;
                    value |= paint << (i * SHIFT);
                    break;
                }
            }
        }

        Ok(SpecifiedValue(value))
    }

    impl ToCss for SpecifiedValue {
        fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
            if self.0 == 0 {
                return dest.write_str("normal")
            }

            let mut last_pos_to_serialize = 0;
            for i in (1..COUNT).rev() {
                let component = self.order_at(i);
                let earlier_component = self.order_at(i - 1);
                if component < earlier_component {
                    last_pos_to_serialize = i - 1;
                    break;
                }
            }

            for pos in 0..last_pos_to_serialize + 1 {
                if pos != 0 {
                    dest.write_str(" ")?
                }
                self.order_at(pos).to_css(dest)?;
            }
            Ok(())
        }
    }

        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::PaintOrder(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::PaintOrder);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_paint_order(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset_paint_order();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit_paint_order();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::PaintOrder)
            }
    }



    
    
    /// Nonstandard (Internal-only)
    pub mod _moz_context_properties {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        use std::fmt;
        use style_traits::{Separator, ToCss};

        pub mod single_value {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::ShorthandId;
            #[allow(unused_imports)]
            use selectors::parser::SelectorParseError;
            #[allow(unused_imports)]
            use style_traits::{ParseError, StyleParseError};
            #[allow(unused_imports)]
            use values::computed::{Context, ToComputedValue};
            #[allow(unused_imports)]
            use values::{computed, specified};
            #[allow(unused_imports)]
            use values::{Auto, Either, None_, Normal};
            
    use values::CustomIdent;

    pub type SpecifiedValue = CustomIdent;

    pub mod computed_value {
        pub type T = super::SpecifiedValue;
    }


    pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                         -> Result<SpecifiedValue, ParseError<'i>> {
        let i = input.expect_ident()?;
        CustomIdent::from_ident(i, &["all", "none", "auto"])
    }

        }

        /// The definition of the computed value for -moz-context-properties.
        pub mod computed_value {
            pub use super::single_value::computed_value as single_value;
            pub use self::single_value::T as SingleComputedValue;
            use std::vec::IntoIter;
            use values::computed::ComputedVecIter;

            /// The computed value, effectively a list of single values.
            #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
            #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
            #[derive(Clone, Debug, PartialEq)]
            pub struct T(
                pub Vec<single_value::T>,
            );


            pub type Iter<'a, 'cx, 'cx_a> = ComputedVecIter<'a, 'cx, 'cx_a, super::single_value::SpecifiedValue>;

            impl IntoIterator for T {
                type Item = single_value::T;
                type IntoIter = IntoIter<single_value::T>;
                fn into_iter(self) -> Self::IntoIter {
                    self.0.into_iter()
                }
            }
        }

        impl ToCss for computed_value::T {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        dest.write_str("none")?;
                }
                for i in iter {
                    dest.write_str(::style_traits::Comma::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        /// The specified value of -moz-context-properties.
        #[derive(Clone, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        pub struct SpecifiedValue(pub Vec<single_value::SpecifiedValue>);

        impl ToCss for SpecifiedValue {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        dest.write_str("none")?;
                }
                for i in iter {
                    dest.write_str(::style_traits::Comma::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        pub fn get_initial_value() -> computed_value::T {
                computed_value::T(vec![])
        }

        pub fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                -> Result<SpecifiedValue, ParseError<'i>> {
            use style_traits::Separator;

                if input.try(|input| input.expect_ident_matching("none")).is_ok() {
                    return Ok(SpecifiedValue(Vec::new()))
                }

            ::style_traits::Comma::parse(input, |parser| {
                single_value::parse(context, parser)
            }).map(SpecifiedValue)
        }

        pub use self::single_value::SpecifiedValue as SingleSpecifiedValue;

        impl SpecifiedValue {
            pub fn compute_iter<'a, 'cx, 'cx_a>(&'a self, context: &'cx Context<'cx_a>)
                -> computed_value::Iter<'a, 'cx, 'cx_a> {
                computed_value::Iter::new(context, &self.0)
            }
        }

        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, context: &Context) -> computed_value::T {
                computed_value::T(self.compute_iter(context).collect())
            }

            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                SpecifiedValue(computed.0.iter()
                                    .map(ToComputedValue::from_computed_value)
                                    .collect())
            }
        }
    
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MozContextProperties(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MozContextProperties);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    None;

                match value {
                    DeclaredValue::Value(specified_value) => {
                            // In the case of a vector property we want to pass
                            // down an iterator so that this can be computed
                            // without allocation
                            //
                            // However, computing requires a context, but the
                            // style struct being mutated is on the context. We
                            // temporarily remove it, mutate it, and then put it
                            // back. Vector longhands cannot touch their own
                            // style struct whilst computing, else this will
                            // panic.
                            let mut s =
                                context.builder.take_inheritedsvg();
                            {
                                let iter = specified_value.compute_iter(context);
                                s.set__moz_context_properties(iter);
                            }
                            context.builder.put_inheritedsvg(s);
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Initial => {
                                context.builder.reset__moz_context_properties();
                        },
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Inherit => {
                                context.builder.inherit__moz_context_properties();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MozContextProperties)
            }
    }



    /* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */






    
    

    
        
    
    /// https://www.w3.org/TR/SVG11/text.html#DominantBaselineProperty
    pub mod dominant_baseline {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "auto" => auto,
                    "use-script" => use_script,
                    "no-change" => no_change,
                    "reset-size" => reset_size,
                    "ideographic" => ideographic,
                    "alphabetic" => alphabetic,
                    "hanging" => hanging,
                    "mathematical" => mathematical,
                    "central" => central,
                    "middle" => middle,
                    "text-after-edge" => text_after_edge,
                    "text-before-edge" => text_before_edge,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::auto
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::auto
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::auto => computed_value::T::auto,
                        SpecifiedValue::use_script => computed_value::T::use_script,
                        SpecifiedValue::no_change => computed_value::T::no_change,
                        SpecifiedValue::reset_size => computed_value::T::reset_size,
                        SpecifiedValue::ideographic => computed_value::T::ideographic,
                        SpecifiedValue::alphabetic => computed_value::T::alphabetic,
                        SpecifiedValue::hanging => computed_value::T::hanging,
                        SpecifiedValue::mathematical => computed_value::T::mathematical,
                        SpecifiedValue::central => computed_value::T::central,
                        SpecifiedValue::middle => computed_value::T::middle,
                        SpecifiedValue::text_after_edge => computed_value::T::text_after_edge,
                        SpecifiedValue::text_before_edge => computed_value::T::text_before_edge,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::auto => SpecifiedValue::auto,
                        computed_value::T::use_script => SpecifiedValue::use_script,
                        computed_value::T::no_change => SpecifiedValue::no_change,
                        computed_value::T::reset_size => SpecifiedValue::reset_size,
                        computed_value::T::ideographic => SpecifiedValue::ideographic,
                        computed_value::T::alphabetic => SpecifiedValue::alphabetic,
                        computed_value::T::hanging => SpecifiedValue::hanging,
                        computed_value::T::mathematical => SpecifiedValue::mathematical,
                        computed_value::T::central => SpecifiedValue::central,
                        computed_value::T::middle => SpecifiedValue::middle,
                        computed_value::T::text_after_edge => SpecifiedValue::text_after_edge,
                        computed_value::T::text_before_edge => SpecifiedValue::text_before_edge,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::DominantBaseline(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::DominantBaseline);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::DominantBaseline);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_dominant_baseline(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_dominant_baseline();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_dominant_baseline();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::DominantBaseline)
            }
    }





    
    

    
        
    
    /// https://www.w3.org/TR/SVGTiny12/painting.html#VectorEffectProperty
    pub mod vector_effect {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "none" => none,
                    "non-scaling-stroke" => non_scaling_stroke,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::none
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::none
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::none => computed_value::T::none,
                        SpecifiedValue::non_scaling_stroke => computed_value::T::non_scaling_stroke,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::none => SpecifiedValue::none,
                        computed_value::T::non_scaling_stroke => SpecifiedValue::non_scaling_stroke,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::VectorEffect(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::VectorEffect);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::VectorEffect);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_vector_effect(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_vector_effect();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_vector_effect();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::VectorEffect)
            }
    }




// Section 13 - Gradients and Patterns


    
        
    
    /// https://www.w3.org/TR/SVGTiny12/painting.html#StopColorProperty
    pub mod stop_color {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::RGBAColor as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::RGBAColor as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { RGBA::new(0, 0, 0, 255) }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::RGBAColor::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::StopColor(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::StopColor);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::StopColor);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_stop_color(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_stop_color();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_stop_color();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::StopColor)
            }
    }




    
        
    
    /// https://www.w3.org/TR/SVGTiny12/painting.html#propdef-stop-opacity
    pub mod stop_opacity {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::Opacity as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::Opacity as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { 1.0 }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::Opacity::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::StopOpacity(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::StopOpacity);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::StopOpacity);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_stop_opacity(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_stop_opacity();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_stop_opacity();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::StopOpacity)
            }
    }



// Section 15 - Filter Effects


    
        
    
    /// https://www.w3.org/TR/SVG/filters.html#FloodColorProperty
    pub mod flood_color {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::RGBAColor as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::RGBAColor as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { RGBA::new(0, 0, 0, 255) }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::RGBAColor::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::FloodColor(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::FloodColor);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::FloodColor);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_flood_color(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_flood_color();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_flood_color();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::FloodColor)
            }
    }




    
        
    
    /// https://www.w3.org/TR/SVG/filters.html#FloodOpacityProperty
    pub mod flood_opacity {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::Opacity as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::Opacity as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { 1.0 }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::Opacity::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::FloodOpacity(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::FloodOpacity);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::FloodOpacity);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_flood_opacity(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_flood_opacity();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_flood_opacity();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::FloodOpacity)
            }
    }




    
        
    
    /// https://www.w3.org/TR/SVG/filters.html#LightingColorProperty
    pub mod lighting_color {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::RGBAColor as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::RGBAColor as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { RGBA::new(255, 255, 255, 255) }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::RGBAColor::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::LightingColor(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::LightingColor);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::LightingColor);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_lighting_color(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_lighting_color();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_lighting_color();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::LightingColor)
            }
    }



// CSS Masking Module Level 1
// https://drafts.fxtf.org/css-masking

    
    

    
        
    
    /// https://drafts.fxtf.org/css-masking/#propdef-mask-type
    pub mod mask_type {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "luminance" => luminance,
                    "alpha" => alpha,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::luminance
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::luminance
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::luminance => computed_value::T::luminance,
                        SpecifiedValue::alpha => computed_value::T::alpha,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::luminance => SpecifiedValue::luminance,
                        computed_value::T::alpha => SpecifiedValue::alpha,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MaskType(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MaskType);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::MaskType);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_mask_type(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_mask_type();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_mask_type();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MaskType)
            }
    }





    
        
    
    /// https://drafts.fxtf.org/css-masking/#propdef-clip-path
    pub mod clip_path {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::basic_shape::ClippingShape as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::basic_shape::ClippingShape as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { generics::basic_shape::ShapeSource::None }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::basic_shape::ClippingShape::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::ClipPath(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::ClipPath);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::ClipPath);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = (**specified_value).to_computed_value(context);
                                context.builder.set_clip_path(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_clip_path();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_clip_path();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<Box<SpecifiedValue>, ParseError<'i>> {
                    parse(context, input).map(|result| Box::new(result))
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::ClipPath)
            }
    }




    
    

    
        
    
    
    /// https://drafts.fxtf.org/css-masking/#propdef-mask-mode
    pub mod mask_mode {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        use std::fmt;
        use style_traits::{Separator, ToCss};

        pub mod single_value {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::ShorthandId;
            #[allow(unused_imports)]
            use selectors::parser::SelectorParseError;
            #[allow(unused_imports)]
            use style_traits::{ParseError, StyleParseError};
            #[allow(unused_imports)]
            use values::computed::{Context, ToComputedValue};
            #[allow(unused_imports)]
            use values::{computed, specified};
            #[allow(unused_imports)]
            use values::{Auto, Either, None_, Normal};
            
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "match-source" => match_source,
                    "alpha" => alpha,
                    "luminance" => luminance,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::match_source
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::match_source
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::match_source => computed_value::T::match_source,
                        SpecifiedValue::alpha => computed_value::T::alpha,
                        SpecifiedValue::luminance => computed_value::T::luminance,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::match_source => SpecifiedValue::match_source,
                        computed_value::T::alpha => SpecifiedValue::alpha,
                        computed_value::T::luminance => SpecifiedValue::luminance,
                }
            }
        }
    
        
        }

        /// The definition of the computed value for mask-mode.
        pub mod computed_value {
            pub use super::single_value::computed_value as single_value;
            pub use self::single_value::T as SingleComputedValue;
            use smallvec::{IntoIter, SmallVec};
            use values::computed::ComputedVecIter;

            /// The computed value, effectively a list of single values.
            #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
            #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
            #[derive(Clone, Debug, PartialEq)]
            pub struct T(
                pub SmallVec<[single_value::T; 1]>,
            );


            pub type Iter<'a, 'cx, 'cx_a> = ComputedVecIter<'a, 'cx, 'cx_a, super::single_value::SpecifiedValue>;

            impl IntoIterator for T {
                type Item = single_value::T;
                type IntoIter = IntoIter<[single_value::T; 1]>;
                fn into_iter(self) -> Self::IntoIter {
                    self.0.into_iter()
                }
            }
        }

        impl ToCss for computed_value::T {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        warn!("Found empty value for property mask-mode");
                }
                for i in iter {
                    dest.write_str(::style_traits::Comma::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        /// The specified value of mask-mode.
        #[derive(Clone, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        pub struct SpecifiedValue(pub Vec<single_value::SpecifiedValue>);

        impl ToCss for SpecifiedValue {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        warn!("Found empty value for property mask-mode");
                }
                for i in iter {
                    dest.write_str(::style_traits::Comma::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        pub fn get_initial_value() -> computed_value::T {
                let mut v = SmallVec::new();
                v.push(single_value::get_initial_value());
                computed_value::T(v)
        }

        pub fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                -> Result<SpecifiedValue, ParseError<'i>> {
            use style_traits::Separator;


            ::style_traits::Comma::parse(input, |parser| {
                single_value::parse(context, parser)
            }).map(SpecifiedValue)
        }

        pub use self::single_value::SpecifiedValue as SingleSpecifiedValue;

        impl SpecifiedValue {
            pub fn compute_iter<'a, 'cx, 'cx_a>(&'a self, context: &'cx Context<'cx_a>)
                -> computed_value::Iter<'a, 'cx, 'cx_a> {
                computed_value::Iter::new(context, &self.0)
            }
        }

        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, context: &Context) -> computed_value::T {
                computed_value::T(self.compute_iter(context).collect())
            }

            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                SpecifiedValue(computed.0.iter()
                                    .map(ToComputedValue::from_computed_value)
                                    .collect())
            }
        }
    
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MaskMode(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MaskMode);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::MaskMode);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            // In the case of a vector property we want to pass
                            // down an iterator so that this can be computed
                            // without allocation
                            //
                            // However, computing requires a context, but the
                            // style struct being mutated is on the context. We
                            // temporarily remove it, mutate it, and then put it
                            // back. Vector longhands cannot touch their own
                            // style struct whilst computing, else this will
                            // panic.
                            let mut s =
                                context.builder.take_svg();
                            {
                                let iter = specified_value.compute_iter(context);
                                s.set_mask_mode(iter);
                            }
                            context.builder.put_svg(s);
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_mask_mode();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_mask_mode();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MaskMode)
            }
    }






    
    
    /// https://drafts.fxtf.org/css-masking/#propdef-mask-repeat
    pub mod mask_repeat {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        use std::fmt;
        use style_traits::{Separator, ToCss};

        pub mod single_value {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::ShorthandId;
            #[allow(unused_imports)]
            use selectors::parser::SelectorParseError;
            #[allow(unused_imports)]
            use style_traits::{ParseError, StyleParseError};
            #[allow(unused_imports)]
            use values::computed::{Context, ToComputedValue};
            #[allow(unused_imports)]
            use values::{computed, specified};
            #[allow(unused_imports)]
            use values::{Auto, Either, None_, Normal};
            
    pub use properties::longhands::background_repeat::single_value::parse;
    pub use properties::longhands::background_repeat::single_value::SpecifiedValue;
    pub use properties::longhands::background_repeat::single_value::computed_value;
    pub use properties::longhands::background_repeat::single_value::RepeatKeyword;

    #[inline]
    pub fn get_initial_value() -> computed_value::T {
        computed_value::T(RepeatKeyword::Repeat, RepeatKeyword::Repeat)
    }

    #[inline]
    pub fn get_initial_specified_value() -> SpecifiedValue {
        SpecifiedValue::Other(RepeatKeyword::Repeat, None)
    }

        }

        /// The definition of the computed value for mask-repeat.
        pub mod computed_value {
            pub use super::single_value::computed_value as single_value;
            pub use self::single_value::T as SingleComputedValue;
            use smallvec::{IntoIter, SmallVec};
            use values::computed::ComputedVecIter;

            /// The computed value, effectively a list of single values.
            #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
            #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
            #[derive(Clone, Debug, PartialEq)]
            pub struct T(
                pub SmallVec<[single_value::T; 1]>,
            );


            pub type Iter<'a, 'cx, 'cx_a> = ComputedVecIter<'a, 'cx, 'cx_a, super::single_value::SpecifiedValue>;

            impl IntoIterator for T {
                type Item = single_value::T;
                type IntoIter = IntoIter<[single_value::T; 1]>;
                fn into_iter(self) -> Self::IntoIter {
                    self.0.into_iter()
                }
            }
        }

        impl ToCss for computed_value::T {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        warn!("Found empty value for property mask-repeat");
                }
                for i in iter {
                    dest.write_str(::style_traits::Comma::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        /// The specified value of mask-repeat.
        #[derive(Clone, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        pub struct SpecifiedValue(pub Vec<single_value::SpecifiedValue>);

        impl ToCss for SpecifiedValue {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        warn!("Found empty value for property mask-repeat");
                }
                for i in iter {
                    dest.write_str(::style_traits::Comma::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        pub fn get_initial_value() -> computed_value::T {
                let mut v = SmallVec::new();
                v.push(single_value::get_initial_value());
                computed_value::T(v)
        }

        pub fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                -> Result<SpecifiedValue, ParseError<'i>> {
            use style_traits::Separator;


            ::style_traits::Comma::parse(input, |parser| {
                single_value::parse(context, parser)
            }).map(SpecifiedValue)
        }

        pub use self::single_value::SpecifiedValue as SingleSpecifiedValue;

        impl SpecifiedValue {
            pub fn compute_iter<'a, 'cx, 'cx_a>(&'a self, context: &'cx Context<'cx_a>)
                -> computed_value::Iter<'a, 'cx, 'cx_a> {
                computed_value::Iter::new(context, &self.0)
            }
        }

        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, context: &Context) -> computed_value::T {
                computed_value::T(self.compute_iter(context).collect())
            }

            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                SpecifiedValue(computed.0.iter()
                                    .map(ToComputedValue::from_computed_value)
                                    .collect())
            }
        }
    
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MaskRepeat(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MaskRepeat);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::MaskRepeat);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            // In the case of a vector property we want to pass
                            // down an iterator so that this can be computed
                            // without allocation
                            //
                            // However, computing requires a context, but the
                            // style struct being mutated is on the context. We
                            // temporarily remove it, mutate it, and then put it
                            // back. Vector longhands cannot touch their own
                            // style struct whilst computing, else this will
                            // panic.
                            let mut s =
                                context.builder.take_svg();
                            {
                                let iter = specified_value.compute_iter(context);
                                s.set_mask_repeat(iter);
                            }
                            context.builder.put_svg(s);
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_mask_repeat();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_mask_repeat();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MaskRepeat)
            }
    }



    
    
        
    
    
    /// https://drafts.fxtf.org/css-masking/#propdef-mask-position
    pub mod mask_position_x {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        use std::fmt;
        use style_traits::{Separator, ToCss};

        pub mod single_value {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::ShorthandId;
            #[allow(unused_imports)]
            use selectors::parser::SelectorParseError;
            #[allow(unused_imports)]
            use style_traits::{ParseError, StyleParseError};
            #[allow(unused_imports)]
            use values::computed::{Context, ToComputedValue};
            #[allow(unused_imports)]
            use values::{computed, specified};
            #[allow(unused_imports)]
            use values::{Auto, Either, None_, Normal};
            
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::position::HorizontalPosition as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::position::HorizontalPosition as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::LengthOrPercentage::zero() }
        #[inline] pub fn get_initial_specified_value() -> SpecifiedValue { specified::PositionComponent::Center }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::position::HorizontalPosition::parse(context, input)
        }
    
        
        }

        /// The definition of the computed value for mask-position-x.
        pub mod computed_value {
            pub use super::single_value::computed_value as single_value;
            pub use self::single_value::T as SingleComputedValue;
            use smallvec::{IntoIter, SmallVec};
            use values::computed::ComputedVecIter;

            /// The computed value, effectively a list of single values.
            #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
            #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
            #[derive(Clone, Debug, PartialEq)]
            #[derive(Animate, ComputeSquaredDistance)]
            pub struct T(
                pub SmallVec<[single_value::T; 1]>,
            );

                use values::animated::{ToAnimatedZero};

                impl ToAnimatedZero for T {
                    #[inline]
                    fn to_animated_zero(&self) -> Result<Self, ()> { Err(()) }
                }

            pub type Iter<'a, 'cx, 'cx_a> = ComputedVecIter<'a, 'cx, 'cx_a, super::single_value::SpecifiedValue>;

            impl IntoIterator for T {
                type Item = single_value::T;
                type IntoIter = IntoIter<[single_value::T; 1]>;
                fn into_iter(self) -> Self::IntoIter {
                    self.0.into_iter()
                }
            }
        }

        impl ToCss for computed_value::T {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        warn!("Found empty value for property mask-position-x");
                }
                for i in iter {
                    dest.write_str(::style_traits::Comma::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        /// The specified value of mask-position-x.
        #[derive(Clone, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        pub struct SpecifiedValue(pub Vec<single_value::SpecifiedValue>);

        impl ToCss for SpecifiedValue {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        warn!("Found empty value for property mask-position-x");
                }
                for i in iter {
                    dest.write_str(::style_traits::Comma::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        pub fn get_initial_value() -> computed_value::T {
                let mut v = SmallVec::new();
                v.push(single_value::get_initial_value());
                computed_value::T(v)
        }

        pub fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                -> Result<SpecifiedValue, ParseError<'i>> {
            use style_traits::Separator;


            ::style_traits::Comma::parse(input, |parser| {
                single_value::parse(context, parser)
            }).map(SpecifiedValue)
        }

        pub use self::single_value::SpecifiedValue as SingleSpecifiedValue;

        impl SpecifiedValue {
            pub fn compute_iter<'a, 'cx, 'cx_a>(&'a self, context: &'cx Context<'cx_a>)
                -> computed_value::Iter<'a, 'cx, 'cx_a> {
                computed_value::Iter::new(context, &self.0)
            }
        }

        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, context: &Context) -> computed_value::T {
                computed_value::T(self.compute_iter(context).collect())
            }

            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                SpecifiedValue(computed.0.iter()
                                    .map(ToComputedValue::from_computed_value)
                                    .collect())
            }
        }
    
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MaskPositionX(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MaskPositionX);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::MaskPositionX);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            // In the case of a vector property we want to pass
                            // down an iterator so that this can be computed
                            // without allocation
                            //
                            // However, computing requires a context, but the
                            // style struct being mutated is on the context. We
                            // temporarily remove it, mutate it, and then put it
                            // back. Vector longhands cannot touch their own
                            // style struct whilst computing, else this will
                            // panic.
                            let mut s =
                                context.builder.take_svg();
                            {
                                let iter = specified_value.compute_iter(context);
                                s.set_mask_position_x(iter);
                            }
                            context.builder.put_svg(s);
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_mask_position_x();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_mask_position_x();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MaskPositionX)
            }
    }



    
    
        
    
    
    /// https://drafts.fxtf.org/css-masking/#propdef-mask-position
    pub mod mask_position_y {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        use std::fmt;
        use style_traits::{Separator, ToCss};

        pub mod single_value {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::ShorthandId;
            #[allow(unused_imports)]
            use selectors::parser::SelectorParseError;
            #[allow(unused_imports)]
            use style_traits::{ParseError, StyleParseError};
            #[allow(unused_imports)]
            use values::computed::{Context, ToComputedValue};
            #[allow(unused_imports)]
            use values::{computed, specified};
            #[allow(unused_imports)]
            use values::{Auto, Either, None_, Normal};
            
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::position::VerticalPosition as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::position::VerticalPosition as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { computed::LengthOrPercentage::zero() }
        #[inline] pub fn get_initial_specified_value() -> SpecifiedValue { specified::PositionComponent::Center }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::position::VerticalPosition::parse(context, input)
        }
    
        
        }

        /// The definition of the computed value for mask-position-y.
        pub mod computed_value {
            pub use super::single_value::computed_value as single_value;
            pub use self::single_value::T as SingleComputedValue;
            use smallvec::{IntoIter, SmallVec};
            use values::computed::ComputedVecIter;

            /// The computed value, effectively a list of single values.
            #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
            #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
            #[derive(Clone, Debug, PartialEq)]
            #[derive(Animate, ComputeSquaredDistance)]
            pub struct T(
                pub SmallVec<[single_value::T; 1]>,
            );

                use values::animated::{ToAnimatedZero};

                impl ToAnimatedZero for T {
                    #[inline]
                    fn to_animated_zero(&self) -> Result<Self, ()> { Err(()) }
                }

            pub type Iter<'a, 'cx, 'cx_a> = ComputedVecIter<'a, 'cx, 'cx_a, super::single_value::SpecifiedValue>;

            impl IntoIterator for T {
                type Item = single_value::T;
                type IntoIter = IntoIter<[single_value::T; 1]>;
                fn into_iter(self) -> Self::IntoIter {
                    self.0.into_iter()
                }
            }
        }

        impl ToCss for computed_value::T {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        warn!("Found empty value for property mask-position-y");
                }
                for i in iter {
                    dest.write_str(::style_traits::Comma::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        /// The specified value of mask-position-y.
        #[derive(Clone, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        pub struct SpecifiedValue(pub Vec<single_value::SpecifiedValue>);

        impl ToCss for SpecifiedValue {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        warn!("Found empty value for property mask-position-y");
                }
                for i in iter {
                    dest.write_str(::style_traits::Comma::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        pub fn get_initial_value() -> computed_value::T {
                let mut v = SmallVec::new();
                v.push(single_value::get_initial_value());
                computed_value::T(v)
        }

        pub fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                -> Result<SpecifiedValue, ParseError<'i>> {
            use style_traits::Separator;


            ::style_traits::Comma::parse(input, |parser| {
                single_value::parse(context, parser)
            }).map(SpecifiedValue)
        }

        pub use self::single_value::SpecifiedValue as SingleSpecifiedValue;

        impl SpecifiedValue {
            pub fn compute_iter<'a, 'cx, 'cx_a>(&'a self, context: &'cx Context<'cx_a>)
                -> computed_value::Iter<'a, 'cx, 'cx_a> {
                computed_value::Iter::new(context, &self.0)
            }
        }

        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, context: &Context) -> computed_value::T {
                computed_value::T(self.compute_iter(context).collect())
            }

            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                SpecifiedValue(computed.0.iter()
                                    .map(ToComputedValue::from_computed_value)
                                    .collect())
            }
        }
    
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MaskPositionY(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MaskPositionY);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::MaskPositionY);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            // In the case of a vector property we want to pass
                            // down an iterator so that this can be computed
                            // without allocation
                            //
                            // However, computing requires a context, but the
                            // style struct being mutated is on the context. We
                            // temporarily remove it, mutate it, and then put it
                            // back. Vector longhands cannot touch their own
                            // style struct whilst computing, else this will
                            // panic.
                            let mut s =
                                context.builder.take_svg();
                            {
                                let iter = specified_value.compute_iter(context);
                                s.set_mask_position_y(iter);
                            }
                            context.builder.put_svg(s);
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_mask_position_y();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_mask_position_y();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MaskPositionY)
            }
    }





    
    

    
        
    
    
    /// https://drafts.fxtf.org/css-masking/#propdef-mask-clip
    pub mod mask_clip {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        use std::fmt;
        use style_traits::{Separator, ToCss};

        pub mod single_value {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::ShorthandId;
            #[allow(unused_imports)]
            use selectors::parser::SelectorParseError;
            #[allow(unused_imports)]
            use style_traits::{ParseError, StyleParseError};
            #[allow(unused_imports)]
            use values::computed::{Context, ToComputedValue};
            #[allow(unused_imports)]
            use values::{computed, specified};
            #[allow(unused_imports)]
            use values::{Auto, Either, None_, Normal};
            
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "border-box" => border_box,
                    "content-box" => content_box,
                    "padding-box" => padding_box,
                    "fill-box" => fill_box,
                    "stroke-box" => stroke_box,
                    "view-box" => view_box,
                    "no-clip" => no_clip,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::border_box
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::border_box
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::border_box => computed_value::T::border_box,
                        SpecifiedValue::content_box => computed_value::T::content_box,
                        SpecifiedValue::padding_box => computed_value::T::padding_box,
                        SpecifiedValue::fill_box => computed_value::T::fill_box,
                        SpecifiedValue::stroke_box => computed_value::T::stroke_box,
                        SpecifiedValue::view_box => computed_value::T::view_box,
                        SpecifiedValue::no_clip => computed_value::T::no_clip,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::border_box => SpecifiedValue::border_box,
                        computed_value::T::content_box => SpecifiedValue::content_box,
                        computed_value::T::padding_box => SpecifiedValue::padding_box,
                        computed_value::T::fill_box => SpecifiedValue::fill_box,
                        computed_value::T::stroke_box => SpecifiedValue::stroke_box,
                        computed_value::T::view_box => SpecifiedValue::view_box,
                        computed_value::T::no_clip => SpecifiedValue::no_clip,
                }
            }
        }
    
        
        }

        /// The definition of the computed value for mask-clip.
        pub mod computed_value {
            pub use super::single_value::computed_value as single_value;
            pub use self::single_value::T as SingleComputedValue;
            use smallvec::{IntoIter, SmallVec};
            use values::computed::ComputedVecIter;

            /// The computed value, effectively a list of single values.
            #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
            #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
            #[derive(Clone, Debug, PartialEq)]
            pub struct T(
                pub SmallVec<[single_value::T; 1]>,
            );


            pub type Iter<'a, 'cx, 'cx_a> = ComputedVecIter<'a, 'cx, 'cx_a, super::single_value::SpecifiedValue>;

            impl IntoIterator for T {
                type Item = single_value::T;
                type IntoIter = IntoIter<[single_value::T; 1]>;
                fn into_iter(self) -> Self::IntoIter {
                    self.0.into_iter()
                }
            }
        }

        impl ToCss for computed_value::T {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        warn!("Found empty value for property mask-clip");
                }
                for i in iter {
                    dest.write_str(::style_traits::Comma::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        /// The specified value of mask-clip.
        #[derive(Clone, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        pub struct SpecifiedValue(pub Vec<single_value::SpecifiedValue>);

        impl ToCss for SpecifiedValue {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        warn!("Found empty value for property mask-clip");
                }
                for i in iter {
                    dest.write_str(::style_traits::Comma::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        pub fn get_initial_value() -> computed_value::T {
                let mut v = SmallVec::new();
                v.push(single_value::get_initial_value());
                computed_value::T(v)
        }

        pub fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                -> Result<SpecifiedValue, ParseError<'i>> {
            use style_traits::Separator;


            ::style_traits::Comma::parse(input, |parser| {
                single_value::parse(context, parser)
            }).map(SpecifiedValue)
        }

        pub use self::single_value::SpecifiedValue as SingleSpecifiedValue;

        impl SpecifiedValue {
            pub fn compute_iter<'a, 'cx, 'cx_a>(&'a self, context: &'cx Context<'cx_a>)
                -> computed_value::Iter<'a, 'cx, 'cx_a> {
                computed_value::Iter::new(context, &self.0)
            }
        }

        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, context: &Context) -> computed_value::T {
                computed_value::T(self.compute_iter(context).collect())
            }

            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                SpecifiedValue(computed.0.iter()
                                    .map(ToComputedValue::from_computed_value)
                                    .collect())
            }
        }
    
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MaskClip(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MaskClip);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::MaskClip);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            // In the case of a vector property we want to pass
                            // down an iterator so that this can be computed
                            // without allocation
                            //
                            // However, computing requires a context, but the
                            // style struct being mutated is on the context. We
                            // temporarily remove it, mutate it, and then put it
                            // back. Vector longhands cannot touch their own
                            // style struct whilst computing, else this will
                            // panic.
                            let mut s =
                                context.builder.take_svg();
                            {
                                let iter = specified_value.compute_iter(context);
                                s.set_mask_clip(iter);
                            }
                            context.builder.put_svg(s);
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_mask_clip();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_mask_clip();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MaskClip)
            }
    }






    
    

    
        
    
    
    /// https://drafts.fxtf.org/css-masking/#propdef-mask-origin
    pub mod mask_origin {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        use std::fmt;
        use style_traits::{Separator, ToCss};

        pub mod single_value {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::ShorthandId;
            #[allow(unused_imports)]
            use selectors::parser::SelectorParseError;
            #[allow(unused_imports)]
            use style_traits::{ParseError, StyleParseError};
            #[allow(unused_imports)]
            use values::computed::{Context, ToComputedValue};
            #[allow(unused_imports)]
            use values::{computed, specified};
            #[allow(unused_imports)]
            use values::{Auto, Either, None_, Normal};
            
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "border-box" => border_box,
                    "content-box" => content_box,
                    "padding-box" => padding_box,
                    "fill-box" => fill_box,
                    "stroke-box" => stroke_box,
                    "view-box" => view_box,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::border_box
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::border_box
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::border_box => computed_value::T::border_box,
                        SpecifiedValue::content_box => computed_value::T::content_box,
                        SpecifiedValue::padding_box => computed_value::T::padding_box,
                        SpecifiedValue::fill_box => computed_value::T::fill_box,
                        SpecifiedValue::stroke_box => computed_value::T::stroke_box,
                        SpecifiedValue::view_box => computed_value::T::view_box,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::border_box => SpecifiedValue::border_box,
                        computed_value::T::content_box => SpecifiedValue::content_box,
                        computed_value::T::padding_box => SpecifiedValue::padding_box,
                        computed_value::T::fill_box => SpecifiedValue::fill_box,
                        computed_value::T::stroke_box => SpecifiedValue::stroke_box,
                        computed_value::T::view_box => SpecifiedValue::view_box,
                }
            }
        }
    
        
        }

        /// The definition of the computed value for mask-origin.
        pub mod computed_value {
            pub use super::single_value::computed_value as single_value;
            pub use self::single_value::T as SingleComputedValue;
            use smallvec::{IntoIter, SmallVec};
            use values::computed::ComputedVecIter;

            /// The computed value, effectively a list of single values.
            #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
            #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
            #[derive(Clone, Debug, PartialEq)]
            pub struct T(
                pub SmallVec<[single_value::T; 1]>,
            );


            pub type Iter<'a, 'cx, 'cx_a> = ComputedVecIter<'a, 'cx, 'cx_a, super::single_value::SpecifiedValue>;

            impl IntoIterator for T {
                type Item = single_value::T;
                type IntoIter = IntoIter<[single_value::T; 1]>;
                fn into_iter(self) -> Self::IntoIter {
                    self.0.into_iter()
                }
            }
        }

        impl ToCss for computed_value::T {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        warn!("Found empty value for property mask-origin");
                }
                for i in iter {
                    dest.write_str(::style_traits::Comma::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        /// The specified value of mask-origin.
        #[derive(Clone, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        pub struct SpecifiedValue(pub Vec<single_value::SpecifiedValue>);

        impl ToCss for SpecifiedValue {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        warn!("Found empty value for property mask-origin");
                }
                for i in iter {
                    dest.write_str(::style_traits::Comma::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        pub fn get_initial_value() -> computed_value::T {
                let mut v = SmallVec::new();
                v.push(single_value::get_initial_value());
                computed_value::T(v)
        }

        pub fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                -> Result<SpecifiedValue, ParseError<'i>> {
            use style_traits::Separator;


            ::style_traits::Comma::parse(input, |parser| {
                single_value::parse(context, parser)
            }).map(SpecifiedValue)
        }

        pub use self::single_value::SpecifiedValue as SingleSpecifiedValue;

        impl SpecifiedValue {
            pub fn compute_iter<'a, 'cx, 'cx_a>(&'a self, context: &'cx Context<'cx_a>)
                -> computed_value::Iter<'a, 'cx, 'cx_a> {
                computed_value::Iter::new(context, &self.0)
            }
        }

        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, context: &Context) -> computed_value::T {
                computed_value::T(self.compute_iter(context).collect())
            }

            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                SpecifiedValue(computed.0.iter()
                                    .map(ToComputedValue::from_computed_value)
                                    .collect())
            }
        }
    
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MaskOrigin(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MaskOrigin);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::MaskOrigin);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            // In the case of a vector property we want to pass
                            // down an iterator so that this can be computed
                            // without allocation
                            //
                            // However, computing requires a context, but the
                            // style struct being mutated is on the context. We
                            // temporarily remove it, mutate it, and then put it
                            // back. Vector longhands cannot touch their own
                            // style struct whilst computing, else this will
                            // panic.
                            let mut s =
                                context.builder.take_svg();
                            {
                                let iter = specified_value.compute_iter(context);
                                s.set_mask_origin(iter);
                            }
                            context.builder.put_svg(s);
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_mask_origin();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_mask_origin();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MaskOrigin)
            }
    }






    
    /// https://drafts.fxtf.org/css-masking/#propdef-mask-size
    pub mod mask_size {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
    use properties::longhands::background_size;
    pub use ::properties::longhands::background_size::SpecifiedValue;
    pub use ::properties::longhands::background_size::single_value as single_value;
    pub use ::properties::longhands::background_size::computed_value as computed_value;

    #[inline]
    pub fn get_initial_value() -> computed_value::T {
        background_size::get_initial_value()
    }

    pub fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                         -> Result<SpecifiedValue,ParseError<'i>> {
        background_size::parse(context, input)
    }

        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MaskSize(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MaskSize);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::MaskSize);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set_mask_size(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_mask_size();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_mask_size();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MaskSize)
            }
    }



    
    

    
        
    
    
    /// https://drafts.fxtf.org/css-masking/#propdef-mask-composite
    pub mod mask_composite {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        use std::fmt;
        use style_traits::{Separator, ToCss};

        pub mod single_value {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::ShorthandId;
            #[allow(unused_imports)]
            use selectors::parser::SelectorParseError;
            #[allow(unused_imports)]
            use style_traits::{ParseError, StyleParseError};
            #[allow(unused_imports)]
            use values::computed::{Context, ToComputedValue};
            #[allow(unused_imports)]
            use values::{computed, specified};
            #[allow(unused_imports)]
            use values::{Auto, Either, None_, Normal};
            
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "add" => add,
                    "subtract" => subtract,
                    "intersect" => intersect,
                    "exclude" => exclude,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::add
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::add
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::add => computed_value::T::add,
                        SpecifiedValue::subtract => computed_value::T::subtract,
                        SpecifiedValue::intersect => computed_value::T::intersect,
                        SpecifiedValue::exclude => computed_value::T::exclude,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::add => SpecifiedValue::add,
                        computed_value::T::subtract => SpecifiedValue::subtract,
                        computed_value::T::intersect => SpecifiedValue::intersect,
                        computed_value::T::exclude => SpecifiedValue::exclude,
                }
            }
        }
    
        
        }

        /// The definition of the computed value for mask-composite.
        pub mod computed_value {
            pub use super::single_value::computed_value as single_value;
            pub use self::single_value::T as SingleComputedValue;
            use smallvec::{IntoIter, SmallVec};
            use values::computed::ComputedVecIter;

            /// The computed value, effectively a list of single values.
            #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
            #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
            #[derive(Clone, Debug, PartialEq)]
            pub struct T(
                pub SmallVec<[single_value::T; 1]>,
            );


            pub type Iter<'a, 'cx, 'cx_a> = ComputedVecIter<'a, 'cx, 'cx_a, super::single_value::SpecifiedValue>;

            impl IntoIterator for T {
                type Item = single_value::T;
                type IntoIter = IntoIter<[single_value::T; 1]>;
                fn into_iter(self) -> Self::IntoIter {
                    self.0.into_iter()
                }
            }
        }

        impl ToCss for computed_value::T {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        warn!("Found empty value for property mask-composite");
                }
                for i in iter {
                    dest.write_str(::style_traits::Comma::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        /// The specified value of mask-composite.
        #[derive(Clone, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        pub struct SpecifiedValue(pub Vec<single_value::SpecifiedValue>);

        impl ToCss for SpecifiedValue {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        warn!("Found empty value for property mask-composite");
                }
                for i in iter {
                    dest.write_str(::style_traits::Comma::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        pub fn get_initial_value() -> computed_value::T {
                let mut v = SmallVec::new();
                v.push(single_value::get_initial_value());
                computed_value::T(v)
        }

        pub fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                -> Result<SpecifiedValue, ParseError<'i>> {
            use style_traits::Separator;


            ::style_traits::Comma::parse(input, |parser| {
                single_value::parse(context, parser)
            }).map(SpecifiedValue)
        }

        pub use self::single_value::SpecifiedValue as SingleSpecifiedValue;

        impl SpecifiedValue {
            pub fn compute_iter<'a, 'cx, 'cx_a>(&'a self, context: &'cx Context<'cx_a>)
                -> computed_value::Iter<'a, 'cx, 'cx_a> {
                computed_value::Iter::new(context, &self.0)
            }
        }

        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, context: &Context) -> computed_value::T {
                computed_value::T(self.compute_iter(context).collect())
            }

            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                SpecifiedValue(computed.0.iter()
                                    .map(ToComputedValue::from_computed_value)
                                    .collect())
            }
        }
    
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MaskComposite(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MaskComposite);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::MaskComposite);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            // In the case of a vector property we want to pass
                            // down an iterator so that this can be computed
                            // without allocation
                            //
                            // However, computing requires a context, but the
                            // style struct being mutated is on the context. We
                            // temporarily remove it, mutate it, and then put it
                            // back. Vector longhands cannot touch their own
                            // style struct whilst computing, else this will
                            // panic.
                            let mut s =
                                context.builder.take_svg();
                            {
                                let iter = specified_value.compute_iter(context);
                                s.set_mask_composite(iter);
                            }
                            context.builder.put_svg(s);
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_mask_composite();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_mask_composite();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MaskComposite)
            }
    }






    
        
    
    
    /// https://drafts.fxtf.org/css-masking/#propdef-mask-image
    pub mod mask_image {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        use std::fmt;
        use style_traits::{Separator, ToCss};

        pub mod single_value {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::ShorthandId;
            #[allow(unused_imports)]
            use selectors::parser::SelectorParseError;
            #[allow(unused_imports)]
            use style_traits::{ParseError, StyleParseError};
            #[allow(unused_imports)]
            use values::computed::{Context, ToComputedValue};
            #[allow(unused_imports)]
            use values::{computed, specified};
            #[allow(unused_imports)]
            use values::{Auto, Either, None_, Normal};
            
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::ImageLayer as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::ImageLayer as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { Either::First(None_) }
        #[inline] pub fn get_initial_specified_value() -> SpecifiedValue { Either::First(None_) }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::ImageLayer::parse(context, input)
        }
    
        
        }

        /// The definition of the computed value for mask-image.
        pub mod computed_value {
            pub use super::single_value::computed_value as single_value;
            pub use self::single_value::T as SingleComputedValue;
            use smallvec::{IntoIter, SmallVec};
            use values::computed::ComputedVecIter;

            /// The computed value, effectively a list of single values.
            #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
            #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
            #[derive(Clone, Debug, PartialEq)]
            pub struct T(
                pub SmallVec<[single_value::T; 1]>,
            );


            pub type Iter<'a, 'cx, 'cx_a> = ComputedVecIter<'a, 'cx, 'cx_a, super::single_value::SpecifiedValue>;

            impl IntoIterator for T {
                type Item = single_value::T;
                type IntoIter = IntoIter<[single_value::T; 1]>;
                fn into_iter(self) -> Self::IntoIter {
                    self.0.into_iter()
                }
            }
        }

        impl ToCss for computed_value::T {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        warn!("Found empty value for property mask-image");
                }
                for i in iter {
                    dest.write_str(::style_traits::Comma::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        /// The specified value of mask-image.
        #[derive(Clone, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        pub struct SpecifiedValue(pub Vec<single_value::SpecifiedValue>);

        impl ToCss for SpecifiedValue {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result
                where W: fmt::Write,
            {
                let mut iter = self.0.iter();
                if let Some(val) = iter.next() {
                    val.to_css(dest)?;
                } else {
                        warn!("Found empty value for property mask-image");
                }
                for i in iter {
                    dest.write_str(::style_traits::Comma::separator())?;
                    i.to_css(dest)?;
                }
                Ok(())
            }
        }

        pub fn get_initial_value() -> computed_value::T {
                let mut v = SmallVec::new();
                v.push(single_value::get_initial_value());
                computed_value::T(v)
        }

        pub fn parse<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                -> Result<SpecifiedValue, ParseError<'i>> {
            use style_traits::Separator;


            ::style_traits::Comma::parse(input, |parser| {
                single_value::parse(context, parser)
            }).map(SpecifiedValue)
        }

        pub use self::single_value::SpecifiedValue as SingleSpecifiedValue;

        impl SpecifiedValue {
            pub fn compute_iter<'a, 'cx, 'cx_a>(&'a self, context: &'cx Context<'cx_a>)
                -> computed_value::Iter<'a, 'cx, 'cx_a> {
                computed_value::Iter::new(context, &self.0)
            }
        }

        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, context: &Context) -> computed_value::T {
                computed_value::T(self.compute_iter(context).collect())
            }

            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                SpecifiedValue(computed.0.iter()
                                    .map(ToComputedValue::from_computed_value)
                                    .collect())
            }
        }
    
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MaskImage(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MaskImage);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::MaskImage);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            // In the case of a vector property we want to pass
                            // down an iterator so that this can be computed
                            // without allocation
                            //
                            // However, computing requires a context, but the
                            // style struct being mutated is on the context. We
                            // temporarily remove it, mutate it, and then put it
                            // back. Vector longhands cannot touch their own
                            // style struct whilst computing, else this will
                            // panic.
                            let mut s =
                                context.builder.take_svg();
                            {
                                let iter = specified_value.compute_iter(context);
                                s.set_mask_image(iter);
                            }
                            context.builder.put_svg(s);
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset_mask_image();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit_mask_image();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MaskImage)
            }
    }




    /* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */




// Non-standard properties that Gecko uses for XUL elements.



    
    

    
        
    
    /// Nonstandard (https://developer.mozilla.org/en-US/docs/Web/CSS/box-align)
    pub mod _moz_box_align {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "stretch" => stretch,
                    "start" => start,
                    "center" => center,
                    "baseline" => baseline,
                    "end" => end,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::stretch
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::stretch
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::stretch => computed_value::T::stretch,
                        SpecifiedValue::start => computed_value::T::start,
                        SpecifiedValue::center => computed_value::T::center,
                        SpecifiedValue::baseline => computed_value::T::baseline,
                        SpecifiedValue::end => computed_value::T::end,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::stretch => SpecifiedValue::stretch,
                        computed_value::T::start => SpecifiedValue::start,
                        computed_value::T::center => SpecifiedValue::center,
                        computed_value::T::baseline => SpecifiedValue::baseline,
                        computed_value::T::end => SpecifiedValue::end,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MozBoxAlign(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MozBoxAlign);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::MozBoxAlign);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set__moz_box_align(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset__moz_box_align();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit__moz_box_align();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MozBoxAlign)
            }
    }





    
    

    
        
    
    /// Nonstandard (https://developer.mozilla.org/en-US/docs/Web/CSS/box-direction)
    pub mod _moz_box_direction {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "normal" => normal,
                    "reverse" => reverse,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::normal
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::normal
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::normal => computed_value::T::normal,
                        SpecifiedValue::reverse => computed_value::T::reverse,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::normal => SpecifiedValue::normal,
                        computed_value::T::reverse => SpecifiedValue::reverse,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MozBoxDirection(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MozBoxDirection);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::MozBoxDirection);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set__moz_box_direction(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset__moz_box_direction();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit__moz_box_direction();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MozBoxDirection)
            }
    }





    
        
    
    /// Nonstandard (https://developer.mozilla.org/en-US/docs/Web/CSS/box-flex)
    pub mod _moz_box_flex {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::NonNegativeNumber as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::NonNegativeNumber as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { From::from(0.) }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::NonNegativeNumber::parse(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MozBoxFlex(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MozBoxFlex);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::MozBoxFlex);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set__moz_box_flex(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset__moz_box_flex();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit__moz_box_flex();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MozBoxFlex)
            }
    }




    
    

    
        
    
    /// Nonstandard (https://developer.mozilla.org/en-US/docs/Web/CSS/box-orient)
    pub mod _moz_box_orient {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            use style_traits::ToCss;
            define_css_keyword_enum! { SpecifiedValue:
                values {
                        "horizontal" => horizontal,
                        "vertical" => vertical,
                }
                aliases {
                        "inline-axis" => horizontal,
                        "block-axis" => vertical,
                }
            }
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "horizontal" => horizontal,
                    "vertical" => vertical,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::horizontal
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::horizontal
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::horizontal => computed_value::T::horizontal,
                        SpecifiedValue::vertical => computed_value::T::vertical,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::horizontal => SpecifiedValue::horizontal,
                        computed_value::T::vertical => SpecifiedValue::vertical,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MozBoxOrient(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MozBoxOrient);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::MozBoxOrient);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set__moz_box_orient(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset__moz_box_orient();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit__moz_box_orient();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MozBoxOrient)
            }
    }





    
    

    
        
    
    /// Nonstandard (https://developer.mozilla.org/en-US/docs/Web/CSS/box-pack)
    pub mod _moz_box_pack {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "start" => start,
                    "center" => center,
                    "end" => end,
                    "justify" => justify,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::start
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::start
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::start => computed_value::T::start,
                        SpecifiedValue::center => computed_value::T::center,
                        SpecifiedValue::end => computed_value::T::end,
                        SpecifiedValue::justify => computed_value::T::justify,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::start => SpecifiedValue::start,
                        computed_value::T::center => SpecifiedValue::center,
                        computed_value::T::end => SpecifiedValue::end,
                        computed_value::T::justify => SpecifiedValue::justify,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MozBoxPack(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MozBoxPack);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::MozBoxPack);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set__moz_box_pack(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset__moz_box_pack();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit__moz_box_pack();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MozBoxPack)
            }
    }





    
    

    
        
    
    /// Nonstandard (https://developer.mozilla.org/en-US/docs/Web/CSS/-moz-stack-sizing)
    pub mod _moz_stack_sizing {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
            pub use self::computed_value::T as SpecifiedValue;
        pub mod computed_value {
            use style_traits::ToCss;
            define_css_keyword_enum! { T:
                    "stretch-to-fit" => stretch_to_fit,
                    "ignore" => ignore,
                    "ignore-horizontal" => ignore_horizontal,
                    "ignore-vertical" => ignore_vertical,
            }
        }
        #[inline]
        pub fn get_initial_value() -> computed_value::T {
            computed_value::T::stretch_to_fit
        }
        #[inline]
        pub fn get_initial_specified_value() -> SpecifiedValue {
            SpecifiedValue::stretch_to_fit
        }
        #[inline]
        pub fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            SpecifiedValue::parse(input)
        }
        impl Parse for SpecifiedValue {
            #[inline]
            fn parse<'i, 't>(_context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
                SpecifiedValue::parse(input)
            }
        }

    
            
        impl ToComputedValue for SpecifiedValue {
            type ComputedValue = computed_value::T;

            #[inline]
            fn to_computed_value(&self, _context: &Context) -> computed_value::T {
                match *self {
                        SpecifiedValue::stretch_to_fit => computed_value::T::stretch_to_fit,
                        SpecifiedValue::ignore => computed_value::T::ignore,
                        SpecifiedValue::ignore_horizontal => computed_value::T::ignore_horizontal,
                        SpecifiedValue::ignore_vertical => computed_value::T::ignore_vertical,
                }
            }
            #[inline]
            fn from_computed_value(computed: &computed_value::T) -> Self {
                match *computed {
                        computed_value::T::stretch_to_fit => SpecifiedValue::stretch_to_fit,
                        computed_value::T::ignore => SpecifiedValue::ignore,
                        computed_value::T::ignore_horizontal => SpecifiedValue::ignore_horizontal,
                        computed_value::T::ignore_vertical => SpecifiedValue::ignore_vertical,
                }
            }
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MozStackSizing(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MozStackSizing);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::MozStackSizing);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set__moz_stack_sizing(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset__moz_stack_sizing();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit__moz_stack_sizing();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MozStackSizing)
            }
    }





    
        
    
    /// Nonstandard (https://developer.mozilla.org/en-US/docs/Web/CSS/-moz-box-ordinal-group)
    pub mod _moz_box_ordinal_group {
            #[allow(unused_imports)]
            use cssparser::{Parser, BasicParseError, Token};
            #[allow(unused_imports)]
            use parser::{Parse, ParserContext};
            #[allow(unused_imports)]
            use properties::{UnparsedValue, ShorthandId};
        #[allow(unused_imports)]
        use values::{Auto, Either, None_, Normal};
        #[allow(unused_imports)]
        use error_reporting::ParseErrorReporter;
        #[allow(unused_imports)]
        use properties::longhands;
        #[allow(unused_imports)]
        use properties::{DeclaredValue, LonghandId, LonghandIdSet};
        #[allow(unused_imports)]
        use properties::{CSSWideKeyword, ComputedValues, PropertyDeclaration};
        #[allow(unused_imports)]
        use properties::style_structs;
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use servo_arc::Arc;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use values::computed::{Context, ToComputedValue};
        #[allow(unused_imports)]
        use values::{computed, generics, specified};
        #[allow(unused_imports)]
        use Atom;
        
            
        #[allow(unused_imports)]
        use app_units::Au;
        #[allow(unused_imports)]
        use cssparser::{Color as CSSParserColor, RGBA};
        #[allow(unused_imports)]
        use values::specified::AllowQuirks;
        #[allow(unused_imports)]
        use smallvec::SmallVec;
        pub use values::specified::Integer as SpecifiedValue;
        pub mod computed_value {
            pub use values::computed::Integer as T;
        }
        #[inline] pub fn get_initial_value() -> computed_value::T { 0 }
        #[allow(unused_variables)]
        #[inline]
        pub fn parse<'i, 't>(context: &ParserContext,
                             input: &mut Parser<'i, 't>)
                             -> Result<SpecifiedValue, ParseError<'i>> {
            specified::Integer::parse_non_negative(context, input)
        }
    
        
        #[allow(unused_variables)]
        pub fn cascade_property(
            declaration: &PropertyDeclaration,
            context: &mut computed::Context,
        ) {
            let value = match *declaration {
                PropertyDeclaration::MozBoxOrdinalGroup(ref value) => {
                    DeclaredValue::Value(value)
                },
                PropertyDeclaration::CSSWideKeyword(id, value) => {
                    debug_assert!(id == LonghandId::MozBoxOrdinalGroup);
                    DeclaredValue::CSSWideKeyword(value)
                },
                PropertyDeclaration::WithVariables(..) => {
                    panic!("variables should already have been substituted")
                }
                _ => panic!("entered the wrong cascade_property() implementation"),
            };

            context.for_non_inherited_property =
                    Some(LonghandId::MozBoxOrdinalGroup);

                match value {
                    DeclaredValue::Value(specified_value) => {
                            let computed = specified_value.to_computed_value(context);
                                context.builder.set__moz_box_ordinal_group(computed)
                    }
                    DeclaredValue::WithVariables(_) => unreachable!(),
                    DeclaredValue::CSSWideKeyword(keyword) => match keyword {
                        CSSWideKeyword::Unset |
                        CSSWideKeyword::Initial => {
                                context.builder.reset__moz_box_ordinal_group();
                        },
                        CSSWideKeyword::Inherit => {
                                context.rule_cache_conditions.borrow_mut().set_uncacheable();
                                context.builder.inherit__moz_box_ordinal_group();
                        }
                    }
                }

        }
            pub fn parse_specified<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<SpecifiedValue, ParseError<'i>> {
                        parse(context, input)
            }
            pub fn parse_declared<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                          -> Result<PropertyDeclaration, ParseError<'i>> {
                parse_specified(context, input).map(PropertyDeclaration::MozBoxOrdinalGroup)
            }
    }



}

macro_rules! unwrap_or_initial {
    ($prop: ident) => (unwrap_or_initial!($prop, $prop));
    ($prop: ident, $expr: expr) =>
        ($expr.unwrap_or_else(|| $prop::get_initial_specified_value()));
}

/// A module with code for all the shorthand css properties, and a few
/// serialization helpers.
#[allow(missing_docs)]
pub mod shorthands {
    use cssparser::Parser;
    use parser::{Parse, ParserContext};
    use style_traits::{ParseError, StyleParseError};
    use values::specified;

    /* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

use style_traits::ToCss;
use values::specified::{BorderStyle, Color};
use std::fmt;

fn serialize_directional_border<W, I,>(dest: &mut W,
                                       width: &I,
                                       style: &BorderStyle,
                                       color: &Color)
    -> fmt::Result where W: fmt::Write, I: ToCss {
    width.to_css(dest)?;
    dest.write_str(" ")?;
    style.to_css(dest)?;
    if *color != Color::CurrentColor {
        dest.write_str(" ")?;
        color.to_css(dest)?;
    }
    Ok(())
}

    /* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */



// TODO: other background-* properties


    /// https://drafts.csswg.org/css-backgrounds/#the-background
    pub mod background {
        use cssparser::Parser;
        use parser::ParserContext;
        use properties::{PropertyDeclaration, SourcePropertyDeclaration, MaybeBoxed, longhands};
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use std::fmt;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use style_traits::ToCss;

        pub struct Longhands {
                pub background_color:
                    longhands::background_color::SpecifiedValue
                    ,
                pub background_position_x:
                    longhands::background_position_x::SpecifiedValue
                    ,
                pub background_position_y:
                    longhands::background_position_y::SpecifiedValue
                    ,
                pub background_repeat:
                    longhands::background_repeat::SpecifiedValue
                    ,
                pub background_attachment:
                    longhands::background_attachment::SpecifiedValue
                    ,
                pub background_image:
                    longhands::background_image::SpecifiedValue
                    ,
                pub background_size:
                    longhands::background_size::SpecifiedValue
                    ,
                pub background_origin:
                    longhands::background_origin::SpecifiedValue
                    ,
                pub background_clip:
                    longhands::background_clip::SpecifiedValue
                    ,
        }

        /// Represents a serializable set of all of the longhand properties that
        /// correspond to a shorthand.
        pub struct LonghandsToSerialize<'a> {
                pub background_color:
                    &'a longhands::background_color::SpecifiedValue,
                pub background_position_x:
                    &'a longhands::background_position_x::SpecifiedValue,
                pub background_position_y:
                    &'a longhands::background_position_y::SpecifiedValue,
                pub background_repeat:
                    &'a longhands::background_repeat::SpecifiedValue,
                pub background_attachment:
                    &'a longhands::background_attachment::SpecifiedValue,
                pub background_image:
                    &'a longhands::background_image::SpecifiedValue,
                pub background_size:
                    &'a longhands::background_size::SpecifiedValue,
                pub background_origin:
                    &'a longhands::background_origin::SpecifiedValue,
                pub background_clip:
                    &'a longhands::background_clip::SpecifiedValue,
        }

        impl<'a> LonghandsToSerialize<'a> {
            /// Tries to get a serializable set of longhands given a set of
            /// property declarations.
            pub fn from_iter<I>(iter: I) -> Result<Self, ()>
                where I: Iterator<Item=&'a PropertyDeclaration>,
            {
                // Define all of the expected variables that correspond to the shorthand
                    let mut background_color = None;
                    let mut background_position_x = None;
                    let mut background_position_y = None;
                    let mut background_repeat = None;
                    let mut background_attachment = None;
                    let mut background_image = None;
                    let mut background_size = None;
                    let mut background_origin = None;
                    let mut background_clip = None;

                // Attempt to assign the incoming declarations to the expected variables
                for longhand in iter {
                    match *longhand {
                            PropertyDeclaration::BackgroundColor(ref value) => {
                                background_color = Some(value)
                            },
                            PropertyDeclaration::BackgroundPositionX(ref value) => {
                                background_position_x = Some(value)
                            },
                            PropertyDeclaration::BackgroundPositionY(ref value) => {
                                background_position_y = Some(value)
                            },
                            PropertyDeclaration::BackgroundRepeat(ref value) => {
                                background_repeat = Some(value)
                            },
                            PropertyDeclaration::BackgroundAttachment(ref value) => {
                                background_attachment = Some(value)
                            },
                            PropertyDeclaration::BackgroundImage(ref value) => {
                                background_image = Some(value)
                            },
                            PropertyDeclaration::BackgroundSize(ref value) => {
                                background_size = Some(value)
                            },
                            PropertyDeclaration::BackgroundOrigin(ref value) => {
                                background_origin = Some(value)
                            },
                            PropertyDeclaration::BackgroundClip(ref value) => {
                                background_clip = Some(value)
                            },
                        _ => {}
                    };
                }

                // If any of the expected variables are missing, return an error
                match (
                        background_color,
                        background_position_x,
                        background_position_y,
                        background_repeat,
                        background_attachment,
                        background_image,
                        background_size,
                        background_origin,
                        background_clip,
                ) {

                    (
                        Some(background_color),
                        Some(background_position_x),
                        Some(background_position_y),
                        Some(background_repeat),
                        Some(background_attachment),
                        Some(background_image),
                        Some(background_size),
                        Some(background_origin),
                        Some(background_clip),
                    ) =>
                    Ok(LonghandsToSerialize {
                            background_color: background_color,
                            background_position_x: background_position_x,
                            background_position_y: background_position_y,
                            background_repeat: background_repeat,
                            background_attachment: background_attachment,
                            background_image: background_image,
                            background_size: background_size,
                            background_origin: background_origin,
                            background_clip: background_clip,
                    }),
                    _ => Err(())
                }
            }
        }

        /// Parse the given shorthand and fill the result into the
        /// `declarations` vector.
        pub fn parse_into<'i, 't>(declarations: &mut SourcePropertyDeclaration,
                                  context: &ParserContext, input: &mut Parser<'i, 't>)
                                  -> Result<(), ParseError<'i>> {
            input.parse_entirely(|input| parse_value(context, input)).map(|longhands| {
                    declarations.push(PropertyDeclaration::BackgroundColor(
                        longhands.background_color
                    ));
                    declarations.push(PropertyDeclaration::BackgroundPositionX(
                        longhands.background_position_x
                    ));
                    declarations.push(PropertyDeclaration::BackgroundPositionY(
                        longhands.background_position_y
                    ));
                    declarations.push(PropertyDeclaration::BackgroundRepeat(
                        longhands.background_repeat
                    ));
                    declarations.push(PropertyDeclaration::BackgroundAttachment(
                        longhands.background_attachment
                    ));
                    declarations.push(PropertyDeclaration::BackgroundImage(
                        longhands.background_image
                    ));
                    declarations.push(PropertyDeclaration::BackgroundSize(
                        longhands.background_size
                    ));
                    declarations.push(PropertyDeclaration::BackgroundOrigin(
                        longhands.background_origin
                    ));
                    declarations.push(PropertyDeclaration::BackgroundClip(
                        longhands.background_clip
                    ));
            })
        }

        
    use properties::longhands::{background_position_x, background_position_y, background_repeat};
    use properties::longhands::{background_attachment, background_image, background_size, background_origin};
    use properties::longhands::background_clip;
    use properties::longhands::background_clip::single_value::computed_value::T as Clip;
    use properties::longhands::background_origin::single_value::computed_value::T as Origin;
    use values::specified::{Color, Position, PositionComponent};
    use parser::Parse;

    impl From<background_origin::single_value::SpecifiedValue> for background_clip::single_value::SpecifiedValue {
        fn from(origin: background_origin::single_value::SpecifiedValue) ->
            background_clip::single_value::SpecifiedValue {
            match origin {
                background_origin::single_value::SpecifiedValue::content_box =>
                    background_clip::single_value::SpecifiedValue::content_box,
                background_origin::single_value::SpecifiedValue::padding_box =>
                    background_clip::single_value::SpecifiedValue::padding_box,
                background_origin::single_value::SpecifiedValue::border_box =>
                    background_clip::single_value::SpecifiedValue::border_box,
            }
        }
    }

    pub fn parse_value<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                               -> Result<Longhands, ParseError<'i>> {
        let mut background_color = None;

            // Vec grows from 0 to 4 by default on first push().  So allocate
            // with capacity 1, so in the common case of only one item we don't
            // way overallocate.  Note that we always push at least one item if
            // parsing succeeds.
            let mut background_image = background_image::SpecifiedValue(Vec::with_capacity(1));
            // Vec grows from 0 to 4 by default on first push().  So allocate
            // with capacity 1, so in the common case of only one item we don't
            // way overallocate.  Note that we always push at least one item if
            // parsing succeeds.
            let mut background_position_x = background_position_x::SpecifiedValue(Vec::with_capacity(1));
            // Vec grows from 0 to 4 by default on first push().  So allocate
            // with capacity 1, so in the common case of only one item we don't
            // way overallocate.  Note that we always push at least one item if
            // parsing succeeds.
            let mut background_position_y = background_position_y::SpecifiedValue(Vec::with_capacity(1));
            // Vec grows from 0 to 4 by default on first push().  So allocate
            // with capacity 1, so in the common case of only one item we don't
            // way overallocate.  Note that we always push at least one item if
            // parsing succeeds.
            let mut background_repeat = background_repeat::SpecifiedValue(Vec::with_capacity(1));
            // Vec grows from 0 to 4 by default on first push().  So allocate
            // with capacity 1, so in the common case of only one item we don't
            // way overallocate.  Note that we always push at least one item if
            // parsing succeeds.
            let mut background_size = background_size::SpecifiedValue(Vec::with_capacity(1));
            // Vec grows from 0 to 4 by default on first push().  So allocate
            // with capacity 1, so in the common case of only one item we don't
            // way overallocate.  Note that we always push at least one item if
            // parsing succeeds.
            let mut background_attachment = background_attachment::SpecifiedValue(Vec::with_capacity(1));
            // Vec grows from 0 to 4 by default on first push().  So allocate
            // with capacity 1, so in the common case of only one item we don't
            // way overallocate.  Note that we always push at least one item if
            // parsing succeeds.
            let mut background_origin = background_origin::SpecifiedValue(Vec::with_capacity(1));
            // Vec grows from 0 to 4 by default on first push().  So allocate
            // with capacity 1, so in the common case of only one item we don't
            // way overallocate.  Note that we always push at least one item if
            // parsing succeeds.
            let mut background_clip = background_clip::SpecifiedValue(Vec::with_capacity(1));
        input.parse_comma_separated(|input| {
            // background-color can only be in the last element, so if it
            // is parsed anywhere before, the value is invalid.
            if background_color.is_some() {
                return Err(StyleParseError::UnspecifiedError.into());
            }

                let mut image = None;
                let mut position = None;
                let mut repeat = None;
                let mut size = None;
                let mut attachment = None;
                let mut origin = None;
                let mut clip = None;
            loop {
                if background_color.is_none() {
                    if let Ok(value) = input.try(|i| Color::parse(context, i)) {
                        background_color = Some(value);
                        continue
                    }
                }
                if position.is_none() {
                    if let Ok(value) = input.try(|input| Position::parse(context, input)) {
                        position = Some(value);

                        // Parse background size, if applicable.
                        size = input.try(|input| {
                            input.expect_delim('/')?;
                            background_size::single_value::parse(context, input)
                        }).ok();

                        continue
                    }
                }
                    if image.is_none() {
                        if let Ok(value) = input.try(|input| background_image::single_value
                                                                               ::parse(context, input)) {
                            image = Some(value);
                            continue
                        }
                    }
                    if repeat.is_none() {
                        if let Ok(value) = input.try(|input| background_repeat::single_value
                                                                               ::parse(context, input)) {
                            repeat = Some(value);
                            continue
                        }
                    }
                    if attachment.is_none() {
                        if let Ok(value) = input.try(|input| background_attachment::single_value
                                                                               ::parse(context, input)) {
                            attachment = Some(value);
                            continue
                        }
                    }
                    if origin.is_none() {
                        if let Ok(value) = input.try(|input| background_origin::single_value
                                                                               ::parse(context, input)) {
                            origin = Some(value);
                            continue
                        }
                    }
                    if clip.is_none() {
                        if let Ok(value) = input.try(|input| background_clip::single_value
                                                                               ::parse(context, input)) {
                            clip = Some(value);
                            continue
                        }
                    }
                break
            }
            if clip.is_none() {
                if let Some(origin) = origin {
                    clip = Some(background_clip::single_value::SpecifiedValue::from(origin));
                }
            }
            let mut any = false;
                any = any || image.is_some();
                any = any || position.is_some();
                any = any || repeat.is_some();
                any = any || size.is_some();
                any = any || attachment.is_some();
                any = any || origin.is_some();
                any = any || clip.is_some();
            any = any || background_color.is_some();
            if any {
                if let Some(position) = position {
                    background_position_x.0.push(position.horizontal);
                    background_position_y.0.push(position.vertical);
                } else {
                    background_position_x.0.push(PositionComponent::zero());
                    background_position_y.0.push(PositionComponent::zero());
                }
                    if let Some(bg_image) = image {
                        background_image.0.push(bg_image);
                    } else {
                        background_image.0.push(background_image::single_value
                                                                    ::get_initial_specified_value());
                    }
                    if let Some(bg_repeat) = repeat {
                        background_repeat.0.push(bg_repeat);
                    } else {
                        background_repeat.0.push(background_repeat::single_value
                                                                    ::get_initial_specified_value());
                    }
                    if let Some(bg_size) = size {
                        background_size.0.push(bg_size);
                    } else {
                        background_size.0.push(background_size::single_value
                                                                    ::get_initial_specified_value());
                    }
                    if let Some(bg_attachment) = attachment {
                        background_attachment.0.push(bg_attachment);
                    } else {
                        background_attachment.0.push(background_attachment::single_value
                                                                    ::get_initial_specified_value());
                    }
                    if let Some(bg_origin) = origin {
                        background_origin.0.push(bg_origin);
                    } else {
                        background_origin.0.push(background_origin::single_value
                                                                    ::get_initial_specified_value());
                    }
                    if let Some(bg_clip) = clip {
                        background_clip.0.push(bg_clip);
                    } else {
                        background_clip.0.push(background_clip::single_value
                                                                    ::get_initial_specified_value());
                    }
                Ok(())
            } else {
                Err(StyleParseError::UnspecifiedError.into())
            }
        })?;

        Ok(expanded! {
             background_color: background_color.unwrap_or(Color::transparent()),
             background_image: background_image,
             background_position_x: background_position_x,
             background_position_y: background_position_y,
             background_repeat: background_repeat,
             background_attachment: background_attachment,
             background_size: background_size,
             background_origin: background_origin,
             background_clip: background_clip,
         })
    }

    impl<'a> ToCss for LonghandsToSerialize<'a>  {
        fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
            let len = self.background_image.0.len();
            // There should be at least one declared value
            if len == 0 {
                return Ok(());
            }

            // If a value list length is differs then we don't do a shorthand serialization.
            // The exceptions to this is color which appears once only and is serialized
            // with the last item.
                if len != self.background_image.0.len() {
                    return Ok(());
                }
                if len != self.background_position_x.0.len() {
                    return Ok(());
                }
                if len != self.background_position_y.0.len() {
                    return Ok(());
                }
                if len != self.background_size.0.len() {
                    return Ok(());
                }
                if len != self.background_repeat.0.len() {
                    return Ok(());
                }
                if len != self.background_origin.0.len() {
                    return Ok(());
                }
                if len != self.background_clip.0.len() {
                    return Ok(());
                }
                if len != self.background_attachment.0.len() {
                    return Ok(());
                }

            for i in 0..len {
                    let image = &self.background_image.0[i];
                    let position_x = &self.background_position_x.0[i];
                    let position_y = &self.background_position_y.0[i];
                    let repeat = &self.background_repeat.0[i];
                    let size = &self.background_size.0[i];
                    let attachment = &self.background_attachment.0[i];
                    let origin = &self.background_origin.0[i];
                    let clip = &self.background_clip.0[i];

                if i != 0 {
                    dest.write_str(", ")?;
                }

                if i == len - 1 {
                    self.background_color.to_css(dest)?;
                    dest.write_str(" ")?;
                }

                image.to_css(dest)?;
                    dest.write_str(" ")?;
                    repeat.to_css(dest)?;
                    dest.write_str(" ")?;
                    attachment.to_css(dest)?;

                dest.write_str(" ")?;
                Position {
                    horizontal: position_x.clone(),
                    vertical: position_y.clone()
                }.to_css(dest)?;

                if *size != background_size::single_value::get_initial_specified_value() {
                    dest.write_str(" / ")?;
                    size.to_css(dest)?;
                }

                if *origin != Origin::padding_box || *clip != Clip::border_box {
                    dest.write_str(" ")?;
                    origin.to_css(dest)?;
                    if *clip != From::from(*origin) {
                        dest.write_str(" ")?;
                        clip.to_css(dest)?;
                    }
                }
            }

            Ok(())
        }
    }

    }




    /// https://drafts.csswg.org/css-backgrounds-4/#the-background-position
    pub mod background_position {
        use cssparser::Parser;
        use parser::ParserContext;
        use properties::{PropertyDeclaration, SourcePropertyDeclaration, MaybeBoxed, longhands};
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use std::fmt;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use style_traits::ToCss;

        pub struct Longhands {
                pub background_position_x:
                    longhands::background_position_x::SpecifiedValue
                    ,
                pub background_position_y:
                    longhands::background_position_y::SpecifiedValue
                    ,
        }

        /// Represents a serializable set of all of the longhand properties that
        /// correspond to a shorthand.
        pub struct LonghandsToSerialize<'a> {
                pub background_position_x:
                    &'a longhands::background_position_x::SpecifiedValue,
                pub background_position_y:
                    &'a longhands::background_position_y::SpecifiedValue,
        }

        impl<'a> LonghandsToSerialize<'a> {
            /// Tries to get a serializable set of longhands given a set of
            /// property declarations.
            pub fn from_iter<I>(iter: I) -> Result<Self, ()>
                where I: Iterator<Item=&'a PropertyDeclaration>,
            {
                // Define all of the expected variables that correspond to the shorthand
                    let mut background_position_x = None;
                    let mut background_position_y = None;

                // Attempt to assign the incoming declarations to the expected variables
                for longhand in iter {
                    match *longhand {
                            PropertyDeclaration::BackgroundPositionX(ref value) => {
                                background_position_x = Some(value)
                            },
                            PropertyDeclaration::BackgroundPositionY(ref value) => {
                                background_position_y = Some(value)
                            },
                        _ => {}
                    };
                }

                // If any of the expected variables are missing, return an error
                match (
                        background_position_x,
                        background_position_y,
                ) {

                    (
                        Some(background_position_x),
                        Some(background_position_y),
                    ) =>
                    Ok(LonghandsToSerialize {
                            background_position_x: background_position_x,
                            background_position_y: background_position_y,
                    }),
                    _ => Err(())
                }
            }
        }

        /// Parse the given shorthand and fill the result into the
        /// `declarations` vector.
        pub fn parse_into<'i, 't>(declarations: &mut SourcePropertyDeclaration,
                                  context: &ParserContext, input: &mut Parser<'i, 't>)
                                  -> Result<(), ParseError<'i>> {
            input.parse_entirely(|input| parse_value(context, input)).map(|longhands| {
                    declarations.push(PropertyDeclaration::BackgroundPositionX(
                        longhands.background_position_x
                    ));
                    declarations.push(PropertyDeclaration::BackgroundPositionY(
                        longhands.background_position_y
                    ));
            })
        }

        
    use properties::longhands::{background_position_x, background_position_y};
    use values::specified::AllowQuirks;
    use values::specified::position::Position;

    pub fn parse_value<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                               -> Result<Longhands, ParseError<'i>> {
        // Vec grows from 0 to 4 by default on first push().  So allocate with
        // capacity 1, so in the common case of only one item we don't way
        // overallocate.  Note that we always push at least one item if parsing
        // succeeds.
        let mut position_x = background_position_x::SpecifiedValue(Vec::with_capacity(1));
        let mut position_y = background_position_y::SpecifiedValue(Vec::with_capacity(1));
        let mut any = false;

        input.parse_comma_separated(|input| {
            let value = Position::parse_quirky(context, input, AllowQuirks::Yes)?;
            position_x.0.push(value.horizontal);
            position_y.0.push(value.vertical);
            any = true;
            Ok(())
        })?;
        if !any {
            return Err(StyleParseError::UnspecifiedError.into());
        }

        Ok(expanded! {
            background_position_x: position_x,
            background_position_y: position_y,
        })
    }

    impl<'a> ToCss for LonghandsToSerialize<'a>  {
        fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
            let len = self.background_position_x.0.len();
            if len == 0 || len != self.background_position_y.0.len() {
                return Ok(());
            }
            for i in 0..len {
                Position {
                    horizontal: self.background_position_x.0[i].clone(),
                    vertical: self.background_position_y.0[i].clone()
                }.to_css(dest)?;

                if i < len - 1 {
                    dest.write_str(", ")?;
                }
            }
            Ok(())
        }
    }

    }


    /* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */





    
    

    /// https://drafts.csswg.org/css-backgrounds/#border-color
    pub mod border_color {
        use cssparser::Parser;
        use parser::ParserContext;
        use properties::{PropertyDeclaration, SourcePropertyDeclaration, MaybeBoxed, longhands};
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use std::fmt;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use style_traits::ToCss;

        pub struct Longhands {
                pub border_top_color:
                    longhands::border_top_color::SpecifiedValue
                    ,
                pub border_right_color:
                    longhands::border_right_color::SpecifiedValue
                    ,
                pub border_bottom_color:
                    longhands::border_bottom_color::SpecifiedValue
                    ,
                pub border_left_color:
                    longhands::border_left_color::SpecifiedValue
                    ,
        }

        /// Represents a serializable set of all of the longhand properties that
        /// correspond to a shorthand.
        pub struct LonghandsToSerialize<'a> {
                pub border_top_color:
                    &'a longhands::border_top_color::SpecifiedValue,
                pub border_right_color:
                    &'a longhands::border_right_color::SpecifiedValue,
                pub border_bottom_color:
                    &'a longhands::border_bottom_color::SpecifiedValue,
                pub border_left_color:
                    &'a longhands::border_left_color::SpecifiedValue,
        }

        impl<'a> LonghandsToSerialize<'a> {
            /// Tries to get a serializable set of longhands given a set of
            /// property declarations.
            pub fn from_iter<I>(iter: I) -> Result<Self, ()>
                where I: Iterator<Item=&'a PropertyDeclaration>,
            {
                // Define all of the expected variables that correspond to the shorthand
                    let mut border_top_color = None;
                    let mut border_right_color = None;
                    let mut border_bottom_color = None;
                    let mut border_left_color = None;

                // Attempt to assign the incoming declarations to the expected variables
                for longhand in iter {
                    match *longhand {
                            PropertyDeclaration::BorderTopColor(ref value) => {
                                border_top_color = Some(value)
                            },
                            PropertyDeclaration::BorderRightColor(ref value) => {
                                border_right_color = Some(value)
                            },
                            PropertyDeclaration::BorderBottomColor(ref value) => {
                                border_bottom_color = Some(value)
                            },
                            PropertyDeclaration::BorderLeftColor(ref value) => {
                                border_left_color = Some(value)
                            },
                        _ => {}
                    };
                }

                // If any of the expected variables are missing, return an error
                match (
                        border_top_color,
                        border_right_color,
                        border_bottom_color,
                        border_left_color,
                ) {

                    (
                        Some(border_top_color),
                        Some(border_right_color),
                        Some(border_bottom_color),
                        Some(border_left_color),
                    ) =>
                    Ok(LonghandsToSerialize {
                            border_top_color: border_top_color,
                            border_right_color: border_right_color,
                            border_bottom_color: border_bottom_color,
                            border_left_color: border_left_color,
                    }),
                    _ => Err(())
                }
            }
        }

        /// Parse the given shorthand and fill the result into the
        /// `declarations` vector.
        pub fn parse_into<'i, 't>(declarations: &mut SourcePropertyDeclaration,
                                  context: &ParserContext, input: &mut Parser<'i, 't>)
                                  -> Result<(), ParseError<'i>> {
            input.parse_entirely(|input| parse_value(context, input)).map(|longhands| {
                    declarations.push(PropertyDeclaration::BorderTopColor(
                        longhands.border_top_color
                    ));
                    declarations.push(PropertyDeclaration::BorderRightColor(
                        longhands.border_right_color
                    ));
                    declarations.push(PropertyDeclaration::BorderBottomColor(
                        longhands.border_bottom_color
                    ));
                    declarations.push(PropertyDeclaration::BorderLeftColor(
                        longhands.border_left_color
                    ));
            })
        }

        
        #[allow(unused_imports)]
        use parser::Parse;
        use values::generics::rect::Rect;
        use values::specified;

        pub fn parse_value<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<Longhands, ParseError<'i>> {
            let rect = Rect::parse_with(context, input, |_c, i| {
                specified::Color::parse_quirky(_c, i, specified::AllowQuirks::Yes)
            })?;
            Ok(expanded! {
                    border_top_color: rect.0,
                    border_right_color: rect.1,
                    border_bottom_color: rect.2,
                    border_left_color: rect.3,
            })
        }

        impl<'a> ToCss for LonghandsToSerialize<'a> {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
                let rect = Rect::new(
                    &self.border_top_color,
                    &self.border_right_color,
                    &self.border_bottom_color,
                    &self.border_left_color,
                );
                rect.to_css(dest)
            }
        }
    
    }




    
    

    /// https://drafts.csswg.org/css-backgrounds/#border-style
    pub mod border_style {
        use cssparser::Parser;
        use parser::ParserContext;
        use properties::{PropertyDeclaration, SourcePropertyDeclaration, MaybeBoxed, longhands};
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use std::fmt;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use style_traits::ToCss;

        pub struct Longhands {
                pub border_top_style:
                    longhands::border_top_style::SpecifiedValue
                    ,
                pub border_right_style:
                    longhands::border_right_style::SpecifiedValue
                    ,
                pub border_bottom_style:
                    longhands::border_bottom_style::SpecifiedValue
                    ,
                pub border_left_style:
                    longhands::border_left_style::SpecifiedValue
                    ,
        }

        /// Represents a serializable set of all of the longhand properties that
        /// correspond to a shorthand.
        pub struct LonghandsToSerialize<'a> {
                pub border_top_style:
                    &'a longhands::border_top_style::SpecifiedValue,
                pub border_right_style:
                    &'a longhands::border_right_style::SpecifiedValue,
                pub border_bottom_style:
                    &'a longhands::border_bottom_style::SpecifiedValue,
                pub border_left_style:
                    &'a longhands::border_left_style::SpecifiedValue,
        }

        impl<'a> LonghandsToSerialize<'a> {
            /// Tries to get a serializable set of longhands given a set of
            /// property declarations.
            pub fn from_iter<I>(iter: I) -> Result<Self, ()>
                where I: Iterator<Item=&'a PropertyDeclaration>,
            {
                // Define all of the expected variables that correspond to the shorthand
                    let mut border_top_style = None;
                    let mut border_right_style = None;
                    let mut border_bottom_style = None;
                    let mut border_left_style = None;

                // Attempt to assign the incoming declarations to the expected variables
                for longhand in iter {
                    match *longhand {
                            PropertyDeclaration::BorderTopStyle(ref value) => {
                                border_top_style = Some(value)
                            },
                            PropertyDeclaration::BorderRightStyle(ref value) => {
                                border_right_style = Some(value)
                            },
                            PropertyDeclaration::BorderBottomStyle(ref value) => {
                                border_bottom_style = Some(value)
                            },
                            PropertyDeclaration::BorderLeftStyle(ref value) => {
                                border_left_style = Some(value)
                            },
                        _ => {}
                    };
                }

                // If any of the expected variables are missing, return an error
                match (
                        border_top_style,
                        border_right_style,
                        border_bottom_style,
                        border_left_style,
                ) {

                    (
                        Some(border_top_style),
                        Some(border_right_style),
                        Some(border_bottom_style),
                        Some(border_left_style),
                    ) =>
                    Ok(LonghandsToSerialize {
                            border_top_style: border_top_style,
                            border_right_style: border_right_style,
                            border_bottom_style: border_bottom_style,
                            border_left_style: border_left_style,
                    }),
                    _ => Err(())
                }
            }
        }

        /// Parse the given shorthand and fill the result into the
        /// `declarations` vector.
        pub fn parse_into<'i, 't>(declarations: &mut SourcePropertyDeclaration,
                                  context: &ParserContext, input: &mut Parser<'i, 't>)
                                  -> Result<(), ParseError<'i>> {
            input.parse_entirely(|input| parse_value(context, input)).map(|longhands| {
                    declarations.push(PropertyDeclaration::BorderTopStyle(
                        longhands.border_top_style
                    ));
                    declarations.push(PropertyDeclaration::BorderRightStyle(
                        longhands.border_right_style
                    ));
                    declarations.push(PropertyDeclaration::BorderBottomStyle(
                        longhands.border_bottom_style
                    ));
                    declarations.push(PropertyDeclaration::BorderLeftStyle(
                        longhands.border_left_style
                    ));
            })
        }

        
        #[allow(unused_imports)]
        use parser::Parse;
        use values::generics::rect::Rect;
        use values::specified;

        pub fn parse_value<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<Longhands, ParseError<'i>> {
            let rect = Rect::parse_with(context, input, |_c, i| {
                specified::BorderStyle::parse(_c, i)
            })?;
            Ok(expanded! {
                    border_top_style: rect.0,
                    border_right_style: rect.1,
                    border_bottom_style: rect.2,
                    border_left_style: rect.3,
            })
        }

        impl<'a> ToCss for LonghandsToSerialize<'a> {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
                let rect = Rect::new(
                    &self.border_top_style,
                    &self.border_right_style,
                    &self.border_bottom_style,
                    &self.border_left_style,
                );
                rect.to_css(dest)
            }
        }
    
    }





    /// https://drafts.csswg.org/css-backgrounds/#border-width
    pub mod border_width {
        use cssparser::Parser;
        use parser::ParserContext;
        use properties::{PropertyDeclaration, SourcePropertyDeclaration, MaybeBoxed, longhands};
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use std::fmt;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use style_traits::ToCss;

        pub struct Longhands {
                pub border_top_width:
                    longhands::border_top_width::SpecifiedValue
                    ,
                pub border_left_width:
                    longhands::border_left_width::SpecifiedValue
                    ,
                pub border_bottom_width:
                    longhands::border_bottom_width::SpecifiedValue
                    ,
                pub border_right_width:
                    longhands::border_right_width::SpecifiedValue
                    ,
        }

        /// Represents a serializable set of all of the longhand properties that
        /// correspond to a shorthand.
        pub struct LonghandsToSerialize<'a> {
                pub border_top_width:
                    &'a longhands::border_top_width::SpecifiedValue,
                pub border_left_width:
                    &'a longhands::border_left_width::SpecifiedValue,
                pub border_bottom_width:
                    &'a longhands::border_bottom_width::SpecifiedValue,
                pub border_right_width:
                    &'a longhands::border_right_width::SpecifiedValue,
        }

        impl<'a> LonghandsToSerialize<'a> {
            /// Tries to get a serializable set of longhands given a set of
            /// property declarations.
            pub fn from_iter<I>(iter: I) -> Result<Self, ()>
                where I: Iterator<Item=&'a PropertyDeclaration>,
            {
                // Define all of the expected variables that correspond to the shorthand
                    let mut border_top_width = None;
                    let mut border_left_width = None;
                    let mut border_bottom_width = None;
                    let mut border_right_width = None;

                // Attempt to assign the incoming declarations to the expected variables
                for longhand in iter {
                    match *longhand {
                            PropertyDeclaration::BorderTopWidth(ref value) => {
                                border_top_width = Some(value)
                            },
                            PropertyDeclaration::BorderLeftWidth(ref value) => {
                                border_left_width = Some(value)
                            },
                            PropertyDeclaration::BorderBottomWidth(ref value) => {
                                border_bottom_width = Some(value)
                            },
                            PropertyDeclaration::BorderRightWidth(ref value) => {
                                border_right_width = Some(value)
                            },
                        _ => {}
                    };
                }

                // If any of the expected variables are missing, return an error
                match (
                        border_top_width,
                        border_left_width,
                        border_bottom_width,
                        border_right_width,
                ) {

                    (
                        Some(border_top_width),
                        Some(border_left_width),
                        Some(border_bottom_width),
                        Some(border_right_width),
                    ) =>
                    Ok(LonghandsToSerialize {
                            border_top_width: border_top_width,
                            border_left_width: border_left_width,
                            border_bottom_width: border_bottom_width,
                            border_right_width: border_right_width,
                    }),
                    _ => Err(())
                }
            }
        }

        /// Parse the given shorthand and fill the result into the
        /// `declarations` vector.
        pub fn parse_into<'i, 't>(declarations: &mut SourcePropertyDeclaration,
                                  context: &ParserContext, input: &mut Parser<'i, 't>)
                                  -> Result<(), ParseError<'i>> {
            input.parse_entirely(|input| parse_value(context, input)).map(|longhands| {
                    declarations.push(PropertyDeclaration::BorderTopWidth(
                        longhands.border_top_width
                    ));
                    declarations.push(PropertyDeclaration::BorderLeftWidth(
                        longhands.border_left_width
                    ));
                    declarations.push(PropertyDeclaration::BorderBottomWidth(
                        longhands.border_bottom_width
                    ));
                    declarations.push(PropertyDeclaration::BorderRightWidth(
                        longhands.border_right_width
                    ));
            })
        }

        
    use values::generics::rect::Rect;
    use values::specified::{AllowQuirks, BorderSideWidth};

    pub fn parse_value<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                               -> Result<Longhands, ParseError<'i>> {
        let rect = Rect::parse_with(context, input, |_, i| {
            BorderSideWidth::parse_quirky(context, i, AllowQuirks::Yes)
        })?;
        Ok(expanded! {
            border_top_width: rect.0,
            border_right_width: rect.1,
            border_bottom_width: rect.2,
            border_left_width: rect.3,
        })
    }

    impl<'a> ToCss for LonghandsToSerialize<'a>  {
        fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
            let top = &self.border_top_width;
            let left = &self.border_left_width;
            let bottom = &self.border_bottom_width;
            let right = &self.border_right_width;
            Rect::new(top, right, bottom, left).to_css(dest)
        }
    }

    }



pub fn parse_border<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                            -> Result<(specified::Color,
                                       specified::BorderStyle,
                                       specified::BorderSideWidth), ParseError<'i>> {
    use values::specified::{Color, BorderStyle, BorderSideWidth};
    let _unused = context;
    let mut color = None;
    let mut style = None;
    let mut width = None;
    let mut any = false;
    loop {
        if color.is_none() {
            if let Ok(value) = input.try(|i| Color::parse(context, i)) {
                color = Some(value);
                any = true;
                continue
            }
        }
        if style.is_none() {
            if let Ok(value) = input.try(|i| BorderStyle::parse(context, i)) {
                style = Some(value);
                any = true;
                continue
            }
        }
        if width.is_none() {
            if let Ok(value) = input.try(|i| BorderSideWidth::parse(context, i)) {
                width = Some(value);
                any = true;
                continue
            }
        }
        break
    }
    if any {
        Ok((color.unwrap_or_else(|| Color::currentcolor()),
            style.unwrap_or(BorderStyle::none),
            width.unwrap_or(BorderSideWidth::Medium)))
    } else {
        Err(StyleParseError::UnspecifiedError.into())
    }
}

    
    

    /// https://drafts.csswg.org/css-backgrounds/#border-top
    pub mod border_top {
        use cssparser::Parser;
        use parser::ParserContext;
        use properties::{PropertyDeclaration, SourcePropertyDeclaration, MaybeBoxed, longhands};
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use std::fmt;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use style_traits::ToCss;

        pub struct Longhands {
                pub border_top_color:
                    longhands::border_top_color::SpecifiedValue
                    ,
                pub border_top_style:
                    longhands::border_top_style::SpecifiedValue
                    ,
                pub border_top_width:
                    longhands::border_top_width::SpecifiedValue
                    ,
        }

        /// Represents a serializable set of all of the longhand properties that
        /// correspond to a shorthand.
        pub struct LonghandsToSerialize<'a> {
                pub border_top_color:
                    &'a longhands::border_top_color::SpecifiedValue,
                pub border_top_style:
                    &'a longhands::border_top_style::SpecifiedValue,
                pub border_top_width:
                    &'a longhands::border_top_width::SpecifiedValue,
        }

        impl<'a> LonghandsToSerialize<'a> {
            /// Tries to get a serializable set of longhands given a set of
            /// property declarations.
            pub fn from_iter<I>(iter: I) -> Result<Self, ()>
                where I: Iterator<Item=&'a PropertyDeclaration>,
            {
                // Define all of the expected variables that correspond to the shorthand
                    let mut border_top_color = None;
                    let mut border_top_style = None;
                    let mut border_top_width = None;

                // Attempt to assign the incoming declarations to the expected variables
                for longhand in iter {
                    match *longhand {
                            PropertyDeclaration::BorderTopColor(ref value) => {
                                border_top_color = Some(value)
                            },
                            PropertyDeclaration::BorderTopStyle(ref value) => {
                                border_top_style = Some(value)
                            },
                            PropertyDeclaration::BorderTopWidth(ref value) => {
                                border_top_width = Some(value)
                            },
                        _ => {}
                    };
                }

                // If any of the expected variables are missing, return an error
                match (
                        border_top_color,
                        border_top_style,
                        border_top_width,
                ) {

                    (
                        Some(border_top_color),
                        Some(border_top_style),
                        Some(border_top_width),
                    ) =>
                    Ok(LonghandsToSerialize {
                            border_top_color: border_top_color,
                            border_top_style: border_top_style,
                            border_top_width: border_top_width,
                    }),
                    _ => Err(())
                }
            }
        }

        /// Parse the given shorthand and fill the result into the
        /// `declarations` vector.
        pub fn parse_into<'i, 't>(declarations: &mut SourcePropertyDeclaration,
                                  context: &ParserContext, input: &mut Parser<'i, 't>)
                                  -> Result<(), ParseError<'i>> {
            input.parse_entirely(|input| parse_value(context, input)).map(|longhands| {
                    declarations.push(PropertyDeclaration::BorderTopColor(
                        longhands.border_top_color
                    ));
                    declarations.push(PropertyDeclaration::BorderTopStyle(
                        longhands.border_top_style
                    ));
                    declarations.push(PropertyDeclaration::BorderTopWidth(
                        longhands.border_top_width
                    ));
            })
        }

        

    pub fn parse_value<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                               -> Result<Longhands, ParseError<'i>> {
        let (color, style, width) = super::parse_border(context, input)?;
        Ok(expanded! {
            border_top_color: color,
            border_top_style: style,
            border_top_width: width
        })
    }

    impl<'a> ToCss for LonghandsToSerialize<'a>  {
        fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
            super::serialize_directional_border(
                dest,
                self.border_top_width,
                self.border_top_style,
                self.border_top_color
            )
        }
    }

    
    }

    
    

    /// https://drafts.csswg.org/css-backgrounds/#border-left
    pub mod border_left {
        use cssparser::Parser;
        use parser::ParserContext;
        use properties::{PropertyDeclaration, SourcePropertyDeclaration, MaybeBoxed, longhands};
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use std::fmt;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use style_traits::ToCss;

        pub struct Longhands {
                pub border_left_color:
                    longhands::border_left_color::SpecifiedValue
                    ,
                pub border_left_style:
                    longhands::border_left_style::SpecifiedValue
                    ,
                pub border_left_width:
                    longhands::border_left_width::SpecifiedValue
                    ,
        }

        /// Represents a serializable set of all of the longhand properties that
        /// correspond to a shorthand.
        pub struct LonghandsToSerialize<'a> {
                pub border_left_color:
                    &'a longhands::border_left_color::SpecifiedValue,
                pub border_left_style:
                    &'a longhands::border_left_style::SpecifiedValue,
                pub border_left_width:
                    &'a longhands::border_left_width::SpecifiedValue,
        }

        impl<'a> LonghandsToSerialize<'a> {
            /// Tries to get a serializable set of longhands given a set of
            /// property declarations.
            pub fn from_iter<I>(iter: I) -> Result<Self, ()>
                where I: Iterator<Item=&'a PropertyDeclaration>,
            {
                // Define all of the expected variables that correspond to the shorthand
                    let mut border_left_color = None;
                    let mut border_left_style = None;
                    let mut border_left_width = None;

                // Attempt to assign the incoming declarations to the expected variables
                for longhand in iter {
                    match *longhand {
                            PropertyDeclaration::BorderLeftColor(ref value) => {
                                border_left_color = Some(value)
                            },
                            PropertyDeclaration::BorderLeftStyle(ref value) => {
                                border_left_style = Some(value)
                            },
                            PropertyDeclaration::BorderLeftWidth(ref value) => {
                                border_left_width = Some(value)
                            },
                        _ => {}
                    };
                }

                // If any of the expected variables are missing, return an error
                match (
                        border_left_color,
                        border_left_style,
                        border_left_width,
                ) {

                    (
                        Some(border_left_color),
                        Some(border_left_style),
                        Some(border_left_width),
                    ) =>
                    Ok(LonghandsToSerialize {
                            border_left_color: border_left_color,
                            border_left_style: border_left_style,
                            border_left_width: border_left_width,
                    }),
                    _ => Err(())
                }
            }
        }

        /// Parse the given shorthand and fill the result into the
        /// `declarations` vector.
        pub fn parse_into<'i, 't>(declarations: &mut SourcePropertyDeclaration,
                                  context: &ParserContext, input: &mut Parser<'i, 't>)
                                  -> Result<(), ParseError<'i>> {
            input.parse_entirely(|input| parse_value(context, input)).map(|longhands| {
                    declarations.push(PropertyDeclaration::BorderLeftColor(
                        longhands.border_left_color
                    ));
                    declarations.push(PropertyDeclaration::BorderLeftStyle(
                        longhands.border_left_style
                    ));
                    declarations.push(PropertyDeclaration::BorderLeftWidth(
                        longhands.border_left_width
                    ));
            })
        }

        

    pub fn parse_value<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                               -> Result<Longhands, ParseError<'i>> {
        let (color, style, width) = super::parse_border(context, input)?;
        Ok(expanded! {
            border_left_color: color,
            border_left_style: style,
            border_left_width: width
        })
    }

    impl<'a> ToCss for LonghandsToSerialize<'a>  {
        fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
            super::serialize_directional_border(
                dest,
                self.border_left_width,
                self.border_left_style,
                self.border_left_color
            )
        }
    }

    
    }

    
    

    /// https://drafts.csswg.org/css-backgrounds/#border-bottom
    pub mod border_bottom {
        use cssparser::Parser;
        use parser::ParserContext;
        use properties::{PropertyDeclaration, SourcePropertyDeclaration, MaybeBoxed, longhands};
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use std::fmt;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use style_traits::ToCss;

        pub struct Longhands {
                pub border_bottom_color:
                    longhands::border_bottom_color::SpecifiedValue
                    ,
                pub border_bottom_style:
                    longhands::border_bottom_style::SpecifiedValue
                    ,
                pub border_bottom_width:
                    longhands::border_bottom_width::SpecifiedValue
                    ,
        }

        /// Represents a serializable set of all of the longhand properties that
        /// correspond to a shorthand.
        pub struct LonghandsToSerialize<'a> {
                pub border_bottom_color:
                    &'a longhands::border_bottom_color::SpecifiedValue,
                pub border_bottom_style:
                    &'a longhands::border_bottom_style::SpecifiedValue,
                pub border_bottom_width:
                    &'a longhands::border_bottom_width::SpecifiedValue,
        }

        impl<'a> LonghandsToSerialize<'a> {
            /// Tries to get a serializable set of longhands given a set of
            /// property declarations.
            pub fn from_iter<I>(iter: I) -> Result<Self, ()>
                where I: Iterator<Item=&'a PropertyDeclaration>,
            {
                // Define all of the expected variables that correspond to the shorthand
                    let mut border_bottom_color = None;
                    let mut border_bottom_style = None;
                    let mut border_bottom_width = None;

                // Attempt to assign the incoming declarations to the expected variables
                for longhand in iter {
                    match *longhand {
                            PropertyDeclaration::BorderBottomColor(ref value) => {
                                border_bottom_color = Some(value)
                            },
                            PropertyDeclaration::BorderBottomStyle(ref value) => {
                                border_bottom_style = Some(value)
                            },
                            PropertyDeclaration::BorderBottomWidth(ref value) => {
                                border_bottom_width = Some(value)
                            },
                        _ => {}
                    };
                }

                // If any of the expected variables are missing, return an error
                match (
                        border_bottom_color,
                        border_bottom_style,
                        border_bottom_width,
                ) {

                    (
                        Some(border_bottom_color),
                        Some(border_bottom_style),
                        Some(border_bottom_width),
                    ) =>
                    Ok(LonghandsToSerialize {
                            border_bottom_color: border_bottom_color,
                            border_bottom_style: border_bottom_style,
                            border_bottom_width: border_bottom_width,
                    }),
                    _ => Err(())
                }
            }
        }

        /// Parse the given shorthand and fill the result into the
        /// `declarations` vector.
        pub fn parse_into<'i, 't>(declarations: &mut SourcePropertyDeclaration,
                                  context: &ParserContext, input: &mut Parser<'i, 't>)
                                  -> Result<(), ParseError<'i>> {
            input.parse_entirely(|input| parse_value(context, input)).map(|longhands| {
                    declarations.push(PropertyDeclaration::BorderBottomColor(
                        longhands.border_bottom_color
                    ));
                    declarations.push(PropertyDeclaration::BorderBottomStyle(
                        longhands.border_bottom_style
                    ));
                    declarations.push(PropertyDeclaration::BorderBottomWidth(
                        longhands.border_bottom_width
                    ));
            })
        }

        

    pub fn parse_value<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                               -> Result<Longhands, ParseError<'i>> {
        let (color, style, width) = super::parse_border(context, input)?;
        Ok(expanded! {
            border_bottom_color: color,
            border_bottom_style: style,
            border_bottom_width: width
        })
    }

    impl<'a> ToCss for LonghandsToSerialize<'a>  {
        fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
            super::serialize_directional_border(
                dest,
                self.border_bottom_width,
                self.border_bottom_style,
                self.border_bottom_color
            )
        }
    }

    
    }

    
    

    /// https://drafts.csswg.org/css-backgrounds/#border-right
    pub mod border_right {
        use cssparser::Parser;
        use parser::ParserContext;
        use properties::{PropertyDeclaration, SourcePropertyDeclaration, MaybeBoxed, longhands};
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use std::fmt;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use style_traits::ToCss;

        pub struct Longhands {
                pub border_right_color:
                    longhands::border_right_color::SpecifiedValue
                    ,
                pub border_right_style:
                    longhands::border_right_style::SpecifiedValue
                    ,
                pub border_right_width:
                    longhands::border_right_width::SpecifiedValue
                    ,
        }

        /// Represents a serializable set of all of the longhand properties that
        /// correspond to a shorthand.
        pub struct LonghandsToSerialize<'a> {
                pub border_right_color:
                    &'a longhands::border_right_color::SpecifiedValue,
                pub border_right_style:
                    &'a longhands::border_right_style::SpecifiedValue,
                pub border_right_width:
                    &'a longhands::border_right_width::SpecifiedValue,
        }

        impl<'a> LonghandsToSerialize<'a> {
            /// Tries to get a serializable set of longhands given a set of
            /// property declarations.
            pub fn from_iter<I>(iter: I) -> Result<Self, ()>
                where I: Iterator<Item=&'a PropertyDeclaration>,
            {
                // Define all of the expected variables that correspond to the shorthand
                    let mut border_right_color = None;
                    let mut border_right_style = None;
                    let mut border_right_width = None;

                // Attempt to assign the incoming declarations to the expected variables
                for longhand in iter {
                    match *longhand {
                            PropertyDeclaration::BorderRightColor(ref value) => {
                                border_right_color = Some(value)
                            },
                            PropertyDeclaration::BorderRightStyle(ref value) => {
                                border_right_style = Some(value)
                            },
                            PropertyDeclaration::BorderRightWidth(ref value) => {
                                border_right_width = Some(value)
                            },
                        _ => {}
                    };
                }

                // If any of the expected variables are missing, return an error
                match (
                        border_right_color,
                        border_right_style,
                        border_right_width,
                ) {

                    (
                        Some(border_right_color),
                        Some(border_right_style),
                        Some(border_right_width),
                    ) =>
                    Ok(LonghandsToSerialize {
                            border_right_color: border_right_color,
                            border_right_style: border_right_style,
                            border_right_width: border_right_width,
                    }),
                    _ => Err(())
                }
            }
        }

        /// Parse the given shorthand and fill the result into the
        /// `declarations` vector.
        pub fn parse_into<'i, 't>(declarations: &mut SourcePropertyDeclaration,
                                  context: &ParserContext, input: &mut Parser<'i, 't>)
                                  -> Result<(), ParseError<'i>> {
            input.parse_entirely(|input| parse_value(context, input)).map(|longhands| {
                    declarations.push(PropertyDeclaration::BorderRightColor(
                        longhands.border_right_color
                    ));
                    declarations.push(PropertyDeclaration::BorderRightStyle(
                        longhands.border_right_style
                    ));
                    declarations.push(PropertyDeclaration::BorderRightWidth(
                        longhands.border_right_width
                    ));
            })
        }

        

    pub fn parse_value<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                               -> Result<Longhands, ParseError<'i>> {
        let (color, style, width) = super::parse_border(context, input)?;
        Ok(expanded! {
            border_right_color: color,
            border_right_style: style,
            border_right_width: width
        })
    }

    impl<'a> ToCss for LonghandsToSerialize<'a>  {
        fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
            super::serialize_directional_border(
                dest,
                self.border_right_width,
                self.border_right_style,
                self.border_right_color
            )
        }
    }

    
    }

    
    

    /// https://drafts.csswg.org/css-logical-props/#propdef-border-block-start
    pub mod border_block_start {
        use cssparser::Parser;
        use parser::ParserContext;
        use properties::{PropertyDeclaration, SourcePropertyDeclaration, MaybeBoxed, longhands};
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use std::fmt;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use style_traits::ToCss;

        pub struct Longhands {
                pub border_block_start_color:
                    longhands::border_block_start_color::SpecifiedValue
                    ,
                pub border_block_start_style:
                    longhands::border_block_start_style::SpecifiedValue
                    ,
                pub border_block_start_width:
                    longhands::border_block_start_width::SpecifiedValue
                    ,
        }

        /// Represents a serializable set of all of the longhand properties that
        /// correspond to a shorthand.
        pub struct LonghandsToSerialize<'a> {
                pub border_block_start_color:
                    &'a longhands::border_block_start_color::SpecifiedValue,
                pub border_block_start_style:
                    &'a longhands::border_block_start_style::SpecifiedValue,
                pub border_block_start_width:
                    &'a longhands::border_block_start_width::SpecifiedValue,
        }

        impl<'a> LonghandsToSerialize<'a> {
            /// Tries to get a serializable set of longhands given a set of
            /// property declarations.
            pub fn from_iter<I>(iter: I) -> Result<Self, ()>
                where I: Iterator<Item=&'a PropertyDeclaration>,
            {
                // Define all of the expected variables that correspond to the shorthand
                    let mut border_block_start_color = None;
                    let mut border_block_start_style = None;
                    let mut border_block_start_width = None;

                // Attempt to assign the incoming declarations to the expected variables
                for longhand in iter {
                    match *longhand {
                            PropertyDeclaration::BorderBlockStartColor(ref value) => {
                                border_block_start_color = Some(value)
                            },
                            PropertyDeclaration::BorderBlockStartStyle(ref value) => {
                                border_block_start_style = Some(value)
                            },
                            PropertyDeclaration::BorderBlockStartWidth(ref value) => {
                                border_block_start_width = Some(value)
                            },
                        _ => {}
                    };
                }

                // If any of the expected variables are missing, return an error
                match (
                        border_block_start_color,
                        border_block_start_style,
                        border_block_start_width,
                ) {

                    (
                        Some(border_block_start_color),
                        Some(border_block_start_style),
                        Some(border_block_start_width),
                    ) =>
                    Ok(LonghandsToSerialize {
                            border_block_start_color: border_block_start_color,
                            border_block_start_style: border_block_start_style,
                            border_block_start_width: border_block_start_width,
                    }),
                    _ => Err(())
                }
            }
        }

        /// Parse the given shorthand and fill the result into the
        /// `declarations` vector.
        pub fn parse_into<'i, 't>(declarations: &mut SourcePropertyDeclaration,
                                  context: &ParserContext, input: &mut Parser<'i, 't>)
                                  -> Result<(), ParseError<'i>> {
            input.parse_entirely(|input| parse_value(context, input)).map(|longhands| {
                    declarations.push(PropertyDeclaration::BorderBlockStartColor(
                        longhands.border_block_start_color
                    ));
                    declarations.push(PropertyDeclaration::BorderBlockStartStyle(
                        longhands.border_block_start_style
                    ));
                    declarations.push(PropertyDeclaration::BorderBlockStartWidth(
                        longhands.border_block_start_width
                    ));
            })
        }

        

    pub fn parse_value<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                               -> Result<Longhands, ParseError<'i>> {
        let (color, style, width) = super::parse_border(context, input)?;
        Ok(expanded! {
            border_block_start_color: color,
            border_block_start_style: style,
            border_block_start_width: width
        })
    }

    impl<'a> ToCss for LonghandsToSerialize<'a>  {
        fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
            super::serialize_directional_border(
                dest,
                self.border_block_start_width,
                self.border_block_start_style,
                self.border_block_start_color
            )
        }
    }

    
    }

    
    

    /// https://drafts.csswg.org/css-logical-props/#propdef-border-block-end
    pub mod border_block_end {
        use cssparser::Parser;
        use parser::ParserContext;
        use properties::{PropertyDeclaration, SourcePropertyDeclaration, MaybeBoxed, longhands};
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use std::fmt;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use style_traits::ToCss;

        pub struct Longhands {
                pub border_block_end_color:
                    longhands::border_block_end_color::SpecifiedValue
                    ,
                pub border_block_end_style:
                    longhands::border_block_end_style::SpecifiedValue
                    ,
                pub border_block_end_width:
                    longhands::border_block_end_width::SpecifiedValue
                    ,
        }

        /// Represents a serializable set of all of the longhand properties that
        /// correspond to a shorthand.
        pub struct LonghandsToSerialize<'a> {
                pub border_block_end_color:
                    &'a longhands::border_block_end_color::SpecifiedValue,
                pub border_block_end_style:
                    &'a longhands::border_block_end_style::SpecifiedValue,
                pub border_block_end_width:
                    &'a longhands::border_block_end_width::SpecifiedValue,
        }

        impl<'a> LonghandsToSerialize<'a> {
            /// Tries to get a serializable set of longhands given a set of
            /// property declarations.
            pub fn from_iter<I>(iter: I) -> Result<Self, ()>
                where I: Iterator<Item=&'a PropertyDeclaration>,
            {
                // Define all of the expected variables that correspond to the shorthand
                    let mut border_block_end_color = None;
                    let mut border_block_end_style = None;
                    let mut border_block_end_width = None;

                // Attempt to assign the incoming declarations to the expected variables
                for longhand in iter {
                    match *longhand {
                            PropertyDeclaration::BorderBlockEndColor(ref value) => {
                                border_block_end_color = Some(value)
                            },
                            PropertyDeclaration::BorderBlockEndStyle(ref value) => {
                                border_block_end_style = Some(value)
                            },
                            PropertyDeclaration::BorderBlockEndWidth(ref value) => {
                                border_block_end_width = Some(value)
                            },
                        _ => {}
                    };
                }

                // If any of the expected variables are missing, return an error
                match (
                        border_block_end_color,
                        border_block_end_style,
                        border_block_end_width,
                ) {

                    (
                        Some(border_block_end_color),
                        Some(border_block_end_style),
                        Some(border_block_end_width),
                    ) =>
                    Ok(LonghandsToSerialize {
                            border_block_end_color: border_block_end_color,
                            border_block_end_style: border_block_end_style,
                            border_block_end_width: border_block_end_width,
                    }),
                    _ => Err(())
                }
            }
        }

        /// Parse the given shorthand and fill the result into the
        /// `declarations` vector.
        pub fn parse_into<'i, 't>(declarations: &mut SourcePropertyDeclaration,
                                  context: &ParserContext, input: &mut Parser<'i, 't>)
                                  -> Result<(), ParseError<'i>> {
            input.parse_entirely(|input| parse_value(context, input)).map(|longhands| {
                    declarations.push(PropertyDeclaration::BorderBlockEndColor(
                        longhands.border_block_end_color
                    ));
                    declarations.push(PropertyDeclaration::BorderBlockEndStyle(
                        longhands.border_block_end_style
                    ));
                    declarations.push(PropertyDeclaration::BorderBlockEndWidth(
                        longhands.border_block_end_width
                    ));
            })
        }

        

    pub fn parse_value<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                               -> Result<Longhands, ParseError<'i>> {
        let (color, style, width) = super::parse_border(context, input)?;
        Ok(expanded! {
            border_block_end_color: color,
            border_block_end_style: style,
            border_block_end_width: width
        })
    }

    impl<'a> ToCss for LonghandsToSerialize<'a>  {
        fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
            super::serialize_directional_border(
                dest,
                self.border_block_end_width,
                self.border_block_end_style,
                self.border_block_end_color
            )
        }
    }

    
    }

    
    

    /// https://drafts.csswg.org/css-logical-props/#propdef-border-inline-start
    pub mod border_inline_start {
        use cssparser::Parser;
        use parser::ParserContext;
        use properties::{PropertyDeclaration, SourcePropertyDeclaration, MaybeBoxed, longhands};
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use std::fmt;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use style_traits::ToCss;

        pub struct Longhands {
                pub border_inline_start_color:
                    longhands::border_inline_start_color::SpecifiedValue
                    ,
                pub border_inline_start_style:
                    longhands::border_inline_start_style::SpecifiedValue
                    ,
                pub border_inline_start_width:
                    longhands::border_inline_start_width::SpecifiedValue
                    ,
        }

        /// Represents a serializable set of all of the longhand properties that
        /// correspond to a shorthand.
        pub struct LonghandsToSerialize<'a> {
                pub border_inline_start_color:
                    &'a longhands::border_inline_start_color::SpecifiedValue,
                pub border_inline_start_style:
                    &'a longhands::border_inline_start_style::SpecifiedValue,
                pub border_inline_start_width:
                    &'a longhands::border_inline_start_width::SpecifiedValue,
        }

        impl<'a> LonghandsToSerialize<'a> {
            /// Tries to get a serializable set of longhands given a set of
            /// property declarations.
            pub fn from_iter<I>(iter: I) -> Result<Self, ()>
                where I: Iterator<Item=&'a PropertyDeclaration>,
            {
                // Define all of the expected variables that correspond to the shorthand
                    let mut border_inline_start_color = None;
                    let mut border_inline_start_style = None;
                    let mut border_inline_start_width = None;

                // Attempt to assign the incoming declarations to the expected variables
                for longhand in iter {
                    match *longhand {
                            PropertyDeclaration::BorderInlineStartColor(ref value) => {
                                border_inline_start_color = Some(value)
                            },
                            PropertyDeclaration::BorderInlineStartStyle(ref value) => {
                                border_inline_start_style = Some(value)
                            },
                            PropertyDeclaration::BorderInlineStartWidth(ref value) => {
                                border_inline_start_width = Some(value)
                            },
                        _ => {}
                    };
                }

                // If any of the expected variables are missing, return an error
                match (
                        border_inline_start_color,
                        border_inline_start_style,
                        border_inline_start_width,
                ) {

                    (
                        Some(border_inline_start_color),
                        Some(border_inline_start_style),
                        Some(border_inline_start_width),
                    ) =>
                    Ok(LonghandsToSerialize {
                            border_inline_start_color: border_inline_start_color,
                            border_inline_start_style: border_inline_start_style,
                            border_inline_start_width: border_inline_start_width,
                    }),
                    _ => Err(())
                }
            }
        }

        /// Parse the given shorthand and fill the result into the
        /// `declarations` vector.
        pub fn parse_into<'i, 't>(declarations: &mut SourcePropertyDeclaration,
                                  context: &ParserContext, input: &mut Parser<'i, 't>)
                                  -> Result<(), ParseError<'i>> {
            input.parse_entirely(|input| parse_value(context, input)).map(|longhands| {
                    declarations.push(PropertyDeclaration::BorderInlineStartColor(
                        longhands.border_inline_start_color
                    ));
                    declarations.push(PropertyDeclaration::BorderInlineStartStyle(
                        longhands.border_inline_start_style
                    ));
                    declarations.push(PropertyDeclaration::BorderInlineStartWidth(
                        longhands.border_inline_start_width
                    ));
            })
        }

        

    pub fn parse_value<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                               -> Result<Longhands, ParseError<'i>> {
        let (color, style, width) = super::parse_border(context, input)?;
        Ok(expanded! {
            border_inline_start_color: color,
            border_inline_start_style: style,
            border_inline_start_width: width
        })
    }

    impl<'a> ToCss for LonghandsToSerialize<'a>  {
        fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
            super::serialize_directional_border(
                dest,
                self.border_inline_start_width,
                self.border_inline_start_style,
                self.border_inline_start_color
            )
        }
    }

    
    }

    
    

    /// https://drafts.csswg.org/css-logical-props/#propdef-border-inline-end
    pub mod border_inline_end {
        use cssparser::Parser;
        use parser::ParserContext;
        use properties::{PropertyDeclaration, SourcePropertyDeclaration, MaybeBoxed, longhands};
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use std::fmt;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use style_traits::ToCss;

        pub struct Longhands {
                pub border_inline_end_color:
                    longhands::border_inline_end_color::SpecifiedValue
                    ,
                pub border_inline_end_style:
                    longhands::border_inline_end_style::SpecifiedValue
                    ,
                pub border_inline_end_width:
                    longhands::border_inline_end_width::SpecifiedValue
                    ,
        }

        /// Represents a serializable set of all of the longhand properties that
        /// correspond to a shorthand.
        pub struct LonghandsToSerialize<'a> {
                pub border_inline_end_color:
                    &'a longhands::border_inline_end_color::SpecifiedValue,
                pub border_inline_end_style:
                    &'a longhands::border_inline_end_style::SpecifiedValue,
                pub border_inline_end_width:
                    &'a longhands::border_inline_end_width::SpecifiedValue,
        }

        impl<'a> LonghandsToSerialize<'a> {
            /// Tries to get a serializable set of longhands given a set of
            /// property declarations.
            pub fn from_iter<I>(iter: I) -> Result<Self, ()>
                where I: Iterator<Item=&'a PropertyDeclaration>,
            {
                // Define all of the expected variables that correspond to the shorthand
                    let mut border_inline_end_color = None;
                    let mut border_inline_end_style = None;
                    let mut border_inline_end_width = None;

                // Attempt to assign the incoming declarations to the expected variables
                for longhand in iter {
                    match *longhand {
                            PropertyDeclaration::BorderInlineEndColor(ref value) => {
                                border_inline_end_color = Some(value)
                            },
                            PropertyDeclaration::BorderInlineEndStyle(ref value) => {
                                border_inline_end_style = Some(value)
                            },
                            PropertyDeclaration::BorderInlineEndWidth(ref value) => {
                                border_inline_end_width = Some(value)
                            },
                        _ => {}
                    };
                }

                // If any of the expected variables are missing, return an error
                match (
                        border_inline_end_color,
                        border_inline_end_style,
                        border_inline_end_width,
                ) {

                    (
                        Some(border_inline_end_color),
                        Some(border_inline_end_style),
                        Some(border_inline_end_width),
                    ) =>
                    Ok(LonghandsToSerialize {
                            border_inline_end_color: border_inline_end_color,
                            border_inline_end_style: border_inline_end_style,
                            border_inline_end_width: border_inline_end_width,
                    }),
                    _ => Err(())
                }
            }
        }

        /// Parse the given shorthand and fill the result into the
        /// `declarations` vector.
        pub fn parse_into<'i, 't>(declarations: &mut SourcePropertyDeclaration,
                                  context: &ParserContext, input: &mut Parser<'i, 't>)
                                  -> Result<(), ParseError<'i>> {
            input.parse_entirely(|input| parse_value(context, input)).map(|longhands| {
                    declarations.push(PropertyDeclaration::BorderInlineEndColor(
                        longhands.border_inline_end_color
                    ));
                    declarations.push(PropertyDeclaration::BorderInlineEndStyle(
                        longhands.border_inline_end_style
                    ));
                    declarations.push(PropertyDeclaration::BorderInlineEndWidth(
                        longhands.border_inline_end_width
                    ));
            })
        }

        

    pub fn parse_value<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                               -> Result<Longhands, ParseError<'i>> {
        let (color, style, width) = super::parse_border(context, input)?;
        Ok(expanded! {
            border_inline_end_color: color,
            border_inline_end_style: style,
            border_inline_end_width: width
        })
    }

    impl<'a> ToCss for LonghandsToSerialize<'a>  {
        fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
            super::serialize_directional_border(
                dest,
                self.border_inline_end_width,
                self.border_inline_end_style,
                self.border_inline_end_color
            )
        }
    }

    
    }




    /// https://drafts.csswg.org/css-backgrounds/#border
    pub mod border {
        use cssparser::Parser;
        use parser::ParserContext;
        use properties::{PropertyDeclaration, SourcePropertyDeclaration, MaybeBoxed, longhands};
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use std::fmt;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use style_traits::ToCss;

        pub struct Longhands {
                pub border_top_color:
                    longhands::border_top_color::SpecifiedValue
                    ,
                pub border_top_style:
                    longhands::border_top_style::SpecifiedValue
                    ,
                pub border_top_width:
                    longhands::border_top_width::SpecifiedValue
                    ,
                pub border_left_color:
                    longhands::border_left_color::SpecifiedValue
                    ,
                pub border_left_style:
                    longhands::border_left_style::SpecifiedValue
                    ,
                pub border_left_width:
                    longhands::border_left_width::SpecifiedValue
                    ,
                pub border_bottom_color:
                    longhands::border_bottom_color::SpecifiedValue
                    ,
                pub border_bottom_style:
                    longhands::border_bottom_style::SpecifiedValue
                    ,
                pub border_bottom_width:
                    longhands::border_bottom_width::SpecifiedValue
                    ,
                pub border_right_color:
                    longhands::border_right_color::SpecifiedValue
                    ,
                pub border_right_style:
                    longhands::border_right_style::SpecifiedValue
                    ,
                pub border_right_width:
                    longhands::border_right_width::SpecifiedValue
                    ,
                pub border_image_outset:
                        Box<
                    longhands::border_image_outset::SpecifiedValue
                        >
                    ,
                pub border_image_repeat:
                    longhands::border_image_repeat::SpecifiedValue
                    ,
                pub border_image_slice:
                        Box<
                    longhands::border_image_slice::SpecifiedValue
                        >
                    ,
                pub border_image_source:
                        Box<
                    longhands::border_image_source::SpecifiedValue
                        >
                    ,
                pub border_image_width:
                        Box<
                    longhands::border_image_width::SpecifiedValue
                        >
                    ,
                pub _moz_border_top_colors:
                    longhands::_moz_border_top_colors::SpecifiedValue
                    ,
                pub _moz_border_left_colors:
                    longhands::_moz_border_left_colors::SpecifiedValue
                    ,
                pub _moz_border_bottom_colors:
                    longhands::_moz_border_bottom_colors::SpecifiedValue
                    ,
                pub _moz_border_right_colors:
                    longhands::_moz_border_right_colors::SpecifiedValue
                    ,
        }

        /// Represents a serializable set of all of the longhand properties that
        /// correspond to a shorthand.
        pub struct LonghandsToSerialize<'a> {
                pub border_top_color:
                    &'a longhands::border_top_color::SpecifiedValue,
                pub border_top_style:
                    &'a longhands::border_top_style::SpecifiedValue,
                pub border_top_width:
                    &'a longhands::border_top_width::SpecifiedValue,
                pub border_left_color:
                    &'a longhands::border_left_color::SpecifiedValue,
                pub border_left_style:
                    &'a longhands::border_left_style::SpecifiedValue,
                pub border_left_width:
                    &'a longhands::border_left_width::SpecifiedValue,
                pub border_bottom_color:
                    &'a longhands::border_bottom_color::SpecifiedValue,
                pub border_bottom_style:
                    &'a longhands::border_bottom_style::SpecifiedValue,
                pub border_bottom_width:
                    &'a longhands::border_bottom_width::SpecifiedValue,
                pub border_right_color:
                    &'a longhands::border_right_color::SpecifiedValue,
                pub border_right_style:
                    &'a longhands::border_right_style::SpecifiedValue,
                pub border_right_width:
                    &'a longhands::border_right_width::SpecifiedValue,
                pub border_image_outset:
                    &'a longhands::border_image_outset::SpecifiedValue,
                pub border_image_repeat:
                    &'a longhands::border_image_repeat::SpecifiedValue,
                pub border_image_slice:
                    &'a longhands::border_image_slice::SpecifiedValue,
                pub border_image_source:
                    &'a longhands::border_image_source::SpecifiedValue,
                pub border_image_width:
                    &'a longhands::border_image_width::SpecifiedValue,
                pub _moz_border_top_colors:
                    &'a longhands::_moz_border_top_colors::SpecifiedValue,
                pub _moz_border_left_colors:
                    &'a longhands::_moz_border_left_colors::SpecifiedValue,
                pub _moz_border_bottom_colors:
                    &'a longhands::_moz_border_bottom_colors::SpecifiedValue,
                pub _moz_border_right_colors:
                    &'a longhands::_moz_border_right_colors::SpecifiedValue,
        }

        impl<'a> LonghandsToSerialize<'a> {
            /// Tries to get a serializable set of longhands given a set of
            /// property declarations.
            pub fn from_iter<I>(iter: I) -> Result<Self, ()>
                where I: Iterator<Item=&'a PropertyDeclaration>,
            {
                // Define all of the expected variables that correspond to the shorthand
                    let mut border_top_color = None;
                    let mut border_top_style = None;
                    let mut border_top_width = None;
                    let mut border_left_color = None;
                    let mut border_left_style = None;
                    let mut border_left_width = None;
                    let mut border_bottom_color = None;
                    let mut border_bottom_style = None;
                    let mut border_bottom_width = None;
                    let mut border_right_color = None;
                    let mut border_right_style = None;
                    let mut border_right_width = None;
                    let mut border_image_outset = None;
                    let mut border_image_repeat = None;
                    let mut border_image_slice = None;
                    let mut border_image_source = None;
                    let mut border_image_width = None;
                    let mut _moz_border_top_colors = None;
                    let mut _moz_border_left_colors = None;
                    let mut _moz_border_bottom_colors = None;
                    let mut _moz_border_right_colors = None;

                // Attempt to assign the incoming declarations to the expected variables
                for longhand in iter {
                    match *longhand {
                            PropertyDeclaration::BorderTopColor(ref value) => {
                                border_top_color = Some(value)
                            },
                            PropertyDeclaration::BorderTopStyle(ref value) => {
                                border_top_style = Some(value)
                            },
                            PropertyDeclaration::BorderTopWidth(ref value) => {
                                border_top_width = Some(value)
                            },
                            PropertyDeclaration::BorderLeftColor(ref value) => {
                                border_left_color = Some(value)
                            },
                            PropertyDeclaration::BorderLeftStyle(ref value) => {
                                border_left_style = Some(value)
                            },
                            PropertyDeclaration::BorderLeftWidth(ref value) => {
                                border_left_width = Some(value)
                            },
                            PropertyDeclaration::BorderBottomColor(ref value) => {
                                border_bottom_color = Some(value)
                            },
                            PropertyDeclaration::BorderBottomStyle(ref value) => {
                                border_bottom_style = Some(value)
                            },
                            PropertyDeclaration::BorderBottomWidth(ref value) => {
                                border_bottom_width = Some(value)
                            },
                            PropertyDeclaration::BorderRightColor(ref value) => {
                                border_right_color = Some(value)
                            },
                            PropertyDeclaration::BorderRightStyle(ref value) => {
                                border_right_style = Some(value)
                            },
                            PropertyDeclaration::BorderRightWidth(ref value) => {
                                border_right_width = Some(value)
                            },
                            PropertyDeclaration::BorderImageOutset(ref value) => {
                                border_image_outset = Some(value)
                            },
                            PropertyDeclaration::BorderImageRepeat(ref value) => {
                                border_image_repeat = Some(value)
                            },
                            PropertyDeclaration::BorderImageSlice(ref value) => {
                                border_image_slice = Some(value)
                            },
                            PropertyDeclaration::BorderImageSource(ref value) => {
                                border_image_source = Some(value)
                            },
                            PropertyDeclaration::BorderImageWidth(ref value) => {
                                border_image_width = Some(value)
                            },
                            PropertyDeclaration::MozBorderTopColors(ref value) => {
                                _moz_border_top_colors = Some(value)
                            },
                            PropertyDeclaration::MozBorderLeftColors(ref value) => {
                                _moz_border_left_colors = Some(value)
                            },
                            PropertyDeclaration::MozBorderBottomColors(ref value) => {
                                _moz_border_bottom_colors = Some(value)
                            },
                            PropertyDeclaration::MozBorderRightColors(ref value) => {
                                _moz_border_right_colors = Some(value)
                            },
                        _ => {}
                    };
                }

                // If any of the expected variables are missing, return an error
                match (
                        border_top_color,
                        border_top_style,
                        border_top_width,
                        border_left_color,
                        border_left_style,
                        border_left_width,
                        border_bottom_color,
                        border_bottom_style,
                        border_bottom_width,
                        border_right_color,
                        border_right_style,
                        border_right_width,
                        border_image_outset,
                        border_image_repeat,
                        border_image_slice,
                        border_image_source,
                        border_image_width,
                        _moz_border_top_colors,
                        _moz_border_left_colors,
                        _moz_border_bottom_colors,
                        _moz_border_right_colors,
                ) {

                    (
                        Some(border_top_color),
                        Some(border_top_style),
                        Some(border_top_width),
                        Some(border_left_color),
                        Some(border_left_style),
                        Some(border_left_width),
                        Some(border_bottom_color),
                        Some(border_bottom_style),
                        Some(border_bottom_width),
                        Some(border_right_color),
                        Some(border_right_style),
                        Some(border_right_width),
                        Some(border_image_outset),
                        Some(border_image_repeat),
                        Some(border_image_slice),
                        Some(border_image_source),
                        Some(border_image_width),
                        Some(_moz_border_top_colors),
                        Some(_moz_border_left_colors),
                        Some(_moz_border_bottom_colors),
                        Some(_moz_border_right_colors),
                    ) =>
                    Ok(LonghandsToSerialize {
                            border_top_color: border_top_color,
                            border_top_style: border_top_style,
                            border_top_width: border_top_width,
                            border_left_color: border_left_color,
                            border_left_style: border_left_style,
                            border_left_width: border_left_width,
                            border_bottom_color: border_bottom_color,
                            border_bottom_style: border_bottom_style,
                            border_bottom_width: border_bottom_width,
                            border_right_color: border_right_color,
                            border_right_style: border_right_style,
                            border_right_width: border_right_width,
                            border_image_outset: border_image_outset,
                            border_image_repeat: border_image_repeat,
                            border_image_slice: border_image_slice,
                            border_image_source: border_image_source,
                            border_image_width: border_image_width,
                            _moz_border_top_colors: _moz_border_top_colors,
                            _moz_border_left_colors: _moz_border_left_colors,
                            _moz_border_bottom_colors: _moz_border_bottom_colors,
                            _moz_border_right_colors: _moz_border_right_colors,
                    }),
                    _ => Err(())
                }
            }
        }

        /// Parse the given shorthand and fill the result into the
        /// `declarations` vector.
        pub fn parse_into<'i, 't>(declarations: &mut SourcePropertyDeclaration,
                                  context: &ParserContext, input: &mut Parser<'i, 't>)
                                  -> Result<(), ParseError<'i>> {
            input.parse_entirely(|input| parse_value(context, input)).map(|longhands| {
                    declarations.push(PropertyDeclaration::BorderTopColor(
                        longhands.border_top_color
                    ));
                    declarations.push(PropertyDeclaration::BorderTopStyle(
                        longhands.border_top_style
                    ));
                    declarations.push(PropertyDeclaration::BorderTopWidth(
                        longhands.border_top_width
                    ));
                    declarations.push(PropertyDeclaration::BorderLeftColor(
                        longhands.border_left_color
                    ));
                    declarations.push(PropertyDeclaration::BorderLeftStyle(
                        longhands.border_left_style
                    ));
                    declarations.push(PropertyDeclaration::BorderLeftWidth(
                        longhands.border_left_width
                    ));
                    declarations.push(PropertyDeclaration::BorderBottomColor(
                        longhands.border_bottom_color
                    ));
                    declarations.push(PropertyDeclaration::BorderBottomStyle(
                        longhands.border_bottom_style
                    ));
                    declarations.push(PropertyDeclaration::BorderBottomWidth(
                        longhands.border_bottom_width
                    ));
                    declarations.push(PropertyDeclaration::BorderRightColor(
                        longhands.border_right_color
                    ));
                    declarations.push(PropertyDeclaration::BorderRightStyle(
                        longhands.border_right_style
                    ));
                    declarations.push(PropertyDeclaration::BorderRightWidth(
                        longhands.border_right_width
                    ));
                    declarations.push(PropertyDeclaration::BorderImageOutset(
                        longhands.border_image_outset
                    ));
                    declarations.push(PropertyDeclaration::BorderImageRepeat(
                        longhands.border_image_repeat
                    ));
                    declarations.push(PropertyDeclaration::BorderImageSlice(
                        longhands.border_image_slice
                    ));
                    declarations.push(PropertyDeclaration::BorderImageSource(
                        longhands.border_image_source
                    ));
                    declarations.push(PropertyDeclaration::BorderImageWidth(
                        longhands.border_image_width
                    ));
                    declarations.push(PropertyDeclaration::MozBorderTopColors(
                        longhands._moz_border_top_colors
                    ));
                    declarations.push(PropertyDeclaration::MozBorderLeftColors(
                        longhands._moz_border_left_colors
                    ));
                    declarations.push(PropertyDeclaration::MozBorderBottomColors(
                        longhands._moz_border_bottom_colors
                    ));
                    declarations.push(PropertyDeclaration::MozBorderRightColors(
                        longhands._moz_border_right_colors
                    ));
            })
        }

        

        use properties::longhands::{_moz_border_top_colors, _moz_border_right_colors,
                                    _moz_border_bottom_colors, _moz_border_left_colors};

    pub fn parse_value<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                               -> Result<Longhands, ParseError<'i>> {
        use properties::longhands::{border_image_outset, border_image_repeat, border_image_slice};
        use properties::longhands::{border_image_source, border_image_width};

        let (color, style, width) = super::parse_border(context, input)?;
        Ok(expanded! {
                border_top_color: color.clone(),
                border_top_style: style,
                border_top_width: width.clone(),
                    _moz_border_top_colors: _moz_border_top_colors::get_initial_specified_value(),
                border_left_color: color.clone(),
                border_left_style: style,
                border_left_width: width.clone(),
                    _moz_border_left_colors: _moz_border_left_colors::get_initial_specified_value(),
                border_bottom_color: color.clone(),
                border_bottom_style: style,
                border_bottom_width: width.clone(),
                    _moz_border_bottom_colors: _moz_border_bottom_colors::get_initial_specified_value(),
                border_right_color: color.clone(),
                border_right_style: style,
                border_right_width: width.clone(),
                    _moz_border_right_colors: _moz_border_right_colors::get_initial_specified_value(),

            // The ‘border’ shorthand resets ‘border-image’ to its initial value.
            // See https://drafts.csswg.org/css-backgrounds-3/#the-border-shorthands
                border_image_outset: border_image_outset::get_initial_specified_value(),
                border_image_repeat: border_image_repeat::get_initial_specified_value(),
                border_image_slice: border_image_slice::get_initial_specified_value(),
                border_image_source: border_image_source::get_initial_specified_value(),
                border_image_width: border_image_width::get_initial_specified_value(),
        })
    }

    impl<'a> ToCss for LonghandsToSerialize<'a>  {
        fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
            let all_equal = {
                  let border_top_width = self.border_top_width;
                  let border_top_style = self.border_top_style;
                  let border_top_color = self.border_top_color;
                  let border_left_width = self.border_left_width;
                  let border_left_style = self.border_left_style;
                  let border_left_color = self.border_left_color;
                  let border_bottom_width = self.border_bottom_width;
                  let border_bottom_style = self.border_bottom_style;
                  let border_bottom_color = self.border_bottom_color;
                  let border_right_width = self.border_right_width;
                  let border_right_style = self.border_right_style;
                  let border_right_color = self.border_right_color;

                border_top_width == border_right_width &&
                border_right_width == border_bottom_width &&
                border_bottom_width == border_left_width &&

                border_top_style == border_right_style &&
                border_right_style == border_bottom_style &&
                border_bottom_style == border_left_style &&

                border_top_color == border_right_color &&
                border_right_color == border_bottom_color &&
                border_bottom_color == border_left_color
            };

            // If all longhands are all present, then all sides should be the same,
            // so we can just one set of color/style/width
            if all_equal {
                super::serialize_directional_border(
                    dest,
                    self.border_right_width,
                    self.border_right_style,
                    self.border_right_color
                )
            } else {
                Ok(())
            }
        }
    }


    }




    /// https://drafts.csswg.org/css-backgrounds/#border-radius
    pub mod border_radius {
        use cssparser::Parser;
        use parser::ParserContext;
        use properties::{PropertyDeclaration, SourcePropertyDeclaration, MaybeBoxed, longhands};
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use std::fmt;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use style_traits::ToCss;

        pub struct Longhands {
                pub border_top_left_radius:
                        Box<
                    longhands::border_top_left_radius::SpecifiedValue
                        >
                    ,
                pub border_top_right_radius:
                        Box<
                    longhands::border_top_right_radius::SpecifiedValue
                        >
                    ,
                pub border_bottom_right_radius:
                        Box<
                    longhands::border_bottom_right_radius::SpecifiedValue
                        >
                    ,
                pub border_bottom_left_radius:
                        Box<
                    longhands::border_bottom_left_radius::SpecifiedValue
                        >
                    ,
        }

        /// Represents a serializable set of all of the longhand properties that
        /// correspond to a shorthand.
        pub struct LonghandsToSerialize<'a> {
                pub border_top_left_radius:
                    &'a longhands::border_top_left_radius::SpecifiedValue,
                pub border_top_right_radius:
                    &'a longhands::border_top_right_radius::SpecifiedValue,
                pub border_bottom_right_radius:
                    &'a longhands::border_bottom_right_radius::SpecifiedValue,
                pub border_bottom_left_radius:
                    &'a longhands::border_bottom_left_radius::SpecifiedValue,
        }

        impl<'a> LonghandsToSerialize<'a> {
            /// Tries to get a serializable set of longhands given a set of
            /// property declarations.
            pub fn from_iter<I>(iter: I) -> Result<Self, ()>
                where I: Iterator<Item=&'a PropertyDeclaration>,
            {
                // Define all of the expected variables that correspond to the shorthand
                    let mut border_top_left_radius = None;
                    let mut border_top_right_radius = None;
                    let mut border_bottom_right_radius = None;
                    let mut border_bottom_left_radius = None;

                // Attempt to assign the incoming declarations to the expected variables
                for longhand in iter {
                    match *longhand {
                            PropertyDeclaration::BorderTopLeftRadius(ref value) => {
                                border_top_left_radius = Some(value)
                            },
                            PropertyDeclaration::BorderTopRightRadius(ref value) => {
                                border_top_right_radius = Some(value)
                            },
                            PropertyDeclaration::BorderBottomRightRadius(ref value) => {
                                border_bottom_right_radius = Some(value)
                            },
                            PropertyDeclaration::BorderBottomLeftRadius(ref value) => {
                                border_bottom_left_radius = Some(value)
                            },
                        _ => {}
                    };
                }

                // If any of the expected variables are missing, return an error
                match (
                        border_top_left_radius,
                        border_top_right_radius,
                        border_bottom_right_radius,
                        border_bottom_left_radius,
                ) {

                    (
                        Some(border_top_left_radius),
                        Some(border_top_right_radius),
                        Some(border_bottom_right_radius),
                        Some(border_bottom_left_radius),
                    ) =>
                    Ok(LonghandsToSerialize {
                            border_top_left_radius: border_top_left_radius,
                            border_top_right_radius: border_top_right_radius,
                            border_bottom_right_radius: border_bottom_right_radius,
                            border_bottom_left_radius: border_bottom_left_radius,
                    }),
                    _ => Err(())
                }
            }
        }

        /// Parse the given shorthand and fill the result into the
        /// `declarations` vector.
        pub fn parse_into<'i, 't>(declarations: &mut SourcePropertyDeclaration,
                                  context: &ParserContext, input: &mut Parser<'i, 't>)
                                  -> Result<(), ParseError<'i>> {
            input.parse_entirely(|input| parse_value(context, input)).map(|longhands| {
                    declarations.push(PropertyDeclaration::BorderTopLeftRadius(
                        longhands.border_top_left_radius
                    ));
                    declarations.push(PropertyDeclaration::BorderTopRightRadius(
                        longhands.border_top_right_radius
                    ));
                    declarations.push(PropertyDeclaration::BorderBottomRightRadius(
                        longhands.border_bottom_right_radius
                    ));
                    declarations.push(PropertyDeclaration::BorderBottomLeftRadius(
                        longhands.border_bottom_left_radius
                    ));
            })
        }

        
    use values::generics::rect::Rect;
    use values::generics::border::BorderCornerRadius;
    use values::specified::border::BorderRadius;
    use parser::Parse;

    pub fn parse_value<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                               -> Result<Longhands, ParseError<'i>> {
        let radii = BorderRadius::parse(context, input)?;
        Ok(expanded! {
            border_top_left_radius: radii.top_left,
            border_top_right_radius: radii.top_right,
            border_bottom_right_radius: radii.bottom_right,
            border_bottom_left_radius: radii.bottom_left,
        })
    }

    impl<'a> ToCss for LonghandsToSerialize<'a>  {
        fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
            let LonghandsToSerialize {
                border_top_left_radius: &BorderCornerRadius(ref tl),
                border_top_right_radius: &BorderCornerRadius(ref tr),
                border_bottom_right_radius: &BorderCornerRadius(ref br),
                border_bottom_left_radius: &BorderCornerRadius(ref bl),
            } = *self;


            let widths = Rect::new(tl.width(), tr.width(), br.width(), bl.width());
            let heights = Rect::new(tl.height(), tr.height(), br.height(), bl.height());

            BorderRadius::serialize_rects(widths, heights, dest)
        }
    }

    }




    /// https://drafts.csswg.org/css-backgrounds-3/#border-image
    pub mod border_image {
        use cssparser::Parser;
        use parser::ParserContext;
        use properties::{PropertyDeclaration, SourcePropertyDeclaration, MaybeBoxed, longhands};
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use std::fmt;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use style_traits::ToCss;

        pub struct Longhands {
                pub border_image_outset:
                        Box<
                    longhands::border_image_outset::SpecifiedValue
                        >
                    ,
                pub border_image_repeat:
                    longhands::border_image_repeat::SpecifiedValue
                    ,
                pub border_image_slice:
                        Box<
                    longhands::border_image_slice::SpecifiedValue
                        >
                    ,
                pub border_image_source:
                        Box<
                    longhands::border_image_source::SpecifiedValue
                        >
                    ,
                pub border_image_width:
                        Box<
                    longhands::border_image_width::SpecifiedValue
                        >
                    ,
        }

        /// Represents a serializable set of all of the longhand properties that
        /// correspond to a shorthand.
        pub struct LonghandsToSerialize<'a> {
                pub border_image_outset:
                    &'a longhands::border_image_outset::SpecifiedValue,
                pub border_image_repeat:
                    &'a longhands::border_image_repeat::SpecifiedValue,
                pub border_image_slice:
                    &'a longhands::border_image_slice::SpecifiedValue,
                pub border_image_source:
                    &'a longhands::border_image_source::SpecifiedValue,
                pub border_image_width:
                    &'a longhands::border_image_width::SpecifiedValue,
        }

        impl<'a> LonghandsToSerialize<'a> {
            /// Tries to get a serializable set of longhands given a set of
            /// property declarations.
            pub fn from_iter<I>(iter: I) -> Result<Self, ()>
                where I: Iterator<Item=&'a PropertyDeclaration>,
            {
                // Define all of the expected variables that correspond to the shorthand
                    let mut border_image_outset = None;
                    let mut border_image_repeat = None;
                    let mut border_image_slice = None;
                    let mut border_image_source = None;
                    let mut border_image_width = None;

                // Attempt to assign the incoming declarations to the expected variables
                for longhand in iter {
                    match *longhand {
                            PropertyDeclaration::BorderImageOutset(ref value) => {
                                border_image_outset = Some(value)
                            },
                            PropertyDeclaration::BorderImageRepeat(ref value) => {
                                border_image_repeat = Some(value)
                            },
                            PropertyDeclaration::BorderImageSlice(ref value) => {
                                border_image_slice = Some(value)
                            },
                            PropertyDeclaration::BorderImageSource(ref value) => {
                                border_image_source = Some(value)
                            },
                            PropertyDeclaration::BorderImageWidth(ref value) => {
                                border_image_width = Some(value)
                            },
                        _ => {}
                    };
                }

                // If any of the expected variables are missing, return an error
                match (
                        border_image_outset,
                        border_image_repeat,
                        border_image_slice,
                        border_image_source,
                        border_image_width,
                ) {

                    (
                        Some(border_image_outset),
                        Some(border_image_repeat),
                        Some(border_image_slice),
                        Some(border_image_source),
                        Some(border_image_width),
                    ) =>
                    Ok(LonghandsToSerialize {
                            border_image_outset: border_image_outset,
                            border_image_repeat: border_image_repeat,
                            border_image_slice: border_image_slice,
                            border_image_source: border_image_source,
                            border_image_width: border_image_width,
                    }),
                    _ => Err(())
                }
            }
        }

        /// Parse the given shorthand and fill the result into the
        /// `declarations` vector.
        pub fn parse_into<'i, 't>(declarations: &mut SourcePropertyDeclaration,
                                  context: &ParserContext, input: &mut Parser<'i, 't>)
                                  -> Result<(), ParseError<'i>> {
            input.parse_entirely(|input| parse_value(context, input)).map(|longhands| {
                    declarations.push(PropertyDeclaration::BorderImageOutset(
                        longhands.border_image_outset
                    ));
                    declarations.push(PropertyDeclaration::BorderImageRepeat(
                        longhands.border_image_repeat
                    ));
                    declarations.push(PropertyDeclaration::BorderImageSlice(
                        longhands.border_image_slice
                    ));
                    declarations.push(PropertyDeclaration::BorderImageSource(
                        longhands.border_image_source
                    ));
                    declarations.push(PropertyDeclaration::BorderImageWidth(
                        longhands.border_image_width
                    ));
            })
        }

        
    use properties::longhands::{border_image_outset, border_image_repeat, border_image_slice};
    use properties::longhands::{border_image_source, border_image_width};

    pub fn parse_value<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                               -> Result<Longhands, ParseError<'i>> {
            let mut border_image_outset = border_image_outset::get_initial_specified_value();
            let mut border_image_repeat = border_image_repeat::get_initial_specified_value();
            let mut border_image_slice = border_image_slice::get_initial_specified_value();
            let mut border_image_source = border_image_source::get_initial_specified_value();
            let mut border_image_width = border_image_width::get_initial_specified_value();

        let result: Result<_, ParseError> = input.try(|input| {
                let mut outset = None;
                let mut repeat = None;
                let mut slice = None;
                let mut source = None;
                let mut width = None;
            loop {
                if slice.is_none() {
                    if let Ok(value) = input.try(|input| border_image_slice::parse(context, input)) {
                        slice = Some(value);
                        // Parse border image width and outset, if applicable.
                        let maybe_width_outset: Result<_, ParseError> = input.try(|input| {
                            input.expect_delim('/')?;

                            // Parse border image width, if applicable.
                            let w = input.try(|input|
                                border_image_width::parse(context, input)).ok();

                            // Parse border image outset if applicable.
                            let o = input.try(|input| {
                                input.expect_delim('/')?;
                                border_image_outset::parse(context, input)
                            }).ok();
                            if w.is_none() && o.is_none() {
                               Err(StyleParseError::UnspecifiedError.into())
                            }
                            else {
                               Ok((w, o))
                            }
                        });
                        if let Ok((w, o)) = maybe_width_outset {
                            width = w;
                            outset = o;
                        }

                        continue
                    }
                }
                    if source.is_none() {
                        if let Ok(value) = input.try(|input| border_image_source::parse(context, input)) {
                            source = Some(value);
                            continue
                        }
                    }
                    if repeat.is_none() {
                        if let Ok(value) = input.try(|input| border_image_repeat::parse(context, input)) {
                            repeat = Some(value);
                            continue
                        }
                    }
                break
            }
            let mut any = false;
                any = any || outset.is_some();
                any = any || repeat.is_some();
                any = any || slice.is_some();
                any = any || source.is_some();
                any = any || width.is_some();
            if any {
                    if let Some(b_outset) = outset {
                        border_image_outset = b_outset;
                    }
                    if let Some(b_repeat) = repeat {
                        border_image_repeat = b_repeat;
                    }
                    if let Some(b_slice) = slice {
                        border_image_slice = b_slice;
                    }
                    if let Some(b_source) = source {
                        border_image_source = b_source;
                    }
                    if let Some(b_width) = width {
                        border_image_width = b_width;
                    }
                Ok(())
            } else {
                Err(StyleParseError::UnspecifiedError.into())
            }
        });
        result?;

        Ok(expanded! {
                border_image_outset: border_image_outset,
                border_image_repeat: border_image_repeat,
                border_image_slice: border_image_slice,
                border_image_source: border_image_source,
                border_image_width: border_image_width,
         })
    }

    impl<'a> ToCss for LonghandsToSerialize<'a>  {
        fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
            self.border_image_source.to_css(dest)?;
            dest.write_str(" ")?;
            self.border_image_slice.to_css(dest)?;
            dest.write_str(" / ")?;
            self.border_image_width.to_css(dest)?;
            dest.write_str(" / ")?;
            self.border_image_outset.to_css(dest)?;
            dest.write_str(" ")?;
            self.border_image_repeat.to_css(dest)
        }
    }

    }


    /* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */





    /// https://drafts.csswg.org/css-overflow/#propdef-overflow
    pub mod overflow {
        use cssparser::Parser;
        use parser::ParserContext;
        use properties::{PropertyDeclaration, SourcePropertyDeclaration, MaybeBoxed, longhands};
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use std::fmt;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use style_traits::ToCss;

        pub struct Longhands {
                pub overflow_x:
                    longhands::overflow_x::SpecifiedValue
                    ,
                pub overflow_y:
                    longhands::overflow_y::SpecifiedValue
                    ,
        }

        /// Represents a serializable set of all of the longhand properties that
        /// correspond to a shorthand.
        pub struct LonghandsToSerialize<'a> {
                pub overflow_x:
                    &'a longhands::overflow_x::SpecifiedValue,
                pub overflow_y:
                    &'a longhands::overflow_y::SpecifiedValue,
        }

        impl<'a> LonghandsToSerialize<'a> {
            /// Tries to get a serializable set of longhands given a set of
            /// property declarations.
            pub fn from_iter<I>(iter: I) -> Result<Self, ()>
                where I: Iterator<Item=&'a PropertyDeclaration>,
            {
                // Define all of the expected variables that correspond to the shorthand
                    let mut overflow_x = None;
                    let mut overflow_y = None;

                // Attempt to assign the incoming declarations to the expected variables
                for longhand in iter {
                    match *longhand {
                            PropertyDeclaration::OverflowX(ref value) => {
                                overflow_x = Some(value)
                            },
                            PropertyDeclaration::OverflowY(ref value) => {
                                overflow_y = Some(value)
                            },
                        _ => {}
                    };
                }

                // If any of the expected variables are missing, return an error
                match (
                        overflow_x,
                        overflow_y,
                ) {

                    (
                        Some(overflow_x),
                        Some(overflow_y),
                    ) =>
                    Ok(LonghandsToSerialize {
                            overflow_x: overflow_x,
                            overflow_y: overflow_y,
                    }),
                    _ => Err(())
                }
            }
        }

        /// Parse the given shorthand and fill the result into the
        /// `declarations` vector.
        pub fn parse_into<'i, 't>(declarations: &mut SourcePropertyDeclaration,
                                  context: &ParserContext, input: &mut Parser<'i, 't>)
                                  -> Result<(), ParseError<'i>> {
            input.parse_entirely(|input| parse_value(context, input)).map(|longhands| {
                    declarations.push(PropertyDeclaration::OverflowX(
                        longhands.overflow_x
                    ));
                    declarations.push(PropertyDeclaration::OverflowY(
                        longhands.overflow_y
                    ));
            })
        }

        
    use properties::longhands::overflow_x::parse as parse_overflow;
        use properties::longhands::overflow_x::SpecifiedValue;

    pub fn parse_value<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                               -> Result<Longhands, ParseError<'i>> {
            let moz_kw_found = input.try(|i| {
                try_match_ident_ignore_ascii_case! { i.expect_ident()?,
                    "-moz-scrollbars-horizontal" => {
                        Ok(expanded! {
                            overflow_x: SpecifiedValue::scroll,
                            overflow_y: SpecifiedValue::hidden,
                        })
                    }
                    "-moz-scrollbars-vertical" => {
                        Ok(expanded! {
                            overflow_x: SpecifiedValue::hidden,
                            overflow_y: SpecifiedValue::scroll,
                        })
                    }
                    "-moz-scrollbars-none" => {
                        Ok(expanded! {
                            overflow_x: SpecifiedValue::hidden,
                            overflow_y: SpecifiedValue::hidden,
                        })
                    }
                }
            });
            if moz_kw_found.is_ok() {
                return moz_kw_found
            }
        let overflow = parse_overflow(context, input)?;
        Ok(expanded! {
            overflow_x: overflow,
            overflow_y: overflow,
        })
    }

    impl<'a> ToCss for LonghandsToSerialize<'a>  {
        fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
            if self.overflow_x == self.overflow_y {
                self.overflow_x.to_css(dest)
            } else {
                Ok(())
            }
        }
    }

    }


macro_rules! try_parse_one {
    ($input: expr, $var: ident, $prop_module: ident) => {
        if $var.is_none() {
            if let Ok(value) = $input.try($prop_module::SingleSpecifiedValue::parse) {
                $var = Some(value);
                continue;
            }
        }
    };
    ($context: expr, $input: expr, $var: ident, $prop_module: ident) => {
        if $var.is_none() {
            if let Ok(value) = $input.try(|i| {
                $prop_module::SingleSpecifiedValue::parse($context, i)
            }) {
                $var = Some(value);
                continue;
            }
        }
    };
}



    /// https://drafts.csswg.org/css-transitions/#propdef-transition
    pub mod transition {
        use cssparser::Parser;
        use parser::ParserContext;
        use properties::{PropertyDeclaration, SourcePropertyDeclaration, MaybeBoxed, longhands};
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use std::fmt;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use style_traits::ToCss;

        pub struct Longhands {
                pub transition_property:
                    longhands::transition_property::SpecifiedValue
                    ,
                pub transition_duration:
                    longhands::transition_duration::SpecifiedValue
                    ,
                pub transition_timing_function:
                    longhands::transition_timing_function::SpecifiedValue
                    ,
                pub transition_delay:
                    longhands::transition_delay::SpecifiedValue
                    ,
        }

        /// Represents a serializable set of all of the longhand properties that
        /// correspond to a shorthand.
        pub struct LonghandsToSerialize<'a> {
                pub transition_property:
                    &'a longhands::transition_property::SpecifiedValue,
                pub transition_duration:
                    &'a longhands::transition_duration::SpecifiedValue,
                pub transition_timing_function:
                    &'a longhands::transition_timing_function::SpecifiedValue,
                pub transition_delay:
                    &'a longhands::transition_delay::SpecifiedValue,
        }

        impl<'a> LonghandsToSerialize<'a> {
            /// Tries to get a serializable set of longhands given a set of
            /// property declarations.
            pub fn from_iter<I>(iter: I) -> Result<Self, ()>
                where I: Iterator<Item=&'a PropertyDeclaration>,
            {
                // Define all of the expected variables that correspond to the shorthand
                    let mut transition_property = None;
                    let mut transition_duration = None;
                    let mut transition_timing_function = None;
                    let mut transition_delay = None;

                // Attempt to assign the incoming declarations to the expected variables
                for longhand in iter {
                    match *longhand {
                            PropertyDeclaration::TransitionProperty(ref value) => {
                                transition_property = Some(value)
                            },
                            PropertyDeclaration::TransitionDuration(ref value) => {
                                transition_duration = Some(value)
                            },
                            PropertyDeclaration::TransitionTimingFunction(ref value) => {
                                transition_timing_function = Some(value)
                            },
                            PropertyDeclaration::TransitionDelay(ref value) => {
                                transition_delay = Some(value)
                            },
                        _ => {}
                    };
                }

                // If any of the expected variables are missing, return an error
                match (
                        transition_property,
                        transition_duration,
                        transition_timing_function,
                        transition_delay,
                ) {

                    (
                        Some(transition_property),
                        Some(transition_duration),
                        Some(transition_timing_function),
                        Some(transition_delay),
                    ) =>
                    Ok(LonghandsToSerialize {
                            transition_property: transition_property,
                            transition_duration: transition_duration,
                            transition_timing_function: transition_timing_function,
                            transition_delay: transition_delay,
                    }),
                    _ => Err(())
                }
            }
        }

        /// Parse the given shorthand and fill the result into the
        /// `declarations` vector.
        pub fn parse_into<'i, 't>(declarations: &mut SourcePropertyDeclaration,
                                  context: &ParserContext, input: &mut Parser<'i, 't>)
                                  -> Result<(), ParseError<'i>> {
            input.parse_entirely(|input| parse_value(context, input)).map(|longhands| {
                    declarations.push(PropertyDeclaration::TransitionProperty(
                        longhands.transition_property
                    ));
                    declarations.push(PropertyDeclaration::TransitionDuration(
                        longhands.transition_duration
                    ));
                    declarations.push(PropertyDeclaration::TransitionTimingFunction(
                        longhands.transition_timing_function
                    ));
                    declarations.push(PropertyDeclaration::TransitionDelay(
                        longhands.transition_delay
                    ));
            })
        }

        
    use parser::Parse;
    use properties::longhands::transition_delay;
    use properties::longhands::transition_duration;
    use properties::longhands::transition_property;
    use properties::longhands::transition_timing_function;

    pub fn parse_value<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                               -> Result<Longhands, ParseError<'i>> {
        struct SingleTransition {
            transition_duration: transition_duration::SingleSpecifiedValue,
            transition_timing_function: transition_timing_function::SingleSpecifiedValue,
            transition_delay: transition_delay::SingleSpecifiedValue,
            // Unlike other properties, transition-property uses an Option<> to
            // represent 'none' as `None`.
            transition_property: Option<transition_property::SingleSpecifiedValue>,
        }

        fn parse_one_transition<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                        -> Result<SingleTransition,ParseError<'i>> {
            let mut property = None;
            let mut duration = None;
            let mut timing_function = None;
            let mut delay = None;

            let mut parsed = 0;
            loop {
                parsed += 1;

                try_parse_one!(context, input, duration, transition_duration);
                try_parse_one!(context, input, timing_function, transition_timing_function);
                try_parse_one!(context, input, delay, transition_delay);
                // Must check 'transition-property' after 'transition-timing-function' since
                // 'transition-property' accepts any keyword.
                if property.is_none() {
                    if let Ok(value) = input.try(|i| transition_property::SingleSpecifiedValue::parse(i)) {
                        property = Some(Some(value));
                        continue;
                    } else if input.try(|i| i.expect_ident_matching("none")).is_ok() {
                        // 'none' is not a valid value for <single-transition-property>,
                        // so it's not acceptable in the function above.
                        property = Some(None);
                        continue;
                    }
                }

                parsed -= 1;
                break
            }

            if parsed != 0 {
                Ok(SingleTransition {
                    transition_duration: duration.unwrap_or_else(transition_duration::single_value
                                                                                 ::get_initial_specified_value),
                    transition_timing_function: timing_function.unwrap_or_else(transition_timing_function::single_value
                                                                                 ::get_initial_specified_value),
                    transition_delay: delay.unwrap_or_else(transition_delay::single_value
                                                                                 ::get_initial_specified_value),
                    transition_property: property.unwrap_or(
                        Some(transition_property::single_value::get_initial_specified_value())),
                })
            } else {
                Err(StyleParseError::UnspecifiedError.into())
            }
        }

        let mut propertys = Vec::new();
        let mut durations = Vec::new();
        let mut timing_functions = Vec::new();
        let mut delays = Vec::new();

        let results = input.parse_comma_separated(|i| parse_one_transition(context, i))?;
        let multiple_items = results.len() >= 2;
        for result in results {
            if let Some(value) = result.transition_property {
                propertys.push(value);
            } else if multiple_items {
                // If there is more than one item, and any of transitions has 'none',
                // then it's invalid. Othersize, leave propertys to be empty (which
                // means "transition-property: none");
                return Err(StyleParseError::UnspecifiedError.into());
            }

            durations.push(result.transition_duration);
            timing_functions.push(result.transition_timing_function);
            delays.push(result.transition_delay);
        }

        Ok(expanded! {
            transition_property: transition_property::SpecifiedValue(propertys),
            transition_duration: transition_duration::SpecifiedValue(durations),
            transition_timing_function: transition_timing_function::SpecifiedValue(timing_functions),
            transition_delay: transition_delay::SpecifiedValue(delays),
        })
    }

    impl<'a> ToCss for LonghandsToSerialize<'a>  {
        fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
            let property_len = self.transition_property.0.len();

            // There are two cases that we can do shorthand serialization:
            // * when all value lists have the same length, or
            // * when transition-property is none, and other value lists have exactly one item.
            if property_len == 0 {
                    if self.transition_duration.0.len() != 1 {
                        return Ok(());
                    }
                    if self.transition_delay.0.len() != 1 {
                        return Ok(());
                    }
                    if self.transition_timing_function.0.len() != 1 {
                        return Ok(());
                    }
            } else {
                    if self.transition_duration.0.len() != property_len {
                        return Ok(());
                    }
                    if self.transition_delay.0.len() != property_len {
                        return Ok(());
                    }
                    if self.transition_timing_function.0.len() != property_len {
                        return Ok(());
                    }
            }

            // Representative length.
            let len = self.transition_duration.0.len();

            for i in 0..len {
                if i != 0 {
                    dest.write_str(", ")?;
                }
                if property_len == 0 {
                    dest.write_str("none")?;
                } else {
                    self.transition_property.0[i].to_css(dest)?;
                }
                    dest.write_str(" ")?;
                    self.transition_duration.0[i].to_css(dest)?;
                    dest.write_str(" ")?;
                    self.transition_timing_function.0[i].to_css(dest)?;
                    dest.write_str(" ")?;
                    self.transition_delay.0[i].to_css(dest)?;
            }
            Ok(())
        }
    }

    }




    /// https://drafts.csswg.org/css-animations/#propdef-animation
    pub mod animation {
        use cssparser::Parser;
        use parser::ParserContext;
        use properties::{PropertyDeclaration, SourcePropertyDeclaration, MaybeBoxed, longhands};
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use std::fmt;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use style_traits::ToCss;

        pub struct Longhands {
                pub animation_name:
                    longhands::animation_name::SpecifiedValue
                    ,
                pub animation_duration:
                    longhands::animation_duration::SpecifiedValue
                    ,
                pub animation_timing_function:
                    longhands::animation_timing_function::SpecifiedValue
                    ,
                pub animation_delay:
                    longhands::animation_delay::SpecifiedValue
                    ,
                pub animation_iteration_count:
                    longhands::animation_iteration_count::SpecifiedValue
                    ,
                pub animation_direction:
                    longhands::animation_direction::SpecifiedValue
                    ,
                pub animation_fill_mode:
                    longhands::animation_fill_mode::SpecifiedValue
                    ,
                pub animation_play_state:
                    longhands::animation_play_state::SpecifiedValue
                    ,
        }

        /// Represents a serializable set of all of the longhand properties that
        /// correspond to a shorthand.
        pub struct LonghandsToSerialize<'a> {
                pub animation_name:
                    &'a longhands::animation_name::SpecifiedValue,
                pub animation_duration:
                    &'a longhands::animation_duration::SpecifiedValue,
                pub animation_timing_function:
                    &'a longhands::animation_timing_function::SpecifiedValue,
                pub animation_delay:
                    &'a longhands::animation_delay::SpecifiedValue,
                pub animation_iteration_count:
                    &'a longhands::animation_iteration_count::SpecifiedValue,
                pub animation_direction:
                    &'a longhands::animation_direction::SpecifiedValue,
                pub animation_fill_mode:
                    &'a longhands::animation_fill_mode::SpecifiedValue,
                pub animation_play_state:
                    &'a longhands::animation_play_state::SpecifiedValue,
        }

        impl<'a> LonghandsToSerialize<'a> {
            /// Tries to get a serializable set of longhands given a set of
            /// property declarations.
            pub fn from_iter<I>(iter: I) -> Result<Self, ()>
                where I: Iterator<Item=&'a PropertyDeclaration>,
            {
                // Define all of the expected variables that correspond to the shorthand
                    let mut animation_name = None;
                    let mut animation_duration = None;
                    let mut animation_timing_function = None;
                    let mut animation_delay = None;
                    let mut animation_iteration_count = None;
                    let mut animation_direction = None;
                    let mut animation_fill_mode = None;
                    let mut animation_play_state = None;

                // Attempt to assign the incoming declarations to the expected variables
                for longhand in iter {
                    match *longhand {
                            PropertyDeclaration::AnimationName(ref value) => {
                                animation_name = Some(value)
                            },
                            PropertyDeclaration::AnimationDuration(ref value) => {
                                animation_duration = Some(value)
                            },
                            PropertyDeclaration::AnimationTimingFunction(ref value) => {
                                animation_timing_function = Some(value)
                            },
                            PropertyDeclaration::AnimationDelay(ref value) => {
                                animation_delay = Some(value)
                            },
                            PropertyDeclaration::AnimationIterationCount(ref value) => {
                                animation_iteration_count = Some(value)
                            },
                            PropertyDeclaration::AnimationDirection(ref value) => {
                                animation_direction = Some(value)
                            },
                            PropertyDeclaration::AnimationFillMode(ref value) => {
                                animation_fill_mode = Some(value)
                            },
                            PropertyDeclaration::AnimationPlayState(ref value) => {
                                animation_play_state = Some(value)
                            },
                        _ => {}
                    };
                }

                // If any of the expected variables are missing, return an error
                match (
                        animation_name,
                        animation_duration,
                        animation_timing_function,
                        animation_delay,
                        animation_iteration_count,
                        animation_direction,
                        animation_fill_mode,
                        animation_play_state,
                ) {

                    (
                        Some(animation_name),
                        Some(animation_duration),
                        Some(animation_timing_function),
                        Some(animation_delay),
                        Some(animation_iteration_count),
                        Some(animation_direction),
                        Some(animation_fill_mode),
                        Some(animation_play_state),
                    ) =>
                    Ok(LonghandsToSerialize {
                            animation_name: animation_name,
                            animation_duration: animation_duration,
                            animation_timing_function: animation_timing_function,
                            animation_delay: animation_delay,
                            animation_iteration_count: animation_iteration_count,
                            animation_direction: animation_direction,
                            animation_fill_mode: animation_fill_mode,
                            animation_play_state: animation_play_state,
                    }),
                    _ => Err(())
                }
            }
        }

        /// Parse the given shorthand and fill the result into the
        /// `declarations` vector.
        pub fn parse_into<'i, 't>(declarations: &mut SourcePropertyDeclaration,
                                  context: &ParserContext, input: &mut Parser<'i, 't>)
                                  -> Result<(), ParseError<'i>> {
            input.parse_entirely(|input| parse_value(context, input)).map(|longhands| {
                    declarations.push(PropertyDeclaration::AnimationName(
                        longhands.animation_name
                    ));
                    declarations.push(PropertyDeclaration::AnimationDuration(
                        longhands.animation_duration
                    ));
                    declarations.push(PropertyDeclaration::AnimationTimingFunction(
                        longhands.animation_timing_function
                    ));
                    declarations.push(PropertyDeclaration::AnimationDelay(
                        longhands.animation_delay
                    ));
                    declarations.push(PropertyDeclaration::AnimationIterationCount(
                        longhands.animation_iteration_count
                    ));
                    declarations.push(PropertyDeclaration::AnimationDirection(
                        longhands.animation_direction
                    ));
                    declarations.push(PropertyDeclaration::AnimationFillMode(
                        longhands.animation_fill_mode
                    ));
                    declarations.push(PropertyDeclaration::AnimationPlayState(
                        longhands.animation_play_state
                    ));
            })
        }

        
    
    use parser::Parse;
    use properties::longhands::animation_name;
    use properties::longhands::animation_duration;
    use properties::longhands::animation_timing_function;
    use properties::longhands::animation_delay;
    use properties::longhands::animation_iteration_count;
    use properties::longhands::animation_direction;
    use properties::longhands::animation_fill_mode;
    use properties::longhands::animation_play_state;

    pub fn parse_value<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                               -> Result<Longhands, ParseError<'i>> {
        struct SingleAnimation {
            animation_name: animation_name::SingleSpecifiedValue,
            animation_duration: animation_duration::SingleSpecifiedValue,
            animation_timing_function: animation_timing_function::SingleSpecifiedValue,
            animation_delay: animation_delay::SingleSpecifiedValue,
            animation_iteration_count: animation_iteration_count::SingleSpecifiedValue,
            animation_direction: animation_direction::SingleSpecifiedValue,
            animation_fill_mode: animation_fill_mode::SingleSpecifiedValue,
            animation_play_state: animation_play_state::SingleSpecifiedValue,
        }

        fn parse_one_animation<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                       -> Result<SingleAnimation,ParseError<'i>> {
            let mut name = None;
            let mut duration = None;
            let mut timing_function = None;
            let mut delay = None;
            let mut iteration_count = None;
            let mut direction = None;
            let mut fill_mode = None;
            let mut play_state = None;

            let mut parsed = 0;
            // NB: Name must be the last one here so that keywords valid for other
            // longhands are not interpreted as names.
            //
            // Also, duration must be before delay, see
            // https://drafts.csswg.org/css-animations/#typedef-single-animation
            loop {
                parsed += 1;
                try_parse_one!(context, input, duration, animation_duration);
                try_parse_one!(context, input, timing_function, animation_timing_function);
                try_parse_one!(context, input, delay, animation_delay);
                try_parse_one!(context, input, iteration_count, animation_iteration_count);
                try_parse_one!(input, direction, animation_direction);
                try_parse_one!(input, fill_mode, animation_fill_mode);
                try_parse_one!(input, play_state, animation_play_state);
                try_parse_one!(context, input, name, animation_name);

                parsed -= 1;
                break
            }

            // If nothing is parsed, this is an invalid entry.
            if parsed == 0 {
                Err(StyleParseError::UnspecifiedError.into())
            } else {
                Ok(SingleAnimation {
                    animation_name: name.unwrap_or_else(animation_name::single_value
                                                              ::get_initial_specified_value),
                    animation_duration: duration.unwrap_or_else(animation_duration::single_value
                                                              ::get_initial_specified_value),
                    animation_timing_function: timing_function.unwrap_or_else(animation_timing_function::single_value
                                                              ::get_initial_specified_value),
                    animation_delay: delay.unwrap_or_else(animation_delay::single_value
                                                              ::get_initial_specified_value),
                    animation_iteration_count: iteration_count.unwrap_or_else(animation_iteration_count::single_value
                                                              ::get_initial_specified_value),
                    animation_direction: direction.unwrap_or_else(animation_direction::single_value
                                                              ::get_initial_specified_value),
                    animation_fill_mode: fill_mode.unwrap_or_else(animation_fill_mode::single_value
                                                              ::get_initial_specified_value),
                    animation_play_state: play_state.unwrap_or_else(animation_play_state::single_value
                                                              ::get_initial_specified_value),
                })
            }
        }

        let mut names = vec![];
        let mut durations = vec![];
        let mut timing_functions = vec![];
        let mut delays = vec![];
        let mut iteration_counts = vec![];
        let mut directions = vec![];
        let mut fill_modes = vec![];
        let mut play_states = vec![];

        let results = input.parse_comma_separated(|i| parse_one_animation(context, i))?;
        for result in results.into_iter() {
            names.push(result.animation_name);
            durations.push(result.animation_duration);
            timing_functions.push(result.animation_timing_function);
            delays.push(result.animation_delay);
            iteration_counts.push(result.animation_iteration_count);
            directions.push(result.animation_direction);
            fill_modes.push(result.animation_fill_mode);
            play_states.push(result.animation_play_state);
        }

        Ok(expanded! {
            animation_name: animation_name::SpecifiedValue(names),
            animation_duration: animation_duration::SpecifiedValue(durations),
            animation_timing_function: animation_timing_function::SpecifiedValue(timing_functions),
            animation_delay: animation_delay::SpecifiedValue(delays),
            animation_iteration_count: animation_iteration_count::SpecifiedValue(iteration_counts),
            animation_direction: animation_direction::SpecifiedValue(directions),
            animation_fill_mode: animation_fill_mode::SpecifiedValue(fill_modes),
            animation_play_state: animation_play_state::SpecifiedValue(play_states),
        })
    }

    impl<'a> ToCss for LonghandsToSerialize<'a>  {
        fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
            let len = self.animation_name.0.len();
            // There should be at least one declared value
            if len == 0 {
                return Ok(());
            }

            // If any value list length is differs then we don't do a shorthand serialization
            // either.
                if len != self.animation_duration.0.len() {
                    return Ok(())
                }
                if len != self.animation_timing_function.0.len() {
                    return Ok(())
                }
                if len != self.animation_delay.0.len() {
                    return Ok(())
                }
                if len != self.animation_iteration_count.0.len() {
                    return Ok(())
                }
                if len != self.animation_direction.0.len() {
                    return Ok(())
                }
                if len != self.animation_fill_mode.0.len() {
                    return Ok(())
                }
                if len != self.animation_play_state.0.len() {
                    return Ok(())
                }

            for i in 0..len {
                if i != 0 {
                    dest.write_str(", ")?;
                }

                    self.animation_duration.0[i].to_css(dest)?;
                    dest.write_str(" ")?;
                    self.animation_timing_function.0[i].to_css(dest)?;
                    dest.write_str(" ")?;
                    self.animation_delay.0[i].to_css(dest)?;
                    dest.write_str(" ")?;
                    self.animation_iteration_count.0[i].to_css(dest)?;
                    dest.write_str(" ")?;
                    self.animation_direction.0[i].to_css(dest)?;
                    dest.write_str(" ")?;
                    self.animation_fill_mode.0[i].to_css(dest)?;
                    dest.write_str(" ")?;
                    self.animation_play_state.0[i].to_css(dest)?;
                    dest.write_str(" ")?;
                self.animation_name.0[i].to_css(dest)?;
            }
            Ok(())
        }
    }

    }




    /// https://drafts.csswg.org/css-scroll-snap/#propdef-scroll-snap-type
    pub mod scroll_snap_type {
        use cssparser::Parser;
        use parser::ParserContext;
        use properties::{PropertyDeclaration, SourcePropertyDeclaration, MaybeBoxed, longhands};
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use std::fmt;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use style_traits::ToCss;

        pub struct Longhands {
                pub scroll_snap_type_x:
                    longhands::scroll_snap_type_x::SpecifiedValue
                    ,
                pub scroll_snap_type_y:
                    longhands::scroll_snap_type_y::SpecifiedValue
                    ,
        }

        /// Represents a serializable set of all of the longhand properties that
        /// correspond to a shorthand.
        pub struct LonghandsToSerialize<'a> {
                pub scroll_snap_type_x:
                    &'a longhands::scroll_snap_type_x::SpecifiedValue,
                pub scroll_snap_type_y:
                    &'a longhands::scroll_snap_type_y::SpecifiedValue,
        }

        impl<'a> LonghandsToSerialize<'a> {
            /// Tries to get a serializable set of longhands given a set of
            /// property declarations.
            pub fn from_iter<I>(iter: I) -> Result<Self, ()>
                where I: Iterator<Item=&'a PropertyDeclaration>,
            {
                // Define all of the expected variables that correspond to the shorthand
                    let mut scroll_snap_type_x = None;
                    let mut scroll_snap_type_y = None;

                // Attempt to assign the incoming declarations to the expected variables
                for longhand in iter {
                    match *longhand {
                            PropertyDeclaration::ScrollSnapTypeX(ref value) => {
                                scroll_snap_type_x = Some(value)
                            },
                            PropertyDeclaration::ScrollSnapTypeY(ref value) => {
                                scroll_snap_type_y = Some(value)
                            },
                        _ => {}
                    };
                }

                // If any of the expected variables are missing, return an error
                match (
                        scroll_snap_type_x,
                        scroll_snap_type_y,
                ) {

                    (
                        Some(scroll_snap_type_x),
                        Some(scroll_snap_type_y),
                    ) =>
                    Ok(LonghandsToSerialize {
                            scroll_snap_type_x: scroll_snap_type_x,
                            scroll_snap_type_y: scroll_snap_type_y,
                    }),
                    _ => Err(())
                }
            }
        }

        /// Parse the given shorthand and fill the result into the
        /// `declarations` vector.
        pub fn parse_into<'i, 't>(declarations: &mut SourcePropertyDeclaration,
                                  context: &ParserContext, input: &mut Parser<'i, 't>)
                                  -> Result<(), ParseError<'i>> {
            input.parse_entirely(|input| parse_value(context, input)).map(|longhands| {
                    declarations.push(PropertyDeclaration::ScrollSnapTypeX(
                        longhands.scroll_snap_type_x
                    ));
                    declarations.push(PropertyDeclaration::ScrollSnapTypeY(
                        longhands.scroll_snap_type_y
                    ));
            })
        }

        
    use properties::longhands::scroll_snap_type_x;

    pub fn parse_value<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                               -> Result<Longhands, ParseError<'i>> {
        let result = scroll_snap_type_x::parse(context, input)?;
        Ok(expanded! {
            scroll_snap_type_x: result,
            scroll_snap_type_y: result,
        })
    }

    impl<'a> ToCss for LonghandsToSerialize<'a>  {
        // Serializes into the single keyword value if both scroll-snap-type and scroll-snap-type-y are same.
        // Otherwise into an empty string. This is done to match Gecko's behaviour.
        fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
            if self.scroll_snap_type_x == self.scroll_snap_type_y {
                self.scroll_snap_type_x.to_css(dest)
            } else {
                Ok(())
            }
        }
    }

    }





    /// Non-standard: https://developer.mozilla.org/en-US/docs/Web/CSS/transform
    pub mod _moz_transform {
        use cssparser::Parser;
        use parser::ParserContext;
        use properties::{PropertyDeclaration, SourcePropertyDeclaration, MaybeBoxed, longhands};
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use std::fmt;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use style_traits::ToCss;

        pub struct Longhands {
                pub transform:
                    longhands::transform::SpecifiedValue
                    ,
        }

        /// Represents a serializable set of all of the longhand properties that
        /// correspond to a shorthand.
        #[derive(ToCss)]
        pub struct LonghandsToSerialize<'a> {
                pub transform:
                    &'a longhands::transform::SpecifiedValue,
        }

        impl<'a> LonghandsToSerialize<'a> {
            /// Tries to get a serializable set of longhands given a set of
            /// property declarations.
            pub fn from_iter<I>(iter: I) -> Result<Self, ()>
                where I: Iterator<Item=&'a PropertyDeclaration>,
            {
                // Define all of the expected variables that correspond to the shorthand
                    let mut transform = None;

                // Attempt to assign the incoming declarations to the expected variables
                for longhand in iter {
                    match *longhand {
                            PropertyDeclaration::Transform(ref value) => {
                                transform = Some(value)
                            },
                        _ => {}
                    };
                }

                // If any of the expected variables are missing, return an error
                match (
                        transform,
                ) {

                    (
                        Some(transform),
                    ) =>
                    Ok(LonghandsToSerialize {
                            transform: transform,
                    }),
                    _ => Err(())
                }
            }
        }

        /// Parse the given shorthand and fill the result into the
        /// `declarations` vector.
        pub fn parse_into<'i, 't>(declarations: &mut SourcePropertyDeclaration,
                                  context: &ParserContext, input: &mut Parser<'i, 't>)
                                  -> Result<(), ParseError<'i>> {
            input.parse_entirely(|input| parse_value(context, input)).map(|longhands| {
                    declarations.push(PropertyDeclaration::Transform(
                        longhands.transform
                    ));
            })
        }

        
    use properties::longhands::transform;

    pub fn parse_value<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                               -> Result<Longhands, ParseError<'i>> {
        Ok(expanded! {
            transform: transform::parse_prefixed(context, input)?,
        })
    }

    }


    /* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */





    /// https://drafts.csswg.org/css-multicol/#propdef-columns
    pub mod columns {
        use cssparser::Parser;
        use parser::ParserContext;
        use properties::{PropertyDeclaration, SourcePropertyDeclaration, MaybeBoxed, longhands};
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use std::fmt;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use style_traits::ToCss;

        pub struct Longhands {
                pub column_width:
                    longhands::column_width::SpecifiedValue
                    ,
                pub column_count:
                    longhands::column_count::SpecifiedValue
                    ,
        }

        /// Represents a serializable set of all of the longhand properties that
        /// correspond to a shorthand.
        #[derive(ToCss)]
        pub struct LonghandsToSerialize<'a> {
                pub column_width:
                    &'a longhands::column_width::SpecifiedValue,
                pub column_count:
                    &'a longhands::column_count::SpecifiedValue,
        }

        impl<'a> LonghandsToSerialize<'a> {
            /// Tries to get a serializable set of longhands given a set of
            /// property declarations.
            pub fn from_iter<I>(iter: I) -> Result<Self, ()>
                where I: Iterator<Item=&'a PropertyDeclaration>,
            {
                // Define all of the expected variables that correspond to the shorthand
                    let mut column_width = None;
                    let mut column_count = None;

                // Attempt to assign the incoming declarations to the expected variables
                for longhand in iter {
                    match *longhand {
                            PropertyDeclaration::ColumnWidth(ref value) => {
                                column_width = Some(value)
                            },
                            PropertyDeclaration::ColumnCount(ref value) => {
                                column_count = Some(value)
                            },
                        _ => {}
                    };
                }

                // If any of the expected variables are missing, return an error
                match (
                        column_width,
                        column_count,
                ) {

                    (
                        Some(column_width),
                        Some(column_count),
                    ) =>
                    Ok(LonghandsToSerialize {
                            column_width: column_width,
                            column_count: column_count,
                    }),
                    _ => Err(())
                }
            }
        }

        /// Parse the given shorthand and fill the result into the
        /// `declarations` vector.
        pub fn parse_into<'i, 't>(declarations: &mut SourcePropertyDeclaration,
                                  context: &ParserContext, input: &mut Parser<'i, 't>)
                                  -> Result<(), ParseError<'i>> {
            input.parse_entirely(|input| parse_value(context, input)).map(|longhands| {
                    declarations.push(PropertyDeclaration::ColumnWidth(
                        longhands.column_width
                    ));
                    declarations.push(PropertyDeclaration::ColumnCount(
                        longhands.column_count
                    ));
            })
        }

        
    use properties::longhands::{column_count, column_width};

    pub fn parse_value<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                               -> Result<Longhands, ParseError<'i>> {

        let mut column_count = None;
        let mut column_width = None;
        let mut autos = 0;

        loop {
            if input.try(|input| input.expect_ident_matching("auto")).is_ok() {
                // Leave the options to None, 'auto' is the initial value.
                autos += 1;
                continue
            }

            if column_count.is_none() {
                if let Ok(value) = input.try(|input| column_count::parse(context, input)) {
                    column_count = Some(value);
                    continue
                }
            }

            if column_width.is_none() {
                if let Ok(value) = input.try(|input| column_width::parse(context, input)) {
                    column_width = Some(value);
                    continue
                }
            }

            break
        }

        let values = autos + column_count.iter().len() + column_width.iter().len();
        if values == 0 || values > 2 {
            Err(StyleParseError::UnspecifiedError.into())
        } else {
            Ok(expanded! {
                column_count: unwrap_or_initial!(column_count),
                column_width: unwrap_or_initial!(column_width),
            })
        }
    }

    }




    /// https://drafts.csswg.org/css-multicol/#propdef-column-rule
    pub mod column_rule {
        use cssparser::Parser;
        use parser::ParserContext;
        use properties::{PropertyDeclaration, SourcePropertyDeclaration, MaybeBoxed, longhands};
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use std::fmt;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use style_traits::ToCss;

        pub struct Longhands {
                pub column_rule_width:
                    longhands::column_rule_width::SpecifiedValue
                    ,
                pub column_rule_style:
                    longhands::column_rule_style::SpecifiedValue
                    ,
                pub column_rule_color:
                    longhands::column_rule_color::SpecifiedValue
                    ,
        }

        /// Represents a serializable set of all of the longhand properties that
        /// correspond to a shorthand.
        #[derive(ToCss)]
        pub struct LonghandsToSerialize<'a> {
                pub column_rule_width:
                    &'a longhands::column_rule_width::SpecifiedValue,
                pub column_rule_style:
                    &'a longhands::column_rule_style::SpecifiedValue,
                pub column_rule_color:
                    &'a longhands::column_rule_color::SpecifiedValue,
        }

        impl<'a> LonghandsToSerialize<'a> {
            /// Tries to get a serializable set of longhands given a set of
            /// property declarations.
            pub fn from_iter<I>(iter: I) -> Result<Self, ()>
                where I: Iterator<Item=&'a PropertyDeclaration>,
            {
                // Define all of the expected variables that correspond to the shorthand
                    let mut column_rule_width = None;
                    let mut column_rule_style = None;
                    let mut column_rule_color = None;

                // Attempt to assign the incoming declarations to the expected variables
                for longhand in iter {
                    match *longhand {
                            PropertyDeclaration::ColumnRuleWidth(ref value) => {
                                column_rule_width = Some(value)
                            },
                            PropertyDeclaration::ColumnRuleStyle(ref value) => {
                                column_rule_style = Some(value)
                            },
                            PropertyDeclaration::ColumnRuleColor(ref value) => {
                                column_rule_color = Some(value)
                            },
                        _ => {}
                    };
                }

                // If any of the expected variables are missing, return an error
                match (
                        column_rule_width,
                        column_rule_style,
                        column_rule_color,
                ) {

                    (
                        Some(column_rule_width),
                        Some(column_rule_style),
                        Some(column_rule_color),
                    ) =>
                    Ok(LonghandsToSerialize {
                            column_rule_width: column_rule_width,
                            column_rule_style: column_rule_style,
                            column_rule_color: column_rule_color,
                    }),
                    _ => Err(())
                }
            }
        }

        /// Parse the given shorthand and fill the result into the
        /// `declarations` vector.
        pub fn parse_into<'i, 't>(declarations: &mut SourcePropertyDeclaration,
                                  context: &ParserContext, input: &mut Parser<'i, 't>)
                                  -> Result<(), ParseError<'i>> {
            input.parse_entirely(|input| parse_value(context, input)).map(|longhands| {
                    declarations.push(PropertyDeclaration::ColumnRuleWidth(
                        longhands.column_rule_width
                    ));
                    declarations.push(PropertyDeclaration::ColumnRuleStyle(
                        longhands.column_rule_style
                    ));
                    declarations.push(PropertyDeclaration::ColumnRuleColor(
                        longhands.column_rule_color
                    ));
            })
        }

        
    use properties::longhands::{column_rule_width, column_rule_style};
    use properties::longhands::column_rule_color;

    pub fn parse_value<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                               -> Result<Longhands, ParseError<'i>> {
        let mut column_rule_width = None;
        let mut column_rule_style = None;
        let mut column_rule_color = None;
        let mut any = false;

        loop {
            if column_rule_width.is_none() {
                if let Ok(value) = input.try(|input|
                        column_rule_width::parse(context, input)) {
                    column_rule_width = Some(value);
                    any = true;
                    continue
                }
            }
            if column_rule_style.is_none() {
                if let Ok(value) = input.try(|input|
                        column_rule_style::parse(context, input)) {
                    column_rule_style = Some(value);
                    any = true;
                    continue
                }
            }
            if column_rule_color.is_none() {
                if let Ok(value) = input.try(|input|
                        column_rule_color::parse(context, input)) {
                    column_rule_color = Some(value);
                    any = true;
                    continue
                }
            }

            break
        }
        if any {
            Ok(expanded! {
                column_rule_width: unwrap_or_initial!(column_rule_width),
                column_rule_style: unwrap_or_initial!(column_rule_style),
                column_rule_color: unwrap_or_initial!(column_rule_color),
            })
        } else {
            Err(StyleParseError::UnspecifiedError.into())
        }
    }

    }


    /* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */






    /// https://drafts.csswg.org/css-fonts-3/#propdef-font
    pub mod font {
        use cssparser::Parser;
        use parser::ParserContext;
        use properties::{PropertyDeclaration, SourcePropertyDeclaration, MaybeBoxed, longhands};
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use std::fmt;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use style_traits::ToCss;

        pub struct Longhands {
                pub font_style:
                    longhands::font_style::SpecifiedValue
                    ,
                pub font_variant_caps:
                    longhands::font_variant_caps::SpecifiedValue
                    ,
                pub font_weight:
                    longhands::font_weight::SpecifiedValue
                    ,
                pub font_stretch:
                    longhands::font_stretch::SpecifiedValue
                    ,
                pub font_size:
                    longhands::font_size::SpecifiedValue
                    ,
                pub line_height:
                    longhands::line_height::SpecifiedValue
                    ,
                pub font_family:
                        Box<
                    longhands::font_family::SpecifiedValue
                        >
                    ,
                pub font_size_adjust:
                    longhands::font_size_adjust::SpecifiedValue
                    ,
                pub font_kerning:
                    longhands::font_kerning::SpecifiedValue
                    ,
                pub font_variant_alternates:
                    longhands::font_variant_alternates::SpecifiedValue
                    ,
                pub font_variant_east_asian:
                    longhands::font_variant_east_asian::SpecifiedValue
                    ,
                pub font_variant_ligatures:
                    longhands::font_variant_ligatures::SpecifiedValue
                    ,
                pub font_variant_numeric:
                    longhands::font_variant_numeric::SpecifiedValue
                    ,
                pub font_variant_position:
                    longhands::font_variant_position::SpecifiedValue
                    ,
                pub font_language_override:
                        Box<
                    longhands::font_language_override::SpecifiedValue
                        >
                    ,
                pub font_feature_settings:
                        Box<
                    longhands::font_feature_settings::SpecifiedValue
                        >
                    ,
        }

        /// Represents a serializable set of all of the longhand properties that
        /// correspond to a shorthand.
        pub struct LonghandsToSerialize<'a> {
                pub font_style:
                    &'a longhands::font_style::SpecifiedValue,
                pub font_variant_caps:
                    &'a longhands::font_variant_caps::SpecifiedValue,
                pub font_weight:
                    &'a longhands::font_weight::SpecifiedValue,
                pub font_stretch:
                    &'a longhands::font_stretch::SpecifiedValue,
                pub font_size:
                    &'a longhands::font_size::SpecifiedValue,
                pub line_height:
                    &'a longhands::line_height::SpecifiedValue,
                pub font_family:
                    &'a longhands::font_family::SpecifiedValue,
                pub font_size_adjust:
                    &'a longhands::font_size_adjust::SpecifiedValue,
                pub font_kerning:
                    &'a longhands::font_kerning::SpecifiedValue,
                pub font_variant_alternates:
                    &'a longhands::font_variant_alternates::SpecifiedValue,
                pub font_variant_east_asian:
                    &'a longhands::font_variant_east_asian::SpecifiedValue,
                pub font_variant_ligatures:
                    &'a longhands::font_variant_ligatures::SpecifiedValue,
                pub font_variant_numeric:
                    &'a longhands::font_variant_numeric::SpecifiedValue,
                pub font_variant_position:
                    &'a longhands::font_variant_position::SpecifiedValue,
                pub font_language_override:
                    &'a longhands::font_language_override::SpecifiedValue,
                pub font_feature_settings:
                    &'a longhands::font_feature_settings::SpecifiedValue,
        }

        impl<'a> LonghandsToSerialize<'a> {
            /// Tries to get a serializable set of longhands given a set of
            /// property declarations.
            pub fn from_iter<I>(iter: I) -> Result<Self, ()>
                where I: Iterator<Item=&'a PropertyDeclaration>,
            {
                // Define all of the expected variables that correspond to the shorthand
                    let mut font_style = None;
                    let mut font_variant_caps = None;
                    let mut font_weight = None;
                    let mut font_stretch = None;
                    let mut font_size = None;
                    let mut line_height = None;
                    let mut font_family = None;
                    let mut font_size_adjust = None;
                    let mut font_kerning = None;
                    let mut font_variant_alternates = None;
                    let mut font_variant_east_asian = None;
                    let mut font_variant_ligatures = None;
                    let mut font_variant_numeric = None;
                    let mut font_variant_position = None;
                    let mut font_language_override = None;
                    let mut font_feature_settings = None;

                // Attempt to assign the incoming declarations to the expected variables
                for longhand in iter {
                    match *longhand {
                            PropertyDeclaration::FontStyle(ref value) => {
                                font_style = Some(value)
                            },
                            PropertyDeclaration::FontVariantCaps(ref value) => {
                                font_variant_caps = Some(value)
                            },
                            PropertyDeclaration::FontWeight(ref value) => {
                                font_weight = Some(value)
                            },
                            PropertyDeclaration::FontStretch(ref value) => {
                                font_stretch = Some(value)
                            },
                            PropertyDeclaration::FontSize(ref value) => {
                                font_size = Some(value)
                            },
                            PropertyDeclaration::LineHeight(ref value) => {
                                line_height = Some(value)
                            },
                            PropertyDeclaration::FontFamily(ref value) => {
                                font_family = Some(value)
                            },
                            PropertyDeclaration::FontSizeAdjust(ref value) => {
                                font_size_adjust = Some(value)
                            },
                            PropertyDeclaration::FontKerning(ref value) => {
                                font_kerning = Some(value)
                            },
                            PropertyDeclaration::FontVariantAlternates(ref value) => {
                                font_variant_alternates = Some(value)
                            },
                            PropertyDeclaration::FontVariantEastAsian(ref value) => {
                                font_variant_east_asian = Some(value)
                            },
                            PropertyDeclaration::FontVariantLigatures(ref value) => {
                                font_variant_ligatures = Some(value)
                            },
                            PropertyDeclaration::FontVariantNumeric(ref value) => {
                                font_variant_numeric = Some(value)
                            },
                            PropertyDeclaration::FontVariantPosition(ref value) => {
                                font_variant_position = Some(value)
                            },
                            PropertyDeclaration::FontLanguageOverride(ref value) => {
                                font_language_override = Some(value)
                            },
                            PropertyDeclaration::FontFeatureSettings(ref value) => {
                                font_feature_settings = Some(value)
                            },
                        _ => {}
                    };
                }

                // If any of the expected variables are missing, return an error
                match (
                        font_style,
                        font_variant_caps,
                        font_weight,
                        font_stretch,
                        font_size,
                        line_height,
                        font_family,
                        font_size_adjust,
                        font_kerning,
                        font_variant_alternates,
                        font_variant_east_asian,
                        font_variant_ligatures,
                        font_variant_numeric,
                        font_variant_position,
                        font_language_override,
                        font_feature_settings,
                ) {

                    (
                        Some(font_style),
                        Some(font_variant_caps),
                        Some(font_weight),
                        Some(font_stretch),
                        Some(font_size),
                        Some(line_height),
                        Some(font_family),
                        Some(font_size_adjust),
                        Some(font_kerning),
                        Some(font_variant_alternates),
                        Some(font_variant_east_asian),
                        Some(font_variant_ligatures),
                        Some(font_variant_numeric),
                        Some(font_variant_position),
                        Some(font_language_override),
                        Some(font_feature_settings),
                    ) =>
                    Ok(LonghandsToSerialize {
                            font_style: font_style,
                            font_variant_caps: font_variant_caps,
                            font_weight: font_weight,
                            font_stretch: font_stretch,
                            font_size: font_size,
                            line_height: line_height,
                            font_family: font_family,
                            font_size_adjust: font_size_adjust,
                            font_kerning: font_kerning,
                            font_variant_alternates: font_variant_alternates,
                            font_variant_east_asian: font_variant_east_asian,
                            font_variant_ligatures: font_variant_ligatures,
                            font_variant_numeric: font_variant_numeric,
                            font_variant_position: font_variant_position,
                            font_language_override: font_language_override,
                            font_feature_settings: font_feature_settings,
                    }),
                    _ => Err(())
                }
            }
        }

        /// Parse the given shorthand and fill the result into the
        /// `declarations` vector.
        pub fn parse_into<'i, 't>(declarations: &mut SourcePropertyDeclaration,
                                  context: &ParserContext, input: &mut Parser<'i, 't>)
                                  -> Result<(), ParseError<'i>> {
            input.parse_entirely(|input| parse_value(context, input)).map(|longhands| {
                    declarations.push(PropertyDeclaration::FontStyle(
                        longhands.font_style
                    ));
                    declarations.push(PropertyDeclaration::FontVariantCaps(
                        longhands.font_variant_caps
                    ));
                    declarations.push(PropertyDeclaration::FontWeight(
                        longhands.font_weight
                    ));
                    declarations.push(PropertyDeclaration::FontStretch(
                        longhands.font_stretch
                    ));
                    declarations.push(PropertyDeclaration::FontSize(
                        longhands.font_size
                    ));
                    declarations.push(PropertyDeclaration::LineHeight(
                        longhands.line_height
                    ));
                    declarations.push(PropertyDeclaration::FontFamily(
                        longhands.font_family
                    ));
                    declarations.push(PropertyDeclaration::FontSizeAdjust(
                        longhands.font_size_adjust
                    ));
                    declarations.push(PropertyDeclaration::FontKerning(
                        longhands.font_kerning
                    ));
                    declarations.push(PropertyDeclaration::FontVariantAlternates(
                        longhands.font_variant_alternates
                    ));
                    declarations.push(PropertyDeclaration::FontVariantEastAsian(
                        longhands.font_variant_east_asian
                    ));
                    declarations.push(PropertyDeclaration::FontVariantLigatures(
                        longhands.font_variant_ligatures
                    ));
                    declarations.push(PropertyDeclaration::FontVariantNumeric(
                        longhands.font_variant_numeric
                    ));
                    declarations.push(PropertyDeclaration::FontVariantPosition(
                        longhands.font_variant_position
                    ));
                    declarations.push(PropertyDeclaration::FontLanguageOverride(
                        longhands.font_language_override
                    ));
                    declarations.push(PropertyDeclaration::FontFeatureSettings(
                        longhands.font_feature_settings
                    ));
            })
        }

        
    use parser::Parse;
    use properties::longhands::{font_family, font_style, font_weight, font_stretch};
    use properties::longhands::{font_size, font_variant_caps};
    #[cfg(feature = "gecko")]
    use properties::longhands::system_font::SystemFont;
    use values::specified::text::LineHeight;

    
            use properties::longhands::font_kerning;
            use properties::longhands::font_language_override;
            use properties::longhands::font_size_adjust;
            use properties::longhands::font_variant_alternates;
            use properties::longhands::font_variant_east_asian;
            use properties::longhands::font_variant_ligatures;
            use properties::longhands::font_variant_numeric;
            use properties::longhands::font_variant_position;
            use properties::longhands::font_feature_settings;
    use self::font_family::SpecifiedValue as FontFamily;

    pub fn parse_value<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                               -> Result<Longhands, ParseError<'i>> {
        let mut nb_normals = 0;
        let mut style = None;
        let mut variant_caps = None;
        let mut weight = None;
        let mut stretch = None;
        let size;
            if let Ok(sys) = input.try(SystemFont::parse) {
                return Ok(expanded! {
                         font_family: font_family::SpecifiedValue::system_font(sys),
                         font_size: font_size::SpecifiedValue::system_font(sys),
                         font_style: font_style::SpecifiedValue::system_font(sys),
                         font_variant_caps: font_variant_caps::SpecifiedValue::system_font(sys),
                         font_stretch: font_stretch::SpecifiedValue::system_font(sys),
                         font_kerning: font_kerning::SpecifiedValue::system_font(sys),
                         font_variant_position: font_variant_position::SpecifiedValue::system_font(sys),
                         font_weight: font_weight::SpecifiedValue::system_font(sys),
                         font_size_adjust: font_size_adjust::SpecifiedValue::system_font(sys),
                         font_variant_alternates: font_variant_alternates::SpecifiedValue::system_font(sys),
                         font_variant_ligatures: font_variant_ligatures::SpecifiedValue::system_font(sys),
                         font_variant_east_asian: font_variant_east_asian::SpecifiedValue::system_font(sys),
                         font_variant_numeric: font_variant_numeric::SpecifiedValue::system_font(sys),
                         font_language_override: font_language_override::SpecifiedValue::system_font(sys),
                         font_feature_settings: font_feature_settings::SpecifiedValue::system_font(sys),
                     // line-height is just reset to initial
                     line_height: LineHeight::normal(),
                 })
            }
        loop {
            // Special-case 'normal' because it is valid in each of
            // font-style, font-weight, font-variant and font-stretch.
            // Leaves the values to None, 'normal' is the initial value for each of them.
            if input.try(|input| input.expect_ident_matching("normal")).is_ok() {
                nb_normals += 1;
                continue;
            }
            if style.is_none() {
                if let Ok(value) = input.try(|input| font_style::parse(context, input)) {
                    style = Some(value);
                    continue
                }
            }
            if weight.is_none() {
                if let Ok(value) = input.try(|input| font_weight::parse(context, input)) {
                    weight = Some(value);
                    continue
                }
            }
            if variant_caps.is_none() {
                if let Ok(value) = input.try(|input| font_variant_caps::parse(context, input)) {
                    variant_caps = Some(value);
                    continue
                }
            }
            if stretch.is_none() {
                if let Ok(value) = input.try(|input| font_stretch::parse(context, input)) {
                    stretch = Some(value);
                    continue
                }
            }
            size = Some(font_size::parse(context, input)?);
            break
        }
        #[inline]
        fn count<T>(opt: &Option<T>) -> u8 {
            if opt.is_some() { 1 } else { 0 }
        }
        if size.is_none() ||
           (count(&style) + count(&weight) + count(&variant_caps) + count(&stretch) + nb_normals) > 4 {
            return Err(StyleParseError::UnspecifiedError.into())
        }
        let line_height = if input.try(|input| input.expect_delim('/')).is_ok() {
            Some(LineHeight::parse(context, input)?)
        } else {
            None
        };
        let family = FontFamily::parse(input)?;
        Ok(expanded! {
                font_style: unwrap_or_initial!(font_style, style),
                font_weight: unwrap_or_initial!(font_weight, weight),
                font_stretch: unwrap_or_initial!(font_stretch, stretch),
                font_size: unwrap_or_initial!(font_size, size),
                font_variant_caps: unwrap_or_initial!(font_variant_caps, variant_caps),
            line_height: line_height.unwrap_or(LineHeight::normal()),
            font_family: family,
                    font_kerning: font_kerning::get_initial_specified_value(),
                    font_language_override: font_language_override::get_initial_specified_value(),
                    font_size_adjust: font_size_adjust::get_initial_specified_value(),
                    font_variant_alternates: font_variant_alternates::get_initial_specified_value(),
                    font_variant_east_asian: font_variant_east_asian::get_initial_specified_value(),
                    font_variant_ligatures: font_variant_ligatures::get_initial_specified_value(),
                    font_variant_numeric: font_variant_numeric::get_initial_specified_value(),
                    font_variant_position: font_variant_position::get_initial_specified_value(),
                    font_feature_settings: font_feature_settings::get_initial_specified_value(),
        })
    }

        enum CheckSystemResult {
            AllSystem(SystemFont),
            SomeSystem,
            None
        }
    enum SerializeFor {
        Normal,
        Canvas,
    }

    impl<'a> LonghandsToSerialize<'a> {
        fn to_css_for<W>(&self,
                         serialize_for: SerializeFor,
                         dest: &mut W) -> fmt::Result where W: fmt::Write {
                match self.check_system() {
                    CheckSystemResult::AllSystem(sys) => return sys.to_css(dest),
                    CheckSystemResult::SomeSystem => return Ok(()),
                    CheckSystemResult::None => ()
                }

                    if self.font_kerning != &font_kerning::get_initial_specified_value() {
                        return Ok(());
                    }
                    if self.font_language_override != &font_language_override::get_initial_specified_value() {
                        return Ok(());
                    }
                    if self.font_size_adjust != &font_size_adjust::get_initial_specified_value() {
                        return Ok(());
                    }
                    if self.font_variant_alternates != &font_variant_alternates::get_initial_specified_value() {
                        return Ok(());
                    }
                    if self.font_variant_east_asian != &font_variant_east_asian::get_initial_specified_value() {
                        return Ok(());
                    }
                    if self.font_variant_ligatures != &font_variant_ligatures::get_initial_specified_value() {
                        return Ok(());
                    }
                    if self.font_variant_numeric != &font_variant_numeric::get_initial_specified_value() {
                        return Ok(());
                    }
                    if self.font_variant_position != &font_variant_position::get_initial_specified_value() {
                        return Ok(());
                    }
                    if self.font_feature_settings != &font_feature_settings::get_initial_specified_value() {
                        return Ok(());
                    }

            // In case of serialization for canvas font, we need to drop
            // initial values of properties other than size and family.
                let needs_this_property = match serialize_for {
                    SerializeFor::Normal => true,
                    SerializeFor::Canvas =>
                        self.font_style != &font_style::get_initial_specified_value(),
                };
                if needs_this_property {
                    self.font_style.to_css(dest)?;
                    dest.write_str(" ")?;
                }
                let needs_this_property = match serialize_for {
                    SerializeFor::Normal => true,
                    SerializeFor::Canvas =>
                        self.font_variant_caps != &font_variant_caps::get_initial_specified_value(),
                };
                if needs_this_property {
                    self.font_variant_caps.to_css(dest)?;
                    dest.write_str(" ")?;
                }
                let needs_this_property = match serialize_for {
                    SerializeFor::Normal => true,
                    SerializeFor::Canvas =>
                        self.font_weight != &font_weight::get_initial_specified_value(),
                };
                if needs_this_property {
                    self.font_weight.to_css(dest)?;
                    dest.write_str(" ")?;
                }
                let needs_this_property = match serialize_for {
                    SerializeFor::Normal => true,
                    SerializeFor::Canvas =>
                        self.font_stretch != &font_stretch::get_initial_specified_value(),
                };
                if needs_this_property {
                    self.font_stretch.to_css(dest)?;
                    dest.write_str(" ")?;
                }

            self.font_size.to_css(dest)?;

            if *self.line_height != LineHeight::normal() {
                dest.write_str("/")?;
                self.line_height.to_css(dest)?;
            }

            dest.write_str(" ")?;
            self.font_family.to_css(dest)?;

            Ok(())
        }

            /// Check if some or all members are system fonts
            fn check_system(&self) -> CheckSystemResult {
                let mut sys = None;
                let mut all = true;

                    if let Some(s) = self.font_family.get_system() {
                        debug_assert!(sys.is_none() || s == sys.unwrap());
                        sys = Some(s);
                    } else {
                        all = false;
                    }
                    if let Some(s) = self.font_size.get_system() {
                        debug_assert!(sys.is_none() || s == sys.unwrap());
                        sys = Some(s);
                    } else {
                        all = false;
                    }
                    if let Some(s) = self.font_style.get_system() {
                        debug_assert!(sys.is_none() || s == sys.unwrap());
                        sys = Some(s);
                    } else {
                        all = false;
                    }
                    if let Some(s) = self.font_variant_caps.get_system() {
                        debug_assert!(sys.is_none() || s == sys.unwrap());
                        sys = Some(s);
                    } else {
                        all = false;
                    }
                    if let Some(s) = self.font_stretch.get_system() {
                        debug_assert!(sys.is_none() || s == sys.unwrap());
                        sys = Some(s);
                    } else {
                        all = false;
                    }
                    if let Some(s) = self.font_kerning.get_system() {
                        debug_assert!(sys.is_none() || s == sys.unwrap());
                        sys = Some(s);
                    } else {
                        all = false;
                    }
                    if let Some(s) = self.font_variant_position.get_system() {
                        debug_assert!(sys.is_none() || s == sys.unwrap());
                        sys = Some(s);
                    } else {
                        all = false;
                    }
                    if let Some(s) = self.font_weight.get_system() {
                        debug_assert!(sys.is_none() || s == sys.unwrap());
                        sys = Some(s);
                    } else {
                        all = false;
                    }
                    if let Some(s) = self.font_size_adjust.get_system() {
                        debug_assert!(sys.is_none() || s == sys.unwrap());
                        sys = Some(s);
                    } else {
                        all = false;
                    }
                    if let Some(s) = self.font_variant_alternates.get_system() {
                        debug_assert!(sys.is_none() || s == sys.unwrap());
                        sys = Some(s);
                    } else {
                        all = false;
                    }
                    if let Some(s) = self.font_variant_ligatures.get_system() {
                        debug_assert!(sys.is_none() || s == sys.unwrap());
                        sys = Some(s);
                    } else {
                        all = false;
                    }
                    if let Some(s) = self.font_variant_east_asian.get_system() {
                        debug_assert!(sys.is_none() || s == sys.unwrap());
                        sys = Some(s);
                    } else {
                        all = false;
                    }
                    if let Some(s) = self.font_variant_numeric.get_system() {
                        debug_assert!(sys.is_none() || s == sys.unwrap());
                        sys = Some(s);
                    } else {
                        all = false;
                    }
                    if let Some(s) = self.font_language_override.get_system() {
                        debug_assert!(sys.is_none() || s == sys.unwrap());
                        sys = Some(s);
                    } else {
                        all = false;
                    }
                    if let Some(s) = self.font_feature_settings.get_system() {
                        debug_assert!(sys.is_none() || s == sys.unwrap());
                        sys = Some(s);
                    } else {
                        all = false;
                    }
                if self.line_height != &LineHeight::normal() {
                    all = false
                }
                if all {
                    CheckSystemResult::AllSystem(sys.unwrap())
                } else if sys.is_some() {
                    CheckSystemResult::SomeSystem
                } else {
                    CheckSystemResult::None
                }
            }

            /// Serialize the shorthand value for canvas font attribute.
            pub fn to_css_for_canvas<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
                self.to_css_for(SerializeFor::Canvas, dest)
            }
    }

    // This may be a bit off, unsure, possibly needs changes
    impl<'a> ToCss for LonghandsToSerialize<'a>  {
        fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
            self.to_css_for(SerializeFor::Normal, dest)
        }
    }

    }




    /// https://drafts.csswg.org/css-fonts-3/#propdef-font-variant
    pub mod font_variant {
        use cssparser::Parser;
        use parser::ParserContext;
        use properties::{PropertyDeclaration, SourcePropertyDeclaration, MaybeBoxed, longhands};
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use std::fmt;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use style_traits::ToCss;

        pub struct Longhands {
                pub font_variant_caps:
                    longhands::font_variant_caps::SpecifiedValue
                    ,
                pub font_variant_alternates:
                    longhands::font_variant_alternates::SpecifiedValue
                    ,
                pub font_variant_east_asian:
                    longhands::font_variant_east_asian::SpecifiedValue
                    ,
                pub font_variant_ligatures:
                    longhands::font_variant_ligatures::SpecifiedValue
                    ,
                pub font_variant_numeric:
                    longhands::font_variant_numeric::SpecifiedValue
                    ,
                pub font_variant_position:
                    longhands::font_variant_position::SpecifiedValue
                    ,
        }

        /// Represents a serializable set of all of the longhand properties that
        /// correspond to a shorthand.
        pub struct LonghandsToSerialize<'a> {
                pub font_variant_caps:
                    &'a longhands::font_variant_caps::SpecifiedValue,
                pub font_variant_alternates:
                    &'a longhands::font_variant_alternates::SpecifiedValue,
                pub font_variant_east_asian:
                    &'a longhands::font_variant_east_asian::SpecifiedValue,
                pub font_variant_ligatures:
                    &'a longhands::font_variant_ligatures::SpecifiedValue,
                pub font_variant_numeric:
                    &'a longhands::font_variant_numeric::SpecifiedValue,
                pub font_variant_position:
                    &'a longhands::font_variant_position::SpecifiedValue,
        }

        impl<'a> LonghandsToSerialize<'a> {
            /// Tries to get a serializable set of longhands given a set of
            /// property declarations.
            pub fn from_iter<I>(iter: I) -> Result<Self, ()>
                where I: Iterator<Item=&'a PropertyDeclaration>,
            {
                // Define all of the expected variables that correspond to the shorthand
                    let mut font_variant_caps = None;
                    let mut font_variant_alternates = None;
                    let mut font_variant_east_asian = None;
                    let mut font_variant_ligatures = None;
                    let mut font_variant_numeric = None;
                    let mut font_variant_position = None;

                // Attempt to assign the incoming declarations to the expected variables
                for longhand in iter {
                    match *longhand {
                            PropertyDeclaration::FontVariantCaps(ref value) => {
                                font_variant_caps = Some(value)
                            },
                            PropertyDeclaration::FontVariantAlternates(ref value) => {
                                font_variant_alternates = Some(value)
                            },
                            PropertyDeclaration::FontVariantEastAsian(ref value) => {
                                font_variant_east_asian = Some(value)
                            },
                            PropertyDeclaration::FontVariantLigatures(ref value) => {
                                font_variant_ligatures = Some(value)
                            },
                            PropertyDeclaration::FontVariantNumeric(ref value) => {
                                font_variant_numeric = Some(value)
                            },
                            PropertyDeclaration::FontVariantPosition(ref value) => {
                                font_variant_position = Some(value)
                            },
                        _ => {}
                    };
                }

                // If any of the expected variables are missing, return an error
                match (
                        font_variant_caps,
                        font_variant_alternates,
                        font_variant_east_asian,
                        font_variant_ligatures,
                        font_variant_numeric,
                        font_variant_position,
                ) {

                    (
                        Some(font_variant_caps),
                        Some(font_variant_alternates),
                        Some(font_variant_east_asian),
                        Some(font_variant_ligatures),
                        Some(font_variant_numeric),
                        Some(font_variant_position),
                    ) =>
                    Ok(LonghandsToSerialize {
                            font_variant_caps: font_variant_caps,
                            font_variant_alternates: font_variant_alternates,
                            font_variant_east_asian: font_variant_east_asian,
                            font_variant_ligatures: font_variant_ligatures,
                            font_variant_numeric: font_variant_numeric,
                            font_variant_position: font_variant_position,
                    }),
                    _ => Err(())
                }
            }
        }

        /// Parse the given shorthand and fill the result into the
        /// `declarations` vector.
        pub fn parse_into<'i, 't>(declarations: &mut SourcePropertyDeclaration,
                                  context: &ParserContext, input: &mut Parser<'i, 't>)
                                  -> Result<(), ParseError<'i>> {
            input.parse_entirely(|input| parse_value(context, input)).map(|longhands| {
                    declarations.push(PropertyDeclaration::FontVariantCaps(
                        longhands.font_variant_caps
                    ));
                    declarations.push(PropertyDeclaration::FontVariantAlternates(
                        longhands.font_variant_alternates
                    ));
                    declarations.push(PropertyDeclaration::FontVariantEastAsian(
                        longhands.font_variant_east_asian
                    ));
                    declarations.push(PropertyDeclaration::FontVariantLigatures(
                        longhands.font_variant_ligatures
                    ));
                    declarations.push(PropertyDeclaration::FontVariantNumeric(
                        longhands.font_variant_numeric
                    ));
                    declarations.push(PropertyDeclaration::FontVariantPosition(
                        longhands.font_variant_position
                    ));
            })
        }

        
    
    

    use properties::longhands::font_variant_caps;
    use properties::longhands::font_variant_alternates;
    use properties::longhands::font_variant_east_asian;
    use properties::longhands::font_variant_ligatures;
    use properties::longhands::font_variant_numeric;
    use properties::longhands::font_variant_position;

    pub fn parse_value<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                               -> Result<Longhands, ParseError<'i>> {
        let mut caps = None;
        let mut alternates = None;
        let mut east_asian = None;
        let mut ligatures = None;
        let mut numeric = None;
        let mut position = None;

        if input.try(|input| input.expect_ident_matching("normal")).is_ok() {
            // Leave the values to None, 'normal' is the initial value for all the sub properties.
        } else if input.try(|input| input.expect_ident_matching("none")).is_ok() {
            // The 'none' value sets 'font-variant-ligatures' to 'none' and resets all other sub properties
            // to their initial value.
            ligatures = Some(font_variant_ligatures::get_none_specified_value());
        } else {
            let mut has_custom_value: bool = false;
            loop {
                if input.try(|input| input.expect_ident_matching("normal")).is_ok() ||
                   input.try(|input| input.expect_ident_matching("none")).is_ok() {
                    return Err(StyleParseError::UnspecifiedError.into())
                }
                if caps.is_none() {
                    if let Ok(value) = input.try(|i| font_variant_caps::parse(context, i)) {
                        has_custom_value = true;
                        caps = Some(value);
                        continue
                    }
                }
                if alternates.is_none() {
                    if let Ok(value) = input.try(|i| font_variant_alternates::parse(context, i)) {
                        has_custom_value = true;
                        alternates = Some(value);
                        continue
                    }
                }
                if east_asian.is_none() {
                    if let Ok(value) = input.try(|i| font_variant_east_asian::parse(context, i)) {
                        has_custom_value = true;
                        east_asian = Some(value);
                        continue
                    }
                }
                if ligatures.is_none() {
                    if let Ok(value) = input.try(|i| font_variant_ligatures::parse(context, i)) {
                        has_custom_value = true;
                        ligatures = Some(value);
                        continue
                    }
                }
                if numeric.is_none() {
                    if let Ok(value) = input.try(|i| font_variant_numeric::parse(context, i)) {
                        has_custom_value = true;
                        numeric = Some(value);
                        continue
                    }
                }
                if position.is_none() {
                    if let Ok(value) = input.try(|i| font_variant_position::parse(context, i)) {
                        has_custom_value = true;
                        position = Some(value);
                        continue
                    }
                }

                break
            }

            if !has_custom_value {
                return Err(StyleParseError::UnspecifiedError.into())
            }
        }

        Ok(expanded! {
            font_variant_caps: unwrap_or_initial!(font_variant_caps, caps),
            font_variant_alternates: unwrap_or_initial!(font_variant_alternates, alternates),
            font_variant_east_asian: unwrap_or_initial!(font_variant_east_asian, east_asian),
            font_variant_ligatures: unwrap_or_initial!(font_variant_ligatures, ligatures),
            font_variant_numeric: unwrap_or_initial!(font_variant_numeric, numeric),
            font_variant_position: unwrap_or_initial!(font_variant_position, position),
        })
    }

    impl<'a> ToCss for LonghandsToSerialize<'a>  {
        #[allow(unused_assignments)]
        fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {

            let has_none_ligatures =
                self.font_variant_ligatures == &font_variant_ligatures::get_none_specified_value();

            const TOTAL_SUBPROPS: usize = 6;
            let mut nb_normals = 0;
            if self.font_variant_caps == &font_variant_caps::get_initial_specified_value() {
                nb_normals += 1;
            }
            if self.font_variant_alternates == &font_variant_alternates::get_initial_specified_value() {
                nb_normals += 1;
            }
            if self.font_variant_east_asian == &font_variant_east_asian::get_initial_specified_value() {
                nb_normals += 1;
            }
            if self.font_variant_ligatures == &font_variant_ligatures::get_initial_specified_value() {
                nb_normals += 1;
            }
            if self.font_variant_numeric == &font_variant_numeric::get_initial_specified_value() {
                nb_normals += 1;
            }
            if self.font_variant_position == &font_variant_position::get_initial_specified_value() {
                nb_normals += 1;
            }


            if nb_normals > 0 && nb_normals == TOTAL_SUBPROPS {
                dest.write_str("normal")?;
            } else if has_none_ligatures {
                if nb_normals == TOTAL_SUBPROPS - 1 {
                    // Serialize to 'none' if 'font-variant-ligatures' is set to 'none' and all other
                    // font feature properties are reset to their initial value.
                    dest.write_str("none")?;
                } else {
                    return Ok(())
                }
            } else {
                let mut has_any = false;
                if self.font_variant_caps != &font_variant_caps::get_initial_specified_value() {
                    if has_any {
                        dest.write_str(" ")?;
                    }
                    has_any = true;
                    self.font_variant_caps.to_css(dest)?;
                }
                if self.font_variant_alternates != &font_variant_alternates::get_initial_specified_value() {
                    if has_any {
                        dest.write_str(" ")?;
                    }
                    has_any = true;
                    self.font_variant_alternates.to_css(dest)?;
                }
                if self.font_variant_east_asian != &font_variant_east_asian::get_initial_specified_value() {
                    if has_any {
                        dest.write_str(" ")?;
                    }
                    has_any = true;
                    self.font_variant_east_asian.to_css(dest)?;
                }
                if self.font_variant_ligatures != &font_variant_ligatures::get_initial_specified_value() {
                    if has_any {
                        dest.write_str(" ")?;
                    }
                    has_any = true;
                    self.font_variant_ligatures.to_css(dest)?;
                }
                if self.font_variant_numeric != &font_variant_numeric::get_initial_specified_value() {
                    if has_any {
                        dest.write_str(" ")?;
                    }
                    has_any = true;
                    self.font_variant_numeric.to_css(dest)?;
                }
                if self.font_variant_position != &font_variant_position::get_initial_specified_value() {
                    if has_any {
                        dest.write_str(" ")?;
                    }
                    has_any = true;
                    self.font_variant_position.to_css(dest)?;
                }
            }

            Ok(())
        }
    }

    }


    /* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */





    /// https://drafts.csswg.org/css-text-decor-3/#text-emphasis-property
    pub mod text_emphasis {
        use cssparser::Parser;
        use parser::ParserContext;
        use properties::{PropertyDeclaration, SourcePropertyDeclaration, MaybeBoxed, longhands};
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use std::fmt;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use style_traits::ToCss;

        pub struct Longhands {
                pub text_emphasis_style:
                        Box<
                    longhands::text_emphasis_style::SpecifiedValue
                        >
                    ,
                pub text_emphasis_color:
                    longhands::text_emphasis_color::SpecifiedValue
                    ,
        }

        /// Represents a serializable set of all of the longhand properties that
        /// correspond to a shorthand.
        #[derive(ToCss)]
        pub struct LonghandsToSerialize<'a> {
                pub text_emphasis_style:
                    &'a longhands::text_emphasis_style::SpecifiedValue,
                pub text_emphasis_color:
                    &'a longhands::text_emphasis_color::SpecifiedValue,
        }

        impl<'a> LonghandsToSerialize<'a> {
            /// Tries to get a serializable set of longhands given a set of
            /// property declarations.
            pub fn from_iter<I>(iter: I) -> Result<Self, ()>
                where I: Iterator<Item=&'a PropertyDeclaration>,
            {
                // Define all of the expected variables that correspond to the shorthand
                    let mut text_emphasis_style = None;
                    let mut text_emphasis_color = None;

                // Attempt to assign the incoming declarations to the expected variables
                for longhand in iter {
                    match *longhand {
                            PropertyDeclaration::TextEmphasisStyle(ref value) => {
                                text_emphasis_style = Some(value)
                            },
                            PropertyDeclaration::TextEmphasisColor(ref value) => {
                                text_emphasis_color = Some(value)
                            },
                        _ => {}
                    };
                }

                // If any of the expected variables are missing, return an error
                match (
                        text_emphasis_style,
                        text_emphasis_color,
                ) {

                    (
                        Some(text_emphasis_style),
                        Some(text_emphasis_color),
                    ) =>
                    Ok(LonghandsToSerialize {
                            text_emphasis_style: text_emphasis_style,
                            text_emphasis_color: text_emphasis_color,
                    }),
                    _ => Err(())
                }
            }
        }

        /// Parse the given shorthand and fill the result into the
        /// `declarations` vector.
        pub fn parse_into<'i, 't>(declarations: &mut SourcePropertyDeclaration,
                                  context: &ParserContext, input: &mut Parser<'i, 't>)
                                  -> Result<(), ParseError<'i>> {
            input.parse_entirely(|input| parse_value(context, input)).map(|longhands| {
                    declarations.push(PropertyDeclaration::TextEmphasisStyle(
                        longhands.text_emphasis_style
                    ));
                    declarations.push(PropertyDeclaration::TextEmphasisColor(
                        longhands.text_emphasis_color
                    ));
            })
        }

        
    use properties::longhands::{text_emphasis_color, text_emphasis_style};

    pub fn parse_value<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                               -> Result<Longhands, ParseError<'i>> {
        let mut color = None;
        let mut style = None;

        loop {
            if color.is_none() {
                if let Ok(value) = input.try(|input| text_emphasis_color::parse(context, input)) {
                    color = Some(value);
                    continue
                }
            }
            if style.is_none() {
                if let Ok(value) = input.try(|input| text_emphasis_style::parse(context, input)) {
                    style = Some(value);
                    continue
                }
            }
            break
        }
        if color.is_some() || style.is_some() {
            Ok(expanded! {
                text_emphasis_color: unwrap_or_initial!(text_emphasis_color, color),
                text_emphasis_style: unwrap_or_initial!(text_emphasis_style, style),
            })
        } else {
            Err(StyleParseError::UnspecifiedError.into())
        }
    }

    }


// CSS Compatibility
// https://compat.spec.whatwg.org/


    /// https://compat.spec.whatwg.org/#the-webkit-text-stroke
    pub mod _webkit_text_stroke {
        use cssparser::Parser;
        use parser::ParserContext;
        use properties::{PropertyDeclaration, SourcePropertyDeclaration, MaybeBoxed, longhands};
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use std::fmt;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use style_traits::ToCss;

        pub struct Longhands {
                pub _webkit_text_stroke_width:
                    longhands::_webkit_text_stroke_width::SpecifiedValue
                    ,
                pub _webkit_text_stroke_color:
                    longhands::_webkit_text_stroke_color::SpecifiedValue
                    ,
        }

        /// Represents a serializable set of all of the longhand properties that
        /// correspond to a shorthand.
        #[derive(ToCss)]
        pub struct LonghandsToSerialize<'a> {
                pub _webkit_text_stroke_width:
                    &'a longhands::_webkit_text_stroke_width::SpecifiedValue,
                pub _webkit_text_stroke_color:
                    &'a longhands::_webkit_text_stroke_color::SpecifiedValue,
        }

        impl<'a> LonghandsToSerialize<'a> {
            /// Tries to get a serializable set of longhands given a set of
            /// property declarations.
            pub fn from_iter<I>(iter: I) -> Result<Self, ()>
                where I: Iterator<Item=&'a PropertyDeclaration>,
            {
                // Define all of the expected variables that correspond to the shorthand
                    let mut _webkit_text_stroke_width = None;
                    let mut _webkit_text_stroke_color = None;

                // Attempt to assign the incoming declarations to the expected variables
                for longhand in iter {
                    match *longhand {
                            PropertyDeclaration::WebkitTextStrokeWidth(ref value) => {
                                _webkit_text_stroke_width = Some(value)
                            },
                            PropertyDeclaration::WebkitTextStrokeColor(ref value) => {
                                _webkit_text_stroke_color = Some(value)
                            },
                        _ => {}
                    };
                }

                // If any of the expected variables are missing, return an error
                match (
                        _webkit_text_stroke_width,
                        _webkit_text_stroke_color,
                ) {

                    (
                        Some(_webkit_text_stroke_width),
                        Some(_webkit_text_stroke_color),
                    ) =>
                    Ok(LonghandsToSerialize {
                            _webkit_text_stroke_width: _webkit_text_stroke_width,
                            _webkit_text_stroke_color: _webkit_text_stroke_color,
                    }),
                    _ => Err(())
                }
            }
        }

        /// Parse the given shorthand and fill the result into the
        /// `declarations` vector.
        pub fn parse_into<'i, 't>(declarations: &mut SourcePropertyDeclaration,
                                  context: &ParserContext, input: &mut Parser<'i, 't>)
                                  -> Result<(), ParseError<'i>> {
            input.parse_entirely(|input| parse_value(context, input)).map(|longhands| {
                    declarations.push(PropertyDeclaration::WebkitTextStrokeWidth(
                        longhands._webkit_text_stroke_width
                    ));
                    declarations.push(PropertyDeclaration::WebkitTextStrokeColor(
                        longhands._webkit_text_stroke_color
                    ));
            })
        }

        
    use properties::longhands::{_webkit_text_stroke_color, _webkit_text_stroke_width};

    pub fn parse_value<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                               -> Result<Longhands, ParseError<'i>> {
        let mut color = None;
        let mut width = None;
        loop {
            if color.is_none() {
                if let Ok(value) = input.try(|input| _webkit_text_stroke_color::parse(context, input)) {
                    color = Some(value);
                    continue
                }
            }

            if width.is_none() {
                if let Ok(value) = input.try(|input| _webkit_text_stroke_width::parse(context, input)) {
                    width = Some(value);
                    continue
                }
            }
            break
        }

        if color.is_some() || width.is_some() {
            Ok(expanded! {
                _webkit_text_stroke_color: unwrap_or_initial!(_webkit_text_stroke_color, color),
                _webkit_text_stroke_width: unwrap_or_initial!(_webkit_text_stroke_width, width),
            })
        } else {
            Err(StyleParseError::UnspecifiedError.into())
        }
    }

    }


    /* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */





    /// https://drafts.csswg.org/css-lists/#propdef-list-style
    pub mod list_style {
        use cssparser::Parser;
        use parser::ParserContext;
        use properties::{PropertyDeclaration, SourcePropertyDeclaration, MaybeBoxed, longhands};
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use std::fmt;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use style_traits::ToCss;

        pub struct Longhands {
                pub list_style_position:
                    longhands::list_style_position::SpecifiedValue
                    ,
                pub list_style_image:
                        Box<
                    longhands::list_style_image::SpecifiedValue
                        >
                    ,
                pub list_style_type:
                        Box<
                    longhands::list_style_type::SpecifiedValue
                        >
                    ,
        }

        /// Represents a serializable set of all of the longhand properties that
        /// correspond to a shorthand.
        #[derive(ToCss)]
        pub struct LonghandsToSerialize<'a> {
                pub list_style_position:
                    &'a longhands::list_style_position::SpecifiedValue,
                pub list_style_image:
                    &'a longhands::list_style_image::SpecifiedValue,
                pub list_style_type:
                    &'a longhands::list_style_type::SpecifiedValue,
        }

        impl<'a> LonghandsToSerialize<'a> {
            /// Tries to get a serializable set of longhands given a set of
            /// property declarations.
            pub fn from_iter<I>(iter: I) -> Result<Self, ()>
                where I: Iterator<Item=&'a PropertyDeclaration>,
            {
                // Define all of the expected variables that correspond to the shorthand
                    let mut list_style_position = None;
                    let mut list_style_image = None;
                    let mut list_style_type = None;

                // Attempt to assign the incoming declarations to the expected variables
                for longhand in iter {
                    match *longhand {
                            PropertyDeclaration::ListStylePosition(ref value) => {
                                list_style_position = Some(value)
                            },
                            PropertyDeclaration::ListStyleImage(ref value) => {
                                list_style_image = Some(value)
                            },
                            PropertyDeclaration::ListStyleType(ref value) => {
                                list_style_type = Some(value)
                            },
                        _ => {}
                    };
                }

                // If any of the expected variables are missing, return an error
                match (
                        list_style_position,
                        list_style_image,
                        list_style_type,
                ) {

                    (
                        Some(list_style_position),
                        Some(list_style_image),
                        Some(list_style_type),
                    ) =>
                    Ok(LonghandsToSerialize {
                            list_style_position: list_style_position,
                            list_style_image: list_style_image,
                            list_style_type: list_style_type,
                    }),
                    _ => Err(())
                }
            }
        }

        /// Parse the given shorthand and fill the result into the
        /// `declarations` vector.
        pub fn parse_into<'i, 't>(declarations: &mut SourcePropertyDeclaration,
                                  context: &ParserContext, input: &mut Parser<'i, 't>)
                                  -> Result<(), ParseError<'i>> {
            input.parse_entirely(|input| parse_value(context, input)).map(|longhands| {
                    declarations.push(PropertyDeclaration::ListStylePosition(
                        longhands.list_style_position
                    ));
                    declarations.push(PropertyDeclaration::ListStyleImage(
                        longhands.list_style_image
                    ));
                    declarations.push(PropertyDeclaration::ListStyleType(
                        longhands.list_style_type
                    ));
            })
        }

        
    use properties::longhands::{list_style_image, list_style_position, list_style_type};
    use values::{Either, None_};

    pub fn parse_value<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                               -> Result<Longhands, ParseError<'i>> {
        // `none` is ambiguous until we've finished parsing the shorthands, so we count the number
        // of times we see it.
        let mut nones = 0u8;
        let (mut image, mut position, mut list_style_type, mut any) = (None, None, None, false);
        loop {
            if input.try(|input| input.expect_ident_matching("none")).is_ok() {
                nones = nones + 1;
                if nones > 2 {
                    return Err(SelectorParseError::UnexpectedIdent("none".into()).into())
                }
                any = true;
                continue
            }

            if image.is_none() {
                if let Ok(value) = input.try(|input| list_style_image::parse(context, input)) {
                    image = Some(value);
                    any = true;
                    continue
                }
            }

            if position.is_none() {
                if let Ok(value) = input.try(|input| list_style_position::parse(context, input)) {
                    position = Some(value);
                    any = true;
                    continue
                }
            }

            // list-style-type must be checked the last, because it accepts
            // arbitrary identifier for custom counter style, and thus may
            // affect values of list-style-position.
            if list_style_type.is_none() {
                if let Ok(value) = input.try(|input| list_style_type::parse(context, input)) {
                    list_style_type = Some(value);
                    any = true;
                    continue
                }
            }
            break
        }

        let position = unwrap_or_initial!(list_style_position, position);

        fn list_style_type_none() -> list_style_type::SpecifiedValue {
            use values::generics::CounterStyleOrNone;
            list_style_type::SpecifiedValue::CounterStyle(CounterStyleOrNone::None)
        }

        // If there are two `none`s, then we can't have a type or image; if there is one `none`,
        // then we can't have both a type *and* an image; if there is no `none` then we're fine as
        // long as we parsed something.
        match (any, nones, list_style_type, image) {
            (true, 2, None, None) => {
                Ok(expanded! {
                    list_style_position: position,
                    list_style_image: list_style_image::SpecifiedValue(Either::Second(None_)),
                    list_style_type: list_style_type_none(),
                })
            }
            (true, 1, None, Some(image)) => {
                Ok(expanded! {
                    list_style_position: position,
                    list_style_image: image,
                    list_style_type: list_style_type_none(),
                })
            }
            (true, 1, Some(list_style_type), None) => {
                Ok(expanded! {
                    list_style_position: position,
                    list_style_image: list_style_image::SpecifiedValue(Either::Second(None_)),
                    list_style_type: list_style_type,
                })
            }
            (true, 1, None, None) => {
                Ok(expanded! {
                    list_style_position: position,
                    list_style_image: list_style_image::SpecifiedValue(Either::Second(None_)),
                    list_style_type: list_style_type_none(),
                })
            }
            (true, 0, list_style_type, image) => {
                Ok(expanded! {
                    list_style_position: position,
                    list_style_image: unwrap_or_initial!(list_style_image, image),
                    list_style_type: unwrap_or_initial!(list_style_type),
                })
            }
            _ => Err(StyleParseError::UnspecifiedError.into()),
        }
    }

    }


    /* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */




    
    

    /// https://drafts.csswg.org/css-box/#propdef-margin
    pub mod margin {
        use cssparser::Parser;
        use parser::ParserContext;
        use properties::{PropertyDeclaration, SourcePropertyDeclaration, MaybeBoxed, longhands};
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use std::fmt;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use style_traits::ToCss;

        pub struct Longhands {
                pub margin_top:
                    longhands::margin_top::SpecifiedValue
                    ,
                pub margin_right:
                    longhands::margin_right::SpecifiedValue
                    ,
                pub margin_bottom:
                    longhands::margin_bottom::SpecifiedValue
                    ,
                pub margin_left:
                    longhands::margin_left::SpecifiedValue
                    ,
        }

        /// Represents a serializable set of all of the longhand properties that
        /// correspond to a shorthand.
        pub struct LonghandsToSerialize<'a> {
                pub margin_top:
                    &'a longhands::margin_top::SpecifiedValue,
                pub margin_right:
                    &'a longhands::margin_right::SpecifiedValue,
                pub margin_bottom:
                    &'a longhands::margin_bottom::SpecifiedValue,
                pub margin_left:
                    &'a longhands::margin_left::SpecifiedValue,
        }

        impl<'a> LonghandsToSerialize<'a> {
            /// Tries to get a serializable set of longhands given a set of
            /// property declarations.
            pub fn from_iter<I>(iter: I) -> Result<Self, ()>
                where I: Iterator<Item=&'a PropertyDeclaration>,
            {
                // Define all of the expected variables that correspond to the shorthand
                    let mut margin_top = None;
                    let mut margin_right = None;
                    let mut margin_bottom = None;
                    let mut margin_left = None;

                // Attempt to assign the incoming declarations to the expected variables
                for longhand in iter {
                    match *longhand {
                            PropertyDeclaration::MarginTop(ref value) => {
                                margin_top = Some(value)
                            },
                            PropertyDeclaration::MarginRight(ref value) => {
                                margin_right = Some(value)
                            },
                            PropertyDeclaration::MarginBottom(ref value) => {
                                margin_bottom = Some(value)
                            },
                            PropertyDeclaration::MarginLeft(ref value) => {
                                margin_left = Some(value)
                            },
                        _ => {}
                    };
                }

                // If any of the expected variables are missing, return an error
                match (
                        margin_top,
                        margin_right,
                        margin_bottom,
                        margin_left,
                ) {

                    (
                        Some(margin_top),
                        Some(margin_right),
                        Some(margin_bottom),
                        Some(margin_left),
                    ) =>
                    Ok(LonghandsToSerialize {
                            margin_top: margin_top,
                            margin_right: margin_right,
                            margin_bottom: margin_bottom,
                            margin_left: margin_left,
                    }),
                    _ => Err(())
                }
            }
        }

        /// Parse the given shorthand and fill the result into the
        /// `declarations` vector.
        pub fn parse_into<'i, 't>(declarations: &mut SourcePropertyDeclaration,
                                  context: &ParserContext, input: &mut Parser<'i, 't>)
                                  -> Result<(), ParseError<'i>> {
            input.parse_entirely(|input| parse_value(context, input)).map(|longhands| {
                    declarations.push(PropertyDeclaration::MarginTop(
                        longhands.margin_top
                    ));
                    declarations.push(PropertyDeclaration::MarginRight(
                        longhands.margin_right
                    ));
                    declarations.push(PropertyDeclaration::MarginBottom(
                        longhands.margin_bottom
                    ));
                    declarations.push(PropertyDeclaration::MarginLeft(
                        longhands.margin_left
                    ));
            })
        }

        
        #[allow(unused_imports)]
        use parser::Parse;
        use values::generics::rect::Rect;
        use values::specified;

        pub fn parse_value<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<Longhands, ParseError<'i>> {
            let rect = Rect::parse_with(context, input, |_c, i| {
                specified::LengthOrPercentageOrAuto::parse_quirky(_c, i, specified::AllowQuirks::Yes)
            })?;
            Ok(expanded! {
                    margin_top: rect.0,
                    margin_right: rect.1,
                    margin_bottom: rect.2,
                    margin_left: rect.3,
            })
        }

        impl<'a> ToCss for LonghandsToSerialize<'a> {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
                let rect = Rect::new(
                    &self.margin_top,
                    &self.margin_right,
                    &self.margin_bottom,
                    &self.margin_left,
                );
                rect.to_css(dest)
            }
        }
    
    }



    /* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */





    /// https://drafts.fxtf.org/css-masking/#propdef-mask
    pub mod mask {
        use cssparser::Parser;
        use parser::ParserContext;
        use properties::{PropertyDeclaration, SourcePropertyDeclaration, MaybeBoxed, longhands};
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use std::fmt;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use style_traits::ToCss;

        pub struct Longhands {
                pub mask_mode:
                    longhands::mask_mode::SpecifiedValue
                    ,
                pub mask_repeat:
                    longhands::mask_repeat::SpecifiedValue
                    ,
                pub mask_clip:
                    longhands::mask_clip::SpecifiedValue
                    ,
                pub mask_origin:
                    longhands::mask_origin::SpecifiedValue
                    ,
                pub mask_composite:
                    longhands::mask_composite::SpecifiedValue
                    ,
                pub mask_position_x:
                    longhands::mask_position_x::SpecifiedValue
                    ,
                pub mask_position_y:
                    longhands::mask_position_y::SpecifiedValue
                    ,
                pub mask_size:
                    longhands::mask_size::SpecifiedValue
                    ,
                pub mask_image:
                    longhands::mask_image::SpecifiedValue
                    ,
        }

        /// Represents a serializable set of all of the longhand properties that
        /// correspond to a shorthand.
        pub struct LonghandsToSerialize<'a> {
                pub mask_mode:
                    &'a longhands::mask_mode::SpecifiedValue,
                pub mask_repeat:
                    &'a longhands::mask_repeat::SpecifiedValue,
                pub mask_clip:
                    &'a longhands::mask_clip::SpecifiedValue,
                pub mask_origin:
                    &'a longhands::mask_origin::SpecifiedValue,
                pub mask_composite:
                    &'a longhands::mask_composite::SpecifiedValue,
                pub mask_position_x:
                    &'a longhands::mask_position_x::SpecifiedValue,
                pub mask_position_y:
                    &'a longhands::mask_position_y::SpecifiedValue,
                pub mask_size:
                    &'a longhands::mask_size::SpecifiedValue,
                pub mask_image:
                    &'a longhands::mask_image::SpecifiedValue,
        }

        impl<'a> LonghandsToSerialize<'a> {
            /// Tries to get a serializable set of longhands given a set of
            /// property declarations.
            pub fn from_iter<I>(iter: I) -> Result<Self, ()>
                where I: Iterator<Item=&'a PropertyDeclaration>,
            {
                // Define all of the expected variables that correspond to the shorthand
                    let mut mask_mode = None;
                    let mut mask_repeat = None;
                    let mut mask_clip = None;
                    let mut mask_origin = None;
                    let mut mask_composite = None;
                    let mut mask_position_x = None;
                    let mut mask_position_y = None;
                    let mut mask_size = None;
                    let mut mask_image = None;

                // Attempt to assign the incoming declarations to the expected variables
                for longhand in iter {
                    match *longhand {
                            PropertyDeclaration::MaskMode(ref value) => {
                                mask_mode = Some(value)
                            },
                            PropertyDeclaration::MaskRepeat(ref value) => {
                                mask_repeat = Some(value)
                            },
                            PropertyDeclaration::MaskClip(ref value) => {
                                mask_clip = Some(value)
                            },
                            PropertyDeclaration::MaskOrigin(ref value) => {
                                mask_origin = Some(value)
                            },
                            PropertyDeclaration::MaskComposite(ref value) => {
                                mask_composite = Some(value)
                            },
                            PropertyDeclaration::MaskPositionX(ref value) => {
                                mask_position_x = Some(value)
                            },
                            PropertyDeclaration::MaskPositionY(ref value) => {
                                mask_position_y = Some(value)
                            },
                            PropertyDeclaration::MaskSize(ref value) => {
                                mask_size = Some(value)
                            },
                            PropertyDeclaration::MaskImage(ref value) => {
                                mask_image = Some(value)
                            },
                        _ => {}
                    };
                }

                // If any of the expected variables are missing, return an error
                match (
                        mask_mode,
                        mask_repeat,
                        mask_clip,
                        mask_origin,
                        mask_composite,
                        mask_position_x,
                        mask_position_y,
                        mask_size,
                        mask_image,
                ) {

                    (
                        Some(mask_mode),
                        Some(mask_repeat),
                        Some(mask_clip),
                        Some(mask_origin),
                        Some(mask_composite),
                        Some(mask_position_x),
                        Some(mask_position_y),
                        Some(mask_size),
                        Some(mask_image),
                    ) =>
                    Ok(LonghandsToSerialize {
                            mask_mode: mask_mode,
                            mask_repeat: mask_repeat,
                            mask_clip: mask_clip,
                            mask_origin: mask_origin,
                            mask_composite: mask_composite,
                            mask_position_x: mask_position_x,
                            mask_position_y: mask_position_y,
                            mask_size: mask_size,
                            mask_image: mask_image,
                    }),
                    _ => Err(())
                }
            }
        }

        /// Parse the given shorthand and fill the result into the
        /// `declarations` vector.
        pub fn parse_into<'i, 't>(declarations: &mut SourcePropertyDeclaration,
                                  context: &ParserContext, input: &mut Parser<'i, 't>)
                                  -> Result<(), ParseError<'i>> {
            input.parse_entirely(|input| parse_value(context, input)).map(|longhands| {
                    declarations.push(PropertyDeclaration::MaskMode(
                        longhands.mask_mode
                    ));
                    declarations.push(PropertyDeclaration::MaskRepeat(
                        longhands.mask_repeat
                    ));
                    declarations.push(PropertyDeclaration::MaskClip(
                        longhands.mask_clip
                    ));
                    declarations.push(PropertyDeclaration::MaskOrigin(
                        longhands.mask_origin
                    ));
                    declarations.push(PropertyDeclaration::MaskComposite(
                        longhands.mask_composite
                    ));
                    declarations.push(PropertyDeclaration::MaskPositionX(
                        longhands.mask_position_x
                    ));
                    declarations.push(PropertyDeclaration::MaskPositionY(
                        longhands.mask_position_y
                    ));
                    declarations.push(PropertyDeclaration::MaskSize(
                        longhands.mask_size
                    ));
                    declarations.push(PropertyDeclaration::MaskImage(
                        longhands.mask_image
                    ));
            })
        }

        
    use properties::longhands::{mask_mode, mask_repeat, mask_clip, mask_origin, mask_composite, mask_position_x,
                                mask_position_y};
    use properties::longhands::{mask_size, mask_image};
    use values::specified::{Position, PositionComponent};
    use parser::Parse;

    impl From<mask_origin::single_value::SpecifiedValue> for mask_clip::single_value::SpecifiedValue {
        fn from(origin: mask_origin::single_value::SpecifiedValue) -> mask_clip::single_value::SpecifiedValue {
            match origin {
                mask_origin::single_value::SpecifiedValue::content_box =>
                    mask_clip::single_value::SpecifiedValue::content_box,
                mask_origin::single_value::SpecifiedValue::padding_box =>
                    mask_clip::single_value::SpecifiedValue::padding_box,
                mask_origin::single_value::SpecifiedValue::border_box =>
                    mask_clip::single_value::SpecifiedValue::border_box,
                mask_origin::single_value::SpecifiedValue::fill_box =>
                    mask_clip::single_value::SpecifiedValue::fill_box,
                mask_origin::single_value::SpecifiedValue::stroke_box =>
                    mask_clip::single_value::SpecifiedValue::stroke_box,
                mask_origin::single_value::SpecifiedValue::view_box =>
                    mask_clip::single_value::SpecifiedValue::view_box,
            }
        }
    }

    pub fn parse_value<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                               -> Result<Longhands, ParseError<'i>> {
            // Vec grows from 0 to 4 by default on first push().  So allocate
            // with capacity 1, so in the common case of only one item we don't
            // way overallocate.  Note that we always push at least one item if
            // parsing succeeds.
            let mut mask_image = mask_image::SpecifiedValue(Vec::with_capacity(1));
            // Vec grows from 0 to 4 by default on first push().  So allocate
            // with capacity 1, so in the common case of only one item we don't
            // way overallocate.  Note that we always push at least one item if
            // parsing succeeds.
            let mut mask_mode = mask_mode::SpecifiedValue(Vec::with_capacity(1));
            // Vec grows from 0 to 4 by default on first push().  So allocate
            // with capacity 1, so in the common case of only one item we don't
            // way overallocate.  Note that we always push at least one item if
            // parsing succeeds.
            let mut mask_position_x = mask_position_x::SpecifiedValue(Vec::with_capacity(1));
            // Vec grows from 0 to 4 by default on first push().  So allocate
            // with capacity 1, so in the common case of only one item we don't
            // way overallocate.  Note that we always push at least one item if
            // parsing succeeds.
            let mut mask_position_y = mask_position_y::SpecifiedValue(Vec::with_capacity(1));
            // Vec grows from 0 to 4 by default on first push().  So allocate
            // with capacity 1, so in the common case of only one item we don't
            // way overallocate.  Note that we always push at least one item if
            // parsing succeeds.
            let mut mask_size = mask_size::SpecifiedValue(Vec::with_capacity(1));
            // Vec grows from 0 to 4 by default on first push().  So allocate
            // with capacity 1, so in the common case of only one item we don't
            // way overallocate.  Note that we always push at least one item if
            // parsing succeeds.
            let mut mask_repeat = mask_repeat::SpecifiedValue(Vec::with_capacity(1));
            // Vec grows from 0 to 4 by default on first push().  So allocate
            // with capacity 1, so in the common case of only one item we don't
            // way overallocate.  Note that we always push at least one item if
            // parsing succeeds.
            let mut mask_origin = mask_origin::SpecifiedValue(Vec::with_capacity(1));
            // Vec grows from 0 to 4 by default on first push().  So allocate
            // with capacity 1, so in the common case of only one item we don't
            // way overallocate.  Note that we always push at least one item if
            // parsing succeeds.
            let mut mask_clip = mask_clip::SpecifiedValue(Vec::with_capacity(1));
            // Vec grows from 0 to 4 by default on first push().  So allocate
            // with capacity 1, so in the common case of only one item we don't
            // way overallocate.  Note that we always push at least one item if
            // parsing succeeds.
            let mut mask_composite = mask_composite::SpecifiedValue(Vec::with_capacity(1));

        input.parse_comma_separated(|input| {
                let mut image = None;
                let mut mode = None;
                let mut position = None;
                let mut size = None;
                let mut repeat = None;
                let mut origin = None;
                let mut clip = None;
                let mut composite = None;
            loop {
                if image.is_none() {
                    if let Ok(value) = input.try(|input| mask_image::single_value
                                                                   ::parse(context, input)) {
                        image = Some(value);
                        continue
                    }
                }
                if position.is_none() {
                    if let Ok(value) = input.try(|input| Position::parse(context, input)) {
                        position = Some(value);

                        // Parse mask size, if applicable.
                        size = input.try(|input| {
                            input.expect_delim('/')?;
                            mask_size::single_value::parse(context, input)
                        }).ok();

                        continue
                    }
                }
                    if repeat.is_none() {
                        if let Ok(value) = input.try(|input| mask_repeat::single_value
                                                                               ::parse(context, input)) {
                            repeat = Some(value);
                            continue
                        }
                    }
                    if origin.is_none() {
                        if let Ok(value) = input.try(|input| mask_origin::single_value
                                                                               ::parse(context, input)) {
                            origin = Some(value);
                            continue
                        }
                    }
                    if clip.is_none() {
                        if let Ok(value) = input.try(|input| mask_clip::single_value
                                                                               ::parse(context, input)) {
                            clip = Some(value);
                            continue
                        }
                    }
                    if composite.is_none() {
                        if let Ok(value) = input.try(|input| mask_composite::single_value
                                                                               ::parse(context, input)) {
                            composite = Some(value);
                            continue
                        }
                    }
                    if mode.is_none() {
                        if let Ok(value) = input.try(|input| mask_mode::single_value
                                                                               ::parse(context, input)) {
                            mode = Some(value);
                            continue
                        }
                    }
                break
            }
            if clip.is_none() {
                if let Some(origin) = origin {
                    clip = Some(mask_clip::single_value::SpecifiedValue::from(origin));
                }
            }
            let mut any = false;
                any = any || image.is_some();
                any = any || mode.is_some();
                any = any || position.is_some();
                any = any || size.is_some();
                any = any || repeat.is_some();
                any = any || origin.is_some();
                any = any || clip.is_some();
                any = any || composite.is_some();
            if any {
                if let Some(position) = position {
                    mask_position_x.0.push(position.horizontal);
                    mask_position_y.0.push(position.vertical);
                } else {
                    mask_position_x.0.push(PositionComponent::zero());
                    mask_position_y.0.push(PositionComponent::zero());
                }
                    if let Some(m_image) = image {
                        mask_image.0.push(m_image);
                    } else {
                        mask_image.0.push(mask_image::single_value
                                                        ::get_initial_specified_value());
                    }
                    if let Some(m_mode) = mode {
                        mask_mode.0.push(m_mode);
                    } else {
                        mask_mode.0.push(mask_mode::single_value
                                                        ::get_initial_specified_value());
                    }
                    if let Some(m_size) = size {
                        mask_size.0.push(m_size);
                    } else {
                        mask_size.0.push(mask_size::single_value
                                                        ::get_initial_specified_value());
                    }
                    if let Some(m_repeat) = repeat {
                        mask_repeat.0.push(m_repeat);
                    } else {
                        mask_repeat.0.push(mask_repeat::single_value
                                                        ::get_initial_specified_value());
                    }
                    if let Some(m_origin) = origin {
                        mask_origin.0.push(m_origin);
                    } else {
                        mask_origin.0.push(mask_origin::single_value
                                                        ::get_initial_specified_value());
                    }
                    if let Some(m_clip) = clip {
                        mask_clip.0.push(m_clip);
                    } else {
                        mask_clip.0.push(mask_clip::single_value
                                                        ::get_initial_specified_value());
                    }
                    if let Some(m_composite) = composite {
                        mask_composite.0.push(m_composite);
                    } else {
                        mask_composite.0.push(mask_composite::single_value
                                                        ::get_initial_specified_value());
                    }
                Ok(())
            } else {
                Err(StyleParseError::UnspecifiedError.into())
            }
        })?;

        Ok(expanded! {
                mask_image: mask_image,
                mask_mode: mask_mode,
                mask_position_x: mask_position_x,
                mask_position_y: mask_position_y,
                mask_size: mask_size,
                mask_repeat: mask_repeat,
                mask_origin: mask_origin,
                mask_clip: mask_clip,
                mask_composite: mask_composite,
         })
    }

    impl<'a> ToCss for LonghandsToSerialize<'a>  {
        fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
            use properties::longhands::mask_origin::single_value::computed_value::T as Origin;
            use properties::longhands::mask_clip::single_value::computed_value::T as Clip;

            let len = self.mask_image.0.len();
            if len == 0 {
                return Ok(());
            }
                if self.mask_mode.0.len() != len {
                    return Ok(());
                }
                if self.mask_position_x.0.len() != len {
                    return Ok(());
                }
                if self.mask_position_y.0.len() != len {
                    return Ok(());
                }
                if self.mask_size.0.len() != len {
                    return Ok(());
                }
                if self.mask_repeat.0.len() != len {
                    return Ok(());
                }
                if self.mask_origin.0.len() != len {
                    return Ok(());
                }
                if self.mask_clip.0.len() != len {
                    return Ok(());
                }
                if self.mask_composite.0.len() != len {
                    return Ok(());
                }

            for i in 0..len {
                if i > 0 {
                    dest.write_str(", ")?;
                }

                    let image = &self.mask_image.0[i];
                    let mode = &self.mask_mode.0[i];
                    let position_x = &self.mask_position_x.0[i];
                    let position_y = &self.mask_position_y.0[i];
                    let size = &self.mask_size.0[i];
                    let repeat = &self.mask_repeat.0[i];
                    let origin = &self.mask_origin.0[i];
                    let clip = &self.mask_clip.0[i];
                    let composite = &self.mask_composite.0[i];

                image.to_css(dest)?;
                dest.write_str(" ")?;
                mode.to_css(dest)?;
                dest.write_str(" ")?;

                Position {
                    horizontal: position_x.clone(),
                    vertical: position_y.clone()
                }.to_css(dest)?;

                if *size != mask_size::single_value::get_initial_specified_value() {
                    dest.write_str(" / ")?;
                    size.to_css(dest)?;
                }
                dest.write_str(" ")?;
                repeat.to_css(dest)?;

                if *origin != Origin::border_box || *clip != Clip::border_box {
                    dest.write_str(" ")?;
                    origin.to_css(dest)?;
                    if *clip != From::from(*origin) {
                        dest.write_str(" ")?;
                        clip.to_css(dest)?;
                    }
                }

                dest.write_str(" ")?;
                composite.to_css(dest)?;
            }

            Ok(())
        }
    }

    }




    /// https://drafts.csswg.org/css-masks-4/#the-mask-position
    pub mod mask_position {
        use cssparser::Parser;
        use parser::ParserContext;
        use properties::{PropertyDeclaration, SourcePropertyDeclaration, MaybeBoxed, longhands};
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use std::fmt;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use style_traits::ToCss;

        pub struct Longhands {
                pub mask_position_x:
                    longhands::mask_position_x::SpecifiedValue
                    ,
                pub mask_position_y:
                    longhands::mask_position_y::SpecifiedValue
                    ,
        }

        /// Represents a serializable set of all of the longhand properties that
        /// correspond to a shorthand.
        pub struct LonghandsToSerialize<'a> {
                pub mask_position_x:
                    &'a longhands::mask_position_x::SpecifiedValue,
                pub mask_position_y:
                    &'a longhands::mask_position_y::SpecifiedValue,
        }

        impl<'a> LonghandsToSerialize<'a> {
            /// Tries to get a serializable set of longhands given a set of
            /// property declarations.
            pub fn from_iter<I>(iter: I) -> Result<Self, ()>
                where I: Iterator<Item=&'a PropertyDeclaration>,
            {
                // Define all of the expected variables that correspond to the shorthand
                    let mut mask_position_x = None;
                    let mut mask_position_y = None;

                // Attempt to assign the incoming declarations to the expected variables
                for longhand in iter {
                    match *longhand {
                            PropertyDeclaration::MaskPositionX(ref value) => {
                                mask_position_x = Some(value)
                            },
                            PropertyDeclaration::MaskPositionY(ref value) => {
                                mask_position_y = Some(value)
                            },
                        _ => {}
                    };
                }

                // If any of the expected variables are missing, return an error
                match (
                        mask_position_x,
                        mask_position_y,
                ) {

                    (
                        Some(mask_position_x),
                        Some(mask_position_y),
                    ) =>
                    Ok(LonghandsToSerialize {
                            mask_position_x: mask_position_x,
                            mask_position_y: mask_position_y,
                    }),
                    _ => Err(())
                }
            }
        }

        /// Parse the given shorthand and fill the result into the
        /// `declarations` vector.
        pub fn parse_into<'i, 't>(declarations: &mut SourcePropertyDeclaration,
                                  context: &ParserContext, input: &mut Parser<'i, 't>)
                                  -> Result<(), ParseError<'i>> {
            input.parse_entirely(|input| parse_value(context, input)).map(|longhands| {
                    declarations.push(PropertyDeclaration::MaskPositionX(
                        longhands.mask_position_x
                    ));
                    declarations.push(PropertyDeclaration::MaskPositionY(
                        longhands.mask_position_y
                    ));
            })
        }

        
    use properties::longhands::{mask_position_x,mask_position_y};
    use values::specified::position::Position;
    use parser::Parse;

    pub fn parse_value<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                               -> Result<Longhands, ParseError<'i>> {
        // Vec grows from 0 to 4 by default on first push().  So allocate with
        // capacity 1, so in the common case of only one item we don't way
        // overallocate.  Note that we always push at least one item if parsing
        // succeeds.
        let mut position_x = mask_position_x::SpecifiedValue(Vec::with_capacity(1));
        let mut position_y = mask_position_y::SpecifiedValue(Vec::with_capacity(1));
        let mut any = false;

        input.parse_comma_separated(|input| {
            let value = Position::parse(context, input)?;
            position_x.0.push(value.horizontal);
            position_y.0.push(value.vertical);
            any = true;
            Ok(())
        })?;
        if any == false {
            return Err(StyleParseError::UnspecifiedError.into());
        }

        Ok(expanded! {
            mask_position_x: position_x,
            mask_position_y: position_y,
        })
    }

    impl<'a> ToCss for LonghandsToSerialize<'a>  {
        fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
            let len = self.mask_position_x.0.len();
            if len == 0 || self.mask_position_y.0.len() != len {
                return Ok(());
            }

            for i in 0..len {
                Position {
                    horizontal: self.mask_position_x.0[i].clone(),
                    vertical: self.mask_position_y.0[i].clone()
                }.to_css(dest)?;

                if i < len - 1 {
                    dest.write_str(", ")?;
                }
            }

            Ok(())
        }
    }

    }


    /* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */





    /// https://drafts.csswg.org/css-ui/#propdef-outline
    pub mod outline {
        use cssparser::Parser;
        use parser::ParserContext;
        use properties::{PropertyDeclaration, SourcePropertyDeclaration, MaybeBoxed, longhands};
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use std::fmt;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use style_traits::ToCss;

        pub struct Longhands {
                pub outline_width:
                    longhands::outline_width::SpecifiedValue
                    ,
                pub outline_style:
                    longhands::outline_style::SpecifiedValue
                    ,
                pub outline_color:
                    longhands::outline_color::SpecifiedValue
                    ,
        }

        /// Represents a serializable set of all of the longhand properties that
        /// correspond to a shorthand.
        #[derive(ToCss)]
        pub struct LonghandsToSerialize<'a> {
                pub outline_width:
                    &'a longhands::outline_width::SpecifiedValue,
                pub outline_style:
                    &'a longhands::outline_style::SpecifiedValue,
                pub outline_color:
                    &'a longhands::outline_color::SpecifiedValue,
        }

        impl<'a> LonghandsToSerialize<'a> {
            /// Tries to get a serializable set of longhands given a set of
            /// property declarations.
            pub fn from_iter<I>(iter: I) -> Result<Self, ()>
                where I: Iterator<Item=&'a PropertyDeclaration>,
            {
                // Define all of the expected variables that correspond to the shorthand
                    let mut outline_width = None;
                    let mut outline_style = None;
                    let mut outline_color = None;

                // Attempt to assign the incoming declarations to the expected variables
                for longhand in iter {
                    match *longhand {
                            PropertyDeclaration::OutlineWidth(ref value) => {
                                outline_width = Some(value)
                            },
                            PropertyDeclaration::OutlineStyle(ref value) => {
                                outline_style = Some(value)
                            },
                            PropertyDeclaration::OutlineColor(ref value) => {
                                outline_color = Some(value)
                            },
                        _ => {}
                    };
                }

                // If any of the expected variables are missing, return an error
                match (
                        outline_width,
                        outline_style,
                        outline_color,
                ) {

                    (
                        Some(outline_width),
                        Some(outline_style),
                        Some(outline_color),
                    ) =>
                    Ok(LonghandsToSerialize {
                            outline_width: outline_width,
                            outline_style: outline_style,
                            outline_color: outline_color,
                    }),
                    _ => Err(())
                }
            }
        }

        /// Parse the given shorthand and fill the result into the
        /// `declarations` vector.
        pub fn parse_into<'i, 't>(declarations: &mut SourcePropertyDeclaration,
                                  context: &ParserContext, input: &mut Parser<'i, 't>)
                                  -> Result<(), ParseError<'i>> {
            input.parse_entirely(|input| parse_value(context, input)).map(|longhands| {
                    declarations.push(PropertyDeclaration::OutlineWidth(
                        longhands.outline_width
                    ));
                    declarations.push(PropertyDeclaration::OutlineStyle(
                        longhands.outline_style
                    ));
                    declarations.push(PropertyDeclaration::OutlineColor(
                        longhands.outline_color
                    ));
            })
        }

        
    use properties::longhands::{outline_color, outline_width, outline_style};
    use values::specified;
    use parser::Parse;

    pub fn parse_value<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                               -> Result<Longhands, ParseError<'i>> {
        let _unused = context;
        let mut color = None;
        let mut style = None;
        let mut width = None;
        let mut any = false;
        loop {
            if color.is_none() {
                if let Ok(value) = input.try(|i| specified::Color::parse(context, i)) {
                    color = Some(value);
                    any = true;
                    continue
                }
            }
            if style.is_none() {
                if let Ok(value) = input.try(|input| outline_style::parse(context, input)) {
                    style = Some(value);
                    any = true;
                    continue
                }
            }
            if width.is_none() {
                if let Ok(value) = input.try(|input| outline_width::parse(context, input)) {
                    width = Some(value);
                    any = true;
                    continue
                }
            }
            break
        }
        if any {
            Ok(expanded! {
                outline_color: unwrap_or_initial!(outline_color, color),
                outline_style: unwrap_or_initial!(outline_style, style),
                outline_width: unwrap_or_initial!(outline_width, width),
            })
        } else {
            Err(StyleParseError::UnspecifiedError.into())
        }
    }

    }


// The -moz-outline-radius shorthand is non-standard and not on a standards track.


    /// Nonstandard (https://developer.mozilla.org/en-US/docs/Web/CSS/-moz-outline-radius)
    pub mod _moz_outline_radius {
        use cssparser::Parser;
        use parser::ParserContext;
        use properties::{PropertyDeclaration, SourcePropertyDeclaration, MaybeBoxed, longhands};
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use std::fmt;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use style_traits::ToCss;

        pub struct Longhands {
                pub _moz_outline_radius_topleft:
                        Box<
                    longhands::_moz_outline_radius_topleft::SpecifiedValue
                        >
                    ,
                pub _moz_outline_radius_topright:
                        Box<
                    longhands::_moz_outline_radius_topright::SpecifiedValue
                        >
                    ,
                pub _moz_outline_radius_bottomright:
                        Box<
                    longhands::_moz_outline_radius_bottomright::SpecifiedValue
                        >
                    ,
                pub _moz_outline_radius_bottomleft:
                        Box<
                    longhands::_moz_outline_radius_bottomleft::SpecifiedValue
                        >
                    ,
        }

        /// Represents a serializable set of all of the longhand properties that
        /// correspond to a shorthand.
        pub struct LonghandsToSerialize<'a> {
                pub _moz_outline_radius_topleft:
                    &'a longhands::_moz_outline_radius_topleft::SpecifiedValue,
                pub _moz_outline_radius_topright:
                    &'a longhands::_moz_outline_radius_topright::SpecifiedValue,
                pub _moz_outline_radius_bottomright:
                    &'a longhands::_moz_outline_radius_bottomright::SpecifiedValue,
                pub _moz_outline_radius_bottomleft:
                    &'a longhands::_moz_outline_radius_bottomleft::SpecifiedValue,
        }

        impl<'a> LonghandsToSerialize<'a> {
            /// Tries to get a serializable set of longhands given a set of
            /// property declarations.
            pub fn from_iter<I>(iter: I) -> Result<Self, ()>
                where I: Iterator<Item=&'a PropertyDeclaration>,
            {
                // Define all of the expected variables that correspond to the shorthand
                    let mut _moz_outline_radius_topleft = None;
                    let mut _moz_outline_radius_topright = None;
                    let mut _moz_outline_radius_bottomright = None;
                    let mut _moz_outline_radius_bottomleft = None;

                // Attempt to assign the incoming declarations to the expected variables
                for longhand in iter {
                    match *longhand {
                            PropertyDeclaration::MozOutlineRadiusTopleft(ref value) => {
                                _moz_outline_radius_topleft = Some(value)
                            },
                            PropertyDeclaration::MozOutlineRadiusTopright(ref value) => {
                                _moz_outline_radius_topright = Some(value)
                            },
                            PropertyDeclaration::MozOutlineRadiusBottomright(ref value) => {
                                _moz_outline_radius_bottomright = Some(value)
                            },
                            PropertyDeclaration::MozOutlineRadiusBottomleft(ref value) => {
                                _moz_outline_radius_bottomleft = Some(value)
                            },
                        _ => {}
                    };
                }

                // If any of the expected variables are missing, return an error
                match (
                        _moz_outline_radius_topleft,
                        _moz_outline_radius_topright,
                        _moz_outline_radius_bottomright,
                        _moz_outline_radius_bottomleft,
                ) {

                    (
                        Some(_moz_outline_radius_topleft),
                        Some(_moz_outline_radius_topright),
                        Some(_moz_outline_radius_bottomright),
                        Some(_moz_outline_radius_bottomleft),
                    ) =>
                    Ok(LonghandsToSerialize {
                            _moz_outline_radius_topleft: _moz_outline_radius_topleft,
                            _moz_outline_radius_topright: _moz_outline_radius_topright,
                            _moz_outline_radius_bottomright: _moz_outline_radius_bottomright,
                            _moz_outline_radius_bottomleft: _moz_outline_radius_bottomleft,
                    }),
                    _ => Err(())
                }
            }
        }

        /// Parse the given shorthand and fill the result into the
        /// `declarations` vector.
        pub fn parse_into<'i, 't>(declarations: &mut SourcePropertyDeclaration,
                                  context: &ParserContext, input: &mut Parser<'i, 't>)
                                  -> Result<(), ParseError<'i>> {
            input.parse_entirely(|input| parse_value(context, input)).map(|longhands| {
                    declarations.push(PropertyDeclaration::MozOutlineRadiusTopleft(
                        longhands._moz_outline_radius_topleft
                    ));
                    declarations.push(PropertyDeclaration::MozOutlineRadiusTopright(
                        longhands._moz_outline_radius_topright
                    ));
                    declarations.push(PropertyDeclaration::MozOutlineRadiusBottomright(
                        longhands._moz_outline_radius_bottomright
                    ));
                    declarations.push(PropertyDeclaration::MozOutlineRadiusBottomleft(
                        longhands._moz_outline_radius_bottomleft
                    ));
            })
        }

        
    use values::generics::rect::Rect;
    use values::specified::border::BorderRadius;
    use parser::Parse;

    pub fn parse_value<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                               -> Result<Longhands, ParseError<'i>> {
        let radii = BorderRadius::parse(context, input)?;
        Ok(expanded! {
            _moz_outline_radius_topleft: radii.top_left,
            _moz_outline_radius_topright: radii.top_right,
            _moz_outline_radius_bottomright: radii.bottom_right,
            _moz_outline_radius_bottomleft: radii.bottom_left,
        })
    }

    impl<'a> ToCss for LonghandsToSerialize<'a>  {
        fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
            use values::generics::border::BorderCornerRadius;

            let LonghandsToSerialize {
                _moz_outline_radius_topleft: &BorderCornerRadius(ref tl),
                _moz_outline_radius_topright: &BorderCornerRadius(ref tr),
                _moz_outline_radius_bottomright: &BorderCornerRadius(ref br),
                _moz_outline_radius_bottomleft: &BorderCornerRadius(ref bl),
            } = *self;

            let widths = Rect::new(tl.width(), tr.width(), br.width(), bl.width());
            let heights = Rect::new(tl.height(), tr.height(), br.height(), bl.height());

            BorderRadius::serialize_rects(widths, heights, dest)
        }
    }

    }


    /* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */




    
    

    /// https://drafts.csswg.org/css-box-3/#propdef-padding
    pub mod padding {
        use cssparser::Parser;
        use parser::ParserContext;
        use properties::{PropertyDeclaration, SourcePropertyDeclaration, MaybeBoxed, longhands};
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use std::fmt;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use style_traits::ToCss;

        pub struct Longhands {
                pub padding_top:
                    longhands::padding_top::SpecifiedValue
                    ,
                pub padding_right:
                    longhands::padding_right::SpecifiedValue
                    ,
                pub padding_bottom:
                    longhands::padding_bottom::SpecifiedValue
                    ,
                pub padding_left:
                    longhands::padding_left::SpecifiedValue
                    ,
        }

        /// Represents a serializable set of all of the longhand properties that
        /// correspond to a shorthand.
        pub struct LonghandsToSerialize<'a> {
                pub padding_top:
                    &'a longhands::padding_top::SpecifiedValue,
                pub padding_right:
                    &'a longhands::padding_right::SpecifiedValue,
                pub padding_bottom:
                    &'a longhands::padding_bottom::SpecifiedValue,
                pub padding_left:
                    &'a longhands::padding_left::SpecifiedValue,
        }

        impl<'a> LonghandsToSerialize<'a> {
            /// Tries to get a serializable set of longhands given a set of
            /// property declarations.
            pub fn from_iter<I>(iter: I) -> Result<Self, ()>
                where I: Iterator<Item=&'a PropertyDeclaration>,
            {
                // Define all of the expected variables that correspond to the shorthand
                    let mut padding_top = None;
                    let mut padding_right = None;
                    let mut padding_bottom = None;
                    let mut padding_left = None;

                // Attempt to assign the incoming declarations to the expected variables
                for longhand in iter {
                    match *longhand {
                            PropertyDeclaration::PaddingTop(ref value) => {
                                padding_top = Some(value)
                            },
                            PropertyDeclaration::PaddingRight(ref value) => {
                                padding_right = Some(value)
                            },
                            PropertyDeclaration::PaddingBottom(ref value) => {
                                padding_bottom = Some(value)
                            },
                            PropertyDeclaration::PaddingLeft(ref value) => {
                                padding_left = Some(value)
                            },
                        _ => {}
                    };
                }

                // If any of the expected variables are missing, return an error
                match (
                        padding_top,
                        padding_right,
                        padding_bottom,
                        padding_left,
                ) {

                    (
                        Some(padding_top),
                        Some(padding_right),
                        Some(padding_bottom),
                        Some(padding_left),
                    ) =>
                    Ok(LonghandsToSerialize {
                            padding_top: padding_top,
                            padding_right: padding_right,
                            padding_bottom: padding_bottom,
                            padding_left: padding_left,
                    }),
                    _ => Err(())
                }
            }
        }

        /// Parse the given shorthand and fill the result into the
        /// `declarations` vector.
        pub fn parse_into<'i, 't>(declarations: &mut SourcePropertyDeclaration,
                                  context: &ParserContext, input: &mut Parser<'i, 't>)
                                  -> Result<(), ParseError<'i>> {
            input.parse_entirely(|input| parse_value(context, input)).map(|longhands| {
                    declarations.push(PropertyDeclaration::PaddingTop(
                        longhands.padding_top
                    ));
                    declarations.push(PropertyDeclaration::PaddingRight(
                        longhands.padding_right
                    ));
                    declarations.push(PropertyDeclaration::PaddingBottom(
                        longhands.padding_bottom
                    ));
                    declarations.push(PropertyDeclaration::PaddingLeft(
                        longhands.padding_left
                    ));
            })
        }

        
        #[allow(unused_imports)]
        use parser::Parse;
        use values::generics::rect::Rect;
        use values::specified;

        pub fn parse_value<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                   -> Result<Longhands, ParseError<'i>> {
            let rect = Rect::parse_with(context, input, |_c, i| {
                specified::NonNegativeLengthOrPercentage::parse_quirky(_c, i, specified::AllowQuirks::Yes)
            })?;
            Ok(expanded! {
                    padding_top: rect.0,
                    padding_right: rect.1,
                    padding_bottom: rect.2,
                    padding_left: rect.3,
            })
        }

        impl<'a> ToCss for LonghandsToSerialize<'a> {
            fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
                let rect = Rect::new(
                    &self.padding_top,
                    &self.padding_right,
                    &self.padding_bottom,
                    &self.padding_left,
                );
                rect.to_css(dest)
            }
        }
    
    }



    /* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */





    /// https://drafts.csswg.org/css-flexbox/#flex-flow-property
    pub mod flex_flow {
        use cssparser::Parser;
        use parser::ParserContext;
        use properties::{PropertyDeclaration, SourcePropertyDeclaration, MaybeBoxed, longhands};
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use std::fmt;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use style_traits::ToCss;

        pub struct Longhands {
                pub flex_direction:
                    longhands::flex_direction::SpecifiedValue
                    ,
                pub flex_wrap:
                    longhands::flex_wrap::SpecifiedValue
                    ,
        }

        /// Represents a serializable set of all of the longhand properties that
        /// correspond to a shorthand.
        #[derive(ToCss)]
        pub struct LonghandsToSerialize<'a> {
                pub flex_direction:
                    &'a longhands::flex_direction::SpecifiedValue,
                pub flex_wrap:
                    &'a longhands::flex_wrap::SpecifiedValue,
        }

        impl<'a> LonghandsToSerialize<'a> {
            /// Tries to get a serializable set of longhands given a set of
            /// property declarations.
            pub fn from_iter<I>(iter: I) -> Result<Self, ()>
                where I: Iterator<Item=&'a PropertyDeclaration>,
            {
                // Define all of the expected variables that correspond to the shorthand
                    let mut flex_direction = None;
                    let mut flex_wrap = None;

                // Attempt to assign the incoming declarations to the expected variables
                for longhand in iter {
                    match *longhand {
                            PropertyDeclaration::FlexDirection(ref value) => {
                                flex_direction = Some(value)
                            },
                            PropertyDeclaration::FlexWrap(ref value) => {
                                flex_wrap = Some(value)
                            },
                        _ => {}
                    };
                }

                // If any of the expected variables are missing, return an error
                match (
                        flex_direction,
                        flex_wrap,
                ) {

                    (
                        Some(flex_direction),
                        Some(flex_wrap),
                    ) =>
                    Ok(LonghandsToSerialize {
                            flex_direction: flex_direction,
                            flex_wrap: flex_wrap,
                    }),
                    _ => Err(())
                }
            }
        }

        /// Parse the given shorthand and fill the result into the
        /// `declarations` vector.
        pub fn parse_into<'i, 't>(declarations: &mut SourcePropertyDeclaration,
                                  context: &ParserContext, input: &mut Parser<'i, 't>)
                                  -> Result<(), ParseError<'i>> {
            input.parse_entirely(|input| parse_value(context, input)).map(|longhands| {
                    declarations.push(PropertyDeclaration::FlexDirection(
                        longhands.flex_direction
                    ));
                    declarations.push(PropertyDeclaration::FlexWrap(
                        longhands.flex_wrap
                    ));
            })
        }

        
    use properties::longhands::{flex_direction, flex_wrap};

    pub fn parse_value<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                               -> Result<Longhands, ParseError<'i>> {
        let mut direction = None;
        let mut wrap = None;
        loop {
            if direction.is_none() {
                if let Ok(value) = input.try(|input| flex_direction::parse(context, input)) {
                    direction = Some(value);
                    continue
                }
            }
            if wrap.is_none() {
                if let Ok(value) = input.try(|input| flex_wrap::parse(context, input)) {
                    wrap = Some(value);
                    continue
                }
            }
            break
        }

        if direction.is_none() && wrap.is_none() {
            return Err(StyleParseError::UnspecifiedError.into())
        }
        Ok(expanded! {
            flex_direction: unwrap_or_initial!(flex_direction, direction),
            flex_wrap: unwrap_or_initial!(flex_wrap, wrap),
        })
    }

    }




    /// https://drafts.csswg.org/css-flexbox/#flex-property
    pub mod flex {
        use cssparser::Parser;
        use parser::ParserContext;
        use properties::{PropertyDeclaration, SourcePropertyDeclaration, MaybeBoxed, longhands};
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use std::fmt;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use style_traits::ToCss;

        pub struct Longhands {
                pub flex_grow:
                    longhands::flex_grow::SpecifiedValue
                    ,
                pub flex_shrink:
                    longhands::flex_shrink::SpecifiedValue
                    ,
                pub flex_basis:
                    longhands::flex_basis::SpecifiedValue
                    ,
        }

        /// Represents a serializable set of all of the longhand properties that
        /// correspond to a shorthand.
        #[derive(ToCss)]
        pub struct LonghandsToSerialize<'a> {
                pub flex_grow:
                    &'a longhands::flex_grow::SpecifiedValue,
                pub flex_shrink:
                    &'a longhands::flex_shrink::SpecifiedValue,
                pub flex_basis:
                    &'a longhands::flex_basis::SpecifiedValue,
        }

        impl<'a> LonghandsToSerialize<'a> {
            /// Tries to get a serializable set of longhands given a set of
            /// property declarations.
            pub fn from_iter<I>(iter: I) -> Result<Self, ()>
                where I: Iterator<Item=&'a PropertyDeclaration>,
            {
                // Define all of the expected variables that correspond to the shorthand
                    let mut flex_grow = None;
                    let mut flex_shrink = None;
                    let mut flex_basis = None;

                // Attempt to assign the incoming declarations to the expected variables
                for longhand in iter {
                    match *longhand {
                            PropertyDeclaration::FlexGrow(ref value) => {
                                flex_grow = Some(value)
                            },
                            PropertyDeclaration::FlexShrink(ref value) => {
                                flex_shrink = Some(value)
                            },
                            PropertyDeclaration::FlexBasis(ref value) => {
                                flex_basis = Some(value)
                            },
                        _ => {}
                    };
                }

                // If any of the expected variables are missing, return an error
                match (
                        flex_grow,
                        flex_shrink,
                        flex_basis,
                ) {

                    (
                        Some(flex_grow),
                        Some(flex_shrink),
                        Some(flex_basis),
                    ) =>
                    Ok(LonghandsToSerialize {
                            flex_grow: flex_grow,
                            flex_shrink: flex_shrink,
                            flex_basis: flex_basis,
                    }),
                    _ => Err(())
                }
            }
        }

        /// Parse the given shorthand and fill the result into the
        /// `declarations` vector.
        pub fn parse_into<'i, 't>(declarations: &mut SourcePropertyDeclaration,
                                  context: &ParserContext, input: &mut Parser<'i, 't>)
                                  -> Result<(), ParseError<'i>> {
            input.parse_entirely(|input| parse_value(context, input)).map(|longhands| {
                    declarations.push(PropertyDeclaration::FlexGrow(
                        longhands.flex_grow
                    ));
                    declarations.push(PropertyDeclaration::FlexShrink(
                        longhands.flex_shrink
                    ));
                    declarations.push(PropertyDeclaration::FlexBasis(
                        longhands.flex_basis
                    ));
            })
        }

        
    use parser::Parse;
    use values::specified::NonNegativeNumber;

    fn parse_flexibility<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                 -> Result<(NonNegativeNumber, Option<NonNegativeNumber>),ParseError<'i>> {
        let grow = NonNegativeNumber::parse(context, input)?;
        let shrink = input.try(|i| NonNegativeNumber::parse(context, i)).ok();
        Ok((grow, shrink))
    }

    pub fn parse_value<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                               -> Result<Longhands, ParseError<'i>> {
        let mut grow = None;
        let mut shrink = None;
        let mut basis = None;

        if input.try(|input| input.expect_ident_matching("none")).is_ok() {
            return Ok(expanded! {
                flex_grow: NonNegativeNumber::new(0.0),
                flex_shrink: NonNegativeNumber::new(0.0),
                flex_basis: longhands::flex_basis::SpecifiedValue::auto(),
            })
        }
        loop {
            if grow.is_none() {
                if let Ok((flex_grow, flex_shrink)) = input.try(|i| parse_flexibility(context, i)) {
                    grow = Some(flex_grow);
                    shrink = flex_shrink;
                    continue
                }
            }
            if basis.is_none() {
                if let Ok(value) = input.try(|input| longhands::flex_basis::parse_specified(context, input)) {
                    basis = Some(value);
                    continue
                }
            }
            break
        }

        if grow.is_none() && basis.is_none() {
            return Err(StyleParseError::UnspecifiedError.into())
        }
        Ok(expanded! {
            flex_grow: grow.unwrap_or(NonNegativeNumber::new(1.0)),
            flex_shrink: shrink.unwrap_or(NonNegativeNumber::new(1.0)),
            // Per spec, this should be SpecifiedValue::zero(), but all
            // browsers currently agree on using `0%`. This is a spec
            // change which hasn't been adopted by browsers:
            // https://github.com/w3c/csswg-drafts/commit/2c446befdf0f686217905bdd7c92409f6bd3921b
            flex_basis: basis.unwrap_or(longhands::flex_basis::SpecifiedValue::zero_percent()),
        })
    }

    }




    /// https://drafts.csswg.org/css-grid/#propdef-grid-gap
    pub mod grid_gap {
        use cssparser::Parser;
        use parser::ParserContext;
        use properties::{PropertyDeclaration, SourcePropertyDeclaration, MaybeBoxed, longhands};
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use std::fmt;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use style_traits::ToCss;

        pub struct Longhands {
                pub grid_row_gap:
                    longhands::grid_row_gap::SpecifiedValue
                    ,
                pub grid_column_gap:
                    longhands::grid_column_gap::SpecifiedValue
                    ,
        }

        /// Represents a serializable set of all of the longhand properties that
        /// correspond to a shorthand.
        pub struct LonghandsToSerialize<'a> {
                pub grid_row_gap:
                    &'a longhands::grid_row_gap::SpecifiedValue,
                pub grid_column_gap:
                    &'a longhands::grid_column_gap::SpecifiedValue,
        }

        impl<'a> LonghandsToSerialize<'a> {
            /// Tries to get a serializable set of longhands given a set of
            /// property declarations.
            pub fn from_iter<I>(iter: I) -> Result<Self, ()>
                where I: Iterator<Item=&'a PropertyDeclaration>,
            {
                // Define all of the expected variables that correspond to the shorthand
                    let mut grid_row_gap = None;
                    let mut grid_column_gap = None;

                // Attempt to assign the incoming declarations to the expected variables
                for longhand in iter {
                    match *longhand {
                            PropertyDeclaration::GridRowGap(ref value) => {
                                grid_row_gap = Some(value)
                            },
                            PropertyDeclaration::GridColumnGap(ref value) => {
                                grid_column_gap = Some(value)
                            },
                        _ => {}
                    };
                }

                // If any of the expected variables are missing, return an error
                match (
                        grid_row_gap,
                        grid_column_gap,
                ) {

                    (
                        Some(grid_row_gap),
                        Some(grid_column_gap),
                    ) =>
                    Ok(LonghandsToSerialize {
                            grid_row_gap: grid_row_gap,
                            grid_column_gap: grid_column_gap,
                    }),
                    _ => Err(())
                }
            }
        }

        /// Parse the given shorthand and fill the result into the
        /// `declarations` vector.
        pub fn parse_into<'i, 't>(declarations: &mut SourcePropertyDeclaration,
                                  context: &ParserContext, input: &mut Parser<'i, 't>)
                                  -> Result<(), ParseError<'i>> {
            input.parse_entirely(|input| parse_value(context, input)).map(|longhands| {
                    declarations.push(PropertyDeclaration::GridRowGap(
                        longhands.grid_row_gap
                    ));
                    declarations.push(PropertyDeclaration::GridColumnGap(
                        longhands.grid_column_gap
                    ));
            })
        }

        
  use properties::longhands::{grid_row_gap, grid_column_gap};

  pub fn parse_value<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                             -> Result<Longhands, ParseError<'i>> {
      let row_gap = grid_row_gap::parse(context, input)?;
      let column_gap = input.try(|input| grid_column_gap::parse(context, input)).unwrap_or(row_gap.clone());

      Ok(expanded! {
        grid_row_gap: row_gap,
        grid_column_gap: column_gap,
      })
  }

  impl<'a> ToCss for LonghandsToSerialize<'a>  {
      fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
          if self.grid_row_gap == self.grid_column_gap {
            self.grid_row_gap.to_css(dest)
          } else {
            self.grid_row_gap.to_css(dest)?;
            dest.write_str(" ")?;
            self.grid_column_gap.to_css(dest)
          }
      }
  }


    }




    /// https://drafts.csswg.org/css-grid/#propdef-grid-row
    pub mod grid_row {
        use cssparser::Parser;
        use parser::ParserContext;
        use properties::{PropertyDeclaration, SourcePropertyDeclaration, MaybeBoxed, longhands};
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use std::fmt;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use style_traits::ToCss;

        pub struct Longhands {
                pub grid_row_start:
                        Box<
                    longhands::grid_row_start::SpecifiedValue
                        >
                    ,
                pub grid_row_end:
                        Box<
                    longhands::grid_row_end::SpecifiedValue
                        >
                    ,
        }

        /// Represents a serializable set of all of the longhand properties that
        /// correspond to a shorthand.
        pub struct LonghandsToSerialize<'a> {
                pub grid_row_start:
                    &'a longhands::grid_row_start::SpecifiedValue,
                pub grid_row_end:
                    &'a longhands::grid_row_end::SpecifiedValue,
        }

        impl<'a> LonghandsToSerialize<'a> {
            /// Tries to get a serializable set of longhands given a set of
            /// property declarations.
            pub fn from_iter<I>(iter: I) -> Result<Self, ()>
                where I: Iterator<Item=&'a PropertyDeclaration>,
            {
                // Define all of the expected variables that correspond to the shorthand
                    let mut grid_row_start = None;
                    let mut grid_row_end = None;

                // Attempt to assign the incoming declarations to the expected variables
                for longhand in iter {
                    match *longhand {
                            PropertyDeclaration::GridRowStart(ref value) => {
                                grid_row_start = Some(value)
                            },
                            PropertyDeclaration::GridRowEnd(ref value) => {
                                grid_row_end = Some(value)
                            },
                        _ => {}
                    };
                }

                // If any of the expected variables are missing, return an error
                match (
                        grid_row_start,
                        grid_row_end,
                ) {

                    (
                        Some(grid_row_start),
                        Some(grid_row_end),
                    ) =>
                    Ok(LonghandsToSerialize {
                            grid_row_start: grid_row_start,
                            grid_row_end: grid_row_end,
                    }),
                    _ => Err(())
                }
            }
        }

        /// Parse the given shorthand and fill the result into the
        /// `declarations` vector.
        pub fn parse_into<'i, 't>(declarations: &mut SourcePropertyDeclaration,
                                  context: &ParserContext, input: &mut Parser<'i, 't>)
                                  -> Result<(), ParseError<'i>> {
            input.parse_entirely(|input| parse_value(context, input)).map(|longhands| {
                    declarations.push(PropertyDeclaration::GridRowStart(
                        longhands.grid_row_start
                    ));
                    declarations.push(PropertyDeclaration::GridRowEnd(
                        longhands.grid_row_end
                    ));
            })
        }

        
    use values::specified::GridLine;
    use parser::Parse;

    // NOTE: Since both the shorthands have the same code, we should (re-)use code from one to implement
    // the other. This might not be a big deal for now, but we should consider looking into this in the future
    // to limit the amount of code generated.
    pub fn parse_value<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                               -> Result<Longhands, ParseError<'i>> {
        let start = input.try(|i| GridLine::parse(context, i))?;
        let end = if input.try(|i| i.expect_delim('/')).is_ok() {
            GridLine::parse(context, input)?
        } else {
            let mut line = GridLine::auto();
            if start.line_num.is_none() && !start.is_span {
                line.ident = start.ident.clone();       // ident from start value should be taken
            }

            line
        };

        Ok(expanded! {
            grid_row_start: start,
            grid_row_end: end,
        })
    }

    impl<'a> ToCss for LonghandsToSerialize<'a> {
        fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
            self.grid_row_start.to_css(dest)?;
            dest.write_str(" / ")?;
            self.grid_row_end.to_css(dest)
        }
    }

    }



    /// https://drafts.csswg.org/css-grid/#propdef-grid-column
    pub mod grid_column {
        use cssparser::Parser;
        use parser::ParserContext;
        use properties::{PropertyDeclaration, SourcePropertyDeclaration, MaybeBoxed, longhands};
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use std::fmt;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use style_traits::ToCss;

        pub struct Longhands {
                pub grid_column_start:
                        Box<
                    longhands::grid_column_start::SpecifiedValue
                        >
                    ,
                pub grid_column_end:
                        Box<
                    longhands::grid_column_end::SpecifiedValue
                        >
                    ,
        }

        /// Represents a serializable set of all of the longhand properties that
        /// correspond to a shorthand.
        pub struct LonghandsToSerialize<'a> {
                pub grid_column_start:
                    &'a longhands::grid_column_start::SpecifiedValue,
                pub grid_column_end:
                    &'a longhands::grid_column_end::SpecifiedValue,
        }

        impl<'a> LonghandsToSerialize<'a> {
            /// Tries to get a serializable set of longhands given a set of
            /// property declarations.
            pub fn from_iter<I>(iter: I) -> Result<Self, ()>
                where I: Iterator<Item=&'a PropertyDeclaration>,
            {
                // Define all of the expected variables that correspond to the shorthand
                    let mut grid_column_start = None;
                    let mut grid_column_end = None;

                // Attempt to assign the incoming declarations to the expected variables
                for longhand in iter {
                    match *longhand {
                            PropertyDeclaration::GridColumnStart(ref value) => {
                                grid_column_start = Some(value)
                            },
                            PropertyDeclaration::GridColumnEnd(ref value) => {
                                grid_column_end = Some(value)
                            },
                        _ => {}
                    };
                }

                // If any of the expected variables are missing, return an error
                match (
                        grid_column_start,
                        grid_column_end,
                ) {

                    (
                        Some(grid_column_start),
                        Some(grid_column_end),
                    ) =>
                    Ok(LonghandsToSerialize {
                            grid_column_start: grid_column_start,
                            grid_column_end: grid_column_end,
                    }),
                    _ => Err(())
                }
            }
        }

        /// Parse the given shorthand and fill the result into the
        /// `declarations` vector.
        pub fn parse_into<'i, 't>(declarations: &mut SourcePropertyDeclaration,
                                  context: &ParserContext, input: &mut Parser<'i, 't>)
                                  -> Result<(), ParseError<'i>> {
            input.parse_entirely(|input| parse_value(context, input)).map(|longhands| {
                    declarations.push(PropertyDeclaration::GridColumnStart(
                        longhands.grid_column_start
                    ));
                    declarations.push(PropertyDeclaration::GridColumnEnd(
                        longhands.grid_column_end
                    ));
            })
        }

        
    use values::specified::GridLine;
    use parser::Parse;

    // NOTE: Since both the shorthands have the same code, we should (re-)use code from one to implement
    // the other. This might not be a big deal for now, but we should consider looking into this in the future
    // to limit the amount of code generated.
    pub fn parse_value<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                               -> Result<Longhands, ParseError<'i>> {
        let start = input.try(|i| GridLine::parse(context, i))?;
        let end = if input.try(|i| i.expect_delim('/')).is_ok() {
            GridLine::parse(context, input)?
        } else {
            let mut line = GridLine::auto();
            if start.line_num.is_none() && !start.is_span {
                line.ident = start.ident.clone();       // ident from start value should be taken
            }

            line
        };

        Ok(expanded! {
            grid_column_start: start,
            grid_column_end: end,
        })
    }

    impl<'a> ToCss for LonghandsToSerialize<'a> {
        fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
            self.grid_column_start.to_css(dest)?;
            dest.write_str(" / ")?;
            self.grid_column_end.to_css(dest)
        }
    }

    }




    /// https://drafts.csswg.org/css-grid/#propdef-grid-area
    pub mod grid_area {
        use cssparser::Parser;
        use parser::ParserContext;
        use properties::{PropertyDeclaration, SourcePropertyDeclaration, MaybeBoxed, longhands};
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use std::fmt;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use style_traits::ToCss;

        pub struct Longhands {
                pub grid_row_start:
                        Box<
                    longhands::grid_row_start::SpecifiedValue
                        >
                    ,
                pub grid_row_end:
                        Box<
                    longhands::grid_row_end::SpecifiedValue
                        >
                    ,
                pub grid_column_start:
                        Box<
                    longhands::grid_column_start::SpecifiedValue
                        >
                    ,
                pub grid_column_end:
                        Box<
                    longhands::grid_column_end::SpecifiedValue
                        >
                    ,
        }

        /// Represents a serializable set of all of the longhand properties that
        /// correspond to a shorthand.
        pub struct LonghandsToSerialize<'a> {
                pub grid_row_start:
                    &'a longhands::grid_row_start::SpecifiedValue,
                pub grid_row_end:
                    &'a longhands::grid_row_end::SpecifiedValue,
                pub grid_column_start:
                    &'a longhands::grid_column_start::SpecifiedValue,
                pub grid_column_end:
                    &'a longhands::grid_column_end::SpecifiedValue,
        }

        impl<'a> LonghandsToSerialize<'a> {
            /// Tries to get a serializable set of longhands given a set of
            /// property declarations.
            pub fn from_iter<I>(iter: I) -> Result<Self, ()>
                where I: Iterator<Item=&'a PropertyDeclaration>,
            {
                // Define all of the expected variables that correspond to the shorthand
                    let mut grid_row_start = None;
                    let mut grid_row_end = None;
                    let mut grid_column_start = None;
                    let mut grid_column_end = None;

                // Attempt to assign the incoming declarations to the expected variables
                for longhand in iter {
                    match *longhand {
                            PropertyDeclaration::GridRowStart(ref value) => {
                                grid_row_start = Some(value)
                            },
                            PropertyDeclaration::GridRowEnd(ref value) => {
                                grid_row_end = Some(value)
                            },
                            PropertyDeclaration::GridColumnStart(ref value) => {
                                grid_column_start = Some(value)
                            },
                            PropertyDeclaration::GridColumnEnd(ref value) => {
                                grid_column_end = Some(value)
                            },
                        _ => {}
                    };
                }

                // If any of the expected variables are missing, return an error
                match (
                        grid_row_start,
                        grid_row_end,
                        grid_column_start,
                        grid_column_end,
                ) {

                    (
                        Some(grid_row_start),
                        Some(grid_row_end),
                        Some(grid_column_start),
                        Some(grid_column_end),
                    ) =>
                    Ok(LonghandsToSerialize {
                            grid_row_start: grid_row_start,
                            grid_row_end: grid_row_end,
                            grid_column_start: grid_column_start,
                            grid_column_end: grid_column_end,
                    }),
                    _ => Err(())
                }
            }
        }

        /// Parse the given shorthand and fill the result into the
        /// `declarations` vector.
        pub fn parse_into<'i, 't>(declarations: &mut SourcePropertyDeclaration,
                                  context: &ParserContext, input: &mut Parser<'i, 't>)
                                  -> Result<(), ParseError<'i>> {
            input.parse_entirely(|input| parse_value(context, input)).map(|longhands| {
                    declarations.push(PropertyDeclaration::GridRowStart(
                        longhands.grid_row_start
                    ));
                    declarations.push(PropertyDeclaration::GridRowEnd(
                        longhands.grid_row_end
                    ));
                    declarations.push(PropertyDeclaration::GridColumnStart(
                        longhands.grid_column_start
                    ));
                    declarations.push(PropertyDeclaration::GridColumnEnd(
                        longhands.grid_column_end
                    ));
            })
        }

        
    use values::specified::GridLine;
    use parser::Parse;

    // The code is the same as `grid-{row,column}` except that this can have four values at most.
    pub fn parse_value<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                               -> Result<Longhands, ParseError<'i>> {
        fn line_with_ident_from(other: &GridLine) -> GridLine {
            let mut this = GridLine::auto();
            if other.line_num.is_none() && !other.is_span {
                this.ident = other.ident.clone();
            }

            this
        }

        let row_start = input.try(|i| GridLine::parse(context, i))?;
        let (column_start, row_end, column_end) = if input.try(|i| i.expect_delim('/')).is_ok() {
            let column_start = GridLine::parse(context, input)?;
            let (row_end, column_end) = if input.try(|i| i.expect_delim('/')).is_ok() {
                let row_end = GridLine::parse(context, input)?;
                let column_end = if input.try(|i| i.expect_delim('/')).is_ok() {
                    GridLine::parse(context, input)?
                } else {        // grid-column-end has not been given
                    line_with_ident_from(&column_start)
                };

                (row_end, column_end)
            } else {        // grid-row-start and grid-column-start has been given
                let row_end = line_with_ident_from(&row_start);
                let column_end = line_with_ident_from(&column_start);
                (row_end, column_end)
            };

            (column_start, row_end, column_end)
        } else {        // only grid-row-start is given
            let line = line_with_ident_from(&row_start);
            (line.clone(), line.clone(), line)
        };

        Ok(expanded! {
            grid_row_start: row_start,
            grid_row_end: row_end,
            grid_column_start: column_start,
            grid_column_end: column_end,
        })
    }

    impl<'a> ToCss for LonghandsToSerialize<'a> {
        fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
            self.grid_row_start.to_css(dest)?;
            let values = [&self.grid_column_start, &self.grid_row_end, &self.grid_column_end];
            for value in &values {
                dest.write_str(" / ")?;
                value.to_css(dest)?;
            }

            Ok(())
        }
    }

    }




    /// https://drafts.csswg.org/css-grid/#propdef-grid-template
    pub mod grid_template {
        use cssparser::Parser;
        use parser::ParserContext;
        use properties::{PropertyDeclaration, SourcePropertyDeclaration, MaybeBoxed, longhands};
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use std::fmt;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use style_traits::ToCss;

        pub struct Longhands {
                pub grid_template_rows:
                        Box<
                    longhands::grid_template_rows::SpecifiedValue
                        >
                    ,
                pub grid_template_columns:
                        Box<
                    longhands::grid_template_columns::SpecifiedValue
                        >
                    ,
                pub grid_template_areas:
                        Box<
                    longhands::grid_template_areas::SpecifiedValue
                        >
                    ,
        }

        /// Represents a serializable set of all of the longhand properties that
        /// correspond to a shorthand.
        pub struct LonghandsToSerialize<'a> {
                pub grid_template_rows:
                    &'a longhands::grid_template_rows::SpecifiedValue,
                pub grid_template_columns:
                    &'a longhands::grid_template_columns::SpecifiedValue,
                pub grid_template_areas:
                    &'a longhands::grid_template_areas::SpecifiedValue,
        }

        impl<'a> LonghandsToSerialize<'a> {
            /// Tries to get a serializable set of longhands given a set of
            /// property declarations.
            pub fn from_iter<I>(iter: I) -> Result<Self, ()>
                where I: Iterator<Item=&'a PropertyDeclaration>,
            {
                // Define all of the expected variables that correspond to the shorthand
                    let mut grid_template_rows = None;
                    let mut grid_template_columns = None;
                    let mut grid_template_areas = None;

                // Attempt to assign the incoming declarations to the expected variables
                for longhand in iter {
                    match *longhand {
                            PropertyDeclaration::GridTemplateRows(ref value) => {
                                grid_template_rows = Some(value)
                            },
                            PropertyDeclaration::GridTemplateColumns(ref value) => {
                                grid_template_columns = Some(value)
                            },
                            PropertyDeclaration::GridTemplateAreas(ref value) => {
                                grid_template_areas = Some(value)
                            },
                        _ => {}
                    };
                }

                // If any of the expected variables are missing, return an error
                match (
                        grid_template_rows,
                        grid_template_columns,
                        grid_template_areas,
                ) {

                    (
                        Some(grid_template_rows),
                        Some(grid_template_columns),
                        Some(grid_template_areas),
                    ) =>
                    Ok(LonghandsToSerialize {
                            grid_template_rows: grid_template_rows,
                            grid_template_columns: grid_template_columns,
                            grid_template_areas: grid_template_areas,
                    }),
                    _ => Err(())
                }
            }
        }

        /// Parse the given shorthand and fill the result into the
        /// `declarations` vector.
        pub fn parse_into<'i, 't>(declarations: &mut SourcePropertyDeclaration,
                                  context: &ParserContext, input: &mut Parser<'i, 't>)
                                  -> Result<(), ParseError<'i>> {
            input.parse_entirely(|input| parse_value(context, input)).map(|longhands| {
                    declarations.push(PropertyDeclaration::GridTemplateRows(
                        longhands.grid_template_rows
                    ));
                    declarations.push(PropertyDeclaration::GridTemplateColumns(
                        longhands.grid_template_columns
                    ));
                    declarations.push(PropertyDeclaration::GridTemplateAreas(
                        longhands.grid_template_areas
                    ));
            })
        }

        
    use parser::Parse;
    use properties::longhands::grid_template_areas::TemplateAreas;
    use values::{Either, None_};
    use values::generics::grid::{LineNameList, TrackSize, TrackList, TrackListType};
    use values::generics::grid::{TrackListValue, concat_serialize_idents};
    use values::specified::{GridTemplateComponent, GenericGridTemplateComponent};
    use values::specified::grid::parse_line_names;

    /// Parsing for `<grid-template>` shorthand (also used by `grid` shorthand).
    pub fn parse_grid_template<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                                       -> Result<(GridTemplateComponent,
                                                  GridTemplateComponent,
                                                  Either<TemplateAreas, None_>), ParseError<'i>> {
        // Other shorthand sub properties also parse `none` and `subgrid` keywords and this
        // shorthand should know after these keywords there is nothing to parse. Otherwise it
        // gets confused and rejects the sub properties that contains `none` or `subgrid`.
        
            if let Ok(x) = input.try(|i| {
                if i.try(|i| i.expect_ident_matching("subgrid")).is_ok() {
                    if i.is_exhausted() {
                        return Ok((GenericGridTemplateComponent::Subgrid(LineNameList::default()),
                                   GenericGridTemplateComponent::Subgrid(LineNameList::default()),
                                   Either::Second(None_)))
                    } else {
                        return Err(());
                    }
                }
                Err(())
            }) {
                return Ok(x);
            }
            if let Ok(x) = input.try(|i| {
                if i.try(|i| i.expect_ident_matching("none")).is_ok() {
                    if i.is_exhausted() {
                        return Ok((GenericGridTemplateComponent::None,
                                   GenericGridTemplateComponent::None,
                                   Either::Second(None_)))
                    } else {
                        return Err(());
                    }
                }
                Err(())
            }) {
                return Ok(x);
            }

        let first_line_names = input.try(parse_line_names).unwrap_or(vec![].into_boxed_slice());
        if let Ok(mut string) = input.try(|i| i.expect_string().map(|s| s.as_ref().into())) {
            let mut strings = vec![];
            let mut values = vec![];
            let mut line_names = vec![];
            let mut names = first_line_names.into_vec();
            loop {
                line_names.push(names.into_boxed_slice());
                strings.push(string);
                let size = input.try(|i| TrackSize::parse(context, i)).unwrap_or_default();
                values.push(TrackListValue::TrackSize(size));
                names = input.try(parse_line_names).unwrap_or(vec![].into_boxed_slice()).into_vec();
                if let Ok(v) = input.try(parse_line_names) {
                    names.extend(v.into_vec());
                }

                string = match input.try(|i| i.expect_string().map(|s| s.as_ref().into())) {
                    Ok(s) => s,
                    _ => {      // only the named area determines whether we should bail out
                        line_names.push(names.into_boxed_slice());
                        break
                    },
                };
            }

            if line_names.len() == values.len() {
                // should be one longer than track sizes
                line_names.push(vec![].into_boxed_slice());
            }

            let template_areas = TemplateAreas::from_vec(strings)
                .map_err(|()| StyleParseError::UnspecifiedError)?;
            let template_rows = TrackList {
                list_type: TrackListType::Normal,
                values: values,
                line_names: line_names.into_boxed_slice(),
                auto_repeat: None,
            };

            let template_cols = if input.try(|i| i.expect_delim('/')).is_ok() {
                let value = GridTemplateComponent::parse_without_none(context, input)?;
                if let GenericGridTemplateComponent::TrackList(ref list) = value {
                    if list.list_type != TrackListType::Explicit {
                        return Err(StyleParseError::UnspecifiedError.into())
                    }
                }

                value
            } else {
                GenericGridTemplateComponent::None
            };

            Ok((GenericGridTemplateComponent::TrackList(template_rows),
                template_cols, Either::First(template_areas)))
        } else {
            let mut template_rows = GridTemplateComponent::parse(context, input)?;
            if let GenericGridTemplateComponent::TrackList(ref mut list) = template_rows {
                // Fist line names are parsed already and it shouldn't be parsed again.
                // If line names are not empty, that means given property value is not acceptable
                if list.line_names[0].is_empty() {
                    list.line_names[0] = first_line_names;      // won't panic
                } else {
                    return Err(StyleParseError::UnspecifiedError.into());
                }
            }

            input.expect_delim('/')?;
            Ok((template_rows, GridTemplateComponent::parse(context, input)?, Either::Second(None_)))
        }
    }

    #[inline]
    pub fn parse_value<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                               -> Result<Longhands, ParseError<'i>> {
        let (rows, columns, areas) = parse_grid_template(context, input)?;
        Ok(expanded! {
            grid_template_rows: rows,
            grid_template_columns: columns,
            grid_template_areas: areas,
        })
    }

    /// Serialization for `<grid-template>` shorthand (also used by `grid` shorthand).
    pub fn serialize_grid_template<W>(template_rows: &GridTemplateComponent,
                                      template_columns: &GridTemplateComponent,
                                      template_areas: &Either<TemplateAreas, None_>,
                                      dest: &mut W) -> fmt::Result where W: fmt::Write {
        match *template_areas {
            Either::Second(_none) => {
                template_rows.to_css(dest)?;
                dest.write_str(" / ")?;
                template_columns.to_css(dest)
            },
            Either::First(ref areas) => {
                // The length of template-area and template-rows values should be equal.
                if areas.strings.len() != template_rows.track_list_len() {
                    return Ok(());
                }

                let track_list = match *template_rows {
                    GenericGridTemplateComponent::TrackList(ref list) => {
                        // We should fail if there is a `repeat` function. `grid` and
                        // `grid-template` shorthands doesn't accept that. Only longhand accepts.
                        if list.auto_repeat.is_some() ||
                           list.values.iter().any(|v| match *v {
                               TrackListValue::TrackRepeat(_) => true,
                               _ => false,
                           }) {
                            return Ok(());
                        }
                        list
                    },
                    // Others template components shouldn't exist with normal shorthand values.
                    // But if we need to serialize a group of longhand sub-properties for
                    // the shorthand, we should be able to return empty string instead of crashing.
                    _ => return Ok(()),
                };

                // We need to check some values that longhand accepts but shorthands don't.
                match *template_columns {
                    // We should fail if there is a `repeat` function. `grid` and
                    // `grid-template` shorthands doesn't accept that. Only longhand accepts that.
                    GenericGridTemplateComponent::TrackList(ref list) => {
                        if list.auto_repeat.is_some() ||
                           list.values.iter().any(|v| match *v {
                               TrackListValue::TrackRepeat(_) => true,
                               _ => false,
                           }) {
                            return Ok(());
                        }
                    },
                    // Also the shorthands don't accept subgrids unlike longhand.
                    // We should fail without an error here.
                    GenericGridTemplateComponent::Subgrid(_) => {
                        return Ok(());
                    },
                    _ => {},
                }

                let mut names_iter = track_list.line_names.iter();
                for (((i, string), names), value) in areas.strings.iter().enumerate()
                                                                  .zip(&mut names_iter)
                                                                  .zip(track_list.values.iter()) {
                    if i > 0 {
                        dest.write_str(" ")?;
                    }

                    if !names.is_empty() {
                        concat_serialize_idents("[", "] ", names, " ", dest)?;
                    }

                    string.to_css(dest)?;
                    dest.write_str(" ")?;
                    value.to_css(dest)?;
                }

                if let Some(names) = names_iter.next() {
                    concat_serialize_idents(" [", "]", names, " ", dest)?;
                }

                if let GenericGridTemplateComponent::TrackList(ref list) = *template_columns {
                    dest.write_str(" / ")?;
                    list.to_css(dest)?;
                }

                Ok(())
            },
        }
    }

    impl<'a> ToCss for LonghandsToSerialize<'a> {
        #[inline]
        fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
            serialize_grid_template(self.grid_template_rows, self.grid_template_columns,
                                    self.grid_template_areas, dest)
        }
    }

    }




    /// https://drafts.csswg.org/css-grid/#propdef-grid
    pub mod grid {
        use cssparser::Parser;
        use parser::ParserContext;
        use properties::{PropertyDeclaration, SourcePropertyDeclaration, MaybeBoxed, longhands};
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use std::fmt;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use style_traits::ToCss;

        pub struct Longhands {
                pub grid_template_rows:
                        Box<
                    longhands::grid_template_rows::SpecifiedValue
                        >
                    ,
                pub grid_template_columns:
                        Box<
                    longhands::grid_template_columns::SpecifiedValue
                        >
                    ,
                pub grid_template_areas:
                        Box<
                    longhands::grid_template_areas::SpecifiedValue
                        >
                    ,
                pub grid_auto_rows:
                        Box<
                    longhands::grid_auto_rows::SpecifiedValue
                        >
                    ,
                pub grid_auto_columns:
                        Box<
                    longhands::grid_auto_columns::SpecifiedValue
                        >
                    ,
                pub grid_auto_flow:
                    longhands::grid_auto_flow::SpecifiedValue
                    ,
        }

        /// Represents a serializable set of all of the longhand properties that
        /// correspond to a shorthand.
        pub struct LonghandsToSerialize<'a> {
                pub grid_template_rows:
                    &'a longhands::grid_template_rows::SpecifiedValue,
                pub grid_template_columns:
                    &'a longhands::grid_template_columns::SpecifiedValue,
                pub grid_template_areas:
                    &'a longhands::grid_template_areas::SpecifiedValue,
                pub grid_auto_rows:
                    &'a longhands::grid_auto_rows::SpecifiedValue,
                pub grid_auto_columns:
                    &'a longhands::grid_auto_columns::SpecifiedValue,
                pub grid_auto_flow:
                    &'a longhands::grid_auto_flow::SpecifiedValue,
        }

        impl<'a> LonghandsToSerialize<'a> {
            /// Tries to get a serializable set of longhands given a set of
            /// property declarations.
            pub fn from_iter<I>(iter: I) -> Result<Self, ()>
                where I: Iterator<Item=&'a PropertyDeclaration>,
            {
                // Define all of the expected variables that correspond to the shorthand
                    let mut grid_template_rows = None;
                    let mut grid_template_columns = None;
                    let mut grid_template_areas = None;
                    let mut grid_auto_rows = None;
                    let mut grid_auto_columns = None;
                    let mut grid_auto_flow = None;

                // Attempt to assign the incoming declarations to the expected variables
                for longhand in iter {
                    match *longhand {
                            PropertyDeclaration::GridTemplateRows(ref value) => {
                                grid_template_rows = Some(value)
                            },
                            PropertyDeclaration::GridTemplateColumns(ref value) => {
                                grid_template_columns = Some(value)
                            },
                            PropertyDeclaration::GridTemplateAreas(ref value) => {
                                grid_template_areas = Some(value)
                            },
                            PropertyDeclaration::GridAutoRows(ref value) => {
                                grid_auto_rows = Some(value)
                            },
                            PropertyDeclaration::GridAutoColumns(ref value) => {
                                grid_auto_columns = Some(value)
                            },
                            PropertyDeclaration::GridAutoFlow(ref value) => {
                                grid_auto_flow = Some(value)
                            },
                        _ => {}
                    };
                }

                // If any of the expected variables are missing, return an error
                match (
                        grid_template_rows,
                        grid_template_columns,
                        grid_template_areas,
                        grid_auto_rows,
                        grid_auto_columns,
                        grid_auto_flow,
                ) {

                    (
                        Some(grid_template_rows),
                        Some(grid_template_columns),
                        Some(grid_template_areas),
                        Some(grid_auto_rows),
                        Some(grid_auto_columns),
                        Some(grid_auto_flow),
                    ) =>
                    Ok(LonghandsToSerialize {
                            grid_template_rows: grid_template_rows,
                            grid_template_columns: grid_template_columns,
                            grid_template_areas: grid_template_areas,
                            grid_auto_rows: grid_auto_rows,
                            grid_auto_columns: grid_auto_columns,
                            grid_auto_flow: grid_auto_flow,
                    }),
                    _ => Err(())
                }
            }
        }

        /// Parse the given shorthand and fill the result into the
        /// `declarations` vector.
        pub fn parse_into<'i, 't>(declarations: &mut SourcePropertyDeclaration,
                                  context: &ParserContext, input: &mut Parser<'i, 't>)
                                  -> Result<(), ParseError<'i>> {
            input.parse_entirely(|input| parse_value(context, input)).map(|longhands| {
                    declarations.push(PropertyDeclaration::GridTemplateRows(
                        longhands.grid_template_rows
                    ));
                    declarations.push(PropertyDeclaration::GridTemplateColumns(
                        longhands.grid_template_columns
                    ));
                    declarations.push(PropertyDeclaration::GridTemplateAreas(
                        longhands.grid_template_areas
                    ));
                    declarations.push(PropertyDeclaration::GridAutoRows(
                        longhands.grid_auto_rows
                    ));
                    declarations.push(PropertyDeclaration::GridAutoColumns(
                        longhands.grid_auto_columns
                    ));
                    declarations.push(PropertyDeclaration::GridAutoFlow(
                        longhands.grid_auto_flow
                    ));
            })
        }

        
    use parser::Parse;
    use properties::longhands::{grid_auto_columns, grid_auto_rows, grid_auto_flow};
    use properties::longhands::grid_auto_flow::computed_value::{AutoFlow, T as SpecifiedAutoFlow};
    use values::{Either, None_};
    use values::generics::grid::{GridTemplateComponent, TrackListType};
    use values::specified::{GenericGridTemplateComponent, TrackSize};

    pub fn parse_value<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                               -> Result<Longhands, ParseError<'i>> {
        let mut temp_rows = GridTemplateComponent::None;
        let mut temp_cols = GridTemplateComponent::None;
        let mut temp_areas = Either::Second(None_);
        let mut auto_rows = TrackSize::default();
        let mut auto_cols = TrackSize::default();
        let mut flow = grid_auto_flow::get_initial_value();

        fn parse_auto_flow<'i, 't>(input: &mut Parser<'i, 't>, is_row: bool)
                                   -> Result<SpecifiedAutoFlow, ParseError<'i>> {
            let mut auto_flow = None;
            let mut dense = false;
            for _ in 0..2 {
                if input.try(|i| i.expect_ident_matching("auto-flow")).is_ok() {
                    auto_flow = if is_row {
                        Some(AutoFlow::Row)
                    } else {
                        Some(AutoFlow::Column)
                    };
                } else if input.try(|i| i.expect_ident_matching("dense")).is_ok() {
                    dense = true;
                } else {
                    break
                }
            }

            auto_flow.map(|flow| {
                SpecifiedAutoFlow {
                    autoflow: flow,
                    dense: dense,
                }
            }).ok_or(StyleParseError::UnspecifiedError.into())
        }

        if let Ok((rows, cols, areas)) = input.try(|i| super::grid_template::parse_grid_template(context, i)) {
            temp_rows = rows;
            temp_cols = cols;
            temp_areas = areas;
        } else if let Ok(rows) = input.try(|i| GridTemplateComponent::parse(context, i)) {
            temp_rows = rows;
            input.expect_delim('/')?;
            flow = parse_auto_flow(input, false)?;
            auto_cols = grid_auto_columns::parse(context, input).unwrap_or_default();
        } else {
            flow = parse_auto_flow(input, true)?;
            auto_rows = input.try(|i| grid_auto_rows::parse(context, i)).unwrap_or_default();
            input.expect_delim('/')?;
            temp_cols = GridTemplateComponent::parse(context, input)?;
        }

        Ok(expanded! {
            grid_template_rows: temp_rows,
            grid_template_columns: temp_cols,
            grid_template_areas: temp_areas,
            grid_auto_rows: auto_rows,
            grid_auto_columns: auto_cols,
            grid_auto_flow: flow,
        })
    }

    impl<'a> LonghandsToSerialize<'a> {
        /// Returns true if other sub properties except template-{rows,columns} are initial.
        fn is_grid_template(&self) -> bool {
            *self.grid_template_areas == Either::Second(None_) &&
            *self.grid_auto_rows == TrackSize::default() &&
            *self.grid_auto_columns == TrackSize::default() &&
            *self.grid_auto_flow == grid_auto_flow::get_initial_value()
        }
    }

    impl<'a> ToCss for LonghandsToSerialize<'a> {
        fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
            if *self.grid_template_areas != Either::Second(None_) ||
               (*self.grid_template_rows != GridTemplateComponent::None &&
                   *self.grid_template_columns != GridTemplateComponent::None) ||
               self.is_grid_template() {
                return super::grid_template::serialize_grid_template(self.grid_template_rows,
                                                                     self.grid_template_columns,
                                                                     self.grid_template_areas, dest);
            }

            if self.grid_auto_flow.autoflow == AutoFlow::Column {
                // It should fail to serialize if other branch of the if condition's values are set.
                if *self.grid_auto_rows != TrackSize::default() ||
                   *self.grid_template_columns != GridTemplateComponent::None {
                    return Ok(());
                }

                // It should fail to serialize if template-rows value is not Explicit.
                if let GenericGridTemplateComponent::TrackList(ref list) = *self.grid_template_rows {
                    if list.list_type != TrackListType::Explicit {
                        return Ok(());
                    }
                }

                self.grid_template_rows.to_css(dest)?;
                dest.write_str(" / auto-flow")?;
                if self.grid_auto_flow.dense {
                    dest.write_str(" dense")?;
                }

                if !self.grid_auto_columns.is_default() {
                    dest.write_str(" ")?;
                    self.grid_auto_columns.to_css(dest)?;
                }
            } else {
                // It should fail to serialize if other branch of the if condition's values are set.
                if *self.grid_auto_columns != TrackSize::default() ||
                   *self.grid_template_rows != GridTemplateComponent::None {
                    return Ok(());
                }

                // It should fail to serialize if template-column value is not Explicit.
                if let GenericGridTemplateComponent::TrackList(ref list) = *self.grid_template_columns {
                    if list.list_type != TrackListType::Explicit {
                        return Ok(());
                    }
                }

                dest.write_str("auto-flow")?;
                if self.grid_auto_flow.dense {
                    dest.write_str(" dense")?;
                }

                if !self.grid_auto_rows.is_default() {
                    dest.write_str(" ")?;
                    self.grid_auto_rows.to_css(dest)?;
                }

                dest.write_str(" / ")?;
                self.grid_template_columns.to_css(dest)?;
            }
            Ok(())
        }
    }

    }




    /// https://drafts.csswg.org/css-align/#propdef-place-content
    pub mod place_content {
        use cssparser::Parser;
        use parser::ParserContext;
        use properties::{PropertyDeclaration, SourcePropertyDeclaration, MaybeBoxed, longhands};
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use std::fmt;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use style_traits::ToCss;

        pub struct Longhands {
                pub align_content:
                    longhands::align_content::SpecifiedValue
                    ,
                pub justify_content:
                    longhands::justify_content::SpecifiedValue
                    ,
        }

        /// Represents a serializable set of all of the longhand properties that
        /// correspond to a shorthand.
        pub struct LonghandsToSerialize<'a> {
                pub align_content:
                    &'a longhands::align_content::SpecifiedValue,
                pub justify_content:
                    &'a longhands::justify_content::SpecifiedValue,
        }

        impl<'a> LonghandsToSerialize<'a> {
            /// Tries to get a serializable set of longhands given a set of
            /// property declarations.
            pub fn from_iter<I>(iter: I) -> Result<Self, ()>
                where I: Iterator<Item=&'a PropertyDeclaration>,
            {
                // Define all of the expected variables that correspond to the shorthand
                    let mut align_content = None;
                    let mut justify_content = None;

                // Attempt to assign the incoming declarations to the expected variables
                for longhand in iter {
                    match *longhand {
                            PropertyDeclaration::AlignContent(ref value) => {
                                align_content = Some(value)
                            },
                            PropertyDeclaration::JustifyContent(ref value) => {
                                justify_content = Some(value)
                            },
                        _ => {}
                    };
                }

                // If any of the expected variables are missing, return an error
                match (
                        align_content,
                        justify_content,
                ) {

                    (
                        Some(align_content),
                        Some(justify_content),
                    ) =>
                    Ok(LonghandsToSerialize {
                            align_content: align_content,
                            justify_content: justify_content,
                    }),
                    _ => Err(())
                }
            }
        }

        /// Parse the given shorthand and fill the result into the
        /// `declarations` vector.
        pub fn parse_into<'i, 't>(declarations: &mut SourcePropertyDeclaration,
                                  context: &ParserContext, input: &mut Parser<'i, 't>)
                                  -> Result<(), ParseError<'i>> {
            input.parse_entirely(|input| parse_value(context, input)).map(|longhands| {
                    declarations.push(PropertyDeclaration::AlignContent(
                        longhands.align_content
                    ));
                    declarations.push(PropertyDeclaration::JustifyContent(
                        longhands.justify_content
                    ));
            })
        }

        
    use properties::longhands::align_content;
    use properties::longhands::justify_content;

    pub fn parse_value<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                               -> Result<Longhands, ParseError<'i>> {
        let align = align_content::parse(context, input)?;
        if align.has_extra_flags() {
            return Err(StyleParseError::UnspecifiedError.into());
        }
        let justify = input.try(|input| justify_content::parse(context, input))
                           .unwrap_or(justify_content::SpecifiedValue::from(align));
        if justify.has_extra_flags() {
            return Err(StyleParseError::UnspecifiedError.into());
        }

        Ok(expanded! {
            align_content: align,
            justify_content: justify,
        })
    }

    impl<'a> ToCss for LonghandsToSerialize<'a> {
        fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
            self.align_content.to_css(dest)?;
            if self.align_content != self.justify_content {
                dest.write_str(" ")?;
                self.justify_content.to_css(dest)?;
            }
            Ok(())
        }
    }

    }




    /// https://drafts.csswg.org/css-align/#place-self-property
    pub mod place_self {
        use cssparser::Parser;
        use parser::ParserContext;
        use properties::{PropertyDeclaration, SourcePropertyDeclaration, MaybeBoxed, longhands};
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use std::fmt;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use style_traits::ToCss;

        pub struct Longhands {
                pub align_self:
                    longhands::align_self::SpecifiedValue
                    ,
                pub justify_self:
                    longhands::justify_self::SpecifiedValue
                    ,
        }

        /// Represents a serializable set of all of the longhand properties that
        /// correspond to a shorthand.
        pub struct LonghandsToSerialize<'a> {
                pub align_self:
                    &'a longhands::align_self::SpecifiedValue,
                pub justify_self:
                    &'a longhands::justify_self::SpecifiedValue,
        }

        impl<'a> LonghandsToSerialize<'a> {
            /// Tries to get a serializable set of longhands given a set of
            /// property declarations.
            pub fn from_iter<I>(iter: I) -> Result<Self, ()>
                where I: Iterator<Item=&'a PropertyDeclaration>,
            {
                // Define all of the expected variables that correspond to the shorthand
                    let mut align_self = None;
                    let mut justify_self = None;

                // Attempt to assign the incoming declarations to the expected variables
                for longhand in iter {
                    match *longhand {
                            PropertyDeclaration::AlignSelf(ref value) => {
                                align_self = Some(value)
                            },
                            PropertyDeclaration::JustifySelf(ref value) => {
                                justify_self = Some(value)
                            },
                        _ => {}
                    };
                }

                // If any of the expected variables are missing, return an error
                match (
                        align_self,
                        justify_self,
                ) {

                    (
                        Some(align_self),
                        Some(justify_self),
                    ) =>
                    Ok(LonghandsToSerialize {
                            align_self: align_self,
                            justify_self: justify_self,
                    }),
                    _ => Err(())
                }
            }
        }

        /// Parse the given shorthand and fill the result into the
        /// `declarations` vector.
        pub fn parse_into<'i, 't>(declarations: &mut SourcePropertyDeclaration,
                                  context: &ParserContext, input: &mut Parser<'i, 't>)
                                  -> Result<(), ParseError<'i>> {
            input.parse_entirely(|input| parse_value(context, input)).map(|longhands| {
                    declarations.push(PropertyDeclaration::AlignSelf(
                        longhands.align_self
                    ));
                    declarations.push(PropertyDeclaration::JustifySelf(
                        longhands.justify_self
                    ));
            })
        }

        
    use values::specified::align::AlignJustifySelf;
    use parser::Parse;

    pub fn parse_value<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                               -> Result<Longhands, ParseError<'i>> {
        let align = AlignJustifySelf::parse(context, input)?;
        if align.has_extra_flags() {
            return Err(StyleParseError::UnspecifiedError.into());
        }
        let justify = input.try(|input| AlignJustifySelf::parse(context, input)).unwrap_or(align.clone());
        if justify.has_extra_flags() {
            return Err(StyleParseError::UnspecifiedError.into());
        }

        Ok(expanded! {
            align_self: align,
            justify_self: justify,
        })
    }

    impl<'a> ToCss for LonghandsToSerialize<'a> {
        fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
            if self.align_self == self.justify_self {
                self.align_self.to_css(dest)
            } else {
                self.align_self.to_css(dest)?;
                dest.write_str(" ")?;
                self.justify_self.to_css(dest)
            }
        }
    }

    }




    /// https://drafts.csswg.org/css-align/#place-items-property
    pub mod place_items {
        use cssparser::Parser;
        use parser::ParserContext;
        use properties::{PropertyDeclaration, SourcePropertyDeclaration, MaybeBoxed, longhands};
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use std::fmt;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use style_traits::ToCss;

        pub struct Longhands {
                pub align_items:
                    longhands::align_items::SpecifiedValue
                    ,
                pub justify_items:
                    longhands::justify_items::SpecifiedValue
                    ,
        }

        /// Represents a serializable set of all of the longhand properties that
        /// correspond to a shorthand.
        pub struct LonghandsToSerialize<'a> {
                pub align_items:
                    &'a longhands::align_items::SpecifiedValue,
                pub justify_items:
                    &'a longhands::justify_items::SpecifiedValue,
        }

        impl<'a> LonghandsToSerialize<'a> {
            /// Tries to get a serializable set of longhands given a set of
            /// property declarations.
            pub fn from_iter<I>(iter: I) -> Result<Self, ()>
                where I: Iterator<Item=&'a PropertyDeclaration>,
            {
                // Define all of the expected variables that correspond to the shorthand
                    let mut align_items = None;
                    let mut justify_items = None;

                // Attempt to assign the incoming declarations to the expected variables
                for longhand in iter {
                    match *longhand {
                            PropertyDeclaration::AlignItems(ref value) => {
                                align_items = Some(value)
                            },
                            PropertyDeclaration::JustifyItems(ref value) => {
                                justify_items = Some(value)
                            },
                        _ => {}
                    };
                }

                // If any of the expected variables are missing, return an error
                match (
                        align_items,
                        justify_items,
                ) {

                    (
                        Some(align_items),
                        Some(justify_items),
                    ) =>
                    Ok(LonghandsToSerialize {
                            align_items: align_items,
                            justify_items: justify_items,
                    }),
                    _ => Err(())
                }
            }
        }

        /// Parse the given shorthand and fill the result into the
        /// `declarations` vector.
        pub fn parse_into<'i, 't>(declarations: &mut SourcePropertyDeclaration,
                                  context: &ParserContext, input: &mut Parser<'i, 't>)
                                  -> Result<(), ParseError<'i>> {
            input.parse_entirely(|input| parse_value(context, input)).map(|longhands| {
                    declarations.push(PropertyDeclaration::AlignItems(
                        longhands.align_items
                    ));
                    declarations.push(PropertyDeclaration::JustifyItems(
                        longhands.justify_items
                    ));
            })
        }

        
    use values::specified::align::{AlignItems, JustifyItems};
    use parser::Parse;

    impl From<AlignItems> for JustifyItems {
        fn from(align: AlignItems) -> JustifyItems {
            JustifyItems(align.0)
        }
    }

    pub fn parse_value<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                               -> Result<Longhands, ParseError<'i>> {
        let align = AlignItems::parse(context, input)?;
        if align.has_extra_flags() {
            return Err(StyleParseError::UnspecifiedError.into());
        }
        let justify = input.try(|input| JustifyItems::parse(context, input))
                           .unwrap_or(JustifyItems::from(align));
        if justify.has_extra_flags() {
            return Err(StyleParseError::UnspecifiedError.into());
        }

        Ok(expanded! {
            align_items: align,
            justify_items: justify,
        })
    }

    impl<'a> ToCss for LonghandsToSerialize<'a> {
        fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
            if self.align_items.0 == self.justify_items.0 {
                self.align_items.to_css(dest)
            } else {
                self.align_items.to_css(dest)?;
                dest.write_str(" ")?;
                self.justify_items.to_css(dest)
            }
        }
    }

    }


    /* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */





    /// https://www.w3.org/TR/SVG2/painting.html#MarkerShorthand
    pub mod marker {
        use cssparser::Parser;
        use parser::ParserContext;
        use properties::{PropertyDeclaration, SourcePropertyDeclaration, MaybeBoxed, longhands};
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use std::fmt;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use style_traits::ToCss;

        pub struct Longhands {
                pub marker_start:
                        Box<
                    longhands::marker_start::SpecifiedValue
                        >
                    ,
                pub marker_end:
                        Box<
                    longhands::marker_end::SpecifiedValue
                        >
                    ,
                pub marker_mid:
                        Box<
                    longhands::marker_mid::SpecifiedValue
                        >
                    ,
        }

        /// Represents a serializable set of all of the longhand properties that
        /// correspond to a shorthand.
        pub struct LonghandsToSerialize<'a> {
                pub marker_start:
                    &'a longhands::marker_start::SpecifiedValue,
                pub marker_end:
                    &'a longhands::marker_end::SpecifiedValue,
                pub marker_mid:
                    &'a longhands::marker_mid::SpecifiedValue,
        }

        impl<'a> LonghandsToSerialize<'a> {
            /// Tries to get a serializable set of longhands given a set of
            /// property declarations.
            pub fn from_iter<I>(iter: I) -> Result<Self, ()>
                where I: Iterator<Item=&'a PropertyDeclaration>,
            {
                // Define all of the expected variables that correspond to the shorthand
                    let mut marker_start = None;
                    let mut marker_end = None;
                    let mut marker_mid = None;

                // Attempt to assign the incoming declarations to the expected variables
                for longhand in iter {
                    match *longhand {
                            PropertyDeclaration::MarkerStart(ref value) => {
                                marker_start = Some(value)
                            },
                            PropertyDeclaration::MarkerEnd(ref value) => {
                                marker_end = Some(value)
                            },
                            PropertyDeclaration::MarkerMid(ref value) => {
                                marker_mid = Some(value)
                            },
                        _ => {}
                    };
                }

                // If any of the expected variables are missing, return an error
                match (
                        marker_start,
                        marker_end,
                        marker_mid,
                ) {

                    (
                        Some(marker_start),
                        Some(marker_end),
                        Some(marker_mid),
                    ) =>
                    Ok(LonghandsToSerialize {
                            marker_start: marker_start,
                            marker_end: marker_end,
                            marker_mid: marker_mid,
                    }),
                    _ => Err(())
                }
            }
        }

        /// Parse the given shorthand and fill the result into the
        /// `declarations` vector.
        pub fn parse_into<'i, 't>(declarations: &mut SourcePropertyDeclaration,
                                  context: &ParserContext, input: &mut Parser<'i, 't>)
                                  -> Result<(), ParseError<'i>> {
            input.parse_entirely(|input| parse_value(context, input)).map(|longhands| {
                    declarations.push(PropertyDeclaration::MarkerStart(
                        longhands.marker_start
                    ));
                    declarations.push(PropertyDeclaration::MarkerEnd(
                        longhands.marker_end
                    ));
                    declarations.push(PropertyDeclaration::MarkerMid(
                        longhands.marker_mid
                    ));
            })
        }

        
    use values::specified::UrlOrNone;

    pub fn parse_value<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                               -> Result<Longhands, ParseError<'i>> {
        use parser::Parse;
        let url = UrlOrNone::parse(context, input)?;

        Ok(expanded! {
            marker_start: url.clone(),
            marker_mid: url.clone(),
            marker_end: url,
        })
    }

    impl<'a> ToCss for LonghandsToSerialize<'a>  {
        fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
            if self.marker_start == self.marker_mid && self.marker_mid == self.marker_end {
                self.marker_start.to_css(dest)
            } else {
                Ok(())
            }
        }
    }

    }


    /* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */





    /// https://drafts.csswg.org/css-text-decor/#propdef-text-decoration
    pub mod text_decoration {
        use cssparser::Parser;
        use parser::ParserContext;
        use properties::{PropertyDeclaration, SourcePropertyDeclaration, MaybeBoxed, longhands};
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseError;
        #[allow(unused_imports)]
        use std::fmt;
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseError};
        #[allow(unused_imports)]
        use style_traits::ToCss;

        pub struct Longhands {
                pub text_decoration_line:
                    longhands::text_decoration_line::SpecifiedValue
                    ,
                pub text_decoration_style:
                    longhands::text_decoration_style::SpecifiedValue
                    ,
                pub text_decoration_color:
                    longhands::text_decoration_color::SpecifiedValue
                    ,
        }

        /// Represents a serializable set of all of the longhand properties that
        /// correspond to a shorthand.
        pub struct LonghandsToSerialize<'a> {
                pub text_decoration_line:
                    &'a longhands::text_decoration_line::SpecifiedValue,
                pub text_decoration_style:
                    &'a longhands::text_decoration_style::SpecifiedValue,
                pub text_decoration_color:
                    &'a longhands::text_decoration_color::SpecifiedValue,
        }

        impl<'a> LonghandsToSerialize<'a> {
            /// Tries to get a serializable set of longhands given a set of
            /// property declarations.
            pub fn from_iter<I>(iter: I) -> Result<Self, ()>
                where I: Iterator<Item=&'a PropertyDeclaration>,
            {
                // Define all of the expected variables that correspond to the shorthand
                    let mut text_decoration_line = None;
                    let mut text_decoration_style = None;
                    let mut text_decoration_color = None;

                // Attempt to assign the incoming declarations to the expected variables
                for longhand in iter {
                    match *longhand {
                            PropertyDeclaration::TextDecorationLine(ref value) => {
                                text_decoration_line = Some(value)
                            },
                            PropertyDeclaration::TextDecorationStyle(ref value) => {
                                text_decoration_style = Some(value)
                            },
                            PropertyDeclaration::TextDecorationColor(ref value) => {
                                text_decoration_color = Some(value)
                            },
                        _ => {}
                    };
                }

                // If any of the expected variables are missing, return an error
                match (
                        text_decoration_line,
                        text_decoration_style,
                        text_decoration_color,
                ) {

                    (
                        Some(text_decoration_line),
                        Some(text_decoration_style),
                        Some(text_decoration_color),
                    ) =>
                    Ok(LonghandsToSerialize {
                            text_decoration_line: text_decoration_line,
                            text_decoration_style: text_decoration_style,
                            text_decoration_color: text_decoration_color,
                    }),
                    _ => Err(())
                }
            }
        }

        /// Parse the given shorthand and fill the result into the
        /// `declarations` vector.
        pub fn parse_into<'i, 't>(declarations: &mut SourcePropertyDeclaration,
                                  context: &ParserContext, input: &mut Parser<'i, 't>)
                                  -> Result<(), ParseError<'i>> {
            input.parse_entirely(|input| parse_value(context, input)).map(|longhands| {
                    declarations.push(PropertyDeclaration::TextDecorationLine(
                        longhands.text_decoration_line
                    ));
                    declarations.push(PropertyDeclaration::TextDecorationStyle(
                        longhands.text_decoration_style
                    ));
                    declarations.push(PropertyDeclaration::TextDecorationColor(
                        longhands.text_decoration_color
                    ));
            })
        }

        

        use values::specified;
        use properties::longhands::{text_decoration_line, text_decoration_style, text_decoration_color};

    pub fn parse_value<'i, 't>(context: &ParserContext, input: &mut Parser<'i, 't>)
                               -> Result<Longhands, ParseError<'i>> {
            let (mut line, mut style, mut color, mut any) = (None, None, None, false);

        loop {
            macro_rules! parse_component {
                ($value:ident, $module:ident) => (
                    if $value.is_none() {
                        if let Ok(value) = input.try(|input| $module::parse(context, input)) {
                            $value = Some(value);
                            any = true;
                            continue;
                        }
                    }
                )
            }

            parse_component!(line, text_decoration_line);

                parse_component!(style, text_decoration_style);
                parse_component!(color, text_decoration_color);

            break;
        }

        if !any {
            return Err(StyleParseError::UnspecifiedError.into());
        }

        Ok(expanded! {
            text_decoration_line: unwrap_or_initial!(text_decoration_line, line),

                text_decoration_style: unwrap_or_initial!(text_decoration_style, style),
                text_decoration_color: unwrap_or_initial!(text_decoration_color, color),
        })
    }

    impl<'a> ToCss for LonghandsToSerialize<'a>  {
        fn to_css<W>(&self, dest: &mut W) -> fmt::Result where W: fmt::Write {
            self.text_decoration_line.to_css(dest)?;

                if self.text_decoration_style != &text_decoration_style::SpecifiedValue::solid {
                    dest.write_str(" ")?;
                    self.text_decoration_style.to_css(dest)?;
                }

                if *self.text_decoration_color != specified::Color::CurrentColor {
                    dest.write_str(" ")?;
                    self.text_decoration_color.to_css(dest)?;
                }

            Ok(())
        }
    }

    }



    // We don't defined the 'all' shorthand using the regular helpers:shorthand
    // mechanism, since it causes some very large types to be generated.
    
}

/// A module with all the code related to animated properties.
///
/// This needs to be "included" by mako at least after all longhand modules,
/// given they populate the global data.
pub mod animated_properties {
    /* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */





use cssparser::Parser;
#[cfg(feature = "gecko")] use gecko_bindings::bindings::RawServoAnimationValueMap;
#[cfg(feature = "gecko")] use gecko_bindings::structs::RawGeckoGfxMatrix4x4;
#[cfg(feature = "gecko")] use gecko_bindings::structs::nsCSSPropertyID;
#[cfg(feature = "gecko")] use gecko_bindings::sugar::ownership::{HasFFI, HasSimpleFFI};
#[cfg(feature = "gecko")] use gecko_string_cache::Atom;
use itertools::{EitherOrBoth, Itertools};
use properties::{CSSWideKeyword, PropertyDeclaration};
use properties::longhands;
use properties::longhands::font_weight::computed_value::T as FontWeight;
use properties::longhands::font_stretch::computed_value::T as FontStretch;
#[cfg(feature = "gecko")]
use properties::longhands::font_variation_settings::computed_value::T as FontVariationSettings;
use properties::longhands::transform::computed_value::ComputedMatrix;
use properties::longhands::transform::computed_value::ComputedOperation as TransformOperation;
use properties::longhands::transform::computed_value::T as TransformList;
use properties::longhands::visibility::computed_value::T as Visibility;
#[cfg(feature = "gecko")] use properties::{PropertyId, PropertyDeclarationId, LonghandId};
#[cfg(feature = "gecko")] use properties::{ShorthandId};
use selectors::parser::SelectorParseError;
use servo_arc::Arc;
use smallvec::SmallVec;
use std::borrow::Cow;
use std::cmp;
use std::fmt;
#[cfg(feature = "gecko")] use hash::FnvHashMap;
use style_traits::ParseError;
use super::ComputedValues;
use values::{CSSFloat, CustomIdent, Either};
use values::animated::{Animate, Procedure, ToAnimatedValue, ToAnimatedZero};
use values::animated::color::RGBA as AnimatedRGBA;
use values::animated::effects::Filter as AnimatedFilter;
use values::animated::effects::FilterList as AnimatedFilterList;
use values::computed::{Angle, CalcLengthOrPercentage};
use values::computed::{ClipRect, Context, ComputedUrl};
use values::computed::{Length, LengthOrPercentage, LengthOrPercentageOrAuto};
use values::computed::{LengthOrPercentageOrNone, MaxLength};
use values::computed::{NonNegativeNumber, Number, NumberOrPercentage, Percentage};
use values::computed::length::NonNegativeLengthOrPercentage;
use values::computed::ToComputedValue;
use values::computed::transform::DirectionVector;
use values::distance::{ComputeSquaredDistance, SquaredDistance};
#[cfg(feature = "gecko")] use values::generics::FontSettings as GenericFontSettings;
#[cfg(feature = "gecko")] use values::generics::FontSettingTag as GenericFontSettingTag;
#[cfg(feature = "gecko")] use values::generics::FontSettingTagFloat;
use values::generics::NonNegative;
use values::generics::effects::Filter;
use values::generics::position as generic_position;
use values::generics::svg::{SVGLength,  SvgLengthOrPercentageOrNumber, SVGPaint};
use values::generics::svg::{SVGPaintKind, SVGStrokeDashArray, SVGOpacity};

/// https://drafts.csswg.org/css-transitions/#animtype-repeatable-list
pub trait RepeatableListAnimatable: Animate {}

/// A longhand property whose animation type is not "none".
///
/// NOTE: This includes the 'display' property since it is animatable from SMIL even though it is
/// not animatable from CSS animations or Web Animations. CSS transitions also does not allow
/// animating 'display', but for CSS transitions we have the separate TransitionProperty type.
#[derive(Clone, Eq, Hash, PartialEq)]
#[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
#[cfg_attr(feature = "servo", derive(HeapSizeOf))]
pub enum AnimatableLonghand {
            /// background-color
            BackgroundColor,
            /// background-image
            BackgroundImage,
            /// background-position-x
            BackgroundPositionX,
            /// background-position-y
            BackgroundPositionY,
            /// background-repeat
            BackgroundRepeat,
            /// background-attachment
            BackgroundAttachment,
            /// background-clip
            BackgroundClip,
            /// background-origin
            BackgroundOrigin,
            /// background-size
            BackgroundSize,
            /// background-blend-mode
            BackgroundBlendMode,
            /// border-top-color
            BorderTopColor,
            /// border-top-style
            BorderTopStyle,
            /// border-top-width
            BorderTopWidth,
            /// border-left-color
            BorderLeftColor,
            /// border-left-style
            BorderLeftStyle,
            /// border-left-width
            BorderLeftWidth,
            /// border-bottom-color
            BorderBottomColor,
            /// border-bottom-style
            BorderBottomStyle,
            /// border-bottom-width
            BorderBottomWidth,
            /// border-right-color
            BorderRightColor,
            /// border-right-style
            BorderRightStyle,
            /// border-right-width
            BorderRightWidth,
            /// border-top-left-radius
            BorderTopLeftRadius,
            /// border-top-right-radius
            BorderTopRightRadius,
            /// border-bottom-right-radius
            BorderBottomRightRadius,
            /// border-bottom-left-radius
            BorderBottomLeftRadius,
            /// -moz-border-top-colors
            MozBorderTopColors,
            /// -moz-border-left-colors
            MozBorderLeftColors,
            /// -moz-border-bottom-colors
            MozBorderBottomColors,
            /// -moz-border-right-colors
            MozBorderRightColors,
            /// box-decoration-break
            BoxDecorationBreak,
            /// -moz-float-edge
            MozFloatEdge,
            /// border-image-source
            BorderImageSource,
            /// border-image-outset
            BorderImageOutset,
            /// border-image-repeat
            BorderImageRepeat,
            /// border-image-width
            BorderImageWidth,
            /// border-image-slice
            BorderImageSlice,
            /// display
            Display,
            /// position
            Position,
            /// float
            Float,
            /// clear
            Clear,
            /// vertical-align
            VerticalAlign,
            /// overflow-clip-box
            OverflowClipBox,
            /// overflow-x
            OverflowX,
            /// overflow-y
            OverflowY,
            /// scroll-snap-points-x
            ScrollSnapPointsX,
            /// scroll-snap-points-y
            ScrollSnapPointsY,
            /// scroll-snap-destination
            ScrollSnapDestination,
            /// scroll-snap-coordinate
            ScrollSnapCoordinate,
            /// transform
            Transform,
            /// scroll-behavior
            ScrollBehavior,
            /// scroll-snap-type-x
            ScrollSnapTypeX,
            /// scroll-snap-type-y
            ScrollSnapTypeY,
            /// isolation
            Isolation,
            /// page-break-after
            PageBreakAfter,
            /// page-break-before
            PageBreakBefore,
            /// page-break-inside
            PageBreakInside,
            /// resize
            Resize,
            /// perspective
            Perspective,
            /// perspective-origin
            PerspectiveOrigin,
            /// backface-visibility
            BackfaceVisibility,
            /// transform-box
            TransformBox,
            /// transform-style
            TransformStyle,
            /// transform-origin
            TransformOrigin,
            /// contain
            Contain,
            /// -moz-appearance
            MozAppearance,
            /// -moz-orient
            MozOrient,
            /// will-change
            WillChange,
            /// shape-outside
            ShapeOutside,
            /// touch-action
            TouchAction,
            /// color
            Color,
            /// column-width
            ColumnWidth,
            /// column-count
            ColumnCount,
            /// column-gap
            ColumnGap,
            /// column-fill
            ColumnFill,
            /// column-rule-width
            ColumnRuleWidth,
            /// column-rule-color
            ColumnRuleColor,
            /// column-span
            ColumnSpan,
            /// column-rule-style
            ColumnRuleStyle,
            /// content
            Content,
            /// counter-increment
            CounterIncrement,
            /// counter-reset
            CounterReset,
            /// opacity
            Opacity,
            /// box-shadow
            BoxShadow,
            /// clip
            Clip,
            /// filter
            Filter,
            /// mix-blend-mode
            MixBlendMode,
            /// font-family
            FontFamily,
            /// font-style
            FontStyle,
            /// font-variant-caps
            FontVariantCaps,
            /// font-weight
            FontWeight,
            /// font-size
            FontSize,
            /// font-size-adjust
            FontSizeAdjust,
            /// font-synthesis
            FontSynthesis,
            /// font-stretch
            FontStretch,
            /// font-kerning
            FontKerning,
            /// font-variant-alternates
            FontVariantAlternates,
            /// font-variant-east-asian
            FontVariantEastAsian,
            /// font-variant-ligatures
            FontVariantLigatures,
            /// font-variant-numeric
            FontVariantNumeric,
            /// font-variant-position
            FontVariantPosition,
            /// font-feature-settings
            FontFeatureSettings,
            /// font-variation-settings
            FontVariationSettings,
            /// font-language-override
            FontLanguageOverride,
            /// -moz-osx-font-smoothing
            MozOsxFontSmoothing,
            /// visibility
            Visibility,
            /// writing-mode
            WritingMode,
            /// direction
            Direction,
            /// text-orientation
            TextOrientation,
            /// color-adjust
            ColorAdjust,
            /// image-rendering
            ImageRendering,
            /// image-orientation
            ImageOrientation,
            /// border-collapse
            BorderCollapse,
            /// empty-cells
            EmptyCells,
            /// caption-side
            CaptionSide,
            /// border-spacing
            BorderSpacing,
            /// line-height
            LineHeight,
            /// text-transform
            TextTransform,
            /// hyphens
            Hyphens,
            /// -moz-text-size-adjust
            MozTextSizeAdjust,
            /// text-indent
            TextIndent,
            /// overflow-wrap
            OverflowWrap,
            /// word-break
            WordBreak,
            /// text-justify
            TextJustify,
            /// text-align-last
            TextAlignLast,
            /// text-align
            TextAlign,
            /// letter-spacing
            LetterSpacing,
            /// word-spacing
            WordSpacing,
            /// white-space
            WhiteSpace,
            /// text-shadow
            TextShadow,
            /// text-emphasis-style
            TextEmphasisStyle,
            /// text-emphasis-position
            TextEmphasisPosition,
            /// text-emphasis-color
            TextEmphasisColor,
            /// -moz-tab-size
            MozTabSize,
            /// -webkit-text-fill-color
            WebkitTextFillColor,
            /// -webkit-text-stroke-color
            WebkitTextStrokeColor,
            /// -webkit-text-stroke-width
            WebkitTextStrokeWidth,
            /// ruby-align
            RubyAlign,
            /// ruby-position
            RubyPosition,
            /// text-combine-upright
            TextCombineUpright,
            /// text-rendering
            TextRendering,
            /// list-style-position
            ListStylePosition,
            /// list-style-type
            ListStyleType,
            /// list-style-image
            ListStyleImage,
            /// quotes
            Quotes,
            /// -moz-image-region
            MozImageRegion,
            /// margin-top
            MarginTop,
            /// margin-left
            MarginLeft,
            /// margin-bottom
            MarginBottom,
            /// margin-right
            MarginRight,
            /// outline-color
            OutlineColor,
            /// outline-style
            OutlineStyle,
            /// outline-width
            OutlineWidth,
            /// -moz-outline-radius-topleft
            MozOutlineRadiusTopleft,
            /// -moz-outline-radius-topright
            MozOutlineRadiusTopright,
            /// -moz-outline-radius-bottomright
            MozOutlineRadiusBottomright,
            /// -moz-outline-radius-bottomleft
            MozOutlineRadiusBottomleft,
            /// outline-offset
            OutlineOffset,
            /// padding-top
            PaddingTop,
            /// padding-left
            PaddingLeft,
            /// padding-bottom
            PaddingBottom,
            /// padding-right
            PaddingRight,
            /// cursor
            Cursor,
            /// pointer-events
            PointerEvents,
            /// -moz-user-input
            MozUserInput,
            /// -moz-user-modify
            MozUserModify,
            /// -moz-user-focus
            MozUserFocus,
            /// caret-color
            CaretColor,
            /// top
            Top,
            /// left
            Left,
            /// bottom
            Bottom,
            /// right
            Right,
            /// z-index
            ZIndex,
            /// flex-direction
            FlexDirection,
            /// flex-wrap
            FlexWrap,
            /// justify-content
            JustifyContent,
            /// align-content
            AlignContent,
            /// align-items
            AlignItems,
            /// justify-items
            JustifyItems,
            /// flex-grow
            FlexGrow,
            /// flex-shrink
            FlexShrink,
            /// align-self
            AlignSelf,
            /// justify-self
            JustifySelf,
            /// order
            Order,
            /// flex-basis
            FlexBasis,
            /// width
            Width,
            /// min-width
            MinWidth,
            /// max-width
            MaxWidth,
            /// height
            Height,
            /// min-height
            MinHeight,
            /// max-height
            MaxHeight,
            /// box-sizing
            BoxSizing,
            /// object-fit
            ObjectFit,
            /// object-position
            ObjectPosition,
            /// grid-row-gap
            GridRowGap,
            /// grid-row-start
            GridRowStart,
            /// grid-row-end
            GridRowEnd,
            /// grid-auto-rows
            GridAutoRows,
            /// grid-template-rows
            GridTemplateRows,
            /// grid-column-gap
            GridColumnGap,
            /// grid-column-start
            GridColumnStart,
            /// grid-column-end
            GridColumnEnd,
            /// grid-auto-columns
            GridAutoColumns,
            /// grid-template-columns
            GridTemplateColumns,
            /// grid-auto-flow
            GridAutoFlow,
            /// grid-template-areas
            GridTemplateAreas,
            /// table-layout
            TableLayout,
            /// text-overflow
            TextOverflow,
            /// unicode-bidi
            UnicodeBidi,
            /// text-decoration-line
            TextDecorationLine,
            /// text-decoration-style
            TextDecorationStyle,
            /// text-decoration-color
            TextDecorationColor,
            /// initial-letter
            InitialLetter,
            /// ime-mode
            ImeMode,
            /// -moz-user-select
            MozUserSelect,
            /// -moz-window-dragging
            MozWindowDragging,
            /// -moz-window-shadow
            MozWindowShadow,
            /// -moz-force-broken-image-icon
            MozForceBrokenImageIcon,
            /// text-anchor
            TextAnchor,
            /// color-interpolation
            ColorInterpolation,
            /// color-interpolation-filters
            ColorInterpolationFilters,
            /// fill
            Fill,
            /// fill-opacity
            FillOpacity,
            /// fill-rule
            FillRule,
            /// shape-rendering
            ShapeRendering,
            /// stroke
            Stroke,
            /// stroke-width
            StrokeWidth,
            /// stroke-linecap
            StrokeLinecap,
            /// stroke-linejoin
            StrokeLinejoin,
            /// stroke-miterlimit
            StrokeMiterlimit,
            /// stroke-opacity
            StrokeOpacity,
            /// stroke-dasharray
            StrokeDasharray,
            /// stroke-dashoffset
            StrokeDashoffset,
            /// clip-rule
            ClipRule,
            /// marker-start
            MarkerStart,
            /// marker-mid
            MarkerMid,
            /// marker-end
            MarkerEnd,
            /// paint-order
            PaintOrder,
            /// dominant-baseline
            DominantBaseline,
            /// vector-effect
            VectorEffect,
            /// stop-color
            StopColor,
            /// stop-opacity
            StopOpacity,
            /// flood-color
            FloodColor,
            /// flood-opacity
            FloodOpacity,
            /// lighting-color
            LightingColor,
            /// mask-type
            MaskType,
            /// clip-path
            ClipPath,
            /// mask-mode
            MaskMode,
            /// mask-repeat
            MaskRepeat,
            /// mask-position-x
            MaskPositionX,
            /// mask-position-y
            MaskPositionY,
            /// mask-clip
            MaskClip,
            /// mask-origin
            MaskOrigin,
            /// mask-size
            MaskSize,
            /// mask-composite
            MaskComposite,
            /// mask-image
            MaskImage,
            /// -moz-box-align
            MozBoxAlign,
            /// -moz-box-direction
            MozBoxDirection,
            /// -moz-box-flex
            MozBoxFlex,
            /// -moz-box-orient
            MozBoxOrient,
            /// -moz-box-pack
            MozBoxPack,
            /// -moz-stack-sizing
            MozStackSizing,
            /// -moz-box-ordinal-group
            MozBoxOrdinalGroup,
}

impl fmt::Debug for AnimatableLonghand {
    fn fmt(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
        let name = match *self {
                    AnimatableLonghand::BackgroundColor => "BackgroundColor",
                    AnimatableLonghand::BackgroundImage => "BackgroundImage",
                    AnimatableLonghand::BackgroundPositionX => "BackgroundPositionX",
                    AnimatableLonghand::BackgroundPositionY => "BackgroundPositionY",
                    AnimatableLonghand::BackgroundRepeat => "BackgroundRepeat",
                    AnimatableLonghand::BackgroundAttachment => "BackgroundAttachment",
                    AnimatableLonghand::BackgroundClip => "BackgroundClip",
                    AnimatableLonghand::BackgroundOrigin => "BackgroundOrigin",
                    AnimatableLonghand::BackgroundSize => "BackgroundSize",
                    AnimatableLonghand::BackgroundBlendMode => "BackgroundBlendMode",
                    AnimatableLonghand::BorderTopColor => "BorderTopColor",
                    AnimatableLonghand::BorderTopStyle => "BorderTopStyle",
                    AnimatableLonghand::BorderTopWidth => "BorderTopWidth",
                    AnimatableLonghand::BorderLeftColor => "BorderLeftColor",
                    AnimatableLonghand::BorderLeftStyle => "BorderLeftStyle",
                    AnimatableLonghand::BorderLeftWidth => "BorderLeftWidth",
                    AnimatableLonghand::BorderBottomColor => "BorderBottomColor",
                    AnimatableLonghand::BorderBottomStyle => "BorderBottomStyle",
                    AnimatableLonghand::BorderBottomWidth => "BorderBottomWidth",
                    AnimatableLonghand::BorderRightColor => "BorderRightColor",
                    AnimatableLonghand::BorderRightStyle => "BorderRightStyle",
                    AnimatableLonghand::BorderRightWidth => "BorderRightWidth",
                    AnimatableLonghand::BorderTopLeftRadius => "BorderTopLeftRadius",
                    AnimatableLonghand::BorderTopRightRadius => "BorderTopRightRadius",
                    AnimatableLonghand::BorderBottomRightRadius => "BorderBottomRightRadius",
                    AnimatableLonghand::BorderBottomLeftRadius => "BorderBottomLeftRadius",
                    AnimatableLonghand::MozBorderTopColors => "MozBorderTopColors",
                    AnimatableLonghand::MozBorderLeftColors => "MozBorderLeftColors",
                    AnimatableLonghand::MozBorderBottomColors => "MozBorderBottomColors",
                    AnimatableLonghand::MozBorderRightColors => "MozBorderRightColors",
                    AnimatableLonghand::BoxDecorationBreak => "BoxDecorationBreak",
                    AnimatableLonghand::MozFloatEdge => "MozFloatEdge",
                    AnimatableLonghand::BorderImageSource => "BorderImageSource",
                    AnimatableLonghand::BorderImageOutset => "BorderImageOutset",
                    AnimatableLonghand::BorderImageRepeat => "BorderImageRepeat",
                    AnimatableLonghand::BorderImageWidth => "BorderImageWidth",
                    AnimatableLonghand::BorderImageSlice => "BorderImageSlice",
                    AnimatableLonghand::Display => "Display",
                    AnimatableLonghand::Position => "Position",
                    AnimatableLonghand::Float => "Float",
                    AnimatableLonghand::Clear => "Clear",
                    AnimatableLonghand::VerticalAlign => "VerticalAlign",
                    AnimatableLonghand::OverflowClipBox => "OverflowClipBox",
                    AnimatableLonghand::OverflowX => "OverflowX",
                    AnimatableLonghand::OverflowY => "OverflowY",
                    AnimatableLonghand::ScrollSnapPointsX => "ScrollSnapPointsX",
                    AnimatableLonghand::ScrollSnapPointsY => "ScrollSnapPointsY",
                    AnimatableLonghand::ScrollSnapDestination => "ScrollSnapDestination",
                    AnimatableLonghand::ScrollSnapCoordinate => "ScrollSnapCoordinate",
                    AnimatableLonghand::Transform => "Transform",
                    AnimatableLonghand::ScrollBehavior => "ScrollBehavior",
                    AnimatableLonghand::ScrollSnapTypeX => "ScrollSnapTypeX",
                    AnimatableLonghand::ScrollSnapTypeY => "ScrollSnapTypeY",
                    AnimatableLonghand::Isolation => "Isolation",
                    AnimatableLonghand::PageBreakAfter => "PageBreakAfter",
                    AnimatableLonghand::PageBreakBefore => "PageBreakBefore",
                    AnimatableLonghand::PageBreakInside => "PageBreakInside",
                    AnimatableLonghand::Resize => "Resize",
                    AnimatableLonghand::Perspective => "Perspective",
                    AnimatableLonghand::PerspectiveOrigin => "PerspectiveOrigin",
                    AnimatableLonghand::BackfaceVisibility => "BackfaceVisibility",
                    AnimatableLonghand::TransformBox => "TransformBox",
                    AnimatableLonghand::TransformStyle => "TransformStyle",
                    AnimatableLonghand::TransformOrigin => "TransformOrigin",
                    AnimatableLonghand::Contain => "Contain",
                    AnimatableLonghand::MozAppearance => "MozAppearance",
                    AnimatableLonghand::MozOrient => "MozOrient",
                    AnimatableLonghand::WillChange => "WillChange",
                    AnimatableLonghand::ShapeOutside => "ShapeOutside",
                    AnimatableLonghand::TouchAction => "TouchAction",
                    AnimatableLonghand::Color => "Color",
                    AnimatableLonghand::ColumnWidth => "ColumnWidth",
                    AnimatableLonghand::ColumnCount => "ColumnCount",
                    AnimatableLonghand::ColumnGap => "ColumnGap",
                    AnimatableLonghand::ColumnFill => "ColumnFill",
                    AnimatableLonghand::ColumnRuleWidth => "ColumnRuleWidth",
                    AnimatableLonghand::ColumnRuleColor => "ColumnRuleColor",
                    AnimatableLonghand::ColumnSpan => "ColumnSpan",
                    AnimatableLonghand::ColumnRuleStyle => "ColumnRuleStyle",
                    AnimatableLonghand::Content => "Content",
                    AnimatableLonghand::CounterIncrement => "CounterIncrement",
                    AnimatableLonghand::CounterReset => "CounterReset",
                    AnimatableLonghand::Opacity => "Opacity",
                    AnimatableLonghand::BoxShadow => "BoxShadow",
                    AnimatableLonghand::Clip => "Clip",
                    AnimatableLonghand::Filter => "Filter",
                    AnimatableLonghand::MixBlendMode => "MixBlendMode",
                    AnimatableLonghand::FontFamily => "FontFamily",
                    AnimatableLonghand::FontStyle => "FontStyle",
                    AnimatableLonghand::FontVariantCaps => "FontVariantCaps",
                    AnimatableLonghand::FontWeight => "FontWeight",
                    AnimatableLonghand::FontSize => "FontSize",
                    AnimatableLonghand::FontSizeAdjust => "FontSizeAdjust",
                    AnimatableLonghand::FontSynthesis => "FontSynthesis",
                    AnimatableLonghand::FontStretch => "FontStretch",
                    AnimatableLonghand::FontKerning => "FontKerning",
                    AnimatableLonghand::FontVariantAlternates => "FontVariantAlternates",
                    AnimatableLonghand::FontVariantEastAsian => "FontVariantEastAsian",
                    AnimatableLonghand::FontVariantLigatures => "FontVariantLigatures",
                    AnimatableLonghand::FontVariantNumeric => "FontVariantNumeric",
                    AnimatableLonghand::FontVariantPosition => "FontVariantPosition",
                    AnimatableLonghand::FontFeatureSettings => "FontFeatureSettings",
                    AnimatableLonghand::FontVariationSettings => "FontVariationSettings",
                    AnimatableLonghand::FontLanguageOverride => "FontLanguageOverride",
                    AnimatableLonghand::MozOsxFontSmoothing => "MozOsxFontSmoothing",
                    AnimatableLonghand::Visibility => "Visibility",
                    AnimatableLonghand::WritingMode => "WritingMode",
                    AnimatableLonghand::Direction => "Direction",
                    AnimatableLonghand::TextOrientation => "TextOrientation",
                    AnimatableLonghand::ColorAdjust => "ColorAdjust",
                    AnimatableLonghand::ImageRendering => "ImageRendering",
                    AnimatableLonghand::ImageOrientation => "ImageOrientation",
                    AnimatableLonghand::BorderCollapse => "BorderCollapse",
                    AnimatableLonghand::EmptyCells => "EmptyCells",
                    AnimatableLonghand::CaptionSide => "CaptionSide",
                    AnimatableLonghand::BorderSpacing => "BorderSpacing",
                    AnimatableLonghand::LineHeight => "LineHeight",
                    AnimatableLonghand::TextTransform => "TextTransform",
                    AnimatableLonghand::Hyphens => "Hyphens",
                    AnimatableLonghand::MozTextSizeAdjust => "MozTextSizeAdjust",
                    AnimatableLonghand::TextIndent => "TextIndent",
                    AnimatableLonghand::OverflowWrap => "OverflowWrap",
                    AnimatableLonghand::WordBreak => "WordBreak",
                    AnimatableLonghand::TextJustify => "TextJustify",
                    AnimatableLonghand::TextAlignLast => "TextAlignLast",
                    AnimatableLonghand::TextAlign => "TextAlign",
                    AnimatableLonghand::LetterSpacing => "LetterSpacing",
                    AnimatableLonghand::WordSpacing => "WordSpacing",
                    AnimatableLonghand::WhiteSpace => "WhiteSpace",
                    AnimatableLonghand::TextShadow => "TextShadow",
                    AnimatableLonghand::TextEmphasisStyle => "TextEmphasisStyle",
                    AnimatableLonghand::TextEmphasisPosition => "TextEmphasisPosition",
                    AnimatableLonghand::TextEmphasisColor => "TextEmphasisColor",
                    AnimatableLonghand::MozTabSize => "MozTabSize",
                    AnimatableLonghand::WebkitTextFillColor => "WebkitTextFillColor",
                    AnimatableLonghand::WebkitTextStrokeColor => "WebkitTextStrokeColor",
                    AnimatableLonghand::WebkitTextStrokeWidth => "WebkitTextStrokeWidth",
                    AnimatableLonghand::RubyAlign => "RubyAlign",
                    AnimatableLonghand::RubyPosition => "RubyPosition",
                    AnimatableLonghand::TextCombineUpright => "TextCombineUpright",
                    AnimatableLonghand::TextRendering => "TextRendering",
                    AnimatableLonghand::ListStylePosition => "ListStylePosition",
                    AnimatableLonghand::ListStyleType => "ListStyleType",
                    AnimatableLonghand::ListStyleImage => "ListStyleImage",
                    AnimatableLonghand::Quotes => "Quotes",
                    AnimatableLonghand::MozImageRegion => "MozImageRegion",
                    AnimatableLonghand::MarginTop => "MarginTop",
                    AnimatableLonghand::MarginLeft => "MarginLeft",
                    AnimatableLonghand::MarginBottom => "MarginBottom",
                    AnimatableLonghand::MarginRight => "MarginRight",
                    AnimatableLonghand::OutlineColor => "OutlineColor",
                    AnimatableLonghand::OutlineStyle => "OutlineStyle",
                    AnimatableLonghand::OutlineWidth => "OutlineWidth",
                    AnimatableLonghand::MozOutlineRadiusTopleft => "MozOutlineRadiusTopleft",
                    AnimatableLonghand::MozOutlineRadiusTopright => "MozOutlineRadiusTopright",
                    AnimatableLonghand::MozOutlineRadiusBottomright => "MozOutlineRadiusBottomright",
                    AnimatableLonghand::MozOutlineRadiusBottomleft => "MozOutlineRadiusBottomleft",
                    AnimatableLonghand::OutlineOffset => "OutlineOffset",
                    AnimatableLonghand::PaddingTop => "PaddingTop",
                    AnimatableLonghand::PaddingLeft => "PaddingLeft",
                    AnimatableLonghand::PaddingBottom => "PaddingBottom",
                    AnimatableLonghand::PaddingRight => "PaddingRight",
                    AnimatableLonghand::Cursor => "Cursor",
                    AnimatableLonghand::PointerEvents => "PointerEvents",
                    AnimatableLonghand::MozUserInput => "MozUserInput",
                    AnimatableLonghand::MozUserModify => "MozUserModify",
                    AnimatableLonghand::MozUserFocus => "MozUserFocus",
                    AnimatableLonghand::CaretColor => "CaretColor",
                    AnimatableLonghand::Top => "Top",
                    AnimatableLonghand::Left => "Left",
                    AnimatableLonghand::Bottom => "Bottom",
                    AnimatableLonghand::Right => "Right",
                    AnimatableLonghand::ZIndex => "ZIndex",
                    AnimatableLonghand::FlexDirection => "FlexDirection",
                    AnimatableLonghand::FlexWrap => "FlexWrap",
                    AnimatableLonghand::JustifyContent => "JustifyContent",
                    AnimatableLonghand::AlignContent => "AlignContent",
                    AnimatableLonghand::AlignItems => "AlignItems",
                    AnimatableLonghand::JustifyItems => "JustifyItems",
                    AnimatableLonghand::FlexGrow => "FlexGrow",
                    AnimatableLonghand::FlexShrink => "FlexShrink",
                    AnimatableLonghand::AlignSelf => "AlignSelf",
                    AnimatableLonghand::JustifySelf => "JustifySelf",
                    AnimatableLonghand::Order => "Order",
                    AnimatableLonghand::FlexBasis => "FlexBasis",
                    AnimatableLonghand::Width => "Width",
                    AnimatableLonghand::MinWidth => "MinWidth",
                    AnimatableLonghand::MaxWidth => "MaxWidth",
                    AnimatableLonghand::Height => "Height",
                    AnimatableLonghand::MinHeight => "MinHeight",
                    AnimatableLonghand::MaxHeight => "MaxHeight",
                    AnimatableLonghand::BoxSizing => "BoxSizing",
                    AnimatableLonghand::ObjectFit => "ObjectFit",
                    AnimatableLonghand::ObjectPosition => "ObjectPosition",
                    AnimatableLonghand::GridRowGap => "GridRowGap",
                    AnimatableLonghand::GridRowStart => "GridRowStart",
                    AnimatableLonghand::GridRowEnd => "GridRowEnd",
                    AnimatableLonghand::GridAutoRows => "GridAutoRows",
                    AnimatableLonghand::GridTemplateRows => "GridTemplateRows",
                    AnimatableLonghand::GridColumnGap => "GridColumnGap",
                    AnimatableLonghand::GridColumnStart => "GridColumnStart",
                    AnimatableLonghand::GridColumnEnd => "GridColumnEnd",
                    AnimatableLonghand::GridAutoColumns => "GridAutoColumns",
                    AnimatableLonghand::GridTemplateColumns => "GridTemplateColumns",
                    AnimatableLonghand::GridAutoFlow => "GridAutoFlow",
                    AnimatableLonghand::GridTemplateAreas => "GridTemplateAreas",
                    AnimatableLonghand::TableLayout => "TableLayout",
                    AnimatableLonghand::TextOverflow => "TextOverflow",
                    AnimatableLonghand::UnicodeBidi => "UnicodeBidi",
                    AnimatableLonghand::TextDecorationLine => "TextDecorationLine",
                    AnimatableLonghand::TextDecorationStyle => "TextDecorationStyle",
                    AnimatableLonghand::TextDecorationColor => "TextDecorationColor",
                    AnimatableLonghand::InitialLetter => "InitialLetter",
                    AnimatableLonghand::ImeMode => "ImeMode",
                    AnimatableLonghand::MozUserSelect => "MozUserSelect",
                    AnimatableLonghand::MozWindowDragging => "MozWindowDragging",
                    AnimatableLonghand::MozWindowShadow => "MozWindowShadow",
                    AnimatableLonghand::MozForceBrokenImageIcon => "MozForceBrokenImageIcon",
                    AnimatableLonghand::TextAnchor => "TextAnchor",
                    AnimatableLonghand::ColorInterpolation => "ColorInterpolation",
                    AnimatableLonghand::ColorInterpolationFilters => "ColorInterpolationFilters",
                    AnimatableLonghand::Fill => "Fill",
                    AnimatableLonghand::FillOpacity => "FillOpacity",
                    AnimatableLonghand::FillRule => "FillRule",
                    AnimatableLonghand::ShapeRendering => "ShapeRendering",
                    AnimatableLonghand::Stroke => "Stroke",
                    AnimatableLonghand::StrokeWidth => "StrokeWidth",
                    AnimatableLonghand::StrokeLinecap => "StrokeLinecap",
                    AnimatableLonghand::StrokeLinejoin => "StrokeLinejoin",
                    AnimatableLonghand::StrokeMiterlimit => "StrokeMiterlimit",
                    AnimatableLonghand::StrokeOpacity => "StrokeOpacity",
                    AnimatableLonghand::StrokeDasharray => "StrokeDasharray",
                    AnimatableLonghand::StrokeDashoffset => "StrokeDashoffset",
                    AnimatableLonghand::ClipRule => "ClipRule",
                    AnimatableLonghand::MarkerStart => "MarkerStart",
                    AnimatableLonghand::MarkerMid => "MarkerMid",
                    AnimatableLonghand::MarkerEnd => "MarkerEnd",
                    AnimatableLonghand::PaintOrder => "PaintOrder",
                    AnimatableLonghand::DominantBaseline => "DominantBaseline",
                    AnimatableLonghand::VectorEffect => "VectorEffect",
                    AnimatableLonghand::StopColor => "StopColor",
                    AnimatableLonghand::StopOpacity => "StopOpacity",
                    AnimatableLonghand::FloodColor => "FloodColor",
                    AnimatableLonghand::FloodOpacity => "FloodOpacity",
                    AnimatableLonghand::LightingColor => "LightingColor",
                    AnimatableLonghand::MaskType => "MaskType",
                    AnimatableLonghand::ClipPath => "ClipPath",
                    AnimatableLonghand::MaskMode => "MaskMode",
                    AnimatableLonghand::MaskRepeat => "MaskRepeat",
                    AnimatableLonghand::MaskPositionX => "MaskPositionX",
                    AnimatableLonghand::MaskPositionY => "MaskPositionY",
                    AnimatableLonghand::MaskClip => "MaskClip",
                    AnimatableLonghand::MaskOrigin => "MaskOrigin",
                    AnimatableLonghand::MaskSize => "MaskSize",
                    AnimatableLonghand::MaskComposite => "MaskComposite",
                    AnimatableLonghand::MaskImage => "MaskImage",
                    AnimatableLonghand::MozBoxAlign => "MozBoxAlign",
                    AnimatableLonghand::MozBoxDirection => "MozBoxDirection",
                    AnimatableLonghand::MozBoxFlex => "MozBoxFlex",
                    AnimatableLonghand::MozBoxOrient => "MozBoxOrient",
                    AnimatableLonghand::MozBoxPack => "MozBoxPack",
                    AnimatableLonghand::MozStackSizing => "MozStackSizing",
                    AnimatableLonghand::MozBoxOrdinalGroup => "MozBoxOrdinalGroup",
        };
        formatter.write_str(name)
    }
}

impl AnimatableLonghand {
    /// Returns true if this AnimatableLonghand is one of the discretely animatable properties.
    pub fn is_discrete(&self) -> bool {
        match *self {
                    AnimatableLonghand::BackgroundImage => true,
                    AnimatableLonghand::BackgroundRepeat => true,
                    AnimatableLonghand::BackgroundAttachment => true,
                    AnimatableLonghand::BackgroundClip => true,
                    AnimatableLonghand::BackgroundOrigin => true,
                    AnimatableLonghand::BackgroundBlendMode => true,
                    AnimatableLonghand::BorderTopStyle => true,
                    AnimatableLonghand::BorderLeftStyle => true,
                    AnimatableLonghand::BorderBottomStyle => true,
                    AnimatableLonghand::BorderRightStyle => true,
                    AnimatableLonghand::MozBorderTopColors => true,
                    AnimatableLonghand::MozBorderLeftColors => true,
                    AnimatableLonghand::MozBorderBottomColors => true,
                    AnimatableLonghand::MozBorderRightColors => true,
                    AnimatableLonghand::BoxDecorationBreak => true,
                    AnimatableLonghand::MozFloatEdge => true,
                    AnimatableLonghand::BorderImageSource => true,
                    AnimatableLonghand::BorderImageOutset => true,
                    AnimatableLonghand::BorderImageRepeat => true,
                    AnimatableLonghand::BorderImageWidth => true,
                    AnimatableLonghand::BorderImageSlice => true,
                    AnimatableLonghand::Display => true,
                    AnimatableLonghand::Position => true,
                    AnimatableLonghand::Float => true,
                    AnimatableLonghand::Clear => true,
                    AnimatableLonghand::OverflowClipBox => true,
                    AnimatableLonghand::OverflowX => true,
                    AnimatableLonghand::OverflowY => true,
                    AnimatableLonghand::ScrollSnapPointsX => true,
                    AnimatableLonghand::ScrollSnapPointsY => true,
                    AnimatableLonghand::ScrollSnapDestination => true,
                    AnimatableLonghand::ScrollSnapCoordinate => true,
                    AnimatableLonghand::ScrollBehavior => true,
                    AnimatableLonghand::ScrollSnapTypeX => true,
                    AnimatableLonghand::ScrollSnapTypeY => true,
                    AnimatableLonghand::Isolation => true,
                    AnimatableLonghand::PageBreakAfter => true,
                    AnimatableLonghand::PageBreakBefore => true,
                    AnimatableLonghand::PageBreakInside => true,
                    AnimatableLonghand::Resize => true,
                    AnimatableLonghand::BackfaceVisibility => true,
                    AnimatableLonghand::TransformBox => true,
                    AnimatableLonghand::TransformStyle => true,
                    AnimatableLonghand::Contain => true,
                    AnimatableLonghand::MozAppearance => true,
                    AnimatableLonghand::MozOrient => true,
                    AnimatableLonghand::WillChange => true,
                    AnimatableLonghand::TouchAction => true,
                    AnimatableLonghand::ColumnFill => true,
                    AnimatableLonghand::ColumnSpan => true,
                    AnimatableLonghand::ColumnRuleStyle => true,
                    AnimatableLonghand::Content => true,
                    AnimatableLonghand::CounterIncrement => true,
                    AnimatableLonghand::CounterReset => true,
                    AnimatableLonghand::MixBlendMode => true,
                    AnimatableLonghand::FontFamily => true,
                    AnimatableLonghand::FontStyle => true,
                    AnimatableLonghand::FontVariantCaps => true,
                    AnimatableLonghand::FontSynthesis => true,
                    AnimatableLonghand::FontKerning => true,
                    AnimatableLonghand::FontVariantAlternates => true,
                    AnimatableLonghand::FontVariantEastAsian => true,
                    AnimatableLonghand::FontVariantLigatures => true,
                    AnimatableLonghand::FontVariantNumeric => true,
                    AnimatableLonghand::FontVariantPosition => true,
                    AnimatableLonghand::FontFeatureSettings => true,
                    AnimatableLonghand::FontLanguageOverride => true,
                    AnimatableLonghand::MozOsxFontSmoothing => true,
                    AnimatableLonghand::WritingMode => true,
                    AnimatableLonghand::Direction => true,
                    AnimatableLonghand::TextOrientation => true,
                    AnimatableLonghand::ColorAdjust => true,
                    AnimatableLonghand::ImageRendering => true,
                    AnimatableLonghand::ImageOrientation => true,
                    AnimatableLonghand::BorderCollapse => true,
                    AnimatableLonghand::EmptyCells => true,
                    AnimatableLonghand::CaptionSide => true,
                    AnimatableLonghand::TextTransform => true,
                    AnimatableLonghand::Hyphens => true,
                    AnimatableLonghand::MozTextSizeAdjust => true,
                    AnimatableLonghand::OverflowWrap => true,
                    AnimatableLonghand::WordBreak => true,
                    AnimatableLonghand::TextJustify => true,
                    AnimatableLonghand::TextAlignLast => true,
                    AnimatableLonghand::TextAlign => true,
                    AnimatableLonghand::WhiteSpace => true,
                    AnimatableLonghand::TextEmphasisStyle => true,
                    AnimatableLonghand::TextEmphasisPosition => true,
                    AnimatableLonghand::WebkitTextStrokeWidth => true,
                    AnimatableLonghand::RubyAlign => true,
                    AnimatableLonghand::RubyPosition => true,
                    AnimatableLonghand::TextCombineUpright => true,
                    AnimatableLonghand::TextRendering => true,
                    AnimatableLonghand::ListStylePosition => true,
                    AnimatableLonghand::ListStyleType => true,
                    AnimatableLonghand::ListStyleImage => true,
                    AnimatableLonghand::Quotes => true,
                    AnimatableLonghand::OutlineStyle => true,
                    AnimatableLonghand::Cursor => true,
                    AnimatableLonghand::PointerEvents => true,
                    AnimatableLonghand::MozUserInput => true,
                    AnimatableLonghand::MozUserModify => true,
                    AnimatableLonghand::MozUserFocus => true,
                    AnimatableLonghand::FlexDirection => true,
                    AnimatableLonghand::FlexWrap => true,
                    AnimatableLonghand::JustifyContent => true,
                    AnimatableLonghand::AlignContent => true,
                    AnimatableLonghand::AlignItems => true,
                    AnimatableLonghand::JustifyItems => true,
                    AnimatableLonghand::AlignSelf => true,
                    AnimatableLonghand::JustifySelf => true,
                    AnimatableLonghand::BoxSizing => true,
                    AnimatableLonghand::ObjectFit => true,
                    AnimatableLonghand::GridRowStart => true,
                    AnimatableLonghand::GridRowEnd => true,
                    AnimatableLonghand::GridAutoRows => true,
                    AnimatableLonghand::GridTemplateRows => true,
                    AnimatableLonghand::GridColumnStart => true,
                    AnimatableLonghand::GridColumnEnd => true,
                    AnimatableLonghand::GridAutoColumns => true,
                    AnimatableLonghand::GridTemplateColumns => true,
                    AnimatableLonghand::GridAutoFlow => true,
                    AnimatableLonghand::GridTemplateAreas => true,
                    AnimatableLonghand::TableLayout => true,
                    AnimatableLonghand::TextOverflow => true,
                    AnimatableLonghand::UnicodeBidi => true,
                    AnimatableLonghand::TextDecorationLine => true,
                    AnimatableLonghand::TextDecorationStyle => true,
                    AnimatableLonghand::InitialLetter => true,
                    AnimatableLonghand::ImeMode => true,
                    AnimatableLonghand::MozUserSelect => true,
                    AnimatableLonghand::MozWindowDragging => true,
                    AnimatableLonghand::MozWindowShadow => true,
                    AnimatableLonghand::MozForceBrokenImageIcon => true,
                    AnimatableLonghand::TextAnchor => true,
                    AnimatableLonghand::ColorInterpolation => true,
                    AnimatableLonghand::ColorInterpolationFilters => true,
                    AnimatableLonghand::FillRule => true,
                    AnimatableLonghand::ShapeRendering => true,
                    AnimatableLonghand::StrokeLinecap => true,
                    AnimatableLonghand::StrokeLinejoin => true,
                    AnimatableLonghand::ClipRule => true,
                    AnimatableLonghand::MarkerStart => true,
                    AnimatableLonghand::MarkerMid => true,
                    AnimatableLonghand::MarkerEnd => true,
                    AnimatableLonghand::PaintOrder => true,
                    AnimatableLonghand::DominantBaseline => true,
                    AnimatableLonghand::VectorEffect => true,
                    AnimatableLonghand::MaskType => true,
                    AnimatableLonghand::MaskMode => true,
                    AnimatableLonghand::MaskRepeat => true,
                    AnimatableLonghand::MaskClip => true,
                    AnimatableLonghand::MaskOrigin => true,
                    AnimatableLonghand::MaskComposite => true,
                    AnimatableLonghand::MaskImage => true,
                    AnimatableLonghand::MozBoxAlign => true,
                    AnimatableLonghand::MozBoxDirection => true,
                    AnimatableLonghand::MozBoxOrient => true,
                    AnimatableLonghand::MozBoxPack => true,
                    AnimatableLonghand::MozStackSizing => true,
                    AnimatableLonghand::MozBoxOrdinalGroup => true,
            _ => false
        }
    }

    /// Converts from an nsCSSPropertyID. Returns None if nsCSSPropertyID is not an animatable
    /// longhand in Servo.
    #[cfg(feature = "gecko")]
    pub fn from_nscsspropertyid(css_property: nsCSSPropertyID) -> Option<Self> {
        match css_property {
                    
    nsCSSPropertyID::eCSSProperty_background_color
                        => Some(AnimatableLonghand::BackgroundColor),
                    
    nsCSSPropertyID::eCSSProperty_background_image
                        => Some(AnimatableLonghand::BackgroundImage),
                    
    nsCSSPropertyID::eCSSProperty_background_position_x
                        => Some(AnimatableLonghand::BackgroundPositionX),
                    
    nsCSSPropertyID::eCSSProperty_background_position_y
                        => Some(AnimatableLonghand::BackgroundPositionY),
                    
    nsCSSPropertyID::eCSSProperty_background_repeat
                        => Some(AnimatableLonghand::BackgroundRepeat),
                    
    nsCSSPropertyID::eCSSProperty_background_attachment
                        => Some(AnimatableLonghand::BackgroundAttachment),
                    
    nsCSSPropertyID::eCSSProperty_background_clip
                        => Some(AnimatableLonghand::BackgroundClip),
                    
    nsCSSPropertyID::eCSSProperty_background_origin
                        => Some(AnimatableLonghand::BackgroundOrigin),
                    
    nsCSSPropertyID::eCSSProperty_background_size
                        => Some(AnimatableLonghand::BackgroundSize),
                    
    nsCSSPropertyID::eCSSProperty_background_blend_mode
                        => Some(AnimatableLonghand::BackgroundBlendMode),
                    
    nsCSSPropertyID::eCSSProperty_border_top_color
                        => Some(AnimatableLonghand::BorderTopColor),
                    
    nsCSSPropertyID::eCSSProperty_border_top_style
                        => Some(AnimatableLonghand::BorderTopStyle),
                    
    nsCSSPropertyID::eCSSProperty_border_top_width
                        => Some(AnimatableLonghand::BorderTopWidth),
                    
    nsCSSPropertyID::eCSSProperty_border_left_color
                        => Some(AnimatableLonghand::BorderLeftColor),
                    
    nsCSSPropertyID::eCSSProperty_border_left_style
                        => Some(AnimatableLonghand::BorderLeftStyle),
                    
    nsCSSPropertyID::eCSSProperty_border_left_width
                        => Some(AnimatableLonghand::BorderLeftWidth),
                    
    nsCSSPropertyID::eCSSProperty_border_bottom_color
                        => Some(AnimatableLonghand::BorderBottomColor),
                    
    nsCSSPropertyID::eCSSProperty_border_bottom_style
                        => Some(AnimatableLonghand::BorderBottomStyle),
                    
    nsCSSPropertyID::eCSSProperty_border_bottom_width
                        => Some(AnimatableLonghand::BorderBottomWidth),
                    
    nsCSSPropertyID::eCSSProperty_border_right_color
                        => Some(AnimatableLonghand::BorderRightColor),
                    
    nsCSSPropertyID::eCSSProperty_border_right_style
                        => Some(AnimatableLonghand::BorderRightStyle),
                    
    nsCSSPropertyID::eCSSProperty_border_right_width
                        => Some(AnimatableLonghand::BorderRightWidth),
                    
    nsCSSPropertyID::eCSSProperty_border_top_left_radius
                        => Some(AnimatableLonghand::BorderTopLeftRadius),
                    
    nsCSSPropertyID::eCSSProperty_border_top_right_radius
                        => Some(AnimatableLonghand::BorderTopRightRadius),
                    
    nsCSSPropertyID::eCSSProperty_border_bottom_right_radius
                        => Some(AnimatableLonghand::BorderBottomRightRadius),
                    
    nsCSSPropertyID::eCSSProperty_border_bottom_left_radius
                        => Some(AnimatableLonghand::BorderBottomLeftRadius),
                    
    nsCSSPropertyID::eCSSProperty__moz_border_top_colors
                        => Some(AnimatableLonghand::MozBorderTopColors),
                    
    nsCSSPropertyID::eCSSProperty__moz_border_left_colors
                        => Some(AnimatableLonghand::MozBorderLeftColors),
                    
    nsCSSPropertyID::eCSSProperty__moz_border_bottom_colors
                        => Some(AnimatableLonghand::MozBorderBottomColors),
                    
    nsCSSPropertyID::eCSSProperty__moz_border_right_colors
                        => Some(AnimatableLonghand::MozBorderRightColors),
                    
    nsCSSPropertyID::eCSSProperty_box_decoration_break
                        => Some(AnimatableLonghand::BoxDecorationBreak),
                    
    nsCSSPropertyID::eCSSProperty__moz_float_edge
                        => Some(AnimatableLonghand::MozFloatEdge),
                    
    nsCSSPropertyID::eCSSProperty_border_image_source
                        => Some(AnimatableLonghand::BorderImageSource),
                    
    nsCSSPropertyID::eCSSProperty_border_image_outset
                        => Some(AnimatableLonghand::BorderImageOutset),
                    
    nsCSSPropertyID::eCSSProperty_border_image_repeat
                        => Some(AnimatableLonghand::BorderImageRepeat),
                    
    nsCSSPropertyID::eCSSProperty_border_image_width
                        => Some(AnimatableLonghand::BorderImageWidth),
                    
    nsCSSPropertyID::eCSSProperty_border_image_slice
                        => Some(AnimatableLonghand::BorderImageSlice),
                    
    nsCSSPropertyID::eCSSProperty_display
                        => Some(AnimatableLonghand::Display),
                    
    nsCSSPropertyID::eCSSProperty_position
                        => Some(AnimatableLonghand::Position),
                    
    nsCSSPropertyID::eCSSProperty_float_
                        => Some(AnimatableLonghand::Float),
                    
    nsCSSPropertyID::eCSSProperty_clear
                        => Some(AnimatableLonghand::Clear),
                    
    nsCSSPropertyID::eCSSProperty_vertical_align
                        => Some(AnimatableLonghand::VerticalAlign),
                    
    nsCSSPropertyID::eCSSProperty_overflow_clip_box
                        => Some(AnimatableLonghand::OverflowClipBox),
                    
    nsCSSPropertyID::eCSSProperty_overflow_x
                        => Some(AnimatableLonghand::OverflowX),
                    
    nsCSSPropertyID::eCSSProperty_overflow_y
                        => Some(AnimatableLonghand::OverflowY),
                    
    nsCSSPropertyID::eCSSProperty_scroll_snap_points_x
                        => Some(AnimatableLonghand::ScrollSnapPointsX),
                    
    nsCSSPropertyID::eCSSProperty_scroll_snap_points_y
                        => Some(AnimatableLonghand::ScrollSnapPointsY),
                    
    nsCSSPropertyID::eCSSProperty_scroll_snap_destination
                        => Some(AnimatableLonghand::ScrollSnapDestination),
                    
    nsCSSPropertyID::eCSSProperty_scroll_snap_coordinate
                        => Some(AnimatableLonghand::ScrollSnapCoordinate),
                    
    nsCSSPropertyID::eCSSProperty_transform
                        => Some(AnimatableLonghand::Transform),
                    
    nsCSSPropertyID::eCSSProperty_scroll_behavior
                        => Some(AnimatableLonghand::ScrollBehavior),
                    
    nsCSSPropertyID::eCSSProperty_scroll_snap_type_x
                        => Some(AnimatableLonghand::ScrollSnapTypeX),
                    
    nsCSSPropertyID::eCSSProperty_scroll_snap_type_y
                        => Some(AnimatableLonghand::ScrollSnapTypeY),
                    
    nsCSSPropertyID::eCSSProperty_isolation
                        => Some(AnimatableLonghand::Isolation),
                    
    nsCSSPropertyID::eCSSProperty_page_break_after
                        => Some(AnimatableLonghand::PageBreakAfter),
                    
    nsCSSPropertyID::eCSSProperty_page_break_before
                        => Some(AnimatableLonghand::PageBreakBefore),
                    
    nsCSSPropertyID::eCSSProperty_page_break_inside
                        => Some(AnimatableLonghand::PageBreakInside),
                    
    nsCSSPropertyID::eCSSProperty_resize
                        => Some(AnimatableLonghand::Resize),
                    
    nsCSSPropertyID::eCSSProperty_perspective
                        => Some(AnimatableLonghand::Perspective),
                    
    nsCSSPropertyID::eCSSProperty_perspective_origin
                        => Some(AnimatableLonghand::PerspectiveOrigin),
                    
    nsCSSPropertyID::eCSSProperty_backface_visibility
                        => Some(AnimatableLonghand::BackfaceVisibility),
                    
    nsCSSPropertyID::eCSSProperty_transform_box
                        => Some(AnimatableLonghand::TransformBox),
                    
    nsCSSPropertyID::eCSSProperty_transform_style
                        => Some(AnimatableLonghand::TransformStyle),
                    
    nsCSSPropertyID::eCSSProperty_transform_origin
                        => Some(AnimatableLonghand::TransformOrigin),
                    
    nsCSSPropertyID::eCSSProperty_contain
                        => Some(AnimatableLonghand::Contain),
                    
    nsCSSPropertyID::eCSSProperty__moz_appearance
                        => Some(AnimatableLonghand::MozAppearance),
                    
    nsCSSPropertyID::eCSSProperty__moz_orient
                        => Some(AnimatableLonghand::MozOrient),
                    
    nsCSSPropertyID::eCSSProperty_will_change
                        => Some(AnimatableLonghand::WillChange),
                    
    nsCSSPropertyID::eCSSProperty_shape_outside
                        => Some(AnimatableLonghand::ShapeOutside),
                    
    nsCSSPropertyID::eCSSProperty_touch_action
                        => Some(AnimatableLonghand::TouchAction),
                    
    nsCSSPropertyID::eCSSProperty_color
                        => Some(AnimatableLonghand::Color),
                    
    nsCSSPropertyID::eCSSProperty_column_width
                        => Some(AnimatableLonghand::ColumnWidth),
                    
    nsCSSPropertyID::eCSSProperty_column_count
                        => Some(AnimatableLonghand::ColumnCount),
                    
    nsCSSPropertyID::eCSSProperty_column_gap
                        => Some(AnimatableLonghand::ColumnGap),
                    
    nsCSSPropertyID::eCSSProperty_column_fill
                        => Some(AnimatableLonghand::ColumnFill),
                    
    nsCSSPropertyID::eCSSProperty_column_rule_width
                        => Some(AnimatableLonghand::ColumnRuleWidth),
                    
    nsCSSPropertyID::eCSSProperty_column_rule_color
                        => Some(AnimatableLonghand::ColumnRuleColor),
                    
    nsCSSPropertyID::eCSSProperty_column_span
                        => Some(AnimatableLonghand::ColumnSpan),
                    
    nsCSSPropertyID::eCSSProperty_column_rule_style
                        => Some(AnimatableLonghand::ColumnRuleStyle),
                    
    nsCSSPropertyID::eCSSProperty_content
                        => Some(AnimatableLonghand::Content),
                    
    nsCSSPropertyID::eCSSProperty_counter_increment
                        => Some(AnimatableLonghand::CounterIncrement),
                    
    nsCSSPropertyID::eCSSProperty_counter_reset
                        => Some(AnimatableLonghand::CounterReset),
                    
    nsCSSPropertyID::eCSSProperty_opacity
                        => Some(AnimatableLonghand::Opacity),
                    
    nsCSSPropertyID::eCSSProperty_box_shadow
                        => Some(AnimatableLonghand::BoxShadow),
                    
    nsCSSPropertyID::eCSSProperty_clip
                        => Some(AnimatableLonghand::Clip),
                    
    nsCSSPropertyID::eCSSProperty_filter
                        => Some(AnimatableLonghand::Filter),
                    
    nsCSSPropertyID::eCSSProperty_mix_blend_mode
                        => Some(AnimatableLonghand::MixBlendMode),
                    
    nsCSSPropertyID::eCSSProperty_font_family
                        => Some(AnimatableLonghand::FontFamily),
                    
    nsCSSPropertyID::eCSSProperty_font_style
                        => Some(AnimatableLonghand::FontStyle),
                    
    nsCSSPropertyID::eCSSProperty_font_variant_caps
                        => Some(AnimatableLonghand::FontVariantCaps),
                    
    nsCSSPropertyID::eCSSProperty_font_weight
                        => Some(AnimatableLonghand::FontWeight),
                    
    nsCSSPropertyID::eCSSProperty_font_size
                        => Some(AnimatableLonghand::FontSize),
                    
    nsCSSPropertyID::eCSSProperty_font_size_adjust
                        => Some(AnimatableLonghand::FontSizeAdjust),
                    
    nsCSSPropertyID::eCSSProperty_font_synthesis
                        => Some(AnimatableLonghand::FontSynthesis),
                    
    nsCSSPropertyID::eCSSProperty_font_stretch
                        => Some(AnimatableLonghand::FontStretch),
                    
    nsCSSPropertyID::eCSSProperty_font_kerning
                        => Some(AnimatableLonghand::FontKerning),
                    
    nsCSSPropertyID::eCSSProperty_font_variant_alternates
                        => Some(AnimatableLonghand::FontVariantAlternates),
                    
    nsCSSPropertyID::eCSSProperty_font_variant_east_asian
                        => Some(AnimatableLonghand::FontVariantEastAsian),
                    
    nsCSSPropertyID::eCSSProperty_font_variant_ligatures
                        => Some(AnimatableLonghand::FontVariantLigatures),
                    
    nsCSSPropertyID::eCSSProperty_font_variant_numeric
                        => Some(AnimatableLonghand::FontVariantNumeric),
                    
    nsCSSPropertyID::eCSSProperty_font_variant_position
                        => Some(AnimatableLonghand::FontVariantPosition),
                    
    nsCSSPropertyID::eCSSProperty_font_feature_settings
                        => Some(AnimatableLonghand::FontFeatureSettings),
                    
    nsCSSPropertyID::eCSSProperty_font_variation_settings
                        => Some(AnimatableLonghand::FontVariationSettings),
                    
    nsCSSPropertyID::eCSSProperty_font_language_override
                        => Some(AnimatableLonghand::FontLanguageOverride),
                    
    nsCSSPropertyID::eCSSProperty__moz_osx_font_smoothing
                        => Some(AnimatableLonghand::MozOsxFontSmoothing),
                    
    nsCSSPropertyID::eCSSProperty_visibility
                        => Some(AnimatableLonghand::Visibility),
                    
    nsCSSPropertyID::eCSSProperty_writing_mode
                        => Some(AnimatableLonghand::WritingMode),
                    
    nsCSSPropertyID::eCSSProperty_direction
                        => Some(AnimatableLonghand::Direction),
                    
    nsCSSPropertyID::eCSSProperty_text_orientation
                        => Some(AnimatableLonghand::TextOrientation),
                    
    nsCSSPropertyID::eCSSProperty_color_adjust
                        => Some(AnimatableLonghand::ColorAdjust),
                    
    nsCSSPropertyID::eCSSProperty_image_rendering
                        => Some(AnimatableLonghand::ImageRendering),
                    
    nsCSSPropertyID::eCSSProperty_image_orientation
                        => Some(AnimatableLonghand::ImageOrientation),
                    
    nsCSSPropertyID::eCSSProperty_border_collapse
                        => Some(AnimatableLonghand::BorderCollapse),
                    
    nsCSSPropertyID::eCSSProperty_empty_cells
                        => Some(AnimatableLonghand::EmptyCells),
                    
    nsCSSPropertyID::eCSSProperty_caption_side
                        => Some(AnimatableLonghand::CaptionSide),
                    
    nsCSSPropertyID::eCSSProperty_border_spacing
                        => Some(AnimatableLonghand::BorderSpacing),
                    
    nsCSSPropertyID::eCSSProperty_line_height
                        => Some(AnimatableLonghand::LineHeight),
                    
    nsCSSPropertyID::eCSSProperty_text_transform
                        => Some(AnimatableLonghand::TextTransform),
                    
    nsCSSPropertyID::eCSSProperty_hyphens
                        => Some(AnimatableLonghand::Hyphens),
                    
    nsCSSPropertyID::eCSSProperty__moz_text_size_adjust
                        => Some(AnimatableLonghand::MozTextSizeAdjust),
                    
    nsCSSPropertyID::eCSSProperty_text_indent
                        => Some(AnimatableLonghand::TextIndent),
                    
    nsCSSPropertyID::eCSSProperty_overflow_wrap
                        => Some(AnimatableLonghand::OverflowWrap),
                    
    nsCSSPropertyID::eCSSProperty_word_break
                        => Some(AnimatableLonghand::WordBreak),
                    
    nsCSSPropertyID::eCSSProperty_text_justify
                        => Some(AnimatableLonghand::TextJustify),
                    
    nsCSSPropertyID::eCSSProperty_text_align_last
                        => Some(AnimatableLonghand::TextAlignLast),
                    
    nsCSSPropertyID::eCSSProperty_text_align
                        => Some(AnimatableLonghand::TextAlign),
                    
    nsCSSPropertyID::eCSSProperty_letter_spacing
                        => Some(AnimatableLonghand::LetterSpacing),
                    
    nsCSSPropertyID::eCSSProperty_word_spacing
                        => Some(AnimatableLonghand::WordSpacing),
                    
    nsCSSPropertyID::eCSSProperty_white_space
                        => Some(AnimatableLonghand::WhiteSpace),
                    
    nsCSSPropertyID::eCSSProperty_text_shadow
                        => Some(AnimatableLonghand::TextShadow),
                    
    nsCSSPropertyID::eCSSProperty_text_emphasis_style
                        => Some(AnimatableLonghand::TextEmphasisStyle),
                    
    nsCSSPropertyID::eCSSProperty_text_emphasis_position
                        => Some(AnimatableLonghand::TextEmphasisPosition),
                    
    nsCSSPropertyID::eCSSProperty_text_emphasis_color
                        => Some(AnimatableLonghand::TextEmphasisColor),
                    
    nsCSSPropertyID::eCSSProperty__moz_tab_size
                        => Some(AnimatableLonghand::MozTabSize),
                    
    nsCSSPropertyID::eCSSProperty__webkit_text_fill_color
                        => Some(AnimatableLonghand::WebkitTextFillColor),
                    
    nsCSSPropertyID::eCSSProperty__webkit_text_stroke_color
                        => Some(AnimatableLonghand::WebkitTextStrokeColor),
                    
    nsCSSPropertyID::eCSSProperty__webkit_text_stroke_width
                        => Some(AnimatableLonghand::WebkitTextStrokeWidth),
                    
    nsCSSPropertyID::eCSSProperty_ruby_align
                        => Some(AnimatableLonghand::RubyAlign),
                    
    nsCSSPropertyID::eCSSProperty_ruby_position
                        => Some(AnimatableLonghand::RubyPosition),
                    
    nsCSSPropertyID::eCSSProperty_text_combine_upright
                        => Some(AnimatableLonghand::TextCombineUpright),
                    
    nsCSSPropertyID::eCSSProperty_text_rendering
                        => Some(AnimatableLonghand::TextRendering),
                    
    nsCSSPropertyID::eCSSProperty_list_style_position
                        => Some(AnimatableLonghand::ListStylePosition),
                    
    nsCSSPropertyID::eCSSProperty_list_style_type
                        => Some(AnimatableLonghand::ListStyleType),
                    
    nsCSSPropertyID::eCSSProperty_list_style_image
                        => Some(AnimatableLonghand::ListStyleImage),
                    
    nsCSSPropertyID::eCSSProperty_quotes
                        => Some(AnimatableLonghand::Quotes),
                    
    nsCSSPropertyID::eCSSProperty__moz_image_region
                        => Some(AnimatableLonghand::MozImageRegion),
                    
    nsCSSPropertyID::eCSSProperty_margin_top
                        => Some(AnimatableLonghand::MarginTop),
                    
    nsCSSPropertyID::eCSSProperty_margin_left
                        => Some(AnimatableLonghand::MarginLeft),
                    
    nsCSSPropertyID::eCSSProperty_margin_bottom
                        => Some(AnimatableLonghand::MarginBottom),
                    
    nsCSSPropertyID::eCSSProperty_margin_right
                        => Some(AnimatableLonghand::MarginRight),
                    
    nsCSSPropertyID::eCSSProperty_outline_color
                        => Some(AnimatableLonghand::OutlineColor),
                    
    nsCSSPropertyID::eCSSProperty_outline_style
                        => Some(AnimatableLonghand::OutlineStyle),
                    
    nsCSSPropertyID::eCSSProperty_outline_width
                        => Some(AnimatableLonghand::OutlineWidth),
                    
    nsCSSPropertyID::eCSSProperty__moz_outline_radius_topleft
                        => Some(AnimatableLonghand::MozOutlineRadiusTopleft),
                    
    nsCSSPropertyID::eCSSProperty__moz_outline_radius_topright
                        => Some(AnimatableLonghand::MozOutlineRadiusTopright),
                    
    nsCSSPropertyID::eCSSProperty__moz_outline_radius_bottomright
                        => Some(AnimatableLonghand::MozOutlineRadiusBottomright),
                    
    nsCSSPropertyID::eCSSProperty__moz_outline_radius_bottomleft
                        => Some(AnimatableLonghand::MozOutlineRadiusBottomleft),
                    
    nsCSSPropertyID::eCSSProperty_outline_offset
                        => Some(AnimatableLonghand::OutlineOffset),
                    
    nsCSSPropertyID::eCSSProperty_padding_top
                        => Some(AnimatableLonghand::PaddingTop),
                    
    nsCSSPropertyID::eCSSProperty_padding_left
                        => Some(AnimatableLonghand::PaddingLeft),
                    
    nsCSSPropertyID::eCSSProperty_padding_bottom
                        => Some(AnimatableLonghand::PaddingBottom),
                    
    nsCSSPropertyID::eCSSProperty_padding_right
                        => Some(AnimatableLonghand::PaddingRight),
                    
    nsCSSPropertyID::eCSSProperty_cursor
                        => Some(AnimatableLonghand::Cursor),
                    
    nsCSSPropertyID::eCSSProperty_pointer_events
                        => Some(AnimatableLonghand::PointerEvents),
                    
    nsCSSPropertyID::eCSSProperty__moz_user_input
                        => Some(AnimatableLonghand::MozUserInput),
                    
    nsCSSPropertyID::eCSSProperty__moz_user_modify
                        => Some(AnimatableLonghand::MozUserModify),
                    
    nsCSSPropertyID::eCSSProperty__moz_user_focus
                        => Some(AnimatableLonghand::MozUserFocus),
                    
    nsCSSPropertyID::eCSSProperty_caret_color
                        => Some(AnimatableLonghand::CaretColor),
                    
    nsCSSPropertyID::eCSSProperty_top
                        => Some(AnimatableLonghand::Top),
                    
    nsCSSPropertyID::eCSSProperty_left
                        => Some(AnimatableLonghand::Left),
                    
    nsCSSPropertyID::eCSSProperty_bottom
                        => Some(AnimatableLonghand::Bottom),
                    
    nsCSSPropertyID::eCSSProperty_right
                        => Some(AnimatableLonghand::Right),
                    
    nsCSSPropertyID::eCSSProperty_z_index
                        => Some(AnimatableLonghand::ZIndex),
                    
    nsCSSPropertyID::eCSSProperty_flex_direction
                        => Some(AnimatableLonghand::FlexDirection),
                    
    nsCSSPropertyID::eCSSProperty_flex_wrap
                        => Some(AnimatableLonghand::FlexWrap),
                    
    nsCSSPropertyID::eCSSProperty_justify_content
                        => Some(AnimatableLonghand::JustifyContent),
                    
    nsCSSPropertyID::eCSSProperty_align_content
                        => Some(AnimatableLonghand::AlignContent),
                    
    nsCSSPropertyID::eCSSProperty_align_items
                        => Some(AnimatableLonghand::AlignItems),
                    
    nsCSSPropertyID::eCSSProperty_justify_items
                        => Some(AnimatableLonghand::JustifyItems),
                    
    nsCSSPropertyID::eCSSProperty_flex_grow
                        => Some(AnimatableLonghand::FlexGrow),
                    
    nsCSSPropertyID::eCSSProperty_flex_shrink
                        => Some(AnimatableLonghand::FlexShrink),
                    
    nsCSSPropertyID::eCSSProperty_align_self
                        => Some(AnimatableLonghand::AlignSelf),
                    
    nsCSSPropertyID::eCSSProperty_justify_self
                        => Some(AnimatableLonghand::JustifySelf),
                    
    nsCSSPropertyID::eCSSProperty_order
                        => Some(AnimatableLonghand::Order),
                    
    nsCSSPropertyID::eCSSProperty_flex_basis
                        => Some(AnimatableLonghand::FlexBasis),
                    
    nsCSSPropertyID::eCSSProperty_width
                        => Some(AnimatableLonghand::Width),
                    
    nsCSSPropertyID::eCSSProperty_min_width
                        => Some(AnimatableLonghand::MinWidth),
                    
    nsCSSPropertyID::eCSSProperty_max_width
                        => Some(AnimatableLonghand::MaxWidth),
                    
    nsCSSPropertyID::eCSSProperty_height
                        => Some(AnimatableLonghand::Height),
                    
    nsCSSPropertyID::eCSSProperty_min_height
                        => Some(AnimatableLonghand::MinHeight),
                    
    nsCSSPropertyID::eCSSProperty_max_height
                        => Some(AnimatableLonghand::MaxHeight),
                    
    nsCSSPropertyID::eCSSProperty_box_sizing
                        => Some(AnimatableLonghand::BoxSizing),
                    
    nsCSSPropertyID::eCSSProperty_object_fit
                        => Some(AnimatableLonghand::ObjectFit),
                    
    nsCSSPropertyID::eCSSProperty_object_position
                        => Some(AnimatableLonghand::ObjectPosition),
                    
    nsCSSPropertyID::eCSSProperty_grid_row_gap
                        => Some(AnimatableLonghand::GridRowGap),
                    
    nsCSSPropertyID::eCSSProperty_grid_row_start
                        => Some(AnimatableLonghand::GridRowStart),
                    
    nsCSSPropertyID::eCSSProperty_grid_row_end
                        => Some(AnimatableLonghand::GridRowEnd),
                    
    nsCSSPropertyID::eCSSProperty_grid_auto_rows
                        => Some(AnimatableLonghand::GridAutoRows),
                    
    nsCSSPropertyID::eCSSProperty_grid_template_rows
                        => Some(AnimatableLonghand::GridTemplateRows),
                    
    nsCSSPropertyID::eCSSProperty_grid_column_gap
                        => Some(AnimatableLonghand::GridColumnGap),
                    
    nsCSSPropertyID::eCSSProperty_grid_column_start
                        => Some(AnimatableLonghand::GridColumnStart),
                    
    nsCSSPropertyID::eCSSProperty_grid_column_end
                        => Some(AnimatableLonghand::GridColumnEnd),
                    
    nsCSSPropertyID::eCSSProperty_grid_auto_columns
                        => Some(AnimatableLonghand::GridAutoColumns),
                    
    nsCSSPropertyID::eCSSProperty_grid_template_columns
                        => Some(AnimatableLonghand::GridTemplateColumns),
                    
    nsCSSPropertyID::eCSSProperty_grid_auto_flow
                        => Some(AnimatableLonghand::GridAutoFlow),
                    
    nsCSSPropertyID::eCSSProperty_grid_template_areas
                        => Some(AnimatableLonghand::GridTemplateAreas),
                    
    nsCSSPropertyID::eCSSProperty_table_layout
                        => Some(AnimatableLonghand::TableLayout),
                    
    nsCSSPropertyID::eCSSProperty_text_overflow
                        => Some(AnimatableLonghand::TextOverflow),
                    
    nsCSSPropertyID::eCSSProperty_unicode_bidi
                        => Some(AnimatableLonghand::UnicodeBidi),
                    
    nsCSSPropertyID::eCSSProperty_text_decoration_line
                        => Some(AnimatableLonghand::TextDecorationLine),
                    
    nsCSSPropertyID::eCSSProperty_text_decoration_style
                        => Some(AnimatableLonghand::TextDecorationStyle),
                    
    nsCSSPropertyID::eCSSProperty_text_decoration_color
                        => Some(AnimatableLonghand::TextDecorationColor),
                    
    nsCSSPropertyID::eCSSProperty_initial_letter
                        => Some(AnimatableLonghand::InitialLetter),
                    
    nsCSSPropertyID::eCSSProperty_ime_mode
                        => Some(AnimatableLonghand::ImeMode),
                    
    nsCSSPropertyID::eCSSProperty__moz_user_select
                        => Some(AnimatableLonghand::MozUserSelect),
                    
    nsCSSPropertyID::eCSSProperty__moz_window_dragging
                        => Some(AnimatableLonghand::MozWindowDragging),
                    
    nsCSSPropertyID::eCSSProperty__moz_window_shadow
                        => Some(AnimatableLonghand::MozWindowShadow),
                    
    nsCSSPropertyID::eCSSProperty__moz_force_broken_image_icon
                        => Some(AnimatableLonghand::MozForceBrokenImageIcon),
                    
    nsCSSPropertyID::eCSSProperty_text_anchor
                        => Some(AnimatableLonghand::TextAnchor),
                    
    nsCSSPropertyID::eCSSProperty_color_interpolation
                        => Some(AnimatableLonghand::ColorInterpolation),
                    
    nsCSSPropertyID::eCSSProperty_color_interpolation_filters
                        => Some(AnimatableLonghand::ColorInterpolationFilters),
                    
    nsCSSPropertyID::eCSSProperty_fill
                        => Some(AnimatableLonghand::Fill),
                    
    nsCSSPropertyID::eCSSProperty_fill_opacity
                        => Some(AnimatableLonghand::FillOpacity),
                    
    nsCSSPropertyID::eCSSProperty_fill_rule
                        => Some(AnimatableLonghand::FillRule),
                    
    nsCSSPropertyID::eCSSProperty_shape_rendering
                        => Some(AnimatableLonghand::ShapeRendering),
                    
    nsCSSPropertyID::eCSSProperty_stroke
                        => Some(AnimatableLonghand::Stroke),
                    
    nsCSSPropertyID::eCSSProperty_stroke_width
                        => Some(AnimatableLonghand::StrokeWidth),
                    
    nsCSSPropertyID::eCSSProperty_stroke_linecap
                        => Some(AnimatableLonghand::StrokeLinecap),
                    
    nsCSSPropertyID::eCSSProperty_stroke_linejoin
                        => Some(AnimatableLonghand::StrokeLinejoin),
                    
    nsCSSPropertyID::eCSSProperty_stroke_miterlimit
                        => Some(AnimatableLonghand::StrokeMiterlimit),
                    
    nsCSSPropertyID::eCSSProperty_stroke_opacity
                        => Some(AnimatableLonghand::StrokeOpacity),
                    
    nsCSSPropertyID::eCSSProperty_stroke_dasharray
                        => Some(AnimatableLonghand::StrokeDasharray),
                    
    nsCSSPropertyID::eCSSProperty_stroke_dashoffset
                        => Some(AnimatableLonghand::StrokeDashoffset),
                    
    nsCSSPropertyID::eCSSProperty_clip_rule
                        => Some(AnimatableLonghand::ClipRule),
                    
    nsCSSPropertyID::eCSSProperty_marker_start
                        => Some(AnimatableLonghand::MarkerStart),
                    
    nsCSSPropertyID::eCSSProperty_marker_mid
                        => Some(AnimatableLonghand::MarkerMid),
                    
    nsCSSPropertyID::eCSSProperty_marker_end
                        => Some(AnimatableLonghand::MarkerEnd),
                    
    nsCSSPropertyID::eCSSProperty_paint_order
                        => Some(AnimatableLonghand::PaintOrder),
                    
    nsCSSPropertyID::eCSSProperty_dominant_baseline
                        => Some(AnimatableLonghand::DominantBaseline),
                    
    nsCSSPropertyID::eCSSProperty_vector_effect
                        => Some(AnimatableLonghand::VectorEffect),
                    
    nsCSSPropertyID::eCSSProperty_stop_color
                        => Some(AnimatableLonghand::StopColor),
                    
    nsCSSPropertyID::eCSSProperty_stop_opacity
                        => Some(AnimatableLonghand::StopOpacity),
                    
    nsCSSPropertyID::eCSSProperty_flood_color
                        => Some(AnimatableLonghand::FloodColor),
                    
    nsCSSPropertyID::eCSSProperty_flood_opacity
                        => Some(AnimatableLonghand::FloodOpacity),
                    
    nsCSSPropertyID::eCSSProperty_lighting_color
                        => Some(AnimatableLonghand::LightingColor),
                    
    nsCSSPropertyID::eCSSProperty_mask_type
                        => Some(AnimatableLonghand::MaskType),
                    
    nsCSSPropertyID::eCSSProperty_clip_path
                        => Some(AnimatableLonghand::ClipPath),
                    
    nsCSSPropertyID::eCSSProperty_mask_mode
                        => Some(AnimatableLonghand::MaskMode),
                    
    nsCSSPropertyID::eCSSProperty_mask_repeat
                        => Some(AnimatableLonghand::MaskRepeat),
                    
    nsCSSPropertyID::eCSSProperty_mask_position_x
                        => Some(AnimatableLonghand::MaskPositionX),
                    
    nsCSSPropertyID::eCSSProperty_mask_position_y
                        => Some(AnimatableLonghand::MaskPositionY),
                    
    nsCSSPropertyID::eCSSProperty_mask_clip
                        => Some(AnimatableLonghand::MaskClip),
                    
    nsCSSPropertyID::eCSSProperty_mask_origin
                        => Some(AnimatableLonghand::MaskOrigin),
                    
    nsCSSPropertyID::eCSSProperty_mask_size
                        => Some(AnimatableLonghand::MaskSize),
                    
    nsCSSPropertyID::eCSSProperty_mask_composite
                        => Some(AnimatableLonghand::MaskComposite),
                    
    nsCSSPropertyID::eCSSProperty_mask_image
                        => Some(AnimatableLonghand::MaskImage),
                    
    nsCSSPropertyID::eCSSProperty__moz_box_align
                        => Some(AnimatableLonghand::MozBoxAlign),
                    
    nsCSSPropertyID::eCSSProperty__moz_box_direction
                        => Some(AnimatableLonghand::MozBoxDirection),
                    
    nsCSSPropertyID::eCSSProperty__moz_box_flex
                        => Some(AnimatableLonghand::MozBoxFlex),
                    
    nsCSSPropertyID::eCSSProperty__moz_box_orient
                        => Some(AnimatableLonghand::MozBoxOrient),
                    
    nsCSSPropertyID::eCSSProperty__moz_box_pack
                        => Some(AnimatableLonghand::MozBoxPack),
                    
    nsCSSPropertyID::eCSSProperty__moz_stack_sizing
                        => Some(AnimatableLonghand::MozStackSizing),
                    
    nsCSSPropertyID::eCSSProperty__moz_box_ordinal_group
                        => Some(AnimatableLonghand::MozBoxOrdinalGroup),
            _ => None
        }
    }

    /// Converts from TransitionProperty. Returns None if the property is not an animatable
    /// longhand.
    pub fn from_transition_property(transition_property: &TransitionProperty) -> Option<Self> {
        match *transition_property {
                    TransitionProperty::BackgroundColor
                        => Some(AnimatableLonghand::BackgroundColor),
                    TransitionProperty::BackgroundPositionX
                        => Some(AnimatableLonghand::BackgroundPositionX),
                    TransitionProperty::BackgroundPositionY
                        => Some(AnimatableLonghand::BackgroundPositionY),
                    TransitionProperty::BackgroundSize
                        => Some(AnimatableLonghand::BackgroundSize),
                    TransitionProperty::BorderTopColor
                        => Some(AnimatableLonghand::BorderTopColor),
                    TransitionProperty::BorderTopWidth
                        => Some(AnimatableLonghand::BorderTopWidth),
                    TransitionProperty::BorderLeftColor
                        => Some(AnimatableLonghand::BorderLeftColor),
                    TransitionProperty::BorderLeftWidth
                        => Some(AnimatableLonghand::BorderLeftWidth),
                    TransitionProperty::BorderBottomColor
                        => Some(AnimatableLonghand::BorderBottomColor),
                    TransitionProperty::BorderBottomWidth
                        => Some(AnimatableLonghand::BorderBottomWidth),
                    TransitionProperty::BorderRightColor
                        => Some(AnimatableLonghand::BorderRightColor),
                    TransitionProperty::BorderRightWidth
                        => Some(AnimatableLonghand::BorderRightWidth),
                    TransitionProperty::BorderTopLeftRadius
                        => Some(AnimatableLonghand::BorderTopLeftRadius),
                    TransitionProperty::BorderTopRightRadius
                        => Some(AnimatableLonghand::BorderTopRightRadius),
                    TransitionProperty::BorderBottomRightRadius
                        => Some(AnimatableLonghand::BorderBottomRightRadius),
                    TransitionProperty::BorderBottomLeftRadius
                        => Some(AnimatableLonghand::BorderBottomLeftRadius),
                    TransitionProperty::VerticalAlign
                        => Some(AnimatableLonghand::VerticalAlign),
                    TransitionProperty::Transform
                        => Some(AnimatableLonghand::Transform),
                    TransitionProperty::Perspective
                        => Some(AnimatableLonghand::Perspective),
                    TransitionProperty::PerspectiveOrigin
                        => Some(AnimatableLonghand::PerspectiveOrigin),
                    TransitionProperty::TransformOrigin
                        => Some(AnimatableLonghand::TransformOrigin),
                    TransitionProperty::ShapeOutside
                        => Some(AnimatableLonghand::ShapeOutside),
                    TransitionProperty::Color
                        => Some(AnimatableLonghand::Color),
                    TransitionProperty::ColumnWidth
                        => Some(AnimatableLonghand::ColumnWidth),
                    TransitionProperty::ColumnCount
                        => Some(AnimatableLonghand::ColumnCount),
                    TransitionProperty::ColumnGap
                        => Some(AnimatableLonghand::ColumnGap),
                    TransitionProperty::ColumnRuleWidth
                        => Some(AnimatableLonghand::ColumnRuleWidth),
                    TransitionProperty::ColumnRuleColor
                        => Some(AnimatableLonghand::ColumnRuleColor),
                    TransitionProperty::Opacity
                        => Some(AnimatableLonghand::Opacity),
                    TransitionProperty::BoxShadow
                        => Some(AnimatableLonghand::BoxShadow),
                    TransitionProperty::Clip
                        => Some(AnimatableLonghand::Clip),
                    TransitionProperty::Filter
                        => Some(AnimatableLonghand::Filter),
                    TransitionProperty::FontWeight
                        => Some(AnimatableLonghand::FontWeight),
                    TransitionProperty::FontSize
                        => Some(AnimatableLonghand::FontSize),
                    TransitionProperty::FontSizeAdjust
                        => Some(AnimatableLonghand::FontSizeAdjust),
                    TransitionProperty::FontStretch
                        => Some(AnimatableLonghand::FontStretch),
                    TransitionProperty::FontVariationSettings
                        => Some(AnimatableLonghand::FontVariationSettings),
                    TransitionProperty::Visibility
                        => Some(AnimatableLonghand::Visibility),
                    TransitionProperty::BorderSpacing
                        => Some(AnimatableLonghand::BorderSpacing),
                    TransitionProperty::LineHeight
                        => Some(AnimatableLonghand::LineHeight),
                    TransitionProperty::TextIndent
                        => Some(AnimatableLonghand::TextIndent),
                    TransitionProperty::LetterSpacing
                        => Some(AnimatableLonghand::LetterSpacing),
                    TransitionProperty::WordSpacing
                        => Some(AnimatableLonghand::WordSpacing),
                    TransitionProperty::TextShadow
                        => Some(AnimatableLonghand::TextShadow),
                    TransitionProperty::TextEmphasisColor
                        => Some(AnimatableLonghand::TextEmphasisColor),
                    TransitionProperty::MozTabSize
                        => Some(AnimatableLonghand::MozTabSize),
                    TransitionProperty::WebkitTextFillColor
                        => Some(AnimatableLonghand::WebkitTextFillColor),
                    TransitionProperty::WebkitTextStrokeColor
                        => Some(AnimatableLonghand::WebkitTextStrokeColor),
                    TransitionProperty::MozImageRegion
                        => Some(AnimatableLonghand::MozImageRegion),
                    TransitionProperty::MarginTop
                        => Some(AnimatableLonghand::MarginTop),
                    TransitionProperty::MarginLeft
                        => Some(AnimatableLonghand::MarginLeft),
                    TransitionProperty::MarginBottom
                        => Some(AnimatableLonghand::MarginBottom),
                    TransitionProperty::MarginRight
                        => Some(AnimatableLonghand::MarginRight),
                    TransitionProperty::OutlineColor
                        => Some(AnimatableLonghand::OutlineColor),
                    TransitionProperty::OutlineWidth
                        => Some(AnimatableLonghand::OutlineWidth),
                    TransitionProperty::MozOutlineRadiusTopleft
                        => Some(AnimatableLonghand::MozOutlineRadiusTopleft),
                    TransitionProperty::MozOutlineRadiusTopright
                        => Some(AnimatableLonghand::MozOutlineRadiusTopright),
                    TransitionProperty::MozOutlineRadiusBottomright
                        => Some(AnimatableLonghand::MozOutlineRadiusBottomright),
                    TransitionProperty::MozOutlineRadiusBottomleft
                        => Some(AnimatableLonghand::MozOutlineRadiusBottomleft),
                    TransitionProperty::OutlineOffset
                        => Some(AnimatableLonghand::OutlineOffset),
                    TransitionProperty::PaddingTop
                        => Some(AnimatableLonghand::PaddingTop),
                    TransitionProperty::PaddingLeft
                        => Some(AnimatableLonghand::PaddingLeft),
                    TransitionProperty::PaddingBottom
                        => Some(AnimatableLonghand::PaddingBottom),
                    TransitionProperty::PaddingRight
                        => Some(AnimatableLonghand::PaddingRight),
                    TransitionProperty::CaretColor
                        => Some(AnimatableLonghand::CaretColor),
                    TransitionProperty::Top
                        => Some(AnimatableLonghand::Top),
                    TransitionProperty::Left
                        => Some(AnimatableLonghand::Left),
                    TransitionProperty::Bottom
                        => Some(AnimatableLonghand::Bottom),
                    TransitionProperty::Right
                        => Some(AnimatableLonghand::Right),
                    TransitionProperty::ZIndex
                        => Some(AnimatableLonghand::ZIndex),
                    TransitionProperty::FlexGrow
                        => Some(AnimatableLonghand::FlexGrow),
                    TransitionProperty::FlexShrink
                        => Some(AnimatableLonghand::FlexShrink),
                    TransitionProperty::Order
                        => Some(AnimatableLonghand::Order),
                    TransitionProperty::FlexBasis
                        => Some(AnimatableLonghand::FlexBasis),
                    TransitionProperty::Width
                        => Some(AnimatableLonghand::Width),
                    TransitionProperty::MinWidth
                        => Some(AnimatableLonghand::MinWidth),
                    TransitionProperty::MaxWidth
                        => Some(AnimatableLonghand::MaxWidth),
                    TransitionProperty::Height
                        => Some(AnimatableLonghand::Height),
                    TransitionProperty::MinHeight
                        => Some(AnimatableLonghand::MinHeight),
                    TransitionProperty::MaxHeight
                        => Some(AnimatableLonghand::MaxHeight),
                    TransitionProperty::ObjectPosition
                        => Some(AnimatableLonghand::ObjectPosition),
                    TransitionProperty::GridRowGap
                        => Some(AnimatableLonghand::GridRowGap),
                    TransitionProperty::GridColumnGap
                        => Some(AnimatableLonghand::GridColumnGap),
                    TransitionProperty::TextDecorationColor
                        => Some(AnimatableLonghand::TextDecorationColor),
                    TransitionProperty::Fill
                        => Some(AnimatableLonghand::Fill),
                    TransitionProperty::FillOpacity
                        => Some(AnimatableLonghand::FillOpacity),
                    TransitionProperty::Stroke
                        => Some(AnimatableLonghand::Stroke),
                    TransitionProperty::StrokeWidth
                        => Some(AnimatableLonghand::StrokeWidth),
                    TransitionProperty::StrokeMiterlimit
                        => Some(AnimatableLonghand::StrokeMiterlimit),
                    TransitionProperty::StrokeOpacity
                        => Some(AnimatableLonghand::StrokeOpacity),
                    TransitionProperty::StrokeDasharray
                        => Some(AnimatableLonghand::StrokeDasharray),
                    TransitionProperty::StrokeDashoffset
                        => Some(AnimatableLonghand::StrokeDashoffset),
                    TransitionProperty::StopColor
                        => Some(AnimatableLonghand::StopColor),
                    TransitionProperty::StopOpacity
                        => Some(AnimatableLonghand::StopOpacity),
                    TransitionProperty::FloodColor
                        => Some(AnimatableLonghand::FloodColor),
                    TransitionProperty::FloodOpacity
                        => Some(AnimatableLonghand::FloodOpacity),
                    TransitionProperty::LightingColor
                        => Some(AnimatableLonghand::LightingColor),
                    TransitionProperty::ClipPath
                        => Some(AnimatableLonghand::ClipPath),
                    TransitionProperty::MaskPositionX
                        => Some(AnimatableLonghand::MaskPositionX),
                    TransitionProperty::MaskPositionY
                        => Some(AnimatableLonghand::MaskPositionY),
                    TransitionProperty::MaskSize
                        => Some(AnimatableLonghand::MaskSize),
                    TransitionProperty::MozBoxFlex
                        => Some(AnimatableLonghand::MozBoxFlex),
            _ => None
        }
    }

    /// Get an animatable longhand property from a property declaration.
    pub fn from_declaration(declaration: &PropertyDeclaration) -> Option<Self> {
        use properties::LonghandId;
        match *declaration {
                    PropertyDeclaration::BackgroundColor(..)
                        => Some(AnimatableLonghand::BackgroundColor),
                    PropertyDeclaration::BackgroundImage(..)
                        => Some(AnimatableLonghand::BackgroundImage),
                    PropertyDeclaration::BackgroundPositionX(..)
                        => Some(AnimatableLonghand::BackgroundPositionX),
                    PropertyDeclaration::BackgroundPositionY(..)
                        => Some(AnimatableLonghand::BackgroundPositionY),
                    PropertyDeclaration::BackgroundRepeat(..)
                        => Some(AnimatableLonghand::BackgroundRepeat),
                    PropertyDeclaration::BackgroundAttachment(..)
                        => Some(AnimatableLonghand::BackgroundAttachment),
                    PropertyDeclaration::BackgroundClip(..)
                        => Some(AnimatableLonghand::BackgroundClip),
                    PropertyDeclaration::BackgroundOrigin(..)
                        => Some(AnimatableLonghand::BackgroundOrigin),
                    PropertyDeclaration::BackgroundSize(..)
                        => Some(AnimatableLonghand::BackgroundSize),
                    PropertyDeclaration::BackgroundBlendMode(..)
                        => Some(AnimatableLonghand::BackgroundBlendMode),
                    PropertyDeclaration::BorderTopColor(..)
                        => Some(AnimatableLonghand::BorderTopColor),
                    PropertyDeclaration::BorderTopStyle(..)
                        => Some(AnimatableLonghand::BorderTopStyle),
                    PropertyDeclaration::BorderTopWidth(..)
                        => Some(AnimatableLonghand::BorderTopWidth),
                    PropertyDeclaration::BorderLeftColor(..)
                        => Some(AnimatableLonghand::BorderLeftColor),
                    PropertyDeclaration::BorderLeftStyle(..)
                        => Some(AnimatableLonghand::BorderLeftStyle),
                    PropertyDeclaration::BorderLeftWidth(..)
                        => Some(AnimatableLonghand::BorderLeftWidth),
                    PropertyDeclaration::BorderBottomColor(..)
                        => Some(AnimatableLonghand::BorderBottomColor),
                    PropertyDeclaration::BorderBottomStyle(..)
                        => Some(AnimatableLonghand::BorderBottomStyle),
                    PropertyDeclaration::BorderBottomWidth(..)
                        => Some(AnimatableLonghand::BorderBottomWidth),
                    PropertyDeclaration::BorderRightColor(..)
                        => Some(AnimatableLonghand::BorderRightColor),
                    PropertyDeclaration::BorderRightStyle(..)
                        => Some(AnimatableLonghand::BorderRightStyle),
                    PropertyDeclaration::BorderRightWidth(..)
                        => Some(AnimatableLonghand::BorderRightWidth),
                    PropertyDeclaration::BorderTopLeftRadius(..)
                        => Some(AnimatableLonghand::BorderTopLeftRadius),
                    PropertyDeclaration::BorderTopRightRadius(..)
                        => Some(AnimatableLonghand::BorderTopRightRadius),
                    PropertyDeclaration::BorderBottomRightRadius(..)
                        => Some(AnimatableLonghand::BorderBottomRightRadius),
                    PropertyDeclaration::BorderBottomLeftRadius(..)
                        => Some(AnimatableLonghand::BorderBottomLeftRadius),
                    PropertyDeclaration::MozBorderTopColors(..)
                        => Some(AnimatableLonghand::MozBorderTopColors),
                    PropertyDeclaration::MozBorderLeftColors(..)
                        => Some(AnimatableLonghand::MozBorderLeftColors),
                    PropertyDeclaration::MozBorderBottomColors(..)
                        => Some(AnimatableLonghand::MozBorderBottomColors),
                    PropertyDeclaration::MozBorderRightColors(..)
                        => Some(AnimatableLonghand::MozBorderRightColors),
                    PropertyDeclaration::BoxDecorationBreak(..)
                        => Some(AnimatableLonghand::BoxDecorationBreak),
                    PropertyDeclaration::MozFloatEdge(..)
                        => Some(AnimatableLonghand::MozFloatEdge),
                    PropertyDeclaration::BorderImageSource(..)
                        => Some(AnimatableLonghand::BorderImageSource),
                    PropertyDeclaration::BorderImageOutset(..)
                        => Some(AnimatableLonghand::BorderImageOutset),
                    PropertyDeclaration::BorderImageRepeat(..)
                        => Some(AnimatableLonghand::BorderImageRepeat),
                    PropertyDeclaration::BorderImageWidth(..)
                        => Some(AnimatableLonghand::BorderImageWidth),
                    PropertyDeclaration::BorderImageSlice(..)
                        => Some(AnimatableLonghand::BorderImageSlice),
                    PropertyDeclaration::Display(..)
                        => Some(AnimatableLonghand::Display),
                    PropertyDeclaration::Position(..)
                        => Some(AnimatableLonghand::Position),
                    PropertyDeclaration::Float(..)
                        => Some(AnimatableLonghand::Float),
                    PropertyDeclaration::Clear(..)
                        => Some(AnimatableLonghand::Clear),
                    PropertyDeclaration::VerticalAlign(..)
                        => Some(AnimatableLonghand::VerticalAlign),
                    PropertyDeclaration::OverflowClipBox(..)
                        => Some(AnimatableLonghand::OverflowClipBox),
                    PropertyDeclaration::OverflowX(..)
                        => Some(AnimatableLonghand::OverflowX),
                    PropertyDeclaration::OverflowY(..)
                        => Some(AnimatableLonghand::OverflowY),
                    PropertyDeclaration::ScrollSnapPointsX(..)
                        => Some(AnimatableLonghand::ScrollSnapPointsX),
                    PropertyDeclaration::ScrollSnapPointsY(..)
                        => Some(AnimatableLonghand::ScrollSnapPointsY),
                    PropertyDeclaration::ScrollSnapDestination(..)
                        => Some(AnimatableLonghand::ScrollSnapDestination),
                    PropertyDeclaration::ScrollSnapCoordinate(..)
                        => Some(AnimatableLonghand::ScrollSnapCoordinate),
                    PropertyDeclaration::Transform(..)
                        => Some(AnimatableLonghand::Transform),
                    PropertyDeclaration::ScrollBehavior(..)
                        => Some(AnimatableLonghand::ScrollBehavior),
                    PropertyDeclaration::ScrollSnapTypeX(..)
                        => Some(AnimatableLonghand::ScrollSnapTypeX),
                    PropertyDeclaration::ScrollSnapTypeY(..)
                        => Some(AnimatableLonghand::ScrollSnapTypeY),
                    PropertyDeclaration::Isolation(..)
                        => Some(AnimatableLonghand::Isolation),
                    PropertyDeclaration::PageBreakAfter(..)
                        => Some(AnimatableLonghand::PageBreakAfter),
                    PropertyDeclaration::PageBreakBefore(..)
                        => Some(AnimatableLonghand::PageBreakBefore),
                    PropertyDeclaration::PageBreakInside(..)
                        => Some(AnimatableLonghand::PageBreakInside),
                    PropertyDeclaration::Resize(..)
                        => Some(AnimatableLonghand::Resize),
                    PropertyDeclaration::Perspective(..)
                        => Some(AnimatableLonghand::Perspective),
                    PropertyDeclaration::PerspectiveOrigin(..)
                        => Some(AnimatableLonghand::PerspectiveOrigin),
                    PropertyDeclaration::BackfaceVisibility(..)
                        => Some(AnimatableLonghand::BackfaceVisibility),
                    PropertyDeclaration::TransformBox(..)
                        => Some(AnimatableLonghand::TransformBox),
                    PropertyDeclaration::TransformStyle(..)
                        => Some(AnimatableLonghand::TransformStyle),
                    PropertyDeclaration::TransformOrigin(..)
                        => Some(AnimatableLonghand::TransformOrigin),
                    PropertyDeclaration::Contain(..)
                        => Some(AnimatableLonghand::Contain),
                    PropertyDeclaration::MozAppearance(..)
                        => Some(AnimatableLonghand::MozAppearance),
                    PropertyDeclaration::MozOrient(..)
                        => Some(AnimatableLonghand::MozOrient),
                    PropertyDeclaration::WillChange(..)
                        => Some(AnimatableLonghand::WillChange),
                    PropertyDeclaration::ShapeOutside(..)
                        => Some(AnimatableLonghand::ShapeOutside),
                    PropertyDeclaration::TouchAction(..)
                        => Some(AnimatableLonghand::TouchAction),
                    PropertyDeclaration::Color(..)
                        => Some(AnimatableLonghand::Color),
                    PropertyDeclaration::ColumnWidth(..)
                        => Some(AnimatableLonghand::ColumnWidth),
                    PropertyDeclaration::ColumnCount(..)
                        => Some(AnimatableLonghand::ColumnCount),
                    PropertyDeclaration::ColumnGap(..)
                        => Some(AnimatableLonghand::ColumnGap),
                    PropertyDeclaration::ColumnFill(..)
                        => Some(AnimatableLonghand::ColumnFill),
                    PropertyDeclaration::ColumnRuleWidth(..)
                        => Some(AnimatableLonghand::ColumnRuleWidth),
                    PropertyDeclaration::ColumnRuleColor(..)
                        => Some(AnimatableLonghand::ColumnRuleColor),
                    PropertyDeclaration::ColumnSpan(..)
                        => Some(AnimatableLonghand::ColumnSpan),
                    PropertyDeclaration::ColumnRuleStyle(..)
                        => Some(AnimatableLonghand::ColumnRuleStyle),
                    PropertyDeclaration::Content(..)
                        => Some(AnimatableLonghand::Content),
                    PropertyDeclaration::CounterIncrement(..)
                        => Some(AnimatableLonghand::CounterIncrement),
                    PropertyDeclaration::CounterReset(..)
                        => Some(AnimatableLonghand::CounterReset),
                    PropertyDeclaration::Opacity(..)
                        => Some(AnimatableLonghand::Opacity),
                    PropertyDeclaration::BoxShadow(..)
                        => Some(AnimatableLonghand::BoxShadow),
                    PropertyDeclaration::Clip(..)
                        => Some(AnimatableLonghand::Clip),
                    PropertyDeclaration::Filter(..)
                        => Some(AnimatableLonghand::Filter),
                    PropertyDeclaration::MixBlendMode(..)
                        => Some(AnimatableLonghand::MixBlendMode),
                    PropertyDeclaration::FontFamily(..)
                        => Some(AnimatableLonghand::FontFamily),
                    PropertyDeclaration::FontStyle(..)
                        => Some(AnimatableLonghand::FontStyle),
                    PropertyDeclaration::FontVariantCaps(..)
                        => Some(AnimatableLonghand::FontVariantCaps),
                    PropertyDeclaration::FontWeight(..)
                        => Some(AnimatableLonghand::FontWeight),
                    PropertyDeclaration::FontSize(..)
                        => Some(AnimatableLonghand::FontSize),
                    PropertyDeclaration::FontSizeAdjust(..)
                        => Some(AnimatableLonghand::FontSizeAdjust),
                    PropertyDeclaration::FontSynthesis(..)
                        => Some(AnimatableLonghand::FontSynthesis),
                    PropertyDeclaration::FontStretch(..)
                        => Some(AnimatableLonghand::FontStretch),
                    PropertyDeclaration::FontKerning(..)
                        => Some(AnimatableLonghand::FontKerning),
                    PropertyDeclaration::FontVariantAlternates(..)
                        => Some(AnimatableLonghand::FontVariantAlternates),
                    PropertyDeclaration::FontVariantEastAsian(..)
                        => Some(AnimatableLonghand::FontVariantEastAsian),
                    PropertyDeclaration::FontVariantLigatures(..)
                        => Some(AnimatableLonghand::FontVariantLigatures),
                    PropertyDeclaration::FontVariantNumeric(..)
                        => Some(AnimatableLonghand::FontVariantNumeric),
                    PropertyDeclaration::FontVariantPosition(..)
                        => Some(AnimatableLonghand::FontVariantPosition),
                    PropertyDeclaration::FontFeatureSettings(..)
                        => Some(AnimatableLonghand::FontFeatureSettings),
                    PropertyDeclaration::FontVariationSettings(..)
                        => Some(AnimatableLonghand::FontVariationSettings),
                    PropertyDeclaration::FontLanguageOverride(..)
                        => Some(AnimatableLonghand::FontLanguageOverride),
                    PropertyDeclaration::MozOsxFontSmoothing(..)
                        => Some(AnimatableLonghand::MozOsxFontSmoothing),
                    PropertyDeclaration::Visibility(..)
                        => Some(AnimatableLonghand::Visibility),
                    PropertyDeclaration::WritingMode(..)
                        => Some(AnimatableLonghand::WritingMode),
                    PropertyDeclaration::Direction(..)
                        => Some(AnimatableLonghand::Direction),
                    PropertyDeclaration::TextOrientation(..)
                        => Some(AnimatableLonghand::TextOrientation),
                    PropertyDeclaration::ColorAdjust(..)
                        => Some(AnimatableLonghand::ColorAdjust),
                    PropertyDeclaration::ImageRendering(..)
                        => Some(AnimatableLonghand::ImageRendering),
                    PropertyDeclaration::ImageOrientation(..)
                        => Some(AnimatableLonghand::ImageOrientation),
                    PropertyDeclaration::BorderCollapse(..)
                        => Some(AnimatableLonghand::BorderCollapse),
                    PropertyDeclaration::EmptyCells(..)
                        => Some(AnimatableLonghand::EmptyCells),
                    PropertyDeclaration::CaptionSide(..)
                        => Some(AnimatableLonghand::CaptionSide),
                    PropertyDeclaration::BorderSpacing(..)
                        => Some(AnimatableLonghand::BorderSpacing),
                    PropertyDeclaration::LineHeight(..)
                        => Some(AnimatableLonghand::LineHeight),
                    PropertyDeclaration::TextTransform(..)
                        => Some(AnimatableLonghand::TextTransform),
                    PropertyDeclaration::Hyphens(..)
                        => Some(AnimatableLonghand::Hyphens),
                    PropertyDeclaration::MozTextSizeAdjust(..)
                        => Some(AnimatableLonghand::MozTextSizeAdjust),
                    PropertyDeclaration::TextIndent(..)
                        => Some(AnimatableLonghand::TextIndent),
                    PropertyDeclaration::OverflowWrap(..)
                        => Some(AnimatableLonghand::OverflowWrap),
                    PropertyDeclaration::WordBreak(..)
                        => Some(AnimatableLonghand::WordBreak),
                    PropertyDeclaration::TextJustify(..)
                        => Some(AnimatableLonghand::TextJustify),
                    PropertyDeclaration::TextAlignLast(..)
                        => Some(AnimatableLonghand::TextAlignLast),
                    PropertyDeclaration::TextAlign(..)
                        => Some(AnimatableLonghand::TextAlign),
                    PropertyDeclaration::LetterSpacing(..)
                        => Some(AnimatableLonghand::LetterSpacing),
                    PropertyDeclaration::WordSpacing(..)
                        => Some(AnimatableLonghand::WordSpacing),
                    PropertyDeclaration::WhiteSpace(..)
                        => Some(AnimatableLonghand::WhiteSpace),
                    PropertyDeclaration::TextShadow(..)
                        => Some(AnimatableLonghand::TextShadow),
                    PropertyDeclaration::TextEmphasisStyle(..)
                        => Some(AnimatableLonghand::TextEmphasisStyle),
                    PropertyDeclaration::TextEmphasisPosition(..)
                        => Some(AnimatableLonghand::TextEmphasisPosition),
                    PropertyDeclaration::TextEmphasisColor(..)
                        => Some(AnimatableLonghand::TextEmphasisColor),
                    PropertyDeclaration::MozTabSize(..)
                        => Some(AnimatableLonghand::MozTabSize),
                    PropertyDeclaration::WebkitTextFillColor(..)
                        => Some(AnimatableLonghand::WebkitTextFillColor),
                    PropertyDeclaration::WebkitTextStrokeColor(..)
                        => Some(AnimatableLonghand::WebkitTextStrokeColor),
                    PropertyDeclaration::WebkitTextStrokeWidth(..)
                        => Some(AnimatableLonghand::WebkitTextStrokeWidth),
                    PropertyDeclaration::RubyAlign(..)
                        => Some(AnimatableLonghand::RubyAlign),
                    PropertyDeclaration::RubyPosition(..)
                        => Some(AnimatableLonghand::RubyPosition),
                    PropertyDeclaration::TextCombineUpright(..)
                        => Some(AnimatableLonghand::TextCombineUpright),
                    PropertyDeclaration::TextRendering(..)
                        => Some(AnimatableLonghand::TextRendering),
                    PropertyDeclaration::ListStylePosition(..)
                        => Some(AnimatableLonghand::ListStylePosition),
                    PropertyDeclaration::ListStyleType(..)
                        => Some(AnimatableLonghand::ListStyleType),
                    PropertyDeclaration::ListStyleImage(..)
                        => Some(AnimatableLonghand::ListStyleImage),
                    PropertyDeclaration::Quotes(..)
                        => Some(AnimatableLonghand::Quotes),
                    PropertyDeclaration::MozImageRegion(..)
                        => Some(AnimatableLonghand::MozImageRegion),
                    PropertyDeclaration::MarginTop(..)
                        => Some(AnimatableLonghand::MarginTop),
                    PropertyDeclaration::MarginLeft(..)
                        => Some(AnimatableLonghand::MarginLeft),
                    PropertyDeclaration::MarginBottom(..)
                        => Some(AnimatableLonghand::MarginBottom),
                    PropertyDeclaration::MarginRight(..)
                        => Some(AnimatableLonghand::MarginRight),
                    PropertyDeclaration::OutlineColor(..)
                        => Some(AnimatableLonghand::OutlineColor),
                    PropertyDeclaration::OutlineStyle(..)
                        => Some(AnimatableLonghand::OutlineStyle),
                    PropertyDeclaration::OutlineWidth(..)
                        => Some(AnimatableLonghand::OutlineWidth),
                    PropertyDeclaration::MozOutlineRadiusTopleft(..)
                        => Some(AnimatableLonghand::MozOutlineRadiusTopleft),
                    PropertyDeclaration::MozOutlineRadiusTopright(..)
                        => Some(AnimatableLonghand::MozOutlineRadiusTopright),
                    PropertyDeclaration::MozOutlineRadiusBottomright(..)
                        => Some(AnimatableLonghand::MozOutlineRadiusBottomright),
                    PropertyDeclaration::MozOutlineRadiusBottomleft(..)
                        => Some(AnimatableLonghand::MozOutlineRadiusBottomleft),
                    PropertyDeclaration::OutlineOffset(..)
                        => Some(AnimatableLonghand::OutlineOffset),
                    PropertyDeclaration::PaddingTop(..)
                        => Some(AnimatableLonghand::PaddingTop),
                    PropertyDeclaration::PaddingLeft(..)
                        => Some(AnimatableLonghand::PaddingLeft),
                    PropertyDeclaration::PaddingBottom(..)
                        => Some(AnimatableLonghand::PaddingBottom),
                    PropertyDeclaration::PaddingRight(..)
                        => Some(AnimatableLonghand::PaddingRight),
                    PropertyDeclaration::Cursor(..)
                        => Some(AnimatableLonghand::Cursor),
                    PropertyDeclaration::PointerEvents(..)
                        => Some(AnimatableLonghand::PointerEvents),
                    PropertyDeclaration::MozUserInput(..)
                        => Some(AnimatableLonghand::MozUserInput),
                    PropertyDeclaration::MozUserModify(..)
                        => Some(AnimatableLonghand::MozUserModify),
                    PropertyDeclaration::MozUserFocus(..)
                        => Some(AnimatableLonghand::MozUserFocus),
                    PropertyDeclaration::CaretColor(..)
                        => Some(AnimatableLonghand::CaretColor),
                    PropertyDeclaration::Top(..)
                        => Some(AnimatableLonghand::Top),
                    PropertyDeclaration::Left(..)
                        => Some(AnimatableLonghand::Left),
                    PropertyDeclaration::Bottom(..)
                        => Some(AnimatableLonghand::Bottom),
                    PropertyDeclaration::Right(..)
                        => Some(AnimatableLonghand::Right),
                    PropertyDeclaration::ZIndex(..)
                        => Some(AnimatableLonghand::ZIndex),
                    PropertyDeclaration::FlexDirection(..)
                        => Some(AnimatableLonghand::FlexDirection),
                    PropertyDeclaration::FlexWrap(..)
                        => Some(AnimatableLonghand::FlexWrap),
                    PropertyDeclaration::JustifyContent(..)
                        => Some(AnimatableLonghand::JustifyContent),
                    PropertyDeclaration::AlignContent(..)
                        => Some(AnimatableLonghand::AlignContent),
                    PropertyDeclaration::AlignItems(..)
                        => Some(AnimatableLonghand::AlignItems),
                    PropertyDeclaration::JustifyItems(..)
                        => Some(AnimatableLonghand::JustifyItems),
                    PropertyDeclaration::FlexGrow(..)
                        => Some(AnimatableLonghand::FlexGrow),
                    PropertyDeclaration::FlexShrink(..)
                        => Some(AnimatableLonghand::FlexShrink),
                    PropertyDeclaration::AlignSelf(..)
                        => Some(AnimatableLonghand::AlignSelf),
                    PropertyDeclaration::JustifySelf(..)
                        => Some(AnimatableLonghand::JustifySelf),
                    PropertyDeclaration::Order(..)
                        => Some(AnimatableLonghand::Order),
                    PropertyDeclaration::FlexBasis(..)
                        => Some(AnimatableLonghand::FlexBasis),
                    PropertyDeclaration::Width(..)
                        => Some(AnimatableLonghand::Width),
                    PropertyDeclaration::MinWidth(..)
                        => Some(AnimatableLonghand::MinWidth),
                    PropertyDeclaration::MaxWidth(..)
                        => Some(AnimatableLonghand::MaxWidth),
                    PropertyDeclaration::Height(..)
                        => Some(AnimatableLonghand::Height),
                    PropertyDeclaration::MinHeight(..)
                        => Some(AnimatableLonghand::MinHeight),
                    PropertyDeclaration::MaxHeight(..)
                        => Some(AnimatableLonghand::MaxHeight),
                    PropertyDeclaration::BoxSizing(..)
                        => Some(AnimatableLonghand::BoxSizing),
                    PropertyDeclaration::ObjectFit(..)
                        => Some(AnimatableLonghand::ObjectFit),
                    PropertyDeclaration::ObjectPosition(..)
                        => Some(AnimatableLonghand::ObjectPosition),
                    PropertyDeclaration::GridRowGap(..)
                        => Some(AnimatableLonghand::GridRowGap),
                    PropertyDeclaration::GridRowStart(..)
                        => Some(AnimatableLonghand::GridRowStart),
                    PropertyDeclaration::GridRowEnd(..)
                        => Some(AnimatableLonghand::GridRowEnd),
                    PropertyDeclaration::GridAutoRows(..)
                        => Some(AnimatableLonghand::GridAutoRows),
                    PropertyDeclaration::GridTemplateRows(..)
                        => Some(AnimatableLonghand::GridTemplateRows),
                    PropertyDeclaration::GridColumnGap(..)
                        => Some(AnimatableLonghand::GridColumnGap),
                    PropertyDeclaration::GridColumnStart(..)
                        => Some(AnimatableLonghand::GridColumnStart),
                    PropertyDeclaration::GridColumnEnd(..)
                        => Some(AnimatableLonghand::GridColumnEnd),
                    PropertyDeclaration::GridAutoColumns(..)
                        => Some(AnimatableLonghand::GridAutoColumns),
                    PropertyDeclaration::GridTemplateColumns(..)
                        => Some(AnimatableLonghand::GridTemplateColumns),
                    PropertyDeclaration::GridAutoFlow(..)
                        => Some(AnimatableLonghand::GridAutoFlow),
                    PropertyDeclaration::GridTemplateAreas(..)
                        => Some(AnimatableLonghand::GridTemplateAreas),
                    PropertyDeclaration::TableLayout(..)
                        => Some(AnimatableLonghand::TableLayout),
                    PropertyDeclaration::TextOverflow(..)
                        => Some(AnimatableLonghand::TextOverflow),
                    PropertyDeclaration::UnicodeBidi(..)
                        => Some(AnimatableLonghand::UnicodeBidi),
                    PropertyDeclaration::TextDecorationLine(..)
                        => Some(AnimatableLonghand::TextDecorationLine),
                    PropertyDeclaration::TextDecorationStyle(..)
                        => Some(AnimatableLonghand::TextDecorationStyle),
                    PropertyDeclaration::TextDecorationColor(..)
                        => Some(AnimatableLonghand::TextDecorationColor),
                    PropertyDeclaration::InitialLetter(..)
                        => Some(AnimatableLonghand::InitialLetter),
                    PropertyDeclaration::ImeMode(..)
                        => Some(AnimatableLonghand::ImeMode),
                    PropertyDeclaration::MozUserSelect(..)
                        => Some(AnimatableLonghand::MozUserSelect),
                    PropertyDeclaration::MozWindowDragging(..)
                        => Some(AnimatableLonghand::MozWindowDragging),
                    PropertyDeclaration::MozWindowShadow(..)
                        => Some(AnimatableLonghand::MozWindowShadow),
                    PropertyDeclaration::MozForceBrokenImageIcon(..)
                        => Some(AnimatableLonghand::MozForceBrokenImageIcon),
                    PropertyDeclaration::TextAnchor(..)
                        => Some(AnimatableLonghand::TextAnchor),
                    PropertyDeclaration::ColorInterpolation(..)
                        => Some(AnimatableLonghand::ColorInterpolation),
                    PropertyDeclaration::ColorInterpolationFilters(..)
                        => Some(AnimatableLonghand::ColorInterpolationFilters),
                    PropertyDeclaration::Fill(..)
                        => Some(AnimatableLonghand::Fill),
                    PropertyDeclaration::FillOpacity(..)
                        => Some(AnimatableLonghand::FillOpacity),
                    PropertyDeclaration::FillRule(..)
                        => Some(AnimatableLonghand::FillRule),
                    PropertyDeclaration::ShapeRendering(..)
                        => Some(AnimatableLonghand::ShapeRendering),
                    PropertyDeclaration::Stroke(..)
                        => Some(AnimatableLonghand::Stroke),
                    PropertyDeclaration::StrokeWidth(..)
                        => Some(AnimatableLonghand::StrokeWidth),
                    PropertyDeclaration::StrokeLinecap(..)
                        => Some(AnimatableLonghand::StrokeLinecap),
                    PropertyDeclaration::StrokeLinejoin(..)
                        => Some(AnimatableLonghand::StrokeLinejoin),
                    PropertyDeclaration::StrokeMiterlimit(..)
                        => Some(AnimatableLonghand::StrokeMiterlimit),
                    PropertyDeclaration::StrokeOpacity(..)
                        => Some(AnimatableLonghand::StrokeOpacity),
                    PropertyDeclaration::StrokeDasharray(..)
                        => Some(AnimatableLonghand::StrokeDasharray),
                    PropertyDeclaration::StrokeDashoffset(..)
                        => Some(AnimatableLonghand::StrokeDashoffset),
                    PropertyDeclaration::ClipRule(..)
                        => Some(AnimatableLonghand::ClipRule),
                    PropertyDeclaration::MarkerStart(..)
                        => Some(AnimatableLonghand::MarkerStart),
                    PropertyDeclaration::MarkerMid(..)
                        => Some(AnimatableLonghand::MarkerMid),
                    PropertyDeclaration::MarkerEnd(..)
                        => Some(AnimatableLonghand::MarkerEnd),
                    PropertyDeclaration::PaintOrder(..)
                        => Some(AnimatableLonghand::PaintOrder),
                    PropertyDeclaration::DominantBaseline(..)
                        => Some(AnimatableLonghand::DominantBaseline),
                    PropertyDeclaration::VectorEffect(..)
                        => Some(AnimatableLonghand::VectorEffect),
                    PropertyDeclaration::StopColor(..)
                        => Some(AnimatableLonghand::StopColor),
                    PropertyDeclaration::StopOpacity(..)
                        => Some(AnimatableLonghand::StopOpacity),
                    PropertyDeclaration::FloodColor(..)
                        => Some(AnimatableLonghand::FloodColor),
                    PropertyDeclaration::FloodOpacity(..)
                        => Some(AnimatableLonghand::FloodOpacity),
                    PropertyDeclaration::LightingColor(..)
                        => Some(AnimatableLonghand::LightingColor),
                    PropertyDeclaration::MaskType(..)
                        => Some(AnimatableLonghand::MaskType),
                    PropertyDeclaration::ClipPath(..)
                        => Some(AnimatableLonghand::ClipPath),
                    PropertyDeclaration::MaskMode(..)
                        => Some(AnimatableLonghand::MaskMode),
                    PropertyDeclaration::MaskRepeat(..)
                        => Some(AnimatableLonghand::MaskRepeat),
                    PropertyDeclaration::MaskPositionX(..)
                        => Some(AnimatableLonghand::MaskPositionX),
                    PropertyDeclaration::MaskPositionY(..)
                        => Some(AnimatableLonghand::MaskPositionY),
                    PropertyDeclaration::MaskClip(..)
                        => Some(AnimatableLonghand::MaskClip),
                    PropertyDeclaration::MaskOrigin(..)
                        => Some(AnimatableLonghand::MaskOrigin),
                    PropertyDeclaration::MaskSize(..)
                        => Some(AnimatableLonghand::MaskSize),
                    PropertyDeclaration::MaskComposite(..)
                        => Some(AnimatableLonghand::MaskComposite),
                    PropertyDeclaration::MaskImage(..)
                        => Some(AnimatableLonghand::MaskImage),
                    PropertyDeclaration::MozBoxAlign(..)
                        => Some(AnimatableLonghand::MozBoxAlign),
                    PropertyDeclaration::MozBoxDirection(..)
                        => Some(AnimatableLonghand::MozBoxDirection),
                    PropertyDeclaration::MozBoxFlex(..)
                        => Some(AnimatableLonghand::MozBoxFlex),
                    PropertyDeclaration::MozBoxOrient(..)
                        => Some(AnimatableLonghand::MozBoxOrient),
                    PropertyDeclaration::MozBoxPack(..)
                        => Some(AnimatableLonghand::MozBoxPack),
                    PropertyDeclaration::MozStackSizing(..)
                        => Some(AnimatableLonghand::MozStackSizing),
                    PropertyDeclaration::MozBoxOrdinalGroup(..)
                        => Some(AnimatableLonghand::MozBoxOrdinalGroup),
            PropertyDeclaration::CSSWideKeyword(id, _) |
            PropertyDeclaration::WithVariables(id, _) => {
                match id {
                            LonghandId::BackgroundColor =>
                                Some(AnimatableLonghand::BackgroundColor),
                            LonghandId::BackgroundImage =>
                                Some(AnimatableLonghand::BackgroundImage),
                            LonghandId::BackgroundPositionX =>
                                Some(AnimatableLonghand::BackgroundPositionX),
                            LonghandId::BackgroundPositionY =>
                                Some(AnimatableLonghand::BackgroundPositionY),
                            LonghandId::BackgroundRepeat =>
                                Some(AnimatableLonghand::BackgroundRepeat),
                            LonghandId::BackgroundAttachment =>
                                Some(AnimatableLonghand::BackgroundAttachment),
                            LonghandId::BackgroundClip =>
                                Some(AnimatableLonghand::BackgroundClip),
                            LonghandId::BackgroundOrigin =>
                                Some(AnimatableLonghand::BackgroundOrigin),
                            LonghandId::BackgroundSize =>
                                Some(AnimatableLonghand::BackgroundSize),
                            LonghandId::BackgroundBlendMode =>
                                Some(AnimatableLonghand::BackgroundBlendMode),
                            LonghandId::BorderTopColor =>
                                Some(AnimatableLonghand::BorderTopColor),
                            LonghandId::BorderTopStyle =>
                                Some(AnimatableLonghand::BorderTopStyle),
                            LonghandId::BorderTopWidth =>
                                Some(AnimatableLonghand::BorderTopWidth),
                            LonghandId::BorderLeftColor =>
                                Some(AnimatableLonghand::BorderLeftColor),
                            LonghandId::BorderLeftStyle =>
                                Some(AnimatableLonghand::BorderLeftStyle),
                            LonghandId::BorderLeftWidth =>
                                Some(AnimatableLonghand::BorderLeftWidth),
                            LonghandId::BorderBottomColor =>
                                Some(AnimatableLonghand::BorderBottomColor),
                            LonghandId::BorderBottomStyle =>
                                Some(AnimatableLonghand::BorderBottomStyle),
                            LonghandId::BorderBottomWidth =>
                                Some(AnimatableLonghand::BorderBottomWidth),
                            LonghandId::BorderRightColor =>
                                Some(AnimatableLonghand::BorderRightColor),
                            LonghandId::BorderRightStyle =>
                                Some(AnimatableLonghand::BorderRightStyle),
                            LonghandId::BorderRightWidth =>
                                Some(AnimatableLonghand::BorderRightWidth),
                            LonghandId::BorderTopLeftRadius =>
                                Some(AnimatableLonghand::BorderTopLeftRadius),
                            LonghandId::BorderTopRightRadius =>
                                Some(AnimatableLonghand::BorderTopRightRadius),
                            LonghandId::BorderBottomRightRadius =>
                                Some(AnimatableLonghand::BorderBottomRightRadius),
                            LonghandId::BorderBottomLeftRadius =>
                                Some(AnimatableLonghand::BorderBottomLeftRadius),
                            LonghandId::MozBorderTopColors =>
                                Some(AnimatableLonghand::MozBorderTopColors),
                            LonghandId::MozBorderLeftColors =>
                                Some(AnimatableLonghand::MozBorderLeftColors),
                            LonghandId::MozBorderBottomColors =>
                                Some(AnimatableLonghand::MozBorderBottomColors),
                            LonghandId::MozBorderRightColors =>
                                Some(AnimatableLonghand::MozBorderRightColors),
                            LonghandId::BoxDecorationBreak =>
                                Some(AnimatableLonghand::BoxDecorationBreak),
                            LonghandId::MozFloatEdge =>
                                Some(AnimatableLonghand::MozFloatEdge),
                            LonghandId::BorderImageSource =>
                                Some(AnimatableLonghand::BorderImageSource),
                            LonghandId::BorderImageOutset =>
                                Some(AnimatableLonghand::BorderImageOutset),
                            LonghandId::BorderImageRepeat =>
                                Some(AnimatableLonghand::BorderImageRepeat),
                            LonghandId::BorderImageWidth =>
                                Some(AnimatableLonghand::BorderImageWidth),
                            LonghandId::BorderImageSlice =>
                                Some(AnimatableLonghand::BorderImageSlice),
                            LonghandId::Display =>
                                Some(AnimatableLonghand::Display),
                            LonghandId::Position =>
                                Some(AnimatableLonghand::Position),
                            LonghandId::Float =>
                                Some(AnimatableLonghand::Float),
                            LonghandId::Clear =>
                                Some(AnimatableLonghand::Clear),
                            LonghandId::VerticalAlign =>
                                Some(AnimatableLonghand::VerticalAlign),
                            LonghandId::OverflowClipBox =>
                                Some(AnimatableLonghand::OverflowClipBox),
                            LonghandId::OverflowX =>
                                Some(AnimatableLonghand::OverflowX),
                            LonghandId::OverflowY =>
                                Some(AnimatableLonghand::OverflowY),
                            LonghandId::ScrollSnapPointsX =>
                                Some(AnimatableLonghand::ScrollSnapPointsX),
                            LonghandId::ScrollSnapPointsY =>
                                Some(AnimatableLonghand::ScrollSnapPointsY),
                            LonghandId::ScrollSnapDestination =>
                                Some(AnimatableLonghand::ScrollSnapDestination),
                            LonghandId::ScrollSnapCoordinate =>
                                Some(AnimatableLonghand::ScrollSnapCoordinate),
                            LonghandId::Transform =>
                                Some(AnimatableLonghand::Transform),
                            LonghandId::ScrollBehavior =>
                                Some(AnimatableLonghand::ScrollBehavior),
                            LonghandId::ScrollSnapTypeX =>
                                Some(AnimatableLonghand::ScrollSnapTypeX),
                            LonghandId::ScrollSnapTypeY =>
                                Some(AnimatableLonghand::ScrollSnapTypeY),
                            LonghandId::Isolation =>
                                Some(AnimatableLonghand::Isolation),
                            LonghandId::PageBreakAfter =>
                                Some(AnimatableLonghand::PageBreakAfter),
                            LonghandId::PageBreakBefore =>
                                Some(AnimatableLonghand::PageBreakBefore),
                            LonghandId::PageBreakInside =>
                                Some(AnimatableLonghand::PageBreakInside),
                            LonghandId::Resize =>
                                Some(AnimatableLonghand::Resize),
                            LonghandId::Perspective =>
                                Some(AnimatableLonghand::Perspective),
                            LonghandId::PerspectiveOrigin =>
                                Some(AnimatableLonghand::PerspectiveOrigin),
                            LonghandId::BackfaceVisibility =>
                                Some(AnimatableLonghand::BackfaceVisibility),
                            LonghandId::TransformBox =>
                                Some(AnimatableLonghand::TransformBox),
                            LonghandId::TransformStyle =>
                                Some(AnimatableLonghand::TransformStyle),
                            LonghandId::TransformOrigin =>
                                Some(AnimatableLonghand::TransformOrigin),
                            LonghandId::Contain =>
                                Some(AnimatableLonghand::Contain),
                            LonghandId::MozAppearance =>
                                Some(AnimatableLonghand::MozAppearance),
                            LonghandId::MozOrient =>
                                Some(AnimatableLonghand::MozOrient),
                            LonghandId::WillChange =>
                                Some(AnimatableLonghand::WillChange),
                            LonghandId::ShapeOutside =>
                                Some(AnimatableLonghand::ShapeOutside),
                            LonghandId::TouchAction =>
                                Some(AnimatableLonghand::TouchAction),
                            LonghandId::Color =>
                                Some(AnimatableLonghand::Color),
                            LonghandId::ColumnWidth =>
                                Some(AnimatableLonghand::ColumnWidth),
                            LonghandId::ColumnCount =>
                                Some(AnimatableLonghand::ColumnCount),
                            LonghandId::ColumnGap =>
                                Some(AnimatableLonghand::ColumnGap),
                            LonghandId::ColumnFill =>
                                Some(AnimatableLonghand::ColumnFill),
                            LonghandId::ColumnRuleWidth =>
                                Some(AnimatableLonghand::ColumnRuleWidth),
                            LonghandId::ColumnRuleColor =>
                                Some(AnimatableLonghand::ColumnRuleColor),
                            LonghandId::ColumnSpan =>
                                Some(AnimatableLonghand::ColumnSpan),
                            LonghandId::ColumnRuleStyle =>
                                Some(AnimatableLonghand::ColumnRuleStyle),
                            LonghandId::Content =>
                                Some(AnimatableLonghand::Content),
                            LonghandId::CounterIncrement =>
                                Some(AnimatableLonghand::CounterIncrement),
                            LonghandId::CounterReset =>
                                Some(AnimatableLonghand::CounterReset),
                            LonghandId::Opacity =>
                                Some(AnimatableLonghand::Opacity),
                            LonghandId::BoxShadow =>
                                Some(AnimatableLonghand::BoxShadow),
                            LonghandId::Clip =>
                                Some(AnimatableLonghand::Clip),
                            LonghandId::Filter =>
                                Some(AnimatableLonghand::Filter),
                            LonghandId::MixBlendMode =>
                                Some(AnimatableLonghand::MixBlendMode),
                            LonghandId::FontFamily =>
                                Some(AnimatableLonghand::FontFamily),
                            LonghandId::FontStyle =>
                                Some(AnimatableLonghand::FontStyle),
                            LonghandId::FontVariantCaps =>
                                Some(AnimatableLonghand::FontVariantCaps),
                            LonghandId::FontWeight =>
                                Some(AnimatableLonghand::FontWeight),
                            LonghandId::FontSize =>
                                Some(AnimatableLonghand::FontSize),
                            LonghandId::FontSizeAdjust =>
                                Some(AnimatableLonghand::FontSizeAdjust),
                            LonghandId::FontSynthesis =>
                                Some(AnimatableLonghand::FontSynthesis),
                            LonghandId::FontStretch =>
                                Some(AnimatableLonghand::FontStretch),
                            LonghandId::FontKerning =>
                                Some(AnimatableLonghand::FontKerning),
                            LonghandId::FontVariantAlternates =>
                                Some(AnimatableLonghand::FontVariantAlternates),
                            LonghandId::FontVariantEastAsian =>
                                Some(AnimatableLonghand::FontVariantEastAsian),
                            LonghandId::FontVariantLigatures =>
                                Some(AnimatableLonghand::FontVariantLigatures),
                            LonghandId::FontVariantNumeric =>
                                Some(AnimatableLonghand::FontVariantNumeric),
                            LonghandId::FontVariantPosition =>
                                Some(AnimatableLonghand::FontVariantPosition),
                            LonghandId::FontFeatureSettings =>
                                Some(AnimatableLonghand::FontFeatureSettings),
                            LonghandId::FontVariationSettings =>
                                Some(AnimatableLonghand::FontVariationSettings),
                            LonghandId::FontLanguageOverride =>
                                Some(AnimatableLonghand::FontLanguageOverride),
                            LonghandId::MozOsxFontSmoothing =>
                                Some(AnimatableLonghand::MozOsxFontSmoothing),
                            LonghandId::Visibility =>
                                Some(AnimatableLonghand::Visibility),
                            LonghandId::WritingMode =>
                                Some(AnimatableLonghand::WritingMode),
                            LonghandId::Direction =>
                                Some(AnimatableLonghand::Direction),
                            LonghandId::TextOrientation =>
                                Some(AnimatableLonghand::TextOrientation),
                            LonghandId::ColorAdjust =>
                                Some(AnimatableLonghand::ColorAdjust),
                            LonghandId::ImageRendering =>
                                Some(AnimatableLonghand::ImageRendering),
                            LonghandId::ImageOrientation =>
                                Some(AnimatableLonghand::ImageOrientation),
                            LonghandId::BorderCollapse =>
                                Some(AnimatableLonghand::BorderCollapse),
                            LonghandId::EmptyCells =>
                                Some(AnimatableLonghand::EmptyCells),
                            LonghandId::CaptionSide =>
                                Some(AnimatableLonghand::CaptionSide),
                            LonghandId::BorderSpacing =>
                                Some(AnimatableLonghand::BorderSpacing),
                            LonghandId::LineHeight =>
                                Some(AnimatableLonghand::LineHeight),
                            LonghandId::TextTransform =>
                                Some(AnimatableLonghand::TextTransform),
                            LonghandId::Hyphens =>
                                Some(AnimatableLonghand::Hyphens),
                            LonghandId::MozTextSizeAdjust =>
                                Some(AnimatableLonghand::MozTextSizeAdjust),
                            LonghandId::TextIndent =>
                                Some(AnimatableLonghand::TextIndent),
                            LonghandId::OverflowWrap =>
                                Some(AnimatableLonghand::OverflowWrap),
                            LonghandId::WordBreak =>
                                Some(AnimatableLonghand::WordBreak),
                            LonghandId::TextJustify =>
                                Some(AnimatableLonghand::TextJustify),
                            LonghandId::TextAlignLast =>
                                Some(AnimatableLonghand::TextAlignLast),
                            LonghandId::TextAlign =>
                                Some(AnimatableLonghand::TextAlign),
                            LonghandId::LetterSpacing =>
                                Some(AnimatableLonghand::LetterSpacing),
                            LonghandId::WordSpacing =>
                                Some(AnimatableLonghand::WordSpacing),
                            LonghandId::WhiteSpace =>
                                Some(AnimatableLonghand::WhiteSpace),
                            LonghandId::TextShadow =>
                                Some(AnimatableLonghand::TextShadow),
                            LonghandId::TextEmphasisStyle =>
                                Some(AnimatableLonghand::TextEmphasisStyle),
                            LonghandId::TextEmphasisPosition =>
                                Some(AnimatableLonghand::TextEmphasisPosition),
                            LonghandId::TextEmphasisColor =>
                                Some(AnimatableLonghand::TextEmphasisColor),
                            LonghandId::MozTabSize =>
                                Some(AnimatableLonghand::MozTabSize),
                            LonghandId::WebkitTextFillColor =>
                                Some(AnimatableLonghand::WebkitTextFillColor),
                            LonghandId::WebkitTextStrokeColor =>
                                Some(AnimatableLonghand::WebkitTextStrokeColor),
                            LonghandId::WebkitTextStrokeWidth =>
                                Some(AnimatableLonghand::WebkitTextStrokeWidth),
                            LonghandId::RubyAlign =>
                                Some(AnimatableLonghand::RubyAlign),
                            LonghandId::RubyPosition =>
                                Some(AnimatableLonghand::RubyPosition),
                            LonghandId::TextCombineUpright =>
                                Some(AnimatableLonghand::TextCombineUpright),
                            LonghandId::TextRendering =>
                                Some(AnimatableLonghand::TextRendering),
                            LonghandId::ListStylePosition =>
                                Some(AnimatableLonghand::ListStylePosition),
                            LonghandId::ListStyleType =>
                                Some(AnimatableLonghand::ListStyleType),
                            LonghandId::ListStyleImage =>
                                Some(AnimatableLonghand::ListStyleImage),
                            LonghandId::Quotes =>
                                Some(AnimatableLonghand::Quotes),
                            LonghandId::MozImageRegion =>
                                Some(AnimatableLonghand::MozImageRegion),
                            LonghandId::MarginTop =>
                                Some(AnimatableLonghand::MarginTop),
                            LonghandId::MarginLeft =>
                                Some(AnimatableLonghand::MarginLeft),
                            LonghandId::MarginBottom =>
                                Some(AnimatableLonghand::MarginBottom),
                            LonghandId::MarginRight =>
                                Some(AnimatableLonghand::MarginRight),
                            LonghandId::OutlineColor =>
                                Some(AnimatableLonghand::OutlineColor),
                            LonghandId::OutlineStyle =>
                                Some(AnimatableLonghand::OutlineStyle),
                            LonghandId::OutlineWidth =>
                                Some(AnimatableLonghand::OutlineWidth),
                            LonghandId::MozOutlineRadiusTopleft =>
                                Some(AnimatableLonghand::MozOutlineRadiusTopleft),
                            LonghandId::MozOutlineRadiusTopright =>
                                Some(AnimatableLonghand::MozOutlineRadiusTopright),
                            LonghandId::MozOutlineRadiusBottomright =>
                                Some(AnimatableLonghand::MozOutlineRadiusBottomright),
                            LonghandId::MozOutlineRadiusBottomleft =>
                                Some(AnimatableLonghand::MozOutlineRadiusBottomleft),
                            LonghandId::OutlineOffset =>
                                Some(AnimatableLonghand::OutlineOffset),
                            LonghandId::PaddingTop =>
                                Some(AnimatableLonghand::PaddingTop),
                            LonghandId::PaddingLeft =>
                                Some(AnimatableLonghand::PaddingLeft),
                            LonghandId::PaddingBottom =>
                                Some(AnimatableLonghand::PaddingBottom),
                            LonghandId::PaddingRight =>
                                Some(AnimatableLonghand::PaddingRight),
                            LonghandId::Cursor =>
                                Some(AnimatableLonghand::Cursor),
                            LonghandId::PointerEvents =>
                                Some(AnimatableLonghand::PointerEvents),
                            LonghandId::MozUserInput =>
                                Some(AnimatableLonghand::MozUserInput),
                            LonghandId::MozUserModify =>
                                Some(AnimatableLonghand::MozUserModify),
                            LonghandId::MozUserFocus =>
                                Some(AnimatableLonghand::MozUserFocus),
                            LonghandId::CaretColor =>
                                Some(AnimatableLonghand::CaretColor),
                            LonghandId::Top =>
                                Some(AnimatableLonghand::Top),
                            LonghandId::Left =>
                                Some(AnimatableLonghand::Left),
                            LonghandId::Bottom =>
                                Some(AnimatableLonghand::Bottom),
                            LonghandId::Right =>
                                Some(AnimatableLonghand::Right),
                            LonghandId::ZIndex =>
                                Some(AnimatableLonghand::ZIndex),
                            LonghandId::FlexDirection =>
                                Some(AnimatableLonghand::FlexDirection),
                            LonghandId::FlexWrap =>
                                Some(AnimatableLonghand::FlexWrap),
                            LonghandId::JustifyContent =>
                                Some(AnimatableLonghand::JustifyContent),
                            LonghandId::AlignContent =>
                                Some(AnimatableLonghand::AlignContent),
                            LonghandId::AlignItems =>
                                Some(AnimatableLonghand::AlignItems),
                            LonghandId::JustifyItems =>
                                Some(AnimatableLonghand::JustifyItems),
                            LonghandId::FlexGrow =>
                                Some(AnimatableLonghand::FlexGrow),
                            LonghandId::FlexShrink =>
                                Some(AnimatableLonghand::FlexShrink),
                            LonghandId::AlignSelf =>
                                Some(AnimatableLonghand::AlignSelf),
                            LonghandId::JustifySelf =>
                                Some(AnimatableLonghand::JustifySelf),
                            LonghandId::Order =>
                                Some(AnimatableLonghand::Order),
                            LonghandId::FlexBasis =>
                                Some(AnimatableLonghand::FlexBasis),
                            LonghandId::Width =>
                                Some(AnimatableLonghand::Width),
                            LonghandId::MinWidth =>
                                Some(AnimatableLonghand::MinWidth),
                            LonghandId::MaxWidth =>
                                Some(AnimatableLonghand::MaxWidth),
                            LonghandId::Height =>
                                Some(AnimatableLonghand::Height),
                            LonghandId::MinHeight =>
                                Some(AnimatableLonghand::MinHeight),
                            LonghandId::MaxHeight =>
                                Some(AnimatableLonghand::MaxHeight),
                            LonghandId::BoxSizing =>
                                Some(AnimatableLonghand::BoxSizing),
                            LonghandId::ObjectFit =>
                                Some(AnimatableLonghand::ObjectFit),
                            LonghandId::ObjectPosition =>
                                Some(AnimatableLonghand::ObjectPosition),
                            LonghandId::GridRowGap =>
                                Some(AnimatableLonghand::GridRowGap),
                            LonghandId::GridRowStart =>
                                Some(AnimatableLonghand::GridRowStart),
                            LonghandId::GridRowEnd =>
                                Some(AnimatableLonghand::GridRowEnd),
                            LonghandId::GridAutoRows =>
                                Some(AnimatableLonghand::GridAutoRows),
                            LonghandId::GridTemplateRows =>
                                Some(AnimatableLonghand::GridTemplateRows),
                            LonghandId::GridColumnGap =>
                                Some(AnimatableLonghand::GridColumnGap),
                            LonghandId::GridColumnStart =>
                                Some(AnimatableLonghand::GridColumnStart),
                            LonghandId::GridColumnEnd =>
                                Some(AnimatableLonghand::GridColumnEnd),
                            LonghandId::GridAutoColumns =>
                                Some(AnimatableLonghand::GridAutoColumns),
                            LonghandId::GridTemplateColumns =>
                                Some(AnimatableLonghand::GridTemplateColumns),
                            LonghandId::GridAutoFlow =>
                                Some(AnimatableLonghand::GridAutoFlow),
                            LonghandId::GridTemplateAreas =>
                                Some(AnimatableLonghand::GridTemplateAreas),
                            LonghandId::TableLayout =>
                                Some(AnimatableLonghand::TableLayout),
                            LonghandId::TextOverflow =>
                                Some(AnimatableLonghand::TextOverflow),
                            LonghandId::UnicodeBidi =>
                                Some(AnimatableLonghand::UnicodeBidi),
                            LonghandId::TextDecorationLine =>
                                Some(AnimatableLonghand::TextDecorationLine),
                            LonghandId::TextDecorationStyle =>
                                Some(AnimatableLonghand::TextDecorationStyle),
                            LonghandId::TextDecorationColor =>
                                Some(AnimatableLonghand::TextDecorationColor),
                            LonghandId::InitialLetter =>
                                Some(AnimatableLonghand::InitialLetter),
                            LonghandId::ImeMode =>
                                Some(AnimatableLonghand::ImeMode),
                            LonghandId::MozUserSelect =>
                                Some(AnimatableLonghand::MozUserSelect),
                            LonghandId::MozWindowDragging =>
                                Some(AnimatableLonghand::MozWindowDragging),
                            LonghandId::MozWindowShadow =>
                                Some(AnimatableLonghand::MozWindowShadow),
                            LonghandId::MozForceBrokenImageIcon =>
                                Some(AnimatableLonghand::MozForceBrokenImageIcon),
                            LonghandId::TextAnchor =>
                                Some(AnimatableLonghand::TextAnchor),
                            LonghandId::ColorInterpolation =>
                                Some(AnimatableLonghand::ColorInterpolation),
                            LonghandId::ColorInterpolationFilters =>
                                Some(AnimatableLonghand::ColorInterpolationFilters),
                            LonghandId::Fill =>
                                Some(AnimatableLonghand::Fill),
                            LonghandId::FillOpacity =>
                                Some(AnimatableLonghand::FillOpacity),
                            LonghandId::FillRule =>
                                Some(AnimatableLonghand::FillRule),
                            LonghandId::ShapeRendering =>
                                Some(AnimatableLonghand::ShapeRendering),
                            LonghandId::Stroke =>
                                Some(AnimatableLonghand::Stroke),
                            LonghandId::StrokeWidth =>
                                Some(AnimatableLonghand::StrokeWidth),
                            LonghandId::StrokeLinecap =>
                                Some(AnimatableLonghand::StrokeLinecap),
                            LonghandId::StrokeLinejoin =>
                                Some(AnimatableLonghand::StrokeLinejoin),
                            LonghandId::StrokeMiterlimit =>
                                Some(AnimatableLonghand::StrokeMiterlimit),
                            LonghandId::StrokeOpacity =>
                                Some(AnimatableLonghand::StrokeOpacity),
                            LonghandId::StrokeDasharray =>
                                Some(AnimatableLonghand::StrokeDasharray),
                            LonghandId::StrokeDashoffset =>
                                Some(AnimatableLonghand::StrokeDashoffset),
                            LonghandId::ClipRule =>
                                Some(AnimatableLonghand::ClipRule),
                            LonghandId::MarkerStart =>
                                Some(AnimatableLonghand::MarkerStart),
                            LonghandId::MarkerMid =>
                                Some(AnimatableLonghand::MarkerMid),
                            LonghandId::MarkerEnd =>
                                Some(AnimatableLonghand::MarkerEnd),
                            LonghandId::PaintOrder =>
                                Some(AnimatableLonghand::PaintOrder),
                            LonghandId::DominantBaseline =>
                                Some(AnimatableLonghand::DominantBaseline),
                            LonghandId::VectorEffect =>
                                Some(AnimatableLonghand::VectorEffect),
                            LonghandId::StopColor =>
                                Some(AnimatableLonghand::StopColor),
                            LonghandId::StopOpacity =>
                                Some(AnimatableLonghand::StopOpacity),
                            LonghandId::FloodColor =>
                                Some(AnimatableLonghand::FloodColor),
                            LonghandId::FloodOpacity =>
                                Some(AnimatableLonghand::FloodOpacity),
                            LonghandId::LightingColor =>
                                Some(AnimatableLonghand::LightingColor),
                            LonghandId::MaskType =>
                                Some(AnimatableLonghand::MaskType),
                            LonghandId::ClipPath =>
                                Some(AnimatableLonghand::ClipPath),
                            LonghandId::MaskMode =>
                                Some(AnimatableLonghand::MaskMode),
                            LonghandId::MaskRepeat =>
                                Some(AnimatableLonghand::MaskRepeat),
                            LonghandId::MaskPositionX =>
                                Some(AnimatableLonghand::MaskPositionX),
                            LonghandId::MaskPositionY =>
                                Some(AnimatableLonghand::MaskPositionY),
                            LonghandId::MaskClip =>
                                Some(AnimatableLonghand::MaskClip),
                            LonghandId::MaskOrigin =>
                                Some(AnimatableLonghand::MaskOrigin),
                            LonghandId::MaskSize =>
                                Some(AnimatableLonghand::MaskSize),
                            LonghandId::MaskComposite =>
                                Some(AnimatableLonghand::MaskComposite),
                            LonghandId::MaskImage =>
                                Some(AnimatableLonghand::MaskImage),
                            LonghandId::MozBoxAlign =>
                                Some(AnimatableLonghand::MozBoxAlign),
                            LonghandId::MozBoxDirection =>
                                Some(AnimatableLonghand::MozBoxDirection),
                            LonghandId::MozBoxFlex =>
                                Some(AnimatableLonghand::MozBoxFlex),
                            LonghandId::MozBoxOrient =>
                                Some(AnimatableLonghand::MozBoxOrient),
                            LonghandId::MozBoxPack =>
                                Some(AnimatableLonghand::MozBoxPack),
                            LonghandId::MozStackSizing =>
                                Some(AnimatableLonghand::MozStackSizing),
                            LonghandId::MozBoxOrdinalGroup =>
                                Some(AnimatableLonghand::MozBoxOrdinalGroup),
                    _ => None,
                }
            },
            _ => None,
        }
    }
}

/// Convert to nsCSSPropertyID.
#[cfg(feature = "gecko")]
#[allow(non_upper_case_globals)]
impl<'a> From< &'a AnimatableLonghand> for nsCSSPropertyID {
    fn from(property: &'a AnimatableLonghand) -> nsCSSPropertyID {
        match *property {
                    AnimatableLonghand::BackgroundColor
                        => 
    nsCSSPropertyID::eCSSProperty_background_color,
                    AnimatableLonghand::BackgroundImage
                        => 
    nsCSSPropertyID::eCSSProperty_background_image,
                    AnimatableLonghand::BackgroundPositionX
                        => 
    nsCSSPropertyID::eCSSProperty_background_position_x,
                    AnimatableLonghand::BackgroundPositionY
                        => 
    nsCSSPropertyID::eCSSProperty_background_position_y,
                    AnimatableLonghand::BackgroundRepeat
                        => 
    nsCSSPropertyID::eCSSProperty_background_repeat,
                    AnimatableLonghand::BackgroundAttachment
                        => 
    nsCSSPropertyID::eCSSProperty_background_attachment,
                    AnimatableLonghand::BackgroundClip
                        => 
    nsCSSPropertyID::eCSSProperty_background_clip,
                    AnimatableLonghand::BackgroundOrigin
                        => 
    nsCSSPropertyID::eCSSProperty_background_origin,
                    AnimatableLonghand::BackgroundSize
                        => 
    nsCSSPropertyID::eCSSProperty_background_size,
                    AnimatableLonghand::BackgroundBlendMode
                        => 
    nsCSSPropertyID::eCSSProperty_background_blend_mode,
                    AnimatableLonghand::BorderTopColor
                        => 
    nsCSSPropertyID::eCSSProperty_border_top_color,
                    AnimatableLonghand::BorderTopStyle
                        => 
    nsCSSPropertyID::eCSSProperty_border_top_style,
                    AnimatableLonghand::BorderTopWidth
                        => 
    nsCSSPropertyID::eCSSProperty_border_top_width,
                    AnimatableLonghand::BorderLeftColor
                        => 
    nsCSSPropertyID::eCSSProperty_border_left_color,
                    AnimatableLonghand::BorderLeftStyle
                        => 
    nsCSSPropertyID::eCSSProperty_border_left_style,
                    AnimatableLonghand::BorderLeftWidth
                        => 
    nsCSSPropertyID::eCSSProperty_border_left_width,
                    AnimatableLonghand::BorderBottomColor
                        => 
    nsCSSPropertyID::eCSSProperty_border_bottom_color,
                    AnimatableLonghand::BorderBottomStyle
                        => 
    nsCSSPropertyID::eCSSProperty_border_bottom_style,
                    AnimatableLonghand::BorderBottomWidth
                        => 
    nsCSSPropertyID::eCSSProperty_border_bottom_width,
                    AnimatableLonghand::BorderRightColor
                        => 
    nsCSSPropertyID::eCSSProperty_border_right_color,
                    AnimatableLonghand::BorderRightStyle
                        => 
    nsCSSPropertyID::eCSSProperty_border_right_style,
                    AnimatableLonghand::BorderRightWidth
                        => 
    nsCSSPropertyID::eCSSProperty_border_right_width,
                    AnimatableLonghand::BorderTopLeftRadius
                        => 
    nsCSSPropertyID::eCSSProperty_border_top_left_radius,
                    AnimatableLonghand::BorderTopRightRadius
                        => 
    nsCSSPropertyID::eCSSProperty_border_top_right_radius,
                    AnimatableLonghand::BorderBottomRightRadius
                        => 
    nsCSSPropertyID::eCSSProperty_border_bottom_right_radius,
                    AnimatableLonghand::BorderBottomLeftRadius
                        => 
    nsCSSPropertyID::eCSSProperty_border_bottom_left_radius,
                    AnimatableLonghand::MozBorderTopColors
                        => 
    nsCSSPropertyID::eCSSProperty__moz_border_top_colors,
                    AnimatableLonghand::MozBorderLeftColors
                        => 
    nsCSSPropertyID::eCSSProperty__moz_border_left_colors,
                    AnimatableLonghand::MozBorderBottomColors
                        => 
    nsCSSPropertyID::eCSSProperty__moz_border_bottom_colors,
                    AnimatableLonghand::MozBorderRightColors
                        => 
    nsCSSPropertyID::eCSSProperty__moz_border_right_colors,
                    AnimatableLonghand::BoxDecorationBreak
                        => 
    nsCSSPropertyID::eCSSProperty_box_decoration_break,
                    AnimatableLonghand::MozFloatEdge
                        => 
    nsCSSPropertyID::eCSSProperty__moz_float_edge,
                    AnimatableLonghand::BorderImageSource
                        => 
    nsCSSPropertyID::eCSSProperty_border_image_source,
                    AnimatableLonghand::BorderImageOutset
                        => 
    nsCSSPropertyID::eCSSProperty_border_image_outset,
                    AnimatableLonghand::BorderImageRepeat
                        => 
    nsCSSPropertyID::eCSSProperty_border_image_repeat,
                    AnimatableLonghand::BorderImageWidth
                        => 
    nsCSSPropertyID::eCSSProperty_border_image_width,
                    AnimatableLonghand::BorderImageSlice
                        => 
    nsCSSPropertyID::eCSSProperty_border_image_slice,
                    AnimatableLonghand::Display
                        => 
    nsCSSPropertyID::eCSSProperty_display,
                    AnimatableLonghand::Position
                        => 
    nsCSSPropertyID::eCSSProperty_position,
                    AnimatableLonghand::Float
                        => 
    nsCSSPropertyID::eCSSProperty_float_,
                    AnimatableLonghand::Clear
                        => 
    nsCSSPropertyID::eCSSProperty_clear,
                    AnimatableLonghand::VerticalAlign
                        => 
    nsCSSPropertyID::eCSSProperty_vertical_align,
                    AnimatableLonghand::OverflowClipBox
                        => 
    nsCSSPropertyID::eCSSProperty_overflow_clip_box,
                    AnimatableLonghand::OverflowX
                        => 
    nsCSSPropertyID::eCSSProperty_overflow_x,
                    AnimatableLonghand::OverflowY
                        => 
    nsCSSPropertyID::eCSSProperty_overflow_y,
                    AnimatableLonghand::ScrollSnapPointsX
                        => 
    nsCSSPropertyID::eCSSProperty_scroll_snap_points_x,
                    AnimatableLonghand::ScrollSnapPointsY
                        => 
    nsCSSPropertyID::eCSSProperty_scroll_snap_points_y,
                    AnimatableLonghand::ScrollSnapDestination
                        => 
    nsCSSPropertyID::eCSSProperty_scroll_snap_destination,
                    AnimatableLonghand::ScrollSnapCoordinate
                        => 
    nsCSSPropertyID::eCSSProperty_scroll_snap_coordinate,
                    AnimatableLonghand::Transform
                        => 
    nsCSSPropertyID::eCSSProperty_transform,
                    AnimatableLonghand::ScrollBehavior
                        => 
    nsCSSPropertyID::eCSSProperty_scroll_behavior,
                    AnimatableLonghand::ScrollSnapTypeX
                        => 
    nsCSSPropertyID::eCSSProperty_scroll_snap_type_x,
                    AnimatableLonghand::ScrollSnapTypeY
                        => 
    nsCSSPropertyID::eCSSProperty_scroll_snap_type_y,
                    AnimatableLonghand::Isolation
                        => 
    nsCSSPropertyID::eCSSProperty_isolation,
                    AnimatableLonghand::PageBreakAfter
                        => 
    nsCSSPropertyID::eCSSProperty_page_break_after,
                    AnimatableLonghand::PageBreakBefore
                        => 
    nsCSSPropertyID::eCSSProperty_page_break_before,
                    AnimatableLonghand::PageBreakInside
                        => 
    nsCSSPropertyID::eCSSProperty_page_break_inside,
                    AnimatableLonghand::Resize
                        => 
    nsCSSPropertyID::eCSSProperty_resize,
                    AnimatableLonghand::Perspective
                        => 
    nsCSSPropertyID::eCSSProperty_perspective,
                    AnimatableLonghand::PerspectiveOrigin
                        => 
    nsCSSPropertyID::eCSSProperty_perspective_origin,
                    AnimatableLonghand::BackfaceVisibility
                        => 
    nsCSSPropertyID::eCSSProperty_backface_visibility,
                    AnimatableLonghand::TransformBox
                        => 
    nsCSSPropertyID::eCSSProperty_transform_box,
                    AnimatableLonghand::TransformStyle
                        => 
    nsCSSPropertyID::eCSSProperty_transform_style,
                    AnimatableLonghand::TransformOrigin
                        => 
    nsCSSPropertyID::eCSSProperty_transform_origin,
                    AnimatableLonghand::Contain
                        => 
    nsCSSPropertyID::eCSSProperty_contain,
                    AnimatableLonghand::MozAppearance
                        => 
    nsCSSPropertyID::eCSSProperty__moz_appearance,
                    AnimatableLonghand::MozOrient
                        => 
    nsCSSPropertyID::eCSSProperty__moz_orient,
                    AnimatableLonghand::WillChange
                        => 
    nsCSSPropertyID::eCSSProperty_will_change,
                    AnimatableLonghand::ShapeOutside
                        => 
    nsCSSPropertyID::eCSSProperty_shape_outside,
                    AnimatableLonghand::TouchAction
                        => 
    nsCSSPropertyID::eCSSProperty_touch_action,
                    AnimatableLonghand::Color
                        => 
    nsCSSPropertyID::eCSSProperty_color,
                    AnimatableLonghand::ColumnWidth
                        => 
    nsCSSPropertyID::eCSSProperty_column_width,
                    AnimatableLonghand::ColumnCount
                        => 
    nsCSSPropertyID::eCSSProperty_column_count,
                    AnimatableLonghand::ColumnGap
                        => 
    nsCSSPropertyID::eCSSProperty_column_gap,
                    AnimatableLonghand::ColumnFill
                        => 
    nsCSSPropertyID::eCSSProperty_column_fill,
                    AnimatableLonghand::ColumnRuleWidth
                        => 
    nsCSSPropertyID::eCSSProperty_column_rule_width,
                    AnimatableLonghand::ColumnRuleColor
                        => 
    nsCSSPropertyID::eCSSProperty_column_rule_color,
                    AnimatableLonghand::ColumnSpan
                        => 
    nsCSSPropertyID::eCSSProperty_column_span,
                    AnimatableLonghand::ColumnRuleStyle
                        => 
    nsCSSPropertyID::eCSSProperty_column_rule_style,
                    AnimatableLonghand::Content
                        => 
    nsCSSPropertyID::eCSSProperty_content,
                    AnimatableLonghand::CounterIncrement
                        => 
    nsCSSPropertyID::eCSSProperty_counter_increment,
                    AnimatableLonghand::CounterReset
                        => 
    nsCSSPropertyID::eCSSProperty_counter_reset,
                    AnimatableLonghand::Opacity
                        => 
    nsCSSPropertyID::eCSSProperty_opacity,
                    AnimatableLonghand::BoxShadow
                        => 
    nsCSSPropertyID::eCSSProperty_box_shadow,
                    AnimatableLonghand::Clip
                        => 
    nsCSSPropertyID::eCSSProperty_clip,
                    AnimatableLonghand::Filter
                        => 
    nsCSSPropertyID::eCSSProperty_filter,
                    AnimatableLonghand::MixBlendMode
                        => 
    nsCSSPropertyID::eCSSProperty_mix_blend_mode,
                    AnimatableLonghand::FontFamily
                        => 
    nsCSSPropertyID::eCSSProperty_font_family,
                    AnimatableLonghand::FontStyle
                        => 
    nsCSSPropertyID::eCSSProperty_font_style,
                    AnimatableLonghand::FontVariantCaps
                        => 
    nsCSSPropertyID::eCSSProperty_font_variant_caps,
                    AnimatableLonghand::FontWeight
                        => 
    nsCSSPropertyID::eCSSProperty_font_weight,
                    AnimatableLonghand::FontSize
                        => 
    nsCSSPropertyID::eCSSProperty_font_size,
                    AnimatableLonghand::FontSizeAdjust
                        => 
    nsCSSPropertyID::eCSSProperty_font_size_adjust,
                    AnimatableLonghand::FontSynthesis
                        => 
    nsCSSPropertyID::eCSSProperty_font_synthesis,
                    AnimatableLonghand::FontStretch
                        => 
    nsCSSPropertyID::eCSSProperty_font_stretch,
                    AnimatableLonghand::FontKerning
                        => 
    nsCSSPropertyID::eCSSProperty_font_kerning,
                    AnimatableLonghand::FontVariantAlternates
                        => 
    nsCSSPropertyID::eCSSProperty_font_variant_alternates,
                    AnimatableLonghand::FontVariantEastAsian
                        => 
    nsCSSPropertyID::eCSSProperty_font_variant_east_asian,
                    AnimatableLonghand::FontVariantLigatures
                        => 
    nsCSSPropertyID::eCSSProperty_font_variant_ligatures,
                    AnimatableLonghand::FontVariantNumeric
                        => 
    nsCSSPropertyID::eCSSProperty_font_variant_numeric,
                    AnimatableLonghand::FontVariantPosition
                        => 
    nsCSSPropertyID::eCSSProperty_font_variant_position,
                    AnimatableLonghand::FontFeatureSettings
                        => 
    nsCSSPropertyID::eCSSProperty_font_feature_settings,
                    AnimatableLonghand::FontVariationSettings
                        => 
    nsCSSPropertyID::eCSSProperty_font_variation_settings,
                    AnimatableLonghand::FontLanguageOverride
                        => 
    nsCSSPropertyID::eCSSProperty_font_language_override,
                    AnimatableLonghand::MozOsxFontSmoothing
                        => 
    nsCSSPropertyID::eCSSProperty__moz_osx_font_smoothing,
                    AnimatableLonghand::Visibility
                        => 
    nsCSSPropertyID::eCSSProperty_visibility,
                    AnimatableLonghand::WritingMode
                        => 
    nsCSSPropertyID::eCSSProperty_writing_mode,
                    AnimatableLonghand::Direction
                        => 
    nsCSSPropertyID::eCSSProperty_direction,
                    AnimatableLonghand::TextOrientation
                        => 
    nsCSSPropertyID::eCSSProperty_text_orientation,
                    AnimatableLonghand::ColorAdjust
                        => 
    nsCSSPropertyID::eCSSProperty_color_adjust,
                    AnimatableLonghand::ImageRendering
                        => 
    nsCSSPropertyID::eCSSProperty_image_rendering,
                    AnimatableLonghand::ImageOrientation
                        => 
    nsCSSPropertyID::eCSSProperty_image_orientation,
                    AnimatableLonghand::BorderCollapse
                        => 
    nsCSSPropertyID::eCSSProperty_border_collapse,
                    AnimatableLonghand::EmptyCells
                        => 
    nsCSSPropertyID::eCSSProperty_empty_cells,
                    AnimatableLonghand::CaptionSide
                        => 
    nsCSSPropertyID::eCSSProperty_caption_side,
                    AnimatableLonghand::BorderSpacing
                        => 
    nsCSSPropertyID::eCSSProperty_border_spacing,
                    AnimatableLonghand::LineHeight
                        => 
    nsCSSPropertyID::eCSSProperty_line_height,
                    AnimatableLonghand::TextTransform
                        => 
    nsCSSPropertyID::eCSSProperty_text_transform,
                    AnimatableLonghand::Hyphens
                        => 
    nsCSSPropertyID::eCSSProperty_hyphens,
                    AnimatableLonghand::MozTextSizeAdjust
                        => 
    nsCSSPropertyID::eCSSProperty__moz_text_size_adjust,
                    AnimatableLonghand::TextIndent
                        => 
    nsCSSPropertyID::eCSSProperty_text_indent,
                    AnimatableLonghand::OverflowWrap
                        => 
    nsCSSPropertyID::eCSSProperty_overflow_wrap,
                    AnimatableLonghand::WordBreak
                        => 
    nsCSSPropertyID::eCSSProperty_word_break,
                    AnimatableLonghand::TextJustify
                        => 
    nsCSSPropertyID::eCSSProperty_text_justify,
                    AnimatableLonghand::TextAlignLast
                        => 
    nsCSSPropertyID::eCSSProperty_text_align_last,
                    AnimatableLonghand::TextAlign
                        => 
    nsCSSPropertyID::eCSSProperty_text_align,
                    AnimatableLonghand::LetterSpacing
                        => 
    nsCSSPropertyID::eCSSProperty_letter_spacing,
                    AnimatableLonghand::WordSpacing
                        => 
    nsCSSPropertyID::eCSSProperty_word_spacing,
                    AnimatableLonghand::WhiteSpace
                        => 
    nsCSSPropertyID::eCSSProperty_white_space,
                    AnimatableLonghand::TextShadow
                        => 
    nsCSSPropertyID::eCSSProperty_text_shadow,
                    AnimatableLonghand::TextEmphasisStyle
                        => 
    nsCSSPropertyID::eCSSProperty_text_emphasis_style,
                    AnimatableLonghand::TextEmphasisPosition
                        => 
    nsCSSPropertyID::eCSSProperty_text_emphasis_position,
                    AnimatableLonghand::TextEmphasisColor
                        => 
    nsCSSPropertyID::eCSSProperty_text_emphasis_color,
                    AnimatableLonghand::MozTabSize
                        => 
    nsCSSPropertyID::eCSSProperty__moz_tab_size,
                    AnimatableLonghand::WebkitTextFillColor
                        => 
    nsCSSPropertyID::eCSSProperty__webkit_text_fill_color,
                    AnimatableLonghand::WebkitTextStrokeColor
                        => 
    nsCSSPropertyID::eCSSProperty__webkit_text_stroke_color,
                    AnimatableLonghand::WebkitTextStrokeWidth
                        => 
    nsCSSPropertyID::eCSSProperty__webkit_text_stroke_width,
                    AnimatableLonghand::RubyAlign
                        => 
    nsCSSPropertyID::eCSSProperty_ruby_align,
                    AnimatableLonghand::RubyPosition
                        => 
    nsCSSPropertyID::eCSSProperty_ruby_position,
                    AnimatableLonghand::TextCombineUpright
                        => 
    nsCSSPropertyID::eCSSProperty_text_combine_upright,
                    AnimatableLonghand::TextRendering
                        => 
    nsCSSPropertyID::eCSSProperty_text_rendering,
                    AnimatableLonghand::ListStylePosition
                        => 
    nsCSSPropertyID::eCSSProperty_list_style_position,
                    AnimatableLonghand::ListStyleType
                        => 
    nsCSSPropertyID::eCSSProperty_list_style_type,
                    AnimatableLonghand::ListStyleImage
                        => 
    nsCSSPropertyID::eCSSProperty_list_style_image,
                    AnimatableLonghand::Quotes
                        => 
    nsCSSPropertyID::eCSSProperty_quotes,
                    AnimatableLonghand::MozImageRegion
                        => 
    nsCSSPropertyID::eCSSProperty__moz_image_region,
                    AnimatableLonghand::MarginTop
                        => 
    nsCSSPropertyID::eCSSProperty_margin_top,
                    AnimatableLonghand::MarginLeft
                        => 
    nsCSSPropertyID::eCSSProperty_margin_left,
                    AnimatableLonghand::MarginBottom
                        => 
    nsCSSPropertyID::eCSSProperty_margin_bottom,
                    AnimatableLonghand::MarginRight
                        => 
    nsCSSPropertyID::eCSSProperty_margin_right,
                    AnimatableLonghand::OutlineColor
                        => 
    nsCSSPropertyID::eCSSProperty_outline_color,
                    AnimatableLonghand::OutlineStyle
                        => 
    nsCSSPropertyID::eCSSProperty_outline_style,
                    AnimatableLonghand::OutlineWidth
                        => 
    nsCSSPropertyID::eCSSProperty_outline_width,
                    AnimatableLonghand::MozOutlineRadiusTopleft
                        => 
    nsCSSPropertyID::eCSSProperty__moz_outline_radius_topleft,
                    AnimatableLonghand::MozOutlineRadiusTopright
                        => 
    nsCSSPropertyID::eCSSProperty__moz_outline_radius_topright,
                    AnimatableLonghand::MozOutlineRadiusBottomright
                        => 
    nsCSSPropertyID::eCSSProperty__moz_outline_radius_bottomright,
                    AnimatableLonghand::MozOutlineRadiusBottomleft
                        => 
    nsCSSPropertyID::eCSSProperty__moz_outline_radius_bottomleft,
                    AnimatableLonghand::OutlineOffset
                        => 
    nsCSSPropertyID::eCSSProperty_outline_offset,
                    AnimatableLonghand::PaddingTop
                        => 
    nsCSSPropertyID::eCSSProperty_padding_top,
                    AnimatableLonghand::PaddingLeft
                        => 
    nsCSSPropertyID::eCSSProperty_padding_left,
                    AnimatableLonghand::PaddingBottom
                        => 
    nsCSSPropertyID::eCSSProperty_padding_bottom,
                    AnimatableLonghand::PaddingRight
                        => 
    nsCSSPropertyID::eCSSProperty_padding_right,
                    AnimatableLonghand::Cursor
                        => 
    nsCSSPropertyID::eCSSProperty_cursor,
                    AnimatableLonghand::PointerEvents
                        => 
    nsCSSPropertyID::eCSSProperty_pointer_events,
                    AnimatableLonghand::MozUserInput
                        => 
    nsCSSPropertyID::eCSSProperty__moz_user_input,
                    AnimatableLonghand::MozUserModify
                        => 
    nsCSSPropertyID::eCSSProperty__moz_user_modify,
                    AnimatableLonghand::MozUserFocus
                        => 
    nsCSSPropertyID::eCSSProperty__moz_user_focus,
                    AnimatableLonghand::CaretColor
                        => 
    nsCSSPropertyID::eCSSProperty_caret_color,
                    AnimatableLonghand::Top
                        => 
    nsCSSPropertyID::eCSSProperty_top,
                    AnimatableLonghand::Left
                        => 
    nsCSSPropertyID::eCSSProperty_left,
                    AnimatableLonghand::Bottom
                        => 
    nsCSSPropertyID::eCSSProperty_bottom,
                    AnimatableLonghand::Right
                        => 
    nsCSSPropertyID::eCSSProperty_right,
                    AnimatableLonghand::ZIndex
                        => 
    nsCSSPropertyID::eCSSProperty_z_index,
                    AnimatableLonghand::FlexDirection
                        => 
    nsCSSPropertyID::eCSSProperty_flex_direction,
                    AnimatableLonghand::FlexWrap
                        => 
    nsCSSPropertyID::eCSSProperty_flex_wrap,
                    AnimatableLonghand::JustifyContent
                        => 
    nsCSSPropertyID::eCSSProperty_justify_content,
                    AnimatableLonghand::AlignContent
                        => 
    nsCSSPropertyID::eCSSProperty_align_content,
                    AnimatableLonghand::AlignItems
                        => 
    nsCSSPropertyID::eCSSProperty_align_items,
                    AnimatableLonghand::JustifyItems
                        => 
    nsCSSPropertyID::eCSSProperty_justify_items,
                    AnimatableLonghand::FlexGrow
                        => 
    nsCSSPropertyID::eCSSProperty_flex_grow,
                    AnimatableLonghand::FlexShrink
                        => 
    nsCSSPropertyID::eCSSProperty_flex_shrink,
                    AnimatableLonghand::AlignSelf
                        => 
    nsCSSPropertyID::eCSSProperty_align_self,
                    AnimatableLonghand::JustifySelf
                        => 
    nsCSSPropertyID::eCSSProperty_justify_self,
                    AnimatableLonghand::Order
                        => 
    nsCSSPropertyID::eCSSProperty_order,
                    AnimatableLonghand::FlexBasis
                        => 
    nsCSSPropertyID::eCSSProperty_flex_basis,
                    AnimatableLonghand::Width
                        => 
    nsCSSPropertyID::eCSSProperty_width,
                    AnimatableLonghand::MinWidth
                        => 
    nsCSSPropertyID::eCSSProperty_min_width,
                    AnimatableLonghand::MaxWidth
                        => 
    nsCSSPropertyID::eCSSProperty_max_width,
                    AnimatableLonghand::Height
                        => 
    nsCSSPropertyID::eCSSProperty_height,
                    AnimatableLonghand::MinHeight
                        => 
    nsCSSPropertyID::eCSSProperty_min_height,
                    AnimatableLonghand::MaxHeight
                        => 
    nsCSSPropertyID::eCSSProperty_max_height,
                    AnimatableLonghand::BoxSizing
                        => 
    nsCSSPropertyID::eCSSProperty_box_sizing,
                    AnimatableLonghand::ObjectFit
                        => 
    nsCSSPropertyID::eCSSProperty_object_fit,
                    AnimatableLonghand::ObjectPosition
                        => 
    nsCSSPropertyID::eCSSProperty_object_position,
                    AnimatableLonghand::GridRowGap
                        => 
    nsCSSPropertyID::eCSSProperty_grid_row_gap,
                    AnimatableLonghand::GridRowStart
                        => 
    nsCSSPropertyID::eCSSProperty_grid_row_start,
                    AnimatableLonghand::GridRowEnd
                        => 
    nsCSSPropertyID::eCSSProperty_grid_row_end,
                    AnimatableLonghand::GridAutoRows
                        => 
    nsCSSPropertyID::eCSSProperty_grid_auto_rows,
                    AnimatableLonghand::GridTemplateRows
                        => 
    nsCSSPropertyID::eCSSProperty_grid_template_rows,
                    AnimatableLonghand::GridColumnGap
                        => 
    nsCSSPropertyID::eCSSProperty_grid_column_gap,
                    AnimatableLonghand::GridColumnStart
                        => 
    nsCSSPropertyID::eCSSProperty_grid_column_start,
                    AnimatableLonghand::GridColumnEnd
                        => 
    nsCSSPropertyID::eCSSProperty_grid_column_end,
                    AnimatableLonghand::GridAutoColumns
                        => 
    nsCSSPropertyID::eCSSProperty_grid_auto_columns,
                    AnimatableLonghand::GridTemplateColumns
                        => 
    nsCSSPropertyID::eCSSProperty_grid_template_columns,
                    AnimatableLonghand::GridAutoFlow
                        => 
    nsCSSPropertyID::eCSSProperty_grid_auto_flow,
                    AnimatableLonghand::GridTemplateAreas
                        => 
    nsCSSPropertyID::eCSSProperty_grid_template_areas,
                    AnimatableLonghand::TableLayout
                        => 
    nsCSSPropertyID::eCSSProperty_table_layout,
                    AnimatableLonghand::TextOverflow
                        => 
    nsCSSPropertyID::eCSSProperty_text_overflow,
                    AnimatableLonghand::UnicodeBidi
                        => 
    nsCSSPropertyID::eCSSProperty_unicode_bidi,
                    AnimatableLonghand::TextDecorationLine
                        => 
    nsCSSPropertyID::eCSSProperty_text_decoration_line,
                    AnimatableLonghand::TextDecorationStyle
                        => 
    nsCSSPropertyID::eCSSProperty_text_decoration_style,
                    AnimatableLonghand::TextDecorationColor
                        => 
    nsCSSPropertyID::eCSSProperty_text_decoration_color,
                    AnimatableLonghand::InitialLetter
                        => 
    nsCSSPropertyID::eCSSProperty_initial_letter,
                    AnimatableLonghand::ImeMode
                        => 
    nsCSSPropertyID::eCSSProperty_ime_mode,
                    AnimatableLonghand::MozUserSelect
                        => 
    nsCSSPropertyID::eCSSProperty__moz_user_select,
                    AnimatableLonghand::MozWindowDragging
                        => 
    nsCSSPropertyID::eCSSProperty__moz_window_dragging,
                    AnimatableLonghand::MozWindowShadow
                        => 
    nsCSSPropertyID::eCSSProperty__moz_window_shadow,
                    AnimatableLonghand::MozForceBrokenImageIcon
                        => 
    nsCSSPropertyID::eCSSProperty__moz_force_broken_image_icon,
                    AnimatableLonghand::TextAnchor
                        => 
    nsCSSPropertyID::eCSSProperty_text_anchor,
                    AnimatableLonghand::ColorInterpolation
                        => 
    nsCSSPropertyID::eCSSProperty_color_interpolation,
                    AnimatableLonghand::ColorInterpolationFilters
                        => 
    nsCSSPropertyID::eCSSProperty_color_interpolation_filters,
                    AnimatableLonghand::Fill
                        => 
    nsCSSPropertyID::eCSSProperty_fill,
                    AnimatableLonghand::FillOpacity
                        => 
    nsCSSPropertyID::eCSSProperty_fill_opacity,
                    AnimatableLonghand::FillRule
                        => 
    nsCSSPropertyID::eCSSProperty_fill_rule,
                    AnimatableLonghand::ShapeRendering
                        => 
    nsCSSPropertyID::eCSSProperty_shape_rendering,
                    AnimatableLonghand::Stroke
                        => 
    nsCSSPropertyID::eCSSProperty_stroke,
                    AnimatableLonghand::StrokeWidth
                        => 
    nsCSSPropertyID::eCSSProperty_stroke_width,
                    AnimatableLonghand::StrokeLinecap
                        => 
    nsCSSPropertyID::eCSSProperty_stroke_linecap,
                    AnimatableLonghand::StrokeLinejoin
                        => 
    nsCSSPropertyID::eCSSProperty_stroke_linejoin,
                    AnimatableLonghand::StrokeMiterlimit
                        => 
    nsCSSPropertyID::eCSSProperty_stroke_miterlimit,
                    AnimatableLonghand::StrokeOpacity
                        => 
    nsCSSPropertyID::eCSSProperty_stroke_opacity,
                    AnimatableLonghand::StrokeDasharray
                        => 
    nsCSSPropertyID::eCSSProperty_stroke_dasharray,
                    AnimatableLonghand::StrokeDashoffset
                        => 
    nsCSSPropertyID::eCSSProperty_stroke_dashoffset,
                    AnimatableLonghand::ClipRule
                        => 
    nsCSSPropertyID::eCSSProperty_clip_rule,
                    AnimatableLonghand::MarkerStart
                        => 
    nsCSSPropertyID::eCSSProperty_marker_start,
                    AnimatableLonghand::MarkerMid
                        => 
    nsCSSPropertyID::eCSSProperty_marker_mid,
                    AnimatableLonghand::MarkerEnd
                        => 
    nsCSSPropertyID::eCSSProperty_marker_end,
                    AnimatableLonghand::PaintOrder
                        => 
    nsCSSPropertyID::eCSSProperty_paint_order,
                    AnimatableLonghand::DominantBaseline
                        => 
    nsCSSPropertyID::eCSSProperty_dominant_baseline,
                    AnimatableLonghand::VectorEffect
                        => 
    nsCSSPropertyID::eCSSProperty_vector_effect,
                    AnimatableLonghand::StopColor
                        => 
    nsCSSPropertyID::eCSSProperty_stop_color,
                    AnimatableLonghand::StopOpacity
                        => 
    nsCSSPropertyID::eCSSProperty_stop_opacity,
                    AnimatableLonghand::FloodColor
                        => 
    nsCSSPropertyID::eCSSProperty_flood_color,
                    AnimatableLonghand::FloodOpacity
                        => 
    nsCSSPropertyID::eCSSProperty_flood_opacity,
                    AnimatableLonghand::LightingColor
                        => 
    nsCSSPropertyID::eCSSProperty_lighting_color,
                    AnimatableLonghand::MaskType
                        => 
    nsCSSPropertyID::eCSSProperty_mask_type,
                    AnimatableLonghand::ClipPath
                        => 
    nsCSSPropertyID::eCSSProperty_clip_path,
                    AnimatableLonghand::MaskMode
                        => 
    nsCSSPropertyID::eCSSProperty_mask_mode,
                    AnimatableLonghand::MaskRepeat
                        => 
    nsCSSPropertyID::eCSSProperty_mask_repeat,
                    AnimatableLonghand::MaskPositionX
                        => 
    nsCSSPropertyID::eCSSProperty_mask_position_x,
                    AnimatableLonghand::MaskPositionY
                        => 
    nsCSSPropertyID::eCSSProperty_mask_position_y,
                    AnimatableLonghand::MaskClip
                        => 
    nsCSSPropertyID::eCSSProperty_mask_clip,
                    AnimatableLonghand::MaskOrigin
                        => 
    nsCSSPropertyID::eCSSProperty_mask_origin,
                    AnimatableLonghand::MaskSize
                        => 
    nsCSSPropertyID::eCSSProperty_mask_size,
                    AnimatableLonghand::MaskComposite
                        => 
    nsCSSPropertyID::eCSSProperty_mask_composite,
                    AnimatableLonghand::MaskImage
                        => 
    nsCSSPropertyID::eCSSProperty_mask_image,
                    AnimatableLonghand::MozBoxAlign
                        => 
    nsCSSPropertyID::eCSSProperty__moz_box_align,
                    AnimatableLonghand::MozBoxDirection
                        => 
    nsCSSPropertyID::eCSSProperty__moz_box_direction,
                    AnimatableLonghand::MozBoxFlex
                        => 
    nsCSSPropertyID::eCSSProperty__moz_box_flex,
                    AnimatableLonghand::MozBoxOrient
                        => 
    nsCSSPropertyID::eCSSProperty__moz_box_orient,
                    AnimatableLonghand::MozBoxPack
                        => 
    nsCSSPropertyID::eCSSProperty__moz_box_pack,
                    AnimatableLonghand::MozStackSizing
                        => 
    nsCSSPropertyID::eCSSProperty__moz_stack_sizing,
                    AnimatableLonghand::MozBoxOrdinalGroup
                        => 
    nsCSSPropertyID::eCSSProperty__moz_box_ordinal_group,
        }
    }
}

/// Convert to PropertyDeclarationId.
#[cfg(feature = "gecko")]
#[allow(non_upper_case_globals)]
impl<'a> From<AnimatableLonghand> for PropertyDeclarationId<'a> {
    fn from(property: AnimatableLonghand) -> PropertyDeclarationId<'a> {
        match property {
                    AnimatableLonghand::BackgroundColor
                        => PropertyDeclarationId::Longhand(LonghandId::BackgroundColor),
                    AnimatableLonghand::BackgroundImage
                        => PropertyDeclarationId::Longhand(LonghandId::BackgroundImage),
                    AnimatableLonghand::BackgroundPositionX
                        => PropertyDeclarationId::Longhand(LonghandId::BackgroundPositionX),
                    AnimatableLonghand::BackgroundPositionY
                        => PropertyDeclarationId::Longhand(LonghandId::BackgroundPositionY),
                    AnimatableLonghand::BackgroundRepeat
                        => PropertyDeclarationId::Longhand(LonghandId::BackgroundRepeat),
                    AnimatableLonghand::BackgroundAttachment
                        => PropertyDeclarationId::Longhand(LonghandId::BackgroundAttachment),
                    AnimatableLonghand::BackgroundClip
                        => PropertyDeclarationId::Longhand(LonghandId::BackgroundClip),
                    AnimatableLonghand::BackgroundOrigin
                        => PropertyDeclarationId::Longhand(LonghandId::BackgroundOrigin),
                    AnimatableLonghand::BackgroundSize
                        => PropertyDeclarationId::Longhand(LonghandId::BackgroundSize),
                    AnimatableLonghand::BackgroundBlendMode
                        => PropertyDeclarationId::Longhand(LonghandId::BackgroundBlendMode),
                    AnimatableLonghand::BorderTopColor
                        => PropertyDeclarationId::Longhand(LonghandId::BorderTopColor),
                    AnimatableLonghand::BorderTopStyle
                        => PropertyDeclarationId::Longhand(LonghandId::BorderTopStyle),
                    AnimatableLonghand::BorderTopWidth
                        => PropertyDeclarationId::Longhand(LonghandId::BorderTopWidth),
                    AnimatableLonghand::BorderLeftColor
                        => PropertyDeclarationId::Longhand(LonghandId::BorderLeftColor),
                    AnimatableLonghand::BorderLeftStyle
                        => PropertyDeclarationId::Longhand(LonghandId::BorderLeftStyle),
                    AnimatableLonghand::BorderLeftWidth
                        => PropertyDeclarationId::Longhand(LonghandId::BorderLeftWidth),
                    AnimatableLonghand::BorderBottomColor
                        => PropertyDeclarationId::Longhand(LonghandId::BorderBottomColor),
                    AnimatableLonghand::BorderBottomStyle
                        => PropertyDeclarationId::Longhand(LonghandId::BorderBottomStyle),
                    AnimatableLonghand::BorderBottomWidth
                        => PropertyDeclarationId::Longhand(LonghandId::BorderBottomWidth),
                    AnimatableLonghand::BorderRightColor
                        => PropertyDeclarationId::Longhand(LonghandId::BorderRightColor),
                    AnimatableLonghand::BorderRightStyle
                        => PropertyDeclarationId::Longhand(LonghandId::BorderRightStyle),
                    AnimatableLonghand::BorderRightWidth
                        => PropertyDeclarationId::Longhand(LonghandId::BorderRightWidth),
                    AnimatableLonghand::BorderTopLeftRadius
                        => PropertyDeclarationId::Longhand(LonghandId::BorderTopLeftRadius),
                    AnimatableLonghand::BorderTopRightRadius
                        => PropertyDeclarationId::Longhand(LonghandId::BorderTopRightRadius),
                    AnimatableLonghand::BorderBottomRightRadius
                        => PropertyDeclarationId::Longhand(LonghandId::BorderBottomRightRadius),
                    AnimatableLonghand::BorderBottomLeftRadius
                        => PropertyDeclarationId::Longhand(LonghandId::BorderBottomLeftRadius),
                    AnimatableLonghand::MozBorderTopColors
                        => PropertyDeclarationId::Longhand(LonghandId::MozBorderTopColors),
                    AnimatableLonghand::MozBorderLeftColors
                        => PropertyDeclarationId::Longhand(LonghandId::MozBorderLeftColors),
                    AnimatableLonghand::MozBorderBottomColors
                        => PropertyDeclarationId::Longhand(LonghandId::MozBorderBottomColors),
                    AnimatableLonghand::MozBorderRightColors
                        => PropertyDeclarationId::Longhand(LonghandId::MozBorderRightColors),
                    AnimatableLonghand::BoxDecorationBreak
                        => PropertyDeclarationId::Longhand(LonghandId::BoxDecorationBreak),
                    AnimatableLonghand::MozFloatEdge
                        => PropertyDeclarationId::Longhand(LonghandId::MozFloatEdge),
                    AnimatableLonghand::BorderImageSource
                        => PropertyDeclarationId::Longhand(LonghandId::BorderImageSource),
                    AnimatableLonghand::BorderImageOutset
                        => PropertyDeclarationId::Longhand(LonghandId::BorderImageOutset),
                    AnimatableLonghand::BorderImageRepeat
                        => PropertyDeclarationId::Longhand(LonghandId::BorderImageRepeat),
                    AnimatableLonghand::BorderImageWidth
                        => PropertyDeclarationId::Longhand(LonghandId::BorderImageWidth),
                    AnimatableLonghand::BorderImageSlice
                        => PropertyDeclarationId::Longhand(LonghandId::BorderImageSlice),
                    AnimatableLonghand::Display
                        => PropertyDeclarationId::Longhand(LonghandId::Display),
                    AnimatableLonghand::Position
                        => PropertyDeclarationId::Longhand(LonghandId::Position),
                    AnimatableLonghand::Float
                        => PropertyDeclarationId::Longhand(LonghandId::Float),
                    AnimatableLonghand::Clear
                        => PropertyDeclarationId::Longhand(LonghandId::Clear),
                    AnimatableLonghand::VerticalAlign
                        => PropertyDeclarationId::Longhand(LonghandId::VerticalAlign),
                    AnimatableLonghand::OverflowClipBox
                        => PropertyDeclarationId::Longhand(LonghandId::OverflowClipBox),
                    AnimatableLonghand::OverflowX
                        => PropertyDeclarationId::Longhand(LonghandId::OverflowX),
                    AnimatableLonghand::OverflowY
                        => PropertyDeclarationId::Longhand(LonghandId::OverflowY),
                    AnimatableLonghand::ScrollSnapPointsX
                        => PropertyDeclarationId::Longhand(LonghandId::ScrollSnapPointsX),
                    AnimatableLonghand::ScrollSnapPointsY
                        => PropertyDeclarationId::Longhand(LonghandId::ScrollSnapPointsY),
                    AnimatableLonghand::ScrollSnapDestination
                        => PropertyDeclarationId::Longhand(LonghandId::ScrollSnapDestination),
                    AnimatableLonghand::ScrollSnapCoordinate
                        => PropertyDeclarationId::Longhand(LonghandId::ScrollSnapCoordinate),
                    AnimatableLonghand::Transform
                        => PropertyDeclarationId::Longhand(LonghandId::Transform),
                    AnimatableLonghand::ScrollBehavior
                        => PropertyDeclarationId::Longhand(LonghandId::ScrollBehavior),
                    AnimatableLonghand::ScrollSnapTypeX
                        => PropertyDeclarationId::Longhand(LonghandId::ScrollSnapTypeX),
                    AnimatableLonghand::ScrollSnapTypeY
                        => PropertyDeclarationId::Longhand(LonghandId::ScrollSnapTypeY),
                    AnimatableLonghand::Isolation
                        => PropertyDeclarationId::Longhand(LonghandId::Isolation),
                    AnimatableLonghand::PageBreakAfter
                        => PropertyDeclarationId::Longhand(LonghandId::PageBreakAfter),
                    AnimatableLonghand::PageBreakBefore
                        => PropertyDeclarationId::Longhand(LonghandId::PageBreakBefore),
                    AnimatableLonghand::PageBreakInside
                        => PropertyDeclarationId::Longhand(LonghandId::PageBreakInside),
                    AnimatableLonghand::Resize
                        => PropertyDeclarationId::Longhand(LonghandId::Resize),
                    AnimatableLonghand::Perspective
                        => PropertyDeclarationId::Longhand(LonghandId::Perspective),
                    AnimatableLonghand::PerspectiveOrigin
                        => PropertyDeclarationId::Longhand(LonghandId::PerspectiveOrigin),
                    AnimatableLonghand::BackfaceVisibility
                        => PropertyDeclarationId::Longhand(LonghandId::BackfaceVisibility),
                    AnimatableLonghand::TransformBox
                        => PropertyDeclarationId::Longhand(LonghandId::TransformBox),
                    AnimatableLonghand::TransformStyle
                        => PropertyDeclarationId::Longhand(LonghandId::TransformStyle),
                    AnimatableLonghand::TransformOrigin
                        => PropertyDeclarationId::Longhand(LonghandId::TransformOrigin),
                    AnimatableLonghand::Contain
                        => PropertyDeclarationId::Longhand(LonghandId::Contain),
                    AnimatableLonghand::MozAppearance
                        => PropertyDeclarationId::Longhand(LonghandId::MozAppearance),
                    AnimatableLonghand::MozOrient
                        => PropertyDeclarationId::Longhand(LonghandId::MozOrient),
                    AnimatableLonghand::WillChange
                        => PropertyDeclarationId::Longhand(LonghandId::WillChange),
                    AnimatableLonghand::ShapeOutside
                        => PropertyDeclarationId::Longhand(LonghandId::ShapeOutside),
                    AnimatableLonghand::TouchAction
                        => PropertyDeclarationId::Longhand(LonghandId::TouchAction),
                    AnimatableLonghand::Color
                        => PropertyDeclarationId::Longhand(LonghandId::Color),
                    AnimatableLonghand::ColumnWidth
                        => PropertyDeclarationId::Longhand(LonghandId::ColumnWidth),
                    AnimatableLonghand::ColumnCount
                        => PropertyDeclarationId::Longhand(LonghandId::ColumnCount),
                    AnimatableLonghand::ColumnGap
                        => PropertyDeclarationId::Longhand(LonghandId::ColumnGap),
                    AnimatableLonghand::ColumnFill
                        => PropertyDeclarationId::Longhand(LonghandId::ColumnFill),
                    AnimatableLonghand::ColumnRuleWidth
                        => PropertyDeclarationId::Longhand(LonghandId::ColumnRuleWidth),
                    AnimatableLonghand::ColumnRuleColor
                        => PropertyDeclarationId::Longhand(LonghandId::ColumnRuleColor),
                    AnimatableLonghand::ColumnSpan
                        => PropertyDeclarationId::Longhand(LonghandId::ColumnSpan),
                    AnimatableLonghand::ColumnRuleStyle
                        => PropertyDeclarationId::Longhand(LonghandId::ColumnRuleStyle),
                    AnimatableLonghand::Content
                        => PropertyDeclarationId::Longhand(LonghandId::Content),
                    AnimatableLonghand::CounterIncrement
                        => PropertyDeclarationId::Longhand(LonghandId::CounterIncrement),
                    AnimatableLonghand::CounterReset
                        => PropertyDeclarationId::Longhand(LonghandId::CounterReset),
                    AnimatableLonghand::Opacity
                        => PropertyDeclarationId::Longhand(LonghandId::Opacity),
                    AnimatableLonghand::BoxShadow
                        => PropertyDeclarationId::Longhand(LonghandId::BoxShadow),
                    AnimatableLonghand::Clip
                        => PropertyDeclarationId::Longhand(LonghandId::Clip),
                    AnimatableLonghand::Filter
                        => PropertyDeclarationId::Longhand(LonghandId::Filter),
                    AnimatableLonghand::MixBlendMode
                        => PropertyDeclarationId::Longhand(LonghandId::MixBlendMode),
                    AnimatableLonghand::FontFamily
                        => PropertyDeclarationId::Longhand(LonghandId::FontFamily),
                    AnimatableLonghand::FontStyle
                        => PropertyDeclarationId::Longhand(LonghandId::FontStyle),
                    AnimatableLonghand::FontVariantCaps
                        => PropertyDeclarationId::Longhand(LonghandId::FontVariantCaps),
                    AnimatableLonghand::FontWeight
                        => PropertyDeclarationId::Longhand(LonghandId::FontWeight),
                    AnimatableLonghand::FontSize
                        => PropertyDeclarationId::Longhand(LonghandId::FontSize),
                    AnimatableLonghand::FontSizeAdjust
                        => PropertyDeclarationId::Longhand(LonghandId::FontSizeAdjust),
                    AnimatableLonghand::FontSynthesis
                        => PropertyDeclarationId::Longhand(LonghandId::FontSynthesis),
                    AnimatableLonghand::FontStretch
                        => PropertyDeclarationId::Longhand(LonghandId::FontStretch),
                    AnimatableLonghand::FontKerning
                        => PropertyDeclarationId::Longhand(LonghandId::FontKerning),
                    AnimatableLonghand::FontVariantAlternates
                        => PropertyDeclarationId::Longhand(LonghandId::FontVariantAlternates),
                    AnimatableLonghand::FontVariantEastAsian
                        => PropertyDeclarationId::Longhand(LonghandId::FontVariantEastAsian),
                    AnimatableLonghand::FontVariantLigatures
                        => PropertyDeclarationId::Longhand(LonghandId::FontVariantLigatures),
                    AnimatableLonghand::FontVariantNumeric
                        => PropertyDeclarationId::Longhand(LonghandId::FontVariantNumeric),
                    AnimatableLonghand::FontVariantPosition
                        => PropertyDeclarationId::Longhand(LonghandId::FontVariantPosition),
                    AnimatableLonghand::FontFeatureSettings
                        => PropertyDeclarationId::Longhand(LonghandId::FontFeatureSettings),
                    AnimatableLonghand::FontVariationSettings
                        => PropertyDeclarationId::Longhand(LonghandId::FontVariationSettings),
                    AnimatableLonghand::FontLanguageOverride
                        => PropertyDeclarationId::Longhand(LonghandId::FontLanguageOverride),
                    AnimatableLonghand::MozOsxFontSmoothing
                        => PropertyDeclarationId::Longhand(LonghandId::MozOsxFontSmoothing),
                    AnimatableLonghand::Visibility
                        => PropertyDeclarationId::Longhand(LonghandId::Visibility),
                    AnimatableLonghand::WritingMode
                        => PropertyDeclarationId::Longhand(LonghandId::WritingMode),
                    AnimatableLonghand::Direction
                        => PropertyDeclarationId::Longhand(LonghandId::Direction),
                    AnimatableLonghand::TextOrientation
                        => PropertyDeclarationId::Longhand(LonghandId::TextOrientation),
                    AnimatableLonghand::ColorAdjust
                        => PropertyDeclarationId::Longhand(LonghandId::ColorAdjust),
                    AnimatableLonghand::ImageRendering
                        => PropertyDeclarationId::Longhand(LonghandId::ImageRendering),
                    AnimatableLonghand::ImageOrientation
                        => PropertyDeclarationId::Longhand(LonghandId::ImageOrientation),
                    AnimatableLonghand::BorderCollapse
                        => PropertyDeclarationId::Longhand(LonghandId::BorderCollapse),
                    AnimatableLonghand::EmptyCells
                        => PropertyDeclarationId::Longhand(LonghandId::EmptyCells),
                    AnimatableLonghand::CaptionSide
                        => PropertyDeclarationId::Longhand(LonghandId::CaptionSide),
                    AnimatableLonghand::BorderSpacing
                        => PropertyDeclarationId::Longhand(LonghandId::BorderSpacing),
                    AnimatableLonghand::LineHeight
                        => PropertyDeclarationId::Longhand(LonghandId::LineHeight),
                    AnimatableLonghand::TextTransform
                        => PropertyDeclarationId::Longhand(LonghandId::TextTransform),
                    AnimatableLonghand::Hyphens
                        => PropertyDeclarationId::Longhand(LonghandId::Hyphens),
                    AnimatableLonghand::MozTextSizeAdjust
                        => PropertyDeclarationId::Longhand(LonghandId::MozTextSizeAdjust),
                    AnimatableLonghand::TextIndent
                        => PropertyDeclarationId::Longhand(LonghandId::TextIndent),
                    AnimatableLonghand::OverflowWrap
                        => PropertyDeclarationId::Longhand(LonghandId::OverflowWrap),
                    AnimatableLonghand::WordBreak
                        => PropertyDeclarationId::Longhand(LonghandId::WordBreak),
                    AnimatableLonghand::TextJustify
                        => PropertyDeclarationId::Longhand(LonghandId::TextJustify),
                    AnimatableLonghand::TextAlignLast
                        => PropertyDeclarationId::Longhand(LonghandId::TextAlignLast),
                    AnimatableLonghand::TextAlign
                        => PropertyDeclarationId::Longhand(LonghandId::TextAlign),
                    AnimatableLonghand::LetterSpacing
                        => PropertyDeclarationId::Longhand(LonghandId::LetterSpacing),
                    AnimatableLonghand::WordSpacing
                        => PropertyDeclarationId::Longhand(LonghandId::WordSpacing),
                    AnimatableLonghand::WhiteSpace
                        => PropertyDeclarationId::Longhand(LonghandId::WhiteSpace),
                    AnimatableLonghand::TextShadow
                        => PropertyDeclarationId::Longhand(LonghandId::TextShadow),
                    AnimatableLonghand::TextEmphasisStyle
                        => PropertyDeclarationId::Longhand(LonghandId::TextEmphasisStyle),
                    AnimatableLonghand::TextEmphasisPosition
                        => PropertyDeclarationId::Longhand(LonghandId::TextEmphasisPosition),
                    AnimatableLonghand::TextEmphasisColor
                        => PropertyDeclarationId::Longhand(LonghandId::TextEmphasisColor),
                    AnimatableLonghand::MozTabSize
                        => PropertyDeclarationId::Longhand(LonghandId::MozTabSize),
                    AnimatableLonghand::WebkitTextFillColor
                        => PropertyDeclarationId::Longhand(LonghandId::WebkitTextFillColor),
                    AnimatableLonghand::WebkitTextStrokeColor
                        => PropertyDeclarationId::Longhand(LonghandId::WebkitTextStrokeColor),
                    AnimatableLonghand::WebkitTextStrokeWidth
                        => PropertyDeclarationId::Longhand(LonghandId::WebkitTextStrokeWidth),
                    AnimatableLonghand::RubyAlign
                        => PropertyDeclarationId::Longhand(LonghandId::RubyAlign),
                    AnimatableLonghand::RubyPosition
                        => PropertyDeclarationId::Longhand(LonghandId::RubyPosition),
                    AnimatableLonghand::TextCombineUpright
                        => PropertyDeclarationId::Longhand(LonghandId::TextCombineUpright),
                    AnimatableLonghand::TextRendering
                        => PropertyDeclarationId::Longhand(LonghandId::TextRendering),
                    AnimatableLonghand::ListStylePosition
                        => PropertyDeclarationId::Longhand(LonghandId::ListStylePosition),
                    AnimatableLonghand::ListStyleType
                        => PropertyDeclarationId::Longhand(LonghandId::ListStyleType),
                    AnimatableLonghand::ListStyleImage
                        => PropertyDeclarationId::Longhand(LonghandId::ListStyleImage),
                    AnimatableLonghand::Quotes
                        => PropertyDeclarationId::Longhand(LonghandId::Quotes),
                    AnimatableLonghand::MozImageRegion
                        => PropertyDeclarationId::Longhand(LonghandId::MozImageRegion),
                    AnimatableLonghand::MarginTop
                        => PropertyDeclarationId::Longhand(LonghandId::MarginTop),
                    AnimatableLonghand::MarginLeft
                        => PropertyDeclarationId::Longhand(LonghandId::MarginLeft),
                    AnimatableLonghand::MarginBottom
                        => PropertyDeclarationId::Longhand(LonghandId::MarginBottom),
                    AnimatableLonghand::MarginRight
                        => PropertyDeclarationId::Longhand(LonghandId::MarginRight),
                    AnimatableLonghand::OutlineColor
                        => PropertyDeclarationId::Longhand(LonghandId::OutlineColor),
                    AnimatableLonghand::OutlineStyle
                        => PropertyDeclarationId::Longhand(LonghandId::OutlineStyle),
                    AnimatableLonghand::OutlineWidth
                        => PropertyDeclarationId::Longhand(LonghandId::OutlineWidth),
                    AnimatableLonghand::MozOutlineRadiusTopleft
                        => PropertyDeclarationId::Longhand(LonghandId::MozOutlineRadiusTopleft),
                    AnimatableLonghand::MozOutlineRadiusTopright
                        => PropertyDeclarationId::Longhand(LonghandId::MozOutlineRadiusTopright),
                    AnimatableLonghand::MozOutlineRadiusBottomright
                        => PropertyDeclarationId::Longhand(LonghandId::MozOutlineRadiusBottomright),
                    AnimatableLonghand::MozOutlineRadiusBottomleft
                        => PropertyDeclarationId::Longhand(LonghandId::MozOutlineRadiusBottomleft),
                    AnimatableLonghand::OutlineOffset
                        => PropertyDeclarationId::Longhand(LonghandId::OutlineOffset),
                    AnimatableLonghand::PaddingTop
                        => PropertyDeclarationId::Longhand(LonghandId::PaddingTop),
                    AnimatableLonghand::PaddingLeft
                        => PropertyDeclarationId::Longhand(LonghandId::PaddingLeft),
                    AnimatableLonghand::PaddingBottom
                        => PropertyDeclarationId::Longhand(LonghandId::PaddingBottom),
                    AnimatableLonghand::PaddingRight
                        => PropertyDeclarationId::Longhand(LonghandId::PaddingRight),
                    AnimatableLonghand::Cursor
                        => PropertyDeclarationId::Longhand(LonghandId::Cursor),
                    AnimatableLonghand::PointerEvents
                        => PropertyDeclarationId::Longhand(LonghandId::PointerEvents),
                    AnimatableLonghand::MozUserInput
                        => PropertyDeclarationId::Longhand(LonghandId::MozUserInput),
                    AnimatableLonghand::MozUserModify
                        => PropertyDeclarationId::Longhand(LonghandId::MozUserModify),
                    AnimatableLonghand::MozUserFocus
                        => PropertyDeclarationId::Longhand(LonghandId::MozUserFocus),
                    AnimatableLonghand::CaretColor
                        => PropertyDeclarationId::Longhand(LonghandId::CaretColor),
                    AnimatableLonghand::Top
                        => PropertyDeclarationId::Longhand(LonghandId::Top),
                    AnimatableLonghand::Left
                        => PropertyDeclarationId::Longhand(LonghandId::Left),
                    AnimatableLonghand::Bottom
                        => PropertyDeclarationId::Longhand(LonghandId::Bottom),
                    AnimatableLonghand::Right
                        => PropertyDeclarationId::Longhand(LonghandId::Right),
                    AnimatableLonghand::ZIndex
                        => PropertyDeclarationId::Longhand(LonghandId::ZIndex),
                    AnimatableLonghand::FlexDirection
                        => PropertyDeclarationId::Longhand(LonghandId::FlexDirection),
                    AnimatableLonghand::FlexWrap
                        => PropertyDeclarationId::Longhand(LonghandId::FlexWrap),
                    AnimatableLonghand::JustifyContent
                        => PropertyDeclarationId::Longhand(LonghandId::JustifyContent),
                    AnimatableLonghand::AlignContent
                        => PropertyDeclarationId::Longhand(LonghandId::AlignContent),
                    AnimatableLonghand::AlignItems
                        => PropertyDeclarationId::Longhand(LonghandId::AlignItems),
                    AnimatableLonghand::JustifyItems
                        => PropertyDeclarationId::Longhand(LonghandId::JustifyItems),
                    AnimatableLonghand::FlexGrow
                        => PropertyDeclarationId::Longhand(LonghandId::FlexGrow),
                    AnimatableLonghand::FlexShrink
                        => PropertyDeclarationId::Longhand(LonghandId::FlexShrink),
                    AnimatableLonghand::AlignSelf
                        => PropertyDeclarationId::Longhand(LonghandId::AlignSelf),
                    AnimatableLonghand::JustifySelf
                        => PropertyDeclarationId::Longhand(LonghandId::JustifySelf),
                    AnimatableLonghand::Order
                        => PropertyDeclarationId::Longhand(LonghandId::Order),
                    AnimatableLonghand::FlexBasis
                        => PropertyDeclarationId::Longhand(LonghandId::FlexBasis),
                    AnimatableLonghand::Width
                        => PropertyDeclarationId::Longhand(LonghandId::Width),
                    AnimatableLonghand::MinWidth
                        => PropertyDeclarationId::Longhand(LonghandId::MinWidth),
                    AnimatableLonghand::MaxWidth
                        => PropertyDeclarationId::Longhand(LonghandId::MaxWidth),
                    AnimatableLonghand::Height
                        => PropertyDeclarationId::Longhand(LonghandId::Height),
                    AnimatableLonghand::MinHeight
                        => PropertyDeclarationId::Longhand(LonghandId::MinHeight),
                    AnimatableLonghand::MaxHeight
                        => PropertyDeclarationId::Longhand(LonghandId::MaxHeight),
                    AnimatableLonghand::BoxSizing
                        => PropertyDeclarationId::Longhand(LonghandId::BoxSizing),
                    AnimatableLonghand::ObjectFit
                        => PropertyDeclarationId::Longhand(LonghandId::ObjectFit),
                    AnimatableLonghand::ObjectPosition
                        => PropertyDeclarationId::Longhand(LonghandId::ObjectPosition),
                    AnimatableLonghand::GridRowGap
                        => PropertyDeclarationId::Longhand(LonghandId::GridRowGap),
                    AnimatableLonghand::GridRowStart
                        => PropertyDeclarationId::Longhand(LonghandId::GridRowStart),
                    AnimatableLonghand::GridRowEnd
                        => PropertyDeclarationId::Longhand(LonghandId::GridRowEnd),
                    AnimatableLonghand::GridAutoRows
                        => PropertyDeclarationId::Longhand(LonghandId::GridAutoRows),
                    AnimatableLonghand::GridTemplateRows
                        => PropertyDeclarationId::Longhand(LonghandId::GridTemplateRows),
                    AnimatableLonghand::GridColumnGap
                        => PropertyDeclarationId::Longhand(LonghandId::GridColumnGap),
                    AnimatableLonghand::GridColumnStart
                        => PropertyDeclarationId::Longhand(LonghandId::GridColumnStart),
                    AnimatableLonghand::GridColumnEnd
                        => PropertyDeclarationId::Longhand(LonghandId::GridColumnEnd),
                    AnimatableLonghand::GridAutoColumns
                        => PropertyDeclarationId::Longhand(LonghandId::GridAutoColumns),
                    AnimatableLonghand::GridTemplateColumns
                        => PropertyDeclarationId::Longhand(LonghandId::GridTemplateColumns),
                    AnimatableLonghand::GridAutoFlow
                        => PropertyDeclarationId::Longhand(LonghandId::GridAutoFlow),
                    AnimatableLonghand::GridTemplateAreas
                        => PropertyDeclarationId::Longhand(LonghandId::GridTemplateAreas),
                    AnimatableLonghand::TableLayout
                        => PropertyDeclarationId::Longhand(LonghandId::TableLayout),
                    AnimatableLonghand::TextOverflow
                        => PropertyDeclarationId::Longhand(LonghandId::TextOverflow),
                    AnimatableLonghand::UnicodeBidi
                        => PropertyDeclarationId::Longhand(LonghandId::UnicodeBidi),
                    AnimatableLonghand::TextDecorationLine
                        => PropertyDeclarationId::Longhand(LonghandId::TextDecorationLine),
                    AnimatableLonghand::TextDecorationStyle
                        => PropertyDeclarationId::Longhand(LonghandId::TextDecorationStyle),
                    AnimatableLonghand::TextDecorationColor
                        => PropertyDeclarationId::Longhand(LonghandId::TextDecorationColor),
                    AnimatableLonghand::InitialLetter
                        => PropertyDeclarationId::Longhand(LonghandId::InitialLetter),
                    AnimatableLonghand::ImeMode
                        => PropertyDeclarationId::Longhand(LonghandId::ImeMode),
                    AnimatableLonghand::MozUserSelect
                        => PropertyDeclarationId::Longhand(LonghandId::MozUserSelect),
                    AnimatableLonghand::MozWindowDragging
                        => PropertyDeclarationId::Longhand(LonghandId::MozWindowDragging),
                    AnimatableLonghand::MozWindowShadow
                        => PropertyDeclarationId::Longhand(LonghandId::MozWindowShadow),
                    AnimatableLonghand::MozForceBrokenImageIcon
                        => PropertyDeclarationId::Longhand(LonghandId::MozForceBrokenImageIcon),
                    AnimatableLonghand::TextAnchor
                        => PropertyDeclarationId::Longhand(LonghandId::TextAnchor),
                    AnimatableLonghand::ColorInterpolation
                        => PropertyDeclarationId::Longhand(LonghandId::ColorInterpolation),
                    AnimatableLonghand::ColorInterpolationFilters
                        => PropertyDeclarationId::Longhand(LonghandId::ColorInterpolationFilters),
                    AnimatableLonghand::Fill
                        => PropertyDeclarationId::Longhand(LonghandId::Fill),
                    AnimatableLonghand::FillOpacity
                        => PropertyDeclarationId::Longhand(LonghandId::FillOpacity),
                    AnimatableLonghand::FillRule
                        => PropertyDeclarationId::Longhand(LonghandId::FillRule),
                    AnimatableLonghand::ShapeRendering
                        => PropertyDeclarationId::Longhand(LonghandId::ShapeRendering),
                    AnimatableLonghand::Stroke
                        => PropertyDeclarationId::Longhand(LonghandId::Stroke),
                    AnimatableLonghand::StrokeWidth
                        => PropertyDeclarationId::Longhand(LonghandId::StrokeWidth),
                    AnimatableLonghand::StrokeLinecap
                        => PropertyDeclarationId::Longhand(LonghandId::StrokeLinecap),
                    AnimatableLonghand::StrokeLinejoin
                        => PropertyDeclarationId::Longhand(LonghandId::StrokeLinejoin),
                    AnimatableLonghand::StrokeMiterlimit
                        => PropertyDeclarationId::Longhand(LonghandId::StrokeMiterlimit),
                    AnimatableLonghand::StrokeOpacity
                        => PropertyDeclarationId::Longhand(LonghandId::StrokeOpacity),
                    AnimatableLonghand::StrokeDasharray
                        => PropertyDeclarationId::Longhand(LonghandId::StrokeDasharray),
                    AnimatableLonghand::StrokeDashoffset
                        => PropertyDeclarationId::Longhand(LonghandId::StrokeDashoffset),
                    AnimatableLonghand::ClipRule
                        => PropertyDeclarationId::Longhand(LonghandId::ClipRule),
                    AnimatableLonghand::MarkerStart
                        => PropertyDeclarationId::Longhand(LonghandId::MarkerStart),
                    AnimatableLonghand::MarkerMid
                        => PropertyDeclarationId::Longhand(LonghandId::MarkerMid),
                    AnimatableLonghand::MarkerEnd
                        => PropertyDeclarationId::Longhand(LonghandId::MarkerEnd),
                    AnimatableLonghand::PaintOrder
                        => PropertyDeclarationId::Longhand(LonghandId::PaintOrder),
                    AnimatableLonghand::DominantBaseline
                        => PropertyDeclarationId::Longhand(LonghandId::DominantBaseline),
                    AnimatableLonghand::VectorEffect
                        => PropertyDeclarationId::Longhand(LonghandId::VectorEffect),
                    AnimatableLonghand::StopColor
                        => PropertyDeclarationId::Longhand(LonghandId::StopColor),
                    AnimatableLonghand::StopOpacity
                        => PropertyDeclarationId::Longhand(LonghandId::StopOpacity),
                    AnimatableLonghand::FloodColor
                        => PropertyDeclarationId::Longhand(LonghandId::FloodColor),
                    AnimatableLonghand::FloodOpacity
                        => PropertyDeclarationId::Longhand(LonghandId::FloodOpacity),
                    AnimatableLonghand::LightingColor
                        => PropertyDeclarationId::Longhand(LonghandId::LightingColor),
                    AnimatableLonghand::MaskType
                        => PropertyDeclarationId::Longhand(LonghandId::MaskType),
                    AnimatableLonghand::ClipPath
                        => PropertyDeclarationId::Longhand(LonghandId::ClipPath),
                    AnimatableLonghand::MaskMode
                        => PropertyDeclarationId::Longhand(LonghandId::MaskMode),
                    AnimatableLonghand::MaskRepeat
                        => PropertyDeclarationId::Longhand(LonghandId::MaskRepeat),
                    AnimatableLonghand::MaskPositionX
                        => PropertyDeclarationId::Longhand(LonghandId::MaskPositionX),
                    AnimatableLonghand::MaskPositionY
                        => PropertyDeclarationId::Longhand(LonghandId::MaskPositionY),
                    AnimatableLonghand::MaskClip
                        => PropertyDeclarationId::Longhand(LonghandId::MaskClip),
                    AnimatableLonghand::MaskOrigin
                        => PropertyDeclarationId::Longhand(LonghandId::MaskOrigin),
                    AnimatableLonghand::MaskSize
                        => PropertyDeclarationId::Longhand(LonghandId::MaskSize),
                    AnimatableLonghand::MaskComposite
                        => PropertyDeclarationId::Longhand(LonghandId::MaskComposite),
                    AnimatableLonghand::MaskImage
                        => PropertyDeclarationId::Longhand(LonghandId::MaskImage),
                    AnimatableLonghand::MozBoxAlign
                        => PropertyDeclarationId::Longhand(LonghandId::MozBoxAlign),
                    AnimatableLonghand::MozBoxDirection
                        => PropertyDeclarationId::Longhand(LonghandId::MozBoxDirection),
                    AnimatableLonghand::MozBoxFlex
                        => PropertyDeclarationId::Longhand(LonghandId::MozBoxFlex),
                    AnimatableLonghand::MozBoxOrient
                        => PropertyDeclarationId::Longhand(LonghandId::MozBoxOrient),
                    AnimatableLonghand::MozBoxPack
                        => PropertyDeclarationId::Longhand(LonghandId::MozBoxPack),
                    AnimatableLonghand::MozStackSizing
                        => PropertyDeclarationId::Longhand(LonghandId::MozStackSizing),
                    AnimatableLonghand::MozBoxOrdinalGroup
                        => PropertyDeclarationId::Longhand(LonghandId::MozBoxOrdinalGroup),
        }
    }
}

/// Returns true if this nsCSSPropertyID is one of the animatable properties.
#[cfg(feature = "gecko")]
pub fn nscsspropertyid_is_animatable(property: nsCSSPropertyID) -> bool {
    match property {
                
    nsCSSPropertyID::eCSSProperty_background_color => true,
                
    nsCSSPropertyID::eCSSProperty_background_image => true,
                
    nsCSSPropertyID::eCSSProperty_background_position_x => true,
                
    nsCSSPropertyID::eCSSProperty_background_position_y => true,
                
    nsCSSPropertyID::eCSSProperty_background_repeat => true,
                
    nsCSSPropertyID::eCSSProperty_background_attachment => true,
                
    nsCSSPropertyID::eCSSProperty_background_clip => true,
                
    nsCSSPropertyID::eCSSProperty_background_origin => true,
                
    nsCSSPropertyID::eCSSProperty_background_size => true,
                
    nsCSSPropertyID::eCSSProperty_background_blend_mode => true,
                
    nsCSSPropertyID::eCSSProperty_border_top_color => true,
                
    nsCSSPropertyID::eCSSProperty_border_top_style => true,
                
    nsCSSPropertyID::eCSSProperty_border_top_width => true,
                
    nsCSSPropertyID::eCSSProperty_border_left_color => true,
                
    nsCSSPropertyID::eCSSProperty_border_left_style => true,
                
    nsCSSPropertyID::eCSSProperty_border_left_width => true,
                
    nsCSSPropertyID::eCSSProperty_border_bottom_color => true,
                
    nsCSSPropertyID::eCSSProperty_border_bottom_style => true,
                
    nsCSSPropertyID::eCSSProperty_border_bottom_width => true,
                
    nsCSSPropertyID::eCSSProperty_border_right_color => true,
                
    nsCSSPropertyID::eCSSProperty_border_right_style => true,
                
    nsCSSPropertyID::eCSSProperty_border_right_width => true,
                
    nsCSSPropertyID::eCSSProperty_border_top_left_radius => true,
                
    nsCSSPropertyID::eCSSProperty_border_top_right_radius => true,
                
    nsCSSPropertyID::eCSSProperty_border_bottom_right_radius => true,
                
    nsCSSPropertyID::eCSSProperty_border_bottom_left_radius => true,
                
    nsCSSPropertyID::eCSSProperty__moz_border_top_colors => true,
                
    nsCSSPropertyID::eCSSProperty__moz_border_left_colors => true,
                
    nsCSSPropertyID::eCSSProperty__moz_border_bottom_colors => true,
                
    nsCSSPropertyID::eCSSProperty__moz_border_right_colors => true,
                
    nsCSSPropertyID::eCSSProperty_box_decoration_break => true,
                
    nsCSSPropertyID::eCSSProperty__moz_float_edge => true,
                
    nsCSSPropertyID::eCSSProperty_border_image_source => true,
                
    nsCSSPropertyID::eCSSProperty_border_image_outset => true,
                
    nsCSSPropertyID::eCSSProperty_border_image_repeat => true,
                
    nsCSSPropertyID::eCSSProperty_border_image_width => true,
                
    nsCSSPropertyID::eCSSProperty_border_image_slice => true,
                
    nsCSSPropertyID::eCSSProperty_display => true,
                
    nsCSSPropertyID::eCSSProperty_position => true,
                
    nsCSSPropertyID::eCSSProperty_float_ => true,
                
    nsCSSPropertyID::eCSSProperty_clear => true,
                
    nsCSSPropertyID::eCSSProperty_vertical_align => true,
                
    nsCSSPropertyID::eCSSProperty_overflow_clip_box => true,
                
    nsCSSPropertyID::eCSSProperty_overflow_x => true,
                
    nsCSSPropertyID::eCSSProperty_overflow_y => true,
                
    nsCSSPropertyID::eCSSProperty_scroll_snap_points_x => true,
                
    nsCSSPropertyID::eCSSProperty_scroll_snap_points_y => true,
                
    nsCSSPropertyID::eCSSProperty_scroll_snap_destination => true,
                
    nsCSSPropertyID::eCSSProperty_scroll_snap_coordinate => true,
                
    nsCSSPropertyID::eCSSProperty_transform => true,
                
    nsCSSPropertyID::eCSSProperty_scroll_behavior => true,
                
    nsCSSPropertyID::eCSSProperty_scroll_snap_type_x => true,
                
    nsCSSPropertyID::eCSSProperty_scroll_snap_type_y => true,
                
    nsCSSPropertyID::eCSSProperty_isolation => true,
                
    nsCSSPropertyID::eCSSProperty_page_break_after => true,
                
    nsCSSPropertyID::eCSSProperty_page_break_before => true,
                
    nsCSSPropertyID::eCSSProperty_page_break_inside => true,
                
    nsCSSPropertyID::eCSSProperty_resize => true,
                
    nsCSSPropertyID::eCSSProperty_perspective => true,
                
    nsCSSPropertyID::eCSSProperty_perspective_origin => true,
                
    nsCSSPropertyID::eCSSProperty_backface_visibility => true,
                
    nsCSSPropertyID::eCSSProperty_transform_box => true,
                
    nsCSSPropertyID::eCSSProperty_transform_style => true,
                
    nsCSSPropertyID::eCSSProperty_transform_origin => true,
                
    nsCSSPropertyID::eCSSProperty_contain => true,
                
    nsCSSPropertyID::eCSSProperty__moz_appearance => true,
                
    nsCSSPropertyID::eCSSProperty__moz_orient => true,
                
    nsCSSPropertyID::eCSSProperty_will_change => true,
                
    nsCSSPropertyID::eCSSProperty_shape_outside => true,
                
    nsCSSPropertyID::eCSSProperty_touch_action => true,
                
    nsCSSPropertyID::eCSSProperty_color => true,
                
    nsCSSPropertyID::eCSSProperty_column_width => true,
                
    nsCSSPropertyID::eCSSProperty_column_count => true,
                
    nsCSSPropertyID::eCSSProperty_column_gap => true,
                
    nsCSSPropertyID::eCSSProperty_column_fill => true,
                
    nsCSSPropertyID::eCSSProperty_column_rule_width => true,
                
    nsCSSPropertyID::eCSSProperty_column_rule_color => true,
                
    nsCSSPropertyID::eCSSProperty_column_span => true,
                
    nsCSSPropertyID::eCSSProperty_column_rule_style => true,
                
    nsCSSPropertyID::eCSSProperty_content => true,
                
    nsCSSPropertyID::eCSSProperty_counter_increment => true,
                
    nsCSSPropertyID::eCSSProperty_counter_reset => true,
                
    nsCSSPropertyID::eCSSProperty_opacity => true,
                
    nsCSSPropertyID::eCSSProperty_box_shadow => true,
                
    nsCSSPropertyID::eCSSProperty_clip => true,
                
    nsCSSPropertyID::eCSSProperty_filter => true,
                
    nsCSSPropertyID::eCSSProperty_mix_blend_mode => true,
                
    nsCSSPropertyID::eCSSProperty_font_family => true,
                
    nsCSSPropertyID::eCSSProperty_font_style => true,
                
    nsCSSPropertyID::eCSSProperty_font_variant_caps => true,
                
    nsCSSPropertyID::eCSSProperty_font_weight => true,
                
    nsCSSPropertyID::eCSSProperty_font_size => true,
                
    nsCSSPropertyID::eCSSProperty_font_size_adjust => true,
                
    nsCSSPropertyID::eCSSProperty_font_synthesis => true,
                
    nsCSSPropertyID::eCSSProperty_font_stretch => true,
                
    nsCSSPropertyID::eCSSProperty_font_kerning => true,
                
    nsCSSPropertyID::eCSSProperty_font_variant_alternates => true,
                
    nsCSSPropertyID::eCSSProperty_font_variant_east_asian => true,
                
    nsCSSPropertyID::eCSSProperty_font_variant_ligatures => true,
                
    nsCSSPropertyID::eCSSProperty_font_variant_numeric => true,
                
    nsCSSPropertyID::eCSSProperty_font_variant_position => true,
                
    nsCSSPropertyID::eCSSProperty_font_feature_settings => true,
                
    nsCSSPropertyID::eCSSProperty_font_variation_settings => true,
                
    nsCSSPropertyID::eCSSProperty_font_language_override => true,
                
    nsCSSPropertyID::eCSSProperty__moz_osx_font_smoothing => true,
                
    nsCSSPropertyID::eCSSProperty_visibility => true,
                
    nsCSSPropertyID::eCSSProperty_writing_mode => true,
                
    nsCSSPropertyID::eCSSProperty_direction => true,
                
    nsCSSPropertyID::eCSSProperty_text_orientation => true,
                
    nsCSSPropertyID::eCSSProperty_color_adjust => true,
                
    nsCSSPropertyID::eCSSProperty_image_rendering => true,
                
    nsCSSPropertyID::eCSSProperty_image_orientation => true,
                
    nsCSSPropertyID::eCSSProperty_border_collapse => true,
                
    nsCSSPropertyID::eCSSProperty_empty_cells => true,
                
    nsCSSPropertyID::eCSSProperty_caption_side => true,
                
    nsCSSPropertyID::eCSSProperty_border_spacing => true,
                
    nsCSSPropertyID::eCSSProperty_line_height => true,
                
    nsCSSPropertyID::eCSSProperty_text_transform => true,
                
    nsCSSPropertyID::eCSSProperty_hyphens => true,
                
    nsCSSPropertyID::eCSSProperty__moz_text_size_adjust => true,
                
    nsCSSPropertyID::eCSSProperty_text_indent => true,
                
    nsCSSPropertyID::eCSSProperty_overflow_wrap => true,
                
    nsCSSPropertyID::eCSSProperty_word_break => true,
                
    nsCSSPropertyID::eCSSProperty_text_justify => true,
                
    nsCSSPropertyID::eCSSProperty_text_align_last => true,
                
    nsCSSPropertyID::eCSSProperty_text_align => true,
                
    nsCSSPropertyID::eCSSProperty_letter_spacing => true,
                
    nsCSSPropertyID::eCSSProperty_word_spacing => true,
                
    nsCSSPropertyID::eCSSProperty_white_space => true,
                
    nsCSSPropertyID::eCSSProperty_text_shadow => true,
                
    nsCSSPropertyID::eCSSProperty_text_emphasis_style => true,
                
    nsCSSPropertyID::eCSSProperty_text_emphasis_position => true,
                
    nsCSSPropertyID::eCSSProperty_text_emphasis_color => true,
                
    nsCSSPropertyID::eCSSProperty__moz_tab_size => true,
                
    nsCSSPropertyID::eCSSProperty__webkit_text_fill_color => true,
                
    nsCSSPropertyID::eCSSProperty__webkit_text_stroke_color => true,
                
    nsCSSPropertyID::eCSSProperty__webkit_text_stroke_width => true,
                
    nsCSSPropertyID::eCSSProperty_ruby_align => true,
                
    nsCSSPropertyID::eCSSProperty_ruby_position => true,
                
    nsCSSPropertyID::eCSSProperty_text_combine_upright => true,
                
    nsCSSPropertyID::eCSSProperty_text_rendering => true,
                
    nsCSSPropertyID::eCSSProperty_list_style_position => true,
                
    nsCSSPropertyID::eCSSProperty_list_style_type => true,
                
    nsCSSPropertyID::eCSSProperty_list_style_image => true,
                
    nsCSSPropertyID::eCSSProperty_quotes => true,
                
    nsCSSPropertyID::eCSSProperty__moz_image_region => true,
                
    nsCSSPropertyID::eCSSProperty_margin_top => true,
                
    nsCSSPropertyID::eCSSProperty_margin_left => true,
                
    nsCSSPropertyID::eCSSProperty_margin_bottom => true,
                
    nsCSSPropertyID::eCSSProperty_margin_right => true,
                
    nsCSSPropertyID::eCSSProperty_outline_color => true,
                
    nsCSSPropertyID::eCSSProperty_outline_style => true,
                
    nsCSSPropertyID::eCSSProperty_outline_width => true,
                
    nsCSSPropertyID::eCSSProperty__moz_outline_radius_topleft => true,
                
    nsCSSPropertyID::eCSSProperty__moz_outline_radius_topright => true,
                
    nsCSSPropertyID::eCSSProperty__moz_outline_radius_bottomright => true,
                
    nsCSSPropertyID::eCSSProperty__moz_outline_radius_bottomleft => true,
                
    nsCSSPropertyID::eCSSProperty_outline_offset => true,
                
    nsCSSPropertyID::eCSSProperty_padding_top => true,
                
    nsCSSPropertyID::eCSSProperty_padding_left => true,
                
    nsCSSPropertyID::eCSSProperty_padding_bottom => true,
                
    nsCSSPropertyID::eCSSProperty_padding_right => true,
                
    nsCSSPropertyID::eCSSProperty_cursor => true,
                
    nsCSSPropertyID::eCSSProperty_pointer_events => true,
                
    nsCSSPropertyID::eCSSProperty__moz_user_input => true,
                
    nsCSSPropertyID::eCSSProperty__moz_user_modify => true,
                
    nsCSSPropertyID::eCSSProperty__moz_user_focus => true,
                
    nsCSSPropertyID::eCSSProperty_caret_color => true,
                
    nsCSSPropertyID::eCSSProperty_top => true,
                
    nsCSSPropertyID::eCSSProperty_left => true,
                
    nsCSSPropertyID::eCSSProperty_bottom => true,
                
    nsCSSPropertyID::eCSSProperty_right => true,
                
    nsCSSPropertyID::eCSSProperty_z_index => true,
                
    nsCSSPropertyID::eCSSProperty_flex_direction => true,
                
    nsCSSPropertyID::eCSSProperty_flex_wrap => true,
                
    nsCSSPropertyID::eCSSProperty_justify_content => true,
                
    nsCSSPropertyID::eCSSProperty_align_content => true,
                
    nsCSSPropertyID::eCSSProperty_align_items => true,
                
    nsCSSPropertyID::eCSSProperty_justify_items => true,
                
    nsCSSPropertyID::eCSSProperty_flex_grow => true,
                
    nsCSSPropertyID::eCSSProperty_flex_shrink => true,
                
    nsCSSPropertyID::eCSSProperty_align_self => true,
                
    nsCSSPropertyID::eCSSProperty_justify_self => true,
                
    nsCSSPropertyID::eCSSProperty_order => true,
                
    nsCSSPropertyID::eCSSProperty_flex_basis => true,
                
    nsCSSPropertyID::eCSSProperty_width => true,
                
    nsCSSPropertyID::eCSSProperty_min_width => true,
                
    nsCSSPropertyID::eCSSProperty_max_width => true,
                
    nsCSSPropertyID::eCSSProperty_height => true,
                
    nsCSSPropertyID::eCSSProperty_min_height => true,
                
    nsCSSPropertyID::eCSSProperty_max_height => true,
                
    nsCSSPropertyID::eCSSProperty_box_sizing => true,
                
    nsCSSPropertyID::eCSSProperty_object_fit => true,
                
    nsCSSPropertyID::eCSSProperty_object_position => true,
                
    nsCSSPropertyID::eCSSProperty_grid_row_gap => true,
                
    nsCSSPropertyID::eCSSProperty_grid_row_start => true,
                
    nsCSSPropertyID::eCSSProperty_grid_row_end => true,
                
    nsCSSPropertyID::eCSSProperty_grid_auto_rows => true,
                
    nsCSSPropertyID::eCSSProperty_grid_template_rows => true,
                
    nsCSSPropertyID::eCSSProperty_grid_column_gap => true,
                
    nsCSSPropertyID::eCSSProperty_grid_column_start => true,
                
    nsCSSPropertyID::eCSSProperty_grid_column_end => true,
                
    nsCSSPropertyID::eCSSProperty_grid_auto_columns => true,
                
    nsCSSPropertyID::eCSSProperty_grid_template_columns => true,
                
    nsCSSPropertyID::eCSSProperty_grid_auto_flow => true,
                
    nsCSSPropertyID::eCSSProperty_grid_template_areas => true,
                
    nsCSSPropertyID::eCSSProperty_table_layout => true,
                
    nsCSSPropertyID::eCSSProperty_text_overflow => true,
                
    nsCSSPropertyID::eCSSProperty_unicode_bidi => true,
                
    nsCSSPropertyID::eCSSProperty_text_decoration_line => true,
                
    nsCSSPropertyID::eCSSProperty_text_decoration_style => true,
                
    nsCSSPropertyID::eCSSProperty_text_decoration_color => true,
                
    nsCSSPropertyID::eCSSProperty_initial_letter => true,
                
    nsCSSPropertyID::eCSSProperty_ime_mode => true,
                
    nsCSSPropertyID::eCSSProperty__moz_user_select => true,
                
    nsCSSPropertyID::eCSSProperty__moz_window_dragging => true,
                
    nsCSSPropertyID::eCSSProperty__moz_window_shadow => true,
                
    nsCSSPropertyID::eCSSProperty__moz_force_broken_image_icon => true,
                
    nsCSSPropertyID::eCSSProperty_text_anchor => true,
                
    nsCSSPropertyID::eCSSProperty_color_interpolation => true,
                
    nsCSSPropertyID::eCSSProperty_color_interpolation_filters => true,
                
    nsCSSPropertyID::eCSSProperty_fill => true,
                
    nsCSSPropertyID::eCSSProperty_fill_opacity => true,
                
    nsCSSPropertyID::eCSSProperty_fill_rule => true,
                
    nsCSSPropertyID::eCSSProperty_shape_rendering => true,
                
    nsCSSPropertyID::eCSSProperty_stroke => true,
                
    nsCSSPropertyID::eCSSProperty_stroke_width => true,
                
    nsCSSPropertyID::eCSSProperty_stroke_linecap => true,
                
    nsCSSPropertyID::eCSSProperty_stroke_linejoin => true,
                
    nsCSSPropertyID::eCSSProperty_stroke_miterlimit => true,
                
    nsCSSPropertyID::eCSSProperty_stroke_opacity => true,
                
    nsCSSPropertyID::eCSSProperty_stroke_dasharray => true,
                
    nsCSSPropertyID::eCSSProperty_stroke_dashoffset => true,
                
    nsCSSPropertyID::eCSSProperty_clip_rule => true,
                
    nsCSSPropertyID::eCSSProperty_marker_start => true,
                
    nsCSSPropertyID::eCSSProperty_marker_mid => true,
                
    nsCSSPropertyID::eCSSProperty_marker_end => true,
                
    nsCSSPropertyID::eCSSProperty_paint_order => true,
                
    nsCSSPropertyID::eCSSProperty_dominant_baseline => true,
                
    nsCSSPropertyID::eCSSProperty_vector_effect => true,
                
    nsCSSPropertyID::eCSSProperty_stop_color => true,
                
    nsCSSPropertyID::eCSSProperty_stop_opacity => true,
                
    nsCSSPropertyID::eCSSProperty_flood_color => true,
                
    nsCSSPropertyID::eCSSProperty_flood_opacity => true,
                
    nsCSSPropertyID::eCSSProperty_lighting_color => true,
                
    nsCSSPropertyID::eCSSProperty_mask_type => true,
                
    nsCSSPropertyID::eCSSProperty_clip_path => true,
                
    nsCSSPropertyID::eCSSProperty_mask_mode => true,
                
    nsCSSPropertyID::eCSSProperty_mask_repeat => true,
                
    nsCSSPropertyID::eCSSProperty_mask_position_x => true,
                
    nsCSSPropertyID::eCSSProperty_mask_position_y => true,
                
    nsCSSPropertyID::eCSSProperty_mask_clip => true,
                
    nsCSSPropertyID::eCSSProperty_mask_origin => true,
                
    nsCSSPropertyID::eCSSProperty_mask_size => true,
                
    nsCSSPropertyID::eCSSProperty_mask_composite => true,
                
    nsCSSPropertyID::eCSSProperty_mask_image => true,
                
    nsCSSPropertyID::eCSSProperty__moz_box_align => true,
                
    nsCSSPropertyID::eCSSProperty__moz_box_direction => true,
                
    nsCSSPropertyID::eCSSProperty__moz_box_flex => true,
                
    nsCSSPropertyID::eCSSProperty__moz_box_orient => true,
                
    nsCSSPropertyID::eCSSProperty__moz_box_pack => true,
                
    nsCSSPropertyID::eCSSProperty__moz_stack_sizing => true,
                
    nsCSSPropertyID::eCSSProperty__moz_box_ordinal_group => true,
                
    nsCSSPropertyID::eCSSProperty_background => true,
                
    nsCSSPropertyID::eCSSProperty_background_position => true,
                
    nsCSSPropertyID::eCSSProperty_border_color => true,
                
    nsCSSPropertyID::eCSSProperty_border_style => true,
                
    nsCSSPropertyID::eCSSProperty_border_width => true,
                
    nsCSSPropertyID::eCSSProperty_border_top => true,
                
    nsCSSPropertyID::eCSSProperty_border_left => true,
                
    nsCSSPropertyID::eCSSProperty_border_bottom => true,
                
    nsCSSPropertyID::eCSSProperty_border_right => true,
                
    nsCSSPropertyID::eCSSProperty_border => true,
                
    nsCSSPropertyID::eCSSProperty_border_radius => true,
                
    nsCSSPropertyID::eCSSProperty_border_image => true,
                
    nsCSSPropertyID::eCSSProperty_overflow => true,
                
    nsCSSPropertyID::eCSSProperty_scroll_snap_type => true,
                
    nsCSSPropertyID::eCSSProperty__moz_transform => true,
                
    nsCSSPropertyID::eCSSProperty_columns => true,
                
    nsCSSPropertyID::eCSSProperty_column_rule => true,
                
    nsCSSPropertyID::eCSSProperty_font => true,
                
    nsCSSPropertyID::eCSSProperty_font_variant => true,
                
    nsCSSPropertyID::eCSSProperty_text_emphasis => true,
                
    nsCSSPropertyID::eCSSProperty__webkit_text_stroke => true,
                
    nsCSSPropertyID::eCSSProperty_list_style => true,
                
    nsCSSPropertyID::eCSSProperty_margin => true,
                
    nsCSSPropertyID::eCSSProperty_mask => true,
                
    nsCSSPropertyID::eCSSProperty_mask_position => true,
                
    nsCSSPropertyID::eCSSProperty_outline => true,
                
    nsCSSPropertyID::eCSSProperty__moz_outline_radius => true,
                
    nsCSSPropertyID::eCSSProperty_padding => true,
                
    nsCSSPropertyID::eCSSProperty_flex_flow => true,
                
    nsCSSPropertyID::eCSSProperty_flex => true,
                
    nsCSSPropertyID::eCSSProperty_grid_gap => true,
                
    nsCSSPropertyID::eCSSProperty_grid_row => true,
                
    nsCSSPropertyID::eCSSProperty_grid_column => true,
                
    nsCSSPropertyID::eCSSProperty_grid_area => true,
                
    nsCSSPropertyID::eCSSProperty_grid_template => true,
                
    nsCSSPropertyID::eCSSProperty_grid => true,
                
    nsCSSPropertyID::eCSSProperty_place_content => true,
                
    nsCSSPropertyID::eCSSProperty_place_self => true,
                
    nsCSSPropertyID::eCSSProperty_place_items => true,
                
    nsCSSPropertyID::eCSSProperty_marker => true,
                
    nsCSSPropertyID::eCSSProperty_text_decoration => true,
        _ => false
    }
}

/// A given transition property, that is either `All`, a transitionable longhand property,
/// a shorthand with at least one transitionable longhand component, or an unsupported property.
// NB: This needs to be here because it needs all the longhands generated
// beforehand.
#[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
#[cfg_attr(feature = "servo", derive(HeapSizeOf))]
#[derive(Clone, Eq, Hash, PartialEq, ToCss, ToComputedValue)]
pub enum TransitionProperty {
    /// All, any transitionable property changing should generate a transition.
    All,
            /// background-color
            BackgroundColor,
            /// background-position-x
            BackgroundPositionX,
            /// background-position-y
            BackgroundPositionY,
            /// background-size
            BackgroundSize,
            /// border-top-color
            BorderTopColor,
            /// border-top-width
            BorderTopWidth,
            /// border-left-color
            BorderLeftColor,
            /// border-left-width
            BorderLeftWidth,
            /// border-bottom-color
            BorderBottomColor,
            /// border-bottom-width
            BorderBottomWidth,
            /// border-right-color
            BorderRightColor,
            /// border-right-width
            BorderRightWidth,
            /// border-top-left-radius
            BorderTopLeftRadius,
            /// border-top-right-radius
            BorderTopRightRadius,
            /// border-bottom-right-radius
            BorderBottomRightRadius,
            /// border-bottom-left-radius
            BorderBottomLeftRadius,
            /// vertical-align
            VerticalAlign,
            /// transform
            Transform,
            /// perspective
            Perspective,
            /// perspective-origin
            PerspectiveOrigin,
            /// transform-origin
            TransformOrigin,
            /// shape-outside
            ShapeOutside,
            /// color
            Color,
            /// column-width
            ColumnWidth,
            /// column-count
            ColumnCount,
            /// column-gap
            ColumnGap,
            /// column-rule-width
            ColumnRuleWidth,
            /// column-rule-color
            ColumnRuleColor,
            /// opacity
            Opacity,
            /// box-shadow
            BoxShadow,
            /// clip
            Clip,
            /// filter
            Filter,
            /// font-weight
            FontWeight,
            /// font-size
            FontSize,
            /// font-size-adjust
            FontSizeAdjust,
            /// font-stretch
            FontStretch,
            /// font-variation-settings
            FontVariationSettings,
            /// visibility
            Visibility,
            /// border-spacing
            BorderSpacing,
            /// line-height
            LineHeight,
            /// text-indent
            TextIndent,
            /// letter-spacing
            LetterSpacing,
            /// word-spacing
            WordSpacing,
            /// text-shadow
            TextShadow,
            /// text-emphasis-color
            TextEmphasisColor,
            /// -moz-tab-size
            MozTabSize,
            /// -webkit-text-fill-color
            WebkitTextFillColor,
            /// -webkit-text-stroke-color
            WebkitTextStrokeColor,
            /// -moz-image-region
            MozImageRegion,
            /// margin-top
            MarginTop,
            /// margin-left
            MarginLeft,
            /// margin-bottom
            MarginBottom,
            /// margin-right
            MarginRight,
            /// outline-color
            OutlineColor,
            /// outline-width
            OutlineWidth,
            /// -moz-outline-radius-topleft
            MozOutlineRadiusTopleft,
            /// -moz-outline-radius-topright
            MozOutlineRadiusTopright,
            /// -moz-outline-radius-bottomright
            MozOutlineRadiusBottomright,
            /// -moz-outline-radius-bottomleft
            MozOutlineRadiusBottomleft,
            /// outline-offset
            OutlineOffset,
            /// padding-top
            PaddingTop,
            /// padding-left
            PaddingLeft,
            /// padding-bottom
            PaddingBottom,
            /// padding-right
            PaddingRight,
            /// caret-color
            CaretColor,
            /// top
            Top,
            /// left
            Left,
            /// bottom
            Bottom,
            /// right
            Right,
            /// z-index
            ZIndex,
            /// flex-grow
            FlexGrow,
            /// flex-shrink
            FlexShrink,
            /// order
            Order,
            /// flex-basis
            FlexBasis,
            /// width
            Width,
            /// min-width
            MinWidth,
            /// max-width
            MaxWidth,
            /// height
            Height,
            /// min-height
            MinHeight,
            /// max-height
            MaxHeight,
            /// object-position
            ObjectPosition,
            /// grid-row-gap
            GridRowGap,
            /// grid-column-gap
            GridColumnGap,
            /// text-decoration-color
            TextDecorationColor,
            /// fill
            Fill,
            /// fill-opacity
            FillOpacity,
            /// stroke
            Stroke,
            /// stroke-width
            StrokeWidth,
            /// stroke-miterlimit
            StrokeMiterlimit,
            /// stroke-opacity
            StrokeOpacity,
            /// stroke-dasharray
            StrokeDasharray,
            /// stroke-dashoffset
            StrokeDashoffset,
            /// stop-color
            StopColor,
            /// stop-opacity
            StopOpacity,
            /// flood-color
            FloodColor,
            /// flood-opacity
            FloodOpacity,
            /// lighting-color
            LightingColor,
            /// clip-path
            ClipPath,
            /// mask-position-x
            MaskPositionX,
            /// mask-position-y
            MaskPositionY,
            /// mask-size
            MaskSize,
            /// -moz-box-flex
            MozBoxFlex,
            /// background
            Background,
            /// background-position
            BackgroundPosition,
            /// border-color
            BorderColor,
            /// border-width
            BorderWidth,
            /// border-top
            BorderTop,
            /// border-left
            BorderLeft,
            /// border-bottom
            BorderBottom,
            /// border-right
            BorderRight,
            /// border
            Border,
            /// border-radius
            BorderRadius,
            /// -moz-transform
            MozTransform,
            /// columns
            Columns,
            /// column-rule
            ColumnRule,
            /// font
            Font,
            /// text-emphasis
            TextEmphasis,
            /// -webkit-text-stroke
            WebkitTextStroke,
            /// margin
            Margin,
            /// mask
            Mask,
            /// mask-position
            MaskPosition,
            /// outline
            Outline,
            /// -moz-outline-radius
            MozOutlineRadius,
            /// padding
            Padding,
            /// flex
            Flex,
            /// grid-gap
            GridGap,
            /// text-decoration
            TextDecoration,
    /// Unrecognized property which could be any non-transitionable, custom property, or
    /// unknown property.
    Unsupported(CustomIdent)
}

impl fmt::Debug for TransitionProperty {
    fn fmt(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
        let name = match *self {
                    TransitionProperty::BackgroundColor => "BackgroundColor",
                    TransitionProperty::BackgroundPositionX => "BackgroundPositionX",
                    TransitionProperty::BackgroundPositionY => "BackgroundPositionY",
                    TransitionProperty::BackgroundSize => "BackgroundSize",
                    TransitionProperty::BorderTopColor => "BorderTopColor",
                    TransitionProperty::BorderTopWidth => "BorderTopWidth",
                    TransitionProperty::BorderLeftColor => "BorderLeftColor",
                    TransitionProperty::BorderLeftWidth => "BorderLeftWidth",
                    TransitionProperty::BorderBottomColor => "BorderBottomColor",
                    TransitionProperty::BorderBottomWidth => "BorderBottomWidth",
                    TransitionProperty::BorderRightColor => "BorderRightColor",
                    TransitionProperty::BorderRightWidth => "BorderRightWidth",
                    TransitionProperty::BorderTopLeftRadius => "BorderTopLeftRadius",
                    TransitionProperty::BorderTopRightRadius => "BorderTopRightRadius",
                    TransitionProperty::BorderBottomRightRadius => "BorderBottomRightRadius",
                    TransitionProperty::BorderBottomLeftRadius => "BorderBottomLeftRadius",
                    TransitionProperty::VerticalAlign => "VerticalAlign",
                    TransitionProperty::Transform => "Transform",
                    TransitionProperty::Perspective => "Perspective",
                    TransitionProperty::PerspectiveOrigin => "PerspectiveOrigin",
                    TransitionProperty::TransformOrigin => "TransformOrigin",
                    TransitionProperty::ShapeOutside => "ShapeOutside",
                    TransitionProperty::Color => "Color",
                    TransitionProperty::ColumnWidth => "ColumnWidth",
                    TransitionProperty::ColumnCount => "ColumnCount",
                    TransitionProperty::ColumnGap => "ColumnGap",
                    TransitionProperty::ColumnRuleWidth => "ColumnRuleWidth",
                    TransitionProperty::ColumnRuleColor => "ColumnRuleColor",
                    TransitionProperty::Opacity => "Opacity",
                    TransitionProperty::BoxShadow => "BoxShadow",
                    TransitionProperty::Clip => "Clip",
                    TransitionProperty::Filter => "Filter",
                    TransitionProperty::FontWeight => "FontWeight",
                    TransitionProperty::FontSize => "FontSize",
                    TransitionProperty::FontSizeAdjust => "FontSizeAdjust",
                    TransitionProperty::FontStretch => "FontStretch",
                    TransitionProperty::FontVariationSettings => "FontVariationSettings",
                    TransitionProperty::Visibility => "Visibility",
                    TransitionProperty::BorderSpacing => "BorderSpacing",
                    TransitionProperty::LineHeight => "LineHeight",
                    TransitionProperty::TextIndent => "TextIndent",
                    TransitionProperty::LetterSpacing => "LetterSpacing",
                    TransitionProperty::WordSpacing => "WordSpacing",
                    TransitionProperty::TextShadow => "TextShadow",
                    TransitionProperty::TextEmphasisColor => "TextEmphasisColor",
                    TransitionProperty::MozTabSize => "MozTabSize",
                    TransitionProperty::WebkitTextFillColor => "WebkitTextFillColor",
                    TransitionProperty::WebkitTextStrokeColor => "WebkitTextStrokeColor",
                    TransitionProperty::MozImageRegion => "MozImageRegion",
                    TransitionProperty::MarginTop => "MarginTop",
                    TransitionProperty::MarginLeft => "MarginLeft",
                    TransitionProperty::MarginBottom => "MarginBottom",
                    TransitionProperty::MarginRight => "MarginRight",
                    TransitionProperty::OutlineColor => "OutlineColor",
                    TransitionProperty::OutlineWidth => "OutlineWidth",
                    TransitionProperty::MozOutlineRadiusTopleft => "MozOutlineRadiusTopleft",
                    TransitionProperty::MozOutlineRadiusTopright => "MozOutlineRadiusTopright",
                    TransitionProperty::MozOutlineRadiusBottomright => "MozOutlineRadiusBottomright",
                    TransitionProperty::MozOutlineRadiusBottomleft => "MozOutlineRadiusBottomleft",
                    TransitionProperty::OutlineOffset => "OutlineOffset",
                    TransitionProperty::PaddingTop => "PaddingTop",
                    TransitionProperty::PaddingLeft => "PaddingLeft",
                    TransitionProperty::PaddingBottom => "PaddingBottom",
                    TransitionProperty::PaddingRight => "PaddingRight",
                    TransitionProperty::CaretColor => "CaretColor",
                    TransitionProperty::Top => "Top",
                    TransitionProperty::Left => "Left",
                    TransitionProperty::Bottom => "Bottom",
                    TransitionProperty::Right => "Right",
                    TransitionProperty::ZIndex => "ZIndex",
                    TransitionProperty::FlexGrow => "FlexGrow",
                    TransitionProperty::FlexShrink => "FlexShrink",
                    TransitionProperty::Order => "Order",
                    TransitionProperty::FlexBasis => "FlexBasis",
                    TransitionProperty::Width => "Width",
                    TransitionProperty::MinWidth => "MinWidth",
                    TransitionProperty::MaxWidth => "MaxWidth",
                    TransitionProperty::Height => "Height",
                    TransitionProperty::MinHeight => "MinHeight",
                    TransitionProperty::MaxHeight => "MaxHeight",
                    TransitionProperty::ObjectPosition => "ObjectPosition",
                    TransitionProperty::GridRowGap => "GridRowGap",
                    TransitionProperty::GridColumnGap => "GridColumnGap",
                    TransitionProperty::TextDecorationColor => "TextDecorationColor",
                    TransitionProperty::Fill => "Fill",
                    TransitionProperty::FillOpacity => "FillOpacity",
                    TransitionProperty::Stroke => "Stroke",
                    TransitionProperty::StrokeWidth => "StrokeWidth",
                    TransitionProperty::StrokeMiterlimit => "StrokeMiterlimit",
                    TransitionProperty::StrokeOpacity => "StrokeOpacity",
                    TransitionProperty::StrokeDasharray => "StrokeDasharray",
                    TransitionProperty::StrokeDashoffset => "StrokeDashoffset",
                    TransitionProperty::StopColor => "StopColor",
                    TransitionProperty::StopOpacity => "StopOpacity",
                    TransitionProperty::FloodColor => "FloodColor",
                    TransitionProperty::FloodOpacity => "FloodOpacity",
                    TransitionProperty::LightingColor => "LightingColor",
                    TransitionProperty::ClipPath => "ClipPath",
                    TransitionProperty::MaskPositionX => "MaskPositionX",
                    TransitionProperty::MaskPositionY => "MaskPositionY",
                    TransitionProperty::MaskSize => "MaskSize",
                    TransitionProperty::MozBoxFlex => "MozBoxFlex",
                    TransitionProperty::Background => "Background",
                    TransitionProperty::BackgroundPosition => "BackgroundPosition",
                    TransitionProperty::BorderColor => "BorderColor",
                    TransitionProperty::BorderWidth => "BorderWidth",
                    TransitionProperty::BorderTop => "BorderTop",
                    TransitionProperty::BorderLeft => "BorderLeft",
                    TransitionProperty::BorderBottom => "BorderBottom",
                    TransitionProperty::BorderRight => "BorderRight",
                    TransitionProperty::Border => "Border",
                    TransitionProperty::BorderRadius => "BorderRadius",
                    TransitionProperty::MozTransform => "MozTransform",
                    TransitionProperty::Columns => "Columns",
                    TransitionProperty::ColumnRule => "ColumnRule",
                    TransitionProperty::Font => "Font",
                    TransitionProperty::TextEmphasis => "TextEmphasis",
                    TransitionProperty::WebkitTextStroke => "WebkitTextStroke",
                    TransitionProperty::Margin => "Margin",
                    TransitionProperty::Mask => "Mask",
                    TransitionProperty::MaskPosition => "MaskPosition",
                    TransitionProperty::Outline => "Outline",
                    TransitionProperty::MozOutlineRadius => "MozOutlineRadius",
                    TransitionProperty::Padding => "Padding",
                    TransitionProperty::Flex => "Flex",
                    TransitionProperty::GridGap => "GridGap",
                    TransitionProperty::TextDecoration => "TextDecoration",
            TransitionProperty::All => "All",
            TransitionProperty::Unsupported(ref ident) => {
                formatter.write_str("Unsupported(")?;
                ident.fmt(formatter)?;
                return formatter.write_str(")");
            }
        };
        formatter.write_str(name)
    }
}

impl TransitionProperty {
    /// Iterates over each longhand property.
    pub fn each<F: FnMut(&TransitionProperty) -> ()>(mut cb: F) {
                cb(&TransitionProperty::BackgroundColor);
                cb(&TransitionProperty::BackgroundPositionX);
                cb(&TransitionProperty::BackgroundPositionY);
                cb(&TransitionProperty::BackgroundSize);
                cb(&TransitionProperty::BorderTopColor);
                cb(&TransitionProperty::BorderTopWidth);
                cb(&TransitionProperty::BorderLeftColor);
                cb(&TransitionProperty::BorderLeftWidth);
                cb(&TransitionProperty::BorderBottomColor);
                cb(&TransitionProperty::BorderBottomWidth);
                cb(&TransitionProperty::BorderRightColor);
                cb(&TransitionProperty::BorderRightWidth);
                cb(&TransitionProperty::BorderTopLeftRadius);
                cb(&TransitionProperty::BorderTopRightRadius);
                cb(&TransitionProperty::BorderBottomRightRadius);
                cb(&TransitionProperty::BorderBottomLeftRadius);
                cb(&TransitionProperty::VerticalAlign);
                cb(&TransitionProperty::Transform);
                cb(&TransitionProperty::Perspective);
                cb(&TransitionProperty::PerspectiveOrigin);
                cb(&TransitionProperty::TransformOrigin);
                cb(&TransitionProperty::ShapeOutside);
                cb(&TransitionProperty::Color);
                cb(&TransitionProperty::ColumnWidth);
                cb(&TransitionProperty::ColumnCount);
                cb(&TransitionProperty::ColumnGap);
                cb(&TransitionProperty::ColumnRuleWidth);
                cb(&TransitionProperty::ColumnRuleColor);
                cb(&TransitionProperty::Opacity);
                cb(&TransitionProperty::BoxShadow);
                cb(&TransitionProperty::Clip);
                cb(&TransitionProperty::Filter);
                cb(&TransitionProperty::FontWeight);
                cb(&TransitionProperty::FontSize);
                cb(&TransitionProperty::FontSizeAdjust);
                cb(&TransitionProperty::FontStretch);
                cb(&TransitionProperty::FontVariationSettings);
                cb(&TransitionProperty::Visibility);
                cb(&TransitionProperty::BorderSpacing);
                cb(&TransitionProperty::LineHeight);
                cb(&TransitionProperty::TextIndent);
                cb(&TransitionProperty::LetterSpacing);
                cb(&TransitionProperty::WordSpacing);
                cb(&TransitionProperty::TextShadow);
                cb(&TransitionProperty::TextEmphasisColor);
                cb(&TransitionProperty::MozTabSize);
                cb(&TransitionProperty::WebkitTextFillColor);
                cb(&TransitionProperty::WebkitTextStrokeColor);
                cb(&TransitionProperty::MozImageRegion);
                cb(&TransitionProperty::MarginTop);
                cb(&TransitionProperty::MarginLeft);
                cb(&TransitionProperty::MarginBottom);
                cb(&TransitionProperty::MarginRight);
                cb(&TransitionProperty::OutlineColor);
                cb(&TransitionProperty::OutlineWidth);
                cb(&TransitionProperty::MozOutlineRadiusTopleft);
                cb(&TransitionProperty::MozOutlineRadiusTopright);
                cb(&TransitionProperty::MozOutlineRadiusBottomright);
                cb(&TransitionProperty::MozOutlineRadiusBottomleft);
                cb(&TransitionProperty::OutlineOffset);
                cb(&TransitionProperty::PaddingTop);
                cb(&TransitionProperty::PaddingLeft);
                cb(&TransitionProperty::PaddingBottom);
                cb(&TransitionProperty::PaddingRight);
                cb(&TransitionProperty::CaretColor);
                cb(&TransitionProperty::Top);
                cb(&TransitionProperty::Left);
                cb(&TransitionProperty::Bottom);
                cb(&TransitionProperty::Right);
                cb(&TransitionProperty::ZIndex);
                cb(&TransitionProperty::FlexGrow);
                cb(&TransitionProperty::FlexShrink);
                cb(&TransitionProperty::Order);
                cb(&TransitionProperty::FlexBasis);
                cb(&TransitionProperty::Width);
                cb(&TransitionProperty::MinWidth);
                cb(&TransitionProperty::MaxWidth);
                cb(&TransitionProperty::Height);
                cb(&TransitionProperty::MinHeight);
                cb(&TransitionProperty::MaxHeight);
                cb(&TransitionProperty::ObjectPosition);
                cb(&TransitionProperty::GridRowGap);
                cb(&TransitionProperty::GridColumnGap);
                cb(&TransitionProperty::TextDecorationColor);
                cb(&TransitionProperty::Fill);
                cb(&TransitionProperty::FillOpacity);
                cb(&TransitionProperty::Stroke);
                cb(&TransitionProperty::StrokeWidth);
                cb(&TransitionProperty::StrokeMiterlimit);
                cb(&TransitionProperty::StrokeOpacity);
                cb(&TransitionProperty::StrokeDasharray);
                cb(&TransitionProperty::StrokeDashoffset);
                cb(&TransitionProperty::StopColor);
                cb(&TransitionProperty::StopOpacity);
                cb(&TransitionProperty::FloodColor);
                cb(&TransitionProperty::FloodOpacity);
                cb(&TransitionProperty::LightingColor);
                cb(&TransitionProperty::ClipPath);
                cb(&TransitionProperty::MaskPositionX);
                cb(&TransitionProperty::MaskPositionY);
                cb(&TransitionProperty::MaskSize);
                cb(&TransitionProperty::MozBoxFlex);
    }

    /// Iterates over every longhand property that is not TransitionProperty::All, stopping and
    /// returning true when the provided callback returns true for the first time.
    pub fn any<F: FnMut(&TransitionProperty) -> bool>(mut cb: F) -> bool {
                if cb(&TransitionProperty::BackgroundColor) {
                    return true;
                }
                if cb(&TransitionProperty::BackgroundPositionX) {
                    return true;
                }
                if cb(&TransitionProperty::BackgroundPositionY) {
                    return true;
                }
                if cb(&TransitionProperty::BackgroundSize) {
                    return true;
                }
                if cb(&TransitionProperty::BorderTopColor) {
                    return true;
                }
                if cb(&TransitionProperty::BorderTopWidth) {
                    return true;
                }
                if cb(&TransitionProperty::BorderLeftColor) {
                    return true;
                }
                if cb(&TransitionProperty::BorderLeftWidth) {
                    return true;
                }
                if cb(&TransitionProperty::BorderBottomColor) {
                    return true;
                }
                if cb(&TransitionProperty::BorderBottomWidth) {
                    return true;
                }
                if cb(&TransitionProperty::BorderRightColor) {
                    return true;
                }
                if cb(&TransitionProperty::BorderRightWidth) {
                    return true;
                }
                if cb(&TransitionProperty::BorderTopLeftRadius) {
                    return true;
                }
                if cb(&TransitionProperty::BorderTopRightRadius) {
                    return true;
                }
                if cb(&TransitionProperty::BorderBottomRightRadius) {
                    return true;
                }
                if cb(&TransitionProperty::BorderBottomLeftRadius) {
                    return true;
                }
                if cb(&TransitionProperty::VerticalAlign) {
                    return true;
                }
                if cb(&TransitionProperty::Transform) {
                    return true;
                }
                if cb(&TransitionProperty::Perspective) {
                    return true;
                }
                if cb(&TransitionProperty::PerspectiveOrigin) {
                    return true;
                }
                if cb(&TransitionProperty::TransformOrigin) {
                    return true;
                }
                if cb(&TransitionProperty::ShapeOutside) {
                    return true;
                }
                if cb(&TransitionProperty::Color) {
                    return true;
                }
                if cb(&TransitionProperty::ColumnWidth) {
                    return true;
                }
                if cb(&TransitionProperty::ColumnCount) {
                    return true;
                }
                if cb(&TransitionProperty::ColumnGap) {
                    return true;
                }
                if cb(&TransitionProperty::ColumnRuleWidth) {
                    return true;
                }
                if cb(&TransitionProperty::ColumnRuleColor) {
                    return true;
                }
                if cb(&TransitionProperty::Opacity) {
                    return true;
                }
                if cb(&TransitionProperty::BoxShadow) {
                    return true;
                }
                if cb(&TransitionProperty::Clip) {
                    return true;
                }
                if cb(&TransitionProperty::Filter) {
                    return true;
                }
                if cb(&TransitionProperty::FontWeight) {
                    return true;
                }
                if cb(&TransitionProperty::FontSize) {
                    return true;
                }
                if cb(&TransitionProperty::FontSizeAdjust) {
                    return true;
                }
                if cb(&TransitionProperty::FontStretch) {
                    return true;
                }
                if cb(&TransitionProperty::FontVariationSettings) {
                    return true;
                }
                if cb(&TransitionProperty::Visibility) {
                    return true;
                }
                if cb(&TransitionProperty::BorderSpacing) {
                    return true;
                }
                if cb(&TransitionProperty::LineHeight) {
                    return true;
                }
                if cb(&TransitionProperty::TextIndent) {
                    return true;
                }
                if cb(&TransitionProperty::LetterSpacing) {
                    return true;
                }
                if cb(&TransitionProperty::WordSpacing) {
                    return true;
                }
                if cb(&TransitionProperty::TextShadow) {
                    return true;
                }
                if cb(&TransitionProperty::TextEmphasisColor) {
                    return true;
                }
                if cb(&TransitionProperty::MozTabSize) {
                    return true;
                }
                if cb(&TransitionProperty::WebkitTextFillColor) {
                    return true;
                }
                if cb(&TransitionProperty::WebkitTextStrokeColor) {
                    return true;
                }
                if cb(&TransitionProperty::MozImageRegion) {
                    return true;
                }
                if cb(&TransitionProperty::MarginTop) {
                    return true;
                }
                if cb(&TransitionProperty::MarginLeft) {
                    return true;
                }
                if cb(&TransitionProperty::MarginBottom) {
                    return true;
                }
                if cb(&TransitionProperty::MarginRight) {
                    return true;
                }
                if cb(&TransitionProperty::OutlineColor) {
                    return true;
                }
                if cb(&TransitionProperty::OutlineWidth) {
                    return true;
                }
                if cb(&TransitionProperty::MozOutlineRadiusTopleft) {
                    return true;
                }
                if cb(&TransitionProperty::MozOutlineRadiusTopright) {
                    return true;
                }
                if cb(&TransitionProperty::MozOutlineRadiusBottomright) {
                    return true;
                }
                if cb(&TransitionProperty::MozOutlineRadiusBottomleft) {
                    return true;
                }
                if cb(&TransitionProperty::OutlineOffset) {
                    return true;
                }
                if cb(&TransitionProperty::PaddingTop) {
                    return true;
                }
                if cb(&TransitionProperty::PaddingLeft) {
                    return true;
                }
                if cb(&TransitionProperty::PaddingBottom) {
                    return true;
                }
                if cb(&TransitionProperty::PaddingRight) {
                    return true;
                }
                if cb(&TransitionProperty::CaretColor) {
                    return true;
                }
                if cb(&TransitionProperty::Top) {
                    return true;
                }
                if cb(&TransitionProperty::Left) {
                    return true;
                }
                if cb(&TransitionProperty::Bottom) {
                    return true;
                }
                if cb(&TransitionProperty::Right) {
                    return true;
                }
                if cb(&TransitionProperty::ZIndex) {
                    return true;
                }
                if cb(&TransitionProperty::FlexGrow) {
                    return true;
                }
                if cb(&TransitionProperty::FlexShrink) {
                    return true;
                }
                if cb(&TransitionProperty::Order) {
                    return true;
                }
                if cb(&TransitionProperty::FlexBasis) {
                    return true;
                }
                if cb(&TransitionProperty::Width) {
                    return true;
                }
                if cb(&TransitionProperty::MinWidth) {
                    return true;
                }
                if cb(&TransitionProperty::MaxWidth) {
                    return true;
                }
                if cb(&TransitionProperty::Height) {
                    return true;
                }
                if cb(&TransitionProperty::MinHeight) {
                    return true;
                }
                if cb(&TransitionProperty::MaxHeight) {
                    return true;
                }
                if cb(&TransitionProperty::ObjectPosition) {
                    return true;
                }
                if cb(&TransitionProperty::GridRowGap) {
                    return true;
                }
                if cb(&TransitionProperty::GridColumnGap) {
                    return true;
                }
                if cb(&TransitionProperty::TextDecorationColor) {
                    return true;
                }
                if cb(&TransitionProperty::Fill) {
                    return true;
                }
                if cb(&TransitionProperty::FillOpacity) {
                    return true;
                }
                if cb(&TransitionProperty::Stroke) {
                    return true;
                }
                if cb(&TransitionProperty::StrokeWidth) {
                    return true;
                }
                if cb(&TransitionProperty::StrokeMiterlimit) {
                    return true;
                }
                if cb(&TransitionProperty::StrokeOpacity) {
                    return true;
                }
                if cb(&TransitionProperty::StrokeDasharray) {
                    return true;
                }
                if cb(&TransitionProperty::StrokeDashoffset) {
                    return true;
                }
                if cb(&TransitionProperty::StopColor) {
                    return true;
                }
                if cb(&TransitionProperty::StopOpacity) {
                    return true;
                }
                if cb(&TransitionProperty::FloodColor) {
                    return true;
                }
                if cb(&TransitionProperty::FloodOpacity) {
                    return true;
                }
                if cb(&TransitionProperty::LightingColor) {
                    return true;
                }
                if cb(&TransitionProperty::ClipPath) {
                    return true;
                }
                if cb(&TransitionProperty::MaskPositionX) {
                    return true;
                }
                if cb(&TransitionProperty::MaskPositionY) {
                    return true;
                }
                if cb(&TransitionProperty::MaskSize) {
                    return true;
                }
                if cb(&TransitionProperty::MozBoxFlex) {
                    return true;
                }
        false
    }

    /// Parse a transition-property value.
    pub fn parse<'i, 't>(input: &mut Parser<'i, 't>) -> Result<Self, ParseError<'i>> {
        let ident = input.expect_ident()?;
        match_ignore_ascii_case! { &ident,
            "all" => Ok(TransitionProperty::All),
                    "background-color" => Ok(TransitionProperty::BackgroundColor),
                    "background-position-x" => Ok(TransitionProperty::BackgroundPositionX),
                    "background-position-y" => Ok(TransitionProperty::BackgroundPositionY),
                    "background-size" => Ok(TransitionProperty::BackgroundSize),
                    "border-top-color" => Ok(TransitionProperty::BorderTopColor),
                    "border-top-width" => Ok(TransitionProperty::BorderTopWidth),
                    "border-left-color" => Ok(TransitionProperty::BorderLeftColor),
                    "border-left-width" => Ok(TransitionProperty::BorderLeftWidth),
                    "border-bottom-color" => Ok(TransitionProperty::BorderBottomColor),
                    "border-bottom-width" => Ok(TransitionProperty::BorderBottomWidth),
                    "border-right-color" => Ok(TransitionProperty::BorderRightColor),
                    "border-right-width" => Ok(TransitionProperty::BorderRightWidth),
                    "border-top-left-radius" => Ok(TransitionProperty::BorderTopLeftRadius),
                    "border-top-right-radius" => Ok(TransitionProperty::BorderTopRightRadius),
                    "border-bottom-right-radius" => Ok(TransitionProperty::BorderBottomRightRadius),
                    "border-bottom-left-radius" => Ok(TransitionProperty::BorderBottomLeftRadius),
                    "vertical-align" => Ok(TransitionProperty::VerticalAlign),
                    "transform" => Ok(TransitionProperty::Transform),
                    "perspective" => Ok(TransitionProperty::Perspective),
                    "perspective-origin" => Ok(TransitionProperty::PerspectiveOrigin),
                    "transform-origin" => Ok(TransitionProperty::TransformOrigin),
                    "shape-outside" => Ok(TransitionProperty::ShapeOutside),
                    "color" => Ok(TransitionProperty::Color),
                    "column-width" => Ok(TransitionProperty::ColumnWidth),
                    "column-count" => Ok(TransitionProperty::ColumnCount),
                    "column-gap" => Ok(TransitionProperty::ColumnGap),
                    "column-rule-width" => Ok(TransitionProperty::ColumnRuleWidth),
                    "column-rule-color" => Ok(TransitionProperty::ColumnRuleColor),
                    "opacity" => Ok(TransitionProperty::Opacity),
                    "box-shadow" => Ok(TransitionProperty::BoxShadow),
                    "clip" => Ok(TransitionProperty::Clip),
                    "filter" => Ok(TransitionProperty::Filter),
                    "font-weight" => Ok(TransitionProperty::FontWeight),
                    "font-size" => Ok(TransitionProperty::FontSize),
                    "font-size-adjust" => Ok(TransitionProperty::FontSizeAdjust),
                    "font-stretch" => Ok(TransitionProperty::FontStretch),
                    "font-variation-settings" => Ok(TransitionProperty::FontVariationSettings),
                    "visibility" => Ok(TransitionProperty::Visibility),
                    "border-spacing" => Ok(TransitionProperty::BorderSpacing),
                    "line-height" => Ok(TransitionProperty::LineHeight),
                    "text-indent" => Ok(TransitionProperty::TextIndent),
                    "letter-spacing" => Ok(TransitionProperty::LetterSpacing),
                    "word-spacing" => Ok(TransitionProperty::WordSpacing),
                    "text-shadow" => Ok(TransitionProperty::TextShadow),
                    "text-emphasis-color" => Ok(TransitionProperty::TextEmphasisColor),
                    "-moz-tab-size" => Ok(TransitionProperty::MozTabSize),
                    "-webkit-text-fill-color" => Ok(TransitionProperty::WebkitTextFillColor),
                    "-webkit-text-stroke-color" => Ok(TransitionProperty::WebkitTextStrokeColor),
                    "-moz-image-region" => Ok(TransitionProperty::MozImageRegion),
                    "margin-top" => Ok(TransitionProperty::MarginTop),
                    "margin-left" => Ok(TransitionProperty::MarginLeft),
                    "margin-bottom" => Ok(TransitionProperty::MarginBottom),
                    "margin-right" => Ok(TransitionProperty::MarginRight),
                    "outline-color" => Ok(TransitionProperty::OutlineColor),
                    "outline-width" => Ok(TransitionProperty::OutlineWidth),
                    "-moz-outline-radius-topleft" => Ok(TransitionProperty::MozOutlineRadiusTopleft),
                    "-moz-outline-radius-topright" => Ok(TransitionProperty::MozOutlineRadiusTopright),
                    "-moz-outline-radius-bottomright" => Ok(TransitionProperty::MozOutlineRadiusBottomright),
                    "-moz-outline-radius-bottomleft" => Ok(TransitionProperty::MozOutlineRadiusBottomleft),
                    "outline-offset" => Ok(TransitionProperty::OutlineOffset),
                    "padding-top" => Ok(TransitionProperty::PaddingTop),
                    "padding-left" => Ok(TransitionProperty::PaddingLeft),
                    "padding-bottom" => Ok(TransitionProperty::PaddingBottom),
                    "padding-right" => Ok(TransitionProperty::PaddingRight),
                    "caret-color" => Ok(TransitionProperty::CaretColor),
                    "top" => Ok(TransitionProperty::Top),
                    "left" => Ok(TransitionProperty::Left),
                    "bottom" => Ok(TransitionProperty::Bottom),
                    "right" => Ok(TransitionProperty::Right),
                    "z-index" => Ok(TransitionProperty::ZIndex),
                    "flex-grow" => Ok(TransitionProperty::FlexGrow),
                    "flex-shrink" => Ok(TransitionProperty::FlexShrink),
                    "order" => Ok(TransitionProperty::Order),
                    "flex-basis" => Ok(TransitionProperty::FlexBasis),
                    "width" => Ok(TransitionProperty::Width),
                    "min-width" => Ok(TransitionProperty::MinWidth),
                    "max-width" => Ok(TransitionProperty::MaxWidth),
                    "height" => Ok(TransitionProperty::Height),
                    "min-height" => Ok(TransitionProperty::MinHeight),
                    "max-height" => Ok(TransitionProperty::MaxHeight),
                    "object-position" => Ok(TransitionProperty::ObjectPosition),
                    "grid-row-gap" => Ok(TransitionProperty::GridRowGap),
                    "grid-column-gap" => Ok(TransitionProperty::GridColumnGap),
                    "text-decoration-color" => Ok(TransitionProperty::TextDecorationColor),
                    "fill" => Ok(TransitionProperty::Fill),
                    "fill-opacity" => Ok(TransitionProperty::FillOpacity),
                    "stroke" => Ok(TransitionProperty::Stroke),
                    "stroke-width" => Ok(TransitionProperty::StrokeWidth),
                    "stroke-miterlimit" => Ok(TransitionProperty::StrokeMiterlimit),
                    "stroke-opacity" => Ok(TransitionProperty::StrokeOpacity),
                    "stroke-dasharray" => Ok(TransitionProperty::StrokeDasharray),
                    "stroke-dashoffset" => Ok(TransitionProperty::StrokeDashoffset),
                    "stop-color" => Ok(TransitionProperty::StopColor),
                    "stop-opacity" => Ok(TransitionProperty::StopOpacity),
                    "flood-color" => Ok(TransitionProperty::FloodColor),
                    "flood-opacity" => Ok(TransitionProperty::FloodOpacity),
                    "lighting-color" => Ok(TransitionProperty::LightingColor),
                    "clip-path" => Ok(TransitionProperty::ClipPath),
                    "mask-position-x" => Ok(TransitionProperty::MaskPositionX),
                    "mask-position-y" => Ok(TransitionProperty::MaskPositionY),
                    "mask-size" => Ok(TransitionProperty::MaskSize),
                    "-moz-box-flex" => Ok(TransitionProperty::MozBoxFlex),
                    "background" => Ok(TransitionProperty::Background),
                    "background-position" => Ok(TransitionProperty::BackgroundPosition),
                    "border-color" => Ok(TransitionProperty::BorderColor),
                    "border-width" => Ok(TransitionProperty::BorderWidth),
                    "border-top" => Ok(TransitionProperty::BorderTop),
                    "border-left" => Ok(TransitionProperty::BorderLeft),
                    "border-bottom" => Ok(TransitionProperty::BorderBottom),
                    "border-right" => Ok(TransitionProperty::BorderRight),
                    "border" => Ok(TransitionProperty::Border),
                    "border-radius" => Ok(TransitionProperty::BorderRadius),
                    "-moz-transform" => Ok(TransitionProperty::MozTransform),
                    "columns" => Ok(TransitionProperty::Columns),
                    "column-rule" => Ok(TransitionProperty::ColumnRule),
                    "font" => Ok(TransitionProperty::Font),
                    "text-emphasis" => Ok(TransitionProperty::TextEmphasis),
                    "-webkit-text-stroke" => Ok(TransitionProperty::WebkitTextStroke),
                    "margin" => Ok(TransitionProperty::Margin),
                    "mask" => Ok(TransitionProperty::Mask),
                    "mask-position" => Ok(TransitionProperty::MaskPosition),
                    "outline" => Ok(TransitionProperty::Outline),
                    "-moz-outline-radius" => Ok(TransitionProperty::MozOutlineRadius),
                    "padding" => Ok(TransitionProperty::Padding),
                    "flex" => Ok(TransitionProperty::Flex),
                    "grid-gap" => Ok(TransitionProperty::GridGap),
                    "text-decoration" => Ok(TransitionProperty::TextDecoration),
            "none" => Err(SelectorParseError::UnexpectedIdent(ident.clone()).into()),
            _ => CustomIdent::from_ident(ident, &[]).map(TransitionProperty::Unsupported),
        }
    }

    /// Return transitionable longhands of this shorthand TransitionProperty, except for "all".
    pub fn longhands(&self) -> &'static [TransitionProperty] {
                static BACKGROUND: &'static [TransitionProperty] = &[
                            TransitionProperty::BackgroundColor,
                            TransitionProperty::BackgroundPositionX,
                            TransitionProperty::BackgroundPositionY,
                            TransitionProperty::BackgroundSize,
                ];
                static BACKGROUND_POSITION: &'static [TransitionProperty] = &[
                            TransitionProperty::BackgroundPositionX,
                            TransitionProperty::BackgroundPositionY,
                ];
                static BORDER_COLOR: &'static [TransitionProperty] = &[
                            TransitionProperty::BorderTopColor,
                            TransitionProperty::BorderRightColor,
                            TransitionProperty::BorderBottomColor,
                            TransitionProperty::BorderLeftColor,
                ];
                static BORDER_WIDTH: &'static [TransitionProperty] = &[
                            TransitionProperty::BorderTopWidth,
                            TransitionProperty::BorderLeftWidth,
                            TransitionProperty::BorderBottomWidth,
                            TransitionProperty::BorderRightWidth,
                ];
                static BORDER_TOP: &'static [TransitionProperty] = &[
                            TransitionProperty::BorderTopColor,
                            TransitionProperty::BorderTopWidth,
                ];
                static BORDER_LEFT: &'static [TransitionProperty] = &[
                            TransitionProperty::BorderLeftColor,
                            TransitionProperty::BorderLeftWidth,
                ];
                static BORDER_BOTTOM: &'static [TransitionProperty] = &[
                            TransitionProperty::BorderBottomColor,
                            TransitionProperty::BorderBottomWidth,
                ];
                static BORDER_RIGHT: &'static [TransitionProperty] = &[
                            TransitionProperty::BorderRightColor,
                            TransitionProperty::BorderRightWidth,
                ];
                static BORDER: &'static [TransitionProperty] = &[
                            TransitionProperty::BorderTopColor,
                            TransitionProperty::BorderTopWidth,
                            TransitionProperty::BorderLeftColor,
                            TransitionProperty::BorderLeftWidth,
                            TransitionProperty::BorderBottomColor,
                            TransitionProperty::BorderBottomWidth,
                            TransitionProperty::BorderRightColor,
                            TransitionProperty::BorderRightWidth,
                ];
                static BORDER_RADIUS: &'static [TransitionProperty] = &[
                            TransitionProperty::BorderTopLeftRadius,
                            TransitionProperty::BorderTopRightRadius,
                            TransitionProperty::BorderBottomRightRadius,
                            TransitionProperty::BorderBottomLeftRadius,
                ];
                static _MOZ_TRANSFORM: &'static [TransitionProperty] = &[
                            TransitionProperty::Transform,
                ];
                static COLUMNS: &'static [TransitionProperty] = &[
                            TransitionProperty::ColumnWidth,
                            TransitionProperty::ColumnCount,
                ];
                static COLUMN_RULE: &'static [TransitionProperty] = &[
                            TransitionProperty::ColumnRuleWidth,
                            TransitionProperty::ColumnRuleColor,
                ];
                static FONT: &'static [TransitionProperty] = &[
                            TransitionProperty::FontWeight,
                            TransitionProperty::FontStretch,
                            TransitionProperty::FontSize,
                            TransitionProperty::LineHeight,
                            TransitionProperty::FontSizeAdjust,
                ];
                static TEXT_EMPHASIS: &'static [TransitionProperty] = &[
                            TransitionProperty::TextEmphasisColor,
                ];
                static _WEBKIT_TEXT_STROKE: &'static [TransitionProperty] = &[
                            TransitionProperty::WebkitTextStrokeColor,
                ];
                static MARGIN: &'static [TransitionProperty] = &[
                            TransitionProperty::MarginTop,
                            TransitionProperty::MarginRight,
                            TransitionProperty::MarginBottom,
                            TransitionProperty::MarginLeft,
                ];
                static MASK: &'static [TransitionProperty] = &[
                            TransitionProperty::MaskPositionX,
                            TransitionProperty::MaskPositionY,
                            TransitionProperty::MaskSize,
                ];
                static MASK_POSITION: &'static [TransitionProperty] = &[
                            TransitionProperty::MaskPositionX,
                            TransitionProperty::MaskPositionY,
                ];
                static OUTLINE: &'static [TransitionProperty] = &[
                            TransitionProperty::OutlineWidth,
                            TransitionProperty::OutlineColor,
                ];
                static _MOZ_OUTLINE_RADIUS: &'static [TransitionProperty] = &[
                            TransitionProperty::MozOutlineRadiusTopleft,
                            TransitionProperty::MozOutlineRadiusTopright,
                            TransitionProperty::MozOutlineRadiusBottomright,
                            TransitionProperty::MozOutlineRadiusBottomleft,
                ];
                static PADDING: &'static [TransitionProperty] = &[
                            TransitionProperty::PaddingTop,
                            TransitionProperty::PaddingRight,
                            TransitionProperty::PaddingBottom,
                            TransitionProperty::PaddingLeft,
                ];
                static FLEX: &'static [TransitionProperty] = &[
                            TransitionProperty::FlexGrow,
                            TransitionProperty::FlexShrink,
                            TransitionProperty::FlexBasis,
                ];
                static GRID_GAP: &'static [TransitionProperty] = &[
                            TransitionProperty::GridRowGap,
                            TransitionProperty::GridColumnGap,
                ];
                static TEXT_DECORATION: &'static [TransitionProperty] = &[
                            TransitionProperty::TextDecorationColor,
                ];
        match *self {
                    TransitionProperty::Background => BACKGROUND,
                    TransitionProperty::BackgroundPosition => BACKGROUND_POSITION,
                    TransitionProperty::BorderColor => BORDER_COLOR,
                    TransitionProperty::BorderWidth => BORDER_WIDTH,
                    TransitionProperty::BorderTop => BORDER_TOP,
                    TransitionProperty::BorderLeft => BORDER_LEFT,
                    TransitionProperty::BorderBottom => BORDER_BOTTOM,
                    TransitionProperty::BorderRight => BORDER_RIGHT,
                    TransitionProperty::Border => BORDER,
                    TransitionProperty::BorderRadius => BORDER_RADIUS,
                    TransitionProperty::MozTransform => _MOZ_TRANSFORM,
                    TransitionProperty::Columns => COLUMNS,
                    TransitionProperty::ColumnRule => COLUMN_RULE,
                    TransitionProperty::Font => FONT,
                    TransitionProperty::TextEmphasis => TEXT_EMPHASIS,
                    TransitionProperty::WebkitTextStroke => _WEBKIT_TEXT_STROKE,
                    TransitionProperty::Margin => MARGIN,
                    TransitionProperty::Mask => MASK,
                    TransitionProperty::MaskPosition => MASK_POSITION,
                    TransitionProperty::Outline => OUTLINE,
                    TransitionProperty::MozOutlineRadius => _MOZ_OUTLINE_RADIUS,
                    TransitionProperty::Padding => PADDING,
                    TransitionProperty::Flex => FLEX,
                    TransitionProperty::GridGap => GRID_GAP,
                    TransitionProperty::TextDecoration => TEXT_DECORATION,
            _ => panic!("Not allowed to call longhands() for this TransitionProperty")
        }
    }

    /// Returns true if this TransitionProperty is a shorthand.
    pub fn is_shorthand(&self) -> bool {
        match *self {
                    TransitionProperty::Background => true,
                    TransitionProperty::BackgroundPosition => true,
                    TransitionProperty::BorderColor => true,
                    TransitionProperty::BorderWidth => true,
                    TransitionProperty::BorderTop => true,
                    TransitionProperty::BorderLeft => true,
                    TransitionProperty::BorderBottom => true,
                    TransitionProperty::BorderRight => true,
                    TransitionProperty::Border => true,
                    TransitionProperty::BorderRadius => true,
                    TransitionProperty::MozTransform => true,
                    TransitionProperty::Columns => true,
                    TransitionProperty::ColumnRule => true,
                    TransitionProperty::Font => true,
                    TransitionProperty::TextEmphasis => true,
                    TransitionProperty::WebkitTextStroke => true,
                    TransitionProperty::Margin => true,
                    TransitionProperty::Mask => true,
                    TransitionProperty::MaskPosition => true,
                    TransitionProperty::Outline => true,
                    TransitionProperty::MozOutlineRadius => true,
                    TransitionProperty::Padding => true,
                    TransitionProperty::Flex => true,
                    TransitionProperty::GridGap => true,
                    TransitionProperty::TextDecoration => true,
            _ => false
        }
    }
}

/// Convert to nsCSSPropertyID.
#[cfg(feature = "gecko")]
#[allow(non_upper_case_globals)]
impl<'a> From< &'a TransitionProperty> for nsCSSPropertyID {
    fn from(transition_property: &'a TransitionProperty) -> nsCSSPropertyID {
        match *transition_property {
                    TransitionProperty::BackgroundColor
                        => 
    nsCSSPropertyID::eCSSProperty_background_color,
                    TransitionProperty::BackgroundPositionX
                        => 
    nsCSSPropertyID::eCSSProperty_background_position_x,
                    TransitionProperty::BackgroundPositionY
                        => 
    nsCSSPropertyID::eCSSProperty_background_position_y,
                    TransitionProperty::BackgroundSize
                        => 
    nsCSSPropertyID::eCSSProperty_background_size,
                    TransitionProperty::BorderTopColor
                        => 
    nsCSSPropertyID::eCSSProperty_border_top_color,
                    TransitionProperty::BorderTopWidth
                        => 
    nsCSSPropertyID::eCSSProperty_border_top_width,
                    TransitionProperty::BorderLeftColor
                        => 
    nsCSSPropertyID::eCSSProperty_border_left_color,
                    TransitionProperty::BorderLeftWidth
                        => 
    nsCSSPropertyID::eCSSProperty_border_left_width,
                    TransitionProperty::BorderBottomColor
                        => 
    nsCSSPropertyID::eCSSProperty_border_bottom_color,
                    TransitionProperty::BorderBottomWidth
                        => 
    nsCSSPropertyID::eCSSProperty_border_bottom_width,
                    TransitionProperty::BorderRightColor
                        => 
    nsCSSPropertyID::eCSSProperty_border_right_color,
                    TransitionProperty::BorderRightWidth
                        => 
    nsCSSPropertyID::eCSSProperty_border_right_width,
                    TransitionProperty::BorderTopLeftRadius
                        => 
    nsCSSPropertyID::eCSSProperty_border_top_left_radius,
                    TransitionProperty::BorderTopRightRadius
                        => 
    nsCSSPropertyID::eCSSProperty_border_top_right_radius,
                    TransitionProperty::BorderBottomRightRadius
                        => 
    nsCSSPropertyID::eCSSProperty_border_bottom_right_radius,
                    TransitionProperty::BorderBottomLeftRadius
                        => 
    nsCSSPropertyID::eCSSProperty_border_bottom_left_radius,
                    TransitionProperty::VerticalAlign
                        => 
    nsCSSPropertyID::eCSSProperty_vertical_align,
                    TransitionProperty::Transform
                        => 
    nsCSSPropertyID::eCSSProperty_transform,
                    TransitionProperty::Perspective
                        => 
    nsCSSPropertyID::eCSSProperty_perspective,
                    TransitionProperty::PerspectiveOrigin
                        => 
    nsCSSPropertyID::eCSSProperty_perspective_origin,
                    TransitionProperty::TransformOrigin
                        => 
    nsCSSPropertyID::eCSSProperty_transform_origin,
                    TransitionProperty::ShapeOutside
                        => 
    nsCSSPropertyID::eCSSProperty_shape_outside,
                    TransitionProperty::Color
                        => 
    nsCSSPropertyID::eCSSProperty_color,
                    TransitionProperty::ColumnWidth
                        => 
    nsCSSPropertyID::eCSSProperty_column_width,
                    TransitionProperty::ColumnCount
                        => 
    nsCSSPropertyID::eCSSProperty_column_count,
                    TransitionProperty::ColumnGap
                        => 
    nsCSSPropertyID::eCSSProperty_column_gap,
                    TransitionProperty::ColumnRuleWidth
                        => 
    nsCSSPropertyID::eCSSProperty_column_rule_width,
                    TransitionProperty::ColumnRuleColor
                        => 
    nsCSSPropertyID::eCSSProperty_column_rule_color,
                    TransitionProperty::Opacity
                        => 
    nsCSSPropertyID::eCSSProperty_opacity,
                    TransitionProperty::BoxShadow
                        => 
    nsCSSPropertyID::eCSSProperty_box_shadow,
                    TransitionProperty::Clip
                        => 
    nsCSSPropertyID::eCSSProperty_clip,
                    TransitionProperty::Filter
                        => 
    nsCSSPropertyID::eCSSProperty_filter,
                    TransitionProperty::FontWeight
                        => 
    nsCSSPropertyID::eCSSProperty_font_weight,
                    TransitionProperty::FontSize
                        => 
    nsCSSPropertyID::eCSSProperty_font_size,
                    TransitionProperty::FontSizeAdjust
                        => 
    nsCSSPropertyID::eCSSProperty_font_size_adjust,
                    TransitionProperty::FontStretch
                        => 
    nsCSSPropertyID::eCSSProperty_font_stretch,
                    TransitionProperty::FontVariationSettings
                        => 
    nsCSSPropertyID::eCSSProperty_font_variation_settings,
                    TransitionProperty::Visibility
                        => 
    nsCSSPropertyID::eCSSProperty_visibility,
                    TransitionProperty::BorderSpacing
                        => 
    nsCSSPropertyID::eCSSProperty_border_spacing,
                    TransitionProperty::LineHeight
                        => 
    nsCSSPropertyID::eCSSProperty_line_height,
                    TransitionProperty::TextIndent
                        => 
    nsCSSPropertyID::eCSSProperty_text_indent,
                    TransitionProperty::LetterSpacing
                        => 
    nsCSSPropertyID::eCSSProperty_letter_spacing,
                    TransitionProperty::WordSpacing
                        => 
    nsCSSPropertyID::eCSSProperty_word_spacing,
                    TransitionProperty::TextShadow
                        => 
    nsCSSPropertyID::eCSSProperty_text_shadow,
                    TransitionProperty::TextEmphasisColor
                        => 
    nsCSSPropertyID::eCSSProperty_text_emphasis_color,
                    TransitionProperty::MozTabSize
                        => 
    nsCSSPropertyID::eCSSProperty__moz_tab_size,
                    TransitionProperty::WebkitTextFillColor
                        => 
    nsCSSPropertyID::eCSSProperty__webkit_text_fill_color,
                    TransitionProperty::WebkitTextStrokeColor
                        => 
    nsCSSPropertyID::eCSSProperty__webkit_text_stroke_color,
                    TransitionProperty::MozImageRegion
                        => 
    nsCSSPropertyID::eCSSProperty__moz_image_region,
                    TransitionProperty::MarginTop
                        => 
    nsCSSPropertyID::eCSSProperty_margin_top,
                    TransitionProperty::MarginLeft
                        => 
    nsCSSPropertyID::eCSSProperty_margin_left,
                    TransitionProperty::MarginBottom
                        => 
    nsCSSPropertyID::eCSSProperty_margin_bottom,
                    TransitionProperty::MarginRight
                        => 
    nsCSSPropertyID::eCSSProperty_margin_right,
                    TransitionProperty::OutlineColor
                        => 
    nsCSSPropertyID::eCSSProperty_outline_color,
                    TransitionProperty::OutlineWidth
                        => 
    nsCSSPropertyID::eCSSProperty_outline_width,
                    TransitionProperty::MozOutlineRadiusTopleft
                        => 
    nsCSSPropertyID::eCSSProperty__moz_outline_radius_topleft,
                    TransitionProperty::MozOutlineRadiusTopright
                        => 
    nsCSSPropertyID::eCSSProperty__moz_outline_radius_topright,
                    TransitionProperty::MozOutlineRadiusBottomright
                        => 
    nsCSSPropertyID::eCSSProperty__moz_outline_radius_bottomright,
                    TransitionProperty::MozOutlineRadiusBottomleft
                        => 
    nsCSSPropertyID::eCSSProperty__moz_outline_radius_bottomleft,
                    TransitionProperty::OutlineOffset
                        => 
    nsCSSPropertyID::eCSSProperty_outline_offset,
                    TransitionProperty::PaddingTop
                        => 
    nsCSSPropertyID::eCSSProperty_padding_top,
                    TransitionProperty::PaddingLeft
                        => 
    nsCSSPropertyID::eCSSProperty_padding_left,
                    TransitionProperty::PaddingBottom
                        => 
    nsCSSPropertyID::eCSSProperty_padding_bottom,
                    TransitionProperty::PaddingRight
                        => 
    nsCSSPropertyID::eCSSProperty_padding_right,
                    TransitionProperty::CaretColor
                        => 
    nsCSSPropertyID::eCSSProperty_caret_color,
                    TransitionProperty::Top
                        => 
    nsCSSPropertyID::eCSSProperty_top,
                    TransitionProperty::Left
                        => 
    nsCSSPropertyID::eCSSProperty_left,
                    TransitionProperty::Bottom
                        => 
    nsCSSPropertyID::eCSSProperty_bottom,
                    TransitionProperty::Right
                        => 
    nsCSSPropertyID::eCSSProperty_right,
                    TransitionProperty::ZIndex
                        => 
    nsCSSPropertyID::eCSSProperty_z_index,
                    TransitionProperty::FlexGrow
                        => 
    nsCSSPropertyID::eCSSProperty_flex_grow,
                    TransitionProperty::FlexShrink
                        => 
    nsCSSPropertyID::eCSSProperty_flex_shrink,
                    TransitionProperty::Order
                        => 
    nsCSSPropertyID::eCSSProperty_order,
                    TransitionProperty::FlexBasis
                        => 
    nsCSSPropertyID::eCSSProperty_flex_basis,
                    TransitionProperty::Width
                        => 
    nsCSSPropertyID::eCSSProperty_width,
                    TransitionProperty::MinWidth
                        => 
    nsCSSPropertyID::eCSSProperty_min_width,
                    TransitionProperty::MaxWidth
                        => 
    nsCSSPropertyID::eCSSProperty_max_width,
                    TransitionProperty::Height
                        => 
    nsCSSPropertyID::eCSSProperty_height,
                    TransitionProperty::MinHeight
                        => 
    nsCSSPropertyID::eCSSProperty_min_height,
                    TransitionProperty::MaxHeight
                        => 
    nsCSSPropertyID::eCSSProperty_max_height,
                    TransitionProperty::ObjectPosition
                        => 
    nsCSSPropertyID::eCSSProperty_object_position,
                    TransitionProperty::GridRowGap
                        => 
    nsCSSPropertyID::eCSSProperty_grid_row_gap,
                    TransitionProperty::GridColumnGap
                        => 
    nsCSSPropertyID::eCSSProperty_grid_column_gap,
                    TransitionProperty::TextDecorationColor
                        => 
    nsCSSPropertyID::eCSSProperty_text_decoration_color,
                    TransitionProperty::Fill
                        => 
    nsCSSPropertyID::eCSSProperty_fill,
                    TransitionProperty::FillOpacity
                        => 
    nsCSSPropertyID::eCSSProperty_fill_opacity,
                    TransitionProperty::Stroke
                        => 
    nsCSSPropertyID::eCSSProperty_stroke,
                    TransitionProperty::StrokeWidth
                        => 
    nsCSSPropertyID::eCSSProperty_stroke_width,
                    TransitionProperty::StrokeMiterlimit
                        => 
    nsCSSPropertyID::eCSSProperty_stroke_miterlimit,
                    TransitionProperty::StrokeOpacity
                        => 
    nsCSSPropertyID::eCSSProperty_stroke_opacity,
                    TransitionProperty::StrokeDasharray
                        => 
    nsCSSPropertyID::eCSSProperty_stroke_dasharray,
                    TransitionProperty::StrokeDashoffset
                        => 
    nsCSSPropertyID::eCSSProperty_stroke_dashoffset,
                    TransitionProperty::StopColor
                        => 
    nsCSSPropertyID::eCSSProperty_stop_color,
                    TransitionProperty::StopOpacity
                        => 
    nsCSSPropertyID::eCSSProperty_stop_opacity,
                    TransitionProperty::FloodColor
                        => 
    nsCSSPropertyID::eCSSProperty_flood_color,
                    TransitionProperty::FloodOpacity
                        => 
    nsCSSPropertyID::eCSSProperty_flood_opacity,
                    TransitionProperty::LightingColor
                        => 
    nsCSSPropertyID::eCSSProperty_lighting_color,
                    TransitionProperty::ClipPath
                        => 
    nsCSSPropertyID::eCSSProperty_clip_path,
                    TransitionProperty::MaskPositionX
                        => 
    nsCSSPropertyID::eCSSProperty_mask_position_x,
                    TransitionProperty::MaskPositionY
                        => 
    nsCSSPropertyID::eCSSProperty_mask_position_y,
                    TransitionProperty::MaskSize
                        => 
    nsCSSPropertyID::eCSSProperty_mask_size,
                    TransitionProperty::MozBoxFlex
                        => 
    nsCSSPropertyID::eCSSProperty__moz_box_flex,
                    TransitionProperty::Background
                        => 
    nsCSSPropertyID::eCSSProperty_background,
                    TransitionProperty::BackgroundPosition
                        => 
    nsCSSPropertyID::eCSSProperty_background_position,
                    TransitionProperty::BorderColor
                        => 
    nsCSSPropertyID::eCSSProperty_border_color,
                    TransitionProperty::BorderWidth
                        => 
    nsCSSPropertyID::eCSSProperty_border_width,
                    TransitionProperty::BorderTop
                        => 
    nsCSSPropertyID::eCSSProperty_border_top,
                    TransitionProperty::BorderLeft
                        => 
    nsCSSPropertyID::eCSSProperty_border_left,
                    TransitionProperty::BorderBottom
                        => 
    nsCSSPropertyID::eCSSProperty_border_bottom,
                    TransitionProperty::BorderRight
                        => 
    nsCSSPropertyID::eCSSProperty_border_right,
                    TransitionProperty::Border
                        => 
    nsCSSPropertyID::eCSSProperty_border,
                    TransitionProperty::BorderRadius
                        => 
    nsCSSPropertyID::eCSSProperty_border_radius,
                    TransitionProperty::MozTransform
                        => 
    nsCSSPropertyID::eCSSProperty__moz_transform,
                    TransitionProperty::Columns
                        => 
    nsCSSPropertyID::eCSSProperty_columns,
                    TransitionProperty::ColumnRule
                        => 
    nsCSSPropertyID::eCSSProperty_column_rule,
                    TransitionProperty::Font
                        => 
    nsCSSPropertyID::eCSSProperty_font,
                    TransitionProperty::TextEmphasis
                        => 
    nsCSSPropertyID::eCSSProperty_text_emphasis,
                    TransitionProperty::WebkitTextStroke
                        => 
    nsCSSPropertyID::eCSSProperty__webkit_text_stroke,
                    TransitionProperty::Margin
                        => 
    nsCSSPropertyID::eCSSProperty_margin,
                    TransitionProperty::Mask
                        => 
    nsCSSPropertyID::eCSSProperty_mask,
                    TransitionProperty::MaskPosition
                        => 
    nsCSSPropertyID::eCSSProperty_mask_position,
                    TransitionProperty::Outline
                        => 
    nsCSSPropertyID::eCSSProperty_outline,
                    TransitionProperty::MozOutlineRadius
                        => 
    nsCSSPropertyID::eCSSProperty__moz_outline_radius,
                    TransitionProperty::Padding
                        => 
    nsCSSPropertyID::eCSSProperty_padding,
                    TransitionProperty::Flex
                        => 
    nsCSSPropertyID::eCSSProperty_flex,
                    TransitionProperty::GridGap
                        => 
    nsCSSPropertyID::eCSSProperty_grid_gap,
                    TransitionProperty::TextDecoration
                        => 
    nsCSSPropertyID::eCSSProperty_text_decoration,
            TransitionProperty::All => nsCSSPropertyID::eCSSPropertyExtra_all_properties,
            _ => panic!("Unconvertable Servo transition property: {:?}", transition_property),
        }
    }
}

/// Convert nsCSSPropertyID to TransitionProperty
#[cfg(feature = "gecko")]
#[allow(non_upper_case_globals)]
impl From<nsCSSPropertyID> for TransitionProperty {
    fn from(property: nsCSSPropertyID) -> TransitionProperty {
        let unsupported = match property {
                    
    nsCSSPropertyID::eCSSProperty_background_color
                        => return TransitionProperty::BackgroundColor,
                    
    nsCSSPropertyID::eCSSProperty_background_image
                        => "background_image",
                    
    nsCSSPropertyID::eCSSProperty_background_position_x
                        => return TransitionProperty::BackgroundPositionX,
                    
    nsCSSPropertyID::eCSSProperty_background_position_y
                        => return TransitionProperty::BackgroundPositionY,
                    
    nsCSSPropertyID::eCSSProperty_background_repeat
                        => "background_repeat",
                    
    nsCSSPropertyID::eCSSProperty_background_attachment
                        => "background_attachment",
                    
    nsCSSPropertyID::eCSSProperty_background_clip
                        => "background_clip",
                    
    nsCSSPropertyID::eCSSProperty_background_origin
                        => "background_origin",
                    
    nsCSSPropertyID::eCSSProperty_background_size
                        => return TransitionProperty::BackgroundSize,
                    
    nsCSSPropertyID::eCSSProperty_background_blend_mode
                        => "background_blend_mode",
                    
    nsCSSPropertyID::eCSSProperty_border_top_color
                        => return TransitionProperty::BorderTopColor,
                    
    nsCSSPropertyID::eCSSProperty_border_top_style
                        => "border_top_style",
                    
    nsCSSPropertyID::eCSSProperty_border_top_width
                        => return TransitionProperty::BorderTopWidth,
                    
    nsCSSPropertyID::eCSSProperty_border_left_color
                        => return TransitionProperty::BorderLeftColor,
                    
    nsCSSPropertyID::eCSSProperty_border_left_style
                        => "border_left_style",
                    
    nsCSSPropertyID::eCSSProperty_border_left_width
                        => return TransitionProperty::BorderLeftWidth,
                    
    nsCSSPropertyID::eCSSProperty_border_bottom_color
                        => return TransitionProperty::BorderBottomColor,
                    
    nsCSSPropertyID::eCSSProperty_border_bottom_style
                        => "border_bottom_style",
                    
    nsCSSPropertyID::eCSSProperty_border_bottom_width
                        => return TransitionProperty::BorderBottomWidth,
                    
    nsCSSPropertyID::eCSSProperty_border_right_color
                        => return TransitionProperty::BorderRightColor,
                    
    nsCSSPropertyID::eCSSProperty_border_right_style
                        => "border_right_style",
                    
    nsCSSPropertyID::eCSSProperty_border_right_width
                        => return TransitionProperty::BorderRightWidth,
                    
    nsCSSPropertyID::eCSSProperty_border_block_start_color
                        => "border_block_start_color",
                    
    nsCSSPropertyID::eCSSProperty_border_block_start_style
                        => "border_block_start_style",
                    
    nsCSSPropertyID::eCSSProperty_border_block_start_width
                        => "border_block_start_width",
                    
    nsCSSPropertyID::eCSSProperty_border_block_end_color
                        => "border_block_end_color",
                    
    nsCSSPropertyID::eCSSProperty_border_block_end_style
                        => "border_block_end_style",
                    
    nsCSSPropertyID::eCSSProperty_border_block_end_width
                        => "border_block_end_width",
                    
    nsCSSPropertyID::eCSSProperty_border_inline_start_color
                        => "border_inline_start_color",
                    
    nsCSSPropertyID::eCSSProperty_border_inline_start_style
                        => "border_inline_start_style",
                    
    nsCSSPropertyID::eCSSProperty_border_inline_start_width
                        => "border_inline_start_width",
                    
    nsCSSPropertyID::eCSSProperty_border_inline_end_color
                        => "border_inline_end_color",
                    
    nsCSSPropertyID::eCSSProperty_border_inline_end_style
                        => "border_inline_end_style",
                    
    nsCSSPropertyID::eCSSProperty_border_inline_end_width
                        => "border_inline_end_width",
                    
    nsCSSPropertyID::eCSSProperty_border_top_left_radius
                        => return TransitionProperty::BorderTopLeftRadius,
                    
    nsCSSPropertyID::eCSSProperty_border_top_right_radius
                        => return TransitionProperty::BorderTopRightRadius,
                    
    nsCSSPropertyID::eCSSProperty_border_bottom_right_radius
                        => return TransitionProperty::BorderBottomRightRadius,
                    
    nsCSSPropertyID::eCSSProperty_border_bottom_left_radius
                        => return TransitionProperty::BorderBottomLeftRadius,
                    
    nsCSSPropertyID::eCSSProperty__moz_border_top_colors
                        => "_moz_border_top_colors",
                    
    nsCSSPropertyID::eCSSProperty__moz_border_left_colors
                        => "_moz_border_left_colors",
                    
    nsCSSPropertyID::eCSSProperty__moz_border_bottom_colors
                        => "_moz_border_bottom_colors",
                    
    nsCSSPropertyID::eCSSProperty__moz_border_right_colors
                        => "_moz_border_right_colors",
                    
    nsCSSPropertyID::eCSSProperty_box_decoration_break
                        => "box_decoration_break",
                    
    nsCSSPropertyID::eCSSProperty__moz_float_edge
                        => "_moz_float_edge",
                    
    nsCSSPropertyID::eCSSProperty_border_image_source
                        => "border_image_source",
                    
    nsCSSPropertyID::eCSSProperty_border_image_outset
                        => "border_image_outset",
                    
    nsCSSPropertyID::eCSSProperty_border_image_repeat
                        => "border_image_repeat",
                    
    nsCSSPropertyID::eCSSProperty_border_image_width
                        => "border_image_width",
                    
    nsCSSPropertyID::eCSSProperty_border_image_slice
                        => "border_image_slice",
                    
    nsCSSPropertyID::eCSSProperty_display
                        => "display",
                    
    nsCSSPropertyID::eCSSProperty__moz_top_layer
                        => "_moz_top_layer",
                    
    nsCSSPropertyID::eCSSProperty_position
                        => "position",
                    
    nsCSSPropertyID::eCSSProperty_float_
                        => "float",
                    
    nsCSSPropertyID::eCSSProperty_clear
                        => "clear",
                    
    nsCSSPropertyID::eCSSProperty_vertical_align
                        => return TransitionProperty::VerticalAlign,
                    
    nsCSSPropertyID::eCSSProperty_overflow_clip_box
                        => "overflow_clip_box",
                    
    nsCSSPropertyID::eCSSProperty_overflow_x
                        => "overflow_x",
                    
    nsCSSPropertyID::eCSSProperty_overflow_y
                        => "overflow_y",
                    
    nsCSSPropertyID::eCSSProperty_transition_duration
                        => "transition_duration",
                    
    nsCSSPropertyID::eCSSProperty_transition_timing_function
                        => "transition_timing_function",
                    
    nsCSSPropertyID::eCSSProperty_transition_property
                        => "transition_property",
                    
    nsCSSPropertyID::eCSSProperty_transition_delay
                        => "transition_delay",
                    
    nsCSSPropertyID::eCSSProperty_animation_name
                        => "animation_name",
                    
    nsCSSPropertyID::eCSSProperty_animation_duration
                        => "animation_duration",
                    
    nsCSSPropertyID::eCSSProperty_animation_timing_function
                        => "animation_timing_function",
                    
    nsCSSPropertyID::eCSSProperty_animation_iteration_count
                        => "animation_iteration_count",
                    
    nsCSSPropertyID::eCSSProperty_animation_direction
                        => "animation_direction",
                    
    nsCSSPropertyID::eCSSProperty_animation_play_state
                        => "animation_play_state",
                    
    nsCSSPropertyID::eCSSProperty_animation_fill_mode
                        => "animation_fill_mode",
                    
    nsCSSPropertyID::eCSSProperty_animation_delay
                        => "animation_delay",
                    
    nsCSSPropertyID::eCSSProperty_scroll_snap_points_x
                        => "scroll_snap_points_x",
                    
    nsCSSPropertyID::eCSSProperty_scroll_snap_points_y
                        => "scroll_snap_points_y",
                    
    nsCSSPropertyID::eCSSProperty_scroll_snap_destination
                        => "scroll_snap_destination",
                    
    nsCSSPropertyID::eCSSProperty_scroll_snap_coordinate
                        => "scroll_snap_coordinate",
                    
    nsCSSPropertyID::eCSSProperty_transform
                        => return TransitionProperty::Transform,
                    
    nsCSSPropertyID::eCSSProperty_scroll_behavior
                        => "scroll_behavior",
                    
    nsCSSPropertyID::eCSSProperty_scroll_snap_type_x
                        => "scroll_snap_type_x",
                    
    nsCSSPropertyID::eCSSProperty_scroll_snap_type_y
                        => "scroll_snap_type_y",
                    
    nsCSSPropertyID::eCSSProperty_isolation
                        => "isolation",
                    
    nsCSSPropertyID::eCSSProperty_page_break_after
                        => "page_break_after",
                    
    nsCSSPropertyID::eCSSProperty_page_break_before
                        => "page_break_before",
                    
    nsCSSPropertyID::eCSSProperty_page_break_inside
                        => "page_break_inside",
                    
    nsCSSPropertyID::eCSSProperty_resize
                        => "resize",
                    
    nsCSSPropertyID::eCSSProperty_perspective
                        => return TransitionProperty::Perspective,
                    
    nsCSSPropertyID::eCSSProperty_perspective_origin
                        => return TransitionProperty::PerspectiveOrigin,
                    
    nsCSSPropertyID::eCSSProperty_backface_visibility
                        => "backface_visibility",
                    
    nsCSSPropertyID::eCSSProperty_transform_box
                        => "transform_box",
                    
    nsCSSPropertyID::eCSSProperty_transform_style
                        => "transform_style",
                    
    nsCSSPropertyID::eCSSProperty_transform_origin
                        => return TransitionProperty::TransformOrigin,
                    
    nsCSSPropertyID::eCSSProperty_contain
                        => "contain",
                    
    nsCSSPropertyID::eCSSProperty__moz_appearance
                        => "_moz_appearance",
                    
    nsCSSPropertyID::eCSSProperty__moz_binding
                        => "_moz_binding",
                    
    nsCSSPropertyID::eCSSProperty__moz_orient
                        => "_moz_orient",
                    
    nsCSSPropertyID::eCSSProperty_will_change
                        => "will_change",
                    
    nsCSSPropertyID::eCSSProperty_shape_outside
                        => return TransitionProperty::ShapeOutside,
                    
    nsCSSPropertyID::eCSSProperty_touch_action
                        => "touch_action",
                    
    nsCSSPropertyID::eCSSProperty_color
                        => return TransitionProperty::Color,
                    
    nsCSSPropertyID::eCSSProperty_column_width
                        => return TransitionProperty::ColumnWidth,
                    
    nsCSSPropertyID::eCSSProperty_column_count
                        => return TransitionProperty::ColumnCount,
                    
    nsCSSPropertyID::eCSSProperty_column_gap
                        => return TransitionProperty::ColumnGap,
                    
    nsCSSPropertyID::eCSSProperty_column_fill
                        => "column_fill",
                    
    nsCSSPropertyID::eCSSProperty_column_rule_width
                        => return TransitionProperty::ColumnRuleWidth,
                    
    nsCSSPropertyID::eCSSProperty_column_rule_color
                        => return TransitionProperty::ColumnRuleColor,
                    
    nsCSSPropertyID::eCSSProperty_column_span
                        => "column_span",
                    
    nsCSSPropertyID::eCSSProperty_column_rule_style
                        => "column_rule_style",
                    
    nsCSSPropertyID::eCSSProperty_content
                        => "content",
                    
    nsCSSPropertyID::eCSSProperty_counter_increment
                        => "counter_increment",
                    
    nsCSSPropertyID::eCSSProperty_counter_reset
                        => "counter_reset",
                    
    nsCSSPropertyID::eCSSProperty_opacity
                        => return TransitionProperty::Opacity,
                    
    nsCSSPropertyID::eCSSProperty_box_shadow
                        => return TransitionProperty::BoxShadow,
                    
    nsCSSPropertyID::eCSSProperty_clip
                        => return TransitionProperty::Clip,
                    
    nsCSSPropertyID::eCSSProperty_filter
                        => return TransitionProperty::Filter,
                    
    nsCSSPropertyID::eCSSProperty_mix_blend_mode
                        => "mix_blend_mode",
                    
    nsCSSPropertyID::eCSSProperty_font_family
                        => "font_family",
                    
    nsCSSPropertyID::eCSSProperty_font_style
                        => "font_style",
                    
    nsCSSPropertyID::eCSSProperty_font_variant_caps
                        => "font_variant_caps",
                    
    nsCSSPropertyID::eCSSProperty_font_weight
                        => return TransitionProperty::FontWeight,
                    
    nsCSSPropertyID::eCSSProperty_font_size
                        => return TransitionProperty::FontSize,
                    
    nsCSSPropertyID::eCSSProperty_font_size_adjust
                        => return TransitionProperty::FontSizeAdjust,
                    
    nsCSSPropertyID::eCSSProperty_font_synthesis
                        => "font_synthesis",
                    
    nsCSSPropertyID::eCSSProperty_font_stretch
                        => return TransitionProperty::FontStretch,
                    
    nsCSSPropertyID::eCSSProperty_font_kerning
                        => "font_kerning",
                    
    nsCSSPropertyID::eCSSProperty_font_variant_alternates
                        => "font_variant_alternates",
                    
    nsCSSPropertyID::eCSSProperty_font_variant_east_asian
                        => "font_variant_east_asian",
                    
    nsCSSPropertyID::eCSSProperty_font_variant_ligatures
                        => "font_variant_ligatures",
                    
    nsCSSPropertyID::eCSSProperty_font_variant_numeric
                        => "font_variant_numeric",
                    
    nsCSSPropertyID::eCSSProperty_font_variant_position
                        => "font_variant_position",
                    
    nsCSSPropertyID::eCSSProperty_font_feature_settings
                        => "font_feature_settings",
                    
    nsCSSPropertyID::eCSSProperty_font_variation_settings
                        => return TransitionProperty::FontVariationSettings,
                    
    nsCSSPropertyID::eCSSProperty_font_language_override
                        => "font_language_override",
                    
    nsCSSPropertyID::eCSSProperty__x_lang
                        => "_x_lang",
                    
    nsCSSPropertyID::eCSSProperty__moz_script_size_multiplier
                        => "_moz_script_size_multiplier",
                    
    nsCSSPropertyID::eCSSProperty__moz_script_level
                        => "_moz_script_level",
                    
    nsCSSPropertyID::eCSSProperty__moz_math_display
                        => "_moz_math_display",
                    
    nsCSSPropertyID::eCSSProperty__moz_math_variant
                        => "_moz_math_variant",
                    
    nsCSSPropertyID::eCSSProperty__moz_script_min_size
                        => "_moz_script_min_size",
                    
    nsCSSPropertyID::eCSSProperty__x_text_zoom
                        => "_x_text_zoom",
                    
    nsCSSPropertyID::eCSSProperty__moz_osx_font_smoothing
                        => "_moz_osx_font_smoothing",
                    
    nsCSSPropertyID::eCSSProperty__moz_min_font_size_ratio
                        => "_moz_min_font_size_ratio",
                    
    nsCSSPropertyID::eCSSProperty_visibility
                        => return TransitionProperty::Visibility,
                    
    nsCSSPropertyID::eCSSProperty_writing_mode
                        => "writing_mode",
                    
    nsCSSPropertyID::eCSSProperty_direction
                        => "direction",
                    
    nsCSSPropertyID::eCSSProperty_text_orientation
                        => "text_orientation",
                    
    nsCSSPropertyID::eCSSProperty_color_adjust
                        => "color_adjust",
                    
    nsCSSPropertyID::eCSSProperty_image_rendering
                        => "image_rendering",
                    
    nsCSSPropertyID::eCSSProperty_image_orientation
                        => "image_orientation",
                    
    nsCSSPropertyID::eCSSProperty_border_collapse
                        => "border_collapse",
                    
    nsCSSPropertyID::eCSSProperty_empty_cells
                        => "empty_cells",
                    
    nsCSSPropertyID::eCSSProperty_caption_side
                        => "caption_side",
                    
    nsCSSPropertyID::eCSSProperty_border_spacing
                        => return TransitionProperty::BorderSpacing,
                    
    nsCSSPropertyID::eCSSProperty_line_height
                        => return TransitionProperty::LineHeight,
                    
    nsCSSPropertyID::eCSSProperty_text_transform
                        => "text_transform",
                    
    nsCSSPropertyID::eCSSProperty_hyphens
                        => "hyphens",
                    
    nsCSSPropertyID::eCSSProperty__moz_text_size_adjust
                        => "_moz_text_size_adjust",
                    
    nsCSSPropertyID::eCSSProperty_text_indent
                        => return TransitionProperty::TextIndent,
                    
    nsCSSPropertyID::eCSSProperty_overflow_wrap
                        => "overflow_wrap",
                    
    nsCSSPropertyID::eCSSProperty_word_break
                        => "word_break",
                    
    nsCSSPropertyID::eCSSProperty_text_justify
                        => "text_justify",
                    
    nsCSSPropertyID::eCSSProperty_text_align_last
                        => "text_align_last",
                    
    nsCSSPropertyID::eCSSProperty_text_align
                        => "text_align",
                    
    nsCSSPropertyID::eCSSProperty_letter_spacing
                        => return TransitionProperty::LetterSpacing,
                    
    nsCSSPropertyID::eCSSProperty_word_spacing
                        => return TransitionProperty::WordSpacing,
                    
    nsCSSPropertyID::eCSSProperty_white_space
                        => "white_space",
                    
    nsCSSPropertyID::eCSSProperty_text_shadow
                        => return TransitionProperty::TextShadow,
                    
    nsCSSPropertyID::eCSSProperty_text_emphasis_style
                        => "text_emphasis_style",
                    
    nsCSSPropertyID::eCSSProperty_text_emphasis_position
                        => "text_emphasis_position",
                    
    nsCSSPropertyID::eCSSProperty_text_emphasis_color
                        => return TransitionProperty::TextEmphasisColor,
                    
    nsCSSPropertyID::eCSSProperty__moz_tab_size
                        => return TransitionProperty::MozTabSize,
                    
    nsCSSPropertyID::eCSSProperty__webkit_text_fill_color
                        => return TransitionProperty::WebkitTextFillColor,
                    
    nsCSSPropertyID::eCSSProperty__webkit_text_stroke_color
                        => return TransitionProperty::WebkitTextStrokeColor,
                    
    nsCSSPropertyID::eCSSProperty__webkit_text_stroke_width
                        => "_webkit_text_stroke_width",
                    
    nsCSSPropertyID::eCSSProperty_ruby_align
                        => "ruby_align",
                    
    nsCSSPropertyID::eCSSProperty_ruby_position
                        => "ruby_position",
                    
    nsCSSPropertyID::eCSSProperty_text_combine_upright
                        => "text_combine_upright",
                    
    nsCSSPropertyID::eCSSProperty_text_rendering
                        => "text_rendering",
                    
    nsCSSPropertyID::eCSSProperty__moz_control_character_visibility
                        => "_moz_control_character_visibility",
                    
    nsCSSPropertyID::eCSSProperty_list_style_position
                        => "list_style_position",
                    
    nsCSSPropertyID::eCSSProperty_list_style_type
                        => "list_style_type",
                    
    nsCSSPropertyID::eCSSProperty_list_style_image
                        => "list_style_image",
                    
    nsCSSPropertyID::eCSSProperty_quotes
                        => "quotes",
                    
    nsCSSPropertyID::eCSSProperty__moz_image_region
                        => return TransitionProperty::MozImageRegion,
                    
    nsCSSPropertyID::eCSSProperty_margin_top
                        => return TransitionProperty::MarginTop,
                    
    nsCSSPropertyID::eCSSProperty_margin_left
                        => return TransitionProperty::MarginLeft,
                    
    nsCSSPropertyID::eCSSProperty_margin_bottom
                        => return TransitionProperty::MarginBottom,
                    
    nsCSSPropertyID::eCSSProperty_margin_right
                        => return TransitionProperty::MarginRight,
                    
    nsCSSPropertyID::eCSSProperty_margin_block_start
                        => "margin_block_start",
                    
    nsCSSPropertyID::eCSSProperty_margin_block_end
                        => "margin_block_end",
                    
    nsCSSPropertyID::eCSSProperty_margin_inline_start
                        => "margin_inline_start",
                    
    nsCSSPropertyID::eCSSProperty_margin_inline_end
                        => "margin_inline_end",
                    
    nsCSSPropertyID::eCSSProperty_outline_color
                        => return TransitionProperty::OutlineColor,
                    
    nsCSSPropertyID::eCSSProperty_outline_style
                        => "outline_style",
                    
    nsCSSPropertyID::eCSSProperty_outline_width
                        => return TransitionProperty::OutlineWidth,
                    
    nsCSSPropertyID::eCSSProperty__moz_outline_radius_topleft
                        => return TransitionProperty::MozOutlineRadiusTopleft,
                    
    nsCSSPropertyID::eCSSProperty__moz_outline_radius_topright
                        => return TransitionProperty::MozOutlineRadiusTopright,
                    
    nsCSSPropertyID::eCSSProperty__moz_outline_radius_bottomright
                        => return TransitionProperty::MozOutlineRadiusBottomright,
                    
    nsCSSPropertyID::eCSSProperty__moz_outline_radius_bottomleft
                        => return TransitionProperty::MozOutlineRadiusBottomleft,
                    
    nsCSSPropertyID::eCSSProperty_outline_offset
                        => return TransitionProperty::OutlineOffset,
                    
    nsCSSPropertyID::eCSSProperty_padding_top
                        => return TransitionProperty::PaddingTop,
                    
    nsCSSPropertyID::eCSSProperty_padding_left
                        => return TransitionProperty::PaddingLeft,
                    
    nsCSSPropertyID::eCSSProperty_padding_bottom
                        => return TransitionProperty::PaddingBottom,
                    
    nsCSSPropertyID::eCSSProperty_padding_right
                        => return TransitionProperty::PaddingRight,
                    
    nsCSSPropertyID::eCSSProperty_padding_block_start
                        => "padding_block_start",
                    
    nsCSSPropertyID::eCSSProperty_padding_block_end
                        => "padding_block_end",
                    
    nsCSSPropertyID::eCSSProperty_padding_inline_start
                        => "padding_inline_start",
                    
    nsCSSPropertyID::eCSSProperty_padding_inline_end
                        => "padding_inline_end",
                    
    nsCSSPropertyID::eCSSProperty_cursor
                        => "cursor",
                    
    nsCSSPropertyID::eCSSProperty_pointer_events
                        => "pointer_events",
                    
    nsCSSPropertyID::eCSSProperty__moz_user_input
                        => "_moz_user_input",
                    
    nsCSSPropertyID::eCSSProperty__moz_user_modify
                        => "_moz_user_modify",
                    
    nsCSSPropertyID::eCSSProperty__moz_user_focus
                        => "_moz_user_focus",
                    
    nsCSSPropertyID::eCSSProperty_caret_color
                        => return TransitionProperty::CaretColor,
                    
    nsCSSPropertyID::eCSSProperty_top
                        => return TransitionProperty::Top,
                    
    nsCSSPropertyID::eCSSProperty_left
                        => return TransitionProperty::Left,
                    
    nsCSSPropertyID::eCSSProperty_bottom
                        => return TransitionProperty::Bottom,
                    
    nsCSSPropertyID::eCSSProperty_right
                        => return TransitionProperty::Right,
                    
    nsCSSPropertyID::eCSSProperty_offset_block_start
                        => "offset_block_start",
                    
    nsCSSPropertyID::eCSSProperty_offset_block_end
                        => "offset_block_end",
                    
    nsCSSPropertyID::eCSSProperty_offset_inline_start
                        => "offset_inline_start",
                    
    nsCSSPropertyID::eCSSProperty_offset_inline_end
                        => "offset_inline_end",
                    
    nsCSSPropertyID::eCSSProperty_z_index
                        => return TransitionProperty::ZIndex,
                    
    nsCSSPropertyID::eCSSProperty_flex_direction
                        => "flex_direction",
                    
    nsCSSPropertyID::eCSSProperty_flex_wrap
                        => "flex_wrap",
                    
    nsCSSPropertyID::eCSSProperty_justify_content
                        => "justify_content",
                    
    nsCSSPropertyID::eCSSProperty_align_content
                        => "align_content",
                    
    nsCSSPropertyID::eCSSProperty_align_items
                        => "align_items",
                    
    nsCSSPropertyID::eCSSProperty_justify_items
                        => "justify_items",
                    
    nsCSSPropertyID::eCSSProperty_flex_grow
                        => return TransitionProperty::FlexGrow,
                    
    nsCSSPropertyID::eCSSProperty_flex_shrink
                        => return TransitionProperty::FlexShrink,
                    
    nsCSSPropertyID::eCSSProperty_align_self
                        => "align_self",
                    
    nsCSSPropertyID::eCSSProperty_justify_self
                        => "justify_self",
                    
    nsCSSPropertyID::eCSSProperty_order
                        => return TransitionProperty::Order,
                    
    nsCSSPropertyID::eCSSProperty_flex_basis
                        => return TransitionProperty::FlexBasis,
                    
    nsCSSPropertyID::eCSSProperty_width
                        => return TransitionProperty::Width,
                    
    nsCSSPropertyID::eCSSProperty_min_width
                        => return TransitionProperty::MinWidth,
                    
    nsCSSPropertyID::eCSSProperty_max_width
                        => return TransitionProperty::MaxWidth,
                    
    nsCSSPropertyID::eCSSProperty_height
                        => return TransitionProperty::Height,
                    
    nsCSSPropertyID::eCSSProperty_min_height
                        => return TransitionProperty::MinHeight,
                    
    nsCSSPropertyID::eCSSProperty_max_height
                        => return TransitionProperty::MaxHeight,
                    
    nsCSSPropertyID::eCSSProperty_block_size
                        => "block_size",
                    
    nsCSSPropertyID::eCSSProperty_min_block_size
                        => "min_block_size",
                    
    nsCSSPropertyID::eCSSProperty_max_block_size
                        => "max_block_size",
                    
    nsCSSPropertyID::eCSSProperty_inline_size
                        => "inline_size",
                    
    nsCSSPropertyID::eCSSProperty_min_inline_size
                        => "min_inline_size",
                    
    nsCSSPropertyID::eCSSProperty_max_inline_size
                        => "max_inline_size",
                    
    nsCSSPropertyID::eCSSProperty_box_sizing
                        => "box_sizing",
                    
    nsCSSPropertyID::eCSSProperty_object_fit
                        => "object_fit",
                    
    nsCSSPropertyID::eCSSProperty_object_position
                        => return TransitionProperty::ObjectPosition,
                    
    nsCSSPropertyID::eCSSProperty_grid_row_gap
                        => return TransitionProperty::GridRowGap,
                    
    nsCSSPropertyID::eCSSProperty_grid_row_start
                        => "grid_row_start",
                    
    nsCSSPropertyID::eCSSProperty_grid_row_end
                        => "grid_row_end",
                    
    nsCSSPropertyID::eCSSProperty_grid_auto_rows
                        => "grid_auto_rows",
                    
    nsCSSPropertyID::eCSSProperty_grid_template_rows
                        => "grid_template_rows",
                    
    nsCSSPropertyID::eCSSProperty_grid_column_gap
                        => return TransitionProperty::GridColumnGap,
                    
    nsCSSPropertyID::eCSSProperty_grid_column_start
                        => "grid_column_start",
                    
    nsCSSPropertyID::eCSSProperty_grid_column_end
                        => "grid_column_end",
                    
    nsCSSPropertyID::eCSSProperty_grid_auto_columns
                        => "grid_auto_columns",
                    
    nsCSSPropertyID::eCSSProperty_grid_template_columns
                        => "grid_template_columns",
                    
    nsCSSPropertyID::eCSSProperty_grid_auto_flow
                        => "grid_auto_flow",
                    
    nsCSSPropertyID::eCSSProperty_grid_template_areas
                        => "grid_template_areas",
                    
    nsCSSPropertyID::eCSSProperty_table_layout
                        => "table_layout",
                    
    nsCSSPropertyID::eCSSProperty__x_span
                        => "_x_span",
                    
    nsCSSPropertyID::eCSSProperty_text_overflow
                        => "text_overflow",
                    
    nsCSSPropertyID::eCSSProperty_unicode_bidi
                        => "unicode_bidi",
                    
    nsCSSPropertyID::eCSSProperty_text_decoration_line
                        => "text_decoration_line",
                    
    nsCSSPropertyID::eCSSProperty_text_decoration_style
                        => "text_decoration_style",
                    
    nsCSSPropertyID::eCSSProperty_text_decoration_color
                        => return TransitionProperty::TextDecorationColor,
                    
    nsCSSPropertyID::eCSSProperty_initial_letter
                        => "initial_letter",
                    
    nsCSSPropertyID::eCSSProperty_ime_mode
                        => "ime_mode",
                    
    nsCSSPropertyID::eCSSProperty__moz_user_select
                        => "_moz_user_select",
                    
    nsCSSPropertyID::eCSSProperty__moz_window_dragging
                        => "_moz_window_dragging",
                    
    nsCSSPropertyID::eCSSProperty__moz_window_shadow
                        => "_moz_window_shadow",
                    
    nsCSSPropertyID::eCSSProperty__moz_force_broken_image_icon
                        => "_moz_force_broken_image_icon",
                    
    nsCSSPropertyID::eCSSProperty_text_anchor
                        => "text_anchor",
                    
    nsCSSPropertyID::eCSSProperty_color_interpolation
                        => "color_interpolation",
                    
    nsCSSPropertyID::eCSSProperty_color_interpolation_filters
                        => "color_interpolation_filters",
                    
    nsCSSPropertyID::eCSSProperty_fill
                        => return TransitionProperty::Fill,
                    
    nsCSSPropertyID::eCSSProperty_fill_opacity
                        => return TransitionProperty::FillOpacity,
                    
    nsCSSPropertyID::eCSSProperty_fill_rule
                        => "fill_rule",
                    
    nsCSSPropertyID::eCSSProperty_shape_rendering
                        => "shape_rendering",
                    
    nsCSSPropertyID::eCSSProperty_stroke
                        => return TransitionProperty::Stroke,
                    
    nsCSSPropertyID::eCSSProperty_stroke_width
                        => return TransitionProperty::StrokeWidth,
                    
    nsCSSPropertyID::eCSSProperty_stroke_linecap
                        => "stroke_linecap",
                    
    nsCSSPropertyID::eCSSProperty_stroke_linejoin
                        => "stroke_linejoin",
                    
    nsCSSPropertyID::eCSSProperty_stroke_miterlimit
                        => return TransitionProperty::StrokeMiterlimit,
                    
    nsCSSPropertyID::eCSSProperty_stroke_opacity
                        => return TransitionProperty::StrokeOpacity,
                    
    nsCSSPropertyID::eCSSProperty_stroke_dasharray
                        => return TransitionProperty::StrokeDasharray,
                    
    nsCSSPropertyID::eCSSProperty_stroke_dashoffset
                        => return TransitionProperty::StrokeDashoffset,
                    
    nsCSSPropertyID::eCSSProperty_clip_rule
                        => "clip_rule",
                    
    nsCSSPropertyID::eCSSProperty_marker_start
                        => "marker_start",
                    
    nsCSSPropertyID::eCSSProperty_marker_mid
                        => "marker_mid",
                    
    nsCSSPropertyID::eCSSProperty_marker_end
                        => "marker_end",
                    
    nsCSSPropertyID::eCSSProperty_paint_order
                        => "paint_order",
                    
    nsCSSPropertyID::eCSSProperty__moz_context_properties
                        => "_moz_context_properties",
                    
    nsCSSPropertyID::eCSSProperty_dominant_baseline
                        => "dominant_baseline",
                    
    nsCSSPropertyID::eCSSProperty_vector_effect
                        => "vector_effect",
                    
    nsCSSPropertyID::eCSSProperty_stop_color
                        => return TransitionProperty::StopColor,
                    
    nsCSSPropertyID::eCSSProperty_stop_opacity
                        => return TransitionProperty::StopOpacity,
                    
    nsCSSPropertyID::eCSSProperty_flood_color
                        => return TransitionProperty::FloodColor,
                    
    nsCSSPropertyID::eCSSProperty_flood_opacity
                        => return TransitionProperty::FloodOpacity,
                    
    nsCSSPropertyID::eCSSProperty_lighting_color
                        => return TransitionProperty::LightingColor,
                    
    nsCSSPropertyID::eCSSProperty_mask_type
                        => "mask_type",
                    
    nsCSSPropertyID::eCSSProperty_clip_path
                        => return TransitionProperty::ClipPath,
                    
    nsCSSPropertyID::eCSSProperty_mask_mode
                        => "mask_mode",
                    
    nsCSSPropertyID::eCSSProperty_mask_repeat
                        => "mask_repeat",
                    
    nsCSSPropertyID::eCSSProperty_mask_position_x
                        => return TransitionProperty::MaskPositionX,
                    
    nsCSSPropertyID::eCSSProperty_mask_position_y
                        => return TransitionProperty::MaskPositionY,
                    
    nsCSSPropertyID::eCSSProperty_mask_clip
                        => "mask_clip",
                    
    nsCSSPropertyID::eCSSProperty_mask_origin
                        => "mask_origin",
                    
    nsCSSPropertyID::eCSSProperty_mask_size
                        => return TransitionProperty::MaskSize,
                    
    nsCSSPropertyID::eCSSProperty_mask_composite
                        => "mask_composite",
                    
    nsCSSPropertyID::eCSSProperty_mask_image
                        => "mask_image",
                    
    nsCSSPropertyID::eCSSProperty__moz_box_align
                        => "_moz_box_align",
                    
    nsCSSPropertyID::eCSSProperty__moz_box_direction
                        => "_moz_box_direction",
                    
    nsCSSPropertyID::eCSSProperty__moz_box_flex
                        => return TransitionProperty::MozBoxFlex,
                    
    nsCSSPropertyID::eCSSProperty__moz_box_orient
                        => "_moz_box_orient",
                    
    nsCSSPropertyID::eCSSProperty__moz_box_pack
                        => "_moz_box_pack",
                    
    nsCSSPropertyID::eCSSProperty__moz_stack_sizing
                        => "_moz_stack_sizing",
                    
    nsCSSPropertyID::eCSSProperty__moz_box_ordinal_group
                        => "_moz_box_ordinal_group",
                    
    nsCSSPropertyID::eCSSProperty_background
                        => return TransitionProperty::Background,
                    
    nsCSSPropertyID::eCSSProperty_background_position
                        => return TransitionProperty::BackgroundPosition,
                    
    nsCSSPropertyID::eCSSProperty_border_color
                        => return TransitionProperty::BorderColor,
                    
    nsCSSPropertyID::eCSSProperty_border_style
                        => "border_style",
                    
    nsCSSPropertyID::eCSSProperty_border_width
                        => return TransitionProperty::BorderWidth,
                    
    nsCSSPropertyID::eCSSProperty_border_top
                        => return TransitionProperty::BorderTop,
                    
    nsCSSPropertyID::eCSSProperty_border_left
                        => return TransitionProperty::BorderLeft,
                    
    nsCSSPropertyID::eCSSProperty_border_bottom
                        => return TransitionProperty::BorderBottom,
                    
    nsCSSPropertyID::eCSSProperty_border_right
                        => return TransitionProperty::BorderRight,
                    
    nsCSSPropertyID::eCSSProperty_border_block_start
                        => "border_block_start",
                    
    nsCSSPropertyID::eCSSProperty_border_block_end
                        => "border_block_end",
                    
    nsCSSPropertyID::eCSSProperty_border_inline_start
                        => "border_inline_start",
                    
    nsCSSPropertyID::eCSSProperty_border_inline_end
                        => "border_inline_end",
                    
    nsCSSPropertyID::eCSSProperty_border
                        => return TransitionProperty::Border,
                    
    nsCSSPropertyID::eCSSProperty_border_radius
                        => return TransitionProperty::BorderRadius,
                    
    nsCSSPropertyID::eCSSProperty_border_image
                        => "border_image",
                    
    nsCSSPropertyID::eCSSProperty_overflow
                        => "overflow",
                    
    nsCSSPropertyID::eCSSProperty_transition
                        => "transition",
                    
    nsCSSPropertyID::eCSSProperty_animation
                        => "animation",
                    
    nsCSSPropertyID::eCSSProperty_scroll_snap_type
                        => "scroll_snap_type",
                    
    nsCSSPropertyID::eCSSProperty__moz_transform
                        => return TransitionProperty::MozTransform,
                    
    nsCSSPropertyID::eCSSProperty_columns
                        => return TransitionProperty::Columns,
                    
    nsCSSPropertyID::eCSSProperty_column_rule
                        => return TransitionProperty::ColumnRule,
                    
    nsCSSPropertyID::eCSSProperty_font
                        => return TransitionProperty::Font,
                    
    nsCSSPropertyID::eCSSProperty_font_variant
                        => "font_variant",
                    
    nsCSSPropertyID::eCSSProperty_text_emphasis
                        => return TransitionProperty::TextEmphasis,
                    
    nsCSSPropertyID::eCSSProperty__webkit_text_stroke
                        => return TransitionProperty::WebkitTextStroke,
                    
    nsCSSPropertyID::eCSSProperty_list_style
                        => "list_style",
                    
    nsCSSPropertyID::eCSSProperty_margin
                        => return TransitionProperty::Margin,
                    
    nsCSSPropertyID::eCSSProperty_mask
                        => return TransitionProperty::Mask,
                    
    nsCSSPropertyID::eCSSProperty_mask_position
                        => return TransitionProperty::MaskPosition,
                    
    nsCSSPropertyID::eCSSProperty_outline
                        => return TransitionProperty::Outline,
                    
    nsCSSPropertyID::eCSSProperty__moz_outline_radius
                        => return TransitionProperty::MozOutlineRadius,
                    
    nsCSSPropertyID::eCSSProperty_padding
                        => return TransitionProperty::Padding,
                    
    nsCSSPropertyID::eCSSProperty_flex_flow
                        => "flex_flow",
                    
    nsCSSPropertyID::eCSSProperty_flex
                        => return TransitionProperty::Flex,
                    
    nsCSSPropertyID::eCSSProperty_grid_gap
                        => return TransitionProperty::GridGap,
                    
    nsCSSPropertyID::eCSSProperty_grid_row
                        => "grid_row",
                    
    nsCSSPropertyID::eCSSProperty_grid_column
                        => "grid_column",
                    
    nsCSSPropertyID::eCSSProperty_grid_area
                        => "grid_area",
                    
    nsCSSPropertyID::eCSSProperty_grid_template
                        => "grid_template",
                    
    nsCSSPropertyID::eCSSProperty_grid
                        => "grid",
                    
    nsCSSPropertyID::eCSSProperty_place_content
                        => "place_content",
                    
    nsCSSPropertyID::eCSSProperty_place_self
                        => "place_self",
                    
    nsCSSPropertyID::eCSSProperty_place_items
                        => "place_items",
                    
    nsCSSPropertyID::eCSSProperty_marker
                        => "marker",
                    
    nsCSSPropertyID::eCSSProperty_text_decoration
                        => return TransitionProperty::TextDecoration,
            nsCSSPropertyID::eCSSPropertyExtra_all_properties => return TransitionProperty::All,
            _ => panic!("Unconvertable nsCSSPropertyID: {:?}", property),
        };
        TransitionProperty::Unsupported(CustomIdent(Atom::from(unsupported)))
    }
}

/// Returns true if this nsCSSPropertyID is one of the transitionable properties.
#[cfg(feature = "gecko")]
pub fn nscsspropertyid_is_transitionable(property: nsCSSPropertyID) -> bool {
    match property {
                
    nsCSSPropertyID::eCSSProperty_background_color => true,
                
    nsCSSPropertyID::eCSSProperty_background_position_x => true,
                
    nsCSSPropertyID::eCSSProperty_background_position_y => true,
                
    nsCSSPropertyID::eCSSProperty_background_size => true,
                
    nsCSSPropertyID::eCSSProperty_border_top_color => true,
                
    nsCSSPropertyID::eCSSProperty_border_top_width => true,
                
    nsCSSPropertyID::eCSSProperty_border_left_color => true,
                
    nsCSSPropertyID::eCSSProperty_border_left_width => true,
                
    nsCSSPropertyID::eCSSProperty_border_bottom_color => true,
                
    nsCSSPropertyID::eCSSProperty_border_bottom_width => true,
                
    nsCSSPropertyID::eCSSProperty_border_right_color => true,
                
    nsCSSPropertyID::eCSSProperty_border_right_width => true,
                
    nsCSSPropertyID::eCSSProperty_border_top_left_radius => true,
                
    nsCSSPropertyID::eCSSProperty_border_top_right_radius => true,
                
    nsCSSPropertyID::eCSSProperty_border_bottom_right_radius => true,
                
    nsCSSPropertyID::eCSSProperty_border_bottom_left_radius => true,
                
    nsCSSPropertyID::eCSSProperty_vertical_align => true,
                
    nsCSSPropertyID::eCSSProperty_transform => true,
                
    nsCSSPropertyID::eCSSProperty_perspective => true,
                
    nsCSSPropertyID::eCSSProperty_perspective_origin => true,
                
    nsCSSPropertyID::eCSSProperty_transform_origin => true,
                
    nsCSSPropertyID::eCSSProperty_shape_outside => true,
                
    nsCSSPropertyID::eCSSProperty_color => true,
                
    nsCSSPropertyID::eCSSProperty_column_width => true,
                
    nsCSSPropertyID::eCSSProperty_column_count => true,
                
    nsCSSPropertyID::eCSSProperty_column_gap => true,
                
    nsCSSPropertyID::eCSSProperty_column_rule_width => true,
                
    nsCSSPropertyID::eCSSProperty_column_rule_color => true,
                
    nsCSSPropertyID::eCSSProperty_opacity => true,
                
    nsCSSPropertyID::eCSSProperty_box_shadow => true,
                
    nsCSSPropertyID::eCSSProperty_clip => true,
                
    nsCSSPropertyID::eCSSProperty_filter => true,
                
    nsCSSPropertyID::eCSSProperty_font_weight => true,
                
    nsCSSPropertyID::eCSSProperty_font_size => true,
                
    nsCSSPropertyID::eCSSProperty_font_size_adjust => true,
                
    nsCSSPropertyID::eCSSProperty_font_stretch => true,
                
    nsCSSPropertyID::eCSSProperty_font_variation_settings => true,
                
    nsCSSPropertyID::eCSSProperty_visibility => true,
                
    nsCSSPropertyID::eCSSProperty_border_spacing => true,
                
    nsCSSPropertyID::eCSSProperty_line_height => true,
                
    nsCSSPropertyID::eCSSProperty_text_indent => true,
                
    nsCSSPropertyID::eCSSProperty_letter_spacing => true,
                
    nsCSSPropertyID::eCSSProperty_word_spacing => true,
                
    nsCSSPropertyID::eCSSProperty_text_shadow => true,
                
    nsCSSPropertyID::eCSSProperty_text_emphasis_color => true,
                
    nsCSSPropertyID::eCSSProperty__moz_tab_size => true,
                
    nsCSSPropertyID::eCSSProperty__webkit_text_fill_color => true,
                
    nsCSSPropertyID::eCSSProperty__webkit_text_stroke_color => true,
                
    nsCSSPropertyID::eCSSProperty__moz_image_region => true,
                
    nsCSSPropertyID::eCSSProperty_margin_top => true,
                
    nsCSSPropertyID::eCSSProperty_margin_left => true,
                
    nsCSSPropertyID::eCSSProperty_margin_bottom => true,
                
    nsCSSPropertyID::eCSSProperty_margin_right => true,
                
    nsCSSPropertyID::eCSSProperty_outline_color => true,
                
    nsCSSPropertyID::eCSSProperty_outline_width => true,
                
    nsCSSPropertyID::eCSSProperty__moz_outline_radius_topleft => true,
                
    nsCSSPropertyID::eCSSProperty__moz_outline_radius_topright => true,
                
    nsCSSPropertyID::eCSSProperty__moz_outline_radius_bottomright => true,
                
    nsCSSPropertyID::eCSSProperty__moz_outline_radius_bottomleft => true,
                
    nsCSSPropertyID::eCSSProperty_outline_offset => true,
                
    nsCSSPropertyID::eCSSProperty_padding_top => true,
                
    nsCSSPropertyID::eCSSProperty_padding_left => true,
                
    nsCSSPropertyID::eCSSProperty_padding_bottom => true,
                
    nsCSSPropertyID::eCSSProperty_padding_right => true,
                
    nsCSSPropertyID::eCSSProperty_caret_color => true,
                
    nsCSSPropertyID::eCSSProperty_top => true,
                
    nsCSSPropertyID::eCSSProperty_left => true,
                
    nsCSSPropertyID::eCSSProperty_bottom => true,
                
    nsCSSPropertyID::eCSSProperty_right => true,
                
    nsCSSPropertyID::eCSSProperty_z_index => true,
                
    nsCSSPropertyID::eCSSProperty_flex_grow => true,
                
    nsCSSPropertyID::eCSSProperty_flex_shrink => true,
                
    nsCSSPropertyID::eCSSProperty_order => true,
                
    nsCSSPropertyID::eCSSProperty_flex_basis => true,
                
    nsCSSPropertyID::eCSSProperty_width => true,
                
    nsCSSPropertyID::eCSSProperty_min_width => true,
                
    nsCSSPropertyID::eCSSProperty_max_width => true,
                
    nsCSSPropertyID::eCSSProperty_height => true,
                
    nsCSSPropertyID::eCSSProperty_min_height => true,
                
    nsCSSPropertyID::eCSSProperty_max_height => true,
                
    nsCSSPropertyID::eCSSProperty_object_position => true,
                
    nsCSSPropertyID::eCSSProperty_grid_row_gap => true,
                
    nsCSSPropertyID::eCSSProperty_grid_column_gap => true,
                
    nsCSSPropertyID::eCSSProperty_text_decoration_color => true,
                
    nsCSSPropertyID::eCSSProperty_fill => true,
                
    nsCSSPropertyID::eCSSProperty_fill_opacity => true,
                
    nsCSSPropertyID::eCSSProperty_stroke => true,
                
    nsCSSPropertyID::eCSSProperty_stroke_width => true,
                
    nsCSSPropertyID::eCSSProperty_stroke_miterlimit => true,
                
    nsCSSPropertyID::eCSSProperty_stroke_opacity => true,
                
    nsCSSPropertyID::eCSSProperty_stroke_dasharray => true,
                
    nsCSSPropertyID::eCSSProperty_stroke_dashoffset => true,
                
    nsCSSPropertyID::eCSSProperty_stop_color => true,
                
    nsCSSPropertyID::eCSSProperty_stop_opacity => true,
                
    nsCSSPropertyID::eCSSProperty_flood_color => true,
                
    nsCSSPropertyID::eCSSProperty_flood_opacity => true,
                
    nsCSSPropertyID::eCSSProperty_lighting_color => true,
                
    nsCSSPropertyID::eCSSProperty_clip_path => true,
                
    nsCSSPropertyID::eCSSProperty_mask_position_x => true,
                
    nsCSSPropertyID::eCSSProperty_mask_position_y => true,
                
    nsCSSPropertyID::eCSSProperty_mask_size => true,
                
    nsCSSPropertyID::eCSSProperty__moz_box_flex => true,
                
    nsCSSPropertyID::eCSSProperty_background => true,
                
    nsCSSPropertyID::eCSSProperty_background_position => true,
                
    nsCSSPropertyID::eCSSProperty_border_color => true,
                
    nsCSSPropertyID::eCSSProperty_border_width => true,
                
    nsCSSPropertyID::eCSSProperty_border_top => true,
                
    nsCSSPropertyID::eCSSProperty_border_left => true,
                
    nsCSSPropertyID::eCSSProperty_border_bottom => true,
                
    nsCSSPropertyID::eCSSProperty_border_right => true,
                
    nsCSSPropertyID::eCSSProperty_border => true,
                
    nsCSSPropertyID::eCSSProperty_border_radius => true,
                
    nsCSSPropertyID::eCSSProperty__moz_transform => true,
                
    nsCSSPropertyID::eCSSProperty_columns => true,
                
    nsCSSPropertyID::eCSSProperty_column_rule => true,
                
    nsCSSPropertyID::eCSSProperty_font => true,
                
    nsCSSPropertyID::eCSSProperty_text_emphasis => true,
                
    nsCSSPropertyID::eCSSProperty__webkit_text_stroke => true,
                
    nsCSSPropertyID::eCSSProperty_margin => true,
                
    nsCSSPropertyID::eCSSProperty_mask => true,
                
    nsCSSPropertyID::eCSSProperty_mask_position => true,
                
    nsCSSPropertyID::eCSSProperty_outline => true,
                
    nsCSSPropertyID::eCSSProperty__moz_outline_radius => true,
                
    nsCSSPropertyID::eCSSProperty_padding => true,
                
    nsCSSPropertyID::eCSSProperty_flex => true,
                
    nsCSSPropertyID::eCSSProperty_grid_gap => true,
                
    nsCSSPropertyID::eCSSProperty_text_decoration => true,
        _ => false
    }
}

/// An animated property interpolation between two computed values for that
/// property.
#[cfg(feature = "servo")]
#[derive(Clone, Debug, PartialEq)]
#[cfg_attr(feature = "servo", derive(HeapSizeOf))]
pub enum AnimatedProperty {
            
            /// background-color
            BackgroundColor(<longhands::background_color::computed_value::T as ToAnimatedValue>::AnimatedValue, <longhands::background_color::computed_value::T as ToAnimatedValue>::AnimatedValue),
            
            /// background-image
            BackgroundImage(longhands::background_image::computed_value::T, longhands::background_image::computed_value::T),
            
            /// background-position-x
            BackgroundPositionX(longhands::background_position_x::computed_value::T, longhands::background_position_x::computed_value::T),
            
            /// background-position-y
            BackgroundPositionY(longhands::background_position_y::computed_value::T, longhands::background_position_y::computed_value::T),
            
            /// background-repeat
            BackgroundRepeat(longhands::background_repeat::computed_value::T, longhands::background_repeat::computed_value::T),
            
            /// background-attachment
            BackgroundAttachment(longhands::background_attachment::computed_value::T, longhands::background_attachment::computed_value::T),
            
            /// background-clip
            BackgroundClip(longhands::background_clip::computed_value::T, longhands::background_clip::computed_value::T),
            
            /// background-origin
            BackgroundOrigin(longhands::background_origin::computed_value::T, longhands::background_origin::computed_value::T),
            
            /// background-size
            BackgroundSize(<longhands::background_size::computed_value::T as ToAnimatedValue>::AnimatedValue, <longhands::background_size::computed_value::T as ToAnimatedValue>::AnimatedValue),
            
            /// background-blend-mode
            BackgroundBlendMode(longhands::background_blend_mode::computed_value::T, longhands::background_blend_mode::computed_value::T),
            
            /// border-top-color
            BorderTopColor(<longhands::border_top_color::computed_value::T as ToAnimatedValue>::AnimatedValue, <longhands::border_top_color::computed_value::T as ToAnimatedValue>::AnimatedValue),
            
            /// border-top-style
            BorderTopStyle(longhands::border_top_style::computed_value::T, longhands::border_top_style::computed_value::T),
            
            /// border-top-width
            BorderTopWidth(<longhands::border_top_width::computed_value::T as ToAnimatedValue>::AnimatedValue, <longhands::border_top_width::computed_value::T as ToAnimatedValue>::AnimatedValue),
            
            /// border-left-color
            BorderLeftColor(<longhands::border_left_color::computed_value::T as ToAnimatedValue>::AnimatedValue, <longhands::border_left_color::computed_value::T as ToAnimatedValue>::AnimatedValue),
            
            /// border-left-style
            BorderLeftStyle(longhands::border_left_style::computed_value::T, longhands::border_left_style::computed_value::T),
            
            /// border-left-width
            BorderLeftWidth(<longhands::border_left_width::computed_value::T as ToAnimatedValue>::AnimatedValue, <longhands::border_left_width::computed_value::T as ToAnimatedValue>::AnimatedValue),
            
            /// border-bottom-color
            BorderBottomColor(<longhands::border_bottom_color::computed_value::T as ToAnimatedValue>::AnimatedValue, <longhands::border_bottom_color::computed_value::T as ToAnimatedValue>::AnimatedValue),
            
            /// border-bottom-style
            BorderBottomStyle(longhands::border_bottom_style::computed_value::T, longhands::border_bottom_style::computed_value::T),
            
            /// border-bottom-width
            BorderBottomWidth(<longhands::border_bottom_width::computed_value::T as ToAnimatedValue>::AnimatedValue, <longhands::border_bottom_width::computed_value::T as ToAnimatedValue>::AnimatedValue),
            
            /// border-right-color
            BorderRightColor(<longhands::border_right_color::computed_value::T as ToAnimatedValue>::AnimatedValue, <longhands::border_right_color::computed_value::T as ToAnimatedValue>::AnimatedValue),
            
            /// border-right-style
            BorderRightStyle(longhands::border_right_style::computed_value::T, longhands::border_right_style::computed_value::T),
            
            /// border-right-width
            BorderRightWidth(<longhands::border_right_width::computed_value::T as ToAnimatedValue>::AnimatedValue, <longhands::border_right_width::computed_value::T as ToAnimatedValue>::AnimatedValue),
            
            /// border-top-left-radius
            BorderTopLeftRadius(<longhands::border_top_left_radius::computed_value::T as ToAnimatedValue>::AnimatedValue, <longhands::border_top_left_radius::computed_value::T as ToAnimatedValue>::AnimatedValue),
            
            /// border-top-right-radius
            BorderTopRightRadius(<longhands::border_top_right_radius::computed_value::T as ToAnimatedValue>::AnimatedValue, <longhands::border_top_right_radius::computed_value::T as ToAnimatedValue>::AnimatedValue),
            
            /// border-bottom-right-radius
            BorderBottomRightRadius(<longhands::border_bottom_right_radius::computed_value::T as ToAnimatedValue>::AnimatedValue, <longhands::border_bottom_right_radius::computed_value::T as ToAnimatedValue>::AnimatedValue),
            
            /// border-bottom-left-radius
            BorderBottomLeftRadius(<longhands::border_bottom_left_radius::computed_value::T as ToAnimatedValue>::AnimatedValue, <longhands::border_bottom_left_radius::computed_value::T as ToAnimatedValue>::AnimatedValue),
            
            /// -moz-border-top-colors
            MozBorderTopColors(longhands::_moz_border_top_colors::computed_value::T, longhands::_moz_border_top_colors::computed_value::T),
            
            /// -moz-border-left-colors
            MozBorderLeftColors(longhands::_moz_border_left_colors::computed_value::T, longhands::_moz_border_left_colors::computed_value::T),
            
            /// -moz-border-bottom-colors
            MozBorderBottomColors(longhands::_moz_border_bottom_colors::computed_value::T, longhands::_moz_border_bottom_colors::computed_value::T),
            
            /// -moz-border-right-colors
            MozBorderRightColors(longhands::_moz_border_right_colors::computed_value::T, longhands::_moz_border_right_colors::computed_value::T),
            
            /// box-decoration-break
            BoxDecorationBreak(longhands::box_decoration_break::computed_value::T, longhands::box_decoration_break::computed_value::T),
            
            /// -moz-float-edge
            MozFloatEdge(longhands::_moz_float_edge::computed_value::T, longhands::_moz_float_edge::computed_value::T),
            
            /// border-image-source
            BorderImageSource(longhands::border_image_source::computed_value::T, longhands::border_image_source::computed_value::T),
            
            /// border-image-outset
            BorderImageOutset(longhands::border_image_outset::computed_value::T, longhands::border_image_outset::computed_value::T),
            
            /// border-image-repeat
            BorderImageRepeat(longhands::border_image_repeat::computed_value::T, longhands::border_image_repeat::computed_value::T),
            
            /// border-image-width
            BorderImageWidth(longhands::border_image_width::computed_value::T, longhands::border_image_width::computed_value::T),
            
            /// border-image-slice
            BorderImageSlice(longhands::border_image_slice::computed_value::T, longhands::border_image_slice::computed_value::T),
            
            /// display
            Display(longhands::display::computed_value::T, longhands::display::computed_value::T),
            
            /// position
            Position(longhands::position::computed_value::T, longhands::position::computed_value::T),
            
            /// float
            Float(longhands::float::computed_value::T, longhands::float::computed_value::T),
            
            /// clear
            Clear(longhands::clear::computed_value::T, longhands::clear::computed_value::T),
            
            /// vertical-align
            VerticalAlign(longhands::vertical_align::computed_value::T, longhands::vertical_align::computed_value::T),
            
            /// overflow-clip-box
            OverflowClipBox(longhands::overflow_clip_box::computed_value::T, longhands::overflow_clip_box::computed_value::T),
            
            /// overflow-x
            OverflowX(longhands::overflow_x::computed_value::T, longhands::overflow_x::computed_value::T),
            
            /// overflow-y
            OverflowY(longhands::overflow_y::computed_value::T, longhands::overflow_y::computed_value::T),
            
            /// scroll-snap-points-x
            ScrollSnapPointsX(longhands::scroll_snap_points_x::computed_value::T, longhands::scroll_snap_points_x::computed_value::T),
            
            /// scroll-snap-points-y
            ScrollSnapPointsY(longhands::scroll_snap_points_y::computed_value::T, longhands::scroll_snap_points_y::computed_value::T),
            
            /// scroll-snap-destination
            ScrollSnapDestination(longhands::scroll_snap_destination::computed_value::T, longhands::scroll_snap_destination::computed_value::T),
            
            /// scroll-snap-coordinate
            ScrollSnapCoordinate(longhands::scroll_snap_coordinate::computed_value::T, longhands::scroll_snap_coordinate::computed_value::T),
            
            /// transform
            Transform(longhands::transform::computed_value::T, longhands::transform::computed_value::T),
            
            /// scroll-behavior
            ScrollBehavior(longhands::scroll_behavior::computed_value::T, longhands::scroll_behavior::computed_value::T),
            
            /// scroll-snap-type-x
            ScrollSnapTypeX(longhands::scroll_snap_type_x::computed_value::T, longhands::scroll_snap_type_x::computed_value::T),
            
            /// scroll-snap-type-y
            ScrollSnapTypeY(longhands::scroll_snap_type_y::computed_value::T, longhands::scroll_snap_type_y::computed_value::T),
            
            /// isolation
            Isolation(longhands::isolation::computed_value::T, longhands::isolation::computed_value::T),
            
            /// page-break-after
            PageBreakAfter(longhands::page_break_after::computed_value::T, longhands::page_break_after::computed_value::T),
            
            /// page-break-before
            PageBreakBefore(longhands::page_break_before::computed_value::T, longhands::page_break_before::computed_value::T),
            
            /// page-break-inside
            PageBreakInside(longhands::page_break_inside::computed_value::T, longhands::page_break_inside::computed_value::T),
            
            /// resize
            Resize(longhands::resize::computed_value::T, longhands::resize::computed_value::T),
            
            /// perspective
            Perspective(longhands::perspective::computed_value::T, longhands::perspective::computed_value::T),
            
            /// perspective-origin
            PerspectiveOrigin(longhands::perspective_origin::computed_value::T, longhands::perspective_origin::computed_value::T),
            
            /// backface-visibility
            BackfaceVisibility(longhands::backface_visibility::computed_value::T, longhands::backface_visibility::computed_value::T),
            
            /// transform-box
            TransformBox(longhands::transform_box::computed_value::T, longhands::transform_box::computed_value::T),
            
            /// transform-style
            TransformStyle(longhands::transform_style::computed_value::T, longhands::transform_style::computed_value::T),
            
            /// transform-origin
            TransformOrigin(longhands::transform_origin::computed_value::T, longhands::transform_origin::computed_value::T),
            
            /// contain
            Contain(longhands::contain::computed_value::T, longhands::contain::computed_value::T),
            
            /// -moz-appearance
            MozAppearance(longhands::_moz_appearance::computed_value::T, longhands::_moz_appearance::computed_value::T),
            
            /// -moz-orient
            MozOrient(longhands::_moz_orient::computed_value::T, longhands::_moz_orient::computed_value::T),
            
            /// will-change
            WillChange(longhands::will_change::computed_value::T, longhands::will_change::computed_value::T),
            
            /// shape-outside
            ShapeOutside(longhands::shape_outside::computed_value::T, longhands::shape_outside::computed_value::T),
            
            /// touch-action
            TouchAction(longhands::touch_action::computed_value::T, longhands::touch_action::computed_value::T),
            
            /// color
            Color(<longhands::color::computed_value::T as ToAnimatedValue>::AnimatedValue, <longhands::color::computed_value::T as ToAnimatedValue>::AnimatedValue),
            
            /// column-width
            ColumnWidth(<longhands::column_width::computed_value::T as ToAnimatedValue>::AnimatedValue, <longhands::column_width::computed_value::T as ToAnimatedValue>::AnimatedValue),
            
            /// column-count
            ColumnCount(<longhands::column_count::computed_value::T as ToAnimatedValue>::AnimatedValue, <longhands::column_count::computed_value::T as ToAnimatedValue>::AnimatedValue),
            
            /// column-gap
            ColumnGap(<longhands::column_gap::computed_value::T as ToAnimatedValue>::AnimatedValue, <longhands::column_gap::computed_value::T as ToAnimatedValue>::AnimatedValue),
            
            /// column-fill
            ColumnFill(longhands::column_fill::computed_value::T, longhands::column_fill::computed_value::T),
            
            /// column-rule-width
            ColumnRuleWidth(<longhands::column_rule_width::computed_value::T as ToAnimatedValue>::AnimatedValue, <longhands::column_rule_width::computed_value::T as ToAnimatedValue>::AnimatedValue),
            
            /// column-rule-color
            ColumnRuleColor(<longhands::column_rule_color::computed_value::T as ToAnimatedValue>::AnimatedValue, <longhands::column_rule_color::computed_value::T as ToAnimatedValue>::AnimatedValue),
            
            /// column-span
            ColumnSpan(longhands::column_span::computed_value::T, longhands::column_span::computed_value::T),
            
            /// column-rule-style
            ColumnRuleStyle(longhands::column_rule_style::computed_value::T, longhands::column_rule_style::computed_value::T),
            
            /// content
            Content(longhands::content::computed_value::T, longhands::content::computed_value::T),
            
            /// counter-increment
            CounterIncrement(longhands::counter_increment::computed_value::T, longhands::counter_increment::computed_value::T),
            
            /// counter-reset
            CounterReset(longhands::counter_reset::computed_value::T, longhands::counter_reset::computed_value::T),
            
            /// opacity
            Opacity(longhands::opacity::computed_value::T, longhands::opacity::computed_value::T),
            
            /// box-shadow
            BoxShadow(<longhands::box_shadow::computed_value::T as ToAnimatedValue>::AnimatedValue, <longhands::box_shadow::computed_value::T as ToAnimatedValue>::AnimatedValue),
            
            /// clip
            Clip(longhands::clip::computed_value::T, longhands::clip::computed_value::T),
            
            /// filter
            Filter(<longhands::filter::computed_value::T as ToAnimatedValue>::AnimatedValue, <longhands::filter::computed_value::T as ToAnimatedValue>::AnimatedValue),
            
            /// mix-blend-mode
            MixBlendMode(longhands::mix_blend_mode::computed_value::T, longhands::mix_blend_mode::computed_value::T),
            
            /// font-family
            FontFamily(longhands::font_family::computed_value::T, longhands::font_family::computed_value::T),
            
            /// font-style
            FontStyle(longhands::font_style::computed_value::T, longhands::font_style::computed_value::T),
            
            /// font-variant-caps
            FontVariantCaps(longhands::font_variant_caps::computed_value::T, longhands::font_variant_caps::computed_value::T),
            
            /// font-weight
            FontWeight(longhands::font_weight::computed_value::T, longhands::font_weight::computed_value::T),
            
            /// font-size
            FontSize(<longhands::font_size::computed_value::T as ToAnimatedValue>::AnimatedValue, <longhands::font_size::computed_value::T as ToAnimatedValue>::AnimatedValue),
            
            /// font-size-adjust
            FontSizeAdjust(<longhands::font_size_adjust::computed_value::T as ToAnimatedValue>::AnimatedValue, <longhands::font_size_adjust::computed_value::T as ToAnimatedValue>::AnimatedValue),
            
            /// font-synthesis
            FontSynthesis(longhands::font_synthesis::computed_value::T, longhands::font_synthesis::computed_value::T),
            
            /// font-stretch
            FontStretch(longhands::font_stretch::computed_value::T, longhands::font_stretch::computed_value::T),
            
            /// font-kerning
            FontKerning(longhands::font_kerning::computed_value::T, longhands::font_kerning::computed_value::T),
            
            /// font-variant-alternates
            FontVariantAlternates(longhands::font_variant_alternates::computed_value::T, longhands::font_variant_alternates::computed_value::T),
            
            /// font-variant-east-asian
            FontVariantEastAsian(longhands::font_variant_east_asian::computed_value::T, longhands::font_variant_east_asian::computed_value::T),
            
            /// font-variant-ligatures
            FontVariantLigatures(longhands::font_variant_ligatures::computed_value::T, longhands::font_variant_ligatures::computed_value::T),
            
            /// font-variant-numeric
            FontVariantNumeric(longhands::font_variant_numeric::computed_value::T, longhands::font_variant_numeric::computed_value::T),
            
            /// font-variant-position
            FontVariantPosition(longhands::font_variant_position::computed_value::T, longhands::font_variant_position::computed_value::T),
            
            /// font-feature-settings
            FontFeatureSettings(longhands::font_feature_settings::computed_value::T, longhands::font_feature_settings::computed_value::T),
            
            /// font-variation-settings
            FontVariationSettings(longhands::font_variation_settings::computed_value::T, longhands::font_variation_settings::computed_value::T),
            
            /// font-language-override
            FontLanguageOverride(longhands::font_language_override::computed_value::T, longhands::font_language_override::computed_value::T),
            
            /// -moz-osx-font-smoothing
            MozOsxFontSmoothing(longhands::_moz_osx_font_smoothing::computed_value::T, longhands::_moz_osx_font_smoothing::computed_value::T),
            
            /// visibility
            Visibility(longhands::visibility::computed_value::T, longhands::visibility::computed_value::T),
            
            /// writing-mode
            WritingMode(longhands::writing_mode::computed_value::T, longhands::writing_mode::computed_value::T),
            
            /// direction
            Direction(longhands::direction::computed_value::T, longhands::direction::computed_value::T),
            
            /// text-orientation
            TextOrientation(longhands::text_orientation::computed_value::T, longhands::text_orientation::computed_value::T),
            
            /// color-adjust
            ColorAdjust(longhands::color_adjust::computed_value::T, longhands::color_adjust::computed_value::T),
            
            /// image-rendering
            ImageRendering(longhands::image_rendering::computed_value::T, longhands::image_rendering::computed_value::T),
            
            /// image-orientation
            ImageOrientation(longhands::image_orientation::computed_value::T, longhands::image_orientation::computed_value::T),
            
            /// border-collapse
            BorderCollapse(longhands::border_collapse::computed_value::T, longhands::border_collapse::computed_value::T),
            
            /// empty-cells
            EmptyCells(longhands::empty_cells::computed_value::T, longhands::empty_cells::computed_value::T),
            
            /// caption-side
            CaptionSide(longhands::caption_side::computed_value::T, longhands::caption_side::computed_value::T),
            
            /// border-spacing
            BorderSpacing(<longhands::border_spacing::computed_value::T as ToAnimatedValue>::AnimatedValue, <longhands::border_spacing::computed_value::T as ToAnimatedValue>::AnimatedValue),
            
            /// line-height
            LineHeight(<longhands::line_height::computed_value::T as ToAnimatedValue>::AnimatedValue, <longhands::line_height::computed_value::T as ToAnimatedValue>::AnimatedValue),
            
            /// text-transform
            TextTransform(longhands::text_transform::computed_value::T, longhands::text_transform::computed_value::T),
            
            /// hyphens
            Hyphens(longhands::hyphens::computed_value::T, longhands::hyphens::computed_value::T),
            
            /// -moz-text-size-adjust
            MozTextSizeAdjust(longhands::_moz_text_size_adjust::computed_value::T, longhands::_moz_text_size_adjust::computed_value::T),
            
            /// text-indent
            TextIndent(longhands::text_indent::computed_value::T, longhands::text_indent::computed_value::T),
            
            /// overflow-wrap
            OverflowWrap(longhands::overflow_wrap::computed_value::T, longhands::overflow_wrap::computed_value::T),
            
            /// word-break
            WordBreak(longhands::word_break::computed_value::T, longhands::word_break::computed_value::T),
            
            /// text-justify
            TextJustify(longhands::text_justify::computed_value::T, longhands::text_justify::computed_value::T),
            
            /// text-align-last
            TextAlignLast(longhands::text_align_last::computed_value::T, longhands::text_align_last::computed_value::T),
            
            /// text-align
            TextAlign(longhands::text_align::computed_value::T, longhands::text_align::computed_value::T),
            
            /// letter-spacing
            LetterSpacing(longhands::letter_spacing::computed_value::T, longhands::letter_spacing::computed_value::T),
            
            /// word-spacing
            WordSpacing(longhands::word_spacing::computed_value::T, longhands::word_spacing::computed_value::T),
            
            /// white-space
            WhiteSpace(longhands::white_space::computed_value::T, longhands::white_space::computed_value::T),
            
            /// text-shadow
            TextShadow(<longhands::text_shadow::computed_value::T as ToAnimatedValue>::AnimatedValue, <longhands::text_shadow::computed_value::T as ToAnimatedValue>::AnimatedValue),
            
            /// text-emphasis-style
            TextEmphasisStyle(longhands::text_emphasis_style::computed_value::T, longhands::text_emphasis_style::computed_value::T),
            
            /// text-emphasis-position
            TextEmphasisPosition(longhands::text_emphasis_position::computed_value::T, longhands::text_emphasis_position::computed_value::T),
            
            /// text-emphasis-color
            TextEmphasisColor(<longhands::text_emphasis_color::computed_value::T as ToAnimatedValue>::AnimatedValue, <longhands::text_emphasis_color::computed_value::T as ToAnimatedValue>::AnimatedValue),
            
            /// -moz-tab-size
            MozTabSize(<longhands::_moz_tab_size::computed_value::T as ToAnimatedValue>::AnimatedValue, <longhands::_moz_tab_size::computed_value::T as ToAnimatedValue>::AnimatedValue),
            
            /// -webkit-text-fill-color
            WebkitTextFillColor(<longhands::_webkit_text_fill_color::computed_value::T as ToAnimatedValue>::AnimatedValue, <longhands::_webkit_text_fill_color::computed_value::T as ToAnimatedValue>::AnimatedValue),
            
            /// -webkit-text-stroke-color
            WebkitTextStrokeColor(<longhands::_webkit_text_stroke_color::computed_value::T as ToAnimatedValue>::AnimatedValue, <longhands::_webkit_text_stroke_color::computed_value::T as ToAnimatedValue>::AnimatedValue),
            
            /// -webkit-text-stroke-width
            WebkitTextStrokeWidth(longhands::_webkit_text_stroke_width::computed_value::T, longhands::_webkit_text_stroke_width::computed_value::T),
            
            /// ruby-align
            RubyAlign(longhands::ruby_align::computed_value::T, longhands::ruby_align::computed_value::T),
            
            /// ruby-position
            RubyPosition(longhands::ruby_position::computed_value::T, longhands::ruby_position::computed_value::T),
            
            /// text-combine-upright
            TextCombineUpright(longhands::text_combine_upright::computed_value::T, longhands::text_combine_upright::computed_value::T),
            
            /// text-rendering
            TextRendering(longhands::text_rendering::computed_value::T, longhands::text_rendering::computed_value::T),
            
            /// list-style-position
            ListStylePosition(longhands::list_style_position::computed_value::T, longhands::list_style_position::computed_value::T),
            
            /// list-style-type
            ListStyleType(longhands::list_style_type::computed_value::T, longhands::list_style_type::computed_value::T),
            
            /// list-style-image
            ListStyleImage(longhands::list_style_image::computed_value::T, longhands::list_style_image::computed_value::T),
            
            /// quotes
            Quotes(longhands::quotes::computed_value::T, longhands::quotes::computed_value::T),
            
            /// -moz-image-region
            MozImageRegion(longhands::_moz_image_region::computed_value::T, longhands::_moz_image_region::computed_value::T),
            
            /// margin-top
            MarginTop(longhands::margin_top::computed_value::T, longhands::margin_top::computed_value::T),
            
            /// margin-left
            MarginLeft(longhands::margin_left::computed_value::T, longhands::margin_left::computed_value::T),
            
            /// margin-bottom
            MarginBottom(longhands::margin_bottom::computed_value::T, longhands::margin_bottom::computed_value::T),
            
            /// margin-right
            MarginRight(longhands::margin_right::computed_value::T, longhands::margin_right::computed_value::T),
            
            /// outline-color
            OutlineColor(<longhands::outline_color::computed_value::T as ToAnimatedValue>::AnimatedValue, <longhands::outline_color::computed_value::T as ToAnimatedValue>::AnimatedValue),
            
            /// outline-style
            OutlineStyle(longhands::outline_style::computed_value::T, longhands::outline_style::computed_value::T),
            
            /// outline-width
            OutlineWidth(<longhands::outline_width::computed_value::T as ToAnimatedValue>::AnimatedValue, <longhands::outline_width::computed_value::T as ToAnimatedValue>::AnimatedValue),
            
            /// -moz-outline-radius-topleft
            MozOutlineRadiusTopleft(<longhands::_moz_outline_radius_topleft::computed_value::T as ToAnimatedValue>::AnimatedValue, <longhands::_moz_outline_radius_topleft::computed_value::T as ToAnimatedValue>::AnimatedValue),
            
            /// -moz-outline-radius-topright
            MozOutlineRadiusTopright(<longhands::_moz_outline_radius_topright::computed_value::T as ToAnimatedValue>::AnimatedValue, <longhands::_moz_outline_radius_topright::computed_value::T as ToAnimatedValue>::AnimatedValue),
            
            /// -moz-outline-radius-bottomright
            MozOutlineRadiusBottomright(<longhands::_moz_outline_radius_bottomright::computed_value::T as ToAnimatedValue>::AnimatedValue, <longhands::_moz_outline_radius_bottomright::computed_value::T as ToAnimatedValue>::AnimatedValue),
            
            /// -moz-outline-radius-bottomleft
            MozOutlineRadiusBottomleft(<longhands::_moz_outline_radius_bottomleft::computed_value::T as ToAnimatedValue>::AnimatedValue, <longhands::_moz_outline_radius_bottomleft::computed_value::T as ToAnimatedValue>::AnimatedValue),
            
            /// outline-offset
            OutlineOffset(longhands::outline_offset::computed_value::T, longhands::outline_offset::computed_value::T),
            
            /// padding-top
            PaddingTop(<longhands::padding_top::computed_value::T as ToAnimatedValue>::AnimatedValue, <longhands::padding_top::computed_value::T as ToAnimatedValue>::AnimatedValue),
            
            /// padding-left
            PaddingLeft(<longhands::padding_left::computed_value::T as ToAnimatedValue>::AnimatedValue, <longhands::padding_left::computed_value::T as ToAnimatedValue>::AnimatedValue),
            
            /// padding-bottom
            PaddingBottom(<longhands::padding_bottom::computed_value::T as ToAnimatedValue>::AnimatedValue, <longhands::padding_bottom::computed_value::T as ToAnimatedValue>::AnimatedValue),
            
            /// padding-right
            PaddingRight(<longhands::padding_right::computed_value::T as ToAnimatedValue>::AnimatedValue, <longhands::padding_right::computed_value::T as ToAnimatedValue>::AnimatedValue),
            
            /// cursor
            Cursor(longhands::cursor::computed_value::T, longhands::cursor::computed_value::T),
            
            /// pointer-events
            PointerEvents(longhands::pointer_events::computed_value::T, longhands::pointer_events::computed_value::T),
            
            /// -moz-user-input
            MozUserInput(longhands::_moz_user_input::computed_value::T, longhands::_moz_user_input::computed_value::T),
            
            /// -moz-user-modify
            MozUserModify(longhands::_moz_user_modify::computed_value::T, longhands::_moz_user_modify::computed_value::T),
            
            /// -moz-user-focus
            MozUserFocus(longhands::_moz_user_focus::computed_value::T, longhands::_moz_user_focus::computed_value::T),
            
            /// caret-color
            CaretColor(<longhands::caret_color::computed_value::T as ToAnimatedValue>::AnimatedValue, <longhands::caret_color::computed_value::T as ToAnimatedValue>::AnimatedValue),
            
            /// top
            Top(longhands::top::computed_value::T, longhands::top::computed_value::T),
            
            /// left
            Left(longhands::left::computed_value::T, longhands::left::computed_value::T),
            
            /// bottom
            Bottom(longhands::bottom::computed_value::T, longhands::bottom::computed_value::T),
            
            /// right
            Right(longhands::right::computed_value::T, longhands::right::computed_value::T),
            
            /// z-index
            ZIndex(longhands::z_index::computed_value::T, longhands::z_index::computed_value::T),
            
            /// flex-direction
            FlexDirection(longhands::flex_direction::computed_value::T, longhands::flex_direction::computed_value::T),
            
            /// flex-wrap
            FlexWrap(longhands::flex_wrap::computed_value::T, longhands::flex_wrap::computed_value::T),
            
            /// justify-content
            JustifyContent(longhands::justify_content::computed_value::T, longhands::justify_content::computed_value::T),
            
            /// align-content
            AlignContent(longhands::align_content::computed_value::T, longhands::align_content::computed_value::T),
            
            /// align-items
            AlignItems(longhands::align_items::computed_value::T, longhands::align_items::computed_value::T),
            
            /// justify-items
            JustifyItems(longhands::justify_items::computed_value::T, longhands::justify_items::computed_value::T),
            
            /// flex-grow
            FlexGrow(<longhands::flex_grow::computed_value::T as ToAnimatedValue>::AnimatedValue, <longhands::flex_grow::computed_value::T as ToAnimatedValue>::AnimatedValue),
            
            /// flex-shrink
            FlexShrink(<longhands::flex_shrink::computed_value::T as ToAnimatedValue>::AnimatedValue, <longhands::flex_shrink::computed_value::T as ToAnimatedValue>::AnimatedValue),
            
            /// align-self
            AlignSelf(longhands::align_self::computed_value::T, longhands::align_self::computed_value::T),
            
            /// justify-self
            JustifySelf(longhands::justify_self::computed_value::T, longhands::justify_self::computed_value::T),
            
            /// order
            Order(longhands::order::computed_value::T, longhands::order::computed_value::T),
            
            /// flex-basis
            FlexBasis(<longhands::flex_basis::computed_value::T as ToAnimatedValue>::AnimatedValue, <longhands::flex_basis::computed_value::T as ToAnimatedValue>::AnimatedValue),
            
            /// width
            Width(<longhands::width::computed_value::T as ToAnimatedValue>::AnimatedValue, <longhands::width::computed_value::T as ToAnimatedValue>::AnimatedValue),
            
            /// min-width
            MinWidth(<longhands::min_width::computed_value::T as ToAnimatedValue>::AnimatedValue, <longhands::min_width::computed_value::T as ToAnimatedValue>::AnimatedValue),
            
            /// max-width
            MaxWidth(<longhands::max_width::computed_value::T as ToAnimatedValue>::AnimatedValue, <longhands::max_width::computed_value::T as ToAnimatedValue>::AnimatedValue),
            
            /// height
            Height(<longhands::height::computed_value::T as ToAnimatedValue>::AnimatedValue, <longhands::height::computed_value::T as ToAnimatedValue>::AnimatedValue),
            
            /// min-height
            MinHeight(<longhands::min_height::computed_value::T as ToAnimatedValue>::AnimatedValue, <longhands::min_height::computed_value::T as ToAnimatedValue>::AnimatedValue),
            
            /// max-height
            MaxHeight(<longhands::max_height::computed_value::T as ToAnimatedValue>::AnimatedValue, <longhands::max_height::computed_value::T as ToAnimatedValue>::AnimatedValue),
            
            /// box-sizing
            BoxSizing(longhands::box_sizing::computed_value::T, longhands::box_sizing::computed_value::T),
            
            /// object-fit
            ObjectFit(longhands::object_fit::computed_value::T, longhands::object_fit::computed_value::T),
            
            /// object-position
            ObjectPosition(longhands::object_position::computed_value::T, longhands::object_position::computed_value::T),
            
            /// grid-row-gap
            GridRowGap(<longhands::grid_row_gap::computed_value::T as ToAnimatedValue>::AnimatedValue, <longhands::grid_row_gap::computed_value::T as ToAnimatedValue>::AnimatedValue),
            
            /// grid-row-start
            GridRowStart(longhands::grid_row_start::computed_value::T, longhands::grid_row_start::computed_value::T),
            
            /// grid-row-end
            GridRowEnd(longhands::grid_row_end::computed_value::T, longhands::grid_row_end::computed_value::T),
            
            /// grid-auto-rows
            GridAutoRows(longhands::grid_auto_rows::computed_value::T, longhands::grid_auto_rows::computed_value::T),
            
            /// grid-template-rows
            GridTemplateRows(longhands::grid_template_rows::computed_value::T, longhands::grid_template_rows::computed_value::T),
            
            /// grid-column-gap
            GridColumnGap(<longhands::grid_column_gap::computed_value::T as ToAnimatedValue>::AnimatedValue, <longhands::grid_column_gap::computed_value::T as ToAnimatedValue>::AnimatedValue),
            
            /// grid-column-start
            GridColumnStart(longhands::grid_column_start::computed_value::T, longhands::grid_column_start::computed_value::T),
            
            /// grid-column-end
            GridColumnEnd(longhands::grid_column_end::computed_value::T, longhands::grid_column_end::computed_value::T),
            
            /// grid-auto-columns
            GridAutoColumns(longhands::grid_auto_columns::computed_value::T, longhands::grid_auto_columns::computed_value::T),
            
            /// grid-template-columns
            GridTemplateColumns(longhands::grid_template_columns::computed_value::T, longhands::grid_template_columns::computed_value::T),
            
            /// grid-auto-flow
            GridAutoFlow(longhands::grid_auto_flow::computed_value::T, longhands::grid_auto_flow::computed_value::T),
            
            /// grid-template-areas
            GridTemplateAreas(longhands::grid_template_areas::computed_value::T, longhands::grid_template_areas::computed_value::T),
            
            /// table-layout
            TableLayout(longhands::table_layout::computed_value::T, longhands::table_layout::computed_value::T),
            
            /// text-overflow
            TextOverflow(longhands::text_overflow::computed_value::T, longhands::text_overflow::computed_value::T),
            
            /// unicode-bidi
            UnicodeBidi(longhands::unicode_bidi::computed_value::T, longhands::unicode_bidi::computed_value::T),
            
            /// text-decoration-line
            TextDecorationLine(longhands::text_decoration_line::computed_value::T, longhands::text_decoration_line::computed_value::T),
            
            /// text-decoration-style
            TextDecorationStyle(longhands::text_decoration_style::computed_value::T, longhands::text_decoration_style::computed_value::T),
            
            /// text-decoration-color
            TextDecorationColor(<longhands::text_decoration_color::computed_value::T as ToAnimatedValue>::AnimatedValue, <longhands::text_decoration_color::computed_value::T as ToAnimatedValue>::AnimatedValue),
            
            /// initial-letter
            InitialLetter(longhands::initial_letter::computed_value::T, longhands::initial_letter::computed_value::T),
            
            /// ime-mode
            ImeMode(longhands::ime_mode::computed_value::T, longhands::ime_mode::computed_value::T),
            
            /// -moz-user-select
            MozUserSelect(longhands::_moz_user_select::computed_value::T, longhands::_moz_user_select::computed_value::T),
            
            /// -moz-window-dragging
            MozWindowDragging(longhands::_moz_window_dragging::computed_value::T, longhands::_moz_window_dragging::computed_value::T),
            
            /// -moz-window-shadow
            MozWindowShadow(longhands::_moz_window_shadow::computed_value::T, longhands::_moz_window_shadow::computed_value::T),
            
            /// -moz-force-broken-image-icon
            MozForceBrokenImageIcon(longhands::_moz_force_broken_image_icon::computed_value::T, longhands::_moz_force_broken_image_icon::computed_value::T),
            
            /// text-anchor
            TextAnchor(longhands::text_anchor::computed_value::T, longhands::text_anchor::computed_value::T),
            
            /// color-interpolation
            ColorInterpolation(longhands::color_interpolation::computed_value::T, longhands::color_interpolation::computed_value::T),
            
            /// color-interpolation-filters
            ColorInterpolationFilters(longhands::color_interpolation_filters::computed_value::T, longhands::color_interpolation_filters::computed_value::T),
            
            /// fill
            Fill(<longhands::fill::computed_value::T as ToAnimatedValue>::AnimatedValue, <longhands::fill::computed_value::T as ToAnimatedValue>::AnimatedValue),
            
            /// fill-opacity
            FillOpacity(longhands::fill_opacity::computed_value::T, longhands::fill_opacity::computed_value::T),
            
            /// fill-rule
            FillRule(longhands::fill_rule::computed_value::T, longhands::fill_rule::computed_value::T),
            
            /// shape-rendering
            ShapeRendering(longhands::shape_rendering::computed_value::T, longhands::shape_rendering::computed_value::T),
            
            /// stroke
            Stroke(<longhands::stroke::computed_value::T as ToAnimatedValue>::AnimatedValue, <longhands::stroke::computed_value::T as ToAnimatedValue>::AnimatedValue),
            
            /// stroke-width
            StrokeWidth(<longhands::stroke_width::computed_value::T as ToAnimatedValue>::AnimatedValue, <longhands::stroke_width::computed_value::T as ToAnimatedValue>::AnimatedValue),
            
            /// stroke-linecap
            StrokeLinecap(longhands::stroke_linecap::computed_value::T, longhands::stroke_linecap::computed_value::T),
            
            /// stroke-linejoin
            StrokeLinejoin(longhands::stroke_linejoin::computed_value::T, longhands::stroke_linejoin::computed_value::T),
            
            /// stroke-miterlimit
            StrokeMiterlimit(<longhands::stroke_miterlimit::computed_value::T as ToAnimatedValue>::AnimatedValue, <longhands::stroke_miterlimit::computed_value::T as ToAnimatedValue>::AnimatedValue),
            
            /// stroke-opacity
            StrokeOpacity(longhands::stroke_opacity::computed_value::T, longhands::stroke_opacity::computed_value::T),
            
            /// stroke-dasharray
            StrokeDasharray(<longhands::stroke_dasharray::computed_value::T as ToAnimatedValue>::AnimatedValue, <longhands::stroke_dasharray::computed_value::T as ToAnimatedValue>::AnimatedValue),
            
            /// stroke-dashoffset
            StrokeDashoffset(longhands::stroke_dashoffset::computed_value::T, longhands::stroke_dashoffset::computed_value::T),
            
            /// clip-rule
            ClipRule(longhands::clip_rule::computed_value::T, longhands::clip_rule::computed_value::T),
            
            /// marker-start
            MarkerStart(longhands::marker_start::computed_value::T, longhands::marker_start::computed_value::T),
            
            /// marker-mid
            MarkerMid(longhands::marker_mid::computed_value::T, longhands::marker_mid::computed_value::T),
            
            /// marker-end
            MarkerEnd(longhands::marker_end::computed_value::T, longhands::marker_end::computed_value::T),
            
            /// paint-order
            PaintOrder(longhands::paint_order::computed_value::T, longhands::paint_order::computed_value::T),
            
            /// dominant-baseline
            DominantBaseline(longhands::dominant_baseline::computed_value::T, longhands::dominant_baseline::computed_value::T),
            
            /// vector-effect
            VectorEffect(longhands::vector_effect::computed_value::T, longhands::vector_effect::computed_value::T),
            
            /// stop-color
            StopColor(<longhands::stop_color::computed_value::T as ToAnimatedValue>::AnimatedValue, <longhands::stop_color::computed_value::T as ToAnimatedValue>::AnimatedValue),
            
            /// stop-opacity
            StopOpacity(longhands::stop_opacity::computed_value::T, longhands::stop_opacity::computed_value::T),
            
            /// flood-color
            FloodColor(<longhands::flood_color::computed_value::T as ToAnimatedValue>::AnimatedValue, <longhands::flood_color::computed_value::T as ToAnimatedValue>::AnimatedValue),
            
            /// flood-opacity
            FloodOpacity(longhands::flood_opacity::computed_value::T, longhands::flood_opacity::computed_value::T),
            
            /// lighting-color
            LightingColor(<longhands::lighting_color::computed_value::T as ToAnimatedValue>::AnimatedValue, <longhands::lighting_color::computed_value::T as ToAnimatedValue>::AnimatedValue),
            
            /// mask-type
            MaskType(longhands::mask_type::computed_value::T, longhands::mask_type::computed_value::T),
            
            /// clip-path
            ClipPath(longhands::clip_path::computed_value::T, longhands::clip_path::computed_value::T),
            
            /// mask-mode
            MaskMode(longhands::mask_mode::computed_value::T, longhands::mask_mode::computed_value::T),
            
            /// mask-repeat
            MaskRepeat(longhands::mask_repeat::computed_value::T, longhands::mask_repeat::computed_value::T),
            
            /// mask-position-x
            MaskPositionX(longhands::mask_position_x::computed_value::T, longhands::mask_position_x::computed_value::T),
            
            /// mask-position-y
            MaskPositionY(longhands::mask_position_y::computed_value::T, longhands::mask_position_y::computed_value::T),
            
            /// mask-clip
            MaskClip(longhands::mask_clip::computed_value::T, longhands::mask_clip::computed_value::T),
            
            /// mask-origin
            MaskOrigin(longhands::mask_origin::computed_value::T, longhands::mask_origin::computed_value::T),
            
            /// mask-size
            MaskSize(longhands::mask_size::computed_value::T, longhands::mask_size::computed_value::T),
            
            /// mask-composite
            MaskComposite(longhands::mask_composite::computed_value::T, longhands::mask_composite::computed_value::T),
            
            /// mask-image
            MaskImage(longhands::mask_image::computed_value::T, longhands::mask_image::computed_value::T),
            
            /// -moz-box-align
            MozBoxAlign(longhands::_moz_box_align::computed_value::T, longhands::_moz_box_align::computed_value::T),
            
            /// -moz-box-direction
            MozBoxDirection(longhands::_moz_box_direction::computed_value::T, longhands::_moz_box_direction::computed_value::T),
            
            /// -moz-box-flex
            MozBoxFlex(<longhands::_moz_box_flex::computed_value::T as ToAnimatedValue>::AnimatedValue, <longhands::_moz_box_flex::computed_value::T as ToAnimatedValue>::AnimatedValue),
            
            /// -moz-box-orient
            MozBoxOrient(longhands::_moz_box_orient::computed_value::T, longhands::_moz_box_orient::computed_value::T),
            
            /// -moz-box-pack
            MozBoxPack(longhands::_moz_box_pack::computed_value::T, longhands::_moz_box_pack::computed_value::T),
            
            /// -moz-stack-sizing
            MozStackSizing(longhands::_moz_stack_sizing::computed_value::T, longhands::_moz_stack_sizing::computed_value::T),
            
            /// -moz-box-ordinal-group
            MozBoxOrdinalGroup(longhands::_moz_box_ordinal_group::computed_value::T, longhands::_moz_box_ordinal_group::computed_value::T),
}

#[cfg(feature = "servo")]
impl AnimatedProperty {
    /// Get the name of this property.
    pub fn name(&self) -> &'static str {
        match *self {
                    AnimatedProperty::BackgroundColor(..) => "background-color",
                    AnimatedProperty::BackgroundImage(..) => "background-image",
                    AnimatedProperty::BackgroundPositionX(..) => "background-position-x",
                    AnimatedProperty::BackgroundPositionY(..) => "background-position-y",
                    AnimatedProperty::BackgroundRepeat(..) => "background-repeat",
                    AnimatedProperty::BackgroundAttachment(..) => "background-attachment",
                    AnimatedProperty::BackgroundClip(..) => "background-clip",
                    AnimatedProperty::BackgroundOrigin(..) => "background-origin",
                    AnimatedProperty::BackgroundSize(..) => "background-size",
                    AnimatedProperty::BackgroundBlendMode(..) => "background-blend-mode",
                    AnimatedProperty::BorderTopColor(..) => "border-top-color",
                    AnimatedProperty::BorderTopStyle(..) => "border-top-style",
                    AnimatedProperty::BorderTopWidth(..) => "border-top-width",
                    AnimatedProperty::BorderLeftColor(..) => "border-left-color",
                    AnimatedProperty::BorderLeftStyle(..) => "border-left-style",
                    AnimatedProperty::BorderLeftWidth(..) => "border-left-width",
                    AnimatedProperty::BorderBottomColor(..) => "border-bottom-color",
                    AnimatedProperty::BorderBottomStyle(..) => "border-bottom-style",
                    AnimatedProperty::BorderBottomWidth(..) => "border-bottom-width",
                    AnimatedProperty::BorderRightColor(..) => "border-right-color",
                    AnimatedProperty::BorderRightStyle(..) => "border-right-style",
                    AnimatedProperty::BorderRightWidth(..) => "border-right-width",
                    AnimatedProperty::BorderTopLeftRadius(..) => "border-top-left-radius",
                    AnimatedProperty::BorderTopRightRadius(..) => "border-top-right-radius",
                    AnimatedProperty::BorderBottomRightRadius(..) => "border-bottom-right-radius",
                    AnimatedProperty::BorderBottomLeftRadius(..) => "border-bottom-left-radius",
                    AnimatedProperty::MozBorderTopColors(..) => "-moz-border-top-colors",
                    AnimatedProperty::MozBorderLeftColors(..) => "-moz-border-left-colors",
                    AnimatedProperty::MozBorderBottomColors(..) => "-moz-border-bottom-colors",
                    AnimatedProperty::MozBorderRightColors(..) => "-moz-border-right-colors",
                    AnimatedProperty::BoxDecorationBreak(..) => "box-decoration-break",
                    AnimatedProperty::MozFloatEdge(..) => "-moz-float-edge",
                    AnimatedProperty::BorderImageSource(..) => "border-image-source",
                    AnimatedProperty::BorderImageOutset(..) => "border-image-outset",
                    AnimatedProperty::BorderImageRepeat(..) => "border-image-repeat",
                    AnimatedProperty::BorderImageWidth(..) => "border-image-width",
                    AnimatedProperty::BorderImageSlice(..) => "border-image-slice",
                    AnimatedProperty::Display(..) => "display",
                    AnimatedProperty::Position(..) => "position",
                    AnimatedProperty::Float(..) => "float",
                    AnimatedProperty::Clear(..) => "clear",
                    AnimatedProperty::VerticalAlign(..) => "vertical-align",
                    AnimatedProperty::OverflowClipBox(..) => "overflow-clip-box",
                    AnimatedProperty::OverflowX(..) => "overflow-x",
                    AnimatedProperty::OverflowY(..) => "overflow-y",
                    AnimatedProperty::ScrollSnapPointsX(..) => "scroll-snap-points-x",
                    AnimatedProperty::ScrollSnapPointsY(..) => "scroll-snap-points-y",
                    AnimatedProperty::ScrollSnapDestination(..) => "scroll-snap-destination",
                    AnimatedProperty::ScrollSnapCoordinate(..) => "scroll-snap-coordinate",
                    AnimatedProperty::Transform(..) => "transform",
                    AnimatedProperty::ScrollBehavior(..) => "scroll-behavior",
                    AnimatedProperty::ScrollSnapTypeX(..) => "scroll-snap-type-x",
                    AnimatedProperty::ScrollSnapTypeY(..) => "scroll-snap-type-y",
                    AnimatedProperty::Isolation(..) => "isolation",
                    AnimatedProperty::PageBreakAfter(..) => "page-break-after",
                    AnimatedProperty::PageBreakBefore(..) => "page-break-before",
                    AnimatedProperty::PageBreakInside(..) => "page-break-inside",
                    AnimatedProperty::Resize(..) => "resize",
                    AnimatedProperty::Perspective(..) => "perspective",
                    AnimatedProperty::PerspectiveOrigin(..) => "perspective-origin",
                    AnimatedProperty::BackfaceVisibility(..) => "backface-visibility",
                    AnimatedProperty::TransformBox(..) => "transform-box",
                    AnimatedProperty::TransformStyle(..) => "transform-style",
                    AnimatedProperty::TransformOrigin(..) => "transform-origin",
                    AnimatedProperty::Contain(..) => "contain",
                    AnimatedProperty::MozAppearance(..) => "-moz-appearance",
                    AnimatedProperty::MozOrient(..) => "-moz-orient",
                    AnimatedProperty::WillChange(..) => "will-change",
                    AnimatedProperty::ShapeOutside(..) => "shape-outside",
                    AnimatedProperty::TouchAction(..) => "touch-action",
                    AnimatedProperty::Color(..) => "color",
                    AnimatedProperty::ColumnWidth(..) => "column-width",
                    AnimatedProperty::ColumnCount(..) => "column-count",
                    AnimatedProperty::ColumnGap(..) => "column-gap",
                    AnimatedProperty::ColumnFill(..) => "column-fill",
                    AnimatedProperty::ColumnRuleWidth(..) => "column-rule-width",
                    AnimatedProperty::ColumnRuleColor(..) => "column-rule-color",
                    AnimatedProperty::ColumnSpan(..) => "column-span",
                    AnimatedProperty::ColumnRuleStyle(..) => "column-rule-style",
                    AnimatedProperty::Content(..) => "content",
                    AnimatedProperty::CounterIncrement(..) => "counter-increment",
                    AnimatedProperty::CounterReset(..) => "counter-reset",
                    AnimatedProperty::Opacity(..) => "opacity",
                    AnimatedProperty::BoxShadow(..) => "box-shadow",
                    AnimatedProperty::Clip(..) => "clip",
                    AnimatedProperty::Filter(..) => "filter",
                    AnimatedProperty::MixBlendMode(..) => "mix-blend-mode",
                    AnimatedProperty::FontFamily(..) => "font-family",
                    AnimatedProperty::FontStyle(..) => "font-style",
                    AnimatedProperty::FontVariantCaps(..) => "font-variant-caps",
                    AnimatedProperty::FontWeight(..) => "font-weight",
                    AnimatedProperty::FontSize(..) => "font-size",
                    AnimatedProperty::FontSizeAdjust(..) => "font-size-adjust",
                    AnimatedProperty::FontSynthesis(..) => "font-synthesis",
                    AnimatedProperty::FontStretch(..) => "font-stretch",
                    AnimatedProperty::FontKerning(..) => "font-kerning",
                    AnimatedProperty::FontVariantAlternates(..) => "font-variant-alternates",
                    AnimatedProperty::FontVariantEastAsian(..) => "font-variant-east-asian",
                    AnimatedProperty::FontVariantLigatures(..) => "font-variant-ligatures",
                    AnimatedProperty::FontVariantNumeric(..) => "font-variant-numeric",
                    AnimatedProperty::FontVariantPosition(..) => "font-variant-position",
                    AnimatedProperty::FontFeatureSettings(..) => "font-feature-settings",
                    AnimatedProperty::FontVariationSettings(..) => "font-variation-settings",
                    AnimatedProperty::FontLanguageOverride(..) => "font-language-override",
                    AnimatedProperty::MozOsxFontSmoothing(..) => "-moz-osx-font-smoothing",
                    AnimatedProperty::Visibility(..) => "visibility",
                    AnimatedProperty::WritingMode(..) => "writing-mode",
                    AnimatedProperty::Direction(..) => "direction",
                    AnimatedProperty::TextOrientation(..) => "text-orientation",
                    AnimatedProperty::ColorAdjust(..) => "color-adjust",
                    AnimatedProperty::ImageRendering(..) => "image-rendering",
                    AnimatedProperty::ImageOrientation(..) => "image-orientation",
                    AnimatedProperty::BorderCollapse(..) => "border-collapse",
                    AnimatedProperty::EmptyCells(..) => "empty-cells",
                    AnimatedProperty::CaptionSide(..) => "caption-side",
                    AnimatedProperty::BorderSpacing(..) => "border-spacing",
                    AnimatedProperty::LineHeight(..) => "line-height",
                    AnimatedProperty::TextTransform(..) => "text-transform",
                    AnimatedProperty::Hyphens(..) => "hyphens",
                    AnimatedProperty::MozTextSizeAdjust(..) => "-moz-text-size-adjust",
                    AnimatedProperty::TextIndent(..) => "text-indent",
                    AnimatedProperty::OverflowWrap(..) => "overflow-wrap",
                    AnimatedProperty::WordBreak(..) => "word-break",
                    AnimatedProperty::TextJustify(..) => "text-justify",
                    AnimatedProperty::TextAlignLast(..) => "text-align-last",
                    AnimatedProperty::TextAlign(..) => "text-align",
                    AnimatedProperty::LetterSpacing(..) => "letter-spacing",
                    AnimatedProperty::WordSpacing(..) => "word-spacing",
                    AnimatedProperty::WhiteSpace(..) => "white-space",
                    AnimatedProperty::TextShadow(..) => "text-shadow",
                    AnimatedProperty::TextEmphasisStyle(..) => "text-emphasis-style",
                    AnimatedProperty::TextEmphasisPosition(..) => "text-emphasis-position",
                    AnimatedProperty::TextEmphasisColor(..) => "text-emphasis-color",
                    AnimatedProperty::MozTabSize(..) => "-moz-tab-size",
                    AnimatedProperty::WebkitTextFillColor(..) => "-webkit-text-fill-color",
                    AnimatedProperty::WebkitTextStrokeColor(..) => "-webkit-text-stroke-color",
                    AnimatedProperty::WebkitTextStrokeWidth(..) => "-webkit-text-stroke-width",
                    AnimatedProperty::RubyAlign(..) => "ruby-align",
                    AnimatedProperty::RubyPosition(..) => "ruby-position",
                    AnimatedProperty::TextCombineUpright(..) => "text-combine-upright",
                    AnimatedProperty::TextRendering(..) => "text-rendering",
                    AnimatedProperty::ListStylePosition(..) => "list-style-position",
                    AnimatedProperty::ListStyleType(..) => "list-style-type",
                    AnimatedProperty::ListStyleImage(..) => "list-style-image",
                    AnimatedProperty::Quotes(..) => "quotes",
                    AnimatedProperty::MozImageRegion(..) => "-moz-image-region",
                    AnimatedProperty::MarginTop(..) => "margin-top",
                    AnimatedProperty::MarginLeft(..) => "margin-left",
                    AnimatedProperty::MarginBottom(..) => "margin-bottom",
                    AnimatedProperty::MarginRight(..) => "margin-right",
                    AnimatedProperty::OutlineColor(..) => "outline-color",
                    AnimatedProperty::OutlineStyle(..) => "outline-style",
                    AnimatedProperty::OutlineWidth(..) => "outline-width",
                    AnimatedProperty::MozOutlineRadiusTopleft(..) => "-moz-outline-radius-topleft",
                    AnimatedProperty::MozOutlineRadiusTopright(..) => "-moz-outline-radius-topright",
                    AnimatedProperty::MozOutlineRadiusBottomright(..) => "-moz-outline-radius-bottomright",
                    AnimatedProperty::MozOutlineRadiusBottomleft(..) => "-moz-outline-radius-bottomleft",
                    AnimatedProperty::OutlineOffset(..) => "outline-offset",
                    AnimatedProperty::PaddingTop(..) => "padding-top",
                    AnimatedProperty::PaddingLeft(..) => "padding-left",
                    AnimatedProperty::PaddingBottom(..) => "padding-bottom",
                    AnimatedProperty::PaddingRight(..) => "padding-right",
                    AnimatedProperty::Cursor(..) => "cursor",
                    AnimatedProperty::PointerEvents(..) => "pointer-events",
                    AnimatedProperty::MozUserInput(..) => "-moz-user-input",
                    AnimatedProperty::MozUserModify(..) => "-moz-user-modify",
                    AnimatedProperty::MozUserFocus(..) => "-moz-user-focus",
                    AnimatedProperty::CaretColor(..) => "caret-color",
                    AnimatedProperty::Top(..) => "top",
                    AnimatedProperty::Left(..) => "left",
                    AnimatedProperty::Bottom(..) => "bottom",
                    AnimatedProperty::Right(..) => "right",
                    AnimatedProperty::ZIndex(..) => "z-index",
                    AnimatedProperty::FlexDirection(..) => "flex-direction",
                    AnimatedProperty::FlexWrap(..) => "flex-wrap",
                    AnimatedProperty::JustifyContent(..) => "justify-content",
                    AnimatedProperty::AlignContent(..) => "align-content",
                    AnimatedProperty::AlignItems(..) => "align-items",
                    AnimatedProperty::JustifyItems(..) => "justify-items",
                    AnimatedProperty::FlexGrow(..) => "flex-grow",
                    AnimatedProperty::FlexShrink(..) => "flex-shrink",
                    AnimatedProperty::AlignSelf(..) => "align-self",
                    AnimatedProperty::JustifySelf(..) => "justify-self",
                    AnimatedProperty::Order(..) => "order",
                    AnimatedProperty::FlexBasis(..) => "flex-basis",
                    AnimatedProperty::Width(..) => "width",
                    AnimatedProperty::MinWidth(..) => "min-width",
                    AnimatedProperty::MaxWidth(..) => "max-width",
                    AnimatedProperty::Height(..) => "height",
                    AnimatedProperty::MinHeight(..) => "min-height",
                    AnimatedProperty::MaxHeight(..) => "max-height",
                    AnimatedProperty::BoxSizing(..) => "box-sizing",
                    AnimatedProperty::ObjectFit(..) => "object-fit",
                    AnimatedProperty::ObjectPosition(..) => "object-position",
                    AnimatedProperty::GridRowGap(..) => "grid-row-gap",
                    AnimatedProperty::GridRowStart(..) => "grid-row-start",
                    AnimatedProperty::GridRowEnd(..) => "grid-row-end",
                    AnimatedProperty::GridAutoRows(..) => "grid-auto-rows",
                    AnimatedProperty::GridTemplateRows(..) => "grid-template-rows",
                    AnimatedProperty::GridColumnGap(..) => "grid-column-gap",
                    AnimatedProperty::GridColumnStart(..) => "grid-column-start",
                    AnimatedProperty::GridColumnEnd(..) => "grid-column-end",
                    AnimatedProperty::GridAutoColumns(..) => "grid-auto-columns",
                    AnimatedProperty::GridTemplateColumns(..) => "grid-template-columns",
                    AnimatedProperty::GridAutoFlow(..) => "grid-auto-flow",
                    AnimatedProperty::GridTemplateAreas(..) => "grid-template-areas",
                    AnimatedProperty::TableLayout(..) => "table-layout",
                    AnimatedProperty::TextOverflow(..) => "text-overflow",
                    AnimatedProperty::UnicodeBidi(..) => "unicode-bidi",
                    AnimatedProperty::TextDecorationLine(..) => "text-decoration-line",
                    AnimatedProperty::TextDecorationStyle(..) => "text-decoration-style",
                    AnimatedProperty::TextDecorationColor(..) => "text-decoration-color",
                    AnimatedProperty::InitialLetter(..) => "initial-letter",
                    AnimatedProperty::ImeMode(..) => "ime-mode",
                    AnimatedProperty::MozUserSelect(..) => "-moz-user-select",
                    AnimatedProperty::MozWindowDragging(..) => "-moz-window-dragging",
                    AnimatedProperty::MozWindowShadow(..) => "-moz-window-shadow",
                    AnimatedProperty::MozForceBrokenImageIcon(..) => "-moz-force-broken-image-icon",
                    AnimatedProperty::TextAnchor(..) => "text-anchor",
                    AnimatedProperty::ColorInterpolation(..) => "color-interpolation",
                    AnimatedProperty::ColorInterpolationFilters(..) => "color-interpolation-filters",
                    AnimatedProperty::Fill(..) => "fill",
                    AnimatedProperty::FillOpacity(..) => "fill-opacity",
                    AnimatedProperty::FillRule(..) => "fill-rule",
                    AnimatedProperty::ShapeRendering(..) => "shape-rendering",
                    AnimatedProperty::Stroke(..) => "stroke",
                    AnimatedProperty::StrokeWidth(..) => "stroke-width",
                    AnimatedProperty::StrokeLinecap(..) => "stroke-linecap",
                    AnimatedProperty::StrokeLinejoin(..) => "stroke-linejoin",
                    AnimatedProperty::StrokeMiterlimit(..) => "stroke-miterlimit",
                    AnimatedProperty::StrokeOpacity(..) => "stroke-opacity",
                    AnimatedProperty::StrokeDasharray(..) => "stroke-dasharray",
                    AnimatedProperty::StrokeDashoffset(..) => "stroke-dashoffset",
                    AnimatedProperty::ClipRule(..) => "clip-rule",
                    AnimatedProperty::MarkerStart(..) => "marker-start",
                    AnimatedProperty::MarkerMid(..) => "marker-mid",
                    AnimatedProperty::MarkerEnd(..) => "marker-end",
                    AnimatedProperty::PaintOrder(..) => "paint-order",
                    AnimatedProperty::DominantBaseline(..) => "dominant-baseline",
                    AnimatedProperty::VectorEffect(..) => "vector-effect",
                    AnimatedProperty::StopColor(..) => "stop-color",
                    AnimatedProperty::StopOpacity(..) => "stop-opacity",
                    AnimatedProperty::FloodColor(..) => "flood-color",
                    AnimatedProperty::FloodOpacity(..) => "flood-opacity",
                    AnimatedProperty::LightingColor(..) => "lighting-color",
                    AnimatedProperty::MaskType(..) => "mask-type",
                    AnimatedProperty::ClipPath(..) => "clip-path",
                    AnimatedProperty::MaskMode(..) => "mask-mode",
                    AnimatedProperty::MaskRepeat(..) => "mask-repeat",
                    AnimatedProperty::MaskPositionX(..) => "mask-position-x",
                    AnimatedProperty::MaskPositionY(..) => "mask-position-y",
                    AnimatedProperty::MaskClip(..) => "mask-clip",
                    AnimatedProperty::MaskOrigin(..) => "mask-origin",
                    AnimatedProperty::MaskSize(..) => "mask-size",
                    AnimatedProperty::MaskComposite(..) => "mask-composite",
                    AnimatedProperty::MaskImage(..) => "mask-image",
                    AnimatedProperty::MozBoxAlign(..) => "-moz-box-align",
                    AnimatedProperty::MozBoxDirection(..) => "-moz-box-direction",
                    AnimatedProperty::MozBoxFlex(..) => "-moz-box-flex",
                    AnimatedProperty::MozBoxOrient(..) => "-moz-box-orient",
                    AnimatedProperty::MozBoxPack(..) => "-moz-box-pack",
                    AnimatedProperty::MozStackSizing(..) => "-moz-stack-sizing",
                    AnimatedProperty::MozBoxOrdinalGroup(..) => "-moz-box-ordinal-group",
        }
    }

    /// Whether this interpolation does animate, that is, whether the start and
    /// end values are different.
    pub fn does_animate(&self) -> bool {
        match *self {
                    AnimatedProperty::BackgroundColor(ref from, ref to) => from != to,
                    AnimatedProperty::BackgroundImage(ref from, ref to) => from != to,
                    AnimatedProperty::BackgroundPositionX(ref from, ref to) => from != to,
                    AnimatedProperty::BackgroundPositionY(ref from, ref to) => from != to,
                    AnimatedProperty::BackgroundRepeat(ref from, ref to) => from != to,
                    AnimatedProperty::BackgroundAttachment(ref from, ref to) => from != to,
                    AnimatedProperty::BackgroundClip(ref from, ref to) => from != to,
                    AnimatedProperty::BackgroundOrigin(ref from, ref to) => from != to,
                    AnimatedProperty::BackgroundSize(ref from, ref to) => from != to,
                    AnimatedProperty::BackgroundBlendMode(ref from, ref to) => from != to,
                    AnimatedProperty::BorderTopColor(ref from, ref to) => from != to,
                    AnimatedProperty::BorderTopStyle(ref from, ref to) => from != to,
                    AnimatedProperty::BorderTopWidth(ref from, ref to) => from != to,
                    AnimatedProperty::BorderLeftColor(ref from, ref to) => from != to,
                    AnimatedProperty::BorderLeftStyle(ref from, ref to) => from != to,
                    AnimatedProperty::BorderLeftWidth(ref from, ref to) => from != to,
                    AnimatedProperty::BorderBottomColor(ref from, ref to) => from != to,
                    AnimatedProperty::BorderBottomStyle(ref from, ref to) => from != to,
                    AnimatedProperty::BorderBottomWidth(ref from, ref to) => from != to,
                    AnimatedProperty::BorderRightColor(ref from, ref to) => from != to,
                    AnimatedProperty::BorderRightStyle(ref from, ref to) => from != to,
                    AnimatedProperty::BorderRightWidth(ref from, ref to) => from != to,
                    AnimatedProperty::BorderTopLeftRadius(ref from, ref to) => from != to,
                    AnimatedProperty::BorderTopRightRadius(ref from, ref to) => from != to,
                    AnimatedProperty::BorderBottomRightRadius(ref from, ref to) => from != to,
                    AnimatedProperty::BorderBottomLeftRadius(ref from, ref to) => from != to,
                    AnimatedProperty::MozBorderTopColors(ref from, ref to) => from != to,
                    AnimatedProperty::MozBorderLeftColors(ref from, ref to) => from != to,
                    AnimatedProperty::MozBorderBottomColors(ref from, ref to) => from != to,
                    AnimatedProperty::MozBorderRightColors(ref from, ref to) => from != to,
                    AnimatedProperty::BoxDecorationBreak(ref from, ref to) => from != to,
                    AnimatedProperty::MozFloatEdge(ref from, ref to) => from != to,
                    AnimatedProperty::BorderImageSource(ref from, ref to) => from != to,
                    AnimatedProperty::BorderImageOutset(ref from, ref to) => from != to,
                    AnimatedProperty::BorderImageRepeat(ref from, ref to) => from != to,
                    AnimatedProperty::BorderImageWidth(ref from, ref to) => from != to,
                    AnimatedProperty::BorderImageSlice(ref from, ref to) => from != to,
                    AnimatedProperty::Display(ref from, ref to) => from != to,
                    AnimatedProperty::Position(ref from, ref to) => from != to,
                    AnimatedProperty::Float(ref from, ref to) => from != to,
                    AnimatedProperty::Clear(ref from, ref to) => from != to,
                    AnimatedProperty::VerticalAlign(ref from, ref to) => from != to,
                    AnimatedProperty::OverflowClipBox(ref from, ref to) => from != to,
                    AnimatedProperty::OverflowX(ref from, ref to) => from != to,
                    AnimatedProperty::OverflowY(ref from, ref to) => from != to,
                    AnimatedProperty::ScrollSnapPointsX(ref from, ref to) => from != to,
                    AnimatedProperty::ScrollSnapPointsY(ref from, ref to) => from != to,
                    AnimatedProperty::ScrollSnapDestination(ref from, ref to) => from != to,
                    AnimatedProperty::ScrollSnapCoordinate(ref from, ref to) => from != to,
                    AnimatedProperty::Transform(ref from, ref to) => from != to,
                    AnimatedProperty::ScrollBehavior(ref from, ref to) => from != to,
                    AnimatedProperty::ScrollSnapTypeX(ref from, ref to) => from != to,
                    AnimatedProperty::ScrollSnapTypeY(ref from, ref to) => from != to,
                    AnimatedProperty::Isolation(ref from, ref to) => from != to,
                    AnimatedProperty::PageBreakAfter(ref from, ref to) => from != to,
                    AnimatedProperty::PageBreakBefore(ref from, ref to) => from != to,
                    AnimatedProperty::PageBreakInside(ref from, ref to) => from != to,
                    AnimatedProperty::Resize(ref from, ref to) => from != to,
                    AnimatedProperty::Perspective(ref from, ref to) => from != to,
                    AnimatedProperty::PerspectiveOrigin(ref from, ref to) => from != to,
                    AnimatedProperty::BackfaceVisibility(ref from, ref to) => from != to,
                    AnimatedProperty::TransformBox(ref from, ref to) => from != to,
                    AnimatedProperty::TransformStyle(ref from, ref to) => from != to,
                    AnimatedProperty::TransformOrigin(ref from, ref to) => from != to,
                    AnimatedProperty::Contain(ref from, ref to) => from != to,
                    AnimatedProperty::MozAppearance(ref from, ref to) => from != to,
                    AnimatedProperty::MozOrient(ref from, ref to) => from != to,
                    AnimatedProperty::WillChange(ref from, ref to) => from != to,
                    AnimatedProperty::ShapeOutside(ref from, ref to) => from != to,
                    AnimatedProperty::TouchAction(ref from, ref to) => from != to,
                    AnimatedProperty::Color(ref from, ref to) => from != to,
                    AnimatedProperty::ColumnWidth(ref from, ref to) => from != to,
                    AnimatedProperty::ColumnCount(ref from, ref to) => from != to,
                    AnimatedProperty::ColumnGap(ref from, ref to) => from != to,
                    AnimatedProperty::ColumnFill(ref from, ref to) => from != to,
                    AnimatedProperty::ColumnRuleWidth(ref from, ref to) => from != to,
                    AnimatedProperty::ColumnRuleColor(ref from, ref to) => from != to,
                    AnimatedProperty::ColumnSpan(ref from, ref to) => from != to,
                    AnimatedProperty::ColumnRuleStyle(ref from, ref to) => from != to,
                    AnimatedProperty::Content(ref from, ref to) => from != to,
                    AnimatedProperty::CounterIncrement(ref from, ref to) => from != to,
                    AnimatedProperty::CounterReset(ref from, ref to) => from != to,
                    AnimatedProperty::Opacity(ref from, ref to) => from != to,
                    AnimatedProperty::BoxShadow(ref from, ref to) => from != to,
                    AnimatedProperty::Clip(ref from, ref to) => from != to,
                    AnimatedProperty::Filter(ref from, ref to) => from != to,
                    AnimatedProperty::MixBlendMode(ref from, ref to) => from != to,
                    AnimatedProperty::FontFamily(ref from, ref to) => from != to,
                    AnimatedProperty::FontStyle(ref from, ref to) => from != to,
                    AnimatedProperty::FontVariantCaps(ref from, ref to) => from != to,
                    AnimatedProperty::FontWeight(ref from, ref to) => from != to,
                    AnimatedProperty::FontSize(ref from, ref to) => from != to,
                    AnimatedProperty::FontSizeAdjust(ref from, ref to) => from != to,
                    AnimatedProperty::FontSynthesis(ref from, ref to) => from != to,
                    AnimatedProperty::FontStretch(ref from, ref to) => from != to,
                    AnimatedProperty::FontKerning(ref from, ref to) => from != to,
                    AnimatedProperty::FontVariantAlternates(ref from, ref to) => from != to,
                    AnimatedProperty::FontVariantEastAsian(ref from, ref to) => from != to,
                    AnimatedProperty::FontVariantLigatures(ref from, ref to) => from != to,
                    AnimatedProperty::FontVariantNumeric(ref from, ref to) => from != to,
                    AnimatedProperty::FontVariantPosition(ref from, ref to) => from != to,
                    AnimatedProperty::FontFeatureSettings(ref from, ref to) => from != to,
                    AnimatedProperty::FontVariationSettings(ref from, ref to) => from != to,
                    AnimatedProperty::FontLanguageOverride(ref from, ref to) => from != to,
                    AnimatedProperty::MozOsxFontSmoothing(ref from, ref to) => from != to,
                    AnimatedProperty::Visibility(ref from, ref to) => from != to,
                    AnimatedProperty::WritingMode(ref from, ref to) => from != to,
                    AnimatedProperty::Direction(ref from, ref to) => from != to,
                    AnimatedProperty::TextOrientation(ref from, ref to) => from != to,
                    AnimatedProperty::ColorAdjust(ref from, ref to) => from != to,
                    AnimatedProperty::ImageRendering(ref from, ref to) => from != to,
                    AnimatedProperty::ImageOrientation(ref from, ref to) => from != to,
                    AnimatedProperty::BorderCollapse(ref from, ref to) => from != to,
                    AnimatedProperty::EmptyCells(ref from, ref to) => from != to,
                    AnimatedProperty::CaptionSide(ref from, ref to) => from != to,
                    AnimatedProperty::BorderSpacing(ref from, ref to) => from != to,
                    AnimatedProperty::LineHeight(ref from, ref to) => from != to,
                    AnimatedProperty::TextTransform(ref from, ref to) => from != to,
                    AnimatedProperty::Hyphens(ref from, ref to) => from != to,
                    AnimatedProperty::MozTextSizeAdjust(ref from, ref to) => from != to,
                    AnimatedProperty::TextIndent(ref from, ref to) => from != to,
                    AnimatedProperty::OverflowWrap(ref from, ref to) => from != to,
                    AnimatedProperty::WordBreak(ref from, ref to) => from != to,
                    AnimatedProperty::TextJustify(ref from, ref to) => from != to,
                    AnimatedProperty::TextAlignLast(ref from, ref to) => from != to,
                    AnimatedProperty::TextAlign(ref from, ref to) => from != to,
                    AnimatedProperty::LetterSpacing(ref from, ref to) => from != to,
                    AnimatedProperty::WordSpacing(ref from, ref to) => from != to,
                    AnimatedProperty::WhiteSpace(ref from, ref to) => from != to,
                    AnimatedProperty::TextShadow(ref from, ref to) => from != to,
                    AnimatedProperty::TextEmphasisStyle(ref from, ref to) => from != to,
                    AnimatedProperty::TextEmphasisPosition(ref from, ref to) => from != to,
                    AnimatedProperty::TextEmphasisColor(ref from, ref to) => from != to,
                    AnimatedProperty::MozTabSize(ref from, ref to) => from != to,
                    AnimatedProperty::WebkitTextFillColor(ref from, ref to) => from != to,
                    AnimatedProperty::WebkitTextStrokeColor(ref from, ref to) => from != to,
                    AnimatedProperty::WebkitTextStrokeWidth(ref from, ref to) => from != to,
                    AnimatedProperty::RubyAlign(ref from, ref to) => from != to,
                    AnimatedProperty::RubyPosition(ref from, ref to) => from != to,
                    AnimatedProperty::TextCombineUpright(ref from, ref to) => from != to,
                    AnimatedProperty::TextRendering(ref from, ref to) => from != to,
                    AnimatedProperty::ListStylePosition(ref from, ref to) => from != to,
                    AnimatedProperty::ListStyleType(ref from, ref to) => from != to,
                    AnimatedProperty::ListStyleImage(ref from, ref to) => from != to,
                    AnimatedProperty::Quotes(ref from, ref to) => from != to,
                    AnimatedProperty::MozImageRegion(ref from, ref to) => from != to,
                    AnimatedProperty::MarginTop(ref from, ref to) => from != to,
                    AnimatedProperty::MarginLeft(ref from, ref to) => from != to,
                    AnimatedProperty::MarginBottom(ref from, ref to) => from != to,
                    AnimatedProperty::MarginRight(ref from, ref to) => from != to,
                    AnimatedProperty::OutlineColor(ref from, ref to) => from != to,
                    AnimatedProperty::OutlineStyle(ref from, ref to) => from != to,
                    AnimatedProperty::OutlineWidth(ref from, ref to) => from != to,
                    AnimatedProperty::MozOutlineRadiusTopleft(ref from, ref to) => from != to,
                    AnimatedProperty::MozOutlineRadiusTopright(ref from, ref to) => from != to,
                    AnimatedProperty::MozOutlineRadiusBottomright(ref from, ref to) => from != to,
                    AnimatedProperty::MozOutlineRadiusBottomleft(ref from, ref to) => from != to,
                    AnimatedProperty::OutlineOffset(ref from, ref to) => from != to,
                    AnimatedProperty::PaddingTop(ref from, ref to) => from != to,
                    AnimatedProperty::PaddingLeft(ref from, ref to) => from != to,
                    AnimatedProperty::PaddingBottom(ref from, ref to) => from != to,
                    AnimatedProperty::PaddingRight(ref from, ref to) => from != to,
                    AnimatedProperty::Cursor(ref from, ref to) => from != to,
                    AnimatedProperty::PointerEvents(ref from, ref to) => from != to,
                    AnimatedProperty::MozUserInput(ref from, ref to) => from != to,
                    AnimatedProperty::MozUserModify(ref from, ref to) => from != to,
                    AnimatedProperty::MozUserFocus(ref from, ref to) => from != to,
                    AnimatedProperty::CaretColor(ref from, ref to) => from != to,
                    AnimatedProperty::Top(ref from, ref to) => from != to,
                    AnimatedProperty::Left(ref from, ref to) => from != to,
                    AnimatedProperty::Bottom(ref from, ref to) => from != to,
                    AnimatedProperty::Right(ref from, ref to) => from != to,
                    AnimatedProperty::ZIndex(ref from, ref to) => from != to,
                    AnimatedProperty::FlexDirection(ref from, ref to) => from != to,
                    AnimatedProperty::FlexWrap(ref from, ref to) => from != to,
                    AnimatedProperty::JustifyContent(ref from, ref to) => from != to,
                    AnimatedProperty::AlignContent(ref from, ref to) => from != to,
                    AnimatedProperty::AlignItems(ref from, ref to) => from != to,
                    AnimatedProperty::JustifyItems(ref from, ref to) => from != to,
                    AnimatedProperty::FlexGrow(ref from, ref to) => from != to,
                    AnimatedProperty::FlexShrink(ref from, ref to) => from != to,
                    AnimatedProperty::AlignSelf(ref from, ref to) => from != to,
                    AnimatedProperty::JustifySelf(ref from, ref to) => from != to,
                    AnimatedProperty::Order(ref from, ref to) => from != to,
                    AnimatedProperty::FlexBasis(ref from, ref to) => from != to,
                    AnimatedProperty::Width(ref from, ref to) => from != to,
                    AnimatedProperty::MinWidth(ref from, ref to) => from != to,
                    AnimatedProperty::MaxWidth(ref from, ref to) => from != to,
                    AnimatedProperty::Height(ref from, ref to) => from != to,
                    AnimatedProperty::MinHeight(ref from, ref to) => from != to,
                    AnimatedProperty::MaxHeight(ref from, ref to) => from != to,
                    AnimatedProperty::BoxSizing(ref from, ref to) => from != to,
                    AnimatedProperty::ObjectFit(ref from, ref to) => from != to,
                    AnimatedProperty::ObjectPosition(ref from, ref to) => from != to,
                    AnimatedProperty::GridRowGap(ref from, ref to) => from != to,
                    AnimatedProperty::GridRowStart(ref from, ref to) => from != to,
                    AnimatedProperty::GridRowEnd(ref from, ref to) => from != to,
                    AnimatedProperty::GridAutoRows(ref from, ref to) => from != to,
                    AnimatedProperty::GridTemplateRows(ref from, ref to) => from != to,
                    AnimatedProperty::GridColumnGap(ref from, ref to) => from != to,
                    AnimatedProperty::GridColumnStart(ref from, ref to) => from != to,
                    AnimatedProperty::GridColumnEnd(ref from, ref to) => from != to,
                    AnimatedProperty::GridAutoColumns(ref from, ref to) => from != to,
                    AnimatedProperty::GridTemplateColumns(ref from, ref to) => from != to,
                    AnimatedProperty::GridAutoFlow(ref from, ref to) => from != to,
                    AnimatedProperty::GridTemplateAreas(ref from, ref to) => from != to,
                    AnimatedProperty::TableLayout(ref from, ref to) => from != to,
                    AnimatedProperty::TextOverflow(ref from, ref to) => from != to,
                    AnimatedProperty::UnicodeBidi(ref from, ref to) => from != to,
                    AnimatedProperty::TextDecorationLine(ref from, ref to) => from != to,
                    AnimatedProperty::TextDecorationStyle(ref from, ref to) => from != to,
                    AnimatedProperty::TextDecorationColor(ref from, ref to) => from != to,
                    AnimatedProperty::InitialLetter(ref from, ref to) => from != to,
                    AnimatedProperty::ImeMode(ref from, ref to) => from != to,
                    AnimatedProperty::MozUserSelect(ref from, ref to) => from != to,
                    AnimatedProperty::MozWindowDragging(ref from, ref to) => from != to,
                    AnimatedProperty::MozWindowShadow(ref from, ref to) => from != to,
                    AnimatedProperty::MozForceBrokenImageIcon(ref from, ref to) => from != to,
                    AnimatedProperty::TextAnchor(ref from, ref to) => from != to,
                    AnimatedProperty::ColorInterpolation(ref from, ref to) => from != to,
                    AnimatedProperty::ColorInterpolationFilters(ref from, ref to) => from != to,
                    AnimatedProperty::Fill(ref from, ref to) => from != to,
                    AnimatedProperty::FillOpacity(ref from, ref to) => from != to,
                    AnimatedProperty::FillRule(ref from, ref to) => from != to,
                    AnimatedProperty::ShapeRendering(ref from, ref to) => from != to,
                    AnimatedProperty::Stroke(ref from, ref to) => from != to,
                    AnimatedProperty::StrokeWidth(ref from, ref to) => from != to,
                    AnimatedProperty::StrokeLinecap(ref from, ref to) => from != to,
                    AnimatedProperty::StrokeLinejoin(ref from, ref to) => from != to,
                    AnimatedProperty::StrokeMiterlimit(ref from, ref to) => from != to,
                    AnimatedProperty::StrokeOpacity(ref from, ref to) => from != to,
                    AnimatedProperty::StrokeDasharray(ref from, ref to) => from != to,
                    AnimatedProperty::StrokeDashoffset(ref from, ref to) => from != to,
                    AnimatedProperty::ClipRule(ref from, ref to) => from != to,
                    AnimatedProperty::MarkerStart(ref from, ref to) => from != to,
                    AnimatedProperty::MarkerMid(ref from, ref to) => from != to,
                    AnimatedProperty::MarkerEnd(ref from, ref to) => from != to,
                    AnimatedProperty::PaintOrder(ref from, ref to) => from != to,
                    AnimatedProperty::DominantBaseline(ref from, ref to) => from != to,
                    AnimatedProperty::VectorEffect(ref from, ref to) => from != to,
                    AnimatedProperty::StopColor(ref from, ref to) => from != to,
                    AnimatedProperty::StopOpacity(ref from, ref to) => from != to,
                    AnimatedProperty::FloodColor(ref from, ref to) => from != to,
                    AnimatedProperty::FloodOpacity(ref from, ref to) => from != to,
                    AnimatedProperty::LightingColor(ref from, ref to) => from != to,
                    AnimatedProperty::MaskType(ref from, ref to) => from != to,
                    AnimatedProperty::ClipPath(ref from, ref to) => from != to,
                    AnimatedProperty::MaskMode(ref from, ref to) => from != to,
                    AnimatedProperty::MaskRepeat(ref from, ref to) => from != to,
                    AnimatedProperty::MaskPositionX(ref from, ref to) => from != to,
                    AnimatedProperty::MaskPositionY(ref from, ref to) => from != to,
                    AnimatedProperty::MaskClip(ref from, ref to) => from != to,
                    AnimatedProperty::MaskOrigin(ref from, ref to) => from != to,
                    AnimatedProperty::MaskSize(ref from, ref to) => from != to,
                    AnimatedProperty::MaskComposite(ref from, ref to) => from != to,
                    AnimatedProperty::MaskImage(ref from, ref to) => from != to,
                    AnimatedProperty::MozBoxAlign(ref from, ref to) => from != to,
                    AnimatedProperty::MozBoxDirection(ref from, ref to) => from != to,
                    AnimatedProperty::MozBoxFlex(ref from, ref to) => from != to,
                    AnimatedProperty::MozBoxOrient(ref from, ref to) => from != to,
                    AnimatedProperty::MozBoxPack(ref from, ref to) => from != to,
                    AnimatedProperty::MozStackSizing(ref from, ref to) => from != to,
                    AnimatedProperty::MozBoxOrdinalGroup(ref from, ref to) => from != to,
        }
    }

    /// Whether an animated property has the same end value as another.
    pub fn has_the_same_end_value_as(&self, other: &Self) -> bool {
        match (self, other) {
                    (&AnimatedProperty::BackgroundColor(_, ref this_end_value),
                     &AnimatedProperty::BackgroundColor(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::BackgroundImage(_, ref this_end_value),
                     &AnimatedProperty::BackgroundImage(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::BackgroundPositionX(_, ref this_end_value),
                     &AnimatedProperty::BackgroundPositionX(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::BackgroundPositionY(_, ref this_end_value),
                     &AnimatedProperty::BackgroundPositionY(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::BackgroundRepeat(_, ref this_end_value),
                     &AnimatedProperty::BackgroundRepeat(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::BackgroundAttachment(_, ref this_end_value),
                     &AnimatedProperty::BackgroundAttachment(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::BackgroundClip(_, ref this_end_value),
                     &AnimatedProperty::BackgroundClip(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::BackgroundOrigin(_, ref this_end_value),
                     &AnimatedProperty::BackgroundOrigin(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::BackgroundSize(_, ref this_end_value),
                     &AnimatedProperty::BackgroundSize(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::BackgroundBlendMode(_, ref this_end_value),
                     &AnimatedProperty::BackgroundBlendMode(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::BorderTopColor(_, ref this_end_value),
                     &AnimatedProperty::BorderTopColor(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::BorderTopStyle(_, ref this_end_value),
                     &AnimatedProperty::BorderTopStyle(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::BorderTopWidth(_, ref this_end_value),
                     &AnimatedProperty::BorderTopWidth(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::BorderLeftColor(_, ref this_end_value),
                     &AnimatedProperty::BorderLeftColor(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::BorderLeftStyle(_, ref this_end_value),
                     &AnimatedProperty::BorderLeftStyle(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::BorderLeftWidth(_, ref this_end_value),
                     &AnimatedProperty::BorderLeftWidth(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::BorderBottomColor(_, ref this_end_value),
                     &AnimatedProperty::BorderBottomColor(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::BorderBottomStyle(_, ref this_end_value),
                     &AnimatedProperty::BorderBottomStyle(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::BorderBottomWidth(_, ref this_end_value),
                     &AnimatedProperty::BorderBottomWidth(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::BorderRightColor(_, ref this_end_value),
                     &AnimatedProperty::BorderRightColor(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::BorderRightStyle(_, ref this_end_value),
                     &AnimatedProperty::BorderRightStyle(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::BorderRightWidth(_, ref this_end_value),
                     &AnimatedProperty::BorderRightWidth(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::BorderTopLeftRadius(_, ref this_end_value),
                     &AnimatedProperty::BorderTopLeftRadius(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::BorderTopRightRadius(_, ref this_end_value),
                     &AnimatedProperty::BorderTopRightRadius(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::BorderBottomRightRadius(_, ref this_end_value),
                     &AnimatedProperty::BorderBottomRightRadius(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::BorderBottomLeftRadius(_, ref this_end_value),
                     &AnimatedProperty::BorderBottomLeftRadius(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::MozBorderTopColors(_, ref this_end_value),
                     &AnimatedProperty::MozBorderTopColors(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::MozBorderLeftColors(_, ref this_end_value),
                     &AnimatedProperty::MozBorderLeftColors(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::MozBorderBottomColors(_, ref this_end_value),
                     &AnimatedProperty::MozBorderBottomColors(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::MozBorderRightColors(_, ref this_end_value),
                     &AnimatedProperty::MozBorderRightColors(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::BoxDecorationBreak(_, ref this_end_value),
                     &AnimatedProperty::BoxDecorationBreak(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::MozFloatEdge(_, ref this_end_value),
                     &AnimatedProperty::MozFloatEdge(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::BorderImageSource(_, ref this_end_value),
                     &AnimatedProperty::BorderImageSource(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::BorderImageOutset(_, ref this_end_value),
                     &AnimatedProperty::BorderImageOutset(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::BorderImageRepeat(_, ref this_end_value),
                     &AnimatedProperty::BorderImageRepeat(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::BorderImageWidth(_, ref this_end_value),
                     &AnimatedProperty::BorderImageWidth(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::BorderImageSlice(_, ref this_end_value),
                     &AnimatedProperty::BorderImageSlice(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::Display(_, ref this_end_value),
                     &AnimatedProperty::Display(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::Position(_, ref this_end_value),
                     &AnimatedProperty::Position(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::Float(_, ref this_end_value),
                     &AnimatedProperty::Float(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::Clear(_, ref this_end_value),
                     &AnimatedProperty::Clear(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::VerticalAlign(_, ref this_end_value),
                     &AnimatedProperty::VerticalAlign(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::OverflowClipBox(_, ref this_end_value),
                     &AnimatedProperty::OverflowClipBox(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::OverflowX(_, ref this_end_value),
                     &AnimatedProperty::OverflowX(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::OverflowY(_, ref this_end_value),
                     &AnimatedProperty::OverflowY(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::ScrollSnapPointsX(_, ref this_end_value),
                     &AnimatedProperty::ScrollSnapPointsX(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::ScrollSnapPointsY(_, ref this_end_value),
                     &AnimatedProperty::ScrollSnapPointsY(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::ScrollSnapDestination(_, ref this_end_value),
                     &AnimatedProperty::ScrollSnapDestination(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::ScrollSnapCoordinate(_, ref this_end_value),
                     &AnimatedProperty::ScrollSnapCoordinate(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::Transform(_, ref this_end_value),
                     &AnimatedProperty::Transform(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::ScrollBehavior(_, ref this_end_value),
                     &AnimatedProperty::ScrollBehavior(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::ScrollSnapTypeX(_, ref this_end_value),
                     &AnimatedProperty::ScrollSnapTypeX(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::ScrollSnapTypeY(_, ref this_end_value),
                     &AnimatedProperty::ScrollSnapTypeY(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::Isolation(_, ref this_end_value),
                     &AnimatedProperty::Isolation(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::PageBreakAfter(_, ref this_end_value),
                     &AnimatedProperty::PageBreakAfter(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::PageBreakBefore(_, ref this_end_value),
                     &AnimatedProperty::PageBreakBefore(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::PageBreakInside(_, ref this_end_value),
                     &AnimatedProperty::PageBreakInside(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::Resize(_, ref this_end_value),
                     &AnimatedProperty::Resize(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::Perspective(_, ref this_end_value),
                     &AnimatedProperty::Perspective(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::PerspectiveOrigin(_, ref this_end_value),
                     &AnimatedProperty::PerspectiveOrigin(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::BackfaceVisibility(_, ref this_end_value),
                     &AnimatedProperty::BackfaceVisibility(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::TransformBox(_, ref this_end_value),
                     &AnimatedProperty::TransformBox(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::TransformStyle(_, ref this_end_value),
                     &AnimatedProperty::TransformStyle(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::TransformOrigin(_, ref this_end_value),
                     &AnimatedProperty::TransformOrigin(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::Contain(_, ref this_end_value),
                     &AnimatedProperty::Contain(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::MozAppearance(_, ref this_end_value),
                     &AnimatedProperty::MozAppearance(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::MozOrient(_, ref this_end_value),
                     &AnimatedProperty::MozOrient(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::WillChange(_, ref this_end_value),
                     &AnimatedProperty::WillChange(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::ShapeOutside(_, ref this_end_value),
                     &AnimatedProperty::ShapeOutside(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::TouchAction(_, ref this_end_value),
                     &AnimatedProperty::TouchAction(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::Color(_, ref this_end_value),
                     &AnimatedProperty::Color(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::ColumnWidth(_, ref this_end_value),
                     &AnimatedProperty::ColumnWidth(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::ColumnCount(_, ref this_end_value),
                     &AnimatedProperty::ColumnCount(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::ColumnGap(_, ref this_end_value),
                     &AnimatedProperty::ColumnGap(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::ColumnFill(_, ref this_end_value),
                     &AnimatedProperty::ColumnFill(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::ColumnRuleWidth(_, ref this_end_value),
                     &AnimatedProperty::ColumnRuleWidth(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::ColumnRuleColor(_, ref this_end_value),
                     &AnimatedProperty::ColumnRuleColor(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::ColumnSpan(_, ref this_end_value),
                     &AnimatedProperty::ColumnSpan(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::ColumnRuleStyle(_, ref this_end_value),
                     &AnimatedProperty::ColumnRuleStyle(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::Content(_, ref this_end_value),
                     &AnimatedProperty::Content(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::CounterIncrement(_, ref this_end_value),
                     &AnimatedProperty::CounterIncrement(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::CounterReset(_, ref this_end_value),
                     &AnimatedProperty::CounterReset(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::Opacity(_, ref this_end_value),
                     &AnimatedProperty::Opacity(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::BoxShadow(_, ref this_end_value),
                     &AnimatedProperty::BoxShadow(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::Clip(_, ref this_end_value),
                     &AnimatedProperty::Clip(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::Filter(_, ref this_end_value),
                     &AnimatedProperty::Filter(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::MixBlendMode(_, ref this_end_value),
                     &AnimatedProperty::MixBlendMode(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::FontFamily(_, ref this_end_value),
                     &AnimatedProperty::FontFamily(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::FontStyle(_, ref this_end_value),
                     &AnimatedProperty::FontStyle(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::FontVariantCaps(_, ref this_end_value),
                     &AnimatedProperty::FontVariantCaps(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::FontWeight(_, ref this_end_value),
                     &AnimatedProperty::FontWeight(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::FontSize(_, ref this_end_value),
                     &AnimatedProperty::FontSize(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::FontSizeAdjust(_, ref this_end_value),
                     &AnimatedProperty::FontSizeAdjust(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::FontSynthesis(_, ref this_end_value),
                     &AnimatedProperty::FontSynthesis(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::FontStretch(_, ref this_end_value),
                     &AnimatedProperty::FontStretch(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::FontKerning(_, ref this_end_value),
                     &AnimatedProperty::FontKerning(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::FontVariantAlternates(_, ref this_end_value),
                     &AnimatedProperty::FontVariantAlternates(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::FontVariantEastAsian(_, ref this_end_value),
                     &AnimatedProperty::FontVariantEastAsian(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::FontVariantLigatures(_, ref this_end_value),
                     &AnimatedProperty::FontVariantLigatures(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::FontVariantNumeric(_, ref this_end_value),
                     &AnimatedProperty::FontVariantNumeric(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::FontVariantPosition(_, ref this_end_value),
                     &AnimatedProperty::FontVariantPosition(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::FontFeatureSettings(_, ref this_end_value),
                     &AnimatedProperty::FontFeatureSettings(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::FontVariationSettings(_, ref this_end_value),
                     &AnimatedProperty::FontVariationSettings(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::FontLanguageOverride(_, ref this_end_value),
                     &AnimatedProperty::FontLanguageOverride(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::MozOsxFontSmoothing(_, ref this_end_value),
                     &AnimatedProperty::MozOsxFontSmoothing(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::Visibility(_, ref this_end_value),
                     &AnimatedProperty::Visibility(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::WritingMode(_, ref this_end_value),
                     &AnimatedProperty::WritingMode(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::Direction(_, ref this_end_value),
                     &AnimatedProperty::Direction(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::TextOrientation(_, ref this_end_value),
                     &AnimatedProperty::TextOrientation(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::ColorAdjust(_, ref this_end_value),
                     &AnimatedProperty::ColorAdjust(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::ImageRendering(_, ref this_end_value),
                     &AnimatedProperty::ImageRendering(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::ImageOrientation(_, ref this_end_value),
                     &AnimatedProperty::ImageOrientation(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::BorderCollapse(_, ref this_end_value),
                     &AnimatedProperty::BorderCollapse(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::EmptyCells(_, ref this_end_value),
                     &AnimatedProperty::EmptyCells(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::CaptionSide(_, ref this_end_value),
                     &AnimatedProperty::CaptionSide(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::BorderSpacing(_, ref this_end_value),
                     &AnimatedProperty::BorderSpacing(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::LineHeight(_, ref this_end_value),
                     &AnimatedProperty::LineHeight(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::TextTransform(_, ref this_end_value),
                     &AnimatedProperty::TextTransform(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::Hyphens(_, ref this_end_value),
                     &AnimatedProperty::Hyphens(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::MozTextSizeAdjust(_, ref this_end_value),
                     &AnimatedProperty::MozTextSizeAdjust(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::TextIndent(_, ref this_end_value),
                     &AnimatedProperty::TextIndent(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::OverflowWrap(_, ref this_end_value),
                     &AnimatedProperty::OverflowWrap(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::WordBreak(_, ref this_end_value),
                     &AnimatedProperty::WordBreak(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::TextJustify(_, ref this_end_value),
                     &AnimatedProperty::TextJustify(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::TextAlignLast(_, ref this_end_value),
                     &AnimatedProperty::TextAlignLast(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::TextAlign(_, ref this_end_value),
                     &AnimatedProperty::TextAlign(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::LetterSpacing(_, ref this_end_value),
                     &AnimatedProperty::LetterSpacing(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::WordSpacing(_, ref this_end_value),
                     &AnimatedProperty::WordSpacing(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::WhiteSpace(_, ref this_end_value),
                     &AnimatedProperty::WhiteSpace(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::TextShadow(_, ref this_end_value),
                     &AnimatedProperty::TextShadow(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::TextEmphasisStyle(_, ref this_end_value),
                     &AnimatedProperty::TextEmphasisStyle(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::TextEmphasisPosition(_, ref this_end_value),
                     &AnimatedProperty::TextEmphasisPosition(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::TextEmphasisColor(_, ref this_end_value),
                     &AnimatedProperty::TextEmphasisColor(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::MozTabSize(_, ref this_end_value),
                     &AnimatedProperty::MozTabSize(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::WebkitTextFillColor(_, ref this_end_value),
                     &AnimatedProperty::WebkitTextFillColor(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::WebkitTextStrokeColor(_, ref this_end_value),
                     &AnimatedProperty::WebkitTextStrokeColor(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::WebkitTextStrokeWidth(_, ref this_end_value),
                     &AnimatedProperty::WebkitTextStrokeWidth(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::RubyAlign(_, ref this_end_value),
                     &AnimatedProperty::RubyAlign(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::RubyPosition(_, ref this_end_value),
                     &AnimatedProperty::RubyPosition(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::TextCombineUpright(_, ref this_end_value),
                     &AnimatedProperty::TextCombineUpright(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::TextRendering(_, ref this_end_value),
                     &AnimatedProperty::TextRendering(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::ListStylePosition(_, ref this_end_value),
                     &AnimatedProperty::ListStylePosition(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::ListStyleType(_, ref this_end_value),
                     &AnimatedProperty::ListStyleType(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::ListStyleImage(_, ref this_end_value),
                     &AnimatedProperty::ListStyleImage(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::Quotes(_, ref this_end_value),
                     &AnimatedProperty::Quotes(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::MozImageRegion(_, ref this_end_value),
                     &AnimatedProperty::MozImageRegion(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::MarginTop(_, ref this_end_value),
                     &AnimatedProperty::MarginTop(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::MarginLeft(_, ref this_end_value),
                     &AnimatedProperty::MarginLeft(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::MarginBottom(_, ref this_end_value),
                     &AnimatedProperty::MarginBottom(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::MarginRight(_, ref this_end_value),
                     &AnimatedProperty::MarginRight(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::OutlineColor(_, ref this_end_value),
                     &AnimatedProperty::OutlineColor(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::OutlineStyle(_, ref this_end_value),
                     &AnimatedProperty::OutlineStyle(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::OutlineWidth(_, ref this_end_value),
                     &AnimatedProperty::OutlineWidth(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::MozOutlineRadiusTopleft(_, ref this_end_value),
                     &AnimatedProperty::MozOutlineRadiusTopleft(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::MozOutlineRadiusTopright(_, ref this_end_value),
                     &AnimatedProperty::MozOutlineRadiusTopright(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::MozOutlineRadiusBottomright(_, ref this_end_value),
                     &AnimatedProperty::MozOutlineRadiusBottomright(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::MozOutlineRadiusBottomleft(_, ref this_end_value),
                     &AnimatedProperty::MozOutlineRadiusBottomleft(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::OutlineOffset(_, ref this_end_value),
                     &AnimatedProperty::OutlineOffset(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::PaddingTop(_, ref this_end_value),
                     &AnimatedProperty::PaddingTop(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::PaddingLeft(_, ref this_end_value),
                     &AnimatedProperty::PaddingLeft(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::PaddingBottom(_, ref this_end_value),
                     &AnimatedProperty::PaddingBottom(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::PaddingRight(_, ref this_end_value),
                     &AnimatedProperty::PaddingRight(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::Cursor(_, ref this_end_value),
                     &AnimatedProperty::Cursor(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::PointerEvents(_, ref this_end_value),
                     &AnimatedProperty::PointerEvents(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::MozUserInput(_, ref this_end_value),
                     &AnimatedProperty::MozUserInput(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::MozUserModify(_, ref this_end_value),
                     &AnimatedProperty::MozUserModify(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::MozUserFocus(_, ref this_end_value),
                     &AnimatedProperty::MozUserFocus(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::CaretColor(_, ref this_end_value),
                     &AnimatedProperty::CaretColor(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::Top(_, ref this_end_value),
                     &AnimatedProperty::Top(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::Left(_, ref this_end_value),
                     &AnimatedProperty::Left(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::Bottom(_, ref this_end_value),
                     &AnimatedProperty::Bottom(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::Right(_, ref this_end_value),
                     &AnimatedProperty::Right(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::ZIndex(_, ref this_end_value),
                     &AnimatedProperty::ZIndex(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::FlexDirection(_, ref this_end_value),
                     &AnimatedProperty::FlexDirection(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::FlexWrap(_, ref this_end_value),
                     &AnimatedProperty::FlexWrap(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::JustifyContent(_, ref this_end_value),
                     &AnimatedProperty::JustifyContent(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::AlignContent(_, ref this_end_value),
                     &AnimatedProperty::AlignContent(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::AlignItems(_, ref this_end_value),
                     &AnimatedProperty::AlignItems(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::JustifyItems(_, ref this_end_value),
                     &AnimatedProperty::JustifyItems(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::FlexGrow(_, ref this_end_value),
                     &AnimatedProperty::FlexGrow(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::FlexShrink(_, ref this_end_value),
                     &AnimatedProperty::FlexShrink(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::AlignSelf(_, ref this_end_value),
                     &AnimatedProperty::AlignSelf(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::JustifySelf(_, ref this_end_value),
                     &AnimatedProperty::JustifySelf(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::Order(_, ref this_end_value),
                     &AnimatedProperty::Order(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::FlexBasis(_, ref this_end_value),
                     &AnimatedProperty::FlexBasis(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::Width(_, ref this_end_value),
                     &AnimatedProperty::Width(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::MinWidth(_, ref this_end_value),
                     &AnimatedProperty::MinWidth(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::MaxWidth(_, ref this_end_value),
                     &AnimatedProperty::MaxWidth(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::Height(_, ref this_end_value),
                     &AnimatedProperty::Height(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::MinHeight(_, ref this_end_value),
                     &AnimatedProperty::MinHeight(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::MaxHeight(_, ref this_end_value),
                     &AnimatedProperty::MaxHeight(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::BoxSizing(_, ref this_end_value),
                     &AnimatedProperty::BoxSizing(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::ObjectFit(_, ref this_end_value),
                     &AnimatedProperty::ObjectFit(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::ObjectPosition(_, ref this_end_value),
                     &AnimatedProperty::ObjectPosition(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::GridRowGap(_, ref this_end_value),
                     &AnimatedProperty::GridRowGap(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::GridRowStart(_, ref this_end_value),
                     &AnimatedProperty::GridRowStart(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::GridRowEnd(_, ref this_end_value),
                     &AnimatedProperty::GridRowEnd(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::GridAutoRows(_, ref this_end_value),
                     &AnimatedProperty::GridAutoRows(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::GridTemplateRows(_, ref this_end_value),
                     &AnimatedProperty::GridTemplateRows(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::GridColumnGap(_, ref this_end_value),
                     &AnimatedProperty::GridColumnGap(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::GridColumnStart(_, ref this_end_value),
                     &AnimatedProperty::GridColumnStart(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::GridColumnEnd(_, ref this_end_value),
                     &AnimatedProperty::GridColumnEnd(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::GridAutoColumns(_, ref this_end_value),
                     &AnimatedProperty::GridAutoColumns(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::GridTemplateColumns(_, ref this_end_value),
                     &AnimatedProperty::GridTemplateColumns(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::GridAutoFlow(_, ref this_end_value),
                     &AnimatedProperty::GridAutoFlow(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::GridTemplateAreas(_, ref this_end_value),
                     &AnimatedProperty::GridTemplateAreas(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::TableLayout(_, ref this_end_value),
                     &AnimatedProperty::TableLayout(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::TextOverflow(_, ref this_end_value),
                     &AnimatedProperty::TextOverflow(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::UnicodeBidi(_, ref this_end_value),
                     &AnimatedProperty::UnicodeBidi(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::TextDecorationLine(_, ref this_end_value),
                     &AnimatedProperty::TextDecorationLine(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::TextDecorationStyle(_, ref this_end_value),
                     &AnimatedProperty::TextDecorationStyle(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::TextDecorationColor(_, ref this_end_value),
                     &AnimatedProperty::TextDecorationColor(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::InitialLetter(_, ref this_end_value),
                     &AnimatedProperty::InitialLetter(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::ImeMode(_, ref this_end_value),
                     &AnimatedProperty::ImeMode(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::MozUserSelect(_, ref this_end_value),
                     &AnimatedProperty::MozUserSelect(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::MozWindowDragging(_, ref this_end_value),
                     &AnimatedProperty::MozWindowDragging(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::MozWindowShadow(_, ref this_end_value),
                     &AnimatedProperty::MozWindowShadow(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::MozForceBrokenImageIcon(_, ref this_end_value),
                     &AnimatedProperty::MozForceBrokenImageIcon(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::TextAnchor(_, ref this_end_value),
                     &AnimatedProperty::TextAnchor(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::ColorInterpolation(_, ref this_end_value),
                     &AnimatedProperty::ColorInterpolation(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::ColorInterpolationFilters(_, ref this_end_value),
                     &AnimatedProperty::ColorInterpolationFilters(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::Fill(_, ref this_end_value),
                     &AnimatedProperty::Fill(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::FillOpacity(_, ref this_end_value),
                     &AnimatedProperty::FillOpacity(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::FillRule(_, ref this_end_value),
                     &AnimatedProperty::FillRule(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::ShapeRendering(_, ref this_end_value),
                     &AnimatedProperty::ShapeRendering(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::Stroke(_, ref this_end_value),
                     &AnimatedProperty::Stroke(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::StrokeWidth(_, ref this_end_value),
                     &AnimatedProperty::StrokeWidth(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::StrokeLinecap(_, ref this_end_value),
                     &AnimatedProperty::StrokeLinecap(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::StrokeLinejoin(_, ref this_end_value),
                     &AnimatedProperty::StrokeLinejoin(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::StrokeMiterlimit(_, ref this_end_value),
                     &AnimatedProperty::StrokeMiterlimit(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::StrokeOpacity(_, ref this_end_value),
                     &AnimatedProperty::StrokeOpacity(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::StrokeDasharray(_, ref this_end_value),
                     &AnimatedProperty::StrokeDasharray(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::StrokeDashoffset(_, ref this_end_value),
                     &AnimatedProperty::StrokeDashoffset(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::ClipRule(_, ref this_end_value),
                     &AnimatedProperty::ClipRule(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::MarkerStart(_, ref this_end_value),
                     &AnimatedProperty::MarkerStart(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::MarkerMid(_, ref this_end_value),
                     &AnimatedProperty::MarkerMid(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::MarkerEnd(_, ref this_end_value),
                     &AnimatedProperty::MarkerEnd(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::PaintOrder(_, ref this_end_value),
                     &AnimatedProperty::PaintOrder(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::DominantBaseline(_, ref this_end_value),
                     &AnimatedProperty::DominantBaseline(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::VectorEffect(_, ref this_end_value),
                     &AnimatedProperty::VectorEffect(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::StopColor(_, ref this_end_value),
                     &AnimatedProperty::StopColor(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::StopOpacity(_, ref this_end_value),
                     &AnimatedProperty::StopOpacity(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::FloodColor(_, ref this_end_value),
                     &AnimatedProperty::FloodColor(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::FloodOpacity(_, ref this_end_value),
                     &AnimatedProperty::FloodOpacity(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::LightingColor(_, ref this_end_value),
                     &AnimatedProperty::LightingColor(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::MaskType(_, ref this_end_value),
                     &AnimatedProperty::MaskType(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::ClipPath(_, ref this_end_value),
                     &AnimatedProperty::ClipPath(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::MaskMode(_, ref this_end_value),
                     &AnimatedProperty::MaskMode(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::MaskRepeat(_, ref this_end_value),
                     &AnimatedProperty::MaskRepeat(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::MaskPositionX(_, ref this_end_value),
                     &AnimatedProperty::MaskPositionX(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::MaskPositionY(_, ref this_end_value),
                     &AnimatedProperty::MaskPositionY(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::MaskClip(_, ref this_end_value),
                     &AnimatedProperty::MaskClip(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::MaskOrigin(_, ref this_end_value),
                     &AnimatedProperty::MaskOrigin(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::MaskSize(_, ref this_end_value),
                     &AnimatedProperty::MaskSize(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::MaskComposite(_, ref this_end_value),
                     &AnimatedProperty::MaskComposite(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::MaskImage(_, ref this_end_value),
                     &AnimatedProperty::MaskImage(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::MozBoxAlign(_, ref this_end_value),
                     &AnimatedProperty::MozBoxAlign(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::MozBoxDirection(_, ref this_end_value),
                     &AnimatedProperty::MozBoxDirection(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::MozBoxFlex(_, ref this_end_value),
                     &AnimatedProperty::MozBoxFlex(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::MozBoxOrient(_, ref this_end_value),
                     &AnimatedProperty::MozBoxOrient(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::MozBoxPack(_, ref this_end_value),
                     &AnimatedProperty::MozBoxPack(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::MozStackSizing(_, ref this_end_value),
                     &AnimatedProperty::MozStackSizing(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
                    (&AnimatedProperty::MozBoxOrdinalGroup(_, ref this_end_value),
                     &AnimatedProperty::MozBoxOrdinalGroup(_, ref other_end_value)) => {
                        this_end_value == other_end_value
                    }
            _ => false,
        }
    }

    /// Update `style` with the proper computed style corresponding to this
    /// animation at `progress`.
    pub fn update(&self, style: &mut ComputedValues, progress: f64) {
        match *self {
                    AnimatedProperty::BackgroundColor(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                            let value: longhands::background_color::computed_value::T =
                                ToAnimatedValue::from_animated_value(value);
                        style.mutate_background().set_background_color(value);
                    }
                    AnimatedProperty::BackgroundImage(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_background().set_background_image(value);
                    }
                    AnimatedProperty::BackgroundPositionX(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                        style.mutate_background().set_background_position_x(value);
                    }
                    AnimatedProperty::BackgroundPositionY(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                        style.mutate_background().set_background_position_y(value);
                    }
                    AnimatedProperty::BackgroundRepeat(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_background().set_background_repeat(value);
                    }
                    AnimatedProperty::BackgroundAttachment(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_background().set_background_attachment(value);
                    }
                    AnimatedProperty::BackgroundClip(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_background().set_background_clip(value);
                    }
                    AnimatedProperty::BackgroundOrigin(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_background().set_background_origin(value);
                    }
                    AnimatedProperty::BackgroundSize(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                            let value: longhands::background_size::computed_value::T =
                                ToAnimatedValue::from_animated_value(value);
                        style.mutate_background().set_background_size(value);
                    }
                    AnimatedProperty::BackgroundBlendMode(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_background().set_background_blend_mode(value);
                    }
                    AnimatedProperty::BorderTopColor(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                            let value: longhands::border_top_color::computed_value::T =
                                ToAnimatedValue::from_animated_value(value);
                        style.mutate_border().set_border_top_color(value);
                    }
                    AnimatedProperty::BorderTopStyle(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_border().set_border_top_style(value);
                    }
                    AnimatedProperty::BorderTopWidth(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                            let value: longhands::border_top_width::computed_value::T =
                                ToAnimatedValue::from_animated_value(value);
                        style.mutate_border().set_border_top_width(value);
                    }
                    AnimatedProperty::BorderLeftColor(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                            let value: longhands::border_left_color::computed_value::T =
                                ToAnimatedValue::from_animated_value(value);
                        style.mutate_border().set_border_left_color(value);
                    }
                    AnimatedProperty::BorderLeftStyle(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_border().set_border_left_style(value);
                    }
                    AnimatedProperty::BorderLeftWidth(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                            let value: longhands::border_left_width::computed_value::T =
                                ToAnimatedValue::from_animated_value(value);
                        style.mutate_border().set_border_left_width(value);
                    }
                    AnimatedProperty::BorderBottomColor(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                            let value: longhands::border_bottom_color::computed_value::T =
                                ToAnimatedValue::from_animated_value(value);
                        style.mutate_border().set_border_bottom_color(value);
                    }
                    AnimatedProperty::BorderBottomStyle(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_border().set_border_bottom_style(value);
                    }
                    AnimatedProperty::BorderBottomWidth(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                            let value: longhands::border_bottom_width::computed_value::T =
                                ToAnimatedValue::from_animated_value(value);
                        style.mutate_border().set_border_bottom_width(value);
                    }
                    AnimatedProperty::BorderRightColor(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                            let value: longhands::border_right_color::computed_value::T =
                                ToAnimatedValue::from_animated_value(value);
                        style.mutate_border().set_border_right_color(value);
                    }
                    AnimatedProperty::BorderRightStyle(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_border().set_border_right_style(value);
                    }
                    AnimatedProperty::BorderRightWidth(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                            let value: longhands::border_right_width::computed_value::T =
                                ToAnimatedValue::from_animated_value(value);
                        style.mutate_border().set_border_right_width(value);
                    }
                    AnimatedProperty::BorderTopLeftRadius(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                            let value: longhands::border_top_left_radius::computed_value::T =
                                ToAnimatedValue::from_animated_value(value);
                        style.mutate_border().set_border_top_left_radius(value);
                    }
                    AnimatedProperty::BorderTopRightRadius(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                            let value: longhands::border_top_right_radius::computed_value::T =
                                ToAnimatedValue::from_animated_value(value);
                        style.mutate_border().set_border_top_right_radius(value);
                    }
                    AnimatedProperty::BorderBottomRightRadius(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                            let value: longhands::border_bottom_right_radius::computed_value::T =
                                ToAnimatedValue::from_animated_value(value);
                        style.mutate_border().set_border_bottom_right_radius(value);
                    }
                    AnimatedProperty::BorderBottomLeftRadius(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                            let value: longhands::border_bottom_left_radius::computed_value::T =
                                ToAnimatedValue::from_animated_value(value);
                        style.mutate_border().set_border_bottom_left_radius(value);
                    }
                    AnimatedProperty::MozBorderTopColors(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_border().set__moz_border_top_colors(value);
                    }
                    AnimatedProperty::MozBorderLeftColors(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_border().set__moz_border_left_colors(value);
                    }
                    AnimatedProperty::MozBorderBottomColors(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_border().set__moz_border_bottom_colors(value);
                    }
                    AnimatedProperty::MozBorderRightColors(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_border().set__moz_border_right_colors(value);
                    }
                    AnimatedProperty::BoxDecorationBreak(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_border().set_box_decoration_break(value);
                    }
                    AnimatedProperty::MozFloatEdge(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_border().set__moz_float_edge(value);
                    }
                    AnimatedProperty::BorderImageSource(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_border().set_border_image_source(value);
                    }
                    AnimatedProperty::BorderImageOutset(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_border().set_border_image_outset(value);
                    }
                    AnimatedProperty::BorderImageRepeat(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_border().set_border_image_repeat(value);
                    }
                    AnimatedProperty::BorderImageWidth(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_border().set_border_image_width(value);
                    }
                    AnimatedProperty::BorderImageSlice(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_border().set_border_image_slice(value);
                    }
                    AnimatedProperty::Display(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_box().set_display(value);
                    }
                    AnimatedProperty::Position(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_box().set_position(value);
                    }
                    AnimatedProperty::Float(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_box().set_float(value);
                    }
                    AnimatedProperty::Clear(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_box().set_clear(value);
                    }
                    AnimatedProperty::VerticalAlign(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                        style.mutate_box().set_vertical_align(value);
                    }
                    AnimatedProperty::OverflowClipBox(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_box().set_overflow_clip_box(value);
                    }
                    AnimatedProperty::OverflowX(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_box().set_overflow_x(value);
                    }
                    AnimatedProperty::OverflowY(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_box().set_overflow_y(value);
                    }
                    AnimatedProperty::ScrollSnapPointsX(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_box().set_scroll_snap_points_x(value);
                    }
                    AnimatedProperty::ScrollSnapPointsY(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_box().set_scroll_snap_points_y(value);
                    }
                    AnimatedProperty::ScrollSnapDestination(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_box().set_scroll_snap_destination(value);
                    }
                    AnimatedProperty::ScrollSnapCoordinate(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_box().set_scroll_snap_coordinate(value);
                    }
                    AnimatedProperty::Transform(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                        style.mutate_box().set_transform(value);
                    }
                    AnimatedProperty::ScrollBehavior(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_box().set_scroll_behavior(value);
                    }
                    AnimatedProperty::ScrollSnapTypeX(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_box().set_scroll_snap_type_x(value);
                    }
                    AnimatedProperty::ScrollSnapTypeY(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_box().set_scroll_snap_type_y(value);
                    }
                    AnimatedProperty::Isolation(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_box().set_isolation(value);
                    }
                    AnimatedProperty::PageBreakAfter(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_box().set_page_break_after(value);
                    }
                    AnimatedProperty::PageBreakBefore(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_box().set_page_break_before(value);
                    }
                    AnimatedProperty::PageBreakInside(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_box().set_page_break_inside(value);
                    }
                    AnimatedProperty::Resize(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_box().set_resize(value);
                    }
                    AnimatedProperty::Perspective(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                        style.mutate_box().set_perspective(value);
                    }
                    AnimatedProperty::PerspectiveOrigin(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                        style.mutate_box().set_perspective_origin(value);
                    }
                    AnimatedProperty::BackfaceVisibility(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_box().set_backface_visibility(value);
                    }
                    AnimatedProperty::TransformBox(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_box().set_transform_box(value);
                    }
                    AnimatedProperty::TransformStyle(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_box().set_transform_style(value);
                    }
                    AnimatedProperty::TransformOrigin(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                        style.mutate_box().set_transform_origin(value);
                    }
                    AnimatedProperty::Contain(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_box().set_contain(value);
                    }
                    AnimatedProperty::MozAppearance(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_box().set__moz_appearance(value);
                    }
                    AnimatedProperty::MozOrient(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_box().set__moz_orient(value);
                    }
                    AnimatedProperty::WillChange(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_box().set_will_change(value);
                    }
                    AnimatedProperty::ShapeOutside(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                        style.mutate_box().set_shape_outside(value);
                    }
                    AnimatedProperty::TouchAction(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_box().set_touch_action(value);
                    }
                    AnimatedProperty::Color(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                            let value: longhands::color::computed_value::T =
                                ToAnimatedValue::from_animated_value(value);
                        style.mutate_color().set_color(value);
                    }
                    AnimatedProperty::ColumnWidth(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                            let value: longhands::column_width::computed_value::T =
                                ToAnimatedValue::from_animated_value(value);
                        style.mutate_column().set_column_width(value);
                    }
                    AnimatedProperty::ColumnCount(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                            let value: longhands::column_count::computed_value::T =
                                ToAnimatedValue::from_animated_value(value);
                        style.mutate_column().set_column_count(value);
                    }
                    AnimatedProperty::ColumnGap(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                            let value: longhands::column_gap::computed_value::T =
                                ToAnimatedValue::from_animated_value(value);
                        style.mutate_column().set_column_gap(value);
                    }
                    AnimatedProperty::ColumnFill(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_column().set_column_fill(value);
                    }
                    AnimatedProperty::ColumnRuleWidth(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                            let value: longhands::column_rule_width::computed_value::T =
                                ToAnimatedValue::from_animated_value(value);
                        style.mutate_column().set_column_rule_width(value);
                    }
                    AnimatedProperty::ColumnRuleColor(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                            let value: longhands::column_rule_color::computed_value::T =
                                ToAnimatedValue::from_animated_value(value);
                        style.mutate_column().set_column_rule_color(value);
                    }
                    AnimatedProperty::ColumnSpan(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_column().set_column_span(value);
                    }
                    AnimatedProperty::ColumnRuleStyle(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_column().set_column_rule_style(value);
                    }
                    AnimatedProperty::Content(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_counters().set_content(value);
                    }
                    AnimatedProperty::CounterIncrement(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_counters().set_counter_increment(value);
                    }
                    AnimatedProperty::CounterReset(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_counters().set_counter_reset(value);
                    }
                    AnimatedProperty::Opacity(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                        style.mutate_effects().set_opacity(value);
                    }
                    AnimatedProperty::BoxShadow(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                            let value: longhands::box_shadow::computed_value::T =
                                ToAnimatedValue::from_animated_value(value);
                        style.mutate_effects().set_box_shadow(value);
                    }
                    AnimatedProperty::Clip(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                        style.mutate_effects().set_clip(value);
                    }
                    AnimatedProperty::Filter(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                            let value: longhands::filter::computed_value::T =
                                ToAnimatedValue::from_animated_value(value);
                        style.mutate_effects().set_filter(value);
                    }
                    AnimatedProperty::MixBlendMode(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_effects().set_mix_blend_mode(value);
                    }
                    AnimatedProperty::FontFamily(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_font().set_font_family(value);
                    }
                    AnimatedProperty::FontStyle(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_font().set_font_style(value);
                    }
                    AnimatedProperty::FontVariantCaps(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_font().set_font_variant_caps(value);
                    }
                    AnimatedProperty::FontWeight(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                        style.mutate_font().set_font_weight(value);
                    }
                    AnimatedProperty::FontSize(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                            let value: longhands::font_size::computed_value::T =
                                ToAnimatedValue::from_animated_value(value);
                        style.mutate_font().set_font_size(value);
                    }
                    AnimatedProperty::FontSizeAdjust(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                            let value: longhands::font_size_adjust::computed_value::T =
                                ToAnimatedValue::from_animated_value(value);
                        style.mutate_font().set_font_size_adjust(value);
                    }
                    AnimatedProperty::FontSynthesis(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_font().set_font_synthesis(value);
                    }
                    AnimatedProperty::FontStretch(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                        style.mutate_font().set_font_stretch(value);
                    }
                    AnimatedProperty::FontKerning(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_font().set_font_kerning(value);
                    }
                    AnimatedProperty::FontVariantAlternates(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_font().set_font_variant_alternates(value);
                    }
                    AnimatedProperty::FontVariantEastAsian(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_font().set_font_variant_east_asian(value);
                    }
                    AnimatedProperty::FontVariantLigatures(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_font().set_font_variant_ligatures(value);
                    }
                    AnimatedProperty::FontVariantNumeric(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_font().set_font_variant_numeric(value);
                    }
                    AnimatedProperty::FontVariantPosition(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_font().set_font_variant_position(value);
                    }
                    AnimatedProperty::FontFeatureSettings(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_font().set_font_feature_settings(value);
                    }
                    AnimatedProperty::FontVariationSettings(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                        style.mutate_font().set_font_variation_settings(value);
                    }
                    AnimatedProperty::FontLanguageOverride(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_font().set_font_language_override(value);
                    }
                    AnimatedProperty::MozOsxFontSmoothing(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_font().set__moz_osx_font_smoothing(value);
                    }
                    AnimatedProperty::Visibility(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                        style.mutate_inheritedbox().set_visibility(value);
                    }
                    AnimatedProperty::WritingMode(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_inheritedbox().set_writing_mode(value);
                    }
                    AnimatedProperty::Direction(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_inheritedbox().set_direction(value);
                    }
                    AnimatedProperty::TextOrientation(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_inheritedbox().set_text_orientation(value);
                    }
                    AnimatedProperty::ColorAdjust(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_inheritedbox().set_color_adjust(value);
                    }
                    AnimatedProperty::ImageRendering(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_inheritedbox().set_image_rendering(value);
                    }
                    AnimatedProperty::ImageOrientation(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_inheritedbox().set_image_orientation(value);
                    }
                    AnimatedProperty::BorderCollapse(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_inheritedtable().set_border_collapse(value);
                    }
                    AnimatedProperty::EmptyCells(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_inheritedtable().set_empty_cells(value);
                    }
                    AnimatedProperty::CaptionSide(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_inheritedtable().set_caption_side(value);
                    }
                    AnimatedProperty::BorderSpacing(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                            let value: longhands::border_spacing::computed_value::T =
                                ToAnimatedValue::from_animated_value(value);
                        style.mutate_inheritedtable().set_border_spacing(value);
                    }
                    AnimatedProperty::LineHeight(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                            let value: longhands::line_height::computed_value::T =
                                ToAnimatedValue::from_animated_value(value);
                        style.mutate_inheritedtext().set_line_height(value);
                    }
                    AnimatedProperty::TextTransform(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_inheritedtext().set_text_transform(value);
                    }
                    AnimatedProperty::Hyphens(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_inheritedtext().set_hyphens(value);
                    }
                    AnimatedProperty::MozTextSizeAdjust(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_inheritedtext().set__moz_text_size_adjust(value);
                    }
                    AnimatedProperty::TextIndent(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                        style.mutate_inheritedtext().set_text_indent(value);
                    }
                    AnimatedProperty::OverflowWrap(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_inheritedtext().set_overflow_wrap(value);
                    }
                    AnimatedProperty::WordBreak(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_inheritedtext().set_word_break(value);
                    }
                    AnimatedProperty::TextJustify(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_inheritedtext().set_text_justify(value);
                    }
                    AnimatedProperty::TextAlignLast(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_inheritedtext().set_text_align_last(value);
                    }
                    AnimatedProperty::TextAlign(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_inheritedtext().set_text_align(value);
                    }
                    AnimatedProperty::LetterSpacing(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                        style.mutate_inheritedtext().set_letter_spacing(value);
                    }
                    AnimatedProperty::WordSpacing(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                        style.mutate_inheritedtext().set_word_spacing(value);
                    }
                    AnimatedProperty::WhiteSpace(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_inheritedtext().set_white_space(value);
                    }
                    AnimatedProperty::TextShadow(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                            let value: longhands::text_shadow::computed_value::T =
                                ToAnimatedValue::from_animated_value(value);
                        style.mutate_inheritedtext().set_text_shadow(value);
                    }
                    AnimatedProperty::TextEmphasisStyle(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_inheritedtext().set_text_emphasis_style(value);
                    }
                    AnimatedProperty::TextEmphasisPosition(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_inheritedtext().set_text_emphasis_position(value);
                    }
                    AnimatedProperty::TextEmphasisColor(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                            let value: longhands::text_emphasis_color::computed_value::T =
                                ToAnimatedValue::from_animated_value(value);
                        style.mutate_inheritedtext().set_text_emphasis_color(value);
                    }
                    AnimatedProperty::MozTabSize(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                            let value: longhands::_moz_tab_size::computed_value::T =
                                ToAnimatedValue::from_animated_value(value);
                        style.mutate_inheritedtext().set__moz_tab_size(value);
                    }
                    AnimatedProperty::WebkitTextFillColor(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                            let value: longhands::_webkit_text_fill_color::computed_value::T =
                                ToAnimatedValue::from_animated_value(value);
                        style.mutate_inheritedtext().set__webkit_text_fill_color(value);
                    }
                    AnimatedProperty::WebkitTextStrokeColor(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                            let value: longhands::_webkit_text_stroke_color::computed_value::T =
                                ToAnimatedValue::from_animated_value(value);
                        style.mutate_inheritedtext().set__webkit_text_stroke_color(value);
                    }
                    AnimatedProperty::WebkitTextStrokeWidth(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_inheritedtext().set__webkit_text_stroke_width(value);
                    }
                    AnimatedProperty::RubyAlign(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_inheritedtext().set_ruby_align(value);
                    }
                    AnimatedProperty::RubyPosition(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_inheritedtext().set_ruby_position(value);
                    }
                    AnimatedProperty::TextCombineUpright(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_inheritedtext().set_text_combine_upright(value);
                    }
                    AnimatedProperty::TextRendering(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_inheritedtext().set_text_rendering(value);
                    }
                    AnimatedProperty::ListStylePosition(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_list().set_list_style_position(value);
                    }
                    AnimatedProperty::ListStyleType(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_list().set_list_style_type(value);
                    }
                    AnimatedProperty::ListStyleImage(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_list().set_list_style_image(value);
                    }
                    AnimatedProperty::Quotes(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_list().set_quotes(value);
                    }
                    AnimatedProperty::MozImageRegion(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                        style.mutate_list().set__moz_image_region(value);
                    }
                    AnimatedProperty::MarginTop(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                        style.mutate_margin().set_margin_top(value);
                    }
                    AnimatedProperty::MarginLeft(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                        style.mutate_margin().set_margin_left(value);
                    }
                    AnimatedProperty::MarginBottom(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                        style.mutate_margin().set_margin_bottom(value);
                    }
                    AnimatedProperty::MarginRight(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                        style.mutate_margin().set_margin_right(value);
                    }
                    AnimatedProperty::OutlineColor(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                            let value: longhands::outline_color::computed_value::T =
                                ToAnimatedValue::from_animated_value(value);
                        style.mutate_outline().set_outline_color(value);
                    }
                    AnimatedProperty::OutlineStyle(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_outline().set_outline_style(value);
                    }
                    AnimatedProperty::OutlineWidth(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                            let value: longhands::outline_width::computed_value::T =
                                ToAnimatedValue::from_animated_value(value);
                        style.mutate_outline().set_outline_width(value);
                    }
                    AnimatedProperty::MozOutlineRadiusTopleft(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                            let value: longhands::_moz_outline_radius_topleft::computed_value::T =
                                ToAnimatedValue::from_animated_value(value);
                        style.mutate_outline().set__moz_outline_radius_topleft(value);
                    }
                    AnimatedProperty::MozOutlineRadiusTopright(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                            let value: longhands::_moz_outline_radius_topright::computed_value::T =
                                ToAnimatedValue::from_animated_value(value);
                        style.mutate_outline().set__moz_outline_radius_topright(value);
                    }
                    AnimatedProperty::MozOutlineRadiusBottomright(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                            let value: longhands::_moz_outline_radius_bottomright::computed_value::T =
                                ToAnimatedValue::from_animated_value(value);
                        style.mutate_outline().set__moz_outline_radius_bottomright(value);
                    }
                    AnimatedProperty::MozOutlineRadiusBottomleft(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                            let value: longhands::_moz_outline_radius_bottomleft::computed_value::T =
                                ToAnimatedValue::from_animated_value(value);
                        style.mutate_outline().set__moz_outline_radius_bottomleft(value);
                    }
                    AnimatedProperty::OutlineOffset(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                        style.mutate_outline().set_outline_offset(value);
                    }
                    AnimatedProperty::PaddingTop(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                            let value: longhands::padding_top::computed_value::T =
                                ToAnimatedValue::from_animated_value(value);
                        style.mutate_padding().set_padding_top(value);
                    }
                    AnimatedProperty::PaddingLeft(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                            let value: longhands::padding_left::computed_value::T =
                                ToAnimatedValue::from_animated_value(value);
                        style.mutate_padding().set_padding_left(value);
                    }
                    AnimatedProperty::PaddingBottom(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                            let value: longhands::padding_bottom::computed_value::T =
                                ToAnimatedValue::from_animated_value(value);
                        style.mutate_padding().set_padding_bottom(value);
                    }
                    AnimatedProperty::PaddingRight(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                            let value: longhands::padding_right::computed_value::T =
                                ToAnimatedValue::from_animated_value(value);
                        style.mutate_padding().set_padding_right(value);
                    }
                    AnimatedProperty::Cursor(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_pointing().set_cursor(value);
                    }
                    AnimatedProperty::PointerEvents(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_pointing().set_pointer_events(value);
                    }
                    AnimatedProperty::MozUserInput(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_pointing().set__moz_user_input(value);
                    }
                    AnimatedProperty::MozUserModify(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_pointing().set__moz_user_modify(value);
                    }
                    AnimatedProperty::MozUserFocus(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_pointing().set__moz_user_focus(value);
                    }
                    AnimatedProperty::CaretColor(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                            let value: longhands::caret_color::computed_value::T =
                                ToAnimatedValue::from_animated_value(value);
                        style.mutate_pointing().set_caret_color(value);
                    }
                    AnimatedProperty::Top(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                        style.mutate_position().set_top(value);
                    }
                    AnimatedProperty::Left(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                        style.mutate_position().set_left(value);
                    }
                    AnimatedProperty::Bottom(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                        style.mutate_position().set_bottom(value);
                    }
                    AnimatedProperty::Right(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                        style.mutate_position().set_right(value);
                    }
                    AnimatedProperty::ZIndex(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                        style.mutate_position().set_z_index(value);
                    }
                    AnimatedProperty::FlexDirection(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_position().set_flex_direction(value);
                    }
                    AnimatedProperty::FlexWrap(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_position().set_flex_wrap(value);
                    }
                    AnimatedProperty::JustifyContent(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_position().set_justify_content(value);
                    }
                    AnimatedProperty::AlignContent(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_position().set_align_content(value);
                    }
                    AnimatedProperty::AlignItems(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_position().set_align_items(value);
                    }
                    AnimatedProperty::JustifyItems(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_position().set_justify_items(value);
                    }
                    AnimatedProperty::FlexGrow(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                            let value: longhands::flex_grow::computed_value::T =
                                ToAnimatedValue::from_animated_value(value);
                        style.mutate_position().set_flex_grow(value);
                    }
                    AnimatedProperty::FlexShrink(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                            let value: longhands::flex_shrink::computed_value::T =
                                ToAnimatedValue::from_animated_value(value);
                        style.mutate_position().set_flex_shrink(value);
                    }
                    AnimatedProperty::AlignSelf(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_position().set_align_self(value);
                    }
                    AnimatedProperty::JustifySelf(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_position().set_justify_self(value);
                    }
                    AnimatedProperty::Order(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                        style.mutate_position().set_order(value);
                    }
                    AnimatedProperty::FlexBasis(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                            let value: longhands::flex_basis::computed_value::T =
                                ToAnimatedValue::from_animated_value(value);
                        style.mutate_position().set_flex_basis(value);
                    }
                    AnimatedProperty::Width(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                            let value: longhands::width::computed_value::T =
                                ToAnimatedValue::from_animated_value(value);
                        style.mutate_position().set_width(value);
                    }
                    AnimatedProperty::MinWidth(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                            let value: longhands::min_width::computed_value::T =
                                ToAnimatedValue::from_animated_value(value);
                        style.mutate_position().set_min_width(value);
                    }
                    AnimatedProperty::MaxWidth(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                            let value: longhands::max_width::computed_value::T =
                                ToAnimatedValue::from_animated_value(value);
                        style.mutate_position().set_max_width(value);
                    }
                    AnimatedProperty::Height(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                            let value: longhands::height::computed_value::T =
                                ToAnimatedValue::from_animated_value(value);
                        style.mutate_position().set_height(value);
                    }
                    AnimatedProperty::MinHeight(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                            let value: longhands::min_height::computed_value::T =
                                ToAnimatedValue::from_animated_value(value);
                        style.mutate_position().set_min_height(value);
                    }
                    AnimatedProperty::MaxHeight(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                            let value: longhands::max_height::computed_value::T =
                                ToAnimatedValue::from_animated_value(value);
                        style.mutate_position().set_max_height(value);
                    }
                    AnimatedProperty::BoxSizing(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_position().set_box_sizing(value);
                    }
                    AnimatedProperty::ObjectFit(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_position().set_object_fit(value);
                    }
                    AnimatedProperty::ObjectPosition(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                        style.mutate_position().set_object_position(value);
                    }
                    AnimatedProperty::GridRowGap(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                            let value: longhands::grid_row_gap::computed_value::T =
                                ToAnimatedValue::from_animated_value(value);
                        style.mutate_position().set_grid_row_gap(value);
                    }
                    AnimatedProperty::GridRowStart(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_position().set_grid_row_start(value);
                    }
                    AnimatedProperty::GridRowEnd(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_position().set_grid_row_end(value);
                    }
                    AnimatedProperty::GridAutoRows(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_position().set_grid_auto_rows(value);
                    }
                    AnimatedProperty::GridTemplateRows(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_position().set_grid_template_rows(value);
                    }
                    AnimatedProperty::GridColumnGap(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                            let value: longhands::grid_column_gap::computed_value::T =
                                ToAnimatedValue::from_animated_value(value);
                        style.mutate_position().set_grid_column_gap(value);
                    }
                    AnimatedProperty::GridColumnStart(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_position().set_grid_column_start(value);
                    }
                    AnimatedProperty::GridColumnEnd(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_position().set_grid_column_end(value);
                    }
                    AnimatedProperty::GridAutoColumns(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_position().set_grid_auto_columns(value);
                    }
                    AnimatedProperty::GridTemplateColumns(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_position().set_grid_template_columns(value);
                    }
                    AnimatedProperty::GridAutoFlow(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_position().set_grid_auto_flow(value);
                    }
                    AnimatedProperty::GridTemplateAreas(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_position().set_grid_template_areas(value);
                    }
                    AnimatedProperty::TableLayout(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_table().set_table_layout(value);
                    }
                    AnimatedProperty::TextOverflow(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_text().set_text_overflow(value);
                    }
                    AnimatedProperty::UnicodeBidi(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_text().set_unicode_bidi(value);
                    }
                    AnimatedProperty::TextDecorationLine(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_text().set_text_decoration_line(value);
                    }
                    AnimatedProperty::TextDecorationStyle(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_text().set_text_decoration_style(value);
                    }
                    AnimatedProperty::TextDecorationColor(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                            let value: longhands::text_decoration_color::computed_value::T =
                                ToAnimatedValue::from_animated_value(value);
                        style.mutate_text().set_text_decoration_color(value);
                    }
                    AnimatedProperty::InitialLetter(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_text().set_initial_letter(value);
                    }
                    AnimatedProperty::ImeMode(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_ui().set_ime_mode(value);
                    }
                    AnimatedProperty::MozUserSelect(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_ui().set__moz_user_select(value);
                    }
                    AnimatedProperty::MozWindowDragging(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_ui().set__moz_window_dragging(value);
                    }
                    AnimatedProperty::MozWindowShadow(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_ui().set__moz_window_shadow(value);
                    }
                    AnimatedProperty::MozForceBrokenImageIcon(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_ui().set__moz_force_broken_image_icon(value);
                    }
                    AnimatedProperty::TextAnchor(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_inheritedsvg().set_text_anchor(value);
                    }
                    AnimatedProperty::ColorInterpolation(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_inheritedsvg().set_color_interpolation(value);
                    }
                    AnimatedProperty::ColorInterpolationFilters(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_inheritedsvg().set_color_interpolation_filters(value);
                    }
                    AnimatedProperty::Fill(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                            let value: longhands::fill::computed_value::T =
                                ToAnimatedValue::from_animated_value(value);
                        style.mutate_inheritedsvg().set_fill(value);
                    }
                    AnimatedProperty::FillOpacity(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                        style.mutate_inheritedsvg().set_fill_opacity(value);
                    }
                    AnimatedProperty::FillRule(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_inheritedsvg().set_fill_rule(value);
                    }
                    AnimatedProperty::ShapeRendering(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_inheritedsvg().set_shape_rendering(value);
                    }
                    AnimatedProperty::Stroke(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                            let value: longhands::stroke::computed_value::T =
                                ToAnimatedValue::from_animated_value(value);
                        style.mutate_inheritedsvg().set_stroke(value);
                    }
                    AnimatedProperty::StrokeWidth(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                            let value: longhands::stroke_width::computed_value::T =
                                ToAnimatedValue::from_animated_value(value);
                        style.mutate_inheritedsvg().set_stroke_width(value);
                    }
                    AnimatedProperty::StrokeLinecap(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_inheritedsvg().set_stroke_linecap(value);
                    }
                    AnimatedProperty::StrokeLinejoin(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_inheritedsvg().set_stroke_linejoin(value);
                    }
                    AnimatedProperty::StrokeMiterlimit(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                            let value: longhands::stroke_miterlimit::computed_value::T =
                                ToAnimatedValue::from_animated_value(value);
                        style.mutate_inheritedsvg().set_stroke_miterlimit(value);
                    }
                    AnimatedProperty::StrokeOpacity(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                        style.mutate_inheritedsvg().set_stroke_opacity(value);
                    }
                    AnimatedProperty::StrokeDasharray(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                            let value: longhands::stroke_dasharray::computed_value::T =
                                ToAnimatedValue::from_animated_value(value);
                        style.mutate_inheritedsvg().set_stroke_dasharray(value);
                    }
                    AnimatedProperty::StrokeDashoffset(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                        style.mutate_inheritedsvg().set_stroke_dashoffset(value);
                    }
                    AnimatedProperty::ClipRule(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_inheritedsvg().set_clip_rule(value);
                    }
                    AnimatedProperty::MarkerStart(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_inheritedsvg().set_marker_start(value);
                    }
                    AnimatedProperty::MarkerMid(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_inheritedsvg().set_marker_mid(value);
                    }
                    AnimatedProperty::MarkerEnd(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_inheritedsvg().set_marker_end(value);
                    }
                    AnimatedProperty::PaintOrder(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_inheritedsvg().set_paint_order(value);
                    }
                    AnimatedProperty::DominantBaseline(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_svg().set_dominant_baseline(value);
                    }
                    AnimatedProperty::VectorEffect(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_svg().set_vector_effect(value);
                    }
                    AnimatedProperty::StopColor(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                            let value: longhands::stop_color::computed_value::T =
                                ToAnimatedValue::from_animated_value(value);
                        style.mutate_svg().set_stop_color(value);
                    }
                    AnimatedProperty::StopOpacity(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                        style.mutate_svg().set_stop_opacity(value);
                    }
                    AnimatedProperty::FloodColor(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                            let value: longhands::flood_color::computed_value::T =
                                ToAnimatedValue::from_animated_value(value);
                        style.mutate_svg().set_flood_color(value);
                    }
                    AnimatedProperty::FloodOpacity(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                        style.mutate_svg().set_flood_opacity(value);
                    }
                    AnimatedProperty::LightingColor(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                            let value: longhands::lighting_color::computed_value::T =
                                ToAnimatedValue::from_animated_value(value);
                        style.mutate_svg().set_lighting_color(value);
                    }
                    AnimatedProperty::MaskType(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_svg().set_mask_type(value);
                    }
                    AnimatedProperty::ClipPath(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                        style.mutate_svg().set_clip_path(value);
                    }
                    AnimatedProperty::MaskMode(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_svg().set_mask_mode(value);
                    }
                    AnimatedProperty::MaskRepeat(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_svg().set_mask_repeat(value);
                    }
                    AnimatedProperty::MaskPositionX(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                        style.mutate_svg().set_mask_position_x(value);
                    }
                    AnimatedProperty::MaskPositionY(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                        style.mutate_svg().set_mask_position_y(value);
                    }
                    AnimatedProperty::MaskClip(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_svg().set_mask_clip(value);
                    }
                    AnimatedProperty::MaskOrigin(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_svg().set_mask_origin(value);
                    }
                    AnimatedProperty::MaskSize(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                        style.mutate_svg().set_mask_size(value);
                    }
                    AnimatedProperty::MaskComposite(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_svg().set_mask_composite(value);
                    }
                    AnimatedProperty::MaskImage(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_svg().set_mask_image(value);
                    }
                    AnimatedProperty::MozBoxAlign(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_xul().set__moz_box_align(value);
                    }
                    AnimatedProperty::MozBoxDirection(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_xul().set__moz_box_direction(value);
                    }
                    AnimatedProperty::MozBoxFlex(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = match from.animate(to, Procedure::Interpolate { progress }) {
                                Ok(value) => value,
                                Err(()) => return,
                            };
                            let value: longhands::_moz_box_flex::computed_value::T =
                                ToAnimatedValue::from_animated_value(value);
                        style.mutate_xul().set__moz_box_flex(value);
                    }
                    AnimatedProperty::MozBoxOrient(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_xul().set__moz_box_orient(value);
                    }
                    AnimatedProperty::MozBoxPack(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_xul().set__moz_box_pack(value);
                    }
                    AnimatedProperty::MozStackSizing(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_xul().set__moz_stack_sizing(value);
                    }
                    AnimatedProperty::MozBoxOrdinalGroup(ref from, ref to) => {
                        // https://w3c.github.io/web-animations/#discrete-animation-type
                            let value = if progress < 0.5 { from.clone() } else { to.clone() };
                        style.mutate_xul().set__moz_box_ordinal_group(value);
                    }
        }
    }

    /// Get an animatable value from a transition-property, an old style, and a
    /// new style.
    pub fn from_animatable_longhand(property: &AnimatableLonghand,
                                    old_style: &ComputedValues,
                                    new_style: &ComputedValues)
                                    -> AnimatedProperty {
        match *property {
                AnimatableLonghand::BackgroundColor => {
                    let old_computed = old_style.get_background().clone_background_color();
                    let new_computed = new_style.get_background().clone_background_color();
                    AnimatedProperty::BackgroundColor(
                        old_computed.to_animated_value(),
                        new_computed.to_animated_value(),
                    )
                }
                AnimatableLonghand::BackgroundImage => {
                    let old_computed = old_style.get_background().clone_background_image();
                    let new_computed = new_style.get_background().clone_background_image();
                    AnimatedProperty::BackgroundImage(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::BackgroundPositionX => {
                    let old_computed = old_style.get_background().clone_background_position_x();
                    let new_computed = new_style.get_background().clone_background_position_x();
                    AnimatedProperty::BackgroundPositionX(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::BackgroundPositionY => {
                    let old_computed = old_style.get_background().clone_background_position_y();
                    let new_computed = new_style.get_background().clone_background_position_y();
                    AnimatedProperty::BackgroundPositionY(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::BackgroundRepeat => {
                    let old_computed = old_style.get_background().clone_background_repeat();
                    let new_computed = new_style.get_background().clone_background_repeat();
                    AnimatedProperty::BackgroundRepeat(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::BackgroundAttachment => {
                    let old_computed = old_style.get_background().clone_background_attachment();
                    let new_computed = new_style.get_background().clone_background_attachment();
                    AnimatedProperty::BackgroundAttachment(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::BackgroundClip => {
                    let old_computed = old_style.get_background().clone_background_clip();
                    let new_computed = new_style.get_background().clone_background_clip();
                    AnimatedProperty::BackgroundClip(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::BackgroundOrigin => {
                    let old_computed = old_style.get_background().clone_background_origin();
                    let new_computed = new_style.get_background().clone_background_origin();
                    AnimatedProperty::BackgroundOrigin(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::BackgroundSize => {
                    let old_computed = old_style.get_background().clone_background_size();
                    let new_computed = new_style.get_background().clone_background_size();
                    AnimatedProperty::BackgroundSize(
                        old_computed.to_animated_value(),
                        new_computed.to_animated_value(),
                    )
                }
                AnimatableLonghand::BackgroundBlendMode => {
                    let old_computed = old_style.get_background().clone_background_blend_mode();
                    let new_computed = new_style.get_background().clone_background_blend_mode();
                    AnimatedProperty::BackgroundBlendMode(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::BorderTopColor => {
                    let old_computed = old_style.get_border().clone_border_top_color();
                    let new_computed = new_style.get_border().clone_border_top_color();
                    AnimatedProperty::BorderTopColor(
                        old_computed.to_animated_value(),
                        new_computed.to_animated_value(),
                    )
                }
                AnimatableLonghand::BorderTopStyle => {
                    let old_computed = old_style.get_border().clone_border_top_style();
                    let new_computed = new_style.get_border().clone_border_top_style();
                    AnimatedProperty::BorderTopStyle(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::BorderTopWidth => {
                    let old_computed = old_style.get_border().clone_border_top_width();
                    let new_computed = new_style.get_border().clone_border_top_width();
                    AnimatedProperty::BorderTopWidth(
                        old_computed.to_animated_value(),
                        new_computed.to_animated_value(),
                    )
                }
                AnimatableLonghand::BorderLeftColor => {
                    let old_computed = old_style.get_border().clone_border_left_color();
                    let new_computed = new_style.get_border().clone_border_left_color();
                    AnimatedProperty::BorderLeftColor(
                        old_computed.to_animated_value(),
                        new_computed.to_animated_value(),
                    )
                }
                AnimatableLonghand::BorderLeftStyle => {
                    let old_computed = old_style.get_border().clone_border_left_style();
                    let new_computed = new_style.get_border().clone_border_left_style();
                    AnimatedProperty::BorderLeftStyle(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::BorderLeftWidth => {
                    let old_computed = old_style.get_border().clone_border_left_width();
                    let new_computed = new_style.get_border().clone_border_left_width();
                    AnimatedProperty::BorderLeftWidth(
                        old_computed.to_animated_value(),
                        new_computed.to_animated_value(),
                    )
                }
                AnimatableLonghand::BorderBottomColor => {
                    let old_computed = old_style.get_border().clone_border_bottom_color();
                    let new_computed = new_style.get_border().clone_border_bottom_color();
                    AnimatedProperty::BorderBottomColor(
                        old_computed.to_animated_value(),
                        new_computed.to_animated_value(),
                    )
                }
                AnimatableLonghand::BorderBottomStyle => {
                    let old_computed = old_style.get_border().clone_border_bottom_style();
                    let new_computed = new_style.get_border().clone_border_bottom_style();
                    AnimatedProperty::BorderBottomStyle(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::BorderBottomWidth => {
                    let old_computed = old_style.get_border().clone_border_bottom_width();
                    let new_computed = new_style.get_border().clone_border_bottom_width();
                    AnimatedProperty::BorderBottomWidth(
                        old_computed.to_animated_value(),
                        new_computed.to_animated_value(),
                    )
                }
                AnimatableLonghand::BorderRightColor => {
                    let old_computed = old_style.get_border().clone_border_right_color();
                    let new_computed = new_style.get_border().clone_border_right_color();
                    AnimatedProperty::BorderRightColor(
                        old_computed.to_animated_value(),
                        new_computed.to_animated_value(),
                    )
                }
                AnimatableLonghand::BorderRightStyle => {
                    let old_computed = old_style.get_border().clone_border_right_style();
                    let new_computed = new_style.get_border().clone_border_right_style();
                    AnimatedProperty::BorderRightStyle(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::BorderRightWidth => {
                    let old_computed = old_style.get_border().clone_border_right_width();
                    let new_computed = new_style.get_border().clone_border_right_width();
                    AnimatedProperty::BorderRightWidth(
                        old_computed.to_animated_value(),
                        new_computed.to_animated_value(),
                    )
                }
                AnimatableLonghand::BorderTopLeftRadius => {
                    let old_computed = old_style.get_border().clone_border_top_left_radius();
                    let new_computed = new_style.get_border().clone_border_top_left_radius();
                    AnimatedProperty::BorderTopLeftRadius(
                        old_computed.to_animated_value(),
                        new_computed.to_animated_value(),
                    )
                }
                AnimatableLonghand::BorderTopRightRadius => {
                    let old_computed = old_style.get_border().clone_border_top_right_radius();
                    let new_computed = new_style.get_border().clone_border_top_right_radius();
                    AnimatedProperty::BorderTopRightRadius(
                        old_computed.to_animated_value(),
                        new_computed.to_animated_value(),
                    )
                }
                AnimatableLonghand::BorderBottomRightRadius => {
                    let old_computed = old_style.get_border().clone_border_bottom_right_radius();
                    let new_computed = new_style.get_border().clone_border_bottom_right_radius();
                    AnimatedProperty::BorderBottomRightRadius(
                        old_computed.to_animated_value(),
                        new_computed.to_animated_value(),
                    )
                }
                AnimatableLonghand::BorderBottomLeftRadius => {
                    let old_computed = old_style.get_border().clone_border_bottom_left_radius();
                    let new_computed = new_style.get_border().clone_border_bottom_left_radius();
                    AnimatedProperty::BorderBottomLeftRadius(
                        old_computed.to_animated_value(),
                        new_computed.to_animated_value(),
                    )
                }
                AnimatableLonghand::MozBorderTopColors => {
                    let old_computed = old_style.get_border().clone__moz_border_top_colors();
                    let new_computed = new_style.get_border().clone__moz_border_top_colors();
                    AnimatedProperty::MozBorderTopColors(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::MozBorderLeftColors => {
                    let old_computed = old_style.get_border().clone__moz_border_left_colors();
                    let new_computed = new_style.get_border().clone__moz_border_left_colors();
                    AnimatedProperty::MozBorderLeftColors(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::MozBorderBottomColors => {
                    let old_computed = old_style.get_border().clone__moz_border_bottom_colors();
                    let new_computed = new_style.get_border().clone__moz_border_bottom_colors();
                    AnimatedProperty::MozBorderBottomColors(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::MozBorderRightColors => {
                    let old_computed = old_style.get_border().clone__moz_border_right_colors();
                    let new_computed = new_style.get_border().clone__moz_border_right_colors();
                    AnimatedProperty::MozBorderRightColors(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::BoxDecorationBreak => {
                    let old_computed = old_style.get_border().clone_box_decoration_break();
                    let new_computed = new_style.get_border().clone_box_decoration_break();
                    AnimatedProperty::BoxDecorationBreak(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::MozFloatEdge => {
                    let old_computed = old_style.get_border().clone__moz_float_edge();
                    let new_computed = new_style.get_border().clone__moz_float_edge();
                    AnimatedProperty::MozFloatEdge(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::BorderImageSource => {
                    let old_computed = old_style.get_border().clone_border_image_source();
                    let new_computed = new_style.get_border().clone_border_image_source();
                    AnimatedProperty::BorderImageSource(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::BorderImageOutset => {
                    let old_computed = old_style.get_border().clone_border_image_outset();
                    let new_computed = new_style.get_border().clone_border_image_outset();
                    AnimatedProperty::BorderImageOutset(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::BorderImageRepeat => {
                    let old_computed = old_style.get_border().clone_border_image_repeat();
                    let new_computed = new_style.get_border().clone_border_image_repeat();
                    AnimatedProperty::BorderImageRepeat(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::BorderImageWidth => {
                    let old_computed = old_style.get_border().clone_border_image_width();
                    let new_computed = new_style.get_border().clone_border_image_width();
                    AnimatedProperty::BorderImageWidth(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::BorderImageSlice => {
                    let old_computed = old_style.get_border().clone_border_image_slice();
                    let new_computed = new_style.get_border().clone_border_image_slice();
                    AnimatedProperty::BorderImageSlice(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::Display => {
                    let old_computed = old_style.get_box().clone_display();
                    let new_computed = new_style.get_box().clone_display();
                    AnimatedProperty::Display(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::Position => {
                    let old_computed = old_style.get_box().clone_position();
                    let new_computed = new_style.get_box().clone_position();
                    AnimatedProperty::Position(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::Float => {
                    let old_computed = old_style.get_box().clone_float();
                    let new_computed = new_style.get_box().clone_float();
                    AnimatedProperty::Float(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::Clear => {
                    let old_computed = old_style.get_box().clone_clear();
                    let new_computed = new_style.get_box().clone_clear();
                    AnimatedProperty::Clear(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::VerticalAlign => {
                    let old_computed = old_style.get_box().clone_vertical_align();
                    let new_computed = new_style.get_box().clone_vertical_align();
                    AnimatedProperty::VerticalAlign(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::OverflowClipBox => {
                    let old_computed = old_style.get_box().clone_overflow_clip_box();
                    let new_computed = new_style.get_box().clone_overflow_clip_box();
                    AnimatedProperty::OverflowClipBox(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::OverflowX => {
                    let old_computed = old_style.get_box().clone_overflow_x();
                    let new_computed = new_style.get_box().clone_overflow_x();
                    AnimatedProperty::OverflowX(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::OverflowY => {
                    let old_computed = old_style.get_box().clone_overflow_y();
                    let new_computed = new_style.get_box().clone_overflow_y();
                    AnimatedProperty::OverflowY(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::ScrollSnapPointsX => {
                    let old_computed = old_style.get_box().clone_scroll_snap_points_x();
                    let new_computed = new_style.get_box().clone_scroll_snap_points_x();
                    AnimatedProperty::ScrollSnapPointsX(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::ScrollSnapPointsY => {
                    let old_computed = old_style.get_box().clone_scroll_snap_points_y();
                    let new_computed = new_style.get_box().clone_scroll_snap_points_y();
                    AnimatedProperty::ScrollSnapPointsY(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::ScrollSnapDestination => {
                    let old_computed = old_style.get_box().clone_scroll_snap_destination();
                    let new_computed = new_style.get_box().clone_scroll_snap_destination();
                    AnimatedProperty::ScrollSnapDestination(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::ScrollSnapCoordinate => {
                    let old_computed = old_style.get_box().clone_scroll_snap_coordinate();
                    let new_computed = new_style.get_box().clone_scroll_snap_coordinate();
                    AnimatedProperty::ScrollSnapCoordinate(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::Transform => {
                    let old_computed = old_style.get_box().clone_transform();
                    let new_computed = new_style.get_box().clone_transform();
                    AnimatedProperty::Transform(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::ScrollBehavior => {
                    let old_computed = old_style.get_box().clone_scroll_behavior();
                    let new_computed = new_style.get_box().clone_scroll_behavior();
                    AnimatedProperty::ScrollBehavior(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::ScrollSnapTypeX => {
                    let old_computed = old_style.get_box().clone_scroll_snap_type_x();
                    let new_computed = new_style.get_box().clone_scroll_snap_type_x();
                    AnimatedProperty::ScrollSnapTypeX(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::ScrollSnapTypeY => {
                    let old_computed = old_style.get_box().clone_scroll_snap_type_y();
                    let new_computed = new_style.get_box().clone_scroll_snap_type_y();
                    AnimatedProperty::ScrollSnapTypeY(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::Isolation => {
                    let old_computed = old_style.get_box().clone_isolation();
                    let new_computed = new_style.get_box().clone_isolation();
                    AnimatedProperty::Isolation(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::PageBreakAfter => {
                    let old_computed = old_style.get_box().clone_page_break_after();
                    let new_computed = new_style.get_box().clone_page_break_after();
                    AnimatedProperty::PageBreakAfter(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::PageBreakBefore => {
                    let old_computed = old_style.get_box().clone_page_break_before();
                    let new_computed = new_style.get_box().clone_page_break_before();
                    AnimatedProperty::PageBreakBefore(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::PageBreakInside => {
                    let old_computed = old_style.get_box().clone_page_break_inside();
                    let new_computed = new_style.get_box().clone_page_break_inside();
                    AnimatedProperty::PageBreakInside(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::Resize => {
                    let old_computed = old_style.get_box().clone_resize();
                    let new_computed = new_style.get_box().clone_resize();
                    AnimatedProperty::Resize(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::Perspective => {
                    let old_computed = old_style.get_box().clone_perspective();
                    let new_computed = new_style.get_box().clone_perspective();
                    AnimatedProperty::Perspective(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::PerspectiveOrigin => {
                    let old_computed = old_style.get_box().clone_perspective_origin();
                    let new_computed = new_style.get_box().clone_perspective_origin();
                    AnimatedProperty::PerspectiveOrigin(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::BackfaceVisibility => {
                    let old_computed = old_style.get_box().clone_backface_visibility();
                    let new_computed = new_style.get_box().clone_backface_visibility();
                    AnimatedProperty::BackfaceVisibility(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::TransformBox => {
                    let old_computed = old_style.get_box().clone_transform_box();
                    let new_computed = new_style.get_box().clone_transform_box();
                    AnimatedProperty::TransformBox(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::TransformStyle => {
                    let old_computed = old_style.get_box().clone_transform_style();
                    let new_computed = new_style.get_box().clone_transform_style();
                    AnimatedProperty::TransformStyle(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::TransformOrigin => {
                    let old_computed = old_style.get_box().clone_transform_origin();
                    let new_computed = new_style.get_box().clone_transform_origin();
                    AnimatedProperty::TransformOrigin(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::Contain => {
                    let old_computed = old_style.get_box().clone_contain();
                    let new_computed = new_style.get_box().clone_contain();
                    AnimatedProperty::Contain(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::MozAppearance => {
                    let old_computed = old_style.get_box().clone__moz_appearance();
                    let new_computed = new_style.get_box().clone__moz_appearance();
                    AnimatedProperty::MozAppearance(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::MozOrient => {
                    let old_computed = old_style.get_box().clone__moz_orient();
                    let new_computed = new_style.get_box().clone__moz_orient();
                    AnimatedProperty::MozOrient(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::WillChange => {
                    let old_computed = old_style.get_box().clone_will_change();
                    let new_computed = new_style.get_box().clone_will_change();
                    AnimatedProperty::WillChange(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::ShapeOutside => {
                    let old_computed = old_style.get_box().clone_shape_outside();
                    let new_computed = new_style.get_box().clone_shape_outside();
                    AnimatedProperty::ShapeOutside(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::TouchAction => {
                    let old_computed = old_style.get_box().clone_touch_action();
                    let new_computed = new_style.get_box().clone_touch_action();
                    AnimatedProperty::TouchAction(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::Color => {
                    let old_computed = old_style.get_color().clone_color();
                    let new_computed = new_style.get_color().clone_color();
                    AnimatedProperty::Color(
                        old_computed.to_animated_value(),
                        new_computed.to_animated_value(),
                    )
                }
                AnimatableLonghand::ColumnWidth => {
                    let old_computed = old_style.get_column().clone_column_width();
                    let new_computed = new_style.get_column().clone_column_width();
                    AnimatedProperty::ColumnWidth(
                        old_computed.to_animated_value(),
                        new_computed.to_animated_value(),
                    )
                }
                AnimatableLonghand::ColumnCount => {
                    let old_computed = old_style.get_column().clone_column_count();
                    let new_computed = new_style.get_column().clone_column_count();
                    AnimatedProperty::ColumnCount(
                        old_computed.to_animated_value(),
                        new_computed.to_animated_value(),
                    )
                }
                AnimatableLonghand::ColumnGap => {
                    let old_computed = old_style.get_column().clone_column_gap();
                    let new_computed = new_style.get_column().clone_column_gap();
                    AnimatedProperty::ColumnGap(
                        old_computed.to_animated_value(),
                        new_computed.to_animated_value(),
                    )
                }
                AnimatableLonghand::ColumnFill => {
                    let old_computed = old_style.get_column().clone_column_fill();
                    let new_computed = new_style.get_column().clone_column_fill();
                    AnimatedProperty::ColumnFill(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::ColumnRuleWidth => {
                    let old_computed = old_style.get_column().clone_column_rule_width();
                    let new_computed = new_style.get_column().clone_column_rule_width();
                    AnimatedProperty::ColumnRuleWidth(
                        old_computed.to_animated_value(),
                        new_computed.to_animated_value(),
                    )
                }
                AnimatableLonghand::ColumnRuleColor => {
                    let old_computed = old_style.get_column().clone_column_rule_color();
                    let new_computed = new_style.get_column().clone_column_rule_color();
                    AnimatedProperty::ColumnRuleColor(
                        old_computed.to_animated_value(),
                        new_computed.to_animated_value(),
                    )
                }
                AnimatableLonghand::ColumnSpan => {
                    let old_computed = old_style.get_column().clone_column_span();
                    let new_computed = new_style.get_column().clone_column_span();
                    AnimatedProperty::ColumnSpan(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::ColumnRuleStyle => {
                    let old_computed = old_style.get_column().clone_column_rule_style();
                    let new_computed = new_style.get_column().clone_column_rule_style();
                    AnimatedProperty::ColumnRuleStyle(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::Content => {
                    let old_computed = old_style.get_counters().clone_content();
                    let new_computed = new_style.get_counters().clone_content();
                    AnimatedProperty::Content(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::CounterIncrement => {
                    let old_computed = old_style.get_counters().clone_counter_increment();
                    let new_computed = new_style.get_counters().clone_counter_increment();
                    AnimatedProperty::CounterIncrement(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::CounterReset => {
                    let old_computed = old_style.get_counters().clone_counter_reset();
                    let new_computed = new_style.get_counters().clone_counter_reset();
                    AnimatedProperty::CounterReset(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::Opacity => {
                    let old_computed = old_style.get_effects().clone_opacity();
                    let new_computed = new_style.get_effects().clone_opacity();
                    AnimatedProperty::Opacity(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::BoxShadow => {
                    let old_computed = old_style.get_effects().clone_box_shadow();
                    let new_computed = new_style.get_effects().clone_box_shadow();
                    AnimatedProperty::BoxShadow(
                        old_computed.to_animated_value(),
                        new_computed.to_animated_value(),
                    )
                }
                AnimatableLonghand::Clip => {
                    let old_computed = old_style.get_effects().clone_clip();
                    let new_computed = new_style.get_effects().clone_clip();
                    AnimatedProperty::Clip(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::Filter => {
                    let old_computed = old_style.get_effects().clone_filter();
                    let new_computed = new_style.get_effects().clone_filter();
                    AnimatedProperty::Filter(
                        old_computed.to_animated_value(),
                        new_computed.to_animated_value(),
                    )
                }
                AnimatableLonghand::MixBlendMode => {
                    let old_computed = old_style.get_effects().clone_mix_blend_mode();
                    let new_computed = new_style.get_effects().clone_mix_blend_mode();
                    AnimatedProperty::MixBlendMode(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::FontFamily => {
                    let old_computed = old_style.get_font().clone_font_family();
                    let new_computed = new_style.get_font().clone_font_family();
                    AnimatedProperty::FontFamily(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::FontStyle => {
                    let old_computed = old_style.get_font().clone_font_style();
                    let new_computed = new_style.get_font().clone_font_style();
                    AnimatedProperty::FontStyle(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::FontVariantCaps => {
                    let old_computed = old_style.get_font().clone_font_variant_caps();
                    let new_computed = new_style.get_font().clone_font_variant_caps();
                    AnimatedProperty::FontVariantCaps(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::FontWeight => {
                    let old_computed = old_style.get_font().clone_font_weight();
                    let new_computed = new_style.get_font().clone_font_weight();
                    AnimatedProperty::FontWeight(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::FontSize => {
                    let old_computed = old_style.get_font().clone_font_size();
                    let new_computed = new_style.get_font().clone_font_size();
                    AnimatedProperty::FontSize(
                        old_computed.to_animated_value(),
                        new_computed.to_animated_value(),
                    )
                }
                AnimatableLonghand::FontSizeAdjust => {
                    let old_computed = old_style.get_font().clone_font_size_adjust();
                    let new_computed = new_style.get_font().clone_font_size_adjust();
                    AnimatedProperty::FontSizeAdjust(
                        old_computed.to_animated_value(),
                        new_computed.to_animated_value(),
                    )
                }
                AnimatableLonghand::FontSynthesis => {
                    let old_computed = old_style.get_font().clone_font_synthesis();
                    let new_computed = new_style.get_font().clone_font_synthesis();
                    AnimatedProperty::FontSynthesis(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::FontStretch => {
                    let old_computed = old_style.get_font().clone_font_stretch();
                    let new_computed = new_style.get_font().clone_font_stretch();
                    AnimatedProperty::FontStretch(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::FontKerning => {
                    let old_computed = old_style.get_font().clone_font_kerning();
                    let new_computed = new_style.get_font().clone_font_kerning();
                    AnimatedProperty::FontKerning(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::FontVariantAlternates => {
                    let old_computed = old_style.get_font().clone_font_variant_alternates();
                    let new_computed = new_style.get_font().clone_font_variant_alternates();
                    AnimatedProperty::FontVariantAlternates(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::FontVariantEastAsian => {
                    let old_computed = old_style.get_font().clone_font_variant_east_asian();
                    let new_computed = new_style.get_font().clone_font_variant_east_asian();
                    AnimatedProperty::FontVariantEastAsian(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::FontVariantLigatures => {
                    let old_computed = old_style.get_font().clone_font_variant_ligatures();
                    let new_computed = new_style.get_font().clone_font_variant_ligatures();
                    AnimatedProperty::FontVariantLigatures(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::FontVariantNumeric => {
                    let old_computed = old_style.get_font().clone_font_variant_numeric();
                    let new_computed = new_style.get_font().clone_font_variant_numeric();
                    AnimatedProperty::FontVariantNumeric(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::FontVariantPosition => {
                    let old_computed = old_style.get_font().clone_font_variant_position();
                    let new_computed = new_style.get_font().clone_font_variant_position();
                    AnimatedProperty::FontVariantPosition(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::FontFeatureSettings => {
                    let old_computed = old_style.get_font().clone_font_feature_settings();
                    let new_computed = new_style.get_font().clone_font_feature_settings();
                    AnimatedProperty::FontFeatureSettings(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::FontVariationSettings => {
                    let old_computed = old_style.get_font().clone_font_variation_settings();
                    let new_computed = new_style.get_font().clone_font_variation_settings();
                    AnimatedProperty::FontVariationSettings(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::FontLanguageOverride => {
                    let old_computed = old_style.get_font().clone_font_language_override();
                    let new_computed = new_style.get_font().clone_font_language_override();
                    AnimatedProperty::FontLanguageOverride(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::MozOsxFontSmoothing => {
                    let old_computed = old_style.get_font().clone__moz_osx_font_smoothing();
                    let new_computed = new_style.get_font().clone__moz_osx_font_smoothing();
                    AnimatedProperty::MozOsxFontSmoothing(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::Visibility => {
                    let old_computed = old_style.get_inheritedbox().clone_visibility();
                    let new_computed = new_style.get_inheritedbox().clone_visibility();
                    AnimatedProperty::Visibility(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::WritingMode => {
                    let old_computed = old_style.get_inheritedbox().clone_writing_mode();
                    let new_computed = new_style.get_inheritedbox().clone_writing_mode();
                    AnimatedProperty::WritingMode(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::Direction => {
                    let old_computed = old_style.get_inheritedbox().clone_direction();
                    let new_computed = new_style.get_inheritedbox().clone_direction();
                    AnimatedProperty::Direction(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::TextOrientation => {
                    let old_computed = old_style.get_inheritedbox().clone_text_orientation();
                    let new_computed = new_style.get_inheritedbox().clone_text_orientation();
                    AnimatedProperty::TextOrientation(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::ColorAdjust => {
                    let old_computed = old_style.get_inheritedbox().clone_color_adjust();
                    let new_computed = new_style.get_inheritedbox().clone_color_adjust();
                    AnimatedProperty::ColorAdjust(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::ImageRendering => {
                    let old_computed = old_style.get_inheritedbox().clone_image_rendering();
                    let new_computed = new_style.get_inheritedbox().clone_image_rendering();
                    AnimatedProperty::ImageRendering(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::ImageOrientation => {
                    let old_computed = old_style.get_inheritedbox().clone_image_orientation();
                    let new_computed = new_style.get_inheritedbox().clone_image_orientation();
                    AnimatedProperty::ImageOrientation(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::BorderCollapse => {
                    let old_computed = old_style.get_inheritedtable().clone_border_collapse();
                    let new_computed = new_style.get_inheritedtable().clone_border_collapse();
                    AnimatedProperty::BorderCollapse(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::EmptyCells => {
                    let old_computed = old_style.get_inheritedtable().clone_empty_cells();
                    let new_computed = new_style.get_inheritedtable().clone_empty_cells();
                    AnimatedProperty::EmptyCells(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::CaptionSide => {
                    let old_computed = old_style.get_inheritedtable().clone_caption_side();
                    let new_computed = new_style.get_inheritedtable().clone_caption_side();
                    AnimatedProperty::CaptionSide(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::BorderSpacing => {
                    let old_computed = old_style.get_inheritedtable().clone_border_spacing();
                    let new_computed = new_style.get_inheritedtable().clone_border_spacing();
                    AnimatedProperty::BorderSpacing(
                        old_computed.to_animated_value(),
                        new_computed.to_animated_value(),
                    )
                }
                AnimatableLonghand::LineHeight => {
                    let old_computed = old_style.get_inheritedtext().clone_line_height();
                    let new_computed = new_style.get_inheritedtext().clone_line_height();
                    AnimatedProperty::LineHeight(
                        old_computed.to_animated_value(),
                        new_computed.to_animated_value(),
                    )
                }
                AnimatableLonghand::TextTransform => {
                    let old_computed = old_style.get_inheritedtext().clone_text_transform();
                    let new_computed = new_style.get_inheritedtext().clone_text_transform();
                    AnimatedProperty::TextTransform(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::Hyphens => {
                    let old_computed = old_style.get_inheritedtext().clone_hyphens();
                    let new_computed = new_style.get_inheritedtext().clone_hyphens();
                    AnimatedProperty::Hyphens(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::MozTextSizeAdjust => {
                    let old_computed = old_style.get_inheritedtext().clone__moz_text_size_adjust();
                    let new_computed = new_style.get_inheritedtext().clone__moz_text_size_adjust();
                    AnimatedProperty::MozTextSizeAdjust(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::TextIndent => {
                    let old_computed = old_style.get_inheritedtext().clone_text_indent();
                    let new_computed = new_style.get_inheritedtext().clone_text_indent();
                    AnimatedProperty::TextIndent(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::OverflowWrap => {
                    let old_computed = old_style.get_inheritedtext().clone_overflow_wrap();
                    let new_computed = new_style.get_inheritedtext().clone_overflow_wrap();
                    AnimatedProperty::OverflowWrap(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::WordBreak => {
                    let old_computed = old_style.get_inheritedtext().clone_word_break();
                    let new_computed = new_style.get_inheritedtext().clone_word_break();
                    AnimatedProperty::WordBreak(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::TextJustify => {
                    let old_computed = old_style.get_inheritedtext().clone_text_justify();
                    let new_computed = new_style.get_inheritedtext().clone_text_justify();
                    AnimatedProperty::TextJustify(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::TextAlignLast => {
                    let old_computed = old_style.get_inheritedtext().clone_text_align_last();
                    let new_computed = new_style.get_inheritedtext().clone_text_align_last();
                    AnimatedProperty::TextAlignLast(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::TextAlign => {
                    let old_computed = old_style.get_inheritedtext().clone_text_align();
                    let new_computed = new_style.get_inheritedtext().clone_text_align();
                    AnimatedProperty::TextAlign(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::LetterSpacing => {
                    let old_computed = old_style.get_inheritedtext().clone_letter_spacing();
                    let new_computed = new_style.get_inheritedtext().clone_letter_spacing();
                    AnimatedProperty::LetterSpacing(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::WordSpacing => {
                    let old_computed = old_style.get_inheritedtext().clone_word_spacing();
                    let new_computed = new_style.get_inheritedtext().clone_word_spacing();
                    AnimatedProperty::WordSpacing(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::WhiteSpace => {
                    let old_computed = old_style.get_inheritedtext().clone_white_space();
                    let new_computed = new_style.get_inheritedtext().clone_white_space();
                    AnimatedProperty::WhiteSpace(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::TextShadow => {
                    let old_computed = old_style.get_inheritedtext().clone_text_shadow();
                    let new_computed = new_style.get_inheritedtext().clone_text_shadow();
                    AnimatedProperty::TextShadow(
                        old_computed.to_animated_value(),
                        new_computed.to_animated_value(),
                    )
                }
                AnimatableLonghand::TextEmphasisStyle => {
                    let old_computed = old_style.get_inheritedtext().clone_text_emphasis_style();
                    let new_computed = new_style.get_inheritedtext().clone_text_emphasis_style();
                    AnimatedProperty::TextEmphasisStyle(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::TextEmphasisPosition => {
                    let old_computed = old_style.get_inheritedtext().clone_text_emphasis_position();
                    let new_computed = new_style.get_inheritedtext().clone_text_emphasis_position();
                    AnimatedProperty::TextEmphasisPosition(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::TextEmphasisColor => {
                    let old_computed = old_style.get_inheritedtext().clone_text_emphasis_color();
                    let new_computed = new_style.get_inheritedtext().clone_text_emphasis_color();
                    AnimatedProperty::TextEmphasisColor(
                        old_computed.to_animated_value(),
                        new_computed.to_animated_value(),
                    )
                }
                AnimatableLonghand::MozTabSize => {
                    let old_computed = old_style.get_inheritedtext().clone__moz_tab_size();
                    let new_computed = new_style.get_inheritedtext().clone__moz_tab_size();
                    AnimatedProperty::MozTabSize(
                        old_computed.to_animated_value(),
                        new_computed.to_animated_value(),
                    )
                }
                AnimatableLonghand::WebkitTextFillColor => {
                    let old_computed = old_style.get_inheritedtext().clone__webkit_text_fill_color();
                    let new_computed = new_style.get_inheritedtext().clone__webkit_text_fill_color();
                    AnimatedProperty::WebkitTextFillColor(
                        old_computed.to_animated_value(),
                        new_computed.to_animated_value(),
                    )
                }
                AnimatableLonghand::WebkitTextStrokeColor => {
                    let old_computed = old_style.get_inheritedtext().clone__webkit_text_stroke_color();
                    let new_computed = new_style.get_inheritedtext().clone__webkit_text_stroke_color();
                    AnimatedProperty::WebkitTextStrokeColor(
                        old_computed.to_animated_value(),
                        new_computed.to_animated_value(),
                    )
                }
                AnimatableLonghand::WebkitTextStrokeWidth => {
                    let old_computed = old_style.get_inheritedtext().clone__webkit_text_stroke_width();
                    let new_computed = new_style.get_inheritedtext().clone__webkit_text_stroke_width();
                    AnimatedProperty::WebkitTextStrokeWidth(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::RubyAlign => {
                    let old_computed = old_style.get_inheritedtext().clone_ruby_align();
                    let new_computed = new_style.get_inheritedtext().clone_ruby_align();
                    AnimatedProperty::RubyAlign(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::RubyPosition => {
                    let old_computed = old_style.get_inheritedtext().clone_ruby_position();
                    let new_computed = new_style.get_inheritedtext().clone_ruby_position();
                    AnimatedProperty::RubyPosition(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::TextCombineUpright => {
                    let old_computed = old_style.get_inheritedtext().clone_text_combine_upright();
                    let new_computed = new_style.get_inheritedtext().clone_text_combine_upright();
                    AnimatedProperty::TextCombineUpright(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::TextRendering => {
                    let old_computed = old_style.get_inheritedtext().clone_text_rendering();
                    let new_computed = new_style.get_inheritedtext().clone_text_rendering();
                    AnimatedProperty::TextRendering(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::ListStylePosition => {
                    let old_computed = old_style.get_list().clone_list_style_position();
                    let new_computed = new_style.get_list().clone_list_style_position();
                    AnimatedProperty::ListStylePosition(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::ListStyleType => {
                    let old_computed = old_style.get_list().clone_list_style_type();
                    let new_computed = new_style.get_list().clone_list_style_type();
                    AnimatedProperty::ListStyleType(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::ListStyleImage => {
                    let old_computed = old_style.get_list().clone_list_style_image();
                    let new_computed = new_style.get_list().clone_list_style_image();
                    AnimatedProperty::ListStyleImage(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::Quotes => {
                    let old_computed = old_style.get_list().clone_quotes();
                    let new_computed = new_style.get_list().clone_quotes();
                    AnimatedProperty::Quotes(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::MozImageRegion => {
                    let old_computed = old_style.get_list().clone__moz_image_region();
                    let new_computed = new_style.get_list().clone__moz_image_region();
                    AnimatedProperty::MozImageRegion(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::MarginTop => {
                    let old_computed = old_style.get_margin().clone_margin_top();
                    let new_computed = new_style.get_margin().clone_margin_top();
                    AnimatedProperty::MarginTop(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::MarginLeft => {
                    let old_computed = old_style.get_margin().clone_margin_left();
                    let new_computed = new_style.get_margin().clone_margin_left();
                    AnimatedProperty::MarginLeft(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::MarginBottom => {
                    let old_computed = old_style.get_margin().clone_margin_bottom();
                    let new_computed = new_style.get_margin().clone_margin_bottom();
                    AnimatedProperty::MarginBottom(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::MarginRight => {
                    let old_computed = old_style.get_margin().clone_margin_right();
                    let new_computed = new_style.get_margin().clone_margin_right();
                    AnimatedProperty::MarginRight(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::OutlineColor => {
                    let old_computed = old_style.get_outline().clone_outline_color();
                    let new_computed = new_style.get_outline().clone_outline_color();
                    AnimatedProperty::OutlineColor(
                        old_computed.to_animated_value(),
                        new_computed.to_animated_value(),
                    )
                }
                AnimatableLonghand::OutlineStyle => {
                    let old_computed = old_style.get_outline().clone_outline_style();
                    let new_computed = new_style.get_outline().clone_outline_style();
                    AnimatedProperty::OutlineStyle(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::OutlineWidth => {
                    let old_computed = old_style.get_outline().clone_outline_width();
                    let new_computed = new_style.get_outline().clone_outline_width();
                    AnimatedProperty::OutlineWidth(
                        old_computed.to_animated_value(),
                        new_computed.to_animated_value(),
                    )
                }
                AnimatableLonghand::MozOutlineRadiusTopleft => {
                    let old_computed = old_style.get_outline().clone__moz_outline_radius_topleft();
                    let new_computed = new_style.get_outline().clone__moz_outline_radius_topleft();
                    AnimatedProperty::MozOutlineRadiusTopleft(
                        old_computed.to_animated_value(),
                        new_computed.to_animated_value(),
                    )
                }
                AnimatableLonghand::MozOutlineRadiusTopright => {
                    let old_computed = old_style.get_outline().clone__moz_outline_radius_topright();
                    let new_computed = new_style.get_outline().clone__moz_outline_radius_topright();
                    AnimatedProperty::MozOutlineRadiusTopright(
                        old_computed.to_animated_value(),
                        new_computed.to_animated_value(),
                    )
                }
                AnimatableLonghand::MozOutlineRadiusBottomright => {
                    let old_computed = old_style.get_outline().clone__moz_outline_radius_bottomright();
                    let new_computed = new_style.get_outline().clone__moz_outline_radius_bottomright();
                    AnimatedProperty::MozOutlineRadiusBottomright(
                        old_computed.to_animated_value(),
                        new_computed.to_animated_value(),
                    )
                }
                AnimatableLonghand::MozOutlineRadiusBottomleft => {
                    let old_computed = old_style.get_outline().clone__moz_outline_radius_bottomleft();
                    let new_computed = new_style.get_outline().clone__moz_outline_radius_bottomleft();
                    AnimatedProperty::MozOutlineRadiusBottomleft(
                        old_computed.to_animated_value(),
                        new_computed.to_animated_value(),
                    )
                }
                AnimatableLonghand::OutlineOffset => {
                    let old_computed = old_style.get_outline().clone_outline_offset();
                    let new_computed = new_style.get_outline().clone_outline_offset();
                    AnimatedProperty::OutlineOffset(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::PaddingTop => {
                    let old_computed = old_style.get_padding().clone_padding_top();
                    let new_computed = new_style.get_padding().clone_padding_top();
                    AnimatedProperty::PaddingTop(
                        old_computed.to_animated_value(),
                        new_computed.to_animated_value(),
                    )
                }
                AnimatableLonghand::PaddingLeft => {
                    let old_computed = old_style.get_padding().clone_padding_left();
                    let new_computed = new_style.get_padding().clone_padding_left();
                    AnimatedProperty::PaddingLeft(
                        old_computed.to_animated_value(),
                        new_computed.to_animated_value(),
                    )
                }
                AnimatableLonghand::PaddingBottom => {
                    let old_computed = old_style.get_padding().clone_padding_bottom();
                    let new_computed = new_style.get_padding().clone_padding_bottom();
                    AnimatedProperty::PaddingBottom(
                        old_computed.to_animated_value(),
                        new_computed.to_animated_value(),
                    )
                }
                AnimatableLonghand::PaddingRight => {
                    let old_computed = old_style.get_padding().clone_padding_right();
                    let new_computed = new_style.get_padding().clone_padding_right();
                    AnimatedProperty::PaddingRight(
                        old_computed.to_animated_value(),
                        new_computed.to_animated_value(),
                    )
                }
                AnimatableLonghand::Cursor => {
                    let old_computed = old_style.get_pointing().clone_cursor();
                    let new_computed = new_style.get_pointing().clone_cursor();
                    AnimatedProperty::Cursor(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::PointerEvents => {
                    let old_computed = old_style.get_pointing().clone_pointer_events();
                    let new_computed = new_style.get_pointing().clone_pointer_events();
                    AnimatedProperty::PointerEvents(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::MozUserInput => {
                    let old_computed = old_style.get_pointing().clone__moz_user_input();
                    let new_computed = new_style.get_pointing().clone__moz_user_input();
                    AnimatedProperty::MozUserInput(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::MozUserModify => {
                    let old_computed = old_style.get_pointing().clone__moz_user_modify();
                    let new_computed = new_style.get_pointing().clone__moz_user_modify();
                    AnimatedProperty::MozUserModify(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::MozUserFocus => {
                    let old_computed = old_style.get_pointing().clone__moz_user_focus();
                    let new_computed = new_style.get_pointing().clone__moz_user_focus();
                    AnimatedProperty::MozUserFocus(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::CaretColor => {
                    let old_computed = old_style.get_pointing().clone_caret_color();
                    let new_computed = new_style.get_pointing().clone_caret_color();
                    AnimatedProperty::CaretColor(
                        old_computed.to_animated_value(),
                        new_computed.to_animated_value(),
                    )
                }
                AnimatableLonghand::Top => {
                    let old_computed = old_style.get_position().clone_top();
                    let new_computed = new_style.get_position().clone_top();
                    AnimatedProperty::Top(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::Left => {
                    let old_computed = old_style.get_position().clone_left();
                    let new_computed = new_style.get_position().clone_left();
                    AnimatedProperty::Left(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::Bottom => {
                    let old_computed = old_style.get_position().clone_bottom();
                    let new_computed = new_style.get_position().clone_bottom();
                    AnimatedProperty::Bottom(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::Right => {
                    let old_computed = old_style.get_position().clone_right();
                    let new_computed = new_style.get_position().clone_right();
                    AnimatedProperty::Right(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::ZIndex => {
                    let old_computed = old_style.get_position().clone_z_index();
                    let new_computed = new_style.get_position().clone_z_index();
                    AnimatedProperty::ZIndex(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::FlexDirection => {
                    let old_computed = old_style.get_position().clone_flex_direction();
                    let new_computed = new_style.get_position().clone_flex_direction();
                    AnimatedProperty::FlexDirection(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::FlexWrap => {
                    let old_computed = old_style.get_position().clone_flex_wrap();
                    let new_computed = new_style.get_position().clone_flex_wrap();
                    AnimatedProperty::FlexWrap(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::JustifyContent => {
                    let old_computed = old_style.get_position().clone_justify_content();
                    let new_computed = new_style.get_position().clone_justify_content();
                    AnimatedProperty::JustifyContent(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::AlignContent => {
                    let old_computed = old_style.get_position().clone_align_content();
                    let new_computed = new_style.get_position().clone_align_content();
                    AnimatedProperty::AlignContent(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::AlignItems => {
                    let old_computed = old_style.get_position().clone_align_items();
                    let new_computed = new_style.get_position().clone_align_items();
                    AnimatedProperty::AlignItems(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::JustifyItems => {
                    let old_computed = old_style.get_position().clone_justify_items();
                    let new_computed = new_style.get_position().clone_justify_items();
                    AnimatedProperty::JustifyItems(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::FlexGrow => {
                    let old_computed = old_style.get_position().clone_flex_grow();
                    let new_computed = new_style.get_position().clone_flex_grow();
                    AnimatedProperty::FlexGrow(
                        old_computed.to_animated_value(),
                        new_computed.to_animated_value(),
                    )
                }
                AnimatableLonghand::FlexShrink => {
                    let old_computed = old_style.get_position().clone_flex_shrink();
                    let new_computed = new_style.get_position().clone_flex_shrink();
                    AnimatedProperty::FlexShrink(
                        old_computed.to_animated_value(),
                        new_computed.to_animated_value(),
                    )
                }
                AnimatableLonghand::AlignSelf => {
                    let old_computed = old_style.get_position().clone_align_self();
                    let new_computed = new_style.get_position().clone_align_self();
                    AnimatedProperty::AlignSelf(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::JustifySelf => {
                    let old_computed = old_style.get_position().clone_justify_self();
                    let new_computed = new_style.get_position().clone_justify_self();
                    AnimatedProperty::JustifySelf(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::Order => {
                    let old_computed = old_style.get_position().clone_order();
                    let new_computed = new_style.get_position().clone_order();
                    AnimatedProperty::Order(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::FlexBasis => {
                    let old_computed = old_style.get_position().clone_flex_basis();
                    let new_computed = new_style.get_position().clone_flex_basis();
                    AnimatedProperty::FlexBasis(
                        old_computed.to_animated_value(),
                        new_computed.to_animated_value(),
                    )
                }
                AnimatableLonghand::Width => {
                    let old_computed = old_style.get_position().clone_width();
                    let new_computed = new_style.get_position().clone_width();
                    AnimatedProperty::Width(
                        old_computed.to_animated_value(),
                        new_computed.to_animated_value(),
                    )
                }
                AnimatableLonghand::MinWidth => {
                    let old_computed = old_style.get_position().clone_min_width();
                    let new_computed = new_style.get_position().clone_min_width();
                    AnimatedProperty::MinWidth(
                        old_computed.to_animated_value(),
                        new_computed.to_animated_value(),
                    )
                }
                AnimatableLonghand::MaxWidth => {
                    let old_computed = old_style.get_position().clone_max_width();
                    let new_computed = new_style.get_position().clone_max_width();
                    AnimatedProperty::MaxWidth(
                        old_computed.to_animated_value(),
                        new_computed.to_animated_value(),
                    )
                }
                AnimatableLonghand::Height => {
                    let old_computed = old_style.get_position().clone_height();
                    let new_computed = new_style.get_position().clone_height();
                    AnimatedProperty::Height(
                        old_computed.to_animated_value(),
                        new_computed.to_animated_value(),
                    )
                }
                AnimatableLonghand::MinHeight => {
                    let old_computed = old_style.get_position().clone_min_height();
                    let new_computed = new_style.get_position().clone_min_height();
                    AnimatedProperty::MinHeight(
                        old_computed.to_animated_value(),
                        new_computed.to_animated_value(),
                    )
                }
                AnimatableLonghand::MaxHeight => {
                    let old_computed = old_style.get_position().clone_max_height();
                    let new_computed = new_style.get_position().clone_max_height();
                    AnimatedProperty::MaxHeight(
                        old_computed.to_animated_value(),
                        new_computed.to_animated_value(),
                    )
                }
                AnimatableLonghand::BoxSizing => {
                    let old_computed = old_style.get_position().clone_box_sizing();
                    let new_computed = new_style.get_position().clone_box_sizing();
                    AnimatedProperty::BoxSizing(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::ObjectFit => {
                    let old_computed = old_style.get_position().clone_object_fit();
                    let new_computed = new_style.get_position().clone_object_fit();
                    AnimatedProperty::ObjectFit(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::ObjectPosition => {
                    let old_computed = old_style.get_position().clone_object_position();
                    let new_computed = new_style.get_position().clone_object_position();
                    AnimatedProperty::ObjectPosition(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::GridRowGap => {
                    let old_computed = old_style.get_position().clone_grid_row_gap();
                    let new_computed = new_style.get_position().clone_grid_row_gap();
                    AnimatedProperty::GridRowGap(
                        old_computed.to_animated_value(),
                        new_computed.to_animated_value(),
                    )
                }
                AnimatableLonghand::GridRowStart => {
                    let old_computed = old_style.get_position().clone_grid_row_start();
                    let new_computed = new_style.get_position().clone_grid_row_start();
                    AnimatedProperty::GridRowStart(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::GridRowEnd => {
                    let old_computed = old_style.get_position().clone_grid_row_end();
                    let new_computed = new_style.get_position().clone_grid_row_end();
                    AnimatedProperty::GridRowEnd(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::GridAutoRows => {
                    let old_computed = old_style.get_position().clone_grid_auto_rows();
                    let new_computed = new_style.get_position().clone_grid_auto_rows();
                    AnimatedProperty::GridAutoRows(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::GridTemplateRows => {
                    let old_computed = old_style.get_position().clone_grid_template_rows();
                    let new_computed = new_style.get_position().clone_grid_template_rows();
                    AnimatedProperty::GridTemplateRows(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::GridColumnGap => {
                    let old_computed = old_style.get_position().clone_grid_column_gap();
                    let new_computed = new_style.get_position().clone_grid_column_gap();
                    AnimatedProperty::GridColumnGap(
                        old_computed.to_animated_value(),
                        new_computed.to_animated_value(),
                    )
                }
                AnimatableLonghand::GridColumnStart => {
                    let old_computed = old_style.get_position().clone_grid_column_start();
                    let new_computed = new_style.get_position().clone_grid_column_start();
                    AnimatedProperty::GridColumnStart(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::GridColumnEnd => {
                    let old_computed = old_style.get_position().clone_grid_column_end();
                    let new_computed = new_style.get_position().clone_grid_column_end();
                    AnimatedProperty::GridColumnEnd(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::GridAutoColumns => {
                    let old_computed = old_style.get_position().clone_grid_auto_columns();
                    let new_computed = new_style.get_position().clone_grid_auto_columns();
                    AnimatedProperty::GridAutoColumns(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::GridTemplateColumns => {
                    let old_computed = old_style.get_position().clone_grid_template_columns();
                    let new_computed = new_style.get_position().clone_grid_template_columns();
                    AnimatedProperty::GridTemplateColumns(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::GridAutoFlow => {
                    let old_computed = old_style.get_position().clone_grid_auto_flow();
                    let new_computed = new_style.get_position().clone_grid_auto_flow();
                    AnimatedProperty::GridAutoFlow(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::GridTemplateAreas => {
                    let old_computed = old_style.get_position().clone_grid_template_areas();
                    let new_computed = new_style.get_position().clone_grid_template_areas();
                    AnimatedProperty::GridTemplateAreas(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::TableLayout => {
                    let old_computed = old_style.get_table().clone_table_layout();
                    let new_computed = new_style.get_table().clone_table_layout();
                    AnimatedProperty::TableLayout(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::TextOverflow => {
                    let old_computed = old_style.get_text().clone_text_overflow();
                    let new_computed = new_style.get_text().clone_text_overflow();
                    AnimatedProperty::TextOverflow(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::UnicodeBidi => {
                    let old_computed = old_style.get_text().clone_unicode_bidi();
                    let new_computed = new_style.get_text().clone_unicode_bidi();
                    AnimatedProperty::UnicodeBidi(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::TextDecorationLine => {
                    let old_computed = old_style.get_text().clone_text_decoration_line();
                    let new_computed = new_style.get_text().clone_text_decoration_line();
                    AnimatedProperty::TextDecorationLine(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::TextDecorationStyle => {
                    let old_computed = old_style.get_text().clone_text_decoration_style();
                    let new_computed = new_style.get_text().clone_text_decoration_style();
                    AnimatedProperty::TextDecorationStyle(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::TextDecorationColor => {
                    let old_computed = old_style.get_text().clone_text_decoration_color();
                    let new_computed = new_style.get_text().clone_text_decoration_color();
                    AnimatedProperty::TextDecorationColor(
                        old_computed.to_animated_value(),
                        new_computed.to_animated_value(),
                    )
                }
                AnimatableLonghand::InitialLetter => {
                    let old_computed = old_style.get_text().clone_initial_letter();
                    let new_computed = new_style.get_text().clone_initial_letter();
                    AnimatedProperty::InitialLetter(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::ImeMode => {
                    let old_computed = old_style.get_ui().clone_ime_mode();
                    let new_computed = new_style.get_ui().clone_ime_mode();
                    AnimatedProperty::ImeMode(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::MozUserSelect => {
                    let old_computed = old_style.get_ui().clone__moz_user_select();
                    let new_computed = new_style.get_ui().clone__moz_user_select();
                    AnimatedProperty::MozUserSelect(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::MozWindowDragging => {
                    let old_computed = old_style.get_ui().clone__moz_window_dragging();
                    let new_computed = new_style.get_ui().clone__moz_window_dragging();
                    AnimatedProperty::MozWindowDragging(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::MozWindowShadow => {
                    let old_computed = old_style.get_ui().clone__moz_window_shadow();
                    let new_computed = new_style.get_ui().clone__moz_window_shadow();
                    AnimatedProperty::MozWindowShadow(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::MozForceBrokenImageIcon => {
                    let old_computed = old_style.get_ui().clone__moz_force_broken_image_icon();
                    let new_computed = new_style.get_ui().clone__moz_force_broken_image_icon();
                    AnimatedProperty::MozForceBrokenImageIcon(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::TextAnchor => {
                    let old_computed = old_style.get_inheritedsvg().clone_text_anchor();
                    let new_computed = new_style.get_inheritedsvg().clone_text_anchor();
                    AnimatedProperty::TextAnchor(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::ColorInterpolation => {
                    let old_computed = old_style.get_inheritedsvg().clone_color_interpolation();
                    let new_computed = new_style.get_inheritedsvg().clone_color_interpolation();
                    AnimatedProperty::ColorInterpolation(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::ColorInterpolationFilters => {
                    let old_computed = old_style.get_inheritedsvg().clone_color_interpolation_filters();
                    let new_computed = new_style.get_inheritedsvg().clone_color_interpolation_filters();
                    AnimatedProperty::ColorInterpolationFilters(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::Fill => {
                    let old_computed = old_style.get_inheritedsvg().clone_fill();
                    let new_computed = new_style.get_inheritedsvg().clone_fill();
                    AnimatedProperty::Fill(
                        old_computed.to_animated_value(),
                        new_computed.to_animated_value(),
                    )
                }
                AnimatableLonghand::FillOpacity => {
                    let old_computed = old_style.get_inheritedsvg().clone_fill_opacity();
                    let new_computed = new_style.get_inheritedsvg().clone_fill_opacity();
                    AnimatedProperty::FillOpacity(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::FillRule => {
                    let old_computed = old_style.get_inheritedsvg().clone_fill_rule();
                    let new_computed = new_style.get_inheritedsvg().clone_fill_rule();
                    AnimatedProperty::FillRule(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::ShapeRendering => {
                    let old_computed = old_style.get_inheritedsvg().clone_shape_rendering();
                    let new_computed = new_style.get_inheritedsvg().clone_shape_rendering();
                    AnimatedProperty::ShapeRendering(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::Stroke => {
                    let old_computed = old_style.get_inheritedsvg().clone_stroke();
                    let new_computed = new_style.get_inheritedsvg().clone_stroke();
                    AnimatedProperty::Stroke(
                        old_computed.to_animated_value(),
                        new_computed.to_animated_value(),
                    )
                }
                AnimatableLonghand::StrokeWidth => {
                    let old_computed = old_style.get_inheritedsvg().clone_stroke_width();
                    let new_computed = new_style.get_inheritedsvg().clone_stroke_width();
                    AnimatedProperty::StrokeWidth(
                        old_computed.to_animated_value(),
                        new_computed.to_animated_value(),
                    )
                }
                AnimatableLonghand::StrokeLinecap => {
                    let old_computed = old_style.get_inheritedsvg().clone_stroke_linecap();
                    let new_computed = new_style.get_inheritedsvg().clone_stroke_linecap();
                    AnimatedProperty::StrokeLinecap(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::StrokeLinejoin => {
                    let old_computed = old_style.get_inheritedsvg().clone_stroke_linejoin();
                    let new_computed = new_style.get_inheritedsvg().clone_stroke_linejoin();
                    AnimatedProperty::StrokeLinejoin(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::StrokeMiterlimit => {
                    let old_computed = old_style.get_inheritedsvg().clone_stroke_miterlimit();
                    let new_computed = new_style.get_inheritedsvg().clone_stroke_miterlimit();
                    AnimatedProperty::StrokeMiterlimit(
                        old_computed.to_animated_value(),
                        new_computed.to_animated_value(),
                    )
                }
                AnimatableLonghand::StrokeOpacity => {
                    let old_computed = old_style.get_inheritedsvg().clone_stroke_opacity();
                    let new_computed = new_style.get_inheritedsvg().clone_stroke_opacity();
                    AnimatedProperty::StrokeOpacity(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::StrokeDasharray => {
                    let old_computed = old_style.get_inheritedsvg().clone_stroke_dasharray();
                    let new_computed = new_style.get_inheritedsvg().clone_stroke_dasharray();
                    AnimatedProperty::StrokeDasharray(
                        old_computed.to_animated_value(),
                        new_computed.to_animated_value(),
                    )
                }
                AnimatableLonghand::StrokeDashoffset => {
                    let old_computed = old_style.get_inheritedsvg().clone_stroke_dashoffset();
                    let new_computed = new_style.get_inheritedsvg().clone_stroke_dashoffset();
                    AnimatedProperty::StrokeDashoffset(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::ClipRule => {
                    let old_computed = old_style.get_inheritedsvg().clone_clip_rule();
                    let new_computed = new_style.get_inheritedsvg().clone_clip_rule();
                    AnimatedProperty::ClipRule(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::MarkerStart => {
                    let old_computed = old_style.get_inheritedsvg().clone_marker_start();
                    let new_computed = new_style.get_inheritedsvg().clone_marker_start();
                    AnimatedProperty::MarkerStart(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::MarkerMid => {
                    let old_computed = old_style.get_inheritedsvg().clone_marker_mid();
                    let new_computed = new_style.get_inheritedsvg().clone_marker_mid();
                    AnimatedProperty::MarkerMid(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::MarkerEnd => {
                    let old_computed = old_style.get_inheritedsvg().clone_marker_end();
                    let new_computed = new_style.get_inheritedsvg().clone_marker_end();
                    AnimatedProperty::MarkerEnd(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::PaintOrder => {
                    let old_computed = old_style.get_inheritedsvg().clone_paint_order();
                    let new_computed = new_style.get_inheritedsvg().clone_paint_order();
                    AnimatedProperty::PaintOrder(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::DominantBaseline => {
                    let old_computed = old_style.get_svg().clone_dominant_baseline();
                    let new_computed = new_style.get_svg().clone_dominant_baseline();
                    AnimatedProperty::DominantBaseline(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::VectorEffect => {
                    let old_computed = old_style.get_svg().clone_vector_effect();
                    let new_computed = new_style.get_svg().clone_vector_effect();
                    AnimatedProperty::VectorEffect(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::StopColor => {
                    let old_computed = old_style.get_svg().clone_stop_color();
                    let new_computed = new_style.get_svg().clone_stop_color();
                    AnimatedProperty::StopColor(
                        old_computed.to_animated_value(),
                        new_computed.to_animated_value(),
                    )
                }
                AnimatableLonghand::StopOpacity => {
                    let old_computed = old_style.get_svg().clone_stop_opacity();
                    let new_computed = new_style.get_svg().clone_stop_opacity();
                    AnimatedProperty::StopOpacity(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::FloodColor => {
                    let old_computed = old_style.get_svg().clone_flood_color();
                    let new_computed = new_style.get_svg().clone_flood_color();
                    AnimatedProperty::FloodColor(
                        old_computed.to_animated_value(),
                        new_computed.to_animated_value(),
                    )
                }
                AnimatableLonghand::FloodOpacity => {
                    let old_computed = old_style.get_svg().clone_flood_opacity();
                    let new_computed = new_style.get_svg().clone_flood_opacity();
                    AnimatedProperty::FloodOpacity(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::LightingColor => {
                    let old_computed = old_style.get_svg().clone_lighting_color();
                    let new_computed = new_style.get_svg().clone_lighting_color();
                    AnimatedProperty::LightingColor(
                        old_computed.to_animated_value(),
                        new_computed.to_animated_value(),
                    )
                }
                AnimatableLonghand::MaskType => {
                    let old_computed = old_style.get_svg().clone_mask_type();
                    let new_computed = new_style.get_svg().clone_mask_type();
                    AnimatedProperty::MaskType(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::ClipPath => {
                    let old_computed = old_style.get_svg().clone_clip_path();
                    let new_computed = new_style.get_svg().clone_clip_path();
                    AnimatedProperty::ClipPath(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::MaskMode => {
                    let old_computed = old_style.get_svg().clone_mask_mode();
                    let new_computed = new_style.get_svg().clone_mask_mode();
                    AnimatedProperty::MaskMode(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::MaskRepeat => {
                    let old_computed = old_style.get_svg().clone_mask_repeat();
                    let new_computed = new_style.get_svg().clone_mask_repeat();
                    AnimatedProperty::MaskRepeat(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::MaskPositionX => {
                    let old_computed = old_style.get_svg().clone_mask_position_x();
                    let new_computed = new_style.get_svg().clone_mask_position_x();
                    AnimatedProperty::MaskPositionX(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::MaskPositionY => {
                    let old_computed = old_style.get_svg().clone_mask_position_y();
                    let new_computed = new_style.get_svg().clone_mask_position_y();
                    AnimatedProperty::MaskPositionY(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::MaskClip => {
                    let old_computed = old_style.get_svg().clone_mask_clip();
                    let new_computed = new_style.get_svg().clone_mask_clip();
                    AnimatedProperty::MaskClip(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::MaskOrigin => {
                    let old_computed = old_style.get_svg().clone_mask_origin();
                    let new_computed = new_style.get_svg().clone_mask_origin();
                    AnimatedProperty::MaskOrigin(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::MaskSize => {
                    let old_computed = old_style.get_svg().clone_mask_size();
                    let new_computed = new_style.get_svg().clone_mask_size();
                    AnimatedProperty::MaskSize(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::MaskComposite => {
                    let old_computed = old_style.get_svg().clone_mask_composite();
                    let new_computed = new_style.get_svg().clone_mask_composite();
                    AnimatedProperty::MaskComposite(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::MaskImage => {
                    let old_computed = old_style.get_svg().clone_mask_image();
                    let new_computed = new_style.get_svg().clone_mask_image();
                    AnimatedProperty::MaskImage(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::MozBoxAlign => {
                    let old_computed = old_style.get_xul().clone__moz_box_align();
                    let new_computed = new_style.get_xul().clone__moz_box_align();
                    AnimatedProperty::MozBoxAlign(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::MozBoxDirection => {
                    let old_computed = old_style.get_xul().clone__moz_box_direction();
                    let new_computed = new_style.get_xul().clone__moz_box_direction();
                    AnimatedProperty::MozBoxDirection(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::MozBoxFlex => {
                    let old_computed = old_style.get_xul().clone__moz_box_flex();
                    let new_computed = new_style.get_xul().clone__moz_box_flex();
                    AnimatedProperty::MozBoxFlex(
                        old_computed.to_animated_value(),
                        new_computed.to_animated_value(),
                    )
                }
                AnimatableLonghand::MozBoxOrient => {
                    let old_computed = old_style.get_xul().clone__moz_box_orient();
                    let new_computed = new_style.get_xul().clone__moz_box_orient();
                    AnimatedProperty::MozBoxOrient(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::MozBoxPack => {
                    let old_computed = old_style.get_xul().clone__moz_box_pack();
                    let new_computed = new_style.get_xul().clone__moz_box_pack();
                    AnimatedProperty::MozBoxPack(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::MozStackSizing => {
                    let old_computed = old_style.get_xul().clone__moz_stack_sizing();
                    let new_computed = new_style.get_xul().clone__moz_stack_sizing();
                    AnimatedProperty::MozStackSizing(
                        old_computed,
                        new_computed,
                    )
                }
                AnimatableLonghand::MozBoxOrdinalGroup => {
                    let old_computed = old_style.get_xul().clone__moz_box_ordinal_group();
                    let new_computed = new_style.get_xul().clone__moz_box_ordinal_group();
                    AnimatedProperty::MozBoxOrdinalGroup(
                        old_computed,
                        new_computed,
                    )
                }
        }
    }
}

/// A collection of AnimationValue that were composed on an element.
/// This HashMap stores the values that are the last AnimationValue to be
/// composed for each TransitionProperty.
#[cfg(feature = "gecko")]
pub type AnimationValueMap = FnvHashMap<AnimatableLonghand, AnimationValue>;
#[cfg(feature = "gecko")]
unsafe impl HasFFI for AnimationValueMap {
    type FFIType = RawServoAnimationValueMap;
}
#[cfg(feature = "gecko")]
unsafe impl HasSimpleFFI for AnimationValueMap {}

/// An enum to represent a single computed value belonging to an animated
/// property in order to be interpolated with another one. When interpolating,
/// both values need to belong to the same property.
///
/// This is different to AnimatedProperty in the sense that AnimatedProperty
/// also knows the final value to be used during the animation.
///
/// This is to be used in Gecko integration code.
///
/// FIXME: We need to add a path for custom properties, but that's trivial after
/// this (is a similar path to that of PropertyDeclaration).
#[derive(Clone, PartialEq)]
#[cfg_attr(feature = "servo", derive(HeapSizeOf))]
pub enum AnimationValue {
            /// background-color
                BackgroundColor(<longhands::background_color::computed_value::T as ToAnimatedValue>::AnimatedValue),
            /// background-image
                BackgroundImage(longhands::background_image::computed_value::T),
            /// background-position-x
                BackgroundPositionX(longhands::background_position_x::computed_value::T),
            /// background-position-y
                BackgroundPositionY(longhands::background_position_y::computed_value::T),
            /// background-repeat
                BackgroundRepeat(longhands::background_repeat::computed_value::T),
            /// background-attachment
                BackgroundAttachment(longhands::background_attachment::computed_value::T),
            /// background-clip
                BackgroundClip(longhands::background_clip::computed_value::T),
            /// background-origin
                BackgroundOrigin(longhands::background_origin::computed_value::T),
            /// background-size
                BackgroundSize(<longhands::background_size::computed_value::T as ToAnimatedValue>::AnimatedValue),
            /// background-blend-mode
                BackgroundBlendMode(longhands::background_blend_mode::computed_value::T),
            /// border-top-color
                BorderTopColor(<longhands::border_top_color::computed_value::T as ToAnimatedValue>::AnimatedValue),
            /// border-top-style
                BorderTopStyle(longhands::border_top_style::computed_value::T),
            /// border-top-width
                BorderTopWidth(<longhands::border_top_width::computed_value::T as ToAnimatedValue>::AnimatedValue),
            /// border-left-color
                BorderLeftColor(<longhands::border_left_color::computed_value::T as ToAnimatedValue>::AnimatedValue),
            /// border-left-style
                BorderLeftStyle(longhands::border_left_style::computed_value::T),
            /// border-left-width
                BorderLeftWidth(<longhands::border_left_width::computed_value::T as ToAnimatedValue>::AnimatedValue),
            /// border-bottom-color
                BorderBottomColor(<longhands::border_bottom_color::computed_value::T as ToAnimatedValue>::AnimatedValue),
            /// border-bottom-style
                BorderBottomStyle(longhands::border_bottom_style::computed_value::T),
            /// border-bottom-width
                BorderBottomWidth(<longhands::border_bottom_width::computed_value::T as ToAnimatedValue>::AnimatedValue),
            /// border-right-color
                BorderRightColor(<longhands::border_right_color::computed_value::T as ToAnimatedValue>::AnimatedValue),
            /// border-right-style
                BorderRightStyle(longhands::border_right_style::computed_value::T),
            /// border-right-width
                BorderRightWidth(<longhands::border_right_width::computed_value::T as ToAnimatedValue>::AnimatedValue),
            /// border-top-left-radius
                BorderTopLeftRadius(<longhands::border_top_left_radius::computed_value::T as ToAnimatedValue>::AnimatedValue),
            /// border-top-right-radius
                BorderTopRightRadius(<longhands::border_top_right_radius::computed_value::T as ToAnimatedValue>::AnimatedValue),
            /// border-bottom-right-radius
                BorderBottomRightRadius(<longhands::border_bottom_right_radius::computed_value::T as ToAnimatedValue>::AnimatedValue),
            /// border-bottom-left-radius
                BorderBottomLeftRadius(<longhands::border_bottom_left_radius::computed_value::T as ToAnimatedValue>::AnimatedValue),
            /// -moz-border-top-colors
                MozBorderTopColors(longhands::_moz_border_top_colors::computed_value::T),
            /// -moz-border-left-colors
                MozBorderLeftColors(longhands::_moz_border_left_colors::computed_value::T),
            /// -moz-border-bottom-colors
                MozBorderBottomColors(longhands::_moz_border_bottom_colors::computed_value::T),
            /// -moz-border-right-colors
                MozBorderRightColors(longhands::_moz_border_right_colors::computed_value::T),
            /// box-decoration-break
                BoxDecorationBreak(longhands::box_decoration_break::computed_value::T),
            /// -moz-float-edge
                MozFloatEdge(longhands::_moz_float_edge::computed_value::T),
            /// border-image-source
                BorderImageSource(longhands::border_image_source::computed_value::T),
            /// border-image-outset
                BorderImageOutset(longhands::border_image_outset::computed_value::T),
            /// border-image-repeat
                BorderImageRepeat(longhands::border_image_repeat::computed_value::T),
            /// border-image-width
                BorderImageWidth(longhands::border_image_width::computed_value::T),
            /// border-image-slice
                BorderImageSlice(longhands::border_image_slice::computed_value::T),
            /// display
                Display(longhands::display::computed_value::T),
            /// position
                Position(longhands::position::computed_value::T),
            /// float
                Float(longhands::float::computed_value::T),
            /// clear
                Clear(longhands::clear::computed_value::T),
            /// vertical-align
                VerticalAlign(longhands::vertical_align::computed_value::T),
            /// overflow-clip-box
                OverflowClipBox(longhands::overflow_clip_box::computed_value::T),
            /// overflow-x
                OverflowX(longhands::overflow_x::computed_value::T),
            /// overflow-y
                OverflowY(longhands::overflow_y::computed_value::T),
            /// scroll-snap-points-x
                ScrollSnapPointsX(longhands::scroll_snap_points_x::computed_value::T),
            /// scroll-snap-points-y
                ScrollSnapPointsY(longhands::scroll_snap_points_y::computed_value::T),
            /// scroll-snap-destination
                ScrollSnapDestination(longhands::scroll_snap_destination::computed_value::T),
            /// scroll-snap-coordinate
                ScrollSnapCoordinate(longhands::scroll_snap_coordinate::computed_value::T),
            /// transform
                Transform(longhands::transform::computed_value::T),
            /// scroll-behavior
                ScrollBehavior(longhands::scroll_behavior::computed_value::T),
            /// scroll-snap-type-x
                ScrollSnapTypeX(longhands::scroll_snap_type_x::computed_value::T),
            /// scroll-snap-type-y
                ScrollSnapTypeY(longhands::scroll_snap_type_y::computed_value::T),
            /// isolation
                Isolation(longhands::isolation::computed_value::T),
            /// page-break-after
                PageBreakAfter(longhands::page_break_after::computed_value::T),
            /// page-break-before
                PageBreakBefore(longhands::page_break_before::computed_value::T),
            /// page-break-inside
                PageBreakInside(longhands::page_break_inside::computed_value::T),
            /// resize
                Resize(longhands::resize::computed_value::T),
            /// perspective
                Perspective(longhands::perspective::computed_value::T),
            /// perspective-origin
                PerspectiveOrigin(longhands::perspective_origin::computed_value::T),
            /// backface-visibility
                BackfaceVisibility(longhands::backface_visibility::computed_value::T),
            /// transform-box
                TransformBox(longhands::transform_box::computed_value::T),
            /// transform-style
                TransformStyle(longhands::transform_style::computed_value::T),
            /// transform-origin
                TransformOrigin(longhands::transform_origin::computed_value::T),
            /// contain
                Contain(longhands::contain::computed_value::T),
            /// -moz-appearance
                MozAppearance(longhands::_moz_appearance::computed_value::T),
            /// -moz-orient
                MozOrient(longhands::_moz_orient::computed_value::T),
            /// will-change
                WillChange(longhands::will_change::computed_value::T),
            /// shape-outside
                ShapeOutside(longhands::shape_outside::computed_value::T),
            /// touch-action
                TouchAction(longhands::touch_action::computed_value::T),
            /// color
                Color(<longhands::color::computed_value::T as ToAnimatedValue>::AnimatedValue),
            /// column-width
                ColumnWidth(<longhands::column_width::computed_value::T as ToAnimatedValue>::AnimatedValue),
            /// column-count
                ColumnCount(<longhands::column_count::computed_value::T as ToAnimatedValue>::AnimatedValue),
            /// column-gap
                ColumnGap(<longhands::column_gap::computed_value::T as ToAnimatedValue>::AnimatedValue),
            /// column-fill
                ColumnFill(longhands::column_fill::computed_value::T),
            /// column-rule-width
                ColumnRuleWidth(<longhands::column_rule_width::computed_value::T as ToAnimatedValue>::AnimatedValue),
            /// column-rule-color
                ColumnRuleColor(<longhands::column_rule_color::computed_value::T as ToAnimatedValue>::AnimatedValue),
            /// column-span
                ColumnSpan(longhands::column_span::computed_value::T),
            /// column-rule-style
                ColumnRuleStyle(longhands::column_rule_style::computed_value::T),
            /// content
                Content(longhands::content::computed_value::T),
            /// counter-increment
                CounterIncrement(longhands::counter_increment::computed_value::T),
            /// counter-reset
                CounterReset(longhands::counter_reset::computed_value::T),
            /// opacity
                Opacity(longhands::opacity::computed_value::T),
            /// box-shadow
                BoxShadow(<longhands::box_shadow::computed_value::T as ToAnimatedValue>::AnimatedValue),
            /// clip
                Clip(longhands::clip::computed_value::T),
            /// filter
                Filter(<longhands::filter::computed_value::T as ToAnimatedValue>::AnimatedValue),
            /// mix-blend-mode
                MixBlendMode(longhands::mix_blend_mode::computed_value::T),
            /// font-family
                FontFamily(longhands::font_family::computed_value::T),
            /// font-style
                FontStyle(longhands::font_style::computed_value::T),
            /// font-variant-caps
                FontVariantCaps(longhands::font_variant_caps::computed_value::T),
            /// font-weight
                FontWeight(longhands::font_weight::computed_value::T),
            /// font-size
                FontSize(<longhands::font_size::computed_value::T as ToAnimatedValue>::AnimatedValue),
            /// font-size-adjust
                FontSizeAdjust(<longhands::font_size_adjust::computed_value::T as ToAnimatedValue>::AnimatedValue),
            /// font-synthesis
                FontSynthesis(longhands::font_synthesis::computed_value::T),
            /// font-stretch
                FontStretch(longhands::font_stretch::computed_value::T),
            /// font-kerning
                FontKerning(longhands::font_kerning::computed_value::T),
            /// font-variant-alternates
                FontVariantAlternates(longhands::font_variant_alternates::computed_value::T),
            /// font-variant-east-asian
                FontVariantEastAsian(longhands::font_variant_east_asian::computed_value::T),
            /// font-variant-ligatures
                FontVariantLigatures(longhands::font_variant_ligatures::computed_value::T),
            /// font-variant-numeric
                FontVariantNumeric(longhands::font_variant_numeric::computed_value::T),
            /// font-variant-position
                FontVariantPosition(longhands::font_variant_position::computed_value::T),
            /// font-feature-settings
                FontFeatureSettings(longhands::font_feature_settings::computed_value::T),
            /// font-variation-settings
                FontVariationSettings(longhands::font_variation_settings::computed_value::T),
            /// font-language-override
                FontLanguageOverride(longhands::font_language_override::computed_value::T),
            /// -moz-osx-font-smoothing
                MozOsxFontSmoothing(longhands::_moz_osx_font_smoothing::computed_value::T),
            /// visibility
                Visibility(longhands::visibility::computed_value::T),
            /// writing-mode
                WritingMode(longhands::writing_mode::computed_value::T),
            /// direction
                Direction(longhands::direction::computed_value::T),
            /// text-orientation
                TextOrientation(longhands::text_orientation::computed_value::T),
            /// color-adjust
                ColorAdjust(longhands::color_adjust::computed_value::T),
            /// image-rendering
                ImageRendering(longhands::image_rendering::computed_value::T),
            /// image-orientation
                ImageOrientation(longhands::image_orientation::computed_value::T),
            /// border-collapse
                BorderCollapse(longhands::border_collapse::computed_value::T),
            /// empty-cells
                EmptyCells(longhands::empty_cells::computed_value::T),
            /// caption-side
                CaptionSide(longhands::caption_side::computed_value::T),
            /// border-spacing
                BorderSpacing(<longhands::border_spacing::computed_value::T as ToAnimatedValue>::AnimatedValue),
            /// line-height
                LineHeight(<longhands::line_height::computed_value::T as ToAnimatedValue>::AnimatedValue),
            /// text-transform
                TextTransform(longhands::text_transform::computed_value::T),
            /// hyphens
                Hyphens(longhands::hyphens::computed_value::T),
            /// -moz-text-size-adjust
                MozTextSizeAdjust(longhands::_moz_text_size_adjust::computed_value::T),
            /// text-indent
                TextIndent(longhands::text_indent::computed_value::T),
            /// overflow-wrap
                OverflowWrap(longhands::overflow_wrap::computed_value::T),
            /// word-break
                WordBreak(longhands::word_break::computed_value::T),
            /// text-justify
                TextJustify(longhands::text_justify::computed_value::T),
            /// text-align-last
                TextAlignLast(longhands::text_align_last::computed_value::T),
            /// text-align
                TextAlign(longhands::text_align::computed_value::T),
            /// letter-spacing
                LetterSpacing(longhands::letter_spacing::computed_value::T),
            /// word-spacing
                WordSpacing(longhands::word_spacing::computed_value::T),
            /// white-space
                WhiteSpace(longhands::white_space::computed_value::T),
            /// text-shadow
                TextShadow(<longhands::text_shadow::computed_value::T as ToAnimatedValue>::AnimatedValue),
            /// text-emphasis-style
                TextEmphasisStyle(longhands::text_emphasis_style::computed_value::T),
            /// text-emphasis-position
                TextEmphasisPosition(longhands::text_emphasis_position::computed_value::T),
            /// text-emphasis-color
                TextEmphasisColor(<longhands::text_emphasis_color::computed_value::T as ToAnimatedValue>::AnimatedValue),
            /// -moz-tab-size
                MozTabSize(<longhands::_moz_tab_size::computed_value::T as ToAnimatedValue>::AnimatedValue),
            /// -webkit-text-fill-color
                WebkitTextFillColor(<longhands::_webkit_text_fill_color::computed_value::T as ToAnimatedValue>::AnimatedValue),
            /// -webkit-text-stroke-color
                WebkitTextStrokeColor(<longhands::_webkit_text_stroke_color::computed_value::T as ToAnimatedValue>::AnimatedValue),
            /// -webkit-text-stroke-width
                WebkitTextStrokeWidth(longhands::_webkit_text_stroke_width::computed_value::T),
            /// ruby-align
                RubyAlign(longhands::ruby_align::computed_value::T),
            /// ruby-position
                RubyPosition(longhands::ruby_position::computed_value::T),
            /// text-combine-upright
                TextCombineUpright(longhands::text_combine_upright::computed_value::T),
            /// text-rendering
                TextRendering(longhands::text_rendering::computed_value::T),
            /// list-style-position
                ListStylePosition(longhands::list_style_position::computed_value::T),
            /// list-style-type
                ListStyleType(longhands::list_style_type::computed_value::T),
            /// list-style-image
                ListStyleImage(longhands::list_style_image::computed_value::T),
            /// quotes
                Quotes(longhands::quotes::computed_value::T),
            /// -moz-image-region
                MozImageRegion(longhands::_moz_image_region::computed_value::T),
            /// margin-top
                MarginTop(longhands::margin_top::computed_value::T),
            /// margin-left
                MarginLeft(longhands::margin_left::computed_value::T),
            /// margin-bottom
                MarginBottom(longhands::margin_bottom::computed_value::T),
            /// margin-right
                MarginRight(longhands::margin_right::computed_value::T),
            /// outline-color
                OutlineColor(<longhands::outline_color::computed_value::T as ToAnimatedValue>::AnimatedValue),
            /// outline-style
                OutlineStyle(longhands::outline_style::computed_value::T),
            /// outline-width
                OutlineWidth(<longhands::outline_width::computed_value::T as ToAnimatedValue>::AnimatedValue),
            /// -moz-outline-radius-topleft
                MozOutlineRadiusTopleft(<longhands::_moz_outline_radius_topleft::computed_value::T as ToAnimatedValue>::AnimatedValue),
            /// -moz-outline-radius-topright
                MozOutlineRadiusTopright(<longhands::_moz_outline_radius_topright::computed_value::T as ToAnimatedValue>::AnimatedValue),
            /// -moz-outline-radius-bottomright
                MozOutlineRadiusBottomright(<longhands::_moz_outline_radius_bottomright::computed_value::T as ToAnimatedValue>::AnimatedValue),
            /// -moz-outline-radius-bottomleft
                MozOutlineRadiusBottomleft(<longhands::_moz_outline_radius_bottomleft::computed_value::T as ToAnimatedValue>::AnimatedValue),
            /// outline-offset
                OutlineOffset(longhands::outline_offset::computed_value::T),
            /// padding-top
                PaddingTop(<longhands::padding_top::computed_value::T as ToAnimatedValue>::AnimatedValue),
            /// padding-left
                PaddingLeft(<longhands::padding_left::computed_value::T as ToAnimatedValue>::AnimatedValue),
            /// padding-bottom
                PaddingBottom(<longhands::padding_bottom::computed_value::T as ToAnimatedValue>::AnimatedValue),
            /// padding-right
                PaddingRight(<longhands::padding_right::computed_value::T as ToAnimatedValue>::AnimatedValue),
            /// cursor
                Cursor(longhands::cursor::computed_value::T),
            /// pointer-events
                PointerEvents(longhands::pointer_events::computed_value::T),
            /// -moz-user-input
                MozUserInput(longhands::_moz_user_input::computed_value::T),
            /// -moz-user-modify
                MozUserModify(longhands::_moz_user_modify::computed_value::T),
            /// -moz-user-focus
                MozUserFocus(longhands::_moz_user_focus::computed_value::T),
            /// caret-color
                CaretColor(<longhands::caret_color::computed_value::T as ToAnimatedValue>::AnimatedValue),
            /// top
                Top(longhands::top::computed_value::T),
            /// left
                Left(longhands::left::computed_value::T),
            /// bottom
                Bottom(longhands::bottom::computed_value::T),
            /// right
                Right(longhands::right::computed_value::T),
            /// z-index
                ZIndex(longhands::z_index::computed_value::T),
            /// flex-direction
                FlexDirection(longhands::flex_direction::computed_value::T),
            /// flex-wrap
                FlexWrap(longhands::flex_wrap::computed_value::T),
            /// justify-content
                JustifyContent(longhands::justify_content::computed_value::T),
            /// align-content
                AlignContent(longhands::align_content::computed_value::T),
            /// align-items
                AlignItems(longhands::align_items::computed_value::T),
            /// justify-items
                JustifyItems(longhands::justify_items::computed_value::T),
            /// flex-grow
                FlexGrow(<longhands::flex_grow::computed_value::T as ToAnimatedValue>::AnimatedValue),
            /// flex-shrink
                FlexShrink(<longhands::flex_shrink::computed_value::T as ToAnimatedValue>::AnimatedValue),
            /// align-self
                AlignSelf(longhands::align_self::computed_value::T),
            /// justify-self
                JustifySelf(longhands::justify_self::computed_value::T),
            /// order
                Order(longhands::order::computed_value::T),
            /// flex-basis
                FlexBasis(<longhands::flex_basis::computed_value::T as ToAnimatedValue>::AnimatedValue),
            /// width
                Width(<longhands::width::computed_value::T as ToAnimatedValue>::AnimatedValue),
            /// min-width
                MinWidth(<longhands::min_width::computed_value::T as ToAnimatedValue>::AnimatedValue),
            /// max-width
                MaxWidth(<longhands::max_width::computed_value::T as ToAnimatedValue>::AnimatedValue),
            /// height
                Height(<longhands::height::computed_value::T as ToAnimatedValue>::AnimatedValue),
            /// min-height
                MinHeight(<longhands::min_height::computed_value::T as ToAnimatedValue>::AnimatedValue),
            /// max-height
                MaxHeight(<longhands::max_height::computed_value::T as ToAnimatedValue>::AnimatedValue),
            /// box-sizing
                BoxSizing(longhands::box_sizing::computed_value::T),
            /// object-fit
                ObjectFit(longhands::object_fit::computed_value::T),
            /// object-position
                ObjectPosition(longhands::object_position::computed_value::T),
            /// grid-row-gap
                GridRowGap(<longhands::grid_row_gap::computed_value::T as ToAnimatedValue>::AnimatedValue),
            /// grid-row-start
                GridRowStart(longhands::grid_row_start::computed_value::T),
            /// grid-row-end
                GridRowEnd(longhands::grid_row_end::computed_value::T),
            /// grid-auto-rows
                GridAutoRows(longhands::grid_auto_rows::computed_value::T),
            /// grid-template-rows
                GridTemplateRows(longhands::grid_template_rows::computed_value::T),
            /// grid-column-gap
                GridColumnGap(<longhands::grid_column_gap::computed_value::T as ToAnimatedValue>::AnimatedValue),
            /// grid-column-start
                GridColumnStart(longhands::grid_column_start::computed_value::T),
            /// grid-column-end
                GridColumnEnd(longhands::grid_column_end::computed_value::T),
            /// grid-auto-columns
                GridAutoColumns(longhands::grid_auto_columns::computed_value::T),
            /// grid-template-columns
                GridTemplateColumns(longhands::grid_template_columns::computed_value::T),
            /// grid-auto-flow
                GridAutoFlow(longhands::grid_auto_flow::computed_value::T),
            /// grid-template-areas
                GridTemplateAreas(longhands::grid_template_areas::computed_value::T),
            /// table-layout
                TableLayout(longhands::table_layout::computed_value::T),
            /// text-overflow
                TextOverflow(longhands::text_overflow::computed_value::T),
            /// unicode-bidi
                UnicodeBidi(longhands::unicode_bidi::computed_value::T),
            /// text-decoration-line
                TextDecorationLine(longhands::text_decoration_line::computed_value::T),
            /// text-decoration-style
                TextDecorationStyle(longhands::text_decoration_style::computed_value::T),
            /// text-decoration-color
                TextDecorationColor(<longhands::text_decoration_color::computed_value::T as ToAnimatedValue>::AnimatedValue),
            /// initial-letter
                InitialLetter(longhands::initial_letter::computed_value::T),
            /// ime-mode
                ImeMode(longhands::ime_mode::computed_value::T),
            /// -moz-user-select
                MozUserSelect(longhands::_moz_user_select::computed_value::T),
            /// -moz-window-dragging
                MozWindowDragging(longhands::_moz_window_dragging::computed_value::T),
            /// -moz-window-shadow
                MozWindowShadow(longhands::_moz_window_shadow::computed_value::T),
            /// -moz-force-broken-image-icon
                MozForceBrokenImageIcon(longhands::_moz_force_broken_image_icon::computed_value::T),
            /// text-anchor
                TextAnchor(longhands::text_anchor::computed_value::T),
            /// color-interpolation
                ColorInterpolation(longhands::color_interpolation::computed_value::T),
            /// color-interpolation-filters
                ColorInterpolationFilters(longhands::color_interpolation_filters::computed_value::T),
            /// fill
                Fill(<longhands::fill::computed_value::T as ToAnimatedValue>::AnimatedValue),
            /// fill-opacity
                FillOpacity(longhands::fill_opacity::computed_value::T),
            /// fill-rule
                FillRule(longhands::fill_rule::computed_value::T),
            /// shape-rendering
                ShapeRendering(longhands::shape_rendering::computed_value::T),
            /// stroke
                Stroke(<longhands::stroke::computed_value::T as ToAnimatedValue>::AnimatedValue),
            /// stroke-width
                StrokeWidth(<longhands::stroke_width::computed_value::T as ToAnimatedValue>::AnimatedValue),
            /// stroke-linecap
                StrokeLinecap(longhands::stroke_linecap::computed_value::T),
            /// stroke-linejoin
                StrokeLinejoin(longhands::stroke_linejoin::computed_value::T),
            /// stroke-miterlimit
                StrokeMiterlimit(<longhands::stroke_miterlimit::computed_value::T as ToAnimatedValue>::AnimatedValue),
            /// stroke-opacity
                StrokeOpacity(longhands::stroke_opacity::computed_value::T),
            /// stroke-dasharray
                StrokeDasharray(<longhands::stroke_dasharray::computed_value::T as ToAnimatedValue>::AnimatedValue),
            /// stroke-dashoffset
                StrokeDashoffset(longhands::stroke_dashoffset::computed_value::T),
            /// clip-rule
                ClipRule(longhands::clip_rule::computed_value::T),
            /// marker-start
                MarkerStart(longhands::marker_start::computed_value::T),
            /// marker-mid
                MarkerMid(longhands::marker_mid::computed_value::T),
            /// marker-end
                MarkerEnd(longhands::marker_end::computed_value::T),
            /// paint-order
                PaintOrder(longhands::paint_order::computed_value::T),
            /// dominant-baseline
                DominantBaseline(longhands::dominant_baseline::computed_value::T),
            /// vector-effect
                VectorEffect(longhands::vector_effect::computed_value::T),
            /// stop-color
                StopColor(<longhands::stop_color::computed_value::T as ToAnimatedValue>::AnimatedValue),
            /// stop-opacity
                StopOpacity(longhands::stop_opacity::computed_value::T),
            /// flood-color
                FloodColor(<longhands::flood_color::computed_value::T as ToAnimatedValue>::AnimatedValue),
            /// flood-opacity
                FloodOpacity(longhands::flood_opacity::computed_value::T),
            /// lighting-color
                LightingColor(<longhands::lighting_color::computed_value::T as ToAnimatedValue>::AnimatedValue),
            /// mask-type
                MaskType(longhands::mask_type::computed_value::T),
            /// clip-path
                ClipPath(longhands::clip_path::computed_value::T),
            /// mask-mode
                MaskMode(longhands::mask_mode::computed_value::T),
            /// mask-repeat
                MaskRepeat(longhands::mask_repeat::computed_value::T),
            /// mask-position-x
                MaskPositionX(longhands::mask_position_x::computed_value::T),
            /// mask-position-y
                MaskPositionY(longhands::mask_position_y::computed_value::T),
            /// mask-clip
                MaskClip(longhands::mask_clip::computed_value::T),
            /// mask-origin
                MaskOrigin(longhands::mask_origin::computed_value::T),
            /// mask-size
                MaskSize(longhands::mask_size::computed_value::T),
            /// mask-composite
                MaskComposite(longhands::mask_composite::computed_value::T),
            /// mask-image
                MaskImage(longhands::mask_image::computed_value::T),
            /// -moz-box-align
                MozBoxAlign(longhands::_moz_box_align::computed_value::T),
            /// -moz-box-direction
                MozBoxDirection(longhands::_moz_box_direction::computed_value::T),
            /// -moz-box-flex
                MozBoxFlex(<longhands::_moz_box_flex::computed_value::T as ToAnimatedValue>::AnimatedValue),
            /// -moz-box-orient
                MozBoxOrient(longhands::_moz_box_orient::computed_value::T),
            /// -moz-box-pack
                MozBoxPack(longhands::_moz_box_pack::computed_value::T),
            /// -moz-stack-sizing
                MozStackSizing(longhands::_moz_stack_sizing::computed_value::T),
            /// -moz-box-ordinal-group
                MozBoxOrdinalGroup(longhands::_moz_box_ordinal_group::computed_value::T),
}

impl fmt::Debug for AnimationValue {
    fn fmt(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
        let name = match *self {
                AnimationValue::BackgroundColor(..) => "BackgroundColor",
                AnimationValue::BackgroundImage(..) => "BackgroundImage",
                AnimationValue::BackgroundPositionX(..) => "BackgroundPositionX",
                AnimationValue::BackgroundPositionY(..) => "BackgroundPositionY",
                AnimationValue::BackgroundRepeat(..) => "BackgroundRepeat",
                AnimationValue::BackgroundAttachment(..) => "BackgroundAttachment",
                AnimationValue::BackgroundClip(..) => "BackgroundClip",
                AnimationValue::BackgroundOrigin(..) => "BackgroundOrigin",
                AnimationValue::BackgroundSize(..) => "BackgroundSize",
                AnimationValue::BackgroundBlendMode(..) => "BackgroundBlendMode",
                AnimationValue::BorderTopColor(..) => "BorderTopColor",
                AnimationValue::BorderTopStyle(..) => "BorderTopStyle",
                AnimationValue::BorderTopWidth(..) => "BorderTopWidth",
                AnimationValue::BorderLeftColor(..) => "BorderLeftColor",
                AnimationValue::BorderLeftStyle(..) => "BorderLeftStyle",
                AnimationValue::BorderLeftWidth(..) => "BorderLeftWidth",
                AnimationValue::BorderBottomColor(..) => "BorderBottomColor",
                AnimationValue::BorderBottomStyle(..) => "BorderBottomStyle",
                AnimationValue::BorderBottomWidth(..) => "BorderBottomWidth",
                AnimationValue::BorderRightColor(..) => "BorderRightColor",
                AnimationValue::BorderRightStyle(..) => "BorderRightStyle",
                AnimationValue::BorderRightWidth(..) => "BorderRightWidth",
                AnimationValue::BorderTopLeftRadius(..) => "BorderTopLeftRadius",
                AnimationValue::BorderTopRightRadius(..) => "BorderTopRightRadius",
                AnimationValue::BorderBottomRightRadius(..) => "BorderBottomRightRadius",
                AnimationValue::BorderBottomLeftRadius(..) => "BorderBottomLeftRadius",
                AnimationValue::MozBorderTopColors(..) => "MozBorderTopColors",
                AnimationValue::MozBorderLeftColors(..) => "MozBorderLeftColors",
                AnimationValue::MozBorderBottomColors(..) => "MozBorderBottomColors",
                AnimationValue::MozBorderRightColors(..) => "MozBorderRightColors",
                AnimationValue::BoxDecorationBreak(..) => "BoxDecorationBreak",
                AnimationValue::MozFloatEdge(..) => "MozFloatEdge",
                AnimationValue::BorderImageSource(..) => "BorderImageSource",
                AnimationValue::BorderImageOutset(..) => "BorderImageOutset",
                AnimationValue::BorderImageRepeat(..) => "BorderImageRepeat",
                AnimationValue::BorderImageWidth(..) => "BorderImageWidth",
                AnimationValue::BorderImageSlice(..) => "BorderImageSlice",
                AnimationValue::Display(..) => "Display",
                AnimationValue::Position(..) => "Position",
                AnimationValue::Float(..) => "Float",
                AnimationValue::Clear(..) => "Clear",
                AnimationValue::VerticalAlign(..) => "VerticalAlign",
                AnimationValue::OverflowClipBox(..) => "OverflowClipBox",
                AnimationValue::OverflowX(..) => "OverflowX",
                AnimationValue::OverflowY(..) => "OverflowY",
                AnimationValue::ScrollSnapPointsX(..) => "ScrollSnapPointsX",
                AnimationValue::ScrollSnapPointsY(..) => "ScrollSnapPointsY",
                AnimationValue::ScrollSnapDestination(..) => "ScrollSnapDestination",
                AnimationValue::ScrollSnapCoordinate(..) => "ScrollSnapCoordinate",
                AnimationValue::Transform(..) => "Transform",
                AnimationValue::ScrollBehavior(..) => "ScrollBehavior",
                AnimationValue::ScrollSnapTypeX(..) => "ScrollSnapTypeX",
                AnimationValue::ScrollSnapTypeY(..) => "ScrollSnapTypeY",
                AnimationValue::Isolation(..) => "Isolation",
                AnimationValue::PageBreakAfter(..) => "PageBreakAfter",
                AnimationValue::PageBreakBefore(..) => "PageBreakBefore",
                AnimationValue::PageBreakInside(..) => "PageBreakInside",
                AnimationValue::Resize(..) => "Resize",
                AnimationValue::Perspective(..) => "Perspective",
                AnimationValue::PerspectiveOrigin(..) => "PerspectiveOrigin",
                AnimationValue::BackfaceVisibility(..) => "BackfaceVisibility",
                AnimationValue::TransformBox(..) => "TransformBox",
                AnimationValue::TransformStyle(..) => "TransformStyle",
                AnimationValue::TransformOrigin(..) => "TransformOrigin",
                AnimationValue::Contain(..) => "Contain",
                AnimationValue::MozAppearance(..) => "MozAppearance",
                AnimationValue::MozOrient(..) => "MozOrient",
                AnimationValue::WillChange(..) => "WillChange",
                AnimationValue::ShapeOutside(..) => "ShapeOutside",
                AnimationValue::TouchAction(..) => "TouchAction",
                AnimationValue::Color(..) => "Color",
                AnimationValue::ColumnWidth(..) => "ColumnWidth",
                AnimationValue::ColumnCount(..) => "ColumnCount",
                AnimationValue::ColumnGap(..) => "ColumnGap",
                AnimationValue::ColumnFill(..) => "ColumnFill",
                AnimationValue::ColumnRuleWidth(..) => "ColumnRuleWidth",
                AnimationValue::ColumnRuleColor(..) => "ColumnRuleColor",
                AnimationValue::ColumnSpan(..) => "ColumnSpan",
                AnimationValue::ColumnRuleStyle(..) => "ColumnRuleStyle",
                AnimationValue::Content(..) => "Content",
                AnimationValue::CounterIncrement(..) => "CounterIncrement",
                AnimationValue::CounterReset(..) => "CounterReset",
                AnimationValue::Opacity(..) => "Opacity",
                AnimationValue::BoxShadow(..) => "BoxShadow",
                AnimationValue::Clip(..) => "Clip",
                AnimationValue::Filter(..) => "Filter",
                AnimationValue::MixBlendMode(..) => "MixBlendMode",
                AnimationValue::FontFamily(..) => "FontFamily",
                AnimationValue::FontStyle(..) => "FontStyle",
                AnimationValue::FontVariantCaps(..) => "FontVariantCaps",
                AnimationValue::FontWeight(..) => "FontWeight",
                AnimationValue::FontSize(..) => "FontSize",
                AnimationValue::FontSizeAdjust(..) => "FontSizeAdjust",
                AnimationValue::FontSynthesis(..) => "FontSynthesis",
                AnimationValue::FontStretch(..) => "FontStretch",
                AnimationValue::FontKerning(..) => "FontKerning",
                AnimationValue::FontVariantAlternates(..) => "FontVariantAlternates",
                AnimationValue::FontVariantEastAsian(..) => "FontVariantEastAsian",
                AnimationValue::FontVariantLigatures(..) => "FontVariantLigatures",
                AnimationValue::FontVariantNumeric(..) => "FontVariantNumeric",
                AnimationValue::FontVariantPosition(..) => "FontVariantPosition",
                AnimationValue::FontFeatureSettings(..) => "FontFeatureSettings",
                AnimationValue::FontVariationSettings(..) => "FontVariationSettings",
                AnimationValue::FontLanguageOverride(..) => "FontLanguageOverride",
                AnimationValue::MozOsxFontSmoothing(..) => "MozOsxFontSmoothing",
                AnimationValue::Visibility(..) => "Visibility",
                AnimationValue::WritingMode(..) => "WritingMode",
                AnimationValue::Direction(..) => "Direction",
                AnimationValue::TextOrientation(..) => "TextOrientation",
                AnimationValue::ColorAdjust(..) => "ColorAdjust",
                AnimationValue::ImageRendering(..) => "ImageRendering",
                AnimationValue::ImageOrientation(..) => "ImageOrientation",
                AnimationValue::BorderCollapse(..) => "BorderCollapse",
                AnimationValue::EmptyCells(..) => "EmptyCells",
                AnimationValue::CaptionSide(..) => "CaptionSide",
                AnimationValue::BorderSpacing(..) => "BorderSpacing",
                AnimationValue::LineHeight(..) => "LineHeight",
                AnimationValue::TextTransform(..) => "TextTransform",
                AnimationValue::Hyphens(..) => "Hyphens",
                AnimationValue::MozTextSizeAdjust(..) => "MozTextSizeAdjust",
                AnimationValue::TextIndent(..) => "TextIndent",
                AnimationValue::OverflowWrap(..) => "OverflowWrap",
                AnimationValue::WordBreak(..) => "WordBreak",
                AnimationValue::TextJustify(..) => "TextJustify",
                AnimationValue::TextAlignLast(..) => "TextAlignLast",
                AnimationValue::TextAlign(..) => "TextAlign",
                AnimationValue::LetterSpacing(..) => "LetterSpacing",
                AnimationValue::WordSpacing(..) => "WordSpacing",
                AnimationValue::WhiteSpace(..) => "WhiteSpace",
                AnimationValue::TextShadow(..) => "TextShadow",
                AnimationValue::TextEmphasisStyle(..) => "TextEmphasisStyle",
                AnimationValue::TextEmphasisPosition(..) => "TextEmphasisPosition",
                AnimationValue::TextEmphasisColor(..) => "TextEmphasisColor",
                AnimationValue::MozTabSize(..) => "MozTabSize",
                AnimationValue::WebkitTextFillColor(..) => "WebkitTextFillColor",
                AnimationValue::WebkitTextStrokeColor(..) => "WebkitTextStrokeColor",
                AnimationValue::WebkitTextStrokeWidth(..) => "WebkitTextStrokeWidth",
                AnimationValue::RubyAlign(..) => "RubyAlign",
                AnimationValue::RubyPosition(..) => "RubyPosition",
                AnimationValue::TextCombineUpright(..) => "TextCombineUpright",
                AnimationValue::TextRendering(..) => "TextRendering",
                AnimationValue::ListStylePosition(..) => "ListStylePosition",
                AnimationValue::ListStyleType(..) => "ListStyleType",
                AnimationValue::ListStyleImage(..) => "ListStyleImage",
                AnimationValue::Quotes(..) => "Quotes",
                AnimationValue::MozImageRegion(..) => "MozImageRegion",
                AnimationValue::MarginTop(..) => "MarginTop",
                AnimationValue::MarginLeft(..) => "MarginLeft",
                AnimationValue::MarginBottom(..) => "MarginBottom",
                AnimationValue::MarginRight(..) => "MarginRight",
                AnimationValue::OutlineColor(..) => "OutlineColor",
                AnimationValue::OutlineStyle(..) => "OutlineStyle",
                AnimationValue::OutlineWidth(..) => "OutlineWidth",
                AnimationValue::MozOutlineRadiusTopleft(..) => "MozOutlineRadiusTopleft",
                AnimationValue::MozOutlineRadiusTopright(..) => "MozOutlineRadiusTopright",
                AnimationValue::MozOutlineRadiusBottomright(..) => "MozOutlineRadiusBottomright",
                AnimationValue::MozOutlineRadiusBottomleft(..) => "MozOutlineRadiusBottomleft",
                AnimationValue::OutlineOffset(..) => "OutlineOffset",
                AnimationValue::PaddingTop(..) => "PaddingTop",
                AnimationValue::PaddingLeft(..) => "PaddingLeft",
                AnimationValue::PaddingBottom(..) => "PaddingBottom",
                AnimationValue::PaddingRight(..) => "PaddingRight",
                AnimationValue::Cursor(..) => "Cursor",
                AnimationValue::PointerEvents(..) => "PointerEvents",
                AnimationValue::MozUserInput(..) => "MozUserInput",
                AnimationValue::MozUserModify(..) => "MozUserModify",
                AnimationValue::MozUserFocus(..) => "MozUserFocus",
                AnimationValue::CaretColor(..) => "CaretColor",
                AnimationValue::Top(..) => "Top",
                AnimationValue::Left(..) => "Left",
                AnimationValue::Bottom(..) => "Bottom",
                AnimationValue::Right(..) => "Right",
                AnimationValue::ZIndex(..) => "ZIndex",
                AnimationValue::FlexDirection(..) => "FlexDirection",
                AnimationValue::FlexWrap(..) => "FlexWrap",
                AnimationValue::JustifyContent(..) => "JustifyContent",
                AnimationValue::AlignContent(..) => "AlignContent",
                AnimationValue::AlignItems(..) => "AlignItems",
                AnimationValue::JustifyItems(..) => "JustifyItems",
                AnimationValue::FlexGrow(..) => "FlexGrow",
                AnimationValue::FlexShrink(..) => "FlexShrink",
                AnimationValue::AlignSelf(..) => "AlignSelf",
                AnimationValue::JustifySelf(..) => "JustifySelf",
                AnimationValue::Order(..) => "Order",
                AnimationValue::FlexBasis(..) => "FlexBasis",
                AnimationValue::Width(..) => "Width",
                AnimationValue::MinWidth(..) => "MinWidth",
                AnimationValue::MaxWidth(..) => "MaxWidth",
                AnimationValue::Height(..) => "Height",
                AnimationValue::MinHeight(..) => "MinHeight",
                AnimationValue::MaxHeight(..) => "MaxHeight",
                AnimationValue::BoxSizing(..) => "BoxSizing",
                AnimationValue::ObjectFit(..) => "ObjectFit",
                AnimationValue::ObjectPosition(..) => "ObjectPosition",
                AnimationValue::GridRowGap(..) => "GridRowGap",
                AnimationValue::GridRowStart(..) => "GridRowStart",
                AnimationValue::GridRowEnd(..) => "GridRowEnd",
                AnimationValue::GridAutoRows(..) => "GridAutoRows",
                AnimationValue::GridTemplateRows(..) => "GridTemplateRows",
                AnimationValue::GridColumnGap(..) => "GridColumnGap",
                AnimationValue::GridColumnStart(..) => "GridColumnStart",
                AnimationValue::GridColumnEnd(..) => "GridColumnEnd",
                AnimationValue::GridAutoColumns(..) => "GridAutoColumns",
                AnimationValue::GridTemplateColumns(..) => "GridTemplateColumns",
                AnimationValue::GridAutoFlow(..) => "GridAutoFlow",
                AnimationValue::GridTemplateAreas(..) => "GridTemplateAreas",
                AnimationValue::TableLayout(..) => "TableLayout",
                AnimationValue::TextOverflow(..) => "TextOverflow",
                AnimationValue::UnicodeBidi(..) => "UnicodeBidi",
                AnimationValue::TextDecorationLine(..) => "TextDecorationLine",
                AnimationValue::TextDecorationStyle(..) => "TextDecorationStyle",
                AnimationValue::TextDecorationColor(..) => "TextDecorationColor",
                AnimationValue::InitialLetter(..) => "InitialLetter",
                AnimationValue::ImeMode(..) => "ImeMode",
                AnimationValue::MozUserSelect(..) => "MozUserSelect",
                AnimationValue::MozWindowDragging(..) => "MozWindowDragging",
                AnimationValue::MozWindowShadow(..) => "MozWindowShadow",
                AnimationValue::MozForceBrokenImageIcon(..) => "MozForceBrokenImageIcon",
                AnimationValue::TextAnchor(..) => "TextAnchor",
                AnimationValue::ColorInterpolation(..) => "ColorInterpolation",
                AnimationValue::ColorInterpolationFilters(..) => "ColorInterpolationFilters",
                AnimationValue::Fill(..) => "Fill",
                AnimationValue::FillOpacity(..) => "FillOpacity",
                AnimationValue::FillRule(..) => "FillRule",
                AnimationValue::ShapeRendering(..) => "ShapeRendering",
                AnimationValue::Stroke(..) => "Stroke",
                AnimationValue::StrokeWidth(..) => "StrokeWidth",
                AnimationValue::StrokeLinecap(..) => "StrokeLinecap",
                AnimationValue::StrokeLinejoin(..) => "StrokeLinejoin",
                AnimationValue::StrokeMiterlimit(..) => "StrokeMiterlimit",
                AnimationValue::StrokeOpacity(..) => "StrokeOpacity",
                AnimationValue::StrokeDasharray(..) => "StrokeDasharray",
                AnimationValue::StrokeDashoffset(..) => "StrokeDashoffset",
                AnimationValue::ClipRule(..) => "ClipRule",
                AnimationValue::MarkerStart(..) => "MarkerStart",
                AnimationValue::MarkerMid(..) => "MarkerMid",
                AnimationValue::MarkerEnd(..) => "MarkerEnd",
                AnimationValue::PaintOrder(..) => "PaintOrder",
                AnimationValue::DominantBaseline(..) => "DominantBaseline",
                AnimationValue::VectorEffect(..) => "VectorEffect",
                AnimationValue::StopColor(..) => "StopColor",
                AnimationValue::StopOpacity(..) => "StopOpacity",
                AnimationValue::FloodColor(..) => "FloodColor",
                AnimationValue::FloodOpacity(..) => "FloodOpacity",
                AnimationValue::LightingColor(..) => "LightingColor",
                AnimationValue::MaskType(..) => "MaskType",
                AnimationValue::ClipPath(..) => "ClipPath",
                AnimationValue::MaskMode(..) => "MaskMode",
                AnimationValue::MaskRepeat(..) => "MaskRepeat",
                AnimationValue::MaskPositionX(..) => "MaskPositionX",
                AnimationValue::MaskPositionY(..) => "MaskPositionY",
                AnimationValue::MaskClip(..) => "MaskClip",
                AnimationValue::MaskOrigin(..) => "MaskOrigin",
                AnimationValue::MaskSize(..) => "MaskSize",
                AnimationValue::MaskComposite(..) => "MaskComposite",
                AnimationValue::MaskImage(..) => "MaskImage",
                AnimationValue::MozBoxAlign(..) => "MozBoxAlign",
                AnimationValue::MozBoxDirection(..) => "MozBoxDirection",
                AnimationValue::MozBoxFlex(..) => "MozBoxFlex",
                AnimationValue::MozBoxOrient(..) => "MozBoxOrient",
                AnimationValue::MozBoxPack(..) => "MozBoxPack",
                AnimationValue::MozStackSizing(..) => "MozStackSizing",
                AnimationValue::MozBoxOrdinalGroup(..) => "MozBoxOrdinalGroup",
        };
        formatter.write_str(name)
    }
}

impl AnimationValue {
    /// "Uncompute" this animation value in order to be used inside the CSS
    /// cascade.
    pub fn uncompute(&self) -> PropertyDeclaration {
        use properties::longhands;
        match *self {
                    AnimationValue::BackgroundColor(ref from) => {
                        PropertyDeclaration::BackgroundColor(
                                longhands::background_color::SpecifiedValue::from_computed_value(
                                    &ToAnimatedValue::from_animated_value(from.clone())
                                ))
                    }
                    AnimationValue::BackgroundImage(ref from) => {
                        PropertyDeclaration::BackgroundImage(
                                longhands::background_image::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::BackgroundPositionX(ref from) => {
                        PropertyDeclaration::BackgroundPositionX(
                                longhands::background_position_x::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::BackgroundPositionY(ref from) => {
                        PropertyDeclaration::BackgroundPositionY(
                                longhands::background_position_y::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::BackgroundRepeat(ref from) => {
                        PropertyDeclaration::BackgroundRepeat(
                                longhands::background_repeat::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::BackgroundAttachment(ref from) => {
                        PropertyDeclaration::BackgroundAttachment(
                                longhands::background_attachment::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::BackgroundClip(ref from) => {
                        PropertyDeclaration::BackgroundClip(
                                longhands::background_clip::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::BackgroundOrigin(ref from) => {
                        PropertyDeclaration::BackgroundOrigin(
                                longhands::background_origin::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::BackgroundSize(ref from) => {
                        PropertyDeclaration::BackgroundSize(
                                longhands::background_size::SpecifiedValue::from_computed_value(
                                    &ToAnimatedValue::from_animated_value(from.clone())
                                ))
                    }
                    AnimationValue::BackgroundBlendMode(ref from) => {
                        PropertyDeclaration::BackgroundBlendMode(
                                longhands::background_blend_mode::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::BorderTopColor(ref from) => {
                        PropertyDeclaration::BorderTopColor(
                                longhands::border_top_color::SpecifiedValue::from_computed_value(
                                    &ToAnimatedValue::from_animated_value(from.clone())
                                ))
                    }
                    AnimationValue::BorderTopStyle(ref from) => {
                        PropertyDeclaration::BorderTopStyle(
                                longhands::border_top_style::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::BorderTopWidth(ref from) => {
                        PropertyDeclaration::BorderTopWidth(
                                longhands::border_top_width::SpecifiedValue::from_computed_value(
                                    &ToAnimatedValue::from_animated_value(from.clone())
                                ))
                    }
                    AnimationValue::BorderLeftColor(ref from) => {
                        PropertyDeclaration::BorderLeftColor(
                                longhands::border_left_color::SpecifiedValue::from_computed_value(
                                    &ToAnimatedValue::from_animated_value(from.clone())
                                ))
                    }
                    AnimationValue::BorderLeftStyle(ref from) => {
                        PropertyDeclaration::BorderLeftStyle(
                                longhands::border_left_style::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::BorderLeftWidth(ref from) => {
                        PropertyDeclaration::BorderLeftWidth(
                                longhands::border_left_width::SpecifiedValue::from_computed_value(
                                    &ToAnimatedValue::from_animated_value(from.clone())
                                ))
                    }
                    AnimationValue::BorderBottomColor(ref from) => {
                        PropertyDeclaration::BorderBottomColor(
                                longhands::border_bottom_color::SpecifiedValue::from_computed_value(
                                    &ToAnimatedValue::from_animated_value(from.clone())
                                ))
                    }
                    AnimationValue::BorderBottomStyle(ref from) => {
                        PropertyDeclaration::BorderBottomStyle(
                                longhands::border_bottom_style::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::BorderBottomWidth(ref from) => {
                        PropertyDeclaration::BorderBottomWidth(
                                longhands::border_bottom_width::SpecifiedValue::from_computed_value(
                                    &ToAnimatedValue::from_animated_value(from.clone())
                                ))
                    }
                    AnimationValue::BorderRightColor(ref from) => {
                        PropertyDeclaration::BorderRightColor(
                                longhands::border_right_color::SpecifiedValue::from_computed_value(
                                    &ToAnimatedValue::from_animated_value(from.clone())
                                ))
                    }
                    AnimationValue::BorderRightStyle(ref from) => {
                        PropertyDeclaration::BorderRightStyle(
                                longhands::border_right_style::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::BorderRightWidth(ref from) => {
                        PropertyDeclaration::BorderRightWidth(
                                longhands::border_right_width::SpecifiedValue::from_computed_value(
                                    &ToAnimatedValue::from_animated_value(from.clone())
                                ))
                    }
                    AnimationValue::BorderTopLeftRadius(ref from) => {
                        PropertyDeclaration::BorderTopLeftRadius(
                            Box::new(
                                longhands::border_top_left_radius::SpecifiedValue::from_computed_value(
                                    &ToAnimatedValue::from_animated_value(from.clone())
                                ))
                            )
                    }
                    AnimationValue::BorderTopRightRadius(ref from) => {
                        PropertyDeclaration::BorderTopRightRadius(
                            Box::new(
                                longhands::border_top_right_radius::SpecifiedValue::from_computed_value(
                                    &ToAnimatedValue::from_animated_value(from.clone())
                                ))
                            )
                    }
                    AnimationValue::BorderBottomRightRadius(ref from) => {
                        PropertyDeclaration::BorderBottomRightRadius(
                            Box::new(
                                longhands::border_bottom_right_radius::SpecifiedValue::from_computed_value(
                                    &ToAnimatedValue::from_animated_value(from.clone())
                                ))
                            )
                    }
                    AnimationValue::BorderBottomLeftRadius(ref from) => {
                        PropertyDeclaration::BorderBottomLeftRadius(
                            Box::new(
                                longhands::border_bottom_left_radius::SpecifiedValue::from_computed_value(
                                    &ToAnimatedValue::from_animated_value(from.clone())
                                ))
                            )
                    }
                    AnimationValue::MozBorderTopColors(ref from) => {
                        PropertyDeclaration::MozBorderTopColors(
                                longhands::_moz_border_top_colors::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::MozBorderLeftColors(ref from) => {
                        PropertyDeclaration::MozBorderLeftColors(
                                longhands::_moz_border_left_colors::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::MozBorderBottomColors(ref from) => {
                        PropertyDeclaration::MozBorderBottomColors(
                                longhands::_moz_border_bottom_colors::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::MozBorderRightColors(ref from) => {
                        PropertyDeclaration::MozBorderRightColors(
                                longhands::_moz_border_right_colors::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::BoxDecorationBreak(ref from) => {
                        PropertyDeclaration::BoxDecorationBreak(
                                longhands::box_decoration_break::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::MozFloatEdge(ref from) => {
                        PropertyDeclaration::MozFloatEdge(
                                longhands::_moz_float_edge::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::BorderImageSource(ref from) => {
                        PropertyDeclaration::BorderImageSource(
                            Box::new(
                                longhands::border_image_source::SpecifiedValue::from_computed_value(
                                    from
                                ))
                            )
                    }
                    AnimationValue::BorderImageOutset(ref from) => {
                        PropertyDeclaration::BorderImageOutset(
                            Box::new(
                                longhands::border_image_outset::SpecifiedValue::from_computed_value(
                                    from
                                ))
                            )
                    }
                    AnimationValue::BorderImageRepeat(ref from) => {
                        PropertyDeclaration::BorderImageRepeat(
                                longhands::border_image_repeat::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::BorderImageWidth(ref from) => {
                        PropertyDeclaration::BorderImageWidth(
                            Box::new(
                                longhands::border_image_width::SpecifiedValue::from_computed_value(
                                    from
                                ))
                            )
                    }
                    AnimationValue::BorderImageSlice(ref from) => {
                        PropertyDeclaration::BorderImageSlice(
                            Box::new(
                                longhands::border_image_slice::SpecifiedValue::from_computed_value(
                                    from
                                ))
                            )
                    }
                    AnimationValue::Display(ref from) => {
                        PropertyDeclaration::Display(
                                longhands::display::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::Position(ref from) => {
                        PropertyDeclaration::Position(
                                longhands::position::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::Float(ref from) => {
                        PropertyDeclaration::Float(
                                longhands::float::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::Clear(ref from) => {
                        PropertyDeclaration::Clear(
                                longhands::clear::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::VerticalAlign(ref from) => {
                        PropertyDeclaration::VerticalAlign(
                                longhands::vertical_align::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::OverflowClipBox(ref from) => {
                        PropertyDeclaration::OverflowClipBox(
                                longhands::overflow_clip_box::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::OverflowX(ref from) => {
                        PropertyDeclaration::OverflowX(
                                longhands::overflow_x::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::OverflowY(ref from) => {
                        PropertyDeclaration::OverflowY(
                                longhands::overflow_y::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::ScrollSnapPointsX(ref from) => {
                        PropertyDeclaration::ScrollSnapPointsX(
                                longhands::scroll_snap_points_x::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::ScrollSnapPointsY(ref from) => {
                        PropertyDeclaration::ScrollSnapPointsY(
                                longhands::scroll_snap_points_y::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::ScrollSnapDestination(ref from) => {
                        PropertyDeclaration::ScrollSnapDestination(
                            Box::new(
                                longhands::scroll_snap_destination::SpecifiedValue::from_computed_value(
                                    from
                                ))
                            )
                    }
                    AnimationValue::ScrollSnapCoordinate(ref from) => {
                        PropertyDeclaration::ScrollSnapCoordinate(
                                longhands::scroll_snap_coordinate::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::Transform(ref from) => {
                        PropertyDeclaration::Transform(
                                longhands::transform::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::ScrollBehavior(ref from) => {
                        PropertyDeclaration::ScrollBehavior(
                                longhands::scroll_behavior::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::ScrollSnapTypeX(ref from) => {
                        PropertyDeclaration::ScrollSnapTypeX(
                                longhands::scroll_snap_type_x::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::ScrollSnapTypeY(ref from) => {
                        PropertyDeclaration::ScrollSnapTypeY(
                                longhands::scroll_snap_type_y::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::Isolation(ref from) => {
                        PropertyDeclaration::Isolation(
                                longhands::isolation::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::PageBreakAfter(ref from) => {
                        PropertyDeclaration::PageBreakAfter(
                                longhands::page_break_after::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::PageBreakBefore(ref from) => {
                        PropertyDeclaration::PageBreakBefore(
                                longhands::page_break_before::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::PageBreakInside(ref from) => {
                        PropertyDeclaration::PageBreakInside(
                                longhands::page_break_inside::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::Resize(ref from) => {
                        PropertyDeclaration::Resize(
                                longhands::resize::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::Perspective(ref from) => {
                        PropertyDeclaration::Perspective(
                                longhands::perspective::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::PerspectiveOrigin(ref from) => {
                        PropertyDeclaration::PerspectiveOrigin(
                            Box::new(
                                longhands::perspective_origin::SpecifiedValue::from_computed_value(
                                    from
                                ))
                            )
                    }
                    AnimationValue::BackfaceVisibility(ref from) => {
                        PropertyDeclaration::BackfaceVisibility(
                                longhands::backface_visibility::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::TransformBox(ref from) => {
                        PropertyDeclaration::TransformBox(
                                longhands::transform_box::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::TransformStyle(ref from) => {
                        PropertyDeclaration::TransformStyle(
                                longhands::transform_style::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::TransformOrigin(ref from) => {
                        PropertyDeclaration::TransformOrigin(
                            Box::new(
                                longhands::transform_origin::SpecifiedValue::from_computed_value(
                                    from
                                ))
                            )
                    }
                    AnimationValue::Contain(ref from) => {
                        PropertyDeclaration::Contain(
                                longhands::contain::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::MozAppearance(ref from) => {
                        PropertyDeclaration::MozAppearance(
                                longhands::_moz_appearance::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::MozOrient(ref from) => {
                        PropertyDeclaration::MozOrient(
                                longhands::_moz_orient::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::WillChange(ref from) => {
                        PropertyDeclaration::WillChange(
                                longhands::will_change::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::ShapeOutside(ref from) => {
                        PropertyDeclaration::ShapeOutside(
                            Box::new(
                                longhands::shape_outside::SpecifiedValue::from_computed_value(
                                    from
                                ))
                            )
                    }
                    AnimationValue::TouchAction(ref from) => {
                        PropertyDeclaration::TouchAction(
                                longhands::touch_action::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::Color(ref from) => {
                        PropertyDeclaration::Color(
                                longhands::color::SpecifiedValue::from_computed_value(
                                    &ToAnimatedValue::from_animated_value(from.clone())
                                ))
                    }
                    AnimationValue::ColumnWidth(ref from) => {
                        PropertyDeclaration::ColumnWidth(
                                longhands::column_width::SpecifiedValue::from_computed_value(
                                    &ToAnimatedValue::from_animated_value(from.clone())
                                ))
                    }
                    AnimationValue::ColumnCount(ref from) => {
                        PropertyDeclaration::ColumnCount(
                                longhands::column_count::SpecifiedValue::from_computed_value(
                                    &ToAnimatedValue::from_animated_value(from.clone())
                                ))
                    }
                    AnimationValue::ColumnGap(ref from) => {
                        PropertyDeclaration::ColumnGap(
                                longhands::column_gap::SpecifiedValue::from_computed_value(
                                    &ToAnimatedValue::from_animated_value(from.clone())
                                ))
                    }
                    AnimationValue::ColumnFill(ref from) => {
                        PropertyDeclaration::ColumnFill(
                                longhands::column_fill::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::ColumnRuleWidth(ref from) => {
                        PropertyDeclaration::ColumnRuleWidth(
                                longhands::column_rule_width::SpecifiedValue::from_computed_value(
                                    &ToAnimatedValue::from_animated_value(from.clone())
                                ))
                    }
                    AnimationValue::ColumnRuleColor(ref from) => {
                        PropertyDeclaration::ColumnRuleColor(
                                longhands::column_rule_color::SpecifiedValue::from_computed_value(
                                    &ToAnimatedValue::from_animated_value(from.clone())
                                ))
                    }
                    AnimationValue::ColumnSpan(ref from) => {
                        PropertyDeclaration::ColumnSpan(
                                longhands::column_span::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::ColumnRuleStyle(ref from) => {
                        PropertyDeclaration::ColumnRuleStyle(
                                longhands::column_rule_style::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::Content(ref from) => {
                        PropertyDeclaration::Content(
                            Box::new(
                                longhands::content::SpecifiedValue::from_computed_value(
                                    from
                                ))
                            )
                    }
                    AnimationValue::CounterIncrement(ref from) => {
                        PropertyDeclaration::CounterIncrement(
                                longhands::counter_increment::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::CounterReset(ref from) => {
                        PropertyDeclaration::CounterReset(
                                longhands::counter_reset::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::Opacity(ref from) => {
                        PropertyDeclaration::Opacity(
                                longhands::opacity::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::BoxShadow(ref from) => {
                        PropertyDeclaration::BoxShadow(
                                longhands::box_shadow::SpecifiedValue::from_computed_value(
                                    &ToAnimatedValue::from_animated_value(from.clone())
                                ))
                    }
                    AnimationValue::Clip(ref from) => {
                        PropertyDeclaration::Clip(
                            Box::new(
                                longhands::clip::SpecifiedValue::from_computed_value(
                                    from
                                ))
                            )
                    }
                    AnimationValue::Filter(ref from) => {
                        PropertyDeclaration::Filter(
                                longhands::filter::SpecifiedValue::from_computed_value(
                                    &ToAnimatedValue::from_animated_value(from.clone())
                                ))
                    }
                    AnimationValue::MixBlendMode(ref from) => {
                        PropertyDeclaration::MixBlendMode(
                                longhands::mix_blend_mode::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::FontFamily(ref from) => {
                        PropertyDeclaration::FontFamily(
                            Box::new(
                                longhands::font_family::SpecifiedValue::from_computed_value(
                                    from
                                ))
                            )
                    }
                    AnimationValue::FontStyle(ref from) => {
                        PropertyDeclaration::FontStyle(
                                longhands::font_style::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::FontVariantCaps(ref from) => {
                        PropertyDeclaration::FontVariantCaps(
                                longhands::font_variant_caps::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::FontWeight(ref from) => {
                        PropertyDeclaration::FontWeight(
                                longhands::font_weight::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::FontSize(ref from) => {
                        PropertyDeclaration::FontSize(
                                longhands::font_size::SpecifiedValue::from_computed_value(
                                    &ToAnimatedValue::from_animated_value(from.clone())
                                ))
                    }
                    AnimationValue::FontSizeAdjust(ref from) => {
                        PropertyDeclaration::FontSizeAdjust(
                                longhands::font_size_adjust::SpecifiedValue::from_computed_value(
                                    &ToAnimatedValue::from_animated_value(from.clone())
                                ))
                    }
                    AnimationValue::FontSynthesis(ref from) => {
                        PropertyDeclaration::FontSynthesis(
                                longhands::font_synthesis::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::FontStretch(ref from) => {
                        PropertyDeclaration::FontStretch(
                                longhands::font_stretch::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::FontKerning(ref from) => {
                        PropertyDeclaration::FontKerning(
                                longhands::font_kerning::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::FontVariantAlternates(ref from) => {
                        PropertyDeclaration::FontVariantAlternates(
                                longhands::font_variant_alternates::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::FontVariantEastAsian(ref from) => {
                        PropertyDeclaration::FontVariantEastAsian(
                                longhands::font_variant_east_asian::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::FontVariantLigatures(ref from) => {
                        PropertyDeclaration::FontVariantLigatures(
                                longhands::font_variant_ligatures::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::FontVariantNumeric(ref from) => {
                        PropertyDeclaration::FontVariantNumeric(
                                longhands::font_variant_numeric::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::FontVariantPosition(ref from) => {
                        PropertyDeclaration::FontVariantPosition(
                                longhands::font_variant_position::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::FontFeatureSettings(ref from) => {
                        PropertyDeclaration::FontFeatureSettings(
                            Box::new(
                                longhands::font_feature_settings::SpecifiedValue::from_computed_value(
                                    from
                                ))
                            )
                    }
                    AnimationValue::FontVariationSettings(ref from) => {
                        PropertyDeclaration::FontVariationSettings(
                                longhands::font_variation_settings::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::FontLanguageOverride(ref from) => {
                        PropertyDeclaration::FontLanguageOverride(
                            Box::new(
                                longhands::font_language_override::SpecifiedValue::from_computed_value(
                                    from
                                ))
                            )
                    }
                    AnimationValue::MozOsxFontSmoothing(ref from) => {
                        PropertyDeclaration::MozOsxFontSmoothing(
                                longhands::_moz_osx_font_smoothing::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::Visibility(ref from) => {
                        PropertyDeclaration::Visibility(
                                longhands::visibility::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::WritingMode(ref from) => {
                        PropertyDeclaration::WritingMode(
                                longhands::writing_mode::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::Direction(ref from) => {
                        PropertyDeclaration::Direction(
                                longhands::direction::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::TextOrientation(ref from) => {
                        PropertyDeclaration::TextOrientation(
                                longhands::text_orientation::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::ColorAdjust(ref from) => {
                        PropertyDeclaration::ColorAdjust(
                                longhands::color_adjust::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::ImageRendering(ref from) => {
                        PropertyDeclaration::ImageRendering(
                                longhands::image_rendering::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::ImageOrientation(ref from) => {
                        PropertyDeclaration::ImageOrientation(
                                longhands::image_orientation::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::BorderCollapse(ref from) => {
                        PropertyDeclaration::BorderCollapse(
                                longhands::border_collapse::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::EmptyCells(ref from) => {
                        PropertyDeclaration::EmptyCells(
                                longhands::empty_cells::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::CaptionSide(ref from) => {
                        PropertyDeclaration::CaptionSide(
                                longhands::caption_side::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::BorderSpacing(ref from) => {
                        PropertyDeclaration::BorderSpacing(
                            Box::new(
                                longhands::border_spacing::SpecifiedValue::from_computed_value(
                                    &ToAnimatedValue::from_animated_value(from.clone())
                                ))
                            )
                    }
                    AnimationValue::LineHeight(ref from) => {
                        PropertyDeclaration::LineHeight(
                                longhands::line_height::SpecifiedValue::from_computed_value(
                                    &ToAnimatedValue::from_animated_value(from.clone())
                                ))
                    }
                    AnimationValue::TextTransform(ref from) => {
                        PropertyDeclaration::TextTransform(
                                longhands::text_transform::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::Hyphens(ref from) => {
                        PropertyDeclaration::Hyphens(
                                longhands::hyphens::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::MozTextSizeAdjust(ref from) => {
                        PropertyDeclaration::MozTextSizeAdjust(
                                longhands::_moz_text_size_adjust::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::TextIndent(ref from) => {
                        PropertyDeclaration::TextIndent(
                                longhands::text_indent::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::OverflowWrap(ref from) => {
                        PropertyDeclaration::OverflowWrap(
                                longhands::overflow_wrap::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::WordBreak(ref from) => {
                        PropertyDeclaration::WordBreak(
                                longhands::word_break::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::TextJustify(ref from) => {
                        PropertyDeclaration::TextJustify(
                                longhands::text_justify::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::TextAlignLast(ref from) => {
                        PropertyDeclaration::TextAlignLast(
                                longhands::text_align_last::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::TextAlign(ref from) => {
                        PropertyDeclaration::TextAlign(
                                longhands::text_align::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::LetterSpacing(ref from) => {
                        PropertyDeclaration::LetterSpacing(
                                longhands::letter_spacing::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::WordSpacing(ref from) => {
                        PropertyDeclaration::WordSpacing(
                                longhands::word_spacing::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::WhiteSpace(ref from) => {
                        PropertyDeclaration::WhiteSpace(
                                longhands::white_space::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::TextShadow(ref from) => {
                        PropertyDeclaration::TextShadow(
                                longhands::text_shadow::SpecifiedValue::from_computed_value(
                                    &ToAnimatedValue::from_animated_value(from.clone())
                                ))
                    }
                    AnimationValue::TextEmphasisStyle(ref from) => {
                        PropertyDeclaration::TextEmphasisStyle(
                            Box::new(
                                longhands::text_emphasis_style::SpecifiedValue::from_computed_value(
                                    from
                                ))
                            )
                    }
                    AnimationValue::TextEmphasisPosition(ref from) => {
                        PropertyDeclaration::TextEmphasisPosition(
                                longhands::text_emphasis_position::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::TextEmphasisColor(ref from) => {
                        PropertyDeclaration::TextEmphasisColor(
                                longhands::text_emphasis_color::SpecifiedValue::from_computed_value(
                                    &ToAnimatedValue::from_animated_value(from.clone())
                                ))
                    }
                    AnimationValue::MozTabSize(ref from) => {
                        PropertyDeclaration::MozTabSize(
                                longhands::_moz_tab_size::SpecifiedValue::from_computed_value(
                                    &ToAnimatedValue::from_animated_value(from.clone())
                                ))
                    }
                    AnimationValue::WebkitTextFillColor(ref from) => {
                        PropertyDeclaration::WebkitTextFillColor(
                                longhands::_webkit_text_fill_color::SpecifiedValue::from_computed_value(
                                    &ToAnimatedValue::from_animated_value(from.clone())
                                ))
                    }
                    AnimationValue::WebkitTextStrokeColor(ref from) => {
                        PropertyDeclaration::WebkitTextStrokeColor(
                                longhands::_webkit_text_stroke_color::SpecifiedValue::from_computed_value(
                                    &ToAnimatedValue::from_animated_value(from.clone())
                                ))
                    }
                    AnimationValue::WebkitTextStrokeWidth(ref from) => {
                        PropertyDeclaration::WebkitTextStrokeWidth(
                                longhands::_webkit_text_stroke_width::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::RubyAlign(ref from) => {
                        PropertyDeclaration::RubyAlign(
                                longhands::ruby_align::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::RubyPosition(ref from) => {
                        PropertyDeclaration::RubyPosition(
                                longhands::ruby_position::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::TextCombineUpright(ref from) => {
                        PropertyDeclaration::TextCombineUpright(
                                longhands::text_combine_upright::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::TextRendering(ref from) => {
                        PropertyDeclaration::TextRendering(
                                longhands::text_rendering::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::ListStylePosition(ref from) => {
                        PropertyDeclaration::ListStylePosition(
                                longhands::list_style_position::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::ListStyleType(ref from) => {
                        PropertyDeclaration::ListStyleType(
                            Box::new(
                                longhands::list_style_type::SpecifiedValue::from_computed_value(
                                    from
                                ))
                            )
                    }
                    AnimationValue::ListStyleImage(ref from) => {
                        PropertyDeclaration::ListStyleImage(
                            Box::new(
                                longhands::list_style_image::SpecifiedValue::from_computed_value(
                                    from
                                ))
                            )
                    }
                    AnimationValue::Quotes(ref from) => {
                        PropertyDeclaration::Quotes(
                                longhands::quotes::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::MozImageRegion(ref from) => {
                        PropertyDeclaration::MozImageRegion(
                            Box::new(
                                longhands::_moz_image_region::SpecifiedValue::from_computed_value(
                                    from
                                ))
                            )
                    }
                    AnimationValue::MarginTop(ref from) => {
                        PropertyDeclaration::MarginTop(
                                longhands::margin_top::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::MarginLeft(ref from) => {
                        PropertyDeclaration::MarginLeft(
                                longhands::margin_left::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::MarginBottom(ref from) => {
                        PropertyDeclaration::MarginBottom(
                                longhands::margin_bottom::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::MarginRight(ref from) => {
                        PropertyDeclaration::MarginRight(
                                longhands::margin_right::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::OutlineColor(ref from) => {
                        PropertyDeclaration::OutlineColor(
                                longhands::outline_color::SpecifiedValue::from_computed_value(
                                    &ToAnimatedValue::from_animated_value(from.clone())
                                ))
                    }
                    AnimationValue::OutlineStyle(ref from) => {
                        PropertyDeclaration::OutlineStyle(
                                longhands::outline_style::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::OutlineWidth(ref from) => {
                        PropertyDeclaration::OutlineWidth(
                                longhands::outline_width::SpecifiedValue::from_computed_value(
                                    &ToAnimatedValue::from_animated_value(from.clone())
                                ))
                    }
                    AnimationValue::MozOutlineRadiusTopleft(ref from) => {
                        PropertyDeclaration::MozOutlineRadiusTopleft(
                            Box::new(
                                longhands::_moz_outline_radius_topleft::SpecifiedValue::from_computed_value(
                                    &ToAnimatedValue::from_animated_value(from.clone())
                                ))
                            )
                    }
                    AnimationValue::MozOutlineRadiusTopright(ref from) => {
                        PropertyDeclaration::MozOutlineRadiusTopright(
                            Box::new(
                                longhands::_moz_outline_radius_topright::SpecifiedValue::from_computed_value(
                                    &ToAnimatedValue::from_animated_value(from.clone())
                                ))
                            )
                    }
                    AnimationValue::MozOutlineRadiusBottomright(ref from) => {
                        PropertyDeclaration::MozOutlineRadiusBottomright(
                            Box::new(
                                longhands::_moz_outline_radius_bottomright::SpecifiedValue::from_computed_value(
                                    &ToAnimatedValue::from_animated_value(from.clone())
                                ))
                            )
                    }
                    AnimationValue::MozOutlineRadiusBottomleft(ref from) => {
                        PropertyDeclaration::MozOutlineRadiusBottomleft(
                            Box::new(
                                longhands::_moz_outline_radius_bottomleft::SpecifiedValue::from_computed_value(
                                    &ToAnimatedValue::from_animated_value(from.clone())
                                ))
                            )
                    }
                    AnimationValue::OutlineOffset(ref from) => {
                        PropertyDeclaration::OutlineOffset(
                                longhands::outline_offset::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::PaddingTop(ref from) => {
                        PropertyDeclaration::PaddingTop(
                                longhands::padding_top::SpecifiedValue::from_computed_value(
                                    &ToAnimatedValue::from_animated_value(from.clone())
                                ))
                    }
                    AnimationValue::PaddingLeft(ref from) => {
                        PropertyDeclaration::PaddingLeft(
                                longhands::padding_left::SpecifiedValue::from_computed_value(
                                    &ToAnimatedValue::from_animated_value(from.clone())
                                ))
                    }
                    AnimationValue::PaddingBottom(ref from) => {
                        PropertyDeclaration::PaddingBottom(
                                longhands::padding_bottom::SpecifiedValue::from_computed_value(
                                    &ToAnimatedValue::from_animated_value(from.clone())
                                ))
                    }
                    AnimationValue::PaddingRight(ref from) => {
                        PropertyDeclaration::PaddingRight(
                                longhands::padding_right::SpecifiedValue::from_computed_value(
                                    &ToAnimatedValue::from_animated_value(from.clone())
                                ))
                    }
                    AnimationValue::Cursor(ref from) => {
                        PropertyDeclaration::Cursor(
                            Box::new(
                                longhands::cursor::SpecifiedValue::from_computed_value(
                                    from
                                ))
                            )
                    }
                    AnimationValue::PointerEvents(ref from) => {
                        PropertyDeclaration::PointerEvents(
                                longhands::pointer_events::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::MozUserInput(ref from) => {
                        PropertyDeclaration::MozUserInput(
                                longhands::_moz_user_input::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::MozUserModify(ref from) => {
                        PropertyDeclaration::MozUserModify(
                                longhands::_moz_user_modify::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::MozUserFocus(ref from) => {
                        PropertyDeclaration::MozUserFocus(
                                longhands::_moz_user_focus::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::CaretColor(ref from) => {
                        PropertyDeclaration::CaretColor(
                            Box::new(
                                longhands::caret_color::SpecifiedValue::from_computed_value(
                                    &ToAnimatedValue::from_animated_value(from.clone())
                                ))
                            )
                    }
                    AnimationValue::Top(ref from) => {
                        PropertyDeclaration::Top(
                                longhands::top::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::Left(ref from) => {
                        PropertyDeclaration::Left(
                                longhands::left::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::Bottom(ref from) => {
                        PropertyDeclaration::Bottom(
                                longhands::bottom::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::Right(ref from) => {
                        PropertyDeclaration::Right(
                                longhands::right::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::ZIndex(ref from) => {
                        PropertyDeclaration::ZIndex(
                                longhands::z_index::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::FlexDirection(ref from) => {
                        PropertyDeclaration::FlexDirection(
                                longhands::flex_direction::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::FlexWrap(ref from) => {
                        PropertyDeclaration::FlexWrap(
                                longhands::flex_wrap::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::JustifyContent(ref from) => {
                        PropertyDeclaration::JustifyContent(
                                longhands::justify_content::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::AlignContent(ref from) => {
                        PropertyDeclaration::AlignContent(
                                longhands::align_content::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::AlignItems(ref from) => {
                        PropertyDeclaration::AlignItems(
                                longhands::align_items::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::JustifyItems(ref from) => {
                        PropertyDeclaration::JustifyItems(
                                longhands::justify_items::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::FlexGrow(ref from) => {
                        PropertyDeclaration::FlexGrow(
                                longhands::flex_grow::SpecifiedValue::from_computed_value(
                                    &ToAnimatedValue::from_animated_value(from.clone())
                                ))
                    }
                    AnimationValue::FlexShrink(ref from) => {
                        PropertyDeclaration::FlexShrink(
                                longhands::flex_shrink::SpecifiedValue::from_computed_value(
                                    &ToAnimatedValue::from_animated_value(from.clone())
                                ))
                    }
                    AnimationValue::AlignSelf(ref from) => {
                        PropertyDeclaration::AlignSelf(
                                longhands::align_self::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::JustifySelf(ref from) => {
                        PropertyDeclaration::JustifySelf(
                                longhands::justify_self::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::Order(ref from) => {
                        PropertyDeclaration::Order(
                                longhands::order::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::FlexBasis(ref from) => {
                        PropertyDeclaration::FlexBasis(
                                longhands::flex_basis::SpecifiedValue::from_computed_value(
                                    &ToAnimatedValue::from_animated_value(from.clone())
                                ))
                    }
                    AnimationValue::Width(ref from) => {
                        PropertyDeclaration::Width(
                                longhands::width::SpecifiedValue::from_computed_value(
                                    &ToAnimatedValue::from_animated_value(from.clone())
                                ))
                    }
                    AnimationValue::MinWidth(ref from) => {
                        PropertyDeclaration::MinWidth(
                                longhands::min_width::SpecifiedValue::from_computed_value(
                                    &ToAnimatedValue::from_animated_value(from.clone())
                                ))
                    }
                    AnimationValue::MaxWidth(ref from) => {
                        PropertyDeclaration::MaxWidth(
                                longhands::max_width::SpecifiedValue::from_computed_value(
                                    &ToAnimatedValue::from_animated_value(from.clone())
                                ))
                    }
                    AnimationValue::Height(ref from) => {
                        PropertyDeclaration::Height(
                                longhands::height::SpecifiedValue::from_computed_value(
                                    &ToAnimatedValue::from_animated_value(from.clone())
                                ))
                    }
                    AnimationValue::MinHeight(ref from) => {
                        PropertyDeclaration::MinHeight(
                                longhands::min_height::SpecifiedValue::from_computed_value(
                                    &ToAnimatedValue::from_animated_value(from.clone())
                                ))
                    }
                    AnimationValue::MaxHeight(ref from) => {
                        PropertyDeclaration::MaxHeight(
                                longhands::max_height::SpecifiedValue::from_computed_value(
                                    &ToAnimatedValue::from_animated_value(from.clone())
                                ))
                    }
                    AnimationValue::BoxSizing(ref from) => {
                        PropertyDeclaration::BoxSizing(
                                longhands::box_sizing::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::ObjectFit(ref from) => {
                        PropertyDeclaration::ObjectFit(
                                longhands::object_fit::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::ObjectPosition(ref from) => {
                        PropertyDeclaration::ObjectPosition(
                            Box::new(
                                longhands::object_position::SpecifiedValue::from_computed_value(
                                    from
                                ))
                            )
                    }
                    AnimationValue::GridRowGap(ref from) => {
                        PropertyDeclaration::GridRowGap(
                                longhands::grid_row_gap::SpecifiedValue::from_computed_value(
                                    &ToAnimatedValue::from_animated_value(from.clone())
                                ))
                    }
                    AnimationValue::GridRowStart(ref from) => {
                        PropertyDeclaration::GridRowStart(
                            Box::new(
                                longhands::grid_row_start::SpecifiedValue::from_computed_value(
                                    from
                                ))
                            )
                    }
                    AnimationValue::GridRowEnd(ref from) => {
                        PropertyDeclaration::GridRowEnd(
                            Box::new(
                                longhands::grid_row_end::SpecifiedValue::from_computed_value(
                                    from
                                ))
                            )
                    }
                    AnimationValue::GridAutoRows(ref from) => {
                        PropertyDeclaration::GridAutoRows(
                            Box::new(
                                longhands::grid_auto_rows::SpecifiedValue::from_computed_value(
                                    from
                                ))
                            )
                    }
                    AnimationValue::GridTemplateRows(ref from) => {
                        PropertyDeclaration::GridTemplateRows(
                            Box::new(
                                longhands::grid_template_rows::SpecifiedValue::from_computed_value(
                                    from
                                ))
                            )
                    }
                    AnimationValue::GridColumnGap(ref from) => {
                        PropertyDeclaration::GridColumnGap(
                                longhands::grid_column_gap::SpecifiedValue::from_computed_value(
                                    &ToAnimatedValue::from_animated_value(from.clone())
                                ))
                    }
                    AnimationValue::GridColumnStart(ref from) => {
                        PropertyDeclaration::GridColumnStart(
                            Box::new(
                                longhands::grid_column_start::SpecifiedValue::from_computed_value(
                                    from
                                ))
                            )
                    }
                    AnimationValue::GridColumnEnd(ref from) => {
                        PropertyDeclaration::GridColumnEnd(
                            Box::new(
                                longhands::grid_column_end::SpecifiedValue::from_computed_value(
                                    from
                                ))
                            )
                    }
                    AnimationValue::GridAutoColumns(ref from) => {
                        PropertyDeclaration::GridAutoColumns(
                            Box::new(
                                longhands::grid_auto_columns::SpecifiedValue::from_computed_value(
                                    from
                                ))
                            )
                    }
                    AnimationValue::GridTemplateColumns(ref from) => {
                        PropertyDeclaration::GridTemplateColumns(
                            Box::new(
                                longhands::grid_template_columns::SpecifiedValue::from_computed_value(
                                    from
                                ))
                            )
                    }
                    AnimationValue::GridAutoFlow(ref from) => {
                        PropertyDeclaration::GridAutoFlow(
                                longhands::grid_auto_flow::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::GridTemplateAreas(ref from) => {
                        PropertyDeclaration::GridTemplateAreas(
                            Box::new(
                                longhands::grid_template_areas::SpecifiedValue::from_computed_value(
                                    from
                                ))
                            )
                    }
                    AnimationValue::TableLayout(ref from) => {
                        PropertyDeclaration::TableLayout(
                                longhands::table_layout::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::TextOverflow(ref from) => {
                        PropertyDeclaration::TextOverflow(
                            Box::new(
                                longhands::text_overflow::SpecifiedValue::from_computed_value(
                                    from
                                ))
                            )
                    }
                    AnimationValue::UnicodeBidi(ref from) => {
                        PropertyDeclaration::UnicodeBidi(
                                longhands::unicode_bidi::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::TextDecorationLine(ref from) => {
                        PropertyDeclaration::TextDecorationLine(
                                longhands::text_decoration_line::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::TextDecorationStyle(ref from) => {
                        PropertyDeclaration::TextDecorationStyle(
                                longhands::text_decoration_style::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::TextDecorationColor(ref from) => {
                        PropertyDeclaration::TextDecorationColor(
                                longhands::text_decoration_color::SpecifiedValue::from_computed_value(
                                    &ToAnimatedValue::from_animated_value(from.clone())
                                ))
                    }
                    AnimationValue::InitialLetter(ref from) => {
                        PropertyDeclaration::InitialLetter(
                                longhands::initial_letter::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::ImeMode(ref from) => {
                        PropertyDeclaration::ImeMode(
                                longhands::ime_mode::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::MozUserSelect(ref from) => {
                        PropertyDeclaration::MozUserSelect(
                                longhands::_moz_user_select::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::MozWindowDragging(ref from) => {
                        PropertyDeclaration::MozWindowDragging(
                                longhands::_moz_window_dragging::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::MozWindowShadow(ref from) => {
                        PropertyDeclaration::MozWindowShadow(
                                longhands::_moz_window_shadow::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::MozForceBrokenImageIcon(ref from) => {
                        PropertyDeclaration::MozForceBrokenImageIcon(
                                longhands::_moz_force_broken_image_icon::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::TextAnchor(ref from) => {
                        PropertyDeclaration::TextAnchor(
                                longhands::text_anchor::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::ColorInterpolation(ref from) => {
                        PropertyDeclaration::ColorInterpolation(
                                longhands::color_interpolation::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::ColorInterpolationFilters(ref from) => {
                        PropertyDeclaration::ColorInterpolationFilters(
                                longhands::color_interpolation_filters::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::Fill(ref from) => {
                        PropertyDeclaration::Fill(
                            Box::new(
                                longhands::fill::SpecifiedValue::from_computed_value(
                                    &ToAnimatedValue::from_animated_value(from.clone())
                                ))
                            )
                    }
                    AnimationValue::FillOpacity(ref from) => {
                        PropertyDeclaration::FillOpacity(
                                longhands::fill_opacity::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::FillRule(ref from) => {
                        PropertyDeclaration::FillRule(
                                longhands::fill_rule::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::ShapeRendering(ref from) => {
                        PropertyDeclaration::ShapeRendering(
                                longhands::shape_rendering::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::Stroke(ref from) => {
                        PropertyDeclaration::Stroke(
                            Box::new(
                                longhands::stroke::SpecifiedValue::from_computed_value(
                                    &ToAnimatedValue::from_animated_value(from.clone())
                                ))
                            )
                    }
                    AnimationValue::StrokeWidth(ref from) => {
                        PropertyDeclaration::StrokeWidth(
                            Box::new(
                                longhands::stroke_width::SpecifiedValue::from_computed_value(
                                    &ToAnimatedValue::from_animated_value(from.clone())
                                ))
                            )
                    }
                    AnimationValue::StrokeLinecap(ref from) => {
                        PropertyDeclaration::StrokeLinecap(
                                longhands::stroke_linecap::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::StrokeLinejoin(ref from) => {
                        PropertyDeclaration::StrokeLinejoin(
                                longhands::stroke_linejoin::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::StrokeMiterlimit(ref from) => {
                        PropertyDeclaration::StrokeMiterlimit(
                                longhands::stroke_miterlimit::SpecifiedValue::from_computed_value(
                                    &ToAnimatedValue::from_animated_value(from.clone())
                                ))
                    }
                    AnimationValue::StrokeOpacity(ref from) => {
                        PropertyDeclaration::StrokeOpacity(
                                longhands::stroke_opacity::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::StrokeDasharray(ref from) => {
                        PropertyDeclaration::StrokeDasharray(
                                longhands::stroke_dasharray::SpecifiedValue::from_computed_value(
                                    &ToAnimatedValue::from_animated_value(from.clone())
                                ))
                    }
                    AnimationValue::StrokeDashoffset(ref from) => {
                        PropertyDeclaration::StrokeDashoffset(
                            Box::new(
                                longhands::stroke_dashoffset::SpecifiedValue::from_computed_value(
                                    from
                                ))
                            )
                    }
                    AnimationValue::ClipRule(ref from) => {
                        PropertyDeclaration::ClipRule(
                                longhands::clip_rule::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::MarkerStart(ref from) => {
                        PropertyDeclaration::MarkerStart(
                            Box::new(
                                longhands::marker_start::SpecifiedValue::from_computed_value(
                                    from
                                ))
                            )
                    }
                    AnimationValue::MarkerMid(ref from) => {
                        PropertyDeclaration::MarkerMid(
                            Box::new(
                                longhands::marker_mid::SpecifiedValue::from_computed_value(
                                    from
                                ))
                            )
                    }
                    AnimationValue::MarkerEnd(ref from) => {
                        PropertyDeclaration::MarkerEnd(
                            Box::new(
                                longhands::marker_end::SpecifiedValue::from_computed_value(
                                    from
                                ))
                            )
                    }
                    AnimationValue::PaintOrder(ref from) => {
                        PropertyDeclaration::PaintOrder(
                                longhands::paint_order::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::DominantBaseline(ref from) => {
                        PropertyDeclaration::DominantBaseline(
                                longhands::dominant_baseline::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::VectorEffect(ref from) => {
                        PropertyDeclaration::VectorEffect(
                                longhands::vector_effect::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::StopColor(ref from) => {
                        PropertyDeclaration::StopColor(
                                longhands::stop_color::SpecifiedValue::from_computed_value(
                                    &ToAnimatedValue::from_animated_value(from.clone())
                                ))
                    }
                    AnimationValue::StopOpacity(ref from) => {
                        PropertyDeclaration::StopOpacity(
                                longhands::stop_opacity::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::FloodColor(ref from) => {
                        PropertyDeclaration::FloodColor(
                                longhands::flood_color::SpecifiedValue::from_computed_value(
                                    &ToAnimatedValue::from_animated_value(from.clone())
                                ))
                    }
                    AnimationValue::FloodOpacity(ref from) => {
                        PropertyDeclaration::FloodOpacity(
                                longhands::flood_opacity::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::LightingColor(ref from) => {
                        PropertyDeclaration::LightingColor(
                                longhands::lighting_color::SpecifiedValue::from_computed_value(
                                    &ToAnimatedValue::from_animated_value(from.clone())
                                ))
                    }
                    AnimationValue::MaskType(ref from) => {
                        PropertyDeclaration::MaskType(
                                longhands::mask_type::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::ClipPath(ref from) => {
                        PropertyDeclaration::ClipPath(
                            Box::new(
                                longhands::clip_path::SpecifiedValue::from_computed_value(
                                    from
                                ))
                            )
                    }
                    AnimationValue::MaskMode(ref from) => {
                        PropertyDeclaration::MaskMode(
                                longhands::mask_mode::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::MaskRepeat(ref from) => {
                        PropertyDeclaration::MaskRepeat(
                                longhands::mask_repeat::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::MaskPositionX(ref from) => {
                        PropertyDeclaration::MaskPositionX(
                                longhands::mask_position_x::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::MaskPositionY(ref from) => {
                        PropertyDeclaration::MaskPositionY(
                                longhands::mask_position_y::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::MaskClip(ref from) => {
                        PropertyDeclaration::MaskClip(
                                longhands::mask_clip::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::MaskOrigin(ref from) => {
                        PropertyDeclaration::MaskOrigin(
                                longhands::mask_origin::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::MaskSize(ref from) => {
                        PropertyDeclaration::MaskSize(
                                longhands::mask_size::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::MaskComposite(ref from) => {
                        PropertyDeclaration::MaskComposite(
                                longhands::mask_composite::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::MaskImage(ref from) => {
                        PropertyDeclaration::MaskImage(
                                longhands::mask_image::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::MozBoxAlign(ref from) => {
                        PropertyDeclaration::MozBoxAlign(
                                longhands::_moz_box_align::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::MozBoxDirection(ref from) => {
                        PropertyDeclaration::MozBoxDirection(
                                longhands::_moz_box_direction::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::MozBoxFlex(ref from) => {
                        PropertyDeclaration::MozBoxFlex(
                                longhands::_moz_box_flex::SpecifiedValue::from_computed_value(
                                    &ToAnimatedValue::from_animated_value(from.clone())
                                ))
                    }
                    AnimationValue::MozBoxOrient(ref from) => {
                        PropertyDeclaration::MozBoxOrient(
                                longhands::_moz_box_orient::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::MozBoxPack(ref from) => {
                        PropertyDeclaration::MozBoxPack(
                                longhands::_moz_box_pack::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::MozStackSizing(ref from) => {
                        PropertyDeclaration::MozStackSizing(
                                longhands::_moz_stack_sizing::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
                    AnimationValue::MozBoxOrdinalGroup(ref from) => {
                        PropertyDeclaration::MozBoxOrdinalGroup(
                                longhands::_moz_box_ordinal_group::SpecifiedValue::from_computed_value(
                                    from
                                ))
                    }
        }
    }

    /// Construct an AnimationValue from a property declaration.
    pub fn from_declaration(
        decl: &PropertyDeclaration,
        context: &mut Context,
        extra_custom_properties: &Option<Arc<::custom_properties::CustomPropertiesMap>>,
        initial: &ComputedValues
    ) -> Option<Self> {
        use properties::LonghandId;

        match *decl {
            PropertyDeclaration::BackgroundColor(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::BackgroundColor);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::BackgroundColor(
                computed.to_animated_value()
            ))
            },
            PropertyDeclaration::BackgroundImage(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::BackgroundImage);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::BackgroundImage(
                computed
            ))
            },
            PropertyDeclaration::BackgroundPositionX(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::BackgroundPositionX);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::BackgroundPositionX(
                computed
            ))
            },
            PropertyDeclaration::BackgroundPositionY(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::BackgroundPositionY);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::BackgroundPositionY(
                computed
            ))
            },
            PropertyDeclaration::BackgroundRepeat(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::BackgroundRepeat);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::BackgroundRepeat(
                computed
            ))
            },
            PropertyDeclaration::BackgroundAttachment(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::BackgroundAttachment);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::BackgroundAttachment(
                computed
            ))
            },
            PropertyDeclaration::BackgroundClip(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::BackgroundClip);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::BackgroundClip(
                computed
            ))
            },
            PropertyDeclaration::BackgroundOrigin(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::BackgroundOrigin);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::BackgroundOrigin(
                computed
            ))
            },
            PropertyDeclaration::BackgroundSize(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::BackgroundSize);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::BackgroundSize(
                computed.to_animated_value()
            ))
            },
            PropertyDeclaration::BackgroundBlendMode(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::BackgroundBlendMode);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::BackgroundBlendMode(
                computed
            ))
            },
            PropertyDeclaration::BorderTopColor(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::BorderTopColor);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::BorderTopColor(
                computed.to_animated_value()
            ))
            },
            PropertyDeclaration::BorderTopStyle(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::BorderTopStyle);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::BorderTopStyle(
                computed
            ))
            },
            PropertyDeclaration::BorderTopWidth(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::BorderTopWidth);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::BorderTopWidth(
                computed.to_animated_value()
            ))
            },
            PropertyDeclaration::BorderLeftColor(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::BorderLeftColor);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::BorderLeftColor(
                computed.to_animated_value()
            ))
            },
            PropertyDeclaration::BorderLeftStyle(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::BorderLeftStyle);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::BorderLeftStyle(
                computed
            ))
            },
            PropertyDeclaration::BorderLeftWidth(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::BorderLeftWidth);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::BorderLeftWidth(
                computed.to_animated_value()
            ))
            },
            PropertyDeclaration::BorderBottomColor(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::BorderBottomColor);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::BorderBottomColor(
                computed.to_animated_value()
            ))
            },
            PropertyDeclaration::BorderBottomStyle(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::BorderBottomStyle);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::BorderBottomStyle(
                computed
            ))
            },
            PropertyDeclaration::BorderBottomWidth(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::BorderBottomWidth);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::BorderBottomWidth(
                computed.to_animated_value()
            ))
            },
            PropertyDeclaration::BorderRightColor(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::BorderRightColor);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::BorderRightColor(
                computed.to_animated_value()
            ))
            },
            PropertyDeclaration::BorderRightStyle(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::BorderRightStyle);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::BorderRightStyle(
                computed
            ))
            },
            PropertyDeclaration::BorderRightWidth(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::BorderRightWidth);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::BorderRightWidth(
                computed.to_animated_value()
            ))
            },
            PropertyDeclaration::BorderTopLeftRadius(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::BorderTopLeftRadius);
            let computed = (**val).to_computed_value(context);
            Some(AnimationValue::BorderTopLeftRadius(
                computed.to_animated_value()
            ))
            },
            PropertyDeclaration::BorderTopRightRadius(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::BorderTopRightRadius);
            let computed = (**val).to_computed_value(context);
            Some(AnimationValue::BorderTopRightRadius(
                computed.to_animated_value()
            ))
            },
            PropertyDeclaration::BorderBottomRightRadius(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::BorderBottomRightRadius);
            let computed = (**val).to_computed_value(context);
            Some(AnimationValue::BorderBottomRightRadius(
                computed.to_animated_value()
            ))
            },
            PropertyDeclaration::BorderBottomLeftRadius(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::BorderBottomLeftRadius);
            let computed = (**val).to_computed_value(context);
            Some(AnimationValue::BorderBottomLeftRadius(
                computed.to_animated_value()
            ))
            },
            PropertyDeclaration::MozBorderTopColors(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::MozBorderTopColors);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::MozBorderTopColors(
                computed
            ))
            },
            PropertyDeclaration::MozBorderLeftColors(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::MozBorderLeftColors);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::MozBorderLeftColors(
                computed
            ))
            },
            PropertyDeclaration::MozBorderBottomColors(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::MozBorderBottomColors);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::MozBorderBottomColors(
                computed
            ))
            },
            PropertyDeclaration::MozBorderRightColors(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::MozBorderRightColors);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::MozBorderRightColors(
                computed
            ))
            },
            PropertyDeclaration::BoxDecorationBreak(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::BoxDecorationBreak);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::BoxDecorationBreak(
                computed
            ))
            },
            PropertyDeclaration::MozFloatEdge(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::MozFloatEdge);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::MozFloatEdge(
                computed
            ))
            },
            PropertyDeclaration::BorderImageSource(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::BorderImageSource);
            let computed = (**val).to_computed_value(context);
            Some(AnimationValue::BorderImageSource(
                computed
            ))
            },
            PropertyDeclaration::BorderImageOutset(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::BorderImageOutset);
            let computed = (**val).to_computed_value(context);
            Some(AnimationValue::BorderImageOutset(
                computed
            ))
            },
            PropertyDeclaration::BorderImageRepeat(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::BorderImageRepeat);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::BorderImageRepeat(
                computed
            ))
            },
            PropertyDeclaration::BorderImageWidth(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::BorderImageWidth);
            let computed = (**val).to_computed_value(context);
            Some(AnimationValue::BorderImageWidth(
                computed
            ))
            },
            PropertyDeclaration::BorderImageSlice(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::BorderImageSlice);
            let computed = (**val).to_computed_value(context);
            Some(AnimationValue::BorderImageSlice(
                computed
            ))
            },
            PropertyDeclaration::Display(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::Display);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::Display(
                computed
            ))
            },
            PropertyDeclaration::Position(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::Position);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::Position(
                computed
            ))
            },
            PropertyDeclaration::Float(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::Float);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::Float(
                computed
            ))
            },
            PropertyDeclaration::Clear(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::Clear);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::Clear(
                computed
            ))
            },
            PropertyDeclaration::VerticalAlign(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::VerticalAlign);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::VerticalAlign(
                computed
            ))
            },
            PropertyDeclaration::OverflowClipBox(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::OverflowClipBox);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::OverflowClipBox(
                computed
            ))
            },
            PropertyDeclaration::OverflowX(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::OverflowX);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::OverflowX(
                computed
            ))
            },
            PropertyDeclaration::OverflowY(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::OverflowY);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::OverflowY(
                computed
            ))
            },
            PropertyDeclaration::ScrollSnapPointsX(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::ScrollSnapPointsX);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::ScrollSnapPointsX(
                computed
            ))
            },
            PropertyDeclaration::ScrollSnapPointsY(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::ScrollSnapPointsY);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::ScrollSnapPointsY(
                computed
            ))
            },
            PropertyDeclaration::ScrollSnapDestination(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::ScrollSnapDestination);
            let computed = (**val).to_computed_value(context);
            Some(AnimationValue::ScrollSnapDestination(
                computed
            ))
            },
            PropertyDeclaration::ScrollSnapCoordinate(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::ScrollSnapCoordinate);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::ScrollSnapCoordinate(
                computed
            ))
            },
            PropertyDeclaration::Transform(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::Transform);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::Transform(
                computed
            ))
            },
            PropertyDeclaration::ScrollBehavior(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::ScrollBehavior);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::ScrollBehavior(
                computed
            ))
            },
            PropertyDeclaration::ScrollSnapTypeX(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::ScrollSnapTypeX);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::ScrollSnapTypeX(
                computed
            ))
            },
            PropertyDeclaration::ScrollSnapTypeY(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::ScrollSnapTypeY);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::ScrollSnapTypeY(
                computed
            ))
            },
            PropertyDeclaration::Isolation(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::Isolation);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::Isolation(
                computed
            ))
            },
            PropertyDeclaration::PageBreakAfter(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::PageBreakAfter);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::PageBreakAfter(
                computed
            ))
            },
            PropertyDeclaration::PageBreakBefore(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::PageBreakBefore);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::PageBreakBefore(
                computed
            ))
            },
            PropertyDeclaration::PageBreakInside(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::PageBreakInside);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::PageBreakInside(
                computed
            ))
            },
            PropertyDeclaration::Resize(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::Resize);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::Resize(
                computed
            ))
            },
            PropertyDeclaration::Perspective(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::Perspective);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::Perspective(
                computed
            ))
            },
            PropertyDeclaration::PerspectiveOrigin(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::PerspectiveOrigin);
            let computed = (**val).to_computed_value(context);
            Some(AnimationValue::PerspectiveOrigin(
                computed
            ))
            },
            PropertyDeclaration::BackfaceVisibility(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::BackfaceVisibility);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::BackfaceVisibility(
                computed
            ))
            },
            PropertyDeclaration::TransformBox(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::TransformBox);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::TransformBox(
                computed
            ))
            },
            PropertyDeclaration::TransformStyle(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::TransformStyle);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::TransformStyle(
                computed
            ))
            },
            PropertyDeclaration::TransformOrigin(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::TransformOrigin);
            let computed = (**val).to_computed_value(context);
            Some(AnimationValue::TransformOrigin(
                computed
            ))
            },
            PropertyDeclaration::Contain(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::Contain);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::Contain(
                computed
            ))
            },
            PropertyDeclaration::MozAppearance(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::MozAppearance);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::MozAppearance(
                computed
            ))
            },
            PropertyDeclaration::MozOrient(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::MozOrient);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::MozOrient(
                computed
            ))
            },
            PropertyDeclaration::WillChange(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::WillChange);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::WillChange(
                computed
            ))
            },
            PropertyDeclaration::ShapeOutside(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::ShapeOutside);
            let computed = (**val).to_computed_value(context);
            Some(AnimationValue::ShapeOutside(
                computed
            ))
            },
            PropertyDeclaration::TouchAction(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::TouchAction);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::TouchAction(
                computed
            ))
            },
            PropertyDeclaration::Color(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = val.to_computed_value(context);
            Some(AnimationValue::Color(
                computed.to_animated_value()
            ))
            },
            PropertyDeclaration::ColumnWidth(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::ColumnWidth);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::ColumnWidth(
                computed.to_animated_value()
            ))
            },
            PropertyDeclaration::ColumnCount(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::ColumnCount);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::ColumnCount(
                computed.to_animated_value()
            ))
            },
            PropertyDeclaration::ColumnGap(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::ColumnGap);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::ColumnGap(
                computed.to_animated_value()
            ))
            },
            PropertyDeclaration::ColumnFill(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::ColumnFill);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::ColumnFill(
                computed
            ))
            },
            PropertyDeclaration::ColumnRuleWidth(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::ColumnRuleWidth);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::ColumnRuleWidth(
                computed.to_animated_value()
            ))
            },
            PropertyDeclaration::ColumnRuleColor(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::ColumnRuleColor);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::ColumnRuleColor(
                computed.to_animated_value()
            ))
            },
            PropertyDeclaration::ColumnSpan(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::ColumnSpan);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::ColumnSpan(
                computed
            ))
            },
            PropertyDeclaration::ColumnRuleStyle(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::ColumnRuleStyle);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::ColumnRuleStyle(
                computed
            ))
            },
            PropertyDeclaration::Content(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::Content);
            let computed = (**val).to_computed_value(context);
            Some(AnimationValue::Content(
                computed
            ))
            },
            PropertyDeclaration::CounterIncrement(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::CounterIncrement);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::CounterIncrement(
                computed
            ))
            },
            PropertyDeclaration::CounterReset(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::CounterReset);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::CounterReset(
                computed
            ))
            },
            PropertyDeclaration::Opacity(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::Opacity);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::Opacity(
                computed
            ))
            },
            PropertyDeclaration::BoxShadow(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::BoxShadow);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::BoxShadow(
                computed.to_animated_value()
            ))
            },
            PropertyDeclaration::Clip(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::Clip);
            let computed = (**val).to_computed_value(context);
            Some(AnimationValue::Clip(
                computed
            ))
            },
            PropertyDeclaration::Filter(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::Filter);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::Filter(
                computed.to_animated_value()
            ))
            },
            PropertyDeclaration::MixBlendMode(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::MixBlendMode);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::MixBlendMode(
                computed
            ))
            },
            PropertyDeclaration::FontFamily(ref val) => {
                context.for_non_inherited_property =
                        None;
                if let Some(sf) = val.get_system() {
                    longhands::system_font::resolve_system_font(sf, context);
                }
            let computed = (**val).to_computed_value(context);
            Some(AnimationValue::FontFamily(
                computed
            ))
            },
            PropertyDeclaration::FontStyle(ref val) => {
                context.for_non_inherited_property =
                        None;
                if let Some(sf) = val.get_system() {
                    longhands::system_font::resolve_system_font(sf, context);
                }
            let computed = val.to_computed_value(context);
            Some(AnimationValue::FontStyle(
                computed
            ))
            },
            PropertyDeclaration::FontVariantCaps(ref val) => {
                context.for_non_inherited_property =
                        None;
                if let Some(sf) = val.get_system() {
                    longhands::system_font::resolve_system_font(sf, context);
                }
            let computed = val.to_computed_value(context);
            Some(AnimationValue::FontVariantCaps(
                computed
            ))
            },
            PropertyDeclaration::FontWeight(ref val) => {
                context.for_non_inherited_property =
                        None;
                if let Some(sf) = val.get_system() {
                    longhands::system_font::resolve_system_font(sf, context);
                }
            let computed = val.to_computed_value(context);
            Some(AnimationValue::FontWeight(
                computed
            ))
            },
            PropertyDeclaration::FontSize(ref val) => {
                context.for_non_inherited_property =
                        None;
                if let Some(sf) = val.get_system() {
                    longhands::system_font::resolve_system_font(sf, context);
                }
            let computed = val.to_computed_value(context);
            Some(AnimationValue::FontSize(
                computed.to_animated_value()
            ))
            },
            PropertyDeclaration::FontSizeAdjust(ref val) => {
                context.for_non_inherited_property =
                        None;
                if let Some(sf) = val.get_system() {
                    longhands::system_font::resolve_system_font(sf, context);
                }
            let computed = val.to_computed_value(context);
            Some(AnimationValue::FontSizeAdjust(
                computed.to_animated_value()
            ))
            },
            PropertyDeclaration::FontSynthesis(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = val.to_computed_value(context);
            Some(AnimationValue::FontSynthesis(
                computed
            ))
            },
            PropertyDeclaration::FontStretch(ref val) => {
                context.for_non_inherited_property =
                        None;
                if let Some(sf) = val.get_system() {
                    longhands::system_font::resolve_system_font(sf, context);
                }
            let computed = val.to_computed_value(context);
            Some(AnimationValue::FontStretch(
                computed
            ))
            },
            PropertyDeclaration::FontKerning(ref val) => {
                context.for_non_inherited_property =
                        None;
                if let Some(sf) = val.get_system() {
                    longhands::system_font::resolve_system_font(sf, context);
                }
            let computed = val.to_computed_value(context);
            Some(AnimationValue::FontKerning(
                computed
            ))
            },
            PropertyDeclaration::FontVariantAlternates(ref val) => {
                context.for_non_inherited_property =
                        None;
                if let Some(sf) = val.get_system() {
                    longhands::system_font::resolve_system_font(sf, context);
                }
            let computed = val.to_computed_value(context);
            Some(AnimationValue::FontVariantAlternates(
                computed
            ))
            },
            PropertyDeclaration::FontVariantEastAsian(ref val) => {
                context.for_non_inherited_property =
                        None;
                if let Some(sf) = val.get_system() {
                    longhands::system_font::resolve_system_font(sf, context);
                }
            let computed = val.to_computed_value(context);
            Some(AnimationValue::FontVariantEastAsian(
                computed
            ))
            },
            PropertyDeclaration::FontVariantLigatures(ref val) => {
                context.for_non_inherited_property =
                        None;
                if let Some(sf) = val.get_system() {
                    longhands::system_font::resolve_system_font(sf, context);
                }
            let computed = val.to_computed_value(context);
            Some(AnimationValue::FontVariantLigatures(
                computed
            ))
            },
            PropertyDeclaration::FontVariantNumeric(ref val) => {
                context.for_non_inherited_property =
                        None;
                if let Some(sf) = val.get_system() {
                    longhands::system_font::resolve_system_font(sf, context);
                }
            let computed = val.to_computed_value(context);
            Some(AnimationValue::FontVariantNumeric(
                computed
            ))
            },
            PropertyDeclaration::FontVariantPosition(ref val) => {
                context.for_non_inherited_property =
                        None;
                if let Some(sf) = val.get_system() {
                    longhands::system_font::resolve_system_font(sf, context);
                }
            let computed = val.to_computed_value(context);
            Some(AnimationValue::FontVariantPosition(
                computed
            ))
            },
            PropertyDeclaration::FontFeatureSettings(ref val) => {
                context.for_non_inherited_property =
                        None;
                if let Some(sf) = val.get_system() {
                    longhands::system_font::resolve_system_font(sf, context);
                }
            let computed = (**val).to_computed_value(context);
            Some(AnimationValue::FontFeatureSettings(
                computed
            ))
            },
            PropertyDeclaration::FontVariationSettings(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = val.to_computed_value(context);
            Some(AnimationValue::FontVariationSettings(
                computed
            ))
            },
            PropertyDeclaration::FontLanguageOverride(ref val) => {
                context.for_non_inherited_property =
                        None;
                if let Some(sf) = val.get_system() {
                    longhands::system_font::resolve_system_font(sf, context);
                }
            let computed = (**val).to_computed_value(context);
            Some(AnimationValue::FontLanguageOverride(
                computed
            ))
            },
            PropertyDeclaration::MozOsxFontSmoothing(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = val.to_computed_value(context);
            Some(AnimationValue::MozOsxFontSmoothing(
                computed
            ))
            },
            PropertyDeclaration::Visibility(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = val.to_computed_value(context);
            Some(AnimationValue::Visibility(
                computed
            ))
            },
            PropertyDeclaration::WritingMode(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = val.to_computed_value(context);
            Some(AnimationValue::WritingMode(
                computed
            ))
            },
            PropertyDeclaration::Direction(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = val.to_computed_value(context);
            Some(AnimationValue::Direction(
                computed
            ))
            },
            PropertyDeclaration::TextOrientation(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = val.to_computed_value(context);
            Some(AnimationValue::TextOrientation(
                computed
            ))
            },
            PropertyDeclaration::ColorAdjust(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = val.to_computed_value(context);
            Some(AnimationValue::ColorAdjust(
                computed
            ))
            },
            PropertyDeclaration::ImageRendering(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = val.to_computed_value(context);
            Some(AnimationValue::ImageRendering(
                computed
            ))
            },
            PropertyDeclaration::ImageOrientation(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = val.to_computed_value(context);
            Some(AnimationValue::ImageOrientation(
                computed
            ))
            },
            PropertyDeclaration::BorderCollapse(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = val.to_computed_value(context);
            Some(AnimationValue::BorderCollapse(
                computed
            ))
            },
            PropertyDeclaration::EmptyCells(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = val.to_computed_value(context);
            Some(AnimationValue::EmptyCells(
                computed
            ))
            },
            PropertyDeclaration::CaptionSide(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = val.to_computed_value(context);
            Some(AnimationValue::CaptionSide(
                computed
            ))
            },
            PropertyDeclaration::BorderSpacing(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = (**val).to_computed_value(context);
            Some(AnimationValue::BorderSpacing(
                computed.to_animated_value()
            ))
            },
            PropertyDeclaration::LineHeight(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = val.to_computed_value(context);
            Some(AnimationValue::LineHeight(
                computed.to_animated_value()
            ))
            },
            PropertyDeclaration::TextTransform(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = val.to_computed_value(context);
            Some(AnimationValue::TextTransform(
                computed
            ))
            },
            PropertyDeclaration::Hyphens(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = val.to_computed_value(context);
            Some(AnimationValue::Hyphens(
                computed
            ))
            },
            PropertyDeclaration::MozTextSizeAdjust(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = val.to_computed_value(context);
            Some(AnimationValue::MozTextSizeAdjust(
                computed
            ))
            },
            PropertyDeclaration::TextIndent(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = val.to_computed_value(context);
            Some(AnimationValue::TextIndent(
                computed
            ))
            },
            PropertyDeclaration::OverflowWrap(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = val.to_computed_value(context);
            Some(AnimationValue::OverflowWrap(
                computed
            ))
            },
            PropertyDeclaration::WordBreak(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = val.to_computed_value(context);
            Some(AnimationValue::WordBreak(
                computed
            ))
            },
            PropertyDeclaration::TextJustify(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = val.to_computed_value(context);
            Some(AnimationValue::TextJustify(
                computed
            ))
            },
            PropertyDeclaration::TextAlignLast(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = val.to_computed_value(context);
            Some(AnimationValue::TextAlignLast(
                computed
            ))
            },
            PropertyDeclaration::TextAlign(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = val.to_computed_value(context);
            Some(AnimationValue::TextAlign(
                computed
            ))
            },
            PropertyDeclaration::LetterSpacing(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = val.to_computed_value(context);
            Some(AnimationValue::LetterSpacing(
                computed
            ))
            },
            PropertyDeclaration::WordSpacing(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = val.to_computed_value(context);
            Some(AnimationValue::WordSpacing(
                computed
            ))
            },
            PropertyDeclaration::WhiteSpace(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = val.to_computed_value(context);
            Some(AnimationValue::WhiteSpace(
                computed
            ))
            },
            PropertyDeclaration::TextShadow(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = val.to_computed_value(context);
            Some(AnimationValue::TextShadow(
                computed.to_animated_value()
            ))
            },
            PropertyDeclaration::TextEmphasisStyle(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = (**val).to_computed_value(context);
            Some(AnimationValue::TextEmphasisStyle(
                computed
            ))
            },
            PropertyDeclaration::TextEmphasisPosition(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = val.to_computed_value(context);
            Some(AnimationValue::TextEmphasisPosition(
                computed
            ))
            },
            PropertyDeclaration::TextEmphasisColor(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = val.to_computed_value(context);
            Some(AnimationValue::TextEmphasisColor(
                computed.to_animated_value()
            ))
            },
            PropertyDeclaration::MozTabSize(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = val.to_computed_value(context);
            Some(AnimationValue::MozTabSize(
                computed.to_animated_value()
            ))
            },
            PropertyDeclaration::WebkitTextFillColor(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = val.to_computed_value(context);
            Some(AnimationValue::WebkitTextFillColor(
                computed.to_animated_value()
            ))
            },
            PropertyDeclaration::WebkitTextStrokeColor(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = val.to_computed_value(context);
            Some(AnimationValue::WebkitTextStrokeColor(
                computed.to_animated_value()
            ))
            },
            PropertyDeclaration::WebkitTextStrokeWidth(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = val.to_computed_value(context);
            Some(AnimationValue::WebkitTextStrokeWidth(
                computed
            ))
            },
            PropertyDeclaration::RubyAlign(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = val.to_computed_value(context);
            Some(AnimationValue::RubyAlign(
                computed
            ))
            },
            PropertyDeclaration::RubyPosition(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = val.to_computed_value(context);
            Some(AnimationValue::RubyPosition(
                computed
            ))
            },
            PropertyDeclaration::TextCombineUpright(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = val.to_computed_value(context);
            Some(AnimationValue::TextCombineUpright(
                computed
            ))
            },
            PropertyDeclaration::TextRendering(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = val.to_computed_value(context);
            Some(AnimationValue::TextRendering(
                computed
            ))
            },
            PropertyDeclaration::ListStylePosition(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = val.to_computed_value(context);
            Some(AnimationValue::ListStylePosition(
                computed
            ))
            },
            PropertyDeclaration::ListStyleType(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = (**val).to_computed_value(context);
            Some(AnimationValue::ListStyleType(
                computed
            ))
            },
            PropertyDeclaration::ListStyleImage(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = (**val).to_computed_value(context);
            Some(AnimationValue::ListStyleImage(
                computed
            ))
            },
            PropertyDeclaration::Quotes(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = val.to_computed_value(context);
            Some(AnimationValue::Quotes(
                computed
            ))
            },
            PropertyDeclaration::MozImageRegion(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = (**val).to_computed_value(context);
            Some(AnimationValue::MozImageRegion(
                computed
            ))
            },
            PropertyDeclaration::MarginTop(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::MarginTop);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::MarginTop(
                computed
            ))
            },
            PropertyDeclaration::MarginLeft(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::MarginLeft);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::MarginLeft(
                computed
            ))
            },
            PropertyDeclaration::MarginBottom(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::MarginBottom);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::MarginBottom(
                computed
            ))
            },
            PropertyDeclaration::MarginRight(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::MarginRight);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::MarginRight(
                computed
            ))
            },
            PropertyDeclaration::OutlineColor(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::OutlineColor);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::OutlineColor(
                computed.to_animated_value()
            ))
            },
            PropertyDeclaration::OutlineStyle(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::OutlineStyle);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::OutlineStyle(
                computed
            ))
            },
            PropertyDeclaration::OutlineWidth(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::OutlineWidth);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::OutlineWidth(
                computed.to_animated_value()
            ))
            },
            PropertyDeclaration::MozOutlineRadiusTopleft(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::MozOutlineRadiusTopleft);
            let computed = (**val).to_computed_value(context);
            Some(AnimationValue::MozOutlineRadiusTopleft(
                computed.to_animated_value()
            ))
            },
            PropertyDeclaration::MozOutlineRadiusTopright(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::MozOutlineRadiusTopright);
            let computed = (**val).to_computed_value(context);
            Some(AnimationValue::MozOutlineRadiusTopright(
                computed.to_animated_value()
            ))
            },
            PropertyDeclaration::MozOutlineRadiusBottomright(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::MozOutlineRadiusBottomright);
            let computed = (**val).to_computed_value(context);
            Some(AnimationValue::MozOutlineRadiusBottomright(
                computed.to_animated_value()
            ))
            },
            PropertyDeclaration::MozOutlineRadiusBottomleft(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::MozOutlineRadiusBottomleft);
            let computed = (**val).to_computed_value(context);
            Some(AnimationValue::MozOutlineRadiusBottomleft(
                computed.to_animated_value()
            ))
            },
            PropertyDeclaration::OutlineOffset(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::OutlineOffset);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::OutlineOffset(
                computed
            ))
            },
            PropertyDeclaration::PaddingTop(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::PaddingTop);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::PaddingTop(
                computed.to_animated_value()
            ))
            },
            PropertyDeclaration::PaddingLeft(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::PaddingLeft);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::PaddingLeft(
                computed.to_animated_value()
            ))
            },
            PropertyDeclaration::PaddingBottom(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::PaddingBottom);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::PaddingBottom(
                computed.to_animated_value()
            ))
            },
            PropertyDeclaration::PaddingRight(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::PaddingRight);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::PaddingRight(
                computed.to_animated_value()
            ))
            },
            PropertyDeclaration::Cursor(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = (**val).to_computed_value(context);
            Some(AnimationValue::Cursor(
                computed
            ))
            },
            PropertyDeclaration::PointerEvents(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = val.to_computed_value(context);
            Some(AnimationValue::PointerEvents(
                computed
            ))
            },
            PropertyDeclaration::MozUserInput(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = val.to_computed_value(context);
            Some(AnimationValue::MozUserInput(
                computed
            ))
            },
            PropertyDeclaration::MozUserModify(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = val.to_computed_value(context);
            Some(AnimationValue::MozUserModify(
                computed
            ))
            },
            PropertyDeclaration::MozUserFocus(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = val.to_computed_value(context);
            Some(AnimationValue::MozUserFocus(
                computed
            ))
            },
            PropertyDeclaration::CaretColor(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = (**val).to_computed_value(context);
            Some(AnimationValue::CaretColor(
                computed.to_animated_value()
            ))
            },
            PropertyDeclaration::Top(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::Top);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::Top(
                computed
            ))
            },
            PropertyDeclaration::Left(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::Left);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::Left(
                computed
            ))
            },
            PropertyDeclaration::Bottom(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::Bottom);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::Bottom(
                computed
            ))
            },
            PropertyDeclaration::Right(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::Right);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::Right(
                computed
            ))
            },
            PropertyDeclaration::ZIndex(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::ZIndex);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::ZIndex(
                computed
            ))
            },
            PropertyDeclaration::FlexDirection(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::FlexDirection);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::FlexDirection(
                computed
            ))
            },
            PropertyDeclaration::FlexWrap(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::FlexWrap);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::FlexWrap(
                computed
            ))
            },
            PropertyDeclaration::JustifyContent(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::JustifyContent);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::JustifyContent(
                computed
            ))
            },
            PropertyDeclaration::AlignContent(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::AlignContent);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::AlignContent(
                computed
            ))
            },
            PropertyDeclaration::AlignItems(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::AlignItems);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::AlignItems(
                computed
            ))
            },
            PropertyDeclaration::JustifyItems(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::JustifyItems);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::JustifyItems(
                computed
            ))
            },
            PropertyDeclaration::FlexGrow(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::FlexGrow);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::FlexGrow(
                computed.to_animated_value()
            ))
            },
            PropertyDeclaration::FlexShrink(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::FlexShrink);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::FlexShrink(
                computed.to_animated_value()
            ))
            },
            PropertyDeclaration::AlignSelf(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::AlignSelf);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::AlignSelf(
                computed
            ))
            },
            PropertyDeclaration::JustifySelf(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::JustifySelf);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::JustifySelf(
                computed
            ))
            },
            PropertyDeclaration::Order(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::Order);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::Order(
                computed
            ))
            },
            PropertyDeclaration::FlexBasis(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::FlexBasis);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::FlexBasis(
                computed.to_animated_value()
            ))
            },
            PropertyDeclaration::Width(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::Width);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::Width(
                computed.to_animated_value()
            ))
            },
            PropertyDeclaration::MinWidth(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::MinWidth);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::MinWidth(
                computed.to_animated_value()
            ))
            },
            PropertyDeclaration::MaxWidth(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::MaxWidth);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::MaxWidth(
                computed.to_animated_value()
            ))
            },
            PropertyDeclaration::Height(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::Height);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::Height(
                computed.to_animated_value()
            ))
            },
            PropertyDeclaration::MinHeight(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::MinHeight);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::MinHeight(
                computed.to_animated_value()
            ))
            },
            PropertyDeclaration::MaxHeight(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::MaxHeight);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::MaxHeight(
                computed.to_animated_value()
            ))
            },
            PropertyDeclaration::BoxSizing(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::BoxSizing);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::BoxSizing(
                computed
            ))
            },
            PropertyDeclaration::ObjectFit(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::ObjectFit);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::ObjectFit(
                computed
            ))
            },
            PropertyDeclaration::ObjectPosition(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::ObjectPosition);
            let computed = (**val).to_computed_value(context);
            Some(AnimationValue::ObjectPosition(
                computed
            ))
            },
            PropertyDeclaration::GridRowGap(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::GridRowGap);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::GridRowGap(
                computed.to_animated_value()
            ))
            },
            PropertyDeclaration::GridRowStart(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::GridRowStart);
            let computed = (**val).to_computed_value(context);
            Some(AnimationValue::GridRowStart(
                computed
            ))
            },
            PropertyDeclaration::GridRowEnd(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::GridRowEnd);
            let computed = (**val).to_computed_value(context);
            Some(AnimationValue::GridRowEnd(
                computed
            ))
            },
            PropertyDeclaration::GridAutoRows(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::GridAutoRows);
            let computed = (**val).to_computed_value(context);
            Some(AnimationValue::GridAutoRows(
                computed
            ))
            },
            PropertyDeclaration::GridTemplateRows(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::GridTemplateRows);
            let computed = (**val).to_computed_value(context);
            Some(AnimationValue::GridTemplateRows(
                computed
            ))
            },
            PropertyDeclaration::GridColumnGap(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::GridColumnGap);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::GridColumnGap(
                computed.to_animated_value()
            ))
            },
            PropertyDeclaration::GridColumnStart(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::GridColumnStart);
            let computed = (**val).to_computed_value(context);
            Some(AnimationValue::GridColumnStart(
                computed
            ))
            },
            PropertyDeclaration::GridColumnEnd(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::GridColumnEnd);
            let computed = (**val).to_computed_value(context);
            Some(AnimationValue::GridColumnEnd(
                computed
            ))
            },
            PropertyDeclaration::GridAutoColumns(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::GridAutoColumns);
            let computed = (**val).to_computed_value(context);
            Some(AnimationValue::GridAutoColumns(
                computed
            ))
            },
            PropertyDeclaration::GridTemplateColumns(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::GridTemplateColumns);
            let computed = (**val).to_computed_value(context);
            Some(AnimationValue::GridTemplateColumns(
                computed
            ))
            },
            PropertyDeclaration::GridAutoFlow(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::GridAutoFlow);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::GridAutoFlow(
                computed
            ))
            },
            PropertyDeclaration::GridTemplateAreas(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::GridTemplateAreas);
            let computed = (**val).to_computed_value(context);
            Some(AnimationValue::GridTemplateAreas(
                computed
            ))
            },
            PropertyDeclaration::TableLayout(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::TableLayout);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::TableLayout(
                computed
            ))
            },
            PropertyDeclaration::TextOverflow(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::TextOverflow);
            let computed = (**val).to_computed_value(context);
            Some(AnimationValue::TextOverflow(
                computed
            ))
            },
            PropertyDeclaration::UnicodeBidi(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::UnicodeBidi);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::UnicodeBidi(
                computed
            ))
            },
            PropertyDeclaration::TextDecorationLine(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::TextDecorationLine);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::TextDecorationLine(
                computed
            ))
            },
            PropertyDeclaration::TextDecorationStyle(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::TextDecorationStyle);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::TextDecorationStyle(
                computed
            ))
            },
            PropertyDeclaration::TextDecorationColor(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::TextDecorationColor);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::TextDecorationColor(
                computed.to_animated_value()
            ))
            },
            PropertyDeclaration::InitialLetter(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::InitialLetter);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::InitialLetter(
                computed
            ))
            },
            PropertyDeclaration::ImeMode(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::ImeMode);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::ImeMode(
                computed
            ))
            },
            PropertyDeclaration::MozUserSelect(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::MozUserSelect);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::MozUserSelect(
                computed
            ))
            },
            PropertyDeclaration::MozWindowDragging(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::MozWindowDragging);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::MozWindowDragging(
                computed
            ))
            },
            PropertyDeclaration::MozWindowShadow(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::MozWindowShadow);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::MozWindowShadow(
                computed
            ))
            },
            PropertyDeclaration::MozForceBrokenImageIcon(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::MozForceBrokenImageIcon);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::MozForceBrokenImageIcon(
                computed
            ))
            },
            PropertyDeclaration::TextAnchor(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = val.to_computed_value(context);
            Some(AnimationValue::TextAnchor(
                computed
            ))
            },
            PropertyDeclaration::ColorInterpolation(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = val.to_computed_value(context);
            Some(AnimationValue::ColorInterpolation(
                computed
            ))
            },
            PropertyDeclaration::ColorInterpolationFilters(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = val.to_computed_value(context);
            Some(AnimationValue::ColorInterpolationFilters(
                computed
            ))
            },
            PropertyDeclaration::Fill(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = (**val).to_computed_value(context);
            Some(AnimationValue::Fill(
                computed.to_animated_value()
            ))
            },
            PropertyDeclaration::FillOpacity(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = val.to_computed_value(context);
            Some(AnimationValue::FillOpacity(
                computed
            ))
            },
            PropertyDeclaration::FillRule(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = val.to_computed_value(context);
            Some(AnimationValue::FillRule(
                computed
            ))
            },
            PropertyDeclaration::ShapeRendering(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = val.to_computed_value(context);
            Some(AnimationValue::ShapeRendering(
                computed
            ))
            },
            PropertyDeclaration::Stroke(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = (**val).to_computed_value(context);
            Some(AnimationValue::Stroke(
                computed.to_animated_value()
            ))
            },
            PropertyDeclaration::StrokeWidth(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = (**val).to_computed_value(context);
            Some(AnimationValue::StrokeWidth(
                computed.to_animated_value()
            ))
            },
            PropertyDeclaration::StrokeLinecap(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = val.to_computed_value(context);
            Some(AnimationValue::StrokeLinecap(
                computed
            ))
            },
            PropertyDeclaration::StrokeLinejoin(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = val.to_computed_value(context);
            Some(AnimationValue::StrokeLinejoin(
                computed
            ))
            },
            PropertyDeclaration::StrokeMiterlimit(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = val.to_computed_value(context);
            Some(AnimationValue::StrokeMiterlimit(
                computed.to_animated_value()
            ))
            },
            PropertyDeclaration::StrokeOpacity(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = val.to_computed_value(context);
            Some(AnimationValue::StrokeOpacity(
                computed
            ))
            },
            PropertyDeclaration::StrokeDasharray(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = val.to_computed_value(context);
            Some(AnimationValue::StrokeDasharray(
                computed.to_animated_value()
            ))
            },
            PropertyDeclaration::StrokeDashoffset(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = (**val).to_computed_value(context);
            Some(AnimationValue::StrokeDashoffset(
                computed
            ))
            },
            PropertyDeclaration::ClipRule(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = val.to_computed_value(context);
            Some(AnimationValue::ClipRule(
                computed
            ))
            },
            PropertyDeclaration::MarkerStart(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = (**val).to_computed_value(context);
            Some(AnimationValue::MarkerStart(
                computed
            ))
            },
            PropertyDeclaration::MarkerMid(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = (**val).to_computed_value(context);
            Some(AnimationValue::MarkerMid(
                computed
            ))
            },
            PropertyDeclaration::MarkerEnd(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = (**val).to_computed_value(context);
            Some(AnimationValue::MarkerEnd(
                computed
            ))
            },
            PropertyDeclaration::PaintOrder(ref val) => {
                context.for_non_inherited_property =
                        None;
            let computed = val.to_computed_value(context);
            Some(AnimationValue::PaintOrder(
                computed
            ))
            },
            PropertyDeclaration::DominantBaseline(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::DominantBaseline);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::DominantBaseline(
                computed
            ))
            },
            PropertyDeclaration::VectorEffect(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::VectorEffect);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::VectorEffect(
                computed
            ))
            },
            PropertyDeclaration::StopColor(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::StopColor);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::StopColor(
                computed.to_animated_value()
            ))
            },
            PropertyDeclaration::StopOpacity(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::StopOpacity);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::StopOpacity(
                computed
            ))
            },
            PropertyDeclaration::FloodColor(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::FloodColor);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::FloodColor(
                computed.to_animated_value()
            ))
            },
            PropertyDeclaration::FloodOpacity(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::FloodOpacity);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::FloodOpacity(
                computed
            ))
            },
            PropertyDeclaration::LightingColor(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::LightingColor);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::LightingColor(
                computed.to_animated_value()
            ))
            },
            PropertyDeclaration::MaskType(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::MaskType);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::MaskType(
                computed
            ))
            },
            PropertyDeclaration::ClipPath(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::ClipPath);
            let computed = (**val).to_computed_value(context);
            Some(AnimationValue::ClipPath(
                computed
            ))
            },
            PropertyDeclaration::MaskMode(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::MaskMode);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::MaskMode(
                computed
            ))
            },
            PropertyDeclaration::MaskRepeat(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::MaskRepeat);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::MaskRepeat(
                computed
            ))
            },
            PropertyDeclaration::MaskPositionX(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::MaskPositionX);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::MaskPositionX(
                computed
            ))
            },
            PropertyDeclaration::MaskPositionY(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::MaskPositionY);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::MaskPositionY(
                computed
            ))
            },
            PropertyDeclaration::MaskClip(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::MaskClip);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::MaskClip(
                computed
            ))
            },
            PropertyDeclaration::MaskOrigin(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::MaskOrigin);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::MaskOrigin(
                computed
            ))
            },
            PropertyDeclaration::MaskSize(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::MaskSize);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::MaskSize(
                computed
            ))
            },
            PropertyDeclaration::MaskComposite(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::MaskComposite);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::MaskComposite(
                computed
            ))
            },
            PropertyDeclaration::MaskImage(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::MaskImage);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::MaskImage(
                computed
            ))
            },
            PropertyDeclaration::MozBoxAlign(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::MozBoxAlign);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::MozBoxAlign(
                computed
            ))
            },
            PropertyDeclaration::MozBoxDirection(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::MozBoxDirection);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::MozBoxDirection(
                computed
            ))
            },
            PropertyDeclaration::MozBoxFlex(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::MozBoxFlex);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::MozBoxFlex(
                computed.to_animated_value()
            ))
            },
            PropertyDeclaration::MozBoxOrient(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::MozBoxOrient);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::MozBoxOrient(
                computed
            ))
            },
            PropertyDeclaration::MozBoxPack(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::MozBoxPack);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::MozBoxPack(
                computed
            ))
            },
            PropertyDeclaration::MozStackSizing(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::MozStackSizing);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::MozStackSizing(
                computed
            ))
            },
            PropertyDeclaration::MozBoxOrdinalGroup(ref val) => {
                context.for_non_inherited_property =
                        Some(LonghandId::MozBoxOrdinalGroup);
            let computed = val.to_computed_value(context);
            Some(AnimationValue::MozBoxOrdinalGroup(
                computed
            ))
            },
            PropertyDeclaration::CSSWideKeyword(id, keyword) => {
                match id {
                    // We put all the animatable properties first in the hopes
                    // that it might increase match locality.
                    LonghandId::BackgroundColor => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_background();
                                initial_struct.clone_background_color()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_background();
                                inherit_struct.clone_background_color()
                            },
                        };
                        let computed = computed.to_animated_value();
                        Some(AnimationValue::BackgroundColor(computed))
                    },
                    LonghandId::BackgroundImage => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_background();
                                initial_struct.clone_background_image()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_background();
                                inherit_struct.clone_background_image()
                            },
                        };
                        Some(AnimationValue::BackgroundImage(computed))
                    },
                    LonghandId::BackgroundPositionX => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_background();
                                initial_struct.clone_background_position_x()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_background();
                                inherit_struct.clone_background_position_x()
                            },
                        };
                        Some(AnimationValue::BackgroundPositionX(computed))
                    },
                    LonghandId::BackgroundPositionY => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_background();
                                initial_struct.clone_background_position_y()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_background();
                                inherit_struct.clone_background_position_y()
                            },
                        };
                        Some(AnimationValue::BackgroundPositionY(computed))
                    },
                    LonghandId::BackgroundRepeat => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_background();
                                initial_struct.clone_background_repeat()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_background();
                                inherit_struct.clone_background_repeat()
                            },
                        };
                        Some(AnimationValue::BackgroundRepeat(computed))
                    },
                    LonghandId::BackgroundAttachment => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_background();
                                initial_struct.clone_background_attachment()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_background();
                                inherit_struct.clone_background_attachment()
                            },
                        };
                        Some(AnimationValue::BackgroundAttachment(computed))
                    },
                    LonghandId::BackgroundClip => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_background();
                                initial_struct.clone_background_clip()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_background();
                                inherit_struct.clone_background_clip()
                            },
                        };
                        Some(AnimationValue::BackgroundClip(computed))
                    },
                    LonghandId::BackgroundOrigin => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_background();
                                initial_struct.clone_background_origin()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_background();
                                inherit_struct.clone_background_origin()
                            },
                        };
                        Some(AnimationValue::BackgroundOrigin(computed))
                    },
                    LonghandId::BackgroundSize => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_background();
                                initial_struct.clone_background_size()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_background();
                                inherit_struct.clone_background_size()
                            },
                        };
                        let computed = computed.to_animated_value();
                        Some(AnimationValue::BackgroundSize(computed))
                    },
                    LonghandId::BackgroundBlendMode => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_background();
                                initial_struct.clone_background_blend_mode()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_background();
                                inherit_struct.clone_background_blend_mode()
                            },
                        };
                        Some(AnimationValue::BackgroundBlendMode(computed))
                    },
                    LonghandId::BorderTopColor => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_border();
                                initial_struct.clone_border_top_color()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_border();
                                inherit_struct.clone_border_top_color()
                            },
                        };
                        let computed = computed.to_animated_value();
                        Some(AnimationValue::BorderTopColor(computed))
                    },
                    LonghandId::BorderTopStyle => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_border();
                                initial_struct.clone_border_top_style()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_border();
                                inherit_struct.clone_border_top_style()
                            },
                        };
                        Some(AnimationValue::BorderTopStyle(computed))
                    },
                    LonghandId::BorderTopWidth => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_border();
                                initial_struct.clone_border_top_width()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_border();
                                inherit_struct.clone_border_top_width()
                            },
                        };
                        let computed = computed.to_animated_value();
                        Some(AnimationValue::BorderTopWidth(computed))
                    },
                    LonghandId::BorderLeftColor => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_border();
                                initial_struct.clone_border_left_color()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_border();
                                inherit_struct.clone_border_left_color()
                            },
                        };
                        let computed = computed.to_animated_value();
                        Some(AnimationValue::BorderLeftColor(computed))
                    },
                    LonghandId::BorderLeftStyle => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_border();
                                initial_struct.clone_border_left_style()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_border();
                                inherit_struct.clone_border_left_style()
                            },
                        };
                        Some(AnimationValue::BorderLeftStyle(computed))
                    },
                    LonghandId::BorderLeftWidth => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_border();
                                initial_struct.clone_border_left_width()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_border();
                                inherit_struct.clone_border_left_width()
                            },
                        };
                        let computed = computed.to_animated_value();
                        Some(AnimationValue::BorderLeftWidth(computed))
                    },
                    LonghandId::BorderBottomColor => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_border();
                                initial_struct.clone_border_bottom_color()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_border();
                                inherit_struct.clone_border_bottom_color()
                            },
                        };
                        let computed = computed.to_animated_value();
                        Some(AnimationValue::BorderBottomColor(computed))
                    },
                    LonghandId::BorderBottomStyle => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_border();
                                initial_struct.clone_border_bottom_style()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_border();
                                inherit_struct.clone_border_bottom_style()
                            },
                        };
                        Some(AnimationValue::BorderBottomStyle(computed))
                    },
                    LonghandId::BorderBottomWidth => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_border();
                                initial_struct.clone_border_bottom_width()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_border();
                                inherit_struct.clone_border_bottom_width()
                            },
                        };
                        let computed = computed.to_animated_value();
                        Some(AnimationValue::BorderBottomWidth(computed))
                    },
                    LonghandId::BorderRightColor => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_border();
                                initial_struct.clone_border_right_color()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_border();
                                inherit_struct.clone_border_right_color()
                            },
                        };
                        let computed = computed.to_animated_value();
                        Some(AnimationValue::BorderRightColor(computed))
                    },
                    LonghandId::BorderRightStyle => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_border();
                                initial_struct.clone_border_right_style()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_border();
                                inherit_struct.clone_border_right_style()
                            },
                        };
                        Some(AnimationValue::BorderRightStyle(computed))
                    },
                    LonghandId::BorderRightWidth => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_border();
                                initial_struct.clone_border_right_width()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_border();
                                inherit_struct.clone_border_right_width()
                            },
                        };
                        let computed = computed.to_animated_value();
                        Some(AnimationValue::BorderRightWidth(computed))
                    },
                    LonghandId::BorderTopLeftRadius => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_border();
                                initial_struct.clone_border_top_left_radius()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_border();
                                inherit_struct.clone_border_top_left_radius()
                            },
                        };
                        let computed = computed.to_animated_value();
                        Some(AnimationValue::BorderTopLeftRadius(computed))
                    },
                    LonghandId::BorderTopRightRadius => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_border();
                                initial_struct.clone_border_top_right_radius()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_border();
                                inherit_struct.clone_border_top_right_radius()
                            },
                        };
                        let computed = computed.to_animated_value();
                        Some(AnimationValue::BorderTopRightRadius(computed))
                    },
                    LonghandId::BorderBottomRightRadius => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_border();
                                initial_struct.clone_border_bottom_right_radius()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_border();
                                inherit_struct.clone_border_bottom_right_radius()
                            },
                        };
                        let computed = computed.to_animated_value();
                        Some(AnimationValue::BorderBottomRightRadius(computed))
                    },
                    LonghandId::BorderBottomLeftRadius => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_border();
                                initial_struct.clone_border_bottom_left_radius()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_border();
                                inherit_struct.clone_border_bottom_left_radius()
                            },
                        };
                        let computed = computed.to_animated_value();
                        Some(AnimationValue::BorderBottomLeftRadius(computed))
                    },
                    LonghandId::MozBorderTopColors => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_border();
                                initial_struct.clone__moz_border_top_colors()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_border();
                                inherit_struct.clone__moz_border_top_colors()
                            },
                        };
                        Some(AnimationValue::MozBorderTopColors(computed))
                    },
                    LonghandId::MozBorderLeftColors => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_border();
                                initial_struct.clone__moz_border_left_colors()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_border();
                                inherit_struct.clone__moz_border_left_colors()
                            },
                        };
                        Some(AnimationValue::MozBorderLeftColors(computed))
                    },
                    LonghandId::MozBorderBottomColors => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_border();
                                initial_struct.clone__moz_border_bottom_colors()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_border();
                                inherit_struct.clone__moz_border_bottom_colors()
                            },
                        };
                        Some(AnimationValue::MozBorderBottomColors(computed))
                    },
                    LonghandId::MozBorderRightColors => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_border();
                                initial_struct.clone__moz_border_right_colors()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_border();
                                inherit_struct.clone__moz_border_right_colors()
                            },
                        };
                        Some(AnimationValue::MozBorderRightColors(computed))
                    },
                    LonghandId::BoxDecorationBreak => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_border();
                                initial_struct.clone_box_decoration_break()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_border();
                                inherit_struct.clone_box_decoration_break()
                            },
                        };
                        Some(AnimationValue::BoxDecorationBreak(computed))
                    },
                    LonghandId::MozFloatEdge => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_border();
                                initial_struct.clone__moz_float_edge()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_border();
                                inherit_struct.clone__moz_float_edge()
                            },
                        };
                        Some(AnimationValue::MozFloatEdge(computed))
                    },
                    LonghandId::BorderImageSource => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_border();
                                initial_struct.clone_border_image_source()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_border();
                                inherit_struct.clone_border_image_source()
                            },
                        };
                        Some(AnimationValue::BorderImageSource(computed))
                    },
                    LonghandId::BorderImageOutset => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_border();
                                initial_struct.clone_border_image_outset()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_border();
                                inherit_struct.clone_border_image_outset()
                            },
                        };
                        Some(AnimationValue::BorderImageOutset(computed))
                    },
                    LonghandId::BorderImageRepeat => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_border();
                                initial_struct.clone_border_image_repeat()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_border();
                                inherit_struct.clone_border_image_repeat()
                            },
                        };
                        Some(AnimationValue::BorderImageRepeat(computed))
                    },
                    LonghandId::BorderImageWidth => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_border();
                                initial_struct.clone_border_image_width()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_border();
                                inherit_struct.clone_border_image_width()
                            },
                        };
                        Some(AnimationValue::BorderImageWidth(computed))
                    },
                    LonghandId::BorderImageSlice => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_border();
                                initial_struct.clone_border_image_slice()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_border();
                                inherit_struct.clone_border_image_slice()
                            },
                        };
                        Some(AnimationValue::BorderImageSlice(computed))
                    },
                    LonghandId::Display => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_box();
                                initial_struct.clone_display()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_box();
                                inherit_struct.clone_display()
                            },
                        };
                        Some(AnimationValue::Display(computed))
                    },
                    LonghandId::Position => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_box();
                                initial_struct.clone_position()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_box();
                                inherit_struct.clone_position()
                            },
                        };
                        Some(AnimationValue::Position(computed))
                    },
                    LonghandId::Float => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_box();
                                initial_struct.clone_float()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_box();
                                inherit_struct.clone_float()
                            },
                        };
                        Some(AnimationValue::Float(computed))
                    },
                    LonghandId::Clear => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_box();
                                initial_struct.clone_clear()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_box();
                                inherit_struct.clone_clear()
                            },
                        };
                        Some(AnimationValue::Clear(computed))
                    },
                    LonghandId::VerticalAlign => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_box();
                                initial_struct.clone_vertical_align()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_box();
                                inherit_struct.clone_vertical_align()
                            },
                        };
                        Some(AnimationValue::VerticalAlign(computed))
                    },
                    LonghandId::OverflowClipBox => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_box();
                                initial_struct.clone_overflow_clip_box()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_box();
                                inherit_struct.clone_overflow_clip_box()
                            },
                        };
                        Some(AnimationValue::OverflowClipBox(computed))
                    },
                    LonghandId::OverflowX => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_box();
                                initial_struct.clone_overflow_x()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_box();
                                inherit_struct.clone_overflow_x()
                            },
                        };
                        Some(AnimationValue::OverflowX(computed))
                    },
                    LonghandId::OverflowY => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_box();
                                initial_struct.clone_overflow_y()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_box();
                                inherit_struct.clone_overflow_y()
                            },
                        };
                        Some(AnimationValue::OverflowY(computed))
                    },
                    LonghandId::ScrollSnapPointsX => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_box();
                                initial_struct.clone_scroll_snap_points_x()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_box();
                                inherit_struct.clone_scroll_snap_points_x()
                            },
                        };
                        Some(AnimationValue::ScrollSnapPointsX(computed))
                    },
                    LonghandId::ScrollSnapPointsY => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_box();
                                initial_struct.clone_scroll_snap_points_y()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_box();
                                inherit_struct.clone_scroll_snap_points_y()
                            },
                        };
                        Some(AnimationValue::ScrollSnapPointsY(computed))
                    },
                    LonghandId::ScrollSnapDestination => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_box();
                                initial_struct.clone_scroll_snap_destination()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_box();
                                inherit_struct.clone_scroll_snap_destination()
                            },
                        };
                        Some(AnimationValue::ScrollSnapDestination(computed))
                    },
                    LonghandId::ScrollSnapCoordinate => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_box();
                                initial_struct.clone_scroll_snap_coordinate()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_box();
                                inherit_struct.clone_scroll_snap_coordinate()
                            },
                        };
                        Some(AnimationValue::ScrollSnapCoordinate(computed))
                    },
                    LonghandId::Transform => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_box();
                                initial_struct.clone_transform()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_box();
                                inherit_struct.clone_transform()
                            },
                        };
                        Some(AnimationValue::Transform(computed))
                    },
                    LonghandId::ScrollBehavior => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_box();
                                initial_struct.clone_scroll_behavior()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_box();
                                inherit_struct.clone_scroll_behavior()
                            },
                        };
                        Some(AnimationValue::ScrollBehavior(computed))
                    },
                    LonghandId::ScrollSnapTypeX => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_box();
                                initial_struct.clone_scroll_snap_type_x()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_box();
                                inherit_struct.clone_scroll_snap_type_x()
                            },
                        };
                        Some(AnimationValue::ScrollSnapTypeX(computed))
                    },
                    LonghandId::ScrollSnapTypeY => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_box();
                                initial_struct.clone_scroll_snap_type_y()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_box();
                                inherit_struct.clone_scroll_snap_type_y()
                            },
                        };
                        Some(AnimationValue::ScrollSnapTypeY(computed))
                    },
                    LonghandId::Isolation => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_box();
                                initial_struct.clone_isolation()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_box();
                                inherit_struct.clone_isolation()
                            },
                        };
                        Some(AnimationValue::Isolation(computed))
                    },
                    LonghandId::PageBreakAfter => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_box();
                                initial_struct.clone_page_break_after()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_box();
                                inherit_struct.clone_page_break_after()
                            },
                        };
                        Some(AnimationValue::PageBreakAfter(computed))
                    },
                    LonghandId::PageBreakBefore => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_box();
                                initial_struct.clone_page_break_before()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_box();
                                inherit_struct.clone_page_break_before()
                            },
                        };
                        Some(AnimationValue::PageBreakBefore(computed))
                    },
                    LonghandId::PageBreakInside => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_box();
                                initial_struct.clone_page_break_inside()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_box();
                                inherit_struct.clone_page_break_inside()
                            },
                        };
                        Some(AnimationValue::PageBreakInside(computed))
                    },
                    LonghandId::Resize => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_box();
                                initial_struct.clone_resize()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_box();
                                inherit_struct.clone_resize()
                            },
                        };
                        Some(AnimationValue::Resize(computed))
                    },
                    LonghandId::Perspective => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_box();
                                initial_struct.clone_perspective()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_box();
                                inherit_struct.clone_perspective()
                            },
                        };
                        Some(AnimationValue::Perspective(computed))
                    },
                    LonghandId::PerspectiveOrigin => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_box();
                                initial_struct.clone_perspective_origin()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_box();
                                inherit_struct.clone_perspective_origin()
                            },
                        };
                        Some(AnimationValue::PerspectiveOrigin(computed))
                    },
                    LonghandId::BackfaceVisibility => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_box();
                                initial_struct.clone_backface_visibility()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_box();
                                inherit_struct.clone_backface_visibility()
                            },
                        };
                        Some(AnimationValue::BackfaceVisibility(computed))
                    },
                    LonghandId::TransformBox => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_box();
                                initial_struct.clone_transform_box()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_box();
                                inherit_struct.clone_transform_box()
                            },
                        };
                        Some(AnimationValue::TransformBox(computed))
                    },
                    LonghandId::TransformStyle => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_box();
                                initial_struct.clone_transform_style()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_box();
                                inherit_struct.clone_transform_style()
                            },
                        };
                        Some(AnimationValue::TransformStyle(computed))
                    },
                    LonghandId::TransformOrigin => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_box();
                                initial_struct.clone_transform_origin()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_box();
                                inherit_struct.clone_transform_origin()
                            },
                        };
                        Some(AnimationValue::TransformOrigin(computed))
                    },
                    LonghandId::Contain => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_box();
                                initial_struct.clone_contain()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_box();
                                inherit_struct.clone_contain()
                            },
                        };
                        Some(AnimationValue::Contain(computed))
                    },
                    LonghandId::MozAppearance => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_box();
                                initial_struct.clone__moz_appearance()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_box();
                                inherit_struct.clone__moz_appearance()
                            },
                        };
                        Some(AnimationValue::MozAppearance(computed))
                    },
                    LonghandId::MozOrient => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_box();
                                initial_struct.clone__moz_orient()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_box();
                                inherit_struct.clone__moz_orient()
                            },
                        };
                        Some(AnimationValue::MozOrient(computed))
                    },
                    LonghandId::WillChange => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_box();
                                initial_struct.clone_will_change()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_box();
                                inherit_struct.clone_will_change()
                            },
                        };
                        Some(AnimationValue::WillChange(computed))
                    },
                    LonghandId::ShapeOutside => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_box();
                                initial_struct.clone_shape_outside()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_box();
                                inherit_struct.clone_shape_outside()
                            },
                        };
                        Some(AnimationValue::ShapeOutside(computed))
                    },
                    LonghandId::TouchAction => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_box();
                                initial_struct.clone_touch_action()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_box();
                                inherit_struct.clone_touch_action()
                            },
                        };
                        Some(AnimationValue::TouchAction(computed))
                    },
                    LonghandId::Color => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_color();
                                initial_struct.clone_color()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_color();
                                inherit_struct.clone_color()
                            },
                        };
                        let computed = computed.to_animated_value();
                        Some(AnimationValue::Color(computed))
                    },
                    LonghandId::ColumnWidth => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_column();
                                initial_struct.clone_column_width()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_column();
                                inherit_struct.clone_column_width()
                            },
                        };
                        let computed = computed.to_animated_value();
                        Some(AnimationValue::ColumnWidth(computed))
                    },
                    LonghandId::ColumnCount => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_column();
                                initial_struct.clone_column_count()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_column();
                                inherit_struct.clone_column_count()
                            },
                        };
                        let computed = computed.to_animated_value();
                        Some(AnimationValue::ColumnCount(computed))
                    },
                    LonghandId::ColumnGap => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_column();
                                initial_struct.clone_column_gap()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_column();
                                inherit_struct.clone_column_gap()
                            },
                        };
                        let computed = computed.to_animated_value();
                        Some(AnimationValue::ColumnGap(computed))
                    },
                    LonghandId::ColumnFill => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_column();
                                initial_struct.clone_column_fill()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_column();
                                inherit_struct.clone_column_fill()
                            },
                        };
                        Some(AnimationValue::ColumnFill(computed))
                    },
                    LonghandId::ColumnRuleWidth => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_column();
                                initial_struct.clone_column_rule_width()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_column();
                                inherit_struct.clone_column_rule_width()
                            },
                        };
                        let computed = computed.to_animated_value();
                        Some(AnimationValue::ColumnRuleWidth(computed))
                    },
                    LonghandId::ColumnRuleColor => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_column();
                                initial_struct.clone_column_rule_color()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_column();
                                inherit_struct.clone_column_rule_color()
                            },
                        };
                        let computed = computed.to_animated_value();
                        Some(AnimationValue::ColumnRuleColor(computed))
                    },
                    LonghandId::ColumnSpan => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_column();
                                initial_struct.clone_column_span()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_column();
                                inherit_struct.clone_column_span()
                            },
                        };
                        Some(AnimationValue::ColumnSpan(computed))
                    },
                    LonghandId::ColumnRuleStyle => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_column();
                                initial_struct.clone_column_rule_style()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_column();
                                inherit_struct.clone_column_rule_style()
                            },
                        };
                        Some(AnimationValue::ColumnRuleStyle(computed))
                    },
                    LonghandId::Content => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_counters();
                                initial_struct.clone_content()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_counters();
                                inherit_struct.clone_content()
                            },
                        };
                        Some(AnimationValue::Content(computed))
                    },
                    LonghandId::CounterIncrement => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_counters();
                                initial_struct.clone_counter_increment()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_counters();
                                inherit_struct.clone_counter_increment()
                            },
                        };
                        Some(AnimationValue::CounterIncrement(computed))
                    },
                    LonghandId::CounterReset => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_counters();
                                initial_struct.clone_counter_reset()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_counters();
                                inherit_struct.clone_counter_reset()
                            },
                        };
                        Some(AnimationValue::CounterReset(computed))
                    },
                    LonghandId::Opacity => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_effects();
                                initial_struct.clone_opacity()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_effects();
                                inherit_struct.clone_opacity()
                            },
                        };
                        Some(AnimationValue::Opacity(computed))
                    },
                    LonghandId::BoxShadow => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_effects();
                                initial_struct.clone_box_shadow()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_effects();
                                inherit_struct.clone_box_shadow()
                            },
                        };
                        let computed = computed.to_animated_value();
                        Some(AnimationValue::BoxShadow(computed))
                    },
                    LonghandId::Clip => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_effects();
                                initial_struct.clone_clip()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_effects();
                                inherit_struct.clone_clip()
                            },
                        };
                        Some(AnimationValue::Clip(computed))
                    },
                    LonghandId::Filter => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_effects();
                                initial_struct.clone_filter()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_effects();
                                inherit_struct.clone_filter()
                            },
                        };
                        let computed = computed.to_animated_value();
                        Some(AnimationValue::Filter(computed))
                    },
                    LonghandId::MixBlendMode => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_effects();
                                initial_struct.clone_mix_blend_mode()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_effects();
                                inherit_struct.clone_mix_blend_mode()
                            },
                        };
                        Some(AnimationValue::MixBlendMode(computed))
                    },
                    LonghandId::FontFamily => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_font();
                                initial_struct.clone_font_family()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_font();
                                inherit_struct.clone_font_family()
                            },
                        };
                        Some(AnimationValue::FontFamily(computed))
                    },
                    LonghandId::FontStyle => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_font();
                                initial_struct.clone_font_style()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_font();
                                inherit_struct.clone_font_style()
                            },
                        };
                        Some(AnimationValue::FontStyle(computed))
                    },
                    LonghandId::FontVariantCaps => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_font();
                                initial_struct.clone_font_variant_caps()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_font();
                                inherit_struct.clone_font_variant_caps()
                            },
                        };
                        Some(AnimationValue::FontVariantCaps(computed))
                    },
                    LonghandId::FontWeight => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_font();
                                initial_struct.clone_font_weight()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_font();
                                inherit_struct.clone_font_weight()
                            },
                        };
                        Some(AnimationValue::FontWeight(computed))
                    },
                    LonghandId::FontSize => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_font();
                                initial_struct.clone_font_size()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_font();
                                inherit_struct.clone_font_size()
                            },
                        };
                        let computed = computed.to_animated_value();
                        Some(AnimationValue::FontSize(computed))
                    },
                    LonghandId::FontSizeAdjust => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_font();
                                initial_struct.clone_font_size_adjust()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_font();
                                inherit_struct.clone_font_size_adjust()
                            },
                        };
                        let computed = computed.to_animated_value();
                        Some(AnimationValue::FontSizeAdjust(computed))
                    },
                    LonghandId::FontSynthesis => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_font();
                                initial_struct.clone_font_synthesis()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_font();
                                inherit_struct.clone_font_synthesis()
                            },
                        };
                        Some(AnimationValue::FontSynthesis(computed))
                    },
                    LonghandId::FontStretch => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_font();
                                initial_struct.clone_font_stretch()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_font();
                                inherit_struct.clone_font_stretch()
                            },
                        };
                        Some(AnimationValue::FontStretch(computed))
                    },
                    LonghandId::FontKerning => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_font();
                                initial_struct.clone_font_kerning()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_font();
                                inherit_struct.clone_font_kerning()
                            },
                        };
                        Some(AnimationValue::FontKerning(computed))
                    },
                    LonghandId::FontVariantAlternates => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_font();
                                initial_struct.clone_font_variant_alternates()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_font();
                                inherit_struct.clone_font_variant_alternates()
                            },
                        };
                        Some(AnimationValue::FontVariantAlternates(computed))
                    },
                    LonghandId::FontVariantEastAsian => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_font();
                                initial_struct.clone_font_variant_east_asian()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_font();
                                inherit_struct.clone_font_variant_east_asian()
                            },
                        };
                        Some(AnimationValue::FontVariantEastAsian(computed))
                    },
                    LonghandId::FontVariantLigatures => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_font();
                                initial_struct.clone_font_variant_ligatures()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_font();
                                inherit_struct.clone_font_variant_ligatures()
                            },
                        };
                        Some(AnimationValue::FontVariantLigatures(computed))
                    },
                    LonghandId::FontVariantNumeric => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_font();
                                initial_struct.clone_font_variant_numeric()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_font();
                                inherit_struct.clone_font_variant_numeric()
                            },
                        };
                        Some(AnimationValue::FontVariantNumeric(computed))
                    },
                    LonghandId::FontVariantPosition => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_font();
                                initial_struct.clone_font_variant_position()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_font();
                                inherit_struct.clone_font_variant_position()
                            },
                        };
                        Some(AnimationValue::FontVariantPosition(computed))
                    },
                    LonghandId::FontFeatureSettings => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_font();
                                initial_struct.clone_font_feature_settings()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_font();
                                inherit_struct.clone_font_feature_settings()
                            },
                        };
                        Some(AnimationValue::FontFeatureSettings(computed))
                    },
                    LonghandId::FontVariationSettings => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_font();
                                initial_struct.clone_font_variation_settings()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_font();
                                inherit_struct.clone_font_variation_settings()
                            },
                        };
                        Some(AnimationValue::FontVariationSettings(computed))
                    },
                    LonghandId::FontLanguageOverride => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_font();
                                initial_struct.clone_font_language_override()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_font();
                                inherit_struct.clone_font_language_override()
                            },
                        };
                        Some(AnimationValue::FontLanguageOverride(computed))
                    },
                    LonghandId::MozOsxFontSmoothing => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_font();
                                initial_struct.clone__moz_osx_font_smoothing()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_font();
                                inherit_struct.clone__moz_osx_font_smoothing()
                            },
                        };
                        Some(AnimationValue::MozOsxFontSmoothing(computed))
                    },
                    LonghandId::Visibility => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_inheritedbox();
                                initial_struct.clone_visibility()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_inheritedbox();
                                inherit_struct.clone_visibility()
                            },
                        };
                        Some(AnimationValue::Visibility(computed))
                    },
                    LonghandId::WritingMode => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_inheritedbox();
                                initial_struct.clone_writing_mode()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_inheritedbox();
                                inherit_struct.clone_writing_mode()
                            },
                        };
                        Some(AnimationValue::WritingMode(computed))
                    },
                    LonghandId::Direction => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_inheritedbox();
                                initial_struct.clone_direction()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_inheritedbox();
                                inherit_struct.clone_direction()
                            },
                        };
                        Some(AnimationValue::Direction(computed))
                    },
                    LonghandId::TextOrientation => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_inheritedbox();
                                initial_struct.clone_text_orientation()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_inheritedbox();
                                inherit_struct.clone_text_orientation()
                            },
                        };
                        Some(AnimationValue::TextOrientation(computed))
                    },
                    LonghandId::ColorAdjust => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_inheritedbox();
                                initial_struct.clone_color_adjust()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_inheritedbox();
                                inherit_struct.clone_color_adjust()
                            },
                        };
                        Some(AnimationValue::ColorAdjust(computed))
                    },
                    LonghandId::ImageRendering => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_inheritedbox();
                                initial_struct.clone_image_rendering()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_inheritedbox();
                                inherit_struct.clone_image_rendering()
                            },
                        };
                        Some(AnimationValue::ImageRendering(computed))
                    },
                    LonghandId::ImageOrientation => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_inheritedbox();
                                initial_struct.clone_image_orientation()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_inheritedbox();
                                inherit_struct.clone_image_orientation()
                            },
                        };
                        Some(AnimationValue::ImageOrientation(computed))
                    },
                    LonghandId::BorderCollapse => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_inheritedtable();
                                initial_struct.clone_border_collapse()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_inheritedtable();
                                inherit_struct.clone_border_collapse()
                            },
                        };
                        Some(AnimationValue::BorderCollapse(computed))
                    },
                    LonghandId::EmptyCells => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_inheritedtable();
                                initial_struct.clone_empty_cells()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_inheritedtable();
                                inherit_struct.clone_empty_cells()
                            },
                        };
                        Some(AnimationValue::EmptyCells(computed))
                    },
                    LonghandId::CaptionSide => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_inheritedtable();
                                initial_struct.clone_caption_side()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_inheritedtable();
                                inherit_struct.clone_caption_side()
                            },
                        };
                        Some(AnimationValue::CaptionSide(computed))
                    },
                    LonghandId::BorderSpacing => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_inheritedtable();
                                initial_struct.clone_border_spacing()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_inheritedtable();
                                inherit_struct.clone_border_spacing()
                            },
                        };
                        let computed = computed.to_animated_value();
                        Some(AnimationValue::BorderSpacing(computed))
                    },
                    LonghandId::LineHeight => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_inheritedtext();
                                initial_struct.clone_line_height()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_inheritedtext();
                                inherit_struct.clone_line_height()
                            },
                        };
                        let computed = computed.to_animated_value();
                        Some(AnimationValue::LineHeight(computed))
                    },
                    LonghandId::TextTransform => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_inheritedtext();
                                initial_struct.clone_text_transform()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_inheritedtext();
                                inherit_struct.clone_text_transform()
                            },
                        };
                        Some(AnimationValue::TextTransform(computed))
                    },
                    LonghandId::Hyphens => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_inheritedtext();
                                initial_struct.clone_hyphens()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_inheritedtext();
                                inherit_struct.clone_hyphens()
                            },
                        };
                        Some(AnimationValue::Hyphens(computed))
                    },
                    LonghandId::MozTextSizeAdjust => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_inheritedtext();
                                initial_struct.clone__moz_text_size_adjust()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_inheritedtext();
                                inherit_struct.clone__moz_text_size_adjust()
                            },
                        };
                        Some(AnimationValue::MozTextSizeAdjust(computed))
                    },
                    LonghandId::TextIndent => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_inheritedtext();
                                initial_struct.clone_text_indent()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_inheritedtext();
                                inherit_struct.clone_text_indent()
                            },
                        };
                        Some(AnimationValue::TextIndent(computed))
                    },
                    LonghandId::OverflowWrap => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_inheritedtext();
                                initial_struct.clone_overflow_wrap()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_inheritedtext();
                                inherit_struct.clone_overflow_wrap()
                            },
                        };
                        Some(AnimationValue::OverflowWrap(computed))
                    },
                    LonghandId::WordBreak => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_inheritedtext();
                                initial_struct.clone_word_break()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_inheritedtext();
                                inherit_struct.clone_word_break()
                            },
                        };
                        Some(AnimationValue::WordBreak(computed))
                    },
                    LonghandId::TextJustify => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_inheritedtext();
                                initial_struct.clone_text_justify()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_inheritedtext();
                                inherit_struct.clone_text_justify()
                            },
                        };
                        Some(AnimationValue::TextJustify(computed))
                    },
                    LonghandId::TextAlignLast => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_inheritedtext();
                                initial_struct.clone_text_align_last()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_inheritedtext();
                                inherit_struct.clone_text_align_last()
                            },
                        };
                        Some(AnimationValue::TextAlignLast(computed))
                    },
                    LonghandId::TextAlign => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_inheritedtext();
                                initial_struct.clone_text_align()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_inheritedtext();
                                inherit_struct.clone_text_align()
                            },
                        };
                        Some(AnimationValue::TextAlign(computed))
                    },
                    LonghandId::LetterSpacing => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_inheritedtext();
                                initial_struct.clone_letter_spacing()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_inheritedtext();
                                inherit_struct.clone_letter_spacing()
                            },
                        };
                        Some(AnimationValue::LetterSpacing(computed))
                    },
                    LonghandId::WordSpacing => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_inheritedtext();
                                initial_struct.clone_word_spacing()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_inheritedtext();
                                inherit_struct.clone_word_spacing()
                            },
                        };
                        Some(AnimationValue::WordSpacing(computed))
                    },
                    LonghandId::WhiteSpace => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_inheritedtext();
                                initial_struct.clone_white_space()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_inheritedtext();
                                inherit_struct.clone_white_space()
                            },
                        };
                        Some(AnimationValue::WhiteSpace(computed))
                    },
                    LonghandId::TextShadow => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_inheritedtext();
                                initial_struct.clone_text_shadow()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_inheritedtext();
                                inherit_struct.clone_text_shadow()
                            },
                        };
                        let computed = computed.to_animated_value();
                        Some(AnimationValue::TextShadow(computed))
                    },
                    LonghandId::TextEmphasisStyle => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_inheritedtext();
                                initial_struct.clone_text_emphasis_style()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_inheritedtext();
                                inherit_struct.clone_text_emphasis_style()
                            },
                        };
                        Some(AnimationValue::TextEmphasisStyle(computed))
                    },
                    LonghandId::TextEmphasisPosition => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_inheritedtext();
                                initial_struct.clone_text_emphasis_position()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_inheritedtext();
                                inherit_struct.clone_text_emphasis_position()
                            },
                        };
                        Some(AnimationValue::TextEmphasisPosition(computed))
                    },
                    LonghandId::TextEmphasisColor => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_inheritedtext();
                                initial_struct.clone_text_emphasis_color()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_inheritedtext();
                                inherit_struct.clone_text_emphasis_color()
                            },
                        };
                        let computed = computed.to_animated_value();
                        Some(AnimationValue::TextEmphasisColor(computed))
                    },
                    LonghandId::MozTabSize => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_inheritedtext();
                                initial_struct.clone__moz_tab_size()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_inheritedtext();
                                inherit_struct.clone__moz_tab_size()
                            },
                        };
                        let computed = computed.to_animated_value();
                        Some(AnimationValue::MozTabSize(computed))
                    },
                    LonghandId::WebkitTextFillColor => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_inheritedtext();
                                initial_struct.clone__webkit_text_fill_color()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_inheritedtext();
                                inherit_struct.clone__webkit_text_fill_color()
                            },
                        };
                        let computed = computed.to_animated_value();
                        Some(AnimationValue::WebkitTextFillColor(computed))
                    },
                    LonghandId::WebkitTextStrokeColor => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_inheritedtext();
                                initial_struct.clone__webkit_text_stroke_color()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_inheritedtext();
                                inherit_struct.clone__webkit_text_stroke_color()
                            },
                        };
                        let computed = computed.to_animated_value();
                        Some(AnimationValue::WebkitTextStrokeColor(computed))
                    },
                    LonghandId::WebkitTextStrokeWidth => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_inheritedtext();
                                initial_struct.clone__webkit_text_stroke_width()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_inheritedtext();
                                inherit_struct.clone__webkit_text_stroke_width()
                            },
                        };
                        Some(AnimationValue::WebkitTextStrokeWidth(computed))
                    },
                    LonghandId::RubyAlign => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_inheritedtext();
                                initial_struct.clone_ruby_align()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_inheritedtext();
                                inherit_struct.clone_ruby_align()
                            },
                        };
                        Some(AnimationValue::RubyAlign(computed))
                    },
                    LonghandId::RubyPosition => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_inheritedtext();
                                initial_struct.clone_ruby_position()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_inheritedtext();
                                inherit_struct.clone_ruby_position()
                            },
                        };
                        Some(AnimationValue::RubyPosition(computed))
                    },
                    LonghandId::TextCombineUpright => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_inheritedtext();
                                initial_struct.clone_text_combine_upright()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_inheritedtext();
                                inherit_struct.clone_text_combine_upright()
                            },
                        };
                        Some(AnimationValue::TextCombineUpright(computed))
                    },
                    LonghandId::TextRendering => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_inheritedtext();
                                initial_struct.clone_text_rendering()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_inheritedtext();
                                inherit_struct.clone_text_rendering()
                            },
                        };
                        Some(AnimationValue::TextRendering(computed))
                    },
                    LonghandId::ListStylePosition => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_list();
                                initial_struct.clone_list_style_position()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_list();
                                inherit_struct.clone_list_style_position()
                            },
                        };
                        Some(AnimationValue::ListStylePosition(computed))
                    },
                    LonghandId::ListStyleType => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_list();
                                initial_struct.clone_list_style_type()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_list();
                                inherit_struct.clone_list_style_type()
                            },
                        };
                        Some(AnimationValue::ListStyleType(computed))
                    },
                    LonghandId::ListStyleImage => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_list();
                                initial_struct.clone_list_style_image()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_list();
                                inherit_struct.clone_list_style_image()
                            },
                        };
                        Some(AnimationValue::ListStyleImage(computed))
                    },
                    LonghandId::Quotes => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_list();
                                initial_struct.clone_quotes()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_list();
                                inherit_struct.clone_quotes()
                            },
                        };
                        Some(AnimationValue::Quotes(computed))
                    },
                    LonghandId::MozImageRegion => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_list();
                                initial_struct.clone__moz_image_region()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_list();
                                inherit_struct.clone__moz_image_region()
                            },
                        };
                        Some(AnimationValue::MozImageRegion(computed))
                    },
                    LonghandId::MarginTop => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_margin();
                                initial_struct.clone_margin_top()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_margin();
                                inherit_struct.clone_margin_top()
                            },
                        };
                        Some(AnimationValue::MarginTop(computed))
                    },
                    LonghandId::MarginLeft => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_margin();
                                initial_struct.clone_margin_left()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_margin();
                                inherit_struct.clone_margin_left()
                            },
                        };
                        Some(AnimationValue::MarginLeft(computed))
                    },
                    LonghandId::MarginBottom => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_margin();
                                initial_struct.clone_margin_bottom()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_margin();
                                inherit_struct.clone_margin_bottom()
                            },
                        };
                        Some(AnimationValue::MarginBottom(computed))
                    },
                    LonghandId::MarginRight => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_margin();
                                initial_struct.clone_margin_right()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_margin();
                                inherit_struct.clone_margin_right()
                            },
                        };
                        Some(AnimationValue::MarginRight(computed))
                    },
                    LonghandId::OutlineColor => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_outline();
                                initial_struct.clone_outline_color()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_outline();
                                inherit_struct.clone_outline_color()
                            },
                        };
                        let computed = computed.to_animated_value();
                        Some(AnimationValue::OutlineColor(computed))
                    },
                    LonghandId::OutlineStyle => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_outline();
                                initial_struct.clone_outline_style()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_outline();
                                inherit_struct.clone_outline_style()
                            },
                        };
                        Some(AnimationValue::OutlineStyle(computed))
                    },
                    LonghandId::OutlineWidth => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_outline();
                                initial_struct.clone_outline_width()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_outline();
                                inherit_struct.clone_outline_width()
                            },
                        };
                        let computed = computed.to_animated_value();
                        Some(AnimationValue::OutlineWidth(computed))
                    },
                    LonghandId::MozOutlineRadiusTopleft => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_outline();
                                initial_struct.clone__moz_outline_radius_topleft()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_outline();
                                inherit_struct.clone__moz_outline_radius_topleft()
                            },
                        };
                        let computed = computed.to_animated_value();
                        Some(AnimationValue::MozOutlineRadiusTopleft(computed))
                    },
                    LonghandId::MozOutlineRadiusTopright => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_outline();
                                initial_struct.clone__moz_outline_radius_topright()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_outline();
                                inherit_struct.clone__moz_outline_radius_topright()
                            },
                        };
                        let computed = computed.to_animated_value();
                        Some(AnimationValue::MozOutlineRadiusTopright(computed))
                    },
                    LonghandId::MozOutlineRadiusBottomright => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_outline();
                                initial_struct.clone__moz_outline_radius_bottomright()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_outline();
                                inherit_struct.clone__moz_outline_radius_bottomright()
                            },
                        };
                        let computed = computed.to_animated_value();
                        Some(AnimationValue::MozOutlineRadiusBottomright(computed))
                    },
                    LonghandId::MozOutlineRadiusBottomleft => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_outline();
                                initial_struct.clone__moz_outline_radius_bottomleft()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_outline();
                                inherit_struct.clone__moz_outline_radius_bottomleft()
                            },
                        };
                        let computed = computed.to_animated_value();
                        Some(AnimationValue::MozOutlineRadiusBottomleft(computed))
                    },
                    LonghandId::OutlineOffset => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_outline();
                                initial_struct.clone_outline_offset()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_outline();
                                inherit_struct.clone_outline_offset()
                            },
                        };
                        Some(AnimationValue::OutlineOffset(computed))
                    },
                    LonghandId::PaddingTop => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_padding();
                                initial_struct.clone_padding_top()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_padding();
                                inherit_struct.clone_padding_top()
                            },
                        };
                        let computed = computed.to_animated_value();
                        Some(AnimationValue::PaddingTop(computed))
                    },
                    LonghandId::PaddingLeft => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_padding();
                                initial_struct.clone_padding_left()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_padding();
                                inherit_struct.clone_padding_left()
                            },
                        };
                        let computed = computed.to_animated_value();
                        Some(AnimationValue::PaddingLeft(computed))
                    },
                    LonghandId::PaddingBottom => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_padding();
                                initial_struct.clone_padding_bottom()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_padding();
                                inherit_struct.clone_padding_bottom()
                            },
                        };
                        let computed = computed.to_animated_value();
                        Some(AnimationValue::PaddingBottom(computed))
                    },
                    LonghandId::PaddingRight => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_padding();
                                initial_struct.clone_padding_right()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_padding();
                                inherit_struct.clone_padding_right()
                            },
                        };
                        let computed = computed.to_animated_value();
                        Some(AnimationValue::PaddingRight(computed))
                    },
                    LonghandId::Cursor => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_pointing();
                                initial_struct.clone_cursor()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_pointing();
                                inherit_struct.clone_cursor()
                            },
                        };
                        Some(AnimationValue::Cursor(computed))
                    },
                    LonghandId::PointerEvents => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_pointing();
                                initial_struct.clone_pointer_events()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_pointing();
                                inherit_struct.clone_pointer_events()
                            },
                        };
                        Some(AnimationValue::PointerEvents(computed))
                    },
                    LonghandId::MozUserInput => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_pointing();
                                initial_struct.clone__moz_user_input()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_pointing();
                                inherit_struct.clone__moz_user_input()
                            },
                        };
                        Some(AnimationValue::MozUserInput(computed))
                    },
                    LonghandId::MozUserModify => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_pointing();
                                initial_struct.clone__moz_user_modify()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_pointing();
                                inherit_struct.clone__moz_user_modify()
                            },
                        };
                        Some(AnimationValue::MozUserModify(computed))
                    },
                    LonghandId::MozUserFocus => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_pointing();
                                initial_struct.clone__moz_user_focus()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_pointing();
                                inherit_struct.clone__moz_user_focus()
                            },
                        };
                        Some(AnimationValue::MozUserFocus(computed))
                    },
                    LonghandId::CaretColor => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_pointing();
                                initial_struct.clone_caret_color()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_pointing();
                                inherit_struct.clone_caret_color()
                            },
                        };
                        let computed = computed.to_animated_value();
                        Some(AnimationValue::CaretColor(computed))
                    },
                    LonghandId::Top => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_position();
                                initial_struct.clone_top()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_position();
                                inherit_struct.clone_top()
                            },
                        };
                        Some(AnimationValue::Top(computed))
                    },
                    LonghandId::Left => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_position();
                                initial_struct.clone_left()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_position();
                                inherit_struct.clone_left()
                            },
                        };
                        Some(AnimationValue::Left(computed))
                    },
                    LonghandId::Bottom => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_position();
                                initial_struct.clone_bottom()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_position();
                                inherit_struct.clone_bottom()
                            },
                        };
                        Some(AnimationValue::Bottom(computed))
                    },
                    LonghandId::Right => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_position();
                                initial_struct.clone_right()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_position();
                                inherit_struct.clone_right()
                            },
                        };
                        Some(AnimationValue::Right(computed))
                    },
                    LonghandId::ZIndex => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_position();
                                initial_struct.clone_z_index()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_position();
                                inherit_struct.clone_z_index()
                            },
                        };
                        Some(AnimationValue::ZIndex(computed))
                    },
                    LonghandId::FlexDirection => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_position();
                                initial_struct.clone_flex_direction()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_position();
                                inherit_struct.clone_flex_direction()
                            },
                        };
                        Some(AnimationValue::FlexDirection(computed))
                    },
                    LonghandId::FlexWrap => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_position();
                                initial_struct.clone_flex_wrap()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_position();
                                inherit_struct.clone_flex_wrap()
                            },
                        };
                        Some(AnimationValue::FlexWrap(computed))
                    },
                    LonghandId::JustifyContent => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_position();
                                initial_struct.clone_justify_content()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_position();
                                inherit_struct.clone_justify_content()
                            },
                        };
                        Some(AnimationValue::JustifyContent(computed))
                    },
                    LonghandId::AlignContent => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_position();
                                initial_struct.clone_align_content()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_position();
                                inherit_struct.clone_align_content()
                            },
                        };
                        Some(AnimationValue::AlignContent(computed))
                    },
                    LonghandId::AlignItems => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_position();
                                initial_struct.clone_align_items()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_position();
                                inherit_struct.clone_align_items()
                            },
                        };
                        Some(AnimationValue::AlignItems(computed))
                    },
                    LonghandId::JustifyItems => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_position();
                                initial_struct.clone_justify_items()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_position();
                                inherit_struct.clone_justify_items()
                            },
                        };
                        Some(AnimationValue::JustifyItems(computed))
                    },
                    LonghandId::FlexGrow => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_position();
                                initial_struct.clone_flex_grow()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_position();
                                inherit_struct.clone_flex_grow()
                            },
                        };
                        let computed = computed.to_animated_value();
                        Some(AnimationValue::FlexGrow(computed))
                    },
                    LonghandId::FlexShrink => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_position();
                                initial_struct.clone_flex_shrink()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_position();
                                inherit_struct.clone_flex_shrink()
                            },
                        };
                        let computed = computed.to_animated_value();
                        Some(AnimationValue::FlexShrink(computed))
                    },
                    LonghandId::AlignSelf => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_position();
                                initial_struct.clone_align_self()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_position();
                                inherit_struct.clone_align_self()
                            },
                        };
                        Some(AnimationValue::AlignSelf(computed))
                    },
                    LonghandId::JustifySelf => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_position();
                                initial_struct.clone_justify_self()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_position();
                                inherit_struct.clone_justify_self()
                            },
                        };
                        Some(AnimationValue::JustifySelf(computed))
                    },
                    LonghandId::Order => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_position();
                                initial_struct.clone_order()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_position();
                                inherit_struct.clone_order()
                            },
                        };
                        Some(AnimationValue::Order(computed))
                    },
                    LonghandId::FlexBasis => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_position();
                                initial_struct.clone_flex_basis()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_position();
                                inherit_struct.clone_flex_basis()
                            },
                        };
                        let computed = computed.to_animated_value();
                        Some(AnimationValue::FlexBasis(computed))
                    },
                    LonghandId::Width => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_position();
                                initial_struct.clone_width()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_position();
                                inherit_struct.clone_width()
                            },
                        };
                        let computed = computed.to_animated_value();
                        Some(AnimationValue::Width(computed))
                    },
                    LonghandId::MinWidth => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_position();
                                initial_struct.clone_min_width()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_position();
                                inherit_struct.clone_min_width()
                            },
                        };
                        let computed = computed.to_animated_value();
                        Some(AnimationValue::MinWidth(computed))
                    },
                    LonghandId::MaxWidth => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_position();
                                initial_struct.clone_max_width()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_position();
                                inherit_struct.clone_max_width()
                            },
                        };
                        let computed = computed.to_animated_value();
                        Some(AnimationValue::MaxWidth(computed))
                    },
                    LonghandId::Height => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_position();
                                initial_struct.clone_height()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_position();
                                inherit_struct.clone_height()
                            },
                        };
                        let computed = computed.to_animated_value();
                        Some(AnimationValue::Height(computed))
                    },
                    LonghandId::MinHeight => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_position();
                                initial_struct.clone_min_height()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_position();
                                inherit_struct.clone_min_height()
                            },
                        };
                        let computed = computed.to_animated_value();
                        Some(AnimationValue::MinHeight(computed))
                    },
                    LonghandId::MaxHeight => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_position();
                                initial_struct.clone_max_height()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_position();
                                inherit_struct.clone_max_height()
                            },
                        };
                        let computed = computed.to_animated_value();
                        Some(AnimationValue::MaxHeight(computed))
                    },
                    LonghandId::BoxSizing => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_position();
                                initial_struct.clone_box_sizing()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_position();
                                inherit_struct.clone_box_sizing()
                            },
                        };
                        Some(AnimationValue::BoxSizing(computed))
                    },
                    LonghandId::ObjectFit => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_position();
                                initial_struct.clone_object_fit()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_position();
                                inherit_struct.clone_object_fit()
                            },
                        };
                        Some(AnimationValue::ObjectFit(computed))
                    },
                    LonghandId::ObjectPosition => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_position();
                                initial_struct.clone_object_position()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_position();
                                inherit_struct.clone_object_position()
                            },
                        };
                        Some(AnimationValue::ObjectPosition(computed))
                    },
                    LonghandId::GridRowGap => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_position();
                                initial_struct.clone_grid_row_gap()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_position();
                                inherit_struct.clone_grid_row_gap()
                            },
                        };
                        let computed = computed.to_animated_value();
                        Some(AnimationValue::GridRowGap(computed))
                    },
                    LonghandId::GridRowStart => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_position();
                                initial_struct.clone_grid_row_start()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_position();
                                inherit_struct.clone_grid_row_start()
                            },
                        };
                        Some(AnimationValue::GridRowStart(computed))
                    },
                    LonghandId::GridRowEnd => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_position();
                                initial_struct.clone_grid_row_end()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_position();
                                inherit_struct.clone_grid_row_end()
                            },
                        };
                        Some(AnimationValue::GridRowEnd(computed))
                    },
                    LonghandId::GridAutoRows => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_position();
                                initial_struct.clone_grid_auto_rows()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_position();
                                inherit_struct.clone_grid_auto_rows()
                            },
                        };
                        Some(AnimationValue::GridAutoRows(computed))
                    },
                    LonghandId::GridTemplateRows => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_position();
                                initial_struct.clone_grid_template_rows()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_position();
                                inherit_struct.clone_grid_template_rows()
                            },
                        };
                        Some(AnimationValue::GridTemplateRows(computed))
                    },
                    LonghandId::GridColumnGap => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_position();
                                initial_struct.clone_grid_column_gap()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_position();
                                inherit_struct.clone_grid_column_gap()
                            },
                        };
                        let computed = computed.to_animated_value();
                        Some(AnimationValue::GridColumnGap(computed))
                    },
                    LonghandId::GridColumnStart => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_position();
                                initial_struct.clone_grid_column_start()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_position();
                                inherit_struct.clone_grid_column_start()
                            },
                        };
                        Some(AnimationValue::GridColumnStart(computed))
                    },
                    LonghandId::GridColumnEnd => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_position();
                                initial_struct.clone_grid_column_end()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_position();
                                inherit_struct.clone_grid_column_end()
                            },
                        };
                        Some(AnimationValue::GridColumnEnd(computed))
                    },
                    LonghandId::GridAutoColumns => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_position();
                                initial_struct.clone_grid_auto_columns()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_position();
                                inherit_struct.clone_grid_auto_columns()
                            },
                        };
                        Some(AnimationValue::GridAutoColumns(computed))
                    },
                    LonghandId::GridTemplateColumns => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_position();
                                initial_struct.clone_grid_template_columns()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_position();
                                inherit_struct.clone_grid_template_columns()
                            },
                        };
                        Some(AnimationValue::GridTemplateColumns(computed))
                    },
                    LonghandId::GridAutoFlow => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_position();
                                initial_struct.clone_grid_auto_flow()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_position();
                                inherit_struct.clone_grid_auto_flow()
                            },
                        };
                        Some(AnimationValue::GridAutoFlow(computed))
                    },
                    LonghandId::GridTemplateAreas => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_position();
                                initial_struct.clone_grid_template_areas()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_position();
                                inherit_struct.clone_grid_template_areas()
                            },
                        };
                        Some(AnimationValue::GridTemplateAreas(computed))
                    },
                    LonghandId::TableLayout => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_table();
                                initial_struct.clone_table_layout()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_table();
                                inherit_struct.clone_table_layout()
                            },
                        };
                        Some(AnimationValue::TableLayout(computed))
                    },
                    LonghandId::TextOverflow => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_text();
                                initial_struct.clone_text_overflow()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_text();
                                inherit_struct.clone_text_overflow()
                            },
                        };
                        Some(AnimationValue::TextOverflow(computed))
                    },
                    LonghandId::UnicodeBidi => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_text();
                                initial_struct.clone_unicode_bidi()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_text();
                                inherit_struct.clone_unicode_bidi()
                            },
                        };
                        Some(AnimationValue::UnicodeBidi(computed))
                    },
                    LonghandId::TextDecorationLine => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_text();
                                initial_struct.clone_text_decoration_line()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_text();
                                inherit_struct.clone_text_decoration_line()
                            },
                        };
                        Some(AnimationValue::TextDecorationLine(computed))
                    },
                    LonghandId::TextDecorationStyle => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_text();
                                initial_struct.clone_text_decoration_style()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_text();
                                inherit_struct.clone_text_decoration_style()
                            },
                        };
                        Some(AnimationValue::TextDecorationStyle(computed))
                    },
                    LonghandId::TextDecorationColor => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_text();
                                initial_struct.clone_text_decoration_color()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_text();
                                inherit_struct.clone_text_decoration_color()
                            },
                        };
                        let computed = computed.to_animated_value();
                        Some(AnimationValue::TextDecorationColor(computed))
                    },
                    LonghandId::InitialLetter => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_text();
                                initial_struct.clone_initial_letter()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_text();
                                inherit_struct.clone_initial_letter()
                            },
                        };
                        Some(AnimationValue::InitialLetter(computed))
                    },
                    LonghandId::ImeMode => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_ui();
                                initial_struct.clone_ime_mode()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_ui();
                                inherit_struct.clone_ime_mode()
                            },
                        };
                        Some(AnimationValue::ImeMode(computed))
                    },
                    LonghandId::MozUserSelect => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_ui();
                                initial_struct.clone__moz_user_select()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_ui();
                                inherit_struct.clone__moz_user_select()
                            },
                        };
                        Some(AnimationValue::MozUserSelect(computed))
                    },
                    LonghandId::MozWindowDragging => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_ui();
                                initial_struct.clone__moz_window_dragging()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_ui();
                                inherit_struct.clone__moz_window_dragging()
                            },
                        };
                        Some(AnimationValue::MozWindowDragging(computed))
                    },
                    LonghandId::MozWindowShadow => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_ui();
                                initial_struct.clone__moz_window_shadow()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_ui();
                                inherit_struct.clone__moz_window_shadow()
                            },
                        };
                        Some(AnimationValue::MozWindowShadow(computed))
                    },
                    LonghandId::MozForceBrokenImageIcon => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_ui();
                                initial_struct.clone__moz_force_broken_image_icon()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_ui();
                                inherit_struct.clone__moz_force_broken_image_icon()
                            },
                        };
                        Some(AnimationValue::MozForceBrokenImageIcon(computed))
                    },
                    LonghandId::TextAnchor => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_inheritedsvg();
                                initial_struct.clone_text_anchor()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_inheritedsvg();
                                inherit_struct.clone_text_anchor()
                            },
                        };
                        Some(AnimationValue::TextAnchor(computed))
                    },
                    LonghandId::ColorInterpolation => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_inheritedsvg();
                                initial_struct.clone_color_interpolation()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_inheritedsvg();
                                inherit_struct.clone_color_interpolation()
                            },
                        };
                        Some(AnimationValue::ColorInterpolation(computed))
                    },
                    LonghandId::ColorInterpolationFilters => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_inheritedsvg();
                                initial_struct.clone_color_interpolation_filters()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_inheritedsvg();
                                inherit_struct.clone_color_interpolation_filters()
                            },
                        };
                        Some(AnimationValue::ColorInterpolationFilters(computed))
                    },
                    LonghandId::Fill => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_inheritedsvg();
                                initial_struct.clone_fill()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_inheritedsvg();
                                inherit_struct.clone_fill()
                            },
                        };
                        let computed = computed.to_animated_value();
                        Some(AnimationValue::Fill(computed))
                    },
                    LonghandId::FillOpacity => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_inheritedsvg();
                                initial_struct.clone_fill_opacity()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_inheritedsvg();
                                inherit_struct.clone_fill_opacity()
                            },
                        };
                        Some(AnimationValue::FillOpacity(computed))
                    },
                    LonghandId::FillRule => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_inheritedsvg();
                                initial_struct.clone_fill_rule()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_inheritedsvg();
                                inherit_struct.clone_fill_rule()
                            },
                        };
                        Some(AnimationValue::FillRule(computed))
                    },
                    LonghandId::ShapeRendering => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_inheritedsvg();
                                initial_struct.clone_shape_rendering()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_inheritedsvg();
                                inherit_struct.clone_shape_rendering()
                            },
                        };
                        Some(AnimationValue::ShapeRendering(computed))
                    },
                    LonghandId::Stroke => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_inheritedsvg();
                                initial_struct.clone_stroke()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_inheritedsvg();
                                inherit_struct.clone_stroke()
                            },
                        };
                        let computed = computed.to_animated_value();
                        Some(AnimationValue::Stroke(computed))
                    },
                    LonghandId::StrokeWidth => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_inheritedsvg();
                                initial_struct.clone_stroke_width()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_inheritedsvg();
                                inherit_struct.clone_stroke_width()
                            },
                        };
                        let computed = computed.to_animated_value();
                        Some(AnimationValue::StrokeWidth(computed))
                    },
                    LonghandId::StrokeLinecap => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_inheritedsvg();
                                initial_struct.clone_stroke_linecap()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_inheritedsvg();
                                inherit_struct.clone_stroke_linecap()
                            },
                        };
                        Some(AnimationValue::StrokeLinecap(computed))
                    },
                    LonghandId::StrokeLinejoin => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_inheritedsvg();
                                initial_struct.clone_stroke_linejoin()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_inheritedsvg();
                                inherit_struct.clone_stroke_linejoin()
                            },
                        };
                        Some(AnimationValue::StrokeLinejoin(computed))
                    },
                    LonghandId::StrokeMiterlimit => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_inheritedsvg();
                                initial_struct.clone_stroke_miterlimit()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_inheritedsvg();
                                inherit_struct.clone_stroke_miterlimit()
                            },
                        };
                        let computed = computed.to_animated_value();
                        Some(AnimationValue::StrokeMiterlimit(computed))
                    },
                    LonghandId::StrokeOpacity => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_inheritedsvg();
                                initial_struct.clone_stroke_opacity()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_inheritedsvg();
                                inherit_struct.clone_stroke_opacity()
                            },
                        };
                        Some(AnimationValue::StrokeOpacity(computed))
                    },
                    LonghandId::StrokeDasharray => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_inheritedsvg();
                                initial_struct.clone_stroke_dasharray()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_inheritedsvg();
                                inherit_struct.clone_stroke_dasharray()
                            },
                        };
                        let computed = computed.to_animated_value();
                        Some(AnimationValue::StrokeDasharray(computed))
                    },
                    LonghandId::StrokeDashoffset => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_inheritedsvg();
                                initial_struct.clone_stroke_dashoffset()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_inheritedsvg();
                                inherit_struct.clone_stroke_dashoffset()
                            },
                        };
                        Some(AnimationValue::StrokeDashoffset(computed))
                    },
                    LonghandId::ClipRule => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_inheritedsvg();
                                initial_struct.clone_clip_rule()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_inheritedsvg();
                                inherit_struct.clone_clip_rule()
                            },
                        };
                        Some(AnimationValue::ClipRule(computed))
                    },
                    LonghandId::MarkerStart => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_inheritedsvg();
                                initial_struct.clone_marker_start()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_inheritedsvg();
                                inherit_struct.clone_marker_start()
                            },
                        };
                        Some(AnimationValue::MarkerStart(computed))
                    },
                    LonghandId::MarkerMid => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_inheritedsvg();
                                initial_struct.clone_marker_mid()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_inheritedsvg();
                                inherit_struct.clone_marker_mid()
                            },
                        };
                        Some(AnimationValue::MarkerMid(computed))
                    },
                    LonghandId::MarkerEnd => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_inheritedsvg();
                                initial_struct.clone_marker_end()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_inheritedsvg();
                                inherit_struct.clone_marker_end()
                            },
                        };
                        Some(AnimationValue::MarkerEnd(computed))
                    },
                    LonghandId::PaintOrder => {
                        let computed = match keyword {
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_inheritedsvg();
                                initial_struct.clone_paint_order()
                            },
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_inheritedsvg();
                                inherit_struct.clone_paint_order()
                            },
                        };
                        Some(AnimationValue::PaintOrder(computed))
                    },
                    LonghandId::DominantBaseline => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_svg();
                                initial_struct.clone_dominant_baseline()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_svg();
                                inherit_struct.clone_dominant_baseline()
                            },
                        };
                        Some(AnimationValue::DominantBaseline(computed))
                    },
                    LonghandId::VectorEffect => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_svg();
                                initial_struct.clone_vector_effect()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_svg();
                                inherit_struct.clone_vector_effect()
                            },
                        };
                        Some(AnimationValue::VectorEffect(computed))
                    },
                    LonghandId::StopColor => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_svg();
                                initial_struct.clone_stop_color()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_svg();
                                inherit_struct.clone_stop_color()
                            },
                        };
                        let computed = computed.to_animated_value();
                        Some(AnimationValue::StopColor(computed))
                    },
                    LonghandId::StopOpacity => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_svg();
                                initial_struct.clone_stop_opacity()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_svg();
                                inherit_struct.clone_stop_opacity()
                            },
                        };
                        Some(AnimationValue::StopOpacity(computed))
                    },
                    LonghandId::FloodColor => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_svg();
                                initial_struct.clone_flood_color()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_svg();
                                inherit_struct.clone_flood_color()
                            },
                        };
                        let computed = computed.to_animated_value();
                        Some(AnimationValue::FloodColor(computed))
                    },
                    LonghandId::FloodOpacity => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_svg();
                                initial_struct.clone_flood_opacity()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_svg();
                                inherit_struct.clone_flood_opacity()
                            },
                        };
                        Some(AnimationValue::FloodOpacity(computed))
                    },
                    LonghandId::LightingColor => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_svg();
                                initial_struct.clone_lighting_color()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_svg();
                                inherit_struct.clone_lighting_color()
                            },
                        };
                        let computed = computed.to_animated_value();
                        Some(AnimationValue::LightingColor(computed))
                    },
                    LonghandId::MaskType => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_svg();
                                initial_struct.clone_mask_type()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_svg();
                                inherit_struct.clone_mask_type()
                            },
                        };
                        Some(AnimationValue::MaskType(computed))
                    },
                    LonghandId::ClipPath => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_svg();
                                initial_struct.clone_clip_path()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_svg();
                                inherit_struct.clone_clip_path()
                            },
                        };
                        Some(AnimationValue::ClipPath(computed))
                    },
                    LonghandId::MaskMode => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_svg();
                                initial_struct.clone_mask_mode()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_svg();
                                inherit_struct.clone_mask_mode()
                            },
                        };
                        Some(AnimationValue::MaskMode(computed))
                    },
                    LonghandId::MaskRepeat => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_svg();
                                initial_struct.clone_mask_repeat()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_svg();
                                inherit_struct.clone_mask_repeat()
                            },
                        };
                        Some(AnimationValue::MaskRepeat(computed))
                    },
                    LonghandId::MaskPositionX => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_svg();
                                initial_struct.clone_mask_position_x()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_svg();
                                inherit_struct.clone_mask_position_x()
                            },
                        };
                        Some(AnimationValue::MaskPositionX(computed))
                    },
                    LonghandId::MaskPositionY => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_svg();
                                initial_struct.clone_mask_position_y()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_svg();
                                inherit_struct.clone_mask_position_y()
                            },
                        };
                        Some(AnimationValue::MaskPositionY(computed))
                    },
                    LonghandId::MaskClip => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_svg();
                                initial_struct.clone_mask_clip()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_svg();
                                inherit_struct.clone_mask_clip()
                            },
                        };
                        Some(AnimationValue::MaskClip(computed))
                    },
                    LonghandId::MaskOrigin => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_svg();
                                initial_struct.clone_mask_origin()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_svg();
                                inherit_struct.clone_mask_origin()
                            },
                        };
                        Some(AnimationValue::MaskOrigin(computed))
                    },
                    LonghandId::MaskSize => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_svg();
                                initial_struct.clone_mask_size()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_svg();
                                inherit_struct.clone_mask_size()
                            },
                        };
                        Some(AnimationValue::MaskSize(computed))
                    },
                    LonghandId::MaskComposite => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_svg();
                                initial_struct.clone_mask_composite()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_svg();
                                inherit_struct.clone_mask_composite()
                            },
                        };
                        Some(AnimationValue::MaskComposite(computed))
                    },
                    LonghandId::MaskImage => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_svg();
                                initial_struct.clone_mask_image()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_svg();
                                inherit_struct.clone_mask_image()
                            },
                        };
                        Some(AnimationValue::MaskImage(computed))
                    },
                    LonghandId::MozBoxAlign => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_xul();
                                initial_struct.clone__moz_box_align()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_xul();
                                inherit_struct.clone__moz_box_align()
                            },
                        };
                        Some(AnimationValue::MozBoxAlign(computed))
                    },
                    LonghandId::MozBoxDirection => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_xul();
                                initial_struct.clone__moz_box_direction()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_xul();
                                inherit_struct.clone__moz_box_direction()
                            },
                        };
                        Some(AnimationValue::MozBoxDirection(computed))
                    },
                    LonghandId::MozBoxFlex => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_xul();
                                initial_struct.clone__moz_box_flex()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_xul();
                                inherit_struct.clone__moz_box_flex()
                            },
                        };
                        let computed = computed.to_animated_value();
                        Some(AnimationValue::MozBoxFlex(computed))
                    },
                    LonghandId::MozBoxOrient => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_xul();
                                initial_struct.clone__moz_box_orient()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_xul();
                                inherit_struct.clone__moz_box_orient()
                            },
                        };
                        Some(AnimationValue::MozBoxOrient(computed))
                    },
                    LonghandId::MozBoxPack => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_xul();
                                initial_struct.clone__moz_box_pack()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_xul();
                                inherit_struct.clone__moz_box_pack()
                            },
                        };
                        Some(AnimationValue::MozBoxPack(computed))
                    },
                    LonghandId::MozStackSizing => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_xul();
                                initial_struct.clone__moz_stack_sizing()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_xul();
                                inherit_struct.clone__moz_stack_sizing()
                            },
                        };
                        Some(AnimationValue::MozStackSizing(computed))
                    },
                    LonghandId::MozBoxOrdinalGroup => {
                        let computed = match keyword {
                                CSSWideKeyword::Unset |
                            CSSWideKeyword::Initial => {
                                let initial_struct = initial.get_xul();
                                initial_struct.clone__moz_box_ordinal_group()
                            },
                            CSSWideKeyword::Inherit => {
                                let inherit_struct = context.builder
                                                            .get_parent_xul();
                                inherit_struct.clone__moz_box_ordinal_group()
                            },
                        };
                        Some(AnimationValue::MozBoxOrdinalGroup(computed))
                    },
                    LonghandId::BorderBlockStartColor => None,
                    LonghandId::BorderBlockStartStyle => None,
                    LonghandId::BorderBlockStartWidth => None,
                    LonghandId::BorderBlockEndColor => None,
                    LonghandId::BorderBlockEndStyle => None,
                    LonghandId::BorderBlockEndWidth => None,
                    LonghandId::BorderInlineStartColor => None,
                    LonghandId::BorderInlineStartStyle => None,
                    LonghandId::BorderInlineStartWidth => None,
                    LonghandId::BorderInlineEndColor => None,
                    LonghandId::BorderInlineEndStyle => None,
                    LonghandId::BorderInlineEndWidth => None,
                    LonghandId::MozTopLayer => None,
                    LonghandId::TransitionDuration => None,
                    LonghandId::TransitionTimingFunction => None,
                    LonghandId::TransitionProperty => None,
                    LonghandId::TransitionDelay => None,
                    LonghandId::AnimationName => None,
                    LonghandId::AnimationDuration => None,
                    LonghandId::AnimationTimingFunction => None,
                    LonghandId::AnimationIterationCount => None,
                    LonghandId::AnimationDirection => None,
                    LonghandId::AnimationPlayState => None,
                    LonghandId::AnimationFillMode => None,
                    LonghandId::AnimationDelay => None,
                    LonghandId::MozBinding => None,
                    LonghandId::XLang => None,
                    LonghandId::MozScriptSizeMultiplier => None,
                    LonghandId::MozScriptLevel => None,
                    LonghandId::MozMathDisplay => None,
                    LonghandId::MozMathVariant => None,
                    LonghandId::MozScriptMinSize => None,
                    LonghandId::XTextZoom => None,
                    LonghandId::MozMinFontSizeRatio => None,
                    LonghandId::MozControlCharacterVisibility => None,
                    LonghandId::MarginBlockStart => None,
                    LonghandId::MarginBlockEnd => None,
                    LonghandId::MarginInlineStart => None,
                    LonghandId::MarginInlineEnd => None,
                    LonghandId::PaddingBlockStart => None,
                    LonghandId::PaddingBlockEnd => None,
                    LonghandId::PaddingInlineStart => None,
                    LonghandId::PaddingInlineEnd => None,
                    LonghandId::OffsetBlockStart => None,
                    LonghandId::OffsetBlockEnd => None,
                    LonghandId::OffsetInlineStart => None,
                    LonghandId::OffsetInlineEnd => None,
                    LonghandId::BlockSize => None,
                    LonghandId::MinBlockSize => None,
                    LonghandId::MaxBlockSize => None,
                    LonghandId::InlineSize => None,
                    LonghandId::MinInlineSize => None,
                    LonghandId::MaxInlineSize => None,
                    LonghandId::XSpan => None,
                    LonghandId::MozContextProperties => None,
                }
            },
            PropertyDeclaration::WithVariables(id, ref unparsed) => {
                let substituted = if extra_custom_properties.is_some() {
                    unparsed.substitute_variables(
                        id, &extra_custom_properties, context.quirks_mode)
                } else {
                    unparsed.substitute_variables(
                        id, &context.style().custom_properties(), context.quirks_mode)
                };
                AnimationValue::from_declaration(&substituted, context, extra_custom_properties, initial)
            },
            _ => None // non animatable properties will get included because of shorthands. ignore.
        }
    }

    /// Get an AnimationValue for an AnimatableLonghand from a given computed values.
    pub fn from_computed_values(property: &AnimatableLonghand,
                                computed_values: &ComputedValues)
                                -> Self {
        match *property {
                    AnimatableLonghand::BackgroundColor => {
                        let computed = computed_values
                            .get_background()
                            .clone_background_color();
                        AnimationValue::BackgroundColor(
                            computed.to_animated_value()
                        )
                    }
                    AnimatableLonghand::BackgroundImage => {
                        let computed = computed_values
                            .get_background()
                            .clone_background_image();
                        AnimationValue::BackgroundImage(
                            computed
                        )
                    }
                    AnimatableLonghand::BackgroundPositionX => {
                        let computed = computed_values
                            .get_background()
                            .clone_background_position_x();
                        AnimationValue::BackgroundPositionX(
                            computed
                        )
                    }
                    AnimatableLonghand::BackgroundPositionY => {
                        let computed = computed_values
                            .get_background()
                            .clone_background_position_y();
                        AnimationValue::BackgroundPositionY(
                            computed
                        )
                    }
                    AnimatableLonghand::BackgroundRepeat => {
                        let computed = computed_values
                            .get_background()
                            .clone_background_repeat();
                        AnimationValue::BackgroundRepeat(
                            computed
                        )
                    }
                    AnimatableLonghand::BackgroundAttachment => {
                        let computed = computed_values
                            .get_background()
                            .clone_background_attachment();
                        AnimationValue::BackgroundAttachment(
                            computed
                        )
                    }
                    AnimatableLonghand::BackgroundClip => {
                        let computed = computed_values
                            .get_background()
                            .clone_background_clip();
                        AnimationValue::BackgroundClip(
                            computed
                        )
                    }
                    AnimatableLonghand::BackgroundOrigin => {
                        let computed = computed_values
                            .get_background()
                            .clone_background_origin();
                        AnimationValue::BackgroundOrigin(
                            computed
                        )
                    }
                    AnimatableLonghand::BackgroundSize => {
                        let computed = computed_values
                            .get_background()
                            .clone_background_size();
                        AnimationValue::BackgroundSize(
                            computed.to_animated_value()
                        )
                    }
                    AnimatableLonghand::BackgroundBlendMode => {
                        let computed = computed_values
                            .get_background()
                            .clone_background_blend_mode();
                        AnimationValue::BackgroundBlendMode(
                            computed
                        )
                    }
                    AnimatableLonghand::BorderTopColor => {
                        let computed = computed_values
                            .get_border()
                            .clone_border_top_color();
                        AnimationValue::BorderTopColor(
                            computed.to_animated_value()
                        )
                    }
                    AnimatableLonghand::BorderTopStyle => {
                        let computed = computed_values
                            .get_border()
                            .clone_border_top_style();
                        AnimationValue::BorderTopStyle(
                            computed
                        )
                    }
                    AnimatableLonghand::BorderTopWidth => {
                        let computed = computed_values
                            .get_border()
                            .clone_border_top_width();
                        AnimationValue::BorderTopWidth(
                            computed.to_animated_value()
                        )
                    }
                    AnimatableLonghand::BorderLeftColor => {
                        let computed = computed_values
                            .get_border()
                            .clone_border_left_color();
                        AnimationValue::BorderLeftColor(
                            computed.to_animated_value()
                        )
                    }
                    AnimatableLonghand::BorderLeftStyle => {
                        let computed = computed_values
                            .get_border()
                            .clone_border_left_style();
                        AnimationValue::BorderLeftStyle(
                            computed
                        )
                    }
                    AnimatableLonghand::BorderLeftWidth => {
                        let computed = computed_values
                            .get_border()
                            .clone_border_left_width();
                        AnimationValue::BorderLeftWidth(
                            computed.to_animated_value()
                        )
                    }
                    AnimatableLonghand::BorderBottomColor => {
                        let computed = computed_values
                            .get_border()
                            .clone_border_bottom_color();
                        AnimationValue::BorderBottomColor(
                            computed.to_animated_value()
                        )
                    }
                    AnimatableLonghand::BorderBottomStyle => {
                        let computed = computed_values
                            .get_border()
                            .clone_border_bottom_style();
                        AnimationValue::BorderBottomStyle(
                            computed
                        )
                    }
                    AnimatableLonghand::BorderBottomWidth => {
                        let computed = computed_values
                            .get_border()
                            .clone_border_bottom_width();
                        AnimationValue::BorderBottomWidth(
                            computed.to_animated_value()
                        )
                    }
                    AnimatableLonghand::BorderRightColor => {
                        let computed = computed_values
                            .get_border()
                            .clone_border_right_color();
                        AnimationValue::BorderRightColor(
                            computed.to_animated_value()
                        )
                    }
                    AnimatableLonghand::BorderRightStyle => {
                        let computed = computed_values
                            .get_border()
                            .clone_border_right_style();
                        AnimationValue::BorderRightStyle(
                            computed
                        )
                    }
                    AnimatableLonghand::BorderRightWidth => {
                        let computed = computed_values
                            .get_border()
                            .clone_border_right_width();
                        AnimationValue::BorderRightWidth(
                            computed.to_animated_value()
                        )
                    }
                    AnimatableLonghand::BorderTopLeftRadius => {
                        let computed = computed_values
                            .get_border()
                            .clone_border_top_left_radius();
                        AnimationValue::BorderTopLeftRadius(
                            computed.to_animated_value()
                        )
                    }
                    AnimatableLonghand::BorderTopRightRadius => {
                        let computed = computed_values
                            .get_border()
                            .clone_border_top_right_radius();
                        AnimationValue::BorderTopRightRadius(
                            computed.to_animated_value()
                        )
                    }
                    AnimatableLonghand::BorderBottomRightRadius => {
                        let computed = computed_values
                            .get_border()
                            .clone_border_bottom_right_radius();
                        AnimationValue::BorderBottomRightRadius(
                            computed.to_animated_value()
                        )
                    }
                    AnimatableLonghand::BorderBottomLeftRadius => {
                        let computed = computed_values
                            .get_border()
                            .clone_border_bottom_left_radius();
                        AnimationValue::BorderBottomLeftRadius(
                            computed.to_animated_value()
                        )
                    }
                    AnimatableLonghand::MozBorderTopColors => {
                        let computed = computed_values
                            .get_border()
                            .clone__moz_border_top_colors();
                        AnimationValue::MozBorderTopColors(
                            computed
                        )
                    }
                    AnimatableLonghand::MozBorderLeftColors => {
                        let computed = computed_values
                            .get_border()
                            .clone__moz_border_left_colors();
                        AnimationValue::MozBorderLeftColors(
                            computed
                        )
                    }
                    AnimatableLonghand::MozBorderBottomColors => {
                        let computed = computed_values
                            .get_border()
                            .clone__moz_border_bottom_colors();
                        AnimationValue::MozBorderBottomColors(
                            computed
                        )
                    }
                    AnimatableLonghand::MozBorderRightColors => {
                        let computed = computed_values
                            .get_border()
                            .clone__moz_border_right_colors();
                        AnimationValue::MozBorderRightColors(
                            computed
                        )
                    }
                    AnimatableLonghand::BoxDecorationBreak => {
                        let computed = computed_values
                            .get_border()
                            .clone_box_decoration_break();
                        AnimationValue::BoxDecorationBreak(
                            computed
                        )
                    }
                    AnimatableLonghand::MozFloatEdge => {
                        let computed = computed_values
                            .get_border()
                            .clone__moz_float_edge();
                        AnimationValue::MozFloatEdge(
                            computed
                        )
                    }
                    AnimatableLonghand::BorderImageSource => {
                        let computed = computed_values
                            .get_border()
                            .clone_border_image_source();
                        AnimationValue::BorderImageSource(
                            computed
                        )
                    }
                    AnimatableLonghand::BorderImageOutset => {
                        let computed = computed_values
                            .get_border()
                            .clone_border_image_outset();
                        AnimationValue::BorderImageOutset(
                            computed
                        )
                    }
                    AnimatableLonghand::BorderImageRepeat => {
                        let computed = computed_values
                            .get_border()
                            .clone_border_image_repeat();
                        AnimationValue::BorderImageRepeat(
                            computed
                        )
                    }
                    AnimatableLonghand::BorderImageWidth => {
                        let computed = computed_values
                            .get_border()
                            .clone_border_image_width();
                        AnimationValue::BorderImageWidth(
                            computed
                        )
                    }
                    AnimatableLonghand::BorderImageSlice => {
                        let computed = computed_values
                            .get_border()
                            .clone_border_image_slice();
                        AnimationValue::BorderImageSlice(
                            computed
                        )
                    }
                    AnimatableLonghand::Display => {
                        let computed = computed_values
                            .get_box()
                            .clone_display();
                        AnimationValue::Display(
                            computed
                        )
                    }
                    AnimatableLonghand::Position => {
                        let computed = computed_values
                            .get_box()
                            .clone_position();
                        AnimationValue::Position(
                            computed
                        )
                    }
                    AnimatableLonghand::Float => {
                        let computed = computed_values
                            .get_box()
                            .clone_float();
                        AnimationValue::Float(
                            computed
                        )
                    }
                    AnimatableLonghand::Clear => {
                        let computed = computed_values
                            .get_box()
                            .clone_clear();
                        AnimationValue::Clear(
                            computed
                        )
                    }
                    AnimatableLonghand::VerticalAlign => {
                        let computed = computed_values
                            .get_box()
                            .clone_vertical_align();
                        AnimationValue::VerticalAlign(
                            computed
                        )
                    }
                    AnimatableLonghand::OverflowClipBox => {
                        let computed = computed_values
                            .get_box()
                            .clone_overflow_clip_box();
                        AnimationValue::OverflowClipBox(
                            computed
                        )
                    }
                    AnimatableLonghand::OverflowX => {
                        let computed = computed_values
                            .get_box()
                            .clone_overflow_x();
                        AnimationValue::OverflowX(
                            computed
                        )
                    }
                    AnimatableLonghand::OverflowY => {
                        let computed = computed_values
                            .get_box()
                            .clone_overflow_y();
                        AnimationValue::OverflowY(
                            computed
                        )
                    }
                    AnimatableLonghand::ScrollSnapPointsX => {
                        let computed = computed_values
                            .get_box()
                            .clone_scroll_snap_points_x();
                        AnimationValue::ScrollSnapPointsX(
                            computed
                        )
                    }
                    AnimatableLonghand::ScrollSnapPointsY => {
                        let computed = computed_values
                            .get_box()
                            .clone_scroll_snap_points_y();
                        AnimationValue::ScrollSnapPointsY(
                            computed
                        )
                    }
                    AnimatableLonghand::ScrollSnapDestination => {
                        let computed = computed_values
                            .get_box()
                            .clone_scroll_snap_destination();
                        AnimationValue::ScrollSnapDestination(
                            computed
                        )
                    }
                    AnimatableLonghand::ScrollSnapCoordinate => {
                        let computed = computed_values
                            .get_box()
                            .clone_scroll_snap_coordinate();
                        AnimationValue::ScrollSnapCoordinate(
                            computed
                        )
                    }
                    AnimatableLonghand::Transform => {
                        let computed = computed_values
                            .get_box()
                            .clone_transform();
                        AnimationValue::Transform(
                            computed
                        )
                    }
                    AnimatableLonghand::ScrollBehavior => {
                        let computed = computed_values
                            .get_box()
                            .clone_scroll_behavior();
                        AnimationValue::ScrollBehavior(
                            computed
                        )
                    }
                    AnimatableLonghand::ScrollSnapTypeX => {
                        let computed = computed_values
                            .get_box()
                            .clone_scroll_snap_type_x();
                        AnimationValue::ScrollSnapTypeX(
                            computed
                        )
                    }
                    AnimatableLonghand::ScrollSnapTypeY => {
                        let computed = computed_values
                            .get_box()
                            .clone_scroll_snap_type_y();
                        AnimationValue::ScrollSnapTypeY(
                            computed
                        )
                    }
                    AnimatableLonghand::Isolation => {
                        let computed = computed_values
                            .get_box()
                            .clone_isolation();
                        AnimationValue::Isolation(
                            computed
                        )
                    }
                    AnimatableLonghand::PageBreakAfter => {
                        let computed = computed_values
                            .get_box()
                            .clone_page_break_after();
                        AnimationValue::PageBreakAfter(
                            computed
                        )
                    }
                    AnimatableLonghand::PageBreakBefore => {
                        let computed = computed_values
                            .get_box()
                            .clone_page_break_before();
                        AnimationValue::PageBreakBefore(
                            computed
                        )
                    }
                    AnimatableLonghand::PageBreakInside => {
                        let computed = computed_values
                            .get_box()
                            .clone_page_break_inside();
                        AnimationValue::PageBreakInside(
                            computed
                        )
                    }
                    AnimatableLonghand::Resize => {
                        let computed = computed_values
                            .get_box()
                            .clone_resize();
                        AnimationValue::Resize(
                            computed
                        )
                    }
                    AnimatableLonghand::Perspective => {
                        let computed = computed_values
                            .get_box()
                            .clone_perspective();
                        AnimationValue::Perspective(
                            computed
                        )
                    }
                    AnimatableLonghand::PerspectiveOrigin => {
                        let computed = computed_values
                            .get_box()
                            .clone_perspective_origin();
                        AnimationValue::PerspectiveOrigin(
                            computed
                        )
                    }
                    AnimatableLonghand::BackfaceVisibility => {
                        let computed = computed_values
                            .get_box()
                            .clone_backface_visibility();
                        AnimationValue::BackfaceVisibility(
                            computed
                        )
                    }
                    AnimatableLonghand::TransformBox => {
                        let computed = computed_values
                            .get_box()
                            .clone_transform_box();
                        AnimationValue::TransformBox(
                            computed
                        )
                    }
                    AnimatableLonghand::TransformStyle => {
                        let computed = computed_values
                            .get_box()
                            .clone_transform_style();
                        AnimationValue::TransformStyle(
                            computed
                        )
                    }
                    AnimatableLonghand::TransformOrigin => {
                        let computed = computed_values
                            .get_box()
                            .clone_transform_origin();
                        AnimationValue::TransformOrigin(
                            computed
                        )
                    }
                    AnimatableLonghand::Contain => {
                        let computed = computed_values
                            .get_box()
                            .clone_contain();
                        AnimationValue::Contain(
                            computed
                        )
                    }
                    AnimatableLonghand::MozAppearance => {
                        let computed = computed_values
                            .get_box()
                            .clone__moz_appearance();
                        AnimationValue::MozAppearance(
                            computed
                        )
                    }
                    AnimatableLonghand::MozOrient => {
                        let computed = computed_values
                            .get_box()
                            .clone__moz_orient();
                        AnimationValue::MozOrient(
                            computed
                        )
                    }
                    AnimatableLonghand::WillChange => {
                        let computed = computed_values
                            .get_box()
                            .clone_will_change();
                        AnimationValue::WillChange(
                            computed
                        )
                    }
                    AnimatableLonghand::ShapeOutside => {
                        let computed = computed_values
                            .get_box()
                            .clone_shape_outside();
                        AnimationValue::ShapeOutside(
                            computed
                        )
                    }
                    AnimatableLonghand::TouchAction => {
                        let computed = computed_values
                            .get_box()
                            .clone_touch_action();
                        AnimationValue::TouchAction(
                            computed
                        )
                    }
                    AnimatableLonghand::Color => {
                        let computed = computed_values
                            .get_color()
                            .clone_color();
                        AnimationValue::Color(
                            computed.to_animated_value()
                        )
                    }
                    AnimatableLonghand::ColumnWidth => {
                        let computed = computed_values
                            .get_column()
                            .clone_column_width();
                        AnimationValue::ColumnWidth(
                            computed.to_animated_value()
                        )
                    }
                    AnimatableLonghand::ColumnCount => {
                        let computed = computed_values
                            .get_column()
                            .clone_column_count();
                        AnimationValue::ColumnCount(
                            computed.to_animated_value()
                        )
                    }
                    AnimatableLonghand::ColumnGap => {
                        let computed = computed_values
                            .get_column()
                            .clone_column_gap();
                        AnimationValue::ColumnGap(
                            computed.to_animated_value()
                        )
                    }
                    AnimatableLonghand::ColumnFill => {
                        let computed = computed_values
                            .get_column()
                            .clone_column_fill();
                        AnimationValue::ColumnFill(
                            computed
                        )
                    }
                    AnimatableLonghand::ColumnRuleWidth => {
                        let computed = computed_values
                            .get_column()
                            .clone_column_rule_width();
                        AnimationValue::ColumnRuleWidth(
                            computed.to_animated_value()
                        )
                    }
                    AnimatableLonghand::ColumnRuleColor => {
                        let computed = computed_values
                            .get_column()
                            .clone_column_rule_color();
                        AnimationValue::ColumnRuleColor(
                            computed.to_animated_value()
                        )
                    }
                    AnimatableLonghand::ColumnSpan => {
                        let computed = computed_values
                            .get_column()
                            .clone_column_span();
                        AnimationValue::ColumnSpan(
                            computed
                        )
                    }
                    AnimatableLonghand::ColumnRuleStyle => {
                        let computed = computed_values
                            .get_column()
                            .clone_column_rule_style();
                        AnimationValue::ColumnRuleStyle(
                            computed
                        )
                    }
                    AnimatableLonghand::Content => {
                        let computed = computed_values
                            .get_counters()
                            .clone_content();
                        AnimationValue::Content(
                            computed
                        )
                    }
                    AnimatableLonghand::CounterIncrement => {
                        let computed = computed_values
                            .get_counters()
                            .clone_counter_increment();
                        AnimationValue::CounterIncrement(
                            computed
                        )
                    }
                    AnimatableLonghand::CounterReset => {
                        let computed = computed_values
                            .get_counters()
                            .clone_counter_reset();
                        AnimationValue::CounterReset(
                            computed
                        )
                    }
                    AnimatableLonghand::Opacity => {
                        let computed = computed_values
                            .get_effects()
                            .clone_opacity();
                        AnimationValue::Opacity(
                            computed
                        )
                    }
                    AnimatableLonghand::BoxShadow => {
                        let computed = computed_values
                            .get_effects()
                            .clone_box_shadow();
                        AnimationValue::BoxShadow(
                            computed.to_animated_value()
                        )
                    }
                    AnimatableLonghand::Clip => {
                        let computed = computed_values
                            .get_effects()
                            .clone_clip();
                        AnimationValue::Clip(
                            computed
                        )
                    }
                    AnimatableLonghand::Filter => {
                        let computed = computed_values
                            .get_effects()
                            .clone_filter();
                        AnimationValue::Filter(
                            computed.to_animated_value()
                        )
                    }
                    AnimatableLonghand::MixBlendMode => {
                        let computed = computed_values
                            .get_effects()
                            .clone_mix_blend_mode();
                        AnimationValue::MixBlendMode(
                            computed
                        )
                    }
                    AnimatableLonghand::FontFamily => {
                        let computed = computed_values
                            .get_font()
                            .clone_font_family();
                        AnimationValue::FontFamily(
                            computed
                        )
                    }
                    AnimatableLonghand::FontStyle => {
                        let computed = computed_values
                            .get_font()
                            .clone_font_style();
                        AnimationValue::FontStyle(
                            computed
                        )
                    }
                    AnimatableLonghand::FontVariantCaps => {
                        let computed = computed_values
                            .get_font()
                            .clone_font_variant_caps();
                        AnimationValue::FontVariantCaps(
                            computed
                        )
                    }
                    AnimatableLonghand::FontWeight => {
                        let computed = computed_values
                            .get_font()
                            .clone_font_weight();
                        AnimationValue::FontWeight(
                            computed
                        )
                    }
                    AnimatableLonghand::FontSize => {
                        let computed = computed_values
                            .get_font()
                            .clone_font_size();
                        AnimationValue::FontSize(
                            computed.to_animated_value()
                        )
                    }
                    AnimatableLonghand::FontSizeAdjust => {
                        let computed = computed_values
                            .get_font()
                            .clone_font_size_adjust();
                        AnimationValue::FontSizeAdjust(
                            computed.to_animated_value()
                        )
                    }
                    AnimatableLonghand::FontSynthesis => {
                        let computed = computed_values
                            .get_font()
                            .clone_font_synthesis();
                        AnimationValue::FontSynthesis(
                            computed
                        )
                    }
                    AnimatableLonghand::FontStretch => {
                        let computed = computed_values
                            .get_font()
                            .clone_font_stretch();
                        AnimationValue::FontStretch(
                            computed
                        )
                    }
                    AnimatableLonghand::FontKerning => {
                        let computed = computed_values
                            .get_font()
                            .clone_font_kerning();
                        AnimationValue::FontKerning(
                            computed
                        )
                    }
                    AnimatableLonghand::FontVariantAlternates => {
                        let computed = computed_values
                            .get_font()
                            .clone_font_variant_alternates();
                        AnimationValue::FontVariantAlternates(
                            computed
                        )
                    }
                    AnimatableLonghand::FontVariantEastAsian => {
                        let computed = computed_values
                            .get_font()
                            .clone_font_variant_east_asian();
                        AnimationValue::FontVariantEastAsian(
                            computed
                        )
                    }
                    AnimatableLonghand::FontVariantLigatures => {
                        let computed = computed_values
                            .get_font()
                            .clone_font_variant_ligatures();
                        AnimationValue::FontVariantLigatures(
                            computed
                        )
                    }
                    AnimatableLonghand::FontVariantNumeric => {
                        let computed = computed_values
                            .get_font()
                            .clone_font_variant_numeric();
                        AnimationValue::FontVariantNumeric(
                            computed
                        )
                    }
                    AnimatableLonghand::FontVariantPosition => {
                        let computed = computed_values
                            .get_font()
                            .clone_font_variant_position();
                        AnimationValue::FontVariantPosition(
                            computed
                        )
                    }
                    AnimatableLonghand::FontFeatureSettings => {
                        let computed = computed_values
                            .get_font()
                            .clone_font_feature_settings();
                        AnimationValue::FontFeatureSettings(
                            computed
                        )
                    }
                    AnimatableLonghand::FontVariationSettings => {
                        let computed = computed_values
                            .get_font()
                            .clone_font_variation_settings();
                        AnimationValue::FontVariationSettings(
                            computed
                        )
                    }
                    AnimatableLonghand::FontLanguageOverride => {
                        let computed = computed_values
                            .get_font()
                            .clone_font_language_override();
                        AnimationValue::FontLanguageOverride(
                            computed
                        )
                    }
                    AnimatableLonghand::MozOsxFontSmoothing => {
                        let computed = computed_values
                            .get_font()
                            .clone__moz_osx_font_smoothing();
                        AnimationValue::MozOsxFontSmoothing(
                            computed
                        )
                    }
                    AnimatableLonghand::Visibility => {
                        let computed = computed_values
                            .get_inheritedbox()
                            .clone_visibility();
                        AnimationValue::Visibility(
                            computed
                        )
                    }
                    AnimatableLonghand::WritingMode => {
                        let computed = computed_values
                            .get_inheritedbox()
                            .clone_writing_mode();
                        AnimationValue::WritingMode(
                            computed
                        )
                    }
                    AnimatableLonghand::Direction => {
                        let computed = computed_values
                            .get_inheritedbox()
                            .clone_direction();
                        AnimationValue::Direction(
                            computed
                        )
                    }
                    AnimatableLonghand::TextOrientation => {
                        let computed = computed_values
                            .get_inheritedbox()
                            .clone_text_orientation();
                        AnimationValue::TextOrientation(
                            computed
                        )
                    }
                    AnimatableLonghand::ColorAdjust => {
                        let computed = computed_values
                            .get_inheritedbox()
                            .clone_color_adjust();
                        AnimationValue::ColorAdjust(
                            computed
                        )
                    }
                    AnimatableLonghand::ImageRendering => {
                        let computed = computed_values
                            .get_inheritedbox()
                            .clone_image_rendering();
                        AnimationValue::ImageRendering(
                            computed
                        )
                    }
                    AnimatableLonghand::ImageOrientation => {
                        let computed = computed_values
                            .get_inheritedbox()
                            .clone_image_orientation();
                        AnimationValue::ImageOrientation(
                            computed
                        )
                    }
                    AnimatableLonghand::BorderCollapse => {
                        let computed = computed_values
                            .get_inheritedtable()
                            .clone_border_collapse();
                        AnimationValue::BorderCollapse(
                            computed
                        )
                    }
                    AnimatableLonghand::EmptyCells => {
                        let computed = computed_values
                            .get_inheritedtable()
                            .clone_empty_cells();
                        AnimationValue::EmptyCells(
                            computed
                        )
                    }
                    AnimatableLonghand::CaptionSide => {
                        let computed = computed_values
                            .get_inheritedtable()
                            .clone_caption_side();
                        AnimationValue::CaptionSide(
                            computed
                        )
                    }
                    AnimatableLonghand::BorderSpacing => {
                        let computed = computed_values
                            .get_inheritedtable()
                            .clone_border_spacing();
                        AnimationValue::BorderSpacing(
                            computed.to_animated_value()
                        )
                    }
                    AnimatableLonghand::LineHeight => {
                        let computed = computed_values
                            .get_inheritedtext()
                            .clone_line_height();
                        AnimationValue::LineHeight(
                            computed.to_animated_value()
                        )
                    }
                    AnimatableLonghand::TextTransform => {
                        let computed = computed_values
                            .get_inheritedtext()
                            .clone_text_transform();
                        AnimationValue::TextTransform(
                            computed
                        )
                    }
                    AnimatableLonghand::Hyphens => {
                        let computed = computed_values
                            .get_inheritedtext()
                            .clone_hyphens();
                        AnimationValue::Hyphens(
                            computed
                        )
                    }
                    AnimatableLonghand::MozTextSizeAdjust => {
                        let computed = computed_values
                            .get_inheritedtext()
                            .clone__moz_text_size_adjust();
                        AnimationValue::MozTextSizeAdjust(
                            computed
                        )
                    }
                    AnimatableLonghand::TextIndent => {
                        let computed = computed_values
                            .get_inheritedtext()
                            .clone_text_indent();
                        AnimationValue::TextIndent(
                            computed
                        )
                    }
                    AnimatableLonghand::OverflowWrap => {
                        let computed = computed_values
                            .get_inheritedtext()
                            .clone_overflow_wrap();
                        AnimationValue::OverflowWrap(
                            computed
                        )
                    }
                    AnimatableLonghand::WordBreak => {
                        let computed = computed_values
                            .get_inheritedtext()
                            .clone_word_break();
                        AnimationValue::WordBreak(
                            computed
                        )
                    }
                    AnimatableLonghand::TextJustify => {
                        let computed = computed_values
                            .get_inheritedtext()
                            .clone_text_justify();
                        AnimationValue::TextJustify(
                            computed
                        )
                    }
                    AnimatableLonghand::TextAlignLast => {
                        let computed = computed_values
                            .get_inheritedtext()
                            .clone_text_align_last();
                        AnimationValue::TextAlignLast(
                            computed
                        )
                    }
                    AnimatableLonghand::TextAlign => {
                        let computed = computed_values
                            .get_inheritedtext()
                            .clone_text_align();
                        AnimationValue::TextAlign(
                            computed
                        )
                    }
                    AnimatableLonghand::LetterSpacing => {
                        let computed = computed_values
                            .get_inheritedtext()
                            .clone_letter_spacing();
                        AnimationValue::LetterSpacing(
                            computed
                        )
                    }
                    AnimatableLonghand::WordSpacing => {
                        let computed = computed_values
                            .get_inheritedtext()
                            .clone_word_spacing();
                        AnimationValue::WordSpacing(
                            computed
                        )
                    }
                    AnimatableLonghand::WhiteSpace => {
                        let computed = computed_values
                            .get_inheritedtext()
                            .clone_white_space();
                        AnimationValue::WhiteSpace(
                            computed
                        )
                    }
                    AnimatableLonghand::TextShadow => {
                        let computed = computed_values
                            .get_inheritedtext()
                            .clone_text_shadow();
                        AnimationValue::TextShadow(
                            computed.to_animated_value()
                        )
                    }
                    AnimatableLonghand::TextEmphasisStyle => {
                        let computed = computed_values
                            .get_inheritedtext()
                            .clone_text_emphasis_style();
                        AnimationValue::TextEmphasisStyle(
                            computed
                        )
                    }
                    AnimatableLonghand::TextEmphasisPosition => {
                        let computed = computed_values
                            .get_inheritedtext()
                            .clone_text_emphasis_position();
                        AnimationValue::TextEmphasisPosition(
                            computed
                        )
                    }
                    AnimatableLonghand::TextEmphasisColor => {
                        let computed = computed_values
                            .get_inheritedtext()
                            .clone_text_emphasis_color();
                        AnimationValue::TextEmphasisColor(
                            computed.to_animated_value()
                        )
                    }
                    AnimatableLonghand::MozTabSize => {
                        let computed = computed_values
                            .get_inheritedtext()
                            .clone__moz_tab_size();
                        AnimationValue::MozTabSize(
                            computed.to_animated_value()
                        )
                    }
                    AnimatableLonghand::WebkitTextFillColor => {
                        let computed = computed_values
                            .get_inheritedtext()
                            .clone__webkit_text_fill_color();
                        AnimationValue::WebkitTextFillColor(
                            computed.to_animated_value()
                        )
                    }
                    AnimatableLonghand::WebkitTextStrokeColor => {
                        let computed = computed_values
                            .get_inheritedtext()
                            .clone__webkit_text_stroke_color();
                        AnimationValue::WebkitTextStrokeColor(
                            computed.to_animated_value()
                        )
                    }
                    AnimatableLonghand::WebkitTextStrokeWidth => {
                        let computed = computed_values
                            .get_inheritedtext()
                            .clone__webkit_text_stroke_width();
                        AnimationValue::WebkitTextStrokeWidth(
                            computed
                        )
                    }
                    AnimatableLonghand::RubyAlign => {
                        let computed = computed_values
                            .get_inheritedtext()
                            .clone_ruby_align();
                        AnimationValue::RubyAlign(
                            computed
                        )
                    }
                    AnimatableLonghand::RubyPosition => {
                        let computed = computed_values
                            .get_inheritedtext()
                            .clone_ruby_position();
                        AnimationValue::RubyPosition(
                            computed
                        )
                    }
                    AnimatableLonghand::TextCombineUpright => {
                        let computed = computed_values
                            .get_inheritedtext()
                            .clone_text_combine_upright();
                        AnimationValue::TextCombineUpright(
                            computed
                        )
                    }
                    AnimatableLonghand::TextRendering => {
                        let computed = computed_values
                            .get_inheritedtext()
                            .clone_text_rendering();
                        AnimationValue::TextRendering(
                            computed
                        )
                    }
                    AnimatableLonghand::ListStylePosition => {
                        let computed = computed_values
                            .get_list()
                            .clone_list_style_position();
                        AnimationValue::ListStylePosition(
                            computed
                        )
                    }
                    AnimatableLonghand::ListStyleType => {
                        let computed = computed_values
                            .get_list()
                            .clone_list_style_type();
                        AnimationValue::ListStyleType(
                            computed
                        )
                    }
                    AnimatableLonghand::ListStyleImage => {
                        let computed = computed_values
                            .get_list()
                            .clone_list_style_image();
                        AnimationValue::ListStyleImage(
                            computed
                        )
                    }
                    AnimatableLonghand::Quotes => {
                        let computed = computed_values
                            .get_list()
                            .clone_quotes();
                        AnimationValue::Quotes(
                            computed
                        )
                    }
                    AnimatableLonghand::MozImageRegion => {
                        let computed = computed_values
                            .get_list()
                            .clone__moz_image_region();
                        AnimationValue::MozImageRegion(
                            computed
                        )
                    }
                    AnimatableLonghand::MarginTop => {
                        let computed = computed_values
                            .get_margin()
                            .clone_margin_top();
                        AnimationValue::MarginTop(
                            computed
                        )
                    }
                    AnimatableLonghand::MarginLeft => {
                        let computed = computed_values
                            .get_margin()
                            .clone_margin_left();
                        AnimationValue::MarginLeft(
                            computed
                        )
                    }
                    AnimatableLonghand::MarginBottom => {
                        let computed = computed_values
                            .get_margin()
                            .clone_margin_bottom();
                        AnimationValue::MarginBottom(
                            computed
                        )
                    }
                    AnimatableLonghand::MarginRight => {
                        let computed = computed_values
                            .get_margin()
                            .clone_margin_right();
                        AnimationValue::MarginRight(
                            computed
                        )
                    }
                    AnimatableLonghand::OutlineColor => {
                        let computed = computed_values
                            .get_outline()
                            .clone_outline_color();
                        AnimationValue::OutlineColor(
                            computed.to_animated_value()
                        )
                    }
                    AnimatableLonghand::OutlineStyle => {
                        let computed = computed_values
                            .get_outline()
                            .clone_outline_style();
                        AnimationValue::OutlineStyle(
                            computed
                        )
                    }
                    AnimatableLonghand::OutlineWidth => {
                        let computed = computed_values
                            .get_outline()
                            .clone_outline_width();
                        AnimationValue::OutlineWidth(
                            computed.to_animated_value()
                        )
                    }
                    AnimatableLonghand::MozOutlineRadiusTopleft => {
                        let computed = computed_values
                            .get_outline()
                            .clone__moz_outline_radius_topleft();
                        AnimationValue::MozOutlineRadiusTopleft(
                            computed.to_animated_value()
                        )
                    }
                    AnimatableLonghand::MozOutlineRadiusTopright => {
                        let computed = computed_values
                            .get_outline()
                            .clone__moz_outline_radius_topright();
                        AnimationValue::MozOutlineRadiusTopright(
                            computed.to_animated_value()
                        )
                    }
                    AnimatableLonghand::MozOutlineRadiusBottomright => {
                        let computed = computed_values
                            .get_outline()
                            .clone__moz_outline_radius_bottomright();
                        AnimationValue::MozOutlineRadiusBottomright(
                            computed.to_animated_value()
                        )
                    }
                    AnimatableLonghand::MozOutlineRadiusBottomleft => {
                        let computed = computed_values
                            .get_outline()
                            .clone__moz_outline_radius_bottomleft();
                        AnimationValue::MozOutlineRadiusBottomleft(
                            computed.to_animated_value()
                        )
                    }
                    AnimatableLonghand::OutlineOffset => {
                        let computed = computed_values
                            .get_outline()
                            .clone_outline_offset();
                        AnimationValue::OutlineOffset(
                            computed
                        )
                    }
                    AnimatableLonghand::PaddingTop => {
                        let computed = computed_values
                            .get_padding()
                            .clone_padding_top();
                        AnimationValue::PaddingTop(
                            computed.to_animated_value()
                        )
                    }
                    AnimatableLonghand::PaddingLeft => {
                        let computed = computed_values
                            .get_padding()
                            .clone_padding_left();
                        AnimationValue::PaddingLeft(
                            computed.to_animated_value()
                        )
                    }
                    AnimatableLonghand::PaddingBottom => {
                        let computed = computed_values
                            .get_padding()
                            .clone_padding_bottom();
                        AnimationValue::PaddingBottom(
                            computed.to_animated_value()
                        )
                    }
                    AnimatableLonghand::PaddingRight => {
                        let computed = computed_values
                            .get_padding()
                            .clone_padding_right();
                        AnimationValue::PaddingRight(
                            computed.to_animated_value()
                        )
                    }
                    AnimatableLonghand::Cursor => {
                        let computed = computed_values
                            .get_pointing()
                            .clone_cursor();
                        AnimationValue::Cursor(
                            computed
                        )
                    }
                    AnimatableLonghand::PointerEvents => {
                        let computed = computed_values
                            .get_pointing()
                            .clone_pointer_events();
                        AnimationValue::PointerEvents(
                            computed
                        )
                    }
                    AnimatableLonghand::MozUserInput => {
                        let computed = computed_values
                            .get_pointing()
                            .clone__moz_user_input();
                        AnimationValue::MozUserInput(
                            computed
                        )
                    }
                    AnimatableLonghand::MozUserModify => {
                        let computed = computed_values
                            .get_pointing()
                            .clone__moz_user_modify();
                        AnimationValue::MozUserModify(
                            computed
                        )
                    }
                    AnimatableLonghand::MozUserFocus => {
                        let computed = computed_values
                            .get_pointing()
                            .clone__moz_user_focus();
                        AnimationValue::MozUserFocus(
                            computed
                        )
                    }
                    AnimatableLonghand::CaretColor => {
                        let computed = computed_values
                            .get_pointing()
                            .clone_caret_color();
                        AnimationValue::CaretColor(
                            computed.to_animated_value()
                        )
                    }
                    AnimatableLonghand::Top => {
                        let computed = computed_values
                            .get_position()
                            .clone_top();
                        AnimationValue::Top(
                            computed
                        )
                    }
                    AnimatableLonghand::Left => {
                        let computed = computed_values
                            .get_position()
                            .clone_left();
                        AnimationValue::Left(
                            computed
                        )
                    }
                    AnimatableLonghand::Bottom => {
                        let computed = computed_values
                            .get_position()
                            .clone_bottom();
                        AnimationValue::Bottom(
                            computed
                        )
                    }
                    AnimatableLonghand::Right => {
                        let computed = computed_values
                            .get_position()
                            .clone_right();
                        AnimationValue::Right(
                            computed
                        )
                    }
                    AnimatableLonghand::ZIndex => {
                        let computed = computed_values
                            .get_position()
                            .clone_z_index();
                        AnimationValue::ZIndex(
                            computed
                        )
                    }
                    AnimatableLonghand::FlexDirection => {
                        let computed = computed_values
                            .get_position()
                            .clone_flex_direction();
                        AnimationValue::FlexDirection(
                            computed
                        )
                    }
                    AnimatableLonghand::FlexWrap => {
                        let computed = computed_values
                            .get_position()
                            .clone_flex_wrap();
                        AnimationValue::FlexWrap(
                            computed
                        )
                    }
                    AnimatableLonghand::JustifyContent => {
                        let computed = computed_values
                            .get_position()
                            .clone_justify_content();
                        AnimationValue::JustifyContent(
                            computed
                        )
                    }
                    AnimatableLonghand::AlignContent => {
                        let computed = computed_values
                            .get_position()
                            .clone_align_content();
                        AnimationValue::AlignContent(
                            computed
                        )
                    }
                    AnimatableLonghand::AlignItems => {
                        let computed = computed_values
                            .get_position()
                            .clone_align_items();
                        AnimationValue::AlignItems(
                            computed
                        )
                    }
                    AnimatableLonghand::JustifyItems => {
                        let computed = computed_values
                            .get_position()
                            .clone_justify_items();
                        AnimationValue::JustifyItems(
                            computed
                        )
                    }
                    AnimatableLonghand::FlexGrow => {
                        let computed = computed_values
                            .get_position()
                            .clone_flex_grow();
                        AnimationValue::FlexGrow(
                            computed.to_animated_value()
                        )
                    }
                    AnimatableLonghand::FlexShrink => {
                        let computed = computed_values
                            .get_position()
                            .clone_flex_shrink();
                        AnimationValue::FlexShrink(
                            computed.to_animated_value()
                        )
                    }
                    AnimatableLonghand::AlignSelf => {
                        let computed = computed_values
                            .get_position()
                            .clone_align_self();
                        AnimationValue::AlignSelf(
                            computed
                        )
                    }
                    AnimatableLonghand::JustifySelf => {
                        let computed = computed_values
                            .get_position()
                            .clone_justify_self();
                        AnimationValue::JustifySelf(
                            computed
                        )
                    }
                    AnimatableLonghand::Order => {
                        let computed = computed_values
                            .get_position()
                            .clone_order();
                        AnimationValue::Order(
                            computed
                        )
                    }
                    AnimatableLonghand::FlexBasis => {
                        let computed = computed_values
                            .get_position()
                            .clone_flex_basis();
                        AnimationValue::FlexBasis(
                            computed.to_animated_value()
                        )
                    }
                    AnimatableLonghand::Width => {
                        let computed = computed_values
                            .get_position()
                            .clone_width();
                        AnimationValue::Width(
                            computed.to_animated_value()
                        )
                    }
                    AnimatableLonghand::MinWidth => {
                        let computed = computed_values
                            .get_position()
                            .clone_min_width();
                        AnimationValue::MinWidth(
                            computed.to_animated_value()
                        )
                    }
                    AnimatableLonghand::MaxWidth => {
                        let computed = computed_values
                            .get_position()
                            .clone_max_width();
                        AnimationValue::MaxWidth(
                            computed.to_animated_value()
                        )
                    }
                    AnimatableLonghand::Height => {
                        let computed = computed_values
                            .get_position()
                            .clone_height();
                        AnimationValue::Height(
                            computed.to_animated_value()
                        )
                    }
                    AnimatableLonghand::MinHeight => {
                        let computed = computed_values
                            .get_position()
                            .clone_min_height();
                        AnimationValue::MinHeight(
                            computed.to_animated_value()
                        )
                    }
                    AnimatableLonghand::MaxHeight => {
                        let computed = computed_values
                            .get_position()
                            .clone_max_height();
                        AnimationValue::MaxHeight(
                            computed.to_animated_value()
                        )
                    }
                    AnimatableLonghand::BoxSizing => {
                        let computed = computed_values
                            .get_position()
                            .clone_box_sizing();
                        AnimationValue::BoxSizing(
                            computed
                        )
                    }
                    AnimatableLonghand::ObjectFit => {
                        let computed = computed_values
                            .get_position()
                            .clone_object_fit();
                        AnimationValue::ObjectFit(
                            computed
                        )
                    }
                    AnimatableLonghand::ObjectPosition => {
                        let computed = computed_values
                            .get_position()
                            .clone_object_position();
                        AnimationValue::ObjectPosition(
                            computed
                        )
                    }
                    AnimatableLonghand::GridRowGap => {
                        let computed = computed_values
                            .get_position()
                            .clone_grid_row_gap();
                        AnimationValue::GridRowGap(
                            computed.to_animated_value()
                        )
                    }
                    AnimatableLonghand::GridRowStart => {
                        let computed = computed_values
                            .get_position()
                            .clone_grid_row_start();
                        AnimationValue::GridRowStart(
                            computed
                        )
                    }
                    AnimatableLonghand::GridRowEnd => {
                        let computed = computed_values
                            .get_position()
                            .clone_grid_row_end();
                        AnimationValue::GridRowEnd(
                            computed
                        )
                    }
                    AnimatableLonghand::GridAutoRows => {
                        let computed = computed_values
                            .get_position()
                            .clone_grid_auto_rows();
                        AnimationValue::GridAutoRows(
                            computed
                        )
                    }
                    AnimatableLonghand::GridTemplateRows => {
                        let computed = computed_values
                            .get_position()
                            .clone_grid_template_rows();
                        AnimationValue::GridTemplateRows(
                            computed
                        )
                    }
                    AnimatableLonghand::GridColumnGap => {
                        let computed = computed_values
                            .get_position()
                            .clone_grid_column_gap();
                        AnimationValue::GridColumnGap(
                            computed.to_animated_value()
                        )
                    }
                    AnimatableLonghand::GridColumnStart => {
                        let computed = computed_values
                            .get_position()
                            .clone_grid_column_start();
                        AnimationValue::GridColumnStart(
                            computed
                        )
                    }
                    AnimatableLonghand::GridColumnEnd => {
                        let computed = computed_values
                            .get_position()
                            .clone_grid_column_end();
                        AnimationValue::GridColumnEnd(
                            computed
                        )
                    }
                    AnimatableLonghand::GridAutoColumns => {
                        let computed = computed_values
                            .get_position()
                            .clone_grid_auto_columns();
                        AnimationValue::GridAutoColumns(
                            computed
                        )
                    }
                    AnimatableLonghand::GridTemplateColumns => {
                        let computed = computed_values
                            .get_position()
                            .clone_grid_template_columns();
                        AnimationValue::GridTemplateColumns(
                            computed
                        )
                    }
                    AnimatableLonghand::GridAutoFlow => {
                        let computed = computed_values
                            .get_position()
                            .clone_grid_auto_flow();
                        AnimationValue::GridAutoFlow(
                            computed
                        )
                    }
                    AnimatableLonghand::GridTemplateAreas => {
                        let computed = computed_values
                            .get_position()
                            .clone_grid_template_areas();
                        AnimationValue::GridTemplateAreas(
                            computed
                        )
                    }
                    AnimatableLonghand::TableLayout => {
                        let computed = computed_values
                            .get_table()
                            .clone_table_layout();
                        AnimationValue::TableLayout(
                            computed
                        )
                    }
                    AnimatableLonghand::TextOverflow => {
                        let computed = computed_values
                            .get_text()
                            .clone_text_overflow();
                        AnimationValue::TextOverflow(
                            computed
                        )
                    }
                    AnimatableLonghand::UnicodeBidi => {
                        let computed = computed_values
                            .get_text()
                            .clone_unicode_bidi();
                        AnimationValue::UnicodeBidi(
                            computed
                        )
                    }
                    AnimatableLonghand::TextDecorationLine => {
                        let computed = computed_values
                            .get_text()
                            .clone_text_decoration_line();
                        AnimationValue::TextDecorationLine(
                            computed
                        )
                    }
                    AnimatableLonghand::TextDecorationStyle => {
                        let computed = computed_values
                            .get_text()
                            .clone_text_decoration_style();
                        AnimationValue::TextDecorationStyle(
                            computed
                        )
                    }
                    AnimatableLonghand::TextDecorationColor => {
                        let computed = computed_values
                            .get_text()
                            .clone_text_decoration_color();
                        AnimationValue::TextDecorationColor(
                            computed.to_animated_value()
                        )
                    }
                    AnimatableLonghand::InitialLetter => {
                        let computed = computed_values
                            .get_text()
                            .clone_initial_letter();
                        AnimationValue::InitialLetter(
                            computed
                        )
                    }
                    AnimatableLonghand::ImeMode => {
                        let computed = computed_values
                            .get_ui()
                            .clone_ime_mode();
                        AnimationValue::ImeMode(
                            computed
                        )
                    }
                    AnimatableLonghand::MozUserSelect => {
                        let computed = computed_values
                            .get_ui()
                            .clone__moz_user_select();
                        AnimationValue::MozUserSelect(
                            computed
                        )
                    }
                    AnimatableLonghand::MozWindowDragging => {
                        let computed = computed_values
                            .get_ui()
                            .clone__moz_window_dragging();
                        AnimationValue::MozWindowDragging(
                            computed
                        )
                    }
                    AnimatableLonghand::MozWindowShadow => {
                        let computed = computed_values
                            .get_ui()
                            .clone__moz_window_shadow();
                        AnimationValue::MozWindowShadow(
                            computed
                        )
                    }
                    AnimatableLonghand::MozForceBrokenImageIcon => {
                        let computed = computed_values
                            .get_ui()
                            .clone__moz_force_broken_image_icon();
                        AnimationValue::MozForceBrokenImageIcon(
                            computed
                        )
                    }
                    AnimatableLonghand::TextAnchor => {
                        let computed = computed_values
                            .get_inheritedsvg()
                            .clone_text_anchor();
                        AnimationValue::TextAnchor(
                            computed
                        )
                    }
                    AnimatableLonghand::ColorInterpolation => {
                        let computed = computed_values
                            .get_inheritedsvg()
                            .clone_color_interpolation();
                        AnimationValue::ColorInterpolation(
                            computed
                        )
                    }
                    AnimatableLonghand::ColorInterpolationFilters => {
                        let computed = computed_values
                            .get_inheritedsvg()
                            .clone_color_interpolation_filters();
                        AnimationValue::ColorInterpolationFilters(
                            computed
                        )
                    }
                    AnimatableLonghand::Fill => {
                        let computed = computed_values
                            .get_inheritedsvg()
                            .clone_fill();
                        AnimationValue::Fill(
                            computed.to_animated_value()
                        )
                    }
                    AnimatableLonghand::FillOpacity => {
                        let computed = computed_values
                            .get_inheritedsvg()
                            .clone_fill_opacity();
                        AnimationValue::FillOpacity(
                            computed
                        )
                    }
                    AnimatableLonghand::FillRule => {
                        let computed = computed_values
                            .get_inheritedsvg()
                            .clone_fill_rule();
                        AnimationValue::FillRule(
                            computed
                        )
                    }
                    AnimatableLonghand::ShapeRendering => {
                        let computed = computed_values
                            .get_inheritedsvg()
                            .clone_shape_rendering();
                        AnimationValue::ShapeRendering(
                            computed
                        )
                    }
                    AnimatableLonghand::Stroke => {
                        let computed = computed_values
                            .get_inheritedsvg()
                            .clone_stroke();
                        AnimationValue::Stroke(
                            computed.to_animated_value()
                        )
                    }
                    AnimatableLonghand::StrokeWidth => {
                        let computed = computed_values
                            .get_inheritedsvg()
                            .clone_stroke_width();
                        AnimationValue::StrokeWidth(
                            computed.to_animated_value()
                        )
                    }
                    AnimatableLonghand::StrokeLinecap => {
                        let computed = computed_values
                            .get_inheritedsvg()
                            .clone_stroke_linecap();
                        AnimationValue::StrokeLinecap(
                            computed
                        )
                    }
                    AnimatableLonghand::StrokeLinejoin => {
                        let computed = computed_values
                            .get_inheritedsvg()
                            .clone_stroke_linejoin();
                        AnimationValue::StrokeLinejoin(
                            computed
                        )
                    }
                    AnimatableLonghand::StrokeMiterlimit => {
                        let computed = computed_values
                            .get_inheritedsvg()
                            .clone_stroke_miterlimit();
                        AnimationValue::StrokeMiterlimit(
                            computed.to_animated_value()
                        )
                    }
                    AnimatableLonghand::StrokeOpacity => {
                        let computed = computed_values
                            .get_inheritedsvg()
                            .clone_stroke_opacity();
                        AnimationValue::StrokeOpacity(
                            computed
                        )
                    }
                    AnimatableLonghand::StrokeDasharray => {
                        let computed = computed_values
                            .get_inheritedsvg()
                            .clone_stroke_dasharray();
                        AnimationValue::StrokeDasharray(
                            computed.to_animated_value()
                        )
                    }
                    AnimatableLonghand::StrokeDashoffset => {
                        let computed = computed_values
                            .get_inheritedsvg()
                            .clone_stroke_dashoffset();
                        AnimationValue::StrokeDashoffset(
                            computed
                        )
                    }
                    AnimatableLonghand::ClipRule => {
                        let computed = computed_values
                            .get_inheritedsvg()
                            .clone_clip_rule();
                        AnimationValue::ClipRule(
                            computed
                        )
                    }
                    AnimatableLonghand::MarkerStart => {
                        let computed = computed_values
                            .get_inheritedsvg()
                            .clone_marker_start();
                        AnimationValue::MarkerStart(
                            computed
                        )
                    }
                    AnimatableLonghand::MarkerMid => {
                        let computed = computed_values
                            .get_inheritedsvg()
                            .clone_marker_mid();
                        AnimationValue::MarkerMid(
                            computed
                        )
                    }
                    AnimatableLonghand::MarkerEnd => {
                        let computed = computed_values
                            .get_inheritedsvg()
                            .clone_marker_end();
                        AnimationValue::MarkerEnd(
                            computed
                        )
                    }
                    AnimatableLonghand::PaintOrder => {
                        let computed = computed_values
                            .get_inheritedsvg()
                            .clone_paint_order();
                        AnimationValue::PaintOrder(
                            computed
                        )
                    }
                    AnimatableLonghand::DominantBaseline => {
                        let computed = computed_values
                            .get_svg()
                            .clone_dominant_baseline();
                        AnimationValue::DominantBaseline(
                            computed
                        )
                    }
                    AnimatableLonghand::VectorEffect => {
                        let computed = computed_values
                            .get_svg()
                            .clone_vector_effect();
                        AnimationValue::VectorEffect(
                            computed
                        )
                    }
                    AnimatableLonghand::StopColor => {
                        let computed = computed_values
                            .get_svg()
                            .clone_stop_color();
                        AnimationValue::StopColor(
                            computed.to_animated_value()
                        )
                    }
                    AnimatableLonghand::StopOpacity => {
                        let computed = computed_values
                            .get_svg()
                            .clone_stop_opacity();
                        AnimationValue::StopOpacity(
                            computed
                        )
                    }
                    AnimatableLonghand::FloodColor => {
                        let computed = computed_values
                            .get_svg()
                            .clone_flood_color();
                        AnimationValue::FloodColor(
                            computed.to_animated_value()
                        )
                    }
                    AnimatableLonghand::FloodOpacity => {
                        let computed = computed_values
                            .get_svg()
                            .clone_flood_opacity();
                        AnimationValue::FloodOpacity(
                            computed
                        )
                    }
                    AnimatableLonghand::LightingColor => {
                        let computed = computed_values
                            .get_svg()
                            .clone_lighting_color();
                        AnimationValue::LightingColor(
                            computed.to_animated_value()
                        )
                    }
                    AnimatableLonghand::MaskType => {
                        let computed = computed_values
                            .get_svg()
                            .clone_mask_type();
                        AnimationValue::MaskType(
                            computed
                        )
                    }
                    AnimatableLonghand::ClipPath => {
                        let computed = computed_values
                            .get_svg()
                            .clone_clip_path();
                        AnimationValue::ClipPath(
                            computed
                        )
                    }
                    AnimatableLonghand::MaskMode => {
                        let computed = computed_values
                            .get_svg()
                            .clone_mask_mode();
                        AnimationValue::MaskMode(
                            computed
                        )
                    }
                    AnimatableLonghand::MaskRepeat => {
                        let computed = computed_values
                            .get_svg()
                            .clone_mask_repeat();
                        AnimationValue::MaskRepeat(
                            computed
                        )
                    }
                    AnimatableLonghand::MaskPositionX => {
                        let computed = computed_values
                            .get_svg()
                            .clone_mask_position_x();
                        AnimationValue::MaskPositionX(
                            computed
                        )
                    }
                    AnimatableLonghand::MaskPositionY => {
                        let computed = computed_values
                            .get_svg()
                            .clone_mask_position_y();
                        AnimationValue::MaskPositionY(
                            computed
                        )
                    }
                    AnimatableLonghand::MaskClip => {
                        let computed = computed_values
                            .get_svg()
                            .clone_mask_clip();
                        AnimationValue::MaskClip(
                            computed
                        )
                    }
                    AnimatableLonghand::MaskOrigin => {
                        let computed = computed_values
                            .get_svg()
                            .clone_mask_origin();
                        AnimationValue::MaskOrigin(
                            computed
                        )
                    }
                    AnimatableLonghand::MaskSize => {
                        let computed = computed_values
                            .get_svg()
                            .clone_mask_size();
                        AnimationValue::MaskSize(
                            computed
                        )
                    }
                    AnimatableLonghand::MaskComposite => {
                        let computed = computed_values
                            .get_svg()
                            .clone_mask_composite();
                        AnimationValue::MaskComposite(
                            computed
                        )
                    }
                    AnimatableLonghand::MaskImage => {
                        let computed = computed_values
                            .get_svg()
                            .clone_mask_image();
                        AnimationValue::MaskImage(
                            computed
                        )
                    }
                    AnimatableLonghand::MozBoxAlign => {
                        let computed = computed_values
                            .get_xul()
                            .clone__moz_box_align();
                        AnimationValue::MozBoxAlign(
                            computed
                        )
                    }
                    AnimatableLonghand::MozBoxDirection => {
                        let computed = computed_values
                            .get_xul()
                            .clone__moz_box_direction();
                        AnimationValue::MozBoxDirection(
                            computed
                        )
                    }
                    AnimatableLonghand::MozBoxFlex => {
                        let computed = computed_values
                            .get_xul()
                            .clone__moz_box_flex();
                        AnimationValue::MozBoxFlex(
                            computed.to_animated_value()
                        )
                    }
                    AnimatableLonghand::MozBoxOrient => {
                        let computed = computed_values
                            .get_xul()
                            .clone__moz_box_orient();
                        AnimationValue::MozBoxOrient(
                            computed
                        )
                    }
                    AnimatableLonghand::MozBoxPack => {
                        let computed = computed_values
                            .get_xul()
                            .clone__moz_box_pack();
                        AnimationValue::MozBoxPack(
                            computed
                        )
                    }
                    AnimatableLonghand::MozStackSizing => {
                        let computed = computed_values
                            .get_xul()
                            .clone__moz_stack_sizing();
                        AnimationValue::MozStackSizing(
                            computed
                        )
                    }
                    AnimatableLonghand::MozBoxOrdinalGroup => {
                        let computed = computed_values
                            .get_xul()
                            .clone__moz_box_ordinal_group();
                        AnimationValue::MozBoxOrdinalGroup(
                            computed
                        )
                    }
        }
    }
}

impl Animate for AnimationValue {
    fn animate(&self, other: &Self, procedure: Procedure) -> Result<Self, ()> {
        match (self, other) {
            (
                &AnimationValue::BackgroundColor(ref this),
                &AnimationValue::BackgroundColor(ref other),
            ) => {
                Ok(AnimationValue::BackgroundColor(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::BackgroundImage(ref this),
                &AnimationValue::BackgroundImage(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::BackgroundImage(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::BackgroundPositionX(ref this),
                &AnimationValue::BackgroundPositionX(ref other),
            ) => {
                Ok(AnimationValue::BackgroundPositionX(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::BackgroundPositionY(ref this),
                &AnimationValue::BackgroundPositionY(ref other),
            ) => {
                Ok(AnimationValue::BackgroundPositionY(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::BackgroundRepeat(ref this),
                &AnimationValue::BackgroundRepeat(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::BackgroundRepeat(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::BackgroundAttachment(ref this),
                &AnimationValue::BackgroundAttachment(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::BackgroundAttachment(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::BackgroundClip(ref this),
                &AnimationValue::BackgroundClip(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::BackgroundClip(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::BackgroundOrigin(ref this),
                &AnimationValue::BackgroundOrigin(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::BackgroundOrigin(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::BackgroundSize(ref this),
                &AnimationValue::BackgroundSize(ref other),
            ) => {
                Ok(AnimationValue::BackgroundSize(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::BackgroundBlendMode(ref this),
                &AnimationValue::BackgroundBlendMode(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::BackgroundBlendMode(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::BorderTopColor(ref this),
                &AnimationValue::BorderTopColor(ref other),
            ) => {
                Ok(AnimationValue::BorderTopColor(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::BorderTopStyle(ref this),
                &AnimationValue::BorderTopStyle(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::BorderTopStyle(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::BorderTopWidth(ref this),
                &AnimationValue::BorderTopWidth(ref other),
            ) => {
                Ok(AnimationValue::BorderTopWidth(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::BorderLeftColor(ref this),
                &AnimationValue::BorderLeftColor(ref other),
            ) => {
                Ok(AnimationValue::BorderLeftColor(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::BorderLeftStyle(ref this),
                &AnimationValue::BorderLeftStyle(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::BorderLeftStyle(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::BorderLeftWidth(ref this),
                &AnimationValue::BorderLeftWidth(ref other),
            ) => {
                Ok(AnimationValue::BorderLeftWidth(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::BorderBottomColor(ref this),
                &AnimationValue::BorderBottomColor(ref other),
            ) => {
                Ok(AnimationValue::BorderBottomColor(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::BorderBottomStyle(ref this),
                &AnimationValue::BorderBottomStyle(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::BorderBottomStyle(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::BorderBottomWidth(ref this),
                &AnimationValue::BorderBottomWidth(ref other),
            ) => {
                Ok(AnimationValue::BorderBottomWidth(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::BorderRightColor(ref this),
                &AnimationValue::BorderRightColor(ref other),
            ) => {
                Ok(AnimationValue::BorderRightColor(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::BorderRightStyle(ref this),
                &AnimationValue::BorderRightStyle(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::BorderRightStyle(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::BorderRightWidth(ref this),
                &AnimationValue::BorderRightWidth(ref other),
            ) => {
                Ok(AnimationValue::BorderRightWidth(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::BorderTopLeftRadius(ref this),
                &AnimationValue::BorderTopLeftRadius(ref other),
            ) => {
                Ok(AnimationValue::BorderTopLeftRadius(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::BorderTopRightRadius(ref this),
                &AnimationValue::BorderTopRightRadius(ref other),
            ) => {
                Ok(AnimationValue::BorderTopRightRadius(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::BorderBottomRightRadius(ref this),
                &AnimationValue::BorderBottomRightRadius(ref other),
            ) => {
                Ok(AnimationValue::BorderBottomRightRadius(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::BorderBottomLeftRadius(ref this),
                &AnimationValue::BorderBottomLeftRadius(ref other),
            ) => {
                Ok(AnimationValue::BorderBottomLeftRadius(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::MozBorderTopColors(ref this),
                &AnimationValue::MozBorderTopColors(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::MozBorderTopColors(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::MozBorderLeftColors(ref this),
                &AnimationValue::MozBorderLeftColors(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::MozBorderLeftColors(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::MozBorderBottomColors(ref this),
                &AnimationValue::MozBorderBottomColors(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::MozBorderBottomColors(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::MozBorderRightColors(ref this),
                &AnimationValue::MozBorderRightColors(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::MozBorderRightColors(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::BoxDecorationBreak(ref this),
                &AnimationValue::BoxDecorationBreak(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::BoxDecorationBreak(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::MozFloatEdge(ref this),
                &AnimationValue::MozFloatEdge(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::MozFloatEdge(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::BorderImageSource(ref this),
                &AnimationValue::BorderImageSource(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::BorderImageSource(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::BorderImageOutset(ref this),
                &AnimationValue::BorderImageOutset(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::BorderImageOutset(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::BorderImageRepeat(ref this),
                &AnimationValue::BorderImageRepeat(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::BorderImageRepeat(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::BorderImageWidth(ref this),
                &AnimationValue::BorderImageWidth(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::BorderImageWidth(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::BorderImageSlice(ref this),
                &AnimationValue::BorderImageSlice(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::BorderImageSlice(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::Display(ref this),
                &AnimationValue::Display(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::Display(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::Position(ref this),
                &AnimationValue::Position(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::Position(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::Float(ref this),
                &AnimationValue::Float(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::Float(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::Clear(ref this),
                &AnimationValue::Clear(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::Clear(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::VerticalAlign(ref this),
                &AnimationValue::VerticalAlign(ref other),
            ) => {
                Ok(AnimationValue::VerticalAlign(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::OverflowClipBox(ref this),
                &AnimationValue::OverflowClipBox(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::OverflowClipBox(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::OverflowX(ref this),
                &AnimationValue::OverflowX(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::OverflowX(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::OverflowY(ref this),
                &AnimationValue::OverflowY(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::OverflowY(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::ScrollSnapPointsX(ref this),
                &AnimationValue::ScrollSnapPointsX(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::ScrollSnapPointsX(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::ScrollSnapPointsY(ref this),
                &AnimationValue::ScrollSnapPointsY(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::ScrollSnapPointsY(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::ScrollSnapDestination(ref this),
                &AnimationValue::ScrollSnapDestination(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::ScrollSnapDestination(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::ScrollSnapCoordinate(ref this),
                &AnimationValue::ScrollSnapCoordinate(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::ScrollSnapCoordinate(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::Transform(ref this),
                &AnimationValue::Transform(ref other),
            ) => {
                Ok(AnimationValue::Transform(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::ScrollBehavior(ref this),
                &AnimationValue::ScrollBehavior(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::ScrollBehavior(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::ScrollSnapTypeX(ref this),
                &AnimationValue::ScrollSnapTypeX(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::ScrollSnapTypeX(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::ScrollSnapTypeY(ref this),
                &AnimationValue::ScrollSnapTypeY(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::ScrollSnapTypeY(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::Isolation(ref this),
                &AnimationValue::Isolation(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::Isolation(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::PageBreakAfter(ref this),
                &AnimationValue::PageBreakAfter(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::PageBreakAfter(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::PageBreakBefore(ref this),
                &AnimationValue::PageBreakBefore(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::PageBreakBefore(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::PageBreakInside(ref this),
                &AnimationValue::PageBreakInside(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::PageBreakInside(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::Resize(ref this),
                &AnimationValue::Resize(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::Resize(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::Perspective(ref this),
                &AnimationValue::Perspective(ref other),
            ) => {
                Ok(AnimationValue::Perspective(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::PerspectiveOrigin(ref this),
                &AnimationValue::PerspectiveOrigin(ref other),
            ) => {
                Ok(AnimationValue::PerspectiveOrigin(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::BackfaceVisibility(ref this),
                &AnimationValue::BackfaceVisibility(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::BackfaceVisibility(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::TransformBox(ref this),
                &AnimationValue::TransformBox(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::TransformBox(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::TransformStyle(ref this),
                &AnimationValue::TransformStyle(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::TransformStyle(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::TransformOrigin(ref this),
                &AnimationValue::TransformOrigin(ref other),
            ) => {
                Ok(AnimationValue::TransformOrigin(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::Contain(ref this),
                &AnimationValue::Contain(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::Contain(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::MozAppearance(ref this),
                &AnimationValue::MozAppearance(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::MozAppearance(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::MozOrient(ref this),
                &AnimationValue::MozOrient(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::MozOrient(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::WillChange(ref this),
                &AnimationValue::WillChange(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::WillChange(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::ShapeOutside(ref this),
                &AnimationValue::ShapeOutside(ref other),
            ) => {
                Ok(AnimationValue::ShapeOutside(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::TouchAction(ref this),
                &AnimationValue::TouchAction(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::TouchAction(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::Color(ref this),
                &AnimationValue::Color(ref other),
            ) => {
                Ok(AnimationValue::Color(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::ColumnWidth(ref this),
                &AnimationValue::ColumnWidth(ref other),
            ) => {
                Ok(AnimationValue::ColumnWidth(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::ColumnCount(ref this),
                &AnimationValue::ColumnCount(ref other),
            ) => {
                Ok(AnimationValue::ColumnCount(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::ColumnGap(ref this),
                &AnimationValue::ColumnGap(ref other),
            ) => {
                Ok(AnimationValue::ColumnGap(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::ColumnFill(ref this),
                &AnimationValue::ColumnFill(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::ColumnFill(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::ColumnRuleWidth(ref this),
                &AnimationValue::ColumnRuleWidth(ref other),
            ) => {
                Ok(AnimationValue::ColumnRuleWidth(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::ColumnRuleColor(ref this),
                &AnimationValue::ColumnRuleColor(ref other),
            ) => {
                Ok(AnimationValue::ColumnRuleColor(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::ColumnSpan(ref this),
                &AnimationValue::ColumnSpan(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::ColumnSpan(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::ColumnRuleStyle(ref this),
                &AnimationValue::ColumnRuleStyle(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::ColumnRuleStyle(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::Content(ref this),
                &AnimationValue::Content(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::Content(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::CounterIncrement(ref this),
                &AnimationValue::CounterIncrement(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::CounterIncrement(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::CounterReset(ref this),
                &AnimationValue::CounterReset(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::CounterReset(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::Opacity(ref this),
                &AnimationValue::Opacity(ref other),
            ) => {
                Ok(AnimationValue::Opacity(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::BoxShadow(ref this),
                &AnimationValue::BoxShadow(ref other),
            ) => {
                Ok(AnimationValue::BoxShadow(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::Clip(ref this),
                &AnimationValue::Clip(ref other),
            ) => {
                Ok(AnimationValue::Clip(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::Filter(ref this),
                &AnimationValue::Filter(ref other),
            ) => {
                Ok(AnimationValue::Filter(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::MixBlendMode(ref this),
                &AnimationValue::MixBlendMode(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::MixBlendMode(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::FontFamily(ref this),
                &AnimationValue::FontFamily(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::FontFamily(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::FontStyle(ref this),
                &AnimationValue::FontStyle(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::FontStyle(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::FontVariantCaps(ref this),
                &AnimationValue::FontVariantCaps(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::FontVariantCaps(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::FontWeight(ref this),
                &AnimationValue::FontWeight(ref other),
            ) => {
                Ok(AnimationValue::FontWeight(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::FontSize(ref this),
                &AnimationValue::FontSize(ref other),
            ) => {
                Ok(AnimationValue::FontSize(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::FontSizeAdjust(ref this),
                &AnimationValue::FontSizeAdjust(ref other),
            ) => {
                Ok(AnimationValue::FontSizeAdjust(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::FontSynthesis(ref this),
                &AnimationValue::FontSynthesis(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::FontSynthesis(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::FontStretch(ref this),
                &AnimationValue::FontStretch(ref other),
            ) => {
                Ok(AnimationValue::FontStretch(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::FontKerning(ref this),
                &AnimationValue::FontKerning(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::FontKerning(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::FontVariantAlternates(ref this),
                &AnimationValue::FontVariantAlternates(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::FontVariantAlternates(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::FontVariantEastAsian(ref this),
                &AnimationValue::FontVariantEastAsian(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::FontVariantEastAsian(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::FontVariantLigatures(ref this),
                &AnimationValue::FontVariantLigatures(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::FontVariantLigatures(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::FontVariantNumeric(ref this),
                &AnimationValue::FontVariantNumeric(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::FontVariantNumeric(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::FontVariantPosition(ref this),
                &AnimationValue::FontVariantPosition(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::FontVariantPosition(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::FontFeatureSettings(ref this),
                &AnimationValue::FontFeatureSettings(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::FontFeatureSettings(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::FontVariationSettings(ref this),
                &AnimationValue::FontVariationSettings(ref other),
            ) => {
                Ok(AnimationValue::FontVariationSettings(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::FontLanguageOverride(ref this),
                &AnimationValue::FontLanguageOverride(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::FontLanguageOverride(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::MozOsxFontSmoothing(ref this),
                &AnimationValue::MozOsxFontSmoothing(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::MozOsxFontSmoothing(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::Visibility(ref this),
                &AnimationValue::Visibility(ref other),
            ) => {
                Ok(AnimationValue::Visibility(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::WritingMode(ref this),
                &AnimationValue::WritingMode(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::WritingMode(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::Direction(ref this),
                &AnimationValue::Direction(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::Direction(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::TextOrientation(ref this),
                &AnimationValue::TextOrientation(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::TextOrientation(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::ColorAdjust(ref this),
                &AnimationValue::ColorAdjust(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::ColorAdjust(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::ImageRendering(ref this),
                &AnimationValue::ImageRendering(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::ImageRendering(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::ImageOrientation(ref this),
                &AnimationValue::ImageOrientation(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::ImageOrientation(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::BorderCollapse(ref this),
                &AnimationValue::BorderCollapse(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::BorderCollapse(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::EmptyCells(ref this),
                &AnimationValue::EmptyCells(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::EmptyCells(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::CaptionSide(ref this),
                &AnimationValue::CaptionSide(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::CaptionSide(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::BorderSpacing(ref this),
                &AnimationValue::BorderSpacing(ref other),
            ) => {
                Ok(AnimationValue::BorderSpacing(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::LineHeight(ref this),
                &AnimationValue::LineHeight(ref other),
            ) => {
                Ok(AnimationValue::LineHeight(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::TextTransform(ref this),
                &AnimationValue::TextTransform(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::TextTransform(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::Hyphens(ref this),
                &AnimationValue::Hyphens(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::Hyphens(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::MozTextSizeAdjust(ref this),
                &AnimationValue::MozTextSizeAdjust(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::MozTextSizeAdjust(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::TextIndent(ref this),
                &AnimationValue::TextIndent(ref other),
            ) => {
                Ok(AnimationValue::TextIndent(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::OverflowWrap(ref this),
                &AnimationValue::OverflowWrap(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::OverflowWrap(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::WordBreak(ref this),
                &AnimationValue::WordBreak(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::WordBreak(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::TextJustify(ref this),
                &AnimationValue::TextJustify(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::TextJustify(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::TextAlignLast(ref this),
                &AnimationValue::TextAlignLast(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::TextAlignLast(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::TextAlign(ref this),
                &AnimationValue::TextAlign(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::TextAlign(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::LetterSpacing(ref this),
                &AnimationValue::LetterSpacing(ref other),
            ) => {
                Ok(AnimationValue::LetterSpacing(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::WordSpacing(ref this),
                &AnimationValue::WordSpacing(ref other),
            ) => {
                Ok(AnimationValue::WordSpacing(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::WhiteSpace(ref this),
                &AnimationValue::WhiteSpace(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::WhiteSpace(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::TextShadow(ref this),
                &AnimationValue::TextShadow(ref other),
            ) => {
                Ok(AnimationValue::TextShadow(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::TextEmphasisStyle(ref this),
                &AnimationValue::TextEmphasisStyle(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::TextEmphasisStyle(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::TextEmphasisPosition(ref this),
                &AnimationValue::TextEmphasisPosition(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::TextEmphasisPosition(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::TextEmphasisColor(ref this),
                &AnimationValue::TextEmphasisColor(ref other),
            ) => {
                Ok(AnimationValue::TextEmphasisColor(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::MozTabSize(ref this),
                &AnimationValue::MozTabSize(ref other),
            ) => {
                Ok(AnimationValue::MozTabSize(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::WebkitTextFillColor(ref this),
                &AnimationValue::WebkitTextFillColor(ref other),
            ) => {
                Ok(AnimationValue::WebkitTextFillColor(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::WebkitTextStrokeColor(ref this),
                &AnimationValue::WebkitTextStrokeColor(ref other),
            ) => {
                Ok(AnimationValue::WebkitTextStrokeColor(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::WebkitTextStrokeWidth(ref this),
                &AnimationValue::WebkitTextStrokeWidth(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::WebkitTextStrokeWidth(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::RubyAlign(ref this),
                &AnimationValue::RubyAlign(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::RubyAlign(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::RubyPosition(ref this),
                &AnimationValue::RubyPosition(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::RubyPosition(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::TextCombineUpright(ref this),
                &AnimationValue::TextCombineUpright(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::TextCombineUpright(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::TextRendering(ref this),
                &AnimationValue::TextRendering(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::TextRendering(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::ListStylePosition(ref this),
                &AnimationValue::ListStylePosition(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::ListStylePosition(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::ListStyleType(ref this),
                &AnimationValue::ListStyleType(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::ListStyleType(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::ListStyleImage(ref this),
                &AnimationValue::ListStyleImage(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::ListStyleImage(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::Quotes(ref this),
                &AnimationValue::Quotes(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::Quotes(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::MozImageRegion(ref this),
                &AnimationValue::MozImageRegion(ref other),
            ) => {
                Ok(AnimationValue::MozImageRegion(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::MarginTop(ref this),
                &AnimationValue::MarginTop(ref other),
            ) => {
                Ok(AnimationValue::MarginTop(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::MarginLeft(ref this),
                &AnimationValue::MarginLeft(ref other),
            ) => {
                Ok(AnimationValue::MarginLeft(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::MarginBottom(ref this),
                &AnimationValue::MarginBottom(ref other),
            ) => {
                Ok(AnimationValue::MarginBottom(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::MarginRight(ref this),
                &AnimationValue::MarginRight(ref other),
            ) => {
                Ok(AnimationValue::MarginRight(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::OutlineColor(ref this),
                &AnimationValue::OutlineColor(ref other),
            ) => {
                Ok(AnimationValue::OutlineColor(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::OutlineStyle(ref this),
                &AnimationValue::OutlineStyle(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::OutlineStyle(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::OutlineWidth(ref this),
                &AnimationValue::OutlineWidth(ref other),
            ) => {
                Ok(AnimationValue::OutlineWidth(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::MozOutlineRadiusTopleft(ref this),
                &AnimationValue::MozOutlineRadiusTopleft(ref other),
            ) => {
                Ok(AnimationValue::MozOutlineRadiusTopleft(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::MozOutlineRadiusTopright(ref this),
                &AnimationValue::MozOutlineRadiusTopright(ref other),
            ) => {
                Ok(AnimationValue::MozOutlineRadiusTopright(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::MozOutlineRadiusBottomright(ref this),
                &AnimationValue::MozOutlineRadiusBottomright(ref other),
            ) => {
                Ok(AnimationValue::MozOutlineRadiusBottomright(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::MozOutlineRadiusBottomleft(ref this),
                &AnimationValue::MozOutlineRadiusBottomleft(ref other),
            ) => {
                Ok(AnimationValue::MozOutlineRadiusBottomleft(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::OutlineOffset(ref this),
                &AnimationValue::OutlineOffset(ref other),
            ) => {
                Ok(AnimationValue::OutlineOffset(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::PaddingTop(ref this),
                &AnimationValue::PaddingTop(ref other),
            ) => {
                Ok(AnimationValue::PaddingTop(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::PaddingLeft(ref this),
                &AnimationValue::PaddingLeft(ref other),
            ) => {
                Ok(AnimationValue::PaddingLeft(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::PaddingBottom(ref this),
                &AnimationValue::PaddingBottom(ref other),
            ) => {
                Ok(AnimationValue::PaddingBottom(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::PaddingRight(ref this),
                &AnimationValue::PaddingRight(ref other),
            ) => {
                Ok(AnimationValue::PaddingRight(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::Cursor(ref this),
                &AnimationValue::Cursor(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::Cursor(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::PointerEvents(ref this),
                &AnimationValue::PointerEvents(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::PointerEvents(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::MozUserInput(ref this),
                &AnimationValue::MozUserInput(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::MozUserInput(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::MozUserModify(ref this),
                &AnimationValue::MozUserModify(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::MozUserModify(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::MozUserFocus(ref this),
                &AnimationValue::MozUserFocus(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::MozUserFocus(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::CaretColor(ref this),
                &AnimationValue::CaretColor(ref other),
            ) => {
                Ok(AnimationValue::CaretColor(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::Top(ref this),
                &AnimationValue::Top(ref other),
            ) => {
                Ok(AnimationValue::Top(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::Left(ref this),
                &AnimationValue::Left(ref other),
            ) => {
                Ok(AnimationValue::Left(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::Bottom(ref this),
                &AnimationValue::Bottom(ref other),
            ) => {
                Ok(AnimationValue::Bottom(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::Right(ref this),
                &AnimationValue::Right(ref other),
            ) => {
                Ok(AnimationValue::Right(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::ZIndex(ref this),
                &AnimationValue::ZIndex(ref other),
            ) => {
                Ok(AnimationValue::ZIndex(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::FlexDirection(ref this),
                &AnimationValue::FlexDirection(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::FlexDirection(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::FlexWrap(ref this),
                &AnimationValue::FlexWrap(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::FlexWrap(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::JustifyContent(ref this),
                &AnimationValue::JustifyContent(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::JustifyContent(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::AlignContent(ref this),
                &AnimationValue::AlignContent(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::AlignContent(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::AlignItems(ref this),
                &AnimationValue::AlignItems(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::AlignItems(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::JustifyItems(ref this),
                &AnimationValue::JustifyItems(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::JustifyItems(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::FlexGrow(ref this),
                &AnimationValue::FlexGrow(ref other),
            ) => {
                Ok(AnimationValue::FlexGrow(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::FlexShrink(ref this),
                &AnimationValue::FlexShrink(ref other),
            ) => {
                Ok(AnimationValue::FlexShrink(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::AlignSelf(ref this),
                &AnimationValue::AlignSelf(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::AlignSelf(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::JustifySelf(ref this),
                &AnimationValue::JustifySelf(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::JustifySelf(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::Order(ref this),
                &AnimationValue::Order(ref other),
            ) => {
                Ok(AnimationValue::Order(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::FlexBasis(ref this),
                &AnimationValue::FlexBasis(ref other),
            ) => {
                Ok(AnimationValue::FlexBasis(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::Width(ref this),
                &AnimationValue::Width(ref other),
            ) => {
                Ok(AnimationValue::Width(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::MinWidth(ref this),
                &AnimationValue::MinWidth(ref other),
            ) => {
                Ok(AnimationValue::MinWidth(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::MaxWidth(ref this),
                &AnimationValue::MaxWidth(ref other),
            ) => {
                Ok(AnimationValue::MaxWidth(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::Height(ref this),
                &AnimationValue::Height(ref other),
            ) => {
                Ok(AnimationValue::Height(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::MinHeight(ref this),
                &AnimationValue::MinHeight(ref other),
            ) => {
                Ok(AnimationValue::MinHeight(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::MaxHeight(ref this),
                &AnimationValue::MaxHeight(ref other),
            ) => {
                Ok(AnimationValue::MaxHeight(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::BoxSizing(ref this),
                &AnimationValue::BoxSizing(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::BoxSizing(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::ObjectFit(ref this),
                &AnimationValue::ObjectFit(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::ObjectFit(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::ObjectPosition(ref this),
                &AnimationValue::ObjectPosition(ref other),
            ) => {
                Ok(AnimationValue::ObjectPosition(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::GridRowGap(ref this),
                &AnimationValue::GridRowGap(ref other),
            ) => {
                Ok(AnimationValue::GridRowGap(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::GridRowStart(ref this),
                &AnimationValue::GridRowStart(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::GridRowStart(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::GridRowEnd(ref this),
                &AnimationValue::GridRowEnd(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::GridRowEnd(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::GridAutoRows(ref this),
                &AnimationValue::GridAutoRows(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::GridAutoRows(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::GridTemplateRows(ref this),
                &AnimationValue::GridTemplateRows(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::GridTemplateRows(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::GridColumnGap(ref this),
                &AnimationValue::GridColumnGap(ref other),
            ) => {
                Ok(AnimationValue::GridColumnGap(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::GridColumnStart(ref this),
                &AnimationValue::GridColumnStart(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::GridColumnStart(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::GridColumnEnd(ref this),
                &AnimationValue::GridColumnEnd(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::GridColumnEnd(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::GridAutoColumns(ref this),
                &AnimationValue::GridAutoColumns(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::GridAutoColumns(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::GridTemplateColumns(ref this),
                &AnimationValue::GridTemplateColumns(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::GridTemplateColumns(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::GridAutoFlow(ref this),
                &AnimationValue::GridAutoFlow(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::GridAutoFlow(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::GridTemplateAreas(ref this),
                &AnimationValue::GridTemplateAreas(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::GridTemplateAreas(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::TableLayout(ref this),
                &AnimationValue::TableLayout(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::TableLayout(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::TextOverflow(ref this),
                &AnimationValue::TextOverflow(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::TextOverflow(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::UnicodeBidi(ref this),
                &AnimationValue::UnicodeBidi(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::UnicodeBidi(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::TextDecorationLine(ref this),
                &AnimationValue::TextDecorationLine(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::TextDecorationLine(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::TextDecorationStyle(ref this),
                &AnimationValue::TextDecorationStyle(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::TextDecorationStyle(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::TextDecorationColor(ref this),
                &AnimationValue::TextDecorationColor(ref other),
            ) => {
                Ok(AnimationValue::TextDecorationColor(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::InitialLetter(ref this),
                &AnimationValue::InitialLetter(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::InitialLetter(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::ImeMode(ref this),
                &AnimationValue::ImeMode(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::ImeMode(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::MozUserSelect(ref this),
                &AnimationValue::MozUserSelect(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::MozUserSelect(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::MozWindowDragging(ref this),
                &AnimationValue::MozWindowDragging(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::MozWindowDragging(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::MozWindowShadow(ref this),
                &AnimationValue::MozWindowShadow(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::MozWindowShadow(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::MozForceBrokenImageIcon(ref this),
                &AnimationValue::MozForceBrokenImageIcon(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::MozForceBrokenImageIcon(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::TextAnchor(ref this),
                &AnimationValue::TextAnchor(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::TextAnchor(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::ColorInterpolation(ref this),
                &AnimationValue::ColorInterpolation(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::ColorInterpolation(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::ColorInterpolationFilters(ref this),
                &AnimationValue::ColorInterpolationFilters(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::ColorInterpolationFilters(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::Fill(ref this),
                &AnimationValue::Fill(ref other),
            ) => {
                Ok(AnimationValue::Fill(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::FillOpacity(ref this),
                &AnimationValue::FillOpacity(ref other),
            ) => {
                Ok(AnimationValue::FillOpacity(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::FillRule(ref this),
                &AnimationValue::FillRule(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::FillRule(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::ShapeRendering(ref this),
                &AnimationValue::ShapeRendering(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::ShapeRendering(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::Stroke(ref this),
                &AnimationValue::Stroke(ref other),
            ) => {
                Ok(AnimationValue::Stroke(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::StrokeWidth(ref this),
                &AnimationValue::StrokeWidth(ref other),
            ) => {
                Ok(AnimationValue::StrokeWidth(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::StrokeLinecap(ref this),
                &AnimationValue::StrokeLinecap(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::StrokeLinecap(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::StrokeLinejoin(ref this),
                &AnimationValue::StrokeLinejoin(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::StrokeLinejoin(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::StrokeMiterlimit(ref this),
                &AnimationValue::StrokeMiterlimit(ref other),
            ) => {
                Ok(AnimationValue::StrokeMiterlimit(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::StrokeOpacity(ref this),
                &AnimationValue::StrokeOpacity(ref other),
            ) => {
                Ok(AnimationValue::StrokeOpacity(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::StrokeDasharray(ref this),
                &AnimationValue::StrokeDasharray(ref other),
            ) => {
                Ok(AnimationValue::StrokeDasharray(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::StrokeDashoffset(ref this),
                &AnimationValue::StrokeDashoffset(ref other),
            ) => {
                Ok(AnimationValue::StrokeDashoffset(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::ClipRule(ref this),
                &AnimationValue::ClipRule(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::ClipRule(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::MarkerStart(ref this),
                &AnimationValue::MarkerStart(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::MarkerStart(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::MarkerMid(ref this),
                &AnimationValue::MarkerMid(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::MarkerMid(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::MarkerEnd(ref this),
                &AnimationValue::MarkerEnd(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::MarkerEnd(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::PaintOrder(ref this),
                &AnimationValue::PaintOrder(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::PaintOrder(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::DominantBaseline(ref this),
                &AnimationValue::DominantBaseline(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::DominantBaseline(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::VectorEffect(ref this),
                &AnimationValue::VectorEffect(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::VectorEffect(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::StopColor(ref this),
                &AnimationValue::StopColor(ref other),
            ) => {
                Ok(AnimationValue::StopColor(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::StopOpacity(ref this),
                &AnimationValue::StopOpacity(ref other),
            ) => {
                Ok(AnimationValue::StopOpacity(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::FloodColor(ref this),
                &AnimationValue::FloodColor(ref other),
            ) => {
                Ok(AnimationValue::FloodColor(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::FloodOpacity(ref this),
                &AnimationValue::FloodOpacity(ref other),
            ) => {
                Ok(AnimationValue::FloodOpacity(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::LightingColor(ref this),
                &AnimationValue::LightingColor(ref other),
            ) => {
                Ok(AnimationValue::LightingColor(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::MaskType(ref this),
                &AnimationValue::MaskType(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::MaskType(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::ClipPath(ref this),
                &AnimationValue::ClipPath(ref other),
            ) => {
                Ok(AnimationValue::ClipPath(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::MaskMode(ref this),
                &AnimationValue::MaskMode(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::MaskMode(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::MaskRepeat(ref this),
                &AnimationValue::MaskRepeat(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::MaskRepeat(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::MaskPositionX(ref this),
                &AnimationValue::MaskPositionX(ref other),
            ) => {
                Ok(AnimationValue::MaskPositionX(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::MaskPositionY(ref this),
                &AnimationValue::MaskPositionY(ref other),
            ) => {
                Ok(AnimationValue::MaskPositionY(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::MaskClip(ref this),
                &AnimationValue::MaskClip(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::MaskClip(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::MaskOrigin(ref this),
                &AnimationValue::MaskOrigin(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::MaskOrigin(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::MaskSize(ref this),
                &AnimationValue::MaskSize(ref other),
            ) => {
                Ok(AnimationValue::MaskSize(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::MaskComposite(ref this),
                &AnimationValue::MaskComposite(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::MaskComposite(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::MaskImage(ref this),
                &AnimationValue::MaskImage(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::MaskImage(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::MozBoxAlign(ref this),
                &AnimationValue::MozBoxAlign(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::MozBoxAlign(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::MozBoxDirection(ref this),
                &AnimationValue::MozBoxDirection(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::MozBoxDirection(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::MozBoxFlex(ref this),
                &AnimationValue::MozBoxFlex(ref other),
            ) => {
                Ok(AnimationValue::MozBoxFlex(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &AnimationValue::MozBoxOrient(ref this),
                &AnimationValue::MozBoxOrient(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::MozBoxOrient(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::MozBoxPack(ref this),
                &AnimationValue::MozBoxPack(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::MozBoxPack(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::MozStackSizing(ref this),
                &AnimationValue::MozStackSizing(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::MozStackSizing(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            (
                &AnimationValue::MozBoxOrdinalGroup(ref this),
                &AnimationValue::MozBoxOrdinalGroup(ref other),
            ) => {
                if let Procedure::Interpolate { progress } = procedure {
                    Ok(AnimationValue::MozBoxOrdinalGroup(
                        if progress < 0.5 { this.clone() } else { other.clone() },
                    ))
                } else {
                    Err(())
                }
            },
            _ => {
                panic!("Unexpected AnimationValue::animate call, got: {:?}, {:?}", self, other);
            }
        }
    }
}

impl ComputeSquaredDistance for AnimationValue {
    fn compute_squared_distance(&self, other: &Self) -> Result<SquaredDistance, ()> {
        match (self, other) {
            (&AnimationValue::BackgroundColor(ref this), &AnimationValue::BackgroundColor(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::BackgroundImage(_), &AnimationValue::BackgroundImage(_)) => {
                Err(())
            },
            (&AnimationValue::BackgroundPositionX(ref this), &AnimationValue::BackgroundPositionX(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::BackgroundPositionY(ref this), &AnimationValue::BackgroundPositionY(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::BackgroundRepeat(_), &AnimationValue::BackgroundRepeat(_)) => {
                Err(())
            },
            (&AnimationValue::BackgroundAttachment(_), &AnimationValue::BackgroundAttachment(_)) => {
                Err(())
            },
            (&AnimationValue::BackgroundClip(_), &AnimationValue::BackgroundClip(_)) => {
                Err(())
            },
            (&AnimationValue::BackgroundOrigin(_), &AnimationValue::BackgroundOrigin(_)) => {
                Err(())
            },
            (&AnimationValue::BackgroundSize(ref this), &AnimationValue::BackgroundSize(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::BackgroundBlendMode(_), &AnimationValue::BackgroundBlendMode(_)) => {
                Err(())
            },
            (&AnimationValue::BorderTopColor(ref this), &AnimationValue::BorderTopColor(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::BorderTopStyle(_), &AnimationValue::BorderTopStyle(_)) => {
                Err(())
            },
            (&AnimationValue::BorderTopWidth(ref this), &AnimationValue::BorderTopWidth(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::BorderLeftColor(ref this), &AnimationValue::BorderLeftColor(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::BorderLeftStyle(_), &AnimationValue::BorderLeftStyle(_)) => {
                Err(())
            },
            (&AnimationValue::BorderLeftWidth(ref this), &AnimationValue::BorderLeftWidth(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::BorderBottomColor(ref this), &AnimationValue::BorderBottomColor(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::BorderBottomStyle(_), &AnimationValue::BorderBottomStyle(_)) => {
                Err(())
            },
            (&AnimationValue::BorderBottomWidth(ref this), &AnimationValue::BorderBottomWidth(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::BorderRightColor(ref this), &AnimationValue::BorderRightColor(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::BorderRightStyle(_), &AnimationValue::BorderRightStyle(_)) => {
                Err(())
            },
            (&AnimationValue::BorderRightWidth(ref this), &AnimationValue::BorderRightWidth(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::BorderTopLeftRadius(ref this), &AnimationValue::BorderTopLeftRadius(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::BorderTopRightRadius(ref this), &AnimationValue::BorderTopRightRadius(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::BorderBottomRightRadius(ref this), &AnimationValue::BorderBottomRightRadius(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::BorderBottomLeftRadius(ref this), &AnimationValue::BorderBottomLeftRadius(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::MozBorderTopColors(_), &AnimationValue::MozBorderTopColors(_)) => {
                Err(())
            },
            (&AnimationValue::MozBorderLeftColors(_), &AnimationValue::MozBorderLeftColors(_)) => {
                Err(())
            },
            (&AnimationValue::MozBorderBottomColors(_), &AnimationValue::MozBorderBottomColors(_)) => {
                Err(())
            },
            (&AnimationValue::MozBorderRightColors(_), &AnimationValue::MozBorderRightColors(_)) => {
                Err(())
            },
            (&AnimationValue::BoxDecorationBreak(_), &AnimationValue::BoxDecorationBreak(_)) => {
                Err(())
            },
            (&AnimationValue::MozFloatEdge(_), &AnimationValue::MozFloatEdge(_)) => {
                Err(())
            },
            (&AnimationValue::BorderImageSource(_), &AnimationValue::BorderImageSource(_)) => {
                Err(())
            },
            (&AnimationValue::BorderImageOutset(_), &AnimationValue::BorderImageOutset(_)) => {
                Err(())
            },
            (&AnimationValue::BorderImageRepeat(_), &AnimationValue::BorderImageRepeat(_)) => {
                Err(())
            },
            (&AnimationValue::BorderImageWidth(_), &AnimationValue::BorderImageWidth(_)) => {
                Err(())
            },
            (&AnimationValue::BorderImageSlice(_), &AnimationValue::BorderImageSlice(_)) => {
                Err(())
            },
            (&AnimationValue::Display(_), &AnimationValue::Display(_)) => {
                Err(())
            },
            (&AnimationValue::Position(_), &AnimationValue::Position(_)) => {
                Err(())
            },
            (&AnimationValue::Float(_), &AnimationValue::Float(_)) => {
                Err(())
            },
            (&AnimationValue::Clear(_), &AnimationValue::Clear(_)) => {
                Err(())
            },
            (&AnimationValue::VerticalAlign(ref this), &AnimationValue::VerticalAlign(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::OverflowClipBox(_), &AnimationValue::OverflowClipBox(_)) => {
                Err(())
            },
            (&AnimationValue::OverflowX(_), &AnimationValue::OverflowX(_)) => {
                Err(())
            },
            (&AnimationValue::OverflowY(_), &AnimationValue::OverflowY(_)) => {
                Err(())
            },
            (&AnimationValue::ScrollSnapPointsX(_), &AnimationValue::ScrollSnapPointsX(_)) => {
                Err(())
            },
            (&AnimationValue::ScrollSnapPointsY(_), &AnimationValue::ScrollSnapPointsY(_)) => {
                Err(())
            },
            (&AnimationValue::ScrollSnapDestination(_), &AnimationValue::ScrollSnapDestination(_)) => {
                Err(())
            },
            (&AnimationValue::ScrollSnapCoordinate(_), &AnimationValue::ScrollSnapCoordinate(_)) => {
                Err(())
            },
            (&AnimationValue::Transform(ref this), &AnimationValue::Transform(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::ScrollBehavior(_), &AnimationValue::ScrollBehavior(_)) => {
                Err(())
            },
            (&AnimationValue::ScrollSnapTypeX(_), &AnimationValue::ScrollSnapTypeX(_)) => {
                Err(())
            },
            (&AnimationValue::ScrollSnapTypeY(_), &AnimationValue::ScrollSnapTypeY(_)) => {
                Err(())
            },
            (&AnimationValue::Isolation(_), &AnimationValue::Isolation(_)) => {
                Err(())
            },
            (&AnimationValue::PageBreakAfter(_), &AnimationValue::PageBreakAfter(_)) => {
                Err(())
            },
            (&AnimationValue::PageBreakBefore(_), &AnimationValue::PageBreakBefore(_)) => {
                Err(())
            },
            (&AnimationValue::PageBreakInside(_), &AnimationValue::PageBreakInside(_)) => {
                Err(())
            },
            (&AnimationValue::Resize(_), &AnimationValue::Resize(_)) => {
                Err(())
            },
            (&AnimationValue::Perspective(ref this), &AnimationValue::Perspective(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::PerspectiveOrigin(ref this), &AnimationValue::PerspectiveOrigin(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::BackfaceVisibility(_), &AnimationValue::BackfaceVisibility(_)) => {
                Err(())
            },
            (&AnimationValue::TransformBox(_), &AnimationValue::TransformBox(_)) => {
                Err(())
            },
            (&AnimationValue::TransformStyle(_), &AnimationValue::TransformStyle(_)) => {
                Err(())
            },
            (&AnimationValue::TransformOrigin(ref this), &AnimationValue::TransformOrigin(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::Contain(_), &AnimationValue::Contain(_)) => {
                Err(())
            },
            (&AnimationValue::MozAppearance(_), &AnimationValue::MozAppearance(_)) => {
                Err(())
            },
            (&AnimationValue::MozOrient(_), &AnimationValue::MozOrient(_)) => {
                Err(())
            },
            (&AnimationValue::WillChange(_), &AnimationValue::WillChange(_)) => {
                Err(())
            },
            (&AnimationValue::ShapeOutside(ref this), &AnimationValue::ShapeOutside(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::TouchAction(_), &AnimationValue::TouchAction(_)) => {
                Err(())
            },
            (&AnimationValue::Color(ref this), &AnimationValue::Color(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::ColumnWidth(ref this), &AnimationValue::ColumnWidth(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::ColumnCount(ref this), &AnimationValue::ColumnCount(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::ColumnGap(ref this), &AnimationValue::ColumnGap(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::ColumnFill(_), &AnimationValue::ColumnFill(_)) => {
                Err(())
            },
            (&AnimationValue::ColumnRuleWidth(ref this), &AnimationValue::ColumnRuleWidth(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::ColumnRuleColor(ref this), &AnimationValue::ColumnRuleColor(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::ColumnSpan(_), &AnimationValue::ColumnSpan(_)) => {
                Err(())
            },
            (&AnimationValue::ColumnRuleStyle(_), &AnimationValue::ColumnRuleStyle(_)) => {
                Err(())
            },
            (&AnimationValue::Content(_), &AnimationValue::Content(_)) => {
                Err(())
            },
            (&AnimationValue::CounterIncrement(_), &AnimationValue::CounterIncrement(_)) => {
                Err(())
            },
            (&AnimationValue::CounterReset(_), &AnimationValue::CounterReset(_)) => {
                Err(())
            },
            (&AnimationValue::Opacity(ref this), &AnimationValue::Opacity(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::BoxShadow(ref this), &AnimationValue::BoxShadow(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::Clip(ref this), &AnimationValue::Clip(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::Filter(ref this), &AnimationValue::Filter(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::MixBlendMode(_), &AnimationValue::MixBlendMode(_)) => {
                Err(())
            },
            (&AnimationValue::FontFamily(_), &AnimationValue::FontFamily(_)) => {
                Err(())
            },
            (&AnimationValue::FontStyle(_), &AnimationValue::FontStyle(_)) => {
                Err(())
            },
            (&AnimationValue::FontVariantCaps(_), &AnimationValue::FontVariantCaps(_)) => {
                Err(())
            },
            (&AnimationValue::FontWeight(ref this), &AnimationValue::FontWeight(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::FontSize(ref this), &AnimationValue::FontSize(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::FontSizeAdjust(ref this), &AnimationValue::FontSizeAdjust(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::FontSynthesis(_), &AnimationValue::FontSynthesis(_)) => {
                Err(())
            },
            (&AnimationValue::FontStretch(ref this), &AnimationValue::FontStretch(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::FontKerning(_), &AnimationValue::FontKerning(_)) => {
                Err(())
            },
            (&AnimationValue::FontVariantAlternates(_), &AnimationValue::FontVariantAlternates(_)) => {
                Err(())
            },
            (&AnimationValue::FontVariantEastAsian(_), &AnimationValue::FontVariantEastAsian(_)) => {
                Err(())
            },
            (&AnimationValue::FontVariantLigatures(_), &AnimationValue::FontVariantLigatures(_)) => {
                Err(())
            },
            (&AnimationValue::FontVariantNumeric(_), &AnimationValue::FontVariantNumeric(_)) => {
                Err(())
            },
            (&AnimationValue::FontVariantPosition(_), &AnimationValue::FontVariantPosition(_)) => {
                Err(())
            },
            (&AnimationValue::FontFeatureSettings(_), &AnimationValue::FontFeatureSettings(_)) => {
                Err(())
            },
            (&AnimationValue::FontVariationSettings(ref this), &AnimationValue::FontVariationSettings(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::FontLanguageOverride(_), &AnimationValue::FontLanguageOverride(_)) => {
                Err(())
            },
            (&AnimationValue::MozOsxFontSmoothing(_), &AnimationValue::MozOsxFontSmoothing(_)) => {
                Err(())
            },
            (&AnimationValue::Visibility(ref this), &AnimationValue::Visibility(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::WritingMode(_), &AnimationValue::WritingMode(_)) => {
                Err(())
            },
            (&AnimationValue::Direction(_), &AnimationValue::Direction(_)) => {
                Err(())
            },
            (&AnimationValue::TextOrientation(_), &AnimationValue::TextOrientation(_)) => {
                Err(())
            },
            (&AnimationValue::ColorAdjust(_), &AnimationValue::ColorAdjust(_)) => {
                Err(())
            },
            (&AnimationValue::ImageRendering(_), &AnimationValue::ImageRendering(_)) => {
                Err(())
            },
            (&AnimationValue::ImageOrientation(_), &AnimationValue::ImageOrientation(_)) => {
                Err(())
            },
            (&AnimationValue::BorderCollapse(_), &AnimationValue::BorderCollapse(_)) => {
                Err(())
            },
            (&AnimationValue::EmptyCells(_), &AnimationValue::EmptyCells(_)) => {
                Err(())
            },
            (&AnimationValue::CaptionSide(_), &AnimationValue::CaptionSide(_)) => {
                Err(())
            },
            (&AnimationValue::BorderSpacing(ref this), &AnimationValue::BorderSpacing(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::LineHeight(ref this), &AnimationValue::LineHeight(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::TextTransform(_), &AnimationValue::TextTransform(_)) => {
                Err(())
            },
            (&AnimationValue::Hyphens(_), &AnimationValue::Hyphens(_)) => {
                Err(())
            },
            (&AnimationValue::MozTextSizeAdjust(_), &AnimationValue::MozTextSizeAdjust(_)) => {
                Err(())
            },
            (&AnimationValue::TextIndent(ref this), &AnimationValue::TextIndent(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::OverflowWrap(_), &AnimationValue::OverflowWrap(_)) => {
                Err(())
            },
            (&AnimationValue::WordBreak(_), &AnimationValue::WordBreak(_)) => {
                Err(())
            },
            (&AnimationValue::TextJustify(_), &AnimationValue::TextJustify(_)) => {
                Err(())
            },
            (&AnimationValue::TextAlignLast(_), &AnimationValue::TextAlignLast(_)) => {
                Err(())
            },
            (&AnimationValue::TextAlign(_), &AnimationValue::TextAlign(_)) => {
                Err(())
            },
            (&AnimationValue::LetterSpacing(ref this), &AnimationValue::LetterSpacing(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::WordSpacing(ref this), &AnimationValue::WordSpacing(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::WhiteSpace(_), &AnimationValue::WhiteSpace(_)) => {
                Err(())
            },
            (&AnimationValue::TextShadow(ref this), &AnimationValue::TextShadow(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::TextEmphasisStyle(_), &AnimationValue::TextEmphasisStyle(_)) => {
                Err(())
            },
            (&AnimationValue::TextEmphasisPosition(_), &AnimationValue::TextEmphasisPosition(_)) => {
                Err(())
            },
            (&AnimationValue::TextEmphasisColor(ref this), &AnimationValue::TextEmphasisColor(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::MozTabSize(ref this), &AnimationValue::MozTabSize(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::WebkitTextFillColor(ref this), &AnimationValue::WebkitTextFillColor(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::WebkitTextStrokeColor(ref this), &AnimationValue::WebkitTextStrokeColor(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::WebkitTextStrokeWidth(_), &AnimationValue::WebkitTextStrokeWidth(_)) => {
                Err(())
            },
            (&AnimationValue::RubyAlign(_), &AnimationValue::RubyAlign(_)) => {
                Err(())
            },
            (&AnimationValue::RubyPosition(_), &AnimationValue::RubyPosition(_)) => {
                Err(())
            },
            (&AnimationValue::TextCombineUpright(_), &AnimationValue::TextCombineUpright(_)) => {
                Err(())
            },
            (&AnimationValue::TextRendering(_), &AnimationValue::TextRendering(_)) => {
                Err(())
            },
            (&AnimationValue::ListStylePosition(_), &AnimationValue::ListStylePosition(_)) => {
                Err(())
            },
            (&AnimationValue::ListStyleType(_), &AnimationValue::ListStyleType(_)) => {
                Err(())
            },
            (&AnimationValue::ListStyleImage(_), &AnimationValue::ListStyleImage(_)) => {
                Err(())
            },
            (&AnimationValue::Quotes(_), &AnimationValue::Quotes(_)) => {
                Err(())
            },
            (&AnimationValue::MozImageRegion(ref this), &AnimationValue::MozImageRegion(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::MarginTop(ref this), &AnimationValue::MarginTop(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::MarginLeft(ref this), &AnimationValue::MarginLeft(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::MarginBottom(ref this), &AnimationValue::MarginBottom(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::MarginRight(ref this), &AnimationValue::MarginRight(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::OutlineColor(ref this), &AnimationValue::OutlineColor(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::OutlineStyle(_), &AnimationValue::OutlineStyle(_)) => {
                Err(())
            },
            (&AnimationValue::OutlineWidth(ref this), &AnimationValue::OutlineWidth(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::MozOutlineRadiusTopleft(ref this), &AnimationValue::MozOutlineRadiusTopleft(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::MozOutlineRadiusTopright(ref this), &AnimationValue::MozOutlineRadiusTopright(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::MozOutlineRadiusBottomright(ref this), &AnimationValue::MozOutlineRadiusBottomright(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::MozOutlineRadiusBottomleft(ref this), &AnimationValue::MozOutlineRadiusBottomleft(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::OutlineOffset(ref this), &AnimationValue::OutlineOffset(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::PaddingTop(ref this), &AnimationValue::PaddingTop(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::PaddingLeft(ref this), &AnimationValue::PaddingLeft(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::PaddingBottom(ref this), &AnimationValue::PaddingBottom(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::PaddingRight(ref this), &AnimationValue::PaddingRight(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::Cursor(_), &AnimationValue::Cursor(_)) => {
                Err(())
            },
            (&AnimationValue::PointerEvents(_), &AnimationValue::PointerEvents(_)) => {
                Err(())
            },
            (&AnimationValue::MozUserInput(_), &AnimationValue::MozUserInput(_)) => {
                Err(())
            },
            (&AnimationValue::MozUserModify(_), &AnimationValue::MozUserModify(_)) => {
                Err(())
            },
            (&AnimationValue::MozUserFocus(_), &AnimationValue::MozUserFocus(_)) => {
                Err(())
            },
            (&AnimationValue::CaretColor(ref this), &AnimationValue::CaretColor(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::Top(ref this), &AnimationValue::Top(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::Left(ref this), &AnimationValue::Left(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::Bottom(ref this), &AnimationValue::Bottom(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::Right(ref this), &AnimationValue::Right(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::ZIndex(ref this), &AnimationValue::ZIndex(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::FlexDirection(_), &AnimationValue::FlexDirection(_)) => {
                Err(())
            },
            (&AnimationValue::FlexWrap(_), &AnimationValue::FlexWrap(_)) => {
                Err(())
            },
            (&AnimationValue::JustifyContent(_), &AnimationValue::JustifyContent(_)) => {
                Err(())
            },
            (&AnimationValue::AlignContent(_), &AnimationValue::AlignContent(_)) => {
                Err(())
            },
            (&AnimationValue::AlignItems(_), &AnimationValue::AlignItems(_)) => {
                Err(())
            },
            (&AnimationValue::JustifyItems(_), &AnimationValue::JustifyItems(_)) => {
                Err(())
            },
            (&AnimationValue::FlexGrow(ref this), &AnimationValue::FlexGrow(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::FlexShrink(ref this), &AnimationValue::FlexShrink(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::AlignSelf(_), &AnimationValue::AlignSelf(_)) => {
                Err(())
            },
            (&AnimationValue::JustifySelf(_), &AnimationValue::JustifySelf(_)) => {
                Err(())
            },
            (&AnimationValue::Order(ref this), &AnimationValue::Order(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::FlexBasis(ref this), &AnimationValue::FlexBasis(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::Width(ref this), &AnimationValue::Width(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::MinWidth(ref this), &AnimationValue::MinWidth(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::MaxWidth(ref this), &AnimationValue::MaxWidth(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::Height(ref this), &AnimationValue::Height(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::MinHeight(ref this), &AnimationValue::MinHeight(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::MaxHeight(ref this), &AnimationValue::MaxHeight(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::BoxSizing(_), &AnimationValue::BoxSizing(_)) => {
                Err(())
            },
            (&AnimationValue::ObjectFit(_), &AnimationValue::ObjectFit(_)) => {
                Err(())
            },
            (&AnimationValue::ObjectPosition(ref this), &AnimationValue::ObjectPosition(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::GridRowGap(ref this), &AnimationValue::GridRowGap(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::GridRowStart(_), &AnimationValue::GridRowStart(_)) => {
                Err(())
            },
            (&AnimationValue::GridRowEnd(_), &AnimationValue::GridRowEnd(_)) => {
                Err(())
            },
            (&AnimationValue::GridAutoRows(_), &AnimationValue::GridAutoRows(_)) => {
                Err(())
            },
            (&AnimationValue::GridTemplateRows(_), &AnimationValue::GridTemplateRows(_)) => {
                Err(())
            },
            (&AnimationValue::GridColumnGap(ref this), &AnimationValue::GridColumnGap(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::GridColumnStart(_), &AnimationValue::GridColumnStart(_)) => {
                Err(())
            },
            (&AnimationValue::GridColumnEnd(_), &AnimationValue::GridColumnEnd(_)) => {
                Err(())
            },
            (&AnimationValue::GridAutoColumns(_), &AnimationValue::GridAutoColumns(_)) => {
                Err(())
            },
            (&AnimationValue::GridTemplateColumns(_), &AnimationValue::GridTemplateColumns(_)) => {
                Err(())
            },
            (&AnimationValue::GridAutoFlow(_), &AnimationValue::GridAutoFlow(_)) => {
                Err(())
            },
            (&AnimationValue::GridTemplateAreas(_), &AnimationValue::GridTemplateAreas(_)) => {
                Err(())
            },
            (&AnimationValue::TableLayout(_), &AnimationValue::TableLayout(_)) => {
                Err(())
            },
            (&AnimationValue::TextOverflow(_), &AnimationValue::TextOverflow(_)) => {
                Err(())
            },
            (&AnimationValue::UnicodeBidi(_), &AnimationValue::UnicodeBidi(_)) => {
                Err(())
            },
            (&AnimationValue::TextDecorationLine(_), &AnimationValue::TextDecorationLine(_)) => {
                Err(())
            },
            (&AnimationValue::TextDecorationStyle(_), &AnimationValue::TextDecorationStyle(_)) => {
                Err(())
            },
            (&AnimationValue::TextDecorationColor(ref this), &AnimationValue::TextDecorationColor(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::InitialLetter(_), &AnimationValue::InitialLetter(_)) => {
                Err(())
            },
            (&AnimationValue::ImeMode(_), &AnimationValue::ImeMode(_)) => {
                Err(())
            },
            (&AnimationValue::MozUserSelect(_), &AnimationValue::MozUserSelect(_)) => {
                Err(())
            },
            (&AnimationValue::MozWindowDragging(_), &AnimationValue::MozWindowDragging(_)) => {
                Err(())
            },
            (&AnimationValue::MozWindowShadow(_), &AnimationValue::MozWindowShadow(_)) => {
                Err(())
            },
            (&AnimationValue::MozForceBrokenImageIcon(_), &AnimationValue::MozForceBrokenImageIcon(_)) => {
                Err(())
            },
            (&AnimationValue::TextAnchor(_), &AnimationValue::TextAnchor(_)) => {
                Err(())
            },
            (&AnimationValue::ColorInterpolation(_), &AnimationValue::ColorInterpolation(_)) => {
                Err(())
            },
            (&AnimationValue::ColorInterpolationFilters(_), &AnimationValue::ColorInterpolationFilters(_)) => {
                Err(())
            },
            (&AnimationValue::Fill(ref this), &AnimationValue::Fill(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::FillOpacity(ref this), &AnimationValue::FillOpacity(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::FillRule(_), &AnimationValue::FillRule(_)) => {
                Err(())
            },
            (&AnimationValue::ShapeRendering(_), &AnimationValue::ShapeRendering(_)) => {
                Err(())
            },
            (&AnimationValue::Stroke(ref this), &AnimationValue::Stroke(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::StrokeWidth(ref this), &AnimationValue::StrokeWidth(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::StrokeLinecap(_), &AnimationValue::StrokeLinecap(_)) => {
                Err(())
            },
            (&AnimationValue::StrokeLinejoin(_), &AnimationValue::StrokeLinejoin(_)) => {
                Err(())
            },
            (&AnimationValue::StrokeMiterlimit(ref this), &AnimationValue::StrokeMiterlimit(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::StrokeOpacity(ref this), &AnimationValue::StrokeOpacity(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::StrokeDasharray(ref this), &AnimationValue::StrokeDasharray(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::StrokeDashoffset(ref this), &AnimationValue::StrokeDashoffset(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::ClipRule(_), &AnimationValue::ClipRule(_)) => {
                Err(())
            },
            (&AnimationValue::MarkerStart(_), &AnimationValue::MarkerStart(_)) => {
                Err(())
            },
            (&AnimationValue::MarkerMid(_), &AnimationValue::MarkerMid(_)) => {
                Err(())
            },
            (&AnimationValue::MarkerEnd(_), &AnimationValue::MarkerEnd(_)) => {
                Err(())
            },
            (&AnimationValue::PaintOrder(_), &AnimationValue::PaintOrder(_)) => {
                Err(())
            },
            (&AnimationValue::DominantBaseline(_), &AnimationValue::DominantBaseline(_)) => {
                Err(())
            },
            (&AnimationValue::VectorEffect(_), &AnimationValue::VectorEffect(_)) => {
                Err(())
            },
            (&AnimationValue::StopColor(ref this), &AnimationValue::StopColor(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::StopOpacity(ref this), &AnimationValue::StopOpacity(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::FloodColor(ref this), &AnimationValue::FloodColor(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::FloodOpacity(ref this), &AnimationValue::FloodOpacity(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::LightingColor(ref this), &AnimationValue::LightingColor(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::MaskType(_), &AnimationValue::MaskType(_)) => {
                Err(())
            },
            (&AnimationValue::ClipPath(ref this), &AnimationValue::ClipPath(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::MaskMode(_), &AnimationValue::MaskMode(_)) => {
                Err(())
            },
            (&AnimationValue::MaskRepeat(_), &AnimationValue::MaskRepeat(_)) => {
                Err(())
            },
            (&AnimationValue::MaskPositionX(ref this), &AnimationValue::MaskPositionX(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::MaskPositionY(ref this), &AnimationValue::MaskPositionY(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::MaskClip(_), &AnimationValue::MaskClip(_)) => {
                Err(())
            },
            (&AnimationValue::MaskOrigin(_), &AnimationValue::MaskOrigin(_)) => {
                Err(())
            },
            (&AnimationValue::MaskSize(ref this), &AnimationValue::MaskSize(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::MaskComposite(_), &AnimationValue::MaskComposite(_)) => {
                Err(())
            },
            (&AnimationValue::MaskImage(_), &AnimationValue::MaskImage(_)) => {
                Err(())
            },
            (&AnimationValue::MozBoxAlign(_), &AnimationValue::MozBoxAlign(_)) => {
                Err(())
            },
            (&AnimationValue::MozBoxDirection(_), &AnimationValue::MozBoxDirection(_)) => {
                Err(())
            },
            (&AnimationValue::MozBoxFlex(ref this), &AnimationValue::MozBoxFlex(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&AnimationValue::MozBoxOrient(_), &AnimationValue::MozBoxOrient(_)) => {
                Err(())
            },
            (&AnimationValue::MozBoxPack(_), &AnimationValue::MozBoxPack(_)) => {
                Err(())
            },
            (&AnimationValue::MozStackSizing(_), &AnimationValue::MozStackSizing(_)) => {
                Err(())
            },
            (&AnimationValue::MozBoxOrdinalGroup(_), &AnimationValue::MozBoxOrdinalGroup(_)) => {
                Err(())
            },
            _ => {
                panic!(
                    "computed values should be of the same property, got: {:?}, {:?}",
                    self,
                    other
                );
            },
        }
    }
}

impl ToAnimatedZero for AnimationValue {
    #[inline]
    fn to_animated_zero(&self) -> Result<Self, ()> {
        match *self {
            AnimationValue::BackgroundColor(ref base) => {
                Ok(AnimationValue::BackgroundColor(base.to_animated_zero()?))
            },
            AnimationValue::BackgroundPositionX(ref base) => {
                Ok(AnimationValue::BackgroundPositionX(base.to_animated_zero()?))
            },
            AnimationValue::BackgroundPositionY(ref base) => {
                Ok(AnimationValue::BackgroundPositionY(base.to_animated_zero()?))
            },
            AnimationValue::BackgroundSize(ref base) => {
                Ok(AnimationValue::BackgroundSize(base.to_animated_zero()?))
            },
            AnimationValue::BorderTopColor(ref base) => {
                Ok(AnimationValue::BorderTopColor(base.to_animated_zero()?))
            },
            AnimationValue::BorderTopWidth(ref base) => {
                Ok(AnimationValue::BorderTopWidth(base.to_animated_zero()?))
            },
            AnimationValue::BorderLeftColor(ref base) => {
                Ok(AnimationValue::BorderLeftColor(base.to_animated_zero()?))
            },
            AnimationValue::BorderLeftWidth(ref base) => {
                Ok(AnimationValue::BorderLeftWidth(base.to_animated_zero()?))
            },
            AnimationValue::BorderBottomColor(ref base) => {
                Ok(AnimationValue::BorderBottomColor(base.to_animated_zero()?))
            },
            AnimationValue::BorderBottomWidth(ref base) => {
                Ok(AnimationValue::BorderBottomWidth(base.to_animated_zero()?))
            },
            AnimationValue::BorderRightColor(ref base) => {
                Ok(AnimationValue::BorderRightColor(base.to_animated_zero()?))
            },
            AnimationValue::BorderRightWidth(ref base) => {
                Ok(AnimationValue::BorderRightWidth(base.to_animated_zero()?))
            },
            AnimationValue::BorderTopLeftRadius(ref base) => {
                Ok(AnimationValue::BorderTopLeftRadius(base.to_animated_zero()?))
            },
            AnimationValue::BorderTopRightRadius(ref base) => {
                Ok(AnimationValue::BorderTopRightRadius(base.to_animated_zero()?))
            },
            AnimationValue::BorderBottomRightRadius(ref base) => {
                Ok(AnimationValue::BorderBottomRightRadius(base.to_animated_zero()?))
            },
            AnimationValue::BorderBottomLeftRadius(ref base) => {
                Ok(AnimationValue::BorderBottomLeftRadius(base.to_animated_zero()?))
            },
            AnimationValue::VerticalAlign(ref base) => {
                Ok(AnimationValue::VerticalAlign(base.to_animated_zero()?))
            },
            AnimationValue::Transform(ref base) => {
                Ok(AnimationValue::Transform(base.to_animated_zero()?))
            },
            AnimationValue::Perspective(ref base) => {
                Ok(AnimationValue::Perspective(base.to_animated_zero()?))
            },
            AnimationValue::PerspectiveOrigin(ref base) => {
                Ok(AnimationValue::PerspectiveOrigin(base.to_animated_zero()?))
            },
            AnimationValue::TransformOrigin(ref base) => {
                Ok(AnimationValue::TransformOrigin(base.to_animated_zero()?))
            },
            AnimationValue::ShapeOutside(ref base) => {
                Ok(AnimationValue::ShapeOutside(base.to_animated_zero()?))
            },
            AnimationValue::Color(ref base) => {
                Ok(AnimationValue::Color(base.to_animated_zero()?))
            },
            AnimationValue::ColumnWidth(ref base) => {
                Ok(AnimationValue::ColumnWidth(base.to_animated_zero()?))
            },
            AnimationValue::ColumnCount(ref base) => {
                Ok(AnimationValue::ColumnCount(base.to_animated_zero()?))
            },
            AnimationValue::ColumnGap(ref base) => {
                Ok(AnimationValue::ColumnGap(base.to_animated_zero()?))
            },
            AnimationValue::ColumnRuleWidth(ref base) => {
                Ok(AnimationValue::ColumnRuleWidth(base.to_animated_zero()?))
            },
            AnimationValue::ColumnRuleColor(ref base) => {
                Ok(AnimationValue::ColumnRuleColor(base.to_animated_zero()?))
            },
            AnimationValue::Opacity(ref base) => {
                Ok(AnimationValue::Opacity(base.to_animated_zero()?))
            },
            AnimationValue::BoxShadow(ref base) => {
                Ok(AnimationValue::BoxShadow(base.to_animated_zero()?))
            },
            AnimationValue::Clip(ref base) => {
                Ok(AnimationValue::Clip(base.to_animated_zero()?))
            },
            AnimationValue::Filter(ref base) => {
                Ok(AnimationValue::Filter(base.to_animated_zero()?))
            },
            AnimationValue::FontWeight(ref base) => {
                Ok(AnimationValue::FontWeight(base.to_animated_zero()?))
            },
            AnimationValue::FontSize(ref base) => {
                Ok(AnimationValue::FontSize(base.to_animated_zero()?))
            },
            AnimationValue::FontSizeAdjust(ref base) => {
                Ok(AnimationValue::FontSizeAdjust(base.to_animated_zero()?))
            },
            AnimationValue::FontStretch(ref base) => {
                Ok(AnimationValue::FontStretch(base.to_animated_zero()?))
            },
            AnimationValue::FontVariationSettings(ref base) => {
                Ok(AnimationValue::FontVariationSettings(base.to_animated_zero()?))
            },
            AnimationValue::Visibility(ref base) => {
                Ok(AnimationValue::Visibility(base.to_animated_zero()?))
            },
            AnimationValue::BorderSpacing(ref base) => {
                Ok(AnimationValue::BorderSpacing(base.to_animated_zero()?))
            },
            AnimationValue::LineHeight(ref base) => {
                Ok(AnimationValue::LineHeight(base.to_animated_zero()?))
            },
            AnimationValue::TextIndent(ref base) => {
                Ok(AnimationValue::TextIndent(base.to_animated_zero()?))
            },
            AnimationValue::LetterSpacing(ref base) => {
                Ok(AnimationValue::LetterSpacing(base.to_animated_zero()?))
            },
            AnimationValue::WordSpacing(ref base) => {
                Ok(AnimationValue::WordSpacing(base.to_animated_zero()?))
            },
            AnimationValue::TextShadow(ref base) => {
                Ok(AnimationValue::TextShadow(base.to_animated_zero()?))
            },
            AnimationValue::TextEmphasisColor(ref base) => {
                Ok(AnimationValue::TextEmphasisColor(base.to_animated_zero()?))
            },
            AnimationValue::MozTabSize(ref base) => {
                Ok(AnimationValue::MozTabSize(base.to_animated_zero()?))
            },
            AnimationValue::WebkitTextFillColor(ref base) => {
                Ok(AnimationValue::WebkitTextFillColor(base.to_animated_zero()?))
            },
            AnimationValue::WebkitTextStrokeColor(ref base) => {
                Ok(AnimationValue::WebkitTextStrokeColor(base.to_animated_zero()?))
            },
            AnimationValue::MozImageRegion(ref base) => {
                Ok(AnimationValue::MozImageRegion(base.to_animated_zero()?))
            },
            AnimationValue::MarginTop(ref base) => {
                Ok(AnimationValue::MarginTop(base.to_animated_zero()?))
            },
            AnimationValue::MarginLeft(ref base) => {
                Ok(AnimationValue::MarginLeft(base.to_animated_zero()?))
            },
            AnimationValue::MarginBottom(ref base) => {
                Ok(AnimationValue::MarginBottom(base.to_animated_zero()?))
            },
            AnimationValue::MarginRight(ref base) => {
                Ok(AnimationValue::MarginRight(base.to_animated_zero()?))
            },
            AnimationValue::OutlineColor(ref base) => {
                Ok(AnimationValue::OutlineColor(base.to_animated_zero()?))
            },
            AnimationValue::OutlineWidth(ref base) => {
                Ok(AnimationValue::OutlineWidth(base.to_animated_zero()?))
            },
            AnimationValue::MozOutlineRadiusTopleft(ref base) => {
                Ok(AnimationValue::MozOutlineRadiusTopleft(base.to_animated_zero()?))
            },
            AnimationValue::MozOutlineRadiusTopright(ref base) => {
                Ok(AnimationValue::MozOutlineRadiusTopright(base.to_animated_zero()?))
            },
            AnimationValue::MozOutlineRadiusBottomright(ref base) => {
                Ok(AnimationValue::MozOutlineRadiusBottomright(base.to_animated_zero()?))
            },
            AnimationValue::MozOutlineRadiusBottomleft(ref base) => {
                Ok(AnimationValue::MozOutlineRadiusBottomleft(base.to_animated_zero()?))
            },
            AnimationValue::OutlineOffset(ref base) => {
                Ok(AnimationValue::OutlineOffset(base.to_animated_zero()?))
            },
            AnimationValue::PaddingTop(ref base) => {
                Ok(AnimationValue::PaddingTop(base.to_animated_zero()?))
            },
            AnimationValue::PaddingLeft(ref base) => {
                Ok(AnimationValue::PaddingLeft(base.to_animated_zero()?))
            },
            AnimationValue::PaddingBottom(ref base) => {
                Ok(AnimationValue::PaddingBottom(base.to_animated_zero()?))
            },
            AnimationValue::PaddingRight(ref base) => {
                Ok(AnimationValue::PaddingRight(base.to_animated_zero()?))
            },
            AnimationValue::CaretColor(ref base) => {
                Ok(AnimationValue::CaretColor(base.to_animated_zero()?))
            },
            AnimationValue::Top(ref base) => {
                Ok(AnimationValue::Top(base.to_animated_zero()?))
            },
            AnimationValue::Left(ref base) => {
                Ok(AnimationValue::Left(base.to_animated_zero()?))
            },
            AnimationValue::Bottom(ref base) => {
                Ok(AnimationValue::Bottom(base.to_animated_zero()?))
            },
            AnimationValue::Right(ref base) => {
                Ok(AnimationValue::Right(base.to_animated_zero()?))
            },
            AnimationValue::ZIndex(ref base) => {
                Ok(AnimationValue::ZIndex(base.to_animated_zero()?))
            },
            AnimationValue::FlexGrow(ref base) => {
                Ok(AnimationValue::FlexGrow(base.to_animated_zero()?))
            },
            AnimationValue::FlexShrink(ref base) => {
                Ok(AnimationValue::FlexShrink(base.to_animated_zero()?))
            },
            AnimationValue::Order(ref base) => {
                Ok(AnimationValue::Order(base.to_animated_zero()?))
            },
            AnimationValue::FlexBasis(ref base) => {
                Ok(AnimationValue::FlexBasis(base.to_animated_zero()?))
            },
            AnimationValue::Width(ref base) => {
                Ok(AnimationValue::Width(base.to_animated_zero()?))
            },
            AnimationValue::MinWidth(ref base) => {
                Ok(AnimationValue::MinWidth(base.to_animated_zero()?))
            },
            AnimationValue::MaxWidth(ref base) => {
                Ok(AnimationValue::MaxWidth(base.to_animated_zero()?))
            },
            AnimationValue::Height(ref base) => {
                Ok(AnimationValue::Height(base.to_animated_zero()?))
            },
            AnimationValue::MinHeight(ref base) => {
                Ok(AnimationValue::MinHeight(base.to_animated_zero()?))
            },
            AnimationValue::MaxHeight(ref base) => {
                Ok(AnimationValue::MaxHeight(base.to_animated_zero()?))
            },
            AnimationValue::ObjectPosition(ref base) => {
                Ok(AnimationValue::ObjectPosition(base.to_animated_zero()?))
            },
            AnimationValue::GridRowGap(ref base) => {
                Ok(AnimationValue::GridRowGap(base.to_animated_zero()?))
            },
            AnimationValue::GridColumnGap(ref base) => {
                Ok(AnimationValue::GridColumnGap(base.to_animated_zero()?))
            },
            AnimationValue::TextDecorationColor(ref base) => {
                Ok(AnimationValue::TextDecorationColor(base.to_animated_zero()?))
            },
            AnimationValue::Fill(ref base) => {
                Ok(AnimationValue::Fill(base.to_animated_zero()?))
            },
            AnimationValue::FillOpacity(ref base) => {
                Ok(AnimationValue::FillOpacity(base.to_animated_zero()?))
            },
            AnimationValue::Stroke(ref base) => {
                Ok(AnimationValue::Stroke(base.to_animated_zero()?))
            },
            AnimationValue::StrokeWidth(ref base) => {
                Ok(AnimationValue::StrokeWidth(base.to_animated_zero()?))
            },
            AnimationValue::StrokeMiterlimit(ref base) => {
                Ok(AnimationValue::StrokeMiterlimit(base.to_animated_zero()?))
            },
            AnimationValue::StrokeOpacity(ref base) => {
                Ok(AnimationValue::StrokeOpacity(base.to_animated_zero()?))
            },
            AnimationValue::StrokeDasharray(ref base) => {
                Ok(AnimationValue::StrokeDasharray(base.to_animated_zero()?))
            },
            AnimationValue::StrokeDashoffset(ref base) => {
                Ok(AnimationValue::StrokeDashoffset(base.to_animated_zero()?))
            },
            AnimationValue::StopColor(ref base) => {
                Ok(AnimationValue::StopColor(base.to_animated_zero()?))
            },
            AnimationValue::StopOpacity(ref base) => {
                Ok(AnimationValue::StopOpacity(base.to_animated_zero()?))
            },
            AnimationValue::FloodColor(ref base) => {
                Ok(AnimationValue::FloodColor(base.to_animated_zero()?))
            },
            AnimationValue::FloodOpacity(ref base) => {
                Ok(AnimationValue::FloodOpacity(base.to_animated_zero()?))
            },
            AnimationValue::LightingColor(ref base) => {
                Ok(AnimationValue::LightingColor(base.to_animated_zero()?))
            },
            AnimationValue::ClipPath(ref base) => {
                Ok(AnimationValue::ClipPath(base.to_animated_zero()?))
            },
            AnimationValue::MaskPositionX(ref base) => {
                Ok(AnimationValue::MaskPositionX(base.to_animated_zero()?))
            },
            AnimationValue::MaskPositionY(ref base) => {
                Ok(AnimationValue::MaskPositionY(base.to_animated_zero()?))
            },
            AnimationValue::MaskSize(ref base) => {
                Ok(AnimationValue::MaskSize(base.to_animated_zero()?))
            },
            AnimationValue::MozBoxFlex(ref base) => {
                Ok(AnimationValue::MozBoxFlex(base.to_animated_zero()?))
            },
            _ => Err(()),
        }
    }
}

impl RepeatableListAnimatable for LengthOrPercentage {}
impl RepeatableListAnimatable for Either<f32, LengthOrPercentage> {}
impl RepeatableListAnimatable for Either<NonNegativeNumber, NonNegativeLengthOrPercentage> {}
impl RepeatableListAnimatable for SvgLengthOrPercentageOrNumber<NonNegativeLengthOrPercentage, NonNegativeNumber> {}

macro_rules! repeated_vec_impl {
    ($($ty:ty),*) => {
        $(impl<T> Animate for $ty
        where
            T: RepeatableListAnimatable,
        {
            fn animate(&self, other: &Self, procedure: Procedure) -> Result<Self, ()> {
                // If the length of either list is zero, the least common multiple is undefined.
                if self.is_empty() || other.is_empty() {
                    return Err(());
                }
                use num_integer::lcm;
                let len = lcm(self.len(), other.len());
                self.iter().cycle().zip(other.iter().cycle()).take(len).map(|(this, other)| {
                    this.animate(other, procedure)
                }).collect()
            }
        }

        impl<T> ComputeSquaredDistance for $ty
        where
            T: ComputeSquaredDistance + RepeatableListAnimatable,
        {
            #[inline]
            fn compute_squared_distance(&self, other: &Self) -> Result<SquaredDistance, ()> {
                if self.is_empty() || other.is_empty() {
                    return Err(());
                }
                use num_integer::lcm;
                let len = lcm(self.len(), other.len());
                self.iter().cycle().zip(other.iter().cycle()).take(len).map(|(this, other)| {
                    this.compute_squared_distance(other)
                }).sum()
            }
        })*
    };
}

repeated_vec_impl!(SmallVec<[T; 1]>, Vec<T>);

/// https://drafts.csswg.org/css-transitions/#animtype-visibility
impl Animate for Visibility {
    #[inline]
    fn animate(&self, other: &Self, procedure: Procedure) -> Result<Self, ()> {
        let (this_weight, other_weight) = procedure.weights();
        match (*self, *other) {
            (Visibility::visible, _) => {
                Ok(if this_weight > 0.0 { *self } else { *other })
            },
            (_, Visibility::visible) => {
                Ok(if other_weight > 0.0 { *other } else { *self })
            },
            _ => Err(()),
        }
    }
}

impl ComputeSquaredDistance for Visibility {
    #[inline]
    fn compute_squared_distance(&self, other: &Self) -> Result<SquaredDistance, ()> {
        Ok(SquaredDistance::Value(if *self == *other { 0. } else { 1. }))
    }
}

impl ToAnimatedZero for Visibility {
    #[inline]
    fn to_animated_zero(&self) -> Result<Self, ()> {
        Err(())
    }
}

/// https://drafts.csswg.org/css-transitions/#animtype-lpcalc
impl Animate for CalcLengthOrPercentage {
    #[inline]
    fn animate(&self, other: &Self, procedure: Procedure) -> Result<Self, ()> {
        let animate_percentage_half = |this: Option<Percentage>, other: Option<Percentage>| {
            if this.is_none() && other.is_none() {
                return Ok(None);
            }
            let this = this.unwrap_or_default();
            let other = other.unwrap_or_default();
            Ok(Some(this.animate(&other, procedure)?))
        };

        let length = self.unclamped_length().animate(&other.unclamped_length(), procedure)?;
        let percentage = animate_percentage_half(self.percentage, other.percentage)?;
        Ok(CalcLengthOrPercentage::with_clamping_mode(length, percentage, self.clamping_mode))
    }
}

impl ToAnimatedZero for LengthOrPercentageOrAuto {
    #[inline]
    fn to_animated_zero(&self) -> Result<Self, ()> {
        match *self {
            LengthOrPercentageOrAuto::Length(_) |
            LengthOrPercentageOrAuto::Percentage(_) |
            LengthOrPercentageOrAuto::Calc(_) => {
                Ok(LengthOrPercentageOrAuto::Length(Length::new(0.)))
            },
            LengthOrPercentageOrAuto::Auto => Err(()),
        }
    }
}

impl ToAnimatedZero for LengthOrPercentageOrNone {
    #[inline]
    fn to_animated_zero(&self) -> Result<Self, ()> {
        match *self {
            LengthOrPercentageOrNone::Length(_) |
            LengthOrPercentageOrNone::Percentage(_) |
            LengthOrPercentageOrNone::Calc(_) => {
                Ok(LengthOrPercentageOrNone::Length(Length::new(0.)))
            },
            LengthOrPercentageOrNone::None => Err(()),
        }
    }
}

impl ToAnimatedZero for MaxLength {
    #[inline]
    fn to_animated_zero(&self) -> Result<Self, ()> { Err(()) }
}

/// http://dev.w3.org/csswg/css-transitions/#animtype-font-weight
impl Animate for FontWeight {
    #[inline]
    fn animate(&self, other: &Self, procedure: Procedure) -> Result<Self, ()> {
        let a = self.0 as f64;
        let b = other.0 as f64;
        const NORMAL: f64 = 400.;
        let (this_weight, other_weight) = procedure.weights();
        let weight = (a - NORMAL) * this_weight + (b - NORMAL) * other_weight + NORMAL;
        let weight = (weight.max(100.).min(900.) / 100.).round() * 100.;
        Ok(FontWeight(weight as u16))
    }
}

impl ToAnimatedZero for FontWeight {
    #[inline]
    fn to_animated_zero(&self) -> Result<Self, ()> {
        Ok(FontWeight::normal())
    }
}

/// https://drafts.csswg.org/css-fonts/#font-stretch-prop
impl Animate for FontStretch {
    #[inline]
    fn animate(&self, other: &Self, procedure: Procedure) -> Result<Self, ()>
    {
        let from = f64::from(*self);
        let to = f64::from(*other);
        let normal = f64::from(FontStretch::normal);
        let (this_weight, other_weight) = procedure.weights();
        let result = (from - normal) * this_weight + (to - normal) * other_weight + normal;
        Ok(result.into())
    }
}

impl ComputeSquaredDistance for FontStretch {
    #[inline]
    fn compute_squared_distance(&self, other: &Self) -> Result<SquaredDistance, ()> {
        f64::from(*self).compute_squared_distance(&(*other).into())
    }
}

impl ToAnimatedZero for FontStretch {
    #[inline]
    fn to_animated_zero(&self) -> Result<Self, ()> { Err(()) }
}

/// We should treat font stretch as real number in order to interpolate this property.
/// https://drafts.csswg.org/css-fonts-3/#font-stretch-animation
impl From<FontStretch> for f64 {
    fn from(stretch: FontStretch) -> f64 {
        use self::FontStretch::*;
        match stretch {
            ultra_condensed => 1.0,
            extra_condensed => 2.0,
            condensed       => 3.0,
            semi_condensed  => 4.0,
            normal          => 5.0,
            semi_expanded   => 6.0,
            expanded        => 7.0,
            extra_expanded  => 8.0,
            ultra_expanded  => 9.0,
        }
    }
}

impl Into<FontStretch> for f64 {
    fn into(self) -> FontStretch {
        use properties::longhands::font_stretch::computed_value::T::*;
        let index = (self + 0.5).floor().min(9.0).max(1.0);
        static FONT_STRETCH_ENUM_MAP: [FontStretch; 9] =
            [ ultra_condensed, extra_condensed, condensed, semi_condensed, normal,
              semi_expanded, expanded, extra_expanded, ultra_expanded ];
        FONT_STRETCH_ENUM_MAP[(index - 1.0) as usize]
    }
}

/// https://drafts.csswg.org/css-fonts-4/#font-variation-settings-def
#[cfg(feature = "gecko")]
impl Animate for FontVariationSettings {
    #[inline]
    fn animate(&self, other: &Self, procedure: Procedure) -> Result<Self, ()> {
        FontSettingTagIter::new(self, other)?
            .map(|r| r.and_then(|(st, ot)| st.animate(&ot, procedure)))
            .collect::<Result<Vec<FontSettingTag>, ()>>()
            .map(GenericFontSettings::Tag)
    }
}

#[cfg(feature = "gecko")]
impl ComputeSquaredDistance for FontVariationSettings {
    #[inline]
    fn compute_squared_distance(&self, other: &Self) -> Result<SquaredDistance, ()> {
        FontSettingTagIter::new(self, other)?
            .map(|r| r.and_then(|(st, ot)| st.compute_squared_distance(&ot)))
            .sum()
    }
}

#[cfg(feature = "gecko")]
impl ToAnimatedZero for FontVariationSettings {
    #[inline]
    fn to_animated_zero(&self) -> Result<Self, ()> {
        Err(())
    }
}

#[cfg(feature = "gecko")]
impl Animate for FontSettingTag {
    #[inline]
    fn animate(&self, other: &Self, procedure: Procedure) -> Result<Self, ()> {
        if self.tag != other.tag {
            return Err(());
        }
        let value = self.value.animate(&other.value, procedure)?;
        Ok(FontSettingTag {
            tag: self.tag,
            value,
        })
    }
}

#[cfg(feature = "gecko")]
impl ComputeSquaredDistance for FontSettingTag {
    #[inline]
    fn compute_squared_distance(&self, other: &Self) -> Result<SquaredDistance, ()> {
        if self.tag != other.tag {
            return Err(());
        }
        self.value.compute_squared_distance(&other.value)
    }
}

#[cfg(feature = "gecko")]
type FontSettingTag = GenericFontSettingTag<FontSettingTagFloat>;

#[cfg(feature = "gecko")]
struct FontSettingTagIterState<'a> {
    tags: Vec<(&'a FontSettingTag)>,
    index: usize,
    prev_tag: u32,
}

#[cfg(feature = "gecko")]
impl<'a> FontSettingTagIterState<'a> {
    fn new(tags: Vec<(&'a FontSettingTag)>) -> FontSettingTagIterState<'a> {
        FontSettingTagIterState {
            index: tags.len(),
            tags,
            prev_tag: 0,
        }
    }
}

/// Iterator for font-variation-settings tag lists
///
/// [CSS fonts level 4](https://drafts.csswg.org/css-fonts-4/#descdef-font-face-font-variation-settings)
/// defines the animation of font-variation-settings as follows:
///
///   Two declarations of font-feature-settings[sic] can be animated between if they are "like".
///   "Like" declarations are ones where the same set of properties appear (in any order).
///   Because succesive[sic] duplicate properties are applied instead of prior duplicate
///   properties, two declarations can be "like" even if they have differing number of
///   properties. If two declarations are "like" then animation occurs pairwise between
///   corresponding values in the declarations.
///
/// In other words if we have the following lists:
///
///   "wght" 1.4, "wdth" 5, "wght" 2
///   "wdth" 8, "wght" 4, "wdth" 10
///
/// We should animate between:
///
///   "wdth" 5, "wght" 2
///   "wght" 4, "wdth" 10
///
/// This iterator supports this by sorting the two lists, then iterating them in reverse,
/// and skipping entries with repeated tag names. It will return Some(Err()) if it reaches the
/// end of one list before the other, or if the tag names do not match.
///
/// For the above example, this iterator would return:
///
///   Some(Ok("wght" 2, "wght" 4))
///   Some(Ok("wdth" 5, "wdth" 10))
///   None
///
#[cfg(feature = "gecko")]
struct FontSettingTagIter<'a> {
    a_state: FontSettingTagIterState<'a>,
    b_state: FontSettingTagIterState<'a>,
}

#[cfg(feature = "gecko")]
impl<'a> FontSettingTagIter<'a> {
    fn new(
        a_settings: &'a FontVariationSettings,
        b_settings: &'a FontVariationSettings,
    ) -> Result<FontSettingTagIter<'a>, ()> {
        if let (&GenericFontSettings::Tag(ref a_tags), &GenericFontSettings::Tag(ref b_tags)) = (a_settings, b_settings)
        {
            fn as_new_sorted_tags(tags: &Vec<FontSettingTag>) -> Vec<(&FontSettingTag)> {
                use std::iter::FromIterator;
                let mut sorted_tags: Vec<(&FontSettingTag)> = Vec::from_iter(tags.iter());
                sorted_tags.sort_by_key(|k| k.tag);
                sorted_tags
            };

            Ok(FontSettingTagIter {
                a_state: FontSettingTagIterState::new(as_new_sorted_tags(a_tags)),
                b_state: FontSettingTagIterState::new(as_new_sorted_tags(b_tags)),
            })
        } else {
            Err(())
        }
    }

    fn next_tag(state: &mut FontSettingTagIterState<'a>) -> Option<(&'a FontSettingTag)> {
        if state.index == 0 {
            return None;
        }

        state.index -= 1;
        let tag = state.tags[state.index];
        if tag.tag == state.prev_tag {
            FontSettingTagIter::next_tag(state)
        } else {
            state.prev_tag = tag.tag;
            Some(tag)
        }
    }
}

#[cfg(feature = "gecko")]
impl<'a> Iterator for FontSettingTagIter<'a> {
    type Item = Result<(&'a FontSettingTag, &'a FontSettingTag), ()>;

    fn next(&mut self) -> Option<Result<(&'a FontSettingTag, &'a FontSettingTag), ()>> {
        match (
            FontSettingTagIter::next_tag(&mut self.a_state),
            FontSettingTagIter::next_tag(&mut self.b_state),
        ) {
            (Some(at), Some(bt)) if at.tag == bt.tag => Some(Ok((at, bt))),
            (None, None) => None,
            _ => Some(Err(())), // Mismatch number of unique tags or tag names.
        }
    }
}

impl<H, V> RepeatableListAnimatable for generic_position::Position<H, V>
    where H: RepeatableListAnimatable, V: RepeatableListAnimatable {}

/// https://drafts.csswg.org/css-transitions/#animtype-rect
impl Animate for ClipRect {
    #[inline]
    fn animate(&self, other: &Self, procedure: Procedure) -> Result<Self, ()> {
        use values::computed::Length;
        let animate_component = |this: &Option<Length>, other: &Option<Length>| {
            match (this.animate(other, procedure)?, procedure) {
                (None, Procedure::Interpolate { .. }) => Ok(None),
                (None, _) => Err(()),
                (result, _) => Ok(result),
            }
        };

        Ok(ClipRect {
            top:    animate_component(&self.top, &other.top)?,
            right:  animate_component(&self.right, &other.right)?,
            bottom: animate_component(&self.bottom, &other.bottom)?,
            left:   animate_component(&self.left, &other.left)?,
        })
    }
}

impl ToAnimatedZero for ClipRect {
    #[inline]
    fn to_animated_zero(&self) -> Result<Self, ()> { Err(()) }
}

/// Build an equivalent 'identity transform function list' based
/// on an existing transform list.
/// http://dev.w3.org/csswg/css-transforms/#none-transform-animation
impl ToAnimatedZero for TransformOperation {
    fn to_animated_zero(&self) -> Result<Self, ()> {
        match *self {
            TransformOperation::Matrix(..) => {
                Ok(TransformOperation::Matrix(ComputedMatrix::identity()))
            },
            TransformOperation::MatrixWithPercents(..) => {
                // FIXME(nox): Should be MatrixWithPercents value.
                Ok(TransformOperation::Matrix(ComputedMatrix::identity()))
            },
            TransformOperation::Skew(sx, sy) => {
                Ok(TransformOperation::Skew(
                    sx.to_animated_zero()?,
                    sy.to_animated_zero()?,
                ))
            },
            TransformOperation::Translate(ref tx, ref ty, ref tz) => {
                Ok(TransformOperation::Translate(
                    tx.to_animated_zero()?,
                    ty.to_animated_zero()?,
                    tz.to_animated_zero()?,
                ))
            },
            TransformOperation::Scale(..) => {
                Ok(TransformOperation::Scale(1.0, 1.0, 1.0))
            },
            TransformOperation::Rotate(x, y, z, a) => {
                let (x, y, z, _) = TransformList::get_normalized_vector_and_angle(x, y, z, a);
                Ok(TransformOperation::Rotate(x, y, z, Angle::zero()))
            },
            TransformOperation::Perspective(..) |
            TransformOperation::AccumulateMatrix { .. } |
            TransformOperation::InterpolateMatrix { .. } => {
                // Perspective: We convert a perspective function into an equivalent
                //     ComputedMatrix, and then decompose/interpolate/recompose these matrices.
                // AccumulateMatrix/InterpolateMatrix: We do interpolation on
                //     AccumulateMatrix/InterpolateMatrix by reading it as a ComputedMatrix
                //     (with layout information), and then do matrix interpolation.
                //
                // Therefore, we use an identity matrix to represent the identity transform list.
                // http://dev.w3.org/csswg/css-transforms/#identity-transform-function
                //
                // FIXME(nox): This does not actually work, given the impl of
                // Animate for TransformOperation bails out if the two given
                // values are dissimilar.
                Ok(TransformOperation::Matrix(ComputedMatrix::identity()))
            },
        }
    }
}

fn animate_multiplicative_factor(
    this: CSSFloat,
    other: CSSFloat,
    procedure: Procedure,
) -> Result<CSSFloat, ()> {
    Ok((this - 1.).animate(&(other - 1.), procedure)? + 1.)
}

/// http://dev.w3.org/csswg/css-transforms/#interpolation-of-transforms
impl Animate for TransformOperation {
    fn animate(&self, other: &Self, procedure: Procedure) -> Result<Self, ()> {
        match (self, other) {
            (
                &TransformOperation::Matrix(ref this),
                &TransformOperation::Matrix(ref other),
            ) => {
                Ok(TransformOperation::Matrix(
                    this.animate(other, procedure)?,
                ))
            },
            (
                &TransformOperation::Skew(ref fx, ref fy),
                &TransformOperation::Skew(ref tx, ref ty),
            ) => {
                Ok(TransformOperation::Skew(
                    fx.animate(tx, procedure)?,
                    fy.animate(ty, procedure)?,
                ))
            },
            (
                &TransformOperation::Translate(ref fx, ref fy, ref fz),
                &TransformOperation::Translate(ref tx, ref ty, ref tz),
            ) => {
                Ok(TransformOperation::Translate(
                    fx.animate(tx, procedure)?,
                    fy.animate(ty, procedure)?,
                    fz.animate(tz, procedure)?,
                ))
            },
            (
                &TransformOperation::Scale(ref fx, ref fy, ref fz),
                &TransformOperation::Scale(ref tx, ref ty, ref tz),
            ) => {
                Ok(TransformOperation::Scale(
                    animate_multiplicative_factor(*fx, *tx, procedure)?,
                    animate_multiplicative_factor(*fy, *ty, procedure)?,
                    animate_multiplicative_factor(*fz, *tz, procedure)?,
                ))
            },
            (
                &TransformOperation::Rotate(fx, fy, fz, fa),
                &TransformOperation::Rotate(tx, ty, tz, ta),
            ) => {
                let (fx, fy, fz, fa) =
                    TransformList::get_normalized_vector_and_angle(fx, fy, fz, fa);
                let (tx, ty, tz, ta) =
                    TransformList::get_normalized_vector_and_angle(tx, ty, tz, ta);
                if (fx, fy, fz) == (tx, ty, tz) {
                    let ia = fa.animate(&ta, procedure)?;
                    Ok(TransformOperation::Rotate(fx, fy, fz, ia))
                } else {
                    let matrix_f = rotate_to_matrix(fx, fy, fz, fa);
                    let matrix_t = rotate_to_matrix(tx, ty, tz, ta);
                    Ok(TransformOperation::Matrix(
                        matrix_f.animate(&matrix_t, procedure)?,
                    ))
                }
            },
            (
                &TransformOperation::Perspective(ref fd),
                &TransformOperation::Perspective(ref td),
            ) => {
                let mut fd_matrix = ComputedMatrix::identity();
                let mut td_matrix = ComputedMatrix::identity();
                if fd.px() > 0. {
                    fd_matrix.m34 = -1. / fd.px();
                }
                if td.px() > 0. {
                    td_matrix.m34 = -1. / td.px();
                }
                Ok(TransformOperation::Matrix(
                    fd_matrix.animate(&td_matrix, procedure)?,
                ))
            },
            _ => Err(()),
        }
    }
}

/// https://www.w3.org/TR/css-transforms-1/#Rotate3dDefined
fn rotate_to_matrix(x: f32, y: f32, z: f32, a: Angle) -> ComputedMatrix {
    let half_rad = a.radians() / 2.0;
    let sc = (half_rad).sin() * (half_rad).cos();
    let sq = (half_rad).sin().powi(2);

    ComputedMatrix {
        m11: 1.0 - 2.0 * (y * y + z * z) * sq,
        m12: 2.0 * (x * y * sq + z * sc),
        m13: 2.0 * (x * z * sq - y * sc),
        m14: 0.0,

        m21: 2.0 * (x * y * sq - z * sc),
        m22: 1.0 - 2.0 * (x * x + z * z) * sq,
        m23: 2.0 * (y * z * sq + x * sc),
        m24: 0.0,

        m31: 2.0 * (x * z * sq + y * sc),
        m32: 2.0 * (y * z * sq - x * sc),
        m33: 1.0 - 2.0 * (x * x + y * y) * sq,
        m34: 0.0,

        m41: 0.0,
        m42: 0.0,
        m43: 0.0,
        m44: 1.0
    }
}

/// A 2d matrix for interpolation.
#[derive(Clone, ComputeSquaredDistance, Copy, Debug)]
#[cfg_attr(feature = "servo", derive(HeapSizeOf))]
#[allow(missing_docs)]
// FIXME: We use custom derive for ComputeSquaredDistance. However, If possible, we should convert
// the InnerMatrix2D into types with physical meaning. This custom derive computes the squared
// distance from each matrix item, and this makes the result different from that in Gecko if we
// have skew factor in the ComputedMatrix.
pub struct InnerMatrix2D {
    pub m11: CSSFloat, pub m12: CSSFloat,
    pub m21: CSSFloat, pub m22: CSSFloat,
}

/// A 2d translation function.
#[cfg_attr(feature = "servo", derive(HeapSizeOf))]
#[derive(Animate, Clone, ComputeSquaredDistance, Copy, Debug)]
pub struct Translate2D(f32, f32);

/// A 2d scale function.
#[derive(Clone, ComputeSquaredDistance, Copy, Debug)]
#[cfg_attr(feature = "servo", derive(HeapSizeOf))]
pub struct Scale2D(f32, f32);

/// A decomposed 2d matrix.
#[derive(Clone, Copy, Debug)]
#[cfg_attr(feature = "servo", derive(HeapSizeOf))]
pub struct MatrixDecomposed2D {
    /// The translation function.
    pub translate: Translate2D,
    /// The scale function.
    pub scale: Scale2D,
    /// The rotation angle.
    pub angle: f32,
    /// The inner matrix.
    pub matrix: InnerMatrix2D,
}

impl Animate for InnerMatrix2D {
    fn animate(&self, other: &Self, procedure: Procedure) -> Result<Self, ()> {
        Ok(InnerMatrix2D {
            m11: animate_multiplicative_factor(self.m11, other.m11, procedure)?,
            m12: self.m12.animate(&other.m12, procedure)?,
            m21: self.m21.animate(&other.m21, procedure)?,
            m22: animate_multiplicative_factor(self.m22, other.m22, procedure)?,
        })
    }
}

impl Animate for Scale2D {
    fn animate(&self, other: &Self, procedure: Procedure) -> Result<Self, ()> {
        Ok(Scale2D(
            animate_multiplicative_factor(self.0, other.0, procedure)?,
            animate_multiplicative_factor(self.1, other.1, procedure)?,
        ))
    }
}

impl Animate for MatrixDecomposed2D {
    /// https://drafts.csswg.org/css-transforms/#interpolation-of-decomposed-2d-matrix-values
    fn animate(&self, other: &Self, procedure: Procedure) -> Result<Self, ()> {
        // If x-axis of one is flipped, and y-axis of the other,
        // convert to an unflipped rotation.
        let mut scale = self.scale;
        let mut angle = self.angle;
        let mut other_angle = other.angle;
        if (scale.0 < 0.0 && other.scale.1 < 0.0) || (scale.1 < 0.0 && other.scale.0 < 0.0) {
            scale.0 = -scale.0;
            scale.1 = -scale.1;
            angle += if angle < 0.0 {180.} else {-180.};
        }

        // Don't rotate the long way around.
        if angle == 0.0 {
            angle = 360.
        }
        if other_angle == 0.0 {
            other_angle = 360.
        }

        if (angle - other_angle).abs() > 180. {
            if angle > other_angle {
                angle -= 360.
            }
            else{
                other_angle -= 360.
            }
        }

        // Interpolate all values.
        let translate = self.translate.animate(&other.translate, procedure)?;
        let scale = scale.animate(&other.scale, procedure)?;
        let angle = angle.animate(&other_angle, procedure)?;
        let matrix = self.matrix.animate(&other.matrix, procedure)?;

        Ok(MatrixDecomposed2D {
            translate: translate,
            scale: scale,
            angle: angle,
            matrix: matrix,
        })
    }
}

impl ComputeSquaredDistance for MatrixDecomposed2D {
    #[inline]
    fn compute_squared_distance(&self, other: &Self) -> Result<SquaredDistance, ()> {
        // Use Radian to compute the distance.
        const RAD_PER_DEG: f64 = ::std::f64::consts::PI / 180.0;
        let angle1 = self.angle as f64 * RAD_PER_DEG;
        let angle2 = other.angle as f64 * RAD_PER_DEG;
        Ok(self.translate.compute_squared_distance(&other.translate)? +
           self.scale.compute_squared_distance(&other.scale)? +
           angle1.compute_squared_distance(&angle2)? +
           self.matrix.compute_squared_distance(&other.matrix)?)
    }
}

impl Animate for ComputedMatrix {
    #[cfg(feature = "servo")]
    fn animate(&self, other: &Self, procedure: Procedure) -> Result<Self, ()> {
        if self.is_3d() || other.is_3d() {
            let decomposed_from = decompose_3d_matrix(*self);
            let decomposed_to = decompose_3d_matrix(*other);
            match (decomposed_from, decomposed_to) {
                (Ok(this), Ok(other)) => {
                    Ok(ComputedMatrix::from(this.animate(&other, procedure)?))
                },
                // Matrices can be undecomposable due to couple reasons, e.g.,
                // non-invertible matrices. In this case, we should report Err
                // here, and let the caller do the fallback procedure.
                _ => Err(())
            }
        } else {
            let this = MatrixDecomposed2D::from(*self);
            let other = MatrixDecomposed2D::from(*other);
            Ok(ComputedMatrix::from(this.animate(&other, procedure)?))
        }
    }

    #[cfg(feature = "gecko")]
    fn animate(&self, other: &Self, procedure: Procedure) -> Result<Self, ()> {
        let (from, to) = if self.is_3d() || other.is_3d() {
            (decompose_3d_matrix(*self), decompose_3d_matrix(*other))
        } else {
            (decompose_2d_matrix(self), decompose_2d_matrix(other))
        };
        match (from, to) {
            (Ok(from), Ok(to)) => {
                Ok(ComputedMatrix::from(from.animate(&to, procedure)?))
            },
            // Matrices can be undecomposable due to couple reasons, e.g.,
            // non-invertible matrices. In this case, we should report Err here,
            // and let the caller do the fallback procedure.
            _ => Err(())
        }
    }
}

impl ComputeSquaredDistance for ComputedMatrix {
    #[inline]
    #[cfg(feature = "servo")]
    fn compute_squared_distance(&self, other: &Self) -> Result<SquaredDistance, ()> {
        if self.is_3d() || other.is_3d() {
            let from = decompose_3d_matrix(*self)?;
            let to = decompose_3d_matrix(*other)?;
            from.compute_squared_distance(&to)
        } else {
            let from = MatrixDecomposed2D::from(*self);
            let to = MatrixDecomposed2D::from(*other);
            from.compute_squared_distance(&to)
        }
    }

    #[inline]
    #[cfg(feature = "gecko")]
    fn compute_squared_distance(&self, other: &Self) -> Result<SquaredDistance, ()> {
        let (from, to) = if self.is_3d() || other.is_3d() {
            (decompose_3d_matrix(*self)?, decompose_3d_matrix(*other)?)
        } else {
            (decompose_2d_matrix(self)?, decompose_2d_matrix(other)?)
        };
        from.compute_squared_distance(&to)
    }
}

impl From<ComputedMatrix> for MatrixDecomposed2D {
    /// Decompose a 2D matrix.
    /// https://drafts.csswg.org/css-transforms/#decomposing-a-2d-matrix
    fn from(matrix: ComputedMatrix) -> MatrixDecomposed2D {
        let mut row0x = matrix.m11;
        let mut row0y = matrix.m12;
        let mut row1x = matrix.m21;
        let mut row1y = matrix.m22;

        let translate = Translate2D(matrix.m41, matrix.m42);
        let mut scale = Scale2D((row0x * row0x + row0y * row0y).sqrt(),
                                (row1x * row1x + row1y * row1y).sqrt());

        // If determinant is negative, one axis was flipped.
        let determinant = row0x * row1y - row0y * row1x;
        if determinant < 0. {
            if row0x < row1y {
                scale.0 = -scale.0;
            } else {
                scale.1 = -scale.1;
            }
        }

        // Renormalize matrix to remove scale.
        if scale.0 != 0.0 {
            row0x *= 1. / scale.0;
            row0y *= 1. / scale.0;
        }
        if scale.1 != 0.0 {
            row1x *= 1. / scale.1;
            row1y *= 1. / scale.1;
        }

        // Compute rotation and renormalize matrix.
        let mut angle = row0y.atan2(row0x);
        if angle != 0.0 {
            let sn = -row0y;
            let cs = row0x;
            let m11 = row0x;
            let m12 = row0y;
            let m21 = row1x;
            let m22 = row1y;
            row0x = cs * m11 + sn * m21;
            row0y = cs * m12 + sn * m22;
            row1x = -sn * m11 + cs * m21;
            row1y = -sn * m12 + cs * m22;
        }

        let m = InnerMatrix2D {
            m11: row0x, m12: row0y,
            m21: row1x, m22: row1y,
        };

        // Convert into degrees because our rotation functions expect it.
        angle = angle.to_degrees();
        MatrixDecomposed2D {
            translate: translate,
            scale: scale,
            angle: angle,
            matrix: m,
        }
    }
}

impl From<MatrixDecomposed2D> for ComputedMatrix {
    /// Recompose a 2D matrix.
    /// https://drafts.csswg.org/css-transforms/#recomposing-to-a-2d-matrix
    fn from(decomposed: MatrixDecomposed2D) -> ComputedMatrix {
        let mut computed_matrix = ComputedMatrix::identity();
        computed_matrix.m11 = decomposed.matrix.m11;
        computed_matrix.m12 = decomposed.matrix.m12;
        computed_matrix.m21 = decomposed.matrix.m21;
        computed_matrix.m22 = decomposed.matrix.m22;

        // Translate matrix.
        computed_matrix.m41 = decomposed.translate.0;
        computed_matrix.m42 = decomposed.translate.1;

        // Rotate matrix.
        let angle = decomposed.angle.to_radians();
        let cos_angle = angle.cos();
        let sin_angle = angle.sin();

        let mut rotate_matrix = ComputedMatrix::identity();
        rotate_matrix.m11 = cos_angle;
        rotate_matrix.m12 = sin_angle;
        rotate_matrix.m21 = -sin_angle;
        rotate_matrix.m22 = cos_angle;

        // Multiplication of computed_matrix and rotate_matrix
        computed_matrix = multiply(rotate_matrix, computed_matrix);

        // Scale matrix.
        computed_matrix.m11 *= decomposed.scale.0;
        computed_matrix.m12 *= decomposed.scale.0;
        computed_matrix.m21 *= decomposed.scale.1;
        computed_matrix.m22 *= decomposed.scale.1;
        computed_matrix
    }
}

#[cfg(feature = "gecko")]
impl<'a> From< &'a RawGeckoGfxMatrix4x4> for ComputedMatrix {
    fn from(m: &'a RawGeckoGfxMatrix4x4) -> ComputedMatrix {
        ComputedMatrix {
            m11: m[0],  m12: m[1],  m13: m[2],  m14: m[3],
            m21: m[4],  m22: m[5],  m23: m[6],  m24: m[7],
            m31: m[8],  m32: m[9],  m33: m[10], m34: m[11],
            m41: m[12], m42: m[13], m43: m[14], m44: m[15],
        }
    }
}

#[cfg(feature = "gecko")]
impl From<ComputedMatrix> for RawGeckoGfxMatrix4x4 {
    fn from(matrix: ComputedMatrix) -> RawGeckoGfxMatrix4x4 {
        [ matrix.m11, matrix.m12, matrix.m13, matrix.m14,
          matrix.m21, matrix.m22, matrix.m23, matrix.m24,
          matrix.m31, matrix.m32, matrix.m33, matrix.m34,
          matrix.m41, matrix.m42, matrix.m43, matrix.m44 ]
    }
}

/// A 3d translation.
#[cfg_attr(feature = "servo", derive(HeapSizeOf))]
#[derive(Animate, Clone, ComputeSquaredDistance, Copy, Debug)]
pub struct Translate3D(f32, f32, f32);

/// A 3d scale function.
#[derive(Clone, ComputeSquaredDistance, Copy, Debug)]
#[cfg_attr(feature = "servo", derive(HeapSizeOf))]
pub struct Scale3D(f32, f32, f32);

/// A 3d skew function.
#[cfg_attr(feature = "servo", derive(HeapSizeOf))]
#[derive(Animate, Clone, Copy, Debug)]
pub struct Skew(f32, f32, f32);

/// A 3d perspective transformation.
#[derive(Clone, ComputeSquaredDistance, Copy, Debug)]
#[cfg_attr(feature = "servo", derive(HeapSizeOf))]
pub struct Perspective(f32, f32, f32, f32);

/// A quaternion used to represent a rotation.
#[derive(Clone, Copy, Debug)]
#[cfg_attr(feature = "servo", derive(HeapSizeOf))]
pub struct Quaternion(f64, f64, f64, f64);

/// A decomposed 3d matrix.
#[derive(Clone, ComputeSquaredDistance, Copy, Debug)]
#[cfg_attr(feature = "servo", derive(HeapSizeOf))]
pub struct MatrixDecomposed3D {
    /// A translation function.
    pub translate: Translate3D,
    /// A scale function.
    pub scale: Scale3D,
    /// The skew component of the transformation.
    pub skew: Skew,
    /// The perspective component of the transformation.
    pub perspective: Perspective,
    /// The quaternion used to represent the rotation.
    pub quaternion: Quaternion,
}

impl Quaternion {
    /// Return a quaternion from a unit direction vector and angle (unit: radian).
    #[inline]
    fn from_direction_and_angle(vector: &DirectionVector, angle: f64) -> Self {
        debug_assert!((vector.length() - 1.).abs() < 0.0001,
                      "Only accept an unit direction vector to create a quaternion");
        // Reference:
        // https://en.wikipedia.org/wiki/Quaternions_and_spatial_rotation
        //
        // if the direction axis is (x, y, z) = xi + yj + zk,
        // and the angle is |theta|, this formula can be done using
        // an extension of Euler's formula:
        //   q = cos(theta/2) + (xi + yj + zk)(sin(theta/2))
        //     = cos(theta/2) +
        //       x*sin(theta/2)i + y*sin(theta/2)j + z*sin(theta/2)k
        Quaternion(vector.x as f64 * (angle / 2.).sin(),
                   vector.y as f64 * (angle / 2.).sin(),
                   vector.z as f64 * (angle / 2.).sin(),
                   (angle / 2.).cos())
    }

    /// Calculate the dot product.
    #[inline]
    fn dot(&self, other: &Self) -> f64 {
        self.0 * other.0 + self.1 * other.1 + self.2 * other.2 + self.3 * other.3
    }
}

impl ComputeSquaredDistance for Quaternion {
    #[inline]
    fn compute_squared_distance(&self, other: &Self) -> Result<SquaredDistance, ()> {
        // Use quaternion vectors to get the angle difference. Both q1 and q2 are unit vectors,
        // so we can get their angle difference by:
        // cos(theta/2) = (q1 dot q2) / (|q1| * |q2|) = q1 dot q2.
        let distance = self.dot(other).max(-1.0).min(1.0).acos() * 2.0;
        Ok(SquaredDistance::Value(distance * distance))
    }
}

/// Decompose a 3D matrix.
/// https://drafts.csswg.org/css-transforms/#decomposing-a-3d-matrix
fn decompose_3d_matrix(mut matrix: ComputedMatrix) -> Result<MatrixDecomposed3D, ()> {
    // Normalize the matrix.
    if matrix.m44 == 0.0 {
        return Err(());
    }

    let scaling_factor = matrix.m44;
            matrix.m11 /= scaling_factor;
            matrix.m12 /= scaling_factor;
            matrix.m13 /= scaling_factor;
            matrix.m14 /= scaling_factor;
            matrix.m21 /= scaling_factor;
            matrix.m22 /= scaling_factor;
            matrix.m23 /= scaling_factor;
            matrix.m24 /= scaling_factor;
            matrix.m31 /= scaling_factor;
            matrix.m32 /= scaling_factor;
            matrix.m33 /= scaling_factor;
            matrix.m34 /= scaling_factor;
            matrix.m41 /= scaling_factor;
            matrix.m42 /= scaling_factor;
            matrix.m43 /= scaling_factor;
            matrix.m44 /= scaling_factor;

    // perspective_matrix is used to solve for perspective, but it also provides
    // an easy way to test for singularity of the upper 3x3 component.
    let mut perspective_matrix = matrix;

        perspective_matrix.m14 = 0.0;
        perspective_matrix.m24 = 0.0;
        perspective_matrix.m34 = 0.0;
    perspective_matrix.m44 = 1.0;

    if perspective_matrix.determinant() == 0.0 {
        return Err(());
    }

    // First, isolate perspective.
    let perspective = if matrix.m14 != 0.0 || matrix.m24 != 0.0 || matrix.m34 != 0.0 {
        let right_hand_side: [f32; 4] = [
            matrix.m14,
            matrix.m24,
            matrix.m34,
            matrix.m44
        ];

        perspective_matrix = perspective_matrix.inverse().unwrap();

        // Transpose perspective_matrix
        perspective_matrix = ComputedMatrix {
                    m11: perspective_matrix.m11,
                    m12: perspective_matrix.m21,
                    m13: perspective_matrix.m31,
                    m14: perspective_matrix.m41,
                    m21: perspective_matrix.m12,
                    m22: perspective_matrix.m22,
                    m23: perspective_matrix.m32,
                    m24: perspective_matrix.m42,
                    m31: perspective_matrix.m13,
                    m32: perspective_matrix.m23,
                    m33: perspective_matrix.m33,
                    m34: perspective_matrix.m43,
                    m41: perspective_matrix.m14,
                    m42: perspective_matrix.m24,
                    m43: perspective_matrix.m34,
                    m44: perspective_matrix.m44,
        };

        // Multiply right_hand_side with perspective_matrix
        let mut tmp: [f32; 4] = [0.0; 4];
            tmp[0] = (right_hand_side[0] * perspective_matrix.m11) +
                            (right_hand_side[1] * perspective_matrix.m21) +
                            (right_hand_side[2] * perspective_matrix.m31) +
                            (right_hand_side[3] * perspective_matrix.m41);
            tmp[1] = (right_hand_side[0] * perspective_matrix.m12) +
                            (right_hand_side[1] * perspective_matrix.m22) +
                            (right_hand_side[2] * perspective_matrix.m32) +
                            (right_hand_side[3] * perspective_matrix.m42);
            tmp[2] = (right_hand_side[0] * perspective_matrix.m13) +
                            (right_hand_side[1] * perspective_matrix.m23) +
                            (right_hand_side[2] * perspective_matrix.m33) +
                            (right_hand_side[3] * perspective_matrix.m43);
            tmp[3] = (right_hand_side[0] * perspective_matrix.m14) +
                            (right_hand_side[1] * perspective_matrix.m24) +
                            (right_hand_side[2] * perspective_matrix.m34) +
                            (right_hand_side[3] * perspective_matrix.m44);

        Perspective(tmp[0], tmp[1], tmp[2], tmp[3])
    } else {
        Perspective(0.0, 0.0, 0.0, 1.0)
    };

    // Next take care of translation
    let translate = Translate3D (
        matrix.m41,
        matrix.m42,
        matrix.m43
    );

    // Now get scale and shear. 'row' is a 3 element array of 3 component vectors
    let mut row: [[f32; 3]; 3] = [[0.0; 3]; 3];
        row[0][0] = matrix.m11;
        row[0][1] = matrix.m12;
        row[0][2] = matrix.m13;
        row[1][0] = matrix.m21;
        row[1][1] = matrix.m22;
        row[1][2] = matrix.m23;
        row[2][0] = matrix.m31;
        row[2][1] = matrix.m32;
        row[2][2] = matrix.m33;

    // Compute X scale factor and normalize first row.
    let row0len = (row[0][0] * row[0][0] + row[0][1] * row[0][1] + row[0][2] * row[0][2]).sqrt();
    let mut scale = Scale3D(row0len, 0.0, 0.0);
    row[0] = [row[0][0] / row0len, row[0][1] / row0len, row[0][2] / row0len];

    // Compute XY shear factor and make 2nd row orthogonal to 1st.
    let mut skew = Skew(dot(row[0], row[1]), 0.0, 0.0);
    row[1] = combine(row[1], row[0], 1.0, -skew.0);

    // Now, compute Y scale and normalize 2nd row.
    let row1len = (row[1][0] * row[1][0] + row[1][1] * row[1][1] + row[1][2] * row[1][2]).sqrt();
    scale.1 = row1len;
    row[1] = [row[1][0] / row1len, row[1][1] / row1len, row[1][2] / row1len];
    skew.0 /= scale.1;

    // Compute XZ and YZ shears, orthogonalize 3rd row
    skew.1 = dot(row[0], row[2]);
    row[2] = combine(row[2], row[0], 1.0, -skew.1);
    skew.2 = dot(row[1], row[2]);
    row[2] = combine(row[2], row[1], 1.0, -skew.2);

    // Next, get Z scale and normalize 3rd row.
    let row2len = (row[2][0] * row[2][0] + row[2][1] * row[2][1] + row[2][2] * row[2][2]).sqrt();
    scale.2 = row2len;
    row[2] = [row[2][0] / row2len, row[2][1] / row2len, row[2][2] / row2len];
    skew.1 /= scale.2;
    skew.2 /= scale.2;

    // At this point, the matrix (in rows) is orthonormal.
    // Check for a coordinate system flip.  If the determinant
    // is -1, then negate the matrix and the scaling factors.
    let pdum3 = cross(row[1], row[2]);
    if dot(row[0], pdum3) < 0.0 {
            scale.0 *= -1.0;
            row[0][0] *= -1.0;
            row[0][1] *= -1.0;
            row[0][2] *= -1.0;
            scale.1 *= -1.0;
            row[1][0] *= -1.0;
            row[1][1] *= -1.0;
            row[1][2] *= -1.0;
            scale.2 *= -1.0;
            row[2][0] *= -1.0;
            row[2][1] *= -1.0;
            row[2][2] *= -1.0;
    }

    // Now, get the rotations out
    let mut quaternion = Quaternion (
        0.5 * ((1.0 + row[0][0] - row[1][1] - row[2][2]).max(0.0) as f64).sqrt(),
        0.5 * ((1.0 - row[0][0] + row[1][1] - row[2][2]).max(0.0) as f64).sqrt(),
        0.5 * ((1.0 - row[0][0] - row[1][1] + row[2][2]).max(0.0) as f64).sqrt(),
        0.5 * ((1.0 + row[0][0] + row[1][1] + row[2][2]).max(0.0) as f64).sqrt()
    );

    if row[2][1] > row[1][2] {
        quaternion.0 = -quaternion.0
    }
    if row[0][2] > row[2][0] {
        quaternion.1 = -quaternion.1
    }
    if row[1][0] > row[0][1] {
        quaternion.2 = -quaternion.2
    }

    Ok(MatrixDecomposed3D {
        translate: translate,
        scale: scale,
        skew: skew,
        perspective: perspective,
        quaternion: quaternion
    })
}

/// Decompose a 2D matrix for Gecko.
// Use the algorithm from nsStyleTransformMatrix::Decompose2DMatrix() in Gecko.
#[cfg(feature = "gecko")]
fn decompose_2d_matrix(matrix: &ComputedMatrix) -> Result<MatrixDecomposed3D, ()> {
    // The index is column-major, so the equivalent transform matrix is:
    // | m11 m21  0 m41 |  =>  | m11 m21 | and translate(m41, m42)
    // | m12 m22  0 m42 |      | m12 m22 |
    // |   0   0  1   0 |
    // |   0   0  0   1 |
    let (mut m11, mut m12) = (matrix.m11, matrix.m12);
    let (mut m21, mut m22) = (matrix.m21, matrix.m22);
    // Check if this is a singular matrix.
    if m11 * m22 == m12 * m21 {
        return Err(());
    }

    let mut scale_x = (m11 * m11 + m12 * m12).sqrt();
    m11 /= scale_x;
    m12 /= scale_x;

    let mut shear_xy = m11 * m21 + m12 * m22;
    m21 -= m11 * shear_xy;
    m22 -= m12 * shear_xy;

    let scale_y = (m21 * m21 + m22 * m22).sqrt();
    m21 /= scale_y;
    m22 /= scale_y;
    shear_xy /= scale_y;

    let determinant = m11 * m22 - m12 * m21;
    // Determinant should now be 1 or -1.
    if 0.99 > determinant.abs() || determinant.abs() > 1.01 {
        return Err(());
    }

    if determinant < 0. {
        m11 = -m11;
        m12 = -m12;
        shear_xy = -shear_xy;
        scale_x = -scale_x;
    }

    Ok(MatrixDecomposed3D {
        translate: Translate3D(matrix.m41, matrix.m42, 0.),
        scale: Scale3D(scale_x, scale_y, 1.),
        skew: Skew(shear_xy, 0., 0.),
        perspective: Perspective(0., 0., 0., 1.),
        quaternion: Quaternion::from_direction_and_angle(&DirectionVector::new(0., 0., 1.),
                                                         m12.atan2(m11) as f64)
    })
}

// Combine 2 point.
fn combine(a: [f32; 3], b: [f32; 3], ascl: f32, bscl: f32) -> [f32; 3] {
    [
        (ascl * a[0]) + (bscl * b[0]),
        (ascl * a[1]) + (bscl * b[1]),
        (ascl * a[2]) + (bscl * b[2])
    ]
}

// Dot product.
fn dot(a: [f32; 3], b: [f32; 3]) -> f32 {
    a[0] * b[0] + a[1] * b[1] + a[2] * b[2]
}

// Cross product.
fn cross(row1: [f32; 3], row2: [f32; 3]) -> [f32; 3] {
    [
        row1[1] * row2[2] - row1[2] * row2[1],
        row1[2] * row2[0] - row1[0] * row2[2],
        row1[0] * row2[1] - row1[1] * row2[0]
    ]
}

impl Animate for Scale3D {
    fn animate(&self, other: &Self, procedure: Procedure) -> Result<Self, ()> {
        Ok(Scale3D(
            animate_multiplicative_factor(self.0, other.0, procedure)?,
            animate_multiplicative_factor(self.1, other.1, procedure)?,
            animate_multiplicative_factor(self.2, other.2, procedure)?,
        ))
    }
}

impl ComputeSquaredDistance for Skew {
    // We have to use atan() to convert the skew factors into skew angles, so implement
    // ComputeSquaredDistance manually.
    #[inline]
    fn compute_squared_distance(&self, other: &Self) -> Result<SquaredDistance, ()> {
        Ok(self.0.atan().compute_squared_distance(&other.0.atan())? +
           self.1.atan().compute_squared_distance(&other.1.atan())? +
           self.2.atan().compute_squared_distance(&other.2.atan())?)
    }
}

impl Animate for Perspective {
    fn animate(&self, other: &Self, procedure: Procedure) -> Result<Self, ()> {
        Ok(Perspective(
            self.0.animate(&other.0, procedure)?,
            self.1.animate(&other.1, procedure)?,
            self.2.animate(&other.2, procedure)?,
            animate_multiplicative_factor(self.3, other.3, procedure)?,
        ))
    }
}

impl Animate for MatrixDecomposed3D {
    /// https://drafts.csswg.org/css-transforms/#interpolation-of-decomposed-3d-matrix-values
    fn animate(&self, other: &Self, procedure: Procedure) -> Result<Self, ()> {
        use std::f64;

        let (this_weight, other_weight) = procedure.weights();

        debug_assert!((this_weight + other_weight - 1.0f64).abs() <= f64::EPSILON ||
                      other_weight == 1.0f64 || other_weight == 0.0f64,
                      "animate should only be used for interpolating or accumulating transforms");

        let mut sum = *self;

        // Add translate, scale, skew and perspective components.
        sum.translate = self.translate.animate(&other.translate, procedure)?;
        sum.scale = self.scale.animate(&other.scale, procedure)?;
        sum.skew = self.skew.animate(&other.skew, procedure)?;
        sum.perspective = self.perspective.animate(&other.perspective, procedure)?;

        // Add quaternions using spherical linear interpolation (Slerp).
        //
        // We take a specialized code path for accumulation (where other_weight is 1)
        if other_weight == 1.0 {
            if this_weight == 0.0 {
                return Ok(*other)
            }

            let clamped_w = self.quaternion.3.min(1.0).max(-1.0);

            // Determine the scale factor.
            let mut theta = clamped_w.acos();
            let mut scale = if theta == 0.0 { 0.0 } else { 1.0 / theta.sin() };
            theta *= this_weight;
            scale *= theta.sin();

            // Scale the self matrix by this_weight.
            let mut scaled_self = *self;
                scaled_self.quaternion.0 *= scale;
                scaled_self.quaternion.1 *= scale;
                scaled_self.quaternion.2 *= scale;
            scaled_self.quaternion.3 = theta.cos();

            // Multiply scaled-self by other.
            let a = &scaled_self.quaternion;
            let b = &other.quaternion;
            sum.quaternion = Quaternion(
                a.3 * b.0 + a.0 * b.3 + a.1 * b.2 - a.2 * b.1,
                a.3 * b.1 - a.0 * b.2 + a.1 * b.3 + a.2 * b.0,
                a.3 * b.2 + a.0 * b.1 - a.1 * b.0 + a.2 * b.3,
                a.3 * b.3 - a.0 * b.0 - a.1 * b.1 - a.2 * b.2,
            );
        } else {
            let mut product = self.quaternion.0 * other.quaternion.0 +
                              self.quaternion.1 * other.quaternion.1 +
                              self.quaternion.2 * other.quaternion.2 +
                              self.quaternion.3 * other.quaternion.3;

            // Clamp product to -1.0 <= product <= 1.0
            product = product.min(1.0);
            product = product.max(-1.0);

            if product == 1.0 {
                return Ok(sum);
            }

            let theta = product.acos();
            let w = (other_weight * theta).sin() * 1.0 / (1.0 - product * product).sqrt();

            let mut a = *self;
            let mut b = *other;
                a.quaternion.0 *= (other_weight * theta).cos() - product * w;
                b.quaternion.0 *= w;
                sum.quaternion.0 = a.quaternion.0 + b.quaternion.0;
                a.quaternion.1 *= (other_weight * theta).cos() - product * w;
                b.quaternion.1 *= w;
                sum.quaternion.1 = a.quaternion.1 + b.quaternion.1;
                a.quaternion.2 *= (other_weight * theta).cos() - product * w;
                b.quaternion.2 *= w;
                sum.quaternion.2 = a.quaternion.2 + b.quaternion.2;
                a.quaternion.3 *= (other_weight * theta).cos() - product * w;
                b.quaternion.3 *= w;
                sum.quaternion.3 = a.quaternion.3 + b.quaternion.3;
        }

        Ok(sum)
    }
}

impl From<MatrixDecomposed3D> for ComputedMatrix {
    /// Recompose a 3D matrix.
    /// https://drafts.csswg.org/css-transforms/#recomposing-to-a-3d-matrix
    fn from(decomposed: MatrixDecomposed3D) -> ComputedMatrix {
        let mut matrix = ComputedMatrix::identity();

        // Apply perspective
            matrix.m14 = decomposed.perspective.0;
            matrix.m24 = decomposed.perspective.1;
            matrix.m34 = decomposed.perspective.2;
            matrix.m44 = decomposed.perspective.3;

        // Apply translation
                matrix.m41 += decomposed.translate.0 * matrix.m11;
                matrix.m41 += decomposed.translate.1 * matrix.m21;
                matrix.m41 += decomposed.translate.2 * matrix.m31;
                matrix.m42 += decomposed.translate.0 * matrix.m12;
                matrix.m42 += decomposed.translate.1 * matrix.m22;
                matrix.m42 += decomposed.translate.2 * matrix.m32;
                matrix.m43 += decomposed.translate.0 * matrix.m13;
                matrix.m43 += decomposed.translate.1 * matrix.m23;
                matrix.m43 += decomposed.translate.2 * matrix.m33;

        // Apply rotation
        let x = decomposed.quaternion.0;
        let y = decomposed.quaternion.1;
        let z = decomposed.quaternion.2;
        let w = decomposed.quaternion.3;

        // Construct a composite rotation matrix from the quaternion values
        // rotationMatrix is a identity 4x4 matrix initially
        let mut rotation_matrix = ComputedMatrix::identity();
        rotation_matrix.m11 = 1.0 - 2.0 * (y * y + z * z) as f32;
        rotation_matrix.m12 = 2.0 * (x * y + z * w) as f32;
        rotation_matrix.m13 = 2.0 * (x * z - y * w) as f32;
        rotation_matrix.m21 = 2.0 * (x * y - z * w) as f32;
        rotation_matrix.m22 = 1.0 - 2.0 * (x * x + z * z) as f32;
        rotation_matrix.m23 = 2.0 * (y * z + x * w) as f32;
        rotation_matrix.m31 = 2.0 * (x * z + y * w) as f32;
        rotation_matrix.m32 = 2.0 * (y * z - x * w) as f32;
        rotation_matrix.m33 = 1.0 - 2.0 * (x * x + y * y) as f32;

        matrix = multiply(rotation_matrix, matrix);

        // Apply skew
        let mut temp = ComputedMatrix::identity();
        if decomposed.skew.2 != 0.0 {
            temp.m32 = decomposed.skew.2;
            matrix = multiply(temp, matrix);
        }

        if decomposed.skew.1 != 0.0 {
            temp.m32 = 0.0;
            temp.m31 = decomposed.skew.1;
            matrix = multiply(temp, matrix);
        }

        if decomposed.skew.0 != 0.0 {
            temp.m31 = 0.0;
            temp.m21 = decomposed.skew.0;
            matrix = multiply(temp, matrix);
        }

        // Apply scale
                matrix.m11 *= decomposed.scale.0;
                matrix.m12 *= decomposed.scale.0;
                matrix.m13 *= decomposed.scale.0;
                matrix.m21 *= decomposed.scale.1;
                matrix.m22 *= decomposed.scale.1;
                matrix.m23 *= decomposed.scale.1;
                matrix.m31 *= decomposed.scale.2;
                matrix.m32 *= decomposed.scale.2;
                matrix.m33 *= decomposed.scale.2;

        matrix
    }
}

// Multiplication of two 4x4 matrices.
fn multiply(a: ComputedMatrix, b: ComputedMatrix) -> ComputedMatrix {
    let mut a_clone = a;
            a_clone.m11 = (a.m11 * b.m11) +
                               (a.m12 * b.m21) +
                               (a.m13 * b.m31) +
                               (a.m14 * b.m41);
            a_clone.m12 = (a.m11 * b.m12) +
                               (a.m12 * b.m22) +
                               (a.m13 * b.m32) +
                               (a.m14 * b.m42);
            a_clone.m13 = (a.m11 * b.m13) +
                               (a.m12 * b.m23) +
                               (a.m13 * b.m33) +
                               (a.m14 * b.m43);
            a_clone.m14 = (a.m11 * b.m14) +
                               (a.m12 * b.m24) +
                               (a.m13 * b.m34) +
                               (a.m14 * b.m44);
            a_clone.m21 = (a.m21 * b.m11) +
                               (a.m22 * b.m21) +
                               (a.m23 * b.m31) +
                               (a.m24 * b.m41);
            a_clone.m22 = (a.m21 * b.m12) +
                               (a.m22 * b.m22) +
                               (a.m23 * b.m32) +
                               (a.m24 * b.m42);
            a_clone.m23 = (a.m21 * b.m13) +
                               (a.m22 * b.m23) +
                               (a.m23 * b.m33) +
                               (a.m24 * b.m43);
            a_clone.m24 = (a.m21 * b.m14) +
                               (a.m22 * b.m24) +
                               (a.m23 * b.m34) +
                               (a.m24 * b.m44);
            a_clone.m31 = (a.m31 * b.m11) +
                               (a.m32 * b.m21) +
                               (a.m33 * b.m31) +
                               (a.m34 * b.m41);
            a_clone.m32 = (a.m31 * b.m12) +
                               (a.m32 * b.m22) +
                               (a.m33 * b.m32) +
                               (a.m34 * b.m42);
            a_clone.m33 = (a.m31 * b.m13) +
                               (a.m32 * b.m23) +
                               (a.m33 * b.m33) +
                               (a.m34 * b.m43);
            a_clone.m34 = (a.m31 * b.m14) +
                               (a.m32 * b.m24) +
                               (a.m33 * b.m34) +
                               (a.m34 * b.m44);
            a_clone.m41 = (a.m41 * b.m11) +
                               (a.m42 * b.m21) +
                               (a.m43 * b.m31) +
                               (a.m44 * b.m41);
            a_clone.m42 = (a.m41 * b.m12) +
                               (a.m42 * b.m22) +
                               (a.m43 * b.m32) +
                               (a.m44 * b.m42);
            a_clone.m43 = (a.m41 * b.m13) +
                               (a.m42 * b.m23) +
                               (a.m43 * b.m33) +
                               (a.m44 * b.m43);
            a_clone.m44 = (a.m41 * b.m14) +
                               (a.m42 * b.m24) +
                               (a.m43 * b.m34) +
                               (a.m44 * b.m44);
    a_clone
}

impl ComputedMatrix {
    fn is_3d(&self) -> bool {
        self.m13 != 0.0 || self.m14 != 0.0 ||
        self.m23 != 0.0 || self.m24 != 0.0 ||
        self.m31 != 0.0 || self.m32 != 0.0 || self.m33 != 1.0 || self.m34 != 0.0 ||
        self.m43 != 0.0 || self.m44 != 1.0
    }

    fn determinant(&self) -> CSSFloat {
        self.m14 * self.m23 * self.m32 * self.m41 -
        self.m13 * self.m24 * self.m32 * self.m41 -
        self.m14 * self.m22 * self.m33 * self.m41 +
        self.m12 * self.m24 * self.m33 * self.m41 +
        self.m13 * self.m22 * self.m34 * self.m41 -
        self.m12 * self.m23 * self.m34 * self.m41 -
        self.m14 * self.m23 * self.m31 * self.m42 +
        self.m13 * self.m24 * self.m31 * self.m42 +
        self.m14 * self.m21 * self.m33 * self.m42 -
        self.m11 * self.m24 * self.m33 * self.m42 -
        self.m13 * self.m21 * self.m34 * self.m42 +
        self.m11 * self.m23 * self.m34 * self.m42 +
        self.m14 * self.m22 * self.m31 * self.m43 -
        self.m12 * self.m24 * self.m31 * self.m43 -
        self.m14 * self.m21 * self.m32 * self.m43 +
        self.m11 * self.m24 * self.m32 * self.m43 +
        self.m12 * self.m21 * self.m34 * self.m43 -
        self.m11 * self.m22 * self.m34 * self.m43 -
        self.m13 * self.m22 * self.m31 * self.m44 +
        self.m12 * self.m23 * self.m31 * self.m44 +
        self.m13 * self.m21 * self.m32 * self.m44 -
        self.m11 * self.m23 * self.m32 * self.m44 -
        self.m12 * self.m21 * self.m33 * self.m44 +
        self.m11 * self.m22 * self.m33 * self.m44
    }

    fn inverse(&self) -> Option<ComputedMatrix> {
        let mut det = self.determinant();

        if det == 0.0 {
            return None;
        }

        det = 1.0 / det;
        let x = ComputedMatrix {
            m11: det *
            (self.m23*self.m34*self.m42 - self.m24*self.m33*self.m42 +
             self.m24*self.m32*self.m43 - self.m22*self.m34*self.m43 -
             self.m23*self.m32*self.m44 + self.m22*self.m33*self.m44),
            m12: det *
            (self.m14*self.m33*self.m42 - self.m13*self.m34*self.m42 -
             self.m14*self.m32*self.m43 + self.m12*self.m34*self.m43 +
             self.m13*self.m32*self.m44 - self.m12*self.m33*self.m44),
            m13: det *
            (self.m13*self.m24*self.m42 - self.m14*self.m23*self.m42 +
             self.m14*self.m22*self.m43 - self.m12*self.m24*self.m43 -
             self.m13*self.m22*self.m44 + self.m12*self.m23*self.m44),
            m14: det *
            (self.m14*self.m23*self.m32 - self.m13*self.m24*self.m32 -
             self.m14*self.m22*self.m33 + self.m12*self.m24*self.m33 +
             self.m13*self.m22*self.m34 - self.m12*self.m23*self.m34),
            m21: det *
            (self.m24*self.m33*self.m41 - self.m23*self.m34*self.m41 -
             self.m24*self.m31*self.m43 + self.m21*self.m34*self.m43 +
             self.m23*self.m31*self.m44 - self.m21*self.m33*self.m44),
            m22: det *
            (self.m13*self.m34*self.m41 - self.m14*self.m33*self.m41 +
             self.m14*self.m31*self.m43 - self.m11*self.m34*self.m43 -
             self.m13*self.m31*self.m44 + self.m11*self.m33*self.m44),
            m23: det *
            (self.m14*self.m23*self.m41 - self.m13*self.m24*self.m41 -
             self.m14*self.m21*self.m43 + self.m11*self.m24*self.m43 +
             self.m13*self.m21*self.m44 - self.m11*self.m23*self.m44),
            m24: det *
            (self.m13*self.m24*self.m31 - self.m14*self.m23*self.m31 +
             self.m14*self.m21*self.m33 - self.m11*self.m24*self.m33 -
             self.m13*self.m21*self.m34 + self.m11*self.m23*self.m34),
            m31: det *
            (self.m22*self.m34*self.m41 - self.m24*self.m32*self.m41 +
             self.m24*self.m31*self.m42 - self.m21*self.m34*self.m42 -
             self.m22*self.m31*self.m44 + self.m21*self.m32*self.m44),
            m32: det *
            (self.m14*self.m32*self.m41 - self.m12*self.m34*self.m41 -
             self.m14*self.m31*self.m42 + self.m11*self.m34*self.m42 +
             self.m12*self.m31*self.m44 - self.m11*self.m32*self.m44),
            m33: det *
            (self.m12*self.m24*self.m41 - self.m14*self.m22*self.m41 +
             self.m14*self.m21*self.m42 - self.m11*self.m24*self.m42 -
             self.m12*self.m21*self.m44 + self.m11*self.m22*self.m44),
            m34: det *
            (self.m14*self.m22*self.m31 - self.m12*self.m24*self.m31 -
             self.m14*self.m21*self.m32 + self.m11*self.m24*self.m32 +
             self.m12*self.m21*self.m34 - self.m11*self.m22*self.m34),
            m41: det *
            (self.m23*self.m32*self.m41 - self.m22*self.m33*self.m41 -
             self.m23*self.m31*self.m42 + self.m21*self.m33*self.m42 +
             self.m22*self.m31*self.m43 - self.m21*self.m32*self.m43),
            m42: det *
            (self.m12*self.m33*self.m41 - self.m13*self.m32*self.m41 +
             self.m13*self.m31*self.m42 - self.m11*self.m33*self.m42 -
             self.m12*self.m31*self.m43 + self.m11*self.m32*self.m43),
            m43: det *
            (self.m13*self.m22*self.m41 - self.m12*self.m23*self.m41 -
             self.m13*self.m21*self.m42 + self.m11*self.m23*self.m42 +
             self.m12*self.m21*self.m43 - self.m11*self.m22*self.m43),
            m44: det *
            (self.m12*self.m23*self.m31 - self.m13*self.m22*self.m31 +
             self.m13*self.m21*self.m32 - self.m11*self.m23*self.m32 -
             self.m12*self.m21*self.m33 + self.m11*self.m22*self.m33),
        };

        Some(x)
    }
}

/// https://drafts.csswg.org/css-transforms/#interpolation-of-transforms
impl Animate for TransformList {
    #[inline]
    fn animate(
        &self,
        other: &Self,
        procedure: Procedure,
    ) -> Result<Self, ()> {
        if self.0.is_none() && other.0.is_none() {
            return Ok(TransformList(None));
        }

        if procedure == Procedure::Add {
            let this = self.0.as_ref().map_or(&[][..], |l| l);
            let other = other.0.as_ref().map_or(&[][..], |l| l);
            let result = this.iter().chain(other).cloned().collect::<Vec<_>>();
            return Ok(TransformList(if result.is_empty() {
                None
            } else {
                Some(result)
            }));
        }

        let this = if self.0.is_some() {
            Cow::Borrowed(self)
        } else {
            Cow::Owned(other.to_animated_zero()?)
        };
        let other = if other.0.is_some() {
            Cow::Borrowed(other)
        } else {
            Cow::Owned(self.to_animated_zero()?)
        };

        {
            let this = (*this).0.as_ref().map_or(&[][..], |l| l);
            let other = (*other).0.as_ref().map_or(&[][..], |l| l);
            if this.len() == other.len() {
                let result = this.iter().zip(other).map(|(this, other)| {
                    this.animate(other, procedure)
                }).collect::<Result<Vec<_>, _>>();
                if let Ok(list) = result {
                    return Ok(TransformList(if list.is_empty() {
                        None
                    } else {
                        Some(list)
                    }));
                }
            }
        }

        match procedure {
            Procedure::Add => Err(()),
            Procedure::Interpolate { progress } => {
                Ok(TransformList(Some(vec![TransformOperation::InterpolateMatrix {
                    from_list: this.into_owned(),
                    to_list: other.into_owned(),
                    progress: Percentage(progress as f32),
                }])))
            },
            Procedure::Accumulate { count } => {
                Ok(TransformList(Some(vec![TransformOperation::AccumulateMatrix {
                    from_list: this.into_owned(),
                    to_list: other.into_owned(),
                    count: cmp::min(count, i32::max_value() as u64) as i32,
                }])))
            },
        }
    }
}

// This might not be the most useful definition of distance. It might be better, for example,
// to trace the distance travelled by a point as its transform is interpolated between the two
// lists. That, however, proves to be quite complicated so we take a simple approach for now.
// See https://bugzilla.mozilla.org/show_bug.cgi?id=1318591#c0.
impl ComputeSquaredDistance for TransformOperation {
    fn compute_squared_distance(&self, other: &Self) -> Result<SquaredDistance, ()> {
        match (self, other) {
            (
                &TransformOperation::Matrix(ref this),
                &TransformOperation::Matrix(ref other),
            ) => {
                this.compute_squared_distance(other)
            },
            (
                &TransformOperation::Skew(ref fx, ref fy),
                &TransformOperation::Skew(ref tx, ref ty),
            ) => {
                Ok(
                    fx.compute_squared_distance(&tx)? +
                    fy.compute_squared_distance(&ty)?,
                )
            },
            (
                &TransformOperation::Translate(ref fx, ref fy, ref fz),
                &TransformOperation::Translate(ref tx, ref ty, ref tz),
            ) => {
                // We don't want to require doing layout in order to calculate the result, so
                // drop the percentage part. However, dropping percentage makes us impossible to
                // compute the distance for the percentage-percentage case, but Gecko uses the
                // same formula, so it's fine for now.
                // Note: We use pixel value to compute the distance for translate, so we have to
                // convert Au into px.
                let extract_pixel_length = |lop: &LengthOrPercentage| {
                    match *lop {
                        LengthOrPercentage::Length(px) => px.px(),
                        LengthOrPercentage::Percentage(_) => 0.,
                        LengthOrPercentage::Calc(calc) => calc.length().px(),
                    }
                };

                let fx = extract_pixel_length(&fx);
                let fy = extract_pixel_length(&fy);
                let tx = extract_pixel_length(&tx);
                let ty = extract_pixel_length(&ty);

                Ok(
                    fx.compute_squared_distance(&tx)? +
                    fy.compute_squared_distance(&ty)? +
                    fz.compute_squared_distance(&tz)?,
                )
            },
            (
                &TransformOperation::Scale(ref fx, ref fy, ref fz),
                &TransformOperation::Scale(ref tx, ref ty, ref tz),
            ) => {
                Ok(
                    fx.compute_squared_distance(&tx)? +
                    fy.compute_squared_distance(&ty)? +
                    fz.compute_squared_distance(&tz)?,
                )
            },
            (
                &TransformOperation::Rotate(fx, fy, fz, fa),
                &TransformOperation::Rotate(tx, ty, tz, ta),
            ) => {
                let (fx, fy, fz, angle1) =
                    TransformList::get_normalized_vector_and_angle(fx, fy, fz, fa);
                let (tx, ty, tz, angle2) =
                    TransformList::get_normalized_vector_and_angle(tx, ty, tz, ta);
                if (fx, fy, fz) == (tx, ty, tz) {
                    angle1.compute_squared_distance(&angle2)
                } else {
                    let v1 = DirectionVector::new(fx, fy, fz);
                    let v2 = DirectionVector::new(tx, ty, tz);
                    let q1 = Quaternion::from_direction_and_angle(&v1, angle1.radians64());
                    let q2 = Quaternion::from_direction_and_angle(&v2, angle2.radians64());
                    q1.compute_squared_distance(&q2)
                }
            }
            (
                &TransformOperation::Perspective(ref fd),
                &TransformOperation::Perspective(ref td),
            ) => {
                let mut fd_matrix = ComputedMatrix::identity();
                let mut td_matrix = ComputedMatrix::identity();
                if fd.px() > 0. {
                    fd_matrix.m34 = -1. / fd.px();
                }

                if td.px() > 0. {
                    td_matrix.m34 = -1. / td.px();
                }
                fd_matrix.compute_squared_distance(&td_matrix)
            }
            (
                &TransformOperation::Perspective(ref p),
                &TransformOperation::Matrix(ref m),
            ) | (
                &TransformOperation::Matrix(ref m),
                &TransformOperation::Perspective(ref p),
            ) => {
                let mut p_matrix = ComputedMatrix::identity();
                if p.px() > 0. {
                    p_matrix.m34 = -1. / p.px();
                }
                p_matrix.compute_squared_distance(&m)
            }
            _ => Err(()),
        }
    }
}

impl ComputeSquaredDistance for TransformList {
    #[inline]
    fn compute_squared_distance(&self, other: &Self) -> Result<SquaredDistance, ()> {
        let list1 = self.0.as_ref().map_or(&[][..], |l| l);
        let list2 = other.0.as_ref().map_or(&[][..], |l| l);

        let squared_dist: Result<SquaredDistance, _> = list1.iter().zip_longest(list2).map(|it| {
            match it {
                EitherOrBoth::Both(this, other) => {
                    this.compute_squared_distance(other)
                },
                EitherOrBoth::Left(list) | EitherOrBoth::Right(list) => {
                    list.to_animated_zero()?.compute_squared_distance(list)
                },
            }
        }).sum();

        // Roll back to matrix interpolation if there is any Err(()) in the transform lists, such
        // as mismatched transform functions.
        if let Err(_) = squared_dist {
            let matrix1: ComputedMatrix = self.to_transform_3d_matrix(None).ok_or(())?.into();
            let matrix2: ComputedMatrix = other.to_transform_3d_matrix(None).ok_or(())?.into();
            return matrix1.compute_squared_distance(&matrix2);
        }
        squared_dist
    }
}

impl ToAnimatedZero for TransformList {
    #[inline]
    fn to_animated_zero(&self) -> Result<Self, ()> {
        match self.0 {
            None => Ok(TransformList(None)),
            Some(ref list) => {
                Ok(TransformList(Some(
                    list.iter().map(|op| op.to_animated_zero()).collect::<Result<Vec<_>, _>>()?
                )))
            },
        }
    }
}

/// Animated SVGPaint
pub type IntermediateSVGPaint = SVGPaint<AnimatedRGBA, ComputedUrl>;

/// Animated SVGPaintKind
pub type IntermediateSVGPaintKind = SVGPaintKind<AnimatedRGBA, ComputedUrl>;

impl ToAnimatedZero for IntermediateSVGPaint {
    #[inline]
    fn to_animated_zero(&self) -> Result<Self, ()> {
        Ok(IntermediateSVGPaint {
            kind: self.kind.to_animated_zero()?,
            fallback: self.fallback.and_then(|v| v.to_animated_zero().ok()),
        })
    }
}

impl From<NonNegativeLengthOrPercentage> for NumberOrPercentage {
    fn from(lop: NonNegativeLengthOrPercentage) -> NumberOrPercentage {
        lop.0.into()
    }
}

impl From<NonNegativeNumber> for NumberOrPercentage {
    fn from(num: NonNegativeNumber) -> NumberOrPercentage {
        num.0.into()
    }
}

impl From<LengthOrPercentage> for NumberOrPercentage {
    fn from(lop: LengthOrPercentage) -> NumberOrPercentage {
        match lop {
            LengthOrPercentage::Length(len) => NumberOrPercentage::Number(len.px()),
            LengthOrPercentage::Percentage(p) => NumberOrPercentage::Percentage(p),
            LengthOrPercentage::Calc(_) => {
                panic!("We dont't expected calc interpolation for SvgLengthOrPercentageOrNumber");
            },
        }
    }
}

impl From<Number> for NumberOrPercentage {
    fn from(num: Number) -> NumberOrPercentage {
        NumberOrPercentage::Number(num)
    }
}

fn convert_to_number_or_percentage<LengthOrPercentageType, NumberType>(
    from: SvgLengthOrPercentageOrNumber<LengthOrPercentageType, NumberType>)
    -> NumberOrPercentage
    where LengthOrPercentageType: Into<NumberOrPercentage>,
          NumberType: Into<NumberOrPercentage>
{
    match from {
        SvgLengthOrPercentageOrNumber::LengthOrPercentage(lop) => {
            lop.into()
        }
        SvgLengthOrPercentageOrNumber::Number(num) => {
            num.into()
        }
    }
}

fn convert_from_number_or_percentage<LengthOrPercentageType, NumberType>(
    from: NumberOrPercentage)
    -> SvgLengthOrPercentageOrNumber<LengthOrPercentageType, NumberType>
    where LengthOrPercentageType: From<LengthOrPercentage>,
          NumberType: From<Number>
{
    match from {
        NumberOrPercentage::Number(num) =>
            SvgLengthOrPercentageOrNumber::Number(num.into()),
        NumberOrPercentage::Percentage(p) =>
            SvgLengthOrPercentageOrNumber::LengthOrPercentage(
                (LengthOrPercentage::Percentage(p)).into())
    }
}

impl <L, N> Animate for SvgLengthOrPercentageOrNumber<L, N>
where
    L: Animate + From<LengthOrPercentage> + Into<NumberOrPercentage> + Copy,
    N: Animate + From<Number> + Into<NumberOrPercentage>,
    LengthOrPercentage: From<L>,
    Self: Copy,
{
    #[inline]
    fn animate(&self, other: &Self, procedure: Procedure) -> Result<Self, ()> {
        if self.has_calc() || other.has_calc() {
            // TODO: We need to treat calc value.
            // https://bugzilla.mozilla.org/show_bug.cgi?id=1386967
            return Err(());
        }

        let this = convert_to_number_or_percentage(*self);
        let other = convert_to_number_or_percentage(*other);

        match (this, other) {
            (
                NumberOrPercentage::Number(ref this),
                NumberOrPercentage::Number(ref other),
            ) => {
                Ok(convert_from_number_or_percentage(
                    NumberOrPercentage::Number(this.animate(other, procedure)?)
                ))
            },
            (
                NumberOrPercentage::Percentage(ref this),
                NumberOrPercentage::Percentage(ref other),
            ) => {
                Ok(convert_from_number_or_percentage(
                    NumberOrPercentage::Percentage(this.animate(other, procedure)?)
                ))
            },
            _ => Err(()),
        }
    }
}

impl<L> Animate for SVGLength<L>
where
    L: Animate + Clone,
{
    #[inline]
    fn animate(&self, other: &Self, procedure: Procedure) -> Result<Self, ()> {
        match (self, other) {
            (&SVGLength::Length(ref this), &SVGLength::Length(ref other)) => {
                Ok(SVGLength::Length(this.animate(other, procedure)?))
            },
            _ => Err(()),
        }
    }
}

/// https://www.w3.org/TR/SVG11/painting.html#StrokeDasharrayProperty
impl<L> Animate for SVGStrokeDashArray<L>
where
    L: Clone + RepeatableListAnimatable,
{
    #[inline]
    fn animate(&self, other: &Self, procedure: Procedure) -> Result<Self, ()> {
        if matches!(procedure, Procedure::Add | Procedure::Accumulate { .. }) {
            // Non-additive.
            return Err(());
        }
        match (self, other) {
            (&SVGStrokeDashArray::Values(ref this), &SVGStrokeDashArray::Values(ref other)) => {
                Ok(SVGStrokeDashArray::Values(this.animate(other, procedure)?))
            },
            _ => Err(()),
        }
    }
}

impl<L> ToAnimatedZero for SVGStrokeDashArray<L>
where
    L: ToAnimatedZero,
{
    #[inline]
    fn to_animated_zero(&self) -> Result<Self, ()> {
        match *self {
            SVGStrokeDashArray::Values(ref values) => {
                Ok(SVGStrokeDashArray::Values(
                    values.iter().map(ToAnimatedZero::to_animated_zero).collect::<Result<Vec<_>, _>>()?,
                ))
            }
            SVGStrokeDashArray::ContextValue => Ok(SVGStrokeDashArray::ContextValue),
        }
    }
}

impl<O> Animate for SVGOpacity<O>
where
    O: Animate + Clone,
{
    #[inline]
    fn animate(&self, other: &Self, procedure: Procedure) -> Result<Self, ()> {
        match (self, other) {
            (&SVGOpacity::Opacity(ref this), &SVGOpacity::Opacity(ref other)) => {
                Ok(SVGOpacity::Opacity(this.animate(other, procedure)?))
            },
            _ => Err(()),
        }
    }
}



/// https://drafts.fxtf.org/filters/#animation-of-filters
impl Animate for AnimatedFilter {
    fn animate(
        &self,
        other: &Self,
        procedure: Procedure,
    ) -> Result<Self, ()> {
        match (self, other) {
            (&Filter::Blur(ref this), &Filter::Blur(ref other)) => {
                Ok(Filter::Blur(this.animate(other, procedure)?))
            },
            (&Filter::Grayscale(ref this), &Filter::Grayscale(ref other)) => {
                Ok(Filter::Grayscale(this.animate(other, procedure)?))
            },
            (&Filter::HueRotate(ref this), &Filter::HueRotate(ref other)) => {
                Ok(Filter::HueRotate(this.animate(other, procedure)?))
            },
            (&Filter::Invert(ref this), &Filter::Invert(ref other)) => {
                Ok(Filter::Invert(this.animate(other, procedure)?))
            },
            (&Filter::Sepia(ref this), &Filter::Sepia(ref other)) => {
                Ok(Filter::Sepia(this.animate(other, procedure)?))
            },
            (&Filter::Brightness(ref this), &Filter::Brightness(ref other)) => {
                Ok(Filter::Brightness(NonNegative(animate_multiplicative_factor(
                    this.0,
                    other.0,
                    procedure,
                )?)))
            },
            (&Filter::Contrast(ref this), &Filter::Contrast(ref other)) => {
                Ok(Filter::Contrast(NonNegative(animate_multiplicative_factor(
                    this.0,
                    other.0,
                    procedure,
                )?)))
            },
            (&Filter::Opacity(ref this), &Filter::Opacity(ref other)) => {
                Ok(Filter::Opacity(NonNegative(animate_multiplicative_factor(
                    this.0,
                    other.0,
                    procedure,
                )?)))
            },
            (&Filter::Saturate(ref this), &Filter::Saturate(ref other)) => {
                Ok(Filter::Saturate(NonNegative(animate_multiplicative_factor(
                    this.0,
                    other.0,
                    procedure,
                )?)))
            },
            (&Filter::DropShadow(ref this), &Filter::DropShadow(ref other)) => {
                Ok(Filter::DropShadow(this.animate(other, procedure)?))
            },
            _ => Err(()),
        }
    }
}

/// http://dev.w3.org/csswg/css-transforms/#none-transform-animation
impl ToAnimatedZero for AnimatedFilter {
    fn to_animated_zero(&self) -> Result<Self, ()> {
        match *self {
            Filter::Blur(ref this) => Ok(Filter::Blur(this.to_animated_zero()?)),
            Filter::Grayscale(ref this) => Ok(Filter::Grayscale(this.to_animated_zero()?)),
            Filter::HueRotate(ref this) => Ok(Filter::HueRotate(this.to_animated_zero()?)),
            Filter::Invert(ref this) => Ok(Filter::Invert(this.to_animated_zero()?)),
            Filter::Sepia(ref this) => Ok(Filter::Sepia(this.to_animated_zero()?)),
            Filter::Brightness(_) => Ok(Filter::Brightness(NonNegative(1.))),
            Filter::Contrast(_) => Ok(Filter::Contrast(NonNegative(1.))),
            Filter::Opacity(_) => Ok(Filter::Opacity(NonNegative(1.))),
            Filter::Saturate(_) => Ok(Filter::Saturate(NonNegative(1.))),
            Filter::DropShadow(ref this) => Ok(Filter::DropShadow(this.to_animated_zero()?)),
            _ => Err(()),
        }
    }
}

// FIXME(nox): This should be derived.
impl ComputeSquaredDistance for AnimatedFilter {
    fn compute_squared_distance(&self, other: &Self) -> Result<SquaredDistance, ()> {
        match (self, other) {
            (&Filter::Blur(ref this), &Filter::Blur(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&Filter::Brightness(ref this), &Filter::Brightness(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&Filter::Contrast(ref this), &Filter::Contrast(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&Filter::Grayscale(ref this), &Filter::Grayscale(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&Filter::HueRotate(ref this), &Filter::HueRotate(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&Filter::Invert(ref this), &Filter::Invert(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&Filter::Opacity(ref this), &Filter::Opacity(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&Filter::Saturate(ref this), &Filter::Saturate(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&Filter::Sepia(ref this), &Filter::Sepia(ref other)) => {
                this.compute_squared_distance(other)
            },
            (&Filter::DropShadow(ref this), &Filter::DropShadow(ref other)) => {
                this.compute_squared_distance(other)
            },
            _ => Err(()),
        }
    }
}

impl Animate for AnimatedFilterList {
    #[inline]
    fn animate(
        &self,
        other: &Self,
        procedure: Procedure,
    ) -> Result<Self, ()> {
        if procedure == Procedure::Add {
            return Ok(AnimatedFilterList(
                self.0.iter().chain(other.0.iter()).cloned().collect(),
            ));
        }
        Ok(AnimatedFilterList(self.0.iter().zip_longest(other.0.iter()).map(|it| {
            match it {
                EitherOrBoth::Both(this, other) => {
                    this.animate(other, procedure)
                },
                EitherOrBoth::Left(this) => {
                    this.animate(&this.to_animated_zero()?, procedure)
                },
                EitherOrBoth::Right(other) => {
                    other.to_animated_zero()?.animate(other, procedure)
                },
            }
        }).collect::<Result<Vec<_>, _>>()?))
    }
}

impl ComputeSquaredDistance for AnimatedFilterList {
    #[inline]
    fn compute_squared_distance(&self, other: &Self) -> Result<SquaredDistance, ()> {
        self.0.iter().zip_longest(other.0.iter()).map(|it| {
            match it {
                EitherOrBoth::Both(this, other) => {
                    this.compute_squared_distance(other)
                },
                EitherOrBoth::Left(list) | EitherOrBoth::Right(list) => {
                    list.to_animated_zero()?.compute_squared_distance(list)
                },
            }
        }).sum()
    }
}

/// A comparator to sort PropertyIds such that longhands are sorted before shorthands,
/// shorthands with fewer components are sorted before shorthands with more components,
/// and otherwise shorthands are sorted by IDL name as defined by [Web Animations][property-order].
///
/// Using this allows us to prioritize values specified by longhands (or smaller
/// shorthand subsets) when longhands and shorthands are both specified on the one keyframe.
///
/// Example orderings that result from this:
///
///   margin-left, margin
///
/// and:
///
///   border-top-color, border-color, border-top, border
///
/// [property-order] https://w3c.github.io/web-animations/#calculating-computed-keyframes
#[cfg(feature = "gecko")]
pub fn compare_property_priority(a: &PropertyId, b: &PropertyId) -> cmp::Ordering {
    match (a.as_shorthand(), b.as_shorthand()) {
        // Within shorthands, sort by the number of subproperties, then by IDL name.
        (Ok(a), Ok(b)) => {
            let subprop_count_a = a.longhands().len();
            let subprop_count_b = b.longhands().len();
            subprop_count_a.cmp(&subprop_count_b).then_with(
                || get_idl_name_sort_order(&a).cmp(&get_idl_name_sort_order(&b)))
        },

        // Longhands go before shorthands.
        (Ok(_), Err(_)) => cmp::Ordering::Greater,
        (Err(_), Ok(_)) => cmp::Ordering::Less,

        // Both are longhands or custom properties in which case they don't overlap and should
        // sort equally.
        _ => cmp::Ordering::Equal,
    }
}

#[cfg(feature = "gecko")]
fn get_idl_name_sort_order(shorthand: &ShorthandId) -> u32 {

    match *shorthand {
            ShorthandId::MozOutlineRadius => 0,
            ShorthandId::MozTransform => 1,
            ShorthandId::WebkitTextStroke => 2,
            ShorthandId::All => 3,
            ShorthandId::Animation => 4,
            ShorthandId::Background => 5,
            ShorthandId::BackgroundPosition => 6,
            ShorthandId::Border => 7,
            ShorthandId::BorderBlockEnd => 8,
            ShorthandId::BorderBlockStart => 9,
            ShorthandId::BorderBottom => 10,
            ShorthandId::BorderColor => 11,
            ShorthandId::BorderImage => 12,
            ShorthandId::BorderInlineEnd => 13,
            ShorthandId::BorderInlineStart => 14,
            ShorthandId::BorderLeft => 15,
            ShorthandId::BorderRadius => 16,
            ShorthandId::BorderRight => 17,
            ShorthandId::BorderStyle => 18,
            ShorthandId::BorderTop => 19,
            ShorthandId::BorderWidth => 20,
            ShorthandId::ColumnRule => 21,
            ShorthandId::Columns => 22,
            ShorthandId::Flex => 23,
            ShorthandId::FlexFlow => 24,
            ShorthandId::Font => 25,
            ShorthandId::FontVariant => 26,
            ShorthandId::Grid => 27,
            ShorthandId::GridArea => 28,
            ShorthandId::GridColumn => 29,
            ShorthandId::GridGap => 30,
            ShorthandId::GridRow => 31,
            ShorthandId::GridTemplate => 32,
            ShorthandId::ListStyle => 33,
            ShorthandId::Margin => 34,
            ShorthandId::Marker => 35,
            ShorthandId::Mask => 36,
            ShorthandId::MaskPosition => 37,
            ShorthandId::Outline => 38,
            ShorthandId::Overflow => 39,
            ShorthandId::Padding => 40,
            ShorthandId::PlaceContent => 41,
            ShorthandId::PlaceItems => 42,
            ShorthandId::PlaceSelf => 43,
            ShorthandId::ScrollSnapType => 44,
            ShorthandId::TextDecoration => 45,
            ShorthandId::TextEmphasis => 46,
            ShorthandId::Transition => 47,
    }
}

}

/// A longhand or shorthand porperty
#[derive(Clone, Copy, Debug)]
pub struct NonCustomPropertyId(usize);

impl From<LonghandId> for NonCustomPropertyId {
    fn from(id: LonghandId) -> Self {
        NonCustomPropertyId(id as usize)
    }
}

impl From<ShorthandId> for NonCustomPropertyId {
    fn from(id: ShorthandId) -> Self {
        NonCustomPropertyId((id as usize) + 318)
    }
}

impl From<AliasId> for NonCustomPropertyId {
    fn from(id: AliasId) -> Self {
        NonCustomPropertyId(id as usize + 366)
    }
}

/// A set of all properties
#[derive(Clone, PartialEq)]
pub struct NonCustomPropertyIdSet {
    storage: [u32; (474 - 1 + 32) / 32]
}

impl NonCustomPropertyIdSet {
    /// Return whether the given property is in the set
    #[inline]
    pub fn contains(&self, id: NonCustomPropertyId) -> bool {
        let bit = id.0;
        (self.storage[bit / 32] & (1 << (bit % 32))) != 0
    }
}





/// A set of longhand properties
#[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
#[derive(Clone, PartialEq)]
pub struct LonghandIdSet {
    storage: [u32; (318 - 1 + 32) / 32]
}

impl LonghandIdSet {
    /// Create an empty set
    #[inline]
    pub fn new() -> LonghandIdSet {
        LonghandIdSet { storage: [0; (318 - 1 + 32) / 32] }
    }

    /// Return whether the given property is in the set
    #[inline]
    pub fn contains(&self, id: LonghandId) -> bool {
        let bit = id as usize;
        (self.storage[bit / 32] & (1 << (bit % 32))) != 0
    }

    /// Add the given property to the set
    #[inline]
    pub fn insert(&mut self, id: LonghandId) {
        let bit = id as usize;
        self.storage[bit / 32] |= 1 << (bit % 32);
    }

    /// Remove the given property from the set
    #[inline]
    pub fn remove(&mut self, id: LonghandId) {
        let bit = id as usize;
        self.storage[bit / 32] &= !(1 << (bit % 32));
    }

    /// Clear all bits
    #[inline]
    pub fn clear(&mut self) {
        for cell in &mut self.storage {
            *cell = 0
        }
    }

    /// Set the corresponding bit of AnimatableLonghand.
    pub fn set_animatable_longhand_bit(&mut self, property: &AnimatableLonghand) {
        match *property {
                    AnimatableLonghand::BackgroundColor => self.insert(LonghandId::BackgroundColor),
                    AnimatableLonghand::BackgroundImage => self.insert(LonghandId::BackgroundImage),
                    AnimatableLonghand::BackgroundPositionX => self.insert(LonghandId::BackgroundPositionX),
                    AnimatableLonghand::BackgroundPositionY => self.insert(LonghandId::BackgroundPositionY),
                    AnimatableLonghand::BackgroundRepeat => self.insert(LonghandId::BackgroundRepeat),
                    AnimatableLonghand::BackgroundAttachment => self.insert(LonghandId::BackgroundAttachment),
                    AnimatableLonghand::BackgroundClip => self.insert(LonghandId::BackgroundClip),
                    AnimatableLonghand::BackgroundOrigin => self.insert(LonghandId::BackgroundOrigin),
                    AnimatableLonghand::BackgroundSize => self.insert(LonghandId::BackgroundSize),
                    AnimatableLonghand::BackgroundBlendMode => self.insert(LonghandId::BackgroundBlendMode),
                    AnimatableLonghand::BorderTopColor => self.insert(LonghandId::BorderTopColor),
                    AnimatableLonghand::BorderTopStyle => self.insert(LonghandId::BorderTopStyle),
                    AnimatableLonghand::BorderTopWidth => self.insert(LonghandId::BorderTopWidth),
                    AnimatableLonghand::BorderLeftColor => self.insert(LonghandId::BorderLeftColor),
                    AnimatableLonghand::BorderLeftStyle => self.insert(LonghandId::BorderLeftStyle),
                    AnimatableLonghand::BorderLeftWidth => self.insert(LonghandId::BorderLeftWidth),
                    AnimatableLonghand::BorderBottomColor => self.insert(LonghandId::BorderBottomColor),
                    AnimatableLonghand::BorderBottomStyle => self.insert(LonghandId::BorderBottomStyle),
                    AnimatableLonghand::BorderBottomWidth => self.insert(LonghandId::BorderBottomWidth),
                    AnimatableLonghand::BorderRightColor => self.insert(LonghandId::BorderRightColor),
                    AnimatableLonghand::BorderRightStyle => self.insert(LonghandId::BorderRightStyle),
                    AnimatableLonghand::BorderRightWidth => self.insert(LonghandId::BorderRightWidth),
                    AnimatableLonghand::BorderTopLeftRadius => self.insert(LonghandId::BorderTopLeftRadius),
                    AnimatableLonghand::BorderTopRightRadius => self.insert(LonghandId::BorderTopRightRadius),
                    AnimatableLonghand::BorderBottomRightRadius => self.insert(LonghandId::BorderBottomRightRadius),
                    AnimatableLonghand::BorderBottomLeftRadius => self.insert(LonghandId::BorderBottomLeftRadius),
                    AnimatableLonghand::MozBorderTopColors => self.insert(LonghandId::MozBorderTopColors),
                    AnimatableLonghand::MozBorderLeftColors => self.insert(LonghandId::MozBorderLeftColors),
                    AnimatableLonghand::MozBorderBottomColors => self.insert(LonghandId::MozBorderBottomColors),
                    AnimatableLonghand::MozBorderRightColors => self.insert(LonghandId::MozBorderRightColors),
                    AnimatableLonghand::BoxDecorationBreak => self.insert(LonghandId::BoxDecorationBreak),
                    AnimatableLonghand::MozFloatEdge => self.insert(LonghandId::MozFloatEdge),
                    AnimatableLonghand::BorderImageSource => self.insert(LonghandId::BorderImageSource),
                    AnimatableLonghand::BorderImageOutset => self.insert(LonghandId::BorderImageOutset),
                    AnimatableLonghand::BorderImageRepeat => self.insert(LonghandId::BorderImageRepeat),
                    AnimatableLonghand::BorderImageWidth => self.insert(LonghandId::BorderImageWidth),
                    AnimatableLonghand::BorderImageSlice => self.insert(LonghandId::BorderImageSlice),
                    AnimatableLonghand::Display => self.insert(LonghandId::Display),
                    AnimatableLonghand::Position => self.insert(LonghandId::Position),
                    AnimatableLonghand::Float => self.insert(LonghandId::Float),
                    AnimatableLonghand::Clear => self.insert(LonghandId::Clear),
                    AnimatableLonghand::VerticalAlign => self.insert(LonghandId::VerticalAlign),
                    AnimatableLonghand::OverflowClipBox => self.insert(LonghandId::OverflowClipBox),
                    AnimatableLonghand::OverflowX => self.insert(LonghandId::OverflowX),
                    AnimatableLonghand::OverflowY => self.insert(LonghandId::OverflowY),
                    AnimatableLonghand::ScrollSnapPointsX => self.insert(LonghandId::ScrollSnapPointsX),
                    AnimatableLonghand::ScrollSnapPointsY => self.insert(LonghandId::ScrollSnapPointsY),
                    AnimatableLonghand::ScrollSnapDestination => self.insert(LonghandId::ScrollSnapDestination),
                    AnimatableLonghand::ScrollSnapCoordinate => self.insert(LonghandId::ScrollSnapCoordinate),
                    AnimatableLonghand::Transform => self.insert(LonghandId::Transform),
                    AnimatableLonghand::ScrollBehavior => self.insert(LonghandId::ScrollBehavior),
                    AnimatableLonghand::ScrollSnapTypeX => self.insert(LonghandId::ScrollSnapTypeX),
                    AnimatableLonghand::ScrollSnapTypeY => self.insert(LonghandId::ScrollSnapTypeY),
                    AnimatableLonghand::Isolation => self.insert(LonghandId::Isolation),
                    AnimatableLonghand::PageBreakAfter => self.insert(LonghandId::PageBreakAfter),
                    AnimatableLonghand::PageBreakBefore => self.insert(LonghandId::PageBreakBefore),
                    AnimatableLonghand::PageBreakInside => self.insert(LonghandId::PageBreakInside),
                    AnimatableLonghand::Resize => self.insert(LonghandId::Resize),
                    AnimatableLonghand::Perspective => self.insert(LonghandId::Perspective),
                    AnimatableLonghand::PerspectiveOrigin => self.insert(LonghandId::PerspectiveOrigin),
                    AnimatableLonghand::BackfaceVisibility => self.insert(LonghandId::BackfaceVisibility),
                    AnimatableLonghand::TransformBox => self.insert(LonghandId::TransformBox),
                    AnimatableLonghand::TransformStyle => self.insert(LonghandId::TransformStyle),
                    AnimatableLonghand::TransformOrigin => self.insert(LonghandId::TransformOrigin),
                    AnimatableLonghand::Contain => self.insert(LonghandId::Contain),
                    AnimatableLonghand::MozAppearance => self.insert(LonghandId::MozAppearance),
                    AnimatableLonghand::MozOrient => self.insert(LonghandId::MozOrient),
                    AnimatableLonghand::WillChange => self.insert(LonghandId::WillChange),
                    AnimatableLonghand::ShapeOutside => self.insert(LonghandId::ShapeOutside),
                    AnimatableLonghand::TouchAction => self.insert(LonghandId::TouchAction),
                    AnimatableLonghand::Color => self.insert(LonghandId::Color),
                    AnimatableLonghand::ColumnWidth => self.insert(LonghandId::ColumnWidth),
                    AnimatableLonghand::ColumnCount => self.insert(LonghandId::ColumnCount),
                    AnimatableLonghand::ColumnGap => self.insert(LonghandId::ColumnGap),
                    AnimatableLonghand::ColumnFill => self.insert(LonghandId::ColumnFill),
                    AnimatableLonghand::ColumnRuleWidth => self.insert(LonghandId::ColumnRuleWidth),
                    AnimatableLonghand::ColumnRuleColor => self.insert(LonghandId::ColumnRuleColor),
                    AnimatableLonghand::ColumnSpan => self.insert(LonghandId::ColumnSpan),
                    AnimatableLonghand::ColumnRuleStyle => self.insert(LonghandId::ColumnRuleStyle),
                    AnimatableLonghand::Content => self.insert(LonghandId::Content),
                    AnimatableLonghand::CounterIncrement => self.insert(LonghandId::CounterIncrement),
                    AnimatableLonghand::CounterReset => self.insert(LonghandId::CounterReset),
                    AnimatableLonghand::Opacity => self.insert(LonghandId::Opacity),
                    AnimatableLonghand::BoxShadow => self.insert(LonghandId::BoxShadow),
                    AnimatableLonghand::Clip => self.insert(LonghandId::Clip),
                    AnimatableLonghand::Filter => self.insert(LonghandId::Filter),
                    AnimatableLonghand::MixBlendMode => self.insert(LonghandId::MixBlendMode),
                    AnimatableLonghand::FontFamily => self.insert(LonghandId::FontFamily),
                    AnimatableLonghand::FontStyle => self.insert(LonghandId::FontStyle),
                    AnimatableLonghand::FontVariantCaps => self.insert(LonghandId::FontVariantCaps),
                    AnimatableLonghand::FontWeight => self.insert(LonghandId::FontWeight),
                    AnimatableLonghand::FontSize => self.insert(LonghandId::FontSize),
                    AnimatableLonghand::FontSizeAdjust => self.insert(LonghandId::FontSizeAdjust),
                    AnimatableLonghand::FontSynthesis => self.insert(LonghandId::FontSynthesis),
                    AnimatableLonghand::FontStretch => self.insert(LonghandId::FontStretch),
                    AnimatableLonghand::FontKerning => self.insert(LonghandId::FontKerning),
                    AnimatableLonghand::FontVariantAlternates => self.insert(LonghandId::FontVariantAlternates),
                    AnimatableLonghand::FontVariantEastAsian => self.insert(LonghandId::FontVariantEastAsian),
                    AnimatableLonghand::FontVariantLigatures => self.insert(LonghandId::FontVariantLigatures),
                    AnimatableLonghand::FontVariantNumeric => self.insert(LonghandId::FontVariantNumeric),
                    AnimatableLonghand::FontVariantPosition => self.insert(LonghandId::FontVariantPosition),
                    AnimatableLonghand::FontFeatureSettings => self.insert(LonghandId::FontFeatureSettings),
                    AnimatableLonghand::FontVariationSettings => self.insert(LonghandId::FontVariationSettings),
                    AnimatableLonghand::FontLanguageOverride => self.insert(LonghandId::FontLanguageOverride),
                    AnimatableLonghand::MozOsxFontSmoothing => self.insert(LonghandId::MozOsxFontSmoothing),
                    AnimatableLonghand::Visibility => self.insert(LonghandId::Visibility),
                    AnimatableLonghand::WritingMode => self.insert(LonghandId::WritingMode),
                    AnimatableLonghand::Direction => self.insert(LonghandId::Direction),
                    AnimatableLonghand::TextOrientation => self.insert(LonghandId::TextOrientation),
                    AnimatableLonghand::ColorAdjust => self.insert(LonghandId::ColorAdjust),
                    AnimatableLonghand::ImageRendering => self.insert(LonghandId::ImageRendering),
                    AnimatableLonghand::ImageOrientation => self.insert(LonghandId::ImageOrientation),
                    AnimatableLonghand::BorderCollapse => self.insert(LonghandId::BorderCollapse),
                    AnimatableLonghand::EmptyCells => self.insert(LonghandId::EmptyCells),
                    AnimatableLonghand::CaptionSide => self.insert(LonghandId::CaptionSide),
                    AnimatableLonghand::BorderSpacing => self.insert(LonghandId::BorderSpacing),
                    AnimatableLonghand::LineHeight => self.insert(LonghandId::LineHeight),
                    AnimatableLonghand::TextTransform => self.insert(LonghandId::TextTransform),
                    AnimatableLonghand::Hyphens => self.insert(LonghandId::Hyphens),
                    AnimatableLonghand::MozTextSizeAdjust => self.insert(LonghandId::MozTextSizeAdjust),
                    AnimatableLonghand::TextIndent => self.insert(LonghandId::TextIndent),
                    AnimatableLonghand::OverflowWrap => self.insert(LonghandId::OverflowWrap),
                    AnimatableLonghand::WordBreak => self.insert(LonghandId::WordBreak),
                    AnimatableLonghand::TextJustify => self.insert(LonghandId::TextJustify),
                    AnimatableLonghand::TextAlignLast => self.insert(LonghandId::TextAlignLast),
                    AnimatableLonghand::TextAlign => self.insert(LonghandId::TextAlign),
                    AnimatableLonghand::LetterSpacing => self.insert(LonghandId::LetterSpacing),
                    AnimatableLonghand::WordSpacing => self.insert(LonghandId::WordSpacing),
                    AnimatableLonghand::WhiteSpace => self.insert(LonghandId::WhiteSpace),
                    AnimatableLonghand::TextShadow => self.insert(LonghandId::TextShadow),
                    AnimatableLonghand::TextEmphasisStyle => self.insert(LonghandId::TextEmphasisStyle),
                    AnimatableLonghand::TextEmphasisPosition => self.insert(LonghandId::TextEmphasisPosition),
                    AnimatableLonghand::TextEmphasisColor => self.insert(LonghandId::TextEmphasisColor),
                    AnimatableLonghand::MozTabSize => self.insert(LonghandId::MozTabSize),
                    AnimatableLonghand::WebkitTextFillColor => self.insert(LonghandId::WebkitTextFillColor),
                    AnimatableLonghand::WebkitTextStrokeColor => self.insert(LonghandId::WebkitTextStrokeColor),
                    AnimatableLonghand::WebkitTextStrokeWidth => self.insert(LonghandId::WebkitTextStrokeWidth),
                    AnimatableLonghand::RubyAlign => self.insert(LonghandId::RubyAlign),
                    AnimatableLonghand::RubyPosition => self.insert(LonghandId::RubyPosition),
                    AnimatableLonghand::TextCombineUpright => self.insert(LonghandId::TextCombineUpright),
                    AnimatableLonghand::TextRendering => self.insert(LonghandId::TextRendering),
                    AnimatableLonghand::ListStylePosition => self.insert(LonghandId::ListStylePosition),
                    AnimatableLonghand::ListStyleType => self.insert(LonghandId::ListStyleType),
                    AnimatableLonghand::ListStyleImage => self.insert(LonghandId::ListStyleImage),
                    AnimatableLonghand::Quotes => self.insert(LonghandId::Quotes),
                    AnimatableLonghand::MozImageRegion => self.insert(LonghandId::MozImageRegion),
                    AnimatableLonghand::MarginTop => self.insert(LonghandId::MarginTop),
                    AnimatableLonghand::MarginLeft => self.insert(LonghandId::MarginLeft),
                    AnimatableLonghand::MarginBottom => self.insert(LonghandId::MarginBottom),
                    AnimatableLonghand::MarginRight => self.insert(LonghandId::MarginRight),
                    AnimatableLonghand::OutlineColor => self.insert(LonghandId::OutlineColor),
                    AnimatableLonghand::OutlineStyle => self.insert(LonghandId::OutlineStyle),
                    AnimatableLonghand::OutlineWidth => self.insert(LonghandId::OutlineWidth),
                    AnimatableLonghand::MozOutlineRadiusTopleft => self.insert(LonghandId::MozOutlineRadiusTopleft),
                    AnimatableLonghand::MozOutlineRadiusTopright => self.insert(LonghandId::MozOutlineRadiusTopright),
                    AnimatableLonghand::MozOutlineRadiusBottomright => self.insert(LonghandId::MozOutlineRadiusBottomright),
                    AnimatableLonghand::MozOutlineRadiusBottomleft => self.insert(LonghandId::MozOutlineRadiusBottomleft),
                    AnimatableLonghand::OutlineOffset => self.insert(LonghandId::OutlineOffset),
                    AnimatableLonghand::PaddingTop => self.insert(LonghandId::PaddingTop),
                    AnimatableLonghand::PaddingLeft => self.insert(LonghandId::PaddingLeft),
                    AnimatableLonghand::PaddingBottom => self.insert(LonghandId::PaddingBottom),
                    AnimatableLonghand::PaddingRight => self.insert(LonghandId::PaddingRight),
                    AnimatableLonghand::Cursor => self.insert(LonghandId::Cursor),
                    AnimatableLonghand::PointerEvents => self.insert(LonghandId::PointerEvents),
                    AnimatableLonghand::MozUserInput => self.insert(LonghandId::MozUserInput),
                    AnimatableLonghand::MozUserModify => self.insert(LonghandId::MozUserModify),
                    AnimatableLonghand::MozUserFocus => self.insert(LonghandId::MozUserFocus),
                    AnimatableLonghand::CaretColor => self.insert(LonghandId::CaretColor),
                    AnimatableLonghand::Top => self.insert(LonghandId::Top),
                    AnimatableLonghand::Left => self.insert(LonghandId::Left),
                    AnimatableLonghand::Bottom => self.insert(LonghandId::Bottom),
                    AnimatableLonghand::Right => self.insert(LonghandId::Right),
                    AnimatableLonghand::ZIndex => self.insert(LonghandId::ZIndex),
                    AnimatableLonghand::FlexDirection => self.insert(LonghandId::FlexDirection),
                    AnimatableLonghand::FlexWrap => self.insert(LonghandId::FlexWrap),
                    AnimatableLonghand::JustifyContent => self.insert(LonghandId::JustifyContent),
                    AnimatableLonghand::AlignContent => self.insert(LonghandId::AlignContent),
                    AnimatableLonghand::AlignItems => self.insert(LonghandId::AlignItems),
                    AnimatableLonghand::JustifyItems => self.insert(LonghandId::JustifyItems),
                    AnimatableLonghand::FlexGrow => self.insert(LonghandId::FlexGrow),
                    AnimatableLonghand::FlexShrink => self.insert(LonghandId::FlexShrink),
                    AnimatableLonghand::AlignSelf => self.insert(LonghandId::AlignSelf),
                    AnimatableLonghand::JustifySelf => self.insert(LonghandId::JustifySelf),
                    AnimatableLonghand::Order => self.insert(LonghandId::Order),
                    AnimatableLonghand::FlexBasis => self.insert(LonghandId::FlexBasis),
                    AnimatableLonghand::Width => self.insert(LonghandId::Width),
                    AnimatableLonghand::MinWidth => self.insert(LonghandId::MinWidth),
                    AnimatableLonghand::MaxWidth => self.insert(LonghandId::MaxWidth),
                    AnimatableLonghand::Height => self.insert(LonghandId::Height),
                    AnimatableLonghand::MinHeight => self.insert(LonghandId::MinHeight),
                    AnimatableLonghand::MaxHeight => self.insert(LonghandId::MaxHeight),
                    AnimatableLonghand::BoxSizing => self.insert(LonghandId::BoxSizing),
                    AnimatableLonghand::ObjectFit => self.insert(LonghandId::ObjectFit),
                    AnimatableLonghand::ObjectPosition => self.insert(LonghandId::ObjectPosition),
                    AnimatableLonghand::GridRowGap => self.insert(LonghandId::GridRowGap),
                    AnimatableLonghand::GridRowStart => self.insert(LonghandId::GridRowStart),
                    AnimatableLonghand::GridRowEnd => self.insert(LonghandId::GridRowEnd),
                    AnimatableLonghand::GridAutoRows => self.insert(LonghandId::GridAutoRows),
                    AnimatableLonghand::GridTemplateRows => self.insert(LonghandId::GridTemplateRows),
                    AnimatableLonghand::GridColumnGap => self.insert(LonghandId::GridColumnGap),
                    AnimatableLonghand::GridColumnStart => self.insert(LonghandId::GridColumnStart),
                    AnimatableLonghand::GridColumnEnd => self.insert(LonghandId::GridColumnEnd),
                    AnimatableLonghand::GridAutoColumns => self.insert(LonghandId::GridAutoColumns),
                    AnimatableLonghand::GridTemplateColumns => self.insert(LonghandId::GridTemplateColumns),
                    AnimatableLonghand::GridAutoFlow => self.insert(LonghandId::GridAutoFlow),
                    AnimatableLonghand::GridTemplateAreas => self.insert(LonghandId::GridTemplateAreas),
                    AnimatableLonghand::TableLayout => self.insert(LonghandId::TableLayout),
                    AnimatableLonghand::TextOverflow => self.insert(LonghandId::TextOverflow),
                    AnimatableLonghand::UnicodeBidi => self.insert(LonghandId::UnicodeBidi),
                    AnimatableLonghand::TextDecorationLine => self.insert(LonghandId::TextDecorationLine),
                    AnimatableLonghand::TextDecorationStyle => self.insert(LonghandId::TextDecorationStyle),
                    AnimatableLonghand::TextDecorationColor => self.insert(LonghandId::TextDecorationColor),
                    AnimatableLonghand::InitialLetter => self.insert(LonghandId::InitialLetter),
                    AnimatableLonghand::ImeMode => self.insert(LonghandId::ImeMode),
                    AnimatableLonghand::MozUserSelect => self.insert(LonghandId::MozUserSelect),
                    AnimatableLonghand::MozWindowDragging => self.insert(LonghandId::MozWindowDragging),
                    AnimatableLonghand::MozWindowShadow => self.insert(LonghandId::MozWindowShadow),
                    AnimatableLonghand::MozForceBrokenImageIcon => self.insert(LonghandId::MozForceBrokenImageIcon),
                    AnimatableLonghand::TextAnchor => self.insert(LonghandId::TextAnchor),
                    AnimatableLonghand::ColorInterpolation => self.insert(LonghandId::ColorInterpolation),
                    AnimatableLonghand::ColorInterpolationFilters => self.insert(LonghandId::ColorInterpolationFilters),
                    AnimatableLonghand::Fill => self.insert(LonghandId::Fill),
                    AnimatableLonghand::FillOpacity => self.insert(LonghandId::FillOpacity),
                    AnimatableLonghand::FillRule => self.insert(LonghandId::FillRule),
                    AnimatableLonghand::ShapeRendering => self.insert(LonghandId::ShapeRendering),
                    AnimatableLonghand::Stroke => self.insert(LonghandId::Stroke),
                    AnimatableLonghand::StrokeWidth => self.insert(LonghandId::StrokeWidth),
                    AnimatableLonghand::StrokeLinecap => self.insert(LonghandId::StrokeLinecap),
                    AnimatableLonghand::StrokeLinejoin => self.insert(LonghandId::StrokeLinejoin),
                    AnimatableLonghand::StrokeMiterlimit => self.insert(LonghandId::StrokeMiterlimit),
                    AnimatableLonghand::StrokeOpacity => self.insert(LonghandId::StrokeOpacity),
                    AnimatableLonghand::StrokeDasharray => self.insert(LonghandId::StrokeDasharray),
                    AnimatableLonghand::StrokeDashoffset => self.insert(LonghandId::StrokeDashoffset),
                    AnimatableLonghand::ClipRule => self.insert(LonghandId::ClipRule),
                    AnimatableLonghand::MarkerStart => self.insert(LonghandId::MarkerStart),
                    AnimatableLonghand::MarkerMid => self.insert(LonghandId::MarkerMid),
                    AnimatableLonghand::MarkerEnd => self.insert(LonghandId::MarkerEnd),
                    AnimatableLonghand::PaintOrder => self.insert(LonghandId::PaintOrder),
                    AnimatableLonghand::DominantBaseline => self.insert(LonghandId::DominantBaseline),
                    AnimatableLonghand::VectorEffect => self.insert(LonghandId::VectorEffect),
                    AnimatableLonghand::StopColor => self.insert(LonghandId::StopColor),
                    AnimatableLonghand::StopOpacity => self.insert(LonghandId::StopOpacity),
                    AnimatableLonghand::FloodColor => self.insert(LonghandId::FloodColor),
                    AnimatableLonghand::FloodOpacity => self.insert(LonghandId::FloodOpacity),
                    AnimatableLonghand::LightingColor => self.insert(LonghandId::LightingColor),
                    AnimatableLonghand::MaskType => self.insert(LonghandId::MaskType),
                    AnimatableLonghand::ClipPath => self.insert(LonghandId::ClipPath),
                    AnimatableLonghand::MaskMode => self.insert(LonghandId::MaskMode),
                    AnimatableLonghand::MaskRepeat => self.insert(LonghandId::MaskRepeat),
                    AnimatableLonghand::MaskPositionX => self.insert(LonghandId::MaskPositionX),
                    AnimatableLonghand::MaskPositionY => self.insert(LonghandId::MaskPositionY),
                    AnimatableLonghand::MaskClip => self.insert(LonghandId::MaskClip),
                    AnimatableLonghand::MaskOrigin => self.insert(LonghandId::MaskOrigin),
                    AnimatableLonghand::MaskSize => self.insert(LonghandId::MaskSize),
                    AnimatableLonghand::MaskComposite => self.insert(LonghandId::MaskComposite),
                    AnimatableLonghand::MaskImage => self.insert(LonghandId::MaskImage),
                    AnimatableLonghand::MozBoxAlign => self.insert(LonghandId::MozBoxAlign),
                    AnimatableLonghand::MozBoxDirection => self.insert(LonghandId::MozBoxDirection),
                    AnimatableLonghand::MozBoxFlex => self.insert(LonghandId::MozBoxFlex),
                    AnimatableLonghand::MozBoxOrient => self.insert(LonghandId::MozBoxOrient),
                    AnimatableLonghand::MozBoxPack => self.insert(LonghandId::MozBoxPack),
                    AnimatableLonghand::MozStackSizing => self.insert(LonghandId::MozStackSizing),
                    AnimatableLonghand::MozBoxOrdinalGroup => self.insert(LonghandId::MozBoxOrdinalGroup),
        }
    }

    /// Return true if the corresponding bit of AnimatableLonghand is set.
    pub fn has_animatable_longhand_bit(&self, property: &AnimatableLonghand) -> bool {
        match *property {
                    AnimatableLonghand::BackgroundColor => self.contains(LonghandId::BackgroundColor),
                    AnimatableLonghand::BackgroundImage => self.contains(LonghandId::BackgroundImage),
                    AnimatableLonghand::BackgroundPositionX => self.contains(LonghandId::BackgroundPositionX),
                    AnimatableLonghand::BackgroundPositionY => self.contains(LonghandId::BackgroundPositionY),
                    AnimatableLonghand::BackgroundRepeat => self.contains(LonghandId::BackgroundRepeat),
                    AnimatableLonghand::BackgroundAttachment => self.contains(LonghandId::BackgroundAttachment),
                    AnimatableLonghand::BackgroundClip => self.contains(LonghandId::BackgroundClip),
                    AnimatableLonghand::BackgroundOrigin => self.contains(LonghandId::BackgroundOrigin),
                    AnimatableLonghand::BackgroundSize => self.contains(LonghandId::BackgroundSize),
                    AnimatableLonghand::BackgroundBlendMode => self.contains(LonghandId::BackgroundBlendMode),
                    AnimatableLonghand::BorderTopColor => self.contains(LonghandId::BorderTopColor),
                    AnimatableLonghand::BorderTopStyle => self.contains(LonghandId::BorderTopStyle),
                    AnimatableLonghand::BorderTopWidth => self.contains(LonghandId::BorderTopWidth),
                    AnimatableLonghand::BorderLeftColor => self.contains(LonghandId::BorderLeftColor),
                    AnimatableLonghand::BorderLeftStyle => self.contains(LonghandId::BorderLeftStyle),
                    AnimatableLonghand::BorderLeftWidth => self.contains(LonghandId::BorderLeftWidth),
                    AnimatableLonghand::BorderBottomColor => self.contains(LonghandId::BorderBottomColor),
                    AnimatableLonghand::BorderBottomStyle => self.contains(LonghandId::BorderBottomStyle),
                    AnimatableLonghand::BorderBottomWidth => self.contains(LonghandId::BorderBottomWidth),
                    AnimatableLonghand::BorderRightColor => self.contains(LonghandId::BorderRightColor),
                    AnimatableLonghand::BorderRightStyle => self.contains(LonghandId::BorderRightStyle),
                    AnimatableLonghand::BorderRightWidth => self.contains(LonghandId::BorderRightWidth),
                    AnimatableLonghand::BorderTopLeftRadius => self.contains(LonghandId::BorderTopLeftRadius),
                    AnimatableLonghand::BorderTopRightRadius => self.contains(LonghandId::BorderTopRightRadius),
                    AnimatableLonghand::BorderBottomRightRadius => self.contains(LonghandId::BorderBottomRightRadius),
                    AnimatableLonghand::BorderBottomLeftRadius => self.contains(LonghandId::BorderBottomLeftRadius),
                    AnimatableLonghand::MozBorderTopColors => self.contains(LonghandId::MozBorderTopColors),
                    AnimatableLonghand::MozBorderLeftColors => self.contains(LonghandId::MozBorderLeftColors),
                    AnimatableLonghand::MozBorderBottomColors => self.contains(LonghandId::MozBorderBottomColors),
                    AnimatableLonghand::MozBorderRightColors => self.contains(LonghandId::MozBorderRightColors),
                    AnimatableLonghand::BoxDecorationBreak => self.contains(LonghandId::BoxDecorationBreak),
                    AnimatableLonghand::MozFloatEdge => self.contains(LonghandId::MozFloatEdge),
                    AnimatableLonghand::BorderImageSource => self.contains(LonghandId::BorderImageSource),
                    AnimatableLonghand::BorderImageOutset => self.contains(LonghandId::BorderImageOutset),
                    AnimatableLonghand::BorderImageRepeat => self.contains(LonghandId::BorderImageRepeat),
                    AnimatableLonghand::BorderImageWidth => self.contains(LonghandId::BorderImageWidth),
                    AnimatableLonghand::BorderImageSlice => self.contains(LonghandId::BorderImageSlice),
                    AnimatableLonghand::Display => self.contains(LonghandId::Display),
                    AnimatableLonghand::Position => self.contains(LonghandId::Position),
                    AnimatableLonghand::Float => self.contains(LonghandId::Float),
                    AnimatableLonghand::Clear => self.contains(LonghandId::Clear),
                    AnimatableLonghand::VerticalAlign => self.contains(LonghandId::VerticalAlign),
                    AnimatableLonghand::OverflowClipBox => self.contains(LonghandId::OverflowClipBox),
                    AnimatableLonghand::OverflowX => self.contains(LonghandId::OverflowX),
                    AnimatableLonghand::OverflowY => self.contains(LonghandId::OverflowY),
                    AnimatableLonghand::ScrollSnapPointsX => self.contains(LonghandId::ScrollSnapPointsX),
                    AnimatableLonghand::ScrollSnapPointsY => self.contains(LonghandId::ScrollSnapPointsY),
                    AnimatableLonghand::ScrollSnapDestination => self.contains(LonghandId::ScrollSnapDestination),
                    AnimatableLonghand::ScrollSnapCoordinate => self.contains(LonghandId::ScrollSnapCoordinate),
                    AnimatableLonghand::Transform => self.contains(LonghandId::Transform),
                    AnimatableLonghand::ScrollBehavior => self.contains(LonghandId::ScrollBehavior),
                    AnimatableLonghand::ScrollSnapTypeX => self.contains(LonghandId::ScrollSnapTypeX),
                    AnimatableLonghand::ScrollSnapTypeY => self.contains(LonghandId::ScrollSnapTypeY),
                    AnimatableLonghand::Isolation => self.contains(LonghandId::Isolation),
                    AnimatableLonghand::PageBreakAfter => self.contains(LonghandId::PageBreakAfter),
                    AnimatableLonghand::PageBreakBefore => self.contains(LonghandId::PageBreakBefore),
                    AnimatableLonghand::PageBreakInside => self.contains(LonghandId::PageBreakInside),
                    AnimatableLonghand::Resize => self.contains(LonghandId::Resize),
                    AnimatableLonghand::Perspective => self.contains(LonghandId::Perspective),
                    AnimatableLonghand::PerspectiveOrigin => self.contains(LonghandId::PerspectiveOrigin),
                    AnimatableLonghand::BackfaceVisibility => self.contains(LonghandId::BackfaceVisibility),
                    AnimatableLonghand::TransformBox => self.contains(LonghandId::TransformBox),
                    AnimatableLonghand::TransformStyle => self.contains(LonghandId::TransformStyle),
                    AnimatableLonghand::TransformOrigin => self.contains(LonghandId::TransformOrigin),
                    AnimatableLonghand::Contain => self.contains(LonghandId::Contain),
                    AnimatableLonghand::MozAppearance => self.contains(LonghandId::MozAppearance),
                    AnimatableLonghand::MozOrient => self.contains(LonghandId::MozOrient),
                    AnimatableLonghand::WillChange => self.contains(LonghandId::WillChange),
                    AnimatableLonghand::ShapeOutside => self.contains(LonghandId::ShapeOutside),
                    AnimatableLonghand::TouchAction => self.contains(LonghandId::TouchAction),
                    AnimatableLonghand::Color => self.contains(LonghandId::Color),
                    AnimatableLonghand::ColumnWidth => self.contains(LonghandId::ColumnWidth),
                    AnimatableLonghand::ColumnCount => self.contains(LonghandId::ColumnCount),
                    AnimatableLonghand::ColumnGap => self.contains(LonghandId::ColumnGap),
                    AnimatableLonghand::ColumnFill => self.contains(LonghandId::ColumnFill),
                    AnimatableLonghand::ColumnRuleWidth => self.contains(LonghandId::ColumnRuleWidth),
                    AnimatableLonghand::ColumnRuleColor => self.contains(LonghandId::ColumnRuleColor),
                    AnimatableLonghand::ColumnSpan => self.contains(LonghandId::ColumnSpan),
                    AnimatableLonghand::ColumnRuleStyle => self.contains(LonghandId::ColumnRuleStyle),
                    AnimatableLonghand::Content => self.contains(LonghandId::Content),
                    AnimatableLonghand::CounterIncrement => self.contains(LonghandId::CounterIncrement),
                    AnimatableLonghand::CounterReset => self.contains(LonghandId::CounterReset),
                    AnimatableLonghand::Opacity => self.contains(LonghandId::Opacity),
                    AnimatableLonghand::BoxShadow => self.contains(LonghandId::BoxShadow),
                    AnimatableLonghand::Clip => self.contains(LonghandId::Clip),
                    AnimatableLonghand::Filter => self.contains(LonghandId::Filter),
                    AnimatableLonghand::MixBlendMode => self.contains(LonghandId::MixBlendMode),
                    AnimatableLonghand::FontFamily => self.contains(LonghandId::FontFamily),
                    AnimatableLonghand::FontStyle => self.contains(LonghandId::FontStyle),
                    AnimatableLonghand::FontVariantCaps => self.contains(LonghandId::FontVariantCaps),
                    AnimatableLonghand::FontWeight => self.contains(LonghandId::FontWeight),
                    AnimatableLonghand::FontSize => self.contains(LonghandId::FontSize),
                    AnimatableLonghand::FontSizeAdjust => self.contains(LonghandId::FontSizeAdjust),
                    AnimatableLonghand::FontSynthesis => self.contains(LonghandId::FontSynthesis),
                    AnimatableLonghand::FontStretch => self.contains(LonghandId::FontStretch),
                    AnimatableLonghand::FontKerning => self.contains(LonghandId::FontKerning),
                    AnimatableLonghand::FontVariantAlternates => self.contains(LonghandId::FontVariantAlternates),
                    AnimatableLonghand::FontVariantEastAsian => self.contains(LonghandId::FontVariantEastAsian),
                    AnimatableLonghand::FontVariantLigatures => self.contains(LonghandId::FontVariantLigatures),
                    AnimatableLonghand::FontVariantNumeric => self.contains(LonghandId::FontVariantNumeric),
                    AnimatableLonghand::FontVariantPosition => self.contains(LonghandId::FontVariantPosition),
                    AnimatableLonghand::FontFeatureSettings => self.contains(LonghandId::FontFeatureSettings),
                    AnimatableLonghand::FontVariationSettings => self.contains(LonghandId::FontVariationSettings),
                    AnimatableLonghand::FontLanguageOverride => self.contains(LonghandId::FontLanguageOverride),
                    AnimatableLonghand::MozOsxFontSmoothing => self.contains(LonghandId::MozOsxFontSmoothing),
                    AnimatableLonghand::Visibility => self.contains(LonghandId::Visibility),
                    AnimatableLonghand::WritingMode => self.contains(LonghandId::WritingMode),
                    AnimatableLonghand::Direction => self.contains(LonghandId::Direction),
                    AnimatableLonghand::TextOrientation => self.contains(LonghandId::TextOrientation),
                    AnimatableLonghand::ColorAdjust => self.contains(LonghandId::ColorAdjust),
                    AnimatableLonghand::ImageRendering => self.contains(LonghandId::ImageRendering),
                    AnimatableLonghand::ImageOrientation => self.contains(LonghandId::ImageOrientation),
                    AnimatableLonghand::BorderCollapse => self.contains(LonghandId::BorderCollapse),
                    AnimatableLonghand::EmptyCells => self.contains(LonghandId::EmptyCells),
                    AnimatableLonghand::CaptionSide => self.contains(LonghandId::CaptionSide),
                    AnimatableLonghand::BorderSpacing => self.contains(LonghandId::BorderSpacing),
                    AnimatableLonghand::LineHeight => self.contains(LonghandId::LineHeight),
                    AnimatableLonghand::TextTransform => self.contains(LonghandId::TextTransform),
                    AnimatableLonghand::Hyphens => self.contains(LonghandId::Hyphens),
                    AnimatableLonghand::MozTextSizeAdjust => self.contains(LonghandId::MozTextSizeAdjust),
                    AnimatableLonghand::TextIndent => self.contains(LonghandId::TextIndent),
                    AnimatableLonghand::OverflowWrap => self.contains(LonghandId::OverflowWrap),
                    AnimatableLonghand::WordBreak => self.contains(LonghandId::WordBreak),
                    AnimatableLonghand::TextJustify => self.contains(LonghandId::TextJustify),
                    AnimatableLonghand::TextAlignLast => self.contains(LonghandId::TextAlignLast),
                    AnimatableLonghand::TextAlign => self.contains(LonghandId::TextAlign),
                    AnimatableLonghand::LetterSpacing => self.contains(LonghandId::LetterSpacing),
                    AnimatableLonghand::WordSpacing => self.contains(LonghandId::WordSpacing),
                    AnimatableLonghand::WhiteSpace => self.contains(LonghandId::WhiteSpace),
                    AnimatableLonghand::TextShadow => self.contains(LonghandId::TextShadow),
                    AnimatableLonghand::TextEmphasisStyle => self.contains(LonghandId::TextEmphasisStyle),
                    AnimatableLonghand::TextEmphasisPosition => self.contains(LonghandId::TextEmphasisPosition),
                    AnimatableLonghand::TextEmphasisColor => self.contains(LonghandId::TextEmphasisColor),
                    AnimatableLonghand::MozTabSize => self.contains(LonghandId::MozTabSize),
                    AnimatableLonghand::WebkitTextFillColor => self.contains(LonghandId::WebkitTextFillColor),
                    AnimatableLonghand::WebkitTextStrokeColor => self.contains(LonghandId::WebkitTextStrokeColor),
                    AnimatableLonghand::WebkitTextStrokeWidth => self.contains(LonghandId::WebkitTextStrokeWidth),
                    AnimatableLonghand::RubyAlign => self.contains(LonghandId::RubyAlign),
                    AnimatableLonghand::RubyPosition => self.contains(LonghandId::RubyPosition),
                    AnimatableLonghand::TextCombineUpright => self.contains(LonghandId::TextCombineUpright),
                    AnimatableLonghand::TextRendering => self.contains(LonghandId::TextRendering),
                    AnimatableLonghand::ListStylePosition => self.contains(LonghandId::ListStylePosition),
                    AnimatableLonghand::ListStyleType => self.contains(LonghandId::ListStyleType),
                    AnimatableLonghand::ListStyleImage => self.contains(LonghandId::ListStyleImage),
                    AnimatableLonghand::Quotes => self.contains(LonghandId::Quotes),
                    AnimatableLonghand::MozImageRegion => self.contains(LonghandId::MozImageRegion),
                    AnimatableLonghand::MarginTop => self.contains(LonghandId::MarginTop),
                    AnimatableLonghand::MarginLeft => self.contains(LonghandId::MarginLeft),
                    AnimatableLonghand::MarginBottom => self.contains(LonghandId::MarginBottom),
                    AnimatableLonghand::MarginRight => self.contains(LonghandId::MarginRight),
                    AnimatableLonghand::OutlineColor => self.contains(LonghandId::OutlineColor),
                    AnimatableLonghand::OutlineStyle => self.contains(LonghandId::OutlineStyle),
                    AnimatableLonghand::OutlineWidth => self.contains(LonghandId::OutlineWidth),
                    AnimatableLonghand::MozOutlineRadiusTopleft => self.contains(LonghandId::MozOutlineRadiusTopleft),
                    AnimatableLonghand::MozOutlineRadiusTopright => self.contains(LonghandId::MozOutlineRadiusTopright),
                    AnimatableLonghand::MozOutlineRadiusBottomright => self.contains(LonghandId::MozOutlineRadiusBottomright),
                    AnimatableLonghand::MozOutlineRadiusBottomleft => self.contains(LonghandId::MozOutlineRadiusBottomleft),
                    AnimatableLonghand::OutlineOffset => self.contains(LonghandId::OutlineOffset),
                    AnimatableLonghand::PaddingTop => self.contains(LonghandId::PaddingTop),
                    AnimatableLonghand::PaddingLeft => self.contains(LonghandId::PaddingLeft),
                    AnimatableLonghand::PaddingBottom => self.contains(LonghandId::PaddingBottom),
                    AnimatableLonghand::PaddingRight => self.contains(LonghandId::PaddingRight),
                    AnimatableLonghand::Cursor => self.contains(LonghandId::Cursor),
                    AnimatableLonghand::PointerEvents => self.contains(LonghandId::PointerEvents),
                    AnimatableLonghand::MozUserInput => self.contains(LonghandId::MozUserInput),
                    AnimatableLonghand::MozUserModify => self.contains(LonghandId::MozUserModify),
                    AnimatableLonghand::MozUserFocus => self.contains(LonghandId::MozUserFocus),
                    AnimatableLonghand::CaretColor => self.contains(LonghandId::CaretColor),
                    AnimatableLonghand::Top => self.contains(LonghandId::Top),
                    AnimatableLonghand::Left => self.contains(LonghandId::Left),
                    AnimatableLonghand::Bottom => self.contains(LonghandId::Bottom),
                    AnimatableLonghand::Right => self.contains(LonghandId::Right),
                    AnimatableLonghand::ZIndex => self.contains(LonghandId::ZIndex),
                    AnimatableLonghand::FlexDirection => self.contains(LonghandId::FlexDirection),
                    AnimatableLonghand::FlexWrap => self.contains(LonghandId::FlexWrap),
                    AnimatableLonghand::JustifyContent => self.contains(LonghandId::JustifyContent),
                    AnimatableLonghand::AlignContent => self.contains(LonghandId::AlignContent),
                    AnimatableLonghand::AlignItems => self.contains(LonghandId::AlignItems),
                    AnimatableLonghand::JustifyItems => self.contains(LonghandId::JustifyItems),
                    AnimatableLonghand::FlexGrow => self.contains(LonghandId::FlexGrow),
                    AnimatableLonghand::FlexShrink => self.contains(LonghandId::FlexShrink),
                    AnimatableLonghand::AlignSelf => self.contains(LonghandId::AlignSelf),
                    AnimatableLonghand::JustifySelf => self.contains(LonghandId::JustifySelf),
                    AnimatableLonghand::Order => self.contains(LonghandId::Order),
                    AnimatableLonghand::FlexBasis => self.contains(LonghandId::FlexBasis),
                    AnimatableLonghand::Width => self.contains(LonghandId::Width),
                    AnimatableLonghand::MinWidth => self.contains(LonghandId::MinWidth),
                    AnimatableLonghand::MaxWidth => self.contains(LonghandId::MaxWidth),
                    AnimatableLonghand::Height => self.contains(LonghandId::Height),
                    AnimatableLonghand::MinHeight => self.contains(LonghandId::MinHeight),
                    AnimatableLonghand::MaxHeight => self.contains(LonghandId::MaxHeight),
                    AnimatableLonghand::BoxSizing => self.contains(LonghandId::BoxSizing),
                    AnimatableLonghand::ObjectFit => self.contains(LonghandId::ObjectFit),
                    AnimatableLonghand::ObjectPosition => self.contains(LonghandId::ObjectPosition),
                    AnimatableLonghand::GridRowGap => self.contains(LonghandId::GridRowGap),
                    AnimatableLonghand::GridRowStart => self.contains(LonghandId::GridRowStart),
                    AnimatableLonghand::GridRowEnd => self.contains(LonghandId::GridRowEnd),
                    AnimatableLonghand::GridAutoRows => self.contains(LonghandId::GridAutoRows),
                    AnimatableLonghand::GridTemplateRows => self.contains(LonghandId::GridTemplateRows),
                    AnimatableLonghand::GridColumnGap => self.contains(LonghandId::GridColumnGap),
                    AnimatableLonghand::GridColumnStart => self.contains(LonghandId::GridColumnStart),
                    AnimatableLonghand::GridColumnEnd => self.contains(LonghandId::GridColumnEnd),
                    AnimatableLonghand::GridAutoColumns => self.contains(LonghandId::GridAutoColumns),
                    AnimatableLonghand::GridTemplateColumns => self.contains(LonghandId::GridTemplateColumns),
                    AnimatableLonghand::GridAutoFlow => self.contains(LonghandId::GridAutoFlow),
                    AnimatableLonghand::GridTemplateAreas => self.contains(LonghandId::GridTemplateAreas),
                    AnimatableLonghand::TableLayout => self.contains(LonghandId::TableLayout),
                    AnimatableLonghand::TextOverflow => self.contains(LonghandId::TextOverflow),
                    AnimatableLonghand::UnicodeBidi => self.contains(LonghandId::UnicodeBidi),
                    AnimatableLonghand::TextDecorationLine => self.contains(LonghandId::TextDecorationLine),
                    AnimatableLonghand::TextDecorationStyle => self.contains(LonghandId::TextDecorationStyle),
                    AnimatableLonghand::TextDecorationColor => self.contains(LonghandId::TextDecorationColor),
                    AnimatableLonghand::InitialLetter => self.contains(LonghandId::InitialLetter),
                    AnimatableLonghand::ImeMode => self.contains(LonghandId::ImeMode),
                    AnimatableLonghand::MozUserSelect => self.contains(LonghandId::MozUserSelect),
                    AnimatableLonghand::MozWindowDragging => self.contains(LonghandId::MozWindowDragging),
                    AnimatableLonghand::MozWindowShadow => self.contains(LonghandId::MozWindowShadow),
                    AnimatableLonghand::MozForceBrokenImageIcon => self.contains(LonghandId::MozForceBrokenImageIcon),
                    AnimatableLonghand::TextAnchor => self.contains(LonghandId::TextAnchor),
                    AnimatableLonghand::ColorInterpolation => self.contains(LonghandId::ColorInterpolation),
                    AnimatableLonghand::ColorInterpolationFilters => self.contains(LonghandId::ColorInterpolationFilters),
                    AnimatableLonghand::Fill => self.contains(LonghandId::Fill),
                    AnimatableLonghand::FillOpacity => self.contains(LonghandId::FillOpacity),
                    AnimatableLonghand::FillRule => self.contains(LonghandId::FillRule),
                    AnimatableLonghand::ShapeRendering => self.contains(LonghandId::ShapeRendering),
                    AnimatableLonghand::Stroke => self.contains(LonghandId::Stroke),
                    AnimatableLonghand::StrokeWidth => self.contains(LonghandId::StrokeWidth),
                    AnimatableLonghand::StrokeLinecap => self.contains(LonghandId::StrokeLinecap),
                    AnimatableLonghand::StrokeLinejoin => self.contains(LonghandId::StrokeLinejoin),
                    AnimatableLonghand::StrokeMiterlimit => self.contains(LonghandId::StrokeMiterlimit),
                    AnimatableLonghand::StrokeOpacity => self.contains(LonghandId::StrokeOpacity),
                    AnimatableLonghand::StrokeDasharray => self.contains(LonghandId::StrokeDasharray),
                    AnimatableLonghand::StrokeDashoffset => self.contains(LonghandId::StrokeDashoffset),
                    AnimatableLonghand::ClipRule => self.contains(LonghandId::ClipRule),
                    AnimatableLonghand::MarkerStart => self.contains(LonghandId::MarkerStart),
                    AnimatableLonghand::MarkerMid => self.contains(LonghandId::MarkerMid),
                    AnimatableLonghand::MarkerEnd => self.contains(LonghandId::MarkerEnd),
                    AnimatableLonghand::PaintOrder => self.contains(LonghandId::PaintOrder),
                    AnimatableLonghand::DominantBaseline => self.contains(LonghandId::DominantBaseline),
                    AnimatableLonghand::VectorEffect => self.contains(LonghandId::VectorEffect),
                    AnimatableLonghand::StopColor => self.contains(LonghandId::StopColor),
                    AnimatableLonghand::StopOpacity => self.contains(LonghandId::StopOpacity),
                    AnimatableLonghand::FloodColor => self.contains(LonghandId::FloodColor),
                    AnimatableLonghand::FloodOpacity => self.contains(LonghandId::FloodOpacity),
                    AnimatableLonghand::LightingColor => self.contains(LonghandId::LightingColor),
                    AnimatableLonghand::MaskType => self.contains(LonghandId::MaskType),
                    AnimatableLonghand::ClipPath => self.contains(LonghandId::ClipPath),
                    AnimatableLonghand::MaskMode => self.contains(LonghandId::MaskMode),
                    AnimatableLonghand::MaskRepeat => self.contains(LonghandId::MaskRepeat),
                    AnimatableLonghand::MaskPositionX => self.contains(LonghandId::MaskPositionX),
                    AnimatableLonghand::MaskPositionY => self.contains(LonghandId::MaskPositionY),
                    AnimatableLonghand::MaskClip => self.contains(LonghandId::MaskClip),
                    AnimatableLonghand::MaskOrigin => self.contains(LonghandId::MaskOrigin),
                    AnimatableLonghand::MaskSize => self.contains(LonghandId::MaskSize),
                    AnimatableLonghand::MaskComposite => self.contains(LonghandId::MaskComposite),
                    AnimatableLonghand::MaskImage => self.contains(LonghandId::MaskImage),
                    AnimatableLonghand::MozBoxAlign => self.contains(LonghandId::MozBoxAlign),
                    AnimatableLonghand::MozBoxDirection => self.contains(LonghandId::MozBoxDirection),
                    AnimatableLonghand::MozBoxFlex => self.contains(LonghandId::MozBoxFlex),
                    AnimatableLonghand::MozBoxOrient => self.contains(LonghandId::MozBoxOrient),
                    AnimatableLonghand::MozBoxPack => self.contains(LonghandId::MozBoxPack),
                    AnimatableLonghand::MozStackSizing => self.contains(LonghandId::MozStackSizing),
                    AnimatableLonghand::MozBoxOrdinalGroup => self.contains(LonghandId::MozBoxOrdinalGroup),
        }
    }
}

/// A specialized set of PropertyDeclarationId
pub struct PropertyDeclarationIdSet {
    longhands: LonghandIdSet,

    // FIXME: Use a HashSet instead? This Vec is usually small, so linear scan might be ok.
    custom: Vec<::custom_properties::Name>,
}

impl PropertyDeclarationIdSet {
    /// Empty set
    pub fn new() -> Self {
        PropertyDeclarationIdSet {
            longhands: LonghandIdSet::new(),
            custom: Vec::new(),
        }
    }

    /// Returns whether the given ID is in the set
    pub fn contains(&mut self, id: PropertyDeclarationId) -> bool {
        match id {
            PropertyDeclarationId::Longhand(id) => self.longhands.contains(id),
            PropertyDeclarationId::Custom(name) => self.custom.contains(name),
        }
    }

    /// Insert the given ID in the set
    pub fn insert(&mut self, id: PropertyDeclarationId) {
        match id {
            PropertyDeclarationId::Longhand(id) => self.longhands.insert(id),
            PropertyDeclarationId::Custom(name) => {
                if !self.custom.contains(name) {
                    self.custom.push(name.clone())
                }
            }
        }
    }
}

/// An enum to represent a CSS Wide keyword.
#[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
#[cfg_attr(feature = "servo", derive(HeapSizeOf))]
#[derive(Clone, Copy, Debug, Eq, PartialEq, ToCss)]
pub enum CSSWideKeyword {
    /// The `initial` keyword.
    Initial,
    /// The `inherit` keyword.
    Inherit,
    /// The `unset` keyword.
    Unset,
}

impl CSSWideKeyword {
    fn to_str(&self) -> &'static str {
        match *self {
            CSSWideKeyword::Initial => "initial",
            CSSWideKeyword::Inherit => "inherit",
            CSSWideKeyword::Unset => "unset",
        }
    }

    /// Takes the result of cssparser::Parser::expect_ident() and converts it
    /// to a CSSWideKeyword.
    pub fn from_ident<'i>(ident: &str) -> Option<Self> {
        match_ignore_ascii_case! { ident,
            // If modifying this set of keyword, also update values::CustomIdent::from_ident
            "initial" => Some(CSSWideKeyword::Initial),
            "inherit" => Some(CSSWideKeyword::Inherit),
            "unset" => Some(CSSWideKeyword::Unset),
            _ => None
        }
    }
}

impl CSSWideKeyword {
    fn parse(input: &mut Parser) -> Result<Self, ()> {
        let ident = input.expect_ident().map_err(|_| ())?.clone();
        input.expect_exhausted().map_err(|_| ())?;
        CSSWideKeyword::from_ident(&ident).ok_or(())
    }
}

bitflags! {
    /// A set of flags for properties.
    pub flags PropertyFlags: u8 {
        /// This property requires a stacking context.
        const CREATES_STACKING_CONTEXT = 1 << 0,
        /// This property has values that can establish a containing block for
        /// fixed positioned and absolutely positioned elements.
        const FIXPOS_CB = 1 << 1,
        /// This property has values that can establish a containing block for
        /// absolutely positioned elements.
        const ABSPOS_CB = 1 << 2,
        /// This shorthand property is an alias of another property.
        const SHORTHAND_ALIAS_PROPERTY = 1 << 3,
        /// This longhand property applies to ::first-letter.
        const APPLIES_TO_FIRST_LETTER = 1 << 4,
        /// This longhand property applies to ::first-line.
        const APPLIES_TO_FIRST_LINE = 1 << 5,
        /// This longhand property applies to ::placeholder.
        const APPLIES_TO_PLACEHOLDER = 1 << 6,
    }
}

/// An identifier for a given longhand property.
#[derive(Clone, Copy, Eq, PartialEq)]
#[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
#[cfg_attr(feature = "servo", derive(HeapSizeOf))]
pub enum LonghandId {
        /// background-color
        BackgroundColor = 0,
        /// background-image
        BackgroundImage = 1,
        /// background-position-x
        BackgroundPositionX = 2,
        /// background-position-y
        BackgroundPositionY = 3,
        /// background-repeat
        BackgroundRepeat = 4,
        /// background-attachment
        BackgroundAttachment = 5,
        /// background-clip
        BackgroundClip = 6,
        /// background-origin
        BackgroundOrigin = 7,
        /// background-size
        BackgroundSize = 8,
        /// background-blend-mode
        BackgroundBlendMode = 9,
        /// border-top-color
        BorderTopColor = 10,
        /// border-top-style
        BorderTopStyle = 11,
        /// border-top-width
        BorderTopWidth = 12,
        /// border-left-color
        BorderLeftColor = 13,
        /// border-left-style
        BorderLeftStyle = 14,
        /// border-left-width
        BorderLeftWidth = 15,
        /// border-bottom-color
        BorderBottomColor = 16,
        /// border-bottom-style
        BorderBottomStyle = 17,
        /// border-bottom-width
        BorderBottomWidth = 18,
        /// border-right-color
        BorderRightColor = 19,
        /// border-right-style
        BorderRightStyle = 20,
        /// border-right-width
        BorderRightWidth = 21,
        /// border-block-start-color
        BorderBlockStartColor = 22,
        /// border-block-start-style
        BorderBlockStartStyle = 23,
        /// border-block-start-width
        BorderBlockStartWidth = 24,
        /// border-block-end-color
        BorderBlockEndColor = 25,
        /// border-block-end-style
        BorderBlockEndStyle = 26,
        /// border-block-end-width
        BorderBlockEndWidth = 27,
        /// border-inline-start-color
        BorderInlineStartColor = 28,
        /// border-inline-start-style
        BorderInlineStartStyle = 29,
        /// border-inline-start-width
        BorderInlineStartWidth = 30,
        /// border-inline-end-color
        BorderInlineEndColor = 31,
        /// border-inline-end-style
        BorderInlineEndStyle = 32,
        /// border-inline-end-width
        BorderInlineEndWidth = 33,
        /// border-top-left-radius
        BorderTopLeftRadius = 34,
        /// border-top-right-radius
        BorderTopRightRadius = 35,
        /// border-bottom-right-radius
        BorderBottomRightRadius = 36,
        /// border-bottom-left-radius
        BorderBottomLeftRadius = 37,
        /// -moz-border-top-colors
        MozBorderTopColors = 38,
        /// -moz-border-left-colors
        MozBorderLeftColors = 39,
        /// -moz-border-bottom-colors
        MozBorderBottomColors = 40,
        /// -moz-border-right-colors
        MozBorderRightColors = 41,
        /// box-decoration-break
        BoxDecorationBreak = 42,
        /// -moz-float-edge
        MozFloatEdge = 43,
        /// border-image-source
        BorderImageSource = 44,
        /// border-image-outset
        BorderImageOutset = 45,
        /// border-image-repeat
        BorderImageRepeat = 46,
        /// border-image-width
        BorderImageWidth = 47,
        /// border-image-slice
        BorderImageSlice = 48,
        /// display
        Display = 49,
        /// -moz-top-layer
        MozTopLayer = 50,
        /// position
        Position = 51,
        /// float
        Float = 52,
        /// clear
        Clear = 53,
        /// vertical-align
        VerticalAlign = 54,
        /// overflow-clip-box
        OverflowClipBox = 55,
        /// overflow-x
        OverflowX = 56,
        /// overflow-y
        OverflowY = 57,
        /// transition-duration
        TransitionDuration = 58,
        /// transition-timing-function
        TransitionTimingFunction = 59,
        /// transition-property
        TransitionProperty = 60,
        /// transition-delay
        TransitionDelay = 61,
        /// animation-name
        AnimationName = 62,
        /// animation-duration
        AnimationDuration = 63,
        /// animation-timing-function
        AnimationTimingFunction = 64,
        /// animation-iteration-count
        AnimationIterationCount = 65,
        /// animation-direction
        AnimationDirection = 66,
        /// animation-play-state
        AnimationPlayState = 67,
        /// animation-fill-mode
        AnimationFillMode = 68,
        /// animation-delay
        AnimationDelay = 69,
        /// scroll-snap-points-x
        ScrollSnapPointsX = 70,
        /// scroll-snap-points-y
        ScrollSnapPointsY = 71,
        /// scroll-snap-destination
        ScrollSnapDestination = 72,
        /// scroll-snap-coordinate
        ScrollSnapCoordinate = 73,
        /// transform
        Transform = 74,
        /// scroll-behavior
        ScrollBehavior = 75,
        /// scroll-snap-type-x
        ScrollSnapTypeX = 76,
        /// scroll-snap-type-y
        ScrollSnapTypeY = 77,
        /// isolation
        Isolation = 78,
        /// page-break-after
        PageBreakAfter = 79,
        /// page-break-before
        PageBreakBefore = 80,
        /// page-break-inside
        PageBreakInside = 81,
        /// resize
        Resize = 82,
        /// perspective
        Perspective = 83,
        /// perspective-origin
        PerspectiveOrigin = 84,
        /// backface-visibility
        BackfaceVisibility = 85,
        /// transform-box
        TransformBox = 86,
        /// transform-style
        TransformStyle = 87,
        /// transform-origin
        TransformOrigin = 88,
        /// contain
        Contain = 89,
        /// -moz-appearance
        MozAppearance = 90,
        /// -moz-binding
        MozBinding = 91,
        /// -moz-orient
        MozOrient = 92,
        /// will-change
        WillChange = 93,
        /// shape-outside
        ShapeOutside = 94,
        /// touch-action
        TouchAction = 95,
        /// color
        Color = 96,
        /// column-width
        ColumnWidth = 97,
        /// column-count
        ColumnCount = 98,
        /// column-gap
        ColumnGap = 99,
        /// column-fill
        ColumnFill = 100,
        /// column-rule-width
        ColumnRuleWidth = 101,
        /// column-rule-color
        ColumnRuleColor = 102,
        /// column-span
        ColumnSpan = 103,
        /// column-rule-style
        ColumnRuleStyle = 104,
        /// content
        Content = 105,
        /// counter-increment
        CounterIncrement = 106,
        /// counter-reset
        CounterReset = 107,
        /// opacity
        Opacity = 108,
        /// box-shadow
        BoxShadow = 109,
        /// clip
        Clip = 110,
        /// filter
        Filter = 111,
        /// mix-blend-mode
        MixBlendMode = 112,
        /// font-family
        FontFamily = 113,
        /// font-style
        FontStyle = 114,
        /// font-variant-caps
        FontVariantCaps = 115,
        /// font-weight
        FontWeight = 116,
        /// font-size
        FontSize = 117,
        /// font-size-adjust
        FontSizeAdjust = 118,
        /// font-synthesis
        FontSynthesis = 119,
        /// font-stretch
        FontStretch = 120,
        /// font-kerning
        FontKerning = 121,
        /// font-variant-alternates
        FontVariantAlternates = 122,
        /// font-variant-east-asian
        FontVariantEastAsian = 123,
        /// font-variant-ligatures
        FontVariantLigatures = 124,
        /// font-variant-numeric
        FontVariantNumeric = 125,
        /// font-variant-position
        FontVariantPosition = 126,
        /// font-feature-settings
        FontFeatureSettings = 127,
        /// font-variation-settings
        FontVariationSettings = 128,
        /// font-language-override
        FontLanguageOverride = 129,
        /// -x-lang
        XLang = 130,
        /// -moz-script-size-multiplier
        MozScriptSizeMultiplier = 131,
        /// -moz-script-level
        MozScriptLevel = 132,
        /// -moz-math-display
        MozMathDisplay = 133,
        /// -moz-math-variant
        MozMathVariant = 134,
        /// -moz-script-min-size
        MozScriptMinSize = 135,
        /// -x-text-zoom
        XTextZoom = 136,
        /// -moz-osx-font-smoothing
        MozOsxFontSmoothing = 137,
        /// -moz-min-font-size-ratio
        MozMinFontSizeRatio = 138,
        /// visibility
        Visibility = 139,
        /// writing-mode
        WritingMode = 140,
        /// direction
        Direction = 141,
        /// text-orientation
        TextOrientation = 142,
        /// color-adjust
        ColorAdjust = 143,
        /// image-rendering
        ImageRendering = 144,
        /// image-orientation
        ImageOrientation = 145,
        /// border-collapse
        BorderCollapse = 146,
        /// empty-cells
        EmptyCells = 147,
        /// caption-side
        CaptionSide = 148,
        /// border-spacing
        BorderSpacing = 149,
        /// line-height
        LineHeight = 150,
        /// text-transform
        TextTransform = 151,
        /// hyphens
        Hyphens = 152,
        /// -moz-text-size-adjust
        MozTextSizeAdjust = 153,
        /// text-indent
        TextIndent = 154,
        /// overflow-wrap
        OverflowWrap = 155,
        /// word-break
        WordBreak = 156,
        /// text-justify
        TextJustify = 157,
        /// text-align-last
        TextAlignLast = 158,
        /// text-align
        TextAlign = 159,
        /// letter-spacing
        LetterSpacing = 160,
        /// word-spacing
        WordSpacing = 161,
        /// white-space
        WhiteSpace = 162,
        /// text-shadow
        TextShadow = 163,
        /// text-emphasis-style
        TextEmphasisStyle = 164,
        /// text-emphasis-position
        TextEmphasisPosition = 165,
        /// text-emphasis-color
        TextEmphasisColor = 166,
        /// -moz-tab-size
        MozTabSize = 167,
        /// -webkit-text-fill-color
        WebkitTextFillColor = 168,
        /// -webkit-text-stroke-color
        WebkitTextStrokeColor = 169,
        /// -webkit-text-stroke-width
        WebkitTextStrokeWidth = 170,
        /// ruby-align
        RubyAlign = 171,
        /// ruby-position
        RubyPosition = 172,
        /// text-combine-upright
        TextCombineUpright = 173,
        /// text-rendering
        TextRendering = 174,
        /// -moz-control-character-visibility
        MozControlCharacterVisibility = 175,
        /// list-style-position
        ListStylePosition = 176,
        /// list-style-type
        ListStyleType = 177,
        /// list-style-image
        ListStyleImage = 178,
        /// quotes
        Quotes = 179,
        /// -moz-image-region
        MozImageRegion = 180,
        /// margin-top
        MarginTop = 181,
        /// margin-left
        MarginLeft = 182,
        /// margin-bottom
        MarginBottom = 183,
        /// margin-right
        MarginRight = 184,
        /// margin-block-start
        MarginBlockStart = 185,
        /// margin-block-end
        MarginBlockEnd = 186,
        /// margin-inline-start
        MarginInlineStart = 187,
        /// margin-inline-end
        MarginInlineEnd = 188,
        /// outline-color
        OutlineColor = 189,
        /// outline-style
        OutlineStyle = 190,
        /// outline-width
        OutlineWidth = 191,
        /// -moz-outline-radius-topleft
        MozOutlineRadiusTopleft = 192,
        /// -moz-outline-radius-topright
        MozOutlineRadiusTopright = 193,
        /// -moz-outline-radius-bottomright
        MozOutlineRadiusBottomright = 194,
        /// -moz-outline-radius-bottomleft
        MozOutlineRadiusBottomleft = 195,
        /// outline-offset
        OutlineOffset = 196,
        /// padding-top
        PaddingTop = 197,
        /// padding-left
        PaddingLeft = 198,
        /// padding-bottom
        PaddingBottom = 199,
        /// padding-right
        PaddingRight = 200,
        /// padding-block-start
        PaddingBlockStart = 201,
        /// padding-block-end
        PaddingBlockEnd = 202,
        /// padding-inline-start
        PaddingInlineStart = 203,
        /// padding-inline-end
        PaddingInlineEnd = 204,
        /// cursor
        Cursor = 205,
        /// pointer-events
        PointerEvents = 206,
        /// -moz-user-input
        MozUserInput = 207,
        /// -moz-user-modify
        MozUserModify = 208,
        /// -moz-user-focus
        MozUserFocus = 209,
        /// caret-color
        CaretColor = 210,
        /// top
        Top = 211,
        /// left
        Left = 212,
        /// bottom
        Bottom = 213,
        /// right
        Right = 214,
        /// offset-block-start
        OffsetBlockStart = 215,
        /// offset-block-end
        OffsetBlockEnd = 216,
        /// offset-inline-start
        OffsetInlineStart = 217,
        /// offset-inline-end
        OffsetInlineEnd = 218,
        /// z-index
        ZIndex = 219,
        /// flex-direction
        FlexDirection = 220,
        /// flex-wrap
        FlexWrap = 221,
        /// justify-content
        JustifyContent = 222,
        /// align-content
        AlignContent = 223,
        /// align-items
        AlignItems = 224,
        /// justify-items
        JustifyItems = 225,
        /// flex-grow
        FlexGrow = 226,
        /// flex-shrink
        FlexShrink = 227,
        /// align-self
        AlignSelf = 228,
        /// justify-self
        JustifySelf = 229,
        /// order
        Order = 230,
        /// flex-basis
        FlexBasis = 231,
        /// width
        Width = 232,
        /// min-width
        MinWidth = 233,
        /// max-width
        MaxWidth = 234,
        /// height
        Height = 235,
        /// min-height
        MinHeight = 236,
        /// max-height
        MaxHeight = 237,
        /// block-size
        BlockSize = 238,
        /// min-block-size
        MinBlockSize = 239,
        /// max-block-size
        MaxBlockSize = 240,
        /// inline-size
        InlineSize = 241,
        /// min-inline-size
        MinInlineSize = 242,
        /// max-inline-size
        MaxInlineSize = 243,
        /// box-sizing
        BoxSizing = 244,
        /// object-fit
        ObjectFit = 245,
        /// object-position
        ObjectPosition = 246,
        /// grid-row-gap
        GridRowGap = 247,
        /// grid-row-start
        GridRowStart = 248,
        /// grid-row-end
        GridRowEnd = 249,
        /// grid-auto-rows
        GridAutoRows = 250,
        /// grid-template-rows
        GridTemplateRows = 251,
        /// grid-column-gap
        GridColumnGap = 252,
        /// grid-column-start
        GridColumnStart = 253,
        /// grid-column-end
        GridColumnEnd = 254,
        /// grid-auto-columns
        GridAutoColumns = 255,
        /// grid-template-columns
        GridTemplateColumns = 256,
        /// grid-auto-flow
        GridAutoFlow = 257,
        /// grid-template-areas
        GridTemplateAreas = 258,
        /// table-layout
        TableLayout = 259,
        /// -x-span
        XSpan = 260,
        /// text-overflow
        TextOverflow = 261,
        /// unicode-bidi
        UnicodeBidi = 262,
        /// text-decoration-line
        TextDecorationLine = 263,
        /// text-decoration-style
        TextDecorationStyle = 264,
        /// text-decoration-color
        TextDecorationColor = 265,
        /// initial-letter
        InitialLetter = 266,
        /// ime-mode
        ImeMode = 267,
        /// -moz-user-select
        MozUserSelect = 268,
        /// -moz-window-dragging
        MozWindowDragging = 269,
        /// -moz-window-shadow
        MozWindowShadow = 270,
        /// -moz-force-broken-image-icon
        MozForceBrokenImageIcon = 271,
        /// text-anchor
        TextAnchor = 272,
        /// color-interpolation
        ColorInterpolation = 273,
        /// color-interpolation-filters
        ColorInterpolationFilters = 274,
        /// fill
        Fill = 275,
        /// fill-opacity
        FillOpacity = 276,
        /// fill-rule
        FillRule = 277,
        /// shape-rendering
        ShapeRendering = 278,
        /// stroke
        Stroke = 279,
        /// stroke-width
        StrokeWidth = 280,
        /// stroke-linecap
        StrokeLinecap = 281,
        /// stroke-linejoin
        StrokeLinejoin = 282,
        /// stroke-miterlimit
        StrokeMiterlimit = 283,
        /// stroke-opacity
        StrokeOpacity = 284,
        /// stroke-dasharray
        StrokeDasharray = 285,
        /// stroke-dashoffset
        StrokeDashoffset = 286,
        /// clip-rule
        ClipRule = 287,
        /// marker-start
        MarkerStart = 288,
        /// marker-mid
        MarkerMid = 289,
        /// marker-end
        MarkerEnd = 290,
        /// paint-order
        PaintOrder = 291,
        /// -moz-context-properties
        MozContextProperties = 292,
        /// dominant-baseline
        DominantBaseline = 293,
        /// vector-effect
        VectorEffect = 294,
        /// stop-color
        StopColor = 295,
        /// stop-opacity
        StopOpacity = 296,
        /// flood-color
        FloodColor = 297,
        /// flood-opacity
        FloodOpacity = 298,
        /// lighting-color
        LightingColor = 299,
        /// mask-type
        MaskType = 300,
        /// clip-path
        ClipPath = 301,
        /// mask-mode
        MaskMode = 302,
        /// mask-repeat
        MaskRepeat = 303,
        /// mask-position-x
        MaskPositionX = 304,
        /// mask-position-y
        MaskPositionY = 305,
        /// mask-clip
        MaskClip = 306,
        /// mask-origin
        MaskOrigin = 307,
        /// mask-size
        MaskSize = 308,
        /// mask-composite
        MaskComposite = 309,
        /// mask-image
        MaskImage = 310,
        /// -moz-box-align
        MozBoxAlign = 311,
        /// -moz-box-direction
        MozBoxDirection = 312,
        /// -moz-box-flex
        MozBoxFlex = 313,
        /// -moz-box-orient
        MozBoxOrient = 314,
        /// -moz-box-pack
        MozBoxPack = 315,
        /// -moz-stack-sizing
        MozStackSizing = 316,
        /// -moz-box-ordinal-group
        MozBoxOrdinalGroup = 317,
}

impl fmt::Debug for LonghandId {
    fn fmt(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
        formatter.write_str(self.name())
    }
}

impl LonghandId {
    /// Get the name of this longhand property.
    pub fn name(&self) -> &'static str {
        match *self {
                LonghandId::BackgroundColor => "background-color",
                LonghandId::BackgroundImage => "background-image",
                LonghandId::BackgroundPositionX => "background-position-x",
                LonghandId::BackgroundPositionY => "background-position-y",
                LonghandId::BackgroundRepeat => "background-repeat",
                LonghandId::BackgroundAttachment => "background-attachment",
                LonghandId::BackgroundClip => "background-clip",
                LonghandId::BackgroundOrigin => "background-origin",
                LonghandId::BackgroundSize => "background-size",
                LonghandId::BackgroundBlendMode => "background-blend-mode",
                LonghandId::BorderTopColor => "border-top-color",
                LonghandId::BorderTopStyle => "border-top-style",
                LonghandId::BorderTopWidth => "border-top-width",
                LonghandId::BorderLeftColor => "border-left-color",
                LonghandId::BorderLeftStyle => "border-left-style",
                LonghandId::BorderLeftWidth => "border-left-width",
                LonghandId::BorderBottomColor => "border-bottom-color",
                LonghandId::BorderBottomStyle => "border-bottom-style",
                LonghandId::BorderBottomWidth => "border-bottom-width",
                LonghandId::BorderRightColor => "border-right-color",
                LonghandId::BorderRightStyle => "border-right-style",
                LonghandId::BorderRightWidth => "border-right-width",
                LonghandId::BorderBlockStartColor => "border-block-start-color",
                LonghandId::BorderBlockStartStyle => "border-block-start-style",
                LonghandId::BorderBlockStartWidth => "border-block-start-width",
                LonghandId::BorderBlockEndColor => "border-block-end-color",
                LonghandId::BorderBlockEndStyle => "border-block-end-style",
                LonghandId::BorderBlockEndWidth => "border-block-end-width",
                LonghandId::BorderInlineStartColor => "border-inline-start-color",
                LonghandId::BorderInlineStartStyle => "border-inline-start-style",
                LonghandId::BorderInlineStartWidth => "border-inline-start-width",
                LonghandId::BorderInlineEndColor => "border-inline-end-color",
                LonghandId::BorderInlineEndStyle => "border-inline-end-style",
                LonghandId::BorderInlineEndWidth => "border-inline-end-width",
                LonghandId::BorderTopLeftRadius => "border-top-left-radius",
                LonghandId::BorderTopRightRadius => "border-top-right-radius",
                LonghandId::BorderBottomRightRadius => "border-bottom-right-radius",
                LonghandId::BorderBottomLeftRadius => "border-bottom-left-radius",
                LonghandId::MozBorderTopColors => "-moz-border-top-colors",
                LonghandId::MozBorderLeftColors => "-moz-border-left-colors",
                LonghandId::MozBorderBottomColors => "-moz-border-bottom-colors",
                LonghandId::MozBorderRightColors => "-moz-border-right-colors",
                LonghandId::BoxDecorationBreak => "box-decoration-break",
                LonghandId::MozFloatEdge => "-moz-float-edge",
                LonghandId::BorderImageSource => "border-image-source",
                LonghandId::BorderImageOutset => "border-image-outset",
                LonghandId::BorderImageRepeat => "border-image-repeat",
                LonghandId::BorderImageWidth => "border-image-width",
                LonghandId::BorderImageSlice => "border-image-slice",
                LonghandId::Display => "display",
                LonghandId::MozTopLayer => "-moz-top-layer",
                LonghandId::Position => "position",
                LonghandId::Float => "float",
                LonghandId::Clear => "clear",
                LonghandId::VerticalAlign => "vertical-align",
                LonghandId::OverflowClipBox => "overflow-clip-box",
                LonghandId::OverflowX => "overflow-x",
                LonghandId::OverflowY => "overflow-y",
                LonghandId::TransitionDuration => "transition-duration",
                LonghandId::TransitionTimingFunction => "transition-timing-function",
                LonghandId::TransitionProperty => "transition-property",
                LonghandId::TransitionDelay => "transition-delay",
                LonghandId::AnimationName => "animation-name",
                LonghandId::AnimationDuration => "animation-duration",
                LonghandId::AnimationTimingFunction => "animation-timing-function",
                LonghandId::AnimationIterationCount => "animation-iteration-count",
                LonghandId::AnimationDirection => "animation-direction",
                LonghandId::AnimationPlayState => "animation-play-state",
                LonghandId::AnimationFillMode => "animation-fill-mode",
                LonghandId::AnimationDelay => "animation-delay",
                LonghandId::ScrollSnapPointsX => "scroll-snap-points-x",
                LonghandId::ScrollSnapPointsY => "scroll-snap-points-y",
                LonghandId::ScrollSnapDestination => "scroll-snap-destination",
                LonghandId::ScrollSnapCoordinate => "scroll-snap-coordinate",
                LonghandId::Transform => "transform",
                LonghandId::ScrollBehavior => "scroll-behavior",
                LonghandId::ScrollSnapTypeX => "scroll-snap-type-x",
                LonghandId::ScrollSnapTypeY => "scroll-snap-type-y",
                LonghandId::Isolation => "isolation",
                LonghandId::PageBreakAfter => "page-break-after",
                LonghandId::PageBreakBefore => "page-break-before",
                LonghandId::PageBreakInside => "page-break-inside",
                LonghandId::Resize => "resize",
                LonghandId::Perspective => "perspective",
                LonghandId::PerspectiveOrigin => "perspective-origin",
                LonghandId::BackfaceVisibility => "backface-visibility",
                LonghandId::TransformBox => "transform-box",
                LonghandId::TransformStyle => "transform-style",
                LonghandId::TransformOrigin => "transform-origin",
                LonghandId::Contain => "contain",
                LonghandId::MozAppearance => "-moz-appearance",
                LonghandId::MozBinding => "-moz-binding",
                LonghandId::MozOrient => "-moz-orient",
                LonghandId::WillChange => "will-change",
                LonghandId::ShapeOutside => "shape-outside",
                LonghandId::TouchAction => "touch-action",
                LonghandId::Color => "color",
                LonghandId::ColumnWidth => "column-width",
                LonghandId::ColumnCount => "column-count",
                LonghandId::ColumnGap => "column-gap",
                LonghandId::ColumnFill => "column-fill",
                LonghandId::ColumnRuleWidth => "column-rule-width",
                LonghandId::ColumnRuleColor => "column-rule-color",
                LonghandId::ColumnSpan => "column-span",
                LonghandId::ColumnRuleStyle => "column-rule-style",
                LonghandId::Content => "content",
                LonghandId::CounterIncrement => "counter-increment",
                LonghandId::CounterReset => "counter-reset",
                LonghandId::Opacity => "opacity",
                LonghandId::BoxShadow => "box-shadow",
                LonghandId::Clip => "clip",
                LonghandId::Filter => "filter",
                LonghandId::MixBlendMode => "mix-blend-mode",
                LonghandId::FontFamily => "font-family",
                LonghandId::FontStyle => "font-style",
                LonghandId::FontVariantCaps => "font-variant-caps",
                LonghandId::FontWeight => "font-weight",
                LonghandId::FontSize => "font-size",
                LonghandId::FontSizeAdjust => "font-size-adjust",
                LonghandId::FontSynthesis => "font-synthesis",
                LonghandId::FontStretch => "font-stretch",
                LonghandId::FontKerning => "font-kerning",
                LonghandId::FontVariantAlternates => "font-variant-alternates",
                LonghandId::FontVariantEastAsian => "font-variant-east-asian",
                LonghandId::FontVariantLigatures => "font-variant-ligatures",
                LonghandId::FontVariantNumeric => "font-variant-numeric",
                LonghandId::FontVariantPosition => "font-variant-position",
                LonghandId::FontFeatureSettings => "font-feature-settings",
                LonghandId::FontVariationSettings => "font-variation-settings",
                LonghandId::FontLanguageOverride => "font-language-override",
                LonghandId::XLang => "-x-lang",
                LonghandId::MozScriptSizeMultiplier => "-moz-script-size-multiplier",
                LonghandId::MozScriptLevel => "-moz-script-level",
                LonghandId::MozMathDisplay => "-moz-math-display",
                LonghandId::MozMathVariant => "-moz-math-variant",
                LonghandId::MozScriptMinSize => "-moz-script-min-size",
                LonghandId::XTextZoom => "-x-text-zoom",
                LonghandId::MozOsxFontSmoothing => "-moz-osx-font-smoothing",
                LonghandId::MozMinFontSizeRatio => "-moz-min-font-size-ratio",
                LonghandId::Visibility => "visibility",
                LonghandId::WritingMode => "writing-mode",
                LonghandId::Direction => "direction",
                LonghandId::TextOrientation => "text-orientation",
                LonghandId::ColorAdjust => "color-adjust",
                LonghandId::ImageRendering => "image-rendering",
                LonghandId::ImageOrientation => "image-orientation",
                LonghandId::BorderCollapse => "border-collapse",
                LonghandId::EmptyCells => "empty-cells",
                LonghandId::CaptionSide => "caption-side",
                LonghandId::BorderSpacing => "border-spacing",
                LonghandId::LineHeight => "line-height",
                LonghandId::TextTransform => "text-transform",
                LonghandId::Hyphens => "hyphens",
                LonghandId::MozTextSizeAdjust => "-moz-text-size-adjust",
                LonghandId::TextIndent => "text-indent",
                LonghandId::OverflowWrap => "overflow-wrap",
                LonghandId::WordBreak => "word-break",
                LonghandId::TextJustify => "text-justify",
                LonghandId::TextAlignLast => "text-align-last",
                LonghandId::TextAlign => "text-align",
                LonghandId::LetterSpacing => "letter-spacing",
                LonghandId::WordSpacing => "word-spacing",
                LonghandId::WhiteSpace => "white-space",
                LonghandId::TextShadow => "text-shadow",
                LonghandId::TextEmphasisStyle => "text-emphasis-style",
                LonghandId::TextEmphasisPosition => "text-emphasis-position",
                LonghandId::TextEmphasisColor => "text-emphasis-color",
                LonghandId::MozTabSize => "-moz-tab-size",
                LonghandId::WebkitTextFillColor => "-webkit-text-fill-color",
                LonghandId::WebkitTextStrokeColor => "-webkit-text-stroke-color",
                LonghandId::WebkitTextStrokeWidth => "-webkit-text-stroke-width",
                LonghandId::RubyAlign => "ruby-align",
                LonghandId::RubyPosition => "ruby-position",
                LonghandId::TextCombineUpright => "text-combine-upright",
                LonghandId::TextRendering => "text-rendering",
                LonghandId::MozControlCharacterVisibility => "-moz-control-character-visibility",
                LonghandId::ListStylePosition => "list-style-position",
                LonghandId::ListStyleType => "list-style-type",
                LonghandId::ListStyleImage => "list-style-image",
                LonghandId::Quotes => "quotes",
                LonghandId::MozImageRegion => "-moz-image-region",
                LonghandId::MarginTop => "margin-top",
                LonghandId::MarginLeft => "margin-left",
                LonghandId::MarginBottom => "margin-bottom",
                LonghandId::MarginRight => "margin-right",
                LonghandId::MarginBlockStart => "margin-block-start",
                LonghandId::MarginBlockEnd => "margin-block-end",
                LonghandId::MarginInlineStart => "margin-inline-start",
                LonghandId::MarginInlineEnd => "margin-inline-end",
                LonghandId::OutlineColor => "outline-color",
                LonghandId::OutlineStyle => "outline-style",
                LonghandId::OutlineWidth => "outline-width",
                LonghandId::MozOutlineRadiusTopleft => "-moz-outline-radius-topleft",
                LonghandId::MozOutlineRadiusTopright => "-moz-outline-radius-topright",
                LonghandId::MozOutlineRadiusBottomright => "-moz-outline-radius-bottomright",
                LonghandId::MozOutlineRadiusBottomleft => "-moz-outline-radius-bottomleft",
                LonghandId::OutlineOffset => "outline-offset",
                LonghandId::PaddingTop => "padding-top",
                LonghandId::PaddingLeft => "padding-left",
                LonghandId::PaddingBottom => "padding-bottom",
                LonghandId::PaddingRight => "padding-right",
                LonghandId::PaddingBlockStart => "padding-block-start",
                LonghandId::PaddingBlockEnd => "padding-block-end",
                LonghandId::PaddingInlineStart => "padding-inline-start",
                LonghandId::PaddingInlineEnd => "padding-inline-end",
                LonghandId::Cursor => "cursor",
                LonghandId::PointerEvents => "pointer-events",
                LonghandId::MozUserInput => "-moz-user-input",
                LonghandId::MozUserModify => "-moz-user-modify",
                LonghandId::MozUserFocus => "-moz-user-focus",
                LonghandId::CaretColor => "caret-color",
                LonghandId::Top => "top",
                LonghandId::Left => "left",
                LonghandId::Bottom => "bottom",
                LonghandId::Right => "right",
                LonghandId::OffsetBlockStart => "offset-block-start",
                LonghandId::OffsetBlockEnd => "offset-block-end",
                LonghandId::OffsetInlineStart => "offset-inline-start",
                LonghandId::OffsetInlineEnd => "offset-inline-end",
                LonghandId::ZIndex => "z-index",
                LonghandId::FlexDirection => "flex-direction",
                LonghandId::FlexWrap => "flex-wrap",
                LonghandId::JustifyContent => "justify-content",
                LonghandId::AlignContent => "align-content",
                LonghandId::AlignItems => "align-items",
                LonghandId::JustifyItems => "justify-items",
                LonghandId::FlexGrow => "flex-grow",
                LonghandId::FlexShrink => "flex-shrink",
                LonghandId::AlignSelf => "align-self",
                LonghandId::JustifySelf => "justify-self",
                LonghandId::Order => "order",
                LonghandId::FlexBasis => "flex-basis",
                LonghandId::Width => "width",
                LonghandId::MinWidth => "min-width",
                LonghandId::MaxWidth => "max-width",
                LonghandId::Height => "height",
                LonghandId::MinHeight => "min-height",
                LonghandId::MaxHeight => "max-height",
                LonghandId::BlockSize => "block-size",
                LonghandId::MinBlockSize => "min-block-size",
                LonghandId::MaxBlockSize => "max-block-size",
                LonghandId::InlineSize => "inline-size",
                LonghandId::MinInlineSize => "min-inline-size",
                LonghandId::MaxInlineSize => "max-inline-size",
                LonghandId::BoxSizing => "box-sizing",
                LonghandId::ObjectFit => "object-fit",
                LonghandId::ObjectPosition => "object-position",
                LonghandId::GridRowGap => "grid-row-gap",
                LonghandId::GridRowStart => "grid-row-start",
                LonghandId::GridRowEnd => "grid-row-end",
                LonghandId::GridAutoRows => "grid-auto-rows",
                LonghandId::GridTemplateRows => "grid-template-rows",
                LonghandId::GridColumnGap => "grid-column-gap",
                LonghandId::GridColumnStart => "grid-column-start",
                LonghandId::GridColumnEnd => "grid-column-end",
                LonghandId::GridAutoColumns => "grid-auto-columns",
                LonghandId::GridTemplateColumns => "grid-template-columns",
                LonghandId::GridAutoFlow => "grid-auto-flow",
                LonghandId::GridTemplateAreas => "grid-template-areas",
                LonghandId::TableLayout => "table-layout",
                LonghandId::XSpan => "-x-span",
                LonghandId::TextOverflow => "text-overflow",
                LonghandId::UnicodeBidi => "unicode-bidi",
                LonghandId::TextDecorationLine => "text-decoration-line",
                LonghandId::TextDecorationStyle => "text-decoration-style",
                LonghandId::TextDecorationColor => "text-decoration-color",
                LonghandId::InitialLetter => "initial-letter",
                LonghandId::ImeMode => "ime-mode",
                LonghandId::MozUserSelect => "-moz-user-select",
                LonghandId::MozWindowDragging => "-moz-window-dragging",
                LonghandId::MozWindowShadow => "-moz-window-shadow",
                LonghandId::MozForceBrokenImageIcon => "-moz-force-broken-image-icon",
                LonghandId::TextAnchor => "text-anchor",
                LonghandId::ColorInterpolation => "color-interpolation",
                LonghandId::ColorInterpolationFilters => "color-interpolation-filters",
                LonghandId::Fill => "fill",
                LonghandId::FillOpacity => "fill-opacity",
                LonghandId::FillRule => "fill-rule",
                LonghandId::ShapeRendering => "shape-rendering",
                LonghandId::Stroke => "stroke",
                LonghandId::StrokeWidth => "stroke-width",
                LonghandId::StrokeLinecap => "stroke-linecap",
                LonghandId::StrokeLinejoin => "stroke-linejoin",
                LonghandId::StrokeMiterlimit => "stroke-miterlimit",
                LonghandId::StrokeOpacity => "stroke-opacity",
                LonghandId::StrokeDasharray => "stroke-dasharray",
                LonghandId::StrokeDashoffset => "stroke-dashoffset",
                LonghandId::ClipRule => "clip-rule",
                LonghandId::MarkerStart => "marker-start",
                LonghandId::MarkerMid => "marker-mid",
                LonghandId::MarkerEnd => "marker-end",
                LonghandId::PaintOrder => "paint-order",
                LonghandId::MozContextProperties => "-moz-context-properties",
                LonghandId::DominantBaseline => "dominant-baseline",
                LonghandId::VectorEffect => "vector-effect",
                LonghandId::StopColor => "stop-color",
                LonghandId::StopOpacity => "stop-opacity",
                LonghandId::FloodColor => "flood-color",
                LonghandId::FloodOpacity => "flood-opacity",
                LonghandId::LightingColor => "lighting-color",
                LonghandId::MaskType => "mask-type",
                LonghandId::ClipPath => "clip-path",
                LonghandId::MaskMode => "mask-mode",
                LonghandId::MaskRepeat => "mask-repeat",
                LonghandId::MaskPositionX => "mask-position-x",
                LonghandId::MaskPositionY => "mask-position-y",
                LonghandId::MaskClip => "mask-clip",
                LonghandId::MaskOrigin => "mask-origin",
                LonghandId::MaskSize => "mask-size",
                LonghandId::MaskComposite => "mask-composite",
                LonghandId::MaskImage => "mask-image",
                LonghandId::MozBoxAlign => "-moz-box-align",
                LonghandId::MozBoxDirection => "-moz-box-direction",
                LonghandId::MozBoxFlex => "-moz-box-flex",
                LonghandId::MozBoxOrient => "-moz-box-orient",
                LonghandId::MozBoxPack => "-moz-box-pack",
                LonghandId::MozStackSizing => "-moz-stack-sizing",
                LonghandId::MozBoxOrdinalGroup => "-moz-box-ordinal-group",
        }
    }

    fn inherited(&self) -> bool {
        
static INHERITED: LonghandIdSet = LonghandIdSet {
    
    storage: [0x0, 0x0, 0x0, 0xfffe0001, 0xffffffff, 0x1fffff, 0x7e000, 0x0, 0xffff0000, 0x1f]
};

        INHERITED.contains(*self)
    }

    fn shorthands(&self) -> &'static [ShorthandId] {
        // first generate longhand to shorthands lookup map
        //
        // NOTE(emilio): This currently doesn't exclude the "all" shorthand. It
        // could potentially do so, which would speed up serialization
        // algorithms and what not, I guess.
        

        // based on lookup results for each longhand, create result arrays
            static BACKGROUND_COLOR: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Background,
            ];
            static BACKGROUND_IMAGE: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Background,
            ];
            static BACKGROUND_POSITION_X: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Background,
                    ShorthandId::BackgroundPosition,
            ];
            static BACKGROUND_POSITION_Y: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Background,
                    ShorthandId::BackgroundPosition,
            ];
            static BACKGROUND_REPEAT: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Background,
            ];
            static BACKGROUND_ATTACHMENT: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Background,
            ];
            static BACKGROUND_CLIP: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Background,
            ];
            static BACKGROUND_ORIGIN: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Background,
            ];
            static BACKGROUND_SIZE: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Background,
            ];
            static BACKGROUND_BLEND_MODE: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static BORDER_TOP_COLOR: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Border,
                    ShorthandId::BorderColor,
                    ShorthandId::BorderTop,
            ];
            static BORDER_TOP_STYLE: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Border,
                    ShorthandId::BorderStyle,
                    ShorthandId::BorderTop,
            ];
            static BORDER_TOP_WIDTH: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Border,
                    ShorthandId::BorderWidth,
                    ShorthandId::BorderTop,
            ];
            static BORDER_LEFT_COLOR: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Border,
                    ShorthandId::BorderColor,
                    ShorthandId::BorderLeft,
            ];
            static BORDER_LEFT_STYLE: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Border,
                    ShorthandId::BorderStyle,
                    ShorthandId::BorderLeft,
            ];
            static BORDER_LEFT_WIDTH: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Border,
                    ShorthandId::BorderWidth,
                    ShorthandId::BorderLeft,
            ];
            static BORDER_BOTTOM_COLOR: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Border,
                    ShorthandId::BorderColor,
                    ShorthandId::BorderBottom,
            ];
            static BORDER_BOTTOM_STYLE: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Border,
                    ShorthandId::BorderStyle,
                    ShorthandId::BorderBottom,
            ];
            static BORDER_BOTTOM_WIDTH: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Border,
                    ShorthandId::BorderWidth,
                    ShorthandId::BorderBottom,
            ];
            static BORDER_RIGHT_COLOR: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Border,
                    ShorthandId::BorderColor,
                    ShorthandId::BorderRight,
            ];
            static BORDER_RIGHT_STYLE: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Border,
                    ShorthandId::BorderStyle,
                    ShorthandId::BorderRight,
            ];
            static BORDER_RIGHT_WIDTH: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Border,
                    ShorthandId::BorderWidth,
                    ShorthandId::BorderRight,
            ];
            static BORDER_BLOCK_START_COLOR: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::BorderBlockStart,
            ];
            static BORDER_BLOCK_START_STYLE: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::BorderBlockStart,
            ];
            static BORDER_BLOCK_START_WIDTH: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::BorderBlockStart,
            ];
            static BORDER_BLOCK_END_COLOR: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::BorderBlockEnd,
            ];
            static BORDER_BLOCK_END_STYLE: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::BorderBlockEnd,
            ];
            static BORDER_BLOCK_END_WIDTH: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::BorderBlockEnd,
            ];
            static BORDER_INLINE_START_COLOR: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::BorderInlineStart,
            ];
            static BORDER_INLINE_START_STYLE: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::BorderInlineStart,
            ];
            static BORDER_INLINE_START_WIDTH: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::BorderInlineStart,
            ];
            static BORDER_INLINE_END_COLOR: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::BorderInlineEnd,
            ];
            static BORDER_INLINE_END_STYLE: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::BorderInlineEnd,
            ];
            static BORDER_INLINE_END_WIDTH: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::BorderInlineEnd,
            ];
            static BORDER_TOP_LEFT_RADIUS: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::BorderRadius,
            ];
            static BORDER_TOP_RIGHT_RADIUS: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::BorderRadius,
            ];
            static BORDER_BOTTOM_RIGHT_RADIUS: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::BorderRadius,
            ];
            static BORDER_BOTTOM_LEFT_RADIUS: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::BorderRadius,
            ];
            static _MOZ_BORDER_TOP_COLORS: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Border,
            ];
            static _MOZ_BORDER_LEFT_COLORS: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Border,
            ];
            static _MOZ_BORDER_BOTTOM_COLORS: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Border,
            ];
            static _MOZ_BORDER_RIGHT_COLORS: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Border,
            ];
            static BOX_DECORATION_BREAK: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static _MOZ_FLOAT_EDGE: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static BORDER_IMAGE_SOURCE: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Border,
                    ShorthandId::BorderImage,
            ];
            static BORDER_IMAGE_OUTSET: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Border,
                    ShorthandId::BorderImage,
            ];
            static BORDER_IMAGE_REPEAT: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Border,
                    ShorthandId::BorderImage,
            ];
            static BORDER_IMAGE_WIDTH: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Border,
                    ShorthandId::BorderImage,
            ];
            static BORDER_IMAGE_SLICE: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Border,
                    ShorthandId::BorderImage,
            ];
            static DISPLAY: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static _MOZ_TOP_LAYER: &'static [ShorthandId] = &[
            ];
            static POSITION: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static FLOAT: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static CLEAR: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static VERTICAL_ALIGN: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static OVERFLOW_CLIP_BOX: &'static [ShorthandId] = &[
            ];
            static OVERFLOW_X: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Overflow,
            ];
            static OVERFLOW_Y: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Overflow,
            ];
            static TRANSITION_DURATION: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Transition,
            ];
            static TRANSITION_TIMING_FUNCTION: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Transition,
            ];
            static TRANSITION_PROPERTY: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Transition,
            ];
            static TRANSITION_DELAY: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Transition,
            ];
            static ANIMATION_NAME: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Animation,
            ];
            static ANIMATION_DURATION: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Animation,
            ];
            static ANIMATION_TIMING_FUNCTION: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Animation,
            ];
            static ANIMATION_ITERATION_COUNT: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Animation,
            ];
            static ANIMATION_DIRECTION: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Animation,
            ];
            static ANIMATION_PLAY_STATE: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Animation,
            ];
            static ANIMATION_FILL_MODE: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Animation,
            ];
            static ANIMATION_DELAY: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Animation,
            ];
            static SCROLL_SNAP_POINTS_X: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static SCROLL_SNAP_POINTS_Y: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static SCROLL_SNAP_DESTINATION: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static SCROLL_SNAP_COORDINATE: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static TRANSFORM: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::MozTransform,
            ];
            static SCROLL_BEHAVIOR: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static SCROLL_SNAP_TYPE_X: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::ScrollSnapType,
            ];
            static SCROLL_SNAP_TYPE_Y: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::ScrollSnapType,
            ];
            static ISOLATION: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static PAGE_BREAK_AFTER: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static PAGE_BREAK_BEFORE: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static PAGE_BREAK_INSIDE: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static RESIZE: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static PERSPECTIVE: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static PERSPECTIVE_ORIGIN: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static BACKFACE_VISIBILITY: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static TRANSFORM_BOX: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static TRANSFORM_STYLE: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static TRANSFORM_ORIGIN: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static CONTAIN: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static _MOZ_APPEARANCE: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static _MOZ_BINDING: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static _MOZ_ORIENT: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static WILL_CHANGE: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static SHAPE_OUTSIDE: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static TOUCH_ACTION: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static COLOR: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static COLUMN_WIDTH: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Columns,
            ];
            static COLUMN_COUNT: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Columns,
            ];
            static COLUMN_GAP: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static COLUMN_FILL: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static COLUMN_RULE_WIDTH: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::ColumnRule,
            ];
            static COLUMN_RULE_COLOR: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::ColumnRule,
            ];
            static COLUMN_SPAN: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static COLUMN_RULE_STYLE: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::ColumnRule,
            ];
            static CONTENT: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static COUNTER_INCREMENT: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static COUNTER_RESET: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static OPACITY: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static BOX_SHADOW: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static CLIP: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static FILTER: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static MIX_BLEND_MODE: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static FONT_FAMILY: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Font,
            ];
            static FONT_STYLE: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Font,
            ];
            static FONT_VARIANT_CAPS: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Font,
                    ShorthandId::FontVariant,
            ];
            static FONT_WEIGHT: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Font,
            ];
            static FONT_SIZE: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Font,
            ];
            static FONT_SIZE_ADJUST: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Font,
            ];
            static FONT_SYNTHESIS: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static FONT_STRETCH: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Font,
            ];
            static FONT_KERNING: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Font,
            ];
            static FONT_VARIANT_ALTERNATES: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Font,
                    ShorthandId::FontVariant,
            ];
            static FONT_VARIANT_EAST_ASIAN: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Font,
                    ShorthandId::FontVariant,
            ];
            static FONT_VARIANT_LIGATURES: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Font,
                    ShorthandId::FontVariant,
            ];
            static FONT_VARIANT_NUMERIC: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Font,
                    ShorthandId::FontVariant,
            ];
            static FONT_VARIANT_POSITION: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Font,
                    ShorthandId::FontVariant,
            ];
            static FONT_FEATURE_SETTINGS: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Font,
            ];
            static FONT_VARIATION_SETTINGS: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static FONT_LANGUAGE_OVERRIDE: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Font,
            ];
            static _X_LANG: &'static [ShorthandId] = &[
            ];
            static _MOZ_SCRIPT_SIZE_MULTIPLIER: &'static [ShorthandId] = &[
            ];
            static _MOZ_SCRIPT_LEVEL: &'static [ShorthandId] = &[
            ];
            static _MOZ_MATH_DISPLAY: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static _MOZ_MATH_VARIANT: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static _MOZ_SCRIPT_MIN_SIZE: &'static [ShorthandId] = &[
            ];
            static _X_TEXT_ZOOM: &'static [ShorthandId] = &[
            ];
            static _MOZ_OSX_FONT_SMOOTHING: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static _MOZ_MIN_FONT_SIZE_RATIO: &'static [ShorthandId] = &[
            ];
            static VISIBILITY: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static WRITING_MODE: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static DIRECTION: &'static [ShorthandId] = &[
            ];
            static TEXT_ORIENTATION: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static COLOR_ADJUST: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static IMAGE_RENDERING: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static IMAGE_ORIENTATION: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static BORDER_COLLAPSE: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static EMPTY_CELLS: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static CAPTION_SIDE: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static BORDER_SPACING: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static LINE_HEIGHT: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Font,
            ];
            static TEXT_TRANSFORM: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static HYPHENS: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static _MOZ_TEXT_SIZE_ADJUST: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static TEXT_INDENT: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static OVERFLOW_WRAP: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static WORD_BREAK: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static TEXT_JUSTIFY: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static TEXT_ALIGN_LAST: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static TEXT_ALIGN: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static LETTER_SPACING: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static WORD_SPACING: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static WHITE_SPACE: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static TEXT_SHADOW: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static TEXT_EMPHASIS_STYLE: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::TextEmphasis,
            ];
            static TEXT_EMPHASIS_POSITION: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static TEXT_EMPHASIS_COLOR: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::TextEmphasis,
            ];
            static _MOZ_TAB_SIZE: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static _WEBKIT_TEXT_FILL_COLOR: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static _WEBKIT_TEXT_STROKE_COLOR: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::WebkitTextStroke,
            ];
            static _WEBKIT_TEXT_STROKE_WIDTH: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::WebkitTextStroke,
            ];
            static RUBY_ALIGN: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static RUBY_POSITION: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static TEXT_COMBINE_UPRIGHT: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static TEXT_RENDERING: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static _MOZ_CONTROL_CHARACTER_VISIBILITY: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static LIST_STYLE_POSITION: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::ListStyle,
            ];
            static LIST_STYLE_TYPE: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::ListStyle,
            ];
            static LIST_STYLE_IMAGE: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::ListStyle,
            ];
            static QUOTES: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static _MOZ_IMAGE_REGION: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static MARGIN_TOP: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Margin,
            ];
            static MARGIN_LEFT: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Margin,
            ];
            static MARGIN_BOTTOM: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Margin,
            ];
            static MARGIN_RIGHT: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Margin,
            ];
            static MARGIN_BLOCK_START: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static MARGIN_BLOCK_END: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static MARGIN_INLINE_START: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static MARGIN_INLINE_END: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static OUTLINE_COLOR: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Outline,
            ];
            static OUTLINE_STYLE: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Outline,
            ];
            static OUTLINE_WIDTH: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Outline,
            ];
            static _MOZ_OUTLINE_RADIUS_TOPLEFT: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::MozOutlineRadius,
            ];
            static _MOZ_OUTLINE_RADIUS_TOPRIGHT: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::MozOutlineRadius,
            ];
            static _MOZ_OUTLINE_RADIUS_BOTTOMRIGHT: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::MozOutlineRadius,
            ];
            static _MOZ_OUTLINE_RADIUS_BOTTOMLEFT: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::MozOutlineRadius,
            ];
            static OUTLINE_OFFSET: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static PADDING_TOP: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Padding,
            ];
            static PADDING_LEFT: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Padding,
            ];
            static PADDING_BOTTOM: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Padding,
            ];
            static PADDING_RIGHT: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Padding,
            ];
            static PADDING_BLOCK_START: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static PADDING_BLOCK_END: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static PADDING_INLINE_START: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static PADDING_INLINE_END: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static CURSOR: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static POINTER_EVENTS: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static _MOZ_USER_INPUT: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static _MOZ_USER_MODIFY: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static _MOZ_USER_FOCUS: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static CARET_COLOR: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static TOP: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static LEFT: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static BOTTOM: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static RIGHT: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static OFFSET_BLOCK_START: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static OFFSET_BLOCK_END: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static OFFSET_INLINE_START: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static OFFSET_INLINE_END: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static Z_INDEX: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static FLEX_DIRECTION: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::FlexFlow,
            ];
            static FLEX_WRAP: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::FlexFlow,
            ];
            static JUSTIFY_CONTENT: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::PlaceContent,
            ];
            static ALIGN_CONTENT: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::PlaceContent,
            ];
            static ALIGN_ITEMS: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::PlaceItems,
            ];
            static JUSTIFY_ITEMS: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::PlaceItems,
            ];
            static FLEX_GROW: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Flex,
            ];
            static FLEX_SHRINK: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Flex,
            ];
            static ALIGN_SELF: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::PlaceSelf,
            ];
            static JUSTIFY_SELF: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::PlaceSelf,
            ];
            static ORDER: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static FLEX_BASIS: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Flex,
            ];
            static WIDTH: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static MIN_WIDTH: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static MAX_WIDTH: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static HEIGHT: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static MIN_HEIGHT: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static MAX_HEIGHT: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static BLOCK_SIZE: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static MIN_BLOCK_SIZE: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static MAX_BLOCK_SIZE: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static INLINE_SIZE: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static MIN_INLINE_SIZE: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static MAX_INLINE_SIZE: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static BOX_SIZING: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static OBJECT_FIT: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static OBJECT_POSITION: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static GRID_ROW_GAP: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::GridGap,
            ];
            static GRID_ROW_START: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::GridArea,
                    ShorthandId::GridRow,
            ];
            static GRID_ROW_END: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::GridArea,
                    ShorthandId::GridRow,
            ];
            static GRID_AUTO_ROWS: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Grid,
            ];
            static GRID_TEMPLATE_ROWS: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Grid,
                    ShorthandId::GridTemplate,
            ];
            static GRID_COLUMN_GAP: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::GridGap,
            ];
            static GRID_COLUMN_START: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::GridArea,
                    ShorthandId::GridColumn,
            ];
            static GRID_COLUMN_END: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::GridArea,
                    ShorthandId::GridColumn,
            ];
            static GRID_AUTO_COLUMNS: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Grid,
            ];
            static GRID_TEMPLATE_COLUMNS: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Grid,
                    ShorthandId::GridTemplate,
            ];
            static GRID_AUTO_FLOW: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Grid,
            ];
            static GRID_TEMPLATE_AREAS: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Grid,
                    ShorthandId::GridTemplate,
            ];
            static TABLE_LAYOUT: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static _X_SPAN: &'static [ShorthandId] = &[
            ];
            static TEXT_OVERFLOW: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static UNICODE_BIDI: &'static [ShorthandId] = &[
            ];
            static TEXT_DECORATION_LINE: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::TextDecoration,
            ];
            static TEXT_DECORATION_STYLE: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::TextDecoration,
            ];
            static TEXT_DECORATION_COLOR: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::TextDecoration,
            ];
            static INITIAL_LETTER: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static IME_MODE: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static _MOZ_USER_SELECT: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static _MOZ_WINDOW_DRAGGING: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static _MOZ_WINDOW_SHADOW: &'static [ShorthandId] = &[
            ];
            static _MOZ_FORCE_BROKEN_IMAGE_ICON: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static TEXT_ANCHOR: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static COLOR_INTERPOLATION: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static COLOR_INTERPOLATION_FILTERS: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static FILL: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static FILL_OPACITY: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static FILL_RULE: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static SHAPE_RENDERING: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static STROKE: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static STROKE_WIDTH: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static STROKE_LINECAP: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static STROKE_LINEJOIN: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static STROKE_MITERLIMIT: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static STROKE_OPACITY: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static STROKE_DASHARRAY: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static STROKE_DASHOFFSET: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static CLIP_RULE: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static MARKER_START: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Marker,
            ];
            static MARKER_MID: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Marker,
            ];
            static MARKER_END: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Marker,
            ];
            static PAINT_ORDER: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static _MOZ_CONTEXT_PROPERTIES: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static DOMINANT_BASELINE: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static VECTOR_EFFECT: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static STOP_COLOR: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static STOP_OPACITY: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static FLOOD_COLOR: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static FLOOD_OPACITY: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static LIGHTING_COLOR: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static MASK_TYPE: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static CLIP_PATH: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static MASK_MODE: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Mask,
            ];
            static MASK_REPEAT: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Mask,
            ];
            static MASK_POSITION_X: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Mask,
                    ShorthandId::MaskPosition,
            ];
            static MASK_POSITION_Y: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Mask,
                    ShorthandId::MaskPosition,
            ];
            static MASK_CLIP: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Mask,
            ];
            static MASK_ORIGIN: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Mask,
            ];
            static MASK_SIZE: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Mask,
            ];
            static MASK_COMPOSITE: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Mask,
            ];
            static MASK_IMAGE: &'static [ShorthandId] = &[
                    ShorthandId::All,
                    ShorthandId::Mask,
            ];
            static _MOZ_BOX_ALIGN: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static _MOZ_BOX_DIRECTION: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static _MOZ_BOX_FLEX: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static _MOZ_BOX_ORIENT: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static _MOZ_BOX_PACK: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static _MOZ_STACK_SIZING: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];
            static _MOZ_BOX_ORDINAL_GROUP: &'static [ShorthandId] = &[
                    ShorthandId::All,
            ];

        match *self {
                LonghandId::BackgroundColor => BACKGROUND_COLOR,
                LonghandId::BackgroundImage => BACKGROUND_IMAGE,
                LonghandId::BackgroundPositionX => BACKGROUND_POSITION_X,
                LonghandId::BackgroundPositionY => BACKGROUND_POSITION_Y,
                LonghandId::BackgroundRepeat => BACKGROUND_REPEAT,
                LonghandId::BackgroundAttachment => BACKGROUND_ATTACHMENT,
                LonghandId::BackgroundClip => BACKGROUND_CLIP,
                LonghandId::BackgroundOrigin => BACKGROUND_ORIGIN,
                LonghandId::BackgroundSize => BACKGROUND_SIZE,
                LonghandId::BackgroundBlendMode => BACKGROUND_BLEND_MODE,
                LonghandId::BorderTopColor => BORDER_TOP_COLOR,
                LonghandId::BorderTopStyle => BORDER_TOP_STYLE,
                LonghandId::BorderTopWidth => BORDER_TOP_WIDTH,
                LonghandId::BorderLeftColor => BORDER_LEFT_COLOR,
                LonghandId::BorderLeftStyle => BORDER_LEFT_STYLE,
                LonghandId::BorderLeftWidth => BORDER_LEFT_WIDTH,
                LonghandId::BorderBottomColor => BORDER_BOTTOM_COLOR,
                LonghandId::BorderBottomStyle => BORDER_BOTTOM_STYLE,
                LonghandId::BorderBottomWidth => BORDER_BOTTOM_WIDTH,
                LonghandId::BorderRightColor => BORDER_RIGHT_COLOR,
                LonghandId::BorderRightStyle => BORDER_RIGHT_STYLE,
                LonghandId::BorderRightWidth => BORDER_RIGHT_WIDTH,
                LonghandId::BorderBlockStartColor => BORDER_BLOCK_START_COLOR,
                LonghandId::BorderBlockStartStyle => BORDER_BLOCK_START_STYLE,
                LonghandId::BorderBlockStartWidth => BORDER_BLOCK_START_WIDTH,
                LonghandId::BorderBlockEndColor => BORDER_BLOCK_END_COLOR,
                LonghandId::BorderBlockEndStyle => BORDER_BLOCK_END_STYLE,
                LonghandId::BorderBlockEndWidth => BORDER_BLOCK_END_WIDTH,
                LonghandId::BorderInlineStartColor => BORDER_INLINE_START_COLOR,
                LonghandId::BorderInlineStartStyle => BORDER_INLINE_START_STYLE,
                LonghandId::BorderInlineStartWidth => BORDER_INLINE_START_WIDTH,
                LonghandId::BorderInlineEndColor => BORDER_INLINE_END_COLOR,
                LonghandId::BorderInlineEndStyle => BORDER_INLINE_END_STYLE,
                LonghandId::BorderInlineEndWidth => BORDER_INLINE_END_WIDTH,
                LonghandId::BorderTopLeftRadius => BORDER_TOP_LEFT_RADIUS,
                LonghandId::BorderTopRightRadius => BORDER_TOP_RIGHT_RADIUS,
                LonghandId::BorderBottomRightRadius => BORDER_BOTTOM_RIGHT_RADIUS,
                LonghandId::BorderBottomLeftRadius => BORDER_BOTTOM_LEFT_RADIUS,
                LonghandId::MozBorderTopColors => _MOZ_BORDER_TOP_COLORS,
                LonghandId::MozBorderLeftColors => _MOZ_BORDER_LEFT_COLORS,
                LonghandId::MozBorderBottomColors => _MOZ_BORDER_BOTTOM_COLORS,
                LonghandId::MozBorderRightColors => _MOZ_BORDER_RIGHT_COLORS,
                LonghandId::BoxDecorationBreak => BOX_DECORATION_BREAK,
                LonghandId::MozFloatEdge => _MOZ_FLOAT_EDGE,
                LonghandId::BorderImageSource => BORDER_IMAGE_SOURCE,
                LonghandId::BorderImageOutset => BORDER_IMAGE_OUTSET,
                LonghandId::BorderImageRepeat => BORDER_IMAGE_REPEAT,
                LonghandId::BorderImageWidth => BORDER_IMAGE_WIDTH,
                LonghandId::BorderImageSlice => BORDER_IMAGE_SLICE,
                LonghandId::Display => DISPLAY,
                LonghandId::MozTopLayer => _MOZ_TOP_LAYER,
                LonghandId::Position => POSITION,
                LonghandId::Float => FLOAT,
                LonghandId::Clear => CLEAR,
                LonghandId::VerticalAlign => VERTICAL_ALIGN,
                LonghandId::OverflowClipBox => OVERFLOW_CLIP_BOX,
                LonghandId::OverflowX => OVERFLOW_X,
                LonghandId::OverflowY => OVERFLOW_Y,
                LonghandId::TransitionDuration => TRANSITION_DURATION,
                LonghandId::TransitionTimingFunction => TRANSITION_TIMING_FUNCTION,
                LonghandId::TransitionProperty => TRANSITION_PROPERTY,
                LonghandId::TransitionDelay => TRANSITION_DELAY,
                LonghandId::AnimationName => ANIMATION_NAME,
                LonghandId::AnimationDuration => ANIMATION_DURATION,
                LonghandId::AnimationTimingFunction => ANIMATION_TIMING_FUNCTION,
                LonghandId::AnimationIterationCount => ANIMATION_ITERATION_COUNT,
                LonghandId::AnimationDirection => ANIMATION_DIRECTION,
                LonghandId::AnimationPlayState => ANIMATION_PLAY_STATE,
                LonghandId::AnimationFillMode => ANIMATION_FILL_MODE,
                LonghandId::AnimationDelay => ANIMATION_DELAY,
                LonghandId::ScrollSnapPointsX => SCROLL_SNAP_POINTS_X,
                LonghandId::ScrollSnapPointsY => SCROLL_SNAP_POINTS_Y,
                LonghandId::ScrollSnapDestination => SCROLL_SNAP_DESTINATION,
                LonghandId::ScrollSnapCoordinate => SCROLL_SNAP_COORDINATE,
                LonghandId::Transform => TRANSFORM,
                LonghandId::ScrollBehavior => SCROLL_BEHAVIOR,
                LonghandId::ScrollSnapTypeX => SCROLL_SNAP_TYPE_X,
                LonghandId::ScrollSnapTypeY => SCROLL_SNAP_TYPE_Y,
                LonghandId::Isolation => ISOLATION,
                LonghandId::PageBreakAfter => PAGE_BREAK_AFTER,
                LonghandId::PageBreakBefore => PAGE_BREAK_BEFORE,
                LonghandId::PageBreakInside => PAGE_BREAK_INSIDE,
                LonghandId::Resize => RESIZE,
                LonghandId::Perspective => PERSPECTIVE,
                LonghandId::PerspectiveOrigin => PERSPECTIVE_ORIGIN,
                LonghandId::BackfaceVisibility => BACKFACE_VISIBILITY,
                LonghandId::TransformBox => TRANSFORM_BOX,
                LonghandId::TransformStyle => TRANSFORM_STYLE,
                LonghandId::TransformOrigin => TRANSFORM_ORIGIN,
                LonghandId::Contain => CONTAIN,
                LonghandId::MozAppearance => _MOZ_APPEARANCE,
                LonghandId::MozBinding => _MOZ_BINDING,
                LonghandId::MozOrient => _MOZ_ORIENT,
                LonghandId::WillChange => WILL_CHANGE,
                LonghandId::ShapeOutside => SHAPE_OUTSIDE,
                LonghandId::TouchAction => TOUCH_ACTION,
                LonghandId::Color => COLOR,
                LonghandId::ColumnWidth => COLUMN_WIDTH,
                LonghandId::ColumnCount => COLUMN_COUNT,
                LonghandId::ColumnGap => COLUMN_GAP,
                LonghandId::ColumnFill => COLUMN_FILL,
                LonghandId::ColumnRuleWidth => COLUMN_RULE_WIDTH,
                LonghandId::ColumnRuleColor => COLUMN_RULE_COLOR,
                LonghandId::ColumnSpan => COLUMN_SPAN,
                LonghandId::ColumnRuleStyle => COLUMN_RULE_STYLE,
                LonghandId::Content => CONTENT,
                LonghandId::CounterIncrement => COUNTER_INCREMENT,
                LonghandId::CounterReset => COUNTER_RESET,
                LonghandId::Opacity => OPACITY,
                LonghandId::BoxShadow => BOX_SHADOW,
                LonghandId::Clip => CLIP,
                LonghandId::Filter => FILTER,
                LonghandId::MixBlendMode => MIX_BLEND_MODE,
                LonghandId::FontFamily => FONT_FAMILY,
                LonghandId::FontStyle => FONT_STYLE,
                LonghandId::FontVariantCaps => FONT_VARIANT_CAPS,
                LonghandId::FontWeight => FONT_WEIGHT,
                LonghandId::FontSize => FONT_SIZE,
                LonghandId::FontSizeAdjust => FONT_SIZE_ADJUST,
                LonghandId::FontSynthesis => FONT_SYNTHESIS,
                LonghandId::FontStretch => FONT_STRETCH,
                LonghandId::FontKerning => FONT_KERNING,
                LonghandId::FontVariantAlternates => FONT_VARIANT_ALTERNATES,
                LonghandId::FontVariantEastAsian => FONT_VARIANT_EAST_ASIAN,
                LonghandId::FontVariantLigatures => FONT_VARIANT_LIGATURES,
                LonghandId::FontVariantNumeric => FONT_VARIANT_NUMERIC,
                LonghandId::FontVariantPosition => FONT_VARIANT_POSITION,
                LonghandId::FontFeatureSettings => FONT_FEATURE_SETTINGS,
                LonghandId::FontVariationSettings => FONT_VARIATION_SETTINGS,
                LonghandId::FontLanguageOverride => FONT_LANGUAGE_OVERRIDE,
                LonghandId::XLang => _X_LANG,
                LonghandId::MozScriptSizeMultiplier => _MOZ_SCRIPT_SIZE_MULTIPLIER,
                LonghandId::MozScriptLevel => _MOZ_SCRIPT_LEVEL,
                LonghandId::MozMathDisplay => _MOZ_MATH_DISPLAY,
                LonghandId::MozMathVariant => _MOZ_MATH_VARIANT,
                LonghandId::MozScriptMinSize => _MOZ_SCRIPT_MIN_SIZE,
                LonghandId::XTextZoom => _X_TEXT_ZOOM,
                LonghandId::MozOsxFontSmoothing => _MOZ_OSX_FONT_SMOOTHING,
                LonghandId::MozMinFontSizeRatio => _MOZ_MIN_FONT_SIZE_RATIO,
                LonghandId::Visibility => VISIBILITY,
                LonghandId::WritingMode => WRITING_MODE,
                LonghandId::Direction => DIRECTION,
                LonghandId::TextOrientation => TEXT_ORIENTATION,
                LonghandId::ColorAdjust => COLOR_ADJUST,
                LonghandId::ImageRendering => IMAGE_RENDERING,
                LonghandId::ImageOrientation => IMAGE_ORIENTATION,
                LonghandId::BorderCollapse => BORDER_COLLAPSE,
                LonghandId::EmptyCells => EMPTY_CELLS,
                LonghandId::CaptionSide => CAPTION_SIDE,
                LonghandId::BorderSpacing => BORDER_SPACING,
                LonghandId::LineHeight => LINE_HEIGHT,
                LonghandId::TextTransform => TEXT_TRANSFORM,
                LonghandId::Hyphens => HYPHENS,
                LonghandId::MozTextSizeAdjust => _MOZ_TEXT_SIZE_ADJUST,
                LonghandId::TextIndent => TEXT_INDENT,
                LonghandId::OverflowWrap => OVERFLOW_WRAP,
                LonghandId::WordBreak => WORD_BREAK,
                LonghandId::TextJustify => TEXT_JUSTIFY,
                LonghandId::TextAlignLast => TEXT_ALIGN_LAST,
                LonghandId::TextAlign => TEXT_ALIGN,
                LonghandId::LetterSpacing => LETTER_SPACING,
                LonghandId::WordSpacing => WORD_SPACING,
                LonghandId::WhiteSpace => WHITE_SPACE,
                LonghandId::TextShadow => TEXT_SHADOW,
                LonghandId::TextEmphasisStyle => TEXT_EMPHASIS_STYLE,
                LonghandId::TextEmphasisPosition => TEXT_EMPHASIS_POSITION,
                LonghandId::TextEmphasisColor => TEXT_EMPHASIS_COLOR,
                LonghandId::MozTabSize => _MOZ_TAB_SIZE,
                LonghandId::WebkitTextFillColor => _WEBKIT_TEXT_FILL_COLOR,
                LonghandId::WebkitTextStrokeColor => _WEBKIT_TEXT_STROKE_COLOR,
                LonghandId::WebkitTextStrokeWidth => _WEBKIT_TEXT_STROKE_WIDTH,
                LonghandId::RubyAlign => RUBY_ALIGN,
                LonghandId::RubyPosition => RUBY_POSITION,
                LonghandId::TextCombineUpright => TEXT_COMBINE_UPRIGHT,
                LonghandId::TextRendering => TEXT_RENDERING,
                LonghandId::MozControlCharacterVisibility => _MOZ_CONTROL_CHARACTER_VISIBILITY,
                LonghandId::ListStylePosition => LIST_STYLE_POSITION,
                LonghandId::ListStyleType => LIST_STYLE_TYPE,
                LonghandId::ListStyleImage => LIST_STYLE_IMAGE,
                LonghandId::Quotes => QUOTES,
                LonghandId::MozImageRegion => _MOZ_IMAGE_REGION,
                LonghandId::MarginTop => MARGIN_TOP,
                LonghandId::MarginLeft => MARGIN_LEFT,
                LonghandId::MarginBottom => MARGIN_BOTTOM,
                LonghandId::MarginRight => MARGIN_RIGHT,
                LonghandId::MarginBlockStart => MARGIN_BLOCK_START,
                LonghandId::MarginBlockEnd => MARGIN_BLOCK_END,
                LonghandId::MarginInlineStart => MARGIN_INLINE_START,
                LonghandId::MarginInlineEnd => MARGIN_INLINE_END,
                LonghandId::OutlineColor => OUTLINE_COLOR,
                LonghandId::OutlineStyle => OUTLINE_STYLE,
                LonghandId::OutlineWidth => OUTLINE_WIDTH,
                LonghandId::MozOutlineRadiusTopleft => _MOZ_OUTLINE_RADIUS_TOPLEFT,
                LonghandId::MozOutlineRadiusTopright => _MOZ_OUTLINE_RADIUS_TOPRIGHT,
                LonghandId::MozOutlineRadiusBottomright => _MOZ_OUTLINE_RADIUS_BOTTOMRIGHT,
                LonghandId::MozOutlineRadiusBottomleft => _MOZ_OUTLINE_RADIUS_BOTTOMLEFT,
                LonghandId::OutlineOffset => OUTLINE_OFFSET,
                LonghandId::PaddingTop => PADDING_TOP,
                LonghandId::PaddingLeft => PADDING_LEFT,
                LonghandId::PaddingBottom => PADDING_BOTTOM,
                LonghandId::PaddingRight => PADDING_RIGHT,
                LonghandId::PaddingBlockStart => PADDING_BLOCK_START,
                LonghandId::PaddingBlockEnd => PADDING_BLOCK_END,
                LonghandId::PaddingInlineStart => PADDING_INLINE_START,
                LonghandId::PaddingInlineEnd => PADDING_INLINE_END,
                LonghandId::Cursor => CURSOR,
                LonghandId::PointerEvents => POINTER_EVENTS,
                LonghandId::MozUserInput => _MOZ_USER_INPUT,
                LonghandId::MozUserModify => _MOZ_USER_MODIFY,
                LonghandId::MozUserFocus => _MOZ_USER_FOCUS,
                LonghandId::CaretColor => CARET_COLOR,
                LonghandId::Top => TOP,
                LonghandId::Left => LEFT,
                LonghandId::Bottom => BOTTOM,
                LonghandId::Right => RIGHT,
                LonghandId::OffsetBlockStart => OFFSET_BLOCK_START,
                LonghandId::OffsetBlockEnd => OFFSET_BLOCK_END,
                LonghandId::OffsetInlineStart => OFFSET_INLINE_START,
                LonghandId::OffsetInlineEnd => OFFSET_INLINE_END,
                LonghandId::ZIndex => Z_INDEX,
                LonghandId::FlexDirection => FLEX_DIRECTION,
                LonghandId::FlexWrap => FLEX_WRAP,
                LonghandId::JustifyContent => JUSTIFY_CONTENT,
                LonghandId::AlignContent => ALIGN_CONTENT,
                LonghandId::AlignItems => ALIGN_ITEMS,
                LonghandId::JustifyItems => JUSTIFY_ITEMS,
                LonghandId::FlexGrow => FLEX_GROW,
                LonghandId::FlexShrink => FLEX_SHRINK,
                LonghandId::AlignSelf => ALIGN_SELF,
                LonghandId::JustifySelf => JUSTIFY_SELF,
                LonghandId::Order => ORDER,
                LonghandId::FlexBasis => FLEX_BASIS,
                LonghandId::Width => WIDTH,
                LonghandId::MinWidth => MIN_WIDTH,
                LonghandId::MaxWidth => MAX_WIDTH,
                LonghandId::Height => HEIGHT,
                LonghandId::MinHeight => MIN_HEIGHT,
                LonghandId::MaxHeight => MAX_HEIGHT,
                LonghandId::BlockSize => BLOCK_SIZE,
                LonghandId::MinBlockSize => MIN_BLOCK_SIZE,
                LonghandId::MaxBlockSize => MAX_BLOCK_SIZE,
                LonghandId::InlineSize => INLINE_SIZE,
                LonghandId::MinInlineSize => MIN_INLINE_SIZE,
                LonghandId::MaxInlineSize => MAX_INLINE_SIZE,
                LonghandId::BoxSizing => BOX_SIZING,
                LonghandId::ObjectFit => OBJECT_FIT,
                LonghandId::ObjectPosition => OBJECT_POSITION,
                LonghandId::GridRowGap => GRID_ROW_GAP,
                LonghandId::GridRowStart => GRID_ROW_START,
                LonghandId::GridRowEnd => GRID_ROW_END,
                LonghandId::GridAutoRows => GRID_AUTO_ROWS,
                LonghandId::GridTemplateRows => GRID_TEMPLATE_ROWS,
                LonghandId::GridColumnGap => GRID_COLUMN_GAP,
                LonghandId::GridColumnStart => GRID_COLUMN_START,
                LonghandId::GridColumnEnd => GRID_COLUMN_END,
                LonghandId::GridAutoColumns => GRID_AUTO_COLUMNS,
                LonghandId::GridTemplateColumns => GRID_TEMPLATE_COLUMNS,
                LonghandId::GridAutoFlow => GRID_AUTO_FLOW,
                LonghandId::GridTemplateAreas => GRID_TEMPLATE_AREAS,
                LonghandId::TableLayout => TABLE_LAYOUT,
                LonghandId::XSpan => _X_SPAN,
                LonghandId::TextOverflow => TEXT_OVERFLOW,
                LonghandId::UnicodeBidi => UNICODE_BIDI,
                LonghandId::TextDecorationLine => TEXT_DECORATION_LINE,
                LonghandId::TextDecorationStyle => TEXT_DECORATION_STYLE,
                LonghandId::TextDecorationColor => TEXT_DECORATION_COLOR,
                LonghandId::InitialLetter => INITIAL_LETTER,
                LonghandId::ImeMode => IME_MODE,
                LonghandId::MozUserSelect => _MOZ_USER_SELECT,
                LonghandId::MozWindowDragging => _MOZ_WINDOW_DRAGGING,
                LonghandId::MozWindowShadow => _MOZ_WINDOW_SHADOW,
                LonghandId::MozForceBrokenImageIcon => _MOZ_FORCE_BROKEN_IMAGE_ICON,
                LonghandId::TextAnchor => TEXT_ANCHOR,
                LonghandId::ColorInterpolation => COLOR_INTERPOLATION,
                LonghandId::ColorInterpolationFilters => COLOR_INTERPOLATION_FILTERS,
                LonghandId::Fill => FILL,
                LonghandId::FillOpacity => FILL_OPACITY,
                LonghandId::FillRule => FILL_RULE,
                LonghandId::ShapeRendering => SHAPE_RENDERING,
                LonghandId::Stroke => STROKE,
                LonghandId::StrokeWidth => STROKE_WIDTH,
                LonghandId::StrokeLinecap => STROKE_LINECAP,
                LonghandId::StrokeLinejoin => STROKE_LINEJOIN,
                LonghandId::StrokeMiterlimit => STROKE_MITERLIMIT,
                LonghandId::StrokeOpacity => STROKE_OPACITY,
                LonghandId::StrokeDasharray => STROKE_DASHARRAY,
                LonghandId::StrokeDashoffset => STROKE_DASHOFFSET,
                LonghandId::ClipRule => CLIP_RULE,
                LonghandId::MarkerStart => MARKER_START,
                LonghandId::MarkerMid => MARKER_MID,
                LonghandId::MarkerEnd => MARKER_END,
                LonghandId::PaintOrder => PAINT_ORDER,
                LonghandId::MozContextProperties => _MOZ_CONTEXT_PROPERTIES,
                LonghandId::DominantBaseline => DOMINANT_BASELINE,
                LonghandId::VectorEffect => VECTOR_EFFECT,
                LonghandId::StopColor => STOP_COLOR,
                LonghandId::StopOpacity => STOP_OPACITY,
                LonghandId::FloodColor => FLOOD_COLOR,
                LonghandId::FloodOpacity => FLOOD_OPACITY,
                LonghandId::LightingColor => LIGHTING_COLOR,
                LonghandId::MaskType => MASK_TYPE,
                LonghandId::ClipPath => CLIP_PATH,
                LonghandId::MaskMode => MASK_MODE,
                LonghandId::MaskRepeat => MASK_REPEAT,
                LonghandId::MaskPositionX => MASK_POSITION_X,
                LonghandId::MaskPositionY => MASK_POSITION_Y,
                LonghandId::MaskClip => MASK_CLIP,
                LonghandId::MaskOrigin => MASK_ORIGIN,
                LonghandId::MaskSize => MASK_SIZE,
                LonghandId::MaskComposite => MASK_COMPOSITE,
                LonghandId::MaskImage => MASK_IMAGE,
                LonghandId::MozBoxAlign => _MOZ_BOX_ALIGN,
                LonghandId::MozBoxDirection => _MOZ_BOX_DIRECTION,
                LonghandId::MozBoxFlex => _MOZ_BOX_FLEX,
                LonghandId::MozBoxOrient => _MOZ_BOX_ORIENT,
                LonghandId::MozBoxPack => _MOZ_BOX_PACK,
                LonghandId::MozStackSizing => _MOZ_STACK_SIZING,
                LonghandId::MozBoxOrdinalGroup => _MOZ_BOX_ORDINAL_GROUP,
        }
    }

    fn parse_value<'i, 't>(&self, context: &ParserContext, input: &mut Parser<'i, 't>)
                           -> Result<PropertyDeclaration, ParseError<'i>> {
        match *self {
                LonghandId::BackgroundColor => {
                        longhands::background_color::parse_declared(context, input)
                }
                LonghandId::BackgroundImage => {
                        longhands::background_image::parse_declared(context, input)
                }
                LonghandId::BackgroundPositionX => {
                        longhands::background_position_x::parse_declared(context, input)
                }
                LonghandId::BackgroundPositionY => {
                        longhands::background_position_y::parse_declared(context, input)
                }
                LonghandId::BackgroundRepeat => {
                        longhands::background_repeat::parse_declared(context, input)
                }
                LonghandId::BackgroundAttachment => {
                        longhands::background_attachment::parse_declared(context, input)
                }
                LonghandId::BackgroundClip => {
                        longhands::background_clip::parse_declared(context, input)
                }
                LonghandId::BackgroundOrigin => {
                        longhands::background_origin::parse_declared(context, input)
                }
                LonghandId::BackgroundSize => {
                        longhands::background_size::parse_declared(context, input)
                }
                LonghandId::BackgroundBlendMode => {
                        longhands::background_blend_mode::parse_declared(context, input)
                }
                LonghandId::BorderTopColor => {
                        longhands::border_top_color::parse_declared(context, input)
                }
                LonghandId::BorderTopStyle => {
                        longhands::border_top_style::parse_declared(context, input)
                }
                LonghandId::BorderTopWidth => {
                        longhands::border_top_width::parse_declared(context, input)
                }
                LonghandId::BorderLeftColor => {
                        longhands::border_left_color::parse_declared(context, input)
                }
                LonghandId::BorderLeftStyle => {
                        longhands::border_left_style::parse_declared(context, input)
                }
                LonghandId::BorderLeftWidth => {
                        longhands::border_left_width::parse_declared(context, input)
                }
                LonghandId::BorderBottomColor => {
                        longhands::border_bottom_color::parse_declared(context, input)
                }
                LonghandId::BorderBottomStyle => {
                        longhands::border_bottom_style::parse_declared(context, input)
                }
                LonghandId::BorderBottomWidth => {
                        longhands::border_bottom_width::parse_declared(context, input)
                }
                LonghandId::BorderRightColor => {
                        longhands::border_right_color::parse_declared(context, input)
                }
                LonghandId::BorderRightStyle => {
                        longhands::border_right_style::parse_declared(context, input)
                }
                LonghandId::BorderRightWidth => {
                        longhands::border_right_width::parse_declared(context, input)
                }
                LonghandId::BorderBlockStartColor => {
                        longhands::border_block_start_color::parse_declared(context, input)
                }
                LonghandId::BorderBlockStartStyle => {
                        longhands::border_block_start_style::parse_declared(context, input)
                }
                LonghandId::BorderBlockStartWidth => {
                        longhands::border_block_start_width::parse_declared(context, input)
                }
                LonghandId::BorderBlockEndColor => {
                        longhands::border_block_end_color::parse_declared(context, input)
                }
                LonghandId::BorderBlockEndStyle => {
                        longhands::border_block_end_style::parse_declared(context, input)
                }
                LonghandId::BorderBlockEndWidth => {
                        longhands::border_block_end_width::parse_declared(context, input)
                }
                LonghandId::BorderInlineStartColor => {
                        longhands::border_inline_start_color::parse_declared(context, input)
                }
                LonghandId::BorderInlineStartStyle => {
                        longhands::border_inline_start_style::parse_declared(context, input)
                }
                LonghandId::BorderInlineStartWidth => {
                        longhands::border_inline_start_width::parse_declared(context, input)
                }
                LonghandId::BorderInlineEndColor => {
                        longhands::border_inline_end_color::parse_declared(context, input)
                }
                LonghandId::BorderInlineEndStyle => {
                        longhands::border_inline_end_style::parse_declared(context, input)
                }
                LonghandId::BorderInlineEndWidth => {
                        longhands::border_inline_end_width::parse_declared(context, input)
                }
                LonghandId::BorderTopLeftRadius => {
                        longhands::border_top_left_radius::parse_declared(context, input)
                }
                LonghandId::BorderTopRightRadius => {
                        longhands::border_top_right_radius::parse_declared(context, input)
                }
                LonghandId::BorderBottomRightRadius => {
                        longhands::border_bottom_right_radius::parse_declared(context, input)
                }
                LonghandId::BorderBottomLeftRadius => {
                        longhands::border_bottom_left_radius::parse_declared(context, input)
                }
                LonghandId::MozBorderTopColors => {
                        longhands::_moz_border_top_colors::parse_declared(context, input)
                }
                LonghandId::MozBorderLeftColors => {
                        longhands::_moz_border_left_colors::parse_declared(context, input)
                }
                LonghandId::MozBorderBottomColors => {
                        longhands::_moz_border_bottom_colors::parse_declared(context, input)
                }
                LonghandId::MozBorderRightColors => {
                        longhands::_moz_border_right_colors::parse_declared(context, input)
                }
                LonghandId::BoxDecorationBreak => {
                        longhands::box_decoration_break::parse_declared(context, input)
                }
                LonghandId::MozFloatEdge => {
                        longhands::_moz_float_edge::parse_declared(context, input)
                }
                LonghandId::BorderImageSource => {
                        longhands::border_image_source::parse_declared(context, input)
                }
                LonghandId::BorderImageOutset => {
                        longhands::border_image_outset::parse_declared(context, input)
                }
                LonghandId::BorderImageRepeat => {
                        longhands::border_image_repeat::parse_declared(context, input)
                }
                LonghandId::BorderImageWidth => {
                        longhands::border_image_width::parse_declared(context, input)
                }
                LonghandId::BorderImageSlice => {
                        longhands::border_image_slice::parse_declared(context, input)
                }
                LonghandId::Display => {
                        longhands::display::parse_declared(context, input)
                }
                LonghandId::MozTopLayer => {
                        longhands::_moz_top_layer::parse_declared(context, input)
                }
                LonghandId::Position => {
                        longhands::position::parse_declared(context, input)
                }
                LonghandId::Float => {
                        longhands::float::parse_declared(context, input)
                }
                LonghandId::Clear => {
                        longhands::clear::parse_declared(context, input)
                }
                LonghandId::VerticalAlign => {
                        longhands::vertical_align::parse_declared(context, input)
                }
                LonghandId::OverflowClipBox => {
                        longhands::overflow_clip_box::parse_declared(context, input)
                }
                LonghandId::OverflowX => {
                        longhands::overflow_x::parse_declared(context, input)
                }
                LonghandId::OverflowY => {
                        longhands::overflow_y::parse_declared(context, input)
                }
                LonghandId::TransitionDuration => {
                        longhands::transition_duration::parse_declared(context, input)
                }
                LonghandId::TransitionTimingFunction => {
                        longhands::transition_timing_function::parse_declared(context, input)
                }
                LonghandId::TransitionProperty => {
                        longhands::transition_property::parse_declared(context, input)
                }
                LonghandId::TransitionDelay => {
                        longhands::transition_delay::parse_declared(context, input)
                }
                LonghandId::AnimationName => {
                        longhands::animation_name::parse_declared(context, input)
                }
                LonghandId::AnimationDuration => {
                        longhands::animation_duration::parse_declared(context, input)
                }
                LonghandId::AnimationTimingFunction => {
                        longhands::animation_timing_function::parse_declared(context, input)
                }
                LonghandId::AnimationIterationCount => {
                        longhands::animation_iteration_count::parse_declared(context, input)
                }
                LonghandId::AnimationDirection => {
                        longhands::animation_direction::parse_declared(context, input)
                }
                LonghandId::AnimationPlayState => {
                        longhands::animation_play_state::parse_declared(context, input)
                }
                LonghandId::AnimationFillMode => {
                        longhands::animation_fill_mode::parse_declared(context, input)
                }
                LonghandId::AnimationDelay => {
                        longhands::animation_delay::parse_declared(context, input)
                }
                LonghandId::ScrollSnapPointsX => {
                        longhands::scroll_snap_points_x::parse_declared(context, input)
                }
                LonghandId::ScrollSnapPointsY => {
                        longhands::scroll_snap_points_y::parse_declared(context, input)
                }
                LonghandId::ScrollSnapDestination => {
                        longhands::scroll_snap_destination::parse_declared(context, input)
                }
                LonghandId::ScrollSnapCoordinate => {
                        longhands::scroll_snap_coordinate::parse_declared(context, input)
                }
                LonghandId::Transform => {
                        longhands::transform::parse_declared(context, input)
                }
                LonghandId::ScrollBehavior => {
                        longhands::scroll_behavior::parse_declared(context, input)
                }
                LonghandId::ScrollSnapTypeX => {
                        longhands::scroll_snap_type_x::parse_declared(context, input)
                }
                LonghandId::ScrollSnapTypeY => {
                        longhands::scroll_snap_type_y::parse_declared(context, input)
                }
                LonghandId::Isolation => {
                        longhands::isolation::parse_declared(context, input)
                }
                LonghandId::PageBreakAfter => {
                        longhands::page_break_after::parse_declared(context, input)
                }
                LonghandId::PageBreakBefore => {
                        longhands::page_break_before::parse_declared(context, input)
                }
                LonghandId::PageBreakInside => {
                        longhands::page_break_inside::parse_declared(context, input)
                }
                LonghandId::Resize => {
                        longhands::resize::parse_declared(context, input)
                }
                LonghandId::Perspective => {
                        longhands::perspective::parse_declared(context, input)
                }
                LonghandId::PerspectiveOrigin => {
                        longhands::perspective_origin::parse_declared(context, input)
                }
                LonghandId::BackfaceVisibility => {
                        longhands::backface_visibility::parse_declared(context, input)
                }
                LonghandId::TransformBox => {
                        longhands::transform_box::parse_declared(context, input)
                }
                LonghandId::TransformStyle => {
                        longhands::transform_style::parse_declared(context, input)
                }
                LonghandId::TransformOrigin => {
                        longhands::transform_origin::parse_declared(context, input)
                }
                LonghandId::Contain => {
                        longhands::contain::parse_declared(context, input)
                }
                LonghandId::MozAppearance => {
                        longhands::_moz_appearance::parse_declared(context, input)
                }
                LonghandId::MozBinding => {
                        longhands::_moz_binding::parse_declared(context, input)
                }
                LonghandId::MozOrient => {
                        longhands::_moz_orient::parse_declared(context, input)
                }
                LonghandId::WillChange => {
                        longhands::will_change::parse_declared(context, input)
                }
                LonghandId::ShapeOutside => {
                        longhands::shape_outside::parse_declared(context, input)
                }
                LonghandId::TouchAction => {
                        longhands::touch_action::parse_declared(context, input)
                }
                LonghandId::Color => {
                        longhands::color::parse_declared(context, input)
                }
                LonghandId::ColumnWidth => {
                        longhands::column_width::parse_declared(context, input)
                }
                LonghandId::ColumnCount => {
                        longhands::column_count::parse_declared(context, input)
                }
                LonghandId::ColumnGap => {
                        longhands::column_gap::parse_declared(context, input)
                }
                LonghandId::ColumnFill => {
                        longhands::column_fill::parse_declared(context, input)
                }
                LonghandId::ColumnRuleWidth => {
                        longhands::column_rule_width::parse_declared(context, input)
                }
                LonghandId::ColumnRuleColor => {
                        longhands::column_rule_color::parse_declared(context, input)
                }
                LonghandId::ColumnSpan => {
                        longhands::column_span::parse_declared(context, input)
                }
                LonghandId::ColumnRuleStyle => {
                        longhands::column_rule_style::parse_declared(context, input)
                }
                LonghandId::Content => {
                        longhands::content::parse_declared(context, input)
                }
                LonghandId::CounterIncrement => {
                        longhands::counter_increment::parse_declared(context, input)
                }
                LonghandId::CounterReset => {
                        longhands::counter_reset::parse_declared(context, input)
                }
                LonghandId::Opacity => {
                        longhands::opacity::parse_declared(context, input)
                }
                LonghandId::BoxShadow => {
                        longhands::box_shadow::parse_declared(context, input)
                }
                LonghandId::Clip => {
                        longhands::clip::parse_declared(context, input)
                }
                LonghandId::Filter => {
                        longhands::filter::parse_declared(context, input)
                }
                LonghandId::MixBlendMode => {
                        longhands::mix_blend_mode::parse_declared(context, input)
                }
                LonghandId::FontFamily => {
                        longhands::font_family::parse_declared(context, input)
                }
                LonghandId::FontStyle => {
                        longhands::font_style::parse_declared(context, input)
                }
                LonghandId::FontVariantCaps => {
                        longhands::font_variant_caps::parse_declared(context, input)
                }
                LonghandId::FontWeight => {
                        longhands::font_weight::parse_declared(context, input)
                }
                LonghandId::FontSize => {
                        longhands::font_size::parse_declared(context, input)
                }
                LonghandId::FontSizeAdjust => {
                        longhands::font_size_adjust::parse_declared(context, input)
                }
                LonghandId::FontSynthesis => {
                        longhands::font_synthesis::parse_declared(context, input)
                }
                LonghandId::FontStretch => {
                        longhands::font_stretch::parse_declared(context, input)
                }
                LonghandId::FontKerning => {
                        longhands::font_kerning::parse_declared(context, input)
                }
                LonghandId::FontVariantAlternates => {
                        longhands::font_variant_alternates::parse_declared(context, input)
                }
                LonghandId::FontVariantEastAsian => {
                        longhands::font_variant_east_asian::parse_declared(context, input)
                }
                LonghandId::FontVariantLigatures => {
                        longhands::font_variant_ligatures::parse_declared(context, input)
                }
                LonghandId::FontVariantNumeric => {
                        longhands::font_variant_numeric::parse_declared(context, input)
                }
                LonghandId::FontVariantPosition => {
                        longhands::font_variant_position::parse_declared(context, input)
                }
                LonghandId::FontFeatureSettings => {
                        longhands::font_feature_settings::parse_declared(context, input)
                }
                LonghandId::FontVariationSettings => {
                        longhands::font_variation_settings::parse_declared(context, input)
                }
                LonghandId::FontLanguageOverride => {
                        longhands::font_language_override::parse_declared(context, input)
                }
                LonghandId::XLang => {
                        longhands::_x_lang::parse_declared(context, input)
                }
                LonghandId::MozScriptSizeMultiplier => {
                        longhands::_moz_script_size_multiplier::parse_declared(context, input)
                }
                LonghandId::MozScriptLevel => {
                        longhands::_moz_script_level::parse_declared(context, input)
                }
                LonghandId::MozMathDisplay => {
                        longhands::_moz_math_display::parse_declared(context, input)
                }
                LonghandId::MozMathVariant => {
                        longhands::_moz_math_variant::parse_declared(context, input)
                }
                LonghandId::MozScriptMinSize => {
                        longhands::_moz_script_min_size::parse_declared(context, input)
                }
                LonghandId::XTextZoom => {
                        longhands::_x_text_zoom::parse_declared(context, input)
                }
                LonghandId::MozOsxFontSmoothing => {
                        longhands::_moz_osx_font_smoothing::parse_declared(context, input)
                }
                LonghandId::MozMinFontSizeRatio => {
                        longhands::_moz_min_font_size_ratio::parse_declared(context, input)
                }
                LonghandId::Visibility => {
                        longhands::visibility::parse_declared(context, input)
                }
                LonghandId::WritingMode => {
                        longhands::writing_mode::parse_declared(context, input)
                }
                LonghandId::Direction => {
                        longhands::direction::parse_declared(context, input)
                }
                LonghandId::TextOrientation => {
                        longhands::text_orientation::parse_declared(context, input)
                }
                LonghandId::ColorAdjust => {
                        longhands::color_adjust::parse_declared(context, input)
                }
                LonghandId::ImageRendering => {
                        longhands::image_rendering::parse_declared(context, input)
                }
                LonghandId::ImageOrientation => {
                        longhands::image_orientation::parse_declared(context, input)
                }
                LonghandId::BorderCollapse => {
                        longhands::border_collapse::parse_declared(context, input)
                }
                LonghandId::EmptyCells => {
                        longhands::empty_cells::parse_declared(context, input)
                }
                LonghandId::CaptionSide => {
                        longhands::caption_side::parse_declared(context, input)
                }
                LonghandId::BorderSpacing => {
                        longhands::border_spacing::parse_declared(context, input)
                }
                LonghandId::LineHeight => {
                        longhands::line_height::parse_declared(context, input)
                }
                LonghandId::TextTransform => {
                        longhands::text_transform::parse_declared(context, input)
                }
                LonghandId::Hyphens => {
                        longhands::hyphens::parse_declared(context, input)
                }
                LonghandId::MozTextSizeAdjust => {
                        longhands::_moz_text_size_adjust::parse_declared(context, input)
                }
                LonghandId::TextIndent => {
                        longhands::text_indent::parse_declared(context, input)
                }
                LonghandId::OverflowWrap => {
                        longhands::overflow_wrap::parse_declared(context, input)
                }
                LonghandId::WordBreak => {
                        longhands::word_break::parse_declared(context, input)
                }
                LonghandId::TextJustify => {
                        longhands::text_justify::parse_declared(context, input)
                }
                LonghandId::TextAlignLast => {
                        longhands::text_align_last::parse_declared(context, input)
                }
                LonghandId::TextAlign => {
                        longhands::text_align::parse_declared(context, input)
                }
                LonghandId::LetterSpacing => {
                        longhands::letter_spacing::parse_declared(context, input)
                }
                LonghandId::WordSpacing => {
                        longhands::word_spacing::parse_declared(context, input)
                }
                LonghandId::WhiteSpace => {
                        longhands::white_space::parse_declared(context, input)
                }
                LonghandId::TextShadow => {
                        longhands::text_shadow::parse_declared(context, input)
                }
                LonghandId::TextEmphasisStyle => {
                        longhands::text_emphasis_style::parse_declared(context, input)
                }
                LonghandId::TextEmphasisPosition => {
                        longhands::text_emphasis_position::parse_declared(context, input)
                }
                LonghandId::TextEmphasisColor => {
                        longhands::text_emphasis_color::parse_declared(context, input)
                }
                LonghandId::MozTabSize => {
                        longhands::_moz_tab_size::parse_declared(context, input)
                }
                LonghandId::WebkitTextFillColor => {
                        longhands::_webkit_text_fill_color::parse_declared(context, input)
                }
                LonghandId::WebkitTextStrokeColor => {
                        longhands::_webkit_text_stroke_color::parse_declared(context, input)
                }
                LonghandId::WebkitTextStrokeWidth => {
                        longhands::_webkit_text_stroke_width::parse_declared(context, input)
                }
                LonghandId::RubyAlign => {
                        longhands::ruby_align::parse_declared(context, input)
                }
                LonghandId::RubyPosition => {
                        longhands::ruby_position::parse_declared(context, input)
                }
                LonghandId::TextCombineUpright => {
                        longhands::text_combine_upright::parse_declared(context, input)
                }
                LonghandId::TextRendering => {
                        longhands::text_rendering::parse_declared(context, input)
                }
                LonghandId::MozControlCharacterVisibility => {
                        longhands::_moz_control_character_visibility::parse_declared(context, input)
                }
                LonghandId::ListStylePosition => {
                        longhands::list_style_position::parse_declared(context, input)
                }
                LonghandId::ListStyleType => {
                        longhands::list_style_type::parse_declared(context, input)
                }
                LonghandId::ListStyleImage => {
                        longhands::list_style_image::parse_declared(context, input)
                }
                LonghandId::Quotes => {
                        longhands::quotes::parse_declared(context, input)
                }
                LonghandId::MozImageRegion => {
                        longhands::_moz_image_region::parse_declared(context, input)
                }
                LonghandId::MarginTop => {
                        longhands::margin_top::parse_declared(context, input)
                }
                LonghandId::MarginLeft => {
                        longhands::margin_left::parse_declared(context, input)
                }
                LonghandId::MarginBottom => {
                        longhands::margin_bottom::parse_declared(context, input)
                }
                LonghandId::MarginRight => {
                        longhands::margin_right::parse_declared(context, input)
                }
                LonghandId::MarginBlockStart => {
                        longhands::margin_block_start::parse_declared(context, input)
                }
                LonghandId::MarginBlockEnd => {
                        longhands::margin_block_end::parse_declared(context, input)
                }
                LonghandId::MarginInlineStart => {
                        longhands::margin_inline_start::parse_declared(context, input)
                }
                LonghandId::MarginInlineEnd => {
                        longhands::margin_inline_end::parse_declared(context, input)
                }
                LonghandId::OutlineColor => {
                        longhands::outline_color::parse_declared(context, input)
                }
                LonghandId::OutlineStyle => {
                        longhands::outline_style::parse_declared(context, input)
                }
                LonghandId::OutlineWidth => {
                        longhands::outline_width::parse_declared(context, input)
                }
                LonghandId::MozOutlineRadiusTopleft => {
                        longhands::_moz_outline_radius_topleft::parse_declared(context, input)
                }
                LonghandId::MozOutlineRadiusTopright => {
                        longhands::_moz_outline_radius_topright::parse_declared(context, input)
                }
                LonghandId::MozOutlineRadiusBottomright => {
                        longhands::_moz_outline_radius_bottomright::parse_declared(context, input)
                }
                LonghandId::MozOutlineRadiusBottomleft => {
                        longhands::_moz_outline_radius_bottomleft::parse_declared(context, input)
                }
                LonghandId::OutlineOffset => {
                        longhands::outline_offset::parse_declared(context, input)
                }
                LonghandId::PaddingTop => {
                        longhands::padding_top::parse_declared(context, input)
                }
                LonghandId::PaddingLeft => {
                        longhands::padding_left::parse_declared(context, input)
                }
                LonghandId::PaddingBottom => {
                        longhands::padding_bottom::parse_declared(context, input)
                }
                LonghandId::PaddingRight => {
                        longhands::padding_right::parse_declared(context, input)
                }
                LonghandId::PaddingBlockStart => {
                        longhands::padding_block_start::parse_declared(context, input)
                }
                LonghandId::PaddingBlockEnd => {
                        longhands::padding_block_end::parse_declared(context, input)
                }
                LonghandId::PaddingInlineStart => {
                        longhands::padding_inline_start::parse_declared(context, input)
                }
                LonghandId::PaddingInlineEnd => {
                        longhands::padding_inline_end::parse_declared(context, input)
                }
                LonghandId::Cursor => {
                        longhands::cursor::parse_declared(context, input)
                }
                LonghandId::PointerEvents => {
                        longhands::pointer_events::parse_declared(context, input)
                }
                LonghandId::MozUserInput => {
                        longhands::_moz_user_input::parse_declared(context, input)
                }
                LonghandId::MozUserModify => {
                        longhands::_moz_user_modify::parse_declared(context, input)
                }
                LonghandId::MozUserFocus => {
                        longhands::_moz_user_focus::parse_declared(context, input)
                }
                LonghandId::CaretColor => {
                        longhands::caret_color::parse_declared(context, input)
                }
                LonghandId::Top => {
                        longhands::top::parse_declared(context, input)
                }
                LonghandId::Left => {
                        longhands::left::parse_declared(context, input)
                }
                LonghandId::Bottom => {
                        longhands::bottom::parse_declared(context, input)
                }
                LonghandId::Right => {
                        longhands::right::parse_declared(context, input)
                }
                LonghandId::OffsetBlockStart => {
                        longhands::offset_block_start::parse_declared(context, input)
                }
                LonghandId::OffsetBlockEnd => {
                        longhands::offset_block_end::parse_declared(context, input)
                }
                LonghandId::OffsetInlineStart => {
                        longhands::offset_inline_start::parse_declared(context, input)
                }
                LonghandId::OffsetInlineEnd => {
                        longhands::offset_inline_end::parse_declared(context, input)
                }
                LonghandId::ZIndex => {
                        longhands::z_index::parse_declared(context, input)
                }
                LonghandId::FlexDirection => {
                        longhands::flex_direction::parse_declared(context, input)
                }
                LonghandId::FlexWrap => {
                        longhands::flex_wrap::parse_declared(context, input)
                }
                LonghandId::JustifyContent => {
                        longhands::justify_content::parse_declared(context, input)
                }
                LonghandId::AlignContent => {
                        longhands::align_content::parse_declared(context, input)
                }
                LonghandId::AlignItems => {
                        longhands::align_items::parse_declared(context, input)
                }
                LonghandId::JustifyItems => {
                        longhands::justify_items::parse_declared(context, input)
                }
                LonghandId::FlexGrow => {
                        longhands::flex_grow::parse_declared(context, input)
                }
                LonghandId::FlexShrink => {
                        longhands::flex_shrink::parse_declared(context, input)
                }
                LonghandId::AlignSelf => {
                        longhands::align_self::parse_declared(context, input)
                }
                LonghandId::JustifySelf => {
                        longhands::justify_self::parse_declared(context, input)
                }
                LonghandId::Order => {
                        longhands::order::parse_declared(context, input)
                }
                LonghandId::FlexBasis => {
                        longhands::flex_basis::parse_declared(context, input)
                }
                LonghandId::Width => {
                        longhands::width::parse_declared(context, input)
                }
                LonghandId::MinWidth => {
                        longhands::min_width::parse_declared(context, input)
                }
                LonghandId::MaxWidth => {
                        longhands::max_width::parse_declared(context, input)
                }
                LonghandId::Height => {
                        longhands::height::parse_declared(context, input)
                }
                LonghandId::MinHeight => {
                        longhands::min_height::parse_declared(context, input)
                }
                LonghandId::MaxHeight => {
                        longhands::max_height::parse_declared(context, input)
                }
                LonghandId::BlockSize => {
                        longhands::block_size::parse_declared(context, input)
                }
                LonghandId::MinBlockSize => {
                        longhands::min_block_size::parse_declared(context, input)
                }
                LonghandId::MaxBlockSize => {
                        longhands::max_block_size::parse_declared(context, input)
                }
                LonghandId::InlineSize => {
                        longhands::inline_size::parse_declared(context, input)
                }
                LonghandId::MinInlineSize => {
                        longhands::min_inline_size::parse_declared(context, input)
                }
                LonghandId::MaxInlineSize => {
                        longhands::max_inline_size::parse_declared(context, input)
                }
                LonghandId::BoxSizing => {
                        longhands::box_sizing::parse_declared(context, input)
                }
                LonghandId::ObjectFit => {
                        longhands::object_fit::parse_declared(context, input)
                }
                LonghandId::ObjectPosition => {
                        longhands::object_position::parse_declared(context, input)
                }
                LonghandId::GridRowGap => {
                        longhands::grid_row_gap::parse_declared(context, input)
                }
                LonghandId::GridRowStart => {
                        longhands::grid_row_start::parse_declared(context, input)
                }
                LonghandId::GridRowEnd => {
                        longhands::grid_row_end::parse_declared(context, input)
                }
                LonghandId::GridAutoRows => {
                        longhands::grid_auto_rows::parse_declared(context, input)
                }
                LonghandId::GridTemplateRows => {
                        longhands::grid_template_rows::parse_declared(context, input)
                }
                LonghandId::GridColumnGap => {
                        longhands::grid_column_gap::parse_declared(context, input)
                }
                LonghandId::GridColumnStart => {
                        longhands::grid_column_start::parse_declared(context, input)
                }
                LonghandId::GridColumnEnd => {
                        longhands::grid_column_end::parse_declared(context, input)
                }
                LonghandId::GridAutoColumns => {
                        longhands::grid_auto_columns::parse_declared(context, input)
                }
                LonghandId::GridTemplateColumns => {
                        longhands::grid_template_columns::parse_declared(context, input)
                }
                LonghandId::GridAutoFlow => {
                        longhands::grid_auto_flow::parse_declared(context, input)
                }
                LonghandId::GridTemplateAreas => {
                        longhands::grid_template_areas::parse_declared(context, input)
                }
                LonghandId::TableLayout => {
                        longhands::table_layout::parse_declared(context, input)
                }
                LonghandId::XSpan => {
                        longhands::_x_span::parse_declared(context, input)
                }
                LonghandId::TextOverflow => {
                        longhands::text_overflow::parse_declared(context, input)
                }
                LonghandId::UnicodeBidi => {
                        longhands::unicode_bidi::parse_declared(context, input)
                }
                LonghandId::TextDecorationLine => {
                        longhands::text_decoration_line::parse_declared(context, input)
                }
                LonghandId::TextDecorationStyle => {
                        longhands::text_decoration_style::parse_declared(context, input)
                }
                LonghandId::TextDecorationColor => {
                        longhands::text_decoration_color::parse_declared(context, input)
                }
                LonghandId::InitialLetter => {
                        longhands::initial_letter::parse_declared(context, input)
                }
                LonghandId::ImeMode => {
                        longhands::ime_mode::parse_declared(context, input)
                }
                LonghandId::MozUserSelect => {
                        longhands::_moz_user_select::parse_declared(context, input)
                }
                LonghandId::MozWindowDragging => {
                        longhands::_moz_window_dragging::parse_declared(context, input)
                }
                LonghandId::MozWindowShadow => {
                        longhands::_moz_window_shadow::parse_declared(context, input)
                }
                LonghandId::MozForceBrokenImageIcon => {
                        longhands::_moz_force_broken_image_icon::parse_declared(context, input)
                }
                LonghandId::TextAnchor => {
                        longhands::text_anchor::parse_declared(context, input)
                }
                LonghandId::ColorInterpolation => {
                        longhands::color_interpolation::parse_declared(context, input)
                }
                LonghandId::ColorInterpolationFilters => {
                        longhands::color_interpolation_filters::parse_declared(context, input)
                }
                LonghandId::Fill => {
                        longhands::fill::parse_declared(context, input)
                }
                LonghandId::FillOpacity => {
                        longhands::fill_opacity::parse_declared(context, input)
                }
                LonghandId::FillRule => {
                        longhands::fill_rule::parse_declared(context, input)
                }
                LonghandId::ShapeRendering => {
                        longhands::shape_rendering::parse_declared(context, input)
                }
                LonghandId::Stroke => {
                        longhands::stroke::parse_declared(context, input)
                }
                LonghandId::StrokeWidth => {
                        longhands::stroke_width::parse_declared(context, input)
                }
                LonghandId::StrokeLinecap => {
                        longhands::stroke_linecap::parse_declared(context, input)
                }
                LonghandId::StrokeLinejoin => {
                        longhands::stroke_linejoin::parse_declared(context, input)
                }
                LonghandId::StrokeMiterlimit => {
                        longhands::stroke_miterlimit::parse_declared(context, input)
                }
                LonghandId::StrokeOpacity => {
                        longhands::stroke_opacity::parse_declared(context, input)
                }
                LonghandId::StrokeDasharray => {
                        longhands::stroke_dasharray::parse_declared(context, input)
                }
                LonghandId::StrokeDashoffset => {
                        longhands::stroke_dashoffset::parse_declared(context, input)
                }
                LonghandId::ClipRule => {
                        longhands::clip_rule::parse_declared(context, input)
                }
                LonghandId::MarkerStart => {
                        longhands::marker_start::parse_declared(context, input)
                }
                LonghandId::MarkerMid => {
                        longhands::marker_mid::parse_declared(context, input)
                }
                LonghandId::MarkerEnd => {
                        longhands::marker_end::parse_declared(context, input)
                }
                LonghandId::PaintOrder => {
                        longhands::paint_order::parse_declared(context, input)
                }
                LonghandId::MozContextProperties => {
                        longhands::_moz_context_properties::parse_declared(context, input)
                }
                LonghandId::DominantBaseline => {
                        longhands::dominant_baseline::parse_declared(context, input)
                }
                LonghandId::VectorEffect => {
                        longhands::vector_effect::parse_declared(context, input)
                }
                LonghandId::StopColor => {
                        longhands::stop_color::parse_declared(context, input)
                }
                LonghandId::StopOpacity => {
                        longhands::stop_opacity::parse_declared(context, input)
                }
                LonghandId::FloodColor => {
                        longhands::flood_color::parse_declared(context, input)
                }
                LonghandId::FloodOpacity => {
                        longhands::flood_opacity::parse_declared(context, input)
                }
                LonghandId::LightingColor => {
                        longhands::lighting_color::parse_declared(context, input)
                }
                LonghandId::MaskType => {
                        longhands::mask_type::parse_declared(context, input)
                }
                LonghandId::ClipPath => {
                        longhands::clip_path::parse_declared(context, input)
                }
                LonghandId::MaskMode => {
                        longhands::mask_mode::parse_declared(context, input)
                }
                LonghandId::MaskRepeat => {
                        longhands::mask_repeat::parse_declared(context, input)
                }
                LonghandId::MaskPositionX => {
                        longhands::mask_position_x::parse_declared(context, input)
                }
                LonghandId::MaskPositionY => {
                        longhands::mask_position_y::parse_declared(context, input)
                }
                LonghandId::MaskClip => {
                        longhands::mask_clip::parse_declared(context, input)
                }
                LonghandId::MaskOrigin => {
                        longhands::mask_origin::parse_declared(context, input)
                }
                LonghandId::MaskSize => {
                        longhands::mask_size::parse_declared(context, input)
                }
                LonghandId::MaskComposite => {
                        longhands::mask_composite::parse_declared(context, input)
                }
                LonghandId::MaskImage => {
                        longhands::mask_image::parse_declared(context, input)
                }
                LonghandId::MozBoxAlign => {
                        longhands::_moz_box_align::parse_declared(context, input)
                }
                LonghandId::MozBoxDirection => {
                        longhands::_moz_box_direction::parse_declared(context, input)
                }
                LonghandId::MozBoxFlex => {
                        longhands::_moz_box_flex::parse_declared(context, input)
                }
                LonghandId::MozBoxOrient => {
                        longhands::_moz_box_orient::parse_declared(context, input)
                }
                LonghandId::MozBoxPack => {
                        longhands::_moz_box_pack::parse_declared(context, input)
                }
                LonghandId::MozStackSizing => {
                        longhands::_moz_stack_sizing::parse_declared(context, input)
                }
                LonghandId::MozBoxOrdinalGroup => {
                        longhands::_moz_box_ordinal_group::parse_declared(context, input)
                }
        }
    }

    /// If this is a logical property, return the corresponding physical one in the given writing mode.
    /// Otherwise, return unchanged.
    pub fn to_physical(&self, wm: WritingMode) -> Self {
        match *self {
                    LonghandId::BorderBlockStartColor => {
                        
    
        use logical_geometry::PhysicalSide;
        match wm.block_start_physical_side() {
                PhysicalSide::Top => {
                    
                                LonghandId::BorderTopColor
                            
                }
                PhysicalSide::Left => {
                    
                                LonghandId::BorderLeftColor
                            
                }
                PhysicalSide::Bottom => {
                    
                                LonghandId::BorderBottomColor
                            
                }
                PhysicalSide::Right => {
                    
                                LonghandId::BorderRightColor
                            
                }
        }

                    }
                    LonghandId::BorderBlockStartStyle => {
                        
    
        use logical_geometry::PhysicalSide;
        match wm.block_start_physical_side() {
                PhysicalSide::Top => {
                    
                                LonghandId::BorderTopStyle
                            
                }
                PhysicalSide::Left => {
                    
                                LonghandId::BorderLeftStyle
                            
                }
                PhysicalSide::Bottom => {
                    
                                LonghandId::BorderBottomStyle
                            
                }
                PhysicalSide::Right => {
                    
                                LonghandId::BorderRightStyle
                            
                }
        }

                    }
                    LonghandId::BorderBlockStartWidth => {
                        
    
        use logical_geometry::PhysicalSide;
        match wm.block_start_physical_side() {
                PhysicalSide::Top => {
                    
                                LonghandId::BorderTopWidth
                            
                }
                PhysicalSide::Left => {
                    
                                LonghandId::BorderLeftWidth
                            
                }
                PhysicalSide::Bottom => {
                    
                                LonghandId::BorderBottomWidth
                            
                }
                PhysicalSide::Right => {
                    
                                LonghandId::BorderRightWidth
                            
                }
        }

                    }
                    LonghandId::BorderBlockEndColor => {
                        
    
        use logical_geometry::PhysicalSide;
        match wm.block_end_physical_side() {
                PhysicalSide::Top => {
                    
                                LonghandId::BorderTopColor
                            
                }
                PhysicalSide::Left => {
                    
                                LonghandId::BorderLeftColor
                            
                }
                PhysicalSide::Bottom => {
                    
                                LonghandId::BorderBottomColor
                            
                }
                PhysicalSide::Right => {
                    
                                LonghandId::BorderRightColor
                            
                }
        }

                    }
                    LonghandId::BorderBlockEndStyle => {
                        
    
        use logical_geometry::PhysicalSide;
        match wm.block_end_physical_side() {
                PhysicalSide::Top => {
                    
                                LonghandId::BorderTopStyle
                            
                }
                PhysicalSide::Left => {
                    
                                LonghandId::BorderLeftStyle
                            
                }
                PhysicalSide::Bottom => {
                    
                                LonghandId::BorderBottomStyle
                            
                }
                PhysicalSide::Right => {
                    
                                LonghandId::BorderRightStyle
                            
                }
        }

                    }
                    LonghandId::BorderBlockEndWidth => {
                        
    
        use logical_geometry::PhysicalSide;
        match wm.block_end_physical_side() {
                PhysicalSide::Top => {
                    
                                LonghandId::BorderTopWidth
                            
                }
                PhysicalSide::Left => {
                    
                                LonghandId::BorderLeftWidth
                            
                }
                PhysicalSide::Bottom => {
                    
                                LonghandId::BorderBottomWidth
                            
                }
                PhysicalSide::Right => {
                    
                                LonghandId::BorderRightWidth
                            
                }
        }

                    }
                    LonghandId::BorderInlineStartColor => {
                        
    
        use logical_geometry::PhysicalSide;
        match wm.inline_start_physical_side() {
                PhysicalSide::Top => {
                    
                                LonghandId::BorderTopColor
                            
                }
                PhysicalSide::Left => {
                    
                                LonghandId::BorderLeftColor
                            
                }
                PhysicalSide::Bottom => {
                    
                                LonghandId::BorderBottomColor
                            
                }
                PhysicalSide::Right => {
                    
                                LonghandId::BorderRightColor
                            
                }
        }

                    }
                    LonghandId::BorderInlineStartStyle => {
                        
    
        use logical_geometry::PhysicalSide;
        match wm.inline_start_physical_side() {
                PhysicalSide::Top => {
                    
                                LonghandId::BorderTopStyle
                            
                }
                PhysicalSide::Left => {
                    
                                LonghandId::BorderLeftStyle
                            
                }
                PhysicalSide::Bottom => {
                    
                                LonghandId::BorderBottomStyle
                            
                }
                PhysicalSide::Right => {
                    
                                LonghandId::BorderRightStyle
                            
                }
        }

                    }
                    LonghandId::BorderInlineStartWidth => {
                        
    
        use logical_geometry::PhysicalSide;
        match wm.inline_start_physical_side() {
                PhysicalSide::Top => {
                    
                                LonghandId::BorderTopWidth
                            
                }
                PhysicalSide::Left => {
                    
                                LonghandId::BorderLeftWidth
                            
                }
                PhysicalSide::Bottom => {
                    
                                LonghandId::BorderBottomWidth
                            
                }
                PhysicalSide::Right => {
                    
                                LonghandId::BorderRightWidth
                            
                }
        }

                    }
                    LonghandId::BorderInlineEndColor => {
                        
    
        use logical_geometry::PhysicalSide;
        match wm.inline_end_physical_side() {
                PhysicalSide::Top => {
                    
                                LonghandId::BorderTopColor
                            
                }
                PhysicalSide::Left => {
                    
                                LonghandId::BorderLeftColor
                            
                }
                PhysicalSide::Bottom => {
                    
                                LonghandId::BorderBottomColor
                            
                }
                PhysicalSide::Right => {
                    
                                LonghandId::BorderRightColor
                            
                }
        }

                    }
                    LonghandId::BorderInlineEndStyle => {
                        
    
        use logical_geometry::PhysicalSide;
        match wm.inline_end_physical_side() {
                PhysicalSide::Top => {
                    
                                LonghandId::BorderTopStyle
                            
                }
                PhysicalSide::Left => {
                    
                                LonghandId::BorderLeftStyle
                            
                }
                PhysicalSide::Bottom => {
                    
                                LonghandId::BorderBottomStyle
                            
                }
                PhysicalSide::Right => {
                    
                                LonghandId::BorderRightStyle
                            
                }
        }

                    }
                    LonghandId::BorderInlineEndWidth => {
                        
    
        use logical_geometry::PhysicalSide;
        match wm.inline_end_physical_side() {
                PhysicalSide::Top => {
                    
                                LonghandId::BorderTopWidth
                            
                }
                PhysicalSide::Left => {
                    
                                LonghandId::BorderLeftWidth
                            
                }
                PhysicalSide::Bottom => {
                    
                                LonghandId::BorderBottomWidth
                            
                }
                PhysicalSide::Right => {
                    
                                LonghandId::BorderRightWidth
                            
                }
        }

                    }
                    LonghandId::MarginBlockStart => {
                        
    
        use logical_geometry::PhysicalSide;
        match wm.block_start_physical_side() {
                PhysicalSide::Top => {
                    
                                LonghandId::MarginTop
                            
                }
                PhysicalSide::Left => {
                    
                                LonghandId::MarginLeft
                            
                }
                PhysicalSide::Bottom => {
                    
                                LonghandId::MarginBottom
                            
                }
                PhysicalSide::Right => {
                    
                                LonghandId::MarginRight
                            
                }
        }

                    }
                    LonghandId::MarginBlockEnd => {
                        
    
        use logical_geometry::PhysicalSide;
        match wm.block_end_physical_side() {
                PhysicalSide::Top => {
                    
                                LonghandId::MarginTop
                            
                }
                PhysicalSide::Left => {
                    
                                LonghandId::MarginLeft
                            
                }
                PhysicalSide::Bottom => {
                    
                                LonghandId::MarginBottom
                            
                }
                PhysicalSide::Right => {
                    
                                LonghandId::MarginRight
                            
                }
        }

                    }
                    LonghandId::MarginInlineStart => {
                        
    
        use logical_geometry::PhysicalSide;
        match wm.inline_start_physical_side() {
                PhysicalSide::Top => {
                    
                                LonghandId::MarginTop
                            
                }
                PhysicalSide::Left => {
                    
                                LonghandId::MarginLeft
                            
                }
                PhysicalSide::Bottom => {
                    
                                LonghandId::MarginBottom
                            
                }
                PhysicalSide::Right => {
                    
                                LonghandId::MarginRight
                            
                }
        }

                    }
                    LonghandId::MarginInlineEnd => {
                        
    
        use logical_geometry::PhysicalSide;
        match wm.inline_end_physical_side() {
                PhysicalSide::Top => {
                    
                                LonghandId::MarginTop
                            
                }
                PhysicalSide::Left => {
                    
                                LonghandId::MarginLeft
                            
                }
                PhysicalSide::Bottom => {
                    
                                LonghandId::MarginBottom
                            
                }
                PhysicalSide::Right => {
                    
                                LonghandId::MarginRight
                            
                }
        }

                    }
                    LonghandId::PaddingBlockStart => {
                        
    
        use logical_geometry::PhysicalSide;
        match wm.block_start_physical_side() {
                PhysicalSide::Top => {
                    
                                LonghandId::PaddingTop
                            
                }
                PhysicalSide::Left => {
                    
                                LonghandId::PaddingLeft
                            
                }
                PhysicalSide::Bottom => {
                    
                                LonghandId::PaddingBottom
                            
                }
                PhysicalSide::Right => {
                    
                                LonghandId::PaddingRight
                            
                }
        }

                    }
                    LonghandId::PaddingBlockEnd => {
                        
    
        use logical_geometry::PhysicalSide;
        match wm.block_end_physical_side() {
                PhysicalSide::Top => {
                    
                                LonghandId::PaddingTop
                            
                }
                PhysicalSide::Left => {
                    
                                LonghandId::PaddingLeft
                            
                }
                PhysicalSide::Bottom => {
                    
                                LonghandId::PaddingBottom
                            
                }
                PhysicalSide::Right => {
                    
                                LonghandId::PaddingRight
                            
                }
        }

                    }
                    LonghandId::PaddingInlineStart => {
                        
    
        use logical_geometry::PhysicalSide;
        match wm.inline_start_physical_side() {
                PhysicalSide::Top => {
                    
                                LonghandId::PaddingTop
                            
                }
                PhysicalSide::Left => {
                    
                                LonghandId::PaddingLeft
                            
                }
                PhysicalSide::Bottom => {
                    
                                LonghandId::PaddingBottom
                            
                }
                PhysicalSide::Right => {
                    
                                LonghandId::PaddingRight
                            
                }
        }

                    }
                    LonghandId::PaddingInlineEnd => {
                        
    
        use logical_geometry::PhysicalSide;
        match wm.inline_end_physical_side() {
                PhysicalSide::Top => {
                    
                                LonghandId::PaddingTop
                            
                }
                PhysicalSide::Left => {
                    
                                LonghandId::PaddingLeft
                            
                }
                PhysicalSide::Bottom => {
                    
                                LonghandId::PaddingBottom
                            
                }
                PhysicalSide::Right => {
                    
                                LonghandId::PaddingRight
                            
                }
        }

                    }
                    LonghandId::OffsetBlockStart => {
                        
    
        use logical_geometry::PhysicalSide;
        match wm.block_start_physical_side() {
                PhysicalSide::Top => {
                    
                                LonghandId::Top
                            
                }
                PhysicalSide::Left => {
                    
                                LonghandId::Left
                            
                }
                PhysicalSide::Bottom => {
                    
                                LonghandId::Bottom
                            
                }
                PhysicalSide::Right => {
                    
                                LonghandId::Right
                            
                }
        }

                    }
                    LonghandId::OffsetBlockEnd => {
                        
    
        use logical_geometry::PhysicalSide;
        match wm.block_end_physical_side() {
                PhysicalSide::Top => {
                    
                                LonghandId::Top
                            
                }
                PhysicalSide::Left => {
                    
                                LonghandId::Left
                            
                }
                PhysicalSide::Bottom => {
                    
                                LonghandId::Bottom
                            
                }
                PhysicalSide::Right => {
                    
                                LonghandId::Right
                            
                }
        }

                    }
                    LonghandId::OffsetInlineStart => {
                        
    
        use logical_geometry::PhysicalSide;
        match wm.inline_start_physical_side() {
                PhysicalSide::Top => {
                    
                                LonghandId::Top
                            
                }
                PhysicalSide::Left => {
                    
                                LonghandId::Left
                            
                }
                PhysicalSide::Bottom => {
                    
                                LonghandId::Bottom
                            
                }
                PhysicalSide::Right => {
                    
                                LonghandId::Right
                            
                }
        }

                    }
                    LonghandId::OffsetInlineEnd => {
                        
    
        use logical_geometry::PhysicalSide;
        match wm.inline_end_physical_side() {
                PhysicalSide::Top => {
                    
                                LonghandId::Top
                            
                }
                PhysicalSide::Left => {
                    
                                LonghandId::Left
                            
                }
                PhysicalSide::Bottom => {
                    
                                LonghandId::Bottom
                            
                }
                PhysicalSide::Right => {
                    
                                LonghandId::Right
                            
                }
        }

                    }
                    LonghandId::BlockSize => {
                        
    
        
        if wm.is_vertical() {
            
                                LonghandId::Width
                            
        } else {
            
                                LonghandId::Height
                            
        }

                    }
                    LonghandId::MinBlockSize => {
                        
    
        
        if wm.is_vertical() {
            
                                LonghandId::MinWidth
                            
        } else {
            
                                LonghandId::MinHeight
                            
        }

                    }
                    LonghandId::MaxBlockSize => {
                        
    
        
        if wm.is_vertical() {
            
                                LonghandId::MaxWidth
                            
        } else {
            
                                LonghandId::MaxHeight
                            
        }

                    }
                    LonghandId::InlineSize => {
                        
    
        
        if wm.is_vertical() {
            
                                LonghandId::Height
                            
        } else {
            
                                LonghandId::Width
                            
        }

                    }
                    LonghandId::MinInlineSize => {
                        
    
        
        if wm.is_vertical() {
            
                                LonghandId::MinHeight
                            
        } else {
            
                                LonghandId::MinWidth
                            
        }

                    }
                    LonghandId::MaxInlineSize => {
                        
    
        
        if wm.is_vertical() {
            
                                LonghandId::MaxHeight
                            
        } else {
            
                                LonghandId::MaxWidth
                            
        }

                    }
            _ => *self
        }
    }

    /// Returns PropertyFlags for given longhand property.
    pub fn flags(&self) -> PropertyFlags {
        match *self {
                LonghandId::BackgroundColor =>
                        APPLIES_TO_FIRST_LETTER |
                        APPLIES_TO_FIRST_LINE |
                        APPLIES_TO_PLACEHOLDER |
                    PropertyFlags::empty(),
                LonghandId::BackgroundImage =>
                        APPLIES_TO_FIRST_LETTER |
                        APPLIES_TO_FIRST_LINE |
                        APPLIES_TO_PLACEHOLDER |
                    PropertyFlags::empty(),
                LonghandId::BackgroundPositionX =>
                        APPLIES_TO_FIRST_LETTER |
                        APPLIES_TO_FIRST_LINE |
                        APPLIES_TO_PLACEHOLDER |
                    PropertyFlags::empty(),
                LonghandId::BackgroundPositionY =>
                        APPLIES_TO_FIRST_LETTER |
                        APPLIES_TO_FIRST_LINE |
                        APPLIES_TO_PLACEHOLDER |
                    PropertyFlags::empty(),
                LonghandId::BackgroundRepeat =>
                        APPLIES_TO_FIRST_LETTER |
                        APPLIES_TO_FIRST_LINE |
                        APPLIES_TO_PLACEHOLDER |
                    PropertyFlags::empty(),
                LonghandId::BackgroundAttachment =>
                        APPLIES_TO_FIRST_LETTER |
                        APPLIES_TO_FIRST_LINE |
                        APPLIES_TO_PLACEHOLDER |
                    PropertyFlags::empty(),
                LonghandId::BackgroundClip =>
                        APPLIES_TO_FIRST_LETTER |
                        APPLIES_TO_FIRST_LINE |
                        APPLIES_TO_PLACEHOLDER |
                    PropertyFlags::empty(),
                LonghandId::BackgroundOrigin =>
                        APPLIES_TO_FIRST_LETTER |
                        APPLIES_TO_FIRST_LINE |
                        APPLIES_TO_PLACEHOLDER |
                    PropertyFlags::empty(),
                LonghandId::BackgroundSize =>
                        APPLIES_TO_FIRST_LETTER |
                        APPLIES_TO_FIRST_LINE |
                        APPLIES_TO_PLACEHOLDER |
                    PropertyFlags::empty(),
                LonghandId::BackgroundBlendMode =>
                        APPLIES_TO_FIRST_LETTER |
                        APPLIES_TO_FIRST_LINE |
                        APPLIES_TO_PLACEHOLDER |
                    PropertyFlags::empty(),
                LonghandId::BorderTopColor =>
                        APPLIES_TO_FIRST_LETTER |
                    PropertyFlags::empty(),
                LonghandId::BorderTopStyle =>
                        APPLIES_TO_FIRST_LETTER |
                    PropertyFlags::empty(),
                LonghandId::BorderTopWidth =>
                        APPLIES_TO_FIRST_LETTER |
                    PropertyFlags::empty(),
                LonghandId::BorderLeftColor =>
                        APPLIES_TO_FIRST_LETTER |
                    PropertyFlags::empty(),
                LonghandId::BorderLeftStyle =>
                        APPLIES_TO_FIRST_LETTER |
                    PropertyFlags::empty(),
                LonghandId::BorderLeftWidth =>
                        APPLIES_TO_FIRST_LETTER |
                    PropertyFlags::empty(),
                LonghandId::BorderBottomColor =>
                        APPLIES_TO_FIRST_LETTER |
                    PropertyFlags::empty(),
                LonghandId::BorderBottomStyle =>
                        APPLIES_TO_FIRST_LETTER |
                    PropertyFlags::empty(),
                LonghandId::BorderBottomWidth =>
                        APPLIES_TO_FIRST_LETTER |
                    PropertyFlags::empty(),
                LonghandId::BorderRightColor =>
                        APPLIES_TO_FIRST_LETTER |
                    PropertyFlags::empty(),
                LonghandId::BorderRightStyle =>
                        APPLIES_TO_FIRST_LETTER |
                    PropertyFlags::empty(),
                LonghandId::BorderRightWidth =>
                        APPLIES_TO_FIRST_LETTER |
                    PropertyFlags::empty(),
                LonghandId::BorderBlockStartColor =>
                        APPLIES_TO_FIRST_LETTER |
                    PropertyFlags::empty(),
                LonghandId::BorderBlockStartStyle =>
                        APPLIES_TO_FIRST_LETTER |
                    PropertyFlags::empty(),
                LonghandId::BorderBlockStartWidth =>
                        APPLIES_TO_FIRST_LETTER |
                    PropertyFlags::empty(),
                LonghandId::BorderBlockEndColor =>
                        APPLIES_TO_FIRST_LETTER |
                    PropertyFlags::empty(),
                LonghandId::BorderBlockEndStyle =>
                        APPLIES_TO_FIRST_LETTER |
                    PropertyFlags::empty(),
                LonghandId::BorderBlockEndWidth =>
                        APPLIES_TO_FIRST_LETTER |
                    PropertyFlags::empty(),
                LonghandId::BorderInlineStartColor =>
                        APPLIES_TO_FIRST_LETTER |
                    PropertyFlags::empty(),
                LonghandId::BorderInlineStartStyle =>
                        APPLIES_TO_FIRST_LETTER |
                    PropertyFlags::empty(),
                LonghandId::BorderInlineStartWidth =>
                        APPLIES_TO_FIRST_LETTER |
                    PropertyFlags::empty(),
                LonghandId::BorderInlineEndColor =>
                        APPLIES_TO_FIRST_LETTER |
                    PropertyFlags::empty(),
                LonghandId::BorderInlineEndStyle =>
                        APPLIES_TO_FIRST_LETTER |
                    PropertyFlags::empty(),
                LonghandId::BorderInlineEndWidth =>
                        APPLIES_TO_FIRST_LETTER |
                    PropertyFlags::empty(),
                LonghandId::BorderTopLeftRadius =>
                        APPLIES_TO_FIRST_LETTER |
                    PropertyFlags::empty(),
                LonghandId::BorderTopRightRadius =>
                        APPLIES_TO_FIRST_LETTER |
                    PropertyFlags::empty(),
                LonghandId::BorderBottomRightRadius =>
                        APPLIES_TO_FIRST_LETTER |
                    PropertyFlags::empty(),
                LonghandId::BorderBottomLeftRadius =>
                        APPLIES_TO_FIRST_LETTER |
                    PropertyFlags::empty(),
                LonghandId::MozBorderTopColors =>
                        APPLIES_TO_FIRST_LETTER |
                    PropertyFlags::empty(),
                LonghandId::MozBorderLeftColors =>
                        APPLIES_TO_FIRST_LETTER |
                    PropertyFlags::empty(),
                LonghandId::MozBorderBottomColors =>
                        APPLIES_TO_FIRST_LETTER |
                    PropertyFlags::empty(),
                LonghandId::MozBorderRightColors =>
                        APPLIES_TO_FIRST_LETTER |
                    PropertyFlags::empty(),
                LonghandId::BoxDecorationBreak =>
                    PropertyFlags::empty(),
                LonghandId::MozFloatEdge =>
                    PropertyFlags::empty(),
                LonghandId::BorderImageSource =>
                        APPLIES_TO_FIRST_LETTER |
                    PropertyFlags::empty(),
                LonghandId::BorderImageOutset =>
                        APPLIES_TO_FIRST_LETTER |
                    PropertyFlags::empty(),
                LonghandId::BorderImageRepeat =>
                        APPLIES_TO_FIRST_LETTER |
                    PropertyFlags::empty(),
                LonghandId::BorderImageWidth =>
                        APPLIES_TO_FIRST_LETTER |
                    PropertyFlags::empty(),
                LonghandId::BorderImageSlice =>
                        APPLIES_TO_FIRST_LETTER |
                    PropertyFlags::empty(),
                LonghandId::Display =>
                        APPLIES_TO_PLACEHOLDER |
                    PropertyFlags::empty(),
                LonghandId::MozTopLayer =>
                    PropertyFlags::empty(),
                LonghandId::Position =>
                        CREATES_STACKING_CONTEXT |
                        ABSPOS_CB |
                    PropertyFlags::empty(),
                LonghandId::Float =>
                        APPLIES_TO_FIRST_LETTER |
                    PropertyFlags::empty(),
                LonghandId::Clear =>
                    PropertyFlags::empty(),
                LonghandId::VerticalAlign =>
                        APPLIES_TO_FIRST_LETTER |
                        APPLIES_TO_FIRST_LINE |
                        APPLIES_TO_PLACEHOLDER |
                    PropertyFlags::empty(),
                LonghandId::OverflowClipBox =>
                        APPLIES_TO_PLACEHOLDER |
                    PropertyFlags::empty(),
                LonghandId::OverflowX =>
                        APPLIES_TO_PLACEHOLDER |
                    PropertyFlags::empty(),
                LonghandId::OverflowY =>
                        APPLIES_TO_PLACEHOLDER |
                    PropertyFlags::empty(),
                LonghandId::TransitionDuration =>
                    PropertyFlags::empty(),
                LonghandId::TransitionTimingFunction =>
                    PropertyFlags::empty(),
                LonghandId::TransitionProperty =>
                    PropertyFlags::empty(),
                LonghandId::TransitionDelay =>
                    PropertyFlags::empty(),
                LonghandId::AnimationName =>
                    PropertyFlags::empty(),
                LonghandId::AnimationDuration =>
                    PropertyFlags::empty(),
                LonghandId::AnimationTimingFunction =>
                    PropertyFlags::empty(),
                LonghandId::AnimationIterationCount =>
                    PropertyFlags::empty(),
                LonghandId::AnimationDirection =>
                    PropertyFlags::empty(),
                LonghandId::AnimationPlayState =>
                    PropertyFlags::empty(),
                LonghandId::AnimationFillMode =>
                    PropertyFlags::empty(),
                LonghandId::AnimationDelay =>
                    PropertyFlags::empty(),
                LonghandId::ScrollSnapPointsX =>
                    PropertyFlags::empty(),
                LonghandId::ScrollSnapPointsY =>
                    PropertyFlags::empty(),
                LonghandId::ScrollSnapDestination =>
                    PropertyFlags::empty(),
                LonghandId::ScrollSnapCoordinate =>
                    PropertyFlags::empty(),
                LonghandId::Transform =>
                        CREATES_STACKING_CONTEXT |
                        FIXPOS_CB |
                    PropertyFlags::empty(),
                LonghandId::ScrollBehavior =>
                    PropertyFlags::empty(),
                LonghandId::ScrollSnapTypeX =>
                    PropertyFlags::empty(),
                LonghandId::ScrollSnapTypeY =>
                    PropertyFlags::empty(),
                LonghandId::Isolation =>
                        CREATES_STACKING_CONTEXT |
                    PropertyFlags::empty(),
                LonghandId::PageBreakAfter =>
                    PropertyFlags::empty(),
                LonghandId::PageBreakBefore =>
                    PropertyFlags::empty(),
                LonghandId::PageBreakInside =>
                    PropertyFlags::empty(),
                LonghandId::Resize =>
                        APPLIES_TO_PLACEHOLDER |
                    PropertyFlags::empty(),
                LonghandId::Perspective =>
                        CREATES_STACKING_CONTEXT |
                        FIXPOS_CB |
                    PropertyFlags::empty(),
                LonghandId::PerspectiveOrigin =>
                    PropertyFlags::empty(),
                LonghandId::BackfaceVisibility =>
                    PropertyFlags::empty(),
                LonghandId::TransformBox =>
                    PropertyFlags::empty(),
                LonghandId::TransformStyle =>
                        CREATES_STACKING_CONTEXT |
                        FIXPOS_CB |
                    PropertyFlags::empty(),
                LonghandId::TransformOrigin =>
                    PropertyFlags::empty(),
                LonghandId::Contain =>
                        FIXPOS_CB |
                    PropertyFlags::empty(),
                LonghandId::MozAppearance =>
                    PropertyFlags::empty(),
                LonghandId::MozBinding =>
                    PropertyFlags::empty(),
                LonghandId::MozOrient =>
                    PropertyFlags::empty(),
                LonghandId::WillChange =>
                    PropertyFlags::empty(),
                LonghandId::ShapeOutside =>
                        APPLIES_TO_FIRST_LETTER |
                    PropertyFlags::empty(),
                LonghandId::TouchAction =>
                    PropertyFlags::empty(),
                LonghandId::Color =>
                        APPLIES_TO_FIRST_LETTER |
                        APPLIES_TO_FIRST_LINE |
                        APPLIES_TO_PLACEHOLDER |
                    PropertyFlags::empty(),
                LonghandId::ColumnWidth =>
                    PropertyFlags::empty(),
                LonghandId::ColumnCount =>
                    PropertyFlags::empty(),
                LonghandId::ColumnGap =>
                    PropertyFlags::empty(),
                LonghandId::ColumnFill =>
                    PropertyFlags::empty(),
                LonghandId::ColumnRuleWidth =>
                    PropertyFlags::empty(),
                LonghandId::ColumnRuleColor =>
                    PropertyFlags::empty(),
                LonghandId::ColumnSpan =>
                    PropertyFlags::empty(),
                LonghandId::ColumnRuleStyle =>
                    PropertyFlags::empty(),
                LonghandId::Content =>
                    PropertyFlags::empty(),
                LonghandId::CounterIncrement =>
                    PropertyFlags::empty(),
                LonghandId::CounterReset =>
                    PropertyFlags::empty(),
                LonghandId::Opacity =>
                        CREATES_STACKING_CONTEXT |
                        APPLIES_TO_PLACEHOLDER |
                    PropertyFlags::empty(),
                LonghandId::BoxShadow =>
                        APPLIES_TO_FIRST_LETTER |
                    PropertyFlags::empty(),
                LonghandId::Clip =>
                    PropertyFlags::empty(),
                LonghandId::Filter =>
                        CREATES_STACKING_CONTEXT |
                        FIXPOS_CB |
                    PropertyFlags::empty(),
                LonghandId::MixBlendMode =>
                        CREATES_STACKING_CONTEXT |
                    PropertyFlags::empty(),
                LonghandId::FontFamily =>
                        APPLIES_TO_FIRST_LETTER |
                        APPLIES_TO_FIRST_LINE |
                        APPLIES_TO_PLACEHOLDER |
                    PropertyFlags::empty(),
                LonghandId::FontStyle =>
                        APPLIES_TO_FIRST_LETTER |
                        APPLIES_TO_FIRST_LINE |
                        APPLIES_TO_PLACEHOLDER |
                    PropertyFlags::empty(),
                LonghandId::FontVariantCaps =>
                        APPLIES_TO_FIRST_LETTER |
                        APPLIES_TO_FIRST_LINE |
                        APPLIES_TO_PLACEHOLDER |
                    PropertyFlags::empty(),
                LonghandId::FontWeight =>
                        APPLIES_TO_FIRST_LETTER |
                        APPLIES_TO_FIRST_LINE |
                        APPLIES_TO_PLACEHOLDER |
                    PropertyFlags::empty(),
                LonghandId::FontSize =>
                        APPLIES_TO_FIRST_LETTER |
                        APPLIES_TO_FIRST_LINE |
                        APPLIES_TO_PLACEHOLDER |
                    PropertyFlags::empty(),
                LonghandId::FontSizeAdjust =>
                        APPLIES_TO_FIRST_LETTER |
                        APPLIES_TO_FIRST_LINE |
                        APPLIES_TO_PLACEHOLDER |
                    PropertyFlags::empty(),
                LonghandId::FontSynthesis =>
                        APPLIES_TO_FIRST_LETTER |
                        APPLIES_TO_FIRST_LINE |
                        APPLIES_TO_PLACEHOLDER |
                    PropertyFlags::empty(),
                LonghandId::FontStretch =>
                        APPLIES_TO_FIRST_LETTER |
                        APPLIES_TO_FIRST_LINE |
                        APPLIES_TO_PLACEHOLDER |
                    PropertyFlags::empty(),
                LonghandId::FontKerning =>
                        APPLIES_TO_FIRST_LETTER |
                        APPLIES_TO_FIRST_LINE |
                        APPLIES_TO_PLACEHOLDER |
                    PropertyFlags::empty(),
                LonghandId::FontVariantAlternates =>
                        APPLIES_TO_FIRST_LETTER |
                        APPLIES_TO_FIRST_LINE |
                        APPLIES_TO_PLACEHOLDER |
                    PropertyFlags::empty(),
                LonghandId::FontVariantEastAsian =>
                        APPLIES_TO_FIRST_LETTER |
                        APPLIES_TO_FIRST_LINE |
                        APPLIES_TO_PLACEHOLDER |
                    PropertyFlags::empty(),
                LonghandId::FontVariantLigatures =>
                        APPLIES_TO_FIRST_LETTER |
                        APPLIES_TO_FIRST_LINE |
                        APPLIES_TO_PLACEHOLDER |
                    PropertyFlags::empty(),
                LonghandId::FontVariantNumeric =>
                        APPLIES_TO_FIRST_LETTER |
                        APPLIES_TO_FIRST_LINE |
                        APPLIES_TO_PLACEHOLDER |
                    PropertyFlags::empty(),
                LonghandId::FontVariantPosition =>
                        APPLIES_TO_FIRST_LETTER |
                        APPLIES_TO_FIRST_LINE |
                        APPLIES_TO_PLACEHOLDER |
                    PropertyFlags::empty(),
                LonghandId::FontFeatureSettings =>
                        APPLIES_TO_FIRST_LETTER |
                        APPLIES_TO_FIRST_LINE |
                        APPLIES_TO_PLACEHOLDER |
                    PropertyFlags::empty(),
                LonghandId::FontVariationSettings =>
                        APPLIES_TO_FIRST_LETTER |
                        APPLIES_TO_FIRST_LINE |
                        APPLIES_TO_PLACEHOLDER |
                    PropertyFlags::empty(),
                LonghandId::FontLanguageOverride =>
                        APPLIES_TO_FIRST_LETTER |
                        APPLIES_TO_FIRST_LINE |
                        APPLIES_TO_PLACEHOLDER |
                    PropertyFlags::empty(),
                LonghandId::XLang =>
                    PropertyFlags::empty(),
                LonghandId::MozScriptSizeMultiplier =>
                    PropertyFlags::empty(),
                LonghandId::MozScriptLevel =>
                    PropertyFlags::empty(),
                LonghandId::MozMathDisplay =>
                    PropertyFlags::empty(),
                LonghandId::MozMathVariant =>
                    PropertyFlags::empty(),
                LonghandId::MozScriptMinSize =>
                    PropertyFlags::empty(),
                LonghandId::XTextZoom =>
                    PropertyFlags::empty(),
                LonghandId::MozOsxFontSmoothing =>
                        APPLIES_TO_FIRST_LETTER |
                        APPLIES_TO_FIRST_LINE |
                        APPLIES_TO_PLACEHOLDER |
                    PropertyFlags::empty(),
                LonghandId::MozMinFontSizeRatio =>
                    PropertyFlags::empty(),
                LonghandId::Visibility =>
                    PropertyFlags::empty(),
                LonghandId::WritingMode =>
                    PropertyFlags::empty(),
                LonghandId::Direction =>
                    PropertyFlags::empty(),
                LonghandId::TextOrientation =>
                    PropertyFlags::empty(),
                LonghandId::ColorAdjust =>
                    PropertyFlags::empty(),
                LonghandId::ImageRendering =>
                    PropertyFlags::empty(),
                LonghandId::ImageOrientation =>
                    PropertyFlags::empty(),
                LonghandId::BorderCollapse =>
                    PropertyFlags::empty(),
                LonghandId::EmptyCells =>
                    PropertyFlags::empty(),
                LonghandId::CaptionSide =>
                    PropertyFlags::empty(),
                LonghandId::BorderSpacing =>
                    PropertyFlags::empty(),
                LonghandId::LineHeight =>
                        APPLIES_TO_FIRST_LETTER |
                        APPLIES_TO_FIRST_LINE |
                        APPLIES_TO_PLACEHOLDER |
                    PropertyFlags::empty(),
                LonghandId::TextTransform =>
                        APPLIES_TO_FIRST_LETTER |
                        APPLIES_TO_FIRST_LINE |
                        APPLIES_TO_PLACEHOLDER |
                    PropertyFlags::empty(),
                LonghandId::Hyphens =>
                    PropertyFlags::empty(),
                LonghandId::MozTextSizeAdjust =>
                    PropertyFlags::empty(),
                LonghandId::TextIndent =>
                    PropertyFlags::empty(),
                LonghandId::OverflowWrap =>
                    PropertyFlags::empty(),
                LonghandId::WordBreak =>
                    PropertyFlags::empty(),
                LonghandId::TextJustify =>
                        APPLIES_TO_PLACEHOLDER |
                    PropertyFlags::empty(),
                LonghandId::TextAlignLast =>
                    PropertyFlags::empty(),
                LonghandId::TextAlign =>
                        APPLIES_TO_PLACEHOLDER |
                    PropertyFlags::empty(),
                LonghandId::LetterSpacing =>
                        APPLIES_TO_FIRST_LETTER |
                        APPLIES_TO_FIRST_LINE |
                        APPLIES_TO_PLACEHOLDER |
                    PropertyFlags::empty(),
                LonghandId::WordSpacing =>
                        APPLIES_TO_FIRST_LETTER |
                        APPLIES_TO_FIRST_LINE |
                        APPLIES_TO_PLACEHOLDER |
                    PropertyFlags::empty(),
                LonghandId::WhiteSpace =>
                        APPLIES_TO_PLACEHOLDER |
                    PropertyFlags::empty(),
                LonghandId::TextShadow =>
                        APPLIES_TO_FIRST_LETTER |
                        APPLIES_TO_FIRST_LINE |
                        APPLIES_TO_PLACEHOLDER |
                    PropertyFlags::empty(),
                LonghandId::TextEmphasisStyle =>
                    PropertyFlags::empty(),
                LonghandId::TextEmphasisPosition =>
                    PropertyFlags::empty(),
                LonghandId::TextEmphasisColor =>
                    PropertyFlags::empty(),
                LonghandId::MozTabSize =>
                    PropertyFlags::empty(),
                LonghandId::WebkitTextFillColor =>
                        APPLIES_TO_FIRST_LETTER |
                        APPLIES_TO_FIRST_LINE |
                        APPLIES_TO_PLACEHOLDER |
                    PropertyFlags::empty(),
                LonghandId::WebkitTextStrokeColor =>
                        APPLIES_TO_FIRST_LETTER |
                        APPLIES_TO_FIRST_LINE |
                        APPLIES_TO_PLACEHOLDER |
                    PropertyFlags::empty(),
                LonghandId::WebkitTextStrokeWidth =>
                        APPLIES_TO_FIRST_LETTER |
                        APPLIES_TO_FIRST_LINE |
                        APPLIES_TO_PLACEHOLDER |
                    PropertyFlags::empty(),
                LonghandId::RubyAlign =>
                    PropertyFlags::empty(),
                LonghandId::RubyPosition =>
                    PropertyFlags::empty(),
                LonghandId::TextCombineUpright =>
                    PropertyFlags::empty(),
                LonghandId::TextRendering =>
                    PropertyFlags::empty(),
                LonghandId::MozControlCharacterVisibility =>
                    PropertyFlags::empty(),
                LonghandId::ListStylePosition =>
                    PropertyFlags::empty(),
                LonghandId::ListStyleType =>
                    PropertyFlags::empty(),
                LonghandId::ListStyleImage =>
                    PropertyFlags::empty(),
                LonghandId::Quotes =>
                    PropertyFlags::empty(),
                LonghandId::MozImageRegion =>
                    PropertyFlags::empty(),
                LonghandId::MarginTop =>
                        APPLIES_TO_FIRST_LETTER |
                    PropertyFlags::empty(),
                LonghandId::MarginLeft =>
                        APPLIES_TO_FIRST_LETTER |
                    PropertyFlags::empty(),
                LonghandId::MarginBottom =>
                        APPLIES_TO_FIRST_LETTER |
                    PropertyFlags::empty(),
                LonghandId::MarginRight =>
                        APPLIES_TO_FIRST_LETTER |
                    PropertyFlags::empty(),
                LonghandId::MarginBlockStart =>
                        APPLIES_TO_FIRST_LETTER |
                    PropertyFlags::empty(),
                LonghandId::MarginBlockEnd =>
                        APPLIES_TO_FIRST_LETTER |
                    PropertyFlags::empty(),
                LonghandId::MarginInlineStart =>
                        APPLIES_TO_FIRST_LETTER |
                    PropertyFlags::empty(),
                LonghandId::MarginInlineEnd =>
                        APPLIES_TO_FIRST_LETTER |
                    PropertyFlags::empty(),
                LonghandId::OutlineColor =>
                    PropertyFlags::empty(),
                LonghandId::OutlineStyle =>
                    PropertyFlags::empty(),
                LonghandId::OutlineWidth =>
                    PropertyFlags::empty(),
                LonghandId::MozOutlineRadiusTopleft =>
                    PropertyFlags::empty(),
                LonghandId::MozOutlineRadiusTopright =>
                    PropertyFlags::empty(),
                LonghandId::MozOutlineRadiusBottomright =>
                    PropertyFlags::empty(),
                LonghandId::MozOutlineRadiusBottomleft =>
                    PropertyFlags::empty(),
                LonghandId::OutlineOffset =>
                    PropertyFlags::empty(),
                LonghandId::PaddingTop =>
                        APPLIES_TO_FIRST_LETTER |
                        APPLIES_TO_PLACEHOLDER |
                    PropertyFlags::empty(),
                LonghandId::PaddingLeft =>
                        APPLIES_TO_FIRST_LETTER |
                        APPLIES_TO_PLACEHOLDER |
                    PropertyFlags::empty(),
                LonghandId::PaddingBottom =>
                        APPLIES_TO_FIRST_LETTER |
                        APPLIES_TO_PLACEHOLDER |
                    PropertyFlags::empty(),
                LonghandId::PaddingRight =>
                        APPLIES_TO_FIRST_LETTER |
                        APPLIES_TO_PLACEHOLDER |
                    PropertyFlags::empty(),
                LonghandId::PaddingBlockStart =>
                        APPLIES_TO_FIRST_LETTER |
                        APPLIES_TO_PLACEHOLDER |
                    PropertyFlags::empty(),
                LonghandId::PaddingBlockEnd =>
                        APPLIES_TO_FIRST_LETTER |
                        APPLIES_TO_PLACEHOLDER |
                    PropertyFlags::empty(),
                LonghandId::PaddingInlineStart =>
                        APPLIES_TO_FIRST_LETTER |
                        APPLIES_TO_PLACEHOLDER |
                    PropertyFlags::empty(),
                LonghandId::PaddingInlineEnd =>
                        APPLIES_TO_FIRST_LETTER |
                        APPLIES_TO_PLACEHOLDER |
                    PropertyFlags::empty(),
                LonghandId::Cursor =>
                    PropertyFlags::empty(),
                LonghandId::PointerEvents =>
                        APPLIES_TO_PLACEHOLDER |
                    PropertyFlags::empty(),
                LonghandId::MozUserInput =>
                    PropertyFlags::empty(),
                LonghandId::MozUserModify =>
                    PropertyFlags::empty(),
                LonghandId::MozUserFocus =>
                    PropertyFlags::empty(),
                LonghandId::CaretColor =>
                    PropertyFlags::empty(),
                LonghandId::Top =>
                    PropertyFlags::empty(),
                LonghandId::Left =>
                    PropertyFlags::empty(),
                LonghandId::Bottom =>
                    PropertyFlags::empty(),
                LonghandId::Right =>
                    PropertyFlags::empty(),
                LonghandId::OffsetBlockStart =>
                    PropertyFlags::empty(),
                LonghandId::OffsetBlockEnd =>
                    PropertyFlags::empty(),
                LonghandId::OffsetInlineStart =>
                    PropertyFlags::empty(),
                LonghandId::OffsetInlineEnd =>
                    PropertyFlags::empty(),
                LonghandId::ZIndex =>
                        CREATES_STACKING_CONTEXT |
                    PropertyFlags::empty(),
                LonghandId::FlexDirection =>
                    PropertyFlags::empty(),
                LonghandId::FlexWrap =>
                    PropertyFlags::empty(),
                LonghandId::JustifyContent =>
                    PropertyFlags::empty(),
                LonghandId::AlignContent =>
                    PropertyFlags::empty(),
                LonghandId::AlignItems =>
                    PropertyFlags::empty(),
                LonghandId::JustifyItems =>
                    PropertyFlags::empty(),
                LonghandId::FlexGrow =>
                    PropertyFlags::empty(),
                LonghandId::FlexShrink =>
                    PropertyFlags::empty(),
                LonghandId::AlignSelf =>
                    PropertyFlags::empty(),
                LonghandId::JustifySelf =>
                    PropertyFlags::empty(),
                LonghandId::Order =>
                    PropertyFlags::empty(),
                LonghandId::FlexBasis =>
                    PropertyFlags::empty(),
                LonghandId::Width =>
                    PropertyFlags::empty(),
                LonghandId::MinWidth =>
                    PropertyFlags::empty(),
                LonghandId::MaxWidth =>
                    PropertyFlags::empty(),
                LonghandId::Height =>
                    PropertyFlags::empty(),
                LonghandId::MinHeight =>
                    PropertyFlags::empty(),
                LonghandId::MaxHeight =>
                    PropertyFlags::empty(),
                LonghandId::BlockSize =>
                    PropertyFlags::empty(),
                LonghandId::MinBlockSize =>
                    PropertyFlags::empty(),
                LonghandId::MaxBlockSize =>
                    PropertyFlags::empty(),
                LonghandId::InlineSize =>
                    PropertyFlags::empty(),
                LonghandId::MinInlineSize =>
                    PropertyFlags::empty(),
                LonghandId::MaxInlineSize =>
                    PropertyFlags::empty(),
                LonghandId::BoxSizing =>
                    PropertyFlags::empty(),
                LonghandId::ObjectFit =>
                    PropertyFlags::empty(),
                LonghandId::ObjectPosition =>
                    PropertyFlags::empty(),
                LonghandId::GridRowGap =>
                    PropertyFlags::empty(),
                LonghandId::GridRowStart =>
                    PropertyFlags::empty(),
                LonghandId::GridRowEnd =>
                    PropertyFlags::empty(),
                LonghandId::GridAutoRows =>
                    PropertyFlags::empty(),
                LonghandId::GridTemplateRows =>
                    PropertyFlags::empty(),
                LonghandId::GridColumnGap =>
                    PropertyFlags::empty(),
                LonghandId::GridColumnStart =>
                    PropertyFlags::empty(),
                LonghandId::GridColumnEnd =>
                    PropertyFlags::empty(),
                LonghandId::GridAutoColumns =>
                    PropertyFlags::empty(),
                LonghandId::GridTemplateColumns =>
                    PropertyFlags::empty(),
                LonghandId::GridAutoFlow =>
                    PropertyFlags::empty(),
                LonghandId::GridTemplateAreas =>
                    PropertyFlags::empty(),
                LonghandId::TableLayout =>
                    PropertyFlags::empty(),
                LonghandId::XSpan =>
                    PropertyFlags::empty(),
                LonghandId::TextOverflow =>
                        APPLIES_TO_PLACEHOLDER |
                    PropertyFlags::empty(),
                LonghandId::UnicodeBidi =>
                    PropertyFlags::empty(),
                LonghandId::TextDecorationLine =>
                        APPLIES_TO_FIRST_LETTER |
                        APPLIES_TO_FIRST_LINE |
                        APPLIES_TO_PLACEHOLDER |
                    PropertyFlags::empty(),
                LonghandId::TextDecorationStyle =>
                        APPLIES_TO_FIRST_LETTER |
                        APPLIES_TO_FIRST_LINE |
                        APPLIES_TO_PLACEHOLDER |
                    PropertyFlags::empty(),
                LonghandId::TextDecorationColor =>
                        APPLIES_TO_FIRST_LETTER |
                        APPLIES_TO_FIRST_LINE |
                        APPLIES_TO_PLACEHOLDER |
                    PropertyFlags::empty(),
                LonghandId::InitialLetter =>
                        APPLIES_TO_FIRST_LETTER |
                    PropertyFlags::empty(),
                LonghandId::ImeMode =>
                    PropertyFlags::empty(),
                LonghandId::MozUserSelect =>
                    PropertyFlags::empty(),
                LonghandId::MozWindowDragging =>
                    PropertyFlags::empty(),
                LonghandId::MozWindowShadow =>
                    PropertyFlags::empty(),
                LonghandId::MozForceBrokenImageIcon =>
                    PropertyFlags::empty(),
                LonghandId::TextAnchor =>
                    PropertyFlags::empty(),
                LonghandId::ColorInterpolation =>
                    PropertyFlags::empty(),
                LonghandId::ColorInterpolationFilters =>
                    PropertyFlags::empty(),
                LonghandId::Fill =>
                    PropertyFlags::empty(),
                LonghandId::FillOpacity =>
                    PropertyFlags::empty(),
                LonghandId::FillRule =>
                    PropertyFlags::empty(),
                LonghandId::ShapeRendering =>
                    PropertyFlags::empty(),
                LonghandId::Stroke =>
                    PropertyFlags::empty(),
                LonghandId::StrokeWidth =>
                    PropertyFlags::empty(),
                LonghandId::StrokeLinecap =>
                    PropertyFlags::empty(),
                LonghandId::StrokeLinejoin =>
                    PropertyFlags::empty(),
                LonghandId::StrokeMiterlimit =>
                    PropertyFlags::empty(),
                LonghandId::StrokeOpacity =>
                    PropertyFlags::empty(),
                LonghandId::StrokeDasharray =>
                    PropertyFlags::empty(),
                LonghandId::StrokeDashoffset =>
                    PropertyFlags::empty(),
                LonghandId::ClipRule =>
                    PropertyFlags::empty(),
                LonghandId::MarkerStart =>
                    PropertyFlags::empty(),
                LonghandId::MarkerMid =>
                    PropertyFlags::empty(),
                LonghandId::MarkerEnd =>
                    PropertyFlags::empty(),
                LonghandId::PaintOrder =>
                    PropertyFlags::empty(),
                LonghandId::MozContextProperties =>
                    PropertyFlags::empty(),
                LonghandId::DominantBaseline =>
                    PropertyFlags::empty(),
                LonghandId::VectorEffect =>
                    PropertyFlags::empty(),
                LonghandId::StopColor =>
                    PropertyFlags::empty(),
                LonghandId::StopOpacity =>
                    PropertyFlags::empty(),
                LonghandId::FloodColor =>
                    PropertyFlags::empty(),
                LonghandId::FloodOpacity =>
                    PropertyFlags::empty(),
                LonghandId::LightingColor =>
                    PropertyFlags::empty(),
                LonghandId::MaskType =>
                    PropertyFlags::empty(),
                LonghandId::ClipPath =>
                        CREATES_STACKING_CONTEXT |
                    PropertyFlags::empty(),
                LonghandId::MaskMode =>
                    PropertyFlags::empty(),
                LonghandId::MaskRepeat =>
                    PropertyFlags::empty(),
                LonghandId::MaskPositionX =>
                    PropertyFlags::empty(),
                LonghandId::MaskPositionY =>
                    PropertyFlags::empty(),
                LonghandId::MaskClip =>
                    PropertyFlags::empty(),
                LonghandId::MaskOrigin =>
                    PropertyFlags::empty(),
                LonghandId::MaskSize =>
                    PropertyFlags::empty(),
                LonghandId::MaskComposite =>
                    PropertyFlags::empty(),
                LonghandId::MaskImage =>
                        CREATES_STACKING_CONTEXT |
                    PropertyFlags::empty(),
                LonghandId::MozBoxAlign =>
                    PropertyFlags::empty(),
                LonghandId::MozBoxDirection =>
                    PropertyFlags::empty(),
                LonghandId::MozBoxFlex =>
                    PropertyFlags::empty(),
                LonghandId::MozBoxOrient =>
                    PropertyFlags::empty(),
                LonghandId::MozBoxPack =>
                    PropertyFlags::empty(),
                LonghandId::MozStackSizing =>
                    PropertyFlags::empty(),
                LonghandId::MozBoxOrdinalGroup =>
                    PropertyFlags::empty(),
        }
    }

    /// Only a few properties are allowed to depend on the visited state of
    /// links. When cascading visited styles, we can save time by only
    /// processing these properties.
    fn is_visited_dependent(&self) -> bool {
        matches!(*self,
            LonghandId::ColumnRuleColor |
            LonghandId::TextEmphasisColor |
            LonghandId::WebkitTextFillColor |
            LonghandId::WebkitTextStrokeColor |
            LonghandId::TextDecorationColor |
            LonghandId::Fill |
            LonghandId::Stroke |
            LonghandId::CaretColor |
            LonghandId::Color |
            LonghandId::BackgroundColor |
            LonghandId::BorderTopColor |
            LonghandId::BorderRightColor |
            LonghandId::BorderBottomColor |
            LonghandId::BorderLeftColor |
            LonghandId::OutlineColor
        )
    }

    /// Returns true if the property is one that is ignored when document
    /// colors are disabled.
    fn is_ignored_when_document_colors_disabled(&self) -> bool {
        matches!(*self,
            LonghandId::BackgroundColor | LonghandId::BackgroundImage | LonghandId::BorderTopColor | LonghandId::BorderLeftColor | LonghandId::BorderBottomColor | LonghandId::BorderRightColor | LonghandId::BorderBlockStartColor | LonghandId::BorderBlockEndColor | LonghandId::BorderInlineStartColor | LonghandId::BorderInlineEndColor | LonghandId::MozBorderTopColors | LonghandId::MozBorderLeftColors | LonghandId::MozBorderBottomColors | LonghandId::MozBorderRightColors | LonghandId::Color | LonghandId::ColumnRuleColor | LonghandId::BoxShadow | LonghandId::TextShadow | LonghandId::TextEmphasisColor | LonghandId::WebkitTextFillColor | LonghandId::WebkitTextStrokeColor | LonghandId::OutlineColor | LonghandId::CaretColor | LonghandId::TextDecorationColor
        )
    }

    /// The computed value of some properties depends on the (sometimes
    /// computed) value of *other* properties.
    ///
    /// So we classify properties into "early" and "other", such that the only
    /// dependencies can be from "other" to "early".
    ///
    /// Unfortunately, it’s not easy to check that this classification is
    /// correct.
    fn is_early_property(&self) -> bool {
        matches!(*self,
            LonghandId::TextOrientation |
            LonghandId::AnimationName |
            LonghandId::TransitionProperty |
            LonghandId::XLang |
            LonghandId::XTextZoom |
            LonghandId::MozScriptLevel |
            LonghandId::MozMinFontSizeRatio |
            LonghandId::FontSize |
            LonghandId::FontFamily |
            LonghandId::Color |
            LonghandId::TextDecorationLine |
            LonghandId::WritingMode |
            LonghandId::Direction
        )
    }

    /// Whether computed values of this property lossily convert any complex
    /// colors into RGBA colors.
    ///
    /// In Gecko, there are some properties still that compute currentcolor
    /// down to an RGBA color at computed value time, instead of as
    /// `StyleComplexColor`s. For these properties, we must return `false`,
    /// so that we correctly avoid caching style data in the rule tree.
    pub fn stores_complex_colors_lossily(&self) -> bool {
        matches!(*self,
            LonghandId::StopColor |
            LonghandId::FloodColor |
            LonghandId::LightingColor |
            LonghandId::BackgroundImage |
            LonghandId::BorderImageSource |
            LonghandId::BoxShadow |
            LonghandId::MaskImage |
            LonghandId::MozBorderBottomColors |
            LonghandId::MozBorderLeftColors |
            LonghandId::MozBorderRightColors |
            LonghandId::MozBorderTopColors |
            LonghandId::TextShadow
        )
    }
}

/// An identifier for a given shorthand property.
#[cfg_attr(feature = "servo", derive(HeapSizeOf))]
#[derive(Clone, Copy, Debug, Eq, PartialEq, ToCss)]
pub enum ShorthandId {
        /// background
        Background,
        /// background-position
        BackgroundPosition,
        /// border-color
        BorderColor,
        /// border-style
        BorderStyle,
        /// border-width
        BorderWidth,
        /// border-top
        BorderTop,
        /// border-left
        BorderLeft,
        /// border-bottom
        BorderBottom,
        /// border-right
        BorderRight,
        /// border-block-start
        BorderBlockStart,
        /// border-block-end
        BorderBlockEnd,
        /// border-inline-start
        BorderInlineStart,
        /// border-inline-end
        BorderInlineEnd,
        /// border
        Border,
        /// border-radius
        BorderRadius,
        /// border-image
        BorderImage,
        /// overflow
        Overflow,
        /// transition
        Transition,
        /// animation
        Animation,
        /// scroll-snap-type
        ScrollSnapType,
        /// -moz-transform
        MozTransform,
        /// columns
        Columns,
        /// column-rule
        ColumnRule,
        /// font
        Font,
        /// font-variant
        FontVariant,
        /// text-emphasis
        TextEmphasis,
        /// -webkit-text-stroke
        WebkitTextStroke,
        /// list-style
        ListStyle,
        /// margin
        Margin,
        /// mask
        Mask,
        /// mask-position
        MaskPosition,
        /// outline
        Outline,
        /// -moz-outline-radius
        MozOutlineRadius,
        /// padding
        Padding,
        /// flex-flow
        FlexFlow,
        /// flex
        Flex,
        /// grid-gap
        GridGap,
        /// grid-row
        GridRow,
        /// grid-column
        GridColumn,
        /// grid-area
        GridArea,
        /// grid-template
        GridTemplate,
        /// grid
        Grid,
        /// place-content
        PlaceContent,
        /// place-self
        PlaceSelf,
        /// place-items
        PlaceItems,
        /// marker
        Marker,
        /// text-decoration
        TextDecoration,
        /// all
        All,
}

impl ShorthandId {
    /// Get the name for this shorthand property.
    pub fn name(&self) -> &'static str {
        match *self {
                ShorthandId::Background => "background",
                ShorthandId::BackgroundPosition => "background-position",
                ShorthandId::BorderColor => "border-color",
                ShorthandId::BorderStyle => "border-style",
                ShorthandId::BorderWidth => "border-width",
                ShorthandId::BorderTop => "border-top",
                ShorthandId::BorderLeft => "border-left",
                ShorthandId::BorderBottom => "border-bottom",
                ShorthandId::BorderRight => "border-right",
                ShorthandId::BorderBlockStart => "border-block-start",
                ShorthandId::BorderBlockEnd => "border-block-end",
                ShorthandId::BorderInlineStart => "border-inline-start",
                ShorthandId::BorderInlineEnd => "border-inline-end",
                ShorthandId::Border => "border",
                ShorthandId::BorderRadius => "border-radius",
                ShorthandId::BorderImage => "border-image",
                ShorthandId::Overflow => "overflow",
                ShorthandId::Transition => "transition",
                ShorthandId::Animation => "animation",
                ShorthandId::ScrollSnapType => "scroll-snap-type",
                ShorthandId::MozTransform => "-moz-transform",
                ShorthandId::Columns => "columns",
                ShorthandId::ColumnRule => "column-rule",
                ShorthandId::Font => "font",
                ShorthandId::FontVariant => "font-variant",
                ShorthandId::TextEmphasis => "text-emphasis",
                ShorthandId::WebkitTextStroke => "-webkit-text-stroke",
                ShorthandId::ListStyle => "list-style",
                ShorthandId::Margin => "margin",
                ShorthandId::Mask => "mask",
                ShorthandId::MaskPosition => "mask-position",
                ShorthandId::Outline => "outline",
                ShorthandId::MozOutlineRadius => "-moz-outline-radius",
                ShorthandId::Padding => "padding",
                ShorthandId::FlexFlow => "flex-flow",
                ShorthandId::Flex => "flex",
                ShorthandId::GridGap => "grid-gap",
                ShorthandId::GridRow => "grid-row",
                ShorthandId::GridColumn => "grid-column",
                ShorthandId::GridArea => "grid-area",
                ShorthandId::GridTemplate => "grid-template",
                ShorthandId::Grid => "grid",
                ShorthandId::PlaceContent => "place-content",
                ShorthandId::PlaceSelf => "place-self",
                ShorthandId::PlaceItems => "place-items",
                ShorthandId::Marker => "marker",
                ShorthandId::TextDecoration => "text-decoration",
                ShorthandId::All => "all",
        }
    }

    /// Get the longhand ids that form this shorthand.
    pub fn longhands(&self) -> &'static [LonghandId] {
            static BACKGROUND: &'static [LonghandId] = &[
                    LonghandId::BackgroundColor,
                    LonghandId::BackgroundPositionX,
                    LonghandId::BackgroundPositionY,
                    LonghandId::BackgroundRepeat,
                    LonghandId::BackgroundAttachment,
                    LonghandId::BackgroundImage,
                    LonghandId::BackgroundSize,
                    LonghandId::BackgroundOrigin,
                    LonghandId::BackgroundClip,
            ];
            static BACKGROUND_POSITION: &'static [LonghandId] = &[
                    LonghandId::BackgroundPositionX,
                    LonghandId::BackgroundPositionY,
            ];
            static BORDER_COLOR: &'static [LonghandId] = &[
                    LonghandId::BorderTopColor,
                    LonghandId::BorderRightColor,
                    LonghandId::BorderBottomColor,
                    LonghandId::BorderLeftColor,
            ];
            static BORDER_STYLE: &'static [LonghandId] = &[
                    LonghandId::BorderTopStyle,
                    LonghandId::BorderRightStyle,
                    LonghandId::BorderBottomStyle,
                    LonghandId::BorderLeftStyle,
            ];
            static BORDER_WIDTH: &'static [LonghandId] = &[
                    LonghandId::BorderTopWidth,
                    LonghandId::BorderLeftWidth,
                    LonghandId::BorderBottomWidth,
                    LonghandId::BorderRightWidth,
            ];
            static BORDER_TOP: &'static [LonghandId] = &[
                    LonghandId::BorderTopColor,
                    LonghandId::BorderTopStyle,
                    LonghandId::BorderTopWidth,
            ];
            static BORDER_LEFT: &'static [LonghandId] = &[
                    LonghandId::BorderLeftColor,
                    LonghandId::BorderLeftStyle,
                    LonghandId::BorderLeftWidth,
            ];
            static BORDER_BOTTOM: &'static [LonghandId] = &[
                    LonghandId::BorderBottomColor,
                    LonghandId::BorderBottomStyle,
                    LonghandId::BorderBottomWidth,
            ];
            static BORDER_RIGHT: &'static [LonghandId] = &[
                    LonghandId::BorderRightColor,
                    LonghandId::BorderRightStyle,
                    LonghandId::BorderRightWidth,
            ];
            static BORDER_BLOCK_START: &'static [LonghandId] = &[
                    LonghandId::BorderBlockStartColor,
                    LonghandId::BorderBlockStartStyle,
                    LonghandId::BorderBlockStartWidth,
            ];
            static BORDER_BLOCK_END: &'static [LonghandId] = &[
                    LonghandId::BorderBlockEndColor,
                    LonghandId::BorderBlockEndStyle,
                    LonghandId::BorderBlockEndWidth,
            ];
            static BORDER_INLINE_START: &'static [LonghandId] = &[
                    LonghandId::BorderInlineStartColor,
                    LonghandId::BorderInlineStartStyle,
                    LonghandId::BorderInlineStartWidth,
            ];
            static BORDER_INLINE_END: &'static [LonghandId] = &[
                    LonghandId::BorderInlineEndColor,
                    LonghandId::BorderInlineEndStyle,
                    LonghandId::BorderInlineEndWidth,
            ];
            static BORDER: &'static [LonghandId] = &[
                    LonghandId::BorderTopColor,
                    LonghandId::BorderTopStyle,
                    LonghandId::BorderTopWidth,
                    LonghandId::BorderLeftColor,
                    LonghandId::BorderLeftStyle,
                    LonghandId::BorderLeftWidth,
                    LonghandId::BorderBottomColor,
                    LonghandId::BorderBottomStyle,
                    LonghandId::BorderBottomWidth,
                    LonghandId::BorderRightColor,
                    LonghandId::BorderRightStyle,
                    LonghandId::BorderRightWidth,
                    LonghandId::BorderImageOutset,
                    LonghandId::BorderImageRepeat,
                    LonghandId::BorderImageSlice,
                    LonghandId::BorderImageSource,
                    LonghandId::BorderImageWidth,
                    LonghandId::MozBorderTopColors,
                    LonghandId::MozBorderLeftColors,
                    LonghandId::MozBorderBottomColors,
                    LonghandId::MozBorderRightColors,
            ];
            static BORDER_RADIUS: &'static [LonghandId] = &[
                    LonghandId::BorderTopLeftRadius,
                    LonghandId::BorderTopRightRadius,
                    LonghandId::BorderBottomRightRadius,
                    LonghandId::BorderBottomLeftRadius,
            ];
            static BORDER_IMAGE: &'static [LonghandId] = &[
                    LonghandId::BorderImageOutset,
                    LonghandId::BorderImageRepeat,
                    LonghandId::BorderImageSlice,
                    LonghandId::BorderImageSource,
                    LonghandId::BorderImageWidth,
            ];
            static OVERFLOW: &'static [LonghandId] = &[
                    LonghandId::OverflowX,
                    LonghandId::OverflowY,
            ];
            static TRANSITION: &'static [LonghandId] = &[
                    LonghandId::TransitionProperty,
                    LonghandId::TransitionDuration,
                    LonghandId::TransitionTimingFunction,
                    LonghandId::TransitionDelay,
            ];
            static ANIMATION: &'static [LonghandId] = &[
                    LonghandId::AnimationName,
                    LonghandId::AnimationDuration,
                    LonghandId::AnimationTimingFunction,
                    LonghandId::AnimationDelay,
                    LonghandId::AnimationIterationCount,
                    LonghandId::AnimationDirection,
                    LonghandId::AnimationFillMode,
                    LonghandId::AnimationPlayState,
            ];
            static SCROLL_SNAP_TYPE: &'static [LonghandId] = &[
                    LonghandId::ScrollSnapTypeX,
                    LonghandId::ScrollSnapTypeY,
            ];
            static _MOZ_TRANSFORM: &'static [LonghandId] = &[
                    LonghandId::Transform,
            ];
            static COLUMNS: &'static [LonghandId] = &[
                    LonghandId::ColumnWidth,
                    LonghandId::ColumnCount,
            ];
            static COLUMN_RULE: &'static [LonghandId] = &[
                    LonghandId::ColumnRuleWidth,
                    LonghandId::ColumnRuleStyle,
                    LonghandId::ColumnRuleColor,
            ];
            static FONT: &'static [LonghandId] = &[
                    LonghandId::FontStyle,
                    LonghandId::FontVariantCaps,
                    LonghandId::FontWeight,
                    LonghandId::FontStretch,
                    LonghandId::FontSize,
                    LonghandId::LineHeight,
                    LonghandId::FontFamily,
                    LonghandId::FontSizeAdjust,
                    LonghandId::FontKerning,
                    LonghandId::FontVariantAlternates,
                    LonghandId::FontVariantEastAsian,
                    LonghandId::FontVariantLigatures,
                    LonghandId::FontVariantNumeric,
                    LonghandId::FontVariantPosition,
                    LonghandId::FontLanguageOverride,
                    LonghandId::FontFeatureSettings,
            ];
            static FONT_VARIANT: &'static [LonghandId] = &[
                    LonghandId::FontVariantCaps,
                    LonghandId::FontVariantAlternates,
                    LonghandId::FontVariantEastAsian,
                    LonghandId::FontVariantLigatures,
                    LonghandId::FontVariantNumeric,
                    LonghandId::FontVariantPosition,
            ];
            static TEXT_EMPHASIS: &'static [LonghandId] = &[
                    LonghandId::TextEmphasisStyle,
                    LonghandId::TextEmphasisColor,
            ];
            static _WEBKIT_TEXT_STROKE: &'static [LonghandId] = &[
                    LonghandId::WebkitTextStrokeWidth,
                    LonghandId::WebkitTextStrokeColor,
            ];
            static LIST_STYLE: &'static [LonghandId] = &[
                    LonghandId::ListStylePosition,
                    LonghandId::ListStyleImage,
                    LonghandId::ListStyleType,
            ];
            static MARGIN: &'static [LonghandId] = &[
                    LonghandId::MarginTop,
                    LonghandId::MarginRight,
                    LonghandId::MarginBottom,
                    LonghandId::MarginLeft,
            ];
            static MASK: &'static [LonghandId] = &[
                    LonghandId::MaskMode,
                    LonghandId::MaskRepeat,
                    LonghandId::MaskClip,
                    LonghandId::MaskOrigin,
                    LonghandId::MaskComposite,
                    LonghandId::MaskPositionX,
                    LonghandId::MaskPositionY,
                    LonghandId::MaskSize,
                    LonghandId::MaskImage,
            ];
            static MASK_POSITION: &'static [LonghandId] = &[
                    LonghandId::MaskPositionX,
                    LonghandId::MaskPositionY,
            ];
            static OUTLINE: &'static [LonghandId] = &[
                    LonghandId::OutlineWidth,
                    LonghandId::OutlineStyle,
                    LonghandId::OutlineColor,
            ];
            static _MOZ_OUTLINE_RADIUS: &'static [LonghandId] = &[
                    LonghandId::MozOutlineRadiusTopleft,
                    LonghandId::MozOutlineRadiusTopright,
                    LonghandId::MozOutlineRadiusBottomright,
                    LonghandId::MozOutlineRadiusBottomleft,
            ];
            static PADDING: &'static [LonghandId] = &[
                    LonghandId::PaddingTop,
                    LonghandId::PaddingRight,
                    LonghandId::PaddingBottom,
                    LonghandId::PaddingLeft,
            ];
            static FLEX_FLOW: &'static [LonghandId] = &[
                    LonghandId::FlexDirection,
                    LonghandId::FlexWrap,
            ];
            static FLEX: &'static [LonghandId] = &[
                    LonghandId::FlexGrow,
                    LonghandId::FlexShrink,
                    LonghandId::FlexBasis,
            ];
            static GRID_GAP: &'static [LonghandId] = &[
                    LonghandId::GridRowGap,
                    LonghandId::GridColumnGap,
            ];
            static GRID_ROW: &'static [LonghandId] = &[
                    LonghandId::GridRowStart,
                    LonghandId::GridRowEnd,
            ];
            static GRID_COLUMN: &'static [LonghandId] = &[
                    LonghandId::GridColumnStart,
                    LonghandId::GridColumnEnd,
            ];
            static GRID_AREA: &'static [LonghandId] = &[
                    LonghandId::GridRowStart,
                    LonghandId::GridRowEnd,
                    LonghandId::GridColumnStart,
                    LonghandId::GridColumnEnd,
            ];
            static GRID_TEMPLATE: &'static [LonghandId] = &[
                    LonghandId::GridTemplateRows,
                    LonghandId::GridTemplateColumns,
                    LonghandId::GridTemplateAreas,
            ];
            static GRID: &'static [LonghandId] = &[
                    LonghandId::GridTemplateRows,
                    LonghandId::GridTemplateColumns,
                    LonghandId::GridTemplateAreas,
                    LonghandId::GridAutoRows,
                    LonghandId::GridAutoColumns,
                    LonghandId::GridAutoFlow,
            ];
            static PLACE_CONTENT: &'static [LonghandId] = &[
                    LonghandId::AlignContent,
                    LonghandId::JustifyContent,
            ];
            static PLACE_SELF: &'static [LonghandId] = &[
                    LonghandId::AlignSelf,
                    LonghandId::JustifySelf,
            ];
            static PLACE_ITEMS: &'static [LonghandId] = &[
                    LonghandId::AlignItems,
                    LonghandId::JustifyItems,
            ];
            static MARKER: &'static [LonghandId] = &[
                    LonghandId::MarkerStart,
                    LonghandId::MarkerEnd,
                    LonghandId::MarkerMid,
            ];
            static TEXT_DECORATION: &'static [LonghandId] = &[
                    LonghandId::TextDecorationLine,
                    LonghandId::TextDecorationStyle,
                    LonghandId::TextDecorationColor,
            ];
            static ALL: &'static [LonghandId] = &[
                    LonghandId::BackgroundColor,
                    LonghandId::BackgroundImage,
                    LonghandId::BackgroundPositionX,
                    LonghandId::BackgroundPositionY,
                    LonghandId::BackgroundRepeat,
                    LonghandId::BackgroundAttachment,
                    LonghandId::BackgroundClip,
                    LonghandId::BackgroundOrigin,
                    LonghandId::BackgroundSize,
                    LonghandId::BackgroundBlendMode,
                    LonghandId::BorderTopColor,
                    LonghandId::BorderTopStyle,
                    LonghandId::BorderTopWidth,
                    LonghandId::BorderLeftColor,
                    LonghandId::BorderLeftStyle,
                    LonghandId::BorderLeftWidth,
                    LonghandId::BorderBottomColor,
                    LonghandId::BorderBottomStyle,
                    LonghandId::BorderBottomWidth,
                    LonghandId::BorderRightColor,
                    LonghandId::BorderRightStyle,
                    LonghandId::BorderRightWidth,
                    LonghandId::BorderBlockStartColor,
                    LonghandId::BorderBlockStartStyle,
                    LonghandId::BorderBlockStartWidth,
                    LonghandId::BorderBlockEndColor,
                    LonghandId::BorderBlockEndStyle,
                    LonghandId::BorderBlockEndWidth,
                    LonghandId::BorderInlineStartColor,
                    LonghandId::BorderInlineStartStyle,
                    LonghandId::BorderInlineStartWidth,
                    LonghandId::BorderInlineEndColor,
                    LonghandId::BorderInlineEndStyle,
                    LonghandId::BorderInlineEndWidth,
                    LonghandId::BorderTopLeftRadius,
                    LonghandId::BorderTopRightRadius,
                    LonghandId::BorderBottomRightRadius,
                    LonghandId::BorderBottomLeftRadius,
                    LonghandId::MozBorderTopColors,
                    LonghandId::MozBorderLeftColors,
                    LonghandId::MozBorderBottomColors,
                    LonghandId::MozBorderRightColors,
                    LonghandId::BoxDecorationBreak,
                    LonghandId::MozFloatEdge,
                    LonghandId::BorderImageSource,
                    LonghandId::BorderImageOutset,
                    LonghandId::BorderImageRepeat,
                    LonghandId::BorderImageWidth,
                    LonghandId::BorderImageSlice,
                    LonghandId::Display,
                    LonghandId::Position,
                    LonghandId::Float,
                    LonghandId::Clear,
                    LonghandId::VerticalAlign,
                    LonghandId::OverflowX,
                    LonghandId::OverflowY,
                    LonghandId::TransitionDuration,
                    LonghandId::TransitionTimingFunction,
                    LonghandId::TransitionProperty,
                    LonghandId::TransitionDelay,
                    LonghandId::AnimationName,
                    LonghandId::AnimationDuration,
                    LonghandId::AnimationTimingFunction,
                    LonghandId::AnimationIterationCount,
                    LonghandId::AnimationDirection,
                    LonghandId::AnimationPlayState,
                    LonghandId::AnimationFillMode,
                    LonghandId::AnimationDelay,
                    LonghandId::ScrollSnapPointsX,
                    LonghandId::ScrollSnapPointsY,
                    LonghandId::ScrollSnapDestination,
                    LonghandId::ScrollSnapCoordinate,
                    LonghandId::Transform,
                    LonghandId::ScrollBehavior,
                    LonghandId::ScrollSnapTypeX,
                    LonghandId::ScrollSnapTypeY,
                    LonghandId::Isolation,
                    LonghandId::PageBreakAfter,
                    LonghandId::PageBreakBefore,
                    LonghandId::PageBreakInside,
                    LonghandId::Resize,
                    LonghandId::Perspective,
                    LonghandId::PerspectiveOrigin,
                    LonghandId::BackfaceVisibility,
                    LonghandId::TransformBox,
                    LonghandId::TransformStyle,
                    LonghandId::TransformOrigin,
                    LonghandId::Contain,
                    LonghandId::MozAppearance,
                    LonghandId::MozBinding,
                    LonghandId::MozOrient,
                    LonghandId::WillChange,
                    LonghandId::ShapeOutside,
                    LonghandId::TouchAction,
                    LonghandId::Color,
                    LonghandId::ColumnWidth,
                    LonghandId::ColumnCount,
                    LonghandId::ColumnGap,
                    LonghandId::ColumnFill,
                    LonghandId::ColumnRuleWidth,
                    LonghandId::ColumnRuleColor,
                    LonghandId::ColumnSpan,
                    LonghandId::ColumnRuleStyle,
                    LonghandId::Content,
                    LonghandId::CounterIncrement,
                    LonghandId::CounterReset,
                    LonghandId::Opacity,
                    LonghandId::BoxShadow,
                    LonghandId::Clip,
                    LonghandId::Filter,
                    LonghandId::MixBlendMode,
                    LonghandId::FontFamily,
                    LonghandId::FontStyle,
                    LonghandId::FontVariantCaps,
                    LonghandId::FontWeight,
                    LonghandId::FontSize,
                    LonghandId::FontSizeAdjust,
                    LonghandId::FontSynthesis,
                    LonghandId::FontStretch,
                    LonghandId::FontKerning,
                    LonghandId::FontVariantAlternates,
                    LonghandId::FontVariantEastAsian,
                    LonghandId::FontVariantLigatures,
                    LonghandId::FontVariantNumeric,
                    LonghandId::FontVariantPosition,
                    LonghandId::FontFeatureSettings,
                    LonghandId::FontVariationSettings,
                    LonghandId::FontLanguageOverride,
                    LonghandId::MozMathDisplay,
                    LonghandId::MozMathVariant,
                    LonghandId::MozOsxFontSmoothing,
                    LonghandId::Visibility,
                    LonghandId::WritingMode,
                    LonghandId::TextOrientation,
                    LonghandId::ColorAdjust,
                    LonghandId::ImageRendering,
                    LonghandId::ImageOrientation,
                    LonghandId::BorderCollapse,
                    LonghandId::EmptyCells,
                    LonghandId::CaptionSide,
                    LonghandId::BorderSpacing,
                    LonghandId::LineHeight,
                    LonghandId::TextTransform,
                    LonghandId::Hyphens,
                    LonghandId::MozTextSizeAdjust,
                    LonghandId::TextIndent,
                    LonghandId::OverflowWrap,
                    LonghandId::WordBreak,
                    LonghandId::TextJustify,
                    LonghandId::TextAlignLast,
                    LonghandId::TextAlign,
                    LonghandId::LetterSpacing,
                    LonghandId::WordSpacing,
                    LonghandId::WhiteSpace,
                    LonghandId::TextShadow,
                    LonghandId::TextEmphasisStyle,
                    LonghandId::TextEmphasisPosition,
                    LonghandId::TextEmphasisColor,
                    LonghandId::MozTabSize,
                    LonghandId::WebkitTextFillColor,
                    LonghandId::WebkitTextStrokeColor,
                    LonghandId::WebkitTextStrokeWidth,
                    LonghandId::RubyAlign,
                    LonghandId::RubyPosition,
                    LonghandId::TextCombineUpright,
                    LonghandId::TextRendering,
                    LonghandId::MozControlCharacterVisibility,
                    LonghandId::ListStylePosition,
                    LonghandId::ListStyleType,
                    LonghandId::ListStyleImage,
                    LonghandId::Quotes,
                    LonghandId::MozImageRegion,
                    LonghandId::MarginTop,
                    LonghandId::MarginLeft,
                    LonghandId::MarginBottom,
                    LonghandId::MarginRight,
                    LonghandId::MarginBlockStart,
                    LonghandId::MarginBlockEnd,
                    LonghandId::MarginInlineStart,
                    LonghandId::MarginInlineEnd,
                    LonghandId::OutlineColor,
                    LonghandId::OutlineStyle,
                    LonghandId::OutlineWidth,
                    LonghandId::MozOutlineRadiusTopleft,
                    LonghandId::MozOutlineRadiusTopright,
                    LonghandId::MozOutlineRadiusBottomright,
                    LonghandId::MozOutlineRadiusBottomleft,
                    LonghandId::OutlineOffset,
                    LonghandId::PaddingTop,
                    LonghandId::PaddingLeft,
                    LonghandId::PaddingBottom,
                    LonghandId::PaddingRight,
                    LonghandId::PaddingBlockStart,
                    LonghandId::PaddingBlockEnd,
                    LonghandId::PaddingInlineStart,
                    LonghandId::PaddingInlineEnd,
                    LonghandId::Cursor,
                    LonghandId::PointerEvents,
                    LonghandId::MozUserInput,
                    LonghandId::MozUserModify,
                    LonghandId::MozUserFocus,
                    LonghandId::CaretColor,
                    LonghandId::Top,
                    LonghandId::Left,
                    LonghandId::Bottom,
                    LonghandId::Right,
                    LonghandId::OffsetBlockStart,
                    LonghandId::OffsetBlockEnd,
                    LonghandId::OffsetInlineStart,
                    LonghandId::OffsetInlineEnd,
                    LonghandId::ZIndex,
                    LonghandId::FlexDirection,
                    LonghandId::FlexWrap,
                    LonghandId::JustifyContent,
                    LonghandId::AlignContent,
                    LonghandId::AlignItems,
                    LonghandId::JustifyItems,
                    LonghandId::FlexGrow,
                    LonghandId::FlexShrink,
                    LonghandId::AlignSelf,
                    LonghandId::JustifySelf,
                    LonghandId::Order,
                    LonghandId::FlexBasis,
                    LonghandId::Width,
                    LonghandId::MinWidth,
                    LonghandId::MaxWidth,
                    LonghandId::Height,
                    LonghandId::MinHeight,
                    LonghandId::MaxHeight,
                    LonghandId::BlockSize,
                    LonghandId::MinBlockSize,
                    LonghandId::MaxBlockSize,
                    LonghandId::InlineSize,
                    LonghandId::MinInlineSize,
                    LonghandId::MaxInlineSize,
                    LonghandId::BoxSizing,
                    LonghandId::ObjectFit,
                    LonghandId::ObjectPosition,
                    LonghandId::GridRowGap,
                    LonghandId::GridRowStart,
                    LonghandId::GridRowEnd,
                    LonghandId::GridAutoRows,
                    LonghandId::GridTemplateRows,
                    LonghandId::GridColumnGap,
                    LonghandId::GridColumnStart,
                    LonghandId::GridColumnEnd,
                    LonghandId::GridAutoColumns,
                    LonghandId::GridTemplateColumns,
                    LonghandId::GridAutoFlow,
                    LonghandId::GridTemplateAreas,
                    LonghandId::TableLayout,
                    LonghandId::TextOverflow,
                    LonghandId::TextDecorationLine,
                    LonghandId::TextDecorationStyle,
                    LonghandId::TextDecorationColor,
                    LonghandId::InitialLetter,
                    LonghandId::ImeMode,
                    LonghandId::MozUserSelect,
                    LonghandId::MozWindowDragging,
                    LonghandId::MozForceBrokenImageIcon,
                    LonghandId::TextAnchor,
                    LonghandId::ColorInterpolation,
                    LonghandId::ColorInterpolationFilters,
                    LonghandId::Fill,
                    LonghandId::FillOpacity,
                    LonghandId::FillRule,
                    LonghandId::ShapeRendering,
                    LonghandId::Stroke,
                    LonghandId::StrokeWidth,
                    LonghandId::StrokeLinecap,
                    LonghandId::StrokeLinejoin,
                    LonghandId::StrokeMiterlimit,
                    LonghandId::StrokeOpacity,
                    LonghandId::StrokeDasharray,
                    LonghandId::StrokeDashoffset,
                    LonghandId::ClipRule,
                    LonghandId::MarkerStart,
                    LonghandId::MarkerMid,
                    LonghandId::MarkerEnd,
                    LonghandId::PaintOrder,
                    LonghandId::MozContextProperties,
                    LonghandId::DominantBaseline,
                    LonghandId::VectorEffect,
                    LonghandId::StopColor,
                    LonghandId::StopOpacity,
                    LonghandId::FloodColor,
                    LonghandId::FloodOpacity,
                    LonghandId::LightingColor,
                    LonghandId::MaskType,
                    LonghandId::ClipPath,
                    LonghandId::MaskMode,
                    LonghandId::MaskRepeat,
                    LonghandId::MaskPositionX,
                    LonghandId::MaskPositionY,
                    LonghandId::MaskClip,
                    LonghandId::MaskOrigin,
                    LonghandId::MaskSize,
                    LonghandId::MaskComposite,
                    LonghandId::MaskImage,
                    LonghandId::MozBoxAlign,
                    LonghandId::MozBoxDirection,
                    LonghandId::MozBoxFlex,
                    LonghandId::MozBoxOrient,
                    LonghandId::MozBoxPack,
                    LonghandId::MozStackSizing,
                    LonghandId::MozBoxOrdinalGroup,
            ];
        match *self {
                ShorthandId::Background => BACKGROUND,
                ShorthandId::BackgroundPosition => BACKGROUND_POSITION,
                ShorthandId::BorderColor => BORDER_COLOR,
                ShorthandId::BorderStyle => BORDER_STYLE,
                ShorthandId::BorderWidth => BORDER_WIDTH,
                ShorthandId::BorderTop => BORDER_TOP,
                ShorthandId::BorderLeft => BORDER_LEFT,
                ShorthandId::BorderBottom => BORDER_BOTTOM,
                ShorthandId::BorderRight => BORDER_RIGHT,
                ShorthandId::BorderBlockStart => BORDER_BLOCK_START,
                ShorthandId::BorderBlockEnd => BORDER_BLOCK_END,
                ShorthandId::BorderInlineStart => BORDER_INLINE_START,
                ShorthandId::BorderInlineEnd => BORDER_INLINE_END,
                ShorthandId::Border => BORDER,
                ShorthandId::BorderRadius => BORDER_RADIUS,
                ShorthandId::BorderImage => BORDER_IMAGE,
                ShorthandId::Overflow => OVERFLOW,
                ShorthandId::Transition => TRANSITION,
                ShorthandId::Animation => ANIMATION,
                ShorthandId::ScrollSnapType => SCROLL_SNAP_TYPE,
                ShorthandId::MozTransform => _MOZ_TRANSFORM,
                ShorthandId::Columns => COLUMNS,
                ShorthandId::ColumnRule => COLUMN_RULE,
                ShorthandId::Font => FONT,
                ShorthandId::FontVariant => FONT_VARIANT,
                ShorthandId::TextEmphasis => TEXT_EMPHASIS,
                ShorthandId::WebkitTextStroke => _WEBKIT_TEXT_STROKE,
                ShorthandId::ListStyle => LIST_STYLE,
                ShorthandId::Margin => MARGIN,
                ShorthandId::Mask => MASK,
                ShorthandId::MaskPosition => MASK_POSITION,
                ShorthandId::Outline => OUTLINE,
                ShorthandId::MozOutlineRadius => _MOZ_OUTLINE_RADIUS,
                ShorthandId::Padding => PADDING,
                ShorthandId::FlexFlow => FLEX_FLOW,
                ShorthandId::Flex => FLEX,
                ShorthandId::GridGap => GRID_GAP,
                ShorthandId::GridRow => GRID_ROW,
                ShorthandId::GridColumn => GRID_COLUMN,
                ShorthandId::GridArea => GRID_AREA,
                ShorthandId::GridTemplate => GRID_TEMPLATE,
                ShorthandId::Grid => GRID,
                ShorthandId::PlaceContent => PLACE_CONTENT,
                ShorthandId::PlaceSelf => PLACE_SELF,
                ShorthandId::PlaceItems => PLACE_ITEMS,
                ShorthandId::Marker => MARKER,
                ShorthandId::TextDecoration => TEXT_DECORATION,
                ShorthandId::All => ALL,
        }
    }

    /// Try to serialize the given declarations as this shorthand.
    ///
    /// Returns an error if writing to the stream fails, or if the declarations
    /// do not map to a shorthand.
    pub fn longhands_to_css<'a, W, I>(&self, declarations: I, dest: &mut W) -> fmt::Result
        where W: fmt::Write,
              I: Iterator<Item=&'a PropertyDeclaration>,
    {
        match *self {
            ShorthandId::All => {
                // No need to try to serialize the declarations as the 'all'
                // shorthand, since it only accepts CSS-wide keywords (and
                // variable references), which will be handled in
                // get_shorthand_appendable_value.
                Err(fmt::Error)
            }
                ShorthandId::Background => {
                    match shorthands::background::LonghandsToSerialize::from_iter(declarations) {
                        Ok(longhands) => longhands.to_css(dest),
                        Err(_) => Err(fmt::Error)
                    }
                },
                ShorthandId::BackgroundPosition => {
                    match shorthands::background_position::LonghandsToSerialize::from_iter(declarations) {
                        Ok(longhands) => longhands.to_css(dest),
                        Err(_) => Err(fmt::Error)
                    }
                },
                ShorthandId::BorderColor => {
                    match shorthands::border_color::LonghandsToSerialize::from_iter(declarations) {
                        Ok(longhands) => longhands.to_css(dest),
                        Err(_) => Err(fmt::Error)
                    }
                },
                ShorthandId::BorderStyle => {
                    match shorthands::border_style::LonghandsToSerialize::from_iter(declarations) {
                        Ok(longhands) => longhands.to_css(dest),
                        Err(_) => Err(fmt::Error)
                    }
                },
                ShorthandId::BorderWidth => {
                    match shorthands::border_width::LonghandsToSerialize::from_iter(declarations) {
                        Ok(longhands) => longhands.to_css(dest),
                        Err(_) => Err(fmt::Error)
                    }
                },
                ShorthandId::BorderTop => {
                    match shorthands::border_top::LonghandsToSerialize::from_iter(declarations) {
                        Ok(longhands) => longhands.to_css(dest),
                        Err(_) => Err(fmt::Error)
                    }
                },
                ShorthandId::BorderLeft => {
                    match shorthands::border_left::LonghandsToSerialize::from_iter(declarations) {
                        Ok(longhands) => longhands.to_css(dest),
                        Err(_) => Err(fmt::Error)
                    }
                },
                ShorthandId::BorderBottom => {
                    match shorthands::border_bottom::LonghandsToSerialize::from_iter(declarations) {
                        Ok(longhands) => longhands.to_css(dest),
                        Err(_) => Err(fmt::Error)
                    }
                },
                ShorthandId::BorderRight => {
                    match shorthands::border_right::LonghandsToSerialize::from_iter(declarations) {
                        Ok(longhands) => longhands.to_css(dest),
                        Err(_) => Err(fmt::Error)
                    }
                },
                ShorthandId::BorderBlockStart => {
                    match shorthands::border_block_start::LonghandsToSerialize::from_iter(declarations) {
                        Ok(longhands) => longhands.to_css(dest),
                        Err(_) => Err(fmt::Error)
                    }
                },
                ShorthandId::BorderBlockEnd => {
                    match shorthands::border_block_end::LonghandsToSerialize::from_iter(declarations) {
                        Ok(longhands) => longhands.to_css(dest),
                        Err(_) => Err(fmt::Error)
                    }
                },
                ShorthandId::BorderInlineStart => {
                    match shorthands::border_inline_start::LonghandsToSerialize::from_iter(declarations) {
                        Ok(longhands) => longhands.to_css(dest),
                        Err(_) => Err(fmt::Error)
                    }
                },
                ShorthandId::BorderInlineEnd => {
                    match shorthands::border_inline_end::LonghandsToSerialize::from_iter(declarations) {
                        Ok(longhands) => longhands.to_css(dest),
                        Err(_) => Err(fmt::Error)
                    }
                },
                ShorthandId::Border => {
                    match shorthands::border::LonghandsToSerialize::from_iter(declarations) {
                        Ok(longhands) => longhands.to_css(dest),
                        Err(_) => Err(fmt::Error)
                    }
                },
                ShorthandId::BorderRadius => {
                    match shorthands::border_radius::LonghandsToSerialize::from_iter(declarations) {
                        Ok(longhands) => longhands.to_css(dest),
                        Err(_) => Err(fmt::Error)
                    }
                },
                ShorthandId::BorderImage => {
                    match shorthands::border_image::LonghandsToSerialize::from_iter(declarations) {
                        Ok(longhands) => longhands.to_css(dest),
                        Err(_) => Err(fmt::Error)
                    }
                },
                ShorthandId::Overflow => {
                    match shorthands::overflow::LonghandsToSerialize::from_iter(declarations) {
                        Ok(longhands) => longhands.to_css(dest),
                        Err(_) => Err(fmt::Error)
                    }
                },
                ShorthandId::Transition => {
                    match shorthands::transition::LonghandsToSerialize::from_iter(declarations) {
                        Ok(longhands) => longhands.to_css(dest),
                        Err(_) => Err(fmt::Error)
                    }
                },
                ShorthandId::Animation => {
                    match shorthands::animation::LonghandsToSerialize::from_iter(declarations) {
                        Ok(longhands) => longhands.to_css(dest),
                        Err(_) => Err(fmt::Error)
                    }
                },
                ShorthandId::ScrollSnapType => {
                    match shorthands::scroll_snap_type::LonghandsToSerialize::from_iter(declarations) {
                        Ok(longhands) => longhands.to_css(dest),
                        Err(_) => Err(fmt::Error)
                    }
                },
                ShorthandId::MozTransform => {
                    match shorthands::_moz_transform::LonghandsToSerialize::from_iter(declarations) {
                        Ok(longhands) => longhands.to_css(dest),
                        Err(_) => Err(fmt::Error)
                    }
                },
                ShorthandId::Columns => {
                    match shorthands::columns::LonghandsToSerialize::from_iter(declarations) {
                        Ok(longhands) => longhands.to_css(dest),
                        Err(_) => Err(fmt::Error)
                    }
                },
                ShorthandId::ColumnRule => {
                    match shorthands::column_rule::LonghandsToSerialize::from_iter(declarations) {
                        Ok(longhands) => longhands.to_css(dest),
                        Err(_) => Err(fmt::Error)
                    }
                },
                ShorthandId::Font => {
                    match shorthands::font::LonghandsToSerialize::from_iter(declarations) {
                        Ok(longhands) => longhands.to_css(dest),
                        Err(_) => Err(fmt::Error)
                    }
                },
                ShorthandId::FontVariant => {
                    match shorthands::font_variant::LonghandsToSerialize::from_iter(declarations) {
                        Ok(longhands) => longhands.to_css(dest),
                        Err(_) => Err(fmt::Error)
                    }
                },
                ShorthandId::TextEmphasis => {
                    match shorthands::text_emphasis::LonghandsToSerialize::from_iter(declarations) {
                        Ok(longhands) => longhands.to_css(dest),
                        Err(_) => Err(fmt::Error)
                    }
                },
                ShorthandId::WebkitTextStroke => {
                    match shorthands::_webkit_text_stroke::LonghandsToSerialize::from_iter(declarations) {
                        Ok(longhands) => longhands.to_css(dest),
                        Err(_) => Err(fmt::Error)
                    }
                },
                ShorthandId::ListStyle => {
                    match shorthands::list_style::LonghandsToSerialize::from_iter(declarations) {
                        Ok(longhands) => longhands.to_css(dest),
                        Err(_) => Err(fmt::Error)
                    }
                },
                ShorthandId::Margin => {
                    match shorthands::margin::LonghandsToSerialize::from_iter(declarations) {
                        Ok(longhands) => longhands.to_css(dest),
                        Err(_) => Err(fmt::Error)
                    }
                },
                ShorthandId::Mask => {
                    match shorthands::mask::LonghandsToSerialize::from_iter(declarations) {
                        Ok(longhands) => longhands.to_css(dest),
                        Err(_) => Err(fmt::Error)
                    }
                },
                ShorthandId::MaskPosition => {
                    match shorthands::mask_position::LonghandsToSerialize::from_iter(declarations) {
                        Ok(longhands) => longhands.to_css(dest),
                        Err(_) => Err(fmt::Error)
                    }
                },
                ShorthandId::Outline => {
                    match shorthands::outline::LonghandsToSerialize::from_iter(declarations) {
                        Ok(longhands) => longhands.to_css(dest),
                        Err(_) => Err(fmt::Error)
                    }
                },
                ShorthandId::MozOutlineRadius => {
                    match shorthands::_moz_outline_radius::LonghandsToSerialize::from_iter(declarations) {
                        Ok(longhands) => longhands.to_css(dest),
                        Err(_) => Err(fmt::Error)
                    }
                },
                ShorthandId::Padding => {
                    match shorthands::padding::LonghandsToSerialize::from_iter(declarations) {
                        Ok(longhands) => longhands.to_css(dest),
                        Err(_) => Err(fmt::Error)
                    }
                },
                ShorthandId::FlexFlow => {
                    match shorthands::flex_flow::LonghandsToSerialize::from_iter(declarations) {
                        Ok(longhands) => longhands.to_css(dest),
                        Err(_) => Err(fmt::Error)
                    }
                },
                ShorthandId::Flex => {
                    match shorthands::flex::LonghandsToSerialize::from_iter(declarations) {
                        Ok(longhands) => longhands.to_css(dest),
                        Err(_) => Err(fmt::Error)
                    }
                },
                ShorthandId::GridGap => {
                    match shorthands::grid_gap::LonghandsToSerialize::from_iter(declarations) {
                        Ok(longhands) => longhands.to_css(dest),
                        Err(_) => Err(fmt::Error)
                    }
                },
                ShorthandId::GridRow => {
                    match shorthands::grid_row::LonghandsToSerialize::from_iter(declarations) {
                        Ok(longhands) => longhands.to_css(dest),
                        Err(_) => Err(fmt::Error)
                    }
                },
                ShorthandId::GridColumn => {
                    match shorthands::grid_column::LonghandsToSerialize::from_iter(declarations) {
                        Ok(longhands) => longhands.to_css(dest),
                        Err(_) => Err(fmt::Error)
                    }
                },
                ShorthandId::GridArea => {
                    match shorthands::grid_area::LonghandsToSerialize::from_iter(declarations) {
                        Ok(longhands) => longhands.to_css(dest),
                        Err(_) => Err(fmt::Error)
                    }
                },
                ShorthandId::GridTemplate => {
                    match shorthands::grid_template::LonghandsToSerialize::from_iter(declarations) {
                        Ok(longhands) => longhands.to_css(dest),
                        Err(_) => Err(fmt::Error)
                    }
                },
                ShorthandId::Grid => {
                    match shorthands::grid::LonghandsToSerialize::from_iter(declarations) {
                        Ok(longhands) => longhands.to_css(dest),
                        Err(_) => Err(fmt::Error)
                    }
                },
                ShorthandId::PlaceContent => {
                    match shorthands::place_content::LonghandsToSerialize::from_iter(declarations) {
                        Ok(longhands) => longhands.to_css(dest),
                        Err(_) => Err(fmt::Error)
                    }
                },
                ShorthandId::PlaceSelf => {
                    match shorthands::place_self::LonghandsToSerialize::from_iter(declarations) {
                        Ok(longhands) => longhands.to_css(dest),
                        Err(_) => Err(fmt::Error)
                    }
                },
                ShorthandId::PlaceItems => {
                    match shorthands::place_items::LonghandsToSerialize::from_iter(declarations) {
                        Ok(longhands) => longhands.to_css(dest),
                        Err(_) => Err(fmt::Error)
                    }
                },
                ShorthandId::Marker => {
                    match shorthands::marker::LonghandsToSerialize::from_iter(declarations) {
                        Ok(longhands) => longhands.to_css(dest),
                        Err(_) => Err(fmt::Error)
                    }
                },
                ShorthandId::TextDecoration => {
                    match shorthands::text_decoration::LonghandsToSerialize::from_iter(declarations) {
                        Ok(longhands) => longhands.to_css(dest),
                        Err(_) => Err(fmt::Error)
                    }
                },
        }
    }

    /// Finds and returns an appendable value for the given declarations.
    ///
    /// Returns the optional appendable value.
    pub fn get_shorthand_appendable_value<'a, I>(self,
                                                 declarations: I)
                                                 -> Option<AppendableValue<'a, I::IntoIter>>
        where I: IntoIterator<Item=&'a PropertyDeclaration>,
              I::IntoIter: Clone,
    {
        let declarations = declarations.into_iter();

        // Only cloning iterators (a few pointers each) not declarations.
        let mut declarations2 = declarations.clone();
        let mut declarations3 = declarations.clone();

        let first_declaration = match declarations2.next() {
            Some(declaration) => declaration,
            None => return None
        };

        // https://drafts.csswg.org/css-variables/#variables-in-shorthands
        if let Some(css) = first_declaration.with_variables_from_shorthand(self) {
            if declarations2.all(|d| d.with_variables_from_shorthand(self) == Some(css)) {
               return Some(AppendableValue::Css {
                   css: css,
                   with_variables: true,
               });
            }
            return None;
        }

        // Check whether they are all the same CSS-wide keyword.
        if let Some(keyword) = first_declaration.get_css_wide_keyword() {
            if declarations2.all(|d| d.get_css_wide_keyword() == Some(keyword)) {
                return Some(AppendableValue::Css {
                    css: keyword.to_str(),
                    with_variables: false,
                });
            }
            return None;
        }

        // Check whether all declarations can be serialized as part of shorthand.
        if declarations3.all(|d| d.may_serialize_as_part_of_shorthand()) {
            return Some(AppendableValue::DeclarationsForShorthand(self, declarations));
        }

        None
    }

    /// Returns PropertyFlags for given shorthand property.
    pub fn flags(&self) -> PropertyFlags {
        match *self {
                ShorthandId::Background =>
                    PropertyFlags::empty(),
                ShorthandId::BackgroundPosition =>
                    PropertyFlags::empty(),
                ShorthandId::BorderColor =>
                    PropertyFlags::empty(),
                ShorthandId::BorderStyle =>
                    PropertyFlags::empty(),
                ShorthandId::BorderWidth =>
                    PropertyFlags::empty(),
                ShorthandId::BorderTop =>
                    PropertyFlags::empty(),
                ShorthandId::BorderLeft =>
                    PropertyFlags::empty(),
                ShorthandId::BorderBottom =>
                    PropertyFlags::empty(),
                ShorthandId::BorderRight =>
                    PropertyFlags::empty(),
                ShorthandId::BorderBlockStart =>
                    PropertyFlags::empty(),
                ShorthandId::BorderBlockEnd =>
                    PropertyFlags::empty(),
                ShorthandId::BorderInlineStart =>
                    PropertyFlags::empty(),
                ShorthandId::BorderInlineEnd =>
                    PropertyFlags::empty(),
                ShorthandId::Border =>
                    PropertyFlags::empty(),
                ShorthandId::BorderRadius =>
                    PropertyFlags::empty(),
                ShorthandId::BorderImage =>
                    PropertyFlags::empty(),
                ShorthandId::Overflow =>
                    PropertyFlags::empty(),
                ShorthandId::Transition =>
                    PropertyFlags::empty(),
                ShorthandId::Animation =>
                    PropertyFlags::empty(),
                ShorthandId::ScrollSnapType =>
                    PropertyFlags::empty(),
                ShorthandId::MozTransform =>
                        SHORTHAND_ALIAS_PROPERTY |
                    PropertyFlags::empty(),
                ShorthandId::Columns =>
                    PropertyFlags::empty(),
                ShorthandId::ColumnRule =>
                    PropertyFlags::empty(),
                ShorthandId::Font =>
                    PropertyFlags::empty(),
                ShorthandId::FontVariant =>
                    PropertyFlags::empty(),
                ShorthandId::TextEmphasis =>
                    PropertyFlags::empty(),
                ShorthandId::WebkitTextStroke =>
                    PropertyFlags::empty(),
                ShorthandId::ListStyle =>
                    PropertyFlags::empty(),
                ShorthandId::Margin =>
                    PropertyFlags::empty(),
                ShorthandId::Mask =>
                    PropertyFlags::empty(),
                ShorthandId::MaskPosition =>
                    PropertyFlags::empty(),
                ShorthandId::Outline =>
                    PropertyFlags::empty(),
                ShorthandId::MozOutlineRadius =>
                    PropertyFlags::empty(),
                ShorthandId::Padding =>
                    PropertyFlags::empty(),
                ShorthandId::FlexFlow =>
                    PropertyFlags::empty(),
                ShorthandId::Flex =>
                    PropertyFlags::empty(),
                ShorthandId::GridGap =>
                    PropertyFlags::empty(),
                ShorthandId::GridRow =>
                    PropertyFlags::empty(),
                ShorthandId::GridColumn =>
                    PropertyFlags::empty(),
                ShorthandId::GridArea =>
                    PropertyFlags::empty(),
                ShorthandId::GridTemplate =>
                    PropertyFlags::empty(),
                ShorthandId::Grid =>
                    PropertyFlags::empty(),
                ShorthandId::PlaceContent =>
                    PropertyFlags::empty(),
                ShorthandId::PlaceSelf =>
                    PropertyFlags::empty(),
                ShorthandId::PlaceItems =>
                    PropertyFlags::empty(),
                ShorthandId::Marker =>
                    PropertyFlags::empty(),
                ShorthandId::TextDecoration =>
                    PropertyFlags::empty(),
                ShorthandId::All =>
                    PropertyFlags::empty(),
        }
    }

    fn parse_into<'i, 't>(&self, declarations: &mut SourcePropertyDeclaration,
                          context: &ParserContext, input: &mut Parser<'i, 't>)
                          -> Result<(), ParseError<'i>> {
        match *self {
                ShorthandId::Background => {
                    shorthands::background::parse_into(declarations, context, input)
                }
                ShorthandId::BackgroundPosition => {
                    shorthands::background_position::parse_into(declarations, context, input)
                }
                ShorthandId::BorderColor => {
                    shorthands::border_color::parse_into(declarations, context, input)
                }
                ShorthandId::BorderStyle => {
                    shorthands::border_style::parse_into(declarations, context, input)
                }
                ShorthandId::BorderWidth => {
                    shorthands::border_width::parse_into(declarations, context, input)
                }
                ShorthandId::BorderTop => {
                    shorthands::border_top::parse_into(declarations, context, input)
                }
                ShorthandId::BorderLeft => {
                    shorthands::border_left::parse_into(declarations, context, input)
                }
                ShorthandId::BorderBottom => {
                    shorthands::border_bottom::parse_into(declarations, context, input)
                }
                ShorthandId::BorderRight => {
                    shorthands::border_right::parse_into(declarations, context, input)
                }
                ShorthandId::BorderBlockStart => {
                    shorthands::border_block_start::parse_into(declarations, context, input)
                }
                ShorthandId::BorderBlockEnd => {
                    shorthands::border_block_end::parse_into(declarations, context, input)
                }
                ShorthandId::BorderInlineStart => {
                    shorthands::border_inline_start::parse_into(declarations, context, input)
                }
                ShorthandId::BorderInlineEnd => {
                    shorthands::border_inline_end::parse_into(declarations, context, input)
                }
                ShorthandId::Border => {
                    shorthands::border::parse_into(declarations, context, input)
                }
                ShorthandId::BorderRadius => {
                    shorthands::border_radius::parse_into(declarations, context, input)
                }
                ShorthandId::BorderImage => {
                    shorthands::border_image::parse_into(declarations, context, input)
                }
                ShorthandId::Overflow => {
                    shorthands::overflow::parse_into(declarations, context, input)
                }
                ShorthandId::Transition => {
                    shorthands::transition::parse_into(declarations, context, input)
                }
                ShorthandId::Animation => {
                    shorthands::animation::parse_into(declarations, context, input)
                }
                ShorthandId::ScrollSnapType => {
                    shorthands::scroll_snap_type::parse_into(declarations, context, input)
                }
                ShorthandId::MozTransform => {
                    shorthands::_moz_transform::parse_into(declarations, context, input)
                }
                ShorthandId::Columns => {
                    shorthands::columns::parse_into(declarations, context, input)
                }
                ShorthandId::ColumnRule => {
                    shorthands::column_rule::parse_into(declarations, context, input)
                }
                ShorthandId::Font => {
                    shorthands::font::parse_into(declarations, context, input)
                }
                ShorthandId::FontVariant => {
                    shorthands::font_variant::parse_into(declarations, context, input)
                }
                ShorthandId::TextEmphasis => {
                    shorthands::text_emphasis::parse_into(declarations, context, input)
                }
                ShorthandId::WebkitTextStroke => {
                    shorthands::_webkit_text_stroke::parse_into(declarations, context, input)
                }
                ShorthandId::ListStyle => {
                    shorthands::list_style::parse_into(declarations, context, input)
                }
                ShorthandId::Margin => {
                    shorthands::margin::parse_into(declarations, context, input)
                }
                ShorthandId::Mask => {
                    shorthands::mask::parse_into(declarations, context, input)
                }
                ShorthandId::MaskPosition => {
                    shorthands::mask_position::parse_into(declarations, context, input)
                }
                ShorthandId::Outline => {
                    shorthands::outline::parse_into(declarations, context, input)
                }
                ShorthandId::MozOutlineRadius => {
                    shorthands::_moz_outline_radius::parse_into(declarations, context, input)
                }
                ShorthandId::Padding => {
                    shorthands::padding::parse_into(declarations, context, input)
                }
                ShorthandId::FlexFlow => {
                    shorthands::flex_flow::parse_into(declarations, context, input)
                }
                ShorthandId::Flex => {
                    shorthands::flex::parse_into(declarations, context, input)
                }
                ShorthandId::GridGap => {
                    shorthands::grid_gap::parse_into(declarations, context, input)
                }
                ShorthandId::GridRow => {
                    shorthands::grid_row::parse_into(declarations, context, input)
                }
                ShorthandId::GridColumn => {
                    shorthands::grid_column::parse_into(declarations, context, input)
                }
                ShorthandId::GridArea => {
                    shorthands::grid_area::parse_into(declarations, context, input)
                }
                ShorthandId::GridTemplate => {
                    shorthands::grid_template::parse_into(declarations, context, input)
                }
                ShorthandId::Grid => {
                    shorthands::grid::parse_into(declarations, context, input)
                }
                ShorthandId::PlaceContent => {
                    shorthands::place_content::parse_into(declarations, context, input)
                }
                ShorthandId::PlaceSelf => {
                    shorthands::place_self::parse_into(declarations, context, input)
                }
                ShorthandId::PlaceItems => {
                    shorthands::place_items::parse_into(declarations, context, input)
                }
                ShorthandId::Marker => {
                    shorthands::marker::parse_into(declarations, context, input)
                }
                ShorthandId::TextDecoration => {
                    shorthands::text_decoration::parse_into(declarations, context, input)
                }
            // 'all' accepts no value other than CSS-wide keywords
            ShorthandId::All => Err(StyleParseError::UnspecifiedError.into())
        }
    }
}

/// Servo's representation of a declared value for a given `T`, which is the
/// declared value for that property.
#[derive(Clone, Debug, Eq, PartialEq)]
pub enum DeclaredValue<'a, T: 'a> {
    /// A known specified value from the stylesheet.
    Value(&'a T),
    /// An unparsed value that contains `var()` functions.
    WithVariables(&'a Arc<UnparsedValue>),
    /// An CSS-wide keyword.
    CSSWideKeyword(CSSWideKeyword),
}

/// A variant of DeclaredValue that owns its data. This separation exists so
/// that PropertyDeclaration can avoid embedding a DeclaredValue (and its
/// extra discriminant word) and synthesize dependent DeclaredValues for
/// PropertyDeclaration instances as needed.
#[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
#[derive(Clone, Debug, Eq, PartialEq)]
pub enum DeclaredValueOwned<T> {
    /// A known specified value from the stylesheet.
    Value(T),
    /// An unparsed value that contains `var()` functions.
    WithVariables(
        #[cfg_attr(feature = "gecko", ignore_malloc_size_of = "XXX: how to handle this?")]
        Arc<UnparsedValue>
    ),
    /// An CSS-wide keyword.
    CSSWideKeyword(CSSWideKeyword),
}

impl<T> DeclaredValueOwned<T> {
    /// Creates a dependent DeclaredValue from this DeclaredValueOwned.
    fn borrow(&self) -> DeclaredValue<T> {
        match *self {
            DeclaredValueOwned::Value(ref v) => DeclaredValue::Value(v),
            DeclaredValueOwned::WithVariables(ref v) => DeclaredValue::WithVariables(v),
            DeclaredValueOwned::CSSWideKeyword(v) => DeclaredValue::CSSWideKeyword(v),
        }
    }
}

/// An unparsed property value that contains `var()` functions.
#[derive(Debug, Eq, PartialEq)]
pub struct UnparsedValue {
    /// The css serialization for this value.
    css: String,
    /// The first token type for this serialization.
    first_token_type: TokenSerializationType,
    /// The url data for resolving url values.
    url_data: UrlExtraData,
    /// The shorthand this came from.
    from_shorthand: Option<ShorthandId>,
}

impl UnparsedValue {
    fn substitute_variables(&self, longhand_id: LonghandId,
                            custom_properties: &Option<Arc<::custom_properties::CustomPropertiesMap>>,
                            quirks_mode: QuirksMode)
                            -> PropertyDeclaration {
        ::custom_properties::substitute(&self.css, self.first_token_type, custom_properties)
        .ok()
        .and_then(|css| {
            // As of this writing, only the base URL is used for property values:
            let context = ParserContext::new(Origin::Author,
                                             &self.url_data,
                                             None,
                                             PARSING_MODE_DEFAULT,
                                             quirks_mode);
            let mut input = ParserInput::new(&css);
            Parser::new(&mut input).parse_entirely(|input| {
                match self.from_shorthand {
                    None => longhand_id.parse_value(&context, input),
                    Some(ShorthandId::All) => {
                        // No need to parse the 'all' shorthand as anything other than a CSS-wide
                        // keyword, after variable substitution.
                        Err(SelectorParseError::UnexpectedIdent("all".into()).into())
                    }
                        Some(ShorthandId::Background) => {
                            shorthands::background::parse_value(&context, input)
                            .map(|longhands| {
                                match longhand_id {
                                        LonghandId::BackgroundColor => {
                                            PropertyDeclaration::BackgroundColor(
                                                longhands.background_color
                                            )
                                        }
                                        LonghandId::BackgroundPositionX => {
                                            PropertyDeclaration::BackgroundPositionX(
                                                longhands.background_position_x
                                            )
                                        }
                                        LonghandId::BackgroundPositionY => {
                                            PropertyDeclaration::BackgroundPositionY(
                                                longhands.background_position_y
                                            )
                                        }
                                        LonghandId::BackgroundRepeat => {
                                            PropertyDeclaration::BackgroundRepeat(
                                                longhands.background_repeat
                                            )
                                        }
                                        LonghandId::BackgroundAttachment => {
                                            PropertyDeclaration::BackgroundAttachment(
                                                longhands.background_attachment
                                            )
                                        }
                                        LonghandId::BackgroundImage => {
                                            PropertyDeclaration::BackgroundImage(
                                                longhands.background_image
                                            )
                                        }
                                        LonghandId::BackgroundSize => {
                                            PropertyDeclaration::BackgroundSize(
                                                longhands.background_size
                                            )
                                        }
                                        LonghandId::BackgroundOrigin => {
                                            PropertyDeclaration::BackgroundOrigin(
                                                longhands.background_origin
                                            )
                                        }
                                        LonghandId::BackgroundClip => {
                                            PropertyDeclaration::BackgroundClip(
                                                longhands.background_clip
                                            )
                                        }
                                    _ => unreachable!()
                                }
                            })
                        }
                        Some(ShorthandId::BackgroundPosition) => {
                            shorthands::background_position::parse_value(&context, input)
                            .map(|longhands| {
                                match longhand_id {
                                        LonghandId::BackgroundPositionX => {
                                            PropertyDeclaration::BackgroundPositionX(
                                                longhands.background_position_x
                                            )
                                        }
                                        LonghandId::BackgroundPositionY => {
                                            PropertyDeclaration::BackgroundPositionY(
                                                longhands.background_position_y
                                            )
                                        }
                                    _ => unreachable!()
                                }
                            })
                        }
                        Some(ShorthandId::BorderColor) => {
                            shorthands::border_color::parse_value(&context, input)
                            .map(|longhands| {
                                match longhand_id {
                                        LonghandId::BorderTopColor => {
                                            PropertyDeclaration::BorderTopColor(
                                                longhands.border_top_color
                                            )
                                        }
                                        LonghandId::BorderRightColor => {
                                            PropertyDeclaration::BorderRightColor(
                                                longhands.border_right_color
                                            )
                                        }
                                        LonghandId::BorderBottomColor => {
                                            PropertyDeclaration::BorderBottomColor(
                                                longhands.border_bottom_color
                                            )
                                        }
                                        LonghandId::BorderLeftColor => {
                                            PropertyDeclaration::BorderLeftColor(
                                                longhands.border_left_color
                                            )
                                        }
                                    _ => unreachable!()
                                }
                            })
                        }
                        Some(ShorthandId::BorderStyle) => {
                            shorthands::border_style::parse_value(&context, input)
                            .map(|longhands| {
                                match longhand_id {
                                        LonghandId::BorderTopStyle => {
                                            PropertyDeclaration::BorderTopStyle(
                                                longhands.border_top_style
                                            )
                                        }
                                        LonghandId::BorderRightStyle => {
                                            PropertyDeclaration::BorderRightStyle(
                                                longhands.border_right_style
                                            )
                                        }
                                        LonghandId::BorderBottomStyle => {
                                            PropertyDeclaration::BorderBottomStyle(
                                                longhands.border_bottom_style
                                            )
                                        }
                                        LonghandId::BorderLeftStyle => {
                                            PropertyDeclaration::BorderLeftStyle(
                                                longhands.border_left_style
                                            )
                                        }
                                    _ => unreachable!()
                                }
                            })
                        }
                        Some(ShorthandId::BorderWidth) => {
                            shorthands::border_width::parse_value(&context, input)
                            .map(|longhands| {
                                match longhand_id {
                                        LonghandId::BorderTopWidth => {
                                            PropertyDeclaration::BorderTopWidth(
                                                longhands.border_top_width
                                            )
                                        }
                                        LonghandId::BorderLeftWidth => {
                                            PropertyDeclaration::BorderLeftWidth(
                                                longhands.border_left_width
                                            )
                                        }
                                        LonghandId::BorderBottomWidth => {
                                            PropertyDeclaration::BorderBottomWidth(
                                                longhands.border_bottom_width
                                            )
                                        }
                                        LonghandId::BorderRightWidth => {
                                            PropertyDeclaration::BorderRightWidth(
                                                longhands.border_right_width
                                            )
                                        }
                                    _ => unreachable!()
                                }
                            })
                        }
                        Some(ShorthandId::BorderTop) => {
                            shorthands::border_top::parse_value(&context, input)
                            .map(|longhands| {
                                match longhand_id {
                                        LonghandId::BorderTopColor => {
                                            PropertyDeclaration::BorderTopColor(
                                                longhands.border_top_color
                                            )
                                        }
                                        LonghandId::BorderTopStyle => {
                                            PropertyDeclaration::BorderTopStyle(
                                                longhands.border_top_style
                                            )
                                        }
                                        LonghandId::BorderTopWidth => {
                                            PropertyDeclaration::BorderTopWidth(
                                                longhands.border_top_width
                                            )
                                        }
                                    _ => unreachable!()
                                }
                            })
                        }
                        Some(ShorthandId::BorderLeft) => {
                            shorthands::border_left::parse_value(&context, input)
                            .map(|longhands| {
                                match longhand_id {
                                        LonghandId::BorderLeftColor => {
                                            PropertyDeclaration::BorderLeftColor(
                                                longhands.border_left_color
                                            )
                                        }
                                        LonghandId::BorderLeftStyle => {
                                            PropertyDeclaration::BorderLeftStyle(
                                                longhands.border_left_style
                                            )
                                        }
                                        LonghandId::BorderLeftWidth => {
                                            PropertyDeclaration::BorderLeftWidth(
                                                longhands.border_left_width
                                            )
                                        }
                                    _ => unreachable!()
                                }
                            })
                        }
                        Some(ShorthandId::BorderBottom) => {
                            shorthands::border_bottom::parse_value(&context, input)
                            .map(|longhands| {
                                match longhand_id {
                                        LonghandId::BorderBottomColor => {
                                            PropertyDeclaration::BorderBottomColor(
                                                longhands.border_bottom_color
                                            )
                                        }
                                        LonghandId::BorderBottomStyle => {
                                            PropertyDeclaration::BorderBottomStyle(
                                                longhands.border_bottom_style
                                            )
                                        }
                                        LonghandId::BorderBottomWidth => {
                                            PropertyDeclaration::BorderBottomWidth(
                                                longhands.border_bottom_width
                                            )
                                        }
                                    _ => unreachable!()
                                }
                            })
                        }
                        Some(ShorthandId::BorderRight) => {
                            shorthands::border_right::parse_value(&context, input)
                            .map(|longhands| {
                                match longhand_id {
                                        LonghandId::BorderRightColor => {
                                            PropertyDeclaration::BorderRightColor(
                                                longhands.border_right_color
                                            )
                                        }
                                        LonghandId::BorderRightStyle => {
                                            PropertyDeclaration::BorderRightStyle(
                                                longhands.border_right_style
                                            )
                                        }
                                        LonghandId::BorderRightWidth => {
                                            PropertyDeclaration::BorderRightWidth(
                                                longhands.border_right_width
                                            )
                                        }
                                    _ => unreachable!()
                                }
                            })
                        }
                        Some(ShorthandId::BorderBlockStart) => {
                            shorthands::border_block_start::parse_value(&context, input)
                            .map(|longhands| {
                                match longhand_id {
                                        LonghandId::BorderBlockStartColor => {
                                            PropertyDeclaration::BorderBlockStartColor(
                                                longhands.border_block_start_color
                                            )
                                        }
                                        LonghandId::BorderBlockStartStyle => {
                                            PropertyDeclaration::BorderBlockStartStyle(
                                                longhands.border_block_start_style
                                            )
                                        }
                                        LonghandId::BorderBlockStartWidth => {
                                            PropertyDeclaration::BorderBlockStartWidth(
                                                longhands.border_block_start_width
                                            )
                                        }
                                    _ => unreachable!()
                                }
                            })
                        }
                        Some(ShorthandId::BorderBlockEnd) => {
                            shorthands::border_block_end::parse_value(&context, input)
                            .map(|longhands| {
                                match longhand_id {
                                        LonghandId::BorderBlockEndColor => {
                                            PropertyDeclaration::BorderBlockEndColor(
                                                longhands.border_block_end_color
                                            )
                                        }
                                        LonghandId::BorderBlockEndStyle => {
                                            PropertyDeclaration::BorderBlockEndStyle(
                                                longhands.border_block_end_style
                                            )
                                        }
                                        LonghandId::BorderBlockEndWidth => {
                                            PropertyDeclaration::BorderBlockEndWidth(
                                                longhands.border_block_end_width
                                            )
                                        }
                                    _ => unreachable!()
                                }
                            })
                        }
                        Some(ShorthandId::BorderInlineStart) => {
                            shorthands::border_inline_start::parse_value(&context, input)
                            .map(|longhands| {
                                match longhand_id {
                                        LonghandId::BorderInlineStartColor => {
                                            PropertyDeclaration::BorderInlineStartColor(
                                                longhands.border_inline_start_color
                                            )
                                        }
                                        LonghandId::BorderInlineStartStyle => {
                                            PropertyDeclaration::BorderInlineStartStyle(
                                                longhands.border_inline_start_style
                                            )
                                        }
                                        LonghandId::BorderInlineStartWidth => {
                                            PropertyDeclaration::BorderInlineStartWidth(
                                                longhands.border_inline_start_width
                                            )
                                        }
                                    _ => unreachable!()
                                }
                            })
                        }
                        Some(ShorthandId::BorderInlineEnd) => {
                            shorthands::border_inline_end::parse_value(&context, input)
                            .map(|longhands| {
                                match longhand_id {
                                        LonghandId::BorderInlineEndColor => {
                                            PropertyDeclaration::BorderInlineEndColor(
                                                longhands.border_inline_end_color
                                            )
                                        }
                                        LonghandId::BorderInlineEndStyle => {
                                            PropertyDeclaration::BorderInlineEndStyle(
                                                longhands.border_inline_end_style
                                            )
                                        }
                                        LonghandId::BorderInlineEndWidth => {
                                            PropertyDeclaration::BorderInlineEndWidth(
                                                longhands.border_inline_end_width
                                            )
                                        }
                                    _ => unreachable!()
                                }
                            })
                        }
                        Some(ShorthandId::Border) => {
                            shorthands::border::parse_value(&context, input)
                            .map(|longhands| {
                                match longhand_id {
                                        LonghandId::BorderTopColor => {
                                            PropertyDeclaration::BorderTopColor(
                                                longhands.border_top_color
                                            )
                                        }
                                        LonghandId::BorderTopStyle => {
                                            PropertyDeclaration::BorderTopStyle(
                                                longhands.border_top_style
                                            )
                                        }
                                        LonghandId::BorderTopWidth => {
                                            PropertyDeclaration::BorderTopWidth(
                                                longhands.border_top_width
                                            )
                                        }
                                        LonghandId::BorderLeftColor => {
                                            PropertyDeclaration::BorderLeftColor(
                                                longhands.border_left_color
                                            )
                                        }
                                        LonghandId::BorderLeftStyle => {
                                            PropertyDeclaration::BorderLeftStyle(
                                                longhands.border_left_style
                                            )
                                        }
                                        LonghandId::BorderLeftWidth => {
                                            PropertyDeclaration::BorderLeftWidth(
                                                longhands.border_left_width
                                            )
                                        }
                                        LonghandId::BorderBottomColor => {
                                            PropertyDeclaration::BorderBottomColor(
                                                longhands.border_bottom_color
                                            )
                                        }
                                        LonghandId::BorderBottomStyle => {
                                            PropertyDeclaration::BorderBottomStyle(
                                                longhands.border_bottom_style
                                            )
                                        }
                                        LonghandId::BorderBottomWidth => {
                                            PropertyDeclaration::BorderBottomWidth(
                                                longhands.border_bottom_width
                                            )
                                        }
                                        LonghandId::BorderRightColor => {
                                            PropertyDeclaration::BorderRightColor(
                                                longhands.border_right_color
                                            )
                                        }
                                        LonghandId::BorderRightStyle => {
                                            PropertyDeclaration::BorderRightStyle(
                                                longhands.border_right_style
                                            )
                                        }
                                        LonghandId::BorderRightWidth => {
                                            PropertyDeclaration::BorderRightWidth(
                                                longhands.border_right_width
                                            )
                                        }
                                        LonghandId::BorderImageOutset => {
                                            PropertyDeclaration::BorderImageOutset(
                                                longhands.border_image_outset
                                            )
                                        }
                                        LonghandId::BorderImageRepeat => {
                                            PropertyDeclaration::BorderImageRepeat(
                                                longhands.border_image_repeat
                                            )
                                        }
                                        LonghandId::BorderImageSlice => {
                                            PropertyDeclaration::BorderImageSlice(
                                                longhands.border_image_slice
                                            )
                                        }
                                        LonghandId::BorderImageSource => {
                                            PropertyDeclaration::BorderImageSource(
                                                longhands.border_image_source
                                            )
                                        }
                                        LonghandId::BorderImageWidth => {
                                            PropertyDeclaration::BorderImageWidth(
                                                longhands.border_image_width
                                            )
                                        }
                                        LonghandId::MozBorderTopColors => {
                                            PropertyDeclaration::MozBorderTopColors(
                                                longhands._moz_border_top_colors
                                            )
                                        }
                                        LonghandId::MozBorderLeftColors => {
                                            PropertyDeclaration::MozBorderLeftColors(
                                                longhands._moz_border_left_colors
                                            )
                                        }
                                        LonghandId::MozBorderBottomColors => {
                                            PropertyDeclaration::MozBorderBottomColors(
                                                longhands._moz_border_bottom_colors
                                            )
                                        }
                                        LonghandId::MozBorderRightColors => {
                                            PropertyDeclaration::MozBorderRightColors(
                                                longhands._moz_border_right_colors
                                            )
                                        }
                                    _ => unreachable!()
                                }
                            })
                        }
                        Some(ShorthandId::BorderRadius) => {
                            shorthands::border_radius::parse_value(&context, input)
                            .map(|longhands| {
                                match longhand_id {
                                        LonghandId::BorderTopLeftRadius => {
                                            PropertyDeclaration::BorderTopLeftRadius(
                                                longhands.border_top_left_radius
                                            )
                                        }
                                        LonghandId::BorderTopRightRadius => {
                                            PropertyDeclaration::BorderTopRightRadius(
                                                longhands.border_top_right_radius
                                            )
                                        }
                                        LonghandId::BorderBottomRightRadius => {
                                            PropertyDeclaration::BorderBottomRightRadius(
                                                longhands.border_bottom_right_radius
                                            )
                                        }
                                        LonghandId::BorderBottomLeftRadius => {
                                            PropertyDeclaration::BorderBottomLeftRadius(
                                                longhands.border_bottom_left_radius
                                            )
                                        }
                                    _ => unreachable!()
                                }
                            })
                        }
                        Some(ShorthandId::BorderImage) => {
                            shorthands::border_image::parse_value(&context, input)
                            .map(|longhands| {
                                match longhand_id {
                                        LonghandId::BorderImageOutset => {
                                            PropertyDeclaration::BorderImageOutset(
                                                longhands.border_image_outset
                                            )
                                        }
                                        LonghandId::BorderImageRepeat => {
                                            PropertyDeclaration::BorderImageRepeat(
                                                longhands.border_image_repeat
                                            )
                                        }
                                        LonghandId::BorderImageSlice => {
                                            PropertyDeclaration::BorderImageSlice(
                                                longhands.border_image_slice
                                            )
                                        }
                                        LonghandId::BorderImageSource => {
                                            PropertyDeclaration::BorderImageSource(
                                                longhands.border_image_source
                                            )
                                        }
                                        LonghandId::BorderImageWidth => {
                                            PropertyDeclaration::BorderImageWidth(
                                                longhands.border_image_width
                                            )
                                        }
                                    _ => unreachable!()
                                }
                            })
                        }
                        Some(ShorthandId::Overflow) => {
                            shorthands::overflow::parse_value(&context, input)
                            .map(|longhands| {
                                match longhand_id {
                                        LonghandId::OverflowX => {
                                            PropertyDeclaration::OverflowX(
                                                longhands.overflow_x
                                            )
                                        }
                                        LonghandId::OverflowY => {
                                            PropertyDeclaration::OverflowY(
                                                longhands.overflow_y
                                            )
                                        }
                                    _ => unreachable!()
                                }
                            })
                        }
                        Some(ShorthandId::Transition) => {
                            shorthands::transition::parse_value(&context, input)
                            .map(|longhands| {
                                match longhand_id {
                                        LonghandId::TransitionProperty => {
                                            PropertyDeclaration::TransitionProperty(
                                                longhands.transition_property
                                            )
                                        }
                                        LonghandId::TransitionDuration => {
                                            PropertyDeclaration::TransitionDuration(
                                                longhands.transition_duration
                                            )
                                        }
                                        LonghandId::TransitionTimingFunction => {
                                            PropertyDeclaration::TransitionTimingFunction(
                                                longhands.transition_timing_function
                                            )
                                        }
                                        LonghandId::TransitionDelay => {
                                            PropertyDeclaration::TransitionDelay(
                                                longhands.transition_delay
                                            )
                                        }
                                    _ => unreachable!()
                                }
                            })
                        }
                        Some(ShorthandId::Animation) => {
                            shorthands::animation::parse_value(&context, input)
                            .map(|longhands| {
                                match longhand_id {
                                        LonghandId::AnimationName => {
                                            PropertyDeclaration::AnimationName(
                                                longhands.animation_name
                                            )
                                        }
                                        LonghandId::AnimationDuration => {
                                            PropertyDeclaration::AnimationDuration(
                                                longhands.animation_duration
                                            )
                                        }
                                        LonghandId::AnimationTimingFunction => {
                                            PropertyDeclaration::AnimationTimingFunction(
                                                longhands.animation_timing_function
                                            )
                                        }
                                        LonghandId::AnimationDelay => {
                                            PropertyDeclaration::AnimationDelay(
                                                longhands.animation_delay
                                            )
                                        }
                                        LonghandId::AnimationIterationCount => {
                                            PropertyDeclaration::AnimationIterationCount(
                                                longhands.animation_iteration_count
                                            )
                                        }
                                        LonghandId::AnimationDirection => {
                                            PropertyDeclaration::AnimationDirection(
                                                longhands.animation_direction
                                            )
                                        }
                                        LonghandId::AnimationFillMode => {
                                            PropertyDeclaration::AnimationFillMode(
                                                longhands.animation_fill_mode
                                            )
                                        }
                                        LonghandId::AnimationPlayState => {
                                            PropertyDeclaration::AnimationPlayState(
                                                longhands.animation_play_state
                                            )
                                        }
                                    _ => unreachable!()
                                }
                            })
                        }
                        Some(ShorthandId::ScrollSnapType) => {
                            shorthands::scroll_snap_type::parse_value(&context, input)
                            .map(|longhands| {
                                match longhand_id {
                                        LonghandId::ScrollSnapTypeX => {
                                            PropertyDeclaration::ScrollSnapTypeX(
                                                longhands.scroll_snap_type_x
                                            )
                                        }
                                        LonghandId::ScrollSnapTypeY => {
                                            PropertyDeclaration::ScrollSnapTypeY(
                                                longhands.scroll_snap_type_y
                                            )
                                        }
                                    _ => unreachable!()
                                }
                            })
                        }
                        Some(ShorthandId::MozTransform) => {
                            shorthands::_moz_transform::parse_value(&context, input)
                            .map(|longhands| {
                                match longhand_id {
                                        LonghandId::Transform => {
                                            PropertyDeclaration::Transform(
                                                longhands.transform
                                            )
                                        }
                                    _ => unreachable!()
                                }
                            })
                        }
                        Some(ShorthandId::Columns) => {
                            shorthands::columns::parse_value(&context, input)
                            .map(|longhands| {
                                match longhand_id {
                                        LonghandId::ColumnWidth => {
                                            PropertyDeclaration::ColumnWidth(
                                                longhands.column_width
                                            )
                                        }
                                        LonghandId::ColumnCount => {
                                            PropertyDeclaration::ColumnCount(
                                                longhands.column_count
                                            )
                                        }
                                    _ => unreachable!()
                                }
                            })
                        }
                        Some(ShorthandId::ColumnRule) => {
                            shorthands::column_rule::parse_value(&context, input)
                            .map(|longhands| {
                                match longhand_id {
                                        LonghandId::ColumnRuleWidth => {
                                            PropertyDeclaration::ColumnRuleWidth(
                                                longhands.column_rule_width
                                            )
                                        }
                                        LonghandId::ColumnRuleStyle => {
                                            PropertyDeclaration::ColumnRuleStyle(
                                                longhands.column_rule_style
                                            )
                                        }
                                        LonghandId::ColumnRuleColor => {
                                            PropertyDeclaration::ColumnRuleColor(
                                                longhands.column_rule_color
                                            )
                                        }
                                    _ => unreachable!()
                                }
                            })
                        }
                        Some(ShorthandId::Font) => {
                            shorthands::font::parse_value(&context, input)
                            .map(|longhands| {
                                match longhand_id {
                                        LonghandId::FontStyle => {
                                            PropertyDeclaration::FontStyle(
                                                longhands.font_style
                                            )
                                        }
                                        LonghandId::FontVariantCaps => {
                                            PropertyDeclaration::FontVariantCaps(
                                                longhands.font_variant_caps
                                            )
                                        }
                                        LonghandId::FontWeight => {
                                            PropertyDeclaration::FontWeight(
                                                longhands.font_weight
                                            )
                                        }
                                        LonghandId::FontStretch => {
                                            PropertyDeclaration::FontStretch(
                                                longhands.font_stretch
                                            )
                                        }
                                        LonghandId::FontSize => {
                                            PropertyDeclaration::FontSize(
                                                longhands.font_size
                                            )
                                        }
                                        LonghandId::LineHeight => {
                                            PropertyDeclaration::LineHeight(
                                                longhands.line_height
                                            )
                                        }
                                        LonghandId::FontFamily => {
                                            PropertyDeclaration::FontFamily(
                                                longhands.font_family
                                            )
                                        }
                                        LonghandId::FontSizeAdjust => {
                                            PropertyDeclaration::FontSizeAdjust(
                                                longhands.font_size_adjust
                                            )
                                        }
                                        LonghandId::FontKerning => {
                                            PropertyDeclaration::FontKerning(
                                                longhands.font_kerning
                                            )
                                        }
                                        LonghandId::FontVariantAlternates => {
                                            PropertyDeclaration::FontVariantAlternates(
                                                longhands.font_variant_alternates
                                            )
                                        }
                                        LonghandId::FontVariantEastAsian => {
                                            PropertyDeclaration::FontVariantEastAsian(
                                                longhands.font_variant_east_asian
                                            )
                                        }
                                        LonghandId::FontVariantLigatures => {
                                            PropertyDeclaration::FontVariantLigatures(
                                                longhands.font_variant_ligatures
                                            )
                                        }
                                        LonghandId::FontVariantNumeric => {
                                            PropertyDeclaration::FontVariantNumeric(
                                                longhands.font_variant_numeric
                                            )
                                        }
                                        LonghandId::FontVariantPosition => {
                                            PropertyDeclaration::FontVariantPosition(
                                                longhands.font_variant_position
                                            )
                                        }
                                        LonghandId::FontLanguageOverride => {
                                            PropertyDeclaration::FontLanguageOverride(
                                                longhands.font_language_override
                                            )
                                        }
                                        LonghandId::FontFeatureSettings => {
                                            PropertyDeclaration::FontFeatureSettings(
                                                longhands.font_feature_settings
                                            )
                                        }
                                    _ => unreachable!()
                                }
                            })
                        }
                        Some(ShorthandId::FontVariant) => {
                            shorthands::font_variant::parse_value(&context, input)
                            .map(|longhands| {
                                match longhand_id {
                                        LonghandId::FontVariantCaps => {
                                            PropertyDeclaration::FontVariantCaps(
                                                longhands.font_variant_caps
                                            )
                                        }
                                        LonghandId::FontVariantAlternates => {
                                            PropertyDeclaration::FontVariantAlternates(
                                                longhands.font_variant_alternates
                                            )
                                        }
                                        LonghandId::FontVariantEastAsian => {
                                            PropertyDeclaration::FontVariantEastAsian(
                                                longhands.font_variant_east_asian
                                            )
                                        }
                                        LonghandId::FontVariantLigatures => {
                                            PropertyDeclaration::FontVariantLigatures(
                                                longhands.font_variant_ligatures
                                            )
                                        }
                                        LonghandId::FontVariantNumeric => {
                                            PropertyDeclaration::FontVariantNumeric(
                                                longhands.font_variant_numeric
                                            )
                                        }
                                        LonghandId::FontVariantPosition => {
                                            PropertyDeclaration::FontVariantPosition(
                                                longhands.font_variant_position
                                            )
                                        }
                                    _ => unreachable!()
                                }
                            })
                        }
                        Some(ShorthandId::TextEmphasis) => {
                            shorthands::text_emphasis::parse_value(&context, input)
                            .map(|longhands| {
                                match longhand_id {
                                        LonghandId::TextEmphasisStyle => {
                                            PropertyDeclaration::TextEmphasisStyle(
                                                longhands.text_emphasis_style
                                            )
                                        }
                                        LonghandId::TextEmphasisColor => {
                                            PropertyDeclaration::TextEmphasisColor(
                                                longhands.text_emphasis_color
                                            )
                                        }
                                    _ => unreachable!()
                                }
                            })
                        }
                        Some(ShorthandId::WebkitTextStroke) => {
                            shorthands::_webkit_text_stroke::parse_value(&context, input)
                            .map(|longhands| {
                                match longhand_id {
                                        LonghandId::WebkitTextStrokeWidth => {
                                            PropertyDeclaration::WebkitTextStrokeWidth(
                                                longhands._webkit_text_stroke_width
                                            )
                                        }
                                        LonghandId::WebkitTextStrokeColor => {
                                            PropertyDeclaration::WebkitTextStrokeColor(
                                                longhands._webkit_text_stroke_color
                                            )
                                        }
                                    _ => unreachable!()
                                }
                            })
                        }
                        Some(ShorthandId::ListStyle) => {
                            shorthands::list_style::parse_value(&context, input)
                            .map(|longhands| {
                                match longhand_id {
                                        LonghandId::ListStylePosition => {
                                            PropertyDeclaration::ListStylePosition(
                                                longhands.list_style_position
                                            )
                                        }
                                        LonghandId::ListStyleImage => {
                                            PropertyDeclaration::ListStyleImage(
                                                longhands.list_style_image
                                            )
                                        }
                                        LonghandId::ListStyleType => {
                                            PropertyDeclaration::ListStyleType(
                                                longhands.list_style_type
                                            )
                                        }
                                    _ => unreachable!()
                                }
                            })
                        }
                        Some(ShorthandId::Margin) => {
                            shorthands::margin::parse_value(&context, input)
                            .map(|longhands| {
                                match longhand_id {
                                        LonghandId::MarginTop => {
                                            PropertyDeclaration::MarginTop(
                                                longhands.margin_top
                                            )
                                        }
                                        LonghandId::MarginRight => {
                                            PropertyDeclaration::MarginRight(
                                                longhands.margin_right
                                            )
                                        }
                                        LonghandId::MarginBottom => {
                                            PropertyDeclaration::MarginBottom(
                                                longhands.margin_bottom
                                            )
                                        }
                                        LonghandId::MarginLeft => {
                                            PropertyDeclaration::MarginLeft(
                                                longhands.margin_left
                                            )
                                        }
                                    _ => unreachable!()
                                }
                            })
                        }
                        Some(ShorthandId::Mask) => {
                            shorthands::mask::parse_value(&context, input)
                            .map(|longhands| {
                                match longhand_id {
                                        LonghandId::MaskMode => {
                                            PropertyDeclaration::MaskMode(
                                                longhands.mask_mode
                                            )
                                        }
                                        LonghandId::MaskRepeat => {
                                            PropertyDeclaration::MaskRepeat(
                                                longhands.mask_repeat
                                            )
                                        }
                                        LonghandId::MaskClip => {
                                            PropertyDeclaration::MaskClip(
                                                longhands.mask_clip
                                            )
                                        }
                                        LonghandId::MaskOrigin => {
                                            PropertyDeclaration::MaskOrigin(
                                                longhands.mask_origin
                                            )
                                        }
                                        LonghandId::MaskComposite => {
                                            PropertyDeclaration::MaskComposite(
                                                longhands.mask_composite
                                            )
                                        }
                                        LonghandId::MaskPositionX => {
                                            PropertyDeclaration::MaskPositionX(
                                                longhands.mask_position_x
                                            )
                                        }
                                        LonghandId::MaskPositionY => {
                                            PropertyDeclaration::MaskPositionY(
                                                longhands.mask_position_y
                                            )
                                        }
                                        LonghandId::MaskSize => {
                                            PropertyDeclaration::MaskSize(
                                                longhands.mask_size
                                            )
                                        }
                                        LonghandId::MaskImage => {
                                            PropertyDeclaration::MaskImage(
                                                longhands.mask_image
                                            )
                                        }
                                    _ => unreachable!()
                                }
                            })
                        }
                        Some(ShorthandId::MaskPosition) => {
                            shorthands::mask_position::parse_value(&context, input)
                            .map(|longhands| {
                                match longhand_id {
                                        LonghandId::MaskPositionX => {
                                            PropertyDeclaration::MaskPositionX(
                                                longhands.mask_position_x
                                            )
                                        }
                                        LonghandId::MaskPositionY => {
                                            PropertyDeclaration::MaskPositionY(
                                                longhands.mask_position_y
                                            )
                                        }
                                    _ => unreachable!()
                                }
                            })
                        }
                        Some(ShorthandId::Outline) => {
                            shorthands::outline::parse_value(&context, input)
                            .map(|longhands| {
                                match longhand_id {
                                        LonghandId::OutlineWidth => {
                                            PropertyDeclaration::OutlineWidth(
                                                longhands.outline_width
                                            )
                                        }
                                        LonghandId::OutlineStyle => {
                                            PropertyDeclaration::OutlineStyle(
                                                longhands.outline_style
                                            )
                                        }
                                        LonghandId::OutlineColor => {
                                            PropertyDeclaration::OutlineColor(
                                                longhands.outline_color
                                            )
                                        }
                                    _ => unreachable!()
                                }
                            })
                        }
                        Some(ShorthandId::MozOutlineRadius) => {
                            shorthands::_moz_outline_radius::parse_value(&context, input)
                            .map(|longhands| {
                                match longhand_id {
                                        LonghandId::MozOutlineRadiusTopleft => {
                                            PropertyDeclaration::MozOutlineRadiusTopleft(
                                                longhands._moz_outline_radius_topleft
                                            )
                                        }
                                        LonghandId::MozOutlineRadiusTopright => {
                                            PropertyDeclaration::MozOutlineRadiusTopright(
                                                longhands._moz_outline_radius_topright
                                            )
                                        }
                                        LonghandId::MozOutlineRadiusBottomright => {
                                            PropertyDeclaration::MozOutlineRadiusBottomright(
                                                longhands._moz_outline_radius_bottomright
                                            )
                                        }
                                        LonghandId::MozOutlineRadiusBottomleft => {
                                            PropertyDeclaration::MozOutlineRadiusBottomleft(
                                                longhands._moz_outline_radius_bottomleft
                                            )
                                        }
                                    _ => unreachable!()
                                }
                            })
                        }
                        Some(ShorthandId::Padding) => {
                            shorthands::padding::parse_value(&context, input)
                            .map(|longhands| {
                                match longhand_id {
                                        LonghandId::PaddingTop => {
                                            PropertyDeclaration::PaddingTop(
                                                longhands.padding_top
                                            )
                                        }
                                        LonghandId::PaddingRight => {
                                            PropertyDeclaration::PaddingRight(
                                                longhands.padding_right
                                            )
                                        }
                                        LonghandId::PaddingBottom => {
                                            PropertyDeclaration::PaddingBottom(
                                                longhands.padding_bottom
                                            )
                                        }
                                        LonghandId::PaddingLeft => {
                                            PropertyDeclaration::PaddingLeft(
                                                longhands.padding_left
                                            )
                                        }
                                    _ => unreachable!()
                                }
                            })
                        }
                        Some(ShorthandId::FlexFlow) => {
                            shorthands::flex_flow::parse_value(&context, input)
                            .map(|longhands| {
                                match longhand_id {
                                        LonghandId::FlexDirection => {
                                            PropertyDeclaration::FlexDirection(
                                                longhands.flex_direction
                                            )
                                        }
                                        LonghandId::FlexWrap => {
                                            PropertyDeclaration::FlexWrap(
                                                longhands.flex_wrap
                                            )
                                        }
                                    _ => unreachable!()
                                }
                            })
                        }
                        Some(ShorthandId::Flex) => {
                            shorthands::flex::parse_value(&context, input)
                            .map(|longhands| {
                                match longhand_id {
                                        LonghandId::FlexGrow => {
                                            PropertyDeclaration::FlexGrow(
                                                longhands.flex_grow
                                            )
                                        }
                                        LonghandId::FlexShrink => {
                                            PropertyDeclaration::FlexShrink(
                                                longhands.flex_shrink
                                            )
                                        }
                                        LonghandId::FlexBasis => {
                                            PropertyDeclaration::FlexBasis(
                                                longhands.flex_basis
                                            )
                                        }
                                    _ => unreachable!()
                                }
                            })
                        }
                        Some(ShorthandId::GridGap) => {
                            shorthands::grid_gap::parse_value(&context, input)
                            .map(|longhands| {
                                match longhand_id {
                                        LonghandId::GridRowGap => {
                                            PropertyDeclaration::GridRowGap(
                                                longhands.grid_row_gap
                                            )
                                        }
                                        LonghandId::GridColumnGap => {
                                            PropertyDeclaration::GridColumnGap(
                                                longhands.grid_column_gap
                                            )
                                        }
                                    _ => unreachable!()
                                }
                            })
                        }
                        Some(ShorthandId::GridRow) => {
                            shorthands::grid_row::parse_value(&context, input)
                            .map(|longhands| {
                                match longhand_id {
                                        LonghandId::GridRowStart => {
                                            PropertyDeclaration::GridRowStart(
                                                longhands.grid_row_start
                                            )
                                        }
                                        LonghandId::GridRowEnd => {
                                            PropertyDeclaration::GridRowEnd(
                                                longhands.grid_row_end
                                            )
                                        }
                                    _ => unreachable!()
                                }
                            })
                        }
                        Some(ShorthandId::GridColumn) => {
                            shorthands::grid_column::parse_value(&context, input)
                            .map(|longhands| {
                                match longhand_id {
                                        LonghandId::GridColumnStart => {
                                            PropertyDeclaration::GridColumnStart(
                                                longhands.grid_column_start
                                            )
                                        }
                                        LonghandId::GridColumnEnd => {
                                            PropertyDeclaration::GridColumnEnd(
                                                longhands.grid_column_end
                                            )
                                        }
                                    _ => unreachable!()
                                }
                            })
                        }
                        Some(ShorthandId::GridArea) => {
                            shorthands::grid_area::parse_value(&context, input)
                            .map(|longhands| {
                                match longhand_id {
                                        LonghandId::GridRowStart => {
                                            PropertyDeclaration::GridRowStart(
                                                longhands.grid_row_start
                                            )
                                        }
                                        LonghandId::GridRowEnd => {
                                            PropertyDeclaration::GridRowEnd(
                                                longhands.grid_row_end
                                            )
                                        }
                                        LonghandId::GridColumnStart => {
                                            PropertyDeclaration::GridColumnStart(
                                                longhands.grid_column_start
                                            )
                                        }
                                        LonghandId::GridColumnEnd => {
                                            PropertyDeclaration::GridColumnEnd(
                                                longhands.grid_column_end
                                            )
                                        }
                                    _ => unreachable!()
                                }
                            })
                        }
                        Some(ShorthandId::GridTemplate) => {
                            shorthands::grid_template::parse_value(&context, input)
                            .map(|longhands| {
                                match longhand_id {
                                        LonghandId::GridTemplateRows => {
                                            PropertyDeclaration::GridTemplateRows(
                                                longhands.grid_template_rows
                                            )
                                        }
                                        LonghandId::GridTemplateColumns => {
                                            PropertyDeclaration::GridTemplateColumns(
                                                longhands.grid_template_columns
                                            )
                                        }
                                        LonghandId::GridTemplateAreas => {
                                            PropertyDeclaration::GridTemplateAreas(
                                                longhands.grid_template_areas
                                            )
                                        }
                                    _ => unreachable!()
                                }
                            })
                        }
                        Some(ShorthandId::Grid) => {
                            shorthands::grid::parse_value(&context, input)
                            .map(|longhands| {
                                match longhand_id {
                                        LonghandId::GridTemplateRows => {
                                            PropertyDeclaration::GridTemplateRows(
                                                longhands.grid_template_rows
                                            )
                                        }
                                        LonghandId::GridTemplateColumns => {
                                            PropertyDeclaration::GridTemplateColumns(
                                                longhands.grid_template_columns
                                            )
                                        }
                                        LonghandId::GridTemplateAreas => {
                                            PropertyDeclaration::GridTemplateAreas(
                                                longhands.grid_template_areas
                                            )
                                        }
                                        LonghandId::GridAutoRows => {
                                            PropertyDeclaration::GridAutoRows(
                                                longhands.grid_auto_rows
                                            )
                                        }
                                        LonghandId::GridAutoColumns => {
                                            PropertyDeclaration::GridAutoColumns(
                                                longhands.grid_auto_columns
                                            )
                                        }
                                        LonghandId::GridAutoFlow => {
                                            PropertyDeclaration::GridAutoFlow(
                                                longhands.grid_auto_flow
                                            )
                                        }
                                    _ => unreachable!()
                                }
                            })
                        }
                        Some(ShorthandId::PlaceContent) => {
                            shorthands::place_content::parse_value(&context, input)
                            .map(|longhands| {
                                match longhand_id {
                                        LonghandId::AlignContent => {
                                            PropertyDeclaration::AlignContent(
                                                longhands.align_content
                                            )
                                        }
                                        LonghandId::JustifyContent => {
                                            PropertyDeclaration::JustifyContent(
                                                longhands.justify_content
                                            )
                                        }
                                    _ => unreachable!()
                                }
                            })
                        }
                        Some(ShorthandId::PlaceSelf) => {
                            shorthands::place_self::parse_value(&context, input)
                            .map(|longhands| {
                                match longhand_id {
                                        LonghandId::AlignSelf => {
                                            PropertyDeclaration::AlignSelf(
                                                longhands.align_self
                                            )
                                        }
                                        LonghandId::JustifySelf => {
                                            PropertyDeclaration::JustifySelf(
                                                longhands.justify_self
                                            )
                                        }
                                    _ => unreachable!()
                                }
                            })
                        }
                        Some(ShorthandId::PlaceItems) => {
                            shorthands::place_items::parse_value(&context, input)
                            .map(|longhands| {
                                match longhand_id {
                                        LonghandId::AlignItems => {
                                            PropertyDeclaration::AlignItems(
                                                longhands.align_items
                                            )
                                        }
                                        LonghandId::JustifyItems => {
                                            PropertyDeclaration::JustifyItems(
                                                longhands.justify_items
                                            )
                                        }
                                    _ => unreachable!()
                                }
                            })
                        }
                        Some(ShorthandId::Marker) => {
                            shorthands::marker::parse_value(&context, input)
                            .map(|longhands| {
                                match longhand_id {
                                        LonghandId::MarkerStart => {
                                            PropertyDeclaration::MarkerStart(
                                                longhands.marker_start
                                            )
                                        }
                                        LonghandId::MarkerEnd => {
                                            PropertyDeclaration::MarkerEnd(
                                                longhands.marker_end
                                            )
                                        }
                                        LonghandId::MarkerMid => {
                                            PropertyDeclaration::MarkerMid(
                                                longhands.marker_mid
                                            )
                                        }
                                    _ => unreachable!()
                                }
                            })
                        }
                        Some(ShorthandId::TextDecoration) => {
                            shorthands::text_decoration::parse_value(&context, input)
                            .map(|longhands| {
                                match longhand_id {
                                        LonghandId::TextDecorationLine => {
                                            PropertyDeclaration::TextDecorationLine(
                                                longhands.text_decoration_line
                                            )
                                        }
                                        LonghandId::TextDecorationStyle => {
                                            PropertyDeclaration::TextDecorationStyle(
                                                longhands.text_decoration_style
                                            )
                                        }
                                        LonghandId::TextDecorationColor => {
                                            PropertyDeclaration::TextDecorationColor(
                                                longhands.text_decoration_color
                                            )
                                        }
                                    _ => unreachable!()
                                }
                            })
                        }
                }
            })
            .ok()
        })
        .unwrap_or_else(|| {
            // Invalid at computed-value time.
            let keyword = if longhand_id.inherited() {
                CSSWideKeyword::Inherit
            } else {
                CSSWideKeyword::Initial
            };
            PropertyDeclaration::CSSWideKeyword(longhand_id, keyword)
        })
    }
}

impl<'a, T: ToCss> ToCss for DeclaredValue<'a, T> {
    fn to_css<W>(&self, dest: &mut W) -> fmt::Result
        where W: fmt::Write,
    {
        match *self {
            DeclaredValue::Value(ref inner) => inner.to_css(dest),
            DeclaredValue::WithVariables(ref with_variables) => {
                // https://drafts.csswg.org/css-variables/#variables-in-shorthands
                if with_variables.from_shorthand.is_none() {
                    dest.write_str(&*with_variables.css)?
                }
                Ok(())
            },
            DeclaredValue::CSSWideKeyword(ref keyword) => keyword.to_css(dest),
        }
    }
}

/// An identifier for a given property declaration, which can be either a
/// longhand or a custom property.
#[derive(Clone, Copy, PartialEq)]
#[cfg_attr(feature = "servo", derive(HeapSizeOf))]
pub enum PropertyDeclarationId<'a> {
    /// A longhand.
    Longhand(LonghandId),
    /// A custom property declaration.
    Custom(&'a ::custom_properties::Name),
}

impl<'a> ToCss for PropertyDeclarationId<'a> {
    fn to_css<W>(&self, dest: &mut W) -> fmt::Result
        where W: fmt::Write,
    {
        match *self {
            PropertyDeclarationId::Longhand(id) => dest.write_str(id.name()),
            PropertyDeclarationId::Custom(_) => {
                serialize_identifier(&self.name(), dest)
            }
        }
    }
}

impl<'a> PropertyDeclarationId<'a> {
    /// Whether a given declaration id is either the same as `other`, or a
    /// longhand of it.
    pub fn is_or_is_longhand_of(&self, other: &PropertyId) -> bool {
        match *self {
            PropertyDeclarationId::Longhand(id) => {
                match *other {
                    PropertyId::Longhand(other_id) => id == other_id,
                    PropertyId::Shorthand(shorthand) => self.is_longhand_of(shorthand),
                    PropertyId::Custom(_) => false,
                }
            }
            PropertyDeclarationId::Custom(name) => {
                matches!(*other, PropertyId::Custom(ref other_name) if name == other_name)
            }
        }
    }

    /// Whether a given declaration id is a longhand belonging to this
    /// shorthand.
    pub fn is_longhand_of(&self, shorthand: ShorthandId) -> bool {
        match *self {
            PropertyDeclarationId::Longhand(ref id) => id.shorthands().contains(&shorthand),
            _ => false,
        }
    }

    /// Returns the name of the property without CSS escaping.
    pub fn name(&self) -> Cow<'static, str> {
        match *self {
            PropertyDeclarationId::Longhand(id) => id.name().into(),
            PropertyDeclarationId::Custom(name) => {
                use std::fmt::Write;
                let mut s = String::new();
                write!(&mut s, "--{}", name).unwrap();
                s.into()
            }
        }
    }
}

/// Servo's representation of a CSS property, that is, either a longhand, a
/// shorthand, or a custom property.
#[derive(Clone, Eq, PartialEq)]
pub enum PropertyId {
    /// A longhand property.
    Longhand(LonghandId),
    /// A shorthand property.
    Shorthand(ShorthandId),
    /// A custom property.
    Custom(::custom_properties::Name),
}

impl fmt::Debug for PropertyId {
    fn fmt(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
        self.to_css(formatter)
    }
}

impl ToCss for PropertyId {
    fn to_css<W>(&self, dest: &mut W) -> fmt::Result
        where W: fmt::Write,
    {
        match *self {
            PropertyId::Longhand(id) => dest.write_str(id.name()),
            PropertyId::Shorthand(id) => dest.write_str(id.name()),
            PropertyId::Custom(_) => {
                serialize_identifier(&self.name(), dest)
            }
        }
    }
}

impl PropertyId {
    /// Returns a given property from the string `s`.
    ///
    /// Returns Err(()) for unknown non-custom properties
    /// If caller wants to provide a different context, it can be provided with
    /// Some(context), if None is given, default setting for PropertyParserContext
    /// will be used. It is `Origin::Author` for stylesheet_origin and
    /// `CssRuleType::Style` for rule_type.
    pub fn parse(property_name: &str, context: Option< &PropertyParserContext>) -> Result<Self, ()> {
        // FIXME(https://github.com/rust-lang/rust/issues/33156): remove this enum and use PropertyId
        // when stable Rust allows destructors in statics.
        // ShorthandAlias is not used in servo build. That's why we need to allow dead_code.
        #[allow(dead_code)]
        pub enum StaticId {
            Longhand(LonghandId),
            Shorthand(ShorthandId),
            LonghandAlias(LonghandId, AliasId),
            ShorthandAlias(ShorthandId, AliasId),
        }
        ascii_case_insensitive_phf_map! {
            static_id -> StaticId = {
                        "background-color" => StaticId::Longhand(LonghandId::BackgroundColor),
                        "background-image" => StaticId::Longhand(LonghandId::BackgroundImage),
                        "background-position-x" => StaticId::Longhand(LonghandId::BackgroundPositionX),
                        "background-position-y" => StaticId::Longhand(LonghandId::BackgroundPositionY),
                        "background-repeat" => StaticId::Longhand(LonghandId::BackgroundRepeat),
                        "background-attachment" => StaticId::Longhand(LonghandId::BackgroundAttachment),
                        "background-clip" => StaticId::Longhand(LonghandId::BackgroundClip),
                            "-webkit-background-clip" => {
                                StaticId::LonghandAlias(LonghandId::BackgroundClip,
                                                       AliasId::WebkitBackgroundClip)
                            },
                        "background-origin" => StaticId::Longhand(LonghandId::BackgroundOrigin),
                            "-webkit-background-origin" => {
                                StaticId::LonghandAlias(LonghandId::BackgroundOrigin,
                                                       AliasId::WebkitBackgroundOrigin)
                            },
                        "background-size" => StaticId::Longhand(LonghandId::BackgroundSize),
                            "-webkit-background-size" => {
                                StaticId::LonghandAlias(LonghandId::BackgroundSize,
                                                       AliasId::WebkitBackgroundSize)
                            },
                        "background-blend-mode" => StaticId::Longhand(LonghandId::BackgroundBlendMode),
                        "border-top-color" => StaticId::Longhand(LonghandId::BorderTopColor),
                        "border-top-style" => StaticId::Longhand(LonghandId::BorderTopStyle),
                        "border-top-width" => StaticId::Longhand(LonghandId::BorderTopWidth),
                        "border-left-color" => StaticId::Longhand(LonghandId::BorderLeftColor),
                        "border-left-style" => StaticId::Longhand(LonghandId::BorderLeftStyle),
                        "border-left-width" => StaticId::Longhand(LonghandId::BorderLeftWidth),
                        "border-bottom-color" => StaticId::Longhand(LonghandId::BorderBottomColor),
                        "border-bottom-style" => StaticId::Longhand(LonghandId::BorderBottomStyle),
                        "border-bottom-width" => StaticId::Longhand(LonghandId::BorderBottomWidth),
                        "border-right-color" => StaticId::Longhand(LonghandId::BorderRightColor),
                        "border-right-style" => StaticId::Longhand(LonghandId::BorderRightStyle),
                        "border-right-width" => StaticId::Longhand(LonghandId::BorderRightWidth),
                        "border-block-start-color" => StaticId::Longhand(LonghandId::BorderBlockStartColor),
                        "border-block-start-style" => StaticId::Longhand(LonghandId::BorderBlockStartStyle),
                        "border-block-start-width" => StaticId::Longhand(LonghandId::BorderBlockStartWidth),
                        "border-block-end-color" => StaticId::Longhand(LonghandId::BorderBlockEndColor),
                        "border-block-end-style" => StaticId::Longhand(LonghandId::BorderBlockEndStyle),
                        "border-block-end-width" => StaticId::Longhand(LonghandId::BorderBlockEndWidth),
                        "border-inline-start-color" => StaticId::Longhand(LonghandId::BorderInlineStartColor),
                            "-moz-border-start-color" => {
                                StaticId::LonghandAlias(LonghandId::BorderInlineStartColor,
                                                       AliasId::MozBorderStartColor)
                            },
                        "border-inline-start-style" => StaticId::Longhand(LonghandId::BorderInlineStartStyle),
                            "-moz-border-start-style" => {
                                StaticId::LonghandAlias(LonghandId::BorderInlineStartStyle,
                                                       AliasId::MozBorderStartStyle)
                            },
                        "border-inline-start-width" => StaticId::Longhand(LonghandId::BorderInlineStartWidth),
                            "-moz-border-start-width" => {
                                StaticId::LonghandAlias(LonghandId::BorderInlineStartWidth,
                                                       AliasId::MozBorderStartWidth)
                            },
                        "border-inline-end-color" => StaticId::Longhand(LonghandId::BorderInlineEndColor),
                            "-moz-border-end-color" => {
                                StaticId::LonghandAlias(LonghandId::BorderInlineEndColor,
                                                       AliasId::MozBorderEndColor)
                            },
                        "border-inline-end-style" => StaticId::Longhand(LonghandId::BorderInlineEndStyle),
                            "-moz-border-end-style" => {
                                StaticId::LonghandAlias(LonghandId::BorderInlineEndStyle,
                                                       AliasId::MozBorderEndStyle)
                            },
                        "border-inline-end-width" => StaticId::Longhand(LonghandId::BorderInlineEndWidth),
                            "-moz-border-end-width" => {
                                StaticId::LonghandAlias(LonghandId::BorderInlineEndWidth,
                                                       AliasId::MozBorderEndWidth)
                            },
                        "border-top-left-radius" => StaticId::Longhand(LonghandId::BorderTopLeftRadius),
                            "-webkit-border-top-left-radius" => {
                                StaticId::LonghandAlias(LonghandId::BorderTopLeftRadius,
                                                       AliasId::WebkitBorderTopLeftRadius)
                            },
                        "border-top-right-radius" => StaticId::Longhand(LonghandId::BorderTopRightRadius),
                            "-webkit-border-top-right-radius" => {
                                StaticId::LonghandAlias(LonghandId::BorderTopRightRadius,
                                                       AliasId::WebkitBorderTopRightRadius)
                            },
                        "border-bottom-right-radius" => StaticId::Longhand(LonghandId::BorderBottomRightRadius),
                            "-webkit-border-bottom-right-radius" => {
                                StaticId::LonghandAlias(LonghandId::BorderBottomRightRadius,
                                                       AliasId::WebkitBorderBottomRightRadius)
                            },
                        "border-bottom-left-radius" => StaticId::Longhand(LonghandId::BorderBottomLeftRadius),
                            "-webkit-border-bottom-left-radius" => {
                                StaticId::LonghandAlias(LonghandId::BorderBottomLeftRadius,
                                                       AliasId::WebkitBorderBottomLeftRadius)
                            },
                        "-moz-border-top-colors" => StaticId::Longhand(LonghandId::MozBorderTopColors),
                        "-moz-border-left-colors" => StaticId::Longhand(LonghandId::MozBorderLeftColors),
                        "-moz-border-bottom-colors" => StaticId::Longhand(LonghandId::MozBorderBottomColors),
                        "-moz-border-right-colors" => StaticId::Longhand(LonghandId::MozBorderRightColors),
                        "box-decoration-break" => StaticId::Longhand(LonghandId::BoxDecorationBreak),
                        "-moz-float-edge" => StaticId::Longhand(LonghandId::MozFloatEdge),
                        "border-image-source" => StaticId::Longhand(LonghandId::BorderImageSource),
                        "border-image-outset" => StaticId::Longhand(LonghandId::BorderImageOutset),
                        "border-image-repeat" => StaticId::Longhand(LonghandId::BorderImageRepeat),
                        "border-image-width" => StaticId::Longhand(LonghandId::BorderImageWidth),
                        "border-image-slice" => StaticId::Longhand(LonghandId::BorderImageSlice),
                        "display" => StaticId::Longhand(LonghandId::Display),
                        "-moz-top-layer" => StaticId::Longhand(LonghandId::MozTopLayer),
                        "position" => StaticId::Longhand(LonghandId::Position),
                        "float" => StaticId::Longhand(LonghandId::Float),
                        "clear" => StaticId::Longhand(LonghandId::Clear),
                        "vertical-align" => StaticId::Longhand(LonghandId::VerticalAlign),
                        "overflow-clip-box" => StaticId::Longhand(LonghandId::OverflowClipBox),
                        "overflow-x" => StaticId::Longhand(LonghandId::OverflowX),
                        "overflow-y" => StaticId::Longhand(LonghandId::OverflowY),
                        "transition-duration" => StaticId::Longhand(LonghandId::TransitionDuration),
                            "-moz-transition-duration" => {
                                StaticId::LonghandAlias(LonghandId::TransitionDuration,
                                                       AliasId::MozTransitionDuration)
                            },
                            "-webkit-transition-duration" => {
                                StaticId::LonghandAlias(LonghandId::TransitionDuration,
                                                       AliasId::WebkitTransitionDuration)
                            },
                        "transition-timing-function" => StaticId::Longhand(LonghandId::TransitionTimingFunction),
                            "-moz-transition-timing-function" => {
                                StaticId::LonghandAlias(LonghandId::TransitionTimingFunction,
                                                       AliasId::MozTransitionTimingFunction)
                            },
                            "-webkit-transition-timing-function" => {
                                StaticId::LonghandAlias(LonghandId::TransitionTimingFunction,
                                                       AliasId::WebkitTransitionTimingFunction)
                            },
                        "transition-property" => StaticId::Longhand(LonghandId::TransitionProperty),
                            "-moz-transition-property" => {
                                StaticId::LonghandAlias(LonghandId::TransitionProperty,
                                                       AliasId::MozTransitionProperty)
                            },
                            "-webkit-transition-property" => {
                                StaticId::LonghandAlias(LonghandId::TransitionProperty,
                                                       AliasId::WebkitTransitionProperty)
                            },
                        "transition-delay" => StaticId::Longhand(LonghandId::TransitionDelay),
                            "-moz-transition-delay" => {
                                StaticId::LonghandAlias(LonghandId::TransitionDelay,
                                                       AliasId::MozTransitionDelay)
                            },
                            "-webkit-transition-delay" => {
                                StaticId::LonghandAlias(LonghandId::TransitionDelay,
                                                       AliasId::WebkitTransitionDelay)
                            },
                        "animation-name" => StaticId::Longhand(LonghandId::AnimationName),
                            "-moz-animation-name" => {
                                StaticId::LonghandAlias(LonghandId::AnimationName,
                                                       AliasId::MozAnimationName)
                            },
                            "-webkit-animation-name" => {
                                StaticId::LonghandAlias(LonghandId::AnimationName,
                                                       AliasId::WebkitAnimationName)
                            },
                        "animation-duration" => StaticId::Longhand(LonghandId::AnimationDuration),
                            "-moz-animation-duration" => {
                                StaticId::LonghandAlias(LonghandId::AnimationDuration,
                                                       AliasId::MozAnimationDuration)
                            },
                            "-webkit-animation-duration" => {
                                StaticId::LonghandAlias(LonghandId::AnimationDuration,
                                                       AliasId::WebkitAnimationDuration)
                            },
                        "animation-timing-function" => StaticId::Longhand(LonghandId::AnimationTimingFunction),
                            "-moz-animation-timing-function" => {
                                StaticId::LonghandAlias(LonghandId::AnimationTimingFunction,
                                                       AliasId::MozAnimationTimingFunction)
                            },
                            "-webkit-animation-timing-function" => {
                                StaticId::LonghandAlias(LonghandId::AnimationTimingFunction,
                                                       AliasId::WebkitAnimationTimingFunction)
                            },
                        "animation-iteration-count" => StaticId::Longhand(LonghandId::AnimationIterationCount),
                            "-moz-animation-iteration-count" => {
                                StaticId::LonghandAlias(LonghandId::AnimationIterationCount,
                                                       AliasId::MozAnimationIterationCount)
                            },
                            "-webkit-animation-iteration-count" => {
                                StaticId::LonghandAlias(LonghandId::AnimationIterationCount,
                                                       AliasId::WebkitAnimationIterationCount)
                            },
                        "animation-direction" => StaticId::Longhand(LonghandId::AnimationDirection),
                            "-moz-animation-direction" => {
                                StaticId::LonghandAlias(LonghandId::AnimationDirection,
                                                       AliasId::MozAnimationDirection)
                            },
                            "-webkit-animation-direction" => {
                                StaticId::LonghandAlias(LonghandId::AnimationDirection,
                                                       AliasId::WebkitAnimationDirection)
                            },
                        "animation-play-state" => StaticId::Longhand(LonghandId::AnimationPlayState),
                            "-moz-animation-play-state" => {
                                StaticId::LonghandAlias(LonghandId::AnimationPlayState,
                                                       AliasId::MozAnimationPlayState)
                            },
                            "-webkit-animation-play-state" => {
                                StaticId::LonghandAlias(LonghandId::AnimationPlayState,
                                                       AliasId::WebkitAnimationPlayState)
                            },
                        "animation-fill-mode" => StaticId::Longhand(LonghandId::AnimationFillMode),
                            "-moz-animation-fill-mode" => {
                                StaticId::LonghandAlias(LonghandId::AnimationFillMode,
                                                       AliasId::MozAnimationFillMode)
                            },
                            "-webkit-animation-fill-mode" => {
                                StaticId::LonghandAlias(LonghandId::AnimationFillMode,
                                                       AliasId::WebkitAnimationFillMode)
                            },
                        "animation-delay" => StaticId::Longhand(LonghandId::AnimationDelay),
                            "-moz-animation-delay" => {
                                StaticId::LonghandAlias(LonghandId::AnimationDelay,
                                                       AliasId::MozAnimationDelay)
                            },
                            "-webkit-animation-delay" => {
                                StaticId::LonghandAlias(LonghandId::AnimationDelay,
                                                       AliasId::WebkitAnimationDelay)
                            },
                        "scroll-snap-points-x" => StaticId::Longhand(LonghandId::ScrollSnapPointsX),
                        "scroll-snap-points-y" => StaticId::Longhand(LonghandId::ScrollSnapPointsY),
                        "scroll-snap-destination" => StaticId::Longhand(LonghandId::ScrollSnapDestination),
                        "scroll-snap-coordinate" => StaticId::Longhand(LonghandId::ScrollSnapCoordinate),
                        "transform" => StaticId::Longhand(LonghandId::Transform),
                            "-webkit-transform" => {
                                StaticId::LonghandAlias(LonghandId::Transform,
                                                       AliasId::WebkitTransform)
                            },
                        "scroll-behavior" => StaticId::Longhand(LonghandId::ScrollBehavior),
                        "scroll-snap-type-x" => StaticId::Longhand(LonghandId::ScrollSnapTypeX),
                        "scroll-snap-type-y" => StaticId::Longhand(LonghandId::ScrollSnapTypeY),
                        "isolation" => StaticId::Longhand(LonghandId::Isolation),
                        "page-break-after" => StaticId::Longhand(LonghandId::PageBreakAfter),
                        "page-break-before" => StaticId::Longhand(LonghandId::PageBreakBefore),
                        "page-break-inside" => StaticId::Longhand(LonghandId::PageBreakInside),
                        "resize" => StaticId::Longhand(LonghandId::Resize),
                        "perspective" => StaticId::Longhand(LonghandId::Perspective),
                            "-moz-perspective" => {
                                StaticId::LonghandAlias(LonghandId::Perspective,
                                                       AliasId::MozPerspective)
                            },
                            "-webkit-perspective" => {
                                StaticId::LonghandAlias(LonghandId::Perspective,
                                                       AliasId::WebkitPerspective)
                            },
                        "perspective-origin" => StaticId::Longhand(LonghandId::PerspectiveOrigin),
                            "-moz-perspective-origin" => {
                                StaticId::LonghandAlias(LonghandId::PerspectiveOrigin,
                                                       AliasId::MozPerspectiveOrigin)
                            },
                            "-webkit-perspective-origin" => {
                                StaticId::LonghandAlias(LonghandId::PerspectiveOrigin,
                                                       AliasId::WebkitPerspectiveOrigin)
                            },
                        "backface-visibility" => StaticId::Longhand(LonghandId::BackfaceVisibility),
                            "-moz-backface-visibility" => {
                                StaticId::LonghandAlias(LonghandId::BackfaceVisibility,
                                                       AliasId::MozBackfaceVisibility)
                            },
                            "-webkit-backface-visibility" => {
                                StaticId::LonghandAlias(LonghandId::BackfaceVisibility,
                                                       AliasId::WebkitBackfaceVisibility)
                            },
                        "transform-box" => StaticId::Longhand(LonghandId::TransformBox),
                        "transform-style" => StaticId::Longhand(LonghandId::TransformStyle),
                            "-moz-transform-style" => {
                                StaticId::LonghandAlias(LonghandId::TransformStyle,
                                                       AliasId::MozTransformStyle)
                            },
                            "-webkit-transform-style" => {
                                StaticId::LonghandAlias(LonghandId::TransformStyle,
                                                       AliasId::WebkitTransformStyle)
                            },
                        "transform-origin" => StaticId::Longhand(LonghandId::TransformOrigin),
                            "-moz-transform-origin" => {
                                StaticId::LonghandAlias(LonghandId::TransformOrigin,
                                                       AliasId::MozTransformOrigin)
                            },
                            "-webkit-transform-origin" => {
                                StaticId::LonghandAlias(LonghandId::TransformOrigin,
                                                       AliasId::WebkitTransformOrigin)
                            },
                        "contain" => StaticId::Longhand(LonghandId::Contain),
                        "-moz-appearance" => StaticId::Longhand(LonghandId::MozAppearance),
                        "-moz-binding" => StaticId::Longhand(LonghandId::MozBinding),
                        "-moz-orient" => StaticId::Longhand(LonghandId::MozOrient),
                        "will-change" => StaticId::Longhand(LonghandId::WillChange),
                        "shape-outside" => StaticId::Longhand(LonghandId::ShapeOutside),
                        "touch-action" => StaticId::Longhand(LonghandId::TouchAction),
                        "color" => StaticId::Longhand(LonghandId::Color),
                        "column-width" => StaticId::Longhand(LonghandId::ColumnWidth),
                            "-moz-column-width" => {
                                StaticId::LonghandAlias(LonghandId::ColumnWidth,
                                                       AliasId::MozColumnWidth)
                            },
                        "column-count" => StaticId::Longhand(LonghandId::ColumnCount),
                            "-moz-column-count" => {
                                StaticId::LonghandAlias(LonghandId::ColumnCount,
                                                       AliasId::MozColumnCount)
                            },
                        "column-gap" => StaticId::Longhand(LonghandId::ColumnGap),
                            "-moz-column-gap" => {
                                StaticId::LonghandAlias(LonghandId::ColumnGap,
                                                       AliasId::MozColumnGap)
                            },
                        "column-fill" => StaticId::Longhand(LonghandId::ColumnFill),
                            "-moz-column-fill" => {
                                StaticId::LonghandAlias(LonghandId::ColumnFill,
                                                       AliasId::MozColumnFill)
                            },
                        "column-rule-width" => StaticId::Longhand(LonghandId::ColumnRuleWidth),
                            "-moz-column-rule-width" => {
                                StaticId::LonghandAlias(LonghandId::ColumnRuleWidth,
                                                       AliasId::MozColumnRuleWidth)
                            },
                        "column-rule-color" => StaticId::Longhand(LonghandId::ColumnRuleColor),
                            "-moz-column-rule-color" => {
                                StaticId::LonghandAlias(LonghandId::ColumnRuleColor,
                                                       AliasId::MozColumnRuleColor)
                            },
                        "column-span" => StaticId::Longhand(LonghandId::ColumnSpan),
                        "column-rule-style" => StaticId::Longhand(LonghandId::ColumnRuleStyle),
                            "-moz-column-rule-style" => {
                                StaticId::LonghandAlias(LonghandId::ColumnRuleStyle,
                                                       AliasId::MozColumnRuleStyle)
                            },
                        "content" => StaticId::Longhand(LonghandId::Content),
                        "counter-increment" => StaticId::Longhand(LonghandId::CounterIncrement),
                        "counter-reset" => StaticId::Longhand(LonghandId::CounterReset),
                        "opacity" => StaticId::Longhand(LonghandId::Opacity),
                        "box-shadow" => StaticId::Longhand(LonghandId::BoxShadow),
                            "-webkit-box-shadow" => {
                                StaticId::LonghandAlias(LonghandId::BoxShadow,
                                                       AliasId::WebkitBoxShadow)
                            },
                        "clip" => StaticId::Longhand(LonghandId::Clip),
                        "filter" => StaticId::Longhand(LonghandId::Filter),
                            "-webkit-filter" => {
                                StaticId::LonghandAlias(LonghandId::Filter,
                                                       AliasId::WebkitFilter)
                            },
                        "mix-blend-mode" => StaticId::Longhand(LonghandId::MixBlendMode),
                        "font-family" => StaticId::Longhand(LonghandId::FontFamily),
                        "font-style" => StaticId::Longhand(LonghandId::FontStyle),
                        "font-variant-caps" => StaticId::Longhand(LonghandId::FontVariantCaps),
                        "font-weight" => StaticId::Longhand(LonghandId::FontWeight),
                        "font-size" => StaticId::Longhand(LonghandId::FontSize),
                        "font-size-adjust" => StaticId::Longhand(LonghandId::FontSizeAdjust),
                        "font-synthesis" => StaticId::Longhand(LonghandId::FontSynthesis),
                        "font-stretch" => StaticId::Longhand(LonghandId::FontStretch),
                        "font-kerning" => StaticId::Longhand(LonghandId::FontKerning),
                        "font-variant-alternates" => StaticId::Longhand(LonghandId::FontVariantAlternates),
                        "font-variant-east-asian" => StaticId::Longhand(LonghandId::FontVariantEastAsian),
                        "font-variant-ligatures" => StaticId::Longhand(LonghandId::FontVariantLigatures),
                        "font-variant-numeric" => StaticId::Longhand(LonghandId::FontVariantNumeric),
                        "font-variant-position" => StaticId::Longhand(LonghandId::FontVariantPosition),
                        "font-feature-settings" => StaticId::Longhand(LonghandId::FontFeatureSettings),
                            "-moz-font-feature-settings" => {
                                StaticId::LonghandAlias(LonghandId::FontFeatureSettings,
                                                       AliasId::MozFontFeatureSettings)
                            },
                        "font-variation-settings" => StaticId::Longhand(LonghandId::FontVariationSettings),
                        "font-language-override" => StaticId::Longhand(LonghandId::FontLanguageOverride),
                            "-moz-font-language-override" => {
                                StaticId::LonghandAlias(LonghandId::FontLanguageOverride,
                                                       AliasId::MozFontLanguageOverride)
                            },
                        "-x-lang" => StaticId::Longhand(LonghandId::XLang),
                        "-moz-script-size-multiplier" => StaticId::Longhand(LonghandId::MozScriptSizeMultiplier),
                        "-moz-script-level" => StaticId::Longhand(LonghandId::MozScriptLevel),
                        "-moz-math-display" => StaticId::Longhand(LonghandId::MozMathDisplay),
                        "-moz-math-variant" => StaticId::Longhand(LonghandId::MozMathVariant),
                        "-moz-script-min-size" => StaticId::Longhand(LonghandId::MozScriptMinSize),
                        "-x-text-zoom" => StaticId::Longhand(LonghandId::XTextZoom),
                        "-moz-osx-font-smoothing" => StaticId::Longhand(LonghandId::MozOsxFontSmoothing),
                        "-moz-min-font-size-ratio" => StaticId::Longhand(LonghandId::MozMinFontSizeRatio),
                        "visibility" => StaticId::Longhand(LonghandId::Visibility),
                        "writing-mode" => StaticId::Longhand(LonghandId::WritingMode),
                        "direction" => StaticId::Longhand(LonghandId::Direction),
                        "text-orientation" => StaticId::Longhand(LonghandId::TextOrientation),
                        "color-adjust" => StaticId::Longhand(LonghandId::ColorAdjust),
                        "image-rendering" => StaticId::Longhand(LonghandId::ImageRendering),
                        "image-orientation" => StaticId::Longhand(LonghandId::ImageOrientation),
                        "border-collapse" => StaticId::Longhand(LonghandId::BorderCollapse),
                        "empty-cells" => StaticId::Longhand(LonghandId::EmptyCells),
                        "caption-side" => StaticId::Longhand(LonghandId::CaptionSide),
                        "border-spacing" => StaticId::Longhand(LonghandId::BorderSpacing),
                        "line-height" => StaticId::Longhand(LonghandId::LineHeight),
                        "text-transform" => StaticId::Longhand(LonghandId::TextTransform),
                        "hyphens" => StaticId::Longhand(LonghandId::Hyphens),
                            "-moz-hyphens" => {
                                StaticId::LonghandAlias(LonghandId::Hyphens,
                                                       AliasId::MozHyphens)
                            },
                        "-moz-text-size-adjust" => StaticId::Longhand(LonghandId::MozTextSizeAdjust),
                            "-webkit-text-size-adjust" => {
                                StaticId::LonghandAlias(LonghandId::MozTextSizeAdjust,
                                                       AliasId::WebkitTextSizeAdjust)
                            },
                        "text-indent" => StaticId::Longhand(LonghandId::TextIndent),
                        "overflow-wrap" => StaticId::Longhand(LonghandId::OverflowWrap),
                            "word-wrap" => {
                                StaticId::LonghandAlias(LonghandId::OverflowWrap,
                                                       AliasId::WordWrap)
                            },
                        "word-break" => StaticId::Longhand(LonghandId::WordBreak),
                        "text-justify" => StaticId::Longhand(LonghandId::TextJustify),
                        "text-align-last" => StaticId::Longhand(LonghandId::TextAlignLast),
                        "text-align" => StaticId::Longhand(LonghandId::TextAlign),
                        "letter-spacing" => StaticId::Longhand(LonghandId::LetterSpacing),
                        "word-spacing" => StaticId::Longhand(LonghandId::WordSpacing),
                        "white-space" => StaticId::Longhand(LonghandId::WhiteSpace),
                        "text-shadow" => StaticId::Longhand(LonghandId::TextShadow),
                        "text-emphasis-style" => StaticId::Longhand(LonghandId::TextEmphasisStyle),
                        "text-emphasis-position" => StaticId::Longhand(LonghandId::TextEmphasisPosition),
                        "text-emphasis-color" => StaticId::Longhand(LonghandId::TextEmphasisColor),
                        "-moz-tab-size" => StaticId::Longhand(LonghandId::MozTabSize),
                        "-webkit-text-fill-color" => StaticId::Longhand(LonghandId::WebkitTextFillColor),
                        "-webkit-text-stroke-color" => StaticId::Longhand(LonghandId::WebkitTextStrokeColor),
                        "-webkit-text-stroke-width" => StaticId::Longhand(LonghandId::WebkitTextStrokeWidth),
                        "ruby-align" => StaticId::Longhand(LonghandId::RubyAlign),
                        "ruby-position" => StaticId::Longhand(LonghandId::RubyPosition),
                        "text-combine-upright" => StaticId::Longhand(LonghandId::TextCombineUpright),
                        "text-rendering" => StaticId::Longhand(LonghandId::TextRendering),
                        "-moz-control-character-visibility" => StaticId::Longhand(LonghandId::MozControlCharacterVisibility),
                        "list-style-position" => StaticId::Longhand(LonghandId::ListStylePosition),
                        "list-style-type" => StaticId::Longhand(LonghandId::ListStyleType),
                        "list-style-image" => StaticId::Longhand(LonghandId::ListStyleImage),
                        "quotes" => StaticId::Longhand(LonghandId::Quotes),
                        "-moz-image-region" => StaticId::Longhand(LonghandId::MozImageRegion),
                        "margin-top" => StaticId::Longhand(LonghandId::MarginTop),
                        "margin-left" => StaticId::Longhand(LonghandId::MarginLeft),
                        "margin-bottom" => StaticId::Longhand(LonghandId::MarginBottom),
                        "margin-right" => StaticId::Longhand(LonghandId::MarginRight),
                        "margin-block-start" => StaticId::Longhand(LonghandId::MarginBlockStart),
                        "margin-block-end" => StaticId::Longhand(LonghandId::MarginBlockEnd),
                        "margin-inline-start" => StaticId::Longhand(LonghandId::MarginInlineStart),
                            "-moz-margin-start" => {
                                StaticId::LonghandAlias(LonghandId::MarginInlineStart,
                                                       AliasId::MozMarginStart)
                            },
                        "margin-inline-end" => StaticId::Longhand(LonghandId::MarginInlineEnd),
                            "-moz-margin-end" => {
                                StaticId::LonghandAlias(LonghandId::MarginInlineEnd,
                                                       AliasId::MozMarginEnd)
                            },
                        "outline-color" => StaticId::Longhand(LonghandId::OutlineColor),
                        "outline-style" => StaticId::Longhand(LonghandId::OutlineStyle),
                        "outline-width" => StaticId::Longhand(LonghandId::OutlineWidth),
                        "-moz-outline-radius-topleft" => StaticId::Longhand(LonghandId::MozOutlineRadiusTopleft),
                        "-moz-outline-radius-topright" => StaticId::Longhand(LonghandId::MozOutlineRadiusTopright),
                        "-moz-outline-radius-bottomright" => StaticId::Longhand(LonghandId::MozOutlineRadiusBottomright),
                        "-moz-outline-radius-bottomleft" => StaticId::Longhand(LonghandId::MozOutlineRadiusBottomleft),
                        "outline-offset" => StaticId::Longhand(LonghandId::OutlineOffset),
                        "padding-top" => StaticId::Longhand(LonghandId::PaddingTop),
                        "padding-left" => StaticId::Longhand(LonghandId::PaddingLeft),
                        "padding-bottom" => StaticId::Longhand(LonghandId::PaddingBottom),
                        "padding-right" => StaticId::Longhand(LonghandId::PaddingRight),
                        "padding-block-start" => StaticId::Longhand(LonghandId::PaddingBlockStart),
                        "padding-block-end" => StaticId::Longhand(LonghandId::PaddingBlockEnd),
                        "padding-inline-start" => StaticId::Longhand(LonghandId::PaddingInlineStart),
                            "-moz-padding-start" => {
                                StaticId::LonghandAlias(LonghandId::PaddingInlineStart,
                                                       AliasId::MozPaddingStart)
                            },
                        "padding-inline-end" => StaticId::Longhand(LonghandId::PaddingInlineEnd),
                            "-moz-padding-end" => {
                                StaticId::LonghandAlias(LonghandId::PaddingInlineEnd,
                                                       AliasId::MozPaddingEnd)
                            },
                        "cursor" => StaticId::Longhand(LonghandId::Cursor),
                        "pointer-events" => StaticId::Longhand(LonghandId::PointerEvents),
                        "-moz-user-input" => StaticId::Longhand(LonghandId::MozUserInput),
                        "-moz-user-modify" => StaticId::Longhand(LonghandId::MozUserModify),
                        "-moz-user-focus" => StaticId::Longhand(LonghandId::MozUserFocus),
                        "caret-color" => StaticId::Longhand(LonghandId::CaretColor),
                        "top" => StaticId::Longhand(LonghandId::Top),
                        "left" => StaticId::Longhand(LonghandId::Left),
                        "bottom" => StaticId::Longhand(LonghandId::Bottom),
                        "right" => StaticId::Longhand(LonghandId::Right),
                        "offset-block-start" => StaticId::Longhand(LonghandId::OffsetBlockStart),
                        "offset-block-end" => StaticId::Longhand(LonghandId::OffsetBlockEnd),
                        "offset-inline-start" => StaticId::Longhand(LonghandId::OffsetInlineStart),
                        "offset-inline-end" => StaticId::Longhand(LonghandId::OffsetInlineEnd),
                        "z-index" => StaticId::Longhand(LonghandId::ZIndex),
                        "flex-direction" => StaticId::Longhand(LonghandId::FlexDirection),
                            "-webkit-flex-direction" => {
                                StaticId::LonghandAlias(LonghandId::FlexDirection,
                                                       AliasId::WebkitFlexDirection)
                            },
                        "flex-wrap" => StaticId::Longhand(LonghandId::FlexWrap),
                            "-webkit-flex-wrap" => {
                                StaticId::LonghandAlias(LonghandId::FlexWrap,
                                                       AliasId::WebkitFlexWrap)
                            },
                        "justify-content" => StaticId::Longhand(LonghandId::JustifyContent),
                            "-webkit-justify-content" => {
                                StaticId::LonghandAlias(LonghandId::JustifyContent,
                                                       AliasId::WebkitJustifyContent)
                            },
                        "align-content" => StaticId::Longhand(LonghandId::AlignContent),
                            "-webkit-align-content" => {
                                StaticId::LonghandAlias(LonghandId::AlignContent,
                                                       AliasId::WebkitAlignContent)
                            },
                        "align-items" => StaticId::Longhand(LonghandId::AlignItems),
                            "-webkit-align-items" => {
                                StaticId::LonghandAlias(LonghandId::AlignItems,
                                                       AliasId::WebkitAlignItems)
                            },
                        "justify-items" => StaticId::Longhand(LonghandId::JustifyItems),
                        "flex-grow" => StaticId::Longhand(LonghandId::FlexGrow),
                            "-webkit-flex-grow" => {
                                StaticId::LonghandAlias(LonghandId::FlexGrow,
                                                       AliasId::WebkitFlexGrow)
                            },
                        "flex-shrink" => StaticId::Longhand(LonghandId::FlexShrink),
                            "-webkit-flex-shrink" => {
                                StaticId::LonghandAlias(LonghandId::FlexShrink,
                                                       AliasId::WebkitFlexShrink)
                            },
                        "align-self" => StaticId::Longhand(LonghandId::AlignSelf),
                            "-webkit-align-self" => {
                                StaticId::LonghandAlias(LonghandId::AlignSelf,
                                                       AliasId::WebkitAlignSelf)
                            },
                        "justify-self" => StaticId::Longhand(LonghandId::JustifySelf),
                        "order" => StaticId::Longhand(LonghandId::Order),
                            "-webkit-order" => {
                                StaticId::LonghandAlias(LonghandId::Order,
                                                       AliasId::WebkitOrder)
                            },
                        "flex-basis" => StaticId::Longhand(LonghandId::FlexBasis),
                            "-webkit-flex-basis" => {
                                StaticId::LonghandAlias(LonghandId::FlexBasis,
                                                       AliasId::WebkitFlexBasis)
                            },
                        "width" => StaticId::Longhand(LonghandId::Width),
                        "min-width" => StaticId::Longhand(LonghandId::MinWidth),
                        "max-width" => StaticId::Longhand(LonghandId::MaxWidth),
                        "height" => StaticId::Longhand(LonghandId::Height),
                        "min-height" => StaticId::Longhand(LonghandId::MinHeight),
                        "max-height" => StaticId::Longhand(LonghandId::MaxHeight),
                        "block-size" => StaticId::Longhand(LonghandId::BlockSize),
                        "min-block-size" => StaticId::Longhand(LonghandId::MinBlockSize),
                        "max-block-size" => StaticId::Longhand(LonghandId::MaxBlockSize),
                        "inline-size" => StaticId::Longhand(LonghandId::InlineSize),
                        "min-inline-size" => StaticId::Longhand(LonghandId::MinInlineSize),
                        "max-inline-size" => StaticId::Longhand(LonghandId::MaxInlineSize),
                        "box-sizing" => StaticId::Longhand(LonghandId::BoxSizing),
                            "-moz-box-sizing" => {
                                StaticId::LonghandAlias(LonghandId::BoxSizing,
                                                       AliasId::MozBoxSizing)
                            },
                            "-webkit-box-sizing" => {
                                StaticId::LonghandAlias(LonghandId::BoxSizing,
                                                       AliasId::WebkitBoxSizing)
                            },
                        "object-fit" => StaticId::Longhand(LonghandId::ObjectFit),
                        "object-position" => StaticId::Longhand(LonghandId::ObjectPosition),
                        "grid-row-gap" => StaticId::Longhand(LonghandId::GridRowGap),
                        "grid-row-start" => StaticId::Longhand(LonghandId::GridRowStart),
                        "grid-row-end" => StaticId::Longhand(LonghandId::GridRowEnd),
                        "grid-auto-rows" => StaticId::Longhand(LonghandId::GridAutoRows),
                        "grid-template-rows" => StaticId::Longhand(LonghandId::GridTemplateRows),
                        "grid-column-gap" => StaticId::Longhand(LonghandId::GridColumnGap),
                        "grid-column-start" => StaticId::Longhand(LonghandId::GridColumnStart),
                        "grid-column-end" => StaticId::Longhand(LonghandId::GridColumnEnd),
                        "grid-auto-columns" => StaticId::Longhand(LonghandId::GridAutoColumns),
                        "grid-template-columns" => StaticId::Longhand(LonghandId::GridTemplateColumns),
                        "grid-auto-flow" => StaticId::Longhand(LonghandId::GridAutoFlow),
                        "grid-template-areas" => StaticId::Longhand(LonghandId::GridTemplateAreas),
                        "table-layout" => StaticId::Longhand(LonghandId::TableLayout),
                        "-x-span" => StaticId::Longhand(LonghandId::XSpan),
                        "text-overflow" => StaticId::Longhand(LonghandId::TextOverflow),
                        "unicode-bidi" => StaticId::Longhand(LonghandId::UnicodeBidi),
                        "text-decoration-line" => StaticId::Longhand(LonghandId::TextDecorationLine),
                        "text-decoration-style" => StaticId::Longhand(LonghandId::TextDecorationStyle),
                        "text-decoration-color" => StaticId::Longhand(LonghandId::TextDecorationColor),
                        "initial-letter" => StaticId::Longhand(LonghandId::InitialLetter),
                        "ime-mode" => StaticId::Longhand(LonghandId::ImeMode),
                        "-moz-user-select" => StaticId::Longhand(LonghandId::MozUserSelect),
                            "-webkit-user-select" => {
                                StaticId::LonghandAlias(LonghandId::MozUserSelect,
                                                       AliasId::WebkitUserSelect)
                            },
                        "-moz-window-dragging" => StaticId::Longhand(LonghandId::MozWindowDragging),
                        "-moz-window-shadow" => StaticId::Longhand(LonghandId::MozWindowShadow),
                        "-moz-force-broken-image-icon" => StaticId::Longhand(LonghandId::MozForceBrokenImageIcon),
                        "text-anchor" => StaticId::Longhand(LonghandId::TextAnchor),
                        "color-interpolation" => StaticId::Longhand(LonghandId::ColorInterpolation),
                        "color-interpolation-filters" => StaticId::Longhand(LonghandId::ColorInterpolationFilters),
                        "fill" => StaticId::Longhand(LonghandId::Fill),
                        "fill-opacity" => StaticId::Longhand(LonghandId::FillOpacity),
                        "fill-rule" => StaticId::Longhand(LonghandId::FillRule),
                        "shape-rendering" => StaticId::Longhand(LonghandId::ShapeRendering),
                        "stroke" => StaticId::Longhand(LonghandId::Stroke),
                        "stroke-width" => StaticId::Longhand(LonghandId::StrokeWidth),
                        "stroke-linecap" => StaticId::Longhand(LonghandId::StrokeLinecap),
                        "stroke-linejoin" => StaticId::Longhand(LonghandId::StrokeLinejoin),
                        "stroke-miterlimit" => StaticId::Longhand(LonghandId::StrokeMiterlimit),
                        "stroke-opacity" => StaticId::Longhand(LonghandId::StrokeOpacity),
                        "stroke-dasharray" => StaticId::Longhand(LonghandId::StrokeDasharray),
                        "stroke-dashoffset" => StaticId::Longhand(LonghandId::StrokeDashoffset),
                        "clip-rule" => StaticId::Longhand(LonghandId::ClipRule),
                        "marker-start" => StaticId::Longhand(LonghandId::MarkerStart),
                        "marker-mid" => StaticId::Longhand(LonghandId::MarkerMid),
                        "marker-end" => StaticId::Longhand(LonghandId::MarkerEnd),
                        "paint-order" => StaticId::Longhand(LonghandId::PaintOrder),
                        "-moz-context-properties" => StaticId::Longhand(LonghandId::MozContextProperties),
                        "dominant-baseline" => StaticId::Longhand(LonghandId::DominantBaseline),
                        "vector-effect" => StaticId::Longhand(LonghandId::VectorEffect),
                        "stop-color" => StaticId::Longhand(LonghandId::StopColor),
                        "stop-opacity" => StaticId::Longhand(LonghandId::StopOpacity),
                        "flood-color" => StaticId::Longhand(LonghandId::FloodColor),
                        "flood-opacity" => StaticId::Longhand(LonghandId::FloodOpacity),
                        "lighting-color" => StaticId::Longhand(LonghandId::LightingColor),
                        "mask-type" => StaticId::Longhand(LonghandId::MaskType),
                        "clip-path" => StaticId::Longhand(LonghandId::ClipPath),
                        "mask-mode" => StaticId::Longhand(LonghandId::MaskMode),
                        "mask-repeat" => StaticId::Longhand(LonghandId::MaskRepeat),
                            "-webkit-mask-repeat" => {
                                StaticId::LonghandAlias(LonghandId::MaskRepeat,
                                                       AliasId::WebkitMaskRepeat)
                            },
                        "mask-position-x" => StaticId::Longhand(LonghandId::MaskPositionX),
                            "-webkit-mask-position-x" => {
                                StaticId::LonghandAlias(LonghandId::MaskPositionX,
                                                       AliasId::WebkitMaskPositionX)
                            },
                        "mask-position-y" => StaticId::Longhand(LonghandId::MaskPositionY),
                            "-webkit-mask-position-y" => {
                                StaticId::LonghandAlias(LonghandId::MaskPositionY,
                                                       AliasId::WebkitMaskPositionY)
                            },
                        "mask-clip" => StaticId::Longhand(LonghandId::MaskClip),
                            "-webkit-mask-clip" => {
                                StaticId::LonghandAlias(LonghandId::MaskClip,
                                                       AliasId::WebkitMaskClip)
                            },
                        "mask-origin" => StaticId::Longhand(LonghandId::MaskOrigin),
                            "-webkit-mask-origin" => {
                                StaticId::LonghandAlias(LonghandId::MaskOrigin,
                                                       AliasId::WebkitMaskOrigin)
                            },
                        "mask-size" => StaticId::Longhand(LonghandId::MaskSize),
                            "-webkit-mask-size" => {
                                StaticId::LonghandAlias(LonghandId::MaskSize,
                                                       AliasId::WebkitMaskSize)
                            },
                        "mask-composite" => StaticId::Longhand(LonghandId::MaskComposite),
                            "-webkit-mask-composite" => {
                                StaticId::LonghandAlias(LonghandId::MaskComposite,
                                                       AliasId::WebkitMaskComposite)
                            },
                        "mask-image" => StaticId::Longhand(LonghandId::MaskImage),
                            "-webkit-mask-image" => {
                                StaticId::LonghandAlias(LonghandId::MaskImage,
                                                       AliasId::WebkitMaskImage)
                            },
                        "-moz-box-align" => StaticId::Longhand(LonghandId::MozBoxAlign),
                            "-webkit-box-align" => {
                                StaticId::LonghandAlias(LonghandId::MozBoxAlign,
                                                       AliasId::WebkitBoxAlign)
                            },
                        "-moz-box-direction" => StaticId::Longhand(LonghandId::MozBoxDirection),
                            "-webkit-box-direction" => {
                                StaticId::LonghandAlias(LonghandId::MozBoxDirection,
                                                       AliasId::WebkitBoxDirection)
                            },
                        "-moz-box-flex" => StaticId::Longhand(LonghandId::MozBoxFlex),
                            "-webkit-box-flex" => {
                                StaticId::LonghandAlias(LonghandId::MozBoxFlex,
                                                       AliasId::WebkitBoxFlex)
                            },
                        "-moz-box-orient" => StaticId::Longhand(LonghandId::MozBoxOrient),
                            "-webkit-box-orient" => {
                                StaticId::LonghandAlias(LonghandId::MozBoxOrient,
                                                       AliasId::WebkitBoxOrient)
                            },
                        "-moz-box-pack" => StaticId::Longhand(LonghandId::MozBoxPack),
                            "-webkit-box-pack" => {
                                StaticId::LonghandAlias(LonghandId::MozBoxPack,
                                                       AliasId::WebkitBoxPack)
                            },
                        "-moz-stack-sizing" => StaticId::Longhand(LonghandId::MozStackSizing),
                        "-moz-box-ordinal-group" => StaticId::Longhand(LonghandId::MozBoxOrdinalGroup),
                            "-webkit-box-ordinal-group" => {
                                StaticId::LonghandAlias(LonghandId::MozBoxOrdinalGroup,
                                                       AliasId::WebkitBoxOrdinalGroup)
                            },
                        "background" => StaticId::Shorthand(ShorthandId::Background),
                        "background-position" => StaticId::Shorthand(ShorthandId::BackgroundPosition),
                        "border-color" => StaticId::Shorthand(ShorthandId::BorderColor),
                        "border-style" => StaticId::Shorthand(ShorthandId::BorderStyle),
                        "border-width" => StaticId::Shorthand(ShorthandId::BorderWidth),
                        "border-top" => StaticId::Shorthand(ShorthandId::BorderTop),
                        "border-left" => StaticId::Shorthand(ShorthandId::BorderLeft),
                        "border-bottom" => StaticId::Shorthand(ShorthandId::BorderBottom),
                        "border-right" => StaticId::Shorthand(ShorthandId::BorderRight),
                        "border-block-start" => StaticId::Shorthand(ShorthandId::BorderBlockStart),
                        "border-block-end" => StaticId::Shorthand(ShorthandId::BorderBlockEnd),
                        "border-inline-start" => StaticId::Shorthand(ShorthandId::BorderInlineStart),
                            "-moz-border-start" => {
                                StaticId::ShorthandAlias(ShorthandId::BorderInlineStart,
                                                       AliasId::MozBorderStart)
                            },
                        "border-inline-end" => StaticId::Shorthand(ShorthandId::BorderInlineEnd),
                            "-moz-border-end" => {
                                StaticId::ShorthandAlias(ShorthandId::BorderInlineEnd,
                                                       AliasId::MozBorderEnd)
                            },
                        "border" => StaticId::Shorthand(ShorthandId::Border),
                        "border-radius" => StaticId::Shorthand(ShorthandId::BorderRadius),
                            "-webkit-border-radius" => {
                                StaticId::ShorthandAlias(ShorthandId::BorderRadius,
                                                       AliasId::WebkitBorderRadius)
                            },
                        "border-image" => StaticId::Shorthand(ShorthandId::BorderImage),
                            "-moz-border-image" => {
                                StaticId::ShorthandAlias(ShorthandId::BorderImage,
                                                       AliasId::MozBorderImage)
                            },
                            "-webkit-border-image" => {
                                StaticId::ShorthandAlias(ShorthandId::BorderImage,
                                                       AliasId::WebkitBorderImage)
                            },
                        "overflow" => StaticId::Shorthand(ShorthandId::Overflow),
                        "transition" => StaticId::Shorthand(ShorthandId::Transition),
                            "-moz-transition" => {
                                StaticId::ShorthandAlias(ShorthandId::Transition,
                                                       AliasId::MozTransition)
                            },
                            "-webkit-transition" => {
                                StaticId::ShorthandAlias(ShorthandId::Transition,
                                                       AliasId::WebkitTransition)
                            },
                        "animation" => StaticId::Shorthand(ShorthandId::Animation),
                            "-moz-animation" => {
                                StaticId::ShorthandAlias(ShorthandId::Animation,
                                                       AliasId::MozAnimation)
                            },
                            "-webkit-animation" => {
                                StaticId::ShorthandAlias(ShorthandId::Animation,
                                                       AliasId::WebkitAnimation)
                            },
                        "scroll-snap-type" => StaticId::Shorthand(ShorthandId::ScrollSnapType),
                        "-moz-transform" => StaticId::Shorthand(ShorthandId::MozTransform),
                        "columns" => StaticId::Shorthand(ShorthandId::Columns),
                            "-moz-columns" => {
                                StaticId::ShorthandAlias(ShorthandId::Columns,
                                                       AliasId::MozColumns)
                            },
                        "column-rule" => StaticId::Shorthand(ShorthandId::ColumnRule),
                            "-moz-column-rule" => {
                                StaticId::ShorthandAlias(ShorthandId::ColumnRule,
                                                       AliasId::MozColumnRule)
                            },
                        "font" => StaticId::Shorthand(ShorthandId::Font),
                        "font-variant" => StaticId::Shorthand(ShorthandId::FontVariant),
                        "text-emphasis" => StaticId::Shorthand(ShorthandId::TextEmphasis),
                        "-webkit-text-stroke" => StaticId::Shorthand(ShorthandId::WebkitTextStroke),
                        "list-style" => StaticId::Shorthand(ShorthandId::ListStyle),
                        "margin" => StaticId::Shorthand(ShorthandId::Margin),
                        "mask" => StaticId::Shorthand(ShorthandId::Mask),
                            "-webkit-mask" => {
                                StaticId::ShorthandAlias(ShorthandId::Mask,
                                                       AliasId::WebkitMask)
                            },
                        "mask-position" => StaticId::Shorthand(ShorthandId::MaskPosition),
                            "-webkit-mask-position" => {
                                StaticId::ShorthandAlias(ShorthandId::MaskPosition,
                                                       AliasId::WebkitMaskPosition)
                            },
                        "outline" => StaticId::Shorthand(ShorthandId::Outline),
                        "-moz-outline-radius" => StaticId::Shorthand(ShorthandId::MozOutlineRadius),
                        "padding" => StaticId::Shorthand(ShorthandId::Padding),
                        "flex-flow" => StaticId::Shorthand(ShorthandId::FlexFlow),
                            "-webkit-flex-flow" => {
                                StaticId::ShorthandAlias(ShorthandId::FlexFlow,
                                                       AliasId::WebkitFlexFlow)
                            },
                        "flex" => StaticId::Shorthand(ShorthandId::Flex),
                            "-webkit-flex" => {
                                StaticId::ShorthandAlias(ShorthandId::Flex,
                                                       AliasId::WebkitFlex)
                            },
                        "grid-gap" => StaticId::Shorthand(ShorthandId::GridGap),
                        "grid-row" => StaticId::Shorthand(ShorthandId::GridRow),
                        "grid-column" => StaticId::Shorthand(ShorthandId::GridColumn),
                        "grid-area" => StaticId::Shorthand(ShorthandId::GridArea),
                        "grid-template" => StaticId::Shorthand(ShorthandId::GridTemplate),
                        "grid" => StaticId::Shorthand(ShorthandId::Grid),
                        "place-content" => StaticId::Shorthand(ShorthandId::PlaceContent),
                        "place-self" => StaticId::Shorthand(ShorthandId::PlaceSelf),
                        "place-items" => StaticId::Shorthand(ShorthandId::PlaceItems),
                        "marker" => StaticId::Shorthand(ShorthandId::Marker),
                        "text-decoration" => StaticId::Shorthand(ShorthandId::TextDecoration),
                        "all" => StaticId::Shorthand(ShorthandId::All),
            }
        }

        let default;
        let context = match context {
            Some(context) => context,
            None => {
                default = PropertyParserContext {
                    stylesheet_origin: Origin::Author,
                    rule_type: CssRuleType::Style,
                };
                &default
            }
        };
        let rule_type = context.rule_type;
        debug_assert!(matches!(rule_type, CssRuleType::Keyframe |
                                          CssRuleType::Page |
                                          CssRuleType::Style),
                      "Declarations are only expected inside a keyframe, page, or style rule.");

        let (id, alias) = match static_id(property_name) {
            Some(&StaticId::Longhand(id)) => {
                (PropertyId::Longhand(id), None)
            },
            Some(&StaticId::Shorthand(id)) => {
                (PropertyId::Shorthand(id), None)
            },
            Some(&StaticId::LonghandAlias(id, alias)) => {
                (PropertyId::Longhand(id), Some(alias))
            },
            Some(&StaticId::ShorthandAlias(id, alias)) => {
                (PropertyId::Shorthand(id), Some(alias))
            },
            None => return ::custom_properties::parse_name(property_name)
                .map(|name| PropertyId::Custom(::custom_properties::Name::from(name))),
        };
        id.check_allowed_in(alias, context)?;
        Ok(id)
    }

    /// Returns a property id from Gecko's nsCSSPropertyID.
    #[cfg(feature = "gecko")]
    #[allow(non_upper_case_globals)]
    pub fn from_nscsspropertyid(id: nsCSSPropertyID) -> Result<Self, ()> {
        use gecko_bindings::structs::*;
        match id {
                
    nsCSSPropertyID::eCSSProperty_background_color => {
                    Ok(PropertyId::Longhand(LonghandId::BackgroundColor))
                }
                
    nsCSSPropertyID::eCSSProperty_background_image => {
                    Ok(PropertyId::Longhand(LonghandId::BackgroundImage))
                }
                
    nsCSSPropertyID::eCSSProperty_background_position_x => {
                    Ok(PropertyId::Longhand(LonghandId::BackgroundPositionX))
                }
                
    nsCSSPropertyID::eCSSProperty_background_position_y => {
                    Ok(PropertyId::Longhand(LonghandId::BackgroundPositionY))
                }
                
    nsCSSPropertyID::eCSSProperty_background_repeat => {
                    Ok(PropertyId::Longhand(LonghandId::BackgroundRepeat))
                }
                
    nsCSSPropertyID::eCSSProperty_background_attachment => {
                    Ok(PropertyId::Longhand(LonghandId::BackgroundAttachment))
                }
                
    nsCSSPropertyID::eCSSProperty_background_clip => {
                    Ok(PropertyId::Longhand(LonghandId::BackgroundClip))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_WebkitBackgroundClip => {
                        Ok(PropertyId::Longhand(LonghandId::BackgroundClip))
                    }
                
    nsCSSPropertyID::eCSSProperty_background_origin => {
                    Ok(PropertyId::Longhand(LonghandId::BackgroundOrigin))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_WebkitBackgroundOrigin => {
                        Ok(PropertyId::Longhand(LonghandId::BackgroundOrigin))
                    }
                
    nsCSSPropertyID::eCSSProperty_background_size => {
                    Ok(PropertyId::Longhand(LonghandId::BackgroundSize))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_WebkitBackgroundSize => {
                        Ok(PropertyId::Longhand(LonghandId::BackgroundSize))
                    }
                
    nsCSSPropertyID::eCSSProperty_background_blend_mode => {
                    Ok(PropertyId::Longhand(LonghandId::BackgroundBlendMode))
                }
                
    nsCSSPropertyID::eCSSProperty_border_top_color => {
                    Ok(PropertyId::Longhand(LonghandId::BorderTopColor))
                }
                
    nsCSSPropertyID::eCSSProperty_border_top_style => {
                    Ok(PropertyId::Longhand(LonghandId::BorderTopStyle))
                }
                
    nsCSSPropertyID::eCSSProperty_border_top_width => {
                    Ok(PropertyId::Longhand(LonghandId::BorderTopWidth))
                }
                
    nsCSSPropertyID::eCSSProperty_border_left_color => {
                    Ok(PropertyId::Longhand(LonghandId::BorderLeftColor))
                }
                
    nsCSSPropertyID::eCSSProperty_border_left_style => {
                    Ok(PropertyId::Longhand(LonghandId::BorderLeftStyle))
                }
                
    nsCSSPropertyID::eCSSProperty_border_left_width => {
                    Ok(PropertyId::Longhand(LonghandId::BorderLeftWidth))
                }
                
    nsCSSPropertyID::eCSSProperty_border_bottom_color => {
                    Ok(PropertyId::Longhand(LonghandId::BorderBottomColor))
                }
                
    nsCSSPropertyID::eCSSProperty_border_bottom_style => {
                    Ok(PropertyId::Longhand(LonghandId::BorderBottomStyle))
                }
                
    nsCSSPropertyID::eCSSProperty_border_bottom_width => {
                    Ok(PropertyId::Longhand(LonghandId::BorderBottomWidth))
                }
                
    nsCSSPropertyID::eCSSProperty_border_right_color => {
                    Ok(PropertyId::Longhand(LonghandId::BorderRightColor))
                }
                
    nsCSSPropertyID::eCSSProperty_border_right_style => {
                    Ok(PropertyId::Longhand(LonghandId::BorderRightStyle))
                }
                
    nsCSSPropertyID::eCSSProperty_border_right_width => {
                    Ok(PropertyId::Longhand(LonghandId::BorderRightWidth))
                }
                
    nsCSSPropertyID::eCSSProperty_border_block_start_color => {
                    Ok(PropertyId::Longhand(LonghandId::BorderBlockStartColor))
                }
                
    nsCSSPropertyID::eCSSProperty_border_block_start_style => {
                    Ok(PropertyId::Longhand(LonghandId::BorderBlockStartStyle))
                }
                
    nsCSSPropertyID::eCSSProperty_border_block_start_width => {
                    Ok(PropertyId::Longhand(LonghandId::BorderBlockStartWidth))
                }
                
    nsCSSPropertyID::eCSSProperty_border_block_end_color => {
                    Ok(PropertyId::Longhand(LonghandId::BorderBlockEndColor))
                }
                
    nsCSSPropertyID::eCSSProperty_border_block_end_style => {
                    Ok(PropertyId::Longhand(LonghandId::BorderBlockEndStyle))
                }
                
    nsCSSPropertyID::eCSSProperty_border_block_end_width => {
                    Ok(PropertyId::Longhand(LonghandId::BorderBlockEndWidth))
                }
                
    nsCSSPropertyID::eCSSProperty_border_inline_start_color => {
                    Ok(PropertyId::Longhand(LonghandId::BorderInlineStartColor))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_MozBorderStartColor => {
                        Ok(PropertyId::Longhand(LonghandId::BorderInlineStartColor))
                    }
                
    nsCSSPropertyID::eCSSProperty_border_inline_start_style => {
                    Ok(PropertyId::Longhand(LonghandId::BorderInlineStartStyle))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_MozBorderStartStyle => {
                        Ok(PropertyId::Longhand(LonghandId::BorderInlineStartStyle))
                    }
                
    nsCSSPropertyID::eCSSProperty_border_inline_start_width => {
                    Ok(PropertyId::Longhand(LonghandId::BorderInlineStartWidth))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_MozBorderStartWidth => {
                        Ok(PropertyId::Longhand(LonghandId::BorderInlineStartWidth))
                    }
                
    nsCSSPropertyID::eCSSProperty_border_inline_end_color => {
                    Ok(PropertyId::Longhand(LonghandId::BorderInlineEndColor))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_MozBorderEndColor => {
                        Ok(PropertyId::Longhand(LonghandId::BorderInlineEndColor))
                    }
                
    nsCSSPropertyID::eCSSProperty_border_inline_end_style => {
                    Ok(PropertyId::Longhand(LonghandId::BorderInlineEndStyle))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_MozBorderEndStyle => {
                        Ok(PropertyId::Longhand(LonghandId::BorderInlineEndStyle))
                    }
                
    nsCSSPropertyID::eCSSProperty_border_inline_end_width => {
                    Ok(PropertyId::Longhand(LonghandId::BorderInlineEndWidth))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_MozBorderEndWidth => {
                        Ok(PropertyId::Longhand(LonghandId::BorderInlineEndWidth))
                    }
                
    nsCSSPropertyID::eCSSProperty_border_top_left_radius => {
                    Ok(PropertyId::Longhand(LonghandId::BorderTopLeftRadius))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_WebkitBorderTopLeftRadius => {
                        Ok(PropertyId::Longhand(LonghandId::BorderTopLeftRadius))
                    }
                
    nsCSSPropertyID::eCSSProperty_border_top_right_radius => {
                    Ok(PropertyId::Longhand(LonghandId::BorderTopRightRadius))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_WebkitBorderTopRightRadius => {
                        Ok(PropertyId::Longhand(LonghandId::BorderTopRightRadius))
                    }
                
    nsCSSPropertyID::eCSSProperty_border_bottom_right_radius => {
                    Ok(PropertyId::Longhand(LonghandId::BorderBottomRightRadius))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_WebkitBorderBottomRightRadius => {
                        Ok(PropertyId::Longhand(LonghandId::BorderBottomRightRadius))
                    }
                
    nsCSSPropertyID::eCSSProperty_border_bottom_left_radius => {
                    Ok(PropertyId::Longhand(LonghandId::BorderBottomLeftRadius))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_WebkitBorderBottomLeftRadius => {
                        Ok(PropertyId::Longhand(LonghandId::BorderBottomLeftRadius))
                    }
                
    nsCSSPropertyID::eCSSProperty__moz_border_top_colors => {
                    Ok(PropertyId::Longhand(LonghandId::MozBorderTopColors))
                }
                
    nsCSSPropertyID::eCSSProperty__moz_border_left_colors => {
                    Ok(PropertyId::Longhand(LonghandId::MozBorderLeftColors))
                }
                
    nsCSSPropertyID::eCSSProperty__moz_border_bottom_colors => {
                    Ok(PropertyId::Longhand(LonghandId::MozBorderBottomColors))
                }
                
    nsCSSPropertyID::eCSSProperty__moz_border_right_colors => {
                    Ok(PropertyId::Longhand(LonghandId::MozBorderRightColors))
                }
                
    nsCSSPropertyID::eCSSProperty_box_decoration_break => {
                    Ok(PropertyId::Longhand(LonghandId::BoxDecorationBreak))
                }
                
    nsCSSPropertyID::eCSSProperty__moz_float_edge => {
                    Ok(PropertyId::Longhand(LonghandId::MozFloatEdge))
                }
                
    nsCSSPropertyID::eCSSProperty_border_image_source => {
                    Ok(PropertyId::Longhand(LonghandId::BorderImageSource))
                }
                
    nsCSSPropertyID::eCSSProperty_border_image_outset => {
                    Ok(PropertyId::Longhand(LonghandId::BorderImageOutset))
                }
                
    nsCSSPropertyID::eCSSProperty_border_image_repeat => {
                    Ok(PropertyId::Longhand(LonghandId::BorderImageRepeat))
                }
                
    nsCSSPropertyID::eCSSProperty_border_image_width => {
                    Ok(PropertyId::Longhand(LonghandId::BorderImageWidth))
                }
                
    nsCSSPropertyID::eCSSProperty_border_image_slice => {
                    Ok(PropertyId::Longhand(LonghandId::BorderImageSlice))
                }
                
    nsCSSPropertyID::eCSSProperty_display => {
                    Ok(PropertyId::Longhand(LonghandId::Display))
                }
                
    nsCSSPropertyID::eCSSProperty__moz_top_layer => {
                    Ok(PropertyId::Longhand(LonghandId::MozTopLayer))
                }
                
    nsCSSPropertyID::eCSSProperty_position => {
                    Ok(PropertyId::Longhand(LonghandId::Position))
                }
                
    nsCSSPropertyID::eCSSProperty_float_ => {
                    Ok(PropertyId::Longhand(LonghandId::Float))
                }
                
    nsCSSPropertyID::eCSSProperty_clear => {
                    Ok(PropertyId::Longhand(LonghandId::Clear))
                }
                
    nsCSSPropertyID::eCSSProperty_vertical_align => {
                    Ok(PropertyId::Longhand(LonghandId::VerticalAlign))
                }
                
    nsCSSPropertyID::eCSSProperty_overflow_clip_box => {
                    Ok(PropertyId::Longhand(LonghandId::OverflowClipBox))
                }
                
    nsCSSPropertyID::eCSSProperty_overflow_x => {
                    Ok(PropertyId::Longhand(LonghandId::OverflowX))
                }
                
    nsCSSPropertyID::eCSSProperty_overflow_y => {
                    Ok(PropertyId::Longhand(LonghandId::OverflowY))
                }
                
    nsCSSPropertyID::eCSSProperty_transition_duration => {
                    Ok(PropertyId::Longhand(LonghandId::TransitionDuration))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_MozTransitionDuration => {
                        Ok(PropertyId::Longhand(LonghandId::TransitionDuration))
                    }
                    
    nsCSSPropertyID::eCSSPropertyAlias_WebkitTransitionDuration => {
                        Ok(PropertyId::Longhand(LonghandId::TransitionDuration))
                    }
                
    nsCSSPropertyID::eCSSProperty_transition_timing_function => {
                    Ok(PropertyId::Longhand(LonghandId::TransitionTimingFunction))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_MozTransitionTimingFunction => {
                        Ok(PropertyId::Longhand(LonghandId::TransitionTimingFunction))
                    }
                    
    nsCSSPropertyID::eCSSPropertyAlias_WebkitTransitionTimingFunction => {
                        Ok(PropertyId::Longhand(LonghandId::TransitionTimingFunction))
                    }
                
    nsCSSPropertyID::eCSSProperty_transition_property => {
                    Ok(PropertyId::Longhand(LonghandId::TransitionProperty))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_MozTransitionProperty => {
                        Ok(PropertyId::Longhand(LonghandId::TransitionProperty))
                    }
                    
    nsCSSPropertyID::eCSSPropertyAlias_WebkitTransitionProperty => {
                        Ok(PropertyId::Longhand(LonghandId::TransitionProperty))
                    }
                
    nsCSSPropertyID::eCSSProperty_transition_delay => {
                    Ok(PropertyId::Longhand(LonghandId::TransitionDelay))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_MozTransitionDelay => {
                        Ok(PropertyId::Longhand(LonghandId::TransitionDelay))
                    }
                    
    nsCSSPropertyID::eCSSPropertyAlias_WebkitTransitionDelay => {
                        Ok(PropertyId::Longhand(LonghandId::TransitionDelay))
                    }
                
    nsCSSPropertyID::eCSSProperty_animation_name => {
                    Ok(PropertyId::Longhand(LonghandId::AnimationName))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_MozAnimationName => {
                        Ok(PropertyId::Longhand(LonghandId::AnimationName))
                    }
                    
    nsCSSPropertyID::eCSSPropertyAlias_WebkitAnimationName => {
                        Ok(PropertyId::Longhand(LonghandId::AnimationName))
                    }
                
    nsCSSPropertyID::eCSSProperty_animation_duration => {
                    Ok(PropertyId::Longhand(LonghandId::AnimationDuration))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_MozAnimationDuration => {
                        Ok(PropertyId::Longhand(LonghandId::AnimationDuration))
                    }
                    
    nsCSSPropertyID::eCSSPropertyAlias_WebkitAnimationDuration => {
                        Ok(PropertyId::Longhand(LonghandId::AnimationDuration))
                    }
                
    nsCSSPropertyID::eCSSProperty_animation_timing_function => {
                    Ok(PropertyId::Longhand(LonghandId::AnimationTimingFunction))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_MozAnimationTimingFunction => {
                        Ok(PropertyId::Longhand(LonghandId::AnimationTimingFunction))
                    }
                    
    nsCSSPropertyID::eCSSPropertyAlias_WebkitAnimationTimingFunction => {
                        Ok(PropertyId::Longhand(LonghandId::AnimationTimingFunction))
                    }
                
    nsCSSPropertyID::eCSSProperty_animation_iteration_count => {
                    Ok(PropertyId::Longhand(LonghandId::AnimationIterationCount))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_MozAnimationIterationCount => {
                        Ok(PropertyId::Longhand(LonghandId::AnimationIterationCount))
                    }
                    
    nsCSSPropertyID::eCSSPropertyAlias_WebkitAnimationIterationCount => {
                        Ok(PropertyId::Longhand(LonghandId::AnimationIterationCount))
                    }
                
    nsCSSPropertyID::eCSSProperty_animation_direction => {
                    Ok(PropertyId::Longhand(LonghandId::AnimationDirection))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_MozAnimationDirection => {
                        Ok(PropertyId::Longhand(LonghandId::AnimationDirection))
                    }
                    
    nsCSSPropertyID::eCSSPropertyAlias_WebkitAnimationDirection => {
                        Ok(PropertyId::Longhand(LonghandId::AnimationDirection))
                    }
                
    nsCSSPropertyID::eCSSProperty_animation_play_state => {
                    Ok(PropertyId::Longhand(LonghandId::AnimationPlayState))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_MozAnimationPlayState => {
                        Ok(PropertyId::Longhand(LonghandId::AnimationPlayState))
                    }
                    
    nsCSSPropertyID::eCSSPropertyAlias_WebkitAnimationPlayState => {
                        Ok(PropertyId::Longhand(LonghandId::AnimationPlayState))
                    }
                
    nsCSSPropertyID::eCSSProperty_animation_fill_mode => {
                    Ok(PropertyId::Longhand(LonghandId::AnimationFillMode))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_MozAnimationFillMode => {
                        Ok(PropertyId::Longhand(LonghandId::AnimationFillMode))
                    }
                    
    nsCSSPropertyID::eCSSPropertyAlias_WebkitAnimationFillMode => {
                        Ok(PropertyId::Longhand(LonghandId::AnimationFillMode))
                    }
                
    nsCSSPropertyID::eCSSProperty_animation_delay => {
                    Ok(PropertyId::Longhand(LonghandId::AnimationDelay))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_MozAnimationDelay => {
                        Ok(PropertyId::Longhand(LonghandId::AnimationDelay))
                    }
                    
    nsCSSPropertyID::eCSSPropertyAlias_WebkitAnimationDelay => {
                        Ok(PropertyId::Longhand(LonghandId::AnimationDelay))
                    }
                
    nsCSSPropertyID::eCSSProperty_scroll_snap_points_x => {
                    Ok(PropertyId::Longhand(LonghandId::ScrollSnapPointsX))
                }
                
    nsCSSPropertyID::eCSSProperty_scroll_snap_points_y => {
                    Ok(PropertyId::Longhand(LonghandId::ScrollSnapPointsY))
                }
                
    nsCSSPropertyID::eCSSProperty_scroll_snap_destination => {
                    Ok(PropertyId::Longhand(LonghandId::ScrollSnapDestination))
                }
                
    nsCSSPropertyID::eCSSProperty_scroll_snap_coordinate => {
                    Ok(PropertyId::Longhand(LonghandId::ScrollSnapCoordinate))
                }
                
    nsCSSPropertyID::eCSSProperty_transform => {
                    Ok(PropertyId::Longhand(LonghandId::Transform))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_WebkitTransform => {
                        Ok(PropertyId::Longhand(LonghandId::Transform))
                    }
                
    nsCSSPropertyID::eCSSProperty_scroll_behavior => {
                    Ok(PropertyId::Longhand(LonghandId::ScrollBehavior))
                }
                
    nsCSSPropertyID::eCSSProperty_scroll_snap_type_x => {
                    Ok(PropertyId::Longhand(LonghandId::ScrollSnapTypeX))
                }
                
    nsCSSPropertyID::eCSSProperty_scroll_snap_type_y => {
                    Ok(PropertyId::Longhand(LonghandId::ScrollSnapTypeY))
                }
                
    nsCSSPropertyID::eCSSProperty_isolation => {
                    Ok(PropertyId::Longhand(LonghandId::Isolation))
                }
                
    nsCSSPropertyID::eCSSProperty_page_break_after => {
                    Ok(PropertyId::Longhand(LonghandId::PageBreakAfter))
                }
                
    nsCSSPropertyID::eCSSProperty_page_break_before => {
                    Ok(PropertyId::Longhand(LonghandId::PageBreakBefore))
                }
                
    nsCSSPropertyID::eCSSProperty_page_break_inside => {
                    Ok(PropertyId::Longhand(LonghandId::PageBreakInside))
                }
                
    nsCSSPropertyID::eCSSProperty_resize => {
                    Ok(PropertyId::Longhand(LonghandId::Resize))
                }
                
    nsCSSPropertyID::eCSSProperty_perspective => {
                    Ok(PropertyId::Longhand(LonghandId::Perspective))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_MozPerspective => {
                        Ok(PropertyId::Longhand(LonghandId::Perspective))
                    }
                    
    nsCSSPropertyID::eCSSPropertyAlias_WebkitPerspective => {
                        Ok(PropertyId::Longhand(LonghandId::Perspective))
                    }
                
    nsCSSPropertyID::eCSSProperty_perspective_origin => {
                    Ok(PropertyId::Longhand(LonghandId::PerspectiveOrigin))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_MozPerspectiveOrigin => {
                        Ok(PropertyId::Longhand(LonghandId::PerspectiveOrigin))
                    }
                    
    nsCSSPropertyID::eCSSPropertyAlias_WebkitPerspectiveOrigin => {
                        Ok(PropertyId::Longhand(LonghandId::PerspectiveOrigin))
                    }
                
    nsCSSPropertyID::eCSSProperty_backface_visibility => {
                    Ok(PropertyId::Longhand(LonghandId::BackfaceVisibility))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_MozBackfaceVisibility => {
                        Ok(PropertyId::Longhand(LonghandId::BackfaceVisibility))
                    }
                    
    nsCSSPropertyID::eCSSPropertyAlias_WebkitBackfaceVisibility => {
                        Ok(PropertyId::Longhand(LonghandId::BackfaceVisibility))
                    }
                
    nsCSSPropertyID::eCSSProperty_transform_box => {
                    Ok(PropertyId::Longhand(LonghandId::TransformBox))
                }
                
    nsCSSPropertyID::eCSSProperty_transform_style => {
                    Ok(PropertyId::Longhand(LonghandId::TransformStyle))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_MozTransformStyle => {
                        Ok(PropertyId::Longhand(LonghandId::TransformStyle))
                    }
                    
    nsCSSPropertyID::eCSSPropertyAlias_WebkitTransformStyle => {
                        Ok(PropertyId::Longhand(LonghandId::TransformStyle))
                    }
                
    nsCSSPropertyID::eCSSProperty_transform_origin => {
                    Ok(PropertyId::Longhand(LonghandId::TransformOrigin))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_MozTransformOrigin => {
                        Ok(PropertyId::Longhand(LonghandId::TransformOrigin))
                    }
                    
    nsCSSPropertyID::eCSSPropertyAlias_WebkitTransformOrigin => {
                        Ok(PropertyId::Longhand(LonghandId::TransformOrigin))
                    }
                
    nsCSSPropertyID::eCSSProperty_contain => {
                    Ok(PropertyId::Longhand(LonghandId::Contain))
                }
                
    nsCSSPropertyID::eCSSProperty__moz_appearance => {
                    Ok(PropertyId::Longhand(LonghandId::MozAppearance))
                }
                
    nsCSSPropertyID::eCSSProperty__moz_binding => {
                    Ok(PropertyId::Longhand(LonghandId::MozBinding))
                }
                
    nsCSSPropertyID::eCSSProperty__moz_orient => {
                    Ok(PropertyId::Longhand(LonghandId::MozOrient))
                }
                
    nsCSSPropertyID::eCSSProperty_will_change => {
                    Ok(PropertyId::Longhand(LonghandId::WillChange))
                }
                
    nsCSSPropertyID::eCSSProperty_shape_outside => {
                    Ok(PropertyId::Longhand(LonghandId::ShapeOutside))
                }
                
    nsCSSPropertyID::eCSSProperty_touch_action => {
                    Ok(PropertyId::Longhand(LonghandId::TouchAction))
                }
                
    nsCSSPropertyID::eCSSProperty_color => {
                    Ok(PropertyId::Longhand(LonghandId::Color))
                }
                
    nsCSSPropertyID::eCSSProperty_column_width => {
                    Ok(PropertyId::Longhand(LonghandId::ColumnWidth))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_MozColumnWidth => {
                        Ok(PropertyId::Longhand(LonghandId::ColumnWidth))
                    }
                
    nsCSSPropertyID::eCSSProperty_column_count => {
                    Ok(PropertyId::Longhand(LonghandId::ColumnCount))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_MozColumnCount => {
                        Ok(PropertyId::Longhand(LonghandId::ColumnCount))
                    }
                
    nsCSSPropertyID::eCSSProperty_column_gap => {
                    Ok(PropertyId::Longhand(LonghandId::ColumnGap))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_MozColumnGap => {
                        Ok(PropertyId::Longhand(LonghandId::ColumnGap))
                    }
                
    nsCSSPropertyID::eCSSProperty_column_fill => {
                    Ok(PropertyId::Longhand(LonghandId::ColumnFill))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_MozColumnFill => {
                        Ok(PropertyId::Longhand(LonghandId::ColumnFill))
                    }
                
    nsCSSPropertyID::eCSSProperty_column_rule_width => {
                    Ok(PropertyId::Longhand(LonghandId::ColumnRuleWidth))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_MozColumnRuleWidth => {
                        Ok(PropertyId::Longhand(LonghandId::ColumnRuleWidth))
                    }
                
    nsCSSPropertyID::eCSSProperty_column_rule_color => {
                    Ok(PropertyId::Longhand(LonghandId::ColumnRuleColor))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_MozColumnRuleColor => {
                        Ok(PropertyId::Longhand(LonghandId::ColumnRuleColor))
                    }
                
    nsCSSPropertyID::eCSSProperty_column_span => {
                    Ok(PropertyId::Longhand(LonghandId::ColumnSpan))
                }
                
    nsCSSPropertyID::eCSSProperty_column_rule_style => {
                    Ok(PropertyId::Longhand(LonghandId::ColumnRuleStyle))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_MozColumnRuleStyle => {
                        Ok(PropertyId::Longhand(LonghandId::ColumnRuleStyle))
                    }
                
    nsCSSPropertyID::eCSSProperty_content => {
                    Ok(PropertyId::Longhand(LonghandId::Content))
                }
                
    nsCSSPropertyID::eCSSProperty_counter_increment => {
                    Ok(PropertyId::Longhand(LonghandId::CounterIncrement))
                }
                
    nsCSSPropertyID::eCSSProperty_counter_reset => {
                    Ok(PropertyId::Longhand(LonghandId::CounterReset))
                }
                
    nsCSSPropertyID::eCSSProperty_opacity => {
                    Ok(PropertyId::Longhand(LonghandId::Opacity))
                }
                
    nsCSSPropertyID::eCSSProperty_box_shadow => {
                    Ok(PropertyId::Longhand(LonghandId::BoxShadow))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_WebkitBoxShadow => {
                        Ok(PropertyId::Longhand(LonghandId::BoxShadow))
                    }
                
    nsCSSPropertyID::eCSSProperty_clip => {
                    Ok(PropertyId::Longhand(LonghandId::Clip))
                }
                
    nsCSSPropertyID::eCSSProperty_filter => {
                    Ok(PropertyId::Longhand(LonghandId::Filter))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_WebkitFilter => {
                        Ok(PropertyId::Longhand(LonghandId::Filter))
                    }
                
    nsCSSPropertyID::eCSSProperty_mix_blend_mode => {
                    Ok(PropertyId::Longhand(LonghandId::MixBlendMode))
                }
                
    nsCSSPropertyID::eCSSProperty_font_family => {
                    Ok(PropertyId::Longhand(LonghandId::FontFamily))
                }
                
    nsCSSPropertyID::eCSSProperty_font_style => {
                    Ok(PropertyId::Longhand(LonghandId::FontStyle))
                }
                
    nsCSSPropertyID::eCSSProperty_font_variant_caps => {
                    Ok(PropertyId::Longhand(LonghandId::FontVariantCaps))
                }
                
    nsCSSPropertyID::eCSSProperty_font_weight => {
                    Ok(PropertyId::Longhand(LonghandId::FontWeight))
                }
                
    nsCSSPropertyID::eCSSProperty_font_size => {
                    Ok(PropertyId::Longhand(LonghandId::FontSize))
                }
                
    nsCSSPropertyID::eCSSProperty_font_size_adjust => {
                    Ok(PropertyId::Longhand(LonghandId::FontSizeAdjust))
                }
                
    nsCSSPropertyID::eCSSProperty_font_synthesis => {
                    Ok(PropertyId::Longhand(LonghandId::FontSynthesis))
                }
                
    nsCSSPropertyID::eCSSProperty_font_stretch => {
                    Ok(PropertyId::Longhand(LonghandId::FontStretch))
                }
                
    nsCSSPropertyID::eCSSProperty_font_kerning => {
                    Ok(PropertyId::Longhand(LonghandId::FontKerning))
                }
                
    nsCSSPropertyID::eCSSProperty_font_variant_alternates => {
                    Ok(PropertyId::Longhand(LonghandId::FontVariantAlternates))
                }
                
    nsCSSPropertyID::eCSSProperty_font_variant_east_asian => {
                    Ok(PropertyId::Longhand(LonghandId::FontVariantEastAsian))
                }
                
    nsCSSPropertyID::eCSSProperty_font_variant_ligatures => {
                    Ok(PropertyId::Longhand(LonghandId::FontVariantLigatures))
                }
                
    nsCSSPropertyID::eCSSProperty_font_variant_numeric => {
                    Ok(PropertyId::Longhand(LonghandId::FontVariantNumeric))
                }
                
    nsCSSPropertyID::eCSSProperty_font_variant_position => {
                    Ok(PropertyId::Longhand(LonghandId::FontVariantPosition))
                }
                
    nsCSSPropertyID::eCSSProperty_font_feature_settings => {
                    Ok(PropertyId::Longhand(LonghandId::FontFeatureSettings))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_MozFontFeatureSettings => {
                        Ok(PropertyId::Longhand(LonghandId::FontFeatureSettings))
                    }
                
    nsCSSPropertyID::eCSSProperty_font_variation_settings => {
                    Ok(PropertyId::Longhand(LonghandId::FontVariationSettings))
                }
                
    nsCSSPropertyID::eCSSProperty_font_language_override => {
                    Ok(PropertyId::Longhand(LonghandId::FontLanguageOverride))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_MozFontLanguageOverride => {
                        Ok(PropertyId::Longhand(LonghandId::FontLanguageOverride))
                    }
                
    nsCSSPropertyID::eCSSProperty__x_lang => {
                    Ok(PropertyId::Longhand(LonghandId::XLang))
                }
                
    nsCSSPropertyID::eCSSProperty__moz_script_size_multiplier => {
                    Ok(PropertyId::Longhand(LonghandId::MozScriptSizeMultiplier))
                }
                
    nsCSSPropertyID::eCSSProperty__moz_script_level => {
                    Ok(PropertyId::Longhand(LonghandId::MozScriptLevel))
                }
                
    nsCSSPropertyID::eCSSProperty__moz_math_display => {
                    Ok(PropertyId::Longhand(LonghandId::MozMathDisplay))
                }
                
    nsCSSPropertyID::eCSSProperty__moz_math_variant => {
                    Ok(PropertyId::Longhand(LonghandId::MozMathVariant))
                }
                
    nsCSSPropertyID::eCSSProperty__moz_script_min_size => {
                    Ok(PropertyId::Longhand(LonghandId::MozScriptMinSize))
                }
                
    nsCSSPropertyID::eCSSProperty__x_text_zoom => {
                    Ok(PropertyId::Longhand(LonghandId::XTextZoom))
                }
                
    nsCSSPropertyID::eCSSProperty__moz_osx_font_smoothing => {
                    Ok(PropertyId::Longhand(LonghandId::MozOsxFontSmoothing))
                }
                
    nsCSSPropertyID::eCSSProperty__moz_min_font_size_ratio => {
                    Ok(PropertyId::Longhand(LonghandId::MozMinFontSizeRatio))
                }
                
    nsCSSPropertyID::eCSSProperty_visibility => {
                    Ok(PropertyId::Longhand(LonghandId::Visibility))
                }
                
    nsCSSPropertyID::eCSSProperty_writing_mode => {
                    Ok(PropertyId::Longhand(LonghandId::WritingMode))
                }
                
    nsCSSPropertyID::eCSSProperty_direction => {
                    Ok(PropertyId::Longhand(LonghandId::Direction))
                }
                
    nsCSSPropertyID::eCSSProperty_text_orientation => {
                    Ok(PropertyId::Longhand(LonghandId::TextOrientation))
                }
                
    nsCSSPropertyID::eCSSProperty_color_adjust => {
                    Ok(PropertyId::Longhand(LonghandId::ColorAdjust))
                }
                
    nsCSSPropertyID::eCSSProperty_image_rendering => {
                    Ok(PropertyId::Longhand(LonghandId::ImageRendering))
                }
                
    nsCSSPropertyID::eCSSProperty_image_orientation => {
                    Ok(PropertyId::Longhand(LonghandId::ImageOrientation))
                }
                
    nsCSSPropertyID::eCSSProperty_border_collapse => {
                    Ok(PropertyId::Longhand(LonghandId::BorderCollapse))
                }
                
    nsCSSPropertyID::eCSSProperty_empty_cells => {
                    Ok(PropertyId::Longhand(LonghandId::EmptyCells))
                }
                
    nsCSSPropertyID::eCSSProperty_caption_side => {
                    Ok(PropertyId::Longhand(LonghandId::CaptionSide))
                }
                
    nsCSSPropertyID::eCSSProperty_border_spacing => {
                    Ok(PropertyId::Longhand(LonghandId::BorderSpacing))
                }
                
    nsCSSPropertyID::eCSSProperty_line_height => {
                    Ok(PropertyId::Longhand(LonghandId::LineHeight))
                }
                
    nsCSSPropertyID::eCSSProperty_text_transform => {
                    Ok(PropertyId::Longhand(LonghandId::TextTransform))
                }
                
    nsCSSPropertyID::eCSSProperty_hyphens => {
                    Ok(PropertyId::Longhand(LonghandId::Hyphens))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_MozHyphens => {
                        Ok(PropertyId::Longhand(LonghandId::Hyphens))
                    }
                
    nsCSSPropertyID::eCSSProperty__moz_text_size_adjust => {
                    Ok(PropertyId::Longhand(LonghandId::MozTextSizeAdjust))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_WebkitTextSizeAdjust => {
                        Ok(PropertyId::Longhand(LonghandId::MozTextSizeAdjust))
                    }
                
    nsCSSPropertyID::eCSSProperty_text_indent => {
                    Ok(PropertyId::Longhand(LonghandId::TextIndent))
                }
                
    nsCSSPropertyID::eCSSProperty_overflow_wrap => {
                    Ok(PropertyId::Longhand(LonghandId::OverflowWrap))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_WordWrap => {
                        Ok(PropertyId::Longhand(LonghandId::OverflowWrap))
                    }
                
    nsCSSPropertyID::eCSSProperty_word_break => {
                    Ok(PropertyId::Longhand(LonghandId::WordBreak))
                }
                
    nsCSSPropertyID::eCSSProperty_text_justify => {
                    Ok(PropertyId::Longhand(LonghandId::TextJustify))
                }
                
    nsCSSPropertyID::eCSSProperty_text_align_last => {
                    Ok(PropertyId::Longhand(LonghandId::TextAlignLast))
                }
                
    nsCSSPropertyID::eCSSProperty_text_align => {
                    Ok(PropertyId::Longhand(LonghandId::TextAlign))
                }
                
    nsCSSPropertyID::eCSSProperty_letter_spacing => {
                    Ok(PropertyId::Longhand(LonghandId::LetterSpacing))
                }
                
    nsCSSPropertyID::eCSSProperty_word_spacing => {
                    Ok(PropertyId::Longhand(LonghandId::WordSpacing))
                }
                
    nsCSSPropertyID::eCSSProperty_white_space => {
                    Ok(PropertyId::Longhand(LonghandId::WhiteSpace))
                }
                
    nsCSSPropertyID::eCSSProperty_text_shadow => {
                    Ok(PropertyId::Longhand(LonghandId::TextShadow))
                }
                
    nsCSSPropertyID::eCSSProperty_text_emphasis_style => {
                    Ok(PropertyId::Longhand(LonghandId::TextEmphasisStyle))
                }
                
    nsCSSPropertyID::eCSSProperty_text_emphasis_position => {
                    Ok(PropertyId::Longhand(LonghandId::TextEmphasisPosition))
                }
                
    nsCSSPropertyID::eCSSProperty_text_emphasis_color => {
                    Ok(PropertyId::Longhand(LonghandId::TextEmphasisColor))
                }
                
    nsCSSPropertyID::eCSSProperty__moz_tab_size => {
                    Ok(PropertyId::Longhand(LonghandId::MozTabSize))
                }
                
    nsCSSPropertyID::eCSSProperty__webkit_text_fill_color => {
                    Ok(PropertyId::Longhand(LonghandId::WebkitTextFillColor))
                }
                
    nsCSSPropertyID::eCSSProperty__webkit_text_stroke_color => {
                    Ok(PropertyId::Longhand(LonghandId::WebkitTextStrokeColor))
                }
                
    nsCSSPropertyID::eCSSProperty__webkit_text_stroke_width => {
                    Ok(PropertyId::Longhand(LonghandId::WebkitTextStrokeWidth))
                }
                
    nsCSSPropertyID::eCSSProperty_ruby_align => {
                    Ok(PropertyId::Longhand(LonghandId::RubyAlign))
                }
                
    nsCSSPropertyID::eCSSProperty_ruby_position => {
                    Ok(PropertyId::Longhand(LonghandId::RubyPosition))
                }
                
    nsCSSPropertyID::eCSSProperty_text_combine_upright => {
                    Ok(PropertyId::Longhand(LonghandId::TextCombineUpright))
                }
                
    nsCSSPropertyID::eCSSProperty_text_rendering => {
                    Ok(PropertyId::Longhand(LonghandId::TextRendering))
                }
                
    nsCSSPropertyID::eCSSProperty__moz_control_character_visibility => {
                    Ok(PropertyId::Longhand(LonghandId::MozControlCharacterVisibility))
                }
                
    nsCSSPropertyID::eCSSProperty_list_style_position => {
                    Ok(PropertyId::Longhand(LonghandId::ListStylePosition))
                }
                
    nsCSSPropertyID::eCSSProperty_list_style_type => {
                    Ok(PropertyId::Longhand(LonghandId::ListStyleType))
                }
                
    nsCSSPropertyID::eCSSProperty_list_style_image => {
                    Ok(PropertyId::Longhand(LonghandId::ListStyleImage))
                }
                
    nsCSSPropertyID::eCSSProperty_quotes => {
                    Ok(PropertyId::Longhand(LonghandId::Quotes))
                }
                
    nsCSSPropertyID::eCSSProperty__moz_image_region => {
                    Ok(PropertyId::Longhand(LonghandId::MozImageRegion))
                }
                
    nsCSSPropertyID::eCSSProperty_margin_top => {
                    Ok(PropertyId::Longhand(LonghandId::MarginTop))
                }
                
    nsCSSPropertyID::eCSSProperty_margin_left => {
                    Ok(PropertyId::Longhand(LonghandId::MarginLeft))
                }
                
    nsCSSPropertyID::eCSSProperty_margin_bottom => {
                    Ok(PropertyId::Longhand(LonghandId::MarginBottom))
                }
                
    nsCSSPropertyID::eCSSProperty_margin_right => {
                    Ok(PropertyId::Longhand(LonghandId::MarginRight))
                }
                
    nsCSSPropertyID::eCSSProperty_margin_block_start => {
                    Ok(PropertyId::Longhand(LonghandId::MarginBlockStart))
                }
                
    nsCSSPropertyID::eCSSProperty_margin_block_end => {
                    Ok(PropertyId::Longhand(LonghandId::MarginBlockEnd))
                }
                
    nsCSSPropertyID::eCSSProperty_margin_inline_start => {
                    Ok(PropertyId::Longhand(LonghandId::MarginInlineStart))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_MozMarginStart => {
                        Ok(PropertyId::Longhand(LonghandId::MarginInlineStart))
                    }
                
    nsCSSPropertyID::eCSSProperty_margin_inline_end => {
                    Ok(PropertyId::Longhand(LonghandId::MarginInlineEnd))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_MozMarginEnd => {
                        Ok(PropertyId::Longhand(LonghandId::MarginInlineEnd))
                    }
                
    nsCSSPropertyID::eCSSProperty_outline_color => {
                    Ok(PropertyId::Longhand(LonghandId::OutlineColor))
                }
                
    nsCSSPropertyID::eCSSProperty_outline_style => {
                    Ok(PropertyId::Longhand(LonghandId::OutlineStyle))
                }
                
    nsCSSPropertyID::eCSSProperty_outline_width => {
                    Ok(PropertyId::Longhand(LonghandId::OutlineWidth))
                }
                
    nsCSSPropertyID::eCSSProperty__moz_outline_radius_topleft => {
                    Ok(PropertyId::Longhand(LonghandId::MozOutlineRadiusTopleft))
                }
                
    nsCSSPropertyID::eCSSProperty__moz_outline_radius_topright => {
                    Ok(PropertyId::Longhand(LonghandId::MozOutlineRadiusTopright))
                }
                
    nsCSSPropertyID::eCSSProperty__moz_outline_radius_bottomright => {
                    Ok(PropertyId::Longhand(LonghandId::MozOutlineRadiusBottomright))
                }
                
    nsCSSPropertyID::eCSSProperty__moz_outline_radius_bottomleft => {
                    Ok(PropertyId::Longhand(LonghandId::MozOutlineRadiusBottomleft))
                }
                
    nsCSSPropertyID::eCSSProperty_outline_offset => {
                    Ok(PropertyId::Longhand(LonghandId::OutlineOffset))
                }
                
    nsCSSPropertyID::eCSSProperty_padding_top => {
                    Ok(PropertyId::Longhand(LonghandId::PaddingTop))
                }
                
    nsCSSPropertyID::eCSSProperty_padding_left => {
                    Ok(PropertyId::Longhand(LonghandId::PaddingLeft))
                }
                
    nsCSSPropertyID::eCSSProperty_padding_bottom => {
                    Ok(PropertyId::Longhand(LonghandId::PaddingBottom))
                }
                
    nsCSSPropertyID::eCSSProperty_padding_right => {
                    Ok(PropertyId::Longhand(LonghandId::PaddingRight))
                }
                
    nsCSSPropertyID::eCSSProperty_padding_block_start => {
                    Ok(PropertyId::Longhand(LonghandId::PaddingBlockStart))
                }
                
    nsCSSPropertyID::eCSSProperty_padding_block_end => {
                    Ok(PropertyId::Longhand(LonghandId::PaddingBlockEnd))
                }
                
    nsCSSPropertyID::eCSSProperty_padding_inline_start => {
                    Ok(PropertyId::Longhand(LonghandId::PaddingInlineStart))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_MozPaddingStart => {
                        Ok(PropertyId::Longhand(LonghandId::PaddingInlineStart))
                    }
                
    nsCSSPropertyID::eCSSProperty_padding_inline_end => {
                    Ok(PropertyId::Longhand(LonghandId::PaddingInlineEnd))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_MozPaddingEnd => {
                        Ok(PropertyId::Longhand(LonghandId::PaddingInlineEnd))
                    }
                
    nsCSSPropertyID::eCSSProperty_cursor => {
                    Ok(PropertyId::Longhand(LonghandId::Cursor))
                }
                
    nsCSSPropertyID::eCSSProperty_pointer_events => {
                    Ok(PropertyId::Longhand(LonghandId::PointerEvents))
                }
                
    nsCSSPropertyID::eCSSProperty__moz_user_input => {
                    Ok(PropertyId::Longhand(LonghandId::MozUserInput))
                }
                
    nsCSSPropertyID::eCSSProperty__moz_user_modify => {
                    Ok(PropertyId::Longhand(LonghandId::MozUserModify))
                }
                
    nsCSSPropertyID::eCSSProperty__moz_user_focus => {
                    Ok(PropertyId::Longhand(LonghandId::MozUserFocus))
                }
                
    nsCSSPropertyID::eCSSProperty_caret_color => {
                    Ok(PropertyId::Longhand(LonghandId::CaretColor))
                }
                
    nsCSSPropertyID::eCSSProperty_top => {
                    Ok(PropertyId::Longhand(LonghandId::Top))
                }
                
    nsCSSPropertyID::eCSSProperty_left => {
                    Ok(PropertyId::Longhand(LonghandId::Left))
                }
                
    nsCSSPropertyID::eCSSProperty_bottom => {
                    Ok(PropertyId::Longhand(LonghandId::Bottom))
                }
                
    nsCSSPropertyID::eCSSProperty_right => {
                    Ok(PropertyId::Longhand(LonghandId::Right))
                }
                
    nsCSSPropertyID::eCSSProperty_offset_block_start => {
                    Ok(PropertyId::Longhand(LonghandId::OffsetBlockStart))
                }
                
    nsCSSPropertyID::eCSSProperty_offset_block_end => {
                    Ok(PropertyId::Longhand(LonghandId::OffsetBlockEnd))
                }
                
    nsCSSPropertyID::eCSSProperty_offset_inline_start => {
                    Ok(PropertyId::Longhand(LonghandId::OffsetInlineStart))
                }
                
    nsCSSPropertyID::eCSSProperty_offset_inline_end => {
                    Ok(PropertyId::Longhand(LonghandId::OffsetInlineEnd))
                }
                
    nsCSSPropertyID::eCSSProperty_z_index => {
                    Ok(PropertyId::Longhand(LonghandId::ZIndex))
                }
                
    nsCSSPropertyID::eCSSProperty_flex_direction => {
                    Ok(PropertyId::Longhand(LonghandId::FlexDirection))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_WebkitFlexDirection => {
                        Ok(PropertyId::Longhand(LonghandId::FlexDirection))
                    }
                
    nsCSSPropertyID::eCSSProperty_flex_wrap => {
                    Ok(PropertyId::Longhand(LonghandId::FlexWrap))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_WebkitFlexWrap => {
                        Ok(PropertyId::Longhand(LonghandId::FlexWrap))
                    }
                
    nsCSSPropertyID::eCSSProperty_justify_content => {
                    Ok(PropertyId::Longhand(LonghandId::JustifyContent))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_WebkitJustifyContent => {
                        Ok(PropertyId::Longhand(LonghandId::JustifyContent))
                    }
                
    nsCSSPropertyID::eCSSProperty_align_content => {
                    Ok(PropertyId::Longhand(LonghandId::AlignContent))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_WebkitAlignContent => {
                        Ok(PropertyId::Longhand(LonghandId::AlignContent))
                    }
                
    nsCSSPropertyID::eCSSProperty_align_items => {
                    Ok(PropertyId::Longhand(LonghandId::AlignItems))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_WebkitAlignItems => {
                        Ok(PropertyId::Longhand(LonghandId::AlignItems))
                    }
                
    nsCSSPropertyID::eCSSProperty_justify_items => {
                    Ok(PropertyId::Longhand(LonghandId::JustifyItems))
                }
                
    nsCSSPropertyID::eCSSProperty_flex_grow => {
                    Ok(PropertyId::Longhand(LonghandId::FlexGrow))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_WebkitFlexGrow => {
                        Ok(PropertyId::Longhand(LonghandId::FlexGrow))
                    }
                
    nsCSSPropertyID::eCSSProperty_flex_shrink => {
                    Ok(PropertyId::Longhand(LonghandId::FlexShrink))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_WebkitFlexShrink => {
                        Ok(PropertyId::Longhand(LonghandId::FlexShrink))
                    }
                
    nsCSSPropertyID::eCSSProperty_align_self => {
                    Ok(PropertyId::Longhand(LonghandId::AlignSelf))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_WebkitAlignSelf => {
                        Ok(PropertyId::Longhand(LonghandId::AlignSelf))
                    }
                
    nsCSSPropertyID::eCSSProperty_justify_self => {
                    Ok(PropertyId::Longhand(LonghandId::JustifySelf))
                }
                
    nsCSSPropertyID::eCSSProperty_order => {
                    Ok(PropertyId::Longhand(LonghandId::Order))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_WebkitOrder => {
                        Ok(PropertyId::Longhand(LonghandId::Order))
                    }
                
    nsCSSPropertyID::eCSSProperty_flex_basis => {
                    Ok(PropertyId::Longhand(LonghandId::FlexBasis))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_WebkitFlexBasis => {
                        Ok(PropertyId::Longhand(LonghandId::FlexBasis))
                    }
                
    nsCSSPropertyID::eCSSProperty_width => {
                    Ok(PropertyId::Longhand(LonghandId::Width))
                }
                
    nsCSSPropertyID::eCSSProperty_min_width => {
                    Ok(PropertyId::Longhand(LonghandId::MinWidth))
                }
                
    nsCSSPropertyID::eCSSProperty_max_width => {
                    Ok(PropertyId::Longhand(LonghandId::MaxWidth))
                }
                
    nsCSSPropertyID::eCSSProperty_height => {
                    Ok(PropertyId::Longhand(LonghandId::Height))
                }
                
    nsCSSPropertyID::eCSSProperty_min_height => {
                    Ok(PropertyId::Longhand(LonghandId::MinHeight))
                }
                
    nsCSSPropertyID::eCSSProperty_max_height => {
                    Ok(PropertyId::Longhand(LonghandId::MaxHeight))
                }
                
    nsCSSPropertyID::eCSSProperty_block_size => {
                    Ok(PropertyId::Longhand(LonghandId::BlockSize))
                }
                
    nsCSSPropertyID::eCSSProperty_min_block_size => {
                    Ok(PropertyId::Longhand(LonghandId::MinBlockSize))
                }
                
    nsCSSPropertyID::eCSSProperty_max_block_size => {
                    Ok(PropertyId::Longhand(LonghandId::MaxBlockSize))
                }
                
    nsCSSPropertyID::eCSSProperty_inline_size => {
                    Ok(PropertyId::Longhand(LonghandId::InlineSize))
                }
                
    nsCSSPropertyID::eCSSProperty_min_inline_size => {
                    Ok(PropertyId::Longhand(LonghandId::MinInlineSize))
                }
                
    nsCSSPropertyID::eCSSProperty_max_inline_size => {
                    Ok(PropertyId::Longhand(LonghandId::MaxInlineSize))
                }
                
    nsCSSPropertyID::eCSSProperty_box_sizing => {
                    Ok(PropertyId::Longhand(LonghandId::BoxSizing))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_MozBoxSizing => {
                        Ok(PropertyId::Longhand(LonghandId::BoxSizing))
                    }
                    
    nsCSSPropertyID::eCSSPropertyAlias_WebkitBoxSizing => {
                        Ok(PropertyId::Longhand(LonghandId::BoxSizing))
                    }
                
    nsCSSPropertyID::eCSSProperty_object_fit => {
                    Ok(PropertyId::Longhand(LonghandId::ObjectFit))
                }
                
    nsCSSPropertyID::eCSSProperty_object_position => {
                    Ok(PropertyId::Longhand(LonghandId::ObjectPosition))
                }
                
    nsCSSPropertyID::eCSSProperty_grid_row_gap => {
                    Ok(PropertyId::Longhand(LonghandId::GridRowGap))
                }
                
    nsCSSPropertyID::eCSSProperty_grid_row_start => {
                    Ok(PropertyId::Longhand(LonghandId::GridRowStart))
                }
                
    nsCSSPropertyID::eCSSProperty_grid_row_end => {
                    Ok(PropertyId::Longhand(LonghandId::GridRowEnd))
                }
                
    nsCSSPropertyID::eCSSProperty_grid_auto_rows => {
                    Ok(PropertyId::Longhand(LonghandId::GridAutoRows))
                }
                
    nsCSSPropertyID::eCSSProperty_grid_template_rows => {
                    Ok(PropertyId::Longhand(LonghandId::GridTemplateRows))
                }
                
    nsCSSPropertyID::eCSSProperty_grid_column_gap => {
                    Ok(PropertyId::Longhand(LonghandId::GridColumnGap))
                }
                
    nsCSSPropertyID::eCSSProperty_grid_column_start => {
                    Ok(PropertyId::Longhand(LonghandId::GridColumnStart))
                }
                
    nsCSSPropertyID::eCSSProperty_grid_column_end => {
                    Ok(PropertyId::Longhand(LonghandId::GridColumnEnd))
                }
                
    nsCSSPropertyID::eCSSProperty_grid_auto_columns => {
                    Ok(PropertyId::Longhand(LonghandId::GridAutoColumns))
                }
                
    nsCSSPropertyID::eCSSProperty_grid_template_columns => {
                    Ok(PropertyId::Longhand(LonghandId::GridTemplateColumns))
                }
                
    nsCSSPropertyID::eCSSProperty_grid_auto_flow => {
                    Ok(PropertyId::Longhand(LonghandId::GridAutoFlow))
                }
                
    nsCSSPropertyID::eCSSProperty_grid_template_areas => {
                    Ok(PropertyId::Longhand(LonghandId::GridTemplateAreas))
                }
                
    nsCSSPropertyID::eCSSProperty_table_layout => {
                    Ok(PropertyId::Longhand(LonghandId::TableLayout))
                }
                
    nsCSSPropertyID::eCSSProperty__x_span => {
                    Ok(PropertyId::Longhand(LonghandId::XSpan))
                }
                
    nsCSSPropertyID::eCSSProperty_text_overflow => {
                    Ok(PropertyId::Longhand(LonghandId::TextOverflow))
                }
                
    nsCSSPropertyID::eCSSProperty_unicode_bidi => {
                    Ok(PropertyId::Longhand(LonghandId::UnicodeBidi))
                }
                
    nsCSSPropertyID::eCSSProperty_text_decoration_line => {
                    Ok(PropertyId::Longhand(LonghandId::TextDecorationLine))
                }
                
    nsCSSPropertyID::eCSSProperty_text_decoration_style => {
                    Ok(PropertyId::Longhand(LonghandId::TextDecorationStyle))
                }
                
    nsCSSPropertyID::eCSSProperty_text_decoration_color => {
                    Ok(PropertyId::Longhand(LonghandId::TextDecorationColor))
                }
                
    nsCSSPropertyID::eCSSProperty_initial_letter => {
                    Ok(PropertyId::Longhand(LonghandId::InitialLetter))
                }
                
    nsCSSPropertyID::eCSSProperty_ime_mode => {
                    Ok(PropertyId::Longhand(LonghandId::ImeMode))
                }
                
    nsCSSPropertyID::eCSSProperty__moz_user_select => {
                    Ok(PropertyId::Longhand(LonghandId::MozUserSelect))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_WebkitUserSelect => {
                        Ok(PropertyId::Longhand(LonghandId::MozUserSelect))
                    }
                
    nsCSSPropertyID::eCSSProperty__moz_window_dragging => {
                    Ok(PropertyId::Longhand(LonghandId::MozWindowDragging))
                }
                
    nsCSSPropertyID::eCSSProperty__moz_window_shadow => {
                    Ok(PropertyId::Longhand(LonghandId::MozWindowShadow))
                }
                
    nsCSSPropertyID::eCSSProperty__moz_force_broken_image_icon => {
                    Ok(PropertyId::Longhand(LonghandId::MozForceBrokenImageIcon))
                }
                
    nsCSSPropertyID::eCSSProperty_text_anchor => {
                    Ok(PropertyId::Longhand(LonghandId::TextAnchor))
                }
                
    nsCSSPropertyID::eCSSProperty_color_interpolation => {
                    Ok(PropertyId::Longhand(LonghandId::ColorInterpolation))
                }
                
    nsCSSPropertyID::eCSSProperty_color_interpolation_filters => {
                    Ok(PropertyId::Longhand(LonghandId::ColorInterpolationFilters))
                }
                
    nsCSSPropertyID::eCSSProperty_fill => {
                    Ok(PropertyId::Longhand(LonghandId::Fill))
                }
                
    nsCSSPropertyID::eCSSProperty_fill_opacity => {
                    Ok(PropertyId::Longhand(LonghandId::FillOpacity))
                }
                
    nsCSSPropertyID::eCSSProperty_fill_rule => {
                    Ok(PropertyId::Longhand(LonghandId::FillRule))
                }
                
    nsCSSPropertyID::eCSSProperty_shape_rendering => {
                    Ok(PropertyId::Longhand(LonghandId::ShapeRendering))
                }
                
    nsCSSPropertyID::eCSSProperty_stroke => {
                    Ok(PropertyId::Longhand(LonghandId::Stroke))
                }
                
    nsCSSPropertyID::eCSSProperty_stroke_width => {
                    Ok(PropertyId::Longhand(LonghandId::StrokeWidth))
                }
                
    nsCSSPropertyID::eCSSProperty_stroke_linecap => {
                    Ok(PropertyId::Longhand(LonghandId::StrokeLinecap))
                }
                
    nsCSSPropertyID::eCSSProperty_stroke_linejoin => {
                    Ok(PropertyId::Longhand(LonghandId::StrokeLinejoin))
                }
                
    nsCSSPropertyID::eCSSProperty_stroke_miterlimit => {
                    Ok(PropertyId::Longhand(LonghandId::StrokeMiterlimit))
                }
                
    nsCSSPropertyID::eCSSProperty_stroke_opacity => {
                    Ok(PropertyId::Longhand(LonghandId::StrokeOpacity))
                }
                
    nsCSSPropertyID::eCSSProperty_stroke_dasharray => {
                    Ok(PropertyId::Longhand(LonghandId::StrokeDasharray))
                }
                
    nsCSSPropertyID::eCSSProperty_stroke_dashoffset => {
                    Ok(PropertyId::Longhand(LonghandId::StrokeDashoffset))
                }
                
    nsCSSPropertyID::eCSSProperty_clip_rule => {
                    Ok(PropertyId::Longhand(LonghandId::ClipRule))
                }
                
    nsCSSPropertyID::eCSSProperty_marker_start => {
                    Ok(PropertyId::Longhand(LonghandId::MarkerStart))
                }
                
    nsCSSPropertyID::eCSSProperty_marker_mid => {
                    Ok(PropertyId::Longhand(LonghandId::MarkerMid))
                }
                
    nsCSSPropertyID::eCSSProperty_marker_end => {
                    Ok(PropertyId::Longhand(LonghandId::MarkerEnd))
                }
                
    nsCSSPropertyID::eCSSProperty_paint_order => {
                    Ok(PropertyId::Longhand(LonghandId::PaintOrder))
                }
                
    nsCSSPropertyID::eCSSProperty__moz_context_properties => {
                    Ok(PropertyId::Longhand(LonghandId::MozContextProperties))
                }
                
    nsCSSPropertyID::eCSSProperty_dominant_baseline => {
                    Ok(PropertyId::Longhand(LonghandId::DominantBaseline))
                }
                
    nsCSSPropertyID::eCSSProperty_vector_effect => {
                    Ok(PropertyId::Longhand(LonghandId::VectorEffect))
                }
                
    nsCSSPropertyID::eCSSProperty_stop_color => {
                    Ok(PropertyId::Longhand(LonghandId::StopColor))
                }
                
    nsCSSPropertyID::eCSSProperty_stop_opacity => {
                    Ok(PropertyId::Longhand(LonghandId::StopOpacity))
                }
                
    nsCSSPropertyID::eCSSProperty_flood_color => {
                    Ok(PropertyId::Longhand(LonghandId::FloodColor))
                }
                
    nsCSSPropertyID::eCSSProperty_flood_opacity => {
                    Ok(PropertyId::Longhand(LonghandId::FloodOpacity))
                }
                
    nsCSSPropertyID::eCSSProperty_lighting_color => {
                    Ok(PropertyId::Longhand(LonghandId::LightingColor))
                }
                
    nsCSSPropertyID::eCSSProperty_mask_type => {
                    Ok(PropertyId::Longhand(LonghandId::MaskType))
                }
                
    nsCSSPropertyID::eCSSProperty_clip_path => {
                    Ok(PropertyId::Longhand(LonghandId::ClipPath))
                }
                
    nsCSSPropertyID::eCSSProperty_mask_mode => {
                    Ok(PropertyId::Longhand(LonghandId::MaskMode))
                }
                
    nsCSSPropertyID::eCSSProperty_mask_repeat => {
                    Ok(PropertyId::Longhand(LonghandId::MaskRepeat))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_WebkitMaskRepeat => {
                        Ok(PropertyId::Longhand(LonghandId::MaskRepeat))
                    }
                
    nsCSSPropertyID::eCSSProperty_mask_position_x => {
                    Ok(PropertyId::Longhand(LonghandId::MaskPositionX))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_WebkitMaskPositionX => {
                        Ok(PropertyId::Longhand(LonghandId::MaskPositionX))
                    }
                
    nsCSSPropertyID::eCSSProperty_mask_position_y => {
                    Ok(PropertyId::Longhand(LonghandId::MaskPositionY))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_WebkitMaskPositionY => {
                        Ok(PropertyId::Longhand(LonghandId::MaskPositionY))
                    }
                
    nsCSSPropertyID::eCSSProperty_mask_clip => {
                    Ok(PropertyId::Longhand(LonghandId::MaskClip))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_WebkitMaskClip => {
                        Ok(PropertyId::Longhand(LonghandId::MaskClip))
                    }
                
    nsCSSPropertyID::eCSSProperty_mask_origin => {
                    Ok(PropertyId::Longhand(LonghandId::MaskOrigin))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_WebkitMaskOrigin => {
                        Ok(PropertyId::Longhand(LonghandId::MaskOrigin))
                    }
                
    nsCSSPropertyID::eCSSProperty_mask_size => {
                    Ok(PropertyId::Longhand(LonghandId::MaskSize))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_WebkitMaskSize => {
                        Ok(PropertyId::Longhand(LonghandId::MaskSize))
                    }
                
    nsCSSPropertyID::eCSSProperty_mask_composite => {
                    Ok(PropertyId::Longhand(LonghandId::MaskComposite))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_WebkitMaskComposite => {
                        Ok(PropertyId::Longhand(LonghandId::MaskComposite))
                    }
                
    nsCSSPropertyID::eCSSProperty_mask_image => {
                    Ok(PropertyId::Longhand(LonghandId::MaskImage))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_WebkitMaskImage => {
                        Ok(PropertyId::Longhand(LonghandId::MaskImage))
                    }
                
    nsCSSPropertyID::eCSSProperty__moz_box_align => {
                    Ok(PropertyId::Longhand(LonghandId::MozBoxAlign))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_WebkitBoxAlign => {
                        Ok(PropertyId::Longhand(LonghandId::MozBoxAlign))
                    }
                
    nsCSSPropertyID::eCSSProperty__moz_box_direction => {
                    Ok(PropertyId::Longhand(LonghandId::MozBoxDirection))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_WebkitBoxDirection => {
                        Ok(PropertyId::Longhand(LonghandId::MozBoxDirection))
                    }
                
    nsCSSPropertyID::eCSSProperty__moz_box_flex => {
                    Ok(PropertyId::Longhand(LonghandId::MozBoxFlex))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_WebkitBoxFlex => {
                        Ok(PropertyId::Longhand(LonghandId::MozBoxFlex))
                    }
                
    nsCSSPropertyID::eCSSProperty__moz_box_orient => {
                    Ok(PropertyId::Longhand(LonghandId::MozBoxOrient))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_WebkitBoxOrient => {
                        Ok(PropertyId::Longhand(LonghandId::MozBoxOrient))
                    }
                
    nsCSSPropertyID::eCSSProperty__moz_box_pack => {
                    Ok(PropertyId::Longhand(LonghandId::MozBoxPack))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_WebkitBoxPack => {
                        Ok(PropertyId::Longhand(LonghandId::MozBoxPack))
                    }
                
    nsCSSPropertyID::eCSSProperty__moz_stack_sizing => {
                    Ok(PropertyId::Longhand(LonghandId::MozStackSizing))
                }
                
    nsCSSPropertyID::eCSSProperty__moz_box_ordinal_group => {
                    Ok(PropertyId::Longhand(LonghandId::MozBoxOrdinalGroup))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_WebkitBoxOrdinalGroup => {
                        Ok(PropertyId::Longhand(LonghandId::MozBoxOrdinalGroup))
                    }
                
    nsCSSPropertyID::eCSSProperty_background => {
                    Ok(PropertyId::Shorthand(ShorthandId::Background))
                }
                
    nsCSSPropertyID::eCSSProperty_background_position => {
                    Ok(PropertyId::Shorthand(ShorthandId::BackgroundPosition))
                }
                
    nsCSSPropertyID::eCSSProperty_border_color => {
                    Ok(PropertyId::Shorthand(ShorthandId::BorderColor))
                }
                
    nsCSSPropertyID::eCSSProperty_border_style => {
                    Ok(PropertyId::Shorthand(ShorthandId::BorderStyle))
                }
                
    nsCSSPropertyID::eCSSProperty_border_width => {
                    Ok(PropertyId::Shorthand(ShorthandId::BorderWidth))
                }
                
    nsCSSPropertyID::eCSSProperty_border_top => {
                    Ok(PropertyId::Shorthand(ShorthandId::BorderTop))
                }
                
    nsCSSPropertyID::eCSSProperty_border_left => {
                    Ok(PropertyId::Shorthand(ShorthandId::BorderLeft))
                }
                
    nsCSSPropertyID::eCSSProperty_border_bottom => {
                    Ok(PropertyId::Shorthand(ShorthandId::BorderBottom))
                }
                
    nsCSSPropertyID::eCSSProperty_border_right => {
                    Ok(PropertyId::Shorthand(ShorthandId::BorderRight))
                }
                
    nsCSSPropertyID::eCSSProperty_border_block_start => {
                    Ok(PropertyId::Shorthand(ShorthandId::BorderBlockStart))
                }
                
    nsCSSPropertyID::eCSSProperty_border_block_end => {
                    Ok(PropertyId::Shorthand(ShorthandId::BorderBlockEnd))
                }
                
    nsCSSPropertyID::eCSSProperty_border_inline_start => {
                    Ok(PropertyId::Shorthand(ShorthandId::BorderInlineStart))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_MozBorderStart => {
                        Ok(PropertyId::Shorthand(ShorthandId::BorderInlineStart))
                    }
                
    nsCSSPropertyID::eCSSProperty_border_inline_end => {
                    Ok(PropertyId::Shorthand(ShorthandId::BorderInlineEnd))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_MozBorderEnd => {
                        Ok(PropertyId::Shorthand(ShorthandId::BorderInlineEnd))
                    }
                
    nsCSSPropertyID::eCSSProperty_border => {
                    Ok(PropertyId::Shorthand(ShorthandId::Border))
                }
                
    nsCSSPropertyID::eCSSProperty_border_radius => {
                    Ok(PropertyId::Shorthand(ShorthandId::BorderRadius))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_WebkitBorderRadius => {
                        Ok(PropertyId::Shorthand(ShorthandId::BorderRadius))
                    }
                
    nsCSSPropertyID::eCSSProperty_border_image => {
                    Ok(PropertyId::Shorthand(ShorthandId::BorderImage))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_MozBorderImage => {
                        Ok(PropertyId::Shorthand(ShorthandId::BorderImage))
                    }
                    
    nsCSSPropertyID::eCSSPropertyAlias_WebkitBorderImage => {
                        Ok(PropertyId::Shorthand(ShorthandId::BorderImage))
                    }
                
    nsCSSPropertyID::eCSSProperty_overflow => {
                    Ok(PropertyId::Shorthand(ShorthandId::Overflow))
                }
                
    nsCSSPropertyID::eCSSProperty_transition => {
                    Ok(PropertyId::Shorthand(ShorthandId::Transition))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_MozTransition => {
                        Ok(PropertyId::Shorthand(ShorthandId::Transition))
                    }
                    
    nsCSSPropertyID::eCSSPropertyAlias_WebkitTransition => {
                        Ok(PropertyId::Shorthand(ShorthandId::Transition))
                    }
                
    nsCSSPropertyID::eCSSProperty_animation => {
                    Ok(PropertyId::Shorthand(ShorthandId::Animation))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_MozAnimation => {
                        Ok(PropertyId::Shorthand(ShorthandId::Animation))
                    }
                    
    nsCSSPropertyID::eCSSPropertyAlias_WebkitAnimation => {
                        Ok(PropertyId::Shorthand(ShorthandId::Animation))
                    }
                
    nsCSSPropertyID::eCSSProperty_scroll_snap_type => {
                    Ok(PropertyId::Shorthand(ShorthandId::ScrollSnapType))
                }
                
    nsCSSPropertyID::eCSSProperty__moz_transform => {
                    Ok(PropertyId::Shorthand(ShorthandId::MozTransform))
                }
                
    nsCSSPropertyID::eCSSProperty_columns => {
                    Ok(PropertyId::Shorthand(ShorthandId::Columns))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_MozColumns => {
                        Ok(PropertyId::Shorthand(ShorthandId::Columns))
                    }
                
    nsCSSPropertyID::eCSSProperty_column_rule => {
                    Ok(PropertyId::Shorthand(ShorthandId::ColumnRule))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_MozColumnRule => {
                        Ok(PropertyId::Shorthand(ShorthandId::ColumnRule))
                    }
                
    nsCSSPropertyID::eCSSProperty_font => {
                    Ok(PropertyId::Shorthand(ShorthandId::Font))
                }
                
    nsCSSPropertyID::eCSSProperty_font_variant => {
                    Ok(PropertyId::Shorthand(ShorthandId::FontVariant))
                }
                
    nsCSSPropertyID::eCSSProperty_text_emphasis => {
                    Ok(PropertyId::Shorthand(ShorthandId::TextEmphasis))
                }
                
    nsCSSPropertyID::eCSSProperty__webkit_text_stroke => {
                    Ok(PropertyId::Shorthand(ShorthandId::WebkitTextStroke))
                }
                
    nsCSSPropertyID::eCSSProperty_list_style => {
                    Ok(PropertyId::Shorthand(ShorthandId::ListStyle))
                }
                
    nsCSSPropertyID::eCSSProperty_margin => {
                    Ok(PropertyId::Shorthand(ShorthandId::Margin))
                }
                
    nsCSSPropertyID::eCSSProperty_mask => {
                    Ok(PropertyId::Shorthand(ShorthandId::Mask))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_WebkitMask => {
                        Ok(PropertyId::Shorthand(ShorthandId::Mask))
                    }
                
    nsCSSPropertyID::eCSSProperty_mask_position => {
                    Ok(PropertyId::Shorthand(ShorthandId::MaskPosition))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_WebkitMaskPosition => {
                        Ok(PropertyId::Shorthand(ShorthandId::MaskPosition))
                    }
                
    nsCSSPropertyID::eCSSProperty_outline => {
                    Ok(PropertyId::Shorthand(ShorthandId::Outline))
                }
                
    nsCSSPropertyID::eCSSProperty__moz_outline_radius => {
                    Ok(PropertyId::Shorthand(ShorthandId::MozOutlineRadius))
                }
                
    nsCSSPropertyID::eCSSProperty_padding => {
                    Ok(PropertyId::Shorthand(ShorthandId::Padding))
                }
                
    nsCSSPropertyID::eCSSProperty_flex_flow => {
                    Ok(PropertyId::Shorthand(ShorthandId::FlexFlow))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_WebkitFlexFlow => {
                        Ok(PropertyId::Shorthand(ShorthandId::FlexFlow))
                    }
                
    nsCSSPropertyID::eCSSProperty_flex => {
                    Ok(PropertyId::Shorthand(ShorthandId::Flex))
                }
                    
    nsCSSPropertyID::eCSSPropertyAlias_WebkitFlex => {
                        Ok(PropertyId::Shorthand(ShorthandId::Flex))
                    }
                
    nsCSSPropertyID::eCSSProperty_grid_gap => {
                    Ok(PropertyId::Shorthand(ShorthandId::GridGap))
                }
                
    nsCSSPropertyID::eCSSProperty_grid_row => {
                    Ok(PropertyId::Shorthand(ShorthandId::GridRow))
                }
                
    nsCSSPropertyID::eCSSProperty_grid_column => {
                    Ok(PropertyId::Shorthand(ShorthandId::GridColumn))
                }
                
    nsCSSPropertyID::eCSSProperty_grid_area => {
                    Ok(PropertyId::Shorthand(ShorthandId::GridArea))
                }
                
    nsCSSPropertyID::eCSSProperty_grid_template => {
                    Ok(PropertyId::Shorthand(ShorthandId::GridTemplate))
                }
                
    nsCSSPropertyID::eCSSProperty_grid => {
                    Ok(PropertyId::Shorthand(ShorthandId::Grid))
                }
                
    nsCSSPropertyID::eCSSProperty_place_content => {
                    Ok(PropertyId::Shorthand(ShorthandId::PlaceContent))
                }
                
    nsCSSPropertyID::eCSSProperty_place_self => {
                    Ok(PropertyId::Shorthand(ShorthandId::PlaceSelf))
                }
                
    nsCSSPropertyID::eCSSProperty_place_items => {
                    Ok(PropertyId::Shorthand(ShorthandId::PlaceItems))
                }
                
    nsCSSPropertyID::eCSSProperty_marker => {
                    Ok(PropertyId::Shorthand(ShorthandId::Marker))
                }
                
    nsCSSPropertyID::eCSSProperty_text_decoration => {
                    Ok(PropertyId::Shorthand(ShorthandId::TextDecoration))
                }
                
    nsCSSPropertyID::eCSSProperty_all => {
                    Ok(PropertyId::Shorthand(ShorthandId::All))
                }
            _ => Err(())
        }
    }

    /// Returns an nsCSSPropertyID.
    #[cfg(feature = "gecko")]
    #[allow(non_upper_case_globals)]
    pub fn to_nscsspropertyid(&self) -> Result<nsCSSPropertyID, ()> {
        use gecko_bindings::structs::*;

        match *self {
            PropertyId::Longhand(id) => match id {
                    LonghandId::BackgroundColor => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_background_color)
                    }
                    LonghandId::BackgroundImage => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_background_image)
                    }
                    LonghandId::BackgroundPositionX => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_background_position_x)
                    }
                    LonghandId::BackgroundPositionY => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_background_position_y)
                    }
                    LonghandId::BackgroundRepeat => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_background_repeat)
                    }
                    LonghandId::BackgroundAttachment => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_background_attachment)
                    }
                    LonghandId::BackgroundClip => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_background_clip)
                    }
                    LonghandId::BackgroundOrigin => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_background_origin)
                    }
                    LonghandId::BackgroundSize => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_background_size)
                    }
                    LonghandId::BackgroundBlendMode => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_background_blend_mode)
                    }
                    LonghandId::BorderTopColor => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_border_top_color)
                    }
                    LonghandId::BorderTopStyle => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_border_top_style)
                    }
                    LonghandId::BorderTopWidth => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_border_top_width)
                    }
                    LonghandId::BorderLeftColor => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_border_left_color)
                    }
                    LonghandId::BorderLeftStyle => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_border_left_style)
                    }
                    LonghandId::BorderLeftWidth => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_border_left_width)
                    }
                    LonghandId::BorderBottomColor => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_border_bottom_color)
                    }
                    LonghandId::BorderBottomStyle => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_border_bottom_style)
                    }
                    LonghandId::BorderBottomWidth => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_border_bottom_width)
                    }
                    LonghandId::BorderRightColor => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_border_right_color)
                    }
                    LonghandId::BorderRightStyle => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_border_right_style)
                    }
                    LonghandId::BorderRightWidth => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_border_right_width)
                    }
                    LonghandId::BorderBlockStartColor => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_border_block_start_color)
                    }
                    LonghandId::BorderBlockStartStyle => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_border_block_start_style)
                    }
                    LonghandId::BorderBlockStartWidth => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_border_block_start_width)
                    }
                    LonghandId::BorderBlockEndColor => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_border_block_end_color)
                    }
                    LonghandId::BorderBlockEndStyle => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_border_block_end_style)
                    }
                    LonghandId::BorderBlockEndWidth => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_border_block_end_width)
                    }
                    LonghandId::BorderInlineStartColor => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_border_inline_start_color)
                    }
                    LonghandId::BorderInlineStartStyle => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_border_inline_start_style)
                    }
                    LonghandId::BorderInlineStartWidth => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_border_inline_start_width)
                    }
                    LonghandId::BorderInlineEndColor => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_border_inline_end_color)
                    }
                    LonghandId::BorderInlineEndStyle => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_border_inline_end_style)
                    }
                    LonghandId::BorderInlineEndWidth => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_border_inline_end_width)
                    }
                    LonghandId::BorderTopLeftRadius => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_border_top_left_radius)
                    }
                    LonghandId::BorderTopRightRadius => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_border_top_right_radius)
                    }
                    LonghandId::BorderBottomRightRadius => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_border_bottom_right_radius)
                    }
                    LonghandId::BorderBottomLeftRadius => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_border_bottom_left_radius)
                    }
                    LonghandId::MozBorderTopColors => {
                        Ok(
    nsCSSPropertyID::eCSSProperty__moz_border_top_colors)
                    }
                    LonghandId::MozBorderLeftColors => {
                        Ok(
    nsCSSPropertyID::eCSSProperty__moz_border_left_colors)
                    }
                    LonghandId::MozBorderBottomColors => {
                        Ok(
    nsCSSPropertyID::eCSSProperty__moz_border_bottom_colors)
                    }
                    LonghandId::MozBorderRightColors => {
                        Ok(
    nsCSSPropertyID::eCSSProperty__moz_border_right_colors)
                    }
                    LonghandId::BoxDecorationBreak => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_box_decoration_break)
                    }
                    LonghandId::MozFloatEdge => {
                        Ok(
    nsCSSPropertyID::eCSSProperty__moz_float_edge)
                    }
                    LonghandId::BorderImageSource => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_border_image_source)
                    }
                    LonghandId::BorderImageOutset => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_border_image_outset)
                    }
                    LonghandId::BorderImageRepeat => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_border_image_repeat)
                    }
                    LonghandId::BorderImageWidth => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_border_image_width)
                    }
                    LonghandId::BorderImageSlice => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_border_image_slice)
                    }
                    LonghandId::Display => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_display)
                    }
                    LonghandId::MozTopLayer => {
                        Ok(
    nsCSSPropertyID::eCSSProperty__moz_top_layer)
                    }
                    LonghandId::Position => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_position)
                    }
                    LonghandId::Float => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_float_)
                    }
                    LonghandId::Clear => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_clear)
                    }
                    LonghandId::VerticalAlign => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_vertical_align)
                    }
                    LonghandId::OverflowClipBox => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_overflow_clip_box)
                    }
                    LonghandId::OverflowX => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_overflow_x)
                    }
                    LonghandId::OverflowY => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_overflow_y)
                    }
                    LonghandId::TransitionDuration => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_transition_duration)
                    }
                    LonghandId::TransitionTimingFunction => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_transition_timing_function)
                    }
                    LonghandId::TransitionProperty => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_transition_property)
                    }
                    LonghandId::TransitionDelay => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_transition_delay)
                    }
                    LonghandId::AnimationName => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_animation_name)
                    }
                    LonghandId::AnimationDuration => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_animation_duration)
                    }
                    LonghandId::AnimationTimingFunction => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_animation_timing_function)
                    }
                    LonghandId::AnimationIterationCount => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_animation_iteration_count)
                    }
                    LonghandId::AnimationDirection => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_animation_direction)
                    }
                    LonghandId::AnimationPlayState => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_animation_play_state)
                    }
                    LonghandId::AnimationFillMode => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_animation_fill_mode)
                    }
                    LonghandId::AnimationDelay => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_animation_delay)
                    }
                    LonghandId::ScrollSnapPointsX => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_scroll_snap_points_x)
                    }
                    LonghandId::ScrollSnapPointsY => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_scroll_snap_points_y)
                    }
                    LonghandId::ScrollSnapDestination => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_scroll_snap_destination)
                    }
                    LonghandId::ScrollSnapCoordinate => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_scroll_snap_coordinate)
                    }
                    LonghandId::Transform => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_transform)
                    }
                    LonghandId::ScrollBehavior => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_scroll_behavior)
                    }
                    LonghandId::ScrollSnapTypeX => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_scroll_snap_type_x)
                    }
                    LonghandId::ScrollSnapTypeY => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_scroll_snap_type_y)
                    }
                    LonghandId::Isolation => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_isolation)
                    }
                    LonghandId::PageBreakAfter => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_page_break_after)
                    }
                    LonghandId::PageBreakBefore => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_page_break_before)
                    }
                    LonghandId::PageBreakInside => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_page_break_inside)
                    }
                    LonghandId::Resize => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_resize)
                    }
                    LonghandId::Perspective => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_perspective)
                    }
                    LonghandId::PerspectiveOrigin => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_perspective_origin)
                    }
                    LonghandId::BackfaceVisibility => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_backface_visibility)
                    }
                    LonghandId::TransformBox => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_transform_box)
                    }
                    LonghandId::TransformStyle => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_transform_style)
                    }
                    LonghandId::TransformOrigin => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_transform_origin)
                    }
                    LonghandId::Contain => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_contain)
                    }
                    LonghandId::MozAppearance => {
                        Ok(
    nsCSSPropertyID::eCSSProperty__moz_appearance)
                    }
                    LonghandId::MozBinding => {
                        Ok(
    nsCSSPropertyID::eCSSProperty__moz_binding)
                    }
                    LonghandId::MozOrient => {
                        Ok(
    nsCSSPropertyID::eCSSProperty__moz_orient)
                    }
                    LonghandId::WillChange => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_will_change)
                    }
                    LonghandId::ShapeOutside => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_shape_outside)
                    }
                    LonghandId::TouchAction => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_touch_action)
                    }
                    LonghandId::Color => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_color)
                    }
                    LonghandId::ColumnWidth => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_column_width)
                    }
                    LonghandId::ColumnCount => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_column_count)
                    }
                    LonghandId::ColumnGap => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_column_gap)
                    }
                    LonghandId::ColumnFill => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_column_fill)
                    }
                    LonghandId::ColumnRuleWidth => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_column_rule_width)
                    }
                    LonghandId::ColumnRuleColor => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_column_rule_color)
                    }
                    LonghandId::ColumnSpan => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_column_span)
                    }
                    LonghandId::ColumnRuleStyle => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_column_rule_style)
                    }
                    LonghandId::Content => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_content)
                    }
                    LonghandId::CounterIncrement => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_counter_increment)
                    }
                    LonghandId::CounterReset => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_counter_reset)
                    }
                    LonghandId::Opacity => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_opacity)
                    }
                    LonghandId::BoxShadow => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_box_shadow)
                    }
                    LonghandId::Clip => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_clip)
                    }
                    LonghandId::Filter => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_filter)
                    }
                    LonghandId::MixBlendMode => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_mix_blend_mode)
                    }
                    LonghandId::FontFamily => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_font_family)
                    }
                    LonghandId::FontStyle => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_font_style)
                    }
                    LonghandId::FontVariantCaps => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_font_variant_caps)
                    }
                    LonghandId::FontWeight => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_font_weight)
                    }
                    LonghandId::FontSize => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_font_size)
                    }
                    LonghandId::FontSizeAdjust => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_font_size_adjust)
                    }
                    LonghandId::FontSynthesis => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_font_synthesis)
                    }
                    LonghandId::FontStretch => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_font_stretch)
                    }
                    LonghandId::FontKerning => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_font_kerning)
                    }
                    LonghandId::FontVariantAlternates => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_font_variant_alternates)
                    }
                    LonghandId::FontVariantEastAsian => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_font_variant_east_asian)
                    }
                    LonghandId::FontVariantLigatures => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_font_variant_ligatures)
                    }
                    LonghandId::FontVariantNumeric => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_font_variant_numeric)
                    }
                    LonghandId::FontVariantPosition => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_font_variant_position)
                    }
                    LonghandId::FontFeatureSettings => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_font_feature_settings)
                    }
                    LonghandId::FontVariationSettings => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_font_variation_settings)
                    }
                    LonghandId::FontLanguageOverride => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_font_language_override)
                    }
                    LonghandId::XLang => {
                        Ok(
    nsCSSPropertyID::eCSSProperty__x_lang)
                    }
                    LonghandId::MozScriptSizeMultiplier => {
                        Ok(
    nsCSSPropertyID::eCSSProperty__moz_script_size_multiplier)
                    }
                    LonghandId::MozScriptLevel => {
                        Ok(
    nsCSSPropertyID::eCSSProperty__moz_script_level)
                    }
                    LonghandId::MozMathDisplay => {
                        Ok(
    nsCSSPropertyID::eCSSProperty__moz_math_display)
                    }
                    LonghandId::MozMathVariant => {
                        Ok(
    nsCSSPropertyID::eCSSProperty__moz_math_variant)
                    }
                    LonghandId::MozScriptMinSize => {
                        Ok(
    nsCSSPropertyID::eCSSProperty__moz_script_min_size)
                    }
                    LonghandId::XTextZoom => {
                        Ok(
    nsCSSPropertyID::eCSSProperty__x_text_zoom)
                    }
                    LonghandId::MozOsxFontSmoothing => {
                        Ok(
    nsCSSPropertyID::eCSSProperty__moz_osx_font_smoothing)
                    }
                    LonghandId::MozMinFontSizeRatio => {
                        Ok(
    nsCSSPropertyID::eCSSProperty__moz_min_font_size_ratio)
                    }
                    LonghandId::Visibility => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_visibility)
                    }
                    LonghandId::WritingMode => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_writing_mode)
                    }
                    LonghandId::Direction => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_direction)
                    }
                    LonghandId::TextOrientation => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_text_orientation)
                    }
                    LonghandId::ColorAdjust => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_color_adjust)
                    }
                    LonghandId::ImageRendering => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_image_rendering)
                    }
                    LonghandId::ImageOrientation => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_image_orientation)
                    }
                    LonghandId::BorderCollapse => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_border_collapse)
                    }
                    LonghandId::EmptyCells => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_empty_cells)
                    }
                    LonghandId::CaptionSide => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_caption_side)
                    }
                    LonghandId::BorderSpacing => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_border_spacing)
                    }
                    LonghandId::LineHeight => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_line_height)
                    }
                    LonghandId::TextTransform => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_text_transform)
                    }
                    LonghandId::Hyphens => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_hyphens)
                    }
                    LonghandId::MozTextSizeAdjust => {
                        Ok(
    nsCSSPropertyID::eCSSProperty__moz_text_size_adjust)
                    }
                    LonghandId::TextIndent => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_text_indent)
                    }
                    LonghandId::OverflowWrap => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_overflow_wrap)
                    }
                    LonghandId::WordBreak => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_word_break)
                    }
                    LonghandId::TextJustify => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_text_justify)
                    }
                    LonghandId::TextAlignLast => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_text_align_last)
                    }
                    LonghandId::TextAlign => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_text_align)
                    }
                    LonghandId::LetterSpacing => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_letter_spacing)
                    }
                    LonghandId::WordSpacing => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_word_spacing)
                    }
                    LonghandId::WhiteSpace => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_white_space)
                    }
                    LonghandId::TextShadow => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_text_shadow)
                    }
                    LonghandId::TextEmphasisStyle => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_text_emphasis_style)
                    }
                    LonghandId::TextEmphasisPosition => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_text_emphasis_position)
                    }
                    LonghandId::TextEmphasisColor => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_text_emphasis_color)
                    }
                    LonghandId::MozTabSize => {
                        Ok(
    nsCSSPropertyID::eCSSProperty__moz_tab_size)
                    }
                    LonghandId::WebkitTextFillColor => {
                        Ok(
    nsCSSPropertyID::eCSSProperty__webkit_text_fill_color)
                    }
                    LonghandId::WebkitTextStrokeColor => {
                        Ok(
    nsCSSPropertyID::eCSSProperty__webkit_text_stroke_color)
                    }
                    LonghandId::WebkitTextStrokeWidth => {
                        Ok(
    nsCSSPropertyID::eCSSProperty__webkit_text_stroke_width)
                    }
                    LonghandId::RubyAlign => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_ruby_align)
                    }
                    LonghandId::RubyPosition => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_ruby_position)
                    }
                    LonghandId::TextCombineUpright => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_text_combine_upright)
                    }
                    LonghandId::TextRendering => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_text_rendering)
                    }
                    LonghandId::MozControlCharacterVisibility => {
                        Ok(
    nsCSSPropertyID::eCSSProperty__moz_control_character_visibility)
                    }
                    LonghandId::ListStylePosition => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_list_style_position)
                    }
                    LonghandId::ListStyleType => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_list_style_type)
                    }
                    LonghandId::ListStyleImage => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_list_style_image)
                    }
                    LonghandId::Quotes => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_quotes)
                    }
                    LonghandId::MozImageRegion => {
                        Ok(
    nsCSSPropertyID::eCSSProperty__moz_image_region)
                    }
                    LonghandId::MarginTop => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_margin_top)
                    }
                    LonghandId::MarginLeft => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_margin_left)
                    }
                    LonghandId::MarginBottom => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_margin_bottom)
                    }
                    LonghandId::MarginRight => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_margin_right)
                    }
                    LonghandId::MarginBlockStart => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_margin_block_start)
                    }
                    LonghandId::MarginBlockEnd => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_margin_block_end)
                    }
                    LonghandId::MarginInlineStart => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_margin_inline_start)
                    }
                    LonghandId::MarginInlineEnd => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_margin_inline_end)
                    }
                    LonghandId::OutlineColor => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_outline_color)
                    }
                    LonghandId::OutlineStyle => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_outline_style)
                    }
                    LonghandId::OutlineWidth => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_outline_width)
                    }
                    LonghandId::MozOutlineRadiusTopleft => {
                        Ok(
    nsCSSPropertyID::eCSSProperty__moz_outline_radius_topleft)
                    }
                    LonghandId::MozOutlineRadiusTopright => {
                        Ok(
    nsCSSPropertyID::eCSSProperty__moz_outline_radius_topright)
                    }
                    LonghandId::MozOutlineRadiusBottomright => {
                        Ok(
    nsCSSPropertyID::eCSSProperty__moz_outline_radius_bottomright)
                    }
                    LonghandId::MozOutlineRadiusBottomleft => {
                        Ok(
    nsCSSPropertyID::eCSSProperty__moz_outline_radius_bottomleft)
                    }
                    LonghandId::OutlineOffset => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_outline_offset)
                    }
                    LonghandId::PaddingTop => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_padding_top)
                    }
                    LonghandId::PaddingLeft => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_padding_left)
                    }
                    LonghandId::PaddingBottom => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_padding_bottom)
                    }
                    LonghandId::PaddingRight => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_padding_right)
                    }
                    LonghandId::PaddingBlockStart => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_padding_block_start)
                    }
                    LonghandId::PaddingBlockEnd => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_padding_block_end)
                    }
                    LonghandId::PaddingInlineStart => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_padding_inline_start)
                    }
                    LonghandId::PaddingInlineEnd => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_padding_inline_end)
                    }
                    LonghandId::Cursor => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_cursor)
                    }
                    LonghandId::PointerEvents => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_pointer_events)
                    }
                    LonghandId::MozUserInput => {
                        Ok(
    nsCSSPropertyID::eCSSProperty__moz_user_input)
                    }
                    LonghandId::MozUserModify => {
                        Ok(
    nsCSSPropertyID::eCSSProperty__moz_user_modify)
                    }
                    LonghandId::MozUserFocus => {
                        Ok(
    nsCSSPropertyID::eCSSProperty__moz_user_focus)
                    }
                    LonghandId::CaretColor => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_caret_color)
                    }
                    LonghandId::Top => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_top)
                    }
                    LonghandId::Left => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_left)
                    }
                    LonghandId::Bottom => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_bottom)
                    }
                    LonghandId::Right => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_right)
                    }
                    LonghandId::OffsetBlockStart => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_offset_block_start)
                    }
                    LonghandId::OffsetBlockEnd => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_offset_block_end)
                    }
                    LonghandId::OffsetInlineStart => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_offset_inline_start)
                    }
                    LonghandId::OffsetInlineEnd => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_offset_inline_end)
                    }
                    LonghandId::ZIndex => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_z_index)
                    }
                    LonghandId::FlexDirection => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_flex_direction)
                    }
                    LonghandId::FlexWrap => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_flex_wrap)
                    }
                    LonghandId::JustifyContent => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_justify_content)
                    }
                    LonghandId::AlignContent => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_align_content)
                    }
                    LonghandId::AlignItems => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_align_items)
                    }
                    LonghandId::JustifyItems => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_justify_items)
                    }
                    LonghandId::FlexGrow => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_flex_grow)
                    }
                    LonghandId::FlexShrink => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_flex_shrink)
                    }
                    LonghandId::AlignSelf => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_align_self)
                    }
                    LonghandId::JustifySelf => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_justify_self)
                    }
                    LonghandId::Order => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_order)
                    }
                    LonghandId::FlexBasis => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_flex_basis)
                    }
                    LonghandId::Width => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_width)
                    }
                    LonghandId::MinWidth => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_min_width)
                    }
                    LonghandId::MaxWidth => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_max_width)
                    }
                    LonghandId::Height => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_height)
                    }
                    LonghandId::MinHeight => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_min_height)
                    }
                    LonghandId::MaxHeight => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_max_height)
                    }
                    LonghandId::BlockSize => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_block_size)
                    }
                    LonghandId::MinBlockSize => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_min_block_size)
                    }
                    LonghandId::MaxBlockSize => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_max_block_size)
                    }
                    LonghandId::InlineSize => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_inline_size)
                    }
                    LonghandId::MinInlineSize => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_min_inline_size)
                    }
                    LonghandId::MaxInlineSize => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_max_inline_size)
                    }
                    LonghandId::BoxSizing => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_box_sizing)
                    }
                    LonghandId::ObjectFit => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_object_fit)
                    }
                    LonghandId::ObjectPosition => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_object_position)
                    }
                    LonghandId::GridRowGap => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_grid_row_gap)
                    }
                    LonghandId::GridRowStart => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_grid_row_start)
                    }
                    LonghandId::GridRowEnd => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_grid_row_end)
                    }
                    LonghandId::GridAutoRows => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_grid_auto_rows)
                    }
                    LonghandId::GridTemplateRows => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_grid_template_rows)
                    }
                    LonghandId::GridColumnGap => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_grid_column_gap)
                    }
                    LonghandId::GridColumnStart => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_grid_column_start)
                    }
                    LonghandId::GridColumnEnd => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_grid_column_end)
                    }
                    LonghandId::GridAutoColumns => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_grid_auto_columns)
                    }
                    LonghandId::GridTemplateColumns => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_grid_template_columns)
                    }
                    LonghandId::GridAutoFlow => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_grid_auto_flow)
                    }
                    LonghandId::GridTemplateAreas => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_grid_template_areas)
                    }
                    LonghandId::TableLayout => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_table_layout)
                    }
                    LonghandId::XSpan => {
                        Ok(
    nsCSSPropertyID::eCSSProperty__x_span)
                    }
                    LonghandId::TextOverflow => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_text_overflow)
                    }
                    LonghandId::UnicodeBidi => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_unicode_bidi)
                    }
                    LonghandId::TextDecorationLine => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_text_decoration_line)
                    }
                    LonghandId::TextDecorationStyle => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_text_decoration_style)
                    }
                    LonghandId::TextDecorationColor => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_text_decoration_color)
                    }
                    LonghandId::InitialLetter => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_initial_letter)
                    }
                    LonghandId::ImeMode => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_ime_mode)
                    }
                    LonghandId::MozUserSelect => {
                        Ok(
    nsCSSPropertyID::eCSSProperty__moz_user_select)
                    }
                    LonghandId::MozWindowDragging => {
                        Ok(
    nsCSSPropertyID::eCSSProperty__moz_window_dragging)
                    }
                    LonghandId::MozWindowShadow => {
                        Ok(
    nsCSSPropertyID::eCSSProperty__moz_window_shadow)
                    }
                    LonghandId::MozForceBrokenImageIcon => {
                        Ok(
    nsCSSPropertyID::eCSSProperty__moz_force_broken_image_icon)
                    }
                    LonghandId::TextAnchor => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_text_anchor)
                    }
                    LonghandId::ColorInterpolation => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_color_interpolation)
                    }
                    LonghandId::ColorInterpolationFilters => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_color_interpolation_filters)
                    }
                    LonghandId::Fill => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_fill)
                    }
                    LonghandId::FillOpacity => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_fill_opacity)
                    }
                    LonghandId::FillRule => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_fill_rule)
                    }
                    LonghandId::ShapeRendering => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_shape_rendering)
                    }
                    LonghandId::Stroke => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_stroke)
                    }
                    LonghandId::StrokeWidth => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_stroke_width)
                    }
                    LonghandId::StrokeLinecap => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_stroke_linecap)
                    }
                    LonghandId::StrokeLinejoin => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_stroke_linejoin)
                    }
                    LonghandId::StrokeMiterlimit => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_stroke_miterlimit)
                    }
                    LonghandId::StrokeOpacity => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_stroke_opacity)
                    }
                    LonghandId::StrokeDasharray => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_stroke_dasharray)
                    }
                    LonghandId::StrokeDashoffset => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_stroke_dashoffset)
                    }
                    LonghandId::ClipRule => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_clip_rule)
                    }
                    LonghandId::MarkerStart => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_marker_start)
                    }
                    LonghandId::MarkerMid => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_marker_mid)
                    }
                    LonghandId::MarkerEnd => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_marker_end)
                    }
                    LonghandId::PaintOrder => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_paint_order)
                    }
                    LonghandId::MozContextProperties => {
                        Ok(
    nsCSSPropertyID::eCSSProperty__moz_context_properties)
                    }
                    LonghandId::DominantBaseline => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_dominant_baseline)
                    }
                    LonghandId::VectorEffect => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_vector_effect)
                    }
                    LonghandId::StopColor => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_stop_color)
                    }
                    LonghandId::StopOpacity => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_stop_opacity)
                    }
                    LonghandId::FloodColor => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_flood_color)
                    }
                    LonghandId::FloodOpacity => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_flood_opacity)
                    }
                    LonghandId::LightingColor => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_lighting_color)
                    }
                    LonghandId::MaskType => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_mask_type)
                    }
                    LonghandId::ClipPath => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_clip_path)
                    }
                    LonghandId::MaskMode => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_mask_mode)
                    }
                    LonghandId::MaskRepeat => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_mask_repeat)
                    }
                    LonghandId::MaskPositionX => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_mask_position_x)
                    }
                    LonghandId::MaskPositionY => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_mask_position_y)
                    }
                    LonghandId::MaskClip => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_mask_clip)
                    }
                    LonghandId::MaskOrigin => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_mask_origin)
                    }
                    LonghandId::MaskSize => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_mask_size)
                    }
                    LonghandId::MaskComposite => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_mask_composite)
                    }
                    LonghandId::MaskImage => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_mask_image)
                    }
                    LonghandId::MozBoxAlign => {
                        Ok(
    nsCSSPropertyID::eCSSProperty__moz_box_align)
                    }
                    LonghandId::MozBoxDirection => {
                        Ok(
    nsCSSPropertyID::eCSSProperty__moz_box_direction)
                    }
                    LonghandId::MozBoxFlex => {
                        Ok(
    nsCSSPropertyID::eCSSProperty__moz_box_flex)
                    }
                    LonghandId::MozBoxOrient => {
                        Ok(
    nsCSSPropertyID::eCSSProperty__moz_box_orient)
                    }
                    LonghandId::MozBoxPack => {
                        Ok(
    nsCSSPropertyID::eCSSProperty__moz_box_pack)
                    }
                    LonghandId::MozStackSizing => {
                        Ok(
    nsCSSPropertyID::eCSSProperty__moz_stack_sizing)
                    }
                    LonghandId::MozBoxOrdinalGroup => {
                        Ok(
    nsCSSPropertyID::eCSSProperty__moz_box_ordinal_group)
                    }
            },
            PropertyId::Shorthand(id) => match id {
                    ShorthandId::Background => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_background)
                    }
                    ShorthandId::BackgroundPosition => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_background_position)
                    }
                    ShorthandId::BorderColor => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_border_color)
                    }
                    ShorthandId::BorderStyle => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_border_style)
                    }
                    ShorthandId::BorderWidth => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_border_width)
                    }
                    ShorthandId::BorderTop => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_border_top)
                    }
                    ShorthandId::BorderLeft => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_border_left)
                    }
                    ShorthandId::BorderBottom => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_border_bottom)
                    }
                    ShorthandId::BorderRight => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_border_right)
                    }
                    ShorthandId::BorderBlockStart => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_border_block_start)
                    }
                    ShorthandId::BorderBlockEnd => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_border_block_end)
                    }
                    ShorthandId::BorderInlineStart => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_border_inline_start)
                    }
                    ShorthandId::BorderInlineEnd => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_border_inline_end)
                    }
                    ShorthandId::Border => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_border)
                    }
                    ShorthandId::BorderRadius => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_border_radius)
                    }
                    ShorthandId::BorderImage => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_border_image)
                    }
                    ShorthandId::Overflow => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_overflow)
                    }
                    ShorthandId::Transition => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_transition)
                    }
                    ShorthandId::Animation => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_animation)
                    }
                    ShorthandId::ScrollSnapType => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_scroll_snap_type)
                    }
                    ShorthandId::MozTransform => {
                        Ok(
    nsCSSPropertyID::eCSSProperty__moz_transform)
                    }
                    ShorthandId::Columns => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_columns)
                    }
                    ShorthandId::ColumnRule => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_column_rule)
                    }
                    ShorthandId::Font => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_font)
                    }
                    ShorthandId::FontVariant => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_font_variant)
                    }
                    ShorthandId::TextEmphasis => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_text_emphasis)
                    }
                    ShorthandId::WebkitTextStroke => {
                        Ok(
    nsCSSPropertyID::eCSSProperty__webkit_text_stroke)
                    }
                    ShorthandId::ListStyle => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_list_style)
                    }
                    ShorthandId::Margin => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_margin)
                    }
                    ShorthandId::Mask => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_mask)
                    }
                    ShorthandId::MaskPosition => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_mask_position)
                    }
                    ShorthandId::Outline => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_outline)
                    }
                    ShorthandId::MozOutlineRadius => {
                        Ok(
    nsCSSPropertyID::eCSSProperty__moz_outline_radius)
                    }
                    ShorthandId::Padding => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_padding)
                    }
                    ShorthandId::FlexFlow => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_flex_flow)
                    }
                    ShorthandId::Flex => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_flex)
                    }
                    ShorthandId::GridGap => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_grid_gap)
                    }
                    ShorthandId::GridRow => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_grid_row)
                    }
                    ShorthandId::GridColumn => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_grid_column)
                    }
                    ShorthandId::GridArea => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_grid_area)
                    }
                    ShorthandId::GridTemplate => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_grid_template)
                    }
                    ShorthandId::Grid => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_grid)
                    }
                    ShorthandId::PlaceContent => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_place_content)
                    }
                    ShorthandId::PlaceSelf => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_place_self)
                    }
                    ShorthandId::PlaceItems => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_place_items)
                    }
                    ShorthandId::Marker => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_marker)
                    }
                    ShorthandId::TextDecoration => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_text_decoration)
                    }
                    ShorthandId::All => {
                        Ok(
    nsCSSPropertyID::eCSSProperty_all)
                    }
            },
            _ => Err(())
        }
    }

    /// Given this property id, get it either as a shorthand or as a
    /// `PropertyDeclarationId`.
    pub fn as_shorthand(&self) -> Result<ShorthandId, PropertyDeclarationId> {
        match *self {
            PropertyId::Shorthand(id) => Ok(id),
            PropertyId::Longhand(id) => Err(PropertyDeclarationId::Longhand(id)),
            PropertyId::Custom(ref name) => Err(PropertyDeclarationId::Custom(name)),
        }
    }

    /// Returns the name of the property without CSS escaping.
    pub fn name(&self) -> Cow<'static, str> {
        match *self {
            PropertyId::Shorthand(id) => id.name().into(),
            PropertyId::Longhand(id) => id.name().into(),
            PropertyId::Custom(ref name) => {
                use std::fmt::Write;
                let mut s = String::new();
                write!(&mut s, "--{}", name).unwrap();
                s.into()
            }
        }
    }

    fn check_allowed_in(
        &self,
        alias: Option<AliasId>,
        context: &PropertyParserContext,
    ) -> Result<(), ()> {
        let id: NonCustomPropertyId;
        if let Some(alias_id) = alias {
            id = alias_id.into();
        } else {
            match *self {
                // Custom properties are allowed everywhere
                PropertyId::Custom(_) => return Ok(()),

                PropertyId::Shorthand(shorthand_id) => id = shorthand_id.into(),
                PropertyId::Longhand(longhand_id) => id = longhand_id.into(),
            }
        }

        

        
static DISALLOWED_IN_KEYFRAME_BLOCK: NonCustomPropertyIdSet = NonCustomPropertyIdSet {
    
    storage: [0x0, 0x40000000, 0x3e, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x10000, 0x0, 0x0, 0x0, 0x0]
};

        
static DISALLOWED_IN_PAGE_RULE: NonCustomPropertyIdSet = NonCustomPropertyIdSet {
    
    storage: [0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xe01fffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xfbffffff, 0x3fff, 0x0, 0x0, 0x0]
};

        match context.rule_type {
            CssRuleType::Keyframe if DISALLOWED_IN_KEYFRAME_BLOCK.contains(id) => {
                return Err(());
            }
            CssRuleType::Page if DISALLOWED_IN_PAGE_RULE.contains(id) => {
                return Err(())
            }
            _ => {}
        }

        // For properties that are experimental but not internal, the pref will
        // control its availability in all sheets.   For properties that are
        // both experimental and internal, the pref only controls its
        // availability in non-UA sheets (and in UA sheets it is always available).
        
static INTERNAL: NonCustomPropertyIdSet = NonCustomPropertyIdSet {
    
    storage: [0x0, 0x840000, 0x0, 0x0, 0x59c, 0x0, 0x0, 0x0, 0x4010, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0]
};


            use gecko_bindings::structs::root::mozilla;
            static EXPERIMENTAL: NonCustomPropertyIdSet = NonCustomPropertyIdSet {
                
                storage: [
                        (0
                            | ((mozilla::SERVO_PREF_ENABLED_background_color as u32) << 0)
                            | ((mozilla::SERVO_PREF_ENABLED_background_image as u32) << 1)
                            | ((mozilla::SERVO_PREF_ENABLED_background_position_x as u32) << 2)
                            | ((mozilla::SERVO_PREF_ENABLED_background_position_y as u32) << 3)
                            | ((mozilla::SERVO_PREF_ENABLED_background_repeat as u32) << 4)
                            | ((mozilla::SERVO_PREF_ENABLED_background_attachment as u32) << 5)
                            | ((mozilla::SERVO_PREF_ENABLED_background_clip as u32) << 6)
                            | ((mozilla::SERVO_PREF_ENABLED_background_origin as u32) << 7)
                            | ((mozilla::SERVO_PREF_ENABLED_background_size as u32) << 8)
                            | ((mozilla::SERVO_PREF_ENABLED_background_blend_mode as u32) << 9)
                            | ((mozilla::SERVO_PREF_ENABLED_border_top_color as u32) << 10)
                            | ((mozilla::SERVO_PREF_ENABLED_border_top_style as u32) << 11)
                            | ((mozilla::SERVO_PREF_ENABLED_border_top_width as u32) << 12)
                            | ((mozilla::SERVO_PREF_ENABLED_border_left_color as u32) << 13)
                            | ((mozilla::SERVO_PREF_ENABLED_border_left_style as u32) << 14)
                            | ((mozilla::SERVO_PREF_ENABLED_border_left_width as u32) << 15)
                            | ((mozilla::SERVO_PREF_ENABLED_border_bottom_color as u32) << 16)
                            | ((mozilla::SERVO_PREF_ENABLED_border_bottom_style as u32) << 17)
                            | ((mozilla::SERVO_PREF_ENABLED_border_bottom_width as u32) << 18)
                            | ((mozilla::SERVO_PREF_ENABLED_border_right_color as u32) << 19)
                            | ((mozilla::SERVO_PREF_ENABLED_border_right_style as u32) << 20)
                            | ((mozilla::SERVO_PREF_ENABLED_border_right_width as u32) << 21)
                            | ((mozilla::SERVO_PREF_ENABLED_border_block_start_color as u32) << 22)
                            | ((mozilla::SERVO_PREF_ENABLED_border_block_start_style as u32) << 23)
                            | ((mozilla::SERVO_PREF_ENABLED_border_block_start_width as u32) << 24)
                            | ((mozilla::SERVO_PREF_ENABLED_border_block_end_color as u32) << 25)
                            | ((mozilla::SERVO_PREF_ENABLED_border_block_end_style as u32) << 26)
                            | ((mozilla::SERVO_PREF_ENABLED_border_block_end_width as u32) << 27)
                            | ((mozilla::SERVO_PREF_ENABLED_border_inline_start_color as u32) << 28)
                            | ((mozilla::SERVO_PREF_ENABLED_border_inline_start_style as u32) << 29)
                            | ((mozilla::SERVO_PREF_ENABLED_border_inline_start_width as u32) << 30)
                            | ((mozilla::SERVO_PREF_ENABLED_border_inline_end_color as u32) << 31)
                        ),
                        (0
                            | ((mozilla::SERVO_PREF_ENABLED_border_inline_end_style as u32) << 0)
                            | ((mozilla::SERVO_PREF_ENABLED_border_inline_end_width as u32) << 1)
                            | ((mozilla::SERVO_PREF_ENABLED_border_top_left_radius as u32) << 2)
                            | ((mozilla::SERVO_PREF_ENABLED_border_top_right_radius as u32) << 3)
                            | ((mozilla::SERVO_PREF_ENABLED_border_bottom_right_radius as u32) << 4)
                            | ((mozilla::SERVO_PREF_ENABLED_border_bottom_left_radius as u32) << 5)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_border_top_colors as u32) << 6)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_border_left_colors as u32) << 7)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_border_bottom_colors as u32) << 8)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_border_right_colors as u32) << 9)
                            | ((mozilla::SERVO_PREF_ENABLED_box_decoration_break as u32) << 10)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_float_edge as u32) << 11)
                            | ((mozilla::SERVO_PREF_ENABLED_border_image_source as u32) << 12)
                            | ((mozilla::SERVO_PREF_ENABLED_border_image_outset as u32) << 13)
                            | ((mozilla::SERVO_PREF_ENABLED_border_image_repeat as u32) << 14)
                            | ((mozilla::SERVO_PREF_ENABLED_border_image_width as u32) << 15)
                            | ((mozilla::SERVO_PREF_ENABLED_border_image_slice as u32) << 16)
                            | ((mozilla::SERVO_PREF_ENABLED_display as u32) << 17)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_top_layer as u32) << 18)
                            | ((mozilla::SERVO_PREF_ENABLED_position as u32) << 19)
                            | ((mozilla::SERVO_PREF_ENABLED_float_ as u32) << 20)
                            | ((mozilla::SERVO_PREF_ENABLED_clear as u32) << 21)
                            | ((mozilla::SERVO_PREF_ENABLED_vertical_align as u32) << 22)
                            | ((mozilla::SERVO_PREF_ENABLED_overflow_clip_box as u32) << 23)
                            | ((mozilla::SERVO_PREF_ENABLED_overflow_x as u32) << 24)
                            | ((mozilla::SERVO_PREF_ENABLED_overflow_y as u32) << 25)
                            | ((mozilla::SERVO_PREF_ENABLED_transition_duration as u32) << 26)
                            | ((mozilla::SERVO_PREF_ENABLED_transition_timing_function as u32) << 27)
                            | ((mozilla::SERVO_PREF_ENABLED_transition_property as u32) << 28)
                            | ((mozilla::SERVO_PREF_ENABLED_transition_delay as u32) << 29)
                            | ((mozilla::SERVO_PREF_ENABLED_animation_name as u32) << 30)
                            | ((mozilla::SERVO_PREF_ENABLED_animation_duration as u32) << 31)
                        ),
                        (0
                            | ((mozilla::SERVO_PREF_ENABLED_animation_timing_function as u32) << 0)
                            | ((mozilla::SERVO_PREF_ENABLED_animation_iteration_count as u32) << 1)
                            | ((mozilla::SERVO_PREF_ENABLED_animation_direction as u32) << 2)
                            | ((mozilla::SERVO_PREF_ENABLED_animation_play_state as u32) << 3)
                            | ((mozilla::SERVO_PREF_ENABLED_animation_fill_mode as u32) << 4)
                            | ((mozilla::SERVO_PREF_ENABLED_animation_delay as u32) << 5)
                            | ((mozilla::SERVO_PREF_ENABLED_scroll_snap_points_x as u32) << 6)
                            | ((mozilla::SERVO_PREF_ENABLED_scroll_snap_points_y as u32) << 7)
                            | ((mozilla::SERVO_PREF_ENABLED_scroll_snap_destination as u32) << 8)
                            | ((mozilla::SERVO_PREF_ENABLED_scroll_snap_coordinate as u32) << 9)
                            | ((mozilla::SERVO_PREF_ENABLED_transform as u32) << 10)
                            | ((mozilla::SERVO_PREF_ENABLED_scroll_behavior as u32) << 11)
                            | ((mozilla::SERVO_PREF_ENABLED_scroll_snap_type_x as u32) << 12)
                            | ((mozilla::SERVO_PREF_ENABLED_scroll_snap_type_y as u32) << 13)
                            | ((mozilla::SERVO_PREF_ENABLED_isolation as u32) << 14)
                            | ((mozilla::SERVO_PREF_ENABLED_page_break_after as u32) << 15)
                            | ((mozilla::SERVO_PREF_ENABLED_page_break_before as u32) << 16)
                            | ((mozilla::SERVO_PREF_ENABLED_page_break_inside as u32) << 17)
                            | ((mozilla::SERVO_PREF_ENABLED_resize as u32) << 18)
                            | ((mozilla::SERVO_PREF_ENABLED_perspective as u32) << 19)
                            | ((mozilla::SERVO_PREF_ENABLED_perspective_origin as u32) << 20)
                            | ((mozilla::SERVO_PREF_ENABLED_backface_visibility as u32) << 21)
                            | ((mozilla::SERVO_PREF_ENABLED_transform_box as u32) << 22)
                            | ((mozilla::SERVO_PREF_ENABLED_transform_style as u32) << 23)
                            | ((mozilla::SERVO_PREF_ENABLED_transform_origin as u32) << 24)
                            | ((mozilla::SERVO_PREF_ENABLED_contain as u32) << 25)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_appearance as u32) << 26)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_binding as u32) << 27)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_orient as u32) << 28)
                            | ((mozilla::SERVO_PREF_ENABLED_will_change as u32) << 29)
                            | ((mozilla::SERVO_PREF_ENABLED_shape_outside as u32) << 30)
                            | ((mozilla::SERVO_PREF_ENABLED_touch_action as u32) << 31)
                        ),
                        (0
                            | ((mozilla::SERVO_PREF_ENABLED_color as u32) << 0)
                            | ((mozilla::SERVO_PREF_ENABLED_column_width as u32) << 1)
                            | ((mozilla::SERVO_PREF_ENABLED_column_count as u32) << 2)
                            | ((mozilla::SERVO_PREF_ENABLED_column_gap as u32) << 3)
                            | ((mozilla::SERVO_PREF_ENABLED_column_fill as u32) << 4)
                            | ((mozilla::SERVO_PREF_ENABLED_column_rule_width as u32) << 5)
                            | ((mozilla::SERVO_PREF_ENABLED_column_rule_color as u32) << 6)
                            | ((mozilla::SERVO_PREF_ENABLED_column_span as u32) << 7)
                            | ((mozilla::SERVO_PREF_ENABLED_column_rule_style as u32) << 8)
                            | ((mozilla::SERVO_PREF_ENABLED_content as u32) << 9)
                            | ((mozilla::SERVO_PREF_ENABLED_counter_increment as u32) << 10)
                            | ((mozilla::SERVO_PREF_ENABLED_counter_reset as u32) << 11)
                            | ((mozilla::SERVO_PREF_ENABLED_opacity as u32) << 12)
                            | ((mozilla::SERVO_PREF_ENABLED_box_shadow as u32) << 13)
                            | ((mozilla::SERVO_PREF_ENABLED_clip as u32) << 14)
                            | ((mozilla::SERVO_PREF_ENABLED_filter as u32) << 15)
                            | ((mozilla::SERVO_PREF_ENABLED_mix_blend_mode as u32) << 16)
                            | ((mozilla::SERVO_PREF_ENABLED_font_family as u32) << 17)
                            | ((mozilla::SERVO_PREF_ENABLED_font_style as u32) << 18)
                            | ((mozilla::SERVO_PREF_ENABLED_font_variant_caps as u32) << 19)
                            | ((mozilla::SERVO_PREF_ENABLED_font_weight as u32) << 20)
                            | ((mozilla::SERVO_PREF_ENABLED_font_size as u32) << 21)
                            | ((mozilla::SERVO_PREF_ENABLED_font_size_adjust as u32) << 22)
                            | ((mozilla::SERVO_PREF_ENABLED_font_synthesis as u32) << 23)
                            | ((mozilla::SERVO_PREF_ENABLED_font_stretch as u32) << 24)
                            | ((mozilla::SERVO_PREF_ENABLED_font_kerning as u32) << 25)
                            | ((mozilla::SERVO_PREF_ENABLED_font_variant_alternates as u32) << 26)
                            | ((mozilla::SERVO_PREF_ENABLED_font_variant_east_asian as u32) << 27)
                            | ((mozilla::SERVO_PREF_ENABLED_font_variant_ligatures as u32) << 28)
                            | ((mozilla::SERVO_PREF_ENABLED_font_variant_numeric as u32) << 29)
                            | ((mozilla::SERVO_PREF_ENABLED_font_variant_position as u32) << 30)
                            | ((mozilla::SERVO_PREF_ENABLED_font_feature_settings as u32) << 31)
                        ),
                        (0
                            | ((mozilla::SERVO_PREF_ENABLED_font_variation_settings as u32) << 0)
                            | ((mozilla::SERVO_PREF_ENABLED_font_language_override as u32) << 1)
                            | ((mozilla::SERVO_PREF_ENABLED__x_lang as u32) << 2)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_script_size_multiplier as u32) << 3)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_script_level as u32) << 4)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_math_display as u32) << 5)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_math_variant as u32) << 6)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_script_min_size as u32) << 7)
                            | ((mozilla::SERVO_PREF_ENABLED__x_text_zoom as u32) << 8)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_osx_font_smoothing as u32) << 9)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_min_font_size_ratio as u32) << 10)
                            | ((mozilla::SERVO_PREF_ENABLED_visibility as u32) << 11)
                            | ((mozilla::SERVO_PREF_ENABLED_writing_mode as u32) << 12)
                            | ((mozilla::SERVO_PREF_ENABLED_direction as u32) << 13)
                            | ((mozilla::SERVO_PREF_ENABLED_text_orientation as u32) << 14)
                            | ((mozilla::SERVO_PREF_ENABLED_color_adjust as u32) << 15)
                            | ((mozilla::SERVO_PREF_ENABLED_image_rendering as u32) << 16)
                            | ((mozilla::SERVO_PREF_ENABLED_image_orientation as u32) << 17)
                            | ((mozilla::SERVO_PREF_ENABLED_border_collapse as u32) << 18)
                            | ((mozilla::SERVO_PREF_ENABLED_empty_cells as u32) << 19)
                            | ((mozilla::SERVO_PREF_ENABLED_caption_side as u32) << 20)
                            | ((mozilla::SERVO_PREF_ENABLED_border_spacing as u32) << 21)
                            | ((mozilla::SERVO_PREF_ENABLED_line_height as u32) << 22)
                            | ((mozilla::SERVO_PREF_ENABLED_text_transform as u32) << 23)
                            | ((mozilla::SERVO_PREF_ENABLED_hyphens as u32) << 24)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_text_size_adjust as u32) << 25)
                            | ((mozilla::SERVO_PREF_ENABLED_text_indent as u32) << 26)
                            | ((mozilla::SERVO_PREF_ENABLED_overflow_wrap as u32) << 27)
                            | ((mozilla::SERVO_PREF_ENABLED_word_break as u32) << 28)
                            | ((mozilla::SERVO_PREF_ENABLED_text_justify as u32) << 29)
                            | ((mozilla::SERVO_PREF_ENABLED_text_align_last as u32) << 30)
                            | ((mozilla::SERVO_PREF_ENABLED_text_align as u32) << 31)
                        ),
                        (0
                            | ((mozilla::SERVO_PREF_ENABLED_letter_spacing as u32) << 0)
                            | ((mozilla::SERVO_PREF_ENABLED_word_spacing as u32) << 1)
                            | ((mozilla::SERVO_PREF_ENABLED_white_space as u32) << 2)
                            | ((mozilla::SERVO_PREF_ENABLED_text_shadow as u32) << 3)
                            | ((mozilla::SERVO_PREF_ENABLED_text_emphasis_style as u32) << 4)
                            | ((mozilla::SERVO_PREF_ENABLED_text_emphasis_position as u32) << 5)
                            | ((mozilla::SERVO_PREF_ENABLED_text_emphasis_color as u32) << 6)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_tab_size as u32) << 7)
                            | ((mozilla::SERVO_PREF_ENABLED__webkit_text_fill_color as u32) << 8)
                            | ((mozilla::SERVO_PREF_ENABLED__webkit_text_stroke_color as u32) << 9)
                            | ((mozilla::SERVO_PREF_ENABLED__webkit_text_stroke_width as u32) << 10)
                            | ((mozilla::SERVO_PREF_ENABLED_ruby_align as u32) << 11)
                            | ((mozilla::SERVO_PREF_ENABLED_ruby_position as u32) << 12)
                            | ((mozilla::SERVO_PREF_ENABLED_text_combine_upright as u32) << 13)
                            | ((mozilla::SERVO_PREF_ENABLED_text_rendering as u32) << 14)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_control_character_visibility as u32) << 15)
                            | ((mozilla::SERVO_PREF_ENABLED_list_style_position as u32) << 16)
                            | ((mozilla::SERVO_PREF_ENABLED_list_style_type as u32) << 17)
                            | ((mozilla::SERVO_PREF_ENABLED_list_style_image as u32) << 18)
                            | ((mozilla::SERVO_PREF_ENABLED_quotes as u32) << 19)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_image_region as u32) << 20)
                            | ((mozilla::SERVO_PREF_ENABLED_margin_top as u32) << 21)
                            | ((mozilla::SERVO_PREF_ENABLED_margin_left as u32) << 22)
                            | ((mozilla::SERVO_PREF_ENABLED_margin_bottom as u32) << 23)
                            | ((mozilla::SERVO_PREF_ENABLED_margin_right as u32) << 24)
                            | ((mozilla::SERVO_PREF_ENABLED_margin_block_start as u32) << 25)
                            | ((mozilla::SERVO_PREF_ENABLED_margin_block_end as u32) << 26)
                            | ((mozilla::SERVO_PREF_ENABLED_margin_inline_start as u32) << 27)
                            | ((mozilla::SERVO_PREF_ENABLED_margin_inline_end as u32) << 28)
                            | ((mozilla::SERVO_PREF_ENABLED_outline_color as u32) << 29)
                            | ((mozilla::SERVO_PREF_ENABLED_outline_style as u32) << 30)
                            | ((mozilla::SERVO_PREF_ENABLED_outline_width as u32) << 31)
                        ),
                        (0
                            | ((mozilla::SERVO_PREF_ENABLED__moz_outline_radius_topleft as u32) << 0)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_outline_radius_topright as u32) << 1)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_outline_radius_bottomright as u32) << 2)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_outline_radius_bottomleft as u32) << 3)
                            | ((mozilla::SERVO_PREF_ENABLED_outline_offset as u32) << 4)
                            | ((mozilla::SERVO_PREF_ENABLED_padding_top as u32) << 5)
                            | ((mozilla::SERVO_PREF_ENABLED_padding_left as u32) << 6)
                            | ((mozilla::SERVO_PREF_ENABLED_padding_bottom as u32) << 7)
                            | ((mozilla::SERVO_PREF_ENABLED_padding_right as u32) << 8)
                            | ((mozilla::SERVO_PREF_ENABLED_padding_block_start as u32) << 9)
                            | ((mozilla::SERVO_PREF_ENABLED_padding_block_end as u32) << 10)
                            | ((mozilla::SERVO_PREF_ENABLED_padding_inline_start as u32) << 11)
                            | ((mozilla::SERVO_PREF_ENABLED_padding_inline_end as u32) << 12)
                            | ((mozilla::SERVO_PREF_ENABLED_cursor as u32) << 13)
                            | ((mozilla::SERVO_PREF_ENABLED_pointer_events as u32) << 14)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_user_input as u32) << 15)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_user_modify as u32) << 16)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_user_focus as u32) << 17)
                            | ((mozilla::SERVO_PREF_ENABLED_caret_color as u32) << 18)
                            | ((mozilla::SERVO_PREF_ENABLED_top as u32) << 19)
                            | ((mozilla::SERVO_PREF_ENABLED_left as u32) << 20)
                            | ((mozilla::SERVO_PREF_ENABLED_bottom as u32) << 21)
                            | ((mozilla::SERVO_PREF_ENABLED_right as u32) << 22)
                            | ((mozilla::SERVO_PREF_ENABLED_offset_block_start as u32) << 23)
                            | ((mozilla::SERVO_PREF_ENABLED_offset_block_end as u32) << 24)
                            | ((mozilla::SERVO_PREF_ENABLED_offset_inline_start as u32) << 25)
                            | ((mozilla::SERVO_PREF_ENABLED_offset_inline_end as u32) << 26)
                            | ((mozilla::SERVO_PREF_ENABLED_z_index as u32) << 27)
                            | ((mozilla::SERVO_PREF_ENABLED_flex_direction as u32) << 28)
                            | ((mozilla::SERVO_PREF_ENABLED_flex_wrap as u32) << 29)
                            | ((mozilla::SERVO_PREF_ENABLED_justify_content as u32) << 30)
                            | ((mozilla::SERVO_PREF_ENABLED_align_content as u32) << 31)
                        ),
                        (0
                            | ((mozilla::SERVO_PREF_ENABLED_align_items as u32) << 0)
                            | ((mozilla::SERVO_PREF_ENABLED_justify_items as u32) << 1)
                            | ((mozilla::SERVO_PREF_ENABLED_flex_grow as u32) << 2)
                            | ((mozilla::SERVO_PREF_ENABLED_flex_shrink as u32) << 3)
                            | ((mozilla::SERVO_PREF_ENABLED_align_self as u32) << 4)
                            | ((mozilla::SERVO_PREF_ENABLED_justify_self as u32) << 5)
                            | ((mozilla::SERVO_PREF_ENABLED_order as u32) << 6)
                            | ((mozilla::SERVO_PREF_ENABLED_flex_basis as u32) << 7)
                            | ((mozilla::SERVO_PREF_ENABLED_width as u32) << 8)
                            | ((mozilla::SERVO_PREF_ENABLED_min_width as u32) << 9)
                            | ((mozilla::SERVO_PREF_ENABLED_max_width as u32) << 10)
                            | ((mozilla::SERVO_PREF_ENABLED_height as u32) << 11)
                            | ((mozilla::SERVO_PREF_ENABLED_min_height as u32) << 12)
                            | ((mozilla::SERVO_PREF_ENABLED_max_height as u32) << 13)
                            | ((mozilla::SERVO_PREF_ENABLED_block_size as u32) << 14)
                            | ((mozilla::SERVO_PREF_ENABLED_min_block_size as u32) << 15)
                            | ((mozilla::SERVO_PREF_ENABLED_max_block_size as u32) << 16)
                            | ((mozilla::SERVO_PREF_ENABLED_inline_size as u32) << 17)
                            | ((mozilla::SERVO_PREF_ENABLED_min_inline_size as u32) << 18)
                            | ((mozilla::SERVO_PREF_ENABLED_max_inline_size as u32) << 19)
                            | ((mozilla::SERVO_PREF_ENABLED_box_sizing as u32) << 20)
                            | ((mozilla::SERVO_PREF_ENABLED_object_fit as u32) << 21)
                            | ((mozilla::SERVO_PREF_ENABLED_object_position as u32) << 22)
                            | ((mozilla::SERVO_PREF_ENABLED_grid_row_gap as u32) << 23)
                            | ((mozilla::SERVO_PREF_ENABLED_grid_row_start as u32) << 24)
                            | ((mozilla::SERVO_PREF_ENABLED_grid_row_end as u32) << 25)
                            | ((mozilla::SERVO_PREF_ENABLED_grid_auto_rows as u32) << 26)
                            | ((mozilla::SERVO_PREF_ENABLED_grid_template_rows as u32) << 27)
                            | ((mozilla::SERVO_PREF_ENABLED_grid_column_gap as u32) << 28)
                            | ((mozilla::SERVO_PREF_ENABLED_grid_column_start as u32) << 29)
                            | ((mozilla::SERVO_PREF_ENABLED_grid_column_end as u32) << 30)
                            | ((mozilla::SERVO_PREF_ENABLED_grid_auto_columns as u32) << 31)
                        ),
                        (0
                            | ((mozilla::SERVO_PREF_ENABLED_grid_template_columns as u32) << 0)
                            | ((mozilla::SERVO_PREF_ENABLED_grid_auto_flow as u32) << 1)
                            | ((mozilla::SERVO_PREF_ENABLED_grid_template_areas as u32) << 2)
                            | ((mozilla::SERVO_PREF_ENABLED_table_layout as u32) << 3)
                            | ((mozilla::SERVO_PREF_ENABLED__x_span as u32) << 4)
                            | ((mozilla::SERVO_PREF_ENABLED_text_overflow as u32) << 5)
                            | ((mozilla::SERVO_PREF_ENABLED_unicode_bidi as u32) << 6)
                            | ((mozilla::SERVO_PREF_ENABLED_text_decoration_line as u32) << 7)
                            | ((mozilla::SERVO_PREF_ENABLED_text_decoration_style as u32) << 8)
                            | ((mozilla::SERVO_PREF_ENABLED_text_decoration_color as u32) << 9)
                            | ((mozilla::SERVO_PREF_ENABLED_initial_letter as u32) << 10)
                            | ((mozilla::SERVO_PREF_ENABLED_ime_mode as u32) << 11)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_user_select as u32) << 12)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_window_dragging as u32) << 13)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_window_shadow as u32) << 14)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_force_broken_image_icon as u32) << 15)
                            | ((mozilla::SERVO_PREF_ENABLED_text_anchor as u32) << 16)
                            | ((mozilla::SERVO_PREF_ENABLED_color_interpolation as u32) << 17)
                            | ((mozilla::SERVO_PREF_ENABLED_color_interpolation_filters as u32) << 18)
                            | ((mozilla::SERVO_PREF_ENABLED_fill as u32) << 19)
                            | ((mozilla::SERVO_PREF_ENABLED_fill_opacity as u32) << 20)
                            | ((mozilla::SERVO_PREF_ENABLED_fill_rule as u32) << 21)
                            | ((mozilla::SERVO_PREF_ENABLED_shape_rendering as u32) << 22)
                            | ((mozilla::SERVO_PREF_ENABLED_stroke as u32) << 23)
                            | ((mozilla::SERVO_PREF_ENABLED_stroke_width as u32) << 24)
                            | ((mozilla::SERVO_PREF_ENABLED_stroke_linecap as u32) << 25)
                            | ((mozilla::SERVO_PREF_ENABLED_stroke_linejoin as u32) << 26)
                            | ((mozilla::SERVO_PREF_ENABLED_stroke_miterlimit as u32) << 27)
                            | ((mozilla::SERVO_PREF_ENABLED_stroke_opacity as u32) << 28)
                            | ((mozilla::SERVO_PREF_ENABLED_stroke_dasharray as u32) << 29)
                            | ((mozilla::SERVO_PREF_ENABLED_stroke_dashoffset as u32) << 30)
                            | ((mozilla::SERVO_PREF_ENABLED_clip_rule as u32) << 31)
                        ),
                        (0
                            | ((mozilla::SERVO_PREF_ENABLED_marker_start as u32) << 0)
                            | ((mozilla::SERVO_PREF_ENABLED_marker_mid as u32) << 1)
                            | ((mozilla::SERVO_PREF_ENABLED_marker_end as u32) << 2)
                            | ((mozilla::SERVO_PREF_ENABLED_paint_order as u32) << 3)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_context_properties as u32) << 4)
                            | ((mozilla::SERVO_PREF_ENABLED_dominant_baseline as u32) << 5)
                            | ((mozilla::SERVO_PREF_ENABLED_vector_effect as u32) << 6)
                            | ((mozilla::SERVO_PREF_ENABLED_stop_color as u32) << 7)
                            | ((mozilla::SERVO_PREF_ENABLED_stop_opacity as u32) << 8)
                            | ((mozilla::SERVO_PREF_ENABLED_flood_color as u32) << 9)
                            | ((mozilla::SERVO_PREF_ENABLED_flood_opacity as u32) << 10)
                            | ((mozilla::SERVO_PREF_ENABLED_lighting_color as u32) << 11)
                            | ((mozilla::SERVO_PREF_ENABLED_mask_type as u32) << 12)
                            | ((mozilla::SERVO_PREF_ENABLED_clip_path as u32) << 13)
                            | ((mozilla::SERVO_PREF_ENABLED_mask_mode as u32) << 14)
                            | ((mozilla::SERVO_PREF_ENABLED_mask_repeat as u32) << 15)
                            | ((mozilla::SERVO_PREF_ENABLED_mask_position_x as u32) << 16)
                            | ((mozilla::SERVO_PREF_ENABLED_mask_position_y as u32) << 17)
                            | ((mozilla::SERVO_PREF_ENABLED_mask_clip as u32) << 18)
                            | ((mozilla::SERVO_PREF_ENABLED_mask_origin as u32) << 19)
                            | ((mozilla::SERVO_PREF_ENABLED_mask_size as u32) << 20)
                            | ((mozilla::SERVO_PREF_ENABLED_mask_composite as u32) << 21)
                            | ((mozilla::SERVO_PREF_ENABLED_mask_image as u32) << 22)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_box_align as u32) << 23)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_box_direction as u32) << 24)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_box_flex as u32) << 25)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_box_orient as u32) << 26)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_box_pack as u32) << 27)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_stack_sizing as u32) << 28)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_box_ordinal_group as u32) << 29)
                            | ((mozilla::SERVO_PREF_ENABLED_background as u32) << 30)
                            | ((mozilla::SERVO_PREF_ENABLED_background_position as u32) << 31)
                        ),
                        (0
                            | ((mozilla::SERVO_PREF_ENABLED_border_color as u32) << 0)
                            | ((mozilla::SERVO_PREF_ENABLED_border_style as u32) << 1)
                            | ((mozilla::SERVO_PREF_ENABLED_border_width as u32) << 2)
                            | ((mozilla::SERVO_PREF_ENABLED_border_top as u32) << 3)
                            | ((mozilla::SERVO_PREF_ENABLED_border_left as u32) << 4)
                            | ((mozilla::SERVO_PREF_ENABLED_border_bottom as u32) << 5)
                            | ((mozilla::SERVO_PREF_ENABLED_border_right as u32) << 6)
                            | ((mozilla::SERVO_PREF_ENABLED_border_block_start as u32) << 7)
                            | ((mozilla::SERVO_PREF_ENABLED_border_block_end as u32) << 8)
                            | ((mozilla::SERVO_PREF_ENABLED_border_inline_start as u32) << 9)
                            | ((mozilla::SERVO_PREF_ENABLED_border_inline_end as u32) << 10)
                            | ((mozilla::SERVO_PREF_ENABLED_border as u32) << 11)
                            | ((mozilla::SERVO_PREF_ENABLED_border_radius as u32) << 12)
                            | ((mozilla::SERVO_PREF_ENABLED_border_image as u32) << 13)
                            | ((mozilla::SERVO_PREF_ENABLED_overflow as u32) << 14)
                            | ((mozilla::SERVO_PREF_ENABLED_transition as u32) << 15)
                            | ((mozilla::SERVO_PREF_ENABLED_animation as u32) << 16)
                            | ((mozilla::SERVO_PREF_ENABLED_scroll_snap_type as u32) << 17)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_transform as u32) << 18)
                            | ((mozilla::SERVO_PREF_ENABLED_columns as u32) << 19)
                            | ((mozilla::SERVO_PREF_ENABLED_column_rule as u32) << 20)
                            | ((mozilla::SERVO_PREF_ENABLED_font as u32) << 21)
                            | ((mozilla::SERVO_PREF_ENABLED_font_variant as u32) << 22)
                            | ((mozilla::SERVO_PREF_ENABLED_text_emphasis as u32) << 23)
                            | ((mozilla::SERVO_PREF_ENABLED__webkit_text_stroke as u32) << 24)
                            | ((mozilla::SERVO_PREF_ENABLED_list_style as u32) << 25)
                            | ((mozilla::SERVO_PREF_ENABLED_margin as u32) << 26)
                            | ((mozilla::SERVO_PREF_ENABLED_mask as u32) << 27)
                            | ((mozilla::SERVO_PREF_ENABLED_mask_position as u32) << 28)
                            | ((mozilla::SERVO_PREF_ENABLED_outline as u32) << 29)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_outline_radius as u32) << 30)
                            | ((mozilla::SERVO_PREF_ENABLED_padding as u32) << 31)
                        ),
                        (0
                            | ((mozilla::SERVO_PREF_ENABLED_flex_flow as u32) << 0)
                            | ((mozilla::SERVO_PREF_ENABLED_flex as u32) << 1)
                            | ((mozilla::SERVO_PREF_ENABLED_grid_gap as u32) << 2)
                            | ((mozilla::SERVO_PREF_ENABLED_grid_row as u32) << 3)
                            | ((mozilla::SERVO_PREF_ENABLED_grid_column as u32) << 4)
                            | ((mozilla::SERVO_PREF_ENABLED_grid_area as u32) << 5)
                            | ((mozilla::SERVO_PREF_ENABLED_grid_template as u32) << 6)
                            | ((mozilla::SERVO_PREF_ENABLED_grid as u32) << 7)
                            | ((mozilla::SERVO_PREF_ENABLED_place_content as u32) << 8)
                            | ((mozilla::SERVO_PREF_ENABLED_place_self as u32) << 9)
                            | ((mozilla::SERVO_PREF_ENABLED_place_items as u32) << 10)
                            | ((mozilla::SERVO_PREF_ENABLED_marker as u32) << 11)
                            | ((mozilla::SERVO_PREF_ENABLED_text_decoration as u32) << 12)
                            | ((mozilla::SERVO_PREF_ENABLED_all as u32) << 13)
                            | ((mozilla::SERVO_PREF_ENABLED__webkit_background_clip as u32) << 14)
                            | ((mozilla::SERVO_PREF_ENABLED__webkit_background_origin as u32) << 15)
                            | ((mozilla::SERVO_PREF_ENABLED__webkit_background_size as u32) << 16)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_border_start_color as u32) << 17)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_border_start_style as u32) << 18)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_border_start_width as u32) << 19)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_border_end_color as u32) << 20)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_border_end_style as u32) << 21)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_border_end_width as u32) << 22)
                            | ((mozilla::SERVO_PREF_ENABLED__webkit_border_top_left_radius as u32) << 23)
                            | ((mozilla::SERVO_PREF_ENABLED__webkit_border_top_right_radius as u32) << 24)
                            | ((mozilla::SERVO_PREF_ENABLED__webkit_border_bottom_right_radius as u32) << 25)
                            | ((mozilla::SERVO_PREF_ENABLED__webkit_border_bottom_left_radius as u32) << 26)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_transition_duration as u32) << 27)
                            | ((mozilla::SERVO_PREF_ENABLED__webkit_transition_duration as u32) << 28)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_transition_timing_function as u32) << 29)
                            | ((mozilla::SERVO_PREF_ENABLED__webkit_transition_timing_function as u32) << 30)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_transition_property as u32) << 31)
                        ),
                        (0
                            | ((mozilla::SERVO_PREF_ENABLED__webkit_transition_property as u32) << 0)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_transition_delay as u32) << 1)
                            | ((mozilla::SERVO_PREF_ENABLED__webkit_transition_delay as u32) << 2)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_animation_name as u32) << 3)
                            | ((mozilla::SERVO_PREF_ENABLED__webkit_animation_name as u32) << 4)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_animation_duration as u32) << 5)
                            | ((mozilla::SERVO_PREF_ENABLED__webkit_animation_duration as u32) << 6)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_animation_timing_function as u32) << 7)
                            | ((mozilla::SERVO_PREF_ENABLED__webkit_animation_timing_function as u32) << 8)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_animation_iteration_count as u32) << 9)
                            | ((mozilla::SERVO_PREF_ENABLED__webkit_animation_iteration_count as u32) << 10)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_animation_direction as u32) << 11)
                            | ((mozilla::SERVO_PREF_ENABLED__webkit_animation_direction as u32) << 12)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_animation_play_state as u32) << 13)
                            | ((mozilla::SERVO_PREF_ENABLED__webkit_animation_play_state as u32) << 14)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_animation_fill_mode as u32) << 15)
                            | ((mozilla::SERVO_PREF_ENABLED__webkit_animation_fill_mode as u32) << 16)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_animation_delay as u32) << 17)
                            | ((mozilla::SERVO_PREF_ENABLED__webkit_animation_delay as u32) << 18)
                            | ((mozilla::SERVO_PREF_ENABLED__webkit_transform as u32) << 19)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_perspective as u32) << 20)
                            | ((mozilla::SERVO_PREF_ENABLED__webkit_perspective as u32) << 21)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_perspective_origin as u32) << 22)
                            | ((mozilla::SERVO_PREF_ENABLED__webkit_perspective_origin as u32) << 23)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_backface_visibility as u32) << 24)
                            | ((mozilla::SERVO_PREF_ENABLED__webkit_backface_visibility as u32) << 25)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_transform_style as u32) << 26)
                            | ((mozilla::SERVO_PREF_ENABLED__webkit_transform_style as u32) << 27)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_transform_origin as u32) << 28)
                            | ((mozilla::SERVO_PREF_ENABLED__webkit_transform_origin as u32) << 29)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_column_width as u32) << 30)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_column_count as u32) << 31)
                        ),
                        (0
                            | ((mozilla::SERVO_PREF_ENABLED__moz_column_gap as u32) << 0)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_column_fill as u32) << 1)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_column_rule_width as u32) << 2)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_column_rule_color as u32) << 3)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_column_rule_style as u32) << 4)
                            | ((mozilla::SERVO_PREF_ENABLED__webkit_box_shadow as u32) << 5)
                            | ((mozilla::SERVO_PREF_ENABLED__webkit_filter as u32) << 6)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_font_feature_settings as u32) << 7)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_font_language_override as u32) << 8)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_hyphens as u32) << 9)
                            | ((mozilla::SERVO_PREF_ENABLED__webkit_text_size_adjust as u32) << 10)
                            | ((mozilla::SERVO_PREF_ENABLED_word_wrap as u32) << 11)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_margin_start as u32) << 12)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_margin_end as u32) << 13)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_padding_start as u32) << 14)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_padding_end as u32) << 15)
                            | ((mozilla::SERVO_PREF_ENABLED__webkit_flex_direction as u32) << 16)
                            | ((mozilla::SERVO_PREF_ENABLED__webkit_flex_wrap as u32) << 17)
                            | ((mozilla::SERVO_PREF_ENABLED__webkit_justify_content as u32) << 18)
                            | ((mozilla::SERVO_PREF_ENABLED__webkit_align_content as u32) << 19)
                            | ((mozilla::SERVO_PREF_ENABLED__webkit_align_items as u32) << 20)
                            | ((mozilla::SERVO_PREF_ENABLED__webkit_flex_grow as u32) << 21)
                            | ((mozilla::SERVO_PREF_ENABLED__webkit_flex_shrink as u32) << 22)
                            | ((mozilla::SERVO_PREF_ENABLED__webkit_align_self as u32) << 23)
                            | ((mozilla::SERVO_PREF_ENABLED__webkit_order as u32) << 24)
                            | ((mozilla::SERVO_PREF_ENABLED__webkit_flex_basis as u32) << 25)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_box_sizing as u32) << 26)
                            | ((mozilla::SERVO_PREF_ENABLED__webkit_box_sizing as u32) << 27)
                            | ((mozilla::SERVO_PREF_ENABLED__webkit_user_select as u32) << 28)
                            | ((mozilla::SERVO_PREF_ENABLED__webkit_mask_repeat as u32) << 29)
                            | ((mozilla::SERVO_PREF_ENABLED__webkit_mask_position_x as u32) << 30)
                            | ((mozilla::SERVO_PREF_ENABLED__webkit_mask_position_y as u32) << 31)
                        ),
                        (0
                            | ((mozilla::SERVO_PREF_ENABLED__webkit_mask_clip as u32) << 0)
                            | ((mozilla::SERVO_PREF_ENABLED__webkit_mask_origin as u32) << 1)
                            | ((mozilla::SERVO_PREF_ENABLED__webkit_mask_size as u32) << 2)
                            | ((mozilla::SERVO_PREF_ENABLED__webkit_mask_composite as u32) << 3)
                            | ((mozilla::SERVO_PREF_ENABLED__webkit_mask_image as u32) << 4)
                            | ((mozilla::SERVO_PREF_ENABLED__webkit_box_align as u32) << 5)
                            | ((mozilla::SERVO_PREF_ENABLED__webkit_box_direction as u32) << 6)
                            | ((mozilla::SERVO_PREF_ENABLED__webkit_box_flex as u32) << 7)
                            | ((mozilla::SERVO_PREF_ENABLED__webkit_box_orient as u32) << 8)
                            | ((mozilla::SERVO_PREF_ENABLED__webkit_box_pack as u32) << 9)
                            | ((mozilla::SERVO_PREF_ENABLED__webkit_box_ordinal_group as u32) << 10)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_border_start as u32) << 11)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_border_end as u32) << 12)
                            | ((mozilla::SERVO_PREF_ENABLED__webkit_border_radius as u32) << 13)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_border_image as u32) << 14)
                            | ((mozilla::SERVO_PREF_ENABLED__webkit_border_image as u32) << 15)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_transition as u32) << 16)
                            | ((mozilla::SERVO_PREF_ENABLED__webkit_transition as u32) << 17)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_animation as u32) << 18)
                            | ((mozilla::SERVO_PREF_ENABLED__webkit_animation as u32) << 19)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_columns as u32) << 20)
                            | ((mozilla::SERVO_PREF_ENABLED__moz_column_rule as u32) << 21)
                            | ((mozilla::SERVO_PREF_ENABLED__webkit_mask as u32) << 22)
                            | ((mozilla::SERVO_PREF_ENABLED__webkit_mask_position as u32) << 23)
                            | ((mozilla::SERVO_PREF_ENABLED__webkit_flex_flow as u32) << 24)
                            | ((mozilla::SERVO_PREF_ENABLED__webkit_flex as u32) << 25)
                        ),
                ]
            };

        let passes_pref_check = || {
                let id = match alias {
                    Some(alias_id) => alias_id.to_nscsspropertyid().unwrap(),
                    None => self.to_nscsspropertyid().unwrap(),
                };
                unsafe { structs::nsCSSProps_gPropertyEnabled[id as usize] }
        };

        if INTERNAL.contains(id) {
            if context.stylesheet_origin != Origin::UserAgent {
                if EXPERIMENTAL.contains(id) {
                    if !passes_pref_check() {
                        return Err(())
                    }
                } else {
                    return Err(())
                }
            }
        } else {
            if EXPERIMENTAL.contains(id) && !passes_pref_check() {
                return Err(());
            }
        }

        Ok(())
    }
}

/// Parsing Context for PropertyId.
pub struct PropertyParserContext {
    /// The Origin of the stylesheet, whether it's a user,
    /// author or user-agent stylesheet.
    pub stylesheet_origin: Origin,
    /// The current rule type, if any.
    pub rule_type: CssRuleType,
}

impl PropertyParserContext {
    /// Creates a PropertyParserContext with given stylesheet origin and rule type.
    pub fn new(context: &ParserContext) -> Self {
        Self {
            stylesheet_origin: context.stylesheet_origin,
            rule_type: context.rule_type(),
        }
    }
}

/// Servo's representation for a property declaration.
#[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
#[derive(Clone, PartialEq)]
pub enum PropertyDeclaration {
        /// background-color
            BackgroundColor(longhands::background_color::SpecifiedValue),
        /// background-image
            BackgroundImage(longhands::background_image::SpecifiedValue),
        /// background-position-x
            BackgroundPositionX(longhands::background_position_x::SpecifiedValue),
        /// background-position-y
            BackgroundPositionY(longhands::background_position_y::SpecifiedValue),
        /// background-repeat
            BackgroundRepeat(longhands::background_repeat::SpecifiedValue),
        /// background-attachment
            BackgroundAttachment(longhands::background_attachment::SpecifiedValue),
        /// background-clip
            BackgroundClip(longhands::background_clip::SpecifiedValue),
        /// background-origin
            BackgroundOrigin(longhands::background_origin::SpecifiedValue),
        /// background-size
            BackgroundSize(longhands::background_size::SpecifiedValue),
        /// background-blend-mode
            BackgroundBlendMode(longhands::background_blend_mode::SpecifiedValue),
        /// border-top-color
            BorderTopColor(longhands::border_top_color::SpecifiedValue),
        /// border-top-style
            BorderTopStyle(longhands::border_top_style::SpecifiedValue),
        /// border-top-width
            BorderTopWidth(longhands::border_top_width::SpecifiedValue),
        /// border-left-color
            BorderLeftColor(longhands::border_left_color::SpecifiedValue),
        /// border-left-style
            BorderLeftStyle(longhands::border_left_style::SpecifiedValue),
        /// border-left-width
            BorderLeftWidth(longhands::border_left_width::SpecifiedValue),
        /// border-bottom-color
            BorderBottomColor(longhands::border_bottom_color::SpecifiedValue),
        /// border-bottom-style
            BorderBottomStyle(longhands::border_bottom_style::SpecifiedValue),
        /// border-bottom-width
            BorderBottomWidth(longhands::border_bottom_width::SpecifiedValue),
        /// border-right-color
            BorderRightColor(longhands::border_right_color::SpecifiedValue),
        /// border-right-style
            BorderRightStyle(longhands::border_right_style::SpecifiedValue),
        /// border-right-width
            BorderRightWidth(longhands::border_right_width::SpecifiedValue),
        /// border-block-start-color
            BorderBlockStartColor(longhands::border_block_start_color::SpecifiedValue),
        /// border-block-start-style
            BorderBlockStartStyle(longhands::border_block_start_style::SpecifiedValue),
        /// border-block-start-width
            BorderBlockStartWidth(longhands::border_block_start_width::SpecifiedValue),
        /// border-block-end-color
            BorderBlockEndColor(longhands::border_block_end_color::SpecifiedValue),
        /// border-block-end-style
            BorderBlockEndStyle(longhands::border_block_end_style::SpecifiedValue),
        /// border-block-end-width
            BorderBlockEndWidth(longhands::border_block_end_width::SpecifiedValue),
        /// border-inline-start-color
            BorderInlineStartColor(longhands::border_inline_start_color::SpecifiedValue),
        /// border-inline-start-style
            BorderInlineStartStyle(longhands::border_inline_start_style::SpecifiedValue),
        /// border-inline-start-width
            BorderInlineStartWidth(longhands::border_inline_start_width::SpecifiedValue),
        /// border-inline-end-color
            BorderInlineEndColor(longhands::border_inline_end_color::SpecifiedValue),
        /// border-inline-end-style
            BorderInlineEndStyle(longhands::border_inline_end_style::SpecifiedValue),
        /// border-inline-end-width
            BorderInlineEndWidth(longhands::border_inline_end_width::SpecifiedValue),
        /// border-top-left-radius
            BorderTopLeftRadius(Box<longhands::border_top_left_radius::SpecifiedValue>),
        /// border-top-right-radius
            BorderTopRightRadius(Box<longhands::border_top_right_radius::SpecifiedValue>),
        /// border-bottom-right-radius
            BorderBottomRightRadius(Box<longhands::border_bottom_right_radius::SpecifiedValue>),
        /// border-bottom-left-radius
            BorderBottomLeftRadius(Box<longhands::border_bottom_left_radius::SpecifiedValue>),
        /// -moz-border-top-colors
            MozBorderTopColors(longhands::_moz_border_top_colors::SpecifiedValue),
        /// -moz-border-left-colors
            MozBorderLeftColors(longhands::_moz_border_left_colors::SpecifiedValue),
        /// -moz-border-bottom-colors
            MozBorderBottomColors(longhands::_moz_border_bottom_colors::SpecifiedValue),
        /// -moz-border-right-colors
            MozBorderRightColors(longhands::_moz_border_right_colors::SpecifiedValue),
        /// box-decoration-break
            BoxDecorationBreak(longhands::box_decoration_break::SpecifiedValue),
        /// -moz-float-edge
            MozFloatEdge(longhands::_moz_float_edge::SpecifiedValue),
        /// border-image-source
            BorderImageSource(Box<longhands::border_image_source::SpecifiedValue>),
        /// border-image-outset
            BorderImageOutset(Box<longhands::border_image_outset::SpecifiedValue>),
        /// border-image-repeat
            BorderImageRepeat(longhands::border_image_repeat::SpecifiedValue),
        /// border-image-width
            BorderImageWidth(Box<longhands::border_image_width::SpecifiedValue>),
        /// border-image-slice
            BorderImageSlice(Box<longhands::border_image_slice::SpecifiedValue>),
        /// display
            Display(longhands::display::SpecifiedValue),
        /// -moz-top-layer
            MozTopLayer(longhands::_moz_top_layer::SpecifiedValue),
        /// position
            Position(longhands::position::SpecifiedValue),
        /// float
            Float(longhands::float::SpecifiedValue),
        /// clear
            Clear(longhands::clear::SpecifiedValue),
        /// vertical-align
            VerticalAlign(longhands::vertical_align::SpecifiedValue),
        /// overflow-clip-box
            OverflowClipBox(longhands::overflow_clip_box::SpecifiedValue),
        /// overflow-x
            OverflowX(longhands::overflow_x::SpecifiedValue),
        /// overflow-y
            OverflowY(longhands::overflow_y::SpecifiedValue),
        /// transition-duration
            TransitionDuration(longhands::transition_duration::SpecifiedValue),
        /// transition-timing-function
            TransitionTimingFunction(longhands::transition_timing_function::SpecifiedValue),
        /// transition-property
            TransitionProperty(longhands::transition_property::SpecifiedValue),
        /// transition-delay
            TransitionDelay(longhands::transition_delay::SpecifiedValue),
        /// animation-name
            AnimationName(longhands::animation_name::SpecifiedValue),
        /// animation-duration
            AnimationDuration(longhands::animation_duration::SpecifiedValue),
        /// animation-timing-function
            AnimationTimingFunction(longhands::animation_timing_function::SpecifiedValue),
        /// animation-iteration-count
            AnimationIterationCount(longhands::animation_iteration_count::SpecifiedValue),
        /// animation-direction
            AnimationDirection(longhands::animation_direction::SpecifiedValue),
        /// animation-play-state
            AnimationPlayState(longhands::animation_play_state::SpecifiedValue),
        /// animation-fill-mode
            AnimationFillMode(longhands::animation_fill_mode::SpecifiedValue),
        /// animation-delay
            AnimationDelay(longhands::animation_delay::SpecifiedValue),
        /// scroll-snap-points-x
            ScrollSnapPointsX(longhands::scroll_snap_points_x::SpecifiedValue),
        /// scroll-snap-points-y
            ScrollSnapPointsY(longhands::scroll_snap_points_y::SpecifiedValue),
        /// scroll-snap-destination
            ScrollSnapDestination(Box<longhands::scroll_snap_destination::SpecifiedValue>),
        /// scroll-snap-coordinate
            ScrollSnapCoordinate(longhands::scroll_snap_coordinate::SpecifiedValue),
        /// transform
            Transform(longhands::transform::SpecifiedValue),
        /// scroll-behavior
            ScrollBehavior(longhands::scroll_behavior::SpecifiedValue),
        /// scroll-snap-type-x
            ScrollSnapTypeX(longhands::scroll_snap_type_x::SpecifiedValue),
        /// scroll-snap-type-y
            ScrollSnapTypeY(longhands::scroll_snap_type_y::SpecifiedValue),
        /// isolation
            Isolation(longhands::isolation::SpecifiedValue),
        /// page-break-after
            PageBreakAfter(longhands::page_break_after::SpecifiedValue),
        /// page-break-before
            PageBreakBefore(longhands::page_break_before::SpecifiedValue),
        /// page-break-inside
            PageBreakInside(longhands::page_break_inside::SpecifiedValue),
        /// resize
            Resize(longhands::resize::SpecifiedValue),
        /// perspective
            Perspective(longhands::perspective::SpecifiedValue),
        /// perspective-origin
            PerspectiveOrigin(Box<longhands::perspective_origin::SpecifiedValue>),
        /// backface-visibility
            BackfaceVisibility(longhands::backface_visibility::SpecifiedValue),
        /// transform-box
            TransformBox(longhands::transform_box::SpecifiedValue),
        /// transform-style
            TransformStyle(longhands::transform_style::SpecifiedValue),
        /// transform-origin
            TransformOrigin(Box<longhands::transform_origin::SpecifiedValue>),
        /// contain
            Contain(longhands::contain::SpecifiedValue),
        /// -moz-appearance
            MozAppearance(longhands::_moz_appearance::SpecifiedValue),
        /// -moz-binding
            MozBinding(Box<longhands::_moz_binding::SpecifiedValue>),
        /// -moz-orient
            MozOrient(longhands::_moz_orient::SpecifiedValue),
        /// will-change
            WillChange(longhands::will_change::SpecifiedValue),
        /// shape-outside
            ShapeOutside(Box<longhands::shape_outside::SpecifiedValue>),
        /// touch-action
            TouchAction(longhands::touch_action::SpecifiedValue),
        /// color
            Color(longhands::color::SpecifiedValue),
        /// column-width
            ColumnWidth(longhands::column_width::SpecifiedValue),
        /// column-count
            ColumnCount(longhands::column_count::SpecifiedValue),
        /// column-gap
            ColumnGap(longhands::column_gap::SpecifiedValue),
        /// column-fill
            ColumnFill(longhands::column_fill::SpecifiedValue),
        /// column-rule-width
            ColumnRuleWidth(longhands::column_rule_width::SpecifiedValue),
        /// column-rule-color
            ColumnRuleColor(longhands::column_rule_color::SpecifiedValue),
        /// column-span
            ColumnSpan(longhands::column_span::SpecifiedValue),
        /// column-rule-style
            ColumnRuleStyle(longhands::column_rule_style::SpecifiedValue),
        /// content
            Content(Box<longhands::content::SpecifiedValue>),
        /// counter-increment
            CounterIncrement(longhands::counter_increment::SpecifiedValue),
        /// counter-reset
            CounterReset(longhands::counter_reset::SpecifiedValue),
        /// opacity
            Opacity(longhands::opacity::SpecifiedValue),
        /// box-shadow
            BoxShadow(longhands::box_shadow::SpecifiedValue),
        /// clip
            Clip(Box<longhands::clip::SpecifiedValue>),
        /// filter
            Filter(longhands::filter::SpecifiedValue),
        /// mix-blend-mode
            MixBlendMode(longhands::mix_blend_mode::SpecifiedValue),
        /// font-family
            FontFamily(Box<longhands::font_family::SpecifiedValue>),
        /// font-style
            FontStyle(longhands::font_style::SpecifiedValue),
        /// font-variant-caps
            FontVariantCaps(longhands::font_variant_caps::SpecifiedValue),
        /// font-weight
            FontWeight(longhands::font_weight::SpecifiedValue),
        /// font-size
            FontSize(longhands::font_size::SpecifiedValue),
        /// font-size-adjust
            FontSizeAdjust(longhands::font_size_adjust::SpecifiedValue),
        /// font-synthesis
            FontSynthesis(longhands::font_synthesis::SpecifiedValue),
        /// font-stretch
            FontStretch(longhands::font_stretch::SpecifiedValue),
        /// font-kerning
            FontKerning(longhands::font_kerning::SpecifiedValue),
        /// font-variant-alternates
            FontVariantAlternates(longhands::font_variant_alternates::SpecifiedValue),
        /// font-variant-east-asian
            FontVariantEastAsian(longhands::font_variant_east_asian::SpecifiedValue),
        /// font-variant-ligatures
            FontVariantLigatures(longhands::font_variant_ligatures::SpecifiedValue),
        /// font-variant-numeric
            FontVariantNumeric(longhands::font_variant_numeric::SpecifiedValue),
        /// font-variant-position
            FontVariantPosition(longhands::font_variant_position::SpecifiedValue),
        /// font-feature-settings
            FontFeatureSettings(Box<longhands::font_feature_settings::SpecifiedValue>),
        /// font-variation-settings
            FontVariationSettings(longhands::font_variation_settings::SpecifiedValue),
        /// font-language-override
            FontLanguageOverride(Box<longhands::font_language_override::SpecifiedValue>),
        /// -x-lang
            XLang(longhands::_x_lang::SpecifiedValue),
        /// -moz-script-size-multiplier
            MozScriptSizeMultiplier(longhands::_moz_script_size_multiplier::SpecifiedValue),
        /// -moz-script-level
            MozScriptLevel(longhands::_moz_script_level::SpecifiedValue),
        /// -moz-math-display
            MozMathDisplay(longhands::_moz_math_display::SpecifiedValue),
        /// -moz-math-variant
            MozMathVariant(longhands::_moz_math_variant::SpecifiedValue),
        /// -moz-script-min-size
            MozScriptMinSize(longhands::_moz_script_min_size::SpecifiedValue),
        /// -x-text-zoom
            XTextZoom(longhands::_x_text_zoom::SpecifiedValue),
        /// -moz-osx-font-smoothing
            MozOsxFontSmoothing(longhands::_moz_osx_font_smoothing::SpecifiedValue),
        /// -moz-min-font-size-ratio
            MozMinFontSizeRatio(longhands::_moz_min_font_size_ratio::SpecifiedValue),
        /// visibility
            Visibility(longhands::visibility::SpecifiedValue),
        /// writing-mode
            WritingMode(longhands::writing_mode::SpecifiedValue),
        /// direction
            Direction(longhands::direction::SpecifiedValue),
        /// text-orientation
            TextOrientation(longhands::text_orientation::SpecifiedValue),
        /// color-adjust
            ColorAdjust(longhands::color_adjust::SpecifiedValue),
        /// image-rendering
            ImageRendering(longhands::image_rendering::SpecifiedValue),
        /// image-orientation
            ImageOrientation(longhands::image_orientation::SpecifiedValue),
        /// border-collapse
            BorderCollapse(longhands::border_collapse::SpecifiedValue),
        /// empty-cells
            EmptyCells(longhands::empty_cells::SpecifiedValue),
        /// caption-side
            CaptionSide(longhands::caption_side::SpecifiedValue),
        /// border-spacing
            BorderSpacing(Box<longhands::border_spacing::SpecifiedValue>),
        /// line-height
            LineHeight(longhands::line_height::SpecifiedValue),
        /// text-transform
            TextTransform(longhands::text_transform::SpecifiedValue),
        /// hyphens
            Hyphens(longhands::hyphens::SpecifiedValue),
        /// -moz-text-size-adjust
            MozTextSizeAdjust(longhands::_moz_text_size_adjust::SpecifiedValue),
        /// text-indent
            TextIndent(longhands::text_indent::SpecifiedValue),
        /// overflow-wrap
            OverflowWrap(longhands::overflow_wrap::SpecifiedValue),
        /// word-break
            WordBreak(longhands::word_break::SpecifiedValue),
        /// text-justify
            TextJustify(longhands::text_justify::SpecifiedValue),
        /// text-align-last
            TextAlignLast(longhands::text_align_last::SpecifiedValue),
        /// text-align
            TextAlign(longhands::text_align::SpecifiedValue),
        /// letter-spacing
            LetterSpacing(longhands::letter_spacing::SpecifiedValue),
        /// word-spacing
            WordSpacing(longhands::word_spacing::SpecifiedValue),
        /// white-space
            WhiteSpace(longhands::white_space::SpecifiedValue),
        /// text-shadow
            TextShadow(longhands::text_shadow::SpecifiedValue),
        /// text-emphasis-style
            TextEmphasisStyle(Box<longhands::text_emphasis_style::SpecifiedValue>),
        /// text-emphasis-position
            TextEmphasisPosition(longhands::text_emphasis_position::SpecifiedValue),
        /// text-emphasis-color
            TextEmphasisColor(longhands::text_emphasis_color::SpecifiedValue),
        /// -moz-tab-size
            MozTabSize(longhands::_moz_tab_size::SpecifiedValue),
        /// -webkit-text-fill-color
            WebkitTextFillColor(longhands::_webkit_text_fill_color::SpecifiedValue),
        /// -webkit-text-stroke-color
            WebkitTextStrokeColor(longhands::_webkit_text_stroke_color::SpecifiedValue),
        /// -webkit-text-stroke-width
            WebkitTextStrokeWidth(longhands::_webkit_text_stroke_width::SpecifiedValue),
        /// ruby-align
            RubyAlign(longhands::ruby_align::SpecifiedValue),
        /// ruby-position
            RubyPosition(longhands::ruby_position::SpecifiedValue),
        /// text-combine-upright
            TextCombineUpright(longhands::text_combine_upright::SpecifiedValue),
        /// text-rendering
            TextRendering(longhands::text_rendering::SpecifiedValue),
        /// -moz-control-character-visibility
            MozControlCharacterVisibility(longhands::_moz_control_character_visibility::SpecifiedValue),
        /// list-style-position
            ListStylePosition(longhands::list_style_position::SpecifiedValue),
        /// list-style-type
            ListStyleType(Box<longhands::list_style_type::SpecifiedValue>),
        /// list-style-image
            ListStyleImage(Box<longhands::list_style_image::SpecifiedValue>),
        /// quotes
            Quotes(longhands::quotes::SpecifiedValue),
        /// -moz-image-region
            MozImageRegion(Box<longhands::_moz_image_region::SpecifiedValue>),
        /// margin-top
            MarginTop(longhands::margin_top::SpecifiedValue),
        /// margin-left
            MarginLeft(longhands::margin_left::SpecifiedValue),
        /// margin-bottom
            MarginBottom(longhands::margin_bottom::SpecifiedValue),
        /// margin-right
            MarginRight(longhands::margin_right::SpecifiedValue),
        /// margin-block-start
            MarginBlockStart(longhands::margin_block_start::SpecifiedValue),
        /// margin-block-end
            MarginBlockEnd(longhands::margin_block_end::SpecifiedValue),
        /// margin-inline-start
            MarginInlineStart(longhands::margin_inline_start::SpecifiedValue),
        /// margin-inline-end
            MarginInlineEnd(longhands::margin_inline_end::SpecifiedValue),
        /// outline-color
            OutlineColor(longhands::outline_color::SpecifiedValue),
        /// outline-style
            OutlineStyle(longhands::outline_style::SpecifiedValue),
        /// outline-width
            OutlineWidth(longhands::outline_width::SpecifiedValue),
        /// -moz-outline-radius-topleft
            MozOutlineRadiusTopleft(Box<longhands::_moz_outline_radius_topleft::SpecifiedValue>),
        /// -moz-outline-radius-topright
            MozOutlineRadiusTopright(Box<longhands::_moz_outline_radius_topright::SpecifiedValue>),
        /// -moz-outline-radius-bottomright
            MozOutlineRadiusBottomright(Box<longhands::_moz_outline_radius_bottomright::SpecifiedValue>),
        /// -moz-outline-radius-bottomleft
            MozOutlineRadiusBottomleft(Box<longhands::_moz_outline_radius_bottomleft::SpecifiedValue>),
        /// outline-offset
            OutlineOffset(longhands::outline_offset::SpecifiedValue),
        /// padding-top
            PaddingTop(longhands::padding_top::SpecifiedValue),
        /// padding-left
            PaddingLeft(longhands::padding_left::SpecifiedValue),
        /// padding-bottom
            PaddingBottom(longhands::padding_bottom::SpecifiedValue),
        /// padding-right
            PaddingRight(longhands::padding_right::SpecifiedValue),
        /// padding-block-start
            PaddingBlockStart(longhands::padding_block_start::SpecifiedValue),
        /// padding-block-end
            PaddingBlockEnd(longhands::padding_block_end::SpecifiedValue),
        /// padding-inline-start
            PaddingInlineStart(longhands::padding_inline_start::SpecifiedValue),
        /// padding-inline-end
            PaddingInlineEnd(longhands::padding_inline_end::SpecifiedValue),
        /// cursor
            Cursor(Box<longhands::cursor::SpecifiedValue>),
        /// pointer-events
            PointerEvents(longhands::pointer_events::SpecifiedValue),
        /// -moz-user-input
            MozUserInput(longhands::_moz_user_input::SpecifiedValue),
        /// -moz-user-modify
            MozUserModify(longhands::_moz_user_modify::SpecifiedValue),
        /// -moz-user-focus
            MozUserFocus(longhands::_moz_user_focus::SpecifiedValue),
        /// caret-color
            CaretColor(Box<longhands::caret_color::SpecifiedValue>),
        /// top
            Top(longhands::top::SpecifiedValue),
        /// left
            Left(longhands::left::SpecifiedValue),
        /// bottom
            Bottom(longhands::bottom::SpecifiedValue),
        /// right
            Right(longhands::right::SpecifiedValue),
        /// offset-block-start
            OffsetBlockStart(longhands::offset_block_start::SpecifiedValue),
        /// offset-block-end
            OffsetBlockEnd(longhands::offset_block_end::SpecifiedValue),
        /// offset-inline-start
            OffsetInlineStart(longhands::offset_inline_start::SpecifiedValue),
        /// offset-inline-end
            OffsetInlineEnd(longhands::offset_inline_end::SpecifiedValue),
        /// z-index
            ZIndex(longhands::z_index::SpecifiedValue),
        /// flex-direction
            FlexDirection(longhands::flex_direction::SpecifiedValue),
        /// flex-wrap
            FlexWrap(longhands::flex_wrap::SpecifiedValue),
        /// justify-content
            JustifyContent(longhands::justify_content::SpecifiedValue),
        /// align-content
            AlignContent(longhands::align_content::SpecifiedValue),
        /// align-items
            AlignItems(longhands::align_items::SpecifiedValue),
        /// justify-items
            JustifyItems(longhands::justify_items::SpecifiedValue),
        /// flex-grow
            FlexGrow(longhands::flex_grow::SpecifiedValue),
        /// flex-shrink
            FlexShrink(longhands::flex_shrink::SpecifiedValue),
        /// align-self
            AlignSelf(longhands::align_self::SpecifiedValue),
        /// justify-self
            JustifySelf(longhands::justify_self::SpecifiedValue),
        /// order
            Order(longhands::order::SpecifiedValue),
        /// flex-basis
            FlexBasis(longhands::flex_basis::SpecifiedValue),
        /// width
            Width(longhands::width::SpecifiedValue),
        /// min-width
            MinWidth(longhands::min_width::SpecifiedValue),
        /// max-width
            MaxWidth(longhands::max_width::SpecifiedValue),
        /// height
            Height(longhands::height::SpecifiedValue),
        /// min-height
            MinHeight(longhands::min_height::SpecifiedValue),
        /// max-height
            MaxHeight(longhands::max_height::SpecifiedValue),
        /// block-size
            BlockSize(longhands::block_size::SpecifiedValue),
        /// min-block-size
            MinBlockSize(longhands::min_block_size::SpecifiedValue),
        /// max-block-size
            MaxBlockSize(longhands::max_block_size::SpecifiedValue),
        /// inline-size
            InlineSize(longhands::inline_size::SpecifiedValue),
        /// min-inline-size
            MinInlineSize(longhands::min_inline_size::SpecifiedValue),
        /// max-inline-size
            MaxInlineSize(longhands::max_inline_size::SpecifiedValue),
        /// box-sizing
            BoxSizing(longhands::box_sizing::SpecifiedValue),
        /// object-fit
            ObjectFit(longhands::object_fit::SpecifiedValue),
        /// object-position
            ObjectPosition(Box<longhands::object_position::SpecifiedValue>),
        /// grid-row-gap
            GridRowGap(longhands::grid_row_gap::SpecifiedValue),
        /// grid-row-start
            GridRowStart(Box<longhands::grid_row_start::SpecifiedValue>),
        /// grid-row-end
            GridRowEnd(Box<longhands::grid_row_end::SpecifiedValue>),
        /// grid-auto-rows
            GridAutoRows(Box<longhands::grid_auto_rows::SpecifiedValue>),
        /// grid-template-rows
            GridTemplateRows(Box<longhands::grid_template_rows::SpecifiedValue>),
        /// grid-column-gap
            GridColumnGap(longhands::grid_column_gap::SpecifiedValue),
        /// grid-column-start
            GridColumnStart(Box<longhands::grid_column_start::SpecifiedValue>),
        /// grid-column-end
            GridColumnEnd(Box<longhands::grid_column_end::SpecifiedValue>),
        /// grid-auto-columns
            GridAutoColumns(Box<longhands::grid_auto_columns::SpecifiedValue>),
        /// grid-template-columns
            GridTemplateColumns(Box<longhands::grid_template_columns::SpecifiedValue>),
        /// grid-auto-flow
            GridAutoFlow(longhands::grid_auto_flow::SpecifiedValue),
        /// grid-template-areas
            GridTemplateAreas(Box<longhands::grid_template_areas::SpecifiedValue>),
        /// table-layout
            TableLayout(longhands::table_layout::SpecifiedValue),
        /// -x-span
            XSpan(longhands::_x_span::SpecifiedValue),
        /// text-overflow
            TextOverflow(Box<longhands::text_overflow::SpecifiedValue>),
        /// unicode-bidi
            UnicodeBidi(longhands::unicode_bidi::SpecifiedValue),
        /// text-decoration-line
            TextDecorationLine(longhands::text_decoration_line::SpecifiedValue),
        /// text-decoration-style
            TextDecorationStyle(longhands::text_decoration_style::SpecifiedValue),
        /// text-decoration-color
            TextDecorationColor(longhands::text_decoration_color::SpecifiedValue),
        /// initial-letter
            InitialLetter(longhands::initial_letter::SpecifiedValue),
        /// ime-mode
            ImeMode(longhands::ime_mode::SpecifiedValue),
        /// -moz-user-select
            MozUserSelect(longhands::_moz_user_select::SpecifiedValue),
        /// -moz-window-dragging
            MozWindowDragging(longhands::_moz_window_dragging::SpecifiedValue),
        /// -moz-window-shadow
            MozWindowShadow(longhands::_moz_window_shadow::SpecifiedValue),
        /// -moz-force-broken-image-icon
            MozForceBrokenImageIcon(longhands::_moz_force_broken_image_icon::SpecifiedValue),
        /// text-anchor
            TextAnchor(longhands::text_anchor::SpecifiedValue),
        /// color-interpolation
            ColorInterpolation(longhands::color_interpolation::SpecifiedValue),
        /// color-interpolation-filters
            ColorInterpolationFilters(longhands::color_interpolation_filters::SpecifiedValue),
        /// fill
            Fill(Box<longhands::fill::SpecifiedValue>),
        /// fill-opacity
            FillOpacity(longhands::fill_opacity::SpecifiedValue),
        /// fill-rule
            FillRule(longhands::fill_rule::SpecifiedValue),
        /// shape-rendering
            ShapeRendering(longhands::shape_rendering::SpecifiedValue),
        /// stroke
            Stroke(Box<longhands::stroke::SpecifiedValue>),
        /// stroke-width
            StrokeWidth(Box<longhands::stroke_width::SpecifiedValue>),
        /// stroke-linecap
            StrokeLinecap(longhands::stroke_linecap::SpecifiedValue),
        /// stroke-linejoin
            StrokeLinejoin(longhands::stroke_linejoin::SpecifiedValue),
        /// stroke-miterlimit
            StrokeMiterlimit(longhands::stroke_miterlimit::SpecifiedValue),
        /// stroke-opacity
            StrokeOpacity(longhands::stroke_opacity::SpecifiedValue),
        /// stroke-dasharray
            StrokeDasharray(longhands::stroke_dasharray::SpecifiedValue),
        /// stroke-dashoffset
            StrokeDashoffset(Box<longhands::stroke_dashoffset::SpecifiedValue>),
        /// clip-rule
            ClipRule(longhands::clip_rule::SpecifiedValue),
        /// marker-start
            MarkerStart(Box<longhands::marker_start::SpecifiedValue>),
        /// marker-mid
            MarkerMid(Box<longhands::marker_mid::SpecifiedValue>),
        /// marker-end
            MarkerEnd(Box<longhands::marker_end::SpecifiedValue>),
        /// paint-order
            PaintOrder(longhands::paint_order::SpecifiedValue),
        /// -moz-context-properties
            MozContextProperties(longhands::_moz_context_properties::SpecifiedValue),
        /// dominant-baseline
            DominantBaseline(longhands::dominant_baseline::SpecifiedValue),
        /// vector-effect
            VectorEffect(longhands::vector_effect::SpecifiedValue),
        /// stop-color
            StopColor(longhands::stop_color::SpecifiedValue),
        /// stop-opacity
            StopOpacity(longhands::stop_opacity::SpecifiedValue),
        /// flood-color
            FloodColor(longhands::flood_color::SpecifiedValue),
        /// flood-opacity
            FloodOpacity(longhands::flood_opacity::SpecifiedValue),
        /// lighting-color
            LightingColor(longhands::lighting_color::SpecifiedValue),
        /// mask-type
            MaskType(longhands::mask_type::SpecifiedValue),
        /// clip-path
            ClipPath(Box<longhands::clip_path::SpecifiedValue>),
        /// mask-mode
            MaskMode(longhands::mask_mode::SpecifiedValue),
        /// mask-repeat
            MaskRepeat(longhands::mask_repeat::SpecifiedValue),
        /// mask-position-x
            MaskPositionX(longhands::mask_position_x::SpecifiedValue),
        /// mask-position-y
            MaskPositionY(longhands::mask_position_y::SpecifiedValue),
        /// mask-clip
            MaskClip(longhands::mask_clip::SpecifiedValue),
        /// mask-origin
            MaskOrigin(longhands::mask_origin::SpecifiedValue),
        /// mask-size
            MaskSize(longhands::mask_size::SpecifiedValue),
        /// mask-composite
            MaskComposite(longhands::mask_composite::SpecifiedValue),
        /// mask-image
            MaskImage(longhands::mask_image::SpecifiedValue),
        /// -moz-box-align
            MozBoxAlign(longhands::_moz_box_align::SpecifiedValue),
        /// -moz-box-direction
            MozBoxDirection(longhands::_moz_box_direction::SpecifiedValue),
        /// -moz-box-flex
            MozBoxFlex(longhands::_moz_box_flex::SpecifiedValue),
        /// -moz-box-orient
            MozBoxOrient(longhands::_moz_box_orient::SpecifiedValue),
        /// -moz-box-pack
            MozBoxPack(longhands::_moz_box_pack::SpecifiedValue),
        /// -moz-stack-sizing
            MozStackSizing(longhands::_moz_stack_sizing::SpecifiedValue),
        /// -moz-box-ordinal-group
            MozBoxOrdinalGroup(longhands::_moz_box_ordinal_group::SpecifiedValue),
    /// A css-wide keyword.
    CSSWideKeyword(LonghandId, CSSWideKeyword),
    /// An unparsed value that contains `var()` functions.
    WithVariables(
        LonghandId,
        #[cfg_attr(feature = "gecko", ignore_malloc_size_of = "XXX: how to handle this?")]
        Arc<UnparsedValue>
    ),
    /// A custom property declaration, with the property name and the declared
    /// value.
    Custom(::custom_properties::Name, DeclaredValueOwned<Box<::custom_properties::SpecifiedValue>>),
}

impl fmt::Debug for PropertyDeclaration {
    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
        self.id().to_css(f)?;
        f.write_str(": ")?;
        self.to_css(f)
    }
}

impl ToCss for PropertyDeclaration {
    fn to_css<W>(&self, dest: &mut W) -> fmt::Result
        where W: fmt::Write,
    {
        match *self {
                    PropertyDeclaration::BackgroundColor(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::BackgroundImage(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::BackgroundPositionX(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::BackgroundPositionY(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::BackgroundRepeat(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::BackgroundAttachment(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::BackgroundClip(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::BackgroundOrigin(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::BackgroundSize(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::BackgroundBlendMode(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::BorderTopColor(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::BorderTopStyle(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::BorderTopWidth(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::BorderLeftColor(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::BorderLeftStyle(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::BorderLeftWidth(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::BorderBottomColor(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::BorderBottomStyle(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::BorderBottomWidth(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::BorderRightColor(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::BorderRightStyle(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::BorderRightWidth(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::BorderBlockStartColor(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::BorderBlockStartStyle(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::BorderBlockStartWidth(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::BorderBlockEndColor(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::BorderBlockEndStyle(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::BorderBlockEndWidth(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::BorderInlineStartColor(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::BorderInlineStartStyle(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::BorderInlineStartWidth(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::BorderInlineEndColor(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::BorderInlineEndStyle(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::BorderInlineEndWidth(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::BorderTopLeftRadius(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::BorderTopRightRadius(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::BorderBottomRightRadius(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::BorderBottomLeftRadius(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MozBorderTopColors(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MozBorderLeftColors(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MozBorderBottomColors(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MozBorderRightColors(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::BoxDecorationBreak(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MozFloatEdge(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::BorderImageSource(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::BorderImageOutset(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::BorderImageRepeat(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::BorderImageWidth(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::BorderImageSlice(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::Display(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MozTopLayer(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::Position(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::Float(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::Clear(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::VerticalAlign(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::OverflowClipBox(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::OverflowX(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::OverflowY(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::TransitionDuration(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::TransitionTimingFunction(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::TransitionProperty(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::TransitionDelay(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::AnimationName(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::AnimationDuration(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::AnimationTimingFunction(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::AnimationIterationCount(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::AnimationDirection(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::AnimationPlayState(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::AnimationFillMode(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::AnimationDelay(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::ScrollSnapPointsX(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::ScrollSnapPointsY(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::ScrollSnapDestination(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::ScrollSnapCoordinate(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::Transform(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::ScrollBehavior(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::ScrollSnapTypeX(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::ScrollSnapTypeY(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::Isolation(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::PageBreakAfter(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::PageBreakBefore(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::PageBreakInside(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::Resize(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::Perspective(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::PerspectiveOrigin(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::BackfaceVisibility(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::TransformBox(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::TransformStyle(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::TransformOrigin(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::Contain(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MozAppearance(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MozBinding(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MozOrient(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::WillChange(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::ShapeOutside(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::TouchAction(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::Color(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::ColumnWidth(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::ColumnCount(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::ColumnGap(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::ColumnFill(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::ColumnRuleWidth(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::ColumnRuleColor(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::ColumnSpan(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::ColumnRuleStyle(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::Content(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::CounterIncrement(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::CounterReset(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::Opacity(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::BoxShadow(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::Clip(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::Filter(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MixBlendMode(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::FontFamily(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::FontStyle(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::FontVariantCaps(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::FontWeight(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::FontSize(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::FontSizeAdjust(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::FontSynthesis(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::FontStretch(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::FontKerning(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::FontVariantAlternates(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::FontVariantEastAsian(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::FontVariantLigatures(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::FontVariantNumeric(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::FontVariantPosition(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::FontFeatureSettings(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::FontVariationSettings(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::FontLanguageOverride(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::XLang(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MozScriptSizeMultiplier(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MozScriptLevel(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MozMathDisplay(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MozMathVariant(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MozScriptMinSize(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::XTextZoom(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MozOsxFontSmoothing(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MozMinFontSizeRatio(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::Visibility(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::WritingMode(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::Direction(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::TextOrientation(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::ColorAdjust(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::ImageRendering(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::ImageOrientation(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::BorderCollapse(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::EmptyCells(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::CaptionSide(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::BorderSpacing(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::LineHeight(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::TextTransform(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::Hyphens(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MozTextSizeAdjust(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::TextIndent(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::OverflowWrap(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::WordBreak(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::TextJustify(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::TextAlignLast(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::TextAlign(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::LetterSpacing(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::WordSpacing(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::WhiteSpace(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::TextShadow(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::TextEmphasisStyle(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::TextEmphasisPosition(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::TextEmphasisColor(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MozTabSize(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::WebkitTextFillColor(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::WebkitTextStrokeColor(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::WebkitTextStrokeWidth(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::RubyAlign(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::RubyPosition(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::TextCombineUpright(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::TextRendering(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MozControlCharacterVisibility(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::ListStylePosition(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::ListStyleType(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::ListStyleImage(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::Quotes(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MozImageRegion(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MarginTop(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MarginLeft(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MarginBottom(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MarginRight(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MarginBlockStart(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MarginBlockEnd(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MarginInlineStart(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MarginInlineEnd(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::OutlineColor(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::OutlineStyle(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::OutlineWidth(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MozOutlineRadiusTopleft(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MozOutlineRadiusTopright(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MozOutlineRadiusBottomright(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MozOutlineRadiusBottomleft(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::OutlineOffset(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::PaddingTop(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::PaddingLeft(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::PaddingBottom(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::PaddingRight(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::PaddingBlockStart(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::PaddingBlockEnd(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::PaddingInlineStart(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::PaddingInlineEnd(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::Cursor(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::PointerEvents(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MozUserInput(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MozUserModify(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MozUserFocus(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::CaretColor(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::Top(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::Left(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::Bottom(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::Right(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::OffsetBlockStart(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::OffsetBlockEnd(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::OffsetInlineStart(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::OffsetInlineEnd(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::ZIndex(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::FlexDirection(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::FlexWrap(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::JustifyContent(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::AlignContent(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::AlignItems(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::JustifyItems(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::FlexGrow(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::FlexShrink(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::AlignSelf(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::JustifySelf(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::Order(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::FlexBasis(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::Width(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MinWidth(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MaxWidth(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::Height(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MinHeight(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MaxHeight(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::BlockSize(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MinBlockSize(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MaxBlockSize(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::InlineSize(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MinInlineSize(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MaxInlineSize(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::BoxSizing(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::ObjectFit(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::ObjectPosition(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::GridRowGap(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::GridRowStart(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::GridRowEnd(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::GridAutoRows(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::GridTemplateRows(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::GridColumnGap(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::GridColumnStart(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::GridColumnEnd(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::GridAutoColumns(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::GridTemplateColumns(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::GridAutoFlow(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::GridTemplateAreas(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::TableLayout(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::XSpan(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::TextOverflow(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::UnicodeBidi(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::TextDecorationLine(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::TextDecorationStyle(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::TextDecorationColor(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::InitialLetter(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::ImeMode(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MozUserSelect(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MozWindowDragging(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MozWindowShadow(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MozForceBrokenImageIcon(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::TextAnchor(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::ColorInterpolation(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::ColorInterpolationFilters(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::Fill(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::FillOpacity(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::FillRule(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::ShapeRendering(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::Stroke(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::StrokeWidth(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::StrokeLinecap(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::StrokeLinejoin(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::StrokeMiterlimit(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::StrokeOpacity(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::StrokeDasharray(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::StrokeDashoffset(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::ClipRule(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MarkerStart(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MarkerMid(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MarkerEnd(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::PaintOrder(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MozContextProperties(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::DominantBaseline(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::VectorEffect(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::StopColor(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::StopOpacity(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::FloodColor(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::FloodOpacity(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::LightingColor(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MaskType(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::ClipPath(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MaskMode(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MaskRepeat(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MaskPositionX(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MaskPositionY(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MaskClip(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MaskOrigin(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MaskSize(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MaskComposite(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MaskImage(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MozBoxAlign(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MozBoxDirection(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MozBoxFlex(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MozBoxOrient(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MozBoxPack(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MozStackSizing(ref value) =>
                        value.to_css(dest),
                    PropertyDeclaration::MozBoxOrdinalGroup(ref value) =>
                        value.to_css(dest),
            PropertyDeclaration::CSSWideKeyword(_, keyword) => keyword.to_css(dest),
            PropertyDeclaration::WithVariables(_, ref with_variables) => {
                // https://drafts.csswg.org/css-variables/#variables-in-shorthands
                match with_variables.from_shorthand {
                    // Normally, we shouldn't be printing variables here if they came from
                    // shorthands. But we should allow properties that came from shorthand
                    // aliases. That also matches with the Gecko behavior.
                    Some(shorthand) if shorthand.flags().contains(SHORTHAND_ALIAS_PROPERTY) =>
                        dest.write_str(&*with_variables.css)?,
                    None => dest.write_str(&*with_variables.css)?,
                    _ => {},
                }
                Ok(())
            },
            PropertyDeclaration::Custom(_, ref value) => value.borrow().to_css(dest),
        }
    }
}

impl PropertyDeclaration {
    /// Given a property declaration, return the property declaration id.
    pub fn id(&self) -> PropertyDeclarationId {
        match *self {
            PropertyDeclaration::Custom(ref name, _) => {
                return PropertyDeclarationId::Custom(name)
            }
            PropertyDeclaration::CSSWideKeyword(id, _) |
            PropertyDeclaration::WithVariables(id, _) => {
                return PropertyDeclarationId::Longhand(id)
            }
            _ => {}
        }
        let longhand_id = match *self {
                PropertyDeclaration::BackgroundColor(..) => {
                    LonghandId::BackgroundColor
                }
                PropertyDeclaration::BackgroundImage(..) => {
                    LonghandId::BackgroundImage
                }
                PropertyDeclaration::BackgroundPositionX(..) => {
                    LonghandId::BackgroundPositionX
                }
                PropertyDeclaration::BackgroundPositionY(..) => {
                    LonghandId::BackgroundPositionY
                }
                PropertyDeclaration::BackgroundRepeat(..) => {
                    LonghandId::BackgroundRepeat
                }
                PropertyDeclaration::BackgroundAttachment(..) => {
                    LonghandId::BackgroundAttachment
                }
                PropertyDeclaration::BackgroundClip(..) => {
                    LonghandId::BackgroundClip
                }
                PropertyDeclaration::BackgroundOrigin(..) => {
                    LonghandId::BackgroundOrigin
                }
                PropertyDeclaration::BackgroundSize(..) => {
                    LonghandId::BackgroundSize
                }
                PropertyDeclaration::BackgroundBlendMode(..) => {
                    LonghandId::BackgroundBlendMode
                }
                PropertyDeclaration::BorderTopColor(..) => {
                    LonghandId::BorderTopColor
                }
                PropertyDeclaration::BorderTopStyle(..) => {
                    LonghandId::BorderTopStyle
                }
                PropertyDeclaration::BorderTopWidth(..) => {
                    LonghandId::BorderTopWidth
                }
                PropertyDeclaration::BorderLeftColor(..) => {
                    LonghandId::BorderLeftColor
                }
                PropertyDeclaration::BorderLeftStyle(..) => {
                    LonghandId::BorderLeftStyle
                }
                PropertyDeclaration::BorderLeftWidth(..) => {
                    LonghandId::BorderLeftWidth
                }
                PropertyDeclaration::BorderBottomColor(..) => {
                    LonghandId::BorderBottomColor
                }
                PropertyDeclaration::BorderBottomStyle(..) => {
                    LonghandId::BorderBottomStyle
                }
                PropertyDeclaration::BorderBottomWidth(..) => {
                    LonghandId::BorderBottomWidth
                }
                PropertyDeclaration::BorderRightColor(..) => {
                    LonghandId::BorderRightColor
                }
                PropertyDeclaration::BorderRightStyle(..) => {
                    LonghandId::BorderRightStyle
                }
                PropertyDeclaration::BorderRightWidth(..) => {
                    LonghandId::BorderRightWidth
                }
                PropertyDeclaration::BorderBlockStartColor(..) => {
                    LonghandId::BorderBlockStartColor
                }
                PropertyDeclaration::BorderBlockStartStyle(..) => {
                    LonghandId::BorderBlockStartStyle
                }
                PropertyDeclaration::BorderBlockStartWidth(..) => {
                    LonghandId::BorderBlockStartWidth
                }
                PropertyDeclaration::BorderBlockEndColor(..) => {
                    LonghandId::BorderBlockEndColor
                }
                PropertyDeclaration::BorderBlockEndStyle(..) => {
                    LonghandId::BorderBlockEndStyle
                }
                PropertyDeclaration::BorderBlockEndWidth(..) => {
                    LonghandId::BorderBlockEndWidth
                }
                PropertyDeclaration::BorderInlineStartColor(..) => {
                    LonghandId::BorderInlineStartColor
                }
                PropertyDeclaration::BorderInlineStartStyle(..) => {
                    LonghandId::BorderInlineStartStyle
                }
                PropertyDeclaration::BorderInlineStartWidth(..) => {
                    LonghandId::BorderInlineStartWidth
                }
                PropertyDeclaration::BorderInlineEndColor(..) => {
                    LonghandId::BorderInlineEndColor
                }
                PropertyDeclaration::BorderInlineEndStyle(..) => {
                    LonghandId::BorderInlineEndStyle
                }
                PropertyDeclaration::BorderInlineEndWidth(..) => {
                    LonghandId::BorderInlineEndWidth
                }
                PropertyDeclaration::BorderTopLeftRadius(..) => {
                    LonghandId::BorderTopLeftRadius
                }
                PropertyDeclaration::BorderTopRightRadius(..) => {
                    LonghandId::BorderTopRightRadius
                }
                PropertyDeclaration::BorderBottomRightRadius(..) => {
                    LonghandId::BorderBottomRightRadius
                }
                PropertyDeclaration::BorderBottomLeftRadius(..) => {
                    LonghandId::BorderBottomLeftRadius
                }
                PropertyDeclaration::MozBorderTopColors(..) => {
                    LonghandId::MozBorderTopColors
                }
                PropertyDeclaration::MozBorderLeftColors(..) => {
                    LonghandId::MozBorderLeftColors
                }
                PropertyDeclaration::MozBorderBottomColors(..) => {
                    LonghandId::MozBorderBottomColors
                }
                PropertyDeclaration::MozBorderRightColors(..) => {
                    LonghandId::MozBorderRightColors
                }
                PropertyDeclaration::BoxDecorationBreak(..) => {
                    LonghandId::BoxDecorationBreak
                }
                PropertyDeclaration::MozFloatEdge(..) => {
                    LonghandId::MozFloatEdge
                }
                PropertyDeclaration::BorderImageSource(..) => {
                    LonghandId::BorderImageSource
                }
                PropertyDeclaration::BorderImageOutset(..) => {
                    LonghandId::BorderImageOutset
                }
                PropertyDeclaration::BorderImageRepeat(..) => {
                    LonghandId::BorderImageRepeat
                }
                PropertyDeclaration::BorderImageWidth(..) => {
                    LonghandId::BorderImageWidth
                }
                PropertyDeclaration::BorderImageSlice(..) => {
                    LonghandId::BorderImageSlice
                }
                PropertyDeclaration::Display(..) => {
                    LonghandId::Display
                }
                PropertyDeclaration::MozTopLayer(..) => {
                    LonghandId::MozTopLayer
                }
                PropertyDeclaration::Position(..) => {
                    LonghandId::Position
                }
                PropertyDeclaration::Float(..) => {
                    LonghandId::Float
                }
                PropertyDeclaration::Clear(..) => {
                    LonghandId::Clear
                }
                PropertyDeclaration::VerticalAlign(..) => {
                    LonghandId::VerticalAlign
                }
                PropertyDeclaration::OverflowClipBox(..) => {
                    LonghandId::OverflowClipBox
                }
                PropertyDeclaration::OverflowX(..) => {
                    LonghandId::OverflowX
                }
                PropertyDeclaration::OverflowY(..) => {
                    LonghandId::OverflowY
                }
                PropertyDeclaration::TransitionDuration(..) => {
                    LonghandId::TransitionDuration
                }
                PropertyDeclaration::TransitionTimingFunction(..) => {
                    LonghandId::TransitionTimingFunction
                }
                PropertyDeclaration::TransitionProperty(..) => {
                    LonghandId::TransitionProperty
                }
                PropertyDeclaration::TransitionDelay(..) => {
                    LonghandId::TransitionDelay
                }
                PropertyDeclaration::AnimationName(..) => {
                    LonghandId::AnimationName
                }
                PropertyDeclaration::AnimationDuration(..) => {
                    LonghandId::AnimationDuration
                }
                PropertyDeclaration::AnimationTimingFunction(..) => {
                    LonghandId::AnimationTimingFunction
                }
                PropertyDeclaration::AnimationIterationCount(..) => {
                    LonghandId::AnimationIterationCount
                }
                PropertyDeclaration::AnimationDirection(..) => {
                    LonghandId::AnimationDirection
                }
                PropertyDeclaration::AnimationPlayState(..) => {
                    LonghandId::AnimationPlayState
                }
                PropertyDeclaration::AnimationFillMode(..) => {
                    LonghandId::AnimationFillMode
                }
                PropertyDeclaration::AnimationDelay(..) => {
                    LonghandId::AnimationDelay
                }
                PropertyDeclaration::ScrollSnapPointsX(..) => {
                    LonghandId::ScrollSnapPointsX
                }
                PropertyDeclaration::ScrollSnapPointsY(..) => {
                    LonghandId::ScrollSnapPointsY
                }
                PropertyDeclaration::ScrollSnapDestination(..) => {
                    LonghandId::ScrollSnapDestination
                }
                PropertyDeclaration::ScrollSnapCoordinate(..) => {
                    LonghandId::ScrollSnapCoordinate
                }
                PropertyDeclaration::Transform(..) => {
                    LonghandId::Transform
                }
                PropertyDeclaration::ScrollBehavior(..) => {
                    LonghandId::ScrollBehavior
                }
                PropertyDeclaration::ScrollSnapTypeX(..) => {
                    LonghandId::ScrollSnapTypeX
                }
                PropertyDeclaration::ScrollSnapTypeY(..) => {
                    LonghandId::ScrollSnapTypeY
                }
                PropertyDeclaration::Isolation(..) => {
                    LonghandId::Isolation
                }
                PropertyDeclaration::PageBreakAfter(..) => {
                    LonghandId::PageBreakAfter
                }
                PropertyDeclaration::PageBreakBefore(..) => {
                    LonghandId::PageBreakBefore
                }
                PropertyDeclaration::PageBreakInside(..) => {
                    LonghandId::PageBreakInside
                }
                PropertyDeclaration::Resize(..) => {
                    LonghandId::Resize
                }
                PropertyDeclaration::Perspective(..) => {
                    LonghandId::Perspective
                }
                PropertyDeclaration::PerspectiveOrigin(..) => {
                    LonghandId::PerspectiveOrigin
                }
                PropertyDeclaration::BackfaceVisibility(..) => {
                    LonghandId::BackfaceVisibility
                }
                PropertyDeclaration::TransformBox(..) => {
                    LonghandId::TransformBox
                }
                PropertyDeclaration::TransformStyle(..) => {
                    LonghandId::TransformStyle
                }
                PropertyDeclaration::TransformOrigin(..) => {
                    LonghandId::TransformOrigin
                }
                PropertyDeclaration::Contain(..) => {
                    LonghandId::Contain
                }
                PropertyDeclaration::MozAppearance(..) => {
                    LonghandId::MozAppearance
                }
                PropertyDeclaration::MozBinding(..) => {
                    LonghandId::MozBinding
                }
                PropertyDeclaration::MozOrient(..) => {
                    LonghandId::MozOrient
                }
                PropertyDeclaration::WillChange(..) => {
                    LonghandId::WillChange
                }
                PropertyDeclaration::ShapeOutside(..) => {
                    LonghandId::ShapeOutside
                }
                PropertyDeclaration::TouchAction(..) => {
                    LonghandId::TouchAction
                }
                PropertyDeclaration::Color(..) => {
                    LonghandId::Color
                }
                PropertyDeclaration::ColumnWidth(..) => {
                    LonghandId::ColumnWidth
                }
                PropertyDeclaration::ColumnCount(..) => {
                    LonghandId::ColumnCount
                }
                PropertyDeclaration::ColumnGap(..) => {
                    LonghandId::ColumnGap
                }
                PropertyDeclaration::ColumnFill(..) => {
                    LonghandId::ColumnFill
                }
                PropertyDeclaration::ColumnRuleWidth(..) => {
                    LonghandId::ColumnRuleWidth
                }
                PropertyDeclaration::ColumnRuleColor(..) => {
                    LonghandId::ColumnRuleColor
                }
                PropertyDeclaration::ColumnSpan(..) => {
                    LonghandId::ColumnSpan
                }
                PropertyDeclaration::ColumnRuleStyle(..) => {
                    LonghandId::ColumnRuleStyle
                }
                PropertyDeclaration::Content(..) => {
                    LonghandId::Content
                }
                PropertyDeclaration::CounterIncrement(..) => {
                    LonghandId::CounterIncrement
                }
                PropertyDeclaration::CounterReset(..) => {
                    LonghandId::CounterReset
                }
                PropertyDeclaration::Opacity(..) => {
                    LonghandId::Opacity
                }
                PropertyDeclaration::BoxShadow(..) => {
                    LonghandId::BoxShadow
                }
                PropertyDeclaration::Clip(..) => {
                    LonghandId::Clip
                }
                PropertyDeclaration::Filter(..) => {
                    LonghandId::Filter
                }
                PropertyDeclaration::MixBlendMode(..) => {
                    LonghandId::MixBlendMode
                }
                PropertyDeclaration::FontFamily(..) => {
                    LonghandId::FontFamily
                }
                PropertyDeclaration::FontStyle(..) => {
                    LonghandId::FontStyle
                }
                PropertyDeclaration::FontVariantCaps(..) => {
                    LonghandId::FontVariantCaps
                }
                PropertyDeclaration::FontWeight(..) => {
                    LonghandId::FontWeight
                }
                PropertyDeclaration::FontSize(..) => {
                    LonghandId::FontSize
                }
                PropertyDeclaration::FontSizeAdjust(..) => {
                    LonghandId::FontSizeAdjust
                }
                PropertyDeclaration::FontSynthesis(..) => {
                    LonghandId::FontSynthesis
                }
                PropertyDeclaration::FontStretch(..) => {
                    LonghandId::FontStretch
                }
                PropertyDeclaration::FontKerning(..) => {
                    LonghandId::FontKerning
                }
                PropertyDeclaration::FontVariantAlternates(..) => {
                    LonghandId::FontVariantAlternates
                }
                PropertyDeclaration::FontVariantEastAsian(..) => {
                    LonghandId::FontVariantEastAsian
                }
                PropertyDeclaration::FontVariantLigatures(..) => {
                    LonghandId::FontVariantLigatures
                }
                PropertyDeclaration::FontVariantNumeric(..) => {
                    LonghandId::FontVariantNumeric
                }
                PropertyDeclaration::FontVariantPosition(..) => {
                    LonghandId::FontVariantPosition
                }
                PropertyDeclaration::FontFeatureSettings(..) => {
                    LonghandId::FontFeatureSettings
                }
                PropertyDeclaration::FontVariationSettings(..) => {
                    LonghandId::FontVariationSettings
                }
                PropertyDeclaration::FontLanguageOverride(..) => {
                    LonghandId::FontLanguageOverride
                }
                PropertyDeclaration::XLang(..) => {
                    LonghandId::XLang
                }
                PropertyDeclaration::MozScriptSizeMultiplier(..) => {
                    LonghandId::MozScriptSizeMultiplier
                }
                PropertyDeclaration::MozScriptLevel(..) => {
                    LonghandId::MozScriptLevel
                }
                PropertyDeclaration::MozMathDisplay(..) => {
                    LonghandId::MozMathDisplay
                }
                PropertyDeclaration::MozMathVariant(..) => {
                    LonghandId::MozMathVariant
                }
                PropertyDeclaration::MozScriptMinSize(..) => {
                    LonghandId::MozScriptMinSize
                }
                PropertyDeclaration::XTextZoom(..) => {
                    LonghandId::XTextZoom
                }
                PropertyDeclaration::MozOsxFontSmoothing(..) => {
                    LonghandId::MozOsxFontSmoothing
                }
                PropertyDeclaration::MozMinFontSizeRatio(..) => {
                    LonghandId::MozMinFontSizeRatio
                }
                PropertyDeclaration::Visibility(..) => {
                    LonghandId::Visibility
                }
                PropertyDeclaration::WritingMode(..) => {
                    LonghandId::WritingMode
                }
                PropertyDeclaration::Direction(..) => {
                    LonghandId::Direction
                }
                PropertyDeclaration::TextOrientation(..) => {
                    LonghandId::TextOrientation
                }
                PropertyDeclaration::ColorAdjust(..) => {
                    LonghandId::ColorAdjust
                }
                PropertyDeclaration::ImageRendering(..) => {
                    LonghandId::ImageRendering
                }
                PropertyDeclaration::ImageOrientation(..) => {
                    LonghandId::ImageOrientation
                }
                PropertyDeclaration::BorderCollapse(..) => {
                    LonghandId::BorderCollapse
                }
                PropertyDeclaration::EmptyCells(..) => {
                    LonghandId::EmptyCells
                }
                PropertyDeclaration::CaptionSide(..) => {
                    LonghandId::CaptionSide
                }
                PropertyDeclaration::BorderSpacing(..) => {
                    LonghandId::BorderSpacing
                }
                PropertyDeclaration::LineHeight(..) => {
                    LonghandId::LineHeight
                }
                PropertyDeclaration::TextTransform(..) => {
                    LonghandId::TextTransform
                }
                PropertyDeclaration::Hyphens(..) => {
                    LonghandId::Hyphens
                }
                PropertyDeclaration::MozTextSizeAdjust(..) => {
                    LonghandId::MozTextSizeAdjust
                }
                PropertyDeclaration::TextIndent(..) => {
                    LonghandId::TextIndent
                }
                PropertyDeclaration::OverflowWrap(..) => {
                    LonghandId::OverflowWrap
                }
                PropertyDeclaration::WordBreak(..) => {
                    LonghandId::WordBreak
                }
                PropertyDeclaration::TextJustify(..) => {
                    LonghandId::TextJustify
                }
                PropertyDeclaration::TextAlignLast(..) => {
                    LonghandId::TextAlignLast
                }
                PropertyDeclaration::TextAlign(..) => {
                    LonghandId::TextAlign
                }
                PropertyDeclaration::LetterSpacing(..) => {
                    LonghandId::LetterSpacing
                }
                PropertyDeclaration::WordSpacing(..) => {
                    LonghandId::WordSpacing
                }
                PropertyDeclaration::WhiteSpace(..) => {
                    LonghandId::WhiteSpace
                }
                PropertyDeclaration::TextShadow(..) => {
                    LonghandId::TextShadow
                }
                PropertyDeclaration::TextEmphasisStyle(..) => {
                    LonghandId::TextEmphasisStyle
                }
                PropertyDeclaration::TextEmphasisPosition(..) => {
                    LonghandId::TextEmphasisPosition
                }
                PropertyDeclaration::TextEmphasisColor(..) => {
                    LonghandId::TextEmphasisColor
                }
                PropertyDeclaration::MozTabSize(..) => {
                    LonghandId::MozTabSize
                }
                PropertyDeclaration::WebkitTextFillColor(..) => {
                    LonghandId::WebkitTextFillColor
                }
                PropertyDeclaration::WebkitTextStrokeColor(..) => {
                    LonghandId::WebkitTextStrokeColor
                }
                PropertyDeclaration::WebkitTextStrokeWidth(..) => {
                    LonghandId::WebkitTextStrokeWidth
                }
                PropertyDeclaration::RubyAlign(..) => {
                    LonghandId::RubyAlign
                }
                PropertyDeclaration::RubyPosition(..) => {
                    LonghandId::RubyPosition
                }
                PropertyDeclaration::TextCombineUpright(..) => {
                    LonghandId::TextCombineUpright
                }
                PropertyDeclaration::TextRendering(..) => {
                    LonghandId::TextRendering
                }
                PropertyDeclaration::MozControlCharacterVisibility(..) => {
                    LonghandId::MozControlCharacterVisibility
                }
                PropertyDeclaration::ListStylePosition(..) => {
                    LonghandId::ListStylePosition
                }
                PropertyDeclaration::ListStyleType(..) => {
                    LonghandId::ListStyleType
                }
                PropertyDeclaration::ListStyleImage(..) => {
                    LonghandId::ListStyleImage
                }
                PropertyDeclaration::Quotes(..) => {
                    LonghandId::Quotes
                }
                PropertyDeclaration::MozImageRegion(..) => {
                    LonghandId::MozImageRegion
                }
                PropertyDeclaration::MarginTop(..) => {
                    LonghandId::MarginTop
                }
                PropertyDeclaration::MarginLeft(..) => {
                    LonghandId::MarginLeft
                }
                PropertyDeclaration::MarginBottom(..) => {
                    LonghandId::MarginBottom
                }
                PropertyDeclaration::MarginRight(..) => {
                    LonghandId::MarginRight
                }
                PropertyDeclaration::MarginBlockStart(..) => {
                    LonghandId::MarginBlockStart
                }
                PropertyDeclaration::MarginBlockEnd(..) => {
                    LonghandId::MarginBlockEnd
                }
                PropertyDeclaration::MarginInlineStart(..) => {
                    LonghandId::MarginInlineStart
                }
                PropertyDeclaration::MarginInlineEnd(..) => {
                    LonghandId::MarginInlineEnd
                }
                PropertyDeclaration::OutlineColor(..) => {
                    LonghandId::OutlineColor
                }
                PropertyDeclaration::OutlineStyle(..) => {
                    LonghandId::OutlineStyle
                }
                PropertyDeclaration::OutlineWidth(..) => {
                    LonghandId::OutlineWidth
                }
                PropertyDeclaration::MozOutlineRadiusTopleft(..) => {
                    LonghandId::MozOutlineRadiusTopleft
                }
                PropertyDeclaration::MozOutlineRadiusTopright(..) => {
                    LonghandId::MozOutlineRadiusTopright
                }
                PropertyDeclaration::MozOutlineRadiusBottomright(..) => {
                    LonghandId::MozOutlineRadiusBottomright
                }
                PropertyDeclaration::MozOutlineRadiusBottomleft(..) => {
                    LonghandId::MozOutlineRadiusBottomleft
                }
                PropertyDeclaration::OutlineOffset(..) => {
                    LonghandId::OutlineOffset
                }
                PropertyDeclaration::PaddingTop(..) => {
                    LonghandId::PaddingTop
                }
                PropertyDeclaration::PaddingLeft(..) => {
                    LonghandId::PaddingLeft
                }
                PropertyDeclaration::PaddingBottom(..) => {
                    LonghandId::PaddingBottom
                }
                PropertyDeclaration::PaddingRight(..) => {
                    LonghandId::PaddingRight
                }
                PropertyDeclaration::PaddingBlockStart(..) => {
                    LonghandId::PaddingBlockStart
                }
                PropertyDeclaration::PaddingBlockEnd(..) => {
                    LonghandId::PaddingBlockEnd
                }
                PropertyDeclaration::PaddingInlineStart(..) => {
                    LonghandId::PaddingInlineStart
                }
                PropertyDeclaration::PaddingInlineEnd(..) => {
                    LonghandId::PaddingInlineEnd
                }
                PropertyDeclaration::Cursor(..) => {
                    LonghandId::Cursor
                }
                PropertyDeclaration::PointerEvents(..) => {
                    LonghandId::PointerEvents
                }
                PropertyDeclaration::MozUserInput(..) => {
                    LonghandId::MozUserInput
                }
                PropertyDeclaration::MozUserModify(..) => {
                    LonghandId::MozUserModify
                }
                PropertyDeclaration::MozUserFocus(..) => {
                    LonghandId::MozUserFocus
                }
                PropertyDeclaration::CaretColor(..) => {
                    LonghandId::CaretColor
                }
                PropertyDeclaration::Top(..) => {
                    LonghandId::Top
                }
                PropertyDeclaration::Left(..) => {
                    LonghandId::Left
                }
                PropertyDeclaration::Bottom(..) => {
                    LonghandId::Bottom
                }
                PropertyDeclaration::Right(..) => {
                    LonghandId::Right
                }
                PropertyDeclaration::OffsetBlockStart(..) => {
                    LonghandId::OffsetBlockStart
                }
                PropertyDeclaration::OffsetBlockEnd(..) => {
                    LonghandId::OffsetBlockEnd
                }
                PropertyDeclaration::OffsetInlineStart(..) => {
                    LonghandId::OffsetInlineStart
                }
                PropertyDeclaration::OffsetInlineEnd(..) => {
                    LonghandId::OffsetInlineEnd
                }
                PropertyDeclaration::ZIndex(..) => {
                    LonghandId::ZIndex
                }
                PropertyDeclaration::FlexDirection(..) => {
                    LonghandId::FlexDirection
                }
                PropertyDeclaration::FlexWrap(..) => {
                    LonghandId::FlexWrap
                }
                PropertyDeclaration::JustifyContent(..) => {
                    LonghandId::JustifyContent
                }
                PropertyDeclaration::AlignContent(..) => {
                    LonghandId::AlignContent
                }
                PropertyDeclaration::AlignItems(..) => {
                    LonghandId::AlignItems
                }
                PropertyDeclaration::JustifyItems(..) => {
                    LonghandId::JustifyItems
                }
                PropertyDeclaration::FlexGrow(..) => {
                    LonghandId::FlexGrow
                }
                PropertyDeclaration::FlexShrink(..) => {
                    LonghandId::FlexShrink
                }
                PropertyDeclaration::AlignSelf(..) => {
                    LonghandId::AlignSelf
                }
                PropertyDeclaration::JustifySelf(..) => {
                    LonghandId::JustifySelf
                }
                PropertyDeclaration::Order(..) => {
                    LonghandId::Order
                }
                PropertyDeclaration::FlexBasis(..) => {
                    LonghandId::FlexBasis
                }
                PropertyDeclaration::Width(..) => {
                    LonghandId::Width
                }
                PropertyDeclaration::MinWidth(..) => {
                    LonghandId::MinWidth
                }
                PropertyDeclaration::MaxWidth(..) => {
                    LonghandId::MaxWidth
                }
                PropertyDeclaration::Height(..) => {
                    LonghandId::Height
                }
                PropertyDeclaration::MinHeight(..) => {
                    LonghandId::MinHeight
                }
                PropertyDeclaration::MaxHeight(..) => {
                    LonghandId::MaxHeight
                }
                PropertyDeclaration::BlockSize(..) => {
                    LonghandId::BlockSize
                }
                PropertyDeclaration::MinBlockSize(..) => {
                    LonghandId::MinBlockSize
                }
                PropertyDeclaration::MaxBlockSize(..) => {
                    LonghandId::MaxBlockSize
                }
                PropertyDeclaration::InlineSize(..) => {
                    LonghandId::InlineSize
                }
                PropertyDeclaration::MinInlineSize(..) => {
                    LonghandId::MinInlineSize
                }
                PropertyDeclaration::MaxInlineSize(..) => {
                    LonghandId::MaxInlineSize
                }
                PropertyDeclaration::BoxSizing(..) => {
                    LonghandId::BoxSizing
                }
                PropertyDeclaration::ObjectFit(..) => {
                    LonghandId::ObjectFit
                }
                PropertyDeclaration::ObjectPosition(..) => {
                    LonghandId::ObjectPosition
                }
                PropertyDeclaration::GridRowGap(..) => {
                    LonghandId::GridRowGap
                }
                PropertyDeclaration::GridRowStart(..) => {
                    LonghandId::GridRowStart
                }
                PropertyDeclaration::GridRowEnd(..) => {
                    LonghandId::GridRowEnd
                }
                PropertyDeclaration::GridAutoRows(..) => {
                    LonghandId::GridAutoRows
                }
                PropertyDeclaration::GridTemplateRows(..) => {
                    LonghandId::GridTemplateRows
                }
                PropertyDeclaration::GridColumnGap(..) => {
                    LonghandId::GridColumnGap
                }
                PropertyDeclaration::GridColumnStart(..) => {
                    LonghandId::GridColumnStart
                }
                PropertyDeclaration::GridColumnEnd(..) => {
                    LonghandId::GridColumnEnd
                }
                PropertyDeclaration::GridAutoColumns(..) => {
                    LonghandId::GridAutoColumns
                }
                PropertyDeclaration::GridTemplateColumns(..) => {
                    LonghandId::GridTemplateColumns
                }
                PropertyDeclaration::GridAutoFlow(..) => {
                    LonghandId::GridAutoFlow
                }
                PropertyDeclaration::GridTemplateAreas(..) => {
                    LonghandId::GridTemplateAreas
                }
                PropertyDeclaration::TableLayout(..) => {
                    LonghandId::TableLayout
                }
                PropertyDeclaration::XSpan(..) => {
                    LonghandId::XSpan
                }
                PropertyDeclaration::TextOverflow(..) => {
                    LonghandId::TextOverflow
                }
                PropertyDeclaration::UnicodeBidi(..) => {
                    LonghandId::UnicodeBidi
                }
                PropertyDeclaration::TextDecorationLine(..) => {
                    LonghandId::TextDecorationLine
                }
                PropertyDeclaration::TextDecorationStyle(..) => {
                    LonghandId::TextDecorationStyle
                }
                PropertyDeclaration::TextDecorationColor(..) => {
                    LonghandId::TextDecorationColor
                }
                PropertyDeclaration::InitialLetter(..) => {
                    LonghandId::InitialLetter
                }
                PropertyDeclaration::ImeMode(..) => {
                    LonghandId::ImeMode
                }
                PropertyDeclaration::MozUserSelect(..) => {
                    LonghandId::MozUserSelect
                }
                PropertyDeclaration::MozWindowDragging(..) => {
                    LonghandId::MozWindowDragging
                }
                PropertyDeclaration::MozWindowShadow(..) => {
                    LonghandId::MozWindowShadow
                }
                PropertyDeclaration::MozForceBrokenImageIcon(..) => {
                    LonghandId::MozForceBrokenImageIcon
                }
                PropertyDeclaration::TextAnchor(..) => {
                    LonghandId::TextAnchor
                }
                PropertyDeclaration::ColorInterpolation(..) => {
                    LonghandId::ColorInterpolation
                }
                PropertyDeclaration::ColorInterpolationFilters(..) => {
                    LonghandId::ColorInterpolationFilters
                }
                PropertyDeclaration::Fill(..) => {
                    LonghandId::Fill
                }
                PropertyDeclaration::FillOpacity(..) => {
                    LonghandId::FillOpacity
                }
                PropertyDeclaration::FillRule(..) => {
                    LonghandId::FillRule
                }
                PropertyDeclaration::ShapeRendering(..) => {
                    LonghandId::ShapeRendering
                }
                PropertyDeclaration::Stroke(..) => {
                    LonghandId::Stroke
                }
                PropertyDeclaration::StrokeWidth(..) => {
                    LonghandId::StrokeWidth
                }
                PropertyDeclaration::StrokeLinecap(..) => {
                    LonghandId::StrokeLinecap
                }
                PropertyDeclaration::StrokeLinejoin(..) => {
                    LonghandId::StrokeLinejoin
                }
                PropertyDeclaration::StrokeMiterlimit(..) => {
                    LonghandId::StrokeMiterlimit
                }
                PropertyDeclaration::StrokeOpacity(..) => {
                    LonghandId::StrokeOpacity
                }
                PropertyDeclaration::StrokeDasharray(..) => {
                    LonghandId::StrokeDasharray
                }
                PropertyDeclaration::StrokeDashoffset(..) => {
                    LonghandId::StrokeDashoffset
                }
                PropertyDeclaration::ClipRule(..) => {
                    LonghandId::ClipRule
                }
                PropertyDeclaration::MarkerStart(..) => {
                    LonghandId::MarkerStart
                }
                PropertyDeclaration::MarkerMid(..) => {
                    LonghandId::MarkerMid
                }
                PropertyDeclaration::MarkerEnd(..) => {
                    LonghandId::MarkerEnd
                }
                PropertyDeclaration::PaintOrder(..) => {
                    LonghandId::PaintOrder
                }
                PropertyDeclaration::MozContextProperties(..) => {
                    LonghandId::MozContextProperties
                }
                PropertyDeclaration::DominantBaseline(..) => {
                    LonghandId::DominantBaseline
                }
                PropertyDeclaration::VectorEffect(..) => {
                    LonghandId::VectorEffect
                }
                PropertyDeclaration::StopColor(..) => {
                    LonghandId::StopColor
                }
                PropertyDeclaration::StopOpacity(..) => {
                    LonghandId::StopOpacity
                }
                PropertyDeclaration::FloodColor(..) => {
                    LonghandId::FloodColor
                }
                PropertyDeclaration::FloodOpacity(..) => {
                    LonghandId::FloodOpacity
                }
                PropertyDeclaration::LightingColor(..) => {
                    LonghandId::LightingColor
                }
                PropertyDeclaration::MaskType(..) => {
                    LonghandId::MaskType
                }
                PropertyDeclaration::ClipPath(..) => {
                    LonghandId::ClipPath
                }
                PropertyDeclaration::MaskMode(..) => {
                    LonghandId::MaskMode
                }
                PropertyDeclaration::MaskRepeat(..) => {
                    LonghandId::MaskRepeat
                }
                PropertyDeclaration::MaskPositionX(..) => {
                    LonghandId::MaskPositionX
                }
                PropertyDeclaration::MaskPositionY(..) => {
                    LonghandId::MaskPositionY
                }
                PropertyDeclaration::MaskClip(..) => {
                    LonghandId::MaskClip
                }
                PropertyDeclaration::MaskOrigin(..) => {
                    LonghandId::MaskOrigin
                }
                PropertyDeclaration::MaskSize(..) => {
                    LonghandId::MaskSize
                }
                PropertyDeclaration::MaskComposite(..) => {
                    LonghandId::MaskComposite
                }
                PropertyDeclaration::MaskImage(..) => {
                    LonghandId::MaskImage
                }
                PropertyDeclaration::MozBoxAlign(..) => {
                    LonghandId::MozBoxAlign
                }
                PropertyDeclaration::MozBoxDirection(..) => {
                    LonghandId::MozBoxDirection
                }
                PropertyDeclaration::MozBoxFlex(..) => {
                    LonghandId::MozBoxFlex
                }
                PropertyDeclaration::MozBoxOrient(..) => {
                    LonghandId::MozBoxOrient
                }
                PropertyDeclaration::MozBoxPack(..) => {
                    LonghandId::MozBoxPack
                }
                PropertyDeclaration::MozStackSizing(..) => {
                    LonghandId::MozStackSizing
                }
                PropertyDeclaration::MozBoxOrdinalGroup(..) => {
                    LonghandId::MozBoxOrdinalGroup
                }
            PropertyDeclaration::CSSWideKeyword(..) |
            PropertyDeclaration::WithVariables(..) |
            PropertyDeclaration::Custom(..) => {
                debug_assert!(false, "unreachable");
                // This value is never used, but having an expression of the same "shape"
                // as for other variants helps the optimizer compile this `match` expression
                // to a lookup table.
                LonghandId::BackgroundColor
            }
        };
        PropertyDeclarationId::Longhand(longhand_id)
    }

    fn with_variables_from_shorthand(&self, shorthand: ShorthandId) -> Option< &str> {
        match *self {
            PropertyDeclaration::WithVariables(_, ref with_variables) => {
                if let Some(s) = with_variables.from_shorthand {
                    if s == shorthand {
                        Some(&*with_variables.css)
                    } else { None }
                } else {
                    // Normally, longhand property that doesn't come from a shorthand
                    // should return None here. But we return Some to longhands if they
                    // came from a shorthand alias. Because for example, we should be able to
                    // get -moz-transform's value from transform.
                    if shorthand.flags().contains(SHORTHAND_ALIAS_PROPERTY) {
                        return Some(&*with_variables.css);
                    }
                    None
                }
            },
            _ => None,
        }
    }

    /// Returns a CSS-wide keyword if the declaration's value is one.
    pub fn get_css_wide_keyword(&self) -> Option<CSSWideKeyword> {
        match *self {
            PropertyDeclaration::CSSWideKeyword(_, keyword) => Some(keyword),
            _ => None,
        }
    }

    /// Returns whether or not the property is set by a system font
    #[cfg(feature = "gecko")]
    pub fn get_system(&self) -> Option<SystemFont> {
        match *self {
                PropertyDeclaration::FontFamily(ref prop) => {
                    prop.get_system()
                }
                PropertyDeclaration::FontSize(ref prop) => {
                    prop.get_system()
                }
                PropertyDeclaration::FontStyle(ref prop) => {
                    prop.get_system()
                }
                PropertyDeclaration::FontVariantCaps(ref prop) => {
                    prop.get_system()
                }
                PropertyDeclaration::FontStretch(ref prop) => {
                    prop.get_system()
                }
                PropertyDeclaration::FontKerning(ref prop) => {
                    prop.get_system()
                }
                PropertyDeclaration::FontVariantPosition(ref prop) => {
                    prop.get_system()
                }
                PropertyDeclaration::FontWeight(ref prop) => {
                    prop.get_system()
                }
                PropertyDeclaration::FontSizeAdjust(ref prop) => {
                    prop.get_system()
                }
                PropertyDeclaration::FontVariantAlternates(ref prop) => {
                    prop.get_system()
                }
                PropertyDeclaration::FontVariantLigatures(ref prop) => {
                    prop.get_system()
                }
                PropertyDeclaration::FontVariantEastAsian(ref prop) => {
                    prop.get_system()
                }
                PropertyDeclaration::FontVariantNumeric(ref prop) => {
                    prop.get_system()
                }
                PropertyDeclaration::FontLanguageOverride(ref prop) => {
                    prop.get_system()
                }
                PropertyDeclaration::FontFeatureSettings(ref prop) => {
                    prop.get_system()
                }
            _ => None,
        }
    }

    /// Is it the default value of line-height?
    pub fn is_default_line_height(&self) -> bool {
        match *self {
            PropertyDeclaration::LineHeight(LineHeight::Normal) => true,
            _ => false
        }
    }

    #[cfg(feature = "servo")]
    /// Dummy method to avoid cfg()s
    pub fn get_system(&self) -> Option<()> {
        None
    }

    /// Returns whether the declaration may be serialized as part of a shorthand.
    ///
    /// This method returns false if this declaration contains variable or has a
    /// CSS-wide keyword value, since these values cannot be serialized as part
    /// of a shorthand.
    ///
    /// Caller should check `with_variables_from_shorthand()` and whether all
    /// needed declarations has the same CSS-wide keyword first.
    ///
    /// Note that, serialization of a shorthand may still fail because of other
    /// property-specific requirement even when this method returns true for all
    /// the longhand declarations.
    pub fn may_serialize_as_part_of_shorthand(&self) -> bool {
        match *self {
            PropertyDeclaration::CSSWideKeyword(..) |
            PropertyDeclaration::WithVariables(..) => false,
            PropertyDeclaration::Custom(..) =>
                unreachable!("Serializing a custom property as part of shorthand?"),
            _ => true,
        }
    }

    /// Return whether the value is stored as it was in the CSS source,
    /// preserving whitespace (as opposed to being parsed into a more abstract
    /// data structure).
    ///
    /// This is the case of custom properties and values that contain
    /// unsubstituted variables.
    pub fn value_is_unparsed(&self) -> bool {
      match *self {
          PropertyDeclaration::WithVariables(..) => true,
          PropertyDeclaration::Custom(_, ref value) => {
            !matches!(value.borrow(), DeclaredValue::CSSWideKeyword(..))
          }
          _ => false,
      }
    }

    /// The shorthands that this longhand is part of.
    pub fn shorthands(&self) -> &'static [ShorthandId] {
        match self.id() {
            PropertyDeclarationId::Longhand(id) => id.shorthands(),
            PropertyDeclarationId::Custom(..) => &[],
        }
    }

    /// Returns true if this property is one of the animable properties, false
    /// otherwise.
    pub fn is_animatable(&self) -> bool {
        match *self {
            PropertyDeclaration::BackgroundColor(_) => {
                    true
            }
            PropertyDeclaration::BackgroundImage(_) => {
                    true
            }
            PropertyDeclaration::BackgroundPositionX(_) => {
                    true
            }
            PropertyDeclaration::BackgroundPositionY(_) => {
                    true
            }
            PropertyDeclaration::BackgroundRepeat(_) => {
                    true
            }
            PropertyDeclaration::BackgroundAttachment(_) => {
                    true
            }
            PropertyDeclaration::BackgroundClip(_) => {
                    true
            }
            PropertyDeclaration::BackgroundOrigin(_) => {
                    true
            }
            PropertyDeclaration::BackgroundSize(_) => {
                    true
            }
            PropertyDeclaration::BackgroundBlendMode(_) => {
                    true
            }
            PropertyDeclaration::BorderTopColor(_) => {
                    true
            }
            PropertyDeclaration::BorderTopStyle(_) => {
                    true
            }
            PropertyDeclaration::BorderTopWidth(_) => {
                    true
            }
            PropertyDeclaration::BorderLeftColor(_) => {
                    true
            }
            PropertyDeclaration::BorderLeftStyle(_) => {
                    true
            }
            PropertyDeclaration::BorderLeftWidth(_) => {
                    true
            }
            PropertyDeclaration::BorderBottomColor(_) => {
                    true
            }
            PropertyDeclaration::BorderBottomStyle(_) => {
                    true
            }
            PropertyDeclaration::BorderBottomWidth(_) => {
                    true
            }
            PropertyDeclaration::BorderRightColor(_) => {
                    true
            }
            PropertyDeclaration::BorderRightStyle(_) => {
                    true
            }
            PropertyDeclaration::BorderRightWidth(_) => {
                    true
            }
            PropertyDeclaration::BorderBlockStartColor(_) => {
                    false
            }
            PropertyDeclaration::BorderBlockStartStyle(_) => {
                    false
            }
            PropertyDeclaration::BorderBlockStartWidth(_) => {
                    false
            }
            PropertyDeclaration::BorderBlockEndColor(_) => {
                    false
            }
            PropertyDeclaration::BorderBlockEndStyle(_) => {
                    false
            }
            PropertyDeclaration::BorderBlockEndWidth(_) => {
                    false
            }
            PropertyDeclaration::BorderInlineStartColor(_) => {
                    false
            }
            PropertyDeclaration::BorderInlineStartStyle(_) => {
                    false
            }
            PropertyDeclaration::BorderInlineStartWidth(_) => {
                    false
            }
            PropertyDeclaration::BorderInlineEndColor(_) => {
                    false
            }
            PropertyDeclaration::BorderInlineEndStyle(_) => {
                    false
            }
            PropertyDeclaration::BorderInlineEndWidth(_) => {
                    false
            }
            PropertyDeclaration::BorderTopLeftRadius(_) => {
                    true
            }
            PropertyDeclaration::BorderTopRightRadius(_) => {
                    true
            }
            PropertyDeclaration::BorderBottomRightRadius(_) => {
                    true
            }
            PropertyDeclaration::BorderBottomLeftRadius(_) => {
                    true
            }
            PropertyDeclaration::MozBorderTopColors(_) => {
                    true
            }
            PropertyDeclaration::MozBorderLeftColors(_) => {
                    true
            }
            PropertyDeclaration::MozBorderBottomColors(_) => {
                    true
            }
            PropertyDeclaration::MozBorderRightColors(_) => {
                    true
            }
            PropertyDeclaration::BoxDecorationBreak(_) => {
                    true
            }
            PropertyDeclaration::MozFloatEdge(_) => {
                    true
            }
            PropertyDeclaration::BorderImageSource(_) => {
                    true
            }
            PropertyDeclaration::BorderImageOutset(_) => {
                    true
            }
            PropertyDeclaration::BorderImageRepeat(_) => {
                    true
            }
            PropertyDeclaration::BorderImageWidth(_) => {
                    true
            }
            PropertyDeclaration::BorderImageSlice(_) => {
                    true
            }
            PropertyDeclaration::Display(_) => {
                    true
            }
            PropertyDeclaration::MozTopLayer(_) => {
                    false
            }
            PropertyDeclaration::Position(_) => {
                    true
            }
            PropertyDeclaration::Float(_) => {
                    true
            }
            PropertyDeclaration::Clear(_) => {
                    true
            }
            PropertyDeclaration::VerticalAlign(_) => {
                    true
            }
            PropertyDeclaration::OverflowClipBox(_) => {
                    true
            }
            PropertyDeclaration::OverflowX(_) => {
                    true
            }
            PropertyDeclaration::OverflowY(_) => {
                    true
            }
            PropertyDeclaration::TransitionDuration(_) => {
                    false
            }
            PropertyDeclaration::TransitionTimingFunction(_) => {
                    false
            }
            PropertyDeclaration::TransitionProperty(_) => {
                    false
            }
            PropertyDeclaration::TransitionDelay(_) => {
                    false
            }
            PropertyDeclaration::AnimationName(_) => {
                    false
            }
            PropertyDeclaration::AnimationDuration(_) => {
                    false
            }
            PropertyDeclaration::AnimationTimingFunction(_) => {
                    false
            }
            PropertyDeclaration::AnimationIterationCount(_) => {
                    false
            }
            PropertyDeclaration::AnimationDirection(_) => {
                    false
            }
            PropertyDeclaration::AnimationPlayState(_) => {
                    false
            }
            PropertyDeclaration::AnimationFillMode(_) => {
                    false
            }
            PropertyDeclaration::AnimationDelay(_) => {
                    false
            }
            PropertyDeclaration::ScrollSnapPointsX(_) => {
                    true
            }
            PropertyDeclaration::ScrollSnapPointsY(_) => {
                    true
            }
            PropertyDeclaration::ScrollSnapDestination(_) => {
                    true
            }
            PropertyDeclaration::ScrollSnapCoordinate(_) => {
                    true
            }
            PropertyDeclaration::Transform(_) => {
                    true
            }
            PropertyDeclaration::ScrollBehavior(_) => {
                    true
            }
            PropertyDeclaration::ScrollSnapTypeX(_) => {
                    true
            }
            PropertyDeclaration::ScrollSnapTypeY(_) => {
                    true
            }
            PropertyDeclaration::Isolation(_) => {
                    true
            }
            PropertyDeclaration::PageBreakAfter(_) => {
                    true
            }
            PropertyDeclaration::PageBreakBefore(_) => {
                    true
            }
            PropertyDeclaration::PageBreakInside(_) => {
                    true
            }
            PropertyDeclaration::Resize(_) => {
                    true
            }
            PropertyDeclaration::Perspective(_) => {
                    true
            }
            PropertyDeclaration::PerspectiveOrigin(_) => {
                    true
            }
            PropertyDeclaration::BackfaceVisibility(_) => {
                    true
            }
            PropertyDeclaration::TransformBox(_) => {
                    true
            }
            PropertyDeclaration::TransformStyle(_) => {
                    true
            }
            PropertyDeclaration::TransformOrigin(_) => {
                    true
            }
            PropertyDeclaration::Contain(_) => {
                    true
            }
            PropertyDeclaration::MozAppearance(_) => {
                    true
            }
            PropertyDeclaration::MozBinding(_) => {
                    false
            }
            PropertyDeclaration::MozOrient(_) => {
                    true
            }
            PropertyDeclaration::WillChange(_) => {
                    true
            }
            PropertyDeclaration::ShapeOutside(_) => {
                    true
            }
            PropertyDeclaration::TouchAction(_) => {
                    true
            }
            PropertyDeclaration::Color(_) => {
                    true
            }
            PropertyDeclaration::ColumnWidth(_) => {
                    true
            }
            PropertyDeclaration::ColumnCount(_) => {
                    true
            }
            PropertyDeclaration::ColumnGap(_) => {
                    true
            }
            PropertyDeclaration::ColumnFill(_) => {
                    true
            }
            PropertyDeclaration::ColumnRuleWidth(_) => {
                    true
            }
            PropertyDeclaration::ColumnRuleColor(_) => {
                    true
            }
            PropertyDeclaration::ColumnSpan(_) => {
                    true
            }
            PropertyDeclaration::ColumnRuleStyle(_) => {
                    true
            }
            PropertyDeclaration::Content(_) => {
                    true
            }
            PropertyDeclaration::CounterIncrement(_) => {
                    true
            }
            PropertyDeclaration::CounterReset(_) => {
                    true
            }
            PropertyDeclaration::Opacity(_) => {
                    true
            }
            PropertyDeclaration::BoxShadow(_) => {
                    true
            }
            PropertyDeclaration::Clip(_) => {
                    true
            }
            PropertyDeclaration::Filter(_) => {
                    true
            }
            PropertyDeclaration::MixBlendMode(_) => {
                    true
            }
            PropertyDeclaration::FontFamily(_) => {
                    true
            }
            PropertyDeclaration::FontStyle(_) => {
                    true
            }
            PropertyDeclaration::FontVariantCaps(_) => {
                    true
            }
            PropertyDeclaration::FontWeight(_) => {
                    true
            }
            PropertyDeclaration::FontSize(_) => {
                    true
            }
            PropertyDeclaration::FontSizeAdjust(_) => {
                    true
            }
            PropertyDeclaration::FontSynthesis(_) => {
                    true
            }
            PropertyDeclaration::FontStretch(_) => {
                    true
            }
            PropertyDeclaration::FontKerning(_) => {
                    true
            }
            PropertyDeclaration::FontVariantAlternates(_) => {
                    true
            }
            PropertyDeclaration::FontVariantEastAsian(_) => {
                    true
            }
            PropertyDeclaration::FontVariantLigatures(_) => {
                    true
            }
            PropertyDeclaration::FontVariantNumeric(_) => {
                    true
            }
            PropertyDeclaration::FontVariantPosition(_) => {
                    true
            }
            PropertyDeclaration::FontFeatureSettings(_) => {
                    true
            }
            PropertyDeclaration::FontVariationSettings(_) => {
                    true
            }
            PropertyDeclaration::FontLanguageOverride(_) => {
                    true
            }
            PropertyDeclaration::XLang(_) => {
                    false
            }
            PropertyDeclaration::MozScriptSizeMultiplier(_) => {
                    false
            }
            PropertyDeclaration::MozScriptLevel(_) => {
                    false
            }
            PropertyDeclaration::MozMathDisplay(_) => {
                    false
            }
            PropertyDeclaration::MozMathVariant(_) => {
                    false
            }
            PropertyDeclaration::MozScriptMinSize(_) => {
                    false
            }
            PropertyDeclaration::XTextZoom(_) => {
                    false
            }
            PropertyDeclaration::MozOsxFontSmoothing(_) => {
                    true
            }
            PropertyDeclaration::MozMinFontSizeRatio(_) => {
                    false
            }
            PropertyDeclaration::Visibility(_) => {
                    true
            }
            PropertyDeclaration::WritingMode(_) => {
                    true
            }
            PropertyDeclaration::Direction(_) => {
                    true
            }
            PropertyDeclaration::TextOrientation(_) => {
                    true
            }
            PropertyDeclaration::ColorAdjust(_) => {
                    true
            }
            PropertyDeclaration::ImageRendering(_) => {
                    true
            }
            PropertyDeclaration::ImageOrientation(_) => {
                    true
            }
            PropertyDeclaration::BorderCollapse(_) => {
                    true
            }
            PropertyDeclaration::EmptyCells(_) => {
                    true
            }
            PropertyDeclaration::CaptionSide(_) => {
                    true
            }
            PropertyDeclaration::BorderSpacing(_) => {
                    true
            }
            PropertyDeclaration::LineHeight(_) => {
                    true
            }
            PropertyDeclaration::TextTransform(_) => {
                    true
            }
            PropertyDeclaration::Hyphens(_) => {
                    true
            }
            PropertyDeclaration::MozTextSizeAdjust(_) => {
                    true
            }
            PropertyDeclaration::TextIndent(_) => {
                    true
            }
            PropertyDeclaration::OverflowWrap(_) => {
                    true
            }
            PropertyDeclaration::WordBreak(_) => {
                    true
            }
            PropertyDeclaration::TextJustify(_) => {
                    true
            }
            PropertyDeclaration::TextAlignLast(_) => {
                    true
            }
            PropertyDeclaration::TextAlign(_) => {
                    true
            }
            PropertyDeclaration::LetterSpacing(_) => {
                    true
            }
            PropertyDeclaration::WordSpacing(_) => {
                    true
            }
            PropertyDeclaration::WhiteSpace(_) => {
                    true
            }
            PropertyDeclaration::TextShadow(_) => {
                    true
            }
            PropertyDeclaration::TextEmphasisStyle(_) => {
                    true
            }
            PropertyDeclaration::TextEmphasisPosition(_) => {
                    true
            }
            PropertyDeclaration::TextEmphasisColor(_) => {
                    true
            }
            PropertyDeclaration::MozTabSize(_) => {
                    true
            }
            PropertyDeclaration::WebkitTextFillColor(_) => {
                    true
            }
            PropertyDeclaration::WebkitTextStrokeColor(_) => {
                    true
            }
            PropertyDeclaration::WebkitTextStrokeWidth(_) => {
                    true
            }
            PropertyDeclaration::RubyAlign(_) => {
                    true
            }
            PropertyDeclaration::RubyPosition(_) => {
                    true
            }
            PropertyDeclaration::TextCombineUpright(_) => {
                    true
            }
            PropertyDeclaration::TextRendering(_) => {
                    true
            }
            PropertyDeclaration::MozControlCharacterVisibility(_) => {
                    false
            }
            PropertyDeclaration::ListStylePosition(_) => {
                    true
            }
            PropertyDeclaration::ListStyleType(_) => {
                    true
            }
            PropertyDeclaration::ListStyleImage(_) => {
                    true
            }
            PropertyDeclaration::Quotes(_) => {
                    true
            }
            PropertyDeclaration::MozImageRegion(_) => {
                    true
            }
            PropertyDeclaration::MarginTop(_) => {
                    true
            }
            PropertyDeclaration::MarginLeft(_) => {
                    true
            }
            PropertyDeclaration::MarginBottom(_) => {
                    true
            }
            PropertyDeclaration::MarginRight(_) => {
                    true
            }
            PropertyDeclaration::MarginBlockStart(_) => {
                    false
            }
            PropertyDeclaration::MarginBlockEnd(_) => {
                    false
            }
            PropertyDeclaration::MarginInlineStart(_) => {
                    false
            }
            PropertyDeclaration::MarginInlineEnd(_) => {
                    false
            }
            PropertyDeclaration::OutlineColor(_) => {
                    true
            }
            PropertyDeclaration::OutlineStyle(_) => {
                    true
            }
            PropertyDeclaration::OutlineWidth(_) => {
                    true
            }
            PropertyDeclaration::MozOutlineRadiusTopleft(_) => {
                    true
            }
            PropertyDeclaration::MozOutlineRadiusTopright(_) => {
                    true
            }
            PropertyDeclaration::MozOutlineRadiusBottomright(_) => {
                    true
            }
            PropertyDeclaration::MozOutlineRadiusBottomleft(_) => {
                    true
            }
            PropertyDeclaration::OutlineOffset(_) => {
                    true
            }
            PropertyDeclaration::PaddingTop(_) => {
                    true
            }
            PropertyDeclaration::PaddingLeft(_) => {
                    true
            }
            PropertyDeclaration::PaddingBottom(_) => {
                    true
            }
            PropertyDeclaration::PaddingRight(_) => {
                    true
            }
            PropertyDeclaration::PaddingBlockStart(_) => {
                    false
            }
            PropertyDeclaration::PaddingBlockEnd(_) => {
                    false
            }
            PropertyDeclaration::PaddingInlineStart(_) => {
                    false
            }
            PropertyDeclaration::PaddingInlineEnd(_) => {
                    false
            }
            PropertyDeclaration::Cursor(_) => {
                    true
            }
            PropertyDeclaration::PointerEvents(_) => {
                    true
            }
            PropertyDeclaration::MozUserInput(_) => {
                    true
            }
            PropertyDeclaration::MozUserModify(_) => {
                    true
            }
            PropertyDeclaration::MozUserFocus(_) => {
                    true
            }
            PropertyDeclaration::CaretColor(_) => {
                    true
            }
            PropertyDeclaration::Top(_) => {
                    true
            }
            PropertyDeclaration::Left(_) => {
                    true
            }
            PropertyDeclaration::Bottom(_) => {
                    true
            }
            PropertyDeclaration::Right(_) => {
                    true
            }
            PropertyDeclaration::OffsetBlockStart(_) => {
                    false
            }
            PropertyDeclaration::OffsetBlockEnd(_) => {
                    false
            }
            PropertyDeclaration::OffsetInlineStart(_) => {
                    false
            }
            PropertyDeclaration::OffsetInlineEnd(_) => {
                    false
            }
            PropertyDeclaration::ZIndex(_) => {
                    true
            }
            PropertyDeclaration::FlexDirection(_) => {
                    true
            }
            PropertyDeclaration::FlexWrap(_) => {
                    true
            }
            PropertyDeclaration::JustifyContent(_) => {
                    true
            }
            PropertyDeclaration::AlignContent(_) => {
                    true
            }
            PropertyDeclaration::AlignItems(_) => {
                    true
            }
            PropertyDeclaration::JustifyItems(_) => {
                    true
            }
            PropertyDeclaration::FlexGrow(_) => {
                    true
            }
            PropertyDeclaration::FlexShrink(_) => {
                    true
            }
            PropertyDeclaration::AlignSelf(_) => {
                    true
            }
            PropertyDeclaration::JustifySelf(_) => {
                    true
            }
            PropertyDeclaration::Order(_) => {
                    true
            }
            PropertyDeclaration::FlexBasis(_) => {
                    true
            }
            PropertyDeclaration::Width(_) => {
                    true
            }
            PropertyDeclaration::MinWidth(_) => {
                    true
            }
            PropertyDeclaration::MaxWidth(_) => {
                    true
            }
            PropertyDeclaration::Height(_) => {
                    true
            }
            PropertyDeclaration::MinHeight(_) => {
                    true
            }
            PropertyDeclaration::MaxHeight(_) => {
                    true
            }
            PropertyDeclaration::BlockSize(_) => {
                    false
            }
            PropertyDeclaration::MinBlockSize(_) => {
                    false
            }
            PropertyDeclaration::MaxBlockSize(_) => {
                    false
            }
            PropertyDeclaration::InlineSize(_) => {
                    false
            }
            PropertyDeclaration::MinInlineSize(_) => {
                    false
            }
            PropertyDeclaration::MaxInlineSize(_) => {
                    false
            }
            PropertyDeclaration::BoxSizing(_) => {
                    true
            }
            PropertyDeclaration::ObjectFit(_) => {
                    true
            }
            PropertyDeclaration::ObjectPosition(_) => {
                    true
            }
            PropertyDeclaration::GridRowGap(_) => {
                    true
            }
            PropertyDeclaration::GridRowStart(_) => {
                    true
            }
            PropertyDeclaration::GridRowEnd(_) => {
                    true
            }
            PropertyDeclaration::GridAutoRows(_) => {
                    true
            }
            PropertyDeclaration::GridTemplateRows(_) => {
                    true
            }
            PropertyDeclaration::GridColumnGap(_) => {
                    true
            }
            PropertyDeclaration::GridColumnStart(_) => {
                    true
            }
            PropertyDeclaration::GridColumnEnd(_) => {
                    true
            }
            PropertyDeclaration::GridAutoColumns(_) => {
                    true
            }
            PropertyDeclaration::GridTemplateColumns(_) => {
                    true
            }
            PropertyDeclaration::GridAutoFlow(_) => {
                    true
            }
            PropertyDeclaration::GridTemplateAreas(_) => {
                    true
            }
            PropertyDeclaration::TableLayout(_) => {
                    true
            }
            PropertyDeclaration::XSpan(_) => {
                    false
            }
            PropertyDeclaration::TextOverflow(_) => {
                    true
            }
            PropertyDeclaration::UnicodeBidi(_) => {
                    true
            }
            PropertyDeclaration::TextDecorationLine(_) => {
                    true
            }
            PropertyDeclaration::TextDecorationStyle(_) => {
                    true
            }
            PropertyDeclaration::TextDecorationColor(_) => {
                    true
            }
            PropertyDeclaration::InitialLetter(_) => {
                    true
            }
            PropertyDeclaration::ImeMode(_) => {
                    true
            }
            PropertyDeclaration::MozUserSelect(_) => {
                    true
            }
            PropertyDeclaration::MozWindowDragging(_) => {
                    true
            }
            PropertyDeclaration::MozWindowShadow(_) => {
                    true
            }
            PropertyDeclaration::MozForceBrokenImageIcon(_) => {
                    true
            }
            PropertyDeclaration::TextAnchor(_) => {
                    true
            }
            PropertyDeclaration::ColorInterpolation(_) => {
                    true
            }
            PropertyDeclaration::ColorInterpolationFilters(_) => {
                    true
            }
            PropertyDeclaration::Fill(_) => {
                    true
            }
            PropertyDeclaration::FillOpacity(_) => {
                    true
            }
            PropertyDeclaration::FillRule(_) => {
                    true
            }
            PropertyDeclaration::ShapeRendering(_) => {
                    true
            }
            PropertyDeclaration::Stroke(_) => {
                    true
            }
            PropertyDeclaration::StrokeWidth(_) => {
                    true
            }
            PropertyDeclaration::StrokeLinecap(_) => {
                    true
            }
            PropertyDeclaration::StrokeLinejoin(_) => {
                    true
            }
            PropertyDeclaration::StrokeMiterlimit(_) => {
                    true
            }
            PropertyDeclaration::StrokeOpacity(_) => {
                    true
            }
            PropertyDeclaration::StrokeDasharray(_) => {
                    true
            }
            PropertyDeclaration::StrokeDashoffset(_) => {
                    true
            }
            PropertyDeclaration::ClipRule(_) => {
                    true
            }
            PropertyDeclaration::MarkerStart(_) => {
                    true
            }
            PropertyDeclaration::MarkerMid(_) => {
                    true
            }
            PropertyDeclaration::MarkerEnd(_) => {
                    true
            }
            PropertyDeclaration::PaintOrder(_) => {
                    true
            }
            PropertyDeclaration::MozContextProperties(_) => {
                    false
            }
            PropertyDeclaration::DominantBaseline(_) => {
                    true
            }
            PropertyDeclaration::VectorEffect(_) => {
                    true
            }
            PropertyDeclaration::StopColor(_) => {
                    true
            }
            PropertyDeclaration::StopOpacity(_) => {
                    true
            }
            PropertyDeclaration::FloodColor(_) => {
                    true
            }
            PropertyDeclaration::FloodOpacity(_) => {
                    true
            }
            PropertyDeclaration::LightingColor(_) => {
                    true
            }
            PropertyDeclaration::MaskType(_) => {
                    true
            }
            PropertyDeclaration::ClipPath(_) => {
                    true
            }
            PropertyDeclaration::MaskMode(_) => {
                    true
            }
            PropertyDeclaration::MaskRepeat(_) => {
                    true
            }
            PropertyDeclaration::MaskPositionX(_) => {
                    true
            }
            PropertyDeclaration::MaskPositionY(_) => {
                    true
            }
            PropertyDeclaration::MaskClip(_) => {
                    true
            }
            PropertyDeclaration::MaskOrigin(_) => {
                    true
            }
            PropertyDeclaration::MaskSize(_) => {
                    true
            }
            PropertyDeclaration::MaskComposite(_) => {
                    true
            }
            PropertyDeclaration::MaskImage(_) => {
                    true
            }
            PropertyDeclaration::MozBoxAlign(_) => {
                    true
            }
            PropertyDeclaration::MozBoxDirection(_) => {
                    true
            }
            PropertyDeclaration::MozBoxFlex(_) => {
                    true
            }
            PropertyDeclaration::MozBoxOrient(_) => {
                    true
            }
            PropertyDeclaration::MozBoxPack(_) => {
                    true
            }
            PropertyDeclaration::MozStackSizing(_) => {
                    true
            }
            PropertyDeclaration::MozBoxOrdinalGroup(_) => {
                    true
            }
            PropertyDeclaration::CSSWideKeyword(id, _) |
            PropertyDeclaration::WithVariables(id, _) => match id {
                LonghandId::BackgroundColor => {
                        true
                }
                LonghandId::BackgroundImage => {
                        true
                }
                LonghandId::BackgroundPositionX => {
                        true
                }
                LonghandId::BackgroundPositionY => {
                        true
                }
                LonghandId::BackgroundRepeat => {
                        true
                }
                LonghandId::BackgroundAttachment => {
                        true
                }
                LonghandId::BackgroundClip => {
                        true
                }
                LonghandId::BackgroundOrigin => {
                        true
                }
                LonghandId::BackgroundSize => {
                        true
                }
                LonghandId::BackgroundBlendMode => {
                        true
                }
                LonghandId::BorderTopColor => {
                        true
                }
                LonghandId::BorderTopStyle => {
                        true
                }
                LonghandId::BorderTopWidth => {
                        true
                }
                LonghandId::BorderLeftColor => {
                        true
                }
                LonghandId::BorderLeftStyle => {
                        true
                }
                LonghandId::BorderLeftWidth => {
                        true
                }
                LonghandId::BorderBottomColor => {
                        true
                }
                LonghandId::BorderBottomStyle => {
                        true
                }
                LonghandId::BorderBottomWidth => {
                        true
                }
                LonghandId::BorderRightColor => {
                        true
                }
                LonghandId::BorderRightStyle => {
                        true
                }
                LonghandId::BorderRightWidth => {
                        true
                }
                LonghandId::BorderBlockStartColor => {
                        false
                }
                LonghandId::BorderBlockStartStyle => {
                        false
                }
                LonghandId::BorderBlockStartWidth => {
                        false
                }
                LonghandId::BorderBlockEndColor => {
                        false
                }
                LonghandId::BorderBlockEndStyle => {
                        false
                }
                LonghandId::BorderBlockEndWidth => {
                        false
                }
                LonghandId::BorderInlineStartColor => {
                        false
                }
                LonghandId::BorderInlineStartStyle => {
                        false
                }
                LonghandId::BorderInlineStartWidth => {
                        false
                }
                LonghandId::BorderInlineEndColor => {
                        false
                }
                LonghandId::BorderInlineEndStyle => {
                        false
                }
                LonghandId::BorderInlineEndWidth => {
                        false
                }
                LonghandId::BorderTopLeftRadius => {
                        true
                }
                LonghandId::BorderTopRightRadius => {
                        true
                }
                LonghandId::BorderBottomRightRadius => {
                        true
                }
                LonghandId::BorderBottomLeftRadius => {
                        true
                }
                LonghandId::MozBorderTopColors => {
                        true
                }
                LonghandId::MozBorderLeftColors => {
                        true
                }
                LonghandId::MozBorderBottomColors => {
                        true
                }
                LonghandId::MozBorderRightColors => {
                        true
                }
                LonghandId::BoxDecorationBreak => {
                        true
                }
                LonghandId::MozFloatEdge => {
                        true
                }
                LonghandId::BorderImageSource => {
                        true
                }
                LonghandId::BorderImageOutset => {
                        true
                }
                LonghandId::BorderImageRepeat => {
                        true
                }
                LonghandId::BorderImageWidth => {
                        true
                }
                LonghandId::BorderImageSlice => {
                        true
                }
                LonghandId::Display => {
                        true
                }
                LonghandId::MozTopLayer => {
                        false
                }
                LonghandId::Position => {
                        true
                }
                LonghandId::Float => {
                        true
                }
                LonghandId::Clear => {
                        true
                }
                LonghandId::VerticalAlign => {
                        true
                }
                LonghandId::OverflowClipBox => {
                        true
                }
                LonghandId::OverflowX => {
                        true
                }
                LonghandId::OverflowY => {
                        true
                }
                LonghandId::TransitionDuration => {
                        false
                }
                LonghandId::TransitionTimingFunction => {
                        false
                }
                LonghandId::TransitionProperty => {
                        false
                }
                LonghandId::TransitionDelay => {
                        false
                }
                LonghandId::AnimationName => {
                        false
                }
                LonghandId::AnimationDuration => {
                        false
                }
                LonghandId::AnimationTimingFunction => {
                        false
                }
                LonghandId::AnimationIterationCount => {
                        false
                }
                LonghandId::AnimationDirection => {
                        false
                }
                LonghandId::AnimationPlayState => {
                        false
                }
                LonghandId::AnimationFillMode => {
                        false
                }
                LonghandId::AnimationDelay => {
                        false
                }
                LonghandId::ScrollSnapPointsX => {
                        true
                }
                LonghandId::ScrollSnapPointsY => {
                        true
                }
                LonghandId::ScrollSnapDestination => {
                        true
                }
                LonghandId::ScrollSnapCoordinate => {
                        true
                }
                LonghandId::Transform => {
                        true
                }
                LonghandId::ScrollBehavior => {
                        true
                }
                LonghandId::ScrollSnapTypeX => {
                        true
                }
                LonghandId::ScrollSnapTypeY => {
                        true
                }
                LonghandId::Isolation => {
                        true
                }
                LonghandId::PageBreakAfter => {
                        true
                }
                LonghandId::PageBreakBefore => {
                        true
                }
                LonghandId::PageBreakInside => {
                        true
                }
                LonghandId::Resize => {
                        true
                }
                LonghandId::Perspective => {
                        true
                }
                LonghandId::PerspectiveOrigin => {
                        true
                }
                LonghandId::BackfaceVisibility => {
                        true
                }
                LonghandId::TransformBox => {
                        true
                }
                LonghandId::TransformStyle => {
                        true
                }
                LonghandId::TransformOrigin => {
                        true
                }
                LonghandId::Contain => {
                        true
                }
                LonghandId::MozAppearance => {
                        true
                }
                LonghandId::MozBinding => {
                        false
                }
                LonghandId::MozOrient => {
                        true
                }
                LonghandId::WillChange => {
                        true
                }
                LonghandId::ShapeOutside => {
                        true
                }
                LonghandId::TouchAction => {
                        true
                }
                LonghandId::Color => {
                        true
                }
                LonghandId::ColumnWidth => {
                        true
                }
                LonghandId::ColumnCount => {
                        true
                }
                LonghandId::ColumnGap => {
                        true
                }
                LonghandId::ColumnFill => {
                        true
                }
                LonghandId::ColumnRuleWidth => {
                        true
                }
                LonghandId::ColumnRuleColor => {
                        true
                }
                LonghandId::ColumnSpan => {
                        true
                }
                LonghandId::ColumnRuleStyle => {
                        true
                }
                LonghandId::Content => {
                        true
                }
                LonghandId::CounterIncrement => {
                        true
                }
                LonghandId::CounterReset => {
                        true
                }
                LonghandId::Opacity => {
                        true
                }
                LonghandId::BoxShadow => {
                        true
                }
                LonghandId::Clip => {
                        true
                }
                LonghandId::Filter => {
                        true
                }
                LonghandId::MixBlendMode => {
                        true
                }
                LonghandId::FontFamily => {
                        true
                }
                LonghandId::FontStyle => {
                        true
                }
                LonghandId::FontVariantCaps => {
                        true
                }
                LonghandId::FontWeight => {
                        true
                }
                LonghandId::FontSize => {
                        true
                }
                LonghandId::FontSizeAdjust => {
                        true
                }
                LonghandId::FontSynthesis => {
                        true
                }
                LonghandId::FontStretch => {
                        true
                }
                LonghandId::FontKerning => {
                        true
                }
                LonghandId::FontVariantAlternates => {
                        true
                }
                LonghandId::FontVariantEastAsian => {
                        true
                }
                LonghandId::FontVariantLigatures => {
                        true
                }
                LonghandId::FontVariantNumeric => {
                        true
                }
                LonghandId::FontVariantPosition => {
                        true
                }
                LonghandId::FontFeatureSettings => {
                        true
                }
                LonghandId::FontVariationSettings => {
                        true
                }
                LonghandId::FontLanguageOverride => {
                        true
                }
                LonghandId::XLang => {
                        false
                }
                LonghandId::MozScriptSizeMultiplier => {
                        false
                }
                LonghandId::MozScriptLevel => {
                        false
                }
                LonghandId::MozMathDisplay => {
                        false
                }
                LonghandId::MozMathVariant => {
                        false
                }
                LonghandId::MozScriptMinSize => {
                        false
                }
                LonghandId::XTextZoom => {
                        false
                }
                LonghandId::MozOsxFontSmoothing => {
                        true
                }
                LonghandId::MozMinFontSizeRatio => {
                        false
                }
                LonghandId::Visibility => {
                        true
                }
                LonghandId::WritingMode => {
                        true
                }
                LonghandId::Direction => {
                        true
                }
                LonghandId::TextOrientation => {
                        true
                }
                LonghandId::ColorAdjust => {
                        true
                }
                LonghandId::ImageRendering => {
                        true
                }
                LonghandId::ImageOrientation => {
                        true
                }
                LonghandId::BorderCollapse => {
                        true
                }
                LonghandId::EmptyCells => {
                        true
                }
                LonghandId::CaptionSide => {
                        true
                }
                LonghandId::BorderSpacing => {
                        true
                }
                LonghandId::LineHeight => {
                        true
                }
                LonghandId::TextTransform => {
                        true
                }
                LonghandId::Hyphens => {
                        true
                }
                LonghandId::MozTextSizeAdjust => {
                        true
                }
                LonghandId::TextIndent => {
                        true
                }
                LonghandId::OverflowWrap => {
                        true
                }
                LonghandId::WordBreak => {
                        true
                }
                LonghandId::TextJustify => {
                        true
                }
                LonghandId::TextAlignLast => {
                        true
                }
                LonghandId::TextAlign => {
                        true
                }
                LonghandId::LetterSpacing => {
                        true
                }
                LonghandId::WordSpacing => {
                        true
                }
                LonghandId::WhiteSpace => {
                        true
                }
                LonghandId::TextShadow => {
                        true
                }
                LonghandId::TextEmphasisStyle => {
                        true
                }
                LonghandId::TextEmphasisPosition => {
                        true
                }
                LonghandId::TextEmphasisColor => {
                        true
                }
                LonghandId::MozTabSize => {
                        true
                }
                LonghandId::WebkitTextFillColor => {
                        true
                }
                LonghandId::WebkitTextStrokeColor => {
                        true
                }
                LonghandId::WebkitTextStrokeWidth => {
                        true
                }
                LonghandId::RubyAlign => {
                        true
                }
                LonghandId::RubyPosition => {
                        true
                }
                LonghandId::TextCombineUpright => {
                        true
                }
                LonghandId::TextRendering => {
                        true
                }
                LonghandId::MozControlCharacterVisibility => {
                        false
                }
                LonghandId::ListStylePosition => {
                        true
                }
                LonghandId::ListStyleType => {
                        true
                }
                LonghandId::ListStyleImage => {
                        true
                }
                LonghandId::Quotes => {
                        true
                }
                LonghandId::MozImageRegion => {
                        true
                }
                LonghandId::MarginTop => {
                        true
                }
                LonghandId::MarginLeft => {
                        true
                }
                LonghandId::MarginBottom => {
                        true
                }
                LonghandId::MarginRight => {
                        true
                }
                LonghandId::MarginBlockStart => {
                        false
                }
                LonghandId::MarginBlockEnd => {
                        false
                }
                LonghandId::MarginInlineStart => {
                        false
                }
                LonghandId::MarginInlineEnd => {
                        false
                }
                LonghandId::OutlineColor => {
                        true
                }
                LonghandId::OutlineStyle => {
                        true
                }
                LonghandId::OutlineWidth => {
                        true
                }
                LonghandId::MozOutlineRadiusTopleft => {
                        true
                }
                LonghandId::MozOutlineRadiusTopright => {
                        true
                }
                LonghandId::MozOutlineRadiusBottomright => {
                        true
                }
                LonghandId::MozOutlineRadiusBottomleft => {
                        true
                }
                LonghandId::OutlineOffset => {
                        true
                }
                LonghandId::PaddingTop => {
                        true
                }
                LonghandId::PaddingLeft => {
                        true
                }
                LonghandId::PaddingBottom => {
                        true
                }
                LonghandId::PaddingRight => {
                        true
                }
                LonghandId::PaddingBlockStart => {
                        false
                }
                LonghandId::PaddingBlockEnd => {
                        false
                }
                LonghandId::PaddingInlineStart => {
                        false
                }
                LonghandId::PaddingInlineEnd => {
                        false
                }
                LonghandId::Cursor => {
                        true
                }
                LonghandId::PointerEvents => {
                        true
                }
                LonghandId::MozUserInput => {
                        true
                }
                LonghandId::MozUserModify => {
                        true
                }
                LonghandId::MozUserFocus => {
                        true
                }
                LonghandId::CaretColor => {
                        true
                }
                LonghandId::Top => {
                        true
                }
                LonghandId::Left => {
                        true
                }
                LonghandId::Bottom => {
                        true
                }
                LonghandId::Right => {
                        true
                }
                LonghandId::OffsetBlockStart => {
                        false
                }
                LonghandId::OffsetBlockEnd => {
                        false
                }
                LonghandId::OffsetInlineStart => {
                        false
                }
                LonghandId::OffsetInlineEnd => {
                        false
                }
                LonghandId::ZIndex => {
                        true
                }
                LonghandId::FlexDirection => {
                        true
                }
                LonghandId::FlexWrap => {
                        true
                }
                LonghandId::JustifyContent => {
                        true
                }
                LonghandId::AlignContent => {
                        true
                }
                LonghandId::AlignItems => {
                        true
                }
                LonghandId::JustifyItems => {
                        true
                }
                LonghandId::FlexGrow => {
                        true
                }
                LonghandId::FlexShrink => {
                        true
                }
                LonghandId::AlignSelf => {
                        true
                }
                LonghandId::JustifySelf => {
                        true
                }
                LonghandId::Order => {
                        true
                }
                LonghandId::FlexBasis => {
                        true
                }
                LonghandId::Width => {
                        true
                }
                LonghandId::MinWidth => {
                        true
                }
                LonghandId::MaxWidth => {
                        true
                }
                LonghandId::Height => {
                        true
                }
                LonghandId::MinHeight => {
                        true
                }
                LonghandId::MaxHeight => {
                        true
                }
                LonghandId::BlockSize => {
                        false
                }
                LonghandId::MinBlockSize => {
                        false
                }
                LonghandId::MaxBlockSize => {
                        false
                }
                LonghandId::InlineSize => {
                        false
                }
                LonghandId::MinInlineSize => {
                        false
                }
                LonghandId::MaxInlineSize => {
                        false
                }
                LonghandId::BoxSizing => {
                        true
                }
                LonghandId::ObjectFit => {
                        true
                }
                LonghandId::ObjectPosition => {
                        true
                }
                LonghandId::GridRowGap => {
                        true
                }
                LonghandId::GridRowStart => {
                        true
                }
                LonghandId::GridRowEnd => {
                        true
                }
                LonghandId::GridAutoRows => {
                        true
                }
                LonghandId::GridTemplateRows => {
                        true
                }
                LonghandId::GridColumnGap => {
                        true
                }
                LonghandId::GridColumnStart => {
                        true
                }
                LonghandId::GridColumnEnd => {
                        true
                }
                LonghandId::GridAutoColumns => {
                        true
                }
                LonghandId::GridTemplateColumns => {
                        true
                }
                LonghandId::GridAutoFlow => {
                        true
                }
                LonghandId::GridTemplateAreas => {
                        true
                }
                LonghandId::TableLayout => {
                        true
                }
                LonghandId::XSpan => {
                        false
                }
                LonghandId::TextOverflow => {
                        true
                }
                LonghandId::UnicodeBidi => {
                        true
                }
                LonghandId::TextDecorationLine => {
                        true
                }
                LonghandId::TextDecorationStyle => {
                        true
                }
                LonghandId::TextDecorationColor => {
                        true
                }
                LonghandId::InitialLetter => {
                        true
                }
                LonghandId::ImeMode => {
                        true
                }
                LonghandId::MozUserSelect => {
                        true
                }
                LonghandId::MozWindowDragging => {
                        true
                }
                LonghandId::MozWindowShadow => {
                        true
                }
                LonghandId::MozForceBrokenImageIcon => {
                        true
                }
                LonghandId::TextAnchor => {
                        true
                }
                LonghandId::ColorInterpolation => {
                        true
                }
                LonghandId::ColorInterpolationFilters => {
                        true
                }
                LonghandId::Fill => {
                        true
                }
                LonghandId::FillOpacity => {
                        true
                }
                LonghandId::FillRule => {
                        true
                }
                LonghandId::ShapeRendering => {
                        true
                }
                LonghandId::Stroke => {
                        true
                }
                LonghandId::StrokeWidth => {
                        true
                }
                LonghandId::StrokeLinecap => {
                        true
                }
                LonghandId::StrokeLinejoin => {
                        true
                }
                LonghandId::StrokeMiterlimit => {
                        true
                }
                LonghandId::StrokeOpacity => {
                        true
                }
                LonghandId::StrokeDasharray => {
                        true
                }
                LonghandId::StrokeDashoffset => {
                        true
                }
                LonghandId::ClipRule => {
                        true
                }
                LonghandId::MarkerStart => {
                        true
                }
                LonghandId::MarkerMid => {
                        true
                }
                LonghandId::MarkerEnd => {
                        true
                }
                LonghandId::PaintOrder => {
                        true
                }
                LonghandId::MozContextProperties => {
                        false
                }
                LonghandId::DominantBaseline => {
                        true
                }
                LonghandId::VectorEffect => {
                        true
                }
                LonghandId::StopColor => {
                        true
                }
                LonghandId::StopOpacity => {
                        true
                }
                LonghandId::FloodColor => {
                        true
                }
                LonghandId::FloodOpacity => {
                        true
                }
                LonghandId::LightingColor => {
                        true
                }
                LonghandId::MaskType => {
                        true
                }
                LonghandId::ClipPath => {
                        true
                }
                LonghandId::MaskMode => {
                        true
                }
                LonghandId::MaskRepeat => {
                        true
                }
                LonghandId::MaskPositionX => {
                        true
                }
                LonghandId::MaskPositionY => {
                        true
                }
                LonghandId::MaskClip => {
                        true
                }
                LonghandId::MaskOrigin => {
                        true
                }
                LonghandId::MaskSize => {
                        true
                }
                LonghandId::MaskComposite => {
                        true
                }
                LonghandId::MaskImage => {
                        true
                }
                LonghandId::MozBoxAlign => {
                        true
                }
                LonghandId::MozBoxDirection => {
                        true
                }
                LonghandId::MozBoxFlex => {
                        true
                }
                LonghandId::MozBoxOrient => {
                        true
                }
                LonghandId::MozBoxPack => {
                        true
                }
                LonghandId::MozStackSizing => {
                        true
                }
                LonghandId::MozBoxOrdinalGroup => {
                        true
                }
            },
            PropertyDeclaration::Custom(..) => false,
        }
    }

    /// Returns true if this property is a custom property, false
    /// otherwise.
    pub fn is_custom(&self) -> bool {
        matches!(*self, PropertyDeclaration::Custom(_, _))
    }

    /// The `context` parameter controls this:
    ///
    /// https://drafts.csswg.org/css-animations/#keyframes
    /// > The <declaration-list> inside of <keyframe-block> accepts any CSS property
    /// > except those defined in this specification,
    /// > but does accept the `animation-play-state` property and interprets it specially.
    ///
    /// This will not actually parse Importance values, and will always set things
    /// to Importance::Normal. Parsing Importance values is the job of PropertyDeclarationParser,
    /// we only set them here so that we don't have to reallocate
    pub fn parse_into<'i, 't>(declarations: &mut SourcePropertyDeclaration,
                              id: PropertyId, name: CowRcStr<'i>,
                              context: &ParserContext, input: &mut Parser<'i, 't>)
                              -> Result<(), PropertyDeclarationParseError<'i>> {
        assert!(declarations.is_empty());
        let start = input.state();
        match id {
            PropertyId::Custom(property_name) => {
                // FIXME: fully implement https://github.com/w3c/csswg-drafts/issues/774
                // before adding skip_whitespace here.
                // This probably affects some test results.
                let value = match input.try(|i| CSSWideKeyword::parse(i)) {
                    Ok(keyword) => DeclaredValueOwned::CSSWideKeyword(keyword),
                    Err(()) => match ::custom_properties::SpecifiedValue::parse(context, input) {
                        Ok(value) => DeclaredValueOwned::Value(value),
                        Err(e) => return Err(PropertyDeclarationParseError::InvalidValue(name.to_string().into(),
                        ValueParseError::from_parse_error(e))),
                    }
                };
                declarations.push(PropertyDeclaration::Custom(property_name, value));
                Ok(())
            }
            PropertyId::Longhand(id) => {
                input.skip_whitespace();  // Unnecessary for correctness, but may help try() rewind less.
                input.try(|i| CSSWideKeyword::parse(i)).map(|keyword| {
                    PropertyDeclaration::CSSWideKeyword(id, keyword)
                }).or_else(|()| {
                    input.look_for_var_functions();
                    input.parse_entirely(|input| id.parse_value(context, input))
                    .or_else(|err| {
                        while let Ok(_) = input.next() {}  // Look for var() after the error.
                        if input.seen_var_functions() {
                            input.reset(&start);
                            let (first_token_type, css) =
                                ::custom_properties::parse_non_custom_with_var(input).map_err(|e| {
                                    PropertyDeclarationParseError::InvalidValue(name,
                                        ValueParseError::from_parse_error(e))
                                })?;
                            Ok(PropertyDeclaration::WithVariables(id, Arc::new(UnparsedValue {
                                css: css.into_owned(),
                                first_token_type: first_token_type,
                                url_data: context.url_data.clone(),
                                from_shorthand: None,
                            })))
                        } else {
                            Err(PropertyDeclarationParseError::InvalidValue(name,
                                ValueParseError::from_parse_error(err)))
                        }
                    })
                }).map(|declaration| {
                    declarations.push(declaration)
                })
            }
            PropertyId::Shorthand(id) => {
                input.skip_whitespace();  // Unnecessary for correctness, but may help try() rewind less.
                if let Ok(keyword) = input.try(|i| CSSWideKeyword::parse(i)) {
                    if id == ShorthandId::All {
                        declarations.all_shorthand = AllShorthand::CSSWideKeyword(keyword)
                    } else {
                        for &longhand in id.longhands() {
                            declarations.push(PropertyDeclaration::CSSWideKeyword(longhand, keyword))
                        }
                    }
                    Ok(())
                } else {
                    input.look_for_var_functions();
                    // Not using parse_entirely here: each text_decoration::parse_into function
                    // needs to do so *before* pushing to `declarations`.
                    id.parse_into(declarations, context, input).or_else(|err| {
                        while let Ok(_) = input.next() {}  // Look for var() after the error.
                        if input.seen_var_functions() {
                            input.reset(&start);
                            let (first_token_type, css) =
                                ::custom_properties::parse_non_custom_with_var(input).map_err(|e| {
                                    PropertyDeclarationParseError::InvalidValue(name,
                                        ValueParseError::from_parse_error(e))
                                })?;
                            let unparsed = Arc::new(UnparsedValue {
                                css: css.into_owned(),
                                first_token_type: first_token_type,
                                url_data: context.url_data.clone(),
                                from_shorthand: Some(id),
                            });
                            if id == ShorthandId::All {
                                declarations.all_shorthand = AllShorthand::WithVariables(unparsed)
                            } else {
                                for &longhand in id.longhands() {
                                    declarations.push(
                                        PropertyDeclaration::WithVariables(longhand, unparsed.clone())
                                    )
                                }
                            }
                            Ok(())
                        } else {
                            Err(PropertyDeclarationParseError::InvalidValue(name,
                                ValueParseError::from_parse_error(err)))
                        }
                    })
                }
            }
        }
    }
}

const MAX_SUB_PROPERTIES_PER_SHORTHAND_EXCEPT_ALL: usize =
    21;

type SourcePropertyDeclarationArray =
    [PropertyDeclaration; MAX_SUB_PROPERTIES_PER_SHORTHAND_EXCEPT_ALL];

/// A stack-allocated vector of `PropertyDeclaration`
/// large enough to parse one CSS `key: value` declaration.
/// (Shorthands expand to multiple `PropertyDeclaration`s.)
pub struct SourcePropertyDeclaration {
    declarations: ::arrayvec::ArrayVec<SourcePropertyDeclarationArray>,

    /// Stored separately to keep MAX_SUB_PROPERTIES_PER_SHORTHAND_EXCEPT_ALL smaller.
    all_shorthand: AllShorthand,
}

impl SourcePropertyDeclaration {
    /// Create one. It’s big, try not to move it around.
    #[inline]
    pub fn new() -> Self {
        SourcePropertyDeclaration {
            declarations: ::arrayvec::ArrayVec::new(),
            all_shorthand: AllShorthand::NotSet,
        }
    }

    /// Similar to Vec::drain: leaves this empty when the return value is dropped.
    pub fn drain(&mut self) -> SourcePropertyDeclarationDrain {
        SourcePropertyDeclarationDrain {
            declarations: self.declarations.drain(..),
            all_shorthand: mem::replace(&mut self.all_shorthand, AllShorthand::NotSet),
        }
    }

    /// Reset to initial state
    pub fn clear(&mut self) {
        self.declarations.clear();
        self.all_shorthand = AllShorthand::NotSet;
    }

    fn is_empty(&self) -> bool {
        self.declarations.is_empty() && matches!(self.all_shorthand, AllShorthand::NotSet)
    }

    fn push(&mut self, declaration: PropertyDeclaration) {
        let over_capacity = self.declarations.push(declaration).is_some();
        debug_assert!(!over_capacity);
    }
}

/// Return type of SourcePropertyDeclaration::drain
pub struct SourcePropertyDeclarationDrain<'a> {
    declarations: ::arrayvec::Drain<'a, SourcePropertyDeclarationArray>,
    all_shorthand: AllShorthand,
}

enum AllShorthand {
    NotSet,
    CSSWideKeyword(CSSWideKeyword),
    WithVariables(Arc<UnparsedValue>)
}

#[cfg(feature = "gecko")]
pub use gecko_properties::style_structs;

/// The module where all the style structs are defined.
#[cfg(feature = "servo")]
pub mod style_structs {
    use fnv::FnvHasher;
    use super::longhands;
    use std::hash::{Hash, Hasher};
    use logical_geometry::WritingMode;
    use media_queries::Device;
    use values::computed::NonNegativeLength;

        #[derive(Clone, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        /// The Background style struct.
        pub struct Background {
                /// The background-color computed value.
                pub background_color: longhands::background_color::computed_value::T,
                /// The background-image computed value.
                pub background_image: longhands::background_image::computed_value::T,
                /// The background-position-x computed value.
                pub background_position_x: longhands::background_position_x::computed_value::T,
                /// The background-position-y computed value.
                pub background_position_y: longhands::background_position_y::computed_value::T,
                /// The background-repeat computed value.
                pub background_repeat: longhands::background_repeat::computed_value::T,
                /// The background-attachment computed value.
                pub background_attachment: longhands::background_attachment::computed_value::T,
                /// The background-clip computed value.
                pub background_clip: longhands::background_clip::computed_value::T,
                /// The background-origin computed value.
                pub background_origin: longhands::background_origin::computed_value::T,
                /// The background-size computed value.
                pub background_size: longhands::background_size::computed_value::T,
                /// The background-blend-mode computed value.
                pub background_blend_mode: longhands::background_blend_mode::computed_value::T,
        }

        impl Background {
                        /// Set background-color.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_background_color(&mut self, v: longhands::background_color::computed_value::T) {
                            self.background_color = v;
                        }
                    /// Set background-color from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_background_color_from(&mut self, other: &Self) {
                        self.background_color = other.background_color.clone();
                    }

                    /// Reset background-color from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_background_color(&mut self, other: &Self) {
                        self.copy_background_color_from(other)
                    }

                    /// Get the computed value for background-color.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_background_color(&self) -> longhands::background_color::computed_value::T {
                        self.background_color.clone()
                    }
                        /// Set background-image.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_background_image<I>(&mut self, v: I)
                            where I: IntoIterator<Item = longhands::background_image
                                                                  ::computed_value::single_value::T>,
                                  I::IntoIter: ExactSizeIterator
                        {
                            self.background_image = longhands::background_image::computed_value
                                                              ::T(v.into_iter().collect());
                        }
                    /// Set background-image from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_background_image_from(&mut self, other: &Self) {
                        self.background_image = other.background_image.clone();
                    }

                    /// Reset background-image from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_background_image(&mut self, other: &Self) {
                        self.copy_background_image_from(other)
                    }

                    /// Get the computed value for background-image.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_background_image(&self) -> longhands::background_image::computed_value::T {
                        self.background_image.clone()
                    }
                        /// Set background-position-x.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_background_position_x<I>(&mut self, v: I)
                            where I: IntoIterator<Item = longhands::background_position_x
                                                                  ::computed_value::single_value::T>,
                                  I::IntoIter: ExactSizeIterator
                        {
                            self.background_position_x = longhands::background_position_x::computed_value
                                                              ::T(v.into_iter().collect());
                        }
                    /// Set background-position-x from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_background_position_x_from(&mut self, other: &Self) {
                        self.background_position_x = other.background_position_x.clone();
                    }

                    /// Reset background-position-x from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_background_position_x(&mut self, other: &Self) {
                        self.copy_background_position_x_from(other)
                    }

                    /// Get the computed value for background-position-x.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_background_position_x(&self) -> longhands::background_position_x::computed_value::T {
                        self.background_position_x.clone()
                    }
                        /// Set background-position-y.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_background_position_y<I>(&mut self, v: I)
                            where I: IntoIterator<Item = longhands::background_position_y
                                                                  ::computed_value::single_value::T>,
                                  I::IntoIter: ExactSizeIterator
                        {
                            self.background_position_y = longhands::background_position_y::computed_value
                                                              ::T(v.into_iter().collect());
                        }
                    /// Set background-position-y from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_background_position_y_from(&mut self, other: &Self) {
                        self.background_position_y = other.background_position_y.clone();
                    }

                    /// Reset background-position-y from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_background_position_y(&mut self, other: &Self) {
                        self.copy_background_position_y_from(other)
                    }

                    /// Get the computed value for background-position-y.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_background_position_y(&self) -> longhands::background_position_y::computed_value::T {
                        self.background_position_y.clone()
                    }
                        /// Set background-repeat.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_background_repeat<I>(&mut self, v: I)
                            where I: IntoIterator<Item = longhands::background_repeat
                                                                  ::computed_value::single_value::T>,
                                  I::IntoIter: ExactSizeIterator
                        {
                            self.background_repeat = longhands::background_repeat::computed_value
                                                              ::T(v.into_iter().collect());
                        }
                    /// Set background-repeat from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_background_repeat_from(&mut self, other: &Self) {
                        self.background_repeat = other.background_repeat.clone();
                    }

                    /// Reset background-repeat from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_background_repeat(&mut self, other: &Self) {
                        self.copy_background_repeat_from(other)
                    }

                    /// Get the computed value for background-repeat.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_background_repeat(&self) -> longhands::background_repeat::computed_value::T {
                        self.background_repeat.clone()
                    }
                        /// Set background-attachment.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_background_attachment<I>(&mut self, v: I)
                            where I: IntoIterator<Item = longhands::background_attachment
                                                                  ::computed_value::single_value::T>,
                                  I::IntoIter: ExactSizeIterator
                        {
                            self.background_attachment = longhands::background_attachment::computed_value
                                                              ::T(v.into_iter().collect());
                        }
                    /// Set background-attachment from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_background_attachment_from(&mut self, other: &Self) {
                        self.background_attachment = other.background_attachment.clone();
                    }

                    /// Reset background-attachment from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_background_attachment(&mut self, other: &Self) {
                        self.copy_background_attachment_from(other)
                    }

                    /// Get the computed value for background-attachment.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_background_attachment(&self) -> longhands::background_attachment::computed_value::T {
                        self.background_attachment.clone()
                    }
                        /// Set background-clip.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_background_clip<I>(&mut self, v: I)
                            where I: IntoIterator<Item = longhands::background_clip
                                                                  ::computed_value::single_value::T>,
                                  I::IntoIter: ExactSizeIterator
                        {
                            self.background_clip = longhands::background_clip::computed_value
                                                              ::T(v.into_iter().collect());
                        }
                    /// Set background-clip from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_background_clip_from(&mut self, other: &Self) {
                        self.background_clip = other.background_clip.clone();
                    }

                    /// Reset background-clip from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_background_clip(&mut self, other: &Self) {
                        self.copy_background_clip_from(other)
                    }

                    /// Get the computed value for background-clip.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_background_clip(&self) -> longhands::background_clip::computed_value::T {
                        self.background_clip.clone()
                    }
                        /// Set background-origin.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_background_origin<I>(&mut self, v: I)
                            where I: IntoIterator<Item = longhands::background_origin
                                                                  ::computed_value::single_value::T>,
                                  I::IntoIter: ExactSizeIterator
                        {
                            self.background_origin = longhands::background_origin::computed_value
                                                              ::T(v.into_iter().collect());
                        }
                    /// Set background-origin from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_background_origin_from(&mut self, other: &Self) {
                        self.background_origin = other.background_origin.clone();
                    }

                    /// Reset background-origin from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_background_origin(&mut self, other: &Self) {
                        self.copy_background_origin_from(other)
                    }

                    /// Get the computed value for background-origin.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_background_origin(&self) -> longhands::background_origin::computed_value::T {
                        self.background_origin.clone()
                    }
                        /// Set background-size.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_background_size<I>(&mut self, v: I)
                            where I: IntoIterator<Item = longhands::background_size
                                                                  ::computed_value::single_value::T>,
                                  I::IntoIter: ExactSizeIterator
                        {
                            self.background_size = longhands::background_size::computed_value
                                                              ::T(v.into_iter().collect());
                        }
                    /// Set background-size from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_background_size_from(&mut self, other: &Self) {
                        self.background_size = other.background_size.clone();
                    }

                    /// Reset background-size from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_background_size(&mut self, other: &Self) {
                        self.copy_background_size_from(other)
                    }

                    /// Get the computed value for background-size.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_background_size(&self) -> longhands::background_size::computed_value::T {
                        self.background_size.clone()
                    }
                        /// Set background-blend-mode.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_background_blend_mode<I>(&mut self, v: I)
                            where I: IntoIterator<Item = longhands::background_blend_mode
                                                                  ::computed_value::single_value::T>,
                                  I::IntoIter: ExactSizeIterator
                        {
                            self.background_blend_mode = longhands::background_blend_mode::computed_value
                                                              ::T(v.into_iter().collect());
                        }
                    /// Set background-blend-mode from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_background_blend_mode_from(&mut self, other: &Self) {
                        self.background_blend_mode = other.background_blend_mode.clone();
                    }

                    /// Reset background-blend-mode from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_background_blend_mode(&mut self, other: &Self) {
                        self.copy_background_blend_mode_from(other)
                    }

                    /// Get the computed value for background-blend-mode.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_background_blend_mode(&self) -> longhands::background_blend_mode::computed_value::T {
                        self.background_blend_mode.clone()
                    }
        }

        #[derive(Clone, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        /// The Border style struct.
        pub struct Border {
                /// The border-top-color computed value.
                pub border_top_color: longhands::border_top_color::computed_value::T,
                /// The border-top-style computed value.
                pub border_top_style: longhands::border_top_style::computed_value::T,
                /// The border-top-width computed value.
                pub border_top_width: longhands::border_top_width::computed_value::T,
                /// The border-left-color computed value.
                pub border_left_color: longhands::border_left_color::computed_value::T,
                /// The border-left-style computed value.
                pub border_left_style: longhands::border_left_style::computed_value::T,
                /// The border-left-width computed value.
                pub border_left_width: longhands::border_left_width::computed_value::T,
                /// The border-bottom-color computed value.
                pub border_bottom_color: longhands::border_bottom_color::computed_value::T,
                /// The border-bottom-style computed value.
                pub border_bottom_style: longhands::border_bottom_style::computed_value::T,
                /// The border-bottom-width computed value.
                pub border_bottom_width: longhands::border_bottom_width::computed_value::T,
                /// The border-right-color computed value.
                pub border_right_color: longhands::border_right_color::computed_value::T,
                /// The border-right-style computed value.
                pub border_right_style: longhands::border_right_style::computed_value::T,
                /// The border-right-width computed value.
                pub border_right_width: longhands::border_right_width::computed_value::T,
                /// The border-block-start-color computed value.
                pub border_block_start_color: longhands::border_block_start_color::computed_value::T,
                /// The border-block-start-style computed value.
                pub border_block_start_style: longhands::border_block_start_style::computed_value::T,
                /// The border-block-start-width computed value.
                pub border_block_start_width: longhands::border_block_start_width::computed_value::T,
                /// The border-block-end-color computed value.
                pub border_block_end_color: longhands::border_block_end_color::computed_value::T,
                /// The border-block-end-style computed value.
                pub border_block_end_style: longhands::border_block_end_style::computed_value::T,
                /// The border-block-end-width computed value.
                pub border_block_end_width: longhands::border_block_end_width::computed_value::T,
                /// The border-inline-start-color computed value.
                pub border_inline_start_color: longhands::border_inline_start_color::computed_value::T,
                /// The border-inline-start-style computed value.
                pub border_inline_start_style: longhands::border_inline_start_style::computed_value::T,
                /// The border-inline-start-width computed value.
                pub border_inline_start_width: longhands::border_inline_start_width::computed_value::T,
                /// The border-inline-end-color computed value.
                pub border_inline_end_color: longhands::border_inline_end_color::computed_value::T,
                /// The border-inline-end-style computed value.
                pub border_inline_end_style: longhands::border_inline_end_style::computed_value::T,
                /// The border-inline-end-width computed value.
                pub border_inline_end_width: longhands::border_inline_end_width::computed_value::T,
                /// The border-top-left-radius computed value.
                pub border_top_left_radius: longhands::border_top_left_radius::computed_value::T,
                /// The border-top-right-radius computed value.
                pub border_top_right_radius: longhands::border_top_right_radius::computed_value::T,
                /// The border-bottom-right-radius computed value.
                pub border_bottom_right_radius: longhands::border_bottom_right_radius::computed_value::T,
                /// The border-bottom-left-radius computed value.
                pub border_bottom_left_radius: longhands::border_bottom_left_radius::computed_value::T,
                /// The -moz-border-top-colors computed value.
                pub _moz_border_top_colors: longhands::_moz_border_top_colors::computed_value::T,
                /// The -moz-border-left-colors computed value.
                pub _moz_border_left_colors: longhands::_moz_border_left_colors::computed_value::T,
                /// The -moz-border-bottom-colors computed value.
                pub _moz_border_bottom_colors: longhands::_moz_border_bottom_colors::computed_value::T,
                /// The -moz-border-right-colors computed value.
                pub _moz_border_right_colors: longhands::_moz_border_right_colors::computed_value::T,
                /// The box-decoration-break computed value.
                pub box_decoration_break: longhands::box_decoration_break::computed_value::T,
                /// The -moz-float-edge computed value.
                pub _moz_float_edge: longhands::_moz_float_edge::computed_value::T,
                /// The border-image-source computed value.
                pub border_image_source: longhands::border_image_source::computed_value::T,
                /// The border-image-outset computed value.
                pub border_image_outset: longhands::border_image_outset::computed_value::T,
                /// The border-image-repeat computed value.
                pub border_image_repeat: longhands::border_image_repeat::computed_value::T,
                /// The border-image-width computed value.
                pub border_image_width: longhands::border_image_width::computed_value::T,
                /// The border-image-slice computed value.
                pub border_image_slice: longhands::border_image_slice::computed_value::T,
        }

        impl Border {
                        /// Set border-top-color.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_border_top_color(&mut self, v: longhands::border_top_color::computed_value::T) {
                            self.border_top_color = v;
                        }
                    /// Set border-top-color from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_border_top_color_from(&mut self, other: &Self) {
                        self.border_top_color = other.border_top_color.clone();
                    }

                    /// Reset border-top-color from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_border_top_color(&mut self, other: &Self) {
                        self.copy_border_top_color_from(other)
                    }

                    /// Get the computed value for border-top-color.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_border_top_color(&self) -> longhands::border_top_color::computed_value::T {
                        self.border_top_color.clone()
                    }
                        /// Set border-top-style.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_border_top_style(&mut self, v: longhands::border_top_style::computed_value::T) {
                            self.border_top_style = v;
                        }
                    /// Set border-top-style from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_border_top_style_from(&mut self, other: &Self) {
                        self.border_top_style = other.border_top_style.clone();
                    }

                    /// Reset border-top-style from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_border_top_style(&mut self, other: &Self) {
                        self.copy_border_top_style_from(other)
                    }

                    /// Get the computed value for border-top-style.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_border_top_style(&self) -> longhands::border_top_style::computed_value::T {
                        self.border_top_style.clone()
                    }
                        /// Set border-top-width.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_border_top_width(&mut self, v: longhands::border_top_width::computed_value::T) {
                            self.border_top_width = v;
                        }
                    /// Set border-top-width from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_border_top_width_from(&mut self, other: &Self) {
                        self.border_top_width = other.border_top_width.clone();
                    }

                    /// Reset border-top-width from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_border_top_width(&mut self, other: &Self) {
                        self.copy_border_top_width_from(other)
                    }

                    /// Get the computed value for border-top-width.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_border_top_width(&self) -> longhands::border_top_width::computed_value::T {
                        self.border_top_width.clone()
                    }
                        /// Set border-left-color.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_border_left_color(&mut self, v: longhands::border_left_color::computed_value::T) {
                            self.border_left_color = v;
                        }
                    /// Set border-left-color from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_border_left_color_from(&mut self, other: &Self) {
                        self.border_left_color = other.border_left_color.clone();
                    }

                    /// Reset border-left-color from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_border_left_color(&mut self, other: &Self) {
                        self.copy_border_left_color_from(other)
                    }

                    /// Get the computed value for border-left-color.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_border_left_color(&self) -> longhands::border_left_color::computed_value::T {
                        self.border_left_color.clone()
                    }
                        /// Set border-left-style.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_border_left_style(&mut self, v: longhands::border_left_style::computed_value::T) {
                            self.border_left_style = v;
                        }
                    /// Set border-left-style from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_border_left_style_from(&mut self, other: &Self) {
                        self.border_left_style = other.border_left_style.clone();
                    }

                    /// Reset border-left-style from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_border_left_style(&mut self, other: &Self) {
                        self.copy_border_left_style_from(other)
                    }

                    /// Get the computed value for border-left-style.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_border_left_style(&self) -> longhands::border_left_style::computed_value::T {
                        self.border_left_style.clone()
                    }
                        /// Set border-left-width.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_border_left_width(&mut self, v: longhands::border_left_width::computed_value::T) {
                            self.border_left_width = v;
                        }
                    /// Set border-left-width from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_border_left_width_from(&mut self, other: &Self) {
                        self.border_left_width = other.border_left_width.clone();
                    }

                    /// Reset border-left-width from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_border_left_width(&mut self, other: &Self) {
                        self.copy_border_left_width_from(other)
                    }

                    /// Get the computed value for border-left-width.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_border_left_width(&self) -> longhands::border_left_width::computed_value::T {
                        self.border_left_width.clone()
                    }
                        /// Set border-bottom-color.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_border_bottom_color(&mut self, v: longhands::border_bottom_color::computed_value::T) {
                            self.border_bottom_color = v;
                        }
                    /// Set border-bottom-color from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_border_bottom_color_from(&mut self, other: &Self) {
                        self.border_bottom_color = other.border_bottom_color.clone();
                    }

                    /// Reset border-bottom-color from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_border_bottom_color(&mut self, other: &Self) {
                        self.copy_border_bottom_color_from(other)
                    }

                    /// Get the computed value for border-bottom-color.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_border_bottom_color(&self) -> longhands::border_bottom_color::computed_value::T {
                        self.border_bottom_color.clone()
                    }
                        /// Set border-bottom-style.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_border_bottom_style(&mut self, v: longhands::border_bottom_style::computed_value::T) {
                            self.border_bottom_style = v;
                        }
                    /// Set border-bottom-style from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_border_bottom_style_from(&mut self, other: &Self) {
                        self.border_bottom_style = other.border_bottom_style.clone();
                    }

                    /// Reset border-bottom-style from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_border_bottom_style(&mut self, other: &Self) {
                        self.copy_border_bottom_style_from(other)
                    }

                    /// Get the computed value for border-bottom-style.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_border_bottom_style(&self) -> longhands::border_bottom_style::computed_value::T {
                        self.border_bottom_style.clone()
                    }
                        /// Set border-bottom-width.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_border_bottom_width(&mut self, v: longhands::border_bottom_width::computed_value::T) {
                            self.border_bottom_width = v;
                        }
                    /// Set border-bottom-width from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_border_bottom_width_from(&mut self, other: &Self) {
                        self.border_bottom_width = other.border_bottom_width.clone();
                    }

                    /// Reset border-bottom-width from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_border_bottom_width(&mut self, other: &Self) {
                        self.copy_border_bottom_width_from(other)
                    }

                    /// Get the computed value for border-bottom-width.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_border_bottom_width(&self) -> longhands::border_bottom_width::computed_value::T {
                        self.border_bottom_width.clone()
                    }
                        /// Set border-right-color.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_border_right_color(&mut self, v: longhands::border_right_color::computed_value::T) {
                            self.border_right_color = v;
                        }
                    /// Set border-right-color from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_border_right_color_from(&mut self, other: &Self) {
                        self.border_right_color = other.border_right_color.clone();
                    }

                    /// Reset border-right-color from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_border_right_color(&mut self, other: &Self) {
                        self.copy_border_right_color_from(other)
                    }

                    /// Get the computed value for border-right-color.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_border_right_color(&self) -> longhands::border_right_color::computed_value::T {
                        self.border_right_color.clone()
                    }
                        /// Set border-right-style.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_border_right_style(&mut self, v: longhands::border_right_style::computed_value::T) {
                            self.border_right_style = v;
                        }
                    /// Set border-right-style from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_border_right_style_from(&mut self, other: &Self) {
                        self.border_right_style = other.border_right_style.clone();
                    }

                    /// Reset border-right-style from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_border_right_style(&mut self, other: &Self) {
                        self.copy_border_right_style_from(other)
                    }

                    /// Get the computed value for border-right-style.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_border_right_style(&self) -> longhands::border_right_style::computed_value::T {
                        self.border_right_style.clone()
                    }
                        /// Set border-right-width.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_border_right_width(&mut self, v: longhands::border_right_width::computed_value::T) {
                            self.border_right_width = v;
                        }
                    /// Set border-right-width from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_border_right_width_from(&mut self, other: &Self) {
                        self.border_right_width = other.border_right_width.clone();
                    }

                    /// Reset border-right-width from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_border_right_width(&mut self, other: &Self) {
                        self.copy_border_right_width_from(other)
                    }

                    /// Get the computed value for border-right-width.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_border_right_width(&self) -> longhands::border_right_width::computed_value::T {
                        self.border_right_width.clone()
                    }
                    
    /// Set the appropriate physical property for border-block-start-color given a writing mode.
    pub fn set_border_block_start_color(&mut self,
                                      v: longhands::border_block_start_color::computed_value::T,
                                      wm: WritingMode) {
        
    
        use logical_geometry::PhysicalSide;
        match wm.block_start_physical_side() {
                PhysicalSide::Top => {
                    
                self.set_border_top_color(v)
            
                }
                PhysicalSide::Left => {
                    
                self.set_border_left_color(v)
            
                }
                PhysicalSide::Bottom => {
                    
                self.set_border_bottom_color(v)
            
                }
                PhysicalSide::Right => {
                    
                self.set_border_right_color(v)
            
                }
        }

    }

    /// Copy the appropriate physical property from another struct for border-block-start-color
    /// given a writing mode.
    pub fn copy_border_block_start_color_from(&mut self,
                                            other: &Self,
                                            wm: WritingMode) {
        
    
        use logical_geometry::PhysicalSide;
        match wm.block_start_physical_side() {
                PhysicalSide::Top => {
                    
                self.copy_border_top_color_from(other)
            
                }
                PhysicalSide::Left => {
                    
                self.copy_border_left_color_from(other)
            
                }
                PhysicalSide::Bottom => {
                    
                self.copy_border_bottom_color_from(other)
            
                }
                PhysicalSide::Right => {
                    
                self.copy_border_right_color_from(other)
            
                }
        }

    }

    /// Copy the appropriate physical property from another struct for border-block-start-color
    /// given a writing mode.
    pub fn reset_border_block_start_color(&mut self,
                                        other: &Self,
                                        wm: WritingMode) {
        self.copy_border_block_start_color_from(other, wm)
    }

    /// Get the computed value for the appropriate physical property for
    /// border-block-start-color given a writing mode.
    pub fn clone_border_block_start_color(&self, wm: WritingMode)
        -> longhands::border_block_start_color::computed_value::T {
    
    
        use logical_geometry::PhysicalSide;
        match wm.block_start_physical_side() {
                PhysicalSide::Top => {
                    
            self.clone_border_top_color()
        
                }
                PhysicalSide::Left => {
                    
            self.clone_border_left_color()
        
                }
                PhysicalSide::Bottom => {
                    
            self.clone_border_bottom_color()
        
                }
                PhysicalSide::Right => {
                    
            self.clone_border_right_color()
        
                }
        }

    }

                    
    /// Set the appropriate physical property for border-block-start-style given a writing mode.
    pub fn set_border_block_start_style(&mut self,
                                      v: longhands::border_block_start_style::computed_value::T,
                                      wm: WritingMode) {
        
    
        use logical_geometry::PhysicalSide;
        match wm.block_start_physical_side() {
                PhysicalSide::Top => {
                    
                self.set_border_top_style(v)
            
                }
                PhysicalSide::Left => {
                    
                self.set_border_left_style(v)
            
                }
                PhysicalSide::Bottom => {
                    
                self.set_border_bottom_style(v)
            
                }
                PhysicalSide::Right => {
                    
                self.set_border_right_style(v)
            
                }
        }

    }

    /// Copy the appropriate physical property from another struct for border-block-start-style
    /// given a writing mode.
    pub fn copy_border_block_start_style_from(&mut self,
                                            other: &Self,
                                            wm: WritingMode) {
        
    
        use logical_geometry::PhysicalSide;
        match wm.block_start_physical_side() {
                PhysicalSide::Top => {
                    
                self.copy_border_top_style_from(other)
            
                }
                PhysicalSide::Left => {
                    
                self.copy_border_left_style_from(other)
            
                }
                PhysicalSide::Bottom => {
                    
                self.copy_border_bottom_style_from(other)
            
                }
                PhysicalSide::Right => {
                    
                self.copy_border_right_style_from(other)
            
                }
        }

    }

    /// Copy the appropriate physical property from another struct for border-block-start-style
    /// given a writing mode.
    pub fn reset_border_block_start_style(&mut self,
                                        other: &Self,
                                        wm: WritingMode) {
        self.copy_border_block_start_style_from(other, wm)
    }

    /// Get the computed value for the appropriate physical property for
    /// border-block-start-style given a writing mode.
    pub fn clone_border_block_start_style(&self, wm: WritingMode)
        -> longhands::border_block_start_style::computed_value::T {
    
    
        use logical_geometry::PhysicalSide;
        match wm.block_start_physical_side() {
                PhysicalSide::Top => {
                    
            self.clone_border_top_style()
        
                }
                PhysicalSide::Left => {
                    
            self.clone_border_left_style()
        
                }
                PhysicalSide::Bottom => {
                    
            self.clone_border_bottom_style()
        
                }
                PhysicalSide::Right => {
                    
            self.clone_border_right_style()
        
                }
        }

    }

                    
    /// Set the appropriate physical property for border-block-start-width given a writing mode.
    pub fn set_border_block_start_width(&mut self,
                                      v: longhands::border_block_start_width::computed_value::T,
                                      wm: WritingMode) {
        
    
        use logical_geometry::PhysicalSide;
        match wm.block_start_physical_side() {
                PhysicalSide::Top => {
                    
                self.set_border_top_width(v)
            
                }
                PhysicalSide::Left => {
                    
                self.set_border_left_width(v)
            
                }
                PhysicalSide::Bottom => {
                    
                self.set_border_bottom_width(v)
            
                }
                PhysicalSide::Right => {
                    
                self.set_border_right_width(v)
            
                }
        }

    }

    /// Copy the appropriate physical property from another struct for border-block-start-width
    /// given a writing mode.
    pub fn copy_border_block_start_width_from(&mut self,
                                            other: &Self,
                                            wm: WritingMode) {
        
    
        use logical_geometry::PhysicalSide;
        match wm.block_start_physical_side() {
                PhysicalSide::Top => {
                    
                self.copy_border_top_width_from(other)
            
                }
                PhysicalSide::Left => {
                    
                self.copy_border_left_width_from(other)
            
                }
                PhysicalSide::Bottom => {
                    
                self.copy_border_bottom_width_from(other)
            
                }
                PhysicalSide::Right => {
                    
                self.copy_border_right_width_from(other)
            
                }
        }

    }

    /// Copy the appropriate physical property from another struct for border-block-start-width
    /// given a writing mode.
    pub fn reset_border_block_start_width(&mut self,
                                        other: &Self,
                                        wm: WritingMode) {
        self.copy_border_block_start_width_from(other, wm)
    }

    /// Get the computed value for the appropriate physical property for
    /// border-block-start-width given a writing mode.
    pub fn clone_border_block_start_width(&self, wm: WritingMode)
        -> longhands::border_block_start_width::computed_value::T {
    
    
        use logical_geometry::PhysicalSide;
        match wm.block_start_physical_side() {
                PhysicalSide::Top => {
                    
            self.clone_border_top_width()
        
                }
                PhysicalSide::Left => {
                    
            self.clone_border_left_width()
        
                }
                PhysicalSide::Bottom => {
                    
            self.clone_border_bottom_width()
        
                }
                PhysicalSide::Right => {
                    
            self.clone_border_right_width()
        
                }
        }

    }

                    
    /// Set the appropriate physical property for border-block-end-color given a writing mode.
    pub fn set_border_block_end_color(&mut self,
                                      v: longhands::border_block_end_color::computed_value::T,
                                      wm: WritingMode) {
        
    
        use logical_geometry::PhysicalSide;
        match wm.block_end_physical_side() {
                PhysicalSide::Top => {
                    
                self.set_border_top_color(v)
            
                }
                PhysicalSide::Left => {
                    
                self.set_border_left_color(v)
            
                }
                PhysicalSide::Bottom => {
                    
                self.set_border_bottom_color(v)
            
                }
                PhysicalSide::Right => {
                    
                self.set_border_right_color(v)
            
                }
        }

    }

    /// Copy the appropriate physical property from another struct for border-block-end-color
    /// given a writing mode.
    pub fn copy_border_block_end_color_from(&mut self,
                                            other: &Self,
                                            wm: WritingMode) {
        
    
        use logical_geometry::PhysicalSide;
        match wm.block_end_physical_side() {
                PhysicalSide::Top => {
                    
                self.copy_border_top_color_from(other)
            
                }
                PhysicalSide::Left => {
                    
                self.copy_border_left_color_from(other)
            
                }
                PhysicalSide::Bottom => {
                    
                self.copy_border_bottom_color_from(other)
            
                }
                PhysicalSide::Right => {
                    
                self.copy_border_right_color_from(other)
            
                }
        }

    }

    /// Copy the appropriate physical property from another struct for border-block-end-color
    /// given a writing mode.
    pub fn reset_border_block_end_color(&mut self,
                                        other: &Self,
                                        wm: WritingMode) {
        self.copy_border_block_end_color_from(other, wm)
    }

    /// Get the computed value for the appropriate physical property for
    /// border-block-end-color given a writing mode.
    pub fn clone_border_block_end_color(&self, wm: WritingMode)
        -> longhands::border_block_end_color::computed_value::T {
    
    
        use logical_geometry::PhysicalSide;
        match wm.block_end_physical_side() {
                PhysicalSide::Top => {
                    
            self.clone_border_top_color()
        
                }
                PhysicalSide::Left => {
                    
            self.clone_border_left_color()
        
                }
                PhysicalSide::Bottom => {
                    
            self.clone_border_bottom_color()
        
                }
                PhysicalSide::Right => {
                    
            self.clone_border_right_color()
        
                }
        }

    }

                    
    /// Set the appropriate physical property for border-block-end-style given a writing mode.
    pub fn set_border_block_end_style(&mut self,
                                      v: longhands::border_block_end_style::computed_value::T,
                                      wm: WritingMode) {
        
    
        use logical_geometry::PhysicalSide;
        match wm.block_end_physical_side() {
                PhysicalSide::Top => {
                    
                self.set_border_top_style(v)
            
                }
                PhysicalSide::Left => {
                    
                self.set_border_left_style(v)
            
                }
                PhysicalSide::Bottom => {
                    
                self.set_border_bottom_style(v)
            
                }
                PhysicalSide::Right => {
                    
                self.set_border_right_style(v)
            
                }
        }

    }

    /// Copy the appropriate physical property from another struct for border-block-end-style
    /// given a writing mode.
    pub fn copy_border_block_end_style_from(&mut self,
                                            other: &Self,
                                            wm: WritingMode) {
        
    
        use logical_geometry::PhysicalSide;
        match wm.block_end_physical_side() {
                PhysicalSide::Top => {
                    
                self.copy_border_top_style_from(other)
            
                }
                PhysicalSide::Left => {
                    
                self.copy_border_left_style_from(other)
            
                }
                PhysicalSide::Bottom => {
                    
                self.copy_border_bottom_style_from(other)
            
                }
                PhysicalSide::Right => {
                    
                self.copy_border_right_style_from(other)
            
                }
        }

    }

    /// Copy the appropriate physical property from another struct for border-block-end-style
    /// given a writing mode.
    pub fn reset_border_block_end_style(&mut self,
                                        other: &Self,
                                        wm: WritingMode) {
        self.copy_border_block_end_style_from(other, wm)
    }

    /// Get the computed value for the appropriate physical property for
    /// border-block-end-style given a writing mode.
    pub fn clone_border_block_end_style(&self, wm: WritingMode)
        -> longhands::border_block_end_style::computed_value::T {
    
    
        use logical_geometry::PhysicalSide;
        match wm.block_end_physical_side() {
                PhysicalSide::Top => {
                    
            self.clone_border_top_style()
        
                }
                PhysicalSide::Left => {
                    
            self.clone_border_left_style()
        
                }
                PhysicalSide::Bottom => {
                    
            self.clone_border_bottom_style()
        
                }
                PhysicalSide::Right => {
                    
            self.clone_border_right_style()
        
                }
        }

    }

                    
    /// Set the appropriate physical property for border-block-end-width given a writing mode.
    pub fn set_border_block_end_width(&mut self,
                                      v: longhands::border_block_end_width::computed_value::T,
                                      wm: WritingMode) {
        
    
        use logical_geometry::PhysicalSide;
        match wm.block_end_physical_side() {
                PhysicalSide::Top => {
                    
                self.set_border_top_width(v)
            
                }
                PhysicalSide::Left => {
                    
                self.set_border_left_width(v)
            
                }
                PhysicalSide::Bottom => {
                    
                self.set_border_bottom_width(v)
            
                }
                PhysicalSide::Right => {
                    
                self.set_border_right_width(v)
            
                }
        }

    }

    /// Copy the appropriate physical property from another struct for border-block-end-width
    /// given a writing mode.
    pub fn copy_border_block_end_width_from(&mut self,
                                            other: &Self,
                                            wm: WritingMode) {
        
    
        use logical_geometry::PhysicalSide;
        match wm.block_end_physical_side() {
                PhysicalSide::Top => {
                    
                self.copy_border_top_width_from(other)
            
                }
                PhysicalSide::Left => {
                    
                self.copy_border_left_width_from(other)
            
                }
                PhysicalSide::Bottom => {
                    
                self.copy_border_bottom_width_from(other)
            
                }
                PhysicalSide::Right => {
                    
                self.copy_border_right_width_from(other)
            
                }
        }

    }

    /// Copy the appropriate physical property from another struct for border-block-end-width
    /// given a writing mode.
    pub fn reset_border_block_end_width(&mut self,
                                        other: &Self,
                                        wm: WritingMode) {
        self.copy_border_block_end_width_from(other, wm)
    }

    /// Get the computed value for the appropriate physical property for
    /// border-block-end-width given a writing mode.
    pub fn clone_border_block_end_width(&self, wm: WritingMode)
        -> longhands::border_block_end_width::computed_value::T {
    
    
        use logical_geometry::PhysicalSide;
        match wm.block_end_physical_side() {
                PhysicalSide::Top => {
                    
            self.clone_border_top_width()
        
                }
                PhysicalSide::Left => {
                    
            self.clone_border_left_width()
        
                }
                PhysicalSide::Bottom => {
                    
            self.clone_border_bottom_width()
        
                }
                PhysicalSide::Right => {
                    
            self.clone_border_right_width()
        
                }
        }

    }

                    
    /// Set the appropriate physical property for border-inline-start-color given a writing mode.
    pub fn set_border_inline_start_color(&mut self,
                                      v: longhands::border_inline_start_color::computed_value::T,
                                      wm: WritingMode) {
        
    
        use logical_geometry::PhysicalSide;
        match wm.inline_start_physical_side() {
                PhysicalSide::Top => {
                    
                self.set_border_top_color(v)
            
                }
                PhysicalSide::Left => {
                    
                self.set_border_left_color(v)
            
                }
                PhysicalSide::Bottom => {
                    
                self.set_border_bottom_color(v)
            
                }
                PhysicalSide::Right => {
                    
                self.set_border_right_color(v)
            
                }
        }

    }

    /// Copy the appropriate physical property from another struct for border-inline-start-color
    /// given a writing mode.
    pub fn copy_border_inline_start_color_from(&mut self,
                                            other: &Self,
                                            wm: WritingMode) {
        
    
        use logical_geometry::PhysicalSide;
        match wm.inline_start_physical_side() {
                PhysicalSide::Top => {
                    
                self.copy_border_top_color_from(other)
            
                }
                PhysicalSide::Left => {
                    
                self.copy_border_left_color_from(other)
            
                }
                PhysicalSide::Bottom => {
                    
                self.copy_border_bottom_color_from(other)
            
                }
                PhysicalSide::Right => {
                    
                self.copy_border_right_color_from(other)
            
                }
        }

    }

    /// Copy the appropriate physical property from another struct for border-inline-start-color
    /// given a writing mode.
    pub fn reset_border_inline_start_color(&mut self,
                                        other: &Self,
                                        wm: WritingMode) {
        self.copy_border_inline_start_color_from(other, wm)
    }

    /// Get the computed value for the appropriate physical property for
    /// border-inline-start-color given a writing mode.
    pub fn clone_border_inline_start_color(&self, wm: WritingMode)
        -> longhands::border_inline_start_color::computed_value::T {
    
    
        use logical_geometry::PhysicalSide;
        match wm.inline_start_physical_side() {
                PhysicalSide::Top => {
                    
            self.clone_border_top_color()
        
                }
                PhysicalSide::Left => {
                    
            self.clone_border_left_color()
        
                }
                PhysicalSide::Bottom => {
                    
            self.clone_border_bottom_color()
        
                }
                PhysicalSide::Right => {
                    
            self.clone_border_right_color()
        
                }
        }

    }

                    
    /// Set the appropriate physical property for border-inline-start-style given a writing mode.
    pub fn set_border_inline_start_style(&mut self,
                                      v: longhands::border_inline_start_style::computed_value::T,
                                      wm: WritingMode) {
        
    
        use logical_geometry::PhysicalSide;
        match wm.inline_start_physical_side() {
                PhysicalSide::Top => {
                    
                self.set_border_top_style(v)
            
                }
                PhysicalSide::Left => {
                    
                self.set_border_left_style(v)
            
                }
                PhysicalSide::Bottom => {
                    
                self.set_border_bottom_style(v)
            
                }
                PhysicalSide::Right => {
                    
                self.set_border_right_style(v)
            
                }
        }

    }

    /// Copy the appropriate physical property from another struct for border-inline-start-style
    /// given a writing mode.
    pub fn copy_border_inline_start_style_from(&mut self,
                                            other: &Self,
                                            wm: WritingMode) {
        
    
        use logical_geometry::PhysicalSide;
        match wm.inline_start_physical_side() {
                PhysicalSide::Top => {
                    
                self.copy_border_top_style_from(other)
            
                }
                PhysicalSide::Left => {
                    
                self.copy_border_left_style_from(other)
            
                }
                PhysicalSide::Bottom => {
                    
                self.copy_border_bottom_style_from(other)
            
                }
                PhysicalSide::Right => {
                    
                self.copy_border_right_style_from(other)
            
                }
        }

    }

    /// Copy the appropriate physical property from another struct for border-inline-start-style
    /// given a writing mode.
    pub fn reset_border_inline_start_style(&mut self,
                                        other: &Self,
                                        wm: WritingMode) {
        self.copy_border_inline_start_style_from(other, wm)
    }

    /// Get the computed value for the appropriate physical property for
    /// border-inline-start-style given a writing mode.
    pub fn clone_border_inline_start_style(&self, wm: WritingMode)
        -> longhands::border_inline_start_style::computed_value::T {
    
    
        use logical_geometry::PhysicalSide;
        match wm.inline_start_physical_side() {
                PhysicalSide::Top => {
                    
            self.clone_border_top_style()
        
                }
                PhysicalSide::Left => {
                    
            self.clone_border_left_style()
        
                }
                PhysicalSide::Bottom => {
                    
            self.clone_border_bottom_style()
        
                }
                PhysicalSide::Right => {
                    
            self.clone_border_right_style()
        
                }
        }

    }

                    
    /// Set the appropriate physical property for border-inline-start-width given a writing mode.
    pub fn set_border_inline_start_width(&mut self,
                                      v: longhands::border_inline_start_width::computed_value::T,
                                      wm: WritingMode) {
        
    
        use logical_geometry::PhysicalSide;
        match wm.inline_start_physical_side() {
                PhysicalSide::Top => {
                    
                self.set_border_top_width(v)
            
                }
                PhysicalSide::Left => {
                    
                self.set_border_left_width(v)
            
                }
                PhysicalSide::Bottom => {
                    
                self.set_border_bottom_width(v)
            
                }
                PhysicalSide::Right => {
                    
                self.set_border_right_width(v)
            
                }
        }

    }

    /// Copy the appropriate physical property from another struct for border-inline-start-width
    /// given a writing mode.
    pub fn copy_border_inline_start_width_from(&mut self,
                                            other: &Self,
                                            wm: WritingMode) {
        
    
        use logical_geometry::PhysicalSide;
        match wm.inline_start_physical_side() {
                PhysicalSide::Top => {
                    
                self.copy_border_top_width_from(other)
            
                }
                PhysicalSide::Left => {
                    
                self.copy_border_left_width_from(other)
            
                }
                PhysicalSide::Bottom => {
                    
                self.copy_border_bottom_width_from(other)
            
                }
                PhysicalSide::Right => {
                    
                self.copy_border_right_width_from(other)
            
                }
        }

    }

    /// Copy the appropriate physical property from another struct for border-inline-start-width
    /// given a writing mode.
    pub fn reset_border_inline_start_width(&mut self,
                                        other: &Self,
                                        wm: WritingMode) {
        self.copy_border_inline_start_width_from(other, wm)
    }

    /// Get the computed value for the appropriate physical property for
    /// border-inline-start-width given a writing mode.
    pub fn clone_border_inline_start_width(&self, wm: WritingMode)
        -> longhands::border_inline_start_width::computed_value::T {
    
    
        use logical_geometry::PhysicalSide;
        match wm.inline_start_physical_side() {
                PhysicalSide::Top => {
                    
            self.clone_border_top_width()
        
                }
                PhysicalSide::Left => {
                    
            self.clone_border_left_width()
        
                }
                PhysicalSide::Bottom => {
                    
            self.clone_border_bottom_width()
        
                }
                PhysicalSide::Right => {
                    
            self.clone_border_right_width()
        
                }
        }

    }

                    
    /// Set the appropriate physical property for border-inline-end-color given a writing mode.
    pub fn set_border_inline_end_color(&mut self,
                                      v: longhands::border_inline_end_color::computed_value::T,
                                      wm: WritingMode) {
        
    
        use logical_geometry::PhysicalSide;
        match wm.inline_end_physical_side() {
                PhysicalSide::Top => {
                    
                self.set_border_top_color(v)
            
                }
                PhysicalSide::Left => {
                    
                self.set_border_left_color(v)
            
                }
                PhysicalSide::Bottom => {
                    
                self.set_border_bottom_color(v)
            
                }
                PhysicalSide::Right => {
                    
                self.set_border_right_color(v)
            
                }
        }

    }

    /// Copy the appropriate physical property from another struct for border-inline-end-color
    /// given a writing mode.
    pub fn copy_border_inline_end_color_from(&mut self,
                                            other: &Self,
                                            wm: WritingMode) {
        
    
        use logical_geometry::PhysicalSide;
        match wm.inline_end_physical_side() {
                PhysicalSide::Top => {
                    
                self.copy_border_top_color_from(other)
            
                }
                PhysicalSide::Left => {
                    
                self.copy_border_left_color_from(other)
            
                }
                PhysicalSide::Bottom => {
                    
                self.copy_border_bottom_color_from(other)
            
                }
                PhysicalSide::Right => {
                    
                self.copy_border_right_color_from(other)
            
                }
        }

    }

    /// Copy the appropriate physical property from another struct for border-inline-end-color
    /// given a writing mode.
    pub fn reset_border_inline_end_color(&mut self,
                                        other: &Self,
                                        wm: WritingMode) {
        self.copy_border_inline_end_color_from(other, wm)
    }

    /// Get the computed value for the appropriate physical property for
    /// border-inline-end-color given a writing mode.
    pub fn clone_border_inline_end_color(&self, wm: WritingMode)
        -> longhands::border_inline_end_color::computed_value::T {
    
    
        use logical_geometry::PhysicalSide;
        match wm.inline_end_physical_side() {
                PhysicalSide::Top => {
                    
            self.clone_border_top_color()
        
                }
                PhysicalSide::Left => {
                    
            self.clone_border_left_color()
        
                }
                PhysicalSide::Bottom => {
                    
            self.clone_border_bottom_color()
        
                }
                PhysicalSide::Right => {
                    
            self.clone_border_right_color()
        
                }
        }

    }

                    
    /// Set the appropriate physical property for border-inline-end-style given a writing mode.
    pub fn set_border_inline_end_style(&mut self,
                                      v: longhands::border_inline_end_style::computed_value::T,
                                      wm: WritingMode) {
        
    
        use logical_geometry::PhysicalSide;
        match wm.inline_end_physical_side() {
                PhysicalSide::Top => {
                    
                self.set_border_top_style(v)
            
                }
                PhysicalSide::Left => {
                    
                self.set_border_left_style(v)
            
                }
                PhysicalSide::Bottom => {
                    
                self.set_border_bottom_style(v)
            
                }
                PhysicalSide::Right => {
                    
                self.set_border_right_style(v)
            
                }
        }

    }

    /// Copy the appropriate physical property from another struct for border-inline-end-style
    /// given a writing mode.
    pub fn copy_border_inline_end_style_from(&mut self,
                                            other: &Self,
                                            wm: WritingMode) {
        
    
        use logical_geometry::PhysicalSide;
        match wm.inline_end_physical_side() {
                PhysicalSide::Top => {
                    
                self.copy_border_top_style_from(other)
            
                }
                PhysicalSide::Left => {
                    
                self.copy_border_left_style_from(other)
            
                }
                PhysicalSide::Bottom => {
                    
                self.copy_border_bottom_style_from(other)
            
                }
                PhysicalSide::Right => {
                    
                self.copy_border_right_style_from(other)
            
                }
        }

    }

    /// Copy the appropriate physical property from another struct for border-inline-end-style
    /// given a writing mode.
    pub fn reset_border_inline_end_style(&mut self,
                                        other: &Self,
                                        wm: WritingMode) {
        self.copy_border_inline_end_style_from(other, wm)
    }

    /// Get the computed value for the appropriate physical property for
    /// border-inline-end-style given a writing mode.
    pub fn clone_border_inline_end_style(&self, wm: WritingMode)
        -> longhands::border_inline_end_style::computed_value::T {
    
    
        use logical_geometry::PhysicalSide;
        match wm.inline_end_physical_side() {
                PhysicalSide::Top => {
                    
            self.clone_border_top_style()
        
                }
                PhysicalSide::Left => {
                    
            self.clone_border_left_style()
        
                }
                PhysicalSide::Bottom => {
                    
            self.clone_border_bottom_style()
        
                }
                PhysicalSide::Right => {
                    
            self.clone_border_right_style()
        
                }
        }

    }

                    
    /// Set the appropriate physical property for border-inline-end-width given a writing mode.
    pub fn set_border_inline_end_width(&mut self,
                                      v: longhands::border_inline_end_width::computed_value::T,
                                      wm: WritingMode) {
        
    
        use logical_geometry::PhysicalSide;
        match wm.inline_end_physical_side() {
                PhysicalSide::Top => {
                    
                self.set_border_top_width(v)
            
                }
                PhysicalSide::Left => {
                    
                self.set_border_left_width(v)
            
                }
                PhysicalSide::Bottom => {
                    
                self.set_border_bottom_width(v)
            
                }
                PhysicalSide::Right => {
                    
                self.set_border_right_width(v)
            
                }
        }

    }

    /// Copy the appropriate physical property from another struct for border-inline-end-width
    /// given a writing mode.
    pub fn copy_border_inline_end_width_from(&mut self,
                                            other: &Self,
                                            wm: WritingMode) {
        
    
        use logical_geometry::PhysicalSide;
        match wm.inline_end_physical_side() {
                PhysicalSide::Top => {
                    
                self.copy_border_top_width_from(other)
            
                }
                PhysicalSide::Left => {
                    
                self.copy_border_left_width_from(other)
            
                }
                PhysicalSide::Bottom => {
                    
                self.copy_border_bottom_width_from(other)
            
                }
                PhysicalSide::Right => {
                    
                self.copy_border_right_width_from(other)
            
                }
        }

    }

    /// Copy the appropriate physical property from another struct for border-inline-end-width
    /// given a writing mode.
    pub fn reset_border_inline_end_width(&mut self,
                                        other: &Self,
                                        wm: WritingMode) {
        self.copy_border_inline_end_width_from(other, wm)
    }

    /// Get the computed value for the appropriate physical property for
    /// border-inline-end-width given a writing mode.
    pub fn clone_border_inline_end_width(&self, wm: WritingMode)
        -> longhands::border_inline_end_width::computed_value::T {
    
    
        use logical_geometry::PhysicalSide;
        match wm.inline_end_physical_side() {
                PhysicalSide::Top => {
                    
            self.clone_border_top_width()
        
                }
                PhysicalSide::Left => {
                    
            self.clone_border_left_width()
        
                }
                PhysicalSide::Bottom => {
                    
            self.clone_border_bottom_width()
        
                }
                PhysicalSide::Right => {
                    
            self.clone_border_right_width()
        
                }
        }

    }

                        /// Set border-top-left-radius.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_border_top_left_radius(&mut self, v: longhands::border_top_left_radius::computed_value::T) {
                            self.border_top_left_radius = v;
                        }
                    /// Set border-top-left-radius from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_border_top_left_radius_from(&mut self, other: &Self) {
                        self.border_top_left_radius = other.border_top_left_radius.clone();
                    }

                    /// Reset border-top-left-radius from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_border_top_left_radius(&mut self, other: &Self) {
                        self.copy_border_top_left_radius_from(other)
                    }

                    /// Get the computed value for border-top-left-radius.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_border_top_left_radius(&self) -> longhands::border_top_left_radius::computed_value::T {
                        self.border_top_left_radius.clone()
                    }
                        /// Set border-top-right-radius.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_border_top_right_radius(&mut self, v: longhands::border_top_right_radius::computed_value::T) {
                            self.border_top_right_radius = v;
                        }
                    /// Set border-top-right-radius from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_border_top_right_radius_from(&mut self, other: &Self) {
                        self.border_top_right_radius = other.border_top_right_radius.clone();
                    }

                    /// Reset border-top-right-radius from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_border_top_right_radius(&mut self, other: &Self) {
                        self.copy_border_top_right_radius_from(other)
                    }

                    /// Get the computed value for border-top-right-radius.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_border_top_right_radius(&self) -> longhands::border_top_right_radius::computed_value::T {
                        self.border_top_right_radius.clone()
                    }
                        /// Set border-bottom-right-radius.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_border_bottom_right_radius(&mut self, v: longhands::border_bottom_right_radius::computed_value::T) {
                            self.border_bottom_right_radius = v;
                        }
                    /// Set border-bottom-right-radius from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_border_bottom_right_radius_from(&mut self, other: &Self) {
                        self.border_bottom_right_radius = other.border_bottom_right_radius.clone();
                    }

                    /// Reset border-bottom-right-radius from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_border_bottom_right_radius(&mut self, other: &Self) {
                        self.copy_border_bottom_right_radius_from(other)
                    }

                    /// Get the computed value for border-bottom-right-radius.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_border_bottom_right_radius(&self) -> longhands::border_bottom_right_radius::computed_value::T {
                        self.border_bottom_right_radius.clone()
                    }
                        /// Set border-bottom-left-radius.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_border_bottom_left_radius(&mut self, v: longhands::border_bottom_left_radius::computed_value::T) {
                            self.border_bottom_left_radius = v;
                        }
                    /// Set border-bottom-left-radius from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_border_bottom_left_radius_from(&mut self, other: &Self) {
                        self.border_bottom_left_radius = other.border_bottom_left_radius.clone();
                    }

                    /// Reset border-bottom-left-radius from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_border_bottom_left_radius(&mut self, other: &Self) {
                        self.copy_border_bottom_left_radius_from(other)
                    }

                    /// Get the computed value for border-bottom-left-radius.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_border_bottom_left_radius(&self) -> longhands::border_bottom_left_radius::computed_value::T {
                        self.border_bottom_left_radius.clone()
                    }
                        /// Set -moz-border-top-colors.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set__moz_border_top_colors(&mut self, v: longhands::_moz_border_top_colors::computed_value::T) {
                            self._moz_border_top_colors = v;
                        }
                    /// Set -moz-border-top-colors from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy__moz_border_top_colors_from(&mut self, other: &Self) {
                        self._moz_border_top_colors = other._moz_border_top_colors.clone();
                    }

                    /// Reset -moz-border-top-colors from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset__moz_border_top_colors(&mut self, other: &Self) {
                        self.copy__moz_border_top_colors_from(other)
                    }

                    /// Get the computed value for -moz-border-top-colors.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone__moz_border_top_colors(&self) -> longhands::_moz_border_top_colors::computed_value::T {
                        self._moz_border_top_colors.clone()
                    }
                        /// Set -moz-border-left-colors.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set__moz_border_left_colors(&mut self, v: longhands::_moz_border_left_colors::computed_value::T) {
                            self._moz_border_left_colors = v;
                        }
                    /// Set -moz-border-left-colors from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy__moz_border_left_colors_from(&mut self, other: &Self) {
                        self._moz_border_left_colors = other._moz_border_left_colors.clone();
                    }

                    /// Reset -moz-border-left-colors from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset__moz_border_left_colors(&mut self, other: &Self) {
                        self.copy__moz_border_left_colors_from(other)
                    }

                    /// Get the computed value for -moz-border-left-colors.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone__moz_border_left_colors(&self) -> longhands::_moz_border_left_colors::computed_value::T {
                        self._moz_border_left_colors.clone()
                    }
                        /// Set -moz-border-bottom-colors.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set__moz_border_bottom_colors(&mut self, v: longhands::_moz_border_bottom_colors::computed_value::T) {
                            self._moz_border_bottom_colors = v;
                        }
                    /// Set -moz-border-bottom-colors from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy__moz_border_bottom_colors_from(&mut self, other: &Self) {
                        self._moz_border_bottom_colors = other._moz_border_bottom_colors.clone();
                    }

                    /// Reset -moz-border-bottom-colors from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset__moz_border_bottom_colors(&mut self, other: &Self) {
                        self.copy__moz_border_bottom_colors_from(other)
                    }

                    /// Get the computed value for -moz-border-bottom-colors.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone__moz_border_bottom_colors(&self) -> longhands::_moz_border_bottom_colors::computed_value::T {
                        self._moz_border_bottom_colors.clone()
                    }
                        /// Set -moz-border-right-colors.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set__moz_border_right_colors(&mut self, v: longhands::_moz_border_right_colors::computed_value::T) {
                            self._moz_border_right_colors = v;
                        }
                    /// Set -moz-border-right-colors from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy__moz_border_right_colors_from(&mut self, other: &Self) {
                        self._moz_border_right_colors = other._moz_border_right_colors.clone();
                    }

                    /// Reset -moz-border-right-colors from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset__moz_border_right_colors(&mut self, other: &Self) {
                        self.copy__moz_border_right_colors_from(other)
                    }

                    /// Get the computed value for -moz-border-right-colors.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone__moz_border_right_colors(&self) -> longhands::_moz_border_right_colors::computed_value::T {
                        self._moz_border_right_colors.clone()
                    }
                        /// Set box-decoration-break.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_box_decoration_break(&mut self, v: longhands::box_decoration_break::computed_value::T) {
                            self.box_decoration_break = v;
                        }
                    /// Set box-decoration-break from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_box_decoration_break_from(&mut self, other: &Self) {
                        self.box_decoration_break = other.box_decoration_break.clone();
                    }

                    /// Reset box-decoration-break from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_box_decoration_break(&mut self, other: &Self) {
                        self.copy_box_decoration_break_from(other)
                    }

                    /// Get the computed value for box-decoration-break.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_box_decoration_break(&self) -> longhands::box_decoration_break::computed_value::T {
                        self.box_decoration_break.clone()
                    }
                        /// Set -moz-float-edge.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set__moz_float_edge(&mut self, v: longhands::_moz_float_edge::computed_value::T) {
                            self._moz_float_edge = v;
                        }
                    /// Set -moz-float-edge from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy__moz_float_edge_from(&mut self, other: &Self) {
                        self._moz_float_edge = other._moz_float_edge.clone();
                    }

                    /// Reset -moz-float-edge from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset__moz_float_edge(&mut self, other: &Self) {
                        self.copy__moz_float_edge_from(other)
                    }

                    /// Get the computed value for -moz-float-edge.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone__moz_float_edge(&self) -> longhands::_moz_float_edge::computed_value::T {
                        self._moz_float_edge.clone()
                    }
                        /// Set border-image-source.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_border_image_source(&mut self, v: longhands::border_image_source::computed_value::T) {
                            self.border_image_source = v;
                        }
                    /// Set border-image-source from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_border_image_source_from(&mut self, other: &Self) {
                        self.border_image_source = other.border_image_source.clone();
                    }

                    /// Reset border-image-source from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_border_image_source(&mut self, other: &Self) {
                        self.copy_border_image_source_from(other)
                    }

                    /// Get the computed value for border-image-source.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_border_image_source(&self) -> longhands::border_image_source::computed_value::T {
                        self.border_image_source.clone()
                    }
                        /// Set border-image-outset.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_border_image_outset(&mut self, v: longhands::border_image_outset::computed_value::T) {
                            self.border_image_outset = v;
                        }
                    /// Set border-image-outset from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_border_image_outset_from(&mut self, other: &Self) {
                        self.border_image_outset = other.border_image_outset.clone();
                    }

                    /// Reset border-image-outset from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_border_image_outset(&mut self, other: &Self) {
                        self.copy_border_image_outset_from(other)
                    }

                    /// Get the computed value for border-image-outset.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_border_image_outset(&self) -> longhands::border_image_outset::computed_value::T {
                        self.border_image_outset.clone()
                    }
                        /// Set border-image-repeat.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_border_image_repeat(&mut self, v: longhands::border_image_repeat::computed_value::T) {
                            self.border_image_repeat = v;
                        }
                    /// Set border-image-repeat from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_border_image_repeat_from(&mut self, other: &Self) {
                        self.border_image_repeat = other.border_image_repeat.clone();
                    }

                    /// Reset border-image-repeat from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_border_image_repeat(&mut self, other: &Self) {
                        self.copy_border_image_repeat_from(other)
                    }

                    /// Get the computed value for border-image-repeat.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_border_image_repeat(&self) -> longhands::border_image_repeat::computed_value::T {
                        self.border_image_repeat.clone()
                    }
                        /// Set border-image-width.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_border_image_width(&mut self, v: longhands::border_image_width::computed_value::T) {
                            self.border_image_width = v;
                        }
                    /// Set border-image-width from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_border_image_width_from(&mut self, other: &Self) {
                        self.border_image_width = other.border_image_width.clone();
                    }

                    /// Reset border-image-width from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_border_image_width(&mut self, other: &Self) {
                        self.copy_border_image_width_from(other)
                    }

                    /// Get the computed value for border-image-width.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_border_image_width(&self) -> longhands::border_image_width::computed_value::T {
                        self.border_image_width.clone()
                    }
                        /// Set border-image-slice.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_border_image_slice(&mut self, v: longhands::border_image_slice::computed_value::T) {
                            self.border_image_slice = v;
                        }
                    /// Set border-image-slice from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_border_image_slice_from(&mut self, other: &Self) {
                        self.border_image_slice = other.border_image_slice.clone();
                    }

                    /// Reset border-image-slice from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_border_image_slice(&mut self, other: &Self) {
                        self.copy_border_image_slice_from(other)
                    }

                    /// Get the computed value for border-image-slice.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_border_image_slice(&self) -> longhands::border_image_slice::computed_value::T {
                        self.border_image_slice.clone()
                    }
                    /// Whether the border-top property has nonzero width.
                    #[allow(non_snake_case)]
                    pub fn border_top_has_nonzero_width(&self) -> bool {
                        self.border_top_width != NonNegativeLength::zero()
                    }
                    /// Whether the border-right property has nonzero width.
                    #[allow(non_snake_case)]
                    pub fn border_right_has_nonzero_width(&self) -> bool {
                        self.border_right_width != NonNegativeLength::zero()
                    }
                    /// Whether the border-bottom property has nonzero width.
                    #[allow(non_snake_case)]
                    pub fn border_bottom_has_nonzero_width(&self) -> bool {
                        self.border_bottom_width != NonNegativeLength::zero()
                    }
                    /// Whether the border-left property has nonzero width.
                    #[allow(non_snake_case)]
                    pub fn border_left_has_nonzero_width(&self) -> bool {
                        self.border_left_width != NonNegativeLength::zero()
                    }
        }

        #[derive(Clone, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        /// The Box style struct.
        pub struct Box {
                /// The display computed value.
                pub display: longhands::display::computed_value::T,
                /// The -moz-top-layer computed value.
                pub _moz_top_layer: longhands::_moz_top_layer::computed_value::T,
                /// The position computed value.
                pub position: longhands::position::computed_value::T,
                /// The float computed value.
                pub float: longhands::float::computed_value::T,
                /// The clear computed value.
                pub clear: longhands::clear::computed_value::T,
                /// The vertical-align computed value.
                pub vertical_align: longhands::vertical_align::computed_value::T,
                /// The overflow-clip-box computed value.
                pub overflow_clip_box: longhands::overflow_clip_box::computed_value::T,
                /// The overflow-x computed value.
                pub overflow_x: longhands::overflow_x::computed_value::T,
                /// The overflow-y computed value.
                pub overflow_y: longhands::overflow_y::computed_value::T,
                /// The transition-duration computed value.
                pub transition_duration: longhands::transition_duration::computed_value::T,
                /// The transition-timing-function computed value.
                pub transition_timing_function: longhands::transition_timing_function::computed_value::T,
                /// The transition-property computed value.
                pub transition_property: longhands::transition_property::computed_value::T,
                /// The transition-delay computed value.
                pub transition_delay: longhands::transition_delay::computed_value::T,
                /// The animation-name computed value.
                pub animation_name: longhands::animation_name::computed_value::T,
                /// The animation-duration computed value.
                pub animation_duration: longhands::animation_duration::computed_value::T,
                /// The animation-timing-function computed value.
                pub animation_timing_function: longhands::animation_timing_function::computed_value::T,
                /// The animation-iteration-count computed value.
                pub animation_iteration_count: longhands::animation_iteration_count::computed_value::T,
                /// The animation-direction computed value.
                pub animation_direction: longhands::animation_direction::computed_value::T,
                /// The animation-play-state computed value.
                pub animation_play_state: longhands::animation_play_state::computed_value::T,
                /// The animation-fill-mode computed value.
                pub animation_fill_mode: longhands::animation_fill_mode::computed_value::T,
                /// The animation-delay computed value.
                pub animation_delay: longhands::animation_delay::computed_value::T,
                /// The scroll-snap-points-x computed value.
                pub scroll_snap_points_x: longhands::scroll_snap_points_x::computed_value::T,
                /// The scroll-snap-points-y computed value.
                pub scroll_snap_points_y: longhands::scroll_snap_points_y::computed_value::T,
                /// The scroll-snap-destination computed value.
                pub scroll_snap_destination: longhands::scroll_snap_destination::computed_value::T,
                /// The scroll-snap-coordinate computed value.
                pub scroll_snap_coordinate: longhands::scroll_snap_coordinate::computed_value::T,
                /// The transform computed value.
                pub transform: longhands::transform::computed_value::T,
                /// The scroll-behavior computed value.
                pub scroll_behavior: longhands::scroll_behavior::computed_value::T,
                /// The scroll-snap-type-x computed value.
                pub scroll_snap_type_x: longhands::scroll_snap_type_x::computed_value::T,
                /// The scroll-snap-type-y computed value.
                pub scroll_snap_type_y: longhands::scroll_snap_type_y::computed_value::T,
                /// The isolation computed value.
                pub isolation: longhands::isolation::computed_value::T,
                /// The page-break-after computed value.
                pub page_break_after: longhands::page_break_after::computed_value::T,
                /// The page-break-before computed value.
                pub page_break_before: longhands::page_break_before::computed_value::T,
                /// The page-break-inside computed value.
                pub page_break_inside: longhands::page_break_inside::computed_value::T,
                /// The resize computed value.
                pub resize: longhands::resize::computed_value::T,
                /// The perspective computed value.
                pub perspective: longhands::perspective::computed_value::T,
                /// The perspective-origin computed value.
                pub perspective_origin: longhands::perspective_origin::computed_value::T,
                /// The backface-visibility computed value.
                pub backface_visibility: longhands::backface_visibility::computed_value::T,
                /// The transform-box computed value.
                pub transform_box: longhands::transform_box::computed_value::T,
                /// The transform-style computed value.
                pub transform_style: longhands::transform_style::computed_value::T,
                /// The transform-origin computed value.
                pub transform_origin: longhands::transform_origin::computed_value::T,
                /// The contain computed value.
                pub contain: longhands::contain::computed_value::T,
                /// The -moz-appearance computed value.
                pub _moz_appearance: longhands::_moz_appearance::computed_value::T,
                /// The -moz-binding computed value.
                pub _moz_binding: longhands::_moz_binding::computed_value::T,
                /// The -moz-orient computed value.
                pub _moz_orient: longhands::_moz_orient::computed_value::T,
                /// The will-change computed value.
                pub will_change: longhands::will_change::computed_value::T,
                /// The shape-outside computed value.
                pub shape_outside: longhands::shape_outside::computed_value::T,
                /// The touch-action computed value.
                pub touch_action: longhands::touch_action::computed_value::T,
        }

        impl Box {
                        /// Set display.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_display(&mut self, v: longhands::display::computed_value::T) {
                            self.display = v;
                        }
                    /// Set display from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_display_from(&mut self, other: &Self) {
                        self.display = other.display.clone();
                    }

                    /// Reset display from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_display(&mut self, other: &Self) {
                        self.copy_display_from(other)
                    }

                    /// Get the computed value for display.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_display(&self) -> longhands::display::computed_value::T {
                        self.display.clone()
                    }
                        /// Set -moz-top-layer.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set__moz_top_layer(&mut self, v: longhands::_moz_top_layer::computed_value::T) {
                            self._moz_top_layer = v;
                        }
                    /// Set -moz-top-layer from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy__moz_top_layer_from(&mut self, other: &Self) {
                        self._moz_top_layer = other._moz_top_layer.clone();
                    }

                    /// Reset -moz-top-layer from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset__moz_top_layer(&mut self, other: &Self) {
                        self.copy__moz_top_layer_from(other)
                    }

                    /// Get the computed value for -moz-top-layer.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone__moz_top_layer(&self) -> longhands::_moz_top_layer::computed_value::T {
                        self._moz_top_layer.clone()
                    }
                        /// Set position.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_position(&mut self, v: longhands::position::computed_value::T) {
                            self.position = v;
                        }
                    /// Set position from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_position_from(&mut self, other: &Self) {
                        self.position = other.position.clone();
                    }

                    /// Reset position from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_position(&mut self, other: &Self) {
                        self.copy_position_from(other)
                    }

                    /// Get the computed value for position.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_position(&self) -> longhands::position::computed_value::T {
                        self.position.clone()
                    }
                        /// Set float.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_float(&mut self, v: longhands::float::computed_value::T) {
                            self.float = v;
                        }
                    /// Set float from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_float_from(&mut self, other: &Self) {
                        self.float = other.float.clone();
                    }

                    /// Reset float from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_float(&mut self, other: &Self) {
                        self.copy_float_from(other)
                    }

                    /// Get the computed value for float.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_float(&self) -> longhands::float::computed_value::T {
                        self.float.clone()
                    }
                        /// Set clear.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_clear(&mut self, v: longhands::clear::computed_value::T) {
                            self.clear = v;
                        }
                    /// Set clear from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_clear_from(&mut self, other: &Self) {
                        self.clear = other.clear.clone();
                    }

                    /// Reset clear from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_clear(&mut self, other: &Self) {
                        self.copy_clear_from(other)
                    }

                    /// Get the computed value for clear.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_clear(&self) -> longhands::clear::computed_value::T {
                        self.clear.clone()
                    }
                        /// Set vertical-align.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_vertical_align(&mut self, v: longhands::vertical_align::computed_value::T) {
                            self.vertical_align = v;
                        }
                    /// Set vertical-align from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_vertical_align_from(&mut self, other: &Self) {
                        self.vertical_align = other.vertical_align.clone();
                    }

                    /// Reset vertical-align from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_vertical_align(&mut self, other: &Self) {
                        self.copy_vertical_align_from(other)
                    }

                    /// Get the computed value for vertical-align.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_vertical_align(&self) -> longhands::vertical_align::computed_value::T {
                        self.vertical_align.clone()
                    }
                        /// Set overflow-clip-box.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_overflow_clip_box(&mut self, v: longhands::overflow_clip_box::computed_value::T) {
                            self.overflow_clip_box = v;
                        }
                    /// Set overflow-clip-box from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_overflow_clip_box_from(&mut self, other: &Self) {
                        self.overflow_clip_box = other.overflow_clip_box.clone();
                    }

                    /// Reset overflow-clip-box from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_overflow_clip_box(&mut self, other: &Self) {
                        self.copy_overflow_clip_box_from(other)
                    }

                    /// Get the computed value for overflow-clip-box.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_overflow_clip_box(&self) -> longhands::overflow_clip_box::computed_value::T {
                        self.overflow_clip_box.clone()
                    }
                        /// Set overflow-x.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_overflow_x(&mut self, v: longhands::overflow_x::computed_value::T) {
                            self.overflow_x = v;
                        }
                    /// Set overflow-x from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_overflow_x_from(&mut self, other: &Self) {
                        self.overflow_x = other.overflow_x.clone();
                    }

                    /// Reset overflow-x from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_overflow_x(&mut self, other: &Self) {
                        self.copy_overflow_x_from(other)
                    }

                    /// Get the computed value for overflow-x.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_overflow_x(&self) -> longhands::overflow_x::computed_value::T {
                        self.overflow_x.clone()
                    }
                        /// Set overflow-y.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_overflow_y(&mut self, v: longhands::overflow_y::computed_value::T) {
                            self.overflow_y = v;
                        }
                    /// Set overflow-y from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_overflow_y_from(&mut self, other: &Self) {
                        self.overflow_y = other.overflow_y.clone();
                    }

                    /// Reset overflow-y from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_overflow_y(&mut self, other: &Self) {
                        self.copy_overflow_y_from(other)
                    }

                    /// Get the computed value for overflow-y.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_overflow_y(&self) -> longhands::overflow_y::computed_value::T {
                        self.overflow_y.clone()
                    }
                        /// Set transition-duration.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_transition_duration<I>(&mut self, v: I)
                            where I: IntoIterator<Item = longhands::transition_duration
                                                                  ::computed_value::single_value::T>,
                                  I::IntoIter: ExactSizeIterator
                        {
                            self.transition_duration = longhands::transition_duration::computed_value
                                                              ::T(v.into_iter().collect());
                        }
                    /// Set transition-duration from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_transition_duration_from(&mut self, other: &Self) {
                        self.transition_duration = other.transition_duration.clone();
                    }

                    /// Reset transition-duration from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_transition_duration(&mut self, other: &Self) {
                        self.copy_transition_duration_from(other)
                    }

                    /// Get the computed value for transition-duration.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_transition_duration(&self) -> longhands::transition_duration::computed_value::T {
                        self.transition_duration.clone()
                    }
                    /// If this longhand is indexed, get the number of elements.
                    #[allow(non_snake_case)]
                    pub fn transition_duration_count(&self) -> usize {
                        self.transition_duration.0.len()
                    }

                    /// If this longhand is indexed, get the element at given
                    /// index.
                    #[allow(non_snake_case)]
                    pub fn transition_duration_at(&self, index: usize)
                        -> longhands::transition_duration::computed_value::SingleComputedValue {
                        self.transition_duration.0[index].clone()
                    }
                        /// Set transition-timing-function.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_transition_timing_function<I>(&mut self, v: I)
                            where I: IntoIterator<Item = longhands::transition_timing_function
                                                                  ::computed_value::single_value::T>,
                                  I::IntoIter: ExactSizeIterator
                        {
                            self.transition_timing_function = longhands::transition_timing_function::computed_value
                                                              ::T(v.into_iter().collect());
                        }
                    /// Set transition-timing-function from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_transition_timing_function_from(&mut self, other: &Self) {
                        self.transition_timing_function = other.transition_timing_function.clone();
                    }

                    /// Reset transition-timing-function from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_transition_timing_function(&mut self, other: &Self) {
                        self.copy_transition_timing_function_from(other)
                    }

                    /// Get the computed value for transition-timing-function.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_transition_timing_function(&self) -> longhands::transition_timing_function::computed_value::T {
                        self.transition_timing_function.clone()
                    }
                    /// If this longhand is indexed, get the number of elements.
                    #[allow(non_snake_case)]
                    pub fn transition_timing_function_count(&self) -> usize {
                        self.transition_timing_function.0.len()
                    }

                    /// If this longhand is indexed, get the element at given
                    /// index.
                    #[allow(non_snake_case)]
                    pub fn transition_timing_function_at(&self, index: usize)
                        -> longhands::transition_timing_function::computed_value::SingleComputedValue {
                        self.transition_timing_function.0[index].clone()
                    }
                        /// Set transition-property.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_transition_property<I>(&mut self, v: I)
                            where I: IntoIterator<Item = longhands::transition_property
                                                                  ::computed_value::single_value::T>,
                                  I::IntoIter: ExactSizeIterator
                        {
                            self.transition_property = longhands::transition_property::computed_value
                                                              ::T(v.into_iter().collect());
                        }
                    /// Set transition-property from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_transition_property_from(&mut self, other: &Self) {
                        self.transition_property = other.transition_property.clone();
                    }

                    /// Reset transition-property from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_transition_property(&mut self, other: &Self) {
                        self.copy_transition_property_from(other)
                    }

                    /// Get the computed value for transition-property.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_transition_property(&self) -> longhands::transition_property::computed_value::T {
                        self.transition_property.clone()
                    }
                    /// If this longhand is indexed, get the number of elements.
                    #[allow(non_snake_case)]
                    pub fn transition_property_count(&self) -> usize {
                        self.transition_property.0.len()
                    }

                    /// If this longhand is indexed, get the element at given
                    /// index.
                    #[allow(non_snake_case)]
                    pub fn transition_property_at(&self, index: usize)
                        -> longhands::transition_property::computed_value::SingleComputedValue {
                        self.transition_property.0[index].clone()
                    }
                        /// Set transition-delay.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_transition_delay<I>(&mut self, v: I)
                            where I: IntoIterator<Item = longhands::transition_delay
                                                                  ::computed_value::single_value::T>,
                                  I::IntoIter: ExactSizeIterator
                        {
                            self.transition_delay = longhands::transition_delay::computed_value
                                                              ::T(v.into_iter().collect());
                        }
                    /// Set transition-delay from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_transition_delay_from(&mut self, other: &Self) {
                        self.transition_delay = other.transition_delay.clone();
                    }

                    /// Reset transition-delay from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_transition_delay(&mut self, other: &Self) {
                        self.copy_transition_delay_from(other)
                    }

                    /// Get the computed value for transition-delay.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_transition_delay(&self) -> longhands::transition_delay::computed_value::T {
                        self.transition_delay.clone()
                    }
                    /// If this longhand is indexed, get the number of elements.
                    #[allow(non_snake_case)]
                    pub fn transition_delay_count(&self) -> usize {
                        self.transition_delay.0.len()
                    }

                    /// If this longhand is indexed, get the element at given
                    /// index.
                    #[allow(non_snake_case)]
                    pub fn transition_delay_at(&self, index: usize)
                        -> longhands::transition_delay::computed_value::SingleComputedValue {
                        self.transition_delay.0[index].clone()
                    }
                        /// Set animation-name.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_animation_name<I>(&mut self, v: I)
                            where I: IntoIterator<Item = longhands::animation_name
                                                                  ::computed_value::single_value::T>,
                                  I::IntoIter: ExactSizeIterator
                        {
                            self.animation_name = longhands::animation_name::computed_value
                                                              ::T(v.into_iter().collect());
                        }
                    /// Set animation-name from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_animation_name_from(&mut self, other: &Self) {
                        self.animation_name = other.animation_name.clone();
                    }

                    /// Reset animation-name from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_animation_name(&mut self, other: &Self) {
                        self.copy_animation_name_from(other)
                    }

                    /// Get the computed value for animation-name.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_animation_name(&self) -> longhands::animation_name::computed_value::T {
                        self.animation_name.clone()
                    }
                    /// If this longhand is indexed, get the number of elements.
                    #[allow(non_snake_case)]
                    pub fn animation_name_count(&self) -> usize {
                        self.animation_name.0.len()
                    }

                    /// If this longhand is indexed, get the element at given
                    /// index.
                    #[allow(non_snake_case)]
                    pub fn animation_name_at(&self, index: usize)
                        -> longhands::animation_name::computed_value::SingleComputedValue {
                        self.animation_name.0[index].clone()
                    }
                        /// Set animation-duration.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_animation_duration<I>(&mut self, v: I)
                            where I: IntoIterator<Item = longhands::animation_duration
                                                                  ::computed_value::single_value::T>,
                                  I::IntoIter: ExactSizeIterator
                        {
                            self.animation_duration = longhands::animation_duration::computed_value
                                                              ::T(v.into_iter().collect());
                        }
                    /// Set animation-duration from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_animation_duration_from(&mut self, other: &Self) {
                        self.animation_duration = other.animation_duration.clone();
                    }

                    /// Reset animation-duration from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_animation_duration(&mut self, other: &Self) {
                        self.copy_animation_duration_from(other)
                    }

                    /// Get the computed value for animation-duration.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_animation_duration(&self) -> longhands::animation_duration::computed_value::T {
                        self.animation_duration.clone()
                    }
                    /// If this longhand is indexed, get the number of elements.
                    #[allow(non_snake_case)]
                    pub fn animation_duration_count(&self) -> usize {
                        self.animation_duration.0.len()
                    }

                    /// If this longhand is indexed, get the element at given
                    /// index.
                    #[allow(non_snake_case)]
                    pub fn animation_duration_at(&self, index: usize)
                        -> longhands::animation_duration::computed_value::SingleComputedValue {
                        self.animation_duration.0[index].clone()
                    }
                        /// Set animation-timing-function.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_animation_timing_function<I>(&mut self, v: I)
                            where I: IntoIterator<Item = longhands::animation_timing_function
                                                                  ::computed_value::single_value::T>,
                                  I::IntoIter: ExactSizeIterator
                        {
                            self.animation_timing_function = longhands::animation_timing_function::computed_value
                                                              ::T(v.into_iter().collect());
                        }
                    /// Set animation-timing-function from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_animation_timing_function_from(&mut self, other: &Self) {
                        self.animation_timing_function = other.animation_timing_function.clone();
                    }

                    /// Reset animation-timing-function from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_animation_timing_function(&mut self, other: &Self) {
                        self.copy_animation_timing_function_from(other)
                    }

                    /// Get the computed value for animation-timing-function.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_animation_timing_function(&self) -> longhands::animation_timing_function::computed_value::T {
                        self.animation_timing_function.clone()
                    }
                    /// If this longhand is indexed, get the number of elements.
                    #[allow(non_snake_case)]
                    pub fn animation_timing_function_count(&self) -> usize {
                        self.animation_timing_function.0.len()
                    }

                    /// If this longhand is indexed, get the element at given
                    /// index.
                    #[allow(non_snake_case)]
                    pub fn animation_timing_function_at(&self, index: usize)
                        -> longhands::animation_timing_function::computed_value::SingleComputedValue {
                        self.animation_timing_function.0[index].clone()
                    }
                        /// Set animation-iteration-count.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_animation_iteration_count<I>(&mut self, v: I)
                            where I: IntoIterator<Item = longhands::animation_iteration_count
                                                                  ::computed_value::single_value::T>,
                                  I::IntoIter: ExactSizeIterator
                        {
                            self.animation_iteration_count = longhands::animation_iteration_count::computed_value
                                                              ::T(v.into_iter().collect());
                        }
                    /// Set animation-iteration-count from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_animation_iteration_count_from(&mut self, other: &Self) {
                        self.animation_iteration_count = other.animation_iteration_count.clone();
                    }

                    /// Reset animation-iteration-count from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_animation_iteration_count(&mut self, other: &Self) {
                        self.copy_animation_iteration_count_from(other)
                    }

                    /// Get the computed value for animation-iteration-count.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_animation_iteration_count(&self) -> longhands::animation_iteration_count::computed_value::T {
                        self.animation_iteration_count.clone()
                    }
                    /// If this longhand is indexed, get the number of elements.
                    #[allow(non_snake_case)]
                    pub fn animation_iteration_count_count(&self) -> usize {
                        self.animation_iteration_count.0.len()
                    }

                    /// If this longhand is indexed, get the element at given
                    /// index.
                    #[allow(non_snake_case)]
                    pub fn animation_iteration_count_at(&self, index: usize)
                        -> longhands::animation_iteration_count::computed_value::SingleComputedValue {
                        self.animation_iteration_count.0[index].clone()
                    }
                        /// Set animation-direction.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_animation_direction<I>(&mut self, v: I)
                            where I: IntoIterator<Item = longhands::animation_direction
                                                                  ::computed_value::single_value::T>,
                                  I::IntoIter: ExactSizeIterator
                        {
                            self.animation_direction = longhands::animation_direction::computed_value
                                                              ::T(v.into_iter().collect());
                        }
                    /// Set animation-direction from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_animation_direction_from(&mut self, other: &Self) {
                        self.animation_direction = other.animation_direction.clone();
                    }

                    /// Reset animation-direction from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_animation_direction(&mut self, other: &Self) {
                        self.copy_animation_direction_from(other)
                    }

                    /// Get the computed value for animation-direction.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_animation_direction(&self) -> longhands::animation_direction::computed_value::T {
                        self.animation_direction.clone()
                    }
                    /// If this longhand is indexed, get the number of elements.
                    #[allow(non_snake_case)]
                    pub fn animation_direction_count(&self) -> usize {
                        self.animation_direction.0.len()
                    }

                    /// If this longhand is indexed, get the element at given
                    /// index.
                    #[allow(non_snake_case)]
                    pub fn animation_direction_at(&self, index: usize)
                        -> longhands::animation_direction::computed_value::SingleComputedValue {
                        self.animation_direction.0[index].clone()
                    }
                        /// Set animation-play-state.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_animation_play_state<I>(&mut self, v: I)
                            where I: IntoIterator<Item = longhands::animation_play_state
                                                                  ::computed_value::single_value::T>,
                                  I::IntoIter: ExactSizeIterator
                        {
                            self.animation_play_state = longhands::animation_play_state::computed_value
                                                              ::T(v.into_iter().collect());
                        }
                    /// Set animation-play-state from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_animation_play_state_from(&mut self, other: &Self) {
                        self.animation_play_state = other.animation_play_state.clone();
                    }

                    /// Reset animation-play-state from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_animation_play_state(&mut self, other: &Self) {
                        self.copy_animation_play_state_from(other)
                    }

                    /// Get the computed value for animation-play-state.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_animation_play_state(&self) -> longhands::animation_play_state::computed_value::T {
                        self.animation_play_state.clone()
                    }
                    /// If this longhand is indexed, get the number of elements.
                    #[allow(non_snake_case)]
                    pub fn animation_play_state_count(&self) -> usize {
                        self.animation_play_state.0.len()
                    }

                    /// If this longhand is indexed, get the element at given
                    /// index.
                    #[allow(non_snake_case)]
                    pub fn animation_play_state_at(&self, index: usize)
                        -> longhands::animation_play_state::computed_value::SingleComputedValue {
                        self.animation_play_state.0[index].clone()
                    }
                        /// Set animation-fill-mode.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_animation_fill_mode<I>(&mut self, v: I)
                            where I: IntoIterator<Item = longhands::animation_fill_mode
                                                                  ::computed_value::single_value::T>,
                                  I::IntoIter: ExactSizeIterator
                        {
                            self.animation_fill_mode = longhands::animation_fill_mode::computed_value
                                                              ::T(v.into_iter().collect());
                        }
                    /// Set animation-fill-mode from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_animation_fill_mode_from(&mut self, other: &Self) {
                        self.animation_fill_mode = other.animation_fill_mode.clone();
                    }

                    /// Reset animation-fill-mode from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_animation_fill_mode(&mut self, other: &Self) {
                        self.copy_animation_fill_mode_from(other)
                    }

                    /// Get the computed value for animation-fill-mode.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_animation_fill_mode(&self) -> longhands::animation_fill_mode::computed_value::T {
                        self.animation_fill_mode.clone()
                    }
                    /// If this longhand is indexed, get the number of elements.
                    #[allow(non_snake_case)]
                    pub fn animation_fill_mode_count(&self) -> usize {
                        self.animation_fill_mode.0.len()
                    }

                    /// If this longhand is indexed, get the element at given
                    /// index.
                    #[allow(non_snake_case)]
                    pub fn animation_fill_mode_at(&self, index: usize)
                        -> longhands::animation_fill_mode::computed_value::SingleComputedValue {
                        self.animation_fill_mode.0[index].clone()
                    }
                        /// Set animation-delay.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_animation_delay<I>(&mut self, v: I)
                            where I: IntoIterator<Item = longhands::animation_delay
                                                                  ::computed_value::single_value::T>,
                                  I::IntoIter: ExactSizeIterator
                        {
                            self.animation_delay = longhands::animation_delay::computed_value
                                                              ::T(v.into_iter().collect());
                        }
                    /// Set animation-delay from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_animation_delay_from(&mut self, other: &Self) {
                        self.animation_delay = other.animation_delay.clone();
                    }

                    /// Reset animation-delay from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_animation_delay(&mut self, other: &Self) {
                        self.copy_animation_delay_from(other)
                    }

                    /// Get the computed value for animation-delay.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_animation_delay(&self) -> longhands::animation_delay::computed_value::T {
                        self.animation_delay.clone()
                    }
                    /// If this longhand is indexed, get the number of elements.
                    #[allow(non_snake_case)]
                    pub fn animation_delay_count(&self) -> usize {
                        self.animation_delay.0.len()
                    }

                    /// If this longhand is indexed, get the element at given
                    /// index.
                    #[allow(non_snake_case)]
                    pub fn animation_delay_at(&self, index: usize)
                        -> longhands::animation_delay::computed_value::SingleComputedValue {
                        self.animation_delay.0[index].clone()
                    }
                        /// Set scroll-snap-points-x.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_scroll_snap_points_x(&mut self, v: longhands::scroll_snap_points_x::computed_value::T) {
                            self.scroll_snap_points_x = v;
                        }
                    /// Set scroll-snap-points-x from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_scroll_snap_points_x_from(&mut self, other: &Self) {
                        self.scroll_snap_points_x = other.scroll_snap_points_x.clone();
                    }

                    /// Reset scroll-snap-points-x from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_scroll_snap_points_x(&mut self, other: &Self) {
                        self.copy_scroll_snap_points_x_from(other)
                    }

                    /// Get the computed value for scroll-snap-points-x.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_scroll_snap_points_x(&self) -> longhands::scroll_snap_points_x::computed_value::T {
                        self.scroll_snap_points_x.clone()
                    }
                        /// Set scroll-snap-points-y.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_scroll_snap_points_y(&mut self, v: longhands::scroll_snap_points_y::computed_value::T) {
                            self.scroll_snap_points_y = v;
                        }
                    /// Set scroll-snap-points-y from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_scroll_snap_points_y_from(&mut self, other: &Self) {
                        self.scroll_snap_points_y = other.scroll_snap_points_y.clone();
                    }

                    /// Reset scroll-snap-points-y from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_scroll_snap_points_y(&mut self, other: &Self) {
                        self.copy_scroll_snap_points_y_from(other)
                    }

                    /// Get the computed value for scroll-snap-points-y.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_scroll_snap_points_y(&self) -> longhands::scroll_snap_points_y::computed_value::T {
                        self.scroll_snap_points_y.clone()
                    }
                        /// Set scroll-snap-destination.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_scroll_snap_destination(&mut self, v: longhands::scroll_snap_destination::computed_value::T) {
                            self.scroll_snap_destination = v;
                        }
                    /// Set scroll-snap-destination from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_scroll_snap_destination_from(&mut self, other: &Self) {
                        self.scroll_snap_destination = other.scroll_snap_destination.clone();
                    }

                    /// Reset scroll-snap-destination from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_scroll_snap_destination(&mut self, other: &Self) {
                        self.copy_scroll_snap_destination_from(other)
                    }

                    /// Get the computed value for scroll-snap-destination.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_scroll_snap_destination(&self) -> longhands::scroll_snap_destination::computed_value::T {
                        self.scroll_snap_destination.clone()
                    }
                        /// Set scroll-snap-coordinate.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_scroll_snap_coordinate<I>(&mut self, v: I)
                            where I: IntoIterator<Item = longhands::scroll_snap_coordinate
                                                                  ::computed_value::single_value::T>,
                                  I::IntoIter: ExactSizeIterator
                        {
                            self.scroll_snap_coordinate = longhands::scroll_snap_coordinate::computed_value
                                                              ::T(v.into_iter().collect());
                        }
                    /// Set scroll-snap-coordinate from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_scroll_snap_coordinate_from(&mut self, other: &Self) {
                        self.scroll_snap_coordinate = other.scroll_snap_coordinate.clone();
                    }

                    /// Reset scroll-snap-coordinate from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_scroll_snap_coordinate(&mut self, other: &Self) {
                        self.copy_scroll_snap_coordinate_from(other)
                    }

                    /// Get the computed value for scroll-snap-coordinate.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_scroll_snap_coordinate(&self) -> longhands::scroll_snap_coordinate::computed_value::T {
                        self.scroll_snap_coordinate.clone()
                    }
                        /// Set transform.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_transform(&mut self, v: longhands::transform::computed_value::T) {
                            self.transform = v;
                        }
                    /// Set transform from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_transform_from(&mut self, other: &Self) {
                        self.transform = other.transform.clone();
                    }

                    /// Reset transform from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_transform(&mut self, other: &Self) {
                        self.copy_transform_from(other)
                    }

                    /// Get the computed value for transform.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_transform(&self) -> longhands::transform::computed_value::T {
                        self.transform.clone()
                    }
                        /// Set scroll-behavior.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_scroll_behavior(&mut self, v: longhands::scroll_behavior::computed_value::T) {
                            self.scroll_behavior = v;
                        }
                    /// Set scroll-behavior from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_scroll_behavior_from(&mut self, other: &Self) {
                        self.scroll_behavior = other.scroll_behavior.clone();
                    }

                    /// Reset scroll-behavior from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_scroll_behavior(&mut self, other: &Self) {
                        self.copy_scroll_behavior_from(other)
                    }

                    /// Get the computed value for scroll-behavior.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_scroll_behavior(&self) -> longhands::scroll_behavior::computed_value::T {
                        self.scroll_behavior.clone()
                    }
                        /// Set scroll-snap-type-x.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_scroll_snap_type_x(&mut self, v: longhands::scroll_snap_type_x::computed_value::T) {
                            self.scroll_snap_type_x = v;
                        }
                    /// Set scroll-snap-type-x from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_scroll_snap_type_x_from(&mut self, other: &Self) {
                        self.scroll_snap_type_x = other.scroll_snap_type_x.clone();
                    }

                    /// Reset scroll-snap-type-x from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_scroll_snap_type_x(&mut self, other: &Self) {
                        self.copy_scroll_snap_type_x_from(other)
                    }

                    /// Get the computed value for scroll-snap-type-x.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_scroll_snap_type_x(&self) -> longhands::scroll_snap_type_x::computed_value::T {
                        self.scroll_snap_type_x.clone()
                    }
                        /// Set scroll-snap-type-y.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_scroll_snap_type_y(&mut self, v: longhands::scroll_snap_type_y::computed_value::T) {
                            self.scroll_snap_type_y = v;
                        }
                    /// Set scroll-snap-type-y from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_scroll_snap_type_y_from(&mut self, other: &Self) {
                        self.scroll_snap_type_y = other.scroll_snap_type_y.clone();
                    }

                    /// Reset scroll-snap-type-y from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_scroll_snap_type_y(&mut self, other: &Self) {
                        self.copy_scroll_snap_type_y_from(other)
                    }

                    /// Get the computed value for scroll-snap-type-y.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_scroll_snap_type_y(&self) -> longhands::scroll_snap_type_y::computed_value::T {
                        self.scroll_snap_type_y.clone()
                    }
                        /// Set isolation.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_isolation(&mut self, v: longhands::isolation::computed_value::T) {
                            self.isolation = v;
                        }
                    /// Set isolation from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_isolation_from(&mut self, other: &Self) {
                        self.isolation = other.isolation.clone();
                    }

                    /// Reset isolation from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_isolation(&mut self, other: &Self) {
                        self.copy_isolation_from(other)
                    }

                    /// Get the computed value for isolation.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_isolation(&self) -> longhands::isolation::computed_value::T {
                        self.isolation.clone()
                    }
                        /// Set page-break-after.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_page_break_after(&mut self, v: longhands::page_break_after::computed_value::T) {
                            self.page_break_after = v;
                        }
                    /// Set page-break-after from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_page_break_after_from(&mut self, other: &Self) {
                        self.page_break_after = other.page_break_after.clone();
                    }

                    /// Reset page-break-after from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_page_break_after(&mut self, other: &Self) {
                        self.copy_page_break_after_from(other)
                    }

                    /// Get the computed value for page-break-after.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_page_break_after(&self) -> longhands::page_break_after::computed_value::T {
                        self.page_break_after.clone()
                    }
                        /// Set page-break-before.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_page_break_before(&mut self, v: longhands::page_break_before::computed_value::T) {
                            self.page_break_before = v;
                        }
                    /// Set page-break-before from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_page_break_before_from(&mut self, other: &Self) {
                        self.page_break_before = other.page_break_before.clone();
                    }

                    /// Reset page-break-before from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_page_break_before(&mut self, other: &Self) {
                        self.copy_page_break_before_from(other)
                    }

                    /// Get the computed value for page-break-before.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_page_break_before(&self) -> longhands::page_break_before::computed_value::T {
                        self.page_break_before.clone()
                    }
                        /// Set page-break-inside.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_page_break_inside(&mut self, v: longhands::page_break_inside::computed_value::T) {
                            self.page_break_inside = v;
                        }
                    /// Set page-break-inside from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_page_break_inside_from(&mut self, other: &Self) {
                        self.page_break_inside = other.page_break_inside.clone();
                    }

                    /// Reset page-break-inside from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_page_break_inside(&mut self, other: &Self) {
                        self.copy_page_break_inside_from(other)
                    }

                    /// Get the computed value for page-break-inside.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_page_break_inside(&self) -> longhands::page_break_inside::computed_value::T {
                        self.page_break_inside.clone()
                    }
                        /// Set resize.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_resize(&mut self, v: longhands::resize::computed_value::T) {
                            self.resize = v;
                        }
                    /// Set resize from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_resize_from(&mut self, other: &Self) {
                        self.resize = other.resize.clone();
                    }

                    /// Reset resize from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_resize(&mut self, other: &Self) {
                        self.copy_resize_from(other)
                    }

                    /// Get the computed value for resize.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_resize(&self) -> longhands::resize::computed_value::T {
                        self.resize.clone()
                    }
                        /// Set perspective.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_perspective(&mut self, v: longhands::perspective::computed_value::T) {
                            self.perspective = v;
                        }
                    /// Set perspective from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_perspective_from(&mut self, other: &Self) {
                        self.perspective = other.perspective.clone();
                    }

                    /// Reset perspective from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_perspective(&mut self, other: &Self) {
                        self.copy_perspective_from(other)
                    }

                    /// Get the computed value for perspective.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_perspective(&self) -> longhands::perspective::computed_value::T {
                        self.perspective.clone()
                    }
                        /// Set perspective-origin.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_perspective_origin(&mut self, v: longhands::perspective_origin::computed_value::T) {
                            self.perspective_origin = v;
                        }
                    /// Set perspective-origin from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_perspective_origin_from(&mut self, other: &Self) {
                        self.perspective_origin = other.perspective_origin.clone();
                    }

                    /// Reset perspective-origin from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_perspective_origin(&mut self, other: &Self) {
                        self.copy_perspective_origin_from(other)
                    }

                    /// Get the computed value for perspective-origin.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_perspective_origin(&self) -> longhands::perspective_origin::computed_value::T {
                        self.perspective_origin.clone()
                    }
                        /// Set backface-visibility.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_backface_visibility(&mut self, v: longhands::backface_visibility::computed_value::T) {
                            self.backface_visibility = v;
                        }
                    /// Set backface-visibility from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_backface_visibility_from(&mut self, other: &Self) {
                        self.backface_visibility = other.backface_visibility.clone();
                    }

                    /// Reset backface-visibility from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_backface_visibility(&mut self, other: &Self) {
                        self.copy_backface_visibility_from(other)
                    }

                    /// Get the computed value for backface-visibility.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_backface_visibility(&self) -> longhands::backface_visibility::computed_value::T {
                        self.backface_visibility.clone()
                    }
                        /// Set transform-box.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_transform_box(&mut self, v: longhands::transform_box::computed_value::T) {
                            self.transform_box = v;
                        }
                    /// Set transform-box from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_transform_box_from(&mut self, other: &Self) {
                        self.transform_box = other.transform_box.clone();
                    }

                    /// Reset transform-box from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_transform_box(&mut self, other: &Self) {
                        self.copy_transform_box_from(other)
                    }

                    /// Get the computed value for transform-box.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_transform_box(&self) -> longhands::transform_box::computed_value::T {
                        self.transform_box.clone()
                    }
                        /// Set transform-style.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_transform_style(&mut self, v: longhands::transform_style::computed_value::T) {
                            self.transform_style = v;
                        }
                    /// Set transform-style from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_transform_style_from(&mut self, other: &Self) {
                        self.transform_style = other.transform_style.clone();
                    }

                    /// Reset transform-style from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_transform_style(&mut self, other: &Self) {
                        self.copy_transform_style_from(other)
                    }

                    /// Get the computed value for transform-style.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_transform_style(&self) -> longhands::transform_style::computed_value::T {
                        self.transform_style.clone()
                    }
                        /// Set transform-origin.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_transform_origin(&mut self, v: longhands::transform_origin::computed_value::T) {
                            self.transform_origin = v;
                        }
                    /// Set transform-origin from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_transform_origin_from(&mut self, other: &Self) {
                        self.transform_origin = other.transform_origin.clone();
                    }

                    /// Reset transform-origin from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_transform_origin(&mut self, other: &Self) {
                        self.copy_transform_origin_from(other)
                    }

                    /// Get the computed value for transform-origin.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_transform_origin(&self) -> longhands::transform_origin::computed_value::T {
                        self.transform_origin.clone()
                    }
                        /// Set contain.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_contain(&mut self, v: longhands::contain::computed_value::T) {
                            self.contain = v;
                        }
                    /// Set contain from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_contain_from(&mut self, other: &Self) {
                        self.contain = other.contain.clone();
                    }

                    /// Reset contain from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_contain(&mut self, other: &Self) {
                        self.copy_contain_from(other)
                    }

                    /// Get the computed value for contain.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_contain(&self) -> longhands::contain::computed_value::T {
                        self.contain.clone()
                    }
                        /// Set -moz-appearance.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set__moz_appearance(&mut self, v: longhands::_moz_appearance::computed_value::T) {
                            self._moz_appearance = v;
                        }
                    /// Set -moz-appearance from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy__moz_appearance_from(&mut self, other: &Self) {
                        self._moz_appearance = other._moz_appearance.clone();
                    }

                    /// Reset -moz-appearance from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset__moz_appearance(&mut self, other: &Self) {
                        self.copy__moz_appearance_from(other)
                    }

                    /// Get the computed value for -moz-appearance.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone__moz_appearance(&self) -> longhands::_moz_appearance::computed_value::T {
                        self._moz_appearance.clone()
                    }
                        /// Set -moz-binding.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set__moz_binding(&mut self, v: longhands::_moz_binding::computed_value::T) {
                            self._moz_binding = v;
                        }
                    /// Set -moz-binding from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy__moz_binding_from(&mut self, other: &Self) {
                        self._moz_binding = other._moz_binding.clone();
                    }

                    /// Reset -moz-binding from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset__moz_binding(&mut self, other: &Self) {
                        self.copy__moz_binding_from(other)
                    }

                    /// Get the computed value for -moz-binding.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone__moz_binding(&self) -> longhands::_moz_binding::computed_value::T {
                        self._moz_binding.clone()
                    }
                        /// Set -moz-orient.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set__moz_orient(&mut self, v: longhands::_moz_orient::computed_value::T) {
                            self._moz_orient = v;
                        }
                    /// Set -moz-orient from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy__moz_orient_from(&mut self, other: &Self) {
                        self._moz_orient = other._moz_orient.clone();
                    }

                    /// Reset -moz-orient from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset__moz_orient(&mut self, other: &Self) {
                        self.copy__moz_orient_from(other)
                    }

                    /// Get the computed value for -moz-orient.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone__moz_orient(&self) -> longhands::_moz_orient::computed_value::T {
                        self._moz_orient.clone()
                    }
                        /// Set will-change.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_will_change(&mut self, v: longhands::will_change::computed_value::T) {
                            self.will_change = v;
                        }
                    /// Set will-change from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_will_change_from(&mut self, other: &Self) {
                        self.will_change = other.will_change.clone();
                    }

                    /// Reset will-change from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_will_change(&mut self, other: &Self) {
                        self.copy_will_change_from(other)
                    }

                    /// Get the computed value for will-change.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_will_change(&self) -> longhands::will_change::computed_value::T {
                        self.will_change.clone()
                    }
                        /// Set shape-outside.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_shape_outside(&mut self, v: longhands::shape_outside::computed_value::T) {
                            self.shape_outside = v;
                        }
                    /// Set shape-outside from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_shape_outside_from(&mut self, other: &Self) {
                        self.shape_outside = other.shape_outside.clone();
                    }

                    /// Reset shape-outside from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_shape_outside(&mut self, other: &Self) {
                        self.copy_shape_outside_from(other)
                    }

                    /// Get the computed value for shape-outside.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_shape_outside(&self) -> longhands::shape_outside::computed_value::T {
                        self.shape_outside.clone()
                    }
                        /// Set touch-action.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_touch_action(&mut self, v: longhands::touch_action::computed_value::T) {
                            self.touch_action = v;
                        }
                    /// Set touch-action from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_touch_action_from(&mut self, other: &Self) {
                        self.touch_action = other.touch_action.clone();
                    }

                    /// Reset touch-action from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_touch_action(&mut self, other: &Self) {
                        self.copy_touch_action_from(other)
                    }

                    /// Get the computed value for touch-action.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_touch_action(&self) -> longhands::touch_action::computed_value::T {
                        self.touch_action.clone()
                    }
                /// Sets the display property, but without touching
                /// __servo_display_for_hypothetical_box, except when the
                /// adjustment comes from root or item display fixups.
                pub fn set_adjusted_display(&mut self,
                                            dpy: longhands::display::computed_value::T,
                                            is_item_or_root: bool) {
                    self.set_display(dpy);
                    if is_item_or_root {
                        self.set__servo_display_for_hypothetical_box(dpy);
                    }
                }
        }

        #[derive(Clone, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        /// The Color style struct.
        pub struct Color {
                /// The color computed value.
                pub color: longhands::color::computed_value::T,
        }

        impl Color {
                        /// Set color.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_color(&mut self, v: longhands::color::computed_value::T) {
                            self.color = v;
                        }
                    /// Set color from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_color_from(&mut self, other: &Self) {
                        self.color = other.color.clone();
                    }

                    /// Reset color from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_color(&mut self, other: &Self) {
                        self.copy_color_from(other)
                    }

                    /// Get the computed value for color.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_color(&self) -> longhands::color::computed_value::T {
                        self.color.clone()
                    }
        }

        #[derive(Clone, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        /// The Column style struct.
        pub struct Column {
                /// The column-width computed value.
                pub column_width: longhands::column_width::computed_value::T,
                /// The column-count computed value.
                pub column_count: longhands::column_count::computed_value::T,
                /// The column-gap computed value.
                pub column_gap: longhands::column_gap::computed_value::T,
                /// The column-fill computed value.
                pub column_fill: longhands::column_fill::computed_value::T,
                /// The column-rule-width computed value.
                pub column_rule_width: longhands::column_rule_width::computed_value::T,
                /// The column-rule-color computed value.
                pub column_rule_color: longhands::column_rule_color::computed_value::T,
                /// The column-span computed value.
                pub column_span: longhands::column_span::computed_value::T,
                /// The column-rule-style computed value.
                pub column_rule_style: longhands::column_rule_style::computed_value::T,
        }

        impl Column {
                        /// Set column-width.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_column_width(&mut self, v: longhands::column_width::computed_value::T) {
                            self.column_width = v;
                        }
                    /// Set column-width from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_column_width_from(&mut self, other: &Self) {
                        self.column_width = other.column_width.clone();
                    }

                    /// Reset column-width from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_column_width(&mut self, other: &Self) {
                        self.copy_column_width_from(other)
                    }

                    /// Get the computed value for column-width.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_column_width(&self) -> longhands::column_width::computed_value::T {
                        self.column_width.clone()
                    }
                        /// Set column-count.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_column_count(&mut self, v: longhands::column_count::computed_value::T) {
                            self.column_count = v;
                        }
                    /// Set column-count from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_column_count_from(&mut self, other: &Self) {
                        self.column_count = other.column_count.clone();
                    }

                    /// Reset column-count from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_column_count(&mut self, other: &Self) {
                        self.copy_column_count_from(other)
                    }

                    /// Get the computed value for column-count.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_column_count(&self) -> longhands::column_count::computed_value::T {
                        self.column_count.clone()
                    }
                        /// Set column-gap.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_column_gap(&mut self, v: longhands::column_gap::computed_value::T) {
                            self.column_gap = v;
                        }
                    /// Set column-gap from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_column_gap_from(&mut self, other: &Self) {
                        self.column_gap = other.column_gap.clone();
                    }

                    /// Reset column-gap from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_column_gap(&mut self, other: &Self) {
                        self.copy_column_gap_from(other)
                    }

                    /// Get the computed value for column-gap.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_column_gap(&self) -> longhands::column_gap::computed_value::T {
                        self.column_gap.clone()
                    }
                        /// Set column-fill.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_column_fill(&mut self, v: longhands::column_fill::computed_value::T) {
                            self.column_fill = v;
                        }
                    /// Set column-fill from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_column_fill_from(&mut self, other: &Self) {
                        self.column_fill = other.column_fill.clone();
                    }

                    /// Reset column-fill from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_column_fill(&mut self, other: &Self) {
                        self.copy_column_fill_from(other)
                    }

                    /// Get the computed value for column-fill.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_column_fill(&self) -> longhands::column_fill::computed_value::T {
                        self.column_fill.clone()
                    }
                        /// Set column-rule-width.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_column_rule_width(&mut self, v: longhands::column_rule_width::computed_value::T) {
                            self.column_rule_width = v;
                        }
                    /// Set column-rule-width from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_column_rule_width_from(&mut self, other: &Self) {
                        self.column_rule_width = other.column_rule_width.clone();
                    }

                    /// Reset column-rule-width from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_column_rule_width(&mut self, other: &Self) {
                        self.copy_column_rule_width_from(other)
                    }

                    /// Get the computed value for column-rule-width.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_column_rule_width(&self) -> longhands::column_rule_width::computed_value::T {
                        self.column_rule_width.clone()
                    }
                        /// Set column-rule-color.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_column_rule_color(&mut self, v: longhands::column_rule_color::computed_value::T) {
                            self.column_rule_color = v;
                        }
                    /// Set column-rule-color from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_column_rule_color_from(&mut self, other: &Self) {
                        self.column_rule_color = other.column_rule_color.clone();
                    }

                    /// Reset column-rule-color from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_column_rule_color(&mut self, other: &Self) {
                        self.copy_column_rule_color_from(other)
                    }

                    /// Get the computed value for column-rule-color.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_column_rule_color(&self) -> longhands::column_rule_color::computed_value::T {
                        self.column_rule_color.clone()
                    }
                        /// Set column-span.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_column_span(&mut self, v: longhands::column_span::computed_value::T) {
                            self.column_span = v;
                        }
                    /// Set column-span from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_column_span_from(&mut self, other: &Self) {
                        self.column_span = other.column_span.clone();
                    }

                    /// Reset column-span from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_column_span(&mut self, other: &Self) {
                        self.copy_column_span_from(other)
                    }

                    /// Get the computed value for column-span.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_column_span(&self) -> longhands::column_span::computed_value::T {
                        self.column_span.clone()
                    }
                        /// Set column-rule-style.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_column_rule_style(&mut self, v: longhands::column_rule_style::computed_value::T) {
                            self.column_rule_style = v;
                        }
                    /// Set column-rule-style from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_column_rule_style_from(&mut self, other: &Self) {
                        self.column_rule_style = other.column_rule_style.clone();
                    }

                    /// Reset column-rule-style from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_column_rule_style(&mut self, other: &Self) {
                        self.copy_column_rule_style_from(other)
                    }

                    /// Get the computed value for column-rule-style.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_column_rule_style(&self) -> longhands::column_rule_style::computed_value::T {
                        self.column_rule_style.clone()
                    }
        }

        #[derive(Clone, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        /// The Counters style struct.
        pub struct Counters {
                /// The content computed value.
                pub content: longhands::content::computed_value::T,
                /// The counter-increment computed value.
                pub counter_increment: longhands::counter_increment::computed_value::T,
                /// The counter-reset computed value.
                pub counter_reset: longhands::counter_reset::computed_value::T,
        }

        impl Counters {
                        /// Set content.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_content(&mut self, v: longhands::content::computed_value::T) {
                            self.content = v;
                        }
                    /// Set content from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_content_from(&mut self, other: &Self) {
                        self.content = other.content.clone();
                    }

                    /// Reset content from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_content(&mut self, other: &Self) {
                        self.copy_content_from(other)
                    }

                    /// Get the computed value for content.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_content(&self) -> longhands::content::computed_value::T {
                        self.content.clone()
                    }
                        /// Set counter-increment.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_counter_increment(&mut self, v: longhands::counter_increment::computed_value::T) {
                            self.counter_increment = v;
                        }
                    /// Set counter-increment from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_counter_increment_from(&mut self, other: &Self) {
                        self.counter_increment = other.counter_increment.clone();
                    }

                    /// Reset counter-increment from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_counter_increment(&mut self, other: &Self) {
                        self.copy_counter_increment_from(other)
                    }

                    /// Get the computed value for counter-increment.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_counter_increment(&self) -> longhands::counter_increment::computed_value::T {
                        self.counter_increment.clone()
                    }
                        /// Set counter-reset.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_counter_reset(&mut self, v: longhands::counter_reset::computed_value::T) {
                            self.counter_reset = v;
                        }
                    /// Set counter-reset from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_counter_reset_from(&mut self, other: &Self) {
                        self.counter_reset = other.counter_reset.clone();
                    }

                    /// Reset counter-reset from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_counter_reset(&mut self, other: &Self) {
                        self.copy_counter_reset_from(other)
                    }

                    /// Get the computed value for counter-reset.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_counter_reset(&self) -> longhands::counter_reset::computed_value::T {
                        self.counter_reset.clone()
                    }
        }

        #[derive(Clone, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        /// The Effects style struct.
        pub struct Effects {
                /// The opacity computed value.
                pub opacity: longhands::opacity::computed_value::T,
                /// The box-shadow computed value.
                pub box_shadow: longhands::box_shadow::computed_value::T,
                /// The clip computed value.
                pub clip: longhands::clip::computed_value::T,
                /// The filter computed value.
                pub filter: longhands::filter::computed_value::T,
                /// The mix-blend-mode computed value.
                pub mix_blend_mode: longhands::mix_blend_mode::computed_value::T,
        }

        impl Effects {
                        /// Set opacity.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_opacity(&mut self, v: longhands::opacity::computed_value::T) {
                            self.opacity = v;
                        }
                    /// Set opacity from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_opacity_from(&mut self, other: &Self) {
                        self.opacity = other.opacity.clone();
                    }

                    /// Reset opacity from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_opacity(&mut self, other: &Self) {
                        self.copy_opacity_from(other)
                    }

                    /// Get the computed value for opacity.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_opacity(&self) -> longhands::opacity::computed_value::T {
                        self.opacity.clone()
                    }
                        /// Set box-shadow.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_box_shadow<I>(&mut self, v: I)
                            where I: IntoIterator<Item = longhands::box_shadow
                                                                  ::computed_value::single_value::T>,
                                  I::IntoIter: ExactSizeIterator
                        {
                            self.box_shadow = longhands::box_shadow::computed_value
                                                              ::T(v.into_iter().collect());
                        }
                    /// Set box-shadow from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_box_shadow_from(&mut self, other: &Self) {
                        self.box_shadow = other.box_shadow.clone();
                    }

                    /// Reset box-shadow from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_box_shadow(&mut self, other: &Self) {
                        self.copy_box_shadow_from(other)
                    }

                    /// Get the computed value for box-shadow.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_box_shadow(&self) -> longhands::box_shadow::computed_value::T {
                        self.box_shadow.clone()
                    }
                        /// Set clip.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_clip(&mut self, v: longhands::clip::computed_value::T) {
                            self.clip = v;
                        }
                    /// Set clip from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_clip_from(&mut self, other: &Self) {
                        self.clip = other.clip.clone();
                    }

                    /// Reset clip from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_clip(&mut self, other: &Self) {
                        self.copy_clip_from(other)
                    }

                    /// Get the computed value for clip.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_clip(&self) -> longhands::clip::computed_value::T {
                        self.clip.clone()
                    }
                        /// Set filter.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_filter<I>(&mut self, v: I)
                            where I: IntoIterator<Item = longhands::filter
                                                                  ::computed_value::single_value::T>,
                                  I::IntoIter: ExactSizeIterator
                        {
                            self.filter = longhands::filter::computed_value
                                                              ::T(v.into_iter().collect());
                        }
                    /// Set filter from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_filter_from(&mut self, other: &Self) {
                        self.filter = other.filter.clone();
                    }

                    /// Reset filter from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_filter(&mut self, other: &Self) {
                        self.copy_filter_from(other)
                    }

                    /// Get the computed value for filter.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_filter(&self) -> longhands::filter::computed_value::T {
                        self.filter.clone()
                    }
                        /// Set mix-blend-mode.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_mix_blend_mode(&mut self, v: longhands::mix_blend_mode::computed_value::T) {
                            self.mix_blend_mode = v;
                        }
                    /// Set mix-blend-mode from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_mix_blend_mode_from(&mut self, other: &Self) {
                        self.mix_blend_mode = other.mix_blend_mode.clone();
                    }

                    /// Reset mix-blend-mode from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_mix_blend_mode(&mut self, other: &Self) {
                        self.copy_mix_blend_mode_from(other)
                    }

                    /// Get the computed value for mix-blend-mode.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_mix_blend_mode(&self) -> longhands::mix_blend_mode::computed_value::T {
                        self.mix_blend_mode.clone()
                    }
        }

        #[derive(Clone, Debug)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        /// The Font style struct.
        pub struct Font {
                /// The font-family computed value.
                pub font_family: longhands::font_family::computed_value::T,
                /// The font-style computed value.
                pub font_style: longhands::font_style::computed_value::T,
                /// The font-variant-caps computed value.
                pub font_variant_caps: longhands::font_variant_caps::computed_value::T,
                /// The font-weight computed value.
                pub font_weight: longhands::font_weight::computed_value::T,
                /// The font-size computed value.
                pub font_size: longhands::font_size::computed_value::T,
                /// The font-size-adjust computed value.
                pub font_size_adjust: longhands::font_size_adjust::computed_value::T,
                /// The font-synthesis computed value.
                pub font_synthesis: longhands::font_synthesis::computed_value::T,
                /// The font-stretch computed value.
                pub font_stretch: longhands::font_stretch::computed_value::T,
                /// The font-kerning computed value.
                pub font_kerning: longhands::font_kerning::computed_value::T,
                /// The font-variant-alternates computed value.
                pub font_variant_alternates: longhands::font_variant_alternates::computed_value::T,
                /// The font-variant-east-asian computed value.
                pub font_variant_east_asian: longhands::font_variant_east_asian::computed_value::T,
                /// The font-variant-ligatures computed value.
                pub font_variant_ligatures: longhands::font_variant_ligatures::computed_value::T,
                /// The font-variant-numeric computed value.
                pub font_variant_numeric: longhands::font_variant_numeric::computed_value::T,
                /// The font-variant-position computed value.
                pub font_variant_position: longhands::font_variant_position::computed_value::T,
                /// The font-feature-settings computed value.
                pub font_feature_settings: longhands::font_feature_settings::computed_value::T,
                /// The font-variation-settings computed value.
                pub font_variation_settings: longhands::font_variation_settings::computed_value::T,
                /// The font-language-override computed value.
                pub font_language_override: longhands::font_language_override::computed_value::T,
                /// The -x-lang computed value.
                pub _x_lang: longhands::_x_lang::computed_value::T,
                /// The -moz-script-size-multiplier computed value.
                pub _moz_script_size_multiplier: longhands::_moz_script_size_multiplier::computed_value::T,
                /// The -moz-script-level computed value.
                pub _moz_script_level: longhands::_moz_script_level::computed_value::T,
                /// The -moz-math-display computed value.
                pub _moz_math_display: longhands::_moz_math_display::computed_value::T,
                /// The -moz-math-variant computed value.
                pub _moz_math_variant: longhands::_moz_math_variant::computed_value::T,
                /// The -moz-script-min-size computed value.
                pub _moz_script_min_size: longhands::_moz_script_min_size::computed_value::T,
                /// The -x-text-zoom computed value.
                pub _x_text_zoom: longhands::_x_text_zoom::computed_value::T,
                /// The -moz-osx-font-smoothing computed value.
                pub _moz_osx_font_smoothing: longhands::_moz_osx_font_smoothing::computed_value::T,
                /// The -moz-min-font-size-ratio computed value.
                pub _moz_min_font_size_ratio: longhands::_moz_min_font_size_ratio::computed_value::T,
                /// The font hash, used for font caching.
                pub hash: u64,
        }

        impl PartialEq for Font {
            fn eq(&self, other: &Font) -> bool {
                self.hash == other.hash
                    && self.font_family == other.font_family
                    && self.font_style == other.font_style
                    && self.font_variant_caps == other.font_variant_caps
                    && self.font_weight == other.font_weight
                    && self.font_size == other.font_size
                    && self.font_size_adjust == other.font_size_adjust
                    && self.font_synthesis == other.font_synthesis
                    && self.font_stretch == other.font_stretch
                    && self.font_kerning == other.font_kerning
                    && self.font_variant_alternates == other.font_variant_alternates
                    && self.font_variant_east_asian == other.font_variant_east_asian
                    && self.font_variant_ligatures == other.font_variant_ligatures
                    && self.font_variant_numeric == other.font_variant_numeric
                    && self.font_variant_position == other.font_variant_position
                    && self.font_feature_settings == other.font_feature_settings
                    && self.font_variation_settings == other.font_variation_settings
                    && self.font_language_override == other.font_language_override
                    && self._x_lang == other._x_lang
                    && self._moz_script_size_multiplier == other._moz_script_size_multiplier
                    && self._moz_script_level == other._moz_script_level
                    && self._moz_math_display == other._moz_math_display
                    && self._moz_math_variant == other._moz_math_variant
                    && self._moz_script_min_size == other._moz_script_min_size
                    && self._x_text_zoom == other._x_text_zoom
                    && self._moz_osx_font_smoothing == other._moz_osx_font_smoothing
                    && self._moz_min_font_size_ratio == other._moz_min_font_size_ratio
            }
        }

        impl Font {
                        /// Set font-family.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_font_family(&mut self, v: longhands::font_family::computed_value::T) {
                            self.font_family = v;
                        }
                    /// Set font-family from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_font_family_from(&mut self, other: &Self) {
                        self.font_family = other.font_family.clone();
                    }

                    /// Reset font-family from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_font_family(&mut self, other: &Self) {
                        self.copy_font_family_from(other)
                    }

                    /// Get the computed value for font-family.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_font_family(&self) -> longhands::font_family::computed_value::T {
                        self.font_family.clone()
                    }
                        /// Set font-style.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_font_style(&mut self, v: longhands::font_style::computed_value::T) {
                            self.font_style = v;
                        }
                    /// Set font-style from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_font_style_from(&mut self, other: &Self) {
                        self.font_style = other.font_style.clone();
                    }

                    /// Reset font-style from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_font_style(&mut self, other: &Self) {
                        self.copy_font_style_from(other)
                    }

                    /// Get the computed value for font-style.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_font_style(&self) -> longhands::font_style::computed_value::T {
                        self.font_style.clone()
                    }
                        /// Set font-variant-caps.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_font_variant_caps(&mut self, v: longhands::font_variant_caps::computed_value::T) {
                            self.font_variant_caps = v;
                        }
                    /// Set font-variant-caps from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_font_variant_caps_from(&mut self, other: &Self) {
                        self.font_variant_caps = other.font_variant_caps.clone();
                    }

                    /// Reset font-variant-caps from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_font_variant_caps(&mut self, other: &Self) {
                        self.copy_font_variant_caps_from(other)
                    }

                    /// Get the computed value for font-variant-caps.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_font_variant_caps(&self) -> longhands::font_variant_caps::computed_value::T {
                        self.font_variant_caps.clone()
                    }
                        /// Set font-weight.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_font_weight(&mut self, v: longhands::font_weight::computed_value::T) {
                            self.font_weight = v;
                        }
                    /// Set font-weight from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_font_weight_from(&mut self, other: &Self) {
                        self.font_weight = other.font_weight.clone();
                    }

                    /// Reset font-weight from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_font_weight(&mut self, other: &Self) {
                        self.copy_font_weight_from(other)
                    }

                    /// Get the computed value for font-weight.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_font_weight(&self) -> longhands::font_weight::computed_value::T {
                        self.font_weight.clone()
                    }
                        /// Set font-size.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_font_size(&mut self, v: longhands::font_size::computed_value::T) {
                            self.font_size = v;
                        }
                    /// Set font-size from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_font_size_from(&mut self, other: &Self) {
                        self.font_size = other.font_size.clone();
                    }

                    /// Reset font-size from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_font_size(&mut self, other: &Self) {
                        self.copy_font_size_from(other)
                    }

                    /// Get the computed value for font-size.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_font_size(&self) -> longhands::font_size::computed_value::T {
                        self.font_size.clone()
                    }
                        /// Set font-size-adjust.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_font_size_adjust(&mut self, v: longhands::font_size_adjust::computed_value::T) {
                            self.font_size_adjust = v;
                        }
                    /// Set font-size-adjust from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_font_size_adjust_from(&mut self, other: &Self) {
                        self.font_size_adjust = other.font_size_adjust.clone();
                    }

                    /// Reset font-size-adjust from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_font_size_adjust(&mut self, other: &Self) {
                        self.copy_font_size_adjust_from(other)
                    }

                    /// Get the computed value for font-size-adjust.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_font_size_adjust(&self) -> longhands::font_size_adjust::computed_value::T {
                        self.font_size_adjust.clone()
                    }
                        /// Set font-synthesis.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_font_synthesis(&mut self, v: longhands::font_synthesis::computed_value::T) {
                            self.font_synthesis = v;
                        }
                    /// Set font-synthesis from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_font_synthesis_from(&mut self, other: &Self) {
                        self.font_synthesis = other.font_synthesis.clone();
                    }

                    /// Reset font-synthesis from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_font_synthesis(&mut self, other: &Self) {
                        self.copy_font_synthesis_from(other)
                    }

                    /// Get the computed value for font-synthesis.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_font_synthesis(&self) -> longhands::font_synthesis::computed_value::T {
                        self.font_synthesis.clone()
                    }
                        /// Set font-stretch.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_font_stretch(&mut self, v: longhands::font_stretch::computed_value::T) {
                            self.font_stretch = v;
                        }
                    /// Set font-stretch from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_font_stretch_from(&mut self, other: &Self) {
                        self.font_stretch = other.font_stretch.clone();
                    }

                    /// Reset font-stretch from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_font_stretch(&mut self, other: &Self) {
                        self.copy_font_stretch_from(other)
                    }

                    /// Get the computed value for font-stretch.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_font_stretch(&self) -> longhands::font_stretch::computed_value::T {
                        self.font_stretch.clone()
                    }
                        /// Set font-kerning.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_font_kerning(&mut self, v: longhands::font_kerning::computed_value::T) {
                            self.font_kerning = v;
                        }
                    /// Set font-kerning from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_font_kerning_from(&mut self, other: &Self) {
                        self.font_kerning = other.font_kerning.clone();
                    }

                    /// Reset font-kerning from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_font_kerning(&mut self, other: &Self) {
                        self.copy_font_kerning_from(other)
                    }

                    /// Get the computed value for font-kerning.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_font_kerning(&self) -> longhands::font_kerning::computed_value::T {
                        self.font_kerning.clone()
                    }
                        /// Set font-variant-alternates.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_font_variant_alternates(&mut self, v: longhands::font_variant_alternates::computed_value::T) {
                            self.font_variant_alternates = v;
                        }
                    /// Set font-variant-alternates from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_font_variant_alternates_from(&mut self, other: &Self) {
                        self.font_variant_alternates = other.font_variant_alternates.clone();
                    }

                    /// Reset font-variant-alternates from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_font_variant_alternates(&mut self, other: &Self) {
                        self.copy_font_variant_alternates_from(other)
                    }

                    /// Get the computed value for font-variant-alternates.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_font_variant_alternates(&self) -> longhands::font_variant_alternates::computed_value::T {
                        self.font_variant_alternates.clone()
                    }
                        /// Set font-variant-east-asian.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_font_variant_east_asian(&mut self, v: longhands::font_variant_east_asian::computed_value::T) {
                            self.font_variant_east_asian = v;
                        }
                    /// Set font-variant-east-asian from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_font_variant_east_asian_from(&mut self, other: &Self) {
                        self.font_variant_east_asian = other.font_variant_east_asian.clone();
                    }

                    /// Reset font-variant-east-asian from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_font_variant_east_asian(&mut self, other: &Self) {
                        self.copy_font_variant_east_asian_from(other)
                    }

                    /// Get the computed value for font-variant-east-asian.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_font_variant_east_asian(&self) -> longhands::font_variant_east_asian::computed_value::T {
                        self.font_variant_east_asian.clone()
                    }
                        /// Set font-variant-ligatures.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_font_variant_ligatures(&mut self, v: longhands::font_variant_ligatures::computed_value::T) {
                            self.font_variant_ligatures = v;
                        }
                    /// Set font-variant-ligatures from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_font_variant_ligatures_from(&mut self, other: &Self) {
                        self.font_variant_ligatures = other.font_variant_ligatures.clone();
                    }

                    /// Reset font-variant-ligatures from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_font_variant_ligatures(&mut self, other: &Self) {
                        self.copy_font_variant_ligatures_from(other)
                    }

                    /// Get the computed value for font-variant-ligatures.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_font_variant_ligatures(&self) -> longhands::font_variant_ligatures::computed_value::T {
                        self.font_variant_ligatures.clone()
                    }
                        /// Set font-variant-numeric.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_font_variant_numeric(&mut self, v: longhands::font_variant_numeric::computed_value::T) {
                            self.font_variant_numeric = v;
                        }
                    /// Set font-variant-numeric from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_font_variant_numeric_from(&mut self, other: &Self) {
                        self.font_variant_numeric = other.font_variant_numeric.clone();
                    }

                    /// Reset font-variant-numeric from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_font_variant_numeric(&mut self, other: &Self) {
                        self.copy_font_variant_numeric_from(other)
                    }

                    /// Get the computed value for font-variant-numeric.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_font_variant_numeric(&self) -> longhands::font_variant_numeric::computed_value::T {
                        self.font_variant_numeric.clone()
                    }
                        /// Set font-variant-position.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_font_variant_position(&mut self, v: longhands::font_variant_position::computed_value::T) {
                            self.font_variant_position = v;
                        }
                    /// Set font-variant-position from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_font_variant_position_from(&mut self, other: &Self) {
                        self.font_variant_position = other.font_variant_position.clone();
                    }

                    /// Reset font-variant-position from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_font_variant_position(&mut self, other: &Self) {
                        self.copy_font_variant_position_from(other)
                    }

                    /// Get the computed value for font-variant-position.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_font_variant_position(&self) -> longhands::font_variant_position::computed_value::T {
                        self.font_variant_position.clone()
                    }
                        /// Set font-feature-settings.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_font_feature_settings(&mut self, v: longhands::font_feature_settings::computed_value::T) {
                            self.font_feature_settings = v;
                        }
                    /// Set font-feature-settings from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_font_feature_settings_from(&mut self, other: &Self) {
                        self.font_feature_settings = other.font_feature_settings.clone();
                    }

                    /// Reset font-feature-settings from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_font_feature_settings(&mut self, other: &Self) {
                        self.copy_font_feature_settings_from(other)
                    }

                    /// Get the computed value for font-feature-settings.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_font_feature_settings(&self) -> longhands::font_feature_settings::computed_value::T {
                        self.font_feature_settings.clone()
                    }
                        /// Set font-variation-settings.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_font_variation_settings(&mut self, v: longhands::font_variation_settings::computed_value::T) {
                            self.font_variation_settings = v;
                        }
                    /// Set font-variation-settings from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_font_variation_settings_from(&mut self, other: &Self) {
                        self.font_variation_settings = other.font_variation_settings.clone();
                    }

                    /// Reset font-variation-settings from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_font_variation_settings(&mut self, other: &Self) {
                        self.copy_font_variation_settings_from(other)
                    }

                    /// Get the computed value for font-variation-settings.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_font_variation_settings(&self) -> longhands::font_variation_settings::computed_value::T {
                        self.font_variation_settings.clone()
                    }
                        /// Set font-language-override.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_font_language_override(&mut self, v: longhands::font_language_override::computed_value::T) {
                            self.font_language_override = v;
                        }
                    /// Set font-language-override from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_font_language_override_from(&mut self, other: &Self) {
                        self.font_language_override = other.font_language_override.clone();
                    }

                    /// Reset font-language-override from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_font_language_override(&mut self, other: &Self) {
                        self.copy_font_language_override_from(other)
                    }

                    /// Get the computed value for font-language-override.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_font_language_override(&self) -> longhands::font_language_override::computed_value::T {
                        self.font_language_override.clone()
                    }
                        /// Set -x-lang.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set__x_lang(&mut self, v: longhands::_x_lang::computed_value::T) {
                            self._x_lang = v;
                        }
                    /// Set -x-lang from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy__x_lang_from(&mut self, other: &Self) {
                        self._x_lang = other._x_lang.clone();
                    }

                    /// Reset -x-lang from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset__x_lang(&mut self, other: &Self) {
                        self.copy__x_lang_from(other)
                    }

                    /// Get the computed value for -x-lang.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone__x_lang(&self) -> longhands::_x_lang::computed_value::T {
                        self._x_lang.clone()
                    }
                        /// Set -moz-script-size-multiplier.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set__moz_script_size_multiplier(&mut self, v: longhands::_moz_script_size_multiplier::computed_value::T) {
                            self._moz_script_size_multiplier = v;
                        }
                    /// Set -moz-script-size-multiplier from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy__moz_script_size_multiplier_from(&mut self, other: &Self) {
                        self._moz_script_size_multiplier = other._moz_script_size_multiplier.clone();
                    }

                    /// Reset -moz-script-size-multiplier from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset__moz_script_size_multiplier(&mut self, other: &Self) {
                        self.copy__moz_script_size_multiplier_from(other)
                    }

                    /// Get the computed value for -moz-script-size-multiplier.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone__moz_script_size_multiplier(&self) -> longhands::_moz_script_size_multiplier::computed_value::T {
                        self._moz_script_size_multiplier.clone()
                    }
                        /// Set -moz-script-level.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set__moz_script_level(&mut self, v: longhands::_moz_script_level::computed_value::T) {
                            self._moz_script_level = v;
                        }
                    /// Set -moz-script-level from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy__moz_script_level_from(&mut self, other: &Self) {
                        self._moz_script_level = other._moz_script_level.clone();
                    }

                    /// Reset -moz-script-level from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset__moz_script_level(&mut self, other: &Self) {
                        self.copy__moz_script_level_from(other)
                    }

                    /// Get the computed value for -moz-script-level.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone__moz_script_level(&self) -> longhands::_moz_script_level::computed_value::T {
                        self._moz_script_level.clone()
                    }
                        /// Set -moz-math-display.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set__moz_math_display(&mut self, v: longhands::_moz_math_display::computed_value::T) {
                            self._moz_math_display = v;
                        }
                    /// Set -moz-math-display from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy__moz_math_display_from(&mut self, other: &Self) {
                        self._moz_math_display = other._moz_math_display.clone();
                    }

                    /// Reset -moz-math-display from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset__moz_math_display(&mut self, other: &Self) {
                        self.copy__moz_math_display_from(other)
                    }

                    /// Get the computed value for -moz-math-display.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone__moz_math_display(&self) -> longhands::_moz_math_display::computed_value::T {
                        self._moz_math_display.clone()
                    }
                        /// Set -moz-math-variant.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set__moz_math_variant(&mut self, v: longhands::_moz_math_variant::computed_value::T) {
                            self._moz_math_variant = v;
                        }
                    /// Set -moz-math-variant from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy__moz_math_variant_from(&mut self, other: &Self) {
                        self._moz_math_variant = other._moz_math_variant.clone();
                    }

                    /// Reset -moz-math-variant from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset__moz_math_variant(&mut self, other: &Self) {
                        self.copy__moz_math_variant_from(other)
                    }

                    /// Get the computed value for -moz-math-variant.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone__moz_math_variant(&self) -> longhands::_moz_math_variant::computed_value::T {
                        self._moz_math_variant.clone()
                    }
                        /// Set -moz-script-min-size.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set__moz_script_min_size(&mut self, v: longhands::_moz_script_min_size::computed_value::T) {
                            self._moz_script_min_size = v;
                        }
                    /// Set -moz-script-min-size from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy__moz_script_min_size_from(&mut self, other: &Self) {
                        self._moz_script_min_size = other._moz_script_min_size.clone();
                    }

                    /// Reset -moz-script-min-size from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset__moz_script_min_size(&mut self, other: &Self) {
                        self.copy__moz_script_min_size_from(other)
                    }

                    /// Get the computed value for -moz-script-min-size.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone__moz_script_min_size(&self) -> longhands::_moz_script_min_size::computed_value::T {
                        self._moz_script_min_size.clone()
                    }
                        /// Set -x-text-zoom.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set__x_text_zoom(&mut self, v: longhands::_x_text_zoom::computed_value::T) {
                            self._x_text_zoom = v;
                        }
                    /// Set -x-text-zoom from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy__x_text_zoom_from(&mut self, other: &Self) {
                        self._x_text_zoom = other._x_text_zoom.clone();
                    }

                    /// Reset -x-text-zoom from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset__x_text_zoom(&mut self, other: &Self) {
                        self.copy__x_text_zoom_from(other)
                    }

                    /// Get the computed value for -x-text-zoom.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone__x_text_zoom(&self) -> longhands::_x_text_zoom::computed_value::T {
                        self._x_text_zoom.clone()
                    }
                        /// Set -moz-osx-font-smoothing.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set__moz_osx_font_smoothing(&mut self, v: longhands::_moz_osx_font_smoothing::computed_value::T) {
                            self._moz_osx_font_smoothing = v;
                        }
                    /// Set -moz-osx-font-smoothing from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy__moz_osx_font_smoothing_from(&mut self, other: &Self) {
                        self._moz_osx_font_smoothing = other._moz_osx_font_smoothing.clone();
                    }

                    /// Reset -moz-osx-font-smoothing from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset__moz_osx_font_smoothing(&mut self, other: &Self) {
                        self.copy__moz_osx_font_smoothing_from(other)
                    }

                    /// Get the computed value for -moz-osx-font-smoothing.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone__moz_osx_font_smoothing(&self) -> longhands::_moz_osx_font_smoothing::computed_value::T {
                        self._moz_osx_font_smoothing.clone()
                    }
                        /// Set -moz-min-font-size-ratio.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set__moz_min_font_size_ratio(&mut self, v: longhands::_moz_min_font_size_ratio::computed_value::T) {
                            self._moz_min_font_size_ratio = v;
                        }
                    /// Set -moz-min-font-size-ratio from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy__moz_min_font_size_ratio_from(&mut self, other: &Self) {
                        self._moz_min_font_size_ratio = other._moz_min_font_size_ratio.clone();
                    }

                    /// Reset -moz-min-font-size-ratio from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset__moz_min_font_size_ratio(&mut self, other: &Self) {
                        self.copy__moz_min_font_size_ratio_from(other)
                    }

                    /// Get the computed value for -moz-min-font-size-ratio.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone__moz_min_font_size_ratio(&self) -> longhands::_moz_min_font_size_ratio::computed_value::T {
                        self._moz_min_font_size_ratio.clone()
                    }
                /// Computes a font hash in order to be able to cache fonts
                /// effectively in GFX and layout.
                pub fn compute_font_hash(&mut self) {
                    // Corresponds to the fields in
                    // `gfx::font_template::FontTemplateDescriptor`.
                    let mut hasher: FnvHasher = Default::default();
                    hasher.write_u16(self.font_weight.0);
                    self.font_stretch.hash(&mut hasher);
                    self.font_family.hash(&mut hasher);
                    self.hash = hasher.finish()
                }

                /// (Servo does not handle MathML, so this just calls copy_font_size_from)
                pub fn inherit_font_size_from(&mut self, parent: &Self,
                                              _: Option<NonNegativeLength>,
                                              _: &Device) -> bool {
                    self.copy_font_size_from(parent);
                    false
                }
                /// (Servo does not handle MathML, so this just calls set_font_size)
                pub fn apply_font_size(&mut self,
                                       v: longhands::font_size::computed_value::T,
                                       _: &Self,
                                       _: &Device) -> Option<NonNegativeLength> {
                    self.set_font_size(v);
                    None
                }
                /// (Servo does not handle MathML, so this does nothing)
                pub fn apply_unconstrained_font_size(&mut self, _: NonNegativeLength) {
                }

        }

        #[derive(Clone, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        /// The InheritedBox style struct.
        pub struct InheritedBox {
                /// The visibility computed value.
                pub visibility: longhands::visibility::computed_value::T,
                /// The writing-mode computed value.
                pub writing_mode: longhands::writing_mode::computed_value::T,
                /// The direction computed value.
                pub direction: longhands::direction::computed_value::T,
                /// The text-orientation computed value.
                pub text_orientation: longhands::text_orientation::computed_value::T,
                /// The color-adjust computed value.
                pub color_adjust: longhands::color_adjust::computed_value::T,
                /// The image-rendering computed value.
                pub image_rendering: longhands::image_rendering::computed_value::T,
                /// The image-orientation computed value.
                pub image_orientation: longhands::image_orientation::computed_value::T,
        }

        impl InheritedBox {
                        /// Set visibility.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_visibility(&mut self, v: longhands::visibility::computed_value::T) {
                            self.visibility = v;
                        }
                    /// Set visibility from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_visibility_from(&mut self, other: &Self) {
                        self.visibility = other.visibility.clone();
                    }

                    /// Reset visibility from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_visibility(&mut self, other: &Self) {
                        self.copy_visibility_from(other)
                    }

                    /// Get the computed value for visibility.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_visibility(&self) -> longhands::visibility::computed_value::T {
                        self.visibility.clone()
                    }
                        /// Set writing-mode.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_writing_mode(&mut self, v: longhands::writing_mode::computed_value::T) {
                            self.writing_mode = v;
                        }
                    /// Set writing-mode from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_writing_mode_from(&mut self, other: &Self) {
                        self.writing_mode = other.writing_mode.clone();
                    }

                    /// Reset writing-mode from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_writing_mode(&mut self, other: &Self) {
                        self.copy_writing_mode_from(other)
                    }

                    /// Get the computed value for writing-mode.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_writing_mode(&self) -> longhands::writing_mode::computed_value::T {
                        self.writing_mode.clone()
                    }
                        /// Set direction.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_direction(&mut self, v: longhands::direction::computed_value::T) {
                            self.direction = v;
                        }
                    /// Set direction from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_direction_from(&mut self, other: &Self) {
                        self.direction = other.direction.clone();
                    }

                    /// Reset direction from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_direction(&mut self, other: &Self) {
                        self.copy_direction_from(other)
                    }

                    /// Get the computed value for direction.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_direction(&self) -> longhands::direction::computed_value::T {
                        self.direction.clone()
                    }
                        /// Set text-orientation.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_text_orientation(&mut self, v: longhands::text_orientation::computed_value::T) {
                            self.text_orientation = v;
                        }
                    /// Set text-orientation from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_text_orientation_from(&mut self, other: &Self) {
                        self.text_orientation = other.text_orientation.clone();
                    }

                    /// Reset text-orientation from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_text_orientation(&mut self, other: &Self) {
                        self.copy_text_orientation_from(other)
                    }

                    /// Get the computed value for text-orientation.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_text_orientation(&self) -> longhands::text_orientation::computed_value::T {
                        self.text_orientation.clone()
                    }
                        /// Set color-adjust.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_color_adjust(&mut self, v: longhands::color_adjust::computed_value::T) {
                            self.color_adjust = v;
                        }
                    /// Set color-adjust from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_color_adjust_from(&mut self, other: &Self) {
                        self.color_adjust = other.color_adjust.clone();
                    }

                    /// Reset color-adjust from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_color_adjust(&mut self, other: &Self) {
                        self.copy_color_adjust_from(other)
                    }

                    /// Get the computed value for color-adjust.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_color_adjust(&self) -> longhands::color_adjust::computed_value::T {
                        self.color_adjust.clone()
                    }
                        /// Set image-rendering.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_image_rendering(&mut self, v: longhands::image_rendering::computed_value::T) {
                            self.image_rendering = v;
                        }
                    /// Set image-rendering from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_image_rendering_from(&mut self, other: &Self) {
                        self.image_rendering = other.image_rendering.clone();
                    }

                    /// Reset image-rendering from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_image_rendering(&mut self, other: &Self) {
                        self.copy_image_rendering_from(other)
                    }

                    /// Get the computed value for image-rendering.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_image_rendering(&self) -> longhands::image_rendering::computed_value::T {
                        self.image_rendering.clone()
                    }
                        /// Set image-orientation.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_image_orientation(&mut self, v: longhands::image_orientation::computed_value::T) {
                            self.image_orientation = v;
                        }
                    /// Set image-orientation from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_image_orientation_from(&mut self, other: &Self) {
                        self.image_orientation = other.image_orientation.clone();
                    }

                    /// Reset image-orientation from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_image_orientation(&mut self, other: &Self) {
                        self.copy_image_orientation_from(other)
                    }

                    /// Get the computed value for image-orientation.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_image_orientation(&self) -> longhands::image_orientation::computed_value::T {
                        self.image_orientation.clone()
                    }
        }

        #[derive(Clone, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        /// The InheritedTable style struct.
        pub struct InheritedTable {
                /// The border-collapse computed value.
                pub border_collapse: longhands::border_collapse::computed_value::T,
                /// The empty-cells computed value.
                pub empty_cells: longhands::empty_cells::computed_value::T,
                /// The caption-side computed value.
                pub caption_side: longhands::caption_side::computed_value::T,
                /// The border-spacing computed value.
                pub border_spacing: longhands::border_spacing::computed_value::T,
        }

        impl InheritedTable {
                        /// Set border-collapse.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_border_collapse(&mut self, v: longhands::border_collapse::computed_value::T) {
                            self.border_collapse = v;
                        }
                    /// Set border-collapse from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_border_collapse_from(&mut self, other: &Self) {
                        self.border_collapse = other.border_collapse.clone();
                    }

                    /// Reset border-collapse from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_border_collapse(&mut self, other: &Self) {
                        self.copy_border_collapse_from(other)
                    }

                    /// Get the computed value for border-collapse.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_border_collapse(&self) -> longhands::border_collapse::computed_value::T {
                        self.border_collapse.clone()
                    }
                        /// Set empty-cells.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_empty_cells(&mut self, v: longhands::empty_cells::computed_value::T) {
                            self.empty_cells = v;
                        }
                    /// Set empty-cells from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_empty_cells_from(&mut self, other: &Self) {
                        self.empty_cells = other.empty_cells.clone();
                    }

                    /// Reset empty-cells from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_empty_cells(&mut self, other: &Self) {
                        self.copy_empty_cells_from(other)
                    }

                    /// Get the computed value for empty-cells.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_empty_cells(&self) -> longhands::empty_cells::computed_value::T {
                        self.empty_cells.clone()
                    }
                        /// Set caption-side.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_caption_side(&mut self, v: longhands::caption_side::computed_value::T) {
                            self.caption_side = v;
                        }
                    /// Set caption-side from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_caption_side_from(&mut self, other: &Self) {
                        self.caption_side = other.caption_side.clone();
                    }

                    /// Reset caption-side from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_caption_side(&mut self, other: &Self) {
                        self.copy_caption_side_from(other)
                    }

                    /// Get the computed value for caption-side.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_caption_side(&self) -> longhands::caption_side::computed_value::T {
                        self.caption_side.clone()
                    }
                        /// Set border-spacing.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_border_spacing(&mut self, v: longhands::border_spacing::computed_value::T) {
                            self.border_spacing = v;
                        }
                    /// Set border-spacing from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_border_spacing_from(&mut self, other: &Self) {
                        self.border_spacing = other.border_spacing.clone();
                    }

                    /// Reset border-spacing from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_border_spacing(&mut self, other: &Self) {
                        self.copy_border_spacing_from(other)
                    }

                    /// Get the computed value for border-spacing.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_border_spacing(&self) -> longhands::border_spacing::computed_value::T {
                        self.border_spacing.clone()
                    }
        }

        #[derive(Clone, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        /// The InheritedText style struct.
        pub struct InheritedText {
                /// The line-height computed value.
                pub line_height: longhands::line_height::computed_value::T,
                /// The text-transform computed value.
                pub text_transform: longhands::text_transform::computed_value::T,
                /// The hyphens computed value.
                pub hyphens: longhands::hyphens::computed_value::T,
                /// The -moz-text-size-adjust computed value.
                pub _moz_text_size_adjust: longhands::_moz_text_size_adjust::computed_value::T,
                /// The text-indent computed value.
                pub text_indent: longhands::text_indent::computed_value::T,
                /// The overflow-wrap computed value.
                pub overflow_wrap: longhands::overflow_wrap::computed_value::T,
                /// The word-break computed value.
                pub word_break: longhands::word_break::computed_value::T,
                /// The text-justify computed value.
                pub text_justify: longhands::text_justify::computed_value::T,
                /// The text-align-last computed value.
                pub text_align_last: longhands::text_align_last::computed_value::T,
                /// The text-align computed value.
                pub text_align: longhands::text_align::computed_value::T,
                /// The letter-spacing computed value.
                pub letter_spacing: longhands::letter_spacing::computed_value::T,
                /// The word-spacing computed value.
                pub word_spacing: longhands::word_spacing::computed_value::T,
                /// The white-space computed value.
                pub white_space: longhands::white_space::computed_value::T,
                /// The text-shadow computed value.
                pub text_shadow: longhands::text_shadow::computed_value::T,
                /// The text-emphasis-style computed value.
                pub text_emphasis_style: longhands::text_emphasis_style::computed_value::T,
                /// The text-emphasis-position computed value.
                pub text_emphasis_position: longhands::text_emphasis_position::computed_value::T,
                /// The text-emphasis-color computed value.
                pub text_emphasis_color: longhands::text_emphasis_color::computed_value::T,
                /// The -moz-tab-size computed value.
                pub _moz_tab_size: longhands::_moz_tab_size::computed_value::T,
                /// The -webkit-text-fill-color computed value.
                pub _webkit_text_fill_color: longhands::_webkit_text_fill_color::computed_value::T,
                /// The -webkit-text-stroke-color computed value.
                pub _webkit_text_stroke_color: longhands::_webkit_text_stroke_color::computed_value::T,
                /// The -webkit-text-stroke-width computed value.
                pub _webkit_text_stroke_width: longhands::_webkit_text_stroke_width::computed_value::T,
                /// The ruby-align computed value.
                pub ruby_align: longhands::ruby_align::computed_value::T,
                /// The ruby-position computed value.
                pub ruby_position: longhands::ruby_position::computed_value::T,
                /// The text-combine-upright computed value.
                pub text_combine_upright: longhands::text_combine_upright::computed_value::T,
                /// The text-rendering computed value.
                pub text_rendering: longhands::text_rendering::computed_value::T,
                /// The -moz-control-character-visibility computed value.
                pub _moz_control_character_visibility: longhands::_moz_control_character_visibility::computed_value::T,
        }

        impl InheritedText {
                        /// Set line-height.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_line_height(&mut self, v: longhands::line_height::computed_value::T) {
                            self.line_height = v;
                        }
                    /// Set line-height from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_line_height_from(&mut self, other: &Self) {
                        self.line_height = other.line_height.clone();
                    }

                    /// Reset line-height from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_line_height(&mut self, other: &Self) {
                        self.copy_line_height_from(other)
                    }

                    /// Get the computed value for line-height.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_line_height(&self) -> longhands::line_height::computed_value::T {
                        self.line_height.clone()
                    }
                        /// Set text-transform.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_text_transform(&mut self, v: longhands::text_transform::computed_value::T) {
                            self.text_transform = v;
                        }
                    /// Set text-transform from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_text_transform_from(&mut self, other: &Self) {
                        self.text_transform = other.text_transform.clone();
                    }

                    /// Reset text-transform from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_text_transform(&mut self, other: &Self) {
                        self.copy_text_transform_from(other)
                    }

                    /// Get the computed value for text-transform.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_text_transform(&self) -> longhands::text_transform::computed_value::T {
                        self.text_transform.clone()
                    }
                        /// Set hyphens.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_hyphens(&mut self, v: longhands::hyphens::computed_value::T) {
                            self.hyphens = v;
                        }
                    /// Set hyphens from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_hyphens_from(&mut self, other: &Self) {
                        self.hyphens = other.hyphens.clone();
                    }

                    /// Reset hyphens from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_hyphens(&mut self, other: &Self) {
                        self.copy_hyphens_from(other)
                    }

                    /// Get the computed value for hyphens.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_hyphens(&self) -> longhands::hyphens::computed_value::T {
                        self.hyphens.clone()
                    }
                        /// Set -moz-text-size-adjust.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set__moz_text_size_adjust(&mut self, v: longhands::_moz_text_size_adjust::computed_value::T) {
                            self._moz_text_size_adjust = v;
                        }
                    /// Set -moz-text-size-adjust from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy__moz_text_size_adjust_from(&mut self, other: &Self) {
                        self._moz_text_size_adjust = other._moz_text_size_adjust.clone();
                    }

                    /// Reset -moz-text-size-adjust from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset__moz_text_size_adjust(&mut self, other: &Self) {
                        self.copy__moz_text_size_adjust_from(other)
                    }

                    /// Get the computed value for -moz-text-size-adjust.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone__moz_text_size_adjust(&self) -> longhands::_moz_text_size_adjust::computed_value::T {
                        self._moz_text_size_adjust.clone()
                    }
                        /// Set text-indent.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_text_indent(&mut self, v: longhands::text_indent::computed_value::T) {
                            self.text_indent = v;
                        }
                    /// Set text-indent from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_text_indent_from(&mut self, other: &Self) {
                        self.text_indent = other.text_indent.clone();
                    }

                    /// Reset text-indent from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_text_indent(&mut self, other: &Self) {
                        self.copy_text_indent_from(other)
                    }

                    /// Get the computed value for text-indent.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_text_indent(&self) -> longhands::text_indent::computed_value::T {
                        self.text_indent.clone()
                    }
                        /// Set overflow-wrap.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_overflow_wrap(&mut self, v: longhands::overflow_wrap::computed_value::T) {
                            self.overflow_wrap = v;
                        }
                    /// Set overflow-wrap from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_overflow_wrap_from(&mut self, other: &Self) {
                        self.overflow_wrap = other.overflow_wrap.clone();
                    }

                    /// Reset overflow-wrap from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_overflow_wrap(&mut self, other: &Self) {
                        self.copy_overflow_wrap_from(other)
                    }

                    /// Get the computed value for overflow-wrap.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_overflow_wrap(&self) -> longhands::overflow_wrap::computed_value::T {
                        self.overflow_wrap.clone()
                    }
                        /// Set word-break.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_word_break(&mut self, v: longhands::word_break::computed_value::T) {
                            self.word_break = v;
                        }
                    /// Set word-break from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_word_break_from(&mut self, other: &Self) {
                        self.word_break = other.word_break.clone();
                    }

                    /// Reset word-break from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_word_break(&mut self, other: &Self) {
                        self.copy_word_break_from(other)
                    }

                    /// Get the computed value for word-break.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_word_break(&self) -> longhands::word_break::computed_value::T {
                        self.word_break.clone()
                    }
                        /// Set text-justify.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_text_justify(&mut self, v: longhands::text_justify::computed_value::T) {
                            self.text_justify = v;
                        }
                    /// Set text-justify from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_text_justify_from(&mut self, other: &Self) {
                        self.text_justify = other.text_justify.clone();
                    }

                    /// Reset text-justify from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_text_justify(&mut self, other: &Self) {
                        self.copy_text_justify_from(other)
                    }

                    /// Get the computed value for text-justify.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_text_justify(&self) -> longhands::text_justify::computed_value::T {
                        self.text_justify.clone()
                    }
                        /// Set text-align-last.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_text_align_last(&mut self, v: longhands::text_align_last::computed_value::T) {
                            self.text_align_last = v;
                        }
                    /// Set text-align-last from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_text_align_last_from(&mut self, other: &Self) {
                        self.text_align_last = other.text_align_last.clone();
                    }

                    /// Reset text-align-last from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_text_align_last(&mut self, other: &Self) {
                        self.copy_text_align_last_from(other)
                    }

                    /// Get the computed value for text-align-last.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_text_align_last(&self) -> longhands::text_align_last::computed_value::T {
                        self.text_align_last.clone()
                    }
                        /// Set text-align.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_text_align(&mut self, v: longhands::text_align::computed_value::T) {
                            self.text_align = v;
                        }
                    /// Set text-align from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_text_align_from(&mut self, other: &Self) {
                        self.text_align = other.text_align.clone();
                    }

                    /// Reset text-align from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_text_align(&mut self, other: &Self) {
                        self.copy_text_align_from(other)
                    }

                    /// Get the computed value for text-align.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_text_align(&self) -> longhands::text_align::computed_value::T {
                        self.text_align.clone()
                    }
                        /// Set letter-spacing.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_letter_spacing(&mut self, v: longhands::letter_spacing::computed_value::T) {
                            self.letter_spacing = v;
                        }
                    /// Set letter-spacing from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_letter_spacing_from(&mut self, other: &Self) {
                        self.letter_spacing = other.letter_spacing.clone();
                    }

                    /// Reset letter-spacing from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_letter_spacing(&mut self, other: &Self) {
                        self.copy_letter_spacing_from(other)
                    }

                    /// Get the computed value for letter-spacing.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_letter_spacing(&self) -> longhands::letter_spacing::computed_value::T {
                        self.letter_spacing.clone()
                    }
                        /// Set word-spacing.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_word_spacing(&mut self, v: longhands::word_spacing::computed_value::T) {
                            self.word_spacing = v;
                        }
                    /// Set word-spacing from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_word_spacing_from(&mut self, other: &Self) {
                        self.word_spacing = other.word_spacing.clone();
                    }

                    /// Reset word-spacing from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_word_spacing(&mut self, other: &Self) {
                        self.copy_word_spacing_from(other)
                    }

                    /// Get the computed value for word-spacing.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_word_spacing(&self) -> longhands::word_spacing::computed_value::T {
                        self.word_spacing.clone()
                    }
                        /// Set white-space.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_white_space(&mut self, v: longhands::white_space::computed_value::T) {
                            self.white_space = v;
                        }
                    /// Set white-space from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_white_space_from(&mut self, other: &Self) {
                        self.white_space = other.white_space.clone();
                    }

                    /// Reset white-space from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_white_space(&mut self, other: &Self) {
                        self.copy_white_space_from(other)
                    }

                    /// Get the computed value for white-space.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_white_space(&self) -> longhands::white_space::computed_value::T {
                        self.white_space.clone()
                    }
                        /// Set text-shadow.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_text_shadow<I>(&mut self, v: I)
                            where I: IntoIterator<Item = longhands::text_shadow
                                                                  ::computed_value::single_value::T>,
                                  I::IntoIter: ExactSizeIterator
                        {
                            self.text_shadow = longhands::text_shadow::computed_value
                                                              ::T(v.into_iter().collect());
                        }
                    /// Set text-shadow from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_text_shadow_from(&mut self, other: &Self) {
                        self.text_shadow = other.text_shadow.clone();
                    }

                    /// Reset text-shadow from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_text_shadow(&mut self, other: &Self) {
                        self.copy_text_shadow_from(other)
                    }

                    /// Get the computed value for text-shadow.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_text_shadow(&self) -> longhands::text_shadow::computed_value::T {
                        self.text_shadow.clone()
                    }
                        /// Set text-emphasis-style.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_text_emphasis_style(&mut self, v: longhands::text_emphasis_style::computed_value::T) {
                            self.text_emphasis_style = v;
                        }
                    /// Set text-emphasis-style from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_text_emphasis_style_from(&mut self, other: &Self) {
                        self.text_emphasis_style = other.text_emphasis_style.clone();
                    }

                    /// Reset text-emphasis-style from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_text_emphasis_style(&mut self, other: &Self) {
                        self.copy_text_emphasis_style_from(other)
                    }

                    /// Get the computed value for text-emphasis-style.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_text_emphasis_style(&self) -> longhands::text_emphasis_style::computed_value::T {
                        self.text_emphasis_style.clone()
                    }
                        /// Set text-emphasis-position.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_text_emphasis_position(&mut self, v: longhands::text_emphasis_position::computed_value::T) {
                            self.text_emphasis_position = v;
                        }
                    /// Set text-emphasis-position from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_text_emphasis_position_from(&mut self, other: &Self) {
                        self.text_emphasis_position = other.text_emphasis_position.clone();
                    }

                    /// Reset text-emphasis-position from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_text_emphasis_position(&mut self, other: &Self) {
                        self.copy_text_emphasis_position_from(other)
                    }

                    /// Get the computed value for text-emphasis-position.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_text_emphasis_position(&self) -> longhands::text_emphasis_position::computed_value::T {
                        self.text_emphasis_position.clone()
                    }
                        /// Set text-emphasis-color.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_text_emphasis_color(&mut self, v: longhands::text_emphasis_color::computed_value::T) {
                            self.text_emphasis_color = v;
                        }
                    /// Set text-emphasis-color from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_text_emphasis_color_from(&mut self, other: &Self) {
                        self.text_emphasis_color = other.text_emphasis_color.clone();
                    }

                    /// Reset text-emphasis-color from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_text_emphasis_color(&mut self, other: &Self) {
                        self.copy_text_emphasis_color_from(other)
                    }

                    /// Get the computed value for text-emphasis-color.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_text_emphasis_color(&self) -> longhands::text_emphasis_color::computed_value::T {
                        self.text_emphasis_color.clone()
                    }
                        /// Set -moz-tab-size.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set__moz_tab_size(&mut self, v: longhands::_moz_tab_size::computed_value::T) {
                            self._moz_tab_size = v;
                        }
                    /// Set -moz-tab-size from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy__moz_tab_size_from(&mut self, other: &Self) {
                        self._moz_tab_size = other._moz_tab_size.clone();
                    }

                    /// Reset -moz-tab-size from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset__moz_tab_size(&mut self, other: &Self) {
                        self.copy__moz_tab_size_from(other)
                    }

                    /// Get the computed value for -moz-tab-size.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone__moz_tab_size(&self) -> longhands::_moz_tab_size::computed_value::T {
                        self._moz_tab_size.clone()
                    }
                        /// Set -webkit-text-fill-color.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set__webkit_text_fill_color(&mut self, v: longhands::_webkit_text_fill_color::computed_value::T) {
                            self._webkit_text_fill_color = v;
                        }
                    /// Set -webkit-text-fill-color from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy__webkit_text_fill_color_from(&mut self, other: &Self) {
                        self._webkit_text_fill_color = other._webkit_text_fill_color.clone();
                    }

                    /// Reset -webkit-text-fill-color from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset__webkit_text_fill_color(&mut self, other: &Self) {
                        self.copy__webkit_text_fill_color_from(other)
                    }

                    /// Get the computed value for -webkit-text-fill-color.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone__webkit_text_fill_color(&self) -> longhands::_webkit_text_fill_color::computed_value::T {
                        self._webkit_text_fill_color.clone()
                    }
                        /// Set -webkit-text-stroke-color.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set__webkit_text_stroke_color(&mut self, v: longhands::_webkit_text_stroke_color::computed_value::T) {
                            self._webkit_text_stroke_color = v;
                        }
                    /// Set -webkit-text-stroke-color from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy__webkit_text_stroke_color_from(&mut self, other: &Self) {
                        self._webkit_text_stroke_color = other._webkit_text_stroke_color.clone();
                    }

                    /// Reset -webkit-text-stroke-color from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset__webkit_text_stroke_color(&mut self, other: &Self) {
                        self.copy__webkit_text_stroke_color_from(other)
                    }

                    /// Get the computed value for -webkit-text-stroke-color.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone__webkit_text_stroke_color(&self) -> longhands::_webkit_text_stroke_color::computed_value::T {
                        self._webkit_text_stroke_color.clone()
                    }
                        /// Set -webkit-text-stroke-width.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set__webkit_text_stroke_width(&mut self, v: longhands::_webkit_text_stroke_width::computed_value::T) {
                            self._webkit_text_stroke_width = v;
                        }
                    /// Set -webkit-text-stroke-width from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy__webkit_text_stroke_width_from(&mut self, other: &Self) {
                        self._webkit_text_stroke_width = other._webkit_text_stroke_width.clone();
                    }

                    /// Reset -webkit-text-stroke-width from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset__webkit_text_stroke_width(&mut self, other: &Self) {
                        self.copy__webkit_text_stroke_width_from(other)
                    }

                    /// Get the computed value for -webkit-text-stroke-width.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone__webkit_text_stroke_width(&self) -> longhands::_webkit_text_stroke_width::computed_value::T {
                        self._webkit_text_stroke_width.clone()
                    }
                        /// Set ruby-align.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_ruby_align(&mut self, v: longhands::ruby_align::computed_value::T) {
                            self.ruby_align = v;
                        }
                    /// Set ruby-align from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_ruby_align_from(&mut self, other: &Self) {
                        self.ruby_align = other.ruby_align.clone();
                    }

                    /// Reset ruby-align from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_ruby_align(&mut self, other: &Self) {
                        self.copy_ruby_align_from(other)
                    }

                    /// Get the computed value for ruby-align.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_ruby_align(&self) -> longhands::ruby_align::computed_value::T {
                        self.ruby_align.clone()
                    }
                        /// Set ruby-position.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_ruby_position(&mut self, v: longhands::ruby_position::computed_value::T) {
                            self.ruby_position = v;
                        }
                    /// Set ruby-position from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_ruby_position_from(&mut self, other: &Self) {
                        self.ruby_position = other.ruby_position.clone();
                    }

                    /// Reset ruby-position from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_ruby_position(&mut self, other: &Self) {
                        self.copy_ruby_position_from(other)
                    }

                    /// Get the computed value for ruby-position.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_ruby_position(&self) -> longhands::ruby_position::computed_value::T {
                        self.ruby_position.clone()
                    }
                        /// Set text-combine-upright.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_text_combine_upright(&mut self, v: longhands::text_combine_upright::computed_value::T) {
                            self.text_combine_upright = v;
                        }
                    /// Set text-combine-upright from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_text_combine_upright_from(&mut self, other: &Self) {
                        self.text_combine_upright = other.text_combine_upright.clone();
                    }

                    /// Reset text-combine-upright from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_text_combine_upright(&mut self, other: &Self) {
                        self.copy_text_combine_upright_from(other)
                    }

                    /// Get the computed value for text-combine-upright.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_text_combine_upright(&self) -> longhands::text_combine_upright::computed_value::T {
                        self.text_combine_upright.clone()
                    }
                        /// Set text-rendering.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_text_rendering(&mut self, v: longhands::text_rendering::computed_value::T) {
                            self.text_rendering = v;
                        }
                    /// Set text-rendering from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_text_rendering_from(&mut self, other: &Self) {
                        self.text_rendering = other.text_rendering.clone();
                    }

                    /// Reset text-rendering from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_text_rendering(&mut self, other: &Self) {
                        self.copy_text_rendering_from(other)
                    }

                    /// Get the computed value for text-rendering.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_text_rendering(&self) -> longhands::text_rendering::computed_value::T {
                        self.text_rendering.clone()
                    }
                        /// Set -moz-control-character-visibility.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set__moz_control_character_visibility(&mut self, v: longhands::_moz_control_character_visibility::computed_value::T) {
                            self._moz_control_character_visibility = v;
                        }
                    /// Set -moz-control-character-visibility from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy__moz_control_character_visibility_from(&mut self, other: &Self) {
                        self._moz_control_character_visibility = other._moz_control_character_visibility.clone();
                    }

                    /// Reset -moz-control-character-visibility from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset__moz_control_character_visibility(&mut self, other: &Self) {
                        self.copy__moz_control_character_visibility_from(other)
                    }

                    /// Get the computed value for -moz-control-character-visibility.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone__moz_control_character_visibility(&self) -> longhands::_moz_control_character_visibility::computed_value::T {
                        self._moz_control_character_visibility.clone()
                    }
        }

        #[derive(Clone, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        /// The List style struct.
        pub struct List {
                /// The list-style-position computed value.
                pub list_style_position: longhands::list_style_position::computed_value::T,
                /// The list-style-type computed value.
                pub list_style_type: longhands::list_style_type::computed_value::T,
                /// The list-style-image computed value.
                pub list_style_image: longhands::list_style_image::computed_value::T,
                /// The quotes computed value.
                pub quotes: longhands::quotes::computed_value::T,
                /// The -moz-image-region computed value.
                pub _moz_image_region: longhands::_moz_image_region::computed_value::T,
        }

        impl List {
                        /// Set list-style-position.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_list_style_position(&mut self, v: longhands::list_style_position::computed_value::T) {
                            self.list_style_position = v;
                        }
                    /// Set list-style-position from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_list_style_position_from(&mut self, other: &Self) {
                        self.list_style_position = other.list_style_position.clone();
                    }

                    /// Reset list-style-position from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_list_style_position(&mut self, other: &Self) {
                        self.copy_list_style_position_from(other)
                    }

                    /// Get the computed value for list-style-position.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_list_style_position(&self) -> longhands::list_style_position::computed_value::T {
                        self.list_style_position.clone()
                    }
                        /// Set list-style-type.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_list_style_type(&mut self, v: longhands::list_style_type::computed_value::T) {
                            self.list_style_type = v;
                        }
                    /// Set list-style-type from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_list_style_type_from(&mut self, other: &Self) {
                        self.list_style_type = other.list_style_type.clone();
                    }

                    /// Reset list-style-type from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_list_style_type(&mut self, other: &Self) {
                        self.copy_list_style_type_from(other)
                    }

                    /// Get the computed value for list-style-type.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_list_style_type(&self) -> longhands::list_style_type::computed_value::T {
                        self.list_style_type.clone()
                    }
                        /// Set list-style-image.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_list_style_image(&mut self, v: longhands::list_style_image::computed_value::T) {
                            self.list_style_image = v;
                        }
                    /// Set list-style-image from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_list_style_image_from(&mut self, other: &Self) {
                        self.list_style_image = other.list_style_image.clone();
                    }

                    /// Reset list-style-image from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_list_style_image(&mut self, other: &Self) {
                        self.copy_list_style_image_from(other)
                    }

                    /// Get the computed value for list-style-image.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_list_style_image(&self) -> longhands::list_style_image::computed_value::T {
                        self.list_style_image.clone()
                    }
                        /// Set quotes.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_quotes(&mut self, v: longhands::quotes::computed_value::T) {
                            self.quotes = v;
                        }
                    /// Set quotes from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_quotes_from(&mut self, other: &Self) {
                        self.quotes = other.quotes.clone();
                    }

                    /// Reset quotes from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_quotes(&mut self, other: &Self) {
                        self.copy_quotes_from(other)
                    }

                    /// Get the computed value for quotes.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_quotes(&self) -> longhands::quotes::computed_value::T {
                        self.quotes.clone()
                    }
                        /// Set -moz-image-region.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set__moz_image_region(&mut self, v: longhands::_moz_image_region::computed_value::T) {
                            self._moz_image_region = v;
                        }
                    /// Set -moz-image-region from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy__moz_image_region_from(&mut self, other: &Self) {
                        self._moz_image_region = other._moz_image_region.clone();
                    }

                    /// Reset -moz-image-region from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset__moz_image_region(&mut self, other: &Self) {
                        self.copy__moz_image_region_from(other)
                    }

                    /// Get the computed value for -moz-image-region.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone__moz_image_region(&self) -> longhands::_moz_image_region::computed_value::T {
                        self._moz_image_region.clone()
                    }
        }

        #[derive(Clone, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        /// The Margin style struct.
        pub struct Margin {
                /// The margin-top computed value.
                pub margin_top: longhands::margin_top::computed_value::T,
                /// The margin-left computed value.
                pub margin_left: longhands::margin_left::computed_value::T,
                /// The margin-bottom computed value.
                pub margin_bottom: longhands::margin_bottom::computed_value::T,
                /// The margin-right computed value.
                pub margin_right: longhands::margin_right::computed_value::T,
                /// The margin-block-start computed value.
                pub margin_block_start: longhands::margin_block_start::computed_value::T,
                /// The margin-block-end computed value.
                pub margin_block_end: longhands::margin_block_end::computed_value::T,
                /// The margin-inline-start computed value.
                pub margin_inline_start: longhands::margin_inline_start::computed_value::T,
                /// The margin-inline-end computed value.
                pub margin_inline_end: longhands::margin_inline_end::computed_value::T,
        }

        impl Margin {
                        /// Set margin-top.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_margin_top(&mut self, v: longhands::margin_top::computed_value::T) {
                            self.margin_top = v;
                        }
                    /// Set margin-top from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_margin_top_from(&mut self, other: &Self) {
                        self.margin_top = other.margin_top.clone();
                    }

                    /// Reset margin-top from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_margin_top(&mut self, other: &Self) {
                        self.copy_margin_top_from(other)
                    }

                    /// Get the computed value for margin-top.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_margin_top(&self) -> longhands::margin_top::computed_value::T {
                        self.margin_top.clone()
                    }
                        /// Set margin-left.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_margin_left(&mut self, v: longhands::margin_left::computed_value::T) {
                            self.margin_left = v;
                        }
                    /// Set margin-left from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_margin_left_from(&mut self, other: &Self) {
                        self.margin_left = other.margin_left.clone();
                    }

                    /// Reset margin-left from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_margin_left(&mut self, other: &Self) {
                        self.copy_margin_left_from(other)
                    }

                    /// Get the computed value for margin-left.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_margin_left(&self) -> longhands::margin_left::computed_value::T {
                        self.margin_left.clone()
                    }
                        /// Set margin-bottom.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_margin_bottom(&mut self, v: longhands::margin_bottom::computed_value::T) {
                            self.margin_bottom = v;
                        }
                    /// Set margin-bottom from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_margin_bottom_from(&mut self, other: &Self) {
                        self.margin_bottom = other.margin_bottom.clone();
                    }

                    /// Reset margin-bottom from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_margin_bottom(&mut self, other: &Self) {
                        self.copy_margin_bottom_from(other)
                    }

                    /// Get the computed value for margin-bottom.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_margin_bottom(&self) -> longhands::margin_bottom::computed_value::T {
                        self.margin_bottom.clone()
                    }
                        /// Set margin-right.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_margin_right(&mut self, v: longhands::margin_right::computed_value::T) {
                            self.margin_right = v;
                        }
                    /// Set margin-right from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_margin_right_from(&mut self, other: &Self) {
                        self.margin_right = other.margin_right.clone();
                    }

                    /// Reset margin-right from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_margin_right(&mut self, other: &Self) {
                        self.copy_margin_right_from(other)
                    }

                    /// Get the computed value for margin-right.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_margin_right(&self) -> longhands::margin_right::computed_value::T {
                        self.margin_right.clone()
                    }
                    
    /// Set the appropriate physical property for margin-block-start given a writing mode.
    pub fn set_margin_block_start(&mut self,
                                      v: longhands::margin_block_start::computed_value::T,
                                      wm: WritingMode) {
        
    
        use logical_geometry::PhysicalSide;
        match wm.block_start_physical_side() {
                PhysicalSide::Top => {
                    
                self.set_margin_top(v)
            
                }
                PhysicalSide::Left => {
                    
                self.set_margin_left(v)
            
                }
                PhysicalSide::Bottom => {
                    
                self.set_margin_bottom(v)
            
                }
                PhysicalSide::Right => {
                    
                self.set_margin_right(v)
            
                }
        }

    }

    /// Copy the appropriate physical property from another struct for margin-block-start
    /// given a writing mode.
    pub fn copy_margin_block_start_from(&mut self,
                                            other: &Self,
                                            wm: WritingMode) {
        
    
        use logical_geometry::PhysicalSide;
        match wm.block_start_physical_side() {
                PhysicalSide::Top => {
                    
                self.copy_margin_top_from(other)
            
                }
                PhysicalSide::Left => {
                    
                self.copy_margin_left_from(other)
            
                }
                PhysicalSide::Bottom => {
                    
                self.copy_margin_bottom_from(other)
            
                }
                PhysicalSide::Right => {
                    
                self.copy_margin_right_from(other)
            
                }
        }

    }

    /// Copy the appropriate physical property from another struct for margin-block-start
    /// given a writing mode.
    pub fn reset_margin_block_start(&mut self,
                                        other: &Self,
                                        wm: WritingMode) {
        self.copy_margin_block_start_from(other, wm)
    }

    /// Get the computed value for the appropriate physical property for
    /// margin-block-start given a writing mode.
    pub fn clone_margin_block_start(&self, wm: WritingMode)
        -> longhands::margin_block_start::computed_value::T {
    
    
        use logical_geometry::PhysicalSide;
        match wm.block_start_physical_side() {
                PhysicalSide::Top => {
                    
            self.clone_margin_top()
        
                }
                PhysicalSide::Left => {
                    
            self.clone_margin_left()
        
                }
                PhysicalSide::Bottom => {
                    
            self.clone_margin_bottom()
        
                }
                PhysicalSide::Right => {
                    
            self.clone_margin_right()
        
                }
        }

    }

                    
    /// Set the appropriate physical property for margin-block-end given a writing mode.
    pub fn set_margin_block_end(&mut self,
                                      v: longhands::margin_block_end::computed_value::T,
                                      wm: WritingMode) {
        
    
        use logical_geometry::PhysicalSide;
        match wm.block_end_physical_side() {
                PhysicalSide::Top => {
                    
                self.set_margin_top(v)
            
                }
                PhysicalSide::Left => {
                    
                self.set_margin_left(v)
            
                }
                PhysicalSide::Bottom => {
                    
                self.set_margin_bottom(v)
            
                }
                PhysicalSide::Right => {
                    
                self.set_margin_right(v)
            
                }
        }

    }

    /// Copy the appropriate physical property from another struct for margin-block-end
    /// given a writing mode.
    pub fn copy_margin_block_end_from(&mut self,
                                            other: &Self,
                                            wm: WritingMode) {
        
    
        use logical_geometry::PhysicalSide;
        match wm.block_end_physical_side() {
                PhysicalSide::Top => {
                    
                self.copy_margin_top_from(other)
            
                }
                PhysicalSide::Left => {
                    
                self.copy_margin_left_from(other)
            
                }
                PhysicalSide::Bottom => {
                    
                self.copy_margin_bottom_from(other)
            
                }
                PhysicalSide::Right => {
                    
                self.copy_margin_right_from(other)
            
                }
        }

    }

    /// Copy the appropriate physical property from another struct for margin-block-end
    /// given a writing mode.
    pub fn reset_margin_block_end(&mut self,
                                        other: &Self,
                                        wm: WritingMode) {
        self.copy_margin_block_end_from(other, wm)
    }

    /// Get the computed value for the appropriate physical property for
    /// margin-block-end given a writing mode.
    pub fn clone_margin_block_end(&self, wm: WritingMode)
        -> longhands::margin_block_end::computed_value::T {
    
    
        use logical_geometry::PhysicalSide;
        match wm.block_end_physical_side() {
                PhysicalSide::Top => {
                    
            self.clone_margin_top()
        
                }
                PhysicalSide::Left => {
                    
            self.clone_margin_left()
        
                }
                PhysicalSide::Bottom => {
                    
            self.clone_margin_bottom()
        
                }
                PhysicalSide::Right => {
                    
            self.clone_margin_right()
        
                }
        }

    }

                    
    /// Set the appropriate physical property for margin-inline-start given a writing mode.
    pub fn set_margin_inline_start(&mut self,
                                      v: longhands::margin_inline_start::computed_value::T,
                                      wm: WritingMode) {
        
    
        use logical_geometry::PhysicalSide;
        match wm.inline_start_physical_side() {
                PhysicalSide::Top => {
                    
                self.set_margin_top(v)
            
                }
                PhysicalSide::Left => {
                    
                self.set_margin_left(v)
            
                }
                PhysicalSide::Bottom => {
                    
                self.set_margin_bottom(v)
            
                }
                PhysicalSide::Right => {
                    
                self.set_margin_right(v)
            
                }
        }

    }

    /// Copy the appropriate physical property from another struct for margin-inline-start
    /// given a writing mode.
    pub fn copy_margin_inline_start_from(&mut self,
                                            other: &Self,
                                            wm: WritingMode) {
        
    
        use logical_geometry::PhysicalSide;
        match wm.inline_start_physical_side() {
                PhysicalSide::Top => {
                    
                self.copy_margin_top_from(other)
            
                }
                PhysicalSide::Left => {
                    
                self.copy_margin_left_from(other)
            
                }
                PhysicalSide::Bottom => {
                    
                self.copy_margin_bottom_from(other)
            
                }
                PhysicalSide::Right => {
                    
                self.copy_margin_right_from(other)
            
                }
        }

    }

    /// Copy the appropriate physical property from another struct for margin-inline-start
    /// given a writing mode.
    pub fn reset_margin_inline_start(&mut self,
                                        other: &Self,
                                        wm: WritingMode) {
        self.copy_margin_inline_start_from(other, wm)
    }

    /// Get the computed value for the appropriate physical property for
    /// margin-inline-start given a writing mode.
    pub fn clone_margin_inline_start(&self, wm: WritingMode)
        -> longhands::margin_inline_start::computed_value::T {
    
    
        use logical_geometry::PhysicalSide;
        match wm.inline_start_physical_side() {
                PhysicalSide::Top => {
                    
            self.clone_margin_top()
        
                }
                PhysicalSide::Left => {
                    
            self.clone_margin_left()
        
                }
                PhysicalSide::Bottom => {
                    
            self.clone_margin_bottom()
        
                }
                PhysicalSide::Right => {
                    
            self.clone_margin_right()
        
                }
        }

    }

                    
    /// Set the appropriate physical property for margin-inline-end given a writing mode.
    pub fn set_margin_inline_end(&mut self,
                                      v: longhands::margin_inline_end::computed_value::T,
                                      wm: WritingMode) {
        
    
        use logical_geometry::PhysicalSide;
        match wm.inline_end_physical_side() {
                PhysicalSide::Top => {
                    
                self.set_margin_top(v)
            
                }
                PhysicalSide::Left => {
                    
                self.set_margin_left(v)
            
                }
                PhysicalSide::Bottom => {
                    
                self.set_margin_bottom(v)
            
                }
                PhysicalSide::Right => {
                    
                self.set_margin_right(v)
            
                }
        }

    }

    /// Copy the appropriate physical property from another struct for margin-inline-end
    /// given a writing mode.
    pub fn copy_margin_inline_end_from(&mut self,
                                            other: &Self,
                                            wm: WritingMode) {
        
    
        use logical_geometry::PhysicalSide;
        match wm.inline_end_physical_side() {
                PhysicalSide::Top => {
                    
                self.copy_margin_top_from(other)
            
                }
                PhysicalSide::Left => {
                    
                self.copy_margin_left_from(other)
            
                }
                PhysicalSide::Bottom => {
                    
                self.copy_margin_bottom_from(other)
            
                }
                PhysicalSide::Right => {
                    
                self.copy_margin_right_from(other)
            
                }
        }

    }

    /// Copy the appropriate physical property from another struct for margin-inline-end
    /// given a writing mode.
    pub fn reset_margin_inline_end(&mut self,
                                        other: &Self,
                                        wm: WritingMode) {
        self.copy_margin_inline_end_from(other, wm)
    }

    /// Get the computed value for the appropriate physical property for
    /// margin-inline-end given a writing mode.
    pub fn clone_margin_inline_end(&self, wm: WritingMode)
        -> longhands::margin_inline_end::computed_value::T {
    
    
        use logical_geometry::PhysicalSide;
        match wm.inline_end_physical_side() {
                PhysicalSide::Top => {
                    
            self.clone_margin_top()
        
                }
                PhysicalSide::Left => {
                    
            self.clone_margin_left()
        
                }
                PhysicalSide::Bottom => {
                    
            self.clone_margin_bottom()
        
                }
                PhysicalSide::Right => {
                    
            self.clone_margin_right()
        
                }
        }

    }

        }

        #[derive(Clone, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        /// The Outline style struct.
        pub struct Outline {
                /// The outline-color computed value.
                pub outline_color: longhands::outline_color::computed_value::T,
                /// The outline-style computed value.
                pub outline_style: longhands::outline_style::computed_value::T,
                /// The outline-width computed value.
                pub outline_width: longhands::outline_width::computed_value::T,
                /// The -moz-outline-radius-topleft computed value.
                pub _moz_outline_radius_topleft: longhands::_moz_outline_radius_topleft::computed_value::T,
                /// The -moz-outline-radius-topright computed value.
                pub _moz_outline_radius_topright: longhands::_moz_outline_radius_topright::computed_value::T,
                /// The -moz-outline-radius-bottomright computed value.
                pub _moz_outline_radius_bottomright: longhands::_moz_outline_radius_bottomright::computed_value::T,
                /// The -moz-outline-radius-bottomleft computed value.
                pub _moz_outline_radius_bottomleft: longhands::_moz_outline_radius_bottomleft::computed_value::T,
                /// The outline-offset computed value.
                pub outline_offset: longhands::outline_offset::computed_value::T,
        }

        impl Outline {
                        /// Set outline-color.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_outline_color(&mut self, v: longhands::outline_color::computed_value::T) {
                            self.outline_color = v;
                        }
                    /// Set outline-color from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_outline_color_from(&mut self, other: &Self) {
                        self.outline_color = other.outline_color.clone();
                    }

                    /// Reset outline-color from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_outline_color(&mut self, other: &Self) {
                        self.copy_outline_color_from(other)
                    }

                    /// Get the computed value for outline-color.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_outline_color(&self) -> longhands::outline_color::computed_value::T {
                        self.outline_color.clone()
                    }
                        /// Set outline-style.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_outline_style(&mut self, v: longhands::outline_style::computed_value::T) {
                            self.outline_style = v;
                        }
                    /// Set outline-style from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_outline_style_from(&mut self, other: &Self) {
                        self.outline_style = other.outline_style.clone();
                    }

                    /// Reset outline-style from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_outline_style(&mut self, other: &Self) {
                        self.copy_outline_style_from(other)
                    }

                    /// Get the computed value for outline-style.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_outline_style(&self) -> longhands::outline_style::computed_value::T {
                        self.outline_style.clone()
                    }
                        /// Set outline-width.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_outline_width(&mut self, v: longhands::outline_width::computed_value::T) {
                            self.outline_width = v;
                        }
                    /// Set outline-width from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_outline_width_from(&mut self, other: &Self) {
                        self.outline_width = other.outline_width.clone();
                    }

                    /// Reset outline-width from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_outline_width(&mut self, other: &Self) {
                        self.copy_outline_width_from(other)
                    }

                    /// Get the computed value for outline-width.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_outline_width(&self) -> longhands::outline_width::computed_value::T {
                        self.outline_width.clone()
                    }
                        /// Set -moz-outline-radius-topleft.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set__moz_outline_radius_topleft(&mut self, v: longhands::_moz_outline_radius_topleft::computed_value::T) {
                            self._moz_outline_radius_topleft = v;
                        }
                    /// Set -moz-outline-radius-topleft from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy__moz_outline_radius_topleft_from(&mut self, other: &Self) {
                        self._moz_outline_radius_topleft = other._moz_outline_radius_topleft.clone();
                    }

                    /// Reset -moz-outline-radius-topleft from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset__moz_outline_radius_topleft(&mut self, other: &Self) {
                        self.copy__moz_outline_radius_topleft_from(other)
                    }

                    /// Get the computed value for -moz-outline-radius-topleft.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone__moz_outline_radius_topleft(&self) -> longhands::_moz_outline_radius_topleft::computed_value::T {
                        self._moz_outline_radius_topleft.clone()
                    }
                        /// Set -moz-outline-radius-topright.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set__moz_outline_radius_topright(&mut self, v: longhands::_moz_outline_radius_topright::computed_value::T) {
                            self._moz_outline_radius_topright = v;
                        }
                    /// Set -moz-outline-radius-topright from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy__moz_outline_radius_topright_from(&mut self, other: &Self) {
                        self._moz_outline_radius_topright = other._moz_outline_radius_topright.clone();
                    }

                    /// Reset -moz-outline-radius-topright from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset__moz_outline_radius_topright(&mut self, other: &Self) {
                        self.copy__moz_outline_radius_topright_from(other)
                    }

                    /// Get the computed value for -moz-outline-radius-topright.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone__moz_outline_radius_topright(&self) -> longhands::_moz_outline_radius_topright::computed_value::T {
                        self._moz_outline_radius_topright.clone()
                    }
                        /// Set -moz-outline-radius-bottomright.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set__moz_outline_radius_bottomright(&mut self, v: longhands::_moz_outline_radius_bottomright::computed_value::T) {
                            self._moz_outline_radius_bottomright = v;
                        }
                    /// Set -moz-outline-radius-bottomright from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy__moz_outline_radius_bottomright_from(&mut self, other: &Self) {
                        self._moz_outline_radius_bottomright = other._moz_outline_radius_bottomright.clone();
                    }

                    /// Reset -moz-outline-radius-bottomright from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset__moz_outline_radius_bottomright(&mut self, other: &Self) {
                        self.copy__moz_outline_radius_bottomright_from(other)
                    }

                    /// Get the computed value for -moz-outline-radius-bottomright.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone__moz_outline_radius_bottomright(&self) -> longhands::_moz_outline_radius_bottomright::computed_value::T {
                        self._moz_outline_radius_bottomright.clone()
                    }
                        /// Set -moz-outline-radius-bottomleft.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set__moz_outline_radius_bottomleft(&mut self, v: longhands::_moz_outline_radius_bottomleft::computed_value::T) {
                            self._moz_outline_radius_bottomleft = v;
                        }
                    /// Set -moz-outline-radius-bottomleft from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy__moz_outline_radius_bottomleft_from(&mut self, other: &Self) {
                        self._moz_outline_radius_bottomleft = other._moz_outline_radius_bottomleft.clone();
                    }

                    /// Reset -moz-outline-radius-bottomleft from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset__moz_outline_radius_bottomleft(&mut self, other: &Self) {
                        self.copy__moz_outline_radius_bottomleft_from(other)
                    }

                    /// Get the computed value for -moz-outline-radius-bottomleft.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone__moz_outline_radius_bottomleft(&self) -> longhands::_moz_outline_radius_bottomleft::computed_value::T {
                        self._moz_outline_radius_bottomleft.clone()
                    }
                        /// Set outline-offset.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_outline_offset(&mut self, v: longhands::outline_offset::computed_value::T) {
                            self.outline_offset = v;
                        }
                    /// Set outline-offset from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_outline_offset_from(&mut self, other: &Self) {
                        self.outline_offset = other.outline_offset.clone();
                    }

                    /// Reset outline-offset from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_outline_offset(&mut self, other: &Self) {
                        self.copy_outline_offset_from(other)
                    }

                    /// Get the computed value for outline-offset.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_outline_offset(&self) -> longhands::outline_offset::computed_value::T {
                        self.outline_offset.clone()
                    }
                /// Whether the outline-width property is non-zero.
                #[inline]
                pub fn outline_has_nonzero_width(&self) -> bool {
                    self.outline_width != NonNegativeLength::zero()
                }
        }

        #[derive(Clone, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        /// The Padding style struct.
        pub struct Padding {
                /// The padding-top computed value.
                pub padding_top: longhands::padding_top::computed_value::T,
                /// The padding-left computed value.
                pub padding_left: longhands::padding_left::computed_value::T,
                /// The padding-bottom computed value.
                pub padding_bottom: longhands::padding_bottom::computed_value::T,
                /// The padding-right computed value.
                pub padding_right: longhands::padding_right::computed_value::T,
                /// The padding-block-start computed value.
                pub padding_block_start: longhands::padding_block_start::computed_value::T,
                /// The padding-block-end computed value.
                pub padding_block_end: longhands::padding_block_end::computed_value::T,
                /// The padding-inline-start computed value.
                pub padding_inline_start: longhands::padding_inline_start::computed_value::T,
                /// The padding-inline-end computed value.
                pub padding_inline_end: longhands::padding_inline_end::computed_value::T,
        }

        impl Padding {
                        /// Set padding-top.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_padding_top(&mut self, v: longhands::padding_top::computed_value::T) {
                            self.padding_top = v;
                        }
                    /// Set padding-top from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_padding_top_from(&mut self, other: &Self) {
                        self.padding_top = other.padding_top.clone();
                    }

                    /// Reset padding-top from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_padding_top(&mut self, other: &Self) {
                        self.copy_padding_top_from(other)
                    }

                    /// Get the computed value for padding-top.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_padding_top(&self) -> longhands::padding_top::computed_value::T {
                        self.padding_top.clone()
                    }
                        /// Set padding-left.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_padding_left(&mut self, v: longhands::padding_left::computed_value::T) {
                            self.padding_left = v;
                        }
                    /// Set padding-left from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_padding_left_from(&mut self, other: &Self) {
                        self.padding_left = other.padding_left.clone();
                    }

                    /// Reset padding-left from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_padding_left(&mut self, other: &Self) {
                        self.copy_padding_left_from(other)
                    }

                    /// Get the computed value for padding-left.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_padding_left(&self) -> longhands::padding_left::computed_value::T {
                        self.padding_left.clone()
                    }
                        /// Set padding-bottom.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_padding_bottom(&mut self, v: longhands::padding_bottom::computed_value::T) {
                            self.padding_bottom = v;
                        }
                    /// Set padding-bottom from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_padding_bottom_from(&mut self, other: &Self) {
                        self.padding_bottom = other.padding_bottom.clone();
                    }

                    /// Reset padding-bottom from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_padding_bottom(&mut self, other: &Self) {
                        self.copy_padding_bottom_from(other)
                    }

                    /// Get the computed value for padding-bottom.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_padding_bottom(&self) -> longhands::padding_bottom::computed_value::T {
                        self.padding_bottom.clone()
                    }
                        /// Set padding-right.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_padding_right(&mut self, v: longhands::padding_right::computed_value::T) {
                            self.padding_right = v;
                        }
                    /// Set padding-right from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_padding_right_from(&mut self, other: &Self) {
                        self.padding_right = other.padding_right.clone();
                    }

                    /// Reset padding-right from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_padding_right(&mut self, other: &Self) {
                        self.copy_padding_right_from(other)
                    }

                    /// Get the computed value for padding-right.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_padding_right(&self) -> longhands::padding_right::computed_value::T {
                        self.padding_right.clone()
                    }
                    
    /// Set the appropriate physical property for padding-block-start given a writing mode.
    pub fn set_padding_block_start(&mut self,
                                      v: longhands::padding_block_start::computed_value::T,
                                      wm: WritingMode) {
        
    
        use logical_geometry::PhysicalSide;
        match wm.block_start_physical_side() {
                PhysicalSide::Top => {
                    
                self.set_padding_top(v)
            
                }
                PhysicalSide::Left => {
                    
                self.set_padding_left(v)
            
                }
                PhysicalSide::Bottom => {
                    
                self.set_padding_bottom(v)
            
                }
                PhysicalSide::Right => {
                    
                self.set_padding_right(v)
            
                }
        }

    }

    /// Copy the appropriate physical property from another struct for padding-block-start
    /// given a writing mode.
    pub fn copy_padding_block_start_from(&mut self,
                                            other: &Self,
                                            wm: WritingMode) {
        
    
        use logical_geometry::PhysicalSide;
        match wm.block_start_physical_side() {
                PhysicalSide::Top => {
                    
                self.copy_padding_top_from(other)
            
                }
                PhysicalSide::Left => {
                    
                self.copy_padding_left_from(other)
            
                }
                PhysicalSide::Bottom => {
                    
                self.copy_padding_bottom_from(other)
            
                }
                PhysicalSide::Right => {
                    
                self.copy_padding_right_from(other)
            
                }
        }

    }

    /// Copy the appropriate physical property from another struct for padding-block-start
    /// given a writing mode.
    pub fn reset_padding_block_start(&mut self,
                                        other: &Self,
                                        wm: WritingMode) {
        self.copy_padding_block_start_from(other, wm)
    }

    /// Get the computed value for the appropriate physical property for
    /// padding-block-start given a writing mode.
    pub fn clone_padding_block_start(&self, wm: WritingMode)
        -> longhands::padding_block_start::computed_value::T {
    
    
        use logical_geometry::PhysicalSide;
        match wm.block_start_physical_side() {
                PhysicalSide::Top => {
                    
            self.clone_padding_top()
        
                }
                PhysicalSide::Left => {
                    
            self.clone_padding_left()
        
                }
                PhysicalSide::Bottom => {
                    
            self.clone_padding_bottom()
        
                }
                PhysicalSide::Right => {
                    
            self.clone_padding_right()
        
                }
        }

    }

                    
    /// Set the appropriate physical property for padding-block-end given a writing mode.
    pub fn set_padding_block_end(&mut self,
                                      v: longhands::padding_block_end::computed_value::T,
                                      wm: WritingMode) {
        
    
        use logical_geometry::PhysicalSide;
        match wm.block_end_physical_side() {
                PhysicalSide::Top => {
                    
                self.set_padding_top(v)
            
                }
                PhysicalSide::Left => {
                    
                self.set_padding_left(v)
            
                }
                PhysicalSide::Bottom => {
                    
                self.set_padding_bottom(v)
            
                }
                PhysicalSide::Right => {
                    
                self.set_padding_right(v)
            
                }
        }

    }

    /// Copy the appropriate physical property from another struct for padding-block-end
    /// given a writing mode.
    pub fn copy_padding_block_end_from(&mut self,
                                            other: &Self,
                                            wm: WritingMode) {
        
    
        use logical_geometry::PhysicalSide;
        match wm.block_end_physical_side() {
                PhysicalSide::Top => {
                    
                self.copy_padding_top_from(other)
            
                }
                PhysicalSide::Left => {
                    
                self.copy_padding_left_from(other)
            
                }
                PhysicalSide::Bottom => {
                    
                self.copy_padding_bottom_from(other)
            
                }
                PhysicalSide::Right => {
                    
                self.copy_padding_right_from(other)
            
                }
        }

    }

    /// Copy the appropriate physical property from another struct for padding-block-end
    /// given a writing mode.
    pub fn reset_padding_block_end(&mut self,
                                        other: &Self,
                                        wm: WritingMode) {
        self.copy_padding_block_end_from(other, wm)
    }

    /// Get the computed value for the appropriate physical property for
    /// padding-block-end given a writing mode.
    pub fn clone_padding_block_end(&self, wm: WritingMode)
        -> longhands::padding_block_end::computed_value::T {
    
    
        use logical_geometry::PhysicalSide;
        match wm.block_end_physical_side() {
                PhysicalSide::Top => {
                    
            self.clone_padding_top()
        
                }
                PhysicalSide::Left => {
                    
            self.clone_padding_left()
        
                }
                PhysicalSide::Bottom => {
                    
            self.clone_padding_bottom()
        
                }
                PhysicalSide::Right => {
                    
            self.clone_padding_right()
        
                }
        }

    }

                    
    /// Set the appropriate physical property for padding-inline-start given a writing mode.
    pub fn set_padding_inline_start(&mut self,
                                      v: longhands::padding_inline_start::computed_value::T,
                                      wm: WritingMode) {
        
    
        use logical_geometry::PhysicalSide;
        match wm.inline_start_physical_side() {
                PhysicalSide::Top => {
                    
                self.set_padding_top(v)
            
                }
                PhysicalSide::Left => {
                    
                self.set_padding_left(v)
            
                }
                PhysicalSide::Bottom => {
                    
                self.set_padding_bottom(v)
            
                }
                PhysicalSide::Right => {
                    
                self.set_padding_right(v)
            
                }
        }

    }

    /// Copy the appropriate physical property from another struct for padding-inline-start
    /// given a writing mode.
    pub fn copy_padding_inline_start_from(&mut self,
                                            other: &Self,
                                            wm: WritingMode) {
        
    
        use logical_geometry::PhysicalSide;
        match wm.inline_start_physical_side() {
                PhysicalSide::Top => {
                    
                self.copy_padding_top_from(other)
            
                }
                PhysicalSide::Left => {
                    
                self.copy_padding_left_from(other)
            
                }
                PhysicalSide::Bottom => {
                    
                self.copy_padding_bottom_from(other)
            
                }
                PhysicalSide::Right => {
                    
                self.copy_padding_right_from(other)
            
                }
        }

    }

    /// Copy the appropriate physical property from another struct for padding-inline-start
    /// given a writing mode.
    pub fn reset_padding_inline_start(&mut self,
                                        other: &Self,
                                        wm: WritingMode) {
        self.copy_padding_inline_start_from(other, wm)
    }

    /// Get the computed value for the appropriate physical property for
    /// padding-inline-start given a writing mode.
    pub fn clone_padding_inline_start(&self, wm: WritingMode)
        -> longhands::padding_inline_start::computed_value::T {
    
    
        use logical_geometry::PhysicalSide;
        match wm.inline_start_physical_side() {
                PhysicalSide::Top => {
                    
            self.clone_padding_top()
        
                }
                PhysicalSide::Left => {
                    
            self.clone_padding_left()
        
                }
                PhysicalSide::Bottom => {
                    
            self.clone_padding_bottom()
        
                }
                PhysicalSide::Right => {
                    
            self.clone_padding_right()
        
                }
        }

    }

                    
    /// Set the appropriate physical property for padding-inline-end given a writing mode.
    pub fn set_padding_inline_end(&mut self,
                                      v: longhands::padding_inline_end::computed_value::T,
                                      wm: WritingMode) {
        
    
        use logical_geometry::PhysicalSide;
        match wm.inline_end_physical_side() {
                PhysicalSide::Top => {
                    
                self.set_padding_top(v)
            
                }
                PhysicalSide::Left => {
                    
                self.set_padding_left(v)
            
                }
                PhysicalSide::Bottom => {
                    
                self.set_padding_bottom(v)
            
                }
                PhysicalSide::Right => {
                    
                self.set_padding_right(v)
            
                }
        }

    }

    /// Copy the appropriate physical property from another struct for padding-inline-end
    /// given a writing mode.
    pub fn copy_padding_inline_end_from(&mut self,
                                            other: &Self,
                                            wm: WritingMode) {
        
    
        use logical_geometry::PhysicalSide;
        match wm.inline_end_physical_side() {
                PhysicalSide::Top => {
                    
                self.copy_padding_top_from(other)
            
                }
                PhysicalSide::Left => {
                    
                self.copy_padding_left_from(other)
            
                }
                PhysicalSide::Bottom => {
                    
                self.copy_padding_bottom_from(other)
            
                }
                PhysicalSide::Right => {
                    
                self.copy_padding_right_from(other)
            
                }
        }

    }

    /// Copy the appropriate physical property from another struct for padding-inline-end
    /// given a writing mode.
    pub fn reset_padding_inline_end(&mut self,
                                        other: &Self,
                                        wm: WritingMode) {
        self.copy_padding_inline_end_from(other, wm)
    }

    /// Get the computed value for the appropriate physical property for
    /// padding-inline-end given a writing mode.
    pub fn clone_padding_inline_end(&self, wm: WritingMode)
        -> longhands::padding_inline_end::computed_value::T {
    
    
        use logical_geometry::PhysicalSide;
        match wm.inline_end_physical_side() {
                PhysicalSide::Top => {
                    
            self.clone_padding_top()
        
                }
                PhysicalSide::Left => {
                    
            self.clone_padding_left()
        
                }
                PhysicalSide::Bottom => {
                    
            self.clone_padding_bottom()
        
                }
                PhysicalSide::Right => {
                    
            self.clone_padding_right()
        
                }
        }

    }

        }

        #[derive(Clone, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        /// The Pointing style struct.
        pub struct Pointing {
                /// The cursor computed value.
                pub cursor: longhands::cursor::computed_value::T,
                /// The pointer-events computed value.
                pub pointer_events: longhands::pointer_events::computed_value::T,
                /// The -moz-user-input computed value.
                pub _moz_user_input: longhands::_moz_user_input::computed_value::T,
                /// The -moz-user-modify computed value.
                pub _moz_user_modify: longhands::_moz_user_modify::computed_value::T,
                /// The -moz-user-focus computed value.
                pub _moz_user_focus: longhands::_moz_user_focus::computed_value::T,
                /// The caret-color computed value.
                pub caret_color: longhands::caret_color::computed_value::T,
        }

        impl Pointing {
                        /// Set cursor.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_cursor(&mut self, v: longhands::cursor::computed_value::T) {
                            self.cursor = v;
                        }
                    /// Set cursor from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_cursor_from(&mut self, other: &Self) {
                        self.cursor = other.cursor.clone();
                    }

                    /// Reset cursor from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_cursor(&mut self, other: &Self) {
                        self.copy_cursor_from(other)
                    }

                    /// Get the computed value for cursor.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_cursor(&self) -> longhands::cursor::computed_value::T {
                        self.cursor.clone()
                    }
                        /// Set pointer-events.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_pointer_events(&mut self, v: longhands::pointer_events::computed_value::T) {
                            self.pointer_events = v;
                        }
                    /// Set pointer-events from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_pointer_events_from(&mut self, other: &Self) {
                        self.pointer_events = other.pointer_events.clone();
                    }

                    /// Reset pointer-events from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_pointer_events(&mut self, other: &Self) {
                        self.copy_pointer_events_from(other)
                    }

                    /// Get the computed value for pointer-events.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_pointer_events(&self) -> longhands::pointer_events::computed_value::T {
                        self.pointer_events.clone()
                    }
                        /// Set -moz-user-input.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set__moz_user_input(&mut self, v: longhands::_moz_user_input::computed_value::T) {
                            self._moz_user_input = v;
                        }
                    /// Set -moz-user-input from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy__moz_user_input_from(&mut self, other: &Self) {
                        self._moz_user_input = other._moz_user_input.clone();
                    }

                    /// Reset -moz-user-input from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset__moz_user_input(&mut self, other: &Self) {
                        self.copy__moz_user_input_from(other)
                    }

                    /// Get the computed value for -moz-user-input.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone__moz_user_input(&self) -> longhands::_moz_user_input::computed_value::T {
                        self._moz_user_input.clone()
                    }
                        /// Set -moz-user-modify.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set__moz_user_modify(&mut self, v: longhands::_moz_user_modify::computed_value::T) {
                            self._moz_user_modify = v;
                        }
                    /// Set -moz-user-modify from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy__moz_user_modify_from(&mut self, other: &Self) {
                        self._moz_user_modify = other._moz_user_modify.clone();
                    }

                    /// Reset -moz-user-modify from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset__moz_user_modify(&mut self, other: &Self) {
                        self.copy__moz_user_modify_from(other)
                    }

                    /// Get the computed value for -moz-user-modify.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone__moz_user_modify(&self) -> longhands::_moz_user_modify::computed_value::T {
                        self._moz_user_modify.clone()
                    }
                        /// Set -moz-user-focus.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set__moz_user_focus(&mut self, v: longhands::_moz_user_focus::computed_value::T) {
                            self._moz_user_focus = v;
                        }
                    /// Set -moz-user-focus from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy__moz_user_focus_from(&mut self, other: &Self) {
                        self._moz_user_focus = other._moz_user_focus.clone();
                    }

                    /// Reset -moz-user-focus from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset__moz_user_focus(&mut self, other: &Self) {
                        self.copy__moz_user_focus_from(other)
                    }

                    /// Get the computed value for -moz-user-focus.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone__moz_user_focus(&self) -> longhands::_moz_user_focus::computed_value::T {
                        self._moz_user_focus.clone()
                    }
                        /// Set caret-color.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_caret_color(&mut self, v: longhands::caret_color::computed_value::T) {
                            self.caret_color = v;
                        }
                    /// Set caret-color from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_caret_color_from(&mut self, other: &Self) {
                        self.caret_color = other.caret_color.clone();
                    }

                    /// Reset caret-color from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_caret_color(&mut self, other: &Self) {
                        self.copy_caret_color_from(other)
                    }

                    /// Get the computed value for caret-color.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_caret_color(&self) -> longhands::caret_color::computed_value::T {
                        self.caret_color.clone()
                    }
        }

        #[derive(Clone, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        /// The Position style struct.
        pub struct Position {
                /// The top computed value.
                pub top: longhands::top::computed_value::T,
                /// The left computed value.
                pub left: longhands::left::computed_value::T,
                /// The bottom computed value.
                pub bottom: longhands::bottom::computed_value::T,
                /// The right computed value.
                pub right: longhands::right::computed_value::T,
                /// The offset-block-start computed value.
                pub offset_block_start: longhands::offset_block_start::computed_value::T,
                /// The offset-block-end computed value.
                pub offset_block_end: longhands::offset_block_end::computed_value::T,
                /// The offset-inline-start computed value.
                pub offset_inline_start: longhands::offset_inline_start::computed_value::T,
                /// The offset-inline-end computed value.
                pub offset_inline_end: longhands::offset_inline_end::computed_value::T,
                /// The z-index computed value.
                pub z_index: longhands::z_index::computed_value::T,
                /// The flex-direction computed value.
                pub flex_direction: longhands::flex_direction::computed_value::T,
                /// The flex-wrap computed value.
                pub flex_wrap: longhands::flex_wrap::computed_value::T,
                /// The justify-content computed value.
                pub justify_content: longhands::justify_content::computed_value::T,
                /// The align-content computed value.
                pub align_content: longhands::align_content::computed_value::T,
                /// The align-items computed value.
                pub align_items: longhands::align_items::computed_value::T,
                /// The justify-items computed value.
                pub justify_items: longhands::justify_items::computed_value::T,
                /// The flex-grow computed value.
                pub flex_grow: longhands::flex_grow::computed_value::T,
                /// The flex-shrink computed value.
                pub flex_shrink: longhands::flex_shrink::computed_value::T,
                /// The align-self computed value.
                pub align_self: longhands::align_self::computed_value::T,
                /// The justify-self computed value.
                pub justify_self: longhands::justify_self::computed_value::T,
                /// The order computed value.
                pub order: longhands::order::computed_value::T,
                /// The flex-basis computed value.
                pub flex_basis: longhands::flex_basis::computed_value::T,
                /// The width computed value.
                pub width: longhands::width::computed_value::T,
                /// The min-width computed value.
                pub min_width: longhands::min_width::computed_value::T,
                /// The max-width computed value.
                pub max_width: longhands::max_width::computed_value::T,
                /// The height computed value.
                pub height: longhands::height::computed_value::T,
                /// The min-height computed value.
                pub min_height: longhands::min_height::computed_value::T,
                /// The max-height computed value.
                pub max_height: longhands::max_height::computed_value::T,
                /// The block-size computed value.
                pub block_size: longhands::block_size::computed_value::T,
                /// The min-block-size computed value.
                pub min_block_size: longhands::min_block_size::computed_value::T,
                /// The max-block-size computed value.
                pub max_block_size: longhands::max_block_size::computed_value::T,
                /// The inline-size computed value.
                pub inline_size: longhands::inline_size::computed_value::T,
                /// The min-inline-size computed value.
                pub min_inline_size: longhands::min_inline_size::computed_value::T,
                /// The max-inline-size computed value.
                pub max_inline_size: longhands::max_inline_size::computed_value::T,
                /// The box-sizing computed value.
                pub box_sizing: longhands::box_sizing::computed_value::T,
                /// The object-fit computed value.
                pub object_fit: longhands::object_fit::computed_value::T,
                /// The object-position computed value.
                pub object_position: longhands::object_position::computed_value::T,
                /// The grid-row-gap computed value.
                pub grid_row_gap: longhands::grid_row_gap::computed_value::T,
                /// The grid-row-start computed value.
                pub grid_row_start: longhands::grid_row_start::computed_value::T,
                /// The grid-row-end computed value.
                pub grid_row_end: longhands::grid_row_end::computed_value::T,
                /// The grid-auto-rows computed value.
                pub grid_auto_rows: longhands::grid_auto_rows::computed_value::T,
                /// The grid-template-rows computed value.
                pub grid_template_rows: longhands::grid_template_rows::computed_value::T,
                /// The grid-column-gap computed value.
                pub grid_column_gap: longhands::grid_column_gap::computed_value::T,
                /// The grid-column-start computed value.
                pub grid_column_start: longhands::grid_column_start::computed_value::T,
                /// The grid-column-end computed value.
                pub grid_column_end: longhands::grid_column_end::computed_value::T,
                /// The grid-auto-columns computed value.
                pub grid_auto_columns: longhands::grid_auto_columns::computed_value::T,
                /// The grid-template-columns computed value.
                pub grid_template_columns: longhands::grid_template_columns::computed_value::T,
                /// The grid-auto-flow computed value.
                pub grid_auto_flow: longhands::grid_auto_flow::computed_value::T,
                /// The grid-template-areas computed value.
                pub grid_template_areas: longhands::grid_template_areas::computed_value::T,
        }

        impl Position {
                        /// Set top.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_top(&mut self, v: longhands::top::computed_value::T) {
                            self.top = v;
                        }
                    /// Set top from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_top_from(&mut self, other: &Self) {
                        self.top = other.top.clone();
                    }

                    /// Reset top from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_top(&mut self, other: &Self) {
                        self.copy_top_from(other)
                    }

                    /// Get the computed value for top.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_top(&self) -> longhands::top::computed_value::T {
                        self.top.clone()
                    }
                        /// Set left.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_left(&mut self, v: longhands::left::computed_value::T) {
                            self.left = v;
                        }
                    /// Set left from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_left_from(&mut self, other: &Self) {
                        self.left = other.left.clone();
                    }

                    /// Reset left from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_left(&mut self, other: &Self) {
                        self.copy_left_from(other)
                    }

                    /// Get the computed value for left.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_left(&self) -> longhands::left::computed_value::T {
                        self.left.clone()
                    }
                        /// Set bottom.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_bottom(&mut self, v: longhands::bottom::computed_value::T) {
                            self.bottom = v;
                        }
                    /// Set bottom from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_bottom_from(&mut self, other: &Self) {
                        self.bottom = other.bottom.clone();
                    }

                    /// Reset bottom from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_bottom(&mut self, other: &Self) {
                        self.copy_bottom_from(other)
                    }

                    /// Get the computed value for bottom.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_bottom(&self) -> longhands::bottom::computed_value::T {
                        self.bottom.clone()
                    }
                        /// Set right.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_right(&mut self, v: longhands::right::computed_value::T) {
                            self.right = v;
                        }
                    /// Set right from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_right_from(&mut self, other: &Self) {
                        self.right = other.right.clone();
                    }

                    /// Reset right from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_right(&mut self, other: &Self) {
                        self.copy_right_from(other)
                    }

                    /// Get the computed value for right.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_right(&self) -> longhands::right::computed_value::T {
                        self.right.clone()
                    }
                    
    /// Set the appropriate physical property for offset-block-start given a writing mode.
    pub fn set_offset_block_start(&mut self,
                                      v: longhands::offset_block_start::computed_value::T,
                                      wm: WritingMode) {
        
    
        use logical_geometry::PhysicalSide;
        match wm.block_start_physical_side() {
                PhysicalSide::Top => {
                    
                self.set_top(v)
            
                }
                PhysicalSide::Left => {
                    
                self.set_left(v)
            
                }
                PhysicalSide::Bottom => {
                    
                self.set_bottom(v)
            
                }
                PhysicalSide::Right => {
                    
                self.set_right(v)
            
                }
        }

    }

    /// Copy the appropriate physical property from another struct for offset-block-start
    /// given a writing mode.
    pub fn copy_offset_block_start_from(&mut self,
                                            other: &Self,
                                            wm: WritingMode) {
        
    
        use logical_geometry::PhysicalSide;
        match wm.block_start_physical_side() {
                PhysicalSide::Top => {
                    
                self.copy_top_from(other)
            
                }
                PhysicalSide::Left => {
                    
                self.copy_left_from(other)
            
                }
                PhysicalSide::Bottom => {
                    
                self.copy_bottom_from(other)
            
                }
                PhysicalSide::Right => {
                    
                self.copy_right_from(other)
            
                }
        }

    }

    /// Copy the appropriate physical property from another struct for offset-block-start
    /// given a writing mode.
    pub fn reset_offset_block_start(&mut self,
                                        other: &Self,
                                        wm: WritingMode) {
        self.copy_offset_block_start_from(other, wm)
    }

    /// Get the computed value for the appropriate physical property for
    /// offset-block-start given a writing mode.
    pub fn clone_offset_block_start(&self, wm: WritingMode)
        -> longhands::offset_block_start::computed_value::T {
    
    
        use logical_geometry::PhysicalSide;
        match wm.block_start_physical_side() {
                PhysicalSide::Top => {
                    
            self.clone_top()
        
                }
                PhysicalSide::Left => {
                    
            self.clone_left()
        
                }
                PhysicalSide::Bottom => {
                    
            self.clone_bottom()
        
                }
                PhysicalSide::Right => {
                    
            self.clone_right()
        
                }
        }

    }

                    
    /// Set the appropriate physical property for offset-block-end given a writing mode.
    pub fn set_offset_block_end(&mut self,
                                      v: longhands::offset_block_end::computed_value::T,
                                      wm: WritingMode) {
        
    
        use logical_geometry::PhysicalSide;
        match wm.block_end_physical_side() {
                PhysicalSide::Top => {
                    
                self.set_top(v)
            
                }
                PhysicalSide::Left => {
                    
                self.set_left(v)
            
                }
                PhysicalSide::Bottom => {
                    
                self.set_bottom(v)
            
                }
                PhysicalSide::Right => {
                    
                self.set_right(v)
            
                }
        }

    }

    /// Copy the appropriate physical property from another struct for offset-block-end
    /// given a writing mode.
    pub fn copy_offset_block_end_from(&mut self,
                                            other: &Self,
                                            wm: WritingMode) {
        
    
        use logical_geometry::PhysicalSide;
        match wm.block_end_physical_side() {
                PhysicalSide::Top => {
                    
                self.copy_top_from(other)
            
                }
                PhysicalSide::Left => {
                    
                self.copy_left_from(other)
            
                }
                PhysicalSide::Bottom => {
                    
                self.copy_bottom_from(other)
            
                }
                PhysicalSide::Right => {
                    
                self.copy_right_from(other)
            
                }
        }

    }

    /// Copy the appropriate physical property from another struct for offset-block-end
    /// given a writing mode.
    pub fn reset_offset_block_end(&mut self,
                                        other: &Self,
                                        wm: WritingMode) {
        self.copy_offset_block_end_from(other, wm)
    }

    /// Get the computed value for the appropriate physical property for
    /// offset-block-end given a writing mode.
    pub fn clone_offset_block_end(&self, wm: WritingMode)
        -> longhands::offset_block_end::computed_value::T {
    
    
        use logical_geometry::PhysicalSide;
        match wm.block_end_physical_side() {
                PhysicalSide::Top => {
                    
            self.clone_top()
        
                }
                PhysicalSide::Left => {
                    
            self.clone_left()
        
                }
                PhysicalSide::Bottom => {
                    
            self.clone_bottom()
        
                }
                PhysicalSide::Right => {
                    
            self.clone_right()
        
                }
        }

    }

                    
    /// Set the appropriate physical property for offset-inline-start given a writing mode.
    pub fn set_offset_inline_start(&mut self,
                                      v: longhands::offset_inline_start::computed_value::T,
                                      wm: WritingMode) {
        
    
        use logical_geometry::PhysicalSide;
        match wm.inline_start_physical_side() {
                PhysicalSide::Top => {
                    
                self.set_top(v)
            
                }
                PhysicalSide::Left => {
                    
                self.set_left(v)
            
                }
                PhysicalSide::Bottom => {
                    
                self.set_bottom(v)
            
                }
                PhysicalSide::Right => {
                    
                self.set_right(v)
            
                }
        }

    }

    /// Copy the appropriate physical property from another struct for offset-inline-start
    /// given a writing mode.
    pub fn copy_offset_inline_start_from(&mut self,
                                            other: &Self,
                                            wm: WritingMode) {
        
    
        use logical_geometry::PhysicalSide;
        match wm.inline_start_physical_side() {
                PhysicalSide::Top => {
                    
                self.copy_top_from(other)
            
                }
                PhysicalSide::Left => {
                    
                self.copy_left_from(other)
            
                }
                PhysicalSide::Bottom => {
                    
                self.copy_bottom_from(other)
            
                }
                PhysicalSide::Right => {
                    
                self.copy_right_from(other)
            
                }
        }

    }

    /// Copy the appropriate physical property from another struct for offset-inline-start
    /// given a writing mode.
    pub fn reset_offset_inline_start(&mut self,
                                        other: &Self,
                                        wm: WritingMode) {
        self.copy_offset_inline_start_from(other, wm)
    }

    /// Get the computed value for the appropriate physical property for
    /// offset-inline-start given a writing mode.
    pub fn clone_offset_inline_start(&self, wm: WritingMode)
        -> longhands::offset_inline_start::computed_value::T {
    
    
        use logical_geometry::PhysicalSide;
        match wm.inline_start_physical_side() {
                PhysicalSide::Top => {
                    
            self.clone_top()
        
                }
                PhysicalSide::Left => {
                    
            self.clone_left()
        
                }
                PhysicalSide::Bottom => {
                    
            self.clone_bottom()
        
                }
                PhysicalSide::Right => {
                    
            self.clone_right()
        
                }
        }

    }

                    
    /// Set the appropriate physical property for offset-inline-end given a writing mode.
    pub fn set_offset_inline_end(&mut self,
                                      v: longhands::offset_inline_end::computed_value::T,
                                      wm: WritingMode) {
        
    
        use logical_geometry::PhysicalSide;
        match wm.inline_end_physical_side() {
                PhysicalSide::Top => {
                    
                self.set_top(v)
            
                }
                PhysicalSide::Left => {
                    
                self.set_left(v)
            
                }
                PhysicalSide::Bottom => {
                    
                self.set_bottom(v)
            
                }
                PhysicalSide::Right => {
                    
                self.set_right(v)
            
                }
        }

    }

    /// Copy the appropriate physical property from another struct for offset-inline-end
    /// given a writing mode.
    pub fn copy_offset_inline_end_from(&mut self,
                                            other: &Self,
                                            wm: WritingMode) {
        
    
        use logical_geometry::PhysicalSide;
        match wm.inline_end_physical_side() {
                PhysicalSide::Top => {
                    
                self.copy_top_from(other)
            
                }
                PhysicalSide::Left => {
                    
                self.copy_left_from(other)
            
                }
                PhysicalSide::Bottom => {
                    
                self.copy_bottom_from(other)
            
                }
                PhysicalSide::Right => {
                    
                self.copy_right_from(other)
            
                }
        }

    }

    /// Copy the appropriate physical property from another struct for offset-inline-end
    /// given a writing mode.
    pub fn reset_offset_inline_end(&mut self,
                                        other: &Self,
                                        wm: WritingMode) {
        self.copy_offset_inline_end_from(other, wm)
    }

    /// Get the computed value for the appropriate physical property for
    /// offset-inline-end given a writing mode.
    pub fn clone_offset_inline_end(&self, wm: WritingMode)
        -> longhands::offset_inline_end::computed_value::T {
    
    
        use logical_geometry::PhysicalSide;
        match wm.inline_end_physical_side() {
                PhysicalSide::Top => {
                    
            self.clone_top()
        
                }
                PhysicalSide::Left => {
                    
            self.clone_left()
        
                }
                PhysicalSide::Bottom => {
                    
            self.clone_bottom()
        
                }
                PhysicalSide::Right => {
                    
            self.clone_right()
        
                }
        }

    }

                        /// Set z-index.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_z_index(&mut self, v: longhands::z_index::computed_value::T) {
                            self.z_index = v;
                        }
                    /// Set z-index from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_z_index_from(&mut self, other: &Self) {
                        self.z_index = other.z_index.clone();
                    }

                    /// Reset z-index from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_z_index(&mut self, other: &Self) {
                        self.copy_z_index_from(other)
                    }

                    /// Get the computed value for z-index.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_z_index(&self) -> longhands::z_index::computed_value::T {
                        self.z_index.clone()
                    }
                        /// Set flex-direction.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_flex_direction(&mut self, v: longhands::flex_direction::computed_value::T) {
                            self.flex_direction = v;
                        }
                    /// Set flex-direction from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_flex_direction_from(&mut self, other: &Self) {
                        self.flex_direction = other.flex_direction.clone();
                    }

                    /// Reset flex-direction from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_flex_direction(&mut self, other: &Self) {
                        self.copy_flex_direction_from(other)
                    }

                    /// Get the computed value for flex-direction.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_flex_direction(&self) -> longhands::flex_direction::computed_value::T {
                        self.flex_direction.clone()
                    }
                        /// Set flex-wrap.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_flex_wrap(&mut self, v: longhands::flex_wrap::computed_value::T) {
                            self.flex_wrap = v;
                        }
                    /// Set flex-wrap from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_flex_wrap_from(&mut self, other: &Self) {
                        self.flex_wrap = other.flex_wrap.clone();
                    }

                    /// Reset flex-wrap from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_flex_wrap(&mut self, other: &Self) {
                        self.copy_flex_wrap_from(other)
                    }

                    /// Get the computed value for flex-wrap.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_flex_wrap(&self) -> longhands::flex_wrap::computed_value::T {
                        self.flex_wrap.clone()
                    }
                        /// Set justify-content.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_justify_content(&mut self, v: longhands::justify_content::computed_value::T) {
                            self.justify_content = v;
                        }
                    /// Set justify-content from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_justify_content_from(&mut self, other: &Self) {
                        self.justify_content = other.justify_content.clone();
                    }

                    /// Reset justify-content from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_justify_content(&mut self, other: &Self) {
                        self.copy_justify_content_from(other)
                    }

                    /// Get the computed value for justify-content.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_justify_content(&self) -> longhands::justify_content::computed_value::T {
                        self.justify_content.clone()
                    }
                        /// Set align-content.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_align_content(&mut self, v: longhands::align_content::computed_value::T) {
                            self.align_content = v;
                        }
                    /// Set align-content from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_align_content_from(&mut self, other: &Self) {
                        self.align_content = other.align_content.clone();
                    }

                    /// Reset align-content from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_align_content(&mut self, other: &Self) {
                        self.copy_align_content_from(other)
                    }

                    /// Get the computed value for align-content.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_align_content(&self) -> longhands::align_content::computed_value::T {
                        self.align_content.clone()
                    }
                        /// Set align-items.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_align_items(&mut self, v: longhands::align_items::computed_value::T) {
                            self.align_items = v;
                        }
                    /// Set align-items from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_align_items_from(&mut self, other: &Self) {
                        self.align_items = other.align_items.clone();
                    }

                    /// Reset align-items from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_align_items(&mut self, other: &Self) {
                        self.copy_align_items_from(other)
                    }

                    /// Get the computed value for align-items.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_align_items(&self) -> longhands::align_items::computed_value::T {
                        self.align_items.clone()
                    }
                        /// Set justify-items.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_justify_items(&mut self, v: longhands::justify_items::computed_value::T) {
                            self.justify_items = v;
                        }
                    /// Set justify-items from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_justify_items_from(&mut self, other: &Self) {
                        self.justify_items = other.justify_items.clone();
                    }

                    /// Reset justify-items from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_justify_items(&mut self, other: &Self) {
                        self.copy_justify_items_from(other)
                    }

                    /// Get the computed value for justify-items.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_justify_items(&self) -> longhands::justify_items::computed_value::T {
                        self.justify_items.clone()
                    }
                        /// Set flex-grow.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_flex_grow(&mut self, v: longhands::flex_grow::computed_value::T) {
                            self.flex_grow = v;
                        }
                    /// Set flex-grow from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_flex_grow_from(&mut self, other: &Self) {
                        self.flex_grow = other.flex_grow.clone();
                    }

                    /// Reset flex-grow from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_flex_grow(&mut self, other: &Self) {
                        self.copy_flex_grow_from(other)
                    }

                    /// Get the computed value for flex-grow.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_flex_grow(&self) -> longhands::flex_grow::computed_value::T {
                        self.flex_grow.clone()
                    }
                        /// Set flex-shrink.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_flex_shrink(&mut self, v: longhands::flex_shrink::computed_value::T) {
                            self.flex_shrink = v;
                        }
                    /// Set flex-shrink from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_flex_shrink_from(&mut self, other: &Self) {
                        self.flex_shrink = other.flex_shrink.clone();
                    }

                    /// Reset flex-shrink from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_flex_shrink(&mut self, other: &Self) {
                        self.copy_flex_shrink_from(other)
                    }

                    /// Get the computed value for flex-shrink.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_flex_shrink(&self) -> longhands::flex_shrink::computed_value::T {
                        self.flex_shrink.clone()
                    }
                        /// Set align-self.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_align_self(&mut self, v: longhands::align_self::computed_value::T) {
                            self.align_self = v;
                        }
                    /// Set align-self from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_align_self_from(&mut self, other: &Self) {
                        self.align_self = other.align_self.clone();
                    }

                    /// Reset align-self from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_align_self(&mut self, other: &Self) {
                        self.copy_align_self_from(other)
                    }

                    /// Get the computed value for align-self.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_align_self(&self) -> longhands::align_self::computed_value::T {
                        self.align_self.clone()
                    }
                        /// Set justify-self.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_justify_self(&mut self, v: longhands::justify_self::computed_value::T) {
                            self.justify_self = v;
                        }
                    /// Set justify-self from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_justify_self_from(&mut self, other: &Self) {
                        self.justify_self = other.justify_self.clone();
                    }

                    /// Reset justify-self from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_justify_self(&mut self, other: &Self) {
                        self.copy_justify_self_from(other)
                    }

                    /// Get the computed value for justify-self.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_justify_self(&self) -> longhands::justify_self::computed_value::T {
                        self.justify_self.clone()
                    }
                        /// Set order.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_order(&mut self, v: longhands::order::computed_value::T) {
                            self.order = v;
                        }
                    /// Set order from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_order_from(&mut self, other: &Self) {
                        self.order = other.order.clone();
                    }

                    /// Reset order from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_order(&mut self, other: &Self) {
                        self.copy_order_from(other)
                    }

                    /// Get the computed value for order.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_order(&self) -> longhands::order::computed_value::T {
                        self.order.clone()
                    }
                        /// Set flex-basis.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_flex_basis(&mut self, v: longhands::flex_basis::computed_value::T) {
                            self.flex_basis = v;
                        }
                    /// Set flex-basis from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_flex_basis_from(&mut self, other: &Self) {
                        self.flex_basis = other.flex_basis.clone();
                    }

                    /// Reset flex-basis from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_flex_basis(&mut self, other: &Self) {
                        self.copy_flex_basis_from(other)
                    }

                    /// Get the computed value for flex-basis.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_flex_basis(&self) -> longhands::flex_basis::computed_value::T {
                        self.flex_basis.clone()
                    }
                        /// Set width.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_width(&mut self, v: longhands::width::computed_value::T) {
                            self.width = v;
                        }
                    /// Set width from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_width_from(&mut self, other: &Self) {
                        self.width = other.width.clone();
                    }

                    /// Reset width from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_width(&mut self, other: &Self) {
                        self.copy_width_from(other)
                    }

                    /// Get the computed value for width.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_width(&self) -> longhands::width::computed_value::T {
                        self.width.clone()
                    }
                        /// Set min-width.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_min_width(&mut self, v: longhands::min_width::computed_value::T) {
                            self.min_width = v;
                        }
                    /// Set min-width from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_min_width_from(&mut self, other: &Self) {
                        self.min_width = other.min_width.clone();
                    }

                    /// Reset min-width from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_min_width(&mut self, other: &Self) {
                        self.copy_min_width_from(other)
                    }

                    /// Get the computed value for min-width.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_min_width(&self) -> longhands::min_width::computed_value::T {
                        self.min_width.clone()
                    }
                        /// Set max-width.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_max_width(&mut self, v: longhands::max_width::computed_value::T) {
                            self.max_width = v;
                        }
                    /// Set max-width from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_max_width_from(&mut self, other: &Self) {
                        self.max_width = other.max_width.clone();
                    }

                    /// Reset max-width from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_max_width(&mut self, other: &Self) {
                        self.copy_max_width_from(other)
                    }

                    /// Get the computed value for max-width.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_max_width(&self) -> longhands::max_width::computed_value::T {
                        self.max_width.clone()
                    }
                        /// Set height.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_height(&mut self, v: longhands::height::computed_value::T) {
                            self.height = v;
                        }
                    /// Set height from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_height_from(&mut self, other: &Self) {
                        self.height = other.height.clone();
                    }

                    /// Reset height from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_height(&mut self, other: &Self) {
                        self.copy_height_from(other)
                    }

                    /// Get the computed value for height.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_height(&self) -> longhands::height::computed_value::T {
                        self.height.clone()
                    }
                        /// Set min-height.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_min_height(&mut self, v: longhands::min_height::computed_value::T) {
                            self.min_height = v;
                        }
                    /// Set min-height from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_min_height_from(&mut self, other: &Self) {
                        self.min_height = other.min_height.clone();
                    }

                    /// Reset min-height from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_min_height(&mut self, other: &Self) {
                        self.copy_min_height_from(other)
                    }

                    /// Get the computed value for min-height.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_min_height(&self) -> longhands::min_height::computed_value::T {
                        self.min_height.clone()
                    }
                        /// Set max-height.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_max_height(&mut self, v: longhands::max_height::computed_value::T) {
                            self.max_height = v;
                        }
                    /// Set max-height from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_max_height_from(&mut self, other: &Self) {
                        self.max_height = other.max_height.clone();
                    }

                    /// Reset max-height from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_max_height(&mut self, other: &Self) {
                        self.copy_max_height_from(other)
                    }

                    /// Get the computed value for max-height.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_max_height(&self) -> longhands::max_height::computed_value::T {
                        self.max_height.clone()
                    }
                    
    /// Set the appropriate physical property for block-size given a writing mode.
    pub fn set_block_size(&mut self,
                                      v: longhands::block_size::computed_value::T,
                                      wm: WritingMode) {
        
    
        
        if wm.is_vertical() {
            
                self.set_width(v)
            
        } else {
            
                self.set_height(v)
            
        }

    }

    /// Copy the appropriate physical property from another struct for block-size
    /// given a writing mode.
    pub fn copy_block_size_from(&mut self,
                                            other: &Self,
                                            wm: WritingMode) {
        
    
        
        if wm.is_vertical() {
            
                self.copy_width_from(other)
            
        } else {
            
                self.copy_height_from(other)
            
        }

    }

    /// Copy the appropriate physical property from another struct for block-size
    /// given a writing mode.
    pub fn reset_block_size(&mut self,
                                        other: &Self,
                                        wm: WritingMode) {
        self.copy_block_size_from(other, wm)
    }

    /// Get the computed value for the appropriate physical property for
    /// block-size given a writing mode.
    pub fn clone_block_size(&self, wm: WritingMode)
        -> longhands::block_size::computed_value::T {
    
    
        
        if wm.is_vertical() {
            
            self.clone_width()
        
        } else {
            
            self.clone_height()
        
        }

    }

                    
    /// Set the appropriate physical property for min-block-size given a writing mode.
    pub fn set_min_block_size(&mut self,
                                      v: longhands::min_block_size::computed_value::T,
                                      wm: WritingMode) {
        
    
        
        if wm.is_vertical() {
            
                self.set_min_width(v)
            
        } else {
            
                self.set_min_height(v)
            
        }

    }

    /// Copy the appropriate physical property from another struct for min-block-size
    /// given a writing mode.
    pub fn copy_min_block_size_from(&mut self,
                                            other: &Self,
                                            wm: WritingMode) {
        
    
        
        if wm.is_vertical() {
            
                self.copy_min_width_from(other)
            
        } else {
            
                self.copy_min_height_from(other)
            
        }

    }

    /// Copy the appropriate physical property from another struct for min-block-size
    /// given a writing mode.
    pub fn reset_min_block_size(&mut self,
                                        other: &Self,
                                        wm: WritingMode) {
        self.copy_min_block_size_from(other, wm)
    }

    /// Get the computed value for the appropriate physical property for
    /// min-block-size given a writing mode.
    pub fn clone_min_block_size(&self, wm: WritingMode)
        -> longhands::min_block_size::computed_value::T {
    
    
        
        if wm.is_vertical() {
            
            self.clone_min_width()
        
        } else {
            
            self.clone_min_height()
        
        }

    }

                    
    /// Set the appropriate physical property for max-block-size given a writing mode.
    pub fn set_max_block_size(&mut self,
                                      v: longhands::max_block_size::computed_value::T,
                                      wm: WritingMode) {
        
    
        
        if wm.is_vertical() {
            
                self.set_max_width(v)
            
        } else {
            
                self.set_max_height(v)
            
        }

    }

    /// Copy the appropriate physical property from another struct for max-block-size
    /// given a writing mode.
    pub fn copy_max_block_size_from(&mut self,
                                            other: &Self,
                                            wm: WritingMode) {
        
    
        
        if wm.is_vertical() {
            
                self.copy_max_width_from(other)
            
        } else {
            
                self.copy_max_height_from(other)
            
        }

    }

    /// Copy the appropriate physical property from another struct for max-block-size
    /// given a writing mode.
    pub fn reset_max_block_size(&mut self,
                                        other: &Self,
                                        wm: WritingMode) {
        self.copy_max_block_size_from(other, wm)
    }

    /// Get the computed value for the appropriate physical property for
    /// max-block-size given a writing mode.
    pub fn clone_max_block_size(&self, wm: WritingMode)
        -> longhands::max_block_size::computed_value::T {
    
    
        
        if wm.is_vertical() {
            
            self.clone_max_width()
        
        } else {
            
            self.clone_max_height()
        
        }

    }

                    
    /// Set the appropriate physical property for inline-size given a writing mode.
    pub fn set_inline_size(&mut self,
                                      v: longhands::inline_size::computed_value::T,
                                      wm: WritingMode) {
        
    
        
        if wm.is_vertical() {
            
                self.set_height(v)
            
        } else {
            
                self.set_width(v)
            
        }

    }

    /// Copy the appropriate physical property from another struct for inline-size
    /// given a writing mode.
    pub fn copy_inline_size_from(&mut self,
                                            other: &Self,
                                            wm: WritingMode) {
        
    
        
        if wm.is_vertical() {
            
                self.copy_height_from(other)
            
        } else {
            
                self.copy_width_from(other)
            
        }

    }

    /// Copy the appropriate physical property from another struct for inline-size
    /// given a writing mode.
    pub fn reset_inline_size(&mut self,
                                        other: &Self,
                                        wm: WritingMode) {
        self.copy_inline_size_from(other, wm)
    }

    /// Get the computed value for the appropriate physical property for
    /// inline-size given a writing mode.
    pub fn clone_inline_size(&self, wm: WritingMode)
        -> longhands::inline_size::computed_value::T {
    
    
        
        if wm.is_vertical() {
            
            self.clone_height()
        
        } else {
            
            self.clone_width()
        
        }

    }

                    
    /// Set the appropriate physical property for min-inline-size given a writing mode.
    pub fn set_min_inline_size(&mut self,
                                      v: longhands::min_inline_size::computed_value::T,
                                      wm: WritingMode) {
        
    
        
        if wm.is_vertical() {
            
                self.set_min_height(v)
            
        } else {
            
                self.set_min_width(v)
            
        }

    }

    /// Copy the appropriate physical property from another struct for min-inline-size
    /// given a writing mode.
    pub fn copy_min_inline_size_from(&mut self,
                                            other: &Self,
                                            wm: WritingMode) {
        
    
        
        if wm.is_vertical() {
            
                self.copy_min_height_from(other)
            
        } else {
            
                self.copy_min_width_from(other)
            
        }

    }

    /// Copy the appropriate physical property from another struct for min-inline-size
    /// given a writing mode.
    pub fn reset_min_inline_size(&mut self,
                                        other: &Self,
                                        wm: WritingMode) {
        self.copy_min_inline_size_from(other, wm)
    }

    /// Get the computed value for the appropriate physical property for
    /// min-inline-size given a writing mode.
    pub fn clone_min_inline_size(&self, wm: WritingMode)
        -> longhands::min_inline_size::computed_value::T {
    
    
        
        if wm.is_vertical() {
            
            self.clone_min_height()
        
        } else {
            
            self.clone_min_width()
        
        }

    }

                    
    /// Set the appropriate physical property for max-inline-size given a writing mode.
    pub fn set_max_inline_size(&mut self,
                                      v: longhands::max_inline_size::computed_value::T,
                                      wm: WritingMode) {
        
    
        
        if wm.is_vertical() {
            
                self.set_max_height(v)
            
        } else {
            
                self.set_max_width(v)
            
        }

    }

    /// Copy the appropriate physical property from another struct for max-inline-size
    /// given a writing mode.
    pub fn copy_max_inline_size_from(&mut self,
                                            other: &Self,
                                            wm: WritingMode) {
        
    
        
        if wm.is_vertical() {
            
                self.copy_max_height_from(other)
            
        } else {
            
                self.copy_max_width_from(other)
            
        }

    }

    /// Copy the appropriate physical property from another struct for max-inline-size
    /// given a writing mode.
    pub fn reset_max_inline_size(&mut self,
                                        other: &Self,
                                        wm: WritingMode) {
        self.copy_max_inline_size_from(other, wm)
    }

    /// Get the computed value for the appropriate physical property for
    /// max-inline-size given a writing mode.
    pub fn clone_max_inline_size(&self, wm: WritingMode)
        -> longhands::max_inline_size::computed_value::T {
    
    
        
        if wm.is_vertical() {
            
            self.clone_max_height()
        
        } else {
            
            self.clone_max_width()
        
        }

    }

                        /// Set box-sizing.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_box_sizing(&mut self, v: longhands::box_sizing::computed_value::T) {
                            self.box_sizing = v;
                        }
                    /// Set box-sizing from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_box_sizing_from(&mut self, other: &Self) {
                        self.box_sizing = other.box_sizing.clone();
                    }

                    /// Reset box-sizing from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_box_sizing(&mut self, other: &Self) {
                        self.copy_box_sizing_from(other)
                    }

                    /// Get the computed value for box-sizing.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_box_sizing(&self) -> longhands::box_sizing::computed_value::T {
                        self.box_sizing.clone()
                    }
                        /// Set object-fit.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_object_fit(&mut self, v: longhands::object_fit::computed_value::T) {
                            self.object_fit = v;
                        }
                    /// Set object-fit from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_object_fit_from(&mut self, other: &Self) {
                        self.object_fit = other.object_fit.clone();
                    }

                    /// Reset object-fit from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_object_fit(&mut self, other: &Self) {
                        self.copy_object_fit_from(other)
                    }

                    /// Get the computed value for object-fit.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_object_fit(&self) -> longhands::object_fit::computed_value::T {
                        self.object_fit.clone()
                    }
                        /// Set object-position.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_object_position(&mut self, v: longhands::object_position::computed_value::T) {
                            self.object_position = v;
                        }
                    /// Set object-position from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_object_position_from(&mut self, other: &Self) {
                        self.object_position = other.object_position.clone();
                    }

                    /// Reset object-position from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_object_position(&mut self, other: &Self) {
                        self.copy_object_position_from(other)
                    }

                    /// Get the computed value for object-position.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_object_position(&self) -> longhands::object_position::computed_value::T {
                        self.object_position.clone()
                    }
                        /// Set grid-row-gap.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_grid_row_gap(&mut self, v: longhands::grid_row_gap::computed_value::T) {
                            self.grid_row_gap = v;
                        }
                    /// Set grid-row-gap from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_grid_row_gap_from(&mut self, other: &Self) {
                        self.grid_row_gap = other.grid_row_gap.clone();
                    }

                    /// Reset grid-row-gap from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_grid_row_gap(&mut self, other: &Self) {
                        self.copy_grid_row_gap_from(other)
                    }

                    /// Get the computed value for grid-row-gap.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_grid_row_gap(&self) -> longhands::grid_row_gap::computed_value::T {
                        self.grid_row_gap.clone()
                    }
                        /// Set grid-row-start.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_grid_row_start(&mut self, v: longhands::grid_row_start::computed_value::T) {
                            self.grid_row_start = v;
                        }
                    /// Set grid-row-start from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_grid_row_start_from(&mut self, other: &Self) {
                        self.grid_row_start = other.grid_row_start.clone();
                    }

                    /// Reset grid-row-start from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_grid_row_start(&mut self, other: &Self) {
                        self.copy_grid_row_start_from(other)
                    }

                    /// Get the computed value for grid-row-start.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_grid_row_start(&self) -> longhands::grid_row_start::computed_value::T {
                        self.grid_row_start.clone()
                    }
                        /// Set grid-row-end.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_grid_row_end(&mut self, v: longhands::grid_row_end::computed_value::T) {
                            self.grid_row_end = v;
                        }
                    /// Set grid-row-end from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_grid_row_end_from(&mut self, other: &Self) {
                        self.grid_row_end = other.grid_row_end.clone();
                    }

                    /// Reset grid-row-end from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_grid_row_end(&mut self, other: &Self) {
                        self.copy_grid_row_end_from(other)
                    }

                    /// Get the computed value for grid-row-end.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_grid_row_end(&self) -> longhands::grid_row_end::computed_value::T {
                        self.grid_row_end.clone()
                    }
                        /// Set grid-auto-rows.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_grid_auto_rows(&mut self, v: longhands::grid_auto_rows::computed_value::T) {
                            self.grid_auto_rows = v;
                        }
                    /// Set grid-auto-rows from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_grid_auto_rows_from(&mut self, other: &Self) {
                        self.grid_auto_rows = other.grid_auto_rows.clone();
                    }

                    /// Reset grid-auto-rows from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_grid_auto_rows(&mut self, other: &Self) {
                        self.copy_grid_auto_rows_from(other)
                    }

                    /// Get the computed value for grid-auto-rows.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_grid_auto_rows(&self) -> longhands::grid_auto_rows::computed_value::T {
                        self.grid_auto_rows.clone()
                    }
                        /// Set grid-template-rows.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_grid_template_rows(&mut self, v: longhands::grid_template_rows::computed_value::T) {
                            self.grid_template_rows = v;
                        }
                    /// Set grid-template-rows from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_grid_template_rows_from(&mut self, other: &Self) {
                        self.grid_template_rows = other.grid_template_rows.clone();
                    }

                    /// Reset grid-template-rows from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_grid_template_rows(&mut self, other: &Self) {
                        self.copy_grid_template_rows_from(other)
                    }

                    /// Get the computed value for grid-template-rows.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_grid_template_rows(&self) -> longhands::grid_template_rows::computed_value::T {
                        self.grid_template_rows.clone()
                    }
                        /// Set grid-column-gap.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_grid_column_gap(&mut self, v: longhands::grid_column_gap::computed_value::T) {
                            self.grid_column_gap = v;
                        }
                    /// Set grid-column-gap from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_grid_column_gap_from(&mut self, other: &Self) {
                        self.grid_column_gap = other.grid_column_gap.clone();
                    }

                    /// Reset grid-column-gap from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_grid_column_gap(&mut self, other: &Self) {
                        self.copy_grid_column_gap_from(other)
                    }

                    /// Get the computed value for grid-column-gap.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_grid_column_gap(&self) -> longhands::grid_column_gap::computed_value::T {
                        self.grid_column_gap.clone()
                    }
                        /// Set grid-column-start.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_grid_column_start(&mut self, v: longhands::grid_column_start::computed_value::T) {
                            self.grid_column_start = v;
                        }
                    /// Set grid-column-start from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_grid_column_start_from(&mut self, other: &Self) {
                        self.grid_column_start = other.grid_column_start.clone();
                    }

                    /// Reset grid-column-start from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_grid_column_start(&mut self, other: &Self) {
                        self.copy_grid_column_start_from(other)
                    }

                    /// Get the computed value for grid-column-start.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_grid_column_start(&self) -> longhands::grid_column_start::computed_value::T {
                        self.grid_column_start.clone()
                    }
                        /// Set grid-column-end.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_grid_column_end(&mut self, v: longhands::grid_column_end::computed_value::T) {
                            self.grid_column_end = v;
                        }
                    /// Set grid-column-end from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_grid_column_end_from(&mut self, other: &Self) {
                        self.grid_column_end = other.grid_column_end.clone();
                    }

                    /// Reset grid-column-end from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_grid_column_end(&mut self, other: &Self) {
                        self.copy_grid_column_end_from(other)
                    }

                    /// Get the computed value for grid-column-end.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_grid_column_end(&self) -> longhands::grid_column_end::computed_value::T {
                        self.grid_column_end.clone()
                    }
                        /// Set grid-auto-columns.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_grid_auto_columns(&mut self, v: longhands::grid_auto_columns::computed_value::T) {
                            self.grid_auto_columns = v;
                        }
                    /// Set grid-auto-columns from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_grid_auto_columns_from(&mut self, other: &Self) {
                        self.grid_auto_columns = other.grid_auto_columns.clone();
                    }

                    /// Reset grid-auto-columns from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_grid_auto_columns(&mut self, other: &Self) {
                        self.copy_grid_auto_columns_from(other)
                    }

                    /// Get the computed value for grid-auto-columns.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_grid_auto_columns(&self) -> longhands::grid_auto_columns::computed_value::T {
                        self.grid_auto_columns.clone()
                    }
                        /// Set grid-template-columns.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_grid_template_columns(&mut self, v: longhands::grid_template_columns::computed_value::T) {
                            self.grid_template_columns = v;
                        }
                    /// Set grid-template-columns from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_grid_template_columns_from(&mut self, other: &Self) {
                        self.grid_template_columns = other.grid_template_columns.clone();
                    }

                    /// Reset grid-template-columns from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_grid_template_columns(&mut self, other: &Self) {
                        self.copy_grid_template_columns_from(other)
                    }

                    /// Get the computed value for grid-template-columns.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_grid_template_columns(&self) -> longhands::grid_template_columns::computed_value::T {
                        self.grid_template_columns.clone()
                    }
                        /// Set grid-auto-flow.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_grid_auto_flow(&mut self, v: longhands::grid_auto_flow::computed_value::T) {
                            self.grid_auto_flow = v;
                        }
                    /// Set grid-auto-flow from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_grid_auto_flow_from(&mut self, other: &Self) {
                        self.grid_auto_flow = other.grid_auto_flow.clone();
                    }

                    /// Reset grid-auto-flow from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_grid_auto_flow(&mut self, other: &Self) {
                        self.copy_grid_auto_flow_from(other)
                    }

                    /// Get the computed value for grid-auto-flow.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_grid_auto_flow(&self) -> longhands::grid_auto_flow::computed_value::T {
                        self.grid_auto_flow.clone()
                    }
                        /// Set grid-template-areas.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_grid_template_areas(&mut self, v: longhands::grid_template_areas::computed_value::T) {
                            self.grid_template_areas = v;
                        }
                    /// Set grid-template-areas from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_grid_template_areas_from(&mut self, other: &Self) {
                        self.grid_template_areas = other.grid_template_areas.clone();
                    }

                    /// Reset grid-template-areas from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_grid_template_areas(&mut self, other: &Self) {
                        self.copy_grid_template_areas_from(other)
                    }

                    /// Get the computed value for grid-template-areas.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_grid_template_areas(&self) -> longhands::grid_template_areas::computed_value::T {
                        self.grid_template_areas.clone()
                    }
        }

        #[derive(Clone, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        /// The Table style struct.
        pub struct Table {
                /// The table-layout computed value.
                pub table_layout: longhands::table_layout::computed_value::T,
                /// The -x-span computed value.
                pub _x_span: longhands::_x_span::computed_value::T,
        }

        impl Table {
                        /// Set table-layout.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_table_layout(&mut self, v: longhands::table_layout::computed_value::T) {
                            self.table_layout = v;
                        }
                    /// Set table-layout from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_table_layout_from(&mut self, other: &Self) {
                        self.table_layout = other.table_layout.clone();
                    }

                    /// Reset table-layout from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_table_layout(&mut self, other: &Self) {
                        self.copy_table_layout_from(other)
                    }

                    /// Get the computed value for table-layout.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_table_layout(&self) -> longhands::table_layout::computed_value::T {
                        self.table_layout.clone()
                    }
                        /// Set -x-span.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set__x_span(&mut self, v: longhands::_x_span::computed_value::T) {
                            self._x_span = v;
                        }
                    /// Set -x-span from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy__x_span_from(&mut self, other: &Self) {
                        self._x_span = other._x_span.clone();
                    }

                    /// Reset -x-span from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset__x_span(&mut self, other: &Self) {
                        self.copy__x_span_from(other)
                    }

                    /// Get the computed value for -x-span.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone__x_span(&self) -> longhands::_x_span::computed_value::T {
                        self._x_span.clone()
                    }
        }

        #[derive(Clone, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        /// The Text style struct.
        pub struct Text {
                /// The text-overflow computed value.
                pub text_overflow: longhands::text_overflow::computed_value::T,
                /// The unicode-bidi computed value.
                pub unicode_bidi: longhands::unicode_bidi::computed_value::T,
                /// The text-decoration-line computed value.
                pub text_decoration_line: longhands::text_decoration_line::computed_value::T,
                /// The text-decoration-style computed value.
                pub text_decoration_style: longhands::text_decoration_style::computed_value::T,
                /// The text-decoration-color computed value.
                pub text_decoration_color: longhands::text_decoration_color::computed_value::T,
                /// The initial-letter computed value.
                pub initial_letter: longhands::initial_letter::computed_value::T,
        }

        impl Text {
                        /// Set text-overflow.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_text_overflow(&mut self, v: longhands::text_overflow::computed_value::T) {
                            self.text_overflow = v;
                        }
                    /// Set text-overflow from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_text_overflow_from(&mut self, other: &Self) {
                        self.text_overflow = other.text_overflow.clone();
                    }

                    /// Reset text-overflow from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_text_overflow(&mut self, other: &Self) {
                        self.copy_text_overflow_from(other)
                    }

                    /// Get the computed value for text-overflow.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_text_overflow(&self) -> longhands::text_overflow::computed_value::T {
                        self.text_overflow.clone()
                    }
                        /// Set unicode-bidi.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_unicode_bidi(&mut self, v: longhands::unicode_bidi::computed_value::T) {
                            self.unicode_bidi = v;
                        }
                    /// Set unicode-bidi from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_unicode_bidi_from(&mut self, other: &Self) {
                        self.unicode_bidi = other.unicode_bidi.clone();
                    }

                    /// Reset unicode-bidi from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_unicode_bidi(&mut self, other: &Self) {
                        self.copy_unicode_bidi_from(other)
                    }

                    /// Get the computed value for unicode-bidi.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_unicode_bidi(&self) -> longhands::unicode_bidi::computed_value::T {
                        self.unicode_bidi.clone()
                    }
                        /// Set text-decoration-line.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_text_decoration_line(&mut self, v: longhands::text_decoration_line::computed_value::T) {
                            self.text_decoration_line = v;
                        }
                    /// Set text-decoration-line from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_text_decoration_line_from(&mut self, other: &Self) {
                        self.text_decoration_line = other.text_decoration_line.clone();
                    }

                    /// Reset text-decoration-line from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_text_decoration_line(&mut self, other: &Self) {
                        self.copy_text_decoration_line_from(other)
                    }

                    /// Get the computed value for text-decoration-line.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_text_decoration_line(&self) -> longhands::text_decoration_line::computed_value::T {
                        self.text_decoration_line.clone()
                    }
                        /// Set text-decoration-style.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_text_decoration_style(&mut self, v: longhands::text_decoration_style::computed_value::T) {
                            self.text_decoration_style = v;
                        }
                    /// Set text-decoration-style from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_text_decoration_style_from(&mut self, other: &Self) {
                        self.text_decoration_style = other.text_decoration_style.clone();
                    }

                    /// Reset text-decoration-style from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_text_decoration_style(&mut self, other: &Self) {
                        self.copy_text_decoration_style_from(other)
                    }

                    /// Get the computed value for text-decoration-style.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_text_decoration_style(&self) -> longhands::text_decoration_style::computed_value::T {
                        self.text_decoration_style.clone()
                    }
                        /// Set text-decoration-color.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_text_decoration_color(&mut self, v: longhands::text_decoration_color::computed_value::T) {
                            self.text_decoration_color = v;
                        }
                    /// Set text-decoration-color from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_text_decoration_color_from(&mut self, other: &Self) {
                        self.text_decoration_color = other.text_decoration_color.clone();
                    }

                    /// Reset text-decoration-color from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_text_decoration_color(&mut self, other: &Self) {
                        self.copy_text_decoration_color_from(other)
                    }

                    /// Get the computed value for text-decoration-color.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_text_decoration_color(&self) -> longhands::text_decoration_color::computed_value::T {
                        self.text_decoration_color.clone()
                    }
                        /// Set initial-letter.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_initial_letter(&mut self, v: longhands::initial_letter::computed_value::T) {
                            self.initial_letter = v;
                        }
                    /// Set initial-letter from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_initial_letter_from(&mut self, other: &Self) {
                        self.initial_letter = other.initial_letter.clone();
                    }

                    /// Reset initial-letter from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_initial_letter(&mut self, other: &Self) {
                        self.copy_initial_letter_from(other)
                    }

                    /// Get the computed value for initial-letter.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_initial_letter(&self) -> longhands::initial_letter::computed_value::T {
                        self.initial_letter.clone()
                    }
                /// Whether the text decoration has an underline.
                #[inline]
                pub fn has_underline(&self) -> bool {
                    self.text_decoration_line.contains(longhands::text_decoration_line::UNDERLINE)
                }

                /// Whether the text decoration has an overline.
                #[inline]
                pub fn has_overline(&self) -> bool {
                    self.text_decoration_line.contains(longhands::text_decoration_line::OVERLINE)
                }

                /// Whether the text decoration has a line through.
                #[inline]
                pub fn has_line_through(&self) -> bool {
                    self.text_decoration_line.contains(longhands::text_decoration_line::LINE_THROUGH)
                }
        }

        #[derive(Clone, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        /// The UI style struct.
        pub struct UI {
                /// The ime-mode computed value.
                pub ime_mode: longhands::ime_mode::computed_value::T,
                /// The -moz-user-select computed value.
                pub _moz_user_select: longhands::_moz_user_select::computed_value::T,
                /// The -moz-window-dragging computed value.
                pub _moz_window_dragging: longhands::_moz_window_dragging::computed_value::T,
                /// The -moz-window-shadow computed value.
                pub _moz_window_shadow: longhands::_moz_window_shadow::computed_value::T,
                /// The -moz-force-broken-image-icon computed value.
                pub _moz_force_broken_image_icon: longhands::_moz_force_broken_image_icon::computed_value::T,
        }

        impl UI {
                        /// Set ime-mode.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_ime_mode(&mut self, v: longhands::ime_mode::computed_value::T) {
                            self.ime_mode = v;
                        }
                    /// Set ime-mode from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_ime_mode_from(&mut self, other: &Self) {
                        self.ime_mode = other.ime_mode.clone();
                    }

                    /// Reset ime-mode from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_ime_mode(&mut self, other: &Self) {
                        self.copy_ime_mode_from(other)
                    }

                    /// Get the computed value for ime-mode.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_ime_mode(&self) -> longhands::ime_mode::computed_value::T {
                        self.ime_mode.clone()
                    }
                        /// Set -moz-user-select.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set__moz_user_select(&mut self, v: longhands::_moz_user_select::computed_value::T) {
                            self._moz_user_select = v;
                        }
                    /// Set -moz-user-select from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy__moz_user_select_from(&mut self, other: &Self) {
                        self._moz_user_select = other._moz_user_select.clone();
                    }

                    /// Reset -moz-user-select from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset__moz_user_select(&mut self, other: &Self) {
                        self.copy__moz_user_select_from(other)
                    }

                    /// Get the computed value for -moz-user-select.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone__moz_user_select(&self) -> longhands::_moz_user_select::computed_value::T {
                        self._moz_user_select.clone()
                    }
                        /// Set -moz-window-dragging.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set__moz_window_dragging(&mut self, v: longhands::_moz_window_dragging::computed_value::T) {
                            self._moz_window_dragging = v;
                        }
                    /// Set -moz-window-dragging from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy__moz_window_dragging_from(&mut self, other: &Self) {
                        self._moz_window_dragging = other._moz_window_dragging.clone();
                    }

                    /// Reset -moz-window-dragging from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset__moz_window_dragging(&mut self, other: &Self) {
                        self.copy__moz_window_dragging_from(other)
                    }

                    /// Get the computed value for -moz-window-dragging.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone__moz_window_dragging(&self) -> longhands::_moz_window_dragging::computed_value::T {
                        self._moz_window_dragging.clone()
                    }
                        /// Set -moz-window-shadow.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set__moz_window_shadow(&mut self, v: longhands::_moz_window_shadow::computed_value::T) {
                            self._moz_window_shadow = v;
                        }
                    /// Set -moz-window-shadow from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy__moz_window_shadow_from(&mut self, other: &Self) {
                        self._moz_window_shadow = other._moz_window_shadow.clone();
                    }

                    /// Reset -moz-window-shadow from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset__moz_window_shadow(&mut self, other: &Self) {
                        self.copy__moz_window_shadow_from(other)
                    }

                    /// Get the computed value for -moz-window-shadow.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone__moz_window_shadow(&self) -> longhands::_moz_window_shadow::computed_value::T {
                        self._moz_window_shadow.clone()
                    }
                        /// Set -moz-force-broken-image-icon.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set__moz_force_broken_image_icon(&mut self, v: longhands::_moz_force_broken_image_icon::computed_value::T) {
                            self._moz_force_broken_image_icon = v;
                        }
                    /// Set -moz-force-broken-image-icon from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy__moz_force_broken_image_icon_from(&mut self, other: &Self) {
                        self._moz_force_broken_image_icon = other._moz_force_broken_image_icon.clone();
                    }

                    /// Reset -moz-force-broken-image-icon from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset__moz_force_broken_image_icon(&mut self, other: &Self) {
                        self.copy__moz_force_broken_image_icon_from(other)
                    }

                    /// Get the computed value for -moz-force-broken-image-icon.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone__moz_force_broken_image_icon(&self) -> longhands::_moz_force_broken_image_icon::computed_value::T {
                        self._moz_force_broken_image_icon.clone()
                    }
        }

        #[derive(Clone, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        /// The InheritedSVG style struct.
        pub struct InheritedSVG {
                /// The text-anchor computed value.
                pub text_anchor: longhands::text_anchor::computed_value::T,
                /// The color-interpolation computed value.
                pub color_interpolation: longhands::color_interpolation::computed_value::T,
                /// The color-interpolation-filters computed value.
                pub color_interpolation_filters: longhands::color_interpolation_filters::computed_value::T,
                /// The fill computed value.
                pub fill: longhands::fill::computed_value::T,
                /// The fill-opacity computed value.
                pub fill_opacity: longhands::fill_opacity::computed_value::T,
                /// The fill-rule computed value.
                pub fill_rule: longhands::fill_rule::computed_value::T,
                /// The shape-rendering computed value.
                pub shape_rendering: longhands::shape_rendering::computed_value::T,
                /// The stroke computed value.
                pub stroke: longhands::stroke::computed_value::T,
                /// The stroke-width computed value.
                pub stroke_width: longhands::stroke_width::computed_value::T,
                /// The stroke-linecap computed value.
                pub stroke_linecap: longhands::stroke_linecap::computed_value::T,
                /// The stroke-linejoin computed value.
                pub stroke_linejoin: longhands::stroke_linejoin::computed_value::T,
                /// The stroke-miterlimit computed value.
                pub stroke_miterlimit: longhands::stroke_miterlimit::computed_value::T,
                /// The stroke-opacity computed value.
                pub stroke_opacity: longhands::stroke_opacity::computed_value::T,
                /// The stroke-dasharray computed value.
                pub stroke_dasharray: longhands::stroke_dasharray::computed_value::T,
                /// The stroke-dashoffset computed value.
                pub stroke_dashoffset: longhands::stroke_dashoffset::computed_value::T,
                /// The clip-rule computed value.
                pub clip_rule: longhands::clip_rule::computed_value::T,
                /// The marker-start computed value.
                pub marker_start: longhands::marker_start::computed_value::T,
                /// The marker-mid computed value.
                pub marker_mid: longhands::marker_mid::computed_value::T,
                /// The marker-end computed value.
                pub marker_end: longhands::marker_end::computed_value::T,
                /// The paint-order computed value.
                pub paint_order: longhands::paint_order::computed_value::T,
                /// The -moz-context-properties computed value.
                pub _moz_context_properties: longhands::_moz_context_properties::computed_value::T,
        }

        impl InheritedSVG {
                        /// Set text-anchor.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_text_anchor(&mut self, v: longhands::text_anchor::computed_value::T) {
                            self.text_anchor = v;
                        }
                    /// Set text-anchor from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_text_anchor_from(&mut self, other: &Self) {
                        self.text_anchor = other.text_anchor.clone();
                    }

                    /// Reset text-anchor from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_text_anchor(&mut self, other: &Self) {
                        self.copy_text_anchor_from(other)
                    }

                    /// Get the computed value for text-anchor.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_text_anchor(&self) -> longhands::text_anchor::computed_value::T {
                        self.text_anchor.clone()
                    }
                        /// Set color-interpolation.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_color_interpolation(&mut self, v: longhands::color_interpolation::computed_value::T) {
                            self.color_interpolation = v;
                        }
                    /// Set color-interpolation from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_color_interpolation_from(&mut self, other: &Self) {
                        self.color_interpolation = other.color_interpolation.clone();
                    }

                    /// Reset color-interpolation from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_color_interpolation(&mut self, other: &Self) {
                        self.copy_color_interpolation_from(other)
                    }

                    /// Get the computed value for color-interpolation.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_color_interpolation(&self) -> longhands::color_interpolation::computed_value::T {
                        self.color_interpolation.clone()
                    }
                        /// Set color-interpolation-filters.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_color_interpolation_filters(&mut self, v: longhands::color_interpolation_filters::computed_value::T) {
                            self.color_interpolation_filters = v;
                        }
                    /// Set color-interpolation-filters from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_color_interpolation_filters_from(&mut self, other: &Self) {
                        self.color_interpolation_filters = other.color_interpolation_filters.clone();
                    }

                    /// Reset color-interpolation-filters from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_color_interpolation_filters(&mut self, other: &Self) {
                        self.copy_color_interpolation_filters_from(other)
                    }

                    /// Get the computed value for color-interpolation-filters.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_color_interpolation_filters(&self) -> longhands::color_interpolation_filters::computed_value::T {
                        self.color_interpolation_filters.clone()
                    }
                        /// Set fill.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_fill(&mut self, v: longhands::fill::computed_value::T) {
                            self.fill = v;
                        }
                    /// Set fill from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_fill_from(&mut self, other: &Self) {
                        self.fill = other.fill.clone();
                    }

                    /// Reset fill from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_fill(&mut self, other: &Self) {
                        self.copy_fill_from(other)
                    }

                    /// Get the computed value for fill.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_fill(&self) -> longhands::fill::computed_value::T {
                        self.fill.clone()
                    }
                        /// Set fill-opacity.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_fill_opacity(&mut self, v: longhands::fill_opacity::computed_value::T) {
                            self.fill_opacity = v;
                        }
                    /// Set fill-opacity from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_fill_opacity_from(&mut self, other: &Self) {
                        self.fill_opacity = other.fill_opacity.clone();
                    }

                    /// Reset fill-opacity from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_fill_opacity(&mut self, other: &Self) {
                        self.copy_fill_opacity_from(other)
                    }

                    /// Get the computed value for fill-opacity.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_fill_opacity(&self) -> longhands::fill_opacity::computed_value::T {
                        self.fill_opacity.clone()
                    }
                        /// Set fill-rule.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_fill_rule(&mut self, v: longhands::fill_rule::computed_value::T) {
                            self.fill_rule = v;
                        }
                    /// Set fill-rule from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_fill_rule_from(&mut self, other: &Self) {
                        self.fill_rule = other.fill_rule.clone();
                    }

                    /// Reset fill-rule from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_fill_rule(&mut self, other: &Self) {
                        self.copy_fill_rule_from(other)
                    }

                    /// Get the computed value for fill-rule.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_fill_rule(&self) -> longhands::fill_rule::computed_value::T {
                        self.fill_rule.clone()
                    }
                        /// Set shape-rendering.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_shape_rendering(&mut self, v: longhands::shape_rendering::computed_value::T) {
                            self.shape_rendering = v;
                        }
                    /// Set shape-rendering from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_shape_rendering_from(&mut self, other: &Self) {
                        self.shape_rendering = other.shape_rendering.clone();
                    }

                    /// Reset shape-rendering from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_shape_rendering(&mut self, other: &Self) {
                        self.copy_shape_rendering_from(other)
                    }

                    /// Get the computed value for shape-rendering.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_shape_rendering(&self) -> longhands::shape_rendering::computed_value::T {
                        self.shape_rendering.clone()
                    }
                        /// Set stroke.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_stroke(&mut self, v: longhands::stroke::computed_value::T) {
                            self.stroke = v;
                        }
                    /// Set stroke from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_stroke_from(&mut self, other: &Self) {
                        self.stroke = other.stroke.clone();
                    }

                    /// Reset stroke from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_stroke(&mut self, other: &Self) {
                        self.copy_stroke_from(other)
                    }

                    /// Get the computed value for stroke.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_stroke(&self) -> longhands::stroke::computed_value::T {
                        self.stroke.clone()
                    }
                        /// Set stroke-width.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_stroke_width(&mut self, v: longhands::stroke_width::computed_value::T) {
                            self.stroke_width = v;
                        }
                    /// Set stroke-width from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_stroke_width_from(&mut self, other: &Self) {
                        self.stroke_width = other.stroke_width.clone();
                    }

                    /// Reset stroke-width from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_stroke_width(&mut self, other: &Self) {
                        self.copy_stroke_width_from(other)
                    }

                    /// Get the computed value for stroke-width.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_stroke_width(&self) -> longhands::stroke_width::computed_value::T {
                        self.stroke_width.clone()
                    }
                        /// Set stroke-linecap.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_stroke_linecap(&mut self, v: longhands::stroke_linecap::computed_value::T) {
                            self.stroke_linecap = v;
                        }
                    /// Set stroke-linecap from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_stroke_linecap_from(&mut self, other: &Self) {
                        self.stroke_linecap = other.stroke_linecap.clone();
                    }

                    /// Reset stroke-linecap from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_stroke_linecap(&mut self, other: &Self) {
                        self.copy_stroke_linecap_from(other)
                    }

                    /// Get the computed value for stroke-linecap.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_stroke_linecap(&self) -> longhands::stroke_linecap::computed_value::T {
                        self.stroke_linecap.clone()
                    }
                        /// Set stroke-linejoin.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_stroke_linejoin(&mut self, v: longhands::stroke_linejoin::computed_value::T) {
                            self.stroke_linejoin = v;
                        }
                    /// Set stroke-linejoin from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_stroke_linejoin_from(&mut self, other: &Self) {
                        self.stroke_linejoin = other.stroke_linejoin.clone();
                    }

                    /// Reset stroke-linejoin from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_stroke_linejoin(&mut self, other: &Self) {
                        self.copy_stroke_linejoin_from(other)
                    }

                    /// Get the computed value for stroke-linejoin.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_stroke_linejoin(&self) -> longhands::stroke_linejoin::computed_value::T {
                        self.stroke_linejoin.clone()
                    }
                        /// Set stroke-miterlimit.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_stroke_miterlimit(&mut self, v: longhands::stroke_miterlimit::computed_value::T) {
                            self.stroke_miterlimit = v;
                        }
                    /// Set stroke-miterlimit from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_stroke_miterlimit_from(&mut self, other: &Self) {
                        self.stroke_miterlimit = other.stroke_miterlimit.clone();
                    }

                    /// Reset stroke-miterlimit from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_stroke_miterlimit(&mut self, other: &Self) {
                        self.copy_stroke_miterlimit_from(other)
                    }

                    /// Get the computed value for stroke-miterlimit.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_stroke_miterlimit(&self) -> longhands::stroke_miterlimit::computed_value::T {
                        self.stroke_miterlimit.clone()
                    }
                        /// Set stroke-opacity.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_stroke_opacity(&mut self, v: longhands::stroke_opacity::computed_value::T) {
                            self.stroke_opacity = v;
                        }
                    /// Set stroke-opacity from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_stroke_opacity_from(&mut self, other: &Self) {
                        self.stroke_opacity = other.stroke_opacity.clone();
                    }

                    /// Reset stroke-opacity from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_stroke_opacity(&mut self, other: &Self) {
                        self.copy_stroke_opacity_from(other)
                    }

                    /// Get the computed value for stroke-opacity.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_stroke_opacity(&self) -> longhands::stroke_opacity::computed_value::T {
                        self.stroke_opacity.clone()
                    }
                        /// Set stroke-dasharray.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_stroke_dasharray(&mut self, v: longhands::stroke_dasharray::computed_value::T) {
                            self.stroke_dasharray = v;
                        }
                    /// Set stroke-dasharray from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_stroke_dasharray_from(&mut self, other: &Self) {
                        self.stroke_dasharray = other.stroke_dasharray.clone();
                    }

                    /// Reset stroke-dasharray from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_stroke_dasharray(&mut self, other: &Self) {
                        self.copy_stroke_dasharray_from(other)
                    }

                    /// Get the computed value for stroke-dasharray.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_stroke_dasharray(&self) -> longhands::stroke_dasharray::computed_value::T {
                        self.stroke_dasharray.clone()
                    }
                        /// Set stroke-dashoffset.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_stroke_dashoffset(&mut self, v: longhands::stroke_dashoffset::computed_value::T) {
                            self.stroke_dashoffset = v;
                        }
                    /// Set stroke-dashoffset from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_stroke_dashoffset_from(&mut self, other: &Self) {
                        self.stroke_dashoffset = other.stroke_dashoffset.clone();
                    }

                    /// Reset stroke-dashoffset from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_stroke_dashoffset(&mut self, other: &Self) {
                        self.copy_stroke_dashoffset_from(other)
                    }

                    /// Get the computed value for stroke-dashoffset.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_stroke_dashoffset(&self) -> longhands::stroke_dashoffset::computed_value::T {
                        self.stroke_dashoffset.clone()
                    }
                        /// Set clip-rule.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_clip_rule(&mut self, v: longhands::clip_rule::computed_value::T) {
                            self.clip_rule = v;
                        }
                    /// Set clip-rule from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_clip_rule_from(&mut self, other: &Self) {
                        self.clip_rule = other.clip_rule.clone();
                    }

                    /// Reset clip-rule from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_clip_rule(&mut self, other: &Self) {
                        self.copy_clip_rule_from(other)
                    }

                    /// Get the computed value for clip-rule.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_clip_rule(&self) -> longhands::clip_rule::computed_value::T {
                        self.clip_rule.clone()
                    }
                        /// Set marker-start.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_marker_start(&mut self, v: longhands::marker_start::computed_value::T) {
                            self.marker_start = v;
                        }
                    /// Set marker-start from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_marker_start_from(&mut self, other: &Self) {
                        self.marker_start = other.marker_start.clone();
                    }

                    /// Reset marker-start from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_marker_start(&mut self, other: &Self) {
                        self.copy_marker_start_from(other)
                    }

                    /// Get the computed value for marker-start.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_marker_start(&self) -> longhands::marker_start::computed_value::T {
                        self.marker_start.clone()
                    }
                        /// Set marker-mid.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_marker_mid(&mut self, v: longhands::marker_mid::computed_value::T) {
                            self.marker_mid = v;
                        }
                    /// Set marker-mid from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_marker_mid_from(&mut self, other: &Self) {
                        self.marker_mid = other.marker_mid.clone();
                    }

                    /// Reset marker-mid from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_marker_mid(&mut self, other: &Self) {
                        self.copy_marker_mid_from(other)
                    }

                    /// Get the computed value for marker-mid.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_marker_mid(&self) -> longhands::marker_mid::computed_value::T {
                        self.marker_mid.clone()
                    }
                        /// Set marker-end.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_marker_end(&mut self, v: longhands::marker_end::computed_value::T) {
                            self.marker_end = v;
                        }
                    /// Set marker-end from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_marker_end_from(&mut self, other: &Self) {
                        self.marker_end = other.marker_end.clone();
                    }

                    /// Reset marker-end from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_marker_end(&mut self, other: &Self) {
                        self.copy_marker_end_from(other)
                    }

                    /// Get the computed value for marker-end.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_marker_end(&self) -> longhands::marker_end::computed_value::T {
                        self.marker_end.clone()
                    }
                        /// Set paint-order.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_paint_order(&mut self, v: longhands::paint_order::computed_value::T) {
                            self.paint_order = v;
                        }
                    /// Set paint-order from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_paint_order_from(&mut self, other: &Self) {
                        self.paint_order = other.paint_order.clone();
                    }

                    /// Reset paint-order from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_paint_order(&mut self, other: &Self) {
                        self.copy_paint_order_from(other)
                    }

                    /// Get the computed value for paint-order.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_paint_order(&self) -> longhands::paint_order::computed_value::T {
                        self.paint_order.clone()
                    }
                        /// Set -moz-context-properties.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set__moz_context_properties<I>(&mut self, v: I)
                            where I: IntoIterator<Item = longhands::_moz_context_properties
                                                                  ::computed_value::single_value::T>,
                                  I::IntoIter: ExactSizeIterator
                        {
                            self._moz_context_properties = longhands::_moz_context_properties::computed_value
                                                              ::T(v.into_iter().collect());
                        }
                    /// Set -moz-context-properties from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy__moz_context_properties_from(&mut self, other: &Self) {
                        self._moz_context_properties = other._moz_context_properties.clone();
                    }

                    /// Reset -moz-context-properties from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset__moz_context_properties(&mut self, other: &Self) {
                        self.copy__moz_context_properties_from(other)
                    }

                    /// Get the computed value for -moz-context-properties.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone__moz_context_properties(&self) -> longhands::_moz_context_properties::computed_value::T {
                        self._moz_context_properties.clone()
                    }
        }

        #[derive(Clone, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        /// The SVG style struct.
        pub struct SVG {
                /// The dominant-baseline computed value.
                pub dominant_baseline: longhands::dominant_baseline::computed_value::T,
                /// The vector-effect computed value.
                pub vector_effect: longhands::vector_effect::computed_value::T,
                /// The stop-color computed value.
                pub stop_color: longhands::stop_color::computed_value::T,
                /// The stop-opacity computed value.
                pub stop_opacity: longhands::stop_opacity::computed_value::T,
                /// The flood-color computed value.
                pub flood_color: longhands::flood_color::computed_value::T,
                /// The flood-opacity computed value.
                pub flood_opacity: longhands::flood_opacity::computed_value::T,
                /// The lighting-color computed value.
                pub lighting_color: longhands::lighting_color::computed_value::T,
                /// The mask-type computed value.
                pub mask_type: longhands::mask_type::computed_value::T,
                /// The clip-path computed value.
                pub clip_path: longhands::clip_path::computed_value::T,
                /// The mask-mode computed value.
                pub mask_mode: longhands::mask_mode::computed_value::T,
                /// The mask-repeat computed value.
                pub mask_repeat: longhands::mask_repeat::computed_value::T,
                /// The mask-position-x computed value.
                pub mask_position_x: longhands::mask_position_x::computed_value::T,
                /// The mask-position-y computed value.
                pub mask_position_y: longhands::mask_position_y::computed_value::T,
                /// The mask-clip computed value.
                pub mask_clip: longhands::mask_clip::computed_value::T,
                /// The mask-origin computed value.
                pub mask_origin: longhands::mask_origin::computed_value::T,
                /// The mask-size computed value.
                pub mask_size: longhands::mask_size::computed_value::T,
                /// The mask-composite computed value.
                pub mask_composite: longhands::mask_composite::computed_value::T,
                /// The mask-image computed value.
                pub mask_image: longhands::mask_image::computed_value::T,
        }

        impl SVG {
                        /// Set dominant-baseline.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_dominant_baseline(&mut self, v: longhands::dominant_baseline::computed_value::T) {
                            self.dominant_baseline = v;
                        }
                    /// Set dominant-baseline from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_dominant_baseline_from(&mut self, other: &Self) {
                        self.dominant_baseline = other.dominant_baseline.clone();
                    }

                    /// Reset dominant-baseline from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_dominant_baseline(&mut self, other: &Self) {
                        self.copy_dominant_baseline_from(other)
                    }

                    /// Get the computed value for dominant-baseline.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_dominant_baseline(&self) -> longhands::dominant_baseline::computed_value::T {
                        self.dominant_baseline.clone()
                    }
                        /// Set vector-effect.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_vector_effect(&mut self, v: longhands::vector_effect::computed_value::T) {
                            self.vector_effect = v;
                        }
                    /// Set vector-effect from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_vector_effect_from(&mut self, other: &Self) {
                        self.vector_effect = other.vector_effect.clone();
                    }

                    /// Reset vector-effect from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_vector_effect(&mut self, other: &Self) {
                        self.copy_vector_effect_from(other)
                    }

                    /// Get the computed value for vector-effect.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_vector_effect(&self) -> longhands::vector_effect::computed_value::T {
                        self.vector_effect.clone()
                    }
                        /// Set stop-color.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_stop_color(&mut self, v: longhands::stop_color::computed_value::T) {
                            self.stop_color = v;
                        }
                    /// Set stop-color from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_stop_color_from(&mut self, other: &Self) {
                        self.stop_color = other.stop_color.clone();
                    }

                    /// Reset stop-color from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_stop_color(&mut self, other: &Self) {
                        self.copy_stop_color_from(other)
                    }

                    /// Get the computed value for stop-color.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_stop_color(&self) -> longhands::stop_color::computed_value::T {
                        self.stop_color.clone()
                    }
                        /// Set stop-opacity.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_stop_opacity(&mut self, v: longhands::stop_opacity::computed_value::T) {
                            self.stop_opacity = v;
                        }
                    /// Set stop-opacity from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_stop_opacity_from(&mut self, other: &Self) {
                        self.stop_opacity = other.stop_opacity.clone();
                    }

                    /// Reset stop-opacity from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_stop_opacity(&mut self, other: &Self) {
                        self.copy_stop_opacity_from(other)
                    }

                    /// Get the computed value for stop-opacity.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_stop_opacity(&self) -> longhands::stop_opacity::computed_value::T {
                        self.stop_opacity.clone()
                    }
                        /// Set flood-color.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_flood_color(&mut self, v: longhands::flood_color::computed_value::T) {
                            self.flood_color = v;
                        }
                    /// Set flood-color from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_flood_color_from(&mut self, other: &Self) {
                        self.flood_color = other.flood_color.clone();
                    }

                    /// Reset flood-color from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_flood_color(&mut self, other: &Self) {
                        self.copy_flood_color_from(other)
                    }

                    /// Get the computed value for flood-color.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_flood_color(&self) -> longhands::flood_color::computed_value::T {
                        self.flood_color.clone()
                    }
                        /// Set flood-opacity.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_flood_opacity(&mut self, v: longhands::flood_opacity::computed_value::T) {
                            self.flood_opacity = v;
                        }
                    /// Set flood-opacity from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_flood_opacity_from(&mut self, other: &Self) {
                        self.flood_opacity = other.flood_opacity.clone();
                    }

                    /// Reset flood-opacity from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_flood_opacity(&mut self, other: &Self) {
                        self.copy_flood_opacity_from(other)
                    }

                    /// Get the computed value for flood-opacity.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_flood_opacity(&self) -> longhands::flood_opacity::computed_value::T {
                        self.flood_opacity.clone()
                    }
                        /// Set lighting-color.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_lighting_color(&mut self, v: longhands::lighting_color::computed_value::T) {
                            self.lighting_color = v;
                        }
                    /// Set lighting-color from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_lighting_color_from(&mut self, other: &Self) {
                        self.lighting_color = other.lighting_color.clone();
                    }

                    /// Reset lighting-color from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_lighting_color(&mut self, other: &Self) {
                        self.copy_lighting_color_from(other)
                    }

                    /// Get the computed value for lighting-color.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_lighting_color(&self) -> longhands::lighting_color::computed_value::T {
                        self.lighting_color.clone()
                    }
                        /// Set mask-type.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_mask_type(&mut self, v: longhands::mask_type::computed_value::T) {
                            self.mask_type = v;
                        }
                    /// Set mask-type from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_mask_type_from(&mut self, other: &Self) {
                        self.mask_type = other.mask_type.clone();
                    }

                    /// Reset mask-type from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_mask_type(&mut self, other: &Self) {
                        self.copy_mask_type_from(other)
                    }

                    /// Get the computed value for mask-type.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_mask_type(&self) -> longhands::mask_type::computed_value::T {
                        self.mask_type.clone()
                    }
                        /// Set clip-path.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_clip_path(&mut self, v: longhands::clip_path::computed_value::T) {
                            self.clip_path = v;
                        }
                    /// Set clip-path from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_clip_path_from(&mut self, other: &Self) {
                        self.clip_path = other.clip_path.clone();
                    }

                    /// Reset clip-path from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_clip_path(&mut self, other: &Self) {
                        self.copy_clip_path_from(other)
                    }

                    /// Get the computed value for clip-path.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_clip_path(&self) -> longhands::clip_path::computed_value::T {
                        self.clip_path.clone()
                    }
                        /// Set mask-mode.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_mask_mode<I>(&mut self, v: I)
                            where I: IntoIterator<Item = longhands::mask_mode
                                                                  ::computed_value::single_value::T>,
                                  I::IntoIter: ExactSizeIterator
                        {
                            self.mask_mode = longhands::mask_mode::computed_value
                                                              ::T(v.into_iter().collect());
                        }
                    /// Set mask-mode from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_mask_mode_from(&mut self, other: &Self) {
                        self.mask_mode = other.mask_mode.clone();
                    }

                    /// Reset mask-mode from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_mask_mode(&mut self, other: &Self) {
                        self.copy_mask_mode_from(other)
                    }

                    /// Get the computed value for mask-mode.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_mask_mode(&self) -> longhands::mask_mode::computed_value::T {
                        self.mask_mode.clone()
                    }
                        /// Set mask-repeat.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_mask_repeat<I>(&mut self, v: I)
                            where I: IntoIterator<Item = longhands::mask_repeat
                                                                  ::computed_value::single_value::T>,
                                  I::IntoIter: ExactSizeIterator
                        {
                            self.mask_repeat = longhands::mask_repeat::computed_value
                                                              ::T(v.into_iter().collect());
                        }
                    /// Set mask-repeat from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_mask_repeat_from(&mut self, other: &Self) {
                        self.mask_repeat = other.mask_repeat.clone();
                    }

                    /// Reset mask-repeat from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_mask_repeat(&mut self, other: &Self) {
                        self.copy_mask_repeat_from(other)
                    }

                    /// Get the computed value for mask-repeat.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_mask_repeat(&self) -> longhands::mask_repeat::computed_value::T {
                        self.mask_repeat.clone()
                    }
                        /// Set mask-position-x.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_mask_position_x<I>(&mut self, v: I)
                            where I: IntoIterator<Item = longhands::mask_position_x
                                                                  ::computed_value::single_value::T>,
                                  I::IntoIter: ExactSizeIterator
                        {
                            self.mask_position_x = longhands::mask_position_x::computed_value
                                                              ::T(v.into_iter().collect());
                        }
                    /// Set mask-position-x from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_mask_position_x_from(&mut self, other: &Self) {
                        self.mask_position_x = other.mask_position_x.clone();
                    }

                    /// Reset mask-position-x from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_mask_position_x(&mut self, other: &Self) {
                        self.copy_mask_position_x_from(other)
                    }

                    /// Get the computed value for mask-position-x.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_mask_position_x(&self) -> longhands::mask_position_x::computed_value::T {
                        self.mask_position_x.clone()
                    }
                        /// Set mask-position-y.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_mask_position_y<I>(&mut self, v: I)
                            where I: IntoIterator<Item = longhands::mask_position_y
                                                                  ::computed_value::single_value::T>,
                                  I::IntoIter: ExactSizeIterator
                        {
                            self.mask_position_y = longhands::mask_position_y::computed_value
                                                              ::T(v.into_iter().collect());
                        }
                    /// Set mask-position-y from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_mask_position_y_from(&mut self, other: &Self) {
                        self.mask_position_y = other.mask_position_y.clone();
                    }

                    /// Reset mask-position-y from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_mask_position_y(&mut self, other: &Self) {
                        self.copy_mask_position_y_from(other)
                    }

                    /// Get the computed value for mask-position-y.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_mask_position_y(&self) -> longhands::mask_position_y::computed_value::T {
                        self.mask_position_y.clone()
                    }
                        /// Set mask-clip.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_mask_clip<I>(&mut self, v: I)
                            where I: IntoIterator<Item = longhands::mask_clip
                                                                  ::computed_value::single_value::T>,
                                  I::IntoIter: ExactSizeIterator
                        {
                            self.mask_clip = longhands::mask_clip::computed_value
                                                              ::T(v.into_iter().collect());
                        }
                    /// Set mask-clip from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_mask_clip_from(&mut self, other: &Self) {
                        self.mask_clip = other.mask_clip.clone();
                    }

                    /// Reset mask-clip from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_mask_clip(&mut self, other: &Self) {
                        self.copy_mask_clip_from(other)
                    }

                    /// Get the computed value for mask-clip.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_mask_clip(&self) -> longhands::mask_clip::computed_value::T {
                        self.mask_clip.clone()
                    }
                        /// Set mask-origin.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_mask_origin<I>(&mut self, v: I)
                            where I: IntoIterator<Item = longhands::mask_origin
                                                                  ::computed_value::single_value::T>,
                                  I::IntoIter: ExactSizeIterator
                        {
                            self.mask_origin = longhands::mask_origin::computed_value
                                                              ::T(v.into_iter().collect());
                        }
                    /// Set mask-origin from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_mask_origin_from(&mut self, other: &Self) {
                        self.mask_origin = other.mask_origin.clone();
                    }

                    /// Reset mask-origin from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_mask_origin(&mut self, other: &Self) {
                        self.copy_mask_origin_from(other)
                    }

                    /// Get the computed value for mask-origin.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_mask_origin(&self) -> longhands::mask_origin::computed_value::T {
                        self.mask_origin.clone()
                    }
                        /// Set mask-size.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_mask_size(&mut self, v: longhands::mask_size::computed_value::T) {
                            self.mask_size = v;
                        }
                    /// Set mask-size from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_mask_size_from(&mut self, other: &Self) {
                        self.mask_size = other.mask_size.clone();
                    }

                    /// Reset mask-size from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_mask_size(&mut self, other: &Self) {
                        self.copy_mask_size_from(other)
                    }

                    /// Get the computed value for mask-size.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_mask_size(&self) -> longhands::mask_size::computed_value::T {
                        self.mask_size.clone()
                    }
                        /// Set mask-composite.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_mask_composite<I>(&mut self, v: I)
                            where I: IntoIterator<Item = longhands::mask_composite
                                                                  ::computed_value::single_value::T>,
                                  I::IntoIter: ExactSizeIterator
                        {
                            self.mask_composite = longhands::mask_composite::computed_value
                                                              ::T(v.into_iter().collect());
                        }
                    /// Set mask-composite from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_mask_composite_from(&mut self, other: &Self) {
                        self.mask_composite = other.mask_composite.clone();
                    }

                    /// Reset mask-composite from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_mask_composite(&mut self, other: &Self) {
                        self.copy_mask_composite_from(other)
                    }

                    /// Get the computed value for mask-composite.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_mask_composite(&self) -> longhands::mask_composite::computed_value::T {
                        self.mask_composite.clone()
                    }
                        /// Set mask-image.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set_mask_image<I>(&mut self, v: I)
                            where I: IntoIterator<Item = longhands::mask_image
                                                                  ::computed_value::single_value::T>,
                                  I::IntoIter: ExactSizeIterator
                        {
                            self.mask_image = longhands::mask_image::computed_value
                                                              ::T(v.into_iter().collect());
                        }
                    /// Set mask-image from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy_mask_image_from(&mut self, other: &Self) {
                        self.mask_image = other.mask_image.clone();
                    }

                    /// Reset mask-image from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset_mask_image(&mut self, other: &Self) {
                        self.copy_mask_image_from(other)
                    }

                    /// Get the computed value for mask-image.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone_mask_image(&self) -> longhands::mask_image::computed_value::T {
                        self.mask_image.clone()
                    }
        }

        #[derive(Clone, Debug, PartialEq)]
        #[cfg_attr(feature = "gecko", derive(MallocSizeOf))]
        #[cfg_attr(feature = "servo", derive(HeapSizeOf))]
        /// The XUL style struct.
        pub struct XUL {
                /// The -moz-box-align computed value.
                pub _moz_box_align: longhands::_moz_box_align::computed_value::T,
                /// The -moz-box-direction computed value.
                pub _moz_box_direction: longhands::_moz_box_direction::computed_value::T,
                /// The -moz-box-flex computed value.
                pub _moz_box_flex: longhands::_moz_box_flex::computed_value::T,
                /// The -moz-box-orient computed value.
                pub _moz_box_orient: longhands::_moz_box_orient::computed_value::T,
                /// The -moz-box-pack computed value.
                pub _moz_box_pack: longhands::_moz_box_pack::computed_value::T,
                /// The -moz-stack-sizing computed value.
                pub _moz_stack_sizing: longhands::_moz_stack_sizing::computed_value::T,
                /// The -moz-box-ordinal-group computed value.
                pub _moz_box_ordinal_group: longhands::_moz_box_ordinal_group::computed_value::T,
        }

        impl XUL {
                        /// Set -moz-box-align.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set__moz_box_align(&mut self, v: longhands::_moz_box_align::computed_value::T) {
                            self._moz_box_align = v;
                        }
                    /// Set -moz-box-align from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy__moz_box_align_from(&mut self, other: &Self) {
                        self._moz_box_align = other._moz_box_align.clone();
                    }

                    /// Reset -moz-box-align from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset__moz_box_align(&mut self, other: &Self) {
                        self.copy__moz_box_align_from(other)
                    }

                    /// Get the computed value for -moz-box-align.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone__moz_box_align(&self) -> longhands::_moz_box_align::computed_value::T {
                        self._moz_box_align.clone()
                    }
                        /// Set -moz-box-direction.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set__moz_box_direction(&mut self, v: longhands::_moz_box_direction::computed_value::T) {
                            self._moz_box_direction = v;
                        }
                    /// Set -moz-box-direction from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy__moz_box_direction_from(&mut self, other: &Self) {
                        self._moz_box_direction = other._moz_box_direction.clone();
                    }

                    /// Reset -moz-box-direction from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset__moz_box_direction(&mut self, other: &Self) {
                        self.copy__moz_box_direction_from(other)
                    }

                    /// Get the computed value for -moz-box-direction.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone__moz_box_direction(&self) -> longhands::_moz_box_direction::computed_value::T {
                        self._moz_box_direction.clone()
                    }
                        /// Set -moz-box-flex.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set__moz_box_flex(&mut self, v: longhands::_moz_box_flex::computed_value::T) {
                            self._moz_box_flex = v;
                        }
                    /// Set -moz-box-flex from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy__moz_box_flex_from(&mut self, other: &Self) {
                        self._moz_box_flex = other._moz_box_flex.clone();
                    }

                    /// Reset -moz-box-flex from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset__moz_box_flex(&mut self, other: &Self) {
                        self.copy__moz_box_flex_from(other)
                    }

                    /// Get the computed value for -moz-box-flex.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone__moz_box_flex(&self) -> longhands::_moz_box_flex::computed_value::T {
                        self._moz_box_flex.clone()
                    }
                        /// Set -moz-box-orient.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set__moz_box_orient(&mut self, v: longhands::_moz_box_orient::computed_value::T) {
                            self._moz_box_orient = v;
                        }
                    /// Set -moz-box-orient from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy__moz_box_orient_from(&mut self, other: &Self) {
                        self._moz_box_orient = other._moz_box_orient.clone();
                    }

                    /// Reset -moz-box-orient from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset__moz_box_orient(&mut self, other: &Self) {
                        self.copy__moz_box_orient_from(other)
                    }

                    /// Get the computed value for -moz-box-orient.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone__moz_box_orient(&self) -> longhands::_moz_box_orient::computed_value::T {
                        self._moz_box_orient.clone()
                    }
                        /// Set -moz-box-pack.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set__moz_box_pack(&mut self, v: longhands::_moz_box_pack::computed_value::T) {
                            self._moz_box_pack = v;
                        }
                    /// Set -moz-box-pack from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy__moz_box_pack_from(&mut self, other: &Self) {
                        self._moz_box_pack = other._moz_box_pack.clone();
                    }

                    /// Reset -moz-box-pack from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset__moz_box_pack(&mut self, other: &Self) {
                        self.copy__moz_box_pack_from(other)
                    }

                    /// Get the computed value for -moz-box-pack.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone__moz_box_pack(&self) -> longhands::_moz_box_pack::computed_value::T {
                        self._moz_box_pack.clone()
                    }
                        /// Set -moz-stack-sizing.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set__moz_stack_sizing(&mut self, v: longhands::_moz_stack_sizing::computed_value::T) {
                            self._moz_stack_sizing = v;
                        }
                    /// Set -moz-stack-sizing from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy__moz_stack_sizing_from(&mut self, other: &Self) {
                        self._moz_stack_sizing = other._moz_stack_sizing.clone();
                    }

                    /// Reset -moz-stack-sizing from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset__moz_stack_sizing(&mut self, other: &Self) {
                        self.copy__moz_stack_sizing_from(other)
                    }

                    /// Get the computed value for -moz-stack-sizing.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone__moz_stack_sizing(&self) -> longhands::_moz_stack_sizing::computed_value::T {
                        self._moz_stack_sizing.clone()
                    }
                        /// Set -moz-box-ordinal-group.
                        #[allow(non_snake_case)]
                        #[inline]
                        pub fn set__moz_box_ordinal_group(&mut self, v: longhands::_moz_box_ordinal_group::computed_value::T) {
                            self._moz_box_ordinal_group = v;
                        }
                    /// Set -moz-box-ordinal-group from other struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn copy__moz_box_ordinal_group_from(&mut self, other: &Self) {
                        self._moz_box_ordinal_group = other._moz_box_ordinal_group.clone();
                    }

                    /// Reset -moz-box-ordinal-group from the initial struct.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn reset__moz_box_ordinal_group(&mut self, other: &Self) {
                        self.copy__moz_box_ordinal_group_from(other)
                    }

                    /// Get the computed value for -moz-box-ordinal-group.
                    #[allow(non_snake_case)]
                    #[inline]
                    pub fn clone__moz_box_ordinal_group(&self) -> longhands::_moz_box_ordinal_group::computed_value::T {
                        self._moz_box_ordinal_group.clone()
                    }
        }

}

    impl style_structs::Background {

    }

    impl style_structs::Border {

    }

    impl style_structs::Box {
                /// Iterate over the values of transition-duration.
                #[allow(non_snake_case)]
                #[inline]
                pub fn transition_duration_iter(&self) -> TransitionDurationIter {
                    TransitionDurationIter {
                        style_struct: self,
                        current: 0,
                        max: self.transition_duration_count(),
                    }
                }

                /// Get a value mod `index` for the property transition-duration.
                #[allow(non_snake_case)]
                #[inline]
                pub fn transition_duration_mod(&self, index: usize)
                    -> longhands::transition_duration::computed_value::SingleComputedValue {
                    self.transition_duration_at(index % self.transition_duration_count())
                }
                /// Iterate over the values of transition-timing-function.
                #[allow(non_snake_case)]
                #[inline]
                pub fn transition_timing_function_iter(&self) -> TransitionTimingFunctionIter {
                    TransitionTimingFunctionIter {
                        style_struct: self,
                        current: 0,
                        max: self.transition_timing_function_count(),
                    }
                }

                /// Get a value mod `index` for the property transition-timing-function.
                #[allow(non_snake_case)]
                #[inline]
                pub fn transition_timing_function_mod(&self, index: usize)
                    -> longhands::transition_timing_function::computed_value::SingleComputedValue {
                    self.transition_timing_function_at(index % self.transition_timing_function_count())
                }
                /// Iterate over the values of transition-property.
                #[allow(non_snake_case)]
                #[inline]
                pub fn transition_property_iter(&self) -> TransitionPropertyIter {
                    TransitionPropertyIter {
                        style_struct: self,
                        current: 0,
                        max: self.transition_property_count(),
                    }
                }

                /// Get a value mod `index` for the property transition-property.
                #[allow(non_snake_case)]
                #[inline]
                pub fn transition_property_mod(&self, index: usize)
                    -> longhands::transition_property::computed_value::SingleComputedValue {
                    self.transition_property_at(index % self.transition_property_count())
                }
                /// Iterate over the values of transition-delay.
                #[allow(non_snake_case)]
                #[inline]
                pub fn transition_delay_iter(&self) -> TransitionDelayIter {
                    TransitionDelayIter {
                        style_struct: self,
                        current: 0,
                        max: self.transition_delay_count(),
                    }
                }

                /// Get a value mod `index` for the property transition-delay.
                #[allow(non_snake_case)]
                #[inline]
                pub fn transition_delay_mod(&self, index: usize)
                    -> longhands::transition_delay::computed_value::SingleComputedValue {
                    self.transition_delay_at(index % self.transition_delay_count())
                }
                /// Iterate over the values of animation-name.
                #[allow(non_snake_case)]
                #[inline]
                pub fn animation_name_iter(&self) -> AnimationNameIter {
                    AnimationNameIter {
                        style_struct: self,
                        current: 0,
                        max: self.animation_name_count(),
                    }
                }

                /// Get a value mod `index` for the property animation-name.
                #[allow(non_snake_case)]
                #[inline]
                pub fn animation_name_mod(&self, index: usize)
                    -> longhands::animation_name::computed_value::SingleComputedValue {
                    self.animation_name_at(index % self.animation_name_count())
                }
                /// Iterate over the values of animation-duration.
                #[allow(non_snake_case)]
                #[inline]
                pub fn animation_duration_iter(&self) -> AnimationDurationIter {
                    AnimationDurationIter {
                        style_struct: self,
                        current: 0,
                        max: self.animation_duration_count(),
                    }
                }

                /// Get a value mod `index` for the property animation-duration.
                #[allow(non_snake_case)]
                #[inline]
                pub fn animation_duration_mod(&self, index: usize)
                    -> longhands::animation_duration::computed_value::SingleComputedValue {
                    self.animation_duration_at(index % self.animation_duration_count())
                }
                /// Iterate over the values of animation-timing-function.
                #[allow(non_snake_case)]
                #[inline]
                pub fn animation_timing_function_iter(&self) -> AnimationTimingFunctionIter {
                    AnimationTimingFunctionIter {
                        style_struct: self,
                        current: 0,
                        max: self.animation_timing_function_count(),
                    }
                }

                /// Get a value mod `index` for the property animation-timing-function.
                #[allow(non_snake_case)]
                #[inline]
                pub fn animation_timing_function_mod(&self, index: usize)
                    -> longhands::animation_timing_function::computed_value::SingleComputedValue {
                    self.animation_timing_function_at(index % self.animation_timing_function_count())
                }
                /// Iterate over the values of animation-iteration-count.
                #[allow(non_snake_case)]
                #[inline]
                pub fn animation_iteration_count_iter(&self) -> AnimationIterationCountIter {
                    AnimationIterationCountIter {
                        style_struct: self,
                        current: 0,
                        max: self.animation_iteration_count_count(),
                    }
                }

                /// Get a value mod `index` for the property animation-iteration-count.
                #[allow(non_snake_case)]
                #[inline]
                pub fn animation_iteration_count_mod(&self, index: usize)
                    -> longhands::animation_iteration_count::computed_value::SingleComputedValue {
                    self.animation_iteration_count_at(index % self.animation_iteration_count_count())
                }
                /// Iterate over the values of animation-direction.
                #[allow(non_snake_case)]
                #[inline]
                pub fn animation_direction_iter(&self) -> AnimationDirectionIter {
                    AnimationDirectionIter {
                        style_struct: self,
                        current: 0,
                        max: self.animation_direction_count(),
                    }
                }

                /// Get a value mod `index` for the property animation-direction.
                #[allow(non_snake_case)]
                #[inline]
                pub fn animation_direction_mod(&self, index: usize)
                    -> longhands::animation_direction::computed_value::SingleComputedValue {
                    self.animation_direction_at(index % self.animation_direction_count())
                }
                /// Iterate over the values of animation-play-state.
                #[allow(non_snake_case)]
                #[inline]
                pub fn animation_play_state_iter(&self) -> AnimationPlayStateIter {
                    AnimationPlayStateIter {
                        style_struct: self,
                        current: 0,
                        max: self.animation_play_state_count(),
                    }
                }

                /// Get a value mod `index` for the property animation-play-state.
                #[allow(non_snake_case)]
                #[inline]
                pub fn animation_play_state_mod(&self, index: usize)
                    -> longhands::animation_play_state::computed_value::SingleComputedValue {
                    self.animation_play_state_at(index % self.animation_play_state_count())
                }
                /// Iterate over the values of animation-fill-mode.
                #[allow(non_snake_case)]
                #[inline]
                pub fn animation_fill_mode_iter(&self) -> AnimationFillModeIter {
                    AnimationFillModeIter {
                        style_struct: self,
                        current: 0,
                        max: self.animation_fill_mode_count(),
                    }
                }

                /// Get a value mod `index` for the property animation-fill-mode.
                #[allow(non_snake_case)]
                #[inline]
                pub fn animation_fill_mode_mod(&self, index: usize)
                    -> longhands::animation_fill_mode::computed_value::SingleComputedValue {
                    self.animation_fill_mode_at(index % self.animation_fill_mode_count())
                }
                /// Iterate over the values of animation-delay.
                #[allow(non_snake_case)]
                #[inline]
                pub fn animation_delay_iter(&self) -> AnimationDelayIter {
                    AnimationDelayIter {
                        style_struct: self,
                        current: 0,
                        max: self.animation_delay_count(),
                    }
                }

                /// Get a value mod `index` for the property animation-delay.
                #[allow(non_snake_case)]
                #[inline]
                pub fn animation_delay_mod(&self, index: usize)
                    -> longhands::animation_delay::computed_value::SingleComputedValue {
                    self.animation_delay_at(index % self.animation_delay_count())
                }

            /// Returns whether there is any animation specified with
            /// animation-name other than `none`.
            pub fn specifies_animations(&self) -> bool {
                self.animation_name_iter().any(|name| name.0.is_some())
            }

            /// Returns whether there are any transitions specified.
            #[cfg(feature = "servo")]
            pub fn specifies_transitions(&self) -> bool {
                self.transition_duration_iter()
                    .take(self.transition_property_count())
                    .any(|t| t.seconds() > 0.)
            }
    }

            /// An iterator over the values of the transition-duration properties.
            pub struct TransitionDurationIter<'a> {
                style_struct: &'a style_structs::Box,
                current: usize,
                max: usize,
            }

            impl<'a> Iterator for TransitionDurationIter<'a> {
                type Item = longhands::transition_duration::computed_value::SingleComputedValue;

                fn next(&mut self) -> Option<Self::Item> {
                    self.current += 1;
                    if self.current <= self.max {
                        Some(self.style_struct.transition_duration_at(self.current - 1))
                    } else {
                        None
                    }
                }
            }
            /// An iterator over the values of the transition-timing-function properties.
            pub struct TransitionTimingFunctionIter<'a> {
                style_struct: &'a style_structs::Box,
                current: usize,
                max: usize,
            }

            impl<'a> Iterator for TransitionTimingFunctionIter<'a> {
                type Item = longhands::transition_timing_function::computed_value::SingleComputedValue;

                fn next(&mut self) -> Option<Self::Item> {
                    self.current += 1;
                    if self.current <= self.max {
                        Some(self.style_struct.transition_timing_function_at(self.current - 1))
                    } else {
                        None
                    }
                }
            }
            /// An iterator over the values of the transition-property properties.
            pub struct TransitionPropertyIter<'a> {
                style_struct: &'a style_structs::Box,
                current: usize,
                max: usize,
            }

            impl<'a> Iterator for TransitionPropertyIter<'a> {
                type Item = longhands::transition_property::computed_value::SingleComputedValue;

                fn next(&mut self) -> Option<Self::Item> {
                    self.current += 1;
                    if self.current <= self.max {
                        Some(self.style_struct.transition_property_at(self.current - 1))
                    } else {
                        None
                    }
                }
            }
            /// An iterator over the values of the transition-delay properties.
            pub struct TransitionDelayIter<'a> {
                style_struct: &'a style_structs::Box,
                current: usize,
                max: usize,
            }

            impl<'a> Iterator for TransitionDelayIter<'a> {
                type Item = longhands::transition_delay::computed_value::SingleComputedValue;

                fn next(&mut self) -> Option<Self::Item> {
                    self.current += 1;
                    if self.current <= self.max {
                        Some(self.style_struct.transition_delay_at(self.current - 1))
                    } else {
                        None
                    }
                }
            }
            /// An iterator over the values of the animation-name properties.
            pub struct AnimationNameIter<'a> {
                style_struct: &'a style_structs::Box,
                current: usize,
                max: usize,
            }

            impl<'a> Iterator for AnimationNameIter<'a> {
                type Item = longhands::animation_name::computed_value::SingleComputedValue;

                fn next(&mut self) -> Option<Self::Item> {
                    self.current += 1;
                    if self.current <= self.max {
                        Some(self.style_struct.animation_name_at(self.current - 1))
                    } else {
                        None
                    }
                }
            }
            /// An iterator over the values of the animation-duration properties.
            pub struct AnimationDurationIter<'a> {
                style_struct: &'a style_structs::Box,
                current: usize,
                max: usize,
            }

            impl<'a> Iterator for AnimationDurationIter<'a> {
                type Item = longhands::animation_duration::computed_value::SingleComputedValue;

                fn next(&mut self) -> Option<Self::Item> {
                    self.current += 1;
                    if self.current <= self.max {
                        Some(self.style_struct.animation_duration_at(self.current - 1))
                    } else {
                        None
                    }
                }
            }
            /// An iterator over the values of the animation-timing-function properties.
            pub struct AnimationTimingFunctionIter<'a> {
                style_struct: &'a style_structs::Box,
                current: usize,
                max: usize,
            }

            impl<'a> Iterator for AnimationTimingFunctionIter<'a> {
                type Item = longhands::animation_timing_function::computed_value::SingleComputedValue;

                fn next(&mut self) -> Option<Self::Item> {
                    self.current += 1;
                    if self.current <= self.max {
                        Some(self.style_struct.animation_timing_function_at(self.current - 1))
                    } else {
                        None
                    }
                }
            }
            /// An iterator over the values of the animation-iteration-count properties.
            pub struct AnimationIterationCountIter<'a> {
                style_struct: &'a style_structs::Box,
                current: usize,
                max: usize,
            }

            impl<'a> Iterator for AnimationIterationCountIter<'a> {
                type Item = longhands::animation_iteration_count::computed_value::SingleComputedValue;

                fn next(&mut self) -> Option<Self::Item> {
                    self.current += 1;
                    if self.current <= self.max {
                        Some(self.style_struct.animation_iteration_count_at(self.current - 1))
                    } else {
                        None
                    }
                }
            }
            /// An iterator over the values of the animation-direction properties.
            pub struct AnimationDirectionIter<'a> {
                style_struct: &'a style_structs::Box,
                current: usize,
                max: usize,
            }

            impl<'a> Iterator for AnimationDirectionIter<'a> {
                type Item = longhands::animation_direction::computed_value::SingleComputedValue;

                fn next(&mut self) -> Option<Self::Item> {
                    self.current += 1;
                    if self.current <= self.max {
                        Some(self.style_struct.animation_direction_at(self.current - 1))
                    } else {
                        None
                    }
                }
            }
            /// An iterator over the values of the animation-play-state properties.
            pub struct AnimationPlayStateIter<'a> {
                style_struct: &'a style_structs::Box,
                current: usize,
                max: usize,
            }

            impl<'a> Iterator for AnimationPlayStateIter<'a> {
                type Item = longhands::animation_play_state::computed_value::SingleComputedValue;

                fn next(&mut self) -> Option<Self::Item> {
                    self.current += 1;
                    if self.current <= self.max {
                        Some(self.style_struct.animation_play_state_at(self.current - 1))
                    } else {
                        None
                    }
                }
            }
            /// An iterator over the values of the animation-fill-mode properties.
            pub struct AnimationFillModeIter<'a> {
                style_struct: &'a style_structs::Box,
                current: usize,
                max: usize,
            }

            impl<'a> Iterator for AnimationFillModeIter<'a> {
                type Item = longhands::animation_fill_mode::computed_value::SingleComputedValue;

                fn next(&mut self) -> Option<Self::Item> {
                    self.current += 1;
                    if self.current <= self.max {
                        Some(self.style_struct.animation_fill_mode_at(self.current - 1))
                    } else {
                        None
                    }
                }
            }
            /// An iterator over the values of the animation-delay properties.
            pub struct AnimationDelayIter<'a> {
                style_struct: &'a style_structs::Box,
                current: usize,
                max: usize,
            }

            impl<'a> Iterator for AnimationDelayIter<'a> {
                type Item = longhands::animation_delay::computed_value::SingleComputedValue;

                fn next(&mut self) -> Option<Self::Item> {
                    self.current += 1;
                    if self.current <= self.max {
                        Some(self.style_struct.animation_delay_at(self.current - 1))
                    } else {
                        None
                    }
                }
            }
    impl style_structs::Color {

    }

    impl style_structs::Column {

    }

    impl style_structs::Counters {

    }

    impl style_structs::Effects {

    }

    impl style_structs::Font {

    }

    impl style_structs::InheritedBox {

    }

    impl style_structs::InheritedTable {

    }

    impl style_structs::InheritedText {

    }

    impl style_structs::List {

    }

    impl style_structs::Margin {

    }

    impl style_structs::Outline {

    }

    impl style_structs::Padding {

    }

    impl style_structs::Pointing {

    }

    impl style_structs::Position {

    }

    impl style_structs::Table {

    }

    impl style_structs::Text {

    }

    impl style_structs::UI {

    }

    impl style_structs::InheritedSVG {

    }

    impl style_structs::SVG {

    }

    impl style_structs::XUL {

    }



#[cfg(feature = "gecko")]
pub use gecko_properties::{ComputedValues, ComputedValuesInner};

#[cfg(feature = "servo")]
#[cfg_attr(feature = "servo", derive(Clone, Debug))]
/// Actual data of ComputedValues, to match up with Gecko
pub struct ComputedValuesInner {
        background: Arc<style_structs::Background>,
        border: Arc<style_structs::Border>,
        box_: Arc<style_structs::Box>,
        color: Arc<style_structs::Color>,
        column: Arc<style_structs::Column>,
        counters: Arc<style_structs::Counters>,
        effects: Arc<style_structs::Effects>,
        font: Arc<style_structs::Font>,
        inheritedbox: Arc<style_structs::InheritedBox>,
        inheritedtable: Arc<style_structs::InheritedTable>,
        inheritedtext: Arc<style_structs::InheritedText>,
        list: Arc<style_structs::List>,
        margin: Arc<style_structs::Margin>,
        outline: Arc<style_structs::Outline>,
        padding: Arc<style_structs::Padding>,
        pointing: Arc<style_structs::Pointing>,
        position: Arc<style_structs::Position>,
        table: Arc<style_structs::Table>,
        text: Arc<style_structs::Text>,
        ui: Arc<style_structs::UI>,
        inheritedsvg: Arc<style_structs::InheritedSVG>,
        svg: Arc<style_structs::SVG>,
        xul: Arc<style_structs::XUL>,
    custom_properties: Option<Arc<::custom_properties::CustomPropertiesMap>>,
    /// The writing mode of this computed values struct.
    pub writing_mode: WritingMode,

    /// The keyword behind the current font-size property, if any
    pub font_computation_data: FontComputationData,

    /// A set of flags we use to store misc information regarding this style.
    pub flags: ComputedValueFlags,

    /// The rule node representing the ordered list of rules matched for this
    /// node.  Can be None for default values and text nodes.  This is
    /// essentially an optimization to avoid referencing the root rule node.
    pub rules: Option<StrongRuleNode>,

    /// The element's computed values if visited, only computed if there's a
    /// relevant link for this element. A element's "relevant link" is the
    /// element being matched if it is a link or the nearest ancestor link.
    visited_style: Option<Arc<ComputedValues>>,
}

/// The struct that Servo uses to represent computed values.
///
/// This struct contains an immutable atomically-reference-counted pointer to
/// every kind of style struct.
///
/// When needed, the structs may be copied in order to get mutated.
#[cfg(feature = "servo")]
#[cfg_attr(feature = "servo", derive(Clone, Debug))]
pub struct ComputedValues {
    /// The actual computed values
    ///
    /// In Gecko the outer ComputedValues is actually a style context,
    /// whereas ComputedValuesInner is the core set of computed values.
    ///
    /// We maintain this distinction in servo to reduce the amount of special casing.
    inner: ComputedValuesInner,
}

impl ComputedValues {
    /// Returns the visited rules, if applicable.
    pub fn visited_rules(&self) -> Option<&StrongRuleNode> {
        self.visited_style.as_ref().and_then(|s| s.rules.as_ref())
    }

    /// Returns whether we're in a display: none subtree.
    pub fn is_in_display_none_subtree(&self) -> bool {
        use properties::computed_value_flags::IS_IN_DISPLAY_NONE_SUBTREE;

        self.flags.contains(IS_IN_DISPLAY_NONE_SUBTREE)
    }
}

#[cfg(feature = "servo")]
impl ComputedValues {
    /// Create a new refcounted `ComputedValues`
    pub fn new(
        _: &Device,
        _: Option<&ComputedValues>,
        _: Option<&PseudoElement>,
        custom_properties: Option<Arc<::custom_properties::CustomPropertiesMap>>,
        writing_mode: WritingMode,
        font_size_keyword: FontComputationData,
        flags: ComputedValueFlags,
        rules: Option<StrongRuleNode>,
        visited_style: Option<Arc<ComputedValues>>,
        background: Arc<style_structs::Background>,
        border: Arc<style_structs::Border>,
        box_: Arc<style_structs::Box>,
        color: Arc<style_structs::Color>,
        column: Arc<style_structs::Column>,
        counters: Arc<style_structs::Counters>,
        effects: Arc<style_structs::Effects>,
        font: Arc<style_structs::Font>,
        inheritedbox: Arc<style_structs::InheritedBox>,
        inheritedtable: Arc<style_structs::InheritedTable>,
        inheritedtext: Arc<style_structs::InheritedText>,
        list: Arc<style_structs::List>,
        margin: Arc<style_structs::Margin>,
        outline: Arc<style_structs::Outline>,
        padding: Arc<style_structs::Padding>,
        pointing: Arc<style_structs::Pointing>,
        position: Arc<style_structs::Position>,
        table: Arc<style_structs::Table>,
        text: Arc<style_structs::Text>,
        ui: Arc<style_structs::UI>,
        inheritedsvg: Arc<style_structs::InheritedSVG>,
        svg: Arc<style_structs::SVG>,
        xul: Arc<style_structs::XUL>,
    ) -> Arc<Self> {
        Arc::new(Self {
            inner: ComputedValuesInner::new(
                custom_properties,
                writing_mode,
                font_size_keyword,
                flags,
                rules,
                visited_style,
                background,
                border,
                box_,
                color,
                column,
                counters,
                effects,
                font,
                inheritedbox,
                inheritedtable,
                inheritedtext,
                list,
                margin,
                outline,
                padding,
                pointing,
                position,
                table,
                text,
                ui,
                inheritedsvg,
                svg,
                xul,
            )
        })
    }

    /// Get the initial computed values.
    pub fn initial_values() -> &'static Self { &*INITIAL_SERVO_VALUES }
}

#[cfg(feature = "servo")]
impl ComputedValuesInner {
    /// Construct a `ComputedValuesInner` instance.
    pub fn new(
        custom_properties: Option<Arc<::custom_properties::CustomPropertiesMap>>,
        writing_mode: WritingMode,
        font_size_keyword: FontComputationData,
        flags: ComputedValueFlags,
        rules: Option<StrongRuleNode>,
        visited_style: Option<Arc<ComputedValues>>,
        background: Arc<style_structs::Background>,
        border: Arc<style_structs::Border>,
        box_: Arc<style_structs::Box>,
        color: Arc<style_structs::Color>,
        column: Arc<style_structs::Column>,
        counters: Arc<style_structs::Counters>,
        effects: Arc<style_structs::Effects>,
        font: Arc<style_structs::Font>,
        inheritedbox: Arc<style_structs::InheritedBox>,
        inheritedtable: Arc<style_structs::InheritedTable>,
        inheritedtext: Arc<style_structs::InheritedText>,
        list: Arc<style_structs::List>,
        margin: Arc<style_structs::Margin>,
        outline: Arc<style_structs::Outline>,
        padding: Arc<style_structs::Padding>,
        pointing: Arc<style_structs::Pointing>,
        position: Arc<style_structs::Position>,
        table: Arc<style_structs::Table>,
        text: Arc<style_structs::Text>,
        ui: Arc<style_structs::UI>,
        inheritedsvg: Arc<style_structs::InheritedSVG>,
        svg: Arc<style_structs::SVG>,
        xul: Arc<style_structs::XUL>,
    ) -> Self {
        ComputedValuesInner {
            custom_properties: custom_properties,
            writing_mode: writing_mode,
            font_computation_data: font_size_keyword,
            rules: rules,
            visited_style: visited_style,
            flags: flags,
            background: background,
            border: border,
            box_: box_,
            color: color,
            column: column,
            counters: counters,
            effects: effects,
            font: font,
            inheritedbox: inheritedbox,
            inheritedtable: inheritedtable,
            inheritedtext: inheritedtext,
            list: list,
            margin: margin,
            outline: outline,
            padding: padding,
            pointing: pointing,
            position: position,
            table: table,
            text: text,
            ui: ui,
            inheritedsvg: inheritedsvg,
            svg: svg,
            xul: xul,
        }
    }
}

#[cfg(feature = "servo")]
impl ops::Deref for ComputedValues {
    type Target = ComputedValuesInner;
    fn deref(&self) -> &ComputedValuesInner {
        &self.inner
    }
}

#[cfg(feature = "servo")]
impl ops::DerefMut for ComputedValues {
    fn deref_mut(&mut self) -> &mut ComputedValuesInner {
        &mut self.inner
    }
}

#[cfg(feature = "servo")]
impl ComputedValuesInner {
        /// Clone the Background struct.
        #[inline]
        pub fn clone_background(&self) -> Arc<style_structs::Background> {
            self.background.clone()
        }

        /// Get a immutable reference to the Background struct.
        #[inline]
        pub fn get_background(&self) -> &style_structs::Background {
            &self.background
        }

        /// Gets an immutable reference to the refcounted value that wraps
        /// `Background`.
        pub fn background_arc(&self) -> &Arc<style_structs::Background> {
            &self.background
        }

        /// Get a mutable reference to the Background struct.
        #[inline]
        pub fn mutate_background(&mut self) -> &mut style_structs::Background {
            Arc::make_mut(&mut self.background)
        }
        /// Clone the Border struct.
        #[inline]
        pub fn clone_border(&self) -> Arc<style_structs::Border> {
            self.border.clone()
        }

        /// Get a immutable reference to the Border struct.
        #[inline]
        pub fn get_border(&self) -> &style_structs::Border {
            &self.border
        }

        /// Gets an immutable reference to the refcounted value that wraps
        /// `Border`.
        pub fn border_arc(&self) -> &Arc<style_structs::Border> {
            &self.border
        }

        /// Get a mutable reference to the Border struct.
        #[inline]
        pub fn mutate_border(&mut self) -> &mut style_structs::Border {
            Arc::make_mut(&mut self.border)
        }
        /// Clone the Box struct.
        #[inline]
        pub fn clone_box(&self) -> Arc<style_structs::Box> {
            self.box_.clone()
        }

        /// Get a immutable reference to the Box struct.
        #[inline]
        pub fn get_box(&self) -> &style_structs::Box {
            &self.box_
        }

        /// Gets an immutable reference to the refcounted value that wraps
        /// `Box`.
        pub fn box_arc(&self) -> &Arc<style_structs::Box> {
            &self.box_
        }

        /// Get a mutable reference to the Box struct.
        #[inline]
        pub fn mutate_box(&mut self) -> &mut style_structs::Box {
            Arc::make_mut(&mut self.box_)
        }
        /// Clone the Color struct.
        #[inline]
        pub fn clone_color(&self) -> Arc<style_structs::Color> {
            self.color.clone()
        }

        /// Get a immutable reference to the Color struct.
        #[inline]
        pub fn get_color(&self) -> &style_structs::Color {
            &self.color
        }

        /// Gets an immutable reference to the refcounted value that wraps
        /// `Color`.
        pub fn color_arc(&self) -> &Arc<style_structs::Color> {
            &self.color
        }

        /// Get a mutable reference to the Color struct.
        #[inline]
        pub fn mutate_color(&mut self) -> &mut style_structs::Color {
            Arc::make_mut(&mut self.color)
        }
        /// Clone the Column struct.
        #[inline]
        pub fn clone_column(&self) -> Arc<style_structs::Column> {
            self.column.clone()
        }

        /// Get a immutable reference to the Column struct.
        #[inline]
        pub fn get_column(&self) -> &style_structs::Column {
            &self.column
        }

        /// Gets an immutable reference to the refcounted value that wraps
        /// `Column`.
        pub fn column_arc(&self) -> &Arc<style_structs::Column> {
            &self.column
        }

        /// Get a mutable reference to the Column struct.
        #[inline]
        pub fn mutate_column(&mut self) -> &mut style_structs::Column {
            Arc::make_mut(&mut self.column)
        }
        /// Clone the Counters struct.
        #[inline]
        pub fn clone_counters(&self) -> Arc<style_structs::Counters> {
            self.counters.clone()
        }

        /// Get a immutable reference to the Counters struct.
        #[inline]
        pub fn get_counters(&self) -> &style_structs::Counters {
            &self.counters
        }

        /// Gets an immutable reference to the refcounted value that wraps
        /// `Counters`.
        pub fn counters_arc(&self) -> &Arc<style_structs::Counters> {
            &self.counters
        }

        /// Get a mutable reference to the Counters struct.
        #[inline]
        pub fn mutate_counters(&mut self) -> &mut style_structs::Counters {
            Arc::make_mut(&mut self.counters)
        }
        /// Clone the Effects struct.
        #[inline]
        pub fn clone_effects(&self) -> Arc<style_structs::Effects> {
            self.effects.clone()
        }

        /// Get a immutable reference to the Effects struct.
        #[inline]
        pub fn get_effects(&self) -> &style_structs::Effects {
            &self.effects
        }

        /// Gets an immutable reference to the refcounted value that wraps
        /// `Effects`.
        pub fn effects_arc(&self) -> &Arc<style_structs::Effects> {
            &self.effects
        }

        /// Get a mutable reference to the Effects struct.
        #[inline]
        pub fn mutate_effects(&mut self) -> &mut style_structs::Effects {
            Arc::make_mut(&mut self.effects)
        }
        /// Clone the Font struct.
        #[inline]
        pub fn clone_font(&self) -> Arc<style_structs::Font> {
            self.font.clone()
        }

        /// Get a immutable reference to the Font struct.
        #[inline]
        pub fn get_font(&self) -> &style_structs::Font {
            &self.font
        }

        /// Gets an immutable reference to the refcounted value that wraps
        /// `Font`.
        pub fn font_arc(&self) -> &Arc<style_structs::Font> {
            &self.font
        }

        /// Get a mutable reference to the Font struct.
        #[inline]
        pub fn mutate_font(&mut self) -> &mut style_structs::Font {
            Arc::make_mut(&mut self.font)
        }
        /// Clone the InheritedBox struct.
        #[inline]
        pub fn clone_inheritedbox(&self) -> Arc<style_structs::InheritedBox> {
            self.inheritedbox.clone()
        }

        /// Get a immutable reference to the InheritedBox struct.
        #[inline]
        pub fn get_inheritedbox(&self) -> &style_structs::InheritedBox {
            &self.inheritedbox
        }

        /// Gets an immutable reference to the refcounted value that wraps
        /// `InheritedBox`.
        pub fn inheritedbox_arc(&self) -> &Arc<style_structs::InheritedBox> {
            &self.inheritedbox
        }

        /// Get a mutable reference to the InheritedBox struct.
        #[inline]
        pub fn mutate_inheritedbox(&mut self) -> &mut style_structs::InheritedBox {
            Arc::make_mut(&mut self.inheritedbox)
        }
        /// Clone the InheritedTable struct.
        #[inline]
        pub fn clone_inheritedtable(&self) -> Arc<style_structs::InheritedTable> {
            self.inheritedtable.clone()
        }

        /// Get a immutable reference to the InheritedTable struct.
        #[inline]
        pub fn get_inheritedtable(&self) -> &style_structs::InheritedTable {
            &self.inheritedtable
        }

        /// Gets an immutable reference to the refcounted value that wraps
        /// `InheritedTable`.
        pub fn inheritedtable_arc(&self) -> &Arc<style_structs::InheritedTable> {
            &self.inheritedtable
        }

        /// Get a mutable reference to the InheritedTable struct.
        #[inline]
        pub fn mutate_inheritedtable(&mut self) -> &mut style_structs::InheritedTable {
            Arc::make_mut(&mut self.inheritedtable)
        }
        /// Clone the InheritedText struct.
        #[inline]
        pub fn clone_inheritedtext(&self) -> Arc<style_structs::InheritedText> {
            self.inheritedtext.clone()
        }

        /// Get a immutable reference to the InheritedText struct.
        #[inline]
        pub fn get_inheritedtext(&self) -> &style_structs::InheritedText {
            &self.inheritedtext
        }

        /// Gets an immutable reference to the refcounted value that wraps
        /// `InheritedText`.
        pub fn inheritedtext_arc(&self) -> &Arc<style_structs::InheritedText> {
            &self.inheritedtext
        }

        /// Get a mutable reference to the InheritedText struct.
        #[inline]
        pub fn mutate_inheritedtext(&mut self) -> &mut style_structs::InheritedText {
            Arc::make_mut(&mut self.inheritedtext)
        }
        /// Clone the List struct.
        #[inline]
        pub fn clone_list(&self) -> Arc<style_structs::List> {
            self.list.clone()
        }

        /// Get a immutable reference to the List struct.
        #[inline]
        pub fn get_list(&self) -> &style_structs::List {
            &self.list
        }

        /// Gets an immutable reference to the refcounted value that wraps
        /// `List`.
        pub fn list_arc(&self) -> &Arc<style_structs::List> {
            &self.list
        }

        /// Get a mutable reference to the List struct.
        #[inline]
        pub fn mutate_list(&mut self) -> &mut style_structs::List {
            Arc::make_mut(&mut self.list)
        }
        /// Clone the Margin struct.
        #[inline]
        pub fn clone_margin(&self) -> Arc<style_structs::Margin> {
            self.margin.clone()
        }

        /// Get a immutable reference to the Margin struct.
        #[inline]
        pub fn get_margin(&self) -> &style_structs::Margin {
            &self.margin
        }

        /// Gets an immutable reference to the refcounted value that wraps
        /// `Margin`.
        pub fn margin_arc(&self) -> &Arc<style_structs::Margin> {
            &self.margin
        }

        /// Get a mutable reference to the Margin struct.
        #[inline]
        pub fn mutate_margin(&mut self) -> &mut style_structs::Margin {
            Arc::make_mut(&mut self.margin)
        }
        /// Clone the Outline struct.
        #[inline]
        pub fn clone_outline(&self) -> Arc<style_structs::Outline> {
            self.outline.clone()
        }

        /// Get a immutable reference to the Outline struct.
        #[inline]
        pub fn get_outline(&self) -> &style_structs::Outline {
            &self.outline
        }

        /// Gets an immutable reference to the refcounted value that wraps
        /// `Outline`.
        pub fn outline_arc(&self) -> &Arc<style_structs::Outline> {
            &self.outline
        }

        /// Get a mutable reference to the Outline struct.
        #[inline]
        pub fn mutate_outline(&mut self) -> &mut style_structs::Outline {
            Arc::make_mut(&mut self.outline)
        }
        /// Clone the Padding struct.
        #[inline]
        pub fn clone_padding(&self) -> Arc<style_structs::Padding> {
            self.padding.clone()
        }

        /// Get a immutable reference to the Padding struct.
        #[inline]
        pub fn get_padding(&self) -> &style_structs::Padding {
            &self.padding
        }

        /// Gets an immutable reference to the refcounted value that wraps
        /// `Padding`.
        pub fn padding_arc(&self) -> &Arc<style_structs::Padding> {
            &self.padding
        }

        /// Get a mutable reference to the Padding struct.
        #[inline]
        pub fn mutate_padding(&mut self) -> &mut style_structs::Padding {
            Arc::make_mut(&mut self.padding)
        }
        /// Clone the Pointing struct.
        #[inline]
        pub fn clone_pointing(&self) -> Arc<style_structs::Pointing> {
            self.pointing.clone()
        }

        /// Get a immutable reference to the Pointing struct.
        #[inline]
        pub fn get_pointing(&self) -> &style_structs::Pointing {
            &self.pointing
        }

        /// Gets an immutable reference to the refcounted value that wraps
        /// `Pointing`.
        pub fn pointing_arc(&self) -> &Arc<style_structs::Pointing> {
            &self.pointing
        }

        /// Get a mutable reference to the Pointing struct.
        #[inline]
        pub fn mutate_pointing(&mut self) -> &mut style_structs::Pointing {
            Arc::make_mut(&mut self.pointing)
        }
        /// Clone the Position struct.
        #[inline]
        pub fn clone_position(&self) -> Arc<style_structs::Position> {
            self.position.clone()
        }

        /// Get a immutable reference to the Position struct.
        #[inline]
        pub fn get_position(&self) -> &style_structs::Position {
            &self.position
        }

        /// Gets an immutable reference to the refcounted value that wraps
        /// `Position`.
        pub fn position_arc(&self) -> &Arc<style_structs::Position> {
            &self.position
        }

        /// Get a mutable reference to the Position struct.
        #[inline]
        pub fn mutate_position(&mut self) -> &mut style_structs::Position {
            Arc::make_mut(&mut self.position)
        }
        /// Clone the Table struct.
        #[inline]
        pub fn clone_table(&self) -> Arc<style_structs::Table> {
            self.table.clone()
        }

        /// Get a immutable reference to the Table struct.
        #[inline]
        pub fn get_table(&self) -> &style_structs::Table {
            &self.table
        }

        /// Gets an immutable reference to the refcounted value that wraps
        /// `Table`.
        pub fn table_arc(&self) -> &Arc<style_structs::Table> {
            &self.table
        }

        /// Get a mutable reference to the Table struct.
        #[inline]
        pub fn mutate_table(&mut self) -> &mut style_structs::Table {
            Arc::make_mut(&mut self.table)
        }
        /// Clone the Text struct.
        #[inline]
        pub fn clone_text(&self) -> Arc<style_structs::Text> {
            self.text.clone()
        }

        /// Get a immutable reference to the Text struct.
        #[inline]
        pub fn get_text(&self) -> &style_structs::Text {
            &self.text
        }

        /// Gets an immutable reference to the refcounted value that wraps
        /// `Text`.
        pub fn text_arc(&self) -> &Arc<style_structs::Text> {
            &self.text
        }

        /// Get a mutable reference to the Text struct.
        #[inline]
        pub fn mutate_text(&mut self) -> &mut style_structs::Text {
            Arc::make_mut(&mut self.text)
        }
        /// Clone the UI struct.
        #[inline]
        pub fn clone_ui(&self) -> Arc<style_structs::UI> {
            self.ui.clone()
        }

        /// Get a immutable reference to the UI struct.
        #[inline]
        pub fn get_ui(&self) -> &style_structs::UI {
            &self.ui
        }

        /// Gets an immutable reference to the refcounted value that wraps
        /// `UI`.
        pub fn ui_arc(&self) -> &Arc<style_structs::UI> {
            &self.ui
        }

        /// Get a mutable reference to the UI struct.
        #[inline]
        pub fn mutate_ui(&mut self) -> &mut style_structs::UI {
            Arc::make_mut(&mut self.ui)
        }
        /// Clone the InheritedSVG struct.
        #[inline]
        pub fn clone_inheritedsvg(&self) -> Arc<style_structs::InheritedSVG> {
            self.inheritedsvg.clone()
        }

        /// Get a immutable reference to the InheritedSVG struct.
        #[inline]
        pub fn get_inheritedsvg(&self) -> &style_structs::InheritedSVG {
            &self.inheritedsvg
        }

        /// Gets an immutable reference to the refcounted value that wraps
        /// `InheritedSVG`.
        pub fn inheritedsvg_arc(&self) -> &Arc<style_structs::InheritedSVG> {
            &self.inheritedsvg
        }

        /// Get a mutable reference to the InheritedSVG struct.
        #[inline]
        pub fn mutate_inheritedsvg(&mut self) -> &mut style_structs::InheritedSVG {
            Arc::make_mut(&mut self.inheritedsvg)
        }
        /// Clone the SVG struct.
        #[inline]
        pub fn clone_svg(&self) -> Arc<style_structs::SVG> {
            self.svg.clone()
        }

        /// Get a immutable reference to the SVG struct.
        #[inline]
        pub fn get_svg(&self) -> &style_structs::SVG {
            &self.svg
        }

        /// Gets an immutable reference to the refcounted value that wraps
        /// `SVG`.
        pub fn svg_arc(&self) -> &Arc<style_structs::SVG> {
            &self.svg
        }

        /// Get a mutable reference to the SVG struct.
        #[inline]
        pub fn mutate_svg(&mut self) -> &mut style_structs::SVG {
            Arc::make_mut(&mut self.svg)
        }
        /// Clone the XUL struct.
        #[inline]
        pub fn clone_xul(&self) -> Arc<style_structs::XUL> {
            self.xul.clone()
        }

        /// Get a immutable reference to the XUL struct.
        #[inline]
        pub fn get_xul(&self) -> &style_structs::XUL {
            &self.xul
        }

        /// Gets an immutable reference to the refcounted value that wraps
        /// `XUL`.
        pub fn xul_arc(&self) -> &Arc<style_structs::XUL> {
            &self.xul
        }

        /// Get a mutable reference to the XUL struct.
        #[inline]
        pub fn mutate_xul(&mut self) -> &mut style_structs::XUL {
            Arc::make_mut(&mut self.xul)
        }

    /// Gets a reference to the rule node. Panic if no rule node exists.
    pub fn rules(&self) -> &StrongRuleNode {
        self.rules.as_ref().unwrap()
    }

    /// Whether there is a visited style.
    pub fn has_visited_style(&self) -> bool {
        self.visited_style.is_some()
    }

    /// Gets a reference to the visited style, if any.
    pub fn get_visited_style(&self) -> Option< & ComputedValues> {
        self.visited_style.as_ref().map(|x| &**x)
    }

    /// Gets a reference to the visited style. Panic if no visited style exists.
    pub fn visited_style(&self) -> &ComputedValues {
        self.get_visited_style().unwrap()
    }

    /// Clone the visited style.  Used for inheriting parent styles in
    /// StyleBuilder::for_derived_style.
    pub fn clone_visited_style(&self) -> Option<Arc<ComputedValues>> {
        self.visited_style.clone()
    }

    // Aah! The << in the return type below is not valid syntax, but we must
    // escape < that way for Mako.
    /// Gets a reference to the custom properties map (if one exists).
    pub fn get_custom_properties(&self) -> Option<&::custom_properties::CustomPropertiesMap> {
        self.custom_properties.as_ref().map(|x| &**x)
    }

    /// Get the custom properties map if necessary.
    ///
    /// Cloning the Arc here is fine because it only happens in the case where
    /// we have custom properties, and those are both rare and expensive.
    pub fn custom_properties(&self) -> Option<Arc<::custom_properties::CustomPropertiesMap>> {
        self.custom_properties.clone()
    }

    /// Whether this style has a -moz-binding value. This is always false for
    /// Servo for obvious reasons.
    pub fn has_moz_binding(&self) -> bool { false }

    /// Whether we're a visited style.
    pub fn is_style_if_visited(&self) -> bool {
        self.flags.contains(IS_STYLE_IF_VISITED)
    }

    /// Returns whether this style's display value is equal to contents.
    ///
    /// Since this isn't supported in Servo, this is always false for Servo.
    pub fn is_display_contents(&self) -> bool { false }

    #[inline]
    /// Returns whether the "content" property for the given style is completely
    /// ineffective, and would yield an empty `::before` or `::after`
    /// pseudo-element.
    pub fn ineffective_content_property(&self) -> bool {
        use properties::longhands::content::computed_value::T;
        match self.get_counters().content {
            T::Normal | T::None => true,
            T::Items(ref items) => items.is_empty(),
        }
    }

    /// Whether the current style is multicolumn.
    #[inline]
    pub fn is_multicol(&self) -> bool {
        let style = self.get_column();
        match style.column_width {
            Either::First(_width) => true,
            Either::Second(_auto) => match style.column_count {
                Either::First(_n) => true,
                Either::Second(_auto) => false,
            }
        }
    }

    /// Resolves the currentColor keyword.
    ///
    /// Any color value from computed values (except for the 'color' property
    /// itself) should go through this method.
    ///
    /// Usage example:
    /// let top_color = style.resolve_color(style.Border.border_top_color);
    #[inline]
    pub fn resolve_color(&self, color: computed::Color) -> RGBA {
        color.to_rgba(self.get_color().color)
    }

    /// Get the logical computed inline size.
    #[inline]
    pub fn content_inline_size(&self) -> computed::LengthOrPercentageOrAuto {
        let position_style = self.get_position();
        if self.writing_mode.is_vertical() {
            position_style.height
        } else {
            position_style.width
        }
    }

    /// Get the logical computed block size.
    #[inline]
    pub fn content_block_size(&self) -> computed::LengthOrPercentageOrAuto {
        let position_style = self.get_position();
        if self.writing_mode.is_vertical() { position_style.width } else { position_style.height }
    }

    /// Get the logical computed min inline size.
    #[inline]
    pub fn min_inline_size(&self) -> computed::LengthOrPercentage {
        let position_style = self.get_position();
        if self.writing_mode.is_vertical() { position_style.min_height } else { position_style.min_width }
    }

    /// Get the logical computed min block size.
    #[inline]
    pub fn min_block_size(&self) -> computed::LengthOrPercentage {
        let position_style = self.get_position();
        if self.writing_mode.is_vertical() { position_style.min_width } else { position_style.min_height }
    }

    /// Get the logical computed max inline size.
    #[inline]
    pub fn max_inline_size(&self) -> computed::LengthOrPercentageOrNone {
        let position_style = self.get_position();
        if self.writing_mode.is_vertical() { position_style.max_height } else { position_style.max_width }
    }

    /// Get the logical computed max block size.
    #[inline]
    pub fn max_block_size(&self) -> computed::LengthOrPercentageOrNone {
        let position_style = self.get_position();
        if self.writing_mode.is_vertical() { position_style.max_width } else { position_style.max_height }
    }

    /// Get the logical computed padding for this writing mode.
    #[inline]
    pub fn logical_padding(&self) -> LogicalMargin<computed::LengthOrPercentage> {
        let padding_style = self.get_padding();
        LogicalMargin::from_physical(self.writing_mode, SideOffsets2D::new(
            padding_style.padding_top.0,
            padding_style.padding_right.0,
            padding_style.padding_bottom.0,
            padding_style.padding_left.0,
        ))
    }

    /// Get the logical border width
    #[inline]
    pub fn border_width_for_writing_mode(&self, writing_mode: WritingMode) -> LogicalMargin<Au> {
        let border_style = self.get_border();
        LogicalMargin::from_physical(writing_mode, SideOffsets2D::new(
            Au::from(border_style.border_top_width),
            Au::from(border_style.border_right_width),
            Au::from(border_style.border_bottom_width),
            Au::from(border_style.border_left_width),
        ))
    }

    /// Gets the logical computed border widths for this style.
    #[inline]
    pub fn logical_border_width(&self) -> LogicalMargin<Au> {
        self.border_width_for_writing_mode(self.writing_mode)
    }

    /// Gets the logical computed margin from this style.
    #[inline]
    pub fn logical_margin(&self) -> LogicalMargin<computed::LengthOrPercentageOrAuto> {
        let margin_style = self.get_margin();
        LogicalMargin::from_physical(self.writing_mode, SideOffsets2D::new(
            margin_style.margin_top,
            margin_style.margin_right,
            margin_style.margin_bottom,
            margin_style.margin_left,
        ))
    }

    /// Gets the logical position from this style.
    #[inline]
    pub fn logical_position(&self) -> LogicalMargin<computed::LengthOrPercentageOrAuto> {
        // FIXME(SimonSapin): should be the writing mode of the containing block, maybe?
        let position_style = self.get_position();
        LogicalMargin::from_physical(self.writing_mode, SideOffsets2D::new(
            position_style.top,
            position_style.right,
            position_style.bottom,
            position_style.left,
        ))
    }

    /// Return true if the effects force the transform style to be Flat
    pub fn overrides_transform_style(&self) -> bool {
        use computed_values::mix_blend_mode;

        let effects = self.get_effects();
        // TODO(gw): Add clip-path, isolation, mask-image, mask-border-source when supported.
        effects.opacity < 1.0 ||
           !effects.filter.0.is_empty() ||
           !effects.clip.is_auto() ||
           effects.mix_blend_mode != mix_blend_mode::T::normal
    }

    /// https://drafts.csswg.org/css-transforms/#grouping-property-values
    pub fn get_used_transform_style(&self) -> computed_values::transform_style::T {
        use computed_values::transform_style;

        let box_ = self.get_box();

        if self.overrides_transform_style() {
            transform_style::T::flat
        } else {
            // Return the computed value if not overridden by the above exceptions
            box_.transform_style
        }
    }

    /// Whether given this transform value, the compositor would require a
    /// layer.
    pub fn transform_requires_layer(&self) -> bool {
        // Check if the transform matrix is 2D or 3D
        if let Some(ref transform_list) = self.get_box().transform.0 {
            for transform in transform_list {
                match *transform {
                    computed_values::transform::ComputedOperation::Perspective(..) => {
                        return true;
                    }
                    computed_values::transform::ComputedOperation::Matrix(m) => {
                        // See http://dev.w3.org/csswg/css-transforms/#2d-matrix
                        if m.m31 != 0.0 || m.m32 != 0.0 ||
                           m.m13 != 0.0 || m.m23 != 0.0 ||
                           m.m43 != 0.0 || m.m14 != 0.0 ||
                           m.m24 != 0.0 || m.m34 != 0.0 ||
                           m.m33 != 1.0 || m.m44 != 1.0 {
                            return true;
                        }
                    }
                    computed_values::transform::ComputedOperation::Translate(_, _, z) => {
                        if z.px() != 0. {
                            return true;
                        }
                    }
                    _ => {}
                }
            }
        }

        // Neither perspective nor transform present
        false
    }

    /// Serializes the computed value of this property as a string.
    pub fn computed_value_to_string(&self, property: PropertyDeclarationId) -> String {
        match property {
                    PropertyDeclarationId::Longhand(LonghandId::BackgroundColor) => {
                        self.background.background_color.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::BackgroundImage) => {
                        self.background.background_image.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::BackgroundPositionX) => {
                        self.background.background_position_x.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::BackgroundPositionY) => {
                        self.background.background_position_y.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::BackgroundRepeat) => {
                        self.background.background_repeat.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::BackgroundAttachment) => {
                        self.background.background_attachment.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::BackgroundClip) => {
                        self.background.background_clip.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::BackgroundOrigin) => {
                        self.background.background_origin.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::BackgroundSize) => {
                        self.background.background_size.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::BackgroundBlendMode) => {
                        self.background.background_blend_mode.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::BorderTopColor) => {
                        self.border.border_top_color.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::BorderTopStyle) => {
                        self.border.border_top_style.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::BorderTopWidth) => {
                        self.border.border_top_width.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::BorderLeftColor) => {
                        self.border.border_left_color.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::BorderLeftStyle) => {
                        self.border.border_left_style.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::BorderLeftWidth) => {
                        self.border.border_left_width.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::BorderBottomColor) => {
                        self.border.border_bottom_color.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::BorderBottomStyle) => {
                        self.border.border_bottom_style.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::BorderBottomWidth) => {
                        self.border.border_bottom_width.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::BorderRightColor) => {
                        self.border.border_right_color.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::BorderRightStyle) => {
                        self.border.border_right_style.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::BorderRightWidth) => {
                        self.border.border_right_width.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::BorderBlockStartColor) => {
                        self.border.border_block_start_color.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::BorderBlockStartStyle) => {
                        self.border.border_block_start_style.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::BorderBlockStartWidth) => {
                        self.border.border_block_start_width.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::BorderBlockEndColor) => {
                        self.border.border_block_end_color.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::BorderBlockEndStyle) => {
                        self.border.border_block_end_style.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::BorderBlockEndWidth) => {
                        self.border.border_block_end_width.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::BorderInlineStartColor) => {
                        self.border.border_inline_start_color.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::BorderInlineStartStyle) => {
                        self.border.border_inline_start_style.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::BorderInlineStartWidth) => {
                        self.border.border_inline_start_width.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::BorderInlineEndColor) => {
                        self.border.border_inline_end_color.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::BorderInlineEndStyle) => {
                        self.border.border_inline_end_style.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::BorderInlineEndWidth) => {
                        self.border.border_inline_end_width.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::BorderTopLeftRadius) => {
                        self.border.border_top_left_radius.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::BorderTopRightRadius) => {
                        self.border.border_top_right_radius.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::BorderBottomRightRadius) => {
                        self.border.border_bottom_right_radius.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::BorderBottomLeftRadius) => {
                        self.border.border_bottom_left_radius.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MozBorderTopColors) => {
                        self.border._moz_border_top_colors.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MozBorderLeftColors) => {
                        self.border._moz_border_left_colors.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MozBorderBottomColors) => {
                        self.border._moz_border_bottom_colors.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MozBorderRightColors) => {
                        self.border._moz_border_right_colors.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::BoxDecorationBreak) => {
                        self.border.box_decoration_break.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MozFloatEdge) => {
                        self.border._moz_float_edge.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::BorderImageSource) => {
                        self.border.border_image_source.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::BorderImageOutset) => {
                        self.border.border_image_outset.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::BorderImageRepeat) => {
                        self.border.border_image_repeat.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::BorderImageWidth) => {
                        self.border.border_image_width.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::BorderImageSlice) => {
                        self.border.border_image_slice.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::Display) => {
                        self.box_.display.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MozTopLayer) => {
                        self.box_._moz_top_layer.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::Position) => {
                        self.box_.position.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::Float) => {
                        self.box_.float.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::Clear) => {
                        self.box_.clear.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::VerticalAlign) => {
                        self.box_.vertical_align.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::OverflowClipBox) => {
                        self.box_.overflow_clip_box.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::OverflowX) => {
                        self.box_.overflow_x.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::OverflowY) => {
                        self.box_.overflow_y.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::TransitionDuration) => {
                        self.box_.transition_duration.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::TransitionTimingFunction) => {
                        self.box_.transition_timing_function.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::TransitionProperty) => {
                        self.box_.transition_property.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::TransitionDelay) => {
                        self.box_.transition_delay.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::AnimationName) => {
                        self.box_.animation_name.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::AnimationDuration) => {
                        self.box_.animation_duration.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::AnimationTimingFunction) => {
                        self.box_.animation_timing_function.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::AnimationIterationCount) => {
                        self.box_.animation_iteration_count.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::AnimationDirection) => {
                        self.box_.animation_direction.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::AnimationPlayState) => {
                        self.box_.animation_play_state.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::AnimationFillMode) => {
                        self.box_.animation_fill_mode.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::AnimationDelay) => {
                        self.box_.animation_delay.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::ScrollSnapPointsX) => {
                        self.box_.scroll_snap_points_x.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::ScrollSnapPointsY) => {
                        self.box_.scroll_snap_points_y.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::ScrollSnapDestination) => {
                        self.box_.scroll_snap_destination.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::ScrollSnapCoordinate) => {
                        self.box_.scroll_snap_coordinate.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::Transform) => {
                        self.box_.transform.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::ScrollBehavior) => {
                        self.box_.scroll_behavior.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::ScrollSnapTypeX) => {
                        self.box_.scroll_snap_type_x.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::ScrollSnapTypeY) => {
                        self.box_.scroll_snap_type_y.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::Isolation) => {
                        self.box_.isolation.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::PageBreakAfter) => {
                        self.box_.page_break_after.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::PageBreakBefore) => {
                        self.box_.page_break_before.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::PageBreakInside) => {
                        self.box_.page_break_inside.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::Resize) => {
                        self.box_.resize.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::Perspective) => {
                        self.box_.perspective.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::PerspectiveOrigin) => {
                        self.box_.perspective_origin.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::BackfaceVisibility) => {
                        self.box_.backface_visibility.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::TransformBox) => {
                        self.box_.transform_box.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::TransformStyle) => {
                        self.box_.transform_style.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::TransformOrigin) => {
                        self.box_.transform_origin.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::Contain) => {
                        self.box_.contain.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MozAppearance) => {
                        self.box_._moz_appearance.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MozBinding) => {
                        self.box_._moz_binding.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MozOrient) => {
                        self.box_._moz_orient.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::WillChange) => {
                        self.box_.will_change.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::ShapeOutside) => {
                        self.box_.shape_outside.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::TouchAction) => {
                        self.box_.touch_action.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::Color) => {
                        self.color.color.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::ColumnWidth) => {
                        self.column.column_width.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::ColumnCount) => {
                        self.column.column_count.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::ColumnGap) => {
                        self.column.column_gap.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::ColumnFill) => {
                        self.column.column_fill.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::ColumnRuleWidth) => {
                        self.column.column_rule_width.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::ColumnRuleColor) => {
                        self.column.column_rule_color.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::ColumnSpan) => {
                        self.column.column_span.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::ColumnRuleStyle) => {
                        self.column.column_rule_style.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::Content) => {
                        self.counters.content.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::CounterIncrement) => {
                        self.counters.counter_increment.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::CounterReset) => {
                        self.counters.counter_reset.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::Opacity) => {
                        self.effects.opacity.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::BoxShadow) => {
                        self.effects.box_shadow.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::Clip) => {
                        self.effects.clip.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::Filter) => {
                        self.effects.filter.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MixBlendMode) => {
                        self.effects.mix_blend_mode.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::FontFamily) => {
                        self.font.font_family.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::FontStyle) => {
                        self.font.font_style.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::FontVariantCaps) => {
                        self.font.font_variant_caps.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::FontWeight) => {
                        self.font.font_weight.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::FontSize) => {
                        self.font.font_size.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::FontSizeAdjust) => {
                        self.font.font_size_adjust.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::FontSynthesis) => {
                        self.font.font_synthesis.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::FontStretch) => {
                        self.font.font_stretch.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::FontKerning) => {
                        self.font.font_kerning.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::FontVariantAlternates) => {
                        self.font.font_variant_alternates.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::FontVariantEastAsian) => {
                        self.font.font_variant_east_asian.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::FontVariantLigatures) => {
                        self.font.font_variant_ligatures.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::FontVariantNumeric) => {
                        self.font.font_variant_numeric.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::FontVariantPosition) => {
                        self.font.font_variant_position.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::FontFeatureSettings) => {
                        self.font.font_feature_settings.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::FontVariationSettings) => {
                        self.font.font_variation_settings.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::FontLanguageOverride) => {
                        self.font.font_language_override.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::XLang) => {
                        self.font._x_lang.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MozScriptSizeMultiplier) => {
                        self.font._moz_script_size_multiplier.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MozScriptLevel) => {
                        self.font._moz_script_level.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MozMathDisplay) => {
                        self.font._moz_math_display.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MozMathVariant) => {
                        self.font._moz_math_variant.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MozScriptMinSize) => {
                        self.font._moz_script_min_size.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::XTextZoom) => {
                        self.font._x_text_zoom.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MozOsxFontSmoothing) => {
                        self.font._moz_osx_font_smoothing.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MozMinFontSizeRatio) => {
                        self.font._moz_min_font_size_ratio.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::Visibility) => {
                        self.inheritedbox.visibility.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::WritingMode) => {
                        self.inheritedbox.writing_mode.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::Direction) => {
                        self.inheritedbox.direction.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::TextOrientation) => {
                        self.inheritedbox.text_orientation.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::ColorAdjust) => {
                        self.inheritedbox.color_adjust.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::ImageRendering) => {
                        self.inheritedbox.image_rendering.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::ImageOrientation) => {
                        self.inheritedbox.image_orientation.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::BorderCollapse) => {
                        self.inheritedtable.border_collapse.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::EmptyCells) => {
                        self.inheritedtable.empty_cells.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::CaptionSide) => {
                        self.inheritedtable.caption_side.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::BorderSpacing) => {
                        self.inheritedtable.border_spacing.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::LineHeight) => {
                        self.inheritedtext.line_height.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::TextTransform) => {
                        self.inheritedtext.text_transform.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::Hyphens) => {
                        self.inheritedtext.hyphens.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MozTextSizeAdjust) => {
                        self.inheritedtext._moz_text_size_adjust.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::TextIndent) => {
                        self.inheritedtext.text_indent.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::OverflowWrap) => {
                        self.inheritedtext.overflow_wrap.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::WordBreak) => {
                        self.inheritedtext.word_break.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::TextJustify) => {
                        self.inheritedtext.text_justify.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::TextAlignLast) => {
                        self.inheritedtext.text_align_last.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::TextAlign) => {
                        self.inheritedtext.text_align.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::LetterSpacing) => {
                        self.inheritedtext.letter_spacing.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::WordSpacing) => {
                        self.inheritedtext.word_spacing.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::WhiteSpace) => {
                        self.inheritedtext.white_space.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::TextShadow) => {
                        self.inheritedtext.text_shadow.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::TextEmphasisStyle) => {
                        self.inheritedtext.text_emphasis_style.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::TextEmphasisPosition) => {
                        self.inheritedtext.text_emphasis_position.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::TextEmphasisColor) => {
                        self.inheritedtext.text_emphasis_color.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MozTabSize) => {
                        self.inheritedtext._moz_tab_size.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::WebkitTextFillColor) => {
                        self.inheritedtext._webkit_text_fill_color.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::WebkitTextStrokeColor) => {
                        self.inheritedtext._webkit_text_stroke_color.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::WebkitTextStrokeWidth) => {
                        self.inheritedtext._webkit_text_stroke_width.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::RubyAlign) => {
                        self.inheritedtext.ruby_align.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::RubyPosition) => {
                        self.inheritedtext.ruby_position.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::TextCombineUpright) => {
                        self.inheritedtext.text_combine_upright.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::TextRendering) => {
                        self.inheritedtext.text_rendering.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MozControlCharacterVisibility) => {
                        self.inheritedtext._moz_control_character_visibility.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::ListStylePosition) => {
                        self.list.list_style_position.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::ListStyleType) => {
                        self.list.list_style_type.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::ListStyleImage) => {
                        self.list.list_style_image.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::Quotes) => {
                        self.list.quotes.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MozImageRegion) => {
                        self.list._moz_image_region.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MarginTop) => {
                        self.margin.margin_top.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MarginLeft) => {
                        self.margin.margin_left.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MarginBottom) => {
                        self.margin.margin_bottom.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MarginRight) => {
                        self.margin.margin_right.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MarginBlockStart) => {
                        self.margin.margin_block_start.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MarginBlockEnd) => {
                        self.margin.margin_block_end.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MarginInlineStart) => {
                        self.margin.margin_inline_start.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MarginInlineEnd) => {
                        self.margin.margin_inline_end.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::OutlineColor) => {
                        self.outline.outline_color.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::OutlineStyle) => {
                        self.outline.outline_style.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::OutlineWidth) => {
                        self.outline.outline_width.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MozOutlineRadiusTopleft) => {
                        self.outline._moz_outline_radius_topleft.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MozOutlineRadiusTopright) => {
                        self.outline._moz_outline_radius_topright.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MozOutlineRadiusBottomright) => {
                        self.outline._moz_outline_radius_bottomright.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MozOutlineRadiusBottomleft) => {
                        self.outline._moz_outline_radius_bottomleft.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::OutlineOffset) => {
                        self.outline.outline_offset.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::PaddingTop) => {
                        self.padding.padding_top.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::PaddingLeft) => {
                        self.padding.padding_left.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::PaddingBottom) => {
                        self.padding.padding_bottom.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::PaddingRight) => {
                        self.padding.padding_right.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::PaddingBlockStart) => {
                        self.padding.padding_block_start.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::PaddingBlockEnd) => {
                        self.padding.padding_block_end.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::PaddingInlineStart) => {
                        self.padding.padding_inline_start.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::PaddingInlineEnd) => {
                        self.padding.padding_inline_end.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::Cursor) => {
                        self.pointing.cursor.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::PointerEvents) => {
                        self.pointing.pointer_events.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MozUserInput) => {
                        self.pointing._moz_user_input.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MozUserModify) => {
                        self.pointing._moz_user_modify.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MozUserFocus) => {
                        self.pointing._moz_user_focus.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::CaretColor) => {
                        self.pointing.caret_color.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::Top) => {
                        self.position.top.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::Left) => {
                        self.position.left.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::Bottom) => {
                        self.position.bottom.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::Right) => {
                        self.position.right.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::OffsetBlockStart) => {
                        self.position.offset_block_start.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::OffsetBlockEnd) => {
                        self.position.offset_block_end.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::OffsetInlineStart) => {
                        self.position.offset_inline_start.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::OffsetInlineEnd) => {
                        self.position.offset_inline_end.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::ZIndex) => {
                        self.position.z_index.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::FlexDirection) => {
                        self.position.flex_direction.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::FlexWrap) => {
                        self.position.flex_wrap.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::JustifyContent) => {
                        self.position.justify_content.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::AlignContent) => {
                        self.position.align_content.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::AlignItems) => {
                        self.position.align_items.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::JustifyItems) => {
                        self.position.justify_items.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::FlexGrow) => {
                        self.position.flex_grow.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::FlexShrink) => {
                        self.position.flex_shrink.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::AlignSelf) => {
                        self.position.align_self.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::JustifySelf) => {
                        self.position.justify_self.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::Order) => {
                        self.position.order.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::FlexBasis) => {
                        self.position.flex_basis.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::Width) => {
                        self.position.width.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MinWidth) => {
                        self.position.min_width.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MaxWidth) => {
                        self.position.max_width.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::Height) => {
                        self.position.height.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MinHeight) => {
                        self.position.min_height.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MaxHeight) => {
                        self.position.max_height.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::BlockSize) => {
                        self.position.block_size.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MinBlockSize) => {
                        self.position.min_block_size.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MaxBlockSize) => {
                        self.position.max_block_size.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::InlineSize) => {
                        self.position.inline_size.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MinInlineSize) => {
                        self.position.min_inline_size.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MaxInlineSize) => {
                        self.position.max_inline_size.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::BoxSizing) => {
                        self.position.box_sizing.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::ObjectFit) => {
                        self.position.object_fit.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::ObjectPosition) => {
                        self.position.object_position.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::GridRowGap) => {
                        self.position.grid_row_gap.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::GridRowStart) => {
                        self.position.grid_row_start.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::GridRowEnd) => {
                        self.position.grid_row_end.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::GridAutoRows) => {
                        self.position.grid_auto_rows.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::GridTemplateRows) => {
                        self.position.grid_template_rows.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::GridColumnGap) => {
                        self.position.grid_column_gap.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::GridColumnStart) => {
                        self.position.grid_column_start.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::GridColumnEnd) => {
                        self.position.grid_column_end.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::GridAutoColumns) => {
                        self.position.grid_auto_columns.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::GridTemplateColumns) => {
                        self.position.grid_template_columns.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::GridAutoFlow) => {
                        self.position.grid_auto_flow.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::GridTemplateAreas) => {
                        self.position.grid_template_areas.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::TableLayout) => {
                        self.table.table_layout.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::XSpan) => {
                        self.table._x_span.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::TextOverflow) => {
                        self.text.text_overflow.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::UnicodeBidi) => {
                        self.text.unicode_bidi.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::TextDecorationLine) => {
                        self.text.text_decoration_line.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::TextDecorationStyle) => {
                        self.text.text_decoration_style.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::TextDecorationColor) => {
                        self.text.text_decoration_color.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::InitialLetter) => {
                        self.text.initial_letter.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::ImeMode) => {
                        self.ui.ime_mode.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MozUserSelect) => {
                        self.ui._moz_user_select.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MozWindowDragging) => {
                        self.ui._moz_window_dragging.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MozWindowShadow) => {
                        self.ui._moz_window_shadow.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MozForceBrokenImageIcon) => {
                        self.ui._moz_force_broken_image_icon.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::TextAnchor) => {
                        self.inheritedsvg.text_anchor.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::ColorInterpolation) => {
                        self.inheritedsvg.color_interpolation.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::ColorInterpolationFilters) => {
                        self.inheritedsvg.color_interpolation_filters.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::Fill) => {
                        self.inheritedsvg.fill.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::FillOpacity) => {
                        self.inheritedsvg.fill_opacity.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::FillRule) => {
                        self.inheritedsvg.fill_rule.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::ShapeRendering) => {
                        self.inheritedsvg.shape_rendering.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::Stroke) => {
                        self.inheritedsvg.stroke.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::StrokeWidth) => {
                        self.inheritedsvg.stroke_width.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::StrokeLinecap) => {
                        self.inheritedsvg.stroke_linecap.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::StrokeLinejoin) => {
                        self.inheritedsvg.stroke_linejoin.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::StrokeMiterlimit) => {
                        self.inheritedsvg.stroke_miterlimit.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::StrokeOpacity) => {
                        self.inheritedsvg.stroke_opacity.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::StrokeDasharray) => {
                        self.inheritedsvg.stroke_dasharray.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::StrokeDashoffset) => {
                        self.inheritedsvg.stroke_dashoffset.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::ClipRule) => {
                        self.inheritedsvg.clip_rule.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MarkerStart) => {
                        self.inheritedsvg.marker_start.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MarkerMid) => {
                        self.inheritedsvg.marker_mid.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MarkerEnd) => {
                        self.inheritedsvg.marker_end.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::PaintOrder) => {
                        self.inheritedsvg.paint_order.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MozContextProperties) => {
                        self.inheritedsvg._moz_context_properties.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::DominantBaseline) => {
                        self.svg.dominant_baseline.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::VectorEffect) => {
                        self.svg.vector_effect.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::StopColor) => {
                        self.svg.stop_color.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::StopOpacity) => {
                        self.svg.stop_opacity.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::FloodColor) => {
                        self.svg.flood_color.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::FloodOpacity) => {
                        self.svg.flood_opacity.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::LightingColor) => {
                        self.svg.lighting_color.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MaskType) => {
                        self.svg.mask_type.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::ClipPath) => {
                        self.svg.clip_path.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MaskMode) => {
                        self.svg.mask_mode.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MaskRepeat) => {
                        self.svg.mask_repeat.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MaskPositionX) => {
                        self.svg.mask_position_x.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MaskPositionY) => {
                        self.svg.mask_position_y.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MaskClip) => {
                        self.svg.mask_clip.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MaskOrigin) => {
                        self.svg.mask_origin.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MaskSize) => {
                        self.svg.mask_size.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MaskComposite) => {
                        self.svg.mask_composite.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MaskImage) => {
                        self.svg.mask_image.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MozBoxAlign) => {
                        self.xul._moz_box_align.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MozBoxDirection) => {
                        self.xul._moz_box_direction.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MozBoxFlex) => {
                        self.xul._moz_box_flex.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MozBoxOrient) => {
                        self.xul._moz_box_orient.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MozBoxPack) => {
                        self.xul._moz_box_pack.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MozStackSizing) => {
                        self.xul._moz_stack_sizing.to_css_string()
                    }
                    PropertyDeclarationId::Longhand(LonghandId::MozBoxOrdinalGroup) => {
                        self.xul._moz_box_ordinal_group.to_css_string()
                    }
            PropertyDeclarationId::Custom(name) => {
                self.custom_properties
                    .as_ref()
                    .and_then(|map| map.get(name))
                    .map(|value| value.to_css_string())
                    .unwrap_or(String::new())
            }
        }
    }
}

/// Return a WritingMode bitflags from the relevant CSS properties.
pub fn get_writing_mode(inheritedbox_style: &style_structs::InheritedBox) -> WritingMode {
    use logical_geometry;
    let mut flags = WritingMode::empty();
    match inheritedbox_style.clone_direction() {
        computed_values::direction::T::ltr => {},
        computed_values::direction::T::rtl => {
            flags.insert(logical_geometry::FLAG_RTL);
        },
    }
    match inheritedbox_style.clone_writing_mode() {
        computed_values::writing_mode::T::horizontal_tb => {},
        computed_values::writing_mode::T::vertical_rl => {
            flags.insert(logical_geometry::FLAG_VERTICAL);
        },
        computed_values::writing_mode::T::vertical_lr => {
            flags.insert(logical_geometry::FLAG_VERTICAL);
            flags.insert(logical_geometry::FLAG_VERTICAL_LR);
        },
        computed_values::writing_mode::T::sideways_rl => {
            flags.insert(logical_geometry::FLAG_VERTICAL);
            flags.insert(logical_geometry::FLAG_SIDEWAYS);
        },
        computed_values::writing_mode::T::sideways_lr => {
            flags.insert(logical_geometry::FLAG_VERTICAL);
            flags.insert(logical_geometry::FLAG_VERTICAL_LR);
            flags.insert(logical_geometry::FLAG_LINE_INVERTED);
            flags.insert(logical_geometry::FLAG_SIDEWAYS);
        },
    }
    // If FLAG_SIDEWAYS is already set, this means writing-mode is either
    // sideways-rl or sideways-lr, and for both of these values,
    // text-orientation has no effect.
    if !flags.intersects(logical_geometry::FLAG_SIDEWAYS) {
        match inheritedbox_style.clone_text_orientation() {
            computed_values::text_orientation::T::mixed => {},
            computed_values::text_orientation::T::upright => {
                flags.insert(logical_geometry::FLAG_UPRIGHT);
            },
            computed_values::text_orientation::T::sideways => {
                flags.insert(logical_geometry::FLAG_SIDEWAYS);
            },
        }
    }
    flags
}

    pub use ::servo_arc::RawOffsetArc as BuilderArc;
    /// Clone an arc, returning a regular arc
    fn clone_arc<T: 'static>(x: &BuilderArc<T>) -> Arc<T> {
        Arc::from_raw_offset(x.clone())
    }

/// A reference to a style struct of the parent, or our own style struct.
pub enum StyleStructRef<'a, T: 'static> {
    /// A borrowed struct from the parent, for example, for inheriting style.
    Borrowed(&'a BuilderArc<T>),
    /// An owned struct, that we've already mutated.
    Owned(UniqueArc<T>),
    /// Temporarily vacated, will panic if accessed
    Vacated,
}

impl<'a, T: 'a> StyleStructRef<'a, T>
    where T: Clone,
{
    /// Ensure a mutable reference of this value exists, either cloning the
    /// borrowed value, or returning the owned one.
    pub fn mutate(&mut self) -> &mut T {
        if let StyleStructRef::Borrowed(v) = *self {
            *self = StyleStructRef::Owned(UniqueArc::new((**v).clone()));
        }

        match *self {
            StyleStructRef::Owned(ref mut v) => v,
            StyleStructRef::Borrowed(..) => unreachable!(),
            StyleStructRef::Vacated => panic!("Accessed vacated style struct")
        }
    }

    /// Extract a unique Arc from this struct, vacating it.
    ///
    /// The vacated state is a transient one, please put the Arc back
    /// when done via `put()`. This function is to be used to separate
    /// the struct being mutated from the computed context
    pub fn take(&mut self) -> UniqueArc<T> {
        use std::mem::replace;
        let inner = replace(self, StyleStructRef::Vacated);

        match inner {
            StyleStructRef::Owned(arc) => arc,
            StyleStructRef::Borrowed(arc) => UniqueArc::new((**arc).clone()),
            StyleStructRef::Vacated => panic!("Accessed vacated style struct"),
        }
    }

    /// Replace vacated ref with an arc
    pub fn put(&mut self, arc: UniqueArc<T>) {
        debug_assert!(matches!(*self, StyleStructRef::Vacated));
        *self = StyleStructRef::Owned(arc);
    }

    /// Get a mutable reference to the owned struct, or `None` if the struct
    /// hasn't been mutated.
    pub fn get_if_mutated(&mut self) -> Option<&mut T> {
        match *self {
            StyleStructRef::Owned(ref mut v) => Some(v),
            StyleStructRef::Borrowed(..) => None,
            StyleStructRef::Vacated => panic!("Accessed vacated style struct")
        }
    }

    /// Returns an `Arc` to the internal struct, constructing one if
    /// appropriate.
    pub fn build(self) -> Arc<T> {
        match self {
            StyleStructRef::Owned(v) => v.shareable(),
            StyleStructRef::Borrowed(v) => clone_arc(v),
            StyleStructRef::Vacated => panic!("Accessed vacated style struct")
        }
    }
}

impl<'a, T: 'a> ops::Deref for StyleStructRef<'a, T> {
    type Target = T;

    fn deref(&self) -> &T {
        match *self {
            StyleStructRef::Owned(ref v) => &**v,
            StyleStructRef::Borrowed(v) => &**v,
            StyleStructRef::Vacated => panic!("Accessed vacated style struct")
        }
    }
}

/// A type used to compute a struct with minimal overhead.
///
/// This allows holding references to the parent/default computed values without
/// actually cloning them, until we either build the style, or mutate the
/// inherited value.
pub struct StyleBuilder<'a> {
    /// The device we're using to compute style.
    ///
    /// This provides access to viewport unit ratios, etc.
    pub device: &'a Device,

    /// The style we're inheriting from.
    ///
    /// This is effectively
    /// `parent_style.unwrap_or(device.default_computed_values())`.
    inherited_style: &'a ComputedValues,

    /// The style we're inheriting from for properties that don't inherit from
    /// ::first-line.  This is the same as inherited_style, unless
    /// inherited_style is a ::first-line style.
    inherited_style_ignoring_first_line: &'a ComputedValues,

    /// The style we're getting reset structs from.
    reset_style: &'a ComputedValues,

    /// The style we're inheriting from explicitly, or none if we're the root of
    /// a subtree.
    parent_style: Option<&'a ComputedValues>,

    /// The rule node representing the ordered list of rules matched for this
    /// node.
    pub rules: Option<StrongRuleNode>,

    custom_properties: Option<Arc<::custom_properties::CustomPropertiesMap>>,

    /// The pseudo-element this style will represent.
    pub pseudo: Option<&'a PseudoElement>,

    /// Whether we have mutated any reset structs since the the last time
    /// `clear_modified_reset` was called.  This is used to tell whether the
    /// `StyleAdjuster` did any work.
    modified_reset: bool,

    /// The writing mode flags.
    ///
    /// TODO(emilio): Make private.
    pub writing_mode: WritingMode,
    /// The keyword behind the current font-size property, if any.
    pub font_size_keyword: FontComputationData,
    /// Flags for the computed value.
    pub flags: ComputedValueFlags,
    /// The element's style if visited, only computed if there's a relevant link
    /// for this element.  A element's "relevant link" is the element being
    /// matched if it is a link or the nearest ancestor link.
    visited_style: Option<Arc<ComputedValues>>,
        background: StyleStructRef<'a, style_structs::Background>,
        border: StyleStructRef<'a, style_structs::Border>,
        box_: StyleStructRef<'a, style_structs::Box>,
        color: StyleStructRef<'a, style_structs::Color>,
        column: StyleStructRef<'a, style_structs::Column>,
        counters: StyleStructRef<'a, style_structs::Counters>,
        effects: StyleStructRef<'a, style_structs::Effects>,
        font: StyleStructRef<'a, style_structs::Font>,
        inheritedbox: StyleStructRef<'a, style_structs::InheritedBox>,
        inheritedtable: StyleStructRef<'a, style_structs::InheritedTable>,
        inheritedtext: StyleStructRef<'a, style_structs::InheritedText>,
        list: StyleStructRef<'a, style_structs::List>,
        margin: StyleStructRef<'a, style_structs::Margin>,
        outline: StyleStructRef<'a, style_structs::Outline>,
        padding: StyleStructRef<'a, style_structs::Padding>,
        pointing: StyleStructRef<'a, style_structs::Pointing>,
        position: StyleStructRef<'a, style_structs::Position>,
        table: StyleStructRef<'a, style_structs::Table>,
        text: StyleStructRef<'a, style_structs::Text>,
        ui: StyleStructRef<'a, style_structs::UI>,
        inheritedsvg: StyleStructRef<'a, style_structs::InheritedSVG>,
        svg: StyleStructRef<'a, style_structs::SVG>,
        xul: StyleStructRef<'a, style_structs::XUL>,
}

impl<'a> StyleBuilder<'a> {
    /// Trivially construct a `StyleBuilder`.
    fn new(
        device: &'a Device,
        parent_style: Option<&'a ComputedValues>,
        parent_style_ignoring_first_line: Option<&'a ComputedValues>,
        pseudo: Option<&'a PseudoElement>,
        cascade_flags: CascadeFlags,
        rules: Option<StrongRuleNode>,
        custom_properties: Option<Arc<::custom_properties::CustomPropertiesMap>>,
        writing_mode: WritingMode,
        font_size_keyword: FontComputationData,
        mut flags: ComputedValueFlags,
        visited_style: Option<Arc<ComputedValues>>,
    ) -> Self {
        debug_assert_eq!(parent_style.is_some(), parent_style_ignoring_first_line.is_some());
        #[cfg(feature = "gecko")]
        debug_assert!(parent_style.is_none() ||
                      ptr::eq(parent_style.unwrap(),
                              parent_style_ignoring_first_line.unwrap()) ||
                      parent_style.unwrap().pseudo() == Some(PseudoElement::FirstLine));
        let reset_style = device.default_computed_values();
        let inherited_style = parent_style.unwrap_or(reset_style);
        let inherited_style_ignoring_first_line = parent_style_ignoring_first_line.unwrap_or(reset_style);
        // FIXME(bz): INHERIT_ALL seems like a fundamentally broken idea.  I'm
        // 99% sure it should give incorrect behavior for table anonymous box
        // backgrounds, for example.  This code doesn't attempt to make it play
        // nice with inherited_style_ignoring_first_line.
        let reset_style = if cascade_flags.contains(INHERIT_ALL) {
            inherited_style
        } else {
            reset_style
        };

        if cascade_flags.contains(VISITED_DEPENDENT_ONLY) {
            flags.insert(IS_STYLE_IF_VISITED);
        }

        StyleBuilder {
            device,
            parent_style,
            inherited_style,
            inherited_style_ignoring_first_line,
            reset_style,
            pseudo,
            rules,
            modified_reset: false,
            custom_properties,
            writing_mode,
            font_size_keyword,
            flags,
            visited_style,
            background: StyleStructRef::Borrowed(reset_style.background_arc()),
            border: StyleStructRef::Borrowed(reset_style.border_arc()),
            box_: StyleStructRef::Borrowed(reset_style.box_arc()),
            color: StyleStructRef::Borrowed(inherited_style.color_arc()),
            column: StyleStructRef::Borrowed(reset_style.column_arc()),
            counters: StyleStructRef::Borrowed(reset_style.counters_arc()),
            effects: StyleStructRef::Borrowed(reset_style.effects_arc()),
            font: StyleStructRef::Borrowed(inherited_style.font_arc()),
            inheritedbox: StyleStructRef::Borrowed(inherited_style.inheritedbox_arc()),
            inheritedtable: StyleStructRef::Borrowed(inherited_style.inheritedtable_arc()),
            inheritedtext: StyleStructRef::Borrowed(inherited_style.inheritedtext_arc()),
            list: StyleStructRef::Borrowed(inherited_style.list_arc()),
            margin: StyleStructRef::Borrowed(reset_style.margin_arc()),
            outline: StyleStructRef::Borrowed(reset_style.outline_arc()),
            padding: StyleStructRef::Borrowed(reset_style.padding_arc()),
            pointing: StyleStructRef::Borrowed(inherited_style.pointing_arc()),
            position: StyleStructRef::Borrowed(reset_style.position_arc()),
            table: StyleStructRef::Borrowed(reset_style.table_arc()),
            text: StyleStructRef::Borrowed(reset_style.text_arc()),
            ui: StyleStructRef::Borrowed(reset_style.ui_arc()),
            inheritedsvg: StyleStructRef::Borrowed(inherited_style.inheritedsvg_arc()),
            svg: StyleStructRef::Borrowed(reset_style.svg_arc()),
            xul: StyleStructRef::Borrowed(reset_style.xul_arc()),
        }
    }

    /// Whether we're a visited style.
    pub fn is_style_if_visited(&self) -> bool {
        self.flags.contains(IS_STYLE_IF_VISITED)
    }

    /// Creates a StyleBuilder holding only references to the structs of `s`, in
    /// order to create a derived style.
    pub fn for_derived_style(
        device: &'a Device,
        style_to_derive_from: &'a ComputedValues,
        parent_style: Option<&'a ComputedValues>,
        pseudo: Option<&'a PseudoElement>,
    ) -> Self {
        let reset_style = device.default_computed_values();
        let inherited_style = parent_style.unwrap_or(reset_style);
        #[cfg(feature = "gecko")]
        debug_assert!(parent_style.is_none() ||
                      parent_style.unwrap().pseudo() != Some(PseudoElement::FirstLine));
        StyleBuilder {
            device,
            parent_style,
            inherited_style,
            // None of our callers pass in ::first-line parent styles.
            inherited_style_ignoring_first_line: inherited_style,
            reset_style,
            pseudo,
            modified_reset: false,
            rules: None, // FIXME(emilio): Dubious...
            custom_properties: style_to_derive_from.custom_properties(),
            writing_mode: style_to_derive_from.writing_mode,
            font_size_keyword: style_to_derive_from.font_computation_data,
            flags: style_to_derive_from.flags,
            visited_style: style_to_derive_from.clone_visited_style(),
            background: StyleStructRef::Borrowed(
                style_to_derive_from.background_arc()
            ),
            border: StyleStructRef::Borrowed(
                style_to_derive_from.border_arc()
            ),
            box_: StyleStructRef::Borrowed(
                style_to_derive_from.box_arc()
            ),
            color: StyleStructRef::Borrowed(
                style_to_derive_from.color_arc()
            ),
            column: StyleStructRef::Borrowed(
                style_to_derive_from.column_arc()
            ),
            counters: StyleStructRef::Borrowed(
                style_to_derive_from.counters_arc()
            ),
            effects: StyleStructRef::Borrowed(
                style_to_derive_from.effects_arc()
            ),
            font: StyleStructRef::Borrowed(
                style_to_derive_from.font_arc()
            ),
            inheritedbox: StyleStructRef::Borrowed(
                style_to_derive_from.inheritedbox_arc()
            ),
            inheritedtable: StyleStructRef::Borrowed(
                style_to_derive_from.inheritedtable_arc()
            ),
            inheritedtext: StyleStructRef::Borrowed(
                style_to_derive_from.inheritedtext_arc()
            ),
            list: StyleStructRef::Borrowed(
                style_to_derive_from.list_arc()
            ),
            margin: StyleStructRef::Borrowed(
                style_to_derive_from.margin_arc()
            ),
            outline: StyleStructRef::Borrowed(
                style_to_derive_from.outline_arc()
            ),
            padding: StyleStructRef::Borrowed(
                style_to_derive_from.padding_arc()
            ),
            pointing: StyleStructRef::Borrowed(
                style_to_derive_from.pointing_arc()
            ),
            position: StyleStructRef::Borrowed(
                style_to_derive_from.position_arc()
            ),
            table: StyleStructRef::Borrowed(
                style_to_derive_from.table_arc()
            ),
            text: StyleStructRef::Borrowed(
                style_to_derive_from.text_arc()
            ),
            ui: StyleStructRef::Borrowed(
                style_to_derive_from.ui_arc()
            ),
            inheritedsvg: StyleStructRef::Borrowed(
                style_to_derive_from.inheritedsvg_arc()
            ),
            svg: StyleStructRef::Borrowed(
                style_to_derive_from.svg_arc()
            ),
            xul: StyleStructRef::Borrowed(
                style_to_derive_from.xul_arc()
            ),
        }
    }

    /// Copy the reset properties from `style`.
    pub fn copy_reset_from(&mut self, style: &'a ComputedValues) {
        self.background =
            StyleStructRef::Borrowed(style.background_arc());
        self.border =
            StyleStructRef::Borrowed(style.border_arc());
        self.box_ =
            StyleStructRef::Borrowed(style.box_arc());
        self.column =
            StyleStructRef::Borrowed(style.column_arc());
        self.counters =
            StyleStructRef::Borrowed(style.counters_arc());
        self.effects =
            StyleStructRef::Borrowed(style.effects_arc());
        self.margin =
            StyleStructRef::Borrowed(style.margin_arc());
        self.outline =
            StyleStructRef::Borrowed(style.outline_arc());
        self.padding =
            StyleStructRef::Borrowed(style.padding_arc());
        self.position =
            StyleStructRef::Borrowed(style.position_arc());
        self.table =
            StyleStructRef::Borrowed(style.table_arc());
        self.text =
            StyleStructRef::Borrowed(style.text_arc());
        self.ui =
            StyleStructRef::Borrowed(style.ui_arc());
        self.svg =
            StyleStructRef::Borrowed(style.svg_arc());
        self.xul =
            StyleStructRef::Borrowed(style.xul_arc());
    }

    /// Inherit `background_color` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_background_color(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_background();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.background.mutate()
            .copy_background_color_from(
                inherited_struct,
            );
    }

    /// Reset `background_color` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_background_color(&mut self) {
        let reset_struct =
            self.reset_style.get_background();

        self.modified_reset = true;

        self.background.mutate()
            .reset_background_color(
                reset_struct,
            );
    }

    /// Set the `background_color` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_background_color(
        &mut self,
        value: longhands::background_color::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.background.mutate()
            .set_background_color(
                value,
            );
    }
    /// Inherit `background_image` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_background_image(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_background();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.background.mutate()
            .copy_background_image_from(
                inherited_struct,
            );
    }

    /// Reset `background_image` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_background_image(&mut self) {
        let reset_struct =
            self.reset_style.get_background();

        self.modified_reset = true;

        self.background.mutate()
            .reset_background_image(
                reset_struct,
            );
    }

    /// Inherit `background_position_x` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_background_position_x(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_background();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.background.mutate()
            .copy_background_position_x_from(
                inherited_struct,
            );
    }

    /// Reset `background_position_x` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_background_position_x(&mut self) {
        let reset_struct =
            self.reset_style.get_background();

        self.modified_reset = true;

        self.background.mutate()
            .reset_background_position_x(
                reset_struct,
            );
    }

    /// Inherit `background_position_y` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_background_position_y(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_background();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.background.mutate()
            .copy_background_position_y_from(
                inherited_struct,
            );
    }

    /// Reset `background_position_y` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_background_position_y(&mut self) {
        let reset_struct =
            self.reset_style.get_background();

        self.modified_reset = true;

        self.background.mutate()
            .reset_background_position_y(
                reset_struct,
            );
    }

    /// Inherit `background_repeat` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_background_repeat(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_background();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.background.mutate()
            .copy_background_repeat_from(
                inherited_struct,
            );
    }

    /// Reset `background_repeat` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_background_repeat(&mut self) {
        let reset_struct =
            self.reset_style.get_background();

        self.modified_reset = true;

        self.background.mutate()
            .reset_background_repeat(
                reset_struct,
            );
    }

    /// Inherit `background_attachment` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_background_attachment(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_background();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.background.mutate()
            .copy_background_attachment_from(
                inherited_struct,
            );
    }

    /// Reset `background_attachment` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_background_attachment(&mut self) {
        let reset_struct =
            self.reset_style.get_background();

        self.modified_reset = true;

        self.background.mutate()
            .reset_background_attachment(
                reset_struct,
            );
    }

    /// Inherit `background_clip` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_background_clip(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_background();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.background.mutate()
            .copy_background_clip_from(
                inherited_struct,
            );
    }

    /// Reset `background_clip` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_background_clip(&mut self) {
        let reset_struct =
            self.reset_style.get_background();

        self.modified_reset = true;

        self.background.mutate()
            .reset_background_clip(
                reset_struct,
            );
    }

    /// Inherit `background_origin` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_background_origin(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_background();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.background.mutate()
            .copy_background_origin_from(
                inherited_struct,
            );
    }

    /// Reset `background_origin` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_background_origin(&mut self) {
        let reset_struct =
            self.reset_style.get_background();

        self.modified_reset = true;

        self.background.mutate()
            .reset_background_origin(
                reset_struct,
            );
    }

    /// Inherit `background_size` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_background_size(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_background();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.background.mutate()
            .copy_background_size_from(
                inherited_struct,
            );
    }

    /// Reset `background_size` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_background_size(&mut self) {
        let reset_struct =
            self.reset_style.get_background();

        self.modified_reset = true;

        self.background.mutate()
            .reset_background_size(
                reset_struct,
            );
    }

    /// Inherit `background_blend_mode` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_background_blend_mode(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_background();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.background.mutate()
            .copy_background_blend_mode_from(
                inherited_struct,
            );
    }

    /// Reset `background_blend_mode` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_background_blend_mode(&mut self) {
        let reset_struct =
            self.reset_style.get_background();

        self.modified_reset = true;

        self.background.mutate()
            .reset_background_blend_mode(
                reset_struct,
            );
    }

    /// Inherit `border_top_color` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_border_top_color(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_border();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.border.mutate()
            .copy_border_top_color_from(
                inherited_struct,
            );
    }

    /// Reset `border_top_color` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_border_top_color(&mut self) {
        let reset_struct =
            self.reset_style.get_border();

        self.modified_reset = true;

        self.border.mutate()
            .reset_border_top_color(
                reset_struct,
            );
    }

    /// Set the `border_top_color` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_border_top_color(
        &mut self,
        value: longhands::border_top_color::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.border.mutate()
            .set_border_top_color(
                value,
            );
    }
    /// Inherit `border_top_style` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_border_top_style(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_border();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.border.mutate()
            .copy_border_top_style_from(
                inherited_struct,
            );
    }

    /// Reset `border_top_style` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_border_top_style(&mut self) {
        let reset_struct =
            self.reset_style.get_border();

        self.modified_reset = true;

        self.border.mutate()
            .reset_border_top_style(
                reset_struct,
            );
    }

    /// Set the `border_top_style` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_border_top_style(
        &mut self,
        value: longhands::border_top_style::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.border.mutate()
            .set_border_top_style(
                value,
            );
    }
    /// Inherit `border_top_width` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_border_top_width(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_border();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.border.mutate()
            .copy_border_top_width_from(
                inherited_struct,
            );
    }

    /// Reset `border_top_width` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_border_top_width(&mut self) {
        let reset_struct =
            self.reset_style.get_border();

        self.modified_reset = true;

        self.border.mutate()
            .reset_border_top_width(
                reset_struct,
            );
    }

    /// Set the `border_top_width` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_border_top_width(
        &mut self,
        value: longhands::border_top_width::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.border.mutate()
            .set_border_top_width(
                value,
            );
    }
    /// Inherit `border_left_color` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_border_left_color(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_border();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.border.mutate()
            .copy_border_left_color_from(
                inherited_struct,
            );
    }

    /// Reset `border_left_color` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_border_left_color(&mut self) {
        let reset_struct =
            self.reset_style.get_border();

        self.modified_reset = true;

        self.border.mutate()
            .reset_border_left_color(
                reset_struct,
            );
    }

    /// Set the `border_left_color` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_border_left_color(
        &mut self,
        value: longhands::border_left_color::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.border.mutate()
            .set_border_left_color(
                value,
            );
    }
    /// Inherit `border_left_style` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_border_left_style(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_border();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.border.mutate()
            .copy_border_left_style_from(
                inherited_struct,
            );
    }

    /// Reset `border_left_style` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_border_left_style(&mut self) {
        let reset_struct =
            self.reset_style.get_border();

        self.modified_reset = true;

        self.border.mutate()
            .reset_border_left_style(
                reset_struct,
            );
    }

    /// Set the `border_left_style` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_border_left_style(
        &mut self,
        value: longhands::border_left_style::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.border.mutate()
            .set_border_left_style(
                value,
            );
    }
    /// Inherit `border_left_width` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_border_left_width(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_border();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.border.mutate()
            .copy_border_left_width_from(
                inherited_struct,
            );
    }

    /// Reset `border_left_width` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_border_left_width(&mut self) {
        let reset_struct =
            self.reset_style.get_border();

        self.modified_reset = true;

        self.border.mutate()
            .reset_border_left_width(
                reset_struct,
            );
    }

    /// Set the `border_left_width` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_border_left_width(
        &mut self,
        value: longhands::border_left_width::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.border.mutate()
            .set_border_left_width(
                value,
            );
    }
    /// Inherit `border_bottom_color` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_border_bottom_color(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_border();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.border.mutate()
            .copy_border_bottom_color_from(
                inherited_struct,
            );
    }

    /// Reset `border_bottom_color` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_border_bottom_color(&mut self) {
        let reset_struct =
            self.reset_style.get_border();

        self.modified_reset = true;

        self.border.mutate()
            .reset_border_bottom_color(
                reset_struct,
            );
    }

    /// Set the `border_bottom_color` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_border_bottom_color(
        &mut self,
        value: longhands::border_bottom_color::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.border.mutate()
            .set_border_bottom_color(
                value,
            );
    }
    /// Inherit `border_bottom_style` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_border_bottom_style(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_border();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.border.mutate()
            .copy_border_bottom_style_from(
                inherited_struct,
            );
    }

    /// Reset `border_bottom_style` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_border_bottom_style(&mut self) {
        let reset_struct =
            self.reset_style.get_border();

        self.modified_reset = true;

        self.border.mutate()
            .reset_border_bottom_style(
                reset_struct,
            );
    }

    /// Set the `border_bottom_style` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_border_bottom_style(
        &mut self,
        value: longhands::border_bottom_style::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.border.mutate()
            .set_border_bottom_style(
                value,
            );
    }
    /// Inherit `border_bottom_width` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_border_bottom_width(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_border();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.border.mutate()
            .copy_border_bottom_width_from(
                inherited_struct,
            );
    }

    /// Reset `border_bottom_width` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_border_bottom_width(&mut self) {
        let reset_struct =
            self.reset_style.get_border();

        self.modified_reset = true;

        self.border.mutate()
            .reset_border_bottom_width(
                reset_struct,
            );
    }

    /// Set the `border_bottom_width` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_border_bottom_width(
        &mut self,
        value: longhands::border_bottom_width::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.border.mutate()
            .set_border_bottom_width(
                value,
            );
    }
    /// Inherit `border_right_color` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_border_right_color(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_border();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.border.mutate()
            .copy_border_right_color_from(
                inherited_struct,
            );
    }

    /// Reset `border_right_color` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_border_right_color(&mut self) {
        let reset_struct =
            self.reset_style.get_border();

        self.modified_reset = true;

        self.border.mutate()
            .reset_border_right_color(
                reset_struct,
            );
    }

    /// Set the `border_right_color` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_border_right_color(
        &mut self,
        value: longhands::border_right_color::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.border.mutate()
            .set_border_right_color(
                value,
            );
    }
    /// Inherit `border_right_style` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_border_right_style(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_border();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.border.mutate()
            .copy_border_right_style_from(
                inherited_struct,
            );
    }

    /// Reset `border_right_style` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_border_right_style(&mut self) {
        let reset_struct =
            self.reset_style.get_border();

        self.modified_reset = true;

        self.border.mutate()
            .reset_border_right_style(
                reset_struct,
            );
    }

    /// Set the `border_right_style` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_border_right_style(
        &mut self,
        value: longhands::border_right_style::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.border.mutate()
            .set_border_right_style(
                value,
            );
    }
    /// Inherit `border_right_width` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_border_right_width(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_border();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.border.mutate()
            .copy_border_right_width_from(
                inherited_struct,
            );
    }

    /// Reset `border_right_width` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_border_right_width(&mut self) {
        let reset_struct =
            self.reset_style.get_border();

        self.modified_reset = true;

        self.border.mutate()
            .reset_border_right_width(
                reset_struct,
            );
    }

    /// Set the `border_right_width` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_border_right_width(
        &mut self,
        value: longhands::border_right_width::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.border.mutate()
            .set_border_right_width(
                value,
            );
    }
    /// Inherit `border_block_start_color` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_border_block_start_color(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_border();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.border.mutate()
            .copy_border_block_start_color_from(
                inherited_struct,
                self.writing_mode,
            );
    }

    /// Reset `border_block_start_color` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_border_block_start_color(&mut self) {
        let reset_struct =
            self.reset_style.get_border();

        self.modified_reset = true;

        self.border.mutate()
            .reset_border_block_start_color(
                reset_struct,
                self.writing_mode,
            );
    }

    /// Set the `border_block_start_color` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_border_block_start_color(
        &mut self,
        value: longhands::border_block_start_color::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.border.mutate()
            .set_border_block_start_color(
                value,
                self.writing_mode,
            );
    }
    /// Inherit `border_block_start_style` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_border_block_start_style(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_border();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.border.mutate()
            .copy_border_block_start_style_from(
                inherited_struct,
                self.writing_mode,
            );
    }

    /// Reset `border_block_start_style` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_border_block_start_style(&mut self) {
        let reset_struct =
            self.reset_style.get_border();

        self.modified_reset = true;

        self.border.mutate()
            .reset_border_block_start_style(
                reset_struct,
                self.writing_mode,
            );
    }

    /// Set the `border_block_start_style` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_border_block_start_style(
        &mut self,
        value: longhands::border_block_start_style::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.border.mutate()
            .set_border_block_start_style(
                value,
                self.writing_mode,
            );
    }
    /// Inherit `border_block_start_width` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_border_block_start_width(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_border();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.border.mutate()
            .copy_border_block_start_width_from(
                inherited_struct,
                self.writing_mode,
            );
    }

    /// Reset `border_block_start_width` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_border_block_start_width(&mut self) {
        let reset_struct =
            self.reset_style.get_border();

        self.modified_reset = true;

        self.border.mutate()
            .reset_border_block_start_width(
                reset_struct,
                self.writing_mode,
            );
    }

    /// Set the `border_block_start_width` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_border_block_start_width(
        &mut self,
        value: longhands::border_block_start_width::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.border.mutate()
            .set_border_block_start_width(
                value,
                self.writing_mode,
            );
    }
    /// Inherit `border_block_end_color` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_border_block_end_color(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_border();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.border.mutate()
            .copy_border_block_end_color_from(
                inherited_struct,
                self.writing_mode,
            );
    }

    /// Reset `border_block_end_color` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_border_block_end_color(&mut self) {
        let reset_struct =
            self.reset_style.get_border();

        self.modified_reset = true;

        self.border.mutate()
            .reset_border_block_end_color(
                reset_struct,
                self.writing_mode,
            );
    }

    /// Set the `border_block_end_color` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_border_block_end_color(
        &mut self,
        value: longhands::border_block_end_color::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.border.mutate()
            .set_border_block_end_color(
                value,
                self.writing_mode,
            );
    }
    /// Inherit `border_block_end_style` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_border_block_end_style(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_border();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.border.mutate()
            .copy_border_block_end_style_from(
                inherited_struct,
                self.writing_mode,
            );
    }

    /// Reset `border_block_end_style` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_border_block_end_style(&mut self) {
        let reset_struct =
            self.reset_style.get_border();

        self.modified_reset = true;

        self.border.mutate()
            .reset_border_block_end_style(
                reset_struct,
                self.writing_mode,
            );
    }

    /// Set the `border_block_end_style` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_border_block_end_style(
        &mut self,
        value: longhands::border_block_end_style::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.border.mutate()
            .set_border_block_end_style(
                value,
                self.writing_mode,
            );
    }
    /// Inherit `border_block_end_width` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_border_block_end_width(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_border();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.border.mutate()
            .copy_border_block_end_width_from(
                inherited_struct,
                self.writing_mode,
            );
    }

    /// Reset `border_block_end_width` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_border_block_end_width(&mut self) {
        let reset_struct =
            self.reset_style.get_border();

        self.modified_reset = true;

        self.border.mutate()
            .reset_border_block_end_width(
                reset_struct,
                self.writing_mode,
            );
    }

    /// Set the `border_block_end_width` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_border_block_end_width(
        &mut self,
        value: longhands::border_block_end_width::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.border.mutate()
            .set_border_block_end_width(
                value,
                self.writing_mode,
            );
    }
    /// Inherit `border_inline_start_color` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_border_inline_start_color(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_border();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.border.mutate()
            .copy_border_inline_start_color_from(
                inherited_struct,
                self.writing_mode,
            );
    }

    /// Reset `border_inline_start_color` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_border_inline_start_color(&mut self) {
        let reset_struct =
            self.reset_style.get_border();

        self.modified_reset = true;

        self.border.mutate()
            .reset_border_inline_start_color(
                reset_struct,
                self.writing_mode,
            );
    }

    /// Set the `border_inline_start_color` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_border_inline_start_color(
        &mut self,
        value: longhands::border_inline_start_color::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.border.mutate()
            .set_border_inline_start_color(
                value,
                self.writing_mode,
            );
    }
    /// Inherit `border_inline_start_style` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_border_inline_start_style(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_border();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.border.mutate()
            .copy_border_inline_start_style_from(
                inherited_struct,
                self.writing_mode,
            );
    }

    /// Reset `border_inline_start_style` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_border_inline_start_style(&mut self) {
        let reset_struct =
            self.reset_style.get_border();

        self.modified_reset = true;

        self.border.mutate()
            .reset_border_inline_start_style(
                reset_struct,
                self.writing_mode,
            );
    }

    /// Set the `border_inline_start_style` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_border_inline_start_style(
        &mut self,
        value: longhands::border_inline_start_style::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.border.mutate()
            .set_border_inline_start_style(
                value,
                self.writing_mode,
            );
    }
    /// Inherit `border_inline_start_width` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_border_inline_start_width(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_border();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.border.mutate()
            .copy_border_inline_start_width_from(
                inherited_struct,
                self.writing_mode,
            );
    }

    /// Reset `border_inline_start_width` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_border_inline_start_width(&mut self) {
        let reset_struct =
            self.reset_style.get_border();

        self.modified_reset = true;

        self.border.mutate()
            .reset_border_inline_start_width(
                reset_struct,
                self.writing_mode,
            );
    }

    /// Set the `border_inline_start_width` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_border_inline_start_width(
        &mut self,
        value: longhands::border_inline_start_width::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.border.mutate()
            .set_border_inline_start_width(
                value,
                self.writing_mode,
            );
    }
    /// Inherit `border_inline_end_color` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_border_inline_end_color(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_border();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.border.mutate()
            .copy_border_inline_end_color_from(
                inherited_struct,
                self.writing_mode,
            );
    }

    /// Reset `border_inline_end_color` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_border_inline_end_color(&mut self) {
        let reset_struct =
            self.reset_style.get_border();

        self.modified_reset = true;

        self.border.mutate()
            .reset_border_inline_end_color(
                reset_struct,
                self.writing_mode,
            );
    }

    /// Set the `border_inline_end_color` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_border_inline_end_color(
        &mut self,
        value: longhands::border_inline_end_color::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.border.mutate()
            .set_border_inline_end_color(
                value,
                self.writing_mode,
            );
    }
    /// Inherit `border_inline_end_style` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_border_inline_end_style(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_border();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.border.mutate()
            .copy_border_inline_end_style_from(
                inherited_struct,
                self.writing_mode,
            );
    }

    /// Reset `border_inline_end_style` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_border_inline_end_style(&mut self) {
        let reset_struct =
            self.reset_style.get_border();

        self.modified_reset = true;

        self.border.mutate()
            .reset_border_inline_end_style(
                reset_struct,
                self.writing_mode,
            );
    }

    /// Set the `border_inline_end_style` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_border_inline_end_style(
        &mut self,
        value: longhands::border_inline_end_style::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.border.mutate()
            .set_border_inline_end_style(
                value,
                self.writing_mode,
            );
    }
    /// Inherit `border_inline_end_width` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_border_inline_end_width(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_border();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.border.mutate()
            .copy_border_inline_end_width_from(
                inherited_struct,
                self.writing_mode,
            );
    }

    /// Reset `border_inline_end_width` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_border_inline_end_width(&mut self) {
        let reset_struct =
            self.reset_style.get_border();

        self.modified_reset = true;

        self.border.mutate()
            .reset_border_inline_end_width(
                reset_struct,
                self.writing_mode,
            );
    }

    /// Set the `border_inline_end_width` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_border_inline_end_width(
        &mut self,
        value: longhands::border_inline_end_width::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.border.mutate()
            .set_border_inline_end_width(
                value,
                self.writing_mode,
            );
    }
    /// Inherit `border_top_left_radius` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_border_top_left_radius(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_border();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.border.mutate()
            .copy_border_top_left_radius_from(
                inherited_struct,
            );
    }

    /// Reset `border_top_left_radius` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_border_top_left_radius(&mut self) {
        let reset_struct =
            self.reset_style.get_border();

        self.modified_reset = true;

        self.border.mutate()
            .reset_border_top_left_radius(
                reset_struct,
            );
    }

    /// Set the `border_top_left_radius` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_border_top_left_radius(
        &mut self,
        value: longhands::border_top_left_radius::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.border.mutate()
            .set_border_top_left_radius(
                value,
            );
    }
    /// Inherit `border_top_right_radius` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_border_top_right_radius(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_border();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.border.mutate()
            .copy_border_top_right_radius_from(
                inherited_struct,
            );
    }

    /// Reset `border_top_right_radius` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_border_top_right_radius(&mut self) {
        let reset_struct =
            self.reset_style.get_border();

        self.modified_reset = true;

        self.border.mutate()
            .reset_border_top_right_radius(
                reset_struct,
            );
    }

    /// Set the `border_top_right_radius` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_border_top_right_radius(
        &mut self,
        value: longhands::border_top_right_radius::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.border.mutate()
            .set_border_top_right_radius(
                value,
            );
    }
    /// Inherit `border_bottom_right_radius` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_border_bottom_right_radius(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_border();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.border.mutate()
            .copy_border_bottom_right_radius_from(
                inherited_struct,
            );
    }

    /// Reset `border_bottom_right_radius` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_border_bottom_right_radius(&mut self) {
        let reset_struct =
            self.reset_style.get_border();

        self.modified_reset = true;

        self.border.mutate()
            .reset_border_bottom_right_radius(
                reset_struct,
            );
    }

    /// Set the `border_bottom_right_radius` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_border_bottom_right_radius(
        &mut self,
        value: longhands::border_bottom_right_radius::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.border.mutate()
            .set_border_bottom_right_radius(
                value,
            );
    }
    /// Inherit `border_bottom_left_radius` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_border_bottom_left_radius(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_border();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.border.mutate()
            .copy_border_bottom_left_radius_from(
                inherited_struct,
            );
    }

    /// Reset `border_bottom_left_radius` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_border_bottom_left_radius(&mut self) {
        let reset_struct =
            self.reset_style.get_border();

        self.modified_reset = true;

        self.border.mutate()
            .reset_border_bottom_left_radius(
                reset_struct,
            );
    }

    /// Set the `border_bottom_left_radius` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_border_bottom_left_radius(
        &mut self,
        value: longhands::border_bottom_left_radius::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.border.mutate()
            .set_border_bottom_left_radius(
                value,
            );
    }
    /// Inherit `_moz_border_top_colors` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit__moz_border_top_colors(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_border();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.border.mutate()
            .copy__moz_border_top_colors_from(
                inherited_struct,
            );
    }

    /// Reset `_moz_border_top_colors` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset__moz_border_top_colors(&mut self) {
        let reset_struct =
            self.reset_style.get_border();

        self.modified_reset = true;

        self.border.mutate()
            .reset__moz_border_top_colors(
                reset_struct,
            );
    }

    /// Set the `_moz_border_top_colors` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set__moz_border_top_colors(
        &mut self,
        value: longhands::_moz_border_top_colors::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.border.mutate()
            .set__moz_border_top_colors(
                value,
            );
    }
    /// Inherit `_moz_border_left_colors` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit__moz_border_left_colors(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_border();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.border.mutate()
            .copy__moz_border_left_colors_from(
                inherited_struct,
            );
    }

    /// Reset `_moz_border_left_colors` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset__moz_border_left_colors(&mut self) {
        let reset_struct =
            self.reset_style.get_border();

        self.modified_reset = true;

        self.border.mutate()
            .reset__moz_border_left_colors(
                reset_struct,
            );
    }

    /// Set the `_moz_border_left_colors` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set__moz_border_left_colors(
        &mut self,
        value: longhands::_moz_border_left_colors::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.border.mutate()
            .set__moz_border_left_colors(
                value,
            );
    }
    /// Inherit `_moz_border_bottom_colors` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit__moz_border_bottom_colors(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_border();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.border.mutate()
            .copy__moz_border_bottom_colors_from(
                inherited_struct,
            );
    }

    /// Reset `_moz_border_bottom_colors` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset__moz_border_bottom_colors(&mut self) {
        let reset_struct =
            self.reset_style.get_border();

        self.modified_reset = true;

        self.border.mutate()
            .reset__moz_border_bottom_colors(
                reset_struct,
            );
    }

    /// Set the `_moz_border_bottom_colors` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set__moz_border_bottom_colors(
        &mut self,
        value: longhands::_moz_border_bottom_colors::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.border.mutate()
            .set__moz_border_bottom_colors(
                value,
            );
    }
    /// Inherit `_moz_border_right_colors` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit__moz_border_right_colors(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_border();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.border.mutate()
            .copy__moz_border_right_colors_from(
                inherited_struct,
            );
    }

    /// Reset `_moz_border_right_colors` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset__moz_border_right_colors(&mut self) {
        let reset_struct =
            self.reset_style.get_border();

        self.modified_reset = true;

        self.border.mutate()
            .reset__moz_border_right_colors(
                reset_struct,
            );
    }

    /// Set the `_moz_border_right_colors` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set__moz_border_right_colors(
        &mut self,
        value: longhands::_moz_border_right_colors::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.border.mutate()
            .set__moz_border_right_colors(
                value,
            );
    }
    /// Inherit `box_decoration_break` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_box_decoration_break(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_border();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.border.mutate()
            .copy_box_decoration_break_from(
                inherited_struct,
            );
    }

    /// Reset `box_decoration_break` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_box_decoration_break(&mut self) {
        let reset_struct =
            self.reset_style.get_border();

        self.modified_reset = true;

        self.border.mutate()
            .reset_box_decoration_break(
                reset_struct,
            );
    }

    /// Set the `box_decoration_break` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_box_decoration_break(
        &mut self,
        value: longhands::box_decoration_break::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.border.mutate()
            .set_box_decoration_break(
                value,
            );
    }
    /// Inherit `_moz_float_edge` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit__moz_float_edge(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_border();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.border.mutate()
            .copy__moz_float_edge_from(
                inherited_struct,
            );
    }

    /// Reset `_moz_float_edge` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset__moz_float_edge(&mut self) {
        let reset_struct =
            self.reset_style.get_border();

        self.modified_reset = true;

        self.border.mutate()
            .reset__moz_float_edge(
                reset_struct,
            );
    }

    /// Set the `_moz_float_edge` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set__moz_float_edge(
        &mut self,
        value: longhands::_moz_float_edge::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.border.mutate()
            .set__moz_float_edge(
                value,
            );
    }
    /// Inherit `border_image_source` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_border_image_source(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_border();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.border.mutate()
            .copy_border_image_source_from(
                inherited_struct,
            );
    }

    /// Reset `border_image_source` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_border_image_source(&mut self) {
        let reset_struct =
            self.reset_style.get_border();

        self.modified_reset = true;

        self.border.mutate()
            .reset_border_image_source(
                reset_struct,
            );
    }

    /// Set the `border_image_source` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_border_image_source(
        &mut self,
        value: longhands::border_image_source::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.border.mutate()
            .set_border_image_source(
                value,
            );
    }
    /// Inherit `border_image_outset` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_border_image_outset(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_border();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.border.mutate()
            .copy_border_image_outset_from(
                inherited_struct,
            );
    }

    /// Reset `border_image_outset` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_border_image_outset(&mut self) {
        let reset_struct =
            self.reset_style.get_border();

        self.modified_reset = true;

        self.border.mutate()
            .reset_border_image_outset(
                reset_struct,
            );
    }

    /// Set the `border_image_outset` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_border_image_outset(
        &mut self,
        value: longhands::border_image_outset::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.border.mutate()
            .set_border_image_outset(
                value,
            );
    }
    /// Inherit `border_image_repeat` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_border_image_repeat(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_border();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.border.mutate()
            .copy_border_image_repeat_from(
                inherited_struct,
            );
    }

    /// Reset `border_image_repeat` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_border_image_repeat(&mut self) {
        let reset_struct =
            self.reset_style.get_border();

        self.modified_reset = true;

        self.border.mutate()
            .reset_border_image_repeat(
                reset_struct,
            );
    }

    /// Set the `border_image_repeat` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_border_image_repeat(
        &mut self,
        value: longhands::border_image_repeat::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.border.mutate()
            .set_border_image_repeat(
                value,
            );
    }
    /// Inherit `border_image_width` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_border_image_width(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_border();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.border.mutate()
            .copy_border_image_width_from(
                inherited_struct,
            );
    }

    /// Reset `border_image_width` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_border_image_width(&mut self) {
        let reset_struct =
            self.reset_style.get_border();

        self.modified_reset = true;

        self.border.mutate()
            .reset_border_image_width(
                reset_struct,
            );
    }

    /// Set the `border_image_width` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_border_image_width(
        &mut self,
        value: longhands::border_image_width::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.border.mutate()
            .set_border_image_width(
                value,
            );
    }
    /// Inherit `border_image_slice` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_border_image_slice(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_border();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.border.mutate()
            .copy_border_image_slice_from(
                inherited_struct,
            );
    }

    /// Reset `border_image_slice` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_border_image_slice(&mut self) {
        let reset_struct =
            self.reset_style.get_border();

        self.modified_reset = true;

        self.border.mutate()
            .reset_border_image_slice(
                reset_struct,
            );
    }

    /// Set the `border_image_slice` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_border_image_slice(
        &mut self,
        value: longhands::border_image_slice::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.border.mutate()
            .set_border_image_slice(
                value,
            );
    }
    /// Inherit `display` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_display(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_box();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;


        self.flags.insert(::properties::computed_value_flags::INHERITS_DISPLAY);

        self.box_.mutate()
            .copy_display_from(
                inherited_struct,
            );
    }

    /// Reset `display` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_display(&mut self) {
        let reset_struct =
            self.reset_style.get_box();

        self.modified_reset = true;

        self.box_.mutate()
            .reset_display(
                reset_struct,
            );
    }

    /// Set the `display` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_display(
        &mut self,
        value: longhands::display::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.box_.mutate()
            .set_display(
                value,
            );
    }
    /// Inherit `_moz_top_layer` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit__moz_top_layer(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_box();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.box_.mutate()
            .copy__moz_top_layer_from(
                inherited_struct,
            );
    }

    /// Reset `_moz_top_layer` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset__moz_top_layer(&mut self) {
        let reset_struct =
            self.reset_style.get_box();

        self.modified_reset = true;

        self.box_.mutate()
            .reset__moz_top_layer(
                reset_struct,
            );
    }

    /// Set the `_moz_top_layer` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set__moz_top_layer(
        &mut self,
        value: longhands::_moz_top_layer::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.box_.mutate()
            .set__moz_top_layer(
                value,
            );
    }
    /// Inherit `position` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_position(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_box();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.box_.mutate()
            .copy_position_from(
                inherited_struct,
            );
    }

    /// Reset `position` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_position(&mut self) {
        let reset_struct =
            self.reset_style.get_box();

        self.modified_reset = true;

        self.box_.mutate()
            .reset_position(
                reset_struct,
            );
    }

    /// Set the `position` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_position(
        &mut self,
        value: longhands::position::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.box_.mutate()
            .set_position(
                value,
            );
    }
    /// Inherit `float` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_float(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_box();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.box_.mutate()
            .copy_float_from(
                inherited_struct,
            );
    }

    /// Reset `float` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_float(&mut self) {
        let reset_struct =
            self.reset_style.get_box();

        self.modified_reset = true;

        self.box_.mutate()
            .reset_float(
                reset_struct,
            );
    }

    /// Set the `float` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_float(
        &mut self,
        value: longhands::float::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.box_.mutate()
            .set_float(
                value,
            );
    }
    /// Inherit `clear` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_clear(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_box();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.box_.mutate()
            .copy_clear_from(
                inherited_struct,
            );
    }

    /// Reset `clear` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_clear(&mut self) {
        let reset_struct =
            self.reset_style.get_box();

        self.modified_reset = true;

        self.box_.mutate()
            .reset_clear(
                reset_struct,
            );
    }

    /// Set the `clear` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_clear(
        &mut self,
        value: longhands::clear::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.box_.mutate()
            .set_clear(
                value,
            );
    }
    /// Inherit `vertical_align` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_vertical_align(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_box();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.box_.mutate()
            .copy_vertical_align_from(
                inherited_struct,
            );
    }

    /// Reset `vertical_align` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_vertical_align(&mut self) {
        let reset_struct =
            self.reset_style.get_box();

        self.modified_reset = true;

        self.box_.mutate()
            .reset_vertical_align(
                reset_struct,
            );
    }

    /// Set the `vertical_align` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_vertical_align(
        &mut self,
        value: longhands::vertical_align::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.box_.mutate()
            .set_vertical_align(
                value,
            );
    }
    /// Inherit `overflow_clip_box` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_overflow_clip_box(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_box();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.box_.mutate()
            .copy_overflow_clip_box_from(
                inherited_struct,
            );
    }

    /// Reset `overflow_clip_box` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_overflow_clip_box(&mut self) {
        let reset_struct =
            self.reset_style.get_box();

        self.modified_reset = true;

        self.box_.mutate()
            .reset_overflow_clip_box(
                reset_struct,
            );
    }

    /// Set the `overflow_clip_box` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_overflow_clip_box(
        &mut self,
        value: longhands::overflow_clip_box::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.box_.mutate()
            .set_overflow_clip_box(
                value,
            );
    }
    /// Inherit `overflow_x` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_overflow_x(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_box();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.box_.mutate()
            .copy_overflow_x_from(
                inherited_struct,
            );
    }

    /// Reset `overflow_x` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_overflow_x(&mut self) {
        let reset_struct =
            self.reset_style.get_box();

        self.modified_reset = true;

        self.box_.mutate()
            .reset_overflow_x(
                reset_struct,
            );
    }

    /// Set the `overflow_x` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_overflow_x(
        &mut self,
        value: longhands::overflow_x::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.box_.mutate()
            .set_overflow_x(
                value,
            );
    }
    /// Inherit `overflow_y` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_overflow_y(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_box();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.box_.mutate()
            .copy_overflow_y_from(
                inherited_struct,
            );
    }

    /// Reset `overflow_y` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_overflow_y(&mut self) {
        let reset_struct =
            self.reset_style.get_box();

        self.modified_reset = true;

        self.box_.mutate()
            .reset_overflow_y(
                reset_struct,
            );
    }

    /// Set the `overflow_y` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_overflow_y(
        &mut self,
        value: longhands::overflow_y::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.box_.mutate()
            .set_overflow_y(
                value,
            );
    }
    /// Inherit `transition_duration` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_transition_duration(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_box();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.box_.mutate()
            .copy_transition_duration_from(
                inherited_struct,
            );
    }

    /// Reset `transition_duration` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_transition_duration(&mut self) {
        let reset_struct =
            self.reset_style.get_box();

        self.modified_reset = true;

        self.box_.mutate()
            .reset_transition_duration(
                reset_struct,
            );
    }

    /// Inherit `transition_timing_function` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_transition_timing_function(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_box();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.box_.mutate()
            .copy_transition_timing_function_from(
                inherited_struct,
            );
    }

    /// Reset `transition_timing_function` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_transition_timing_function(&mut self) {
        let reset_struct =
            self.reset_style.get_box();

        self.modified_reset = true;

        self.box_.mutate()
            .reset_transition_timing_function(
                reset_struct,
            );
    }

    /// Inherit `transition_property` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_transition_property(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_box();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.box_.mutate()
            .copy_transition_property_from(
                inherited_struct,
            );
    }

    /// Reset `transition_property` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_transition_property(&mut self) {
        let reset_struct =
            self.reset_style.get_box();

        self.modified_reset = true;

        self.box_.mutate()
            .reset_transition_property(
                reset_struct,
            );
    }

    /// Inherit `transition_delay` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_transition_delay(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_box();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.box_.mutate()
            .copy_transition_delay_from(
                inherited_struct,
            );
    }

    /// Reset `transition_delay` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_transition_delay(&mut self) {
        let reset_struct =
            self.reset_style.get_box();

        self.modified_reset = true;

        self.box_.mutate()
            .reset_transition_delay(
                reset_struct,
            );
    }

    /// Inherit `animation_name` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_animation_name(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_box();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.box_.mutate()
            .copy_animation_name_from(
                inherited_struct,
            );
    }

    /// Reset `animation_name` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_animation_name(&mut self) {
        let reset_struct =
            self.reset_style.get_box();

        self.modified_reset = true;

        self.box_.mutate()
            .reset_animation_name(
                reset_struct,
            );
    }

    /// Inherit `animation_duration` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_animation_duration(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_box();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.box_.mutate()
            .copy_animation_duration_from(
                inherited_struct,
            );
    }

    /// Reset `animation_duration` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_animation_duration(&mut self) {
        let reset_struct =
            self.reset_style.get_box();

        self.modified_reset = true;

        self.box_.mutate()
            .reset_animation_duration(
                reset_struct,
            );
    }

    /// Inherit `animation_timing_function` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_animation_timing_function(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_box();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.box_.mutate()
            .copy_animation_timing_function_from(
                inherited_struct,
            );
    }

    /// Reset `animation_timing_function` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_animation_timing_function(&mut self) {
        let reset_struct =
            self.reset_style.get_box();

        self.modified_reset = true;

        self.box_.mutate()
            .reset_animation_timing_function(
                reset_struct,
            );
    }

    /// Inherit `animation_iteration_count` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_animation_iteration_count(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_box();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.box_.mutate()
            .copy_animation_iteration_count_from(
                inherited_struct,
            );
    }

    /// Reset `animation_iteration_count` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_animation_iteration_count(&mut self) {
        let reset_struct =
            self.reset_style.get_box();

        self.modified_reset = true;

        self.box_.mutate()
            .reset_animation_iteration_count(
                reset_struct,
            );
    }

    /// Inherit `animation_direction` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_animation_direction(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_box();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.box_.mutate()
            .copy_animation_direction_from(
                inherited_struct,
            );
    }

    /// Reset `animation_direction` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_animation_direction(&mut self) {
        let reset_struct =
            self.reset_style.get_box();

        self.modified_reset = true;

        self.box_.mutate()
            .reset_animation_direction(
                reset_struct,
            );
    }

    /// Inherit `animation_play_state` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_animation_play_state(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_box();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.box_.mutate()
            .copy_animation_play_state_from(
                inherited_struct,
            );
    }

    /// Reset `animation_play_state` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_animation_play_state(&mut self) {
        let reset_struct =
            self.reset_style.get_box();

        self.modified_reset = true;

        self.box_.mutate()
            .reset_animation_play_state(
                reset_struct,
            );
    }

    /// Inherit `animation_fill_mode` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_animation_fill_mode(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_box();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.box_.mutate()
            .copy_animation_fill_mode_from(
                inherited_struct,
            );
    }

    /// Reset `animation_fill_mode` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_animation_fill_mode(&mut self) {
        let reset_struct =
            self.reset_style.get_box();

        self.modified_reset = true;

        self.box_.mutate()
            .reset_animation_fill_mode(
                reset_struct,
            );
    }

    /// Inherit `animation_delay` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_animation_delay(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_box();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.box_.mutate()
            .copy_animation_delay_from(
                inherited_struct,
            );
    }

    /// Reset `animation_delay` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_animation_delay(&mut self) {
        let reset_struct =
            self.reset_style.get_box();

        self.modified_reset = true;

        self.box_.mutate()
            .reset_animation_delay(
                reset_struct,
            );
    }

    /// Inherit `scroll_snap_points_x` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_scroll_snap_points_x(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_box();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.box_.mutate()
            .copy_scroll_snap_points_x_from(
                inherited_struct,
            );
    }

    /// Reset `scroll_snap_points_x` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_scroll_snap_points_x(&mut self) {
        let reset_struct =
            self.reset_style.get_box();

        self.modified_reset = true;

        self.box_.mutate()
            .reset_scroll_snap_points_x(
                reset_struct,
            );
    }

    /// Set the `scroll_snap_points_x` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_scroll_snap_points_x(
        &mut self,
        value: longhands::scroll_snap_points_x::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.box_.mutate()
            .set_scroll_snap_points_x(
                value,
            );
    }
    /// Inherit `scroll_snap_points_y` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_scroll_snap_points_y(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_box();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.box_.mutate()
            .copy_scroll_snap_points_y_from(
                inherited_struct,
            );
    }

    /// Reset `scroll_snap_points_y` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_scroll_snap_points_y(&mut self) {
        let reset_struct =
            self.reset_style.get_box();

        self.modified_reset = true;

        self.box_.mutate()
            .reset_scroll_snap_points_y(
                reset_struct,
            );
    }

    /// Set the `scroll_snap_points_y` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_scroll_snap_points_y(
        &mut self,
        value: longhands::scroll_snap_points_y::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.box_.mutate()
            .set_scroll_snap_points_y(
                value,
            );
    }
    /// Inherit `scroll_snap_destination` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_scroll_snap_destination(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_box();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.box_.mutate()
            .copy_scroll_snap_destination_from(
                inherited_struct,
            );
    }

    /// Reset `scroll_snap_destination` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_scroll_snap_destination(&mut self) {
        let reset_struct =
            self.reset_style.get_box();

        self.modified_reset = true;

        self.box_.mutate()
            .reset_scroll_snap_destination(
                reset_struct,
            );
    }

    /// Set the `scroll_snap_destination` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_scroll_snap_destination(
        &mut self,
        value: longhands::scroll_snap_destination::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.box_.mutate()
            .set_scroll_snap_destination(
                value,
            );
    }
    /// Inherit `scroll_snap_coordinate` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_scroll_snap_coordinate(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_box();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.box_.mutate()
            .copy_scroll_snap_coordinate_from(
                inherited_struct,
            );
    }

    /// Reset `scroll_snap_coordinate` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_scroll_snap_coordinate(&mut self) {
        let reset_struct =
            self.reset_style.get_box();

        self.modified_reset = true;

        self.box_.mutate()
            .reset_scroll_snap_coordinate(
                reset_struct,
            );
    }

    /// Inherit `transform` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_transform(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_box();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.box_.mutate()
            .copy_transform_from(
                inherited_struct,
            );
    }

    /// Reset `transform` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_transform(&mut self) {
        let reset_struct =
            self.reset_style.get_box();

        self.modified_reset = true;

        self.box_.mutate()
            .reset_transform(
                reset_struct,
            );
    }

    /// Set the `transform` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_transform(
        &mut self,
        value: longhands::transform::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.box_.mutate()
            .set_transform(
                value,
            );
    }
    /// Inherit `scroll_behavior` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_scroll_behavior(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_box();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.box_.mutate()
            .copy_scroll_behavior_from(
                inherited_struct,
            );
    }

    /// Reset `scroll_behavior` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_scroll_behavior(&mut self) {
        let reset_struct =
            self.reset_style.get_box();

        self.modified_reset = true;

        self.box_.mutate()
            .reset_scroll_behavior(
                reset_struct,
            );
    }

    /// Set the `scroll_behavior` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_scroll_behavior(
        &mut self,
        value: longhands::scroll_behavior::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.box_.mutate()
            .set_scroll_behavior(
                value,
            );
    }
    /// Inherit `scroll_snap_type_x` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_scroll_snap_type_x(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_box();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.box_.mutate()
            .copy_scroll_snap_type_x_from(
                inherited_struct,
            );
    }

    /// Reset `scroll_snap_type_x` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_scroll_snap_type_x(&mut self) {
        let reset_struct =
            self.reset_style.get_box();

        self.modified_reset = true;

        self.box_.mutate()
            .reset_scroll_snap_type_x(
                reset_struct,
            );
    }

    /// Set the `scroll_snap_type_x` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_scroll_snap_type_x(
        &mut self,
        value: longhands::scroll_snap_type_x::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.box_.mutate()
            .set_scroll_snap_type_x(
                value,
            );
    }
    /// Inherit `scroll_snap_type_y` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_scroll_snap_type_y(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_box();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.box_.mutate()
            .copy_scroll_snap_type_y_from(
                inherited_struct,
            );
    }

    /// Reset `scroll_snap_type_y` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_scroll_snap_type_y(&mut self) {
        let reset_struct =
            self.reset_style.get_box();

        self.modified_reset = true;

        self.box_.mutate()
            .reset_scroll_snap_type_y(
                reset_struct,
            );
    }

    /// Set the `scroll_snap_type_y` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_scroll_snap_type_y(
        &mut self,
        value: longhands::scroll_snap_type_y::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.box_.mutate()
            .set_scroll_snap_type_y(
                value,
            );
    }
    /// Inherit `isolation` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_isolation(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_box();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.box_.mutate()
            .copy_isolation_from(
                inherited_struct,
            );
    }

    /// Reset `isolation` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_isolation(&mut self) {
        let reset_struct =
            self.reset_style.get_box();

        self.modified_reset = true;

        self.box_.mutate()
            .reset_isolation(
                reset_struct,
            );
    }

    /// Set the `isolation` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_isolation(
        &mut self,
        value: longhands::isolation::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.box_.mutate()
            .set_isolation(
                value,
            );
    }
    /// Inherit `page_break_after` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_page_break_after(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_box();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.box_.mutate()
            .copy_page_break_after_from(
                inherited_struct,
            );
    }

    /// Reset `page_break_after` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_page_break_after(&mut self) {
        let reset_struct =
            self.reset_style.get_box();

        self.modified_reset = true;

        self.box_.mutate()
            .reset_page_break_after(
                reset_struct,
            );
    }

    /// Set the `page_break_after` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_page_break_after(
        &mut self,
        value: longhands::page_break_after::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.box_.mutate()
            .set_page_break_after(
                value,
            );
    }
    /// Inherit `page_break_before` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_page_break_before(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_box();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.box_.mutate()
            .copy_page_break_before_from(
                inherited_struct,
            );
    }

    /// Reset `page_break_before` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_page_break_before(&mut self) {
        let reset_struct =
            self.reset_style.get_box();

        self.modified_reset = true;

        self.box_.mutate()
            .reset_page_break_before(
                reset_struct,
            );
    }

    /// Set the `page_break_before` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_page_break_before(
        &mut self,
        value: longhands::page_break_before::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.box_.mutate()
            .set_page_break_before(
                value,
            );
    }
    /// Inherit `page_break_inside` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_page_break_inside(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_box();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.box_.mutate()
            .copy_page_break_inside_from(
                inherited_struct,
            );
    }

    /// Reset `page_break_inside` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_page_break_inside(&mut self) {
        let reset_struct =
            self.reset_style.get_box();

        self.modified_reset = true;

        self.box_.mutate()
            .reset_page_break_inside(
                reset_struct,
            );
    }

    /// Set the `page_break_inside` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_page_break_inside(
        &mut self,
        value: longhands::page_break_inside::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.box_.mutate()
            .set_page_break_inside(
                value,
            );
    }
    /// Inherit `resize` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_resize(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_box();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.box_.mutate()
            .copy_resize_from(
                inherited_struct,
            );
    }

    /// Reset `resize` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_resize(&mut self) {
        let reset_struct =
            self.reset_style.get_box();

        self.modified_reset = true;

        self.box_.mutate()
            .reset_resize(
                reset_struct,
            );
    }

    /// Set the `resize` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_resize(
        &mut self,
        value: longhands::resize::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.box_.mutate()
            .set_resize(
                value,
            );
    }
    /// Inherit `perspective` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_perspective(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_box();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.box_.mutate()
            .copy_perspective_from(
                inherited_struct,
            );
    }

    /// Reset `perspective` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_perspective(&mut self) {
        let reset_struct =
            self.reset_style.get_box();

        self.modified_reset = true;

        self.box_.mutate()
            .reset_perspective(
                reset_struct,
            );
    }

    /// Set the `perspective` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_perspective(
        &mut self,
        value: longhands::perspective::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.box_.mutate()
            .set_perspective(
                value,
            );
    }
    /// Inherit `perspective_origin` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_perspective_origin(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_box();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.box_.mutate()
            .copy_perspective_origin_from(
                inherited_struct,
            );
    }

    /// Reset `perspective_origin` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_perspective_origin(&mut self) {
        let reset_struct =
            self.reset_style.get_box();

        self.modified_reset = true;

        self.box_.mutate()
            .reset_perspective_origin(
                reset_struct,
            );
    }

    /// Set the `perspective_origin` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_perspective_origin(
        &mut self,
        value: longhands::perspective_origin::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.box_.mutate()
            .set_perspective_origin(
                value,
            );
    }
    /// Inherit `backface_visibility` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_backface_visibility(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_box();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.box_.mutate()
            .copy_backface_visibility_from(
                inherited_struct,
            );
    }

    /// Reset `backface_visibility` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_backface_visibility(&mut self) {
        let reset_struct =
            self.reset_style.get_box();

        self.modified_reset = true;

        self.box_.mutate()
            .reset_backface_visibility(
                reset_struct,
            );
    }

    /// Set the `backface_visibility` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_backface_visibility(
        &mut self,
        value: longhands::backface_visibility::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.box_.mutate()
            .set_backface_visibility(
                value,
            );
    }
    /// Inherit `transform_box` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_transform_box(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_box();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.box_.mutate()
            .copy_transform_box_from(
                inherited_struct,
            );
    }

    /// Reset `transform_box` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_transform_box(&mut self) {
        let reset_struct =
            self.reset_style.get_box();

        self.modified_reset = true;

        self.box_.mutate()
            .reset_transform_box(
                reset_struct,
            );
    }

    /// Set the `transform_box` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_transform_box(
        &mut self,
        value: longhands::transform_box::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.box_.mutate()
            .set_transform_box(
                value,
            );
    }
    /// Inherit `transform_style` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_transform_style(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_box();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.box_.mutate()
            .copy_transform_style_from(
                inherited_struct,
            );
    }

    /// Reset `transform_style` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_transform_style(&mut self) {
        let reset_struct =
            self.reset_style.get_box();

        self.modified_reset = true;

        self.box_.mutate()
            .reset_transform_style(
                reset_struct,
            );
    }

    /// Set the `transform_style` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_transform_style(
        &mut self,
        value: longhands::transform_style::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.box_.mutate()
            .set_transform_style(
                value,
            );
    }
    /// Inherit `transform_origin` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_transform_origin(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_box();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.box_.mutate()
            .copy_transform_origin_from(
                inherited_struct,
            );
    }

    /// Reset `transform_origin` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_transform_origin(&mut self) {
        let reset_struct =
            self.reset_style.get_box();

        self.modified_reset = true;

        self.box_.mutate()
            .reset_transform_origin(
                reset_struct,
            );
    }

    /// Set the `transform_origin` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_transform_origin(
        &mut self,
        value: longhands::transform_origin::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.box_.mutate()
            .set_transform_origin(
                value,
            );
    }
    /// Inherit `contain` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_contain(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_box();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.box_.mutate()
            .copy_contain_from(
                inherited_struct,
            );
    }

    /// Reset `contain` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_contain(&mut self) {
        let reset_struct =
            self.reset_style.get_box();

        self.modified_reset = true;

        self.box_.mutate()
            .reset_contain(
                reset_struct,
            );
    }

    /// Set the `contain` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_contain(
        &mut self,
        value: longhands::contain::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.box_.mutate()
            .set_contain(
                value,
            );
    }
    /// Inherit `_moz_appearance` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit__moz_appearance(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_box();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.box_.mutate()
            .copy__moz_appearance_from(
                inherited_struct,
            );
    }

    /// Reset `_moz_appearance` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset__moz_appearance(&mut self) {
        let reset_struct =
            self.reset_style.get_box();

        self.modified_reset = true;

        self.box_.mutate()
            .reset__moz_appearance(
                reset_struct,
            );
    }

    /// Set the `_moz_appearance` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set__moz_appearance(
        &mut self,
        value: longhands::_moz_appearance::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.box_.mutate()
            .set__moz_appearance(
                value,
            );
    }
    /// Inherit `_moz_binding` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit__moz_binding(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_box();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.box_.mutate()
            .copy__moz_binding_from(
                inherited_struct,
            );
    }

    /// Reset `_moz_binding` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset__moz_binding(&mut self) {
        let reset_struct =
            self.reset_style.get_box();

        self.modified_reset = true;

        self.box_.mutate()
            .reset__moz_binding(
                reset_struct,
            );
    }

    /// Set the `_moz_binding` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set__moz_binding(
        &mut self,
        value: longhands::_moz_binding::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.box_.mutate()
            .set__moz_binding(
                value,
            );
    }
    /// Inherit `_moz_orient` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit__moz_orient(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_box();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.box_.mutate()
            .copy__moz_orient_from(
                inherited_struct,
            );
    }

    /// Reset `_moz_orient` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset__moz_orient(&mut self) {
        let reset_struct =
            self.reset_style.get_box();

        self.modified_reset = true;

        self.box_.mutate()
            .reset__moz_orient(
                reset_struct,
            );
    }

    /// Set the `_moz_orient` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set__moz_orient(
        &mut self,
        value: longhands::_moz_orient::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.box_.mutate()
            .set__moz_orient(
                value,
            );
    }
    /// Inherit `will_change` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_will_change(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_box();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.box_.mutate()
            .copy_will_change_from(
                inherited_struct,
            );
    }

    /// Reset `will_change` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_will_change(&mut self) {
        let reset_struct =
            self.reset_style.get_box();

        self.modified_reset = true;

        self.box_.mutate()
            .reset_will_change(
                reset_struct,
            );
    }

    /// Set the `will_change` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_will_change(
        &mut self,
        value: longhands::will_change::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.box_.mutate()
            .set_will_change(
                value,
            );
    }
    /// Inherit `shape_outside` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_shape_outside(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_box();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.box_.mutate()
            .copy_shape_outside_from(
                inherited_struct,
            );
    }

    /// Reset `shape_outside` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_shape_outside(&mut self) {
        let reset_struct =
            self.reset_style.get_box();

        self.modified_reset = true;

        self.box_.mutate()
            .reset_shape_outside(
                reset_struct,
            );
    }

    /// Set the `shape_outside` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_shape_outside(
        &mut self,
        value: longhands::shape_outside::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.box_.mutate()
            .set_shape_outside(
                value,
            );
    }
    /// Inherit `touch_action` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_touch_action(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_box();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.box_.mutate()
            .copy_touch_action_from(
                inherited_struct,
            );
    }

    /// Reset `touch_action` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_touch_action(&mut self) {
        let reset_struct =
            self.reset_style.get_box();

        self.modified_reset = true;

        self.box_.mutate()
            .reset_touch_action(
                reset_struct,
            );
    }

    /// Set the `touch_action` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_touch_action(
        &mut self,
        value: longhands::touch_action::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.box_.mutate()
            .set_touch_action(
                value,
            );
    }
    /// Inherit `color` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_color(&mut self) {
        let inherited_struct =
            self.inherited_style.get_color();




        self.color.mutate()
            .copy_color_from(
                inherited_struct,
            );
    }

    /// Reset `color` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_color(&mut self) {
        let reset_struct =
            self.reset_style.get_color();


        self.color.mutate()
            .reset_color(
                reset_struct,
            );
    }

    /// Set the `color` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_color(
        &mut self,
        value: longhands::color::computed_value::T
    ) {

        
        self.color.mutate()
            .set_color(
                value,
            );
    }
    /// Inherit `column_width` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_column_width(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_column();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.column.mutate()
            .copy_column_width_from(
                inherited_struct,
            );
    }

    /// Reset `column_width` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_column_width(&mut self) {
        let reset_struct =
            self.reset_style.get_column();

        self.modified_reset = true;

        self.column.mutate()
            .reset_column_width(
                reset_struct,
            );
    }

    /// Set the `column_width` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_column_width(
        &mut self,
        value: longhands::column_width::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.column.mutate()
            .set_column_width(
                value,
            );
    }
    /// Inherit `column_count` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_column_count(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_column();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.column.mutate()
            .copy_column_count_from(
                inherited_struct,
            );
    }

    /// Reset `column_count` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_column_count(&mut self) {
        let reset_struct =
            self.reset_style.get_column();

        self.modified_reset = true;

        self.column.mutate()
            .reset_column_count(
                reset_struct,
            );
    }

    /// Set the `column_count` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_column_count(
        &mut self,
        value: longhands::column_count::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.column.mutate()
            .set_column_count(
                value,
            );
    }
    /// Inherit `column_gap` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_column_gap(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_column();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.column.mutate()
            .copy_column_gap_from(
                inherited_struct,
            );
    }

    /// Reset `column_gap` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_column_gap(&mut self) {
        let reset_struct =
            self.reset_style.get_column();

        self.modified_reset = true;

        self.column.mutate()
            .reset_column_gap(
                reset_struct,
            );
    }

    /// Set the `column_gap` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_column_gap(
        &mut self,
        value: longhands::column_gap::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.column.mutate()
            .set_column_gap(
                value,
            );
    }
    /// Inherit `column_fill` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_column_fill(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_column();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.column.mutate()
            .copy_column_fill_from(
                inherited_struct,
            );
    }

    /// Reset `column_fill` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_column_fill(&mut self) {
        let reset_struct =
            self.reset_style.get_column();

        self.modified_reset = true;

        self.column.mutate()
            .reset_column_fill(
                reset_struct,
            );
    }

    /// Set the `column_fill` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_column_fill(
        &mut self,
        value: longhands::column_fill::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.column.mutate()
            .set_column_fill(
                value,
            );
    }
    /// Inherit `column_rule_width` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_column_rule_width(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_column();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.column.mutate()
            .copy_column_rule_width_from(
                inherited_struct,
            );
    }

    /// Reset `column_rule_width` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_column_rule_width(&mut self) {
        let reset_struct =
            self.reset_style.get_column();

        self.modified_reset = true;

        self.column.mutate()
            .reset_column_rule_width(
                reset_struct,
            );
    }

    /// Set the `column_rule_width` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_column_rule_width(
        &mut self,
        value: longhands::column_rule_width::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.column.mutate()
            .set_column_rule_width(
                value,
            );
    }
    /// Inherit `column_rule_color` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_column_rule_color(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_column();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.column.mutate()
            .copy_column_rule_color_from(
                inherited_struct,
            );
    }

    /// Reset `column_rule_color` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_column_rule_color(&mut self) {
        let reset_struct =
            self.reset_style.get_column();

        self.modified_reset = true;

        self.column.mutate()
            .reset_column_rule_color(
                reset_struct,
            );
    }

    /// Set the `column_rule_color` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_column_rule_color(
        &mut self,
        value: longhands::column_rule_color::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.column.mutate()
            .set_column_rule_color(
                value,
            );
    }
    /// Inherit `column_span` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_column_span(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_column();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.column.mutate()
            .copy_column_span_from(
                inherited_struct,
            );
    }

    /// Reset `column_span` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_column_span(&mut self) {
        let reset_struct =
            self.reset_style.get_column();

        self.modified_reset = true;

        self.column.mutate()
            .reset_column_span(
                reset_struct,
            );
    }

    /// Set the `column_span` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_column_span(
        &mut self,
        value: longhands::column_span::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.column.mutate()
            .set_column_span(
                value,
            );
    }
    /// Inherit `column_rule_style` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_column_rule_style(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_column();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.column.mutate()
            .copy_column_rule_style_from(
                inherited_struct,
            );
    }

    /// Reset `column_rule_style` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_column_rule_style(&mut self) {
        let reset_struct =
            self.reset_style.get_column();

        self.modified_reset = true;

        self.column.mutate()
            .reset_column_rule_style(
                reset_struct,
            );
    }

    /// Set the `column_rule_style` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_column_rule_style(
        &mut self,
        value: longhands::column_rule_style::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.column.mutate()
            .set_column_rule_style(
                value,
            );
    }
    /// Inherit `content` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_content(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_counters();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;

        self.flags.insert(::properties::computed_value_flags::INHERITS_CONTENT);


        self.counters.mutate()
            .copy_content_from(
                inherited_struct,
            );
    }

    /// Reset `content` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_content(&mut self) {
        let reset_struct =
            self.reset_style.get_counters();

        self.modified_reset = true;

        self.counters.mutate()
            .reset_content(
                reset_struct,
            );
    }

    /// Set the `content` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_content(
        &mut self,
        value: longhands::content::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.counters.mutate()
            .set_content(
                value,
                self.device,
            );
    }
    /// Inherit `counter_increment` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_counter_increment(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_counters();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.counters.mutate()
            .copy_counter_increment_from(
                inherited_struct,
            );
    }

    /// Reset `counter_increment` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_counter_increment(&mut self) {
        let reset_struct =
            self.reset_style.get_counters();

        self.modified_reset = true;

        self.counters.mutate()
            .reset_counter_increment(
                reset_struct,
            );
    }

    /// Set the `counter_increment` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_counter_increment(
        &mut self,
        value: longhands::counter_increment::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.counters.mutate()
            .set_counter_increment(
                value,
            );
    }
    /// Inherit `counter_reset` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_counter_reset(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_counters();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.counters.mutate()
            .copy_counter_reset_from(
                inherited_struct,
            );
    }

    /// Reset `counter_reset` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_counter_reset(&mut self) {
        let reset_struct =
            self.reset_style.get_counters();

        self.modified_reset = true;

        self.counters.mutate()
            .reset_counter_reset(
                reset_struct,
            );
    }

    /// Set the `counter_reset` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_counter_reset(
        &mut self,
        value: longhands::counter_reset::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.counters.mutate()
            .set_counter_reset(
                value,
            );
    }
    /// Inherit `opacity` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_opacity(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_effects();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.effects.mutate()
            .copy_opacity_from(
                inherited_struct,
            );
    }

    /// Reset `opacity` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_opacity(&mut self) {
        let reset_struct =
            self.reset_style.get_effects();

        self.modified_reset = true;

        self.effects.mutate()
            .reset_opacity(
                reset_struct,
            );
    }

    /// Set the `opacity` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_opacity(
        &mut self,
        value: longhands::opacity::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.effects.mutate()
            .set_opacity(
                value,
            );
    }
    /// Inherit `box_shadow` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_box_shadow(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_effects();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.effects.mutate()
            .copy_box_shadow_from(
                inherited_struct,
            );
    }

    /// Reset `box_shadow` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_box_shadow(&mut self) {
        let reset_struct =
            self.reset_style.get_effects();

        self.modified_reset = true;

        self.effects.mutate()
            .reset_box_shadow(
                reset_struct,
            );
    }

    /// Inherit `clip` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_clip(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_effects();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.effects.mutate()
            .copy_clip_from(
                inherited_struct,
            );
    }

    /// Reset `clip` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_clip(&mut self) {
        let reset_struct =
            self.reset_style.get_effects();

        self.modified_reset = true;

        self.effects.mutate()
            .reset_clip(
                reset_struct,
            );
    }

    /// Set the `clip` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_clip(
        &mut self,
        value: longhands::clip::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.effects.mutate()
            .set_clip(
                value,
            );
    }
    /// Inherit `filter` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_filter(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_effects();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.effects.mutate()
            .copy_filter_from(
                inherited_struct,
            );
    }

    /// Reset `filter` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_filter(&mut self) {
        let reset_struct =
            self.reset_style.get_effects();

        self.modified_reset = true;

        self.effects.mutate()
            .reset_filter(
                reset_struct,
            );
    }

    /// Inherit `mix_blend_mode` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_mix_blend_mode(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_effects();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.effects.mutate()
            .copy_mix_blend_mode_from(
                inherited_struct,
            );
    }

    /// Reset `mix_blend_mode` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_mix_blend_mode(&mut self) {
        let reset_struct =
            self.reset_style.get_effects();

        self.modified_reset = true;

        self.effects.mutate()
            .reset_mix_blend_mode(
                reset_struct,
            );
    }

    /// Set the `mix_blend_mode` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_mix_blend_mode(
        &mut self,
        value: longhands::mix_blend_mode::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.effects.mutate()
            .set_mix_blend_mode(
                value,
            );
    }
    /// Inherit `font_family` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_font_family(&mut self) {
        let inherited_struct =
            self.inherited_style.get_font();




        self.font.mutate()
            .copy_font_family_from(
                inherited_struct,
            );
    }

    /// Reset `font_family` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_font_family(&mut self) {
        let reset_struct =
            self.reset_style.get_font();


        self.font.mutate()
            .reset_font_family(
                reset_struct,
            );
    }

    /// Set the `font_family` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_font_family(
        &mut self,
        value: longhands::font_family::computed_value::T
    ) {

        
        self.font.mutate()
            .set_font_family(
                value,
            );
    }
    /// Inherit `font_style` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_font_style(&mut self) {
        let inherited_struct =
            self.inherited_style.get_font();




        self.font.mutate()
            .copy_font_style_from(
                inherited_struct,
            );
    }

    /// Reset `font_style` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_font_style(&mut self) {
        let reset_struct =
            self.reset_style.get_font();


        self.font.mutate()
            .reset_font_style(
                reset_struct,
            );
    }

    /// Set the `font_style` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_font_style(
        &mut self,
        value: longhands::font_style::computed_value::T
    ) {

        
        self.font.mutate()
            .set_font_style(
                value,
            );
    }
    /// Inherit `font_variant_caps` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_font_variant_caps(&mut self) {
        let inherited_struct =
            self.inherited_style.get_font();




        self.font.mutate()
            .copy_font_variant_caps_from(
                inherited_struct,
            );
    }

    /// Reset `font_variant_caps` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_font_variant_caps(&mut self) {
        let reset_struct =
            self.reset_style.get_font();


        self.font.mutate()
            .reset_font_variant_caps(
                reset_struct,
            );
    }

    /// Set the `font_variant_caps` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_font_variant_caps(
        &mut self,
        value: longhands::font_variant_caps::computed_value::T
    ) {

        
        self.font.mutate()
            .set_font_variant_caps(
                value,
            );
    }
    /// Inherit `font_weight` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_font_weight(&mut self) {
        let inherited_struct =
            self.inherited_style.get_font();




        self.font.mutate()
            .copy_font_weight_from(
                inherited_struct,
            );
    }

    /// Reset `font_weight` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_font_weight(&mut self) {
        let reset_struct =
            self.reset_style.get_font();


        self.font.mutate()
            .reset_font_weight(
                reset_struct,
            );
    }

    /// Set the `font_weight` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_font_weight(
        &mut self,
        value: longhands::font_weight::computed_value::T
    ) {

        
        self.font.mutate()
            .set_font_weight(
                value,
            );
    }
    /// Inherit `font_size_adjust` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_font_size_adjust(&mut self) {
        let inherited_struct =
            self.inherited_style.get_font();




        self.font.mutate()
            .copy_font_size_adjust_from(
                inherited_struct,
            );
    }

    /// Reset `font_size_adjust` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_font_size_adjust(&mut self) {
        let reset_struct =
            self.reset_style.get_font();


        self.font.mutate()
            .reset_font_size_adjust(
                reset_struct,
            );
    }

    /// Set the `font_size_adjust` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_font_size_adjust(
        &mut self,
        value: longhands::font_size_adjust::computed_value::T
    ) {

        
        self.font.mutate()
            .set_font_size_adjust(
                value,
            );
    }
    /// Inherit `font_synthesis` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_font_synthesis(&mut self) {
        let inherited_struct =
            self.inherited_style.get_font();




        self.font.mutate()
            .copy_font_synthesis_from(
                inherited_struct,
            );
    }

    /// Reset `font_synthesis` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_font_synthesis(&mut self) {
        let reset_struct =
            self.reset_style.get_font();


        self.font.mutate()
            .reset_font_synthesis(
                reset_struct,
            );
    }

    /// Set the `font_synthesis` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_font_synthesis(
        &mut self,
        value: longhands::font_synthesis::computed_value::T
    ) {

        
        self.font.mutate()
            .set_font_synthesis(
                value,
            );
    }
    /// Inherit `font_stretch` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_font_stretch(&mut self) {
        let inherited_struct =
            self.inherited_style.get_font();




        self.font.mutate()
            .copy_font_stretch_from(
                inherited_struct,
            );
    }

    /// Reset `font_stretch` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_font_stretch(&mut self) {
        let reset_struct =
            self.reset_style.get_font();


        self.font.mutate()
            .reset_font_stretch(
                reset_struct,
            );
    }

    /// Set the `font_stretch` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_font_stretch(
        &mut self,
        value: longhands::font_stretch::computed_value::T
    ) {

        
        self.font.mutate()
            .set_font_stretch(
                value,
            );
    }
    /// Inherit `font_kerning` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_font_kerning(&mut self) {
        let inherited_struct =
            self.inherited_style.get_font();




        self.font.mutate()
            .copy_font_kerning_from(
                inherited_struct,
            );
    }

    /// Reset `font_kerning` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_font_kerning(&mut self) {
        let reset_struct =
            self.reset_style.get_font();


        self.font.mutate()
            .reset_font_kerning(
                reset_struct,
            );
    }

    /// Set the `font_kerning` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_font_kerning(
        &mut self,
        value: longhands::font_kerning::computed_value::T
    ) {

        
        self.font.mutate()
            .set_font_kerning(
                value,
            );
    }
    /// Inherit `font_variant_alternates` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_font_variant_alternates(&mut self) {
        let inherited_struct =
            self.inherited_style.get_font();




        self.font.mutate()
            .copy_font_variant_alternates_from(
                inherited_struct,
            );
    }

    /// Reset `font_variant_alternates` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_font_variant_alternates(&mut self) {
        let reset_struct =
            self.reset_style.get_font();


        self.font.mutate()
            .reset_font_variant_alternates(
                reset_struct,
            );
    }

    /// Set the `font_variant_alternates` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_font_variant_alternates(
        &mut self,
        value: longhands::font_variant_alternates::computed_value::T
    ) {

        
        self.font.mutate()
            .set_font_variant_alternates(
                value,
                self.device,
            );
    }
    /// Inherit `font_variant_east_asian` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_font_variant_east_asian(&mut self) {
        let inherited_struct =
            self.inherited_style.get_font();




        self.font.mutate()
            .copy_font_variant_east_asian_from(
                inherited_struct,
            );
    }

    /// Reset `font_variant_east_asian` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_font_variant_east_asian(&mut self) {
        let reset_struct =
            self.reset_style.get_font();


        self.font.mutate()
            .reset_font_variant_east_asian(
                reset_struct,
            );
    }

    /// Set the `font_variant_east_asian` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_font_variant_east_asian(
        &mut self,
        value: longhands::font_variant_east_asian::computed_value::T
    ) {

        
        self.font.mutate()
            .set_font_variant_east_asian(
                value,
            );
    }
    /// Inherit `font_variant_ligatures` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_font_variant_ligatures(&mut self) {
        let inherited_struct =
            self.inherited_style.get_font();




        self.font.mutate()
            .copy_font_variant_ligatures_from(
                inherited_struct,
            );
    }

    /// Reset `font_variant_ligatures` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_font_variant_ligatures(&mut self) {
        let reset_struct =
            self.reset_style.get_font();


        self.font.mutate()
            .reset_font_variant_ligatures(
                reset_struct,
            );
    }

    /// Set the `font_variant_ligatures` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_font_variant_ligatures(
        &mut self,
        value: longhands::font_variant_ligatures::computed_value::T
    ) {

        
        self.font.mutate()
            .set_font_variant_ligatures(
                value,
            );
    }
    /// Inherit `font_variant_numeric` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_font_variant_numeric(&mut self) {
        let inherited_struct =
            self.inherited_style.get_font();




        self.font.mutate()
            .copy_font_variant_numeric_from(
                inherited_struct,
            );
    }

    /// Reset `font_variant_numeric` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_font_variant_numeric(&mut self) {
        let reset_struct =
            self.reset_style.get_font();


        self.font.mutate()
            .reset_font_variant_numeric(
                reset_struct,
            );
    }

    /// Set the `font_variant_numeric` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_font_variant_numeric(
        &mut self,
        value: longhands::font_variant_numeric::computed_value::T
    ) {

        
        self.font.mutate()
            .set_font_variant_numeric(
                value,
            );
    }
    /// Inherit `font_variant_position` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_font_variant_position(&mut self) {
        let inherited_struct =
            self.inherited_style.get_font();




        self.font.mutate()
            .copy_font_variant_position_from(
                inherited_struct,
            );
    }

    /// Reset `font_variant_position` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_font_variant_position(&mut self) {
        let reset_struct =
            self.reset_style.get_font();


        self.font.mutate()
            .reset_font_variant_position(
                reset_struct,
            );
    }

    /// Set the `font_variant_position` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_font_variant_position(
        &mut self,
        value: longhands::font_variant_position::computed_value::T
    ) {

        
        self.font.mutate()
            .set_font_variant_position(
                value,
            );
    }
    /// Inherit `font_feature_settings` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_font_feature_settings(&mut self) {
        let inherited_struct =
            self.inherited_style.get_font();




        self.font.mutate()
            .copy_font_feature_settings_from(
                inherited_struct,
            );
    }

    /// Reset `font_feature_settings` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_font_feature_settings(&mut self) {
        let reset_struct =
            self.reset_style.get_font();


        self.font.mutate()
            .reset_font_feature_settings(
                reset_struct,
            );
    }

    /// Set the `font_feature_settings` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_font_feature_settings(
        &mut self,
        value: longhands::font_feature_settings::computed_value::T
    ) {

        
        self.font.mutate()
            .set_font_feature_settings(
                value,
            );
    }
    /// Inherit `font_variation_settings` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_font_variation_settings(&mut self) {
        let inherited_struct =
            self.inherited_style.get_font();




        self.font.mutate()
            .copy_font_variation_settings_from(
                inherited_struct,
            );
    }

    /// Reset `font_variation_settings` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_font_variation_settings(&mut self) {
        let reset_struct =
            self.reset_style.get_font();


        self.font.mutate()
            .reset_font_variation_settings(
                reset_struct,
            );
    }

    /// Set the `font_variation_settings` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_font_variation_settings(
        &mut self,
        value: longhands::font_variation_settings::computed_value::T
    ) {

        
        self.font.mutate()
            .set_font_variation_settings(
                value,
            );
    }
    /// Inherit `font_language_override` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_font_language_override(&mut self) {
        let inherited_struct =
            self.inherited_style.get_font();




        self.font.mutate()
            .copy_font_language_override_from(
                inherited_struct,
            );
    }

    /// Reset `font_language_override` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_font_language_override(&mut self) {
        let reset_struct =
            self.reset_style.get_font();


        self.font.mutate()
            .reset_font_language_override(
                reset_struct,
            );
    }

    /// Set the `font_language_override` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_font_language_override(
        &mut self,
        value: longhands::font_language_override::computed_value::T
    ) {

        
        self.font.mutate()
            .set_font_language_override(
                value,
            );
    }
    /// Inherit `_x_lang` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit__x_lang(&mut self) {
        let inherited_struct =
            self.inherited_style.get_font();




        self.font.mutate()
            .copy__x_lang_from(
                inherited_struct,
            );
    }

    /// Reset `_x_lang` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset__x_lang(&mut self) {
        let reset_struct =
            self.reset_style.get_font();


        self.font.mutate()
            .reset__x_lang(
                reset_struct,
            );
    }

    /// Set the `_x_lang` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set__x_lang(
        &mut self,
        value: longhands::_x_lang::computed_value::T
    ) {

        
        self.font.mutate()
            .set__x_lang(
                value,
            );
    }
    /// Inherit `_moz_script_size_multiplier` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit__moz_script_size_multiplier(&mut self) {
        let inherited_struct =
            self.inherited_style.get_font();




        self.font.mutate()
            .copy__moz_script_size_multiplier_from(
                inherited_struct,
            );
    }

    /// Reset `_moz_script_size_multiplier` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset__moz_script_size_multiplier(&mut self) {
        let reset_struct =
            self.reset_style.get_font();


        self.font.mutate()
            .reset__moz_script_size_multiplier(
                reset_struct,
            );
    }

    /// Set the `_moz_script_size_multiplier` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set__moz_script_size_multiplier(
        &mut self,
        value: longhands::_moz_script_size_multiplier::computed_value::T
    ) {

        
        self.font.mutate()
            .set__moz_script_size_multiplier(
                value,
            );
    }
    /// Inherit `_moz_script_level` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit__moz_script_level(&mut self) {
        let inherited_struct =
            self.inherited_style.get_font();




        self.font.mutate()
            .copy__moz_script_level_from(
                inherited_struct,
            );
    }

    /// Reset `_moz_script_level` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset__moz_script_level(&mut self) {
        let reset_struct =
            self.reset_style.get_font();


        self.font.mutate()
            .reset__moz_script_level(
                reset_struct,
            );
    }

    /// Set the `_moz_script_level` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set__moz_script_level(
        &mut self,
        value: longhands::_moz_script_level::computed_value::T
    ) {

        
        self.font.mutate()
            .set__moz_script_level(
                value,
            );
    }
    /// Inherit `_moz_math_display` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit__moz_math_display(&mut self) {
        let inherited_struct =
            self.inherited_style.get_font();




        self.font.mutate()
            .copy__moz_math_display_from(
                inherited_struct,
            );
    }

    /// Reset `_moz_math_display` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset__moz_math_display(&mut self) {
        let reset_struct =
            self.reset_style.get_font();


        self.font.mutate()
            .reset__moz_math_display(
                reset_struct,
            );
    }

    /// Set the `_moz_math_display` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set__moz_math_display(
        &mut self,
        value: longhands::_moz_math_display::computed_value::T
    ) {

        
        self.font.mutate()
            .set__moz_math_display(
                value,
            );
    }
    /// Inherit `_moz_math_variant` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit__moz_math_variant(&mut self) {
        let inherited_struct =
            self.inherited_style.get_font();




        self.font.mutate()
            .copy__moz_math_variant_from(
                inherited_struct,
            );
    }

    /// Reset `_moz_math_variant` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset__moz_math_variant(&mut self) {
        let reset_struct =
            self.reset_style.get_font();


        self.font.mutate()
            .reset__moz_math_variant(
                reset_struct,
            );
    }

    /// Set the `_moz_math_variant` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set__moz_math_variant(
        &mut self,
        value: longhands::_moz_math_variant::computed_value::T
    ) {

        
        self.font.mutate()
            .set__moz_math_variant(
                value,
            );
    }
    /// Inherit `_moz_script_min_size` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit__moz_script_min_size(&mut self) {
        let inherited_struct =
            self.inherited_style.get_font();




        self.font.mutate()
            .copy__moz_script_min_size_from(
                inherited_struct,
            );
    }

    /// Reset `_moz_script_min_size` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset__moz_script_min_size(&mut self) {
        let reset_struct =
            self.reset_style.get_font();


        self.font.mutate()
            .reset__moz_script_min_size(
                reset_struct,
            );
    }

    /// Set the `_moz_script_min_size` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set__moz_script_min_size(
        &mut self,
        value: longhands::_moz_script_min_size::computed_value::T
    ) {

        
        self.font.mutate()
            .set__moz_script_min_size(
                value,
            );
    }
    /// Inherit `_x_text_zoom` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit__x_text_zoom(&mut self) {
        let inherited_struct =
            self.inherited_style.get_font();




        self.font.mutate()
            .copy__x_text_zoom_from(
                inherited_struct,
            );
    }

    /// Reset `_x_text_zoom` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset__x_text_zoom(&mut self) {
        let reset_struct =
            self.reset_style.get_font();


        self.font.mutate()
            .reset__x_text_zoom(
                reset_struct,
            );
    }

    /// Set the `_x_text_zoom` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set__x_text_zoom(
        &mut self,
        value: longhands::_x_text_zoom::computed_value::T
    ) {

        
        self.font.mutate()
            .set__x_text_zoom(
                value,
            );
    }
    /// Inherit `_moz_osx_font_smoothing` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit__moz_osx_font_smoothing(&mut self) {
        let inherited_struct =
            self.inherited_style.get_font();




        self.font.mutate()
            .copy__moz_osx_font_smoothing_from(
                inherited_struct,
            );
    }

    /// Reset `_moz_osx_font_smoothing` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset__moz_osx_font_smoothing(&mut self) {
        let reset_struct =
            self.reset_style.get_font();


        self.font.mutate()
            .reset__moz_osx_font_smoothing(
                reset_struct,
            );
    }

    /// Set the `_moz_osx_font_smoothing` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set__moz_osx_font_smoothing(
        &mut self,
        value: longhands::_moz_osx_font_smoothing::computed_value::T
    ) {

        
        self.font.mutate()
            .set__moz_osx_font_smoothing(
                value,
            );
    }
    /// Inherit `_moz_min_font_size_ratio` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit__moz_min_font_size_ratio(&mut self) {
        let inherited_struct =
            self.inherited_style.get_font();




        self.font.mutate()
            .copy__moz_min_font_size_ratio_from(
                inherited_struct,
            );
    }

    /// Reset `_moz_min_font_size_ratio` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset__moz_min_font_size_ratio(&mut self) {
        let reset_struct =
            self.reset_style.get_font();


        self.font.mutate()
            .reset__moz_min_font_size_ratio(
                reset_struct,
            );
    }

    /// Set the `_moz_min_font_size_ratio` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set__moz_min_font_size_ratio(
        &mut self,
        value: longhands::_moz_min_font_size_ratio::computed_value::T
    ) {

        
        self.font.mutate()
            .set__moz_min_font_size_ratio(
                value,
            );
    }
    /// Inherit `visibility` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_visibility(&mut self) {
        let inherited_struct =
            self.inherited_style.get_inheritedbox();




        self.inheritedbox.mutate()
            .copy_visibility_from(
                inherited_struct,
            );
    }

    /// Reset `visibility` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_visibility(&mut self) {
        let reset_struct =
            self.reset_style.get_inheritedbox();


        self.inheritedbox.mutate()
            .reset_visibility(
                reset_struct,
            );
    }

    /// Set the `visibility` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_visibility(
        &mut self,
        value: longhands::visibility::computed_value::T
    ) {

        
        self.inheritedbox.mutate()
            .set_visibility(
                value,
            );
    }
    /// Inherit `writing_mode` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_writing_mode(&mut self) {
        let inherited_struct =
            self.inherited_style.get_inheritedbox();




        self.inheritedbox.mutate()
            .copy_writing_mode_from(
                inherited_struct,
            );
    }

    /// Reset `writing_mode` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_writing_mode(&mut self) {
        let reset_struct =
            self.reset_style.get_inheritedbox();


        self.inheritedbox.mutate()
            .reset_writing_mode(
                reset_struct,
            );
    }

    /// Set the `writing_mode` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_writing_mode(
        &mut self,
        value: longhands::writing_mode::computed_value::T
    ) {

        
        self.inheritedbox.mutate()
            .set_writing_mode(
                value,
            );
    }
    /// Inherit `direction` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_direction(&mut self) {
        let inherited_struct =
            self.inherited_style.get_inheritedbox();




        self.inheritedbox.mutate()
            .copy_direction_from(
                inherited_struct,
            );
    }

    /// Reset `direction` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_direction(&mut self) {
        let reset_struct =
            self.reset_style.get_inheritedbox();


        self.inheritedbox.mutate()
            .reset_direction(
                reset_struct,
            );
    }

    /// Set the `direction` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_direction(
        &mut self,
        value: longhands::direction::computed_value::T
    ) {

        
        self.inheritedbox.mutate()
            .set_direction(
                value,
            );
    }
    /// Inherit `text_orientation` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_text_orientation(&mut self) {
        let inherited_struct =
            self.inherited_style.get_inheritedbox();




        self.inheritedbox.mutate()
            .copy_text_orientation_from(
                inherited_struct,
            );
    }

    /// Reset `text_orientation` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_text_orientation(&mut self) {
        let reset_struct =
            self.reset_style.get_inheritedbox();


        self.inheritedbox.mutate()
            .reset_text_orientation(
                reset_struct,
            );
    }

    /// Set the `text_orientation` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_text_orientation(
        &mut self,
        value: longhands::text_orientation::computed_value::T
    ) {

        
        self.inheritedbox.mutate()
            .set_text_orientation(
                value,
            );
    }
    /// Inherit `color_adjust` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_color_adjust(&mut self) {
        let inherited_struct =
            self.inherited_style.get_inheritedbox();




        self.inheritedbox.mutate()
            .copy_color_adjust_from(
                inherited_struct,
            );
    }

    /// Reset `color_adjust` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_color_adjust(&mut self) {
        let reset_struct =
            self.reset_style.get_inheritedbox();


        self.inheritedbox.mutate()
            .reset_color_adjust(
                reset_struct,
            );
    }

    /// Set the `color_adjust` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_color_adjust(
        &mut self,
        value: longhands::color_adjust::computed_value::T
    ) {

        
        self.inheritedbox.mutate()
            .set_color_adjust(
                value,
            );
    }
    /// Inherit `image_rendering` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_image_rendering(&mut self) {
        let inherited_struct =
            self.inherited_style.get_inheritedbox();




        self.inheritedbox.mutate()
            .copy_image_rendering_from(
                inherited_struct,
            );
    }

    /// Reset `image_rendering` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_image_rendering(&mut self) {
        let reset_struct =
            self.reset_style.get_inheritedbox();


        self.inheritedbox.mutate()
            .reset_image_rendering(
                reset_struct,
            );
    }

    /// Set the `image_rendering` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_image_rendering(
        &mut self,
        value: longhands::image_rendering::computed_value::T
    ) {

        
        self.inheritedbox.mutate()
            .set_image_rendering(
                value,
            );
    }
    /// Inherit `image_orientation` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_image_orientation(&mut self) {
        let inherited_struct =
            self.inherited_style.get_inheritedbox();




        self.inheritedbox.mutate()
            .copy_image_orientation_from(
                inherited_struct,
            );
    }

    /// Reset `image_orientation` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_image_orientation(&mut self) {
        let reset_struct =
            self.reset_style.get_inheritedbox();


        self.inheritedbox.mutate()
            .reset_image_orientation(
                reset_struct,
            );
    }

    /// Set the `image_orientation` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_image_orientation(
        &mut self,
        value: longhands::image_orientation::computed_value::T
    ) {

        
        self.inheritedbox.mutate()
            .set_image_orientation(
                value,
            );
    }
    /// Inherit `border_collapse` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_border_collapse(&mut self) {
        let inherited_struct =
            self.inherited_style.get_inheritedtable();




        self.inheritedtable.mutate()
            .copy_border_collapse_from(
                inherited_struct,
            );
    }

    /// Reset `border_collapse` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_border_collapse(&mut self) {
        let reset_struct =
            self.reset_style.get_inheritedtable();


        self.inheritedtable.mutate()
            .reset_border_collapse(
                reset_struct,
            );
    }

    /// Set the `border_collapse` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_border_collapse(
        &mut self,
        value: longhands::border_collapse::computed_value::T
    ) {

        
        self.inheritedtable.mutate()
            .set_border_collapse(
                value,
            );
    }
    /// Inherit `empty_cells` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_empty_cells(&mut self) {
        let inherited_struct =
            self.inherited_style.get_inheritedtable();




        self.inheritedtable.mutate()
            .copy_empty_cells_from(
                inherited_struct,
            );
    }

    /// Reset `empty_cells` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_empty_cells(&mut self) {
        let reset_struct =
            self.reset_style.get_inheritedtable();


        self.inheritedtable.mutate()
            .reset_empty_cells(
                reset_struct,
            );
    }

    /// Set the `empty_cells` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_empty_cells(
        &mut self,
        value: longhands::empty_cells::computed_value::T
    ) {

        
        self.inheritedtable.mutate()
            .set_empty_cells(
                value,
            );
    }
    /// Inherit `caption_side` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_caption_side(&mut self) {
        let inherited_struct =
            self.inherited_style.get_inheritedtable();




        self.inheritedtable.mutate()
            .copy_caption_side_from(
                inherited_struct,
            );
    }

    /// Reset `caption_side` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_caption_side(&mut self) {
        let reset_struct =
            self.reset_style.get_inheritedtable();


        self.inheritedtable.mutate()
            .reset_caption_side(
                reset_struct,
            );
    }

    /// Set the `caption_side` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_caption_side(
        &mut self,
        value: longhands::caption_side::computed_value::T
    ) {

        
        self.inheritedtable.mutate()
            .set_caption_side(
                value,
            );
    }
    /// Inherit `border_spacing` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_border_spacing(&mut self) {
        let inherited_struct =
            self.inherited_style.get_inheritedtable();




        self.inheritedtable.mutate()
            .copy_border_spacing_from(
                inherited_struct,
            );
    }

    /// Reset `border_spacing` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_border_spacing(&mut self) {
        let reset_struct =
            self.reset_style.get_inheritedtable();


        self.inheritedtable.mutate()
            .reset_border_spacing(
                reset_struct,
            );
    }

    /// Set the `border_spacing` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_border_spacing(
        &mut self,
        value: longhands::border_spacing::computed_value::T
    ) {

        
        self.inheritedtable.mutate()
            .set_border_spacing(
                value,
            );
    }
    /// Inherit `line_height` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_line_height(&mut self) {
        let inherited_struct =
            self.inherited_style.get_inheritedtext();




        self.inheritedtext.mutate()
            .copy_line_height_from(
                inherited_struct,
            );
    }

    /// Reset `line_height` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_line_height(&mut self) {
        let reset_struct =
            self.reset_style.get_inheritedtext();


        self.inheritedtext.mutate()
            .reset_line_height(
                reset_struct,
            );
    }

    /// Set the `line_height` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_line_height(
        &mut self,
        value: longhands::line_height::computed_value::T
    ) {

        
        self.inheritedtext.mutate()
            .set_line_height(
                value,
            );
    }
    /// Inherit `text_transform` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_text_transform(&mut self) {
        let inherited_struct =
            self.inherited_style.get_inheritedtext();




        self.inheritedtext.mutate()
            .copy_text_transform_from(
                inherited_struct,
            );
    }

    /// Reset `text_transform` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_text_transform(&mut self) {
        let reset_struct =
            self.reset_style.get_inheritedtext();


        self.inheritedtext.mutate()
            .reset_text_transform(
                reset_struct,
            );
    }

    /// Set the `text_transform` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_text_transform(
        &mut self,
        value: longhands::text_transform::computed_value::T
    ) {

        
        self.inheritedtext.mutate()
            .set_text_transform(
                value,
            );
    }
    /// Inherit `hyphens` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_hyphens(&mut self) {
        let inherited_struct =
            self.inherited_style.get_inheritedtext();




        self.inheritedtext.mutate()
            .copy_hyphens_from(
                inherited_struct,
            );
    }

    /// Reset `hyphens` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_hyphens(&mut self) {
        let reset_struct =
            self.reset_style.get_inheritedtext();


        self.inheritedtext.mutate()
            .reset_hyphens(
                reset_struct,
            );
    }

    /// Set the `hyphens` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_hyphens(
        &mut self,
        value: longhands::hyphens::computed_value::T
    ) {

        
        self.inheritedtext.mutate()
            .set_hyphens(
                value,
            );
    }
    /// Inherit `_moz_text_size_adjust` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit__moz_text_size_adjust(&mut self) {
        let inherited_struct =
            self.inherited_style.get_inheritedtext();




        self.inheritedtext.mutate()
            .copy__moz_text_size_adjust_from(
                inherited_struct,
            );
    }

    /// Reset `_moz_text_size_adjust` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset__moz_text_size_adjust(&mut self) {
        let reset_struct =
            self.reset_style.get_inheritedtext();


        self.inheritedtext.mutate()
            .reset__moz_text_size_adjust(
                reset_struct,
            );
    }

    /// Set the `_moz_text_size_adjust` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set__moz_text_size_adjust(
        &mut self,
        value: longhands::_moz_text_size_adjust::computed_value::T
    ) {

        
        self.inheritedtext.mutate()
            .set__moz_text_size_adjust(
                value,
            );
    }
    /// Inherit `text_indent` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_text_indent(&mut self) {
        let inherited_struct =
            self.inherited_style.get_inheritedtext();




        self.inheritedtext.mutate()
            .copy_text_indent_from(
                inherited_struct,
            );
    }

    /// Reset `text_indent` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_text_indent(&mut self) {
        let reset_struct =
            self.reset_style.get_inheritedtext();


        self.inheritedtext.mutate()
            .reset_text_indent(
                reset_struct,
            );
    }

    /// Set the `text_indent` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_text_indent(
        &mut self,
        value: longhands::text_indent::computed_value::T
    ) {

        
        self.inheritedtext.mutate()
            .set_text_indent(
                value,
            );
    }
    /// Inherit `overflow_wrap` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_overflow_wrap(&mut self) {
        let inherited_struct =
            self.inherited_style.get_inheritedtext();




        self.inheritedtext.mutate()
            .copy_overflow_wrap_from(
                inherited_struct,
            );
    }

    /// Reset `overflow_wrap` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_overflow_wrap(&mut self) {
        let reset_struct =
            self.reset_style.get_inheritedtext();


        self.inheritedtext.mutate()
            .reset_overflow_wrap(
                reset_struct,
            );
    }

    /// Set the `overflow_wrap` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_overflow_wrap(
        &mut self,
        value: longhands::overflow_wrap::computed_value::T
    ) {

        
        self.inheritedtext.mutate()
            .set_overflow_wrap(
                value,
            );
    }
    /// Inherit `word_break` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_word_break(&mut self) {
        let inherited_struct =
            self.inherited_style.get_inheritedtext();




        self.inheritedtext.mutate()
            .copy_word_break_from(
                inherited_struct,
            );
    }

    /// Reset `word_break` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_word_break(&mut self) {
        let reset_struct =
            self.reset_style.get_inheritedtext();


        self.inheritedtext.mutate()
            .reset_word_break(
                reset_struct,
            );
    }

    /// Set the `word_break` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_word_break(
        &mut self,
        value: longhands::word_break::computed_value::T
    ) {

        
        self.inheritedtext.mutate()
            .set_word_break(
                value,
            );
    }
    /// Inherit `text_justify` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_text_justify(&mut self) {
        let inherited_struct =
            self.inherited_style.get_inheritedtext();




        self.inheritedtext.mutate()
            .copy_text_justify_from(
                inherited_struct,
            );
    }

    /// Reset `text_justify` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_text_justify(&mut self) {
        let reset_struct =
            self.reset_style.get_inheritedtext();


        self.inheritedtext.mutate()
            .reset_text_justify(
                reset_struct,
            );
    }

    /// Set the `text_justify` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_text_justify(
        &mut self,
        value: longhands::text_justify::computed_value::T
    ) {

        
        self.inheritedtext.mutate()
            .set_text_justify(
                value,
            );
    }
    /// Inherit `text_align_last` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_text_align_last(&mut self) {
        let inherited_struct =
            self.inherited_style.get_inheritedtext();




        self.inheritedtext.mutate()
            .copy_text_align_last_from(
                inherited_struct,
            );
    }

    /// Reset `text_align_last` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_text_align_last(&mut self) {
        let reset_struct =
            self.reset_style.get_inheritedtext();


        self.inheritedtext.mutate()
            .reset_text_align_last(
                reset_struct,
            );
    }

    /// Set the `text_align_last` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_text_align_last(
        &mut self,
        value: longhands::text_align_last::computed_value::T
    ) {

        
        self.inheritedtext.mutate()
            .set_text_align_last(
                value,
            );
    }
    /// Inherit `text_align` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_text_align(&mut self) {
        let inherited_struct =
            self.inherited_style.get_inheritedtext();




        self.inheritedtext.mutate()
            .copy_text_align_from(
                inherited_struct,
            );
    }

    /// Reset `text_align` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_text_align(&mut self) {
        let reset_struct =
            self.reset_style.get_inheritedtext();


        self.inheritedtext.mutate()
            .reset_text_align(
                reset_struct,
            );
    }

    /// Set the `text_align` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_text_align(
        &mut self,
        value: longhands::text_align::computed_value::T
    ) {

        
        self.inheritedtext.mutate()
            .set_text_align(
                value,
            );
    }
    /// Inherit `letter_spacing` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_letter_spacing(&mut self) {
        let inherited_struct =
            self.inherited_style.get_inheritedtext();




        self.inheritedtext.mutate()
            .copy_letter_spacing_from(
                inherited_struct,
            );
    }

    /// Reset `letter_spacing` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_letter_spacing(&mut self) {
        let reset_struct =
            self.reset_style.get_inheritedtext();


        self.inheritedtext.mutate()
            .reset_letter_spacing(
                reset_struct,
            );
    }

    /// Set the `letter_spacing` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_letter_spacing(
        &mut self,
        value: longhands::letter_spacing::computed_value::T
    ) {

        
        self.inheritedtext.mutate()
            .set_letter_spacing(
                value,
            );
    }
    /// Inherit `word_spacing` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_word_spacing(&mut self) {
        let inherited_struct =
            self.inherited_style.get_inheritedtext();




        self.inheritedtext.mutate()
            .copy_word_spacing_from(
                inherited_struct,
            );
    }

    /// Reset `word_spacing` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_word_spacing(&mut self) {
        let reset_struct =
            self.reset_style.get_inheritedtext();


        self.inheritedtext.mutate()
            .reset_word_spacing(
                reset_struct,
            );
    }

    /// Set the `word_spacing` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_word_spacing(
        &mut self,
        value: longhands::word_spacing::computed_value::T
    ) {

        
        self.inheritedtext.mutate()
            .set_word_spacing(
                value,
            );
    }
    /// Inherit `white_space` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_white_space(&mut self) {
        let inherited_struct =
            self.inherited_style.get_inheritedtext();




        self.inheritedtext.mutate()
            .copy_white_space_from(
                inherited_struct,
            );
    }

    /// Reset `white_space` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_white_space(&mut self) {
        let reset_struct =
            self.reset_style.get_inheritedtext();


        self.inheritedtext.mutate()
            .reset_white_space(
                reset_struct,
            );
    }

    /// Set the `white_space` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_white_space(
        &mut self,
        value: longhands::white_space::computed_value::T
    ) {

        
        self.inheritedtext.mutate()
            .set_white_space(
                value,
            );
    }
    /// Inherit `text_shadow` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_text_shadow(&mut self) {
        let inherited_struct =
            self.inherited_style.get_inheritedtext();




        self.inheritedtext.mutate()
            .copy_text_shadow_from(
                inherited_struct,
            );
    }

    /// Reset `text_shadow` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_text_shadow(&mut self) {
        let reset_struct =
            self.reset_style.get_inheritedtext();


        self.inheritedtext.mutate()
            .reset_text_shadow(
                reset_struct,
            );
    }

    /// Inherit `text_emphasis_style` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_text_emphasis_style(&mut self) {
        let inherited_struct =
            self.inherited_style.get_inheritedtext();




        self.inheritedtext.mutate()
            .copy_text_emphasis_style_from(
                inherited_struct,
            );
    }

    /// Reset `text_emphasis_style` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_text_emphasis_style(&mut self) {
        let reset_struct =
            self.reset_style.get_inheritedtext();


        self.inheritedtext.mutate()
            .reset_text_emphasis_style(
                reset_struct,
            );
    }

    /// Set the `text_emphasis_style` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_text_emphasis_style(
        &mut self,
        value: longhands::text_emphasis_style::computed_value::T
    ) {

        
        self.inheritedtext.mutate()
            .set_text_emphasis_style(
                value,
            );
    }
    /// Inherit `text_emphasis_position` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_text_emphasis_position(&mut self) {
        let inherited_struct =
            self.inherited_style.get_inheritedtext();




        self.inheritedtext.mutate()
            .copy_text_emphasis_position_from(
                inherited_struct,
            );
    }

    /// Reset `text_emphasis_position` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_text_emphasis_position(&mut self) {
        let reset_struct =
            self.reset_style.get_inheritedtext();


        self.inheritedtext.mutate()
            .reset_text_emphasis_position(
                reset_struct,
            );
    }

    /// Set the `text_emphasis_position` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_text_emphasis_position(
        &mut self,
        value: longhands::text_emphasis_position::computed_value::T
    ) {

        
        self.inheritedtext.mutate()
            .set_text_emphasis_position(
                value,
            );
    }
    /// Inherit `text_emphasis_color` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_text_emphasis_color(&mut self) {
        let inherited_struct =
            self.inherited_style.get_inheritedtext();




        self.inheritedtext.mutate()
            .copy_text_emphasis_color_from(
                inherited_struct,
            );
    }

    /// Reset `text_emphasis_color` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_text_emphasis_color(&mut self) {
        let reset_struct =
            self.reset_style.get_inheritedtext();


        self.inheritedtext.mutate()
            .reset_text_emphasis_color(
                reset_struct,
            );
    }

    /// Set the `text_emphasis_color` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_text_emphasis_color(
        &mut self,
        value: longhands::text_emphasis_color::computed_value::T
    ) {

        
        self.inheritedtext.mutate()
            .set_text_emphasis_color(
                value,
            );
    }
    /// Inherit `_moz_tab_size` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit__moz_tab_size(&mut self) {
        let inherited_struct =
            self.inherited_style.get_inheritedtext();




        self.inheritedtext.mutate()
            .copy__moz_tab_size_from(
                inherited_struct,
            );
    }

    /// Reset `_moz_tab_size` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset__moz_tab_size(&mut self) {
        let reset_struct =
            self.reset_style.get_inheritedtext();


        self.inheritedtext.mutate()
            .reset__moz_tab_size(
                reset_struct,
            );
    }

    /// Set the `_moz_tab_size` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set__moz_tab_size(
        &mut self,
        value: longhands::_moz_tab_size::computed_value::T
    ) {

        
        self.inheritedtext.mutate()
            .set__moz_tab_size(
                value,
            );
    }
    /// Inherit `_webkit_text_fill_color` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit__webkit_text_fill_color(&mut self) {
        let inherited_struct =
            self.inherited_style.get_inheritedtext();




        self.inheritedtext.mutate()
            .copy__webkit_text_fill_color_from(
                inherited_struct,
            );
    }

    /// Reset `_webkit_text_fill_color` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset__webkit_text_fill_color(&mut self) {
        let reset_struct =
            self.reset_style.get_inheritedtext();


        self.inheritedtext.mutate()
            .reset__webkit_text_fill_color(
                reset_struct,
            );
    }

    /// Set the `_webkit_text_fill_color` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set__webkit_text_fill_color(
        &mut self,
        value: longhands::_webkit_text_fill_color::computed_value::T
    ) {

        
        self.inheritedtext.mutate()
            .set__webkit_text_fill_color(
                value,
            );
    }
    /// Inherit `_webkit_text_stroke_color` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit__webkit_text_stroke_color(&mut self) {
        let inherited_struct =
            self.inherited_style.get_inheritedtext();




        self.inheritedtext.mutate()
            .copy__webkit_text_stroke_color_from(
                inherited_struct,
            );
    }

    /// Reset `_webkit_text_stroke_color` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset__webkit_text_stroke_color(&mut self) {
        let reset_struct =
            self.reset_style.get_inheritedtext();


        self.inheritedtext.mutate()
            .reset__webkit_text_stroke_color(
                reset_struct,
            );
    }

    /// Set the `_webkit_text_stroke_color` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set__webkit_text_stroke_color(
        &mut self,
        value: longhands::_webkit_text_stroke_color::computed_value::T
    ) {

        
        self.inheritedtext.mutate()
            .set__webkit_text_stroke_color(
                value,
            );
    }
    /// Inherit `_webkit_text_stroke_width` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit__webkit_text_stroke_width(&mut self) {
        let inherited_struct =
            self.inherited_style.get_inheritedtext();




        self.inheritedtext.mutate()
            .copy__webkit_text_stroke_width_from(
                inherited_struct,
            );
    }

    /// Reset `_webkit_text_stroke_width` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset__webkit_text_stroke_width(&mut self) {
        let reset_struct =
            self.reset_style.get_inheritedtext();


        self.inheritedtext.mutate()
            .reset__webkit_text_stroke_width(
                reset_struct,
            );
    }

    /// Set the `_webkit_text_stroke_width` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set__webkit_text_stroke_width(
        &mut self,
        value: longhands::_webkit_text_stroke_width::computed_value::T
    ) {

        
        self.inheritedtext.mutate()
            .set__webkit_text_stroke_width(
                value,
            );
    }
    /// Inherit `ruby_align` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_ruby_align(&mut self) {
        let inherited_struct =
            self.inherited_style.get_inheritedtext();




        self.inheritedtext.mutate()
            .copy_ruby_align_from(
                inherited_struct,
            );
    }

    /// Reset `ruby_align` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_ruby_align(&mut self) {
        let reset_struct =
            self.reset_style.get_inheritedtext();


        self.inheritedtext.mutate()
            .reset_ruby_align(
                reset_struct,
            );
    }

    /// Set the `ruby_align` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_ruby_align(
        &mut self,
        value: longhands::ruby_align::computed_value::T
    ) {

        
        self.inheritedtext.mutate()
            .set_ruby_align(
                value,
            );
    }
    /// Inherit `ruby_position` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_ruby_position(&mut self) {
        let inherited_struct =
            self.inherited_style.get_inheritedtext();




        self.inheritedtext.mutate()
            .copy_ruby_position_from(
                inherited_struct,
            );
    }

    /// Reset `ruby_position` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_ruby_position(&mut self) {
        let reset_struct =
            self.reset_style.get_inheritedtext();


        self.inheritedtext.mutate()
            .reset_ruby_position(
                reset_struct,
            );
    }

    /// Set the `ruby_position` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_ruby_position(
        &mut self,
        value: longhands::ruby_position::computed_value::T
    ) {

        
        self.inheritedtext.mutate()
            .set_ruby_position(
                value,
            );
    }
    /// Inherit `text_combine_upright` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_text_combine_upright(&mut self) {
        let inherited_struct =
            self.inherited_style.get_inheritedtext();




        self.inheritedtext.mutate()
            .copy_text_combine_upright_from(
                inherited_struct,
            );
    }

    /// Reset `text_combine_upright` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_text_combine_upright(&mut self) {
        let reset_struct =
            self.reset_style.get_inheritedtext();


        self.inheritedtext.mutate()
            .reset_text_combine_upright(
                reset_struct,
            );
    }

    /// Set the `text_combine_upright` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_text_combine_upright(
        &mut self,
        value: longhands::text_combine_upright::computed_value::T
    ) {

        
        self.inheritedtext.mutate()
            .set_text_combine_upright(
                value,
            );
    }
    /// Inherit `text_rendering` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_text_rendering(&mut self) {
        let inherited_struct =
            self.inherited_style.get_inheritedtext();




        self.inheritedtext.mutate()
            .copy_text_rendering_from(
                inherited_struct,
            );
    }

    /// Reset `text_rendering` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_text_rendering(&mut self) {
        let reset_struct =
            self.reset_style.get_inheritedtext();


        self.inheritedtext.mutate()
            .reset_text_rendering(
                reset_struct,
            );
    }

    /// Set the `text_rendering` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_text_rendering(
        &mut self,
        value: longhands::text_rendering::computed_value::T
    ) {

        
        self.inheritedtext.mutate()
            .set_text_rendering(
                value,
            );
    }
    /// Inherit `_moz_control_character_visibility` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit__moz_control_character_visibility(&mut self) {
        let inherited_struct =
            self.inherited_style.get_inheritedtext();




        self.inheritedtext.mutate()
            .copy__moz_control_character_visibility_from(
                inherited_struct,
            );
    }

    /// Reset `_moz_control_character_visibility` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset__moz_control_character_visibility(&mut self) {
        let reset_struct =
            self.reset_style.get_inheritedtext();


        self.inheritedtext.mutate()
            .reset__moz_control_character_visibility(
                reset_struct,
            );
    }

    /// Set the `_moz_control_character_visibility` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set__moz_control_character_visibility(
        &mut self,
        value: longhands::_moz_control_character_visibility::computed_value::T
    ) {

        
        self.inheritedtext.mutate()
            .set__moz_control_character_visibility(
                value,
            );
    }
    /// Inherit `list_style_position` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_list_style_position(&mut self) {
        let inherited_struct =
            self.inherited_style.get_list();




        self.list.mutate()
            .copy_list_style_position_from(
                inherited_struct,
            );
    }

    /// Reset `list_style_position` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_list_style_position(&mut self) {
        let reset_struct =
            self.reset_style.get_list();


        self.list.mutate()
            .reset_list_style_position(
                reset_struct,
            );
    }

    /// Set the `list_style_position` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_list_style_position(
        &mut self,
        value: longhands::list_style_position::computed_value::T
    ) {

        
        self.list.mutate()
            .set_list_style_position(
                value,
            );
    }
    /// Inherit `list_style_type` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_list_style_type(&mut self) {
        let inherited_struct =
            self.inherited_style.get_list();




        self.list.mutate()
            .copy_list_style_type_from(
                inherited_struct,
            );
    }

    /// Reset `list_style_type` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_list_style_type(&mut self) {
        let reset_struct =
            self.reset_style.get_list();


        self.list.mutate()
            .reset_list_style_type(
                reset_struct,
            );
    }

    /// Set the `list_style_type` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_list_style_type(
        &mut self,
        value: longhands::list_style_type::computed_value::T
    ) {

        
        self.list.mutate()
            .set_list_style_type(
                value,
                self.device,
            );
    }
    /// Inherit `list_style_image` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_list_style_image(&mut self) {
        let inherited_struct =
            self.inherited_style.get_list();




        self.list.mutate()
            .copy_list_style_image_from(
                inherited_struct,
            );
    }

    /// Reset `list_style_image` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_list_style_image(&mut self) {
        let reset_struct =
            self.reset_style.get_list();


        self.list.mutate()
            .reset_list_style_image(
                reset_struct,
            );
    }

    /// Set the `list_style_image` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_list_style_image(
        &mut self,
        value: longhands::list_style_image::computed_value::T
    ) {

        
        self.list.mutate()
            .set_list_style_image(
                value,
            );
    }
    /// Inherit `quotes` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_quotes(&mut self) {
        let inherited_struct =
            self.inherited_style.get_list();




        self.list.mutate()
            .copy_quotes_from(
                inherited_struct,
            );
    }

    /// Reset `quotes` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_quotes(&mut self) {
        let reset_struct =
            self.reset_style.get_list();


        self.list.mutate()
            .reset_quotes(
                reset_struct,
            );
    }

    /// Set the `quotes` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_quotes(
        &mut self,
        value: longhands::quotes::computed_value::T
    ) {

        
        self.list.mutate()
            .set_quotes(
                value,
            );
    }
    /// Inherit `_moz_image_region` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit__moz_image_region(&mut self) {
        let inherited_struct =
            self.inherited_style.get_list();




        self.list.mutate()
            .copy__moz_image_region_from(
                inherited_struct,
            );
    }

    /// Reset `_moz_image_region` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset__moz_image_region(&mut self) {
        let reset_struct =
            self.reset_style.get_list();


        self.list.mutate()
            .reset__moz_image_region(
                reset_struct,
            );
    }

    /// Set the `_moz_image_region` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set__moz_image_region(
        &mut self,
        value: longhands::_moz_image_region::computed_value::T
    ) {

        
        self.list.mutate()
            .set__moz_image_region(
                value,
            );
    }
    /// Inherit `margin_top` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_margin_top(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_margin();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.margin.mutate()
            .copy_margin_top_from(
                inherited_struct,
            );
    }

    /// Reset `margin_top` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_margin_top(&mut self) {
        let reset_struct =
            self.reset_style.get_margin();

        self.modified_reset = true;

        self.margin.mutate()
            .reset_margin_top(
                reset_struct,
            );
    }

    /// Set the `margin_top` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_margin_top(
        &mut self,
        value: longhands::margin_top::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.margin.mutate()
            .set_margin_top(
                value,
            );
    }
    /// Inherit `margin_left` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_margin_left(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_margin();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.margin.mutate()
            .copy_margin_left_from(
                inherited_struct,
            );
    }

    /// Reset `margin_left` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_margin_left(&mut self) {
        let reset_struct =
            self.reset_style.get_margin();

        self.modified_reset = true;

        self.margin.mutate()
            .reset_margin_left(
                reset_struct,
            );
    }

    /// Set the `margin_left` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_margin_left(
        &mut self,
        value: longhands::margin_left::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.margin.mutate()
            .set_margin_left(
                value,
            );
    }
    /// Inherit `margin_bottom` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_margin_bottom(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_margin();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.margin.mutate()
            .copy_margin_bottom_from(
                inherited_struct,
            );
    }

    /// Reset `margin_bottom` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_margin_bottom(&mut self) {
        let reset_struct =
            self.reset_style.get_margin();

        self.modified_reset = true;

        self.margin.mutate()
            .reset_margin_bottom(
                reset_struct,
            );
    }

    /// Set the `margin_bottom` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_margin_bottom(
        &mut self,
        value: longhands::margin_bottom::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.margin.mutate()
            .set_margin_bottom(
                value,
            );
    }
    /// Inherit `margin_right` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_margin_right(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_margin();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.margin.mutate()
            .copy_margin_right_from(
                inherited_struct,
            );
    }

    /// Reset `margin_right` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_margin_right(&mut self) {
        let reset_struct =
            self.reset_style.get_margin();

        self.modified_reset = true;

        self.margin.mutate()
            .reset_margin_right(
                reset_struct,
            );
    }

    /// Set the `margin_right` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_margin_right(
        &mut self,
        value: longhands::margin_right::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.margin.mutate()
            .set_margin_right(
                value,
            );
    }
    /// Inherit `margin_block_start` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_margin_block_start(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_margin();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.margin.mutate()
            .copy_margin_block_start_from(
                inherited_struct,
                self.writing_mode,
            );
    }

    /// Reset `margin_block_start` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_margin_block_start(&mut self) {
        let reset_struct =
            self.reset_style.get_margin();

        self.modified_reset = true;

        self.margin.mutate()
            .reset_margin_block_start(
                reset_struct,
                self.writing_mode,
            );
    }

    /// Set the `margin_block_start` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_margin_block_start(
        &mut self,
        value: longhands::margin_block_start::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.margin.mutate()
            .set_margin_block_start(
                value,
                self.writing_mode,
            );
    }
    /// Inherit `margin_block_end` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_margin_block_end(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_margin();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.margin.mutate()
            .copy_margin_block_end_from(
                inherited_struct,
                self.writing_mode,
            );
    }

    /// Reset `margin_block_end` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_margin_block_end(&mut self) {
        let reset_struct =
            self.reset_style.get_margin();

        self.modified_reset = true;

        self.margin.mutate()
            .reset_margin_block_end(
                reset_struct,
                self.writing_mode,
            );
    }

    /// Set the `margin_block_end` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_margin_block_end(
        &mut self,
        value: longhands::margin_block_end::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.margin.mutate()
            .set_margin_block_end(
                value,
                self.writing_mode,
            );
    }
    /// Inherit `margin_inline_start` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_margin_inline_start(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_margin();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.margin.mutate()
            .copy_margin_inline_start_from(
                inherited_struct,
                self.writing_mode,
            );
    }

    /// Reset `margin_inline_start` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_margin_inline_start(&mut self) {
        let reset_struct =
            self.reset_style.get_margin();

        self.modified_reset = true;

        self.margin.mutate()
            .reset_margin_inline_start(
                reset_struct,
                self.writing_mode,
            );
    }

    /// Set the `margin_inline_start` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_margin_inline_start(
        &mut self,
        value: longhands::margin_inline_start::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.margin.mutate()
            .set_margin_inline_start(
                value,
                self.writing_mode,
            );
    }
    /// Inherit `margin_inline_end` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_margin_inline_end(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_margin();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.margin.mutate()
            .copy_margin_inline_end_from(
                inherited_struct,
                self.writing_mode,
            );
    }

    /// Reset `margin_inline_end` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_margin_inline_end(&mut self) {
        let reset_struct =
            self.reset_style.get_margin();

        self.modified_reset = true;

        self.margin.mutate()
            .reset_margin_inline_end(
                reset_struct,
                self.writing_mode,
            );
    }

    /// Set the `margin_inline_end` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_margin_inline_end(
        &mut self,
        value: longhands::margin_inline_end::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.margin.mutate()
            .set_margin_inline_end(
                value,
                self.writing_mode,
            );
    }
    /// Inherit `outline_color` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_outline_color(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_outline();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.outline.mutate()
            .copy_outline_color_from(
                inherited_struct,
            );
    }

    /// Reset `outline_color` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_outline_color(&mut self) {
        let reset_struct =
            self.reset_style.get_outline();

        self.modified_reset = true;

        self.outline.mutate()
            .reset_outline_color(
                reset_struct,
            );
    }

    /// Set the `outline_color` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_outline_color(
        &mut self,
        value: longhands::outline_color::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.outline.mutate()
            .set_outline_color(
                value,
            );
    }
    /// Inherit `outline_style` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_outline_style(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_outline();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.outline.mutate()
            .copy_outline_style_from(
                inherited_struct,
            );
    }

    /// Reset `outline_style` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_outline_style(&mut self) {
        let reset_struct =
            self.reset_style.get_outline();

        self.modified_reset = true;

        self.outline.mutate()
            .reset_outline_style(
                reset_struct,
            );
    }

    /// Set the `outline_style` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_outline_style(
        &mut self,
        value: longhands::outline_style::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.outline.mutate()
            .set_outline_style(
                value,
            );
    }
    /// Inherit `outline_width` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_outline_width(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_outline();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.outline.mutate()
            .copy_outline_width_from(
                inherited_struct,
            );
    }

    /// Reset `outline_width` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_outline_width(&mut self) {
        let reset_struct =
            self.reset_style.get_outline();

        self.modified_reset = true;

        self.outline.mutate()
            .reset_outline_width(
                reset_struct,
            );
    }

    /// Set the `outline_width` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_outline_width(
        &mut self,
        value: longhands::outline_width::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.outline.mutate()
            .set_outline_width(
                value,
            );
    }
    /// Inherit `_moz_outline_radius_topleft` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit__moz_outline_radius_topleft(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_outline();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.outline.mutate()
            .copy__moz_outline_radius_topleft_from(
                inherited_struct,
            );
    }

    /// Reset `_moz_outline_radius_topleft` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset__moz_outline_radius_topleft(&mut self) {
        let reset_struct =
            self.reset_style.get_outline();

        self.modified_reset = true;

        self.outline.mutate()
            .reset__moz_outline_radius_topleft(
                reset_struct,
            );
    }

    /// Set the `_moz_outline_radius_topleft` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set__moz_outline_radius_topleft(
        &mut self,
        value: longhands::_moz_outline_radius_topleft::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.outline.mutate()
            .set__moz_outline_radius_topleft(
                value,
            );
    }
    /// Inherit `_moz_outline_radius_topright` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit__moz_outline_radius_topright(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_outline();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.outline.mutate()
            .copy__moz_outline_radius_topright_from(
                inherited_struct,
            );
    }

    /// Reset `_moz_outline_radius_topright` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset__moz_outline_radius_topright(&mut self) {
        let reset_struct =
            self.reset_style.get_outline();

        self.modified_reset = true;

        self.outline.mutate()
            .reset__moz_outline_radius_topright(
                reset_struct,
            );
    }

    /// Set the `_moz_outline_radius_topright` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set__moz_outline_radius_topright(
        &mut self,
        value: longhands::_moz_outline_radius_topright::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.outline.mutate()
            .set__moz_outline_radius_topright(
                value,
            );
    }
    /// Inherit `_moz_outline_radius_bottomright` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit__moz_outline_radius_bottomright(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_outline();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.outline.mutate()
            .copy__moz_outline_radius_bottomright_from(
                inherited_struct,
            );
    }

    /// Reset `_moz_outline_radius_bottomright` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset__moz_outline_radius_bottomright(&mut self) {
        let reset_struct =
            self.reset_style.get_outline();

        self.modified_reset = true;

        self.outline.mutate()
            .reset__moz_outline_radius_bottomright(
                reset_struct,
            );
    }

    /// Set the `_moz_outline_radius_bottomright` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set__moz_outline_radius_bottomright(
        &mut self,
        value: longhands::_moz_outline_radius_bottomright::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.outline.mutate()
            .set__moz_outline_radius_bottomright(
                value,
            );
    }
    /// Inherit `_moz_outline_radius_bottomleft` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit__moz_outline_radius_bottomleft(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_outline();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.outline.mutate()
            .copy__moz_outline_radius_bottomleft_from(
                inherited_struct,
            );
    }

    /// Reset `_moz_outline_radius_bottomleft` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset__moz_outline_radius_bottomleft(&mut self) {
        let reset_struct =
            self.reset_style.get_outline();

        self.modified_reset = true;

        self.outline.mutate()
            .reset__moz_outline_radius_bottomleft(
                reset_struct,
            );
    }

    /// Set the `_moz_outline_radius_bottomleft` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set__moz_outline_radius_bottomleft(
        &mut self,
        value: longhands::_moz_outline_radius_bottomleft::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.outline.mutate()
            .set__moz_outline_radius_bottomleft(
                value,
            );
    }
    /// Inherit `outline_offset` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_outline_offset(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_outline();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.outline.mutate()
            .copy_outline_offset_from(
                inherited_struct,
            );
    }

    /// Reset `outline_offset` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_outline_offset(&mut self) {
        let reset_struct =
            self.reset_style.get_outline();

        self.modified_reset = true;

        self.outline.mutate()
            .reset_outline_offset(
                reset_struct,
            );
    }

    /// Set the `outline_offset` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_outline_offset(
        &mut self,
        value: longhands::outline_offset::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.outline.mutate()
            .set_outline_offset(
                value,
            );
    }
    /// Inherit `padding_top` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_padding_top(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_padding();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.padding.mutate()
            .copy_padding_top_from(
                inherited_struct,
            );
    }

    /// Reset `padding_top` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_padding_top(&mut self) {
        let reset_struct =
            self.reset_style.get_padding();

        self.modified_reset = true;

        self.padding.mutate()
            .reset_padding_top(
                reset_struct,
            );
    }

    /// Set the `padding_top` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_padding_top(
        &mut self,
        value: longhands::padding_top::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.padding.mutate()
            .set_padding_top(
                value,
            );
    }
    /// Inherit `padding_left` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_padding_left(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_padding();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.padding.mutate()
            .copy_padding_left_from(
                inherited_struct,
            );
    }

    /// Reset `padding_left` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_padding_left(&mut self) {
        let reset_struct =
            self.reset_style.get_padding();

        self.modified_reset = true;

        self.padding.mutate()
            .reset_padding_left(
                reset_struct,
            );
    }

    /// Set the `padding_left` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_padding_left(
        &mut self,
        value: longhands::padding_left::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.padding.mutate()
            .set_padding_left(
                value,
            );
    }
    /// Inherit `padding_bottom` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_padding_bottom(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_padding();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.padding.mutate()
            .copy_padding_bottom_from(
                inherited_struct,
            );
    }

    /// Reset `padding_bottom` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_padding_bottom(&mut self) {
        let reset_struct =
            self.reset_style.get_padding();

        self.modified_reset = true;

        self.padding.mutate()
            .reset_padding_bottom(
                reset_struct,
            );
    }

    /// Set the `padding_bottom` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_padding_bottom(
        &mut self,
        value: longhands::padding_bottom::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.padding.mutate()
            .set_padding_bottom(
                value,
            );
    }
    /// Inherit `padding_right` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_padding_right(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_padding();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.padding.mutate()
            .copy_padding_right_from(
                inherited_struct,
            );
    }

    /// Reset `padding_right` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_padding_right(&mut self) {
        let reset_struct =
            self.reset_style.get_padding();

        self.modified_reset = true;

        self.padding.mutate()
            .reset_padding_right(
                reset_struct,
            );
    }

    /// Set the `padding_right` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_padding_right(
        &mut self,
        value: longhands::padding_right::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.padding.mutate()
            .set_padding_right(
                value,
            );
    }
    /// Inherit `padding_block_start` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_padding_block_start(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_padding();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.padding.mutate()
            .copy_padding_block_start_from(
                inherited_struct,
                self.writing_mode,
            );
    }

    /// Reset `padding_block_start` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_padding_block_start(&mut self) {
        let reset_struct =
            self.reset_style.get_padding();

        self.modified_reset = true;

        self.padding.mutate()
            .reset_padding_block_start(
                reset_struct,
                self.writing_mode,
            );
    }

    /// Set the `padding_block_start` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_padding_block_start(
        &mut self,
        value: longhands::padding_block_start::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.padding.mutate()
            .set_padding_block_start(
                value,
                self.writing_mode,
            );
    }
    /// Inherit `padding_block_end` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_padding_block_end(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_padding();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.padding.mutate()
            .copy_padding_block_end_from(
                inherited_struct,
                self.writing_mode,
            );
    }

    /// Reset `padding_block_end` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_padding_block_end(&mut self) {
        let reset_struct =
            self.reset_style.get_padding();

        self.modified_reset = true;

        self.padding.mutate()
            .reset_padding_block_end(
                reset_struct,
                self.writing_mode,
            );
    }

    /// Set the `padding_block_end` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_padding_block_end(
        &mut self,
        value: longhands::padding_block_end::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.padding.mutate()
            .set_padding_block_end(
                value,
                self.writing_mode,
            );
    }
    /// Inherit `padding_inline_start` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_padding_inline_start(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_padding();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.padding.mutate()
            .copy_padding_inline_start_from(
                inherited_struct,
                self.writing_mode,
            );
    }

    /// Reset `padding_inline_start` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_padding_inline_start(&mut self) {
        let reset_struct =
            self.reset_style.get_padding();

        self.modified_reset = true;

        self.padding.mutate()
            .reset_padding_inline_start(
                reset_struct,
                self.writing_mode,
            );
    }

    /// Set the `padding_inline_start` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_padding_inline_start(
        &mut self,
        value: longhands::padding_inline_start::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.padding.mutate()
            .set_padding_inline_start(
                value,
                self.writing_mode,
            );
    }
    /// Inherit `padding_inline_end` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_padding_inline_end(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_padding();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.padding.mutate()
            .copy_padding_inline_end_from(
                inherited_struct,
                self.writing_mode,
            );
    }

    /// Reset `padding_inline_end` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_padding_inline_end(&mut self) {
        let reset_struct =
            self.reset_style.get_padding();

        self.modified_reset = true;

        self.padding.mutate()
            .reset_padding_inline_end(
                reset_struct,
                self.writing_mode,
            );
    }

    /// Set the `padding_inline_end` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_padding_inline_end(
        &mut self,
        value: longhands::padding_inline_end::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.padding.mutate()
            .set_padding_inline_end(
                value,
                self.writing_mode,
            );
    }
    /// Inherit `cursor` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_cursor(&mut self) {
        let inherited_struct =
            self.inherited_style.get_pointing();




        self.pointing.mutate()
            .copy_cursor_from(
                inherited_struct,
            );
    }

    /// Reset `cursor` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_cursor(&mut self) {
        let reset_struct =
            self.reset_style.get_pointing();


        self.pointing.mutate()
            .reset_cursor(
                reset_struct,
            );
    }

    /// Set the `cursor` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_cursor(
        &mut self,
        value: longhands::cursor::computed_value::T
    ) {

        
        self.pointing.mutate()
            .set_cursor(
                value,
            );
    }
    /// Inherit `pointer_events` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_pointer_events(&mut self) {
        let inherited_struct =
            self.inherited_style.get_pointing();




        self.pointing.mutate()
            .copy_pointer_events_from(
                inherited_struct,
            );
    }

    /// Reset `pointer_events` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_pointer_events(&mut self) {
        let reset_struct =
            self.reset_style.get_pointing();


        self.pointing.mutate()
            .reset_pointer_events(
                reset_struct,
            );
    }

    /// Set the `pointer_events` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_pointer_events(
        &mut self,
        value: longhands::pointer_events::computed_value::T
    ) {

        
        self.pointing.mutate()
            .set_pointer_events(
                value,
            );
    }
    /// Inherit `_moz_user_input` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit__moz_user_input(&mut self) {
        let inherited_struct =
            self.inherited_style.get_pointing();




        self.pointing.mutate()
            .copy__moz_user_input_from(
                inherited_struct,
            );
    }

    /// Reset `_moz_user_input` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset__moz_user_input(&mut self) {
        let reset_struct =
            self.reset_style.get_pointing();


        self.pointing.mutate()
            .reset__moz_user_input(
                reset_struct,
            );
    }

    /// Set the `_moz_user_input` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set__moz_user_input(
        &mut self,
        value: longhands::_moz_user_input::computed_value::T
    ) {

        
        self.pointing.mutate()
            .set__moz_user_input(
                value,
            );
    }
    /// Inherit `_moz_user_modify` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit__moz_user_modify(&mut self) {
        let inherited_struct =
            self.inherited_style.get_pointing();




        self.pointing.mutate()
            .copy__moz_user_modify_from(
                inherited_struct,
            );
    }

    /// Reset `_moz_user_modify` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset__moz_user_modify(&mut self) {
        let reset_struct =
            self.reset_style.get_pointing();


        self.pointing.mutate()
            .reset__moz_user_modify(
                reset_struct,
            );
    }

    /// Set the `_moz_user_modify` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set__moz_user_modify(
        &mut self,
        value: longhands::_moz_user_modify::computed_value::T
    ) {

        
        self.pointing.mutate()
            .set__moz_user_modify(
                value,
            );
    }
    /// Inherit `_moz_user_focus` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit__moz_user_focus(&mut self) {
        let inherited_struct =
            self.inherited_style.get_pointing();




        self.pointing.mutate()
            .copy__moz_user_focus_from(
                inherited_struct,
            );
    }

    /// Reset `_moz_user_focus` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset__moz_user_focus(&mut self) {
        let reset_struct =
            self.reset_style.get_pointing();


        self.pointing.mutate()
            .reset__moz_user_focus(
                reset_struct,
            );
    }

    /// Set the `_moz_user_focus` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set__moz_user_focus(
        &mut self,
        value: longhands::_moz_user_focus::computed_value::T
    ) {

        
        self.pointing.mutate()
            .set__moz_user_focus(
                value,
            );
    }
    /// Inherit `caret_color` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_caret_color(&mut self) {
        let inherited_struct =
            self.inherited_style.get_pointing();




        self.pointing.mutate()
            .copy_caret_color_from(
                inherited_struct,
            );
    }

    /// Reset `caret_color` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_caret_color(&mut self) {
        let reset_struct =
            self.reset_style.get_pointing();


        self.pointing.mutate()
            .reset_caret_color(
                reset_struct,
            );
    }

    /// Set the `caret_color` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_caret_color(
        &mut self,
        value: longhands::caret_color::computed_value::T
    ) {

        
        self.pointing.mutate()
            .set_caret_color(
                value,
            );
    }
    /// Inherit `top` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_top(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_position();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.position.mutate()
            .copy_top_from(
                inherited_struct,
            );
    }

    /// Reset `top` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_top(&mut self) {
        let reset_struct =
            self.reset_style.get_position();

        self.modified_reset = true;

        self.position.mutate()
            .reset_top(
                reset_struct,
            );
    }

    /// Set the `top` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_top(
        &mut self,
        value: longhands::top::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.position.mutate()
            .set_top(
                value,
            );
    }
    /// Inherit `left` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_left(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_position();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.position.mutate()
            .copy_left_from(
                inherited_struct,
            );
    }

    /// Reset `left` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_left(&mut self) {
        let reset_struct =
            self.reset_style.get_position();

        self.modified_reset = true;

        self.position.mutate()
            .reset_left(
                reset_struct,
            );
    }

    /// Set the `left` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_left(
        &mut self,
        value: longhands::left::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.position.mutate()
            .set_left(
                value,
            );
    }
    /// Inherit `bottom` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_bottom(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_position();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.position.mutate()
            .copy_bottom_from(
                inherited_struct,
            );
    }

    /// Reset `bottom` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_bottom(&mut self) {
        let reset_struct =
            self.reset_style.get_position();

        self.modified_reset = true;

        self.position.mutate()
            .reset_bottom(
                reset_struct,
            );
    }

    /// Set the `bottom` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_bottom(
        &mut self,
        value: longhands::bottom::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.position.mutate()
            .set_bottom(
                value,
            );
    }
    /// Inherit `right` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_right(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_position();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.position.mutate()
            .copy_right_from(
                inherited_struct,
            );
    }

    /// Reset `right` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_right(&mut self) {
        let reset_struct =
            self.reset_style.get_position();

        self.modified_reset = true;

        self.position.mutate()
            .reset_right(
                reset_struct,
            );
    }

    /// Set the `right` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_right(
        &mut self,
        value: longhands::right::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.position.mutate()
            .set_right(
                value,
            );
    }
    /// Inherit `offset_block_start` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_offset_block_start(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_position();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.position.mutate()
            .copy_offset_block_start_from(
                inherited_struct,
                self.writing_mode,
            );
    }

    /// Reset `offset_block_start` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_offset_block_start(&mut self) {
        let reset_struct =
            self.reset_style.get_position();

        self.modified_reset = true;

        self.position.mutate()
            .reset_offset_block_start(
                reset_struct,
                self.writing_mode,
            );
    }

    /// Set the `offset_block_start` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_offset_block_start(
        &mut self,
        value: longhands::offset_block_start::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.position.mutate()
            .set_offset_block_start(
                value,
                self.writing_mode,
            );
    }
    /// Inherit `offset_block_end` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_offset_block_end(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_position();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.position.mutate()
            .copy_offset_block_end_from(
                inherited_struct,
                self.writing_mode,
            );
    }

    /// Reset `offset_block_end` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_offset_block_end(&mut self) {
        let reset_struct =
            self.reset_style.get_position();

        self.modified_reset = true;

        self.position.mutate()
            .reset_offset_block_end(
                reset_struct,
                self.writing_mode,
            );
    }

    /// Set the `offset_block_end` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_offset_block_end(
        &mut self,
        value: longhands::offset_block_end::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.position.mutate()
            .set_offset_block_end(
                value,
                self.writing_mode,
            );
    }
    /// Inherit `offset_inline_start` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_offset_inline_start(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_position();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.position.mutate()
            .copy_offset_inline_start_from(
                inherited_struct,
                self.writing_mode,
            );
    }

    /// Reset `offset_inline_start` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_offset_inline_start(&mut self) {
        let reset_struct =
            self.reset_style.get_position();

        self.modified_reset = true;

        self.position.mutate()
            .reset_offset_inline_start(
                reset_struct,
                self.writing_mode,
            );
    }

    /// Set the `offset_inline_start` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_offset_inline_start(
        &mut self,
        value: longhands::offset_inline_start::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.position.mutate()
            .set_offset_inline_start(
                value,
                self.writing_mode,
            );
    }
    /// Inherit `offset_inline_end` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_offset_inline_end(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_position();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.position.mutate()
            .copy_offset_inline_end_from(
                inherited_struct,
                self.writing_mode,
            );
    }

    /// Reset `offset_inline_end` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_offset_inline_end(&mut self) {
        let reset_struct =
            self.reset_style.get_position();

        self.modified_reset = true;

        self.position.mutate()
            .reset_offset_inline_end(
                reset_struct,
                self.writing_mode,
            );
    }

    /// Set the `offset_inline_end` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_offset_inline_end(
        &mut self,
        value: longhands::offset_inline_end::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.position.mutate()
            .set_offset_inline_end(
                value,
                self.writing_mode,
            );
    }
    /// Inherit `z_index` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_z_index(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_position();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.position.mutate()
            .copy_z_index_from(
                inherited_struct,
            );
    }

    /// Reset `z_index` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_z_index(&mut self) {
        let reset_struct =
            self.reset_style.get_position();

        self.modified_reset = true;

        self.position.mutate()
            .reset_z_index(
                reset_struct,
            );
    }

    /// Set the `z_index` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_z_index(
        &mut self,
        value: longhands::z_index::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.position.mutate()
            .set_z_index(
                value,
            );
    }
    /// Inherit `flex_direction` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_flex_direction(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_position();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.position.mutate()
            .copy_flex_direction_from(
                inherited_struct,
            );
    }

    /// Reset `flex_direction` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_flex_direction(&mut self) {
        let reset_struct =
            self.reset_style.get_position();

        self.modified_reset = true;

        self.position.mutate()
            .reset_flex_direction(
                reset_struct,
            );
    }

    /// Set the `flex_direction` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_flex_direction(
        &mut self,
        value: longhands::flex_direction::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.position.mutate()
            .set_flex_direction(
                value,
            );
    }
    /// Inherit `flex_wrap` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_flex_wrap(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_position();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.position.mutate()
            .copy_flex_wrap_from(
                inherited_struct,
            );
    }

    /// Reset `flex_wrap` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_flex_wrap(&mut self) {
        let reset_struct =
            self.reset_style.get_position();

        self.modified_reset = true;

        self.position.mutate()
            .reset_flex_wrap(
                reset_struct,
            );
    }

    /// Set the `flex_wrap` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_flex_wrap(
        &mut self,
        value: longhands::flex_wrap::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.position.mutate()
            .set_flex_wrap(
                value,
            );
    }
    /// Inherit `justify_content` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_justify_content(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_position();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.position.mutate()
            .copy_justify_content_from(
                inherited_struct,
            );
    }

    /// Reset `justify_content` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_justify_content(&mut self) {
        let reset_struct =
            self.reset_style.get_position();

        self.modified_reset = true;

        self.position.mutate()
            .reset_justify_content(
                reset_struct,
            );
    }

    /// Set the `justify_content` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_justify_content(
        &mut self,
        value: longhands::justify_content::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.position.mutate()
            .set_justify_content(
                value,
            );
    }
    /// Inherit `align_content` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_align_content(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_position();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.position.mutate()
            .copy_align_content_from(
                inherited_struct,
            );
    }

    /// Reset `align_content` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_align_content(&mut self) {
        let reset_struct =
            self.reset_style.get_position();

        self.modified_reset = true;

        self.position.mutate()
            .reset_align_content(
                reset_struct,
            );
    }

    /// Set the `align_content` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_align_content(
        &mut self,
        value: longhands::align_content::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.position.mutate()
            .set_align_content(
                value,
            );
    }
    /// Inherit `align_items` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_align_items(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_position();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.position.mutate()
            .copy_align_items_from(
                inherited_struct,
            );
    }

    /// Reset `align_items` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_align_items(&mut self) {
        let reset_struct =
            self.reset_style.get_position();

        self.modified_reset = true;

        self.position.mutate()
            .reset_align_items(
                reset_struct,
            );
    }

    /// Set the `align_items` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_align_items(
        &mut self,
        value: longhands::align_items::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.position.mutate()
            .set_align_items(
                value,
            );
    }
    /// Inherit `justify_items` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_justify_items(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_position();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.position.mutate()
            .copy_justify_items_from(
                inherited_struct,
            );
    }

    /// Reset `justify_items` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_justify_items(&mut self) {
        let reset_struct =
            self.reset_style.get_position();

        self.modified_reset = true;

        self.position.mutate()
            .reset_justify_items(
                reset_struct,
            );
    }

    /// Set the `justify_items` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_justify_items(
        &mut self,
        value: longhands::justify_items::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.position.mutate()
            .set_justify_items(
                value,
            );
    }
    /// Inherit `flex_grow` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_flex_grow(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_position();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.position.mutate()
            .copy_flex_grow_from(
                inherited_struct,
            );
    }

    /// Reset `flex_grow` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_flex_grow(&mut self) {
        let reset_struct =
            self.reset_style.get_position();

        self.modified_reset = true;

        self.position.mutate()
            .reset_flex_grow(
                reset_struct,
            );
    }

    /// Set the `flex_grow` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_flex_grow(
        &mut self,
        value: longhands::flex_grow::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.position.mutate()
            .set_flex_grow(
                value,
            );
    }
    /// Inherit `flex_shrink` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_flex_shrink(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_position();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.position.mutate()
            .copy_flex_shrink_from(
                inherited_struct,
            );
    }

    /// Reset `flex_shrink` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_flex_shrink(&mut self) {
        let reset_struct =
            self.reset_style.get_position();

        self.modified_reset = true;

        self.position.mutate()
            .reset_flex_shrink(
                reset_struct,
            );
    }

    /// Set the `flex_shrink` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_flex_shrink(
        &mut self,
        value: longhands::flex_shrink::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.position.mutate()
            .set_flex_shrink(
                value,
            );
    }
    /// Inherit `align_self` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_align_self(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_position();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.position.mutate()
            .copy_align_self_from(
                inherited_struct,
            );
    }

    /// Reset `align_self` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_align_self(&mut self) {
        let reset_struct =
            self.reset_style.get_position();

        self.modified_reset = true;

        self.position.mutate()
            .reset_align_self(
                reset_struct,
            );
    }

    /// Set the `align_self` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_align_self(
        &mut self,
        value: longhands::align_self::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.position.mutate()
            .set_align_self(
                value,
            );
    }
    /// Inherit `justify_self` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_justify_self(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_position();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.position.mutate()
            .copy_justify_self_from(
                inherited_struct,
            );
    }

    /// Reset `justify_self` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_justify_self(&mut self) {
        let reset_struct =
            self.reset_style.get_position();

        self.modified_reset = true;

        self.position.mutate()
            .reset_justify_self(
                reset_struct,
            );
    }

    /// Set the `justify_self` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_justify_self(
        &mut self,
        value: longhands::justify_self::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.position.mutate()
            .set_justify_self(
                value,
            );
    }
    /// Inherit `order` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_order(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_position();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.position.mutate()
            .copy_order_from(
                inherited_struct,
            );
    }

    /// Reset `order` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_order(&mut self) {
        let reset_struct =
            self.reset_style.get_position();

        self.modified_reset = true;

        self.position.mutate()
            .reset_order(
                reset_struct,
            );
    }

    /// Set the `order` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_order(
        &mut self,
        value: longhands::order::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.position.mutate()
            .set_order(
                value,
            );
    }
    /// Inherit `flex_basis` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_flex_basis(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_position();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.position.mutate()
            .copy_flex_basis_from(
                inherited_struct,
            );
    }

    /// Reset `flex_basis` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_flex_basis(&mut self) {
        let reset_struct =
            self.reset_style.get_position();

        self.modified_reset = true;

        self.position.mutate()
            .reset_flex_basis(
                reset_struct,
            );
    }

    /// Set the `flex_basis` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_flex_basis(
        &mut self,
        value: longhands::flex_basis::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.position.mutate()
            .set_flex_basis(
                value,
            );
    }
    /// Inherit `width` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_width(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_position();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.position.mutate()
            .copy_width_from(
                inherited_struct,
            );
    }

    /// Reset `width` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_width(&mut self) {
        let reset_struct =
            self.reset_style.get_position();

        self.modified_reset = true;

        self.position.mutate()
            .reset_width(
                reset_struct,
            );
    }

    /// Set the `width` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_width(
        &mut self,
        value: longhands::width::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.position.mutate()
            .set_width(
                value,
            );
    }
    /// Inherit `min_width` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_min_width(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_position();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.position.mutate()
            .copy_min_width_from(
                inherited_struct,
            );
    }

    /// Reset `min_width` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_min_width(&mut self) {
        let reset_struct =
            self.reset_style.get_position();

        self.modified_reset = true;

        self.position.mutate()
            .reset_min_width(
                reset_struct,
            );
    }

    /// Set the `min_width` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_min_width(
        &mut self,
        value: longhands::min_width::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.position.mutate()
            .set_min_width(
                value,
            );
    }
    /// Inherit `max_width` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_max_width(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_position();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.position.mutate()
            .copy_max_width_from(
                inherited_struct,
            );
    }

    /// Reset `max_width` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_max_width(&mut self) {
        let reset_struct =
            self.reset_style.get_position();

        self.modified_reset = true;

        self.position.mutate()
            .reset_max_width(
                reset_struct,
            );
    }

    /// Set the `max_width` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_max_width(
        &mut self,
        value: longhands::max_width::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.position.mutate()
            .set_max_width(
                value,
            );
    }
    /// Inherit `height` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_height(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_position();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.position.mutate()
            .copy_height_from(
                inherited_struct,
            );
    }

    /// Reset `height` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_height(&mut self) {
        let reset_struct =
            self.reset_style.get_position();

        self.modified_reset = true;

        self.position.mutate()
            .reset_height(
                reset_struct,
            );
    }

    /// Set the `height` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_height(
        &mut self,
        value: longhands::height::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.position.mutate()
            .set_height(
                value,
            );
    }
    /// Inherit `min_height` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_min_height(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_position();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.position.mutate()
            .copy_min_height_from(
                inherited_struct,
            );
    }

    /// Reset `min_height` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_min_height(&mut self) {
        let reset_struct =
            self.reset_style.get_position();

        self.modified_reset = true;

        self.position.mutate()
            .reset_min_height(
                reset_struct,
            );
    }

    /// Set the `min_height` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_min_height(
        &mut self,
        value: longhands::min_height::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.position.mutate()
            .set_min_height(
                value,
            );
    }
    /// Inherit `max_height` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_max_height(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_position();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.position.mutate()
            .copy_max_height_from(
                inherited_struct,
            );
    }

    /// Reset `max_height` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_max_height(&mut self) {
        let reset_struct =
            self.reset_style.get_position();

        self.modified_reset = true;

        self.position.mutate()
            .reset_max_height(
                reset_struct,
            );
    }

    /// Set the `max_height` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_max_height(
        &mut self,
        value: longhands::max_height::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.position.mutate()
            .set_max_height(
                value,
            );
    }
    /// Inherit `block_size` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_block_size(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_position();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.position.mutate()
            .copy_block_size_from(
                inherited_struct,
                self.writing_mode,
            );
    }

    /// Reset `block_size` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_block_size(&mut self) {
        let reset_struct =
            self.reset_style.get_position();

        self.modified_reset = true;

        self.position.mutate()
            .reset_block_size(
                reset_struct,
                self.writing_mode,
            );
    }

    /// Set the `block_size` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_block_size(
        &mut self,
        value: longhands::block_size::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.position.mutate()
            .set_block_size(
                value,
                self.writing_mode,
            );
    }
    /// Inherit `min_block_size` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_min_block_size(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_position();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.position.mutate()
            .copy_min_block_size_from(
                inherited_struct,
                self.writing_mode,
            );
    }

    /// Reset `min_block_size` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_min_block_size(&mut self) {
        let reset_struct =
            self.reset_style.get_position();

        self.modified_reset = true;

        self.position.mutate()
            .reset_min_block_size(
                reset_struct,
                self.writing_mode,
            );
    }

    /// Set the `min_block_size` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_min_block_size(
        &mut self,
        value: longhands::min_block_size::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.position.mutate()
            .set_min_block_size(
                value,
                self.writing_mode,
            );
    }
    /// Inherit `max_block_size` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_max_block_size(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_position();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.position.mutate()
            .copy_max_block_size_from(
                inherited_struct,
                self.writing_mode,
            );
    }

    /// Reset `max_block_size` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_max_block_size(&mut self) {
        let reset_struct =
            self.reset_style.get_position();

        self.modified_reset = true;

        self.position.mutate()
            .reset_max_block_size(
                reset_struct,
                self.writing_mode,
            );
    }

    /// Set the `max_block_size` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_max_block_size(
        &mut self,
        value: longhands::max_block_size::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.position.mutate()
            .set_max_block_size(
                value,
                self.writing_mode,
            );
    }
    /// Inherit `inline_size` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_inline_size(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_position();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.position.mutate()
            .copy_inline_size_from(
                inherited_struct,
                self.writing_mode,
            );
    }

    /// Reset `inline_size` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_inline_size(&mut self) {
        let reset_struct =
            self.reset_style.get_position();

        self.modified_reset = true;

        self.position.mutate()
            .reset_inline_size(
                reset_struct,
                self.writing_mode,
            );
    }

    /// Set the `inline_size` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_inline_size(
        &mut self,
        value: longhands::inline_size::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.position.mutate()
            .set_inline_size(
                value,
                self.writing_mode,
            );
    }
    /// Inherit `min_inline_size` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_min_inline_size(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_position();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.position.mutate()
            .copy_min_inline_size_from(
                inherited_struct,
                self.writing_mode,
            );
    }

    /// Reset `min_inline_size` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_min_inline_size(&mut self) {
        let reset_struct =
            self.reset_style.get_position();

        self.modified_reset = true;

        self.position.mutate()
            .reset_min_inline_size(
                reset_struct,
                self.writing_mode,
            );
    }

    /// Set the `min_inline_size` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_min_inline_size(
        &mut self,
        value: longhands::min_inline_size::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.position.mutate()
            .set_min_inline_size(
                value,
                self.writing_mode,
            );
    }
    /// Inherit `max_inline_size` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_max_inline_size(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_position();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.position.mutate()
            .copy_max_inline_size_from(
                inherited_struct,
                self.writing_mode,
            );
    }

    /// Reset `max_inline_size` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_max_inline_size(&mut self) {
        let reset_struct =
            self.reset_style.get_position();

        self.modified_reset = true;

        self.position.mutate()
            .reset_max_inline_size(
                reset_struct,
                self.writing_mode,
            );
    }

    /// Set the `max_inline_size` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_max_inline_size(
        &mut self,
        value: longhands::max_inline_size::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.position.mutate()
            .set_max_inline_size(
                value,
                self.writing_mode,
            );
    }
    /// Inherit `box_sizing` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_box_sizing(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_position();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.position.mutate()
            .copy_box_sizing_from(
                inherited_struct,
            );
    }

    /// Reset `box_sizing` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_box_sizing(&mut self) {
        let reset_struct =
            self.reset_style.get_position();

        self.modified_reset = true;

        self.position.mutate()
            .reset_box_sizing(
                reset_struct,
            );
    }

    /// Set the `box_sizing` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_box_sizing(
        &mut self,
        value: longhands::box_sizing::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.position.mutate()
            .set_box_sizing(
                value,
            );
    }
    /// Inherit `object_fit` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_object_fit(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_position();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.position.mutate()
            .copy_object_fit_from(
                inherited_struct,
            );
    }

    /// Reset `object_fit` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_object_fit(&mut self) {
        let reset_struct =
            self.reset_style.get_position();

        self.modified_reset = true;

        self.position.mutate()
            .reset_object_fit(
                reset_struct,
            );
    }

    /// Set the `object_fit` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_object_fit(
        &mut self,
        value: longhands::object_fit::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.position.mutate()
            .set_object_fit(
                value,
            );
    }
    /// Inherit `object_position` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_object_position(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_position();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.position.mutate()
            .copy_object_position_from(
                inherited_struct,
            );
    }

    /// Reset `object_position` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_object_position(&mut self) {
        let reset_struct =
            self.reset_style.get_position();

        self.modified_reset = true;

        self.position.mutate()
            .reset_object_position(
                reset_struct,
            );
    }

    /// Set the `object_position` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_object_position(
        &mut self,
        value: longhands::object_position::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.position.mutate()
            .set_object_position(
                value,
            );
    }
    /// Inherit `grid_row_gap` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_grid_row_gap(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_position();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.position.mutate()
            .copy_grid_row_gap_from(
                inherited_struct,
            );
    }

    /// Reset `grid_row_gap` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_grid_row_gap(&mut self) {
        let reset_struct =
            self.reset_style.get_position();

        self.modified_reset = true;

        self.position.mutate()
            .reset_grid_row_gap(
                reset_struct,
            );
    }

    /// Set the `grid_row_gap` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_grid_row_gap(
        &mut self,
        value: longhands::grid_row_gap::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.position.mutate()
            .set_grid_row_gap(
                value,
            );
    }
    /// Inherit `grid_row_start` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_grid_row_start(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_position();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.position.mutate()
            .copy_grid_row_start_from(
                inherited_struct,
            );
    }

    /// Reset `grid_row_start` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_grid_row_start(&mut self) {
        let reset_struct =
            self.reset_style.get_position();

        self.modified_reset = true;

        self.position.mutate()
            .reset_grid_row_start(
                reset_struct,
            );
    }

    /// Set the `grid_row_start` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_grid_row_start(
        &mut self,
        value: longhands::grid_row_start::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.position.mutate()
            .set_grid_row_start(
                value,
            );
    }
    /// Inherit `grid_row_end` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_grid_row_end(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_position();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.position.mutate()
            .copy_grid_row_end_from(
                inherited_struct,
            );
    }

    /// Reset `grid_row_end` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_grid_row_end(&mut self) {
        let reset_struct =
            self.reset_style.get_position();

        self.modified_reset = true;

        self.position.mutate()
            .reset_grid_row_end(
                reset_struct,
            );
    }

    /// Set the `grid_row_end` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_grid_row_end(
        &mut self,
        value: longhands::grid_row_end::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.position.mutate()
            .set_grid_row_end(
                value,
            );
    }
    /// Inherit `grid_auto_rows` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_grid_auto_rows(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_position();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.position.mutate()
            .copy_grid_auto_rows_from(
                inherited_struct,
            );
    }

    /// Reset `grid_auto_rows` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_grid_auto_rows(&mut self) {
        let reset_struct =
            self.reset_style.get_position();

        self.modified_reset = true;

        self.position.mutate()
            .reset_grid_auto_rows(
                reset_struct,
            );
    }

    /// Set the `grid_auto_rows` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_grid_auto_rows(
        &mut self,
        value: longhands::grid_auto_rows::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.position.mutate()
            .set_grid_auto_rows(
                value,
            );
    }
    /// Inherit `grid_template_rows` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_grid_template_rows(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_position();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.position.mutate()
            .copy_grid_template_rows_from(
                inherited_struct,
            );
    }

    /// Reset `grid_template_rows` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_grid_template_rows(&mut self) {
        let reset_struct =
            self.reset_style.get_position();

        self.modified_reset = true;

        self.position.mutate()
            .reset_grid_template_rows(
                reset_struct,
            );
    }

    /// Set the `grid_template_rows` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_grid_template_rows(
        &mut self,
        value: longhands::grid_template_rows::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.position.mutate()
            .set_grid_template_rows(
                value,
            );
    }
    /// Inherit `grid_column_gap` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_grid_column_gap(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_position();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.position.mutate()
            .copy_grid_column_gap_from(
                inherited_struct,
            );
    }

    /// Reset `grid_column_gap` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_grid_column_gap(&mut self) {
        let reset_struct =
            self.reset_style.get_position();

        self.modified_reset = true;

        self.position.mutate()
            .reset_grid_column_gap(
                reset_struct,
            );
    }

    /// Set the `grid_column_gap` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_grid_column_gap(
        &mut self,
        value: longhands::grid_column_gap::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.position.mutate()
            .set_grid_column_gap(
                value,
            );
    }
    /// Inherit `grid_column_start` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_grid_column_start(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_position();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.position.mutate()
            .copy_grid_column_start_from(
                inherited_struct,
            );
    }

    /// Reset `grid_column_start` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_grid_column_start(&mut self) {
        let reset_struct =
            self.reset_style.get_position();

        self.modified_reset = true;

        self.position.mutate()
            .reset_grid_column_start(
                reset_struct,
            );
    }

    /// Set the `grid_column_start` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_grid_column_start(
        &mut self,
        value: longhands::grid_column_start::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.position.mutate()
            .set_grid_column_start(
                value,
            );
    }
    /// Inherit `grid_column_end` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_grid_column_end(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_position();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.position.mutate()
            .copy_grid_column_end_from(
                inherited_struct,
            );
    }

    /// Reset `grid_column_end` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_grid_column_end(&mut self) {
        let reset_struct =
            self.reset_style.get_position();

        self.modified_reset = true;

        self.position.mutate()
            .reset_grid_column_end(
                reset_struct,
            );
    }

    /// Set the `grid_column_end` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_grid_column_end(
        &mut self,
        value: longhands::grid_column_end::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.position.mutate()
            .set_grid_column_end(
                value,
            );
    }
    /// Inherit `grid_auto_columns` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_grid_auto_columns(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_position();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.position.mutate()
            .copy_grid_auto_columns_from(
                inherited_struct,
            );
    }

    /// Reset `grid_auto_columns` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_grid_auto_columns(&mut self) {
        let reset_struct =
            self.reset_style.get_position();

        self.modified_reset = true;

        self.position.mutate()
            .reset_grid_auto_columns(
                reset_struct,
            );
    }

    /// Set the `grid_auto_columns` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_grid_auto_columns(
        &mut self,
        value: longhands::grid_auto_columns::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.position.mutate()
            .set_grid_auto_columns(
                value,
            );
    }
    /// Inherit `grid_template_columns` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_grid_template_columns(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_position();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.position.mutate()
            .copy_grid_template_columns_from(
                inherited_struct,
            );
    }

    /// Reset `grid_template_columns` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_grid_template_columns(&mut self) {
        let reset_struct =
            self.reset_style.get_position();

        self.modified_reset = true;

        self.position.mutate()
            .reset_grid_template_columns(
                reset_struct,
            );
    }

    /// Set the `grid_template_columns` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_grid_template_columns(
        &mut self,
        value: longhands::grid_template_columns::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.position.mutate()
            .set_grid_template_columns(
                value,
            );
    }
    /// Inherit `grid_auto_flow` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_grid_auto_flow(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_position();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.position.mutate()
            .copy_grid_auto_flow_from(
                inherited_struct,
            );
    }

    /// Reset `grid_auto_flow` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_grid_auto_flow(&mut self) {
        let reset_struct =
            self.reset_style.get_position();

        self.modified_reset = true;

        self.position.mutate()
            .reset_grid_auto_flow(
                reset_struct,
            );
    }

    /// Set the `grid_auto_flow` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_grid_auto_flow(
        &mut self,
        value: longhands::grid_auto_flow::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.position.mutate()
            .set_grid_auto_flow(
                value,
            );
    }
    /// Inherit `grid_template_areas` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_grid_template_areas(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_position();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.position.mutate()
            .copy_grid_template_areas_from(
                inherited_struct,
            );
    }

    /// Reset `grid_template_areas` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_grid_template_areas(&mut self) {
        let reset_struct =
            self.reset_style.get_position();

        self.modified_reset = true;

        self.position.mutate()
            .reset_grid_template_areas(
                reset_struct,
            );
    }

    /// Set the `grid_template_areas` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_grid_template_areas(
        &mut self,
        value: longhands::grid_template_areas::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.position.mutate()
            .set_grid_template_areas(
                value,
            );
    }
    /// Inherit `table_layout` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_table_layout(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_table();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.table.mutate()
            .copy_table_layout_from(
                inherited_struct,
            );
    }

    /// Reset `table_layout` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_table_layout(&mut self) {
        let reset_struct =
            self.reset_style.get_table();

        self.modified_reset = true;

        self.table.mutate()
            .reset_table_layout(
                reset_struct,
            );
    }

    /// Set the `table_layout` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_table_layout(
        &mut self,
        value: longhands::table_layout::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.table.mutate()
            .set_table_layout(
                value,
            );
    }
    /// Inherit `_x_span` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit__x_span(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_table();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.table.mutate()
            .copy__x_span_from(
                inherited_struct,
            );
    }

    /// Reset `_x_span` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset__x_span(&mut self) {
        let reset_struct =
            self.reset_style.get_table();

        self.modified_reset = true;

        self.table.mutate()
            .reset__x_span(
                reset_struct,
            );
    }

    /// Set the `_x_span` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set__x_span(
        &mut self,
        value: longhands::_x_span::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.table.mutate()
            .set__x_span(
                value,
            );
    }
    /// Inherit `text_overflow` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_text_overflow(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_text();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.text.mutate()
            .copy_text_overflow_from(
                inherited_struct,
            );
    }

    /// Reset `text_overflow` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_text_overflow(&mut self) {
        let reset_struct =
            self.reset_style.get_text();

        self.modified_reset = true;

        self.text.mutate()
            .reset_text_overflow(
                reset_struct,
            );
    }

    /// Set the `text_overflow` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_text_overflow(
        &mut self,
        value: longhands::text_overflow::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.text.mutate()
            .set_text_overflow(
                value,
            );
    }
    /// Inherit `unicode_bidi` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_unicode_bidi(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_text();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.text.mutate()
            .copy_unicode_bidi_from(
                inherited_struct,
            );
    }

    /// Reset `unicode_bidi` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_unicode_bidi(&mut self) {
        let reset_struct =
            self.reset_style.get_text();

        self.modified_reset = true;

        self.text.mutate()
            .reset_unicode_bidi(
                reset_struct,
            );
    }

    /// Set the `unicode_bidi` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_unicode_bidi(
        &mut self,
        value: longhands::unicode_bidi::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.text.mutate()
            .set_unicode_bidi(
                value,
            );
    }
    /// Inherit `text_decoration_line` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_text_decoration_line(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_text();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.text.mutate()
            .copy_text_decoration_line_from(
                inherited_struct,
            );
    }

    /// Reset `text_decoration_line` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_text_decoration_line(&mut self) {
        let reset_struct =
            self.reset_style.get_text();

        self.modified_reset = true;

        self.text.mutate()
            .reset_text_decoration_line(
                reset_struct,
            );
    }

    /// Set the `text_decoration_line` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_text_decoration_line(
        &mut self,
        value: longhands::text_decoration_line::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.text.mutate()
            .set_text_decoration_line(
                value,
            );
    }
    /// Inherit `text_decoration_style` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_text_decoration_style(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_text();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.text.mutate()
            .copy_text_decoration_style_from(
                inherited_struct,
            );
    }

    /// Reset `text_decoration_style` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_text_decoration_style(&mut self) {
        let reset_struct =
            self.reset_style.get_text();

        self.modified_reset = true;

        self.text.mutate()
            .reset_text_decoration_style(
                reset_struct,
            );
    }

    /// Set the `text_decoration_style` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_text_decoration_style(
        &mut self,
        value: longhands::text_decoration_style::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.text.mutate()
            .set_text_decoration_style(
                value,
            );
    }
    /// Inherit `text_decoration_color` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_text_decoration_color(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_text();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.text.mutate()
            .copy_text_decoration_color_from(
                inherited_struct,
            );
    }

    /// Reset `text_decoration_color` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_text_decoration_color(&mut self) {
        let reset_struct =
            self.reset_style.get_text();

        self.modified_reset = true;

        self.text.mutate()
            .reset_text_decoration_color(
                reset_struct,
            );
    }

    /// Set the `text_decoration_color` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_text_decoration_color(
        &mut self,
        value: longhands::text_decoration_color::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.text.mutate()
            .set_text_decoration_color(
                value,
            );
    }
    /// Inherit `initial_letter` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_initial_letter(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_text();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.text.mutate()
            .copy_initial_letter_from(
                inherited_struct,
            );
    }

    /// Reset `initial_letter` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_initial_letter(&mut self) {
        let reset_struct =
            self.reset_style.get_text();

        self.modified_reset = true;

        self.text.mutate()
            .reset_initial_letter(
                reset_struct,
            );
    }

    /// Set the `initial_letter` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_initial_letter(
        &mut self,
        value: longhands::initial_letter::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.text.mutate()
            .set_initial_letter(
                value,
            );
    }
    /// Inherit `ime_mode` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_ime_mode(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_ui();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.ui.mutate()
            .copy_ime_mode_from(
                inherited_struct,
            );
    }

    /// Reset `ime_mode` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_ime_mode(&mut self) {
        let reset_struct =
            self.reset_style.get_ui();

        self.modified_reset = true;

        self.ui.mutate()
            .reset_ime_mode(
                reset_struct,
            );
    }

    /// Set the `ime_mode` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_ime_mode(
        &mut self,
        value: longhands::ime_mode::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.ui.mutate()
            .set_ime_mode(
                value,
            );
    }
    /// Inherit `_moz_user_select` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit__moz_user_select(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_ui();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.ui.mutate()
            .copy__moz_user_select_from(
                inherited_struct,
            );
    }

    /// Reset `_moz_user_select` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset__moz_user_select(&mut self) {
        let reset_struct =
            self.reset_style.get_ui();

        self.modified_reset = true;

        self.ui.mutate()
            .reset__moz_user_select(
                reset_struct,
            );
    }

    /// Set the `_moz_user_select` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set__moz_user_select(
        &mut self,
        value: longhands::_moz_user_select::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.ui.mutate()
            .set__moz_user_select(
                value,
            );
    }
    /// Inherit `_moz_window_dragging` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit__moz_window_dragging(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_ui();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.ui.mutate()
            .copy__moz_window_dragging_from(
                inherited_struct,
            );
    }

    /// Reset `_moz_window_dragging` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset__moz_window_dragging(&mut self) {
        let reset_struct =
            self.reset_style.get_ui();

        self.modified_reset = true;

        self.ui.mutate()
            .reset__moz_window_dragging(
                reset_struct,
            );
    }

    /// Set the `_moz_window_dragging` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set__moz_window_dragging(
        &mut self,
        value: longhands::_moz_window_dragging::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.ui.mutate()
            .set__moz_window_dragging(
                value,
            );
    }
    /// Inherit `_moz_window_shadow` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit__moz_window_shadow(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_ui();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.ui.mutate()
            .copy__moz_window_shadow_from(
                inherited_struct,
            );
    }

    /// Reset `_moz_window_shadow` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset__moz_window_shadow(&mut self) {
        let reset_struct =
            self.reset_style.get_ui();

        self.modified_reset = true;

        self.ui.mutate()
            .reset__moz_window_shadow(
                reset_struct,
            );
    }

    /// Set the `_moz_window_shadow` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set__moz_window_shadow(
        &mut self,
        value: longhands::_moz_window_shadow::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.ui.mutate()
            .set__moz_window_shadow(
                value,
            );
    }
    /// Inherit `_moz_force_broken_image_icon` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit__moz_force_broken_image_icon(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_ui();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.ui.mutate()
            .copy__moz_force_broken_image_icon_from(
                inherited_struct,
            );
    }

    /// Reset `_moz_force_broken_image_icon` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset__moz_force_broken_image_icon(&mut self) {
        let reset_struct =
            self.reset_style.get_ui();

        self.modified_reset = true;

        self.ui.mutate()
            .reset__moz_force_broken_image_icon(
                reset_struct,
            );
    }

    /// Set the `_moz_force_broken_image_icon` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set__moz_force_broken_image_icon(
        &mut self,
        value: longhands::_moz_force_broken_image_icon::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.ui.mutate()
            .set__moz_force_broken_image_icon(
                value,
            );
    }
    /// Inherit `text_anchor` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_text_anchor(&mut self) {
        let inherited_struct =
            self.inherited_style.get_inheritedsvg();




        self.inheritedsvg.mutate()
            .copy_text_anchor_from(
                inherited_struct,
            );
    }

    /// Reset `text_anchor` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_text_anchor(&mut self) {
        let reset_struct =
            self.reset_style.get_inheritedsvg();


        self.inheritedsvg.mutate()
            .reset_text_anchor(
                reset_struct,
            );
    }

    /// Set the `text_anchor` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_text_anchor(
        &mut self,
        value: longhands::text_anchor::computed_value::T
    ) {

        
        self.inheritedsvg.mutate()
            .set_text_anchor(
                value,
            );
    }
    /// Inherit `color_interpolation` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_color_interpolation(&mut self) {
        let inherited_struct =
            self.inherited_style.get_inheritedsvg();




        self.inheritedsvg.mutate()
            .copy_color_interpolation_from(
                inherited_struct,
            );
    }

    /// Reset `color_interpolation` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_color_interpolation(&mut self) {
        let reset_struct =
            self.reset_style.get_inheritedsvg();


        self.inheritedsvg.mutate()
            .reset_color_interpolation(
                reset_struct,
            );
    }

    /// Set the `color_interpolation` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_color_interpolation(
        &mut self,
        value: longhands::color_interpolation::computed_value::T
    ) {

        
        self.inheritedsvg.mutate()
            .set_color_interpolation(
                value,
            );
    }
    /// Inherit `color_interpolation_filters` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_color_interpolation_filters(&mut self) {
        let inherited_struct =
            self.inherited_style.get_inheritedsvg();




        self.inheritedsvg.mutate()
            .copy_color_interpolation_filters_from(
                inherited_struct,
            );
    }

    /// Reset `color_interpolation_filters` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_color_interpolation_filters(&mut self) {
        let reset_struct =
            self.reset_style.get_inheritedsvg();


        self.inheritedsvg.mutate()
            .reset_color_interpolation_filters(
                reset_struct,
            );
    }

    /// Set the `color_interpolation_filters` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_color_interpolation_filters(
        &mut self,
        value: longhands::color_interpolation_filters::computed_value::T
    ) {

        
        self.inheritedsvg.mutate()
            .set_color_interpolation_filters(
                value,
            );
    }
    /// Inherit `fill` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_fill(&mut self) {
        let inherited_struct =
            self.inherited_style.get_inheritedsvg();




        self.inheritedsvg.mutate()
            .copy_fill_from(
                inherited_struct,
            );
    }

    /// Reset `fill` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_fill(&mut self) {
        let reset_struct =
            self.reset_style.get_inheritedsvg();


        self.inheritedsvg.mutate()
            .reset_fill(
                reset_struct,
            );
    }

    /// Set the `fill` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_fill(
        &mut self,
        value: longhands::fill::computed_value::T
    ) {

        
        self.inheritedsvg.mutate()
            .set_fill(
                value,
            );
    }
    /// Inherit `fill_opacity` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_fill_opacity(&mut self) {
        let inherited_struct =
            self.inherited_style.get_inheritedsvg();




        self.inheritedsvg.mutate()
            .copy_fill_opacity_from(
                inherited_struct,
            );
    }

    /// Reset `fill_opacity` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_fill_opacity(&mut self) {
        let reset_struct =
            self.reset_style.get_inheritedsvg();


        self.inheritedsvg.mutate()
            .reset_fill_opacity(
                reset_struct,
            );
    }

    /// Set the `fill_opacity` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_fill_opacity(
        &mut self,
        value: longhands::fill_opacity::computed_value::T
    ) {

        
        self.inheritedsvg.mutate()
            .set_fill_opacity(
                value,
            );
    }
    /// Inherit `fill_rule` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_fill_rule(&mut self) {
        let inherited_struct =
            self.inherited_style.get_inheritedsvg();




        self.inheritedsvg.mutate()
            .copy_fill_rule_from(
                inherited_struct,
            );
    }

    /// Reset `fill_rule` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_fill_rule(&mut self) {
        let reset_struct =
            self.reset_style.get_inheritedsvg();


        self.inheritedsvg.mutate()
            .reset_fill_rule(
                reset_struct,
            );
    }

    /// Set the `fill_rule` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_fill_rule(
        &mut self,
        value: longhands::fill_rule::computed_value::T
    ) {

        
        self.inheritedsvg.mutate()
            .set_fill_rule(
                value,
            );
    }
    /// Inherit `shape_rendering` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_shape_rendering(&mut self) {
        let inherited_struct =
            self.inherited_style.get_inheritedsvg();




        self.inheritedsvg.mutate()
            .copy_shape_rendering_from(
                inherited_struct,
            );
    }

    /// Reset `shape_rendering` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_shape_rendering(&mut self) {
        let reset_struct =
            self.reset_style.get_inheritedsvg();


        self.inheritedsvg.mutate()
            .reset_shape_rendering(
                reset_struct,
            );
    }

    /// Set the `shape_rendering` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_shape_rendering(
        &mut self,
        value: longhands::shape_rendering::computed_value::T
    ) {

        
        self.inheritedsvg.mutate()
            .set_shape_rendering(
                value,
            );
    }
    /// Inherit `stroke` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_stroke(&mut self) {
        let inherited_struct =
            self.inherited_style.get_inheritedsvg();




        self.inheritedsvg.mutate()
            .copy_stroke_from(
                inherited_struct,
            );
    }

    /// Reset `stroke` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_stroke(&mut self) {
        let reset_struct =
            self.reset_style.get_inheritedsvg();


        self.inheritedsvg.mutate()
            .reset_stroke(
                reset_struct,
            );
    }

    /// Set the `stroke` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_stroke(
        &mut self,
        value: longhands::stroke::computed_value::T
    ) {

        
        self.inheritedsvg.mutate()
            .set_stroke(
                value,
            );
    }
    /// Inherit `stroke_width` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_stroke_width(&mut self) {
        let inherited_struct =
            self.inherited_style.get_inheritedsvg();




        self.inheritedsvg.mutate()
            .copy_stroke_width_from(
                inherited_struct,
            );
    }

    /// Reset `stroke_width` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_stroke_width(&mut self) {
        let reset_struct =
            self.reset_style.get_inheritedsvg();


        self.inheritedsvg.mutate()
            .reset_stroke_width(
                reset_struct,
            );
    }

    /// Set the `stroke_width` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_stroke_width(
        &mut self,
        value: longhands::stroke_width::computed_value::T
    ) {

        
        self.inheritedsvg.mutate()
            .set_stroke_width(
                value,
            );
    }
    /// Inherit `stroke_linecap` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_stroke_linecap(&mut self) {
        let inherited_struct =
            self.inherited_style.get_inheritedsvg();




        self.inheritedsvg.mutate()
            .copy_stroke_linecap_from(
                inherited_struct,
            );
    }

    /// Reset `stroke_linecap` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_stroke_linecap(&mut self) {
        let reset_struct =
            self.reset_style.get_inheritedsvg();


        self.inheritedsvg.mutate()
            .reset_stroke_linecap(
                reset_struct,
            );
    }

    /// Set the `stroke_linecap` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_stroke_linecap(
        &mut self,
        value: longhands::stroke_linecap::computed_value::T
    ) {

        
        self.inheritedsvg.mutate()
            .set_stroke_linecap(
                value,
            );
    }
    /// Inherit `stroke_linejoin` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_stroke_linejoin(&mut self) {
        let inherited_struct =
            self.inherited_style.get_inheritedsvg();




        self.inheritedsvg.mutate()
            .copy_stroke_linejoin_from(
                inherited_struct,
            );
    }

    /// Reset `stroke_linejoin` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_stroke_linejoin(&mut self) {
        let reset_struct =
            self.reset_style.get_inheritedsvg();


        self.inheritedsvg.mutate()
            .reset_stroke_linejoin(
                reset_struct,
            );
    }

    /// Set the `stroke_linejoin` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_stroke_linejoin(
        &mut self,
        value: longhands::stroke_linejoin::computed_value::T
    ) {

        
        self.inheritedsvg.mutate()
            .set_stroke_linejoin(
                value,
            );
    }
    /// Inherit `stroke_miterlimit` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_stroke_miterlimit(&mut self) {
        let inherited_struct =
            self.inherited_style.get_inheritedsvg();




        self.inheritedsvg.mutate()
            .copy_stroke_miterlimit_from(
                inherited_struct,
            );
    }

    /// Reset `stroke_miterlimit` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_stroke_miterlimit(&mut self) {
        let reset_struct =
            self.reset_style.get_inheritedsvg();


        self.inheritedsvg.mutate()
            .reset_stroke_miterlimit(
                reset_struct,
            );
    }

    /// Set the `stroke_miterlimit` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_stroke_miterlimit(
        &mut self,
        value: longhands::stroke_miterlimit::computed_value::T
    ) {

        
        self.inheritedsvg.mutate()
            .set_stroke_miterlimit(
                value,
            );
    }
    /// Inherit `stroke_opacity` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_stroke_opacity(&mut self) {
        let inherited_struct =
            self.inherited_style.get_inheritedsvg();




        self.inheritedsvg.mutate()
            .copy_stroke_opacity_from(
                inherited_struct,
            );
    }

    /// Reset `stroke_opacity` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_stroke_opacity(&mut self) {
        let reset_struct =
            self.reset_style.get_inheritedsvg();


        self.inheritedsvg.mutate()
            .reset_stroke_opacity(
                reset_struct,
            );
    }

    /// Set the `stroke_opacity` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_stroke_opacity(
        &mut self,
        value: longhands::stroke_opacity::computed_value::T
    ) {

        
        self.inheritedsvg.mutate()
            .set_stroke_opacity(
                value,
            );
    }
    /// Inherit `stroke_dasharray` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_stroke_dasharray(&mut self) {
        let inherited_struct =
            self.inherited_style.get_inheritedsvg();




        self.inheritedsvg.mutate()
            .copy_stroke_dasharray_from(
                inherited_struct,
            );
    }

    /// Reset `stroke_dasharray` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_stroke_dasharray(&mut self) {
        let reset_struct =
            self.reset_style.get_inheritedsvg();


        self.inheritedsvg.mutate()
            .reset_stroke_dasharray(
                reset_struct,
            );
    }

    /// Set the `stroke_dasharray` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_stroke_dasharray(
        &mut self,
        value: longhands::stroke_dasharray::computed_value::T
    ) {

        
        self.inheritedsvg.mutate()
            .set_stroke_dasharray(
                value,
            );
    }
    /// Inherit `stroke_dashoffset` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_stroke_dashoffset(&mut self) {
        let inherited_struct =
            self.inherited_style.get_inheritedsvg();




        self.inheritedsvg.mutate()
            .copy_stroke_dashoffset_from(
                inherited_struct,
            );
    }

    /// Reset `stroke_dashoffset` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_stroke_dashoffset(&mut self) {
        let reset_struct =
            self.reset_style.get_inheritedsvg();


        self.inheritedsvg.mutate()
            .reset_stroke_dashoffset(
                reset_struct,
            );
    }

    /// Set the `stroke_dashoffset` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_stroke_dashoffset(
        &mut self,
        value: longhands::stroke_dashoffset::computed_value::T
    ) {

        
        self.inheritedsvg.mutate()
            .set_stroke_dashoffset(
                value,
            );
    }
    /// Inherit `clip_rule` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_clip_rule(&mut self) {
        let inherited_struct =
            self.inherited_style.get_inheritedsvg();




        self.inheritedsvg.mutate()
            .copy_clip_rule_from(
                inherited_struct,
            );
    }

    /// Reset `clip_rule` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_clip_rule(&mut self) {
        let reset_struct =
            self.reset_style.get_inheritedsvg();


        self.inheritedsvg.mutate()
            .reset_clip_rule(
                reset_struct,
            );
    }

    /// Set the `clip_rule` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_clip_rule(
        &mut self,
        value: longhands::clip_rule::computed_value::T
    ) {

        
        self.inheritedsvg.mutate()
            .set_clip_rule(
                value,
            );
    }
    /// Inherit `marker_start` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_marker_start(&mut self) {
        let inherited_struct =
            self.inherited_style.get_inheritedsvg();




        self.inheritedsvg.mutate()
            .copy_marker_start_from(
                inherited_struct,
            );
    }

    /// Reset `marker_start` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_marker_start(&mut self) {
        let reset_struct =
            self.reset_style.get_inheritedsvg();


        self.inheritedsvg.mutate()
            .reset_marker_start(
                reset_struct,
            );
    }

    /// Set the `marker_start` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_marker_start(
        &mut self,
        value: longhands::marker_start::computed_value::T
    ) {

        
        self.inheritedsvg.mutate()
            .set_marker_start(
                value,
            );
    }
    /// Inherit `marker_mid` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_marker_mid(&mut self) {
        let inherited_struct =
            self.inherited_style.get_inheritedsvg();




        self.inheritedsvg.mutate()
            .copy_marker_mid_from(
                inherited_struct,
            );
    }

    /// Reset `marker_mid` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_marker_mid(&mut self) {
        let reset_struct =
            self.reset_style.get_inheritedsvg();


        self.inheritedsvg.mutate()
            .reset_marker_mid(
                reset_struct,
            );
    }

    /// Set the `marker_mid` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_marker_mid(
        &mut self,
        value: longhands::marker_mid::computed_value::T
    ) {

        
        self.inheritedsvg.mutate()
            .set_marker_mid(
                value,
            );
    }
    /// Inherit `marker_end` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_marker_end(&mut self) {
        let inherited_struct =
            self.inherited_style.get_inheritedsvg();




        self.inheritedsvg.mutate()
            .copy_marker_end_from(
                inherited_struct,
            );
    }

    /// Reset `marker_end` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_marker_end(&mut self) {
        let reset_struct =
            self.reset_style.get_inheritedsvg();


        self.inheritedsvg.mutate()
            .reset_marker_end(
                reset_struct,
            );
    }

    /// Set the `marker_end` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_marker_end(
        &mut self,
        value: longhands::marker_end::computed_value::T
    ) {

        
        self.inheritedsvg.mutate()
            .set_marker_end(
                value,
            );
    }
    /// Inherit `paint_order` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_paint_order(&mut self) {
        let inherited_struct =
            self.inherited_style.get_inheritedsvg();




        self.inheritedsvg.mutate()
            .copy_paint_order_from(
                inherited_struct,
            );
    }

    /// Reset `paint_order` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_paint_order(&mut self) {
        let reset_struct =
            self.reset_style.get_inheritedsvg();


        self.inheritedsvg.mutate()
            .reset_paint_order(
                reset_struct,
            );
    }

    /// Set the `paint_order` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_paint_order(
        &mut self,
        value: longhands::paint_order::computed_value::T
    ) {

        
        self.inheritedsvg.mutate()
            .set_paint_order(
                value,
            );
    }
    /// Inherit `_moz_context_properties` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit__moz_context_properties(&mut self) {
        let inherited_struct =
            self.inherited_style.get_inheritedsvg();




        self.inheritedsvg.mutate()
            .copy__moz_context_properties_from(
                inherited_struct,
            );
    }

    /// Reset `_moz_context_properties` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset__moz_context_properties(&mut self) {
        let reset_struct =
            self.reset_style.get_inheritedsvg();


        self.inheritedsvg.mutate()
            .reset__moz_context_properties(
                reset_struct,
            );
    }

    /// Inherit `dominant_baseline` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_dominant_baseline(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_svg();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.svg.mutate()
            .copy_dominant_baseline_from(
                inherited_struct,
            );
    }

    /// Reset `dominant_baseline` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_dominant_baseline(&mut self) {
        let reset_struct =
            self.reset_style.get_svg();

        self.modified_reset = true;

        self.svg.mutate()
            .reset_dominant_baseline(
                reset_struct,
            );
    }

    /// Set the `dominant_baseline` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_dominant_baseline(
        &mut self,
        value: longhands::dominant_baseline::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.svg.mutate()
            .set_dominant_baseline(
                value,
            );
    }
    /// Inherit `vector_effect` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_vector_effect(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_svg();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.svg.mutate()
            .copy_vector_effect_from(
                inherited_struct,
            );
    }

    /// Reset `vector_effect` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_vector_effect(&mut self) {
        let reset_struct =
            self.reset_style.get_svg();

        self.modified_reset = true;

        self.svg.mutate()
            .reset_vector_effect(
                reset_struct,
            );
    }

    /// Set the `vector_effect` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_vector_effect(
        &mut self,
        value: longhands::vector_effect::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.svg.mutate()
            .set_vector_effect(
                value,
            );
    }
    /// Inherit `stop_color` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_stop_color(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_svg();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.svg.mutate()
            .copy_stop_color_from(
                inherited_struct,
            );
    }

    /// Reset `stop_color` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_stop_color(&mut self) {
        let reset_struct =
            self.reset_style.get_svg();

        self.modified_reset = true;

        self.svg.mutate()
            .reset_stop_color(
                reset_struct,
            );
    }

    /// Set the `stop_color` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_stop_color(
        &mut self,
        value: longhands::stop_color::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.svg.mutate()
            .set_stop_color(
                value,
            );
    }
    /// Inherit `stop_opacity` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_stop_opacity(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_svg();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.svg.mutate()
            .copy_stop_opacity_from(
                inherited_struct,
            );
    }

    /// Reset `stop_opacity` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_stop_opacity(&mut self) {
        let reset_struct =
            self.reset_style.get_svg();

        self.modified_reset = true;

        self.svg.mutate()
            .reset_stop_opacity(
                reset_struct,
            );
    }

    /// Set the `stop_opacity` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_stop_opacity(
        &mut self,
        value: longhands::stop_opacity::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.svg.mutate()
            .set_stop_opacity(
                value,
            );
    }
    /// Inherit `flood_color` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_flood_color(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_svg();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.svg.mutate()
            .copy_flood_color_from(
                inherited_struct,
            );
    }

    /// Reset `flood_color` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_flood_color(&mut self) {
        let reset_struct =
            self.reset_style.get_svg();

        self.modified_reset = true;

        self.svg.mutate()
            .reset_flood_color(
                reset_struct,
            );
    }

    /// Set the `flood_color` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_flood_color(
        &mut self,
        value: longhands::flood_color::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.svg.mutate()
            .set_flood_color(
                value,
            );
    }
    /// Inherit `flood_opacity` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_flood_opacity(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_svg();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.svg.mutate()
            .copy_flood_opacity_from(
                inherited_struct,
            );
    }

    /// Reset `flood_opacity` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_flood_opacity(&mut self) {
        let reset_struct =
            self.reset_style.get_svg();

        self.modified_reset = true;

        self.svg.mutate()
            .reset_flood_opacity(
                reset_struct,
            );
    }

    /// Set the `flood_opacity` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_flood_opacity(
        &mut self,
        value: longhands::flood_opacity::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.svg.mutate()
            .set_flood_opacity(
                value,
            );
    }
    /// Inherit `lighting_color` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_lighting_color(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_svg();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.svg.mutate()
            .copy_lighting_color_from(
                inherited_struct,
            );
    }

    /// Reset `lighting_color` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_lighting_color(&mut self) {
        let reset_struct =
            self.reset_style.get_svg();

        self.modified_reset = true;

        self.svg.mutate()
            .reset_lighting_color(
                reset_struct,
            );
    }

    /// Set the `lighting_color` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_lighting_color(
        &mut self,
        value: longhands::lighting_color::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.svg.mutate()
            .set_lighting_color(
                value,
            );
    }
    /// Inherit `mask_type` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_mask_type(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_svg();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.svg.mutate()
            .copy_mask_type_from(
                inherited_struct,
            );
    }

    /// Reset `mask_type` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_mask_type(&mut self) {
        let reset_struct =
            self.reset_style.get_svg();

        self.modified_reset = true;

        self.svg.mutate()
            .reset_mask_type(
                reset_struct,
            );
    }

    /// Set the `mask_type` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_mask_type(
        &mut self,
        value: longhands::mask_type::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.svg.mutate()
            .set_mask_type(
                value,
            );
    }
    /// Inherit `clip_path` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_clip_path(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_svg();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.svg.mutate()
            .copy_clip_path_from(
                inherited_struct,
            );
    }

    /// Reset `clip_path` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_clip_path(&mut self) {
        let reset_struct =
            self.reset_style.get_svg();

        self.modified_reset = true;

        self.svg.mutate()
            .reset_clip_path(
                reset_struct,
            );
    }

    /// Set the `clip_path` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_clip_path(
        &mut self,
        value: longhands::clip_path::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.svg.mutate()
            .set_clip_path(
                value,
            );
    }
    /// Inherit `mask_mode` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_mask_mode(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_svg();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.svg.mutate()
            .copy_mask_mode_from(
                inherited_struct,
            );
    }

    /// Reset `mask_mode` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_mask_mode(&mut self) {
        let reset_struct =
            self.reset_style.get_svg();

        self.modified_reset = true;

        self.svg.mutate()
            .reset_mask_mode(
                reset_struct,
            );
    }

    /// Inherit `mask_repeat` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_mask_repeat(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_svg();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.svg.mutate()
            .copy_mask_repeat_from(
                inherited_struct,
            );
    }

    /// Reset `mask_repeat` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_mask_repeat(&mut self) {
        let reset_struct =
            self.reset_style.get_svg();

        self.modified_reset = true;

        self.svg.mutate()
            .reset_mask_repeat(
                reset_struct,
            );
    }

    /// Inherit `mask_position_x` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_mask_position_x(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_svg();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.svg.mutate()
            .copy_mask_position_x_from(
                inherited_struct,
            );
    }

    /// Reset `mask_position_x` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_mask_position_x(&mut self) {
        let reset_struct =
            self.reset_style.get_svg();

        self.modified_reset = true;

        self.svg.mutate()
            .reset_mask_position_x(
                reset_struct,
            );
    }

    /// Inherit `mask_position_y` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_mask_position_y(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_svg();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.svg.mutate()
            .copy_mask_position_y_from(
                inherited_struct,
            );
    }

    /// Reset `mask_position_y` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_mask_position_y(&mut self) {
        let reset_struct =
            self.reset_style.get_svg();

        self.modified_reset = true;

        self.svg.mutate()
            .reset_mask_position_y(
                reset_struct,
            );
    }

    /// Inherit `mask_clip` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_mask_clip(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_svg();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.svg.mutate()
            .copy_mask_clip_from(
                inherited_struct,
            );
    }

    /// Reset `mask_clip` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_mask_clip(&mut self) {
        let reset_struct =
            self.reset_style.get_svg();

        self.modified_reset = true;

        self.svg.mutate()
            .reset_mask_clip(
                reset_struct,
            );
    }

    /// Inherit `mask_origin` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_mask_origin(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_svg();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.svg.mutate()
            .copy_mask_origin_from(
                inherited_struct,
            );
    }

    /// Reset `mask_origin` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_mask_origin(&mut self) {
        let reset_struct =
            self.reset_style.get_svg();

        self.modified_reset = true;

        self.svg.mutate()
            .reset_mask_origin(
                reset_struct,
            );
    }

    /// Inherit `mask_size` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_mask_size(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_svg();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.svg.mutate()
            .copy_mask_size_from(
                inherited_struct,
            );
    }

    /// Reset `mask_size` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_mask_size(&mut self) {
        let reset_struct =
            self.reset_style.get_svg();

        self.modified_reset = true;

        self.svg.mutate()
            .reset_mask_size(
                reset_struct,
            );
    }

    /// Set the `mask_size` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set_mask_size(
        &mut self,
        value: longhands::mask_size::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.svg.mutate()
            .set_mask_size(
                value,
            );
    }
    /// Inherit `mask_composite` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_mask_composite(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_svg();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.svg.mutate()
            .copy_mask_composite_from(
                inherited_struct,
            );
    }

    /// Reset `mask_composite` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_mask_composite(&mut self) {
        let reset_struct =
            self.reset_style.get_svg();

        self.modified_reset = true;

        self.svg.mutate()
            .reset_mask_composite(
                reset_struct,
            );
    }

    /// Inherit `mask_image` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit_mask_image(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_svg();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.svg.mutate()
            .copy_mask_image_from(
                inherited_struct,
            );
    }

    /// Reset `mask_image` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset_mask_image(&mut self) {
        let reset_struct =
            self.reset_style.get_svg();

        self.modified_reset = true;

        self.svg.mutate()
            .reset_mask_image(
                reset_struct,
            );
    }

    /// Inherit `_moz_box_align` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit__moz_box_align(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_xul();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.xul.mutate()
            .copy__moz_box_align_from(
                inherited_struct,
            );
    }

    /// Reset `_moz_box_align` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset__moz_box_align(&mut self) {
        let reset_struct =
            self.reset_style.get_xul();

        self.modified_reset = true;

        self.xul.mutate()
            .reset__moz_box_align(
                reset_struct,
            );
    }

    /// Set the `_moz_box_align` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set__moz_box_align(
        &mut self,
        value: longhands::_moz_box_align::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.xul.mutate()
            .set__moz_box_align(
                value,
            );
    }
    /// Inherit `_moz_box_direction` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit__moz_box_direction(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_xul();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.xul.mutate()
            .copy__moz_box_direction_from(
                inherited_struct,
            );
    }

    /// Reset `_moz_box_direction` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset__moz_box_direction(&mut self) {
        let reset_struct =
            self.reset_style.get_xul();

        self.modified_reset = true;

        self.xul.mutate()
            .reset__moz_box_direction(
                reset_struct,
            );
    }

    /// Set the `_moz_box_direction` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set__moz_box_direction(
        &mut self,
        value: longhands::_moz_box_direction::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.xul.mutate()
            .set__moz_box_direction(
                value,
            );
    }
    /// Inherit `_moz_box_flex` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit__moz_box_flex(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_xul();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.xul.mutate()
            .copy__moz_box_flex_from(
                inherited_struct,
            );
    }

    /// Reset `_moz_box_flex` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset__moz_box_flex(&mut self) {
        let reset_struct =
            self.reset_style.get_xul();

        self.modified_reset = true;

        self.xul.mutate()
            .reset__moz_box_flex(
                reset_struct,
            );
    }

    /// Set the `_moz_box_flex` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set__moz_box_flex(
        &mut self,
        value: longhands::_moz_box_flex::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.xul.mutate()
            .set__moz_box_flex(
                value,
            );
    }
    /// Inherit `_moz_box_orient` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit__moz_box_orient(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_xul();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.xul.mutate()
            .copy__moz_box_orient_from(
                inherited_struct,
            );
    }

    /// Reset `_moz_box_orient` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset__moz_box_orient(&mut self) {
        let reset_struct =
            self.reset_style.get_xul();

        self.modified_reset = true;

        self.xul.mutate()
            .reset__moz_box_orient(
                reset_struct,
            );
    }

    /// Set the `_moz_box_orient` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set__moz_box_orient(
        &mut self,
        value: longhands::_moz_box_orient::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.xul.mutate()
            .set__moz_box_orient(
                value,
            );
    }
    /// Inherit `_moz_box_pack` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit__moz_box_pack(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_xul();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.xul.mutate()
            .copy__moz_box_pack_from(
                inherited_struct,
            );
    }

    /// Reset `_moz_box_pack` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset__moz_box_pack(&mut self) {
        let reset_struct =
            self.reset_style.get_xul();

        self.modified_reset = true;

        self.xul.mutate()
            .reset__moz_box_pack(
                reset_struct,
            );
    }

    /// Set the `_moz_box_pack` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set__moz_box_pack(
        &mut self,
        value: longhands::_moz_box_pack::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.xul.mutate()
            .set__moz_box_pack(
                value,
            );
    }
    /// Inherit `_moz_stack_sizing` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit__moz_stack_sizing(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_xul();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.xul.mutate()
            .copy__moz_stack_sizing_from(
                inherited_struct,
            );
    }

    /// Reset `_moz_stack_sizing` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset__moz_stack_sizing(&mut self) {
        let reset_struct =
            self.reset_style.get_xul();

        self.modified_reset = true;

        self.xul.mutate()
            .reset__moz_stack_sizing(
                reset_struct,
            );
    }

    /// Set the `_moz_stack_sizing` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set__moz_stack_sizing(
        &mut self,
        value: longhands::_moz_stack_sizing::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.xul.mutate()
            .set__moz_stack_sizing(
                value,
            );
    }
    /// Inherit `_moz_box_ordinal_group` from our parent style.
    #[allow(non_snake_case)]
    pub fn inherit__moz_box_ordinal_group(&mut self) {
        let inherited_struct =
            self.inherited_style_ignoring_first_line
                .get_xul();

        self.flags.insert(::properties::computed_value_flags::INHERITS_RESET_STYLE);
        self.modified_reset = true;



        self.xul.mutate()
            .copy__moz_box_ordinal_group_from(
                inherited_struct,
            );
    }

    /// Reset `_moz_box_ordinal_group` to the initial value.
    #[allow(non_snake_case)]
    pub fn reset__moz_box_ordinal_group(&mut self) {
        let reset_struct =
            self.reset_style.get_xul();

        self.modified_reset = true;

        self.xul.mutate()
            .reset__moz_box_ordinal_group(
                reset_struct,
            );
    }

    /// Set the `_moz_box_ordinal_group` to the computed value `value`.
    #[allow(non_snake_case)]
    pub fn set__moz_box_ordinal_group(
        &mut self,
        value: longhands::_moz_box_ordinal_group::computed_value::T
    ) {
        self.modified_reset = true;

        
        self.xul.mutate()
            .set__moz_box_ordinal_group(
                value,
            );
    }

    /// Inherits style from the parent element, accounting for the default
    /// computed values that need to be provided as well.
    pub fn for_inheritance(
        device: &'a Device,
        parent: &'a ComputedValues,
        pseudo: Option<&'a PseudoElement>,
    ) -> Self {
        // Rebuild the visited style from the parent, ensuring that it will also
        // not have rules.  This matches the unvisited style that will be
        // produced by this builder.  This assumes that the caller doesn't need
        // to adjust or process visited style, so we can just build visited
        // style here for simplicity.
        let visited_style = parent.get_visited_style().map(|style| {
            Self::for_inheritance(
                device,
                style,
                pseudo,
            ).build()
        });
        // FIXME(emilio): This Some(parent) here is inconsistent with what we
        // usually do if `parent` is the default computed values, but that's
        // fine, and we want to eventually get rid of it.
        Self::new(
            device,
            Some(parent),
            Some(parent),
            pseudo,
            CascadeFlags::empty(),
            /* rules = */ None,
            parent.custom_properties(),
            parent.writing_mode,
            parent.font_computation_data,
            parent.flags,
            visited_style,
        )
    }

    /// Returns whether we have a visited style.
    pub fn has_visited_style(&self) -> bool {
        self.visited_style.is_some()
    }

    /// Returns whether we're a pseudo-elements style.
    pub fn is_pseudo_element(&self) -> bool {
        self.pseudo.map_or(false, |p| !p.is_anon_box())
    }

    /// Returns the style we're getting reset properties from.
    pub fn default_style(&self) -> &'a ComputedValues {
        self.reset_style
    }

        /// Gets an immutable view of the current `Background` style.
        pub fn get_background(&self) -> &style_structs::Background {
            &self.background
        }

        /// Gets a mutable view of the current `Background` style.
        pub fn mutate_background(&mut self) -> &mut style_structs::Background {
            self.modified_reset = true;
            self.background.mutate()
        }

        /// Gets a mutable view of the current `Background` style.
        pub fn take_background(&mut self) -> UniqueArc<style_structs::Background> {
            self.modified_reset = true;
            self.background.take()
        }

        /// Gets a mutable view of the current `Background` style.
        pub fn put_background(&mut self, s: UniqueArc<style_structs::Background>) {
            self.background.put(s)
        }

        /// Gets a mutable view of the current `Background` style,
        /// only if it's been mutated before.
        pub fn get_background_if_mutated(&mut self)
                                                         -> Option<&mut style_structs::Background> {
            self.background.get_if_mutated()
        }

        /// Reset the current `Background` style to its default value.
        pub fn reset_background_struct(&mut self) {
            self.background =
                StyleStructRef::Borrowed(self.reset_style.background_arc());
        }
        /// Gets an immutable view of the current `Border` style.
        pub fn get_border(&self) -> &style_structs::Border {
            &self.border
        }

        /// Gets a mutable view of the current `Border` style.
        pub fn mutate_border(&mut self) -> &mut style_structs::Border {
            self.modified_reset = true;
            self.border.mutate()
        }

        /// Gets a mutable view of the current `Border` style.
        pub fn take_border(&mut self) -> UniqueArc<style_structs::Border> {
            self.modified_reset = true;
            self.border.take()
        }

        /// Gets a mutable view of the current `Border` style.
        pub fn put_border(&mut self, s: UniqueArc<style_structs::Border>) {
            self.border.put(s)
        }

        /// Gets a mutable view of the current `Border` style,
        /// only if it's been mutated before.
        pub fn get_border_if_mutated(&mut self)
                                                         -> Option<&mut style_structs::Border> {
            self.border.get_if_mutated()
        }

        /// Reset the current `Border` style to its default value.
        pub fn reset_border_struct(&mut self) {
            self.border =
                StyleStructRef::Borrowed(self.reset_style.border_arc());
        }
        /// Gets an immutable view of the current `Box` style.
        pub fn get_box(&self) -> &style_structs::Box {
            &self.box_
        }

        /// Gets a mutable view of the current `Box` style.
        pub fn mutate_box(&mut self) -> &mut style_structs::Box {
            self.modified_reset = true;
            self.box_.mutate()
        }

        /// Gets a mutable view of the current `Box` style.
        pub fn take_box(&mut self) -> UniqueArc<style_structs::Box> {
            self.modified_reset = true;
            self.box_.take()
        }

        /// Gets a mutable view of the current `Box` style.
        pub fn put_box(&mut self, s: UniqueArc<style_structs::Box>) {
            self.box_.put(s)
        }

        /// Gets a mutable view of the current `Box` style,
        /// only if it's been mutated before.
        pub fn get_box_if_mutated(&mut self)
                                                         -> Option<&mut style_structs::Box> {
            self.box_.get_if_mutated()
        }

        /// Reset the current `Box` style to its default value.
        pub fn reset_box_struct(&mut self) {
            self.box_ =
                StyleStructRef::Borrowed(self.reset_style.box_arc());
        }
        /// Gets an immutable view of the current `Color` style.
        pub fn get_color(&self) -> &style_structs::Color {
            &self.color
        }

        /// Gets a mutable view of the current `Color` style.
        pub fn mutate_color(&mut self) -> &mut style_structs::Color {
            self.modified_reset = true;
            self.color.mutate()
        }

        /// Gets a mutable view of the current `Color` style.
        pub fn take_color(&mut self) -> UniqueArc<style_structs::Color> {
            self.modified_reset = true;
            self.color.take()
        }

        /// Gets a mutable view of the current `Color` style.
        pub fn put_color(&mut self, s: UniqueArc<style_structs::Color>) {
            self.color.put(s)
        }

        /// Gets a mutable view of the current `Color` style,
        /// only if it's been mutated before.
        pub fn get_color_if_mutated(&mut self)
                                                         -> Option<&mut style_structs::Color> {
            self.color.get_if_mutated()
        }

        /// Reset the current `Color` style to its default value.
        pub fn reset_color_struct(&mut self) {
            self.color =
                StyleStructRef::Borrowed(self.reset_style.color_arc());
        }
        /// Gets an immutable view of the current `Column` style.
        pub fn get_column(&self) -> &style_structs::Column {
            &self.column
        }

        /// Gets a mutable view of the current `Column` style.
        pub fn mutate_column(&mut self) -> &mut style_structs::Column {
            self.modified_reset = true;
            self.column.mutate()
        }

        /// Gets a mutable view of the current `Column` style.
        pub fn take_column(&mut self) -> UniqueArc<style_structs::Column> {
            self.modified_reset = true;
            self.column.take()
        }

        /// Gets a mutable view of the current `Column` style.
        pub fn put_column(&mut self, s: UniqueArc<style_structs::Column>) {
            self.column.put(s)
        }

        /// Gets a mutable view of the current `Column` style,
        /// only if it's been mutated before.
        pub fn get_column_if_mutated(&mut self)
                                                         -> Option<&mut style_structs::Column> {
            self.column.get_if_mutated()
        }

        /// Reset the current `Column` style to its default value.
        pub fn reset_column_struct(&mut self) {
            self.column =
                StyleStructRef::Borrowed(self.reset_style.column_arc());
        }
        /// Gets an immutable view of the current `Counters` style.
        pub fn get_counters(&self) -> &style_structs::Counters {
            &self.counters
        }

        /// Gets a mutable view of the current `Counters` style.
        pub fn mutate_counters(&mut self) -> &mut style_structs::Counters {
            self.modified_reset = true;
            self.counters.mutate()
        }

        /// Gets a mutable view of the current `Counters` style.
        pub fn take_counters(&mut self) -> UniqueArc<style_structs::Counters> {
            self.modified_reset = true;
            self.counters.take()
        }

        /// Gets a mutable view of the current `Counters` style.
        pub fn put_counters(&mut self, s: UniqueArc<style_structs::Counters>) {
            self.counters.put(s)
        }

        /// Gets a mutable view of the current `Counters` style,
        /// only if it's been mutated before.
        pub fn get_counters_if_mutated(&mut self)
                                                         -> Option<&mut style_structs::Counters> {
            self.counters.get_if_mutated()
        }

        /// Reset the current `Counters` style to its default value.
        pub fn reset_counters_struct(&mut self) {
            self.counters =
                StyleStructRef::Borrowed(self.reset_style.counters_arc());
        }
        /// Gets an immutable view of the current `Effects` style.
        pub fn get_effects(&self) -> &style_structs::Effects {
            &self.effects
        }

        /// Gets a mutable view of the current `Effects` style.
        pub fn mutate_effects(&mut self) -> &mut style_structs::Effects {
            self.modified_reset = true;
            self.effects.mutate()
        }

        /// Gets a mutable view of the current `Effects` style.
        pub fn take_effects(&mut self) -> UniqueArc<style_structs::Effects> {
            self.modified_reset = true;
            self.effects.take()
        }

        /// Gets a mutable view of the current `Effects` style.
        pub fn put_effects(&mut self, s: UniqueArc<style_structs::Effects>) {
            self.effects.put(s)
        }

        /// Gets a mutable view of the current `Effects` style,
        /// only if it's been mutated before.
        pub fn get_effects_if_mutated(&mut self)
                                                         -> Option<&mut style_structs::Effects> {
            self.effects.get_if_mutated()
        }

        /// Reset the current `Effects` style to its default value.
        pub fn reset_effects_struct(&mut self) {
            self.effects =
                StyleStructRef::Borrowed(self.reset_style.effects_arc());
        }
        /// Gets an immutable view of the current `Font` style.
        pub fn get_font(&self) -> &style_structs::Font {
            &self.font
        }

        /// Gets a mutable view of the current `Font` style.
        pub fn mutate_font(&mut self) -> &mut style_structs::Font {
            self.modified_reset = true;
            self.font.mutate()
        }

        /// Gets a mutable view of the current `Font` style.
        pub fn take_font(&mut self) -> UniqueArc<style_structs::Font> {
            self.modified_reset = true;
            self.font.take()
        }

        /// Gets a mutable view of the current `Font` style.
        pub fn put_font(&mut self, s: UniqueArc<style_structs::Font>) {
            self.font.put(s)
        }

        /// Gets a mutable view of the current `Font` style,
        /// only if it's been mutated before.
        pub fn get_font_if_mutated(&mut self)
                                                         -> Option<&mut style_structs::Font> {
            self.font.get_if_mutated()
        }

        /// Reset the current `Font` style to its default value.
        pub fn reset_font_struct(&mut self) {
            self.font =
                StyleStructRef::Borrowed(self.reset_style.font_arc());
        }
        /// Gets an immutable view of the current `InheritedBox` style.
        pub fn get_inheritedbox(&self) -> &style_structs::InheritedBox {
            &self.inheritedbox
        }

        /// Gets a mutable view of the current `InheritedBox` style.
        pub fn mutate_inheritedbox(&mut self) -> &mut style_structs::InheritedBox {
            self.modified_reset = true;
            self.inheritedbox.mutate()
        }

        /// Gets a mutable view of the current `InheritedBox` style.
        pub fn take_inheritedbox(&mut self) -> UniqueArc<style_structs::InheritedBox> {
            self.modified_reset = true;
            self.inheritedbox.take()
        }

        /// Gets a mutable view of the current `InheritedBox` style.
        pub fn put_inheritedbox(&mut self, s: UniqueArc<style_structs::InheritedBox>) {
            self.inheritedbox.put(s)
        }

        /// Gets a mutable view of the current `InheritedBox` style,
        /// only if it's been mutated before.
        pub fn get_inheritedbox_if_mutated(&mut self)
                                                         -> Option<&mut style_structs::InheritedBox> {
            self.inheritedbox.get_if_mutated()
        }

        /// Reset the current `InheritedBox` style to its default value.
        pub fn reset_inheritedbox_struct(&mut self) {
            self.inheritedbox =
                StyleStructRef::Borrowed(self.reset_style.inheritedbox_arc());
        }
        /// Gets an immutable view of the current `InheritedTable` style.
        pub fn get_inheritedtable(&self) -> &style_structs::InheritedTable {
            &self.inheritedtable
        }

        /// Gets a mutable view of the current `InheritedTable` style.
        pub fn mutate_inheritedtable(&mut self) -> &mut style_structs::InheritedTable {
            self.modified_reset = true;
            self.inheritedtable.mutate()
        }

        /// Gets a mutable view of the current `InheritedTable` style.
        pub fn take_inheritedtable(&mut self) -> UniqueArc<style_structs::InheritedTable> {
            self.modified_reset = true;
            self.inheritedtable.take()
        }

        /// Gets a mutable view of the current `InheritedTable` style.
        pub fn put_inheritedtable(&mut self, s: UniqueArc<style_structs::InheritedTable>) {
            self.inheritedtable.put(s)
        }

        /// Gets a mutable view of the current `InheritedTable` style,
        /// only if it's been mutated before.
        pub fn get_inheritedtable_if_mutated(&mut self)
                                                         -> Option<&mut style_structs::InheritedTable> {
            self.inheritedtable.get_if_mutated()
        }

        /// Reset the current `InheritedTable` style to its default value.
        pub fn reset_inheritedtable_struct(&mut self) {
            self.inheritedtable =
                StyleStructRef::Borrowed(self.reset_style.inheritedtable_arc());
        }
        /// Gets an immutable view of the current `InheritedText` style.
        pub fn get_inheritedtext(&self) -> &style_structs::InheritedText {
            &self.inheritedtext
        }

        /// Gets a mutable view of the current `InheritedText` style.
        pub fn mutate_inheritedtext(&mut self) -> &mut style_structs::InheritedText {
            self.modified_reset = true;
            self.inheritedtext.mutate()
        }

        /// Gets a mutable view of the current `InheritedText` style.
        pub fn take_inheritedtext(&mut self) -> UniqueArc<style_structs::InheritedText> {
            self.modified_reset = true;
            self.inheritedtext.take()
        }

        /// Gets a mutable view of the current `InheritedText` style.
        pub fn put_inheritedtext(&mut self, s: UniqueArc<style_structs::InheritedText>) {
            self.inheritedtext.put(s)
        }

        /// Gets a mutable view of the current `InheritedText` style,
        /// only if it's been mutated before.
        pub fn get_inheritedtext_if_mutated(&mut self)
                                                         -> Option<&mut style_structs::InheritedText> {
            self.inheritedtext.get_if_mutated()
        }

        /// Reset the current `InheritedText` style to its default value.
        pub fn reset_inheritedtext_struct(&mut self) {
            self.inheritedtext =
                StyleStructRef::Borrowed(self.reset_style.inheritedtext_arc());
        }
        /// Gets an immutable view of the current `List` style.
        pub fn get_list(&self) -> &style_structs::List {
            &self.list
        }

        /// Gets a mutable view of the current `List` style.
        pub fn mutate_list(&mut self) -> &mut style_structs::List {
            self.modified_reset = true;
            self.list.mutate()
        }

        /// Gets a mutable view of the current `List` style.
        pub fn take_list(&mut self) -> UniqueArc<style_structs::List> {
            self.modified_reset = true;
            self.list.take()
        }

        /// Gets a mutable view of the current `List` style.
        pub fn put_list(&mut self, s: UniqueArc<style_structs::List>) {
            self.list.put(s)
        }

        /// Gets a mutable view of the current `List` style,
        /// only if it's been mutated before.
        pub fn get_list_if_mutated(&mut self)
                                                         -> Option<&mut style_structs::List> {
            self.list.get_if_mutated()
        }

        /// Reset the current `List` style to its default value.
        pub fn reset_list_struct(&mut self) {
            self.list =
                StyleStructRef::Borrowed(self.reset_style.list_arc());
        }
        /// Gets an immutable view of the current `Margin` style.
        pub fn get_margin(&self) -> &style_structs::Margin {
            &self.margin
        }

        /// Gets a mutable view of the current `Margin` style.
        pub fn mutate_margin(&mut self) -> &mut style_structs::Margin {
            self.modified_reset = true;
            self.margin.mutate()
        }

        /// Gets a mutable view of the current `Margin` style.
        pub fn take_margin(&mut self) -> UniqueArc<style_structs::Margin> {
            self.modified_reset = true;
            self.margin.take()
        }

        /// Gets a mutable view of the current `Margin` style.
        pub fn put_margin(&mut self, s: UniqueArc<style_structs::Margin>) {
            self.margin.put(s)
        }

        /// Gets a mutable view of the current `Margin` style,
        /// only if it's been mutated before.
        pub fn get_margin_if_mutated(&mut self)
                                                         -> Option<&mut style_structs::Margin> {
            self.margin.get_if_mutated()
        }

        /// Reset the current `Margin` style to its default value.
        pub fn reset_margin_struct(&mut self) {
            self.margin =
                StyleStructRef::Borrowed(self.reset_style.margin_arc());
        }
        /// Gets an immutable view of the current `Outline` style.
        pub fn get_outline(&self) -> &style_structs::Outline {
            &self.outline
        }

        /// Gets a mutable view of the current `Outline` style.
        pub fn mutate_outline(&mut self) -> &mut style_structs::Outline {
            self.modified_reset = true;
            self.outline.mutate()
        }

        /// Gets a mutable view of the current `Outline` style.
        pub fn take_outline(&mut self) -> UniqueArc<style_structs::Outline> {
            self.modified_reset = true;
            self.outline.take()
        }

        /// Gets a mutable view of the current `Outline` style.
        pub fn put_outline(&mut self, s: UniqueArc<style_structs::Outline>) {
            self.outline.put(s)
        }

        /// Gets a mutable view of the current `Outline` style,
        /// only if it's been mutated before.
        pub fn get_outline_if_mutated(&mut self)
                                                         -> Option<&mut style_structs::Outline> {
            self.outline.get_if_mutated()
        }

        /// Reset the current `Outline` style to its default value.
        pub fn reset_outline_struct(&mut self) {
            self.outline =
                StyleStructRef::Borrowed(self.reset_style.outline_arc());
        }
        /// Gets an immutable view of the current `Padding` style.
        pub fn get_padding(&self) -> &style_structs::Padding {
            &self.padding
        }

        /// Gets a mutable view of the current `Padding` style.
        pub fn mutate_padding(&mut self) -> &mut style_structs::Padding {
            self.modified_reset = true;
            self.padding.mutate()
        }

        /// Gets a mutable view of the current `Padding` style.
        pub fn take_padding(&mut self) -> UniqueArc<style_structs::Padding> {
            self.modified_reset = true;
            self.padding.take()
        }

        /// Gets a mutable view of the current `Padding` style.
        pub fn put_padding(&mut self, s: UniqueArc<style_structs::Padding>) {
            self.padding.put(s)
        }

        /// Gets a mutable view of the current `Padding` style,
        /// only if it's been mutated before.
        pub fn get_padding_if_mutated(&mut self)
                                                         -> Option<&mut style_structs::Padding> {
            self.padding.get_if_mutated()
        }

        /// Reset the current `Padding` style to its default value.
        pub fn reset_padding_struct(&mut self) {
            self.padding =
                StyleStructRef::Borrowed(self.reset_style.padding_arc());
        }
        /// Gets an immutable view of the current `Pointing` style.
        pub fn get_pointing(&self) -> &style_structs::Pointing {
            &self.pointing
        }

        /// Gets a mutable view of the current `Pointing` style.
        pub fn mutate_pointing(&mut self) -> &mut style_structs::Pointing {
            self.modified_reset = true;
            self.pointing.mutate()
        }

        /// Gets a mutable view of the current `Pointing` style.
        pub fn take_pointing(&mut self) -> UniqueArc<style_structs::Pointing> {
            self.modified_reset = true;
            self.pointing.take()
        }

        /// Gets a mutable view of the current `Pointing` style.
        pub fn put_pointing(&mut self, s: UniqueArc<style_structs::Pointing>) {
            self.pointing.put(s)
        }

        /// Gets a mutable view of the current `Pointing` style,
        /// only if it's been mutated before.
        pub fn get_pointing_if_mutated(&mut self)
                                                         -> Option<&mut style_structs::Pointing> {
            self.pointing.get_if_mutated()
        }

        /// Reset the current `Pointing` style to its default value.
        pub fn reset_pointing_struct(&mut self) {
            self.pointing =
                StyleStructRef::Borrowed(self.reset_style.pointing_arc());
        }
        /// Gets an immutable view of the current `Position` style.
        pub fn get_position(&self) -> &style_structs::Position {
            &self.position
        }

        /// Gets a mutable view of the current `Position` style.
        pub fn mutate_position(&mut self) -> &mut style_structs::Position {
            self.modified_reset = true;
            self.position.mutate()
        }

        /// Gets a mutable view of the current `Position` style.
        pub fn take_position(&mut self) -> UniqueArc<style_structs::Position> {
            self.modified_reset = true;
            self.position.take()
        }

        /// Gets a mutable view of the current `Position` style.
        pub fn put_position(&mut self, s: UniqueArc<style_structs::Position>) {
            self.position.put(s)
        }

        /// Gets a mutable view of the current `Position` style,
        /// only if it's been mutated before.
        pub fn get_position_if_mutated(&mut self)
                                                         -> Option<&mut style_structs::Position> {
            self.position.get_if_mutated()
        }

        /// Reset the current `Position` style to its default value.
        pub fn reset_position_struct(&mut self) {
            self.position =
                StyleStructRef::Borrowed(self.reset_style.position_arc());
        }
        /// Gets an immutable view of the current `Table` style.
        pub fn get_table(&self) -> &style_structs::Table {
            &self.table
        }

        /// Gets a mutable view of the current `Table` style.
        pub fn mutate_table(&mut self) -> &mut style_structs::Table {
            self.modified_reset = true;
            self.table.mutate()
        }

        /// Gets a mutable view of the current `Table` style.
        pub fn take_table(&mut self) -> UniqueArc<style_structs::Table> {
            self.modified_reset = true;
            self.table.take()
        }

        /// Gets a mutable view of the current `Table` style.
        pub fn put_table(&mut self, s: UniqueArc<style_structs::Table>) {
            self.table.put(s)
        }

        /// Gets a mutable view of the current `Table` style,
        /// only if it's been mutated before.
        pub fn get_table_if_mutated(&mut self)
                                                         -> Option<&mut style_structs::Table> {
            self.table.get_if_mutated()
        }

        /// Reset the current `Table` style to its default value.
        pub fn reset_table_struct(&mut self) {
            self.table =
                StyleStructRef::Borrowed(self.reset_style.table_arc());
        }
        /// Gets an immutable view of the current `Text` style.
        pub fn get_text(&self) -> &style_structs::Text {
            &self.text
        }

        /// Gets a mutable view of the current `Text` style.
        pub fn mutate_text(&mut self) -> &mut style_structs::Text {
            self.modified_reset = true;
            self.text.mutate()
        }

        /// Gets a mutable view of the current `Text` style.
        pub fn take_text(&mut self) -> UniqueArc<style_structs::Text> {
            self.modified_reset = true;
            self.text.take()
        }

        /// Gets a mutable view of the current `Text` style.
        pub fn put_text(&mut self, s: UniqueArc<style_structs::Text>) {
            self.text.put(s)
        }

        /// Gets a mutable view of the current `Text` style,
        /// only if it's been mutated before.
        pub fn get_text_if_mutated(&mut self)
                                                         -> Option<&mut style_structs::Text> {
            self.text.get_if_mutated()
        }

        /// Reset the current `Text` style to its default value.
        pub fn reset_text_struct(&mut self) {
            self.text =
                StyleStructRef::Borrowed(self.reset_style.text_arc());
        }
        /// Gets an immutable view of the current `UI` style.
        pub fn get_ui(&self) -> &style_structs::UI {
            &self.ui
        }

        /// Gets a mutable view of the current `UI` style.
        pub fn mutate_ui(&mut self) -> &mut style_structs::UI {
            self.modified_reset = true;
            self.ui.mutate()
        }

        /// Gets a mutable view of the current `UI` style.
        pub fn take_ui(&mut self) -> UniqueArc<style_structs::UI> {
            self.modified_reset = true;
            self.ui.take()
        }

        /// Gets a mutable view of the current `UI` style.
        pub fn put_ui(&mut self, s: UniqueArc<style_structs::UI>) {
            self.ui.put(s)
        }

        /// Gets a mutable view of the current `UI` style,
        /// only if it's been mutated before.
        pub fn get_ui_if_mutated(&mut self)
                                                         -> Option<&mut style_structs::UI> {
            self.ui.get_if_mutated()
        }

        /// Reset the current `UI` style to its default value.
        pub fn reset_ui_struct(&mut self) {
            self.ui =
                StyleStructRef::Borrowed(self.reset_style.ui_arc());
        }
        /// Gets an immutable view of the current `InheritedSVG` style.
        pub fn get_inheritedsvg(&self) -> &style_structs::InheritedSVG {
            &self.inheritedsvg
        }

        /// Gets a mutable view of the current `InheritedSVG` style.
        pub fn mutate_inheritedsvg(&mut self) -> &mut style_structs::InheritedSVG {
            self.modified_reset = true;
            self.inheritedsvg.mutate()
        }

        /// Gets a mutable view of the current `InheritedSVG` style.
        pub fn take_inheritedsvg(&mut self) -> UniqueArc<style_structs::InheritedSVG> {
            self.modified_reset = true;
            self.inheritedsvg.take()
        }

        /// Gets a mutable view of the current `InheritedSVG` style.
        pub fn put_inheritedsvg(&mut self, s: UniqueArc<style_structs::InheritedSVG>) {
            self.inheritedsvg.put(s)
        }

        /// Gets a mutable view of the current `InheritedSVG` style,
        /// only if it's been mutated before.
        pub fn get_inheritedsvg_if_mutated(&mut self)
                                                         -> Option<&mut style_structs::InheritedSVG> {
            self.inheritedsvg.get_if_mutated()
        }

        /// Reset the current `InheritedSVG` style to its default value.
        pub fn reset_inheritedsvg_struct(&mut self) {
            self.inheritedsvg =
                StyleStructRef::Borrowed(self.reset_style.inheritedsvg_arc());
        }
        /// Gets an immutable view of the current `SVG` style.
        pub fn get_svg(&self) -> &style_structs::SVG {
            &self.svg
        }

        /// Gets a mutable view of the current `SVG` style.
        pub fn mutate_svg(&mut self) -> &mut style_structs::SVG {
            self.modified_reset = true;
            self.svg.mutate()
        }

        /// Gets a mutable view of the current `SVG` style.
        pub fn take_svg(&mut self) -> UniqueArc<style_structs::SVG> {
            self.modified_reset = true;
            self.svg.take()
        }

        /// Gets a mutable view of the current `SVG` style.
        pub fn put_svg(&mut self, s: UniqueArc<style_structs::SVG>) {
            self.svg.put(s)
        }

        /// Gets a mutable view of the current `SVG` style,
        /// only if it's been mutated before.
        pub fn get_svg_if_mutated(&mut self)
                                                         -> Option<&mut style_structs::SVG> {
            self.svg.get_if_mutated()
        }

        /// Reset the current `SVG` style to its default value.
        pub fn reset_svg_struct(&mut self) {
            self.svg =
                StyleStructRef::Borrowed(self.reset_style.svg_arc());
        }
        /// Gets an immutable view of the current `XUL` style.
        pub fn get_xul(&self) -> &style_structs::XUL {
            &self.xul
        }

        /// Gets a mutable view of the current `XUL` style.
        pub fn mutate_xul(&mut self) -> &mut style_structs::XUL {
            self.modified_reset = true;
            self.xul.mutate()
        }

        /// Gets a mutable view of the current `XUL` style.
        pub fn take_xul(&mut self) -> UniqueArc<style_structs::XUL> {
            self.modified_reset = true;
            self.xul.take()
        }

        /// Gets a mutable view of the current `XUL` style.
        pub fn put_xul(&mut self, s: UniqueArc<style_structs::XUL>) {
            self.xul.put(s)
        }

        /// Gets a mutable view of the current `XUL` style,
        /// only if it's been mutated before.
        pub fn get_xul_if_mutated(&mut self)
                                                         -> Option<&mut style_structs::XUL> {
            self.xul.get_if_mutated()
        }

        /// Reset the current `XUL` style to its default value.
        pub fn reset_xul_struct(&mut self) {
            self.xul =
                StyleStructRef::Borrowed(self.reset_style.xul_arc());
        }

    /// Returns whether this computed style represents a floated object.
    pub fn floated(&self) -> bool {
        self.get_box().clone_float() != longhands::float::computed_value::T::none
    }

    /// Returns whether this computed style represents an out of flow-positioned
    /// object.
    pub fn out_of_flow_positioned(&self) -> bool {
        use properties::longhands::position::computed_value::T as position;
        matches!(self.get_box().clone_position(),
                 position::absolute | position::fixed)
    }

    /// Whether this style has a top-layer style. That's implemented in Gecko
    /// via the -moz-top-layer property, but servo doesn't have any concept of a
    /// top layer (yet, it's needed for fullscreen).
    #[cfg(feature = "servo")]
    pub fn in_top_layer(&self) -> bool { false }

    /// Whether this style has a top-layer style.
    #[cfg(feature = "gecko")]
    pub fn in_top_layer(&self) -> bool {
        matches!(self.get_box().clone__moz_top_layer(),
                 longhands::_moz_top_layer::computed_value::T::top)
    }

    /// Clears the "have any reset structs been modified" flag.
    fn clear_modified_reset(&mut self) {
        self.modified_reset = false;
    }

    /// Returns whether we have mutated any reset structs since the the last
    /// time `clear_modified_reset` was called.
    fn modified_reset(&self) -> bool {
        self.modified_reset
    }

    /// Turns this `StyleBuilder` into a proper `ComputedValues` instance.
    pub fn build(self) -> Arc<ComputedValues> {
        ComputedValues::new(
            self.device,
            self.parent_style,
            self.pseudo,
            self.custom_properties,
            self.writing_mode,
            self.font_size_keyword,
            self.flags,
            self.rules,
            self.visited_style,
            self.background.build(),
            self.border.build(),
            self.box_.build(),
            self.color.build(),
            self.column.build(),
            self.counters.build(),
            self.effects.build(),
            self.font.build(),
            self.inheritedbox.build(),
            self.inheritedtable.build(),
            self.inheritedtext.build(),
            self.list.build(),
            self.margin.build(),
            self.outline.build(),
            self.padding.build(),
            self.pointing.build(),
            self.position.build(),
            self.table.build(),
            self.text.build(),
            self.ui.build(),
            self.inheritedsvg.build(),
            self.svg.build(),
            self.xul.build(),
        )
    }

    /// Get the custom properties map if necessary.
    ///
    /// Cloning the Arc here is fine because it only happens in the case where
    /// we have custom properties, and those are both rare and expensive.
    fn custom_properties(&self) -> Option<Arc<::custom_properties::CustomPropertiesMap>> {
        self.custom_properties.clone()
    }

    /// Access to various information about our inherited styles.  We don't
    /// expose an inherited ComputedValues directly, because in the
    /// ::first-line case some of the inherited information needs to come from
    /// one ComputedValues instance and some from a different one.

    /// Inherited font bits.
    pub fn inherited_font_computation_data(&self) -> &FontComputationData {
        &self.inherited_style.font_computation_data
    }

    /// Inherited writing-mode.
    pub fn inherited_writing_mode(&self) -> &WritingMode {
        &self.inherited_style.writing_mode
    }

    /// Inherited style flags.
    pub fn inherited_flags(&self) -> &ComputedValueFlags {
        &self.inherited_style.flags
    }

    /// And access to inherited style structs.
        /// Gets our inherited `Background`.  We don't name these
        /// accessors `inherited_background` because we already
        /// have things like "box" vs "inherited_box" as struct names.  Do the
        /// next-best thing and call them `parent_background`
        /// instead.
        pub fn get_parent_background(&self) -> &style_structs::Background {
            self.inherited_style_ignoring_first_line.get_background()
        }
        /// Gets our inherited `Border`.  We don't name these
        /// accessors `inherited_border` because we already
        /// have things like "box" vs "inherited_box" as struct names.  Do the
        /// next-best thing and call them `parent_border`
        /// instead.
        pub fn get_parent_border(&self) -> &style_structs::Border {
            self.inherited_style_ignoring_first_line.get_border()
        }
        /// Gets our inherited `Box`.  We don't name these
        /// accessors `inherited_box` because we already
        /// have things like "box" vs "inherited_box" as struct names.  Do the
        /// next-best thing and call them `parent_box`
        /// instead.
        pub fn get_parent_box(&self) -> &style_structs::Box {
            self.inherited_style_ignoring_first_line.get_box()
        }
        /// Gets our inherited `Color`.  We don't name these
        /// accessors `inherited_color` because we already
        /// have things like "box" vs "inherited_box" as struct names.  Do the
        /// next-best thing and call them `parent_color`
        /// instead.
        pub fn get_parent_color(&self) -> &style_structs::Color {
            self.inherited_style.get_color()
        }
        /// Gets our inherited `Column`.  We don't name these
        /// accessors `inherited_column` because we already
        /// have things like "box" vs "inherited_box" as struct names.  Do the
        /// next-best thing and call them `parent_column`
        /// instead.
        pub fn get_parent_column(&self) -> &style_structs::Column {
            self.inherited_style_ignoring_first_line.get_column()
        }
        /// Gets our inherited `Counters`.  We don't name these
        /// accessors `inherited_counters` because we already
        /// have things like "box" vs "inherited_box" as struct names.  Do the
        /// next-best thing and call them `parent_counters`
        /// instead.
        pub fn get_parent_counters(&self) -> &style_structs::Counters {
            self.inherited_style_ignoring_first_line.get_counters()
        }
        /// Gets our inherited `Effects`.  We don't name these
        /// accessors `inherited_effects` because we already
        /// have things like "box" vs "inherited_box" as struct names.  Do the
        /// next-best thing and call them `parent_effects`
        /// instead.
        pub fn get_parent_effects(&self) -> &style_structs::Effects {
            self.inherited_style_ignoring_first_line.get_effects()
        }
        /// Gets our inherited `Font`.  We don't name these
        /// accessors `inherited_font` because we already
        /// have things like "box" vs "inherited_box" as struct names.  Do the
        /// next-best thing and call them `parent_font`
        /// instead.
        pub fn get_parent_font(&self) -> &style_structs::Font {
            self.inherited_style.get_font()
        }
        /// Gets our inherited `InheritedBox`.  We don't name these
        /// accessors `inherited_inheritedbox` because we already
        /// have things like "box" vs "inherited_box" as struct names.  Do the
        /// next-best thing and call them `parent_inheritedbox`
        /// instead.
        pub fn get_parent_inheritedbox(&self) -> &style_structs::InheritedBox {
            self.inherited_style.get_inheritedbox()
        }
        /// Gets our inherited `InheritedTable`.  We don't name these
        /// accessors `inherited_inheritedtable` because we already
        /// have things like "box" vs "inherited_box" as struct names.  Do the
        /// next-best thing and call them `parent_inheritedtable`
        /// instead.
        pub fn get_parent_inheritedtable(&self) -> &style_structs::InheritedTable {
            self.inherited_style.get_inheritedtable()
        }
        /// Gets our inherited `InheritedText`.  We don't name these
        /// accessors `inherited_inheritedtext` because we already
        /// have things like "box" vs "inherited_box" as struct names.  Do the
        /// next-best thing and call them `parent_inheritedtext`
        /// instead.
        pub fn get_parent_inheritedtext(&self) -> &style_structs::InheritedText {
            self.inherited_style.get_inheritedtext()
        }
        /// Gets our inherited `List`.  We don't name these
        /// accessors `inherited_list` because we already
        /// have things like "box" vs "inherited_box" as struct names.  Do the
        /// next-best thing and call them `parent_list`
        /// instead.
        pub fn get_parent_list(&self) -> &style_structs::List {
            self.inherited_style.get_list()
        }
        /// Gets our inherited `Margin`.  We don't name these
        /// accessors `inherited_margin` because we already
        /// have things like "box" vs "inherited_box" as struct names.  Do the
        /// next-best thing and call them `parent_margin`
        /// instead.
        pub fn get_parent_margin(&self) -> &style_structs::Margin {
            self.inherited_style_ignoring_first_line.get_margin()
        }
        /// Gets our inherited `Outline`.  We don't name these
        /// accessors `inherited_outline` because we already
        /// have things like "box" vs "inherited_box" as struct names.  Do the
        /// next-best thing and call them `parent_outline`
        /// instead.
        pub fn get_parent_outline(&self) -> &style_structs::Outline {
            self.inherited_style_ignoring_first_line.get_outline()
        }
        /// Gets our inherited `Padding`.  We don't name these
        /// accessors `inherited_padding` because we already
        /// have things like "box" vs "inherited_box" as struct names.  Do the
        /// next-best thing and call them `parent_padding`
        /// instead.
        pub fn get_parent_padding(&self) -> &style_structs::Padding {
            self.inherited_style_ignoring_first_line.get_padding()
        }
        /// Gets our inherited `Pointing`.  We don't name these
        /// accessors `inherited_pointing` because we already
        /// have things like "box" vs "inherited_box" as struct names.  Do the
        /// next-best thing and call them `parent_pointing`
        /// instead.
        pub fn get_parent_pointing(&self) -> &style_structs::Pointing {
            self.inherited_style.get_pointing()
        }
        /// Gets our inherited `Position`.  We don't name these
        /// accessors `inherited_position` because we already
        /// have things like "box" vs "inherited_box" as struct names.  Do the
        /// next-best thing and call them `parent_position`
        /// instead.
        pub fn get_parent_position(&self) -> &style_structs::Position {
            self.inherited_style_ignoring_first_line.get_position()
        }
        /// Gets our inherited `Table`.  We don't name these
        /// accessors `inherited_table` because we already
        /// have things like "box" vs "inherited_box" as struct names.  Do the
        /// next-best thing and call them `parent_table`
        /// instead.
        pub fn get_parent_table(&self) -> &style_structs::Table {
            self.inherited_style_ignoring_first_line.get_table()
        }
        /// Gets our inherited `Text`.  We don't name these
        /// accessors `inherited_text` because we already
        /// have things like "box" vs "inherited_box" as struct names.  Do the
        /// next-best thing and call them `parent_text`
        /// instead.
        pub fn get_parent_text(&self) -> &style_structs::Text {
            self.inherited_style_ignoring_first_line.get_text()
        }
        /// Gets our inherited `UI`.  We don't name these
        /// accessors `inherited_ui` because we already
        /// have things like "box" vs "inherited_box" as struct names.  Do the
        /// next-best thing and call them `parent_ui`
        /// instead.
        pub fn get_parent_ui(&self) -> &style_structs::UI {
            self.inherited_style_ignoring_first_line.get_ui()
        }
        /// Gets our inherited `InheritedSVG`.  We don't name these
        /// accessors `inherited_inheritedsvg` because we already
        /// have things like "box" vs "inherited_box" as struct names.  Do the
        /// next-best thing and call them `parent_inheritedsvg`
        /// instead.
        pub fn get_parent_inheritedsvg(&self) -> &style_structs::InheritedSVG {
            self.inherited_style.get_inheritedsvg()
        }
        /// Gets our inherited `SVG`.  We don't name these
        /// accessors `inherited_svg` because we already
        /// have things like "box" vs "inherited_box" as struct names.  Do the
        /// next-best thing and call them `parent_svg`
        /// instead.
        pub fn get_parent_svg(&self) -> &style_structs::SVG {
            self.inherited_style_ignoring_first_line.get_svg()
        }
        /// Gets our inherited `XUL`.  We don't name these
        /// accessors `inherited_xul` because we already
        /// have things like "box" vs "inherited_box" as struct names.  Do the
        /// next-best thing and call them `parent_xul`
        /// instead.
        pub fn get_parent_xul(&self) -> &style_structs::XUL {
            self.inherited_style_ignoring_first_line.get_xul()
        }
}

#[cfg(feature = "servo")]
pub use self::lazy_static_module::INITIAL_SERVO_VALUES;

// Use a module to work around #[cfg] on lazy_static! not being applied to every generated item.
#[cfg(feature = "servo")]
#[allow(missing_docs)]
mod lazy_static_module {
    use logical_geometry::WritingMode;
    use servo_arc::Arc;
    use super::{ComputedValues, ComputedValuesInner, longhands, style_structs};
    use super::default_font_size_keyword;
    use super::computed_value_flags::ComputedValueFlags;

    /// The initial values for all style structs as defined by the specification.
    lazy_static! {
        pub static ref INITIAL_SERVO_VALUES: ComputedValues = ComputedValues {
            inner: ComputedValuesInner {
                    background: Arc::new(style_structs::Background {
                            background_color: longhands::background_color::get_initial_value(),
                            background_image: longhands::background_image::get_initial_value(),
                            background_position_x: longhands::background_position_x::get_initial_value(),
                            background_position_y: longhands::background_position_y::get_initial_value(),
                            background_repeat: longhands::background_repeat::get_initial_value(),
                            background_attachment: longhands::background_attachment::get_initial_value(),
                            background_clip: longhands::background_clip::get_initial_value(),
                            background_origin: longhands::background_origin::get_initial_value(),
                            background_size: longhands::background_size::get_initial_value(),
                            background_blend_mode: longhands::background_blend_mode::get_initial_value(),
                    }),
                    border: Arc::new(style_structs::Border {
                            border_top_color: longhands::border_top_color::get_initial_value(),
                            border_top_style: longhands::border_top_style::get_initial_value(),
                            border_top_width: longhands::border_top_width::get_initial_value(),
                            border_left_color: longhands::border_left_color::get_initial_value(),
                            border_left_style: longhands::border_left_style::get_initial_value(),
                            border_left_width: longhands::border_left_width::get_initial_value(),
                            border_bottom_color: longhands::border_bottom_color::get_initial_value(),
                            border_bottom_style: longhands::border_bottom_style::get_initial_value(),
                            border_bottom_width: longhands::border_bottom_width::get_initial_value(),
                            border_right_color: longhands::border_right_color::get_initial_value(),
                            border_right_style: longhands::border_right_style::get_initial_value(),
                            border_right_width: longhands::border_right_width::get_initial_value(),
                            border_block_start_color: longhands::border_block_start_color::get_initial_value(),
                            border_block_start_style: longhands::border_block_start_style::get_initial_value(),
                            border_block_start_width: longhands::border_block_start_width::get_initial_value(),
                            border_block_end_color: longhands::border_block_end_color::get_initial_value(),
                            border_block_end_style: longhands::border_block_end_style::get_initial_value(),
                            border_block_end_width: longhands::border_block_end_width::get_initial_value(),
                            border_inline_start_color: longhands::border_inline_start_color::get_initial_value(),
                            border_inline_start_style: longhands::border_inline_start_style::get_initial_value(),
                            border_inline_start_width: longhands::border_inline_start_width::get_initial_value(),
                            border_inline_end_color: longhands::border_inline_end_color::get_initial_value(),
                            border_inline_end_style: longhands::border_inline_end_style::get_initial_value(),
                            border_inline_end_width: longhands::border_inline_end_width::get_initial_value(),
                            border_top_left_radius: longhands::border_top_left_radius::get_initial_value(),
                            border_top_right_radius: longhands::border_top_right_radius::get_initial_value(),
                            border_bottom_right_radius: longhands::border_bottom_right_radius::get_initial_value(),
                            border_bottom_left_radius: longhands::border_bottom_left_radius::get_initial_value(),
                            _moz_border_top_colors: longhands::_moz_border_top_colors::get_initial_value(),
                            _moz_border_left_colors: longhands::_moz_border_left_colors::get_initial_value(),
                            _moz_border_bottom_colors: longhands::_moz_border_bottom_colors::get_initial_value(),
                            _moz_border_right_colors: longhands::_moz_border_right_colors::get_initial_value(),
                            box_decoration_break: longhands::box_decoration_break::get_initial_value(),
                            _moz_float_edge: longhands::_moz_float_edge::get_initial_value(),
                            border_image_source: longhands::border_image_source::get_initial_value(),
                            border_image_outset: longhands::border_image_outset::get_initial_value(),
                            border_image_repeat: longhands::border_image_repeat::get_initial_value(),
                            border_image_width: longhands::border_image_width::get_initial_value(),
                            border_image_slice: longhands::border_image_slice::get_initial_value(),
                    }),
                    box_: Arc::new(style_structs::Box {
                            display: longhands::display::get_initial_value(),
                            _moz_top_layer: longhands::_moz_top_layer::get_initial_value(),
                            position: longhands::position::get_initial_value(),
                            float: longhands::float::get_initial_value(),
                            clear: longhands::clear::get_initial_value(),
                            vertical_align: longhands::vertical_align::get_initial_value(),
                            overflow_clip_box: longhands::overflow_clip_box::get_initial_value(),
                            overflow_x: longhands::overflow_x::get_initial_value(),
                            overflow_y: longhands::overflow_y::get_initial_value(),
                            transition_duration: longhands::transition_duration::get_initial_value(),
                            transition_timing_function: longhands::transition_timing_function::get_initial_value(),
                            transition_property: longhands::transition_property::get_initial_value(),
                            transition_delay: longhands::transition_delay::get_initial_value(),
                            animation_name: longhands::animation_name::get_initial_value(),
                            animation_duration: longhands::animation_duration::get_initial_value(),
                            animation_timing_function: longhands::animation_timing_function::get_initial_value(),
                            animation_iteration_count: longhands::animation_iteration_count::get_initial_value(),
                            animation_direction: longhands::animation_direction::get_initial_value(),
                            animation_play_state: longhands::animation_play_state::get_initial_value(),
                            animation_fill_mode: longhands::animation_fill_mode::get_initial_value(),
                            animation_delay: longhands::animation_delay::get_initial_value(),
                            scroll_snap_points_x: longhands::scroll_snap_points_x::get_initial_value(),
                            scroll_snap_points_y: longhands::scroll_snap_points_y::get_initial_value(),
                            scroll_snap_destination: longhands::scroll_snap_destination::get_initial_value(),
                            scroll_snap_coordinate: longhands::scroll_snap_coordinate::get_initial_value(),
                            transform: longhands::transform::get_initial_value(),
                            scroll_behavior: longhands::scroll_behavior::get_initial_value(),
                            scroll_snap_type_x: longhands::scroll_snap_type_x::get_initial_value(),
                            scroll_snap_type_y: longhands::scroll_snap_type_y::get_initial_value(),
                            isolation: longhands::isolation::get_initial_value(),
                            page_break_after: longhands::page_break_after::get_initial_value(),
                            page_break_before: longhands::page_break_before::get_initial_value(),
                            page_break_inside: longhands::page_break_inside::get_initial_value(),
                            resize: longhands::resize::get_initial_value(),
                            perspective: longhands::perspective::get_initial_value(),
                            perspective_origin: longhands::perspective_origin::get_initial_value(),
                            backface_visibility: longhands::backface_visibility::get_initial_value(),
                            transform_box: longhands::transform_box::get_initial_value(),
                            transform_style: longhands::transform_style::get_initial_value(),
                            transform_origin: longhands::transform_origin::get_initial_value(),
                            contain: longhands::contain::get_initial_value(),
                            _moz_appearance: longhands::_moz_appearance::get_initial_value(),
                            _moz_binding: longhands::_moz_binding::get_initial_value(),
                            _moz_orient: longhands::_moz_orient::get_initial_value(),
                            will_change: longhands::will_change::get_initial_value(),
                            shape_outside: longhands::shape_outside::get_initial_value(),
                            touch_action: longhands::touch_action::get_initial_value(),
                    }),
                    color: Arc::new(style_structs::Color {
                            color: longhands::color::get_initial_value(),
                    }),
                    column: Arc::new(style_structs::Column {
                            column_width: longhands::column_width::get_initial_value(),
                            column_count: longhands::column_count::get_initial_value(),
                            column_gap: longhands::column_gap::get_initial_value(),
                            column_fill: longhands::column_fill::get_initial_value(),
                            column_rule_width: longhands::column_rule_width::get_initial_value(),
                            column_rule_color: longhands::column_rule_color::get_initial_value(),
                            column_span: longhands::column_span::get_initial_value(),
                            column_rule_style: longhands::column_rule_style::get_initial_value(),
                    }),
                    counters: Arc::new(style_structs::Counters {
                            content: longhands::content::get_initial_value(),
                            counter_increment: longhands::counter_increment::get_initial_value(),
                            counter_reset: longhands::counter_reset::get_initial_value(),
                    }),
                    effects: Arc::new(style_structs::Effects {
                            opacity: longhands::opacity::get_initial_value(),
                            box_shadow: longhands::box_shadow::get_initial_value(),
                            clip: longhands::clip::get_initial_value(),
                            filter: longhands::filter::get_initial_value(),
                            mix_blend_mode: longhands::mix_blend_mode::get_initial_value(),
                    }),
                    font: Arc::new(style_structs::Font {
                            font_family: longhands::font_family::get_initial_value(),
                            font_style: longhands::font_style::get_initial_value(),
                            font_variant_caps: longhands::font_variant_caps::get_initial_value(),
                            font_weight: longhands::font_weight::get_initial_value(),
                            font_size: longhands::font_size::get_initial_value(),
                            font_size_adjust: longhands::font_size_adjust::get_initial_value(),
                            font_synthesis: longhands::font_synthesis::get_initial_value(),
                            font_stretch: longhands::font_stretch::get_initial_value(),
                            font_kerning: longhands::font_kerning::get_initial_value(),
                            font_variant_alternates: longhands::font_variant_alternates::get_initial_value(),
                            font_variant_east_asian: longhands::font_variant_east_asian::get_initial_value(),
                            font_variant_ligatures: longhands::font_variant_ligatures::get_initial_value(),
                            font_variant_numeric: longhands::font_variant_numeric::get_initial_value(),
                            font_variant_position: longhands::font_variant_position::get_initial_value(),
                            font_feature_settings: longhands::font_feature_settings::get_initial_value(),
                            font_variation_settings: longhands::font_variation_settings::get_initial_value(),
                            font_language_override: longhands::font_language_override::get_initial_value(),
                            _x_lang: longhands::_x_lang::get_initial_value(),
                            _moz_script_size_multiplier: longhands::_moz_script_size_multiplier::get_initial_value(),
                            _moz_script_level: longhands::_moz_script_level::get_initial_value(),
                            _moz_math_display: longhands::_moz_math_display::get_initial_value(),
                            _moz_math_variant: longhands::_moz_math_variant::get_initial_value(),
                            _moz_script_min_size: longhands::_moz_script_min_size::get_initial_value(),
                            _x_text_zoom: longhands::_x_text_zoom::get_initial_value(),
                            _moz_osx_font_smoothing: longhands::_moz_osx_font_smoothing::get_initial_value(),
                            _moz_min_font_size_ratio: longhands::_moz_min_font_size_ratio::get_initial_value(),
                            hash: 0,
                    }),
                    inheritedbox: Arc::new(style_structs::InheritedBox {
                            visibility: longhands::visibility::get_initial_value(),
                            writing_mode: longhands::writing_mode::get_initial_value(),
                            direction: longhands::direction::get_initial_value(),
                            text_orientation: longhands::text_orientation::get_initial_value(),
                            color_adjust: longhands::color_adjust::get_initial_value(),
                            image_rendering: longhands::image_rendering::get_initial_value(),
                            image_orientation: longhands::image_orientation::get_initial_value(),
                    }),
                    inheritedtable: Arc::new(style_structs::InheritedTable {
                            border_collapse: longhands::border_collapse::get_initial_value(),
                            empty_cells: longhands::empty_cells::get_initial_value(),
                            caption_side: longhands::caption_side::get_initial_value(),
                            border_spacing: longhands::border_spacing::get_initial_value(),
                    }),
                    inheritedtext: Arc::new(style_structs::InheritedText {
                            line_height: longhands::line_height::get_initial_value(),
                            text_transform: longhands::text_transform::get_initial_value(),
                            hyphens: longhands::hyphens::get_initial_value(),
                            _moz_text_size_adjust: longhands::_moz_text_size_adjust::get_initial_value(),
                            text_indent: longhands::text_indent::get_initial_value(),
                            overflow_wrap: longhands::overflow_wrap::get_initial_value(),
                            word_break: longhands::word_break::get_initial_value(),
                            text_justify: longhands::text_justify::get_initial_value(),
                            text_align_last: longhands::text_align_last::get_initial_value(),
                            text_align: longhands::text_align::get_initial_value(),
                            letter_spacing: longhands::letter_spacing::get_initial_value(),
                            word_spacing: longhands::word_spacing::get_initial_value(),
                            white_space: longhands::white_space::get_initial_value(),
                            text_shadow: longhands::text_shadow::get_initial_value(),
                            text_emphasis_style: longhands::text_emphasis_style::get_initial_value(),
                            text_emphasis_position: longhands::text_emphasis_position::get_initial_value(),
                            text_emphasis_color: longhands::text_emphasis_color::get_initial_value(),
                            _moz_tab_size: longhands::_moz_tab_size::get_initial_value(),
                            _webkit_text_fill_color: longhands::_webkit_text_fill_color::get_initial_value(),
                            _webkit_text_stroke_color: longhands::_webkit_text_stroke_color::get_initial_value(),
                            _webkit_text_stroke_width: longhands::_webkit_text_stroke_width::get_initial_value(),
                            ruby_align: longhands::ruby_align::get_initial_value(),
                            ruby_position: longhands::ruby_position::get_initial_value(),
                            text_combine_upright: longhands::text_combine_upright::get_initial_value(),
                            text_rendering: longhands::text_rendering::get_initial_value(),
                            _moz_control_character_visibility: longhands::_moz_control_character_visibility::get_initial_value(),
                    }),
                    list: Arc::new(style_structs::List {
                            list_style_position: longhands::list_style_position::get_initial_value(),
                            list_style_type: longhands::list_style_type::get_initial_value(),
                            list_style_image: longhands::list_style_image::get_initial_value(),
                            quotes: longhands::quotes::get_initial_value(),
                            _moz_image_region: longhands::_moz_image_region::get_initial_value(),
                    }),
                    margin: Arc::new(style_structs::Margin {
                            margin_top: longhands::margin_top::get_initial_value(),
                            margin_left: longhands::margin_left::get_initial_value(),
                            margin_bottom: longhands::margin_bottom::get_initial_value(),
                            margin_right: longhands::margin_right::get_initial_value(),
                            margin_block_start: longhands::margin_block_start::get_initial_value(),
                            margin_block_end: longhands::margin_block_end::get_initial_value(),
                            margin_inline_start: longhands::margin_inline_start::get_initial_value(),
                            margin_inline_end: longhands::margin_inline_end::get_initial_value(),
                    }),
                    outline: Arc::new(style_structs::Outline {
                            outline_color: longhands::outline_color::get_initial_value(),
                            outline_style: longhands::outline_style::get_initial_value(),
                            outline_width: longhands::outline_width::get_initial_value(),
                            _moz_outline_radius_topleft: longhands::_moz_outline_radius_topleft::get_initial_value(),
                            _moz_outline_radius_topright: longhands::_moz_outline_radius_topright::get_initial_value(),
                            _moz_outline_radius_bottomright: longhands::_moz_outline_radius_bottomright::get_initial_value(),
                            _moz_outline_radius_bottomleft: longhands::_moz_outline_radius_bottomleft::get_initial_value(),
                            outline_offset: longhands::outline_offset::get_initial_value(),
                    }),
                    padding: Arc::new(style_structs::Padding {
                            padding_top: longhands::padding_top::get_initial_value(),
                            padding_left: longhands::padding_left::get_initial_value(),
                            padding_bottom: longhands::padding_bottom::get_initial_value(),
                            padding_right: longhands::padding_right::get_initial_value(),
                            padding_block_start: longhands::padding_block_start::get_initial_value(),
                            padding_block_end: longhands::padding_block_end::get_initial_value(),
                            padding_inline_start: longhands::padding_inline_start::get_initial_value(),
                            padding_inline_end: longhands::padding_inline_end::get_initial_value(),
                    }),
                    pointing: Arc::new(style_structs::Pointing {
                            cursor: longhands::cursor::get_initial_value(),
                            pointer_events: longhands::pointer_events::get_initial_value(),
                            _moz_user_input: longhands::_moz_user_input::get_initial_value(),
                            _moz_user_modify: longhands::_moz_user_modify::get_initial_value(),
                            _moz_user_focus: longhands::_moz_user_focus::get_initial_value(),
                            caret_color: longhands::caret_color::get_initial_value(),
                    }),
                    position: Arc::new(style_structs::Position {
                            top: longhands::top::get_initial_value(),
                            left: longhands::left::get_initial_value(),
                            bottom: longhands::bottom::get_initial_value(),
                            right: longhands::right::get_initial_value(),
                            offset_block_start: longhands::offset_block_start::get_initial_value(),
                            offset_block_end: longhands::offset_block_end::get_initial_value(),
                            offset_inline_start: longhands::offset_inline_start::get_initial_value(),
                            offset_inline_end: longhands::offset_inline_end::get_initial_value(),
                            z_index: longhands::z_index::get_initial_value(),
                            flex_direction: longhands::flex_direction::get_initial_value(),
                            flex_wrap: longhands::flex_wrap::get_initial_value(),
                            justify_content: longhands::justify_content::get_initial_value(),
                            align_content: longhands::align_content::get_initial_value(),
                            align_items: longhands::align_items::get_initial_value(),
                            justify_items: longhands::justify_items::get_initial_value(),
                            flex_grow: longhands::flex_grow::get_initial_value(),
                            flex_shrink: longhands::flex_shrink::get_initial_value(),
                            align_self: longhands::align_self::get_initial_value(),
                            justify_self: longhands::justify_self::get_initial_value(),
                            order: longhands::order::get_initial_value(),
                            flex_basis: longhands::flex_basis::get_initial_value(),
                            width: longhands::width::get_initial_value(),
                            min_width: longhands::min_width::get_initial_value(),
                            max_width: longhands::max_width::get_initial_value(),
                            height: longhands::height::get_initial_value(),
                            min_height: longhands::min_height::get_initial_value(),
                            max_height: longhands::max_height::get_initial_value(),
                            block_size: longhands::block_size::get_initial_value(),
                            min_block_size: longhands::min_block_size::get_initial_value(),
                            max_block_size: longhands::max_block_size::get_initial_value(),
                            inline_size: longhands::inline_size::get_initial_value(),
                            min_inline_size: longhands::min_inline_size::get_initial_value(),
                            max_inline_size: longhands::max_inline_size::get_initial_value(),
                            box_sizing: longhands::box_sizing::get_initial_value(),
                            object_fit: longhands::object_fit::get_initial_value(),
                            object_position: longhands::object_position::get_initial_value(),
                            grid_row_gap: longhands::grid_row_gap::get_initial_value(),
                            grid_row_start: longhands::grid_row_start::get_initial_value(),
                            grid_row_end: longhands::grid_row_end::get_initial_value(),
                            grid_auto_rows: longhands::grid_auto_rows::get_initial_value(),
                            grid_template_rows: longhands::grid_template_rows::get_initial_value(),
                            grid_column_gap: longhands::grid_column_gap::get_initial_value(),
                            grid_column_start: longhands::grid_column_start::get_initial_value(),
                            grid_column_end: longhands::grid_column_end::get_initial_value(),
                            grid_auto_columns: longhands::grid_auto_columns::get_initial_value(),
                            grid_template_columns: longhands::grid_template_columns::get_initial_value(),
                            grid_auto_flow: longhands::grid_auto_flow::get_initial_value(),
                            grid_template_areas: longhands::grid_template_areas::get_initial_value(),
                    }),
                    table: Arc::new(style_structs::Table {
                            table_layout: longhands::table_layout::get_initial_value(),
                            _x_span: longhands::_x_span::get_initial_value(),
                    }),
                    text: Arc::new(style_structs::Text {
                            text_overflow: longhands::text_overflow::get_initial_value(),
                            unicode_bidi: longhands::unicode_bidi::get_initial_value(),
                            text_decoration_line: longhands::text_decoration_line::get_initial_value(),
                            text_decoration_style: longhands::text_decoration_style::get_initial_value(),
                            text_decoration_color: longhands::text_decoration_color::get_initial_value(),
                            initial_letter: longhands::initial_letter::get_initial_value(),
                    }),
                    ui: Arc::new(style_structs::UI {
                            ime_mode: longhands::ime_mode::get_initial_value(),
                            _moz_user_select: longhands::_moz_user_select::get_initial_value(),
                            _moz_window_dragging: longhands::_moz_window_dragging::get_initial_value(),
                            _moz_window_shadow: longhands::_moz_window_shadow::get_initial_value(),
                            _moz_force_broken_image_icon: longhands::_moz_force_broken_image_icon::get_initial_value(),
                    }),
                    inheritedsvg: Arc::new(style_structs::InheritedSVG {
                            text_anchor: longhands::text_anchor::get_initial_value(),
                            color_interpolation: longhands::color_interpolation::get_initial_value(),
                            color_interpolation_filters: longhands::color_interpolation_filters::get_initial_value(),
                            fill: longhands::fill::get_initial_value(),
                            fill_opacity: longhands::fill_opacity::get_initial_value(),
                            fill_rule: longhands::fill_rule::get_initial_value(),
                            shape_rendering: longhands::shape_rendering::get_initial_value(),
                            stroke: longhands::stroke::get_initial_value(),
                            stroke_width: longhands::stroke_width::get_initial_value(),
                            stroke_linecap: longhands::stroke_linecap::get_initial_value(),
                            stroke_linejoin: longhands::stroke_linejoin::get_initial_value(),
                            stroke_miterlimit: longhands::stroke_miterlimit::get_initial_value(),
                            stroke_opacity: longhands::stroke_opacity::get_initial_value(),
                            stroke_dasharray: longhands::stroke_dasharray::get_initial_value(),
                            stroke_dashoffset: longhands::stroke_dashoffset::get_initial_value(),
                            clip_rule: longhands::clip_rule::get_initial_value(),
                            marker_start: longhands::marker_start::get_initial_value(),
                            marker_mid: longhands::marker_mid::get_initial_value(),
                            marker_end: longhands::marker_end::get_initial_value(),
                            paint_order: longhands::paint_order::get_initial_value(),
                            _moz_context_properties: longhands::_moz_context_properties::get_initial_value(),
                    }),
                    svg: Arc::new(style_structs::SVG {
                            dominant_baseline: longhands::dominant_baseline::get_initial_value(),
                            vector_effect: longhands::vector_effect::get_initial_value(),
                            stop_color: longhands::stop_color::get_initial_value(),
                            stop_opacity: longhands::stop_opacity::get_initial_value(),
                            flood_color: longhands::flood_color::get_initial_value(),
                            flood_opacity: longhands::flood_opacity::get_initial_value(),
                            lighting_color: longhands::lighting_color::get_initial_value(),
                            mask_type: longhands::mask_type::get_initial_value(),
                            clip_path: longhands::clip_path::get_initial_value(),
                            mask_mode: longhands::mask_mode::get_initial_value(),
                            mask_repeat: longhands::mask_repeat::get_initial_value(),
                            mask_position_x: longhands::mask_position_x::get_initial_value(),
                            mask_position_y: longhands::mask_position_y::get_initial_value(),
                            mask_clip: longhands::mask_clip::get_initial_value(),
                            mask_origin: longhands::mask_origin::get_initial_value(),
                            mask_size: longhands::mask_size::get_initial_value(),
                            mask_composite: longhands::mask_composite::get_initial_value(),
                            mask_image: longhands::mask_image::get_initial_value(),
                    }),
                    xul: Arc::new(style_structs::XUL {
                            _moz_box_align: longhands::_moz_box_align::get_initial_value(),
                            _moz_box_direction: longhands::_moz_box_direction::get_initial_value(),
                            _moz_box_flex: longhands::_moz_box_flex::get_initial_value(),
                            _moz_box_orient: longhands::_moz_box_orient::get_initial_value(),
                            _moz_box_pack: longhands::_moz_box_pack::get_initial_value(),
                            _moz_stack_sizing: longhands::_moz_stack_sizing::get_initial_value(),
                            _moz_box_ordinal_group: longhands::_moz_box_ordinal_group::get_initial_value(),
                    }),
                custom_properties: None,
                writing_mode: WritingMode::empty(),
                font_computation_data: default_font_size_keyword(),
                rules: None,
                visited_style: None,
                flags: ComputedValueFlags::empty(),
            }
        };
    }
}

/// A per-longhand function that performs the CSS cascade for that longhand.
pub type CascadePropertyFn =
    extern "Rust" fn(
        declaration: &PropertyDeclaration,
        context: &mut computed::Context,
    );

/// A per-longhand array of functions to perform the CSS cascade on each of
/// them, effectively doing virtual dispatch.
static CASCADE_PROPERTY: [CascadePropertyFn; 318] = [
        longhands::background_color::cascade_property,
        longhands::background_image::cascade_property,
        longhands::background_position_x::cascade_property,
        longhands::background_position_y::cascade_property,
        longhands::background_repeat::cascade_property,
        longhands::background_attachment::cascade_property,
        longhands::background_clip::cascade_property,
        longhands::background_origin::cascade_property,
        longhands::background_size::cascade_property,
        longhands::background_blend_mode::cascade_property,
        longhands::border_top_color::cascade_property,
        longhands::border_top_style::cascade_property,
        longhands::border_top_width::cascade_property,
        longhands::border_left_color::cascade_property,
        longhands::border_left_style::cascade_property,
        longhands::border_left_width::cascade_property,
        longhands::border_bottom_color::cascade_property,
        longhands::border_bottom_style::cascade_property,
        longhands::border_bottom_width::cascade_property,
        longhands::border_right_color::cascade_property,
        longhands::border_right_style::cascade_property,
        longhands::border_right_width::cascade_property,
        longhands::border_block_start_color::cascade_property,
        longhands::border_block_start_style::cascade_property,
        longhands::border_block_start_width::cascade_property,
        longhands::border_block_end_color::cascade_property,
        longhands::border_block_end_style::cascade_property,
        longhands::border_block_end_width::cascade_property,
        longhands::border_inline_start_color::cascade_property,
        longhands::border_inline_start_style::cascade_property,
        longhands::border_inline_start_width::cascade_property,
        longhands::border_inline_end_color::cascade_property,
        longhands::border_inline_end_style::cascade_property,
        longhands::border_inline_end_width::cascade_property,
        longhands::border_top_left_radius::cascade_property,
        longhands::border_top_right_radius::cascade_property,
        longhands::border_bottom_right_radius::cascade_property,
        longhands::border_bottom_left_radius::cascade_property,
        longhands::_moz_border_top_colors::cascade_property,
        longhands::_moz_border_left_colors::cascade_property,
        longhands::_moz_border_bottom_colors::cascade_property,
        longhands::_moz_border_right_colors::cascade_property,
        longhands::box_decoration_break::cascade_property,
        longhands::_moz_float_edge::cascade_property,
        longhands::border_image_source::cascade_property,
        longhands::border_image_outset::cascade_property,
        longhands::border_image_repeat::cascade_property,
        longhands::border_image_width::cascade_property,
        longhands::border_image_slice::cascade_property,
        longhands::display::cascade_property,
        longhands::_moz_top_layer::cascade_property,
        longhands::position::cascade_property,
        longhands::float::cascade_property,
        longhands::clear::cascade_property,
        longhands::vertical_align::cascade_property,
        longhands::overflow_clip_box::cascade_property,
        longhands::overflow_x::cascade_property,
        longhands::overflow_y::cascade_property,
        longhands::transition_duration::cascade_property,
        longhands::transition_timing_function::cascade_property,
        longhands::transition_property::cascade_property,
        longhands::transition_delay::cascade_property,
        longhands::animation_name::cascade_property,
        longhands::animation_duration::cascade_property,
        longhands::animation_timing_function::cascade_property,
        longhands::animation_iteration_count::cascade_property,
        longhands::animation_direction::cascade_property,
        longhands::animation_play_state::cascade_property,
        longhands::animation_fill_mode::cascade_property,
        longhands::animation_delay::cascade_property,
        longhands::scroll_snap_points_x::cascade_property,
        longhands::scroll_snap_points_y::cascade_property,
        longhands::scroll_snap_destination::cascade_property,
        longhands::scroll_snap_coordinate::cascade_property,
        longhands::transform::cascade_property,
        longhands::scroll_behavior::cascade_property,
        longhands::scroll_snap_type_x::cascade_property,
        longhands::scroll_snap_type_y::cascade_property,
        longhands::isolation::cascade_property,
        longhands::page_break_after::cascade_property,
        longhands::page_break_before::cascade_property,
        longhands::page_break_inside::cascade_property,
        longhands::resize::cascade_property,
        longhands::perspective::cascade_property,
        longhands::perspective_origin::cascade_property,
        longhands::backface_visibility::cascade_property,
        longhands::transform_box::cascade_property,
        longhands::transform_style::cascade_property,
        longhands::transform_origin::cascade_property,
        longhands::contain::cascade_property,
        longhands::_moz_appearance::cascade_property,
        longhands::_moz_binding::cascade_property,
        longhands::_moz_orient::cascade_property,
        longhands::will_change::cascade_property,
        longhands::shape_outside::cascade_property,
        longhands::touch_action::cascade_property,
        longhands::color::cascade_property,
        longhands::column_width::cascade_property,
        longhands::column_count::cascade_property,
        longhands::column_gap::cascade_property,
        longhands::column_fill::cascade_property,
        longhands::column_rule_width::cascade_property,
        longhands::column_rule_color::cascade_property,
        longhands::column_span::cascade_property,
        longhands::column_rule_style::cascade_property,
        longhands::content::cascade_property,
        longhands::counter_increment::cascade_property,
        longhands::counter_reset::cascade_property,
        longhands::opacity::cascade_property,
        longhands::box_shadow::cascade_property,
        longhands::clip::cascade_property,
        longhands::filter::cascade_property,
        longhands::mix_blend_mode::cascade_property,
        longhands::font_family::cascade_property,
        longhands::font_style::cascade_property,
        longhands::font_variant_caps::cascade_property,
        longhands::font_weight::cascade_property,
        longhands::font_size::cascade_property,
        longhands::font_size_adjust::cascade_property,
        longhands::font_synthesis::cascade_property,
        longhands::font_stretch::cascade_property,
        longhands::font_kerning::cascade_property,
        longhands::font_variant_alternates::cascade_property,
        longhands::font_variant_east_asian::cascade_property,
        longhands::font_variant_ligatures::cascade_property,
        longhands::font_variant_numeric::cascade_property,
        longhands::font_variant_position::cascade_property,
        longhands::font_feature_settings::cascade_property,
        longhands::font_variation_settings::cascade_property,
        longhands::font_language_override::cascade_property,
        longhands::_x_lang::cascade_property,
        longhands::_moz_script_size_multiplier::cascade_property,
        longhands::_moz_script_level::cascade_property,
        longhands::_moz_math_display::cascade_property,
        longhands::_moz_math_variant::cascade_property,
        longhands::_moz_script_min_size::cascade_property,
        longhands::_x_text_zoom::cascade_property,
        longhands::_moz_osx_font_smoothing::cascade_property,
        longhands::_moz_min_font_size_ratio::cascade_property,
        longhands::visibility::cascade_property,
        longhands::writing_mode::cascade_property,
        longhands::direction::cascade_property,
        longhands::text_orientation::cascade_property,
        longhands::color_adjust::cascade_property,
        longhands::image_rendering::cascade_property,
        longhands::image_orientation::cascade_property,
        longhands::border_collapse::cascade_property,
        longhands::empty_cells::cascade_property,
        longhands::caption_side::cascade_property,
        longhands::border_spacing::cascade_property,
        longhands::line_height::cascade_property,
        longhands::text_transform::cascade_property,
        longhands::hyphens::cascade_property,
        longhands::_moz_text_size_adjust::cascade_property,
        longhands::text_indent::cascade_property,
        longhands::overflow_wrap::cascade_property,
        longhands::word_break::cascade_property,
        longhands::text_justify::cascade_property,
        longhands::text_align_last::cascade_property,
        longhands::text_align::cascade_property,
        longhands::letter_spacing::cascade_property,
        longhands::word_spacing::cascade_property,
        longhands::white_space::cascade_property,
        longhands::text_shadow::cascade_property,
        longhands::text_emphasis_style::cascade_property,
        longhands::text_emphasis_position::cascade_property,
        longhands::text_emphasis_color::cascade_property,
        longhands::_moz_tab_size::cascade_property,
        longhands::_webkit_text_fill_color::cascade_property,
        longhands::_webkit_text_stroke_color::cascade_property,
        longhands::_webkit_text_stroke_width::cascade_property,
        longhands::ruby_align::cascade_property,
        longhands::ruby_position::cascade_property,
        longhands::text_combine_upright::cascade_property,
        longhands::text_rendering::cascade_property,
        longhands::_moz_control_character_visibility::cascade_property,
        longhands::list_style_position::cascade_property,
        longhands::list_style_type::cascade_property,
        longhands::list_style_image::cascade_property,
        longhands::quotes::cascade_property,
        longhands::_moz_image_region::cascade_property,
        longhands::margin_top::cascade_property,
        longhands::margin_left::cascade_property,
        longhands::margin_bottom::cascade_property,
        longhands::margin_right::cascade_property,
        longhands::margin_block_start::cascade_property,
        longhands::margin_block_end::cascade_property,
        longhands::margin_inline_start::cascade_property,
        longhands::margin_inline_end::cascade_property,
        longhands::outline_color::cascade_property,
        longhands::outline_style::cascade_property,
        longhands::outline_width::cascade_property,
        longhands::_moz_outline_radius_topleft::cascade_property,
        longhands::_moz_outline_radius_topright::cascade_property,
        longhands::_moz_outline_radius_bottomright::cascade_property,
        longhands::_moz_outline_radius_bottomleft::cascade_property,
        longhands::outline_offset::cascade_property,
        longhands::padding_top::cascade_property,
        longhands::padding_left::cascade_property,
        longhands::padding_bottom::cascade_property,
        longhands::padding_right::cascade_property,
        longhands::padding_block_start::cascade_property,
        longhands::padding_block_end::cascade_property,
        longhands::padding_inline_start::cascade_property,
        longhands::padding_inline_end::cascade_property,
        longhands::cursor::cascade_property,
        longhands::pointer_events::cascade_property,
        longhands::_moz_user_input::cascade_property,
        longhands::_moz_user_modify::cascade_property,
        longhands::_moz_user_focus::cascade_property,
        longhands::caret_color::cascade_property,
        longhands::top::cascade_property,
        longhands::left::cascade_property,
        longhands::bottom::cascade_property,
        longhands::right::cascade_property,
        longhands::offset_block_start::cascade_property,
        longhands::offset_block_end::cascade_property,
        longhands::offset_inline_start::cascade_property,
        longhands::offset_inline_end::cascade_property,
        longhands::z_index::cascade_property,
        longhands::flex_direction::cascade_property,
        longhands::flex_wrap::cascade_property,
        longhands::justify_content::cascade_property,
        longhands::align_content::cascade_property,
        longhands::align_items::cascade_property,
        longhands::justify_items::cascade_property,
        longhands::flex_grow::cascade_property,
        longhands::flex_shrink::cascade_property,
        longhands::align_self::cascade_property,
        longhands::justify_self::cascade_property,
        longhands::order::cascade_property,
        longhands::flex_basis::cascade_property,
        longhands::width::cascade_property,
        longhands::min_width::cascade_property,
        longhands::max_width::cascade_property,
        longhands::height::cascade_property,
        longhands::min_height::cascade_property,
        longhands::max_height::cascade_property,
        longhands::block_size::cascade_property,
        longhands::min_block_size::cascade_property,
        longhands::max_block_size::cascade_property,
        longhands::inline_size::cascade_property,
        longhands::min_inline_size::cascade_property,
        longhands::max_inline_size::cascade_property,
        longhands::box_sizing::cascade_property,
        longhands::object_fit::cascade_property,
        longhands::object_position::cascade_property,
        longhands::grid_row_gap::cascade_property,
        longhands::grid_row_start::cascade_property,
        longhands::grid_row_end::cascade_property,
        longhands::grid_auto_rows::cascade_property,
        longhands::grid_template_rows::cascade_property,
        longhands::grid_column_gap::cascade_property,
        longhands::grid_column_start::cascade_property,
        longhands::grid_column_end::cascade_property,
        longhands::grid_auto_columns::cascade_property,
        longhands::grid_template_columns::cascade_property,
        longhands::grid_auto_flow::cascade_property,
        longhands::grid_template_areas::cascade_property,
        longhands::table_layout::cascade_property,
        longhands::_x_span::cascade_property,
        longhands::text_overflow::cascade_property,
        longhands::unicode_bidi::cascade_property,
        longhands::text_decoration_line::cascade_property,
        longhands::text_decoration_style::cascade_property,
        longhands::text_decoration_color::cascade_property,
        longhands::initial_letter::cascade_property,
        longhands::ime_mode::cascade_property,
        longhands::_moz_user_select::cascade_property,
        longhands::_moz_window_dragging::cascade_property,
        longhands::_moz_window_shadow::cascade_property,
        longhands::_moz_force_broken_image_icon::cascade_property,
        longhands::text_anchor::cascade_property,
        longhands::color_interpolation::cascade_property,
        longhands::color_interpolation_filters::cascade_property,
        longhands::fill::cascade_property,
        longhands::fill_opacity::cascade_property,
        longhands::fill_rule::cascade_property,
        longhands::shape_rendering::cascade_property,
        longhands::stroke::cascade_property,
        longhands::stroke_width::cascade_property,
        longhands::stroke_linecap::cascade_property,
        longhands::stroke_linejoin::cascade_property,
        longhands::stroke_miterlimit::cascade_property,
        longhands::stroke_opacity::cascade_property,
        longhands::stroke_dasharray::cascade_property,
        longhands::stroke_dashoffset::cascade_property,
        longhands::clip_rule::cascade_property,
        longhands::marker_start::cascade_property,
        longhands::marker_mid::cascade_property,
        longhands::marker_end::cascade_property,
        longhands::paint_order::cascade_property,
        longhands::_moz_context_properties::cascade_property,
        longhands::dominant_baseline::cascade_property,
        longhands::vector_effect::cascade_property,
        longhands::stop_color::cascade_property,
        longhands::stop_opacity::cascade_property,
        longhands::flood_color::cascade_property,
        longhands::flood_opacity::cascade_property,
        longhands::lighting_color::cascade_property,
        longhands::mask_type::cascade_property,
        longhands::clip_path::cascade_property,
        longhands::mask_mode::cascade_property,
        longhands::mask_repeat::cascade_property,
        longhands::mask_position_x::cascade_property,
        longhands::mask_position_y::cascade_property,
        longhands::mask_clip::cascade_property,
        longhands::mask_origin::cascade_property,
        longhands::mask_size::cascade_property,
        longhands::mask_composite::cascade_property,
        longhands::mask_image::cascade_property,
        longhands::_moz_box_align::cascade_property,
        longhands::_moz_box_direction::cascade_property,
        longhands::_moz_box_flex::cascade_property,
        longhands::_moz_box_orient::cascade_property,
        longhands::_moz_box_pack::cascade_property,
        longhands::_moz_stack_sizing::cascade_property,
        longhands::_moz_box_ordinal_group::cascade_property,
];

bitflags! {
    /// A set of flags to tweak the behavior of the `cascade` function.
    pub flags CascadeFlags: u8 {
        /// Whether to inherit all styles from the parent. If this flag is not
        /// present, non-inherited styles are reset to their initial values.
        const INHERIT_ALL = 1,

        /// Whether to skip any display style fixup for root element, flex/grid
        /// item, and ruby descendants.
        const SKIP_ROOT_AND_ITEM_BASED_DISPLAY_FIXUP = 1 << 1,

        /// Whether to only cascade properties that are visited dependent.
        const VISITED_DEPENDENT_ONLY = 1 << 2,

        /// Whether the given element we're styling is the document element,
        /// that is, matches :root.
        ///
        /// Not set for native anonymous content since some NAC form their own
        /// root, but share the device.
        ///
        /// This affects some style adjustments, like blockification, and means
        /// that it may affect global state, like the Device's root font-size.
        const IS_ROOT_ELEMENT = 1 << 3,

        /// Whether to convert display:contents into display:inline.  This
        /// is used by Gecko to prevent display:contents on generated
        /// content.
        const PROHIBIT_DISPLAY_CONTENTS = 1 << 4,

        /// Whether we're styling the ::-moz-fieldset-content anonymous box.
        const IS_FIELDSET_CONTENT = 1 << 5,

        /// Whether we're computing the style of a link, either visited or
        /// unvisited.
        const IS_LINK = 1 << 6,

        /// Whether we're computing the style of a link element that happens to
        /// be visited.
        const IS_VISITED_LINK = 1 << 7,
    }
}

/// Performs the CSS cascade, computing new styles for an element from its parent style.
///
/// The arguments are:
///
///   * `device`: Used to get the initial viewport and other external state.
///
///   * `rule_node`: The rule node in the tree that represent the CSS rules that
///   matched.
///
///   * `parent_style`: The parent style, if applicable; if `None`, this is the root node.
///
/// Returns the computed values.
///   * `flags`: Various flags.
///
pub fn cascade(
    device: &Device,
    pseudo: Option<&PseudoElement>,
    rule_node: &StrongRuleNode,
    guards: &StylesheetGuards,
    parent_style: Option<&ComputedValues>,
    parent_style_ignoring_first_line: Option<&ComputedValues>,
    layout_parent_style: Option<&ComputedValues>,
    visited_style: Option<Arc<ComputedValues>>,
    font_metrics_provider: &FontMetricsProvider,
    flags: CascadeFlags,
    quirks_mode: QuirksMode,
    rule_cache: Option<&RuleCache>,
    rule_cache_conditions: &mut RuleCacheConditions,
) -> Arc<ComputedValues> {
    debug_assert_eq!(parent_style.is_some(), parent_style_ignoring_first_line.is_some());
    #[cfg(feature = "gecko")]
    debug_assert!(parent_style.is_none() ||
                  ptr::eq(parent_style.unwrap(),
                          parent_style_ignoring_first_line.unwrap()) ||
                  parent_style.unwrap().pseudo() == Some(PseudoElement::FirstLine));
    let empty = SmallBitVec::new();
    let iter_declarations = || {
        rule_node.self_and_ancestors().flat_map(|node| {
            let cascade_level = node.cascade_level();
            let source = node.style_source();

            let declarations = if source.is_some() {
                source.read(cascade_level.guard(guards)).declaration_importance_iter()
            } else {
                // The root node has no style source.
                DeclarationImportanceIterator::new(&[], &empty)
            };
            let node_importance = node.importance();

            let property_restriction = pseudo.and_then(|p| p.property_restriction());

            declarations
                // Yield declarations later in source order (with more precedence) first.
                .rev()
                .filter_map(move |(declaration, declaration_importance)| {
                    if let Some(property_restriction) = property_restriction {
                        // declaration.id() is either a longhand or a custom
                        // property.  Custom properties are always allowed, but
                        // longhands are only allowed if they have our
                        // property_restriction flag set.
                        if let PropertyDeclarationId::Longhand(id) = declaration.id() {
                            if !id.flags().contains(property_restriction) {
                                return None
                            }
                        }
                    }

                    if declaration_importance == node_importance {
                        Some((declaration, cascade_level))
                    } else {
                        None
                    }
                })
        })
    };
    apply_declarations(
        device,
        pseudo,
        rule_node,
        iter_declarations,
        parent_style,
        parent_style_ignoring_first_line,
        layout_parent_style,
        visited_style,
        font_metrics_provider,
        flags,
        quirks_mode,
        rule_cache,
        rule_cache_conditions,
    )
}

/// NOTE: This function expects the declaration with more priority to appear
/// first.
#[allow(unused_mut)] // conditionally compiled code for "position"
pub fn apply_declarations<'a, F, I>(
    device: &Device,
    pseudo: Option<&PseudoElement>,
    rules: &StrongRuleNode,
    iter_declarations: F,
    parent_style: Option<&ComputedValues>,
    parent_style_ignoring_first_line: Option<&ComputedValues>,
    layout_parent_style: Option<&ComputedValues>,
    visited_style: Option<Arc<ComputedValues>>,
    font_metrics_provider: &FontMetricsProvider,
    flags: CascadeFlags,
    quirks_mode: QuirksMode,
    rule_cache: Option<&RuleCache>,
    rule_cache_conditions: &mut RuleCacheConditions,
) -> Arc<ComputedValues>
where
    F: Fn() -> I,
    I: Iterator<Item = (&'a PropertyDeclaration, CascadeLevel)>,
{
    debug_assert!(layout_parent_style.is_none() || parent_style.is_some());
    debug_assert_eq!(parent_style.is_some(), parent_style_ignoring_first_line.is_some());
    #[cfg(feature = "gecko")]
    debug_assert!(parent_style.is_none() ||
                  ptr::eq(parent_style.unwrap(),
                          parent_style_ignoring_first_line.unwrap()) ||
                  parent_style.unwrap().pseudo() == Some(PseudoElement::FirstLine));
    let (inherited_style, layout_parent_style) = match parent_style {
        Some(parent_style) => {
            (parent_style,
             layout_parent_style.unwrap_or(parent_style))
        },
        None => {
            (device.default_computed_values(),
             device.default_computed_values())
        }
    };

    let inherited_custom_properties = inherited_style.custom_properties();
    let mut custom_properties = None;
    let mut seen_custom = PrecomputedHashSet::default();
    for (declaration, _cascade_level) in iter_declarations() {
        if let PropertyDeclaration::Custom(ref name, ref value) = *declaration {
            ::custom_properties::cascade(
                &mut custom_properties, &inherited_custom_properties,
                &mut seen_custom, name, value.borrow());
        }
    }

    let custom_properties =
        ::custom_properties::finish_cascade(
            custom_properties, &inherited_custom_properties);

    let mut context = computed::Context {
        is_root_element: flags.contains(IS_ROOT_ELEMENT),
        // We'd really like to own the rules here to avoid refcount traffic, but
        // animation's usage of `apply_declarations` make this tricky. See bug
        // 1375525.
        builder: StyleBuilder::new(
            device,
            parent_style,
            parent_style_ignoring_first_line,
            pseudo,
            flags,
            Some(rules.clone()),
            custom_properties,
            WritingMode::empty(),
            inherited_style.font_computation_data,
            ComputedValueFlags::empty(),
            visited_style,
        ),
        cached_system_font: None,
        in_media_query: false,
        for_smil_animation: false,
        for_non_inherited_property: None,
        font_metrics_provider,
        quirks_mode,
        rule_cache_conditions: RefCell::new(rule_cache_conditions),
    };

    let ignore_colors = !device.use_document_colors();
    let default_background_color_decl = if ignore_colors {
        let color = device.default_background_color();
        Some(PropertyDeclaration::BackgroundColor(color.into()))
    } else {
        None
    };

    // Set computed values, overwriting earlier declarations for the same
    // property.
    let mut seen = LonghandIdSet::new();

    // Declaration blocks are stored in increasing precedence order, we want
    // them in decreasing order here.
    //
    // We could (and used to) use a pattern match here, but that bloats this
    // function to over 100K of compiled code!
    //
    // To improve i-cache behavior, we outline the individual functions and use
    // virtual dispatch instead.
    let mut apply_reset = true;
            // Pull these out so that we can compute them in a specific order
            // without introducing more iterations.
            let mut font_size = None;
            let mut font_family = None;
        for (declaration, cascade_level) in iter_declarations() {
            let mut declaration = match *declaration {
                PropertyDeclaration::WithVariables(id, ref unparsed) => {
                    if !id.inherited() {
                        context.rule_cache_conditions.borrow_mut()
                            .set_uncacheable();
                    }
                    Cow::Owned(unparsed.substitute_variables(
                        id,
                        &context.builder.custom_properties,
                        context.quirks_mode
                    ))
                }
                ref d => Cow::Borrowed(d)
            };

            let longhand_id = match declaration.id() {
                PropertyDeclarationId::Longhand(id) => id,
                PropertyDeclarationId::Custom(..) => continue,
            };

            // Only a few properties are allowed to depend on the visited state
            // of links.  When cascading visited styles, we can save time by
            // only processing these properties.
            if flags.contains(VISITED_DEPENDENT_ONLY) &&
               !longhand_id.is_visited_dependent() {
                continue
            }

            if !apply_reset && !longhand_id.inherited() {
                continue;
            }

            // When document colors are disabled, skip properties that are
            // marked as ignored in that mode, if they come from a UA or
            // user style sheet.
            if ignore_colors &&
               longhand_id.is_ignored_when_document_colors_disabled() &&
               !matches!(cascade_level,
                         CascadeLevel::UANormal |
                         CascadeLevel::UserNormal |
                         CascadeLevel::UserImportant |
                         CascadeLevel::UAImportant) {
                let non_transparent_background = match *declaration {
                    PropertyDeclaration::BackgroundColor(ref color) => {
                        // Treat background-color a bit differently.  If the specified
                        // color is anything other than a fully transparent color, convert
                        // it into the Device's default background color.
                        color.is_non_transparent()
                    }
                    _ => continue
                };
                // FIXME: moving this out of `match` is a work around for borrows being lexical.
                if non_transparent_background {
                    declaration = Cow::Borrowed(default_background_color_decl.as_ref().unwrap());
                }
            }

            if
                    !
                longhand_id.is_early_property()
            {
                continue
            }

            
            let physical_longhand_id = longhand_id ;
            if seen.contains(physical_longhand_id) {
                continue
            }
            seen.insert(physical_longhand_id);

                if LonghandId::FontSize == longhand_id {
                    font_size = Some(declaration.clone());
                    continue;
                }
                if LonghandId::FontFamily == longhand_id {
                    font_family = Some(declaration.clone());
                    continue;
                }

            let discriminant = longhand_id as usize;
            (CASCADE_PROPERTY[discriminant])(&*declaration, &mut context);
        }
            let writing_mode = get_writing_mode(context.builder.get_inheritedbox());
            context.builder.writing_mode = writing_mode;

            let mut _skip_font_family = false;


                // <svg:text> is not affected by text zoom, and it uses a preshint to
                // disable it. We fix up the struct when this happens by unzooming
                // its contained font values, which will have been zoomed in the parent
                if seen.contains(LonghandId::XTextZoom) {
                    let zoom = context.builder.get_font().gecko().mAllowZoom;
                    let parent_zoom = context.style().get_parent_font().gecko().mAllowZoom;
                    if  zoom != parent_zoom {
                        debug_assert!(!zoom,
                                      "We only ever disable text zoom (in svg:text), never enable it");
                        // can't borrow both device and font, use the take/put machinery
                        let mut font = context.builder.take_font();
                        font.unzoom_fonts(context.device());
                        context.builder.put_font(font);
                    }
                }

                // Whenever a single generic value is specified, gecko will do a bunch of
                // recalculation walking up the rule tree, including handling the font-size stuff.
                // It basically repopulates the font struct with the default font for a given
                // generic and language. We handle the font-size stuff separately, so this boils
                // down to just copying over the font-family lists (no other aspect of the default
                // font can be configured).

                if seen.contains(LonghandId::XLang) || font_family.is_some() {
                    // if just the language changed, the inherited generic is all we need
                    let mut generic = inherited_style.get_font().gecko().mGenericID;
                    if let Some(ref declaration) = font_family {
                        if let PropertyDeclaration::FontFamily(ref fam) = **declaration {
                            if let Some(id) = fam.single_generic() {
                                generic = id;
                                // In case of a specified font family with a single generic, we will
                                // end up setting font family below, but its value would get
                                // overwritten later in the pipeline when cascading.
                                //
                                // We instead skip cascading font-family in that case.
                                //
                                // In case of the language changing, we wish for a specified font-
                                // family to override this, so we do not skip cascading then.
                                _skip_font_family = true;
                            }
                        }
                    }

                    let pres_context = context.builder.device.pres_context();
                    let gecko_font = context.builder.mutate_font().gecko_mut();
                    gecko_font.mGenericID = generic;
                    unsafe {
                        bindings::Gecko_nsStyleFont_PrefillDefaultForGeneric(
                            gecko_font,
                            pres_context,
                            generic,
                        );
                    }
                }

            // It is important that font_size is computed before
            // the late properties (for em units), but after font-family
            // (for the base-font-size dependence for default and keyword font-sizes)
            // Additionally, when we support system fonts they will have to be
            // computed early, and *before* font_family, so I'm including
            // font_family here preemptively instead of keeping it within
            // the early properties.
            //
            // To avoid an extra iteration, we just pull out the property
            // during the early iteration and cascade them in order
            // after it.
            if !_skip_font_family {
                if let Some(ref declaration) = font_family {

                    let discriminant = LonghandId::FontFamily as usize;
                    (CASCADE_PROPERTY[discriminant])(declaration, &mut context);
                        let device = context.builder.device;
                        if let PropertyDeclaration::FontFamily(ref val) = **declaration {
                            if val.get_system().is_some() {
                                let default = context.cached_system_font
                                                     .as_ref().unwrap().default_font_type;
                                context.builder.mutate_font().fixup_system(default);
                            } else {
                                context.builder.mutate_font().fixup_none_generic(device);
                            }
                        }
                }
            }

            if let Some(ref declaration) = font_size {
                let discriminant = LonghandId::FontSize as usize;
                (CASCADE_PROPERTY[discriminant])(declaration, &mut context);
            // Font size must be explicitly inherited to handle lang changes and
            // scriptlevel changes.
            } else if seen.contains(LonghandId::XLang) ||
                      seen.contains(LonghandId::MozScriptLevel) ||
                      seen.contains(LonghandId::MozMinFontSizeRatio) ||
                      font_family.is_some() {
                let discriminant = LonghandId::FontSize as usize;
                let size = PropertyDeclaration::CSSWideKeyword(
                    LonghandId::FontSize, CSSWideKeyword::Inherit);

                (CASCADE_PROPERTY[discriminant])(&size, &mut context);
            }

            if let Some(style) = rule_cache.and_then(|c| c.find(&context.builder)) {
                context.builder.copy_reset_from(style);
                apply_reset = false;
            }
        for (declaration, cascade_level) in iter_declarations() {
            let mut declaration = match *declaration {
                PropertyDeclaration::WithVariables(id, ref unparsed) => {
                    if !id.inherited() {
                        context.rule_cache_conditions.borrow_mut()
                            .set_uncacheable();
                    }
                    Cow::Owned(unparsed.substitute_variables(
                        id,
                        &context.builder.custom_properties,
                        context.quirks_mode
                    ))
                }
                ref d => Cow::Borrowed(d)
            };

            let longhand_id = match declaration.id() {
                PropertyDeclarationId::Longhand(id) => id,
                PropertyDeclarationId::Custom(..) => continue,
            };

            // Only a few properties are allowed to depend on the visited state
            // of links.  When cascading visited styles, we can save time by
            // only processing these properties.
            if flags.contains(VISITED_DEPENDENT_ONLY) &&
               !longhand_id.is_visited_dependent() {
                continue
            }

            if !apply_reset && !longhand_id.inherited() {
                continue;
            }

            // When document colors are disabled, skip properties that are
            // marked as ignored in that mode, if they come from a UA or
            // user style sheet.
            if ignore_colors &&
               longhand_id.is_ignored_when_document_colors_disabled() &&
               !matches!(cascade_level,
                         CascadeLevel::UANormal |
                         CascadeLevel::UserNormal |
                         CascadeLevel::UserImportant |
                         CascadeLevel::UAImportant) {
                let non_transparent_background = match *declaration {
                    PropertyDeclaration::BackgroundColor(ref color) => {
                        // Treat background-color a bit differently.  If the specified
                        // color is anything other than a fully transparent color, convert
                        // it into the Device's default background color.
                        color.is_non_transparent()
                    }
                    _ => continue
                };
                // FIXME: moving this out of `match` is a work around for borrows being lexical.
                if non_transparent_background {
                    declaration = Cow::Borrowed(default_background_color_decl.as_ref().unwrap());
                }
            }

            if
                longhand_id.is_early_property()
            {
                continue
            }

            
            let physical_longhand_id = longhand_id .to_physical(writing_mode);
            if seen.contains(physical_longhand_id) {
                continue
            }
            seen.insert(physical_longhand_id);


            let discriminant = longhand_id as usize;
            (CASCADE_PROPERTY[discriminant])(&*declaration, &mut context);
        }

    let mut builder = context.builder;

        if let Some(ref mut bg) = builder.get_background_if_mutated() {
            bg.fill_arrays();
        }

        if let Some(ref mut svg) = builder.get_svg_if_mutated() {
            svg.fill_arrays();
        }


    builder.clear_modified_reset();

    StyleAdjuster::new(&mut builder)
        .adjust(layout_parent_style, flags);

    if builder.modified_reset() || !apply_reset {
        // If we adjusted any reset structs, we can't cache this ComputedValues.
        //
        // Also, if we re-used existing reset structs, don't bother caching it
        // back again. (Aside from being wasted effort, it will be wrong, since
        // context.rule_cache_conditions won't be set appropriately if we
        // didn't compute those reset properties.)
        context.rule_cache_conditions.borrow_mut()
            .set_uncacheable();
    }

    builder.build()
}

/// See StyleAdjuster::adjust_for_border_width.
pub fn adjust_border_width(style: &mut StyleBuilder) {
        // Like calling to_computed_value, which wouldn't type check.
        if style.get_border().clone_border_top_style().none_or_hidden() &&
           style.get_border().border_top_has_nonzero_width() {
            style.set_border_top_width(NonNegativeLength::zero());
        }
        // Like calling to_computed_value, which wouldn't type check.
        if style.get_border().clone_border_right_style().none_or_hidden() &&
           style.get_border().border_right_has_nonzero_width() {
            style.set_border_right_width(NonNegativeLength::zero());
        }
        // Like calling to_computed_value, which wouldn't type check.
        if style.get_border().clone_border_bottom_style().none_or_hidden() &&
           style.get_border().border_bottom_has_nonzero_width() {
            style.set_border_bottom_width(NonNegativeLength::zero());
        }
        // Like calling to_computed_value, which wouldn't type check.
        if style.get_border().clone_border_left_style().none_or_hidden() &&
           style.get_border().border_left_has_nonzero_width() {
            style.set_border_left_width(NonNegativeLength::zero());
        }
}

/// Adjusts borders as appropriate to account for a fragment's status as the
/// first or last fragment within the range of an element.
///
/// Specifically, this function sets border widths to zero on the sides for
/// which the fragment is not outermost.
#[cfg(feature = "servo")]
#[inline]
pub fn modify_border_style_for_inline_sides(style: &mut Arc<ComputedValues>,
                                            is_first_fragment_of_element: bool,
                                            is_last_fragment_of_element: bool) {
    fn modify_side(style: &mut Arc<ComputedValues>, side: PhysicalSide) {
        {
            let border = &style.border;
            let current_style = match side {
                PhysicalSide::Left =>   (border.border_left_width,   border.border_left_style),
                PhysicalSide::Right =>  (border.border_right_width,  border.border_right_style),
                PhysicalSide::Top =>    (border.border_top_width,    border.border_top_style),
                PhysicalSide::Bottom => (border.border_bottom_width, border.border_bottom_style),
            };
            if current_style == (NonNegativeLength::zero(), BorderStyle::none) {
                return;
            }
        }
        let style = Arc::make_mut(style);
        let border = Arc::make_mut(&mut style.border);
        match side {
            PhysicalSide::Left => {
                border.border_left_width = NonNegativeLength::zero();
                border.border_left_style = BorderStyle::none;
            }
            PhysicalSide::Right => {
                border.border_right_width = NonNegativeLength::zero();
                border.border_right_style = BorderStyle::none;
            }
            PhysicalSide::Bottom => {
                border.border_bottom_width = NonNegativeLength::zero();
                border.border_bottom_style = BorderStyle::none;
            }
            PhysicalSide::Top => {
                border.border_top_width = NonNegativeLength::zero();
                border.border_top_style = BorderStyle::none;
            }
        }
    }

    if !is_first_fragment_of_element {
        let side = style.writing_mode.inline_start_physical_side();
        modify_side(style, side)
    }

    if !is_last_fragment_of_element {
        let side = style.writing_mode.inline_end_physical_side();
        modify_side(style, side)
    }
}

/// An identifier for a given alias property.
#[derive(Clone, Copy, Eq, PartialEq)]
#[cfg_attr(feature = "servo", derive(HeapSizeOf))]
pub enum AliasId {
        /// -webkit-background-clip
        WebkitBackgroundClip = 0,
        /// -webkit-background-origin
        WebkitBackgroundOrigin = 1,
        /// -webkit-background-size
        WebkitBackgroundSize = 2,
        /// -moz-border-start-color
        MozBorderStartColor = 3,
        /// -moz-border-start-style
        MozBorderStartStyle = 4,
        /// -moz-border-start-width
        MozBorderStartWidth = 5,
        /// -moz-border-end-color
        MozBorderEndColor = 6,
        /// -moz-border-end-style
        MozBorderEndStyle = 7,
        /// -moz-border-end-width
        MozBorderEndWidth = 8,
        /// -webkit-border-top-left-radius
        WebkitBorderTopLeftRadius = 9,
        /// -webkit-border-top-right-radius
        WebkitBorderTopRightRadius = 10,
        /// -webkit-border-bottom-right-radius
        WebkitBorderBottomRightRadius = 11,
        /// -webkit-border-bottom-left-radius
        WebkitBorderBottomLeftRadius = 12,
        /// -moz-transition-duration
        MozTransitionDuration = 13,
        /// -webkit-transition-duration
        WebkitTransitionDuration = 14,
        /// -moz-transition-timing-function
        MozTransitionTimingFunction = 15,
        /// -webkit-transition-timing-function
        WebkitTransitionTimingFunction = 16,
        /// -moz-transition-property
        MozTransitionProperty = 17,
        /// -webkit-transition-property
        WebkitTransitionProperty = 18,
        /// -moz-transition-delay
        MozTransitionDelay = 19,
        /// -webkit-transition-delay
        WebkitTransitionDelay = 20,
        /// -moz-animation-name
        MozAnimationName = 21,
        /// -webkit-animation-name
        WebkitAnimationName = 22,
        /// -moz-animation-duration
        MozAnimationDuration = 23,
        /// -webkit-animation-duration
        WebkitAnimationDuration = 24,
        /// -moz-animation-timing-function
        MozAnimationTimingFunction = 25,
        /// -webkit-animation-timing-function
        WebkitAnimationTimingFunction = 26,
        /// -moz-animation-iteration-count
        MozAnimationIterationCount = 27,
        /// -webkit-animation-iteration-count
        WebkitAnimationIterationCount = 28,
        /// -moz-animation-direction
        MozAnimationDirection = 29,
        /// -webkit-animation-direction
        WebkitAnimationDirection = 30,
        /// -moz-animation-play-state
        MozAnimationPlayState = 31,
        /// -webkit-animation-play-state
        WebkitAnimationPlayState = 32,
        /// -moz-animation-fill-mode
        MozAnimationFillMode = 33,
        /// -webkit-animation-fill-mode
        WebkitAnimationFillMode = 34,
        /// -moz-animation-delay
        MozAnimationDelay = 35,
        /// -webkit-animation-delay
        WebkitAnimationDelay = 36,
        /// -webkit-transform
        WebkitTransform = 37,
        /// -moz-perspective
        MozPerspective = 38,
        /// -webkit-perspective
        WebkitPerspective = 39,
        /// -moz-perspective-origin
        MozPerspectiveOrigin = 40,
        /// -webkit-perspective-origin
        WebkitPerspectiveOrigin = 41,
        /// -moz-backface-visibility
        MozBackfaceVisibility = 42,
        /// -webkit-backface-visibility
        WebkitBackfaceVisibility = 43,
        /// -moz-transform-style
        MozTransformStyle = 44,
        /// -webkit-transform-style
        WebkitTransformStyle = 45,
        /// -moz-transform-origin
        MozTransformOrigin = 46,
        /// -webkit-transform-origin
        WebkitTransformOrigin = 47,
        /// -moz-column-width
        MozColumnWidth = 48,
        /// -moz-column-count
        MozColumnCount = 49,
        /// -moz-column-gap
        MozColumnGap = 50,
        /// -moz-column-fill
        MozColumnFill = 51,
        /// -moz-column-rule-width
        MozColumnRuleWidth = 52,
        /// -moz-column-rule-color
        MozColumnRuleColor = 53,
        /// -moz-column-rule-style
        MozColumnRuleStyle = 54,
        /// -webkit-box-shadow
        WebkitBoxShadow = 55,
        /// -webkit-filter
        WebkitFilter = 56,
        /// -moz-font-feature-settings
        MozFontFeatureSettings = 57,
        /// -moz-font-language-override
        MozFontLanguageOverride = 58,
        /// -moz-hyphens
        MozHyphens = 59,
        /// -webkit-text-size-adjust
        WebkitTextSizeAdjust = 60,
        /// word-wrap
        WordWrap = 61,
        /// -moz-margin-start
        MozMarginStart = 62,
        /// -moz-margin-end
        MozMarginEnd = 63,
        /// -moz-padding-start
        MozPaddingStart = 64,
        /// -moz-padding-end
        MozPaddingEnd = 65,
        /// -webkit-flex-direction
        WebkitFlexDirection = 66,
        /// -webkit-flex-wrap
        WebkitFlexWrap = 67,
        /// -webkit-justify-content
        WebkitJustifyContent = 68,
        /// -webkit-align-content
        WebkitAlignContent = 69,
        /// -webkit-align-items
        WebkitAlignItems = 70,
        /// -webkit-flex-grow
        WebkitFlexGrow = 71,
        /// -webkit-flex-shrink
        WebkitFlexShrink = 72,
        /// -webkit-align-self
        WebkitAlignSelf = 73,
        /// -webkit-order
        WebkitOrder = 74,
        /// -webkit-flex-basis
        WebkitFlexBasis = 75,
        /// -moz-box-sizing
        MozBoxSizing = 76,
        /// -webkit-box-sizing
        WebkitBoxSizing = 77,
        /// -webkit-user-select
        WebkitUserSelect = 78,
        /// -webkit-mask-repeat
        WebkitMaskRepeat = 79,
        /// -webkit-mask-position-x
        WebkitMaskPositionX = 80,
        /// -webkit-mask-position-y
        WebkitMaskPositionY = 81,
        /// -webkit-mask-clip
        WebkitMaskClip = 82,
        /// -webkit-mask-origin
        WebkitMaskOrigin = 83,
        /// -webkit-mask-size
        WebkitMaskSize = 84,
        /// -webkit-mask-composite
        WebkitMaskComposite = 85,
        /// -webkit-mask-image
        WebkitMaskImage = 86,
        /// -webkit-box-align
        WebkitBoxAlign = 87,
        /// -webkit-box-direction
        WebkitBoxDirection = 88,
        /// -webkit-box-flex
        WebkitBoxFlex = 89,
        /// -webkit-box-orient
        WebkitBoxOrient = 90,
        /// -webkit-box-pack
        WebkitBoxPack = 91,
        /// -webkit-box-ordinal-group
        WebkitBoxOrdinalGroup = 92,
        /// -moz-border-start
        MozBorderStart = 93,
        /// -moz-border-end
        MozBorderEnd = 94,
        /// -webkit-border-radius
        WebkitBorderRadius = 95,
        /// -moz-border-image
        MozBorderImage = 96,
        /// -webkit-border-image
        WebkitBorderImage = 97,
        /// -moz-transition
        MozTransition = 98,
        /// -webkit-transition
        WebkitTransition = 99,
        /// -moz-animation
        MozAnimation = 100,
        /// -webkit-animation
        WebkitAnimation = 101,
        /// -moz-columns
        MozColumns = 102,
        /// -moz-column-rule
        MozColumnRule = 103,
        /// -webkit-mask
        WebkitMask = 104,
        /// -webkit-mask-position
        WebkitMaskPosition = 105,
        /// -webkit-flex-flow
        WebkitFlexFlow = 106,
        /// -webkit-flex
        WebkitFlex = 107,
}

impl fmt::Debug for AliasId {
    fn fmt(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
        let name = match *self {
                AliasId::WebkitBackgroundClip => "WebkitBackgroundClip",
                AliasId::WebkitBackgroundOrigin => "WebkitBackgroundOrigin",
                AliasId::WebkitBackgroundSize => "WebkitBackgroundSize",
                AliasId::MozBorderStartColor => "MozBorderStartColor",
                AliasId::MozBorderStartStyle => "MozBorderStartStyle",
                AliasId::MozBorderStartWidth => "MozBorderStartWidth",
                AliasId::MozBorderEndColor => "MozBorderEndColor",
                AliasId::MozBorderEndStyle => "MozBorderEndStyle",
                AliasId::MozBorderEndWidth => "MozBorderEndWidth",
                AliasId::WebkitBorderTopLeftRadius => "WebkitBorderTopLeftRadius",
                AliasId::WebkitBorderTopRightRadius => "WebkitBorderTopRightRadius",
                AliasId::WebkitBorderBottomRightRadius => "WebkitBorderBottomRightRadius",
                AliasId::WebkitBorderBottomLeftRadius => "WebkitBorderBottomLeftRadius",
                AliasId::MozTransitionDuration => "MozTransitionDuration",
                AliasId::WebkitTransitionDuration => "WebkitTransitionDuration",
                AliasId::MozTransitionTimingFunction => "MozTransitionTimingFunction",
                AliasId::WebkitTransitionTimingFunction => "WebkitTransitionTimingFunction",
                AliasId::MozTransitionProperty => "MozTransitionProperty",
                AliasId::WebkitTransitionProperty => "WebkitTransitionProperty",
                AliasId::MozTransitionDelay => "MozTransitionDelay",
                AliasId::WebkitTransitionDelay => "WebkitTransitionDelay",
                AliasId::MozAnimationName => "MozAnimationName",
                AliasId::WebkitAnimationName => "WebkitAnimationName",
                AliasId::MozAnimationDuration => "MozAnimationDuration",
                AliasId::WebkitAnimationDuration => "WebkitAnimationDuration",
                AliasId::MozAnimationTimingFunction => "MozAnimationTimingFunction",
                AliasId::WebkitAnimationTimingFunction => "WebkitAnimationTimingFunction",
                AliasId::MozAnimationIterationCount => "MozAnimationIterationCount",
                AliasId::WebkitAnimationIterationCount => "WebkitAnimationIterationCount",
                AliasId::MozAnimationDirection => "MozAnimationDirection",
                AliasId::WebkitAnimationDirection => "WebkitAnimationDirection",
                AliasId::MozAnimationPlayState => "MozAnimationPlayState",
                AliasId::WebkitAnimationPlayState => "WebkitAnimationPlayState",
                AliasId::MozAnimationFillMode => "MozAnimationFillMode",
                AliasId::WebkitAnimationFillMode => "WebkitAnimationFillMode",
                AliasId::MozAnimationDelay => "MozAnimationDelay",
                AliasId::WebkitAnimationDelay => "WebkitAnimationDelay",
                AliasId::WebkitTransform => "WebkitTransform",
                AliasId::MozPerspective => "MozPerspective",
                AliasId::WebkitPerspective => "WebkitPerspective",
                AliasId::MozPerspectiveOrigin => "MozPerspectiveOrigin",
                AliasId::WebkitPerspectiveOrigin => "WebkitPerspectiveOrigin",
                AliasId::MozBackfaceVisibility => "MozBackfaceVisibility",
                AliasId::WebkitBackfaceVisibility => "WebkitBackfaceVisibility",
                AliasId::MozTransformStyle => "MozTransformStyle",
                AliasId::WebkitTransformStyle => "WebkitTransformStyle",
                AliasId::MozTransformOrigin => "MozTransformOrigin",
                AliasId::WebkitTransformOrigin => "WebkitTransformOrigin",
                AliasId::MozColumnWidth => "MozColumnWidth",
                AliasId::MozColumnCount => "MozColumnCount",
                AliasId::MozColumnGap => "MozColumnGap",
                AliasId::MozColumnFill => "MozColumnFill",
                AliasId::MozColumnRuleWidth => "MozColumnRuleWidth",
                AliasId::MozColumnRuleColor => "MozColumnRuleColor",
                AliasId::MozColumnRuleStyle => "MozColumnRuleStyle",
                AliasId::WebkitBoxShadow => "WebkitBoxShadow",
                AliasId::WebkitFilter => "WebkitFilter",
                AliasId::MozFontFeatureSettings => "MozFontFeatureSettings",
                AliasId::MozFontLanguageOverride => "MozFontLanguageOverride",
                AliasId::MozHyphens => "MozHyphens",
                AliasId::WebkitTextSizeAdjust => "WebkitTextSizeAdjust",
                AliasId::WordWrap => "WordWrap",
                AliasId::MozMarginStart => "MozMarginStart",
                AliasId::MozMarginEnd => "MozMarginEnd",
                AliasId::MozPaddingStart => "MozPaddingStart",
                AliasId::MozPaddingEnd => "MozPaddingEnd",
                AliasId::WebkitFlexDirection => "WebkitFlexDirection",
                AliasId::WebkitFlexWrap => "WebkitFlexWrap",
                AliasId::WebkitJustifyContent => "WebkitJustifyContent",
                AliasId::WebkitAlignContent => "WebkitAlignContent",
                AliasId::WebkitAlignItems => "WebkitAlignItems",
                AliasId::WebkitFlexGrow => "WebkitFlexGrow",
                AliasId::WebkitFlexShrink => "WebkitFlexShrink",
                AliasId::WebkitAlignSelf => "WebkitAlignSelf",
                AliasId::WebkitOrder => "WebkitOrder",
                AliasId::WebkitFlexBasis => "WebkitFlexBasis",
                AliasId::MozBoxSizing => "MozBoxSizing",
                AliasId::WebkitBoxSizing => "WebkitBoxSizing",
                AliasId::WebkitUserSelect => "WebkitUserSelect",
                AliasId::WebkitMaskRepeat => "WebkitMaskRepeat",
                AliasId::WebkitMaskPositionX => "WebkitMaskPositionX",
                AliasId::WebkitMaskPositionY => "WebkitMaskPositionY",
                AliasId::WebkitMaskClip => "WebkitMaskClip",
                AliasId::WebkitMaskOrigin => "WebkitMaskOrigin",
                AliasId::WebkitMaskSize => "WebkitMaskSize",
                AliasId::WebkitMaskComposite => "WebkitMaskComposite",
                AliasId::WebkitMaskImage => "WebkitMaskImage",
                AliasId::WebkitBoxAlign => "WebkitBoxAlign",
                AliasId::WebkitBoxDirection => "WebkitBoxDirection",
                AliasId::WebkitBoxFlex => "WebkitBoxFlex",
                AliasId::WebkitBoxOrient => "WebkitBoxOrient",
                AliasId::WebkitBoxPack => "WebkitBoxPack",
                AliasId::WebkitBoxOrdinalGroup => "WebkitBoxOrdinalGroup",
                AliasId::MozBorderStart => "MozBorderStart",
                AliasId::MozBorderEnd => "MozBorderEnd",
                AliasId::WebkitBorderRadius => "WebkitBorderRadius",
                AliasId::MozBorderImage => "MozBorderImage",
                AliasId::WebkitBorderImage => "WebkitBorderImage",
                AliasId::MozTransition => "MozTransition",
                AliasId::WebkitTransition => "WebkitTransition",
                AliasId::MozAnimation => "MozAnimation",
                AliasId::WebkitAnimation => "WebkitAnimation",
                AliasId::MozColumns => "MozColumns",
                AliasId::MozColumnRule => "MozColumnRule",
                AliasId::WebkitMask => "WebkitMask",
                AliasId::WebkitMaskPosition => "WebkitMaskPosition",
                AliasId::WebkitFlexFlow => "WebkitFlexFlow",
                AliasId::WebkitFlex => "WebkitFlex",
        };
        formatter.write_str(name)
    }
}

impl AliasId {
    /// Returns an nsCSSPropertyID.
    #[cfg(feature = "gecko")]
    #[allow(non_upper_case_globals)]
    pub fn to_nscsspropertyid(&self) -> Result<nsCSSPropertyID, ()> {
        use gecko_bindings::structs::*;

        match *self {
                AliasId::WebkitBackgroundClip => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_WebkitBackgroundClip)
                },
                AliasId::WebkitBackgroundOrigin => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_WebkitBackgroundOrigin)
                },
                AliasId::WebkitBackgroundSize => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_WebkitBackgroundSize)
                },
                AliasId::MozBorderStartColor => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_MozBorderStartColor)
                },
                AliasId::MozBorderStartStyle => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_MozBorderStartStyle)
                },
                AliasId::MozBorderStartWidth => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_MozBorderStartWidth)
                },
                AliasId::MozBorderEndColor => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_MozBorderEndColor)
                },
                AliasId::MozBorderEndStyle => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_MozBorderEndStyle)
                },
                AliasId::MozBorderEndWidth => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_MozBorderEndWidth)
                },
                AliasId::WebkitBorderTopLeftRadius => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_WebkitBorderTopLeftRadius)
                },
                AliasId::WebkitBorderTopRightRadius => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_WebkitBorderTopRightRadius)
                },
                AliasId::WebkitBorderBottomRightRadius => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_WebkitBorderBottomRightRadius)
                },
                AliasId::WebkitBorderBottomLeftRadius => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_WebkitBorderBottomLeftRadius)
                },
                AliasId::MozTransitionDuration => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_MozTransitionDuration)
                },
                AliasId::WebkitTransitionDuration => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_WebkitTransitionDuration)
                },
                AliasId::MozTransitionTimingFunction => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_MozTransitionTimingFunction)
                },
                AliasId::WebkitTransitionTimingFunction => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_WebkitTransitionTimingFunction)
                },
                AliasId::MozTransitionProperty => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_MozTransitionProperty)
                },
                AliasId::WebkitTransitionProperty => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_WebkitTransitionProperty)
                },
                AliasId::MozTransitionDelay => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_MozTransitionDelay)
                },
                AliasId::WebkitTransitionDelay => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_WebkitTransitionDelay)
                },
                AliasId::MozAnimationName => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_MozAnimationName)
                },
                AliasId::WebkitAnimationName => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_WebkitAnimationName)
                },
                AliasId::MozAnimationDuration => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_MozAnimationDuration)
                },
                AliasId::WebkitAnimationDuration => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_WebkitAnimationDuration)
                },
                AliasId::MozAnimationTimingFunction => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_MozAnimationTimingFunction)
                },
                AliasId::WebkitAnimationTimingFunction => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_WebkitAnimationTimingFunction)
                },
                AliasId::MozAnimationIterationCount => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_MozAnimationIterationCount)
                },
                AliasId::WebkitAnimationIterationCount => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_WebkitAnimationIterationCount)
                },
                AliasId::MozAnimationDirection => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_MozAnimationDirection)
                },
                AliasId::WebkitAnimationDirection => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_WebkitAnimationDirection)
                },
                AliasId::MozAnimationPlayState => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_MozAnimationPlayState)
                },
                AliasId::WebkitAnimationPlayState => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_WebkitAnimationPlayState)
                },
                AliasId::MozAnimationFillMode => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_MozAnimationFillMode)
                },
                AliasId::WebkitAnimationFillMode => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_WebkitAnimationFillMode)
                },
                AliasId::MozAnimationDelay => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_MozAnimationDelay)
                },
                AliasId::WebkitAnimationDelay => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_WebkitAnimationDelay)
                },
                AliasId::WebkitTransform => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_WebkitTransform)
                },
                AliasId::MozPerspective => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_MozPerspective)
                },
                AliasId::WebkitPerspective => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_WebkitPerspective)
                },
                AliasId::MozPerspectiveOrigin => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_MozPerspectiveOrigin)
                },
                AliasId::WebkitPerspectiveOrigin => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_WebkitPerspectiveOrigin)
                },
                AliasId::MozBackfaceVisibility => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_MozBackfaceVisibility)
                },
                AliasId::WebkitBackfaceVisibility => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_WebkitBackfaceVisibility)
                },
                AliasId::MozTransformStyle => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_MozTransformStyle)
                },
                AliasId::WebkitTransformStyle => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_WebkitTransformStyle)
                },
                AliasId::MozTransformOrigin => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_MozTransformOrigin)
                },
                AliasId::WebkitTransformOrigin => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_WebkitTransformOrigin)
                },
                AliasId::MozColumnWidth => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_MozColumnWidth)
                },
                AliasId::MozColumnCount => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_MozColumnCount)
                },
                AliasId::MozColumnGap => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_MozColumnGap)
                },
                AliasId::MozColumnFill => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_MozColumnFill)
                },
                AliasId::MozColumnRuleWidth => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_MozColumnRuleWidth)
                },
                AliasId::MozColumnRuleColor => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_MozColumnRuleColor)
                },
                AliasId::MozColumnRuleStyle => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_MozColumnRuleStyle)
                },
                AliasId::WebkitBoxShadow => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_WebkitBoxShadow)
                },
                AliasId::WebkitFilter => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_WebkitFilter)
                },
                AliasId::MozFontFeatureSettings => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_MozFontFeatureSettings)
                },
                AliasId::MozFontLanguageOverride => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_MozFontLanguageOverride)
                },
                AliasId::MozHyphens => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_MozHyphens)
                },
                AliasId::WebkitTextSizeAdjust => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_WebkitTextSizeAdjust)
                },
                AliasId::WordWrap => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_WordWrap)
                },
                AliasId::MozMarginStart => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_MozMarginStart)
                },
                AliasId::MozMarginEnd => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_MozMarginEnd)
                },
                AliasId::MozPaddingStart => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_MozPaddingStart)
                },
                AliasId::MozPaddingEnd => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_MozPaddingEnd)
                },
                AliasId::WebkitFlexDirection => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_WebkitFlexDirection)
                },
                AliasId::WebkitFlexWrap => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_WebkitFlexWrap)
                },
                AliasId::WebkitJustifyContent => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_WebkitJustifyContent)
                },
                AliasId::WebkitAlignContent => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_WebkitAlignContent)
                },
                AliasId::WebkitAlignItems => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_WebkitAlignItems)
                },
                AliasId::WebkitFlexGrow => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_WebkitFlexGrow)
                },
                AliasId::WebkitFlexShrink => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_WebkitFlexShrink)
                },
                AliasId::WebkitAlignSelf => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_WebkitAlignSelf)
                },
                AliasId::WebkitOrder => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_WebkitOrder)
                },
                AliasId::WebkitFlexBasis => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_WebkitFlexBasis)
                },
                AliasId::MozBoxSizing => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_MozBoxSizing)
                },
                AliasId::WebkitBoxSizing => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_WebkitBoxSizing)
                },
                AliasId::WebkitUserSelect => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_WebkitUserSelect)
                },
                AliasId::WebkitMaskRepeat => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_WebkitMaskRepeat)
                },
                AliasId::WebkitMaskPositionX => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_WebkitMaskPositionX)
                },
                AliasId::WebkitMaskPositionY => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_WebkitMaskPositionY)
                },
                AliasId::WebkitMaskClip => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_WebkitMaskClip)
                },
                AliasId::WebkitMaskOrigin => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_WebkitMaskOrigin)
                },
                AliasId::WebkitMaskSize => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_WebkitMaskSize)
                },
                AliasId::WebkitMaskComposite => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_WebkitMaskComposite)
                },
                AliasId::WebkitMaskImage => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_WebkitMaskImage)
                },
                AliasId::WebkitBoxAlign => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_WebkitBoxAlign)
                },
                AliasId::WebkitBoxDirection => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_WebkitBoxDirection)
                },
                AliasId::WebkitBoxFlex => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_WebkitBoxFlex)
                },
                AliasId::WebkitBoxOrient => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_WebkitBoxOrient)
                },
                AliasId::WebkitBoxPack => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_WebkitBoxPack)
                },
                AliasId::WebkitBoxOrdinalGroup => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_WebkitBoxOrdinalGroup)
                },
                AliasId::MozBorderStart => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_MozBorderStart)
                },
                AliasId::MozBorderEnd => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_MozBorderEnd)
                },
                AliasId::WebkitBorderRadius => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_WebkitBorderRadius)
                },
                AliasId::MozBorderImage => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_MozBorderImage)
                },
                AliasId::WebkitBorderImage => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_WebkitBorderImage)
                },
                AliasId::MozTransition => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_MozTransition)
                },
                AliasId::WebkitTransition => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_WebkitTransition)
                },
                AliasId::MozAnimation => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_MozAnimation)
                },
                AliasId::WebkitAnimation => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_WebkitAnimation)
                },
                AliasId::MozColumns => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_MozColumns)
                },
                AliasId::MozColumnRule => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_MozColumnRule)
                },
                AliasId::WebkitMask => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_WebkitMask)
                },
                AliasId::WebkitMaskPosition => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_WebkitMaskPosition)
                },
                AliasId::WebkitFlexFlow => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_WebkitFlexFlow)
                },
                AliasId::WebkitFlex => {
                    Ok(
    nsCSSPropertyID::eCSSPropertyAlias_WebkitFlex)
                },
        }
    }
}

#[macro_export]
macro_rules! css_properties_accessors {
    ($macro_name: ident) => {
        $macro_name! {
                                [Background_color, SetBackground_color,
                                 PropertyId::Longhand(LonghandId::BackgroundColor)],
                            [BackgroundColor, SetBackgroundColor,
                             PropertyId::Longhand(LonghandId::BackgroundColor)],
                                [Background_image, SetBackground_image,
                                 PropertyId::Longhand(LonghandId::BackgroundImage)],
                            [BackgroundImage, SetBackgroundImage,
                             PropertyId::Longhand(LonghandId::BackgroundImage)],
                                [Background_position_x, SetBackground_position_x,
                                 PropertyId::Longhand(LonghandId::BackgroundPositionX)],
                            [BackgroundPositionX, SetBackgroundPositionX,
                             PropertyId::Longhand(LonghandId::BackgroundPositionX)],
                                [Background_position_y, SetBackground_position_y,
                                 PropertyId::Longhand(LonghandId::BackgroundPositionY)],
                            [BackgroundPositionY, SetBackgroundPositionY,
                             PropertyId::Longhand(LonghandId::BackgroundPositionY)],
                                [Background_repeat, SetBackground_repeat,
                                 PropertyId::Longhand(LonghandId::BackgroundRepeat)],
                            [BackgroundRepeat, SetBackgroundRepeat,
                             PropertyId::Longhand(LonghandId::BackgroundRepeat)],
                                [Background_attachment, SetBackground_attachment,
                                 PropertyId::Longhand(LonghandId::BackgroundAttachment)],
                            [BackgroundAttachment, SetBackgroundAttachment,
                             PropertyId::Longhand(LonghandId::BackgroundAttachment)],
                                [Background_clip, SetBackground_clip,
                                 PropertyId::Longhand(LonghandId::BackgroundClip)],
                            [BackgroundClip, SetBackgroundClip,
                             PropertyId::Longhand(LonghandId::BackgroundClip)],
                                [_webkit_background_clip, Set_webkit_background_clip,
                                 PropertyId::Longhand(LonghandId::BackgroundClip)],
                            [WebkitBackgroundClip, SetWebkitBackgroundClip,
                             PropertyId::Longhand(LonghandId::BackgroundClip)],
                                [Background_origin, SetBackground_origin,
                                 PropertyId::Longhand(LonghandId::BackgroundOrigin)],
                            [BackgroundOrigin, SetBackgroundOrigin,
                             PropertyId::Longhand(LonghandId::BackgroundOrigin)],
                                [_webkit_background_origin, Set_webkit_background_origin,
                                 PropertyId::Longhand(LonghandId::BackgroundOrigin)],
                            [WebkitBackgroundOrigin, SetWebkitBackgroundOrigin,
                             PropertyId::Longhand(LonghandId::BackgroundOrigin)],
                                [Background_size, SetBackground_size,
                                 PropertyId::Longhand(LonghandId::BackgroundSize)],
                            [BackgroundSize, SetBackgroundSize,
                             PropertyId::Longhand(LonghandId::BackgroundSize)],
                                [_webkit_background_size, Set_webkit_background_size,
                                 PropertyId::Longhand(LonghandId::BackgroundSize)],
                            [WebkitBackgroundSize, SetWebkitBackgroundSize,
                             PropertyId::Longhand(LonghandId::BackgroundSize)],
                                [Background_blend_mode, SetBackground_blend_mode,
                                 PropertyId::Longhand(LonghandId::BackgroundBlendMode)],
                            [BackgroundBlendMode, SetBackgroundBlendMode,
                             PropertyId::Longhand(LonghandId::BackgroundBlendMode)],
                                [Border_top_color, SetBorder_top_color,
                                 PropertyId::Longhand(LonghandId::BorderTopColor)],
                            [BorderTopColor, SetBorderTopColor,
                             PropertyId::Longhand(LonghandId::BorderTopColor)],
                                [Border_top_style, SetBorder_top_style,
                                 PropertyId::Longhand(LonghandId::BorderTopStyle)],
                            [BorderTopStyle, SetBorderTopStyle,
                             PropertyId::Longhand(LonghandId::BorderTopStyle)],
                                [Border_top_width, SetBorder_top_width,
                                 PropertyId::Longhand(LonghandId::BorderTopWidth)],
                            [BorderTopWidth, SetBorderTopWidth,
                             PropertyId::Longhand(LonghandId::BorderTopWidth)],
                                [Border_left_color, SetBorder_left_color,
                                 PropertyId::Longhand(LonghandId::BorderLeftColor)],
                            [BorderLeftColor, SetBorderLeftColor,
                             PropertyId::Longhand(LonghandId::BorderLeftColor)],
                                [Border_left_style, SetBorder_left_style,
                                 PropertyId::Longhand(LonghandId::BorderLeftStyle)],
                            [BorderLeftStyle, SetBorderLeftStyle,
                             PropertyId::Longhand(LonghandId::BorderLeftStyle)],
                                [Border_left_width, SetBorder_left_width,
                                 PropertyId::Longhand(LonghandId::BorderLeftWidth)],
                            [BorderLeftWidth, SetBorderLeftWidth,
                             PropertyId::Longhand(LonghandId::BorderLeftWidth)],
                                [Border_bottom_color, SetBorder_bottom_color,
                                 PropertyId::Longhand(LonghandId::BorderBottomColor)],
                            [BorderBottomColor, SetBorderBottomColor,
                             PropertyId::Longhand(LonghandId::BorderBottomColor)],
                                [Border_bottom_style, SetBorder_bottom_style,
                                 PropertyId::Longhand(LonghandId::BorderBottomStyle)],
                            [BorderBottomStyle, SetBorderBottomStyle,
                             PropertyId::Longhand(LonghandId::BorderBottomStyle)],
                                [Border_bottom_width, SetBorder_bottom_width,
                                 PropertyId::Longhand(LonghandId::BorderBottomWidth)],
                            [BorderBottomWidth, SetBorderBottomWidth,
                             PropertyId::Longhand(LonghandId::BorderBottomWidth)],
                                [Border_right_color, SetBorder_right_color,
                                 PropertyId::Longhand(LonghandId::BorderRightColor)],
                            [BorderRightColor, SetBorderRightColor,
                             PropertyId::Longhand(LonghandId::BorderRightColor)],
                                [Border_right_style, SetBorder_right_style,
                                 PropertyId::Longhand(LonghandId::BorderRightStyle)],
                            [BorderRightStyle, SetBorderRightStyle,
                             PropertyId::Longhand(LonghandId::BorderRightStyle)],
                                [Border_right_width, SetBorder_right_width,
                                 PropertyId::Longhand(LonghandId::BorderRightWidth)],
                            [BorderRightWidth, SetBorderRightWidth,
                             PropertyId::Longhand(LonghandId::BorderRightWidth)],
                                [Border_block_start_color, SetBorder_block_start_color,
                                 PropertyId::Longhand(LonghandId::BorderBlockStartColor)],
                            [BorderBlockStartColor, SetBorderBlockStartColor,
                             PropertyId::Longhand(LonghandId::BorderBlockStartColor)],
                                [Border_block_start_style, SetBorder_block_start_style,
                                 PropertyId::Longhand(LonghandId::BorderBlockStartStyle)],
                            [BorderBlockStartStyle, SetBorderBlockStartStyle,
                             PropertyId::Longhand(LonghandId::BorderBlockStartStyle)],
                                [Border_block_start_width, SetBorder_block_start_width,
                                 PropertyId::Longhand(LonghandId::BorderBlockStartWidth)],
                            [BorderBlockStartWidth, SetBorderBlockStartWidth,
                             PropertyId::Longhand(LonghandId::BorderBlockStartWidth)],
                                [Border_block_end_color, SetBorder_block_end_color,
                                 PropertyId::Longhand(LonghandId::BorderBlockEndColor)],
                            [BorderBlockEndColor, SetBorderBlockEndColor,
                             PropertyId::Longhand(LonghandId::BorderBlockEndColor)],
                                [Border_block_end_style, SetBorder_block_end_style,
                                 PropertyId::Longhand(LonghandId::BorderBlockEndStyle)],
                            [BorderBlockEndStyle, SetBorderBlockEndStyle,
                             PropertyId::Longhand(LonghandId::BorderBlockEndStyle)],
                                [Border_block_end_width, SetBorder_block_end_width,
                                 PropertyId::Longhand(LonghandId::BorderBlockEndWidth)],
                            [BorderBlockEndWidth, SetBorderBlockEndWidth,
                             PropertyId::Longhand(LonghandId::BorderBlockEndWidth)],
                                [Border_inline_start_color, SetBorder_inline_start_color,
                                 PropertyId::Longhand(LonghandId::BorderInlineStartColor)],
                            [BorderInlineStartColor, SetBorderInlineStartColor,
                             PropertyId::Longhand(LonghandId::BorderInlineStartColor)],
                                [_moz_border_start_color, Set_moz_border_start_color,
                                 PropertyId::Longhand(LonghandId::BorderInlineStartColor)],
                            [MozBorderStartColor, SetMozBorderStartColor,
                             PropertyId::Longhand(LonghandId::BorderInlineStartColor)],
                                [Border_inline_start_style, SetBorder_inline_start_style,
                                 PropertyId::Longhand(LonghandId::BorderInlineStartStyle)],
                            [BorderInlineStartStyle, SetBorderInlineStartStyle,
                             PropertyId::Longhand(LonghandId::BorderInlineStartStyle)],
                                [_moz_border_start_style, Set_moz_border_start_style,
                                 PropertyId::Longhand(LonghandId::BorderInlineStartStyle)],
                            [MozBorderStartStyle, SetMozBorderStartStyle,
                             PropertyId::Longhand(LonghandId::BorderInlineStartStyle)],
                                [Border_inline_start_width, SetBorder_inline_start_width,
                                 PropertyId::Longhand(LonghandId::BorderInlineStartWidth)],
                            [BorderInlineStartWidth, SetBorderInlineStartWidth,
                             PropertyId::Longhand(LonghandId::BorderInlineStartWidth)],
                                [_moz_border_start_width, Set_moz_border_start_width,
                                 PropertyId::Longhand(LonghandId::BorderInlineStartWidth)],
                            [MozBorderStartWidth, SetMozBorderStartWidth,
                             PropertyId::Longhand(LonghandId::BorderInlineStartWidth)],
                                [Border_inline_end_color, SetBorder_inline_end_color,
                                 PropertyId::Longhand(LonghandId::BorderInlineEndColor)],
                            [BorderInlineEndColor, SetBorderInlineEndColor,
                             PropertyId::Longhand(LonghandId::BorderInlineEndColor)],
                                [_moz_border_end_color, Set_moz_border_end_color,
                                 PropertyId::Longhand(LonghandId::BorderInlineEndColor)],
                            [MozBorderEndColor, SetMozBorderEndColor,
                             PropertyId::Longhand(LonghandId::BorderInlineEndColor)],
                                [Border_inline_end_style, SetBorder_inline_end_style,
                                 PropertyId::Longhand(LonghandId::BorderInlineEndStyle)],
                            [BorderInlineEndStyle, SetBorderInlineEndStyle,
                             PropertyId::Longhand(LonghandId::BorderInlineEndStyle)],
                                [_moz_border_end_style, Set_moz_border_end_style,
                                 PropertyId::Longhand(LonghandId::BorderInlineEndStyle)],
                            [MozBorderEndStyle, SetMozBorderEndStyle,
                             PropertyId::Longhand(LonghandId::BorderInlineEndStyle)],
                                [Border_inline_end_width, SetBorder_inline_end_width,
                                 PropertyId::Longhand(LonghandId::BorderInlineEndWidth)],
                            [BorderInlineEndWidth, SetBorderInlineEndWidth,
                             PropertyId::Longhand(LonghandId::BorderInlineEndWidth)],
                                [_moz_border_end_width, Set_moz_border_end_width,
                                 PropertyId::Longhand(LonghandId::BorderInlineEndWidth)],
                            [MozBorderEndWidth, SetMozBorderEndWidth,
                             PropertyId::Longhand(LonghandId::BorderInlineEndWidth)],
                                [Border_top_left_radius, SetBorder_top_left_radius,
                                 PropertyId::Longhand(LonghandId::BorderTopLeftRadius)],
                            [BorderTopLeftRadius, SetBorderTopLeftRadius,
                             PropertyId::Longhand(LonghandId::BorderTopLeftRadius)],
                                [_webkit_border_top_left_radius, Set_webkit_border_top_left_radius,
                                 PropertyId::Longhand(LonghandId::BorderTopLeftRadius)],
                            [WebkitBorderTopLeftRadius, SetWebkitBorderTopLeftRadius,
                             PropertyId::Longhand(LonghandId::BorderTopLeftRadius)],
                                [Border_top_right_radius, SetBorder_top_right_radius,
                                 PropertyId::Longhand(LonghandId::BorderTopRightRadius)],
                            [BorderTopRightRadius, SetBorderTopRightRadius,
                             PropertyId::Longhand(LonghandId::BorderTopRightRadius)],
                                [_webkit_border_top_right_radius, Set_webkit_border_top_right_radius,
                                 PropertyId::Longhand(LonghandId::BorderTopRightRadius)],
                            [WebkitBorderTopRightRadius, SetWebkitBorderTopRightRadius,
                             PropertyId::Longhand(LonghandId::BorderTopRightRadius)],
                                [Border_bottom_right_radius, SetBorder_bottom_right_radius,
                                 PropertyId::Longhand(LonghandId::BorderBottomRightRadius)],
                            [BorderBottomRightRadius, SetBorderBottomRightRadius,
                             PropertyId::Longhand(LonghandId::BorderBottomRightRadius)],
                                [_webkit_border_bottom_right_radius, Set_webkit_border_bottom_right_radius,
                                 PropertyId::Longhand(LonghandId::BorderBottomRightRadius)],
                            [WebkitBorderBottomRightRadius, SetWebkitBorderBottomRightRadius,
                             PropertyId::Longhand(LonghandId::BorderBottomRightRadius)],
                                [Border_bottom_left_radius, SetBorder_bottom_left_radius,
                                 PropertyId::Longhand(LonghandId::BorderBottomLeftRadius)],
                            [BorderBottomLeftRadius, SetBorderBottomLeftRadius,
                             PropertyId::Longhand(LonghandId::BorderBottomLeftRadius)],
                                [_webkit_border_bottom_left_radius, Set_webkit_border_bottom_left_radius,
                                 PropertyId::Longhand(LonghandId::BorderBottomLeftRadius)],
                            [WebkitBorderBottomLeftRadius, SetWebkitBorderBottomLeftRadius,
                             PropertyId::Longhand(LonghandId::BorderBottomLeftRadius)],
                                [_moz_border_top_colors, Set_moz_border_top_colors,
                                 PropertyId::Longhand(LonghandId::MozBorderTopColors)],
                            [MozBorderTopColors, SetMozBorderTopColors,
                             PropertyId::Longhand(LonghandId::MozBorderTopColors)],
                                [_moz_border_left_colors, Set_moz_border_left_colors,
                                 PropertyId::Longhand(LonghandId::MozBorderLeftColors)],
                            [MozBorderLeftColors, SetMozBorderLeftColors,
                             PropertyId::Longhand(LonghandId::MozBorderLeftColors)],
                                [_moz_border_bottom_colors, Set_moz_border_bottom_colors,
                                 PropertyId::Longhand(LonghandId::MozBorderBottomColors)],
                            [MozBorderBottomColors, SetMozBorderBottomColors,
                             PropertyId::Longhand(LonghandId::MozBorderBottomColors)],
                                [_moz_border_right_colors, Set_moz_border_right_colors,
                                 PropertyId::Longhand(LonghandId::MozBorderRightColors)],
                            [MozBorderRightColors, SetMozBorderRightColors,
                             PropertyId::Longhand(LonghandId::MozBorderRightColors)],
                                [Box_decoration_break, SetBox_decoration_break,
                                 PropertyId::Longhand(LonghandId::BoxDecorationBreak)],
                            [BoxDecorationBreak, SetBoxDecorationBreak,
                             PropertyId::Longhand(LonghandId::BoxDecorationBreak)],
                                [_moz_float_edge, Set_moz_float_edge,
                                 PropertyId::Longhand(LonghandId::MozFloatEdge)],
                            [MozFloatEdge, SetMozFloatEdge,
                             PropertyId::Longhand(LonghandId::MozFloatEdge)],
                                [Border_image_source, SetBorder_image_source,
                                 PropertyId::Longhand(LonghandId::BorderImageSource)],
                            [BorderImageSource, SetBorderImageSource,
                             PropertyId::Longhand(LonghandId::BorderImageSource)],
                                [Border_image_outset, SetBorder_image_outset,
                                 PropertyId::Longhand(LonghandId::BorderImageOutset)],
                            [BorderImageOutset, SetBorderImageOutset,
                             PropertyId::Longhand(LonghandId::BorderImageOutset)],
                                [Border_image_repeat, SetBorder_image_repeat,
                                 PropertyId::Longhand(LonghandId::BorderImageRepeat)],
                            [BorderImageRepeat, SetBorderImageRepeat,
                             PropertyId::Longhand(LonghandId::BorderImageRepeat)],
                                [Border_image_width, SetBorder_image_width,
                                 PropertyId::Longhand(LonghandId::BorderImageWidth)],
                            [BorderImageWidth, SetBorderImageWidth,
                             PropertyId::Longhand(LonghandId::BorderImageWidth)],
                                [Border_image_slice, SetBorder_image_slice,
                                 PropertyId::Longhand(LonghandId::BorderImageSlice)],
                            [BorderImageSlice, SetBorderImageSlice,
                             PropertyId::Longhand(LonghandId::BorderImageSlice)],
                            [Display, SetDisplay,
                             PropertyId::Longhand(LonghandId::Display)],
                            [Position, SetPosition,
                             PropertyId::Longhand(LonghandId::Position)],
                            [Float, SetFloat,
                             PropertyId::Longhand(LonghandId::Float)],
                            [Clear, SetClear,
                             PropertyId::Longhand(LonghandId::Clear)],
                                [Vertical_align, SetVertical_align,
                                 PropertyId::Longhand(LonghandId::VerticalAlign)],
                            [VerticalAlign, SetVerticalAlign,
                             PropertyId::Longhand(LonghandId::VerticalAlign)],
                                [Overflow_x, SetOverflow_x,
                                 PropertyId::Longhand(LonghandId::OverflowX)],
                            [OverflowX, SetOverflowX,
                             PropertyId::Longhand(LonghandId::OverflowX)],
                                [Overflow_y, SetOverflow_y,
                                 PropertyId::Longhand(LonghandId::OverflowY)],
                            [OverflowY, SetOverflowY,
                             PropertyId::Longhand(LonghandId::OverflowY)],
                                [Transition_duration, SetTransition_duration,
                                 PropertyId::Longhand(LonghandId::TransitionDuration)],
                            [TransitionDuration, SetTransitionDuration,
                             PropertyId::Longhand(LonghandId::TransitionDuration)],
                                [_moz_transition_duration, Set_moz_transition_duration,
                                 PropertyId::Longhand(LonghandId::TransitionDuration)],
                            [MozTransitionDuration, SetMozTransitionDuration,
                             PropertyId::Longhand(LonghandId::TransitionDuration)],
                                [_webkit_transition_duration, Set_webkit_transition_duration,
                                 PropertyId::Longhand(LonghandId::TransitionDuration)],
                            [WebkitTransitionDuration, SetWebkitTransitionDuration,
                             PropertyId::Longhand(LonghandId::TransitionDuration)],
                                [Transition_timing_function, SetTransition_timing_function,
                                 PropertyId::Longhand(LonghandId::TransitionTimingFunction)],
                            [TransitionTimingFunction, SetTransitionTimingFunction,
                             PropertyId::Longhand(LonghandId::TransitionTimingFunction)],
                                [_moz_transition_timing_function, Set_moz_transition_timing_function,
                                 PropertyId::Longhand(LonghandId::TransitionTimingFunction)],
                            [MozTransitionTimingFunction, SetMozTransitionTimingFunction,
                             PropertyId::Longhand(LonghandId::TransitionTimingFunction)],
                                [_webkit_transition_timing_function, Set_webkit_transition_timing_function,
                                 PropertyId::Longhand(LonghandId::TransitionTimingFunction)],
                            [WebkitTransitionTimingFunction, SetWebkitTransitionTimingFunction,
                             PropertyId::Longhand(LonghandId::TransitionTimingFunction)],
                                [Transition_property, SetTransition_property,
                                 PropertyId::Longhand(LonghandId::TransitionProperty)],
                            [TransitionProperty, SetTransitionProperty,
                             PropertyId::Longhand(LonghandId::TransitionProperty)],
                                [_moz_transition_property, Set_moz_transition_property,
                                 PropertyId::Longhand(LonghandId::TransitionProperty)],
                            [MozTransitionProperty, SetMozTransitionProperty,
                             PropertyId::Longhand(LonghandId::TransitionProperty)],
                                [_webkit_transition_property, Set_webkit_transition_property,
                                 PropertyId::Longhand(LonghandId::TransitionProperty)],
                            [WebkitTransitionProperty, SetWebkitTransitionProperty,
                             PropertyId::Longhand(LonghandId::TransitionProperty)],
                                [Transition_delay, SetTransition_delay,
                                 PropertyId::Longhand(LonghandId::TransitionDelay)],
                            [TransitionDelay, SetTransitionDelay,
                             PropertyId::Longhand(LonghandId::TransitionDelay)],
                                [_moz_transition_delay, Set_moz_transition_delay,
                                 PropertyId::Longhand(LonghandId::TransitionDelay)],
                            [MozTransitionDelay, SetMozTransitionDelay,
                             PropertyId::Longhand(LonghandId::TransitionDelay)],
                                [_webkit_transition_delay, Set_webkit_transition_delay,
                                 PropertyId::Longhand(LonghandId::TransitionDelay)],
                            [WebkitTransitionDelay, SetWebkitTransitionDelay,
                             PropertyId::Longhand(LonghandId::TransitionDelay)],
                                [Animation_name, SetAnimation_name,
                                 PropertyId::Longhand(LonghandId::AnimationName)],
                            [AnimationName, SetAnimationName,
                             PropertyId::Longhand(LonghandId::AnimationName)],
                                [_moz_animation_name, Set_moz_animation_name,
                                 PropertyId::Longhand(LonghandId::AnimationName)],
                            [MozAnimationName, SetMozAnimationName,
                             PropertyId::Longhand(LonghandId::AnimationName)],
                                [_webkit_animation_name, Set_webkit_animation_name,
                                 PropertyId::Longhand(LonghandId::AnimationName)],
                            [WebkitAnimationName, SetWebkitAnimationName,
                             PropertyId::Longhand(LonghandId::AnimationName)],
                                [Animation_duration, SetAnimation_duration,
                                 PropertyId::Longhand(LonghandId::AnimationDuration)],
                            [AnimationDuration, SetAnimationDuration,
                             PropertyId::Longhand(LonghandId::AnimationDuration)],
                                [_moz_animation_duration, Set_moz_animation_duration,
                                 PropertyId::Longhand(LonghandId::AnimationDuration)],
                            [MozAnimationDuration, SetMozAnimationDuration,
                             PropertyId::Longhand(LonghandId::AnimationDuration)],
                                [_webkit_animation_duration, Set_webkit_animation_duration,
                                 PropertyId::Longhand(LonghandId::AnimationDuration)],
                            [WebkitAnimationDuration, SetWebkitAnimationDuration,
                             PropertyId::Longhand(LonghandId::AnimationDuration)],
                                [Animation_timing_function, SetAnimation_timing_function,
                                 PropertyId::Longhand(LonghandId::AnimationTimingFunction)],
                            [AnimationTimingFunction, SetAnimationTimingFunction,
                             PropertyId::Longhand(LonghandId::AnimationTimingFunction)],
                                [_moz_animation_timing_function, Set_moz_animation_timing_function,
                                 PropertyId::Longhand(LonghandId::AnimationTimingFunction)],
                            [MozAnimationTimingFunction, SetMozAnimationTimingFunction,
                             PropertyId::Longhand(LonghandId::AnimationTimingFunction)],
                                [_webkit_animation_timing_function, Set_webkit_animation_timing_function,
                                 PropertyId::Longhand(LonghandId::AnimationTimingFunction)],
                            [WebkitAnimationTimingFunction, SetWebkitAnimationTimingFunction,
                             PropertyId::Longhand(LonghandId::AnimationTimingFunction)],
                                [Animation_iteration_count, SetAnimation_iteration_count,
                                 PropertyId::Longhand(LonghandId::AnimationIterationCount)],
                            [AnimationIterationCount, SetAnimationIterationCount,
                             PropertyId::Longhand(LonghandId::AnimationIterationCount)],
                                [_moz_animation_iteration_count, Set_moz_animation_iteration_count,
                                 PropertyId::Longhand(LonghandId::AnimationIterationCount)],
                            [MozAnimationIterationCount, SetMozAnimationIterationCount,
                             PropertyId::Longhand(LonghandId::AnimationIterationCount)],
                                [_webkit_animation_iteration_count, Set_webkit_animation_iteration_count,
                                 PropertyId::Longhand(LonghandId::AnimationIterationCount)],
                            [WebkitAnimationIterationCount, SetWebkitAnimationIterationCount,
                             PropertyId::Longhand(LonghandId::AnimationIterationCount)],
                                [Animation_direction, SetAnimation_direction,
                                 PropertyId::Longhand(LonghandId::AnimationDirection)],
                            [AnimationDirection, SetAnimationDirection,
                             PropertyId::Longhand(LonghandId::AnimationDirection)],
                                [_moz_animation_direction, Set_moz_animation_direction,
                                 PropertyId::Longhand(LonghandId::AnimationDirection)],
                            [MozAnimationDirection, SetMozAnimationDirection,
                             PropertyId::Longhand(LonghandId::AnimationDirection)],
                                [_webkit_animation_direction, Set_webkit_animation_direction,
                                 PropertyId::Longhand(LonghandId::AnimationDirection)],
                            [WebkitAnimationDirection, SetWebkitAnimationDirection,
                             PropertyId::Longhand(LonghandId::AnimationDirection)],
                                [Animation_play_state, SetAnimation_play_state,
                                 PropertyId::Longhand(LonghandId::AnimationPlayState)],
                            [AnimationPlayState, SetAnimationPlayState,
                             PropertyId::Longhand(LonghandId::AnimationPlayState)],
                                [_moz_animation_play_state, Set_moz_animation_play_state,
                                 PropertyId::Longhand(LonghandId::AnimationPlayState)],
                            [MozAnimationPlayState, SetMozAnimationPlayState,
                             PropertyId::Longhand(LonghandId::AnimationPlayState)],
                                [_webkit_animation_play_state, Set_webkit_animation_play_state,
                                 PropertyId::Longhand(LonghandId::AnimationPlayState)],
                            [WebkitAnimationPlayState, SetWebkitAnimationPlayState,
                             PropertyId::Longhand(LonghandId::AnimationPlayState)],
                                [Animation_fill_mode, SetAnimation_fill_mode,
                                 PropertyId::Longhand(LonghandId::AnimationFillMode)],
                            [AnimationFillMode, SetAnimationFillMode,
                             PropertyId::Longhand(LonghandId::AnimationFillMode)],
                                [_moz_animation_fill_mode, Set_moz_animation_fill_mode,
                                 PropertyId::Longhand(LonghandId::AnimationFillMode)],
                            [MozAnimationFillMode, SetMozAnimationFillMode,
                             PropertyId::Longhand(LonghandId::AnimationFillMode)],
                                [_webkit_animation_fill_mode, Set_webkit_animation_fill_mode,
                                 PropertyId::Longhand(LonghandId::AnimationFillMode)],
                            [WebkitAnimationFillMode, SetWebkitAnimationFillMode,
                             PropertyId::Longhand(LonghandId::AnimationFillMode)],
                                [Animation_delay, SetAnimation_delay,
                                 PropertyId::Longhand(LonghandId::AnimationDelay)],
                            [AnimationDelay, SetAnimationDelay,
                             PropertyId::Longhand(LonghandId::AnimationDelay)],
                                [_moz_animation_delay, Set_moz_animation_delay,
                                 PropertyId::Longhand(LonghandId::AnimationDelay)],
                            [MozAnimationDelay, SetMozAnimationDelay,
                             PropertyId::Longhand(LonghandId::AnimationDelay)],
                                [_webkit_animation_delay, Set_webkit_animation_delay,
                                 PropertyId::Longhand(LonghandId::AnimationDelay)],
                            [WebkitAnimationDelay, SetWebkitAnimationDelay,
                             PropertyId::Longhand(LonghandId::AnimationDelay)],
                                [Scroll_snap_points_x, SetScroll_snap_points_x,
                                 PropertyId::Longhand(LonghandId::ScrollSnapPointsX)],
                            [ScrollSnapPointsX, SetScrollSnapPointsX,
                             PropertyId::Longhand(LonghandId::ScrollSnapPointsX)],
                                [Scroll_snap_points_y, SetScroll_snap_points_y,
                                 PropertyId::Longhand(LonghandId::ScrollSnapPointsY)],
                            [ScrollSnapPointsY, SetScrollSnapPointsY,
                             PropertyId::Longhand(LonghandId::ScrollSnapPointsY)],
                                [Scroll_snap_destination, SetScroll_snap_destination,
                                 PropertyId::Longhand(LonghandId::ScrollSnapDestination)],
                            [ScrollSnapDestination, SetScrollSnapDestination,
                             PropertyId::Longhand(LonghandId::ScrollSnapDestination)],
                                [Scroll_snap_coordinate, SetScroll_snap_coordinate,
                                 PropertyId::Longhand(LonghandId::ScrollSnapCoordinate)],
                            [ScrollSnapCoordinate, SetScrollSnapCoordinate,
                             PropertyId::Longhand(LonghandId::ScrollSnapCoordinate)],
                            [Transform, SetTransform,
                             PropertyId::Longhand(LonghandId::Transform)],
                                [_webkit_transform, Set_webkit_transform,
                                 PropertyId::Longhand(LonghandId::Transform)],
                            [WebkitTransform, SetWebkitTransform,
                             PropertyId::Longhand(LonghandId::Transform)],
                                [Scroll_behavior, SetScroll_behavior,
                                 PropertyId::Longhand(LonghandId::ScrollBehavior)],
                            [ScrollBehavior, SetScrollBehavior,
                             PropertyId::Longhand(LonghandId::ScrollBehavior)],
                                [Scroll_snap_type_x, SetScroll_snap_type_x,
                                 PropertyId::Longhand(LonghandId::ScrollSnapTypeX)],
                            [ScrollSnapTypeX, SetScrollSnapTypeX,
                             PropertyId::Longhand(LonghandId::ScrollSnapTypeX)],
                                [Scroll_snap_type_y, SetScroll_snap_type_y,
                                 PropertyId::Longhand(LonghandId::ScrollSnapTypeY)],
                            [ScrollSnapTypeY, SetScrollSnapTypeY,
                             PropertyId::Longhand(LonghandId::ScrollSnapTypeY)],
                            [Isolation, SetIsolation,
                             PropertyId::Longhand(LonghandId::Isolation)],
                                [Page_break_after, SetPage_break_after,
                                 PropertyId::Longhand(LonghandId::PageBreakAfter)],
                            [PageBreakAfter, SetPageBreakAfter,
                             PropertyId::Longhand(LonghandId::PageBreakAfter)],
                                [Page_break_before, SetPage_break_before,
                                 PropertyId::Longhand(LonghandId::PageBreakBefore)],
                            [PageBreakBefore, SetPageBreakBefore,
                             PropertyId::Longhand(LonghandId::PageBreakBefore)],
                                [Page_break_inside, SetPage_break_inside,
                                 PropertyId::Longhand(LonghandId::PageBreakInside)],
                            [PageBreakInside, SetPageBreakInside,
                             PropertyId::Longhand(LonghandId::PageBreakInside)],
                            [Resize, SetResize,
                             PropertyId::Longhand(LonghandId::Resize)],
                            [Perspective, SetPerspective,
                             PropertyId::Longhand(LonghandId::Perspective)],
                                [_moz_perspective, Set_moz_perspective,
                                 PropertyId::Longhand(LonghandId::Perspective)],
                            [MozPerspective, SetMozPerspective,
                             PropertyId::Longhand(LonghandId::Perspective)],
                                [_webkit_perspective, Set_webkit_perspective,
                                 PropertyId::Longhand(LonghandId::Perspective)],
                            [WebkitPerspective, SetWebkitPerspective,
                             PropertyId::Longhand(LonghandId::Perspective)],
                                [Perspective_origin, SetPerspective_origin,
                                 PropertyId::Longhand(LonghandId::PerspectiveOrigin)],
                            [PerspectiveOrigin, SetPerspectiveOrigin,
                             PropertyId::Longhand(LonghandId::PerspectiveOrigin)],
                                [_moz_perspective_origin, Set_moz_perspective_origin,
                                 PropertyId::Longhand(LonghandId::PerspectiveOrigin)],
                            [MozPerspectiveOrigin, SetMozPerspectiveOrigin,
                             PropertyId::Longhand(LonghandId::PerspectiveOrigin)],
                                [_webkit_perspective_origin, Set_webkit_perspective_origin,
                                 PropertyId::Longhand(LonghandId::PerspectiveOrigin)],
                            [WebkitPerspectiveOrigin, SetWebkitPerspectiveOrigin,
                             PropertyId::Longhand(LonghandId::PerspectiveOrigin)],
                                [Backface_visibility, SetBackface_visibility,
                                 PropertyId::Longhand(LonghandId::BackfaceVisibility)],
                            [BackfaceVisibility, SetBackfaceVisibility,
                             PropertyId::Longhand(LonghandId::BackfaceVisibility)],
                                [_moz_backface_visibility, Set_moz_backface_visibility,
                                 PropertyId::Longhand(LonghandId::BackfaceVisibility)],
                            [MozBackfaceVisibility, SetMozBackfaceVisibility,
                             PropertyId::Longhand(LonghandId::BackfaceVisibility)],
                                [_webkit_backface_visibility, Set_webkit_backface_visibility,
                                 PropertyId::Longhand(LonghandId::BackfaceVisibility)],
                            [WebkitBackfaceVisibility, SetWebkitBackfaceVisibility,
                             PropertyId::Longhand(LonghandId::BackfaceVisibility)],
                                [Transform_box, SetTransform_box,
                                 PropertyId::Longhand(LonghandId::TransformBox)],
                            [TransformBox, SetTransformBox,
                             PropertyId::Longhand(LonghandId::TransformBox)],
                                [Transform_style, SetTransform_style,
                                 PropertyId::Longhand(LonghandId::TransformStyle)],
                            [TransformStyle, SetTransformStyle,
                             PropertyId::Longhand(LonghandId::TransformStyle)],
                                [_moz_transform_style, Set_moz_transform_style,
                                 PropertyId::Longhand(LonghandId::TransformStyle)],
                            [MozTransformStyle, SetMozTransformStyle,
                             PropertyId::Longhand(LonghandId::TransformStyle)],
                                [_webkit_transform_style, Set_webkit_transform_style,
                                 PropertyId::Longhand(LonghandId::TransformStyle)],
                            [WebkitTransformStyle, SetWebkitTransformStyle,
                             PropertyId::Longhand(LonghandId::TransformStyle)],
                                [Transform_origin, SetTransform_origin,
                                 PropertyId::Longhand(LonghandId::TransformOrigin)],
                            [TransformOrigin, SetTransformOrigin,
                             PropertyId::Longhand(LonghandId::TransformOrigin)],
                                [_moz_transform_origin, Set_moz_transform_origin,
                                 PropertyId::Longhand(LonghandId::TransformOrigin)],
                            [MozTransformOrigin, SetMozTransformOrigin,
                             PropertyId::Longhand(LonghandId::TransformOrigin)],
                                [_webkit_transform_origin, Set_webkit_transform_origin,
                                 PropertyId::Longhand(LonghandId::TransformOrigin)],
                            [WebkitTransformOrigin, SetWebkitTransformOrigin,
                             PropertyId::Longhand(LonghandId::TransformOrigin)],
                            [Contain, SetContain,
                             PropertyId::Longhand(LonghandId::Contain)],
                                [_moz_appearance, Set_moz_appearance,
                                 PropertyId::Longhand(LonghandId::MozAppearance)],
                            [MozAppearance, SetMozAppearance,
                             PropertyId::Longhand(LonghandId::MozAppearance)],
                                [_moz_binding, Set_moz_binding,
                                 PropertyId::Longhand(LonghandId::MozBinding)],
                            [MozBinding, SetMozBinding,
                             PropertyId::Longhand(LonghandId::MozBinding)],
                                [_moz_orient, Set_moz_orient,
                                 PropertyId::Longhand(LonghandId::MozOrient)],
                            [MozOrient, SetMozOrient,
                             PropertyId::Longhand(LonghandId::MozOrient)],
                                [Will_change, SetWill_change,
                                 PropertyId::Longhand(LonghandId::WillChange)],
                            [WillChange, SetWillChange,
                             PropertyId::Longhand(LonghandId::WillChange)],
                                [Shape_outside, SetShape_outside,
                                 PropertyId::Longhand(LonghandId::ShapeOutside)],
                            [ShapeOutside, SetShapeOutside,
                             PropertyId::Longhand(LonghandId::ShapeOutside)],
                                [Touch_action, SetTouch_action,
                                 PropertyId::Longhand(LonghandId::TouchAction)],
                            [TouchAction, SetTouchAction,
                             PropertyId::Longhand(LonghandId::TouchAction)],
                            [Color, SetColor,
                             PropertyId::Longhand(LonghandId::Color)],
                                [Column_width, SetColumn_width,
                                 PropertyId::Longhand(LonghandId::ColumnWidth)],
                            [ColumnWidth, SetColumnWidth,
                             PropertyId::Longhand(LonghandId::ColumnWidth)],
                                [_moz_column_width, Set_moz_column_width,
                                 PropertyId::Longhand(LonghandId::ColumnWidth)],
                            [MozColumnWidth, SetMozColumnWidth,
                             PropertyId::Longhand(LonghandId::ColumnWidth)],
                                [Column_count, SetColumn_count,
                                 PropertyId::Longhand(LonghandId::ColumnCount)],
                            [ColumnCount, SetColumnCount,
                             PropertyId::Longhand(LonghandId::ColumnCount)],
                                [_moz_column_count, Set_moz_column_count,
                                 PropertyId::Longhand(LonghandId::ColumnCount)],
                            [MozColumnCount, SetMozColumnCount,
                             PropertyId::Longhand(LonghandId::ColumnCount)],
                                [Column_gap, SetColumn_gap,
                                 PropertyId::Longhand(LonghandId::ColumnGap)],
                            [ColumnGap, SetColumnGap,
                             PropertyId::Longhand(LonghandId::ColumnGap)],
                                [_moz_column_gap, Set_moz_column_gap,
                                 PropertyId::Longhand(LonghandId::ColumnGap)],
                            [MozColumnGap, SetMozColumnGap,
                             PropertyId::Longhand(LonghandId::ColumnGap)],
                                [Column_fill, SetColumn_fill,
                                 PropertyId::Longhand(LonghandId::ColumnFill)],
                            [ColumnFill, SetColumnFill,
                             PropertyId::Longhand(LonghandId::ColumnFill)],
                                [_moz_column_fill, Set_moz_column_fill,
                                 PropertyId::Longhand(LonghandId::ColumnFill)],
                            [MozColumnFill, SetMozColumnFill,
                             PropertyId::Longhand(LonghandId::ColumnFill)],
                                [Column_rule_width, SetColumn_rule_width,
                                 PropertyId::Longhand(LonghandId::ColumnRuleWidth)],
                            [ColumnRuleWidth, SetColumnRuleWidth,
                             PropertyId::Longhand(LonghandId::ColumnRuleWidth)],
                                [_moz_column_rule_width, Set_moz_column_rule_width,
                                 PropertyId::Longhand(LonghandId::ColumnRuleWidth)],
                            [MozColumnRuleWidth, SetMozColumnRuleWidth,
                             PropertyId::Longhand(LonghandId::ColumnRuleWidth)],
                                [Column_rule_color, SetColumn_rule_color,
                                 PropertyId::Longhand(LonghandId::ColumnRuleColor)],
                            [ColumnRuleColor, SetColumnRuleColor,
                             PropertyId::Longhand(LonghandId::ColumnRuleColor)],
                                [_moz_column_rule_color, Set_moz_column_rule_color,
                                 PropertyId::Longhand(LonghandId::ColumnRuleColor)],
                            [MozColumnRuleColor, SetMozColumnRuleColor,
                             PropertyId::Longhand(LonghandId::ColumnRuleColor)],
                                [Column_span, SetColumn_span,
                                 PropertyId::Longhand(LonghandId::ColumnSpan)],
                            [ColumnSpan, SetColumnSpan,
                             PropertyId::Longhand(LonghandId::ColumnSpan)],
                                [Column_rule_style, SetColumn_rule_style,
                                 PropertyId::Longhand(LonghandId::ColumnRuleStyle)],
                            [ColumnRuleStyle, SetColumnRuleStyle,
                             PropertyId::Longhand(LonghandId::ColumnRuleStyle)],
                                [_moz_column_rule_style, Set_moz_column_rule_style,
                                 PropertyId::Longhand(LonghandId::ColumnRuleStyle)],
                            [MozColumnRuleStyle, SetMozColumnRuleStyle,
                             PropertyId::Longhand(LonghandId::ColumnRuleStyle)],
                            [Content, SetContent,
                             PropertyId::Longhand(LonghandId::Content)],
                                [Counter_increment, SetCounter_increment,
                                 PropertyId::Longhand(LonghandId::CounterIncrement)],
                            [CounterIncrement, SetCounterIncrement,
                             PropertyId::Longhand(LonghandId::CounterIncrement)],
                                [Counter_reset, SetCounter_reset,
                                 PropertyId::Longhand(LonghandId::CounterReset)],
                            [CounterReset, SetCounterReset,
                             PropertyId::Longhand(LonghandId::CounterReset)],
                            [Opacity, SetOpacity,
                             PropertyId::Longhand(LonghandId::Opacity)],
                                [Box_shadow, SetBox_shadow,
                                 PropertyId::Longhand(LonghandId::BoxShadow)],
                            [BoxShadow, SetBoxShadow,
                             PropertyId::Longhand(LonghandId::BoxShadow)],
                                [_webkit_box_shadow, Set_webkit_box_shadow,
                                 PropertyId::Longhand(LonghandId::BoxShadow)],
                            [WebkitBoxShadow, SetWebkitBoxShadow,
                             PropertyId::Longhand(LonghandId::BoxShadow)],
                            [Clip, SetClip,
                             PropertyId::Longhand(LonghandId::Clip)],
                            [Filter, SetFilter,
                             PropertyId::Longhand(LonghandId::Filter)],
                                [_webkit_filter, Set_webkit_filter,
                                 PropertyId::Longhand(LonghandId::Filter)],
                            [WebkitFilter, SetWebkitFilter,
                             PropertyId::Longhand(LonghandId::Filter)],
                                [Mix_blend_mode, SetMix_blend_mode,
                                 PropertyId::Longhand(LonghandId::MixBlendMode)],
                            [MixBlendMode, SetMixBlendMode,
                             PropertyId::Longhand(LonghandId::MixBlendMode)],
                                [Font_family, SetFont_family,
                                 PropertyId::Longhand(LonghandId::FontFamily)],
                            [FontFamily, SetFontFamily,
                             PropertyId::Longhand(LonghandId::FontFamily)],
                                [Font_style, SetFont_style,
                                 PropertyId::Longhand(LonghandId::FontStyle)],
                            [FontStyle, SetFontStyle,
                             PropertyId::Longhand(LonghandId::FontStyle)],
                                [Font_variant_caps, SetFont_variant_caps,
                                 PropertyId::Longhand(LonghandId::FontVariantCaps)],
                            [FontVariantCaps, SetFontVariantCaps,
                             PropertyId::Longhand(LonghandId::FontVariantCaps)],
                                [Font_weight, SetFont_weight,
                                 PropertyId::Longhand(LonghandId::FontWeight)],
                            [FontWeight, SetFontWeight,
                             PropertyId::Longhand(LonghandId::FontWeight)],
                                [Font_size, SetFont_size,
                                 PropertyId::Longhand(LonghandId::FontSize)],
                            [FontSize, SetFontSize,
                             PropertyId::Longhand(LonghandId::FontSize)],
                                [Font_size_adjust, SetFont_size_adjust,
                                 PropertyId::Longhand(LonghandId::FontSizeAdjust)],
                            [FontSizeAdjust, SetFontSizeAdjust,
                             PropertyId::Longhand(LonghandId::FontSizeAdjust)],
                                [Font_synthesis, SetFont_synthesis,
                                 PropertyId::Longhand(LonghandId::FontSynthesis)],
                            [FontSynthesis, SetFontSynthesis,
                             PropertyId::Longhand(LonghandId::FontSynthesis)],
                                [Font_stretch, SetFont_stretch,
                                 PropertyId::Longhand(LonghandId::FontStretch)],
                            [FontStretch, SetFontStretch,
                             PropertyId::Longhand(LonghandId::FontStretch)],
                                [Font_kerning, SetFont_kerning,
                                 PropertyId::Longhand(LonghandId::FontKerning)],
                            [FontKerning, SetFontKerning,
                             PropertyId::Longhand(LonghandId::FontKerning)],
                                [Font_variant_alternates, SetFont_variant_alternates,
                                 PropertyId::Longhand(LonghandId::FontVariantAlternates)],
                            [FontVariantAlternates, SetFontVariantAlternates,
                             PropertyId::Longhand(LonghandId::FontVariantAlternates)],
                                [Font_variant_east_asian, SetFont_variant_east_asian,
                                 PropertyId::Longhand(LonghandId::FontVariantEastAsian)],
                            [FontVariantEastAsian, SetFontVariantEastAsian,
                             PropertyId::Longhand(LonghandId::FontVariantEastAsian)],
                                [Font_variant_ligatures, SetFont_variant_ligatures,
                                 PropertyId::Longhand(LonghandId::FontVariantLigatures)],
                            [FontVariantLigatures, SetFontVariantLigatures,
                             PropertyId::Longhand(LonghandId::FontVariantLigatures)],
                                [Font_variant_numeric, SetFont_variant_numeric,
                                 PropertyId::Longhand(LonghandId::FontVariantNumeric)],
                            [FontVariantNumeric, SetFontVariantNumeric,
                             PropertyId::Longhand(LonghandId::FontVariantNumeric)],
                                [Font_variant_position, SetFont_variant_position,
                                 PropertyId::Longhand(LonghandId::FontVariantPosition)],
                            [FontVariantPosition, SetFontVariantPosition,
                             PropertyId::Longhand(LonghandId::FontVariantPosition)],
                                [Font_feature_settings, SetFont_feature_settings,
                                 PropertyId::Longhand(LonghandId::FontFeatureSettings)],
                            [FontFeatureSettings, SetFontFeatureSettings,
                             PropertyId::Longhand(LonghandId::FontFeatureSettings)],
                                [_moz_font_feature_settings, Set_moz_font_feature_settings,
                                 PropertyId::Longhand(LonghandId::FontFeatureSettings)],
                            [MozFontFeatureSettings, SetMozFontFeatureSettings,
                             PropertyId::Longhand(LonghandId::FontFeatureSettings)],
                                [Font_variation_settings, SetFont_variation_settings,
                                 PropertyId::Longhand(LonghandId::FontVariationSettings)],
                            [FontVariationSettings, SetFontVariationSettings,
                             PropertyId::Longhand(LonghandId::FontVariationSettings)],
                                [Font_language_override, SetFont_language_override,
                                 PropertyId::Longhand(LonghandId::FontLanguageOverride)],
                            [FontLanguageOverride, SetFontLanguageOverride,
                             PropertyId::Longhand(LonghandId::FontLanguageOverride)],
                                [_moz_font_language_override, Set_moz_font_language_override,
                                 PropertyId::Longhand(LonghandId::FontLanguageOverride)],
                            [MozFontLanguageOverride, SetMozFontLanguageOverride,
                             PropertyId::Longhand(LonghandId::FontLanguageOverride)],
                                [_moz_math_display, Set_moz_math_display,
                                 PropertyId::Longhand(LonghandId::MozMathDisplay)],
                            [MozMathDisplay, SetMozMathDisplay,
                             PropertyId::Longhand(LonghandId::MozMathDisplay)],
                                [_moz_math_variant, Set_moz_math_variant,
                                 PropertyId::Longhand(LonghandId::MozMathVariant)],
                            [MozMathVariant, SetMozMathVariant,
                             PropertyId::Longhand(LonghandId::MozMathVariant)],
                                [_moz_osx_font_smoothing, Set_moz_osx_font_smoothing,
                                 PropertyId::Longhand(LonghandId::MozOsxFontSmoothing)],
                            [MozOsxFontSmoothing, SetMozOsxFontSmoothing,
                             PropertyId::Longhand(LonghandId::MozOsxFontSmoothing)],
                            [Visibility, SetVisibility,
                             PropertyId::Longhand(LonghandId::Visibility)],
                                [Writing_mode, SetWriting_mode,
                                 PropertyId::Longhand(LonghandId::WritingMode)],
                            [WritingMode, SetWritingMode,
                             PropertyId::Longhand(LonghandId::WritingMode)],
                            [Direction, SetDirection,
                             PropertyId::Longhand(LonghandId::Direction)],
                                [Text_orientation, SetText_orientation,
                                 PropertyId::Longhand(LonghandId::TextOrientation)],
                            [TextOrientation, SetTextOrientation,
                             PropertyId::Longhand(LonghandId::TextOrientation)],
                                [Color_adjust, SetColor_adjust,
                                 PropertyId::Longhand(LonghandId::ColorAdjust)],
                            [ColorAdjust, SetColorAdjust,
                             PropertyId::Longhand(LonghandId::ColorAdjust)],
                                [Image_rendering, SetImage_rendering,
                                 PropertyId::Longhand(LonghandId::ImageRendering)],
                            [ImageRendering, SetImageRendering,
                             PropertyId::Longhand(LonghandId::ImageRendering)],
                                [Image_orientation, SetImage_orientation,
                                 PropertyId::Longhand(LonghandId::ImageOrientation)],
                            [ImageOrientation, SetImageOrientation,
                             PropertyId::Longhand(LonghandId::ImageOrientation)],
                                [Border_collapse, SetBorder_collapse,
                                 PropertyId::Longhand(LonghandId::BorderCollapse)],
                            [BorderCollapse, SetBorderCollapse,
                             PropertyId::Longhand(LonghandId::BorderCollapse)],
                                [Empty_cells, SetEmpty_cells,
                                 PropertyId::Longhand(LonghandId::EmptyCells)],
                            [EmptyCells, SetEmptyCells,
                             PropertyId::Longhand(LonghandId::EmptyCells)],
                                [Caption_side, SetCaption_side,
                                 PropertyId::Longhand(LonghandId::CaptionSide)],
                            [CaptionSide, SetCaptionSide,
                             PropertyId::Longhand(LonghandId::CaptionSide)],
                                [Border_spacing, SetBorder_spacing,
                                 PropertyId::Longhand(LonghandId::BorderSpacing)],
                            [BorderSpacing, SetBorderSpacing,
                             PropertyId::Longhand(LonghandId::BorderSpacing)],
                                [Line_height, SetLine_height,
                                 PropertyId::Longhand(LonghandId::LineHeight)],
                            [LineHeight, SetLineHeight,
                             PropertyId::Longhand(LonghandId::LineHeight)],
                                [Text_transform, SetText_transform,
                                 PropertyId::Longhand(LonghandId::TextTransform)],
                            [TextTransform, SetTextTransform,
                             PropertyId::Longhand(LonghandId::TextTransform)],
                            [Hyphens, SetHyphens,
                             PropertyId::Longhand(LonghandId::Hyphens)],
                                [_moz_hyphens, Set_moz_hyphens,
                                 PropertyId::Longhand(LonghandId::Hyphens)],
                            [MozHyphens, SetMozHyphens,
                             PropertyId::Longhand(LonghandId::Hyphens)],
                                [_moz_text_size_adjust, Set_moz_text_size_adjust,
                                 PropertyId::Longhand(LonghandId::MozTextSizeAdjust)],
                            [MozTextSizeAdjust, SetMozTextSizeAdjust,
                             PropertyId::Longhand(LonghandId::MozTextSizeAdjust)],
                                [_webkit_text_size_adjust, Set_webkit_text_size_adjust,
                                 PropertyId::Longhand(LonghandId::MozTextSizeAdjust)],
                            [WebkitTextSizeAdjust, SetWebkitTextSizeAdjust,
                             PropertyId::Longhand(LonghandId::MozTextSizeAdjust)],
                                [Text_indent, SetText_indent,
                                 PropertyId::Longhand(LonghandId::TextIndent)],
                            [TextIndent, SetTextIndent,
                             PropertyId::Longhand(LonghandId::TextIndent)],
                                [Overflow_wrap, SetOverflow_wrap,
                                 PropertyId::Longhand(LonghandId::OverflowWrap)],
                            [OverflowWrap, SetOverflowWrap,
                             PropertyId::Longhand(LonghandId::OverflowWrap)],
                                [Word_wrap, SetWord_wrap,
                                 PropertyId::Longhand(LonghandId::OverflowWrap)],
                            [WordWrap, SetWordWrap,
                             PropertyId::Longhand(LonghandId::OverflowWrap)],
                                [Word_break, SetWord_break,
                                 PropertyId::Longhand(LonghandId::WordBreak)],
                            [WordBreak, SetWordBreak,
                             PropertyId::Longhand(LonghandId::WordBreak)],
                                [Text_justify, SetText_justify,
                                 PropertyId::Longhand(LonghandId::TextJustify)],
                            [TextJustify, SetTextJustify,
                             PropertyId::Longhand(LonghandId::TextJustify)],
                                [Text_align_last, SetText_align_last,
                                 PropertyId::Longhand(LonghandId::TextAlignLast)],
                            [TextAlignLast, SetTextAlignLast,
                             PropertyId::Longhand(LonghandId::TextAlignLast)],
                                [Text_align, SetText_align,
                                 PropertyId::Longhand(LonghandId::TextAlign)],
                            [TextAlign, SetTextAlign,
                             PropertyId::Longhand(LonghandId::TextAlign)],
                                [Letter_spacing, SetLetter_spacing,
                                 PropertyId::Longhand(LonghandId::LetterSpacing)],
                            [LetterSpacing, SetLetterSpacing,
                             PropertyId::Longhand(LonghandId::LetterSpacing)],
                                [Word_spacing, SetWord_spacing,
                                 PropertyId::Longhand(LonghandId::WordSpacing)],
                            [WordSpacing, SetWordSpacing,
                             PropertyId::Longhand(LonghandId::WordSpacing)],
                                [White_space, SetWhite_space,
                                 PropertyId::Longhand(LonghandId::WhiteSpace)],
                            [WhiteSpace, SetWhiteSpace,
                             PropertyId::Longhand(LonghandId::WhiteSpace)],
                                [Text_shadow, SetText_shadow,
                                 PropertyId::Longhand(LonghandId::TextShadow)],
                            [TextShadow, SetTextShadow,
                             PropertyId::Longhand(LonghandId::TextShadow)],
                                [Text_emphasis_style, SetText_emphasis_style,
                                 PropertyId::Longhand(LonghandId::TextEmphasisStyle)],
                            [TextEmphasisStyle, SetTextEmphasisStyle,
                             PropertyId::Longhand(LonghandId::TextEmphasisStyle)],
                                [Text_emphasis_position, SetText_emphasis_position,
                                 PropertyId::Longhand(LonghandId::TextEmphasisPosition)],
                            [TextEmphasisPosition, SetTextEmphasisPosition,
                             PropertyId::Longhand(LonghandId::TextEmphasisPosition)],
                                [Text_emphasis_color, SetText_emphasis_color,
                                 PropertyId::Longhand(LonghandId::TextEmphasisColor)],
                            [TextEmphasisColor, SetTextEmphasisColor,
                             PropertyId::Longhand(LonghandId::TextEmphasisColor)],
                                [_moz_tab_size, Set_moz_tab_size,
                                 PropertyId::Longhand(LonghandId::MozTabSize)],
                            [MozTabSize, SetMozTabSize,
                             PropertyId::Longhand(LonghandId::MozTabSize)],
                                [_webkit_text_fill_color, Set_webkit_text_fill_color,
                                 PropertyId::Longhand(LonghandId::WebkitTextFillColor)],
                            [WebkitTextFillColor, SetWebkitTextFillColor,
                             PropertyId::Longhand(LonghandId::WebkitTextFillColor)],
                                [_webkit_text_stroke_color, Set_webkit_text_stroke_color,
                                 PropertyId::Longhand(LonghandId::WebkitTextStrokeColor)],
                            [WebkitTextStrokeColor, SetWebkitTextStrokeColor,
                             PropertyId::Longhand(LonghandId::WebkitTextStrokeColor)],
                                [_webkit_text_stroke_width, Set_webkit_text_stroke_width,
                                 PropertyId::Longhand(LonghandId::WebkitTextStrokeWidth)],
                            [WebkitTextStrokeWidth, SetWebkitTextStrokeWidth,
                             PropertyId::Longhand(LonghandId::WebkitTextStrokeWidth)],
                                [Ruby_align, SetRuby_align,
                                 PropertyId::Longhand(LonghandId::RubyAlign)],
                            [RubyAlign, SetRubyAlign,
                             PropertyId::Longhand(LonghandId::RubyAlign)],
                                [Ruby_position, SetRuby_position,
                                 PropertyId::Longhand(LonghandId::RubyPosition)],
                            [RubyPosition, SetRubyPosition,
                             PropertyId::Longhand(LonghandId::RubyPosition)],
                                [Text_combine_upright, SetText_combine_upright,
                                 PropertyId::Longhand(LonghandId::TextCombineUpright)],
                            [TextCombineUpright, SetTextCombineUpright,
                             PropertyId::Longhand(LonghandId::TextCombineUpright)],
                                [Text_rendering, SetText_rendering,
                                 PropertyId::Longhand(LonghandId::TextRendering)],
                            [TextRendering, SetTextRendering,
                             PropertyId::Longhand(LonghandId::TextRendering)],
                                [_moz_control_character_visibility, Set_moz_control_character_visibility,
                                 PropertyId::Longhand(LonghandId::MozControlCharacterVisibility)],
                            [MozControlCharacterVisibility, SetMozControlCharacterVisibility,
                             PropertyId::Longhand(LonghandId::MozControlCharacterVisibility)],
                                [List_style_position, SetList_style_position,
                                 PropertyId::Longhand(LonghandId::ListStylePosition)],
                            [ListStylePosition, SetListStylePosition,
                             PropertyId::Longhand(LonghandId::ListStylePosition)],
                                [List_style_type, SetList_style_type,
                                 PropertyId::Longhand(LonghandId::ListStyleType)],
                            [ListStyleType, SetListStyleType,
                             PropertyId::Longhand(LonghandId::ListStyleType)],
                                [List_style_image, SetList_style_image,
                                 PropertyId::Longhand(LonghandId::ListStyleImage)],
                            [ListStyleImage, SetListStyleImage,
                             PropertyId::Longhand(LonghandId::ListStyleImage)],
                            [Quotes, SetQuotes,
                             PropertyId::Longhand(LonghandId::Quotes)],
                                [_moz_image_region, Set_moz_image_region,
                                 PropertyId::Longhand(LonghandId::MozImageRegion)],
                            [MozImageRegion, SetMozImageRegion,
                             PropertyId::Longhand(LonghandId::MozImageRegion)],
                                [Margin_top, SetMargin_top,
                                 PropertyId::Longhand(LonghandId::MarginTop)],
                            [MarginTop, SetMarginTop,
                             PropertyId::Longhand(LonghandId::MarginTop)],
                                [Margin_left, SetMargin_left,
                                 PropertyId::Longhand(LonghandId::MarginLeft)],
                            [MarginLeft, SetMarginLeft,
                             PropertyId::Longhand(LonghandId::MarginLeft)],
                                [Margin_bottom, SetMargin_bottom,
                                 PropertyId::Longhand(LonghandId::MarginBottom)],
                            [MarginBottom, SetMarginBottom,
                             PropertyId::Longhand(LonghandId::MarginBottom)],
                                [Margin_right, SetMargin_right,
                                 PropertyId::Longhand(LonghandId::MarginRight)],
                            [MarginRight, SetMarginRight,
                             PropertyId::Longhand(LonghandId::MarginRight)],
                                [Margin_block_start, SetMargin_block_start,
                                 PropertyId::Longhand(LonghandId::MarginBlockStart)],
                            [MarginBlockStart, SetMarginBlockStart,
                             PropertyId::Longhand(LonghandId::MarginBlockStart)],
                                [Margin_block_end, SetMargin_block_end,
                                 PropertyId::Longhand(LonghandId::MarginBlockEnd)],
                            [MarginBlockEnd, SetMarginBlockEnd,
                             PropertyId::Longhand(LonghandId::MarginBlockEnd)],
                                [Margin_inline_start, SetMargin_inline_start,
                                 PropertyId::Longhand(LonghandId::MarginInlineStart)],
                            [MarginInlineStart, SetMarginInlineStart,
                             PropertyId::Longhand(LonghandId::MarginInlineStart)],
                                [_moz_margin_start, Set_moz_margin_start,
                                 PropertyId::Longhand(LonghandId::MarginInlineStart)],
                            [MozMarginStart, SetMozMarginStart,
                             PropertyId::Longhand(LonghandId::MarginInlineStart)],
                                [Margin_inline_end, SetMargin_inline_end,
                                 PropertyId::Longhand(LonghandId::MarginInlineEnd)],
                            [MarginInlineEnd, SetMarginInlineEnd,
                             PropertyId::Longhand(LonghandId::MarginInlineEnd)],
                                [_moz_margin_end, Set_moz_margin_end,
                                 PropertyId::Longhand(LonghandId::MarginInlineEnd)],
                            [MozMarginEnd, SetMozMarginEnd,
                             PropertyId::Longhand(LonghandId::MarginInlineEnd)],
                                [Outline_color, SetOutline_color,
                                 PropertyId::Longhand(LonghandId::OutlineColor)],
                            [OutlineColor, SetOutlineColor,
                             PropertyId::Longhand(LonghandId::OutlineColor)],
                                [Outline_style, SetOutline_style,
                                 PropertyId::Longhand(LonghandId::OutlineStyle)],
                            [OutlineStyle, SetOutlineStyle,
                             PropertyId::Longhand(LonghandId::OutlineStyle)],
                                [Outline_width, SetOutline_width,
                                 PropertyId::Longhand(LonghandId::OutlineWidth)],
                            [OutlineWidth, SetOutlineWidth,
                             PropertyId::Longhand(LonghandId::OutlineWidth)],
                                [_moz_outline_radius_topleft, Set_moz_outline_radius_topleft,
                                 PropertyId::Longhand(LonghandId::MozOutlineRadiusTopleft)],
                            [MozOutlineRadiusTopleft, SetMozOutlineRadiusTopleft,
                             PropertyId::Longhand(LonghandId::MozOutlineRadiusTopleft)],
                                [_moz_outline_radius_topright, Set_moz_outline_radius_topright,
                                 PropertyId::Longhand(LonghandId::MozOutlineRadiusTopright)],
                            [MozOutlineRadiusTopright, SetMozOutlineRadiusTopright,
                             PropertyId::Longhand(LonghandId::MozOutlineRadiusTopright)],
                                [_moz_outline_radius_bottomright, Set_moz_outline_radius_bottomright,
                                 PropertyId::Longhand(LonghandId::MozOutlineRadiusBottomright)],
                            [MozOutlineRadiusBottomright, SetMozOutlineRadiusBottomright,
                             PropertyId::Longhand(LonghandId::MozOutlineRadiusBottomright)],
                                [_moz_outline_radius_bottomleft, Set_moz_outline_radius_bottomleft,
                                 PropertyId::Longhand(LonghandId::MozOutlineRadiusBottomleft)],
                            [MozOutlineRadiusBottomleft, SetMozOutlineRadiusBottomleft,
                             PropertyId::Longhand(LonghandId::MozOutlineRadiusBottomleft)],
                                [Outline_offset, SetOutline_offset,
                                 PropertyId::Longhand(LonghandId::OutlineOffset)],
                            [OutlineOffset, SetOutlineOffset,
                             PropertyId::Longhand(LonghandId::OutlineOffset)],
                                [Padding_top, SetPadding_top,
                                 PropertyId::Longhand(LonghandId::PaddingTop)],
                            [PaddingTop, SetPaddingTop,
                             PropertyId::Longhand(LonghandId::PaddingTop)],
                                [Padding_left, SetPadding_left,
                                 PropertyId::Longhand(LonghandId::PaddingLeft)],
                            [PaddingLeft, SetPaddingLeft,
                             PropertyId::Longhand(LonghandId::PaddingLeft)],
                                [Padding_bottom, SetPadding_bottom,
                                 PropertyId::Longhand(LonghandId::PaddingBottom)],
                            [PaddingBottom, SetPaddingBottom,
                             PropertyId::Longhand(LonghandId::PaddingBottom)],
                                [Padding_right, SetPadding_right,
                                 PropertyId::Longhand(LonghandId::PaddingRight)],
                            [PaddingRight, SetPaddingRight,
                             PropertyId::Longhand(LonghandId::PaddingRight)],
                                [Padding_block_start, SetPadding_block_start,
                                 PropertyId::Longhand(LonghandId::PaddingBlockStart)],
                            [PaddingBlockStart, SetPaddingBlockStart,
                             PropertyId::Longhand(LonghandId::PaddingBlockStart)],
                                [Padding_block_end, SetPadding_block_end,
                                 PropertyId::Longhand(LonghandId::PaddingBlockEnd)],
                            [PaddingBlockEnd, SetPaddingBlockEnd,
                             PropertyId::Longhand(LonghandId::PaddingBlockEnd)],
                                [Padding_inline_start, SetPadding_inline_start,
                                 PropertyId::Longhand(LonghandId::PaddingInlineStart)],
                            [PaddingInlineStart, SetPaddingInlineStart,
                             PropertyId::Longhand(LonghandId::PaddingInlineStart)],
                                [_moz_padding_start, Set_moz_padding_start,
                                 PropertyId::Longhand(LonghandId::PaddingInlineStart)],
                            [MozPaddingStart, SetMozPaddingStart,
                             PropertyId::Longhand(LonghandId::PaddingInlineStart)],
                                [Padding_inline_end, SetPadding_inline_end,
                                 PropertyId::Longhand(LonghandId::PaddingInlineEnd)],
                            [PaddingInlineEnd, SetPaddingInlineEnd,
                             PropertyId::Longhand(LonghandId::PaddingInlineEnd)],
                                [_moz_padding_end, Set_moz_padding_end,
                                 PropertyId::Longhand(LonghandId::PaddingInlineEnd)],
                            [MozPaddingEnd, SetMozPaddingEnd,
                             PropertyId::Longhand(LonghandId::PaddingInlineEnd)],
                            [Cursor, SetCursor,
                             PropertyId::Longhand(LonghandId::Cursor)],
                                [Pointer_events, SetPointer_events,
                                 PropertyId::Longhand(LonghandId::PointerEvents)],
                            [PointerEvents, SetPointerEvents,
                             PropertyId::Longhand(LonghandId::PointerEvents)],
                                [_moz_user_input, Set_moz_user_input,
                                 PropertyId::Longhand(LonghandId::MozUserInput)],
                            [MozUserInput, SetMozUserInput,
                             PropertyId::Longhand(LonghandId::MozUserInput)],
                                [_moz_user_modify, Set_moz_user_modify,
                                 PropertyId::Longhand(LonghandId::MozUserModify)],
                            [MozUserModify, SetMozUserModify,
                             PropertyId::Longhand(LonghandId::MozUserModify)],
                                [_moz_user_focus, Set_moz_user_focus,
                                 PropertyId::Longhand(LonghandId::MozUserFocus)],
                            [MozUserFocus, SetMozUserFocus,
                             PropertyId::Longhand(LonghandId::MozUserFocus)],
                                [Caret_color, SetCaret_color,
                                 PropertyId::Longhand(LonghandId::CaretColor)],
                            [CaretColor, SetCaretColor,
                             PropertyId::Longhand(LonghandId::CaretColor)],
                            [Top, SetTop,
                             PropertyId::Longhand(LonghandId::Top)],
                            [Left, SetLeft,
                             PropertyId::Longhand(LonghandId::Left)],
                            [Bottom, SetBottom,
                             PropertyId::Longhand(LonghandId::Bottom)],
                            [Right, SetRight,
                             PropertyId::Longhand(LonghandId::Right)],
                                [Offset_block_start, SetOffset_block_start,
                                 PropertyId::Longhand(LonghandId::OffsetBlockStart)],
                            [OffsetBlockStart, SetOffsetBlockStart,
                             PropertyId::Longhand(LonghandId::OffsetBlockStart)],
                                [Offset_block_end, SetOffset_block_end,
                                 PropertyId::Longhand(LonghandId::OffsetBlockEnd)],
                            [OffsetBlockEnd, SetOffsetBlockEnd,
                             PropertyId::Longhand(LonghandId::OffsetBlockEnd)],
                                [Offset_inline_start, SetOffset_inline_start,
                                 PropertyId::Longhand(LonghandId::OffsetInlineStart)],
                            [OffsetInlineStart, SetOffsetInlineStart,
                             PropertyId::Longhand(LonghandId::OffsetInlineStart)],
                                [Offset_inline_end, SetOffset_inline_end,
                                 PropertyId::Longhand(LonghandId::OffsetInlineEnd)],
                            [OffsetInlineEnd, SetOffsetInlineEnd,
                             PropertyId::Longhand(LonghandId::OffsetInlineEnd)],
                                [Z_index, SetZ_index,
                                 PropertyId::Longhand(LonghandId::ZIndex)],
                            [ZIndex, SetZIndex,
                             PropertyId::Longhand(LonghandId::ZIndex)],
                                [Flex_direction, SetFlex_direction,
                                 PropertyId::Longhand(LonghandId::FlexDirection)],
                            [FlexDirection, SetFlexDirection,
                             PropertyId::Longhand(LonghandId::FlexDirection)],
                                [_webkit_flex_direction, Set_webkit_flex_direction,
                                 PropertyId::Longhand(LonghandId::FlexDirection)],
                            [WebkitFlexDirection, SetWebkitFlexDirection,
                             PropertyId::Longhand(LonghandId::FlexDirection)],
                                [Flex_wrap, SetFlex_wrap,
                                 PropertyId::Longhand(LonghandId::FlexWrap)],
                            [FlexWrap, SetFlexWrap,
                             PropertyId::Longhand(LonghandId::FlexWrap)],
                                [_webkit_flex_wrap, Set_webkit_flex_wrap,
                                 PropertyId::Longhand(LonghandId::FlexWrap)],
                            [WebkitFlexWrap, SetWebkitFlexWrap,
                             PropertyId::Longhand(LonghandId::FlexWrap)],
                                [Justify_content, SetJustify_content,
                                 PropertyId::Longhand(LonghandId::JustifyContent)],
                            [JustifyContent, SetJustifyContent,
                             PropertyId::Longhand(LonghandId::JustifyContent)],
                                [_webkit_justify_content, Set_webkit_justify_content,
                                 PropertyId::Longhand(LonghandId::JustifyContent)],
                            [WebkitJustifyContent, SetWebkitJustifyContent,
                             PropertyId::Longhand(LonghandId::JustifyContent)],
                                [Align_content, SetAlign_content,
                                 PropertyId::Longhand(LonghandId::AlignContent)],
                            [AlignContent, SetAlignContent,
                             PropertyId::Longhand(LonghandId::AlignContent)],
                                [_webkit_align_content, Set_webkit_align_content,
                                 PropertyId::Longhand(LonghandId::AlignContent)],
                            [WebkitAlignContent, SetWebkitAlignContent,
                             PropertyId::Longhand(LonghandId::AlignContent)],
                                [Align_items, SetAlign_items,
                                 PropertyId::Longhand(LonghandId::AlignItems)],
                            [AlignItems, SetAlignItems,
                             PropertyId::Longhand(LonghandId::AlignItems)],
                                [_webkit_align_items, Set_webkit_align_items,
                                 PropertyId::Longhand(LonghandId::AlignItems)],
                            [WebkitAlignItems, SetWebkitAlignItems,
                             PropertyId::Longhand(LonghandId::AlignItems)],
                                [Justify_items, SetJustify_items,
                                 PropertyId::Longhand(LonghandId::JustifyItems)],
                            [JustifyItems, SetJustifyItems,
                             PropertyId::Longhand(LonghandId::JustifyItems)],
                                [Flex_grow, SetFlex_grow,
                                 PropertyId::Longhand(LonghandId::FlexGrow)],
                            [FlexGrow, SetFlexGrow,
                             PropertyId::Longhand(LonghandId::FlexGrow)],
                                [_webkit_flex_grow, Set_webkit_flex_grow,
                                 PropertyId::Longhand(LonghandId::FlexGrow)],
                            [WebkitFlexGrow, SetWebkitFlexGrow,
                             PropertyId::Longhand(LonghandId::FlexGrow)],
                                [Flex_shrink, SetFlex_shrink,
                                 PropertyId::Longhand(LonghandId::FlexShrink)],
                            [FlexShrink, SetFlexShrink,
                             PropertyId::Longhand(LonghandId::FlexShrink)],
                                [_webkit_flex_shrink, Set_webkit_flex_shrink,
                                 PropertyId::Longhand(LonghandId::FlexShrink)],
                            [WebkitFlexShrink, SetWebkitFlexShrink,
                             PropertyId::Longhand(LonghandId::FlexShrink)],
                                [Align_self, SetAlign_self,
                                 PropertyId::Longhand(LonghandId::AlignSelf)],
                            [AlignSelf, SetAlignSelf,
                             PropertyId::Longhand(LonghandId::AlignSelf)],
                                [_webkit_align_self, Set_webkit_align_self,
                                 PropertyId::Longhand(LonghandId::AlignSelf)],
                            [WebkitAlignSelf, SetWebkitAlignSelf,
                             PropertyId::Longhand(LonghandId::AlignSelf)],
                                [Justify_self, SetJustify_self,
                                 PropertyId::Longhand(LonghandId::JustifySelf)],
                            [JustifySelf, SetJustifySelf,
                             PropertyId::Longhand(LonghandId::JustifySelf)],
                            [Order, SetOrder,
                             PropertyId::Longhand(LonghandId::Order)],
                                [_webkit_order, Set_webkit_order,
                                 PropertyId::Longhand(LonghandId::Order)],
                            [WebkitOrder, SetWebkitOrder,
                             PropertyId::Longhand(LonghandId::Order)],
                                [Flex_basis, SetFlex_basis,
                                 PropertyId::Longhand(LonghandId::FlexBasis)],
                            [FlexBasis, SetFlexBasis,
                             PropertyId::Longhand(LonghandId::FlexBasis)],
                                [_webkit_flex_basis, Set_webkit_flex_basis,
                                 PropertyId::Longhand(LonghandId::FlexBasis)],
                            [WebkitFlexBasis, SetWebkitFlexBasis,
                             PropertyId::Longhand(LonghandId::FlexBasis)],
                            [Width, SetWidth,
                             PropertyId::Longhand(LonghandId::Width)],
                                [Min_width, SetMin_width,
                                 PropertyId::Longhand(LonghandId::MinWidth)],
                            [MinWidth, SetMinWidth,
                             PropertyId::Longhand(LonghandId::MinWidth)],
                                [Max_width, SetMax_width,
                                 PropertyId::Longhand(LonghandId::MaxWidth)],
                            [MaxWidth, SetMaxWidth,
                             PropertyId::Longhand(LonghandId::MaxWidth)],
                            [Height, SetHeight,
                             PropertyId::Longhand(LonghandId::Height)],
                                [Min_height, SetMin_height,
                                 PropertyId::Longhand(LonghandId::MinHeight)],
                            [MinHeight, SetMinHeight,
                             PropertyId::Longhand(LonghandId::MinHeight)],
                                [Max_height, SetMax_height,
                                 PropertyId::Longhand(LonghandId::MaxHeight)],
                            [MaxHeight, SetMaxHeight,
                             PropertyId::Longhand(LonghandId::MaxHeight)],
                                [Block_size, SetBlock_size,
                                 PropertyId::Longhand(LonghandId::BlockSize)],
                            [BlockSize, SetBlockSize,
                             PropertyId::Longhand(LonghandId::BlockSize)],
                                [Min_block_size, SetMin_block_size,
                                 PropertyId::Longhand(LonghandId::MinBlockSize)],
                            [MinBlockSize, SetMinBlockSize,
                             PropertyId::Longhand(LonghandId::MinBlockSize)],
                                [Max_block_size, SetMax_block_size,
                                 PropertyId::Longhand(LonghandId::MaxBlockSize)],
                            [MaxBlockSize, SetMaxBlockSize,
                             PropertyId::Longhand(LonghandId::MaxBlockSize)],
                                [Inline_size, SetInline_size,
                                 PropertyId::Longhand(LonghandId::InlineSize)],
                            [InlineSize, SetInlineSize,
                             PropertyId::Longhand(LonghandId::InlineSize)],
                                [Min_inline_size, SetMin_inline_size,
                                 PropertyId::Longhand(LonghandId::MinInlineSize)],
                            [MinInlineSize, SetMinInlineSize,
                             PropertyId::Longhand(LonghandId::MinInlineSize)],
                                [Max_inline_size, SetMax_inline_size,
                                 PropertyId::Longhand(LonghandId::MaxInlineSize)],
                            [MaxInlineSize, SetMaxInlineSize,
                             PropertyId::Longhand(LonghandId::MaxInlineSize)],
                                [Box_sizing, SetBox_sizing,
                                 PropertyId::Longhand(LonghandId::BoxSizing)],
                            [BoxSizing, SetBoxSizing,
                             PropertyId::Longhand(LonghandId::BoxSizing)],
                                [_moz_box_sizing, Set_moz_box_sizing,
                                 PropertyId::Longhand(LonghandId::BoxSizing)],
                            [MozBoxSizing, SetMozBoxSizing,
                             PropertyId::Longhand(LonghandId::BoxSizing)],
                                [_webkit_box_sizing, Set_webkit_box_sizing,
                                 PropertyId::Longhand(LonghandId::BoxSizing)],
                            [WebkitBoxSizing, SetWebkitBoxSizing,
                             PropertyId::Longhand(LonghandId::BoxSizing)],
                                [Object_fit, SetObject_fit,
                                 PropertyId::Longhand(LonghandId::ObjectFit)],
                            [ObjectFit, SetObjectFit,
                             PropertyId::Longhand(LonghandId::ObjectFit)],
                                [Object_position, SetObject_position,
                                 PropertyId::Longhand(LonghandId::ObjectPosition)],
                            [ObjectPosition, SetObjectPosition,
                             PropertyId::Longhand(LonghandId::ObjectPosition)],
                                [Grid_row_gap, SetGrid_row_gap,
                                 PropertyId::Longhand(LonghandId::GridRowGap)],
                            [GridRowGap, SetGridRowGap,
                             PropertyId::Longhand(LonghandId::GridRowGap)],
                                [Grid_row_start, SetGrid_row_start,
                                 PropertyId::Longhand(LonghandId::GridRowStart)],
                            [GridRowStart, SetGridRowStart,
                             PropertyId::Longhand(LonghandId::GridRowStart)],
                                [Grid_row_end, SetGrid_row_end,
                                 PropertyId::Longhand(LonghandId::GridRowEnd)],
                            [GridRowEnd, SetGridRowEnd,
                             PropertyId::Longhand(LonghandId::GridRowEnd)],
                                [Grid_auto_rows, SetGrid_auto_rows,
                                 PropertyId::Longhand(LonghandId::GridAutoRows)],
                            [GridAutoRows, SetGridAutoRows,
                             PropertyId::Longhand(LonghandId::GridAutoRows)],
                                [Grid_template_rows, SetGrid_template_rows,
                                 PropertyId::Longhand(LonghandId::GridTemplateRows)],
                            [GridTemplateRows, SetGridTemplateRows,
                             PropertyId::Longhand(LonghandId::GridTemplateRows)],
                                [Grid_column_gap, SetGrid_column_gap,
                                 PropertyId::Longhand(LonghandId::GridColumnGap)],
                            [GridColumnGap, SetGridColumnGap,
                             PropertyId::Longhand(LonghandId::GridColumnGap)],
                                [Grid_column_start, SetGrid_column_start,
                                 PropertyId::Longhand(LonghandId::GridColumnStart)],
                            [GridColumnStart, SetGridColumnStart,
                             PropertyId::Longhand(LonghandId::GridColumnStart)],
                                [Grid_column_end, SetGrid_column_end,
                                 PropertyId::Longhand(LonghandId::GridColumnEnd)],
                            [GridColumnEnd, SetGridColumnEnd,
                             PropertyId::Longhand(LonghandId::GridColumnEnd)],
                                [Grid_auto_columns, SetGrid_auto_columns,
                                 PropertyId::Longhand(LonghandId::GridAutoColumns)],
                            [GridAutoColumns, SetGridAutoColumns,
                             PropertyId::Longhand(LonghandId::GridAutoColumns)],
                                [Grid_template_columns, SetGrid_template_columns,
                                 PropertyId::Longhand(LonghandId::GridTemplateColumns)],
                            [GridTemplateColumns, SetGridTemplateColumns,
                             PropertyId::Longhand(LonghandId::GridTemplateColumns)],
                                [Grid_auto_flow, SetGrid_auto_flow,
                                 PropertyId::Longhand(LonghandId::GridAutoFlow)],
                            [GridAutoFlow, SetGridAutoFlow,
                             PropertyId::Longhand(LonghandId::GridAutoFlow)],
                                [Grid_template_areas, SetGrid_template_areas,
                                 PropertyId::Longhand(LonghandId::GridTemplateAreas)],
                            [GridTemplateAreas, SetGridTemplateAreas,
                             PropertyId::Longhand(LonghandId::GridTemplateAreas)],
                                [Table_layout, SetTable_layout,
                                 PropertyId::Longhand(LonghandId::TableLayout)],
                            [TableLayout, SetTableLayout,
                             PropertyId::Longhand(LonghandId::TableLayout)],
                                [Text_overflow, SetText_overflow,
                                 PropertyId::Longhand(LonghandId::TextOverflow)],
                            [TextOverflow, SetTextOverflow,
                             PropertyId::Longhand(LonghandId::TextOverflow)],
                                [Unicode_bidi, SetUnicode_bidi,
                                 PropertyId::Longhand(LonghandId::UnicodeBidi)],
                            [UnicodeBidi, SetUnicodeBidi,
                             PropertyId::Longhand(LonghandId::UnicodeBidi)],
                                [Text_decoration_line, SetText_decoration_line,
                                 PropertyId::Longhand(LonghandId::TextDecorationLine)],
                            [TextDecorationLine, SetTextDecorationLine,
                             PropertyId::Longhand(LonghandId::TextDecorationLine)],
                                [Text_decoration_style, SetText_decoration_style,
                                 PropertyId::Longhand(LonghandId::TextDecorationStyle)],
                            [TextDecorationStyle, SetTextDecorationStyle,
                             PropertyId::Longhand(LonghandId::TextDecorationStyle)],
                                [Text_decoration_color, SetText_decoration_color,
                                 PropertyId::Longhand(LonghandId::TextDecorationColor)],
                            [TextDecorationColor, SetTextDecorationColor,
                             PropertyId::Longhand(LonghandId::TextDecorationColor)],
                                [Initial_letter, SetInitial_letter,
                                 PropertyId::Longhand(LonghandId::InitialLetter)],
                            [InitialLetter, SetInitialLetter,
                             PropertyId::Longhand(LonghandId::InitialLetter)],
                                [Ime_mode, SetIme_mode,
                                 PropertyId::Longhand(LonghandId::ImeMode)],
                            [ImeMode, SetImeMode,
                             PropertyId::Longhand(LonghandId::ImeMode)],
                                [_moz_user_select, Set_moz_user_select,
                                 PropertyId::Longhand(LonghandId::MozUserSelect)],
                            [MozUserSelect, SetMozUserSelect,
                             PropertyId::Longhand(LonghandId::MozUserSelect)],
                                [_webkit_user_select, Set_webkit_user_select,
                                 PropertyId::Longhand(LonghandId::MozUserSelect)],
                            [WebkitUserSelect, SetWebkitUserSelect,
                             PropertyId::Longhand(LonghandId::MozUserSelect)],
                                [_moz_window_dragging, Set_moz_window_dragging,
                                 PropertyId::Longhand(LonghandId::MozWindowDragging)],
                            [MozWindowDragging, SetMozWindowDragging,
                             PropertyId::Longhand(LonghandId::MozWindowDragging)],
                                [_moz_force_broken_image_icon, Set_moz_force_broken_image_icon,
                                 PropertyId::Longhand(LonghandId::MozForceBrokenImageIcon)],
                            [MozForceBrokenImageIcon, SetMozForceBrokenImageIcon,
                             PropertyId::Longhand(LonghandId::MozForceBrokenImageIcon)],
                                [Text_anchor, SetText_anchor,
                                 PropertyId::Longhand(LonghandId::TextAnchor)],
                            [TextAnchor, SetTextAnchor,
                             PropertyId::Longhand(LonghandId::TextAnchor)],
                                [Color_interpolation, SetColor_interpolation,
                                 PropertyId::Longhand(LonghandId::ColorInterpolation)],
                            [ColorInterpolation, SetColorInterpolation,
                             PropertyId::Longhand(LonghandId::ColorInterpolation)],
                                [Color_interpolation_filters, SetColor_interpolation_filters,
                                 PropertyId::Longhand(LonghandId::ColorInterpolationFilters)],
                            [ColorInterpolationFilters, SetColorInterpolationFilters,
                             PropertyId::Longhand(LonghandId::ColorInterpolationFilters)],
                            [Fill, SetFill,
                             PropertyId::Longhand(LonghandId::Fill)],
                                [Fill_opacity, SetFill_opacity,
                                 PropertyId::Longhand(LonghandId::FillOpacity)],
                            [FillOpacity, SetFillOpacity,
                             PropertyId::Longhand(LonghandId::FillOpacity)],
                                [Fill_rule, SetFill_rule,
                                 PropertyId::Longhand(LonghandId::FillRule)],
                            [FillRule, SetFillRule,
                             PropertyId::Longhand(LonghandId::FillRule)],
                                [Shape_rendering, SetShape_rendering,
                                 PropertyId::Longhand(LonghandId::ShapeRendering)],
                            [ShapeRendering, SetShapeRendering,
                             PropertyId::Longhand(LonghandId::ShapeRendering)],
                            [Stroke, SetStroke,
                             PropertyId::Longhand(LonghandId::Stroke)],
                                [Stroke_width, SetStroke_width,
                                 PropertyId::Longhand(LonghandId::StrokeWidth)],
                            [StrokeWidth, SetStrokeWidth,
                             PropertyId::Longhand(LonghandId::StrokeWidth)],
                                [Stroke_linecap, SetStroke_linecap,
                                 PropertyId::Longhand(LonghandId::StrokeLinecap)],
                            [StrokeLinecap, SetStrokeLinecap,
                             PropertyId::Longhand(LonghandId::StrokeLinecap)],
                                [Stroke_linejoin, SetStroke_linejoin,
                                 PropertyId::Longhand(LonghandId::StrokeLinejoin)],
                            [StrokeLinejoin, SetStrokeLinejoin,
                             PropertyId::Longhand(LonghandId::StrokeLinejoin)],
                                [Stroke_miterlimit, SetStroke_miterlimit,
                                 PropertyId::Longhand(LonghandId::StrokeMiterlimit)],
                            [StrokeMiterlimit, SetStrokeMiterlimit,
                             PropertyId::Longhand(LonghandId::StrokeMiterlimit)],
                                [Stroke_opacity, SetStroke_opacity,
                                 PropertyId::Longhand(LonghandId::StrokeOpacity)],
                            [StrokeOpacity, SetStrokeOpacity,
                             PropertyId::Longhand(LonghandId::StrokeOpacity)],
                                [Stroke_dasharray, SetStroke_dasharray,
                                 PropertyId::Longhand(LonghandId::StrokeDasharray)],
                            [StrokeDasharray, SetStrokeDasharray,
                             PropertyId::Longhand(LonghandId::StrokeDasharray)],
                                [Stroke_dashoffset, SetStroke_dashoffset,
                                 PropertyId::Longhand(LonghandId::StrokeDashoffset)],
                            [StrokeDashoffset, SetStrokeDashoffset,
                             PropertyId::Longhand(LonghandId::StrokeDashoffset)],
                                [Clip_rule, SetClip_rule,
                                 PropertyId::Longhand(LonghandId::ClipRule)],
                            [ClipRule, SetClipRule,
                             PropertyId::Longhand(LonghandId::ClipRule)],
                                [Marker_start, SetMarker_start,
                                 PropertyId::Longhand(LonghandId::MarkerStart)],
                            [MarkerStart, SetMarkerStart,
                             PropertyId::Longhand(LonghandId::MarkerStart)],
                                [Marker_mid, SetMarker_mid,
                                 PropertyId::Longhand(LonghandId::MarkerMid)],
                            [MarkerMid, SetMarkerMid,
                             PropertyId::Longhand(LonghandId::MarkerMid)],
                                [Marker_end, SetMarker_end,
                                 PropertyId::Longhand(LonghandId::MarkerEnd)],
                            [MarkerEnd, SetMarkerEnd,
                             PropertyId::Longhand(LonghandId::MarkerEnd)],
                                [Paint_order, SetPaint_order,
                                 PropertyId::Longhand(LonghandId::PaintOrder)],
                            [PaintOrder, SetPaintOrder,
                             PropertyId::Longhand(LonghandId::PaintOrder)],
                                [_moz_context_properties, Set_moz_context_properties,
                                 PropertyId::Longhand(LonghandId::MozContextProperties)],
                            [MozContextProperties, SetMozContextProperties,
                             PropertyId::Longhand(LonghandId::MozContextProperties)],
                                [Dominant_baseline, SetDominant_baseline,
                                 PropertyId::Longhand(LonghandId::DominantBaseline)],
                            [DominantBaseline, SetDominantBaseline,
                             PropertyId::Longhand(LonghandId::DominantBaseline)],
                                [Vector_effect, SetVector_effect,
                                 PropertyId::Longhand(LonghandId::VectorEffect)],
                            [VectorEffect, SetVectorEffect,
                             PropertyId::Longhand(LonghandId::VectorEffect)],
                                [Stop_color, SetStop_color,
                                 PropertyId::Longhand(LonghandId::StopColor)],
                            [StopColor, SetStopColor,
                             PropertyId::Longhand(LonghandId::StopColor)],
                                [Stop_opacity, SetStop_opacity,
                                 PropertyId::Longhand(LonghandId::StopOpacity)],
                            [StopOpacity, SetStopOpacity,
                             PropertyId::Longhand(LonghandId::StopOpacity)],
                                [Flood_color, SetFlood_color,
                                 PropertyId::Longhand(LonghandId::FloodColor)],
                            [FloodColor, SetFloodColor,
                             PropertyId::Longhand(LonghandId::FloodColor)],
                                [Flood_opacity, SetFlood_opacity,
                                 PropertyId::Longhand(LonghandId::FloodOpacity)],
                            [FloodOpacity, SetFloodOpacity,
                             PropertyId::Longhand(LonghandId::FloodOpacity)],
                                [Lighting_color, SetLighting_color,
                                 PropertyId::Longhand(LonghandId::LightingColor)],
                            [LightingColor, SetLightingColor,
                             PropertyId::Longhand(LonghandId::LightingColor)],
                                [Mask_type, SetMask_type,
                                 PropertyId::Longhand(LonghandId::MaskType)],
                            [MaskType, SetMaskType,
                             PropertyId::Longhand(LonghandId::MaskType)],
                                [Clip_path, SetClip_path,
                                 PropertyId::Longhand(LonghandId::ClipPath)],
                            [ClipPath, SetClipPath,
                             PropertyId::Longhand(LonghandId::ClipPath)],
                                [Mask_mode, SetMask_mode,
                                 PropertyId::Longhand(LonghandId::MaskMode)],
                            [MaskMode, SetMaskMode,
                             PropertyId::Longhand(LonghandId::MaskMode)],
                                [Mask_repeat, SetMask_repeat,
                                 PropertyId::Longhand(LonghandId::MaskRepeat)],
                            [MaskRepeat, SetMaskRepeat,
                             PropertyId::Longhand(LonghandId::MaskRepeat)],
                                [_webkit_mask_repeat, Set_webkit_mask_repeat,
                                 PropertyId::Longhand(LonghandId::MaskRepeat)],
                            [WebkitMaskRepeat, SetWebkitMaskRepeat,
                             PropertyId::Longhand(LonghandId::MaskRepeat)],
                                [Mask_position_x, SetMask_position_x,
                                 PropertyId::Longhand(LonghandId::MaskPositionX)],
                            [MaskPositionX, SetMaskPositionX,
                             PropertyId::Longhand(LonghandId::MaskPositionX)],
                                [_webkit_mask_position_x, Set_webkit_mask_position_x,
                                 PropertyId::Longhand(LonghandId::MaskPositionX)],
                            [WebkitMaskPositionX, SetWebkitMaskPositionX,
                             PropertyId::Longhand(LonghandId::MaskPositionX)],
                                [Mask_position_y, SetMask_position_y,
                                 PropertyId::Longhand(LonghandId::MaskPositionY)],
                            [MaskPositionY, SetMaskPositionY,
                             PropertyId::Longhand(LonghandId::MaskPositionY)],
                                [_webkit_mask_position_y, Set_webkit_mask_position_y,
                                 PropertyId::Longhand(LonghandId::MaskPositionY)],
                            [WebkitMaskPositionY, SetWebkitMaskPositionY,
                             PropertyId::Longhand(LonghandId::MaskPositionY)],
                                [Mask_clip, SetMask_clip,
                                 PropertyId::Longhand(LonghandId::MaskClip)],
                            [MaskClip, SetMaskClip,
                             PropertyId::Longhand(LonghandId::MaskClip)],
                                [_webkit_mask_clip, Set_webkit_mask_clip,
                                 PropertyId::Longhand(LonghandId::MaskClip)],
                            [WebkitMaskClip, SetWebkitMaskClip,
                             PropertyId::Longhand(LonghandId::MaskClip)],
                                [Mask_origin, SetMask_origin,
                                 PropertyId::Longhand(LonghandId::MaskOrigin)],
                            [MaskOrigin, SetMaskOrigin,
                             PropertyId::Longhand(LonghandId::MaskOrigin)],
                                [_webkit_mask_origin, Set_webkit_mask_origin,
                                 PropertyId::Longhand(LonghandId::MaskOrigin)],
                            [WebkitMaskOrigin, SetWebkitMaskOrigin,
                             PropertyId::Longhand(LonghandId::MaskOrigin)],
                                [Mask_size, SetMask_size,
                                 PropertyId::Longhand(LonghandId::MaskSize)],
                            [MaskSize, SetMaskSize,
                             PropertyId::Longhand(LonghandId::MaskSize)],
                                [_webkit_mask_size, Set_webkit_mask_size,
                                 PropertyId::Longhand(LonghandId::MaskSize)],
                            [WebkitMaskSize, SetWebkitMaskSize,
                             PropertyId::Longhand(LonghandId::MaskSize)],
                                [Mask_composite, SetMask_composite,
                                 PropertyId::Longhand(LonghandId::MaskComposite)],
                            [MaskComposite, SetMaskComposite,
                             PropertyId::Longhand(LonghandId::MaskComposite)],
                                [_webkit_mask_composite, Set_webkit_mask_composite,
                                 PropertyId::Longhand(LonghandId::MaskComposite)],
                            [WebkitMaskComposite, SetWebkitMaskComposite,
                             PropertyId::Longhand(LonghandId::MaskComposite)],
                                [Mask_image, SetMask_image,
                                 PropertyId::Longhand(LonghandId::MaskImage)],
                            [MaskImage, SetMaskImage,
                             PropertyId::Longhand(LonghandId::MaskImage)],
                                [_webkit_mask_image, Set_webkit_mask_image,
                                 PropertyId::Longhand(LonghandId::MaskImage)],
                            [WebkitMaskImage, SetWebkitMaskImage,
                             PropertyId::Longhand(LonghandId::MaskImage)],
                                [_moz_box_align, Set_moz_box_align,
                                 PropertyId::Longhand(LonghandId::MozBoxAlign)],
                            [MozBoxAlign, SetMozBoxAlign,
                             PropertyId::Longhand(LonghandId::MozBoxAlign)],
                                [_webkit_box_align, Set_webkit_box_align,
                                 PropertyId::Longhand(LonghandId::MozBoxAlign)],
                            [WebkitBoxAlign, SetWebkitBoxAlign,
                             PropertyId::Longhand(LonghandId::MozBoxAlign)],
                                [_moz_box_direction, Set_moz_box_direction,
                                 PropertyId::Longhand(LonghandId::MozBoxDirection)],
                            [MozBoxDirection, SetMozBoxDirection,
                             PropertyId::Longhand(LonghandId::MozBoxDirection)],
                                [_webkit_box_direction, Set_webkit_box_direction,
                                 PropertyId::Longhand(LonghandId::MozBoxDirection)],
                            [WebkitBoxDirection, SetWebkitBoxDirection,
                             PropertyId::Longhand(LonghandId::MozBoxDirection)],
                                [_moz_box_flex, Set_moz_box_flex,
                                 PropertyId::Longhand(LonghandId::MozBoxFlex)],
                            [MozBoxFlex, SetMozBoxFlex,
                             PropertyId::Longhand(LonghandId::MozBoxFlex)],
                                [_webkit_box_flex, Set_webkit_box_flex,
                                 PropertyId::Longhand(LonghandId::MozBoxFlex)],
                            [WebkitBoxFlex, SetWebkitBoxFlex,
                             PropertyId::Longhand(LonghandId::MozBoxFlex)],
                                [_moz_box_orient, Set_moz_box_orient,
                                 PropertyId::Longhand(LonghandId::MozBoxOrient)],
                            [MozBoxOrient, SetMozBoxOrient,
                             PropertyId::Longhand(LonghandId::MozBoxOrient)],
                                [_webkit_box_orient, Set_webkit_box_orient,
                                 PropertyId::Longhand(LonghandId::MozBoxOrient)],
                            [WebkitBoxOrient, SetWebkitBoxOrient,
                             PropertyId::Longhand(LonghandId::MozBoxOrient)],
                                [_moz_box_pack, Set_moz_box_pack,
                                 PropertyId::Longhand(LonghandId::MozBoxPack)],
                            [MozBoxPack, SetMozBoxPack,
                             PropertyId::Longhand(LonghandId::MozBoxPack)],
                                [_webkit_box_pack, Set_webkit_box_pack,
                                 PropertyId::Longhand(LonghandId::MozBoxPack)],
                            [WebkitBoxPack, SetWebkitBoxPack,
                             PropertyId::Longhand(LonghandId::MozBoxPack)],
                                [_moz_stack_sizing, Set_moz_stack_sizing,
                                 PropertyId::Longhand(LonghandId::MozStackSizing)],
                            [MozStackSizing, SetMozStackSizing,
                             PropertyId::Longhand(LonghandId::MozStackSizing)],
                                [_moz_box_ordinal_group, Set_moz_box_ordinal_group,
                                 PropertyId::Longhand(LonghandId::MozBoxOrdinalGroup)],
                            [MozBoxOrdinalGroup, SetMozBoxOrdinalGroup,
                             PropertyId::Longhand(LonghandId::MozBoxOrdinalGroup)],
                                [_webkit_box_ordinal_group, Set_webkit_box_ordinal_group,
                                 PropertyId::Longhand(LonghandId::MozBoxOrdinalGroup)],
                            [WebkitBoxOrdinalGroup, SetWebkitBoxOrdinalGroup,
                             PropertyId::Longhand(LonghandId::MozBoxOrdinalGroup)],
                            [Background, SetBackground,
                             PropertyId::Shorthand(ShorthandId::Background)],
                                [Background_position, SetBackground_position,
                                 PropertyId::Shorthand(ShorthandId::BackgroundPosition)],
                            [BackgroundPosition, SetBackgroundPosition,
                             PropertyId::Shorthand(ShorthandId::BackgroundPosition)],
                                [Border_color, SetBorder_color,
                                 PropertyId::Shorthand(ShorthandId::BorderColor)],
                            [BorderColor, SetBorderColor,
                             PropertyId::Shorthand(ShorthandId::BorderColor)],
                                [Border_style, SetBorder_style,
                                 PropertyId::Shorthand(ShorthandId::BorderStyle)],
                            [BorderStyle, SetBorderStyle,
                             PropertyId::Shorthand(ShorthandId::BorderStyle)],
                                [Border_width, SetBorder_width,
                                 PropertyId::Shorthand(ShorthandId::BorderWidth)],
                            [BorderWidth, SetBorderWidth,
                             PropertyId::Shorthand(ShorthandId::BorderWidth)],
                                [Border_top, SetBorder_top,
                                 PropertyId::Shorthand(ShorthandId::BorderTop)],
                            [BorderTop, SetBorderTop,
                             PropertyId::Shorthand(ShorthandId::BorderTop)],
                                [Border_left, SetBorder_left,
                                 PropertyId::Shorthand(ShorthandId::BorderLeft)],
                            [BorderLeft, SetBorderLeft,
                             PropertyId::Shorthand(ShorthandId::BorderLeft)],
                                [Border_bottom, SetBorder_bottom,
                                 PropertyId::Shorthand(ShorthandId::BorderBottom)],
                            [BorderBottom, SetBorderBottom,
                             PropertyId::Shorthand(ShorthandId::BorderBottom)],
                                [Border_right, SetBorder_right,
                                 PropertyId::Shorthand(ShorthandId::BorderRight)],
                            [BorderRight, SetBorderRight,
                             PropertyId::Shorthand(ShorthandId::BorderRight)],
                                [Border_block_start, SetBorder_block_start,
                                 PropertyId::Shorthand(ShorthandId::BorderBlockStart)],
                            [BorderBlockStart, SetBorderBlockStart,
                             PropertyId::Shorthand(ShorthandId::BorderBlockStart)],
                                [Border_block_end, SetBorder_block_end,
                                 PropertyId::Shorthand(ShorthandId::BorderBlockEnd)],
                            [BorderBlockEnd, SetBorderBlockEnd,
                             PropertyId::Shorthand(ShorthandId::BorderBlockEnd)],
                                [Border_inline_start, SetBorder_inline_start,
                                 PropertyId::Shorthand(ShorthandId::BorderInlineStart)],
                            [BorderInlineStart, SetBorderInlineStart,
                             PropertyId::Shorthand(ShorthandId::BorderInlineStart)],
                                [_moz_border_start, Set_moz_border_start,
                                 PropertyId::Shorthand(ShorthandId::BorderInlineStart)],
                            [MozBorderStart, SetMozBorderStart,
                             PropertyId::Shorthand(ShorthandId::BorderInlineStart)],
                                [Border_inline_end, SetBorder_inline_end,
                                 PropertyId::Shorthand(ShorthandId::BorderInlineEnd)],
                            [BorderInlineEnd, SetBorderInlineEnd,
                             PropertyId::Shorthand(ShorthandId::BorderInlineEnd)],
                                [_moz_border_end, Set_moz_border_end,
                                 PropertyId::Shorthand(ShorthandId::BorderInlineEnd)],
                            [MozBorderEnd, SetMozBorderEnd,
                             PropertyId::Shorthand(ShorthandId::BorderInlineEnd)],
                            [Border, SetBorder,
                             PropertyId::Shorthand(ShorthandId::Border)],
                                [Border_radius, SetBorder_radius,
                                 PropertyId::Shorthand(ShorthandId::BorderRadius)],
                            [BorderRadius, SetBorderRadius,
                             PropertyId::Shorthand(ShorthandId::BorderRadius)],
                                [_webkit_border_radius, Set_webkit_border_radius,
                                 PropertyId::Shorthand(ShorthandId::BorderRadius)],
                            [WebkitBorderRadius, SetWebkitBorderRadius,
                             PropertyId::Shorthand(ShorthandId::BorderRadius)],
                                [Border_image, SetBorder_image,
                                 PropertyId::Shorthand(ShorthandId::BorderImage)],
                            [BorderImage, SetBorderImage,
                             PropertyId::Shorthand(ShorthandId::BorderImage)],
                                [_moz_border_image, Set_moz_border_image,
                                 PropertyId::Shorthand(ShorthandId::BorderImage)],
                            [MozBorderImage, SetMozBorderImage,
                             PropertyId::Shorthand(ShorthandId::BorderImage)],
                                [_webkit_border_image, Set_webkit_border_image,
                                 PropertyId::Shorthand(ShorthandId::BorderImage)],
                            [WebkitBorderImage, SetWebkitBorderImage,
                             PropertyId::Shorthand(ShorthandId::BorderImage)],
                            [Overflow, SetOverflow,
                             PropertyId::Shorthand(ShorthandId::Overflow)],
                            [Transition, SetTransition,
                             PropertyId::Shorthand(ShorthandId::Transition)],
                                [_moz_transition, Set_moz_transition,
                                 PropertyId::Shorthand(ShorthandId::Transition)],
                            [MozTransition, SetMozTransition,
                             PropertyId::Shorthand(ShorthandId::Transition)],
                                [_webkit_transition, Set_webkit_transition,
                                 PropertyId::Shorthand(ShorthandId::Transition)],
                            [WebkitTransition, SetWebkitTransition,
                             PropertyId::Shorthand(ShorthandId::Transition)],
                            [Animation, SetAnimation,
                             PropertyId::Shorthand(ShorthandId::Animation)],
                                [_moz_animation, Set_moz_animation,
                                 PropertyId::Shorthand(ShorthandId::Animation)],
                            [MozAnimation, SetMozAnimation,
                             PropertyId::Shorthand(ShorthandId::Animation)],
                                [_webkit_animation, Set_webkit_animation,
                                 PropertyId::Shorthand(ShorthandId::Animation)],
                            [WebkitAnimation, SetWebkitAnimation,
                             PropertyId::Shorthand(ShorthandId::Animation)],
                                [Scroll_snap_type, SetScroll_snap_type,
                                 PropertyId::Shorthand(ShorthandId::ScrollSnapType)],
                            [ScrollSnapType, SetScrollSnapType,
                             PropertyId::Shorthand(ShorthandId::ScrollSnapType)],
                                [_moz_transform, Set_moz_transform,
                                 PropertyId::Shorthand(ShorthandId::MozTransform)],
                            [MozTransform, SetMozTransform,
                             PropertyId::Shorthand(ShorthandId::MozTransform)],
                            [Columns, SetColumns,
                             PropertyId::Shorthand(ShorthandId::Columns)],
                                [_moz_columns, Set_moz_columns,
                                 PropertyId::Shorthand(ShorthandId::Columns)],
                            [MozColumns, SetMozColumns,
                             PropertyId::Shorthand(ShorthandId::Columns)],
                                [Column_rule, SetColumn_rule,
                                 PropertyId::Shorthand(ShorthandId::ColumnRule)],
                            [ColumnRule, SetColumnRule,
                             PropertyId::Shorthand(ShorthandId::ColumnRule)],
                                [_moz_column_rule, Set_moz_column_rule,
                                 PropertyId::Shorthand(ShorthandId::ColumnRule)],
                            [MozColumnRule, SetMozColumnRule,
                             PropertyId::Shorthand(ShorthandId::ColumnRule)],
                            [Font, SetFont,
                             PropertyId::Shorthand(ShorthandId::Font)],
                                [Font_variant, SetFont_variant,
                                 PropertyId::Shorthand(ShorthandId::FontVariant)],
                            [FontVariant, SetFontVariant,
                             PropertyId::Shorthand(ShorthandId::FontVariant)],
                                [Text_emphasis, SetText_emphasis,
                                 PropertyId::Shorthand(ShorthandId::TextEmphasis)],
                            [TextEmphasis, SetTextEmphasis,
                             PropertyId::Shorthand(ShorthandId::TextEmphasis)],
                                [_webkit_text_stroke, Set_webkit_text_stroke,
                                 PropertyId::Shorthand(ShorthandId::WebkitTextStroke)],
                            [WebkitTextStroke, SetWebkitTextStroke,
                             PropertyId::Shorthand(ShorthandId::WebkitTextStroke)],
                                [List_style, SetList_style,
                                 PropertyId::Shorthand(ShorthandId::ListStyle)],
                            [ListStyle, SetListStyle,
                             PropertyId::Shorthand(ShorthandId::ListStyle)],
                            [Margin, SetMargin,
                             PropertyId::Shorthand(ShorthandId::Margin)],
                            [Mask, SetMask,
                             PropertyId::Shorthand(ShorthandId::Mask)],
                                [_webkit_mask, Set_webkit_mask,
                                 PropertyId::Shorthand(ShorthandId::Mask)],
                            [WebkitMask, SetWebkitMask,
                             PropertyId::Shorthand(ShorthandId::Mask)],
                                [Mask_position, SetMask_position,
                                 PropertyId::Shorthand(ShorthandId::MaskPosition)],
                            [MaskPosition, SetMaskPosition,
                             PropertyId::Shorthand(ShorthandId::MaskPosition)],
                                [_webkit_mask_position, Set_webkit_mask_position,
                                 PropertyId::Shorthand(ShorthandId::MaskPosition)],
                            [WebkitMaskPosition, SetWebkitMaskPosition,
                             PropertyId::Shorthand(ShorthandId::MaskPosition)],
                            [Outline, SetOutline,
                             PropertyId::Shorthand(ShorthandId::Outline)],
                                [_moz_outline_radius, Set_moz_outline_radius,
                                 PropertyId::Shorthand(ShorthandId::MozOutlineRadius)],
                            [MozOutlineRadius, SetMozOutlineRadius,
                             PropertyId::Shorthand(ShorthandId::MozOutlineRadius)],
                            [Padding, SetPadding,
                             PropertyId::Shorthand(ShorthandId::Padding)],
                                [Flex_flow, SetFlex_flow,
                                 PropertyId::Shorthand(ShorthandId::FlexFlow)],
                            [FlexFlow, SetFlexFlow,
                             PropertyId::Shorthand(ShorthandId::FlexFlow)],
                                [_webkit_flex_flow, Set_webkit_flex_flow,
                                 PropertyId::Shorthand(ShorthandId::FlexFlow)],
                            [WebkitFlexFlow, SetWebkitFlexFlow,
                             PropertyId::Shorthand(ShorthandId::FlexFlow)],
                            [Flex, SetFlex,
                             PropertyId::Shorthand(ShorthandId::Flex)],
                                [_webkit_flex, Set_webkit_flex,
                                 PropertyId::Shorthand(ShorthandId::Flex)],
                            [WebkitFlex, SetWebkitFlex,
                             PropertyId::Shorthand(ShorthandId::Flex)],
                                [Grid_gap, SetGrid_gap,
                                 PropertyId::Shorthand(ShorthandId::GridGap)],
                            [GridGap, SetGridGap,
                             PropertyId::Shorthand(ShorthandId::GridGap)],
                                [Grid_row, SetGrid_row,
                                 PropertyId::Shorthand(ShorthandId::GridRow)],
                            [GridRow, SetGridRow,
                             PropertyId::Shorthand(ShorthandId::GridRow)],
                                [Grid_column, SetGrid_column,
                                 PropertyId::Shorthand(ShorthandId::GridColumn)],
                            [GridColumn, SetGridColumn,
                             PropertyId::Shorthand(ShorthandId::GridColumn)],
                                [Grid_area, SetGrid_area,
                                 PropertyId::Shorthand(ShorthandId::GridArea)],
                            [GridArea, SetGridArea,
                             PropertyId::Shorthand(ShorthandId::GridArea)],
                                [Grid_template, SetGrid_template,
                                 PropertyId::Shorthand(ShorthandId::GridTemplate)],
                            [GridTemplate, SetGridTemplate,
                             PropertyId::Shorthand(ShorthandId::GridTemplate)],
                            [Grid, SetGrid,
                             PropertyId::Shorthand(ShorthandId::Grid)],
                                [Place_content, SetPlace_content,
                                 PropertyId::Shorthand(ShorthandId::PlaceContent)],
                            [PlaceContent, SetPlaceContent,
                             PropertyId::Shorthand(ShorthandId::PlaceContent)],
                                [Place_self, SetPlace_self,
                                 PropertyId::Shorthand(ShorthandId::PlaceSelf)],
                            [PlaceSelf, SetPlaceSelf,
                             PropertyId::Shorthand(ShorthandId::PlaceSelf)],
                                [Place_items, SetPlace_items,
                                 PropertyId::Shorthand(ShorthandId::PlaceItems)],
                            [PlaceItems, SetPlaceItems,
                             PropertyId::Shorthand(ShorthandId::PlaceItems)],
                            [Marker, SetMarker,
                             PropertyId::Shorthand(ShorthandId::Marker)],
                                [Text_decoration, SetText_decoration,
                                 PropertyId::Shorthand(ShorthandId::TextDecoration)],
                            [TextDecoration, SetTextDecoration,
                             PropertyId::Shorthand(ShorthandId::TextDecoration)],
                            [All, SetAll,
                             PropertyId::Shorthand(ShorthandId::All)],
        }
    }
}

#[macro_export]
macro_rules! longhand_properties_idents {
    ($macro_name: ident) => {
        $macro_name! {
                { background_color, false }
                { background_image, false }
                { background_position_x, false }
                { background_position_y, false }
                { background_repeat, false }
                { background_attachment, false }
                { background_clip, false }
                { background_origin, false }
                { background_size, false }
                { background_blend_mode, false }
                { border_top_color, false }
                { border_top_style, false }
                { border_top_width, false }
                { border_left_color, false }
                { border_left_style, false }
                { border_left_width, false }
                { border_bottom_color, false }
                { border_bottom_style, false }
                { border_bottom_width, false }
                { border_right_color, false }
                { border_right_style, false }
                { border_right_width, false }
                { border_block_start_color, false }
                { border_block_start_style, false }
                { border_block_start_width, false }
                { border_block_end_color, false }
                { border_block_end_style, false }
                { border_block_end_width, false }
                { border_inline_start_color, false }
                { border_inline_start_style, false }
                { border_inline_start_width, false }
                { border_inline_end_color, false }
                { border_inline_end_style, false }
                { border_inline_end_width, false }
                { border_top_left_radius, true }
                { border_top_right_radius, true }
                { border_bottom_right_radius, true }
                { border_bottom_left_radius, true }
                { _moz_border_top_colors, false }
                { _moz_border_left_colors, false }
                { _moz_border_bottom_colors, false }
                { _moz_border_right_colors, false }
                { box_decoration_break, false }
                { _moz_float_edge, false }
                { border_image_source, true }
                { border_image_outset, true }
                { border_image_repeat, false }
                { border_image_width, true }
                { border_image_slice, true }
                { display, false }
                { _moz_top_layer, false }
                { position, false }
                { float, false }
                { clear, false }
                { vertical_align, false }
                { overflow_clip_box, false }
                { overflow_x, false }
                { overflow_y, false }
                { transition_duration, false }
                { transition_timing_function, false }
                { transition_property, false }
                { transition_delay, false }
                { animation_name, false }
                { animation_duration, false }
                { animation_timing_function, false }
                { animation_iteration_count, false }
                { animation_direction, false }
                { animation_play_state, false }
                { animation_fill_mode, false }
                { animation_delay, false }
                { scroll_snap_points_x, false }
                { scroll_snap_points_y, false }
                { scroll_snap_destination, true }
                { scroll_snap_coordinate, false }
                { transform, false }
                { scroll_behavior, false }
                { scroll_snap_type_x, false }
                { scroll_snap_type_y, false }
                { isolation, false }
                { page_break_after, false }
                { page_break_before, false }
                { page_break_inside, false }
                { resize, false }
                { perspective, false }
                { perspective_origin, true }
                { backface_visibility, false }
                { transform_box, false }
                { transform_style, false }
                { transform_origin, true }
                { contain, false }
                { _moz_appearance, false }
                { _moz_binding, true }
                { _moz_orient, false }
                { will_change, false }
                { shape_outside, true }
                { touch_action, false }
                { color, false }
                { column_width, false }
                { column_count, false }
                { column_gap, false }
                { column_fill, false }
                { column_rule_width, false }
                { column_rule_color, false }
                { column_span, false }
                { column_rule_style, false }
                { content, true }
                { counter_increment, false }
                { counter_reset, false }
                { opacity, false }
                { box_shadow, false }
                { clip, true }
                { filter, false }
                { mix_blend_mode, false }
                { font_family, true }
                { font_style, false }
                { font_variant_caps, false }
                { font_weight, false }
                { font_size, false }
                { font_size_adjust, false }
                { font_synthesis, false }
                { font_stretch, false }
                { font_kerning, false }
                { font_variant_alternates, false }
                { font_variant_east_asian, false }
                { font_variant_ligatures, false }
                { font_variant_numeric, false }
                { font_variant_position, false }
                { font_feature_settings, true }
                { font_variation_settings, false }
                { font_language_override, true }
                { _x_lang, false }
                { _moz_script_size_multiplier, false }
                { _moz_script_level, false }
                { _moz_math_display, false }
                { _moz_math_variant, false }
                { _moz_script_min_size, false }
                { _x_text_zoom, false }
                { _moz_osx_font_smoothing, false }
                { _moz_min_font_size_ratio, false }
                { visibility, false }
                { writing_mode, false }
                { direction, false }
                { text_orientation, false }
                { color_adjust, false }
                { image_rendering, false }
                { image_orientation, false }
                { border_collapse, false }
                { empty_cells, false }
                { caption_side, false }
                { border_spacing, true }
                { line_height, false }
                { text_transform, false }
                { hyphens, false }
                { _moz_text_size_adjust, false }
                { text_indent, false }
                { overflow_wrap, false }
                { word_break, false }
                { text_justify, false }
                { text_align_last, false }
                { text_align, false }
                { letter_spacing, false }
                { word_spacing, false }
                { white_space, false }
                { text_shadow, false }
                { text_emphasis_style, true }
                { text_emphasis_position, false }
                { text_emphasis_color, false }
                { _moz_tab_size, false }
                { _webkit_text_fill_color, false }
                { _webkit_text_stroke_color, false }
                { _webkit_text_stroke_width, false }
                { ruby_align, false }
                { ruby_position, false }
                { text_combine_upright, false }
                { text_rendering, false }
                { _moz_control_character_visibility, false }
                { list_style_position, false }
                { list_style_type, true }
                { list_style_image, true }
                { quotes, false }
                { _moz_image_region, true }
                { margin_top, false }
                { margin_left, false }
                { margin_bottom, false }
                { margin_right, false }
                { margin_block_start, false }
                { margin_block_end, false }
                { margin_inline_start, false }
                { margin_inline_end, false }
                { outline_color, false }
                { outline_style, false }
                { outline_width, false }
                { _moz_outline_radius_topleft, true }
                { _moz_outline_radius_topright, true }
                { _moz_outline_radius_bottomright, true }
                { _moz_outline_radius_bottomleft, true }
                { outline_offset, false }
                { padding_top, false }
                { padding_left, false }
                { padding_bottom, false }
                { padding_right, false }
                { padding_block_start, false }
                { padding_block_end, false }
                { padding_inline_start, false }
                { padding_inline_end, false }
                { cursor, true }
                { pointer_events, false }
                { _moz_user_input, false }
                { _moz_user_modify, false }
                { _moz_user_focus, false }
                { caret_color, true }
                { top, false }
                { left, false }
                { bottom, false }
                { right, false }
                { offset_block_start, false }
                { offset_block_end, false }
                { offset_inline_start, false }
                { offset_inline_end, false }
                { z_index, false }
                { flex_direction, false }
                { flex_wrap, false }
                { justify_content, false }
                { align_content, false }
                { align_items, false }
                { justify_items, false }
                { flex_grow, false }
                { flex_shrink, false }
                { align_self, false }
                { justify_self, false }
                { order, false }
                { flex_basis, false }
                { width, false }
                { min_width, false }
                { max_width, false }
                { height, false }
                { min_height, false }
                { max_height, false }
                { block_size, false }
                { min_block_size, false }
                { max_block_size, false }
                { inline_size, false }
                { min_inline_size, false }
                { max_inline_size, false }
                { box_sizing, false }
                { object_fit, false }
                { object_position, true }
                { grid_row_gap, false }
                { grid_row_start, true }
                { grid_row_end, true }
                { grid_auto_rows, true }
                { grid_template_rows, true }
                { grid_column_gap, false }
                { grid_column_start, true }
                { grid_column_end, true }
                { grid_auto_columns, true }
                { grid_template_columns, true }
                { grid_auto_flow, false }
                { grid_template_areas, true }
                { table_layout, false }
                { _x_span, false }
                { text_overflow, true }
                { unicode_bidi, false }
                { text_decoration_line, false }
                { text_decoration_style, false }
                { text_decoration_color, false }
                { initial_letter, false }
                { ime_mode, false }
                { _moz_user_select, false }
                { _moz_window_dragging, false }
                { _moz_window_shadow, false }
                { _moz_force_broken_image_icon, false }
                { text_anchor, false }
                { color_interpolation, false }
                { color_interpolation_filters, false }
                { fill, true }
                { fill_opacity, false }
                { fill_rule, false }
                { shape_rendering, false }
                { stroke, true }
                { stroke_width, true }
                { stroke_linecap, false }
                { stroke_linejoin, false }
                { stroke_miterlimit, false }
                { stroke_opacity, false }
                { stroke_dasharray, false }
                { stroke_dashoffset, true }
                { clip_rule, false }
                { marker_start, true }
                { marker_mid, true }
                { marker_end, true }
                { paint_order, false }
                { _moz_context_properties, false }
                { dominant_baseline, false }
                { vector_effect, false }
                { stop_color, false }
                { stop_opacity, false }
                { flood_color, false }
                { flood_opacity, false }
                { lighting_color, false }
                { mask_type, false }
                { clip_path, true }
                { mask_mode, false }
                { mask_repeat, false }
                { mask_position_x, false }
                { mask_position_y, false }
                { mask_clip, false }
                { mask_origin, false }
                { mask_size, false }
                { mask_composite, false }
                { mask_image, false }
                { _moz_box_align, false }
                { _moz_box_direction, false }
                { _moz_box_flex, false }
                { _moz_box_orient, false }
                { _moz_box_pack, false }
                { _moz_stack_sizing, false }
                { _moz_box_ordinal_group, false }
        }
    }
}
